/* ----------------------------------------------------------------------------
 * This file was automatically generated by SWIG (https://www.swig.org).
 * Version 4.3.1
 *
 * Do not make changes to this file unless you know what you are doing - modify
 * the SWIG interface file instead.
 * ----------------------------------------------------------------------------- */


#define SWIG_VERSION 0x040301
#define SWIGPERL
#define SWIG_CASTRANK_MODE

/* -----------------------------------------------------------------------------
 *  This section contains generic SWIG labels for method/variable
 *  declarations/attributes, and other compiler dependent labels.
 * ----------------------------------------------------------------------------- */

/* template workaround for compilers that cannot correctly implement the C++ standard */
#ifndef SWIGTEMPLATEDISAMBIGUATOR
# if defined(__SUNPRO_CC) && (__SUNPRO_CC <= 0x560)
#  define SWIGTEMPLATEDISAMBIGUATOR template
# elif defined(__HP_aCC)
/* Needed even with `aCC -AA' when `aCC -V' reports HP ANSI C++ B3910B A.03.55 */
/* If we find a maximum version that requires this, the test would be __HP_aCC <= 35500 for A.03.55 */
#  define SWIGTEMPLATEDISAMBIGUATOR template
# else
#  define SWIGTEMPLATEDISAMBIGUATOR
# endif
#endif

/* inline attribute */
#ifndef SWIGINLINE
# if defined(__cplusplus) || (defined(__GNUC__) && !defined(__STRICT_ANSI__))
#   define SWIGINLINE inline
# else
#   define SWIGINLINE
# endif
#endif

/* attribute recognised by some compilers to avoid 'unused' warnings */
#ifndef SWIGUNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define SWIGUNUSED __attribute__ ((__unused__))
#   else
#     define SWIGUNUSED
#   endif
# elif defined(__ICC)
#   define SWIGUNUSED __attribute__ ((__unused__))
# else
#   define SWIGUNUSED
# endif
#endif

#ifndef SWIG_MSC_UNSUPPRESS_4505
# if defined(_MSC_VER)
#   pragma warning(disable : 4505) /* unreferenced local function has been removed */
# endif
#endif

#ifndef SWIGUNUSEDPARM
# ifdef __cplusplus
#   define SWIGUNUSEDPARM(p)
# else
#   define SWIGUNUSEDPARM(p) p SWIGUNUSED
# endif
#endif

/* internal SWIG method */
#ifndef SWIGINTERN
# define SWIGINTERN static SWIGUNUSED
#endif

/* internal inline SWIG method */
#ifndef SWIGINTERNINLINE
# define SWIGINTERNINLINE SWIGINTERN SWIGINLINE
#endif

/* exporting methods */
#if defined(__GNUC__)
#  if (__GNUC__ >= 4) || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4)
#    ifndef GCC_HASCLASSVISIBILITY
#      define GCC_HASCLASSVISIBILITY
#    endif
#  endif
#endif

#ifndef SWIGEXPORT
# if defined(_WIN32) || defined(__WIN32__) || defined(__CYGWIN__)
#   if defined(STATIC_LINKED)
#     define SWIGEXPORT
#   else
#     define SWIGEXPORT __declspec(dllexport)
#   endif
# else
#   if defined(__GNUC__) && defined(GCC_HASCLASSVISIBILITY)
#     define SWIGEXPORT __attribute__ ((visibility("default")))
#   else
#     define SWIGEXPORT
#   endif
# endif
#endif

/* calling conventions for Windows */
#ifndef SWIGSTDCALL
# if defined(_WIN32) || defined(__WIN32__) || defined(__CYGWIN__)
#   define SWIGSTDCALL __stdcall
# else
#   define SWIGSTDCALL
# endif
#endif

/* Deal with Microsoft's attempt at deprecating C standard runtime functions */
#if !defined(SWIG_NO_CRT_SECURE_NO_DEPRECATE) && defined(_MSC_VER) && !defined(_CRT_SECURE_NO_DEPRECATE)
# define _CRT_SECURE_NO_DEPRECATE
#endif

/* Deal with Microsoft's attempt at deprecating methods in the standard C++ library */
#if !defined(SWIG_NO_SCL_SECURE_NO_DEPRECATE) && defined(_MSC_VER) && !defined(_SCL_SECURE_NO_DEPRECATE)
# define _SCL_SECURE_NO_DEPRECATE
#endif

/* Deal with Apple's deprecated 'AssertMacros.h' from Carbon-framework */
#if defined(__APPLE__) && !defined(__ASSERT_MACROS_DEFINE_VERSIONS_WITHOUT_UNDERSCORES)
# define __ASSERT_MACROS_DEFINE_VERSIONS_WITHOUT_UNDERSCORES 0
#endif

/* Intel's compiler complains if a variable which was never initialised is
 * cast to void, which is a common idiom which we use to indicate that we
 * are aware a variable isn't used.  So we just silence that warning.
 * See: https://github.com/swig/swig/issues/192 for more discussion.
 */
#ifdef __INTEL_COMPILER
# pragma warning disable 592
#endif

#if defined(__cplusplus) && __cplusplus >=201103L
# define SWIG_NULLPTR nullptr
#else
# define SWIG_NULLPTR NULL
#endif 

/* -----------------------------------------------------------------------------
 * swigcompat.swg
 *
 * Macros to provide support compatibility with older C and C++ standards.
 *
 * Note that SWIG expects __cplusplus to be defined to the appropriate C++ standard.
 * MSVC users are urged to check and examine the /Zc:__cplusplus compiler option.
 * See https://learn.microsoft.com/en-us/cpp/build/reference/zc-cplusplus.
 * ----------------------------------------------------------------------------- */

/* C99 and C++11 should provide snprintf, but define SWIG_NO_SNPRINTF
 * if you're missing it.
 */
#if ((defined __STDC_VERSION__ && __STDC_VERSION__ >= 199901L) || \
     (defined __cplusplus && __cplusplus >= 201103L) || \
     defined SWIG_HAVE_SNPRINTF) && \
    !defined SWIG_NO_SNPRINTF
# define SWIG_snprintf(O,S,F,A) snprintf(O,S,F,A)
# define SWIG_snprintf2(O,S,F,A,B) snprintf(O,S,F,A,B)
#else
/* Fallback versions ignore the buffer size, but most of our uses either have a
 * fixed maximum possible size or dynamically allocate a buffer that's large
 * enough.
 */
# define SWIG_snprintf(O,S,F,A) sprintf(O,F,A)
# define SWIG_snprintf2(O,S,F,A,B) sprintf(O,F,A,B)
#endif

/* -----------------------------------------------------------------------------
 * swigrun.swg
 *
 * This file contains generic C API SWIG runtime support for pointer
 * type checking.
 * ----------------------------------------------------------------------------- */

/* This should only be incremented when either the layout of swig_type_info changes,
   or for whatever reason, the runtime changes incompatibly */
#define SWIG_RUNTIME_VERSION "4"

/* define SWIG_TYPE_TABLE_NAME as "SWIG_TYPE_TABLE" */
#ifdef SWIG_TYPE_TABLE
# define SWIG_QUOTE_STRING(x) #x
# define SWIG_EXPAND_AND_QUOTE_STRING(x) SWIG_QUOTE_STRING(x)
# define SWIG_TYPE_TABLE_NAME SWIG_EXPAND_AND_QUOTE_STRING(SWIG_TYPE_TABLE)
#else
# define SWIG_TYPE_TABLE_NAME
#endif

/*
  You can use the SWIGRUNTIME and SWIGRUNTIMEINLINE macros for
  creating a static or dynamic library from the SWIG runtime code.
  In 99.9% of the cases, SWIG just needs to declare them as 'static'.

  But only do this if strictly necessary, ie, if you have problems
  with your compiler or suchlike.
*/

#ifndef SWIGRUNTIME
# define SWIGRUNTIME SWIGINTERN
#endif

#ifndef SWIGRUNTIMEINLINE
# define SWIGRUNTIMEINLINE SWIGRUNTIME SWIGINLINE
#endif

/*  Generic buffer size */
#ifndef SWIG_BUFFER_SIZE
# define SWIG_BUFFER_SIZE 1024
#endif

/* Flags for pointer conversions */
#define SWIG_POINTER_DISOWN        0x1
#define SWIG_CAST_NEW_MEMORY       0x2
#define SWIG_POINTER_NO_NULL       0x4
#define SWIG_POINTER_CLEAR         0x8
#define SWIG_POINTER_RELEASE       (SWIG_POINTER_CLEAR | SWIG_POINTER_DISOWN)

/* Flags for new pointer objects */
#define SWIG_POINTER_OWN           0x1


/*
   Flags/methods for returning states.

   The SWIG conversion methods, as ConvertPtr, return an integer
   that tells if the conversion was successful or not. And if not,
   an error code can be returned (see swigerrors.swg for the codes).

   Use the following macros/flags to set or process the returning
   states.

   In old versions of SWIG, code such as the following was usually written:

     if (SWIG_ConvertPtr(obj,vptr,ty.flags) != -1) {
       // success code
     } else {
       //fail code
     }

   Now you can be more explicit:

    int res = SWIG_ConvertPtr(obj,vptr,ty.flags);
    if (SWIG_IsOK(res)) {
      // success code
    } else {
      // fail code
    }

   which is the same really, but now you can also do

    Type *ptr;
    int res = SWIG_ConvertPtr(obj,(void **)(&ptr),ty.flags);
    if (SWIG_IsOK(res)) {
      // success code
      if (SWIG_IsNewObj(res) {
        ...
	delete *ptr;
      } else {
        ...
      }
    } else {
      // fail code
    }

   I.e., now SWIG_ConvertPtr can return new objects and you can
   identify the case and take care of the deallocation. Of course that
   also requires SWIG_ConvertPtr to return new result values, such as

      int SWIG_ConvertPtr(obj, ptr,...) {
        if (<obj is ok>) {
          if (<need new object>) {
            *ptr = <ptr to new allocated object>;
            return SWIG_NEWOBJ;
          } else {
            *ptr = <ptr to old object>;
            return SWIG_OLDOBJ;
          }
        } else {
          return SWIG_BADOBJ;
        }
      }

   Of course, returning the plain '0(success)/-1(fail)' still works, but you can be
   more explicit by returning SWIG_BADOBJ, SWIG_ERROR or any of the
   SWIG errors code.

   Finally, if the SWIG_CASTRANK_MODE is enabled, the result code
   allows returning the 'cast rank', for example, if you have this

       int food(double)
       int fooi(int);

   and you call

      food(1)   // cast rank '1'  (1 -> 1.0)
      fooi(1)   // cast rank '0'

   just use the SWIG_AddCast()/SWIG_CheckState()
*/

#define SWIG_OK                    (0)
/* Runtime errors are < 0 */
#define SWIG_ERROR                 (-1)
/* Errors in range -1 to -99 are in swigerrors.swg (errors for all languages including those not using the runtime) */
/* Errors in range -100 to -199 are language specific errors defined in *errors.swg */
/* Errors < -200 are generic runtime specific errors */
#define SWIG_ERROR_RELEASE_NOT_OWNED (-200)

#define SWIG_IsOK(r)               (r >= 0)
#define SWIG_ArgError(r)           ((r != SWIG_ERROR) ? r : SWIG_TypeError)

/* The CastRankLimit says how many bits are used for the cast rank */
#define SWIG_CASTRANKLIMIT         (1 << 8)
/* The NewMask denotes the object was created (using new/malloc) */
#define SWIG_NEWOBJMASK            (SWIG_CASTRANKLIMIT  << 1)
/* The TmpMask is for in/out typemaps that use temporary objects */
#define SWIG_TMPOBJMASK            (SWIG_NEWOBJMASK << 1)
/* Simple returning values */
#define SWIG_BADOBJ                (SWIG_ERROR)
#define SWIG_OLDOBJ                (SWIG_OK)
#define SWIG_NEWOBJ                (SWIG_OK | SWIG_NEWOBJMASK)
#define SWIG_TMPOBJ                (SWIG_OK | SWIG_TMPOBJMASK)
/* Check, add and del object mask methods */
#define SWIG_AddNewMask(r)         (SWIG_IsOK(r) ? (r | SWIG_NEWOBJMASK) : r)
#define SWIG_DelNewMask(r)         (SWIG_IsOK(r) ? (r & ~SWIG_NEWOBJMASK) : r)
#define SWIG_IsNewObj(r)           (SWIG_IsOK(r) && (r & SWIG_NEWOBJMASK))
#define SWIG_AddTmpMask(r)         (SWIG_IsOK(r) ? (r | SWIG_TMPOBJMASK) : r)
#define SWIG_DelTmpMask(r)         (SWIG_IsOK(r) ? (r & ~SWIG_TMPOBJMASK) : r)
#define SWIG_IsTmpObj(r)           (SWIG_IsOK(r) && (r & SWIG_TMPOBJMASK))

/* Cast-Rank Mode */
#if defined(SWIG_CASTRANK_MODE)
#  ifndef SWIG_TypeRank
#    define SWIG_TypeRank             unsigned long
#  endif
#  ifndef SWIG_MAXCASTRANK            /* Default cast allowed */
#    define SWIG_MAXCASTRANK          (2)
#  endif
#  define SWIG_CASTRANKMASK          ((SWIG_CASTRANKLIMIT) -1)
#  define SWIG_CastRank(r)           (r & SWIG_CASTRANKMASK)
SWIGINTERNINLINE int SWIG_AddCast(int r) {
  return SWIG_IsOK(r) ? ((SWIG_CastRank(r) < SWIG_MAXCASTRANK) ? (r + 1) : SWIG_ERROR) : r;
}
SWIGINTERNINLINE int SWIG_CheckState(int r) {
  return SWIG_IsOK(r) ? SWIG_CastRank(r) + 1 : 0;
}
#else /* no cast-rank mode */
#  define SWIG_AddCast(r) (r)
#  define SWIG_CheckState(r) (SWIG_IsOK(r) ? 1 : 0)
#endif


#include <string.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef void *(*swig_converter_func)(void *, int *);
typedef struct swig_type_info *(*swig_dycast_func)(void **);

/* Structure to store information on one type */
typedef struct swig_type_info {
  const char             *name;			/* mangled name of this type */
  const char             *str;			/* human readable name of this type */
  swig_dycast_func        dcast;		/* dynamic cast function down a hierarchy */
  struct swig_cast_info  *cast;			/* linked list of types that can cast into this type */
  void                   *clientdata;		/* language specific type data */
  int                    owndata;		/* flag if the structure owns the clientdata */
} swig_type_info;

/* Structure to store a type and conversion function used for casting */
typedef struct swig_cast_info {
  swig_type_info         *type;			/* pointer to type that is equivalent to this type */
  swig_converter_func     converter;		/* function to cast the void pointers */
  struct swig_cast_info  *next;			/* pointer to next cast in linked list */
  struct swig_cast_info  *prev;			/* pointer to the previous cast */
} swig_cast_info;

/* Structure used to store module information
 * Each module generates one structure like this, and the runtime collects
 * all of these structures and stores them in a circularly linked list.*/
typedef struct swig_module_info {
  swig_type_info         **types;		/* Array of pointers to swig_type_info structures that are in this module */
  size_t                 size;		        /* Number of types in this module */
  struct swig_module_info *next;		/* Pointer to next element in circularly linked list */
  swig_type_info         **type_initial;	/* Array of initially generated type structures */
  swig_cast_info         **cast_initial;	/* Array of initially generated casting structures */
  void                    *clientdata;		/* Language specific module data */
} swig_module_info;

/*
  Compare two type names skipping the space characters, therefore
  "char*" == "char *" and "Class<int>" == "Class<int >", etc.

  Return 0 when the two name types are equivalent, as in
  strncmp, but skipping ' '.
*/
SWIGRUNTIME int
SWIG_TypeNameComp(const char *f1, const char *l1,
		  const char *f2, const char *l2) {
  for (;(f1 != l1) && (f2 != l2); ++f1, ++f2) {
    while ((*f1 == ' ') && (f1 != l1)) ++f1;
    while ((*f2 == ' ') && (f2 != l2)) ++f2;
    if (*f1 != *f2) return (*f1 > *f2) ? 1 : -1;
  }
  return (int)((l1 - f1) - (l2 - f2));
}

/*
  Check type equivalence in a name list like <name1>|<name2>|...
  Return 0 if equal, -1 if nb < tb, 1 if nb > tb
*/
SWIGRUNTIME int
SWIG_TypeCmp(const char *nb, const char *tb) {
  int equiv = 1;
  const char* te = tb + strlen(tb);
  const char* ne = nb;
  while (equiv != 0 && *ne) {
    for (nb = ne; *ne; ++ne) {
      if (*ne == '|') break;
    }
    equiv = SWIG_TypeNameComp(nb, ne, tb, te);
    if (*ne) ++ne;
  }
  return equiv;
}

/*
  Check type equivalence in a name list like <name1>|<name2>|...
  Return 0 if not equal, 1 if equal
*/
SWIGRUNTIME int
SWIG_TypeEquiv(const char *nb, const char *tb) {
  return SWIG_TypeCmp(nb, tb) == 0 ? 1 : 0;
}

/*
  Check the typename
*/
SWIGRUNTIME swig_cast_info *
SWIG_TypeCheck(const char *c, swig_type_info *ty) {
  if (ty) {
    swig_cast_info *iter = ty->cast;
    while (iter) {
      if (strcmp(iter->type->name, c) == 0) {
        if (iter == ty->cast)
          return iter;
        /* Move iter to the top of the linked list */
        iter->prev->next = iter->next;
        if (iter->next)
          iter->next->prev = iter->prev;
        iter->next = ty->cast;
        iter->prev = 0;
        if (ty->cast) ty->cast->prev = iter;
        ty->cast = iter;
        return iter;
      }
      iter = iter->next;
    }
  }
  return 0;
}

/*
  Identical to SWIG_TypeCheck, except strcmp is replaced with a pointer comparison
*/
SWIGRUNTIME swig_cast_info *
SWIG_TypeCheckStruct(const swig_type_info *from, swig_type_info *ty) {
  if (ty) {
    swig_cast_info *iter = ty->cast;
    while (iter) {
      if (iter->type == from) {
        if (iter == ty->cast)
          return iter;
        /* Move iter to the top of the linked list */
        iter->prev->next = iter->next;
        if (iter->next)
          iter->next->prev = iter->prev;
        iter->next = ty->cast;
        iter->prev = 0;
        if (ty->cast) ty->cast->prev = iter;
        ty->cast = iter;
        return iter;
      }
      iter = iter->next;
    }
  }
  return 0;
}

/*
  Cast a pointer up an inheritance hierarchy
*/
SWIGRUNTIMEINLINE void *
SWIG_TypeCast(swig_cast_info *ty, void *ptr, int *newmemory) {
  return ((!ty) || (!ty->converter)) ? ptr : (*ty->converter)(ptr, newmemory);
}

/*
   Dynamic pointer casting. Down an inheritance hierarchy
*/
SWIGRUNTIME swig_type_info *
SWIG_TypeDynamicCast(swig_type_info *ty, void **ptr) {
  swig_type_info *lastty = ty;
  if (!ty || !ty->dcast) return ty;
  while (ty && (ty->dcast)) {
    ty = (*ty->dcast)(ptr);
    if (ty) lastty = ty;
  }
  return lastty;
}

/*
  Return the name associated with this type
*/
SWIGRUNTIMEINLINE const char *
SWIG_TypeName(const swig_type_info *ty) {
  return ty->name;
}

/*
  Return the pretty name associated with this type,
  that is an unmangled type name in a form presentable to the user.
*/
SWIGRUNTIME const char *
SWIG_TypePrettyName(const swig_type_info *type) {
  /* The "str" field contains the equivalent pretty names of the
     type, separated by vertical-bar characters.  Choose the last
     name. It should be the most specific; a fully resolved name
     but not necessarily with default template parameters expanded. */
  if (!type) return NULL;
  if (type->str != NULL) {
    const char *last_name = type->str;
    const char *s;
    for (s = type->str; *s; s++)
      if (*s == '|') last_name = s+1;
    return last_name;
  }
  else
    return type->name;
}

/*
   Set the clientdata field for a type
*/
SWIGRUNTIME void
SWIG_TypeClientData(swig_type_info *ti, void *clientdata) {
  swig_cast_info *cast = ti->cast;
  /* if (ti->clientdata == clientdata) return; */
  ti->clientdata = clientdata;

  while (cast) {
    if (!cast->converter) {
      swig_type_info *tc = cast->type;
      if (!tc->clientdata) {
	SWIG_TypeClientData(tc, clientdata);
      }
    }
    cast = cast->next;
  }
}
SWIGRUNTIME void
SWIG_TypeNewClientData(swig_type_info *ti, void *clientdata) {
  SWIG_TypeClientData(ti, clientdata);
  ti->owndata = 1;
}

/*
  Search for a swig_type_info structure only by mangled name
  Search is a O(log #types)

  We start searching at module start, and finish searching when start == end.
  Note: if start == end at the beginning of the function, we go all the way around
  the circular list.
*/
SWIGRUNTIME swig_type_info *
SWIG_MangledTypeQueryModule(swig_module_info *start,
                            swig_module_info *end,
		            const char *name) {
  swig_module_info *iter = start;
  do {
    if (iter->size) {
      size_t l = 0;
      size_t r = iter->size - 1;
      do {
	/* since l+r >= 0, we can (>> 1) instead (/ 2) */
	size_t i = (l + r) >> 1;
	const char *iname = iter->types[i]->name;
	if (iname) {
	  int compare = strcmp(name, iname);
	  if (compare == 0) {
	    return iter->types[i];
	  } else if (compare < 0) {
	    if (i) {
	      r = i - 1;
	    } else {
	      break;
	    }
	  } else if (compare > 0) {
	    l = i + 1;
	  }
	} else {
	  break; /* should never happen */
	}
      } while (l <= r);
    }
    iter = iter->next;
  } while (iter != end);
  return 0;
}

/*
  Search for a swig_type_info structure for either a mangled name or a human readable name.
  It first searches the mangled names of the types, which is a O(log #types)
  If a type is not found it then searches the human readable names, which is O(#types).

  We start searching at module start, and finish searching when start == end.
  Note: if start == end at the beginning of the function, we go all the way around
  the circular list.
*/
SWIGRUNTIME swig_type_info *
SWIG_TypeQueryModule(swig_module_info *start,
                     swig_module_info *end,
		     const char *name) {
  /* STEP 1: Search the name field using binary search */
  swig_type_info *ret = SWIG_MangledTypeQueryModule(start, end, name);
  if (ret) {
    return ret;
  } else {
    /* STEP 2: If the type hasn't been found, do a complete search
       of the str field (the human readable name) */
    swig_module_info *iter = start;
    do {
      size_t i = 0;
      for (; i < iter->size; ++i) {
	if (iter->types[i]->str && (SWIG_TypeEquiv(iter->types[i]->str, name)))
	  return iter->types[i];
      }
      iter = iter->next;
    } while (iter != end);
  }

  /* neither found a match */
  return 0;
}

/*
   Pack binary data into a string
*/
SWIGRUNTIME char *
SWIG_PackData(char *c, void *ptr, size_t sz) {
  static const char hex[17] = "0123456789abcdef";
  const unsigned char *u = (unsigned char *) ptr;
  const unsigned char *eu =  u + sz;
  for (; u != eu; ++u) {
    unsigned char uu = *u;
    *(c++) = hex[(uu & 0xf0) >> 4];
    *(c++) = hex[uu & 0xf];
  }
  return c;
}

/*
   Unpack binary data from a string
*/
SWIGRUNTIME const char *
SWIG_UnpackData(const char *c, void *ptr, size_t sz) {
  unsigned char *u = (unsigned char *) ptr;
  const unsigned char *eu = u + sz;
  for (; u != eu; ++u) {
    char d = *(c++);
    unsigned char uu;
    if ((d >= '0') && (d <= '9'))
      uu = (unsigned char)((d - '0') << 4);
    else if ((d >= 'a') && (d <= 'f'))
      uu = (unsigned char)((d - ('a'-10)) << 4);
    else
      return (char *) 0;
    d = *(c++);
    if ((d >= '0') && (d <= '9'))
      uu |= (unsigned char)(d - '0');
    else if ((d >= 'a') && (d <= 'f'))
      uu |= (unsigned char)(d - ('a'-10));
    else
      return (char *) 0;
    *u = uu;
  }
  return c;
}

/*
   Pack 'void *' into a string buffer.
*/
SWIGRUNTIME char *
SWIG_PackVoidPtr(char *buff, void *ptr, const char *name, size_t bsz) {
  char *r = buff;
  if ((2*sizeof(void *) + 2) > bsz) return 0;
  *(r++) = '_';
  r = SWIG_PackData(r,&ptr,sizeof(void *));
  if (strlen(name) + 1 > (bsz - (r - buff))) return 0;
  strcpy(r,name);
  return buff;
}

SWIGRUNTIME const char *
SWIG_UnpackVoidPtr(const char *c, void **ptr, const char *name) {
  if (*c != '_') {
    if (strcmp(c,"NULL") == 0) {
      *ptr = (void *) 0;
      return name;
    } else {
      return 0;
    }
  }
  return SWIG_UnpackData(++c,ptr,sizeof(void *));
}

SWIGRUNTIME char *
SWIG_PackDataName(char *buff, void *ptr, size_t sz, const char *name, size_t bsz) {
  char *r = buff;
  size_t lname = (name ? strlen(name) : 0);
  if ((2*sz + 2 + lname) > bsz) return 0;
  *(r++) = '_';
  r = SWIG_PackData(r,ptr,sz);
  if (lname) {
    strncpy(r,name,lname+1);
  } else {
    *r = 0;
  }
  return buff;
}

SWIGRUNTIME const char *
SWIG_UnpackDataName(const char *c, void *ptr, size_t sz, const char *name) {
  if (*c != '_') {
    if (strcmp(c,"NULL") == 0) {
      memset(ptr,0,sz);
      return name;
    } else {
      return 0;
    }
  }
  return SWIG_UnpackData(++c,ptr,sz);
}

#ifdef __cplusplus
}
#endif

/* SWIG Errors applicable to all language modules, values are reserved from -1 to -99 */
#define  SWIG_UnknownError    	   -1
#define  SWIG_IOError        	   -2
#define  SWIG_RuntimeError   	   -3
#define  SWIG_IndexError     	   -4
#define  SWIG_TypeError      	   -5
#define  SWIG_DivisionByZero 	   -6
#define  SWIG_OverflowError  	   -7
#define  SWIG_SyntaxError    	   -8
#define  SWIG_ValueError     	   -9
#define  SWIG_SystemError    	   -10
#define  SWIG_AttributeError 	   -11
#define  SWIG_MemoryError    	   -12
#define  SWIG_NullReferenceError   -13


#ifdef __cplusplus
/* Needed on some windows machines---since MS plays funny games with the header files under C++ */
#include <math.h>
#include <stdlib.h>
extern "C" {
#endif

#if __GNUC__ >= 10
#if defined(__cplusplus)
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wvolatile"
#endif
#endif

#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

#if __GNUC__ >= 10
#if defined(__cplusplus)
#pragma GCC diagnostic pop
#endif
#endif

/* PERL_REVISION was added in Perl 5.6. */
#if !defined PERL_REVISION || (PERL_REVISION-0 == 5 && PERL_VERSION-0 < 8)
# error SWIG requires Perl >= 5.8.0
#endif

#if defined(WIN32) && defined(PERL_OBJECT) && !defined(PerlIO_exportFILE)
#define PerlIO_exportFILE(fh,fl) (FILE*)(fh)
#endif

#ifndef SvIOK_UV
# define SvIOK_UV(sv)       (SvIOK(sv) && (SvUVX(sv) == SvIVX(sv)))
#endif

#ifndef SvUOK
# define SvUOK(sv)           SvIOK_UV(sv)
#endif

#ifndef IVSIZE
#  ifdef LONGSIZE
#    define IVSIZE LONGSIZE
#  else
#    define IVSIZE 4 /* A bold guess, but the best we can make. */
#  endif
#endif

#ifndef INT2PTR
#  if (IVSIZE == PTRSIZE) && (UVSIZE == PTRSIZE)
#    define PTRV                  UV
#    define INT2PTR(any,d)        (any)(d)
#  else
#    if PTRSIZE == LONGSIZE
#      define PTRV                unsigned long
#    else
#      define PTRV                unsigned
#    endif
#    define INT2PTR(any,d)        (any)(PTRV)(d)
#  endif

#  define NUM2PTR(any,d)  (any)(PTRV)(d)
#  define PTR2IV(p)       INT2PTR(IV,p)
#  define PTR2UV(p)       INT2PTR(UV,p)
#  define PTR2NV(p)       NUM2PTR(NV,p)

#  if PTRSIZE == LONGSIZE
#    define PTR2ul(p)     (unsigned long)(p)
#  else
#    define PTR2ul(p)     INT2PTR(unsigned long,p)
#  endif
#endif /* !INT2PTR */

#ifndef SvPV_nolen
# define SvPV_nolen(x) SvPV(x,PL_na)
#endif

#ifndef get_sv
#  define get_sv perl_get_sv
#endif

#ifndef ERRSV
#  define ERRSV get_sv("@",FALSE)
#endif

#ifndef pTHX_
#define pTHX_
#endif   

#include <string.h>
#ifdef __cplusplus
}
#endif

/* -----------------------------------------------------------------------------
 * error manipulation
 * ----------------------------------------------------------------------------- */

SWIGINTERN const char*
SWIG_Perl_ErrorType(int code) {
  switch(code) {
  case SWIG_MemoryError:
    return "MemoryError";
  case SWIG_IOError:
    return "IOError";
  case SWIG_RuntimeError:
    return "RuntimeError";
  case SWIG_IndexError:
    return "IndexError";
  case SWIG_TypeError:
    return "TypeError";
  case SWIG_DivisionByZero:
    return "ZeroDivisionError";
  case SWIG_OverflowError:
    return "OverflowError";
  case SWIG_SyntaxError:
    return "SyntaxError";
  case SWIG_ValueError:
    return "ValueError";
  case SWIG_SystemError:
    return "SystemError";
  case SWIG_AttributeError:
    return "AttributeError";
  case SWIG_NullReferenceError:
    return "NullReferenceError";
  default:
    return "RuntimeError";
  }
}


/* -----------------------------------------------------------------------------
 * perlrun.swg
 *
 * This file contains the runtime support for Perl modules
 * and includes code for managing global variables and pointer
 * type checking.
 * ----------------------------------------------------------------------------- */

#define SWIG_PERL_OBJECT_DECL
#define SWIG_PERL_OBJECT_CALL

/* Common SWIG API */

/* for raw pointers */
#define SWIG_ConvertPtr(obj, pp, type, flags)           SWIG_Perl_ConvertPtr(SWIG_PERL_OBJECT_CALL obj, pp, type, flags)
#define SWIG_ConvertPtrAndOwn(obj, pp, type, flags,own) SWIG_Perl_ConvertPtrAndOwn(SWIG_PERL_OBJECT_CALL obj, pp, type, flags, own)
#define SWIG_NewPointerObj(p, type, flags)              SWIG_Perl_NewPointerObj(SWIG_PERL_OBJECT_CALL p, type, flags)
#define SWIG_AcquirePtr(ptr, src)                       SWIG_Perl_AcquirePtr(ptr, src)
#define swig_owntype                                    int

/* for raw packed data */
#define SWIG_ConvertPacked(obj, p, s, type)             SWIG_Perl_ConvertPacked(SWIG_PERL_OBJECT_CALL obj, p, s, type)
#define SWIG_NewPackedObj(p, s, type)	                SWIG_Perl_NewPackedObj(SWIG_PERL_OBJECT_CALL p, s, type)

/* for class or struct pointers */
#define SWIG_ConvertInstance(obj, pptr, type, flags)    SWIG_ConvertPtr(obj, pptr, type, flags)
#define SWIG_NewInstanceObj(ptr, type, flags)           SWIG_NewPointerObj(ptr, type, flags)

/* for C or C++ function pointers */
#define SWIG_ConvertFunctionPtr(obj, pptr, type)        SWIG_ConvertPtr(obj, pptr, type, 0)
#define SWIG_NewFunctionPtrObj(ptr, type)               SWIG_NewPointerObj(ptr, type, 0)

/* for C++ member pointers, ie, member methods */
#define SWIG_ConvertMember(obj, ptr, sz, ty)            SWIG_ConvertPacked(obj, ptr, sz, ty)
#define SWIG_NewMemberObj(ptr, sz, type)                SWIG_NewPackedObj(ptr, sz, type)


/* Runtime API */

#define SWIG_GetModule(clientdata)                      SWIG_Perl_GetModule(clientdata)
#define SWIG_SetModule(clientdata, pointer)             SWIG_Perl_SetModule(pointer)


/* Error manipulation */

#define SWIG_ErrorType(code)                            SWIG_Perl_ErrorType(code)               
#define SWIG_Error(code, msg)            		sv_setpvf(get_sv("@", GV_ADD), "%s %s", SWIG_ErrorType(code), msg)
#define SWIG_fail                        		goto fail						    

/* Perl-specific SWIG API */

#define SWIG_MakePtr(sv, ptr, type, flags)              SWIG_Perl_MakePtr(SWIG_PERL_OBJECT_CALL sv, ptr, type, flags)
#define SWIG_MakePackedObj(sv, p, s, type)	        SWIG_Perl_MakePackedObj(SWIG_PERL_OBJECT_CALL sv, p, s, type)
#define SWIG_SetError(str)                              SWIG_Error(SWIG_RuntimeError, str)


#define SWIG_PERL_DECL_ARGS_1(arg1)                     (SWIG_PERL_OBJECT_DECL arg1)
#define SWIG_PERL_CALL_ARGS_1(arg1)                     (SWIG_PERL_OBJECT_CALL arg1)
#define SWIG_PERL_DECL_ARGS_2(arg1, arg2)               (SWIG_PERL_OBJECT_DECL arg1, arg2)
#define SWIG_PERL_CALL_ARGS_2(arg1, arg2)               (SWIG_PERL_OBJECT_CALL arg1, arg2)

/* -----------------------------------------------------------------------------
 * pointers/data manipulation
 * ----------------------------------------------------------------------------- */

/* For backward compatibility only */
#define SWIG_POINTER_EXCEPTION  0

#ifdef __cplusplus
extern "C" {
#endif

#define SWIG_OWNER   SWIG_POINTER_OWN
#define SWIG_SHADOW  SWIG_OWNER << 1

#define SWIG_MAYBE_PERL_OBJECT SWIG_PERL_OBJECT_DECL

/* SWIG Perl macros */

/* Macro to declare an XS function */
#ifndef XSPROTO
#   define XSPROTO(name) void name(pTHX_ CV* cv)
#endif

/* Macro to call an XS function */
#ifndef MULTIPLICITY
#  define SWIG_CALLXS(_name) _name(cv)
#else
#  define SWIG_CALLXS(_name) _name(PERL_GET_THX, cv)
#endif

#define MAGIC_PPERL
#define SWIGCLASS_STATIC static SWIGUNUSED

#ifndef MULTIPLICITY
#define SWIG_MAGIC(a,b) (SV *a, MAGIC *b)

#ifdef __cplusplus
extern "C" {
#endif
typedef int (*SwigMagicFunc)(SV *, MAGIC *);
#ifdef __cplusplus
}
#endif

#else /* MULTIPLICITY */

#define SWIG_MAGIC(a,b) (struct interpreter *interp, SV *a, MAGIC *b)

#ifdef __cplusplus
extern "C" {
#endif
typedef int (*SwigMagicFunc)(struct interpreter *, SV *, MAGIC *);
#ifdef __cplusplus
}
#endif

#endif /* MULTIPLICITY */

static void SWIGUNUSED SWIG_croak_null()
{
  SV *err = get_sv("@", GV_ADD);
  if (sv_isobject(err))
    croak(0);
  else
    croak("%s", SvPV_nolen(err));
}


/* 
   Define how strict is the cast between strings and integers/doubles
   when overloading between these types occurs.
   
   The default is making it as strict as possible by using SWIG_AddCast
   when needed.
   
   You can use -DSWIG_PERL_NO_STRICT_STR2NUM at compilation time to
   disable the SWIG_AddCast, making the casting between string and
   numbers less strict.

   In the end, we try to solve the overloading between strings and
   numerical types in the more natural way, but if you can avoid it,
   well, avoid it using %rename, for example.
*/
#ifndef SWIG_PERL_NO_STRICT_STR2NUM
# ifndef SWIG_PERL_STRICT_STR2NUM
#  define SWIG_PERL_STRICT_STR2NUM
# endif
#endif
#ifdef SWIG_PERL_STRICT_STR2NUM
/* string takes precedence */
#define SWIG_Str2NumCast(x) SWIG_AddCast(x)  
#else
/* number takes precedence */
#define SWIG_Str2NumCast(x) x
#endif



#include <stdlib.h>

SWIGRUNTIME const char *
SWIG_Perl_TypeProxyName(const swig_type_info *type) {
  if (!type) return NULL;
  if (type->clientdata != NULL) {
    return (const char*) type->clientdata;
  } 
  else {
    return type->name;
  }
}

/* Identical to SWIG_TypeCheck, except for strcmp comparison */
SWIGRUNTIME swig_cast_info *
SWIG_TypeProxyCheck(const char *c, swig_type_info *ty) {
  if (ty) {
    swig_cast_info *iter = ty->cast;
    while (iter) {
      if (strcmp(SWIG_Perl_TypeProxyName(iter->type), c) == 0) {
        if (iter == ty->cast)
          return iter;
        /* Move iter to the top of the linked list */
        iter->prev->next = iter->next;
        if (iter->next)
          iter->next->prev = iter->prev;
        iter->next = ty->cast;
        iter->prev = 0;
        if (ty->cast) ty->cast->prev = iter;
        ty->cast = iter;
        return iter;
      }
      iter = iter->next;
    }
  }
  return 0;
}

/* Acquire a pointer value */

SWIGRUNTIME int
SWIG_Perl_AcquirePtr(SWIG_MAYBE_PERL_OBJECT SV *SWIGUNUSEDPARM(sv), int SWIGUNUSEDPARM(own)) {
  /* TODO */
  return 0;
}

/* Function for getting a pointer value */

SWIGRUNTIME int
SWIG_Perl_ConvertPtrAndOwn(SWIG_MAYBE_PERL_OBJECT SV *sv, void **ptr, swig_type_info *_t, int flags, int *own) {
  swig_cast_info *tc;
  void *voidptr = (void *)0;
  SV *tsv = 0;
  int check_owned_pointer_release = (flags & SWIG_POINTER_RELEASE) == SWIG_POINTER_RELEASE;

  if (own)
    *own = 0;

  /* If magical, apply more magic */
  if (SvGMAGICAL(sv))
    mg_get(sv);

  /* Check to see if this is an object */
  if (sv_isobject(sv)) {
    IV tmp = 0;
    tsv = (SV*) SvRV(sv);
    if ((SvTYPE(tsv) == SVt_PVHV)) {
      MAGIC *mg;
      if (SvMAGICAL(tsv)) {
        mg = mg_find(tsv,'P');
        if (mg) {
          sv = mg->mg_obj;
          if (sv_isobject(sv)) {
	    tsv = (SV*)SvRV(sv);
            tmp = SvIV(tsv);
          }
        }
      } else {
        return SWIG_ERROR;
      }
    } else {
      tmp = SvIV(tsv);
    }
    voidptr = INT2PTR(void *,tmp);
  } else if (! SvOK(sv)) {            /* Check for undef */
    *(ptr) = (void *) 0;
    return (flags & SWIG_POINTER_NO_NULL) ? SWIG_NullReferenceError : SWIG_OK;
  } else if (SvTYPE(sv) == SVt_RV) {  /* Check for NULL pointer */
    if (!SvROK(sv)) {
      /* In Perl 5.12 and later, SVt_RV == SVt_IV, so sv could be a valid integer value.  */
      if (SvIOK(sv)) {
        return SWIG_ERROR;
      } else {
        /* NULL pointer (reference to undef). */
        *(ptr) = (void *) 0;
        return SWIG_OK;
      }
    } else {
      return SWIG_ERROR;
    }
  } else {                            /* Don't know what it is */
    return SWIG_ERROR;
  }
  if (_t) {
    /* Now see if the types match */
    char *_c = HvNAME(SvSTASH(SvRV(sv)));
    tc = SWIG_TypeProxyCheck(_c,_t);
#ifdef SWIG_DIRECTORS
    if (!tc && !sv_derived_from(sv,SWIG_Perl_TypeProxyName(_t))) {
#else
    if (!tc) {
#endif
      return SWIG_ERROR;
    }
    {
      int newmemory = 0;
      *ptr = SWIG_TypeCast(tc,voidptr,&newmemory);
      if (newmemory == SWIG_CAST_NEW_MEMORY) {
        assert(own); /* badly formed typemap which will lead to a memory leak - it must set and use own to delete *ptr */
        if (own)
          *own = *own | SWIG_CAST_NEW_MEMORY;
      }
    }
  } else {
    *ptr = voidptr;
  }

  /* 
   *  DISOWN implementation: we need a perl guru to check this one.
   */
  if (tsv && ((flags & SWIG_POINTER_DISOWN) || check_owned_pointer_release)) {
    /* 
     *  almost copy paste code from below SWIG_POINTER_OWN setting
     */
    SV *obj = sv;
    HV *stash = SvSTASH(SvRV(obj));
    GV *gv = *(GV**)hv_fetch(stash, "OWNER", 5, TRUE);
    int owned = 0;
    if (isGV(gv)) {
      HV *hv = GvHVn(gv);
      /*
       * To set ownership (see below), a newSViv(1) entry is added. 
       * Hence, to remove ownership, we delete the entry.
       */
      if (hv_exists_ent(hv, obj, 0)) {
        owned = 1;
        if (flags & SWIG_POINTER_DISOWN) {
          hv_delete_ent(hv, obj, 0, 0);
        }
      }
    }
    if (check_owned_pointer_release && !owned) {
      return SWIG_ERROR_RELEASE_NOT_OWNED;
    }
  }

  if (tsv && (flags & SWIG_POINTER_CLEAR)) {
    SvIV_set(tsv, 0);
  }

  return SWIG_OK;
}

SWIGRUNTIME int
SWIG_Perl_ConvertPtr(SWIG_MAYBE_PERL_OBJECT SV *sv, void **ptr, swig_type_info *_t, int flags) {
  return SWIG_Perl_ConvertPtrAndOwn(sv, ptr, _t, flags, 0);
}

SWIGRUNTIME void
SWIG_Perl_MakePtr(SWIG_MAYBE_PERL_OBJECT SV *sv, void *ptr, swig_type_info *t, int flags) {
  if (ptr && (flags & (SWIG_SHADOW | SWIG_POINTER_OWN))) {
    SV *self;
    SV *obj=newSV(0);
    HV *hash=newHV();
    HV *stash;
    sv_setref_pv(obj, SWIG_Perl_TypeProxyName(t), ptr);
    stash=SvSTASH(SvRV(obj));
    if (flags & SWIG_POINTER_OWN) {
      HV *hv;
      GV *gv = *(GV**)hv_fetch(stash, "OWNER", 5, TRUE);
      if (!isGV(gv))
        gv_init(gv, stash, "OWNER", 5, FALSE);
      hv=GvHVn(gv);
      hv_store_ent(hv, obj, newSViv(1), 0);
    }
    sv_magic((SV *)hash, (SV *)obj, 'P', Nullch, 0);
    SvREFCNT_dec(obj);
    self=newRV_noinc((SV *)hash);
    sv_setsv(sv, self);
    SvREFCNT_dec((SV *)self);
    sv_bless(sv, stash);
  }
  else {
    sv_setref_pv(sv, SWIG_Perl_TypeProxyName(t), ptr);
  }
}

SWIGRUNTIMEINLINE SV *
SWIG_Perl_NewPointerObj(SWIG_MAYBE_PERL_OBJECT void *ptr, swig_type_info *t, int flags) {
  SV *result = sv_newmortal();
  SWIG_MakePtr(result, ptr, t, flags);
  return result;
}

SWIGRUNTIME void
SWIG_Perl_MakePackedObj(SWIG_MAYBE_PERL_OBJECT SV *sv, void *ptr, int sz, swig_type_info *type) {
  char result[1024];
  char *r = result;
  if ((2*sz + 1 + strlen(SWIG_Perl_TypeProxyName(type))) > 1000) return;
  *(r++) = '_';
  r = SWIG_PackData(r,ptr,sz);
  strcpy(r,SWIG_Perl_TypeProxyName(type));
  sv_setpv(sv, result);
}

SWIGRUNTIME SV *
SWIG_Perl_NewPackedObj(SWIG_MAYBE_PERL_OBJECT void *ptr, int sz, swig_type_info *type) {
  SV *result = sv_newmortal();
  SWIG_Perl_MakePackedObj(result, ptr, sz, type);
  return result;
}

/* Convert a packed pointer value */
SWIGRUNTIME int
SWIG_Perl_ConvertPacked(SWIG_MAYBE_PERL_OBJECT SV *obj, void *ptr, int sz, swig_type_info *ty) {
  swig_cast_info *tc;
  const char  *c = 0;

  if ((!obj) || (!SvOK(obj))) return SWIG_ERROR;
  c = SvPV_nolen(obj);
  /* Pointer values must start with leading underscore */
  if (*c != '_') return SWIG_ERROR;
  c++;
  c = SWIG_UnpackData(c,ptr,sz);
  if (ty) {
    tc = SWIG_TypeCheck(c,ty);
    if (!tc) return SWIG_ERROR;
  }
  return SWIG_OK;
}


/* Macros for low-level exception handling */
#define SWIG_croak(x)    { SWIG_Error(SWIG_RuntimeError, x); SWIG_fail; }


typedef XSPROTO(SwigPerlWrapper);
typedef SwigPerlWrapper *SwigPerlWrapperPtr;

/* Structure for command table */
typedef struct {
  const char         *name;
  SwigPerlWrapperPtr  wrapper;
} swig_command_info;

/* Information for constant table */

#define SWIG_INT     1
#define SWIG_FLOAT   2
#define SWIG_STRING  3
#define SWIG_POINTER 4
#define SWIG_BINARY  5

/* Constant information structure */
typedef struct swig_constant_info {
    int              type;
    const char      *name;
    long             lvalue;
    double           dvalue;
    void            *pvalue;
    swig_type_info **ptype;
} swig_constant_info;


/* Structure for variable table */
typedef struct {
  const char   *name;
  SwigMagicFunc   set;
  SwigMagicFunc   get;
  swig_type_info  **type;
} swig_variable_info;

/* Magic variable code */
#ifdef __cplusplus
#  define swig_create_magic(s,a,b,c) _swig_create_magic(s,const_cast<char*>(a),b,c)
#else
#  define swig_create_magic(s,a,b,c) _swig_create_magic(s,(char*)(a),b,c)
#endif
#ifndef MULTIPLICITY
SWIGRUNTIME void _swig_create_magic(SV *sv, char *name, int (*set)(SV *, MAGIC *), int (*get)(SV *,MAGIC *))
#else
SWIGRUNTIME void _swig_create_magic(SV *sv, char *name, int (*set)(struct interpreter*, SV *, MAGIC *), int (*get)(struct interpreter*, SV *,MAGIC *))
#endif
{
  MAGIC *mg;
  sv_magic(sv,sv,'U',name,(I32)strlen(name));
  mg = mg_find(sv,'U');
  mg->mg_virtual = (MGVTBL *) malloc(sizeof(MGVTBL));
  mg->mg_virtual->svt_get = (SwigMagicFunc) get;
  mg->mg_virtual->svt_set = (SwigMagicFunc) set;
  mg->mg_virtual->svt_len = 0;
  mg->mg_virtual->svt_clear = 0;
  mg->mg_virtual->svt_free = 0;
}


SWIGRUNTIME swig_module_info *
SWIG_Perl_GetModule(void *SWIGUNUSEDPARM(clientdata)) {
  static void *type_pointer = (void *)0;
  SV *pointer;

  /* first check if pointer already created */
  if (!type_pointer) {
    pointer = get_sv("swig_runtime_data::type_pointer" SWIG_RUNTIME_VERSION SWIG_TYPE_TABLE_NAME, FALSE | GV_ADDMULTI);
    if (pointer && SvOK(pointer)) {
      type_pointer = INT2PTR(swig_type_info **, SvIV(pointer));
    }
  }

  return (swig_module_info *) type_pointer;
}

SWIGRUNTIME void
SWIG_Perl_SetModule(swig_module_info *module) {
  SV *pointer;

  /* create a new pointer */
  pointer = get_sv("swig_runtime_data::type_pointer" SWIG_RUNTIME_VERSION SWIG_TYPE_TABLE_NAME, TRUE | GV_ADDMULTI);
  sv_setiv(pointer, PTR2IV(module));
}

#ifdef __cplusplus
}
#endif

/* Workaround perl5 global namespace pollution. Note that undefining library
 * functions like fopen will not solve the problem on all platforms as fopen
 * might be a macro on Windows but not necessarily on other operating systems. */
#ifdef do_open
  #undef do_open
#endif
#ifdef do_close
  #undef do_close
#endif
#ifdef do_exec
  #undef do_exec
#endif
#ifdef scalar
  #undef scalar
#endif
#ifdef list
  #undef list
#endif
#ifdef apply
  #undef apply
#endif
#ifdef convert
  #undef convert
#endif
#ifdef Error
  #undef Error
#endif
#ifdef form
  #undef form
#endif
#ifdef vform
  #undef vform
#endif
#ifdef LABEL
  #undef LABEL
#endif
#ifdef METHOD
  #undef METHOD
#endif
#ifdef Move
  #undef Move
#endif
#ifdef yylex
  #undef yylex
#endif
#ifdef yyparse
  #undef yyparse
#endif
#ifdef yyerror
  #undef yyerror
#endif
#ifdef invert
  #undef invert
#endif
#ifdef ref
  #undef ref
#endif
#ifdef read
  #undef read
#endif
#ifdef write
  #undef write
#endif
#ifdef eof
  #undef eof
#endif
#ifdef close
  #undef close
#endif
#ifdef rewind
  #undef rewind
#endif
#ifdef free
  #undef free
#endif
#ifdef malloc
  #undef malloc
#endif
#ifdef calloc
  #undef calloc
#endif
#ifdef Stat
  #undef Stat
#endif
#ifdef check
  #undef check
#endif
#ifdef seekdir
  #undef seekdir
#endif
#ifdef open
  #undef open
#endif
#ifdef readdir
  #undef readdir
#endif
#ifdef bind
  #undef bind
#endif
#ifdef access
  #undef access
#endif
#ifdef stat
  #undef stat
#endif
#ifdef seed
  #undef seed
#endif

#ifdef bool
  /* Leave if macro is from C99 stdbool.h */
  #ifndef __bool_true_false_are_defined
    #undef bool
  #endif
#endif




#define SWIG_exception_fail(code, msg) do { SWIG_Error(code, msg); SWIG_fail; } while(0) 

#define SWIG_contract_assert(expr, msg) do { if (!(expr)) { SWIG_Error(SWIG_RuntimeError, msg); SWIG_fail; } } while (0) 



/* -------- TYPES TABLE (BEGIN) -------- */

#define SWIGTYPE_p_DWG_CLASS_STABILITY swig_types[0]
#define SWIGTYPE_p_DWG_COLOR_METHOD swig_types[1]
#define SWIGTYPE_p_DWG_ENTITY_SECTIONS swig_types[2]
#define SWIGTYPE_p_DWG_ERROR swig_types[3]
#define SWIGTYPE_p_DWG_HDL_CODE swig_types[4]
#define SWIGTYPE_p_DWG_OBJECT_SUPERTYPE swig_types[5]
#define SWIGTYPE_p_DWG_OBJECT_TYPE swig_types[6]
#define SWIGTYPE_p_DWG_OBJECT_TYPE_R11 swig_types[7]
#define SWIGTYPE_p_DWG_SECTION_TYPE swig_types[8]
#define SWIGTYPE_p_DWG_SECTION_TYPE_R11 swig_types[9]
#define SWIGTYPE_p_DWG_SECTION_TYPE_R13 swig_types[10]
#define SWIGTYPE_p_DWG_VERSION_TYPE swig_types[11]
#define SWIGTYPE_p_Dwg_Section_Info swig_types[12]
#define SWIGTYPE_p_Dwg_Section_InfoHdr swig_types[13]
#define SWIGTYPE_p_RESBUF_VALUE_TYPE swig_types[14]
#define SWIGTYPE_p__dwg_3DSOLID_material swig_types[15]
#define SWIGTYPE_p__dwg_3DSOLID_silhouette swig_types[16]
#define SWIGTYPE_p__dwg_3DSOLID_wire swig_types[17]
#define SWIGTYPE_p__dwg_ACSH_HistoryNode swig_types[18]
#define SWIGTYPE_p__dwg_ACSH_SubentColor swig_types[19]
#define SWIGTYPE_p__dwg_ACSH_SubentMaterial swig_types[20]
#define SWIGTYPE_p__dwg_ACTIONBODY swig_types[21]
#define SWIGTYPE_p__dwg_ARRAYITEMLOCATOR swig_types[22]
#define SWIGTYPE_p__dwg_ASSOCACTIONBODY_action swig_types[23]
#define SWIGTYPE_p__dwg_ASSOCACTION_Deps swig_types[24]
#define SWIGTYPE_p__dwg_ASSOCARRAYITEM swig_types[25]
#define SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY swig_types[26]
#define SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY swig_types[27]
#define SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded swig_types[28]
#define SWIGTYPE_p__dwg_AcDs swig_types[29]
#define SWIGTYPE_p__dwg_AcDs_Data swig_types[30]
#define SWIGTYPE_p__dwg_AcDs_DataBlob swig_types[31]
#define SWIGTYPE_p__dwg_AcDs_DataBlob01 swig_types[32]
#define SWIGTYPE_p__dwg_AcDs_DataBlobRef swig_types[33]
#define SWIGTYPE_p__dwg_AcDs_DataBlobRef_Page swig_types[34]
#define SWIGTYPE_p__dwg_AcDs_DataIndex swig_types[35]
#define SWIGTYPE_p__dwg_AcDs_DataIndex_Entry swig_types[36]
#define SWIGTYPE_p__dwg_AcDs_Data_Record swig_types[37]
#define SWIGTYPE_p__dwg_AcDs_Data_RecordHdr swig_types[38]
#define SWIGTYPE_p__dwg_AcDs_Schema swig_types[39]
#define SWIGTYPE_p__dwg_AcDs_SchemaData swig_types[40]
#define SWIGTYPE_p__dwg_AcDs_SchemaData_UProp swig_types[41]
#define SWIGTYPE_p__dwg_AcDs_SchemaIndex swig_types[42]
#define SWIGTYPE_p__dwg_AcDs_SchemaIndex_Prop swig_types[43]
#define SWIGTYPE_p__dwg_AcDs_Schema_Prop swig_types[44]
#define SWIGTYPE_p__dwg_AcDs_Search swig_types[45]
#define SWIGTYPE_p__dwg_AcDs_Search_Data swig_types[46]
#define SWIGTYPE_p__dwg_AcDs_Search_IdIdx swig_types[47]
#define SWIGTYPE_p__dwg_AcDs_Search_IdIdxs swig_types[48]
#define SWIGTYPE_p__dwg_AcDs_Segment swig_types[49]
#define SWIGTYPE_p__dwg_AcDs_SegmentIndex swig_types[50]
#define SWIGTYPE_p__dwg_BLOCKACTION_connectionpts swig_types[51]
#define SWIGTYPE_p__dwg_BLOCKLOOKUPACTION_lut swig_types[52]
#define SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo swig_types[53]
#define SWIGTYPE_p__dwg_BLOCKPARAMETER_connection swig_types[54]
#define SWIGTYPE_p__dwg_BLOCKPARAMVALUESET swig_types[55]
#define SWIGTYPE_p__dwg_BLOCKSTRETCHACTION_codes swig_types[56]
#define SWIGTYPE_p__dwg_BLOCKSTRETCHACTION_handles swig_types[57]
#define SWIGTYPE_p__dwg_BLOCKVISIBILITYPARAMETER_state swig_types[58]
#define SWIGTYPE_p__dwg_COMPOUNDOBJECTID swig_types[59]
#define SWIGTYPE_p__dwg_CONSTRAINTGROUPNODE swig_types[60]
#define SWIGTYPE_p__dwg_CONTEXTDATA_dict swig_types[61]
#define SWIGTYPE_p__dwg_CONTEXTDATA_submgr swig_types[62]
#define SWIGTYPE_p__dwg_CellContentGeometry swig_types[63]
#define SWIGTYPE_p__dwg_CellStyle swig_types[64]
#define SWIGTYPE_p__dwg_ColorRamp swig_types[65]
#define SWIGTYPE_p__dwg_ContentFormat swig_types[66]
#define SWIGTYPE_p__dwg_DATALINK_customdata swig_types[67]
#define SWIGTYPE_p__dwg_DATATABLE_column swig_types[68]
#define SWIGTYPE_p__dwg_DATATABLE_row swig_types[69]
#define SWIGTYPE_p__dwg_DIMASSOC_Ref swig_types[70]
#define SWIGTYPE_p__dwg_DIMENSION_common swig_types[71]
#define SWIGTYPE_p__dwg_EVAL_Edge swig_types[72]
#define SWIGTYPE_p__dwg_EVAL_Node swig_types[73]
#define SWIGTYPE_p__dwg_EvalExpr swig_types[74]
#define SWIGTYPE_p__dwg_EvalExpr_value swig_types[75]
#define SWIGTYPE_p__dwg_EvalVariant swig_types[76]
#define SWIGTYPE_p__dwg_EvalVariant_u swig_types[77]
#define SWIGTYPE_p__dwg_FIELD_ChildValue swig_types[78]
#define SWIGTYPE_p__dwg_FileDepList_Files swig_types[79]
#define SWIGTYPE_p__dwg_FormattedTableData swig_types[80]
#define SWIGTYPE_p__dwg_FormattedTableMerged swig_types[81]
#define SWIGTYPE_p__dwg_GEODATA_meshface swig_types[82]
#define SWIGTYPE_p__dwg_GEODATA_meshpt swig_types[83]
#define SWIGTYPE_p__dwg_GridFormat swig_types[84]
#define SWIGTYPE_p__dwg_HATCH_Color swig_types[85]
#define SWIGTYPE_p__dwg_HATCH_ControlPoint swig_types[86]
#define SWIGTYPE_p__dwg_HATCH_DefLine swig_types[87]
#define SWIGTYPE_p__dwg_HATCH_Path swig_types[88]
#define SWIGTYPE_p__dwg_HATCH_PathSeg swig_types[89]
#define SWIGTYPE_p__dwg_HATCH_PolylinePath swig_types[90]
#define SWIGTYPE_p__dwg_LAYER_entry swig_types[91]
#define SWIGTYPE_p__dwg_LEADER_ArrowHead swig_types[92]
#define SWIGTYPE_p__dwg_LEADER_BlockLabel swig_types[93]
#define SWIGTYPE_p__dwg_LEADER_Break swig_types[94]
#define SWIGTYPE_p__dwg_LEADER_Line swig_types[95]
#define SWIGTYPE_p__dwg_LEADER_Node swig_types[96]
#define SWIGTYPE_p__dwg_LIGHTLIST_light swig_types[97]
#define SWIGTYPE_p__dwg_LTYPE_dash swig_types[98]
#define SWIGTYPE_p__dwg_LWPOLYLINE_width swig_types[99]
#define SWIGTYPE_p__dwg_LinkedData swig_types[100]
#define SWIGTYPE_p__dwg_LinkedTableData swig_types[101]
#define SWIGTYPE_p__dwg_MATERIAL_color swig_types[102]
#define SWIGTYPE_p__dwg_MATERIAL_gentexture swig_types[103]
#define SWIGTYPE_p__dwg_MATERIAL_mapper swig_types[104]
#define SWIGTYPE_p__dwg_MESH_edge swig_types[105]
#define SWIGTYPE_p__dwg_MLEADER_AnnotContext swig_types[106]
#define SWIGTYPE_p__dwg_MLEADER_Content swig_types[107]
#define SWIGTYPE_p__dwg_MLEADER_Content_Block swig_types[108]
#define SWIGTYPE_p__dwg_MLEADER_Content_MText swig_types[109]
#define SWIGTYPE_p__dwg_MLINESTYLE_line swig_types[110]
#define SWIGTYPE_p__dwg_MLINESTYLE_line_lt swig_types[111]
#define SWIGTYPE_p__dwg_MLINE_line swig_types[112]
#define SWIGTYPE_p__dwg_MLINE_vertex swig_types[113]
#define SWIGTYPE_p__dwg_OCD_Dimension swig_types[114]
#define SWIGTYPE_p__dwg_PARTIAL_VIEWING_INDEX_Entry swig_types[115]
#define SWIGTYPE_p__dwg_POINTCLOUDCOLORMAP_Ramp swig_types[116]
#define SWIGTYPE_p__dwg_POINTCLOUDEX_Croppings swig_types[117]
#define SWIGTYPE_p__dwg_POINTCLOUD_Clippings swig_types[118]
#define SWIGTYPE_p__dwg_POINTCLOUD_IntensityStyle swig_types[119]
#define SWIGTYPE_p__dwg_PROXY_LWPOLYLINE swig_types[120]
#define SWIGTYPE_p__dwg_R2004_Header swig_types[121]
#define SWIGTYPE_p__dwg_SECTION_geometrysettings swig_types[122]
#define SWIGTYPE_p__dwg_SECTION_typesettings swig_types[123]
#define SWIGTYPE_p__dwg_SPLINE_control_point swig_types[124]
#define SWIGTYPE_p__dwg_SUNSTUDY_Dates swig_types[125]
#define SWIGTYPE_p__dwg_SummaryInfo_Property swig_types[126]
#define SWIGTYPE_p__dwg_TABLEGEOMETRY_Cell swig_types[127]
#define SWIGTYPE_p__dwg_TABLESTYLE_CellStyle swig_types[128]
#define SWIGTYPE_p__dwg_TABLESTYLE_border swig_types[129]
#define SWIGTYPE_p__dwg_TABLESTYLE_rowstyles swig_types[130]
#define SWIGTYPE_p__dwg_TABLE_AttrDef swig_types[131]
#define SWIGTYPE_p__dwg_TABLE_BreakHeight swig_types[132]
#define SWIGTYPE_p__dwg_TABLE_BreakRow swig_types[133]
#define SWIGTYPE_p__dwg_TABLE_Cell swig_types[134]
#define SWIGTYPE_p__dwg_TABLE_CustomDataItem swig_types[135]
#define SWIGTYPE_p__dwg_TABLE_value swig_types[136]
#define SWIGTYPE_p__dwg_TableCell swig_types[137]
#define SWIGTYPE_p__dwg_TableCellContent swig_types[138]
#define SWIGTYPE_p__dwg_TableCellContent_Attr swig_types[139]
#define SWIGTYPE_p__dwg_TableDataColumn swig_types[140]
#define SWIGTYPE_p__dwg_TableRow swig_types[141]
#define SWIGTYPE_p__dwg_UCS_orthopts swig_types[142]
#define SWIGTYPE_p__dwg_VALUEPARAM swig_types[143]
#define SWIGTYPE_p__dwg_VALUEPARAM_vars swig_types[144]
#define SWIGTYPE_p__dwg_abstractentity_UNDERLAY swig_types[145]
#define SWIGTYPE_p__dwg_abstractobject_ASSOCARRAYPARAMETERS swig_types[146]
#define SWIGTYPE_p__dwg_abstractobject_UNDERLAYDEFINITION swig_types[147]
#define SWIGTYPE_p__dwg_appinfo swig_types[148]
#define SWIGTYPE_p__dwg_appinfohistory swig_types[149]
#define SWIGTYPE_p__dwg_auxheader swig_types[150]
#define SWIGTYPE_p__dwg_binary_chunk swig_types[151]
#define SWIGTYPE_p__dwg_binary_chunk_u swig_types[152]
#define SWIGTYPE_p__dwg_bitcode_2bd swig_types[153]
#define SWIGTYPE_p__dwg_bitcode_2rd swig_types[154]
#define SWIGTYPE_p__dwg_bitcode_3bd swig_types[155]
#define SWIGTYPE_p__dwg_bitcode_3rd swig_types[156]
#define SWIGTYPE_p__dwg_chain swig_types[157]
#define SWIGTYPE_p__dwg_class swig_types[158]
#define SWIGTYPE_p__dwg_color swig_types[159]
#define SWIGTYPE_p__dwg_entity_3DFACE swig_types[160]
#define SWIGTYPE_p__dwg_entity_3DLINE swig_types[161]
#define SWIGTYPE_p__dwg_entity_3DSOLID swig_types[162]
#define SWIGTYPE_p__dwg_entity_ALIGNMENTPARAMETERENTITY swig_types[163]
#define SWIGTYPE_p__dwg_entity_ARC swig_types[164]
#define SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT swig_types[165]
#define SWIGTYPE_p__dwg_entity_ARC_DIMENSION swig_types[166]
#define SWIGTYPE_p__dwg_entity_ATTDEF swig_types[167]
#define SWIGTYPE_p__dwg_entity_ATTRIB swig_types[168]
#define SWIGTYPE_p__dwg_entity_BASEPOINTPARAMETERENTITY swig_types[169]
#define SWIGTYPE_p__dwg_entity_BLOCK swig_types[170]
#define SWIGTYPE_p__dwg_entity_CAMERA swig_types[171]
#define SWIGTYPE_p__dwg_entity_CIRCLE swig_types[172]
#define SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED swig_types[173]
#define SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN swig_types[174]
#define SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT swig_types[175]
#define SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER swig_types[176]
#define SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR swig_types[177]
#define SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE swig_types[178]
#define SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS swig_types[179]
#define SWIGTYPE_p__dwg_entity_ELLIPSE swig_types[180]
#define SWIGTYPE_p__dwg_entity_ENDBLK swig_types[181]
#define SWIGTYPE_p__dwg_entity_ENDREP swig_types[182]
#define SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE swig_types[183]
#define SWIGTYPE_p__dwg_entity_FLIPGRIPENTITY swig_types[184]
#define SWIGTYPE_p__dwg_entity_FLIPPARAMETERENTITY swig_types[185]
#define SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER swig_types[186]
#define SWIGTYPE_p__dwg_entity_HATCH swig_types[187]
#define SWIGTYPE_p__dwg_entity_HELIX swig_types[188]
#define SWIGTYPE_p__dwg_entity_IMAGE swig_types[189]
#define SWIGTYPE_p__dwg_entity_INSERT swig_types[190]
#define SWIGTYPE_p__dwg_entity_JUMP swig_types[191]
#define SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION swig_types[192]
#define SWIGTYPE_p__dwg_entity_LAYOUTPRINTCONFIG swig_types[193]
#define SWIGTYPE_p__dwg_entity_LEADER swig_types[194]
#define SWIGTYPE_p__dwg_entity_LIGHT swig_types[195]
#define SWIGTYPE_p__dwg_entity_LINE swig_types[196]
#define SWIGTYPE_p__dwg_entity_LINEARGRIPENTITY swig_types[197]
#define SWIGTYPE_p__dwg_entity_LINEARPARAMETERENTITY swig_types[198]
#define SWIGTYPE_p__dwg_entity_LOAD swig_types[199]
#define SWIGTYPE_p__dwg_entity_LOFTEDSURFACE swig_types[200]
#define SWIGTYPE_p__dwg_entity_LWPOLYLINE swig_types[201]
#define SWIGTYPE_p__dwg_entity_MESH swig_types[202]
#define SWIGTYPE_p__dwg_entity_MINSERT swig_types[203]
#define SWIGTYPE_p__dwg_entity_MLINE swig_types[204]
#define SWIGTYPE_p__dwg_entity_MPOLYGON swig_types[205]
#define SWIGTYPE_p__dwg_entity_MTEXT swig_types[206]
#define SWIGTYPE_p__dwg_entity_MULTILEADER swig_types[207]
#define SWIGTYPE_p__dwg_entity_NAVISWORKSMODEL swig_types[208]
#define SWIGTYPE_p__dwg_entity_NURBSURFACE swig_types[209]
#define SWIGTYPE_p__dwg_entity_OLE2FRAME swig_types[210]
#define SWIGTYPE_p__dwg_entity_OLEFRAME swig_types[211]
#define SWIGTYPE_p__dwg_entity_PLANESURFACE swig_types[212]
#define SWIGTYPE_p__dwg_entity_POINT swig_types[213]
#define SWIGTYPE_p__dwg_entity_POINTCLOUD swig_types[214]
#define SWIGTYPE_p__dwg_entity_POINTCLOUDEX swig_types[215]
#define SWIGTYPE_p__dwg_entity_POINTPARAMETERENTITY swig_types[216]
#define SWIGTYPE_p__dwg_entity_POLARGRIPENTITY swig_types[217]
#define SWIGTYPE_p__dwg_entity_POLYLINE_2D swig_types[218]
#define SWIGTYPE_p__dwg_entity_POLYLINE_3D swig_types[219]
#define SWIGTYPE_p__dwg_entity_POLYLINE_MESH swig_types[220]
#define SWIGTYPE_p__dwg_entity_POLYLINE_PFACE swig_types[221]
#define SWIGTYPE_p__dwg_entity_PROXY_ENTITY swig_types[222]
#define SWIGTYPE_p__dwg_entity_RAY swig_types[223]
#define SWIGTYPE_p__dwg_entity_REPEAT swig_types[224]
#define SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE swig_types[225]
#define SWIGTYPE_p__dwg_entity_ROTATIONGRIPENTITY swig_types[226]
#define SWIGTYPE_p__dwg_entity_ROTATIONPARAMETERENTITY swig_types[227]
#define SWIGTYPE_p__dwg_entity_RTEXT swig_types[228]
#define SWIGTYPE_p__dwg_entity_SECTIONOBJECT swig_types[229]
#define SWIGTYPE_p__dwg_entity_SEQEND swig_types[230]
#define SWIGTYPE_p__dwg_entity_SHAPE swig_types[231]
#define SWIGTYPE_p__dwg_entity_SOLID swig_types[232]
#define SWIGTYPE_p__dwg_entity_SPLINE swig_types[233]
#define SWIGTYPE_p__dwg_entity_SWEPTSURFACE swig_types[234]
#define SWIGTYPE_p__dwg_entity_TABLE swig_types[235]
#define SWIGTYPE_p__dwg_entity_TEXT swig_types[236]
#define SWIGTYPE_p__dwg_entity_TOLERANCE swig_types[237]
#define SWIGTYPE_p__dwg_entity_TRACE swig_types[238]
#define SWIGTYPE_p__dwg_entity_UNKNOWN_ENT swig_types[239]
#define SWIGTYPE_p__dwg_entity_VERTEX_2D swig_types[240]
#define SWIGTYPE_p__dwg_entity_VERTEX_3D swig_types[241]
#define SWIGTYPE_p__dwg_entity_VERTEX_PFACE_FACE swig_types[242]
#define SWIGTYPE_p__dwg_entity_VIEWPORT swig_types[243]
#define SWIGTYPE_p__dwg_entity_VISIBILITYGRIPENTITY swig_types[244]
#define SWIGTYPE_p__dwg_entity_VISIBILITYPARAMETERENTITY swig_types[245]
#define SWIGTYPE_p__dwg_entity_WIPEOUT swig_types[246]
#define SWIGTYPE_p__dwg_entity_XYGRIPENTITY swig_types[247]
#define SWIGTYPE_p__dwg_entity_XYPARAMETERENTITY swig_types[248]
#define SWIGTYPE_p__dwg_entity_eed swig_types[249]
#define SWIGTYPE_p__dwg_entity_eed_data swig_types[250]
#define SWIGTYPE_p__dwg_filedeplist swig_types[251]
#define SWIGTYPE_p__dwg_handle swig_types[252]
#define SWIGTYPE_p__dwg_header swig_types[253]
#define SWIGTYPE_p__dwg_header_variables swig_types[254]
#define SWIGTYPE_p__dwg_object swig_types[255]
#define SWIGTYPE_p__dwg_object_ACMECOMMANDHISTORY swig_types[256]
#define SWIGTYPE_p__dwg_object_ACMESCOPE swig_types[257]
#define SWIGTYPE_p__dwg_object_ACMESTATEMGR swig_types[258]
#define SWIGTYPE_p__dwg_object_ACSH_BOOLEAN_CLASS swig_types[259]
#define SWIGTYPE_p__dwg_object_ACSH_BOX_CLASS swig_types[260]
#define SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS swig_types[261]
#define SWIGTYPE_p__dwg_object_ACSH_CHAMFER_CLASS swig_types[262]
#define SWIGTYPE_p__dwg_object_ACSH_CONE_CLASS swig_types[263]
#define SWIGTYPE_p__dwg_object_ACSH_CYLINDER_CLASS swig_types[264]
#define SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS swig_types[265]
#define SWIGTYPE_p__dwg_object_ACSH_FILLET_CLASS swig_types[266]
#define SWIGTYPE_p__dwg_object_ACSH_HISTORY_CLASS swig_types[267]
#define SWIGTYPE_p__dwg_object_ACSH_LOFT_CLASS swig_types[268]
#define SWIGTYPE_p__dwg_object_ACSH_PYRAMID_CLASS swig_types[269]
#define SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS swig_types[270]
#define SWIGTYPE_p__dwg_object_ACSH_SPHERE_CLASS swig_types[271]
#define SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS swig_types[272]
#define SWIGTYPE_p__dwg_object_ACSH_TORUS_CLASS swig_types[273]
#define SWIGTYPE_p__dwg_object_ACSH_WEDGE_CLASS swig_types[274]
#define SWIGTYPE_p__dwg_object_ALDIMOBJECTCONTEXTDATA swig_types[275]
#define SWIGTYPE_p__dwg_object_ANGDIMOBJECTCONTEXTDATA swig_types[276]
#define SWIGTYPE_p__dwg_object_ANNOTSCALEOBJECTCONTEXTDATA swig_types[277]
#define SWIGTYPE_p__dwg_object_APPID swig_types[278]
#define SWIGTYPE_p__dwg_object_APPID_CONTROL swig_types[279]
#define SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP swig_types[280]
#define SWIGTYPE_p__dwg_object_ASSOC3POINTANGULARDIMACTIONBODY swig_types[281]
#define SWIGTYPE_p__dwg_object_ASSOCACTION swig_types[282]
#define SWIGTYPE_p__dwg_object_ASSOCACTIONPARAM swig_types[283]
#define SWIGTYPE_p__dwg_object_ASSOCALIGNEDDIMACTIONBODY swig_types[284]
#define SWIGTYPE_p__dwg_object_ASSOCARRAYACTIONBODY swig_types[285]
#define SWIGTYPE_p__dwg_object_ASSOCARRAYMODIFYACTIONBODY swig_types[286]
#define SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM swig_types[287]
#define SWIGTYPE_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY swig_types[288]
#define SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM swig_types[289]
#define SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY swig_types[290]
#define SWIGTYPE_p__dwg_object_ASSOCDIMDEPENDENCYBODY swig_types[291]
#define SWIGTYPE_p__dwg_object_ASSOCEDGEACTIONPARAM swig_types[292]
#define SWIGTYPE_p__dwg_object_ASSOCEDGECHAMFERACTIONBODY swig_types[293]
#define SWIGTYPE_p__dwg_object_ASSOCEDGEFILLETACTIONBODY swig_types[294]
#define SWIGTYPE_p__dwg_object_ASSOCEXTENDSURFACEACTIONBODY swig_types[295]
#define SWIGTYPE_p__dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY swig_types[296]
#define SWIGTYPE_p__dwg_object_ASSOCFACEACTIONPARAM swig_types[297]
#define SWIGTYPE_p__dwg_object_ASSOCFILLETSURFACEACTIONBODY swig_types[298]
#define SWIGTYPE_p__dwg_object_ASSOCGEOMDEPENDENCY swig_types[299]
#define SWIGTYPE_p__dwg_object_ASSOCLOFTEDSURFACEACTIONBODY swig_types[300]
#define SWIGTYPE_p__dwg_object_ASSOCMLEADERACTIONBODY swig_types[301]
#define SWIGTYPE_p__dwg_object_ASSOCNETWORK swig_types[302]
#define SWIGTYPE_p__dwg_object_ASSOCNETWORKSURFACEACTIONBODY swig_types[303]
#define SWIGTYPE_p__dwg_object_ASSOCOBJECTACTIONPARAM swig_types[304]
#define SWIGTYPE_p__dwg_object_ASSOCOFFSETSURFACEACTIONBODY swig_types[305]
#define SWIGTYPE_p__dwg_object_ASSOCORDINATEDIMACTIONBODY swig_types[306]
#define SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM swig_types[307]
#define SWIGTYPE_p__dwg_object_ASSOCPATCHSURFACEACTIONBODY swig_types[308]
#define SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM swig_types[309]
#define SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER swig_types[310]
#define SWIGTYPE_p__dwg_object_ASSOCPLANESURFACEACTIONBODY swig_types[311]
#define SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM swig_types[312]
#define SWIGTYPE_p__dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY swig_types[313]
#define SWIGTYPE_p__dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY swig_types[314]
#define SWIGTYPE_p__dwg_object_ASSOCROTATEDDIMACTIONBODY swig_types[315]
#define SWIGTYPE_p__dwg_object_ASSOCSWEPTSURFACEACTIONBODY swig_types[316]
#define SWIGTYPE_p__dwg_object_ASSOCTRIMSURFACEACTIONBODY swig_types[317]
#define SWIGTYPE_p__dwg_object_ASSOCVALUEDEPENDENCY swig_types[318]
#define SWIGTYPE_p__dwg_object_ASSOCVARIABLE swig_types[319]
#define SWIGTYPE_p__dwg_object_ASSOCVERTEXACTIONPARAM swig_types[320]
#define SWIGTYPE_p__dwg_object_BLKREFOBJECTCONTEXTDATA swig_types[321]
#define SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER swig_types[322]
#define SWIGTYPE_p__dwg_object_BLOCKALIGNMENTGRIP swig_types[323]
#define SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER swig_types[324]
#define SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER swig_types[325]
#define SWIGTYPE_p__dwg_object_BLOCKARRAYACTION swig_types[326]
#define SWIGTYPE_p__dwg_object_BLOCKBASEPOINTPARAMETER swig_types[327]
#define SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER swig_types[328]
#define SWIGTYPE_p__dwg_object_BLOCKFLIPACTION swig_types[329]
#define SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP swig_types[330]
#define SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER swig_types[331]
#define SWIGTYPE_p__dwg_object_BLOCKGRIPLOCATIONCOMPONENT swig_types[332]
#define SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER swig_types[333]
#define SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER swig_types[334]
#define SWIGTYPE_p__dwg_object_BLOCKLINEARGRIP swig_types[335]
#define SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER swig_types[336]
#define SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION swig_types[337]
#define SWIGTYPE_p__dwg_object_BLOCKLOOKUPGRIP swig_types[338]
#define SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER swig_types[339]
#define SWIGTYPE_p__dwg_object_BLOCKMOVEACTION swig_types[340]
#define SWIGTYPE_p__dwg_object_BLOCKPARAMDEPENDENCYBODY swig_types[341]
#define SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER swig_types[342]
#define SWIGTYPE_p__dwg_object_BLOCKPOLARGRIP swig_types[343]
#define SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER swig_types[344]
#define SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION swig_types[345]
#define SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLE swig_types[346]
#define SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLEGRIP swig_types[347]
#define SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER swig_types[348]
#define SWIGTYPE_p__dwg_object_BLOCKREPRESENTATION swig_types[349]
#define SWIGTYPE_p__dwg_object_BLOCKROTATEACTION swig_types[350]
#define SWIGTYPE_p__dwg_object_BLOCKROTATIONGRIP swig_types[351]
#define SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER swig_types[352]
#define SWIGTYPE_p__dwg_object_BLOCKSCALEACTION swig_types[353]
#define SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION swig_types[354]
#define SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER swig_types[355]
#define SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER swig_types[356]
#define SWIGTYPE_p__dwg_object_BLOCKVISIBILITYGRIP swig_types[357]
#define SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER swig_types[358]
#define SWIGTYPE_p__dwg_object_BLOCKXYGRIP swig_types[359]
#define SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER swig_types[360]
#define SWIGTYPE_p__dwg_object_BLOCK_CONTROL swig_types[361]
#define SWIGTYPE_p__dwg_object_BLOCK_HEADER swig_types[362]
#define SWIGTYPE_p__dwg_object_BREAKDATA swig_types[363]
#define SWIGTYPE_p__dwg_object_BREAKPOINTREF swig_types[364]
#define SWIGTYPE_p__dwg_object_CELLSTYLEMAP swig_types[365]
#define SWIGTYPE_p__dwg_object_CONTEXTDATAMANAGER swig_types[366]
#define SWIGTYPE_p__dwg_object_CSACDOCUMENTOPTIONS swig_types[367]
#define SWIGTYPE_p__dwg_object_CURVEPATH swig_types[368]
#define SWIGTYPE_p__dwg_object_DATALINK swig_types[369]
#define SWIGTYPE_p__dwg_object_DATATABLE swig_types[370]
#define SWIGTYPE_p__dwg_object_DBCOLOR swig_types[371]
#define SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE swig_types[372]
#define SWIGTYPE_p__dwg_object_DICTIONARY swig_types[373]
#define SWIGTYPE_p__dwg_object_DICTIONARYVAR swig_types[374]
#define SWIGTYPE_p__dwg_object_DICTIONARYWDFLT swig_types[375]
#define SWIGTYPE_p__dwg_object_DIMASSOC swig_types[376]
#define SWIGTYPE_p__dwg_object_DIMSTYLE swig_types[377]
#define SWIGTYPE_p__dwg_object_DIMSTYLE_CONTROL swig_types[378]
#define SWIGTYPE_p__dwg_object_DMDIMOBJECTCONTEXTDATA swig_types[379]
#define SWIGTYPE_p__dwg_object_DUMMY swig_types[380]
#define SWIGTYPE_p__dwg_object_DYNAMICBLOCKPROXYNODE swig_types[381]
#define SWIGTYPE_p__dwg_object_DYNAMICBLOCKPURGEPREVENTER swig_types[382]
#define SWIGTYPE_p__dwg_object_EVALUATION_GRAPH swig_types[383]
#define SWIGTYPE_p__dwg_object_FCFOBJECTCONTEXTDATA swig_types[384]
#define SWIGTYPE_p__dwg_object_FIELD swig_types[385]
#define SWIGTYPE_p__dwg_object_FIELDLIST swig_types[386]
#define SWIGTYPE_p__dwg_object_GEODATA swig_types[387]
#define SWIGTYPE_p__dwg_object_GEOMAPIMAGE swig_types[388]
#define SWIGTYPE_p__dwg_object_GRADIENT_BACKGROUND swig_types[389]
#define SWIGTYPE_p__dwg_object_GROUND_PLANE_BACKGROUND swig_types[390]
#define SWIGTYPE_p__dwg_object_GROUP swig_types[391]
#define SWIGTYPE_p__dwg_object_IBL_BACKGROUND swig_types[392]
#define SWIGTYPE_p__dwg_object_IDBUFFER swig_types[393]
#define SWIGTYPE_p__dwg_object_IMAGEDEF swig_types[394]
#define SWIGTYPE_p__dwg_object_IMAGEDEF_REACTOR swig_types[395]
#define SWIGTYPE_p__dwg_object_IMAGE_BACKGROUND swig_types[396]
#define SWIGTYPE_p__dwg_object_INDEX swig_types[397]
#define SWIGTYPE_p__dwg_object_LAYER swig_types[398]
#define SWIGTYPE_p__dwg_object_LAYERFILTER swig_types[399]
#define SWIGTYPE_p__dwg_object_LAYER_CONTROL swig_types[400]
#define SWIGTYPE_p__dwg_object_LAYER_INDEX swig_types[401]
#define SWIGTYPE_p__dwg_object_LAYOUT swig_types[402]
#define SWIGTYPE_p__dwg_object_LEADEROBJECTCONTEXTDATA swig_types[403]
#define SWIGTYPE_p__dwg_object_LIGHTLIST swig_types[404]
#define SWIGTYPE_p__dwg_object_LONG_TRANSACTION swig_types[405]
#define SWIGTYPE_p__dwg_object_LTYPE swig_types[406]
#define SWIGTYPE_p__dwg_object_LTYPE_CONTROL swig_types[407]
#define SWIGTYPE_p__dwg_object_MATERIAL swig_types[408]
#define SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS swig_types[409]
#define SWIGTYPE_p__dwg_object_MLEADEROBJECTCONTEXTDATA swig_types[410]
#define SWIGTYPE_p__dwg_object_MLEADERSTYLE swig_types[411]
#define SWIGTYPE_p__dwg_object_MLINESTYLE swig_types[412]
#define SWIGTYPE_p__dwg_object_MOTIONPATH swig_types[413]
#define SWIGTYPE_p__dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA swig_types[414]
#define SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA swig_types[415]
#define SWIGTYPE_p__dwg_object_NAVISWORKSMODELDEF swig_types[416]
#define SWIGTYPE_p__dwg_object_OBJECT_PTR swig_types[417]
#define SWIGTYPE_p__dwg_object_ORDDIMOBJECTCONTEXTDATA swig_types[418]
#define SWIGTYPE_p__dwg_object_PARTIAL_VIEWING_INDEX swig_types[419]
#define SWIGTYPE_p__dwg_object_PERSUBENTMGR swig_types[420]
#define SWIGTYPE_p__dwg_object_PLACEHOLDER swig_types[421]
#define SWIGTYPE_p__dwg_object_PLOTSETTINGS swig_types[422]
#define SWIGTYPE_p__dwg_object_POINTCLOUDCOLORMAP swig_types[423]
#define SWIGTYPE_p__dwg_object_POINTCLOUDDEF swig_types[424]
#define SWIGTYPE_p__dwg_object_POINTCLOUDDEFEX swig_types[425]
#define SWIGTYPE_p__dwg_object_POINTCLOUDDEF_REACTOR swig_types[426]
#define SWIGTYPE_p__dwg_object_POINTCLOUDDEF_REACTOR_EX swig_types[427]
#define SWIGTYPE_p__dwg_object_POINTPATH swig_types[428]
#define SWIGTYPE_p__dwg_object_PROXY_OBJECT swig_types[429]
#define SWIGTYPE_p__dwg_object_RADIMLGOBJECTCONTEXTDATA swig_types[430]
#define SWIGTYPE_p__dwg_object_RADIMOBJECTCONTEXTDATA swig_types[431]
#define SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS swig_types[432]
#define SWIGTYPE_p__dwg_object_RASTERVARIABLES swig_types[433]
#define SWIGTYPE_p__dwg_object_RENDERENTRY swig_types[434]
#define SWIGTYPE_p__dwg_object_RENDERENVIRONMENT swig_types[435]
#define SWIGTYPE_p__dwg_object_RENDERGLOBAL swig_types[436]
#define SWIGTYPE_p__dwg_object_RENDERSETTINGS swig_types[437]
#define SWIGTYPE_p__dwg_object_SCALE swig_types[438]
#define SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE swig_types[439]
#define SWIGTYPE_p__dwg_object_SECTION_MANAGER swig_types[440]
#define SWIGTYPE_p__dwg_object_SECTION_SETTINGS swig_types[441]
#define SWIGTYPE_p__dwg_object_SKYLIGHT_BACKGROUND swig_types[442]
#define SWIGTYPE_p__dwg_object_SOLID_BACKGROUND swig_types[443]
#define SWIGTYPE_p__dwg_object_SORTENTSTABLE swig_types[444]
#define SWIGTYPE_p__dwg_object_SPATIAL_FILTER swig_types[445]
#define SWIGTYPE_p__dwg_object_SPATIAL_INDEX swig_types[446]
#define SWIGTYPE_p__dwg_object_STYLE swig_types[447]
#define SWIGTYPE_p__dwg_object_STYLE_CONTROL swig_types[448]
#define SWIGTYPE_p__dwg_object_SUN swig_types[449]
#define SWIGTYPE_p__dwg_object_SUNSTUDY swig_types[450]
#define SWIGTYPE_p__dwg_object_TABLECONTENT swig_types[451]
#define SWIGTYPE_p__dwg_object_TABLEGEOMETRY swig_types[452]
#define SWIGTYPE_p__dwg_object_TABLESTYLE swig_types[453]
#define SWIGTYPE_p__dwg_object_TEXTOBJECTCONTEXTDATA swig_types[454]
#define SWIGTYPE_p__dwg_object_TVDEVICEPROPERTIES swig_types[455]
#define SWIGTYPE_p__dwg_object_UCS swig_types[456]
#define SWIGTYPE_p__dwg_object_UCS_CONTROL swig_types[457]
#define SWIGTYPE_p__dwg_object_UNKNOWN_OBJ swig_types[458]
#define SWIGTYPE_p__dwg_object_VBA_PROJECT swig_types[459]
#define SWIGTYPE_p__dwg_object_VIEW swig_types[460]
#define SWIGTYPE_p__dwg_object_VIEW_CONTROL swig_types[461]
#define SWIGTYPE_p__dwg_object_VISUALSTYLE swig_types[462]
#define SWIGTYPE_p__dwg_object_VPORT swig_types[463]
#define SWIGTYPE_p__dwg_object_VPORT_CONTROL swig_types[464]
#define SWIGTYPE_p__dwg_object_VX_CONTROL swig_types[465]
#define SWIGTYPE_p__dwg_object_VX_TABLE_RECORD swig_types[466]
#define SWIGTYPE_p__dwg_object_WIPEOUTVARIABLES swig_types[467]
#define SWIGTYPE_p__dwg_object_XRECORD swig_types[468]
#define SWIGTYPE_p__dwg_object_entity swig_types[469]
#define SWIGTYPE_p__dwg_object_entity_tio swig_types[470]
#define SWIGTYPE_p__dwg_object_object swig_types[471]
#define SWIGTYPE_p__dwg_object_object_tio swig_types[472]
#define SWIGTYPE_p__dwg_object_ref swig_types[473]
#define SWIGTYPE_p__dwg_object_tio swig_types[474]
#define SWIGTYPE_p__dwg_objfreespace swig_types[475]
#define SWIGTYPE_p__dwg_resbuf swig_types[476]
#define SWIGTYPE_p__dwg_resbuf_value swig_types[477]
#define SWIGTYPE_p__dwg_revhistory swig_types[478]
#define SWIGTYPE_p__dwg_secondheader swig_types[479]
#define SWIGTYPE_p__dwg_secondheader_handles swig_types[480]
#define SWIGTYPE_p__dwg_secondheader_sections swig_types[481]
#define SWIGTYPE_p__dwg_section swig_types[482]
#define SWIGTYPE_p__dwg_security swig_types[483]
#define SWIGTYPE_p__dwg_struct swig_types[484]
#define SWIGTYPE_p__dwg_struct_fhdr swig_types[485]
#define SWIGTYPE_p__dwg_summaryinfo swig_types[486]
#define SWIGTYPE_p__dwg_template swig_types[487]
#define SWIGTYPE_p__dwg_time_bll swig_types[488]
#define SWIGTYPE_p__dwg_vbaproject swig_types[489]
#define SWIGTYPE_p__inthash swig_types[490]
#define SWIGTYPE_p_char swig_types[491]
#define SWIGTYPE_p_double swig_types[492]
#define SWIGTYPE_p_dwg_entity_DIMENSION_common swig_types[493]
#define SWIGTYPE_p_dwg_field_name_type_offset swig_types[494]
#define SWIGTYPE_p_dwg_obj_ent swig_types[495]
#define SWIGTYPE_p_dwg_obj_generic swig_types[496]
#define SWIGTYPE_p_dwg_obj_layer swig_types[497]
#define SWIGTYPE_p_dwg_obj_obj swig_types[498]
#define SWIGTYPE_p_dwg_object swig_types[499]
#define SWIGTYPE_p_dwg_versions swig_types[500]
#define SWIGTYPE_p_eed_data_t swig_types[501]
#define SWIGTYPE_p_eed_data_t_eed_0 swig_types[502]
#define SWIGTYPE_p_eed_data_t_eed_0_r2007 swig_types[503]
#define SWIGTYPE_p_eed_data_t_eed_1 swig_types[504]
#define SWIGTYPE_p_eed_data_t_eed_10 swig_types[505]
#define SWIGTYPE_p_eed_data_t_eed_2 swig_types[506]
#define SWIGTYPE_p_eed_data_t_eed_3 swig_types[507]
#define SWIGTYPE_p_eed_data_t_eed_4 swig_types[508]
#define SWIGTYPE_p_eed_data_t_eed_40 swig_types[509]
#define SWIGTYPE_p_eed_data_t_eed_5 swig_types[510]
#define SWIGTYPE_p_eed_data_t_eed_70 swig_types[511]
#define SWIGTYPE_p_eed_data_t_eed_71 swig_types[512]
#define SWIGTYPE_p_int swig_types[513]
#define SWIGTYPE_p_long_long swig_types[514]
#define SWIGTYPE_p_p__dwg_abstractentity_UNDERLAY swig_types[515]
#define SWIGTYPE_p_p__dwg_abstractobject_ASSOCARRAYPARAMETERS swig_types[516]
#define SWIGTYPE_p_p__dwg_abstractobject_UNDERLAYDEFINITION swig_types[517]
#define SWIGTYPE_p_p__dwg_entity_3DFACE swig_types[518]
#define SWIGTYPE_p_p__dwg_entity_3DLINE swig_types[519]
#define SWIGTYPE_p_p__dwg_entity_3DSOLID swig_types[520]
#define SWIGTYPE_p_p__dwg_entity_ARC swig_types[521]
#define SWIGTYPE_p_p__dwg_entity_ARC_DIMENSION swig_types[522]
#define SWIGTYPE_p_p__dwg_entity_ATTDEF swig_types[523]
#define SWIGTYPE_p_p__dwg_entity_ATTRIB swig_types[524]
#define SWIGTYPE_p_p__dwg_entity_BLOCK swig_types[525]
#define SWIGTYPE_p_p__dwg_entity_CAMERA swig_types[526]
#define SWIGTYPE_p_p__dwg_entity_CIRCLE swig_types[527]
#define SWIGTYPE_p_p__dwg_entity_DIMENSION_ALIGNED swig_types[528]
#define SWIGTYPE_p_p__dwg_entity_DIMENSION_ANG2LN swig_types[529]
#define SWIGTYPE_p_p__dwg_entity_DIMENSION_ANG3PT swig_types[530]
#define SWIGTYPE_p_p__dwg_entity_DIMENSION_DIAMETER swig_types[531]
#define SWIGTYPE_p_p__dwg_entity_DIMENSION_LINEAR swig_types[532]
#define SWIGTYPE_p_p__dwg_entity_DIMENSION_ORDINATE swig_types[533]
#define SWIGTYPE_p_p__dwg_entity_DIMENSION_RADIUS swig_types[534]
#define SWIGTYPE_p_p__dwg_entity_ELLIPSE swig_types[535]
#define SWIGTYPE_p_p__dwg_entity_ENDBLK swig_types[536]
#define SWIGTYPE_p_p__dwg_entity_ENDREP swig_types[537]
#define SWIGTYPE_p_p__dwg_entity_HATCH swig_types[538]
#define SWIGTYPE_p_p__dwg_entity_HELIX swig_types[539]
#define SWIGTYPE_p_p__dwg_entity_IMAGE swig_types[540]
#define SWIGTYPE_p_p__dwg_entity_INSERT swig_types[541]
#define SWIGTYPE_p_p__dwg_entity_JUMP swig_types[542]
#define SWIGTYPE_p_p__dwg_entity_LARGE_RADIAL_DIMENSION swig_types[543]
#define SWIGTYPE_p_p__dwg_entity_LAYOUTPRINTCONFIG swig_types[544]
#define SWIGTYPE_p_p__dwg_entity_LEADER swig_types[545]
#define SWIGTYPE_p_p__dwg_entity_LIGHT swig_types[546]
#define SWIGTYPE_p_p__dwg_entity_LINE swig_types[547]
#define SWIGTYPE_p_p__dwg_entity_LOAD swig_types[548]
#define SWIGTYPE_p_p__dwg_entity_LWPOLYLINE swig_types[549]
#define SWIGTYPE_p_p__dwg_entity_MESH swig_types[550]
#define SWIGTYPE_p_p__dwg_entity_MINSERT swig_types[551]
#define SWIGTYPE_p_p__dwg_entity_MLINE swig_types[552]
#define SWIGTYPE_p_p__dwg_entity_MTEXT swig_types[553]
#define SWIGTYPE_p_p__dwg_entity_MULTILEADER swig_types[554]
#define SWIGTYPE_p_p__dwg_entity_OLE2FRAME swig_types[555]
#define SWIGTYPE_p_p__dwg_entity_OLEFRAME swig_types[556]
#define SWIGTYPE_p_p__dwg_entity_PLANESURFACE swig_types[557]
#define SWIGTYPE_p_p__dwg_entity_POINT swig_types[558]
#define SWIGTYPE_p_p__dwg_entity_POINTCLOUD swig_types[559]
#define SWIGTYPE_p_p__dwg_entity_POINTCLOUDEX swig_types[560]
#define SWIGTYPE_p_p__dwg_entity_POLYLINE_2D swig_types[561]
#define SWIGTYPE_p_p__dwg_entity_POLYLINE_3D swig_types[562]
#define SWIGTYPE_p_p__dwg_entity_POLYLINE_MESH swig_types[563]
#define SWIGTYPE_p_p__dwg_entity_POLYLINE_PFACE swig_types[564]
#define SWIGTYPE_p_p__dwg_entity_PROXY_ENTITY swig_types[565]
#define SWIGTYPE_p_p__dwg_entity_RAY swig_types[566]
#define SWIGTYPE_p_p__dwg_entity_REPEAT swig_types[567]
#define SWIGTYPE_p_p__dwg_entity_SECTIONOBJECT swig_types[568]
#define SWIGTYPE_p_p__dwg_entity_SEQEND swig_types[569]
#define SWIGTYPE_p_p__dwg_entity_SHAPE swig_types[570]
#define SWIGTYPE_p_p__dwg_entity_SOLID swig_types[571]
#define SWIGTYPE_p_p__dwg_entity_SPLINE swig_types[572]
#define SWIGTYPE_p_p__dwg_entity_TEXT swig_types[573]
#define SWIGTYPE_p_p__dwg_entity_TOLERANCE swig_types[574]
#define SWIGTYPE_p_p__dwg_entity_TRACE swig_types[575]
#define SWIGTYPE_p_p__dwg_entity_UNKNOWN_ENT swig_types[576]
#define SWIGTYPE_p_p__dwg_entity_VERTEX_2D swig_types[577]
#define SWIGTYPE_p_p__dwg_entity_VERTEX_3D swig_types[578]
#define SWIGTYPE_p_p__dwg_entity_VERTEX_PFACE_FACE swig_types[579]
#define SWIGTYPE_p_p__dwg_entity_VIEWPORT swig_types[580]
#define SWIGTYPE_p_p__dwg_entity_WIPEOUT swig_types[581]
#define SWIGTYPE_p_p__dwg_object_ACSH_BOOLEAN_CLASS swig_types[582]
#define SWIGTYPE_p_p__dwg_object_ACSH_BOX_CLASS swig_types[583]
#define SWIGTYPE_p_p__dwg_object_ACSH_BREP_CLASS swig_types[584]
#define SWIGTYPE_p_p__dwg_object_ACSH_CHAMFER_CLASS swig_types[585]
#define SWIGTYPE_p_p__dwg_object_ACSH_CONE_CLASS swig_types[586]
#define SWIGTYPE_p_p__dwg_object_ACSH_CYLINDER_CLASS swig_types[587]
#define SWIGTYPE_p_p__dwg_object_ACSH_FILLET_CLASS swig_types[588]
#define SWIGTYPE_p_p__dwg_object_ACSH_HISTORY_CLASS swig_types[589]
#define SWIGTYPE_p_p__dwg_object_ACSH_PYRAMID_CLASS swig_types[590]
#define SWIGTYPE_p_p__dwg_object_ACSH_SPHERE_CLASS swig_types[591]
#define SWIGTYPE_p_p__dwg_object_ACSH_TORUS_CLASS swig_types[592]
#define SWIGTYPE_p_p__dwg_object_ACSH_WEDGE_CLASS swig_types[593]
#define SWIGTYPE_p_p__dwg_object_ALDIMOBJECTCONTEXTDATA swig_types[594]
#define SWIGTYPE_p_p__dwg_object_APPID swig_types[595]
#define SWIGTYPE_p_p__dwg_object_APPID_CONTROL swig_types[596]
#define SWIGTYPE_p_p__dwg_object_ASSOC2DCONSTRAINTGROUP swig_types[597]
#define SWIGTYPE_p_p__dwg_object_ASSOCACTION swig_types[598]
#define SWIGTYPE_p_p__dwg_object_ASSOCACTIONPARAM swig_types[599]
#define SWIGTYPE_p_p__dwg_object_ASSOCARRAYACTIONBODY swig_types[600]
#define SWIGTYPE_p_p__dwg_object_ASSOCASMBODYACTIONPARAM swig_types[601]
#define SWIGTYPE_p_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY swig_types[602]
#define SWIGTYPE_p_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM swig_types[603]
#define SWIGTYPE_p_p__dwg_object_ASSOCDEPENDENCY swig_types[604]
#define SWIGTYPE_p_p__dwg_object_ASSOCDIMDEPENDENCYBODY swig_types[605]
#define SWIGTYPE_p_p__dwg_object_ASSOCEXTENDSURFACEACTIONBODY swig_types[606]
#define SWIGTYPE_p_p__dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY swig_types[607]
#define SWIGTYPE_p_p__dwg_object_ASSOCFACEACTIONPARAM swig_types[608]
#define SWIGTYPE_p_p__dwg_object_ASSOCFILLETSURFACEACTIONBODY swig_types[609]
#define SWIGTYPE_p_p__dwg_object_ASSOCGEOMDEPENDENCY swig_types[610]
#define SWIGTYPE_p_p__dwg_object_ASSOCLOFTEDSURFACEACTIONBODY swig_types[611]
#define SWIGTYPE_p_p__dwg_object_ASSOCNETWORK swig_types[612]
#define SWIGTYPE_p_p__dwg_object_ASSOCNETWORKSURFACEACTIONBODY swig_types[613]
#define SWIGTYPE_p_p__dwg_object_ASSOCOBJECTACTIONPARAM swig_types[614]
#define SWIGTYPE_p_p__dwg_object_ASSOCOFFSETSURFACEACTIONBODY swig_types[615]
#define SWIGTYPE_p_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM swig_types[616]
#define SWIGTYPE_p_p__dwg_object_ASSOCPATCHSURFACEACTIONBODY swig_types[617]
#define SWIGTYPE_p_p__dwg_object_ASSOCPATHACTIONPARAM swig_types[618]
#define SWIGTYPE_p_p__dwg_object_ASSOCPLANESURFACEACTIONBODY swig_types[619]
#define SWIGTYPE_p_p__dwg_object_ASSOCPOINTREFACTIONPARAM swig_types[620]
#define SWIGTYPE_p_p__dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY swig_types[621]
#define SWIGTYPE_p_p__dwg_object_ASSOCTRIMSURFACEACTIONBODY swig_types[622]
#define SWIGTYPE_p_p__dwg_object_ASSOCVALUEDEPENDENCY swig_types[623]
#define SWIGTYPE_p_p__dwg_object_ASSOCVARIABLE swig_types[624]
#define SWIGTYPE_p_p__dwg_object_ASSOCVERTEXACTIONPARAM swig_types[625]
#define SWIGTYPE_p_p__dwg_object_BLKREFOBJECTCONTEXTDATA swig_types[626]
#define SWIGTYPE_p_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER swig_types[627]
#define SWIGTYPE_p_p__dwg_object_BLOCKALIGNMENTGRIP swig_types[628]
#define SWIGTYPE_p_p__dwg_object_BLOCKALIGNMENTPARAMETER swig_types[629]
#define SWIGTYPE_p_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER swig_types[630]
#define SWIGTYPE_p_p__dwg_object_BLOCKARRAYACTION swig_types[631]
#define SWIGTYPE_p_p__dwg_object_BLOCKBASEPOINTPARAMETER swig_types[632]
#define SWIGTYPE_p_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER swig_types[633]
#define SWIGTYPE_p_p__dwg_object_BLOCKFLIPACTION swig_types[634]
#define SWIGTYPE_p_p__dwg_object_BLOCKFLIPGRIP swig_types[635]
#define SWIGTYPE_p_p__dwg_object_BLOCKFLIPPARAMETER swig_types[636]
#define SWIGTYPE_p_p__dwg_object_BLOCKGRIPLOCATIONCOMPONENT swig_types[637]
#define SWIGTYPE_p_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER swig_types[638]
#define SWIGTYPE_p_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER swig_types[639]
#define SWIGTYPE_p_p__dwg_object_BLOCKLINEARGRIP swig_types[640]
#define SWIGTYPE_p_p__dwg_object_BLOCKLINEARPARAMETER swig_types[641]
#define SWIGTYPE_p_p__dwg_object_BLOCKLOOKUPACTION swig_types[642]
#define SWIGTYPE_p_p__dwg_object_BLOCKLOOKUPGRIP swig_types[643]
#define SWIGTYPE_p_p__dwg_object_BLOCKLOOKUPPARAMETER swig_types[644]
#define SWIGTYPE_p_p__dwg_object_BLOCKMOVEACTION swig_types[645]
#define SWIGTYPE_p_p__dwg_object_BLOCKPARAMDEPENDENCYBODY swig_types[646]
#define SWIGTYPE_p_p__dwg_object_BLOCKPOINTPARAMETER swig_types[647]
#define SWIGTYPE_p_p__dwg_object_BLOCKPOLARGRIP swig_types[648]
#define SWIGTYPE_p_p__dwg_object_BLOCKPOLARPARAMETER swig_types[649]
#define SWIGTYPE_p_p__dwg_object_BLOCKPOLARSTRETCHACTION swig_types[650]
#define SWIGTYPE_p_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER swig_types[651]
#define SWIGTYPE_p_p__dwg_object_BLOCKREPRESENTATION swig_types[652]
#define SWIGTYPE_p_p__dwg_object_BLOCKROTATEACTION swig_types[653]
#define SWIGTYPE_p_p__dwg_object_BLOCKROTATIONGRIP swig_types[654]
#define SWIGTYPE_p_p__dwg_object_BLOCKROTATIONPARAMETER swig_types[655]
#define SWIGTYPE_p_p__dwg_object_BLOCKSCALEACTION swig_types[656]
#define SWIGTYPE_p_p__dwg_object_BLOCKSTRETCHACTION swig_types[657]
#define SWIGTYPE_p_p__dwg_object_BLOCKUSERPARAMETER swig_types[658]
#define SWIGTYPE_p_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER swig_types[659]
#define SWIGTYPE_p_p__dwg_object_BLOCKVISIBILITYGRIP swig_types[660]
#define SWIGTYPE_p_p__dwg_object_BLOCKVISIBILITYPARAMETER swig_types[661]
#define SWIGTYPE_p_p__dwg_object_BLOCKXYGRIP swig_types[662]
#define SWIGTYPE_p_p__dwg_object_BLOCKXYPARAMETER swig_types[663]
#define SWIGTYPE_p_p__dwg_object_BLOCK_CONTROL swig_types[664]
#define SWIGTYPE_p_p__dwg_object_BLOCK_HEADER swig_types[665]
#define SWIGTYPE_p_p__dwg_object_CELLSTYLEMAP swig_types[666]
#define SWIGTYPE_p_p__dwg_object_DATALINK swig_types[667]
#define SWIGTYPE_p_p__dwg_object_DBCOLOR swig_types[668]
#define SWIGTYPE_p_p__dwg_object_DETAILVIEWSTYLE swig_types[669]
#define SWIGTYPE_p_p__dwg_object_DICTIONARY swig_types[670]
#define SWIGTYPE_p_p__dwg_object_DICTIONARYVAR swig_types[671]
#define SWIGTYPE_p_p__dwg_object_DICTIONARYWDFLT swig_types[672]
#define SWIGTYPE_p_p__dwg_object_DIMSTYLE swig_types[673]
#define SWIGTYPE_p_p__dwg_object_DIMSTYLE_CONTROL swig_types[674]
#define SWIGTYPE_p_p__dwg_object_DUMMY swig_types[675]
#define SWIGTYPE_p_p__dwg_object_DYNAMICBLOCKPURGEPREVENTER swig_types[676]
#define SWIGTYPE_p_p__dwg_object_EVALUATION_GRAPH swig_types[677]
#define SWIGTYPE_p_p__dwg_object_FCFOBJECTCONTEXTDATA swig_types[678]
#define SWIGTYPE_p_p__dwg_object_FIELD swig_types[679]
#define SWIGTYPE_p_p__dwg_object_FIELDLIST swig_types[680]
#define SWIGTYPE_p_p__dwg_object_GEODATA swig_types[681]
#define SWIGTYPE_p_p__dwg_object_GRADIENT_BACKGROUND swig_types[682]
#define SWIGTYPE_p_p__dwg_object_GROUND_PLANE_BACKGROUND swig_types[683]
#define SWIGTYPE_p_p__dwg_object_GROUP swig_types[684]
#define SWIGTYPE_p_p__dwg_object_IBL_BACKGROUND swig_types[685]
#define SWIGTYPE_p_p__dwg_object_IDBUFFER swig_types[686]
#define SWIGTYPE_p_p__dwg_object_IMAGEDEF swig_types[687]
#define SWIGTYPE_p_p__dwg_object_IMAGEDEF_REACTOR swig_types[688]
#define SWIGTYPE_p_p__dwg_object_IMAGE_BACKGROUND swig_types[689]
#define SWIGTYPE_p_p__dwg_object_INDEX swig_types[690]
#define SWIGTYPE_p_p__dwg_object_LAYER swig_types[691]
#define SWIGTYPE_p_p__dwg_object_LAYERFILTER swig_types[692]
#define SWIGTYPE_p_p__dwg_object_LAYER_CONTROL swig_types[693]
#define SWIGTYPE_p_p__dwg_object_LAYER_INDEX swig_types[694]
#define SWIGTYPE_p_p__dwg_object_LAYOUT swig_types[695]
#define SWIGTYPE_p_p__dwg_object_LEADEROBJECTCONTEXTDATA swig_types[696]
#define SWIGTYPE_p_p__dwg_object_LIGHTLIST swig_types[697]
#define SWIGTYPE_p_p__dwg_object_LONG_TRANSACTION swig_types[698]
#define SWIGTYPE_p_p__dwg_object_LTYPE swig_types[699]
#define SWIGTYPE_p_p__dwg_object_LTYPE_CONTROL swig_types[700]
#define SWIGTYPE_p_p__dwg_object_MATERIAL swig_types[701]
#define SWIGTYPE_p_p__dwg_object_MENTALRAYRENDERSETTINGS swig_types[702]
#define SWIGTYPE_p_p__dwg_object_MLEADERSTYLE swig_types[703]
#define SWIGTYPE_p_p__dwg_object_MLINESTYLE swig_types[704]
#define SWIGTYPE_p_p__dwg_object_MTEXTOBJECTCONTEXTDATA swig_types[705]
#define SWIGTYPE_p_p__dwg_object_OBJECT_PTR swig_types[706]
#define SWIGTYPE_p_p__dwg_object_PARTIAL_VIEWING_INDEX swig_types[707]
#define SWIGTYPE_p_p__dwg_object_PLACEHOLDER swig_types[708]
#define SWIGTYPE_p_p__dwg_object_PLOTSETTINGS swig_types[709]
#define SWIGTYPE_p_p__dwg_object_POINTCLOUDCOLORMAP swig_types[710]
#define SWIGTYPE_p_p__dwg_object_POINTCLOUDDEF swig_types[711]
#define SWIGTYPE_p_p__dwg_object_POINTCLOUDDEFEX swig_types[712]
#define SWIGTYPE_p_p__dwg_object_POINTCLOUDDEF_REACTOR swig_types[713]
#define SWIGTYPE_p_p__dwg_object_POINTCLOUDDEF_REACTOR_EX swig_types[714]
#define SWIGTYPE_p_p__dwg_object_PROXY_OBJECT swig_types[715]
#define SWIGTYPE_p_p__dwg_object_RAPIDRTRENDERSETTINGS swig_types[716]
#define SWIGTYPE_p_p__dwg_object_RASTERVARIABLES swig_types[717]
#define SWIGTYPE_p_p__dwg_object_RENDERENTRY swig_types[718]
#define SWIGTYPE_p_p__dwg_object_RENDERENVIRONMENT swig_types[719]
#define SWIGTYPE_p_p__dwg_object_RENDERGLOBAL swig_types[720]
#define SWIGTYPE_p_p__dwg_object_RENDERSETTINGS swig_types[721]
#define SWIGTYPE_p_p__dwg_object_SCALE swig_types[722]
#define SWIGTYPE_p_p__dwg_object_SECTIONVIEWSTYLE swig_types[723]
#define SWIGTYPE_p_p__dwg_object_SECTION_MANAGER swig_types[724]
#define SWIGTYPE_p_p__dwg_object_SECTION_SETTINGS swig_types[725]
#define SWIGTYPE_p_p__dwg_object_SKYLIGHT_BACKGROUND swig_types[726]
#define SWIGTYPE_p_p__dwg_object_SOLID_BACKGROUND swig_types[727]
#define SWIGTYPE_p_p__dwg_object_SORTENTSTABLE swig_types[728]
#define SWIGTYPE_p_p__dwg_object_SPATIAL_FILTER swig_types[729]
#define SWIGTYPE_p_p__dwg_object_SPATIAL_INDEX swig_types[730]
#define SWIGTYPE_p_p__dwg_object_STYLE swig_types[731]
#define SWIGTYPE_p_p__dwg_object_STYLE_CONTROL swig_types[732]
#define SWIGTYPE_p_p__dwg_object_SUN swig_types[733]
#define SWIGTYPE_p_p__dwg_object_TABLEGEOMETRY swig_types[734]
#define SWIGTYPE_p_p__dwg_object_TABLESTYLE swig_types[735]
#define SWIGTYPE_p_p__dwg_object_TEXTOBJECTCONTEXTDATA swig_types[736]
#define SWIGTYPE_p_p__dwg_object_UCS swig_types[737]
#define SWIGTYPE_p_p__dwg_object_UCS_CONTROL swig_types[738]
#define SWIGTYPE_p_p__dwg_object_UNKNOWN_OBJ swig_types[739]
#define SWIGTYPE_p_p__dwg_object_VBA_PROJECT swig_types[740]
#define SWIGTYPE_p_p__dwg_object_VIEW swig_types[741]
#define SWIGTYPE_p_p__dwg_object_VIEW_CONTROL swig_types[742]
#define SWIGTYPE_p_p__dwg_object_VISUALSTYLE swig_types[743]
#define SWIGTYPE_p_p__dwg_object_VPORT swig_types[744]
#define SWIGTYPE_p_p__dwg_object_VPORT_CONTROL swig_types[745]
#define SWIGTYPE_p_p__dwg_object_VX_CONTROL swig_types[746]
#define SWIGTYPE_p_p__dwg_object_VX_TABLE_RECORD swig_types[747]
#define SWIGTYPE_p_p__dwg_object_WIPEOUTVARIABLES swig_types[748]
#define SWIGTYPE_p_p__dwg_object_XRECORD swig_types[749]
#define SWIGTYPE_p_p__dwg_object_entity swig_types[750]
#define SWIGTYPE_p_p__dwg_object_ref swig_types[751]
#define SWIGTYPE_p_p__dwg_section swig_types[752]
#define SWIGTYPE_p_p_char swig_types[753]
#define SWIGTYPE_p_r2007_file_header swig_types[754]
#define SWIGTYPE_p_rgbpalette swig_types[755]
#define SWIGTYPE_p_short swig_types[756]
#define SWIGTYPE_p_signed_char swig_types[757]
#define SWIGTYPE_p_unsigned_char swig_types[758]
#define SWIGTYPE_p_unsigned_int swig_types[759]
#define SWIGTYPE_p_unsigned_long_long swig_types[760]
#define SWIGTYPE_p_unsigned_short swig_types[761]
static swig_type_info *swig_types[763];
static swig_module_info swig_module = {swig_types, 762, 0, 0, 0, 0};
#define SWIG_TypeQuery(name) SWIG_TypeQueryModule(&swig_module, &swig_module, name)
#define SWIG_MangledTypeQuery(name) SWIG_MangledTypeQueryModule(&swig_module, &swig_module, name)

/* -------- TYPES TABLE (END) -------- */

#define SWIG_init    boot_LibreDWG

#define SWIG_name   "LibreDWGc::boot_LibreDWG"
#define SWIG_prefix "LibreDWGc::"

#define SWIG_as_voidptr(a) (void *)((const void *)(a)) 
#define SWIG_as_voidptrptr(a) ((void)SWIG_as_voidptr(*a),(void**)(a)) 


#ifdef __cplusplus
extern "C"
#endif
#ifndef MULTIPLICITY
SWIGEXPORT void SWIG_init (CV* cv);
#else
SWIGEXPORT void SWIG_init (pTHXo_ CV* cv);
#endif


#include "../src/config.h"
#include "dwg_api.h"


#include <stdint.h>		// Use the C99 official header


  static Dwg_Object *new_Dwg_Object_Array(size_t nelements) { 
    return (Dwg_Object *)calloc(nelements, sizeof(Dwg_Object));
  }

  static void delete_Dwg_Object_Array(Dwg_Object *ary) {
    free((char*)ary);
  }

  static Dwg_Object Dwg_Object_Array_getitem(Dwg_Object *ary, size_t index) {
    return ary[index];
  }
  static void Dwg_Object_Array_setitem(Dwg_Object *ary, size_t index, Dwg_Object value) {
    ary[index] = value;
  }


#include <limits.h>
#if !defined(SWIG_NO_LLONG_MAX)
# if !defined(LLONG_MAX) && defined(__GNUC__) && defined (__LONG_LONG_MAX__)
#   define LLONG_MAX __LONG_LONG_MAX__
#   define LLONG_MIN (-LLONG_MAX - 1LL)
#   define ULLONG_MAX (LLONG_MAX * 2ULL + 1ULL)
# endif
#endif


#include <stdlib.h>
#ifdef _MSC_VER
# ifndef strtoull
#  define strtoull _strtoui64
# endif
# ifndef strtoll
#  define strtoll _strtoi64
# endif
#endif


SWIGINTERN int
SWIG_AsVal_double SWIG_PERL_DECL_ARGS_2(SV *obj, double *val)
{
  if (SvNIOK(obj)) {
    if (val) *val = SvNV(obj);
    return SWIG_OK;
  } else if (SvIOK(obj)) {
    if (val) *val = (double) SvIV(obj);
    return SWIG_AddCast(SWIG_OK);
  } else {
    const char *nptr = SvPV_nolen(obj);
    if (nptr) {
      char *endptr;
      double v;
      errno = 0;
      v = strtod(nptr, &endptr);
      if (errno == ERANGE) {
	errno = 0;
	return SWIG_OverflowError;
      } else {
	if (*endptr == '\0') {
	  if (val) *val = v;
	  return SWIG_Str2NumCast(SWIG_OK);
	}
      }
    }
  }
  return SWIG_TypeError;
}


#include <float.h>


#include <math.h>


SWIGINTERNINLINE int
SWIG_CanCastAsInteger(double *d, double min, double max) {
  double x = *d;
  if ((min <= x && x <= max)) {
   double fx, cx, rd;
   errno = 0;
   fx = floor(x);
   cx = ceil(x);
   rd =  ((x - fx) < 0.5) ? fx : cx; /* simple rint */
   if ((errno == EDOM) || (errno == ERANGE)) {
     errno = 0;
   } else {
     double summ, reps, diff;
     if (rd < x) {
       diff = x - rd;
     } else if (rd > x) {
       diff = rd - x;
     } else {
       return 1;
     }
     summ = rd + x;
     reps = diff/summ;
     if (reps < 8*DBL_EPSILON) {
       *d = rd;
       return 1;
     }
   }
  }
  return 0;
}


SWIGINTERN int
SWIG_AsVal_unsigned_SS_long SWIG_PERL_DECL_ARGS_2(SV *obj, unsigned long *val) 
{
  if (SvUOK(obj)) {
    UV v = SvUV(obj);
    if (UVSIZE <= sizeof(*val) || v <= ULONG_MAX) {
      if (val) *val = v;
      return SWIG_OK;
    }
    return SWIG_OverflowError;
  } else if (SvIOK(obj)) {
    IV v = SvIV(obj);
    if (v >= 0 && (IVSIZE <= sizeof(*val) || v <= ULONG_MAX)) {
      if (val) *val = v;
      return SWIG_OK;
    }
    return SWIG_OverflowError;
  } else {
    int dispatch = 0;
    const char *nptr = SvPV_nolen(obj);
    if (nptr) {
      char *endptr;
      unsigned long v;
      errno = 0;
      v = strtoul(nptr, &endptr,0);
      if (errno == ERANGE) {
	errno = 0;
	return SWIG_OverflowError;
      } else {
	if (*endptr == '\0') {
	  if (val) *val = v;
	  return SWIG_Str2NumCast(SWIG_OK);
	}
      }
    }
    if (!dispatch) {
      double d;
      int res = SWIG_AddCast(SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(obj,&d));
      if (SWIG_IsOK(res) && SWIG_CanCastAsInteger(&d, 0, ULONG_MAX)) {
	if (val) *val = (unsigned long)(d);
	return res;
      }
    }
  }
  return SWIG_TypeError;
}


#if defined(LLONG_MAX) && !defined(SWIG_LONG_LONG_AVAILABLE)
#  define SWIG_LONG_LONG_AVAILABLE
#endif


#ifdef SWIG_LONG_LONG_AVAILABLE
SWIGINTERN int
SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_DECL_ARGS_2(SV *obj, unsigned long long *val)
{
  if (SvUOK(obj)) {
    /* pretty sure this should be conditional on
     * (UVSIZE <= sizeof(*val) || v <= ULLONG_MAX) */
    if (val) *val = SvUV(obj);
    return SWIG_OK;
  } else if (SvIOK(obj)) {
    IV v = SvIV(obj);
    if (v >= 0 && (IVSIZE <= sizeof(*val) || v <= ULLONG_MAX)) {
      if (val) *val = v;
      return SWIG_OK;
    } else {
      return SWIG_OverflowError;
    }
  } else {
    int dispatch = 0;
    const char *nptr = SvPV_nolen(obj);
    if (nptr) {
      char *endptr;
      unsigned long long v;
      errno = 0;
      v = strtoull(nptr, &endptr,0);
      if (errno == ERANGE) {
	errno = 0;
	return SWIG_OverflowError;
      } else {
	if (*endptr == '\0') {
	  if (val) *val = v;
	  return SWIG_Str2NumCast(SWIG_OK);
	}
      }
    }
    if (!dispatch) {
      const double mant_max = 1LL << DBL_MANT_DIG;
      double d;
      int res = SWIG_AddCast(SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(obj,&d));
      if (SWIG_IsOK(res) && SWIG_CanCastAsInteger(&d, 0, mant_max)) {
	if (val) *val = (unsigned long long)(d);
	return res;
      }
    }
  }
  return SWIG_TypeError;
}
#endif


SWIGINTERNINLINE int
SWIG_AsVal_size_t SWIG_PERL_DECL_ARGS_2(SV * obj, size_t *val)
{
  int res = SWIG_TypeError;
#ifdef SWIG_LONG_LONG_AVAILABLE
  if (sizeof(size_t) <= sizeof(unsigned long)) {
#endif
    unsigned long v;
    res = SWIG_AsVal_unsigned_SS_long SWIG_PERL_CALL_ARGS_2(obj, val ? &v : 0);
    if (SWIG_IsOK(res) && val) *val = (size_t)(v);
#ifdef SWIG_LONG_LONG_AVAILABLE
  } else if (sizeof(size_t) <= sizeof(unsigned long long)) {
    unsigned long long v;
    res = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(obj, val ? &v : 0);
    if (SWIG_IsOK(res) && val) *val = (size_t)(v);
  }
#endif
  return res;
}


SWIGINTERNINLINE SV *
SWIG_From_long  SWIG_PERL_DECL_ARGS_1(long value)
{
  SV *sv;
  if (IVSIZE >= sizeof(value) || (value >= IV_MIN && value <= IV_MAX))
    sv = newSViv(value);
  else
    sv = newSVpvf("%ld", value);
  return sv_2mortal(sv);
}


SWIGINTERNINLINE SV *
SWIG_From_int  SWIG_PERL_DECL_ARGS_1(int value)
{    
  return SWIG_From_long  SWIG_PERL_CALL_ARGS_1(value);
}


SWIGINTERNINLINE SV *
SWIG_FromCharPtrAndSize(const char* carray, size_t size)
{
  SV *obj = sv_newmortal();
  if (carray) {
    sv_setpvn(obj, carray, size);
  } else {
    sv_setsv(obj, &PL_sv_undef);
  }
  return obj;
}


SWIGINTERNINLINE SV * 
SWIG_FromCharPtr(const char *cptr)
{ 
  return SWIG_FromCharPtrAndSize(cptr, (cptr ? strlen(cptr) : 0));
}


SWIGINTERN int
SWIG_AsVal_unsigned_SS_int SWIG_PERL_DECL_ARGS_2(SV * obj, unsigned int *val)
{
  unsigned long v;
  int res = SWIG_AsVal_unsigned_SS_long SWIG_PERL_CALL_ARGS_2(obj, &v);
  if (SWIG_IsOK(res)) {
    if ((v > UINT_MAX)) {
      return SWIG_OverflowError;
    } else {
      if (val) *val = (unsigned int)(v);
    }
  }  
  return res;
}


SWIGINTERNINLINE SV *
SWIG_From_unsigned_SS_long  SWIG_PERL_DECL_ARGS_1(unsigned long value)
{
  SV *sv;
  if (UVSIZE >= sizeof(value) || value <= UV_MAX)
    sv = newSVuv(value);
  else
    sv = newSVpvf("%lu", value);
  return sv_2mortal(sv);
}


SWIGINTERNINLINE SV *
SWIG_From_unsigned_SS_int  SWIG_PERL_DECL_ARGS_1(unsigned int value)
{    
  return SWIG_From_unsigned_SS_long  SWIG_PERL_CALL_ARGS_1(value);
}


SWIGINTERNINLINE SV *
SWIG_From_double  SWIG_PERL_DECL_ARGS_1(double value)
{
  return sv_2mortal(newSVnv(value));
}


SWIGINTERN int
SWIG_AsVal_long SWIG_PERL_DECL_ARGS_2(SV *obj, long* val)
{
  if (SvUOK(obj)) {
    UV v = SvUV(obj);
    if (UVSIZE < sizeof(*val) || v <= LONG_MAX) {
      if (val) *val = v;
      return SWIG_OK;
    }
    return SWIG_OverflowError;
  } else if (SvIOK(obj)) {
    IV v = SvIV(obj);
    if (IVSIZE <= sizeof(*val) || (v >= LONG_MIN && v <= LONG_MAX)) {
      if(val) *val = v;
      return SWIG_OK;
    }
    return SWIG_OverflowError;
  } else {
    int dispatch = 0;
    const char *nptr = SvPV_nolen(obj);
    if (nptr) {
      char *endptr;
      long v;
      errno = 0;
      v = strtol(nptr, &endptr,0);
      if (errno == ERANGE) {
	errno = 0;
	return SWIG_OverflowError;
      } else {
	if (*endptr == '\0') {
	  if (val) *val = v;
	  return SWIG_Str2NumCast(SWIG_OK);
	}
      }
    }
    if (!dispatch) {
      double d;
      int res = SWIG_AddCast(SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(obj,&d));
      if (SWIG_IsOK(res) && SWIG_CanCastAsInteger(&d, LONG_MIN, LONG_MAX)) {
	if (val) *val = (long)(d);
	return res;
      }
    }
  }
  return SWIG_TypeError;
}


SWIGINTERN int
SWIG_AsVal_int SWIG_PERL_DECL_ARGS_2(SV * obj, int *val)
{
  long v;
  int res = SWIG_AsVal_long SWIG_PERL_CALL_ARGS_2(obj, &v);
  if (SWIG_IsOK(res)) {
    if ((v < INT_MIN || v > INT_MAX)) {
      return SWIG_OverflowError;
    } else {
      if (val) *val = (int)(v);
    }
  }  
  return res;
}


SWIGINTERN int
SWIG_AsVal_unsigned_SS_char SWIG_PERL_DECL_ARGS_2(SV * obj, unsigned char *val)
{
  unsigned long v;
  int res = SWIG_AsVal_unsigned_SS_long SWIG_PERL_CALL_ARGS_2(obj, &v);
  if (SWIG_IsOK(res)) {
    if ((v > UCHAR_MAX)) {
      return SWIG_OverflowError;
    } else {
      if (val) *val = (unsigned char)(v);
    }
  }  
  return res;
}


SWIGINTERNINLINE SV *
SWIG_From_unsigned_SS_char  SWIG_PERL_DECL_ARGS_1(unsigned char value)
{    
  return SWIG_From_unsigned_SS_long  SWIG_PERL_CALL_ARGS_1(value);
}


#include <stdio.h>
#if (defined(_MSC_VER) && (_MSC_VER < 1900)) || defined(__BORLANDC__) || defined(_WATCOM)
# ifndef snprintf
#  define snprintf _snprintf
# endif
#endif


#ifdef SWIG_LONG_LONG_AVAILABLE
SWIGINTERNINLINE SV *
SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_DECL_ARGS_1(unsigned long long value)
{
  SV *sv;
  if (UVSIZE >= sizeof(value) || value <= UV_MAX)
    sv = newSVuv((UV)(value));
  else {
    //sv = newSVpvf("%llu", value); doesn't work in non 64bit Perl
    char temp[256];
    SWIG_snprintf(temp, sizeof(temp), "%llu", value);
    sv = newSVpv(temp, 0);
  }
  return sv_2mortal(sv);
}
#endif


SWIGINTERN int
SWIG_AsVal_short SWIG_PERL_DECL_ARGS_2(SV * obj, short *val)
{
  long v;
  int res = SWIG_AsVal_long SWIG_PERL_CALL_ARGS_2(obj, &v);
  if (SWIG_IsOK(res)) {
    if ((v < SHRT_MIN || v > SHRT_MAX)) {
      return SWIG_OverflowError;
    } else {
      if (val) *val = (short)(v);
    }
  }  
  return res;
}


SWIGINTERNINLINE SV *
SWIG_From_short  SWIG_PERL_DECL_ARGS_1(short value)
{    
  return SWIG_From_long  SWIG_PERL_CALL_ARGS_1(value);
}


SWIGINTERN int
SWIG_AsVal_unsigned_SS_short SWIG_PERL_DECL_ARGS_2(SV * obj, unsigned short *val)
{
  unsigned long v;
  int res = SWIG_AsVal_unsigned_SS_long SWIG_PERL_CALL_ARGS_2(obj, &v);
  if (SWIG_IsOK(res)) {
    if ((v > USHRT_MAX)) {
      return SWIG_OverflowError;
    } else {
      if (val) *val = (unsigned short)(v);
    }
  }  
  return res;
}


SWIGINTERNINLINE SV *
SWIG_From_unsigned_SS_short  SWIG_PERL_DECL_ARGS_1(unsigned short value)
{    
  return SWIG_From_unsigned_SS_long  SWIG_PERL_CALL_ARGS_1(value);
}


SWIGINTERN swig_type_info*
SWIG_pchar_descriptor(void)
{
  static swig_type_info* info = 0;
  if (!info) {
    info = SWIG_TypeQuery("_p_char");
  }
  return info;
}


SWIGINTERN int
SWIG_AsCharPtrAndSize(SV *obj, char** cptr, size_t* psize, int *alloc)
{
  if (SvMAGICAL(obj)) {
     SV *tmp = sv_newmortal();
     SvSetSV(tmp, obj);
     obj = tmp;
  }
  if (SvPOK(obj)) {
    STRLEN len = 0;
    char *cstr = SvPV(obj, len); 
    size_t size = len + 1;
    if (cptr) {
      if (alloc) {
	if (*alloc == SWIG_NEWOBJ) {
	  *cptr = (char *)memcpy(malloc((size)*sizeof(char)), cstr, sizeof(char)*(size));
	} else {
	  *cptr = cstr;
	  *alloc = SWIG_OLDOBJ;
	}
      }
    }
    if (psize) *psize = size;
    return SWIG_OK;
  } else {
    swig_type_info* pchar_descriptor = SWIG_pchar_descriptor();
    if (pchar_descriptor) {
      char* vptr = 0; 
      if (SWIG_ConvertPtr(obj, (void**)&vptr, pchar_descriptor, 0) == SWIG_OK) {
	if (cptr) *cptr = vptr;
	if (psize) *psize = vptr ? (strlen(vptr) + 1) : 0;
	if (alloc) *alloc = SWIG_OLDOBJ;
	return SWIG_OK;
      }
    }
  }
  return SWIG_TypeError;
}




typedef union {
  char *data;
  dwg_wchar_t *wdata;
} _dwg_binary_chunk_u;

typedef union {
  double  pt[3];
  char    i8;
  short   i16;
  int     i32;
  BITCODE_BLL i64;
  BITCODE_HV absref;
  double  dbl;
  struct _dwg_binary_chunk str;
} _dwg_resbuf_value;


SWIGINTERN int
SWIG_AsCharArray(SV * obj, char *val, size_t size)
{ 
  char* cptr = 0; size_t csize = 0; int alloc = SWIG_OLDOBJ;
  int res = SWIG_AsCharPtrAndSize(obj, &cptr, &csize, &alloc);
  if (SWIG_IsOK(res)) {
    /* special case of single char conversion when we don't need space for NUL */
    if (size == 1 && csize == 2 && cptr && !cptr[1]) --csize;
    if (csize <= size) {
      if (val) {
	if (csize) memcpy(val, cptr, csize*sizeof(char));
	if (csize < size) memset(val + csize, 0, (size - csize)*sizeof(char));
      }
      if (alloc == SWIG_NEWOBJ) {
	free((char*)cptr);
	res = SWIG_DelNewMask(res);
      }      
      return res;
    }
    if (alloc == SWIG_NEWOBJ) free((char*)cptr);
  }
  return SWIG_TypeError;
}


SWIGINTERN int
SWIG_AsVal_char SWIG_PERL_DECL_ARGS_2(SV * obj, char *val)
{    
  int res = SWIG_AsCharArray(obj, val, 1);
  if (!SWIG_IsOK(res)) {
    long v;
    res = SWIG_AddCast(SWIG_AsVal_long SWIG_PERL_CALL_ARGS_2(obj, &v));
    if (SWIG_IsOK(res)) {
      if ((CHAR_MIN <= v) && (v <= CHAR_MAX)) {
	if (val) *val = (char)(v);
      } else {
	res = SWIG_OverflowError;
      }
    }
  }
  return res;
}


SWIGINTERNINLINE SV *
SWIG_From_char  SWIG_PERL_DECL_ARGS_1(char c) 
{ 
  return SWIG_FromCharPtrAndSize(&c,1);
}

typedef union {
  BITCODE_BSd index;                   
  BITCODE_H   ltype;                   
} _dwg_MLINESTYLE_line_lt;

typedef union {
  BITCODE_BD bd;
  BITCODE_BL bl;
  BITCODE_BS bs;
  BITCODE_RC rc;
  BITCODE_TV text;
  BITCODE_H handle;
} _dwg_EvalVariant_u;

typedef union {
  BITCODE_BD  num40;
  BITCODE_2RD pt2d;
  BITCODE_3BD pt3d;
  BITCODE_TV   text1;
  BITCODE_BL  long90;
  BITCODE_H   handle91;
  BITCODE_BS  short70;
} _dwg_EvalExpr_value;


SWIGINTERN int
SWIG_AsVal_signed_SS_char SWIG_PERL_DECL_ARGS_2(SV * obj, signed char *val)
{
  long v;
  int res = SWIG_AsVal_long SWIG_PERL_CALL_ARGS_2(obj, &v);
  if (SWIG_IsOK(res)) {
    if ((v < SCHAR_MIN || v > SCHAR_MAX)) {
      return SWIG_OverflowError;
    } else {
      if (val) *val = (signed char)(v);
    }
  }  
  return res;
}


SWIGINTERNINLINE SV *
SWIG_From_signed_SS_char  SWIG_PERL_DECL_ARGS_1(signed char value)
{    
  return SWIG_From_long  SWIG_PERL_CALL_ARGS_1(value);
}

typedef struct {
  BITCODE_RS length;                  
  unsigned short codepage:15;            
  unsigned short is_tu:1;
  
  
  
  
  char string[0];                                                            
  
} eed_data_t_eed_0;

typedef struct {
  BITCODE_RS length;
  unsigned short _padding:15;
  unsigned short is_tu:1;
  
  
  
  dwg_wchar_t string[0];
  
} eed_data_t_eed_0_r2007;

typedef struct {
  BITCODE_RS appid_index;
  
  
  
  char invalid[0];
  
} eed_data_t_eed_1;

typedef struct {
  BITCODE_RC close;
} eed_data_t_eed_2;

typedef struct {
  BITCODE_RLL layer;
} eed_data_t_eed_3;

typedef struct {
  BITCODE_RC length;
  unsigned char data[] ;           
} eed_data_t_eed_4;

typedef struct {
  BITCODE_RLL entity;
} eed_data_t_eed_5;

typedef struct {
  BITCODE_3RD point;
} eed_data_t_eed_10;

typedef struct {
  BITCODE_RD real;
} eed_data_t_eed_40;

typedef struct {
  BITCODE_RS rs;
} eed_data_t_eed_70;

typedef struct {
  BITCODE_RL rl;
} eed_data_t_eed_71;


SWIGINTERN size_t
SWIG_strnlen(const char* s, size_t maxlen)
{
  const char *p;
  for (p = s; maxlen-- && *p; p++)
    ;
  return p - s;
}

typedef union {
  Dwg_Entity_UNUSED *UNUSED;
  Dwg_DIMENSION_common *DIMENSION_common;
  
  
  
  Dwg_Entity__3DFACE *_3DFACE;
  Dwg_Entity__3DSOLID *_3DSOLID;
  Dwg_Entity_ARC *ARC;
  Dwg_Entity_ATTDEF *ATTDEF;
  Dwg_Entity_ATTRIB *ATTRIB;
  Dwg_Entity_BLOCK *BLOCK;
  Dwg_Entity_BODY *BODY;
  Dwg_Entity_CIRCLE *CIRCLE;
  Dwg_Entity_DIMENSION_ALIGNED *DIMENSION_ALIGNED;
  Dwg_Entity_DIMENSION_ANG2LN *DIMENSION_ANG2LN;
  Dwg_Entity_DIMENSION_ANG3PT *DIMENSION_ANG3PT;
  Dwg_Entity_DIMENSION_DIAMETER *DIMENSION_DIAMETER;
  Dwg_Entity_DIMENSION_LINEAR *DIMENSION_LINEAR;
  Dwg_Entity_DIMENSION_ORDINATE *DIMENSION_ORDINATE;
  Dwg_Entity_DIMENSION_RADIUS *DIMENSION_RADIUS;
  Dwg_Entity_ELLIPSE *ELLIPSE;
  Dwg_Entity_ENDBLK *ENDBLK;
  Dwg_Entity_INSERT *INSERT;
  Dwg_Entity_LEADER *LEADER;
  Dwg_Entity_LINE *LINE;
  Dwg_Entity_LOAD *LOAD;
  Dwg_Entity_MINSERT *MINSERT;
  Dwg_Entity_MLINE *MLINE;
  Dwg_Entity_MTEXT *MTEXT;
  Dwg_Entity_OLEFRAME *OLEFRAME;
  Dwg_Entity_POINT *POINT;
  Dwg_Entity_POLYLINE_2D *POLYLINE_2D;
  Dwg_Entity_POLYLINE_3D *POLYLINE_3D;
  Dwg_Entity_POLYLINE_MESH *POLYLINE_MESH;
  Dwg_Entity_POLYLINE_PFACE *POLYLINE_PFACE;
  Dwg_Entity_PROXY_ENTITY *PROXY_ENTITY;
  Dwg_Entity_RAY *RAY;
  Dwg_Entity_REGION *REGION;
  Dwg_Entity_SEQEND *SEQEND;
  Dwg_Entity_SHAPE *SHAPE;
  Dwg_Entity_SOLID *SOLID;
  Dwg_Entity_SPLINE *SPLINE;
  Dwg_Entity_TEXT *TEXT;
  Dwg_Entity_TOLERANCE *TOLERANCE;
  Dwg_Entity_TRACE *TRACE;
  Dwg_Entity_UNKNOWN_ENT *UNKNOWN_ENT;
  Dwg_Entity_VERTEX_2D *VERTEX_2D;
  Dwg_Entity_VERTEX_3D *VERTEX_3D;
  Dwg_Entity_VERTEX_MESH *VERTEX_MESH;
  Dwg_Entity_VERTEX_PFACE *VERTEX_PFACE;
  Dwg_Entity_VERTEX_PFACE_FACE *VERTEX_PFACE_FACE;
  Dwg_Entity_VIEWPORT *VIEWPORT;
  Dwg_Entity_XLINE *XLINE;
  
  Dwg_Entity__3DLINE *_3DLINE;
  Dwg_Entity_CAMERA *CAMERA;
  Dwg_Entity_DGNUNDERLAY *DGNUNDERLAY;
  Dwg_Entity_DWFUNDERLAY *DWFUNDERLAY;
  Dwg_Entity_ENDREP *ENDREP;
  Dwg_Entity_HATCH *HATCH;
  Dwg_Entity_IMAGE *IMAGE;
  Dwg_Entity_JUMP *JUMP;
  Dwg_Entity_LIGHT *LIGHT;
  Dwg_Entity_LWPOLYLINE *LWPOLYLINE;
  Dwg_Entity_MESH *MESH;
  Dwg_Entity_MULTILEADER *MULTILEADER;
  Dwg_Entity_OLE2FRAME *OLE2FRAME;
  Dwg_Entity_PDFUNDERLAY *PDFUNDERLAY;
  Dwg_Entity_REPEAT *REPEAT;
  Dwg_Entity_SECTIONOBJECT *SECTIONOBJECT;
  Dwg_Entity_WIPEOUT *WIPEOUT;
  
  Dwg_Entity_ARC_DIMENSION *ARC_DIMENSION;
  Dwg_Entity_HELIX *HELIX;
  Dwg_Entity_LARGE_RADIAL_DIMENSION *LARGE_RADIAL_DIMENSION;
  Dwg_Entity_LAYOUTPRINTCONFIG *LAYOUTPRINTCONFIG;
  Dwg_Entity_PLANESURFACE *PLANESURFACE;
  Dwg_Entity_POINTCLOUD *POINTCLOUD;
  Dwg_Entity_POINTCLOUDEX *POINTCLOUDEX;
  
  Dwg_Entity_ALIGNMENTPARAMETERENTITY *ALIGNMENTPARAMETERENTITY;
  Dwg_Entity_ARCALIGNEDTEXT *ARCALIGNEDTEXT;
  Dwg_Entity_BASEPOINTPARAMETERENTITY *BASEPOINTPARAMETERENTITY;
  Dwg_Entity_EXTRUDEDSURFACE *EXTRUDEDSURFACE;
  Dwg_Entity_FLIPGRIPENTITY *FLIPGRIPENTITY;
  Dwg_Entity_FLIPPARAMETERENTITY *FLIPPARAMETERENTITY;
  Dwg_Entity_GEOPOSITIONMARKER *GEOPOSITIONMARKER;
  Dwg_Entity_LINEARGRIPENTITY *LINEARGRIPENTITY;
  Dwg_Entity_LINEARPARAMETERENTITY *LINEARPARAMETERENTITY;
  Dwg_Entity_LOFTEDSURFACE *LOFTEDSURFACE;
  Dwg_Entity_MPOLYGON *MPOLYGON;
  Dwg_Entity_NAVISWORKSMODEL *NAVISWORKSMODEL;
  Dwg_Entity_NURBSURFACE *NURBSURFACE;
  Dwg_Entity_POINTPARAMETERENTITY *POINTPARAMETERENTITY;
  Dwg_Entity_POLARGRIPENTITY *POLARGRIPENTITY;
  Dwg_Entity_REVOLVEDSURFACE *REVOLVEDSURFACE;
  Dwg_Entity_ROTATIONGRIPENTITY *ROTATIONGRIPENTITY;
  Dwg_Entity_ROTATIONPARAMETERENTITY *ROTATIONPARAMETERENTITY;
  Dwg_Entity_RTEXT *RTEXT;
  Dwg_Entity_SWEPTSURFACE *SWEPTSURFACE;
  Dwg_Entity_TABLE *TABLE;
  Dwg_Entity_VISIBILITYGRIPENTITY *VISIBILITYGRIPENTITY;
  Dwg_Entity_VISIBILITYPARAMETERENTITY *VISIBILITYPARAMETERENTITY;
  Dwg_Entity_XYGRIPENTITY *XYGRIPENTITY;
  Dwg_Entity_XYPARAMETERENTITY *XYPARAMETERENTITY;
  
  
  
} _dwg_object_entity_tio;

typedef union {
  Dwg_Object_APPID *APPID;
  Dwg_Object_APPID_CONTROL *APPID_CONTROL;
  Dwg_Object_BLOCK_CONTROL *BLOCK_CONTROL;
  Dwg_Object_BLOCK_HEADER *BLOCK_HEADER;
  Dwg_Object_DICTIONARY *DICTIONARY;
  Dwg_Object_DIMSTYLE *DIMSTYLE;
  Dwg_Object_DIMSTYLE_CONTROL *DIMSTYLE_CONTROL;
  Dwg_Object_DUMMY *DUMMY;
  Dwg_Object_LAYER *LAYER;
  Dwg_Object_LAYER_CONTROL *LAYER_CONTROL;
  Dwg_Object_LONG_TRANSACTION *LONG_TRANSACTION;
  Dwg_Object_LTYPE *LTYPE;
  Dwg_Object_LTYPE_CONTROL *LTYPE_CONTROL;
  Dwg_Object_MLINESTYLE *MLINESTYLE;
  Dwg_Object_STYLE *STYLE;
  Dwg_Object_STYLE_CONTROL *STYLE_CONTROL;
  Dwg_Object_UCS *UCS;
  Dwg_Object_UCS_CONTROL *UCS_CONTROL;
  Dwg_Object_UNKNOWN_OBJ *UNKNOWN_OBJ;
  Dwg_Object_VIEW *VIEW;
  Dwg_Object_VIEW_CONTROL *VIEW_CONTROL;
  Dwg_Object_VPORT *VPORT;
  Dwg_Object_VPORT_CONTROL *VPORT_CONTROL;
  Dwg_Object_VX_CONTROL *VX_CONTROL;
  Dwg_Object_VX_TABLE_RECORD *VX_TABLE_RECORD;
  
  Dwg_Object_ACSH_BOOLEAN_CLASS *ACSH_BOOLEAN_CLASS;
  Dwg_Object_ACSH_BOX_CLASS *ACSH_BOX_CLASS;
  Dwg_Object_ACSH_CONE_CLASS *ACSH_CONE_CLASS;
  Dwg_Object_ACSH_CYLINDER_CLASS *ACSH_CYLINDER_CLASS;
  Dwg_Object_ACSH_FILLET_CLASS *ACSH_FILLET_CLASS;
  Dwg_Object_ACSH_HISTORY_CLASS *ACSH_HISTORY_CLASS;
  Dwg_Object_ACSH_SPHERE_CLASS *ACSH_SPHERE_CLASS;
  Dwg_Object_ACSH_TORUS_CLASS *ACSH_TORUS_CLASS;
  Dwg_Object_ACSH_WEDGE_CLASS *ACSH_WEDGE_CLASS;
  Dwg_Object_ASSOCGEOMDEPENDENCY *ASSOCGEOMDEPENDENCY;
  Dwg_Object_ASSOCNETWORK *ASSOCNETWORK;
  Dwg_Object_BLOCKALIGNMENTGRIP *BLOCKALIGNMENTGRIP;
  Dwg_Object_BLOCKALIGNMENTPARAMETER *BLOCKALIGNMENTPARAMETER;
  Dwg_Object_BLOCKBASEPOINTPARAMETER *BLOCKBASEPOINTPARAMETER;
  Dwg_Object_BLOCKFLIPACTION *BLOCKFLIPACTION;
  Dwg_Object_BLOCKFLIPGRIP *BLOCKFLIPGRIP;
  Dwg_Object_BLOCKFLIPPARAMETER *BLOCKFLIPPARAMETER;
  Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT *BLOCKGRIPLOCATIONCOMPONENT;
  Dwg_Object_BLOCKLINEARGRIP *BLOCKLINEARGRIP;
  Dwg_Object_BLOCKLOOKUPGRIP *BLOCKLOOKUPGRIP;
  Dwg_Object_BLOCKMOVEACTION *BLOCKMOVEACTION;
  Dwg_Object_BLOCKROTATEACTION *BLOCKROTATEACTION;
  Dwg_Object_BLOCKROTATIONGRIP *BLOCKROTATIONGRIP;
  Dwg_Object_BLOCKSCALEACTION *BLOCKSCALEACTION;
  Dwg_Object_BLOCKVISIBILITYGRIP *BLOCKVISIBILITYGRIP;
  Dwg_Object_CELLSTYLEMAP *CELLSTYLEMAP;
  Dwg_Object_DETAILVIEWSTYLE *DETAILVIEWSTYLE;
  Dwg_Object_DICTIONARYVAR *DICTIONARYVAR;
  Dwg_Object_DICTIONARYWDFLT *DICTIONARYWDFLT;
  Dwg_Object_DYNAMICBLOCKPURGEPREVENTER *DYNAMICBLOCKPURGEPREVENTER;
  Dwg_Object_FIELD *FIELD;
  Dwg_Object_FIELDLIST *FIELDLIST;
  Dwg_Object_GEODATA *GEODATA;
  Dwg_Object_GROUP *GROUP;
  Dwg_Object_IDBUFFER *IDBUFFER;
  Dwg_Object_IMAGEDEF *IMAGEDEF;
  Dwg_Object_IMAGEDEF_REACTOR *IMAGEDEF_REACTOR;
  Dwg_Object_INDEX *INDEX;
  Dwg_Object_LAYERFILTER *LAYERFILTER;
  Dwg_Object_LAYER_INDEX *LAYER_INDEX;
  Dwg_Object_LAYOUT *LAYOUT;
  Dwg_Object_PLACEHOLDER *PLACEHOLDER;
  Dwg_Object_PLOTSETTINGS *PLOTSETTINGS;
  Dwg_Object_RASTERVARIABLES *RASTERVARIABLES;
  Dwg_Object_RENDERENVIRONMENT *RENDERENVIRONMENT;
  Dwg_Object_SCALE *SCALE;
  Dwg_Object_SECTIONVIEWSTYLE *SECTIONVIEWSTYLE;
  Dwg_Object_SECTION_MANAGER *SECTION_MANAGER;
  Dwg_Object_SORTENTSTABLE *SORTENTSTABLE;
  Dwg_Object_SPATIAL_FILTER *SPATIAL_FILTER;
  Dwg_Object_SUN *SUN;
  Dwg_Object_TABLEGEOMETRY *TABLEGEOMETRY;
  Dwg_Object_VBA_PROJECT *VBA_PROJECT;
  Dwg_Object_VISUALSTYLE *VISUALSTYLE;
  Dwg_Object_WIPEOUTVARIABLES *WIPEOUTVARIABLES;
  Dwg_Object_XRECORD *XRECORD;
  Dwg_Object_PDFDEFINITION *PDFDEFINITION;
  Dwg_Object_DGNDEFINITION *DGNDEFINITION;
  Dwg_Object_DWFDEFINITION *DWFDEFINITION;
  
  Dwg_Object_ACSH_BREP_CLASS *ACSH_BREP_CLASS;
  Dwg_Object_ACSH_CHAMFER_CLASS *ACSH_CHAMFER_CLASS;
  Dwg_Object_ACSH_PYRAMID_CLASS *ACSH_PYRAMID_CLASS;
  Dwg_Object_ALDIMOBJECTCONTEXTDATA *ALDIMOBJECTCONTEXTDATA;
  Dwg_Object_ASSOC2DCONSTRAINTGROUP *ASSOC2DCONSTRAINTGROUP;
  Dwg_Object_ASSOCACTION *ASSOCACTION;
  Dwg_Object_ASSOCACTIONPARAM *ASSOCACTIONPARAM;
  Dwg_Object_ASSOCARRAYACTIONBODY *ASSOCARRAYACTIONBODY;
  Dwg_Object_ASSOCASMBODYACTIONPARAM *ASSOCASMBODYACTIONPARAM;
  Dwg_Object_ASSOCBLENDSURFACEACTIONBODY *ASSOCBLENDSURFACEACTIONBODY;
  Dwg_Object_ASSOCCOMPOUNDACTIONPARAM *ASSOCCOMPOUNDACTIONPARAM;
  Dwg_Object_ASSOCDEPENDENCY *ASSOCDEPENDENCY;
  Dwg_Object_ASSOCDIMDEPENDENCYBODY *ASSOCDIMDEPENDENCYBODY;
  Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY *ASSOCEXTENDSURFACEACTIONBODY;
  Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY *ASSOCEXTRUDEDSURFACEACTIONBODY;
  Dwg_Object_ASSOCFACEACTIONPARAM *ASSOCFACEACTIONPARAM;
  Dwg_Object_ASSOCFILLETSURFACEACTIONBODY *ASSOCFILLETSURFACEACTIONBODY;
  Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY *ASSOCLOFTEDSURFACEACTIONBODY;
  Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY *ASSOCNETWORKSURFACEACTIONBODY;
  Dwg_Object_ASSOCOBJECTACTIONPARAM *ASSOCOBJECTACTIONPARAM;
  Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY *ASSOCOFFSETSURFACEACTIONBODY;
  Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM *ASSOCOSNAPPOINTREFACTIONPARAM;
  Dwg_Object_ASSOCPATCHSURFACEACTIONBODY *ASSOCPATCHSURFACEACTIONBODY;
  Dwg_Object_ASSOCPATHACTIONPARAM *ASSOCPATHACTIONPARAM;
  Dwg_Object_ASSOCPLANESURFACEACTIONBODY *ASSOCPLANESURFACEACTIONBODY;
  Dwg_Object_ASSOCPOINTREFACTIONPARAM *ASSOCPOINTREFACTIONPARAM;
  Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY *ASSOCREVOLVEDSURFACEACTIONBODY;
  Dwg_Object_ASSOCTRIMSURFACEACTIONBODY *ASSOCTRIMSURFACEACTIONBODY;
  Dwg_Object_ASSOCVALUEDEPENDENCY *ASSOCVALUEDEPENDENCY;
  Dwg_Object_ASSOCVARIABLE *ASSOCVARIABLE;
  Dwg_Object_ASSOCVERTEXACTIONPARAM *ASSOCVERTEXACTIONPARAM;
  Dwg_Object_BLKREFOBJECTCONTEXTDATA *BLKREFOBJECTCONTEXTDATA;
  Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER *BLOCKALIGNEDCONSTRAINTPARAMETER;
  Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER *BLOCKANGULARCONSTRAINTPARAMETER;
  Dwg_Object_BLOCKARRAYACTION *BLOCKARRAYACTION;
  Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER *BLOCKDIAMETRICCONSTRAINTPARAMETER;
  Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER *BLOCKHORIZONTALCONSTRAINTPARAMETER;
  Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER *BLOCKLINEARCONSTRAINTPARAMETER;
  Dwg_Object_BLOCKLINEARPARAMETER *BLOCKLINEARPARAMETER;
  Dwg_Object_BLOCKLOOKUPACTION *BLOCKLOOKUPACTION;
  Dwg_Object_BLOCKLOOKUPPARAMETER *BLOCKLOOKUPPARAMETER;
  Dwg_Object_BLOCKPARAMDEPENDENCYBODY *BLOCKPARAMDEPENDENCYBODY;
  Dwg_Object_BLOCKPOINTPARAMETER *BLOCKPOINTPARAMETER;
  Dwg_Object_BLOCKPOLARGRIP *BLOCKPOLARGRIP;
  Dwg_Object_BLOCKPOLARPARAMETER *BLOCKPOLARPARAMETER;
  Dwg_Object_BLOCKPOLARSTRETCHACTION *BLOCKPOLARSTRETCHACTION;
  Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER *BLOCKRADIALCONSTRAINTPARAMETER;
  Dwg_Object_BLOCKREPRESENTATION *BLOCKREPRESENTATION;
  Dwg_Object_BLOCKROTATIONPARAMETER *BLOCKROTATIONPARAMETER;
  Dwg_Object_BLOCKSTRETCHACTION *BLOCKSTRETCHACTION;
  Dwg_Object_BLOCKUSERPARAMETER *BLOCKUSERPARAMETER;
  Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER *BLOCKVERTICALCONSTRAINTPARAMETER;
  Dwg_Object_BLOCKVISIBILITYPARAMETER *BLOCKVISIBILITYPARAMETER;
  Dwg_Object_BLOCKXYGRIP *BLOCKXYGRIP;
  Dwg_Object_BLOCKXYPARAMETER *BLOCKXYPARAMETER;
  Dwg_Object_DATALINK *DATALINK;
  Dwg_Object_DBCOLOR *DBCOLOR;
  Dwg_Object_EVALUATION_GRAPH *EVALUATION_GRAPH;
  Dwg_Object_FCFOBJECTCONTEXTDATA *FCFOBJECTCONTEXTDATA;
  Dwg_Object_GRADIENT_BACKGROUND *GRADIENT_BACKGROUND;
  Dwg_Object_GROUND_PLANE_BACKGROUND *GROUND_PLANE_BACKGROUND;
  Dwg_Object_IBL_BACKGROUND *IBL_BACKGROUND;
  Dwg_Object_IMAGE_BACKGROUND *IMAGE_BACKGROUND;
  Dwg_Object_LEADEROBJECTCONTEXTDATA *LEADEROBJECTCONTEXTDATA;
  Dwg_Object_LIGHTLIST *LIGHTLIST;
  Dwg_Object_MATERIAL *MATERIAL;
  Dwg_Object_MENTALRAYRENDERSETTINGS *MENTALRAYRENDERSETTINGS;
  Dwg_Object_MLEADERSTYLE *MLEADERSTYLE;
  Dwg_Object_MTEXTOBJECTCONTEXTDATA *MTEXTOBJECTCONTEXTDATA;
  Dwg_Object_OBJECT_PTR *OBJECT_PTR;
  Dwg_Object_PARTIAL_VIEWING_INDEX *PARTIAL_VIEWING_INDEX;
  Dwg_Object_POINTCLOUDCOLORMAP *POINTCLOUDCOLORMAP;
  Dwg_Object_POINTCLOUDDEF *POINTCLOUDDEF;
  Dwg_Object_POINTCLOUDDEFEX *POINTCLOUDDEFEX;
  Dwg_Object_POINTCLOUDDEF_REACTOR *POINTCLOUDDEF_REACTOR;
  Dwg_Object_POINTCLOUDDEF_REACTOR_EX *POINTCLOUDDEF_REACTOR_EX;
  Dwg_Object_PROXY_OBJECT *PROXY_OBJECT;
  Dwg_Object_RAPIDRTRENDERSETTINGS *RAPIDRTRENDERSETTINGS;
  Dwg_Object_RENDERENTRY *RENDERENTRY;
  Dwg_Object_RENDERGLOBAL *RENDERGLOBAL;
  Dwg_Object_RENDERSETTINGS *RENDERSETTINGS;
  Dwg_Object_SECTION_SETTINGS *SECTION_SETTINGS;
  Dwg_Object_SKYLIGHT_BACKGROUND *SKYLIGHT_BACKGROUND;
  Dwg_Object_SOLID_BACKGROUND *SOLID_BACKGROUND;
  Dwg_Object_SPATIAL_INDEX *SPATIAL_INDEX;
  Dwg_Object_TABLESTYLE *TABLESTYLE;
  Dwg_Object_TEXTOBJECTCONTEXTDATA *TEXTOBJECTCONTEXTDATA;
  Dwg_Object_ASSOCARRAYMODIFYPARAMETERS *ASSOCARRAYMODIFYPARAMETERS;
  Dwg_Object_ASSOCARRAYPATHPARAMETERS *ASSOCARRAYPATHPARAMETERS;
  Dwg_Object_ASSOCARRAYPOLARPARAMETERS *ASSOCARRAYPOLARPARAMETERS;
  Dwg_Object_ASSOCARRAYRECTANGULARPARAMETERS *ASSOCARRAYRECTANGULARPARAMETERS;
  
  Dwg_Object_ACMECOMMANDHISTORY *ACMECOMMANDHISTORY;
  Dwg_Object_ACMESCOPE *ACMESCOPE;
  Dwg_Object_ACMESTATEMGR *ACMESTATEMGR;
  Dwg_Object_ACSH_EXTRUSION_CLASS *ACSH_EXTRUSION_CLASS;
  Dwg_Object_ACSH_LOFT_CLASS *ACSH_LOFT_CLASS;
  Dwg_Object_ACSH_REVOLVE_CLASS *ACSH_REVOLVE_CLASS;
  Dwg_Object_ACSH_SWEEP_CLASS *ACSH_SWEEP_CLASS;
  Dwg_Object_ANGDIMOBJECTCONTEXTDATA *ANGDIMOBJECTCONTEXTDATA;
  Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA *ANNOTSCALEOBJECTCONTEXTDATA;
  Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY *ASSOC3POINTANGULARDIMACTIONBODY;
  Dwg_Object_ASSOCALIGNEDDIMACTIONBODY *ASSOCALIGNEDDIMACTIONBODY;
  Dwg_Object_ASSOCARRAYMODIFYACTIONBODY *ASSOCARRAYMODIFYACTIONBODY;
  Dwg_Object_ASSOCEDGEACTIONPARAM *ASSOCEDGEACTIONPARAM;
  Dwg_Object_ASSOCEDGECHAMFERACTIONBODY *ASSOCEDGECHAMFERACTIONBODY;
  Dwg_Object_ASSOCEDGEFILLETACTIONBODY *ASSOCEDGEFILLETACTIONBODY;
  Dwg_Object_ASSOCMLEADERACTIONBODY *ASSOCMLEADERACTIONBODY;
  Dwg_Object_ASSOCORDINATEDIMACTIONBODY *ASSOCORDINATEDIMACTIONBODY;
  Dwg_Object_ASSOCPERSSUBENTMANAGER *ASSOCPERSSUBENTMANAGER;
  Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY *ASSOCRESTOREENTITYSTATEACTIONBODY;
  Dwg_Object_ASSOCROTATEDDIMACTIONBODY *ASSOCROTATEDDIMACTIONBODY;
  Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY *ASSOCSWEPTSURFACEACTIONBODY;
  Dwg_Object_BLOCKPROPERTIESTABLE *BLOCKPROPERTIESTABLE;
  Dwg_Object_BLOCKPROPERTIESTABLEGRIP *BLOCKPROPERTIESTABLEGRIP;
  Dwg_Object_BREAKDATA *BREAKDATA;
  Dwg_Object_BREAKPOINTREF *BREAKPOINTREF;
  Dwg_Object_CONTEXTDATAMANAGER *CONTEXTDATAMANAGER;
  Dwg_Object_CSACDOCUMENTOPTIONS *CSACDOCUMENTOPTIONS;
  Dwg_Object_CURVEPATH *CURVEPATH;
  Dwg_Object_DATATABLE *DATATABLE;
  Dwg_Object_DIMASSOC *DIMASSOC;
  Dwg_Object_DMDIMOBJECTCONTEXTDATA *DMDIMOBJECTCONTEXTDATA;
  Dwg_Object_DYNAMICBLOCKPROXYNODE *DYNAMICBLOCKPROXYNODE;
  Dwg_Object_GEOMAPIMAGE *GEOMAPIMAGE;
  Dwg_Object_MLEADEROBJECTCONTEXTDATA *MLEADEROBJECTCONTEXTDATA;
  Dwg_Object_MOTIONPATH *MOTIONPATH;
  Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *MTEXTATTRIBUTEOBJECTCONTEXTDATA;
  Dwg_Object_NAVISWORKSMODELDEF *NAVISWORKSMODELDEF;
  Dwg_Object_ORDDIMOBJECTCONTEXTDATA *ORDDIMOBJECTCONTEXTDATA;
  Dwg_Object_PERSUBENTMGR *PERSUBENTMGR;
  Dwg_Object_POINTPATH *POINTPATH;
  Dwg_Object_RADIMLGOBJECTCONTEXTDATA *RADIMLGOBJECTCONTEXTDATA;
  Dwg_Object_RADIMOBJECTCONTEXTDATA *RADIMOBJECTCONTEXTDATA;
  Dwg_Object_SUNSTUDY *SUNSTUDY;
  Dwg_Object_TABLECONTENT *TABLECONTENT;
  Dwg_Object_TVDEVICEPROPERTIES *TVDEVICEPROPERTIES;
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
} _dwg_object_object_tio;


SWIGINTERNINLINE SV *
SWIG_From_size_t  SWIG_PERL_DECL_ARGS_1(size_t value)
{    
#ifdef SWIG_LONG_LONG_AVAILABLE
  if (sizeof(size_t) <= sizeof(unsigned long)) {
#endif
    return SWIG_From_unsigned_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long)(value));
#ifdef SWIG_LONG_LONG_AVAILABLE
  } else {
    /* assume sizeof(size_t) <= sizeof(unsigned long long) */
    return SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(value));
  }
#endif
}

typedef union {
  Dwg_Object_Entity *entity;
  Dwg_Object_Object *object;
} _dwg_object_tio;


#ifdef SWIG_LONG_LONG_AVAILABLE
SWIGINTERN int
SWIG_AsVal_long_SS_long SWIG_PERL_DECL_ARGS_2(SV *obj, long long *val)
{
  if (SvUOK(obj)) {
    UV v = SvUV(obj);
    /* pretty sure this could allow v == LLONG MAX */
    if (UVSIZE < sizeof(*val) || v < LLONG_MAX) {
      if (val) *val = v;
      return SWIG_OK;
    }
    return SWIG_OverflowError;
  } else if (SvIOK(obj)) {
    IV v = SvIV(obj);
    if (IVSIZE <= sizeof(*val) || (v >= LLONG_MIN && v <= LLONG_MAX)) {
      if (val) *val = v;
      return SWIG_OK;
    }
    return SWIG_OverflowError;
  } else {
    int dispatch = 0;
    const char *nptr = SvPV_nolen(obj);
    if (nptr) {
      char *endptr;
      long long v;
      errno = 0;
      v = strtoll(nptr, &endptr,0);
      if (errno == ERANGE) {
	errno = 0;
	return SWIG_OverflowError;
      } else {
	if (*endptr == '\0') {
	  if (val) *val = v;
	  return SWIG_Str2NumCast(SWIG_OK);
	}
      }
    }
    if (!dispatch) {
      const double mant_max = 1LL << DBL_MANT_DIG;
      const double mant_min = -mant_max;
      double d;
      int res = SWIG_AddCast(SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(obj,&d));
      if (SWIG_IsOK(res) && SWIG_CanCastAsInteger(&d, mant_min, mant_max)) {
	if (val) *val = (long long)(d);
	return res;
      }
    }
  }
  return SWIG_TypeError; 
}
#endif


#ifdef SWIG_LONG_LONG_AVAILABLE
SWIGINTERNINLINE SV *
SWIG_From_long_SS_long  SWIG_PERL_DECL_ARGS_1(long long value)
{
  SV *sv;
  if (IVSIZE >= sizeof(value) || (value >= IV_MIN && value <= IV_MAX))
    sv = newSViv((IV)(value));
  else {
    //sv = newSVpvf("%lld", value); doesn't work in non 64bit Perl
    char temp[256];
    SWIG_snprintf(temp, sizeof(temp), "%lld", value);
    sv = newSVpv(temp, 0);
  }
  return sv_2mortal(sv);
}
#endif

typedef union {
  Dwg_R2004_Header r2004_header;                        
  Dwg_R2007_Header r2007_file_header;                        
} _dwg_struct_fhdr;


SWIGINTERNINLINE SV *
SWIG_From_bool  SWIG_PERL_DECL_ARGS_1(bool value)
{
  return boolSV(value);
}


SWIGINTERN int
SWIG_AsVal_bool SWIG_PERL_DECL_ARGS_2(SV *obj, bool* val)
{
  if (obj == &PL_sv_yes) {
    if (val) *val = true;
    return SWIG_OK;
  } else if (obj == &PL_sv_no) { 
    if (val) *val = false;
    return SWIG_OK;
  } else {
    if (val) *val = SvTRUE(obj) ? true : false;
    return SWIG_AddCast(SWIG_OK);    
  }
}

#ifdef __cplusplus
extern "C" {
#endif

#define MAGIC_CLASS
SWIGCLASS_STATIC int swig_magic_readonly(pTHX_ SV *SWIGUNUSEDPARM(sv), MAGIC *SWIGUNUSEDPARM(mg)) {
    MAGIC_PPERL
    croak("Value is read-only.");
    return 0;
}

#ifdef __cplusplus
}
#endif

#ifdef __cplusplus
extern "C" {
#endif
XS(_wrap_new_Dwg_Object_Array) {
  {
    size_t arg1 ;
    size_t val1 ;
    int ecode1 = 0 ;
    int argvi = 0;
    Dwg_Object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: new_Dwg_Object_Array(nelements);");
    }
    ecode1 = SWIG_AsVal_size_t SWIG_PERL_CALL_ARGS_2(ST(0), &val1);
    if (!SWIG_IsOK(ecode1)) {
      SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "new_Dwg_Object_Array" "', argument " "1"" of type '" "size_t""'");
    } 
    arg1 = (size_t)(val1);
    result = (Dwg_Object *)new_Dwg_Object_Array(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_Array) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_Array(ary);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_Array" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    delete_Dwg_Object_Array(arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Array_getitem) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    size_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    size_t val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    Dwg_Object result;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Array_getitem(ary,index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Array_getitem" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    ecode2 = SWIG_AsVal_size_t SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Array_getitem" "', argument " "2"" of type '" "size_t""'");
    } 
    arg2 = (size_t)(val2);
    result = Dwg_Object_Array_getitem(arg1,arg2);
    ST(argvi) = SWIG_NewPointerObj((Dwg_Object *)memcpy((Dwg_Object *)calloc(1,sizeof(Dwg_Object)),&result,sizeof(Dwg_Object)), SWIGTYPE_p__dwg_object, SWIG_POINTER_OWN | SWIG_SHADOW); argvi++ ;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Array_setitem) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    size_t arg2 ;
    Dwg_Object arg3 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    size_t val2 ;
    int ecode2 = 0 ;
    void *argp3 ;
    int res3 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 3) || (items > 3)) {
      SWIG_croak("Usage: Dwg_Object_Array_setitem(ary,index,value);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Array_setitem" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    ecode2 = SWIG_AsVal_size_t SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Array_setitem" "', argument " "2"" of type '" "size_t""'");
    } 
    arg2 = (size_t)(val2);
    {
      res3 = SWIG_ConvertPtr(ST(2), &argp3, SWIGTYPE_p__dwg_object,  0 );
      if (!SWIG_IsOK(res3)) {
        SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "Dwg_Object_Array_setitem" "', argument " "3"" of type '" "Dwg_Object""'"); 
      }  
      if (!argp3) {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in method '" "Dwg_Object_Array_setitem" "', argument " "3"" of type '" "Dwg_Object""'");
      } else {
        arg3 = *((Dwg_Object *)(argp3));
      }
    }
    Dwg_Object_Array_setitem(arg1,arg2,arg3);
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Bitcode_TimeBLL_days_set) {
  {
    struct _dwg_time_bll *arg1 = (struct _dwg_time_bll *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Bitcode_TimeBLL_days_set(self,days);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_time_bll, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Bitcode_TimeBLL_days_set" "', argument " "1"" of type '" "struct _dwg_time_bll *""'"); 
    }
    arg1 = (struct _dwg_time_bll *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Bitcode_TimeBLL_days_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->days = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Bitcode_TimeBLL_days_get) {
  {
    struct _dwg_time_bll *arg1 = (struct _dwg_time_bll *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Bitcode_TimeBLL_days_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_time_bll, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Bitcode_TimeBLL_days_get" "', argument " "1"" of type '" "struct _dwg_time_bll *""'"); 
    }
    arg1 = (struct _dwg_time_bll *)(argp1);
    result = (BITCODE_BL) ((arg1)->days);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Bitcode_TimeBLL_ms_set) {
  {
    struct _dwg_time_bll *arg1 = (struct _dwg_time_bll *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Bitcode_TimeBLL_ms_set(self,ms);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_time_bll, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Bitcode_TimeBLL_ms_set" "', argument " "1"" of type '" "struct _dwg_time_bll *""'"); 
    }
    arg1 = (struct _dwg_time_bll *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Bitcode_TimeBLL_ms_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->ms = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Bitcode_TimeBLL_ms_get) {
  {
    struct _dwg_time_bll *arg1 = (struct _dwg_time_bll *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Bitcode_TimeBLL_ms_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_time_bll, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Bitcode_TimeBLL_ms_get" "', argument " "1"" of type '" "struct _dwg_time_bll *""'"); 
    }
    arg1 = (struct _dwg_time_bll *)(argp1);
    result = (BITCODE_BL) ((arg1)->ms);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Bitcode_TimeBLL_value_set) {
  {
    struct _dwg_time_bll *arg1 = (struct _dwg_time_bll *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Bitcode_TimeBLL_value_set(self,value);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_time_bll, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Bitcode_TimeBLL_value_set" "', argument " "1"" of type '" "struct _dwg_time_bll *""'"); 
    }
    arg1 = (struct _dwg_time_bll *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Bitcode_TimeBLL_value_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->value = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Bitcode_TimeBLL_value_get) {
  {
    struct _dwg_time_bll *arg1 = (struct _dwg_time_bll *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Bitcode_TimeBLL_value_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_time_bll, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Bitcode_TimeBLL_value_get" "', argument " "1"" of type '" "struct _dwg_time_bll *""'"); 
    }
    arg1 = (struct _dwg_time_bll *)(argp1);
    result = (BITCODE_BD) ((arg1)->value);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Bitcode_TimeBLL) {
  {
    int argvi = 0;
    struct _dwg_time_bll *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Bitcode_TimeBLL();");
    }
    result = (struct _dwg_time_bll *)calloc(1, sizeof(struct _dwg_time_bll));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_time_bll, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Bitcode_TimeBLL) {
  {
    struct _dwg_time_bll *arg1 = (struct _dwg_time_bll *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Bitcode_TimeBLL(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_time_bll, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Bitcode_TimeBLL" "', argument " "1"" of type '" "struct _dwg_time_bll *""'"); 
    }
    arg1 = (struct _dwg_time_bll *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Bitcode_2RD_x_set) {
  {
    struct _dwg_bitcode_2rd *arg1 = (struct _dwg_bitcode_2rd *) 0 ;
    BITCODE_RD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Bitcode_2RD_x_set(self,x);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Bitcode_2RD_x_set" "', argument " "1"" of type '" "struct _dwg_bitcode_2rd *""'"); 
    }
    arg1 = (struct _dwg_bitcode_2rd *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Bitcode_2RD_x_set" "', argument " "2"" of type '" "BITCODE_RD""'");
    } 
    arg2 = (BITCODE_RD)(val2);
    if (arg1) (arg1)->x = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Bitcode_2RD_x_get) {
  {
    struct _dwg_bitcode_2rd *arg1 = (struct _dwg_bitcode_2rd *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Bitcode_2RD_x_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Bitcode_2RD_x_get" "', argument " "1"" of type '" "struct _dwg_bitcode_2rd *""'"); 
    }
    arg1 = (struct _dwg_bitcode_2rd *)(argp1);
    result = (BITCODE_RD) ((arg1)->x);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Bitcode_2RD_y_set) {
  {
    struct _dwg_bitcode_2rd *arg1 = (struct _dwg_bitcode_2rd *) 0 ;
    BITCODE_RD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Bitcode_2RD_y_set(self,y);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Bitcode_2RD_y_set" "', argument " "1"" of type '" "struct _dwg_bitcode_2rd *""'"); 
    }
    arg1 = (struct _dwg_bitcode_2rd *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Bitcode_2RD_y_set" "', argument " "2"" of type '" "BITCODE_RD""'");
    } 
    arg2 = (BITCODE_RD)(val2);
    if (arg1) (arg1)->y = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Bitcode_2RD_y_get) {
  {
    struct _dwg_bitcode_2rd *arg1 = (struct _dwg_bitcode_2rd *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Bitcode_2RD_y_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Bitcode_2RD_y_get" "', argument " "1"" of type '" "struct _dwg_bitcode_2rd *""'"); 
    }
    arg1 = (struct _dwg_bitcode_2rd *)(argp1);
    result = (BITCODE_RD) ((arg1)->y);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Bitcode_2RD) {
  {
    int argvi = 0;
    struct _dwg_bitcode_2rd *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Bitcode_2RD();");
    }
    result = (struct _dwg_bitcode_2rd *)calloc(1, sizeof(struct _dwg_bitcode_2rd));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Bitcode_2RD) {
  {
    struct _dwg_bitcode_2rd *arg1 = (struct _dwg_bitcode_2rd *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Bitcode_2RD(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_bitcode_2rd, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Bitcode_2RD" "', argument " "1"" of type '" "struct _dwg_bitcode_2rd *""'"); 
    }
    arg1 = (struct _dwg_bitcode_2rd *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Bitcode_2BD_x_set) {
  {
    struct _dwg_bitcode_2bd *arg1 = (struct _dwg_bitcode_2bd *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Bitcode_2BD_x_set(self,x);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_bitcode_2bd, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Bitcode_2BD_x_set" "', argument " "1"" of type '" "struct _dwg_bitcode_2bd *""'"); 
    }
    arg1 = (struct _dwg_bitcode_2bd *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Bitcode_2BD_x_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->x = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Bitcode_2BD_x_get) {
  {
    struct _dwg_bitcode_2bd *arg1 = (struct _dwg_bitcode_2bd *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Bitcode_2BD_x_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_bitcode_2bd, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Bitcode_2BD_x_get" "', argument " "1"" of type '" "struct _dwg_bitcode_2bd *""'"); 
    }
    arg1 = (struct _dwg_bitcode_2bd *)(argp1);
    result = (BITCODE_BD) ((arg1)->x);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Bitcode_2BD_y_set) {
  {
    struct _dwg_bitcode_2bd *arg1 = (struct _dwg_bitcode_2bd *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Bitcode_2BD_y_set(self,y);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_bitcode_2bd, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Bitcode_2BD_y_set" "', argument " "1"" of type '" "struct _dwg_bitcode_2bd *""'"); 
    }
    arg1 = (struct _dwg_bitcode_2bd *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Bitcode_2BD_y_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->y = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Bitcode_2BD_y_get) {
  {
    struct _dwg_bitcode_2bd *arg1 = (struct _dwg_bitcode_2bd *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Bitcode_2BD_y_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_bitcode_2bd, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Bitcode_2BD_y_get" "', argument " "1"" of type '" "struct _dwg_bitcode_2bd *""'"); 
    }
    arg1 = (struct _dwg_bitcode_2bd *)(argp1);
    result = (BITCODE_BD) ((arg1)->y);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Bitcode_2BD) {
  {
    int argvi = 0;
    struct _dwg_bitcode_2bd *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Bitcode_2BD();");
    }
    result = (struct _dwg_bitcode_2bd *)calloc(1, sizeof(struct _dwg_bitcode_2bd));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2bd, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Bitcode_2BD) {
  {
    struct _dwg_bitcode_2bd *arg1 = (struct _dwg_bitcode_2bd *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Bitcode_2BD(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_bitcode_2bd, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Bitcode_2BD" "', argument " "1"" of type '" "struct _dwg_bitcode_2bd *""'"); 
    }
    arg1 = (struct _dwg_bitcode_2bd *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Bitcode_3RD_x_set) {
  {
    struct _dwg_bitcode_3rd *arg1 = (struct _dwg_bitcode_3rd *) 0 ;
    BITCODE_RD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Bitcode_3RD_x_set(self,x);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_bitcode_3rd, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Bitcode_3RD_x_set" "', argument " "1"" of type '" "struct _dwg_bitcode_3rd *""'"); 
    }
    arg1 = (struct _dwg_bitcode_3rd *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Bitcode_3RD_x_set" "', argument " "2"" of type '" "BITCODE_RD""'");
    } 
    arg2 = (BITCODE_RD)(val2);
    if (arg1) (arg1)->x = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Bitcode_3RD_x_get) {
  {
    struct _dwg_bitcode_3rd *arg1 = (struct _dwg_bitcode_3rd *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Bitcode_3RD_x_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_bitcode_3rd, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Bitcode_3RD_x_get" "', argument " "1"" of type '" "struct _dwg_bitcode_3rd *""'"); 
    }
    arg1 = (struct _dwg_bitcode_3rd *)(argp1);
    result = (BITCODE_RD) ((arg1)->x);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Bitcode_3RD_y_set) {
  {
    struct _dwg_bitcode_3rd *arg1 = (struct _dwg_bitcode_3rd *) 0 ;
    BITCODE_RD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Bitcode_3RD_y_set(self,y);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_bitcode_3rd, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Bitcode_3RD_y_set" "', argument " "1"" of type '" "struct _dwg_bitcode_3rd *""'"); 
    }
    arg1 = (struct _dwg_bitcode_3rd *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Bitcode_3RD_y_set" "', argument " "2"" of type '" "BITCODE_RD""'");
    } 
    arg2 = (BITCODE_RD)(val2);
    if (arg1) (arg1)->y = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Bitcode_3RD_y_get) {
  {
    struct _dwg_bitcode_3rd *arg1 = (struct _dwg_bitcode_3rd *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Bitcode_3RD_y_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_bitcode_3rd, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Bitcode_3RD_y_get" "', argument " "1"" of type '" "struct _dwg_bitcode_3rd *""'"); 
    }
    arg1 = (struct _dwg_bitcode_3rd *)(argp1);
    result = (BITCODE_RD) ((arg1)->y);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Bitcode_3RD_z_set) {
  {
    struct _dwg_bitcode_3rd *arg1 = (struct _dwg_bitcode_3rd *) 0 ;
    BITCODE_RD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Bitcode_3RD_z_set(self,z);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_bitcode_3rd, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Bitcode_3RD_z_set" "', argument " "1"" of type '" "struct _dwg_bitcode_3rd *""'"); 
    }
    arg1 = (struct _dwg_bitcode_3rd *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Bitcode_3RD_z_set" "', argument " "2"" of type '" "BITCODE_RD""'");
    } 
    arg2 = (BITCODE_RD)(val2);
    if (arg1) (arg1)->z = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Bitcode_3RD_z_get) {
  {
    struct _dwg_bitcode_3rd *arg1 = (struct _dwg_bitcode_3rd *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Bitcode_3RD_z_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_bitcode_3rd, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Bitcode_3RD_z_get" "', argument " "1"" of type '" "struct _dwg_bitcode_3rd *""'"); 
    }
    arg1 = (struct _dwg_bitcode_3rd *)(argp1);
    result = (BITCODE_RD) ((arg1)->z);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Bitcode_3RD) {
  {
    int argvi = 0;
    struct _dwg_bitcode_3rd *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Bitcode_3RD();");
    }
    result = (struct _dwg_bitcode_3rd *)calloc(1, sizeof(struct _dwg_bitcode_3rd));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3rd, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Bitcode_3RD) {
  {
    struct _dwg_bitcode_3rd *arg1 = (struct _dwg_bitcode_3rd *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Bitcode_3RD(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_bitcode_3rd, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Bitcode_3RD" "', argument " "1"" of type '" "struct _dwg_bitcode_3rd *""'"); 
    }
    arg1 = (struct _dwg_bitcode_3rd *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Bitcode_3BD_x_set) {
  {
    struct _dwg_bitcode_3bd *arg1 = (struct _dwg_bitcode_3bd *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Bitcode_3BD_x_set(self,x);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Bitcode_3BD_x_set" "', argument " "1"" of type '" "struct _dwg_bitcode_3bd *""'"); 
    }
    arg1 = (struct _dwg_bitcode_3bd *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Bitcode_3BD_x_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->x = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Bitcode_3BD_x_get) {
  {
    struct _dwg_bitcode_3bd *arg1 = (struct _dwg_bitcode_3bd *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Bitcode_3BD_x_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Bitcode_3BD_x_get" "', argument " "1"" of type '" "struct _dwg_bitcode_3bd *""'"); 
    }
    arg1 = (struct _dwg_bitcode_3bd *)(argp1);
    result = (BITCODE_BD) ((arg1)->x);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Bitcode_3BD_y_set) {
  {
    struct _dwg_bitcode_3bd *arg1 = (struct _dwg_bitcode_3bd *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Bitcode_3BD_y_set(self,y);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Bitcode_3BD_y_set" "', argument " "1"" of type '" "struct _dwg_bitcode_3bd *""'"); 
    }
    arg1 = (struct _dwg_bitcode_3bd *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Bitcode_3BD_y_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->y = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Bitcode_3BD_y_get) {
  {
    struct _dwg_bitcode_3bd *arg1 = (struct _dwg_bitcode_3bd *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Bitcode_3BD_y_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Bitcode_3BD_y_get" "', argument " "1"" of type '" "struct _dwg_bitcode_3bd *""'"); 
    }
    arg1 = (struct _dwg_bitcode_3bd *)(argp1);
    result = (BITCODE_BD) ((arg1)->y);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Bitcode_3BD_z_set) {
  {
    struct _dwg_bitcode_3bd *arg1 = (struct _dwg_bitcode_3bd *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Bitcode_3BD_z_set(self,z);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Bitcode_3BD_z_set" "', argument " "1"" of type '" "struct _dwg_bitcode_3bd *""'"); 
    }
    arg1 = (struct _dwg_bitcode_3bd *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Bitcode_3BD_z_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->z = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Bitcode_3BD_z_get) {
  {
    struct _dwg_bitcode_3bd *arg1 = (struct _dwg_bitcode_3bd *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Bitcode_3BD_z_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Bitcode_3BD_z_get" "', argument " "1"" of type '" "struct _dwg_bitcode_3bd *""'"); 
    }
    arg1 = (struct _dwg_bitcode_3bd *)(argp1);
    result = (BITCODE_BD) ((arg1)->z);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Bitcode_3BD) {
  {
    int argvi = 0;
    struct _dwg_bitcode_3bd *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Bitcode_3BD();");
    }
    result = (struct _dwg_bitcode_3bd *)calloc(1, sizeof(struct _dwg_bitcode_3bd));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Bitcode_3BD) {
  {
    struct _dwg_bitcode_3bd *arg1 = (struct _dwg_bitcode_3bd *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Bitcode_3BD(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_bitcode_3bd, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Bitcode_3BD" "', argument " "1"" of type '" "struct _dwg_bitcode_3bd *""'"); 
    }
    arg1 = (struct _dwg_bitcode_3bd *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Versions_r_set) {
  {
    struct dwg_versions *arg1 = (struct dwg_versions *) 0 ;
    Dwg_Version_Type arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Versions_r_set(self,r);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_dwg_versions, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Versions_r_set" "', argument " "1"" of type '" "struct dwg_versions *""'"); 
    }
    arg1 = (struct dwg_versions *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Versions_r_set" "', argument " "2"" of type '" "Dwg_Version_Type""'");
    } 
    arg2 = (Dwg_Version_Type)(val2);
    if (arg1) (arg1)->r = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Versions_r_get) {
  {
    struct dwg_versions *arg1 = (struct dwg_versions *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Version_Type result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Versions_r_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_dwg_versions, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Versions_r_get" "', argument " "1"" of type '" "struct dwg_versions *""'"); 
    }
    arg1 = (struct dwg_versions *)(argp1);
    result = (Dwg_Version_Type) ((arg1)->r);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Versions_type_get) {
  {
    struct dwg_versions *arg1 = (struct dwg_versions *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    char *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Versions_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_dwg_versions, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Versions_type_get" "', argument " "1"" of type '" "struct dwg_versions *""'"); 
    }
    arg1 = (struct dwg_versions *)(argp1);
    result = (char *)(char *) ((arg1)->type);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Versions_hdr_get) {
  {
    struct dwg_versions *arg1 = (struct dwg_versions *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    char *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Versions_hdr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_dwg_versions, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Versions_hdr_get" "', argument " "1"" of type '" "struct dwg_versions *""'"); 
    }
    arg1 = (struct dwg_versions *)(argp1);
    result = (char *)(char *) ((arg1)->hdr);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Versions_desc_get) {
  {
    struct dwg_versions *arg1 = (struct dwg_versions *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    char *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Versions_desc_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_dwg_versions, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Versions_desc_get" "', argument " "1"" of type '" "struct dwg_versions *""'"); 
    }
    arg1 = (struct dwg_versions *)(argp1);
    result = (char *)(char *) ((arg1)->desc);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Versions_dwg_version_set) {
  {
    struct dwg_versions *arg1 = (struct dwg_versions *) 0 ;
    uint8_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Versions_dwg_version_set(self,dwg_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_dwg_versions, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Versions_dwg_version_set" "', argument " "1"" of type '" "struct dwg_versions *""'"); 
    }
    arg1 = (struct dwg_versions *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Versions_dwg_version_set" "', argument " "2"" of type '" "uint8_t""'");
    } 
    arg2 = (uint8_t)(val2);
    if (arg1) (arg1)->dwg_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Versions_dwg_version_get) {
  {
    struct dwg_versions *arg1 = (struct dwg_versions *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    uint8_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Versions_dwg_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_dwg_versions, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Versions_dwg_version_get" "', argument " "1"" of type '" "struct dwg_versions *""'"); 
    }
    arg1 = (struct dwg_versions *)(argp1);
    result = (uint8_t) ((arg1)->dwg_version);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Versions) {
  {
    int argvi = 0;
    struct dwg_versions *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Versions();");
    }
    result = (struct dwg_versions *)calloc(1, sizeof(struct dwg_versions));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_dwg_versions, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Versions) {
  {
    struct dwg_versions *arg1 = (struct dwg_versions *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Versions(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_dwg_versions, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Versions" "', argument " "1"" of type '" "struct dwg_versions *""'"); 
    }
    arg1 = (struct dwg_versions *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Handle_code_set) {
  {
    struct _dwg_handle *arg1 = (struct _dwg_handle *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Handle_code_set(self,code);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_handle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Handle_code_set" "', argument " "1"" of type '" "struct _dwg_handle *""'"); 
    }
    arg1 = (struct _dwg_handle *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Handle_code_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->code = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Handle_code_get) {
  {
    struct _dwg_handle *arg1 = (struct _dwg_handle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Handle_code_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_handle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Handle_code_get" "', argument " "1"" of type '" "struct _dwg_handle *""'"); 
    }
    arg1 = (struct _dwg_handle *)(argp1);
    result = (BITCODE_RC) ((arg1)->code);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Handle_size_set) {
  {
    struct _dwg_handle *arg1 = (struct _dwg_handle *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Handle_size_set(self,size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_handle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Handle_size_set" "', argument " "1"" of type '" "struct _dwg_handle *""'"); 
    }
    arg1 = (struct _dwg_handle *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Handle_size_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Handle_size_get) {
  {
    struct _dwg_handle *arg1 = (struct _dwg_handle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Handle_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_handle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Handle_size_get" "', argument " "1"" of type '" "struct _dwg_handle *""'"); 
    }
    arg1 = (struct _dwg_handle *)(argp1);
    result = (BITCODE_RC) ((arg1)->size);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Handle_value_set) {
  {
    struct _dwg_handle *arg1 = (struct _dwg_handle *) 0 ;
    BITCODE_HV arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Handle_value_set(self,value);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_handle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Handle_value_set" "', argument " "1"" of type '" "struct _dwg_handle *""'"); 
    }
    arg1 = (struct _dwg_handle *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Handle_value_set" "', argument " "2"" of type '" "BITCODE_HV""'");
    } 
    arg2 = (BITCODE_HV)(val2);
    if (arg1) (arg1)->value = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Handle_value_get) {
  {
    struct _dwg_handle *arg1 = (struct _dwg_handle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_HV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Handle_value_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_handle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Handle_value_get" "', argument " "1"" of type '" "struct _dwg_handle *""'"); 
    }
    arg1 = (struct _dwg_handle *)(argp1);
    result = (BITCODE_HV) ((arg1)->value);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Handle_is_global_set) {
  {
    struct _dwg_handle *arg1 = (struct _dwg_handle *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Handle_is_global_set(self,is_global);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_handle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Handle_is_global_set" "', argument " "1"" of type '" "struct _dwg_handle *""'"); 
    }
    arg1 = (struct _dwg_handle *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Handle_is_global_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_global = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Handle_is_global_get) {
  {
    struct _dwg_handle *arg1 = (struct _dwg_handle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Handle_is_global_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_handle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Handle_is_global_get" "', argument " "1"" of type '" "struct _dwg_handle *""'"); 
    }
    arg1 = (struct _dwg_handle *)(argp1);
    result = (BITCODE_B) ((arg1)->is_global);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Handle) {
  {
    int argvi = 0;
    struct _dwg_handle *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Handle();");
    }
    result = (struct _dwg_handle *)calloc(1, sizeof(struct _dwg_handle));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_handle, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Handle) {
  {
    struct _dwg_handle *arg1 = (struct _dwg_handle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Handle(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_handle, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Handle" "', argument " "1"" of type '" "struct _dwg_handle *""'"); 
    }
    arg1 = (struct _dwg_handle *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Ref_obj_set) {
  {
    struct _dwg_object_ref *arg1 = (struct _dwg_object_ref *) 0 ;
    struct _dwg_object *arg2 = (struct _dwg_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Ref_obj_set(self,obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Ref_obj_set" "', argument " "1"" of type '" "struct _dwg_object_ref *""'"); 
    }
    arg1 = (struct _dwg_object_ref *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_Ref_obj_set" "', argument " "2"" of type '" "struct _dwg_object *""'"); 
    }
    arg2 = (struct _dwg_object *)(argp2);
    if (arg1) (arg1)->obj = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Ref_obj_get) {
  {
    struct _dwg_object_ref *arg1 = (struct _dwg_object_ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Ref_obj_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Ref_obj_get" "', argument " "1"" of type '" "struct _dwg_object_ref *""'"); 
    }
    arg1 = (struct _dwg_object_ref *)(argp1);
    result = (struct _dwg_object *) ((arg1)->obj);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Ref_handleref_set) {
  {
    struct _dwg_object_ref *arg1 = (struct _dwg_object_ref *) 0 ;
    Dwg_Handle *arg2 = (Dwg_Handle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Ref_handleref_set(self,handleref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Ref_handleref_set" "', argument " "1"" of type '" "struct _dwg_object_ref *""'"); 
    }
    arg1 = (struct _dwg_object_ref *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_handle, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_Ref_handleref_set" "', argument " "2"" of type '" "Dwg_Handle *""'"); 
    }
    arg2 = (Dwg_Handle *)(argp2);
    if (arg1) (arg1)->handleref = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Ref_handleref_get) {
  {
    struct _dwg_object_ref *arg1 = (struct _dwg_object_ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Handle *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Ref_handleref_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Ref_handleref_get" "', argument " "1"" of type '" "struct _dwg_object_ref *""'"); 
    }
    arg1 = (struct _dwg_object_ref *)(argp1);
    result = (Dwg_Handle *)& ((arg1)->handleref);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_handle, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Ref_absolute_ref_set) {
  {
    struct _dwg_object_ref *arg1 = (struct _dwg_object_ref *) 0 ;
    BITCODE_RLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Ref_absolute_ref_set(self,absolute_ref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Ref_absolute_ref_set" "', argument " "1"" of type '" "struct _dwg_object_ref *""'"); 
    }
    arg1 = (struct _dwg_object_ref *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Ref_absolute_ref_set" "', argument " "2"" of type '" "BITCODE_RLL""'");
    } 
    arg2 = (BITCODE_RLL)(val2);
    if (arg1) (arg1)->absolute_ref = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Ref_absolute_ref_get) {
  {
    struct _dwg_object_ref *arg1 = (struct _dwg_object_ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Ref_absolute_ref_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Ref_absolute_ref_get" "', argument " "1"" of type '" "struct _dwg_object_ref *""'"); 
    }
    arg1 = (struct _dwg_object_ref *)(argp1);
    result = (BITCODE_RLL) ((arg1)->absolute_ref);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Ref_r11_idx_set) {
  {
    struct _dwg_object_ref *arg1 = (struct _dwg_object_ref *) 0 ;
    BITCODE_RSd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Ref_r11_idx_set(self,r11_idx);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Ref_r11_idx_set" "', argument " "1"" of type '" "struct _dwg_object_ref *""'"); 
    }
    arg1 = (struct _dwg_object_ref *)(argp1);
    ecode2 = SWIG_AsVal_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Ref_r11_idx_set" "', argument " "2"" of type '" "BITCODE_RSd""'");
    } 
    arg2 = (BITCODE_RSd)(val2);
    if (arg1) (arg1)->r11_idx = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Ref_r11_idx_get) {
  {
    struct _dwg_object_ref *arg1 = (struct _dwg_object_ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RSd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Ref_r11_idx_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Ref_r11_idx_get" "', argument " "1"" of type '" "struct _dwg_object_ref *""'"); 
    }
    arg1 = (struct _dwg_object_ref *)(argp1);
    result = (BITCODE_RSd) ((arg1)->r11_idx);
    ST(argvi) = SWIG_From_short  SWIG_PERL_CALL_ARGS_1((short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_Ref) {
  {
    int argvi = 0;
    struct _dwg_object_ref *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_Ref();");
    }
    result = (struct _dwg_object_ref *)calloc(1, sizeof(struct _dwg_object_ref));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_Ref) {
  {
    struct _dwg_object_ref *arg1 = (struct _dwg_object_ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_Ref(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_Ref" "', argument " "1"" of type '" "struct _dwg_object_ref *""'"); 
    }
    arg1 = (struct _dwg_object_ref *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Color_index_set) {
  {
    struct _dwg_color *arg1 = (struct _dwg_color *) 0 ;
    BITCODE_BSd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Color_index_set(self,index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Color_index_set" "', argument " "1"" of type '" "struct _dwg_color *""'"); 
    }
    arg1 = (struct _dwg_color *)(argp1);
    ecode2 = SWIG_AsVal_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Color_index_set" "', argument " "2"" of type '" "BITCODE_BSd""'");
    } 
    arg2 = (BITCODE_BSd)(val2);
    if (arg1) (arg1)->index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Color_index_get) {
  {
    struct _dwg_color *arg1 = (struct _dwg_color *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BSd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Color_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Color_index_get" "', argument " "1"" of type '" "struct _dwg_color *""'"); 
    }
    arg1 = (struct _dwg_color *)(argp1);
    result = (BITCODE_BSd) ((arg1)->index);
    ST(argvi) = SWIG_From_short  SWIG_PERL_CALL_ARGS_1((short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Color_flag_set) {
  {
    struct _dwg_color *arg1 = (struct _dwg_color *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Color_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Color_flag_set" "', argument " "1"" of type '" "struct _dwg_color *""'"); 
    }
    arg1 = (struct _dwg_color *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Color_flag_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Color_flag_get) {
  {
    struct _dwg_color *arg1 = (struct _dwg_color *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Color_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Color_flag_get" "', argument " "1"" of type '" "struct _dwg_color *""'"); 
    }
    arg1 = (struct _dwg_color *)(argp1);
    result = (BITCODE_BS) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Color_raw_set) {
  {
    struct _dwg_color *arg1 = (struct _dwg_color *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Color_raw_set(self,raw);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Color_raw_set" "', argument " "1"" of type '" "struct _dwg_color *""'"); 
    }
    arg1 = (struct _dwg_color *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Color_raw_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->raw = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Color_raw_get) {
  {
    struct _dwg_color *arg1 = (struct _dwg_color *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Color_raw_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Color_raw_get" "', argument " "1"" of type '" "struct _dwg_color *""'"); 
    }
    arg1 = (struct _dwg_color *)(argp1);
    result = (BITCODE_BS) ((arg1)->raw);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Color_rgb_set) {
  {
    struct _dwg_color *arg1 = (struct _dwg_color *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Color_rgb_set(self,rgb);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Color_rgb_set" "', argument " "1"" of type '" "struct _dwg_color *""'"); 
    }
    arg1 = (struct _dwg_color *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Color_rgb_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->rgb = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Color_rgb_get) {
  {
    struct _dwg_color *arg1 = (struct _dwg_color *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Color_rgb_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Color_rgb_get" "', argument " "1"" of type '" "struct _dwg_color *""'"); 
    }
    arg1 = (struct _dwg_color *)(argp1);
    result = (BITCODE_BL) ((arg1)->rgb);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Color_method_set) {
  {
    struct _dwg_color *arg1 = (struct _dwg_color *) 0 ;
    Dwg_Color_Method arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Color_method_set(self,method);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Color_method_set" "', argument " "1"" of type '" "struct _dwg_color *""'"); 
    }
    arg1 = (struct _dwg_color *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Color_method_set" "', argument " "2"" of type '" "Dwg_Color_Method""'");
    } 
    arg2 = (Dwg_Color_Method)(val2);
    if (arg1) (arg1)->method = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Color_method_get) {
  {
    struct _dwg_color *arg1 = (struct _dwg_color *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Color_Method result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Color_method_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Color_method_get" "', argument " "1"" of type '" "struct _dwg_color *""'"); 
    }
    arg1 = (struct _dwg_color *)(argp1);
    result = (Dwg_Color_Method) ((arg1)->method);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Color_name_set) {
  {
    struct _dwg_color *arg1 = (struct _dwg_color *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Color_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Color_name_set" "', argument " "1"" of type '" "struct _dwg_color *""'"); 
    }
    arg1 = (struct _dwg_color *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Color_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Color_name_get) {
  {
    struct _dwg_color *arg1 = (struct _dwg_color *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Color_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Color_name_get" "', argument " "1"" of type '" "struct _dwg_color *""'"); 
    }
    arg1 = (struct _dwg_color *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Color_book_name_set) {
  {
    struct _dwg_color *arg1 = (struct _dwg_color *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Color_book_name_set(self,book_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Color_book_name_set" "', argument " "1"" of type '" "struct _dwg_color *""'"); 
    }
    arg1 = (struct _dwg_color *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Color_book_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->book_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->book_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->book_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Color_book_name_get) {
  {
    struct _dwg_color *arg1 = (struct _dwg_color *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Color_book_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Color_book_name_get" "', argument " "1"" of type '" "struct _dwg_color *""'"); 
    }
    arg1 = (struct _dwg_color *)(argp1);
    result = (BITCODE_TV) ((arg1)->book_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Color_handle_set) {
  {
    struct _dwg_color *arg1 = (struct _dwg_color *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Color_handle_set(self,handle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Color_handle_set" "', argument " "1"" of type '" "struct _dwg_color *""'"); 
    }
    arg1 = (struct _dwg_color *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Color_handle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->handle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Color_handle_get) {
  {
    struct _dwg_color *arg1 = (struct _dwg_color *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Color_handle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Color_handle_get" "', argument " "1"" of type '" "struct _dwg_color *""'"); 
    }
    arg1 = (struct _dwg_color *)(argp1);
    result = (BITCODE_H) ((arg1)->handle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Color_alpha_raw_set) {
  {
    struct _dwg_color *arg1 = (struct _dwg_color *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Color_alpha_raw_set(self,alpha_raw);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Color_alpha_raw_set" "', argument " "1"" of type '" "struct _dwg_color *""'"); 
    }
    arg1 = (struct _dwg_color *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Color_alpha_raw_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->alpha_raw = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Color_alpha_raw_get) {
  {
    struct _dwg_color *arg1 = (struct _dwg_color *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Color_alpha_raw_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Color_alpha_raw_get" "', argument " "1"" of type '" "struct _dwg_color *""'"); 
    }
    arg1 = (struct _dwg_color *)(argp1);
    result = (BITCODE_BL) ((arg1)->alpha_raw);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Color_alpha_type_set) {
  {
    struct _dwg_color *arg1 = (struct _dwg_color *) 0 ;
    BITCODE_BB arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Color_alpha_type_set(self,alpha_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Color_alpha_type_set" "', argument " "1"" of type '" "struct _dwg_color *""'"); 
    }
    arg1 = (struct _dwg_color *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Color_alpha_type_set" "', argument " "2"" of type '" "BITCODE_BB""'");
    } 
    arg2 = (BITCODE_BB)(val2);
    if (arg1) (arg1)->alpha_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Color_alpha_type_get) {
  {
    struct _dwg_color *arg1 = (struct _dwg_color *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BB result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Color_alpha_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Color_alpha_type_get" "', argument " "1"" of type '" "struct _dwg_color *""'"); 
    }
    arg1 = (struct _dwg_color *)(argp1);
    result = (BITCODE_BB) ((arg1)->alpha_type);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Color_alpha_set) {
  {
    struct _dwg_color *arg1 = (struct _dwg_color *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Color_alpha_set(self,alpha);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Color_alpha_set" "', argument " "1"" of type '" "struct _dwg_color *""'"); 
    }
    arg1 = (struct _dwg_color *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Color_alpha_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->alpha = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Color_alpha_get) {
  {
    struct _dwg_color *arg1 = (struct _dwg_color *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Color_alpha_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Color_alpha_get" "', argument " "1"" of type '" "struct _dwg_color *""'"); 
    }
    arg1 = (struct _dwg_color *)(argp1);
    result = (BITCODE_RC) ((arg1)->alpha);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Color) {
  {
    int argvi = 0;
    struct _dwg_color *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Color();");
    }
    result = (struct _dwg_color *)calloc(1, sizeof(struct _dwg_color));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Color) {
  {
    struct _dwg_color *arg1 = (struct _dwg_color *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Color(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_color, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Color" "', argument " "1"" of type '" "struct _dwg_color *""'"); 
    }
    arg1 = (struct _dwg_color *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_color_method_name) {
  {
    unsigned int arg1 ;
    unsigned int val1 ;
    int ecode1 = 0 ;
    int argvi = 0;
    char *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_color_method_name(method);");
    }
    ecode1 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(0), &val1);
    if (!SWIG_IsOK(ecode1)) {
      SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "dwg_color_method_name" "', argument " "1"" of type '" "unsigned int""'");
    } 
    arg1 = (unsigned int)(val1);
    result = (char *)dwg_color_method_name(arg1);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_binary_chunk_size_set) {
  {
    struct _dwg_binary_chunk *arg1 = (struct _dwg_binary_chunk *) 0 ;
    unsigned short arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_binary_chunk_size_set(self,size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_binary_chunk, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_binary_chunk_size_set" "', argument " "1"" of type '" "struct _dwg_binary_chunk *""'"); 
    }
    arg1 = (struct _dwg_binary_chunk *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "_dwg_binary_chunk_size_set" "', argument " "2"" of type '" "unsigned short""'");
    } 
    arg2 = (unsigned short)(val2);
    if (arg1) (arg1)->size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_binary_chunk_size_get) {
  {
    struct _dwg_binary_chunk *arg1 = (struct _dwg_binary_chunk *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    unsigned short result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_binary_chunk_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_binary_chunk, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_binary_chunk_size_get" "', argument " "1"" of type '" "struct _dwg_binary_chunk *""'"); 
    }
    arg1 = (struct _dwg_binary_chunk *)(argp1);
    result = (unsigned short) ((arg1)->size);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_binary_chunk_codepage_set) {
  {
    struct _dwg_binary_chunk *arg1 = (struct _dwg_binary_chunk *) 0 ;
    unsigned int arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_binary_chunk_codepage_set(self,codepage);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_binary_chunk, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_binary_chunk_codepage_set" "', argument " "1"" of type '" "struct _dwg_binary_chunk *""'"); 
    }
    arg1 = (struct _dwg_binary_chunk *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "_dwg_binary_chunk_codepage_set" "', argument " "2"" of type '" "unsigned int""'");
    } 
    arg2 = (unsigned int)(val2);
    if (arg1) (arg1)->codepage = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_binary_chunk_codepage_get) {
  {
    struct _dwg_binary_chunk *arg1 = (struct _dwg_binary_chunk *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    unsigned int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_binary_chunk_codepage_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_binary_chunk, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_binary_chunk_codepage_get" "', argument " "1"" of type '" "struct _dwg_binary_chunk *""'"); 
    }
    arg1 = (struct _dwg_binary_chunk *)(argp1);
    result = (unsigned int) ((arg1)->codepage);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_binary_chunk_is_tu_set) {
  {
    struct _dwg_binary_chunk *arg1 = (struct _dwg_binary_chunk *) 0 ;
    unsigned int arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_binary_chunk_is_tu_set(self,is_tu);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_binary_chunk, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_binary_chunk_is_tu_set" "', argument " "1"" of type '" "struct _dwg_binary_chunk *""'"); 
    }
    arg1 = (struct _dwg_binary_chunk *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "_dwg_binary_chunk_is_tu_set" "', argument " "2"" of type '" "unsigned int""'");
    } 
    arg2 = (unsigned int)(val2);
    if (arg1) (arg1)->is_tu = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_binary_chunk_is_tu_get) {
  {
    struct _dwg_binary_chunk *arg1 = (struct _dwg_binary_chunk *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    unsigned int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_binary_chunk_is_tu_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_binary_chunk, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_binary_chunk_is_tu_get" "', argument " "1"" of type '" "struct _dwg_binary_chunk *""'"); 
    }
    arg1 = (struct _dwg_binary_chunk *)(argp1);
    result = (unsigned int) ((arg1)->is_tu);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_binary_chunk_u_get) {
  {
    struct _dwg_binary_chunk *arg1 = (struct _dwg_binary_chunk *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    _dwg_binary_chunk_u *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_binary_chunk_u_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_binary_chunk, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_binary_chunk_u_get" "', argument " "1"" of type '" "struct _dwg_binary_chunk *""'"); 
    }
    arg1 = (struct _dwg_binary_chunk *)(argp1);
    result = (_dwg_binary_chunk_u *)& ((arg1)->u);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_binary_chunk_u, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new__dwg_binary_chunk) {
  {
    int argvi = 0;
    struct _dwg_binary_chunk *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new__dwg_binary_chunk();");
    }
    result = (struct _dwg_binary_chunk *)calloc(1, sizeof(struct _dwg_binary_chunk));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_binary_chunk, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete__dwg_binary_chunk) {
  {
    struct _dwg_binary_chunk *arg1 = (struct _dwg_binary_chunk *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete__dwg_binary_chunk(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_binary_chunk, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete__dwg_binary_chunk" "', argument " "1"" of type '" "struct _dwg_binary_chunk *""'"); 
    }
    arg1 = (struct _dwg_binary_chunk *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_binary_chunk_u_data_set) {
  {
    _dwg_binary_chunk_u *arg1 = (_dwg_binary_chunk_u *) 0 ;
    char *arg2 = (char *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_binary_chunk_u_data_set(self,data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_binary_chunk_u, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_binary_chunk_u_data_set" "', argument " "1"" of type '" "_dwg_binary_chunk_u *""'"); 
    }
    arg1 = (_dwg_binary_chunk_u *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_binary_chunk_u_data_set" "', argument " "2"" of type '" "char *""'");
    }
    arg2 = (char *)(buf2);
    free((char*)arg1->data);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->data = (char *)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->data = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_binary_chunk_u_data_get) {
  {
    _dwg_binary_chunk_u *arg1 = (_dwg_binary_chunk_u *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    char *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_binary_chunk_u_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_binary_chunk_u, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_binary_chunk_u_data_get" "', argument " "1"" of type '" "_dwg_binary_chunk_u *""'"); 
    }
    arg1 = (_dwg_binary_chunk_u *)(argp1);
    result = (char *) ((arg1)->data);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_binary_chunk_u_wdata_set) {
  {
    _dwg_binary_chunk_u *arg1 = (_dwg_binary_chunk_u *) 0 ;
    dwg_wchar_t *arg2 = (dwg_wchar_t *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_binary_chunk_u_wdata_set(self,wdata);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_binary_chunk_u, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_binary_chunk_u_wdata_set" "', argument " "1"" of type '" "_dwg_binary_chunk_u *""'"); 
    }
    arg1 = (_dwg_binary_chunk_u *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_short, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_binary_chunk_u_wdata_set" "', argument " "2"" of type '" "dwg_wchar_t *""'"); 
    }
    arg2 = (dwg_wchar_t *)(argp2);
    if (arg1) (arg1)->wdata = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_binary_chunk_u_wdata_get) {
  {
    _dwg_binary_chunk_u *arg1 = (_dwg_binary_chunk_u *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dwg_wchar_t *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_binary_chunk_u_wdata_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_binary_chunk_u, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_binary_chunk_u_wdata_get" "', argument " "1"" of type '" "_dwg_binary_chunk_u *""'"); 
    }
    arg1 = (_dwg_binary_chunk_u *)(argp1);
    result = (dwg_wchar_t *) ((arg1)->wdata);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_short, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new__dwg_binary_chunk_u) {
  {
    int argvi = 0;
    _dwg_binary_chunk_u *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new__dwg_binary_chunk_u();");
    }
    result = (_dwg_binary_chunk_u *)calloc(1, sizeof(_dwg_binary_chunk_u));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_binary_chunk_u, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete__dwg_binary_chunk_u) {
  {
    _dwg_binary_chunk_u *arg1 = (_dwg_binary_chunk_u *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete__dwg_binary_chunk_u(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_binary_chunk_u, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete__dwg_binary_chunk_u" "', argument " "1"" of type '" "_dwg_binary_chunk_u *""'"); 
    }
    arg1 = (_dwg_binary_chunk_u *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Resbuf_type_set) {
  {
    struct _dwg_resbuf *arg1 = (struct _dwg_resbuf *) 0 ;
    short arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Resbuf_type_set(self,type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_resbuf, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Resbuf_type_set" "', argument " "1"" of type '" "struct _dwg_resbuf *""'"); 
    }
    arg1 = (struct _dwg_resbuf *)(argp1);
    ecode2 = SWIG_AsVal_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Resbuf_type_set" "', argument " "2"" of type '" "short""'");
    } 
    arg2 = (short)(val2);
    if (arg1) (arg1)->type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Resbuf_type_get) {
  {
    struct _dwg_resbuf *arg1 = (struct _dwg_resbuf *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    short result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Resbuf_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_resbuf, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Resbuf_type_get" "', argument " "1"" of type '" "struct _dwg_resbuf *""'"); 
    }
    arg1 = (struct _dwg_resbuf *)(argp1);
    result = (short) ((arg1)->type);
    ST(argvi) = SWIG_From_short  SWIG_PERL_CALL_ARGS_1((short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Resbuf_value_get) {
  {
    struct _dwg_resbuf *arg1 = (struct _dwg_resbuf *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    _dwg_resbuf_value *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Resbuf_value_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_resbuf, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Resbuf_value_get" "', argument " "1"" of type '" "struct _dwg_resbuf *""'"); 
    }
    arg1 = (struct _dwg_resbuf *)(argp1);
    result = (_dwg_resbuf_value *)& ((arg1)->value);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_resbuf_value, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Resbuf_nextrb_set) {
  {
    struct _dwg_resbuf *arg1 = (struct _dwg_resbuf *) 0 ;
    struct _dwg_resbuf *arg2 = (struct _dwg_resbuf *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Resbuf_nextrb_set(self,nextrb);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_resbuf, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Resbuf_nextrb_set" "', argument " "1"" of type '" "struct _dwg_resbuf *""'"); 
    }
    arg1 = (struct _dwg_resbuf *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_resbuf, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Resbuf_nextrb_set" "', argument " "2"" of type '" "struct _dwg_resbuf *""'"); 
    }
    arg2 = (struct _dwg_resbuf *)(argp2);
    if (arg1) (arg1)->nextrb = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Resbuf_nextrb_get) {
  {
    struct _dwg_resbuf *arg1 = (struct _dwg_resbuf *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_resbuf *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Resbuf_nextrb_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_resbuf, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Resbuf_nextrb_get" "', argument " "1"" of type '" "struct _dwg_resbuf *""'"); 
    }
    arg1 = (struct _dwg_resbuf *)(argp1);
    result = (struct _dwg_resbuf *) ((arg1)->nextrb);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_resbuf, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Resbuf) {
  {
    int argvi = 0;
    struct _dwg_resbuf *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Resbuf();");
    }
    result = (struct _dwg_resbuf *)calloc(1, sizeof(struct _dwg_resbuf));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_resbuf, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Resbuf) {
  {
    struct _dwg_resbuf *arg1 = (struct _dwg_resbuf *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Resbuf(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_resbuf, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Resbuf" "', argument " "1"" of type '" "struct _dwg_resbuf *""'"); 
    }
    arg1 = (struct _dwg_resbuf *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_resbuf_value_pt_set) {
  {
    _dwg_resbuf_value *arg1 = (_dwg_resbuf_value *) 0 ;
    double *arg2 = (double *) (double *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_resbuf_value_pt_set(self,pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_resbuf_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_resbuf_value_pt_set" "', argument " "1"" of type '" "_dwg_resbuf_value *""'"); 
    }
    arg1 = (_dwg_resbuf_value *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_resbuf_value_pt_set" "', argument " "2"" of type '" "double [3]""'"); 
    } 
    arg2 = (double *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)3; ++ii) *(double *)&arg1->pt[ii] = *((double *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""pt""' of type '""double [3]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_resbuf_value_pt_get) {
  {
    _dwg_resbuf_value *arg1 = (_dwg_resbuf_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    double *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_resbuf_value_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_resbuf_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_resbuf_value_pt_get" "', argument " "1"" of type '" "_dwg_resbuf_value *""'"); 
    }
    arg1 = (_dwg_resbuf_value *)(argp1);
    result = (double *)(double *) ((arg1)->pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_resbuf_value_i8_set) {
  {
    _dwg_resbuf_value *arg1 = (_dwg_resbuf_value *) 0 ;
    char arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_resbuf_value_i8_set(self,i8);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_resbuf_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_resbuf_value_i8_set" "', argument " "1"" of type '" "_dwg_resbuf_value *""'"); 
    }
    arg1 = (_dwg_resbuf_value *)(argp1);
    ecode2 = SWIG_AsVal_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "_dwg_resbuf_value_i8_set" "', argument " "2"" of type '" "char""'");
    } 
    arg2 = (char)(val2);
    if (arg1) (arg1)->i8 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_resbuf_value_i8_get) {
  {
    _dwg_resbuf_value *arg1 = (_dwg_resbuf_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    char result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_resbuf_value_i8_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_resbuf_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_resbuf_value_i8_get" "', argument " "1"" of type '" "_dwg_resbuf_value *""'"); 
    }
    arg1 = (_dwg_resbuf_value *)(argp1);
    result = (char) ((arg1)->i8);
    ST(argvi) = SWIG_From_char  SWIG_PERL_CALL_ARGS_1((char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_resbuf_value_i16_set) {
  {
    _dwg_resbuf_value *arg1 = (_dwg_resbuf_value *) 0 ;
    short arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_resbuf_value_i16_set(self,i16);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_resbuf_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_resbuf_value_i16_set" "', argument " "1"" of type '" "_dwg_resbuf_value *""'"); 
    }
    arg1 = (_dwg_resbuf_value *)(argp1);
    ecode2 = SWIG_AsVal_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "_dwg_resbuf_value_i16_set" "', argument " "2"" of type '" "short""'");
    } 
    arg2 = (short)(val2);
    if (arg1) (arg1)->i16 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_resbuf_value_i16_get) {
  {
    _dwg_resbuf_value *arg1 = (_dwg_resbuf_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    short result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_resbuf_value_i16_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_resbuf_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_resbuf_value_i16_get" "', argument " "1"" of type '" "_dwg_resbuf_value *""'"); 
    }
    arg1 = (_dwg_resbuf_value *)(argp1);
    result = (short) ((arg1)->i16);
    ST(argvi) = SWIG_From_short  SWIG_PERL_CALL_ARGS_1((short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_resbuf_value_i32_set) {
  {
    _dwg_resbuf_value *arg1 = (_dwg_resbuf_value *) 0 ;
    int arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_resbuf_value_i32_set(self,i32);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_resbuf_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_resbuf_value_i32_set" "', argument " "1"" of type '" "_dwg_resbuf_value *""'"); 
    }
    arg1 = (_dwg_resbuf_value *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "_dwg_resbuf_value_i32_set" "', argument " "2"" of type '" "int""'");
    } 
    arg2 = (int)(val2);
    if (arg1) (arg1)->i32 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_resbuf_value_i32_get) {
  {
    _dwg_resbuf_value *arg1 = (_dwg_resbuf_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_resbuf_value_i32_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_resbuf_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_resbuf_value_i32_get" "', argument " "1"" of type '" "_dwg_resbuf_value *""'"); 
    }
    arg1 = (_dwg_resbuf_value *)(argp1);
    result = (int) ((arg1)->i32);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_resbuf_value_i64_set) {
  {
    _dwg_resbuf_value *arg1 = (_dwg_resbuf_value *) 0 ;
    BITCODE_BLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_resbuf_value_i64_set(self,i64);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_resbuf_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_resbuf_value_i64_set" "', argument " "1"" of type '" "_dwg_resbuf_value *""'"); 
    }
    arg1 = (_dwg_resbuf_value *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "_dwg_resbuf_value_i64_set" "', argument " "2"" of type '" "BITCODE_BLL""'");
    } 
    arg2 = (BITCODE_BLL)(val2);
    if (arg1) (arg1)->i64 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_resbuf_value_i64_get) {
  {
    _dwg_resbuf_value *arg1 = (_dwg_resbuf_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_resbuf_value_i64_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_resbuf_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_resbuf_value_i64_get" "', argument " "1"" of type '" "_dwg_resbuf_value *""'"); 
    }
    arg1 = (_dwg_resbuf_value *)(argp1);
    result = (BITCODE_BLL) ((arg1)->i64);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_resbuf_value_absref_set) {
  {
    _dwg_resbuf_value *arg1 = (_dwg_resbuf_value *) 0 ;
    BITCODE_HV arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_resbuf_value_absref_set(self,absref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_resbuf_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_resbuf_value_absref_set" "', argument " "1"" of type '" "_dwg_resbuf_value *""'"); 
    }
    arg1 = (_dwg_resbuf_value *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "_dwg_resbuf_value_absref_set" "', argument " "2"" of type '" "BITCODE_HV""'");
    } 
    arg2 = (BITCODE_HV)(val2);
    if (arg1) (arg1)->absref = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_resbuf_value_absref_get) {
  {
    _dwg_resbuf_value *arg1 = (_dwg_resbuf_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_HV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_resbuf_value_absref_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_resbuf_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_resbuf_value_absref_get" "', argument " "1"" of type '" "_dwg_resbuf_value *""'"); 
    }
    arg1 = (_dwg_resbuf_value *)(argp1);
    result = (BITCODE_HV) ((arg1)->absref);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_resbuf_value_dbl_set) {
  {
    _dwg_resbuf_value *arg1 = (_dwg_resbuf_value *) 0 ;
    double arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_resbuf_value_dbl_set(self,dbl);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_resbuf_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_resbuf_value_dbl_set" "', argument " "1"" of type '" "_dwg_resbuf_value *""'"); 
    }
    arg1 = (_dwg_resbuf_value *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "_dwg_resbuf_value_dbl_set" "', argument " "2"" of type '" "double""'");
    } 
    arg2 = (double)(val2);
    if (arg1) (arg1)->dbl = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_resbuf_value_dbl_get) {
  {
    _dwg_resbuf_value *arg1 = (_dwg_resbuf_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    double result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_resbuf_value_dbl_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_resbuf_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_resbuf_value_dbl_get" "', argument " "1"" of type '" "_dwg_resbuf_value *""'"); 
    }
    arg1 = (_dwg_resbuf_value *)(argp1);
    result = (double) ((arg1)->dbl);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_resbuf_value_str_set) {
  {
    _dwg_resbuf_value *arg1 = (_dwg_resbuf_value *) 0 ;
    struct _dwg_binary_chunk *arg2 = (struct _dwg_binary_chunk *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_resbuf_value_str_set(self,str);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_resbuf_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_resbuf_value_str_set" "', argument " "1"" of type '" "_dwg_resbuf_value *""'"); 
    }
    arg1 = (_dwg_resbuf_value *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_binary_chunk, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_resbuf_value_str_set" "', argument " "2"" of type '" "struct _dwg_binary_chunk *""'"); 
    }
    arg2 = (struct _dwg_binary_chunk *)(argp2);
    if (arg1) (arg1)->str = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_resbuf_value_str_get) {
  {
    _dwg_resbuf_value *arg1 = (_dwg_resbuf_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_binary_chunk *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_resbuf_value_str_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_resbuf_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_resbuf_value_str_get" "', argument " "1"" of type '" "_dwg_resbuf_value *""'"); 
    }
    arg1 = (_dwg_resbuf_value *)(argp1);
    result = (struct _dwg_binary_chunk *)& ((arg1)->str);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_binary_chunk, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new__dwg_resbuf_value) {
  {
    int argvi = 0;
    _dwg_resbuf_value *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new__dwg_resbuf_value();");
    }
    result = (_dwg_resbuf_value *)calloc(1, sizeof(_dwg_resbuf_value));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_resbuf_value, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete__dwg_resbuf_value) {
  {
    _dwg_resbuf_value *arg1 = (_dwg_resbuf_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete__dwg_resbuf_value(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_resbuf_value, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete__dwg_resbuf_value" "', argument " "1"" of type '" "_dwg_resbuf_value *""'"); 
    }
    arg1 = (_dwg_resbuf_value *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_size_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_size_set(self,size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_size_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_size_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_size_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_size_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RL) ((arg1)->size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_bitsize_hi_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_bitsize_hi_set(self,bitsize_hi);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_bitsize_hi_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_bitsize_hi_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->bitsize_hi = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_bitsize_hi_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_bitsize_hi_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_bitsize_hi_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RL) ((arg1)->bitsize_hi);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_bitsize_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_bitsize_set(self,bitsize);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_bitsize_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_bitsize_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->bitsize = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_bitsize_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_bitsize_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_bitsize_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RL) ((arg1)->bitsize);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_ACADMAINTVER_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_ACADMAINTVER_set(self,ACADMAINTVER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_ACADMAINTVER_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_ACADMAINTVER_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->ACADMAINTVER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_ACADMAINTVER_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_ACADMAINTVER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_ACADMAINTVER_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RC) ((arg1)->ACADMAINTVER);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_REQUIREDVERSIONS_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_REQUIREDVERSIONS_set(self,REQUIREDVERSIONS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_REQUIREDVERSIONS_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_REQUIREDVERSIONS_set" "', argument " "2"" of type '" "BITCODE_BLL""'");
    } 
    arg2 = (BITCODE_BLL)(val2);
    if (arg1) (arg1)->REQUIREDVERSIONS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_REQUIREDVERSIONS_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_REQUIREDVERSIONS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_REQUIREDVERSIONS_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BLL) ((arg1)->REQUIREDVERSIONS);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DWGCODEPAGE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DWGCODEPAGE_set(self,DWGCODEPAGE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DWGCODEPAGE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_DWGCODEPAGE_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->DWGCODEPAGE);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->DWGCODEPAGE = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->DWGCODEPAGE = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DWGCODEPAGE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DWGCODEPAGE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DWGCODEPAGE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_TV) ((arg1)->DWGCODEPAGE);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_codepage_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_codepage_set(self,codepage);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_codepage_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_codepage_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->codepage = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_codepage_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_codepage_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_codepage_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RS) ((arg1)->codepage);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unit1_ratio_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unit1_ratio_set(self,unit1_ratio);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unit1_ratio_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unit1_ratio_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->unit1_ratio = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unit1_ratio_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unit1_ratio_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unit1_ratio_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->unit1_ratio);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unit2_ratio_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unit2_ratio_set(self,unit2_ratio);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unit2_ratio_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unit2_ratio_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->unit2_ratio = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unit2_ratio_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unit2_ratio_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unit2_ratio_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->unit2_ratio);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unit3_ratio_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unit3_ratio_set(self,unit3_ratio);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unit3_ratio_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unit3_ratio_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->unit3_ratio = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unit3_ratio_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unit3_ratio_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unit3_ratio_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->unit3_ratio);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unit4_ratio_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unit4_ratio_set(self,unit4_ratio);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unit4_ratio_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unit4_ratio_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->unit4_ratio = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unit4_ratio_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unit4_ratio_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unit4_ratio_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->unit4_ratio);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unit1_name_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unit1_name_set(self,unit1_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unit1_name_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_unit1_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->unit1_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->unit1_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->unit1_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unit1_name_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unit1_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unit1_name_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_TV) ((arg1)->unit1_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unit2_name_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unit2_name_set(self,unit2_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unit2_name_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_unit2_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->unit2_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->unit2_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->unit2_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unit2_name_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unit2_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unit2_name_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_TV) ((arg1)->unit2_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unit3_name_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unit3_name_set(self,unit3_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unit3_name_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_unit3_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->unit3_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->unit3_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->unit3_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unit3_name_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unit3_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unit3_name_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_TV) ((arg1)->unit3_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unit4_name_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unit4_name_set(self,unit4_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unit4_name_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_unit4_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->unit4_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->unit4_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->unit4_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unit4_name_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unit4_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unit4_name_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_TV) ((arg1)->unit4_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_8_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_8_set(self,unknown_8);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_8_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_8_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_8 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_8_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_8_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_8_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_8);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_9_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_9_set(self,unknown_9);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_9_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_9_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_9 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_9_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_9_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_9_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_9);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_10_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_10_set(self,unknown_10);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_10_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_10_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->unknown_10 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_10_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_10_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_10_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->unknown_10);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_VX_TABLE_RECORD_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_VX_TABLE_RECORD_set(self,VX_TABLE_RECORD);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_VX_TABLE_RECORD_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_VX_TABLE_RECORD_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->VX_TABLE_RECORD = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_VX_TABLE_RECORD_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_VX_TABLE_RECORD_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_VX_TABLE_RECORD_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->VX_TABLE_RECORD);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMASO_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMASO_set(self,DIMASO);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMASO_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMASO_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DIMASO = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMASO_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMASO_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMASO_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->DIMASO);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMSHO_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMSHO_set(self,DIMSHO);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMSHO_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMSHO_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DIMSHO = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMSHO_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMSHO_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMSHO_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->DIMSHO);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMSAV_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMSAV_set(self,DIMSAV);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMSAV_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMSAV_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DIMSAV = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMSAV_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMSAV_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMSAV_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->DIMSAV);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PLINEGEN_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PLINEGEN_set(self,PLINEGEN);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PLINEGEN_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_PLINEGEN_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->PLINEGEN = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PLINEGEN_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PLINEGEN_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PLINEGEN_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->PLINEGEN);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_ORTHOMODE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_ORTHOMODE_set(self,ORTHOMODE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_ORTHOMODE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_ORTHOMODE_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->ORTHOMODE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_ORTHOMODE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_ORTHOMODE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_ORTHOMODE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->ORTHOMODE);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_REGENMODE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_REGENMODE_set(self,REGENMODE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_REGENMODE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_REGENMODE_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->REGENMODE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_REGENMODE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_REGENMODE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_REGENMODE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->REGENMODE);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_FILLMODE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_FILLMODE_set(self,FILLMODE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_FILLMODE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_FILLMODE_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->FILLMODE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_FILLMODE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_FILLMODE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_FILLMODE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->FILLMODE);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_QTEXTMODE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_QTEXTMODE_set(self,QTEXTMODE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_QTEXTMODE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_QTEXTMODE_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->QTEXTMODE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_QTEXTMODE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_QTEXTMODE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_QTEXTMODE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->QTEXTMODE);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PSLTSCALE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PSLTSCALE_set(self,PSLTSCALE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PSLTSCALE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_PSLTSCALE_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->PSLTSCALE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PSLTSCALE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PSLTSCALE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PSLTSCALE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->PSLTSCALE);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LIMCHECK_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LIMCHECK_set(self,LIMCHECK);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LIMCHECK_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_LIMCHECK_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->LIMCHECK = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LIMCHECK_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LIMCHECK_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LIMCHECK_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->LIMCHECK);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_MENUEXT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) (BITCODE_RC *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_MENUEXT_set(self,MENUEXT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_MENUEXT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_MENUEXT_set" "', argument " "2"" of type '" "BITCODE_RC [46]""'"); 
    } 
    arg2 = (BITCODE_RC *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)46; ++ii) *(BITCODE_RC *)&arg1->MENUEXT[ii] = *((BITCODE_RC *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""MENUEXT""' of type '""BITCODE_RC [46]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_MENUEXT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_MENUEXT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_MENUEXT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RC *)(BITCODE_RC *) ((arg1)->MENUEXT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_BLIPMODE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_BLIPMODE_set(self,BLIPMODE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_BLIPMODE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_BLIPMODE_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->BLIPMODE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_BLIPMODE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_BLIPMODE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_BLIPMODE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->BLIPMODE);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_11_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_11_set(self,unknown_11);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_11_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_11_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown_11 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_11_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_11_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_11_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown_11);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_USRTIMER_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_USRTIMER_set(self,USRTIMER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_USRTIMER_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_USRTIMER_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->USRTIMER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_USRTIMER_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_USRTIMER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_USRTIMER_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->USRTIMER);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_FASTZOOM_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_FASTZOOM_set(self,FASTZOOM);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_FASTZOOM_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_FASTZOOM_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->FASTZOOM = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_FASTZOOM_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_FASTZOOM_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_FASTZOOM_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->FASTZOOM);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_FLATLAND_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_FLATLAND_set(self,FLATLAND);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_FLATLAND_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_FLATLAND_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->FLATLAND = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_FLATLAND_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_FLATLAND_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_FLATLAND_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->FLATLAND);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_VIEWMODE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_VIEWMODE_set(self,VIEWMODE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_VIEWMODE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_VIEWMODE_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->VIEWMODE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_VIEWMODE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_VIEWMODE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_VIEWMODE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->VIEWMODE);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SKPOLY_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SKPOLY_set(self,SKPOLY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SKPOLY_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_SKPOLY_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->SKPOLY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SKPOLY_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SKPOLY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SKPOLY_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->SKPOLY);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_mon_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_mon_set(self,unknown_mon);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_mon_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_mon_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->unknown_mon = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_mon_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_mon_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_mon_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RS) ((arg1)->unknown_mon);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_day_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_day_set(self,unknown_day);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_day_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_day_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->unknown_day = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_day_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_day_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_day_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RS) ((arg1)->unknown_day);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_year_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_year_set(self,unknown_year);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_year_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_year_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->unknown_year = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_year_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_year_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_year_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RS) ((arg1)->unknown_year);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_hour_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_hour_set(self,unknown_hour);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_hour_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_hour_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->unknown_hour = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_hour_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_hour_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_hour_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RS) ((arg1)->unknown_hour);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_min_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_min_set(self,unknown_min);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_min_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_min_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->unknown_min = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_min_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_min_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_min_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RS) ((arg1)->unknown_min);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_sec_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_sec_set(self,unknown_sec);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_sec_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_sec_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->unknown_sec = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_sec_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_sec_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_sec_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RS) ((arg1)->unknown_sec);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_ms_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_ms_set(self,unknown_ms);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_ms_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_ms_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->unknown_ms = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_ms_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_ms_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_ms_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RS) ((arg1)->unknown_ms);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_ANGDIR_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_ANGDIR_set(self,ANGDIR);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_ANGDIR_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_ANGDIR_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->ANGDIR = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_ANGDIR_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_ANGDIR_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_ANGDIR_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->ANGDIR);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SPLFRAME_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SPLFRAME_set(self,SPLFRAME);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SPLFRAME_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_SPLFRAME_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->SPLFRAME = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SPLFRAME_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SPLFRAME_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SPLFRAME_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->SPLFRAME);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_ATTREQ_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_ATTREQ_set(self,ATTREQ);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_ATTREQ_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_ATTREQ_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->ATTREQ = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_ATTREQ_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_ATTREQ_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_ATTREQ_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->ATTREQ);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_ATTDIA_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_ATTDIA_set(self,ATTDIA);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_ATTDIA_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_ATTDIA_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->ATTDIA = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_ATTDIA_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_ATTDIA_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_ATTDIA_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->ATTDIA);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_MIRRTEXT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_MIRRTEXT_set(self,MIRRTEXT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_MIRRTEXT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_MIRRTEXT_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->MIRRTEXT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_MIRRTEXT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_MIRRTEXT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_MIRRTEXT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->MIRRTEXT);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_WORLDVIEW_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_WORLDVIEW_set(self,WORLDVIEW);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_WORLDVIEW_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_WORLDVIEW_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->WORLDVIEW = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_WORLDVIEW_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_WORLDVIEW_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_WORLDVIEW_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->WORLDVIEW);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_WIREFRAME_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_WIREFRAME_set(self,WIREFRAME);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_WIREFRAME_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_WIREFRAME_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->WIREFRAME = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_WIREFRAME_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_WIREFRAME_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_WIREFRAME_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->WIREFRAME);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_TILEMODE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_TILEMODE_set(self,TILEMODE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_TILEMODE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_TILEMODE_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->TILEMODE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_TILEMODE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_TILEMODE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_TILEMODE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->TILEMODE);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PLIMCHECK_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PLIMCHECK_set(self,PLIMCHECK);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PLIMCHECK_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_PLIMCHECK_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->PLIMCHECK = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PLIMCHECK_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PLIMCHECK_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PLIMCHECK_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->PLIMCHECK);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_VISRETAIN_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_VISRETAIN_set(self,VISRETAIN);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_VISRETAIN_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_VISRETAIN_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->VISRETAIN = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_VISRETAIN_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_VISRETAIN_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_VISRETAIN_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->VISRETAIN);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DELOBJ_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DELOBJ_set(self,DELOBJ);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DELOBJ_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DELOBJ_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DELOBJ = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DELOBJ_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DELOBJ_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DELOBJ_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->DELOBJ);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DISPSILH_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DISPSILH_set(self,DISPSILH);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DISPSILH_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DISPSILH_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DISPSILH = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DISPSILH_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DISPSILH_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DISPSILH_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->DISPSILH);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PELLIPSE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PELLIPSE_set(self,PELLIPSE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PELLIPSE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_PELLIPSE_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->PELLIPSE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PELLIPSE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PELLIPSE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PELLIPSE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->PELLIPSE);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SAVEIMAGES_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SAVEIMAGES_set(self,SAVEIMAGES);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SAVEIMAGES_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_SAVEIMAGES_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->SAVEIMAGES = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SAVEIMAGES_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SAVEIMAGES_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SAVEIMAGES_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->SAVEIMAGES);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PROXYGRAPHICS_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PROXYGRAPHICS_set(self,PROXYGRAPHICS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PROXYGRAPHICS_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_PROXYGRAPHICS_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->PROXYGRAPHICS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PROXYGRAPHICS_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PROXYGRAPHICS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PROXYGRAPHICS_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->PROXYGRAPHICS);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_MEASUREMENT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_MEASUREMENT_set(self,MEASUREMENT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_MEASUREMENT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_MEASUREMENT_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->MEASUREMENT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_MEASUREMENT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_MEASUREMENT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_MEASUREMENT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->MEASUREMENT);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DRAGMODE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DRAGMODE_set(self,DRAGMODE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DRAGMODE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DRAGMODE_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DRAGMODE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DRAGMODE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DRAGMODE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DRAGMODE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->DRAGMODE);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_TREEDEPTH_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BSd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_TREEDEPTH_set(self,TREEDEPTH);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_TREEDEPTH_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_TREEDEPTH_set" "', argument " "2"" of type '" "BITCODE_BSd""'");
    } 
    arg2 = (BITCODE_BSd)(val2);
    if (arg1) (arg1)->TREEDEPTH = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_TREEDEPTH_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BSd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_TREEDEPTH_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_TREEDEPTH_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BSd) ((arg1)->TREEDEPTH);
    ST(argvi) = SWIG_From_short  SWIG_PERL_CALL_ARGS_1((short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LUNITS_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LUNITS_set(self,LUNITS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LUNITS_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_LUNITS_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->LUNITS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LUNITS_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LUNITS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LUNITS_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->LUNITS);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LUPREC_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LUPREC_set(self,LUPREC);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LUPREC_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_LUPREC_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->LUPREC = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LUPREC_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LUPREC_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LUPREC_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->LUPREC);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_AUNITS_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_AUNITS_set(self,AUNITS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_AUNITS_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_AUNITS_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->AUNITS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_AUNITS_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_AUNITS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_AUNITS_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->AUNITS);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_AUPREC_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_AUPREC_set(self,AUPREC);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_AUPREC_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_AUPREC_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->AUPREC = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_AUPREC_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_AUPREC_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_AUPREC_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->AUPREC);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_ATTMODE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_ATTMODE_set(self,ATTMODE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_ATTMODE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_ATTMODE_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->ATTMODE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_ATTMODE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_ATTMODE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_ATTMODE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->ATTMODE);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_COORDS_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_COORDS_set(self,COORDS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_COORDS_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_COORDS_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->COORDS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_COORDS_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_COORDS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_COORDS_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->COORDS);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PDMODE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PDMODE_set(self,PDMODE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PDMODE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_PDMODE_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->PDMODE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PDMODE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PDMODE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PDMODE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->PDMODE);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PICKSTYLE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PICKSTYLE_set(self,PICKSTYLE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PICKSTYLE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_PICKSTYLE_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->PICKSTYLE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PICKSTYLE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PICKSTYLE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PICKSTYLE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->PICKSTYLE);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_OSMODE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_OSMODE_set(self,OSMODE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_OSMODE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_OSMODE_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->OSMODE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_OSMODE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_OSMODE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_OSMODE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->OSMODE);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_12_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_12_set(self,unknown_12);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_12_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_12_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_12 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_12_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_12_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_12_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_12);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_13_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_13_set(self,unknown_13);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_13_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_13_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_13 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_13_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_13_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_13_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_13);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_14_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_14_set(self,unknown_14);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_14_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_14_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_14 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_14_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_14_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_14_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_14);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_USERI1_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BSd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_USERI1_set(self,USERI1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_USERI1_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_USERI1_set" "', argument " "2"" of type '" "BITCODE_BSd""'");
    } 
    arg2 = (BITCODE_BSd)(val2);
    if (arg1) (arg1)->USERI1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_USERI1_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BSd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_USERI1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_USERI1_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BSd) ((arg1)->USERI1);
    ST(argvi) = SWIG_From_short  SWIG_PERL_CALL_ARGS_1((short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_USERI2_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BSd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_USERI2_set(self,USERI2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_USERI2_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_USERI2_set" "', argument " "2"" of type '" "BITCODE_BSd""'");
    } 
    arg2 = (BITCODE_BSd)(val2);
    if (arg1) (arg1)->USERI2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_USERI2_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BSd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_USERI2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_USERI2_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BSd) ((arg1)->USERI2);
    ST(argvi) = SWIG_From_short  SWIG_PERL_CALL_ARGS_1((short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_USERI3_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BSd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_USERI3_set(self,USERI3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_USERI3_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_USERI3_set" "', argument " "2"" of type '" "BITCODE_BSd""'");
    } 
    arg2 = (BITCODE_BSd)(val2);
    if (arg1) (arg1)->USERI3 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_USERI3_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BSd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_USERI3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_USERI3_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BSd) ((arg1)->USERI3);
    ST(argvi) = SWIG_From_short  SWIG_PERL_CALL_ARGS_1((short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_USERI4_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BSd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_USERI4_set(self,USERI4);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_USERI4_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_USERI4_set" "', argument " "2"" of type '" "BITCODE_BSd""'");
    } 
    arg2 = (BITCODE_BSd)(val2);
    if (arg1) (arg1)->USERI4 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_USERI4_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BSd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_USERI4_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_USERI4_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BSd) ((arg1)->USERI4);
    ST(argvi) = SWIG_From_short  SWIG_PERL_CALL_ARGS_1((short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_USERI5_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BSd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_USERI5_set(self,USERI5);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_USERI5_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_USERI5_set" "', argument " "2"" of type '" "BITCODE_BSd""'");
    } 
    arg2 = (BITCODE_BSd)(val2);
    if (arg1) (arg1)->USERI5 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_USERI5_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BSd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_USERI5_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_USERI5_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BSd) ((arg1)->USERI5);
    ST(argvi) = SWIG_From_short  SWIG_PERL_CALL_ARGS_1((short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SPLINESEGS_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SPLINESEGS_set(self,SPLINESEGS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SPLINESEGS_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_SPLINESEGS_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->SPLINESEGS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SPLINESEGS_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SPLINESEGS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SPLINESEGS_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->SPLINESEGS);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SURFU_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SURFU_set(self,SURFU);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SURFU_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_SURFU_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->SURFU = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SURFU_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SURFU_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SURFU_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->SURFU);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SURFV_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SURFV_set(self,SURFV);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SURFV_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_SURFV_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->SURFV = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SURFV_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SURFV_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SURFV_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->SURFV);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SURFTYPE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SURFTYPE_set(self,SURFTYPE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SURFTYPE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_SURFTYPE_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->SURFTYPE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SURFTYPE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SURFTYPE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SURFTYPE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->SURFTYPE);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SURFTAB1_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SURFTAB1_set(self,SURFTAB1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SURFTAB1_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_SURFTAB1_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->SURFTAB1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SURFTAB1_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SURFTAB1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SURFTAB1_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->SURFTAB1);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SURFTAB2_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SURFTAB2_set(self,SURFTAB2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SURFTAB2_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_SURFTAB2_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->SURFTAB2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SURFTAB2_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SURFTAB2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SURFTAB2_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->SURFTAB2);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SPLINETYPE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SPLINETYPE_set(self,SPLINETYPE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SPLINETYPE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_SPLINETYPE_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->SPLINETYPE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SPLINETYPE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SPLINETYPE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SPLINETYPE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->SPLINETYPE);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SHADEDGE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SHADEDGE_set(self,SHADEDGE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SHADEDGE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_SHADEDGE_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->SHADEDGE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SHADEDGE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SHADEDGE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SHADEDGE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->SHADEDGE);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SHADEDIF_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SHADEDIF_set(self,SHADEDIF);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SHADEDIF_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_SHADEDIF_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->SHADEDIF = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SHADEDIF_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SHADEDIF_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SHADEDIF_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->SHADEDIF);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_UNITMODE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_UNITMODE_set(self,UNITMODE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_UNITMODE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_UNITMODE_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->UNITMODE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_UNITMODE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_UNITMODE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_UNITMODE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->UNITMODE);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_MAXACTVP_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_MAXACTVP_set(self,MAXACTVP);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_MAXACTVP_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_MAXACTVP_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->MAXACTVP = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_MAXACTVP_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_MAXACTVP_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_MAXACTVP_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->MAXACTVP);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_ISOLINES_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_ISOLINES_set(self,ISOLINES);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_ISOLINES_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_ISOLINES_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->ISOLINES = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_ISOLINES_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_ISOLINES_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_ISOLINES_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->ISOLINES);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CMLJUST_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CMLJUST_set(self,CMLJUST);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CMLJUST_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_CMLJUST_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->CMLJUST = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CMLJUST_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CMLJUST_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CMLJUST_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->CMLJUST);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_TEXTQLTY_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_TEXTQLTY_set(self,TEXTQLTY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_TEXTQLTY_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_TEXTQLTY_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->TEXTQLTY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_TEXTQLTY_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_TEXTQLTY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_TEXTQLTY_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->TEXTQLTY);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_14b_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_14b_set(self,unknown_14b);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_14b_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_14b_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_14b = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_14b_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_14b_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_14b_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_14b);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LTSCALE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LTSCALE_set(self,LTSCALE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LTSCALE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_LTSCALE_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->LTSCALE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LTSCALE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LTSCALE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LTSCALE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->LTSCALE);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_TEXTSIZE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_TEXTSIZE_set(self,TEXTSIZE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_TEXTSIZE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_TEXTSIZE_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->TEXTSIZE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_TEXTSIZE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_TEXTSIZE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_TEXTSIZE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->TEXTSIZE);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_TRACEWID_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_TRACEWID_set(self,TRACEWID);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_TRACEWID_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_TRACEWID_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->TRACEWID = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_TRACEWID_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_TRACEWID_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_TRACEWID_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->TRACEWID);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SKETCHINC_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SKETCHINC_set(self,SKETCHINC);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SKETCHINC_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_SKETCHINC_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->SKETCHINC = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SKETCHINC_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SKETCHINC_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SKETCHINC_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->SKETCHINC);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_FILLETRAD_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_FILLETRAD_set(self,FILLETRAD);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_FILLETRAD_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_FILLETRAD_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->FILLETRAD = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_FILLETRAD_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_FILLETRAD_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_FILLETRAD_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->FILLETRAD);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_THICKNESS_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_THICKNESS_set(self,THICKNESS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_THICKNESS_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_THICKNESS_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->THICKNESS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_THICKNESS_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_THICKNESS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_THICKNESS_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->THICKNESS);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_ANGBASE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_ANGBASE_set(self,ANGBASE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_ANGBASE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_ANGBASE_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->ANGBASE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_ANGBASE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_ANGBASE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_ANGBASE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->ANGBASE);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PDSIZE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PDSIZE_set(self,PDSIZE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PDSIZE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_PDSIZE_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->PDSIZE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PDSIZE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PDSIZE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PDSIZE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->PDSIZE);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PLINEWID_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PLINEWID_set(self,PLINEWID);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PLINEWID_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_PLINEWID_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->PLINEWID = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PLINEWID_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PLINEWID_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PLINEWID_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->PLINEWID);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_USERR1_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_USERR1_set(self,USERR1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_USERR1_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_USERR1_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->USERR1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_USERR1_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_USERR1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_USERR1_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->USERR1);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_USERR2_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_USERR2_set(self,USERR2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_USERR2_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_USERR2_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->USERR2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_USERR2_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_USERR2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_USERR2_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->USERR2);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_USERR3_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_USERR3_set(self,USERR3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_USERR3_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_USERR3_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->USERR3 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_USERR3_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_USERR3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_USERR3_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->USERR3);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_USERR4_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_USERR4_set(self,USERR4);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_USERR4_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_USERR4_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->USERR4 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_USERR4_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_USERR4_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_USERR4_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->USERR4);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_USERR5_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_USERR5_set(self,USERR5);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_USERR5_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_USERR5_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->USERR5 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_USERR5_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_USERR5_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_USERR5_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->USERR5);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CHAMFERA_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CHAMFERA_set(self,CHAMFERA);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CHAMFERA_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_CHAMFERA_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->CHAMFERA = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CHAMFERA_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CHAMFERA_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CHAMFERA_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->CHAMFERA);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CHAMFERB_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CHAMFERB_set(self,CHAMFERB);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CHAMFERB_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_CHAMFERB_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->CHAMFERB = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CHAMFERB_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CHAMFERB_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CHAMFERB_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->CHAMFERB);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CHAMFERC_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CHAMFERC_set(self,CHAMFERC);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CHAMFERC_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_CHAMFERC_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->CHAMFERC = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CHAMFERC_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CHAMFERC_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CHAMFERC_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->CHAMFERC);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CHAMFERD_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CHAMFERD_set(self,CHAMFERD);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CHAMFERD_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_CHAMFERD_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->CHAMFERD = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CHAMFERD_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CHAMFERD_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CHAMFERD_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->CHAMFERD);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_FACETRES_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_FACETRES_set(self,FACETRES);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_FACETRES_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_FACETRES_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->FACETRES = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_FACETRES_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_FACETRES_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_FACETRES_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->FACETRES);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CMLSCALE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CMLSCALE_set(self,CMLSCALE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CMLSCALE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_CMLSCALE_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->CMLSCALE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CMLSCALE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CMLSCALE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CMLSCALE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->CMLSCALE);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CELTSCALE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CELTSCALE_set(self,CELTSCALE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CELTSCALE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_CELTSCALE_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->CELTSCALE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CELTSCALE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CELTSCALE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CELTSCALE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->CELTSCALE);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_VIEWTWIST_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_VIEWTWIST_set(self,VIEWTWIST);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_VIEWTWIST_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_VIEWTWIST_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->VIEWTWIST = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_VIEWTWIST_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_VIEWTWIST_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_VIEWTWIST_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->VIEWTWIST);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_MENU_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_MENU_set(self,MENU);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_MENU_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_MENU_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->MENU);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->MENU = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->MENU = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_MENU_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_MENU_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_MENU_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_TV) ((arg1)->MENU);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_TDCREATE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_TIMEBLL *arg2 = (BITCODE_TIMEBLL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_TDCREATE_set(self,TDCREATE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_TDCREATE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_time_bll, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_TDCREATE_set" "', argument " "2"" of type '" "BITCODE_TIMEBLL *""'"); 
    }
    arg2 = (BITCODE_TIMEBLL *)(argp2);
    if (arg1) (arg1)->TDCREATE = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_TDCREATE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TIMEBLL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_TDCREATE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_TDCREATE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_TIMEBLL *)& ((arg1)->TDCREATE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_time_bll, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_TDUPDATE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_TIMEBLL *arg2 = (BITCODE_TIMEBLL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_TDUPDATE_set(self,TDUPDATE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_TDUPDATE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_time_bll, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_TDUPDATE_set" "', argument " "2"" of type '" "BITCODE_TIMEBLL *""'"); 
    }
    arg2 = (BITCODE_TIMEBLL *)(argp2);
    if (arg1) (arg1)->TDUPDATE = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_TDUPDATE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TIMEBLL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_TDUPDATE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_TDUPDATE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_TIMEBLL *)& ((arg1)->TDUPDATE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_time_bll, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_TDUCREATE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_TIMEBLL *arg2 = (BITCODE_TIMEBLL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_TDUCREATE_set(self,TDUCREATE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_TDUCREATE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_time_bll, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_TDUCREATE_set" "', argument " "2"" of type '" "BITCODE_TIMEBLL *""'"); 
    }
    arg2 = (BITCODE_TIMEBLL *)(argp2);
    if (arg1) (arg1)->TDUCREATE = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_TDUCREATE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TIMEBLL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_TDUCREATE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_TDUCREATE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_TIMEBLL *)& ((arg1)->TDUCREATE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_time_bll, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_TDUUPDATE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_TIMEBLL *arg2 = (BITCODE_TIMEBLL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_TDUUPDATE_set(self,TDUUPDATE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_TDUUPDATE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_time_bll, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_TDUUPDATE_set" "', argument " "2"" of type '" "BITCODE_TIMEBLL *""'"); 
    }
    arg2 = (BITCODE_TIMEBLL *)(argp2);
    if (arg1) (arg1)->TDUUPDATE = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_TDUUPDATE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TIMEBLL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_TDUUPDATE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_TDUUPDATE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_TIMEBLL *)& ((arg1)->TDUUPDATE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_time_bll, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_15_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_15_set(self,unknown_15);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_15_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_15_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_15 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_15_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_15_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_15_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_15);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_16_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_16_set(self,unknown_16);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_16_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_16_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_16 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_16_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_16_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_16_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_16);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_17_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_17_set(self,unknown_17);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_17_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_17_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_17 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_17_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_17_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_17_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_17);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_TDINDWG_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_TIMEBLL *arg2 = (BITCODE_TIMEBLL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_TDINDWG_set(self,TDINDWG);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_TDINDWG_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_time_bll, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_TDINDWG_set" "', argument " "2"" of type '" "BITCODE_TIMEBLL *""'"); 
    }
    arg2 = (BITCODE_TIMEBLL *)(argp2);
    if (arg1) (arg1)->TDINDWG = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_TDINDWG_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TIMEBLL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_TDINDWG_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_TDINDWG_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_TIMEBLL *)& ((arg1)->TDINDWG);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_time_bll, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_TDUSRTIMER_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_TIMEBLL *arg2 = (BITCODE_TIMEBLL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_TDUSRTIMER_set(self,TDUSRTIMER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_TDUSRTIMER_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_time_bll, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_TDUSRTIMER_set" "', argument " "2"" of type '" "BITCODE_TIMEBLL *""'"); 
    }
    arg2 = (BITCODE_TIMEBLL *)(argp2);
    if (arg1) (arg1)->TDUSRTIMER = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_TDUSRTIMER_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TIMEBLL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_TDUSRTIMER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_TDUSRTIMER_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_TIMEBLL *)& ((arg1)->TDUSRTIMER);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_time_bll, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CECOLOR_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CECOLOR_set(self,CECOLOR);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CECOLOR_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_CECOLOR_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->CECOLOR = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CECOLOR_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CECOLOR_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CECOLOR_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->CECOLOR);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_HANDLING_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_HANDLING_set(self,HANDLING);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_HANDLING_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_HANDLING_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->HANDLING = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_HANDLING_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_HANDLING_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_HANDLING_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->HANDLING);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_HANDSEED_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_HANDSEED_set(self,HANDSEED);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_HANDSEED_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_HANDSEED_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->HANDSEED = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_HANDSEED_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_HANDSEED_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_HANDSEED_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->HANDSEED);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_4f2_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_4f2_set(self,unknown_4f2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_4f2_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_4f2_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->unknown_4f2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_4f2_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_4f2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_4f2_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RL) ((arg1)->unknown_4f2);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_5_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_5_set(self,unknown_5);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_5_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_5_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->unknown_5 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_5_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_5_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_5_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RS) ((arg1)->unknown_5);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_6_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_6_set(self,unknown_6);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_6_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_6_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->unknown_6 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_6_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_6_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_6_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RS) ((arg1)->unknown_6);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_6a_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_6a_set(self,unknown_6a);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_6a_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_6a_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->unknown_6a = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_6a_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_6a_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_6a_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RS) ((arg1)->unknown_6a);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_6b_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_6b_set(self,unknown_6b);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_6b_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_6b_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->unknown_6b = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_6b_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_6b_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_6b_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RS) ((arg1)->unknown_6b);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_6c_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_6c_set(self,unknown_6c);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_6c_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_6c_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->unknown_6c = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_6c_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_6c_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_6c_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RS) ((arg1)->unknown_6c);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CLAYER_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CLAYER_set(self,CLAYER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CLAYER_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_CLAYER_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->CLAYER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CLAYER_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CLAYER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CLAYER_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->CLAYER);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_TEXTSTYLE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_TEXTSTYLE_set(self,TEXTSTYLE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_TEXTSTYLE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_TEXTSTYLE_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->TEXTSTYLE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_TEXTSTYLE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_TEXTSTYLE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_TEXTSTYLE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->TEXTSTYLE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CELTYPE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CELTYPE_set(self,CELTYPE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CELTYPE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_CELTYPE_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->CELTYPE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CELTYPE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CELTYPE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CELTYPE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->CELTYPE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CMATERIAL_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CMATERIAL_set(self,CMATERIAL);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CMATERIAL_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_CMATERIAL_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->CMATERIAL = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CMATERIAL_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CMATERIAL_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CMATERIAL_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->CMATERIAL);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMSTYLE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMSTYLE_set(self,DIMSTYLE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMSTYLE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_DIMSTYLE_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->DIMSTYLE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMSTYLE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMSTYLE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMSTYLE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->DIMSTYLE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CMLSTYLE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CMLSTYLE_set(self,CMLSTYLE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CMLSTYLE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_CMLSTYLE_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->CMLSTYLE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CMLSTYLE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CMLSTYLE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CMLSTYLE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->CMLSTYLE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PSVPSCALE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PSVPSCALE_set(self,PSVPSCALE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PSVPSCALE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_PSVPSCALE_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->PSVPSCALE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PSVPSCALE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PSVPSCALE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PSVPSCALE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->PSVPSCALE);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PINSBASE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PINSBASE_set(self,PINSBASE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PINSBASE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_PINSBASE_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->PINSBASE = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PINSBASE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PINSBASE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PINSBASE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->PINSBASE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PEXTMIN_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PEXTMIN_set(self,PEXTMIN);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PEXTMIN_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_PEXTMIN_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->PEXTMIN = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PEXTMIN_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PEXTMIN_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PEXTMIN_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->PEXTMIN);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PEXTMAX_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PEXTMAX_set(self,PEXTMAX);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PEXTMAX_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_PEXTMAX_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->PEXTMAX = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PEXTMAX_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PEXTMAX_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PEXTMAX_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->PEXTMAX);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PLIMMIN_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_2DPOINT *arg2 = (BITCODE_2DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PLIMMIN_set(self,PLIMMIN);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PLIMMIN_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_PLIMMIN_set" "', argument " "2"" of type '" "BITCODE_2DPOINT *""'"); 
    }
    arg2 = (BITCODE_2DPOINT *)(argp2);
    if (arg1) (arg1)->PLIMMIN = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PLIMMIN_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PLIMMIN_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PLIMMIN_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_2DPOINT *)& ((arg1)->PLIMMIN);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PLIMMAX_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_2DPOINT *arg2 = (BITCODE_2DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PLIMMAX_set(self,PLIMMAX);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PLIMMAX_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_PLIMMAX_set" "', argument " "2"" of type '" "BITCODE_2DPOINT *""'"); 
    }
    arg2 = (BITCODE_2DPOINT *)(argp2);
    if (arg1) (arg1)->PLIMMAX = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PLIMMAX_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PLIMMAX_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PLIMMAX_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_2DPOINT *)& ((arg1)->PLIMMAX);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PELEVATION_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PELEVATION_set(self,PELEVATION);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PELEVATION_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_PELEVATION_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->PELEVATION = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PELEVATION_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PELEVATION_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PELEVATION_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->PELEVATION);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PUCSORG_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PUCSORG_set(self,PUCSORG);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PUCSORG_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_PUCSORG_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->PUCSORG = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PUCSORG_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PUCSORG_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PUCSORG_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->PUCSORG);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PUCSXDIR_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PUCSXDIR_set(self,PUCSXDIR);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PUCSXDIR_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_PUCSXDIR_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->PUCSXDIR = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PUCSXDIR_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PUCSXDIR_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PUCSXDIR_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->PUCSXDIR);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PUCSYDIR_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PUCSYDIR_set(self,PUCSYDIR);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PUCSYDIR_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_PUCSYDIR_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->PUCSYDIR = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PUCSYDIR_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PUCSYDIR_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PUCSYDIR_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->PUCSYDIR);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PUCSNAME_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PUCSNAME_set(self,PUCSNAME);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PUCSNAME_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_PUCSNAME_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->PUCSNAME = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PUCSNAME_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PUCSNAME_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PUCSNAME_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->PUCSNAME);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PUCSBASE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PUCSBASE_set(self,PUCSBASE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PUCSBASE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_PUCSBASE_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->PUCSBASE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PUCSBASE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PUCSBASE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PUCSBASE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->PUCSBASE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PUCSORTHOREF_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PUCSORTHOREF_set(self,PUCSORTHOREF);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PUCSORTHOREF_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_PUCSORTHOREF_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->PUCSORTHOREF = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PUCSORTHOREF_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PUCSORTHOREF_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PUCSORTHOREF_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->PUCSORTHOREF);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PUCSORTHOVIEW_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PUCSORTHOVIEW_set(self,PUCSORTHOVIEW);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PUCSORTHOVIEW_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_PUCSORTHOVIEW_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->PUCSORTHOVIEW = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PUCSORTHOVIEW_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PUCSORTHOVIEW_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PUCSORTHOVIEW_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->PUCSORTHOVIEW);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PUCSORGTOP_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PUCSORGTOP_set(self,PUCSORGTOP);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PUCSORGTOP_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_PUCSORGTOP_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->PUCSORGTOP = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PUCSORGTOP_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PUCSORGTOP_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PUCSORGTOP_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->PUCSORGTOP);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PUCSORGBOTTOM_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PUCSORGBOTTOM_set(self,PUCSORGBOTTOM);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PUCSORGBOTTOM_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_PUCSORGBOTTOM_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->PUCSORGBOTTOM = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PUCSORGBOTTOM_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PUCSORGBOTTOM_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PUCSORGBOTTOM_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->PUCSORGBOTTOM);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PUCSORGLEFT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PUCSORGLEFT_set(self,PUCSORGLEFT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PUCSORGLEFT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_PUCSORGLEFT_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->PUCSORGLEFT = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PUCSORGLEFT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PUCSORGLEFT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PUCSORGLEFT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->PUCSORGLEFT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PUCSORGRIGHT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PUCSORGRIGHT_set(self,PUCSORGRIGHT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PUCSORGRIGHT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_PUCSORGRIGHT_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->PUCSORGRIGHT = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PUCSORGRIGHT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PUCSORGRIGHT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PUCSORGRIGHT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->PUCSORGRIGHT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PUCSORGFRONT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PUCSORGFRONT_set(self,PUCSORGFRONT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PUCSORGFRONT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_PUCSORGFRONT_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->PUCSORGFRONT = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PUCSORGFRONT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PUCSORGFRONT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PUCSORGFRONT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->PUCSORGFRONT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PUCSORGBACK_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PUCSORGBACK_set(self,PUCSORGBACK);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PUCSORGBACK_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_PUCSORGBACK_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->PUCSORGBACK = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PUCSORGBACK_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PUCSORGBACK_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PUCSORGBACK_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->PUCSORGBACK);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_INSBASE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_INSBASE_set(self,INSBASE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_INSBASE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_INSBASE_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->INSBASE = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_INSBASE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_INSBASE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_INSBASE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->INSBASE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_EXTMIN_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_EXTMIN_set(self,EXTMIN);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_EXTMIN_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_EXTMIN_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->EXTMIN = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_EXTMIN_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_EXTMIN_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_EXTMIN_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->EXTMIN);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_EXTMAX_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_EXTMAX_set(self,EXTMAX);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_EXTMAX_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_EXTMAX_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->EXTMAX = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_EXTMAX_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_EXTMAX_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_EXTMAX_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->EXTMAX);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_VIEWDIR_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_VIEWDIR_set(self,VIEWDIR);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_VIEWDIR_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_VIEWDIR_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->VIEWDIR = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_VIEWDIR_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_VIEWDIR_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_VIEWDIR_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->VIEWDIR);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_TARGET_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_TARGET_set(self,TARGET);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_TARGET_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_TARGET_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->TARGET = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_TARGET_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_TARGET_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_TARGET_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->TARGET);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LIMMIN_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_2DPOINT *arg2 = (BITCODE_2DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LIMMIN_set(self,LIMMIN);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LIMMIN_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_LIMMIN_set" "', argument " "2"" of type '" "BITCODE_2DPOINT *""'"); 
    }
    arg2 = (BITCODE_2DPOINT *)(argp2);
    if (arg1) (arg1)->LIMMIN = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LIMMIN_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LIMMIN_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LIMMIN_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_2DPOINT *)& ((arg1)->LIMMIN);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LIMMAX_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_2DPOINT *arg2 = (BITCODE_2DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LIMMAX_set(self,LIMMAX);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LIMMAX_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_LIMMAX_set" "', argument " "2"" of type '" "BITCODE_2DPOINT *""'"); 
    }
    arg2 = (BITCODE_2DPOINT *)(argp2);
    if (arg1) (arg1)->LIMMAX = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LIMMAX_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LIMMAX_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LIMMAX_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_2DPOINT *)& ((arg1)->LIMMAX);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_VIEWCTR_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_3RD *arg2 = (BITCODE_3RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_VIEWCTR_set(self,VIEWCTR);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_VIEWCTR_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_VIEWCTR_set" "', argument " "2"" of type '" "BITCODE_3RD *""'"); 
    }
    arg2 = (BITCODE_3RD *)(argp2);
    if (arg1) (arg1)->VIEWCTR = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_VIEWCTR_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_VIEWCTR_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_VIEWCTR_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_3RD *)& ((arg1)->VIEWCTR);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_ELEVATION_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_ELEVATION_set(self,ELEVATION);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_ELEVATION_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_ELEVATION_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->ELEVATION = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_ELEVATION_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_ELEVATION_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_ELEVATION_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->ELEVATION);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_VIEWSIZE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_VIEWSIZE_set(self,VIEWSIZE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_VIEWSIZE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_VIEWSIZE_set" "', argument " "2"" of type '" "BITCODE_RD""'");
    } 
    arg2 = (BITCODE_RD)(val2);
    if (arg1) (arg1)->VIEWSIZE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_VIEWSIZE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_VIEWSIZE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_VIEWSIZE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RD) ((arg1)->VIEWSIZE);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SNAPMODE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SNAPMODE_set(self,SNAPMODE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SNAPMODE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_SNAPMODE_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->SNAPMODE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SNAPMODE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SNAPMODE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SNAPMODE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RS) ((arg1)->SNAPMODE);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SNAPUNIT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SNAPUNIT_set(self,SNAPUNIT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SNAPUNIT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_SNAPUNIT_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->SNAPUNIT = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SNAPUNIT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SNAPUNIT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SNAPUNIT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->SNAPUNIT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SNAPBASE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SNAPBASE_set(self,SNAPBASE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SNAPBASE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_SNAPBASE_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->SNAPBASE = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SNAPBASE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SNAPBASE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SNAPBASE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->SNAPBASE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SNAPANG_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SNAPANG_set(self,SNAPANG);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SNAPANG_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_SNAPANG_set" "', argument " "2"" of type '" "BITCODE_RD""'");
    } 
    arg2 = (BITCODE_RD)(val2);
    if (arg1) (arg1)->SNAPANG = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SNAPANG_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SNAPANG_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SNAPANG_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RD) ((arg1)->SNAPANG);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SNAPSTYLE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SNAPSTYLE_set(self,SNAPSTYLE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SNAPSTYLE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_SNAPSTYLE_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->SNAPSTYLE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SNAPSTYLE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SNAPSTYLE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SNAPSTYLE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RS) ((arg1)->SNAPSTYLE);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SNAPISOPAIR_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SNAPISOPAIR_set(self,SNAPISOPAIR);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SNAPISOPAIR_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_SNAPISOPAIR_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->SNAPISOPAIR = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SNAPISOPAIR_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SNAPISOPAIR_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SNAPISOPAIR_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RS) ((arg1)->SNAPISOPAIR);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_GRIDMODE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_GRIDMODE_set(self,GRIDMODE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_GRIDMODE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_GRIDMODE_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->GRIDMODE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_GRIDMODE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_GRIDMODE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_GRIDMODE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RS) ((arg1)->GRIDMODE);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_GRIDUNIT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_GRIDUNIT_set(self,GRIDUNIT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_GRIDUNIT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_GRIDUNIT_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->GRIDUNIT = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_GRIDUNIT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_GRIDUNIT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_GRIDUNIT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->GRIDUNIT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_AXISMODE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_AXISMODE_set(self,AXISMODE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_AXISMODE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_AXISMODE_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->AXISMODE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_AXISMODE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_AXISMODE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_AXISMODE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->AXISMODE);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_AXISUNIT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_AXISUNIT_set(self,AXISUNIT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_AXISUNIT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_AXISUNIT_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->AXISUNIT = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_AXISUNIT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_AXISUNIT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_AXISUNIT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->AXISUNIT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_VPOINTX_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_3RD *arg2 = (BITCODE_3RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_VPOINTX_set(self,VPOINTX);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_VPOINTX_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_VPOINTX_set" "', argument " "2"" of type '" "BITCODE_3RD *""'"); 
    }
    arg2 = (BITCODE_3RD *)(argp2);
    if (arg1) (arg1)->VPOINTX = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_VPOINTX_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_VPOINTX_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_VPOINTX_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_3RD *)& ((arg1)->VPOINTX);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_VPOINTY_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_3RD *arg2 = (BITCODE_3RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_VPOINTY_set(self,VPOINTY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_VPOINTY_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_VPOINTY_set" "', argument " "2"" of type '" "BITCODE_3RD *""'"); 
    }
    arg2 = (BITCODE_3RD *)(argp2);
    if (arg1) (arg1)->VPOINTY = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_VPOINTY_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_VPOINTY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_VPOINTY_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_3RD *)& ((arg1)->VPOINTY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_VPOINTZ_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_3RD *arg2 = (BITCODE_3RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_VPOINTZ_set(self,VPOINTZ);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_VPOINTZ_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_VPOINTZ_set" "', argument " "2"" of type '" "BITCODE_3RD *""'"); 
    }
    arg2 = (BITCODE_3RD *)(argp2);
    if (arg1) (arg1)->VPOINTZ = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_VPOINTZ_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_VPOINTZ_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_VPOINTZ_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_3RD *)& ((arg1)->VPOINTZ);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_VPOINTXALT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_3RD *arg2 = (BITCODE_3RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_VPOINTXALT_set(self,VPOINTXALT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_VPOINTXALT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_VPOINTXALT_set" "', argument " "2"" of type '" "BITCODE_3RD *""'"); 
    }
    arg2 = (BITCODE_3RD *)(argp2);
    if (arg1) (arg1)->VPOINTXALT = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_VPOINTXALT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_VPOINTXALT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_VPOINTXALT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_3RD *)& ((arg1)->VPOINTXALT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_VPOINTYALT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_3RD *arg2 = (BITCODE_3RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_VPOINTYALT_set(self,VPOINTYALT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_VPOINTYALT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_VPOINTYALT_set" "', argument " "2"" of type '" "BITCODE_3RD *""'"); 
    }
    arg2 = (BITCODE_3RD *)(argp2);
    if (arg1) (arg1)->VPOINTYALT = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_VPOINTYALT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_VPOINTYALT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_VPOINTYALT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_3RD *)& ((arg1)->VPOINTYALT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_VPOINTZALT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_3RD *arg2 = (BITCODE_3RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_VPOINTZALT_set(self,VPOINTZALT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_VPOINTZALT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_VPOINTZALT_set" "', argument " "2"" of type '" "BITCODE_3RD *""'"); 
    }
    arg2 = (BITCODE_3RD *)(argp2);
    if (arg1) (arg1)->VPOINTZALT = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_VPOINTZALT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_VPOINTZALT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_VPOINTZALT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_3RD *)& ((arg1)->VPOINTZALT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_flag_3d_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_flag_3d_set(self,flag_3d);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_flag_3d_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_flag_3d_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->flag_3d = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_flag_3d_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_flag_3d_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_flag_3d_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RS) ((arg1)->flag_3d);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_UCSORG_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_UCSORG_set(self,UCSORG);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_UCSORG_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_UCSORG_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->UCSORG = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_UCSORG_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_UCSORG_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_UCSORG_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->UCSORG);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_UCSXDIR_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_UCSXDIR_set(self,UCSXDIR);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_UCSXDIR_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_UCSXDIR_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->UCSXDIR = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_UCSXDIR_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_UCSXDIR_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_UCSXDIR_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->UCSXDIR);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_UCSYDIR_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_UCSYDIR_set(self,UCSYDIR);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_UCSYDIR_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_UCSYDIR_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->UCSYDIR = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_UCSYDIR_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_UCSYDIR_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_UCSYDIR_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->UCSYDIR);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_UCSNAME_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_UCSNAME_set(self,UCSNAME);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_UCSNAME_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_UCSNAME_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->UCSNAME = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_UCSNAME_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_UCSNAME_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_UCSNAME_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->UCSNAME);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_UCSBASE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_UCSBASE_set(self,UCSBASE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_UCSBASE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_UCSBASE_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->UCSBASE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_UCSBASE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_UCSBASE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_UCSBASE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->UCSBASE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_UCSORTHOVIEW_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_UCSORTHOVIEW_set(self,UCSORTHOVIEW);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_UCSORTHOVIEW_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_UCSORTHOVIEW_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->UCSORTHOVIEW = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_UCSORTHOVIEW_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_UCSORTHOVIEW_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_UCSORTHOVIEW_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->UCSORTHOVIEW);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_UCSORTHOREF_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_UCSORTHOREF_set(self,UCSORTHOREF);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_UCSORTHOREF_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_UCSORTHOREF_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->UCSORTHOREF = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_UCSORTHOREF_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_UCSORTHOREF_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_UCSORTHOREF_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->UCSORTHOREF);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_UCSORGTOP_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_UCSORGTOP_set(self,UCSORGTOP);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_UCSORGTOP_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_UCSORGTOP_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->UCSORGTOP = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_UCSORGTOP_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_UCSORGTOP_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_UCSORGTOP_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->UCSORGTOP);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_UCSORGBOTTOM_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_UCSORGBOTTOM_set(self,UCSORGBOTTOM);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_UCSORGBOTTOM_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_UCSORGBOTTOM_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->UCSORGBOTTOM = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_UCSORGBOTTOM_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_UCSORGBOTTOM_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_UCSORGBOTTOM_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->UCSORGBOTTOM);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_UCSORGLEFT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_UCSORGLEFT_set(self,UCSORGLEFT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_UCSORGLEFT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_UCSORGLEFT_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->UCSORGLEFT = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_UCSORGLEFT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_UCSORGLEFT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_UCSORGLEFT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->UCSORGLEFT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_UCSORGRIGHT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_UCSORGRIGHT_set(self,UCSORGRIGHT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_UCSORGRIGHT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_UCSORGRIGHT_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->UCSORGRIGHT = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_UCSORGRIGHT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_UCSORGRIGHT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_UCSORGRIGHT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->UCSORGRIGHT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_UCSORGFRONT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_UCSORGFRONT_set(self,UCSORGFRONT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_UCSORGFRONT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_UCSORGFRONT_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->UCSORGFRONT = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_UCSORGFRONT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_UCSORGFRONT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_UCSORGFRONT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->UCSORGFRONT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_UCSORGBACK_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_UCSORGBACK_set(self,UCSORGBACK);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_UCSORGBACK_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_UCSORGBACK_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->UCSORGBACK = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_UCSORGBACK_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_UCSORGBACK_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_UCSORGBACK_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->UCSORGBACK);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMPOST_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMPOST_set(self,DIMPOST);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMPOST_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_DIMPOST_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->DIMPOST);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->DIMPOST = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->DIMPOST = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMPOST_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMPOST_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMPOST_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_TV) ((arg1)->DIMPOST);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMAPOST_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMAPOST_set(self,DIMAPOST);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMAPOST_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_DIMAPOST_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->DIMAPOST);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->DIMAPOST = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->DIMAPOST = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMAPOST_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMAPOST_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMAPOST_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_TV) ((arg1)->DIMAPOST);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTOL_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTOL_set(self,DIMTOL);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTOL_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMTOL_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DIMTOL = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTOL_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTOL_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTOL_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->DIMTOL);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMLIM_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMLIM_set(self,DIMLIM);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMLIM_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMLIM_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DIMLIM = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMLIM_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMLIM_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMLIM_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->DIMLIM);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTIH_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTIH_set(self,DIMTIH);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTIH_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMTIH_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DIMTIH = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTIH_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTIH_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTIH_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->DIMTIH);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTOH_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTOH_set(self,DIMTOH);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTOH_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMTOH_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DIMTOH = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTOH_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTOH_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTOH_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->DIMTOH);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMSE1_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMSE1_set(self,DIMSE1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMSE1_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMSE1_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DIMSE1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMSE1_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMSE1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMSE1_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->DIMSE1);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMSE2_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMSE2_set(self,DIMSE2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMSE2_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMSE2_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DIMSE2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMSE2_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMSE2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMSE2_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->DIMSE2);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMALT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMALT_set(self,DIMALT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMALT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMALT_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DIMALT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMALT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMALT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMALT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->DIMALT);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTOFL_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTOFL_set(self,DIMTOFL);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTOFL_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMTOFL_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DIMTOFL = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTOFL_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTOFL_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTOFL_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->DIMTOFL);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMSAH_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMSAH_set(self,DIMSAH);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMSAH_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMSAH_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DIMSAH = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMSAH_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMSAH_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMSAH_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->DIMSAH);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTIX_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTIX_set(self,DIMTIX);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTIX_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMTIX_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DIMTIX = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTIX_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTIX_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTIX_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->DIMTIX);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMSOXD_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMSOXD_set(self,DIMSOXD);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMSOXD_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMSOXD_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DIMSOXD = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMSOXD_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMSOXD_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMSOXD_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->DIMSOXD);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMALTD_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMALTD_set(self,DIMALTD);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMALTD_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMALTD_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMALTD = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMALTD_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMALTD_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMALTD_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMALTD);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMZIN_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMZIN_set(self,DIMZIN);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMZIN_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMZIN_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMZIN = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMZIN_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMZIN_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMZIN_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMZIN);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMSD1_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMSD1_set(self,DIMSD1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMSD1_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMSD1_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DIMSD1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMSD1_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMSD1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMSD1_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->DIMSD1);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMSD2_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMSD2_set(self,DIMSD2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMSD2_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMSD2_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DIMSD2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMSD2_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMSD2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMSD2_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->DIMSD2);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTOLJ_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTOLJ_set(self,DIMTOLJ);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTOLJ_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMTOLJ_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMTOLJ = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTOLJ_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTOLJ_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTOLJ_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMTOLJ);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMJUST_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMJUST_set(self,DIMJUST);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMJUST_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMJUST_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMJUST = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMJUST_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMJUST_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMJUST_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMJUST);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMFIT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMFIT_set(self,DIMFIT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMFIT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMFIT_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMFIT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMFIT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMFIT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMFIT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMFIT);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMUPT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMUPT_set(self,DIMUPT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMUPT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMUPT_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DIMUPT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMUPT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMUPT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMUPT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->DIMUPT);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTZIN_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTZIN_set(self,DIMTZIN);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTZIN_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMTZIN_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMTZIN = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTZIN_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTZIN_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTZIN_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMTZIN);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTAD_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTAD_set(self,DIMTAD);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTAD_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMTAD_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMTAD = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTAD_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTAD_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTAD_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMTAD);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMUNIT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMUNIT_set(self,DIMUNIT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMUNIT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMUNIT_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMUNIT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMUNIT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMUNIT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMUNIT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMUNIT);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMAUNIT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMAUNIT_set(self,DIMAUNIT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMAUNIT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMAUNIT_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMAUNIT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMAUNIT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMAUNIT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMAUNIT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMAUNIT);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMDEC_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMDEC_set(self,DIMDEC);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMDEC_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMDEC_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMDEC = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMDEC_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMDEC_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMDEC_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMDEC);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTDEC_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTDEC_set(self,DIMTDEC);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTDEC_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMTDEC_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMTDEC = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTDEC_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTDEC_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTDEC_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMTDEC);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMALTU_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMALTU_set(self,DIMALTU);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMALTU_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMALTU_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMALTU = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMALTU_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMALTU_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMALTU_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMALTU);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMALTTD_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMALTTD_set(self,DIMALTTD);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMALTTD_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMALTTD_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMALTTD = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMALTTD_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMALTTD_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMALTTD_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMALTTD);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTXSTY_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTXSTY_set(self,DIMTXSTY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTXSTY_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_DIMTXSTY_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->DIMTXSTY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTXSTY_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTXSTY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTXSTY_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->DIMTXSTY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMSCALE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMSCALE_set(self,DIMSCALE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMSCALE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMSCALE_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMSCALE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMSCALE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMSCALE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMSCALE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMSCALE);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMARROW_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMARROW_set(self,DIMARROW);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMARROW_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMARROW_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMARROW = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMARROW_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMARROW_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMARROW_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMARROW);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMASZ_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMASZ_set(self,DIMASZ);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMASZ_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMASZ_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMASZ = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMASZ_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMASZ_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMASZ_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMASZ);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMEXO_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMEXO_set(self,DIMEXO);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMEXO_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMEXO_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMEXO = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMEXO_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMEXO_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMEXO_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMEXO);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMDLI_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMDLI_set(self,DIMDLI);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMDLI_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMDLI_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMDLI = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMDLI_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMDLI_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMDLI_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMDLI);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMEXE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMEXE_set(self,DIMEXE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMEXE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMEXE_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMEXE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMEXE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMEXE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMEXE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMEXE);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMRND_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMRND_set(self,DIMRND);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMRND_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMRND_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMRND = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMRND_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMRND_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMRND_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMRND);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMDLE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMDLE_set(self,DIMDLE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMDLE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMDLE_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMDLE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMDLE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMDLE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMDLE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMDLE);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTP_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTP_set(self,DIMTP);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTP_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMTP_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMTP = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTP_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTP_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTP_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMTP);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTM_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTM_set(self,DIMTM);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTM_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMTM_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMTM = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTM_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTM_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTM_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMTM);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMFXL_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMFXL_set(self,DIMFXL);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMFXL_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMFXL_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMFXL = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMFXL_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMFXL_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMFXL_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMFXL);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMJOGANG_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMJOGANG_set(self,DIMJOGANG);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMJOGANG_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMJOGANG_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMJOGANG = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMJOGANG_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMJOGANG_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMJOGANG_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMJOGANG);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTFILL_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTFILL_set(self,DIMTFILL);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTFILL_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMTFILL_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMTFILL = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTFILL_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTFILL_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTFILL_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMTFILL);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTFILLCLR_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTFILLCLR_set(self,DIMTFILLCLR);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTFILLCLR_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_DIMTFILLCLR_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->DIMTFILLCLR = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTFILLCLR_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTFILLCLR_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTFILLCLR_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->DIMTFILLCLR);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMAZIN_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMAZIN_set(self,DIMAZIN);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMAZIN_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMAZIN_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMAZIN = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMAZIN_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMAZIN_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMAZIN_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMAZIN);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMARCSYM_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMARCSYM_set(self,DIMARCSYM);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMARCSYM_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMARCSYM_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMARCSYM = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMARCSYM_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMARCSYM_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMARCSYM_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMARCSYM);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTXT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTXT_set(self,DIMTXT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTXT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMTXT_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMTXT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTXT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTXT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTXT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMTXT);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMCEN_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMCEN_set(self,DIMCEN);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMCEN_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMCEN_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMCEN = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMCEN_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMCEN_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMCEN_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMCEN);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTSZ_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTSZ_set(self,DIMTSZ);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTSZ_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMTSZ_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMTSZ = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTSZ_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTSZ_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTSZ_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMTSZ);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMALTF_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMALTF_set(self,DIMALTF);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMALTF_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMALTF_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMALTF = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMALTF_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMALTF_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMALTF_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMALTF);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMLFAC_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMLFAC_set(self,DIMLFAC);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMLFAC_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMLFAC_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMLFAC = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMLFAC_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMLFAC_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMLFAC_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMLFAC);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTVP_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTVP_set(self,DIMTVP);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTVP_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMTVP_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMTVP = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTVP_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTVP_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTVP_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMTVP);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTFAC_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTFAC_set(self,DIMTFAC);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTFAC_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMTFAC_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMTFAC = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTFAC_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTFAC_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTFAC_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMTFAC);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMGAP_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMGAP_set(self,DIMGAP);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMGAP_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMGAP_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMGAP = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMGAP_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMGAP_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMGAP_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMGAP);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMPOST_T_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMPOST_T_set(self,DIMPOST_T);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMPOST_T_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_DIMPOST_T_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->DIMPOST_T);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->DIMPOST_T = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->DIMPOST_T = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMPOST_T_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMPOST_T_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMPOST_T_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_TV) ((arg1)->DIMPOST_T);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMAPOST_T_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMAPOST_T_set(self,DIMAPOST_T);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMAPOST_T_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_DIMAPOST_T_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->DIMAPOST_T);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->DIMAPOST_T = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->DIMAPOST_T = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMAPOST_T_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMAPOST_T_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMAPOST_T_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_TV) ((arg1)->DIMAPOST_T);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMBLK_T_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMBLK_T_set(self,DIMBLK_T);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMBLK_T_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_DIMBLK_T_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->DIMBLK_T);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->DIMBLK_T = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->DIMBLK_T = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMBLK_T_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMBLK_T_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMBLK_T_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_TV) ((arg1)->DIMBLK_T);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMBLK1_T_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMBLK1_T_set(self,DIMBLK1_T);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMBLK1_T_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_DIMBLK1_T_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->DIMBLK1_T);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->DIMBLK1_T = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->DIMBLK1_T = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMBLK1_T_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMBLK1_T_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMBLK1_T_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_TV) ((arg1)->DIMBLK1_T);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMBLK2_T_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMBLK2_T_set(self,DIMBLK2_T);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMBLK2_T_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_DIMBLK2_T_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->DIMBLK2_T);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->DIMBLK2_T = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->DIMBLK2_T = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMBLK2_T_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMBLK2_T_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMBLK2_T_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_TV) ((arg1)->DIMBLK2_T);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_string_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_string_set(self,unknown_string);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_string_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_unknown_string_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->unknown_string);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->unknown_string = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->unknown_string = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_string_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_string_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_string_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_TV) ((arg1)->unknown_string);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMALTRND_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMALTRND_set(self,DIMALTRND);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMALTRND_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMALTRND_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMALTRND = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMALTRND_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMALTRND_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMALTRND_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMALTRND);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMCLRD_C_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMCLRD_C_set(self,DIMCLRD_C);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMCLRD_C_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMCLRD_C_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->DIMCLRD_C = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMCLRD_C_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMCLRD_C_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMCLRD_C_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RS) ((arg1)->DIMCLRD_C);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMCLRE_C_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMCLRE_C_set(self,DIMCLRE_C);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMCLRE_C_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMCLRE_C_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->DIMCLRE_C = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMCLRE_C_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMCLRE_C_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMCLRE_C_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RS) ((arg1)->DIMCLRE_C);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMCLRT_C_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMCLRT_C_set(self,DIMCLRT_C);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMCLRT_C_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMCLRT_C_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->DIMCLRT_C = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMCLRT_C_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMCLRT_C_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMCLRT_C_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RS) ((arg1)->DIMCLRT_C);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMCLRD_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMCLRD_set(self,DIMCLRD);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMCLRD_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_DIMCLRD_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->DIMCLRD = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMCLRD_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMCLRD_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMCLRD_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->DIMCLRD);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMCLRE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMCLRE_set(self,DIMCLRE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMCLRE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_DIMCLRE_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->DIMCLRE = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMCLRE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMCLRE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMCLRE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->DIMCLRE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMCLRT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMCLRT_set(self,DIMCLRT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMCLRT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_DIMCLRT_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->DIMCLRT = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMCLRT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMCLRT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMCLRT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->DIMCLRT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMADEC_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMADEC_set(self,DIMADEC);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMADEC_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMADEC_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMADEC = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMADEC_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMADEC_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMADEC_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMADEC);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMFRAC_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMFRAC_set(self,DIMFRAC);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMFRAC_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMFRAC_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMFRAC = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMFRAC_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMFRAC_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMFRAC_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMFRAC);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMLUNIT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMLUNIT_set(self,DIMLUNIT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMLUNIT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMLUNIT_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMLUNIT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMLUNIT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMLUNIT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMLUNIT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMLUNIT);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMDSEP_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMDSEP_set(self,DIMDSEP);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMDSEP_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMDSEP_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMDSEP = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMDSEP_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMDSEP_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMDSEP_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMDSEP);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTMOVE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTMOVE_set(self,DIMTMOVE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTMOVE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMTMOVE_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMTMOVE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTMOVE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTMOVE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTMOVE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMTMOVE);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMALTZ_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMALTZ_set(self,DIMALTZ);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMALTZ_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMALTZ_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMALTZ = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMALTZ_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMALTZ_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMALTZ_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMALTZ);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMALTTZ_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMALTTZ_set(self,DIMALTTZ);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMALTTZ_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMALTTZ_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMALTTZ = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMALTTZ_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMALTTZ_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMALTTZ_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMALTTZ);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMATFIT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMATFIT_set(self,DIMATFIT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMATFIT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMATFIT_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMATFIT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMATFIT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMATFIT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMATFIT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMATFIT);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMFXLON_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMFXLON_set(self,DIMFXLON);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMFXLON_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMFXLON_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DIMFXLON = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMFXLON_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMFXLON_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMFXLON_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->DIMFXLON);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTXTDIRECTION_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTXTDIRECTION_set(self,DIMTXTDIRECTION);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTXTDIRECTION_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMTXTDIRECTION_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DIMTXTDIRECTION = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMTXTDIRECTION_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMTXTDIRECTION_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMTXTDIRECTION_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->DIMTXTDIRECTION);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMALTMZF_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMALTMZF_set(self,DIMALTMZF);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMALTMZF_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMALTMZF_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMALTMZF = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMALTMZF_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMALTMZF_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMALTMZF_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMALTMZF);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMALTMZS_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMALTMZS_set(self,DIMALTMZS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMALTMZS_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_DIMALTMZS_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->DIMALTMZS);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->DIMALTMZS = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->DIMALTMZS = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMALTMZS_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMALTMZS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMALTMZS_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_TV) ((arg1)->DIMALTMZS);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMMZF_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMMZF_set(self,DIMMZF);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMMZF_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMMZF_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMMZF = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMMZF_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMMZF_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMMZF_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMMZF);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMMZS_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMMZS_set(self,DIMMZS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMMZS_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_DIMMZS_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->DIMMZS);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->DIMMZS = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->DIMMZS = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMMZS_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMMZS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMMZS_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_TV) ((arg1)->DIMMZS);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMLDRBLK_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMLDRBLK_set(self,DIMLDRBLK);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMLDRBLK_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_DIMLDRBLK_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->DIMLDRBLK = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMLDRBLK_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMLDRBLK_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMLDRBLK_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->DIMLDRBLK);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMBLK_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMBLK_set(self,DIMBLK);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMBLK_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_DIMBLK_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->DIMBLK = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMBLK_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMBLK_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMBLK_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->DIMBLK);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMBLK1_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMBLK1_set(self,DIMBLK1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMBLK1_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_DIMBLK1_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->DIMBLK1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMBLK1_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMBLK1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMBLK1_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->DIMBLK1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMBLK2_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMBLK2_set(self,DIMBLK2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMBLK2_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_DIMBLK2_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->DIMBLK2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMBLK2_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMBLK2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMBLK2_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->DIMBLK2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMLTYPE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMLTYPE_set(self,DIMLTYPE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMLTYPE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_DIMLTYPE_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->DIMLTYPE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMLTYPE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMLTYPE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMLTYPE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->DIMLTYPE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMLTEX1_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMLTEX1_set(self,DIMLTEX1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMLTEX1_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_DIMLTEX1_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->DIMLTEX1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMLTEX1_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMLTEX1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMLTEX1_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->DIMLTEX1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMLTEX2_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMLTEX2_set(self,DIMLTEX2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMLTEX2_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_DIMLTEX2_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->DIMLTEX2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMLTEX2_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMLTEX2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMLTEX2_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->DIMLTEX2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMLWD_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BSd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMLWD_set(self,DIMLWD);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMLWD_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMLWD_set" "', argument " "2"" of type '" "BITCODE_BSd""'");
    } 
    arg2 = (BITCODE_BSd)(val2);
    if (arg1) (arg1)->DIMLWD = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMLWD_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BSd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMLWD_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMLWD_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BSd) ((arg1)->DIMLWD);
    ST(argvi) = SWIG_From_short  SWIG_PERL_CALL_ARGS_1((short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMLWE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BSd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMLWE_set(self,DIMLWE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMLWE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMLWE_set" "', argument " "2"" of type '" "BITCODE_BSd""'");
    } 
    arg2 = (BITCODE_BSd)(val2);
    if (arg1) (arg1)->DIMLWE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMLWE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BSd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMLWE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMLWE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BSd) ((arg1)->DIMLWE);
    ST(argvi) = SWIG_From_short  SWIG_PERL_CALL_ARGS_1((short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_BLOCK_CONTROL_OBJECT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_BLOCK_CONTROL_OBJECT_set(self,BLOCK_CONTROL_OBJECT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_BLOCK_CONTROL_OBJECT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_BLOCK_CONTROL_OBJECT_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->BLOCK_CONTROL_OBJECT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_BLOCK_CONTROL_OBJECT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_BLOCK_CONTROL_OBJECT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_BLOCK_CONTROL_OBJECT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->BLOCK_CONTROL_OBJECT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LAYER_CONTROL_OBJECT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LAYER_CONTROL_OBJECT_set(self,LAYER_CONTROL_OBJECT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LAYER_CONTROL_OBJECT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_LAYER_CONTROL_OBJECT_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->LAYER_CONTROL_OBJECT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LAYER_CONTROL_OBJECT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LAYER_CONTROL_OBJECT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LAYER_CONTROL_OBJECT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->LAYER_CONTROL_OBJECT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_STYLE_CONTROL_OBJECT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_STYLE_CONTROL_OBJECT_set(self,STYLE_CONTROL_OBJECT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_STYLE_CONTROL_OBJECT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_STYLE_CONTROL_OBJECT_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->STYLE_CONTROL_OBJECT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_STYLE_CONTROL_OBJECT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_STYLE_CONTROL_OBJECT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_STYLE_CONTROL_OBJECT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->STYLE_CONTROL_OBJECT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LTYPE_CONTROL_OBJECT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LTYPE_CONTROL_OBJECT_set(self,LTYPE_CONTROL_OBJECT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LTYPE_CONTROL_OBJECT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_LTYPE_CONTROL_OBJECT_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->LTYPE_CONTROL_OBJECT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LTYPE_CONTROL_OBJECT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LTYPE_CONTROL_OBJECT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LTYPE_CONTROL_OBJECT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->LTYPE_CONTROL_OBJECT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_VIEW_CONTROL_OBJECT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_VIEW_CONTROL_OBJECT_set(self,VIEW_CONTROL_OBJECT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_VIEW_CONTROL_OBJECT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_VIEW_CONTROL_OBJECT_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->VIEW_CONTROL_OBJECT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_VIEW_CONTROL_OBJECT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_VIEW_CONTROL_OBJECT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_VIEW_CONTROL_OBJECT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->VIEW_CONTROL_OBJECT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_UCS_CONTROL_OBJECT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_UCS_CONTROL_OBJECT_set(self,UCS_CONTROL_OBJECT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_UCS_CONTROL_OBJECT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_UCS_CONTROL_OBJECT_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->UCS_CONTROL_OBJECT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_UCS_CONTROL_OBJECT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_UCS_CONTROL_OBJECT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_UCS_CONTROL_OBJECT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->UCS_CONTROL_OBJECT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_VPORT_CONTROL_OBJECT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_VPORT_CONTROL_OBJECT_set(self,VPORT_CONTROL_OBJECT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_VPORT_CONTROL_OBJECT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_VPORT_CONTROL_OBJECT_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->VPORT_CONTROL_OBJECT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_VPORT_CONTROL_OBJECT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_VPORT_CONTROL_OBJECT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_VPORT_CONTROL_OBJECT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->VPORT_CONTROL_OBJECT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_APPID_CONTROL_OBJECT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_APPID_CONTROL_OBJECT_set(self,APPID_CONTROL_OBJECT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_APPID_CONTROL_OBJECT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_APPID_CONTROL_OBJECT_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->APPID_CONTROL_OBJECT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_APPID_CONTROL_OBJECT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_APPID_CONTROL_OBJECT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_APPID_CONTROL_OBJECT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->APPID_CONTROL_OBJECT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMSTYLE_CONTROL_OBJECT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMSTYLE_CONTROL_OBJECT_set(self,DIMSTYLE_CONTROL_OBJECT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMSTYLE_CONTROL_OBJECT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_DIMSTYLE_CONTROL_OBJECT_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->DIMSTYLE_CONTROL_OBJECT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMSTYLE_CONTROL_OBJECT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMSTYLE_CONTROL_OBJECT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMSTYLE_CONTROL_OBJECT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->DIMSTYLE_CONTROL_OBJECT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_VX_CONTROL_OBJECT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_VX_CONTROL_OBJECT_set(self,VX_CONTROL_OBJECT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_VX_CONTROL_OBJECT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_VX_CONTROL_OBJECT_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->VX_CONTROL_OBJECT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_VX_CONTROL_OBJECT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_VX_CONTROL_OBJECT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_VX_CONTROL_OBJECT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->VX_CONTROL_OBJECT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DICTIONARY_ACAD_GROUP_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DICTIONARY_ACAD_GROUP_set(self,DICTIONARY_ACAD_GROUP);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DICTIONARY_ACAD_GROUP_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_DICTIONARY_ACAD_GROUP_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->DICTIONARY_ACAD_GROUP = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DICTIONARY_ACAD_GROUP_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DICTIONARY_ACAD_GROUP_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DICTIONARY_ACAD_GROUP_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->DICTIONARY_ACAD_GROUP);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DICTIONARY_ACAD_MLINESTYLE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DICTIONARY_ACAD_MLINESTYLE_set(self,DICTIONARY_ACAD_MLINESTYLE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DICTIONARY_ACAD_MLINESTYLE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_DICTIONARY_ACAD_MLINESTYLE_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->DICTIONARY_ACAD_MLINESTYLE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DICTIONARY_ACAD_MLINESTYLE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DICTIONARY_ACAD_MLINESTYLE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DICTIONARY_ACAD_MLINESTYLE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->DICTIONARY_ACAD_MLINESTYLE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DICTIONARY_NAMED_OBJECT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DICTIONARY_NAMED_OBJECT_set(self,DICTIONARY_NAMED_OBJECT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DICTIONARY_NAMED_OBJECT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_DICTIONARY_NAMED_OBJECT_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->DICTIONARY_NAMED_OBJECT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DICTIONARY_NAMED_OBJECT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DICTIONARY_NAMED_OBJECT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DICTIONARY_NAMED_OBJECT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->DICTIONARY_NAMED_OBJECT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_TSTACKALIGN_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_TSTACKALIGN_set(self,TSTACKALIGN);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_TSTACKALIGN_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_TSTACKALIGN_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->TSTACKALIGN = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_TSTACKALIGN_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_TSTACKALIGN_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_TSTACKALIGN_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->TSTACKALIGN);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_TSTACKSIZE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_TSTACKSIZE_set(self,TSTACKSIZE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_TSTACKSIZE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_TSTACKSIZE_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->TSTACKSIZE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_TSTACKSIZE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_TSTACKSIZE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_TSTACKSIZE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->TSTACKSIZE);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_HYPERLINKBASE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_HYPERLINKBASE_set(self,HYPERLINKBASE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_HYPERLINKBASE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_HYPERLINKBASE_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->HYPERLINKBASE);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->HYPERLINKBASE = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->HYPERLINKBASE = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_HYPERLINKBASE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_HYPERLINKBASE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_HYPERLINKBASE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_TV) ((arg1)->HYPERLINKBASE);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_STYLESHEET_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_STYLESHEET_set(self,STYLESHEET);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_STYLESHEET_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_STYLESHEET_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->STYLESHEET);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->STYLESHEET = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->STYLESHEET = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_STYLESHEET_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_STYLESHEET_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_STYLESHEET_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_TV) ((arg1)->STYLESHEET);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DICTIONARY_LAYOUT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DICTIONARY_LAYOUT_set(self,DICTIONARY_LAYOUT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DICTIONARY_LAYOUT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_DICTIONARY_LAYOUT_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->DICTIONARY_LAYOUT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DICTIONARY_LAYOUT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DICTIONARY_LAYOUT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DICTIONARY_LAYOUT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->DICTIONARY_LAYOUT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DICTIONARY_PLOTSETTINGS_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DICTIONARY_PLOTSETTINGS_set(self,DICTIONARY_PLOTSETTINGS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DICTIONARY_PLOTSETTINGS_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_DICTIONARY_PLOTSETTINGS_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->DICTIONARY_PLOTSETTINGS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DICTIONARY_PLOTSETTINGS_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DICTIONARY_PLOTSETTINGS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DICTIONARY_PLOTSETTINGS_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->DICTIONARY_PLOTSETTINGS);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DICTIONARY_PLOTSTYLENAME_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DICTIONARY_PLOTSTYLENAME_set(self,DICTIONARY_PLOTSTYLENAME);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DICTIONARY_PLOTSTYLENAME_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_DICTIONARY_PLOTSTYLENAME_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->DICTIONARY_PLOTSTYLENAME = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DICTIONARY_PLOTSTYLENAME_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DICTIONARY_PLOTSTYLENAME_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DICTIONARY_PLOTSTYLENAME_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->DICTIONARY_PLOTSTYLENAME);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DICTIONARY_MATERIAL_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DICTIONARY_MATERIAL_set(self,DICTIONARY_MATERIAL);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DICTIONARY_MATERIAL_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_DICTIONARY_MATERIAL_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->DICTIONARY_MATERIAL = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DICTIONARY_MATERIAL_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DICTIONARY_MATERIAL_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DICTIONARY_MATERIAL_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->DICTIONARY_MATERIAL);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DICTIONARY_COLOR_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DICTIONARY_COLOR_set(self,DICTIONARY_COLOR);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DICTIONARY_COLOR_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_DICTIONARY_COLOR_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->DICTIONARY_COLOR = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DICTIONARY_COLOR_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DICTIONARY_COLOR_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DICTIONARY_COLOR_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->DICTIONARY_COLOR);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DICTIONARY_VISUALSTYLE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DICTIONARY_VISUALSTYLE_set(self,DICTIONARY_VISUALSTYLE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DICTIONARY_VISUALSTYLE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_DICTIONARY_VISUALSTYLE_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->DICTIONARY_VISUALSTYLE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DICTIONARY_VISUALSTYLE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DICTIONARY_VISUALSTYLE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DICTIONARY_VISUALSTYLE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->DICTIONARY_VISUALSTYLE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DICTIONARY_LIGHTLIST_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DICTIONARY_LIGHTLIST_set(self,DICTIONARY_LIGHTLIST);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DICTIONARY_LIGHTLIST_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_DICTIONARY_LIGHTLIST_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->DICTIONARY_LIGHTLIST = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DICTIONARY_LIGHTLIST_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DICTIONARY_LIGHTLIST_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DICTIONARY_LIGHTLIST_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->DICTIONARY_LIGHTLIST);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_20_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_20_set(self,unknown_20);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_20_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_unknown_20_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->unknown_20 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_20_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_20_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_20_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->unknown_20);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_FLAGS_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_FLAGS_set(self,FLAGS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_FLAGS_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_FLAGS_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->FLAGS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_FLAGS_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_FLAGS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_FLAGS_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BL) ((arg1)->FLAGS);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CELWEIGHT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BSd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CELWEIGHT_set(self,CELWEIGHT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CELWEIGHT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_CELWEIGHT_set" "', argument " "2"" of type '" "BITCODE_BSd""'");
    } 
    arg2 = (BITCODE_BSd)(val2);
    if (arg1) (arg1)->CELWEIGHT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CELWEIGHT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BSd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CELWEIGHT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CELWEIGHT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BSd) ((arg1)->CELWEIGHT);
    ST(argvi) = SWIG_From_short  SWIG_PERL_CALL_ARGS_1((short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_ENDCAPS_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_ENDCAPS_set(self,ENDCAPS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_ENDCAPS_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_ENDCAPS_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->ENDCAPS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_ENDCAPS_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_ENDCAPS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_ENDCAPS_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->ENDCAPS);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_JOINSTYLE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_JOINSTYLE_set(self,JOINSTYLE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_JOINSTYLE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_JOINSTYLE_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->JOINSTYLE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_JOINSTYLE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_JOINSTYLE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_JOINSTYLE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->JOINSTYLE);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LWDISPLAY_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LWDISPLAY_set(self,LWDISPLAY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LWDISPLAY_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_LWDISPLAY_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->LWDISPLAY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LWDISPLAY_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LWDISPLAY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LWDISPLAY_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->LWDISPLAY);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_XEDIT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_XEDIT_set(self,XEDIT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_XEDIT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_XEDIT_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->XEDIT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_XEDIT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_XEDIT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_XEDIT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->XEDIT);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_EXTNAMES_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_EXTNAMES_set(self,EXTNAMES);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_EXTNAMES_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_EXTNAMES_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->EXTNAMES = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_EXTNAMES_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_EXTNAMES_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_EXTNAMES_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->EXTNAMES);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PSTYLEMODE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PSTYLEMODE_set(self,PSTYLEMODE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PSTYLEMODE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_PSTYLEMODE_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->PSTYLEMODE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PSTYLEMODE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PSTYLEMODE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PSTYLEMODE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->PSTYLEMODE);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_OLESTARTUP_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_OLESTARTUP_set(self,OLESTARTUP);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_OLESTARTUP_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_OLESTARTUP_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->OLESTARTUP = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_OLESTARTUP_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_OLESTARTUP_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_OLESTARTUP_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->OLESTARTUP);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_INSUNITS_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_INSUNITS_set(self,INSUNITS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_INSUNITS_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_INSUNITS_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->INSUNITS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_INSUNITS_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_INSUNITS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_INSUNITS_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->INSUNITS);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CEPSNTYPE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CEPSNTYPE_set(self,CEPSNTYPE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CEPSNTYPE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_CEPSNTYPE_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->CEPSNTYPE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CEPSNTYPE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CEPSNTYPE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CEPSNTYPE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->CEPSNTYPE);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CPSNID_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CPSNID_set(self,CPSNID);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CPSNID_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_CPSNID_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->CPSNID = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CPSNID_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CPSNID_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CPSNID_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->CPSNID);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_FINGERPRINTGUID_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_FINGERPRINTGUID_set(self,FINGERPRINTGUID);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_FINGERPRINTGUID_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_FINGERPRINTGUID_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->FINGERPRINTGUID);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->FINGERPRINTGUID = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->FINGERPRINTGUID = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_FINGERPRINTGUID_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_FINGERPRINTGUID_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_FINGERPRINTGUID_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_TV) ((arg1)->FINGERPRINTGUID);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_VERSIONGUID_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_VERSIONGUID_set(self,VERSIONGUID);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_VERSIONGUID_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_VERSIONGUID_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->VERSIONGUID);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->VERSIONGUID = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->VERSIONGUID = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_VERSIONGUID_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_VERSIONGUID_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_VERSIONGUID_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_TV) ((arg1)->VERSIONGUID);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SORTENTS_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SORTENTS_set(self,SORTENTS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SORTENTS_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_SORTENTS_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->SORTENTS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SORTENTS_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SORTENTS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SORTENTS_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RC) ((arg1)->SORTENTS);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_INDEXCTL_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_INDEXCTL_set(self,INDEXCTL);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_INDEXCTL_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_INDEXCTL_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->INDEXCTL = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_INDEXCTL_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_INDEXCTL_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_INDEXCTL_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RC) ((arg1)->INDEXCTL);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_HIDETEXT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_HIDETEXT_set(self,HIDETEXT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_HIDETEXT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_HIDETEXT_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->HIDETEXT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_HIDETEXT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_HIDETEXT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_HIDETEXT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RC) ((arg1)->HIDETEXT);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_XCLIPFRAME_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_XCLIPFRAME_set(self,XCLIPFRAME);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_XCLIPFRAME_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_XCLIPFRAME_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->XCLIPFRAME = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_XCLIPFRAME_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_XCLIPFRAME_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_XCLIPFRAME_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RC) ((arg1)->XCLIPFRAME);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMASSOC_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMASSOC_set(self,DIMASSOC);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMASSOC_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DIMASSOC_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->DIMASSOC = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DIMASSOC_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DIMASSOC_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DIMASSOC_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RC) ((arg1)->DIMASSOC);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_HALOGAP_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_HALOGAP_set(self,HALOGAP);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_HALOGAP_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_HALOGAP_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->HALOGAP = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_HALOGAP_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_HALOGAP_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_HALOGAP_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RC) ((arg1)->HALOGAP);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_OBSCOLOR_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_OBSCOLOR_set(self,OBSCOLOR);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_OBSCOLOR_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_OBSCOLOR_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->OBSCOLOR = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_OBSCOLOR_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_OBSCOLOR_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_OBSCOLOR_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->OBSCOLOR);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_INTERSECTIONCOLOR_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_INTERSECTIONCOLOR_set(self,INTERSECTIONCOLOR);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_INTERSECTIONCOLOR_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_INTERSECTIONCOLOR_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->INTERSECTIONCOLOR = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_INTERSECTIONCOLOR_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_INTERSECTIONCOLOR_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_INTERSECTIONCOLOR_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->INTERSECTIONCOLOR);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_OBSLTYPE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_OBSLTYPE_set(self,OBSLTYPE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_OBSLTYPE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_OBSLTYPE_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->OBSLTYPE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_OBSLTYPE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_OBSLTYPE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_OBSLTYPE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RC) ((arg1)->OBSLTYPE);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_INTERSECTIONDISPLAY_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_INTERSECTIONDISPLAY_set(self,INTERSECTIONDISPLAY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_INTERSECTIONDISPLAY_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_INTERSECTIONDISPLAY_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->INTERSECTIONDISPLAY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_INTERSECTIONDISPLAY_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_INTERSECTIONDISPLAY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_INTERSECTIONDISPLAY_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RC) ((arg1)->INTERSECTIONDISPLAY);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PROJECTNAME_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PROJECTNAME_set(self,PROJECTNAME);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PROJECTNAME_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_PROJECTNAME_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->PROJECTNAME);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->PROJECTNAME = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->PROJECTNAME = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PROJECTNAME_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PROJECTNAME_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PROJECTNAME_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_TV) ((arg1)->PROJECTNAME);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_BLOCK_RECORD_PSPACE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_BLOCK_RECORD_PSPACE_set(self,BLOCK_RECORD_PSPACE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_BLOCK_RECORD_PSPACE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_BLOCK_RECORD_PSPACE_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->BLOCK_RECORD_PSPACE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_BLOCK_RECORD_PSPACE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_BLOCK_RECORD_PSPACE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_BLOCK_RECORD_PSPACE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->BLOCK_RECORD_PSPACE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_BLOCK_RECORD_MSPACE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_BLOCK_RECORD_MSPACE_set(self,BLOCK_RECORD_MSPACE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_BLOCK_RECORD_MSPACE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_BLOCK_RECORD_MSPACE_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->BLOCK_RECORD_MSPACE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_BLOCK_RECORD_MSPACE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_BLOCK_RECORD_MSPACE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_BLOCK_RECORD_MSPACE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->BLOCK_RECORD_MSPACE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LTYPE_BYLAYER_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LTYPE_BYLAYER_set(self,LTYPE_BYLAYER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LTYPE_BYLAYER_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_LTYPE_BYLAYER_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->LTYPE_BYLAYER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LTYPE_BYLAYER_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LTYPE_BYLAYER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LTYPE_BYLAYER_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->LTYPE_BYLAYER);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LTYPE_BYBLOCK_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LTYPE_BYBLOCK_set(self,LTYPE_BYBLOCK);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LTYPE_BYBLOCK_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_LTYPE_BYBLOCK_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->LTYPE_BYBLOCK = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LTYPE_BYBLOCK_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LTYPE_BYBLOCK_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LTYPE_BYBLOCK_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->LTYPE_BYBLOCK);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LTYPE_CONTINUOUS_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LTYPE_CONTINUOUS_set(self,LTYPE_CONTINUOUS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LTYPE_CONTINUOUS_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_LTYPE_CONTINUOUS_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->LTYPE_CONTINUOUS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LTYPE_CONTINUOUS_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LTYPE_CONTINUOUS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LTYPE_CONTINUOUS_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->LTYPE_CONTINUOUS);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CAMERADISPLAY_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CAMERADISPLAY_set(self,CAMERADISPLAY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CAMERADISPLAY_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_CAMERADISPLAY_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->CAMERADISPLAY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CAMERADISPLAY_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CAMERADISPLAY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CAMERADISPLAY_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->CAMERADISPLAY);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_21_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_21_set(self,unknown_21);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_21_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_21_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_21 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_21_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_21_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_21_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_21);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_22_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_22_set(self,unknown_22);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_22_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_22_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_22 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_22_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_22_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_22_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_22);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_23_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_23_set(self,unknown_23);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_23_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_23_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->unknown_23 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_23_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_23_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_23_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->unknown_23);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_STEPSPERSEC_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_STEPSPERSEC_set(self,STEPSPERSEC);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_STEPSPERSEC_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_STEPSPERSEC_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->STEPSPERSEC = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_STEPSPERSEC_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_STEPSPERSEC_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_STEPSPERSEC_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->STEPSPERSEC);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_STEPSIZE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_STEPSIZE_set(self,STEPSIZE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_STEPSIZE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_STEPSIZE_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->STEPSIZE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_STEPSIZE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_STEPSIZE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_STEPSIZE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->STEPSIZE);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables__3DDWFPREC_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables__3DDWFPREC_set(self,_3DDWFPREC);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables__3DDWFPREC_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables__3DDWFPREC_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->_3DDWFPREC = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables__3DDWFPREC_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables__3DDWFPREC_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables__3DDWFPREC_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->_3DDWFPREC);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LENSLENGTH_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LENSLENGTH_set(self,LENSLENGTH);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LENSLENGTH_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_LENSLENGTH_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->LENSLENGTH = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LENSLENGTH_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LENSLENGTH_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LENSLENGTH_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->LENSLENGTH);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CAMERAHEIGHT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CAMERAHEIGHT_set(self,CAMERAHEIGHT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CAMERAHEIGHT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_CAMERAHEIGHT_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->CAMERAHEIGHT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CAMERAHEIGHT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CAMERAHEIGHT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CAMERAHEIGHT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->CAMERAHEIGHT);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SOLIDHIST_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SOLIDHIST_set(self,SOLIDHIST);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SOLIDHIST_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_SOLIDHIST_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->SOLIDHIST = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SOLIDHIST_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SOLIDHIST_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SOLIDHIST_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RC) ((arg1)->SOLIDHIST);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SHOWHIST_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SHOWHIST_set(self,SHOWHIST);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SHOWHIST_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_SHOWHIST_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->SHOWHIST = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SHOWHIST_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SHOWHIST_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SHOWHIST_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RC) ((arg1)->SHOWHIST);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PSOLWIDTH_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PSOLWIDTH_set(self,PSOLWIDTH);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PSOLWIDTH_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_PSOLWIDTH_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->PSOLWIDTH = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PSOLWIDTH_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PSOLWIDTH_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PSOLWIDTH_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->PSOLWIDTH);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PSOLHEIGHT_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PSOLHEIGHT_set(self,PSOLHEIGHT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PSOLHEIGHT_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_PSOLHEIGHT_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->PSOLHEIGHT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_PSOLHEIGHT_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_PSOLHEIGHT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_PSOLHEIGHT_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->PSOLHEIGHT);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LOFTANG1_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LOFTANG1_set(self,LOFTANG1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LOFTANG1_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_LOFTANG1_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->LOFTANG1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LOFTANG1_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LOFTANG1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LOFTANG1_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->LOFTANG1);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LOFTANG2_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LOFTANG2_set(self,LOFTANG2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LOFTANG2_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_LOFTANG2_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->LOFTANG2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LOFTANG2_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LOFTANG2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LOFTANG2_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->LOFTANG2);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LOFTMAG1_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LOFTMAG1_set(self,LOFTMAG1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LOFTMAG1_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_LOFTMAG1_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->LOFTMAG1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LOFTMAG1_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LOFTMAG1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LOFTMAG1_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->LOFTMAG1);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LOFTMAG2_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LOFTMAG2_set(self,LOFTMAG2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LOFTMAG2_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_LOFTMAG2_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->LOFTMAG2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LOFTMAG2_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LOFTMAG2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LOFTMAG2_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->LOFTMAG2);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LOFTPARAM_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LOFTPARAM_set(self,LOFTPARAM);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LOFTPARAM_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_LOFTPARAM_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->LOFTPARAM = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LOFTPARAM_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LOFTPARAM_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LOFTPARAM_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->LOFTPARAM);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LOFTNORMALS_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LOFTNORMALS_set(self,LOFTNORMALS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LOFTNORMALS_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_LOFTNORMALS_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->LOFTNORMALS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LOFTNORMALS_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LOFTNORMALS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LOFTNORMALS_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RC) ((arg1)->LOFTNORMALS);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LATITUDE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LATITUDE_set(self,LATITUDE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LATITUDE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_LATITUDE_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->LATITUDE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LATITUDE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LATITUDE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LATITUDE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->LATITUDE);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LONGITUDE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LONGITUDE_set(self,LONGITUDE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LONGITUDE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_LONGITUDE_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->LONGITUDE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LONGITUDE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LONGITUDE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LONGITUDE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->LONGITUDE);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_NORTHDIRECTION_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_NORTHDIRECTION_set(self,NORTHDIRECTION);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_NORTHDIRECTION_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_NORTHDIRECTION_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->NORTHDIRECTION = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_NORTHDIRECTION_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_NORTHDIRECTION_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_NORTHDIRECTION_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->NORTHDIRECTION);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_TIMEZONE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_TIMEZONE_set(self,TIMEZONE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_TIMEZONE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_TIMEZONE_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->TIMEZONE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_TIMEZONE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_TIMEZONE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_TIMEZONE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BL) ((arg1)->TIMEZONE);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LIGHTGLYPHDISPLAY_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LIGHTGLYPHDISPLAY_set(self,LIGHTGLYPHDISPLAY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LIGHTGLYPHDISPLAY_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_LIGHTGLYPHDISPLAY_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->LIGHTGLYPHDISPLAY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_LIGHTGLYPHDISPLAY_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_LIGHTGLYPHDISPLAY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_LIGHTGLYPHDISPLAY_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RC) ((arg1)->LIGHTGLYPHDISPLAY);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_TILEMODELIGHTSYNCH_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_TILEMODELIGHTSYNCH_set(self,TILEMODELIGHTSYNCH);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_TILEMODELIGHTSYNCH_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_TILEMODELIGHTSYNCH_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->TILEMODELIGHTSYNCH = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_TILEMODELIGHTSYNCH_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_TILEMODELIGHTSYNCH_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_TILEMODELIGHTSYNCH_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RC) ((arg1)->TILEMODELIGHTSYNCH);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DWFFRAME_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DWFFRAME_set(self,DWFFRAME);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DWFFRAME_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DWFFRAME_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->DWFFRAME = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DWFFRAME_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DWFFRAME_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DWFFRAME_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RC) ((arg1)->DWFFRAME);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DGNFRAME_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DGNFRAME_set(self,DGNFRAME);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DGNFRAME_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_DGNFRAME_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->DGNFRAME = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DGNFRAME_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DGNFRAME_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DGNFRAME_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RC) ((arg1)->DGNFRAME);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_REALWORLDSCALE_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_REALWORLDSCALE_set(self,REALWORLDSCALE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_REALWORLDSCALE_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_REALWORLDSCALE_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->REALWORLDSCALE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_REALWORLDSCALE_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_REALWORLDSCALE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_REALWORLDSCALE_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_B) ((arg1)->REALWORLDSCALE);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_INTERFERECOLOR_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_INTERFERECOLOR_set(self,INTERFERECOLOR);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_INTERFERECOLOR_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_INTERFERECOLOR_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->INTERFERECOLOR = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_INTERFERECOLOR_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_INTERFERECOLOR_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_INTERFERECOLOR_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->INTERFERECOLOR);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_INTERFEREOBJVS_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_INTERFEREOBJVS_set(self,INTERFEREOBJVS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_INTERFEREOBJVS_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_INTERFEREOBJVS_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->INTERFEREOBJVS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_INTERFEREOBJVS_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_INTERFEREOBJVS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_INTERFEREOBJVS_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->INTERFEREOBJVS);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_INTERFEREVPVS_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_INTERFEREVPVS_set(self,INTERFEREVPVS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_INTERFEREVPVS_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_INTERFEREVPVS_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->INTERFEREVPVS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_INTERFEREVPVS_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_INTERFEREVPVS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_INTERFEREVPVS_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->INTERFEREVPVS);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DRAGVS_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DRAGVS_set(self,DRAGVS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DRAGVS_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_DRAGVS_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->DRAGVS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_DRAGVS_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_DRAGVS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_DRAGVS_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_H) ((arg1)->DRAGVS);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CSHADOW_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CSHADOW_set(self,CSHADOW);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CSHADOW_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_CSHADOW_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->CSHADOW = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_CSHADOW_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_CSHADOW_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_CSHADOW_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RC) ((arg1)->CSHADOW);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SHADOWPLANELOCATION_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SHADOWPLANELOCATION_set(self,SHADOWPLANELOCATION);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SHADOWPLANELOCATION_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_SHADOWPLANELOCATION_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->SHADOWPLANELOCATION = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_SHADOWPLANELOCATION_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_SHADOWPLANELOCATION_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_SHADOWPLANELOCATION_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->SHADOWPLANELOCATION);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_54_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_54_set(self,unknown_54);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_54_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_54_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->unknown_54 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_54_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_54_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_54_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->unknown_54);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_55_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_55_set(self,unknown_55);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_55_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_55_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->unknown_55 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_55_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_55_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_55_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->unknown_55);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_56_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_56_set(self,unknown_56);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_56_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_56_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->unknown_56 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_56_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_56_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_56_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->unknown_56);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_57_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_57_set(self,unknown_57);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_57_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_57_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->unknown_57 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_57_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_57_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_57_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BS) ((arg1)->unknown_57);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_dwg_size_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_dwg_size_set(self,dwg_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_dwg_size_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_dwg_size_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->dwg_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_dwg_size_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_dwg_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_dwg_size_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RL) ((arg1)->dwg_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_numentities_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_numentities_set(self,numentities);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_numentities_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_numentities_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->numentities = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_numentities_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_numentities_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_numentities_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RS) ((arg1)->numentities);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_circle_zoom_percent_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_circle_zoom_percent_set(self,circle_zoom_percent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_circle_zoom_percent_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_circle_zoom_percent_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->circle_zoom_percent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_circle_zoom_percent_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_circle_zoom_percent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_circle_zoom_percent_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RS) ((arg1)->circle_zoom_percent);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_FRONTZ_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_FRONTZ_set(self,FRONTZ);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_FRONTZ_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_FRONTZ_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->FRONTZ = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_FRONTZ_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_FRONTZ_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_FRONTZ_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->FRONTZ);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_BACKZ_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_BACKZ_set(self,BACKZ);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_BACKZ_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_BACKZ_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->BACKZ = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_BACKZ_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_BACKZ_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_BACKZ_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_BD) ((arg1)->BACKZ);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_UCSICON_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_UCSICON_set(self,UCSICON);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_UCSICON_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_UCSICON_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->UCSICON = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_UCSICON_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_UCSICON_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_UCSICON_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RC) ((arg1)->UCSICON);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_oldCECOLOR_hi_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_oldCECOLOR_hi_set(self,oldCECOLOR_hi);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_oldCECOLOR_hi_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_oldCECOLOR_hi_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->oldCECOLOR_hi = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_oldCECOLOR_hi_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_oldCECOLOR_hi_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_oldCECOLOR_hi_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RL) ((arg1)->oldCECOLOR_hi);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_oldCECOLOR_lo_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_oldCECOLOR_lo_set(self,oldCECOLOR_lo);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_oldCECOLOR_lo_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_oldCECOLOR_lo_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->oldCECOLOR_lo = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_oldCECOLOR_lo_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_oldCECOLOR_lo_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_oldCECOLOR_lo_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RL) ((arg1)->oldCECOLOR_lo);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_layer_colors_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RS *arg2 = (BITCODE_RS *) (BITCODE_RS *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_layer_colors_set(self,layer_colors);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_layer_colors_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_short, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_Variables_layer_colors_set" "', argument " "2"" of type '" "BITCODE_RS [128]""'"); 
    } 
    arg2 = (BITCODE_RS *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)128; ++ii) *(BITCODE_RS *)&arg1->layer_colors[ii] = *((BITCODE_RS *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""layer_colors""' of type '""BITCODE_RS [128]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_layer_colors_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_layer_colors_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_layer_colors_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RS *)(BITCODE_RS *) ((arg1)->layer_colors);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_short, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_51e_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_51e_set(self,unknown_51e);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_51e_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_51e_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->unknown_51e = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_51e_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_51e_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_51e_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RS) ((arg1)->unknown_51e);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_520_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_520_set(self,unknown_520);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_520_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_520_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->unknown_520 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_520_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_520_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_520_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RS) ((arg1)->unknown_520);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_52c_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RSd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_52c_set(self,unknown_52c);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_52c_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_52c_set" "', argument " "2"" of type '" "BITCODE_RSd""'");
    } 
    arg2 = (BITCODE_RSd)(val2);
    if (arg1) (arg1)->unknown_52c = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_52c_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RSd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_52c_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_52c_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RSd) ((arg1)->unknown_52c);
    ST(argvi) = SWIG_From_short  SWIG_PERL_CALL_ARGS_1((short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_52e_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_52e_set(self,unknown_52e);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_52e_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_52e_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->unknown_52e = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_52e_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_52e_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_52e_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RS) ((arg1)->unknown_52e);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_530_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_530_set(self,unknown_530);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_530_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_530_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->unknown_530 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_530_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_530_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_530_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RC) ((arg1)->unknown_530);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_59_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_59_set(self,unknown_59);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_59_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_unknown_59_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->unknown_59 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_unknown_59_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_unknown_59_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_unknown_59_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RS) ((arg1)->unknown_59);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_aspect_ratio_set) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    BITCODE_RD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_Variables_aspect_ratio_set(self,aspect_ratio);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_aspect_ratio_set" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_Variables_aspect_ratio_set" "', argument " "2"" of type '" "BITCODE_RD""'");
    } 
    arg2 = (BITCODE_RD)(val2);
    if (arg1) (arg1)->aspect_ratio = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_Variables_aspect_ratio_get) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_Variables_aspect_ratio_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_Variables_aspect_ratio_get" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    result = (BITCODE_RD) ((arg1)->aspect_ratio);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Header_Variables) {
  {
    int argvi = 0;
    struct _dwg_header_variables *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Header_Variables();");
    }
    result = (struct _dwg_header_variables *)calloc(1, sizeof(struct _dwg_header_variables));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_header_variables, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Header_Variables) {
  {
    struct _dwg_header_variables *arg1 = (struct _dwg_header_variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Header_Variables(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header_variables, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Header_Variables" "', argument " "1"" of type '" "struct _dwg_header_variables *""'"); 
    }
    arg1 = (struct _dwg_header_variables *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TEXT_parent_set) {
  {
    struct _dwg_entity_TEXT *arg1 = (struct _dwg_entity_TEXT *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TEXT_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TEXT_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_TEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_TEXT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TEXT_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TEXT_parent_get) {
  {
    struct _dwg_entity_TEXT *arg1 = (struct _dwg_entity_TEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TEXT_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TEXT_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_TEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_TEXT *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TEXT_dataflags_set) {
  {
    struct _dwg_entity_TEXT *arg1 = (struct _dwg_entity_TEXT *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TEXT_dataflags_set(self,dataflags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TEXT_dataflags_set" "', argument " "1"" of type '" "struct _dwg_entity_TEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_TEXT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TEXT_dataflags_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->dataflags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TEXT_dataflags_get) {
  {
    struct _dwg_entity_TEXT *arg1 = (struct _dwg_entity_TEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TEXT_dataflags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TEXT_dataflags_get" "', argument " "1"" of type '" "struct _dwg_entity_TEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_TEXT *)(argp1);
    result = (BITCODE_RC) ((arg1)->dataflags);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TEXT_elevation_set) {
  {
    struct _dwg_entity_TEXT *arg1 = (struct _dwg_entity_TEXT *) 0 ;
    BITCODE_RD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TEXT_elevation_set(self,elevation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TEXT_elevation_set" "', argument " "1"" of type '" "struct _dwg_entity_TEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_TEXT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TEXT_elevation_set" "', argument " "2"" of type '" "BITCODE_RD""'");
    } 
    arg2 = (BITCODE_RD)(val2);
    if (arg1) (arg1)->elevation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TEXT_elevation_get) {
  {
    struct _dwg_entity_TEXT *arg1 = (struct _dwg_entity_TEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TEXT_elevation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TEXT_elevation_get" "', argument " "1"" of type '" "struct _dwg_entity_TEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_TEXT *)(argp1);
    result = (BITCODE_RD) ((arg1)->elevation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TEXT_ins_pt_set) {
  {
    struct _dwg_entity_TEXT *arg1 = (struct _dwg_entity_TEXT *) 0 ;
    BITCODE_2DPOINT *arg2 = (BITCODE_2DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TEXT_ins_pt_set(self,ins_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TEXT_ins_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_TEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_TEXT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TEXT_ins_pt_set" "', argument " "2"" of type '" "BITCODE_2DPOINT *""'"); 
    }
    arg2 = (BITCODE_2DPOINT *)(argp2);
    if (arg1) (arg1)->ins_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TEXT_ins_pt_get) {
  {
    struct _dwg_entity_TEXT *arg1 = (struct _dwg_entity_TEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TEXT_ins_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TEXT_ins_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_TEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_TEXT *)(argp1);
    result = (BITCODE_2DPOINT *)& ((arg1)->ins_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TEXT_alignment_pt_set) {
  {
    struct _dwg_entity_TEXT *arg1 = (struct _dwg_entity_TEXT *) 0 ;
    BITCODE_2DPOINT *arg2 = (BITCODE_2DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TEXT_alignment_pt_set(self,alignment_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TEXT_alignment_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_TEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_TEXT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TEXT_alignment_pt_set" "', argument " "2"" of type '" "BITCODE_2DPOINT *""'"); 
    }
    arg2 = (BITCODE_2DPOINT *)(argp2);
    if (arg1) (arg1)->alignment_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TEXT_alignment_pt_get) {
  {
    struct _dwg_entity_TEXT *arg1 = (struct _dwg_entity_TEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TEXT_alignment_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TEXT_alignment_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_TEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_TEXT *)(argp1);
    result = (BITCODE_2DPOINT *)& ((arg1)->alignment_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TEXT_extrusion_set) {
  {
    struct _dwg_entity_TEXT *arg1 = (struct _dwg_entity_TEXT *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TEXT_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TEXT_extrusion_set" "', argument " "1"" of type '" "struct _dwg_entity_TEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_TEXT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TEXT_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TEXT_extrusion_get) {
  {
    struct _dwg_entity_TEXT *arg1 = (struct _dwg_entity_TEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TEXT_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TEXT_extrusion_get" "', argument " "1"" of type '" "struct _dwg_entity_TEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_TEXT *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TEXT_thickness_set) {
  {
    struct _dwg_entity_TEXT *arg1 = (struct _dwg_entity_TEXT *) 0 ;
    BITCODE_RD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TEXT_thickness_set(self,thickness);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TEXT_thickness_set" "', argument " "1"" of type '" "struct _dwg_entity_TEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_TEXT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TEXT_thickness_set" "', argument " "2"" of type '" "BITCODE_RD""'");
    } 
    arg2 = (BITCODE_RD)(val2);
    if (arg1) (arg1)->thickness = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TEXT_thickness_get) {
  {
    struct _dwg_entity_TEXT *arg1 = (struct _dwg_entity_TEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TEXT_thickness_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TEXT_thickness_get" "', argument " "1"" of type '" "struct _dwg_entity_TEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_TEXT *)(argp1);
    result = (BITCODE_RD) ((arg1)->thickness);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TEXT_oblique_angle_set) {
  {
    struct _dwg_entity_TEXT *arg1 = (struct _dwg_entity_TEXT *) 0 ;
    BITCODE_RD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TEXT_oblique_angle_set(self,oblique_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TEXT_oblique_angle_set" "', argument " "1"" of type '" "struct _dwg_entity_TEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_TEXT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TEXT_oblique_angle_set" "', argument " "2"" of type '" "BITCODE_RD""'");
    } 
    arg2 = (BITCODE_RD)(val2);
    if (arg1) (arg1)->oblique_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TEXT_oblique_angle_get) {
  {
    struct _dwg_entity_TEXT *arg1 = (struct _dwg_entity_TEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TEXT_oblique_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TEXT_oblique_angle_get" "', argument " "1"" of type '" "struct _dwg_entity_TEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_TEXT *)(argp1);
    result = (BITCODE_RD) ((arg1)->oblique_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TEXT_rotation_set) {
  {
    struct _dwg_entity_TEXT *arg1 = (struct _dwg_entity_TEXT *) 0 ;
    BITCODE_RD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TEXT_rotation_set(self,rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TEXT_rotation_set" "', argument " "1"" of type '" "struct _dwg_entity_TEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_TEXT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TEXT_rotation_set" "', argument " "2"" of type '" "BITCODE_RD""'");
    } 
    arg2 = (BITCODE_RD)(val2);
    if (arg1) (arg1)->rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TEXT_rotation_get) {
  {
    struct _dwg_entity_TEXT *arg1 = (struct _dwg_entity_TEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TEXT_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TEXT_rotation_get" "', argument " "1"" of type '" "struct _dwg_entity_TEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_TEXT *)(argp1);
    result = (BITCODE_RD) ((arg1)->rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TEXT_height_set) {
  {
    struct _dwg_entity_TEXT *arg1 = (struct _dwg_entity_TEXT *) 0 ;
    BITCODE_RD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TEXT_height_set(self,height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TEXT_height_set" "', argument " "1"" of type '" "struct _dwg_entity_TEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_TEXT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TEXT_height_set" "', argument " "2"" of type '" "BITCODE_RD""'");
    } 
    arg2 = (BITCODE_RD)(val2);
    if (arg1) (arg1)->height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TEXT_height_get) {
  {
    struct _dwg_entity_TEXT *arg1 = (struct _dwg_entity_TEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TEXT_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TEXT_height_get" "', argument " "1"" of type '" "struct _dwg_entity_TEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_TEXT *)(argp1);
    result = (BITCODE_RD) ((arg1)->height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TEXT_width_factor_set) {
  {
    struct _dwg_entity_TEXT *arg1 = (struct _dwg_entity_TEXT *) 0 ;
    BITCODE_RD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TEXT_width_factor_set(self,width_factor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TEXT_width_factor_set" "', argument " "1"" of type '" "struct _dwg_entity_TEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_TEXT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TEXT_width_factor_set" "', argument " "2"" of type '" "BITCODE_RD""'");
    } 
    arg2 = (BITCODE_RD)(val2);
    if (arg1) (arg1)->width_factor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TEXT_width_factor_get) {
  {
    struct _dwg_entity_TEXT *arg1 = (struct _dwg_entity_TEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TEXT_width_factor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TEXT_width_factor_get" "', argument " "1"" of type '" "struct _dwg_entity_TEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_TEXT *)(argp1);
    result = (BITCODE_RD) ((arg1)->width_factor);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TEXT_text_value_set) {
  {
    struct _dwg_entity_TEXT *arg1 = (struct _dwg_entity_TEXT *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TEXT_text_value_set(self,text_value);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TEXT_text_value_set" "', argument " "1"" of type '" "struct _dwg_entity_TEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_TEXT *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TEXT_text_value_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->text_value);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->text_value = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->text_value = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TEXT_text_value_get) {
  {
    struct _dwg_entity_TEXT *arg1 = (struct _dwg_entity_TEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TEXT_text_value_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TEXT_text_value_get" "', argument " "1"" of type '" "struct _dwg_entity_TEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_TEXT *)(argp1);
    result = (BITCODE_TV) ((arg1)->text_value);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TEXT_generation_set) {
  {
    struct _dwg_entity_TEXT *arg1 = (struct _dwg_entity_TEXT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TEXT_generation_set(self,generation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TEXT_generation_set" "', argument " "1"" of type '" "struct _dwg_entity_TEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_TEXT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TEXT_generation_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->generation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TEXT_generation_get) {
  {
    struct _dwg_entity_TEXT *arg1 = (struct _dwg_entity_TEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TEXT_generation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TEXT_generation_get" "', argument " "1"" of type '" "struct _dwg_entity_TEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_TEXT *)(argp1);
    result = (BITCODE_BS) ((arg1)->generation);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TEXT_horiz_alignment_set) {
  {
    struct _dwg_entity_TEXT *arg1 = (struct _dwg_entity_TEXT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TEXT_horiz_alignment_set(self,horiz_alignment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TEXT_horiz_alignment_set" "', argument " "1"" of type '" "struct _dwg_entity_TEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_TEXT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TEXT_horiz_alignment_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->horiz_alignment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TEXT_horiz_alignment_get) {
  {
    struct _dwg_entity_TEXT *arg1 = (struct _dwg_entity_TEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TEXT_horiz_alignment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TEXT_horiz_alignment_get" "', argument " "1"" of type '" "struct _dwg_entity_TEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_TEXT *)(argp1);
    result = (BITCODE_BS) ((arg1)->horiz_alignment);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TEXT_vert_alignment_set) {
  {
    struct _dwg_entity_TEXT *arg1 = (struct _dwg_entity_TEXT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TEXT_vert_alignment_set(self,vert_alignment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TEXT_vert_alignment_set" "', argument " "1"" of type '" "struct _dwg_entity_TEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_TEXT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TEXT_vert_alignment_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->vert_alignment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TEXT_vert_alignment_get) {
  {
    struct _dwg_entity_TEXT *arg1 = (struct _dwg_entity_TEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TEXT_vert_alignment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TEXT_vert_alignment_get" "', argument " "1"" of type '" "struct _dwg_entity_TEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_TEXT *)(argp1);
    result = (BITCODE_BS) ((arg1)->vert_alignment);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TEXT_style_set) {
  {
    struct _dwg_entity_TEXT *arg1 = (struct _dwg_entity_TEXT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TEXT_style_set(self,style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TEXT_style_set" "', argument " "1"" of type '" "struct _dwg_entity_TEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_TEXT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TEXT_style_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TEXT_style_get) {
  {
    struct _dwg_entity_TEXT *arg1 = (struct _dwg_entity_TEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TEXT_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TEXT_style_get" "', argument " "1"" of type '" "struct _dwg_entity_TEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_TEXT *)(argp1);
    result = (BITCODE_H) ((arg1)->style);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_TEXT) {
  {
    int argvi = 0;
    struct _dwg_entity_TEXT *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_TEXT();");
    }
    result = (struct _dwg_entity_TEXT *)calloc(1, sizeof(struct _dwg_entity_TEXT));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_TEXT, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_TEXT) {
  {
    struct _dwg_entity_TEXT *arg1 = (struct _dwg_entity_TEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_TEXT(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TEXT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_TEXT" "', argument " "1"" of type '" "struct _dwg_entity_TEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_TEXT *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDbMTextObjectEmbedded_attachment_set) {
  {
    struct _dwg_AcDbMTextObjectEmbedded *arg1 = (struct _dwg_AcDbMTextObjectEmbedded *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDbMTextObjectEmbedded_attachment_set(self,attachment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDbMTextObjectEmbedded_attachment_set" "', argument " "1"" of type '" "struct _dwg_AcDbMTextObjectEmbedded *""'"); 
    }
    arg1 = (struct _dwg_AcDbMTextObjectEmbedded *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDbMTextObjectEmbedded_attachment_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->attachment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDbMTextObjectEmbedded_attachment_get) {
  {
    struct _dwg_AcDbMTextObjectEmbedded *arg1 = (struct _dwg_AcDbMTextObjectEmbedded *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDbMTextObjectEmbedded_attachment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDbMTextObjectEmbedded_attachment_get" "', argument " "1"" of type '" "struct _dwg_AcDbMTextObjectEmbedded *""'"); 
    }
    arg1 = (struct _dwg_AcDbMTextObjectEmbedded *)(argp1);
    result = (BITCODE_BL) ((arg1)->attachment);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDbMTextObjectEmbedded_ins_pt_set) {
  {
    struct _dwg_AcDbMTextObjectEmbedded *arg1 = (struct _dwg_AcDbMTextObjectEmbedded *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDbMTextObjectEmbedded_ins_pt_set(self,ins_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDbMTextObjectEmbedded_ins_pt_set" "', argument " "1"" of type '" "struct _dwg_AcDbMTextObjectEmbedded *""'"); 
    }
    arg1 = (struct _dwg_AcDbMTextObjectEmbedded *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AcDbMTextObjectEmbedded_ins_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ins_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDbMTextObjectEmbedded_ins_pt_get) {
  {
    struct _dwg_AcDbMTextObjectEmbedded *arg1 = (struct _dwg_AcDbMTextObjectEmbedded *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDbMTextObjectEmbedded_ins_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDbMTextObjectEmbedded_ins_pt_get" "', argument " "1"" of type '" "struct _dwg_AcDbMTextObjectEmbedded *""'"); 
    }
    arg1 = (struct _dwg_AcDbMTextObjectEmbedded *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ins_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDbMTextObjectEmbedded_x_axis_dir_set) {
  {
    struct _dwg_AcDbMTextObjectEmbedded *arg1 = (struct _dwg_AcDbMTextObjectEmbedded *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDbMTextObjectEmbedded_x_axis_dir_set(self,x_axis_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDbMTextObjectEmbedded_x_axis_dir_set" "', argument " "1"" of type '" "struct _dwg_AcDbMTextObjectEmbedded *""'"); 
    }
    arg1 = (struct _dwg_AcDbMTextObjectEmbedded *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AcDbMTextObjectEmbedded_x_axis_dir_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->x_axis_dir = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDbMTextObjectEmbedded_x_axis_dir_get) {
  {
    struct _dwg_AcDbMTextObjectEmbedded *arg1 = (struct _dwg_AcDbMTextObjectEmbedded *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDbMTextObjectEmbedded_x_axis_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDbMTextObjectEmbedded_x_axis_dir_get" "', argument " "1"" of type '" "struct _dwg_AcDbMTextObjectEmbedded *""'"); 
    }
    arg1 = (struct _dwg_AcDbMTextObjectEmbedded *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->x_axis_dir);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDbMTextObjectEmbedded_rect_height_set) {
  {
    struct _dwg_AcDbMTextObjectEmbedded *arg1 = (struct _dwg_AcDbMTextObjectEmbedded *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDbMTextObjectEmbedded_rect_height_set(self,rect_height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDbMTextObjectEmbedded_rect_height_set" "', argument " "1"" of type '" "struct _dwg_AcDbMTextObjectEmbedded *""'"); 
    }
    arg1 = (struct _dwg_AcDbMTextObjectEmbedded *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDbMTextObjectEmbedded_rect_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->rect_height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDbMTextObjectEmbedded_rect_height_get) {
  {
    struct _dwg_AcDbMTextObjectEmbedded *arg1 = (struct _dwg_AcDbMTextObjectEmbedded *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDbMTextObjectEmbedded_rect_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDbMTextObjectEmbedded_rect_height_get" "', argument " "1"" of type '" "struct _dwg_AcDbMTextObjectEmbedded *""'"); 
    }
    arg1 = (struct _dwg_AcDbMTextObjectEmbedded *)(argp1);
    result = (BITCODE_BD) ((arg1)->rect_height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDbMTextObjectEmbedded_rect_width_set) {
  {
    struct _dwg_AcDbMTextObjectEmbedded *arg1 = (struct _dwg_AcDbMTextObjectEmbedded *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDbMTextObjectEmbedded_rect_width_set(self,rect_width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDbMTextObjectEmbedded_rect_width_set" "', argument " "1"" of type '" "struct _dwg_AcDbMTextObjectEmbedded *""'"); 
    }
    arg1 = (struct _dwg_AcDbMTextObjectEmbedded *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDbMTextObjectEmbedded_rect_width_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->rect_width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDbMTextObjectEmbedded_rect_width_get) {
  {
    struct _dwg_AcDbMTextObjectEmbedded *arg1 = (struct _dwg_AcDbMTextObjectEmbedded *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDbMTextObjectEmbedded_rect_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDbMTextObjectEmbedded_rect_width_get" "', argument " "1"" of type '" "struct _dwg_AcDbMTextObjectEmbedded *""'"); 
    }
    arg1 = (struct _dwg_AcDbMTextObjectEmbedded *)(argp1);
    result = (BITCODE_BD) ((arg1)->rect_width);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDbMTextObjectEmbedded_extents_width_set) {
  {
    struct _dwg_AcDbMTextObjectEmbedded *arg1 = (struct _dwg_AcDbMTextObjectEmbedded *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDbMTextObjectEmbedded_extents_width_set(self,extents_width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDbMTextObjectEmbedded_extents_width_set" "', argument " "1"" of type '" "struct _dwg_AcDbMTextObjectEmbedded *""'"); 
    }
    arg1 = (struct _dwg_AcDbMTextObjectEmbedded *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDbMTextObjectEmbedded_extents_width_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->extents_width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDbMTextObjectEmbedded_extents_width_get) {
  {
    struct _dwg_AcDbMTextObjectEmbedded *arg1 = (struct _dwg_AcDbMTextObjectEmbedded *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDbMTextObjectEmbedded_extents_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDbMTextObjectEmbedded_extents_width_get" "', argument " "1"" of type '" "struct _dwg_AcDbMTextObjectEmbedded *""'"); 
    }
    arg1 = (struct _dwg_AcDbMTextObjectEmbedded *)(argp1);
    result = (BITCODE_BD) ((arg1)->extents_width);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDbMTextObjectEmbedded_extents_height_set) {
  {
    struct _dwg_AcDbMTextObjectEmbedded *arg1 = (struct _dwg_AcDbMTextObjectEmbedded *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDbMTextObjectEmbedded_extents_height_set(self,extents_height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDbMTextObjectEmbedded_extents_height_set" "', argument " "1"" of type '" "struct _dwg_AcDbMTextObjectEmbedded *""'"); 
    }
    arg1 = (struct _dwg_AcDbMTextObjectEmbedded *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDbMTextObjectEmbedded_extents_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->extents_height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDbMTextObjectEmbedded_extents_height_get) {
  {
    struct _dwg_AcDbMTextObjectEmbedded *arg1 = (struct _dwg_AcDbMTextObjectEmbedded *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDbMTextObjectEmbedded_extents_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDbMTextObjectEmbedded_extents_height_get" "', argument " "1"" of type '" "struct _dwg_AcDbMTextObjectEmbedded *""'"); 
    }
    arg1 = (struct _dwg_AcDbMTextObjectEmbedded *)(argp1);
    result = (BITCODE_BD) ((arg1)->extents_height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDbMTextObjectEmbedded_column_type_set) {
  {
    struct _dwg_AcDbMTextObjectEmbedded *arg1 = (struct _dwg_AcDbMTextObjectEmbedded *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDbMTextObjectEmbedded_column_type_set(self,column_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDbMTextObjectEmbedded_column_type_set" "', argument " "1"" of type '" "struct _dwg_AcDbMTextObjectEmbedded *""'"); 
    }
    arg1 = (struct _dwg_AcDbMTextObjectEmbedded *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDbMTextObjectEmbedded_column_type_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->column_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDbMTextObjectEmbedded_column_type_get) {
  {
    struct _dwg_AcDbMTextObjectEmbedded *arg1 = (struct _dwg_AcDbMTextObjectEmbedded *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDbMTextObjectEmbedded_column_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDbMTextObjectEmbedded_column_type_get" "', argument " "1"" of type '" "struct _dwg_AcDbMTextObjectEmbedded *""'"); 
    }
    arg1 = (struct _dwg_AcDbMTextObjectEmbedded *)(argp1);
    result = (BITCODE_BL) ((arg1)->column_type);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDbMTextObjectEmbedded_column_width_set) {
  {
    struct _dwg_AcDbMTextObjectEmbedded *arg1 = (struct _dwg_AcDbMTextObjectEmbedded *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDbMTextObjectEmbedded_column_width_set(self,column_width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDbMTextObjectEmbedded_column_width_set" "', argument " "1"" of type '" "struct _dwg_AcDbMTextObjectEmbedded *""'"); 
    }
    arg1 = (struct _dwg_AcDbMTextObjectEmbedded *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDbMTextObjectEmbedded_column_width_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->column_width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDbMTextObjectEmbedded_column_width_get) {
  {
    struct _dwg_AcDbMTextObjectEmbedded *arg1 = (struct _dwg_AcDbMTextObjectEmbedded *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDbMTextObjectEmbedded_column_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDbMTextObjectEmbedded_column_width_get" "', argument " "1"" of type '" "struct _dwg_AcDbMTextObjectEmbedded *""'"); 
    }
    arg1 = (struct _dwg_AcDbMTextObjectEmbedded *)(argp1);
    result = (BITCODE_BD) ((arg1)->column_width);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDbMTextObjectEmbedded_gutter_set) {
  {
    struct _dwg_AcDbMTextObjectEmbedded *arg1 = (struct _dwg_AcDbMTextObjectEmbedded *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDbMTextObjectEmbedded_gutter_set(self,gutter);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDbMTextObjectEmbedded_gutter_set" "', argument " "1"" of type '" "struct _dwg_AcDbMTextObjectEmbedded *""'"); 
    }
    arg1 = (struct _dwg_AcDbMTextObjectEmbedded *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDbMTextObjectEmbedded_gutter_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->gutter = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDbMTextObjectEmbedded_gutter_get) {
  {
    struct _dwg_AcDbMTextObjectEmbedded *arg1 = (struct _dwg_AcDbMTextObjectEmbedded *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDbMTextObjectEmbedded_gutter_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDbMTextObjectEmbedded_gutter_get" "', argument " "1"" of type '" "struct _dwg_AcDbMTextObjectEmbedded *""'"); 
    }
    arg1 = (struct _dwg_AcDbMTextObjectEmbedded *)(argp1);
    result = (BITCODE_BD) ((arg1)->gutter);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDbMTextObjectEmbedded_auto_height_set) {
  {
    struct _dwg_AcDbMTextObjectEmbedded *arg1 = (struct _dwg_AcDbMTextObjectEmbedded *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDbMTextObjectEmbedded_auto_height_set(self,auto_height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDbMTextObjectEmbedded_auto_height_set" "', argument " "1"" of type '" "struct _dwg_AcDbMTextObjectEmbedded *""'"); 
    }
    arg1 = (struct _dwg_AcDbMTextObjectEmbedded *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDbMTextObjectEmbedded_auto_height_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->auto_height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDbMTextObjectEmbedded_auto_height_get) {
  {
    struct _dwg_AcDbMTextObjectEmbedded *arg1 = (struct _dwg_AcDbMTextObjectEmbedded *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDbMTextObjectEmbedded_auto_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDbMTextObjectEmbedded_auto_height_get" "', argument " "1"" of type '" "struct _dwg_AcDbMTextObjectEmbedded *""'"); 
    }
    arg1 = (struct _dwg_AcDbMTextObjectEmbedded *)(argp1);
    result = (BITCODE_B) ((arg1)->auto_height);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDbMTextObjectEmbedded_flow_reversed_set) {
  {
    struct _dwg_AcDbMTextObjectEmbedded *arg1 = (struct _dwg_AcDbMTextObjectEmbedded *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDbMTextObjectEmbedded_flow_reversed_set(self,flow_reversed);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDbMTextObjectEmbedded_flow_reversed_set" "', argument " "1"" of type '" "struct _dwg_AcDbMTextObjectEmbedded *""'"); 
    }
    arg1 = (struct _dwg_AcDbMTextObjectEmbedded *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDbMTextObjectEmbedded_flow_reversed_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->flow_reversed = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDbMTextObjectEmbedded_flow_reversed_get) {
  {
    struct _dwg_AcDbMTextObjectEmbedded *arg1 = (struct _dwg_AcDbMTextObjectEmbedded *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDbMTextObjectEmbedded_flow_reversed_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDbMTextObjectEmbedded_flow_reversed_get" "', argument " "1"" of type '" "struct _dwg_AcDbMTextObjectEmbedded *""'"); 
    }
    arg1 = (struct _dwg_AcDbMTextObjectEmbedded *)(argp1);
    result = (BITCODE_B) ((arg1)->flow_reversed);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDbMTextObjectEmbedded_num_column_heights_set) {
  {
    struct _dwg_AcDbMTextObjectEmbedded *arg1 = (struct _dwg_AcDbMTextObjectEmbedded *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDbMTextObjectEmbedded_num_column_heights_set(self,num_column_heights);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDbMTextObjectEmbedded_num_column_heights_set" "', argument " "1"" of type '" "struct _dwg_AcDbMTextObjectEmbedded *""'"); 
    }
    arg1 = (struct _dwg_AcDbMTextObjectEmbedded *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDbMTextObjectEmbedded_num_column_heights_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_column_heights = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDbMTextObjectEmbedded_num_column_heights_get) {
  {
    struct _dwg_AcDbMTextObjectEmbedded *arg1 = (struct _dwg_AcDbMTextObjectEmbedded *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDbMTextObjectEmbedded_num_column_heights_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDbMTextObjectEmbedded_num_column_heights_get" "', argument " "1"" of type '" "struct _dwg_AcDbMTextObjectEmbedded *""'"); 
    }
    arg1 = (struct _dwg_AcDbMTextObjectEmbedded *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_column_heights);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDbMTextObjectEmbedded_column_heights_set) {
  {
    struct _dwg_AcDbMTextObjectEmbedded *arg1 = (struct _dwg_AcDbMTextObjectEmbedded *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDbMTextObjectEmbedded_column_heights_set(self,column_heights);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDbMTextObjectEmbedded_column_heights_set" "', argument " "1"" of type '" "struct _dwg_AcDbMTextObjectEmbedded *""'"); 
    }
    arg1 = (struct _dwg_AcDbMTextObjectEmbedded *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AcDbMTextObjectEmbedded_column_heights_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->column_heights = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDbMTextObjectEmbedded_column_heights_get) {
  {
    struct _dwg_AcDbMTextObjectEmbedded *arg1 = (struct _dwg_AcDbMTextObjectEmbedded *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDbMTextObjectEmbedded_column_heights_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDbMTextObjectEmbedded_column_heights_get" "', argument " "1"" of type '" "struct _dwg_AcDbMTextObjectEmbedded *""'"); 
    }
    arg1 = (struct _dwg_AcDbMTextObjectEmbedded *)(argp1);
    result = (BITCODE_BD *) ((arg1)->column_heights);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_AcDbMTextObjectEmbedded) {
  {
    int argvi = 0;
    struct _dwg_AcDbMTextObjectEmbedded *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_AcDbMTextObjectEmbedded();");
    }
    result = (struct _dwg_AcDbMTextObjectEmbedded *)calloc(1, sizeof(struct _dwg_AcDbMTextObjectEmbedded));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_AcDbMTextObjectEmbedded) {
  {
    struct _dwg_AcDbMTextObjectEmbedded *arg1 = (struct _dwg_AcDbMTextObjectEmbedded *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_AcDbMTextObjectEmbedded(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_AcDbMTextObjectEmbedded" "', argument " "1"" of type '" "struct _dwg_AcDbMTextObjectEmbedded *""'"); 
    }
    arg1 = (struct _dwg_AcDbMTextObjectEmbedded *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_parent_set) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ATTRIB_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_parent_get) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_elevation_set) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_elevation_set(self,elevation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_elevation_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTRIB_elevation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->elevation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_elevation_get) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_elevation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_elevation_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    result = (BITCODE_BD) ((arg1)->elevation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_ins_pt_set) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    BITCODE_2DPOINT *arg2 = (BITCODE_2DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_ins_pt_set(self,ins_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_ins_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ATTRIB_ins_pt_set" "', argument " "2"" of type '" "BITCODE_2DPOINT *""'"); 
    }
    arg2 = (BITCODE_2DPOINT *)(argp2);
    if (arg1) (arg1)->ins_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_ins_pt_get) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_ins_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_ins_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    result = (BITCODE_2DPOINT *)& ((arg1)->ins_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_alignment_pt_set) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    BITCODE_2DPOINT *arg2 = (BITCODE_2DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_alignment_pt_set(self,alignment_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_alignment_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ATTRIB_alignment_pt_set" "', argument " "2"" of type '" "BITCODE_2DPOINT *""'"); 
    }
    arg2 = (BITCODE_2DPOINT *)(argp2);
    if (arg1) (arg1)->alignment_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_alignment_pt_get) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_alignment_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_alignment_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    result = (BITCODE_2DPOINT *)& ((arg1)->alignment_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_extrusion_set) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_extrusion_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ATTRIB_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_extrusion_get) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_extrusion_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_thickness_set) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    BITCODE_RD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_thickness_set(self,thickness);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_thickness_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTRIB_thickness_set" "', argument " "2"" of type '" "BITCODE_RD""'");
    } 
    arg2 = (BITCODE_RD)(val2);
    if (arg1) (arg1)->thickness = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_thickness_get) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_thickness_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_thickness_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    result = (BITCODE_RD) ((arg1)->thickness);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_oblique_angle_set) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    BITCODE_RD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_oblique_angle_set(self,oblique_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_oblique_angle_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTRIB_oblique_angle_set" "', argument " "2"" of type '" "BITCODE_RD""'");
    } 
    arg2 = (BITCODE_RD)(val2);
    if (arg1) (arg1)->oblique_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_oblique_angle_get) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_oblique_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_oblique_angle_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    result = (BITCODE_RD) ((arg1)->oblique_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_rotation_set) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    BITCODE_RD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_rotation_set(self,rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_rotation_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTRIB_rotation_set" "', argument " "2"" of type '" "BITCODE_RD""'");
    } 
    arg2 = (BITCODE_RD)(val2);
    if (arg1) (arg1)->rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_rotation_get) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_rotation_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    result = (BITCODE_RD) ((arg1)->rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_height_set) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    BITCODE_RD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_height_set(self,height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_height_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTRIB_height_set" "', argument " "2"" of type '" "BITCODE_RD""'");
    } 
    arg2 = (BITCODE_RD)(val2);
    if (arg1) (arg1)->height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_height_get) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_height_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    result = (BITCODE_RD) ((arg1)->height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_width_factor_set) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    BITCODE_RD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_width_factor_set(self,width_factor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_width_factor_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTRIB_width_factor_set" "', argument " "2"" of type '" "BITCODE_RD""'");
    } 
    arg2 = (BITCODE_RD)(val2);
    if (arg1) (arg1)->width_factor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_width_factor_get) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_width_factor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_width_factor_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    result = (BITCODE_RD) ((arg1)->width_factor);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_text_value_set) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_text_value_set(self,text_value);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_text_value_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ATTRIB_text_value_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->text_value);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->text_value = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->text_value = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_text_value_get) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_text_value_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_text_value_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    result = (BITCODE_TV) ((arg1)->text_value);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_generation_set) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_generation_set(self,generation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_generation_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTRIB_generation_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->generation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_generation_get) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_generation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_generation_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    result = (BITCODE_BS) ((arg1)->generation);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_horiz_alignment_set) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_horiz_alignment_set(self,horiz_alignment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_horiz_alignment_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTRIB_horiz_alignment_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->horiz_alignment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_horiz_alignment_get) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_horiz_alignment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_horiz_alignment_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    result = (BITCODE_BS) ((arg1)->horiz_alignment);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_vert_alignment_set) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_vert_alignment_set(self,vert_alignment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_vert_alignment_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTRIB_vert_alignment_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->vert_alignment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_vert_alignment_get) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_vert_alignment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_vert_alignment_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    result = (BITCODE_BS) ((arg1)->vert_alignment);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_dataflags_set) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_dataflags_set(self,dataflags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_dataflags_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTRIB_dataflags_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->dataflags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_dataflags_get) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_dataflags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_dataflags_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    result = (BITCODE_RC) ((arg1)->dataflags);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_is_locked_in_block_set) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_is_locked_in_block_set(self,is_locked_in_block);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_is_locked_in_block_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTRIB_is_locked_in_block_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->is_locked_in_block = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_is_locked_in_block_get) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_is_locked_in_block_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_is_locked_in_block_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    result = (BITCODE_RC) ((arg1)->is_locked_in_block);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_keep_duplicate_records_set) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_keep_duplicate_records_set(self,keep_duplicate_records);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_keep_duplicate_records_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTRIB_keep_duplicate_records_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->keep_duplicate_records = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_keep_duplicate_records_get) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_keep_duplicate_records_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_keep_duplicate_records_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    result = (BITCODE_RC) ((arg1)->keep_duplicate_records);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_tag_set) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_tag_set(self,tag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_tag_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ATTRIB_tag_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->tag);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->tag = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->tag = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_tag_get) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_tag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_tag_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    result = (BITCODE_TV) ((arg1)->tag);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_field_length_set) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_field_length_set(self,field_length);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_field_length_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTRIB_field_length_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->field_length = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_field_length_get) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_field_length_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_field_length_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    result = (BITCODE_BS) ((arg1)->field_length);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_flags_set) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_flags_set(self,flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_flags_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTRIB_flags_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_flags_get) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_flags_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    result = (BITCODE_RC) ((arg1)->flags);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_lock_position_flag_set) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_lock_position_flag_set(self,lock_position_flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_lock_position_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTRIB_lock_position_flag_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->lock_position_flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_lock_position_flag_get) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_lock_position_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_lock_position_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    result = (BITCODE_B) ((arg1)->lock_position_flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_style_set) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_style_set(self,style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_style_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ATTRIB_style_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_style_get) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_style_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    result = (BITCODE_H) ((arg1)->style);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_mtext_type_set) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_mtext_type_set(self,mtext_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_mtext_type_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTRIB_mtext_type_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->mtext_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_mtext_type_get) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_mtext_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_mtext_type_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    result = (BITCODE_RC) ((arg1)->mtext_type);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_mtext_set) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    Dwg_AcDbMTextObjectEmbedded *arg2 = (Dwg_AcDbMTextObjectEmbedded *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_mtext_set(self,mtext);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_mtext_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ATTRIB_mtext_set" "', argument " "2"" of type '" "Dwg_AcDbMTextObjectEmbedded *""'"); 
    }
    arg2 = (Dwg_AcDbMTextObjectEmbedded *)(argp2);
    if (arg1) (arg1)->mtext = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_mtext_get) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_AcDbMTextObjectEmbedded *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_mtext_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_mtext_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    result = (Dwg_AcDbMTextObjectEmbedded *)& ((arg1)->mtext);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_is_really_locked_set) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_is_really_locked_set(self,is_really_locked);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_is_really_locked_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTRIB_is_really_locked_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_really_locked = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_is_really_locked_get) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_is_really_locked_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_is_really_locked_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    result = (BITCODE_B) ((arg1)->is_really_locked);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_annotative_data_size_set) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_annotative_data_size_set(self,annotative_data_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_annotative_data_size_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTRIB_annotative_data_size_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->annotative_data_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_annotative_data_size_get) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_annotative_data_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_annotative_data_size_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    result = (BITCODE_BS) ((arg1)->annotative_data_size);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_annotative_data_set) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_annotative_data_set(self,annotative_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_annotative_data_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ATTRIB_annotative_data_set" "', argument " "2"" of type '" "BITCODE_RC *""'"); 
    }
    arg2 = (BITCODE_RC *)(argp2);
    if (arg1) (arg1)->annotative_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_annotative_data_get) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_annotative_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_annotative_data_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    result = (BITCODE_RC *) ((arg1)->annotative_data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_annotative_flag_set) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_annotative_flag_set(self,annotative_flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_annotative_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTRIB_annotative_flag_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->annotative_flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_annotative_flag_get) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_annotative_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_annotative_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    result = (BITCODE_BS) ((arg1)->annotative_flag);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_annotative_style_set) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_annotative_style_set(self,annotative_style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_annotative_style_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ATTRIB_annotative_style_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->annotative_style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTRIB_annotative_style_get) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTRIB_annotative_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTRIB_annotative_style_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    result = (BITCODE_H) ((arg1)->annotative_style);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_ATTRIB) {
  {
    int argvi = 0;
    struct _dwg_entity_ATTRIB *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_ATTRIB();");
    }
    result = (struct _dwg_entity_ATTRIB *)calloc(1, sizeof(struct _dwg_entity_ATTRIB));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_ATTRIB, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_ATTRIB) {
  {
    struct _dwg_entity_ATTRIB *arg1 = (struct _dwg_entity_ATTRIB *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_ATTRIB(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTRIB, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_ATTRIB" "', argument " "1"" of type '" "struct _dwg_entity_ATTRIB *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTRIB *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_parent_set) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ATTDEF_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_parent_get) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_elevation_set) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_elevation_set(self,elevation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_elevation_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTDEF_elevation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->elevation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_elevation_get) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_elevation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_elevation_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    result = (BITCODE_BD) ((arg1)->elevation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_ins_pt_set) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    BITCODE_2DPOINT *arg2 = (BITCODE_2DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_ins_pt_set(self,ins_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_ins_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ATTDEF_ins_pt_set" "', argument " "2"" of type '" "BITCODE_2DPOINT *""'"); 
    }
    arg2 = (BITCODE_2DPOINT *)(argp2);
    if (arg1) (arg1)->ins_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_ins_pt_get) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_ins_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_ins_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    result = (BITCODE_2DPOINT *)& ((arg1)->ins_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_alignment_pt_set) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    BITCODE_2DPOINT *arg2 = (BITCODE_2DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_alignment_pt_set(self,alignment_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_alignment_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ATTDEF_alignment_pt_set" "', argument " "2"" of type '" "BITCODE_2DPOINT *""'"); 
    }
    arg2 = (BITCODE_2DPOINT *)(argp2);
    if (arg1) (arg1)->alignment_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_alignment_pt_get) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_alignment_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_alignment_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    result = (BITCODE_2DPOINT *)& ((arg1)->alignment_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_extrusion_set) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_extrusion_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ATTDEF_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_extrusion_get) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_extrusion_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_thickness_set) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    BITCODE_RD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_thickness_set(self,thickness);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_thickness_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTDEF_thickness_set" "', argument " "2"" of type '" "BITCODE_RD""'");
    } 
    arg2 = (BITCODE_RD)(val2);
    if (arg1) (arg1)->thickness = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_thickness_get) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_thickness_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_thickness_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    result = (BITCODE_RD) ((arg1)->thickness);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_oblique_angle_set) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    BITCODE_RD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_oblique_angle_set(self,oblique_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_oblique_angle_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTDEF_oblique_angle_set" "', argument " "2"" of type '" "BITCODE_RD""'");
    } 
    arg2 = (BITCODE_RD)(val2);
    if (arg1) (arg1)->oblique_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_oblique_angle_get) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_oblique_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_oblique_angle_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    result = (BITCODE_RD) ((arg1)->oblique_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_rotation_set) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    BITCODE_RD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_rotation_set(self,rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_rotation_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTDEF_rotation_set" "', argument " "2"" of type '" "BITCODE_RD""'");
    } 
    arg2 = (BITCODE_RD)(val2);
    if (arg1) (arg1)->rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_rotation_get) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_rotation_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    result = (BITCODE_RD) ((arg1)->rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_height_set) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    BITCODE_RD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_height_set(self,height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_height_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTDEF_height_set" "', argument " "2"" of type '" "BITCODE_RD""'");
    } 
    arg2 = (BITCODE_RD)(val2);
    if (arg1) (arg1)->height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_height_get) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_height_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    result = (BITCODE_RD) ((arg1)->height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_width_factor_set) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    BITCODE_RD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_width_factor_set(self,width_factor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_width_factor_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTDEF_width_factor_set" "', argument " "2"" of type '" "BITCODE_RD""'");
    } 
    arg2 = (BITCODE_RD)(val2);
    if (arg1) (arg1)->width_factor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_width_factor_get) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_width_factor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_width_factor_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    result = (BITCODE_RD) ((arg1)->width_factor);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_default_value_set) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_default_value_set(self,default_value);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_default_value_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ATTDEF_default_value_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->default_value);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->default_value = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->default_value = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_default_value_get) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_default_value_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_default_value_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    result = (BITCODE_TV) ((arg1)->default_value);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_generation_set) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_generation_set(self,generation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_generation_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTDEF_generation_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->generation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_generation_get) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_generation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_generation_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    result = (BITCODE_BS) ((arg1)->generation);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_horiz_alignment_set) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_horiz_alignment_set(self,horiz_alignment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_horiz_alignment_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTDEF_horiz_alignment_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->horiz_alignment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_horiz_alignment_get) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_horiz_alignment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_horiz_alignment_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    result = (BITCODE_BS) ((arg1)->horiz_alignment);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_vert_alignment_set) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_vert_alignment_set(self,vert_alignment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_vert_alignment_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTDEF_vert_alignment_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->vert_alignment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_vert_alignment_get) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_vert_alignment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_vert_alignment_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    result = (BITCODE_BS) ((arg1)->vert_alignment);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_dataflags_set) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_dataflags_set(self,dataflags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_dataflags_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTDEF_dataflags_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->dataflags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_dataflags_get) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_dataflags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_dataflags_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    result = (BITCODE_RC) ((arg1)->dataflags);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_is_locked_in_block_set) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_is_locked_in_block_set(self,is_locked_in_block);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_is_locked_in_block_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTDEF_is_locked_in_block_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->is_locked_in_block = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_is_locked_in_block_get) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_is_locked_in_block_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_is_locked_in_block_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    result = (BITCODE_RC) ((arg1)->is_locked_in_block);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_keep_duplicate_records_set) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_keep_duplicate_records_set(self,keep_duplicate_records);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_keep_duplicate_records_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTDEF_keep_duplicate_records_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->keep_duplicate_records = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_keep_duplicate_records_get) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_keep_duplicate_records_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_keep_duplicate_records_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    result = (BITCODE_RC) ((arg1)->keep_duplicate_records);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_tag_set) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_tag_set(self,tag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_tag_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ATTDEF_tag_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->tag);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->tag = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->tag = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_tag_get) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_tag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_tag_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    result = (BITCODE_TV) ((arg1)->tag);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_field_length_set) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_field_length_set(self,field_length);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_field_length_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTDEF_field_length_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->field_length = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_field_length_get) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_field_length_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_field_length_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    result = (BITCODE_BS) ((arg1)->field_length);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_flags_set) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_flags_set(self,flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_flags_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTDEF_flags_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_flags_get) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_flags_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    result = (BITCODE_RC) ((arg1)->flags);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_lock_position_flag_set) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_lock_position_flag_set(self,lock_position_flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_lock_position_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTDEF_lock_position_flag_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->lock_position_flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_lock_position_flag_get) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_lock_position_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_lock_position_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    result = (BITCODE_B) ((arg1)->lock_position_flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_style_set) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_style_set(self,style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_style_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ATTDEF_style_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_style_get) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_style_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    result = (BITCODE_H) ((arg1)->style);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_mtext_type_set) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_mtext_type_set(self,mtext_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_mtext_type_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTDEF_mtext_type_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->mtext_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_mtext_type_get) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_mtext_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_mtext_type_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    result = (BITCODE_RC) ((arg1)->mtext_type);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_mtext_set) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    Dwg_AcDbMTextObjectEmbedded *arg2 = (Dwg_AcDbMTextObjectEmbedded *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_mtext_set(self,mtext);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_mtext_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ATTDEF_mtext_set" "', argument " "2"" of type '" "Dwg_AcDbMTextObjectEmbedded *""'"); 
    }
    arg2 = (Dwg_AcDbMTextObjectEmbedded *)(argp2);
    if (arg1) (arg1)->mtext = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_mtext_get) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_AcDbMTextObjectEmbedded *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_mtext_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_mtext_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    result = (Dwg_AcDbMTextObjectEmbedded *)& ((arg1)->mtext);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_is_really_locked_set) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_is_really_locked_set(self,is_really_locked);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_is_really_locked_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTDEF_is_really_locked_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_really_locked = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_is_really_locked_get) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_is_really_locked_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_is_really_locked_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    result = (BITCODE_B) ((arg1)->is_really_locked);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_annotative_data_size_set) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_annotative_data_size_set(self,annotative_data_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_annotative_data_size_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTDEF_annotative_data_size_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->annotative_data_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_annotative_data_size_get) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_annotative_data_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_annotative_data_size_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    result = (BITCODE_BS) ((arg1)->annotative_data_size);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_annotative_data_set) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_annotative_data_set(self,annotative_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_annotative_data_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ATTDEF_annotative_data_set" "', argument " "2"" of type '" "BITCODE_RC *""'"); 
    }
    arg2 = (BITCODE_RC *)(argp2);
    if (arg1) (arg1)->annotative_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_annotative_data_get) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_annotative_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_annotative_data_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    result = (BITCODE_RC *) ((arg1)->annotative_data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_annotative_flag_set) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_annotative_flag_set(self,annotative_flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_annotative_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ATTDEF_annotative_flag_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->annotative_flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_annotative_flag_get) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_annotative_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_annotative_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    result = (BITCODE_BS) ((arg1)->annotative_flag);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_annotative_style_set) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_annotative_style_set(self,annotative_style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_annotative_style_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ATTDEF_annotative_style_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->annotative_style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_annotative_style_get) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_annotative_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_annotative_style_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    result = (BITCODE_H) ((arg1)->annotative_style);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_prompt_set) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_prompt_set(self,prompt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_prompt_set" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ATTDEF_prompt_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->prompt);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->prompt = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->prompt = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ATTDEF_prompt_get) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ATTDEF_prompt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ATTDEF_prompt_get" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    result = (BITCODE_TV) ((arg1)->prompt);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_ATTDEF) {
  {
    int argvi = 0;
    struct _dwg_entity_ATTDEF *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_ATTDEF();");
    }
    result = (struct _dwg_entity_ATTDEF *)calloc(1, sizeof(struct _dwg_entity_ATTDEF));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_ATTDEF, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_ATTDEF) {
  {
    struct _dwg_entity_ATTDEF *arg1 = (struct _dwg_entity_ATTDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_ATTDEF(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ATTDEF, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_ATTDEF" "', argument " "1"" of type '" "struct _dwg_entity_ATTDEF *""'"); 
    }
    arg1 = (struct _dwg_entity_ATTDEF *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_BLOCK_parent_set) {
  {
    struct _dwg_entity_BLOCK *arg1 = (struct _dwg_entity_BLOCK *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_BLOCK_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_BLOCK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_BLOCK_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_BLOCK *""'"); 
    }
    arg1 = (struct _dwg_entity_BLOCK *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_BLOCK_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_BLOCK_parent_get) {
  {
    struct _dwg_entity_BLOCK *arg1 = (struct _dwg_entity_BLOCK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_BLOCK_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_BLOCK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_BLOCK_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_BLOCK *""'"); 
    }
    arg1 = (struct _dwg_entity_BLOCK *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_BLOCK_name_set) {
  {
    struct _dwg_entity_BLOCK *arg1 = (struct _dwg_entity_BLOCK *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_BLOCK_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_BLOCK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_BLOCK_name_set" "', argument " "1"" of type '" "struct _dwg_entity_BLOCK *""'"); 
    }
    arg1 = (struct _dwg_entity_BLOCK *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_BLOCK_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_BLOCK_name_get) {
  {
    struct _dwg_entity_BLOCK *arg1 = (struct _dwg_entity_BLOCK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_BLOCK_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_BLOCK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_BLOCK_name_get" "', argument " "1"" of type '" "struct _dwg_entity_BLOCK *""'"); 
    }
    arg1 = (struct _dwg_entity_BLOCK *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_BLOCK_xref_pname_set) {
  {
    struct _dwg_entity_BLOCK *arg1 = (struct _dwg_entity_BLOCK *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_BLOCK_xref_pname_set(self,xref_pname);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_BLOCK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_BLOCK_xref_pname_set" "', argument " "1"" of type '" "struct _dwg_entity_BLOCK *""'"); 
    }
    arg1 = (struct _dwg_entity_BLOCK *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_BLOCK_xref_pname_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->xref_pname);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->xref_pname = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->xref_pname = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_BLOCK_xref_pname_get) {
  {
    struct _dwg_entity_BLOCK *arg1 = (struct _dwg_entity_BLOCK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_BLOCK_xref_pname_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_BLOCK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_BLOCK_xref_pname_get" "', argument " "1"" of type '" "struct _dwg_entity_BLOCK *""'"); 
    }
    arg1 = (struct _dwg_entity_BLOCK *)(argp1);
    result = (BITCODE_TV) ((arg1)->xref_pname);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_BLOCK_base_pt_set) {
  {
    struct _dwg_entity_BLOCK *arg1 = (struct _dwg_entity_BLOCK *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_BLOCK_base_pt_set(self,base_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_BLOCK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_BLOCK_base_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_BLOCK *""'"); 
    }
    arg1 = (struct _dwg_entity_BLOCK *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_BLOCK_base_pt_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->base_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_BLOCK_base_pt_get) {
  {
    struct _dwg_entity_BLOCK *arg1 = (struct _dwg_entity_BLOCK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_BLOCK_base_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_BLOCK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_BLOCK_base_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_BLOCK *""'"); 
    }
    arg1 = (struct _dwg_entity_BLOCK *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->base_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_BLOCK) {
  {
    int argvi = 0;
    struct _dwg_entity_BLOCK *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_BLOCK();");
    }
    result = (struct _dwg_entity_BLOCK *)calloc(1, sizeof(struct _dwg_entity_BLOCK));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_BLOCK, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_BLOCK) {
  {
    struct _dwg_entity_BLOCK *arg1 = (struct _dwg_entity_BLOCK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_BLOCK(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_BLOCK, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_BLOCK" "', argument " "1"" of type '" "struct _dwg_entity_BLOCK *""'"); 
    }
    arg1 = (struct _dwg_entity_BLOCK *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ENDBLK_parent_set) {
  {
    struct _dwg_entity_ENDBLK *arg1 = (struct _dwg_entity_ENDBLK *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ENDBLK_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ENDBLK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ENDBLK_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_ENDBLK *""'"); 
    }
    arg1 = (struct _dwg_entity_ENDBLK *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ENDBLK_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ENDBLK_parent_get) {
  {
    struct _dwg_entity_ENDBLK *arg1 = (struct _dwg_entity_ENDBLK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ENDBLK_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ENDBLK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ENDBLK_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_ENDBLK *""'"); 
    }
    arg1 = (struct _dwg_entity_ENDBLK *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_ENDBLK) {
  {
    int argvi = 0;
    struct _dwg_entity_ENDBLK *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_ENDBLK();");
    }
    result = (struct _dwg_entity_ENDBLK *)calloc(1, sizeof(struct _dwg_entity_ENDBLK));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_ENDBLK, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_ENDBLK) {
  {
    struct _dwg_entity_ENDBLK *arg1 = (struct _dwg_entity_ENDBLK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_ENDBLK(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ENDBLK, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_ENDBLK" "', argument " "1"" of type '" "struct _dwg_entity_ENDBLK *""'"); 
    }
    arg1 = (struct _dwg_entity_ENDBLK *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SEQEND_parent_set) {
  {
    struct _dwg_entity_SEQEND *arg1 = (struct _dwg_entity_SEQEND *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SEQEND_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SEQEND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SEQEND_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_SEQEND *""'"); 
    }
    arg1 = (struct _dwg_entity_SEQEND *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SEQEND_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SEQEND_parent_get) {
  {
    struct _dwg_entity_SEQEND *arg1 = (struct _dwg_entity_SEQEND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SEQEND_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SEQEND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SEQEND_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_SEQEND *""'"); 
    }
    arg1 = (struct _dwg_entity_SEQEND *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SEQEND_begin_addr_r11_set) {
  {
    struct _dwg_entity_SEQEND *arg1 = (struct _dwg_entity_SEQEND *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SEQEND_begin_addr_r11_set(self,begin_addr_r11);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SEQEND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SEQEND_begin_addr_r11_set" "', argument " "1"" of type '" "struct _dwg_entity_SEQEND *""'"); 
    }
    arg1 = (struct _dwg_entity_SEQEND *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SEQEND_begin_addr_r11_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->begin_addr_r11 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SEQEND_begin_addr_r11_get) {
  {
    struct _dwg_entity_SEQEND *arg1 = (struct _dwg_entity_SEQEND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SEQEND_begin_addr_r11_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SEQEND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SEQEND_begin_addr_r11_get" "', argument " "1"" of type '" "struct _dwg_entity_SEQEND *""'"); 
    }
    arg1 = (struct _dwg_entity_SEQEND *)(argp1);
    result = (BITCODE_RL) ((arg1)->begin_addr_r11);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_SEQEND) {
  {
    int argvi = 0;
    struct _dwg_entity_SEQEND *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_SEQEND();");
    }
    result = (struct _dwg_entity_SEQEND *)calloc(1, sizeof(struct _dwg_entity_SEQEND));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_SEQEND, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_SEQEND) {
  {
    struct _dwg_entity_SEQEND *arg1 = (struct _dwg_entity_SEQEND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_SEQEND(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SEQEND, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_SEQEND" "', argument " "1"" of type '" "struct _dwg_entity_SEQEND *""'"); 
    }
    arg1 = (struct _dwg_entity_SEQEND *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_parent_set) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_INSERT_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_parent_get) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_ins_pt_set) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    BITCODE_3DPOINT *arg2 = (BITCODE_3DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_ins_pt_set(self,ins_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_ins_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_INSERT_ins_pt_set" "', argument " "2"" of type '" "BITCODE_3DPOINT *""'"); 
    }
    arg2 = (BITCODE_3DPOINT *)(argp2);
    if (arg1) (arg1)->ins_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_ins_pt_get) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_ins_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_ins_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    result = (BITCODE_3DPOINT *)& ((arg1)->ins_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_scale_flag_set) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    BITCODE_BB arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_scale_flag_set(self,scale_flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_scale_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_INSERT_scale_flag_set" "', argument " "2"" of type '" "BITCODE_BB""'");
    } 
    arg2 = (BITCODE_BB)(val2);
    if (arg1) (arg1)->scale_flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_scale_flag_get) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BB result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_scale_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_scale_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    result = (BITCODE_BB) ((arg1)->scale_flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_scale_set) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    BITCODE_3DPOINT *arg2 = (BITCODE_3DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_scale_set(self,scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_scale_set" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_INSERT_scale_set" "', argument " "2"" of type '" "BITCODE_3DPOINT *""'"); 
    }
    arg2 = (BITCODE_3DPOINT *)(argp2);
    if (arg1) (arg1)->scale = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_scale_get) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_scale_get" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    result = (BITCODE_3DPOINT *)& ((arg1)->scale);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_rotation_set) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_rotation_set(self,rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_rotation_set" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_INSERT_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_rotation_get) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_rotation_get" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    result = (BITCODE_BD) ((arg1)->rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_extrusion_set) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_extrusion_set" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_INSERT_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_extrusion_get) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_extrusion_get" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_has_attribs_set) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_has_attribs_set(self,has_attribs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_has_attribs_set" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_INSERT_has_attribs_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_attribs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_has_attribs_get) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_has_attribs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_has_attribs_get" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    result = (BITCODE_B) ((arg1)->has_attribs);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_num_owned_set) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_num_owned_set(self,num_owned);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_num_owned_set" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_INSERT_num_owned_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_owned = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_num_owned_get) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_num_owned_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_num_owned_get" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_owned);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_block_header_set) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_block_header_set(self,block_header);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_block_header_set" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_INSERT_block_header_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->block_header = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_block_header_get) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_block_header_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_block_header_get" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    result = (BITCODE_H) ((arg1)->block_header);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_first_attrib_set) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_first_attrib_set(self,first_attrib);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_first_attrib_set" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_INSERT_first_attrib_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->first_attrib = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_first_attrib_get) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_first_attrib_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_first_attrib_get" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    result = (BITCODE_H) ((arg1)->first_attrib);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_last_attrib_set) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_last_attrib_set(self,last_attrib);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_last_attrib_set" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_INSERT_last_attrib_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->last_attrib = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_last_attrib_get) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_last_attrib_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_last_attrib_get" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    result = (BITCODE_H) ((arg1)->last_attrib);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_attribs_set) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_attribs_set(self,attribs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_attribs_set" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_INSERT_attribs_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->attribs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_attribs_get) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_attribs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_attribs_get" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    result = (BITCODE_H *) ((arg1)->attribs);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_seqend_set) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_seqend_set(self,seqend);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_seqend_set" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_INSERT_seqend_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->seqend = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_seqend_get) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_seqend_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_seqend_get" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    result = (BITCODE_H) ((arg1)->seqend);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_num_cols_set) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_num_cols_set(self,num_cols);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_num_cols_set" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_INSERT_num_cols_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->num_cols = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_num_cols_get) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_num_cols_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_num_cols_get" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    result = (BITCODE_RS) ((arg1)->num_cols);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_num_rows_set) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_num_rows_set(self,num_rows);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_num_rows_set" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_INSERT_num_rows_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->num_rows = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_num_rows_get) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_num_rows_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_num_rows_get" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    result = (BITCODE_RS) ((arg1)->num_rows);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_col_spacing_set) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    BITCODE_RD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_col_spacing_set(self,col_spacing);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_col_spacing_set" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_INSERT_col_spacing_set" "', argument " "2"" of type '" "BITCODE_RD""'");
    } 
    arg2 = (BITCODE_RD)(val2);
    if (arg1) (arg1)->col_spacing = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_col_spacing_get) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_col_spacing_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_col_spacing_get" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    result = (BITCODE_RD) ((arg1)->col_spacing);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_row_spacing_set) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    BITCODE_RD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_row_spacing_set(self,row_spacing);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_row_spacing_set" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_INSERT_row_spacing_set" "', argument " "2"" of type '" "BITCODE_RD""'");
    } 
    arg2 = (BITCODE_RD)(val2);
    if (arg1) (arg1)->row_spacing = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_row_spacing_get) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_row_spacing_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_row_spacing_get" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    result = (BITCODE_RD) ((arg1)->row_spacing);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_block_name_set) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_block_name_set(self,block_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_block_name_set" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_INSERT_block_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->block_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->block_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->block_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_INSERT_block_name_get) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_INSERT_block_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_INSERT_block_name_get" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    result = (BITCODE_TV) ((arg1)->block_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_INSERT) {
  {
    int argvi = 0;
    struct _dwg_entity_INSERT *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_INSERT();");
    }
    result = (struct _dwg_entity_INSERT *)calloc(1, sizeof(struct _dwg_entity_INSERT));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_INSERT, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_INSERT) {
  {
    struct _dwg_entity_INSERT *arg1 = (struct _dwg_entity_INSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_INSERT(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_INSERT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_INSERT" "', argument " "1"" of type '" "struct _dwg_entity_INSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_INSERT *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MINSERT_parent_set) {
  {
    struct _dwg_entity_MINSERT *arg1 = (struct _dwg_entity_MINSERT *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MINSERT_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MINSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MINSERT_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_MINSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_MINSERT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MINSERT_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MINSERT_parent_get) {
  {
    struct _dwg_entity_MINSERT *arg1 = (struct _dwg_entity_MINSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MINSERT_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MINSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MINSERT_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_MINSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_MINSERT *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MINSERT_ins_pt_set) {
  {
    struct _dwg_entity_MINSERT *arg1 = (struct _dwg_entity_MINSERT *) 0 ;
    BITCODE_3DPOINT *arg2 = (BITCODE_3DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MINSERT_ins_pt_set(self,ins_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MINSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MINSERT_ins_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_MINSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_MINSERT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MINSERT_ins_pt_set" "', argument " "2"" of type '" "BITCODE_3DPOINT *""'"); 
    }
    arg2 = (BITCODE_3DPOINT *)(argp2);
    if (arg1) (arg1)->ins_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MINSERT_ins_pt_get) {
  {
    struct _dwg_entity_MINSERT *arg1 = (struct _dwg_entity_MINSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MINSERT_ins_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MINSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MINSERT_ins_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_MINSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_MINSERT *)(argp1);
    result = (BITCODE_3DPOINT *)& ((arg1)->ins_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MINSERT_scale_flag_set) {
  {
    struct _dwg_entity_MINSERT *arg1 = (struct _dwg_entity_MINSERT *) 0 ;
    BITCODE_BB arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MINSERT_scale_flag_set(self,scale_flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MINSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MINSERT_scale_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_MINSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_MINSERT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MINSERT_scale_flag_set" "', argument " "2"" of type '" "BITCODE_BB""'");
    } 
    arg2 = (BITCODE_BB)(val2);
    if (arg1) (arg1)->scale_flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MINSERT_scale_flag_get) {
  {
    struct _dwg_entity_MINSERT *arg1 = (struct _dwg_entity_MINSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BB result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MINSERT_scale_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MINSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MINSERT_scale_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_MINSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_MINSERT *)(argp1);
    result = (BITCODE_BB) ((arg1)->scale_flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MINSERT_scale_set) {
  {
    struct _dwg_entity_MINSERT *arg1 = (struct _dwg_entity_MINSERT *) 0 ;
    BITCODE_3DPOINT *arg2 = (BITCODE_3DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MINSERT_scale_set(self,scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MINSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MINSERT_scale_set" "', argument " "1"" of type '" "struct _dwg_entity_MINSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_MINSERT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MINSERT_scale_set" "', argument " "2"" of type '" "BITCODE_3DPOINT *""'"); 
    }
    arg2 = (BITCODE_3DPOINT *)(argp2);
    if (arg1) (arg1)->scale = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MINSERT_scale_get) {
  {
    struct _dwg_entity_MINSERT *arg1 = (struct _dwg_entity_MINSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MINSERT_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MINSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MINSERT_scale_get" "', argument " "1"" of type '" "struct _dwg_entity_MINSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_MINSERT *)(argp1);
    result = (BITCODE_3DPOINT *)& ((arg1)->scale);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MINSERT_rotation_set) {
  {
    struct _dwg_entity_MINSERT *arg1 = (struct _dwg_entity_MINSERT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MINSERT_rotation_set(self,rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MINSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MINSERT_rotation_set" "', argument " "1"" of type '" "struct _dwg_entity_MINSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_MINSERT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MINSERT_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MINSERT_rotation_get) {
  {
    struct _dwg_entity_MINSERT *arg1 = (struct _dwg_entity_MINSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MINSERT_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MINSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MINSERT_rotation_get" "', argument " "1"" of type '" "struct _dwg_entity_MINSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_MINSERT *)(argp1);
    result = (BITCODE_BD) ((arg1)->rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MINSERT_extrusion_set) {
  {
    struct _dwg_entity_MINSERT *arg1 = (struct _dwg_entity_MINSERT *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MINSERT_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MINSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MINSERT_extrusion_set" "', argument " "1"" of type '" "struct _dwg_entity_MINSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_MINSERT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MINSERT_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MINSERT_extrusion_get) {
  {
    struct _dwg_entity_MINSERT *arg1 = (struct _dwg_entity_MINSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MINSERT_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MINSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MINSERT_extrusion_get" "', argument " "1"" of type '" "struct _dwg_entity_MINSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_MINSERT *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MINSERT_has_attribs_set) {
  {
    struct _dwg_entity_MINSERT *arg1 = (struct _dwg_entity_MINSERT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MINSERT_has_attribs_set(self,has_attribs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MINSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MINSERT_has_attribs_set" "', argument " "1"" of type '" "struct _dwg_entity_MINSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_MINSERT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MINSERT_has_attribs_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_attribs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MINSERT_has_attribs_get) {
  {
    struct _dwg_entity_MINSERT *arg1 = (struct _dwg_entity_MINSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MINSERT_has_attribs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MINSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MINSERT_has_attribs_get" "', argument " "1"" of type '" "struct _dwg_entity_MINSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_MINSERT *)(argp1);
    result = (BITCODE_B) ((arg1)->has_attribs);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MINSERT_num_owned_set) {
  {
    struct _dwg_entity_MINSERT *arg1 = (struct _dwg_entity_MINSERT *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MINSERT_num_owned_set(self,num_owned);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MINSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MINSERT_num_owned_set" "', argument " "1"" of type '" "struct _dwg_entity_MINSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_MINSERT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MINSERT_num_owned_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_owned = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MINSERT_num_owned_get) {
  {
    struct _dwg_entity_MINSERT *arg1 = (struct _dwg_entity_MINSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MINSERT_num_owned_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MINSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MINSERT_num_owned_get" "', argument " "1"" of type '" "struct _dwg_entity_MINSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_MINSERT *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_owned);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MINSERT_num_cols_set) {
  {
    struct _dwg_entity_MINSERT *arg1 = (struct _dwg_entity_MINSERT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MINSERT_num_cols_set(self,num_cols);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MINSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MINSERT_num_cols_set" "', argument " "1"" of type '" "struct _dwg_entity_MINSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_MINSERT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MINSERT_num_cols_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_cols = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MINSERT_num_cols_get) {
  {
    struct _dwg_entity_MINSERT *arg1 = (struct _dwg_entity_MINSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MINSERT_num_cols_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MINSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MINSERT_num_cols_get" "', argument " "1"" of type '" "struct _dwg_entity_MINSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_MINSERT *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_cols);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MINSERT_num_rows_set) {
  {
    struct _dwg_entity_MINSERT *arg1 = (struct _dwg_entity_MINSERT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MINSERT_num_rows_set(self,num_rows);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MINSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MINSERT_num_rows_set" "', argument " "1"" of type '" "struct _dwg_entity_MINSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_MINSERT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MINSERT_num_rows_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_rows = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MINSERT_num_rows_get) {
  {
    struct _dwg_entity_MINSERT *arg1 = (struct _dwg_entity_MINSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MINSERT_num_rows_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MINSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MINSERT_num_rows_get" "', argument " "1"" of type '" "struct _dwg_entity_MINSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_MINSERT *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_rows);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MINSERT_col_spacing_set) {
  {
    struct _dwg_entity_MINSERT *arg1 = (struct _dwg_entity_MINSERT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MINSERT_col_spacing_set(self,col_spacing);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MINSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MINSERT_col_spacing_set" "', argument " "1"" of type '" "struct _dwg_entity_MINSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_MINSERT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MINSERT_col_spacing_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->col_spacing = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MINSERT_col_spacing_get) {
  {
    struct _dwg_entity_MINSERT *arg1 = (struct _dwg_entity_MINSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MINSERT_col_spacing_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MINSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MINSERT_col_spacing_get" "', argument " "1"" of type '" "struct _dwg_entity_MINSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_MINSERT *)(argp1);
    result = (BITCODE_BD) ((arg1)->col_spacing);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MINSERT_row_spacing_set) {
  {
    struct _dwg_entity_MINSERT *arg1 = (struct _dwg_entity_MINSERT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MINSERT_row_spacing_set(self,row_spacing);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MINSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MINSERT_row_spacing_set" "', argument " "1"" of type '" "struct _dwg_entity_MINSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_MINSERT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MINSERT_row_spacing_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->row_spacing = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MINSERT_row_spacing_get) {
  {
    struct _dwg_entity_MINSERT *arg1 = (struct _dwg_entity_MINSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MINSERT_row_spacing_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MINSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MINSERT_row_spacing_get" "', argument " "1"" of type '" "struct _dwg_entity_MINSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_MINSERT *)(argp1);
    result = (BITCODE_BD) ((arg1)->row_spacing);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MINSERT_block_header_set) {
  {
    struct _dwg_entity_MINSERT *arg1 = (struct _dwg_entity_MINSERT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MINSERT_block_header_set(self,block_header);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MINSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MINSERT_block_header_set" "', argument " "1"" of type '" "struct _dwg_entity_MINSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_MINSERT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MINSERT_block_header_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->block_header = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MINSERT_block_header_get) {
  {
    struct _dwg_entity_MINSERT *arg1 = (struct _dwg_entity_MINSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MINSERT_block_header_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MINSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MINSERT_block_header_get" "', argument " "1"" of type '" "struct _dwg_entity_MINSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_MINSERT *)(argp1);
    result = (BITCODE_H) ((arg1)->block_header);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MINSERT_first_attrib_set) {
  {
    struct _dwg_entity_MINSERT *arg1 = (struct _dwg_entity_MINSERT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MINSERT_first_attrib_set(self,first_attrib);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MINSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MINSERT_first_attrib_set" "', argument " "1"" of type '" "struct _dwg_entity_MINSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_MINSERT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MINSERT_first_attrib_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->first_attrib = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MINSERT_first_attrib_get) {
  {
    struct _dwg_entity_MINSERT *arg1 = (struct _dwg_entity_MINSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MINSERT_first_attrib_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MINSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MINSERT_first_attrib_get" "', argument " "1"" of type '" "struct _dwg_entity_MINSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_MINSERT *)(argp1);
    result = (BITCODE_H) ((arg1)->first_attrib);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MINSERT_last_attrib_set) {
  {
    struct _dwg_entity_MINSERT *arg1 = (struct _dwg_entity_MINSERT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MINSERT_last_attrib_set(self,last_attrib);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MINSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MINSERT_last_attrib_set" "', argument " "1"" of type '" "struct _dwg_entity_MINSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_MINSERT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MINSERT_last_attrib_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->last_attrib = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MINSERT_last_attrib_get) {
  {
    struct _dwg_entity_MINSERT *arg1 = (struct _dwg_entity_MINSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MINSERT_last_attrib_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MINSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MINSERT_last_attrib_get" "', argument " "1"" of type '" "struct _dwg_entity_MINSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_MINSERT *)(argp1);
    result = (BITCODE_H) ((arg1)->last_attrib);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MINSERT_attribs_set) {
  {
    struct _dwg_entity_MINSERT *arg1 = (struct _dwg_entity_MINSERT *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MINSERT_attribs_set(self,attribs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MINSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MINSERT_attribs_set" "', argument " "1"" of type '" "struct _dwg_entity_MINSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_MINSERT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MINSERT_attribs_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->attribs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MINSERT_attribs_get) {
  {
    struct _dwg_entity_MINSERT *arg1 = (struct _dwg_entity_MINSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MINSERT_attribs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MINSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MINSERT_attribs_get" "', argument " "1"" of type '" "struct _dwg_entity_MINSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_MINSERT *)(argp1);
    result = (BITCODE_H *) ((arg1)->attribs);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MINSERT_seqend_set) {
  {
    struct _dwg_entity_MINSERT *arg1 = (struct _dwg_entity_MINSERT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MINSERT_seqend_set(self,seqend);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MINSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MINSERT_seqend_set" "', argument " "1"" of type '" "struct _dwg_entity_MINSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_MINSERT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MINSERT_seqend_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->seqend = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MINSERT_seqend_get) {
  {
    struct _dwg_entity_MINSERT *arg1 = (struct _dwg_entity_MINSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MINSERT_seqend_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MINSERT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MINSERT_seqend_get" "', argument " "1"" of type '" "struct _dwg_entity_MINSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_MINSERT *)(argp1);
    result = (BITCODE_H) ((arg1)->seqend);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_MINSERT) {
  {
    int argvi = 0;
    struct _dwg_entity_MINSERT *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_MINSERT();");
    }
    result = (struct _dwg_entity_MINSERT *)calloc(1, sizeof(struct _dwg_entity_MINSERT));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_MINSERT, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_MINSERT) {
  {
    struct _dwg_entity_MINSERT *arg1 = (struct _dwg_entity_MINSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_MINSERT(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MINSERT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_MINSERT" "', argument " "1"" of type '" "struct _dwg_entity_MINSERT *""'"); 
    }
    arg1 = (struct _dwg_entity_MINSERT *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VERTEX_2D_parent_set) {
  {
    struct _dwg_entity_VERTEX_2D *arg1 = (struct _dwg_entity_VERTEX_2D *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VERTEX_2D_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VERTEX_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VERTEX_2D_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_VERTEX_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_VERTEX_2D *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_VERTEX_2D_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VERTEX_2D_parent_get) {
  {
    struct _dwg_entity_VERTEX_2D *arg1 = (struct _dwg_entity_VERTEX_2D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VERTEX_2D_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VERTEX_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VERTEX_2D_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_VERTEX_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_VERTEX_2D *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VERTEX_2D_flag_set) {
  {
    struct _dwg_entity_VERTEX_2D *arg1 = (struct _dwg_entity_VERTEX_2D *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VERTEX_2D_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VERTEX_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VERTEX_2D_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_VERTEX_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_VERTEX_2D *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_VERTEX_2D_flag_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VERTEX_2D_flag_get) {
  {
    struct _dwg_entity_VERTEX_2D *arg1 = (struct _dwg_entity_VERTEX_2D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VERTEX_2D_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VERTEX_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VERTEX_2D_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_VERTEX_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_VERTEX_2D *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VERTEX_2D_point_set) {
  {
    struct _dwg_entity_VERTEX_2D *arg1 = (struct _dwg_entity_VERTEX_2D *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VERTEX_2D_point_set(self,point);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VERTEX_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VERTEX_2D_point_set" "', argument " "1"" of type '" "struct _dwg_entity_VERTEX_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_VERTEX_2D *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_VERTEX_2D_point_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->point = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VERTEX_2D_point_get) {
  {
    struct _dwg_entity_VERTEX_2D *arg1 = (struct _dwg_entity_VERTEX_2D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VERTEX_2D_point_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VERTEX_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VERTEX_2D_point_get" "', argument " "1"" of type '" "struct _dwg_entity_VERTEX_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_VERTEX_2D *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->point);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VERTEX_2D_start_width_set) {
  {
    struct _dwg_entity_VERTEX_2D *arg1 = (struct _dwg_entity_VERTEX_2D *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VERTEX_2D_start_width_set(self,start_width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VERTEX_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VERTEX_2D_start_width_set" "', argument " "1"" of type '" "struct _dwg_entity_VERTEX_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_VERTEX_2D *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_VERTEX_2D_start_width_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->start_width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VERTEX_2D_start_width_get) {
  {
    struct _dwg_entity_VERTEX_2D *arg1 = (struct _dwg_entity_VERTEX_2D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VERTEX_2D_start_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VERTEX_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VERTEX_2D_start_width_get" "', argument " "1"" of type '" "struct _dwg_entity_VERTEX_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_VERTEX_2D *)(argp1);
    result = (BITCODE_BD) ((arg1)->start_width);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VERTEX_2D_end_width_set) {
  {
    struct _dwg_entity_VERTEX_2D *arg1 = (struct _dwg_entity_VERTEX_2D *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VERTEX_2D_end_width_set(self,end_width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VERTEX_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VERTEX_2D_end_width_set" "', argument " "1"" of type '" "struct _dwg_entity_VERTEX_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_VERTEX_2D *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_VERTEX_2D_end_width_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->end_width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VERTEX_2D_end_width_get) {
  {
    struct _dwg_entity_VERTEX_2D *arg1 = (struct _dwg_entity_VERTEX_2D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VERTEX_2D_end_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VERTEX_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VERTEX_2D_end_width_get" "', argument " "1"" of type '" "struct _dwg_entity_VERTEX_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_VERTEX_2D *)(argp1);
    result = (BITCODE_BD) ((arg1)->end_width);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VERTEX_2D_id_set) {
  {
    struct _dwg_entity_VERTEX_2D *arg1 = (struct _dwg_entity_VERTEX_2D *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VERTEX_2D_id_set(self,id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VERTEX_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VERTEX_2D_id_set" "', argument " "1"" of type '" "struct _dwg_entity_VERTEX_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_VERTEX_2D *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_VERTEX_2D_id_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VERTEX_2D_id_get) {
  {
    struct _dwg_entity_VERTEX_2D *arg1 = (struct _dwg_entity_VERTEX_2D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VERTEX_2D_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VERTEX_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VERTEX_2D_id_get" "', argument " "1"" of type '" "struct _dwg_entity_VERTEX_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_VERTEX_2D *)(argp1);
    result = (BITCODE_BL) ((arg1)->id);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VERTEX_2D_bulge_set) {
  {
    struct _dwg_entity_VERTEX_2D *arg1 = (struct _dwg_entity_VERTEX_2D *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VERTEX_2D_bulge_set(self,bulge);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VERTEX_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VERTEX_2D_bulge_set" "', argument " "1"" of type '" "struct _dwg_entity_VERTEX_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_VERTEX_2D *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_VERTEX_2D_bulge_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->bulge = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VERTEX_2D_bulge_get) {
  {
    struct _dwg_entity_VERTEX_2D *arg1 = (struct _dwg_entity_VERTEX_2D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VERTEX_2D_bulge_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VERTEX_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VERTEX_2D_bulge_get" "', argument " "1"" of type '" "struct _dwg_entity_VERTEX_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_VERTEX_2D *)(argp1);
    result = (BITCODE_BD) ((arg1)->bulge);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VERTEX_2D_tangent_dir_set) {
  {
    struct _dwg_entity_VERTEX_2D *arg1 = (struct _dwg_entity_VERTEX_2D *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VERTEX_2D_tangent_dir_set(self,tangent_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VERTEX_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VERTEX_2D_tangent_dir_set" "', argument " "1"" of type '" "struct _dwg_entity_VERTEX_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_VERTEX_2D *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_VERTEX_2D_tangent_dir_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->tangent_dir = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VERTEX_2D_tangent_dir_get) {
  {
    struct _dwg_entity_VERTEX_2D *arg1 = (struct _dwg_entity_VERTEX_2D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VERTEX_2D_tangent_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VERTEX_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VERTEX_2D_tangent_dir_get" "', argument " "1"" of type '" "struct _dwg_entity_VERTEX_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_VERTEX_2D *)(argp1);
    result = (BITCODE_BD) ((arg1)->tangent_dir);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_VERTEX_2D) {
  {
    int argvi = 0;
    struct _dwg_entity_VERTEX_2D *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_VERTEX_2D();");
    }
    result = (struct _dwg_entity_VERTEX_2D *)calloc(1, sizeof(struct _dwg_entity_VERTEX_2D));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_VERTEX_2D, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_VERTEX_2D) {
  {
    struct _dwg_entity_VERTEX_2D *arg1 = (struct _dwg_entity_VERTEX_2D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_VERTEX_2D(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VERTEX_2D, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_VERTEX_2D" "', argument " "1"" of type '" "struct _dwg_entity_VERTEX_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_VERTEX_2D *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VERTEX_3D_parent_set) {
  {
    struct _dwg_entity_VERTEX_3D *arg1 = (struct _dwg_entity_VERTEX_3D *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VERTEX_3D_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VERTEX_3D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VERTEX_3D_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_VERTEX_3D *""'"); 
    }
    arg1 = (struct _dwg_entity_VERTEX_3D *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_VERTEX_3D_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VERTEX_3D_parent_get) {
  {
    struct _dwg_entity_VERTEX_3D *arg1 = (struct _dwg_entity_VERTEX_3D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VERTEX_3D_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VERTEX_3D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VERTEX_3D_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_VERTEX_3D *""'"); 
    }
    arg1 = (struct _dwg_entity_VERTEX_3D *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VERTEX_3D_flag_set) {
  {
    struct _dwg_entity_VERTEX_3D *arg1 = (struct _dwg_entity_VERTEX_3D *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VERTEX_3D_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VERTEX_3D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VERTEX_3D_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_VERTEX_3D *""'"); 
    }
    arg1 = (struct _dwg_entity_VERTEX_3D *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_VERTEX_3D_flag_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VERTEX_3D_flag_get) {
  {
    struct _dwg_entity_VERTEX_3D *arg1 = (struct _dwg_entity_VERTEX_3D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VERTEX_3D_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VERTEX_3D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VERTEX_3D_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_VERTEX_3D *""'"); 
    }
    arg1 = (struct _dwg_entity_VERTEX_3D *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VERTEX_3D_point_set) {
  {
    struct _dwg_entity_VERTEX_3D *arg1 = (struct _dwg_entity_VERTEX_3D *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VERTEX_3D_point_set(self,point);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VERTEX_3D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VERTEX_3D_point_set" "', argument " "1"" of type '" "struct _dwg_entity_VERTEX_3D *""'"); 
    }
    arg1 = (struct _dwg_entity_VERTEX_3D *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_VERTEX_3D_point_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->point = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VERTEX_3D_point_get) {
  {
    struct _dwg_entity_VERTEX_3D *arg1 = (struct _dwg_entity_VERTEX_3D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VERTEX_3D_point_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VERTEX_3D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VERTEX_3D_point_get" "', argument " "1"" of type '" "struct _dwg_entity_VERTEX_3D *""'"); 
    }
    arg1 = (struct _dwg_entity_VERTEX_3D *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->point);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_VERTEX_3D) {
  {
    int argvi = 0;
    struct _dwg_entity_VERTEX_3D *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_VERTEX_3D();");
    }
    result = (struct _dwg_entity_VERTEX_3D *)calloc(1, sizeof(struct _dwg_entity_VERTEX_3D));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_VERTEX_3D, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_VERTEX_3D) {
  {
    struct _dwg_entity_VERTEX_3D *arg1 = (struct _dwg_entity_VERTEX_3D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_VERTEX_3D(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VERTEX_3D, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_VERTEX_3D" "', argument " "1"" of type '" "struct _dwg_entity_VERTEX_3D *""'"); 
    }
    arg1 = (struct _dwg_entity_VERTEX_3D *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VERTEX_PFACE_FACE_parent_set) {
  {
    struct _dwg_entity_VERTEX_PFACE_FACE *arg1 = (struct _dwg_entity_VERTEX_PFACE_FACE *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VERTEX_PFACE_FACE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VERTEX_PFACE_FACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VERTEX_PFACE_FACE_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_VERTEX_PFACE_FACE *""'"); 
    }
    arg1 = (struct _dwg_entity_VERTEX_PFACE_FACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_VERTEX_PFACE_FACE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VERTEX_PFACE_FACE_parent_get) {
  {
    struct _dwg_entity_VERTEX_PFACE_FACE *arg1 = (struct _dwg_entity_VERTEX_PFACE_FACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VERTEX_PFACE_FACE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VERTEX_PFACE_FACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VERTEX_PFACE_FACE_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_VERTEX_PFACE_FACE *""'"); 
    }
    arg1 = (struct _dwg_entity_VERTEX_PFACE_FACE *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VERTEX_PFACE_FACE_flag_set) {
  {
    struct _dwg_entity_VERTEX_PFACE_FACE *arg1 = (struct _dwg_entity_VERTEX_PFACE_FACE *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VERTEX_PFACE_FACE_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VERTEX_PFACE_FACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VERTEX_PFACE_FACE_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_VERTEX_PFACE_FACE *""'"); 
    }
    arg1 = (struct _dwg_entity_VERTEX_PFACE_FACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_VERTEX_PFACE_FACE_flag_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VERTEX_PFACE_FACE_flag_get) {
  {
    struct _dwg_entity_VERTEX_PFACE_FACE *arg1 = (struct _dwg_entity_VERTEX_PFACE_FACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VERTEX_PFACE_FACE_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VERTEX_PFACE_FACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VERTEX_PFACE_FACE_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_VERTEX_PFACE_FACE *""'"); 
    }
    arg1 = (struct _dwg_entity_VERTEX_PFACE_FACE *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VERTEX_PFACE_FACE_vertind_set) {
  {
    struct _dwg_entity_VERTEX_PFACE_FACE *arg1 = (struct _dwg_entity_VERTEX_PFACE_FACE *) 0 ;
    BITCODE_BSd *arg2 = (BITCODE_BSd *) (BITCODE_BSd *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VERTEX_PFACE_FACE_vertind_set(self,vertind);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VERTEX_PFACE_FACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VERTEX_PFACE_FACE_vertind_set" "', argument " "1"" of type '" "struct _dwg_entity_VERTEX_PFACE_FACE *""'"); 
    }
    arg1 = (struct _dwg_entity_VERTEX_PFACE_FACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_short, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_VERTEX_PFACE_FACE_vertind_set" "', argument " "2"" of type '" "BITCODE_BSd [4]""'"); 
    } 
    arg2 = (BITCODE_BSd *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)4; ++ii) *(BITCODE_BSd *)&arg1->vertind[ii] = *((BITCODE_BSd *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""vertind""' of type '""BITCODE_BSd [4]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VERTEX_PFACE_FACE_vertind_get) {
  {
    struct _dwg_entity_VERTEX_PFACE_FACE *arg1 = (struct _dwg_entity_VERTEX_PFACE_FACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BSd *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VERTEX_PFACE_FACE_vertind_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VERTEX_PFACE_FACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VERTEX_PFACE_FACE_vertind_get" "', argument " "1"" of type '" "struct _dwg_entity_VERTEX_PFACE_FACE *""'"); 
    }
    arg1 = (struct _dwg_entity_VERTEX_PFACE_FACE *)(argp1);
    result = (BITCODE_BSd *)(BITCODE_BSd *) ((arg1)->vertind);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_short, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_VERTEX_PFACE_FACE) {
  {
    int argvi = 0;
    struct _dwg_entity_VERTEX_PFACE_FACE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_VERTEX_PFACE_FACE();");
    }
    result = (struct _dwg_entity_VERTEX_PFACE_FACE *)calloc(1, sizeof(struct _dwg_entity_VERTEX_PFACE_FACE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_VERTEX_PFACE_FACE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_VERTEX_PFACE_FACE) {
  {
    struct _dwg_entity_VERTEX_PFACE_FACE *arg1 = (struct _dwg_entity_VERTEX_PFACE_FACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_VERTEX_PFACE_FACE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VERTEX_PFACE_FACE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_VERTEX_PFACE_FACE" "', argument " "1"" of type '" "struct _dwg_entity_VERTEX_PFACE_FACE *""'"); 
    }
    arg1 = (struct _dwg_entity_VERTEX_PFACE_FACE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_parent_set) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POLYLINE_2D_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_parent_get) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_has_vertex_set) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_has_vertex_set(self,has_vertex);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_has_vertex_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POLYLINE_2D_has_vertex_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_vertex = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_has_vertex_get) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_has_vertex_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_has_vertex_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    result = (BITCODE_B) ((arg1)->has_vertex);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_num_owned_set) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_num_owned_set(self,num_owned);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_num_owned_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POLYLINE_2D_num_owned_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_owned = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_num_owned_get) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_num_owned_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_num_owned_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_owned);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_first_vertex_set) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_first_vertex_set(self,first_vertex);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_first_vertex_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POLYLINE_2D_first_vertex_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->first_vertex = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_first_vertex_get) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_first_vertex_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_first_vertex_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    result = (BITCODE_H) ((arg1)->first_vertex);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_last_vertex_set) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_last_vertex_set(self,last_vertex);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_last_vertex_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POLYLINE_2D_last_vertex_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->last_vertex = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_last_vertex_get) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_last_vertex_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_last_vertex_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    result = (BITCODE_H) ((arg1)->last_vertex);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_vertex_set) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_vertex_set(self,vertex);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_vertex_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POLYLINE_2D_vertex_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->vertex = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_vertex_get) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_vertex_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_vertex_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    result = (BITCODE_H *) ((arg1)->vertex);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_seqend_set) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_seqend_set(self,seqend);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_seqend_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POLYLINE_2D_seqend_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->seqend = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_seqend_get) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_seqend_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_seqend_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    result = (BITCODE_H) ((arg1)->seqend);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_flag_set) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POLYLINE_2D_flag_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_flag_get) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    result = (BITCODE_BS) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_curve_type_set) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_curve_type_set(self,curve_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_curve_type_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POLYLINE_2D_curve_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->curve_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_curve_type_get) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_curve_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_curve_type_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    result = (BITCODE_BS) ((arg1)->curve_type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_start_width_set) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_start_width_set(self,start_width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_start_width_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POLYLINE_2D_start_width_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->start_width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_start_width_get) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_start_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_start_width_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    result = (BITCODE_BD) ((arg1)->start_width);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_end_width_set) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_end_width_set(self,end_width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_end_width_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POLYLINE_2D_end_width_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->end_width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_end_width_get) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_end_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_end_width_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    result = (BITCODE_BD) ((arg1)->end_width);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_thickness_set) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    BITCODE_BT arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_thickness_set(self,thickness);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_thickness_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POLYLINE_2D_thickness_set" "', argument " "2"" of type '" "BITCODE_BT""'");
    } 
    arg2 = (BITCODE_BT)(val2);
    if (arg1) (arg1)->thickness = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_thickness_get) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BT result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_thickness_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_thickness_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    result = (BITCODE_BT) ((arg1)->thickness);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_elevation_set) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_elevation_set(self,elevation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_elevation_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POLYLINE_2D_elevation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->elevation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_elevation_get) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_elevation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_elevation_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    result = (BITCODE_BD) ((arg1)->elevation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_extrusion_set) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_extrusion_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POLYLINE_2D_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_extrusion_get) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_extrusion_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_extra_r11_size_set) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_extra_r11_size_set(self,extra_r11_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_extra_r11_size_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POLYLINE_2D_extra_r11_size_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->extra_r11_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_extra_r11_size_get) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_extra_r11_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_extra_r11_size_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    result = (BITCODE_BL) ((arg1)->extra_r11_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_extra_r11_text_set) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_extra_r11_text_set(self,extra_r11_text);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_extra_r11_text_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POLYLINE_2D_extra_r11_text_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->extra_r11_text);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->extra_r11_text = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->extra_r11_text = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_extra_r11_text_get) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_extra_r11_text_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_extra_r11_text_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    result = (BITCODE_TV) ((arg1)->extra_r11_text);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_num_m_verts_set) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_num_m_verts_set(self,num_m_verts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_num_m_verts_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POLYLINE_2D_num_m_verts_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_m_verts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_num_m_verts_get) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_num_m_verts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_num_m_verts_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_m_verts);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_num_n_verts_set) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_num_n_verts_set(self,num_n_verts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_num_n_verts_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POLYLINE_2D_num_n_verts_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_n_verts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_2D_num_n_verts_get) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_2D_num_n_verts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_2D_num_n_verts_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_n_verts);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_POLYLINE_2D) {
  {
    int argvi = 0;
    struct _dwg_entity_POLYLINE_2D *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_POLYLINE_2D();");
    }
    result = (struct _dwg_entity_POLYLINE_2D *)calloc(1, sizeof(struct _dwg_entity_POLYLINE_2D));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_POLYLINE_2D, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_POLYLINE_2D) {
  {
    struct _dwg_entity_POLYLINE_2D *arg1 = (struct _dwg_entity_POLYLINE_2D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_POLYLINE_2D(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_2D, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_POLYLINE_2D" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_2D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_2D *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_3D_parent_set) {
  {
    struct _dwg_entity_POLYLINE_3D *arg1 = (struct _dwg_entity_POLYLINE_3D *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_3D_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_3D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_3D_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_3D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_3D *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POLYLINE_3D_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_3D_parent_get) {
  {
    struct _dwg_entity_POLYLINE_3D *arg1 = (struct _dwg_entity_POLYLINE_3D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_3D_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_3D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_3D_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_3D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_3D *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_3D_has_vertex_set) {
  {
    struct _dwg_entity_POLYLINE_3D *arg1 = (struct _dwg_entity_POLYLINE_3D *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_3D_has_vertex_set(self,has_vertex);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_3D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_3D_has_vertex_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_3D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_3D *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POLYLINE_3D_has_vertex_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_vertex = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_3D_has_vertex_get) {
  {
    struct _dwg_entity_POLYLINE_3D *arg1 = (struct _dwg_entity_POLYLINE_3D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_3D_has_vertex_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_3D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_3D_has_vertex_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_3D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_3D *)(argp1);
    result = (BITCODE_B) ((arg1)->has_vertex);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_3D_num_owned_set) {
  {
    struct _dwg_entity_POLYLINE_3D *arg1 = (struct _dwg_entity_POLYLINE_3D *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_3D_num_owned_set(self,num_owned);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_3D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_3D_num_owned_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_3D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_3D *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POLYLINE_3D_num_owned_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_owned = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_3D_num_owned_get) {
  {
    struct _dwg_entity_POLYLINE_3D *arg1 = (struct _dwg_entity_POLYLINE_3D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_3D_num_owned_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_3D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_3D_num_owned_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_3D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_3D *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_owned);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_3D_first_vertex_set) {
  {
    struct _dwg_entity_POLYLINE_3D *arg1 = (struct _dwg_entity_POLYLINE_3D *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_3D_first_vertex_set(self,first_vertex);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_3D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_3D_first_vertex_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_3D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_3D *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POLYLINE_3D_first_vertex_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->first_vertex = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_3D_first_vertex_get) {
  {
    struct _dwg_entity_POLYLINE_3D *arg1 = (struct _dwg_entity_POLYLINE_3D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_3D_first_vertex_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_3D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_3D_first_vertex_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_3D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_3D *)(argp1);
    result = (BITCODE_H) ((arg1)->first_vertex);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_3D_last_vertex_set) {
  {
    struct _dwg_entity_POLYLINE_3D *arg1 = (struct _dwg_entity_POLYLINE_3D *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_3D_last_vertex_set(self,last_vertex);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_3D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_3D_last_vertex_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_3D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_3D *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POLYLINE_3D_last_vertex_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->last_vertex = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_3D_last_vertex_get) {
  {
    struct _dwg_entity_POLYLINE_3D *arg1 = (struct _dwg_entity_POLYLINE_3D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_3D_last_vertex_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_3D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_3D_last_vertex_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_3D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_3D *)(argp1);
    result = (BITCODE_H) ((arg1)->last_vertex);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_3D_vertex_set) {
  {
    struct _dwg_entity_POLYLINE_3D *arg1 = (struct _dwg_entity_POLYLINE_3D *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_3D_vertex_set(self,vertex);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_3D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_3D_vertex_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_3D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_3D *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POLYLINE_3D_vertex_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->vertex = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_3D_vertex_get) {
  {
    struct _dwg_entity_POLYLINE_3D *arg1 = (struct _dwg_entity_POLYLINE_3D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_3D_vertex_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_3D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_3D_vertex_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_3D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_3D *)(argp1);
    result = (BITCODE_H *) ((arg1)->vertex);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_3D_seqend_set) {
  {
    struct _dwg_entity_POLYLINE_3D *arg1 = (struct _dwg_entity_POLYLINE_3D *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_3D_seqend_set(self,seqend);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_3D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_3D_seqend_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_3D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_3D *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POLYLINE_3D_seqend_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->seqend = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_3D_seqend_get) {
  {
    struct _dwg_entity_POLYLINE_3D *arg1 = (struct _dwg_entity_POLYLINE_3D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_3D_seqend_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_3D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_3D_seqend_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_3D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_3D *)(argp1);
    result = (BITCODE_H) ((arg1)->seqend);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_3D_curve_type_set) {
  {
    struct _dwg_entity_POLYLINE_3D *arg1 = (struct _dwg_entity_POLYLINE_3D *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_3D_curve_type_set(self,curve_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_3D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_3D_curve_type_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_3D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_3D *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POLYLINE_3D_curve_type_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->curve_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_3D_curve_type_get) {
  {
    struct _dwg_entity_POLYLINE_3D *arg1 = (struct _dwg_entity_POLYLINE_3D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_3D_curve_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_3D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_3D_curve_type_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_3D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_3D *)(argp1);
    result = (BITCODE_RC) ((arg1)->curve_type);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_3D_start_width_set) {
  {
    struct _dwg_entity_POLYLINE_3D *arg1 = (struct _dwg_entity_POLYLINE_3D *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_3D_start_width_set(self,start_width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_3D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_3D_start_width_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_3D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_3D *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POLYLINE_3D_start_width_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->start_width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_3D_start_width_get) {
  {
    struct _dwg_entity_POLYLINE_3D *arg1 = (struct _dwg_entity_POLYLINE_3D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_3D_start_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_3D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_3D_start_width_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_3D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_3D *)(argp1);
    result = (BITCODE_BD) ((arg1)->start_width);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_3D_end_width_set) {
  {
    struct _dwg_entity_POLYLINE_3D *arg1 = (struct _dwg_entity_POLYLINE_3D *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_3D_end_width_set(self,end_width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_3D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_3D_end_width_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_3D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_3D *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POLYLINE_3D_end_width_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->end_width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_3D_end_width_get) {
  {
    struct _dwg_entity_POLYLINE_3D *arg1 = (struct _dwg_entity_POLYLINE_3D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_3D_end_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_3D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_3D_end_width_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_3D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_3D *)(argp1);
    result = (BITCODE_BD) ((arg1)->end_width);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_3D_flag_set) {
  {
    struct _dwg_entity_POLYLINE_3D *arg1 = (struct _dwg_entity_POLYLINE_3D *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_3D_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_3D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_3D_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_3D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_3D *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POLYLINE_3D_flag_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_3D_flag_get) {
  {
    struct _dwg_entity_POLYLINE_3D *arg1 = (struct _dwg_entity_POLYLINE_3D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_3D_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_3D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_3D_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_3D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_3D *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_3D_extrusion_set) {
  {
    struct _dwg_entity_POLYLINE_3D *arg1 = (struct _dwg_entity_POLYLINE_3D *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_3D_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_3D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_3D_extrusion_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_3D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_3D *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POLYLINE_3D_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_3D_extrusion_get) {
  {
    struct _dwg_entity_POLYLINE_3D *arg1 = (struct _dwg_entity_POLYLINE_3D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_3D_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_3D, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_3D_extrusion_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_3D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_3D *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_POLYLINE_3D) {
  {
    int argvi = 0;
    struct _dwg_entity_POLYLINE_3D *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_POLYLINE_3D();");
    }
    result = (struct _dwg_entity_POLYLINE_3D *)calloc(1, sizeof(struct _dwg_entity_POLYLINE_3D));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_POLYLINE_3D, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_POLYLINE_3D) {
  {
    struct _dwg_entity_POLYLINE_3D *arg1 = (struct _dwg_entity_POLYLINE_3D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_POLYLINE_3D(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_3D, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_POLYLINE_3D" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_3D *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_3D *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_parent_set) {
  {
    struct _dwg_entity_ARC *arg1 = (struct _dwg_entity_ARC *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ARC_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_parent_get) {
  {
    struct _dwg_entity_ARC *arg1 = (struct _dwg_entity_ARC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_center_set) {
  {
    struct _dwg_entity_ARC *arg1 = (struct _dwg_entity_ARC *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_center_set(self,center);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_center_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ARC_center_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->center = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_center_get) {
  {
    struct _dwg_entity_ARC *arg1 = (struct _dwg_entity_ARC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_center_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_center_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->center);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_radius_set) {
  {
    struct _dwg_entity_ARC *arg1 = (struct _dwg_entity_ARC *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_radius_set(self,radius);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_radius_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARC_radius_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->radius = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_radius_get) {
  {
    struct _dwg_entity_ARC *arg1 = (struct _dwg_entity_ARC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_radius_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_radius_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC *)(argp1);
    result = (BITCODE_BD) ((arg1)->radius);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_thickness_set) {
  {
    struct _dwg_entity_ARC *arg1 = (struct _dwg_entity_ARC *) 0 ;
    BITCODE_BT arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_thickness_set(self,thickness);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_thickness_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARC_thickness_set" "', argument " "2"" of type '" "BITCODE_BT""'");
    } 
    arg2 = (BITCODE_BT)(val2);
    if (arg1) (arg1)->thickness = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_thickness_get) {
  {
    struct _dwg_entity_ARC *arg1 = (struct _dwg_entity_ARC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BT result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_thickness_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_thickness_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC *)(argp1);
    result = (BITCODE_BT) ((arg1)->thickness);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_extrusion_set) {
  {
    struct _dwg_entity_ARC *arg1 = (struct _dwg_entity_ARC *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_extrusion_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ARC_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_extrusion_get) {
  {
    struct _dwg_entity_ARC *arg1 = (struct _dwg_entity_ARC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_extrusion_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_start_angle_set) {
  {
    struct _dwg_entity_ARC *arg1 = (struct _dwg_entity_ARC *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_start_angle_set(self,start_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_start_angle_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARC_start_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->start_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_start_angle_get) {
  {
    struct _dwg_entity_ARC *arg1 = (struct _dwg_entity_ARC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_start_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_start_angle_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC *)(argp1);
    result = (BITCODE_BD) ((arg1)->start_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_end_angle_set) {
  {
    struct _dwg_entity_ARC *arg1 = (struct _dwg_entity_ARC *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_end_angle_set(self,end_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_end_angle_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARC_end_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->end_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_end_angle_get) {
  {
    struct _dwg_entity_ARC *arg1 = (struct _dwg_entity_ARC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_end_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_end_angle_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC *)(argp1);
    result = (BITCODE_BD) ((arg1)->end_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_ARC) {
  {
    int argvi = 0;
    struct _dwg_entity_ARC *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_ARC();");
    }
    result = (struct _dwg_entity_ARC *)calloc(1, sizeof(struct _dwg_entity_ARC));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_ARC, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_ARC) {
  {
    struct _dwg_entity_ARC *arg1 = (struct _dwg_entity_ARC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_ARC(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_ARC" "', argument " "1"" of type '" "struct _dwg_entity_ARC *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_CIRCLE_parent_set) {
  {
    struct _dwg_entity_CIRCLE *arg1 = (struct _dwg_entity_CIRCLE *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_CIRCLE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_CIRCLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_CIRCLE_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_CIRCLE *""'"); 
    }
    arg1 = (struct _dwg_entity_CIRCLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_CIRCLE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_CIRCLE_parent_get) {
  {
    struct _dwg_entity_CIRCLE *arg1 = (struct _dwg_entity_CIRCLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_CIRCLE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_CIRCLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_CIRCLE_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_CIRCLE *""'"); 
    }
    arg1 = (struct _dwg_entity_CIRCLE *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_CIRCLE_center_set) {
  {
    struct _dwg_entity_CIRCLE *arg1 = (struct _dwg_entity_CIRCLE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_CIRCLE_center_set(self,center);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_CIRCLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_CIRCLE_center_set" "', argument " "1"" of type '" "struct _dwg_entity_CIRCLE *""'"); 
    }
    arg1 = (struct _dwg_entity_CIRCLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_CIRCLE_center_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->center = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_CIRCLE_center_get) {
  {
    struct _dwg_entity_CIRCLE *arg1 = (struct _dwg_entity_CIRCLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_CIRCLE_center_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_CIRCLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_CIRCLE_center_get" "', argument " "1"" of type '" "struct _dwg_entity_CIRCLE *""'"); 
    }
    arg1 = (struct _dwg_entity_CIRCLE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->center);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_CIRCLE_radius_set) {
  {
    struct _dwg_entity_CIRCLE *arg1 = (struct _dwg_entity_CIRCLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_CIRCLE_radius_set(self,radius);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_CIRCLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_CIRCLE_radius_set" "', argument " "1"" of type '" "struct _dwg_entity_CIRCLE *""'"); 
    }
    arg1 = (struct _dwg_entity_CIRCLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_CIRCLE_radius_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->radius = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_CIRCLE_radius_get) {
  {
    struct _dwg_entity_CIRCLE *arg1 = (struct _dwg_entity_CIRCLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_CIRCLE_radius_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_CIRCLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_CIRCLE_radius_get" "', argument " "1"" of type '" "struct _dwg_entity_CIRCLE *""'"); 
    }
    arg1 = (struct _dwg_entity_CIRCLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->radius);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_CIRCLE_thickness_set) {
  {
    struct _dwg_entity_CIRCLE *arg1 = (struct _dwg_entity_CIRCLE *) 0 ;
    BITCODE_BT arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_CIRCLE_thickness_set(self,thickness);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_CIRCLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_CIRCLE_thickness_set" "', argument " "1"" of type '" "struct _dwg_entity_CIRCLE *""'"); 
    }
    arg1 = (struct _dwg_entity_CIRCLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_CIRCLE_thickness_set" "', argument " "2"" of type '" "BITCODE_BT""'");
    } 
    arg2 = (BITCODE_BT)(val2);
    if (arg1) (arg1)->thickness = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_CIRCLE_thickness_get) {
  {
    struct _dwg_entity_CIRCLE *arg1 = (struct _dwg_entity_CIRCLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BT result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_CIRCLE_thickness_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_CIRCLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_CIRCLE_thickness_get" "', argument " "1"" of type '" "struct _dwg_entity_CIRCLE *""'"); 
    }
    arg1 = (struct _dwg_entity_CIRCLE *)(argp1);
    result = (BITCODE_BT) ((arg1)->thickness);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_CIRCLE_extrusion_set) {
  {
    struct _dwg_entity_CIRCLE *arg1 = (struct _dwg_entity_CIRCLE *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_CIRCLE_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_CIRCLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_CIRCLE_extrusion_set" "', argument " "1"" of type '" "struct _dwg_entity_CIRCLE *""'"); 
    }
    arg1 = (struct _dwg_entity_CIRCLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_CIRCLE_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_CIRCLE_extrusion_get) {
  {
    struct _dwg_entity_CIRCLE *arg1 = (struct _dwg_entity_CIRCLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_CIRCLE_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_CIRCLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_CIRCLE_extrusion_get" "', argument " "1"" of type '" "struct _dwg_entity_CIRCLE *""'"); 
    }
    arg1 = (struct _dwg_entity_CIRCLE *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_CIRCLE) {
  {
    int argvi = 0;
    struct _dwg_entity_CIRCLE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_CIRCLE();");
    }
    result = (struct _dwg_entity_CIRCLE *)calloc(1, sizeof(struct _dwg_entity_CIRCLE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_CIRCLE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_CIRCLE) {
  {
    struct _dwg_entity_CIRCLE *arg1 = (struct _dwg_entity_CIRCLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_CIRCLE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_CIRCLE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_CIRCLE" "', argument " "1"" of type '" "struct _dwg_entity_CIRCLE *""'"); 
    }
    arg1 = (struct _dwg_entity_CIRCLE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LINE_parent_set) {
  {
    struct _dwg_entity_LINE *arg1 = (struct _dwg_entity_LINE *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LINE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LINE_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_LINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LINE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LINE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LINE_parent_get) {
  {
    struct _dwg_entity_LINE *arg1 = (struct _dwg_entity_LINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LINE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LINE_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_LINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LINE *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LINE_z_is_zero_set) {
  {
    struct _dwg_entity_LINE *arg1 = (struct _dwg_entity_LINE *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LINE_z_is_zero_set(self,z_is_zero);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LINE_z_is_zero_set" "', argument " "1"" of type '" "struct _dwg_entity_LINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LINE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LINE_z_is_zero_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->z_is_zero = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LINE_z_is_zero_get) {
  {
    struct _dwg_entity_LINE *arg1 = (struct _dwg_entity_LINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LINE_z_is_zero_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LINE_z_is_zero_get" "', argument " "1"" of type '" "struct _dwg_entity_LINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LINE *)(argp1);
    result = (BITCODE_RC) ((arg1)->z_is_zero);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LINE_start_set) {
  {
    struct _dwg_entity_LINE *arg1 = (struct _dwg_entity_LINE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LINE_start_set(self,start);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LINE_start_set" "', argument " "1"" of type '" "struct _dwg_entity_LINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LINE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LINE_start_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->start = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LINE_start_get) {
  {
    struct _dwg_entity_LINE *arg1 = (struct _dwg_entity_LINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LINE_start_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LINE_start_get" "', argument " "1"" of type '" "struct _dwg_entity_LINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LINE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->start);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LINE_end_set) {
  {
    struct _dwg_entity_LINE *arg1 = (struct _dwg_entity_LINE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LINE_end_set(self,end);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LINE_end_set" "', argument " "1"" of type '" "struct _dwg_entity_LINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LINE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LINE_end_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->end = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LINE_end_get) {
  {
    struct _dwg_entity_LINE *arg1 = (struct _dwg_entity_LINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LINE_end_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LINE_end_get" "', argument " "1"" of type '" "struct _dwg_entity_LINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LINE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->end);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LINE_thickness_set) {
  {
    struct _dwg_entity_LINE *arg1 = (struct _dwg_entity_LINE *) 0 ;
    BITCODE_BT arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LINE_thickness_set(self,thickness);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LINE_thickness_set" "', argument " "1"" of type '" "struct _dwg_entity_LINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LINE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LINE_thickness_set" "', argument " "2"" of type '" "BITCODE_BT""'");
    } 
    arg2 = (BITCODE_BT)(val2);
    if (arg1) (arg1)->thickness = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LINE_thickness_get) {
  {
    struct _dwg_entity_LINE *arg1 = (struct _dwg_entity_LINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BT result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LINE_thickness_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LINE_thickness_get" "', argument " "1"" of type '" "struct _dwg_entity_LINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LINE *)(argp1);
    result = (BITCODE_BT) ((arg1)->thickness);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LINE_extrusion_set) {
  {
    struct _dwg_entity_LINE *arg1 = (struct _dwg_entity_LINE *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LINE_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LINE_extrusion_set" "', argument " "1"" of type '" "struct _dwg_entity_LINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LINE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LINE_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LINE_extrusion_get) {
  {
    struct _dwg_entity_LINE *arg1 = (struct _dwg_entity_LINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LINE_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LINE_extrusion_get" "', argument " "1"" of type '" "struct _dwg_entity_LINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LINE *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LINE_unknown_r11_set) {
  {
    struct _dwg_entity_LINE *arg1 = (struct _dwg_entity_LINE *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LINE_unknown_r11_set(self,unknown_r11);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LINE_unknown_r11_set" "', argument " "1"" of type '" "struct _dwg_entity_LINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LINE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LINE_unknown_r11_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->unknown_r11 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LINE_unknown_r11_get) {
  {
    struct _dwg_entity_LINE *arg1 = (struct _dwg_entity_LINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LINE_unknown_r11_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LINE_unknown_r11_get" "', argument " "1"" of type '" "struct _dwg_entity_LINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LINE *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->unknown_r11);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_LINE) {
  {
    int argvi = 0;
    struct _dwg_entity_LINE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_LINE();");
    }
    result = (struct _dwg_entity_LINE *)calloc(1, sizeof(struct _dwg_entity_LINE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_LINE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_LINE) {
  {
    struct _dwg_entity_LINE *arg1 = (struct _dwg_entity_LINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_LINE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LINE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_LINE" "', argument " "1"" of type '" "struct _dwg_entity_LINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LINE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_parent_set) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_parent_set" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_DIMENSION_common_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_parent_get) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_parent_get" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_class_version_set) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_class_version_set" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_DIMENSION_common_class_version_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_class_version_get) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_class_version_get" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    result = (BITCODE_RC) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_extrusion_set) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_extrusion_set" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_DIMENSION_common_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_extrusion_get) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_extrusion_get" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_def_pt_set) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_def_pt_set(self,def_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_def_pt_set" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_DIMENSION_common_def_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_def_pt_get) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_def_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_def_pt_get" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_text_midpt_set) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_text_midpt_set(self,text_midpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_text_midpt_set" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_DIMENSION_common_text_midpt_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->text_midpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_text_midpt_get) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_text_midpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_text_midpt_get" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->text_midpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_elevation_set) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_elevation_set(self,elevation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_elevation_set" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_DIMENSION_common_elevation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->elevation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_elevation_get) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_elevation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_elevation_get" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    result = (BITCODE_BD) ((arg1)->elevation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_flag_set) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_flag_set" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_DIMENSION_common_flag_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_flag_get) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_flag_get" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_flag1_set) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_flag1_set(self,flag1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_flag1_set" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_DIMENSION_common_flag1_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_flag1_get) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_flag1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_flag1_get" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag1);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_user_text_set) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_user_text_set(self,user_text);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_user_text_set" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_DIMENSION_common_user_text_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->user_text);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->user_text = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->user_text = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_user_text_get) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_user_text_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_user_text_get" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    result = (BITCODE_TV) ((arg1)->user_text);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_text_rotation_set) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_text_rotation_set(self,text_rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_text_rotation_set" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_DIMENSION_common_text_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->text_rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_text_rotation_get) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_text_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_text_rotation_get" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    result = (BITCODE_BD) ((arg1)->text_rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_horiz_dir_set) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_horiz_dir_set(self,horiz_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_horiz_dir_set" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_DIMENSION_common_horiz_dir_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->horiz_dir = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_horiz_dir_get) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_horiz_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_horiz_dir_get" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    result = (BITCODE_BD) ((arg1)->horiz_dir);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_ins_scale_set) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_ins_scale_set(self,ins_scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_ins_scale_set" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_DIMENSION_common_ins_scale_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ins_scale = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_ins_scale_get) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_ins_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_ins_scale_get" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ins_scale);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_ins_rotation_set) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_ins_rotation_set(self,ins_rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_ins_rotation_set" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_DIMENSION_common_ins_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->ins_rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_ins_rotation_get) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_ins_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_ins_rotation_get" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    result = (BITCODE_BD) ((arg1)->ins_rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_attachment_set) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_attachment_set(self,attachment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_attachment_set" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_DIMENSION_common_attachment_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->attachment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_attachment_get) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_attachment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_attachment_get" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    result = (BITCODE_BS) ((arg1)->attachment);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_lspace_style_set) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_lspace_style_set(self,lspace_style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_lspace_style_set" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_DIMENSION_common_lspace_style_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->lspace_style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_lspace_style_get) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_lspace_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_lspace_style_get" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    result = (BITCODE_BS) ((arg1)->lspace_style);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_lspace_factor_set) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_lspace_factor_set(self,lspace_factor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_lspace_factor_set" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_DIMENSION_common_lspace_factor_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->lspace_factor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_lspace_factor_get) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_lspace_factor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_lspace_factor_get" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    result = (BITCODE_BD) ((arg1)->lspace_factor);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_act_measurement_set) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_act_measurement_set(self,act_measurement);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_act_measurement_set" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_DIMENSION_common_act_measurement_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->act_measurement = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_act_measurement_get) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_act_measurement_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_act_measurement_get" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    result = (BITCODE_BD) ((arg1)->act_measurement);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_unknown_set) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_unknown_set(self,unknown);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_unknown_set" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_DIMENSION_common_unknown_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_unknown_get) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_unknown_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_unknown_get" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_flip_arrow1_set) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_flip_arrow1_set(self,flip_arrow1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_flip_arrow1_set" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_DIMENSION_common_flip_arrow1_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->flip_arrow1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_flip_arrow1_get) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_flip_arrow1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_flip_arrow1_get" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    result = (BITCODE_B) ((arg1)->flip_arrow1);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_flip_arrow2_set) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_flip_arrow2_set(self,flip_arrow2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_flip_arrow2_set" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_DIMENSION_common_flip_arrow2_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->flip_arrow2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_flip_arrow2_get) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_flip_arrow2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_flip_arrow2_get" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    result = (BITCODE_B) ((arg1)->flip_arrow2);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_clone_ins_pt_set) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_clone_ins_pt_set(self,clone_ins_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_clone_ins_pt_set" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_DIMENSION_common_clone_ins_pt_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->clone_ins_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_clone_ins_pt_get) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_clone_ins_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_clone_ins_pt_get" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->clone_ins_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_dimstyle_set) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_dimstyle_set(self,dimstyle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_dimstyle_set" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_DIMENSION_common_dimstyle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->dimstyle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_dimstyle_get) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_dimstyle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_dimstyle_get" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    result = (BITCODE_H) ((arg1)->dimstyle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_block_set) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_block_set(self,block);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_block_set" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_DIMENSION_common_block_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->block = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMENSION_common_block_get) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMENSION_common_block_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMENSION_common_block_get" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    result = (BITCODE_H) ((arg1)->block);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_DIMENSION_common) {
  {
    int argvi = 0;
    struct _dwg_DIMENSION_common *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_DIMENSION_common();");
    }
    result = (struct _dwg_DIMENSION_common *)calloc(1, sizeof(struct _dwg_DIMENSION_common));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_DIMENSION_common, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_DIMENSION_common) {
  {
    struct _dwg_DIMENSION_common *arg1 = (struct _dwg_DIMENSION_common *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_DIMENSION_common(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMENSION_common, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_DIMENSION_common" "', argument " "1"" of type '" "struct _dwg_DIMENSION_common *""'"); 
    }
    arg1 = (struct _dwg_DIMENSION_common *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_parent_set) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_parent_get) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_class_version_set) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_class_version_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_class_version_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_class_version_get) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_class_version_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    result = (BITCODE_RC) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_extrusion_set) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_extrusion_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_extrusion_get) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_extrusion_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_def_pt_set) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_def_pt_set(self,def_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_def_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_def_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_def_pt_get) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_def_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_def_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_text_midpt_set) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_text_midpt_set(self,text_midpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_text_midpt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_text_midpt_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->text_midpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_text_midpt_get) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_text_midpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_text_midpt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->text_midpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_elevation_set) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_elevation_set(self,elevation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_elevation_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_elevation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->elevation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_elevation_get) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_elevation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_elevation_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    result = (BITCODE_BD) ((arg1)->elevation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_flag_set) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_flag_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_flag_get) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_flag1_set) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_flag1_set(self,flag1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_flag1_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_flag1_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_flag1_get) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_flag1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_flag1_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag1);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_user_text_set) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_user_text_set(self,user_text);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_user_text_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_user_text_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->user_text);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->user_text = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->user_text = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_user_text_get) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_user_text_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_user_text_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    result = (BITCODE_TV) ((arg1)->user_text);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_text_rotation_set) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_text_rotation_set(self,text_rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_text_rotation_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_text_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->text_rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_text_rotation_get) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_text_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_text_rotation_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    result = (BITCODE_BD) ((arg1)->text_rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_horiz_dir_set) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_horiz_dir_set(self,horiz_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_horiz_dir_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_horiz_dir_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->horiz_dir = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_horiz_dir_get) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_horiz_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_horiz_dir_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    result = (BITCODE_BD) ((arg1)->horiz_dir);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_ins_scale_set) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_ins_scale_set(self,ins_scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_ins_scale_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_ins_scale_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ins_scale = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_ins_scale_get) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_ins_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_ins_scale_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ins_scale);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_ins_rotation_set) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_ins_rotation_set(self,ins_rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_ins_rotation_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_ins_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->ins_rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_ins_rotation_get) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_ins_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_ins_rotation_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    result = (BITCODE_BD) ((arg1)->ins_rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_attachment_set) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_attachment_set(self,attachment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_attachment_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_attachment_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->attachment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_attachment_get) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_attachment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_attachment_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    result = (BITCODE_BS) ((arg1)->attachment);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_lspace_style_set) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_lspace_style_set(self,lspace_style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_lspace_style_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_lspace_style_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->lspace_style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_lspace_style_get) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_lspace_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_lspace_style_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    result = (BITCODE_BS) ((arg1)->lspace_style);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_lspace_factor_set) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_lspace_factor_set(self,lspace_factor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_lspace_factor_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_lspace_factor_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->lspace_factor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_lspace_factor_get) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_lspace_factor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_lspace_factor_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    result = (BITCODE_BD) ((arg1)->lspace_factor);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_act_measurement_set) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_act_measurement_set(self,act_measurement);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_act_measurement_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_act_measurement_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->act_measurement = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_act_measurement_get) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_act_measurement_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_act_measurement_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    result = (BITCODE_BD) ((arg1)->act_measurement);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_unknown_set) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_unknown_set(self,unknown);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_unknown_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_unknown_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_unknown_get) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_unknown_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_unknown_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_flip_arrow1_set) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_flip_arrow1_set(self,flip_arrow1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_flip_arrow1_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_flip_arrow1_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->flip_arrow1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_flip_arrow1_get) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_flip_arrow1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_flip_arrow1_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    result = (BITCODE_B) ((arg1)->flip_arrow1);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_flip_arrow2_set) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_flip_arrow2_set(self,flip_arrow2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_flip_arrow2_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_flip_arrow2_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->flip_arrow2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_flip_arrow2_get) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_flip_arrow2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_flip_arrow2_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    result = (BITCODE_B) ((arg1)->flip_arrow2);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_clone_ins_pt_set) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_clone_ins_pt_set(self,clone_ins_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_clone_ins_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_clone_ins_pt_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->clone_ins_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_clone_ins_pt_get) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_clone_ins_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_clone_ins_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->clone_ins_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_dimstyle_set) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_dimstyle_set(self,dimstyle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_dimstyle_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_dimstyle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->dimstyle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_dimstyle_get) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_dimstyle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_dimstyle_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    result = (BITCODE_H) ((arg1)->dimstyle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_block_set) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_block_set(self,block);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_block_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_block_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->block = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_block_get) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_block_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_block_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    result = (BITCODE_H) ((arg1)->block);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_feature_location_pt_set) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_feature_location_pt_set(self,feature_location_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_feature_location_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_feature_location_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->feature_location_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_feature_location_pt_get) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_feature_location_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_feature_location_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->feature_location_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_leader_endpt_set) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_leader_endpt_set(self,leader_endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_leader_endpt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_leader_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->leader_endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_leader_endpt_get) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_leader_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_leader_endpt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->leader_endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_flag2_set) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_flag2_set(self,flag2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_flag2_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_flag2_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ORDINATE_flag2_get) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ORDINATE_flag2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ORDINATE_flag2_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag2);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_DIMENSION_ORDINATE) {
  {
    int argvi = 0;
    struct _dwg_entity_DIMENSION_ORDINATE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_DIMENSION_ORDINATE();");
    }
    result = (struct _dwg_entity_DIMENSION_ORDINATE *)calloc(1, sizeof(struct _dwg_entity_DIMENSION_ORDINATE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_DIMENSION_ORDINATE) {
  {
    struct _dwg_entity_DIMENSION_ORDINATE *arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_DIMENSION_ORDINATE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_DIMENSION_ORDINATE" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ORDINATE *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ORDINATE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_parent_set) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_LINEAR_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_parent_get) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_class_version_set) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_class_version_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_LINEAR_class_version_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_class_version_get) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_class_version_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    result = (BITCODE_RC) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_extrusion_set) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_extrusion_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_LINEAR_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_extrusion_get) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_extrusion_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_def_pt_set) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_def_pt_set(self,def_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_def_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_LINEAR_def_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_def_pt_get) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_def_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_def_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_text_midpt_set) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_text_midpt_set(self,text_midpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_text_midpt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_LINEAR_text_midpt_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->text_midpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_text_midpt_get) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_text_midpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_text_midpt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->text_midpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_elevation_set) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_elevation_set(self,elevation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_elevation_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_LINEAR_elevation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->elevation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_elevation_get) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_elevation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_elevation_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    result = (BITCODE_BD) ((arg1)->elevation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_flag_set) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_LINEAR_flag_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_flag_get) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_flag1_set) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_flag1_set(self,flag1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_flag1_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_LINEAR_flag1_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_flag1_get) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_flag1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_flag1_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag1);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_user_text_set) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_user_text_set(self,user_text);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_user_text_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_LINEAR_user_text_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->user_text);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->user_text = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->user_text = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_user_text_get) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_user_text_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_user_text_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    result = (BITCODE_TV) ((arg1)->user_text);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_text_rotation_set) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_text_rotation_set(self,text_rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_text_rotation_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_LINEAR_text_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->text_rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_text_rotation_get) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_text_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_text_rotation_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    result = (BITCODE_BD) ((arg1)->text_rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_horiz_dir_set) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_horiz_dir_set(self,horiz_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_horiz_dir_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_LINEAR_horiz_dir_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->horiz_dir = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_horiz_dir_get) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_horiz_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_horiz_dir_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    result = (BITCODE_BD) ((arg1)->horiz_dir);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_ins_scale_set) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_ins_scale_set(self,ins_scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_ins_scale_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_LINEAR_ins_scale_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ins_scale = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_ins_scale_get) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_ins_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_ins_scale_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ins_scale);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_ins_rotation_set) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_ins_rotation_set(self,ins_rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_ins_rotation_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_LINEAR_ins_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->ins_rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_ins_rotation_get) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_ins_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_ins_rotation_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    result = (BITCODE_BD) ((arg1)->ins_rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_attachment_set) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_attachment_set(self,attachment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_attachment_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_LINEAR_attachment_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->attachment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_attachment_get) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_attachment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_attachment_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    result = (BITCODE_BS) ((arg1)->attachment);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_lspace_style_set) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_lspace_style_set(self,lspace_style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_lspace_style_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_LINEAR_lspace_style_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->lspace_style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_lspace_style_get) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_lspace_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_lspace_style_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    result = (BITCODE_BS) ((arg1)->lspace_style);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_lspace_factor_set) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_lspace_factor_set(self,lspace_factor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_lspace_factor_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_LINEAR_lspace_factor_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->lspace_factor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_lspace_factor_get) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_lspace_factor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_lspace_factor_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    result = (BITCODE_BD) ((arg1)->lspace_factor);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_act_measurement_set) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_act_measurement_set(self,act_measurement);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_act_measurement_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_LINEAR_act_measurement_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->act_measurement = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_act_measurement_get) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_act_measurement_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_act_measurement_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    result = (BITCODE_BD) ((arg1)->act_measurement);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_unknown_set) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_unknown_set(self,unknown);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_unknown_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_LINEAR_unknown_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_unknown_get) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_unknown_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_unknown_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_flip_arrow1_set) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_flip_arrow1_set(self,flip_arrow1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_flip_arrow1_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_LINEAR_flip_arrow1_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->flip_arrow1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_flip_arrow1_get) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_flip_arrow1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_flip_arrow1_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    result = (BITCODE_B) ((arg1)->flip_arrow1);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_flip_arrow2_set) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_flip_arrow2_set(self,flip_arrow2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_flip_arrow2_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_LINEAR_flip_arrow2_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->flip_arrow2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_flip_arrow2_get) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_flip_arrow2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_flip_arrow2_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    result = (BITCODE_B) ((arg1)->flip_arrow2);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_clone_ins_pt_set) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_clone_ins_pt_set(self,clone_ins_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_clone_ins_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_LINEAR_clone_ins_pt_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->clone_ins_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_clone_ins_pt_get) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_clone_ins_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_clone_ins_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->clone_ins_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_dimstyle_set) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_dimstyle_set(self,dimstyle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_dimstyle_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_LINEAR_dimstyle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->dimstyle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_dimstyle_get) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_dimstyle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_dimstyle_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    result = (BITCODE_H) ((arg1)->dimstyle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_block_set) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_block_set(self,block);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_block_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_LINEAR_block_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->block = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_block_get) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_block_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_block_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    result = (BITCODE_H) ((arg1)->block);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_xline1_pt_set) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_xline1_pt_set(self,xline1_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_xline1_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_LINEAR_xline1_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->xline1_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_xline1_pt_get) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_xline1_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_xline1_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->xline1_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_xline2_pt_set) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_xline2_pt_set(self,xline2_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_xline2_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_LINEAR_xline2_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->xline2_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_xline2_pt_get) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_xline2_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_xline2_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->xline2_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_oblique_angle_set) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_oblique_angle_set(self,oblique_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_oblique_angle_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_LINEAR_oblique_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->oblique_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_oblique_angle_get) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_oblique_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_oblique_angle_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    result = (BITCODE_BD) ((arg1)->oblique_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_dim_rotation_set) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_dim_rotation_set(self,dim_rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_dim_rotation_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_LINEAR_dim_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->dim_rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_LINEAR_dim_rotation_get) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_LINEAR_dim_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_LINEAR_dim_rotation_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    result = (BITCODE_BD) ((arg1)->dim_rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_DIMENSION_LINEAR) {
  {
    int argvi = 0;
    struct _dwg_entity_DIMENSION_LINEAR *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_DIMENSION_LINEAR();");
    }
    result = (struct _dwg_entity_DIMENSION_LINEAR *)calloc(1, sizeof(struct _dwg_entity_DIMENSION_LINEAR));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_DIMENSION_LINEAR) {
  {
    struct _dwg_entity_DIMENSION_LINEAR *arg1 = (struct _dwg_entity_DIMENSION_LINEAR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_DIMENSION_LINEAR(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_DIMENSION_LINEAR" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_LINEAR *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_LINEAR *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_parent_set) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_parent_get) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_class_version_set) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_class_version_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_class_version_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_class_version_get) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_class_version_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    result = (BITCODE_RC) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_extrusion_set) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_extrusion_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_extrusion_get) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_extrusion_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_def_pt_set) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_def_pt_set(self,def_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_def_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_def_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_def_pt_get) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_def_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_def_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_text_midpt_set) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_text_midpt_set(self,text_midpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_text_midpt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_text_midpt_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->text_midpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_text_midpt_get) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_text_midpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_text_midpt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->text_midpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_elevation_set) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_elevation_set(self,elevation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_elevation_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_elevation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->elevation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_elevation_get) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_elevation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_elevation_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    result = (BITCODE_BD) ((arg1)->elevation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_flag_set) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_flag_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_flag_get) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_flag1_set) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_flag1_set(self,flag1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_flag1_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_flag1_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_flag1_get) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_flag1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_flag1_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag1);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_user_text_set) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_user_text_set(self,user_text);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_user_text_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_user_text_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->user_text);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->user_text = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->user_text = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_user_text_get) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_user_text_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_user_text_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    result = (BITCODE_TV) ((arg1)->user_text);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_text_rotation_set) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_text_rotation_set(self,text_rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_text_rotation_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_text_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->text_rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_text_rotation_get) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_text_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_text_rotation_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    result = (BITCODE_BD) ((arg1)->text_rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_horiz_dir_set) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_horiz_dir_set(self,horiz_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_horiz_dir_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_horiz_dir_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->horiz_dir = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_horiz_dir_get) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_horiz_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_horiz_dir_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    result = (BITCODE_BD) ((arg1)->horiz_dir);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_ins_scale_set) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_ins_scale_set(self,ins_scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_ins_scale_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_ins_scale_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ins_scale = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_ins_scale_get) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_ins_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_ins_scale_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ins_scale);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_ins_rotation_set) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_ins_rotation_set(self,ins_rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_ins_rotation_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_ins_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->ins_rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_ins_rotation_get) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_ins_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_ins_rotation_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    result = (BITCODE_BD) ((arg1)->ins_rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_attachment_set) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_attachment_set(self,attachment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_attachment_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_attachment_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->attachment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_attachment_get) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_attachment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_attachment_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    result = (BITCODE_BS) ((arg1)->attachment);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_lspace_style_set) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_lspace_style_set(self,lspace_style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_lspace_style_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_lspace_style_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->lspace_style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_lspace_style_get) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_lspace_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_lspace_style_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    result = (BITCODE_BS) ((arg1)->lspace_style);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_lspace_factor_set) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_lspace_factor_set(self,lspace_factor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_lspace_factor_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_lspace_factor_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->lspace_factor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_lspace_factor_get) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_lspace_factor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_lspace_factor_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    result = (BITCODE_BD) ((arg1)->lspace_factor);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_act_measurement_set) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_act_measurement_set(self,act_measurement);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_act_measurement_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_act_measurement_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->act_measurement = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_act_measurement_get) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_act_measurement_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_act_measurement_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    result = (BITCODE_BD) ((arg1)->act_measurement);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_unknown_set) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_unknown_set(self,unknown);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_unknown_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_unknown_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_unknown_get) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_unknown_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_unknown_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_flip_arrow1_set) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_flip_arrow1_set(self,flip_arrow1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_flip_arrow1_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_flip_arrow1_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->flip_arrow1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_flip_arrow1_get) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_flip_arrow1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_flip_arrow1_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    result = (BITCODE_B) ((arg1)->flip_arrow1);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_flip_arrow2_set) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_flip_arrow2_set(self,flip_arrow2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_flip_arrow2_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_flip_arrow2_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->flip_arrow2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_flip_arrow2_get) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_flip_arrow2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_flip_arrow2_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    result = (BITCODE_B) ((arg1)->flip_arrow2);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_clone_ins_pt_set) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_clone_ins_pt_set(self,clone_ins_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_clone_ins_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_clone_ins_pt_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->clone_ins_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_clone_ins_pt_get) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_clone_ins_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_clone_ins_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->clone_ins_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_dimstyle_set) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_dimstyle_set(self,dimstyle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_dimstyle_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_dimstyle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->dimstyle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_dimstyle_get) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_dimstyle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_dimstyle_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    result = (BITCODE_H) ((arg1)->dimstyle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_block_set) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_block_set(self,block);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_block_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_block_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->block = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_block_get) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_block_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_block_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    result = (BITCODE_H) ((arg1)->block);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_xline1_pt_set) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_xline1_pt_set(self,xline1_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_xline1_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_xline1_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->xline1_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_xline1_pt_get) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_xline1_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_xline1_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->xline1_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_xline2_pt_set) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_xline2_pt_set(self,xline2_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_xline2_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_xline2_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->xline2_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_xline2_pt_get) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_xline2_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_xline2_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->xline2_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_oblique_angle_set) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_oblique_angle_set(self,oblique_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_oblique_angle_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_oblique_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->oblique_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ALIGNED_oblique_angle_get) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ALIGNED_oblique_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ALIGNED_oblique_angle_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    result = (BITCODE_BD) ((arg1)->oblique_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_DIMENSION_ALIGNED) {
  {
    int argvi = 0;
    struct _dwg_entity_DIMENSION_ALIGNED *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_DIMENSION_ALIGNED();");
    }
    result = (struct _dwg_entity_DIMENSION_ALIGNED *)calloc(1, sizeof(struct _dwg_entity_DIMENSION_ALIGNED));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_DIMENSION_ALIGNED) {
  {
    struct _dwg_entity_DIMENSION_ALIGNED *arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_DIMENSION_ALIGNED(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_DIMENSION_ALIGNED" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ALIGNED *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ALIGNED *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_parent_set) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_parent_get) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_class_version_set) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_class_version_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_class_version_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_class_version_get) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_class_version_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    result = (BITCODE_RC) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_extrusion_set) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_extrusion_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_extrusion_get) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_extrusion_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_def_pt_set) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_def_pt_set(self,def_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_def_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_def_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_def_pt_get) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_def_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_def_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_text_midpt_set) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_text_midpt_set(self,text_midpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_text_midpt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_text_midpt_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->text_midpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_text_midpt_get) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_text_midpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_text_midpt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->text_midpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_elevation_set) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_elevation_set(self,elevation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_elevation_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_elevation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->elevation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_elevation_get) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_elevation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_elevation_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    result = (BITCODE_BD) ((arg1)->elevation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_flag_set) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_flag_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_flag_get) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_flag1_set) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_flag1_set(self,flag1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_flag1_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_flag1_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_flag1_get) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_flag1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_flag1_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag1);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_user_text_set) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_user_text_set(self,user_text);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_user_text_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_user_text_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->user_text);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->user_text = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->user_text = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_user_text_get) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_user_text_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_user_text_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    result = (BITCODE_TV) ((arg1)->user_text);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_text_rotation_set) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_text_rotation_set(self,text_rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_text_rotation_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_text_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->text_rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_text_rotation_get) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_text_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_text_rotation_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    result = (BITCODE_BD) ((arg1)->text_rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_horiz_dir_set) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_horiz_dir_set(self,horiz_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_horiz_dir_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_horiz_dir_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->horiz_dir = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_horiz_dir_get) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_horiz_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_horiz_dir_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    result = (BITCODE_BD) ((arg1)->horiz_dir);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_ins_scale_set) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_ins_scale_set(self,ins_scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_ins_scale_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_ins_scale_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ins_scale = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_ins_scale_get) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_ins_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_ins_scale_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ins_scale);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_ins_rotation_set) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_ins_rotation_set(self,ins_rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_ins_rotation_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_ins_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->ins_rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_ins_rotation_get) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_ins_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_ins_rotation_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    result = (BITCODE_BD) ((arg1)->ins_rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_attachment_set) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_attachment_set(self,attachment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_attachment_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_attachment_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->attachment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_attachment_get) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_attachment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_attachment_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    result = (BITCODE_BS) ((arg1)->attachment);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_lspace_style_set) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_lspace_style_set(self,lspace_style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_lspace_style_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_lspace_style_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->lspace_style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_lspace_style_get) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_lspace_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_lspace_style_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    result = (BITCODE_BS) ((arg1)->lspace_style);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_lspace_factor_set) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_lspace_factor_set(self,lspace_factor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_lspace_factor_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_lspace_factor_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->lspace_factor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_lspace_factor_get) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_lspace_factor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_lspace_factor_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    result = (BITCODE_BD) ((arg1)->lspace_factor);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_act_measurement_set) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_act_measurement_set(self,act_measurement);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_act_measurement_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_act_measurement_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->act_measurement = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_act_measurement_get) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_act_measurement_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_act_measurement_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    result = (BITCODE_BD) ((arg1)->act_measurement);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_unknown_set) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_unknown_set(self,unknown);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_unknown_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_unknown_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_unknown_get) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_unknown_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_unknown_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_flip_arrow1_set) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_flip_arrow1_set(self,flip_arrow1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_flip_arrow1_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_flip_arrow1_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->flip_arrow1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_flip_arrow1_get) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_flip_arrow1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_flip_arrow1_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    result = (BITCODE_B) ((arg1)->flip_arrow1);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_flip_arrow2_set) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_flip_arrow2_set(self,flip_arrow2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_flip_arrow2_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_flip_arrow2_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->flip_arrow2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_flip_arrow2_get) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_flip_arrow2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_flip_arrow2_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    result = (BITCODE_B) ((arg1)->flip_arrow2);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_clone_ins_pt_set) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_clone_ins_pt_set(self,clone_ins_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_clone_ins_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_clone_ins_pt_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->clone_ins_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_clone_ins_pt_get) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_clone_ins_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_clone_ins_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->clone_ins_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_dimstyle_set) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_dimstyle_set(self,dimstyle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_dimstyle_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_dimstyle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->dimstyle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_dimstyle_get) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_dimstyle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_dimstyle_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    result = (BITCODE_H) ((arg1)->dimstyle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_block_set) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_block_set(self,block);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_block_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_block_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->block = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_block_get) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_block_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_block_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    result = (BITCODE_H) ((arg1)->block);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_xline1_pt_set) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_xline1_pt_set(self,xline1_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_xline1_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_xline1_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->xline1_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_xline1_pt_get) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_xline1_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_xline1_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->xline1_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_xline2_pt_set) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_xline2_pt_set(self,xline2_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_xline2_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_xline2_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->xline2_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_xline2_pt_get) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_xline2_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_xline2_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->xline2_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_center_pt_set) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_center_pt_set(self,center_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_center_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_center_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->center_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_center_pt_get) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_center_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_center_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->center_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_xline2end_pt_set) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    BITCODE_3RD *arg2 = (BITCODE_3RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_xline2end_pt_set(self,xline2end_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_xline2end_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_xline2end_pt_set" "', argument " "2"" of type '" "BITCODE_3RD *""'"); 
    }
    arg2 = (BITCODE_3RD *)(argp2);
    if (arg1) (arg1)->xline2end_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG3PT_xline2end_pt_get) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG3PT_xline2end_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG3PT_xline2end_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    result = (BITCODE_3RD *)& ((arg1)->xline2end_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_DIMENSION_ANG3PT) {
  {
    int argvi = 0;
    struct _dwg_entity_DIMENSION_ANG3PT *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_DIMENSION_ANG3PT();");
    }
    result = (struct _dwg_entity_DIMENSION_ANG3PT *)calloc(1, sizeof(struct _dwg_entity_DIMENSION_ANG3PT));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_DIMENSION_ANG3PT) {
  {
    struct _dwg_entity_DIMENSION_ANG3PT *arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_DIMENSION_ANG3PT(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_DIMENSION_ANG3PT" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG3PT *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG3PT *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_parent_set) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_parent_get) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_class_version_set) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_class_version_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_class_version_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_class_version_get) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_class_version_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    result = (BITCODE_RC) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_extrusion_set) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_extrusion_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_extrusion_get) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_extrusion_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_def_pt_set) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_def_pt_set(self,def_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_def_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_def_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_def_pt_get) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_def_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_def_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_text_midpt_set) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_text_midpt_set(self,text_midpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_text_midpt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_text_midpt_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->text_midpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_text_midpt_get) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_text_midpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_text_midpt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->text_midpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_elevation_set) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_elevation_set(self,elevation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_elevation_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_elevation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->elevation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_elevation_get) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_elevation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_elevation_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    result = (BITCODE_BD) ((arg1)->elevation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_flag_set) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_flag_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_flag_get) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_flag1_set) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_flag1_set(self,flag1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_flag1_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_flag1_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_flag1_get) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_flag1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_flag1_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag1);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_user_text_set) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_user_text_set(self,user_text);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_user_text_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_user_text_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->user_text);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->user_text = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->user_text = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_user_text_get) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_user_text_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_user_text_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    result = (BITCODE_TV) ((arg1)->user_text);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_text_rotation_set) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_text_rotation_set(self,text_rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_text_rotation_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_text_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->text_rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_text_rotation_get) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_text_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_text_rotation_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    result = (BITCODE_BD) ((arg1)->text_rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_horiz_dir_set) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_horiz_dir_set(self,horiz_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_horiz_dir_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_horiz_dir_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->horiz_dir = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_horiz_dir_get) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_horiz_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_horiz_dir_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    result = (BITCODE_BD) ((arg1)->horiz_dir);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_ins_scale_set) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_ins_scale_set(self,ins_scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_ins_scale_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_ins_scale_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ins_scale = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_ins_scale_get) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_ins_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_ins_scale_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ins_scale);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_ins_rotation_set) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_ins_rotation_set(self,ins_rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_ins_rotation_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_ins_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->ins_rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_ins_rotation_get) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_ins_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_ins_rotation_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    result = (BITCODE_BD) ((arg1)->ins_rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_attachment_set) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_attachment_set(self,attachment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_attachment_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_attachment_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->attachment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_attachment_get) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_attachment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_attachment_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    result = (BITCODE_BS) ((arg1)->attachment);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_lspace_style_set) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_lspace_style_set(self,lspace_style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_lspace_style_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_lspace_style_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->lspace_style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_lspace_style_get) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_lspace_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_lspace_style_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    result = (BITCODE_BS) ((arg1)->lspace_style);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_lspace_factor_set) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_lspace_factor_set(self,lspace_factor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_lspace_factor_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_lspace_factor_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->lspace_factor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_lspace_factor_get) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_lspace_factor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_lspace_factor_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    result = (BITCODE_BD) ((arg1)->lspace_factor);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_act_measurement_set) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_act_measurement_set(self,act_measurement);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_act_measurement_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_act_measurement_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->act_measurement = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_act_measurement_get) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_act_measurement_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_act_measurement_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    result = (BITCODE_BD) ((arg1)->act_measurement);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_unknown_set) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_unknown_set(self,unknown);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_unknown_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_unknown_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_unknown_get) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_unknown_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_unknown_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_flip_arrow1_set) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_flip_arrow1_set(self,flip_arrow1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_flip_arrow1_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_flip_arrow1_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->flip_arrow1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_flip_arrow1_get) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_flip_arrow1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_flip_arrow1_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    result = (BITCODE_B) ((arg1)->flip_arrow1);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_flip_arrow2_set) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_flip_arrow2_set(self,flip_arrow2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_flip_arrow2_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_flip_arrow2_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->flip_arrow2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_flip_arrow2_get) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_flip_arrow2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_flip_arrow2_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    result = (BITCODE_B) ((arg1)->flip_arrow2);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_clone_ins_pt_set) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_clone_ins_pt_set(self,clone_ins_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_clone_ins_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_clone_ins_pt_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->clone_ins_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_clone_ins_pt_get) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_clone_ins_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_clone_ins_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->clone_ins_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_dimstyle_set) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_dimstyle_set(self,dimstyle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_dimstyle_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_dimstyle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->dimstyle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_dimstyle_get) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_dimstyle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_dimstyle_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    result = (BITCODE_H) ((arg1)->dimstyle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_block_set) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_block_set(self,block);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_block_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_block_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->block = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_block_get) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_block_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_block_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    result = (BITCODE_H) ((arg1)->block);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_xline1start_pt_set) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_xline1start_pt_set(self,xline1start_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_xline1start_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_xline1start_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->xline1start_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_xline1start_pt_get) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_xline1start_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_xline1start_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->xline1start_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_xline1end_pt_set) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_xline1end_pt_set(self,xline1end_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_xline1end_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_xline1end_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->xline1end_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_xline1end_pt_get) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_xline1end_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_xline1end_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->xline1end_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_xline2start_pt_set) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_xline2start_pt_set(self,xline2start_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_xline2start_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_xline2start_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->xline2start_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_xline2start_pt_get) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_xline2start_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_xline2start_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->xline2start_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_xline2end_pt_set) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_xline2end_pt_set(self,xline2end_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_xline2end_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_xline2end_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->xline2end_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_ANG2LN_xline2end_pt_get) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_ANG2LN_xline2end_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_ANG2LN_xline2end_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->xline2end_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_DIMENSION_ANG2LN) {
  {
    int argvi = 0;
    struct _dwg_entity_DIMENSION_ANG2LN *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_DIMENSION_ANG2LN();");
    }
    result = (struct _dwg_entity_DIMENSION_ANG2LN *)calloc(1, sizeof(struct _dwg_entity_DIMENSION_ANG2LN));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_DIMENSION_ANG2LN) {
  {
    struct _dwg_entity_DIMENSION_ANG2LN *arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_DIMENSION_ANG2LN(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_DIMENSION_ANG2LN" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_ANG2LN *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_ANG2LN *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_parent_set) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_RADIUS_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_parent_get) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_class_version_set) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_class_version_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_RADIUS_class_version_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_class_version_get) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_class_version_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    result = (BITCODE_RC) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_extrusion_set) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_extrusion_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_RADIUS_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_extrusion_get) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_extrusion_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_def_pt_set) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_def_pt_set(self,def_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_def_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_RADIUS_def_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_def_pt_get) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_def_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_def_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_text_midpt_set) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_text_midpt_set(self,text_midpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_text_midpt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_RADIUS_text_midpt_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->text_midpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_text_midpt_get) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_text_midpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_text_midpt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->text_midpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_elevation_set) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_elevation_set(self,elevation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_elevation_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_RADIUS_elevation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->elevation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_elevation_get) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_elevation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_elevation_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    result = (BITCODE_BD) ((arg1)->elevation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_flag_set) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_RADIUS_flag_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_flag_get) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_flag1_set) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_flag1_set(self,flag1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_flag1_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_RADIUS_flag1_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_flag1_get) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_flag1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_flag1_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag1);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_user_text_set) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_user_text_set(self,user_text);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_user_text_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_RADIUS_user_text_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->user_text);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->user_text = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->user_text = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_user_text_get) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_user_text_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_user_text_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    result = (BITCODE_TV) ((arg1)->user_text);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_text_rotation_set) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_text_rotation_set(self,text_rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_text_rotation_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_RADIUS_text_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->text_rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_text_rotation_get) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_text_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_text_rotation_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    result = (BITCODE_BD) ((arg1)->text_rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_horiz_dir_set) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_horiz_dir_set(self,horiz_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_horiz_dir_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_RADIUS_horiz_dir_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->horiz_dir = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_horiz_dir_get) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_horiz_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_horiz_dir_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    result = (BITCODE_BD) ((arg1)->horiz_dir);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_ins_scale_set) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_ins_scale_set(self,ins_scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_ins_scale_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_RADIUS_ins_scale_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ins_scale = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_ins_scale_get) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_ins_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_ins_scale_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ins_scale);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_ins_rotation_set) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_ins_rotation_set(self,ins_rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_ins_rotation_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_RADIUS_ins_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->ins_rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_ins_rotation_get) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_ins_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_ins_rotation_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    result = (BITCODE_BD) ((arg1)->ins_rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_attachment_set) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_attachment_set(self,attachment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_attachment_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_RADIUS_attachment_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->attachment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_attachment_get) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_attachment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_attachment_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    result = (BITCODE_BS) ((arg1)->attachment);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_lspace_style_set) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_lspace_style_set(self,lspace_style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_lspace_style_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_RADIUS_lspace_style_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->lspace_style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_lspace_style_get) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_lspace_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_lspace_style_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    result = (BITCODE_BS) ((arg1)->lspace_style);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_lspace_factor_set) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_lspace_factor_set(self,lspace_factor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_lspace_factor_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_RADIUS_lspace_factor_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->lspace_factor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_lspace_factor_get) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_lspace_factor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_lspace_factor_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    result = (BITCODE_BD) ((arg1)->lspace_factor);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_act_measurement_set) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_act_measurement_set(self,act_measurement);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_act_measurement_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_RADIUS_act_measurement_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->act_measurement = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_act_measurement_get) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_act_measurement_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_act_measurement_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    result = (BITCODE_BD) ((arg1)->act_measurement);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_unknown_set) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_unknown_set(self,unknown);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_unknown_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_RADIUS_unknown_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_unknown_get) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_unknown_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_unknown_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_flip_arrow1_set) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_flip_arrow1_set(self,flip_arrow1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_flip_arrow1_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_RADIUS_flip_arrow1_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->flip_arrow1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_flip_arrow1_get) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_flip_arrow1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_flip_arrow1_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    result = (BITCODE_B) ((arg1)->flip_arrow1);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_flip_arrow2_set) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_flip_arrow2_set(self,flip_arrow2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_flip_arrow2_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_RADIUS_flip_arrow2_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->flip_arrow2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_flip_arrow2_get) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_flip_arrow2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_flip_arrow2_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    result = (BITCODE_B) ((arg1)->flip_arrow2);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_clone_ins_pt_set) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_clone_ins_pt_set(self,clone_ins_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_clone_ins_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_RADIUS_clone_ins_pt_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->clone_ins_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_clone_ins_pt_get) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_clone_ins_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_clone_ins_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->clone_ins_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_dimstyle_set) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_dimstyle_set(self,dimstyle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_dimstyle_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_RADIUS_dimstyle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->dimstyle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_dimstyle_get) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_dimstyle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_dimstyle_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    result = (BITCODE_H) ((arg1)->dimstyle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_block_set) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_block_set(self,block);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_block_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_RADIUS_block_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->block = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_block_get) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_block_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_block_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    result = (BITCODE_H) ((arg1)->block);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_first_arc_pt_set) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_first_arc_pt_set(self,first_arc_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_first_arc_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_RADIUS_first_arc_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->first_arc_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_first_arc_pt_get) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_first_arc_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_first_arc_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->first_arc_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_leader_len_set) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_leader_len_set(self,leader_len);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_leader_len_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_RADIUS_leader_len_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->leader_len = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_RADIUS_leader_len_get) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_RADIUS_leader_len_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_RADIUS_leader_len_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    result = (BITCODE_BD) ((arg1)->leader_len);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_DIMENSION_RADIUS) {
  {
    int argvi = 0;
    struct _dwg_entity_DIMENSION_RADIUS *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_DIMENSION_RADIUS();");
    }
    result = (struct _dwg_entity_DIMENSION_RADIUS *)calloc(1, sizeof(struct _dwg_entity_DIMENSION_RADIUS));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_DIMENSION_RADIUS) {
  {
    struct _dwg_entity_DIMENSION_RADIUS *arg1 = (struct _dwg_entity_DIMENSION_RADIUS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_DIMENSION_RADIUS(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_DIMENSION_RADIUS" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_RADIUS *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_RADIUS *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_parent_set) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_parent_get) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_class_version_set) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_class_version_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_class_version_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_class_version_get) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_class_version_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    result = (BITCODE_RC) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_extrusion_set) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_extrusion_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_extrusion_get) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_extrusion_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_def_pt_set) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_def_pt_set(self,def_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_def_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_def_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_def_pt_get) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_def_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_def_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_text_midpt_set) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_text_midpt_set(self,text_midpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_text_midpt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_text_midpt_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->text_midpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_text_midpt_get) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_text_midpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_text_midpt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->text_midpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_elevation_set) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_elevation_set(self,elevation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_elevation_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_elevation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->elevation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_elevation_get) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_elevation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_elevation_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    result = (BITCODE_BD) ((arg1)->elevation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_flag_set) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_flag_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_flag_get) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_flag1_set) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_flag1_set(self,flag1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_flag1_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_flag1_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_flag1_get) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_flag1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_flag1_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag1);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_user_text_set) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_user_text_set(self,user_text);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_user_text_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_user_text_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->user_text);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->user_text = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->user_text = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_user_text_get) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_user_text_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_user_text_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->user_text);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_text_rotation_set) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_text_rotation_set(self,text_rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_text_rotation_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_text_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->text_rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_text_rotation_get) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_text_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_text_rotation_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    result = (BITCODE_BD) ((arg1)->text_rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_horiz_dir_set) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_horiz_dir_set(self,horiz_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_horiz_dir_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_horiz_dir_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->horiz_dir = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_horiz_dir_get) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_horiz_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_horiz_dir_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    result = (BITCODE_BD) ((arg1)->horiz_dir);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_ins_scale_set) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_ins_scale_set(self,ins_scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_ins_scale_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_ins_scale_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ins_scale = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_ins_scale_get) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_ins_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_ins_scale_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ins_scale);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_ins_rotation_set) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_ins_rotation_set(self,ins_rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_ins_rotation_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_ins_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->ins_rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_ins_rotation_get) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_ins_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_ins_rotation_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    result = (BITCODE_BD) ((arg1)->ins_rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_attachment_set) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_attachment_set(self,attachment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_attachment_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_attachment_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->attachment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_attachment_get) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_attachment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_attachment_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    result = (BITCODE_BS) ((arg1)->attachment);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_lspace_style_set) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_lspace_style_set(self,lspace_style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_lspace_style_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_lspace_style_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->lspace_style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_lspace_style_get) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_lspace_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_lspace_style_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    result = (BITCODE_BS) ((arg1)->lspace_style);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_lspace_factor_set) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_lspace_factor_set(self,lspace_factor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_lspace_factor_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_lspace_factor_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->lspace_factor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_lspace_factor_get) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_lspace_factor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_lspace_factor_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    result = (BITCODE_BD) ((arg1)->lspace_factor);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_act_measurement_set) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_act_measurement_set(self,act_measurement);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_act_measurement_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_act_measurement_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->act_measurement = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_act_measurement_get) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_act_measurement_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_act_measurement_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    result = (BITCODE_BD) ((arg1)->act_measurement);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_unknown_set) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_unknown_set(self,unknown);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_unknown_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_unknown_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_unknown_get) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_unknown_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_unknown_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_flip_arrow1_set) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_flip_arrow1_set(self,flip_arrow1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_flip_arrow1_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_flip_arrow1_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->flip_arrow1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_flip_arrow1_get) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_flip_arrow1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_flip_arrow1_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->flip_arrow1);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_flip_arrow2_set) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_flip_arrow2_set(self,flip_arrow2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_flip_arrow2_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_flip_arrow2_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->flip_arrow2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_flip_arrow2_get) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_flip_arrow2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_flip_arrow2_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->flip_arrow2);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_clone_ins_pt_set) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_clone_ins_pt_set(self,clone_ins_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_clone_ins_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_clone_ins_pt_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->clone_ins_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_clone_ins_pt_get) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_clone_ins_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_clone_ins_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->clone_ins_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_dimstyle_set) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_dimstyle_set(self,dimstyle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_dimstyle_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_dimstyle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->dimstyle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_dimstyle_get) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_dimstyle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_dimstyle_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    result = (BITCODE_H) ((arg1)->dimstyle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_block_set) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_block_set(self,block);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_block_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_block_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->block = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_block_get) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_block_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_block_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    result = (BITCODE_H) ((arg1)->block);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_first_arc_pt_set) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_first_arc_pt_set(self,first_arc_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_first_arc_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_first_arc_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->first_arc_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_first_arc_pt_get) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_first_arc_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_first_arc_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->first_arc_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_leader_len_set) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_leader_len_set(self,leader_len);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_leader_len_set" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_leader_len_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->leader_len = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_DIMENSION_DIAMETER_leader_len_get) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_DIMENSION_DIAMETER_leader_len_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_DIMENSION_DIAMETER_leader_len_get" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    result = (BITCODE_BD) ((arg1)->leader_len);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_DIMENSION_DIAMETER) {
  {
    int argvi = 0;
    struct _dwg_entity_DIMENSION_DIAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_DIMENSION_DIAMETER();");
    }
    result = (struct _dwg_entity_DIMENSION_DIAMETER *)calloc(1, sizeof(struct _dwg_entity_DIMENSION_DIAMETER));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_DIMENSION_DIAMETER) {
  {
    struct _dwg_entity_DIMENSION_DIAMETER *arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_DIMENSION_DIAMETER(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_DIMENSION_DIAMETER" "', argument " "1"" of type '" "struct _dwg_entity_DIMENSION_DIAMETER *""'"); 
    }
    arg1 = (struct _dwg_entity_DIMENSION_DIAMETER *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_parent_set) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ARC_DIMENSION_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_parent_get) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_class_version_set) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_class_version_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARC_DIMENSION_class_version_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_class_version_get) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_class_version_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    result = (BITCODE_RC) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_extrusion_set) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_extrusion_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ARC_DIMENSION_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_extrusion_get) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_extrusion_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_def_pt_set) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_def_pt_set(self,def_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_def_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ARC_DIMENSION_def_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_def_pt_get) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_def_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_def_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_text_midpt_set) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_text_midpt_set(self,text_midpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_text_midpt_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ARC_DIMENSION_text_midpt_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->text_midpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_text_midpt_get) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_text_midpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_text_midpt_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->text_midpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_elevation_set) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_elevation_set(self,elevation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_elevation_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARC_DIMENSION_elevation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->elevation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_elevation_get) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_elevation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_elevation_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    result = (BITCODE_BD) ((arg1)->elevation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_flag_set) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARC_DIMENSION_flag_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_flag_get) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_flag1_set) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_flag1_set(self,flag1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_flag1_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARC_DIMENSION_flag1_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_flag1_get) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_flag1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_flag1_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag1);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_user_text_set) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_user_text_set(self,user_text);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_user_text_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ARC_DIMENSION_user_text_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->user_text);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->user_text = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->user_text = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_user_text_get) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_user_text_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_user_text_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    result = (BITCODE_TV) ((arg1)->user_text);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_text_rotation_set) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_text_rotation_set(self,text_rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_text_rotation_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARC_DIMENSION_text_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->text_rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_text_rotation_get) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_text_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_text_rotation_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    result = (BITCODE_BD) ((arg1)->text_rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_horiz_dir_set) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_horiz_dir_set(self,horiz_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_horiz_dir_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARC_DIMENSION_horiz_dir_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->horiz_dir = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_horiz_dir_get) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_horiz_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_horiz_dir_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    result = (BITCODE_BD) ((arg1)->horiz_dir);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_ins_scale_set) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_ins_scale_set(self,ins_scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_ins_scale_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ARC_DIMENSION_ins_scale_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ins_scale = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_ins_scale_get) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_ins_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_ins_scale_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ins_scale);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_ins_rotation_set) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_ins_rotation_set(self,ins_rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_ins_rotation_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARC_DIMENSION_ins_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->ins_rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_ins_rotation_get) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_ins_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_ins_rotation_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    result = (BITCODE_BD) ((arg1)->ins_rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_attachment_set) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_attachment_set(self,attachment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_attachment_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARC_DIMENSION_attachment_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->attachment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_attachment_get) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_attachment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_attachment_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    result = (BITCODE_BS) ((arg1)->attachment);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_lspace_style_set) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_lspace_style_set(self,lspace_style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_lspace_style_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARC_DIMENSION_lspace_style_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->lspace_style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_lspace_style_get) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_lspace_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_lspace_style_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    result = (BITCODE_BS) ((arg1)->lspace_style);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_lspace_factor_set) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_lspace_factor_set(self,lspace_factor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_lspace_factor_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARC_DIMENSION_lspace_factor_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->lspace_factor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_lspace_factor_get) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_lspace_factor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_lspace_factor_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    result = (BITCODE_BD) ((arg1)->lspace_factor);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_act_measurement_set) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_act_measurement_set(self,act_measurement);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_act_measurement_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARC_DIMENSION_act_measurement_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->act_measurement = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_act_measurement_get) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_act_measurement_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_act_measurement_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    result = (BITCODE_BD) ((arg1)->act_measurement);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_unknown_set) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_unknown_set(self,unknown);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_unknown_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARC_DIMENSION_unknown_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_unknown_get) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_unknown_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_unknown_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_flip_arrow1_set) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_flip_arrow1_set(self,flip_arrow1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_flip_arrow1_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARC_DIMENSION_flip_arrow1_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->flip_arrow1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_flip_arrow1_get) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_flip_arrow1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_flip_arrow1_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    result = (BITCODE_B) ((arg1)->flip_arrow1);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_flip_arrow2_set) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_flip_arrow2_set(self,flip_arrow2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_flip_arrow2_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARC_DIMENSION_flip_arrow2_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->flip_arrow2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_flip_arrow2_get) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_flip_arrow2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_flip_arrow2_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    result = (BITCODE_B) ((arg1)->flip_arrow2);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_clone_ins_pt_set) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_clone_ins_pt_set(self,clone_ins_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_clone_ins_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ARC_DIMENSION_clone_ins_pt_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->clone_ins_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_clone_ins_pt_get) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_clone_ins_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_clone_ins_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->clone_ins_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_dimstyle_set) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_dimstyle_set(self,dimstyle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_dimstyle_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ARC_DIMENSION_dimstyle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->dimstyle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_dimstyle_get) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_dimstyle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_dimstyle_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    result = (BITCODE_H) ((arg1)->dimstyle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_block_set) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_block_set(self,block);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_block_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ARC_DIMENSION_block_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->block = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_block_get) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_block_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_block_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    result = (BITCODE_H) ((arg1)->block);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_xline1_pt_set) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_xline1_pt_set(self,xline1_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_xline1_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ARC_DIMENSION_xline1_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->xline1_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_xline1_pt_get) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_xline1_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_xline1_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->xline1_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_xline2_pt_set) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_xline2_pt_set(self,xline2_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_xline2_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ARC_DIMENSION_xline2_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->xline2_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_xline2_pt_get) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_xline2_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_xline2_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->xline2_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_center_pt_set) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_center_pt_set(self,center_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_center_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ARC_DIMENSION_center_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->center_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_center_pt_get) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_center_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_center_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->center_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_is_partial_set) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_is_partial_set(self,is_partial);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_is_partial_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARC_DIMENSION_is_partial_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_partial = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_is_partial_get) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_is_partial_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_is_partial_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    result = (BITCODE_B) ((arg1)->is_partial);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_arc_start_param_set) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_arc_start_param_set(self,arc_start_param);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_arc_start_param_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARC_DIMENSION_arc_start_param_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->arc_start_param = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_arc_start_param_get) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_arc_start_param_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_arc_start_param_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    result = (BITCODE_BD) ((arg1)->arc_start_param);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_arc_end_param_set) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_arc_end_param_set(self,arc_end_param);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_arc_end_param_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARC_DIMENSION_arc_end_param_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->arc_end_param = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_arc_end_param_get) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_arc_end_param_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_arc_end_param_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    result = (BITCODE_BD) ((arg1)->arc_end_param);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_has_leader_set) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_has_leader_set(self,has_leader);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_has_leader_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARC_DIMENSION_has_leader_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_leader = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_has_leader_get) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_has_leader_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_has_leader_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    result = (BITCODE_B) ((arg1)->has_leader);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_leader1_pt_set) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_leader1_pt_set(self,leader1_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_leader1_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ARC_DIMENSION_leader1_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->leader1_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_leader1_pt_get) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_leader1_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_leader1_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->leader1_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_leader2_pt_set) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_leader2_pt_set(self,leader2_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_leader2_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ARC_DIMENSION_leader2_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->leader2_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARC_DIMENSION_leader2_pt_get) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARC_DIMENSION_leader2_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARC_DIMENSION_leader2_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->leader2_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_ARC_DIMENSION) {
  {
    int argvi = 0;
    struct _dwg_entity_ARC_DIMENSION *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_ARC_DIMENSION();");
    }
    result = (struct _dwg_entity_ARC_DIMENSION *)calloc(1, sizeof(struct _dwg_entity_ARC_DIMENSION));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_ARC_DIMENSION, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_ARC_DIMENSION) {
  {
    struct _dwg_entity_ARC_DIMENSION *arg1 = (struct _dwg_entity_ARC_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_ARC_DIMENSION(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_ARC_DIMENSION" "', argument " "1"" of type '" "struct _dwg_entity_ARC_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_ARC_DIMENSION *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_parent_set) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_parent_get) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_class_version_set) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_class_version_set" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_class_version_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_class_version_get) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_class_version_get" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    result = (BITCODE_RC) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_extrusion_set) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_extrusion_set" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_extrusion_get) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_extrusion_get" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_def_pt_set) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_def_pt_set(self,def_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_def_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_def_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_def_pt_get) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_def_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_def_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_text_midpt_set) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_text_midpt_set(self,text_midpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_text_midpt_set" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_text_midpt_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->text_midpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_text_midpt_get) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_text_midpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_text_midpt_get" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->text_midpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_elevation_set) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_elevation_set(self,elevation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_elevation_set" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_elevation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->elevation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_elevation_get) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_elevation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_elevation_get" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    result = (BITCODE_BD) ((arg1)->elevation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_flag_set) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_flag_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_flag_get) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_flag1_set) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_flag1_set(self,flag1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_flag1_set" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_flag1_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_flag1_get) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_flag1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_flag1_get" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag1);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_user_text_set) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_user_text_set(self,user_text);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_user_text_set" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_user_text_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->user_text);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->user_text = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->user_text = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_user_text_get) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_user_text_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_user_text_get" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    result = (BITCODE_TV) ((arg1)->user_text);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_text_rotation_set) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_text_rotation_set(self,text_rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_text_rotation_set" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_text_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->text_rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_text_rotation_get) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_text_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_text_rotation_get" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    result = (BITCODE_BD) ((arg1)->text_rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_horiz_dir_set) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_horiz_dir_set(self,horiz_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_horiz_dir_set" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_horiz_dir_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->horiz_dir = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_horiz_dir_get) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_horiz_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_horiz_dir_get" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    result = (BITCODE_BD) ((arg1)->horiz_dir);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_ins_scale_set) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_ins_scale_set(self,ins_scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_ins_scale_set" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_ins_scale_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ins_scale = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_ins_scale_get) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_ins_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_ins_scale_get" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ins_scale);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_ins_rotation_set) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_ins_rotation_set(self,ins_rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_ins_rotation_set" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_ins_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->ins_rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_ins_rotation_get) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_ins_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_ins_rotation_get" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    result = (BITCODE_BD) ((arg1)->ins_rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_attachment_set) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_attachment_set(self,attachment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_attachment_set" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_attachment_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->attachment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_attachment_get) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_attachment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_attachment_get" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    result = (BITCODE_BS) ((arg1)->attachment);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_lspace_style_set) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_lspace_style_set(self,lspace_style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_lspace_style_set" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_lspace_style_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->lspace_style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_lspace_style_get) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_lspace_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_lspace_style_get" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    result = (BITCODE_BS) ((arg1)->lspace_style);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_lspace_factor_set) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_lspace_factor_set(self,lspace_factor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_lspace_factor_set" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_lspace_factor_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->lspace_factor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_lspace_factor_get) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_lspace_factor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_lspace_factor_get" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    result = (BITCODE_BD) ((arg1)->lspace_factor);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_act_measurement_set) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_act_measurement_set(self,act_measurement);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_act_measurement_set" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_act_measurement_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->act_measurement = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_act_measurement_get) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_act_measurement_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_act_measurement_get" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    result = (BITCODE_BD) ((arg1)->act_measurement);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_unknown_set) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_unknown_set(self,unknown);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_unknown_set" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_unknown_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_unknown_get) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_unknown_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_unknown_get" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_flip_arrow1_set) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_flip_arrow1_set(self,flip_arrow1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_flip_arrow1_set" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_flip_arrow1_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->flip_arrow1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_flip_arrow1_get) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_flip_arrow1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_flip_arrow1_get" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    result = (BITCODE_B) ((arg1)->flip_arrow1);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_flip_arrow2_set) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_flip_arrow2_set(self,flip_arrow2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_flip_arrow2_set" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_flip_arrow2_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->flip_arrow2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_flip_arrow2_get) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_flip_arrow2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_flip_arrow2_get" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    result = (BITCODE_B) ((arg1)->flip_arrow2);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_clone_ins_pt_set) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_clone_ins_pt_set(self,clone_ins_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_clone_ins_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_clone_ins_pt_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->clone_ins_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_clone_ins_pt_get) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_clone_ins_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_clone_ins_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->clone_ins_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_dimstyle_set) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_dimstyle_set(self,dimstyle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_dimstyle_set" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_dimstyle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->dimstyle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_dimstyle_get) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_dimstyle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_dimstyle_get" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    result = (BITCODE_H) ((arg1)->dimstyle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_block_set) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_block_set(self,block);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_block_set" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_block_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->block = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_block_get) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_block_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_block_get" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    result = (BITCODE_H) ((arg1)->block);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_jog_angle_set) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_jog_angle_set(self,jog_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_jog_angle_set" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_jog_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->jog_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_jog_angle_get) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_jog_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_jog_angle_get" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    result = (BITCODE_BD) ((arg1)->jog_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_jog_pt_set) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_jog_pt_set(self,jog_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_jog_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_jog_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->jog_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_jog_pt_get) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_jog_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_jog_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->jog_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_ovr_center_set) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_ovr_center_set(self,ovr_center);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_ovr_center_set" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_ovr_center_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ovr_center = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_ovr_center_get) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_ovr_center_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_ovr_center_get" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ovr_center);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_chord_pt_set) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_chord_pt_set(self,chord_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_chord_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_chord_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->chord_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_chord_pt_get) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LARGE_RADIAL_DIMENSION_chord_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LARGE_RADIAL_DIMENSION_chord_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->chord_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_LARGE_RADIAL_DIMENSION) {
  {
    int argvi = 0;
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_LARGE_RADIAL_DIMENSION();");
    }
    result = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)calloc(1, sizeof(struct _dwg_entity_LARGE_RADIAL_DIMENSION));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_LARGE_RADIAL_DIMENSION) {
  {
    struct _dwg_entity_LARGE_RADIAL_DIMENSION *arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_LARGE_RADIAL_DIMENSION(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_LARGE_RADIAL_DIMENSION" "', argument " "1"" of type '" "struct _dwg_entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg1 = (struct _dwg_entity_LARGE_RADIAL_DIMENSION *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINT_parent_set) {
  {
    struct _dwg_entity_POINT *arg1 = (struct _dwg_entity_POINT *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINT_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINT_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_POINT *""'"); 
    }
    arg1 = (struct _dwg_entity_POINT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POINT_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINT_parent_get) {
  {
    struct _dwg_entity_POINT *arg1 = (struct _dwg_entity_POINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINT_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINT_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_POINT *""'"); 
    }
    arg1 = (struct _dwg_entity_POINT *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINT_x_set) {
  {
    struct _dwg_entity_POINT *arg1 = (struct _dwg_entity_POINT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINT_x_set(self,x);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINT_x_set" "', argument " "1"" of type '" "struct _dwg_entity_POINT *""'"); 
    }
    arg1 = (struct _dwg_entity_POINT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POINT_x_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->x = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINT_x_get) {
  {
    struct _dwg_entity_POINT *arg1 = (struct _dwg_entity_POINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINT_x_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINT_x_get" "', argument " "1"" of type '" "struct _dwg_entity_POINT *""'"); 
    }
    arg1 = (struct _dwg_entity_POINT *)(argp1);
    result = (BITCODE_BD) ((arg1)->x);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINT_y_set) {
  {
    struct _dwg_entity_POINT *arg1 = (struct _dwg_entity_POINT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINT_y_set(self,y);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINT_y_set" "', argument " "1"" of type '" "struct _dwg_entity_POINT *""'"); 
    }
    arg1 = (struct _dwg_entity_POINT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POINT_y_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->y = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINT_y_get) {
  {
    struct _dwg_entity_POINT *arg1 = (struct _dwg_entity_POINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINT_y_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINT_y_get" "', argument " "1"" of type '" "struct _dwg_entity_POINT *""'"); 
    }
    arg1 = (struct _dwg_entity_POINT *)(argp1);
    result = (BITCODE_BD) ((arg1)->y);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINT_z_set) {
  {
    struct _dwg_entity_POINT *arg1 = (struct _dwg_entity_POINT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINT_z_set(self,z);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINT_z_set" "', argument " "1"" of type '" "struct _dwg_entity_POINT *""'"); 
    }
    arg1 = (struct _dwg_entity_POINT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POINT_z_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->z = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINT_z_get) {
  {
    struct _dwg_entity_POINT *arg1 = (struct _dwg_entity_POINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINT_z_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINT_z_get" "', argument " "1"" of type '" "struct _dwg_entity_POINT *""'"); 
    }
    arg1 = (struct _dwg_entity_POINT *)(argp1);
    result = (BITCODE_BD) ((arg1)->z);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINT_thickness_set) {
  {
    struct _dwg_entity_POINT *arg1 = (struct _dwg_entity_POINT *) 0 ;
    BITCODE_BT arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINT_thickness_set(self,thickness);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINT_thickness_set" "', argument " "1"" of type '" "struct _dwg_entity_POINT *""'"); 
    }
    arg1 = (struct _dwg_entity_POINT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POINT_thickness_set" "', argument " "2"" of type '" "BITCODE_BT""'");
    } 
    arg2 = (BITCODE_BT)(val2);
    if (arg1) (arg1)->thickness = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINT_thickness_get) {
  {
    struct _dwg_entity_POINT *arg1 = (struct _dwg_entity_POINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BT result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINT_thickness_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINT_thickness_get" "', argument " "1"" of type '" "struct _dwg_entity_POINT *""'"); 
    }
    arg1 = (struct _dwg_entity_POINT *)(argp1);
    result = (BITCODE_BT) ((arg1)->thickness);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINT_extrusion_set) {
  {
    struct _dwg_entity_POINT *arg1 = (struct _dwg_entity_POINT *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINT_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINT_extrusion_set" "', argument " "1"" of type '" "struct _dwg_entity_POINT *""'"); 
    }
    arg1 = (struct _dwg_entity_POINT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POINT_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINT_extrusion_get) {
  {
    struct _dwg_entity_POINT *arg1 = (struct _dwg_entity_POINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINT_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINT_extrusion_get" "', argument " "1"" of type '" "struct _dwg_entity_POINT *""'"); 
    }
    arg1 = (struct _dwg_entity_POINT *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINT_x_ang_set) {
  {
    struct _dwg_entity_POINT *arg1 = (struct _dwg_entity_POINT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINT_x_ang_set(self,x_ang);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINT_x_ang_set" "', argument " "1"" of type '" "struct _dwg_entity_POINT *""'"); 
    }
    arg1 = (struct _dwg_entity_POINT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POINT_x_ang_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->x_ang = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINT_x_ang_get) {
  {
    struct _dwg_entity_POINT *arg1 = (struct _dwg_entity_POINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINT_x_ang_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINT_x_ang_get" "', argument " "1"" of type '" "struct _dwg_entity_POINT *""'"); 
    }
    arg1 = (struct _dwg_entity_POINT *)(argp1);
    result = (BITCODE_BD) ((arg1)->x_ang);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_POINT) {
  {
    int argvi = 0;
    struct _dwg_entity_POINT *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_POINT();");
    }
    result = (struct _dwg_entity_POINT *)calloc(1, sizeof(struct _dwg_entity_POINT));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_POINT, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_POINT) {
  {
    struct _dwg_entity_POINT *arg1 = (struct _dwg_entity_POINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_POINT(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_POINT" "', argument " "1"" of type '" "struct _dwg_entity_POINT *""'"); 
    }
    arg1 = (struct _dwg_entity_POINT *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DFACE_parent_set) {
  {
    struct _dwg_entity_3DFACE *arg1 = (struct _dwg_entity_3DFACE *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DFACE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DFACE_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_3DFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_3DFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity__3DFACE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DFACE_parent_get) {
  {
    struct _dwg_entity_3DFACE *arg1 = (struct _dwg_entity_3DFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DFACE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DFACE_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_3DFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_3DFACE *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DFACE_has_no_flags_set) {
  {
    struct _dwg_entity_3DFACE *arg1 = (struct _dwg_entity_3DFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DFACE_has_no_flags_set(self,has_no_flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DFACE_has_no_flags_set" "', argument " "1"" of type '" "struct _dwg_entity_3DFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_3DFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity__3DFACE_has_no_flags_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_no_flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DFACE_has_no_flags_get) {
  {
    struct _dwg_entity_3DFACE *arg1 = (struct _dwg_entity_3DFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DFACE_has_no_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DFACE_has_no_flags_get" "', argument " "1"" of type '" "struct _dwg_entity_3DFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_3DFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->has_no_flags);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DFACE_z_is_zero_set) {
  {
    struct _dwg_entity_3DFACE *arg1 = (struct _dwg_entity_3DFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DFACE_z_is_zero_set(self,z_is_zero);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DFACE_z_is_zero_set" "', argument " "1"" of type '" "struct _dwg_entity_3DFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_3DFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity__3DFACE_z_is_zero_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->z_is_zero = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DFACE_z_is_zero_get) {
  {
    struct _dwg_entity_3DFACE *arg1 = (struct _dwg_entity_3DFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DFACE_z_is_zero_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DFACE_z_is_zero_get" "', argument " "1"" of type '" "struct _dwg_entity_3DFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_3DFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->z_is_zero);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DFACE_corner1_set) {
  {
    struct _dwg_entity_3DFACE *arg1 = (struct _dwg_entity_3DFACE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DFACE_corner1_set(self,corner1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DFACE_corner1_set" "', argument " "1"" of type '" "struct _dwg_entity_3DFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_3DFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity__3DFACE_corner1_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->corner1 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DFACE_corner1_get) {
  {
    struct _dwg_entity_3DFACE *arg1 = (struct _dwg_entity_3DFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DFACE_corner1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DFACE_corner1_get" "', argument " "1"" of type '" "struct _dwg_entity_3DFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_3DFACE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->corner1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DFACE_corner2_set) {
  {
    struct _dwg_entity_3DFACE *arg1 = (struct _dwg_entity_3DFACE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DFACE_corner2_set(self,corner2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DFACE_corner2_set" "', argument " "1"" of type '" "struct _dwg_entity_3DFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_3DFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity__3DFACE_corner2_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->corner2 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DFACE_corner2_get) {
  {
    struct _dwg_entity_3DFACE *arg1 = (struct _dwg_entity_3DFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DFACE_corner2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DFACE_corner2_get" "', argument " "1"" of type '" "struct _dwg_entity_3DFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_3DFACE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->corner2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DFACE_corner3_set) {
  {
    struct _dwg_entity_3DFACE *arg1 = (struct _dwg_entity_3DFACE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DFACE_corner3_set(self,corner3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DFACE_corner3_set" "', argument " "1"" of type '" "struct _dwg_entity_3DFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_3DFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity__3DFACE_corner3_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->corner3 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DFACE_corner3_get) {
  {
    struct _dwg_entity_3DFACE *arg1 = (struct _dwg_entity_3DFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DFACE_corner3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DFACE_corner3_get" "', argument " "1"" of type '" "struct _dwg_entity_3DFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_3DFACE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->corner3);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DFACE_corner4_set) {
  {
    struct _dwg_entity_3DFACE *arg1 = (struct _dwg_entity_3DFACE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DFACE_corner4_set(self,corner4);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DFACE_corner4_set" "', argument " "1"" of type '" "struct _dwg_entity_3DFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_3DFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity__3DFACE_corner4_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->corner4 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DFACE_corner4_get) {
  {
    struct _dwg_entity_3DFACE *arg1 = (struct _dwg_entity_3DFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DFACE_corner4_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DFACE_corner4_get" "', argument " "1"" of type '" "struct _dwg_entity_3DFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_3DFACE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->corner4);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DFACE_invis_flags_set) {
  {
    struct _dwg_entity_3DFACE *arg1 = (struct _dwg_entity_3DFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DFACE_invis_flags_set(self,invis_flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DFACE_invis_flags_set" "', argument " "1"" of type '" "struct _dwg_entity_3DFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_3DFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity__3DFACE_invis_flags_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->invis_flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DFACE_invis_flags_get) {
  {
    struct _dwg_entity_3DFACE *arg1 = (struct _dwg_entity_3DFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DFACE_invis_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DFACE_invis_flags_get" "', argument " "1"" of type '" "struct _dwg_entity_3DFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_3DFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->invis_flags);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity__3DFACE) {
  {
    int argvi = 0;
    struct _dwg_entity_3DFACE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity__3DFACE();");
    }
    result = (struct _dwg_entity_3DFACE *)calloc(1, sizeof(struct _dwg_entity_3DFACE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_3DFACE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity__3DFACE) {
  {
    struct _dwg_entity_3DFACE *arg1 = (struct _dwg_entity_3DFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity__3DFACE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DFACE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity__3DFACE" "', argument " "1"" of type '" "struct _dwg_entity_3DFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_3DFACE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_PFACE_parent_set) {
  {
    struct _dwg_entity_POLYLINE_PFACE *arg1 = (struct _dwg_entity_POLYLINE_PFACE *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_PFACE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_PFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_PFACE_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_PFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_PFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POLYLINE_PFACE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_PFACE_parent_get) {
  {
    struct _dwg_entity_POLYLINE_PFACE *arg1 = (struct _dwg_entity_POLYLINE_PFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_PFACE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_PFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_PFACE_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_PFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_PFACE *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_PFACE_has_vertex_set) {
  {
    struct _dwg_entity_POLYLINE_PFACE *arg1 = (struct _dwg_entity_POLYLINE_PFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_PFACE_has_vertex_set(self,has_vertex);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_PFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_PFACE_has_vertex_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_PFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_PFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POLYLINE_PFACE_has_vertex_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_vertex = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_PFACE_has_vertex_get) {
  {
    struct _dwg_entity_POLYLINE_PFACE *arg1 = (struct _dwg_entity_POLYLINE_PFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_PFACE_has_vertex_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_PFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_PFACE_has_vertex_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_PFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_PFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->has_vertex);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_PFACE_num_owned_set) {
  {
    struct _dwg_entity_POLYLINE_PFACE *arg1 = (struct _dwg_entity_POLYLINE_PFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_PFACE_num_owned_set(self,num_owned);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_PFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_PFACE_num_owned_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_PFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_PFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POLYLINE_PFACE_num_owned_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_owned = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_PFACE_num_owned_get) {
  {
    struct _dwg_entity_POLYLINE_PFACE *arg1 = (struct _dwg_entity_POLYLINE_PFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_PFACE_num_owned_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_PFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_PFACE_num_owned_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_PFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_PFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_owned);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_PFACE_first_vertex_set) {
  {
    struct _dwg_entity_POLYLINE_PFACE *arg1 = (struct _dwg_entity_POLYLINE_PFACE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_PFACE_first_vertex_set(self,first_vertex);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_PFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_PFACE_first_vertex_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_PFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_PFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POLYLINE_PFACE_first_vertex_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->first_vertex = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_PFACE_first_vertex_get) {
  {
    struct _dwg_entity_POLYLINE_PFACE *arg1 = (struct _dwg_entity_POLYLINE_PFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_PFACE_first_vertex_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_PFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_PFACE_first_vertex_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_PFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_PFACE *)(argp1);
    result = (BITCODE_H) ((arg1)->first_vertex);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_PFACE_last_vertex_set) {
  {
    struct _dwg_entity_POLYLINE_PFACE *arg1 = (struct _dwg_entity_POLYLINE_PFACE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_PFACE_last_vertex_set(self,last_vertex);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_PFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_PFACE_last_vertex_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_PFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_PFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POLYLINE_PFACE_last_vertex_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->last_vertex = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_PFACE_last_vertex_get) {
  {
    struct _dwg_entity_POLYLINE_PFACE *arg1 = (struct _dwg_entity_POLYLINE_PFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_PFACE_last_vertex_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_PFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_PFACE_last_vertex_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_PFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_PFACE *)(argp1);
    result = (BITCODE_H) ((arg1)->last_vertex);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_PFACE_vertex_set) {
  {
    struct _dwg_entity_POLYLINE_PFACE *arg1 = (struct _dwg_entity_POLYLINE_PFACE *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_PFACE_vertex_set(self,vertex);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_PFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_PFACE_vertex_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_PFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_PFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POLYLINE_PFACE_vertex_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->vertex = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_PFACE_vertex_get) {
  {
    struct _dwg_entity_POLYLINE_PFACE *arg1 = (struct _dwg_entity_POLYLINE_PFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_PFACE_vertex_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_PFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_PFACE_vertex_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_PFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_PFACE *)(argp1);
    result = (BITCODE_H *) ((arg1)->vertex);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_PFACE_seqend_set) {
  {
    struct _dwg_entity_POLYLINE_PFACE *arg1 = (struct _dwg_entity_POLYLINE_PFACE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_PFACE_seqend_set(self,seqend);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_PFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_PFACE_seqend_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_PFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_PFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POLYLINE_PFACE_seqend_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->seqend = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_PFACE_seqend_get) {
  {
    struct _dwg_entity_POLYLINE_PFACE *arg1 = (struct _dwg_entity_POLYLINE_PFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_PFACE_seqend_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_PFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_PFACE_seqend_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_PFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_PFACE *)(argp1);
    result = (BITCODE_H) ((arg1)->seqend);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_PFACE_flag_set) {
  {
    struct _dwg_entity_POLYLINE_PFACE *arg1 = (struct _dwg_entity_POLYLINE_PFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_PFACE_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_PFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_PFACE_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_PFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_PFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POLYLINE_PFACE_flag_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_PFACE_flag_get) {
  {
    struct _dwg_entity_POLYLINE_PFACE *arg1 = (struct _dwg_entity_POLYLINE_PFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_PFACE_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_PFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_PFACE_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_PFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_PFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_PFACE_numverts_set) {
  {
    struct _dwg_entity_POLYLINE_PFACE *arg1 = (struct _dwg_entity_POLYLINE_PFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_PFACE_numverts_set(self,numverts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_PFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_PFACE_numverts_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_PFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_PFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POLYLINE_PFACE_numverts_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->numverts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_PFACE_numverts_get) {
  {
    struct _dwg_entity_POLYLINE_PFACE *arg1 = (struct _dwg_entity_POLYLINE_PFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_PFACE_numverts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_PFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_PFACE_numverts_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_PFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_PFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->numverts);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_PFACE_numfaces_set) {
  {
    struct _dwg_entity_POLYLINE_PFACE *arg1 = (struct _dwg_entity_POLYLINE_PFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_PFACE_numfaces_set(self,numfaces);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_PFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_PFACE_numfaces_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_PFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_PFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POLYLINE_PFACE_numfaces_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->numfaces = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_PFACE_numfaces_get) {
  {
    struct _dwg_entity_POLYLINE_PFACE *arg1 = (struct _dwg_entity_POLYLINE_PFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_PFACE_numfaces_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_PFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_PFACE_numfaces_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_PFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_PFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->numfaces);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_POLYLINE_PFACE) {
  {
    int argvi = 0;
    struct _dwg_entity_POLYLINE_PFACE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_POLYLINE_PFACE();");
    }
    result = (struct _dwg_entity_POLYLINE_PFACE *)calloc(1, sizeof(struct _dwg_entity_POLYLINE_PFACE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_POLYLINE_PFACE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_POLYLINE_PFACE) {
  {
    struct _dwg_entity_POLYLINE_PFACE *arg1 = (struct _dwg_entity_POLYLINE_PFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_POLYLINE_PFACE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_PFACE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_POLYLINE_PFACE" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_PFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_PFACE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_MESH_parent_set) {
  {
    struct _dwg_entity_POLYLINE_MESH *arg1 = (struct _dwg_entity_POLYLINE_MESH *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_MESH_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_MESH_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_MESH *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POLYLINE_MESH_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_MESH_parent_get) {
  {
    struct _dwg_entity_POLYLINE_MESH *arg1 = (struct _dwg_entity_POLYLINE_MESH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_MESH_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_MESH_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_MESH *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_MESH_has_vertex_set) {
  {
    struct _dwg_entity_POLYLINE_MESH *arg1 = (struct _dwg_entity_POLYLINE_MESH *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_MESH_has_vertex_set(self,has_vertex);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_MESH_has_vertex_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_MESH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POLYLINE_MESH_has_vertex_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_vertex = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_MESH_has_vertex_get) {
  {
    struct _dwg_entity_POLYLINE_MESH *arg1 = (struct _dwg_entity_POLYLINE_MESH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_MESH_has_vertex_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_MESH_has_vertex_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_MESH *)(argp1);
    result = (BITCODE_B) ((arg1)->has_vertex);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_MESH_num_owned_set) {
  {
    struct _dwg_entity_POLYLINE_MESH *arg1 = (struct _dwg_entity_POLYLINE_MESH *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_MESH_num_owned_set(self,num_owned);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_MESH_num_owned_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_MESH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POLYLINE_MESH_num_owned_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_owned = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_MESH_num_owned_get) {
  {
    struct _dwg_entity_POLYLINE_MESH *arg1 = (struct _dwg_entity_POLYLINE_MESH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_MESH_num_owned_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_MESH_num_owned_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_MESH *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_owned);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_MESH_first_vertex_set) {
  {
    struct _dwg_entity_POLYLINE_MESH *arg1 = (struct _dwg_entity_POLYLINE_MESH *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_MESH_first_vertex_set(self,first_vertex);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_MESH_first_vertex_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_MESH *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POLYLINE_MESH_first_vertex_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->first_vertex = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_MESH_first_vertex_get) {
  {
    struct _dwg_entity_POLYLINE_MESH *arg1 = (struct _dwg_entity_POLYLINE_MESH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_MESH_first_vertex_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_MESH_first_vertex_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_MESH *)(argp1);
    result = (BITCODE_H) ((arg1)->first_vertex);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_MESH_last_vertex_set) {
  {
    struct _dwg_entity_POLYLINE_MESH *arg1 = (struct _dwg_entity_POLYLINE_MESH *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_MESH_last_vertex_set(self,last_vertex);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_MESH_last_vertex_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_MESH *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POLYLINE_MESH_last_vertex_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->last_vertex = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_MESH_last_vertex_get) {
  {
    struct _dwg_entity_POLYLINE_MESH *arg1 = (struct _dwg_entity_POLYLINE_MESH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_MESH_last_vertex_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_MESH_last_vertex_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_MESH *)(argp1);
    result = (BITCODE_H) ((arg1)->last_vertex);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_MESH_vertex_set) {
  {
    struct _dwg_entity_POLYLINE_MESH *arg1 = (struct _dwg_entity_POLYLINE_MESH *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_MESH_vertex_set(self,vertex);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_MESH_vertex_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_MESH *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POLYLINE_MESH_vertex_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->vertex = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_MESH_vertex_get) {
  {
    struct _dwg_entity_POLYLINE_MESH *arg1 = (struct _dwg_entity_POLYLINE_MESH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_MESH_vertex_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_MESH_vertex_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_MESH *)(argp1);
    result = (BITCODE_H *) ((arg1)->vertex);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_MESH_seqend_set) {
  {
    struct _dwg_entity_POLYLINE_MESH *arg1 = (struct _dwg_entity_POLYLINE_MESH *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_MESH_seqend_set(self,seqend);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_MESH_seqend_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_MESH *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POLYLINE_MESH_seqend_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->seqend = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_MESH_seqend_get) {
  {
    struct _dwg_entity_POLYLINE_MESH *arg1 = (struct _dwg_entity_POLYLINE_MESH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_MESH_seqend_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_MESH_seqend_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_MESH *)(argp1);
    result = (BITCODE_H) ((arg1)->seqend);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_MESH_flag_set) {
  {
    struct _dwg_entity_POLYLINE_MESH *arg1 = (struct _dwg_entity_POLYLINE_MESH *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_MESH_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_MESH_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_MESH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POLYLINE_MESH_flag_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_MESH_flag_get) {
  {
    struct _dwg_entity_POLYLINE_MESH *arg1 = (struct _dwg_entity_POLYLINE_MESH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_MESH_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_MESH_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_MESH *)(argp1);
    result = (BITCODE_BS) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_MESH_curve_type_set) {
  {
    struct _dwg_entity_POLYLINE_MESH *arg1 = (struct _dwg_entity_POLYLINE_MESH *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_MESH_curve_type_set(self,curve_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_MESH_curve_type_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_MESH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POLYLINE_MESH_curve_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->curve_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_MESH_curve_type_get) {
  {
    struct _dwg_entity_POLYLINE_MESH *arg1 = (struct _dwg_entity_POLYLINE_MESH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_MESH_curve_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_MESH_curve_type_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_MESH *)(argp1);
    result = (BITCODE_BS) ((arg1)->curve_type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_MESH_num_m_verts_set) {
  {
    struct _dwg_entity_POLYLINE_MESH *arg1 = (struct _dwg_entity_POLYLINE_MESH *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_MESH_num_m_verts_set(self,num_m_verts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_MESH_num_m_verts_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_MESH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POLYLINE_MESH_num_m_verts_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_m_verts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_MESH_num_m_verts_get) {
  {
    struct _dwg_entity_POLYLINE_MESH *arg1 = (struct _dwg_entity_POLYLINE_MESH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_MESH_num_m_verts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_MESH_num_m_verts_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_MESH *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_m_verts);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_MESH_num_n_verts_set) {
  {
    struct _dwg_entity_POLYLINE_MESH *arg1 = (struct _dwg_entity_POLYLINE_MESH *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_MESH_num_n_verts_set(self,num_n_verts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_MESH_num_n_verts_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_MESH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POLYLINE_MESH_num_n_verts_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_n_verts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_MESH_num_n_verts_get) {
  {
    struct _dwg_entity_POLYLINE_MESH *arg1 = (struct _dwg_entity_POLYLINE_MESH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_MESH_num_n_verts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_MESH_num_n_verts_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_MESH *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_n_verts);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_MESH_m_density_set) {
  {
    struct _dwg_entity_POLYLINE_MESH *arg1 = (struct _dwg_entity_POLYLINE_MESH *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_MESH_m_density_set(self,m_density);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_MESH_m_density_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_MESH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POLYLINE_MESH_m_density_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->m_density = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_MESH_m_density_get) {
  {
    struct _dwg_entity_POLYLINE_MESH *arg1 = (struct _dwg_entity_POLYLINE_MESH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_MESH_m_density_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_MESH_m_density_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_MESH *)(argp1);
    result = (BITCODE_BS) ((arg1)->m_density);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_MESH_n_density_set) {
  {
    struct _dwg_entity_POLYLINE_MESH *arg1 = (struct _dwg_entity_POLYLINE_MESH *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_MESH_n_density_set(self,n_density);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_MESH_n_density_set" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_MESH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POLYLINE_MESH_n_density_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->n_density = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLYLINE_MESH_n_density_get) {
  {
    struct _dwg_entity_POLYLINE_MESH *arg1 = (struct _dwg_entity_POLYLINE_MESH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLYLINE_MESH_n_density_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLYLINE_MESH_n_density_get" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_MESH *)(argp1);
    result = (BITCODE_BS) ((arg1)->n_density);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_POLYLINE_MESH) {
  {
    int argvi = 0;
    struct _dwg_entity_POLYLINE_MESH *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_POLYLINE_MESH();");
    }
    result = (struct _dwg_entity_POLYLINE_MESH *)calloc(1, sizeof(struct _dwg_entity_POLYLINE_MESH));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_POLYLINE_MESH, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_POLYLINE_MESH) {
  {
    struct _dwg_entity_POLYLINE_MESH *arg1 = (struct _dwg_entity_POLYLINE_MESH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_POLYLINE_MESH(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLYLINE_MESH, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_POLYLINE_MESH" "', argument " "1"" of type '" "struct _dwg_entity_POLYLINE_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_POLYLINE_MESH *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SOLID_parent_set) {
  {
    struct _dwg_entity_SOLID *arg1 = (struct _dwg_entity_SOLID *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SOLID_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SOLID_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_SOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_SOLID *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SOLID_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SOLID_parent_get) {
  {
    struct _dwg_entity_SOLID *arg1 = (struct _dwg_entity_SOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SOLID_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SOLID_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_SOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_SOLID *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SOLID_thickness_set) {
  {
    struct _dwg_entity_SOLID *arg1 = (struct _dwg_entity_SOLID *) 0 ;
    BITCODE_BT arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SOLID_thickness_set(self,thickness);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SOLID_thickness_set" "', argument " "1"" of type '" "struct _dwg_entity_SOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_SOLID *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SOLID_thickness_set" "', argument " "2"" of type '" "BITCODE_BT""'");
    } 
    arg2 = (BITCODE_BT)(val2);
    if (arg1) (arg1)->thickness = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SOLID_thickness_get) {
  {
    struct _dwg_entity_SOLID *arg1 = (struct _dwg_entity_SOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BT result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SOLID_thickness_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SOLID_thickness_get" "', argument " "1"" of type '" "struct _dwg_entity_SOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_SOLID *)(argp1);
    result = (BITCODE_BT) ((arg1)->thickness);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SOLID_elevation_set) {
  {
    struct _dwg_entity_SOLID *arg1 = (struct _dwg_entity_SOLID *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SOLID_elevation_set(self,elevation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SOLID_elevation_set" "', argument " "1"" of type '" "struct _dwg_entity_SOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_SOLID *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SOLID_elevation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->elevation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SOLID_elevation_get) {
  {
    struct _dwg_entity_SOLID *arg1 = (struct _dwg_entity_SOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SOLID_elevation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SOLID_elevation_get" "', argument " "1"" of type '" "struct _dwg_entity_SOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_SOLID *)(argp1);
    result = (BITCODE_BD) ((arg1)->elevation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SOLID_corner1_set) {
  {
    struct _dwg_entity_SOLID *arg1 = (struct _dwg_entity_SOLID *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SOLID_corner1_set(self,corner1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SOLID_corner1_set" "', argument " "1"" of type '" "struct _dwg_entity_SOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_SOLID *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SOLID_corner1_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->corner1 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SOLID_corner1_get) {
  {
    struct _dwg_entity_SOLID *arg1 = (struct _dwg_entity_SOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SOLID_corner1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SOLID_corner1_get" "', argument " "1"" of type '" "struct _dwg_entity_SOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_SOLID *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->corner1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SOLID_corner2_set) {
  {
    struct _dwg_entity_SOLID *arg1 = (struct _dwg_entity_SOLID *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SOLID_corner2_set(self,corner2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SOLID_corner2_set" "', argument " "1"" of type '" "struct _dwg_entity_SOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_SOLID *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SOLID_corner2_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->corner2 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SOLID_corner2_get) {
  {
    struct _dwg_entity_SOLID *arg1 = (struct _dwg_entity_SOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SOLID_corner2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SOLID_corner2_get" "', argument " "1"" of type '" "struct _dwg_entity_SOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_SOLID *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->corner2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SOLID_corner3_set) {
  {
    struct _dwg_entity_SOLID *arg1 = (struct _dwg_entity_SOLID *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SOLID_corner3_set(self,corner3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SOLID_corner3_set" "', argument " "1"" of type '" "struct _dwg_entity_SOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_SOLID *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SOLID_corner3_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->corner3 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SOLID_corner3_get) {
  {
    struct _dwg_entity_SOLID *arg1 = (struct _dwg_entity_SOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SOLID_corner3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SOLID_corner3_get" "', argument " "1"" of type '" "struct _dwg_entity_SOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_SOLID *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->corner3);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SOLID_corner4_set) {
  {
    struct _dwg_entity_SOLID *arg1 = (struct _dwg_entity_SOLID *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SOLID_corner4_set(self,corner4);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SOLID_corner4_set" "', argument " "1"" of type '" "struct _dwg_entity_SOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_SOLID *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SOLID_corner4_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->corner4 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SOLID_corner4_get) {
  {
    struct _dwg_entity_SOLID *arg1 = (struct _dwg_entity_SOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SOLID_corner4_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SOLID_corner4_get" "', argument " "1"" of type '" "struct _dwg_entity_SOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_SOLID *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->corner4);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SOLID_extrusion_set) {
  {
    struct _dwg_entity_SOLID *arg1 = (struct _dwg_entity_SOLID *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SOLID_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SOLID_extrusion_set" "', argument " "1"" of type '" "struct _dwg_entity_SOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_SOLID *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SOLID_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SOLID_extrusion_get) {
  {
    struct _dwg_entity_SOLID *arg1 = (struct _dwg_entity_SOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SOLID_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SOLID_extrusion_get" "', argument " "1"" of type '" "struct _dwg_entity_SOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_SOLID *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_SOLID) {
  {
    int argvi = 0;
    struct _dwg_entity_SOLID *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_SOLID();");
    }
    result = (struct _dwg_entity_SOLID *)calloc(1, sizeof(struct _dwg_entity_SOLID));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_SOLID, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_SOLID) {
  {
    struct _dwg_entity_SOLID *arg1 = (struct _dwg_entity_SOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_SOLID(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SOLID, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_SOLID" "', argument " "1"" of type '" "struct _dwg_entity_SOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_SOLID *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TRACE_parent_set) {
  {
    struct _dwg_entity_TRACE *arg1 = (struct _dwg_entity_TRACE *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TRACE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TRACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TRACE_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_TRACE *""'"); 
    }
    arg1 = (struct _dwg_entity_TRACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TRACE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TRACE_parent_get) {
  {
    struct _dwg_entity_TRACE *arg1 = (struct _dwg_entity_TRACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TRACE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TRACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TRACE_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_TRACE *""'"); 
    }
    arg1 = (struct _dwg_entity_TRACE *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TRACE_thickness_set) {
  {
    struct _dwg_entity_TRACE *arg1 = (struct _dwg_entity_TRACE *) 0 ;
    BITCODE_BT arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TRACE_thickness_set(self,thickness);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TRACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TRACE_thickness_set" "', argument " "1"" of type '" "struct _dwg_entity_TRACE *""'"); 
    }
    arg1 = (struct _dwg_entity_TRACE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TRACE_thickness_set" "', argument " "2"" of type '" "BITCODE_BT""'");
    } 
    arg2 = (BITCODE_BT)(val2);
    if (arg1) (arg1)->thickness = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TRACE_thickness_get) {
  {
    struct _dwg_entity_TRACE *arg1 = (struct _dwg_entity_TRACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BT result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TRACE_thickness_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TRACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TRACE_thickness_get" "', argument " "1"" of type '" "struct _dwg_entity_TRACE *""'"); 
    }
    arg1 = (struct _dwg_entity_TRACE *)(argp1);
    result = (BITCODE_BT) ((arg1)->thickness);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TRACE_elevation_set) {
  {
    struct _dwg_entity_TRACE *arg1 = (struct _dwg_entity_TRACE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TRACE_elevation_set(self,elevation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TRACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TRACE_elevation_set" "', argument " "1"" of type '" "struct _dwg_entity_TRACE *""'"); 
    }
    arg1 = (struct _dwg_entity_TRACE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TRACE_elevation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->elevation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TRACE_elevation_get) {
  {
    struct _dwg_entity_TRACE *arg1 = (struct _dwg_entity_TRACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TRACE_elevation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TRACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TRACE_elevation_get" "', argument " "1"" of type '" "struct _dwg_entity_TRACE *""'"); 
    }
    arg1 = (struct _dwg_entity_TRACE *)(argp1);
    result = (BITCODE_BD) ((arg1)->elevation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TRACE_corner1_set) {
  {
    struct _dwg_entity_TRACE *arg1 = (struct _dwg_entity_TRACE *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TRACE_corner1_set(self,corner1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TRACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TRACE_corner1_set" "', argument " "1"" of type '" "struct _dwg_entity_TRACE *""'"); 
    }
    arg1 = (struct _dwg_entity_TRACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TRACE_corner1_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->corner1 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TRACE_corner1_get) {
  {
    struct _dwg_entity_TRACE *arg1 = (struct _dwg_entity_TRACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TRACE_corner1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TRACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TRACE_corner1_get" "', argument " "1"" of type '" "struct _dwg_entity_TRACE *""'"); 
    }
    arg1 = (struct _dwg_entity_TRACE *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->corner1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TRACE_corner2_set) {
  {
    struct _dwg_entity_TRACE *arg1 = (struct _dwg_entity_TRACE *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TRACE_corner2_set(self,corner2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TRACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TRACE_corner2_set" "', argument " "1"" of type '" "struct _dwg_entity_TRACE *""'"); 
    }
    arg1 = (struct _dwg_entity_TRACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TRACE_corner2_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->corner2 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TRACE_corner2_get) {
  {
    struct _dwg_entity_TRACE *arg1 = (struct _dwg_entity_TRACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TRACE_corner2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TRACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TRACE_corner2_get" "', argument " "1"" of type '" "struct _dwg_entity_TRACE *""'"); 
    }
    arg1 = (struct _dwg_entity_TRACE *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->corner2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TRACE_corner3_set) {
  {
    struct _dwg_entity_TRACE *arg1 = (struct _dwg_entity_TRACE *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TRACE_corner3_set(self,corner3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TRACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TRACE_corner3_set" "', argument " "1"" of type '" "struct _dwg_entity_TRACE *""'"); 
    }
    arg1 = (struct _dwg_entity_TRACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TRACE_corner3_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->corner3 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TRACE_corner3_get) {
  {
    struct _dwg_entity_TRACE *arg1 = (struct _dwg_entity_TRACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TRACE_corner3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TRACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TRACE_corner3_get" "', argument " "1"" of type '" "struct _dwg_entity_TRACE *""'"); 
    }
    arg1 = (struct _dwg_entity_TRACE *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->corner3);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TRACE_corner4_set) {
  {
    struct _dwg_entity_TRACE *arg1 = (struct _dwg_entity_TRACE *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TRACE_corner4_set(self,corner4);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TRACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TRACE_corner4_set" "', argument " "1"" of type '" "struct _dwg_entity_TRACE *""'"); 
    }
    arg1 = (struct _dwg_entity_TRACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TRACE_corner4_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->corner4 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TRACE_corner4_get) {
  {
    struct _dwg_entity_TRACE *arg1 = (struct _dwg_entity_TRACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TRACE_corner4_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TRACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TRACE_corner4_get" "', argument " "1"" of type '" "struct _dwg_entity_TRACE *""'"); 
    }
    arg1 = (struct _dwg_entity_TRACE *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->corner4);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TRACE_extrusion_set) {
  {
    struct _dwg_entity_TRACE *arg1 = (struct _dwg_entity_TRACE *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TRACE_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TRACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TRACE_extrusion_set" "', argument " "1"" of type '" "struct _dwg_entity_TRACE *""'"); 
    }
    arg1 = (struct _dwg_entity_TRACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TRACE_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TRACE_extrusion_get) {
  {
    struct _dwg_entity_TRACE *arg1 = (struct _dwg_entity_TRACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TRACE_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TRACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TRACE_extrusion_get" "', argument " "1"" of type '" "struct _dwg_entity_TRACE *""'"); 
    }
    arg1 = (struct _dwg_entity_TRACE *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_TRACE) {
  {
    int argvi = 0;
    struct _dwg_entity_TRACE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_TRACE();");
    }
    result = (struct _dwg_entity_TRACE *)calloc(1, sizeof(struct _dwg_entity_TRACE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_TRACE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_TRACE) {
  {
    struct _dwg_entity_TRACE *arg1 = (struct _dwg_entity_TRACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_TRACE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TRACE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_TRACE" "', argument " "1"" of type '" "struct _dwg_entity_TRACE *""'"); 
    }
    arg1 = (struct _dwg_entity_TRACE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SHAPE_parent_set) {
  {
    struct _dwg_entity_SHAPE *arg1 = (struct _dwg_entity_SHAPE *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SHAPE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SHAPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SHAPE_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_SHAPE *""'"); 
    }
    arg1 = (struct _dwg_entity_SHAPE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SHAPE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SHAPE_parent_get) {
  {
    struct _dwg_entity_SHAPE *arg1 = (struct _dwg_entity_SHAPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SHAPE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SHAPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SHAPE_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_SHAPE *""'"); 
    }
    arg1 = (struct _dwg_entity_SHAPE *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SHAPE_ins_pt_set) {
  {
    struct _dwg_entity_SHAPE *arg1 = (struct _dwg_entity_SHAPE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SHAPE_ins_pt_set(self,ins_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SHAPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SHAPE_ins_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_SHAPE *""'"); 
    }
    arg1 = (struct _dwg_entity_SHAPE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SHAPE_ins_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ins_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SHAPE_ins_pt_get) {
  {
    struct _dwg_entity_SHAPE *arg1 = (struct _dwg_entity_SHAPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SHAPE_ins_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SHAPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SHAPE_ins_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_SHAPE *""'"); 
    }
    arg1 = (struct _dwg_entity_SHAPE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ins_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SHAPE_scale_set) {
  {
    struct _dwg_entity_SHAPE *arg1 = (struct _dwg_entity_SHAPE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SHAPE_scale_set(self,scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SHAPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SHAPE_scale_set" "', argument " "1"" of type '" "struct _dwg_entity_SHAPE *""'"); 
    }
    arg1 = (struct _dwg_entity_SHAPE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SHAPE_scale_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->scale = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SHAPE_scale_get) {
  {
    struct _dwg_entity_SHAPE *arg1 = (struct _dwg_entity_SHAPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SHAPE_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SHAPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SHAPE_scale_get" "', argument " "1"" of type '" "struct _dwg_entity_SHAPE *""'"); 
    }
    arg1 = (struct _dwg_entity_SHAPE *)(argp1);
    result = (BITCODE_BD) ((arg1)->scale);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SHAPE_rotation_set) {
  {
    struct _dwg_entity_SHAPE *arg1 = (struct _dwg_entity_SHAPE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SHAPE_rotation_set(self,rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SHAPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SHAPE_rotation_set" "', argument " "1"" of type '" "struct _dwg_entity_SHAPE *""'"); 
    }
    arg1 = (struct _dwg_entity_SHAPE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SHAPE_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SHAPE_rotation_get) {
  {
    struct _dwg_entity_SHAPE *arg1 = (struct _dwg_entity_SHAPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SHAPE_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SHAPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SHAPE_rotation_get" "', argument " "1"" of type '" "struct _dwg_entity_SHAPE *""'"); 
    }
    arg1 = (struct _dwg_entity_SHAPE *)(argp1);
    result = (BITCODE_BD) ((arg1)->rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SHAPE_width_factor_set) {
  {
    struct _dwg_entity_SHAPE *arg1 = (struct _dwg_entity_SHAPE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SHAPE_width_factor_set(self,width_factor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SHAPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SHAPE_width_factor_set" "', argument " "1"" of type '" "struct _dwg_entity_SHAPE *""'"); 
    }
    arg1 = (struct _dwg_entity_SHAPE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SHAPE_width_factor_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->width_factor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SHAPE_width_factor_get) {
  {
    struct _dwg_entity_SHAPE *arg1 = (struct _dwg_entity_SHAPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SHAPE_width_factor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SHAPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SHAPE_width_factor_get" "', argument " "1"" of type '" "struct _dwg_entity_SHAPE *""'"); 
    }
    arg1 = (struct _dwg_entity_SHAPE *)(argp1);
    result = (BITCODE_BD) ((arg1)->width_factor);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SHAPE_oblique_angle_set) {
  {
    struct _dwg_entity_SHAPE *arg1 = (struct _dwg_entity_SHAPE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SHAPE_oblique_angle_set(self,oblique_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SHAPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SHAPE_oblique_angle_set" "', argument " "1"" of type '" "struct _dwg_entity_SHAPE *""'"); 
    }
    arg1 = (struct _dwg_entity_SHAPE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SHAPE_oblique_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->oblique_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SHAPE_oblique_angle_get) {
  {
    struct _dwg_entity_SHAPE *arg1 = (struct _dwg_entity_SHAPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SHAPE_oblique_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SHAPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SHAPE_oblique_angle_get" "', argument " "1"" of type '" "struct _dwg_entity_SHAPE *""'"); 
    }
    arg1 = (struct _dwg_entity_SHAPE *)(argp1);
    result = (BITCODE_BD) ((arg1)->oblique_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SHAPE_thickness_set) {
  {
    struct _dwg_entity_SHAPE *arg1 = (struct _dwg_entity_SHAPE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SHAPE_thickness_set(self,thickness);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SHAPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SHAPE_thickness_set" "', argument " "1"" of type '" "struct _dwg_entity_SHAPE *""'"); 
    }
    arg1 = (struct _dwg_entity_SHAPE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SHAPE_thickness_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->thickness = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SHAPE_thickness_get) {
  {
    struct _dwg_entity_SHAPE *arg1 = (struct _dwg_entity_SHAPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SHAPE_thickness_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SHAPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SHAPE_thickness_get" "', argument " "1"" of type '" "struct _dwg_entity_SHAPE *""'"); 
    }
    arg1 = (struct _dwg_entity_SHAPE *)(argp1);
    result = (BITCODE_BD) ((arg1)->thickness);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SHAPE_style_id_set) {
  {
    struct _dwg_entity_SHAPE *arg1 = (struct _dwg_entity_SHAPE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SHAPE_style_id_set(self,style_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SHAPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SHAPE_style_id_set" "', argument " "1"" of type '" "struct _dwg_entity_SHAPE *""'"); 
    }
    arg1 = (struct _dwg_entity_SHAPE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SHAPE_style_id_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->style_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SHAPE_style_id_get) {
  {
    struct _dwg_entity_SHAPE *arg1 = (struct _dwg_entity_SHAPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SHAPE_style_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SHAPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SHAPE_style_id_get" "', argument " "1"" of type '" "struct _dwg_entity_SHAPE *""'"); 
    }
    arg1 = (struct _dwg_entity_SHAPE *)(argp1);
    result = (BITCODE_BS) ((arg1)->style_id);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SHAPE_extrusion_set) {
  {
    struct _dwg_entity_SHAPE *arg1 = (struct _dwg_entity_SHAPE *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SHAPE_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SHAPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SHAPE_extrusion_set" "', argument " "1"" of type '" "struct _dwg_entity_SHAPE *""'"); 
    }
    arg1 = (struct _dwg_entity_SHAPE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SHAPE_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SHAPE_extrusion_get) {
  {
    struct _dwg_entity_SHAPE *arg1 = (struct _dwg_entity_SHAPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SHAPE_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SHAPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SHAPE_extrusion_get" "', argument " "1"" of type '" "struct _dwg_entity_SHAPE *""'"); 
    }
    arg1 = (struct _dwg_entity_SHAPE *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SHAPE_style_set) {
  {
    struct _dwg_entity_SHAPE *arg1 = (struct _dwg_entity_SHAPE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SHAPE_style_set(self,style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SHAPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SHAPE_style_set" "', argument " "1"" of type '" "struct _dwg_entity_SHAPE *""'"); 
    }
    arg1 = (struct _dwg_entity_SHAPE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SHAPE_style_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SHAPE_style_get) {
  {
    struct _dwg_entity_SHAPE *arg1 = (struct _dwg_entity_SHAPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SHAPE_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SHAPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SHAPE_style_get" "', argument " "1"" of type '" "struct _dwg_entity_SHAPE *""'"); 
    }
    arg1 = (struct _dwg_entity_SHAPE *)(argp1);
    result = (BITCODE_H) ((arg1)->style);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_SHAPE) {
  {
    int argvi = 0;
    struct _dwg_entity_SHAPE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_SHAPE();");
    }
    result = (struct _dwg_entity_SHAPE *)calloc(1, sizeof(struct _dwg_entity_SHAPE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_SHAPE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_SHAPE) {
  {
    struct _dwg_entity_SHAPE *arg1 = (struct _dwg_entity_SHAPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_SHAPE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SHAPE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_SHAPE" "', argument " "1"" of type '" "struct _dwg_entity_SHAPE *""'"); 
    }
    arg1 = (struct _dwg_entity_SHAPE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_parent_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_VIEWPORT_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_parent_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_center_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_center_set(self,center);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_center_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_VIEWPORT_center_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->center = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_center_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_center_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_center_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->center);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_width_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_width_set(self,width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_width_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_VIEWPORT_width_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_width_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_width_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_BD) ((arg1)->width);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_height_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_height_set(self,height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_height_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_VIEWPORT_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_height_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_height_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_BD) ((arg1)->height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_on_off_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_on_off_set(self,on_off);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_on_off_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_VIEWPORT_on_off_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->on_off = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_on_off_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_on_off_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_on_off_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_RS) ((arg1)->on_off);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_id_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_id_set(self,id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_id_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_VIEWPORT_id_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_id_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_id_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_RS) ((arg1)->id);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_view_target_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_view_target_set(self,view_target);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_view_target_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_VIEWPORT_view_target_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->view_target = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_view_target_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_view_target_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_view_target_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->view_target);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_VIEWDIR_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_VIEWDIR_set(self,VIEWDIR);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_VIEWDIR_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_VIEWPORT_VIEWDIR_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->VIEWDIR = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_VIEWDIR_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_VIEWDIR_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_VIEWDIR_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->VIEWDIR);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_twist_angle_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_twist_angle_set(self,twist_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_twist_angle_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_VIEWPORT_twist_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->twist_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_twist_angle_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_twist_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_twist_angle_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_BD) ((arg1)->twist_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_VIEWSIZE_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_VIEWSIZE_set(self,VIEWSIZE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_VIEWSIZE_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_VIEWPORT_VIEWSIZE_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->VIEWSIZE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_VIEWSIZE_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_VIEWSIZE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_VIEWSIZE_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_BD) ((arg1)->VIEWSIZE);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_lens_length_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_lens_length_set(self,lens_length);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_lens_length_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_VIEWPORT_lens_length_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->lens_length = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_lens_length_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_lens_length_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_lens_length_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_BD) ((arg1)->lens_length);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_front_clip_z_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_front_clip_z_set(self,front_clip_z);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_front_clip_z_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_VIEWPORT_front_clip_z_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->front_clip_z = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_front_clip_z_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_front_clip_z_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_front_clip_z_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_BD) ((arg1)->front_clip_z);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_back_clip_z_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_back_clip_z_set(self,back_clip_z);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_back_clip_z_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_VIEWPORT_back_clip_z_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->back_clip_z = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_back_clip_z_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_back_clip_z_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_back_clip_z_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_BD) ((arg1)->back_clip_z);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_SNAPANG_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_SNAPANG_set(self,SNAPANG);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_SNAPANG_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_VIEWPORT_SNAPANG_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->SNAPANG = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_SNAPANG_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_SNAPANG_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_SNAPANG_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_BD) ((arg1)->SNAPANG);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_VIEWCTR_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_VIEWCTR_set(self,VIEWCTR);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_VIEWCTR_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_VIEWPORT_VIEWCTR_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->VIEWCTR = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_VIEWCTR_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_VIEWCTR_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_VIEWCTR_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->VIEWCTR);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_SNAPBASE_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_SNAPBASE_set(self,SNAPBASE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_SNAPBASE_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_VIEWPORT_SNAPBASE_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->SNAPBASE = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_SNAPBASE_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_SNAPBASE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_SNAPBASE_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->SNAPBASE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_SNAPUNIT_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_SNAPUNIT_set(self,SNAPUNIT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_SNAPUNIT_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_VIEWPORT_SNAPUNIT_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->SNAPUNIT = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_SNAPUNIT_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_SNAPUNIT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_SNAPUNIT_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->SNAPUNIT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_GRIDUNIT_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_GRIDUNIT_set(self,GRIDUNIT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_GRIDUNIT_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_VIEWPORT_GRIDUNIT_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->GRIDUNIT = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_GRIDUNIT_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_GRIDUNIT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_GRIDUNIT_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->GRIDUNIT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_circle_zoom_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_circle_zoom_set(self,circle_zoom);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_circle_zoom_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_VIEWPORT_circle_zoom_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->circle_zoom = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_circle_zoom_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_circle_zoom_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_circle_zoom_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_BS) ((arg1)->circle_zoom);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_grid_major_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_grid_major_set(self,grid_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_grid_major_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_VIEWPORT_grid_major_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->grid_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_grid_major_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_grid_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_grid_major_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_BS) ((arg1)->grid_major);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_num_frozen_layers_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_num_frozen_layers_set(self,num_frozen_layers);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_num_frozen_layers_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_VIEWPORT_num_frozen_layers_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_frozen_layers = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_num_frozen_layers_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_num_frozen_layers_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_num_frozen_layers_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_frozen_layers);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_status_flag_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_status_flag_set(self,status_flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_status_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_VIEWPORT_status_flag_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->status_flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_status_flag_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_status_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_status_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_BL) ((arg1)->status_flag);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_style_sheet_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_style_sheet_set(self,style_sheet);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_style_sheet_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_VIEWPORT_style_sheet_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->style_sheet);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->style_sheet = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->style_sheet = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_style_sheet_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_style_sheet_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_style_sheet_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_TV) ((arg1)->style_sheet);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_render_mode_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_render_mode_set(self,render_mode);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_render_mode_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_VIEWPORT_render_mode_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->render_mode = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_render_mode_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_render_mode_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_render_mode_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_RC) ((arg1)->render_mode);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_ucs_at_origin_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_ucs_at_origin_set(self,ucs_at_origin);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_ucs_at_origin_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_VIEWPORT_ucs_at_origin_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->ucs_at_origin = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_ucs_at_origin_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_ucs_at_origin_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_ucs_at_origin_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_B) ((arg1)->ucs_at_origin);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_UCSVP_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_UCSVP_set(self,UCSVP);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_UCSVP_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_VIEWPORT_UCSVP_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->UCSVP = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_UCSVP_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_UCSVP_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_UCSVP_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_B) ((arg1)->UCSVP);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_ucsorg_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_ucsorg_set(self,ucsorg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_ucsorg_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_VIEWPORT_ucsorg_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ucsorg = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_ucsorg_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_ucsorg_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_ucsorg_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ucsorg);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_ucsxdir_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_ucsxdir_set(self,ucsxdir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_ucsxdir_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_VIEWPORT_ucsxdir_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ucsxdir = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_ucsxdir_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_ucsxdir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_ucsxdir_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ucsxdir);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_ucsydir_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_ucsydir_set(self,ucsydir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_ucsydir_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_VIEWPORT_ucsydir_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ucsydir = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_ucsydir_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_ucsydir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_ucsydir_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ucsydir);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_ucs_elevation_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_ucs_elevation_set(self,ucs_elevation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_ucs_elevation_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_VIEWPORT_ucs_elevation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->ucs_elevation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_ucs_elevation_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_ucs_elevation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_ucs_elevation_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_BD) ((arg1)->ucs_elevation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_UCSORTHOVIEW_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_UCSORTHOVIEW_set(self,UCSORTHOVIEW);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_UCSORTHOVIEW_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_VIEWPORT_UCSORTHOVIEW_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->UCSORTHOVIEW = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_UCSORTHOVIEW_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_UCSORTHOVIEW_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_UCSORTHOVIEW_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_BS) ((arg1)->UCSORTHOVIEW);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_shadeplot_mode_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_shadeplot_mode_set(self,shadeplot_mode);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_shadeplot_mode_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_VIEWPORT_shadeplot_mode_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->shadeplot_mode = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_shadeplot_mode_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_shadeplot_mode_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_shadeplot_mode_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_BS) ((arg1)->shadeplot_mode);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_use_default_lights_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_use_default_lights_set(self,use_default_lights);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_use_default_lights_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_VIEWPORT_use_default_lights_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->use_default_lights = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_use_default_lights_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_use_default_lights_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_use_default_lights_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_B) ((arg1)->use_default_lights);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_default_lighting_type_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_default_lighting_type_set(self,default_lighting_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_default_lighting_type_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_VIEWPORT_default_lighting_type_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->default_lighting_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_default_lighting_type_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_default_lighting_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_default_lighting_type_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_RC) ((arg1)->default_lighting_type);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_brightness_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_brightness_set(self,brightness);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_brightness_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_VIEWPORT_brightness_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->brightness = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_brightness_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_brightness_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_brightness_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_BD) ((arg1)->brightness);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_contrast_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_contrast_set(self,contrast);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_contrast_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_VIEWPORT_contrast_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->contrast = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_contrast_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_contrast_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_contrast_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_BD) ((arg1)->contrast);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_ambient_color_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_ambient_color_set(self,ambient_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_ambient_color_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_VIEWPORT_ambient_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->ambient_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_ambient_color_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_ambient_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_ambient_color_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->ambient_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_vport_entity_header_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_vport_entity_header_set(self,vport_entity_header);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_vport_entity_header_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_VIEWPORT_vport_entity_header_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->vport_entity_header = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_vport_entity_header_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_vport_entity_header_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_vport_entity_header_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_H) ((arg1)->vport_entity_header);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_frozen_layers_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_frozen_layers_set(self,frozen_layers);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_frozen_layers_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_VIEWPORT_frozen_layers_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->frozen_layers = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_frozen_layers_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_frozen_layers_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_frozen_layers_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_H *) ((arg1)->frozen_layers);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_clip_boundary_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_clip_boundary_set(self,clip_boundary);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_clip_boundary_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_VIEWPORT_clip_boundary_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->clip_boundary = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_clip_boundary_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_clip_boundary_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_clip_boundary_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_H) ((arg1)->clip_boundary);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_named_ucs_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_named_ucs_set(self,named_ucs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_named_ucs_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_VIEWPORT_named_ucs_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->named_ucs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_named_ucs_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_named_ucs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_named_ucs_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_H) ((arg1)->named_ucs);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_base_ucs_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_base_ucs_set(self,base_ucs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_base_ucs_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_VIEWPORT_base_ucs_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->base_ucs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_base_ucs_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_base_ucs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_base_ucs_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_H) ((arg1)->base_ucs);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_background_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_background_set(self,background);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_background_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_VIEWPORT_background_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->background = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_background_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_background_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_background_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_H) ((arg1)->background);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_visualstyle_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_visualstyle_set(self,visualstyle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_visualstyle_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_VIEWPORT_visualstyle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->visualstyle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_visualstyle_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_visualstyle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_visualstyle_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_H) ((arg1)->visualstyle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_shadeplot_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_shadeplot_set(self,shadeplot);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_shadeplot_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_VIEWPORT_shadeplot_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->shadeplot = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_shadeplot_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_shadeplot_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_shadeplot_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_H) ((arg1)->shadeplot);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_sun_set) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_sun_set(self,sun);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_sun_set" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_VIEWPORT_sun_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->sun = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VIEWPORT_sun_get) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VIEWPORT_sun_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VIEWPORT_sun_get" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    result = (BITCODE_H) ((arg1)->sun);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_VIEWPORT) {
  {
    int argvi = 0;
    struct _dwg_entity_VIEWPORT *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_VIEWPORT();");
    }
    result = (struct _dwg_entity_VIEWPORT *)calloc(1, sizeof(struct _dwg_entity_VIEWPORT));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_VIEWPORT, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_VIEWPORT) {
  {
    struct _dwg_entity_VIEWPORT *arg1 = (struct _dwg_entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_VIEWPORT(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VIEWPORT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_VIEWPORT" "', argument " "1"" of type '" "struct _dwg_entity_VIEWPORT *""'"); 
    }
    arg1 = (struct _dwg_entity_VIEWPORT *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ELLIPSE_parent_set) {
  {
    struct _dwg_entity_ELLIPSE *arg1 = (struct _dwg_entity_ELLIPSE *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ELLIPSE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ELLIPSE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ELLIPSE_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_ELLIPSE *""'"); 
    }
    arg1 = (struct _dwg_entity_ELLIPSE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ELLIPSE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ELLIPSE_parent_get) {
  {
    struct _dwg_entity_ELLIPSE *arg1 = (struct _dwg_entity_ELLIPSE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ELLIPSE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ELLIPSE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ELLIPSE_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_ELLIPSE *""'"); 
    }
    arg1 = (struct _dwg_entity_ELLIPSE *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ELLIPSE_center_set) {
  {
    struct _dwg_entity_ELLIPSE *arg1 = (struct _dwg_entity_ELLIPSE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ELLIPSE_center_set(self,center);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ELLIPSE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ELLIPSE_center_set" "', argument " "1"" of type '" "struct _dwg_entity_ELLIPSE *""'"); 
    }
    arg1 = (struct _dwg_entity_ELLIPSE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ELLIPSE_center_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->center = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ELLIPSE_center_get) {
  {
    struct _dwg_entity_ELLIPSE *arg1 = (struct _dwg_entity_ELLIPSE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ELLIPSE_center_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ELLIPSE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ELLIPSE_center_get" "', argument " "1"" of type '" "struct _dwg_entity_ELLIPSE *""'"); 
    }
    arg1 = (struct _dwg_entity_ELLIPSE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->center);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ELLIPSE_sm_axis_set) {
  {
    struct _dwg_entity_ELLIPSE *arg1 = (struct _dwg_entity_ELLIPSE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ELLIPSE_sm_axis_set(self,sm_axis);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ELLIPSE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ELLIPSE_sm_axis_set" "', argument " "1"" of type '" "struct _dwg_entity_ELLIPSE *""'"); 
    }
    arg1 = (struct _dwg_entity_ELLIPSE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ELLIPSE_sm_axis_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->sm_axis = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ELLIPSE_sm_axis_get) {
  {
    struct _dwg_entity_ELLIPSE *arg1 = (struct _dwg_entity_ELLIPSE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ELLIPSE_sm_axis_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ELLIPSE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ELLIPSE_sm_axis_get" "', argument " "1"" of type '" "struct _dwg_entity_ELLIPSE *""'"); 
    }
    arg1 = (struct _dwg_entity_ELLIPSE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->sm_axis);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ELLIPSE_extrusion_set) {
  {
    struct _dwg_entity_ELLIPSE *arg1 = (struct _dwg_entity_ELLIPSE *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ELLIPSE_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ELLIPSE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ELLIPSE_extrusion_set" "', argument " "1"" of type '" "struct _dwg_entity_ELLIPSE *""'"); 
    }
    arg1 = (struct _dwg_entity_ELLIPSE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ELLIPSE_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ELLIPSE_extrusion_get) {
  {
    struct _dwg_entity_ELLIPSE *arg1 = (struct _dwg_entity_ELLIPSE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ELLIPSE_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ELLIPSE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ELLIPSE_extrusion_get" "', argument " "1"" of type '" "struct _dwg_entity_ELLIPSE *""'"); 
    }
    arg1 = (struct _dwg_entity_ELLIPSE *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ELLIPSE_axis_ratio_set) {
  {
    struct _dwg_entity_ELLIPSE *arg1 = (struct _dwg_entity_ELLIPSE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ELLIPSE_axis_ratio_set(self,axis_ratio);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ELLIPSE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ELLIPSE_axis_ratio_set" "', argument " "1"" of type '" "struct _dwg_entity_ELLIPSE *""'"); 
    }
    arg1 = (struct _dwg_entity_ELLIPSE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ELLIPSE_axis_ratio_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->axis_ratio = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ELLIPSE_axis_ratio_get) {
  {
    struct _dwg_entity_ELLIPSE *arg1 = (struct _dwg_entity_ELLIPSE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ELLIPSE_axis_ratio_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ELLIPSE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ELLIPSE_axis_ratio_get" "', argument " "1"" of type '" "struct _dwg_entity_ELLIPSE *""'"); 
    }
    arg1 = (struct _dwg_entity_ELLIPSE *)(argp1);
    result = (BITCODE_BD) ((arg1)->axis_ratio);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ELLIPSE_start_angle_set) {
  {
    struct _dwg_entity_ELLIPSE *arg1 = (struct _dwg_entity_ELLIPSE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ELLIPSE_start_angle_set(self,start_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ELLIPSE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ELLIPSE_start_angle_set" "', argument " "1"" of type '" "struct _dwg_entity_ELLIPSE *""'"); 
    }
    arg1 = (struct _dwg_entity_ELLIPSE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ELLIPSE_start_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->start_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ELLIPSE_start_angle_get) {
  {
    struct _dwg_entity_ELLIPSE *arg1 = (struct _dwg_entity_ELLIPSE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ELLIPSE_start_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ELLIPSE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ELLIPSE_start_angle_get" "', argument " "1"" of type '" "struct _dwg_entity_ELLIPSE *""'"); 
    }
    arg1 = (struct _dwg_entity_ELLIPSE *)(argp1);
    result = (BITCODE_BD) ((arg1)->start_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ELLIPSE_end_angle_set) {
  {
    struct _dwg_entity_ELLIPSE *arg1 = (struct _dwg_entity_ELLIPSE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ELLIPSE_end_angle_set(self,end_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ELLIPSE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ELLIPSE_end_angle_set" "', argument " "1"" of type '" "struct _dwg_entity_ELLIPSE *""'"); 
    }
    arg1 = (struct _dwg_entity_ELLIPSE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ELLIPSE_end_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->end_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ELLIPSE_end_angle_get) {
  {
    struct _dwg_entity_ELLIPSE *arg1 = (struct _dwg_entity_ELLIPSE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ELLIPSE_end_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ELLIPSE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ELLIPSE_end_angle_get" "', argument " "1"" of type '" "struct _dwg_entity_ELLIPSE *""'"); 
    }
    arg1 = (struct _dwg_entity_ELLIPSE *)(argp1);
    result = (BITCODE_BD) ((arg1)->end_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_ELLIPSE) {
  {
    int argvi = 0;
    struct _dwg_entity_ELLIPSE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_ELLIPSE();");
    }
    result = (struct _dwg_entity_ELLIPSE *)calloc(1, sizeof(struct _dwg_entity_ELLIPSE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_ELLIPSE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_ELLIPSE) {
  {
    struct _dwg_entity_ELLIPSE *arg1 = (struct _dwg_entity_ELLIPSE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_ELLIPSE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ELLIPSE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_ELLIPSE" "', argument " "1"" of type '" "struct _dwg_entity_ELLIPSE *""'"); 
    }
    arg1 = (struct _dwg_entity_ELLIPSE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SPLINE_control_point_parent_set) {
  {
    struct _dwg_SPLINE_control_point *arg1 = (struct _dwg_SPLINE_control_point *) 0 ;
    struct _dwg_entity_SPLINE *arg2 = (struct _dwg_entity_SPLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SPLINE_control_point_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SPLINE_control_point, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SPLINE_control_point_parent_set" "', argument " "1"" of type '" "struct _dwg_SPLINE_control_point *""'"); 
    }
    arg1 = (struct _dwg_SPLINE_control_point *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_SPLINE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_SPLINE_control_point_parent_set" "', argument " "2"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg2 = (struct _dwg_entity_SPLINE *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SPLINE_control_point_parent_get) {
  {
    struct _dwg_SPLINE_control_point *arg1 = (struct _dwg_SPLINE_control_point *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_entity_SPLINE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SPLINE_control_point_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SPLINE_control_point, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SPLINE_control_point_parent_get" "', argument " "1"" of type '" "struct _dwg_SPLINE_control_point *""'"); 
    }
    arg1 = (struct _dwg_SPLINE_control_point *)(argp1);
    result = (struct _dwg_entity_SPLINE *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_SPLINE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SPLINE_control_point_x_set) {
  {
    struct _dwg_SPLINE_control_point *arg1 = (struct _dwg_SPLINE_control_point *) 0 ;
    double arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SPLINE_control_point_x_set(self,x);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SPLINE_control_point, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SPLINE_control_point_x_set" "', argument " "1"" of type '" "struct _dwg_SPLINE_control_point *""'"); 
    }
    arg1 = (struct _dwg_SPLINE_control_point *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SPLINE_control_point_x_set" "', argument " "2"" of type '" "double""'");
    } 
    arg2 = (double)(val2);
    if (arg1) (arg1)->x = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SPLINE_control_point_x_get) {
  {
    struct _dwg_SPLINE_control_point *arg1 = (struct _dwg_SPLINE_control_point *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    double result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SPLINE_control_point_x_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SPLINE_control_point, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SPLINE_control_point_x_get" "', argument " "1"" of type '" "struct _dwg_SPLINE_control_point *""'"); 
    }
    arg1 = (struct _dwg_SPLINE_control_point *)(argp1);
    result = (double) ((arg1)->x);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SPLINE_control_point_y_set) {
  {
    struct _dwg_SPLINE_control_point *arg1 = (struct _dwg_SPLINE_control_point *) 0 ;
    double arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SPLINE_control_point_y_set(self,y);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SPLINE_control_point, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SPLINE_control_point_y_set" "', argument " "1"" of type '" "struct _dwg_SPLINE_control_point *""'"); 
    }
    arg1 = (struct _dwg_SPLINE_control_point *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SPLINE_control_point_y_set" "', argument " "2"" of type '" "double""'");
    } 
    arg2 = (double)(val2);
    if (arg1) (arg1)->y = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SPLINE_control_point_y_get) {
  {
    struct _dwg_SPLINE_control_point *arg1 = (struct _dwg_SPLINE_control_point *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    double result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SPLINE_control_point_y_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SPLINE_control_point, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SPLINE_control_point_y_get" "', argument " "1"" of type '" "struct _dwg_SPLINE_control_point *""'"); 
    }
    arg1 = (struct _dwg_SPLINE_control_point *)(argp1);
    result = (double) ((arg1)->y);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SPLINE_control_point_z_set) {
  {
    struct _dwg_SPLINE_control_point *arg1 = (struct _dwg_SPLINE_control_point *) 0 ;
    double arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SPLINE_control_point_z_set(self,z);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SPLINE_control_point, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SPLINE_control_point_z_set" "', argument " "1"" of type '" "struct _dwg_SPLINE_control_point *""'"); 
    }
    arg1 = (struct _dwg_SPLINE_control_point *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SPLINE_control_point_z_set" "', argument " "2"" of type '" "double""'");
    } 
    arg2 = (double)(val2);
    if (arg1) (arg1)->z = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SPLINE_control_point_z_get) {
  {
    struct _dwg_SPLINE_control_point *arg1 = (struct _dwg_SPLINE_control_point *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    double result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SPLINE_control_point_z_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SPLINE_control_point, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SPLINE_control_point_z_get" "', argument " "1"" of type '" "struct _dwg_SPLINE_control_point *""'"); 
    }
    arg1 = (struct _dwg_SPLINE_control_point *)(argp1);
    result = (double) ((arg1)->z);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SPLINE_control_point_w_set) {
  {
    struct _dwg_SPLINE_control_point *arg1 = (struct _dwg_SPLINE_control_point *) 0 ;
    double arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SPLINE_control_point_w_set(self,w);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SPLINE_control_point, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SPLINE_control_point_w_set" "', argument " "1"" of type '" "struct _dwg_SPLINE_control_point *""'"); 
    }
    arg1 = (struct _dwg_SPLINE_control_point *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SPLINE_control_point_w_set" "', argument " "2"" of type '" "double""'");
    } 
    arg2 = (double)(val2);
    if (arg1) (arg1)->w = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SPLINE_control_point_w_get) {
  {
    struct _dwg_SPLINE_control_point *arg1 = (struct _dwg_SPLINE_control_point *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    double result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SPLINE_control_point_w_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SPLINE_control_point, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SPLINE_control_point_w_get" "', argument " "1"" of type '" "struct _dwg_SPLINE_control_point *""'"); 
    }
    arg1 = (struct _dwg_SPLINE_control_point *)(argp1);
    result = (double) ((arg1)->w);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_SPLINE_control_point) {
  {
    int argvi = 0;
    struct _dwg_SPLINE_control_point *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_SPLINE_control_point();");
    }
    result = (struct _dwg_SPLINE_control_point *)calloc(1, sizeof(struct _dwg_SPLINE_control_point));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_SPLINE_control_point, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_SPLINE_control_point) {
  {
    struct _dwg_SPLINE_control_point *arg1 = (struct _dwg_SPLINE_control_point *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_SPLINE_control_point(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SPLINE_control_point, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_SPLINE_control_point" "', argument " "1"" of type '" "struct _dwg_SPLINE_control_point *""'"); 
    }
    arg1 = (struct _dwg_SPLINE_control_point *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_parent_set) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SPLINE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_parent_get) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_flag_set) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SPLINE_flag_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_flag_get) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    result = (BITCODE_RS) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_scenario_set) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_scenario_set(self,scenario);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_scenario_set" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SPLINE_scenario_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->scenario = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_scenario_get) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_scenario_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_scenario_get" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    result = (BITCODE_BS) ((arg1)->scenario);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_degree_set) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_degree_set(self,degree);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_degree_set" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SPLINE_degree_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->degree = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_degree_get) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_degree_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_degree_get" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    result = (BITCODE_BS) ((arg1)->degree);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_splineflags_set) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_splineflags_set(self,splineflags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_splineflags_set" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SPLINE_splineflags_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->splineflags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_splineflags_get) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_splineflags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_splineflags_get" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    result = (BITCODE_BL) ((arg1)->splineflags);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_knotparam_set) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_knotparam_set(self,knotparam);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_knotparam_set" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SPLINE_knotparam_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->knotparam = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_knotparam_get) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_knotparam_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_knotparam_get" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    result = (BITCODE_BL) ((arg1)->knotparam);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_fit_tol_set) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_fit_tol_set(self,fit_tol);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_fit_tol_set" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SPLINE_fit_tol_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->fit_tol = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_fit_tol_get) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_fit_tol_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_fit_tol_get" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    result = (BITCODE_BD) ((arg1)->fit_tol);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_beg_tan_vec_set) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_beg_tan_vec_set(self,beg_tan_vec);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_beg_tan_vec_set" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SPLINE_beg_tan_vec_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->beg_tan_vec = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_beg_tan_vec_get) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_beg_tan_vec_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_beg_tan_vec_get" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->beg_tan_vec);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_end_tan_vec_set) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_end_tan_vec_set(self,end_tan_vec);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_end_tan_vec_set" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SPLINE_end_tan_vec_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->end_tan_vec = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_end_tan_vec_get) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_end_tan_vec_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_end_tan_vec_get" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->end_tan_vec);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_closed_b_set) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_closed_b_set(self,closed_b);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_closed_b_set" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SPLINE_closed_b_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->closed_b = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_closed_b_get) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_closed_b_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_closed_b_get" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    result = (BITCODE_B) ((arg1)->closed_b);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_periodic_set) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_periodic_set(self,periodic);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_periodic_set" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SPLINE_periodic_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->periodic = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_periodic_get) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_periodic_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_periodic_get" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    result = (BITCODE_B) ((arg1)->periodic);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_rational_set) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_rational_set(self,rational);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_rational_set" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SPLINE_rational_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->rational = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_rational_get) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_rational_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_rational_get" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    result = (BITCODE_B) ((arg1)->rational);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_weighted_set) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_weighted_set(self,weighted);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_weighted_set" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SPLINE_weighted_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->weighted = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_weighted_get) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_weighted_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_weighted_get" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    result = (BITCODE_B) ((arg1)->weighted);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_knot_tol_set) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_knot_tol_set(self,knot_tol);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_knot_tol_set" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SPLINE_knot_tol_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->knot_tol = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_knot_tol_get) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_knot_tol_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_knot_tol_get" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    result = (BITCODE_BD) ((arg1)->knot_tol);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_ctrl_tol_set) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_ctrl_tol_set(self,ctrl_tol);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_ctrl_tol_set" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SPLINE_ctrl_tol_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->ctrl_tol = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_ctrl_tol_get) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_ctrl_tol_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_ctrl_tol_get" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    result = (BITCODE_BD) ((arg1)->ctrl_tol);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_num_fit_pts_set) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_num_fit_pts_set(self,num_fit_pts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_num_fit_pts_set" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SPLINE_num_fit_pts_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_fit_pts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_num_fit_pts_get) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_num_fit_pts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_num_fit_pts_get" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_fit_pts);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_fit_pts_set) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    BITCODE_3DPOINT *arg2 = (BITCODE_3DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_fit_pts_set(self,fit_pts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_fit_pts_set" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SPLINE_fit_pts_set" "', argument " "2"" of type '" "BITCODE_3DPOINT *""'"); 
    }
    arg2 = (BITCODE_3DPOINT *)(argp2);
    if (arg1) (arg1)->fit_pts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_fit_pts_get) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_fit_pts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_fit_pts_get" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    result = (BITCODE_3DPOINT *) ((arg1)->fit_pts);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_num_knots_set) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_num_knots_set(self,num_knots);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_num_knots_set" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SPLINE_num_knots_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_knots = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_num_knots_get) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_num_knots_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_num_knots_get" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_knots);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_knots_set) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_knots_set(self,knots);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_knots_set" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SPLINE_knots_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->knots = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_knots_get) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_knots_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_knots_get" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    result = (BITCODE_BD *) ((arg1)->knots);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_num_ctrl_pts_set) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_num_ctrl_pts_set(self,num_ctrl_pts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_num_ctrl_pts_set" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SPLINE_num_ctrl_pts_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_ctrl_pts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_num_ctrl_pts_get) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_num_ctrl_pts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_num_ctrl_pts_get" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_ctrl_pts);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_ctrl_pts_set) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    Dwg_SPLINE_control_point *arg2 = (Dwg_SPLINE_control_point *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_ctrl_pts_set(self,ctrl_pts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_ctrl_pts_set" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_SPLINE_control_point, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SPLINE_ctrl_pts_set" "', argument " "2"" of type '" "Dwg_SPLINE_control_point *""'"); 
    }
    arg2 = (Dwg_SPLINE_control_point *)(argp2);
    if (arg1) (arg1)->ctrl_pts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SPLINE_ctrl_pts_get) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_SPLINE_control_point *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SPLINE_ctrl_pts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SPLINE_ctrl_pts_get" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    result = (Dwg_SPLINE_control_point *) ((arg1)->ctrl_pts);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_SPLINE_control_point, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_SPLINE) {
  {
    int argvi = 0;
    struct _dwg_entity_SPLINE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_SPLINE();");
    }
    result = (struct _dwg_entity_SPLINE *)calloc(1, sizeof(struct _dwg_entity_SPLINE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_SPLINE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_SPLINE) {
  {
    struct _dwg_entity_SPLINE *arg1 = (struct _dwg_entity_SPLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_SPLINE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SPLINE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_SPLINE" "', argument " "1"" of type '" "struct _dwg_entity_SPLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_SPLINE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_wire_parent_set) {
  {
    struct _dwg_3DSOLID_wire *arg1 = (struct _dwg_3DSOLID_wire *) 0 ;
    struct _dwg_entity_3DSOLID *arg2 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_3DSOLID_wire_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_wire, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_wire_parent_set" "', argument " "1"" of type '" "struct _dwg_3DSOLID_wire *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_wire *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_3DSOLID, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_3DSOLID_wire_parent_set" "', argument " "2"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg2 = (struct _dwg_entity_3DSOLID *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_wire_parent_get) {
  {
    struct _dwg_3DSOLID_wire *arg1 = (struct _dwg_3DSOLID_wire *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_entity_3DSOLID *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_3DSOLID_wire_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_wire, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_wire_parent_get" "', argument " "1"" of type '" "struct _dwg_3DSOLID_wire *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_wire *)(argp1);
    result = (struct _dwg_entity_3DSOLID *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_3DSOLID, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_wire_type_set) {
  {
    struct _dwg_3DSOLID_wire *arg1 = (struct _dwg_3DSOLID_wire *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_3DSOLID_wire_type_set(self,type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_wire, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_wire_type_set" "', argument " "1"" of type '" "struct _dwg_3DSOLID_wire *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_wire *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_3DSOLID_wire_type_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_wire_type_get) {
  {
    struct _dwg_3DSOLID_wire *arg1 = (struct _dwg_3DSOLID_wire *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_3DSOLID_wire_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_wire, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_wire_type_get" "', argument " "1"" of type '" "struct _dwg_3DSOLID_wire *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_wire *)(argp1);
    result = (BITCODE_RC) ((arg1)->type);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_wire_selection_marker_set) {
  {
    struct _dwg_3DSOLID_wire *arg1 = (struct _dwg_3DSOLID_wire *) 0 ;
    BITCODE_BLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_3DSOLID_wire_selection_marker_set(self,selection_marker);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_wire, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_wire_selection_marker_set" "', argument " "1"" of type '" "struct _dwg_3DSOLID_wire *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_wire *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_3DSOLID_wire_selection_marker_set" "', argument " "2"" of type '" "BITCODE_BLd""'");
    } 
    arg2 = (BITCODE_BLd)(val2);
    if (arg1) (arg1)->selection_marker = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_wire_selection_marker_get) {
  {
    struct _dwg_3DSOLID_wire *arg1 = (struct _dwg_3DSOLID_wire *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_3DSOLID_wire_selection_marker_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_wire, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_wire_selection_marker_get" "', argument " "1"" of type '" "struct _dwg_3DSOLID_wire *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_wire *)(argp1);
    result = (BITCODE_BLd) ((arg1)->selection_marker);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_wire_color_set) {
  {
    struct _dwg_3DSOLID_wire *arg1 = (struct _dwg_3DSOLID_wire *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_3DSOLID_wire_color_set(self,color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_wire, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_wire_color_set" "', argument " "1"" of type '" "struct _dwg_3DSOLID_wire *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_wire *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_3DSOLID_wire_color_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->color = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_wire_color_get) {
  {
    struct _dwg_3DSOLID_wire *arg1 = (struct _dwg_3DSOLID_wire *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_3DSOLID_wire_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_wire, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_wire_color_get" "', argument " "1"" of type '" "struct _dwg_3DSOLID_wire *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_wire *)(argp1);
    result = (BITCODE_BL) ((arg1)->color);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_wire_acis_index_set) {
  {
    struct _dwg_3DSOLID_wire *arg1 = (struct _dwg_3DSOLID_wire *) 0 ;
    BITCODE_BLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_3DSOLID_wire_acis_index_set(self,acis_index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_wire, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_wire_acis_index_set" "', argument " "1"" of type '" "struct _dwg_3DSOLID_wire *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_wire *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_3DSOLID_wire_acis_index_set" "', argument " "2"" of type '" "BITCODE_BLd""'");
    } 
    arg2 = (BITCODE_BLd)(val2);
    if (arg1) (arg1)->acis_index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_wire_acis_index_get) {
  {
    struct _dwg_3DSOLID_wire *arg1 = (struct _dwg_3DSOLID_wire *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_3DSOLID_wire_acis_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_wire, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_wire_acis_index_get" "', argument " "1"" of type '" "struct _dwg_3DSOLID_wire *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_wire *)(argp1);
    result = (BITCODE_BLd) ((arg1)->acis_index);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_wire_num_points_set) {
  {
    struct _dwg_3DSOLID_wire *arg1 = (struct _dwg_3DSOLID_wire *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_3DSOLID_wire_num_points_set(self,num_points);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_wire, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_wire_num_points_set" "', argument " "1"" of type '" "struct _dwg_3DSOLID_wire *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_wire *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_3DSOLID_wire_num_points_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_points = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_wire_num_points_get) {
  {
    struct _dwg_3DSOLID_wire *arg1 = (struct _dwg_3DSOLID_wire *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_3DSOLID_wire_num_points_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_wire, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_wire_num_points_get" "', argument " "1"" of type '" "struct _dwg_3DSOLID_wire *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_wire *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_points);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_wire_points_set) {
  {
    struct _dwg_3DSOLID_wire *arg1 = (struct _dwg_3DSOLID_wire *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_3DSOLID_wire_points_set(self,points);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_wire, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_wire_points_set" "', argument " "1"" of type '" "struct _dwg_3DSOLID_wire *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_wire *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_3DSOLID_wire_points_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->points = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_wire_points_get) {
  {
    struct _dwg_3DSOLID_wire *arg1 = (struct _dwg_3DSOLID_wire *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_3DSOLID_wire_points_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_wire, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_wire_points_get" "', argument " "1"" of type '" "struct _dwg_3DSOLID_wire *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_wire *)(argp1);
    result = (BITCODE_3BD *) ((arg1)->points);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_wire_transform_present_set) {
  {
    struct _dwg_3DSOLID_wire *arg1 = (struct _dwg_3DSOLID_wire *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_3DSOLID_wire_transform_present_set(self,transform_present);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_wire, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_wire_transform_present_set" "', argument " "1"" of type '" "struct _dwg_3DSOLID_wire *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_wire *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_3DSOLID_wire_transform_present_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->transform_present = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_wire_transform_present_get) {
  {
    struct _dwg_3DSOLID_wire *arg1 = (struct _dwg_3DSOLID_wire *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_3DSOLID_wire_transform_present_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_wire, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_wire_transform_present_get" "', argument " "1"" of type '" "struct _dwg_3DSOLID_wire *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_wire *)(argp1);
    result = (BITCODE_B) ((arg1)->transform_present);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_wire_axis_x_set) {
  {
    struct _dwg_3DSOLID_wire *arg1 = (struct _dwg_3DSOLID_wire *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_3DSOLID_wire_axis_x_set(self,axis_x);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_wire, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_wire_axis_x_set" "', argument " "1"" of type '" "struct _dwg_3DSOLID_wire *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_wire *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_3DSOLID_wire_axis_x_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->axis_x = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_wire_axis_x_get) {
  {
    struct _dwg_3DSOLID_wire *arg1 = (struct _dwg_3DSOLID_wire *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_3DSOLID_wire_axis_x_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_wire, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_wire_axis_x_get" "', argument " "1"" of type '" "struct _dwg_3DSOLID_wire *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_wire *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->axis_x);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_wire_axis_y_set) {
  {
    struct _dwg_3DSOLID_wire *arg1 = (struct _dwg_3DSOLID_wire *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_3DSOLID_wire_axis_y_set(self,axis_y);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_wire, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_wire_axis_y_set" "', argument " "1"" of type '" "struct _dwg_3DSOLID_wire *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_wire *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_3DSOLID_wire_axis_y_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->axis_y = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_wire_axis_y_get) {
  {
    struct _dwg_3DSOLID_wire *arg1 = (struct _dwg_3DSOLID_wire *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_3DSOLID_wire_axis_y_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_wire, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_wire_axis_y_get" "', argument " "1"" of type '" "struct _dwg_3DSOLID_wire *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_wire *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->axis_y);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_wire_axis_z_set) {
  {
    struct _dwg_3DSOLID_wire *arg1 = (struct _dwg_3DSOLID_wire *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_3DSOLID_wire_axis_z_set(self,axis_z);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_wire, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_wire_axis_z_set" "', argument " "1"" of type '" "struct _dwg_3DSOLID_wire *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_wire *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_3DSOLID_wire_axis_z_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->axis_z = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_wire_axis_z_get) {
  {
    struct _dwg_3DSOLID_wire *arg1 = (struct _dwg_3DSOLID_wire *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_3DSOLID_wire_axis_z_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_wire, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_wire_axis_z_get" "', argument " "1"" of type '" "struct _dwg_3DSOLID_wire *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_wire *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->axis_z);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_wire_translation_set) {
  {
    struct _dwg_3DSOLID_wire *arg1 = (struct _dwg_3DSOLID_wire *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_3DSOLID_wire_translation_set(self,translation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_wire, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_wire_translation_set" "', argument " "1"" of type '" "struct _dwg_3DSOLID_wire *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_wire *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_3DSOLID_wire_translation_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->translation = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_wire_translation_get) {
  {
    struct _dwg_3DSOLID_wire *arg1 = (struct _dwg_3DSOLID_wire *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_3DSOLID_wire_translation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_wire, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_wire_translation_get" "', argument " "1"" of type '" "struct _dwg_3DSOLID_wire *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_wire *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->translation);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_wire_scale_set) {
  {
    struct _dwg_3DSOLID_wire *arg1 = (struct _dwg_3DSOLID_wire *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_3DSOLID_wire_scale_set(self,scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_wire, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_wire_scale_set" "', argument " "1"" of type '" "struct _dwg_3DSOLID_wire *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_wire *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_3DSOLID_wire_scale_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->scale = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_wire_scale_get) {
  {
    struct _dwg_3DSOLID_wire *arg1 = (struct _dwg_3DSOLID_wire *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_3DSOLID_wire_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_wire, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_wire_scale_get" "', argument " "1"" of type '" "struct _dwg_3DSOLID_wire *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_wire *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->scale);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_wire_has_rotation_set) {
  {
    struct _dwg_3DSOLID_wire *arg1 = (struct _dwg_3DSOLID_wire *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_3DSOLID_wire_has_rotation_set(self,has_rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_wire, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_wire_has_rotation_set" "', argument " "1"" of type '" "struct _dwg_3DSOLID_wire *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_wire *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_3DSOLID_wire_has_rotation_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_wire_has_rotation_get) {
  {
    struct _dwg_3DSOLID_wire *arg1 = (struct _dwg_3DSOLID_wire *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_3DSOLID_wire_has_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_wire, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_wire_has_rotation_get" "', argument " "1"" of type '" "struct _dwg_3DSOLID_wire *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_wire *)(argp1);
    result = (BITCODE_B) ((arg1)->has_rotation);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_wire_has_reflection_set) {
  {
    struct _dwg_3DSOLID_wire *arg1 = (struct _dwg_3DSOLID_wire *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_3DSOLID_wire_has_reflection_set(self,has_reflection);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_wire, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_wire_has_reflection_set" "', argument " "1"" of type '" "struct _dwg_3DSOLID_wire *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_wire *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_3DSOLID_wire_has_reflection_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_reflection = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_wire_has_reflection_get) {
  {
    struct _dwg_3DSOLID_wire *arg1 = (struct _dwg_3DSOLID_wire *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_3DSOLID_wire_has_reflection_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_wire, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_wire_has_reflection_get" "', argument " "1"" of type '" "struct _dwg_3DSOLID_wire *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_wire *)(argp1);
    result = (BITCODE_B) ((arg1)->has_reflection);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_wire_has_shear_set) {
  {
    struct _dwg_3DSOLID_wire *arg1 = (struct _dwg_3DSOLID_wire *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_3DSOLID_wire_has_shear_set(self,has_shear);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_wire, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_wire_has_shear_set" "', argument " "1"" of type '" "struct _dwg_3DSOLID_wire *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_wire *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_3DSOLID_wire_has_shear_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_shear = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_wire_has_shear_get) {
  {
    struct _dwg_3DSOLID_wire *arg1 = (struct _dwg_3DSOLID_wire *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_3DSOLID_wire_has_shear_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_wire, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_wire_has_shear_get" "', argument " "1"" of type '" "struct _dwg_3DSOLID_wire *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_wire *)(argp1);
    result = (BITCODE_B) ((arg1)->has_shear);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_3DSOLID_wire) {
  {
    int argvi = 0;
    struct _dwg_3DSOLID_wire *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_3DSOLID_wire();");
    }
    result = (struct _dwg_3DSOLID_wire *)calloc(1, sizeof(struct _dwg_3DSOLID_wire));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_3DSOLID_wire, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_3DSOLID_wire) {
  {
    struct _dwg_3DSOLID_wire *arg1 = (struct _dwg_3DSOLID_wire *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_3DSOLID_wire(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_wire, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_3DSOLID_wire" "', argument " "1"" of type '" "struct _dwg_3DSOLID_wire *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_wire *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_silhouette_parent_set) {
  {
    struct _dwg_3DSOLID_silhouette *arg1 = (struct _dwg_3DSOLID_silhouette *) 0 ;
    struct _dwg_entity_3DSOLID *arg2 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_3DSOLID_silhouette_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_silhouette, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_silhouette_parent_set" "', argument " "1"" of type '" "struct _dwg_3DSOLID_silhouette *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_silhouette *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_3DSOLID, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_3DSOLID_silhouette_parent_set" "', argument " "2"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg2 = (struct _dwg_entity_3DSOLID *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_silhouette_parent_get) {
  {
    struct _dwg_3DSOLID_silhouette *arg1 = (struct _dwg_3DSOLID_silhouette *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_entity_3DSOLID *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_3DSOLID_silhouette_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_silhouette, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_silhouette_parent_get" "', argument " "1"" of type '" "struct _dwg_3DSOLID_silhouette *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_silhouette *)(argp1);
    result = (struct _dwg_entity_3DSOLID *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_3DSOLID, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_silhouette_vp_id_set) {
  {
    struct _dwg_3DSOLID_silhouette *arg1 = (struct _dwg_3DSOLID_silhouette *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_3DSOLID_silhouette_vp_id_set(self,vp_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_silhouette, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_silhouette_vp_id_set" "', argument " "1"" of type '" "struct _dwg_3DSOLID_silhouette *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_silhouette *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_3DSOLID_silhouette_vp_id_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->vp_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_silhouette_vp_id_get) {
  {
    struct _dwg_3DSOLID_silhouette *arg1 = (struct _dwg_3DSOLID_silhouette *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_3DSOLID_silhouette_vp_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_silhouette, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_silhouette_vp_id_get" "', argument " "1"" of type '" "struct _dwg_3DSOLID_silhouette *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_silhouette *)(argp1);
    result = (BITCODE_BL) ((arg1)->vp_id);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_silhouette_vp_target_set) {
  {
    struct _dwg_3DSOLID_silhouette *arg1 = (struct _dwg_3DSOLID_silhouette *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_3DSOLID_silhouette_vp_target_set(self,vp_target);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_silhouette, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_silhouette_vp_target_set" "', argument " "1"" of type '" "struct _dwg_3DSOLID_silhouette *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_silhouette *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_3DSOLID_silhouette_vp_target_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->vp_target = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_silhouette_vp_target_get) {
  {
    struct _dwg_3DSOLID_silhouette *arg1 = (struct _dwg_3DSOLID_silhouette *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_3DSOLID_silhouette_vp_target_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_silhouette, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_silhouette_vp_target_get" "', argument " "1"" of type '" "struct _dwg_3DSOLID_silhouette *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_silhouette *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->vp_target);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_silhouette_vp_dir_from_target_set) {
  {
    struct _dwg_3DSOLID_silhouette *arg1 = (struct _dwg_3DSOLID_silhouette *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_3DSOLID_silhouette_vp_dir_from_target_set(self,vp_dir_from_target);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_silhouette, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_silhouette_vp_dir_from_target_set" "', argument " "1"" of type '" "struct _dwg_3DSOLID_silhouette *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_silhouette *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_3DSOLID_silhouette_vp_dir_from_target_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->vp_dir_from_target = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_silhouette_vp_dir_from_target_get) {
  {
    struct _dwg_3DSOLID_silhouette *arg1 = (struct _dwg_3DSOLID_silhouette *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_3DSOLID_silhouette_vp_dir_from_target_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_silhouette, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_silhouette_vp_dir_from_target_get" "', argument " "1"" of type '" "struct _dwg_3DSOLID_silhouette *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_silhouette *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->vp_dir_from_target);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_silhouette_vp_up_dir_set) {
  {
    struct _dwg_3DSOLID_silhouette *arg1 = (struct _dwg_3DSOLID_silhouette *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_3DSOLID_silhouette_vp_up_dir_set(self,vp_up_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_silhouette, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_silhouette_vp_up_dir_set" "', argument " "1"" of type '" "struct _dwg_3DSOLID_silhouette *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_silhouette *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_3DSOLID_silhouette_vp_up_dir_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->vp_up_dir = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_silhouette_vp_up_dir_get) {
  {
    struct _dwg_3DSOLID_silhouette *arg1 = (struct _dwg_3DSOLID_silhouette *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_3DSOLID_silhouette_vp_up_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_silhouette, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_silhouette_vp_up_dir_get" "', argument " "1"" of type '" "struct _dwg_3DSOLID_silhouette *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_silhouette *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->vp_up_dir);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_silhouette_vp_perspective_set) {
  {
    struct _dwg_3DSOLID_silhouette *arg1 = (struct _dwg_3DSOLID_silhouette *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_3DSOLID_silhouette_vp_perspective_set(self,vp_perspective);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_silhouette, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_silhouette_vp_perspective_set" "', argument " "1"" of type '" "struct _dwg_3DSOLID_silhouette *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_silhouette *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_3DSOLID_silhouette_vp_perspective_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->vp_perspective = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_silhouette_vp_perspective_get) {
  {
    struct _dwg_3DSOLID_silhouette *arg1 = (struct _dwg_3DSOLID_silhouette *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_3DSOLID_silhouette_vp_perspective_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_silhouette, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_silhouette_vp_perspective_get" "', argument " "1"" of type '" "struct _dwg_3DSOLID_silhouette *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_silhouette *)(argp1);
    result = (BITCODE_B) ((arg1)->vp_perspective);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_silhouette_has_wires_set) {
  {
    struct _dwg_3DSOLID_silhouette *arg1 = (struct _dwg_3DSOLID_silhouette *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_3DSOLID_silhouette_has_wires_set(self,has_wires);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_silhouette, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_silhouette_has_wires_set" "', argument " "1"" of type '" "struct _dwg_3DSOLID_silhouette *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_silhouette *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_3DSOLID_silhouette_has_wires_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_wires = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_silhouette_has_wires_get) {
  {
    struct _dwg_3DSOLID_silhouette *arg1 = (struct _dwg_3DSOLID_silhouette *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_3DSOLID_silhouette_has_wires_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_silhouette, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_silhouette_has_wires_get" "', argument " "1"" of type '" "struct _dwg_3DSOLID_silhouette *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_silhouette *)(argp1);
    result = (BITCODE_B) ((arg1)->has_wires);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_silhouette_num_wires_set) {
  {
    struct _dwg_3DSOLID_silhouette *arg1 = (struct _dwg_3DSOLID_silhouette *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_3DSOLID_silhouette_num_wires_set(self,num_wires);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_silhouette, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_silhouette_num_wires_set" "', argument " "1"" of type '" "struct _dwg_3DSOLID_silhouette *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_silhouette *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_3DSOLID_silhouette_num_wires_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_wires = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_silhouette_num_wires_get) {
  {
    struct _dwg_3DSOLID_silhouette *arg1 = (struct _dwg_3DSOLID_silhouette *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_3DSOLID_silhouette_num_wires_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_silhouette, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_silhouette_num_wires_get" "', argument " "1"" of type '" "struct _dwg_3DSOLID_silhouette *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_silhouette *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_wires);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_silhouette_wires_set) {
  {
    struct _dwg_3DSOLID_silhouette *arg1 = (struct _dwg_3DSOLID_silhouette *) 0 ;
    Dwg_3DSOLID_wire *arg2 = (Dwg_3DSOLID_wire *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_3DSOLID_silhouette_wires_set(self,wires);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_silhouette, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_silhouette_wires_set" "', argument " "1"" of type '" "struct _dwg_3DSOLID_silhouette *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_silhouette *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_3DSOLID_wire, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_3DSOLID_silhouette_wires_set" "', argument " "2"" of type '" "Dwg_3DSOLID_wire *""'"); 
    }
    arg2 = (Dwg_3DSOLID_wire *)(argp2);
    if (arg1) (arg1)->wires = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_silhouette_wires_get) {
  {
    struct _dwg_3DSOLID_silhouette *arg1 = (struct _dwg_3DSOLID_silhouette *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_3DSOLID_wire *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_3DSOLID_silhouette_wires_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_silhouette, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_silhouette_wires_get" "', argument " "1"" of type '" "struct _dwg_3DSOLID_silhouette *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_silhouette *)(argp1);
    result = (Dwg_3DSOLID_wire *) ((arg1)->wires);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_3DSOLID_wire, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_3DSOLID_silhouette) {
  {
    int argvi = 0;
    struct _dwg_3DSOLID_silhouette *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_3DSOLID_silhouette();");
    }
    result = (struct _dwg_3DSOLID_silhouette *)calloc(1, sizeof(struct _dwg_3DSOLID_silhouette));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_3DSOLID_silhouette, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_3DSOLID_silhouette) {
  {
    struct _dwg_3DSOLID_silhouette *arg1 = (struct _dwg_3DSOLID_silhouette *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_3DSOLID_silhouette(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_silhouette, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_3DSOLID_silhouette" "', argument " "1"" of type '" "struct _dwg_3DSOLID_silhouette *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_silhouette *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_material_parent_set) {
  {
    struct _dwg_3DSOLID_material *arg1 = (struct _dwg_3DSOLID_material *) 0 ;
    struct _dwg_entity_3DSOLID *arg2 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_3DSOLID_material_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_material, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_material_parent_set" "', argument " "1"" of type '" "struct _dwg_3DSOLID_material *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_material *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_3DSOLID, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_3DSOLID_material_parent_set" "', argument " "2"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg2 = (struct _dwg_entity_3DSOLID *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_material_parent_get) {
  {
    struct _dwg_3DSOLID_material *arg1 = (struct _dwg_3DSOLID_material *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_entity_3DSOLID *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_3DSOLID_material_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_material, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_material_parent_get" "', argument " "1"" of type '" "struct _dwg_3DSOLID_material *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_material *)(argp1);
    result = (struct _dwg_entity_3DSOLID *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_3DSOLID, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_material_array_index_set) {
  {
    struct _dwg_3DSOLID_material *arg1 = (struct _dwg_3DSOLID_material *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_3DSOLID_material_array_index_set(self,array_index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_material, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_material_array_index_set" "', argument " "1"" of type '" "struct _dwg_3DSOLID_material *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_material *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_3DSOLID_material_array_index_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->array_index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_material_array_index_get) {
  {
    struct _dwg_3DSOLID_material *arg1 = (struct _dwg_3DSOLID_material *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_3DSOLID_material_array_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_material, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_material_array_index_get" "', argument " "1"" of type '" "struct _dwg_3DSOLID_material *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_material *)(argp1);
    result = (BITCODE_BL) ((arg1)->array_index);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_material_mat_absref_set) {
  {
    struct _dwg_3DSOLID_material *arg1 = (struct _dwg_3DSOLID_material *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_3DSOLID_material_mat_absref_set(self,mat_absref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_material, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_material_mat_absref_set" "', argument " "1"" of type '" "struct _dwg_3DSOLID_material *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_material *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_3DSOLID_material_mat_absref_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->mat_absref = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_material_mat_absref_get) {
  {
    struct _dwg_3DSOLID_material *arg1 = (struct _dwg_3DSOLID_material *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_3DSOLID_material_mat_absref_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_material, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_material_mat_absref_get" "', argument " "1"" of type '" "struct _dwg_3DSOLID_material *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_material *)(argp1);
    result = (BITCODE_BL) ((arg1)->mat_absref);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_material_material_handle_set) {
  {
    struct _dwg_3DSOLID_material *arg1 = (struct _dwg_3DSOLID_material *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_3DSOLID_material_material_handle_set(self,material_handle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_material, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_material_material_handle_set" "', argument " "1"" of type '" "struct _dwg_3DSOLID_material *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_material *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_3DSOLID_material_material_handle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->material_handle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_3DSOLID_material_material_handle_get) {
  {
    struct _dwg_3DSOLID_material *arg1 = (struct _dwg_3DSOLID_material *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_3DSOLID_material_material_handle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_material, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_3DSOLID_material_material_handle_get" "', argument " "1"" of type '" "struct _dwg_3DSOLID_material *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_material *)(argp1);
    result = (BITCODE_H) ((arg1)->material_handle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_3DSOLID_material) {
  {
    int argvi = 0;
    struct _dwg_3DSOLID_material *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_3DSOLID_material();");
    }
    result = (struct _dwg_3DSOLID_material *)calloc(1, sizeof(struct _dwg_3DSOLID_material));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_3DSOLID_material, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_3DSOLID_material) {
  {
    struct _dwg_3DSOLID_material *arg1 = (struct _dwg_3DSOLID_material *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_3DSOLID_material(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_3DSOLID_material, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_3DSOLID_material" "', argument " "1"" of type '" "struct _dwg_3DSOLID_material *""'"); 
    }
    arg1 = (struct _dwg_3DSOLID_material *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_parent_set) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity__3DSOLID_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_parent_get) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_acis_empty_set) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_acis_empty_set(self,acis_empty);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_acis_empty_set" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity__3DSOLID_acis_empty_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->acis_empty = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_acis_empty_get) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_acis_empty_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_acis_empty_get" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    result = (BITCODE_B) ((arg1)->acis_empty);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_unknown_set) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_unknown_set(self,unknown);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_unknown_set" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity__3DSOLID_unknown_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_unknown_get) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_unknown_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_unknown_get" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_version_set) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_version_set(self,version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_version_set" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity__3DSOLID_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_version_get) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_version_get" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    result = (BITCODE_BS) ((arg1)->version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_num_blocks_set) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_num_blocks_set(self,num_blocks);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_num_blocks_set" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity__3DSOLID_num_blocks_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_blocks = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_num_blocks_get) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_num_blocks_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_num_blocks_get" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_blocks);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_block_size_set) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_block_size_set(self,block_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_block_size_set" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity__3DSOLID_block_size_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->block_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_block_size_get) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_block_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_block_size_get" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    result = (BITCODE_BL *) ((arg1)->block_size);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_encr_sat_data_set) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    char **arg2 = (char **) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_encr_sat_data_set(self,encr_sat_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_encr_sat_data_set" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity__3DSOLID_encr_sat_data_set" "', argument " "2"" of type '" "char **""'"); 
    }
    arg2 = (char **)(argp2);
    if (arg1) (arg1)->encr_sat_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_encr_sat_data_get) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    char **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_encr_sat_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_encr_sat_data_get" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    result = (char **) ((arg1)->encr_sat_data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_sab_size_set) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_sab_size_set(self,sab_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_sab_size_set" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity__3DSOLID_sab_size_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->sab_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_sab_size_get) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_sab_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_sab_size_get" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    result = (BITCODE_BL) ((arg1)->sab_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_acis_data_set) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_acis_data_set(self,acis_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_acis_data_set" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity__3DSOLID_acis_data_set" "', argument " "2"" of type '" "BITCODE_RC *""'"); 
    }
    arg2 = (BITCODE_RC *)(argp2);
    if (arg1) (arg1)->acis_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_acis_data_get) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_acis_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_acis_data_get" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    result = (BITCODE_RC *) ((arg1)->acis_data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_wireframe_data_present_set) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_wireframe_data_present_set(self,wireframe_data_present);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_wireframe_data_present_set" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity__3DSOLID_wireframe_data_present_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->wireframe_data_present = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_wireframe_data_present_get) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_wireframe_data_present_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_wireframe_data_present_get" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    result = (BITCODE_B) ((arg1)->wireframe_data_present);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_point_present_set) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_point_present_set(self,point_present);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_point_present_set" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity__3DSOLID_point_present_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->point_present = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_point_present_get) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_point_present_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_point_present_get" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    result = (BITCODE_B) ((arg1)->point_present);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_point_set) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_point_set(self,point);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_point_set" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity__3DSOLID_point_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->point = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_point_get) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_point_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_point_get" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->point);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_isolines_set) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_isolines_set(self,isolines);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_isolines_set" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity__3DSOLID_isolines_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->isolines = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_isolines_get) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_isolines_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_isolines_get" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    result = (BITCODE_BL) ((arg1)->isolines);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_isoline_present_set) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_isoline_present_set(self,isoline_present);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_isoline_present_set" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity__3DSOLID_isoline_present_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->isoline_present = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_isoline_present_get) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_isoline_present_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_isoline_present_get" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    result = (BITCODE_B) ((arg1)->isoline_present);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_num_wires_set) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_num_wires_set(self,num_wires);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_num_wires_set" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity__3DSOLID_num_wires_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_wires = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_num_wires_get) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_num_wires_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_num_wires_get" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_wires);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_wires_set) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    Dwg_3DSOLID_wire *arg2 = (Dwg_3DSOLID_wire *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_wires_set(self,wires);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_wires_set" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_3DSOLID_wire, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity__3DSOLID_wires_set" "', argument " "2"" of type '" "Dwg_3DSOLID_wire *""'"); 
    }
    arg2 = (Dwg_3DSOLID_wire *)(argp2);
    if (arg1) (arg1)->wires = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_wires_get) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_3DSOLID_wire *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_wires_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_wires_get" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    result = (Dwg_3DSOLID_wire *) ((arg1)->wires);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_3DSOLID_wire, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_num_silhouettes_set) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_num_silhouettes_set(self,num_silhouettes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_num_silhouettes_set" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity__3DSOLID_num_silhouettes_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_silhouettes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_num_silhouettes_get) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_num_silhouettes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_num_silhouettes_get" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_silhouettes);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_silhouettes_set) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    Dwg_3DSOLID_silhouette *arg2 = (Dwg_3DSOLID_silhouette *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_silhouettes_set(self,silhouettes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_silhouettes_set" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_3DSOLID_silhouette, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity__3DSOLID_silhouettes_set" "', argument " "2"" of type '" "Dwg_3DSOLID_silhouette *""'"); 
    }
    arg2 = (Dwg_3DSOLID_silhouette *)(argp2);
    if (arg1) (arg1)->silhouettes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_silhouettes_get) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_3DSOLID_silhouette *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_silhouettes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_silhouettes_get" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    result = (Dwg_3DSOLID_silhouette *) ((arg1)->silhouettes);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_3DSOLID_silhouette, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID__dxf_sab_converted_set) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID__dxf_sab_converted_set(self,_dxf_sab_converted);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID__dxf_sab_converted_set" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity__3DSOLID__dxf_sab_converted_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->_dxf_sab_converted = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID__dxf_sab_converted_get) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID__dxf_sab_converted_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID__dxf_sab_converted_get" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    result = (BITCODE_B) ((arg1)->_dxf_sab_converted);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_acis_empty2_set) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_acis_empty2_set(self,acis_empty2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_acis_empty2_set" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity__3DSOLID_acis_empty2_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->acis_empty2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_acis_empty2_get) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_acis_empty2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_acis_empty2_get" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    result = (BITCODE_B) ((arg1)->acis_empty2);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_extra_acis_data_set) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    struct _dwg_entity_3DSOLID *arg2 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_extra_acis_data_set(self,extra_acis_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_extra_acis_data_set" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_3DSOLID, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity__3DSOLID_extra_acis_data_set" "', argument " "2"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg2 = (struct _dwg_entity_3DSOLID *)(argp2);
    if (arg1) (arg1)->extra_acis_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_extra_acis_data_get) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_entity_3DSOLID *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_extra_acis_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_extra_acis_data_get" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    result = (struct _dwg_entity_3DSOLID *) ((arg1)->extra_acis_data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_3DSOLID, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_num_materials_set) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_num_materials_set(self,num_materials);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_num_materials_set" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity__3DSOLID_num_materials_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_materials = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_num_materials_get) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_num_materials_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_num_materials_get" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_materials);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_materials_set) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    Dwg_3DSOLID_material *arg2 = (Dwg_3DSOLID_material *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_materials_set(self,materials);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_materials_set" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_3DSOLID_material, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity__3DSOLID_materials_set" "', argument " "2"" of type '" "Dwg_3DSOLID_material *""'"); 
    }
    arg2 = (Dwg_3DSOLID_material *)(argp2);
    if (arg1) (arg1)->materials = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_materials_get) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_3DSOLID_material *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_materials_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_materials_get" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    result = (Dwg_3DSOLID_material *) ((arg1)->materials);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_3DSOLID_material, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_revision_guid_set) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) (BITCODE_RC *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_revision_guid_set(self,revision_guid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_revision_guid_set" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity__3DSOLID_revision_guid_set" "', argument " "2"" of type '" "BITCODE_RC [39]""'"); 
    } 
    arg2 = (BITCODE_RC *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)39; ++ii) *(BITCODE_RC *)&arg1->revision_guid[ii] = *((BITCODE_RC *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""revision_guid""' of type '""BITCODE_RC [39]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_revision_guid_get) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_revision_guid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_revision_guid_get" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    result = (BITCODE_RC *)(BITCODE_RC *) ((arg1)->revision_guid);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_revision_major_set) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_revision_major_set(self,revision_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_revision_major_set" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity__3DSOLID_revision_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->revision_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_revision_major_get) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_revision_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_revision_major_get" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    result = (BITCODE_BL) ((arg1)->revision_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_revision_minor1_set) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_revision_minor1_set(self,revision_minor1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_revision_minor1_set" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity__3DSOLID_revision_minor1_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->revision_minor1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_revision_minor1_get) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_revision_minor1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_revision_minor1_get" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    result = (BITCODE_BS) ((arg1)->revision_minor1);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_revision_minor2_set) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_revision_minor2_set(self,revision_minor2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_revision_minor2_set" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity__3DSOLID_revision_minor2_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->revision_minor2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_revision_minor2_get) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_revision_minor2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_revision_minor2_get" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    result = (BITCODE_BS) ((arg1)->revision_minor2);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_revision_bytes_set) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) (BITCODE_RC *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_revision_bytes_set(self,revision_bytes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_revision_bytes_set" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity__3DSOLID_revision_bytes_set" "', argument " "2"" of type '" "BITCODE_RC [9]""'"); 
    } 
    arg2 = (BITCODE_RC *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)9; ++ii) *(BITCODE_RC *)&arg1->revision_bytes[ii] = *((BITCODE_RC *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""revision_bytes""' of type '""BITCODE_RC [9]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_revision_bytes_get) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_revision_bytes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_revision_bytes_get" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    result = (BITCODE_RC *)(BITCODE_RC *) ((arg1)->revision_bytes);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_end_marker_set) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_end_marker_set(self,end_marker);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_end_marker_set" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity__3DSOLID_end_marker_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->end_marker = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_end_marker_get) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_end_marker_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_end_marker_get" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    result = (BITCODE_BL) ((arg1)->end_marker);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_history_id_set) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_history_id_set(self,history_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_history_id_set" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity__3DSOLID_history_id_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->history_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_history_id_get) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_history_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_history_id_get" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    result = (BITCODE_H) ((arg1)->history_id);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_has_revision_guid_set) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_has_revision_guid_set(self,has_revision_guid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_has_revision_guid_set" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity__3DSOLID_has_revision_guid_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_revision_guid = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_has_revision_guid_get) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_has_revision_guid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_has_revision_guid_get" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    result = (BITCODE_B) ((arg1)->has_revision_guid);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_acis_empty_bit_set) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_acis_empty_bit_set(self,acis_empty_bit);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_acis_empty_bit_set" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity__3DSOLID_acis_empty_bit_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->acis_empty_bit = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DSOLID_acis_empty_bit_get) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DSOLID_acis_empty_bit_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DSOLID_acis_empty_bit_get" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    result = (BITCODE_B) ((arg1)->acis_empty_bit);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity__3DSOLID) {
  {
    int argvi = 0;
    struct _dwg_entity_3DSOLID *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity__3DSOLID();");
    }
    result = (struct _dwg_entity_3DSOLID *)calloc(1, sizeof(struct _dwg_entity_3DSOLID));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_3DSOLID, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity__3DSOLID) {
  {
    struct _dwg_entity_3DSOLID *arg1 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity__3DSOLID(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity__3DSOLID" "', argument " "1"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg1 = (struct _dwg_entity_3DSOLID *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_RAY_parent_set) {
  {
    struct _dwg_entity_RAY *arg1 = (struct _dwg_entity_RAY *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_RAY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_RAY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_RAY_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_RAY *""'"); 
    }
    arg1 = (struct _dwg_entity_RAY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_RAY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_RAY_parent_get) {
  {
    struct _dwg_entity_RAY *arg1 = (struct _dwg_entity_RAY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_RAY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_RAY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_RAY_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_RAY *""'"); 
    }
    arg1 = (struct _dwg_entity_RAY *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_RAY_point_set) {
  {
    struct _dwg_entity_RAY *arg1 = (struct _dwg_entity_RAY *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_RAY_point_set(self,point);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_RAY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_RAY_point_set" "', argument " "1"" of type '" "struct _dwg_entity_RAY *""'"); 
    }
    arg1 = (struct _dwg_entity_RAY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_RAY_point_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->point = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_RAY_point_get) {
  {
    struct _dwg_entity_RAY *arg1 = (struct _dwg_entity_RAY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_RAY_point_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_RAY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_RAY_point_get" "', argument " "1"" of type '" "struct _dwg_entity_RAY *""'"); 
    }
    arg1 = (struct _dwg_entity_RAY *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->point);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_RAY_vector_set) {
  {
    struct _dwg_entity_RAY *arg1 = (struct _dwg_entity_RAY *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_RAY_vector_set(self,vector);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_RAY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_RAY_vector_set" "', argument " "1"" of type '" "struct _dwg_entity_RAY *""'"); 
    }
    arg1 = (struct _dwg_entity_RAY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_RAY_vector_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->vector = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_RAY_vector_get) {
  {
    struct _dwg_entity_RAY *arg1 = (struct _dwg_entity_RAY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_RAY_vector_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_RAY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_RAY_vector_get" "', argument " "1"" of type '" "struct _dwg_entity_RAY *""'"); 
    }
    arg1 = (struct _dwg_entity_RAY *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->vector);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_RAY) {
  {
    int argvi = 0;
    struct _dwg_entity_RAY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_RAY();");
    }
    result = (struct _dwg_entity_RAY *)calloc(1, sizeof(struct _dwg_entity_RAY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_RAY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_RAY) {
  {
    struct _dwg_entity_RAY *arg1 = (struct _dwg_entity_RAY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_RAY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_RAY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_RAY" "', argument " "1"" of type '" "struct _dwg_entity_RAY *""'"); 
    }
    arg1 = (struct _dwg_entity_RAY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DICTIONARY_parent_set) {
  {
    struct _dwg_object_DICTIONARY *arg1 = (struct _dwg_object_DICTIONARY *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DICTIONARY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DICTIONARY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DICTIONARY_parent_set" "', argument " "1"" of type '" "struct _dwg_object_DICTIONARY *""'"); 
    }
    arg1 = (struct _dwg_object_DICTIONARY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DICTIONARY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DICTIONARY_parent_get) {
  {
    struct _dwg_object_DICTIONARY *arg1 = (struct _dwg_object_DICTIONARY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DICTIONARY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DICTIONARY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DICTIONARY_parent_get" "', argument " "1"" of type '" "struct _dwg_object_DICTIONARY *""'"); 
    }
    arg1 = (struct _dwg_object_DICTIONARY *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DICTIONARY_numitems_set) {
  {
    struct _dwg_object_DICTIONARY *arg1 = (struct _dwg_object_DICTIONARY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DICTIONARY_numitems_set(self,numitems);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DICTIONARY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DICTIONARY_numitems_set" "', argument " "1"" of type '" "struct _dwg_object_DICTIONARY *""'"); 
    }
    arg1 = (struct _dwg_object_DICTIONARY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DICTIONARY_numitems_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->numitems = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DICTIONARY_numitems_get) {
  {
    struct _dwg_object_DICTIONARY *arg1 = (struct _dwg_object_DICTIONARY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DICTIONARY_numitems_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DICTIONARY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DICTIONARY_numitems_get" "', argument " "1"" of type '" "struct _dwg_object_DICTIONARY *""'"); 
    }
    arg1 = (struct _dwg_object_DICTIONARY *)(argp1);
    result = (BITCODE_BL) ((arg1)->numitems);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DICTIONARY_is_hardowner_set) {
  {
    struct _dwg_object_DICTIONARY *arg1 = (struct _dwg_object_DICTIONARY *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DICTIONARY_is_hardowner_set(self,is_hardowner);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DICTIONARY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DICTIONARY_is_hardowner_set" "', argument " "1"" of type '" "struct _dwg_object_DICTIONARY *""'"); 
    }
    arg1 = (struct _dwg_object_DICTIONARY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DICTIONARY_is_hardowner_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->is_hardowner = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DICTIONARY_is_hardowner_get) {
  {
    struct _dwg_object_DICTIONARY *arg1 = (struct _dwg_object_DICTIONARY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DICTIONARY_is_hardowner_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DICTIONARY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DICTIONARY_is_hardowner_get" "', argument " "1"" of type '" "struct _dwg_object_DICTIONARY *""'"); 
    }
    arg1 = (struct _dwg_object_DICTIONARY *)(argp1);
    result = (BITCODE_RC) ((arg1)->is_hardowner);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DICTIONARY_cloning_set) {
  {
    struct _dwg_object_DICTIONARY *arg1 = (struct _dwg_object_DICTIONARY *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DICTIONARY_cloning_set(self,cloning);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DICTIONARY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DICTIONARY_cloning_set" "', argument " "1"" of type '" "struct _dwg_object_DICTIONARY *""'"); 
    }
    arg1 = (struct _dwg_object_DICTIONARY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DICTIONARY_cloning_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->cloning = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DICTIONARY_cloning_get) {
  {
    struct _dwg_object_DICTIONARY *arg1 = (struct _dwg_object_DICTIONARY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DICTIONARY_cloning_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DICTIONARY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DICTIONARY_cloning_get" "', argument " "1"" of type '" "struct _dwg_object_DICTIONARY *""'"); 
    }
    arg1 = (struct _dwg_object_DICTIONARY *)(argp1);
    result = (BITCODE_BS) ((arg1)->cloning);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DICTIONARY_texts_set) {
  {
    struct _dwg_object_DICTIONARY *arg1 = (struct _dwg_object_DICTIONARY *) 0 ;
    BITCODE_TV *arg2 = (BITCODE_TV *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DICTIONARY_texts_set(self,texts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DICTIONARY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DICTIONARY_texts_set" "', argument " "1"" of type '" "struct _dwg_object_DICTIONARY *""'"); 
    }
    arg1 = (struct _dwg_object_DICTIONARY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DICTIONARY_texts_set" "', argument " "2"" of type '" "BITCODE_TV *""'"); 
    }
    arg2 = (BITCODE_TV *)(argp2);
    if (arg1) (arg1)->texts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DICTIONARY_texts_get) {
  {
    struct _dwg_object_DICTIONARY *arg1 = (struct _dwg_object_DICTIONARY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DICTIONARY_texts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DICTIONARY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DICTIONARY_texts_get" "', argument " "1"" of type '" "struct _dwg_object_DICTIONARY *""'"); 
    }
    arg1 = (struct _dwg_object_DICTIONARY *)(argp1);
    result = (BITCODE_TV *) ((arg1)->texts);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DICTIONARY_itemhandles_set) {
  {
    struct _dwg_object_DICTIONARY *arg1 = (struct _dwg_object_DICTIONARY *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DICTIONARY_itemhandles_set(self,itemhandles);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DICTIONARY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DICTIONARY_itemhandles_set" "', argument " "1"" of type '" "struct _dwg_object_DICTIONARY *""'"); 
    }
    arg1 = (struct _dwg_object_DICTIONARY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DICTIONARY_itemhandles_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->itemhandles = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DICTIONARY_itemhandles_get) {
  {
    struct _dwg_object_DICTIONARY *arg1 = (struct _dwg_object_DICTIONARY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DICTIONARY_itemhandles_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DICTIONARY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DICTIONARY_itemhandles_get" "', argument " "1"" of type '" "struct _dwg_object_DICTIONARY *""'"); 
    }
    arg1 = (struct _dwg_object_DICTIONARY *)(argp1);
    result = (BITCODE_H *) ((arg1)->itemhandles);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_DICTIONARY) {
  {
    int argvi = 0;
    struct _dwg_object_DICTIONARY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_DICTIONARY();");
    }
    result = (struct _dwg_object_DICTIONARY *)calloc(1, sizeof(struct _dwg_object_DICTIONARY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DICTIONARY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_DICTIONARY) {
  {
    struct _dwg_object_DICTIONARY *arg1 = (struct _dwg_object_DICTIONARY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_DICTIONARY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DICTIONARY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_DICTIONARY" "', argument " "1"" of type '" "struct _dwg_object_DICTIONARY *""'"); 
    }
    arg1 = (struct _dwg_object_DICTIONARY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DICTIONARYWDFLT_parent_set) {
  {
    struct _dwg_object_DICTIONARYWDFLT *arg1 = (struct _dwg_object_DICTIONARYWDFLT *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DICTIONARYWDFLT_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DICTIONARYWDFLT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DICTIONARYWDFLT_parent_set" "', argument " "1"" of type '" "struct _dwg_object_DICTIONARYWDFLT *""'"); 
    }
    arg1 = (struct _dwg_object_DICTIONARYWDFLT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DICTIONARYWDFLT_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DICTIONARYWDFLT_parent_get) {
  {
    struct _dwg_object_DICTIONARYWDFLT *arg1 = (struct _dwg_object_DICTIONARYWDFLT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DICTIONARYWDFLT_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DICTIONARYWDFLT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DICTIONARYWDFLT_parent_get" "', argument " "1"" of type '" "struct _dwg_object_DICTIONARYWDFLT *""'"); 
    }
    arg1 = (struct _dwg_object_DICTIONARYWDFLT *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DICTIONARYWDFLT_numitems_set) {
  {
    struct _dwg_object_DICTIONARYWDFLT *arg1 = (struct _dwg_object_DICTIONARYWDFLT *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DICTIONARYWDFLT_numitems_set(self,numitems);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DICTIONARYWDFLT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DICTIONARYWDFLT_numitems_set" "', argument " "1"" of type '" "struct _dwg_object_DICTIONARYWDFLT *""'"); 
    }
    arg1 = (struct _dwg_object_DICTIONARYWDFLT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DICTIONARYWDFLT_numitems_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->numitems = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DICTIONARYWDFLT_numitems_get) {
  {
    struct _dwg_object_DICTIONARYWDFLT *arg1 = (struct _dwg_object_DICTIONARYWDFLT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DICTIONARYWDFLT_numitems_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DICTIONARYWDFLT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DICTIONARYWDFLT_numitems_get" "', argument " "1"" of type '" "struct _dwg_object_DICTIONARYWDFLT *""'"); 
    }
    arg1 = (struct _dwg_object_DICTIONARYWDFLT *)(argp1);
    result = (BITCODE_BL) ((arg1)->numitems);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DICTIONARYWDFLT_is_hardowner_set) {
  {
    struct _dwg_object_DICTIONARYWDFLT *arg1 = (struct _dwg_object_DICTIONARYWDFLT *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DICTIONARYWDFLT_is_hardowner_set(self,is_hardowner);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DICTIONARYWDFLT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DICTIONARYWDFLT_is_hardowner_set" "', argument " "1"" of type '" "struct _dwg_object_DICTIONARYWDFLT *""'"); 
    }
    arg1 = (struct _dwg_object_DICTIONARYWDFLT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DICTIONARYWDFLT_is_hardowner_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->is_hardowner = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DICTIONARYWDFLT_is_hardowner_get) {
  {
    struct _dwg_object_DICTIONARYWDFLT *arg1 = (struct _dwg_object_DICTIONARYWDFLT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DICTIONARYWDFLT_is_hardowner_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DICTIONARYWDFLT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DICTIONARYWDFLT_is_hardowner_get" "', argument " "1"" of type '" "struct _dwg_object_DICTIONARYWDFLT *""'"); 
    }
    arg1 = (struct _dwg_object_DICTIONARYWDFLT *)(argp1);
    result = (BITCODE_RC) ((arg1)->is_hardowner);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DICTIONARYWDFLT_cloning_set) {
  {
    struct _dwg_object_DICTIONARYWDFLT *arg1 = (struct _dwg_object_DICTIONARYWDFLT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DICTIONARYWDFLT_cloning_set(self,cloning);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DICTIONARYWDFLT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DICTIONARYWDFLT_cloning_set" "', argument " "1"" of type '" "struct _dwg_object_DICTIONARYWDFLT *""'"); 
    }
    arg1 = (struct _dwg_object_DICTIONARYWDFLT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DICTIONARYWDFLT_cloning_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->cloning = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DICTIONARYWDFLT_cloning_get) {
  {
    struct _dwg_object_DICTIONARYWDFLT *arg1 = (struct _dwg_object_DICTIONARYWDFLT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DICTIONARYWDFLT_cloning_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DICTIONARYWDFLT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DICTIONARYWDFLT_cloning_get" "', argument " "1"" of type '" "struct _dwg_object_DICTIONARYWDFLT *""'"); 
    }
    arg1 = (struct _dwg_object_DICTIONARYWDFLT *)(argp1);
    result = (BITCODE_BS) ((arg1)->cloning);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DICTIONARYWDFLT_texts_set) {
  {
    struct _dwg_object_DICTIONARYWDFLT *arg1 = (struct _dwg_object_DICTIONARYWDFLT *) 0 ;
    BITCODE_TV *arg2 = (BITCODE_TV *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DICTIONARYWDFLT_texts_set(self,texts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DICTIONARYWDFLT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DICTIONARYWDFLT_texts_set" "', argument " "1"" of type '" "struct _dwg_object_DICTIONARYWDFLT *""'"); 
    }
    arg1 = (struct _dwg_object_DICTIONARYWDFLT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DICTIONARYWDFLT_texts_set" "', argument " "2"" of type '" "BITCODE_TV *""'"); 
    }
    arg2 = (BITCODE_TV *)(argp2);
    if (arg1) (arg1)->texts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DICTIONARYWDFLT_texts_get) {
  {
    struct _dwg_object_DICTIONARYWDFLT *arg1 = (struct _dwg_object_DICTIONARYWDFLT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DICTIONARYWDFLT_texts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DICTIONARYWDFLT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DICTIONARYWDFLT_texts_get" "', argument " "1"" of type '" "struct _dwg_object_DICTIONARYWDFLT *""'"); 
    }
    arg1 = (struct _dwg_object_DICTIONARYWDFLT *)(argp1);
    result = (BITCODE_TV *) ((arg1)->texts);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DICTIONARYWDFLT_itemhandles_set) {
  {
    struct _dwg_object_DICTIONARYWDFLT *arg1 = (struct _dwg_object_DICTIONARYWDFLT *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DICTIONARYWDFLT_itemhandles_set(self,itemhandles);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DICTIONARYWDFLT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DICTIONARYWDFLT_itemhandles_set" "', argument " "1"" of type '" "struct _dwg_object_DICTIONARYWDFLT *""'"); 
    }
    arg1 = (struct _dwg_object_DICTIONARYWDFLT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DICTIONARYWDFLT_itemhandles_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->itemhandles = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DICTIONARYWDFLT_itemhandles_get) {
  {
    struct _dwg_object_DICTIONARYWDFLT *arg1 = (struct _dwg_object_DICTIONARYWDFLT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DICTIONARYWDFLT_itemhandles_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DICTIONARYWDFLT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DICTIONARYWDFLT_itemhandles_get" "', argument " "1"" of type '" "struct _dwg_object_DICTIONARYWDFLT *""'"); 
    }
    arg1 = (struct _dwg_object_DICTIONARYWDFLT *)(argp1);
    result = (BITCODE_H *) ((arg1)->itemhandles);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DICTIONARYWDFLT_defaultid_set) {
  {
    struct _dwg_object_DICTIONARYWDFLT *arg1 = (struct _dwg_object_DICTIONARYWDFLT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DICTIONARYWDFLT_defaultid_set(self,defaultid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DICTIONARYWDFLT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DICTIONARYWDFLT_defaultid_set" "', argument " "1"" of type '" "struct _dwg_object_DICTIONARYWDFLT *""'"); 
    }
    arg1 = (struct _dwg_object_DICTIONARYWDFLT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DICTIONARYWDFLT_defaultid_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->defaultid = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DICTIONARYWDFLT_defaultid_get) {
  {
    struct _dwg_object_DICTIONARYWDFLT *arg1 = (struct _dwg_object_DICTIONARYWDFLT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DICTIONARYWDFLT_defaultid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DICTIONARYWDFLT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DICTIONARYWDFLT_defaultid_get" "', argument " "1"" of type '" "struct _dwg_object_DICTIONARYWDFLT *""'"); 
    }
    arg1 = (struct _dwg_object_DICTIONARYWDFLT *)(argp1);
    result = (BITCODE_H) ((arg1)->defaultid);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_DICTIONARYWDFLT) {
  {
    int argvi = 0;
    struct _dwg_object_DICTIONARYWDFLT *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_DICTIONARYWDFLT();");
    }
    result = (struct _dwg_object_DICTIONARYWDFLT *)calloc(1, sizeof(struct _dwg_object_DICTIONARYWDFLT));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DICTIONARYWDFLT, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_DICTIONARYWDFLT) {
  {
    struct _dwg_object_DICTIONARYWDFLT *arg1 = (struct _dwg_object_DICTIONARYWDFLT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_DICTIONARYWDFLT(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DICTIONARYWDFLT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_DICTIONARYWDFLT" "', argument " "1"" of type '" "struct _dwg_object_DICTIONARYWDFLT *""'"); 
    }
    arg1 = (struct _dwg_object_DICTIONARYWDFLT *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_OLEFRAME_parent_set) {
  {
    struct _dwg_entity_OLEFRAME *arg1 = (struct _dwg_entity_OLEFRAME *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_OLEFRAME_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_OLEFRAME, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_OLEFRAME_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_OLEFRAME *""'"); 
    }
    arg1 = (struct _dwg_entity_OLEFRAME *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_OLEFRAME_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_OLEFRAME_parent_get) {
  {
    struct _dwg_entity_OLEFRAME *arg1 = (struct _dwg_entity_OLEFRAME *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_OLEFRAME_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_OLEFRAME, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_OLEFRAME_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_OLEFRAME *""'"); 
    }
    arg1 = (struct _dwg_entity_OLEFRAME *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_OLEFRAME_flag_set) {
  {
    struct _dwg_entity_OLEFRAME *arg1 = (struct _dwg_entity_OLEFRAME *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_OLEFRAME_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_OLEFRAME, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_OLEFRAME_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_OLEFRAME *""'"); 
    }
    arg1 = (struct _dwg_entity_OLEFRAME *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_OLEFRAME_flag_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_OLEFRAME_flag_get) {
  {
    struct _dwg_entity_OLEFRAME *arg1 = (struct _dwg_entity_OLEFRAME *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_OLEFRAME_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_OLEFRAME, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_OLEFRAME_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_OLEFRAME *""'"); 
    }
    arg1 = (struct _dwg_entity_OLEFRAME *)(argp1);
    result = (BITCODE_BS) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_OLEFRAME_mode_set) {
  {
    struct _dwg_entity_OLEFRAME *arg1 = (struct _dwg_entity_OLEFRAME *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_OLEFRAME_mode_set(self,mode);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_OLEFRAME, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_OLEFRAME_mode_set" "', argument " "1"" of type '" "struct _dwg_entity_OLEFRAME *""'"); 
    }
    arg1 = (struct _dwg_entity_OLEFRAME *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_OLEFRAME_mode_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->mode = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_OLEFRAME_mode_get) {
  {
    struct _dwg_entity_OLEFRAME *arg1 = (struct _dwg_entity_OLEFRAME *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_OLEFRAME_mode_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_OLEFRAME, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_OLEFRAME_mode_get" "', argument " "1"" of type '" "struct _dwg_entity_OLEFRAME *""'"); 
    }
    arg1 = (struct _dwg_entity_OLEFRAME *)(argp1);
    result = (BITCODE_BS) ((arg1)->mode);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_OLEFRAME_data_size_set) {
  {
    struct _dwg_entity_OLEFRAME *arg1 = (struct _dwg_entity_OLEFRAME *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_OLEFRAME_data_size_set(self,data_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_OLEFRAME, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_OLEFRAME_data_size_set" "', argument " "1"" of type '" "struct _dwg_entity_OLEFRAME *""'"); 
    }
    arg1 = (struct _dwg_entity_OLEFRAME *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_OLEFRAME_data_size_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->data_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_OLEFRAME_data_size_get) {
  {
    struct _dwg_entity_OLEFRAME *arg1 = (struct _dwg_entity_OLEFRAME *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_OLEFRAME_data_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_OLEFRAME, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_OLEFRAME_data_size_get" "', argument " "1"" of type '" "struct _dwg_entity_OLEFRAME *""'"); 
    }
    arg1 = (struct _dwg_entity_OLEFRAME *)(argp1);
    result = (BITCODE_BL) ((arg1)->data_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_OLEFRAME_data_set) {
  {
    struct _dwg_entity_OLEFRAME *arg1 = (struct _dwg_entity_OLEFRAME *) 0 ;
    BITCODE_TF arg2 = (BITCODE_TF) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_OLEFRAME_data_set(self,data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_OLEFRAME, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_OLEFRAME_data_set" "', argument " "1"" of type '" "struct _dwg_entity_OLEFRAME *""'"); 
    }
    arg1 = (struct _dwg_entity_OLEFRAME *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_OLEFRAME_data_set" "', argument " "2"" of type '" "BITCODE_TF""'"); 
    }
    arg2 = (BITCODE_TF)(argp2);
    if (arg1) (arg1)->data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_OLEFRAME_data_get) {
  {
    struct _dwg_entity_OLEFRAME *arg1 = (struct _dwg_entity_OLEFRAME *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TF result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_OLEFRAME_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_OLEFRAME, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_OLEFRAME_data_get" "', argument " "1"" of type '" "struct _dwg_entity_OLEFRAME *""'"); 
    }
    arg1 = (struct _dwg_entity_OLEFRAME *)(argp1);
    result = (BITCODE_TF) ((arg1)->data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_OLEFRAME) {
  {
    int argvi = 0;
    struct _dwg_entity_OLEFRAME *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_OLEFRAME();");
    }
    result = (struct _dwg_entity_OLEFRAME *)calloc(1, sizeof(struct _dwg_entity_OLEFRAME));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_OLEFRAME, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_OLEFRAME) {
  {
    struct _dwg_entity_OLEFRAME *arg1 = (struct _dwg_entity_OLEFRAME *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_OLEFRAME(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_OLEFRAME, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_OLEFRAME" "', argument " "1"" of type '" "struct _dwg_entity_OLEFRAME *""'"); 
    }
    arg1 = (struct _dwg_entity_OLEFRAME *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_parent_set) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MTEXT_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_parent_get) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_ins_pt_set) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_ins_pt_set(self,ins_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_ins_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MTEXT_ins_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ins_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_ins_pt_get) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_ins_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_ins_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ins_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_extrusion_set) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_extrusion_set" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MTEXT_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_extrusion_get) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_extrusion_get" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_x_axis_dir_set) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_x_axis_dir_set(self,x_axis_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_x_axis_dir_set" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MTEXT_x_axis_dir_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->x_axis_dir = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_x_axis_dir_get) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_x_axis_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_x_axis_dir_get" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->x_axis_dir);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_rect_height_set) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_rect_height_set(self,rect_height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_rect_height_set" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MTEXT_rect_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->rect_height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_rect_height_get) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_rect_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_rect_height_get" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    result = (BITCODE_BD) ((arg1)->rect_height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_rect_width_set) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_rect_width_set(self,rect_width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_rect_width_set" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MTEXT_rect_width_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->rect_width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_rect_width_get) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_rect_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_rect_width_get" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    result = (BITCODE_BD) ((arg1)->rect_width);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_text_height_set) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_text_height_set(self,text_height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_text_height_set" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MTEXT_text_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->text_height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_text_height_get) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_text_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_text_height_get" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    result = (BITCODE_BD) ((arg1)->text_height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_attachment_set) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_attachment_set(self,attachment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_attachment_set" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MTEXT_attachment_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->attachment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_attachment_get) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_attachment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_attachment_get" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    result = (BITCODE_BS) ((arg1)->attachment);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_flow_dir_set) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_flow_dir_set(self,flow_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_flow_dir_set" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MTEXT_flow_dir_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->flow_dir = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_flow_dir_get) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_flow_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_flow_dir_get" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    result = (BITCODE_BS) ((arg1)->flow_dir);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_extents_width_set) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_extents_width_set(self,extents_width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_extents_width_set" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MTEXT_extents_width_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->extents_width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_extents_width_get) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_extents_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_extents_width_get" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    result = (BITCODE_BD) ((arg1)->extents_width);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_extents_height_set) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_extents_height_set(self,extents_height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_extents_height_set" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MTEXT_extents_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->extents_height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_extents_height_get) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_extents_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_extents_height_get" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    result = (BITCODE_BD) ((arg1)->extents_height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_text_set) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_text_set(self,text);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_text_set" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MTEXT_text_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->text);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->text = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->text = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_text_get) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_text_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_text_get" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    result = (BITCODE_TV) ((arg1)->text);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_style_set) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_style_set(self,style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_style_set" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MTEXT_style_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_style_get) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_style_get" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    result = (BITCODE_H) ((arg1)->style);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_linespace_style_set) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_linespace_style_set(self,linespace_style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_linespace_style_set" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MTEXT_linespace_style_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->linespace_style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_linespace_style_get) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_linespace_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_linespace_style_get" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    result = (BITCODE_BS) ((arg1)->linespace_style);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_linespace_factor_set) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_linespace_factor_set(self,linespace_factor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_linespace_factor_set" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MTEXT_linespace_factor_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->linespace_factor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_linespace_factor_get) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_linespace_factor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_linespace_factor_get" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    result = (BITCODE_BD) ((arg1)->linespace_factor);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_unknown_b0_set) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_unknown_b0_set(self,unknown_b0);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_unknown_b0_set" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MTEXT_unknown_b0_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown_b0 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_unknown_b0_get) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_unknown_b0_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_unknown_b0_get" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown_b0);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_bg_fill_flag_set) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_bg_fill_flag_set(self,bg_fill_flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_bg_fill_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MTEXT_bg_fill_flag_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bg_fill_flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_bg_fill_flag_get) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_bg_fill_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_bg_fill_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    result = (BITCODE_BL) ((arg1)->bg_fill_flag);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_bg_fill_scale_set) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_bg_fill_scale_set(self,bg_fill_scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_bg_fill_scale_set" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MTEXT_bg_fill_scale_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bg_fill_scale = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_bg_fill_scale_get) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_bg_fill_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_bg_fill_scale_get" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    result = (BITCODE_BL) ((arg1)->bg_fill_scale);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_bg_fill_color_set) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_bg_fill_color_set(self,bg_fill_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_bg_fill_color_set" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MTEXT_bg_fill_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->bg_fill_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_bg_fill_color_get) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_bg_fill_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_bg_fill_color_get" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->bg_fill_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_bg_fill_trans_set) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_bg_fill_trans_set(self,bg_fill_trans);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_bg_fill_trans_set" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MTEXT_bg_fill_trans_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bg_fill_trans = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_bg_fill_trans_get) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_bg_fill_trans_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_bg_fill_trans_get" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    result = (BITCODE_BL) ((arg1)->bg_fill_trans);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_is_not_annotative_set) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_is_not_annotative_set(self,is_not_annotative);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_is_not_annotative_set" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MTEXT_is_not_annotative_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_not_annotative = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_is_not_annotative_get) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_is_not_annotative_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_is_not_annotative_get" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    result = (BITCODE_B) ((arg1)->is_not_annotative);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_class_version_set) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_class_version_set" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MTEXT_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_class_version_get) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_class_version_get" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_default_flag_set) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_default_flag_set(self,default_flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_default_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MTEXT_default_flag_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->default_flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_default_flag_get) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_default_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_default_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    result = (BITCODE_B) ((arg1)->default_flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_appid_set) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_appid_set(self,appid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_appid_set" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MTEXT_appid_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->appid = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_appid_get) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_appid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_appid_get" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    result = (BITCODE_H) ((arg1)->appid);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_ignore_attachment_set) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_ignore_attachment_set(self,ignore_attachment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_ignore_attachment_set" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MTEXT_ignore_attachment_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->ignore_attachment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_ignore_attachment_get) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_ignore_attachment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_ignore_attachment_get" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    result = (BITCODE_BL) ((arg1)->ignore_attachment);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_column_type_set) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_column_type_set(self,column_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_column_type_set" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MTEXT_column_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->column_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_column_type_get) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_column_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_column_type_get" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    result = (BITCODE_BS) ((arg1)->column_type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_numfragments_set) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_numfragments_set(self,numfragments);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_numfragments_set" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MTEXT_numfragments_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->numfragments = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_numfragments_get) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_numfragments_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_numfragments_get" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    result = (BITCODE_BL) ((arg1)->numfragments);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_column_width_set) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_column_width_set(self,column_width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_column_width_set" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MTEXT_column_width_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->column_width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_column_width_get) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_column_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_column_width_get" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    result = (BITCODE_BD) ((arg1)->column_width);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_gutter_set) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_gutter_set(self,gutter);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_gutter_set" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MTEXT_gutter_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->gutter = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_gutter_get) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_gutter_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_gutter_get" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    result = (BITCODE_BD) ((arg1)->gutter);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_auto_height_set) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_auto_height_set(self,auto_height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_auto_height_set" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MTEXT_auto_height_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->auto_height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_auto_height_get) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_auto_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_auto_height_get" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    result = (BITCODE_B) ((arg1)->auto_height);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_flow_reversed_set) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_flow_reversed_set(self,flow_reversed);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_flow_reversed_set" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MTEXT_flow_reversed_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->flow_reversed = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_flow_reversed_get) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_flow_reversed_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_flow_reversed_get" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    result = (BITCODE_B) ((arg1)->flow_reversed);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_num_column_heights_set) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_num_column_heights_set(self,num_column_heights);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_num_column_heights_set" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MTEXT_num_column_heights_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_column_heights = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_num_column_heights_get) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_num_column_heights_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_num_column_heights_get" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_column_heights);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_column_heights_set) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_column_heights_set(self,column_heights);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_column_heights_set" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MTEXT_column_heights_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->column_heights = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MTEXT_column_heights_get) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MTEXT_column_heights_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MTEXT_column_heights_get" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    result = (BITCODE_BD *) ((arg1)->column_heights);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_MTEXT) {
  {
    int argvi = 0;
    struct _dwg_entity_MTEXT *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_MTEXT();");
    }
    result = (struct _dwg_entity_MTEXT *)calloc(1, sizeof(struct _dwg_entity_MTEXT));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_MTEXT, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_MTEXT) {
  {
    struct _dwg_entity_MTEXT *arg1 = (struct _dwg_entity_MTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_MTEXT(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MTEXT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_MTEXT" "', argument " "1"" of type '" "struct _dwg_entity_MTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_MTEXT *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_parent_set) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LEADER_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_parent_get) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_unknown_bit_1_set) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_unknown_bit_1_set(self,unknown_bit_1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_unknown_bit_1_set" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LEADER_unknown_bit_1_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown_bit_1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_unknown_bit_1_get) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_unknown_bit_1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_unknown_bit_1_get" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown_bit_1);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_path_type_set) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_path_type_set(self,path_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_path_type_set" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LEADER_path_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->path_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_path_type_get) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_path_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_path_type_get" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    result = (BITCODE_BS) ((arg1)->path_type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_annot_type_set) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_annot_type_set(self,annot_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_annot_type_set" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LEADER_annot_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->annot_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_annot_type_get) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_annot_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_annot_type_get" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    result = (BITCODE_BS) ((arg1)->annot_type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_num_points_set) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_num_points_set(self,num_points);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_num_points_set" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LEADER_num_points_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_points = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_num_points_get) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_num_points_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_num_points_get" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_points);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_points_set) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    BITCODE_3DPOINT *arg2 = (BITCODE_3DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_points_set(self,points);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_points_set" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LEADER_points_set" "', argument " "2"" of type '" "BITCODE_3DPOINT *""'"); 
    }
    arg2 = (BITCODE_3DPOINT *)(argp2);
    if (arg1) (arg1)->points = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_points_get) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_points_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_points_get" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    result = (BITCODE_3DPOINT *) ((arg1)->points);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_origin_set) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    BITCODE_3DPOINT *arg2 = (BITCODE_3DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_origin_set(self,origin);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_origin_set" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LEADER_origin_set" "', argument " "2"" of type '" "BITCODE_3DPOINT *""'"); 
    }
    arg2 = (BITCODE_3DPOINT *)(argp2);
    if (arg1) (arg1)->origin = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_origin_get) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_origin_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_origin_get" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    result = (BITCODE_3DPOINT *)& ((arg1)->origin);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_extrusion_set) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_extrusion_set" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LEADER_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_extrusion_get) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_extrusion_get" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_x_direction_set) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    BITCODE_3DPOINT *arg2 = (BITCODE_3DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_x_direction_set(self,x_direction);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_x_direction_set" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LEADER_x_direction_set" "', argument " "2"" of type '" "BITCODE_3DPOINT *""'"); 
    }
    arg2 = (BITCODE_3DPOINT *)(argp2);
    if (arg1) (arg1)->x_direction = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_x_direction_get) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_x_direction_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_x_direction_get" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    result = (BITCODE_3DPOINT *)& ((arg1)->x_direction);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_inspt_offset_set) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    BITCODE_3DPOINT *arg2 = (BITCODE_3DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_inspt_offset_set(self,inspt_offset);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_inspt_offset_set" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LEADER_inspt_offset_set" "', argument " "2"" of type '" "BITCODE_3DPOINT *""'"); 
    }
    arg2 = (BITCODE_3DPOINT *)(argp2);
    if (arg1) (arg1)->inspt_offset = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_inspt_offset_get) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_inspt_offset_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_inspt_offset_get" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    result = (BITCODE_3DPOINT *)& ((arg1)->inspt_offset);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_endptproj_set) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    BITCODE_3DPOINT *arg2 = (BITCODE_3DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_endptproj_set(self,endptproj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_endptproj_set" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LEADER_endptproj_set" "', argument " "2"" of type '" "BITCODE_3DPOINT *""'"); 
    }
    arg2 = (BITCODE_3DPOINT *)(argp2);
    if (arg1) (arg1)->endptproj = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_endptproj_get) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_endptproj_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_endptproj_get" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    result = (BITCODE_3DPOINT *)& ((arg1)->endptproj);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_dimgap_set) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_dimgap_set(self,dimgap);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_dimgap_set" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LEADER_dimgap_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->dimgap = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_dimgap_get) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_dimgap_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_dimgap_get" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    result = (BITCODE_BD) ((arg1)->dimgap);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_box_height_set) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_box_height_set(self,box_height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_box_height_set" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LEADER_box_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->box_height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_box_height_get) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_box_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_box_height_get" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    result = (BITCODE_BD) ((arg1)->box_height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_box_width_set) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_box_width_set(self,box_width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_box_width_set" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LEADER_box_width_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->box_width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_box_width_get) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_box_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_box_width_get" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    result = (BITCODE_BD) ((arg1)->box_width);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_hookline_dir_set) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_hookline_dir_set(self,hookline_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_hookline_dir_set" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LEADER_hookline_dir_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->hookline_dir = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_hookline_dir_get) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_hookline_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_hookline_dir_get" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    result = (BITCODE_B) ((arg1)->hookline_dir);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_hookline_on_set) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_hookline_on_set(self,hookline_on);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_hookline_on_set" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LEADER_hookline_on_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->hookline_on = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_hookline_on_get) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_hookline_on_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_hookline_on_get" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    result = (BITCODE_B) ((arg1)->hookline_on);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_arrowhead_on_set) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_arrowhead_on_set(self,arrowhead_on);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_arrowhead_on_set" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LEADER_arrowhead_on_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->arrowhead_on = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_arrowhead_on_get) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_arrowhead_on_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_arrowhead_on_get" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    result = (BITCODE_B) ((arg1)->arrowhead_on);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_arrowhead_type_set) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_arrowhead_type_set(self,arrowhead_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_arrowhead_type_set" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LEADER_arrowhead_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->arrowhead_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_arrowhead_type_get) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_arrowhead_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_arrowhead_type_get" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    result = (BITCODE_BS) ((arg1)->arrowhead_type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_dimasz_set) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_dimasz_set(self,dimasz);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_dimasz_set" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LEADER_dimasz_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->dimasz = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_dimasz_get) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_dimasz_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_dimasz_get" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    result = (BITCODE_BD) ((arg1)->dimasz);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_unknown_bit_2_set) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_unknown_bit_2_set(self,unknown_bit_2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_unknown_bit_2_set" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LEADER_unknown_bit_2_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown_bit_2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_unknown_bit_2_get) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_unknown_bit_2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_unknown_bit_2_get" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown_bit_2);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_unknown_bit_3_set) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_unknown_bit_3_set(self,unknown_bit_3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_unknown_bit_3_set" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LEADER_unknown_bit_3_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown_bit_3 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_unknown_bit_3_get) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_unknown_bit_3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_unknown_bit_3_get" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown_bit_3);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_unknown_short_1_set) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_unknown_short_1_set(self,unknown_short_1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_unknown_short_1_set" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LEADER_unknown_short_1_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->unknown_short_1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_unknown_short_1_get) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_unknown_short_1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_unknown_short_1_get" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    result = (BITCODE_BS) ((arg1)->unknown_short_1);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_byblock_color_set) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_byblock_color_set(self,byblock_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_byblock_color_set" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LEADER_byblock_color_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->byblock_color = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_byblock_color_get) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_byblock_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_byblock_color_get" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    result = (BITCODE_BS) ((arg1)->byblock_color);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_unknown_bit_4_set) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_unknown_bit_4_set(self,unknown_bit_4);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_unknown_bit_4_set" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LEADER_unknown_bit_4_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown_bit_4 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_unknown_bit_4_get) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_unknown_bit_4_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_unknown_bit_4_get" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown_bit_4);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_unknown_bit_5_set) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_unknown_bit_5_set(self,unknown_bit_5);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_unknown_bit_5_set" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LEADER_unknown_bit_5_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown_bit_5 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_unknown_bit_5_get) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_unknown_bit_5_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_unknown_bit_5_get" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown_bit_5);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_associated_annotation_set) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_associated_annotation_set(self,associated_annotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_associated_annotation_set" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LEADER_associated_annotation_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->associated_annotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_associated_annotation_get) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_associated_annotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_associated_annotation_get" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    result = (BITCODE_H) ((arg1)->associated_annotation);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_dimstyle_set) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_dimstyle_set(self,dimstyle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_dimstyle_set" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LEADER_dimstyle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->dimstyle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LEADER_dimstyle_get) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LEADER_dimstyle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LEADER_dimstyle_get" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    result = (BITCODE_H) ((arg1)->dimstyle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_LEADER) {
  {
    int argvi = 0;
    struct _dwg_entity_LEADER *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_LEADER();");
    }
    result = (struct _dwg_entity_LEADER *)calloc(1, sizeof(struct _dwg_entity_LEADER));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_LEADER, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_LEADER) {
  {
    struct _dwg_entity_LEADER *arg1 = (struct _dwg_entity_LEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_LEADER(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LEADER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_LEADER" "', argument " "1"" of type '" "struct _dwg_entity_LEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_LEADER *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TOLERANCE_parent_set) {
  {
    struct _dwg_entity_TOLERANCE *arg1 = (struct _dwg_entity_TOLERANCE *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TOLERANCE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TOLERANCE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TOLERANCE_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_TOLERANCE *""'"); 
    }
    arg1 = (struct _dwg_entity_TOLERANCE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TOLERANCE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TOLERANCE_parent_get) {
  {
    struct _dwg_entity_TOLERANCE *arg1 = (struct _dwg_entity_TOLERANCE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TOLERANCE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TOLERANCE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TOLERANCE_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_TOLERANCE *""'"); 
    }
    arg1 = (struct _dwg_entity_TOLERANCE *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TOLERANCE_unknown_short_set) {
  {
    struct _dwg_entity_TOLERANCE *arg1 = (struct _dwg_entity_TOLERANCE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TOLERANCE_unknown_short_set(self,unknown_short);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TOLERANCE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TOLERANCE_unknown_short_set" "', argument " "1"" of type '" "struct _dwg_entity_TOLERANCE *""'"); 
    }
    arg1 = (struct _dwg_entity_TOLERANCE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TOLERANCE_unknown_short_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->unknown_short = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TOLERANCE_unknown_short_get) {
  {
    struct _dwg_entity_TOLERANCE *arg1 = (struct _dwg_entity_TOLERANCE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TOLERANCE_unknown_short_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TOLERANCE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TOLERANCE_unknown_short_get" "', argument " "1"" of type '" "struct _dwg_entity_TOLERANCE *""'"); 
    }
    arg1 = (struct _dwg_entity_TOLERANCE *)(argp1);
    result = (BITCODE_BS) ((arg1)->unknown_short);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TOLERANCE_height_set) {
  {
    struct _dwg_entity_TOLERANCE *arg1 = (struct _dwg_entity_TOLERANCE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TOLERANCE_height_set(self,height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TOLERANCE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TOLERANCE_height_set" "', argument " "1"" of type '" "struct _dwg_entity_TOLERANCE *""'"); 
    }
    arg1 = (struct _dwg_entity_TOLERANCE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TOLERANCE_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TOLERANCE_height_get) {
  {
    struct _dwg_entity_TOLERANCE *arg1 = (struct _dwg_entity_TOLERANCE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TOLERANCE_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TOLERANCE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TOLERANCE_height_get" "', argument " "1"" of type '" "struct _dwg_entity_TOLERANCE *""'"); 
    }
    arg1 = (struct _dwg_entity_TOLERANCE *)(argp1);
    result = (BITCODE_BD) ((arg1)->height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TOLERANCE_dimgap_set) {
  {
    struct _dwg_entity_TOLERANCE *arg1 = (struct _dwg_entity_TOLERANCE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TOLERANCE_dimgap_set(self,dimgap);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TOLERANCE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TOLERANCE_dimgap_set" "', argument " "1"" of type '" "struct _dwg_entity_TOLERANCE *""'"); 
    }
    arg1 = (struct _dwg_entity_TOLERANCE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TOLERANCE_dimgap_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->dimgap = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TOLERANCE_dimgap_get) {
  {
    struct _dwg_entity_TOLERANCE *arg1 = (struct _dwg_entity_TOLERANCE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TOLERANCE_dimgap_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TOLERANCE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TOLERANCE_dimgap_get" "', argument " "1"" of type '" "struct _dwg_entity_TOLERANCE *""'"); 
    }
    arg1 = (struct _dwg_entity_TOLERANCE *)(argp1);
    result = (BITCODE_BD) ((arg1)->dimgap);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TOLERANCE_ins_pt_set) {
  {
    struct _dwg_entity_TOLERANCE *arg1 = (struct _dwg_entity_TOLERANCE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TOLERANCE_ins_pt_set(self,ins_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TOLERANCE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TOLERANCE_ins_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_TOLERANCE *""'"); 
    }
    arg1 = (struct _dwg_entity_TOLERANCE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TOLERANCE_ins_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ins_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TOLERANCE_ins_pt_get) {
  {
    struct _dwg_entity_TOLERANCE *arg1 = (struct _dwg_entity_TOLERANCE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TOLERANCE_ins_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TOLERANCE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TOLERANCE_ins_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_TOLERANCE *""'"); 
    }
    arg1 = (struct _dwg_entity_TOLERANCE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ins_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TOLERANCE_x_direction_set) {
  {
    struct _dwg_entity_TOLERANCE *arg1 = (struct _dwg_entity_TOLERANCE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TOLERANCE_x_direction_set(self,x_direction);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TOLERANCE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TOLERANCE_x_direction_set" "', argument " "1"" of type '" "struct _dwg_entity_TOLERANCE *""'"); 
    }
    arg1 = (struct _dwg_entity_TOLERANCE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TOLERANCE_x_direction_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->x_direction = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TOLERANCE_x_direction_get) {
  {
    struct _dwg_entity_TOLERANCE *arg1 = (struct _dwg_entity_TOLERANCE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TOLERANCE_x_direction_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TOLERANCE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TOLERANCE_x_direction_get" "', argument " "1"" of type '" "struct _dwg_entity_TOLERANCE *""'"); 
    }
    arg1 = (struct _dwg_entity_TOLERANCE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->x_direction);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TOLERANCE_extrusion_set) {
  {
    struct _dwg_entity_TOLERANCE *arg1 = (struct _dwg_entity_TOLERANCE *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TOLERANCE_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TOLERANCE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TOLERANCE_extrusion_set" "', argument " "1"" of type '" "struct _dwg_entity_TOLERANCE *""'"); 
    }
    arg1 = (struct _dwg_entity_TOLERANCE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TOLERANCE_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TOLERANCE_extrusion_get) {
  {
    struct _dwg_entity_TOLERANCE *arg1 = (struct _dwg_entity_TOLERANCE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TOLERANCE_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TOLERANCE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TOLERANCE_extrusion_get" "', argument " "1"" of type '" "struct _dwg_entity_TOLERANCE *""'"); 
    }
    arg1 = (struct _dwg_entity_TOLERANCE *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TOLERANCE_text_value_set) {
  {
    struct _dwg_entity_TOLERANCE *arg1 = (struct _dwg_entity_TOLERANCE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TOLERANCE_text_value_set(self,text_value);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TOLERANCE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TOLERANCE_text_value_set" "', argument " "1"" of type '" "struct _dwg_entity_TOLERANCE *""'"); 
    }
    arg1 = (struct _dwg_entity_TOLERANCE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TOLERANCE_text_value_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->text_value);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->text_value = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->text_value = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TOLERANCE_text_value_get) {
  {
    struct _dwg_entity_TOLERANCE *arg1 = (struct _dwg_entity_TOLERANCE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TOLERANCE_text_value_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TOLERANCE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TOLERANCE_text_value_get" "', argument " "1"" of type '" "struct _dwg_entity_TOLERANCE *""'"); 
    }
    arg1 = (struct _dwg_entity_TOLERANCE *)(argp1);
    result = (BITCODE_TV) ((arg1)->text_value);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TOLERANCE_dimstyle_set) {
  {
    struct _dwg_entity_TOLERANCE *arg1 = (struct _dwg_entity_TOLERANCE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TOLERANCE_dimstyle_set(self,dimstyle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TOLERANCE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TOLERANCE_dimstyle_set" "', argument " "1"" of type '" "struct _dwg_entity_TOLERANCE *""'"); 
    }
    arg1 = (struct _dwg_entity_TOLERANCE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TOLERANCE_dimstyle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->dimstyle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TOLERANCE_dimstyle_get) {
  {
    struct _dwg_entity_TOLERANCE *arg1 = (struct _dwg_entity_TOLERANCE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TOLERANCE_dimstyle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TOLERANCE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TOLERANCE_dimstyle_get" "', argument " "1"" of type '" "struct _dwg_entity_TOLERANCE *""'"); 
    }
    arg1 = (struct _dwg_entity_TOLERANCE *)(argp1);
    result = (BITCODE_H) ((arg1)->dimstyle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_TOLERANCE) {
  {
    int argvi = 0;
    struct _dwg_entity_TOLERANCE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_TOLERANCE();");
    }
    result = (struct _dwg_entity_TOLERANCE *)calloc(1, sizeof(struct _dwg_entity_TOLERANCE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_TOLERANCE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_TOLERANCE) {
  {
    struct _dwg_entity_TOLERANCE *arg1 = (struct _dwg_entity_TOLERANCE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_TOLERANCE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TOLERANCE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_TOLERANCE" "', argument " "1"" of type '" "struct _dwg_entity_TOLERANCE *""'"); 
    }
    arg1 = (struct _dwg_entity_TOLERANCE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLINE_line_parent_set) {
  {
    struct _dwg_MLINE_line *arg1 = (struct _dwg_MLINE_line *) 0 ;
    struct _dwg_MLINE_vertex *arg2 = (struct _dwg_MLINE_vertex *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLINE_line_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINE_line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLINE_line_parent_set" "', argument " "1"" of type '" "struct _dwg_MLINE_line *""'"); 
    }
    arg1 = (struct _dwg_MLINE_line *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_MLINE_vertex, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MLINE_line_parent_set" "', argument " "2"" of type '" "struct _dwg_MLINE_vertex *""'"); 
    }
    arg2 = (struct _dwg_MLINE_vertex *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLINE_line_parent_get) {
  {
    struct _dwg_MLINE_line *arg1 = (struct _dwg_MLINE_line *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_MLINE_vertex *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLINE_line_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINE_line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLINE_line_parent_get" "', argument " "1"" of type '" "struct _dwg_MLINE_line *""'"); 
    }
    arg1 = (struct _dwg_MLINE_line *)(argp1);
    result = (struct _dwg_MLINE_vertex *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_MLINE_vertex, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLINE_line_num_segparms_set) {
  {
    struct _dwg_MLINE_line *arg1 = (struct _dwg_MLINE_line *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLINE_line_num_segparms_set(self,num_segparms);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINE_line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLINE_line_num_segparms_set" "', argument " "1"" of type '" "struct _dwg_MLINE_line *""'"); 
    }
    arg1 = (struct _dwg_MLINE_line *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLINE_line_num_segparms_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_segparms = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLINE_line_num_segparms_get) {
  {
    struct _dwg_MLINE_line *arg1 = (struct _dwg_MLINE_line *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLINE_line_num_segparms_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINE_line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLINE_line_num_segparms_get" "', argument " "1"" of type '" "struct _dwg_MLINE_line *""'"); 
    }
    arg1 = (struct _dwg_MLINE_line *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_segparms);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLINE_line_segparms_set) {
  {
    struct _dwg_MLINE_line *arg1 = (struct _dwg_MLINE_line *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLINE_line_segparms_set(self,segparms);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINE_line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLINE_line_segparms_set" "', argument " "1"" of type '" "struct _dwg_MLINE_line *""'"); 
    }
    arg1 = (struct _dwg_MLINE_line *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MLINE_line_segparms_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->segparms = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLINE_line_segparms_get) {
  {
    struct _dwg_MLINE_line *arg1 = (struct _dwg_MLINE_line *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLINE_line_segparms_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINE_line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLINE_line_segparms_get" "', argument " "1"" of type '" "struct _dwg_MLINE_line *""'"); 
    }
    arg1 = (struct _dwg_MLINE_line *)(argp1);
    result = (BITCODE_BD *) ((arg1)->segparms);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLINE_line_num_areafillparms_set) {
  {
    struct _dwg_MLINE_line *arg1 = (struct _dwg_MLINE_line *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLINE_line_num_areafillparms_set(self,num_areafillparms);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINE_line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLINE_line_num_areafillparms_set" "', argument " "1"" of type '" "struct _dwg_MLINE_line *""'"); 
    }
    arg1 = (struct _dwg_MLINE_line *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLINE_line_num_areafillparms_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_areafillparms = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLINE_line_num_areafillparms_get) {
  {
    struct _dwg_MLINE_line *arg1 = (struct _dwg_MLINE_line *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLINE_line_num_areafillparms_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINE_line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLINE_line_num_areafillparms_get" "', argument " "1"" of type '" "struct _dwg_MLINE_line *""'"); 
    }
    arg1 = (struct _dwg_MLINE_line *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_areafillparms);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLINE_line_areafillparms_set) {
  {
    struct _dwg_MLINE_line *arg1 = (struct _dwg_MLINE_line *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLINE_line_areafillparms_set(self,areafillparms);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINE_line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLINE_line_areafillparms_set" "', argument " "1"" of type '" "struct _dwg_MLINE_line *""'"); 
    }
    arg1 = (struct _dwg_MLINE_line *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MLINE_line_areafillparms_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->areafillparms = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLINE_line_areafillparms_get) {
  {
    struct _dwg_MLINE_line *arg1 = (struct _dwg_MLINE_line *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLINE_line_areafillparms_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINE_line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLINE_line_areafillparms_get" "', argument " "1"" of type '" "struct _dwg_MLINE_line *""'"); 
    }
    arg1 = (struct _dwg_MLINE_line *)(argp1);
    result = (BITCODE_BD *) ((arg1)->areafillparms);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_MLINE_line) {
  {
    int argvi = 0;
    struct _dwg_MLINE_line *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_MLINE_line();");
    }
    result = (struct _dwg_MLINE_line *)calloc(1, sizeof(struct _dwg_MLINE_line));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_MLINE_line, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_MLINE_line) {
  {
    struct _dwg_MLINE_line *arg1 = (struct _dwg_MLINE_line *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_MLINE_line(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINE_line, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_MLINE_line" "', argument " "1"" of type '" "struct _dwg_MLINE_line *""'"); 
    }
    arg1 = (struct _dwg_MLINE_line *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLINE_vertex_parent_set) {
  {
    struct _dwg_MLINE_vertex *arg1 = (struct _dwg_MLINE_vertex *) 0 ;
    struct _dwg_entity_MLINE *arg2 = (struct _dwg_entity_MLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLINE_vertex_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINE_vertex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLINE_vertex_parent_set" "', argument " "1"" of type '" "struct _dwg_MLINE_vertex *""'"); 
    }
    arg1 = (struct _dwg_MLINE_vertex *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_MLINE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MLINE_vertex_parent_set" "', argument " "2"" of type '" "struct _dwg_entity_MLINE *""'"); 
    }
    arg2 = (struct _dwg_entity_MLINE *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLINE_vertex_parent_get) {
  {
    struct _dwg_MLINE_vertex *arg1 = (struct _dwg_MLINE_vertex *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_entity_MLINE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLINE_vertex_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINE_vertex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLINE_vertex_parent_get" "', argument " "1"" of type '" "struct _dwg_MLINE_vertex *""'"); 
    }
    arg1 = (struct _dwg_MLINE_vertex *)(argp1);
    result = (struct _dwg_entity_MLINE *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_MLINE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLINE_vertex_vertex_set) {
  {
    struct _dwg_MLINE_vertex *arg1 = (struct _dwg_MLINE_vertex *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLINE_vertex_vertex_set(self,vertex);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINE_vertex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLINE_vertex_vertex_set" "', argument " "1"" of type '" "struct _dwg_MLINE_vertex *""'"); 
    }
    arg1 = (struct _dwg_MLINE_vertex *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MLINE_vertex_vertex_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->vertex = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLINE_vertex_vertex_get) {
  {
    struct _dwg_MLINE_vertex *arg1 = (struct _dwg_MLINE_vertex *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLINE_vertex_vertex_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINE_vertex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLINE_vertex_vertex_get" "', argument " "1"" of type '" "struct _dwg_MLINE_vertex *""'"); 
    }
    arg1 = (struct _dwg_MLINE_vertex *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->vertex);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLINE_vertex_vertex_direction_set) {
  {
    struct _dwg_MLINE_vertex *arg1 = (struct _dwg_MLINE_vertex *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLINE_vertex_vertex_direction_set(self,vertex_direction);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINE_vertex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLINE_vertex_vertex_direction_set" "', argument " "1"" of type '" "struct _dwg_MLINE_vertex *""'"); 
    }
    arg1 = (struct _dwg_MLINE_vertex *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MLINE_vertex_vertex_direction_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->vertex_direction = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLINE_vertex_vertex_direction_get) {
  {
    struct _dwg_MLINE_vertex *arg1 = (struct _dwg_MLINE_vertex *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLINE_vertex_vertex_direction_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINE_vertex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLINE_vertex_vertex_direction_get" "', argument " "1"" of type '" "struct _dwg_MLINE_vertex *""'"); 
    }
    arg1 = (struct _dwg_MLINE_vertex *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->vertex_direction);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLINE_vertex_miter_direction_set) {
  {
    struct _dwg_MLINE_vertex *arg1 = (struct _dwg_MLINE_vertex *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLINE_vertex_miter_direction_set(self,miter_direction);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINE_vertex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLINE_vertex_miter_direction_set" "', argument " "1"" of type '" "struct _dwg_MLINE_vertex *""'"); 
    }
    arg1 = (struct _dwg_MLINE_vertex *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MLINE_vertex_miter_direction_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->miter_direction = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLINE_vertex_miter_direction_get) {
  {
    struct _dwg_MLINE_vertex *arg1 = (struct _dwg_MLINE_vertex *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLINE_vertex_miter_direction_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINE_vertex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLINE_vertex_miter_direction_get" "', argument " "1"" of type '" "struct _dwg_MLINE_vertex *""'"); 
    }
    arg1 = (struct _dwg_MLINE_vertex *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->miter_direction);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLINE_vertex_num_lines_set) {
  {
    struct _dwg_MLINE_vertex *arg1 = (struct _dwg_MLINE_vertex *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLINE_vertex_num_lines_set(self,num_lines);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINE_vertex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLINE_vertex_num_lines_set" "', argument " "1"" of type '" "struct _dwg_MLINE_vertex *""'"); 
    }
    arg1 = (struct _dwg_MLINE_vertex *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLINE_vertex_num_lines_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->num_lines = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLINE_vertex_num_lines_get) {
  {
    struct _dwg_MLINE_vertex *arg1 = (struct _dwg_MLINE_vertex *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLINE_vertex_num_lines_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINE_vertex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLINE_vertex_num_lines_get" "', argument " "1"" of type '" "struct _dwg_MLINE_vertex *""'"); 
    }
    arg1 = (struct _dwg_MLINE_vertex *)(argp1);
    result = (BITCODE_RC) ((arg1)->num_lines);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLINE_vertex_lines_set) {
  {
    struct _dwg_MLINE_vertex *arg1 = (struct _dwg_MLINE_vertex *) 0 ;
    Dwg_MLINE_line *arg2 = (Dwg_MLINE_line *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLINE_vertex_lines_set(self,lines);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINE_vertex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLINE_vertex_lines_set" "', argument " "1"" of type '" "struct _dwg_MLINE_vertex *""'"); 
    }
    arg1 = (struct _dwg_MLINE_vertex *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_MLINE_line, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MLINE_vertex_lines_set" "', argument " "2"" of type '" "Dwg_MLINE_line *""'"); 
    }
    arg2 = (Dwg_MLINE_line *)(argp2);
    if (arg1) (arg1)->lines = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLINE_vertex_lines_get) {
  {
    struct _dwg_MLINE_vertex *arg1 = (struct _dwg_MLINE_vertex *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_MLINE_line *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLINE_vertex_lines_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINE_vertex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLINE_vertex_lines_get" "', argument " "1"" of type '" "struct _dwg_MLINE_vertex *""'"); 
    }
    arg1 = (struct _dwg_MLINE_vertex *)(argp1);
    result = (Dwg_MLINE_line *) ((arg1)->lines);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_MLINE_line, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_MLINE_vertex) {
  {
    int argvi = 0;
    struct _dwg_MLINE_vertex *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_MLINE_vertex();");
    }
    result = (struct _dwg_MLINE_vertex *)calloc(1, sizeof(struct _dwg_MLINE_vertex));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_MLINE_vertex, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_MLINE_vertex) {
  {
    struct _dwg_MLINE_vertex *arg1 = (struct _dwg_MLINE_vertex *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_MLINE_vertex(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINE_vertex, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_MLINE_vertex" "', argument " "1"" of type '" "struct _dwg_MLINE_vertex *""'"); 
    }
    arg1 = (struct _dwg_MLINE_vertex *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MLINE_parent_set) {
  {
    struct _dwg_entity_MLINE *arg1 = (struct _dwg_entity_MLINE *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MLINE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MLINE_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_MLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_MLINE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MLINE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MLINE_parent_get) {
  {
    struct _dwg_entity_MLINE *arg1 = (struct _dwg_entity_MLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MLINE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MLINE_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_MLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_MLINE *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MLINE_scale_set) {
  {
    struct _dwg_entity_MLINE *arg1 = (struct _dwg_entity_MLINE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MLINE_scale_set(self,scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MLINE_scale_set" "', argument " "1"" of type '" "struct _dwg_entity_MLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_MLINE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MLINE_scale_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->scale = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MLINE_scale_get) {
  {
    struct _dwg_entity_MLINE *arg1 = (struct _dwg_entity_MLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MLINE_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MLINE_scale_get" "', argument " "1"" of type '" "struct _dwg_entity_MLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_MLINE *)(argp1);
    result = (BITCODE_BD) ((arg1)->scale);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MLINE_justification_set) {
  {
    struct _dwg_entity_MLINE *arg1 = (struct _dwg_entity_MLINE *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MLINE_justification_set(self,justification);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MLINE_justification_set" "', argument " "1"" of type '" "struct _dwg_entity_MLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_MLINE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MLINE_justification_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->justification = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MLINE_justification_get) {
  {
    struct _dwg_entity_MLINE *arg1 = (struct _dwg_entity_MLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MLINE_justification_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MLINE_justification_get" "', argument " "1"" of type '" "struct _dwg_entity_MLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_MLINE *)(argp1);
    result = (BITCODE_RC) ((arg1)->justification);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MLINE_base_point_set) {
  {
    struct _dwg_entity_MLINE *arg1 = (struct _dwg_entity_MLINE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MLINE_base_point_set(self,base_point);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MLINE_base_point_set" "', argument " "1"" of type '" "struct _dwg_entity_MLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_MLINE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MLINE_base_point_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->base_point = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MLINE_base_point_get) {
  {
    struct _dwg_entity_MLINE *arg1 = (struct _dwg_entity_MLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MLINE_base_point_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MLINE_base_point_get" "', argument " "1"" of type '" "struct _dwg_entity_MLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_MLINE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->base_point);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MLINE_extrusion_set) {
  {
    struct _dwg_entity_MLINE *arg1 = (struct _dwg_entity_MLINE *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MLINE_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MLINE_extrusion_set" "', argument " "1"" of type '" "struct _dwg_entity_MLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_MLINE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MLINE_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MLINE_extrusion_get) {
  {
    struct _dwg_entity_MLINE *arg1 = (struct _dwg_entity_MLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MLINE_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MLINE_extrusion_get" "', argument " "1"" of type '" "struct _dwg_entity_MLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_MLINE *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MLINE_flags_set) {
  {
    struct _dwg_entity_MLINE *arg1 = (struct _dwg_entity_MLINE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MLINE_flags_set(self,flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MLINE_flags_set" "', argument " "1"" of type '" "struct _dwg_entity_MLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_MLINE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MLINE_flags_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MLINE_flags_get) {
  {
    struct _dwg_entity_MLINE *arg1 = (struct _dwg_entity_MLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MLINE_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MLINE_flags_get" "', argument " "1"" of type '" "struct _dwg_entity_MLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_MLINE *)(argp1);
    result = (BITCODE_BS) ((arg1)->flags);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MLINE_num_lines_set) {
  {
    struct _dwg_entity_MLINE *arg1 = (struct _dwg_entity_MLINE *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MLINE_num_lines_set(self,num_lines);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MLINE_num_lines_set" "', argument " "1"" of type '" "struct _dwg_entity_MLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_MLINE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MLINE_num_lines_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->num_lines = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MLINE_num_lines_get) {
  {
    struct _dwg_entity_MLINE *arg1 = (struct _dwg_entity_MLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MLINE_num_lines_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MLINE_num_lines_get" "', argument " "1"" of type '" "struct _dwg_entity_MLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_MLINE *)(argp1);
    result = (BITCODE_RC) ((arg1)->num_lines);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MLINE_num_verts_set) {
  {
    struct _dwg_entity_MLINE *arg1 = (struct _dwg_entity_MLINE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MLINE_num_verts_set(self,num_verts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MLINE_num_verts_set" "', argument " "1"" of type '" "struct _dwg_entity_MLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_MLINE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MLINE_num_verts_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_verts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MLINE_num_verts_get) {
  {
    struct _dwg_entity_MLINE *arg1 = (struct _dwg_entity_MLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MLINE_num_verts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MLINE_num_verts_get" "', argument " "1"" of type '" "struct _dwg_entity_MLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_MLINE *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_verts);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MLINE_verts_set) {
  {
    struct _dwg_entity_MLINE *arg1 = (struct _dwg_entity_MLINE *) 0 ;
    Dwg_MLINE_vertex *arg2 = (Dwg_MLINE_vertex *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MLINE_verts_set(self,verts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MLINE_verts_set" "', argument " "1"" of type '" "struct _dwg_entity_MLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_MLINE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_MLINE_vertex, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MLINE_verts_set" "', argument " "2"" of type '" "Dwg_MLINE_vertex *""'"); 
    }
    arg2 = (Dwg_MLINE_vertex *)(argp2);
    if (arg1) (arg1)->verts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MLINE_verts_get) {
  {
    struct _dwg_entity_MLINE *arg1 = (struct _dwg_entity_MLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_MLINE_vertex *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MLINE_verts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MLINE_verts_get" "', argument " "1"" of type '" "struct _dwg_entity_MLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_MLINE *)(argp1);
    result = (Dwg_MLINE_vertex *) ((arg1)->verts);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_MLINE_vertex, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MLINE_mlinestyle_set) {
  {
    struct _dwg_entity_MLINE *arg1 = (struct _dwg_entity_MLINE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MLINE_mlinestyle_set(self,mlinestyle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MLINE_mlinestyle_set" "', argument " "1"" of type '" "struct _dwg_entity_MLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_MLINE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MLINE_mlinestyle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->mlinestyle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MLINE_mlinestyle_get) {
  {
    struct _dwg_entity_MLINE *arg1 = (struct _dwg_entity_MLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MLINE_mlinestyle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MLINE_mlinestyle_get" "', argument " "1"" of type '" "struct _dwg_entity_MLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_MLINE *)(argp1);
    result = (BITCODE_H) ((arg1)->mlinestyle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_MLINE) {
  {
    int argvi = 0;
    struct _dwg_entity_MLINE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_MLINE();");
    }
    result = (struct _dwg_entity_MLINE *)calloc(1, sizeof(struct _dwg_entity_MLINE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_MLINE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_MLINE) {
  {
    struct _dwg_entity_MLINE *arg1 = (struct _dwg_entity_MLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_MLINE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MLINE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_MLINE" "', argument " "1"" of type '" "struct _dwg_entity_MLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_MLINE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_CONTROL_parent_set) {
  {
    struct _dwg_object_BLOCK_CONTROL *arg1 = (struct _dwg_object_BLOCK_CONTROL *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_CONTROL_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_CONTROL_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_CONTROL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCK_CONTROL_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_CONTROL_parent_get) {
  {
    struct _dwg_object_BLOCK_CONTROL *arg1 = (struct _dwg_object_BLOCK_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_CONTROL_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_CONTROL_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_CONTROL *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_CONTROL_num_entries_set) {
  {
    struct _dwg_object_BLOCK_CONTROL *arg1 = (struct _dwg_object_BLOCK_CONTROL *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_CONTROL_num_entries_set(self,num_entries);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_CONTROL_num_entries_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_CONTROL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCK_CONTROL_num_entries_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_entries = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_CONTROL_num_entries_get) {
  {
    struct _dwg_object_BLOCK_CONTROL *arg1 = (struct _dwg_object_BLOCK_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_CONTROL_num_entries_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_CONTROL_num_entries_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_CONTROL *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_entries);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_CONTROL_entries_set) {
  {
    struct _dwg_object_BLOCK_CONTROL *arg1 = (struct _dwg_object_BLOCK_CONTROL *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_CONTROL_entries_set(self,entries);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_CONTROL_entries_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_CONTROL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCK_CONTROL_entries_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->entries = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_CONTROL_entries_get) {
  {
    struct _dwg_object_BLOCK_CONTROL *arg1 = (struct _dwg_object_BLOCK_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_CONTROL_entries_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_CONTROL_entries_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_CONTROL *)(argp1);
    result = (BITCODE_H *) ((arg1)->entries);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_CONTROL_flags_r11_set) {
  {
    struct _dwg_object_BLOCK_CONTROL *arg1 = (struct _dwg_object_BLOCK_CONTROL *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_CONTROL_flags_r11_set(self,flags_r11);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_CONTROL_flags_r11_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_CONTROL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCK_CONTROL_flags_r11_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->flags_r11 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_CONTROL_flags_r11_get) {
  {
    struct _dwg_object_BLOCK_CONTROL *arg1 = (struct _dwg_object_BLOCK_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_CONTROL_flags_r11_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_CONTROL_flags_r11_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_CONTROL *)(argp1);
    result = (BITCODE_RS) ((arg1)->flags_r11);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_CONTROL_model_space_set) {
  {
    struct _dwg_object_BLOCK_CONTROL *arg1 = (struct _dwg_object_BLOCK_CONTROL *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_CONTROL_model_space_set(self,model_space);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_CONTROL_model_space_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_CONTROL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCK_CONTROL_model_space_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->model_space = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_CONTROL_model_space_get) {
  {
    struct _dwg_object_BLOCK_CONTROL *arg1 = (struct _dwg_object_BLOCK_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_CONTROL_model_space_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_CONTROL_model_space_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_CONTROL *)(argp1);
    result = (BITCODE_H) ((arg1)->model_space);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_CONTROL_paper_space_set) {
  {
    struct _dwg_object_BLOCK_CONTROL *arg1 = (struct _dwg_object_BLOCK_CONTROL *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_CONTROL_paper_space_set(self,paper_space);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_CONTROL_paper_space_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_CONTROL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCK_CONTROL_paper_space_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->paper_space = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_CONTROL_paper_space_get) {
  {
    struct _dwg_object_BLOCK_CONTROL *arg1 = (struct _dwg_object_BLOCK_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_CONTROL_paper_space_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_CONTROL_paper_space_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_CONTROL *)(argp1);
    result = (BITCODE_H) ((arg1)->paper_space);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCK_CONTROL) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCK_CONTROL *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCK_CONTROL();");
    }
    result = (struct _dwg_object_BLOCK_CONTROL *)calloc(1, sizeof(struct _dwg_object_BLOCK_CONTROL));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCK_CONTROL, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCK_CONTROL) {
  {
    struct _dwg_object_BLOCK_CONTROL *arg1 = (struct _dwg_object_BLOCK_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCK_CONTROL(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_CONTROL, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCK_CONTROL" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_CONTROL *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_parent_set) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCK_HEADER_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_parent_get) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_flag_set) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_flag_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCK_HEADER_flag_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_flag_get) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_flag_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_name_set) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCK_HEADER_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_name_get) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_used_set) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    BITCODE_RSd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_used_set(self,used);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_used_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    ecode2 = SWIG_AsVal_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCK_HEADER_used_set" "', argument " "2"" of type '" "BITCODE_RSd""'");
    } 
    arg2 = (BITCODE_RSd)(val2);
    if (arg1) (arg1)->used = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_used_get) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RSd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_used_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_used_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    result = (BITCODE_RSd) ((arg1)->used);
    ST(argvi) = SWIG_From_short  SWIG_PERL_CALL_ARGS_1((short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_is_xref_ref_set) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_is_xref_ref_set(self,is_xref_ref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_is_xref_ref_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCK_HEADER_is_xref_ref_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_xref_ref = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_is_xref_ref_get) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_is_xref_ref_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_is_xref_ref_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    result = (BITCODE_B) ((arg1)->is_xref_ref);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_is_xref_resolved_set) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_is_xref_resolved_set(self,is_xref_resolved);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_is_xref_resolved_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCK_HEADER_is_xref_resolved_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->is_xref_resolved = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_is_xref_resolved_get) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_is_xref_resolved_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_is_xref_resolved_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    result = (BITCODE_BS) ((arg1)->is_xref_resolved);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_is_xref_dep_set) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_is_xref_dep_set(self,is_xref_dep);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_is_xref_dep_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCK_HEADER_is_xref_dep_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_xref_dep = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_is_xref_dep_get) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_is_xref_dep_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_is_xref_dep_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    result = (BITCODE_B) ((arg1)->is_xref_dep);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_xref_set) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_xref_set(self,xref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_xref_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCK_HEADER_xref_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->xref = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_xref_get) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_xref_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_xref_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    result = (BITCODE_H) ((arg1)->xref);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER___iterator_set) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER___iterator_set(self,__iterator);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER___iterator_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCK_HEADER___iterator_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->__iterator = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER___iterator_get) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER___iterator_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER___iterator_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    result = (BITCODE_BL) ((arg1)->__iterator);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_anonymous_set) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_anonymous_set(self,anonymous);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_anonymous_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCK_HEADER_anonymous_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->anonymous = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_anonymous_get) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_anonymous_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_anonymous_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    result = (BITCODE_B) ((arg1)->anonymous);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_hasattrs_set) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_hasattrs_set(self,hasattrs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_hasattrs_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCK_HEADER_hasattrs_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->hasattrs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_hasattrs_get) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_hasattrs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_hasattrs_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    result = (BITCODE_B) ((arg1)->hasattrs);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_blkisxref_set) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_blkisxref_set(self,blkisxref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_blkisxref_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCK_HEADER_blkisxref_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->blkisxref = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_blkisxref_get) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_blkisxref_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_blkisxref_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    result = (BITCODE_B) ((arg1)->blkisxref);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_xrefoverlaid_set) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_xrefoverlaid_set(self,xrefoverlaid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_xrefoverlaid_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCK_HEADER_xrefoverlaid_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->xrefoverlaid = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_xrefoverlaid_get) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_xrefoverlaid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_xrefoverlaid_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    result = (BITCODE_B) ((arg1)->xrefoverlaid);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_xref_loaded_set) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_xref_loaded_set(self,xref_loaded);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_xref_loaded_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCK_HEADER_xref_loaded_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->xref_loaded = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_xref_loaded_get) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_xref_loaded_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_xref_loaded_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    result = (BITCODE_B) ((arg1)->xref_loaded);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_num_owned_set) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_num_owned_set(self,num_owned);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_num_owned_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCK_HEADER_num_owned_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_owned = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_num_owned_get) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_num_owned_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_num_owned_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_owned);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_base_pt_set) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    BITCODE_3DPOINT *arg2 = (BITCODE_3DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_base_pt_set(self,base_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_base_pt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCK_HEADER_base_pt_set" "', argument " "2"" of type '" "BITCODE_3DPOINT *""'"); 
    }
    arg2 = (BITCODE_3DPOINT *)(argp2);
    if (arg1) (arg1)->base_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_base_pt_get) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_base_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_base_pt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    result = (BITCODE_3DPOINT *)& ((arg1)->base_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_xref_pname_set) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_xref_pname_set(self,xref_pname);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_xref_pname_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCK_HEADER_xref_pname_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->xref_pname);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->xref_pname = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->xref_pname = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_xref_pname_get) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_xref_pname_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_xref_pname_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    result = (BITCODE_TV) ((arg1)->xref_pname);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_num_inserts_set) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_num_inserts_set(self,num_inserts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_num_inserts_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCK_HEADER_num_inserts_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->num_inserts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_num_inserts_get) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_num_inserts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_num_inserts_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    result = (BITCODE_RL) ((arg1)->num_inserts);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_description_set) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_description_set(self,description);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_description_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCK_HEADER_description_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->description);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->description = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->description = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_description_get) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_description_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_description_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    result = (BITCODE_TV) ((arg1)->description);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_preview_size_set) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_preview_size_set(self,preview_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_preview_size_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCK_HEADER_preview_size_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->preview_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_preview_size_get) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_preview_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_preview_size_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    result = (BITCODE_BL) ((arg1)->preview_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_preview_set) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    BITCODE_TF arg2 = (BITCODE_TF) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_preview_set(self,preview);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_preview_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCK_HEADER_preview_set" "', argument " "2"" of type '" "BITCODE_TF""'"); 
    }
    arg2 = (BITCODE_TF)(argp2);
    if (arg1) (arg1)->preview = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_preview_get) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TF result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_preview_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_preview_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    result = (BITCODE_TF) ((arg1)->preview);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_insert_units_set) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_insert_units_set(self,insert_units);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_insert_units_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCK_HEADER_insert_units_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->insert_units = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_insert_units_get) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_insert_units_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_insert_units_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    result = (BITCODE_BS) ((arg1)->insert_units);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_explodable_set) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_explodable_set(self,explodable);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_explodable_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCK_HEADER_explodable_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->explodable = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_explodable_get) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_explodable_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_explodable_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    result = (BITCODE_B) ((arg1)->explodable);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_block_scaling_set) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_block_scaling_set(self,block_scaling);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_block_scaling_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCK_HEADER_block_scaling_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->block_scaling = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_block_scaling_get) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_block_scaling_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_block_scaling_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    result = (BITCODE_RC) ((arg1)->block_scaling);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_block_entity_set) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_block_entity_set(self,block_entity);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_block_entity_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCK_HEADER_block_entity_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->block_entity = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_block_entity_get) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_block_entity_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_block_entity_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    result = (BITCODE_H) ((arg1)->block_entity);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_first_entity_set) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_first_entity_set(self,first_entity);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_first_entity_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCK_HEADER_first_entity_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->first_entity = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_first_entity_get) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_first_entity_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_first_entity_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    result = (BITCODE_H) ((arg1)->first_entity);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_last_entity_set) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_last_entity_set(self,last_entity);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_last_entity_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCK_HEADER_last_entity_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->last_entity = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_last_entity_get) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_last_entity_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_last_entity_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    result = (BITCODE_H) ((arg1)->last_entity);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_entities_set) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_entities_set(self,entities);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_entities_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCK_HEADER_entities_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->entities = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_entities_get) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_entities_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_entities_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    result = (BITCODE_H *) ((arg1)->entities);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_endblk_entity_set) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_endblk_entity_set(self,endblk_entity);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_endblk_entity_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCK_HEADER_endblk_entity_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->endblk_entity = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_endblk_entity_get) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_endblk_entity_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_endblk_entity_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    result = (BITCODE_H) ((arg1)->endblk_entity);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_inserts_set) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_inserts_set(self,inserts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_inserts_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCK_HEADER_inserts_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->inserts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_inserts_get) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_inserts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_inserts_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    result = (BITCODE_H *) ((arg1)->inserts);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_layout_set) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_layout_set(self,layout);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_layout_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCK_HEADER_layout_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->layout = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_layout_get) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_layout_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_layout_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    result = (BITCODE_H) ((arg1)->layout);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_flag2_set) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_flag2_set(self,flag2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_flag2_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCK_HEADER_flag2_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->flag2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_flag2_get) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_flag2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_flag2_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    result = (BITCODE_RS) ((arg1)->flag2);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_unknown_r11_set) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_unknown_r11_set(self,unknown_r11);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_unknown_r11_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCK_HEADER_unknown_r11_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->unknown_r11 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_unknown_r11_get) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_unknown_r11_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_unknown_r11_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    result = (BITCODE_RC) ((arg1)->unknown_r11);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_block_offset_r11_set) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_block_offset_r11_set(self,block_offset_r11);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_block_offset_r11_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCK_HEADER_block_offset_r11_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->block_offset_r11 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCK_HEADER_block_offset_r11_get) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCK_HEADER_block_offset_r11_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCK_HEADER_block_offset_r11_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    result = (BITCODE_RL) ((arg1)->block_offset_r11);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCK_HEADER) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCK_HEADER *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCK_HEADER();");
    }
    result = (struct _dwg_object_BLOCK_HEADER *)calloc(1, sizeof(struct _dwg_object_BLOCK_HEADER));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCK_HEADER, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCK_HEADER) {
  {
    struct _dwg_object_BLOCK_HEADER *arg1 = (struct _dwg_object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCK_HEADER(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCK_HEADER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCK_HEADER" "', argument " "1"" of type '" "struct _dwg_object_BLOCK_HEADER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCK_HEADER *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_CONTROL_parent_set) {
  {
    struct _dwg_object_LAYER_CONTROL *arg1 = (struct _dwg_object_LAYER_CONTROL *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_CONTROL_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_CONTROL_parent_set" "', argument " "1"" of type '" "struct _dwg_object_LAYER_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER_CONTROL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LAYER_CONTROL_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_CONTROL_parent_get) {
  {
    struct _dwg_object_LAYER_CONTROL *arg1 = (struct _dwg_object_LAYER_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_CONTROL_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_CONTROL_parent_get" "', argument " "1"" of type '" "struct _dwg_object_LAYER_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER_CONTROL *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_CONTROL_num_entries_set) {
  {
    struct _dwg_object_LAYER_CONTROL *arg1 = (struct _dwg_object_LAYER_CONTROL *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_CONTROL_num_entries_set(self,num_entries);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_CONTROL_num_entries_set" "', argument " "1"" of type '" "struct _dwg_object_LAYER_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER_CONTROL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LAYER_CONTROL_num_entries_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_entries = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_CONTROL_num_entries_get) {
  {
    struct _dwg_object_LAYER_CONTROL *arg1 = (struct _dwg_object_LAYER_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_CONTROL_num_entries_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_CONTROL_num_entries_get" "', argument " "1"" of type '" "struct _dwg_object_LAYER_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER_CONTROL *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_entries);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_CONTROL_entries_set) {
  {
    struct _dwg_object_LAYER_CONTROL *arg1 = (struct _dwg_object_LAYER_CONTROL *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_CONTROL_entries_set(self,entries);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_CONTROL_entries_set" "', argument " "1"" of type '" "struct _dwg_object_LAYER_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER_CONTROL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LAYER_CONTROL_entries_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->entries = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_CONTROL_entries_get) {
  {
    struct _dwg_object_LAYER_CONTROL *arg1 = (struct _dwg_object_LAYER_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_CONTROL_entries_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_CONTROL_entries_get" "', argument " "1"" of type '" "struct _dwg_object_LAYER_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER_CONTROL *)(argp1);
    result = (BITCODE_H *) ((arg1)->entries);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_CONTROL_flags_r11_set) {
  {
    struct _dwg_object_LAYER_CONTROL *arg1 = (struct _dwg_object_LAYER_CONTROL *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_CONTROL_flags_r11_set(self,flags_r11);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_CONTROL_flags_r11_set" "', argument " "1"" of type '" "struct _dwg_object_LAYER_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER_CONTROL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LAYER_CONTROL_flags_r11_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->flags_r11 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_CONTROL_flags_r11_get) {
  {
    struct _dwg_object_LAYER_CONTROL *arg1 = (struct _dwg_object_LAYER_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_CONTROL_flags_r11_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_CONTROL_flags_r11_get" "', argument " "1"" of type '" "struct _dwg_object_LAYER_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER_CONTROL *)(argp1);
    result = (BITCODE_RS) ((arg1)->flags_r11);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_LAYER_CONTROL) {
  {
    int argvi = 0;
    struct _dwg_object_LAYER_CONTROL *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_LAYER_CONTROL();");
    }
    result = (struct _dwg_object_LAYER_CONTROL *)calloc(1, sizeof(struct _dwg_object_LAYER_CONTROL));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_LAYER_CONTROL, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_LAYER_CONTROL) {
  {
    struct _dwg_object_LAYER_CONTROL *arg1 = (struct _dwg_object_LAYER_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_LAYER_CONTROL(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER_CONTROL, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_LAYER_CONTROL" "', argument " "1"" of type '" "struct _dwg_object_LAYER_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER_CONTROL *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_parent_set) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_parent_set" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LAYER_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_parent_get) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_parent_get" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_flag_set) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_flag_set" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LAYER_flag_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_flag_get) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_flag_get" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_name_set) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_name_set" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LAYER_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_name_get) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_name_get" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_used_set) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    BITCODE_RSd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_used_set(self,used);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_used_set" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    ecode2 = SWIG_AsVal_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LAYER_used_set" "', argument " "2"" of type '" "BITCODE_RSd""'");
    } 
    arg2 = (BITCODE_RSd)(val2);
    if (arg1) (arg1)->used = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_used_get) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RSd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_used_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_used_get" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    result = (BITCODE_RSd) ((arg1)->used);
    ST(argvi) = SWIG_From_short  SWIG_PERL_CALL_ARGS_1((short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_is_xref_ref_set) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_is_xref_ref_set(self,is_xref_ref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_is_xref_ref_set" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LAYER_is_xref_ref_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_xref_ref = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_is_xref_ref_get) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_is_xref_ref_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_is_xref_ref_get" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    result = (BITCODE_B) ((arg1)->is_xref_ref);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_is_xref_resolved_set) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_is_xref_resolved_set(self,is_xref_resolved);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_is_xref_resolved_set" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LAYER_is_xref_resolved_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->is_xref_resolved = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_is_xref_resolved_get) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_is_xref_resolved_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_is_xref_resolved_get" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    result = (BITCODE_BS) ((arg1)->is_xref_resolved);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_is_xref_dep_set) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_is_xref_dep_set(self,is_xref_dep);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_is_xref_dep_set" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LAYER_is_xref_dep_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_xref_dep = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_is_xref_dep_get) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_is_xref_dep_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_is_xref_dep_get" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    result = (BITCODE_B) ((arg1)->is_xref_dep);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_xref_set) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_xref_set(self,xref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_xref_set" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LAYER_xref_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->xref = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_xref_get) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_xref_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_xref_get" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    result = (BITCODE_H) ((arg1)->xref);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_flag0_set) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_flag0_set(self,flag0);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_flag0_set" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LAYER_flag0_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->flag0 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_flag0_get) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_flag0_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_flag0_get" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    result = (BITCODE_BS) ((arg1)->flag0);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_frozen_set) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_frozen_set(self,frozen);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_frozen_set" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LAYER_frozen_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->frozen = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_frozen_get) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_frozen_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_frozen_get" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    result = (BITCODE_B) ((arg1)->frozen);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_off_set) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_off_set(self,off);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_off_set" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LAYER_off_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->off = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_off_get) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_off_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_off_get" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    result = (BITCODE_B) ((arg1)->off);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_frozen_in_new_set) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_frozen_in_new_set(self,frozen_in_new);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_frozen_in_new_set" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LAYER_frozen_in_new_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->frozen_in_new = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_frozen_in_new_get) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_frozen_in_new_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_frozen_in_new_get" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    result = (BITCODE_B) ((arg1)->frozen_in_new);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_locked_set) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_locked_set(self,locked);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_locked_set" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LAYER_locked_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->locked = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_locked_get) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_locked_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_locked_get" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    result = (BITCODE_B) ((arg1)->locked);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_plotflag_set) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_plotflag_set(self,plotflag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_plotflag_set" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LAYER_plotflag_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->plotflag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_plotflag_get) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_plotflag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_plotflag_get" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    result = (BITCODE_B) ((arg1)->plotflag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_linewt_set) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_linewt_set(self,linewt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_linewt_set" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LAYER_linewt_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->linewt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_linewt_get) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_linewt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_linewt_get" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    result = (BITCODE_RC) ((arg1)->linewt);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_color_set) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_color_set(self,color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_color_set" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LAYER_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_color_get) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_color_get" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_plotstyle_set) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_plotstyle_set(self,plotstyle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_plotstyle_set" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LAYER_plotstyle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->plotstyle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_plotstyle_get) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_plotstyle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_plotstyle_get" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    result = (BITCODE_H) ((arg1)->plotstyle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_material_set) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_material_set(self,material);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_material_set" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LAYER_material_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->material = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_material_get) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_material_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_material_get" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    result = (BITCODE_H) ((arg1)->material);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_ltype_set) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_ltype_set(self,ltype);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_ltype_set" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LAYER_ltype_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->ltype = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_ltype_get) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_ltype_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_ltype_get" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    result = (BITCODE_H) ((arg1)->ltype);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_visualstyle_set) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_visualstyle_set(self,visualstyle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_visualstyle_set" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LAYER_visualstyle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->visualstyle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_visualstyle_get) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_visualstyle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_visualstyle_get" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    result = (BITCODE_H) ((arg1)->visualstyle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_LAYER) {
  {
    int argvi = 0;
    struct _dwg_object_LAYER *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_LAYER();");
    }
    result = (struct _dwg_object_LAYER *)calloc(1, sizeof(struct _dwg_object_LAYER));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_LAYER, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_LAYER) {
  {
    struct _dwg_object_LAYER *arg1 = (struct _dwg_object_LAYER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_LAYER(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_LAYER" "', argument " "1"" of type '" "struct _dwg_object_LAYER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_CONTROL_parent_set) {
  {
    struct _dwg_object_STYLE_CONTROL *arg1 = (struct _dwg_object_STYLE_CONTROL *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_CONTROL_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_CONTROL_parent_set" "', argument " "1"" of type '" "struct _dwg_object_STYLE_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE_CONTROL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_STYLE_CONTROL_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_CONTROL_parent_get) {
  {
    struct _dwg_object_STYLE_CONTROL *arg1 = (struct _dwg_object_STYLE_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_CONTROL_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_CONTROL_parent_get" "', argument " "1"" of type '" "struct _dwg_object_STYLE_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE_CONTROL *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_CONTROL_num_entries_set) {
  {
    struct _dwg_object_STYLE_CONTROL *arg1 = (struct _dwg_object_STYLE_CONTROL *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_CONTROL_num_entries_set(self,num_entries);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_CONTROL_num_entries_set" "', argument " "1"" of type '" "struct _dwg_object_STYLE_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE_CONTROL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_STYLE_CONTROL_num_entries_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_entries = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_CONTROL_num_entries_get) {
  {
    struct _dwg_object_STYLE_CONTROL *arg1 = (struct _dwg_object_STYLE_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_CONTROL_num_entries_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_CONTROL_num_entries_get" "', argument " "1"" of type '" "struct _dwg_object_STYLE_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE_CONTROL *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_entries);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_CONTROL_entries_set) {
  {
    struct _dwg_object_STYLE_CONTROL *arg1 = (struct _dwg_object_STYLE_CONTROL *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_CONTROL_entries_set(self,entries);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_CONTROL_entries_set" "', argument " "1"" of type '" "struct _dwg_object_STYLE_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE_CONTROL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_STYLE_CONTROL_entries_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->entries = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_CONTROL_entries_get) {
  {
    struct _dwg_object_STYLE_CONTROL *arg1 = (struct _dwg_object_STYLE_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_CONTROL_entries_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_CONTROL_entries_get" "', argument " "1"" of type '" "struct _dwg_object_STYLE_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE_CONTROL *)(argp1);
    result = (BITCODE_H *) ((arg1)->entries);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_CONTROL_flags_r11_set) {
  {
    struct _dwg_object_STYLE_CONTROL *arg1 = (struct _dwg_object_STYLE_CONTROL *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_CONTROL_flags_r11_set(self,flags_r11);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_CONTROL_flags_r11_set" "', argument " "1"" of type '" "struct _dwg_object_STYLE_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE_CONTROL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_STYLE_CONTROL_flags_r11_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->flags_r11 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_CONTROL_flags_r11_get) {
  {
    struct _dwg_object_STYLE_CONTROL *arg1 = (struct _dwg_object_STYLE_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_CONTROL_flags_r11_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_CONTROL_flags_r11_get" "', argument " "1"" of type '" "struct _dwg_object_STYLE_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE_CONTROL *)(argp1);
    result = (BITCODE_RS) ((arg1)->flags_r11);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_STYLE_CONTROL) {
  {
    int argvi = 0;
    struct _dwg_object_STYLE_CONTROL *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_STYLE_CONTROL();");
    }
    result = (struct _dwg_object_STYLE_CONTROL *)calloc(1, sizeof(struct _dwg_object_STYLE_CONTROL));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_STYLE_CONTROL, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_STYLE_CONTROL) {
  {
    struct _dwg_object_STYLE_CONTROL *arg1 = (struct _dwg_object_STYLE_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_STYLE_CONTROL(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE_CONTROL, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_STYLE_CONTROL" "', argument " "1"" of type '" "struct _dwg_object_STYLE_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE_CONTROL *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_parent_set) {
  {
    struct _dwg_object_STYLE *arg1 = (struct _dwg_object_STYLE *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_parent_set" "', argument " "1"" of type '" "struct _dwg_object_STYLE *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_STYLE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_parent_get) {
  {
    struct _dwg_object_STYLE *arg1 = (struct _dwg_object_STYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_parent_get" "', argument " "1"" of type '" "struct _dwg_object_STYLE *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_flag_set) {
  {
    struct _dwg_object_STYLE *arg1 = (struct _dwg_object_STYLE *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_flag_set" "', argument " "1"" of type '" "struct _dwg_object_STYLE *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_STYLE_flag_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_flag_get) {
  {
    struct _dwg_object_STYLE *arg1 = (struct _dwg_object_STYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_flag_get" "', argument " "1"" of type '" "struct _dwg_object_STYLE *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_name_set) {
  {
    struct _dwg_object_STYLE *arg1 = (struct _dwg_object_STYLE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_name_set" "', argument " "1"" of type '" "struct _dwg_object_STYLE *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_STYLE_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_name_get) {
  {
    struct _dwg_object_STYLE *arg1 = (struct _dwg_object_STYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_name_get" "', argument " "1"" of type '" "struct _dwg_object_STYLE *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_used_set) {
  {
    struct _dwg_object_STYLE *arg1 = (struct _dwg_object_STYLE *) 0 ;
    BITCODE_RSd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_used_set(self,used);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_used_set" "', argument " "1"" of type '" "struct _dwg_object_STYLE *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE *)(argp1);
    ecode2 = SWIG_AsVal_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_STYLE_used_set" "', argument " "2"" of type '" "BITCODE_RSd""'");
    } 
    arg2 = (BITCODE_RSd)(val2);
    if (arg1) (arg1)->used = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_used_get) {
  {
    struct _dwg_object_STYLE *arg1 = (struct _dwg_object_STYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RSd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_used_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_used_get" "', argument " "1"" of type '" "struct _dwg_object_STYLE *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE *)(argp1);
    result = (BITCODE_RSd) ((arg1)->used);
    ST(argvi) = SWIG_From_short  SWIG_PERL_CALL_ARGS_1((short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_is_xref_ref_set) {
  {
    struct _dwg_object_STYLE *arg1 = (struct _dwg_object_STYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_is_xref_ref_set(self,is_xref_ref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_is_xref_ref_set" "', argument " "1"" of type '" "struct _dwg_object_STYLE *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_STYLE_is_xref_ref_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_xref_ref = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_is_xref_ref_get) {
  {
    struct _dwg_object_STYLE *arg1 = (struct _dwg_object_STYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_is_xref_ref_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_is_xref_ref_get" "', argument " "1"" of type '" "struct _dwg_object_STYLE *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->is_xref_ref);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_is_xref_resolved_set) {
  {
    struct _dwg_object_STYLE *arg1 = (struct _dwg_object_STYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_is_xref_resolved_set(self,is_xref_resolved);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_is_xref_resolved_set" "', argument " "1"" of type '" "struct _dwg_object_STYLE *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_STYLE_is_xref_resolved_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->is_xref_resolved = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_is_xref_resolved_get) {
  {
    struct _dwg_object_STYLE *arg1 = (struct _dwg_object_STYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_is_xref_resolved_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_is_xref_resolved_get" "', argument " "1"" of type '" "struct _dwg_object_STYLE *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->is_xref_resolved);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_is_xref_dep_set) {
  {
    struct _dwg_object_STYLE *arg1 = (struct _dwg_object_STYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_is_xref_dep_set(self,is_xref_dep);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_is_xref_dep_set" "', argument " "1"" of type '" "struct _dwg_object_STYLE *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_STYLE_is_xref_dep_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_xref_dep = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_is_xref_dep_get) {
  {
    struct _dwg_object_STYLE *arg1 = (struct _dwg_object_STYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_is_xref_dep_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_is_xref_dep_get" "', argument " "1"" of type '" "struct _dwg_object_STYLE *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->is_xref_dep);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_xref_set) {
  {
    struct _dwg_object_STYLE *arg1 = (struct _dwg_object_STYLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_xref_set(self,xref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_xref_set" "', argument " "1"" of type '" "struct _dwg_object_STYLE *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_STYLE_xref_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->xref = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_xref_get) {
  {
    struct _dwg_object_STYLE *arg1 = (struct _dwg_object_STYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_xref_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_xref_get" "', argument " "1"" of type '" "struct _dwg_object_STYLE *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE *)(argp1);
    result = (BITCODE_H) ((arg1)->xref);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_is_shape_set) {
  {
    struct _dwg_object_STYLE *arg1 = (struct _dwg_object_STYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_is_shape_set(self,is_shape);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_is_shape_set" "', argument " "1"" of type '" "struct _dwg_object_STYLE *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_STYLE_is_shape_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_shape = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_is_shape_get) {
  {
    struct _dwg_object_STYLE *arg1 = (struct _dwg_object_STYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_is_shape_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_is_shape_get" "', argument " "1"" of type '" "struct _dwg_object_STYLE *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->is_shape);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_is_vertical_set) {
  {
    struct _dwg_object_STYLE *arg1 = (struct _dwg_object_STYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_is_vertical_set(self,is_vertical);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_is_vertical_set" "', argument " "1"" of type '" "struct _dwg_object_STYLE *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_STYLE_is_vertical_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_vertical = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_is_vertical_get) {
  {
    struct _dwg_object_STYLE *arg1 = (struct _dwg_object_STYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_is_vertical_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_is_vertical_get" "', argument " "1"" of type '" "struct _dwg_object_STYLE *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->is_vertical);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_text_size_set) {
  {
    struct _dwg_object_STYLE *arg1 = (struct _dwg_object_STYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_text_size_set(self,text_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_text_size_set" "', argument " "1"" of type '" "struct _dwg_object_STYLE *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_STYLE_text_size_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->text_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_text_size_get) {
  {
    struct _dwg_object_STYLE *arg1 = (struct _dwg_object_STYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_text_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_text_size_get" "', argument " "1"" of type '" "struct _dwg_object_STYLE *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->text_size);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_width_factor_set) {
  {
    struct _dwg_object_STYLE *arg1 = (struct _dwg_object_STYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_width_factor_set(self,width_factor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_width_factor_set" "', argument " "1"" of type '" "struct _dwg_object_STYLE *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_STYLE_width_factor_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->width_factor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_width_factor_get) {
  {
    struct _dwg_object_STYLE *arg1 = (struct _dwg_object_STYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_width_factor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_width_factor_get" "', argument " "1"" of type '" "struct _dwg_object_STYLE *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->width_factor);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_oblique_angle_set) {
  {
    struct _dwg_object_STYLE *arg1 = (struct _dwg_object_STYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_oblique_angle_set(self,oblique_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_oblique_angle_set" "', argument " "1"" of type '" "struct _dwg_object_STYLE *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_STYLE_oblique_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->oblique_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_oblique_angle_get) {
  {
    struct _dwg_object_STYLE *arg1 = (struct _dwg_object_STYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_oblique_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_oblique_angle_get" "', argument " "1"" of type '" "struct _dwg_object_STYLE *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->oblique_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_generation_set) {
  {
    struct _dwg_object_STYLE *arg1 = (struct _dwg_object_STYLE *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_generation_set(self,generation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_generation_set" "', argument " "1"" of type '" "struct _dwg_object_STYLE *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_STYLE_generation_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->generation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_generation_get) {
  {
    struct _dwg_object_STYLE *arg1 = (struct _dwg_object_STYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_generation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_generation_get" "', argument " "1"" of type '" "struct _dwg_object_STYLE *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE *)(argp1);
    result = (BITCODE_RC) ((arg1)->generation);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_last_height_set) {
  {
    struct _dwg_object_STYLE *arg1 = (struct _dwg_object_STYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_last_height_set(self,last_height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_last_height_set" "', argument " "1"" of type '" "struct _dwg_object_STYLE *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_STYLE_last_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->last_height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_last_height_get) {
  {
    struct _dwg_object_STYLE *arg1 = (struct _dwg_object_STYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_last_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_last_height_get" "', argument " "1"" of type '" "struct _dwg_object_STYLE *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->last_height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_font_file_set) {
  {
    struct _dwg_object_STYLE *arg1 = (struct _dwg_object_STYLE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_font_file_set(self,font_file);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_font_file_set" "', argument " "1"" of type '" "struct _dwg_object_STYLE *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_STYLE_font_file_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->font_file);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->font_file = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->font_file = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_font_file_get) {
  {
    struct _dwg_object_STYLE *arg1 = (struct _dwg_object_STYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_font_file_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_font_file_get" "', argument " "1"" of type '" "struct _dwg_object_STYLE *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE *)(argp1);
    result = (BITCODE_TV) ((arg1)->font_file);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_bigfont_file_set) {
  {
    struct _dwg_object_STYLE *arg1 = (struct _dwg_object_STYLE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_bigfont_file_set(self,bigfont_file);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_bigfont_file_set" "', argument " "1"" of type '" "struct _dwg_object_STYLE *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_STYLE_bigfont_file_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->bigfont_file);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->bigfont_file = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->bigfont_file = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_STYLE_bigfont_file_get) {
  {
    struct _dwg_object_STYLE *arg1 = (struct _dwg_object_STYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_STYLE_bigfont_file_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_STYLE_bigfont_file_get" "', argument " "1"" of type '" "struct _dwg_object_STYLE *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE *)(argp1);
    result = (BITCODE_TV) ((arg1)->bigfont_file);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_STYLE) {
  {
    int argvi = 0;
    struct _dwg_object_STYLE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_STYLE();");
    }
    result = (struct _dwg_object_STYLE *)calloc(1, sizeof(struct _dwg_object_STYLE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_STYLE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_STYLE) {
  {
    struct _dwg_object_STYLE *arg1 = (struct _dwg_object_STYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_STYLE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_STYLE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_STYLE" "', argument " "1"" of type '" "struct _dwg_object_STYLE *""'"); 
    }
    arg1 = (struct _dwg_object_STYLE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_CONTROL_parent_set) {
  {
    struct _dwg_object_LTYPE_CONTROL *arg1 = (struct _dwg_object_LTYPE_CONTROL *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_CONTROL_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_CONTROL_parent_set" "', argument " "1"" of type '" "struct _dwg_object_LTYPE_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE_CONTROL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LTYPE_CONTROL_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_CONTROL_parent_get) {
  {
    struct _dwg_object_LTYPE_CONTROL *arg1 = (struct _dwg_object_LTYPE_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_CONTROL_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_CONTROL_parent_get" "', argument " "1"" of type '" "struct _dwg_object_LTYPE_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE_CONTROL *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_CONTROL_num_entries_set) {
  {
    struct _dwg_object_LTYPE_CONTROL *arg1 = (struct _dwg_object_LTYPE_CONTROL *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_CONTROL_num_entries_set(self,num_entries);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_CONTROL_num_entries_set" "', argument " "1"" of type '" "struct _dwg_object_LTYPE_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE_CONTROL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LTYPE_CONTROL_num_entries_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_entries = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_CONTROL_num_entries_get) {
  {
    struct _dwg_object_LTYPE_CONTROL *arg1 = (struct _dwg_object_LTYPE_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_CONTROL_num_entries_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_CONTROL_num_entries_get" "', argument " "1"" of type '" "struct _dwg_object_LTYPE_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE_CONTROL *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_entries);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_CONTROL_entries_set) {
  {
    struct _dwg_object_LTYPE_CONTROL *arg1 = (struct _dwg_object_LTYPE_CONTROL *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_CONTROL_entries_set(self,entries);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_CONTROL_entries_set" "', argument " "1"" of type '" "struct _dwg_object_LTYPE_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE_CONTROL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LTYPE_CONTROL_entries_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->entries = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_CONTROL_entries_get) {
  {
    struct _dwg_object_LTYPE_CONTROL *arg1 = (struct _dwg_object_LTYPE_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_CONTROL_entries_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_CONTROL_entries_get" "', argument " "1"" of type '" "struct _dwg_object_LTYPE_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE_CONTROL *)(argp1);
    result = (BITCODE_H *) ((arg1)->entries);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_CONTROL_flags_r11_set) {
  {
    struct _dwg_object_LTYPE_CONTROL *arg1 = (struct _dwg_object_LTYPE_CONTROL *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_CONTROL_flags_r11_set(self,flags_r11);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_CONTROL_flags_r11_set" "', argument " "1"" of type '" "struct _dwg_object_LTYPE_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE_CONTROL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LTYPE_CONTROL_flags_r11_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->flags_r11 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_CONTROL_flags_r11_get) {
  {
    struct _dwg_object_LTYPE_CONTROL *arg1 = (struct _dwg_object_LTYPE_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_CONTROL_flags_r11_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_CONTROL_flags_r11_get" "', argument " "1"" of type '" "struct _dwg_object_LTYPE_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE_CONTROL *)(argp1);
    result = (BITCODE_RS) ((arg1)->flags_r11);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_CONTROL_bylayer_set) {
  {
    struct _dwg_object_LTYPE_CONTROL *arg1 = (struct _dwg_object_LTYPE_CONTROL *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_CONTROL_bylayer_set(self,bylayer);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_CONTROL_bylayer_set" "', argument " "1"" of type '" "struct _dwg_object_LTYPE_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE_CONTROL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LTYPE_CONTROL_bylayer_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->bylayer = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_CONTROL_bylayer_get) {
  {
    struct _dwg_object_LTYPE_CONTROL *arg1 = (struct _dwg_object_LTYPE_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_CONTROL_bylayer_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_CONTROL_bylayer_get" "', argument " "1"" of type '" "struct _dwg_object_LTYPE_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE_CONTROL *)(argp1);
    result = (BITCODE_H) ((arg1)->bylayer);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_CONTROL_byblock_set) {
  {
    struct _dwg_object_LTYPE_CONTROL *arg1 = (struct _dwg_object_LTYPE_CONTROL *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_CONTROL_byblock_set(self,byblock);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_CONTROL_byblock_set" "', argument " "1"" of type '" "struct _dwg_object_LTYPE_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE_CONTROL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LTYPE_CONTROL_byblock_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->byblock = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_CONTROL_byblock_get) {
  {
    struct _dwg_object_LTYPE_CONTROL *arg1 = (struct _dwg_object_LTYPE_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_CONTROL_byblock_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_CONTROL_byblock_get" "', argument " "1"" of type '" "struct _dwg_object_LTYPE_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE_CONTROL *)(argp1);
    result = (BITCODE_H) ((arg1)->byblock);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_LTYPE_CONTROL) {
  {
    int argvi = 0;
    struct _dwg_object_LTYPE_CONTROL *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_LTYPE_CONTROL();");
    }
    result = (struct _dwg_object_LTYPE_CONTROL *)calloc(1, sizeof(struct _dwg_object_LTYPE_CONTROL));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_LTYPE_CONTROL, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_LTYPE_CONTROL) {
  {
    struct _dwg_object_LTYPE_CONTROL *arg1 = (struct _dwg_object_LTYPE_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_LTYPE_CONTROL(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE_CONTROL, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_LTYPE_CONTROL" "', argument " "1"" of type '" "struct _dwg_object_LTYPE_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE_CONTROL *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LTYPE_dash_parent_set) {
  {
    struct _dwg_LTYPE_dash *arg1 = (struct _dwg_LTYPE_dash *) 0 ;
    struct _dwg_object_LTYPE *arg2 = (struct _dwg_object_LTYPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LTYPE_dash_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LTYPE_dash, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LTYPE_dash_parent_set" "', argument " "1"" of type '" "struct _dwg_LTYPE_dash *""'"); 
    }
    arg1 = (struct _dwg_LTYPE_dash *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_LTYPE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_LTYPE_dash_parent_set" "', argument " "2"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg2 = (struct _dwg_object_LTYPE *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LTYPE_dash_parent_get) {
  {
    struct _dwg_LTYPE_dash *arg1 = (struct _dwg_LTYPE_dash *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_LTYPE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LTYPE_dash_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LTYPE_dash, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LTYPE_dash_parent_get" "', argument " "1"" of type '" "struct _dwg_LTYPE_dash *""'"); 
    }
    arg1 = (struct _dwg_LTYPE_dash *)(argp1);
    result = (struct _dwg_object_LTYPE *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_LTYPE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LTYPE_dash_length_set) {
  {
    struct _dwg_LTYPE_dash *arg1 = (struct _dwg_LTYPE_dash *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LTYPE_dash_length_set(self,length);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LTYPE_dash, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LTYPE_dash_length_set" "', argument " "1"" of type '" "struct _dwg_LTYPE_dash *""'"); 
    }
    arg1 = (struct _dwg_LTYPE_dash *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_LTYPE_dash_length_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->length = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LTYPE_dash_length_get) {
  {
    struct _dwg_LTYPE_dash *arg1 = (struct _dwg_LTYPE_dash *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LTYPE_dash_length_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LTYPE_dash, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LTYPE_dash_length_get" "', argument " "1"" of type '" "struct _dwg_LTYPE_dash *""'"); 
    }
    arg1 = (struct _dwg_LTYPE_dash *)(argp1);
    result = (BITCODE_BD) ((arg1)->length);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LTYPE_dash_complex_shapecode_set) {
  {
    struct _dwg_LTYPE_dash *arg1 = (struct _dwg_LTYPE_dash *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LTYPE_dash_complex_shapecode_set(self,complex_shapecode);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LTYPE_dash, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LTYPE_dash_complex_shapecode_set" "', argument " "1"" of type '" "struct _dwg_LTYPE_dash *""'"); 
    }
    arg1 = (struct _dwg_LTYPE_dash *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_LTYPE_dash_complex_shapecode_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->complex_shapecode = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LTYPE_dash_complex_shapecode_get) {
  {
    struct _dwg_LTYPE_dash *arg1 = (struct _dwg_LTYPE_dash *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LTYPE_dash_complex_shapecode_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LTYPE_dash, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LTYPE_dash_complex_shapecode_get" "', argument " "1"" of type '" "struct _dwg_LTYPE_dash *""'"); 
    }
    arg1 = (struct _dwg_LTYPE_dash *)(argp1);
    result = (BITCODE_BS) ((arg1)->complex_shapecode);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LTYPE_dash_style_set) {
  {
    struct _dwg_LTYPE_dash *arg1 = (struct _dwg_LTYPE_dash *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LTYPE_dash_style_set(self,style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LTYPE_dash, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LTYPE_dash_style_set" "', argument " "1"" of type '" "struct _dwg_LTYPE_dash *""'"); 
    }
    arg1 = (struct _dwg_LTYPE_dash *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_LTYPE_dash_style_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LTYPE_dash_style_get) {
  {
    struct _dwg_LTYPE_dash *arg1 = (struct _dwg_LTYPE_dash *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LTYPE_dash_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LTYPE_dash, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LTYPE_dash_style_get" "', argument " "1"" of type '" "struct _dwg_LTYPE_dash *""'"); 
    }
    arg1 = (struct _dwg_LTYPE_dash *)(argp1);
    result = (BITCODE_H) ((arg1)->style);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LTYPE_dash_x_offset_set) {
  {
    struct _dwg_LTYPE_dash *arg1 = (struct _dwg_LTYPE_dash *) 0 ;
    BITCODE_RD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LTYPE_dash_x_offset_set(self,x_offset);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LTYPE_dash, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LTYPE_dash_x_offset_set" "', argument " "1"" of type '" "struct _dwg_LTYPE_dash *""'"); 
    }
    arg1 = (struct _dwg_LTYPE_dash *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_LTYPE_dash_x_offset_set" "', argument " "2"" of type '" "BITCODE_RD""'");
    } 
    arg2 = (BITCODE_RD)(val2);
    if (arg1) (arg1)->x_offset = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LTYPE_dash_x_offset_get) {
  {
    struct _dwg_LTYPE_dash *arg1 = (struct _dwg_LTYPE_dash *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LTYPE_dash_x_offset_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LTYPE_dash, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LTYPE_dash_x_offset_get" "', argument " "1"" of type '" "struct _dwg_LTYPE_dash *""'"); 
    }
    arg1 = (struct _dwg_LTYPE_dash *)(argp1);
    result = (BITCODE_RD) ((arg1)->x_offset);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LTYPE_dash_y_offset_set) {
  {
    struct _dwg_LTYPE_dash *arg1 = (struct _dwg_LTYPE_dash *) 0 ;
    BITCODE_RD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LTYPE_dash_y_offset_set(self,y_offset);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LTYPE_dash, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LTYPE_dash_y_offset_set" "', argument " "1"" of type '" "struct _dwg_LTYPE_dash *""'"); 
    }
    arg1 = (struct _dwg_LTYPE_dash *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_LTYPE_dash_y_offset_set" "', argument " "2"" of type '" "BITCODE_RD""'");
    } 
    arg2 = (BITCODE_RD)(val2);
    if (arg1) (arg1)->y_offset = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LTYPE_dash_y_offset_get) {
  {
    struct _dwg_LTYPE_dash *arg1 = (struct _dwg_LTYPE_dash *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LTYPE_dash_y_offset_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LTYPE_dash, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LTYPE_dash_y_offset_get" "', argument " "1"" of type '" "struct _dwg_LTYPE_dash *""'"); 
    }
    arg1 = (struct _dwg_LTYPE_dash *)(argp1);
    result = (BITCODE_RD) ((arg1)->y_offset);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LTYPE_dash_scale_set) {
  {
    struct _dwg_LTYPE_dash *arg1 = (struct _dwg_LTYPE_dash *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LTYPE_dash_scale_set(self,scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LTYPE_dash, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LTYPE_dash_scale_set" "', argument " "1"" of type '" "struct _dwg_LTYPE_dash *""'"); 
    }
    arg1 = (struct _dwg_LTYPE_dash *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_LTYPE_dash_scale_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->scale = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LTYPE_dash_scale_get) {
  {
    struct _dwg_LTYPE_dash *arg1 = (struct _dwg_LTYPE_dash *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LTYPE_dash_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LTYPE_dash, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LTYPE_dash_scale_get" "', argument " "1"" of type '" "struct _dwg_LTYPE_dash *""'"); 
    }
    arg1 = (struct _dwg_LTYPE_dash *)(argp1);
    result = (BITCODE_BD) ((arg1)->scale);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LTYPE_dash_rotation_set) {
  {
    struct _dwg_LTYPE_dash *arg1 = (struct _dwg_LTYPE_dash *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LTYPE_dash_rotation_set(self,rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LTYPE_dash, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LTYPE_dash_rotation_set" "', argument " "1"" of type '" "struct _dwg_LTYPE_dash *""'"); 
    }
    arg1 = (struct _dwg_LTYPE_dash *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_LTYPE_dash_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LTYPE_dash_rotation_get) {
  {
    struct _dwg_LTYPE_dash *arg1 = (struct _dwg_LTYPE_dash *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LTYPE_dash_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LTYPE_dash, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LTYPE_dash_rotation_get" "', argument " "1"" of type '" "struct _dwg_LTYPE_dash *""'"); 
    }
    arg1 = (struct _dwg_LTYPE_dash *)(argp1);
    result = (BITCODE_BD) ((arg1)->rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LTYPE_dash_shape_flag_set) {
  {
    struct _dwg_LTYPE_dash *arg1 = (struct _dwg_LTYPE_dash *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LTYPE_dash_shape_flag_set(self,shape_flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LTYPE_dash, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LTYPE_dash_shape_flag_set" "', argument " "1"" of type '" "struct _dwg_LTYPE_dash *""'"); 
    }
    arg1 = (struct _dwg_LTYPE_dash *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_LTYPE_dash_shape_flag_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->shape_flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LTYPE_dash_shape_flag_get) {
  {
    struct _dwg_LTYPE_dash *arg1 = (struct _dwg_LTYPE_dash *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LTYPE_dash_shape_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LTYPE_dash, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LTYPE_dash_shape_flag_get" "', argument " "1"" of type '" "struct _dwg_LTYPE_dash *""'"); 
    }
    arg1 = (struct _dwg_LTYPE_dash *)(argp1);
    result = (BITCODE_BS) ((arg1)->shape_flag);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LTYPE_dash_text_set) {
  {
    struct _dwg_LTYPE_dash *arg1 = (struct _dwg_LTYPE_dash *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LTYPE_dash_text_set(self,text);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LTYPE_dash, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LTYPE_dash_text_set" "', argument " "1"" of type '" "struct _dwg_LTYPE_dash *""'"); 
    }
    arg1 = (struct _dwg_LTYPE_dash *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_LTYPE_dash_text_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->text);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->text = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->text = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LTYPE_dash_text_get) {
  {
    struct _dwg_LTYPE_dash *arg1 = (struct _dwg_LTYPE_dash *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LTYPE_dash_text_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LTYPE_dash, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LTYPE_dash_text_get" "', argument " "1"" of type '" "struct _dwg_LTYPE_dash *""'"); 
    }
    arg1 = (struct _dwg_LTYPE_dash *)(argp1);
    result = (BITCODE_TV) ((arg1)->text);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_LTYPE_dash) {
  {
    int argvi = 0;
    struct _dwg_LTYPE_dash *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_LTYPE_dash();");
    }
    result = (struct _dwg_LTYPE_dash *)calloc(1, sizeof(struct _dwg_LTYPE_dash));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_LTYPE_dash, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_LTYPE_dash) {
  {
    struct _dwg_LTYPE_dash *arg1 = (struct _dwg_LTYPE_dash *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_LTYPE_dash(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LTYPE_dash, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_LTYPE_dash" "', argument " "1"" of type '" "struct _dwg_LTYPE_dash *""'"); 
    }
    arg1 = (struct _dwg_LTYPE_dash *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_parent_set) {
  {
    struct _dwg_object_LTYPE *arg1 = (struct _dwg_object_LTYPE *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_parent_set" "', argument " "1"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LTYPE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_parent_get) {
  {
    struct _dwg_object_LTYPE *arg1 = (struct _dwg_object_LTYPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_parent_get" "', argument " "1"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_flag_set) {
  {
    struct _dwg_object_LTYPE *arg1 = (struct _dwg_object_LTYPE *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_flag_set" "', argument " "1"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LTYPE_flag_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_flag_get) {
  {
    struct _dwg_object_LTYPE *arg1 = (struct _dwg_object_LTYPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_flag_get" "', argument " "1"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_name_set) {
  {
    struct _dwg_object_LTYPE *arg1 = (struct _dwg_object_LTYPE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_name_set" "', argument " "1"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LTYPE_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_name_get) {
  {
    struct _dwg_object_LTYPE *arg1 = (struct _dwg_object_LTYPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_name_get" "', argument " "1"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_used_set) {
  {
    struct _dwg_object_LTYPE *arg1 = (struct _dwg_object_LTYPE *) 0 ;
    BITCODE_RSd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_used_set(self,used);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_used_set" "', argument " "1"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE *)(argp1);
    ecode2 = SWIG_AsVal_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LTYPE_used_set" "', argument " "2"" of type '" "BITCODE_RSd""'");
    } 
    arg2 = (BITCODE_RSd)(val2);
    if (arg1) (arg1)->used = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_used_get) {
  {
    struct _dwg_object_LTYPE *arg1 = (struct _dwg_object_LTYPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RSd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_used_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_used_get" "', argument " "1"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE *)(argp1);
    result = (BITCODE_RSd) ((arg1)->used);
    ST(argvi) = SWIG_From_short  SWIG_PERL_CALL_ARGS_1((short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_is_xref_ref_set) {
  {
    struct _dwg_object_LTYPE *arg1 = (struct _dwg_object_LTYPE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_is_xref_ref_set(self,is_xref_ref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_is_xref_ref_set" "', argument " "1"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LTYPE_is_xref_ref_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_xref_ref = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_is_xref_ref_get) {
  {
    struct _dwg_object_LTYPE *arg1 = (struct _dwg_object_LTYPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_is_xref_ref_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_is_xref_ref_get" "', argument " "1"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE *)(argp1);
    result = (BITCODE_B) ((arg1)->is_xref_ref);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_is_xref_resolved_set) {
  {
    struct _dwg_object_LTYPE *arg1 = (struct _dwg_object_LTYPE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_is_xref_resolved_set(self,is_xref_resolved);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_is_xref_resolved_set" "', argument " "1"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LTYPE_is_xref_resolved_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->is_xref_resolved = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_is_xref_resolved_get) {
  {
    struct _dwg_object_LTYPE *arg1 = (struct _dwg_object_LTYPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_is_xref_resolved_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_is_xref_resolved_get" "', argument " "1"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE *)(argp1);
    result = (BITCODE_BS) ((arg1)->is_xref_resolved);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_is_xref_dep_set) {
  {
    struct _dwg_object_LTYPE *arg1 = (struct _dwg_object_LTYPE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_is_xref_dep_set(self,is_xref_dep);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_is_xref_dep_set" "', argument " "1"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LTYPE_is_xref_dep_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_xref_dep = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_is_xref_dep_get) {
  {
    struct _dwg_object_LTYPE *arg1 = (struct _dwg_object_LTYPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_is_xref_dep_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_is_xref_dep_get" "', argument " "1"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE *)(argp1);
    result = (BITCODE_B) ((arg1)->is_xref_dep);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_xref_set) {
  {
    struct _dwg_object_LTYPE *arg1 = (struct _dwg_object_LTYPE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_xref_set(self,xref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_xref_set" "', argument " "1"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LTYPE_xref_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->xref = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_xref_get) {
  {
    struct _dwg_object_LTYPE *arg1 = (struct _dwg_object_LTYPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_xref_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_xref_get" "', argument " "1"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE *)(argp1);
    result = (BITCODE_H) ((arg1)->xref);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_description_set) {
  {
    struct _dwg_object_LTYPE *arg1 = (struct _dwg_object_LTYPE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_description_set(self,description);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_description_set" "', argument " "1"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LTYPE_description_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->description);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->description = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->description = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_description_get) {
  {
    struct _dwg_object_LTYPE *arg1 = (struct _dwg_object_LTYPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_description_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_description_get" "', argument " "1"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE *)(argp1);
    result = (BITCODE_TV) ((arg1)->description);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_pattern_len_set) {
  {
    struct _dwg_object_LTYPE *arg1 = (struct _dwg_object_LTYPE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_pattern_len_set(self,pattern_len);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_pattern_len_set" "', argument " "1"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LTYPE_pattern_len_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->pattern_len = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_pattern_len_get) {
  {
    struct _dwg_object_LTYPE *arg1 = (struct _dwg_object_LTYPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_pattern_len_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_pattern_len_get" "', argument " "1"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE *)(argp1);
    result = (BITCODE_BD) ((arg1)->pattern_len);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_alignment_set) {
  {
    struct _dwg_object_LTYPE *arg1 = (struct _dwg_object_LTYPE *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_alignment_set(self,alignment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_alignment_set" "', argument " "1"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LTYPE_alignment_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->alignment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_alignment_get) {
  {
    struct _dwg_object_LTYPE *arg1 = (struct _dwg_object_LTYPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_alignment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_alignment_get" "', argument " "1"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE *)(argp1);
    result = (BITCODE_RC) ((arg1)->alignment);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_numdashes_set) {
  {
    struct _dwg_object_LTYPE *arg1 = (struct _dwg_object_LTYPE *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_numdashes_set(self,numdashes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_numdashes_set" "', argument " "1"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LTYPE_numdashes_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->numdashes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_numdashes_get) {
  {
    struct _dwg_object_LTYPE *arg1 = (struct _dwg_object_LTYPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_numdashes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_numdashes_get" "', argument " "1"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE *)(argp1);
    result = (BITCODE_RC) ((arg1)->numdashes);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_dashes_set) {
  {
    struct _dwg_object_LTYPE *arg1 = (struct _dwg_object_LTYPE *) 0 ;
    Dwg_LTYPE_dash *arg2 = (Dwg_LTYPE_dash *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_dashes_set(self,dashes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_dashes_set" "', argument " "1"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_LTYPE_dash, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LTYPE_dashes_set" "', argument " "2"" of type '" "Dwg_LTYPE_dash *""'"); 
    }
    arg2 = (Dwg_LTYPE_dash *)(argp2);
    if (arg1) (arg1)->dashes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_dashes_get) {
  {
    struct _dwg_object_LTYPE *arg1 = (struct _dwg_object_LTYPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_LTYPE_dash *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_dashes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_dashes_get" "', argument " "1"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE *)(argp1);
    result = (Dwg_LTYPE_dash *) ((arg1)->dashes);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_LTYPE_dash, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_dashes_r11_set) {
  {
    struct _dwg_object_LTYPE *arg1 = (struct _dwg_object_LTYPE *) 0 ;
    BITCODE_RD *arg2 = (BITCODE_RD *) (BITCODE_RD *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_dashes_r11_set(self,dashes_r11);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_dashes_r11_set" "', argument " "1"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LTYPE_dashes_r11_set" "', argument " "2"" of type '" "BITCODE_RD [12]""'"); 
    } 
    arg2 = (BITCODE_RD *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)12; ++ii) *(BITCODE_RD *)&arg1->dashes_r11[ii] = *((BITCODE_RD *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""dashes_r11""' of type '""BITCODE_RD [12]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_dashes_r11_get) {
  {
    struct _dwg_object_LTYPE *arg1 = (struct _dwg_object_LTYPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_dashes_r11_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_dashes_r11_get" "', argument " "1"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE *)(argp1);
    result = (BITCODE_RD *)(BITCODE_RD *) ((arg1)->dashes_r11);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_has_strings_area_set) {
  {
    struct _dwg_object_LTYPE *arg1 = (struct _dwg_object_LTYPE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_has_strings_area_set(self,has_strings_area);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_has_strings_area_set" "', argument " "1"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LTYPE_has_strings_area_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_strings_area = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_has_strings_area_get) {
  {
    struct _dwg_object_LTYPE *arg1 = (struct _dwg_object_LTYPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_has_strings_area_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_has_strings_area_get" "', argument " "1"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE *)(argp1);
    result = (BITCODE_B) ((arg1)->has_strings_area);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_strings_area_set) {
  {
    struct _dwg_object_LTYPE *arg1 = (struct _dwg_object_LTYPE *) 0 ;
    BITCODE_TF arg2 = (BITCODE_TF) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_strings_area_set(self,strings_area);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_strings_area_set" "', argument " "1"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LTYPE_strings_area_set" "', argument " "2"" of type '" "BITCODE_TF""'"); 
    }
    arg2 = (BITCODE_TF)(argp2);
    if (arg1) (arg1)->strings_area = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_strings_area_get) {
  {
    struct _dwg_object_LTYPE *arg1 = (struct _dwg_object_LTYPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TF result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_strings_area_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_strings_area_get" "', argument " "1"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE *)(argp1);
    result = (BITCODE_TF) ((arg1)->strings_area);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_unknown_r11_set) {
  {
    struct _dwg_object_LTYPE *arg1 = (struct _dwg_object_LTYPE *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_unknown_r11_set(self,unknown_r11);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_unknown_r11_set" "', argument " "1"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LTYPE_unknown_r11_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->unknown_r11 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LTYPE_unknown_r11_get) {
  {
    struct _dwg_object_LTYPE *arg1 = (struct _dwg_object_LTYPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LTYPE_unknown_r11_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LTYPE_unknown_r11_get" "', argument " "1"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE *)(argp1);
    result = (BITCODE_RC) ((arg1)->unknown_r11);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_LTYPE) {
  {
    int argvi = 0;
    struct _dwg_object_LTYPE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_LTYPE();");
    }
    result = (struct _dwg_object_LTYPE *)calloc(1, sizeof(struct _dwg_object_LTYPE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_LTYPE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_LTYPE) {
  {
    struct _dwg_object_LTYPE *arg1 = (struct _dwg_object_LTYPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_LTYPE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LTYPE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_LTYPE" "', argument " "1"" of type '" "struct _dwg_object_LTYPE *""'"); 
    }
    arg1 = (struct _dwg_object_LTYPE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_CONTROL_parent_set) {
  {
    struct _dwg_object_VIEW_CONTROL *arg1 = (struct _dwg_object_VIEW_CONTROL *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_CONTROL_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_CONTROL_parent_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW_CONTROL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VIEW_CONTROL_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_CONTROL_parent_get) {
  {
    struct _dwg_object_VIEW_CONTROL *arg1 = (struct _dwg_object_VIEW_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_CONTROL_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_CONTROL_parent_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW_CONTROL *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_CONTROL_num_entries_set) {
  {
    struct _dwg_object_VIEW_CONTROL *arg1 = (struct _dwg_object_VIEW_CONTROL *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_CONTROL_num_entries_set(self,num_entries);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_CONTROL_num_entries_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW_CONTROL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VIEW_CONTROL_num_entries_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_entries = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_CONTROL_num_entries_get) {
  {
    struct _dwg_object_VIEW_CONTROL *arg1 = (struct _dwg_object_VIEW_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_CONTROL_num_entries_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_CONTROL_num_entries_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW_CONTROL *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_entries);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_CONTROL_entries_set) {
  {
    struct _dwg_object_VIEW_CONTROL *arg1 = (struct _dwg_object_VIEW_CONTROL *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_CONTROL_entries_set(self,entries);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_CONTROL_entries_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW_CONTROL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VIEW_CONTROL_entries_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->entries = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_CONTROL_entries_get) {
  {
    struct _dwg_object_VIEW_CONTROL *arg1 = (struct _dwg_object_VIEW_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_CONTROL_entries_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_CONTROL_entries_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW_CONTROL *)(argp1);
    result = (BITCODE_H *) ((arg1)->entries);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_CONTROL_flags_r11_set) {
  {
    struct _dwg_object_VIEW_CONTROL *arg1 = (struct _dwg_object_VIEW_CONTROL *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_CONTROL_flags_r11_set(self,flags_r11);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_CONTROL_flags_r11_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW_CONTROL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VIEW_CONTROL_flags_r11_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->flags_r11 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_CONTROL_flags_r11_get) {
  {
    struct _dwg_object_VIEW_CONTROL *arg1 = (struct _dwg_object_VIEW_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_CONTROL_flags_r11_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_CONTROL_flags_r11_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW_CONTROL *)(argp1);
    result = (BITCODE_RS) ((arg1)->flags_r11);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_VIEW_CONTROL) {
  {
    int argvi = 0;
    struct _dwg_object_VIEW_CONTROL *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_VIEW_CONTROL();");
    }
    result = (struct _dwg_object_VIEW_CONTROL *)calloc(1, sizeof(struct _dwg_object_VIEW_CONTROL));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_VIEW_CONTROL, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_VIEW_CONTROL) {
  {
    struct _dwg_object_VIEW_CONTROL *arg1 = (struct _dwg_object_VIEW_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_VIEW_CONTROL(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW_CONTROL, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_VIEW_CONTROL" "', argument " "1"" of type '" "struct _dwg_object_VIEW_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW_CONTROL *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_parent_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_parent_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VIEW_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_parent_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_parent_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_flag_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_flag_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VIEW_flag_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_flag_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_flag_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_name_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_name_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VIEW_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_name_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_name_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_used_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_RSd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_used_set(self,used);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_used_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    ecode2 = SWIG_AsVal_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VIEW_used_set" "', argument " "2"" of type '" "BITCODE_RSd""'");
    } 
    arg2 = (BITCODE_RSd)(val2);
    if (arg1) (arg1)->used = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_used_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RSd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_used_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_used_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_RSd) ((arg1)->used);
    ST(argvi) = SWIG_From_short  SWIG_PERL_CALL_ARGS_1((short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_is_xref_ref_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_is_xref_ref_set(self,is_xref_ref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_is_xref_ref_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VIEW_is_xref_ref_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_xref_ref = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_is_xref_ref_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_is_xref_ref_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_is_xref_ref_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_B) ((arg1)->is_xref_ref);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_is_xref_resolved_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_is_xref_resolved_set(self,is_xref_resolved);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_is_xref_resolved_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VIEW_is_xref_resolved_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->is_xref_resolved = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_is_xref_resolved_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_is_xref_resolved_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_is_xref_resolved_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_BS) ((arg1)->is_xref_resolved);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_is_xref_dep_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_is_xref_dep_set(self,is_xref_dep);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_is_xref_dep_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VIEW_is_xref_dep_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_xref_dep = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_is_xref_dep_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_is_xref_dep_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_is_xref_dep_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_B) ((arg1)->is_xref_dep);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_xref_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_xref_set(self,xref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_xref_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VIEW_xref_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->xref = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_xref_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_xref_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_xref_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_H) ((arg1)->xref);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_VIEWSIZE_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_VIEWSIZE_set(self,VIEWSIZE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_VIEWSIZE_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VIEW_VIEWSIZE_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->VIEWSIZE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_VIEWSIZE_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_VIEWSIZE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_VIEWSIZE_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_BD) ((arg1)->VIEWSIZE);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_view_width_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_view_width_set(self,view_width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_view_width_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VIEW_view_width_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->view_width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_view_width_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_view_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_view_width_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_BD) ((arg1)->view_width);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_aspect_ratio_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_aspect_ratio_set(self,aspect_ratio);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_aspect_ratio_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VIEW_aspect_ratio_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->aspect_ratio = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_aspect_ratio_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_aspect_ratio_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_aspect_ratio_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_BD) ((arg1)->aspect_ratio);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_VIEWCTR_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_VIEWCTR_set(self,VIEWCTR);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_VIEWCTR_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VIEW_VIEWCTR_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->VIEWCTR = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_VIEWCTR_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_VIEWCTR_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_VIEWCTR_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->VIEWCTR);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_view_target_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_view_target_set(self,view_target);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_view_target_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VIEW_view_target_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->view_target = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_view_target_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_view_target_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_view_target_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->view_target);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_VIEWDIR_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_VIEWDIR_set(self,VIEWDIR);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_VIEWDIR_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VIEW_VIEWDIR_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->VIEWDIR = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_VIEWDIR_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_VIEWDIR_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_VIEWDIR_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->VIEWDIR);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_twist_angle_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_twist_angle_set(self,twist_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_twist_angle_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VIEW_twist_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->twist_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_twist_angle_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_twist_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_twist_angle_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_BD) ((arg1)->twist_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_lens_length_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_lens_length_set(self,lens_length);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_lens_length_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VIEW_lens_length_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->lens_length = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_lens_length_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_lens_length_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_lens_length_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_BD) ((arg1)->lens_length);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_front_clip_z_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_front_clip_z_set(self,front_clip_z);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_front_clip_z_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VIEW_front_clip_z_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->front_clip_z = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_front_clip_z_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_front_clip_z_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_front_clip_z_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_BD) ((arg1)->front_clip_z);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_back_clip_z_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_back_clip_z_set(self,back_clip_z);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_back_clip_z_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VIEW_back_clip_z_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->back_clip_z = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_back_clip_z_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_back_clip_z_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_back_clip_z_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_BD) ((arg1)->back_clip_z);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_VIEWMODE_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_4BITS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_VIEWMODE_set(self,VIEWMODE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_VIEWMODE_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VIEW_VIEWMODE_set" "', argument " "2"" of type '" "BITCODE_4BITS""'");
    } 
    arg2 = (BITCODE_4BITS)(val2);
    if (arg1) (arg1)->VIEWMODE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_VIEWMODE_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_4BITS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_VIEWMODE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_VIEWMODE_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_4BITS) ((arg1)->VIEWMODE);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_render_mode_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_render_mode_set(self,render_mode);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_render_mode_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VIEW_render_mode_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->render_mode = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_render_mode_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_render_mode_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_render_mode_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_RC) ((arg1)->render_mode);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_use_default_lights_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_use_default_lights_set(self,use_default_lights);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_use_default_lights_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VIEW_use_default_lights_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->use_default_lights = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_use_default_lights_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_use_default_lights_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_use_default_lights_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_B) ((arg1)->use_default_lights);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_default_lightning_type_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_default_lightning_type_set(self,default_lightning_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_default_lightning_type_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VIEW_default_lightning_type_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->default_lightning_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_default_lightning_type_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_default_lightning_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_default_lightning_type_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_RC) ((arg1)->default_lightning_type);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_brightness_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_brightness_set(self,brightness);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_brightness_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VIEW_brightness_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->brightness = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_brightness_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_brightness_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_brightness_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_BD) ((arg1)->brightness);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_contrast_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_contrast_set(self,contrast);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_contrast_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VIEW_contrast_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->contrast = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_contrast_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_contrast_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_contrast_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_BD) ((arg1)->contrast);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_ambient_color_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_ambient_color_set(self,ambient_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_ambient_color_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VIEW_ambient_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->ambient_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_ambient_color_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_ambient_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_ambient_color_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->ambient_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_is_pspace_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_is_pspace_set(self,is_pspace);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_is_pspace_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VIEW_is_pspace_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_pspace = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_is_pspace_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_is_pspace_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_is_pspace_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_B) ((arg1)->is_pspace);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_associated_ucs_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_associated_ucs_set(self,associated_ucs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_associated_ucs_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VIEW_associated_ucs_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->associated_ucs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_associated_ucs_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_associated_ucs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_associated_ucs_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_B) ((arg1)->associated_ucs);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_ucsorg_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_ucsorg_set(self,ucsorg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_ucsorg_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VIEW_ucsorg_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ucsorg = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_ucsorg_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_ucsorg_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_ucsorg_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ucsorg);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_ucsxdir_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_ucsxdir_set(self,ucsxdir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_ucsxdir_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VIEW_ucsxdir_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ucsxdir = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_ucsxdir_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_ucsxdir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_ucsxdir_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ucsxdir);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_ucsydir_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_ucsydir_set(self,ucsydir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_ucsydir_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VIEW_ucsydir_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ucsydir = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_ucsydir_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_ucsydir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_ucsydir_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ucsydir);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_ucs_elevation_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_ucs_elevation_set(self,ucs_elevation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_ucs_elevation_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VIEW_ucs_elevation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->ucs_elevation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_ucs_elevation_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_ucs_elevation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_ucs_elevation_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_BD) ((arg1)->ucs_elevation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_UCSORTHOVIEW_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_UCSORTHOVIEW_set(self,UCSORTHOVIEW);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_UCSORTHOVIEW_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VIEW_UCSORTHOVIEW_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->UCSORTHOVIEW = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_UCSORTHOVIEW_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_UCSORTHOVIEW_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_UCSORTHOVIEW_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_BS) ((arg1)->UCSORTHOVIEW);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_is_camera_plottable_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_is_camera_plottable_set(self,is_camera_plottable);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_is_camera_plottable_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VIEW_is_camera_plottable_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_camera_plottable = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_is_camera_plottable_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_is_camera_plottable_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_is_camera_plottable_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_B) ((arg1)->is_camera_plottable);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_background_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_background_set(self,background);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_background_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VIEW_background_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->background = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_background_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_background_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_background_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_H) ((arg1)->background);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_visualstyle_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_visualstyle_set(self,visualstyle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_visualstyle_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VIEW_visualstyle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->visualstyle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_visualstyle_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_visualstyle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_visualstyle_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_H) ((arg1)->visualstyle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_sun_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_sun_set(self,sun);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_sun_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VIEW_sun_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->sun = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_sun_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_sun_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_sun_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_H) ((arg1)->sun);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_base_ucs_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_base_ucs_set(self,base_ucs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_base_ucs_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VIEW_base_ucs_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->base_ucs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_base_ucs_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_base_ucs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_base_ucs_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_H) ((arg1)->base_ucs);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_named_ucs_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_named_ucs_set(self,named_ucs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_named_ucs_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VIEW_named_ucs_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->named_ucs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_named_ucs_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_named_ucs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_named_ucs_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_H) ((arg1)->named_ucs);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_livesection_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_livesection_set(self,livesection);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_livesection_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VIEW_livesection_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->livesection = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_livesection_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_livesection_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_livesection_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_H) ((arg1)->livesection);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_flag_3d_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_flag_3d_set(self,flag_3d);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_flag_3d_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VIEW_flag_3d_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->flag_3d = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_flag_3d_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_flag_3d_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_flag_3d_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_RS) ((arg1)->flag_3d);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_unknown_r2_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_unknown_r2_set(self,unknown_r2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_unknown_r2_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VIEW_unknown_r2_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->unknown_r2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_unknown_r2_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_unknown_r2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_unknown_r2_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_RC) ((arg1)->unknown_r2);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_unknown_r11_set) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_unknown_r11_set(self,unknown_r11);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_unknown_r11_set" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VIEW_unknown_r11_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->unknown_r11 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VIEW_unknown_r11_get) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VIEW_unknown_r11_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VIEW_unknown_r11_get" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    result = (BITCODE_RC) ((arg1)->unknown_r11);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_VIEW) {
  {
    int argvi = 0;
    struct _dwg_object_VIEW *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_VIEW();");
    }
    result = (struct _dwg_object_VIEW *)calloc(1, sizeof(struct _dwg_object_VIEW));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_VIEW, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_VIEW) {
  {
    struct _dwg_object_VIEW *arg1 = (struct _dwg_object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_VIEW(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VIEW, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_VIEW" "', argument " "1"" of type '" "struct _dwg_object_VIEW *""'"); 
    }
    arg1 = (struct _dwg_object_VIEW *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_CONTROL_parent_set) {
  {
    struct _dwg_object_UCS_CONTROL *arg1 = (struct _dwg_object_UCS_CONTROL *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_UCS_CONTROL_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_CONTROL_parent_set" "', argument " "1"" of type '" "struct _dwg_object_UCS_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_UCS_CONTROL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_UCS_CONTROL_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_CONTROL_parent_get) {
  {
    struct _dwg_object_UCS_CONTROL *arg1 = (struct _dwg_object_UCS_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_UCS_CONTROL_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_CONTROL_parent_get" "', argument " "1"" of type '" "struct _dwg_object_UCS_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_UCS_CONTROL *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_CONTROL_num_entries_set) {
  {
    struct _dwg_object_UCS_CONTROL *arg1 = (struct _dwg_object_UCS_CONTROL *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_UCS_CONTROL_num_entries_set(self,num_entries);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_CONTROL_num_entries_set" "', argument " "1"" of type '" "struct _dwg_object_UCS_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_UCS_CONTROL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_UCS_CONTROL_num_entries_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_entries = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_CONTROL_num_entries_get) {
  {
    struct _dwg_object_UCS_CONTROL *arg1 = (struct _dwg_object_UCS_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_UCS_CONTROL_num_entries_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_CONTROL_num_entries_get" "', argument " "1"" of type '" "struct _dwg_object_UCS_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_UCS_CONTROL *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_entries);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_CONTROL_entries_set) {
  {
    struct _dwg_object_UCS_CONTROL *arg1 = (struct _dwg_object_UCS_CONTROL *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_UCS_CONTROL_entries_set(self,entries);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_CONTROL_entries_set" "', argument " "1"" of type '" "struct _dwg_object_UCS_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_UCS_CONTROL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_UCS_CONTROL_entries_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->entries = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_CONTROL_entries_get) {
  {
    struct _dwg_object_UCS_CONTROL *arg1 = (struct _dwg_object_UCS_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_UCS_CONTROL_entries_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_CONTROL_entries_get" "', argument " "1"" of type '" "struct _dwg_object_UCS_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_UCS_CONTROL *)(argp1);
    result = (BITCODE_H *) ((arg1)->entries);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_CONTROL_flags_r11_set) {
  {
    struct _dwg_object_UCS_CONTROL *arg1 = (struct _dwg_object_UCS_CONTROL *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_UCS_CONTROL_flags_r11_set(self,flags_r11);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_CONTROL_flags_r11_set" "', argument " "1"" of type '" "struct _dwg_object_UCS_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_UCS_CONTROL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_UCS_CONTROL_flags_r11_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->flags_r11 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_CONTROL_flags_r11_get) {
  {
    struct _dwg_object_UCS_CONTROL *arg1 = (struct _dwg_object_UCS_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_UCS_CONTROL_flags_r11_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_CONTROL_flags_r11_get" "', argument " "1"" of type '" "struct _dwg_object_UCS_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_UCS_CONTROL *)(argp1);
    result = (BITCODE_RS) ((arg1)->flags_r11);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_UCS_CONTROL) {
  {
    int argvi = 0;
    struct _dwg_object_UCS_CONTROL *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_UCS_CONTROL();");
    }
    result = (struct _dwg_object_UCS_CONTROL *)calloc(1, sizeof(struct _dwg_object_UCS_CONTROL));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_UCS_CONTROL, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_UCS_CONTROL) {
  {
    struct _dwg_object_UCS_CONTROL *arg1 = (struct _dwg_object_UCS_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_UCS_CONTROL(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS_CONTROL, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_UCS_CONTROL" "', argument " "1"" of type '" "struct _dwg_object_UCS_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_UCS_CONTROL *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_UCS_orthopts_parent_set) {
  {
    struct _dwg_UCS_orthopts *arg1 = (struct _dwg_UCS_orthopts *) 0 ;
    struct _dwg_object_UCS *arg2 = (struct _dwg_object_UCS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_UCS_orthopts_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_UCS_orthopts, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_UCS_orthopts_parent_set" "', argument " "1"" of type '" "struct _dwg_UCS_orthopts *""'"); 
    }
    arg1 = (struct _dwg_UCS_orthopts *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_UCS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_UCS_orthopts_parent_set" "', argument " "2"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg2 = (struct _dwg_object_UCS *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_UCS_orthopts_parent_get) {
  {
    struct _dwg_UCS_orthopts *arg1 = (struct _dwg_UCS_orthopts *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_UCS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_UCS_orthopts_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_UCS_orthopts, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_UCS_orthopts_parent_get" "', argument " "1"" of type '" "struct _dwg_UCS_orthopts *""'"); 
    }
    arg1 = (struct _dwg_UCS_orthopts *)(argp1);
    result = (struct _dwg_object_UCS *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_UCS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_UCS_orthopts_type_set) {
  {
    struct _dwg_UCS_orthopts *arg1 = (struct _dwg_UCS_orthopts *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_UCS_orthopts_type_set(self,type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_UCS_orthopts, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_UCS_orthopts_type_set" "', argument " "1"" of type '" "struct _dwg_UCS_orthopts *""'"); 
    }
    arg1 = (struct _dwg_UCS_orthopts *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_UCS_orthopts_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_UCS_orthopts_type_get) {
  {
    struct _dwg_UCS_orthopts *arg1 = (struct _dwg_UCS_orthopts *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_UCS_orthopts_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_UCS_orthopts, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_UCS_orthopts_type_get" "', argument " "1"" of type '" "struct _dwg_UCS_orthopts *""'"); 
    }
    arg1 = (struct _dwg_UCS_orthopts *)(argp1);
    result = (BITCODE_BS) ((arg1)->type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_UCS_orthopts_pt_set) {
  {
    struct _dwg_UCS_orthopts *arg1 = (struct _dwg_UCS_orthopts *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_UCS_orthopts_pt_set(self,pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_UCS_orthopts, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_UCS_orthopts_pt_set" "', argument " "1"" of type '" "struct _dwg_UCS_orthopts *""'"); 
    }
    arg1 = (struct _dwg_UCS_orthopts *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_UCS_orthopts_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_UCS_orthopts_pt_get) {
  {
    struct _dwg_UCS_orthopts *arg1 = (struct _dwg_UCS_orthopts *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_UCS_orthopts_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_UCS_orthopts, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_UCS_orthopts_pt_get" "', argument " "1"" of type '" "struct _dwg_UCS_orthopts *""'"); 
    }
    arg1 = (struct _dwg_UCS_orthopts *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_UCS_orthopts) {
  {
    int argvi = 0;
    struct _dwg_UCS_orthopts *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_UCS_orthopts();");
    }
    result = (struct _dwg_UCS_orthopts *)calloc(1, sizeof(struct _dwg_UCS_orthopts));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_UCS_orthopts, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_UCS_orthopts) {
  {
    struct _dwg_UCS_orthopts *arg1 = (struct _dwg_UCS_orthopts *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_UCS_orthopts(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_UCS_orthopts, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_UCS_orthopts" "', argument " "1"" of type '" "struct _dwg_UCS_orthopts *""'"); 
    }
    arg1 = (struct _dwg_UCS_orthopts *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_parent_set) {
  {
    struct _dwg_object_UCS *arg1 = (struct _dwg_object_UCS *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_UCS_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_parent_set" "', argument " "1"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg1 = (struct _dwg_object_UCS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_UCS_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_parent_get) {
  {
    struct _dwg_object_UCS *arg1 = (struct _dwg_object_UCS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_UCS_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_parent_get" "', argument " "1"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg1 = (struct _dwg_object_UCS *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_flag_set) {
  {
    struct _dwg_object_UCS *arg1 = (struct _dwg_object_UCS *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_UCS_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_flag_set" "', argument " "1"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg1 = (struct _dwg_object_UCS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_UCS_flag_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_flag_get) {
  {
    struct _dwg_object_UCS *arg1 = (struct _dwg_object_UCS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_UCS_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_flag_get" "', argument " "1"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg1 = (struct _dwg_object_UCS *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_name_set) {
  {
    struct _dwg_object_UCS *arg1 = (struct _dwg_object_UCS *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_UCS_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_name_set" "', argument " "1"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg1 = (struct _dwg_object_UCS *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_UCS_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_name_get) {
  {
    struct _dwg_object_UCS *arg1 = (struct _dwg_object_UCS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_UCS_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_name_get" "', argument " "1"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg1 = (struct _dwg_object_UCS *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_used_set) {
  {
    struct _dwg_object_UCS *arg1 = (struct _dwg_object_UCS *) 0 ;
    BITCODE_RSd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_UCS_used_set(self,used);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_used_set" "', argument " "1"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg1 = (struct _dwg_object_UCS *)(argp1);
    ecode2 = SWIG_AsVal_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_UCS_used_set" "', argument " "2"" of type '" "BITCODE_RSd""'");
    } 
    arg2 = (BITCODE_RSd)(val2);
    if (arg1) (arg1)->used = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_used_get) {
  {
    struct _dwg_object_UCS *arg1 = (struct _dwg_object_UCS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RSd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_UCS_used_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_used_get" "', argument " "1"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg1 = (struct _dwg_object_UCS *)(argp1);
    result = (BITCODE_RSd) ((arg1)->used);
    ST(argvi) = SWIG_From_short  SWIG_PERL_CALL_ARGS_1((short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_is_xref_ref_set) {
  {
    struct _dwg_object_UCS *arg1 = (struct _dwg_object_UCS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_UCS_is_xref_ref_set(self,is_xref_ref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_is_xref_ref_set" "', argument " "1"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg1 = (struct _dwg_object_UCS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_UCS_is_xref_ref_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_xref_ref = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_is_xref_ref_get) {
  {
    struct _dwg_object_UCS *arg1 = (struct _dwg_object_UCS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_UCS_is_xref_ref_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_is_xref_ref_get" "', argument " "1"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg1 = (struct _dwg_object_UCS *)(argp1);
    result = (BITCODE_B) ((arg1)->is_xref_ref);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_is_xref_resolved_set) {
  {
    struct _dwg_object_UCS *arg1 = (struct _dwg_object_UCS *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_UCS_is_xref_resolved_set(self,is_xref_resolved);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_is_xref_resolved_set" "', argument " "1"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg1 = (struct _dwg_object_UCS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_UCS_is_xref_resolved_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->is_xref_resolved = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_is_xref_resolved_get) {
  {
    struct _dwg_object_UCS *arg1 = (struct _dwg_object_UCS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_UCS_is_xref_resolved_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_is_xref_resolved_get" "', argument " "1"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg1 = (struct _dwg_object_UCS *)(argp1);
    result = (BITCODE_BS) ((arg1)->is_xref_resolved);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_is_xref_dep_set) {
  {
    struct _dwg_object_UCS *arg1 = (struct _dwg_object_UCS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_UCS_is_xref_dep_set(self,is_xref_dep);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_is_xref_dep_set" "', argument " "1"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg1 = (struct _dwg_object_UCS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_UCS_is_xref_dep_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_xref_dep = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_is_xref_dep_get) {
  {
    struct _dwg_object_UCS *arg1 = (struct _dwg_object_UCS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_UCS_is_xref_dep_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_is_xref_dep_get" "', argument " "1"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg1 = (struct _dwg_object_UCS *)(argp1);
    result = (BITCODE_B) ((arg1)->is_xref_dep);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_xref_set) {
  {
    struct _dwg_object_UCS *arg1 = (struct _dwg_object_UCS *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_UCS_xref_set(self,xref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_xref_set" "', argument " "1"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg1 = (struct _dwg_object_UCS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_UCS_xref_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->xref = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_xref_get) {
  {
    struct _dwg_object_UCS *arg1 = (struct _dwg_object_UCS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_UCS_xref_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_xref_get" "', argument " "1"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg1 = (struct _dwg_object_UCS *)(argp1);
    result = (BITCODE_H) ((arg1)->xref);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_ucsorg_set) {
  {
    struct _dwg_object_UCS *arg1 = (struct _dwg_object_UCS *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_UCS_ucsorg_set(self,ucsorg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_ucsorg_set" "', argument " "1"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg1 = (struct _dwg_object_UCS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_UCS_ucsorg_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ucsorg = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_ucsorg_get) {
  {
    struct _dwg_object_UCS *arg1 = (struct _dwg_object_UCS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_UCS_ucsorg_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_ucsorg_get" "', argument " "1"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg1 = (struct _dwg_object_UCS *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ucsorg);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_ucsxdir_set) {
  {
    struct _dwg_object_UCS *arg1 = (struct _dwg_object_UCS *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_UCS_ucsxdir_set(self,ucsxdir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_ucsxdir_set" "', argument " "1"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg1 = (struct _dwg_object_UCS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_UCS_ucsxdir_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ucsxdir = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_ucsxdir_get) {
  {
    struct _dwg_object_UCS *arg1 = (struct _dwg_object_UCS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_UCS_ucsxdir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_ucsxdir_get" "', argument " "1"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg1 = (struct _dwg_object_UCS *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ucsxdir);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_ucsydir_set) {
  {
    struct _dwg_object_UCS *arg1 = (struct _dwg_object_UCS *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_UCS_ucsydir_set(self,ucsydir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_ucsydir_set" "', argument " "1"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg1 = (struct _dwg_object_UCS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_UCS_ucsydir_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ucsydir = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_ucsydir_get) {
  {
    struct _dwg_object_UCS *arg1 = (struct _dwg_object_UCS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_UCS_ucsydir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_ucsydir_get" "', argument " "1"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg1 = (struct _dwg_object_UCS *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ucsydir);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_ucs_elevation_set) {
  {
    struct _dwg_object_UCS *arg1 = (struct _dwg_object_UCS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_UCS_ucs_elevation_set(self,ucs_elevation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_ucs_elevation_set" "', argument " "1"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg1 = (struct _dwg_object_UCS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_UCS_ucs_elevation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->ucs_elevation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_ucs_elevation_get) {
  {
    struct _dwg_object_UCS *arg1 = (struct _dwg_object_UCS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_UCS_ucs_elevation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_ucs_elevation_get" "', argument " "1"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg1 = (struct _dwg_object_UCS *)(argp1);
    result = (BITCODE_BD) ((arg1)->ucs_elevation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_UCSORTHOVIEW_set) {
  {
    struct _dwg_object_UCS *arg1 = (struct _dwg_object_UCS *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_UCS_UCSORTHOVIEW_set(self,UCSORTHOVIEW);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_UCSORTHOVIEW_set" "', argument " "1"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg1 = (struct _dwg_object_UCS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_UCS_UCSORTHOVIEW_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->UCSORTHOVIEW = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_UCSORTHOVIEW_get) {
  {
    struct _dwg_object_UCS *arg1 = (struct _dwg_object_UCS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_UCS_UCSORTHOVIEW_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_UCSORTHOVIEW_get" "', argument " "1"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg1 = (struct _dwg_object_UCS *)(argp1);
    result = (BITCODE_BS) ((arg1)->UCSORTHOVIEW);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_base_ucs_set) {
  {
    struct _dwg_object_UCS *arg1 = (struct _dwg_object_UCS *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_UCS_base_ucs_set(self,base_ucs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_base_ucs_set" "', argument " "1"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg1 = (struct _dwg_object_UCS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_UCS_base_ucs_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->base_ucs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_base_ucs_get) {
  {
    struct _dwg_object_UCS *arg1 = (struct _dwg_object_UCS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_UCS_base_ucs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_base_ucs_get" "', argument " "1"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg1 = (struct _dwg_object_UCS *)(argp1);
    result = (BITCODE_H) ((arg1)->base_ucs);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_named_ucs_set) {
  {
    struct _dwg_object_UCS *arg1 = (struct _dwg_object_UCS *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_UCS_named_ucs_set(self,named_ucs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_named_ucs_set" "', argument " "1"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg1 = (struct _dwg_object_UCS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_UCS_named_ucs_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->named_ucs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_named_ucs_get) {
  {
    struct _dwg_object_UCS *arg1 = (struct _dwg_object_UCS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_UCS_named_ucs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_named_ucs_get" "', argument " "1"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg1 = (struct _dwg_object_UCS *)(argp1);
    result = (BITCODE_H) ((arg1)->named_ucs);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_num_orthopts_set) {
  {
    struct _dwg_object_UCS *arg1 = (struct _dwg_object_UCS *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_UCS_num_orthopts_set(self,num_orthopts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_num_orthopts_set" "', argument " "1"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg1 = (struct _dwg_object_UCS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_UCS_num_orthopts_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_orthopts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_num_orthopts_get) {
  {
    struct _dwg_object_UCS *arg1 = (struct _dwg_object_UCS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_UCS_num_orthopts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_num_orthopts_get" "', argument " "1"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg1 = (struct _dwg_object_UCS *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_orthopts);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_orthopts_set) {
  {
    struct _dwg_object_UCS *arg1 = (struct _dwg_object_UCS *) 0 ;
    Dwg_UCS_orthopts *arg2 = (Dwg_UCS_orthopts *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_UCS_orthopts_set(self,orthopts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_orthopts_set" "', argument " "1"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg1 = (struct _dwg_object_UCS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_UCS_orthopts, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_UCS_orthopts_set" "', argument " "2"" of type '" "Dwg_UCS_orthopts *""'"); 
    }
    arg2 = (Dwg_UCS_orthopts *)(argp2);
    if (arg1) (arg1)->orthopts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UCS_orthopts_get) {
  {
    struct _dwg_object_UCS *arg1 = (struct _dwg_object_UCS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_UCS_orthopts *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_UCS_orthopts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UCS_orthopts_get" "', argument " "1"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg1 = (struct _dwg_object_UCS *)(argp1);
    result = (Dwg_UCS_orthopts *) ((arg1)->orthopts);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_UCS_orthopts, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_UCS) {
  {
    int argvi = 0;
    struct _dwg_object_UCS *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_UCS();");
    }
    result = (struct _dwg_object_UCS *)calloc(1, sizeof(struct _dwg_object_UCS));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_UCS, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_UCS) {
  {
    struct _dwg_object_UCS *arg1 = (struct _dwg_object_UCS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_UCS(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UCS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_UCS" "', argument " "1"" of type '" "struct _dwg_object_UCS *""'"); 
    }
    arg1 = (struct _dwg_object_UCS *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_CONTROL_parent_set) {
  {
    struct _dwg_object_VPORT_CONTROL *arg1 = (struct _dwg_object_VPORT_CONTROL *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_CONTROL_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_CONTROL_parent_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT_CONTROL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VPORT_CONTROL_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_CONTROL_parent_get) {
  {
    struct _dwg_object_VPORT_CONTROL *arg1 = (struct _dwg_object_VPORT_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_CONTROL_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_CONTROL_parent_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT_CONTROL *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_CONTROL_num_entries_set) {
  {
    struct _dwg_object_VPORT_CONTROL *arg1 = (struct _dwg_object_VPORT_CONTROL *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_CONTROL_num_entries_set(self,num_entries);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_CONTROL_num_entries_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT_CONTROL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VPORT_CONTROL_num_entries_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_entries = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_CONTROL_num_entries_get) {
  {
    struct _dwg_object_VPORT_CONTROL *arg1 = (struct _dwg_object_VPORT_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_CONTROL_num_entries_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_CONTROL_num_entries_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT_CONTROL *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_entries);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_CONTROL_entries_set) {
  {
    struct _dwg_object_VPORT_CONTROL *arg1 = (struct _dwg_object_VPORT_CONTROL *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_CONTROL_entries_set(self,entries);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_CONTROL_entries_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT_CONTROL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VPORT_CONTROL_entries_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->entries = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_CONTROL_entries_get) {
  {
    struct _dwg_object_VPORT_CONTROL *arg1 = (struct _dwg_object_VPORT_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_CONTROL_entries_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_CONTROL_entries_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT_CONTROL *)(argp1);
    result = (BITCODE_H *) ((arg1)->entries);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_CONTROL_flags_r11_set) {
  {
    struct _dwg_object_VPORT_CONTROL *arg1 = (struct _dwg_object_VPORT_CONTROL *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_CONTROL_flags_r11_set(self,flags_r11);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_CONTROL_flags_r11_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT_CONTROL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VPORT_CONTROL_flags_r11_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->flags_r11 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_CONTROL_flags_r11_get) {
  {
    struct _dwg_object_VPORT_CONTROL *arg1 = (struct _dwg_object_VPORT_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_CONTROL_flags_r11_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_CONTROL_flags_r11_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT_CONTROL *)(argp1);
    result = (BITCODE_RS) ((arg1)->flags_r11);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_VPORT_CONTROL) {
  {
    int argvi = 0;
    struct _dwg_object_VPORT_CONTROL *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_VPORT_CONTROL();");
    }
    result = (struct _dwg_object_VPORT_CONTROL *)calloc(1, sizeof(struct _dwg_object_VPORT_CONTROL));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_VPORT_CONTROL, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_VPORT_CONTROL) {
  {
    struct _dwg_object_VPORT_CONTROL *arg1 = (struct _dwg_object_VPORT_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_VPORT_CONTROL(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT_CONTROL, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_VPORT_CONTROL" "', argument " "1"" of type '" "struct _dwg_object_VPORT_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT_CONTROL *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_parent_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_parent_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VPORT_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_parent_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_parent_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_flag_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_flag_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VPORT_flag_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_flag_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_flag_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_name_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_name_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VPORT_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_name_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_name_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_used_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_RSd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_used_set(self,used);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_used_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    ecode2 = SWIG_AsVal_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VPORT_used_set" "', argument " "2"" of type '" "BITCODE_RSd""'");
    } 
    arg2 = (BITCODE_RSd)(val2);
    if (arg1) (arg1)->used = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_used_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RSd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_used_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_used_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_RSd) ((arg1)->used);
    ST(argvi) = SWIG_From_short  SWIG_PERL_CALL_ARGS_1((short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_is_xref_ref_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_is_xref_ref_set(self,is_xref_ref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_is_xref_ref_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VPORT_is_xref_ref_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_xref_ref = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_is_xref_ref_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_is_xref_ref_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_is_xref_ref_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_B) ((arg1)->is_xref_ref);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_is_xref_resolved_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_is_xref_resolved_set(self,is_xref_resolved);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_is_xref_resolved_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VPORT_is_xref_resolved_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->is_xref_resolved = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_is_xref_resolved_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_is_xref_resolved_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_is_xref_resolved_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_BS) ((arg1)->is_xref_resolved);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_is_xref_dep_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_is_xref_dep_set(self,is_xref_dep);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_is_xref_dep_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VPORT_is_xref_dep_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_xref_dep = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_is_xref_dep_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_is_xref_dep_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_is_xref_dep_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_B) ((arg1)->is_xref_dep);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_xref_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_xref_set(self,xref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_xref_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VPORT_xref_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->xref = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_xref_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_xref_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_xref_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_H) ((arg1)->xref);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_VIEWSIZE_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_VIEWSIZE_set(self,VIEWSIZE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_VIEWSIZE_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VPORT_VIEWSIZE_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->VIEWSIZE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_VIEWSIZE_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_VIEWSIZE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_VIEWSIZE_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_BD) ((arg1)->VIEWSIZE);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_view_width_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_view_width_set(self,view_width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_view_width_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VPORT_view_width_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->view_width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_view_width_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_view_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_view_width_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_BD) ((arg1)->view_width);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_aspect_ratio_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_aspect_ratio_set(self,aspect_ratio);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_aspect_ratio_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VPORT_aspect_ratio_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->aspect_ratio = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_aspect_ratio_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_aspect_ratio_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_aspect_ratio_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_BD) ((arg1)->aspect_ratio);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_VIEWCTR_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_VIEWCTR_set(self,VIEWCTR);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_VIEWCTR_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VPORT_VIEWCTR_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->VIEWCTR = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_VIEWCTR_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_VIEWCTR_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_VIEWCTR_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->VIEWCTR);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_view_target_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_view_target_set(self,view_target);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_view_target_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VPORT_view_target_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->view_target = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_view_target_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_view_target_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_view_target_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->view_target);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_VIEWDIR_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_VIEWDIR_set(self,VIEWDIR);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_VIEWDIR_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VPORT_VIEWDIR_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->VIEWDIR = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_VIEWDIR_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_VIEWDIR_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_VIEWDIR_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->VIEWDIR);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_view_twist_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_view_twist_set(self,view_twist);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_view_twist_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VPORT_view_twist_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->view_twist = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_view_twist_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_view_twist_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_view_twist_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_BD) ((arg1)->view_twist);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_lens_length_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_lens_length_set(self,lens_length);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_lens_length_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VPORT_lens_length_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->lens_length = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_lens_length_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_lens_length_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_lens_length_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_BD) ((arg1)->lens_length);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_front_clip_z_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_front_clip_z_set(self,front_clip_z);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_front_clip_z_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VPORT_front_clip_z_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->front_clip_z = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_front_clip_z_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_front_clip_z_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_front_clip_z_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_BD) ((arg1)->front_clip_z);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_back_clip_z_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_back_clip_z_set(self,back_clip_z);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_back_clip_z_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VPORT_back_clip_z_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->back_clip_z = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_back_clip_z_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_back_clip_z_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_back_clip_z_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_BD) ((arg1)->back_clip_z);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_VIEWMODE_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_4BITS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_VIEWMODE_set(self,VIEWMODE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_VIEWMODE_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VPORT_VIEWMODE_set" "', argument " "2"" of type '" "BITCODE_4BITS""'");
    } 
    arg2 = (BITCODE_4BITS)(val2);
    if (arg1) (arg1)->VIEWMODE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_VIEWMODE_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_4BITS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_VIEWMODE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_VIEWMODE_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_4BITS) ((arg1)->VIEWMODE);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_render_mode_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_render_mode_set(self,render_mode);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_render_mode_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VPORT_render_mode_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->render_mode = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_render_mode_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_render_mode_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_render_mode_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_RC) ((arg1)->render_mode);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_use_default_lights_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_use_default_lights_set(self,use_default_lights);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_use_default_lights_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VPORT_use_default_lights_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->use_default_lights = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_use_default_lights_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_use_default_lights_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_use_default_lights_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_B) ((arg1)->use_default_lights);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_default_lightning_type_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_default_lightning_type_set(self,default_lightning_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_default_lightning_type_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VPORT_default_lightning_type_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->default_lightning_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_default_lightning_type_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_default_lightning_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_default_lightning_type_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_RC) ((arg1)->default_lightning_type);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_brightness_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_brightness_set(self,brightness);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_brightness_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VPORT_brightness_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->brightness = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_brightness_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_brightness_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_brightness_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_BD) ((arg1)->brightness);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_contrast_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_contrast_set(self,contrast);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_contrast_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VPORT_contrast_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->contrast = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_contrast_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_contrast_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_contrast_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_BD) ((arg1)->contrast);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_ambient_color_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_ambient_color_set(self,ambient_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_ambient_color_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VPORT_ambient_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->ambient_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_ambient_color_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_ambient_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_ambient_color_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->ambient_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_lower_left_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_lower_left_set(self,lower_left);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_lower_left_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VPORT_lower_left_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->lower_left = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_lower_left_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_lower_left_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_lower_left_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->lower_left);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_upper_right_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_upper_right_set(self,upper_right);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_upper_right_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VPORT_upper_right_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->upper_right = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_upper_right_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_upper_right_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_upper_right_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->upper_right);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_UCSFOLLOW_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_UCSFOLLOW_set(self,UCSFOLLOW);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_UCSFOLLOW_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VPORT_UCSFOLLOW_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->UCSFOLLOW = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_UCSFOLLOW_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_UCSFOLLOW_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_UCSFOLLOW_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_B) ((arg1)->UCSFOLLOW);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_circle_zoom_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_circle_zoom_set(self,circle_zoom);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_circle_zoom_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VPORT_circle_zoom_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->circle_zoom = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_circle_zoom_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_circle_zoom_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_circle_zoom_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_BS) ((arg1)->circle_zoom);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_FASTZOOM_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_FASTZOOM_set(self,FASTZOOM);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_FASTZOOM_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VPORT_FASTZOOM_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->FASTZOOM = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_FASTZOOM_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_FASTZOOM_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_FASTZOOM_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_B) ((arg1)->FASTZOOM);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_UCSICON_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_UCSICON_set(self,UCSICON);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_UCSICON_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VPORT_UCSICON_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->UCSICON = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_UCSICON_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_UCSICON_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_UCSICON_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_RC) ((arg1)->UCSICON);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_GRIDMODE_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_GRIDMODE_set(self,GRIDMODE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_GRIDMODE_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VPORT_GRIDMODE_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->GRIDMODE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_GRIDMODE_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_GRIDMODE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_GRIDMODE_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_B) ((arg1)->GRIDMODE);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_GRIDUNIT_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_GRIDUNIT_set(self,GRIDUNIT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_GRIDUNIT_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VPORT_GRIDUNIT_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->GRIDUNIT = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_GRIDUNIT_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_GRIDUNIT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_GRIDUNIT_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->GRIDUNIT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_SNAPMODE_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_SNAPMODE_set(self,SNAPMODE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_SNAPMODE_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VPORT_SNAPMODE_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->SNAPMODE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_SNAPMODE_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_SNAPMODE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_SNAPMODE_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_B) ((arg1)->SNAPMODE);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_SNAPSTYLE_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_SNAPSTYLE_set(self,SNAPSTYLE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_SNAPSTYLE_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VPORT_SNAPSTYLE_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->SNAPSTYLE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_SNAPSTYLE_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_SNAPSTYLE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_SNAPSTYLE_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_B) ((arg1)->SNAPSTYLE);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_SNAPISOPAIR_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_SNAPISOPAIR_set(self,SNAPISOPAIR);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_SNAPISOPAIR_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VPORT_SNAPISOPAIR_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->SNAPISOPAIR = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_SNAPISOPAIR_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_SNAPISOPAIR_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_SNAPISOPAIR_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_BS) ((arg1)->SNAPISOPAIR);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_SNAPANG_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_SNAPANG_set(self,SNAPANG);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_SNAPANG_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VPORT_SNAPANG_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->SNAPANG = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_SNAPANG_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_SNAPANG_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_SNAPANG_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_BD) ((arg1)->SNAPANG);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_SNAPBASE_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_SNAPBASE_set(self,SNAPBASE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_SNAPBASE_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VPORT_SNAPBASE_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->SNAPBASE = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_SNAPBASE_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_SNAPBASE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_SNAPBASE_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->SNAPBASE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_SNAPUNIT_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_SNAPUNIT_set(self,SNAPUNIT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_SNAPUNIT_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VPORT_SNAPUNIT_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->SNAPUNIT = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_SNAPUNIT_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_SNAPUNIT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_SNAPUNIT_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->SNAPUNIT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_ucs_at_origin_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_ucs_at_origin_set(self,ucs_at_origin);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_ucs_at_origin_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VPORT_ucs_at_origin_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->ucs_at_origin = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_ucs_at_origin_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_ucs_at_origin_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_ucs_at_origin_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_B) ((arg1)->ucs_at_origin);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_UCSVP_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_UCSVP_set(self,UCSVP);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_UCSVP_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VPORT_UCSVP_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->UCSVP = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_UCSVP_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_UCSVP_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_UCSVP_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_B) ((arg1)->UCSVP);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_ucsorg_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_ucsorg_set(self,ucsorg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_ucsorg_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VPORT_ucsorg_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ucsorg = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_ucsorg_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_ucsorg_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_ucsorg_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ucsorg);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_ucsxdir_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_ucsxdir_set(self,ucsxdir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_ucsxdir_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VPORT_ucsxdir_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ucsxdir = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_ucsxdir_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_ucsxdir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_ucsxdir_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ucsxdir);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_ucsydir_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_ucsydir_set(self,ucsydir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_ucsydir_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VPORT_ucsydir_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ucsydir = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_ucsydir_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_ucsydir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_ucsydir_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ucsydir);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_ucs_elevation_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_ucs_elevation_set(self,ucs_elevation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_ucs_elevation_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VPORT_ucs_elevation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->ucs_elevation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_ucs_elevation_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_ucs_elevation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_ucs_elevation_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_BD) ((arg1)->ucs_elevation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_UCSORTHOVIEW_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_UCSORTHOVIEW_set(self,UCSORTHOVIEW);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_UCSORTHOVIEW_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VPORT_UCSORTHOVIEW_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->UCSORTHOVIEW = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_UCSORTHOVIEW_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_UCSORTHOVIEW_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_UCSORTHOVIEW_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_BS) ((arg1)->UCSORTHOVIEW);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_grid_flags_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_grid_flags_set(self,grid_flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_grid_flags_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VPORT_grid_flags_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->grid_flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_grid_flags_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_grid_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_grid_flags_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_BS) ((arg1)->grid_flags);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_grid_major_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_grid_major_set(self,grid_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_grid_major_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VPORT_grid_major_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->grid_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_grid_major_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_grid_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_grid_major_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_BS) ((arg1)->grid_major);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_background_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_background_set(self,background);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_background_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VPORT_background_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->background = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_background_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_background_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_background_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_H) ((arg1)->background);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_visualstyle_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_visualstyle_set(self,visualstyle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_visualstyle_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VPORT_visualstyle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->visualstyle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_visualstyle_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_visualstyle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_visualstyle_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_H) ((arg1)->visualstyle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_sun_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_sun_set(self,sun);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_sun_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VPORT_sun_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->sun = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_sun_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_sun_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_sun_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_H) ((arg1)->sun);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_named_ucs_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_named_ucs_set(self,named_ucs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_named_ucs_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VPORT_named_ucs_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->named_ucs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_named_ucs_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_named_ucs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_named_ucs_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_H) ((arg1)->named_ucs);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_base_ucs_set) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_base_ucs_set(self,base_ucs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_base_ucs_set" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VPORT_base_ucs_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->base_ucs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VPORT_base_ucs_get) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VPORT_base_ucs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VPORT_base_ucs_get" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    result = (BITCODE_H) ((arg1)->base_ucs);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_VPORT) {
  {
    int argvi = 0;
    struct _dwg_object_VPORT *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_VPORT();");
    }
    result = (struct _dwg_object_VPORT *)calloc(1, sizeof(struct _dwg_object_VPORT));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_VPORT, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_VPORT) {
  {
    struct _dwg_object_VPORT *arg1 = (struct _dwg_object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_VPORT(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VPORT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_VPORT" "', argument " "1"" of type '" "struct _dwg_object_VPORT *""'"); 
    }
    arg1 = (struct _dwg_object_VPORT *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_APPID_CONTROL_parent_set) {
  {
    struct _dwg_object_APPID_CONTROL *arg1 = (struct _dwg_object_APPID_CONTROL *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_APPID_CONTROL_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_APPID_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_APPID_CONTROL_parent_set" "', argument " "1"" of type '" "struct _dwg_object_APPID_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_APPID_CONTROL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_APPID_CONTROL_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_APPID_CONTROL_parent_get) {
  {
    struct _dwg_object_APPID_CONTROL *arg1 = (struct _dwg_object_APPID_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_APPID_CONTROL_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_APPID_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_APPID_CONTROL_parent_get" "', argument " "1"" of type '" "struct _dwg_object_APPID_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_APPID_CONTROL *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_APPID_CONTROL_num_entries_set) {
  {
    struct _dwg_object_APPID_CONTROL *arg1 = (struct _dwg_object_APPID_CONTROL *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_APPID_CONTROL_num_entries_set(self,num_entries);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_APPID_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_APPID_CONTROL_num_entries_set" "', argument " "1"" of type '" "struct _dwg_object_APPID_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_APPID_CONTROL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_APPID_CONTROL_num_entries_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_entries = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_APPID_CONTROL_num_entries_get) {
  {
    struct _dwg_object_APPID_CONTROL *arg1 = (struct _dwg_object_APPID_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_APPID_CONTROL_num_entries_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_APPID_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_APPID_CONTROL_num_entries_get" "', argument " "1"" of type '" "struct _dwg_object_APPID_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_APPID_CONTROL *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_entries);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_APPID_CONTROL_entries_set) {
  {
    struct _dwg_object_APPID_CONTROL *arg1 = (struct _dwg_object_APPID_CONTROL *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_APPID_CONTROL_entries_set(self,entries);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_APPID_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_APPID_CONTROL_entries_set" "', argument " "1"" of type '" "struct _dwg_object_APPID_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_APPID_CONTROL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_APPID_CONTROL_entries_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->entries = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_APPID_CONTROL_entries_get) {
  {
    struct _dwg_object_APPID_CONTROL *arg1 = (struct _dwg_object_APPID_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_APPID_CONTROL_entries_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_APPID_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_APPID_CONTROL_entries_get" "', argument " "1"" of type '" "struct _dwg_object_APPID_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_APPID_CONTROL *)(argp1);
    result = (BITCODE_H *) ((arg1)->entries);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_APPID_CONTROL_flags_r11_set) {
  {
    struct _dwg_object_APPID_CONTROL *arg1 = (struct _dwg_object_APPID_CONTROL *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_APPID_CONTROL_flags_r11_set(self,flags_r11);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_APPID_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_APPID_CONTROL_flags_r11_set" "', argument " "1"" of type '" "struct _dwg_object_APPID_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_APPID_CONTROL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_APPID_CONTROL_flags_r11_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->flags_r11 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_APPID_CONTROL_flags_r11_get) {
  {
    struct _dwg_object_APPID_CONTROL *arg1 = (struct _dwg_object_APPID_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_APPID_CONTROL_flags_r11_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_APPID_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_APPID_CONTROL_flags_r11_get" "', argument " "1"" of type '" "struct _dwg_object_APPID_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_APPID_CONTROL *)(argp1);
    result = (BITCODE_RS) ((arg1)->flags_r11);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_APPID_CONTROL) {
  {
    int argvi = 0;
    struct _dwg_object_APPID_CONTROL *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_APPID_CONTROL();");
    }
    result = (struct _dwg_object_APPID_CONTROL *)calloc(1, sizeof(struct _dwg_object_APPID_CONTROL));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_APPID_CONTROL, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_APPID_CONTROL) {
  {
    struct _dwg_object_APPID_CONTROL *arg1 = (struct _dwg_object_APPID_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_APPID_CONTROL(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_APPID_CONTROL, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_APPID_CONTROL" "', argument " "1"" of type '" "struct _dwg_object_APPID_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_APPID_CONTROL *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_APPID_parent_set) {
  {
    struct _dwg_object_APPID *arg1 = (struct _dwg_object_APPID *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_APPID_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_APPID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_APPID_parent_set" "', argument " "1"" of type '" "struct _dwg_object_APPID *""'"); 
    }
    arg1 = (struct _dwg_object_APPID *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_APPID_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_APPID_parent_get) {
  {
    struct _dwg_object_APPID *arg1 = (struct _dwg_object_APPID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_APPID_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_APPID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_APPID_parent_get" "', argument " "1"" of type '" "struct _dwg_object_APPID *""'"); 
    }
    arg1 = (struct _dwg_object_APPID *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_APPID_flag_set) {
  {
    struct _dwg_object_APPID *arg1 = (struct _dwg_object_APPID *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_APPID_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_APPID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_APPID_flag_set" "', argument " "1"" of type '" "struct _dwg_object_APPID *""'"); 
    }
    arg1 = (struct _dwg_object_APPID *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_APPID_flag_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_APPID_flag_get) {
  {
    struct _dwg_object_APPID *arg1 = (struct _dwg_object_APPID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_APPID_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_APPID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_APPID_flag_get" "', argument " "1"" of type '" "struct _dwg_object_APPID *""'"); 
    }
    arg1 = (struct _dwg_object_APPID *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_APPID_name_set) {
  {
    struct _dwg_object_APPID *arg1 = (struct _dwg_object_APPID *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_APPID_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_APPID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_APPID_name_set" "', argument " "1"" of type '" "struct _dwg_object_APPID *""'"); 
    }
    arg1 = (struct _dwg_object_APPID *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_APPID_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_APPID_name_get) {
  {
    struct _dwg_object_APPID *arg1 = (struct _dwg_object_APPID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_APPID_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_APPID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_APPID_name_get" "', argument " "1"" of type '" "struct _dwg_object_APPID *""'"); 
    }
    arg1 = (struct _dwg_object_APPID *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_APPID_used_set) {
  {
    struct _dwg_object_APPID *arg1 = (struct _dwg_object_APPID *) 0 ;
    BITCODE_RSd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_APPID_used_set(self,used);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_APPID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_APPID_used_set" "', argument " "1"" of type '" "struct _dwg_object_APPID *""'"); 
    }
    arg1 = (struct _dwg_object_APPID *)(argp1);
    ecode2 = SWIG_AsVal_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_APPID_used_set" "', argument " "2"" of type '" "BITCODE_RSd""'");
    } 
    arg2 = (BITCODE_RSd)(val2);
    if (arg1) (arg1)->used = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_APPID_used_get) {
  {
    struct _dwg_object_APPID *arg1 = (struct _dwg_object_APPID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RSd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_APPID_used_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_APPID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_APPID_used_get" "', argument " "1"" of type '" "struct _dwg_object_APPID *""'"); 
    }
    arg1 = (struct _dwg_object_APPID *)(argp1);
    result = (BITCODE_RSd) ((arg1)->used);
    ST(argvi) = SWIG_From_short  SWIG_PERL_CALL_ARGS_1((short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_APPID_is_xref_ref_set) {
  {
    struct _dwg_object_APPID *arg1 = (struct _dwg_object_APPID *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_APPID_is_xref_ref_set(self,is_xref_ref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_APPID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_APPID_is_xref_ref_set" "', argument " "1"" of type '" "struct _dwg_object_APPID *""'"); 
    }
    arg1 = (struct _dwg_object_APPID *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_APPID_is_xref_ref_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_xref_ref = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_APPID_is_xref_ref_get) {
  {
    struct _dwg_object_APPID *arg1 = (struct _dwg_object_APPID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_APPID_is_xref_ref_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_APPID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_APPID_is_xref_ref_get" "', argument " "1"" of type '" "struct _dwg_object_APPID *""'"); 
    }
    arg1 = (struct _dwg_object_APPID *)(argp1);
    result = (BITCODE_B) ((arg1)->is_xref_ref);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_APPID_is_xref_resolved_set) {
  {
    struct _dwg_object_APPID *arg1 = (struct _dwg_object_APPID *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_APPID_is_xref_resolved_set(self,is_xref_resolved);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_APPID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_APPID_is_xref_resolved_set" "', argument " "1"" of type '" "struct _dwg_object_APPID *""'"); 
    }
    arg1 = (struct _dwg_object_APPID *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_APPID_is_xref_resolved_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->is_xref_resolved = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_APPID_is_xref_resolved_get) {
  {
    struct _dwg_object_APPID *arg1 = (struct _dwg_object_APPID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_APPID_is_xref_resolved_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_APPID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_APPID_is_xref_resolved_get" "', argument " "1"" of type '" "struct _dwg_object_APPID *""'"); 
    }
    arg1 = (struct _dwg_object_APPID *)(argp1);
    result = (BITCODE_BS) ((arg1)->is_xref_resolved);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_APPID_is_xref_dep_set) {
  {
    struct _dwg_object_APPID *arg1 = (struct _dwg_object_APPID *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_APPID_is_xref_dep_set(self,is_xref_dep);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_APPID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_APPID_is_xref_dep_set" "', argument " "1"" of type '" "struct _dwg_object_APPID *""'"); 
    }
    arg1 = (struct _dwg_object_APPID *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_APPID_is_xref_dep_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_xref_dep = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_APPID_is_xref_dep_get) {
  {
    struct _dwg_object_APPID *arg1 = (struct _dwg_object_APPID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_APPID_is_xref_dep_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_APPID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_APPID_is_xref_dep_get" "', argument " "1"" of type '" "struct _dwg_object_APPID *""'"); 
    }
    arg1 = (struct _dwg_object_APPID *)(argp1);
    result = (BITCODE_B) ((arg1)->is_xref_dep);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_APPID_xref_set) {
  {
    struct _dwg_object_APPID *arg1 = (struct _dwg_object_APPID *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_APPID_xref_set(self,xref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_APPID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_APPID_xref_set" "', argument " "1"" of type '" "struct _dwg_object_APPID *""'"); 
    }
    arg1 = (struct _dwg_object_APPID *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_APPID_xref_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->xref = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_APPID_xref_get) {
  {
    struct _dwg_object_APPID *arg1 = (struct _dwg_object_APPID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_APPID_xref_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_APPID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_APPID_xref_get" "', argument " "1"" of type '" "struct _dwg_object_APPID *""'"); 
    }
    arg1 = (struct _dwg_object_APPID *)(argp1);
    result = (BITCODE_H) ((arg1)->xref);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_APPID_unknown_set) {
  {
    struct _dwg_object_APPID *arg1 = (struct _dwg_object_APPID *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_APPID_unknown_set(self,unknown);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_APPID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_APPID_unknown_set" "', argument " "1"" of type '" "struct _dwg_object_APPID *""'"); 
    }
    arg1 = (struct _dwg_object_APPID *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_APPID_unknown_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->unknown = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_APPID_unknown_get) {
  {
    struct _dwg_object_APPID *arg1 = (struct _dwg_object_APPID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_APPID_unknown_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_APPID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_APPID_unknown_get" "', argument " "1"" of type '" "struct _dwg_object_APPID *""'"); 
    }
    arg1 = (struct _dwg_object_APPID *)(argp1);
    result = (BITCODE_RC) ((arg1)->unknown);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_APPID) {
  {
    int argvi = 0;
    struct _dwg_object_APPID *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_APPID();");
    }
    result = (struct _dwg_object_APPID *)calloc(1, sizeof(struct _dwg_object_APPID));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_APPID, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_APPID) {
  {
    struct _dwg_object_APPID *arg1 = (struct _dwg_object_APPID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_APPID(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_APPID, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_APPID" "', argument " "1"" of type '" "struct _dwg_object_APPID *""'"); 
    }
    arg1 = (struct _dwg_object_APPID *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_CONTROL_parent_set) {
  {
    struct _dwg_object_DIMSTYLE_CONTROL *arg1 = (struct _dwg_object_DIMSTYLE_CONTROL *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_CONTROL_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_CONTROL_parent_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE_CONTROL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DIMSTYLE_CONTROL_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_CONTROL_parent_get) {
  {
    struct _dwg_object_DIMSTYLE_CONTROL *arg1 = (struct _dwg_object_DIMSTYLE_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_CONTROL_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_CONTROL_parent_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE_CONTROL *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_CONTROL_num_entries_set) {
  {
    struct _dwg_object_DIMSTYLE_CONTROL *arg1 = (struct _dwg_object_DIMSTYLE_CONTROL *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_CONTROL_num_entries_set(self,num_entries);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_CONTROL_num_entries_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE_CONTROL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_CONTROL_num_entries_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_entries = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_CONTROL_num_entries_get) {
  {
    struct _dwg_object_DIMSTYLE_CONTROL *arg1 = (struct _dwg_object_DIMSTYLE_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_CONTROL_num_entries_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_CONTROL_num_entries_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE_CONTROL *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_entries);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_CONTROL_entries_set) {
  {
    struct _dwg_object_DIMSTYLE_CONTROL *arg1 = (struct _dwg_object_DIMSTYLE_CONTROL *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_CONTROL_entries_set(self,entries);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_CONTROL_entries_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE_CONTROL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DIMSTYLE_CONTROL_entries_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->entries = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_CONTROL_entries_get) {
  {
    struct _dwg_object_DIMSTYLE_CONTROL *arg1 = (struct _dwg_object_DIMSTYLE_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_CONTROL_entries_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_CONTROL_entries_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE_CONTROL *)(argp1);
    result = (BITCODE_H *) ((arg1)->entries);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_CONTROL_flags_r11_set) {
  {
    struct _dwg_object_DIMSTYLE_CONTROL *arg1 = (struct _dwg_object_DIMSTYLE_CONTROL *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_CONTROL_flags_r11_set(self,flags_r11);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_CONTROL_flags_r11_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE_CONTROL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_CONTROL_flags_r11_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->flags_r11 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_CONTROL_flags_r11_get) {
  {
    struct _dwg_object_DIMSTYLE_CONTROL *arg1 = (struct _dwg_object_DIMSTYLE_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_CONTROL_flags_r11_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_CONTROL_flags_r11_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE_CONTROL *)(argp1);
    result = (BITCODE_RS) ((arg1)->flags_r11);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_CONTROL_num_morehandles_set) {
  {
    struct _dwg_object_DIMSTYLE_CONTROL *arg1 = (struct _dwg_object_DIMSTYLE_CONTROL *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_CONTROL_num_morehandles_set(self,num_morehandles);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_CONTROL_num_morehandles_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE_CONTROL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_CONTROL_num_morehandles_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->num_morehandles = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_CONTROL_num_morehandles_get) {
  {
    struct _dwg_object_DIMSTYLE_CONTROL *arg1 = (struct _dwg_object_DIMSTYLE_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_CONTROL_num_morehandles_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_CONTROL_num_morehandles_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE_CONTROL *)(argp1);
    result = (BITCODE_RC) ((arg1)->num_morehandles);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_CONTROL_morehandles_set) {
  {
    struct _dwg_object_DIMSTYLE_CONTROL *arg1 = (struct _dwg_object_DIMSTYLE_CONTROL *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_CONTROL_morehandles_set(self,morehandles);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_CONTROL_morehandles_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE_CONTROL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DIMSTYLE_CONTROL_morehandles_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->morehandles = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_CONTROL_morehandles_get) {
  {
    struct _dwg_object_DIMSTYLE_CONTROL *arg1 = (struct _dwg_object_DIMSTYLE_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_CONTROL_morehandles_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_CONTROL_morehandles_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE_CONTROL *)(argp1);
    result = (BITCODE_H *) ((arg1)->morehandles);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_DIMSTYLE_CONTROL) {
  {
    int argvi = 0;
    struct _dwg_object_DIMSTYLE_CONTROL *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_DIMSTYLE_CONTROL();");
    }
    result = (struct _dwg_object_DIMSTYLE_CONTROL *)calloc(1, sizeof(struct _dwg_object_DIMSTYLE_CONTROL));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DIMSTYLE_CONTROL, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_DIMSTYLE_CONTROL) {
  {
    struct _dwg_object_DIMSTYLE_CONTROL *arg1 = (struct _dwg_object_DIMSTYLE_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_DIMSTYLE_CONTROL(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE_CONTROL, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_DIMSTYLE_CONTROL" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE_CONTROL *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_parent_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_parent_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DIMSTYLE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_parent_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_parent_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_flag_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_flag_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_flag_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_flag_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_flag_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_name_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_name_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DIMSTYLE_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_name_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_name_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_used_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_RSd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_used_set(self,used);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_used_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_used_set" "', argument " "2"" of type '" "BITCODE_RSd""'");
    } 
    arg2 = (BITCODE_RSd)(val2);
    if (arg1) (arg1)->used = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_used_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RSd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_used_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_used_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_RSd) ((arg1)->used);
    ST(argvi) = SWIG_From_short  SWIG_PERL_CALL_ARGS_1((short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_is_xref_ref_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_is_xref_ref_set(self,is_xref_ref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_is_xref_ref_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_is_xref_ref_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_xref_ref = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_is_xref_ref_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_is_xref_ref_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_is_xref_ref_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->is_xref_ref);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_is_xref_resolved_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_is_xref_resolved_set(self,is_xref_resolved);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_is_xref_resolved_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_is_xref_resolved_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->is_xref_resolved = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_is_xref_resolved_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_is_xref_resolved_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_is_xref_resolved_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->is_xref_resolved);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_is_xref_dep_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_is_xref_dep_set(self,is_xref_dep);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_is_xref_dep_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_is_xref_dep_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_xref_dep = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_is_xref_dep_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_is_xref_dep_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_is_xref_dep_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->is_xref_dep);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_xref_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_xref_set(self,xref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_xref_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DIMSTYLE_xref_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->xref = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_xref_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_xref_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_xref_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_H) ((arg1)->xref);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTOL_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTOL_set(self,DIMTOL);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTOL_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMTOL_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DIMTOL = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTOL_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTOL_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTOL_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->DIMTOL);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMLIM_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMLIM_set(self,DIMLIM);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMLIM_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMLIM_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DIMLIM = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMLIM_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMLIM_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMLIM_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->DIMLIM);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTIH_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTIH_set(self,DIMTIH);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTIH_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMTIH_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DIMTIH = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTIH_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTIH_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTIH_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->DIMTIH);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTOH_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTOH_set(self,DIMTOH);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTOH_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMTOH_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DIMTOH = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTOH_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTOH_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTOH_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->DIMTOH);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMSE1_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMSE1_set(self,DIMSE1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMSE1_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMSE1_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DIMSE1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMSE1_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMSE1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMSE1_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->DIMSE1);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMSE2_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMSE2_set(self,DIMSE2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMSE2_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMSE2_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DIMSE2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMSE2_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMSE2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMSE2_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->DIMSE2);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMALT_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMALT_set(self,DIMALT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMALT_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMALT_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DIMALT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMALT_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMALT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMALT_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->DIMALT);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTOFL_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTOFL_set(self,DIMTOFL);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTOFL_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMTOFL_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DIMTOFL = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTOFL_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTOFL_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTOFL_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->DIMTOFL);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMSAH_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMSAH_set(self,DIMSAH);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMSAH_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMSAH_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DIMSAH = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMSAH_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMSAH_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMSAH_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->DIMSAH);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTIX_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTIX_set(self,DIMTIX);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTIX_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMTIX_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DIMTIX = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTIX_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTIX_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTIX_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->DIMTIX);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMSOXD_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMSOXD_set(self,DIMSOXD);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMSOXD_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMSOXD_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DIMSOXD = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMSOXD_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMSOXD_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMSOXD_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->DIMSOXD);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMALTD_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMALTD_set(self,DIMALTD);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMALTD_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMALTD_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMALTD = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMALTD_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMALTD_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMALTD_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMALTD);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMZIN_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMZIN_set(self,DIMZIN);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMZIN_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMZIN_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMZIN = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMZIN_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMZIN_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMZIN_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMZIN);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMSD1_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMSD1_set(self,DIMSD1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMSD1_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMSD1_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DIMSD1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMSD1_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMSD1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMSD1_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->DIMSD1);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMSD2_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMSD2_set(self,DIMSD2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMSD2_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMSD2_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DIMSD2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMSD2_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMSD2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMSD2_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->DIMSD2);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTOLJ_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTOLJ_set(self,DIMTOLJ);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTOLJ_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMTOLJ_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMTOLJ = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTOLJ_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTOLJ_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTOLJ_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMTOLJ);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMJUST_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMJUST_set(self,DIMJUST);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMJUST_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMJUST_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMJUST = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMJUST_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMJUST_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMJUST_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMJUST);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMFIT_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMFIT_set(self,DIMFIT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMFIT_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMFIT_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMFIT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMFIT_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMFIT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMFIT_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMFIT);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMUPT_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMUPT_set(self,DIMUPT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMUPT_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMUPT_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DIMUPT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMUPT_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMUPT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMUPT_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->DIMUPT);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTZIN_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTZIN_set(self,DIMTZIN);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTZIN_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMTZIN_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMTZIN = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTZIN_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTZIN_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTZIN_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMTZIN);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMALTZ_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMALTZ_set(self,DIMALTZ);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMALTZ_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMALTZ_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMALTZ = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMALTZ_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMALTZ_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMALTZ_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMALTZ);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMALTTZ_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMALTTZ_set(self,DIMALTTZ);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMALTTZ_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMALTTZ_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMALTTZ = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMALTTZ_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMALTTZ_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMALTTZ_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMALTTZ);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTAD_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTAD_set(self,DIMTAD);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTAD_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMTAD_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMTAD = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTAD_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTAD_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTAD_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMTAD);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMUNIT_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMUNIT_set(self,DIMUNIT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMUNIT_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMUNIT_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMUNIT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMUNIT_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMUNIT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMUNIT_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMUNIT);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMAUNIT_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMAUNIT_set(self,DIMAUNIT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMAUNIT_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMAUNIT_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMAUNIT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMAUNIT_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMAUNIT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMAUNIT_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMAUNIT);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMDEC_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMDEC_set(self,DIMDEC);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMDEC_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMDEC_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMDEC = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMDEC_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMDEC_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMDEC_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMDEC);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTDEC_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTDEC_set(self,DIMTDEC);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTDEC_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMTDEC_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMTDEC = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTDEC_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTDEC_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTDEC_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMTDEC);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMALTU_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMALTU_set(self,DIMALTU);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMALTU_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMALTU_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMALTU = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMALTU_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMALTU_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMALTU_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMALTU);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMALTTD_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMALTTD_set(self,DIMALTTD);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMALTTD_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMALTTD_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMALTTD = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMALTTD_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMALTTD_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMALTTD_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMALTTD);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMSCALE_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMSCALE_set(self,DIMSCALE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMSCALE_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMSCALE_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMSCALE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMSCALE_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMSCALE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMSCALE_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMSCALE);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMASZ_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMASZ_set(self,DIMASZ);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMASZ_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMASZ_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMASZ = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMASZ_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMASZ_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMASZ_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMASZ);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMEXO_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMEXO_set(self,DIMEXO);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMEXO_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMEXO_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMEXO = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMEXO_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMEXO_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMEXO_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMEXO);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMDLI_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMDLI_set(self,DIMDLI);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMDLI_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMDLI_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMDLI = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMDLI_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMDLI_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMDLI_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMDLI);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMEXE_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMEXE_set(self,DIMEXE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMEXE_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMEXE_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMEXE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMEXE_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMEXE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMEXE_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMEXE);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMRND_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMRND_set(self,DIMRND);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMRND_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMRND_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMRND = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMRND_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMRND_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMRND_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMRND);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMDLE_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMDLE_set(self,DIMDLE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMDLE_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMDLE_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMDLE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMDLE_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMDLE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMDLE_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMDLE);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTP_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTP_set(self,DIMTP);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTP_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMTP_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMTP = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTP_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTP_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTP_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMTP);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTM_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTM_set(self,DIMTM);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTM_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMTM_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMTM = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTM_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTM_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTM_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMTM);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMFXL_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMFXL_set(self,DIMFXL);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMFXL_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMFXL_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMFXL = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMFXL_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMFXL_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMFXL_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMFXL);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMJOGANG_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMJOGANG_set(self,DIMJOGANG);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMJOGANG_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMJOGANG_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMJOGANG = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMJOGANG_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMJOGANG_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMJOGANG_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMJOGANG);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTFILL_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTFILL_set(self,DIMTFILL);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTFILL_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMTFILL_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMTFILL = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTFILL_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTFILL_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTFILL_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMTFILL);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTFILLCLR_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTFILLCLR_set(self,DIMTFILLCLR);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTFILLCLR_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DIMSTYLE_DIMTFILLCLR_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->DIMTFILLCLR = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTFILLCLR_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTFILLCLR_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTFILLCLR_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->DIMTFILLCLR);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMAZIN_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMAZIN_set(self,DIMAZIN);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMAZIN_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMAZIN_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMAZIN = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMAZIN_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMAZIN_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMAZIN_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMAZIN);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMARCSYM_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMARCSYM_set(self,DIMARCSYM);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMARCSYM_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMARCSYM_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMARCSYM = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMARCSYM_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMARCSYM_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMARCSYM_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMARCSYM);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTXT_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTXT_set(self,DIMTXT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTXT_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMTXT_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMTXT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTXT_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTXT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTXT_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMTXT);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMCEN_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMCEN_set(self,DIMCEN);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMCEN_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMCEN_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMCEN = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMCEN_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMCEN_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMCEN_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMCEN);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTSZ_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTSZ_set(self,DIMTSZ);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTSZ_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMTSZ_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMTSZ = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTSZ_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTSZ_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTSZ_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMTSZ);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMALTF_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMALTF_set(self,DIMALTF);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMALTF_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMALTF_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMALTF = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMALTF_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMALTF_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMALTF_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMALTF);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMLFAC_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMLFAC_set(self,DIMLFAC);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMLFAC_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMLFAC_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMLFAC = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMLFAC_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMLFAC_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMLFAC_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMLFAC);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTVP_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTVP_set(self,DIMTVP);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTVP_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMTVP_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMTVP = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTVP_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTVP_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTVP_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMTVP);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTFAC_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTFAC_set(self,DIMTFAC);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTFAC_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMTFAC_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMTFAC = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTFAC_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTFAC_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTFAC_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMTFAC);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMGAP_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMGAP_set(self,DIMGAP);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMGAP_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMGAP_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMGAP = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMGAP_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMGAP_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMGAP_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMGAP);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMPOST_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMPOST_set(self,DIMPOST);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMPOST_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DIMSTYLE_DIMPOST_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->DIMPOST);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->DIMPOST = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->DIMPOST = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMPOST_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMPOST_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMPOST_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_TV) ((arg1)->DIMPOST);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMAPOST_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMAPOST_set(self,DIMAPOST);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMAPOST_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DIMSTYLE_DIMAPOST_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->DIMAPOST);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->DIMAPOST = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->DIMAPOST = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMAPOST_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMAPOST_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMAPOST_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_TV) ((arg1)->DIMAPOST);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMBLK_T_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMBLK_T_set(self,DIMBLK_T);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMBLK_T_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DIMSTYLE_DIMBLK_T_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->DIMBLK_T);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->DIMBLK_T = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->DIMBLK_T = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMBLK_T_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMBLK_T_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMBLK_T_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_TV) ((arg1)->DIMBLK_T);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMBLK1_T_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMBLK1_T_set(self,DIMBLK1_T);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMBLK1_T_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DIMSTYLE_DIMBLK1_T_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->DIMBLK1_T);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->DIMBLK1_T = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->DIMBLK1_T = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMBLK1_T_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMBLK1_T_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMBLK1_T_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_TV) ((arg1)->DIMBLK1_T);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMBLK2_T_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMBLK2_T_set(self,DIMBLK2_T);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMBLK2_T_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DIMSTYLE_DIMBLK2_T_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->DIMBLK2_T);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->DIMBLK2_T = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->DIMBLK2_T = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMBLK2_T_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMBLK2_T_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMBLK2_T_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_TV) ((arg1)->DIMBLK2_T);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMALTRND_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMALTRND_set(self,DIMALTRND);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMALTRND_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMALTRND_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMALTRND = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMALTRND_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMALTRND_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMALTRND_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMALTRND);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMCLRD_N_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMCLRD_N_set(self,DIMCLRD_N);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMCLRD_N_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMCLRD_N_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->DIMCLRD_N = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMCLRD_N_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMCLRD_N_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMCLRD_N_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_RS) ((arg1)->DIMCLRD_N);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMCLRE_N_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMCLRE_N_set(self,DIMCLRE_N);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMCLRE_N_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMCLRE_N_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->DIMCLRE_N = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMCLRE_N_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMCLRE_N_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMCLRE_N_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_RS) ((arg1)->DIMCLRE_N);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMCLRT_N_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMCLRT_N_set(self,DIMCLRT_N);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMCLRT_N_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMCLRT_N_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->DIMCLRT_N = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMCLRT_N_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMCLRT_N_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMCLRT_N_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_RS) ((arg1)->DIMCLRT_N);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMCLRD_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMCLRD_set(self,DIMCLRD);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMCLRD_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DIMSTYLE_DIMCLRD_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->DIMCLRD = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMCLRD_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMCLRD_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMCLRD_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->DIMCLRD);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMCLRE_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMCLRE_set(self,DIMCLRE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMCLRE_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DIMSTYLE_DIMCLRE_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->DIMCLRE = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMCLRE_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMCLRE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMCLRE_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->DIMCLRE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMCLRT_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMCLRT_set(self,DIMCLRT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMCLRT_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DIMSTYLE_DIMCLRT_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->DIMCLRT = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMCLRT_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMCLRT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMCLRT_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->DIMCLRT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMADEC_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMADEC_set(self,DIMADEC);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMADEC_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMADEC_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMADEC = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMADEC_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMADEC_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMADEC_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMADEC);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMFRAC_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMFRAC_set(self,DIMFRAC);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMFRAC_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMFRAC_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMFRAC = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMFRAC_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMFRAC_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMFRAC_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMFRAC);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMLUNIT_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMLUNIT_set(self,DIMLUNIT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMLUNIT_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMLUNIT_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMLUNIT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMLUNIT_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMLUNIT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMLUNIT_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMLUNIT);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMDSEP_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMDSEP_set(self,DIMDSEP);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMDSEP_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMDSEP_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMDSEP = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMDSEP_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMDSEP_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMDSEP_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMDSEP);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTMOVE_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTMOVE_set(self,DIMTMOVE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTMOVE_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMTMOVE_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMTMOVE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTMOVE_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTMOVE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTMOVE_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMTMOVE);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMATFIT_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMATFIT_set(self,DIMATFIT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMATFIT_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMATFIT_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->DIMATFIT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMATFIT_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMATFIT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMATFIT_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->DIMATFIT);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMFXLON_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMFXLON_set(self,DIMFXLON);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMFXLON_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMFXLON_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DIMFXLON = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMFXLON_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMFXLON_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMFXLON_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->DIMFXLON);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTXTDIRECTION_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTXTDIRECTION_set(self,DIMTXTDIRECTION);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTXTDIRECTION_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMTXTDIRECTION_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->DIMTXTDIRECTION = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTXTDIRECTION_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTXTDIRECTION_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTXTDIRECTION_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->DIMTXTDIRECTION);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMALTMZF_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMALTMZF_set(self,DIMALTMZF);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMALTMZF_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMALTMZF_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMALTMZF = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMALTMZF_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMALTMZF_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMALTMZF_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMALTMZF);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMALTMZS_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMALTMZS_set(self,DIMALTMZS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMALTMZS_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DIMSTYLE_DIMALTMZS_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->DIMALTMZS);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->DIMALTMZS = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->DIMALTMZS = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMALTMZS_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMALTMZS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMALTMZS_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_TV) ((arg1)->DIMALTMZS);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMMZF_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMMZF_set(self,DIMMZF);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMMZF_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMMZF_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->DIMMZF = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMMZF_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMMZF_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMMZF_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->DIMMZF);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMMZS_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMMZS_set(self,DIMMZS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMMZS_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DIMSTYLE_DIMMZS_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->DIMMZS);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->DIMMZS = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->DIMMZS = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMMZS_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMMZS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMMZS_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_TV) ((arg1)->DIMMZS);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMLWD_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BSd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMLWD_set(self,DIMLWD);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMLWD_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMLWD_set" "', argument " "2"" of type '" "BITCODE_BSd""'");
    } 
    arg2 = (BITCODE_BSd)(val2);
    if (arg1) (arg1)->DIMLWD = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMLWD_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BSd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMLWD_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMLWD_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BSd) ((arg1)->DIMLWD);
    ST(argvi) = SWIG_From_short  SWIG_PERL_CALL_ARGS_1((short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMLWE_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_BSd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMLWE_set(self,DIMLWE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMLWE_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_DIMLWE_set" "', argument " "2"" of type '" "BITCODE_BSd""'");
    } 
    arg2 = (BITCODE_BSd)(val2);
    if (arg1) (arg1)->DIMLWE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMLWE_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BSd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMLWE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMLWE_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_BSd) ((arg1)->DIMLWE);
    ST(argvi) = SWIG_From_short  SWIG_PERL_CALL_ARGS_1((short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_flag0_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_flag0_set(self,flag0);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_flag0_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMSTYLE_flag0_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->flag0 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_flag0_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_flag0_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_flag0_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->flag0);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTXSTY_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTXSTY_set(self,DIMTXSTY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTXSTY_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DIMSTYLE_DIMTXSTY_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->DIMTXSTY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMTXSTY_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMTXSTY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMTXSTY_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_H) ((arg1)->DIMTXSTY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMLDRBLK_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMLDRBLK_set(self,DIMLDRBLK);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMLDRBLK_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DIMSTYLE_DIMLDRBLK_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->DIMLDRBLK = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMLDRBLK_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMLDRBLK_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMLDRBLK_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_H) ((arg1)->DIMLDRBLK);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMBLK_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMBLK_set(self,DIMBLK);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMBLK_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DIMSTYLE_DIMBLK_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->DIMBLK = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMBLK_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMBLK_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMBLK_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_H) ((arg1)->DIMBLK);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMBLK1_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMBLK1_set(self,DIMBLK1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMBLK1_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DIMSTYLE_DIMBLK1_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->DIMBLK1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMBLK1_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMBLK1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMBLK1_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_H) ((arg1)->DIMBLK1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMBLK2_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMBLK2_set(self,DIMBLK2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMBLK2_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DIMSTYLE_DIMBLK2_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->DIMBLK2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMBLK2_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMBLK2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMBLK2_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_H) ((arg1)->DIMBLK2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMLTYPE_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMLTYPE_set(self,DIMLTYPE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMLTYPE_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DIMSTYLE_DIMLTYPE_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->DIMLTYPE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMLTYPE_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMLTYPE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMLTYPE_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_H) ((arg1)->DIMLTYPE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMLTEX1_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMLTEX1_set(self,DIMLTEX1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMLTEX1_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DIMSTYLE_DIMLTEX1_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->DIMLTEX1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMLTEX1_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMLTEX1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMLTEX1_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_H) ((arg1)->DIMLTEX1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMLTEX2_set) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMLTEX2_set(self,DIMLTEX2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMLTEX2_set" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DIMSTYLE_DIMLTEX2_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->DIMLTEX2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMSTYLE_DIMLTEX2_get) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMSTYLE_DIMLTEX2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMSTYLE_DIMLTEX2_get" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    result = (BITCODE_H) ((arg1)->DIMLTEX2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_DIMSTYLE) {
  {
    int argvi = 0;
    struct _dwg_object_DIMSTYLE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_DIMSTYLE();");
    }
    result = (struct _dwg_object_DIMSTYLE *)calloc(1, sizeof(struct _dwg_object_DIMSTYLE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DIMSTYLE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_DIMSTYLE) {
  {
    struct _dwg_object_DIMSTYLE *arg1 = (struct _dwg_object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_DIMSTYLE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMSTYLE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_DIMSTYLE" "', argument " "1"" of type '" "struct _dwg_object_DIMSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DIMSTYLE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_CONTROL_parent_set) {
  {
    struct _dwg_object_VX_CONTROL *arg1 = (struct _dwg_object_VX_CONTROL *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VX_CONTROL_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_CONTROL_parent_set" "', argument " "1"" of type '" "struct _dwg_object_VX_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_VX_CONTROL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VX_CONTROL_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_CONTROL_parent_get) {
  {
    struct _dwg_object_VX_CONTROL *arg1 = (struct _dwg_object_VX_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VX_CONTROL_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_CONTROL_parent_get" "', argument " "1"" of type '" "struct _dwg_object_VX_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_VX_CONTROL *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_CONTROL_num_entries_set) {
  {
    struct _dwg_object_VX_CONTROL *arg1 = (struct _dwg_object_VX_CONTROL *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VX_CONTROL_num_entries_set(self,num_entries);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_CONTROL_num_entries_set" "', argument " "1"" of type '" "struct _dwg_object_VX_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_VX_CONTROL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VX_CONTROL_num_entries_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_entries = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_CONTROL_num_entries_get) {
  {
    struct _dwg_object_VX_CONTROL *arg1 = (struct _dwg_object_VX_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VX_CONTROL_num_entries_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_CONTROL_num_entries_get" "', argument " "1"" of type '" "struct _dwg_object_VX_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_VX_CONTROL *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_entries);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_CONTROL_entries_set) {
  {
    struct _dwg_object_VX_CONTROL *arg1 = (struct _dwg_object_VX_CONTROL *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VX_CONTROL_entries_set(self,entries);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_CONTROL_entries_set" "', argument " "1"" of type '" "struct _dwg_object_VX_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_VX_CONTROL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VX_CONTROL_entries_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->entries = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_CONTROL_entries_get) {
  {
    struct _dwg_object_VX_CONTROL *arg1 = (struct _dwg_object_VX_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VX_CONTROL_entries_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_CONTROL_entries_get" "', argument " "1"" of type '" "struct _dwg_object_VX_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_VX_CONTROL *)(argp1);
    result = (BITCODE_H *) ((arg1)->entries);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_CONTROL_flags_r11_set) {
  {
    struct _dwg_object_VX_CONTROL *arg1 = (struct _dwg_object_VX_CONTROL *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VX_CONTROL_flags_r11_set(self,flags_r11);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_CONTROL_flags_r11_set" "', argument " "1"" of type '" "struct _dwg_object_VX_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_VX_CONTROL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VX_CONTROL_flags_r11_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->flags_r11 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_CONTROL_flags_r11_get) {
  {
    struct _dwg_object_VX_CONTROL *arg1 = (struct _dwg_object_VX_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VX_CONTROL_flags_r11_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_CONTROL_flags_r11_get" "', argument " "1"" of type '" "struct _dwg_object_VX_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_VX_CONTROL *)(argp1);
    result = (BITCODE_RS) ((arg1)->flags_r11);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_VX_CONTROL) {
  {
    int argvi = 0;
    struct _dwg_object_VX_CONTROL *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_VX_CONTROL();");
    }
    result = (struct _dwg_object_VX_CONTROL *)calloc(1, sizeof(struct _dwg_object_VX_CONTROL));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_VX_CONTROL, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_VX_CONTROL) {
  {
    struct _dwg_object_VX_CONTROL *arg1 = (struct _dwg_object_VX_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_VX_CONTROL(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_CONTROL, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_VX_CONTROL" "', argument " "1"" of type '" "struct _dwg_object_VX_CONTROL *""'"); 
    }
    arg1 = (struct _dwg_object_VX_CONTROL *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_TABLE_RECORD_parent_set) {
  {
    struct _dwg_object_VX_TABLE_RECORD *arg1 = (struct _dwg_object_VX_TABLE_RECORD *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VX_TABLE_RECORD_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_TABLE_RECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_TABLE_RECORD_parent_set" "', argument " "1"" of type '" "struct _dwg_object_VX_TABLE_RECORD *""'"); 
    }
    arg1 = (struct _dwg_object_VX_TABLE_RECORD *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VX_TABLE_RECORD_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_TABLE_RECORD_parent_get) {
  {
    struct _dwg_object_VX_TABLE_RECORD *arg1 = (struct _dwg_object_VX_TABLE_RECORD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VX_TABLE_RECORD_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_TABLE_RECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_TABLE_RECORD_parent_get" "', argument " "1"" of type '" "struct _dwg_object_VX_TABLE_RECORD *""'"); 
    }
    arg1 = (struct _dwg_object_VX_TABLE_RECORD *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_TABLE_RECORD_flag_set) {
  {
    struct _dwg_object_VX_TABLE_RECORD *arg1 = (struct _dwg_object_VX_TABLE_RECORD *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VX_TABLE_RECORD_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_TABLE_RECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_TABLE_RECORD_flag_set" "', argument " "1"" of type '" "struct _dwg_object_VX_TABLE_RECORD *""'"); 
    }
    arg1 = (struct _dwg_object_VX_TABLE_RECORD *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VX_TABLE_RECORD_flag_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_TABLE_RECORD_flag_get) {
  {
    struct _dwg_object_VX_TABLE_RECORD *arg1 = (struct _dwg_object_VX_TABLE_RECORD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VX_TABLE_RECORD_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_TABLE_RECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_TABLE_RECORD_flag_get" "', argument " "1"" of type '" "struct _dwg_object_VX_TABLE_RECORD *""'"); 
    }
    arg1 = (struct _dwg_object_VX_TABLE_RECORD *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_TABLE_RECORD_name_set) {
  {
    struct _dwg_object_VX_TABLE_RECORD *arg1 = (struct _dwg_object_VX_TABLE_RECORD *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VX_TABLE_RECORD_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_TABLE_RECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_TABLE_RECORD_name_set" "', argument " "1"" of type '" "struct _dwg_object_VX_TABLE_RECORD *""'"); 
    }
    arg1 = (struct _dwg_object_VX_TABLE_RECORD *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VX_TABLE_RECORD_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_TABLE_RECORD_name_get) {
  {
    struct _dwg_object_VX_TABLE_RECORD *arg1 = (struct _dwg_object_VX_TABLE_RECORD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VX_TABLE_RECORD_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_TABLE_RECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_TABLE_RECORD_name_get" "', argument " "1"" of type '" "struct _dwg_object_VX_TABLE_RECORD *""'"); 
    }
    arg1 = (struct _dwg_object_VX_TABLE_RECORD *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_TABLE_RECORD_used_set) {
  {
    struct _dwg_object_VX_TABLE_RECORD *arg1 = (struct _dwg_object_VX_TABLE_RECORD *) 0 ;
    BITCODE_RSd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VX_TABLE_RECORD_used_set(self,used);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_TABLE_RECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_TABLE_RECORD_used_set" "', argument " "1"" of type '" "struct _dwg_object_VX_TABLE_RECORD *""'"); 
    }
    arg1 = (struct _dwg_object_VX_TABLE_RECORD *)(argp1);
    ecode2 = SWIG_AsVal_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VX_TABLE_RECORD_used_set" "', argument " "2"" of type '" "BITCODE_RSd""'");
    } 
    arg2 = (BITCODE_RSd)(val2);
    if (arg1) (arg1)->used = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_TABLE_RECORD_used_get) {
  {
    struct _dwg_object_VX_TABLE_RECORD *arg1 = (struct _dwg_object_VX_TABLE_RECORD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RSd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VX_TABLE_RECORD_used_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_TABLE_RECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_TABLE_RECORD_used_get" "', argument " "1"" of type '" "struct _dwg_object_VX_TABLE_RECORD *""'"); 
    }
    arg1 = (struct _dwg_object_VX_TABLE_RECORD *)(argp1);
    result = (BITCODE_RSd) ((arg1)->used);
    ST(argvi) = SWIG_From_short  SWIG_PERL_CALL_ARGS_1((short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_TABLE_RECORD_is_xref_ref_set) {
  {
    struct _dwg_object_VX_TABLE_RECORD *arg1 = (struct _dwg_object_VX_TABLE_RECORD *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VX_TABLE_RECORD_is_xref_ref_set(self,is_xref_ref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_TABLE_RECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_TABLE_RECORD_is_xref_ref_set" "', argument " "1"" of type '" "struct _dwg_object_VX_TABLE_RECORD *""'"); 
    }
    arg1 = (struct _dwg_object_VX_TABLE_RECORD *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VX_TABLE_RECORD_is_xref_ref_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_xref_ref = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_TABLE_RECORD_is_xref_ref_get) {
  {
    struct _dwg_object_VX_TABLE_RECORD *arg1 = (struct _dwg_object_VX_TABLE_RECORD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VX_TABLE_RECORD_is_xref_ref_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_TABLE_RECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_TABLE_RECORD_is_xref_ref_get" "', argument " "1"" of type '" "struct _dwg_object_VX_TABLE_RECORD *""'"); 
    }
    arg1 = (struct _dwg_object_VX_TABLE_RECORD *)(argp1);
    result = (BITCODE_B) ((arg1)->is_xref_ref);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_TABLE_RECORD_is_xref_resolved_set) {
  {
    struct _dwg_object_VX_TABLE_RECORD *arg1 = (struct _dwg_object_VX_TABLE_RECORD *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VX_TABLE_RECORD_is_xref_resolved_set(self,is_xref_resolved);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_TABLE_RECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_TABLE_RECORD_is_xref_resolved_set" "', argument " "1"" of type '" "struct _dwg_object_VX_TABLE_RECORD *""'"); 
    }
    arg1 = (struct _dwg_object_VX_TABLE_RECORD *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VX_TABLE_RECORD_is_xref_resolved_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->is_xref_resolved = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_TABLE_RECORD_is_xref_resolved_get) {
  {
    struct _dwg_object_VX_TABLE_RECORD *arg1 = (struct _dwg_object_VX_TABLE_RECORD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VX_TABLE_RECORD_is_xref_resolved_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_TABLE_RECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_TABLE_RECORD_is_xref_resolved_get" "', argument " "1"" of type '" "struct _dwg_object_VX_TABLE_RECORD *""'"); 
    }
    arg1 = (struct _dwg_object_VX_TABLE_RECORD *)(argp1);
    result = (BITCODE_BS) ((arg1)->is_xref_resolved);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_TABLE_RECORD_is_xref_dep_set) {
  {
    struct _dwg_object_VX_TABLE_RECORD *arg1 = (struct _dwg_object_VX_TABLE_RECORD *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VX_TABLE_RECORD_is_xref_dep_set(self,is_xref_dep);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_TABLE_RECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_TABLE_RECORD_is_xref_dep_set" "', argument " "1"" of type '" "struct _dwg_object_VX_TABLE_RECORD *""'"); 
    }
    arg1 = (struct _dwg_object_VX_TABLE_RECORD *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VX_TABLE_RECORD_is_xref_dep_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_xref_dep = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_TABLE_RECORD_is_xref_dep_get) {
  {
    struct _dwg_object_VX_TABLE_RECORD *arg1 = (struct _dwg_object_VX_TABLE_RECORD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VX_TABLE_RECORD_is_xref_dep_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_TABLE_RECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_TABLE_RECORD_is_xref_dep_get" "', argument " "1"" of type '" "struct _dwg_object_VX_TABLE_RECORD *""'"); 
    }
    arg1 = (struct _dwg_object_VX_TABLE_RECORD *)(argp1);
    result = (BITCODE_B) ((arg1)->is_xref_dep);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_TABLE_RECORD_xref_set) {
  {
    struct _dwg_object_VX_TABLE_RECORD *arg1 = (struct _dwg_object_VX_TABLE_RECORD *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VX_TABLE_RECORD_xref_set(self,xref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_TABLE_RECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_TABLE_RECORD_xref_set" "', argument " "1"" of type '" "struct _dwg_object_VX_TABLE_RECORD *""'"); 
    }
    arg1 = (struct _dwg_object_VX_TABLE_RECORD *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VX_TABLE_RECORD_xref_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->xref = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_TABLE_RECORD_xref_get) {
  {
    struct _dwg_object_VX_TABLE_RECORD *arg1 = (struct _dwg_object_VX_TABLE_RECORD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VX_TABLE_RECORD_xref_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_TABLE_RECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_TABLE_RECORD_xref_get" "', argument " "1"" of type '" "struct _dwg_object_VX_TABLE_RECORD *""'"); 
    }
    arg1 = (struct _dwg_object_VX_TABLE_RECORD *)(argp1);
    result = (BITCODE_H) ((arg1)->xref);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_TABLE_RECORD_is_on_set) {
  {
    struct _dwg_object_VX_TABLE_RECORD *arg1 = (struct _dwg_object_VX_TABLE_RECORD *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VX_TABLE_RECORD_is_on_set(self,is_on);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_TABLE_RECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_TABLE_RECORD_is_on_set" "', argument " "1"" of type '" "struct _dwg_object_VX_TABLE_RECORD *""'"); 
    }
    arg1 = (struct _dwg_object_VX_TABLE_RECORD *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VX_TABLE_RECORD_is_on_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_on = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_TABLE_RECORD_is_on_get) {
  {
    struct _dwg_object_VX_TABLE_RECORD *arg1 = (struct _dwg_object_VX_TABLE_RECORD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VX_TABLE_RECORD_is_on_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_TABLE_RECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_TABLE_RECORD_is_on_get" "', argument " "1"" of type '" "struct _dwg_object_VX_TABLE_RECORD *""'"); 
    }
    arg1 = (struct _dwg_object_VX_TABLE_RECORD *)(argp1);
    result = (BITCODE_B) ((arg1)->is_on);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_TABLE_RECORD_viewport_set) {
  {
    struct _dwg_object_VX_TABLE_RECORD *arg1 = (struct _dwg_object_VX_TABLE_RECORD *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VX_TABLE_RECORD_viewport_set(self,viewport);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_TABLE_RECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_TABLE_RECORD_viewport_set" "', argument " "1"" of type '" "struct _dwg_object_VX_TABLE_RECORD *""'"); 
    }
    arg1 = (struct _dwg_object_VX_TABLE_RECORD *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VX_TABLE_RECORD_viewport_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->viewport = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_TABLE_RECORD_viewport_get) {
  {
    struct _dwg_object_VX_TABLE_RECORD *arg1 = (struct _dwg_object_VX_TABLE_RECORD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VX_TABLE_RECORD_viewport_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_TABLE_RECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_TABLE_RECORD_viewport_get" "', argument " "1"" of type '" "struct _dwg_object_VX_TABLE_RECORD *""'"); 
    }
    arg1 = (struct _dwg_object_VX_TABLE_RECORD *)(argp1);
    result = (BITCODE_H) ((arg1)->viewport);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_TABLE_RECORD_prev_entry_set) {
  {
    struct _dwg_object_VX_TABLE_RECORD *arg1 = (struct _dwg_object_VX_TABLE_RECORD *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VX_TABLE_RECORD_prev_entry_set(self,prev_entry);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_TABLE_RECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_TABLE_RECORD_prev_entry_set" "', argument " "1"" of type '" "struct _dwg_object_VX_TABLE_RECORD *""'"); 
    }
    arg1 = (struct _dwg_object_VX_TABLE_RECORD *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VX_TABLE_RECORD_prev_entry_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->prev_entry = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_TABLE_RECORD_prev_entry_get) {
  {
    struct _dwg_object_VX_TABLE_RECORD *arg1 = (struct _dwg_object_VX_TABLE_RECORD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VX_TABLE_RECORD_prev_entry_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_TABLE_RECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_TABLE_RECORD_prev_entry_get" "', argument " "1"" of type '" "struct _dwg_object_VX_TABLE_RECORD *""'"); 
    }
    arg1 = (struct _dwg_object_VX_TABLE_RECORD *)(argp1);
    result = (BITCODE_H) ((arg1)->prev_entry);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_TABLE_RECORD_vport_entity_address_set) {
  {
    struct _dwg_object_VX_TABLE_RECORD *arg1 = (struct _dwg_object_VX_TABLE_RECORD *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VX_TABLE_RECORD_vport_entity_address_set(self,vport_entity_address);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_TABLE_RECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_TABLE_RECORD_vport_entity_address_set" "', argument " "1"" of type '" "struct _dwg_object_VX_TABLE_RECORD *""'"); 
    }
    arg1 = (struct _dwg_object_VX_TABLE_RECORD *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VX_TABLE_RECORD_vport_entity_address_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->vport_entity_address = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_TABLE_RECORD_vport_entity_address_get) {
  {
    struct _dwg_object_VX_TABLE_RECORD *arg1 = (struct _dwg_object_VX_TABLE_RECORD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VX_TABLE_RECORD_vport_entity_address_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_TABLE_RECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_TABLE_RECORD_vport_entity_address_get" "', argument " "1"" of type '" "struct _dwg_object_VX_TABLE_RECORD *""'"); 
    }
    arg1 = (struct _dwg_object_VX_TABLE_RECORD *)(argp1);
    result = (BITCODE_RS) ((arg1)->vport_entity_address);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_TABLE_RECORD_r11_viewport_index_set) {
  {
    struct _dwg_object_VX_TABLE_RECORD *arg1 = (struct _dwg_object_VX_TABLE_RECORD *) 0 ;
    BITCODE_RSd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VX_TABLE_RECORD_r11_viewport_index_set(self,r11_viewport_index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_TABLE_RECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_TABLE_RECORD_r11_viewport_index_set" "', argument " "1"" of type '" "struct _dwg_object_VX_TABLE_RECORD *""'"); 
    }
    arg1 = (struct _dwg_object_VX_TABLE_RECORD *)(argp1);
    ecode2 = SWIG_AsVal_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VX_TABLE_RECORD_r11_viewport_index_set" "', argument " "2"" of type '" "BITCODE_RSd""'");
    } 
    arg2 = (BITCODE_RSd)(val2);
    if (arg1) (arg1)->r11_viewport_index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_TABLE_RECORD_r11_viewport_index_get) {
  {
    struct _dwg_object_VX_TABLE_RECORD *arg1 = (struct _dwg_object_VX_TABLE_RECORD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RSd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VX_TABLE_RECORD_r11_viewport_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_TABLE_RECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_TABLE_RECORD_r11_viewport_index_get" "', argument " "1"" of type '" "struct _dwg_object_VX_TABLE_RECORD *""'"); 
    }
    arg1 = (struct _dwg_object_VX_TABLE_RECORD *)(argp1);
    result = (BITCODE_RSd) ((arg1)->r11_viewport_index);
    ST(argvi) = SWIG_From_short  SWIG_PERL_CALL_ARGS_1((short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_TABLE_RECORD_r11_prev_entry_index_set) {
  {
    struct _dwg_object_VX_TABLE_RECORD *arg1 = (struct _dwg_object_VX_TABLE_RECORD *) 0 ;
    BITCODE_RSd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VX_TABLE_RECORD_r11_prev_entry_index_set(self,r11_prev_entry_index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_TABLE_RECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_TABLE_RECORD_r11_prev_entry_index_set" "', argument " "1"" of type '" "struct _dwg_object_VX_TABLE_RECORD *""'"); 
    }
    arg1 = (struct _dwg_object_VX_TABLE_RECORD *)(argp1);
    ecode2 = SWIG_AsVal_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VX_TABLE_RECORD_r11_prev_entry_index_set" "', argument " "2"" of type '" "BITCODE_RSd""'");
    } 
    arg2 = (BITCODE_RSd)(val2);
    if (arg1) (arg1)->r11_prev_entry_index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VX_TABLE_RECORD_r11_prev_entry_index_get) {
  {
    struct _dwg_object_VX_TABLE_RECORD *arg1 = (struct _dwg_object_VX_TABLE_RECORD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RSd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VX_TABLE_RECORD_r11_prev_entry_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_TABLE_RECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VX_TABLE_RECORD_r11_prev_entry_index_get" "', argument " "1"" of type '" "struct _dwg_object_VX_TABLE_RECORD *""'"); 
    }
    arg1 = (struct _dwg_object_VX_TABLE_RECORD *)(argp1);
    result = (BITCODE_RSd) ((arg1)->r11_prev_entry_index);
    ST(argvi) = SWIG_From_short  SWIG_PERL_CALL_ARGS_1((short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_VX_TABLE_RECORD) {
  {
    int argvi = 0;
    struct _dwg_object_VX_TABLE_RECORD *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_VX_TABLE_RECORD();");
    }
    result = (struct _dwg_object_VX_TABLE_RECORD *)calloc(1, sizeof(struct _dwg_object_VX_TABLE_RECORD));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_VX_TABLE_RECORD, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_VX_TABLE_RECORD) {
  {
    struct _dwg_object_VX_TABLE_RECORD *arg1 = (struct _dwg_object_VX_TABLE_RECORD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_VX_TABLE_RECORD(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VX_TABLE_RECORD, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_VX_TABLE_RECORD" "', argument " "1"" of type '" "struct _dwg_object_VX_TABLE_RECORD *""'"); 
    }
    arg1 = (struct _dwg_object_VX_TABLE_RECORD *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GROUP_parent_set) {
  {
    struct _dwg_object_GROUP *arg1 = (struct _dwg_object_GROUP *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GROUP_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GROUP_parent_set" "', argument " "1"" of type '" "struct _dwg_object_GROUP *""'"); 
    }
    arg1 = (struct _dwg_object_GROUP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_GROUP_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GROUP_parent_get) {
  {
    struct _dwg_object_GROUP *arg1 = (struct _dwg_object_GROUP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GROUP_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GROUP_parent_get" "', argument " "1"" of type '" "struct _dwg_object_GROUP *""'"); 
    }
    arg1 = (struct _dwg_object_GROUP *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GROUP_name_set) {
  {
    struct _dwg_object_GROUP *arg1 = (struct _dwg_object_GROUP *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GROUP_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GROUP_name_set" "', argument " "1"" of type '" "struct _dwg_object_GROUP *""'"); 
    }
    arg1 = (struct _dwg_object_GROUP *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_GROUP_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GROUP_name_get) {
  {
    struct _dwg_object_GROUP *arg1 = (struct _dwg_object_GROUP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GROUP_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GROUP_name_get" "', argument " "1"" of type '" "struct _dwg_object_GROUP *""'"); 
    }
    arg1 = (struct _dwg_object_GROUP *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GROUP_unnamed_set) {
  {
    struct _dwg_object_GROUP *arg1 = (struct _dwg_object_GROUP *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GROUP_unnamed_set(self,unnamed);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GROUP_unnamed_set" "', argument " "1"" of type '" "struct _dwg_object_GROUP *""'"); 
    }
    arg1 = (struct _dwg_object_GROUP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GROUP_unnamed_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->unnamed = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GROUP_unnamed_get) {
  {
    struct _dwg_object_GROUP *arg1 = (struct _dwg_object_GROUP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GROUP_unnamed_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GROUP_unnamed_get" "', argument " "1"" of type '" "struct _dwg_object_GROUP *""'"); 
    }
    arg1 = (struct _dwg_object_GROUP *)(argp1);
    result = (BITCODE_BS) ((arg1)->unnamed);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GROUP_selectable_set) {
  {
    struct _dwg_object_GROUP *arg1 = (struct _dwg_object_GROUP *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GROUP_selectable_set(self,selectable);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GROUP_selectable_set" "', argument " "1"" of type '" "struct _dwg_object_GROUP *""'"); 
    }
    arg1 = (struct _dwg_object_GROUP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GROUP_selectable_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->selectable = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GROUP_selectable_get) {
  {
    struct _dwg_object_GROUP *arg1 = (struct _dwg_object_GROUP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GROUP_selectable_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GROUP_selectable_get" "', argument " "1"" of type '" "struct _dwg_object_GROUP *""'"); 
    }
    arg1 = (struct _dwg_object_GROUP *)(argp1);
    result = (BITCODE_BS) ((arg1)->selectable);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GROUP_num_groups_set) {
  {
    struct _dwg_object_GROUP *arg1 = (struct _dwg_object_GROUP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GROUP_num_groups_set(self,num_groups);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GROUP_num_groups_set" "', argument " "1"" of type '" "struct _dwg_object_GROUP *""'"); 
    }
    arg1 = (struct _dwg_object_GROUP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GROUP_num_groups_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_groups = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GROUP_num_groups_get) {
  {
    struct _dwg_object_GROUP *arg1 = (struct _dwg_object_GROUP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GROUP_num_groups_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GROUP_num_groups_get" "', argument " "1"" of type '" "struct _dwg_object_GROUP *""'"); 
    }
    arg1 = (struct _dwg_object_GROUP *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_groups);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GROUP_groups_set) {
  {
    struct _dwg_object_GROUP *arg1 = (struct _dwg_object_GROUP *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GROUP_groups_set(self,groups);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GROUP_groups_set" "', argument " "1"" of type '" "struct _dwg_object_GROUP *""'"); 
    }
    arg1 = (struct _dwg_object_GROUP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_GROUP_groups_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->groups = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GROUP_groups_get) {
  {
    struct _dwg_object_GROUP *arg1 = (struct _dwg_object_GROUP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GROUP_groups_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GROUP_groups_get" "', argument " "1"" of type '" "struct _dwg_object_GROUP *""'"); 
    }
    arg1 = (struct _dwg_object_GROUP *)(argp1);
    result = (BITCODE_H *) ((arg1)->groups);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_GROUP) {
  {
    int argvi = 0;
    struct _dwg_object_GROUP *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_GROUP();");
    }
    result = (struct _dwg_object_GROUP *)calloc(1, sizeof(struct _dwg_object_GROUP));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_GROUP, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_GROUP) {
  {
    struct _dwg_object_GROUP *arg1 = (struct _dwg_object_GROUP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_GROUP(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GROUP, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_GROUP" "', argument " "1"" of type '" "struct _dwg_object_GROUP *""'"); 
    }
    arg1 = (struct _dwg_object_GROUP *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLINESTYLE_line_parent_set) {
  {
    struct _dwg_MLINESTYLE_line *arg1 = (struct _dwg_MLINESTYLE_line *) 0 ;
    struct _dwg_object_MLINESTYLE *arg2 = (struct _dwg_object_MLINESTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLINESTYLE_line_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINESTYLE_line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLINESTYLE_line_parent_set" "', argument " "1"" of type '" "struct _dwg_MLINESTYLE_line *""'"); 
    }
    arg1 = (struct _dwg_MLINESTYLE_line *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_MLINESTYLE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MLINESTYLE_line_parent_set" "', argument " "2"" of type '" "struct _dwg_object_MLINESTYLE *""'"); 
    }
    arg2 = (struct _dwg_object_MLINESTYLE *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLINESTYLE_line_parent_get) {
  {
    struct _dwg_MLINESTYLE_line *arg1 = (struct _dwg_MLINESTYLE_line *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_MLINESTYLE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLINESTYLE_line_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINESTYLE_line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLINESTYLE_line_parent_get" "', argument " "1"" of type '" "struct _dwg_MLINESTYLE_line *""'"); 
    }
    arg1 = (struct _dwg_MLINESTYLE_line *)(argp1);
    result = (struct _dwg_object_MLINESTYLE *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_MLINESTYLE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLINESTYLE_line_offset_set) {
  {
    struct _dwg_MLINESTYLE_line *arg1 = (struct _dwg_MLINESTYLE_line *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLINESTYLE_line_offset_set(self,offset);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINESTYLE_line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLINESTYLE_line_offset_set" "', argument " "1"" of type '" "struct _dwg_MLINESTYLE_line *""'"); 
    }
    arg1 = (struct _dwg_MLINESTYLE_line *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLINESTYLE_line_offset_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->offset = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLINESTYLE_line_offset_get) {
  {
    struct _dwg_MLINESTYLE_line *arg1 = (struct _dwg_MLINESTYLE_line *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLINESTYLE_line_offset_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINESTYLE_line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLINESTYLE_line_offset_get" "', argument " "1"" of type '" "struct _dwg_MLINESTYLE_line *""'"); 
    }
    arg1 = (struct _dwg_MLINESTYLE_line *)(argp1);
    result = (BITCODE_BD) ((arg1)->offset);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLINESTYLE_line_color_set) {
  {
    struct _dwg_MLINESTYLE_line *arg1 = (struct _dwg_MLINESTYLE_line *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLINESTYLE_line_color_set(self,color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINESTYLE_line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLINESTYLE_line_color_set" "', argument " "1"" of type '" "struct _dwg_MLINESTYLE_line *""'"); 
    }
    arg1 = (struct _dwg_MLINESTYLE_line *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MLINESTYLE_line_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLINESTYLE_line_color_get) {
  {
    struct _dwg_MLINESTYLE_line *arg1 = (struct _dwg_MLINESTYLE_line *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLINESTYLE_line_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINESTYLE_line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLINESTYLE_line_color_get" "', argument " "1"" of type '" "struct _dwg_MLINESTYLE_line *""'"); 
    }
    arg1 = (struct _dwg_MLINESTYLE_line *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLINESTYLE_line_lt_get) {
  {
    struct _dwg_MLINESTYLE_line *arg1 = (struct _dwg_MLINESTYLE_line *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    _dwg_MLINESTYLE_line_lt *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLINESTYLE_line_lt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINESTYLE_line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLINESTYLE_line_lt_get" "', argument " "1"" of type '" "struct _dwg_MLINESTYLE_line *""'"); 
    }
    arg1 = (struct _dwg_MLINESTYLE_line *)(argp1);
    result = (_dwg_MLINESTYLE_line_lt *)& ((arg1)->lt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_MLINESTYLE_line_lt, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_MLINESTYLE_line) {
  {
    int argvi = 0;
    struct _dwg_MLINESTYLE_line *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_MLINESTYLE_line();");
    }
    result = (struct _dwg_MLINESTYLE_line *)calloc(1, sizeof(struct _dwg_MLINESTYLE_line));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_MLINESTYLE_line, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_MLINESTYLE_line) {
  {
    struct _dwg_MLINESTYLE_line *arg1 = (struct _dwg_MLINESTYLE_line *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_MLINESTYLE_line(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINESTYLE_line, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_MLINESTYLE_line" "', argument " "1"" of type '" "struct _dwg_MLINESTYLE_line *""'"); 
    }
    arg1 = (struct _dwg_MLINESTYLE_line *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_MLINESTYLE_line_lt_index_set) {
  {
    _dwg_MLINESTYLE_line_lt *arg1 = (_dwg_MLINESTYLE_line_lt *) 0 ;
    BITCODE_BSd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_MLINESTYLE_line_lt_index_set(self,index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINESTYLE_line_lt, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_MLINESTYLE_line_lt_index_set" "', argument " "1"" of type '" "_dwg_MLINESTYLE_line_lt *""'"); 
    }
    arg1 = (_dwg_MLINESTYLE_line_lt *)(argp1);
    ecode2 = SWIG_AsVal_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "_dwg_MLINESTYLE_line_lt_index_set" "', argument " "2"" of type '" "BITCODE_BSd""'");
    } 
    arg2 = (BITCODE_BSd)(val2);
    if (arg1) (arg1)->index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_MLINESTYLE_line_lt_index_get) {
  {
    _dwg_MLINESTYLE_line_lt *arg1 = (_dwg_MLINESTYLE_line_lt *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BSd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_MLINESTYLE_line_lt_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINESTYLE_line_lt, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_MLINESTYLE_line_lt_index_get" "', argument " "1"" of type '" "_dwg_MLINESTYLE_line_lt *""'"); 
    }
    arg1 = (_dwg_MLINESTYLE_line_lt *)(argp1);
    result = (BITCODE_BSd) ((arg1)->index);
    ST(argvi) = SWIG_From_short  SWIG_PERL_CALL_ARGS_1((short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_MLINESTYLE_line_lt_ltype_set) {
  {
    _dwg_MLINESTYLE_line_lt *arg1 = (_dwg_MLINESTYLE_line_lt *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_MLINESTYLE_line_lt_ltype_set(self,ltype);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINESTYLE_line_lt, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_MLINESTYLE_line_lt_ltype_set" "', argument " "1"" of type '" "_dwg_MLINESTYLE_line_lt *""'"); 
    }
    arg1 = (_dwg_MLINESTYLE_line_lt *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_MLINESTYLE_line_lt_ltype_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->ltype = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_MLINESTYLE_line_lt_ltype_get) {
  {
    _dwg_MLINESTYLE_line_lt *arg1 = (_dwg_MLINESTYLE_line_lt *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_MLINESTYLE_line_lt_ltype_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINESTYLE_line_lt, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_MLINESTYLE_line_lt_ltype_get" "', argument " "1"" of type '" "_dwg_MLINESTYLE_line_lt *""'"); 
    }
    arg1 = (_dwg_MLINESTYLE_line_lt *)(argp1);
    result = (BITCODE_H) ((arg1)->ltype);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new__dwg_MLINESTYLE_line_lt) {
  {
    int argvi = 0;
    _dwg_MLINESTYLE_line_lt *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new__dwg_MLINESTYLE_line_lt();");
    }
    result = (_dwg_MLINESTYLE_line_lt *)calloc(1, sizeof(_dwg_MLINESTYLE_line_lt));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_MLINESTYLE_line_lt, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete__dwg_MLINESTYLE_line_lt) {
  {
    _dwg_MLINESTYLE_line_lt *arg1 = (_dwg_MLINESTYLE_line_lt *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete__dwg_MLINESTYLE_line_lt(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLINESTYLE_line_lt, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete__dwg_MLINESTYLE_line_lt" "', argument " "1"" of type '" "_dwg_MLINESTYLE_line_lt *""'"); 
    }
    arg1 = (_dwg_MLINESTYLE_line_lt *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLINESTYLE_parent_set) {
  {
    struct _dwg_object_MLINESTYLE *arg1 = (struct _dwg_object_MLINESTYLE *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLINESTYLE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLINESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLINESTYLE_parent_set" "', argument " "1"" of type '" "struct _dwg_object_MLINESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLINESTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MLINESTYLE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLINESTYLE_parent_get) {
  {
    struct _dwg_object_MLINESTYLE *arg1 = (struct _dwg_object_MLINESTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLINESTYLE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLINESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLINESTYLE_parent_get" "', argument " "1"" of type '" "struct _dwg_object_MLINESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLINESTYLE *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLINESTYLE_name_set) {
  {
    struct _dwg_object_MLINESTYLE *arg1 = (struct _dwg_object_MLINESTYLE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLINESTYLE_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLINESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLINESTYLE_name_set" "', argument " "1"" of type '" "struct _dwg_object_MLINESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLINESTYLE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MLINESTYLE_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLINESTYLE_name_get) {
  {
    struct _dwg_object_MLINESTYLE *arg1 = (struct _dwg_object_MLINESTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLINESTYLE_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLINESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLINESTYLE_name_get" "', argument " "1"" of type '" "struct _dwg_object_MLINESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLINESTYLE *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLINESTYLE_description_set) {
  {
    struct _dwg_object_MLINESTYLE *arg1 = (struct _dwg_object_MLINESTYLE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLINESTYLE_description_set(self,description);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLINESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLINESTYLE_description_set" "', argument " "1"" of type '" "struct _dwg_object_MLINESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLINESTYLE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MLINESTYLE_description_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->description);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->description = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->description = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLINESTYLE_description_get) {
  {
    struct _dwg_object_MLINESTYLE *arg1 = (struct _dwg_object_MLINESTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLINESTYLE_description_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLINESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLINESTYLE_description_get" "', argument " "1"" of type '" "struct _dwg_object_MLINESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLINESTYLE *)(argp1);
    result = (BITCODE_TV) ((arg1)->description);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLINESTYLE_flag_set) {
  {
    struct _dwg_object_MLINESTYLE *arg1 = (struct _dwg_object_MLINESTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLINESTYLE_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLINESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLINESTYLE_flag_set" "', argument " "1"" of type '" "struct _dwg_object_MLINESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLINESTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLINESTYLE_flag_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLINESTYLE_flag_get) {
  {
    struct _dwg_object_MLINESTYLE *arg1 = (struct _dwg_object_MLINESTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLINESTYLE_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLINESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLINESTYLE_flag_get" "', argument " "1"" of type '" "struct _dwg_object_MLINESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLINESTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLINESTYLE_fill_color_set) {
  {
    struct _dwg_object_MLINESTYLE *arg1 = (struct _dwg_object_MLINESTYLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLINESTYLE_fill_color_set(self,fill_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLINESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLINESTYLE_fill_color_set" "', argument " "1"" of type '" "struct _dwg_object_MLINESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLINESTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MLINESTYLE_fill_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->fill_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLINESTYLE_fill_color_get) {
  {
    struct _dwg_object_MLINESTYLE *arg1 = (struct _dwg_object_MLINESTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLINESTYLE_fill_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLINESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLINESTYLE_fill_color_get" "', argument " "1"" of type '" "struct _dwg_object_MLINESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLINESTYLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->fill_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLINESTYLE_start_angle_set) {
  {
    struct _dwg_object_MLINESTYLE *arg1 = (struct _dwg_object_MLINESTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLINESTYLE_start_angle_set(self,start_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLINESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLINESTYLE_start_angle_set" "', argument " "1"" of type '" "struct _dwg_object_MLINESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLINESTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLINESTYLE_start_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->start_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLINESTYLE_start_angle_get) {
  {
    struct _dwg_object_MLINESTYLE *arg1 = (struct _dwg_object_MLINESTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLINESTYLE_start_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLINESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLINESTYLE_start_angle_get" "', argument " "1"" of type '" "struct _dwg_object_MLINESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLINESTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->start_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLINESTYLE_end_angle_set) {
  {
    struct _dwg_object_MLINESTYLE *arg1 = (struct _dwg_object_MLINESTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLINESTYLE_end_angle_set(self,end_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLINESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLINESTYLE_end_angle_set" "', argument " "1"" of type '" "struct _dwg_object_MLINESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLINESTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLINESTYLE_end_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->end_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLINESTYLE_end_angle_get) {
  {
    struct _dwg_object_MLINESTYLE *arg1 = (struct _dwg_object_MLINESTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLINESTYLE_end_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLINESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLINESTYLE_end_angle_get" "', argument " "1"" of type '" "struct _dwg_object_MLINESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLINESTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->end_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLINESTYLE_num_lines_set) {
  {
    struct _dwg_object_MLINESTYLE *arg1 = (struct _dwg_object_MLINESTYLE *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLINESTYLE_num_lines_set(self,num_lines);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLINESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLINESTYLE_num_lines_set" "', argument " "1"" of type '" "struct _dwg_object_MLINESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLINESTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLINESTYLE_num_lines_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->num_lines = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLINESTYLE_num_lines_get) {
  {
    struct _dwg_object_MLINESTYLE *arg1 = (struct _dwg_object_MLINESTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLINESTYLE_num_lines_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLINESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLINESTYLE_num_lines_get" "', argument " "1"" of type '" "struct _dwg_object_MLINESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLINESTYLE *)(argp1);
    result = (BITCODE_RC) ((arg1)->num_lines);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLINESTYLE_lines_set) {
  {
    struct _dwg_object_MLINESTYLE *arg1 = (struct _dwg_object_MLINESTYLE *) 0 ;
    Dwg_MLINESTYLE_line *arg2 = (Dwg_MLINESTYLE_line *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLINESTYLE_lines_set(self,lines);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLINESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLINESTYLE_lines_set" "', argument " "1"" of type '" "struct _dwg_object_MLINESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLINESTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_MLINESTYLE_line, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MLINESTYLE_lines_set" "', argument " "2"" of type '" "Dwg_MLINESTYLE_line *""'"); 
    }
    arg2 = (Dwg_MLINESTYLE_line *)(argp2);
    if (arg1) (arg1)->lines = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLINESTYLE_lines_get) {
  {
    struct _dwg_object_MLINESTYLE *arg1 = (struct _dwg_object_MLINESTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_MLINESTYLE_line *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLINESTYLE_lines_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLINESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLINESTYLE_lines_get" "', argument " "1"" of type '" "struct _dwg_object_MLINESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLINESTYLE *)(argp1);
    result = (Dwg_MLINESTYLE_line *) ((arg1)->lines);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_MLINESTYLE_line, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_MLINESTYLE) {
  {
    int argvi = 0;
    struct _dwg_object_MLINESTYLE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_MLINESTYLE();");
    }
    result = (struct _dwg_object_MLINESTYLE *)calloc(1, sizeof(struct _dwg_object_MLINESTYLE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_MLINESTYLE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_MLINESTYLE) {
  {
    struct _dwg_object_MLINESTYLE *arg1 = (struct _dwg_object_MLINESTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_MLINESTYLE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLINESTYLE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_MLINESTYLE" "', argument " "1"" of type '" "struct _dwg_object_MLINESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLINESTYLE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_OLE2FRAME_parent_set) {
  {
    struct _dwg_entity_OLE2FRAME *arg1 = (struct _dwg_entity_OLE2FRAME *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_OLE2FRAME_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_OLE2FRAME, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_OLE2FRAME_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_OLE2FRAME *""'"); 
    }
    arg1 = (struct _dwg_entity_OLE2FRAME *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_OLE2FRAME_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_OLE2FRAME_parent_get) {
  {
    struct _dwg_entity_OLE2FRAME *arg1 = (struct _dwg_entity_OLE2FRAME *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_OLE2FRAME_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_OLE2FRAME, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_OLE2FRAME_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_OLE2FRAME *""'"); 
    }
    arg1 = (struct _dwg_entity_OLE2FRAME *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_OLE2FRAME_type_set) {
  {
    struct _dwg_entity_OLE2FRAME *arg1 = (struct _dwg_entity_OLE2FRAME *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_OLE2FRAME_type_set(self,type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_OLE2FRAME, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_OLE2FRAME_type_set" "', argument " "1"" of type '" "struct _dwg_entity_OLE2FRAME *""'"); 
    }
    arg1 = (struct _dwg_entity_OLE2FRAME *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_OLE2FRAME_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_OLE2FRAME_type_get) {
  {
    struct _dwg_entity_OLE2FRAME *arg1 = (struct _dwg_entity_OLE2FRAME *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_OLE2FRAME_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_OLE2FRAME, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_OLE2FRAME_type_get" "', argument " "1"" of type '" "struct _dwg_entity_OLE2FRAME *""'"); 
    }
    arg1 = (struct _dwg_entity_OLE2FRAME *)(argp1);
    result = (BITCODE_BS) ((arg1)->type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_OLE2FRAME_mode_set) {
  {
    struct _dwg_entity_OLE2FRAME *arg1 = (struct _dwg_entity_OLE2FRAME *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_OLE2FRAME_mode_set(self,mode);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_OLE2FRAME, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_OLE2FRAME_mode_set" "', argument " "1"" of type '" "struct _dwg_entity_OLE2FRAME *""'"); 
    }
    arg1 = (struct _dwg_entity_OLE2FRAME *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_OLE2FRAME_mode_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->mode = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_OLE2FRAME_mode_get) {
  {
    struct _dwg_entity_OLE2FRAME *arg1 = (struct _dwg_entity_OLE2FRAME *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_OLE2FRAME_mode_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_OLE2FRAME, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_OLE2FRAME_mode_get" "', argument " "1"" of type '" "struct _dwg_entity_OLE2FRAME *""'"); 
    }
    arg1 = (struct _dwg_entity_OLE2FRAME *)(argp1);
    result = (BITCODE_BS) ((arg1)->mode);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_OLE2FRAME_lock_aspect_set) {
  {
    struct _dwg_entity_OLE2FRAME *arg1 = (struct _dwg_entity_OLE2FRAME *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_OLE2FRAME_lock_aspect_set(self,lock_aspect);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_OLE2FRAME, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_OLE2FRAME_lock_aspect_set" "', argument " "1"" of type '" "struct _dwg_entity_OLE2FRAME *""'"); 
    }
    arg1 = (struct _dwg_entity_OLE2FRAME *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_OLE2FRAME_lock_aspect_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->lock_aspect = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_OLE2FRAME_lock_aspect_get) {
  {
    struct _dwg_entity_OLE2FRAME *arg1 = (struct _dwg_entity_OLE2FRAME *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_OLE2FRAME_lock_aspect_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_OLE2FRAME, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_OLE2FRAME_lock_aspect_get" "', argument " "1"" of type '" "struct _dwg_entity_OLE2FRAME *""'"); 
    }
    arg1 = (struct _dwg_entity_OLE2FRAME *)(argp1);
    result = (BITCODE_RC) ((arg1)->lock_aspect);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_OLE2FRAME_data_size_set) {
  {
    struct _dwg_entity_OLE2FRAME *arg1 = (struct _dwg_entity_OLE2FRAME *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_OLE2FRAME_data_size_set(self,data_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_OLE2FRAME, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_OLE2FRAME_data_size_set" "', argument " "1"" of type '" "struct _dwg_entity_OLE2FRAME *""'"); 
    }
    arg1 = (struct _dwg_entity_OLE2FRAME *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_OLE2FRAME_data_size_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->data_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_OLE2FRAME_data_size_get) {
  {
    struct _dwg_entity_OLE2FRAME *arg1 = (struct _dwg_entity_OLE2FRAME *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_OLE2FRAME_data_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_OLE2FRAME, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_OLE2FRAME_data_size_get" "', argument " "1"" of type '" "struct _dwg_entity_OLE2FRAME *""'"); 
    }
    arg1 = (struct _dwg_entity_OLE2FRAME *)(argp1);
    result = (BITCODE_BL) ((arg1)->data_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_OLE2FRAME_data_set) {
  {
    struct _dwg_entity_OLE2FRAME *arg1 = (struct _dwg_entity_OLE2FRAME *) 0 ;
    BITCODE_TF arg2 = (BITCODE_TF) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_OLE2FRAME_data_set(self,data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_OLE2FRAME, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_OLE2FRAME_data_set" "', argument " "1"" of type '" "struct _dwg_entity_OLE2FRAME *""'"); 
    }
    arg1 = (struct _dwg_entity_OLE2FRAME *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_OLE2FRAME_data_set" "', argument " "2"" of type '" "BITCODE_TF""'"); 
    }
    arg2 = (BITCODE_TF)(argp2);
    if (arg1) (arg1)->data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_OLE2FRAME_data_get) {
  {
    struct _dwg_entity_OLE2FRAME *arg1 = (struct _dwg_entity_OLE2FRAME *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TF result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_OLE2FRAME_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_OLE2FRAME, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_OLE2FRAME_data_get" "', argument " "1"" of type '" "struct _dwg_entity_OLE2FRAME *""'"); 
    }
    arg1 = (struct _dwg_entity_OLE2FRAME *)(argp1);
    result = (BITCODE_TF) ((arg1)->data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_OLE2FRAME_oleversion_set) {
  {
    struct _dwg_entity_OLE2FRAME *arg1 = (struct _dwg_entity_OLE2FRAME *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_OLE2FRAME_oleversion_set(self,oleversion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_OLE2FRAME, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_OLE2FRAME_oleversion_set" "', argument " "1"" of type '" "struct _dwg_entity_OLE2FRAME *""'"); 
    }
    arg1 = (struct _dwg_entity_OLE2FRAME *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_OLE2FRAME_oleversion_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->oleversion = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_OLE2FRAME_oleversion_get) {
  {
    struct _dwg_entity_OLE2FRAME *arg1 = (struct _dwg_entity_OLE2FRAME *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_OLE2FRAME_oleversion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_OLE2FRAME, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_OLE2FRAME_oleversion_get" "', argument " "1"" of type '" "struct _dwg_entity_OLE2FRAME *""'"); 
    }
    arg1 = (struct _dwg_entity_OLE2FRAME *)(argp1);
    result = (BITCODE_BS) ((arg1)->oleversion);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_OLE2FRAME_oleclient_set) {
  {
    struct _dwg_entity_OLE2FRAME *arg1 = (struct _dwg_entity_OLE2FRAME *) 0 ;
    BITCODE_TF arg2 = (BITCODE_TF) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_OLE2FRAME_oleclient_set(self,oleclient);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_OLE2FRAME, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_OLE2FRAME_oleclient_set" "', argument " "1"" of type '" "struct _dwg_entity_OLE2FRAME *""'"); 
    }
    arg1 = (struct _dwg_entity_OLE2FRAME *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_OLE2FRAME_oleclient_set" "', argument " "2"" of type '" "BITCODE_TF""'"); 
    }
    arg2 = (BITCODE_TF)(argp2);
    if (arg1) (arg1)->oleclient = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_OLE2FRAME_oleclient_get) {
  {
    struct _dwg_entity_OLE2FRAME *arg1 = (struct _dwg_entity_OLE2FRAME *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TF result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_OLE2FRAME_oleclient_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_OLE2FRAME, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_OLE2FRAME_oleclient_get" "', argument " "1"" of type '" "struct _dwg_entity_OLE2FRAME *""'"); 
    }
    arg1 = (struct _dwg_entity_OLE2FRAME *)(argp1);
    result = (BITCODE_TF) ((arg1)->oleclient);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_OLE2FRAME_pt1_set) {
  {
    struct _dwg_entity_OLE2FRAME *arg1 = (struct _dwg_entity_OLE2FRAME *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_OLE2FRAME_pt1_set(self,pt1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_OLE2FRAME, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_OLE2FRAME_pt1_set" "', argument " "1"" of type '" "struct _dwg_entity_OLE2FRAME *""'"); 
    }
    arg1 = (struct _dwg_entity_OLE2FRAME *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_OLE2FRAME_pt1_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->pt1 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_OLE2FRAME_pt1_get) {
  {
    struct _dwg_entity_OLE2FRAME *arg1 = (struct _dwg_entity_OLE2FRAME *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_OLE2FRAME_pt1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_OLE2FRAME, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_OLE2FRAME_pt1_get" "', argument " "1"" of type '" "struct _dwg_entity_OLE2FRAME *""'"); 
    }
    arg1 = (struct _dwg_entity_OLE2FRAME *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->pt1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_OLE2FRAME_pt2_set) {
  {
    struct _dwg_entity_OLE2FRAME *arg1 = (struct _dwg_entity_OLE2FRAME *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_OLE2FRAME_pt2_set(self,pt2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_OLE2FRAME, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_OLE2FRAME_pt2_set" "', argument " "1"" of type '" "struct _dwg_entity_OLE2FRAME *""'"); 
    }
    arg1 = (struct _dwg_entity_OLE2FRAME *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_OLE2FRAME_pt2_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->pt2 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_OLE2FRAME_pt2_get) {
  {
    struct _dwg_entity_OLE2FRAME *arg1 = (struct _dwg_entity_OLE2FRAME *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_OLE2FRAME_pt2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_OLE2FRAME, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_OLE2FRAME_pt2_get" "', argument " "1"" of type '" "struct _dwg_entity_OLE2FRAME *""'"); 
    }
    arg1 = (struct _dwg_entity_OLE2FRAME *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->pt2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_OLE2FRAME) {
  {
    int argvi = 0;
    struct _dwg_entity_OLE2FRAME *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_OLE2FRAME();");
    }
    result = (struct _dwg_entity_OLE2FRAME *)calloc(1, sizeof(struct _dwg_entity_OLE2FRAME));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_OLE2FRAME, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_OLE2FRAME) {
  {
    struct _dwg_entity_OLE2FRAME *arg1 = (struct _dwg_entity_OLE2FRAME *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_OLE2FRAME(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_OLE2FRAME, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_OLE2FRAME" "', argument " "1"" of type '" "struct _dwg_entity_OLE2FRAME *""'"); 
    }
    arg1 = (struct _dwg_entity_OLE2FRAME *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DUMMY_parent_set) {
  {
    struct _dwg_object_DUMMY *arg1 = (struct _dwg_object_DUMMY *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DUMMY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DUMMY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DUMMY_parent_set" "', argument " "1"" of type '" "struct _dwg_object_DUMMY *""'"); 
    }
    arg1 = (struct _dwg_object_DUMMY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DUMMY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DUMMY_parent_get) {
  {
    struct _dwg_object_DUMMY *arg1 = (struct _dwg_object_DUMMY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DUMMY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DUMMY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DUMMY_parent_get" "', argument " "1"" of type '" "struct _dwg_object_DUMMY *""'"); 
    }
    arg1 = (struct _dwg_object_DUMMY *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_DUMMY) {
  {
    int argvi = 0;
    struct _dwg_object_DUMMY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_DUMMY();");
    }
    result = (struct _dwg_object_DUMMY *)calloc(1, sizeof(struct _dwg_object_DUMMY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DUMMY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_DUMMY) {
  {
    struct _dwg_object_DUMMY *arg1 = (struct _dwg_object_DUMMY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_DUMMY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DUMMY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_DUMMY" "', argument " "1"" of type '" "struct _dwg_object_DUMMY *""'"); 
    }
    arg1 = (struct _dwg_object_DUMMY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LONG_TRANSACTION_parent_set) {
  {
    struct _dwg_object_LONG_TRANSACTION *arg1 = (struct _dwg_object_LONG_TRANSACTION *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LONG_TRANSACTION_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LONG_TRANSACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LONG_TRANSACTION_parent_set" "', argument " "1"" of type '" "struct _dwg_object_LONG_TRANSACTION *""'"); 
    }
    arg1 = (struct _dwg_object_LONG_TRANSACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LONG_TRANSACTION_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LONG_TRANSACTION_parent_get) {
  {
    struct _dwg_object_LONG_TRANSACTION *arg1 = (struct _dwg_object_LONG_TRANSACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LONG_TRANSACTION_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LONG_TRANSACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LONG_TRANSACTION_parent_get" "', argument " "1"" of type '" "struct _dwg_object_LONG_TRANSACTION *""'"); 
    }
    arg1 = (struct _dwg_object_LONG_TRANSACTION *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_LONG_TRANSACTION) {
  {
    int argvi = 0;
    struct _dwg_object_LONG_TRANSACTION *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_LONG_TRANSACTION();");
    }
    result = (struct _dwg_object_LONG_TRANSACTION *)calloc(1, sizeof(struct _dwg_object_LONG_TRANSACTION));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_LONG_TRANSACTION, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_LONG_TRANSACTION) {
  {
    struct _dwg_object_LONG_TRANSACTION *arg1 = (struct _dwg_object_LONG_TRANSACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_LONG_TRANSACTION(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LONG_TRANSACTION, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_LONG_TRANSACTION" "', argument " "1"" of type '" "struct _dwg_object_LONG_TRANSACTION *""'"); 
    }
    arg1 = (struct _dwg_object_LONG_TRANSACTION *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LWPOLYLINE_width_start_set) {
  {
    struct _dwg_LWPOLYLINE_width *arg1 = (struct _dwg_LWPOLYLINE_width *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LWPOLYLINE_width_start_set(self,start);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LWPOLYLINE_width, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LWPOLYLINE_width_start_set" "', argument " "1"" of type '" "struct _dwg_LWPOLYLINE_width *""'"); 
    }
    arg1 = (struct _dwg_LWPOLYLINE_width *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_LWPOLYLINE_width_start_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->start = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LWPOLYLINE_width_start_get) {
  {
    struct _dwg_LWPOLYLINE_width *arg1 = (struct _dwg_LWPOLYLINE_width *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LWPOLYLINE_width_start_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LWPOLYLINE_width, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LWPOLYLINE_width_start_get" "', argument " "1"" of type '" "struct _dwg_LWPOLYLINE_width *""'"); 
    }
    arg1 = (struct _dwg_LWPOLYLINE_width *)(argp1);
    result = (BITCODE_BD) ((arg1)->start);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LWPOLYLINE_width_end_set) {
  {
    struct _dwg_LWPOLYLINE_width *arg1 = (struct _dwg_LWPOLYLINE_width *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LWPOLYLINE_width_end_set(self,end);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LWPOLYLINE_width, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LWPOLYLINE_width_end_set" "', argument " "1"" of type '" "struct _dwg_LWPOLYLINE_width *""'"); 
    }
    arg1 = (struct _dwg_LWPOLYLINE_width *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_LWPOLYLINE_width_end_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->end = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LWPOLYLINE_width_end_get) {
  {
    struct _dwg_LWPOLYLINE_width *arg1 = (struct _dwg_LWPOLYLINE_width *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LWPOLYLINE_width_end_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LWPOLYLINE_width, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LWPOLYLINE_width_end_get" "', argument " "1"" of type '" "struct _dwg_LWPOLYLINE_width *""'"); 
    }
    arg1 = (struct _dwg_LWPOLYLINE_width *)(argp1);
    result = (BITCODE_BD) ((arg1)->end);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_LWPOLYLINE_width) {
  {
    int argvi = 0;
    struct _dwg_LWPOLYLINE_width *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_LWPOLYLINE_width();");
    }
    result = (struct _dwg_LWPOLYLINE_width *)calloc(1, sizeof(struct _dwg_LWPOLYLINE_width));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_LWPOLYLINE_width, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_LWPOLYLINE_width) {
  {
    struct _dwg_LWPOLYLINE_width *arg1 = (struct _dwg_LWPOLYLINE_width *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_LWPOLYLINE_width(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LWPOLYLINE_width, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_LWPOLYLINE_width" "', argument " "1"" of type '" "struct _dwg_LWPOLYLINE_width *""'"); 
    }
    arg1 = (struct _dwg_LWPOLYLINE_width *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PROXY_LWPOLYLINE_parent_set) {
  {
    struct _dwg_PROXY_LWPOLYLINE *arg1 = (struct _dwg_PROXY_LWPOLYLINE *) 0 ;
    struct _dwg_entity_PROXY_ENTITY *arg2 = (struct _dwg_entity_PROXY_ENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_PROXY_LWPOLYLINE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PROXY_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PROXY_LWPOLYLINE_parent_set" "', argument " "1"" of type '" "struct _dwg_PROXY_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_PROXY_LWPOLYLINE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_PROXY_ENTITY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_PROXY_LWPOLYLINE_parent_set" "', argument " "2"" of type '" "struct _dwg_entity_PROXY_ENTITY *""'"); 
    }
    arg2 = (struct _dwg_entity_PROXY_ENTITY *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PROXY_LWPOLYLINE_parent_get) {
  {
    struct _dwg_PROXY_LWPOLYLINE *arg1 = (struct _dwg_PROXY_LWPOLYLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_entity_PROXY_ENTITY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_PROXY_LWPOLYLINE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PROXY_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PROXY_LWPOLYLINE_parent_get" "', argument " "1"" of type '" "struct _dwg_PROXY_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_PROXY_LWPOLYLINE *)(argp1);
    result = (struct _dwg_entity_PROXY_ENTITY *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_PROXY_ENTITY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PROXY_LWPOLYLINE_data_size_set) {
  {
    struct _dwg_PROXY_LWPOLYLINE *arg1 = (struct _dwg_PROXY_LWPOLYLINE *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_PROXY_LWPOLYLINE_data_size_set(self,data_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PROXY_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PROXY_LWPOLYLINE_data_size_set" "', argument " "1"" of type '" "struct _dwg_PROXY_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_PROXY_LWPOLYLINE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_PROXY_LWPOLYLINE_data_size_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->data_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PROXY_LWPOLYLINE_data_size_get) {
  {
    struct _dwg_PROXY_LWPOLYLINE *arg1 = (struct _dwg_PROXY_LWPOLYLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_PROXY_LWPOLYLINE_data_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PROXY_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PROXY_LWPOLYLINE_data_size_get" "', argument " "1"" of type '" "struct _dwg_PROXY_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_PROXY_LWPOLYLINE *)(argp1);
    result = (BITCODE_RL) ((arg1)->data_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PROXY_LWPOLYLINE_flags_set) {
  {
    struct _dwg_PROXY_LWPOLYLINE *arg1 = (struct _dwg_PROXY_LWPOLYLINE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_PROXY_LWPOLYLINE_flags_set(self,flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PROXY_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PROXY_LWPOLYLINE_flags_set" "', argument " "1"" of type '" "struct _dwg_PROXY_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_PROXY_LWPOLYLINE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_PROXY_LWPOLYLINE_flags_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PROXY_LWPOLYLINE_flags_get) {
  {
    struct _dwg_PROXY_LWPOLYLINE *arg1 = (struct _dwg_PROXY_LWPOLYLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_PROXY_LWPOLYLINE_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PROXY_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PROXY_LWPOLYLINE_flags_get" "', argument " "1"" of type '" "struct _dwg_PROXY_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_PROXY_LWPOLYLINE *)(argp1);
    result = (BITCODE_BS) ((arg1)->flags);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PROXY_LWPOLYLINE_const_width_set) {
  {
    struct _dwg_PROXY_LWPOLYLINE *arg1 = (struct _dwg_PROXY_LWPOLYLINE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_PROXY_LWPOLYLINE_const_width_set(self,const_width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PROXY_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PROXY_LWPOLYLINE_const_width_set" "', argument " "1"" of type '" "struct _dwg_PROXY_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_PROXY_LWPOLYLINE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_PROXY_LWPOLYLINE_const_width_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->const_width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PROXY_LWPOLYLINE_const_width_get) {
  {
    struct _dwg_PROXY_LWPOLYLINE *arg1 = (struct _dwg_PROXY_LWPOLYLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_PROXY_LWPOLYLINE_const_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PROXY_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PROXY_LWPOLYLINE_const_width_get" "', argument " "1"" of type '" "struct _dwg_PROXY_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_PROXY_LWPOLYLINE *)(argp1);
    result = (BITCODE_BD) ((arg1)->const_width);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PROXY_LWPOLYLINE_elevation_set) {
  {
    struct _dwg_PROXY_LWPOLYLINE *arg1 = (struct _dwg_PROXY_LWPOLYLINE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_PROXY_LWPOLYLINE_elevation_set(self,elevation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PROXY_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PROXY_LWPOLYLINE_elevation_set" "', argument " "1"" of type '" "struct _dwg_PROXY_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_PROXY_LWPOLYLINE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_PROXY_LWPOLYLINE_elevation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->elevation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PROXY_LWPOLYLINE_elevation_get) {
  {
    struct _dwg_PROXY_LWPOLYLINE *arg1 = (struct _dwg_PROXY_LWPOLYLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_PROXY_LWPOLYLINE_elevation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PROXY_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PROXY_LWPOLYLINE_elevation_get" "', argument " "1"" of type '" "struct _dwg_PROXY_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_PROXY_LWPOLYLINE *)(argp1);
    result = (BITCODE_BD) ((arg1)->elevation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PROXY_LWPOLYLINE_thickness_set) {
  {
    struct _dwg_PROXY_LWPOLYLINE *arg1 = (struct _dwg_PROXY_LWPOLYLINE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_PROXY_LWPOLYLINE_thickness_set(self,thickness);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PROXY_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PROXY_LWPOLYLINE_thickness_set" "', argument " "1"" of type '" "struct _dwg_PROXY_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_PROXY_LWPOLYLINE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_PROXY_LWPOLYLINE_thickness_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->thickness = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PROXY_LWPOLYLINE_thickness_get) {
  {
    struct _dwg_PROXY_LWPOLYLINE *arg1 = (struct _dwg_PROXY_LWPOLYLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_PROXY_LWPOLYLINE_thickness_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PROXY_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PROXY_LWPOLYLINE_thickness_get" "', argument " "1"" of type '" "struct _dwg_PROXY_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_PROXY_LWPOLYLINE *)(argp1);
    result = (BITCODE_BD) ((arg1)->thickness);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PROXY_LWPOLYLINE_extrusion_set) {
  {
    struct _dwg_PROXY_LWPOLYLINE *arg1 = (struct _dwg_PROXY_LWPOLYLINE *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_PROXY_LWPOLYLINE_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PROXY_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PROXY_LWPOLYLINE_extrusion_set" "', argument " "1"" of type '" "struct _dwg_PROXY_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_PROXY_LWPOLYLINE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_PROXY_LWPOLYLINE_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PROXY_LWPOLYLINE_extrusion_get) {
  {
    struct _dwg_PROXY_LWPOLYLINE *arg1 = (struct _dwg_PROXY_LWPOLYLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_PROXY_LWPOLYLINE_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PROXY_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PROXY_LWPOLYLINE_extrusion_get" "', argument " "1"" of type '" "struct _dwg_PROXY_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_PROXY_LWPOLYLINE *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PROXY_LWPOLYLINE_num_points_set) {
  {
    struct _dwg_PROXY_LWPOLYLINE *arg1 = (struct _dwg_PROXY_LWPOLYLINE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_PROXY_LWPOLYLINE_num_points_set(self,num_points);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PROXY_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PROXY_LWPOLYLINE_num_points_set" "', argument " "1"" of type '" "struct _dwg_PROXY_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_PROXY_LWPOLYLINE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_PROXY_LWPOLYLINE_num_points_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_points = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PROXY_LWPOLYLINE_num_points_get) {
  {
    struct _dwg_PROXY_LWPOLYLINE *arg1 = (struct _dwg_PROXY_LWPOLYLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_PROXY_LWPOLYLINE_num_points_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PROXY_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PROXY_LWPOLYLINE_num_points_get" "', argument " "1"" of type '" "struct _dwg_PROXY_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_PROXY_LWPOLYLINE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_points);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PROXY_LWPOLYLINE_points_set) {
  {
    struct _dwg_PROXY_LWPOLYLINE *arg1 = (struct _dwg_PROXY_LWPOLYLINE *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_PROXY_LWPOLYLINE_points_set(self,points);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PROXY_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PROXY_LWPOLYLINE_points_set" "', argument " "1"" of type '" "struct _dwg_PROXY_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_PROXY_LWPOLYLINE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_PROXY_LWPOLYLINE_points_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->points = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PROXY_LWPOLYLINE_points_get) {
  {
    struct _dwg_PROXY_LWPOLYLINE *arg1 = (struct _dwg_PROXY_LWPOLYLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_PROXY_LWPOLYLINE_points_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PROXY_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PROXY_LWPOLYLINE_points_get" "', argument " "1"" of type '" "struct _dwg_PROXY_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_PROXY_LWPOLYLINE *)(argp1);
    result = (BITCODE_2RD *) ((arg1)->points);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PROXY_LWPOLYLINE_num_bulges_set) {
  {
    struct _dwg_PROXY_LWPOLYLINE *arg1 = (struct _dwg_PROXY_LWPOLYLINE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_PROXY_LWPOLYLINE_num_bulges_set(self,num_bulges);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PROXY_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PROXY_LWPOLYLINE_num_bulges_set" "', argument " "1"" of type '" "struct _dwg_PROXY_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_PROXY_LWPOLYLINE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_PROXY_LWPOLYLINE_num_bulges_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_bulges = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PROXY_LWPOLYLINE_num_bulges_get) {
  {
    struct _dwg_PROXY_LWPOLYLINE *arg1 = (struct _dwg_PROXY_LWPOLYLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_PROXY_LWPOLYLINE_num_bulges_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PROXY_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PROXY_LWPOLYLINE_num_bulges_get" "', argument " "1"" of type '" "struct _dwg_PROXY_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_PROXY_LWPOLYLINE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_bulges);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PROXY_LWPOLYLINE_bulges_set) {
  {
    struct _dwg_PROXY_LWPOLYLINE *arg1 = (struct _dwg_PROXY_LWPOLYLINE *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_PROXY_LWPOLYLINE_bulges_set(self,bulges);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PROXY_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PROXY_LWPOLYLINE_bulges_set" "', argument " "1"" of type '" "struct _dwg_PROXY_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_PROXY_LWPOLYLINE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_PROXY_LWPOLYLINE_bulges_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->bulges = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PROXY_LWPOLYLINE_bulges_get) {
  {
    struct _dwg_PROXY_LWPOLYLINE *arg1 = (struct _dwg_PROXY_LWPOLYLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_PROXY_LWPOLYLINE_bulges_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PROXY_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PROXY_LWPOLYLINE_bulges_get" "', argument " "1"" of type '" "struct _dwg_PROXY_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_PROXY_LWPOLYLINE *)(argp1);
    result = (BITCODE_BD *) ((arg1)->bulges);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PROXY_LWPOLYLINE_num_widths_set) {
  {
    struct _dwg_PROXY_LWPOLYLINE *arg1 = (struct _dwg_PROXY_LWPOLYLINE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_PROXY_LWPOLYLINE_num_widths_set(self,num_widths);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PROXY_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PROXY_LWPOLYLINE_num_widths_set" "', argument " "1"" of type '" "struct _dwg_PROXY_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_PROXY_LWPOLYLINE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_PROXY_LWPOLYLINE_num_widths_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_widths = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PROXY_LWPOLYLINE_num_widths_get) {
  {
    struct _dwg_PROXY_LWPOLYLINE *arg1 = (struct _dwg_PROXY_LWPOLYLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_PROXY_LWPOLYLINE_num_widths_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PROXY_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PROXY_LWPOLYLINE_num_widths_get" "', argument " "1"" of type '" "struct _dwg_PROXY_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_PROXY_LWPOLYLINE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_widths);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PROXY_LWPOLYLINE_widths_set) {
  {
    struct _dwg_PROXY_LWPOLYLINE *arg1 = (struct _dwg_PROXY_LWPOLYLINE *) 0 ;
    Dwg_LWPOLYLINE_width *arg2 = (Dwg_LWPOLYLINE_width *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_PROXY_LWPOLYLINE_widths_set(self,widths);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PROXY_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PROXY_LWPOLYLINE_widths_set" "', argument " "1"" of type '" "struct _dwg_PROXY_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_PROXY_LWPOLYLINE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_LWPOLYLINE_width, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_PROXY_LWPOLYLINE_widths_set" "', argument " "2"" of type '" "Dwg_LWPOLYLINE_width *""'"); 
    }
    arg2 = (Dwg_LWPOLYLINE_width *)(argp2);
    if (arg1) (arg1)->widths = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PROXY_LWPOLYLINE_widths_get) {
  {
    struct _dwg_PROXY_LWPOLYLINE *arg1 = (struct _dwg_PROXY_LWPOLYLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_LWPOLYLINE_width *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_PROXY_LWPOLYLINE_widths_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PROXY_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PROXY_LWPOLYLINE_widths_get" "', argument " "1"" of type '" "struct _dwg_PROXY_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_PROXY_LWPOLYLINE *)(argp1);
    result = (Dwg_LWPOLYLINE_width *) ((arg1)->widths);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_LWPOLYLINE_width, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PROXY_LWPOLYLINE_unknown_1_set) {
  {
    struct _dwg_PROXY_LWPOLYLINE *arg1 = (struct _dwg_PROXY_LWPOLYLINE *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_PROXY_LWPOLYLINE_unknown_1_set(self,unknown_1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PROXY_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PROXY_LWPOLYLINE_unknown_1_set" "', argument " "1"" of type '" "struct _dwg_PROXY_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_PROXY_LWPOLYLINE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_PROXY_LWPOLYLINE_unknown_1_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->unknown_1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PROXY_LWPOLYLINE_unknown_1_get) {
  {
    struct _dwg_PROXY_LWPOLYLINE *arg1 = (struct _dwg_PROXY_LWPOLYLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_PROXY_LWPOLYLINE_unknown_1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PROXY_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PROXY_LWPOLYLINE_unknown_1_get" "', argument " "1"" of type '" "struct _dwg_PROXY_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_PROXY_LWPOLYLINE *)(argp1);
    result = (BITCODE_RC) ((arg1)->unknown_1);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PROXY_LWPOLYLINE_unknown_2_set) {
  {
    struct _dwg_PROXY_LWPOLYLINE *arg1 = (struct _dwg_PROXY_LWPOLYLINE *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_PROXY_LWPOLYLINE_unknown_2_set(self,unknown_2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PROXY_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PROXY_LWPOLYLINE_unknown_2_set" "', argument " "1"" of type '" "struct _dwg_PROXY_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_PROXY_LWPOLYLINE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_PROXY_LWPOLYLINE_unknown_2_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->unknown_2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PROXY_LWPOLYLINE_unknown_2_get) {
  {
    struct _dwg_PROXY_LWPOLYLINE *arg1 = (struct _dwg_PROXY_LWPOLYLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_PROXY_LWPOLYLINE_unknown_2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PROXY_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PROXY_LWPOLYLINE_unknown_2_get" "', argument " "1"" of type '" "struct _dwg_PROXY_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_PROXY_LWPOLYLINE *)(argp1);
    result = (BITCODE_RC) ((arg1)->unknown_2);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PROXY_LWPOLYLINE_unknown_3_set) {
  {
    struct _dwg_PROXY_LWPOLYLINE *arg1 = (struct _dwg_PROXY_LWPOLYLINE *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_PROXY_LWPOLYLINE_unknown_3_set(self,unknown_3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PROXY_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PROXY_LWPOLYLINE_unknown_3_set" "', argument " "1"" of type '" "struct _dwg_PROXY_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_PROXY_LWPOLYLINE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_PROXY_LWPOLYLINE_unknown_3_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->unknown_3 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PROXY_LWPOLYLINE_unknown_3_get) {
  {
    struct _dwg_PROXY_LWPOLYLINE *arg1 = (struct _dwg_PROXY_LWPOLYLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_PROXY_LWPOLYLINE_unknown_3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PROXY_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PROXY_LWPOLYLINE_unknown_3_get" "', argument " "1"" of type '" "struct _dwg_PROXY_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_PROXY_LWPOLYLINE *)(argp1);
    result = (BITCODE_RC) ((arg1)->unknown_3);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_PROXY_LWPOLYLINE) {
  {
    int argvi = 0;
    struct _dwg_PROXY_LWPOLYLINE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_PROXY_LWPOLYLINE();");
    }
    result = (struct _dwg_PROXY_LWPOLYLINE *)calloc(1, sizeof(struct _dwg_PROXY_LWPOLYLINE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_PROXY_LWPOLYLINE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_PROXY_LWPOLYLINE) {
  {
    struct _dwg_PROXY_LWPOLYLINE *arg1 = (struct _dwg_PROXY_LWPOLYLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_PROXY_LWPOLYLINE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PROXY_LWPOLYLINE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_PROXY_LWPOLYLINE" "', argument " "1"" of type '" "struct _dwg_PROXY_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_PROXY_LWPOLYLINE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PROXY_ENTITY_parent_set) {
  {
    struct _dwg_entity_PROXY_ENTITY *arg1 = (struct _dwg_entity_PROXY_ENTITY *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PROXY_ENTITY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PROXY_ENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PROXY_ENTITY_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_PROXY_ENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_PROXY_ENTITY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_PROXY_ENTITY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PROXY_ENTITY_parent_get) {
  {
    struct _dwg_entity_PROXY_ENTITY *arg1 = (struct _dwg_entity_PROXY_ENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PROXY_ENTITY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PROXY_ENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PROXY_ENTITY_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_PROXY_ENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_PROXY_ENTITY *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PROXY_ENTITY_proxy_id_set) {
  {
    struct _dwg_entity_PROXY_ENTITY *arg1 = (struct _dwg_entity_PROXY_ENTITY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PROXY_ENTITY_proxy_id_set(self,proxy_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PROXY_ENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PROXY_ENTITY_proxy_id_set" "', argument " "1"" of type '" "struct _dwg_entity_PROXY_ENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_PROXY_ENTITY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_PROXY_ENTITY_proxy_id_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->proxy_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PROXY_ENTITY_proxy_id_get) {
  {
    struct _dwg_entity_PROXY_ENTITY *arg1 = (struct _dwg_entity_PROXY_ENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PROXY_ENTITY_proxy_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PROXY_ENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PROXY_ENTITY_proxy_id_get" "', argument " "1"" of type '" "struct _dwg_entity_PROXY_ENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_PROXY_ENTITY *)(argp1);
    result = (BITCODE_BL) ((arg1)->proxy_id);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PROXY_ENTITY_class_id_set) {
  {
    struct _dwg_entity_PROXY_ENTITY *arg1 = (struct _dwg_entity_PROXY_ENTITY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PROXY_ENTITY_class_id_set(self,class_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PROXY_ENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PROXY_ENTITY_class_id_set" "', argument " "1"" of type '" "struct _dwg_entity_PROXY_ENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_PROXY_ENTITY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_PROXY_ENTITY_class_id_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PROXY_ENTITY_class_id_get) {
  {
    struct _dwg_entity_PROXY_ENTITY *arg1 = (struct _dwg_entity_PROXY_ENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PROXY_ENTITY_class_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PROXY_ENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PROXY_ENTITY_class_id_get" "', argument " "1"" of type '" "struct _dwg_entity_PROXY_ENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_PROXY_ENTITY *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_id);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PROXY_ENTITY_dwg_versions_set) {
  {
    struct _dwg_entity_PROXY_ENTITY *arg1 = (struct _dwg_entity_PROXY_ENTITY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PROXY_ENTITY_dwg_versions_set(self,dwg_versions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PROXY_ENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PROXY_ENTITY_dwg_versions_set" "', argument " "1"" of type '" "struct _dwg_entity_PROXY_ENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_PROXY_ENTITY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_PROXY_ENTITY_dwg_versions_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->dwg_versions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PROXY_ENTITY_dwg_versions_get) {
  {
    struct _dwg_entity_PROXY_ENTITY *arg1 = (struct _dwg_entity_PROXY_ENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PROXY_ENTITY_dwg_versions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PROXY_ENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PROXY_ENTITY_dwg_versions_get" "', argument " "1"" of type '" "struct _dwg_entity_PROXY_ENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_PROXY_ENTITY *)(argp1);
    result = (BITCODE_BL) ((arg1)->dwg_versions);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PROXY_ENTITY_maint_version_set) {
  {
    struct _dwg_entity_PROXY_ENTITY *arg1 = (struct _dwg_entity_PROXY_ENTITY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PROXY_ENTITY_maint_version_set(self,maint_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PROXY_ENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PROXY_ENTITY_maint_version_set" "', argument " "1"" of type '" "struct _dwg_entity_PROXY_ENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_PROXY_ENTITY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_PROXY_ENTITY_maint_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->maint_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PROXY_ENTITY_maint_version_get) {
  {
    struct _dwg_entity_PROXY_ENTITY *arg1 = (struct _dwg_entity_PROXY_ENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PROXY_ENTITY_maint_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PROXY_ENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PROXY_ENTITY_maint_version_get" "', argument " "1"" of type '" "struct _dwg_entity_PROXY_ENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_PROXY_ENTITY *)(argp1);
    result = (BITCODE_BL) ((arg1)->maint_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PROXY_ENTITY_dwg_version_set) {
  {
    struct _dwg_entity_PROXY_ENTITY *arg1 = (struct _dwg_entity_PROXY_ENTITY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PROXY_ENTITY_dwg_version_set(self,dwg_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PROXY_ENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PROXY_ENTITY_dwg_version_set" "', argument " "1"" of type '" "struct _dwg_entity_PROXY_ENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_PROXY_ENTITY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_PROXY_ENTITY_dwg_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->dwg_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PROXY_ENTITY_dwg_version_get) {
  {
    struct _dwg_entity_PROXY_ENTITY *arg1 = (struct _dwg_entity_PROXY_ENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PROXY_ENTITY_dwg_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PROXY_ENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PROXY_ENTITY_dwg_version_get" "', argument " "1"" of type '" "struct _dwg_entity_PROXY_ENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_PROXY_ENTITY *)(argp1);
    result = (BITCODE_BL) ((arg1)->dwg_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PROXY_ENTITY_from_dxf_set) {
  {
    struct _dwg_entity_PROXY_ENTITY *arg1 = (struct _dwg_entity_PROXY_ENTITY *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PROXY_ENTITY_from_dxf_set(self,from_dxf);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PROXY_ENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PROXY_ENTITY_from_dxf_set" "', argument " "1"" of type '" "struct _dwg_entity_PROXY_ENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_PROXY_ENTITY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_PROXY_ENTITY_from_dxf_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->from_dxf = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PROXY_ENTITY_from_dxf_get) {
  {
    struct _dwg_entity_PROXY_ENTITY *arg1 = (struct _dwg_entity_PROXY_ENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PROXY_ENTITY_from_dxf_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PROXY_ENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PROXY_ENTITY_from_dxf_get" "', argument " "1"" of type '" "struct _dwg_entity_PROXY_ENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_PROXY_ENTITY *)(argp1);
    result = (BITCODE_B) ((arg1)->from_dxf);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PROXY_ENTITY_proxy_data_size_set) {
  {
    struct _dwg_entity_PROXY_ENTITY *arg1 = (struct _dwg_entity_PROXY_ENTITY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PROXY_ENTITY_proxy_data_size_set(self,proxy_data_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PROXY_ENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PROXY_ENTITY_proxy_data_size_set" "', argument " "1"" of type '" "struct _dwg_entity_PROXY_ENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_PROXY_ENTITY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_PROXY_ENTITY_proxy_data_size_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->proxy_data_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PROXY_ENTITY_proxy_data_size_get) {
  {
    struct _dwg_entity_PROXY_ENTITY *arg1 = (struct _dwg_entity_PROXY_ENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PROXY_ENTITY_proxy_data_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PROXY_ENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PROXY_ENTITY_proxy_data_size_get" "', argument " "1"" of type '" "struct _dwg_entity_PROXY_ENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_PROXY_ENTITY *)(argp1);
    result = (BITCODE_BL) ((arg1)->proxy_data_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PROXY_ENTITY_proxy_data_set) {
  {
    struct _dwg_entity_PROXY_ENTITY *arg1 = (struct _dwg_entity_PROXY_ENTITY *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PROXY_ENTITY_proxy_data_set(self,proxy_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PROXY_ENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PROXY_ENTITY_proxy_data_set" "', argument " "1"" of type '" "struct _dwg_entity_PROXY_ENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_PROXY_ENTITY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_PROXY_ENTITY_proxy_data_set" "', argument " "2"" of type '" "BITCODE_RC *""'"); 
    }
    arg2 = (BITCODE_RC *)(argp2);
    if (arg1) (arg1)->proxy_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PROXY_ENTITY_proxy_data_get) {
  {
    struct _dwg_entity_PROXY_ENTITY *arg1 = (struct _dwg_entity_PROXY_ENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PROXY_ENTITY_proxy_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PROXY_ENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PROXY_ENTITY_proxy_data_get" "', argument " "1"" of type '" "struct _dwg_entity_PROXY_ENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_PROXY_ENTITY *)(argp1);
    result = (BITCODE_RC *) ((arg1)->proxy_data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PROXY_ENTITY_data_numbits_set) {
  {
    struct _dwg_entity_PROXY_ENTITY *arg1 = (struct _dwg_entity_PROXY_ENTITY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PROXY_ENTITY_data_numbits_set(self,data_numbits);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PROXY_ENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PROXY_ENTITY_data_numbits_set" "', argument " "1"" of type '" "struct _dwg_entity_PROXY_ENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_PROXY_ENTITY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_PROXY_ENTITY_data_numbits_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->data_numbits = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PROXY_ENTITY_data_numbits_get) {
  {
    struct _dwg_entity_PROXY_ENTITY *arg1 = (struct _dwg_entity_PROXY_ENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PROXY_ENTITY_data_numbits_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PROXY_ENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PROXY_ENTITY_data_numbits_get" "', argument " "1"" of type '" "struct _dwg_entity_PROXY_ENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_PROXY_ENTITY *)(argp1);
    result = (BITCODE_BL) ((arg1)->data_numbits);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PROXY_ENTITY_data_size_set) {
  {
    struct _dwg_entity_PROXY_ENTITY *arg1 = (struct _dwg_entity_PROXY_ENTITY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PROXY_ENTITY_data_size_set(self,data_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PROXY_ENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PROXY_ENTITY_data_size_set" "', argument " "1"" of type '" "struct _dwg_entity_PROXY_ENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_PROXY_ENTITY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_PROXY_ENTITY_data_size_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->data_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PROXY_ENTITY_data_size_get) {
  {
    struct _dwg_entity_PROXY_ENTITY *arg1 = (struct _dwg_entity_PROXY_ENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PROXY_ENTITY_data_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PROXY_ENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PROXY_ENTITY_data_size_get" "', argument " "1"" of type '" "struct _dwg_entity_PROXY_ENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_PROXY_ENTITY *)(argp1);
    result = (BITCODE_BL) ((arg1)->data_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PROXY_ENTITY_data_set) {
  {
    struct _dwg_entity_PROXY_ENTITY *arg1 = (struct _dwg_entity_PROXY_ENTITY *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PROXY_ENTITY_data_set(self,data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PROXY_ENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PROXY_ENTITY_data_set" "', argument " "1"" of type '" "struct _dwg_entity_PROXY_ENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_PROXY_ENTITY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_PROXY_ENTITY_data_set" "', argument " "2"" of type '" "BITCODE_RC *""'"); 
    }
    arg2 = (BITCODE_RC *)(argp2);
    if (arg1) (arg1)->data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PROXY_ENTITY_data_get) {
  {
    struct _dwg_entity_PROXY_ENTITY *arg1 = (struct _dwg_entity_PROXY_ENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PROXY_ENTITY_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PROXY_ENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PROXY_ENTITY_data_get" "', argument " "1"" of type '" "struct _dwg_entity_PROXY_ENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_PROXY_ENTITY *)(argp1);
    result = (BITCODE_RC *) ((arg1)->data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PROXY_ENTITY_num_objids_set) {
  {
    struct _dwg_entity_PROXY_ENTITY *arg1 = (struct _dwg_entity_PROXY_ENTITY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PROXY_ENTITY_num_objids_set(self,num_objids);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PROXY_ENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PROXY_ENTITY_num_objids_set" "', argument " "1"" of type '" "struct _dwg_entity_PROXY_ENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_PROXY_ENTITY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_PROXY_ENTITY_num_objids_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_objids = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PROXY_ENTITY_num_objids_get) {
  {
    struct _dwg_entity_PROXY_ENTITY *arg1 = (struct _dwg_entity_PROXY_ENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PROXY_ENTITY_num_objids_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PROXY_ENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PROXY_ENTITY_num_objids_get" "', argument " "1"" of type '" "struct _dwg_entity_PROXY_ENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_PROXY_ENTITY *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_objids);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PROXY_ENTITY_objids_set) {
  {
    struct _dwg_entity_PROXY_ENTITY *arg1 = (struct _dwg_entity_PROXY_ENTITY *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PROXY_ENTITY_objids_set(self,objids);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PROXY_ENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PROXY_ENTITY_objids_set" "', argument " "1"" of type '" "struct _dwg_entity_PROXY_ENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_PROXY_ENTITY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_PROXY_ENTITY_objids_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->objids = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PROXY_ENTITY_objids_get) {
  {
    struct _dwg_entity_PROXY_ENTITY *arg1 = (struct _dwg_entity_PROXY_ENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PROXY_ENTITY_objids_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PROXY_ENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PROXY_ENTITY_objids_get" "', argument " "1"" of type '" "struct _dwg_entity_PROXY_ENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_PROXY_ENTITY *)(argp1);
    result = (BITCODE_H *) ((arg1)->objids);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_PROXY_ENTITY) {
  {
    int argvi = 0;
    struct _dwg_entity_PROXY_ENTITY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_PROXY_ENTITY();");
    }
    result = (struct _dwg_entity_PROXY_ENTITY *)calloc(1, sizeof(struct _dwg_entity_PROXY_ENTITY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_PROXY_ENTITY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_PROXY_ENTITY) {
  {
    struct _dwg_entity_PROXY_ENTITY *arg1 = (struct _dwg_entity_PROXY_ENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_PROXY_ENTITY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PROXY_ENTITY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_PROXY_ENTITY" "', argument " "1"" of type '" "struct _dwg_entity_PROXY_ENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_PROXY_ENTITY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PROXY_OBJECT_parent_set) {
  {
    struct _dwg_object_PROXY_OBJECT *arg1 = (struct _dwg_object_PROXY_OBJECT *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PROXY_OBJECT_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PROXY_OBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PROXY_OBJECT_parent_set" "', argument " "1"" of type '" "struct _dwg_object_PROXY_OBJECT *""'"); 
    }
    arg1 = (struct _dwg_object_PROXY_OBJECT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_PROXY_OBJECT_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PROXY_OBJECT_parent_get) {
  {
    struct _dwg_object_PROXY_OBJECT *arg1 = (struct _dwg_object_PROXY_OBJECT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PROXY_OBJECT_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PROXY_OBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PROXY_OBJECT_parent_get" "', argument " "1"" of type '" "struct _dwg_object_PROXY_OBJECT *""'"); 
    }
    arg1 = (struct _dwg_object_PROXY_OBJECT *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PROXY_OBJECT_proxy_id_set) {
  {
    struct _dwg_object_PROXY_OBJECT *arg1 = (struct _dwg_object_PROXY_OBJECT *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PROXY_OBJECT_proxy_id_set(self,proxy_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PROXY_OBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PROXY_OBJECT_proxy_id_set" "', argument " "1"" of type '" "struct _dwg_object_PROXY_OBJECT *""'"); 
    }
    arg1 = (struct _dwg_object_PROXY_OBJECT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_PROXY_OBJECT_proxy_id_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->proxy_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PROXY_OBJECT_proxy_id_get) {
  {
    struct _dwg_object_PROXY_OBJECT *arg1 = (struct _dwg_object_PROXY_OBJECT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PROXY_OBJECT_proxy_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PROXY_OBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PROXY_OBJECT_proxy_id_get" "', argument " "1"" of type '" "struct _dwg_object_PROXY_OBJECT *""'"); 
    }
    arg1 = (struct _dwg_object_PROXY_OBJECT *)(argp1);
    result = (BITCODE_BL) ((arg1)->proxy_id);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PROXY_OBJECT_class_id_set) {
  {
    struct _dwg_object_PROXY_OBJECT *arg1 = (struct _dwg_object_PROXY_OBJECT *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PROXY_OBJECT_class_id_set(self,class_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PROXY_OBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PROXY_OBJECT_class_id_set" "', argument " "1"" of type '" "struct _dwg_object_PROXY_OBJECT *""'"); 
    }
    arg1 = (struct _dwg_object_PROXY_OBJECT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_PROXY_OBJECT_class_id_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PROXY_OBJECT_class_id_get) {
  {
    struct _dwg_object_PROXY_OBJECT *arg1 = (struct _dwg_object_PROXY_OBJECT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PROXY_OBJECT_class_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PROXY_OBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PROXY_OBJECT_class_id_get" "', argument " "1"" of type '" "struct _dwg_object_PROXY_OBJECT *""'"); 
    }
    arg1 = (struct _dwg_object_PROXY_OBJECT *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_id);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PROXY_OBJECT_dwg_versions_set) {
  {
    struct _dwg_object_PROXY_OBJECT *arg1 = (struct _dwg_object_PROXY_OBJECT *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PROXY_OBJECT_dwg_versions_set(self,dwg_versions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PROXY_OBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PROXY_OBJECT_dwg_versions_set" "', argument " "1"" of type '" "struct _dwg_object_PROXY_OBJECT *""'"); 
    }
    arg1 = (struct _dwg_object_PROXY_OBJECT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_PROXY_OBJECT_dwg_versions_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->dwg_versions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PROXY_OBJECT_dwg_versions_get) {
  {
    struct _dwg_object_PROXY_OBJECT *arg1 = (struct _dwg_object_PROXY_OBJECT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PROXY_OBJECT_dwg_versions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PROXY_OBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PROXY_OBJECT_dwg_versions_get" "', argument " "1"" of type '" "struct _dwg_object_PROXY_OBJECT *""'"); 
    }
    arg1 = (struct _dwg_object_PROXY_OBJECT *)(argp1);
    result = (BITCODE_BL) ((arg1)->dwg_versions);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PROXY_OBJECT_maint_version_set) {
  {
    struct _dwg_object_PROXY_OBJECT *arg1 = (struct _dwg_object_PROXY_OBJECT *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PROXY_OBJECT_maint_version_set(self,maint_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PROXY_OBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PROXY_OBJECT_maint_version_set" "', argument " "1"" of type '" "struct _dwg_object_PROXY_OBJECT *""'"); 
    }
    arg1 = (struct _dwg_object_PROXY_OBJECT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_PROXY_OBJECT_maint_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->maint_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PROXY_OBJECT_maint_version_get) {
  {
    struct _dwg_object_PROXY_OBJECT *arg1 = (struct _dwg_object_PROXY_OBJECT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PROXY_OBJECT_maint_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PROXY_OBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PROXY_OBJECT_maint_version_get" "', argument " "1"" of type '" "struct _dwg_object_PROXY_OBJECT *""'"); 
    }
    arg1 = (struct _dwg_object_PROXY_OBJECT *)(argp1);
    result = (BITCODE_BL) ((arg1)->maint_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PROXY_OBJECT_dwg_version_set) {
  {
    struct _dwg_object_PROXY_OBJECT *arg1 = (struct _dwg_object_PROXY_OBJECT *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PROXY_OBJECT_dwg_version_set(self,dwg_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PROXY_OBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PROXY_OBJECT_dwg_version_set" "', argument " "1"" of type '" "struct _dwg_object_PROXY_OBJECT *""'"); 
    }
    arg1 = (struct _dwg_object_PROXY_OBJECT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_PROXY_OBJECT_dwg_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->dwg_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PROXY_OBJECT_dwg_version_get) {
  {
    struct _dwg_object_PROXY_OBJECT *arg1 = (struct _dwg_object_PROXY_OBJECT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PROXY_OBJECT_dwg_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PROXY_OBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PROXY_OBJECT_dwg_version_get" "', argument " "1"" of type '" "struct _dwg_object_PROXY_OBJECT *""'"); 
    }
    arg1 = (struct _dwg_object_PROXY_OBJECT *)(argp1);
    result = (BITCODE_BL) ((arg1)->dwg_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PROXY_OBJECT_from_dxf_set) {
  {
    struct _dwg_object_PROXY_OBJECT *arg1 = (struct _dwg_object_PROXY_OBJECT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PROXY_OBJECT_from_dxf_set(self,from_dxf);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PROXY_OBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PROXY_OBJECT_from_dxf_set" "', argument " "1"" of type '" "struct _dwg_object_PROXY_OBJECT *""'"); 
    }
    arg1 = (struct _dwg_object_PROXY_OBJECT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_PROXY_OBJECT_from_dxf_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->from_dxf = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PROXY_OBJECT_from_dxf_get) {
  {
    struct _dwg_object_PROXY_OBJECT *arg1 = (struct _dwg_object_PROXY_OBJECT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PROXY_OBJECT_from_dxf_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PROXY_OBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PROXY_OBJECT_from_dxf_get" "', argument " "1"" of type '" "struct _dwg_object_PROXY_OBJECT *""'"); 
    }
    arg1 = (struct _dwg_object_PROXY_OBJECT *)(argp1);
    result = (BITCODE_B) ((arg1)->from_dxf);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PROXY_OBJECT_data_numbits_set) {
  {
    struct _dwg_object_PROXY_OBJECT *arg1 = (struct _dwg_object_PROXY_OBJECT *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PROXY_OBJECT_data_numbits_set(self,data_numbits);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PROXY_OBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PROXY_OBJECT_data_numbits_set" "', argument " "1"" of type '" "struct _dwg_object_PROXY_OBJECT *""'"); 
    }
    arg1 = (struct _dwg_object_PROXY_OBJECT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_PROXY_OBJECT_data_numbits_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->data_numbits = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PROXY_OBJECT_data_numbits_get) {
  {
    struct _dwg_object_PROXY_OBJECT *arg1 = (struct _dwg_object_PROXY_OBJECT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PROXY_OBJECT_data_numbits_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PROXY_OBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PROXY_OBJECT_data_numbits_get" "', argument " "1"" of type '" "struct _dwg_object_PROXY_OBJECT *""'"); 
    }
    arg1 = (struct _dwg_object_PROXY_OBJECT *)(argp1);
    result = (BITCODE_BL) ((arg1)->data_numbits);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PROXY_OBJECT_data_size_set) {
  {
    struct _dwg_object_PROXY_OBJECT *arg1 = (struct _dwg_object_PROXY_OBJECT *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PROXY_OBJECT_data_size_set(self,data_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PROXY_OBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PROXY_OBJECT_data_size_set" "', argument " "1"" of type '" "struct _dwg_object_PROXY_OBJECT *""'"); 
    }
    arg1 = (struct _dwg_object_PROXY_OBJECT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_PROXY_OBJECT_data_size_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->data_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PROXY_OBJECT_data_size_get) {
  {
    struct _dwg_object_PROXY_OBJECT *arg1 = (struct _dwg_object_PROXY_OBJECT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PROXY_OBJECT_data_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PROXY_OBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PROXY_OBJECT_data_size_get" "', argument " "1"" of type '" "struct _dwg_object_PROXY_OBJECT *""'"); 
    }
    arg1 = (struct _dwg_object_PROXY_OBJECT *)(argp1);
    result = (BITCODE_BL) ((arg1)->data_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PROXY_OBJECT_data_set) {
  {
    struct _dwg_object_PROXY_OBJECT *arg1 = (struct _dwg_object_PROXY_OBJECT *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PROXY_OBJECT_data_set(self,data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PROXY_OBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PROXY_OBJECT_data_set" "', argument " "1"" of type '" "struct _dwg_object_PROXY_OBJECT *""'"); 
    }
    arg1 = (struct _dwg_object_PROXY_OBJECT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_PROXY_OBJECT_data_set" "', argument " "2"" of type '" "BITCODE_RC *""'"); 
    }
    arg2 = (BITCODE_RC *)(argp2);
    if (arg1) (arg1)->data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PROXY_OBJECT_data_get) {
  {
    struct _dwg_object_PROXY_OBJECT *arg1 = (struct _dwg_object_PROXY_OBJECT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PROXY_OBJECT_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PROXY_OBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PROXY_OBJECT_data_get" "', argument " "1"" of type '" "struct _dwg_object_PROXY_OBJECT *""'"); 
    }
    arg1 = (struct _dwg_object_PROXY_OBJECT *)(argp1);
    result = (BITCODE_RC *) ((arg1)->data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PROXY_OBJECT_num_objids_set) {
  {
    struct _dwg_object_PROXY_OBJECT *arg1 = (struct _dwg_object_PROXY_OBJECT *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PROXY_OBJECT_num_objids_set(self,num_objids);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PROXY_OBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PROXY_OBJECT_num_objids_set" "', argument " "1"" of type '" "struct _dwg_object_PROXY_OBJECT *""'"); 
    }
    arg1 = (struct _dwg_object_PROXY_OBJECT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_PROXY_OBJECT_num_objids_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_objids = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PROXY_OBJECT_num_objids_get) {
  {
    struct _dwg_object_PROXY_OBJECT *arg1 = (struct _dwg_object_PROXY_OBJECT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PROXY_OBJECT_num_objids_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PROXY_OBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PROXY_OBJECT_num_objids_get" "', argument " "1"" of type '" "struct _dwg_object_PROXY_OBJECT *""'"); 
    }
    arg1 = (struct _dwg_object_PROXY_OBJECT *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_objids);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PROXY_OBJECT_objids_set) {
  {
    struct _dwg_object_PROXY_OBJECT *arg1 = (struct _dwg_object_PROXY_OBJECT *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PROXY_OBJECT_objids_set(self,objids);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PROXY_OBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PROXY_OBJECT_objids_set" "', argument " "1"" of type '" "struct _dwg_object_PROXY_OBJECT *""'"); 
    }
    arg1 = (struct _dwg_object_PROXY_OBJECT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_PROXY_OBJECT_objids_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->objids = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PROXY_OBJECT_objids_get) {
  {
    struct _dwg_object_PROXY_OBJECT *arg1 = (struct _dwg_object_PROXY_OBJECT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PROXY_OBJECT_objids_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PROXY_OBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PROXY_OBJECT_objids_get" "', argument " "1"" of type '" "struct _dwg_object_PROXY_OBJECT *""'"); 
    }
    arg1 = (struct _dwg_object_PROXY_OBJECT *)(argp1);
    result = (BITCODE_H *) ((arg1)->objids);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_PROXY_OBJECT) {
  {
    int argvi = 0;
    struct _dwg_object_PROXY_OBJECT *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_PROXY_OBJECT();");
    }
    result = (struct _dwg_object_PROXY_OBJECT *)calloc(1, sizeof(struct _dwg_object_PROXY_OBJECT));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_PROXY_OBJECT, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_PROXY_OBJECT) {
  {
    struct _dwg_object_PROXY_OBJECT *arg1 = (struct _dwg_object_PROXY_OBJECT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_PROXY_OBJECT(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PROXY_OBJECT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_PROXY_OBJECT" "', argument " "1"" of type '" "struct _dwg_object_PROXY_OBJECT *""'"); 
    }
    arg1 = (struct _dwg_object_PROXY_OBJECT *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_Color_parent_set) {
  {
    struct _dwg_HATCH_Color *arg1 = (struct _dwg_HATCH_Color *) 0 ;
    struct _dwg_entity_HATCH *arg2 = (struct _dwg_entity_HATCH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_Color_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_Color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_Color_parent_set" "', argument " "1"" of type '" "struct _dwg_HATCH_Color *""'"); 
    }
    arg1 = (struct _dwg_HATCH_Color *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_HATCH, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_HATCH_Color_parent_set" "', argument " "2"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg2 = (struct _dwg_entity_HATCH *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_Color_parent_get) {
  {
    struct _dwg_HATCH_Color *arg1 = (struct _dwg_HATCH_Color *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_entity_HATCH *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_Color_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_Color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_Color_parent_get" "', argument " "1"" of type '" "struct _dwg_HATCH_Color *""'"); 
    }
    arg1 = (struct _dwg_HATCH_Color *)(argp1);
    result = (struct _dwg_entity_HATCH *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_HATCH, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_Color_shift_value_set) {
  {
    struct _dwg_HATCH_Color *arg1 = (struct _dwg_HATCH_Color *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_Color_shift_value_set(self,shift_value);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_Color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_Color_shift_value_set" "', argument " "1"" of type '" "struct _dwg_HATCH_Color *""'"); 
    }
    arg1 = (struct _dwg_HATCH_Color *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_HATCH_Color_shift_value_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->shift_value = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_Color_shift_value_get) {
  {
    struct _dwg_HATCH_Color *arg1 = (struct _dwg_HATCH_Color *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_Color_shift_value_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_Color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_Color_shift_value_get" "', argument " "1"" of type '" "struct _dwg_HATCH_Color *""'"); 
    }
    arg1 = (struct _dwg_HATCH_Color *)(argp1);
    result = (BITCODE_BD) ((arg1)->shift_value);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_Color_color_set) {
  {
    struct _dwg_HATCH_Color *arg1 = (struct _dwg_HATCH_Color *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_Color_color_set(self,color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_Color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_Color_color_set" "', argument " "1"" of type '" "struct _dwg_HATCH_Color *""'"); 
    }
    arg1 = (struct _dwg_HATCH_Color *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_HATCH_Color_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_Color_color_get) {
  {
    struct _dwg_HATCH_Color *arg1 = (struct _dwg_HATCH_Color *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_Color_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_Color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_Color_color_get" "', argument " "1"" of type '" "struct _dwg_HATCH_Color *""'"); 
    }
    arg1 = (struct _dwg_HATCH_Color *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_HATCH_Color) {
  {
    int argvi = 0;
    struct _dwg_HATCH_Color *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_HATCH_Color();");
    }
    result = (struct _dwg_HATCH_Color *)calloc(1, sizeof(struct _dwg_HATCH_Color));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_HATCH_Color, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_HATCH_Color) {
  {
    struct _dwg_HATCH_Color *arg1 = (struct _dwg_HATCH_Color *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_HATCH_Color(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_Color, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_HATCH_Color" "', argument " "1"" of type '" "struct _dwg_HATCH_Color *""'"); 
    }
    arg1 = (struct _dwg_HATCH_Color *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_ControlPoint_parent_set) {
  {
    struct _dwg_HATCH_ControlPoint *arg1 = (struct _dwg_HATCH_ControlPoint *) 0 ;
    struct _dwg_HATCH_PathSeg *arg2 = (struct _dwg_HATCH_PathSeg *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_ControlPoint_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_ControlPoint, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_ControlPoint_parent_set" "', argument " "1"" of type '" "struct _dwg_HATCH_ControlPoint *""'"); 
    }
    arg1 = (struct _dwg_HATCH_ControlPoint *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_HATCH_PathSeg, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_HATCH_ControlPoint_parent_set" "', argument " "2"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg2 = (struct _dwg_HATCH_PathSeg *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_ControlPoint_parent_get) {
  {
    struct _dwg_HATCH_ControlPoint *arg1 = (struct _dwg_HATCH_ControlPoint *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_HATCH_PathSeg *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_ControlPoint_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_ControlPoint, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_ControlPoint_parent_get" "', argument " "1"" of type '" "struct _dwg_HATCH_ControlPoint *""'"); 
    }
    arg1 = (struct _dwg_HATCH_ControlPoint *)(argp1);
    result = (struct _dwg_HATCH_PathSeg *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_HATCH_PathSeg, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_ControlPoint_point_set) {
  {
    struct _dwg_HATCH_ControlPoint *arg1 = (struct _dwg_HATCH_ControlPoint *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_ControlPoint_point_set(self,point);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_ControlPoint, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_ControlPoint_point_set" "', argument " "1"" of type '" "struct _dwg_HATCH_ControlPoint *""'"); 
    }
    arg1 = (struct _dwg_HATCH_ControlPoint *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_HATCH_ControlPoint_point_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->point = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_ControlPoint_point_get) {
  {
    struct _dwg_HATCH_ControlPoint *arg1 = (struct _dwg_HATCH_ControlPoint *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_ControlPoint_point_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_ControlPoint, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_ControlPoint_point_get" "', argument " "1"" of type '" "struct _dwg_HATCH_ControlPoint *""'"); 
    }
    arg1 = (struct _dwg_HATCH_ControlPoint *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->point);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_ControlPoint_weight_set) {
  {
    struct _dwg_HATCH_ControlPoint *arg1 = (struct _dwg_HATCH_ControlPoint *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_ControlPoint_weight_set(self,weight);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_ControlPoint, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_ControlPoint_weight_set" "', argument " "1"" of type '" "struct _dwg_HATCH_ControlPoint *""'"); 
    }
    arg1 = (struct _dwg_HATCH_ControlPoint *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_HATCH_ControlPoint_weight_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->weight = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_ControlPoint_weight_get) {
  {
    struct _dwg_HATCH_ControlPoint *arg1 = (struct _dwg_HATCH_ControlPoint *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_ControlPoint_weight_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_ControlPoint, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_ControlPoint_weight_get" "', argument " "1"" of type '" "struct _dwg_HATCH_ControlPoint *""'"); 
    }
    arg1 = (struct _dwg_HATCH_ControlPoint *)(argp1);
    result = (BITCODE_BD) ((arg1)->weight);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_HATCH_ControlPoint) {
  {
    int argvi = 0;
    struct _dwg_HATCH_ControlPoint *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_HATCH_ControlPoint();");
    }
    result = (struct _dwg_HATCH_ControlPoint *)calloc(1, sizeof(struct _dwg_HATCH_ControlPoint));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_HATCH_ControlPoint, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_HATCH_ControlPoint) {
  {
    struct _dwg_HATCH_ControlPoint *arg1 = (struct _dwg_HATCH_ControlPoint *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_HATCH_ControlPoint(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_ControlPoint, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_HATCH_ControlPoint" "', argument " "1"" of type '" "struct _dwg_HATCH_ControlPoint *""'"); 
    }
    arg1 = (struct _dwg_HATCH_ControlPoint *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_parent_set) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    struct _dwg_HATCH_Path *arg2 = (struct _dwg_HATCH_Path *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_parent_set" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_HATCH_Path, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_HATCH_PathSeg_parent_set" "', argument " "2"" of type '" "struct _dwg_HATCH_Path *""'"); 
    }
    arg2 = (struct _dwg_HATCH_Path *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_parent_get) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_HATCH_Path *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_parent_get" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    result = (struct _dwg_HATCH_Path *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_HATCH_Path, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_curve_type_set) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_curve_type_set(self,curve_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_curve_type_set" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_HATCH_PathSeg_curve_type_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->curve_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_curve_type_get) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_curve_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_curve_type_get" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    result = (BITCODE_RC) ((arg1)->curve_type);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_first_endpoint_set) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_first_endpoint_set(self,first_endpoint);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_first_endpoint_set" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_HATCH_PathSeg_first_endpoint_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->first_endpoint = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_first_endpoint_get) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_first_endpoint_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_first_endpoint_get" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->first_endpoint);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_second_endpoint_set) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_second_endpoint_set(self,second_endpoint);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_second_endpoint_set" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_HATCH_PathSeg_second_endpoint_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->second_endpoint = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_second_endpoint_get) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_second_endpoint_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_second_endpoint_get" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->second_endpoint);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_center_set) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_center_set(self,center);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_center_set" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_HATCH_PathSeg_center_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->center = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_center_get) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_center_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_center_get" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->center);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_radius_set) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_radius_set(self,radius);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_radius_set" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_HATCH_PathSeg_radius_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->radius = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_radius_get) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_radius_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_radius_get" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    result = (BITCODE_BD) ((arg1)->radius);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_start_angle_set) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_start_angle_set(self,start_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_start_angle_set" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_HATCH_PathSeg_start_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->start_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_start_angle_get) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_start_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_start_angle_get" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    result = (BITCODE_BD) ((arg1)->start_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_end_angle_set) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_end_angle_set(self,end_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_end_angle_set" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_HATCH_PathSeg_end_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->end_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_end_angle_get) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_end_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_end_angle_get" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    result = (BITCODE_BD) ((arg1)->end_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_is_ccw_set) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_is_ccw_set(self,is_ccw);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_is_ccw_set" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_HATCH_PathSeg_is_ccw_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_ccw = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_is_ccw_get) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_is_ccw_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_is_ccw_get" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    result = (BITCODE_B) ((arg1)->is_ccw);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_endpoint_set) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_endpoint_set(self,endpoint);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_endpoint_set" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_HATCH_PathSeg_endpoint_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->endpoint = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_endpoint_get) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_endpoint_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_endpoint_get" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->endpoint);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_minor_major_ratio_set) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_minor_major_ratio_set(self,minor_major_ratio);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_minor_major_ratio_set" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_HATCH_PathSeg_minor_major_ratio_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->minor_major_ratio = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_minor_major_ratio_get) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_minor_major_ratio_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_minor_major_ratio_get" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    result = (BITCODE_BD) ((arg1)->minor_major_ratio);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_degree_set) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_degree_set(self,degree);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_degree_set" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_HATCH_PathSeg_degree_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->degree = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_degree_get) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_degree_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_degree_get" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    result = (BITCODE_BL) ((arg1)->degree);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_is_rational_set) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_is_rational_set(self,is_rational);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_is_rational_set" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_HATCH_PathSeg_is_rational_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_rational = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_is_rational_get) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_is_rational_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_is_rational_get" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    result = (BITCODE_B) ((arg1)->is_rational);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_is_periodic_set) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_is_periodic_set(self,is_periodic);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_is_periodic_set" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_HATCH_PathSeg_is_periodic_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_periodic = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_is_periodic_get) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_is_periodic_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_is_periodic_get" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    result = (BITCODE_B) ((arg1)->is_periodic);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_num_knots_set) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_num_knots_set(self,num_knots);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_num_knots_set" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_HATCH_PathSeg_num_knots_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_knots = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_num_knots_get) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_num_knots_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_num_knots_get" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_knots);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_num_control_points_set) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_num_control_points_set(self,num_control_points);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_num_control_points_set" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_HATCH_PathSeg_num_control_points_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_control_points = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_num_control_points_get) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_num_control_points_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_num_control_points_get" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_control_points);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_knots_set) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_knots_set(self,knots);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_knots_set" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_HATCH_PathSeg_knots_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->knots = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_knots_get) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_knots_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_knots_get" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    result = (BITCODE_BD *) ((arg1)->knots);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_control_points_set) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    Dwg_HATCH_ControlPoint *arg2 = (Dwg_HATCH_ControlPoint *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_control_points_set(self,control_points);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_control_points_set" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_HATCH_ControlPoint, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_HATCH_PathSeg_control_points_set" "', argument " "2"" of type '" "Dwg_HATCH_ControlPoint *""'"); 
    }
    arg2 = (Dwg_HATCH_ControlPoint *)(argp2);
    if (arg1) (arg1)->control_points = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_control_points_get) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_HATCH_ControlPoint *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_control_points_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_control_points_get" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    result = (Dwg_HATCH_ControlPoint *) ((arg1)->control_points);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_HATCH_ControlPoint, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_num_fitpts_set) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_num_fitpts_set(self,num_fitpts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_num_fitpts_set" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_HATCH_PathSeg_num_fitpts_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_fitpts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_num_fitpts_get) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_num_fitpts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_num_fitpts_get" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_fitpts);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_fitpts_set) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_fitpts_set(self,fitpts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_fitpts_set" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_HATCH_PathSeg_fitpts_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->fitpts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_fitpts_get) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_fitpts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_fitpts_get" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    result = (BITCODE_2RD *) ((arg1)->fitpts);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_start_tangent_set) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_start_tangent_set(self,start_tangent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_start_tangent_set" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_HATCH_PathSeg_start_tangent_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->start_tangent = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_start_tangent_get) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_start_tangent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_start_tangent_get" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->start_tangent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_end_tangent_set) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_end_tangent_set(self,end_tangent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_end_tangent_set" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_HATCH_PathSeg_end_tangent_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->end_tangent = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PathSeg_end_tangent_get) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_PathSeg_end_tangent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PathSeg_end_tangent_get" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->end_tangent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_HATCH_PathSeg) {
  {
    int argvi = 0;
    struct _dwg_HATCH_PathSeg *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_HATCH_PathSeg();");
    }
    result = (struct _dwg_HATCH_PathSeg *)calloc(1, sizeof(struct _dwg_HATCH_PathSeg));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_HATCH_PathSeg, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_HATCH_PathSeg) {
  {
    struct _dwg_HATCH_PathSeg *arg1 = (struct _dwg_HATCH_PathSeg *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_HATCH_PathSeg(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PathSeg, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_HATCH_PathSeg" "', argument " "1"" of type '" "struct _dwg_HATCH_PathSeg *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PathSeg *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PolylinePath_parent_set) {
  {
    struct _dwg_HATCH_PolylinePath *arg1 = (struct _dwg_HATCH_PolylinePath *) 0 ;
    struct _dwg_HATCH_Path *arg2 = (struct _dwg_HATCH_Path *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_PolylinePath_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PolylinePath, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PolylinePath_parent_set" "', argument " "1"" of type '" "struct _dwg_HATCH_PolylinePath *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PolylinePath *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_HATCH_Path, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_HATCH_PolylinePath_parent_set" "', argument " "2"" of type '" "struct _dwg_HATCH_Path *""'"); 
    }
    arg2 = (struct _dwg_HATCH_Path *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PolylinePath_parent_get) {
  {
    struct _dwg_HATCH_PolylinePath *arg1 = (struct _dwg_HATCH_PolylinePath *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_HATCH_Path *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_PolylinePath_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PolylinePath, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PolylinePath_parent_get" "', argument " "1"" of type '" "struct _dwg_HATCH_PolylinePath *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PolylinePath *)(argp1);
    result = (struct _dwg_HATCH_Path *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_HATCH_Path, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PolylinePath_point_set) {
  {
    struct _dwg_HATCH_PolylinePath *arg1 = (struct _dwg_HATCH_PolylinePath *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_PolylinePath_point_set(self,point);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PolylinePath, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PolylinePath_point_set" "', argument " "1"" of type '" "struct _dwg_HATCH_PolylinePath *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PolylinePath *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_HATCH_PolylinePath_point_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->point = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PolylinePath_point_get) {
  {
    struct _dwg_HATCH_PolylinePath *arg1 = (struct _dwg_HATCH_PolylinePath *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_PolylinePath_point_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PolylinePath, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PolylinePath_point_get" "', argument " "1"" of type '" "struct _dwg_HATCH_PolylinePath *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PolylinePath *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->point);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PolylinePath_bulge_set) {
  {
    struct _dwg_HATCH_PolylinePath *arg1 = (struct _dwg_HATCH_PolylinePath *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_PolylinePath_bulge_set(self,bulge);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PolylinePath, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PolylinePath_bulge_set" "', argument " "1"" of type '" "struct _dwg_HATCH_PolylinePath *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PolylinePath *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_HATCH_PolylinePath_bulge_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->bulge = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_PolylinePath_bulge_get) {
  {
    struct _dwg_HATCH_PolylinePath *arg1 = (struct _dwg_HATCH_PolylinePath *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_PolylinePath_bulge_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PolylinePath, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_PolylinePath_bulge_get" "', argument " "1"" of type '" "struct _dwg_HATCH_PolylinePath *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PolylinePath *)(argp1);
    result = (BITCODE_BD) ((arg1)->bulge);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_HATCH_PolylinePath) {
  {
    int argvi = 0;
    struct _dwg_HATCH_PolylinePath *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_HATCH_PolylinePath();");
    }
    result = (struct _dwg_HATCH_PolylinePath *)calloc(1, sizeof(struct _dwg_HATCH_PolylinePath));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_HATCH_PolylinePath, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_HATCH_PolylinePath) {
  {
    struct _dwg_HATCH_PolylinePath *arg1 = (struct _dwg_HATCH_PolylinePath *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_HATCH_PolylinePath(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_PolylinePath, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_HATCH_PolylinePath" "', argument " "1"" of type '" "struct _dwg_HATCH_PolylinePath *""'"); 
    }
    arg1 = (struct _dwg_HATCH_PolylinePath *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_Path_parent_set) {
  {
    struct _dwg_HATCH_Path *arg1 = (struct _dwg_HATCH_Path *) 0 ;
    struct _dwg_entity_HATCH *arg2 = (struct _dwg_entity_HATCH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_Path_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_Path, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_Path_parent_set" "', argument " "1"" of type '" "struct _dwg_HATCH_Path *""'"); 
    }
    arg1 = (struct _dwg_HATCH_Path *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_HATCH, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_HATCH_Path_parent_set" "', argument " "2"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg2 = (struct _dwg_entity_HATCH *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_Path_parent_get) {
  {
    struct _dwg_HATCH_Path *arg1 = (struct _dwg_HATCH_Path *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_entity_HATCH *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_Path_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_Path, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_Path_parent_get" "', argument " "1"" of type '" "struct _dwg_HATCH_Path *""'"); 
    }
    arg1 = (struct _dwg_HATCH_Path *)(argp1);
    result = (struct _dwg_entity_HATCH *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_HATCH, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_Path_flag_set) {
  {
    struct _dwg_HATCH_Path *arg1 = (struct _dwg_HATCH_Path *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_Path_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_Path, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_Path_flag_set" "', argument " "1"" of type '" "struct _dwg_HATCH_Path *""'"); 
    }
    arg1 = (struct _dwg_HATCH_Path *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_HATCH_Path_flag_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_Path_flag_get) {
  {
    struct _dwg_HATCH_Path *arg1 = (struct _dwg_HATCH_Path *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_Path_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_Path, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_Path_flag_get" "', argument " "1"" of type '" "struct _dwg_HATCH_Path *""'"); 
    }
    arg1 = (struct _dwg_HATCH_Path *)(argp1);
    result = (BITCODE_BL) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_Path_num_segs_or_paths_set) {
  {
    struct _dwg_HATCH_Path *arg1 = (struct _dwg_HATCH_Path *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_Path_num_segs_or_paths_set(self,num_segs_or_paths);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_Path, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_Path_num_segs_or_paths_set" "', argument " "1"" of type '" "struct _dwg_HATCH_Path *""'"); 
    }
    arg1 = (struct _dwg_HATCH_Path *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_HATCH_Path_num_segs_or_paths_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_segs_or_paths = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_Path_num_segs_or_paths_get) {
  {
    struct _dwg_HATCH_Path *arg1 = (struct _dwg_HATCH_Path *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_Path_num_segs_or_paths_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_Path, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_Path_num_segs_or_paths_get" "', argument " "1"" of type '" "struct _dwg_HATCH_Path *""'"); 
    }
    arg1 = (struct _dwg_HATCH_Path *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_segs_or_paths);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_Path_segs_set) {
  {
    struct _dwg_HATCH_Path *arg1 = (struct _dwg_HATCH_Path *) 0 ;
    Dwg_HATCH_PathSeg *arg2 = (Dwg_HATCH_PathSeg *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_Path_segs_set(self,segs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_Path, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_Path_segs_set" "', argument " "1"" of type '" "struct _dwg_HATCH_Path *""'"); 
    }
    arg1 = (struct _dwg_HATCH_Path *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_HATCH_PathSeg, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_HATCH_Path_segs_set" "', argument " "2"" of type '" "Dwg_HATCH_PathSeg *""'"); 
    }
    arg2 = (Dwg_HATCH_PathSeg *)(argp2);
    if (arg1) (arg1)->segs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_Path_segs_get) {
  {
    struct _dwg_HATCH_Path *arg1 = (struct _dwg_HATCH_Path *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_HATCH_PathSeg *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_Path_segs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_Path, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_Path_segs_get" "', argument " "1"" of type '" "struct _dwg_HATCH_Path *""'"); 
    }
    arg1 = (struct _dwg_HATCH_Path *)(argp1);
    result = (Dwg_HATCH_PathSeg *) ((arg1)->segs);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_HATCH_PathSeg, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_Path_bulges_present_set) {
  {
    struct _dwg_HATCH_Path *arg1 = (struct _dwg_HATCH_Path *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_Path_bulges_present_set(self,bulges_present);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_Path, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_Path_bulges_present_set" "', argument " "1"" of type '" "struct _dwg_HATCH_Path *""'"); 
    }
    arg1 = (struct _dwg_HATCH_Path *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_HATCH_Path_bulges_present_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->bulges_present = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_Path_bulges_present_get) {
  {
    struct _dwg_HATCH_Path *arg1 = (struct _dwg_HATCH_Path *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_Path_bulges_present_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_Path, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_Path_bulges_present_get" "', argument " "1"" of type '" "struct _dwg_HATCH_Path *""'"); 
    }
    arg1 = (struct _dwg_HATCH_Path *)(argp1);
    result = (BITCODE_B) ((arg1)->bulges_present);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_Path_closed_set) {
  {
    struct _dwg_HATCH_Path *arg1 = (struct _dwg_HATCH_Path *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_Path_closed_set(self,closed);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_Path, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_Path_closed_set" "', argument " "1"" of type '" "struct _dwg_HATCH_Path *""'"); 
    }
    arg1 = (struct _dwg_HATCH_Path *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_HATCH_Path_closed_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->closed = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_Path_closed_get) {
  {
    struct _dwg_HATCH_Path *arg1 = (struct _dwg_HATCH_Path *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_Path_closed_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_Path, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_Path_closed_get" "', argument " "1"" of type '" "struct _dwg_HATCH_Path *""'"); 
    }
    arg1 = (struct _dwg_HATCH_Path *)(argp1);
    result = (BITCODE_B) ((arg1)->closed);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_Path_polyline_paths_set) {
  {
    struct _dwg_HATCH_Path *arg1 = (struct _dwg_HATCH_Path *) 0 ;
    Dwg_HATCH_PolylinePath *arg2 = (Dwg_HATCH_PolylinePath *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_Path_polyline_paths_set(self,polyline_paths);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_Path, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_Path_polyline_paths_set" "', argument " "1"" of type '" "struct _dwg_HATCH_Path *""'"); 
    }
    arg1 = (struct _dwg_HATCH_Path *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_HATCH_PolylinePath, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_HATCH_Path_polyline_paths_set" "', argument " "2"" of type '" "Dwg_HATCH_PolylinePath *""'"); 
    }
    arg2 = (Dwg_HATCH_PolylinePath *)(argp2);
    if (arg1) (arg1)->polyline_paths = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_Path_polyline_paths_get) {
  {
    struct _dwg_HATCH_Path *arg1 = (struct _dwg_HATCH_Path *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_HATCH_PolylinePath *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_Path_polyline_paths_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_Path, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_Path_polyline_paths_get" "', argument " "1"" of type '" "struct _dwg_HATCH_Path *""'"); 
    }
    arg1 = (struct _dwg_HATCH_Path *)(argp1);
    result = (Dwg_HATCH_PolylinePath *) ((arg1)->polyline_paths);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_HATCH_PolylinePath, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_Path_num_boundary_handles_set) {
  {
    struct _dwg_HATCH_Path *arg1 = (struct _dwg_HATCH_Path *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_Path_num_boundary_handles_set(self,num_boundary_handles);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_Path, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_Path_num_boundary_handles_set" "', argument " "1"" of type '" "struct _dwg_HATCH_Path *""'"); 
    }
    arg1 = (struct _dwg_HATCH_Path *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_HATCH_Path_num_boundary_handles_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_boundary_handles = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_Path_num_boundary_handles_get) {
  {
    struct _dwg_HATCH_Path *arg1 = (struct _dwg_HATCH_Path *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_Path_num_boundary_handles_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_Path, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_Path_num_boundary_handles_get" "', argument " "1"" of type '" "struct _dwg_HATCH_Path *""'"); 
    }
    arg1 = (struct _dwg_HATCH_Path *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_boundary_handles);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_Path_boundary_handles_set) {
  {
    struct _dwg_HATCH_Path *arg1 = (struct _dwg_HATCH_Path *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_Path_boundary_handles_set(self,boundary_handles);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_Path, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_Path_boundary_handles_set" "', argument " "1"" of type '" "struct _dwg_HATCH_Path *""'"); 
    }
    arg1 = (struct _dwg_HATCH_Path *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_HATCH_Path_boundary_handles_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->boundary_handles = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_Path_boundary_handles_get) {
  {
    struct _dwg_HATCH_Path *arg1 = (struct _dwg_HATCH_Path *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_Path_boundary_handles_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_Path, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_Path_boundary_handles_get" "', argument " "1"" of type '" "struct _dwg_HATCH_Path *""'"); 
    }
    arg1 = (struct _dwg_HATCH_Path *)(argp1);
    result = (BITCODE_H *) ((arg1)->boundary_handles);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_HATCH_Path) {
  {
    int argvi = 0;
    struct _dwg_HATCH_Path *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_HATCH_Path();");
    }
    result = (struct _dwg_HATCH_Path *)calloc(1, sizeof(struct _dwg_HATCH_Path));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_HATCH_Path, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_HATCH_Path) {
  {
    struct _dwg_HATCH_Path *arg1 = (struct _dwg_HATCH_Path *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_HATCH_Path(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_Path, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_HATCH_Path" "', argument " "1"" of type '" "struct _dwg_HATCH_Path *""'"); 
    }
    arg1 = (struct _dwg_HATCH_Path *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_DefLine_parent_set) {
  {
    struct _dwg_HATCH_DefLine *arg1 = (struct _dwg_HATCH_DefLine *) 0 ;
    struct _dwg_entity_HATCH *arg2 = (struct _dwg_entity_HATCH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_DefLine_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_DefLine, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_DefLine_parent_set" "', argument " "1"" of type '" "struct _dwg_HATCH_DefLine *""'"); 
    }
    arg1 = (struct _dwg_HATCH_DefLine *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_HATCH, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_HATCH_DefLine_parent_set" "', argument " "2"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg2 = (struct _dwg_entity_HATCH *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_DefLine_parent_get) {
  {
    struct _dwg_HATCH_DefLine *arg1 = (struct _dwg_HATCH_DefLine *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_entity_HATCH *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_DefLine_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_DefLine, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_DefLine_parent_get" "', argument " "1"" of type '" "struct _dwg_HATCH_DefLine *""'"); 
    }
    arg1 = (struct _dwg_HATCH_DefLine *)(argp1);
    result = (struct _dwg_entity_HATCH *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_HATCH, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_DefLine_angle_set) {
  {
    struct _dwg_HATCH_DefLine *arg1 = (struct _dwg_HATCH_DefLine *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_DefLine_angle_set(self,angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_DefLine, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_DefLine_angle_set" "', argument " "1"" of type '" "struct _dwg_HATCH_DefLine *""'"); 
    }
    arg1 = (struct _dwg_HATCH_DefLine *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_HATCH_DefLine_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_DefLine_angle_get) {
  {
    struct _dwg_HATCH_DefLine *arg1 = (struct _dwg_HATCH_DefLine *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_DefLine_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_DefLine, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_DefLine_angle_get" "', argument " "1"" of type '" "struct _dwg_HATCH_DefLine *""'"); 
    }
    arg1 = (struct _dwg_HATCH_DefLine *)(argp1);
    result = (BITCODE_BD) ((arg1)->angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_DefLine_pt0_set) {
  {
    struct _dwg_HATCH_DefLine *arg1 = (struct _dwg_HATCH_DefLine *) 0 ;
    BITCODE_2BD *arg2 = (BITCODE_2BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_DefLine_pt0_set(self,pt0);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_DefLine, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_DefLine_pt0_set" "', argument " "1"" of type '" "struct _dwg_HATCH_DefLine *""'"); 
    }
    arg1 = (struct _dwg_HATCH_DefLine *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_HATCH_DefLine_pt0_set" "', argument " "2"" of type '" "BITCODE_2BD *""'"); 
    }
    arg2 = (BITCODE_2BD *)(argp2);
    if (arg1) (arg1)->pt0 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_DefLine_pt0_get) {
  {
    struct _dwg_HATCH_DefLine *arg1 = (struct _dwg_HATCH_DefLine *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_DefLine_pt0_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_DefLine, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_DefLine_pt0_get" "', argument " "1"" of type '" "struct _dwg_HATCH_DefLine *""'"); 
    }
    arg1 = (struct _dwg_HATCH_DefLine *)(argp1);
    result = (BITCODE_2BD *)& ((arg1)->pt0);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_DefLine_offset_set) {
  {
    struct _dwg_HATCH_DefLine *arg1 = (struct _dwg_HATCH_DefLine *) 0 ;
    BITCODE_2BD *arg2 = (BITCODE_2BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_DefLine_offset_set(self,offset);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_DefLine, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_DefLine_offset_set" "', argument " "1"" of type '" "struct _dwg_HATCH_DefLine *""'"); 
    }
    arg1 = (struct _dwg_HATCH_DefLine *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_HATCH_DefLine_offset_set" "', argument " "2"" of type '" "BITCODE_2BD *""'"); 
    }
    arg2 = (BITCODE_2BD *)(argp2);
    if (arg1) (arg1)->offset = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_DefLine_offset_get) {
  {
    struct _dwg_HATCH_DefLine *arg1 = (struct _dwg_HATCH_DefLine *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_DefLine_offset_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_DefLine, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_DefLine_offset_get" "', argument " "1"" of type '" "struct _dwg_HATCH_DefLine *""'"); 
    }
    arg1 = (struct _dwg_HATCH_DefLine *)(argp1);
    result = (BITCODE_2BD *)& ((arg1)->offset);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_DefLine_num_dashes_set) {
  {
    struct _dwg_HATCH_DefLine *arg1 = (struct _dwg_HATCH_DefLine *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_DefLine_num_dashes_set(self,num_dashes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_DefLine, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_DefLine_num_dashes_set" "', argument " "1"" of type '" "struct _dwg_HATCH_DefLine *""'"); 
    }
    arg1 = (struct _dwg_HATCH_DefLine *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_HATCH_DefLine_num_dashes_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_dashes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_DefLine_num_dashes_get) {
  {
    struct _dwg_HATCH_DefLine *arg1 = (struct _dwg_HATCH_DefLine *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_DefLine_num_dashes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_DefLine, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_DefLine_num_dashes_get" "', argument " "1"" of type '" "struct _dwg_HATCH_DefLine *""'"); 
    }
    arg1 = (struct _dwg_HATCH_DefLine *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_dashes);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_DefLine_dashes_set) {
  {
    struct _dwg_HATCH_DefLine *arg1 = (struct _dwg_HATCH_DefLine *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_HATCH_DefLine_dashes_set(self,dashes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_DefLine, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_DefLine_dashes_set" "', argument " "1"" of type '" "struct _dwg_HATCH_DefLine *""'"); 
    }
    arg1 = (struct _dwg_HATCH_DefLine *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_HATCH_DefLine_dashes_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->dashes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_HATCH_DefLine_dashes_get) {
  {
    struct _dwg_HATCH_DefLine *arg1 = (struct _dwg_HATCH_DefLine *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_HATCH_DefLine_dashes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_DefLine, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_HATCH_DefLine_dashes_get" "', argument " "1"" of type '" "struct _dwg_HATCH_DefLine *""'"); 
    }
    arg1 = (struct _dwg_HATCH_DefLine *)(argp1);
    result = (BITCODE_BD *) ((arg1)->dashes);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_HATCH_DefLine) {
  {
    int argvi = 0;
    struct _dwg_HATCH_DefLine *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_HATCH_DefLine();");
    }
    result = (struct _dwg_HATCH_DefLine *)calloc(1, sizeof(struct _dwg_HATCH_DefLine));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_HATCH_DefLine, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_HATCH_DefLine) {
  {
    struct _dwg_HATCH_DefLine *arg1 = (struct _dwg_HATCH_DefLine *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_HATCH_DefLine(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_HATCH_DefLine, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_HATCH_DefLine" "', argument " "1"" of type '" "struct _dwg_HATCH_DefLine *""'"); 
    }
    arg1 = (struct _dwg_HATCH_DefLine *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_parent_set) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_HATCH_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_parent_get) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_is_gradient_fill_set) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_is_gradient_fill_set(self,is_gradient_fill);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_is_gradient_fill_set" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HATCH_is_gradient_fill_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->is_gradient_fill = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_is_gradient_fill_get) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_is_gradient_fill_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_is_gradient_fill_get" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    result = (BITCODE_BL) ((arg1)->is_gradient_fill);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_reserved_set) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_reserved_set(self,reserved);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_reserved_set" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HATCH_reserved_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->reserved = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_reserved_get) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_reserved_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_reserved_get" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    result = (BITCODE_BL) ((arg1)->reserved);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_gradient_angle_set) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_gradient_angle_set(self,gradient_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_gradient_angle_set" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HATCH_gradient_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->gradient_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_gradient_angle_get) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_gradient_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_gradient_angle_get" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    result = (BITCODE_BD) ((arg1)->gradient_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_gradient_shift_set) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_gradient_shift_set(self,gradient_shift);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_gradient_shift_set" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HATCH_gradient_shift_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->gradient_shift = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_gradient_shift_get) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_gradient_shift_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_gradient_shift_get" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    result = (BITCODE_BD) ((arg1)->gradient_shift);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_single_color_gradient_set) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_single_color_gradient_set(self,single_color_gradient);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_single_color_gradient_set" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HATCH_single_color_gradient_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->single_color_gradient = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_single_color_gradient_get) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_single_color_gradient_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_single_color_gradient_get" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    result = (BITCODE_BL) ((arg1)->single_color_gradient);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_gradient_tint_set) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_gradient_tint_set(self,gradient_tint);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_gradient_tint_set" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HATCH_gradient_tint_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->gradient_tint = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_gradient_tint_get) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_gradient_tint_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_gradient_tint_get" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    result = (BITCODE_BD) ((arg1)->gradient_tint);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_num_colors_set) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_num_colors_set(self,num_colors);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_num_colors_set" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HATCH_num_colors_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_colors = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_num_colors_get) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_num_colors_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_num_colors_get" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_colors);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_colors_set) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    Dwg_HATCH_Color *arg2 = (Dwg_HATCH_Color *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_colors_set(self,colors);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_colors_set" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_HATCH_Color, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_HATCH_colors_set" "', argument " "2"" of type '" "Dwg_HATCH_Color *""'"); 
    }
    arg2 = (Dwg_HATCH_Color *)(argp2);
    if (arg1) (arg1)->colors = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_colors_get) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_HATCH_Color *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_colors_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_colors_get" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    result = (Dwg_HATCH_Color *) ((arg1)->colors);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_HATCH_Color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_gradient_name_set) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_gradient_name_set(self,gradient_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_gradient_name_set" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_HATCH_gradient_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->gradient_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->gradient_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->gradient_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_gradient_name_get) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_gradient_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_gradient_name_get" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    result = (BITCODE_TV) ((arg1)->gradient_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_elevation_set) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_elevation_set(self,elevation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_elevation_set" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HATCH_elevation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->elevation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_elevation_get) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_elevation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_elevation_get" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    result = (BITCODE_BD) ((arg1)->elevation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_extrusion_set) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_extrusion_set" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_HATCH_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_extrusion_get) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_extrusion_get" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_name_set) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_name_set" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_HATCH_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_name_get) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_name_get" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_is_solid_fill_set) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_is_solid_fill_set(self,is_solid_fill);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_is_solid_fill_set" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HATCH_is_solid_fill_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_solid_fill = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_is_solid_fill_get) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_is_solid_fill_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_is_solid_fill_get" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    result = (BITCODE_B) ((arg1)->is_solid_fill);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_is_associative_set) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_is_associative_set(self,is_associative);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_is_associative_set" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HATCH_is_associative_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_associative = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_is_associative_get) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_is_associative_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_is_associative_get" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    result = (BITCODE_B) ((arg1)->is_associative);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_num_paths_set) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_num_paths_set(self,num_paths);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_num_paths_set" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HATCH_num_paths_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_paths = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_num_paths_get) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_num_paths_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_num_paths_get" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_paths);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_paths_set) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    Dwg_HATCH_Path *arg2 = (Dwg_HATCH_Path *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_paths_set(self,paths);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_paths_set" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_HATCH_Path, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_HATCH_paths_set" "', argument " "2"" of type '" "Dwg_HATCH_Path *""'"); 
    }
    arg2 = (Dwg_HATCH_Path *)(argp2);
    if (arg1) (arg1)->paths = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_paths_get) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_HATCH_Path *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_paths_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_paths_get" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    result = (Dwg_HATCH_Path *) ((arg1)->paths);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_HATCH_Path, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_style_set) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_style_set(self,style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_style_set" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HATCH_style_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_style_get) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_style_get" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    result = (BITCODE_BS) ((arg1)->style);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_pattern_type_set) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_pattern_type_set(self,pattern_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_pattern_type_set" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HATCH_pattern_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->pattern_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_pattern_type_get) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_pattern_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_pattern_type_get" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    result = (BITCODE_BS) ((arg1)->pattern_type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_angle_set) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_angle_set(self,angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_angle_set" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HATCH_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_angle_get) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_angle_get" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    result = (BITCODE_BD) ((arg1)->angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_scale_spacing_set) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_scale_spacing_set(self,scale_spacing);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_scale_spacing_set" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HATCH_scale_spacing_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->scale_spacing = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_scale_spacing_get) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_scale_spacing_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_scale_spacing_get" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    result = (BITCODE_BD) ((arg1)->scale_spacing);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_double_flag_set) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_double_flag_set(self,double_flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_double_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HATCH_double_flag_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->double_flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_double_flag_get) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_double_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_double_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    result = (BITCODE_B) ((arg1)->double_flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_num_deflines_set) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_num_deflines_set(self,num_deflines);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_num_deflines_set" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HATCH_num_deflines_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_deflines = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_num_deflines_get) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_num_deflines_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_num_deflines_get" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_deflines);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_deflines_set) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    Dwg_HATCH_DefLine *arg2 = (Dwg_HATCH_DefLine *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_deflines_set(self,deflines);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_deflines_set" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_HATCH_DefLine, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_HATCH_deflines_set" "', argument " "2"" of type '" "Dwg_HATCH_DefLine *""'"); 
    }
    arg2 = (Dwg_HATCH_DefLine *)(argp2);
    if (arg1) (arg1)->deflines = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_deflines_get) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_HATCH_DefLine *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_deflines_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_deflines_get" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    result = (Dwg_HATCH_DefLine *) ((arg1)->deflines);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_HATCH_DefLine, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_has_derived_set) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_has_derived_set(self,has_derived);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_has_derived_set" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HATCH_has_derived_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_derived = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_has_derived_get) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_has_derived_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_has_derived_get" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    result = (BITCODE_B) ((arg1)->has_derived);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_pixel_size_set) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_pixel_size_set(self,pixel_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_pixel_size_set" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HATCH_pixel_size_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->pixel_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_pixel_size_get) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_pixel_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_pixel_size_get" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    result = (BITCODE_BD) ((arg1)->pixel_size);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_num_seeds_set) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_num_seeds_set(self,num_seeds);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_num_seeds_set" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HATCH_num_seeds_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_seeds = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_num_seeds_get) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_num_seeds_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_num_seeds_get" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_seeds);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_seeds_set) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_seeds_set(self,seeds);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_seeds_set" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_HATCH_seeds_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->seeds = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HATCH_seeds_get) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HATCH_seeds_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HATCH_seeds_get" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    result = (BITCODE_2RD *) ((arg1)->seeds);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_HATCH) {
  {
    int argvi = 0;
    struct _dwg_entity_HATCH *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_HATCH();");
    }
    result = (struct _dwg_entity_HATCH *)calloc(1, sizeof(struct _dwg_entity_HATCH));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_HATCH, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_HATCH) {
  {
    struct _dwg_entity_HATCH *arg1 = (struct _dwg_entity_HATCH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_HATCH(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HATCH, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_HATCH" "', argument " "1"" of type '" "struct _dwg_entity_HATCH *""'"); 
    }
    arg1 = (struct _dwg_entity_HATCH *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_parent_set) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MPOLYGON_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_parent_get) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_is_gradient_fill_set) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_is_gradient_fill_set(self,is_gradient_fill);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_is_gradient_fill_set" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MPOLYGON_is_gradient_fill_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->is_gradient_fill = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_is_gradient_fill_get) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_is_gradient_fill_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_is_gradient_fill_get" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    result = (BITCODE_BL) ((arg1)->is_gradient_fill);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_reserved_set) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_reserved_set(self,reserved);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_reserved_set" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MPOLYGON_reserved_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->reserved = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_reserved_get) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_reserved_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_reserved_get" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    result = (BITCODE_BL) ((arg1)->reserved);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_gradient_angle_set) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_gradient_angle_set(self,gradient_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_gradient_angle_set" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MPOLYGON_gradient_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->gradient_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_gradient_angle_get) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_gradient_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_gradient_angle_get" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    result = (BITCODE_BD) ((arg1)->gradient_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_gradient_shift_set) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_gradient_shift_set(self,gradient_shift);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_gradient_shift_set" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MPOLYGON_gradient_shift_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->gradient_shift = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_gradient_shift_get) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_gradient_shift_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_gradient_shift_get" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    result = (BITCODE_BD) ((arg1)->gradient_shift);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_single_color_gradient_set) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_single_color_gradient_set(self,single_color_gradient);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_single_color_gradient_set" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MPOLYGON_single_color_gradient_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->single_color_gradient = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_single_color_gradient_get) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_single_color_gradient_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_single_color_gradient_get" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    result = (BITCODE_BL) ((arg1)->single_color_gradient);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_gradient_tint_set) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_gradient_tint_set(self,gradient_tint);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_gradient_tint_set" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MPOLYGON_gradient_tint_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->gradient_tint = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_gradient_tint_get) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_gradient_tint_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_gradient_tint_get" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    result = (BITCODE_BD) ((arg1)->gradient_tint);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_num_colors_set) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_num_colors_set(self,num_colors);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_num_colors_set" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MPOLYGON_num_colors_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_colors = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_num_colors_get) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_num_colors_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_num_colors_get" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_colors);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_colors_set) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    Dwg_HATCH_Color *arg2 = (Dwg_HATCH_Color *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_colors_set(self,colors);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_colors_set" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_HATCH_Color, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MPOLYGON_colors_set" "', argument " "2"" of type '" "Dwg_HATCH_Color *""'"); 
    }
    arg2 = (Dwg_HATCH_Color *)(argp2);
    if (arg1) (arg1)->colors = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_colors_get) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_HATCH_Color *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_colors_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_colors_get" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    result = (Dwg_HATCH_Color *) ((arg1)->colors);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_HATCH_Color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_gradient_name_set) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_gradient_name_set(self,gradient_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_gradient_name_set" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MPOLYGON_gradient_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->gradient_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->gradient_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->gradient_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_gradient_name_get) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_gradient_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_gradient_name_get" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    result = (BITCODE_TV) ((arg1)->gradient_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_elevation_set) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_elevation_set(self,elevation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_elevation_set" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MPOLYGON_elevation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->elevation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_elevation_get) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_elevation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_elevation_get" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    result = (BITCODE_BD) ((arg1)->elevation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_extrusion_set) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_extrusion_set" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MPOLYGON_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_extrusion_get) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_extrusion_get" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_name_set) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_name_set" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MPOLYGON_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_name_get) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_name_get" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_is_solid_fill_set) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_is_solid_fill_set(self,is_solid_fill);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_is_solid_fill_set" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MPOLYGON_is_solid_fill_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_solid_fill = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_is_solid_fill_get) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_is_solid_fill_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_is_solid_fill_get" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    result = (BITCODE_B) ((arg1)->is_solid_fill);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_is_associative_set) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_is_associative_set(self,is_associative);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_is_associative_set" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MPOLYGON_is_associative_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_associative = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_is_associative_get) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_is_associative_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_is_associative_get" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    result = (BITCODE_B) ((arg1)->is_associative);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_num_paths_set) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_num_paths_set(self,num_paths);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_num_paths_set" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MPOLYGON_num_paths_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_paths = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_num_paths_get) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_num_paths_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_num_paths_get" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_paths);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_paths_set) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    Dwg_HATCH_Path *arg2 = (Dwg_HATCH_Path *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_paths_set(self,paths);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_paths_set" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_HATCH_Path, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MPOLYGON_paths_set" "', argument " "2"" of type '" "Dwg_HATCH_Path *""'"); 
    }
    arg2 = (Dwg_HATCH_Path *)(argp2);
    if (arg1) (arg1)->paths = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_paths_get) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_HATCH_Path *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_paths_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_paths_get" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    result = (Dwg_HATCH_Path *) ((arg1)->paths);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_HATCH_Path, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_style_set) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_style_set(self,style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_style_set" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MPOLYGON_style_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_style_get) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_style_get" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    result = (BITCODE_BS) ((arg1)->style);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_pattern_type_set) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_pattern_type_set(self,pattern_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_pattern_type_set" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MPOLYGON_pattern_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->pattern_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_pattern_type_get) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_pattern_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_pattern_type_get" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    result = (BITCODE_BS) ((arg1)->pattern_type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_angle_set) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_angle_set(self,angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_angle_set" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MPOLYGON_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_angle_get) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_angle_get" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    result = (BITCODE_BD) ((arg1)->angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_scale_spacing_set) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_scale_spacing_set(self,scale_spacing);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_scale_spacing_set" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MPOLYGON_scale_spacing_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->scale_spacing = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_scale_spacing_get) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_scale_spacing_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_scale_spacing_get" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    result = (BITCODE_BD) ((arg1)->scale_spacing);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_double_flag_set) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_double_flag_set(self,double_flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_double_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MPOLYGON_double_flag_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->double_flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_double_flag_get) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_double_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_double_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    result = (BITCODE_B) ((arg1)->double_flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_num_deflines_set) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_num_deflines_set(self,num_deflines);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_num_deflines_set" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MPOLYGON_num_deflines_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_deflines = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_num_deflines_get) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_num_deflines_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_num_deflines_get" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_deflines);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_deflines_set) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    Dwg_HATCH_DefLine *arg2 = (Dwg_HATCH_DefLine *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_deflines_set(self,deflines);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_deflines_set" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_HATCH_DefLine, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MPOLYGON_deflines_set" "', argument " "2"" of type '" "Dwg_HATCH_DefLine *""'"); 
    }
    arg2 = (Dwg_HATCH_DefLine *)(argp2);
    if (arg1) (arg1)->deflines = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_deflines_get) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_HATCH_DefLine *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_deflines_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_deflines_get" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    result = (Dwg_HATCH_DefLine *) ((arg1)->deflines);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_HATCH_DefLine, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_hatch_color_set) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_hatch_color_set(self,hatch_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_hatch_color_set" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MPOLYGON_hatch_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->hatch_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_hatch_color_get) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_hatch_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_hatch_color_get" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->hatch_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_x_dir_set) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_x_dir_set(self,x_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_x_dir_set" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MPOLYGON_x_dir_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->x_dir = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_x_dir_get) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_x_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_x_dir_get" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->x_dir);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_num_boundary_handles_set) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_num_boundary_handles_set(self,num_boundary_handles);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_num_boundary_handles_set" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MPOLYGON_num_boundary_handles_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_boundary_handles = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MPOLYGON_num_boundary_handles_get) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MPOLYGON_num_boundary_handles_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MPOLYGON_num_boundary_handles_get" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_boundary_handles);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_MPOLYGON) {
  {
    int argvi = 0;
    struct _dwg_entity_MPOLYGON *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_MPOLYGON();");
    }
    result = (struct _dwg_entity_MPOLYGON *)calloc(1, sizeof(struct _dwg_entity_MPOLYGON));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_MPOLYGON, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_MPOLYGON) {
  {
    struct _dwg_entity_MPOLYGON *arg1 = (struct _dwg_entity_MPOLYGON *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_MPOLYGON(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MPOLYGON, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_MPOLYGON" "', argument " "1"" of type '" "struct _dwg_entity_MPOLYGON *""'"); 
    }
    arg1 = (struct _dwg_entity_MPOLYGON *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_XRECORD_parent_set) {
  {
    struct _dwg_object_XRECORD *arg1 = (struct _dwg_object_XRECORD *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_XRECORD_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_XRECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_XRECORD_parent_set" "', argument " "1"" of type '" "struct _dwg_object_XRECORD *""'"); 
    }
    arg1 = (struct _dwg_object_XRECORD *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_XRECORD_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_XRECORD_parent_get) {
  {
    struct _dwg_object_XRECORD *arg1 = (struct _dwg_object_XRECORD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_XRECORD_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_XRECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_XRECORD_parent_get" "', argument " "1"" of type '" "struct _dwg_object_XRECORD *""'"); 
    }
    arg1 = (struct _dwg_object_XRECORD *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_XRECORD_cloning_set) {
  {
    struct _dwg_object_XRECORD *arg1 = (struct _dwg_object_XRECORD *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_XRECORD_cloning_set(self,cloning);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_XRECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_XRECORD_cloning_set" "', argument " "1"" of type '" "struct _dwg_object_XRECORD *""'"); 
    }
    arg1 = (struct _dwg_object_XRECORD *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_XRECORD_cloning_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->cloning = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_XRECORD_cloning_get) {
  {
    struct _dwg_object_XRECORD *arg1 = (struct _dwg_object_XRECORD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_XRECORD_cloning_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_XRECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_XRECORD_cloning_get" "', argument " "1"" of type '" "struct _dwg_object_XRECORD *""'"); 
    }
    arg1 = (struct _dwg_object_XRECORD *)(argp1);
    result = (BITCODE_BS) ((arg1)->cloning);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_XRECORD_xdata_size_set) {
  {
    struct _dwg_object_XRECORD *arg1 = (struct _dwg_object_XRECORD *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_XRECORD_xdata_size_set(self,xdata_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_XRECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_XRECORD_xdata_size_set" "', argument " "1"" of type '" "struct _dwg_object_XRECORD *""'"); 
    }
    arg1 = (struct _dwg_object_XRECORD *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_XRECORD_xdata_size_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->xdata_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_XRECORD_xdata_size_get) {
  {
    struct _dwg_object_XRECORD *arg1 = (struct _dwg_object_XRECORD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_XRECORD_xdata_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_XRECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_XRECORD_xdata_size_get" "', argument " "1"" of type '" "struct _dwg_object_XRECORD *""'"); 
    }
    arg1 = (struct _dwg_object_XRECORD *)(argp1);
    result = (BITCODE_BL) ((arg1)->xdata_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_XRECORD_num_xdata_set) {
  {
    struct _dwg_object_XRECORD *arg1 = (struct _dwg_object_XRECORD *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_XRECORD_num_xdata_set(self,num_xdata);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_XRECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_XRECORD_num_xdata_set" "', argument " "1"" of type '" "struct _dwg_object_XRECORD *""'"); 
    }
    arg1 = (struct _dwg_object_XRECORD *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_XRECORD_num_xdata_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_xdata = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_XRECORD_num_xdata_get) {
  {
    struct _dwg_object_XRECORD *arg1 = (struct _dwg_object_XRECORD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_XRECORD_num_xdata_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_XRECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_XRECORD_num_xdata_get" "', argument " "1"" of type '" "struct _dwg_object_XRECORD *""'"); 
    }
    arg1 = (struct _dwg_object_XRECORD *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_xdata);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_XRECORD_xdata_set) {
  {
    struct _dwg_object_XRECORD *arg1 = (struct _dwg_object_XRECORD *) 0 ;
    Dwg_Resbuf *arg2 = (Dwg_Resbuf *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_XRECORD_xdata_set(self,xdata);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_XRECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_XRECORD_xdata_set" "', argument " "1"" of type '" "struct _dwg_object_XRECORD *""'"); 
    }
    arg1 = (struct _dwg_object_XRECORD *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_resbuf, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_XRECORD_xdata_set" "', argument " "2"" of type '" "Dwg_Resbuf *""'"); 
    }
    arg2 = (Dwg_Resbuf *)(argp2);
    if (arg1) (arg1)->xdata = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_XRECORD_xdata_get) {
  {
    struct _dwg_object_XRECORD *arg1 = (struct _dwg_object_XRECORD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Resbuf *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_XRECORD_xdata_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_XRECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_XRECORD_xdata_get" "', argument " "1"" of type '" "struct _dwg_object_XRECORD *""'"); 
    }
    arg1 = (struct _dwg_object_XRECORD *)(argp1);
    result = (Dwg_Resbuf *) ((arg1)->xdata);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_resbuf, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_XRECORD_num_objid_handles_set) {
  {
    struct _dwg_object_XRECORD *arg1 = (struct _dwg_object_XRECORD *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_XRECORD_num_objid_handles_set(self,num_objid_handles);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_XRECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_XRECORD_num_objid_handles_set" "', argument " "1"" of type '" "struct _dwg_object_XRECORD *""'"); 
    }
    arg1 = (struct _dwg_object_XRECORD *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_XRECORD_num_objid_handles_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_objid_handles = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_XRECORD_num_objid_handles_get) {
  {
    struct _dwg_object_XRECORD *arg1 = (struct _dwg_object_XRECORD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_XRECORD_num_objid_handles_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_XRECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_XRECORD_num_objid_handles_get" "', argument " "1"" of type '" "struct _dwg_object_XRECORD *""'"); 
    }
    arg1 = (struct _dwg_object_XRECORD *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_objid_handles);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_XRECORD_objid_handles_set) {
  {
    struct _dwg_object_XRECORD *arg1 = (struct _dwg_object_XRECORD *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_XRECORD_objid_handles_set(self,objid_handles);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_XRECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_XRECORD_objid_handles_set" "', argument " "1"" of type '" "struct _dwg_object_XRECORD *""'"); 
    }
    arg1 = (struct _dwg_object_XRECORD *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_XRECORD_objid_handles_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->objid_handles = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_XRECORD_objid_handles_get) {
  {
    struct _dwg_object_XRECORD *arg1 = (struct _dwg_object_XRECORD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_XRECORD_objid_handles_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_XRECORD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_XRECORD_objid_handles_get" "', argument " "1"" of type '" "struct _dwg_object_XRECORD *""'"); 
    }
    arg1 = (struct _dwg_object_XRECORD *)(argp1);
    result = (BITCODE_H *) ((arg1)->objid_handles);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_XRECORD) {
  {
    int argvi = 0;
    struct _dwg_object_XRECORD *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_XRECORD();");
    }
    result = (struct _dwg_object_XRECORD *)calloc(1, sizeof(struct _dwg_object_XRECORD));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_XRECORD, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_XRECORD) {
  {
    struct _dwg_object_XRECORD *arg1 = (struct _dwg_object_XRECORD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_XRECORD(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_XRECORD, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_XRECORD" "', argument " "1"" of type '" "struct _dwg_object_XRECORD *""'"); 
    }
    arg1 = (struct _dwg_object_XRECORD *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLACEHOLDER_parent_set) {
  {
    struct _dwg_object_PLACEHOLDER *arg1 = (struct _dwg_object_PLACEHOLDER *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PLACEHOLDER_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLACEHOLDER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLACEHOLDER_parent_set" "', argument " "1"" of type '" "struct _dwg_object_PLACEHOLDER *""'"); 
    }
    arg1 = (struct _dwg_object_PLACEHOLDER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_PLACEHOLDER_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLACEHOLDER_parent_get) {
  {
    struct _dwg_object_PLACEHOLDER *arg1 = (struct _dwg_object_PLACEHOLDER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PLACEHOLDER_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLACEHOLDER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLACEHOLDER_parent_get" "', argument " "1"" of type '" "struct _dwg_object_PLACEHOLDER *""'"); 
    }
    arg1 = (struct _dwg_object_PLACEHOLDER *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_PLACEHOLDER) {
  {
    int argvi = 0;
    struct _dwg_object_PLACEHOLDER *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_PLACEHOLDER();");
    }
    result = (struct _dwg_object_PLACEHOLDER *)calloc(1, sizeof(struct _dwg_object_PLACEHOLDER));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_PLACEHOLDER, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_PLACEHOLDER) {
  {
    struct _dwg_object_PLACEHOLDER *arg1 = (struct _dwg_object_PLACEHOLDER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_PLACEHOLDER(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLACEHOLDER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_PLACEHOLDER" "', argument " "1"" of type '" "struct _dwg_object_PLACEHOLDER *""'"); 
    }
    arg1 = (struct _dwg_object_PLACEHOLDER *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Break_parent_set) {
  {
    struct _dwg_LEADER_Break *arg1 = (struct _dwg_LEADER_Break *) 0 ;
    struct _dwg_LEADER_Line *arg2 = (struct _dwg_LEADER_Line *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_Break_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Break, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Break_parent_set" "', argument " "1"" of type '" "struct _dwg_LEADER_Break *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Break *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_LEADER_Line, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_LEADER_Break_parent_set" "', argument " "2"" of type '" "struct _dwg_LEADER_Line *""'"); 
    }
    arg2 = (struct _dwg_LEADER_Line *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Break_parent_get) {
  {
    struct _dwg_LEADER_Break *arg1 = (struct _dwg_LEADER_Break *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_LEADER_Line *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_Break_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Break, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Break_parent_get" "', argument " "1"" of type '" "struct _dwg_LEADER_Break *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Break *)(argp1);
    result = (struct _dwg_LEADER_Line *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_LEADER_Line, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Break_start_set) {
  {
    struct _dwg_LEADER_Break *arg1 = (struct _dwg_LEADER_Break *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_Break_start_set(self,start);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Break, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Break_start_set" "', argument " "1"" of type '" "struct _dwg_LEADER_Break *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Break *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_LEADER_Break_start_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->start = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Break_start_get) {
  {
    struct _dwg_LEADER_Break *arg1 = (struct _dwg_LEADER_Break *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_Break_start_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Break, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Break_start_get" "', argument " "1"" of type '" "struct _dwg_LEADER_Break *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Break *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->start);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Break_end_set) {
  {
    struct _dwg_LEADER_Break *arg1 = (struct _dwg_LEADER_Break *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_Break_end_set(self,end);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Break, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Break_end_set" "', argument " "1"" of type '" "struct _dwg_LEADER_Break *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Break *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_LEADER_Break_end_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->end = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Break_end_get) {
  {
    struct _dwg_LEADER_Break *arg1 = (struct _dwg_LEADER_Break *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_Break_end_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Break, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Break_end_get" "', argument " "1"" of type '" "struct _dwg_LEADER_Break *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Break *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->end);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_LEADER_Break) {
  {
    int argvi = 0;
    struct _dwg_LEADER_Break *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_LEADER_Break();");
    }
    result = (struct _dwg_LEADER_Break *)calloc(1, sizeof(struct _dwg_LEADER_Break));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_LEADER_Break, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_LEADER_Break) {
  {
    struct _dwg_LEADER_Break *arg1 = (struct _dwg_LEADER_Break *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_LEADER_Break(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Break, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_LEADER_Break" "', argument " "1"" of type '" "struct _dwg_LEADER_Break *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Break *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Line_parent_set) {
  {
    struct _dwg_LEADER_Line *arg1 = (struct _dwg_LEADER_Line *) 0 ;
    struct _dwg_LEADER_Node *arg2 = (struct _dwg_LEADER_Node *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_Line_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Line_parent_set" "', argument " "1"" of type '" "struct _dwg_LEADER_Line *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Line *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_LEADER_Node, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_LEADER_Line_parent_set" "', argument " "2"" of type '" "struct _dwg_LEADER_Node *""'"); 
    }
    arg2 = (struct _dwg_LEADER_Node *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Line_parent_get) {
  {
    struct _dwg_LEADER_Line *arg1 = (struct _dwg_LEADER_Line *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_LEADER_Node *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_Line_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Line_parent_get" "', argument " "1"" of type '" "struct _dwg_LEADER_Line *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Line *)(argp1);
    result = (struct _dwg_LEADER_Node *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_LEADER_Node, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Line_num_points_set) {
  {
    struct _dwg_LEADER_Line *arg1 = (struct _dwg_LEADER_Line *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_Line_num_points_set(self,num_points);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Line_num_points_set" "', argument " "1"" of type '" "struct _dwg_LEADER_Line *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Line *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_LEADER_Line_num_points_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_points = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Line_num_points_get) {
  {
    struct _dwg_LEADER_Line *arg1 = (struct _dwg_LEADER_Line *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_Line_num_points_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Line_num_points_get" "', argument " "1"" of type '" "struct _dwg_LEADER_Line *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Line *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_points);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Line_points_set) {
  {
    struct _dwg_LEADER_Line *arg1 = (struct _dwg_LEADER_Line *) 0 ;
    BITCODE_3DPOINT *arg2 = (BITCODE_3DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_Line_points_set(self,points);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Line_points_set" "', argument " "1"" of type '" "struct _dwg_LEADER_Line *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Line *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_LEADER_Line_points_set" "', argument " "2"" of type '" "BITCODE_3DPOINT *""'"); 
    }
    arg2 = (BITCODE_3DPOINT *)(argp2);
    if (arg1) (arg1)->points = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Line_points_get) {
  {
    struct _dwg_LEADER_Line *arg1 = (struct _dwg_LEADER_Line *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_Line_points_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Line_points_get" "', argument " "1"" of type '" "struct _dwg_LEADER_Line *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Line *)(argp1);
    result = (BITCODE_3DPOINT *) ((arg1)->points);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Line_num_breaks_set) {
  {
    struct _dwg_LEADER_Line *arg1 = (struct _dwg_LEADER_Line *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_Line_num_breaks_set(self,num_breaks);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Line_num_breaks_set" "', argument " "1"" of type '" "struct _dwg_LEADER_Line *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Line *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_LEADER_Line_num_breaks_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_breaks = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Line_num_breaks_get) {
  {
    struct _dwg_LEADER_Line *arg1 = (struct _dwg_LEADER_Line *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_Line_num_breaks_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Line_num_breaks_get" "', argument " "1"" of type '" "struct _dwg_LEADER_Line *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Line *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_breaks);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Line_breaks_set) {
  {
    struct _dwg_LEADER_Line *arg1 = (struct _dwg_LEADER_Line *) 0 ;
    Dwg_LEADER_Break *arg2 = (Dwg_LEADER_Break *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_Line_breaks_set(self,breaks);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Line_breaks_set" "', argument " "1"" of type '" "struct _dwg_LEADER_Line *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Line *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_LEADER_Break, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_LEADER_Line_breaks_set" "', argument " "2"" of type '" "Dwg_LEADER_Break *""'"); 
    }
    arg2 = (Dwg_LEADER_Break *)(argp2);
    if (arg1) (arg1)->breaks = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Line_breaks_get) {
  {
    struct _dwg_LEADER_Line *arg1 = (struct _dwg_LEADER_Line *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_LEADER_Break *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_Line_breaks_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Line_breaks_get" "', argument " "1"" of type '" "struct _dwg_LEADER_Line *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Line *)(argp1);
    result = (Dwg_LEADER_Break *) ((arg1)->breaks);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_LEADER_Break, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Line_line_index_set) {
  {
    struct _dwg_LEADER_Line *arg1 = (struct _dwg_LEADER_Line *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_Line_line_index_set(self,line_index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Line_line_index_set" "', argument " "1"" of type '" "struct _dwg_LEADER_Line *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Line *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_LEADER_Line_line_index_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->line_index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Line_line_index_get) {
  {
    struct _dwg_LEADER_Line *arg1 = (struct _dwg_LEADER_Line *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_Line_line_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Line_line_index_get" "', argument " "1"" of type '" "struct _dwg_LEADER_Line *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Line *)(argp1);
    result = (BITCODE_BL) ((arg1)->line_index);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Line_type_set) {
  {
    struct _dwg_LEADER_Line *arg1 = (struct _dwg_LEADER_Line *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_Line_type_set(self,type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Line_type_set" "', argument " "1"" of type '" "struct _dwg_LEADER_Line *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Line *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_LEADER_Line_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Line_type_get) {
  {
    struct _dwg_LEADER_Line *arg1 = (struct _dwg_LEADER_Line *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_Line_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Line_type_get" "', argument " "1"" of type '" "struct _dwg_LEADER_Line *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Line *)(argp1);
    result = (BITCODE_BS) ((arg1)->type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Line_color_set) {
  {
    struct _dwg_LEADER_Line *arg1 = (struct _dwg_LEADER_Line *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_Line_color_set(self,color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Line_color_set" "', argument " "1"" of type '" "struct _dwg_LEADER_Line *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Line *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_LEADER_Line_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Line_color_get) {
  {
    struct _dwg_LEADER_Line *arg1 = (struct _dwg_LEADER_Line *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_Line_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Line_color_get" "', argument " "1"" of type '" "struct _dwg_LEADER_Line *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Line *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Line_ltype_set) {
  {
    struct _dwg_LEADER_Line *arg1 = (struct _dwg_LEADER_Line *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_Line_ltype_set(self,ltype);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Line_ltype_set" "', argument " "1"" of type '" "struct _dwg_LEADER_Line *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Line *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_LEADER_Line_ltype_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->ltype = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Line_ltype_get) {
  {
    struct _dwg_LEADER_Line *arg1 = (struct _dwg_LEADER_Line *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_Line_ltype_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Line_ltype_get" "', argument " "1"" of type '" "struct _dwg_LEADER_Line *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Line *)(argp1);
    result = (BITCODE_H) ((arg1)->ltype);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Line_linewt_set) {
  {
    struct _dwg_LEADER_Line *arg1 = (struct _dwg_LEADER_Line *) 0 ;
    BITCODE_BLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_Line_linewt_set(self,linewt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Line_linewt_set" "', argument " "1"" of type '" "struct _dwg_LEADER_Line *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Line *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_LEADER_Line_linewt_set" "', argument " "2"" of type '" "BITCODE_BLd""'");
    } 
    arg2 = (BITCODE_BLd)(val2);
    if (arg1) (arg1)->linewt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Line_linewt_get) {
  {
    struct _dwg_LEADER_Line *arg1 = (struct _dwg_LEADER_Line *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_Line_linewt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Line_linewt_get" "', argument " "1"" of type '" "struct _dwg_LEADER_Line *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Line *)(argp1);
    result = (BITCODE_BLd) ((arg1)->linewt);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Line_arrow_size_set) {
  {
    struct _dwg_LEADER_Line *arg1 = (struct _dwg_LEADER_Line *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_Line_arrow_size_set(self,arrow_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Line_arrow_size_set" "', argument " "1"" of type '" "struct _dwg_LEADER_Line *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Line *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_LEADER_Line_arrow_size_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->arrow_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Line_arrow_size_get) {
  {
    struct _dwg_LEADER_Line *arg1 = (struct _dwg_LEADER_Line *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_Line_arrow_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Line_arrow_size_get" "', argument " "1"" of type '" "struct _dwg_LEADER_Line *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Line *)(argp1);
    result = (BITCODE_BD) ((arg1)->arrow_size);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Line_arrow_handle_set) {
  {
    struct _dwg_LEADER_Line *arg1 = (struct _dwg_LEADER_Line *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_Line_arrow_handle_set(self,arrow_handle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Line_arrow_handle_set" "', argument " "1"" of type '" "struct _dwg_LEADER_Line *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Line *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_LEADER_Line_arrow_handle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->arrow_handle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Line_arrow_handle_get) {
  {
    struct _dwg_LEADER_Line *arg1 = (struct _dwg_LEADER_Line *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_Line_arrow_handle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Line_arrow_handle_get" "', argument " "1"" of type '" "struct _dwg_LEADER_Line *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Line *)(argp1);
    result = (BITCODE_H) ((arg1)->arrow_handle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Line_flags_set) {
  {
    struct _dwg_LEADER_Line *arg1 = (struct _dwg_LEADER_Line *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_Line_flags_set(self,flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Line_flags_set" "', argument " "1"" of type '" "struct _dwg_LEADER_Line *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Line *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_LEADER_Line_flags_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Line_flags_get) {
  {
    struct _dwg_LEADER_Line *arg1 = (struct _dwg_LEADER_Line *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_Line_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Line, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Line_flags_get" "', argument " "1"" of type '" "struct _dwg_LEADER_Line *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Line *)(argp1);
    result = (BITCODE_BL) ((arg1)->flags);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_LEADER_Line) {
  {
    int argvi = 0;
    struct _dwg_LEADER_Line *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_LEADER_Line();");
    }
    result = (struct _dwg_LEADER_Line *)calloc(1, sizeof(struct _dwg_LEADER_Line));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_LEADER_Line, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_LEADER_Line) {
  {
    struct _dwg_LEADER_Line *arg1 = (struct _dwg_LEADER_Line *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_LEADER_Line(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Line, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_LEADER_Line" "', argument " "1"" of type '" "struct _dwg_LEADER_Line *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Line *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_ArrowHead_parent_set) {
  {
    struct _dwg_LEADER_ArrowHead *arg1 = (struct _dwg_LEADER_ArrowHead *) 0 ;
    struct _dwg_entity_MULTILEADER *arg2 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_ArrowHead_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_ArrowHead, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_ArrowHead_parent_set" "', argument " "1"" of type '" "struct _dwg_LEADER_ArrowHead *""'"); 
    }
    arg1 = (struct _dwg_LEADER_ArrowHead *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_MULTILEADER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_LEADER_ArrowHead_parent_set" "', argument " "2"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg2 = (struct _dwg_entity_MULTILEADER *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_ArrowHead_parent_get) {
  {
    struct _dwg_LEADER_ArrowHead *arg1 = (struct _dwg_LEADER_ArrowHead *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_entity_MULTILEADER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_ArrowHead_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_ArrowHead, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_ArrowHead_parent_get" "', argument " "1"" of type '" "struct _dwg_LEADER_ArrowHead *""'"); 
    }
    arg1 = (struct _dwg_LEADER_ArrowHead *)(argp1);
    result = (struct _dwg_entity_MULTILEADER *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_MULTILEADER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_ArrowHead_is_default_set) {
  {
    struct _dwg_LEADER_ArrowHead *arg1 = (struct _dwg_LEADER_ArrowHead *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_ArrowHead_is_default_set(self,is_default);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_ArrowHead, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_ArrowHead_is_default_set" "', argument " "1"" of type '" "struct _dwg_LEADER_ArrowHead *""'"); 
    }
    arg1 = (struct _dwg_LEADER_ArrowHead *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_LEADER_ArrowHead_is_default_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_default = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_ArrowHead_is_default_get) {
  {
    struct _dwg_LEADER_ArrowHead *arg1 = (struct _dwg_LEADER_ArrowHead *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_ArrowHead_is_default_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_ArrowHead, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_ArrowHead_is_default_get" "', argument " "1"" of type '" "struct _dwg_LEADER_ArrowHead *""'"); 
    }
    arg1 = (struct _dwg_LEADER_ArrowHead *)(argp1);
    result = (BITCODE_B) ((arg1)->is_default);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_ArrowHead_arrowhead_set) {
  {
    struct _dwg_LEADER_ArrowHead *arg1 = (struct _dwg_LEADER_ArrowHead *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_ArrowHead_arrowhead_set(self,arrowhead);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_ArrowHead, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_ArrowHead_arrowhead_set" "', argument " "1"" of type '" "struct _dwg_LEADER_ArrowHead *""'"); 
    }
    arg1 = (struct _dwg_LEADER_ArrowHead *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_LEADER_ArrowHead_arrowhead_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->arrowhead = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_ArrowHead_arrowhead_get) {
  {
    struct _dwg_LEADER_ArrowHead *arg1 = (struct _dwg_LEADER_ArrowHead *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_ArrowHead_arrowhead_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_ArrowHead, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_ArrowHead_arrowhead_get" "', argument " "1"" of type '" "struct _dwg_LEADER_ArrowHead *""'"); 
    }
    arg1 = (struct _dwg_LEADER_ArrowHead *)(argp1);
    result = (BITCODE_H) ((arg1)->arrowhead);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_LEADER_ArrowHead) {
  {
    int argvi = 0;
    struct _dwg_LEADER_ArrowHead *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_LEADER_ArrowHead();");
    }
    result = (struct _dwg_LEADER_ArrowHead *)calloc(1, sizeof(struct _dwg_LEADER_ArrowHead));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_LEADER_ArrowHead, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_LEADER_ArrowHead) {
  {
    struct _dwg_LEADER_ArrowHead *arg1 = (struct _dwg_LEADER_ArrowHead *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_LEADER_ArrowHead(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_ArrowHead, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_LEADER_ArrowHead" "', argument " "1"" of type '" "struct _dwg_LEADER_ArrowHead *""'"); 
    }
    arg1 = (struct _dwg_LEADER_ArrowHead *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_BlockLabel_parent_set) {
  {
    struct _dwg_LEADER_BlockLabel *arg1 = (struct _dwg_LEADER_BlockLabel *) 0 ;
    struct _dwg_entity_MULTILEADER *arg2 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_BlockLabel_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_BlockLabel, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_BlockLabel_parent_set" "', argument " "1"" of type '" "struct _dwg_LEADER_BlockLabel *""'"); 
    }
    arg1 = (struct _dwg_LEADER_BlockLabel *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_MULTILEADER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_LEADER_BlockLabel_parent_set" "', argument " "2"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg2 = (struct _dwg_entity_MULTILEADER *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_BlockLabel_parent_get) {
  {
    struct _dwg_LEADER_BlockLabel *arg1 = (struct _dwg_LEADER_BlockLabel *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_entity_MULTILEADER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_BlockLabel_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_BlockLabel, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_BlockLabel_parent_get" "', argument " "1"" of type '" "struct _dwg_LEADER_BlockLabel *""'"); 
    }
    arg1 = (struct _dwg_LEADER_BlockLabel *)(argp1);
    result = (struct _dwg_entity_MULTILEADER *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_MULTILEADER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_BlockLabel_attdef_set) {
  {
    struct _dwg_LEADER_BlockLabel *arg1 = (struct _dwg_LEADER_BlockLabel *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_BlockLabel_attdef_set(self,attdef);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_BlockLabel, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_BlockLabel_attdef_set" "', argument " "1"" of type '" "struct _dwg_LEADER_BlockLabel *""'"); 
    }
    arg1 = (struct _dwg_LEADER_BlockLabel *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_LEADER_BlockLabel_attdef_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->attdef = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_BlockLabel_attdef_get) {
  {
    struct _dwg_LEADER_BlockLabel *arg1 = (struct _dwg_LEADER_BlockLabel *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_BlockLabel_attdef_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_BlockLabel, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_BlockLabel_attdef_get" "', argument " "1"" of type '" "struct _dwg_LEADER_BlockLabel *""'"); 
    }
    arg1 = (struct _dwg_LEADER_BlockLabel *)(argp1);
    result = (BITCODE_H) ((arg1)->attdef);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_BlockLabel_label_text_set) {
  {
    struct _dwg_LEADER_BlockLabel *arg1 = (struct _dwg_LEADER_BlockLabel *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_BlockLabel_label_text_set(self,label_text);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_BlockLabel, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_BlockLabel_label_text_set" "', argument " "1"" of type '" "struct _dwg_LEADER_BlockLabel *""'"); 
    }
    arg1 = (struct _dwg_LEADER_BlockLabel *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_LEADER_BlockLabel_label_text_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->label_text);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->label_text = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->label_text = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_BlockLabel_label_text_get) {
  {
    struct _dwg_LEADER_BlockLabel *arg1 = (struct _dwg_LEADER_BlockLabel *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_BlockLabel_label_text_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_BlockLabel, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_BlockLabel_label_text_get" "', argument " "1"" of type '" "struct _dwg_LEADER_BlockLabel *""'"); 
    }
    arg1 = (struct _dwg_LEADER_BlockLabel *)(argp1);
    result = (BITCODE_TV) ((arg1)->label_text);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_BlockLabel_ui_index_set) {
  {
    struct _dwg_LEADER_BlockLabel *arg1 = (struct _dwg_LEADER_BlockLabel *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_BlockLabel_ui_index_set(self,ui_index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_BlockLabel, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_BlockLabel_ui_index_set" "', argument " "1"" of type '" "struct _dwg_LEADER_BlockLabel *""'"); 
    }
    arg1 = (struct _dwg_LEADER_BlockLabel *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_LEADER_BlockLabel_ui_index_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->ui_index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_BlockLabel_ui_index_get) {
  {
    struct _dwg_LEADER_BlockLabel *arg1 = (struct _dwg_LEADER_BlockLabel *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_BlockLabel_ui_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_BlockLabel, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_BlockLabel_ui_index_get" "', argument " "1"" of type '" "struct _dwg_LEADER_BlockLabel *""'"); 
    }
    arg1 = (struct _dwg_LEADER_BlockLabel *)(argp1);
    result = (BITCODE_BS) ((arg1)->ui_index);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_BlockLabel_width_set) {
  {
    struct _dwg_LEADER_BlockLabel *arg1 = (struct _dwg_LEADER_BlockLabel *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_BlockLabel_width_set(self,width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_BlockLabel, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_BlockLabel_width_set" "', argument " "1"" of type '" "struct _dwg_LEADER_BlockLabel *""'"); 
    }
    arg1 = (struct _dwg_LEADER_BlockLabel *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_LEADER_BlockLabel_width_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_BlockLabel_width_get) {
  {
    struct _dwg_LEADER_BlockLabel *arg1 = (struct _dwg_LEADER_BlockLabel *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_BlockLabel_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_BlockLabel, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_BlockLabel_width_get" "', argument " "1"" of type '" "struct _dwg_LEADER_BlockLabel *""'"); 
    }
    arg1 = (struct _dwg_LEADER_BlockLabel *)(argp1);
    result = (BITCODE_BD) ((arg1)->width);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_LEADER_BlockLabel) {
  {
    int argvi = 0;
    struct _dwg_LEADER_BlockLabel *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_LEADER_BlockLabel();");
    }
    result = (struct _dwg_LEADER_BlockLabel *)calloc(1, sizeof(struct _dwg_LEADER_BlockLabel));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_LEADER_BlockLabel, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_LEADER_BlockLabel) {
  {
    struct _dwg_LEADER_BlockLabel *arg1 = (struct _dwg_LEADER_BlockLabel *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_LEADER_BlockLabel(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_BlockLabel, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_LEADER_BlockLabel" "', argument " "1"" of type '" "struct _dwg_LEADER_BlockLabel *""'"); 
    }
    arg1 = (struct _dwg_LEADER_BlockLabel *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Node_parent_set) {
  {
    struct _dwg_LEADER_Node *arg1 = (struct _dwg_LEADER_Node *) 0 ;
    struct _dwg_entity_MULTILEADER *arg2 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_Node_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Node_parent_set" "', argument " "1"" of type '" "struct _dwg_LEADER_Node *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Node *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_MULTILEADER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_LEADER_Node_parent_set" "', argument " "2"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg2 = (struct _dwg_entity_MULTILEADER *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Node_parent_get) {
  {
    struct _dwg_LEADER_Node *arg1 = (struct _dwg_LEADER_Node *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_entity_MULTILEADER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_Node_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Node_parent_get" "', argument " "1"" of type '" "struct _dwg_LEADER_Node *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Node *)(argp1);
    result = (struct _dwg_entity_MULTILEADER *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_MULTILEADER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Node_has_lastleaderlinepoint_set) {
  {
    struct _dwg_LEADER_Node *arg1 = (struct _dwg_LEADER_Node *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_Node_has_lastleaderlinepoint_set(self,has_lastleaderlinepoint);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Node_has_lastleaderlinepoint_set" "', argument " "1"" of type '" "struct _dwg_LEADER_Node *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Node *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_LEADER_Node_has_lastleaderlinepoint_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_lastleaderlinepoint = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Node_has_lastleaderlinepoint_get) {
  {
    struct _dwg_LEADER_Node *arg1 = (struct _dwg_LEADER_Node *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_Node_has_lastleaderlinepoint_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Node_has_lastleaderlinepoint_get" "', argument " "1"" of type '" "struct _dwg_LEADER_Node *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Node *)(argp1);
    result = (BITCODE_B) ((arg1)->has_lastleaderlinepoint);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Node_has_dogleg_set) {
  {
    struct _dwg_LEADER_Node *arg1 = (struct _dwg_LEADER_Node *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_Node_has_dogleg_set(self,has_dogleg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Node_has_dogleg_set" "', argument " "1"" of type '" "struct _dwg_LEADER_Node *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Node *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_LEADER_Node_has_dogleg_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_dogleg = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Node_has_dogleg_get) {
  {
    struct _dwg_LEADER_Node *arg1 = (struct _dwg_LEADER_Node *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_Node_has_dogleg_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Node_has_dogleg_get" "', argument " "1"" of type '" "struct _dwg_LEADER_Node *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Node *)(argp1);
    result = (BITCODE_B) ((arg1)->has_dogleg);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Node_lastleaderlinepoint_set) {
  {
    struct _dwg_LEADER_Node *arg1 = (struct _dwg_LEADER_Node *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_Node_lastleaderlinepoint_set(self,lastleaderlinepoint);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Node_lastleaderlinepoint_set" "', argument " "1"" of type '" "struct _dwg_LEADER_Node *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Node *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_LEADER_Node_lastleaderlinepoint_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->lastleaderlinepoint = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Node_lastleaderlinepoint_get) {
  {
    struct _dwg_LEADER_Node *arg1 = (struct _dwg_LEADER_Node *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_Node_lastleaderlinepoint_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Node_lastleaderlinepoint_get" "', argument " "1"" of type '" "struct _dwg_LEADER_Node *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Node *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->lastleaderlinepoint);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Node_dogleg_vector_set) {
  {
    struct _dwg_LEADER_Node *arg1 = (struct _dwg_LEADER_Node *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_Node_dogleg_vector_set(self,dogleg_vector);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Node_dogleg_vector_set" "', argument " "1"" of type '" "struct _dwg_LEADER_Node *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Node *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_LEADER_Node_dogleg_vector_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->dogleg_vector = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Node_dogleg_vector_get) {
  {
    struct _dwg_LEADER_Node *arg1 = (struct _dwg_LEADER_Node *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_Node_dogleg_vector_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Node_dogleg_vector_get" "', argument " "1"" of type '" "struct _dwg_LEADER_Node *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Node *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->dogleg_vector);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Node_branch_index_set) {
  {
    struct _dwg_LEADER_Node *arg1 = (struct _dwg_LEADER_Node *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_Node_branch_index_set(self,branch_index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Node_branch_index_set" "', argument " "1"" of type '" "struct _dwg_LEADER_Node *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Node *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_LEADER_Node_branch_index_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->branch_index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Node_branch_index_get) {
  {
    struct _dwg_LEADER_Node *arg1 = (struct _dwg_LEADER_Node *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_Node_branch_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Node_branch_index_get" "', argument " "1"" of type '" "struct _dwg_LEADER_Node *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Node *)(argp1);
    result = (BITCODE_BL) ((arg1)->branch_index);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Node_dogleg_length_set) {
  {
    struct _dwg_LEADER_Node *arg1 = (struct _dwg_LEADER_Node *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_Node_dogleg_length_set(self,dogleg_length);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Node_dogleg_length_set" "', argument " "1"" of type '" "struct _dwg_LEADER_Node *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Node *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_LEADER_Node_dogleg_length_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->dogleg_length = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Node_dogleg_length_get) {
  {
    struct _dwg_LEADER_Node *arg1 = (struct _dwg_LEADER_Node *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_Node_dogleg_length_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Node_dogleg_length_get" "', argument " "1"" of type '" "struct _dwg_LEADER_Node *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Node *)(argp1);
    result = (BITCODE_BD) ((arg1)->dogleg_length);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Node_num_lines_set) {
  {
    struct _dwg_LEADER_Node *arg1 = (struct _dwg_LEADER_Node *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_Node_num_lines_set(self,num_lines);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Node_num_lines_set" "', argument " "1"" of type '" "struct _dwg_LEADER_Node *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Node *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_LEADER_Node_num_lines_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_lines = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Node_num_lines_get) {
  {
    struct _dwg_LEADER_Node *arg1 = (struct _dwg_LEADER_Node *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_Node_num_lines_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Node_num_lines_get" "', argument " "1"" of type '" "struct _dwg_LEADER_Node *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Node *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_lines);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Node_lines_set) {
  {
    struct _dwg_LEADER_Node *arg1 = (struct _dwg_LEADER_Node *) 0 ;
    Dwg_LEADER_Line *arg2 = (Dwg_LEADER_Line *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_Node_lines_set(self,lines);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Node_lines_set" "', argument " "1"" of type '" "struct _dwg_LEADER_Node *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Node *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_LEADER_Line, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_LEADER_Node_lines_set" "', argument " "2"" of type '" "Dwg_LEADER_Line *""'"); 
    }
    arg2 = (Dwg_LEADER_Line *)(argp2);
    if (arg1) (arg1)->lines = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Node_lines_get) {
  {
    struct _dwg_LEADER_Node *arg1 = (struct _dwg_LEADER_Node *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_LEADER_Line *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_Node_lines_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Node_lines_get" "', argument " "1"" of type '" "struct _dwg_LEADER_Node *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Node *)(argp1);
    result = (Dwg_LEADER_Line *) ((arg1)->lines);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_LEADER_Line, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Node_num_breaks_set) {
  {
    struct _dwg_LEADER_Node *arg1 = (struct _dwg_LEADER_Node *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_Node_num_breaks_set(self,num_breaks);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Node_num_breaks_set" "', argument " "1"" of type '" "struct _dwg_LEADER_Node *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Node *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_LEADER_Node_num_breaks_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_breaks = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Node_num_breaks_get) {
  {
    struct _dwg_LEADER_Node *arg1 = (struct _dwg_LEADER_Node *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_Node_num_breaks_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Node_num_breaks_get" "', argument " "1"" of type '" "struct _dwg_LEADER_Node *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Node *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_breaks);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Node_breaks_set) {
  {
    struct _dwg_LEADER_Node *arg1 = (struct _dwg_LEADER_Node *) 0 ;
    Dwg_LEADER_Break *arg2 = (Dwg_LEADER_Break *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_Node_breaks_set(self,breaks);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Node_breaks_set" "', argument " "1"" of type '" "struct _dwg_LEADER_Node *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Node *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_LEADER_Break, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_LEADER_Node_breaks_set" "', argument " "2"" of type '" "Dwg_LEADER_Break *""'"); 
    }
    arg2 = (Dwg_LEADER_Break *)(argp2);
    if (arg1) (arg1)->breaks = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Node_breaks_get) {
  {
    struct _dwg_LEADER_Node *arg1 = (struct _dwg_LEADER_Node *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_LEADER_Break *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_Node_breaks_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Node_breaks_get" "', argument " "1"" of type '" "struct _dwg_LEADER_Node *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Node *)(argp1);
    result = (Dwg_LEADER_Break *) ((arg1)->breaks);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_LEADER_Break, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Node_attach_dir_set) {
  {
    struct _dwg_LEADER_Node *arg1 = (struct _dwg_LEADER_Node *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LEADER_Node_attach_dir_set(self,attach_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Node_attach_dir_set" "', argument " "1"" of type '" "struct _dwg_LEADER_Node *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Node *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_LEADER_Node_attach_dir_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->attach_dir = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LEADER_Node_attach_dir_get) {
  {
    struct _dwg_LEADER_Node *arg1 = (struct _dwg_LEADER_Node *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LEADER_Node_attach_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LEADER_Node_attach_dir_get" "', argument " "1"" of type '" "struct _dwg_LEADER_Node *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Node *)(argp1);
    result = (BITCODE_BS) ((arg1)->attach_dir);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_LEADER_Node) {
  {
    int argvi = 0;
    struct _dwg_LEADER_Node *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_LEADER_Node();");
    }
    result = (struct _dwg_LEADER_Node *)calloc(1, sizeof(struct _dwg_LEADER_Node));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_LEADER_Node, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_LEADER_Node) {
  {
    struct _dwg_LEADER_Node *arg1 = (struct _dwg_LEADER_Node *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_LEADER_Node(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LEADER_Node, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_LEADER_Node" "', argument " "1"" of type '" "struct _dwg_LEADER_Node *""'"); 
    }
    arg1 = (struct _dwg_LEADER_Node *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_type_set) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_type_set(self,type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_type_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_Content_MText_type_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_type_get) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_type_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    result = (BITCODE_RC) ((arg1)->type);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_normal_set) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_normal_set(self,normal);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_normal_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MLEADER_Content_MText_normal_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->normal = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_normal_get) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_normal_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_normal_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->normal);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_location_set) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_location_set(self,location);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_location_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MLEADER_Content_MText_location_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->location = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_location_get) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_location_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_location_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->location);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_rotation_set) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_rotation_set(self,rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_rotation_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_Content_MText_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_rotation_get) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_rotation_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    result = (BITCODE_BD) ((arg1)->rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_style_set) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_style_set(self,style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_style_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MLEADER_Content_MText_style_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_style_get) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_style_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    result = (BITCODE_H) ((arg1)->style);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_direction_set) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_direction_set(self,direction);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_direction_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MLEADER_Content_MText_direction_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->direction = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_direction_get) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_direction_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_direction_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->direction);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_color_set) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_color_set(self,color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_color_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MLEADER_Content_MText_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_color_get) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_color_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_width_set) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_width_set(self,width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_width_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_Content_MText_width_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_width_get) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_width_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    result = (BITCODE_BD) ((arg1)->width);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_height_set) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_height_set(self,height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_height_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_Content_MText_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_height_get) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_height_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    result = (BITCODE_BD) ((arg1)->height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_line_spacing_factor_set) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_line_spacing_factor_set(self,line_spacing_factor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_line_spacing_factor_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_Content_MText_line_spacing_factor_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->line_spacing_factor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_line_spacing_factor_get) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_line_spacing_factor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_line_spacing_factor_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    result = (BITCODE_BD) ((arg1)->line_spacing_factor);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_default_text_set) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_default_text_set(self,default_text);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_default_text_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MLEADER_Content_MText_default_text_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->default_text);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->default_text = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->default_text = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_default_text_get) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_default_text_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_default_text_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    result = (BITCODE_TV) ((arg1)->default_text);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_line_spacing_style_set) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_line_spacing_style_set(self,line_spacing_style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_line_spacing_style_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_Content_MText_line_spacing_style_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->line_spacing_style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_line_spacing_style_get) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_line_spacing_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_line_spacing_style_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    result = (BITCODE_BS) ((arg1)->line_spacing_style);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_alignment_set) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_alignment_set(self,alignment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_alignment_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_Content_MText_alignment_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->alignment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_alignment_get) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_alignment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_alignment_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    result = (BITCODE_BS) ((arg1)->alignment);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_flow_set) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_flow_set(self,flow);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_flow_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_Content_MText_flow_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->flow = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_flow_get) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_flow_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_flow_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    result = (BITCODE_BS) ((arg1)->flow);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_bg_color_set) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_bg_color_set(self,bg_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_bg_color_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MLEADER_Content_MText_bg_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->bg_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_bg_color_get) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_bg_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_bg_color_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->bg_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_bg_scale_set) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_bg_scale_set(self,bg_scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_bg_scale_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_Content_MText_bg_scale_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->bg_scale = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_bg_scale_get) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_bg_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_bg_scale_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    result = (BITCODE_BD) ((arg1)->bg_scale);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_bg_transparency_set) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_bg_transparency_set(self,bg_transparency);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_bg_transparency_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_Content_MText_bg_transparency_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bg_transparency = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_bg_transparency_get) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_bg_transparency_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_bg_transparency_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    result = (BITCODE_BL) ((arg1)->bg_transparency);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_is_bg_fill_set) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_is_bg_fill_set(self,is_bg_fill);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_is_bg_fill_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_Content_MText_is_bg_fill_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_bg_fill = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_is_bg_fill_get) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_is_bg_fill_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_is_bg_fill_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    result = (BITCODE_B) ((arg1)->is_bg_fill);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_is_bg_mask_fill_set) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_is_bg_mask_fill_set(self,is_bg_mask_fill);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_is_bg_mask_fill_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_Content_MText_is_bg_mask_fill_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_bg_mask_fill = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_is_bg_mask_fill_get) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_is_bg_mask_fill_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_is_bg_mask_fill_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    result = (BITCODE_B) ((arg1)->is_bg_mask_fill);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_col_type_set) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_col_type_set(self,col_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_col_type_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_Content_MText_col_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->col_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_col_type_get) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_col_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_col_type_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    result = (BITCODE_BS) ((arg1)->col_type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_is_height_auto_set) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_is_height_auto_set(self,is_height_auto);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_is_height_auto_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_Content_MText_is_height_auto_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_height_auto = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_is_height_auto_get) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_is_height_auto_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_is_height_auto_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    result = (BITCODE_B) ((arg1)->is_height_auto);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_col_width_set) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_col_width_set(self,col_width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_col_width_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_Content_MText_col_width_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->col_width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_col_width_get) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_col_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_col_width_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    result = (BITCODE_BD) ((arg1)->col_width);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_col_gutter_set) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_col_gutter_set(self,col_gutter);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_col_gutter_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_Content_MText_col_gutter_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->col_gutter = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_col_gutter_get) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_col_gutter_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_col_gutter_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    result = (BITCODE_BD) ((arg1)->col_gutter);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_is_col_flow_reversed_set) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_is_col_flow_reversed_set(self,is_col_flow_reversed);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_is_col_flow_reversed_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_Content_MText_is_col_flow_reversed_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_col_flow_reversed = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_is_col_flow_reversed_get) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_is_col_flow_reversed_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_is_col_flow_reversed_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    result = (BITCODE_B) ((arg1)->is_col_flow_reversed);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_num_col_sizes_set) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_num_col_sizes_set(self,num_col_sizes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_num_col_sizes_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_Content_MText_num_col_sizes_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_col_sizes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_num_col_sizes_get) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_num_col_sizes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_num_col_sizes_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_col_sizes);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_col_sizes_set) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_col_sizes_set(self,col_sizes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_col_sizes_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MLEADER_Content_MText_col_sizes_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->col_sizes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_col_sizes_get) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_col_sizes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_col_sizes_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    result = (BITCODE_BD *) ((arg1)->col_sizes);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_word_break_set) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_word_break_set(self,word_break);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_word_break_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_Content_MText_word_break_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->word_break = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_word_break_get) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_word_break_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_word_break_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    result = (BITCODE_B) ((arg1)->word_break);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_unknown_set) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_unknown_set(self,unknown);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_unknown_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_Content_MText_unknown_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_MText_unknown_get) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_MText_unknown_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_MText_unknown_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_MLEADER_Content_MText) {
  {
    int argvi = 0;
    struct _dwg_MLEADER_Content_MText *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_MLEADER_Content_MText();");
    }
    result = (struct _dwg_MLEADER_Content_MText *)calloc(1, sizeof(struct _dwg_MLEADER_Content_MText));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_MLEADER_Content_MText, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_MLEADER_Content_MText) {
  {
    struct _dwg_MLEADER_Content_MText *arg1 = (struct _dwg_MLEADER_Content_MText *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_MLEADER_Content_MText(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_MText, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_MLEADER_Content_MText" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_MText *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_MText *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_Block_type_set) {
  {
    struct _dwg_MLEADER_Content_Block *arg1 = (struct _dwg_MLEADER_Content_Block *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_Block_type_set(self,type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_Block, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_Block_type_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_Block *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_Block *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_Content_Block_type_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_Block_type_get) {
  {
    struct _dwg_MLEADER_Content_Block *arg1 = (struct _dwg_MLEADER_Content_Block *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_Block_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_Block, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_Block_type_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_Block *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_Block *)(argp1);
    result = (BITCODE_RC) ((arg1)->type);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_Block_normal_set) {
  {
    struct _dwg_MLEADER_Content_Block *arg1 = (struct _dwg_MLEADER_Content_Block *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_Block_normal_set(self,normal);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_Block, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_Block_normal_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_Block *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_Block *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MLEADER_Content_Block_normal_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->normal = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_Block_normal_get) {
  {
    struct _dwg_MLEADER_Content_Block *arg1 = (struct _dwg_MLEADER_Content_Block *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_Block_normal_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_Block, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_Block_normal_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_Block *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_Block *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->normal);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_Block_location_set) {
  {
    struct _dwg_MLEADER_Content_Block *arg1 = (struct _dwg_MLEADER_Content_Block *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_Block_location_set(self,location);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_Block, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_Block_location_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_Block *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_Block *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MLEADER_Content_Block_location_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->location = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_Block_location_get) {
  {
    struct _dwg_MLEADER_Content_Block *arg1 = (struct _dwg_MLEADER_Content_Block *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_Block_location_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_Block, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_Block_location_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_Block *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_Block *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->location);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_Block_rotation_set) {
  {
    struct _dwg_MLEADER_Content_Block *arg1 = (struct _dwg_MLEADER_Content_Block *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_Block_rotation_set(self,rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_Block, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_Block_rotation_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_Block *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_Block *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_Content_Block_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_Block_rotation_get) {
  {
    struct _dwg_MLEADER_Content_Block *arg1 = (struct _dwg_MLEADER_Content_Block *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_Block_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_Block, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_Block_rotation_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_Block *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_Block *)(argp1);
    result = (BITCODE_BD) ((arg1)->rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_Block_block_table_set) {
  {
    struct _dwg_MLEADER_Content_Block *arg1 = (struct _dwg_MLEADER_Content_Block *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_Block_block_table_set(self,block_table);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_Block, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_Block_block_table_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_Block *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_Block *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MLEADER_Content_Block_block_table_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->block_table = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_Block_block_table_get) {
  {
    struct _dwg_MLEADER_Content_Block *arg1 = (struct _dwg_MLEADER_Content_Block *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_Block_block_table_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_Block, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_Block_block_table_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_Block *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_Block *)(argp1);
    result = (BITCODE_H) ((arg1)->block_table);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_Block_scale_set) {
  {
    struct _dwg_MLEADER_Content_Block *arg1 = (struct _dwg_MLEADER_Content_Block *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_Block_scale_set(self,scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_Block, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_Block_scale_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_Block *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_Block *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MLEADER_Content_Block_scale_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->scale = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_Block_scale_get) {
  {
    struct _dwg_MLEADER_Content_Block *arg1 = (struct _dwg_MLEADER_Content_Block *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_Block_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_Block, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_Block_scale_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_Block *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_Block *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->scale);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_Block_color_set) {
  {
    struct _dwg_MLEADER_Content_Block *arg1 = (struct _dwg_MLEADER_Content_Block *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_Block_color_set(self,color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_Block, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_Block_color_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_Block *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_Block *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MLEADER_Content_Block_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_Block_color_get) {
  {
    struct _dwg_MLEADER_Content_Block *arg1 = (struct _dwg_MLEADER_Content_Block *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_Block_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_Block, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_Block_color_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_Block *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_Block *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_Block_transform_set) {
  {
    struct _dwg_MLEADER_Content_Block *arg1 = (struct _dwg_MLEADER_Content_Block *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_Block_transform_set(self,transform);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_Block, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_Block_transform_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_Block *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_Block *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MLEADER_Content_Block_transform_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->transform = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_Block_transform_get) {
  {
    struct _dwg_MLEADER_Content_Block *arg1 = (struct _dwg_MLEADER_Content_Block *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_Block_transform_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_Block, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_Block_transform_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_Block *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_Block *)(argp1);
    result = (BITCODE_BD *) ((arg1)->transform);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_MLEADER_Content_Block) {
  {
    int argvi = 0;
    struct _dwg_MLEADER_Content_Block *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_MLEADER_Content_Block();");
    }
    result = (struct _dwg_MLEADER_Content_Block *)calloc(1, sizeof(struct _dwg_MLEADER_Content_Block));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_MLEADER_Content_Block, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_MLEADER_Content_Block) {
  {
    struct _dwg_MLEADER_Content_Block *arg1 = (struct _dwg_MLEADER_Content_Block *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_MLEADER_Content_Block(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content_Block, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_MLEADER_Content_Block" "', argument " "1"" of type '" "struct _dwg_MLEADER_Content_Block *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_Content_Block *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_txt_set) {
  {
    union _dwg_MLEADER_Content *arg1 = (union _dwg_MLEADER_Content *) 0 ;
    Dwg_MLEADER_Content_MText *arg2 = (Dwg_MLEADER_Content_MText *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_txt_set(self,txt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_txt_set" "', argument " "1"" of type '" "union _dwg_MLEADER_Content *""'"); 
    }
    arg1 = (union _dwg_MLEADER_Content *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MLEADER_Content_txt_set" "', argument " "2"" of type '" "Dwg_MLEADER_Content_MText *""'"); 
    }
    arg2 = (Dwg_MLEADER_Content_MText *)(argp2);
    if (arg1) (arg1)->txt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_txt_get) {
  {
    union _dwg_MLEADER_Content *arg1 = (union _dwg_MLEADER_Content *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_MLEADER_Content_MText *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_txt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_txt_get" "', argument " "1"" of type '" "union _dwg_MLEADER_Content *""'"); 
    }
    arg1 = (union _dwg_MLEADER_Content *)(argp1);
    result = (Dwg_MLEADER_Content_MText *)& ((arg1)->txt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_MLEADER_Content_MText, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_blk_set) {
  {
    union _dwg_MLEADER_Content *arg1 = (union _dwg_MLEADER_Content *) 0 ;
    Dwg_MLEADER_Content_Block *arg2 = (Dwg_MLEADER_Content_Block *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_blk_set(self,blk);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_blk_set" "', argument " "1"" of type '" "union _dwg_MLEADER_Content *""'"); 
    }
    arg1 = (union _dwg_MLEADER_Content *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_MLEADER_Content_Block, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MLEADER_Content_blk_set" "', argument " "2"" of type '" "Dwg_MLEADER_Content_Block *""'"); 
    }
    arg2 = (Dwg_MLEADER_Content_Block *)(argp2);
    if (arg1) (arg1)->blk = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_Content_blk_get) {
  {
    union _dwg_MLEADER_Content *arg1 = (union _dwg_MLEADER_Content *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_MLEADER_Content_Block *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_Content_blk_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_Content_blk_get" "', argument " "1"" of type '" "union _dwg_MLEADER_Content *""'"); 
    }
    arg1 = (union _dwg_MLEADER_Content *)(argp1);
    result = (Dwg_MLEADER_Content_Block *)& ((arg1)->blk);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_MLEADER_Content_Block, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_MLEADER_Content) {
  {
    int argvi = 0;
    union _dwg_MLEADER_Content *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_MLEADER_Content();");
    }
    result = (union _dwg_MLEADER_Content *)calloc(1, sizeof(union _dwg_MLEADER_Content));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_MLEADER_Content, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_MLEADER_Content) {
  {
    union _dwg_MLEADER_Content *arg1 = (union _dwg_MLEADER_Content *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_MLEADER_Content(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_Content, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_MLEADER_Content" "', argument " "1"" of type '" "union _dwg_MLEADER_Content *""'"); 
    }
    arg1 = (union _dwg_MLEADER_Content *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_num_leaders_set) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_num_leaders_set(self,num_leaders);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_num_leaders_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_AnnotContext_num_leaders_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_leaders = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_num_leaders_get) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_num_leaders_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_num_leaders_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_leaders);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_leaders_set) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    Dwg_LEADER_Node *arg2 = (Dwg_LEADER_Node *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_leaders_set(self,leaders);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_leaders_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_LEADER_Node, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MLEADER_AnnotContext_leaders_set" "', argument " "2"" of type '" "Dwg_LEADER_Node *""'"); 
    }
    arg2 = (Dwg_LEADER_Node *)(argp2);
    if (arg1) (arg1)->leaders = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_leaders_get) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_LEADER_Node *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_leaders_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_leaders_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    result = (Dwg_LEADER_Node *) ((arg1)->leaders);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_LEADER_Node, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_attach_dir_set) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_attach_dir_set(self,attach_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_attach_dir_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_AnnotContext_attach_dir_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->attach_dir = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_attach_dir_get) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_attach_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_attach_dir_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    result = (BITCODE_BS) ((arg1)->attach_dir);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_scale_factor_set) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_scale_factor_set(self,scale_factor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_scale_factor_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_AnnotContext_scale_factor_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->scale_factor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_scale_factor_get) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_scale_factor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_scale_factor_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    result = (BITCODE_BD) ((arg1)->scale_factor);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_content_base_set) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_content_base_set(self,content_base);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_content_base_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MLEADER_AnnotContext_content_base_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->content_base = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_content_base_get) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_content_base_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_content_base_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->content_base);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_text_height_set) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_text_height_set(self,text_height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_text_height_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_AnnotContext_text_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->text_height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_text_height_get) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_text_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_text_height_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    result = (BITCODE_BD) ((arg1)->text_height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_arrow_size_set) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_arrow_size_set(self,arrow_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_arrow_size_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_AnnotContext_arrow_size_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->arrow_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_arrow_size_get) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_arrow_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_arrow_size_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    result = (BITCODE_BD) ((arg1)->arrow_size);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_landing_gap_set) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_landing_gap_set(self,landing_gap);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_landing_gap_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_AnnotContext_landing_gap_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->landing_gap = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_landing_gap_get) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_landing_gap_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_landing_gap_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    result = (BITCODE_BD) ((arg1)->landing_gap);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_text_left_set) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_text_left_set(self,text_left);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_text_left_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_AnnotContext_text_left_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->text_left = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_text_left_get) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_text_left_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_text_left_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    result = (BITCODE_BS) ((arg1)->text_left);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_text_right_set) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_text_right_set(self,text_right);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_text_right_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_AnnotContext_text_right_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->text_right = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_text_right_get) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_text_right_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_text_right_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    result = (BITCODE_BS) ((arg1)->text_right);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_text_angletype_set) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_text_angletype_set(self,text_angletype);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_text_angletype_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_AnnotContext_text_angletype_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->text_angletype = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_text_angletype_get) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_text_angletype_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_text_angletype_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    result = (BITCODE_BS) ((arg1)->text_angletype);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_text_alignment_set) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_text_alignment_set(self,text_alignment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_text_alignment_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_AnnotContext_text_alignment_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->text_alignment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_text_alignment_get) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_text_alignment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_text_alignment_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    result = (BITCODE_BS) ((arg1)->text_alignment);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_has_content_txt_set) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_has_content_txt_set(self,has_content_txt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_has_content_txt_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_AnnotContext_has_content_txt_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_content_txt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_has_content_txt_get) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_has_content_txt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_has_content_txt_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    result = (BITCODE_B) ((arg1)->has_content_txt);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_has_content_blk_set) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_has_content_blk_set(self,has_content_blk);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_has_content_blk_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_AnnotContext_has_content_blk_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_content_blk = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_has_content_blk_get) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_has_content_blk_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_has_content_blk_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    result = (BITCODE_B) ((arg1)->has_content_blk);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_content_set) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    Dwg_MLEADER_Content *arg2 = (Dwg_MLEADER_Content *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_content_set(self,content);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_content_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_MLEADER_Content, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MLEADER_AnnotContext_content_set" "', argument " "2"" of type '" "Dwg_MLEADER_Content *""'"); 
    }
    arg2 = (Dwg_MLEADER_Content *)(argp2);
    if (arg1) (arg1)->content = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_content_get) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_MLEADER_Content *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_content_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_content_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    result = (Dwg_MLEADER_Content *)& ((arg1)->content);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_MLEADER_Content, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_base_set) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_base_set(self,base);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_base_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MLEADER_AnnotContext_base_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->base = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_base_get) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_base_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_base_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->base);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_base_dir_set) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_base_dir_set(self,base_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_base_dir_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MLEADER_AnnotContext_base_dir_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->base_dir = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_base_dir_get) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_base_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_base_dir_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->base_dir);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_base_vert_set) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_base_vert_set(self,base_vert);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_base_vert_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MLEADER_AnnotContext_base_vert_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->base_vert = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_base_vert_get) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_base_vert_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_base_vert_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->base_vert);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_is_normal_reversed_set) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_is_normal_reversed_set(self,is_normal_reversed);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_is_normal_reversed_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_AnnotContext_is_normal_reversed_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_normal_reversed = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_is_normal_reversed_get) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_is_normal_reversed_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_is_normal_reversed_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    result = (BITCODE_B) ((arg1)->is_normal_reversed);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_text_top_set) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_text_top_set(self,text_top);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_text_top_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_AnnotContext_text_top_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->text_top = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_text_top_get) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_text_top_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_text_top_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    result = (BITCODE_BS) ((arg1)->text_top);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_text_bottom_set) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_text_bottom_set(self,text_bottom);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_text_bottom_set" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MLEADER_AnnotContext_text_bottom_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->text_bottom = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MLEADER_AnnotContext_text_bottom_get) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MLEADER_AnnotContext_text_bottom_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MLEADER_AnnotContext_text_bottom_get" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    result = (BITCODE_BS) ((arg1)->text_bottom);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_MLEADER_AnnotContext) {
  {
    int argvi = 0;
    struct _dwg_MLEADER_AnnotContext *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_MLEADER_AnnotContext();");
    }
    result = (struct _dwg_MLEADER_AnnotContext *)calloc(1, sizeof(struct _dwg_MLEADER_AnnotContext));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_MLEADER_AnnotContext, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_MLEADER_AnnotContext) {
  {
    struct _dwg_MLEADER_AnnotContext *arg1 = (struct _dwg_MLEADER_AnnotContext *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_MLEADER_AnnotContext(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MLEADER_AnnotContext, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_MLEADER_AnnotContext" "', argument " "1"" of type '" "struct _dwg_MLEADER_AnnotContext *""'"); 
    }
    arg1 = (struct _dwg_MLEADER_AnnotContext *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_parent_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MULTILEADER_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_parent_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_class_version_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_class_version_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MULTILEADER_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_class_version_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_class_version_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_ctx_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    Dwg_MLEADER_AnnotContext *arg2 = (Dwg_MLEADER_AnnotContext *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_ctx_set(self,ctx);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_ctx_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MULTILEADER_ctx_set" "', argument " "2"" of type '" "Dwg_MLEADER_AnnotContext *""'"); 
    }
    arg2 = (Dwg_MLEADER_AnnotContext *)(argp2);
    if (arg1) (arg1)->ctx = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_ctx_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_MLEADER_AnnotContext *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_ctx_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_ctx_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (Dwg_MLEADER_AnnotContext *)& ((arg1)->ctx);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_MLEADER_AnnotContext, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_mleaderstyle_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_mleaderstyle_set(self,mleaderstyle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_mleaderstyle_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MULTILEADER_mleaderstyle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->mleaderstyle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_mleaderstyle_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_mleaderstyle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_mleaderstyle_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_H) ((arg1)->mleaderstyle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_flags_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_flags_set(self,flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_flags_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MULTILEADER_flags_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_flags_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_flags_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_BL) ((arg1)->flags);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_type_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_type_set(self,type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_type_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MULTILEADER_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_type_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_type_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_BS) ((arg1)->type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_line_color_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_line_color_set(self,line_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_line_color_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MULTILEADER_line_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->line_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_line_color_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_line_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_line_color_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->line_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_line_ltype_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_line_ltype_set(self,line_ltype);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_line_ltype_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MULTILEADER_line_ltype_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->line_ltype = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_line_ltype_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_line_ltype_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_line_ltype_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_H) ((arg1)->line_ltype);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_line_linewt_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_BLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_line_linewt_set(self,line_linewt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_line_linewt_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MULTILEADER_line_linewt_set" "', argument " "2"" of type '" "BITCODE_BLd""'");
    } 
    arg2 = (BITCODE_BLd)(val2);
    if (arg1) (arg1)->line_linewt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_line_linewt_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_line_linewt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_line_linewt_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_BLd) ((arg1)->line_linewt);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_has_landing_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_has_landing_set(self,has_landing);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_has_landing_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MULTILEADER_has_landing_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_landing = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_has_landing_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_has_landing_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_has_landing_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_B) ((arg1)->has_landing);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_has_dogleg_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_has_dogleg_set(self,has_dogleg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_has_dogleg_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MULTILEADER_has_dogleg_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_dogleg = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_has_dogleg_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_has_dogleg_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_has_dogleg_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_B) ((arg1)->has_dogleg);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_landing_dist_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_landing_dist_set(self,landing_dist);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_landing_dist_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MULTILEADER_landing_dist_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->landing_dist = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_landing_dist_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_landing_dist_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_landing_dist_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_BD) ((arg1)->landing_dist);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_arrow_handle_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_arrow_handle_set(self,arrow_handle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_arrow_handle_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MULTILEADER_arrow_handle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->arrow_handle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_arrow_handle_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_arrow_handle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_arrow_handle_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_H) ((arg1)->arrow_handle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_arrow_size_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_arrow_size_set(self,arrow_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_arrow_size_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MULTILEADER_arrow_size_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->arrow_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_arrow_size_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_arrow_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_arrow_size_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_BD) ((arg1)->arrow_size);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_style_content_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_style_content_set(self,style_content);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_style_content_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MULTILEADER_style_content_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->style_content = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_style_content_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_style_content_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_style_content_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_BS) ((arg1)->style_content);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_text_style_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_text_style_set(self,text_style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_text_style_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MULTILEADER_text_style_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->text_style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_text_style_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_text_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_text_style_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_H) ((arg1)->text_style);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_text_left_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_text_left_set(self,text_left);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_text_left_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MULTILEADER_text_left_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->text_left = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_text_left_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_text_left_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_text_left_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_BS) ((arg1)->text_left);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_text_right_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_text_right_set(self,text_right);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_text_right_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MULTILEADER_text_right_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->text_right = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_text_right_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_text_right_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_text_right_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_BS) ((arg1)->text_right);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_text_angletype_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_text_angletype_set(self,text_angletype);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_text_angletype_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MULTILEADER_text_angletype_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->text_angletype = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_text_angletype_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_text_angletype_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_text_angletype_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_BS) ((arg1)->text_angletype);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_text_alignment_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_text_alignment_set(self,text_alignment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_text_alignment_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MULTILEADER_text_alignment_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->text_alignment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_text_alignment_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_text_alignment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_text_alignment_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_BS) ((arg1)->text_alignment);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_text_color_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_text_color_set(self,text_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_text_color_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MULTILEADER_text_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->text_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_text_color_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_text_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_text_color_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->text_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_has_text_frame_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_has_text_frame_set(self,has_text_frame);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_has_text_frame_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MULTILEADER_has_text_frame_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_text_frame = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_has_text_frame_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_has_text_frame_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_has_text_frame_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_B) ((arg1)->has_text_frame);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_block_style_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_block_style_set(self,block_style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_block_style_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MULTILEADER_block_style_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->block_style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_block_style_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_block_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_block_style_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_H) ((arg1)->block_style);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_block_color_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_block_color_set(self,block_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_block_color_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MULTILEADER_block_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->block_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_block_color_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_block_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_block_color_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->block_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_block_scale_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_block_scale_set(self,block_scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_block_scale_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MULTILEADER_block_scale_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->block_scale = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_block_scale_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_block_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_block_scale_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->block_scale);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_block_rotation_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_block_rotation_set(self,block_rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_block_rotation_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MULTILEADER_block_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->block_rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_block_rotation_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_block_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_block_rotation_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_BD) ((arg1)->block_rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_style_attachment_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_style_attachment_set(self,style_attachment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_style_attachment_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MULTILEADER_style_attachment_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->style_attachment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_style_attachment_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_style_attachment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_style_attachment_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_BS) ((arg1)->style_attachment);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_is_annotative_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_is_annotative_set(self,is_annotative);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_is_annotative_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MULTILEADER_is_annotative_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_annotative = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_is_annotative_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_is_annotative_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_is_annotative_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_B) ((arg1)->is_annotative);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_num_arrowheads_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_num_arrowheads_set(self,num_arrowheads);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_num_arrowheads_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MULTILEADER_num_arrowheads_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_arrowheads = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_num_arrowheads_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_num_arrowheads_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_num_arrowheads_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_arrowheads);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_arrowheads_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    Dwg_LEADER_ArrowHead *arg2 = (Dwg_LEADER_ArrowHead *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_arrowheads_set(self,arrowheads);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_arrowheads_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_LEADER_ArrowHead, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MULTILEADER_arrowheads_set" "', argument " "2"" of type '" "Dwg_LEADER_ArrowHead *""'"); 
    }
    arg2 = (Dwg_LEADER_ArrowHead *)(argp2);
    if (arg1) (arg1)->arrowheads = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_arrowheads_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_LEADER_ArrowHead *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_arrowheads_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_arrowheads_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (Dwg_LEADER_ArrowHead *) ((arg1)->arrowheads);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_LEADER_ArrowHead, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_num_blocklabels_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_num_blocklabels_set(self,num_blocklabels);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_num_blocklabels_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MULTILEADER_num_blocklabels_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_blocklabels = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_num_blocklabels_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_num_blocklabels_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_num_blocklabels_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_blocklabels);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_blocklabels_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    Dwg_LEADER_BlockLabel *arg2 = (Dwg_LEADER_BlockLabel *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_blocklabels_set(self,blocklabels);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_blocklabels_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_LEADER_BlockLabel, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MULTILEADER_blocklabels_set" "', argument " "2"" of type '" "Dwg_LEADER_BlockLabel *""'"); 
    }
    arg2 = (Dwg_LEADER_BlockLabel *)(argp2);
    if (arg1) (arg1)->blocklabels = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_blocklabels_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_LEADER_BlockLabel *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_blocklabels_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_blocklabels_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (Dwg_LEADER_BlockLabel *) ((arg1)->blocklabels);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_LEADER_BlockLabel, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_is_neg_textdir_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_is_neg_textdir_set(self,is_neg_textdir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_is_neg_textdir_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MULTILEADER_is_neg_textdir_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_neg_textdir = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_is_neg_textdir_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_is_neg_textdir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_is_neg_textdir_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_B) ((arg1)->is_neg_textdir);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_ipe_alignment_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_ipe_alignment_set(self,ipe_alignment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_ipe_alignment_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MULTILEADER_ipe_alignment_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->ipe_alignment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_ipe_alignment_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_ipe_alignment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_ipe_alignment_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_BS) ((arg1)->ipe_alignment);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_justification_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_justification_set(self,justification);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_justification_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MULTILEADER_justification_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->justification = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_justification_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_justification_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_justification_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_BS) ((arg1)->justification);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_scale_factor_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_scale_factor_set(self,scale_factor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_scale_factor_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MULTILEADER_scale_factor_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->scale_factor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_scale_factor_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_scale_factor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_scale_factor_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_BD) ((arg1)->scale_factor);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_attach_dir_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_attach_dir_set(self,attach_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_attach_dir_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MULTILEADER_attach_dir_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->attach_dir = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_attach_dir_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_attach_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_attach_dir_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_BS) ((arg1)->attach_dir);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_attach_top_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_attach_top_set(self,attach_top);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_attach_top_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MULTILEADER_attach_top_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->attach_top = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_attach_top_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_attach_top_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_attach_top_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_BS) ((arg1)->attach_top);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_attach_bottom_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_attach_bottom_set(self,attach_bottom);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_attach_bottom_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MULTILEADER_attach_bottom_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->attach_bottom = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_attach_bottom_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_attach_bottom_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_attach_bottom_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_BS) ((arg1)->attach_bottom);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_is_text_extended_set) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_is_text_extended_set(self,is_text_extended);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_is_text_extended_set" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MULTILEADER_is_text_extended_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_text_extended = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MULTILEADER_is_text_extended_get) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MULTILEADER_is_text_extended_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MULTILEADER_is_text_extended_get" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    result = (BITCODE_B) ((arg1)->is_text_extended);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_MULTILEADER) {
  {
    int argvi = 0;
    struct _dwg_entity_MULTILEADER *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_MULTILEADER();");
    }
    result = (struct _dwg_entity_MULTILEADER *)calloc(1, sizeof(struct _dwg_entity_MULTILEADER));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_MULTILEADER, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_MULTILEADER) {
  {
    struct _dwg_entity_MULTILEADER *arg1 = (struct _dwg_entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_MULTILEADER(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MULTILEADER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_MULTILEADER" "', argument " "1"" of type '" "struct _dwg_entity_MULTILEADER *""'"); 
    }
    arg1 = (struct _dwg_entity_MULTILEADER *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_parent_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_parent_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MLEADERSTYLE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_parent_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_parent_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_class_version_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADERSTYLE_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_class_version_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_content_type_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_content_type_set(self,content_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_content_type_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADERSTYLE_content_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->content_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_content_type_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_content_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_content_type_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->content_type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_mleader_order_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_mleader_order_set(self,mleader_order);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_mleader_order_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADERSTYLE_mleader_order_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->mleader_order = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_mleader_order_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_mleader_order_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_mleader_order_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->mleader_order);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_leader_order_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_leader_order_set(self,leader_order);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_leader_order_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADERSTYLE_leader_order_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->leader_order = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_leader_order_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_leader_order_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_leader_order_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->leader_order);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_max_points_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_max_points_set(self,max_points);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_max_points_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADERSTYLE_max_points_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->max_points = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_max_points_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_max_points_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_max_points_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->max_points);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_first_seg_angle_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_first_seg_angle_set(self,first_seg_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_first_seg_angle_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADERSTYLE_first_seg_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->first_seg_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_first_seg_angle_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_first_seg_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_first_seg_angle_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->first_seg_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_second_seg_angle_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_second_seg_angle_set(self,second_seg_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_second_seg_angle_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADERSTYLE_second_seg_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->second_seg_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_second_seg_angle_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_second_seg_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_second_seg_angle_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->second_seg_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_type_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_type_set(self,type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_type_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADERSTYLE_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_type_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_type_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_line_color_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_line_color_set(self,line_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_line_color_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MLEADERSTYLE_line_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->line_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_line_color_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_line_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_line_color_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->line_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_line_type_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_line_type_set(self,line_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_line_type_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MLEADERSTYLE_line_type_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->line_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_line_type_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_line_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_line_type_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_H) ((arg1)->line_type);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_linewt_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_BLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_linewt_set(self,linewt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_linewt_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADERSTYLE_linewt_set" "', argument " "2"" of type '" "BITCODE_BLd""'");
    } 
    arg2 = (BITCODE_BLd)(val2);
    if (arg1) (arg1)->linewt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_linewt_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_linewt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_linewt_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_BLd) ((arg1)->linewt);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_has_landing_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_has_landing_set(self,has_landing);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_has_landing_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADERSTYLE_has_landing_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_landing = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_has_landing_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_has_landing_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_has_landing_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->has_landing);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_has_dogleg_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_has_dogleg_set(self,has_dogleg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_has_dogleg_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADERSTYLE_has_dogleg_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_dogleg = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_has_dogleg_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_has_dogleg_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_has_dogleg_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->has_dogleg);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_landing_gap_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_landing_gap_set(self,landing_gap);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_landing_gap_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADERSTYLE_landing_gap_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->landing_gap = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_landing_gap_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_landing_gap_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_landing_gap_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->landing_gap);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_landing_dist_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_landing_dist_set(self,landing_dist);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_landing_dist_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADERSTYLE_landing_dist_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->landing_dist = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_landing_dist_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_landing_dist_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_landing_dist_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->landing_dist);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_description_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_description_set(self,description);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_description_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MLEADERSTYLE_description_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->description);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->description = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->description = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_description_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_description_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_description_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_TV) ((arg1)->description);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_arrow_head_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_arrow_head_set(self,arrow_head);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_arrow_head_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MLEADERSTYLE_arrow_head_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->arrow_head = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_arrow_head_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_arrow_head_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_arrow_head_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_H) ((arg1)->arrow_head);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_arrow_head_size_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_arrow_head_size_set(self,arrow_head_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_arrow_head_size_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADERSTYLE_arrow_head_size_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->arrow_head_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_arrow_head_size_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_arrow_head_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_arrow_head_size_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->arrow_head_size);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_text_default_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_text_default_set(self,text_default);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_text_default_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MLEADERSTYLE_text_default_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->text_default);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->text_default = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->text_default = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_text_default_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_text_default_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_text_default_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_TV) ((arg1)->text_default);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_text_style_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_text_style_set(self,text_style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_text_style_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MLEADERSTYLE_text_style_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->text_style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_text_style_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_text_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_text_style_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_H) ((arg1)->text_style);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_attach_left_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_attach_left_set(self,attach_left);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_attach_left_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADERSTYLE_attach_left_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->attach_left = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_attach_left_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_attach_left_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_attach_left_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->attach_left);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_attach_right_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_attach_right_set(self,attach_right);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_attach_right_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADERSTYLE_attach_right_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->attach_right = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_attach_right_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_attach_right_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_attach_right_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->attach_right);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_text_angle_type_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_text_angle_type_set(self,text_angle_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_text_angle_type_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADERSTYLE_text_angle_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->text_angle_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_text_angle_type_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_text_angle_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_text_angle_type_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->text_angle_type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_text_align_type_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_text_align_type_set(self,text_align_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_text_align_type_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADERSTYLE_text_align_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->text_align_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_text_align_type_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_text_align_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_text_align_type_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->text_align_type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_text_color_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_text_color_set(self,text_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_text_color_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MLEADERSTYLE_text_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->text_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_text_color_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_text_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_text_color_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->text_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_text_height_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_text_height_set(self,text_height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_text_height_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADERSTYLE_text_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->text_height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_text_height_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_text_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_text_height_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->text_height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_has_text_frame_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_has_text_frame_set(self,has_text_frame);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_has_text_frame_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADERSTYLE_has_text_frame_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_text_frame = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_has_text_frame_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_has_text_frame_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_has_text_frame_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->has_text_frame);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_text_always_left_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_text_always_left_set(self,text_always_left);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_text_always_left_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADERSTYLE_text_always_left_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->text_always_left = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_text_always_left_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_text_always_left_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_text_always_left_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->text_always_left);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_align_space_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_align_space_set(self,align_space);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_align_space_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADERSTYLE_align_space_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->align_space = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_align_space_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_align_space_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_align_space_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->align_space);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_block_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_block_set(self,block);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_block_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MLEADERSTYLE_block_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->block = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_block_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_block_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_block_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_H) ((arg1)->block);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_block_color_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_block_color_set(self,block_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_block_color_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MLEADERSTYLE_block_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->block_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_block_color_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_block_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_block_color_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->block_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_block_scale_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_block_scale_set(self,block_scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_block_scale_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MLEADERSTYLE_block_scale_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->block_scale = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_block_scale_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_block_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_block_scale_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->block_scale);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_use_block_scale_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_use_block_scale_set(self,use_block_scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_use_block_scale_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADERSTYLE_use_block_scale_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->use_block_scale = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_use_block_scale_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_use_block_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_use_block_scale_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->use_block_scale);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_block_rotation_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_block_rotation_set(self,block_rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_block_rotation_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADERSTYLE_block_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->block_rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_block_rotation_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_block_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_block_rotation_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->block_rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_use_block_rotation_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_use_block_rotation_set(self,use_block_rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_use_block_rotation_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADERSTYLE_use_block_rotation_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->use_block_rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_use_block_rotation_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_use_block_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_use_block_rotation_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->use_block_rotation);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_block_connection_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_block_connection_set(self,block_connection);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_block_connection_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADERSTYLE_block_connection_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->block_connection = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_block_connection_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_block_connection_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_block_connection_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->block_connection);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_scale_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_scale_set(self,scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_scale_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADERSTYLE_scale_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->scale = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_scale_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_scale_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->scale);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_is_changed_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_is_changed_set(self,is_changed);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_is_changed_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADERSTYLE_is_changed_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_changed = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_is_changed_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_is_changed_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_is_changed_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->is_changed);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_is_annotative_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_is_annotative_set(self,is_annotative);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_is_annotative_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADERSTYLE_is_annotative_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_annotative = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_is_annotative_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_is_annotative_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_is_annotative_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->is_annotative);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_break_size_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_break_size_set(self,break_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_break_size_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADERSTYLE_break_size_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->break_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_break_size_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_break_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_break_size_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->break_size);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_attach_dir_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_attach_dir_set(self,attach_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_attach_dir_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADERSTYLE_attach_dir_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->attach_dir = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_attach_dir_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_attach_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_attach_dir_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->attach_dir);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_attach_top_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_attach_top_set(self,attach_top);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_attach_top_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADERSTYLE_attach_top_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->attach_top = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_attach_top_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_attach_top_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_attach_top_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->attach_top);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_attach_bottom_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_attach_bottom_set(self,attach_bottom);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_attach_bottom_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADERSTYLE_attach_bottom_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->attach_bottom = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_attach_bottom_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_attach_bottom_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_attach_bottom_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->attach_bottom);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_text_extended_set) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_text_extended_set(self,text_extended);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_text_extended_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADERSTYLE_text_extended_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->text_extended = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADERSTYLE_text_extended_get) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADERSTYLE_text_extended_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADERSTYLE_text_extended_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->text_extended);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_MLEADERSTYLE) {
  {
    int argvi = 0;
    struct _dwg_object_MLEADERSTYLE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_MLEADERSTYLE();");
    }
    result = (struct _dwg_object_MLEADERSTYLE *)calloc(1, sizeof(struct _dwg_object_MLEADERSTYLE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_MLEADERSTYLE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_MLEADERSTYLE) {
  {
    struct _dwg_object_MLEADERSTYLE *arg1 = (struct _dwg_object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_MLEADERSTYLE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADERSTYLE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_MLEADERSTYLE" "', argument " "1"" of type '" "struct _dwg_object_MLEADERSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADERSTYLE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VBA_PROJECT_parent_set) {
  {
    struct _dwg_object_VBA_PROJECT *arg1 = (struct _dwg_object_VBA_PROJECT *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VBA_PROJECT_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VBA_PROJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VBA_PROJECT_parent_set" "', argument " "1"" of type '" "struct _dwg_object_VBA_PROJECT *""'"); 
    }
    arg1 = (struct _dwg_object_VBA_PROJECT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VBA_PROJECT_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VBA_PROJECT_parent_get) {
  {
    struct _dwg_object_VBA_PROJECT *arg1 = (struct _dwg_object_VBA_PROJECT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VBA_PROJECT_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VBA_PROJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VBA_PROJECT_parent_get" "', argument " "1"" of type '" "struct _dwg_object_VBA_PROJECT *""'"); 
    }
    arg1 = (struct _dwg_object_VBA_PROJECT *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VBA_PROJECT_data_size_set) {
  {
    struct _dwg_object_VBA_PROJECT *arg1 = (struct _dwg_object_VBA_PROJECT *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VBA_PROJECT_data_size_set(self,data_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VBA_PROJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VBA_PROJECT_data_size_set" "', argument " "1"" of type '" "struct _dwg_object_VBA_PROJECT *""'"); 
    }
    arg1 = (struct _dwg_object_VBA_PROJECT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VBA_PROJECT_data_size_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->data_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VBA_PROJECT_data_size_get) {
  {
    struct _dwg_object_VBA_PROJECT *arg1 = (struct _dwg_object_VBA_PROJECT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VBA_PROJECT_data_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VBA_PROJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VBA_PROJECT_data_size_get" "', argument " "1"" of type '" "struct _dwg_object_VBA_PROJECT *""'"); 
    }
    arg1 = (struct _dwg_object_VBA_PROJECT *)(argp1);
    result = (BITCODE_BL) ((arg1)->data_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VBA_PROJECT_data_set) {
  {
    struct _dwg_object_VBA_PROJECT *arg1 = (struct _dwg_object_VBA_PROJECT *) 0 ;
    BITCODE_TF arg2 = (BITCODE_TF) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VBA_PROJECT_data_set(self,data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VBA_PROJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VBA_PROJECT_data_set" "', argument " "1"" of type '" "struct _dwg_object_VBA_PROJECT *""'"); 
    }
    arg1 = (struct _dwg_object_VBA_PROJECT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VBA_PROJECT_data_set" "', argument " "2"" of type '" "BITCODE_TF""'"); 
    }
    arg2 = (BITCODE_TF)(argp2);
    if (arg1) (arg1)->data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VBA_PROJECT_data_get) {
  {
    struct _dwg_object_VBA_PROJECT *arg1 = (struct _dwg_object_VBA_PROJECT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TF result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VBA_PROJECT_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VBA_PROJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VBA_PROJECT_data_get" "', argument " "1"" of type '" "struct _dwg_object_VBA_PROJECT *""'"); 
    }
    arg1 = (struct _dwg_object_VBA_PROJECT *)(argp1);
    result = (BITCODE_TF) ((arg1)->data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_VBA_PROJECT) {
  {
    int argvi = 0;
    struct _dwg_object_VBA_PROJECT *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_VBA_PROJECT();");
    }
    result = (struct _dwg_object_VBA_PROJECT *)calloc(1, sizeof(struct _dwg_object_VBA_PROJECT));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_VBA_PROJECT, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_VBA_PROJECT) {
  {
    struct _dwg_object_VBA_PROJECT *arg1 = (struct _dwg_object_VBA_PROJECT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_VBA_PROJECT(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VBA_PROJECT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_VBA_PROJECT" "', argument " "1"" of type '" "struct _dwg_object_VBA_PROJECT *""'"); 
    }
    arg1 = (struct _dwg_object_VBA_PROJECT *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_parent_set) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_parent_set" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_PLOTSETTINGS_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_parent_get) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_parent_get" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_printer_cfg_file_set) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_printer_cfg_file_set(self,printer_cfg_file);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_printer_cfg_file_set" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_PLOTSETTINGS_printer_cfg_file_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->printer_cfg_file);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->printer_cfg_file = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->printer_cfg_file = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_printer_cfg_file_get) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_printer_cfg_file_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_printer_cfg_file_get" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    result = (BITCODE_TV) ((arg1)->printer_cfg_file);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_paper_size_set) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_paper_size_set(self,paper_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_paper_size_set" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_PLOTSETTINGS_paper_size_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->paper_size);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->paper_size = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->paper_size = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_paper_size_get) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_paper_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_paper_size_get" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    result = (BITCODE_TV) ((arg1)->paper_size);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_canonical_media_name_set) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_canonical_media_name_set(self,canonical_media_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_canonical_media_name_set" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_PLOTSETTINGS_canonical_media_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->canonical_media_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->canonical_media_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->canonical_media_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_canonical_media_name_get) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_canonical_media_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_canonical_media_name_get" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    result = (BITCODE_TV) ((arg1)->canonical_media_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_plot_flags_set) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_plot_flags_set(self,plot_flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_plot_flags_set" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_PLOTSETTINGS_plot_flags_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->plot_flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_plot_flags_get) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_plot_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_plot_flags_get" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    result = (BITCODE_BS) ((arg1)->plot_flags);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_plotview_set) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_plotview_set(self,plotview);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_plotview_set" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_PLOTSETTINGS_plotview_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->plotview = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_plotview_get) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_plotview_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_plotview_get" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    result = (BITCODE_H) ((arg1)->plotview);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_plotview_name_set) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_plotview_name_set(self,plotview_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_plotview_name_set" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_PLOTSETTINGS_plotview_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->plotview_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->plotview_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->plotview_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_plotview_name_get) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_plotview_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_plotview_name_get" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    result = (BITCODE_TV) ((arg1)->plotview_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_left_margin_set) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_left_margin_set(self,left_margin);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_left_margin_set" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_PLOTSETTINGS_left_margin_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->left_margin = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_left_margin_get) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_left_margin_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_left_margin_get" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    result = (BITCODE_BD) ((arg1)->left_margin);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_bottom_margin_set) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_bottom_margin_set(self,bottom_margin);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_bottom_margin_set" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_PLOTSETTINGS_bottom_margin_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->bottom_margin = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_bottom_margin_get) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_bottom_margin_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_bottom_margin_get" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    result = (BITCODE_BD) ((arg1)->bottom_margin);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_right_margin_set) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_right_margin_set(self,right_margin);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_right_margin_set" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_PLOTSETTINGS_right_margin_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->right_margin = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_right_margin_get) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_right_margin_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_right_margin_get" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    result = (BITCODE_BD) ((arg1)->right_margin);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_top_margin_set) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_top_margin_set(self,top_margin);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_top_margin_set" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_PLOTSETTINGS_top_margin_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->top_margin = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_top_margin_get) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_top_margin_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_top_margin_get" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    result = (BITCODE_BD) ((arg1)->top_margin);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_paper_width_set) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_paper_width_set(self,paper_width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_paper_width_set" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_PLOTSETTINGS_paper_width_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->paper_width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_paper_width_get) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_paper_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_paper_width_get" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    result = (BITCODE_BD) ((arg1)->paper_width);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_paper_height_set) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_paper_height_set(self,paper_height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_paper_height_set" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_PLOTSETTINGS_paper_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->paper_height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_paper_height_get) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_paper_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_paper_height_get" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    result = (BITCODE_BD) ((arg1)->paper_height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_plot_origin_set) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    BITCODE_2BD_1 *arg2 = (BITCODE_2BD_1 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_plot_origin_set(self,plot_origin);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_plot_origin_set" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_PLOTSETTINGS_plot_origin_set" "', argument " "2"" of type '" "BITCODE_2BD_1 *""'"); 
    }
    arg2 = (BITCODE_2BD_1 *)(argp2);
    if (arg1) (arg1)->plot_origin = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_plot_origin_get) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2BD_1 *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_plot_origin_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_plot_origin_get" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    result = (BITCODE_2BD_1 *)& ((arg1)->plot_origin);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_plot_window_ll_set) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    BITCODE_2BD_1 *arg2 = (BITCODE_2BD_1 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_plot_window_ll_set(self,plot_window_ll);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_plot_window_ll_set" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_PLOTSETTINGS_plot_window_ll_set" "', argument " "2"" of type '" "BITCODE_2BD_1 *""'"); 
    }
    arg2 = (BITCODE_2BD_1 *)(argp2);
    if (arg1) (arg1)->plot_window_ll = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_plot_window_ll_get) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2BD_1 *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_plot_window_ll_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_plot_window_ll_get" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    result = (BITCODE_2BD_1 *)& ((arg1)->plot_window_ll);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_plot_window_ur_set) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    BITCODE_2BD_1 *arg2 = (BITCODE_2BD_1 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_plot_window_ur_set(self,plot_window_ur);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_plot_window_ur_set" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_PLOTSETTINGS_plot_window_ur_set" "', argument " "2"" of type '" "BITCODE_2BD_1 *""'"); 
    }
    arg2 = (BITCODE_2BD_1 *)(argp2);
    if (arg1) (arg1)->plot_window_ur = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_plot_window_ur_get) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2BD_1 *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_plot_window_ur_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_plot_window_ur_get" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    result = (BITCODE_2BD_1 *)& ((arg1)->plot_window_ur);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_plot_paper_unit_set) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_plot_paper_unit_set(self,plot_paper_unit);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_plot_paper_unit_set" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_PLOTSETTINGS_plot_paper_unit_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->plot_paper_unit = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_plot_paper_unit_get) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_plot_paper_unit_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_plot_paper_unit_get" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    result = (BITCODE_BS) ((arg1)->plot_paper_unit);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_plot_rotation_mode_set) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_plot_rotation_mode_set(self,plot_rotation_mode);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_plot_rotation_mode_set" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_PLOTSETTINGS_plot_rotation_mode_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->plot_rotation_mode = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_plot_rotation_mode_get) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_plot_rotation_mode_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_plot_rotation_mode_get" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    result = (BITCODE_BS) ((arg1)->plot_rotation_mode);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_plot_type_set) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_plot_type_set(self,plot_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_plot_type_set" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_PLOTSETTINGS_plot_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->plot_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_plot_type_get) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_plot_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_plot_type_get" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    result = (BITCODE_BS) ((arg1)->plot_type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_paper_units_set) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_paper_units_set(self,paper_units);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_paper_units_set" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_PLOTSETTINGS_paper_units_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->paper_units = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_paper_units_get) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_paper_units_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_paper_units_get" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    result = (BITCODE_BD) ((arg1)->paper_units);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_drawing_units_set) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_drawing_units_set(self,drawing_units);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_drawing_units_set" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_PLOTSETTINGS_drawing_units_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->drawing_units = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_drawing_units_get) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_drawing_units_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_drawing_units_get" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    result = (BITCODE_BD) ((arg1)->drawing_units);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_stylesheet_set) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_stylesheet_set(self,stylesheet);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_stylesheet_set" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_PLOTSETTINGS_stylesheet_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->stylesheet);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->stylesheet = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->stylesheet = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_stylesheet_get) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_stylesheet_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_stylesheet_get" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    result = (BITCODE_TV) ((arg1)->stylesheet);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_std_scale_type_set) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_std_scale_type_set(self,std_scale_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_std_scale_type_set" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_PLOTSETTINGS_std_scale_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->std_scale_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_std_scale_type_get) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_std_scale_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_std_scale_type_get" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    result = (BITCODE_BS) ((arg1)->std_scale_type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_std_scale_factor_set) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_std_scale_factor_set(self,std_scale_factor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_std_scale_factor_set" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_PLOTSETTINGS_std_scale_factor_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->std_scale_factor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_std_scale_factor_get) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_std_scale_factor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_std_scale_factor_get" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    result = (BITCODE_BD) ((arg1)->std_scale_factor);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_paper_image_origin_set) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    BITCODE_2BD_1 *arg2 = (BITCODE_2BD_1 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_paper_image_origin_set(self,paper_image_origin);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_paper_image_origin_set" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_PLOTSETTINGS_paper_image_origin_set" "', argument " "2"" of type '" "BITCODE_2BD_1 *""'"); 
    }
    arg2 = (BITCODE_2BD_1 *)(argp2);
    if (arg1) (arg1)->paper_image_origin = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_paper_image_origin_get) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2BD_1 *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_paper_image_origin_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_paper_image_origin_get" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    result = (BITCODE_2BD_1 *)& ((arg1)->paper_image_origin);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_shadeplot_type_set) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_shadeplot_type_set(self,shadeplot_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_shadeplot_type_set" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_PLOTSETTINGS_shadeplot_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->shadeplot_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_shadeplot_type_get) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_shadeplot_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_shadeplot_type_get" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    result = (BITCODE_BS) ((arg1)->shadeplot_type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_shadeplot_reslevel_set) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_shadeplot_reslevel_set(self,shadeplot_reslevel);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_shadeplot_reslevel_set" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_PLOTSETTINGS_shadeplot_reslevel_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->shadeplot_reslevel = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_shadeplot_reslevel_get) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_shadeplot_reslevel_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_shadeplot_reslevel_get" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    result = (BITCODE_BS) ((arg1)->shadeplot_reslevel);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_shadeplot_customdpi_set) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_shadeplot_customdpi_set(self,shadeplot_customdpi);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_shadeplot_customdpi_set" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_PLOTSETTINGS_shadeplot_customdpi_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->shadeplot_customdpi = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_shadeplot_customdpi_get) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_shadeplot_customdpi_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_shadeplot_customdpi_get" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    result = (BITCODE_BS) ((arg1)->shadeplot_customdpi);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_shadeplot_set) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_shadeplot_set(self,shadeplot);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_shadeplot_set" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_PLOTSETTINGS_shadeplot_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->shadeplot = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PLOTSETTINGS_shadeplot_get) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PLOTSETTINGS_shadeplot_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PLOTSETTINGS_shadeplot_get" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    result = (BITCODE_H) ((arg1)->shadeplot);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_PLOTSETTINGS) {
  {
    int argvi = 0;
    struct _dwg_object_PLOTSETTINGS *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_PLOTSETTINGS();");
    }
    result = (struct _dwg_object_PLOTSETTINGS *)calloc(1, sizeof(struct _dwg_object_PLOTSETTINGS));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_PLOTSETTINGS, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_PLOTSETTINGS) {
  {
    struct _dwg_object_PLOTSETTINGS *arg1 = (struct _dwg_object_PLOTSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_PLOTSETTINGS(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PLOTSETTINGS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_PLOTSETTINGS" "', argument " "1"" of type '" "struct _dwg_object_PLOTSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_PLOTSETTINGS *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_parent_set) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_parent_set" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LAYOUT_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_parent_get) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_parent_get" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_plotsettings_set) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    Dwg_Object_PLOTSETTINGS *arg2 = (Dwg_Object_PLOTSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_plotsettings_set(self,plotsettings);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_plotsettings_set" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LAYOUT_plotsettings_set" "', argument " "2"" of type '" "Dwg_Object_PLOTSETTINGS *""'"); 
    }
    arg2 = (Dwg_Object_PLOTSETTINGS *)(argp2);
    if (arg1) (arg1)->plotsettings = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_plotsettings_get) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_PLOTSETTINGS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_plotsettings_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_plotsettings_get" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    result = (Dwg_Object_PLOTSETTINGS *)& ((arg1)->plotsettings);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_layout_name_set) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_layout_name_set(self,layout_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_layout_name_set" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LAYOUT_layout_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->layout_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->layout_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->layout_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_layout_name_get) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_layout_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_layout_name_get" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    result = (BITCODE_TV) ((arg1)->layout_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_tab_order_set) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_tab_order_set(self,tab_order);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_tab_order_set" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LAYOUT_tab_order_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->tab_order = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_tab_order_get) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_tab_order_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_tab_order_get" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    result = (BITCODE_BS) ((arg1)->tab_order);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_layout_flags_set) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_layout_flags_set(self,layout_flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_layout_flags_set" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LAYOUT_layout_flags_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->layout_flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_layout_flags_get) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_layout_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_layout_flags_get" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    result = (BITCODE_BS) ((arg1)->layout_flags);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_INSBASE_set) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    BITCODE_3DPOINT *arg2 = (BITCODE_3DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_INSBASE_set(self,INSBASE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_INSBASE_set" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LAYOUT_INSBASE_set" "', argument " "2"" of type '" "BITCODE_3DPOINT *""'"); 
    }
    arg2 = (BITCODE_3DPOINT *)(argp2);
    if (arg1) (arg1)->INSBASE = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_INSBASE_get) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_INSBASE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_INSBASE_get" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    result = (BITCODE_3DPOINT *)& ((arg1)->INSBASE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_LIMMIN_set) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    BITCODE_2DPOINT *arg2 = (BITCODE_2DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_LIMMIN_set(self,LIMMIN);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_LIMMIN_set" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LAYOUT_LIMMIN_set" "', argument " "2"" of type '" "BITCODE_2DPOINT *""'"); 
    }
    arg2 = (BITCODE_2DPOINT *)(argp2);
    if (arg1) (arg1)->LIMMIN = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_LIMMIN_get) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_LIMMIN_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_LIMMIN_get" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    result = (BITCODE_2DPOINT *)& ((arg1)->LIMMIN);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_LIMMAX_set) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    BITCODE_2DPOINT *arg2 = (BITCODE_2DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_LIMMAX_set(self,LIMMAX);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_LIMMAX_set" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LAYOUT_LIMMAX_set" "', argument " "2"" of type '" "BITCODE_2DPOINT *""'"); 
    }
    arg2 = (BITCODE_2DPOINT *)(argp2);
    if (arg1) (arg1)->LIMMAX = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_LIMMAX_get) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_LIMMAX_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_LIMMAX_get" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    result = (BITCODE_2DPOINT *)& ((arg1)->LIMMAX);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_UCSORG_set) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    BITCODE_3DPOINT *arg2 = (BITCODE_3DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_UCSORG_set(self,UCSORG);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_UCSORG_set" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LAYOUT_UCSORG_set" "', argument " "2"" of type '" "BITCODE_3DPOINT *""'"); 
    }
    arg2 = (BITCODE_3DPOINT *)(argp2);
    if (arg1) (arg1)->UCSORG = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_UCSORG_get) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_UCSORG_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_UCSORG_get" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    result = (BITCODE_3DPOINT *)& ((arg1)->UCSORG);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_UCSXDIR_set) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    BITCODE_3DPOINT *arg2 = (BITCODE_3DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_UCSXDIR_set(self,UCSXDIR);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_UCSXDIR_set" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LAYOUT_UCSXDIR_set" "', argument " "2"" of type '" "BITCODE_3DPOINT *""'"); 
    }
    arg2 = (BITCODE_3DPOINT *)(argp2);
    if (arg1) (arg1)->UCSXDIR = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_UCSXDIR_get) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_UCSXDIR_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_UCSXDIR_get" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    result = (BITCODE_3DPOINT *)& ((arg1)->UCSXDIR);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_UCSYDIR_set) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    BITCODE_3DPOINT *arg2 = (BITCODE_3DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_UCSYDIR_set(self,UCSYDIR);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_UCSYDIR_set" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LAYOUT_UCSYDIR_set" "', argument " "2"" of type '" "BITCODE_3DPOINT *""'"); 
    }
    arg2 = (BITCODE_3DPOINT *)(argp2);
    if (arg1) (arg1)->UCSYDIR = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_UCSYDIR_get) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_UCSYDIR_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_UCSYDIR_get" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    result = (BITCODE_3DPOINT *)& ((arg1)->UCSYDIR);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_ucs_elevation_set) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_ucs_elevation_set(self,ucs_elevation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_ucs_elevation_set" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LAYOUT_ucs_elevation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->ucs_elevation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_ucs_elevation_get) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_ucs_elevation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_ucs_elevation_get" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    result = (BITCODE_BD) ((arg1)->ucs_elevation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_UCSORTHOVIEW_set) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_UCSORTHOVIEW_set(self,UCSORTHOVIEW);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_UCSORTHOVIEW_set" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LAYOUT_UCSORTHOVIEW_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->UCSORTHOVIEW = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_UCSORTHOVIEW_get) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_UCSORTHOVIEW_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_UCSORTHOVIEW_get" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    result = (BITCODE_BS) ((arg1)->UCSORTHOVIEW);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_EXTMIN_set) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    BITCODE_3DPOINT *arg2 = (BITCODE_3DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_EXTMIN_set(self,EXTMIN);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_EXTMIN_set" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LAYOUT_EXTMIN_set" "', argument " "2"" of type '" "BITCODE_3DPOINT *""'"); 
    }
    arg2 = (BITCODE_3DPOINT *)(argp2);
    if (arg1) (arg1)->EXTMIN = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_EXTMIN_get) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_EXTMIN_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_EXTMIN_get" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    result = (BITCODE_3DPOINT *)& ((arg1)->EXTMIN);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_EXTMAX_set) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    BITCODE_3DPOINT *arg2 = (BITCODE_3DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_EXTMAX_set(self,EXTMAX);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_EXTMAX_set" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LAYOUT_EXTMAX_set" "', argument " "2"" of type '" "BITCODE_3DPOINT *""'"); 
    }
    arg2 = (BITCODE_3DPOINT *)(argp2);
    if (arg1) (arg1)->EXTMAX = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_EXTMAX_get) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_EXTMAX_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_EXTMAX_get" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    result = (BITCODE_3DPOINT *)& ((arg1)->EXTMAX);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_block_header_set) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_block_header_set(self,block_header);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_block_header_set" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LAYOUT_block_header_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->block_header = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_block_header_get) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_block_header_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_block_header_get" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    result = (BITCODE_H) ((arg1)->block_header);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_active_viewport_set) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_active_viewport_set(self,active_viewport);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_active_viewport_set" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LAYOUT_active_viewport_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->active_viewport = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_active_viewport_get) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_active_viewport_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_active_viewport_get" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    result = (BITCODE_H) ((arg1)->active_viewport);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_base_ucs_set) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_base_ucs_set(self,base_ucs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_base_ucs_set" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LAYOUT_base_ucs_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->base_ucs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_base_ucs_get) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_base_ucs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_base_ucs_get" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    result = (BITCODE_H) ((arg1)->base_ucs);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_named_ucs_set) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_named_ucs_set(self,named_ucs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_named_ucs_set" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LAYOUT_named_ucs_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->named_ucs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_named_ucs_get) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_named_ucs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_named_ucs_get" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    result = (BITCODE_H) ((arg1)->named_ucs);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_num_viewports_set) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_num_viewports_set(self,num_viewports);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_num_viewports_set" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LAYOUT_num_viewports_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_viewports = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_num_viewports_get) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_num_viewports_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_num_viewports_get" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_viewports);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_viewports_set) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_viewports_set(self,viewports);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_viewports_set" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LAYOUT_viewports_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->viewports = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYOUT_viewports_get) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYOUT_viewports_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYOUT_viewports_get" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    result = (BITCODE_H *) ((arg1)->viewports);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_LAYOUT) {
  {
    int argvi = 0;
    struct _dwg_object_LAYOUT *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_LAYOUT();");
    }
    result = (struct _dwg_object_LAYOUT *)calloc(1, sizeof(struct _dwg_object_LAYOUT));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_LAYOUT, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_LAYOUT) {
  {
    struct _dwg_object_LAYOUT *arg1 = (struct _dwg_object_LAYOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_LAYOUT(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYOUT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_LAYOUT" "', argument " "1"" of type '" "struct _dwg_object_LAYOUT *""'"); 
    }
    arg1 = (struct _dwg_object_LAYOUT *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DICTIONARYVAR_parent_set) {
  {
    struct _dwg_object_DICTIONARYVAR *arg1 = (struct _dwg_object_DICTIONARYVAR *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DICTIONARYVAR_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DICTIONARYVAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DICTIONARYVAR_parent_set" "', argument " "1"" of type '" "struct _dwg_object_DICTIONARYVAR *""'"); 
    }
    arg1 = (struct _dwg_object_DICTIONARYVAR *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DICTIONARYVAR_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DICTIONARYVAR_parent_get) {
  {
    struct _dwg_object_DICTIONARYVAR *arg1 = (struct _dwg_object_DICTIONARYVAR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DICTIONARYVAR_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DICTIONARYVAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DICTIONARYVAR_parent_get" "', argument " "1"" of type '" "struct _dwg_object_DICTIONARYVAR *""'"); 
    }
    arg1 = (struct _dwg_object_DICTIONARYVAR *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DICTIONARYVAR_schema_set) {
  {
    struct _dwg_object_DICTIONARYVAR *arg1 = (struct _dwg_object_DICTIONARYVAR *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DICTIONARYVAR_schema_set(self,schema);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DICTIONARYVAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DICTIONARYVAR_schema_set" "', argument " "1"" of type '" "struct _dwg_object_DICTIONARYVAR *""'"); 
    }
    arg1 = (struct _dwg_object_DICTIONARYVAR *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DICTIONARYVAR_schema_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->schema = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DICTIONARYVAR_schema_get) {
  {
    struct _dwg_object_DICTIONARYVAR *arg1 = (struct _dwg_object_DICTIONARYVAR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DICTIONARYVAR_schema_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DICTIONARYVAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DICTIONARYVAR_schema_get" "', argument " "1"" of type '" "struct _dwg_object_DICTIONARYVAR *""'"); 
    }
    arg1 = (struct _dwg_object_DICTIONARYVAR *)(argp1);
    result = (BITCODE_RC) ((arg1)->schema);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DICTIONARYVAR_strvalue_set) {
  {
    struct _dwg_object_DICTIONARYVAR *arg1 = (struct _dwg_object_DICTIONARYVAR *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DICTIONARYVAR_strvalue_set(self,strvalue);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DICTIONARYVAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DICTIONARYVAR_strvalue_set" "', argument " "1"" of type '" "struct _dwg_object_DICTIONARYVAR *""'"); 
    }
    arg1 = (struct _dwg_object_DICTIONARYVAR *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DICTIONARYVAR_strvalue_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->strvalue);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->strvalue = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->strvalue = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DICTIONARYVAR_strvalue_get) {
  {
    struct _dwg_object_DICTIONARYVAR *arg1 = (struct _dwg_object_DICTIONARYVAR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DICTIONARYVAR_strvalue_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DICTIONARYVAR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DICTIONARYVAR_strvalue_get" "', argument " "1"" of type '" "struct _dwg_object_DICTIONARYVAR *""'"); 
    }
    arg1 = (struct _dwg_object_DICTIONARYVAR *)(argp1);
    result = (BITCODE_TV) ((arg1)->strvalue);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_DICTIONARYVAR) {
  {
    int argvi = 0;
    struct _dwg_object_DICTIONARYVAR *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_DICTIONARYVAR();");
    }
    result = (struct _dwg_object_DICTIONARYVAR *)calloc(1, sizeof(struct _dwg_object_DICTIONARYVAR));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DICTIONARYVAR, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_DICTIONARYVAR) {
  {
    struct _dwg_object_DICTIONARYVAR *arg1 = (struct _dwg_object_DICTIONARYVAR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_DICTIONARYVAR(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DICTIONARYVAR, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_DICTIONARYVAR" "', argument " "1"" of type '" "struct _dwg_object_DICTIONARYVAR *""'"); 
    }
    arg1 = (struct _dwg_object_DICTIONARYVAR *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_value_flags_set) {
  {
    struct _dwg_TABLE_value *arg1 = (struct _dwg_TABLE_value *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_value_flags_set(self,flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_value_flags_set" "', argument " "1"" of type '" "struct _dwg_TABLE_value *""'"); 
    }
    arg1 = (struct _dwg_TABLE_value *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_value_flags_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_value_flags_get) {
  {
    struct _dwg_TABLE_value *arg1 = (struct _dwg_TABLE_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_value_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_value_flags_get" "', argument " "1"" of type '" "struct _dwg_TABLE_value *""'"); 
    }
    arg1 = (struct _dwg_TABLE_value *)(argp1);
    result = (BITCODE_BL) ((arg1)->flags);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_value_format_flags_set) {
  {
    struct _dwg_TABLE_value *arg1 = (struct _dwg_TABLE_value *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_value_format_flags_set(self,format_flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_value_format_flags_set" "', argument " "1"" of type '" "struct _dwg_TABLE_value *""'"); 
    }
    arg1 = (struct _dwg_TABLE_value *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_value_format_flags_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->format_flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_value_format_flags_get) {
  {
    struct _dwg_TABLE_value *arg1 = (struct _dwg_TABLE_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_value_format_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_value_format_flags_get" "', argument " "1"" of type '" "struct _dwg_TABLE_value *""'"); 
    }
    arg1 = (struct _dwg_TABLE_value *)(argp1);
    result = (BITCODE_BL) ((arg1)->format_flags);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_value_data_type_set) {
  {
    struct _dwg_TABLE_value *arg1 = (struct _dwg_TABLE_value *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_value_data_type_set(self,data_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_value_data_type_set" "', argument " "1"" of type '" "struct _dwg_TABLE_value *""'"); 
    }
    arg1 = (struct _dwg_TABLE_value *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_value_data_type_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->data_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_value_data_type_get) {
  {
    struct _dwg_TABLE_value *arg1 = (struct _dwg_TABLE_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_value_data_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_value_data_type_get" "', argument " "1"" of type '" "struct _dwg_TABLE_value *""'"); 
    }
    arg1 = (struct _dwg_TABLE_value *)(argp1);
    result = (BITCODE_BL) ((arg1)->data_type);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_value_data_size_set) {
  {
    struct _dwg_TABLE_value *arg1 = (struct _dwg_TABLE_value *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_value_data_size_set(self,data_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_value_data_size_set" "', argument " "1"" of type '" "struct _dwg_TABLE_value *""'"); 
    }
    arg1 = (struct _dwg_TABLE_value *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_value_data_size_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->data_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_value_data_size_get) {
  {
    struct _dwg_TABLE_value *arg1 = (struct _dwg_TABLE_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_value_data_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_value_data_size_get" "', argument " "1"" of type '" "struct _dwg_TABLE_value *""'"); 
    }
    arg1 = (struct _dwg_TABLE_value *)(argp1);
    result = (BITCODE_BL) ((arg1)->data_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_value_data_long_set) {
  {
    struct _dwg_TABLE_value *arg1 = (struct _dwg_TABLE_value *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_value_data_long_set(self,data_long);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_value_data_long_set" "', argument " "1"" of type '" "struct _dwg_TABLE_value *""'"); 
    }
    arg1 = (struct _dwg_TABLE_value *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_value_data_long_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->data_long = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_value_data_long_get) {
  {
    struct _dwg_TABLE_value *arg1 = (struct _dwg_TABLE_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_value_data_long_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_value_data_long_get" "', argument " "1"" of type '" "struct _dwg_TABLE_value *""'"); 
    }
    arg1 = (struct _dwg_TABLE_value *)(argp1);
    result = (BITCODE_BL) ((arg1)->data_long);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_value_data_double_set) {
  {
    struct _dwg_TABLE_value *arg1 = (struct _dwg_TABLE_value *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_value_data_double_set(self,data_double);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_value_data_double_set" "', argument " "1"" of type '" "struct _dwg_TABLE_value *""'"); 
    }
    arg1 = (struct _dwg_TABLE_value *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_value_data_double_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->data_double = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_value_data_double_get) {
  {
    struct _dwg_TABLE_value *arg1 = (struct _dwg_TABLE_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_value_data_double_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_value_data_double_get" "', argument " "1"" of type '" "struct _dwg_TABLE_value *""'"); 
    }
    arg1 = (struct _dwg_TABLE_value *)(argp1);
    result = (BITCODE_BD) ((arg1)->data_double);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_value_data_string_set) {
  {
    struct _dwg_TABLE_value *arg1 = (struct _dwg_TABLE_value *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_value_data_string_set(self,data_string);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_value_data_string_set" "', argument " "1"" of type '" "struct _dwg_TABLE_value *""'"); 
    }
    arg1 = (struct _dwg_TABLE_value *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLE_value_data_string_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->data_string);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->data_string = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->data_string = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_value_data_string_get) {
  {
    struct _dwg_TABLE_value *arg1 = (struct _dwg_TABLE_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_value_data_string_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_value_data_string_get" "', argument " "1"" of type '" "struct _dwg_TABLE_value *""'"); 
    }
    arg1 = (struct _dwg_TABLE_value *)(argp1);
    result = (BITCODE_TV) ((arg1)->data_string);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_value_data_date_set) {
  {
    struct _dwg_TABLE_value *arg1 = (struct _dwg_TABLE_value *) 0 ;
    BITCODE_TF arg2 = (BITCODE_TF) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_value_data_date_set(self,data_date);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_value_data_date_set" "', argument " "1"" of type '" "struct _dwg_TABLE_value *""'"); 
    }
    arg1 = (struct _dwg_TABLE_value *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLE_value_data_date_set" "', argument " "2"" of type '" "BITCODE_TF""'"); 
    }
    arg2 = (BITCODE_TF)(argp2);
    if (arg1) (arg1)->data_date = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_value_data_date_get) {
  {
    struct _dwg_TABLE_value *arg1 = (struct _dwg_TABLE_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TF result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_value_data_date_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_value_data_date_get" "', argument " "1"" of type '" "struct _dwg_TABLE_value *""'"); 
    }
    arg1 = (struct _dwg_TABLE_value *)(argp1);
    result = (BITCODE_TF) ((arg1)->data_date);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_value_data_point_set) {
  {
    struct _dwg_TABLE_value *arg1 = (struct _dwg_TABLE_value *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_value_data_point_set(self,data_point);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_value_data_point_set" "', argument " "1"" of type '" "struct _dwg_TABLE_value *""'"); 
    }
    arg1 = (struct _dwg_TABLE_value *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLE_value_data_point_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->data_point = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_value_data_point_get) {
  {
    struct _dwg_TABLE_value *arg1 = (struct _dwg_TABLE_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_value_data_point_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_value_data_point_get" "', argument " "1"" of type '" "struct _dwg_TABLE_value *""'"); 
    }
    arg1 = (struct _dwg_TABLE_value *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->data_point);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_value_data_3dpoint_set) {
  {
    struct _dwg_TABLE_value *arg1 = (struct _dwg_TABLE_value *) 0 ;
    BITCODE_3RD *arg2 = (BITCODE_3RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_value_data_3dpoint_set(self,data_3dpoint);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_value_data_3dpoint_set" "', argument " "1"" of type '" "struct _dwg_TABLE_value *""'"); 
    }
    arg1 = (struct _dwg_TABLE_value *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLE_value_data_3dpoint_set" "', argument " "2"" of type '" "BITCODE_3RD *""'"); 
    }
    arg2 = (BITCODE_3RD *)(argp2);
    if (arg1) (arg1)->data_3dpoint = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_value_data_3dpoint_get) {
  {
    struct _dwg_TABLE_value *arg1 = (struct _dwg_TABLE_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_value_data_3dpoint_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_value_data_3dpoint_get" "', argument " "1"" of type '" "struct _dwg_TABLE_value *""'"); 
    }
    arg1 = (struct _dwg_TABLE_value *)(argp1);
    result = (BITCODE_3RD *)& ((arg1)->data_3dpoint);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_value_data_handle_set) {
  {
    struct _dwg_TABLE_value *arg1 = (struct _dwg_TABLE_value *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_value_data_handle_set(self,data_handle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_value_data_handle_set" "', argument " "1"" of type '" "struct _dwg_TABLE_value *""'"); 
    }
    arg1 = (struct _dwg_TABLE_value *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLE_value_data_handle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->data_handle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_value_data_handle_get) {
  {
    struct _dwg_TABLE_value *arg1 = (struct _dwg_TABLE_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_value_data_handle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_value_data_handle_get" "', argument " "1"" of type '" "struct _dwg_TABLE_value *""'"); 
    }
    arg1 = (struct _dwg_TABLE_value *)(argp1);
    result = (BITCODE_H) ((arg1)->data_handle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_value_unit_type_set) {
  {
    struct _dwg_TABLE_value *arg1 = (struct _dwg_TABLE_value *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_value_unit_type_set(self,unit_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_value_unit_type_set" "', argument " "1"" of type '" "struct _dwg_TABLE_value *""'"); 
    }
    arg1 = (struct _dwg_TABLE_value *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_value_unit_type_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unit_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_value_unit_type_get) {
  {
    struct _dwg_TABLE_value *arg1 = (struct _dwg_TABLE_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_value_unit_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_value_unit_type_get" "', argument " "1"" of type '" "struct _dwg_TABLE_value *""'"); 
    }
    arg1 = (struct _dwg_TABLE_value *)(argp1);
    result = (BITCODE_BL) ((arg1)->unit_type);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_value_format_string_set) {
  {
    struct _dwg_TABLE_value *arg1 = (struct _dwg_TABLE_value *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_value_format_string_set(self,format_string);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_value_format_string_set" "', argument " "1"" of type '" "struct _dwg_TABLE_value *""'"); 
    }
    arg1 = (struct _dwg_TABLE_value *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLE_value_format_string_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->format_string);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->format_string = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->format_string = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_value_format_string_get) {
  {
    struct _dwg_TABLE_value *arg1 = (struct _dwg_TABLE_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_value_format_string_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_value_format_string_get" "', argument " "1"" of type '" "struct _dwg_TABLE_value *""'"); 
    }
    arg1 = (struct _dwg_TABLE_value *)(argp1);
    result = (BITCODE_TV) ((arg1)->format_string);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_value_value_string_set) {
  {
    struct _dwg_TABLE_value *arg1 = (struct _dwg_TABLE_value *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_value_value_string_set(self,value_string);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_value_value_string_set" "', argument " "1"" of type '" "struct _dwg_TABLE_value *""'"); 
    }
    arg1 = (struct _dwg_TABLE_value *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLE_value_value_string_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->value_string);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->value_string = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->value_string = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_value_value_string_get) {
  {
    struct _dwg_TABLE_value *arg1 = (struct _dwg_TABLE_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_value_value_string_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_value_value_string_get" "', argument " "1"" of type '" "struct _dwg_TABLE_value *""'"); 
    }
    arg1 = (struct _dwg_TABLE_value *)(argp1);
    result = (BITCODE_TV) ((arg1)->value_string);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_TABLE_value) {
  {
    int argvi = 0;
    struct _dwg_TABLE_value *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_TABLE_value();");
    }
    result = (struct _dwg_TABLE_value *)calloc(1, sizeof(struct _dwg_TABLE_value));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TABLE_value, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_TABLE_value) {
  {
    struct _dwg_TABLE_value *arg1 = (struct _dwg_TABLE_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_TABLE_value(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_value, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_TABLE_value" "', argument " "1"" of type '" "struct _dwg_TABLE_value *""'"); 
    }
    arg1 = (struct _dwg_TABLE_value *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_CustomDataItem_name_set) {
  {
    struct _dwg_TABLE_CustomDataItem *arg1 = (struct _dwg_TABLE_CustomDataItem *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_CustomDataItem_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_CustomDataItem, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_CustomDataItem_name_set" "', argument " "1"" of type '" "struct _dwg_TABLE_CustomDataItem *""'"); 
    }
    arg1 = (struct _dwg_TABLE_CustomDataItem *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLE_CustomDataItem_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_CustomDataItem_name_get) {
  {
    struct _dwg_TABLE_CustomDataItem *arg1 = (struct _dwg_TABLE_CustomDataItem *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_CustomDataItem_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_CustomDataItem, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_CustomDataItem_name_get" "', argument " "1"" of type '" "struct _dwg_TABLE_CustomDataItem *""'"); 
    }
    arg1 = (struct _dwg_TABLE_CustomDataItem *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_CustomDataItem_value_set) {
  {
    struct _dwg_TABLE_CustomDataItem *arg1 = (struct _dwg_TABLE_CustomDataItem *) 0 ;
    Dwg_TABLE_value *arg2 = (Dwg_TABLE_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_CustomDataItem_value_set(self,value);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_CustomDataItem, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_CustomDataItem_value_set" "', argument " "1"" of type '" "struct _dwg_TABLE_CustomDataItem *""'"); 
    }
    arg1 = (struct _dwg_TABLE_CustomDataItem *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_TABLE_value, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLE_CustomDataItem_value_set" "', argument " "2"" of type '" "Dwg_TABLE_value *""'"); 
    }
    arg2 = (Dwg_TABLE_value *)(argp2);
    if (arg1) (arg1)->value = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_CustomDataItem_value_get) {
  {
    struct _dwg_TABLE_CustomDataItem *arg1 = (struct _dwg_TABLE_CustomDataItem *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_TABLE_value *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_CustomDataItem_value_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_CustomDataItem, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_CustomDataItem_value_get" "', argument " "1"" of type '" "struct _dwg_TABLE_CustomDataItem *""'"); 
    }
    arg1 = (struct _dwg_TABLE_CustomDataItem *)(argp1);
    result = (Dwg_TABLE_value *)& ((arg1)->value);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TABLE_value, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_CustomDataItem_cell_parent_set) {
  {
    struct _dwg_TABLE_CustomDataItem *arg1 = (struct _dwg_TABLE_CustomDataItem *) 0 ;
    struct _dwg_TableCell *arg2 = (struct _dwg_TableCell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_CustomDataItem_cell_parent_set(self,cell_parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_CustomDataItem, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_CustomDataItem_cell_parent_set" "', argument " "1"" of type '" "struct _dwg_TABLE_CustomDataItem *""'"); 
    }
    arg1 = (struct _dwg_TABLE_CustomDataItem *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_TableCell, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLE_CustomDataItem_cell_parent_set" "', argument " "2"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg2 = (struct _dwg_TableCell *)(argp2);
    if (arg1) (arg1)->cell_parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_CustomDataItem_cell_parent_get) {
  {
    struct _dwg_TABLE_CustomDataItem *arg1 = (struct _dwg_TABLE_CustomDataItem *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_TableCell *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_CustomDataItem_cell_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_CustomDataItem, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_CustomDataItem_cell_parent_get" "', argument " "1"" of type '" "struct _dwg_TABLE_CustomDataItem *""'"); 
    }
    arg1 = (struct _dwg_TABLE_CustomDataItem *)(argp1);
    result = (struct _dwg_TableCell *) ((arg1)->cell_parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TableCell, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_CustomDataItem_row_parent_set) {
  {
    struct _dwg_TABLE_CustomDataItem *arg1 = (struct _dwg_TABLE_CustomDataItem *) 0 ;
    struct _dwg_TableRow *arg2 = (struct _dwg_TableRow *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_CustomDataItem_row_parent_set(self,row_parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_CustomDataItem, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_CustomDataItem_row_parent_set" "', argument " "1"" of type '" "struct _dwg_TABLE_CustomDataItem *""'"); 
    }
    arg1 = (struct _dwg_TABLE_CustomDataItem *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_TableRow, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLE_CustomDataItem_row_parent_set" "', argument " "2"" of type '" "struct _dwg_TableRow *""'"); 
    }
    arg2 = (struct _dwg_TableRow *)(argp2);
    if (arg1) (arg1)->row_parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_CustomDataItem_row_parent_get) {
  {
    struct _dwg_TABLE_CustomDataItem *arg1 = (struct _dwg_TABLE_CustomDataItem *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_TableRow *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_CustomDataItem_row_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_CustomDataItem, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_CustomDataItem_row_parent_get" "', argument " "1"" of type '" "struct _dwg_TABLE_CustomDataItem *""'"); 
    }
    arg1 = (struct _dwg_TABLE_CustomDataItem *)(argp1);
    result = (struct _dwg_TableRow *) ((arg1)->row_parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TableRow, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_TABLE_CustomDataItem) {
  {
    int argvi = 0;
    struct _dwg_TABLE_CustomDataItem *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_TABLE_CustomDataItem();");
    }
    result = (struct _dwg_TABLE_CustomDataItem *)calloc(1, sizeof(struct _dwg_TABLE_CustomDataItem));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TABLE_CustomDataItem, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_TABLE_CustomDataItem) {
  {
    struct _dwg_TABLE_CustomDataItem *arg1 = (struct _dwg_TABLE_CustomDataItem *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_TABLE_CustomDataItem(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_CustomDataItem, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_TABLE_CustomDataItem" "', argument " "1"" of type '" "struct _dwg_TABLE_CustomDataItem *""'"); 
    }
    arg1 = (struct _dwg_TABLE_CustomDataItem *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_AttrDef_parent_set) {
  {
    struct _dwg_TABLE_AttrDef *arg1 = (struct _dwg_TABLE_AttrDef *) 0 ;
    struct _dwg_TABLE_Cell *arg2 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_AttrDef_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_AttrDef, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_AttrDef_parent_set" "', argument " "1"" of type '" "struct _dwg_TABLE_AttrDef *""'"); 
    }
    arg1 = (struct _dwg_TABLE_AttrDef *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_TABLE_Cell, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLE_AttrDef_parent_set" "', argument " "2"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg2 = (struct _dwg_TABLE_Cell *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_AttrDef_parent_get) {
  {
    struct _dwg_TABLE_AttrDef *arg1 = (struct _dwg_TABLE_AttrDef *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_TABLE_Cell *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_AttrDef_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_AttrDef, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_AttrDef_parent_get" "', argument " "1"" of type '" "struct _dwg_TABLE_AttrDef *""'"); 
    }
    arg1 = (struct _dwg_TABLE_AttrDef *)(argp1);
    result = (struct _dwg_TABLE_Cell *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TABLE_Cell, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_AttrDef_attdef_set) {
  {
    struct _dwg_TABLE_AttrDef *arg1 = (struct _dwg_TABLE_AttrDef *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_AttrDef_attdef_set(self,attdef);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_AttrDef, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_AttrDef_attdef_set" "', argument " "1"" of type '" "struct _dwg_TABLE_AttrDef *""'"); 
    }
    arg1 = (struct _dwg_TABLE_AttrDef *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLE_AttrDef_attdef_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->attdef = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_AttrDef_attdef_get) {
  {
    struct _dwg_TABLE_AttrDef *arg1 = (struct _dwg_TABLE_AttrDef *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_AttrDef_attdef_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_AttrDef, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_AttrDef_attdef_get" "', argument " "1"" of type '" "struct _dwg_TABLE_AttrDef *""'"); 
    }
    arg1 = (struct _dwg_TABLE_AttrDef *)(argp1);
    result = (BITCODE_H) ((arg1)->attdef);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_AttrDef_index_set) {
  {
    struct _dwg_TABLE_AttrDef *arg1 = (struct _dwg_TABLE_AttrDef *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_AttrDef_index_set(self,index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_AttrDef, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_AttrDef_index_set" "', argument " "1"" of type '" "struct _dwg_TABLE_AttrDef *""'"); 
    }
    arg1 = (struct _dwg_TABLE_AttrDef *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_AttrDef_index_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_AttrDef_index_get) {
  {
    struct _dwg_TABLE_AttrDef *arg1 = (struct _dwg_TABLE_AttrDef *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_AttrDef_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_AttrDef, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_AttrDef_index_get" "', argument " "1"" of type '" "struct _dwg_TABLE_AttrDef *""'"); 
    }
    arg1 = (struct _dwg_TABLE_AttrDef *)(argp1);
    result = (BITCODE_BS) ((arg1)->index);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_AttrDef_text_set) {
  {
    struct _dwg_TABLE_AttrDef *arg1 = (struct _dwg_TABLE_AttrDef *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_AttrDef_text_set(self,text);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_AttrDef, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_AttrDef_text_set" "', argument " "1"" of type '" "struct _dwg_TABLE_AttrDef *""'"); 
    }
    arg1 = (struct _dwg_TABLE_AttrDef *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLE_AttrDef_text_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->text);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->text = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->text = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_AttrDef_text_get) {
  {
    struct _dwg_TABLE_AttrDef *arg1 = (struct _dwg_TABLE_AttrDef *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_AttrDef_text_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_AttrDef, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_AttrDef_text_get" "', argument " "1"" of type '" "struct _dwg_TABLE_AttrDef *""'"); 
    }
    arg1 = (struct _dwg_TABLE_AttrDef *)(argp1);
    result = (BITCODE_TV) ((arg1)->text);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_TABLE_AttrDef) {
  {
    int argvi = 0;
    struct _dwg_TABLE_AttrDef *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_TABLE_AttrDef();");
    }
    result = (struct _dwg_TABLE_AttrDef *)calloc(1, sizeof(struct _dwg_TABLE_AttrDef));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TABLE_AttrDef, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_TABLE_AttrDef) {
  {
    struct _dwg_TABLE_AttrDef *arg1 = (struct _dwg_TABLE_AttrDef *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_TABLE_AttrDef(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_AttrDef, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_TABLE_AttrDef" "', argument " "1"" of type '" "struct _dwg_TABLE_AttrDef *""'"); 
    }
    arg1 = (struct _dwg_TABLE_AttrDef *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_parent_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    struct _dwg_entity_TABLE *arg2 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_parent_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_TABLE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLE_Cell_parent_set" "', argument " "2"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg2 = (struct _dwg_entity_TABLE *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_parent_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_entity_TABLE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_parent_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (struct _dwg_entity_TABLE *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_TABLE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_type_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_type_set(self,type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_type_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_Cell_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_type_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_type_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (BITCODE_BS) ((arg1)->type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_flags_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_flags_set(self,flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_flags_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_Cell_flags_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_flags_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_flags_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (BITCODE_RC) ((arg1)->flags);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_is_merged_value_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_is_merged_value_set(self,is_merged_value);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_is_merged_value_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_Cell_is_merged_value_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_merged_value = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_is_merged_value_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_is_merged_value_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_is_merged_value_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (BITCODE_B) ((arg1)->is_merged_value);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_is_autofit_flag_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_is_autofit_flag_set(self,is_autofit_flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_is_autofit_flag_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_Cell_is_autofit_flag_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_autofit_flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_is_autofit_flag_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_is_autofit_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_is_autofit_flag_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (BITCODE_B) ((arg1)->is_autofit_flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_merged_width_flag_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_merged_width_flag_set(self,merged_width_flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_merged_width_flag_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_Cell_merged_width_flag_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->merged_width_flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_merged_width_flag_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_merged_width_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_merged_width_flag_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (BITCODE_BL) ((arg1)->merged_width_flag);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_merged_height_flag_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_merged_height_flag_set(self,merged_height_flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_merged_height_flag_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_Cell_merged_height_flag_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->merged_height_flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_merged_height_flag_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_merged_height_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_merged_height_flag_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (BITCODE_BL) ((arg1)->merged_height_flag);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_rotation_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_rotation_set(self,rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_rotation_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_Cell_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_rotation_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_rotation_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (BITCODE_BD) ((arg1)->rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_text_value_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_text_value_set(self,text_value);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_text_value_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLE_Cell_text_value_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->text_value);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->text_value = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->text_value = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_text_value_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_text_value_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_text_value_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (BITCODE_TV) ((arg1)->text_value);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_text_style_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_text_style_set(self,text_style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_text_style_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLE_Cell_text_style_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->text_style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_text_style_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_text_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_text_style_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (BITCODE_H) ((arg1)->text_style);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_block_handle_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_block_handle_set(self,block_handle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_block_handle_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLE_Cell_block_handle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->block_handle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_block_handle_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_block_handle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_block_handle_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (BITCODE_H) ((arg1)->block_handle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_block_scale_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_block_scale_set(self,block_scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_block_scale_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_Cell_block_scale_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->block_scale = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_block_scale_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_block_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_block_scale_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (BITCODE_BD) ((arg1)->block_scale);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_additional_data_flag_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_additional_data_flag_set(self,additional_data_flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_additional_data_flag_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_Cell_additional_data_flag_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->additional_data_flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_additional_data_flag_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_additional_data_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_additional_data_flag_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (BITCODE_B) ((arg1)->additional_data_flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_cell_flag_override_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_cell_flag_override_set(self,cell_flag_override);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_cell_flag_override_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_Cell_cell_flag_override_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->cell_flag_override = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_cell_flag_override_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_cell_flag_override_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_cell_flag_override_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (BITCODE_BL) ((arg1)->cell_flag_override);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_virtual_edge_flag_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_virtual_edge_flag_set(self,virtual_edge_flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_virtual_edge_flag_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_Cell_virtual_edge_flag_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->virtual_edge_flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_virtual_edge_flag_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_virtual_edge_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_virtual_edge_flag_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (BITCODE_RC) ((arg1)->virtual_edge_flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_cell_alignment_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_cell_alignment_set(self,cell_alignment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_cell_alignment_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_Cell_cell_alignment_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->cell_alignment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_cell_alignment_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_cell_alignment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_cell_alignment_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (BITCODE_RS) ((arg1)->cell_alignment);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_bg_fill_none_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_bg_fill_none_set(self,bg_fill_none);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_bg_fill_none_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_Cell_bg_fill_none_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->bg_fill_none = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_bg_fill_none_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_bg_fill_none_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_bg_fill_none_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (BITCODE_B) ((arg1)->bg_fill_none);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_bg_color_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_bg_color_set(self,bg_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_bg_color_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLE_Cell_bg_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->bg_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_bg_color_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_bg_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_bg_color_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->bg_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_content_color_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_content_color_set(self,content_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_content_color_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLE_Cell_content_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->content_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_content_color_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_content_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_content_color_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->content_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_text_height_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_text_height_set(self,text_height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_text_height_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_Cell_text_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->text_height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_text_height_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_text_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_text_height_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (BITCODE_BD) ((arg1)->text_height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_top_grid_color_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_top_grid_color_set(self,top_grid_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_top_grid_color_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLE_Cell_top_grid_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->top_grid_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_top_grid_color_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_top_grid_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_top_grid_color_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->top_grid_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_top_grid_linewt_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_top_grid_linewt_set(self,top_grid_linewt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_top_grid_linewt_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_Cell_top_grid_linewt_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->top_grid_linewt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_top_grid_linewt_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_top_grid_linewt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_top_grid_linewt_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (BITCODE_BS) ((arg1)->top_grid_linewt);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_top_visibility_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_top_visibility_set(self,top_visibility);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_top_visibility_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_Cell_top_visibility_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->top_visibility = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_top_visibility_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_top_visibility_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_top_visibility_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (BITCODE_BS) ((arg1)->top_visibility);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_right_grid_color_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_right_grid_color_set(self,right_grid_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_right_grid_color_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLE_Cell_right_grid_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->right_grid_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_right_grid_color_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_right_grid_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_right_grid_color_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->right_grid_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_right_grid_linewt_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_right_grid_linewt_set(self,right_grid_linewt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_right_grid_linewt_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_Cell_right_grid_linewt_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->right_grid_linewt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_right_grid_linewt_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_right_grid_linewt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_right_grid_linewt_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (BITCODE_BS) ((arg1)->right_grid_linewt);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_right_visibility_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_right_visibility_set(self,right_visibility);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_right_visibility_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_Cell_right_visibility_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->right_visibility = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_right_visibility_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_right_visibility_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_right_visibility_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (BITCODE_BS) ((arg1)->right_visibility);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_bottom_grid_color_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_bottom_grid_color_set(self,bottom_grid_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_bottom_grid_color_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLE_Cell_bottom_grid_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->bottom_grid_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_bottom_grid_color_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_bottom_grid_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_bottom_grid_color_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->bottom_grid_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_bottom_grid_linewt_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_bottom_grid_linewt_set(self,bottom_grid_linewt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_bottom_grid_linewt_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_Cell_bottom_grid_linewt_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->bottom_grid_linewt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_bottom_grid_linewt_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_bottom_grid_linewt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_bottom_grid_linewt_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (BITCODE_BS) ((arg1)->bottom_grid_linewt);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_bottom_visibility_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_bottom_visibility_set(self,bottom_visibility);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_bottom_visibility_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_Cell_bottom_visibility_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->bottom_visibility = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_bottom_visibility_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_bottom_visibility_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_bottom_visibility_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (BITCODE_BS) ((arg1)->bottom_visibility);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_left_grid_color_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_left_grid_color_set(self,left_grid_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_left_grid_color_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLE_Cell_left_grid_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->left_grid_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_left_grid_color_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_left_grid_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_left_grid_color_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->left_grid_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_left_grid_linewt_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_left_grid_linewt_set(self,left_grid_linewt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_left_grid_linewt_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_Cell_left_grid_linewt_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->left_grid_linewt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_left_grid_linewt_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_left_grid_linewt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_left_grid_linewt_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (BITCODE_BS) ((arg1)->left_grid_linewt);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_left_visibility_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_left_visibility_set(self,left_visibility);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_left_visibility_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_Cell_left_visibility_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->left_visibility = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_left_visibility_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_left_visibility_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_left_visibility_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (BITCODE_BS) ((arg1)->left_visibility);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_unknown_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_unknown_set(self,unknown);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_unknown_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_Cell_unknown_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_unknown_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_unknown_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_unknown_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_value_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    Dwg_TABLE_value *arg2 = (Dwg_TABLE_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_value_set(self,value);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_value_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_TABLE_value, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLE_Cell_value_set" "', argument " "2"" of type '" "Dwg_TABLE_value *""'"); 
    }
    arg2 = (Dwg_TABLE_value *)(argp2);
    if (arg1) (arg1)->value = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_value_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_TABLE_value *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_value_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_value_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (Dwg_TABLE_value *)& ((arg1)->value);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TABLE_value, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_num_attr_defs_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_num_attr_defs_set(self,num_attr_defs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_num_attr_defs_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_Cell_num_attr_defs_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_attr_defs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_num_attr_defs_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_num_attr_defs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_num_attr_defs_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_attr_defs);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_attr_defs_set) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    Dwg_TABLE_AttrDef *arg2 = (Dwg_TABLE_AttrDef *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_attr_defs_set(self,attr_defs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_attr_defs_set" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_TABLE_AttrDef, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLE_Cell_attr_defs_set" "', argument " "2"" of type '" "Dwg_TABLE_AttrDef *""'"); 
    }
    arg2 = (Dwg_TABLE_AttrDef *)(argp2);
    if (arg1) (arg1)->attr_defs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_Cell_attr_defs_get) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_TABLE_AttrDef *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_Cell_attr_defs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_Cell_attr_defs_get" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    result = (Dwg_TABLE_AttrDef *) ((arg1)->attr_defs);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TABLE_AttrDef, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_TABLE_Cell) {
  {
    int argvi = 0;
    struct _dwg_TABLE_Cell *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_TABLE_Cell();");
    }
    result = (struct _dwg_TABLE_Cell *)calloc(1, sizeof(struct _dwg_TABLE_Cell));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TABLE_Cell, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_TABLE_Cell) {
  {
    struct _dwg_TABLE_Cell *arg1 = (struct _dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_TABLE_Cell(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_Cell, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_TABLE_Cell" "', argument " "1"" of type '" "struct _dwg_TABLE_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLE_Cell *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_BreakHeight_parent_set) {
  {
    struct _dwg_TABLE_BreakHeight *arg1 = (struct _dwg_TABLE_BreakHeight *) 0 ;
    struct _dwg_entity_TABLE *arg2 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_BreakHeight_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_BreakHeight, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_BreakHeight_parent_set" "', argument " "1"" of type '" "struct _dwg_TABLE_BreakHeight *""'"); 
    }
    arg1 = (struct _dwg_TABLE_BreakHeight *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_TABLE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLE_BreakHeight_parent_set" "', argument " "2"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg2 = (struct _dwg_entity_TABLE *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_BreakHeight_parent_get) {
  {
    struct _dwg_TABLE_BreakHeight *arg1 = (struct _dwg_TABLE_BreakHeight *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_entity_TABLE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_BreakHeight_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_BreakHeight, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_BreakHeight_parent_get" "', argument " "1"" of type '" "struct _dwg_TABLE_BreakHeight *""'"); 
    }
    arg1 = (struct _dwg_TABLE_BreakHeight *)(argp1);
    result = (struct _dwg_entity_TABLE *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_TABLE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_BreakHeight_position_set) {
  {
    struct _dwg_TABLE_BreakHeight *arg1 = (struct _dwg_TABLE_BreakHeight *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_BreakHeight_position_set(self,position);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_BreakHeight, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_BreakHeight_position_set" "', argument " "1"" of type '" "struct _dwg_TABLE_BreakHeight *""'"); 
    }
    arg1 = (struct _dwg_TABLE_BreakHeight *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLE_BreakHeight_position_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->position = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_BreakHeight_position_get) {
  {
    struct _dwg_TABLE_BreakHeight *arg1 = (struct _dwg_TABLE_BreakHeight *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_BreakHeight_position_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_BreakHeight, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_BreakHeight_position_get" "', argument " "1"" of type '" "struct _dwg_TABLE_BreakHeight *""'"); 
    }
    arg1 = (struct _dwg_TABLE_BreakHeight *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->position);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_BreakHeight_height_set) {
  {
    struct _dwg_TABLE_BreakHeight *arg1 = (struct _dwg_TABLE_BreakHeight *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_BreakHeight_height_set(self,height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_BreakHeight, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_BreakHeight_height_set" "', argument " "1"" of type '" "struct _dwg_TABLE_BreakHeight *""'"); 
    }
    arg1 = (struct _dwg_TABLE_BreakHeight *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_BreakHeight_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_BreakHeight_height_get) {
  {
    struct _dwg_TABLE_BreakHeight *arg1 = (struct _dwg_TABLE_BreakHeight *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_BreakHeight_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_BreakHeight, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_BreakHeight_height_get" "', argument " "1"" of type '" "struct _dwg_TABLE_BreakHeight *""'"); 
    }
    arg1 = (struct _dwg_TABLE_BreakHeight *)(argp1);
    result = (BITCODE_BD) ((arg1)->height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_BreakHeight_flag_set) {
  {
    struct _dwg_TABLE_BreakHeight *arg1 = (struct _dwg_TABLE_BreakHeight *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_BreakHeight_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_BreakHeight, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_BreakHeight_flag_set" "', argument " "1"" of type '" "struct _dwg_TABLE_BreakHeight *""'"); 
    }
    arg1 = (struct _dwg_TABLE_BreakHeight *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_BreakHeight_flag_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_BreakHeight_flag_get) {
  {
    struct _dwg_TABLE_BreakHeight *arg1 = (struct _dwg_TABLE_BreakHeight *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_BreakHeight_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_BreakHeight, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_BreakHeight_flag_get" "', argument " "1"" of type '" "struct _dwg_TABLE_BreakHeight *""'"); 
    }
    arg1 = (struct _dwg_TABLE_BreakHeight *)(argp1);
    result = (BITCODE_BL) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_TABLE_BreakHeight) {
  {
    int argvi = 0;
    struct _dwg_TABLE_BreakHeight *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_TABLE_BreakHeight();");
    }
    result = (struct _dwg_TABLE_BreakHeight *)calloc(1, sizeof(struct _dwg_TABLE_BreakHeight));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TABLE_BreakHeight, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_TABLE_BreakHeight) {
  {
    struct _dwg_TABLE_BreakHeight *arg1 = (struct _dwg_TABLE_BreakHeight *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_TABLE_BreakHeight(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_BreakHeight, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_TABLE_BreakHeight" "', argument " "1"" of type '" "struct _dwg_TABLE_BreakHeight *""'"); 
    }
    arg1 = (struct _dwg_TABLE_BreakHeight *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_BreakRow_parent_set) {
  {
    struct _dwg_TABLE_BreakRow *arg1 = (struct _dwg_TABLE_BreakRow *) 0 ;
    struct _dwg_entity_TABLE *arg2 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_BreakRow_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_BreakRow, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_BreakRow_parent_set" "', argument " "1"" of type '" "struct _dwg_TABLE_BreakRow *""'"); 
    }
    arg1 = (struct _dwg_TABLE_BreakRow *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_TABLE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLE_BreakRow_parent_set" "', argument " "2"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg2 = (struct _dwg_entity_TABLE *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_BreakRow_parent_get) {
  {
    struct _dwg_TABLE_BreakRow *arg1 = (struct _dwg_TABLE_BreakRow *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_entity_TABLE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_BreakRow_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_BreakRow, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_BreakRow_parent_get" "', argument " "1"" of type '" "struct _dwg_TABLE_BreakRow *""'"); 
    }
    arg1 = (struct _dwg_TABLE_BreakRow *)(argp1);
    result = (struct _dwg_entity_TABLE *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_TABLE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_BreakRow_position_set) {
  {
    struct _dwg_TABLE_BreakRow *arg1 = (struct _dwg_TABLE_BreakRow *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_BreakRow_position_set(self,position);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_BreakRow, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_BreakRow_position_set" "', argument " "1"" of type '" "struct _dwg_TABLE_BreakRow *""'"); 
    }
    arg1 = (struct _dwg_TABLE_BreakRow *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLE_BreakRow_position_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->position = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_BreakRow_position_get) {
  {
    struct _dwg_TABLE_BreakRow *arg1 = (struct _dwg_TABLE_BreakRow *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_BreakRow_position_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_BreakRow, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_BreakRow_position_get" "', argument " "1"" of type '" "struct _dwg_TABLE_BreakRow *""'"); 
    }
    arg1 = (struct _dwg_TABLE_BreakRow *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->position);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_BreakRow_start_set) {
  {
    struct _dwg_TABLE_BreakRow *arg1 = (struct _dwg_TABLE_BreakRow *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_BreakRow_start_set(self,start);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_BreakRow, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_BreakRow_start_set" "', argument " "1"" of type '" "struct _dwg_TABLE_BreakRow *""'"); 
    }
    arg1 = (struct _dwg_TABLE_BreakRow *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_BreakRow_start_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->start = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_BreakRow_start_get) {
  {
    struct _dwg_TABLE_BreakRow *arg1 = (struct _dwg_TABLE_BreakRow *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_BreakRow_start_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_BreakRow, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_BreakRow_start_get" "', argument " "1"" of type '" "struct _dwg_TABLE_BreakRow *""'"); 
    }
    arg1 = (struct _dwg_TABLE_BreakRow *)(argp1);
    result = (BITCODE_BL) ((arg1)->start);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_BreakRow_end_set) {
  {
    struct _dwg_TABLE_BreakRow *arg1 = (struct _dwg_TABLE_BreakRow *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLE_BreakRow_end_set(self,end);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_BreakRow, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_BreakRow_end_set" "', argument " "1"" of type '" "struct _dwg_TABLE_BreakRow *""'"); 
    }
    arg1 = (struct _dwg_TABLE_BreakRow *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLE_BreakRow_end_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->end = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLE_BreakRow_end_get) {
  {
    struct _dwg_TABLE_BreakRow *arg1 = (struct _dwg_TABLE_BreakRow *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLE_BreakRow_end_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_BreakRow, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLE_BreakRow_end_get" "', argument " "1"" of type '" "struct _dwg_TABLE_BreakRow *""'"); 
    }
    arg1 = (struct _dwg_TABLE_BreakRow *)(argp1);
    result = (BITCODE_BL) ((arg1)->end);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_TABLE_BreakRow) {
  {
    int argvi = 0;
    struct _dwg_TABLE_BreakRow *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_TABLE_BreakRow();");
    }
    result = (struct _dwg_TABLE_BreakRow *)calloc(1, sizeof(struct _dwg_TABLE_BreakRow));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TABLE_BreakRow, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_TABLE_BreakRow) {
  {
    struct _dwg_TABLE_BreakRow *arg1 = (struct _dwg_TABLE_BreakRow *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_TABLE_BreakRow(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLE_BreakRow, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_TABLE_BreakRow" "', argument " "1"" of type '" "struct _dwg_TABLE_BreakRow *""'"); 
    }
    arg1 = (struct _dwg_TABLE_BreakRow *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LinkedData_name_set) {
  {
    struct _dwg_LinkedData *arg1 = (struct _dwg_LinkedData *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LinkedData_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LinkedData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LinkedData_name_set" "', argument " "1"" of type '" "struct _dwg_LinkedData *""'"); 
    }
    arg1 = (struct _dwg_LinkedData *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_LinkedData_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LinkedData_name_get) {
  {
    struct _dwg_LinkedData *arg1 = (struct _dwg_LinkedData *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LinkedData_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LinkedData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LinkedData_name_get" "', argument " "1"" of type '" "struct _dwg_LinkedData *""'"); 
    }
    arg1 = (struct _dwg_LinkedData *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LinkedData_description_set) {
  {
    struct _dwg_LinkedData *arg1 = (struct _dwg_LinkedData *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LinkedData_description_set(self,description);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LinkedData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LinkedData_description_set" "', argument " "1"" of type '" "struct _dwg_LinkedData *""'"); 
    }
    arg1 = (struct _dwg_LinkedData *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_LinkedData_description_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->description);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->description = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->description = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LinkedData_description_get) {
  {
    struct _dwg_LinkedData *arg1 = (struct _dwg_LinkedData *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LinkedData_description_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LinkedData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LinkedData_description_get" "', argument " "1"" of type '" "struct _dwg_LinkedData *""'"); 
    }
    arg1 = (struct _dwg_LinkedData *)(argp1);
    result = (BITCODE_TV) ((arg1)->description);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_LinkedData) {
  {
    int argvi = 0;
    struct _dwg_LinkedData *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_LinkedData();");
    }
    result = (struct _dwg_LinkedData *)calloc(1, sizeof(struct _dwg_LinkedData));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_LinkedData, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_LinkedData) {
  {
    struct _dwg_LinkedData *arg1 = (struct _dwg_LinkedData *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_LinkedData(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LinkedData, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_LinkedData" "', argument " "1"" of type '" "struct _dwg_LinkedData *""'"); 
    }
    arg1 = (struct _dwg_LinkedData *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCellContent_Attr_parent_set) {
  {
    struct _dwg_TableCellContent_Attr *arg1 = (struct _dwg_TableCellContent_Attr *) 0 ;
    struct _dwg_TableCellContent *arg2 = (struct _dwg_TableCellContent *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableCellContent_Attr_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCellContent_Attr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCellContent_Attr_parent_set" "', argument " "1"" of type '" "struct _dwg_TableCellContent_Attr *""'"); 
    }
    arg1 = (struct _dwg_TableCellContent_Attr *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_TableCellContent, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TableCellContent_Attr_parent_set" "', argument " "2"" of type '" "struct _dwg_TableCellContent *""'"); 
    }
    arg2 = (struct _dwg_TableCellContent *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCellContent_Attr_parent_get) {
  {
    struct _dwg_TableCellContent_Attr *arg1 = (struct _dwg_TableCellContent_Attr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_TableCellContent *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableCellContent_Attr_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCellContent_Attr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCellContent_Attr_parent_get" "', argument " "1"" of type '" "struct _dwg_TableCellContent_Attr *""'"); 
    }
    arg1 = (struct _dwg_TableCellContent_Attr *)(argp1);
    result = (struct _dwg_TableCellContent *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TableCellContent, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCellContent_Attr_attdef_set) {
  {
    struct _dwg_TableCellContent_Attr *arg1 = (struct _dwg_TableCellContent_Attr *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableCellContent_Attr_attdef_set(self,attdef);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCellContent_Attr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCellContent_Attr_attdef_set" "', argument " "1"" of type '" "struct _dwg_TableCellContent_Attr *""'"); 
    }
    arg1 = (struct _dwg_TableCellContent_Attr *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TableCellContent_Attr_attdef_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->attdef = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCellContent_Attr_attdef_get) {
  {
    struct _dwg_TableCellContent_Attr *arg1 = (struct _dwg_TableCellContent_Attr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableCellContent_Attr_attdef_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCellContent_Attr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCellContent_Attr_attdef_get" "', argument " "1"" of type '" "struct _dwg_TableCellContent_Attr *""'"); 
    }
    arg1 = (struct _dwg_TableCellContent_Attr *)(argp1);
    result = (BITCODE_H) ((arg1)->attdef);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCellContent_Attr_value_set) {
  {
    struct _dwg_TableCellContent_Attr *arg1 = (struct _dwg_TableCellContent_Attr *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableCellContent_Attr_value_set(self,value);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCellContent_Attr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCellContent_Attr_value_set" "', argument " "1"" of type '" "struct _dwg_TableCellContent_Attr *""'"); 
    }
    arg1 = (struct _dwg_TableCellContent_Attr *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TableCellContent_Attr_value_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->value);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->value = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->value = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCellContent_Attr_value_get) {
  {
    struct _dwg_TableCellContent_Attr *arg1 = (struct _dwg_TableCellContent_Attr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableCellContent_Attr_value_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCellContent_Attr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCellContent_Attr_value_get" "', argument " "1"" of type '" "struct _dwg_TableCellContent_Attr *""'"); 
    }
    arg1 = (struct _dwg_TableCellContent_Attr *)(argp1);
    result = (BITCODE_TV) ((arg1)->value);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCellContent_Attr_index_set) {
  {
    struct _dwg_TableCellContent_Attr *arg1 = (struct _dwg_TableCellContent_Attr *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableCellContent_Attr_index_set(self,index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCellContent_Attr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCellContent_Attr_index_set" "', argument " "1"" of type '" "struct _dwg_TableCellContent_Attr *""'"); 
    }
    arg1 = (struct _dwg_TableCellContent_Attr *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TableCellContent_Attr_index_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCellContent_Attr_index_get) {
  {
    struct _dwg_TableCellContent_Attr *arg1 = (struct _dwg_TableCellContent_Attr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableCellContent_Attr_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCellContent_Attr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCellContent_Attr_index_get" "', argument " "1"" of type '" "struct _dwg_TableCellContent_Attr *""'"); 
    }
    arg1 = (struct _dwg_TableCellContent_Attr *)(argp1);
    result = (BITCODE_BL) ((arg1)->index);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_TableCellContent_Attr) {
  {
    int argvi = 0;
    struct _dwg_TableCellContent_Attr *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_TableCellContent_Attr();");
    }
    result = (struct _dwg_TableCellContent_Attr *)calloc(1, sizeof(struct _dwg_TableCellContent_Attr));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TableCellContent_Attr, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_TableCellContent_Attr) {
  {
    struct _dwg_TableCellContent_Attr *arg1 = (struct _dwg_TableCellContent_Attr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_TableCellContent_Attr(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCellContent_Attr, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_TableCellContent_Attr" "', argument " "1"" of type '" "struct _dwg_TableCellContent_Attr *""'"); 
    }
    arg1 = (struct _dwg_TableCellContent_Attr *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ContentFormat_property_override_flags_set) {
  {
    struct _dwg_ContentFormat *arg1 = (struct _dwg_ContentFormat *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ContentFormat_property_override_flags_set(self,property_override_flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ContentFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ContentFormat_property_override_flags_set" "', argument " "1"" of type '" "struct _dwg_ContentFormat *""'"); 
    }
    arg1 = (struct _dwg_ContentFormat *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ContentFormat_property_override_flags_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->property_override_flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ContentFormat_property_override_flags_get) {
  {
    struct _dwg_ContentFormat *arg1 = (struct _dwg_ContentFormat *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ContentFormat_property_override_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ContentFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ContentFormat_property_override_flags_get" "', argument " "1"" of type '" "struct _dwg_ContentFormat *""'"); 
    }
    arg1 = (struct _dwg_ContentFormat *)(argp1);
    result = (BITCODE_BL) ((arg1)->property_override_flags);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ContentFormat_property_flags_set) {
  {
    struct _dwg_ContentFormat *arg1 = (struct _dwg_ContentFormat *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ContentFormat_property_flags_set(self,property_flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ContentFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ContentFormat_property_flags_set" "', argument " "1"" of type '" "struct _dwg_ContentFormat *""'"); 
    }
    arg1 = (struct _dwg_ContentFormat *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ContentFormat_property_flags_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->property_flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ContentFormat_property_flags_get) {
  {
    struct _dwg_ContentFormat *arg1 = (struct _dwg_ContentFormat *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ContentFormat_property_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ContentFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ContentFormat_property_flags_get" "', argument " "1"" of type '" "struct _dwg_ContentFormat *""'"); 
    }
    arg1 = (struct _dwg_ContentFormat *)(argp1);
    result = (BITCODE_BL) ((arg1)->property_flags);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ContentFormat_value_data_type_set) {
  {
    struct _dwg_ContentFormat *arg1 = (struct _dwg_ContentFormat *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ContentFormat_value_data_type_set(self,value_data_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ContentFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ContentFormat_value_data_type_set" "', argument " "1"" of type '" "struct _dwg_ContentFormat *""'"); 
    }
    arg1 = (struct _dwg_ContentFormat *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ContentFormat_value_data_type_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->value_data_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ContentFormat_value_data_type_get) {
  {
    struct _dwg_ContentFormat *arg1 = (struct _dwg_ContentFormat *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ContentFormat_value_data_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ContentFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ContentFormat_value_data_type_get" "', argument " "1"" of type '" "struct _dwg_ContentFormat *""'"); 
    }
    arg1 = (struct _dwg_ContentFormat *)(argp1);
    result = (BITCODE_BL) ((arg1)->value_data_type);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ContentFormat_value_unit_type_set) {
  {
    struct _dwg_ContentFormat *arg1 = (struct _dwg_ContentFormat *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ContentFormat_value_unit_type_set(self,value_unit_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ContentFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ContentFormat_value_unit_type_set" "', argument " "1"" of type '" "struct _dwg_ContentFormat *""'"); 
    }
    arg1 = (struct _dwg_ContentFormat *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ContentFormat_value_unit_type_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->value_unit_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ContentFormat_value_unit_type_get) {
  {
    struct _dwg_ContentFormat *arg1 = (struct _dwg_ContentFormat *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ContentFormat_value_unit_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ContentFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ContentFormat_value_unit_type_get" "', argument " "1"" of type '" "struct _dwg_ContentFormat *""'"); 
    }
    arg1 = (struct _dwg_ContentFormat *)(argp1);
    result = (BITCODE_BL) ((arg1)->value_unit_type);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ContentFormat_value_format_string_set) {
  {
    struct _dwg_ContentFormat *arg1 = (struct _dwg_ContentFormat *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ContentFormat_value_format_string_set(self,value_format_string);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ContentFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ContentFormat_value_format_string_set" "', argument " "1"" of type '" "struct _dwg_ContentFormat *""'"); 
    }
    arg1 = (struct _dwg_ContentFormat *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_ContentFormat_value_format_string_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->value_format_string);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->value_format_string = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->value_format_string = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ContentFormat_value_format_string_get) {
  {
    struct _dwg_ContentFormat *arg1 = (struct _dwg_ContentFormat *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ContentFormat_value_format_string_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ContentFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ContentFormat_value_format_string_get" "', argument " "1"" of type '" "struct _dwg_ContentFormat *""'"); 
    }
    arg1 = (struct _dwg_ContentFormat *)(argp1);
    result = (BITCODE_TV) ((arg1)->value_format_string);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ContentFormat_rotation_set) {
  {
    struct _dwg_ContentFormat *arg1 = (struct _dwg_ContentFormat *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ContentFormat_rotation_set(self,rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ContentFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ContentFormat_rotation_set" "', argument " "1"" of type '" "struct _dwg_ContentFormat *""'"); 
    }
    arg1 = (struct _dwg_ContentFormat *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ContentFormat_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ContentFormat_rotation_get) {
  {
    struct _dwg_ContentFormat *arg1 = (struct _dwg_ContentFormat *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ContentFormat_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ContentFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ContentFormat_rotation_get" "', argument " "1"" of type '" "struct _dwg_ContentFormat *""'"); 
    }
    arg1 = (struct _dwg_ContentFormat *)(argp1);
    result = (BITCODE_BD) ((arg1)->rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ContentFormat_block_scale_set) {
  {
    struct _dwg_ContentFormat *arg1 = (struct _dwg_ContentFormat *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ContentFormat_block_scale_set(self,block_scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ContentFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ContentFormat_block_scale_set" "', argument " "1"" of type '" "struct _dwg_ContentFormat *""'"); 
    }
    arg1 = (struct _dwg_ContentFormat *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ContentFormat_block_scale_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->block_scale = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ContentFormat_block_scale_get) {
  {
    struct _dwg_ContentFormat *arg1 = (struct _dwg_ContentFormat *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ContentFormat_block_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ContentFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ContentFormat_block_scale_get" "', argument " "1"" of type '" "struct _dwg_ContentFormat *""'"); 
    }
    arg1 = (struct _dwg_ContentFormat *)(argp1);
    result = (BITCODE_BD) ((arg1)->block_scale);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ContentFormat_cell_alignment_set) {
  {
    struct _dwg_ContentFormat *arg1 = (struct _dwg_ContentFormat *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ContentFormat_cell_alignment_set(self,cell_alignment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ContentFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ContentFormat_cell_alignment_set" "', argument " "1"" of type '" "struct _dwg_ContentFormat *""'"); 
    }
    arg1 = (struct _dwg_ContentFormat *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ContentFormat_cell_alignment_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->cell_alignment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ContentFormat_cell_alignment_get) {
  {
    struct _dwg_ContentFormat *arg1 = (struct _dwg_ContentFormat *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ContentFormat_cell_alignment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ContentFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ContentFormat_cell_alignment_get" "', argument " "1"" of type '" "struct _dwg_ContentFormat *""'"); 
    }
    arg1 = (struct _dwg_ContentFormat *)(argp1);
    result = (BITCODE_BL) ((arg1)->cell_alignment);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ContentFormat_content_color_set) {
  {
    struct _dwg_ContentFormat *arg1 = (struct _dwg_ContentFormat *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ContentFormat_content_color_set(self,content_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ContentFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ContentFormat_content_color_set" "', argument " "1"" of type '" "struct _dwg_ContentFormat *""'"); 
    }
    arg1 = (struct _dwg_ContentFormat *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_ContentFormat_content_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->content_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ContentFormat_content_color_get) {
  {
    struct _dwg_ContentFormat *arg1 = (struct _dwg_ContentFormat *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ContentFormat_content_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ContentFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ContentFormat_content_color_get" "', argument " "1"" of type '" "struct _dwg_ContentFormat *""'"); 
    }
    arg1 = (struct _dwg_ContentFormat *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->content_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ContentFormat_text_style_set) {
  {
    struct _dwg_ContentFormat *arg1 = (struct _dwg_ContentFormat *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ContentFormat_text_style_set(self,text_style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ContentFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ContentFormat_text_style_set" "', argument " "1"" of type '" "struct _dwg_ContentFormat *""'"); 
    }
    arg1 = (struct _dwg_ContentFormat *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_ContentFormat_text_style_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->text_style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ContentFormat_text_style_get) {
  {
    struct _dwg_ContentFormat *arg1 = (struct _dwg_ContentFormat *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ContentFormat_text_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ContentFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ContentFormat_text_style_get" "', argument " "1"" of type '" "struct _dwg_ContentFormat *""'"); 
    }
    arg1 = (struct _dwg_ContentFormat *)(argp1);
    result = (BITCODE_H) ((arg1)->text_style);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ContentFormat_text_height_set) {
  {
    struct _dwg_ContentFormat *arg1 = (struct _dwg_ContentFormat *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ContentFormat_text_height_set(self,text_height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ContentFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ContentFormat_text_height_set" "', argument " "1"" of type '" "struct _dwg_ContentFormat *""'"); 
    }
    arg1 = (struct _dwg_ContentFormat *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ContentFormat_text_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->text_height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ContentFormat_text_height_get) {
  {
    struct _dwg_ContentFormat *arg1 = (struct _dwg_ContentFormat *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ContentFormat_text_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ContentFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ContentFormat_text_height_get" "', argument " "1"" of type '" "struct _dwg_ContentFormat *""'"); 
    }
    arg1 = (struct _dwg_ContentFormat *)(argp1);
    result = (BITCODE_BD) ((arg1)->text_height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_ContentFormat) {
  {
    int argvi = 0;
    struct _dwg_ContentFormat *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_ContentFormat();");
    }
    result = (struct _dwg_ContentFormat *)calloc(1, sizeof(struct _dwg_ContentFormat));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ContentFormat, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_ContentFormat) {
  {
    struct _dwg_ContentFormat *arg1 = (struct _dwg_ContentFormat *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_ContentFormat(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ContentFormat, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_ContentFormat" "', argument " "1"" of type '" "struct _dwg_ContentFormat *""'"); 
    }
    arg1 = (struct _dwg_ContentFormat *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCellContent_parent_set) {
  {
    struct _dwg_TableCellContent *arg1 = (struct _dwg_TableCellContent *) 0 ;
    struct _dwg_TableCell *arg2 = (struct _dwg_TableCell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableCellContent_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCellContent, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCellContent_parent_set" "', argument " "1"" of type '" "struct _dwg_TableCellContent *""'"); 
    }
    arg1 = (struct _dwg_TableCellContent *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_TableCell, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TableCellContent_parent_set" "', argument " "2"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg2 = (struct _dwg_TableCell *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCellContent_parent_get) {
  {
    struct _dwg_TableCellContent *arg1 = (struct _dwg_TableCellContent *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_TableCell *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableCellContent_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCellContent, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCellContent_parent_get" "', argument " "1"" of type '" "struct _dwg_TableCellContent *""'"); 
    }
    arg1 = (struct _dwg_TableCellContent *)(argp1);
    result = (struct _dwg_TableCell *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TableCell, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCellContent_type_set) {
  {
    struct _dwg_TableCellContent *arg1 = (struct _dwg_TableCellContent *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableCellContent_type_set(self,type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCellContent, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCellContent_type_set" "', argument " "1"" of type '" "struct _dwg_TableCellContent *""'"); 
    }
    arg1 = (struct _dwg_TableCellContent *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TableCellContent_type_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCellContent_type_get) {
  {
    struct _dwg_TableCellContent *arg1 = (struct _dwg_TableCellContent *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableCellContent_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCellContent, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCellContent_type_get" "', argument " "1"" of type '" "struct _dwg_TableCellContent *""'"); 
    }
    arg1 = (struct _dwg_TableCellContent *)(argp1);
    result = (BITCODE_BL) ((arg1)->type);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCellContent_value_set) {
  {
    struct _dwg_TableCellContent *arg1 = (struct _dwg_TableCellContent *) 0 ;
    Dwg_TABLE_value *arg2 = (Dwg_TABLE_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableCellContent_value_set(self,value);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCellContent, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCellContent_value_set" "', argument " "1"" of type '" "struct _dwg_TableCellContent *""'"); 
    }
    arg1 = (struct _dwg_TableCellContent *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_TABLE_value, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TableCellContent_value_set" "', argument " "2"" of type '" "Dwg_TABLE_value *""'"); 
    }
    arg2 = (Dwg_TABLE_value *)(argp2);
    if (arg1) (arg1)->value = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCellContent_value_get) {
  {
    struct _dwg_TableCellContent *arg1 = (struct _dwg_TableCellContent *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_TABLE_value *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableCellContent_value_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCellContent, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCellContent_value_get" "', argument " "1"" of type '" "struct _dwg_TableCellContent *""'"); 
    }
    arg1 = (struct _dwg_TableCellContent *)(argp1);
    result = (Dwg_TABLE_value *)& ((arg1)->value);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TABLE_value, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCellContent_handle_set) {
  {
    struct _dwg_TableCellContent *arg1 = (struct _dwg_TableCellContent *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableCellContent_handle_set(self,handle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCellContent, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCellContent_handle_set" "', argument " "1"" of type '" "struct _dwg_TableCellContent *""'"); 
    }
    arg1 = (struct _dwg_TableCellContent *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TableCellContent_handle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->handle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCellContent_handle_get) {
  {
    struct _dwg_TableCellContent *arg1 = (struct _dwg_TableCellContent *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableCellContent_handle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCellContent, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCellContent_handle_get" "', argument " "1"" of type '" "struct _dwg_TableCellContent *""'"); 
    }
    arg1 = (struct _dwg_TableCellContent *)(argp1);
    result = (BITCODE_H) ((arg1)->handle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCellContent_num_attrs_set) {
  {
    struct _dwg_TableCellContent *arg1 = (struct _dwg_TableCellContent *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableCellContent_num_attrs_set(self,num_attrs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCellContent, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCellContent_num_attrs_set" "', argument " "1"" of type '" "struct _dwg_TableCellContent *""'"); 
    }
    arg1 = (struct _dwg_TableCellContent *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TableCellContent_num_attrs_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_attrs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCellContent_num_attrs_get) {
  {
    struct _dwg_TableCellContent *arg1 = (struct _dwg_TableCellContent *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableCellContent_num_attrs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCellContent, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCellContent_num_attrs_get" "', argument " "1"" of type '" "struct _dwg_TableCellContent *""'"); 
    }
    arg1 = (struct _dwg_TableCellContent *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_attrs);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCellContent_attrs_set) {
  {
    struct _dwg_TableCellContent *arg1 = (struct _dwg_TableCellContent *) 0 ;
    Dwg_TableCellContent_Attr *arg2 = (Dwg_TableCellContent_Attr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableCellContent_attrs_set(self,attrs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCellContent, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCellContent_attrs_set" "', argument " "1"" of type '" "struct _dwg_TableCellContent *""'"); 
    }
    arg1 = (struct _dwg_TableCellContent *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_TableCellContent_Attr, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TableCellContent_attrs_set" "', argument " "2"" of type '" "Dwg_TableCellContent_Attr *""'"); 
    }
    arg2 = (Dwg_TableCellContent_Attr *)(argp2);
    if (arg1) (arg1)->attrs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCellContent_attrs_get) {
  {
    struct _dwg_TableCellContent *arg1 = (struct _dwg_TableCellContent *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_TableCellContent_Attr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableCellContent_attrs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCellContent, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCellContent_attrs_get" "', argument " "1"" of type '" "struct _dwg_TableCellContent *""'"); 
    }
    arg1 = (struct _dwg_TableCellContent *)(argp1);
    result = (Dwg_TableCellContent_Attr *) ((arg1)->attrs);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TableCellContent_Attr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCellContent_has_content_format_overrides_set) {
  {
    struct _dwg_TableCellContent *arg1 = (struct _dwg_TableCellContent *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableCellContent_has_content_format_overrides_set(self,has_content_format_overrides);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCellContent, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCellContent_has_content_format_overrides_set" "', argument " "1"" of type '" "struct _dwg_TableCellContent *""'"); 
    }
    arg1 = (struct _dwg_TableCellContent *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TableCellContent_has_content_format_overrides_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->has_content_format_overrides = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCellContent_has_content_format_overrides_get) {
  {
    struct _dwg_TableCellContent *arg1 = (struct _dwg_TableCellContent *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableCellContent_has_content_format_overrides_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCellContent, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCellContent_has_content_format_overrides_get" "', argument " "1"" of type '" "struct _dwg_TableCellContent *""'"); 
    }
    arg1 = (struct _dwg_TableCellContent *)(argp1);
    result = (BITCODE_BS) ((arg1)->has_content_format_overrides);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCellContent_content_format_set) {
  {
    struct _dwg_TableCellContent *arg1 = (struct _dwg_TableCellContent *) 0 ;
    Dwg_ContentFormat *arg2 = (Dwg_ContentFormat *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableCellContent_content_format_set(self,content_format);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCellContent, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCellContent_content_format_set" "', argument " "1"" of type '" "struct _dwg_TableCellContent *""'"); 
    }
    arg1 = (struct _dwg_TableCellContent *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ContentFormat, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TableCellContent_content_format_set" "', argument " "2"" of type '" "Dwg_ContentFormat *""'"); 
    }
    arg2 = (Dwg_ContentFormat *)(argp2);
    if (arg1) (arg1)->content_format = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCellContent_content_format_get) {
  {
    struct _dwg_TableCellContent *arg1 = (struct _dwg_TableCellContent *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ContentFormat *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableCellContent_content_format_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCellContent, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCellContent_content_format_get" "', argument " "1"" of type '" "struct _dwg_TableCellContent *""'"); 
    }
    arg1 = (struct _dwg_TableCellContent *)(argp1);
    result = (Dwg_ContentFormat *)& ((arg1)->content_format);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ContentFormat, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_TableCellContent) {
  {
    int argvi = 0;
    struct _dwg_TableCellContent *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_TableCellContent();");
    }
    result = (struct _dwg_TableCellContent *)calloc(1, sizeof(struct _dwg_TableCellContent));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TableCellContent, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_TableCellContent) {
  {
    struct _dwg_TableCellContent *arg1 = (struct _dwg_TableCellContent *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_TableCellContent(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCellContent, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_TableCellContent" "', argument " "1"" of type '" "struct _dwg_TableCellContent *""'"); 
    }
    arg1 = (struct _dwg_TableCellContent *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellContentGeometry_dist_top_left_set) {
  {
    struct _dwg_CellContentGeometry *arg1 = (struct _dwg_CellContentGeometry *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CellContentGeometry_dist_top_left_set(self,dist_top_left);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellContentGeometry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellContentGeometry_dist_top_left_set" "', argument " "1"" of type '" "struct _dwg_CellContentGeometry *""'"); 
    }
    arg1 = (struct _dwg_CellContentGeometry *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_CellContentGeometry_dist_top_left_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->dist_top_left = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellContentGeometry_dist_top_left_get) {
  {
    struct _dwg_CellContentGeometry *arg1 = (struct _dwg_CellContentGeometry *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CellContentGeometry_dist_top_left_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellContentGeometry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellContentGeometry_dist_top_left_get" "', argument " "1"" of type '" "struct _dwg_CellContentGeometry *""'"); 
    }
    arg1 = (struct _dwg_CellContentGeometry *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->dist_top_left);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellContentGeometry_dist_center_set) {
  {
    struct _dwg_CellContentGeometry *arg1 = (struct _dwg_CellContentGeometry *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CellContentGeometry_dist_center_set(self,dist_center);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellContentGeometry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellContentGeometry_dist_center_set" "', argument " "1"" of type '" "struct _dwg_CellContentGeometry *""'"); 
    }
    arg1 = (struct _dwg_CellContentGeometry *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_CellContentGeometry_dist_center_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->dist_center = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellContentGeometry_dist_center_get) {
  {
    struct _dwg_CellContentGeometry *arg1 = (struct _dwg_CellContentGeometry *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CellContentGeometry_dist_center_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellContentGeometry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellContentGeometry_dist_center_get" "', argument " "1"" of type '" "struct _dwg_CellContentGeometry *""'"); 
    }
    arg1 = (struct _dwg_CellContentGeometry *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->dist_center);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellContentGeometry_content_width_set) {
  {
    struct _dwg_CellContentGeometry *arg1 = (struct _dwg_CellContentGeometry *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CellContentGeometry_content_width_set(self,content_width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellContentGeometry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellContentGeometry_content_width_set" "', argument " "1"" of type '" "struct _dwg_CellContentGeometry *""'"); 
    }
    arg1 = (struct _dwg_CellContentGeometry *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_CellContentGeometry_content_width_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->content_width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellContentGeometry_content_width_get) {
  {
    struct _dwg_CellContentGeometry *arg1 = (struct _dwg_CellContentGeometry *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CellContentGeometry_content_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellContentGeometry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellContentGeometry_content_width_get" "', argument " "1"" of type '" "struct _dwg_CellContentGeometry *""'"); 
    }
    arg1 = (struct _dwg_CellContentGeometry *)(argp1);
    result = (BITCODE_BD) ((arg1)->content_width);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellContentGeometry_content_height_set) {
  {
    struct _dwg_CellContentGeometry *arg1 = (struct _dwg_CellContentGeometry *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CellContentGeometry_content_height_set(self,content_height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellContentGeometry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellContentGeometry_content_height_set" "', argument " "1"" of type '" "struct _dwg_CellContentGeometry *""'"); 
    }
    arg1 = (struct _dwg_CellContentGeometry *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_CellContentGeometry_content_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->content_height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellContentGeometry_content_height_get) {
  {
    struct _dwg_CellContentGeometry *arg1 = (struct _dwg_CellContentGeometry *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CellContentGeometry_content_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellContentGeometry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellContentGeometry_content_height_get" "', argument " "1"" of type '" "struct _dwg_CellContentGeometry *""'"); 
    }
    arg1 = (struct _dwg_CellContentGeometry *)(argp1);
    result = (BITCODE_BD) ((arg1)->content_height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellContentGeometry_width_set) {
  {
    struct _dwg_CellContentGeometry *arg1 = (struct _dwg_CellContentGeometry *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CellContentGeometry_width_set(self,width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellContentGeometry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellContentGeometry_width_set" "', argument " "1"" of type '" "struct _dwg_CellContentGeometry *""'"); 
    }
    arg1 = (struct _dwg_CellContentGeometry *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_CellContentGeometry_width_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellContentGeometry_width_get) {
  {
    struct _dwg_CellContentGeometry *arg1 = (struct _dwg_CellContentGeometry *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CellContentGeometry_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellContentGeometry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellContentGeometry_width_get" "', argument " "1"" of type '" "struct _dwg_CellContentGeometry *""'"); 
    }
    arg1 = (struct _dwg_CellContentGeometry *)(argp1);
    result = (BITCODE_BD) ((arg1)->width);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellContentGeometry_height_set) {
  {
    struct _dwg_CellContentGeometry *arg1 = (struct _dwg_CellContentGeometry *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CellContentGeometry_height_set(self,height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellContentGeometry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellContentGeometry_height_set" "', argument " "1"" of type '" "struct _dwg_CellContentGeometry *""'"); 
    }
    arg1 = (struct _dwg_CellContentGeometry *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_CellContentGeometry_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellContentGeometry_height_get) {
  {
    struct _dwg_CellContentGeometry *arg1 = (struct _dwg_CellContentGeometry *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CellContentGeometry_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellContentGeometry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellContentGeometry_height_get" "', argument " "1"" of type '" "struct _dwg_CellContentGeometry *""'"); 
    }
    arg1 = (struct _dwg_CellContentGeometry *)(argp1);
    result = (BITCODE_BD) ((arg1)->height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellContentGeometry_unknown_set) {
  {
    struct _dwg_CellContentGeometry *arg1 = (struct _dwg_CellContentGeometry *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CellContentGeometry_unknown_set(self,unknown);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellContentGeometry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellContentGeometry_unknown_set" "', argument " "1"" of type '" "struct _dwg_CellContentGeometry *""'"); 
    }
    arg1 = (struct _dwg_CellContentGeometry *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_CellContentGeometry_unknown_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellContentGeometry_unknown_get) {
  {
    struct _dwg_CellContentGeometry *arg1 = (struct _dwg_CellContentGeometry *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CellContentGeometry_unknown_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellContentGeometry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellContentGeometry_unknown_get" "', argument " "1"" of type '" "struct _dwg_CellContentGeometry *""'"); 
    }
    arg1 = (struct _dwg_CellContentGeometry *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellContentGeometry_cell_parent_set) {
  {
    struct _dwg_CellContentGeometry *arg1 = (struct _dwg_CellContentGeometry *) 0 ;
    struct _dwg_TableCell *arg2 = (struct _dwg_TableCell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CellContentGeometry_cell_parent_set(self,cell_parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellContentGeometry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellContentGeometry_cell_parent_set" "', argument " "1"" of type '" "struct _dwg_CellContentGeometry *""'"); 
    }
    arg1 = (struct _dwg_CellContentGeometry *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_TableCell, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_CellContentGeometry_cell_parent_set" "', argument " "2"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg2 = (struct _dwg_TableCell *)(argp2);
    if (arg1) (arg1)->cell_parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellContentGeometry_cell_parent_get) {
  {
    struct _dwg_CellContentGeometry *arg1 = (struct _dwg_CellContentGeometry *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_TableCell *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CellContentGeometry_cell_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellContentGeometry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellContentGeometry_cell_parent_get" "', argument " "1"" of type '" "struct _dwg_CellContentGeometry *""'"); 
    }
    arg1 = (struct _dwg_CellContentGeometry *)(argp1);
    result = (struct _dwg_TableCell *) ((arg1)->cell_parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TableCell, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellContentGeometry_geom_parent_set) {
  {
    struct _dwg_CellContentGeometry *arg1 = (struct _dwg_CellContentGeometry *) 0 ;
    struct _dwg_TABLEGEOMETRY_Cell *arg2 = (struct _dwg_TABLEGEOMETRY_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CellContentGeometry_geom_parent_set(self,geom_parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellContentGeometry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellContentGeometry_geom_parent_set" "', argument " "1"" of type '" "struct _dwg_CellContentGeometry *""'"); 
    }
    arg1 = (struct _dwg_CellContentGeometry *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_TABLEGEOMETRY_Cell, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_CellContentGeometry_geom_parent_set" "', argument " "2"" of type '" "struct _dwg_TABLEGEOMETRY_Cell *""'"); 
    }
    arg2 = (struct _dwg_TABLEGEOMETRY_Cell *)(argp2);
    if (arg1) (arg1)->geom_parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellContentGeometry_geom_parent_get) {
  {
    struct _dwg_CellContentGeometry *arg1 = (struct _dwg_CellContentGeometry *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_TABLEGEOMETRY_Cell *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CellContentGeometry_geom_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellContentGeometry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellContentGeometry_geom_parent_get" "', argument " "1"" of type '" "struct _dwg_CellContentGeometry *""'"); 
    }
    arg1 = (struct _dwg_CellContentGeometry *)(argp1);
    result = (struct _dwg_TABLEGEOMETRY_Cell *) ((arg1)->geom_parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TABLEGEOMETRY_Cell, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_CellContentGeometry) {
  {
    int argvi = 0;
    struct _dwg_CellContentGeometry *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_CellContentGeometry();");
    }
    result = (struct _dwg_CellContentGeometry *)calloc(1, sizeof(struct _dwg_CellContentGeometry));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_CellContentGeometry, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_CellContentGeometry) {
  {
    struct _dwg_CellContentGeometry *arg1 = (struct _dwg_CellContentGeometry *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_CellContentGeometry(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellContentGeometry, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_CellContentGeometry" "', argument " "1"" of type '" "struct _dwg_CellContentGeometry *""'"); 
    }
    arg1 = (struct _dwg_CellContentGeometry *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_flag_set) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableCell_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_flag_set" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TableCell_flag_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_flag_get) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableCell_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_flag_get" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    result = (BITCODE_BL) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_tooltip_set) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableCell_tooltip_set(self,tooltip);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_tooltip_set" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TableCell_tooltip_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->tooltip);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->tooltip = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->tooltip = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_tooltip_get) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableCell_tooltip_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_tooltip_get" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    result = (BITCODE_TV) ((arg1)->tooltip);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_customdata_set) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableCell_customdata_set(self,customdata);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_customdata_set" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TableCell_customdata_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->customdata = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_customdata_get) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableCell_customdata_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_customdata_get" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    result = (BITCODE_BL) ((arg1)->customdata);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_num_customdata_items_set) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableCell_num_customdata_items_set(self,num_customdata_items);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_num_customdata_items_set" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TableCell_num_customdata_items_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_customdata_items = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_num_customdata_items_get) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableCell_num_customdata_items_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_num_customdata_items_get" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_customdata_items);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_customdata_items_set) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    Dwg_TABLE_CustomDataItem *arg2 = (Dwg_TABLE_CustomDataItem *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableCell_customdata_items_set(self,customdata_items);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_customdata_items_set" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_TABLE_CustomDataItem, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TableCell_customdata_items_set" "', argument " "2"" of type '" "Dwg_TABLE_CustomDataItem *""'"); 
    }
    arg2 = (Dwg_TABLE_CustomDataItem *)(argp2);
    if (arg1) (arg1)->customdata_items = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_customdata_items_get) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_TABLE_CustomDataItem *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableCell_customdata_items_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_customdata_items_get" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    result = (Dwg_TABLE_CustomDataItem *) ((arg1)->customdata_items);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TABLE_CustomDataItem, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_has_linked_data_set) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableCell_has_linked_data_set(self,has_linked_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_has_linked_data_set" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TableCell_has_linked_data_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->has_linked_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_has_linked_data_get) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableCell_has_linked_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_has_linked_data_get" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    result = (BITCODE_BL) ((arg1)->has_linked_data);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_data_link_set) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableCell_data_link_set(self,data_link);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_data_link_set" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TableCell_data_link_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->data_link = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_data_link_get) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableCell_data_link_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_data_link_get" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    result = (BITCODE_H) ((arg1)->data_link);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_num_rows_set) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableCell_num_rows_set(self,num_rows);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_num_rows_set" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TableCell_num_rows_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_rows = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_num_rows_get) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableCell_num_rows_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_num_rows_get" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_rows);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_num_cols_set) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableCell_num_cols_set(self,num_cols);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_num_cols_set" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TableCell_num_cols_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_cols = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_num_cols_get) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableCell_num_cols_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_num_cols_get" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_cols);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_unknown_set) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableCell_unknown_set(self,unknown);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_unknown_set" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TableCell_unknown_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_unknown_get) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableCell_unknown_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_unknown_get" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_num_cell_contents_set) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableCell_num_cell_contents_set(self,num_cell_contents);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_num_cell_contents_set" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TableCell_num_cell_contents_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_cell_contents = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_num_cell_contents_get) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableCell_num_cell_contents_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_num_cell_contents_get" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_cell_contents);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_cell_contents_set) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    Dwg_TableCellContent *arg2 = (Dwg_TableCellContent *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableCell_cell_contents_set(self,cell_contents);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_cell_contents_set" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_TableCellContent, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TableCell_cell_contents_set" "', argument " "2"" of type '" "Dwg_TableCellContent *""'"); 
    }
    arg2 = (Dwg_TableCellContent *)(argp2);
    if (arg1) (arg1)->cell_contents = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_cell_contents_get) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_TableCellContent *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableCell_cell_contents_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_cell_contents_get" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    result = (Dwg_TableCellContent *) ((arg1)->cell_contents);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TableCellContent, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_style_id_set) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableCell_style_id_set(self,style_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_style_id_set" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TableCell_style_id_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->style_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_style_id_get) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableCell_style_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_style_id_get" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    result = (BITCODE_BL) ((arg1)->style_id);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_has_geom_data_set) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableCell_has_geom_data_set(self,has_geom_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_has_geom_data_set" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TableCell_has_geom_data_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->has_geom_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_has_geom_data_get) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableCell_has_geom_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_has_geom_data_get" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    result = (BITCODE_BL) ((arg1)->has_geom_data);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_geom_data_flag_set) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableCell_geom_data_flag_set(self,geom_data_flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_geom_data_flag_set" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TableCell_geom_data_flag_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->geom_data_flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_geom_data_flag_get) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableCell_geom_data_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_geom_data_flag_get" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    result = (BITCODE_BL) ((arg1)->geom_data_flag);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_width_w_gap_set) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableCell_width_w_gap_set(self,width_w_gap);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_width_w_gap_set" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TableCell_width_w_gap_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->width_w_gap = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_width_w_gap_get) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableCell_width_w_gap_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_width_w_gap_get" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    result = (BITCODE_BD) ((arg1)->width_w_gap);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_height_w_gap_set) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableCell_height_w_gap_set(self,height_w_gap);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_height_w_gap_set" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TableCell_height_w_gap_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->height_w_gap = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_height_w_gap_get) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableCell_height_w_gap_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_height_w_gap_get" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    result = (BITCODE_BD) ((arg1)->height_w_gap);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_tablegeometry_set) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableCell_tablegeometry_set(self,tablegeometry);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_tablegeometry_set" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TableCell_tablegeometry_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->tablegeometry = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_tablegeometry_get) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableCell_tablegeometry_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_tablegeometry_get" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    result = (BITCODE_H) ((arg1)->tablegeometry);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_num_geometry_set) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableCell_num_geometry_set(self,num_geometry);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_num_geometry_set" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TableCell_num_geometry_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_geometry = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_num_geometry_get) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableCell_num_geometry_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_num_geometry_get" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_geometry);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_geometry_set) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    Dwg_CellContentGeometry *arg2 = (Dwg_CellContentGeometry *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableCell_geometry_set(self,geometry);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_geometry_set" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_CellContentGeometry, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TableCell_geometry_set" "', argument " "2"" of type '" "Dwg_CellContentGeometry *""'"); 
    }
    arg2 = (Dwg_CellContentGeometry *)(argp2);
    if (arg1) (arg1)->geometry = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_geometry_get) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_CellContentGeometry *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableCell_geometry_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_geometry_get" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    result = (Dwg_CellContentGeometry *) ((arg1)->geometry);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_CellContentGeometry, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_style_parent_set) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    struct _dwg_CellStyle *arg2 = (struct _dwg_CellStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableCell_style_parent_set(self,style_parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_style_parent_set" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_CellStyle, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TableCell_style_parent_set" "', argument " "2"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg2 = (struct _dwg_CellStyle *)(argp2);
    if (arg1) (arg1)->style_parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_style_parent_get) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_CellStyle *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableCell_style_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_style_parent_get" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    result = (struct _dwg_CellStyle *) ((arg1)->style_parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_CellStyle, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_row_parent_set) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    struct _dwg_TableRow *arg2 = (struct _dwg_TableRow *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableCell_row_parent_set(self,row_parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_row_parent_set" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_TableRow, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TableCell_row_parent_set" "', argument " "2"" of type '" "struct _dwg_TableRow *""'"); 
    }
    arg2 = (struct _dwg_TableRow *)(argp2);
    if (arg1) (arg1)->row_parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableCell_row_parent_get) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_TableRow *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableCell_row_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableCell_row_parent_get" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    result = (struct _dwg_TableRow *) ((arg1)->row_parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TableRow, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_TableCell) {
  {
    int argvi = 0;
    struct _dwg_TableCell *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_TableCell();");
    }
    result = (struct _dwg_TableCell *)calloc(1, sizeof(struct _dwg_TableCell));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TableCell, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_TableCell) {
  {
    struct _dwg_TableCell *arg1 = (struct _dwg_TableCell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_TableCell(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableCell, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_TableCell" "', argument " "1"" of type '" "struct _dwg_TableCell *""'"); 
    }
    arg1 = (struct _dwg_TableCell *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_GridFormat_parent_set) {
  {
    struct _dwg_GridFormat *arg1 = (struct _dwg_GridFormat *) 0 ;
    struct _dwg_CellStyle *arg2 = (struct _dwg_CellStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_GridFormat_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_GridFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_GridFormat_parent_set" "', argument " "1"" of type '" "struct _dwg_GridFormat *""'"); 
    }
    arg1 = (struct _dwg_GridFormat *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_CellStyle, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_GridFormat_parent_set" "', argument " "2"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg2 = (struct _dwg_CellStyle *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_GridFormat_parent_get) {
  {
    struct _dwg_GridFormat *arg1 = (struct _dwg_GridFormat *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_CellStyle *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_GridFormat_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_GridFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_GridFormat_parent_get" "', argument " "1"" of type '" "struct _dwg_GridFormat *""'"); 
    }
    arg1 = (struct _dwg_GridFormat *)(argp1);
    result = (struct _dwg_CellStyle *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_CellStyle, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_GridFormat_index_mask_set) {
  {
    struct _dwg_GridFormat *arg1 = (struct _dwg_GridFormat *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_GridFormat_index_mask_set(self,index_mask);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_GridFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_GridFormat_index_mask_set" "', argument " "1"" of type '" "struct _dwg_GridFormat *""'"); 
    }
    arg1 = (struct _dwg_GridFormat *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_GridFormat_index_mask_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->index_mask = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_GridFormat_index_mask_get) {
  {
    struct _dwg_GridFormat *arg1 = (struct _dwg_GridFormat *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_GridFormat_index_mask_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_GridFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_GridFormat_index_mask_get" "', argument " "1"" of type '" "struct _dwg_GridFormat *""'"); 
    }
    arg1 = (struct _dwg_GridFormat *)(argp1);
    result = (BITCODE_BL) ((arg1)->index_mask);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_GridFormat_border_overrides_set) {
  {
    struct _dwg_GridFormat *arg1 = (struct _dwg_GridFormat *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_GridFormat_border_overrides_set(self,border_overrides);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_GridFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_GridFormat_border_overrides_set" "', argument " "1"" of type '" "struct _dwg_GridFormat *""'"); 
    }
    arg1 = (struct _dwg_GridFormat *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_GridFormat_border_overrides_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->border_overrides = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_GridFormat_border_overrides_get) {
  {
    struct _dwg_GridFormat *arg1 = (struct _dwg_GridFormat *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_GridFormat_border_overrides_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_GridFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_GridFormat_border_overrides_get" "', argument " "1"" of type '" "struct _dwg_GridFormat *""'"); 
    }
    arg1 = (struct _dwg_GridFormat *)(argp1);
    result = (BITCODE_BL) ((arg1)->border_overrides);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_GridFormat_border_type_set) {
  {
    struct _dwg_GridFormat *arg1 = (struct _dwg_GridFormat *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_GridFormat_border_type_set(self,border_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_GridFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_GridFormat_border_type_set" "', argument " "1"" of type '" "struct _dwg_GridFormat *""'"); 
    }
    arg1 = (struct _dwg_GridFormat *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_GridFormat_border_type_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->border_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_GridFormat_border_type_get) {
  {
    struct _dwg_GridFormat *arg1 = (struct _dwg_GridFormat *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_GridFormat_border_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_GridFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_GridFormat_border_type_get" "', argument " "1"" of type '" "struct _dwg_GridFormat *""'"); 
    }
    arg1 = (struct _dwg_GridFormat *)(argp1);
    result = (BITCODE_BL) ((arg1)->border_type);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_GridFormat_color_set) {
  {
    struct _dwg_GridFormat *arg1 = (struct _dwg_GridFormat *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_GridFormat_color_set(self,color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_GridFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_GridFormat_color_set" "', argument " "1"" of type '" "struct _dwg_GridFormat *""'"); 
    }
    arg1 = (struct _dwg_GridFormat *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_GridFormat_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_GridFormat_color_get) {
  {
    struct _dwg_GridFormat *arg1 = (struct _dwg_GridFormat *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_GridFormat_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_GridFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_GridFormat_color_get" "', argument " "1"" of type '" "struct _dwg_GridFormat *""'"); 
    }
    arg1 = (struct _dwg_GridFormat *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_GridFormat_linewt_set) {
  {
    struct _dwg_GridFormat *arg1 = (struct _dwg_GridFormat *) 0 ;
    BITCODE_BLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_GridFormat_linewt_set(self,linewt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_GridFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_GridFormat_linewt_set" "', argument " "1"" of type '" "struct _dwg_GridFormat *""'"); 
    }
    arg1 = (struct _dwg_GridFormat *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_GridFormat_linewt_set" "', argument " "2"" of type '" "BITCODE_BLd""'");
    } 
    arg2 = (BITCODE_BLd)(val2);
    if (arg1) (arg1)->linewt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_GridFormat_linewt_get) {
  {
    struct _dwg_GridFormat *arg1 = (struct _dwg_GridFormat *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_GridFormat_linewt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_GridFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_GridFormat_linewt_get" "', argument " "1"" of type '" "struct _dwg_GridFormat *""'"); 
    }
    arg1 = (struct _dwg_GridFormat *)(argp1);
    result = (BITCODE_BLd) ((arg1)->linewt);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_GridFormat_ltype_set) {
  {
    struct _dwg_GridFormat *arg1 = (struct _dwg_GridFormat *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_GridFormat_ltype_set(self,ltype);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_GridFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_GridFormat_ltype_set" "', argument " "1"" of type '" "struct _dwg_GridFormat *""'"); 
    }
    arg1 = (struct _dwg_GridFormat *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_GridFormat_ltype_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->ltype = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_GridFormat_ltype_get) {
  {
    struct _dwg_GridFormat *arg1 = (struct _dwg_GridFormat *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_GridFormat_ltype_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_GridFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_GridFormat_ltype_get" "', argument " "1"" of type '" "struct _dwg_GridFormat *""'"); 
    }
    arg1 = (struct _dwg_GridFormat *)(argp1);
    result = (BITCODE_H) ((arg1)->ltype);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_GridFormat_visible_set) {
  {
    struct _dwg_GridFormat *arg1 = (struct _dwg_GridFormat *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_GridFormat_visible_set(self,visible);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_GridFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_GridFormat_visible_set" "', argument " "1"" of type '" "struct _dwg_GridFormat *""'"); 
    }
    arg1 = (struct _dwg_GridFormat *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_GridFormat_visible_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->visible = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_GridFormat_visible_get) {
  {
    struct _dwg_GridFormat *arg1 = (struct _dwg_GridFormat *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_GridFormat_visible_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_GridFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_GridFormat_visible_get" "', argument " "1"" of type '" "struct _dwg_GridFormat *""'"); 
    }
    arg1 = (struct _dwg_GridFormat *)(argp1);
    result = (BITCODE_B) ((arg1)->visible);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_GridFormat_double_line_spacing_set) {
  {
    struct _dwg_GridFormat *arg1 = (struct _dwg_GridFormat *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_GridFormat_double_line_spacing_set(self,double_line_spacing);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_GridFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_GridFormat_double_line_spacing_set" "', argument " "1"" of type '" "struct _dwg_GridFormat *""'"); 
    }
    arg1 = (struct _dwg_GridFormat *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_GridFormat_double_line_spacing_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->double_line_spacing = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_GridFormat_double_line_spacing_get) {
  {
    struct _dwg_GridFormat *arg1 = (struct _dwg_GridFormat *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_GridFormat_double_line_spacing_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_GridFormat, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_GridFormat_double_line_spacing_get" "', argument " "1"" of type '" "struct _dwg_GridFormat *""'"); 
    }
    arg1 = (struct _dwg_GridFormat *)(argp1);
    result = (BITCODE_BD) ((arg1)->double_line_spacing);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_GridFormat) {
  {
    int argvi = 0;
    struct _dwg_GridFormat *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_GridFormat();");
    }
    result = (struct _dwg_GridFormat *)calloc(1, sizeof(struct _dwg_GridFormat));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_GridFormat, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_GridFormat) {
  {
    struct _dwg_GridFormat *arg1 = (struct _dwg_GridFormat *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_GridFormat(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_GridFormat, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_GridFormat" "', argument " "1"" of type '" "struct _dwg_GridFormat *""'"); 
    }
    arg1 = (struct _dwg_GridFormat *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_type_set) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CellStyle_type_set(self,type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_type_set" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_CellStyle_type_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_type_get) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CellStyle_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_type_get" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    result = (BITCODE_BL) ((arg1)->type);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_data_flags_set) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CellStyle_data_flags_set(self,data_flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_data_flags_set" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_CellStyle_data_flags_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->data_flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_data_flags_get) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CellStyle_data_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_data_flags_get" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    result = (BITCODE_BS) ((arg1)->data_flags);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_property_override_flags_set) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CellStyle_property_override_flags_set(self,property_override_flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_property_override_flags_set" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_CellStyle_property_override_flags_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->property_override_flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_property_override_flags_get) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CellStyle_property_override_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_property_override_flags_get" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    result = (BITCODE_BL) ((arg1)->property_override_flags);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_merge_flags_set) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CellStyle_merge_flags_set(self,merge_flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_merge_flags_set" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_CellStyle_merge_flags_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->merge_flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_merge_flags_get) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CellStyle_merge_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_merge_flags_get" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    result = (BITCODE_BL) ((arg1)->merge_flags);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_bg_color_set) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CellStyle_bg_color_set(self,bg_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_bg_color_set" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_CellStyle_bg_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->bg_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_bg_color_get) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CellStyle_bg_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_bg_color_get" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->bg_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_content_layout_set) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CellStyle_content_layout_set(self,content_layout);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_content_layout_set" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_CellStyle_content_layout_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->content_layout = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_content_layout_get) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CellStyle_content_layout_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_content_layout_get" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    result = (BITCODE_BL) ((arg1)->content_layout);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_content_format_set) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    Dwg_ContentFormat *arg2 = (Dwg_ContentFormat *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CellStyle_content_format_set(self,content_format);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_content_format_set" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ContentFormat, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_CellStyle_content_format_set" "', argument " "2"" of type '" "Dwg_ContentFormat *""'"); 
    }
    arg2 = (Dwg_ContentFormat *)(argp2);
    if (arg1) (arg1)->content_format = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_content_format_get) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ContentFormat *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CellStyle_content_format_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_content_format_get" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    result = (Dwg_ContentFormat *)& ((arg1)->content_format);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ContentFormat, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_margin_override_flags_set) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CellStyle_margin_override_flags_set(self,margin_override_flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_margin_override_flags_set" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_CellStyle_margin_override_flags_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->margin_override_flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_margin_override_flags_get) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CellStyle_margin_override_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_margin_override_flags_get" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    result = (BITCODE_BS) ((arg1)->margin_override_flags);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_vert_margin_set) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CellStyle_vert_margin_set(self,vert_margin);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_vert_margin_set" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_CellStyle_vert_margin_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->vert_margin = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_vert_margin_get) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CellStyle_vert_margin_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_vert_margin_get" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    result = (BITCODE_BD) ((arg1)->vert_margin);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_horiz_margin_set) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CellStyle_horiz_margin_set(self,horiz_margin);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_horiz_margin_set" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_CellStyle_horiz_margin_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->horiz_margin = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_horiz_margin_get) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CellStyle_horiz_margin_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_horiz_margin_get" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    result = (BITCODE_BD) ((arg1)->horiz_margin);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_bottom_margin_set) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CellStyle_bottom_margin_set(self,bottom_margin);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_bottom_margin_set" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_CellStyle_bottom_margin_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->bottom_margin = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_bottom_margin_get) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CellStyle_bottom_margin_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_bottom_margin_get" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    result = (BITCODE_BD) ((arg1)->bottom_margin);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_right_margin_set) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CellStyle_right_margin_set(self,right_margin);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_right_margin_set" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_CellStyle_right_margin_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->right_margin = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_right_margin_get) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CellStyle_right_margin_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_right_margin_get" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    result = (BITCODE_BD) ((arg1)->right_margin);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_margin_horiz_spacing_set) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CellStyle_margin_horiz_spacing_set(self,margin_horiz_spacing);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_margin_horiz_spacing_set" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_CellStyle_margin_horiz_spacing_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->margin_horiz_spacing = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_margin_horiz_spacing_get) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CellStyle_margin_horiz_spacing_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_margin_horiz_spacing_get" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    result = (BITCODE_BD) ((arg1)->margin_horiz_spacing);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_margin_vert_spacing_set) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CellStyle_margin_vert_spacing_set(self,margin_vert_spacing);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_margin_vert_spacing_set" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_CellStyle_margin_vert_spacing_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->margin_vert_spacing = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_margin_vert_spacing_get) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CellStyle_margin_vert_spacing_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_margin_vert_spacing_get" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    result = (BITCODE_BD) ((arg1)->margin_vert_spacing);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_num_borders_set) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CellStyle_num_borders_set(self,num_borders);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_num_borders_set" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_CellStyle_num_borders_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_borders = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_num_borders_get) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CellStyle_num_borders_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_num_borders_get" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_borders);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_borders_set) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    Dwg_GridFormat *arg2 = (Dwg_GridFormat *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CellStyle_borders_set(self,borders);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_borders_set" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_GridFormat, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_CellStyle_borders_set" "', argument " "2"" of type '" "Dwg_GridFormat *""'"); 
    }
    arg2 = (Dwg_GridFormat *)(argp2);
    if (arg1) (arg1)->borders = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_borders_get) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_GridFormat *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CellStyle_borders_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_borders_get" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    result = (Dwg_GridFormat *) ((arg1)->borders);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_GridFormat, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_tablerow_parent_set) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    struct _dwg_TableRow *arg2 = (struct _dwg_TableRow *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CellStyle_tablerow_parent_set(self,tablerow_parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_tablerow_parent_set" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_TableRow, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_CellStyle_tablerow_parent_set" "', argument " "2"" of type '" "struct _dwg_TableRow *""'"); 
    }
    arg2 = (struct _dwg_TableRow *)(argp2);
    if (arg1) (arg1)->tablerow_parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_tablerow_parent_get) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_TableRow *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CellStyle_tablerow_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_tablerow_parent_get" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    result = (struct _dwg_TableRow *) ((arg1)->tablerow_parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TableRow, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_tabledatacolumn_parent_set) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    struct _dwg_TableDataColumn *arg2 = (struct _dwg_TableDataColumn *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CellStyle_tabledatacolumn_parent_set(self,tabledatacolumn_parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_tabledatacolumn_parent_set" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_TableDataColumn, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_CellStyle_tabledatacolumn_parent_set" "', argument " "2"" of type '" "struct _dwg_TableDataColumn *""'"); 
    }
    arg2 = (struct _dwg_TableDataColumn *)(argp2);
    if (arg1) (arg1)->tabledatacolumn_parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CellStyle_tabledatacolumn_parent_get) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_TableDataColumn *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CellStyle_tabledatacolumn_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CellStyle_tabledatacolumn_parent_get" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    result = (struct _dwg_TableDataColumn *) ((arg1)->tabledatacolumn_parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TableDataColumn, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_CellStyle) {
  {
    int argvi = 0;
    struct _dwg_CellStyle *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_CellStyle();");
    }
    result = (struct _dwg_CellStyle *)calloc(1, sizeof(struct _dwg_CellStyle));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_CellStyle, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_CellStyle) {
  {
    struct _dwg_CellStyle *arg1 = (struct _dwg_CellStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_CellStyle(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CellStyle, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_CellStyle" "', argument " "1"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_CellStyle *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableRow_parent_set) {
  {
    struct _dwg_TableRow *arg1 = (struct _dwg_TableRow *) 0 ;
    struct _dwg_LinkedTableData *arg2 = (struct _dwg_LinkedTableData *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableRow_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableRow, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableRow_parent_set" "', argument " "1"" of type '" "struct _dwg_TableRow *""'"); 
    }
    arg1 = (struct _dwg_TableRow *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_LinkedTableData, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TableRow_parent_set" "', argument " "2"" of type '" "struct _dwg_LinkedTableData *""'"); 
    }
    arg2 = (struct _dwg_LinkedTableData *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableRow_parent_get) {
  {
    struct _dwg_TableRow *arg1 = (struct _dwg_TableRow *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_LinkedTableData *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableRow_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableRow, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableRow_parent_get" "', argument " "1"" of type '" "struct _dwg_TableRow *""'"); 
    }
    arg1 = (struct _dwg_TableRow *)(argp1);
    result = (struct _dwg_LinkedTableData *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_LinkedTableData, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableRow_num_cells_set) {
  {
    struct _dwg_TableRow *arg1 = (struct _dwg_TableRow *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableRow_num_cells_set(self,num_cells);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableRow, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableRow_num_cells_set" "', argument " "1"" of type '" "struct _dwg_TableRow *""'"); 
    }
    arg1 = (struct _dwg_TableRow *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TableRow_num_cells_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_cells = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableRow_num_cells_get) {
  {
    struct _dwg_TableRow *arg1 = (struct _dwg_TableRow *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableRow_num_cells_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableRow, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableRow_num_cells_get" "', argument " "1"" of type '" "struct _dwg_TableRow *""'"); 
    }
    arg1 = (struct _dwg_TableRow *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_cells);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableRow_cells_set) {
  {
    struct _dwg_TableRow *arg1 = (struct _dwg_TableRow *) 0 ;
    Dwg_TableCell *arg2 = (Dwg_TableCell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableRow_cells_set(self,cells);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableRow, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableRow_cells_set" "', argument " "1"" of type '" "struct _dwg_TableRow *""'"); 
    }
    arg1 = (struct _dwg_TableRow *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_TableCell, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TableRow_cells_set" "', argument " "2"" of type '" "Dwg_TableCell *""'"); 
    }
    arg2 = (Dwg_TableCell *)(argp2);
    if (arg1) (arg1)->cells = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableRow_cells_get) {
  {
    struct _dwg_TableRow *arg1 = (struct _dwg_TableRow *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_TableCell *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableRow_cells_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableRow, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableRow_cells_get" "', argument " "1"" of type '" "struct _dwg_TableRow *""'"); 
    }
    arg1 = (struct _dwg_TableRow *)(argp1);
    result = (Dwg_TableCell *) ((arg1)->cells);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TableCell, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableRow_custom_data_set) {
  {
    struct _dwg_TableRow *arg1 = (struct _dwg_TableRow *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableRow_custom_data_set(self,custom_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableRow, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableRow_custom_data_set" "', argument " "1"" of type '" "struct _dwg_TableRow *""'"); 
    }
    arg1 = (struct _dwg_TableRow *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TableRow_custom_data_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->custom_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableRow_custom_data_get) {
  {
    struct _dwg_TableRow *arg1 = (struct _dwg_TableRow *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableRow_custom_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableRow, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableRow_custom_data_get" "', argument " "1"" of type '" "struct _dwg_TableRow *""'"); 
    }
    arg1 = (struct _dwg_TableRow *)(argp1);
    result = (BITCODE_BL) ((arg1)->custom_data);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableRow_num_customdata_items_set) {
  {
    struct _dwg_TableRow *arg1 = (struct _dwg_TableRow *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableRow_num_customdata_items_set(self,num_customdata_items);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableRow, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableRow_num_customdata_items_set" "', argument " "1"" of type '" "struct _dwg_TableRow *""'"); 
    }
    arg1 = (struct _dwg_TableRow *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TableRow_num_customdata_items_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_customdata_items = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableRow_num_customdata_items_get) {
  {
    struct _dwg_TableRow *arg1 = (struct _dwg_TableRow *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableRow_num_customdata_items_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableRow, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableRow_num_customdata_items_get" "', argument " "1"" of type '" "struct _dwg_TableRow *""'"); 
    }
    arg1 = (struct _dwg_TableRow *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_customdata_items);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableRow_customdata_items_set) {
  {
    struct _dwg_TableRow *arg1 = (struct _dwg_TableRow *) 0 ;
    Dwg_TABLE_CustomDataItem *arg2 = (Dwg_TABLE_CustomDataItem *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableRow_customdata_items_set(self,customdata_items);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableRow, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableRow_customdata_items_set" "', argument " "1"" of type '" "struct _dwg_TableRow *""'"); 
    }
    arg1 = (struct _dwg_TableRow *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_TABLE_CustomDataItem, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TableRow_customdata_items_set" "', argument " "2"" of type '" "Dwg_TABLE_CustomDataItem *""'"); 
    }
    arg2 = (Dwg_TABLE_CustomDataItem *)(argp2);
    if (arg1) (arg1)->customdata_items = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableRow_customdata_items_get) {
  {
    struct _dwg_TableRow *arg1 = (struct _dwg_TableRow *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_TABLE_CustomDataItem *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableRow_customdata_items_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableRow, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableRow_customdata_items_get" "', argument " "1"" of type '" "struct _dwg_TableRow *""'"); 
    }
    arg1 = (struct _dwg_TableRow *)(argp1);
    result = (Dwg_TABLE_CustomDataItem *) ((arg1)->customdata_items);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TABLE_CustomDataItem, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableRow_cellstyle_set) {
  {
    struct _dwg_TableRow *arg1 = (struct _dwg_TableRow *) 0 ;
    Dwg_CellStyle *arg2 = (Dwg_CellStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableRow_cellstyle_set(self,cellstyle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableRow, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableRow_cellstyle_set" "', argument " "1"" of type '" "struct _dwg_TableRow *""'"); 
    }
    arg1 = (struct _dwg_TableRow *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TableRow_cellstyle_set" "', argument " "2"" of type '" "Dwg_CellStyle *""'"); 
    }
    arg2 = (Dwg_CellStyle *)(argp2);
    if (arg1) (arg1)->cellstyle = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableRow_cellstyle_get) {
  {
    struct _dwg_TableRow *arg1 = (struct _dwg_TableRow *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_CellStyle *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableRow_cellstyle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableRow, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableRow_cellstyle_get" "', argument " "1"" of type '" "struct _dwg_TableRow *""'"); 
    }
    arg1 = (struct _dwg_TableRow *)(argp1);
    result = (Dwg_CellStyle *)& ((arg1)->cellstyle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_CellStyle, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableRow_style_id_set) {
  {
    struct _dwg_TableRow *arg1 = (struct _dwg_TableRow *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableRow_style_id_set(self,style_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableRow, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableRow_style_id_set" "', argument " "1"" of type '" "struct _dwg_TableRow *""'"); 
    }
    arg1 = (struct _dwg_TableRow *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TableRow_style_id_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->style_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableRow_style_id_get) {
  {
    struct _dwg_TableRow *arg1 = (struct _dwg_TableRow *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableRow_style_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableRow, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableRow_style_id_get" "', argument " "1"" of type '" "struct _dwg_TableRow *""'"); 
    }
    arg1 = (struct _dwg_TableRow *)(argp1);
    result = (BITCODE_BL) ((arg1)->style_id);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableRow_height_set) {
  {
    struct _dwg_TableRow *arg1 = (struct _dwg_TableRow *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableRow_height_set(self,height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableRow, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableRow_height_set" "', argument " "1"" of type '" "struct _dwg_TableRow *""'"); 
    }
    arg1 = (struct _dwg_TableRow *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TableRow_height_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableRow_height_get) {
  {
    struct _dwg_TableRow *arg1 = (struct _dwg_TableRow *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableRow_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableRow, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableRow_height_get" "', argument " "1"" of type '" "struct _dwg_TableRow *""'"); 
    }
    arg1 = (struct _dwg_TableRow *)(argp1);
    result = (BITCODE_BL) ((arg1)->height);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_TableRow) {
  {
    int argvi = 0;
    struct _dwg_TableRow *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_TableRow();");
    }
    result = (struct _dwg_TableRow *)calloc(1, sizeof(struct _dwg_TableRow));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TableRow, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_TableRow) {
  {
    struct _dwg_TableRow *arg1 = (struct _dwg_TableRow *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_TableRow(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableRow, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_TableRow" "', argument " "1"" of type '" "struct _dwg_TableRow *""'"); 
    }
    arg1 = (struct _dwg_TableRow *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableDataColumn_parent_set) {
  {
    struct _dwg_TableDataColumn *arg1 = (struct _dwg_TableDataColumn *) 0 ;
    struct _dwg_LinkedTableData *arg2 = (struct _dwg_LinkedTableData *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableDataColumn_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableDataColumn, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableDataColumn_parent_set" "', argument " "1"" of type '" "struct _dwg_TableDataColumn *""'"); 
    }
    arg1 = (struct _dwg_TableDataColumn *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_LinkedTableData, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TableDataColumn_parent_set" "', argument " "2"" of type '" "struct _dwg_LinkedTableData *""'"); 
    }
    arg2 = (struct _dwg_LinkedTableData *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableDataColumn_parent_get) {
  {
    struct _dwg_TableDataColumn *arg1 = (struct _dwg_TableDataColumn *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_LinkedTableData *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableDataColumn_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableDataColumn, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableDataColumn_parent_get" "', argument " "1"" of type '" "struct _dwg_TableDataColumn *""'"); 
    }
    arg1 = (struct _dwg_TableDataColumn *)(argp1);
    result = (struct _dwg_LinkedTableData *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_LinkedTableData, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableDataColumn_name_set) {
  {
    struct _dwg_TableDataColumn *arg1 = (struct _dwg_TableDataColumn *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableDataColumn_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableDataColumn, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableDataColumn_name_set" "', argument " "1"" of type '" "struct _dwg_TableDataColumn *""'"); 
    }
    arg1 = (struct _dwg_TableDataColumn *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TableDataColumn_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableDataColumn_name_get) {
  {
    struct _dwg_TableDataColumn *arg1 = (struct _dwg_TableDataColumn *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableDataColumn_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableDataColumn, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableDataColumn_name_get" "', argument " "1"" of type '" "struct _dwg_TableDataColumn *""'"); 
    }
    arg1 = (struct _dwg_TableDataColumn *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableDataColumn_custom_data_set) {
  {
    struct _dwg_TableDataColumn *arg1 = (struct _dwg_TableDataColumn *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableDataColumn_custom_data_set(self,custom_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableDataColumn, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableDataColumn_custom_data_set" "', argument " "1"" of type '" "struct _dwg_TableDataColumn *""'"); 
    }
    arg1 = (struct _dwg_TableDataColumn *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TableDataColumn_custom_data_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->custom_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableDataColumn_custom_data_get) {
  {
    struct _dwg_TableDataColumn *arg1 = (struct _dwg_TableDataColumn *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableDataColumn_custom_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableDataColumn, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableDataColumn_custom_data_get" "', argument " "1"" of type '" "struct _dwg_TableDataColumn *""'"); 
    }
    arg1 = (struct _dwg_TableDataColumn *)(argp1);
    result = (BITCODE_BL) ((arg1)->custom_data);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableDataColumn_cellstyle_set) {
  {
    struct _dwg_TableDataColumn *arg1 = (struct _dwg_TableDataColumn *) 0 ;
    Dwg_CellStyle *arg2 = (Dwg_CellStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableDataColumn_cellstyle_set(self,cellstyle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableDataColumn, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableDataColumn_cellstyle_set" "', argument " "1"" of type '" "struct _dwg_TableDataColumn *""'"); 
    }
    arg1 = (struct _dwg_TableDataColumn *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TableDataColumn_cellstyle_set" "', argument " "2"" of type '" "Dwg_CellStyle *""'"); 
    }
    arg2 = (Dwg_CellStyle *)(argp2);
    if (arg1) (arg1)->cellstyle = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableDataColumn_cellstyle_get) {
  {
    struct _dwg_TableDataColumn *arg1 = (struct _dwg_TableDataColumn *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_CellStyle *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableDataColumn_cellstyle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableDataColumn, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableDataColumn_cellstyle_get" "', argument " "1"" of type '" "struct _dwg_TableDataColumn *""'"); 
    }
    arg1 = (struct _dwg_TableDataColumn *)(argp1);
    result = (Dwg_CellStyle *)& ((arg1)->cellstyle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_CellStyle, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableDataColumn_cellstyle_id_set) {
  {
    struct _dwg_TableDataColumn *arg1 = (struct _dwg_TableDataColumn *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableDataColumn_cellstyle_id_set(self,cellstyle_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableDataColumn, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableDataColumn_cellstyle_id_set" "', argument " "1"" of type '" "struct _dwg_TableDataColumn *""'"); 
    }
    arg1 = (struct _dwg_TableDataColumn *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TableDataColumn_cellstyle_id_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->cellstyle_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableDataColumn_cellstyle_id_get) {
  {
    struct _dwg_TableDataColumn *arg1 = (struct _dwg_TableDataColumn *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableDataColumn_cellstyle_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableDataColumn, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableDataColumn_cellstyle_id_get" "', argument " "1"" of type '" "struct _dwg_TableDataColumn *""'"); 
    }
    arg1 = (struct _dwg_TableDataColumn *)(argp1);
    result = (BITCODE_BL) ((arg1)->cellstyle_id);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableDataColumn_width_set) {
  {
    struct _dwg_TableDataColumn *arg1 = (struct _dwg_TableDataColumn *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TableDataColumn_width_set(self,width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableDataColumn, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableDataColumn_width_set" "', argument " "1"" of type '" "struct _dwg_TableDataColumn *""'"); 
    }
    arg1 = (struct _dwg_TableDataColumn *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TableDataColumn_width_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TableDataColumn_width_get) {
  {
    struct _dwg_TableDataColumn *arg1 = (struct _dwg_TableDataColumn *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TableDataColumn_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableDataColumn, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TableDataColumn_width_get" "', argument " "1"" of type '" "struct _dwg_TableDataColumn *""'"); 
    }
    arg1 = (struct _dwg_TableDataColumn *)(argp1);
    result = (BITCODE_BL) ((arg1)->width);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_TableDataColumn) {
  {
    int argvi = 0;
    struct _dwg_TableDataColumn *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_TableDataColumn();");
    }
    result = (struct _dwg_TableDataColumn *)calloc(1, sizeof(struct _dwg_TableDataColumn));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TableDataColumn, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_TableDataColumn) {
  {
    struct _dwg_TableDataColumn *arg1 = (struct _dwg_TableDataColumn *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_TableDataColumn(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TableDataColumn, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_TableDataColumn" "', argument " "1"" of type '" "struct _dwg_TableDataColumn *""'"); 
    }
    arg1 = (struct _dwg_TableDataColumn *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LinkedTableData_num_cols_set) {
  {
    struct _dwg_LinkedTableData *arg1 = (struct _dwg_LinkedTableData *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LinkedTableData_num_cols_set(self,num_cols);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LinkedTableData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LinkedTableData_num_cols_set" "', argument " "1"" of type '" "struct _dwg_LinkedTableData *""'"); 
    }
    arg1 = (struct _dwg_LinkedTableData *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_LinkedTableData_num_cols_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_cols = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LinkedTableData_num_cols_get) {
  {
    struct _dwg_LinkedTableData *arg1 = (struct _dwg_LinkedTableData *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LinkedTableData_num_cols_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LinkedTableData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LinkedTableData_num_cols_get" "', argument " "1"" of type '" "struct _dwg_LinkedTableData *""'"); 
    }
    arg1 = (struct _dwg_LinkedTableData *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_cols);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LinkedTableData_cols_set) {
  {
    struct _dwg_LinkedTableData *arg1 = (struct _dwg_LinkedTableData *) 0 ;
    Dwg_TableDataColumn *arg2 = (Dwg_TableDataColumn *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LinkedTableData_cols_set(self,cols);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LinkedTableData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LinkedTableData_cols_set" "', argument " "1"" of type '" "struct _dwg_LinkedTableData *""'"); 
    }
    arg1 = (struct _dwg_LinkedTableData *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_TableDataColumn, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_LinkedTableData_cols_set" "', argument " "2"" of type '" "Dwg_TableDataColumn *""'"); 
    }
    arg2 = (Dwg_TableDataColumn *)(argp2);
    if (arg1) (arg1)->cols = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LinkedTableData_cols_get) {
  {
    struct _dwg_LinkedTableData *arg1 = (struct _dwg_LinkedTableData *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_TableDataColumn *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LinkedTableData_cols_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LinkedTableData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LinkedTableData_cols_get" "', argument " "1"" of type '" "struct _dwg_LinkedTableData *""'"); 
    }
    arg1 = (struct _dwg_LinkedTableData *)(argp1);
    result = (Dwg_TableDataColumn *) ((arg1)->cols);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TableDataColumn, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LinkedTableData_num_rows_set) {
  {
    struct _dwg_LinkedTableData *arg1 = (struct _dwg_LinkedTableData *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LinkedTableData_num_rows_set(self,num_rows);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LinkedTableData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LinkedTableData_num_rows_set" "', argument " "1"" of type '" "struct _dwg_LinkedTableData *""'"); 
    }
    arg1 = (struct _dwg_LinkedTableData *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_LinkedTableData_num_rows_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_rows = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LinkedTableData_num_rows_get) {
  {
    struct _dwg_LinkedTableData *arg1 = (struct _dwg_LinkedTableData *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LinkedTableData_num_rows_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LinkedTableData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LinkedTableData_num_rows_get" "', argument " "1"" of type '" "struct _dwg_LinkedTableData *""'"); 
    }
    arg1 = (struct _dwg_LinkedTableData *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_rows);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LinkedTableData_rows_set) {
  {
    struct _dwg_LinkedTableData *arg1 = (struct _dwg_LinkedTableData *) 0 ;
    Dwg_TableRow *arg2 = (Dwg_TableRow *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LinkedTableData_rows_set(self,rows);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LinkedTableData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LinkedTableData_rows_set" "', argument " "1"" of type '" "struct _dwg_LinkedTableData *""'"); 
    }
    arg1 = (struct _dwg_LinkedTableData *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_TableRow, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_LinkedTableData_rows_set" "', argument " "2"" of type '" "Dwg_TableRow *""'"); 
    }
    arg2 = (Dwg_TableRow *)(argp2);
    if (arg1) (arg1)->rows = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LinkedTableData_rows_get) {
  {
    struct _dwg_LinkedTableData *arg1 = (struct _dwg_LinkedTableData *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_TableRow *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LinkedTableData_rows_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LinkedTableData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LinkedTableData_rows_get" "', argument " "1"" of type '" "struct _dwg_LinkedTableData *""'"); 
    }
    arg1 = (struct _dwg_LinkedTableData *)(argp1);
    result = (Dwg_TableRow *) ((arg1)->rows);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TableRow, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LinkedTableData_num_field_refs_set) {
  {
    struct _dwg_LinkedTableData *arg1 = (struct _dwg_LinkedTableData *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LinkedTableData_num_field_refs_set(self,num_field_refs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LinkedTableData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LinkedTableData_num_field_refs_set" "', argument " "1"" of type '" "struct _dwg_LinkedTableData *""'"); 
    }
    arg1 = (struct _dwg_LinkedTableData *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_LinkedTableData_num_field_refs_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_field_refs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LinkedTableData_num_field_refs_get) {
  {
    struct _dwg_LinkedTableData *arg1 = (struct _dwg_LinkedTableData *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LinkedTableData_num_field_refs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LinkedTableData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LinkedTableData_num_field_refs_get" "', argument " "1"" of type '" "struct _dwg_LinkedTableData *""'"); 
    }
    arg1 = (struct _dwg_LinkedTableData *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_field_refs);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LinkedTableData_field_refs_set) {
  {
    struct _dwg_LinkedTableData *arg1 = (struct _dwg_LinkedTableData *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LinkedTableData_field_refs_set(self,field_refs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LinkedTableData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LinkedTableData_field_refs_set" "', argument " "1"" of type '" "struct _dwg_LinkedTableData *""'"); 
    }
    arg1 = (struct _dwg_LinkedTableData *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_LinkedTableData_field_refs_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->field_refs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LinkedTableData_field_refs_get) {
  {
    struct _dwg_LinkedTableData *arg1 = (struct _dwg_LinkedTableData *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LinkedTableData_field_refs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LinkedTableData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LinkedTableData_field_refs_get" "', argument " "1"" of type '" "struct _dwg_LinkedTableData *""'"); 
    }
    arg1 = (struct _dwg_LinkedTableData *)(argp1);
    result = (BITCODE_H *) ((arg1)->field_refs);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_LinkedTableData) {
  {
    int argvi = 0;
    struct _dwg_LinkedTableData *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_LinkedTableData();");
    }
    result = (struct _dwg_LinkedTableData *)calloc(1, sizeof(struct _dwg_LinkedTableData));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_LinkedTableData, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_LinkedTableData) {
  {
    struct _dwg_LinkedTableData *arg1 = (struct _dwg_LinkedTableData *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_LinkedTableData(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LinkedTableData, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_LinkedTableData" "', argument " "1"" of type '" "struct _dwg_LinkedTableData *""'"); 
    }
    arg1 = (struct _dwg_LinkedTableData *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FormattedTableMerged_parent_set) {
  {
    struct _dwg_FormattedTableMerged *arg1 = (struct _dwg_FormattedTableMerged *) 0 ;
    struct _dwg_FormattedTableData *arg2 = (struct _dwg_FormattedTableData *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_FormattedTableMerged_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FormattedTableMerged, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FormattedTableMerged_parent_set" "', argument " "1"" of type '" "struct _dwg_FormattedTableMerged *""'"); 
    }
    arg1 = (struct _dwg_FormattedTableMerged *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_FormattedTableData, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_FormattedTableMerged_parent_set" "', argument " "2"" of type '" "struct _dwg_FormattedTableData *""'"); 
    }
    arg2 = (struct _dwg_FormattedTableData *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FormattedTableMerged_parent_get) {
  {
    struct _dwg_FormattedTableMerged *arg1 = (struct _dwg_FormattedTableMerged *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_FormattedTableData *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_FormattedTableMerged_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FormattedTableMerged, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FormattedTableMerged_parent_get" "', argument " "1"" of type '" "struct _dwg_FormattedTableMerged *""'"); 
    }
    arg1 = (struct _dwg_FormattedTableMerged *)(argp1);
    result = (struct _dwg_FormattedTableData *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_FormattedTableData, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FormattedTableMerged_top_row_set) {
  {
    struct _dwg_FormattedTableMerged *arg1 = (struct _dwg_FormattedTableMerged *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_FormattedTableMerged_top_row_set(self,top_row);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FormattedTableMerged, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FormattedTableMerged_top_row_set" "', argument " "1"" of type '" "struct _dwg_FormattedTableMerged *""'"); 
    }
    arg1 = (struct _dwg_FormattedTableMerged *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_FormattedTableMerged_top_row_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->top_row = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FormattedTableMerged_top_row_get) {
  {
    struct _dwg_FormattedTableMerged *arg1 = (struct _dwg_FormattedTableMerged *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_FormattedTableMerged_top_row_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FormattedTableMerged, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FormattedTableMerged_top_row_get" "', argument " "1"" of type '" "struct _dwg_FormattedTableMerged *""'"); 
    }
    arg1 = (struct _dwg_FormattedTableMerged *)(argp1);
    result = (BITCODE_BL) ((arg1)->top_row);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FormattedTableMerged_left_col_set) {
  {
    struct _dwg_FormattedTableMerged *arg1 = (struct _dwg_FormattedTableMerged *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_FormattedTableMerged_left_col_set(self,left_col);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FormattedTableMerged, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FormattedTableMerged_left_col_set" "', argument " "1"" of type '" "struct _dwg_FormattedTableMerged *""'"); 
    }
    arg1 = (struct _dwg_FormattedTableMerged *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_FormattedTableMerged_left_col_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->left_col = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FormattedTableMerged_left_col_get) {
  {
    struct _dwg_FormattedTableMerged *arg1 = (struct _dwg_FormattedTableMerged *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_FormattedTableMerged_left_col_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FormattedTableMerged, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FormattedTableMerged_left_col_get" "', argument " "1"" of type '" "struct _dwg_FormattedTableMerged *""'"); 
    }
    arg1 = (struct _dwg_FormattedTableMerged *)(argp1);
    result = (BITCODE_BL) ((arg1)->left_col);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FormattedTableMerged_bottom_row_set) {
  {
    struct _dwg_FormattedTableMerged *arg1 = (struct _dwg_FormattedTableMerged *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_FormattedTableMerged_bottom_row_set(self,bottom_row);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FormattedTableMerged, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FormattedTableMerged_bottom_row_set" "', argument " "1"" of type '" "struct _dwg_FormattedTableMerged *""'"); 
    }
    arg1 = (struct _dwg_FormattedTableMerged *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_FormattedTableMerged_bottom_row_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bottom_row = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FormattedTableMerged_bottom_row_get) {
  {
    struct _dwg_FormattedTableMerged *arg1 = (struct _dwg_FormattedTableMerged *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_FormattedTableMerged_bottom_row_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FormattedTableMerged, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FormattedTableMerged_bottom_row_get" "', argument " "1"" of type '" "struct _dwg_FormattedTableMerged *""'"); 
    }
    arg1 = (struct _dwg_FormattedTableMerged *)(argp1);
    result = (BITCODE_BL) ((arg1)->bottom_row);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FormattedTableMerged_right_col_set) {
  {
    struct _dwg_FormattedTableMerged *arg1 = (struct _dwg_FormattedTableMerged *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_FormattedTableMerged_right_col_set(self,right_col);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FormattedTableMerged, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FormattedTableMerged_right_col_set" "', argument " "1"" of type '" "struct _dwg_FormattedTableMerged *""'"); 
    }
    arg1 = (struct _dwg_FormattedTableMerged *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_FormattedTableMerged_right_col_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->right_col = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FormattedTableMerged_right_col_get) {
  {
    struct _dwg_FormattedTableMerged *arg1 = (struct _dwg_FormattedTableMerged *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_FormattedTableMerged_right_col_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FormattedTableMerged, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FormattedTableMerged_right_col_get" "', argument " "1"" of type '" "struct _dwg_FormattedTableMerged *""'"); 
    }
    arg1 = (struct _dwg_FormattedTableMerged *)(argp1);
    result = (BITCODE_BL) ((arg1)->right_col);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_FormattedTableMerged) {
  {
    int argvi = 0;
    struct _dwg_FormattedTableMerged *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_FormattedTableMerged();");
    }
    result = (struct _dwg_FormattedTableMerged *)calloc(1, sizeof(struct _dwg_FormattedTableMerged));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_FormattedTableMerged, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_FormattedTableMerged) {
  {
    struct _dwg_FormattedTableMerged *arg1 = (struct _dwg_FormattedTableMerged *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_FormattedTableMerged(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FormattedTableMerged, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_FormattedTableMerged" "', argument " "1"" of type '" "struct _dwg_FormattedTableMerged *""'"); 
    }
    arg1 = (struct _dwg_FormattedTableMerged *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FormattedTableData_parent_set) {
  {
    struct _dwg_FormattedTableData *arg1 = (struct _dwg_FormattedTableData *) 0 ;
    struct _dwg_object_TABLECONTENT *arg2 = (struct _dwg_object_TABLECONTENT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_FormattedTableData_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FormattedTableData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FormattedTableData_parent_set" "', argument " "1"" of type '" "struct _dwg_FormattedTableData *""'"); 
    }
    arg1 = (struct _dwg_FormattedTableData *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_TABLECONTENT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_FormattedTableData_parent_set" "', argument " "2"" of type '" "struct _dwg_object_TABLECONTENT *""'"); 
    }
    arg2 = (struct _dwg_object_TABLECONTENT *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FormattedTableData_parent_get) {
  {
    struct _dwg_FormattedTableData *arg1 = (struct _dwg_FormattedTableData *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_TABLECONTENT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_FormattedTableData_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FormattedTableData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FormattedTableData_parent_get" "', argument " "1"" of type '" "struct _dwg_FormattedTableData *""'"); 
    }
    arg1 = (struct _dwg_FormattedTableData *)(argp1);
    result = (struct _dwg_object_TABLECONTENT *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_TABLECONTENT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FormattedTableData_cellstyle_set) {
  {
    struct _dwg_FormattedTableData *arg1 = (struct _dwg_FormattedTableData *) 0 ;
    Dwg_CellStyle *arg2 = (Dwg_CellStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_FormattedTableData_cellstyle_set(self,cellstyle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FormattedTableData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FormattedTableData_cellstyle_set" "', argument " "1"" of type '" "struct _dwg_FormattedTableData *""'"); 
    }
    arg1 = (struct _dwg_FormattedTableData *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_FormattedTableData_cellstyle_set" "', argument " "2"" of type '" "Dwg_CellStyle *""'"); 
    }
    arg2 = (Dwg_CellStyle *)(argp2);
    if (arg1) (arg1)->cellstyle = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FormattedTableData_cellstyle_get) {
  {
    struct _dwg_FormattedTableData *arg1 = (struct _dwg_FormattedTableData *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_CellStyle *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_FormattedTableData_cellstyle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FormattedTableData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FormattedTableData_cellstyle_get" "', argument " "1"" of type '" "struct _dwg_FormattedTableData *""'"); 
    }
    arg1 = (struct _dwg_FormattedTableData *)(argp1);
    result = (Dwg_CellStyle *)& ((arg1)->cellstyle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_CellStyle, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FormattedTableData_num_merged_cells_set) {
  {
    struct _dwg_FormattedTableData *arg1 = (struct _dwg_FormattedTableData *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_FormattedTableData_num_merged_cells_set(self,num_merged_cells);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FormattedTableData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FormattedTableData_num_merged_cells_set" "', argument " "1"" of type '" "struct _dwg_FormattedTableData *""'"); 
    }
    arg1 = (struct _dwg_FormattedTableData *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_FormattedTableData_num_merged_cells_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_merged_cells = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FormattedTableData_num_merged_cells_get) {
  {
    struct _dwg_FormattedTableData *arg1 = (struct _dwg_FormattedTableData *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_FormattedTableData_num_merged_cells_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FormattedTableData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FormattedTableData_num_merged_cells_get" "', argument " "1"" of type '" "struct _dwg_FormattedTableData *""'"); 
    }
    arg1 = (struct _dwg_FormattedTableData *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_merged_cells);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FormattedTableData_merged_cells_set) {
  {
    struct _dwg_FormattedTableData *arg1 = (struct _dwg_FormattedTableData *) 0 ;
    Dwg_FormattedTableMerged *arg2 = (Dwg_FormattedTableMerged *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_FormattedTableData_merged_cells_set(self,merged_cells);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FormattedTableData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FormattedTableData_merged_cells_set" "', argument " "1"" of type '" "struct _dwg_FormattedTableData *""'"); 
    }
    arg1 = (struct _dwg_FormattedTableData *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_FormattedTableMerged, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_FormattedTableData_merged_cells_set" "', argument " "2"" of type '" "Dwg_FormattedTableMerged *""'"); 
    }
    arg2 = (Dwg_FormattedTableMerged *)(argp2);
    if (arg1) (arg1)->merged_cells = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FormattedTableData_merged_cells_get) {
  {
    struct _dwg_FormattedTableData *arg1 = (struct _dwg_FormattedTableData *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_FormattedTableMerged *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_FormattedTableData_merged_cells_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FormattedTableData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FormattedTableData_merged_cells_get" "', argument " "1"" of type '" "struct _dwg_FormattedTableData *""'"); 
    }
    arg1 = (struct _dwg_FormattedTableData *)(argp1);
    result = (Dwg_FormattedTableMerged *) ((arg1)->merged_cells);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_FormattedTableMerged, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_FormattedTableData) {
  {
    int argvi = 0;
    struct _dwg_FormattedTableData *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_FormattedTableData();");
    }
    result = (struct _dwg_FormattedTableData *)calloc(1, sizeof(struct _dwg_FormattedTableData));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_FormattedTableData, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_FormattedTableData) {
  {
    struct _dwg_FormattedTableData *arg1 = (struct _dwg_FormattedTableData *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_FormattedTableData(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FormattedTableData, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_FormattedTableData" "', argument " "1"" of type '" "struct _dwg_FormattedTableData *""'"); 
    }
    arg1 = (struct _dwg_FormattedTableData *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLECONTENT_parent_set) {
  {
    struct _dwg_object_TABLECONTENT *arg1 = (struct _dwg_object_TABLECONTENT *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TABLECONTENT_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLECONTENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLECONTENT_parent_set" "', argument " "1"" of type '" "struct _dwg_object_TABLECONTENT *""'"); 
    }
    arg1 = (struct _dwg_object_TABLECONTENT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_TABLECONTENT_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLECONTENT_parent_get) {
  {
    struct _dwg_object_TABLECONTENT *arg1 = (struct _dwg_object_TABLECONTENT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TABLECONTENT_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLECONTENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLECONTENT_parent_get" "', argument " "1"" of type '" "struct _dwg_object_TABLECONTENT *""'"); 
    }
    arg1 = (struct _dwg_object_TABLECONTENT *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLECONTENT_ldata_set) {
  {
    struct _dwg_object_TABLECONTENT *arg1 = (struct _dwg_object_TABLECONTENT *) 0 ;
    Dwg_LinkedData *arg2 = (Dwg_LinkedData *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TABLECONTENT_ldata_set(self,ldata);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLECONTENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLECONTENT_ldata_set" "', argument " "1"" of type '" "struct _dwg_object_TABLECONTENT *""'"); 
    }
    arg1 = (struct _dwg_object_TABLECONTENT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_LinkedData, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_TABLECONTENT_ldata_set" "', argument " "2"" of type '" "Dwg_LinkedData *""'"); 
    }
    arg2 = (Dwg_LinkedData *)(argp2);
    if (arg1) (arg1)->ldata = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLECONTENT_ldata_get) {
  {
    struct _dwg_object_TABLECONTENT *arg1 = (struct _dwg_object_TABLECONTENT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_LinkedData *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TABLECONTENT_ldata_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLECONTENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLECONTENT_ldata_get" "', argument " "1"" of type '" "struct _dwg_object_TABLECONTENT *""'"); 
    }
    arg1 = (struct _dwg_object_TABLECONTENT *)(argp1);
    result = (Dwg_LinkedData *)& ((arg1)->ldata);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_LinkedData, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLECONTENT_tdata_set) {
  {
    struct _dwg_object_TABLECONTENT *arg1 = (struct _dwg_object_TABLECONTENT *) 0 ;
    Dwg_LinkedTableData *arg2 = (Dwg_LinkedTableData *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TABLECONTENT_tdata_set(self,tdata);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLECONTENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLECONTENT_tdata_set" "', argument " "1"" of type '" "struct _dwg_object_TABLECONTENT *""'"); 
    }
    arg1 = (struct _dwg_object_TABLECONTENT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_LinkedTableData, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_TABLECONTENT_tdata_set" "', argument " "2"" of type '" "Dwg_LinkedTableData *""'"); 
    }
    arg2 = (Dwg_LinkedTableData *)(argp2);
    if (arg1) (arg1)->tdata = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLECONTENT_tdata_get) {
  {
    struct _dwg_object_TABLECONTENT *arg1 = (struct _dwg_object_TABLECONTENT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_LinkedTableData *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TABLECONTENT_tdata_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLECONTENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLECONTENT_tdata_get" "', argument " "1"" of type '" "struct _dwg_object_TABLECONTENT *""'"); 
    }
    arg1 = (struct _dwg_object_TABLECONTENT *)(argp1);
    result = (Dwg_LinkedTableData *)& ((arg1)->tdata);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_LinkedTableData, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLECONTENT_fdata_set) {
  {
    struct _dwg_object_TABLECONTENT *arg1 = (struct _dwg_object_TABLECONTENT *) 0 ;
    Dwg_FormattedTableData *arg2 = (Dwg_FormattedTableData *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TABLECONTENT_fdata_set(self,fdata);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLECONTENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLECONTENT_fdata_set" "', argument " "1"" of type '" "struct _dwg_object_TABLECONTENT *""'"); 
    }
    arg1 = (struct _dwg_object_TABLECONTENT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_FormattedTableData, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_TABLECONTENT_fdata_set" "', argument " "2"" of type '" "Dwg_FormattedTableData *""'"); 
    }
    arg2 = (Dwg_FormattedTableData *)(argp2);
    if (arg1) (arg1)->fdata = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLECONTENT_fdata_get) {
  {
    struct _dwg_object_TABLECONTENT *arg1 = (struct _dwg_object_TABLECONTENT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_FormattedTableData *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TABLECONTENT_fdata_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLECONTENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLECONTENT_fdata_get" "', argument " "1"" of type '" "struct _dwg_object_TABLECONTENT *""'"); 
    }
    arg1 = (struct _dwg_object_TABLECONTENT *)(argp1);
    result = (Dwg_FormattedTableData *)& ((arg1)->fdata);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_FormattedTableData, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLECONTENT_tablestyle_set) {
  {
    struct _dwg_object_TABLECONTENT *arg1 = (struct _dwg_object_TABLECONTENT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TABLECONTENT_tablestyle_set(self,tablestyle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLECONTENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLECONTENT_tablestyle_set" "', argument " "1"" of type '" "struct _dwg_object_TABLECONTENT *""'"); 
    }
    arg1 = (struct _dwg_object_TABLECONTENT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_TABLECONTENT_tablestyle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->tablestyle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLECONTENT_tablestyle_get) {
  {
    struct _dwg_object_TABLECONTENT *arg1 = (struct _dwg_object_TABLECONTENT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TABLECONTENT_tablestyle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLECONTENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLECONTENT_tablestyle_get" "', argument " "1"" of type '" "struct _dwg_object_TABLECONTENT *""'"); 
    }
    arg1 = (struct _dwg_object_TABLECONTENT *)(argp1);
    result = (BITCODE_H) ((arg1)->tablestyle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_TABLECONTENT) {
  {
    int argvi = 0;
    struct _dwg_object_TABLECONTENT *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_TABLECONTENT();");
    }
    result = (struct _dwg_object_TABLECONTENT *)calloc(1, sizeof(struct _dwg_object_TABLECONTENT));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_TABLECONTENT, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_TABLECONTENT) {
  {
    struct _dwg_object_TABLECONTENT *arg1 = (struct _dwg_object_TABLECONTENT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_TABLECONTENT(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLECONTENT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_TABLECONTENT" "', argument " "1"" of type '" "struct _dwg_object_TABLECONTENT *""'"); 
    }
    arg1 = (struct _dwg_object_TABLECONTENT *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_parent_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_parent_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_ldata_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    Dwg_LinkedData *arg2 = (Dwg_LinkedData *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_ldata_set(self,ldata);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_ldata_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_LinkedData, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_ldata_set" "', argument " "2"" of type '" "Dwg_LinkedData *""'"); 
    }
    arg2 = (Dwg_LinkedData *)(argp2);
    if (arg1) (arg1)->ldata = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_ldata_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_LinkedData *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_ldata_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_ldata_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (Dwg_LinkedData *)& ((arg1)->ldata);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_LinkedData, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_tdata_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    Dwg_LinkedTableData *arg2 = (Dwg_LinkedTableData *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_tdata_set(self,tdata);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_tdata_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_LinkedTableData, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_tdata_set" "', argument " "2"" of type '" "Dwg_LinkedTableData *""'"); 
    }
    arg2 = (Dwg_LinkedTableData *)(argp2);
    if (arg1) (arg1)->tdata = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_tdata_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_LinkedTableData *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_tdata_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_tdata_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (Dwg_LinkedTableData *)& ((arg1)->tdata);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_LinkedTableData, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_fdata_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    Dwg_FormattedTableData *arg2 = (Dwg_FormattedTableData *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_fdata_set(self,fdata);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_fdata_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_FormattedTableData, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_fdata_set" "', argument " "2"" of type '" "Dwg_FormattedTableData *""'"); 
    }
    arg2 = (Dwg_FormattedTableData *)(argp2);
    if (arg1) (arg1)->fdata = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_fdata_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_FormattedTableData *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_fdata_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_fdata_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (Dwg_FormattedTableData *)& ((arg1)->fdata);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_FormattedTableData, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_tablestyle_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_tablestyle_set(self,tablestyle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_tablestyle_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_tablestyle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->tablestyle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_tablestyle_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_tablestyle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_tablestyle_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_H) ((arg1)->tablestyle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_unknown_rc_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_unknown_rc_set(self,unknown_rc);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_unknown_rc_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_unknown_rc_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->unknown_rc = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_unknown_rc_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_unknown_rc_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_unknown_rc_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_RC) ((arg1)->unknown_rc);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_unknown_h_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_unknown_h_set(self,unknown_h);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_unknown_h_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_unknown_h_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->unknown_h = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_unknown_h_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_unknown_h_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_unknown_h_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_H) ((arg1)->unknown_h);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_unknown_bl_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_unknown_bl_set(self,unknown_bl);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_unknown_bl_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_unknown_bl_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_unknown_bl_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_unknown_bl_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_unknown_bl_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_unknown_b_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_unknown_b_set(self,unknown_b);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_unknown_b_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_unknown_b_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown_b = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_unknown_b_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_unknown_b_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_unknown_b_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown_b);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_unknown_bl1_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_unknown_bl1_set(self,unknown_bl1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_unknown_bl1_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_unknown_bl1_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_unknown_bl1_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_unknown_bl1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_unknown_bl1_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl1);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_ins_pt_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_ins_pt_set(self,ins_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_ins_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_ins_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ins_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_ins_pt_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_ins_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_ins_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ins_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_scale_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_scale_set(self,scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_scale_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_scale_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->scale = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_scale_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_scale_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->scale);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_scale_flag_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BB arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_scale_flag_set(self,scale_flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_scale_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_scale_flag_set" "', argument " "2"" of type '" "BITCODE_BB""'");
    } 
    arg2 = (BITCODE_BB)(val2);
    if (arg1) (arg1)->scale_flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_scale_flag_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BB result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_scale_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_scale_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BB) ((arg1)->scale_flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_rotation_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_rotation_set(self,rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_rotation_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_rotation_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_rotation_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_extrusion_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_extrusion_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_extrusion_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_extrusion_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_has_attribs_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_has_attribs_set(self,has_attribs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_has_attribs_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_has_attribs_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_attribs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_has_attribs_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_has_attribs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_has_attribs_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_B) ((arg1)->has_attribs);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_num_owned_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_num_owned_set(self,num_owned);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_num_owned_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_num_owned_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_owned = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_num_owned_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_num_owned_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_num_owned_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_owned);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_flag_for_table_value_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_flag_for_table_value_set(self,flag_for_table_value);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_flag_for_table_value_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_flag_for_table_value_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->flag_for_table_value = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_flag_for_table_value_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_flag_for_table_value_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_flag_for_table_value_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->flag_for_table_value);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_horiz_direction_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_horiz_direction_set(self,horiz_direction);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_horiz_direction_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_horiz_direction_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->horiz_direction = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_horiz_direction_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_horiz_direction_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_horiz_direction_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->horiz_direction);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_num_cols_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_num_cols_set(self,num_cols);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_num_cols_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_num_cols_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_cols = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_num_cols_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_num_cols_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_num_cols_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_cols);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_num_rows_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_num_rows_set(self,num_rows);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_num_rows_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_num_rows_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_rows = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_num_rows_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_num_rows_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_num_rows_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_rows);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_num_cells_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    unsigned long arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_num_cells_set(self,num_cells);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_num_cells_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_num_cells_set" "', argument " "2"" of type '" "unsigned long""'");
    } 
    arg2 = (unsigned long)(val2);
    if (arg1) (arg1)->num_cells = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_num_cells_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    unsigned long result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_num_cells_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_num_cells_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (unsigned long) ((arg1)->num_cells);
    ST(argvi) = SWIG_From_unsigned_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_col_widths_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_col_widths_set(self,col_widths);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_col_widths_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_col_widths_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->col_widths = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_col_widths_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_col_widths_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_col_widths_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BD *) ((arg1)->col_widths);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_row_heights_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_row_heights_set(self,row_heights);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_row_heights_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_row_heights_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->row_heights = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_row_heights_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_row_heights_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_row_heights_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BD *) ((arg1)->row_heights);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_cells_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    Dwg_TABLE_Cell *arg2 = (Dwg_TABLE_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_cells_set(self,cells);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_cells_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_TABLE_Cell, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_cells_set" "', argument " "2"" of type '" "Dwg_TABLE_Cell *""'"); 
    }
    arg2 = (Dwg_TABLE_Cell *)(argp2);
    if (arg1) (arg1)->cells = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_cells_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_TABLE_Cell *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_cells_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_cells_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (Dwg_TABLE_Cell *) ((arg1)->cells);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TABLE_Cell, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_has_table_overrides_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_has_table_overrides_set(self,has_table_overrides);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_has_table_overrides_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_has_table_overrides_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_table_overrides = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_has_table_overrides_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_has_table_overrides_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_has_table_overrides_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_B) ((arg1)->has_table_overrides);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_table_flag_override_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_table_flag_override_set(self,table_flag_override);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_table_flag_override_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_table_flag_override_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->table_flag_override = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_table_flag_override_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_table_flag_override_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_table_flag_override_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->table_flag_override);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_suppressed_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_suppressed_set(self,title_suppressed);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_suppressed_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_title_suppressed_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->title_suppressed = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_suppressed_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_suppressed_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_suppressed_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_B) ((arg1)->title_suppressed);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_suppressed_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_suppressed_set(self,header_suppressed);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_suppressed_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_header_suppressed_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->header_suppressed = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_suppressed_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_suppressed_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_suppressed_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_B) ((arg1)->header_suppressed);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_flow_direction_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_flow_direction_set(self,flow_direction);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_flow_direction_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_flow_direction_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->flow_direction = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_flow_direction_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_flow_direction_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_flow_direction_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->flow_direction);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_horiz_cell_margin_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_horiz_cell_margin_set(self,horiz_cell_margin);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_horiz_cell_margin_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_horiz_cell_margin_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->horiz_cell_margin = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_horiz_cell_margin_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_horiz_cell_margin_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_horiz_cell_margin_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->horiz_cell_margin);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_vert_cell_margin_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_vert_cell_margin_set(self,vert_cell_margin);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_vert_cell_margin_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_vert_cell_margin_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->vert_cell_margin = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_vert_cell_margin_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_vert_cell_margin_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_vert_cell_margin_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->vert_cell_margin);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_row_color_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_row_color_set(self,title_row_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_row_color_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_title_row_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->title_row_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_row_color_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_row_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_row_color_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->title_row_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_row_color_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_row_color_set(self,header_row_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_row_color_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_header_row_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->header_row_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_row_color_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_row_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_row_color_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->header_row_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_row_color_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_row_color_set(self,data_row_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_row_color_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_data_row_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->data_row_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_row_color_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_row_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_row_color_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->data_row_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_row_fill_none_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_row_fill_none_set(self,title_row_fill_none);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_row_fill_none_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_title_row_fill_none_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->title_row_fill_none = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_row_fill_none_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_row_fill_none_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_row_fill_none_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_B) ((arg1)->title_row_fill_none);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_row_fill_none_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_row_fill_none_set(self,header_row_fill_none);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_row_fill_none_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_header_row_fill_none_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->header_row_fill_none = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_row_fill_none_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_row_fill_none_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_row_fill_none_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_B) ((arg1)->header_row_fill_none);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_row_fill_none_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_row_fill_none_set(self,data_row_fill_none);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_row_fill_none_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_data_row_fill_none_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->data_row_fill_none = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_row_fill_none_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_row_fill_none_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_row_fill_none_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_B) ((arg1)->data_row_fill_none);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_row_fill_color_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_row_fill_color_set(self,title_row_fill_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_row_fill_color_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_title_row_fill_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->title_row_fill_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_row_fill_color_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_row_fill_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_row_fill_color_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->title_row_fill_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_row_fill_color_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_row_fill_color_set(self,header_row_fill_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_row_fill_color_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_header_row_fill_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->header_row_fill_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_row_fill_color_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_row_fill_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_row_fill_color_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->header_row_fill_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_row_fill_color_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_row_fill_color_set(self,data_row_fill_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_row_fill_color_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_data_row_fill_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->data_row_fill_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_row_fill_color_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_row_fill_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_row_fill_color_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->data_row_fill_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_row_alignment_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_row_alignment_set(self,title_row_alignment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_row_alignment_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_title_row_alignment_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->title_row_alignment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_row_alignment_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_row_alignment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_row_alignment_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->title_row_alignment);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_row_alignment_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_row_alignment_set(self,header_row_alignment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_row_alignment_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_header_row_alignment_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->header_row_alignment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_row_alignment_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_row_alignment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_row_alignment_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->header_row_alignment);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_row_alignment_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_row_alignment_set(self,data_row_alignment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_row_alignment_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_data_row_alignment_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->data_row_alignment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_row_alignment_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_row_alignment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_row_alignment_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->data_row_alignment);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_text_style_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_text_style_set(self,title_text_style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_text_style_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_title_text_style_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->title_text_style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_text_style_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_text_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_text_style_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_H) ((arg1)->title_text_style);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_text_style_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_text_style_set(self,header_text_style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_text_style_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_header_text_style_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->header_text_style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_text_style_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_text_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_text_style_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_H) ((arg1)->header_text_style);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_text_style_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_text_style_set(self,data_text_style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_text_style_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_data_text_style_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->data_text_style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_text_style_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_text_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_text_style_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_H) ((arg1)->data_text_style);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_row_height_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_row_height_set(self,title_row_height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_row_height_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_title_row_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->title_row_height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_row_height_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_row_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_row_height_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->title_row_height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_row_height_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_row_height_set(self,header_row_height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_row_height_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_header_row_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->header_row_height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_row_height_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_row_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_row_height_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->header_row_height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_row_height_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_row_height_set(self,data_row_height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_row_height_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_data_row_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->data_row_height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_row_height_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_row_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_row_height_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->data_row_height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_has_border_color_overrides_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_has_border_color_overrides_set(self,has_border_color_overrides);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_has_border_color_overrides_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_has_border_color_overrides_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_border_color_overrides = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_has_border_color_overrides_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_has_border_color_overrides_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_has_border_color_overrides_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_B) ((arg1)->has_border_color_overrides);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_border_color_overrides_flag_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_border_color_overrides_flag_set(self,border_color_overrides_flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_border_color_overrides_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_border_color_overrides_flag_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->border_color_overrides_flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_border_color_overrides_flag_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_border_color_overrides_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_border_color_overrides_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->border_color_overrides_flag);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_horiz_top_color_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_horiz_top_color_set(self,title_horiz_top_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_horiz_top_color_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_title_horiz_top_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->title_horiz_top_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_horiz_top_color_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_horiz_top_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_horiz_top_color_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->title_horiz_top_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_horiz_ins_color_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_horiz_ins_color_set(self,title_horiz_ins_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_horiz_ins_color_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_title_horiz_ins_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->title_horiz_ins_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_horiz_ins_color_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_horiz_ins_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_horiz_ins_color_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->title_horiz_ins_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_horiz_bottom_color_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_horiz_bottom_color_set(self,title_horiz_bottom_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_horiz_bottom_color_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_title_horiz_bottom_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->title_horiz_bottom_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_horiz_bottom_color_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_horiz_bottom_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_horiz_bottom_color_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->title_horiz_bottom_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_vert_left_color_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_vert_left_color_set(self,title_vert_left_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_vert_left_color_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_title_vert_left_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->title_vert_left_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_vert_left_color_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_vert_left_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_vert_left_color_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->title_vert_left_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_vert_ins_color_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_vert_ins_color_set(self,title_vert_ins_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_vert_ins_color_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_title_vert_ins_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->title_vert_ins_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_vert_ins_color_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_vert_ins_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_vert_ins_color_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->title_vert_ins_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_vert_right_color_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_vert_right_color_set(self,title_vert_right_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_vert_right_color_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_title_vert_right_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->title_vert_right_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_vert_right_color_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_vert_right_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_vert_right_color_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->title_vert_right_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_horiz_top_color_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_horiz_top_color_set(self,header_horiz_top_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_horiz_top_color_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_header_horiz_top_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->header_horiz_top_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_horiz_top_color_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_horiz_top_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_horiz_top_color_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->header_horiz_top_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_horiz_ins_color_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_horiz_ins_color_set(self,header_horiz_ins_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_horiz_ins_color_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_header_horiz_ins_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->header_horiz_ins_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_horiz_ins_color_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_horiz_ins_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_horiz_ins_color_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->header_horiz_ins_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_horiz_bottom_color_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_horiz_bottom_color_set(self,header_horiz_bottom_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_horiz_bottom_color_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_header_horiz_bottom_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->header_horiz_bottom_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_horiz_bottom_color_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_horiz_bottom_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_horiz_bottom_color_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->header_horiz_bottom_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_vert_left_color_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_vert_left_color_set(self,header_vert_left_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_vert_left_color_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_header_vert_left_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->header_vert_left_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_vert_left_color_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_vert_left_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_vert_left_color_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->header_vert_left_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_vert_ins_color_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_vert_ins_color_set(self,header_vert_ins_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_vert_ins_color_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_header_vert_ins_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->header_vert_ins_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_vert_ins_color_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_vert_ins_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_vert_ins_color_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->header_vert_ins_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_vert_right_color_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_vert_right_color_set(self,header_vert_right_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_vert_right_color_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_header_vert_right_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->header_vert_right_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_vert_right_color_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_vert_right_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_vert_right_color_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->header_vert_right_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_horiz_top_color_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_horiz_top_color_set(self,data_horiz_top_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_horiz_top_color_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_data_horiz_top_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->data_horiz_top_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_horiz_top_color_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_horiz_top_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_horiz_top_color_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->data_horiz_top_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_horiz_ins_color_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_horiz_ins_color_set(self,data_horiz_ins_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_horiz_ins_color_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_data_horiz_ins_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->data_horiz_ins_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_horiz_ins_color_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_horiz_ins_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_horiz_ins_color_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->data_horiz_ins_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_horiz_bottom_color_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_horiz_bottom_color_set(self,data_horiz_bottom_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_horiz_bottom_color_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_data_horiz_bottom_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->data_horiz_bottom_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_horiz_bottom_color_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_horiz_bottom_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_horiz_bottom_color_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->data_horiz_bottom_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_vert_left_color_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_vert_left_color_set(self,data_vert_left_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_vert_left_color_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_data_vert_left_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->data_vert_left_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_vert_left_color_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_vert_left_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_vert_left_color_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->data_vert_left_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_vert_ins_color_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_vert_ins_color_set(self,data_vert_ins_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_vert_ins_color_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_data_vert_ins_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->data_vert_ins_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_vert_ins_color_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_vert_ins_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_vert_ins_color_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->data_vert_ins_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_vert_right_color_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_vert_right_color_set(self,data_vert_right_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_vert_right_color_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_data_vert_right_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->data_vert_right_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_vert_right_color_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_vert_right_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_vert_right_color_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->data_vert_right_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_has_border_lineweight_overrides_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_has_border_lineweight_overrides_set(self,has_border_lineweight_overrides);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_has_border_lineweight_overrides_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_has_border_lineweight_overrides_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_border_lineweight_overrides = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_has_border_lineweight_overrides_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_has_border_lineweight_overrides_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_has_border_lineweight_overrides_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_B) ((arg1)->has_border_lineweight_overrides);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_border_lineweight_overrides_flag_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_border_lineweight_overrides_flag_set(self,border_lineweight_overrides_flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_border_lineweight_overrides_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_border_lineweight_overrides_flag_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->border_lineweight_overrides_flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_border_lineweight_overrides_flag_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_border_lineweight_overrides_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_border_lineweight_overrides_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->border_lineweight_overrides_flag);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_horiz_top_linewt_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_horiz_top_linewt_set(self,title_horiz_top_linewt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_horiz_top_linewt_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_title_horiz_top_linewt_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->title_horiz_top_linewt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_horiz_top_linewt_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_horiz_top_linewt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_horiz_top_linewt_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->title_horiz_top_linewt);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_horiz_ins_linewt_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_horiz_ins_linewt_set(self,title_horiz_ins_linewt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_horiz_ins_linewt_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_title_horiz_ins_linewt_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->title_horiz_ins_linewt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_horiz_ins_linewt_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_horiz_ins_linewt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_horiz_ins_linewt_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->title_horiz_ins_linewt);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_horiz_bottom_linewt_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_horiz_bottom_linewt_set(self,title_horiz_bottom_linewt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_horiz_bottom_linewt_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_title_horiz_bottom_linewt_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->title_horiz_bottom_linewt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_horiz_bottom_linewt_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_horiz_bottom_linewt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_horiz_bottom_linewt_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->title_horiz_bottom_linewt);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_vert_left_linewt_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_vert_left_linewt_set(self,title_vert_left_linewt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_vert_left_linewt_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_title_vert_left_linewt_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->title_vert_left_linewt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_vert_left_linewt_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_vert_left_linewt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_vert_left_linewt_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->title_vert_left_linewt);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_vert_ins_linewt_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_vert_ins_linewt_set(self,title_vert_ins_linewt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_vert_ins_linewt_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_title_vert_ins_linewt_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->title_vert_ins_linewt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_vert_ins_linewt_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_vert_ins_linewt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_vert_ins_linewt_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->title_vert_ins_linewt);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_vert_right_linewt_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_vert_right_linewt_set(self,title_vert_right_linewt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_vert_right_linewt_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_title_vert_right_linewt_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->title_vert_right_linewt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_vert_right_linewt_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_vert_right_linewt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_vert_right_linewt_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->title_vert_right_linewt);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_horiz_top_linewt_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_horiz_top_linewt_set(self,header_horiz_top_linewt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_horiz_top_linewt_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_header_horiz_top_linewt_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->header_horiz_top_linewt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_horiz_top_linewt_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_horiz_top_linewt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_horiz_top_linewt_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->header_horiz_top_linewt);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_horiz_ins_linewt_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_horiz_ins_linewt_set(self,header_horiz_ins_linewt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_horiz_ins_linewt_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_header_horiz_ins_linewt_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->header_horiz_ins_linewt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_horiz_ins_linewt_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_horiz_ins_linewt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_horiz_ins_linewt_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->header_horiz_ins_linewt);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_horiz_bottom_linewt_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_horiz_bottom_linewt_set(self,header_horiz_bottom_linewt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_horiz_bottom_linewt_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_header_horiz_bottom_linewt_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->header_horiz_bottom_linewt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_horiz_bottom_linewt_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_horiz_bottom_linewt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_horiz_bottom_linewt_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->header_horiz_bottom_linewt);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_vert_left_linewt_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_vert_left_linewt_set(self,header_vert_left_linewt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_vert_left_linewt_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_header_vert_left_linewt_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->header_vert_left_linewt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_vert_left_linewt_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_vert_left_linewt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_vert_left_linewt_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->header_vert_left_linewt);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_vert_ins_linewt_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_vert_ins_linewt_set(self,header_vert_ins_linewt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_vert_ins_linewt_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_header_vert_ins_linewt_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->header_vert_ins_linewt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_vert_ins_linewt_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_vert_ins_linewt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_vert_ins_linewt_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->header_vert_ins_linewt);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_vert_right_linewt_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_vert_right_linewt_set(self,header_vert_right_linewt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_vert_right_linewt_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_header_vert_right_linewt_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->header_vert_right_linewt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_vert_right_linewt_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_vert_right_linewt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_vert_right_linewt_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->header_vert_right_linewt);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_horiz_top_linewt_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_horiz_top_linewt_set(self,data_horiz_top_linewt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_horiz_top_linewt_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_data_horiz_top_linewt_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->data_horiz_top_linewt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_horiz_top_linewt_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_horiz_top_linewt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_horiz_top_linewt_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->data_horiz_top_linewt);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_horiz_ins_linewt_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_horiz_ins_linewt_set(self,data_horiz_ins_linewt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_horiz_ins_linewt_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_data_horiz_ins_linewt_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->data_horiz_ins_linewt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_horiz_ins_linewt_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_horiz_ins_linewt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_horiz_ins_linewt_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->data_horiz_ins_linewt);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_horiz_bottom_linewt_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_horiz_bottom_linewt_set(self,data_horiz_bottom_linewt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_horiz_bottom_linewt_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_data_horiz_bottom_linewt_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->data_horiz_bottom_linewt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_horiz_bottom_linewt_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_horiz_bottom_linewt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_horiz_bottom_linewt_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->data_horiz_bottom_linewt);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_vert_left_linewt_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_vert_left_linewt_set(self,data_vert_left_linewt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_vert_left_linewt_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_data_vert_left_linewt_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->data_vert_left_linewt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_vert_left_linewt_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_vert_left_linewt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_vert_left_linewt_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->data_vert_left_linewt);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_vert_ins_linewt_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_vert_ins_linewt_set(self,data_vert_ins_linewt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_vert_ins_linewt_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_data_vert_ins_linewt_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->data_vert_ins_linewt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_vert_ins_linewt_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_vert_ins_linewt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_vert_ins_linewt_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->data_vert_ins_linewt);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_vert_right_linewt_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_vert_right_linewt_set(self,data_vert_right_linewt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_vert_right_linewt_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_data_vert_right_linewt_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->data_vert_right_linewt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_vert_right_linewt_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_vert_right_linewt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_vert_right_linewt_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->data_vert_right_linewt);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_has_border_visibility_overrides_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_has_border_visibility_overrides_set(self,has_border_visibility_overrides);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_has_border_visibility_overrides_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_has_border_visibility_overrides_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_border_visibility_overrides = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_has_border_visibility_overrides_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_has_border_visibility_overrides_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_has_border_visibility_overrides_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_B) ((arg1)->has_border_visibility_overrides);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_border_visibility_overrides_flag_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_border_visibility_overrides_flag_set(self,border_visibility_overrides_flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_border_visibility_overrides_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_border_visibility_overrides_flag_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->border_visibility_overrides_flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_border_visibility_overrides_flag_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_border_visibility_overrides_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_border_visibility_overrides_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->border_visibility_overrides_flag);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_horiz_top_visibility_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_horiz_top_visibility_set(self,title_horiz_top_visibility);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_horiz_top_visibility_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_title_horiz_top_visibility_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->title_horiz_top_visibility = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_horiz_top_visibility_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_horiz_top_visibility_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_horiz_top_visibility_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->title_horiz_top_visibility);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_horiz_ins_visibility_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_horiz_ins_visibility_set(self,title_horiz_ins_visibility);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_horiz_ins_visibility_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_title_horiz_ins_visibility_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->title_horiz_ins_visibility = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_horiz_ins_visibility_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_horiz_ins_visibility_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_horiz_ins_visibility_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->title_horiz_ins_visibility);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_horiz_bottom_visibility_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_horiz_bottom_visibility_set(self,title_horiz_bottom_visibility);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_horiz_bottom_visibility_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_title_horiz_bottom_visibility_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->title_horiz_bottom_visibility = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_horiz_bottom_visibility_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_horiz_bottom_visibility_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_horiz_bottom_visibility_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->title_horiz_bottom_visibility);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_vert_left_visibility_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_vert_left_visibility_set(self,title_vert_left_visibility);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_vert_left_visibility_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_title_vert_left_visibility_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->title_vert_left_visibility = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_vert_left_visibility_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_vert_left_visibility_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_vert_left_visibility_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->title_vert_left_visibility);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_vert_ins_visibility_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_vert_ins_visibility_set(self,title_vert_ins_visibility);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_vert_ins_visibility_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_title_vert_ins_visibility_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->title_vert_ins_visibility = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_vert_ins_visibility_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_vert_ins_visibility_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_vert_ins_visibility_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->title_vert_ins_visibility);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_vert_right_visibility_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_vert_right_visibility_set(self,title_vert_right_visibility);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_vert_right_visibility_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_title_vert_right_visibility_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->title_vert_right_visibility = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_vert_right_visibility_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_vert_right_visibility_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_vert_right_visibility_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->title_vert_right_visibility);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_horiz_top_visibility_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_horiz_top_visibility_set(self,header_horiz_top_visibility);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_horiz_top_visibility_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_header_horiz_top_visibility_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->header_horiz_top_visibility = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_horiz_top_visibility_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_horiz_top_visibility_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_horiz_top_visibility_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->header_horiz_top_visibility);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_horiz_ins_visibility_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_horiz_ins_visibility_set(self,header_horiz_ins_visibility);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_horiz_ins_visibility_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_header_horiz_ins_visibility_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->header_horiz_ins_visibility = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_horiz_ins_visibility_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_horiz_ins_visibility_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_horiz_ins_visibility_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->header_horiz_ins_visibility);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_horiz_bottom_visibility_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_horiz_bottom_visibility_set(self,header_horiz_bottom_visibility);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_horiz_bottom_visibility_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_header_horiz_bottom_visibility_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->header_horiz_bottom_visibility = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_horiz_bottom_visibility_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_horiz_bottom_visibility_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_horiz_bottom_visibility_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->header_horiz_bottom_visibility);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_vert_left_visibility_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_vert_left_visibility_set(self,header_vert_left_visibility);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_vert_left_visibility_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_header_vert_left_visibility_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->header_vert_left_visibility = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_vert_left_visibility_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_vert_left_visibility_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_vert_left_visibility_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->header_vert_left_visibility);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_vert_ins_visibility_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_vert_ins_visibility_set(self,header_vert_ins_visibility);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_vert_ins_visibility_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_header_vert_ins_visibility_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->header_vert_ins_visibility = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_vert_ins_visibility_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_vert_ins_visibility_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_vert_ins_visibility_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->header_vert_ins_visibility);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_vert_right_visibility_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_vert_right_visibility_set(self,header_vert_right_visibility);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_vert_right_visibility_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_header_vert_right_visibility_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->header_vert_right_visibility = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_vert_right_visibility_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_vert_right_visibility_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_vert_right_visibility_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->header_vert_right_visibility);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_horiz_top_visibility_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_horiz_top_visibility_set(self,data_horiz_top_visibility);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_horiz_top_visibility_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_data_horiz_top_visibility_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->data_horiz_top_visibility = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_horiz_top_visibility_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_horiz_top_visibility_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_horiz_top_visibility_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->data_horiz_top_visibility);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_horiz_ins_visibility_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_horiz_ins_visibility_set(self,data_horiz_ins_visibility);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_horiz_ins_visibility_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_data_horiz_ins_visibility_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->data_horiz_ins_visibility = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_horiz_ins_visibility_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_horiz_ins_visibility_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_horiz_ins_visibility_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->data_horiz_ins_visibility);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_horiz_bottom_visibility_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_horiz_bottom_visibility_set(self,data_horiz_bottom_visibility);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_horiz_bottom_visibility_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_data_horiz_bottom_visibility_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->data_horiz_bottom_visibility = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_horiz_bottom_visibility_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_horiz_bottom_visibility_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_horiz_bottom_visibility_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->data_horiz_bottom_visibility);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_vert_left_visibility_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_vert_left_visibility_set(self,data_vert_left_visibility);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_vert_left_visibility_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_data_vert_left_visibility_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->data_vert_left_visibility = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_vert_left_visibility_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_vert_left_visibility_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_vert_left_visibility_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->data_vert_left_visibility);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_vert_ins_visibility_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_vert_ins_visibility_set(self,data_vert_ins_visibility);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_vert_ins_visibility_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_data_vert_ins_visibility_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->data_vert_ins_visibility = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_vert_ins_visibility_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_vert_ins_visibility_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_vert_ins_visibility_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->data_vert_ins_visibility);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_vert_right_visibility_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_vert_right_visibility_set(self,data_vert_right_visibility);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_vert_right_visibility_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_data_vert_right_visibility_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->data_vert_right_visibility = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_vert_right_visibility_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_vert_right_visibility_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_vert_right_visibility_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->data_vert_right_visibility);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_block_header_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_block_header_set(self,block_header);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_block_header_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_block_header_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->block_header = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_block_header_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_block_header_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_block_header_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_H) ((arg1)->block_header);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_first_attrib_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_first_attrib_set(self,first_attrib);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_first_attrib_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_first_attrib_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->first_attrib = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_first_attrib_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_first_attrib_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_first_attrib_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_H) ((arg1)->first_attrib);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_last_attrib_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_last_attrib_set(self,last_attrib);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_last_attrib_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_last_attrib_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->last_attrib = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_last_attrib_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_last_attrib_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_last_attrib_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_H) ((arg1)->last_attrib);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_attribs_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_attribs_set(self,attribs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_attribs_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_attribs_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->attribs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_attribs_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_attribs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_attribs_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_H *) ((arg1)->attribs);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_seqend_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_seqend_set(self,seqend);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_seqend_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_seqend_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->seqend = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_seqend_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_seqend_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_seqend_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_H) ((arg1)->seqend);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_row_style_override_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_row_style_override_set(self,title_row_style_override);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_row_style_override_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_title_row_style_override_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->title_row_style_override = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_title_row_style_override_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_title_row_style_override_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_title_row_style_override_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_H) ((arg1)->title_row_style_override);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_row_style_override_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_row_style_override_set(self,header_row_style_override);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_row_style_override_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_header_row_style_override_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->header_row_style_override = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_header_row_style_override_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_header_row_style_override_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_header_row_style_override_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_H) ((arg1)->header_row_style_override);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_row_style_override_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_row_style_override_set(self,data_row_style_override);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_row_style_override_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_data_row_style_override_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->data_row_style_override = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_data_row_style_override_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_data_row_style_override_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_data_row_style_override_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_H) ((arg1)->data_row_style_override);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_unknown_bs_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_unknown_bs_set(self,unknown_bs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_unknown_bs_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_unknown_bs_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->unknown_bs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_unknown_bs_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_unknown_bs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_unknown_bs_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->unknown_bs);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_hor_dir_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_hor_dir_set(self,hor_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_hor_dir_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_hor_dir_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->hor_dir = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_hor_dir_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_hor_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_hor_dir_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->hor_dir);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_has_break_data_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_has_break_data_set(self,has_break_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_has_break_data_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_has_break_data_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->has_break_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_has_break_data_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_has_break_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_has_break_data_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->has_break_data);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_break_flag_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_break_flag_set(self,break_flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_break_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_break_flag_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->break_flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_break_flag_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_break_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_break_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->break_flag);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_break_flow_direction_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_break_flow_direction_set(self,break_flow_direction);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_break_flow_direction_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_break_flow_direction_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->break_flow_direction = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_break_flow_direction_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_break_flow_direction_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_break_flow_direction_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->break_flow_direction);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_break_spacing_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_break_spacing_set(self,break_spacing);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_break_spacing_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_break_spacing_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->break_spacing = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_break_spacing_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_break_spacing_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_break_spacing_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->break_spacing);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_break_unknown1_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_break_unknown1_set(self,break_unknown1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_break_unknown1_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_break_unknown1_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->break_unknown1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_break_unknown1_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_break_unknown1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_break_unknown1_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->break_unknown1);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_break_unknown2_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_break_unknown2_set(self,break_unknown2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_break_unknown2_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_break_unknown2_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->break_unknown2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_break_unknown2_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_break_unknown2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_break_unknown2_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->break_unknown2);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_num_break_heights_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_num_break_heights_set(self,num_break_heights);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_num_break_heights_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_num_break_heights_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_break_heights = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_num_break_heights_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_num_break_heights_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_num_break_heights_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_break_heights);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_break_heights_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    Dwg_TABLE_BreakHeight *arg2 = (Dwg_TABLE_BreakHeight *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_break_heights_set(self,break_heights);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_break_heights_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_TABLE_BreakHeight, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_break_heights_set" "', argument " "2"" of type '" "Dwg_TABLE_BreakHeight *""'"); 
    }
    arg2 = (Dwg_TABLE_BreakHeight *)(argp2);
    if (arg1) (arg1)->break_heights = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_break_heights_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_TABLE_BreakHeight *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_break_heights_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_break_heights_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (Dwg_TABLE_BreakHeight *) ((arg1)->break_heights);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TABLE_BreakHeight, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_num_break_rows_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_num_break_rows_set(self,num_break_rows);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_num_break_rows_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_TABLE_num_break_rows_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_break_rows = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_num_break_rows_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_num_break_rows_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_num_break_rows_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_break_rows);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_break_rows_set) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    Dwg_TABLE_BreakRow *arg2 = (Dwg_TABLE_BreakRow *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_break_rows_set(self,break_rows);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_break_rows_set" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_TABLE_BreakRow, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_TABLE_break_rows_set" "', argument " "2"" of type '" "Dwg_TABLE_BreakRow *""'"); 
    }
    arg2 = (Dwg_TABLE_BreakRow *)(argp2);
    if (arg1) (arg1)->break_rows = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_TABLE_break_rows_get) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_TABLE_BreakRow *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_TABLE_break_rows_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_TABLE_break_rows_get" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    result = (Dwg_TABLE_BreakRow *) ((arg1)->break_rows);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TABLE_BreakRow, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_TABLE) {
  {
    int argvi = 0;
    struct _dwg_entity_TABLE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_TABLE();");
    }
    result = (struct _dwg_entity_TABLE *)calloc(1, sizeof(struct _dwg_entity_TABLE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_TABLE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_TABLE) {
  {
    struct _dwg_entity_TABLE *arg1 = (struct _dwg_entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_TABLE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_TABLE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_TABLE" "', argument " "1"" of type '" "struct _dwg_entity_TABLE *""'"); 
    }
    arg1 = (struct _dwg_entity_TABLE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_CellStyle_parent_set) {
  {
    struct _dwg_TABLESTYLE_CellStyle *arg1 = (struct _dwg_TABLESTYLE_CellStyle *) 0 ;
    struct _dwg_object_TABLESTYLE *arg2 = (struct _dwg_object_TABLESTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_CellStyle_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_CellStyle_parent_set" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_CellStyle *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_TABLESTYLE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLESTYLE_CellStyle_parent_set" "', argument " "2"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg2 = (struct _dwg_object_TABLESTYLE *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_CellStyle_parent_get) {
  {
    struct _dwg_TABLESTYLE_CellStyle *arg1 = (struct _dwg_TABLESTYLE_CellStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_TABLESTYLE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_CellStyle_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_CellStyle_parent_get" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_CellStyle *)(argp1);
    result = (struct _dwg_object_TABLESTYLE *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_TABLESTYLE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_CellStyle_id_set) {
  {
    struct _dwg_TABLESTYLE_CellStyle *arg1 = (struct _dwg_TABLESTYLE_CellStyle *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_CellStyle_id_set(self,id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_CellStyle_id_set" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_CellStyle *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLESTYLE_CellStyle_id_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_CellStyle_id_get) {
  {
    struct _dwg_TABLESTYLE_CellStyle *arg1 = (struct _dwg_TABLESTYLE_CellStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_CellStyle_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_CellStyle_id_get" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_CellStyle *)(argp1);
    result = (BITCODE_BL) ((arg1)->id);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_CellStyle_type_set) {
  {
    struct _dwg_TABLESTYLE_CellStyle *arg1 = (struct _dwg_TABLESTYLE_CellStyle *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_CellStyle_type_set(self,type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_CellStyle_type_set" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_CellStyle *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLESTYLE_CellStyle_type_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_CellStyle_type_get) {
  {
    struct _dwg_TABLESTYLE_CellStyle *arg1 = (struct _dwg_TABLESTYLE_CellStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_CellStyle_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_CellStyle_type_get" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_CellStyle *)(argp1);
    result = (BITCODE_BL) ((arg1)->type);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_CellStyle_name_set) {
  {
    struct _dwg_TABLESTYLE_CellStyle *arg1 = (struct _dwg_TABLESTYLE_CellStyle *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_CellStyle_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_CellStyle_name_set" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_CellStyle *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLESTYLE_CellStyle_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_CellStyle_name_get) {
  {
    struct _dwg_TABLESTYLE_CellStyle *arg1 = (struct _dwg_TABLESTYLE_CellStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_CellStyle_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_CellStyle_name_get" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_CellStyle *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_CellStyle_cellstyle_set) {
  {
    struct _dwg_TABLESTYLE_CellStyle *arg1 = (struct _dwg_TABLESTYLE_CellStyle *) 0 ;
    struct _dwg_CellStyle *arg2 = (struct _dwg_CellStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_CellStyle_cellstyle_set(self,cellstyle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_CellStyle_cellstyle_set" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_CellStyle *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLESTYLE_CellStyle_cellstyle_set" "', argument " "2"" of type '" "struct _dwg_CellStyle *""'"); 
    }
    arg2 = (struct _dwg_CellStyle *)(argp2);
    if (arg1) (arg1)->cellstyle = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_CellStyle_cellstyle_get) {
  {
    struct _dwg_TABLESTYLE_CellStyle *arg1 = (struct _dwg_TABLESTYLE_CellStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_CellStyle *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_CellStyle_cellstyle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_CellStyle_cellstyle_get" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_CellStyle *)(argp1);
    result = (struct _dwg_CellStyle *)& ((arg1)->cellstyle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_CellStyle, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_TABLESTYLE_CellStyle) {
  {
    int argvi = 0;
    struct _dwg_TABLESTYLE_CellStyle *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_TABLESTYLE_CellStyle();");
    }
    result = (struct _dwg_TABLESTYLE_CellStyle *)calloc(1, sizeof(struct _dwg_TABLESTYLE_CellStyle));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TABLESTYLE_CellStyle, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_TABLESTYLE_CellStyle) {
  {
    struct _dwg_TABLESTYLE_CellStyle *arg1 = (struct _dwg_TABLESTYLE_CellStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_TABLESTYLE_CellStyle(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_CellStyle, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_TABLESTYLE_CellStyle" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_CellStyle *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_CellStyle *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_border_linewt_set) {
  {
    struct _dwg_TABLESTYLE_border *arg1 = (struct _dwg_TABLESTYLE_border *) 0 ;
    BITCODE_BSd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_border_linewt_set(self,linewt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_border, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_border_linewt_set" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_border *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_border *)(argp1);
    ecode2 = SWIG_AsVal_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLESTYLE_border_linewt_set" "', argument " "2"" of type '" "BITCODE_BSd""'");
    } 
    arg2 = (BITCODE_BSd)(val2);
    if (arg1) (arg1)->linewt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_border_linewt_get) {
  {
    struct _dwg_TABLESTYLE_border *arg1 = (struct _dwg_TABLESTYLE_border *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BSd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_border_linewt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_border, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_border_linewt_get" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_border *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_border *)(argp1);
    result = (BITCODE_BSd) ((arg1)->linewt);
    ST(argvi) = SWIG_From_short  SWIG_PERL_CALL_ARGS_1((short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_border_visible_set) {
  {
    struct _dwg_TABLESTYLE_border *arg1 = (struct _dwg_TABLESTYLE_border *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_border_visible_set(self,visible);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_border, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_border_visible_set" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_border *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_border *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLESTYLE_border_visible_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->visible = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_border_visible_get) {
  {
    struct _dwg_TABLESTYLE_border *arg1 = (struct _dwg_TABLESTYLE_border *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_border_visible_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_border, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_border_visible_get" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_border *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_border *)(argp1);
    result = (BITCODE_B) ((arg1)->visible);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_border_color_set) {
  {
    struct _dwg_TABLESTYLE_border *arg1 = (struct _dwg_TABLESTYLE_border *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_border_color_set(self,color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_border, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_border_color_set" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_border *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_border *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLESTYLE_border_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_border_color_get) {
  {
    struct _dwg_TABLESTYLE_border *arg1 = (struct _dwg_TABLESTYLE_border *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_border_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_border, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_border_color_get" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_border *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_border *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_TABLESTYLE_border) {
  {
    int argvi = 0;
    struct _dwg_TABLESTYLE_border *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_TABLESTYLE_border();");
    }
    result = (struct _dwg_TABLESTYLE_border *)calloc(1, sizeof(struct _dwg_TABLESTYLE_border));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TABLESTYLE_border, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_TABLESTYLE_border) {
  {
    struct _dwg_TABLESTYLE_border *arg1 = (struct _dwg_TABLESTYLE_border *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_TABLESTYLE_border(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_border, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_TABLESTYLE_border" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_border *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_border *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_rowstyles_parent_set) {
  {
    struct _dwg_TABLESTYLE_rowstyles *arg1 = (struct _dwg_TABLESTYLE_rowstyles *) 0 ;
    struct _dwg_object_TABLESTYLE *arg2 = (struct _dwg_object_TABLESTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_rowstyles_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_rowstyles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_rowstyles_parent_set" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_rowstyles *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_rowstyles *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_TABLESTYLE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLESTYLE_rowstyles_parent_set" "', argument " "2"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg2 = (struct _dwg_object_TABLESTYLE *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_rowstyles_parent_get) {
  {
    struct _dwg_TABLESTYLE_rowstyles *arg1 = (struct _dwg_TABLESTYLE_rowstyles *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_TABLESTYLE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_rowstyles_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_rowstyles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_rowstyles_parent_get" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_rowstyles *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_rowstyles *)(argp1);
    result = (struct _dwg_object_TABLESTYLE *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_TABLESTYLE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_rowstyles_text_style_set) {
  {
    struct _dwg_TABLESTYLE_rowstyles *arg1 = (struct _dwg_TABLESTYLE_rowstyles *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_rowstyles_text_style_set(self,text_style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_rowstyles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_rowstyles_text_style_set" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_rowstyles *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_rowstyles *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLESTYLE_rowstyles_text_style_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->text_style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_rowstyles_text_style_get) {
  {
    struct _dwg_TABLESTYLE_rowstyles *arg1 = (struct _dwg_TABLESTYLE_rowstyles *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_rowstyles_text_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_rowstyles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_rowstyles_text_style_get" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_rowstyles *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_rowstyles *)(argp1);
    result = (BITCODE_H) ((arg1)->text_style);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_rowstyles_text_height_set) {
  {
    struct _dwg_TABLESTYLE_rowstyles *arg1 = (struct _dwg_TABLESTYLE_rowstyles *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_rowstyles_text_height_set(self,text_height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_rowstyles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_rowstyles_text_height_set" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_rowstyles *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_rowstyles *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLESTYLE_rowstyles_text_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->text_height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_rowstyles_text_height_get) {
  {
    struct _dwg_TABLESTYLE_rowstyles *arg1 = (struct _dwg_TABLESTYLE_rowstyles *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_rowstyles_text_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_rowstyles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_rowstyles_text_height_get" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_rowstyles *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_rowstyles *)(argp1);
    result = (BITCODE_BD) ((arg1)->text_height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_rowstyles_text_alignment_set) {
  {
    struct _dwg_TABLESTYLE_rowstyles *arg1 = (struct _dwg_TABLESTYLE_rowstyles *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_rowstyles_text_alignment_set(self,text_alignment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_rowstyles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_rowstyles_text_alignment_set" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_rowstyles *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_rowstyles *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLESTYLE_rowstyles_text_alignment_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->text_alignment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_rowstyles_text_alignment_get) {
  {
    struct _dwg_TABLESTYLE_rowstyles *arg1 = (struct _dwg_TABLESTYLE_rowstyles *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_rowstyles_text_alignment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_rowstyles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_rowstyles_text_alignment_get" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_rowstyles *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_rowstyles *)(argp1);
    result = (BITCODE_BS) ((arg1)->text_alignment);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_rowstyles_text_color_set) {
  {
    struct _dwg_TABLESTYLE_rowstyles *arg1 = (struct _dwg_TABLESTYLE_rowstyles *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_rowstyles_text_color_set(self,text_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_rowstyles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_rowstyles_text_color_set" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_rowstyles *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_rowstyles *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLESTYLE_rowstyles_text_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->text_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_rowstyles_text_color_get) {
  {
    struct _dwg_TABLESTYLE_rowstyles *arg1 = (struct _dwg_TABLESTYLE_rowstyles *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_rowstyles_text_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_rowstyles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_rowstyles_text_color_get" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_rowstyles *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_rowstyles *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->text_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_rowstyles_fill_color_set) {
  {
    struct _dwg_TABLESTYLE_rowstyles *arg1 = (struct _dwg_TABLESTYLE_rowstyles *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_rowstyles_fill_color_set(self,fill_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_rowstyles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_rowstyles_fill_color_set" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_rowstyles *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_rowstyles *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLESTYLE_rowstyles_fill_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->fill_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_rowstyles_fill_color_get) {
  {
    struct _dwg_TABLESTYLE_rowstyles *arg1 = (struct _dwg_TABLESTYLE_rowstyles *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_rowstyles_fill_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_rowstyles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_rowstyles_fill_color_get" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_rowstyles *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_rowstyles *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->fill_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_rowstyles_has_bgcolor_set) {
  {
    struct _dwg_TABLESTYLE_rowstyles *arg1 = (struct _dwg_TABLESTYLE_rowstyles *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_rowstyles_has_bgcolor_set(self,has_bgcolor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_rowstyles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_rowstyles_has_bgcolor_set" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_rowstyles *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_rowstyles *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLESTYLE_rowstyles_has_bgcolor_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_bgcolor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_rowstyles_has_bgcolor_get) {
  {
    struct _dwg_TABLESTYLE_rowstyles *arg1 = (struct _dwg_TABLESTYLE_rowstyles *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_rowstyles_has_bgcolor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_rowstyles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_rowstyles_has_bgcolor_get" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_rowstyles *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_rowstyles *)(argp1);
    result = (BITCODE_B) ((arg1)->has_bgcolor);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_rowstyles_num_borders_set) {
  {
    struct _dwg_TABLESTYLE_rowstyles *arg1 = (struct _dwg_TABLESTYLE_rowstyles *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_rowstyles_num_borders_set(self,num_borders);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_rowstyles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_rowstyles_num_borders_set" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_rowstyles *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_rowstyles *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLESTYLE_rowstyles_num_borders_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_borders = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_rowstyles_num_borders_get) {
  {
    struct _dwg_TABLESTYLE_rowstyles *arg1 = (struct _dwg_TABLESTYLE_rowstyles *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_rowstyles_num_borders_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_rowstyles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_rowstyles_num_borders_get" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_rowstyles *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_rowstyles *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_borders);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_rowstyles_borders_set) {
  {
    struct _dwg_TABLESTYLE_rowstyles *arg1 = (struct _dwg_TABLESTYLE_rowstyles *) 0 ;
    Dwg_TABLESTYLE_border *arg2 = (Dwg_TABLESTYLE_border *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_rowstyles_borders_set(self,borders);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_rowstyles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_rowstyles_borders_set" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_rowstyles *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_rowstyles *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_TABLESTYLE_border, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLESTYLE_rowstyles_borders_set" "', argument " "2"" of type '" "Dwg_TABLESTYLE_border *""'"); 
    }
    arg2 = (Dwg_TABLESTYLE_border *)(argp2);
    if (arg1) (arg1)->borders = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_rowstyles_borders_get) {
  {
    struct _dwg_TABLESTYLE_rowstyles *arg1 = (struct _dwg_TABLESTYLE_rowstyles *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_TABLESTYLE_border *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_rowstyles_borders_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_rowstyles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_rowstyles_borders_get" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_rowstyles *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_rowstyles *)(argp1);
    result = (Dwg_TABLESTYLE_border *) ((arg1)->borders);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TABLESTYLE_border, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_rowstyles_data_type_set) {
  {
    struct _dwg_TABLESTYLE_rowstyles *arg1 = (struct _dwg_TABLESTYLE_rowstyles *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_rowstyles_data_type_set(self,data_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_rowstyles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_rowstyles_data_type_set" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_rowstyles *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_rowstyles *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLESTYLE_rowstyles_data_type_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->data_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_rowstyles_data_type_get) {
  {
    struct _dwg_TABLESTYLE_rowstyles *arg1 = (struct _dwg_TABLESTYLE_rowstyles *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_rowstyles_data_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_rowstyles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_rowstyles_data_type_get" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_rowstyles *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_rowstyles *)(argp1);
    result = (BITCODE_BL) ((arg1)->data_type);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_rowstyles_unit_type_set) {
  {
    struct _dwg_TABLESTYLE_rowstyles *arg1 = (struct _dwg_TABLESTYLE_rowstyles *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_rowstyles_unit_type_set(self,unit_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_rowstyles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_rowstyles_unit_type_set" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_rowstyles *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_rowstyles *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLESTYLE_rowstyles_unit_type_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unit_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_rowstyles_unit_type_get) {
  {
    struct _dwg_TABLESTYLE_rowstyles *arg1 = (struct _dwg_TABLESTYLE_rowstyles *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_rowstyles_unit_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_rowstyles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_rowstyles_unit_type_get" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_rowstyles *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_rowstyles *)(argp1);
    result = (BITCODE_BL) ((arg1)->unit_type);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_rowstyles_format_string_set) {
  {
    struct _dwg_TABLESTYLE_rowstyles *arg1 = (struct _dwg_TABLESTYLE_rowstyles *) 0 ;
    BITCODE_TU arg2 = (BITCODE_TU) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_rowstyles_format_string_set(self,format_string);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_rowstyles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_rowstyles_format_string_set" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_rowstyles *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_rowstyles *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_short, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLESTYLE_rowstyles_format_string_set" "', argument " "2"" of type '" "BITCODE_TU""'"); 
    }
    arg2 = (BITCODE_TU)(argp2);
    if (arg1) (arg1)->format_string = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLESTYLE_rowstyles_format_string_get) {
  {
    struct _dwg_TABLESTYLE_rowstyles *arg1 = (struct _dwg_TABLESTYLE_rowstyles *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TU result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLESTYLE_rowstyles_format_string_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_rowstyles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLESTYLE_rowstyles_format_string_get" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_rowstyles *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_rowstyles *)(argp1);
    result = (BITCODE_TU) ((arg1)->format_string);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_short, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_TABLESTYLE_rowstyles) {
  {
    int argvi = 0;
    struct _dwg_TABLESTYLE_rowstyles *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_TABLESTYLE_rowstyles();");
    }
    result = (struct _dwg_TABLESTYLE_rowstyles *)calloc(1, sizeof(struct _dwg_TABLESTYLE_rowstyles));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TABLESTYLE_rowstyles, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_TABLESTYLE_rowstyles) {
  {
    struct _dwg_TABLESTYLE_rowstyles *arg1 = (struct _dwg_TABLESTYLE_rowstyles *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_TABLESTYLE_rowstyles(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLESTYLE_rowstyles, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_TABLESTYLE_rowstyles" "', argument " "1"" of type '" "struct _dwg_TABLESTYLE_rowstyles *""'"); 
    }
    arg1 = (struct _dwg_TABLESTYLE_rowstyles *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_parent_set) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_parent_set" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_TABLESTYLE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_parent_get) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_parent_get" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_class_version_set) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_TABLESTYLE_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_class_version_get) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_name_set) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_name_set" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_TABLESTYLE_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_name_get) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_name_get" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_flags_set) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_flags_set(self,flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_flags_set" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_TABLESTYLE_flags_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_flags_get) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_flags_get" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->flags);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_flow_direction_set) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_flow_direction_set(self,flow_direction);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_flow_direction_set" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_TABLESTYLE_flow_direction_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->flow_direction = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_flow_direction_get) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_flow_direction_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_flow_direction_get" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->flow_direction);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_horiz_cell_margin_set) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_horiz_cell_margin_set(self,horiz_cell_margin);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_horiz_cell_margin_set" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_TABLESTYLE_horiz_cell_margin_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->horiz_cell_margin = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_horiz_cell_margin_get) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_horiz_cell_margin_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_horiz_cell_margin_get" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->horiz_cell_margin);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_vert_cell_margin_set) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_vert_cell_margin_set(self,vert_cell_margin);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_vert_cell_margin_set" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_TABLESTYLE_vert_cell_margin_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->vert_cell_margin = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_vert_cell_margin_get) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_vert_cell_margin_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_vert_cell_margin_get" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->vert_cell_margin);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_is_title_suppressed_set) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_is_title_suppressed_set(self,is_title_suppressed);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_is_title_suppressed_set" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_TABLESTYLE_is_title_suppressed_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_title_suppressed = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_is_title_suppressed_get) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_is_title_suppressed_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_is_title_suppressed_get" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->is_title_suppressed);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_is_header_suppressed_set) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_is_header_suppressed_set(self,is_header_suppressed);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_is_header_suppressed_set" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_TABLESTYLE_is_header_suppressed_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_header_suppressed = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_is_header_suppressed_get) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_is_header_suppressed_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_is_header_suppressed_get" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->is_header_suppressed);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_unknown_rc_set) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_unknown_rc_set(self,unknown_rc);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_unknown_rc_set" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_TABLESTYLE_unknown_rc_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->unknown_rc = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_unknown_rc_get) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_unknown_rc_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_unknown_rc_get" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    result = (BITCODE_RC) ((arg1)->unknown_rc);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_unknown_bl1_set) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_unknown_bl1_set(self,unknown_bl1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_unknown_bl1_set" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_TABLESTYLE_unknown_bl1_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_unknown_bl1_get) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_unknown_bl1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_unknown_bl1_get" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl1);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_unknown_bl2_set) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_unknown_bl2_set(self,unknown_bl2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_unknown_bl2_set" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_TABLESTYLE_unknown_bl2_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_unknown_bl2_get) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_unknown_bl2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_unknown_bl2_get" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl2);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_cellstyle_set) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_cellstyle_set(self,cellstyle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_cellstyle_set" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_TABLESTYLE_cellstyle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->cellstyle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_cellstyle_get) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_cellstyle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_cellstyle_get" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    result = (BITCODE_H) ((arg1)->cellstyle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_sty_set) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    Dwg_TABLESTYLE_CellStyle *arg2 = (Dwg_TABLESTYLE_CellStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_sty_set(self,sty);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_sty_set" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_TABLESTYLE_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_TABLESTYLE_sty_set" "', argument " "2"" of type '" "Dwg_TABLESTYLE_CellStyle *""'"); 
    }
    arg2 = (Dwg_TABLESTYLE_CellStyle *)(argp2);
    if (arg1) (arg1)->sty = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_sty_get) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_TABLESTYLE_CellStyle *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_sty_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_sty_get" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    result = (Dwg_TABLESTYLE_CellStyle *)& ((arg1)->sty);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TABLESTYLE_CellStyle, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_numoverrides_set) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_numoverrides_set(self,numoverrides);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_numoverrides_set" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_TABLESTYLE_numoverrides_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->numoverrides = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_numoverrides_get) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_numoverrides_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_numoverrides_get" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->numoverrides);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_unknown_bl3_set) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_unknown_bl3_set(self,unknown_bl3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_unknown_bl3_set" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_TABLESTYLE_unknown_bl3_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl3 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_unknown_bl3_get) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_unknown_bl3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_unknown_bl3_get" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl3);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_ovr_set) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    Dwg_TABLESTYLE_CellStyle *arg2 = (Dwg_TABLESTYLE_CellStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_ovr_set(self,ovr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_ovr_set" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_TABLESTYLE_CellStyle, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_TABLESTYLE_ovr_set" "', argument " "2"" of type '" "Dwg_TABLESTYLE_CellStyle *""'"); 
    }
    arg2 = (Dwg_TABLESTYLE_CellStyle *)(argp2);
    if (arg1) (arg1)->ovr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_ovr_get) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_TABLESTYLE_CellStyle *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_ovr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_ovr_get" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    result = (Dwg_TABLESTYLE_CellStyle *)& ((arg1)->ovr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TABLESTYLE_CellStyle, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_num_rowstyles_set) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_num_rowstyles_set(self,num_rowstyles);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_num_rowstyles_set" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_TABLESTYLE_num_rowstyles_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_rowstyles = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_num_rowstyles_get) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_num_rowstyles_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_num_rowstyles_get" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_rowstyles);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_rowstyles_set) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    Dwg_TABLESTYLE_rowstyles *arg2 = (Dwg_TABLESTYLE_rowstyles *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_rowstyles_set(self,rowstyles);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_rowstyles_set" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_TABLESTYLE_rowstyles, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_TABLESTYLE_rowstyles_set" "', argument " "2"" of type '" "Dwg_TABLESTYLE_rowstyles *""'"); 
    }
    arg2 = (Dwg_TABLESTYLE_rowstyles *)(argp2);
    if (arg1) (arg1)->rowstyles = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLESTYLE_rowstyles_get) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_TABLESTYLE_rowstyles *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TABLESTYLE_rowstyles_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLESTYLE_rowstyles_get" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    result = (Dwg_TABLESTYLE_rowstyles *) ((arg1)->rowstyles);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TABLESTYLE_rowstyles, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_TABLESTYLE) {
  {
    int argvi = 0;
    struct _dwg_object_TABLESTYLE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_TABLESTYLE();");
    }
    result = (struct _dwg_object_TABLESTYLE *)calloc(1, sizeof(struct _dwg_object_TABLESTYLE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_TABLESTYLE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_TABLESTYLE) {
  {
    struct _dwg_object_TABLESTYLE *arg1 = (struct _dwg_object_TABLESTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_TABLESTYLE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLESTYLE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_TABLESTYLE" "', argument " "1"" of type '" "struct _dwg_object_TABLESTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_TABLESTYLE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_CELLSTYLEMAP_parent_set) {
  {
    struct _dwg_object_CELLSTYLEMAP *arg1 = (struct _dwg_object_CELLSTYLEMAP *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_CELLSTYLEMAP_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_CELLSTYLEMAP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_CELLSTYLEMAP_parent_set" "', argument " "1"" of type '" "struct _dwg_object_CELLSTYLEMAP *""'"); 
    }
    arg1 = (struct _dwg_object_CELLSTYLEMAP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_CELLSTYLEMAP_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_CELLSTYLEMAP_parent_get) {
  {
    struct _dwg_object_CELLSTYLEMAP *arg1 = (struct _dwg_object_CELLSTYLEMAP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_CELLSTYLEMAP_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_CELLSTYLEMAP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_CELLSTYLEMAP_parent_get" "', argument " "1"" of type '" "struct _dwg_object_CELLSTYLEMAP *""'"); 
    }
    arg1 = (struct _dwg_object_CELLSTYLEMAP *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_CELLSTYLEMAP_num_cells_set) {
  {
    struct _dwg_object_CELLSTYLEMAP *arg1 = (struct _dwg_object_CELLSTYLEMAP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_CELLSTYLEMAP_num_cells_set(self,num_cells);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_CELLSTYLEMAP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_CELLSTYLEMAP_num_cells_set" "', argument " "1"" of type '" "struct _dwg_object_CELLSTYLEMAP *""'"); 
    }
    arg1 = (struct _dwg_object_CELLSTYLEMAP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_CELLSTYLEMAP_num_cells_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_cells = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_CELLSTYLEMAP_num_cells_get) {
  {
    struct _dwg_object_CELLSTYLEMAP *arg1 = (struct _dwg_object_CELLSTYLEMAP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_CELLSTYLEMAP_num_cells_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_CELLSTYLEMAP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_CELLSTYLEMAP_num_cells_get" "', argument " "1"" of type '" "struct _dwg_object_CELLSTYLEMAP *""'"); 
    }
    arg1 = (struct _dwg_object_CELLSTYLEMAP *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_cells);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_CELLSTYLEMAP_cells_set) {
  {
    struct _dwg_object_CELLSTYLEMAP *arg1 = (struct _dwg_object_CELLSTYLEMAP *) 0 ;
    Dwg_TABLESTYLE_CellStyle *arg2 = (Dwg_TABLESTYLE_CellStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_CELLSTYLEMAP_cells_set(self,cells);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_CELLSTYLEMAP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_CELLSTYLEMAP_cells_set" "', argument " "1"" of type '" "struct _dwg_object_CELLSTYLEMAP *""'"); 
    }
    arg1 = (struct _dwg_object_CELLSTYLEMAP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_TABLESTYLE_CellStyle, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_CELLSTYLEMAP_cells_set" "', argument " "2"" of type '" "Dwg_TABLESTYLE_CellStyle *""'"); 
    }
    arg2 = (Dwg_TABLESTYLE_CellStyle *)(argp2);
    if (arg1) (arg1)->cells = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_CELLSTYLEMAP_cells_get) {
  {
    struct _dwg_object_CELLSTYLEMAP *arg1 = (struct _dwg_object_CELLSTYLEMAP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_TABLESTYLE_CellStyle *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_CELLSTYLEMAP_cells_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_CELLSTYLEMAP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_CELLSTYLEMAP_cells_get" "', argument " "1"" of type '" "struct _dwg_object_CELLSTYLEMAP *""'"); 
    }
    arg1 = (struct _dwg_object_CELLSTYLEMAP *)(argp1);
    result = (Dwg_TABLESTYLE_CellStyle *) ((arg1)->cells);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TABLESTYLE_CellStyle, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_CELLSTYLEMAP) {
  {
    int argvi = 0;
    struct _dwg_object_CELLSTYLEMAP *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_CELLSTYLEMAP();");
    }
    result = (struct _dwg_object_CELLSTYLEMAP *)calloc(1, sizeof(struct _dwg_object_CELLSTYLEMAP));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_CELLSTYLEMAP, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_CELLSTYLEMAP) {
  {
    struct _dwg_object_CELLSTYLEMAP *arg1 = (struct _dwg_object_CELLSTYLEMAP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_CELLSTYLEMAP(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_CELLSTYLEMAP, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_CELLSTYLEMAP" "', argument " "1"" of type '" "struct _dwg_object_CELLSTYLEMAP *""'"); 
    }
    arg1 = (struct _dwg_object_CELLSTYLEMAP *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLEGEOMETRY_Cell_parent_set) {
  {
    struct _dwg_TABLEGEOMETRY_Cell *arg1 = (struct _dwg_TABLEGEOMETRY_Cell *) 0 ;
    struct _dwg_object_TABLEGEOMETRY *arg2 = (struct _dwg_object_TABLEGEOMETRY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLEGEOMETRY_Cell_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLEGEOMETRY_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLEGEOMETRY_Cell_parent_set" "', argument " "1"" of type '" "struct _dwg_TABLEGEOMETRY_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLEGEOMETRY_Cell *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_TABLEGEOMETRY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLEGEOMETRY_Cell_parent_set" "', argument " "2"" of type '" "struct _dwg_object_TABLEGEOMETRY *""'"); 
    }
    arg2 = (struct _dwg_object_TABLEGEOMETRY *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLEGEOMETRY_Cell_parent_get) {
  {
    struct _dwg_TABLEGEOMETRY_Cell *arg1 = (struct _dwg_TABLEGEOMETRY_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_TABLEGEOMETRY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLEGEOMETRY_Cell_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLEGEOMETRY_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLEGEOMETRY_Cell_parent_get" "', argument " "1"" of type '" "struct _dwg_TABLEGEOMETRY_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLEGEOMETRY_Cell *)(argp1);
    result = (struct _dwg_object_TABLEGEOMETRY *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_TABLEGEOMETRY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLEGEOMETRY_Cell_geom_data_flag_set) {
  {
    struct _dwg_TABLEGEOMETRY_Cell *arg1 = (struct _dwg_TABLEGEOMETRY_Cell *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLEGEOMETRY_Cell_geom_data_flag_set(self,geom_data_flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLEGEOMETRY_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLEGEOMETRY_Cell_geom_data_flag_set" "', argument " "1"" of type '" "struct _dwg_TABLEGEOMETRY_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLEGEOMETRY_Cell *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLEGEOMETRY_Cell_geom_data_flag_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->geom_data_flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLEGEOMETRY_Cell_geom_data_flag_get) {
  {
    struct _dwg_TABLEGEOMETRY_Cell *arg1 = (struct _dwg_TABLEGEOMETRY_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLEGEOMETRY_Cell_geom_data_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLEGEOMETRY_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLEGEOMETRY_Cell_geom_data_flag_get" "', argument " "1"" of type '" "struct _dwg_TABLEGEOMETRY_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLEGEOMETRY_Cell *)(argp1);
    result = (BITCODE_BL) ((arg1)->geom_data_flag);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLEGEOMETRY_Cell_width_w_gap_set) {
  {
    struct _dwg_TABLEGEOMETRY_Cell *arg1 = (struct _dwg_TABLEGEOMETRY_Cell *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLEGEOMETRY_Cell_width_w_gap_set(self,width_w_gap);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLEGEOMETRY_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLEGEOMETRY_Cell_width_w_gap_set" "', argument " "1"" of type '" "struct _dwg_TABLEGEOMETRY_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLEGEOMETRY_Cell *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLEGEOMETRY_Cell_width_w_gap_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->width_w_gap = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLEGEOMETRY_Cell_width_w_gap_get) {
  {
    struct _dwg_TABLEGEOMETRY_Cell *arg1 = (struct _dwg_TABLEGEOMETRY_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLEGEOMETRY_Cell_width_w_gap_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLEGEOMETRY_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLEGEOMETRY_Cell_width_w_gap_get" "', argument " "1"" of type '" "struct _dwg_TABLEGEOMETRY_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLEGEOMETRY_Cell *)(argp1);
    result = (BITCODE_BD) ((arg1)->width_w_gap);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLEGEOMETRY_Cell_height_w_gap_set) {
  {
    struct _dwg_TABLEGEOMETRY_Cell *arg1 = (struct _dwg_TABLEGEOMETRY_Cell *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLEGEOMETRY_Cell_height_w_gap_set(self,height_w_gap);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLEGEOMETRY_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLEGEOMETRY_Cell_height_w_gap_set" "', argument " "1"" of type '" "struct _dwg_TABLEGEOMETRY_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLEGEOMETRY_Cell *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLEGEOMETRY_Cell_height_w_gap_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->height_w_gap = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLEGEOMETRY_Cell_height_w_gap_get) {
  {
    struct _dwg_TABLEGEOMETRY_Cell *arg1 = (struct _dwg_TABLEGEOMETRY_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLEGEOMETRY_Cell_height_w_gap_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLEGEOMETRY_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLEGEOMETRY_Cell_height_w_gap_get" "', argument " "1"" of type '" "struct _dwg_TABLEGEOMETRY_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLEGEOMETRY_Cell *)(argp1);
    result = (BITCODE_BD) ((arg1)->height_w_gap);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLEGEOMETRY_Cell_tablegeometry_set) {
  {
    struct _dwg_TABLEGEOMETRY_Cell *arg1 = (struct _dwg_TABLEGEOMETRY_Cell *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLEGEOMETRY_Cell_tablegeometry_set(self,tablegeometry);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLEGEOMETRY_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLEGEOMETRY_Cell_tablegeometry_set" "', argument " "1"" of type '" "struct _dwg_TABLEGEOMETRY_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLEGEOMETRY_Cell *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLEGEOMETRY_Cell_tablegeometry_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->tablegeometry = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLEGEOMETRY_Cell_tablegeometry_get) {
  {
    struct _dwg_TABLEGEOMETRY_Cell *arg1 = (struct _dwg_TABLEGEOMETRY_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLEGEOMETRY_Cell_tablegeometry_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLEGEOMETRY_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLEGEOMETRY_Cell_tablegeometry_get" "', argument " "1"" of type '" "struct _dwg_TABLEGEOMETRY_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLEGEOMETRY_Cell *)(argp1);
    result = (BITCODE_H) ((arg1)->tablegeometry);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLEGEOMETRY_Cell_num_geometry_set) {
  {
    struct _dwg_TABLEGEOMETRY_Cell *arg1 = (struct _dwg_TABLEGEOMETRY_Cell *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLEGEOMETRY_Cell_num_geometry_set(self,num_geometry);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLEGEOMETRY_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLEGEOMETRY_Cell_num_geometry_set" "', argument " "1"" of type '" "struct _dwg_TABLEGEOMETRY_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLEGEOMETRY_Cell *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_TABLEGEOMETRY_Cell_num_geometry_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_geometry = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLEGEOMETRY_Cell_num_geometry_get) {
  {
    struct _dwg_TABLEGEOMETRY_Cell *arg1 = (struct _dwg_TABLEGEOMETRY_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLEGEOMETRY_Cell_num_geometry_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLEGEOMETRY_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLEGEOMETRY_Cell_num_geometry_get" "', argument " "1"" of type '" "struct _dwg_TABLEGEOMETRY_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLEGEOMETRY_Cell *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_geometry);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLEGEOMETRY_Cell_geometry_set) {
  {
    struct _dwg_TABLEGEOMETRY_Cell *arg1 = (struct _dwg_TABLEGEOMETRY_Cell *) 0 ;
    Dwg_CellContentGeometry *arg2 = (Dwg_CellContentGeometry *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_TABLEGEOMETRY_Cell_geometry_set(self,geometry);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLEGEOMETRY_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLEGEOMETRY_Cell_geometry_set" "', argument " "1"" of type '" "struct _dwg_TABLEGEOMETRY_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLEGEOMETRY_Cell *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_CellContentGeometry, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_TABLEGEOMETRY_Cell_geometry_set" "', argument " "2"" of type '" "Dwg_CellContentGeometry *""'"); 
    }
    arg2 = (Dwg_CellContentGeometry *)(argp2);
    if (arg1) (arg1)->geometry = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_TABLEGEOMETRY_Cell_geometry_get) {
  {
    struct _dwg_TABLEGEOMETRY_Cell *arg1 = (struct _dwg_TABLEGEOMETRY_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_CellContentGeometry *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_TABLEGEOMETRY_Cell_geometry_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLEGEOMETRY_Cell, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_TABLEGEOMETRY_Cell_geometry_get" "', argument " "1"" of type '" "struct _dwg_TABLEGEOMETRY_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLEGEOMETRY_Cell *)(argp1);
    result = (Dwg_CellContentGeometry *) ((arg1)->geometry);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_CellContentGeometry, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_TABLEGEOMETRY_Cell) {
  {
    int argvi = 0;
    struct _dwg_TABLEGEOMETRY_Cell *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_TABLEGEOMETRY_Cell();");
    }
    result = (struct _dwg_TABLEGEOMETRY_Cell *)calloc(1, sizeof(struct _dwg_TABLEGEOMETRY_Cell));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TABLEGEOMETRY_Cell, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_TABLEGEOMETRY_Cell) {
  {
    struct _dwg_TABLEGEOMETRY_Cell *arg1 = (struct _dwg_TABLEGEOMETRY_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_TABLEGEOMETRY_Cell(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_TABLEGEOMETRY_Cell, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_TABLEGEOMETRY_Cell" "', argument " "1"" of type '" "struct _dwg_TABLEGEOMETRY_Cell *""'"); 
    }
    arg1 = (struct _dwg_TABLEGEOMETRY_Cell *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLEGEOMETRY_parent_set) {
  {
    struct _dwg_object_TABLEGEOMETRY *arg1 = (struct _dwg_object_TABLEGEOMETRY *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TABLEGEOMETRY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLEGEOMETRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLEGEOMETRY_parent_set" "', argument " "1"" of type '" "struct _dwg_object_TABLEGEOMETRY *""'"); 
    }
    arg1 = (struct _dwg_object_TABLEGEOMETRY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_TABLEGEOMETRY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLEGEOMETRY_parent_get) {
  {
    struct _dwg_object_TABLEGEOMETRY *arg1 = (struct _dwg_object_TABLEGEOMETRY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TABLEGEOMETRY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLEGEOMETRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLEGEOMETRY_parent_get" "', argument " "1"" of type '" "struct _dwg_object_TABLEGEOMETRY *""'"); 
    }
    arg1 = (struct _dwg_object_TABLEGEOMETRY *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLEGEOMETRY_numrows_set) {
  {
    struct _dwg_object_TABLEGEOMETRY *arg1 = (struct _dwg_object_TABLEGEOMETRY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TABLEGEOMETRY_numrows_set(self,numrows);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLEGEOMETRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLEGEOMETRY_numrows_set" "', argument " "1"" of type '" "struct _dwg_object_TABLEGEOMETRY *""'"); 
    }
    arg1 = (struct _dwg_object_TABLEGEOMETRY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_TABLEGEOMETRY_numrows_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->numrows = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLEGEOMETRY_numrows_get) {
  {
    struct _dwg_object_TABLEGEOMETRY *arg1 = (struct _dwg_object_TABLEGEOMETRY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TABLEGEOMETRY_numrows_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLEGEOMETRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLEGEOMETRY_numrows_get" "', argument " "1"" of type '" "struct _dwg_object_TABLEGEOMETRY *""'"); 
    }
    arg1 = (struct _dwg_object_TABLEGEOMETRY *)(argp1);
    result = (BITCODE_BL) ((arg1)->numrows);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLEGEOMETRY_numcols_set) {
  {
    struct _dwg_object_TABLEGEOMETRY *arg1 = (struct _dwg_object_TABLEGEOMETRY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TABLEGEOMETRY_numcols_set(self,numcols);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLEGEOMETRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLEGEOMETRY_numcols_set" "', argument " "1"" of type '" "struct _dwg_object_TABLEGEOMETRY *""'"); 
    }
    arg1 = (struct _dwg_object_TABLEGEOMETRY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_TABLEGEOMETRY_numcols_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->numcols = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLEGEOMETRY_numcols_get) {
  {
    struct _dwg_object_TABLEGEOMETRY *arg1 = (struct _dwg_object_TABLEGEOMETRY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TABLEGEOMETRY_numcols_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLEGEOMETRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLEGEOMETRY_numcols_get" "', argument " "1"" of type '" "struct _dwg_object_TABLEGEOMETRY *""'"); 
    }
    arg1 = (struct _dwg_object_TABLEGEOMETRY *)(argp1);
    result = (BITCODE_BL) ((arg1)->numcols);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLEGEOMETRY_num_cells_set) {
  {
    struct _dwg_object_TABLEGEOMETRY *arg1 = (struct _dwg_object_TABLEGEOMETRY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TABLEGEOMETRY_num_cells_set(self,num_cells);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLEGEOMETRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLEGEOMETRY_num_cells_set" "', argument " "1"" of type '" "struct _dwg_object_TABLEGEOMETRY *""'"); 
    }
    arg1 = (struct _dwg_object_TABLEGEOMETRY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_TABLEGEOMETRY_num_cells_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_cells = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLEGEOMETRY_num_cells_get) {
  {
    struct _dwg_object_TABLEGEOMETRY *arg1 = (struct _dwg_object_TABLEGEOMETRY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TABLEGEOMETRY_num_cells_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLEGEOMETRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLEGEOMETRY_num_cells_get" "', argument " "1"" of type '" "struct _dwg_object_TABLEGEOMETRY *""'"); 
    }
    arg1 = (struct _dwg_object_TABLEGEOMETRY *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_cells);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLEGEOMETRY_cells_set) {
  {
    struct _dwg_object_TABLEGEOMETRY *arg1 = (struct _dwg_object_TABLEGEOMETRY *) 0 ;
    Dwg_TABLEGEOMETRY_Cell *arg2 = (Dwg_TABLEGEOMETRY_Cell *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TABLEGEOMETRY_cells_set(self,cells);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLEGEOMETRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLEGEOMETRY_cells_set" "', argument " "1"" of type '" "struct _dwg_object_TABLEGEOMETRY *""'"); 
    }
    arg1 = (struct _dwg_object_TABLEGEOMETRY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_TABLEGEOMETRY_Cell, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_TABLEGEOMETRY_cells_set" "', argument " "2"" of type '" "Dwg_TABLEGEOMETRY_Cell *""'"); 
    }
    arg2 = (Dwg_TABLEGEOMETRY_Cell *)(argp2);
    if (arg1) (arg1)->cells = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TABLEGEOMETRY_cells_get) {
  {
    struct _dwg_object_TABLEGEOMETRY *arg1 = (struct _dwg_object_TABLEGEOMETRY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_TABLEGEOMETRY_Cell *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TABLEGEOMETRY_cells_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLEGEOMETRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TABLEGEOMETRY_cells_get" "', argument " "1"" of type '" "struct _dwg_object_TABLEGEOMETRY *""'"); 
    }
    arg1 = (struct _dwg_object_TABLEGEOMETRY *)(argp1);
    result = (Dwg_TABLEGEOMETRY_Cell *) ((arg1)->cells);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TABLEGEOMETRY_Cell, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_TABLEGEOMETRY) {
  {
    int argvi = 0;
    struct _dwg_object_TABLEGEOMETRY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_TABLEGEOMETRY();");
    }
    result = (struct _dwg_object_TABLEGEOMETRY *)calloc(1, sizeof(struct _dwg_object_TABLEGEOMETRY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_TABLEGEOMETRY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_TABLEGEOMETRY) {
  {
    struct _dwg_object_TABLEGEOMETRY *arg1 = (struct _dwg_object_TABLEGEOMETRY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_TABLEGEOMETRY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TABLEGEOMETRY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_TABLEGEOMETRY" "', argument " "1"" of type '" "struct _dwg_object_TABLEGEOMETRY *""'"); 
    }
    arg1 = (struct _dwg_object_TABLEGEOMETRY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UNDERLAYDEFINITION_parent_set) {
  {
    struct _dwg_abstractobject_UNDERLAYDEFINITION *arg1 = (struct _dwg_abstractobject_UNDERLAYDEFINITION *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_UNDERLAYDEFINITION_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractobject_UNDERLAYDEFINITION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UNDERLAYDEFINITION_parent_set" "', argument " "1"" of type '" "struct _dwg_abstractobject_UNDERLAYDEFINITION *""'"); 
    }
    arg1 = (struct _dwg_abstractobject_UNDERLAYDEFINITION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_UNDERLAYDEFINITION_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UNDERLAYDEFINITION_parent_get) {
  {
    struct _dwg_abstractobject_UNDERLAYDEFINITION *arg1 = (struct _dwg_abstractobject_UNDERLAYDEFINITION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_UNDERLAYDEFINITION_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractobject_UNDERLAYDEFINITION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UNDERLAYDEFINITION_parent_get" "', argument " "1"" of type '" "struct _dwg_abstractobject_UNDERLAYDEFINITION *""'"); 
    }
    arg1 = (struct _dwg_abstractobject_UNDERLAYDEFINITION *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UNDERLAYDEFINITION_filename_set) {
  {
    struct _dwg_abstractobject_UNDERLAYDEFINITION *arg1 = (struct _dwg_abstractobject_UNDERLAYDEFINITION *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_UNDERLAYDEFINITION_filename_set(self,filename);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractobject_UNDERLAYDEFINITION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UNDERLAYDEFINITION_filename_set" "', argument " "1"" of type '" "struct _dwg_abstractobject_UNDERLAYDEFINITION *""'"); 
    }
    arg1 = (struct _dwg_abstractobject_UNDERLAYDEFINITION *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_UNDERLAYDEFINITION_filename_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->filename);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->filename = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->filename = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UNDERLAYDEFINITION_filename_get) {
  {
    struct _dwg_abstractobject_UNDERLAYDEFINITION *arg1 = (struct _dwg_abstractobject_UNDERLAYDEFINITION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_UNDERLAYDEFINITION_filename_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractobject_UNDERLAYDEFINITION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UNDERLAYDEFINITION_filename_get" "', argument " "1"" of type '" "struct _dwg_abstractobject_UNDERLAYDEFINITION *""'"); 
    }
    arg1 = (struct _dwg_abstractobject_UNDERLAYDEFINITION *)(argp1);
    result = (BITCODE_TV) ((arg1)->filename);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UNDERLAYDEFINITION_name_set) {
  {
    struct _dwg_abstractobject_UNDERLAYDEFINITION *arg1 = (struct _dwg_abstractobject_UNDERLAYDEFINITION *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_UNDERLAYDEFINITION_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractobject_UNDERLAYDEFINITION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UNDERLAYDEFINITION_name_set" "', argument " "1"" of type '" "struct _dwg_abstractobject_UNDERLAYDEFINITION *""'"); 
    }
    arg1 = (struct _dwg_abstractobject_UNDERLAYDEFINITION *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_UNDERLAYDEFINITION_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UNDERLAYDEFINITION_name_get) {
  {
    struct _dwg_abstractobject_UNDERLAYDEFINITION *arg1 = (struct _dwg_abstractobject_UNDERLAYDEFINITION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_UNDERLAYDEFINITION_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractobject_UNDERLAYDEFINITION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UNDERLAYDEFINITION_name_get" "', argument " "1"" of type '" "struct _dwg_abstractobject_UNDERLAYDEFINITION *""'"); 
    }
    arg1 = (struct _dwg_abstractobject_UNDERLAYDEFINITION *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_UNDERLAYDEFINITION) {
  {
    int argvi = 0;
    struct _dwg_abstractobject_UNDERLAYDEFINITION *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_UNDERLAYDEFINITION();");
    }
    result = (struct _dwg_abstractobject_UNDERLAYDEFINITION *)calloc(1, sizeof(struct _dwg_abstractobject_UNDERLAYDEFINITION));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_abstractobject_UNDERLAYDEFINITION, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_UNDERLAYDEFINITION) {
  {
    struct _dwg_abstractobject_UNDERLAYDEFINITION *arg1 = (struct _dwg_abstractobject_UNDERLAYDEFINITION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_UNDERLAYDEFINITION(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractobject_UNDERLAYDEFINITION, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_UNDERLAYDEFINITION" "', argument " "1"" of type '" "struct _dwg_abstractobject_UNDERLAYDEFINITION *""'"); 
    }
    arg1 = (struct _dwg_abstractobject_UNDERLAYDEFINITION *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_UNDERLAY_parent_set) {
  {
    struct _dwg_abstractentity_UNDERLAY *arg1 = (struct _dwg_abstractentity_UNDERLAY *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_UNDERLAY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractentity_UNDERLAY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_UNDERLAY_parent_set" "', argument " "1"" of type '" "struct _dwg_abstractentity_UNDERLAY *""'"); 
    }
    arg1 = (struct _dwg_abstractentity_UNDERLAY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_UNDERLAY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_UNDERLAY_parent_get) {
  {
    struct _dwg_abstractentity_UNDERLAY *arg1 = (struct _dwg_abstractentity_UNDERLAY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_UNDERLAY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractentity_UNDERLAY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_UNDERLAY_parent_get" "', argument " "1"" of type '" "struct _dwg_abstractentity_UNDERLAY *""'"); 
    }
    arg1 = (struct _dwg_abstractentity_UNDERLAY *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_UNDERLAY_extrusion_set) {
  {
    struct _dwg_abstractentity_UNDERLAY *arg1 = (struct _dwg_abstractentity_UNDERLAY *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_UNDERLAY_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractentity_UNDERLAY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_UNDERLAY_extrusion_set" "', argument " "1"" of type '" "struct _dwg_abstractentity_UNDERLAY *""'"); 
    }
    arg1 = (struct _dwg_abstractentity_UNDERLAY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_UNDERLAY_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_UNDERLAY_extrusion_get) {
  {
    struct _dwg_abstractentity_UNDERLAY *arg1 = (struct _dwg_abstractentity_UNDERLAY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_UNDERLAY_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractentity_UNDERLAY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_UNDERLAY_extrusion_get" "', argument " "1"" of type '" "struct _dwg_abstractentity_UNDERLAY *""'"); 
    }
    arg1 = (struct _dwg_abstractentity_UNDERLAY *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_UNDERLAY_ins_pt_set) {
  {
    struct _dwg_abstractentity_UNDERLAY *arg1 = (struct _dwg_abstractentity_UNDERLAY *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_UNDERLAY_ins_pt_set(self,ins_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractentity_UNDERLAY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_UNDERLAY_ins_pt_set" "', argument " "1"" of type '" "struct _dwg_abstractentity_UNDERLAY *""'"); 
    }
    arg1 = (struct _dwg_abstractentity_UNDERLAY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_UNDERLAY_ins_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ins_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_UNDERLAY_ins_pt_get) {
  {
    struct _dwg_abstractentity_UNDERLAY *arg1 = (struct _dwg_abstractentity_UNDERLAY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_UNDERLAY_ins_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractentity_UNDERLAY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_UNDERLAY_ins_pt_get" "', argument " "1"" of type '" "struct _dwg_abstractentity_UNDERLAY *""'"); 
    }
    arg1 = (struct _dwg_abstractentity_UNDERLAY *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ins_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_UNDERLAY_scale_set) {
  {
    struct _dwg_abstractentity_UNDERLAY *arg1 = (struct _dwg_abstractentity_UNDERLAY *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_UNDERLAY_scale_set(self,scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractentity_UNDERLAY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_UNDERLAY_scale_set" "', argument " "1"" of type '" "struct _dwg_abstractentity_UNDERLAY *""'"); 
    }
    arg1 = (struct _dwg_abstractentity_UNDERLAY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_UNDERLAY_scale_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->scale = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_UNDERLAY_scale_get) {
  {
    struct _dwg_abstractentity_UNDERLAY *arg1 = (struct _dwg_abstractentity_UNDERLAY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_UNDERLAY_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractentity_UNDERLAY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_UNDERLAY_scale_get" "', argument " "1"" of type '" "struct _dwg_abstractentity_UNDERLAY *""'"); 
    }
    arg1 = (struct _dwg_abstractentity_UNDERLAY *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->scale);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_UNDERLAY_angle_set) {
  {
    struct _dwg_abstractentity_UNDERLAY *arg1 = (struct _dwg_abstractentity_UNDERLAY *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_UNDERLAY_angle_set(self,angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractentity_UNDERLAY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_UNDERLAY_angle_set" "', argument " "1"" of type '" "struct _dwg_abstractentity_UNDERLAY *""'"); 
    }
    arg1 = (struct _dwg_abstractentity_UNDERLAY *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_UNDERLAY_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_UNDERLAY_angle_get) {
  {
    struct _dwg_abstractentity_UNDERLAY *arg1 = (struct _dwg_abstractentity_UNDERLAY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_UNDERLAY_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractentity_UNDERLAY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_UNDERLAY_angle_get" "', argument " "1"" of type '" "struct _dwg_abstractentity_UNDERLAY *""'"); 
    }
    arg1 = (struct _dwg_abstractentity_UNDERLAY *)(argp1);
    result = (BITCODE_BD) ((arg1)->angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_UNDERLAY_flag_set) {
  {
    struct _dwg_abstractentity_UNDERLAY *arg1 = (struct _dwg_abstractentity_UNDERLAY *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_UNDERLAY_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractentity_UNDERLAY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_UNDERLAY_flag_set" "', argument " "1"" of type '" "struct _dwg_abstractentity_UNDERLAY *""'"); 
    }
    arg1 = (struct _dwg_abstractentity_UNDERLAY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_UNDERLAY_flag_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_UNDERLAY_flag_get) {
  {
    struct _dwg_abstractentity_UNDERLAY *arg1 = (struct _dwg_abstractentity_UNDERLAY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_UNDERLAY_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractentity_UNDERLAY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_UNDERLAY_flag_get" "', argument " "1"" of type '" "struct _dwg_abstractentity_UNDERLAY *""'"); 
    }
    arg1 = (struct _dwg_abstractentity_UNDERLAY *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_UNDERLAY_contrast_set) {
  {
    struct _dwg_abstractentity_UNDERLAY *arg1 = (struct _dwg_abstractentity_UNDERLAY *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_UNDERLAY_contrast_set(self,contrast);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractentity_UNDERLAY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_UNDERLAY_contrast_set" "', argument " "1"" of type '" "struct _dwg_abstractentity_UNDERLAY *""'"); 
    }
    arg1 = (struct _dwg_abstractentity_UNDERLAY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_UNDERLAY_contrast_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->contrast = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_UNDERLAY_contrast_get) {
  {
    struct _dwg_abstractentity_UNDERLAY *arg1 = (struct _dwg_abstractentity_UNDERLAY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_UNDERLAY_contrast_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractentity_UNDERLAY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_UNDERLAY_contrast_get" "', argument " "1"" of type '" "struct _dwg_abstractentity_UNDERLAY *""'"); 
    }
    arg1 = (struct _dwg_abstractentity_UNDERLAY *)(argp1);
    result = (BITCODE_RC) ((arg1)->contrast);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_UNDERLAY_fade_set) {
  {
    struct _dwg_abstractentity_UNDERLAY *arg1 = (struct _dwg_abstractentity_UNDERLAY *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_UNDERLAY_fade_set(self,fade);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractentity_UNDERLAY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_UNDERLAY_fade_set" "', argument " "1"" of type '" "struct _dwg_abstractentity_UNDERLAY *""'"); 
    }
    arg1 = (struct _dwg_abstractentity_UNDERLAY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_UNDERLAY_fade_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->fade = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_UNDERLAY_fade_get) {
  {
    struct _dwg_abstractentity_UNDERLAY *arg1 = (struct _dwg_abstractentity_UNDERLAY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_UNDERLAY_fade_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractentity_UNDERLAY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_UNDERLAY_fade_get" "', argument " "1"" of type '" "struct _dwg_abstractentity_UNDERLAY *""'"); 
    }
    arg1 = (struct _dwg_abstractentity_UNDERLAY *)(argp1);
    result = (BITCODE_RC) ((arg1)->fade);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_UNDERLAY_num_clip_verts_set) {
  {
    struct _dwg_abstractentity_UNDERLAY *arg1 = (struct _dwg_abstractentity_UNDERLAY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_UNDERLAY_num_clip_verts_set(self,num_clip_verts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractentity_UNDERLAY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_UNDERLAY_num_clip_verts_set" "', argument " "1"" of type '" "struct _dwg_abstractentity_UNDERLAY *""'"); 
    }
    arg1 = (struct _dwg_abstractentity_UNDERLAY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_UNDERLAY_num_clip_verts_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_clip_verts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_UNDERLAY_num_clip_verts_get) {
  {
    struct _dwg_abstractentity_UNDERLAY *arg1 = (struct _dwg_abstractentity_UNDERLAY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_UNDERLAY_num_clip_verts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractentity_UNDERLAY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_UNDERLAY_num_clip_verts_get" "', argument " "1"" of type '" "struct _dwg_abstractentity_UNDERLAY *""'"); 
    }
    arg1 = (struct _dwg_abstractentity_UNDERLAY *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_clip_verts);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_UNDERLAY_clip_verts_set) {
  {
    struct _dwg_abstractentity_UNDERLAY *arg1 = (struct _dwg_abstractentity_UNDERLAY *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_UNDERLAY_clip_verts_set(self,clip_verts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractentity_UNDERLAY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_UNDERLAY_clip_verts_set" "', argument " "1"" of type '" "struct _dwg_abstractentity_UNDERLAY *""'"); 
    }
    arg1 = (struct _dwg_abstractentity_UNDERLAY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_UNDERLAY_clip_verts_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->clip_verts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_UNDERLAY_clip_verts_get) {
  {
    struct _dwg_abstractentity_UNDERLAY *arg1 = (struct _dwg_abstractentity_UNDERLAY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_UNDERLAY_clip_verts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractentity_UNDERLAY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_UNDERLAY_clip_verts_get" "', argument " "1"" of type '" "struct _dwg_abstractentity_UNDERLAY *""'"); 
    }
    arg1 = (struct _dwg_abstractentity_UNDERLAY *)(argp1);
    result = (BITCODE_2RD *) ((arg1)->clip_verts);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_UNDERLAY_num_clip_inverts_set) {
  {
    struct _dwg_abstractentity_UNDERLAY *arg1 = (struct _dwg_abstractentity_UNDERLAY *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_UNDERLAY_num_clip_inverts_set(self,num_clip_inverts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractentity_UNDERLAY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_UNDERLAY_num_clip_inverts_set" "', argument " "1"" of type '" "struct _dwg_abstractentity_UNDERLAY *""'"); 
    }
    arg1 = (struct _dwg_abstractentity_UNDERLAY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_UNDERLAY_num_clip_inverts_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_clip_inverts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_UNDERLAY_num_clip_inverts_get) {
  {
    struct _dwg_abstractentity_UNDERLAY *arg1 = (struct _dwg_abstractentity_UNDERLAY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_UNDERLAY_num_clip_inverts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractentity_UNDERLAY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_UNDERLAY_num_clip_inverts_get" "', argument " "1"" of type '" "struct _dwg_abstractentity_UNDERLAY *""'"); 
    }
    arg1 = (struct _dwg_abstractentity_UNDERLAY *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_clip_inverts);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_UNDERLAY_clip_inverts_set) {
  {
    struct _dwg_abstractentity_UNDERLAY *arg1 = (struct _dwg_abstractentity_UNDERLAY *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_UNDERLAY_clip_inverts_set(self,clip_inverts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractentity_UNDERLAY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_UNDERLAY_clip_inverts_set" "', argument " "1"" of type '" "struct _dwg_abstractentity_UNDERLAY *""'"); 
    }
    arg1 = (struct _dwg_abstractentity_UNDERLAY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_UNDERLAY_clip_inverts_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->clip_inverts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_UNDERLAY_clip_inverts_get) {
  {
    struct _dwg_abstractentity_UNDERLAY *arg1 = (struct _dwg_abstractentity_UNDERLAY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_UNDERLAY_clip_inverts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractentity_UNDERLAY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_UNDERLAY_clip_inverts_get" "', argument " "1"" of type '" "struct _dwg_abstractentity_UNDERLAY *""'"); 
    }
    arg1 = (struct _dwg_abstractentity_UNDERLAY *)(argp1);
    result = (BITCODE_2RD *) ((arg1)->clip_inverts);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_UNDERLAY_definition_id_set) {
  {
    struct _dwg_abstractentity_UNDERLAY *arg1 = (struct _dwg_abstractentity_UNDERLAY *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_UNDERLAY_definition_id_set(self,definition_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractentity_UNDERLAY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_UNDERLAY_definition_id_set" "', argument " "1"" of type '" "struct _dwg_abstractentity_UNDERLAY *""'"); 
    }
    arg1 = (struct _dwg_abstractentity_UNDERLAY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_UNDERLAY_definition_id_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->definition_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_UNDERLAY_definition_id_get) {
  {
    struct _dwg_abstractentity_UNDERLAY *arg1 = (struct _dwg_abstractentity_UNDERLAY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_UNDERLAY_definition_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractentity_UNDERLAY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_UNDERLAY_definition_id_get" "', argument " "1"" of type '" "struct _dwg_abstractentity_UNDERLAY *""'"); 
    }
    arg1 = (struct _dwg_abstractentity_UNDERLAY *)(argp1);
    result = (BITCODE_H) ((arg1)->definition_id);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_UNDERLAY) {
  {
    int argvi = 0;
    struct _dwg_abstractentity_UNDERLAY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_UNDERLAY();");
    }
    result = (struct _dwg_abstractentity_UNDERLAY *)calloc(1, sizeof(struct _dwg_abstractentity_UNDERLAY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_abstractentity_UNDERLAY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_UNDERLAY) {
  {
    struct _dwg_abstractentity_UNDERLAY *arg1 = (struct _dwg_abstractentity_UNDERLAY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_UNDERLAY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractentity_UNDERLAY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_UNDERLAY" "', argument " "1"" of type '" "struct _dwg_abstractentity_UNDERLAY *""'"); 
    }
    arg1 = (struct _dwg_abstractentity_UNDERLAY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DBCOLOR_parent_set) {
  {
    struct _dwg_object_DBCOLOR *arg1 = (struct _dwg_object_DBCOLOR *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DBCOLOR_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DBCOLOR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DBCOLOR_parent_set" "', argument " "1"" of type '" "struct _dwg_object_DBCOLOR *""'"); 
    }
    arg1 = (struct _dwg_object_DBCOLOR *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DBCOLOR_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DBCOLOR_parent_get) {
  {
    struct _dwg_object_DBCOLOR *arg1 = (struct _dwg_object_DBCOLOR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DBCOLOR_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DBCOLOR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DBCOLOR_parent_get" "', argument " "1"" of type '" "struct _dwg_object_DBCOLOR *""'"); 
    }
    arg1 = (struct _dwg_object_DBCOLOR *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DBCOLOR_color_set) {
  {
    struct _dwg_object_DBCOLOR *arg1 = (struct _dwg_object_DBCOLOR *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DBCOLOR_color_set(self,color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DBCOLOR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DBCOLOR_color_set" "', argument " "1"" of type '" "struct _dwg_object_DBCOLOR *""'"); 
    }
    arg1 = (struct _dwg_object_DBCOLOR *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DBCOLOR_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DBCOLOR_color_get) {
  {
    struct _dwg_object_DBCOLOR *arg1 = (struct _dwg_object_DBCOLOR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DBCOLOR_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DBCOLOR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DBCOLOR_color_get" "', argument " "1"" of type '" "struct _dwg_object_DBCOLOR *""'"); 
    }
    arg1 = (struct _dwg_object_DBCOLOR *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_DBCOLOR) {
  {
    int argvi = 0;
    struct _dwg_object_DBCOLOR *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_DBCOLOR();");
    }
    result = (struct _dwg_object_DBCOLOR *)calloc(1, sizeof(struct _dwg_object_DBCOLOR));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DBCOLOR, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_DBCOLOR) {
  {
    struct _dwg_object_DBCOLOR *arg1 = (struct _dwg_object_DBCOLOR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_DBCOLOR(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DBCOLOR, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_DBCOLOR" "', argument " "1"" of type '" "struct _dwg_object_DBCOLOR *""'"); 
    }
    arg1 = (struct _dwg_object_DBCOLOR *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FIELD_ChildValue_parent_set) {
  {
    struct _dwg_FIELD_ChildValue *arg1 = (struct _dwg_FIELD_ChildValue *) 0 ;
    struct _dwg_object_FIELD *arg2 = (struct _dwg_object_FIELD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_FIELD_ChildValue_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FIELD_ChildValue, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FIELD_ChildValue_parent_set" "', argument " "1"" of type '" "struct _dwg_FIELD_ChildValue *""'"); 
    }
    arg1 = (struct _dwg_FIELD_ChildValue *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_FIELD, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_FIELD_ChildValue_parent_set" "', argument " "2"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg2 = (struct _dwg_object_FIELD *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FIELD_ChildValue_parent_get) {
  {
    struct _dwg_FIELD_ChildValue *arg1 = (struct _dwg_FIELD_ChildValue *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_FIELD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_FIELD_ChildValue_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FIELD_ChildValue, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FIELD_ChildValue_parent_get" "', argument " "1"" of type '" "struct _dwg_FIELD_ChildValue *""'"); 
    }
    arg1 = (struct _dwg_FIELD_ChildValue *)(argp1);
    result = (struct _dwg_object_FIELD *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_FIELD, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FIELD_ChildValue_key_set) {
  {
    struct _dwg_FIELD_ChildValue *arg1 = (struct _dwg_FIELD_ChildValue *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_FIELD_ChildValue_key_set(self,key);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FIELD_ChildValue, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FIELD_ChildValue_key_set" "', argument " "1"" of type '" "struct _dwg_FIELD_ChildValue *""'"); 
    }
    arg1 = (struct _dwg_FIELD_ChildValue *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_FIELD_ChildValue_key_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->key);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->key = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->key = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FIELD_ChildValue_key_get) {
  {
    struct _dwg_FIELD_ChildValue *arg1 = (struct _dwg_FIELD_ChildValue *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_FIELD_ChildValue_key_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FIELD_ChildValue, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FIELD_ChildValue_key_get" "', argument " "1"" of type '" "struct _dwg_FIELD_ChildValue *""'"); 
    }
    arg1 = (struct _dwg_FIELD_ChildValue *)(argp1);
    result = (BITCODE_TV) ((arg1)->key);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FIELD_ChildValue_value_set) {
  {
    struct _dwg_FIELD_ChildValue *arg1 = (struct _dwg_FIELD_ChildValue *) 0 ;
    Dwg_TABLE_value *arg2 = (Dwg_TABLE_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_FIELD_ChildValue_value_set(self,value);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FIELD_ChildValue, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FIELD_ChildValue_value_set" "', argument " "1"" of type '" "struct _dwg_FIELD_ChildValue *""'"); 
    }
    arg1 = (struct _dwg_FIELD_ChildValue *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_TABLE_value, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_FIELD_ChildValue_value_set" "', argument " "2"" of type '" "Dwg_TABLE_value *""'"); 
    }
    arg2 = (Dwg_TABLE_value *)(argp2);
    if (arg1) (arg1)->value = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FIELD_ChildValue_value_get) {
  {
    struct _dwg_FIELD_ChildValue *arg1 = (struct _dwg_FIELD_ChildValue *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_TABLE_value *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_FIELD_ChildValue_value_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FIELD_ChildValue, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FIELD_ChildValue_value_get" "', argument " "1"" of type '" "struct _dwg_FIELD_ChildValue *""'"); 
    }
    arg1 = (struct _dwg_FIELD_ChildValue *)(argp1);
    result = (Dwg_TABLE_value *)& ((arg1)->value);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TABLE_value, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_FIELD_ChildValue) {
  {
    int argvi = 0;
    struct _dwg_FIELD_ChildValue *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_FIELD_ChildValue();");
    }
    result = (struct _dwg_FIELD_ChildValue *)calloc(1, sizeof(struct _dwg_FIELD_ChildValue));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_FIELD_ChildValue, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_FIELD_ChildValue) {
  {
    struct _dwg_FIELD_ChildValue *arg1 = (struct _dwg_FIELD_ChildValue *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_FIELD_ChildValue(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FIELD_ChildValue, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_FIELD_ChildValue" "', argument " "1"" of type '" "struct _dwg_FIELD_ChildValue *""'"); 
    }
    arg1 = (struct _dwg_FIELD_ChildValue *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_parent_set) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_parent_set" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_FIELD_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_parent_get) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_parent_get" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_id_set) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_id_set(self,id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_id_set" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_FIELD_id_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->id);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->id = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->id = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_id_get) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_id_get" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    result = (BITCODE_TV) ((arg1)->id);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_code_set) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_code_set(self,code);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_code_set" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_FIELD_code_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->code);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->code = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->code = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_code_get) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_code_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_code_get" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    result = (BITCODE_TV) ((arg1)->code);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_num_childs_set) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_num_childs_set(self,num_childs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_num_childs_set" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_FIELD_num_childs_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_childs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_num_childs_get) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_num_childs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_num_childs_get" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_childs);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_childs_set) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_childs_set(self,childs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_childs_set" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_FIELD_childs_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->childs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_childs_get) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_childs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_childs_get" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    result = (BITCODE_H *) ((arg1)->childs);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_num_objects_set) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_num_objects_set(self,num_objects);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_num_objects_set" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_FIELD_num_objects_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_objects = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_num_objects_get) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_num_objects_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_num_objects_get" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_objects);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_objects_set) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_objects_set(self,objects);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_objects_set" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_FIELD_objects_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->objects = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_objects_get) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_objects_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_objects_get" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    result = (BITCODE_H *) ((arg1)->objects);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_format_set) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_format_set(self,format);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_format_set" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_FIELD_format_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->format);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->format = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->format = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_format_get) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_format_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_format_get" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    result = (BITCODE_TV) ((arg1)->format);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_evaluation_option_set) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_evaluation_option_set(self,evaluation_option);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_evaluation_option_set" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_FIELD_evaluation_option_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->evaluation_option = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_evaluation_option_get) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_evaluation_option_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_evaluation_option_get" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    result = (BITCODE_BL) ((arg1)->evaluation_option);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_filing_option_set) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_filing_option_set(self,filing_option);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_filing_option_set" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_FIELD_filing_option_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->filing_option = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_filing_option_get) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_filing_option_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_filing_option_get" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    result = (BITCODE_BL) ((arg1)->filing_option);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_field_state_set) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_field_state_set(self,field_state);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_field_state_set" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_FIELD_field_state_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->field_state = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_field_state_get) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_field_state_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_field_state_get" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    result = (BITCODE_BL) ((arg1)->field_state);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_evaluation_status_set) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_evaluation_status_set(self,evaluation_status);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_evaluation_status_set" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_FIELD_evaluation_status_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->evaluation_status = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_evaluation_status_get) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_evaluation_status_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_evaluation_status_get" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    result = (BITCODE_BL) ((arg1)->evaluation_status);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_evaluation_error_code_set) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_evaluation_error_code_set(self,evaluation_error_code);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_evaluation_error_code_set" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_FIELD_evaluation_error_code_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->evaluation_error_code = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_evaluation_error_code_get) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_evaluation_error_code_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_evaluation_error_code_get" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    result = (BITCODE_BL) ((arg1)->evaluation_error_code);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_evaluation_error_msg_set) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_evaluation_error_msg_set(self,evaluation_error_msg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_evaluation_error_msg_set" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_FIELD_evaluation_error_msg_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->evaluation_error_msg);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->evaluation_error_msg = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->evaluation_error_msg = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_evaluation_error_msg_get) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_evaluation_error_msg_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_evaluation_error_msg_get" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    result = (BITCODE_TV) ((arg1)->evaluation_error_msg);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_value_set) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    Dwg_TABLE_value *arg2 = (Dwg_TABLE_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_value_set(self,value);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_value_set" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_TABLE_value, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_FIELD_value_set" "', argument " "2"" of type '" "Dwg_TABLE_value *""'"); 
    }
    arg2 = (Dwg_TABLE_value *)(argp2);
    if (arg1) (arg1)->value = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_value_get) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_TABLE_value *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_value_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_value_get" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    result = (Dwg_TABLE_value *)& ((arg1)->value);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TABLE_value, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_value_string_set) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_value_string_set(self,value_string);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_value_string_set" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_FIELD_value_string_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->value_string);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->value_string = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->value_string = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_value_string_get) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_value_string_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_value_string_get" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    result = (BITCODE_TV) ((arg1)->value_string);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_value_string_length_set) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_value_string_length_set(self,value_string_length);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_value_string_length_set" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_FIELD_value_string_length_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->value_string_length = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_value_string_length_get) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_value_string_length_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_value_string_length_get" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    result = (BITCODE_BL) ((arg1)->value_string_length);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_num_childval_set) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_num_childval_set(self,num_childval);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_num_childval_set" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_FIELD_num_childval_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_childval = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_num_childval_get) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_num_childval_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_num_childval_get" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_childval);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_childval_set) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    Dwg_FIELD_ChildValue *arg2 = (Dwg_FIELD_ChildValue *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_childval_set(self,childval);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_childval_set" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_FIELD_ChildValue, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_FIELD_childval_set" "', argument " "2"" of type '" "Dwg_FIELD_ChildValue *""'"); 
    }
    arg2 = (Dwg_FIELD_ChildValue *)(argp2);
    if (arg1) (arg1)->childval = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELD_childval_get) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_FIELD_ChildValue *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_FIELD_childval_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELD_childval_get" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    result = (Dwg_FIELD_ChildValue *) ((arg1)->childval);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_FIELD_ChildValue, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_FIELD) {
  {
    int argvi = 0;
    struct _dwg_object_FIELD *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_FIELD();");
    }
    result = (struct _dwg_object_FIELD *)calloc(1, sizeof(struct _dwg_object_FIELD));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_FIELD, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_FIELD) {
  {
    struct _dwg_object_FIELD *arg1 = (struct _dwg_object_FIELD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_FIELD(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELD, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_FIELD" "', argument " "1"" of type '" "struct _dwg_object_FIELD *""'"); 
    }
    arg1 = (struct _dwg_object_FIELD *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELDLIST_parent_set) {
  {
    struct _dwg_object_FIELDLIST *arg1 = (struct _dwg_object_FIELDLIST *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_FIELDLIST_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELDLIST, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELDLIST_parent_set" "', argument " "1"" of type '" "struct _dwg_object_FIELDLIST *""'"); 
    }
    arg1 = (struct _dwg_object_FIELDLIST *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_FIELDLIST_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELDLIST_parent_get) {
  {
    struct _dwg_object_FIELDLIST *arg1 = (struct _dwg_object_FIELDLIST *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_FIELDLIST_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELDLIST, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELDLIST_parent_get" "', argument " "1"" of type '" "struct _dwg_object_FIELDLIST *""'"); 
    }
    arg1 = (struct _dwg_object_FIELDLIST *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELDLIST_num_fields_set) {
  {
    struct _dwg_object_FIELDLIST *arg1 = (struct _dwg_object_FIELDLIST *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_FIELDLIST_num_fields_set(self,num_fields);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELDLIST, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELDLIST_num_fields_set" "', argument " "1"" of type '" "struct _dwg_object_FIELDLIST *""'"); 
    }
    arg1 = (struct _dwg_object_FIELDLIST *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_FIELDLIST_num_fields_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_fields = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELDLIST_num_fields_get) {
  {
    struct _dwg_object_FIELDLIST *arg1 = (struct _dwg_object_FIELDLIST *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_FIELDLIST_num_fields_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELDLIST, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELDLIST_num_fields_get" "', argument " "1"" of type '" "struct _dwg_object_FIELDLIST *""'"); 
    }
    arg1 = (struct _dwg_object_FIELDLIST *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_fields);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELDLIST_unknown_set) {
  {
    struct _dwg_object_FIELDLIST *arg1 = (struct _dwg_object_FIELDLIST *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_FIELDLIST_unknown_set(self,unknown);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELDLIST, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELDLIST_unknown_set" "', argument " "1"" of type '" "struct _dwg_object_FIELDLIST *""'"); 
    }
    arg1 = (struct _dwg_object_FIELDLIST *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_FIELDLIST_unknown_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELDLIST_unknown_get) {
  {
    struct _dwg_object_FIELDLIST *arg1 = (struct _dwg_object_FIELDLIST *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_FIELDLIST_unknown_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELDLIST, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELDLIST_unknown_get" "', argument " "1"" of type '" "struct _dwg_object_FIELDLIST *""'"); 
    }
    arg1 = (struct _dwg_object_FIELDLIST *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELDLIST_fields_set) {
  {
    struct _dwg_object_FIELDLIST *arg1 = (struct _dwg_object_FIELDLIST *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_FIELDLIST_fields_set(self,fields);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELDLIST, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELDLIST_fields_set" "', argument " "1"" of type '" "struct _dwg_object_FIELDLIST *""'"); 
    }
    arg1 = (struct _dwg_object_FIELDLIST *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_FIELDLIST_fields_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->fields = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FIELDLIST_fields_get) {
  {
    struct _dwg_object_FIELDLIST *arg1 = (struct _dwg_object_FIELDLIST *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_FIELDLIST_fields_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELDLIST, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FIELDLIST_fields_get" "', argument " "1"" of type '" "struct _dwg_object_FIELDLIST *""'"); 
    }
    arg1 = (struct _dwg_object_FIELDLIST *)(argp1);
    result = (BITCODE_H *) ((arg1)->fields);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_FIELDLIST) {
  {
    int argvi = 0;
    struct _dwg_object_FIELDLIST *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_FIELDLIST();");
    }
    result = (struct _dwg_object_FIELDLIST *)calloc(1, sizeof(struct _dwg_object_FIELDLIST));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_FIELDLIST, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_FIELDLIST) {
  {
    struct _dwg_object_FIELDLIST *arg1 = (struct _dwg_object_FIELDLIST *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_FIELDLIST(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FIELDLIST, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_FIELDLIST" "', argument " "1"" of type '" "struct _dwg_object_FIELDLIST *""'"); 
    }
    arg1 = (struct _dwg_object_FIELDLIST *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_GEODATA_meshpt_source_pt_set) {
  {
    struct _dwg_GEODATA_meshpt *arg1 = (struct _dwg_GEODATA_meshpt *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_GEODATA_meshpt_source_pt_set(self,source_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_GEODATA_meshpt, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_GEODATA_meshpt_source_pt_set" "', argument " "1"" of type '" "struct _dwg_GEODATA_meshpt *""'"); 
    }
    arg1 = (struct _dwg_GEODATA_meshpt *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_GEODATA_meshpt_source_pt_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->source_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_GEODATA_meshpt_source_pt_get) {
  {
    struct _dwg_GEODATA_meshpt *arg1 = (struct _dwg_GEODATA_meshpt *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_GEODATA_meshpt_source_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_GEODATA_meshpt, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_GEODATA_meshpt_source_pt_get" "', argument " "1"" of type '" "struct _dwg_GEODATA_meshpt *""'"); 
    }
    arg1 = (struct _dwg_GEODATA_meshpt *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->source_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_GEODATA_meshpt_dest_pt_set) {
  {
    struct _dwg_GEODATA_meshpt *arg1 = (struct _dwg_GEODATA_meshpt *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_GEODATA_meshpt_dest_pt_set(self,dest_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_GEODATA_meshpt, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_GEODATA_meshpt_dest_pt_set" "', argument " "1"" of type '" "struct _dwg_GEODATA_meshpt *""'"); 
    }
    arg1 = (struct _dwg_GEODATA_meshpt *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_GEODATA_meshpt_dest_pt_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->dest_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_GEODATA_meshpt_dest_pt_get) {
  {
    struct _dwg_GEODATA_meshpt *arg1 = (struct _dwg_GEODATA_meshpt *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_GEODATA_meshpt_dest_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_GEODATA_meshpt, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_GEODATA_meshpt_dest_pt_get" "', argument " "1"" of type '" "struct _dwg_GEODATA_meshpt *""'"); 
    }
    arg1 = (struct _dwg_GEODATA_meshpt *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->dest_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_GEODATA_meshpt) {
  {
    int argvi = 0;
    struct _dwg_GEODATA_meshpt *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_GEODATA_meshpt();");
    }
    result = (struct _dwg_GEODATA_meshpt *)calloc(1, sizeof(struct _dwg_GEODATA_meshpt));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_GEODATA_meshpt, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_GEODATA_meshpt) {
  {
    struct _dwg_GEODATA_meshpt *arg1 = (struct _dwg_GEODATA_meshpt *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_GEODATA_meshpt(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_GEODATA_meshpt, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_GEODATA_meshpt" "', argument " "1"" of type '" "struct _dwg_GEODATA_meshpt *""'"); 
    }
    arg1 = (struct _dwg_GEODATA_meshpt *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_GEODATA_meshface_face1_set) {
  {
    struct _dwg_GEODATA_meshface *arg1 = (struct _dwg_GEODATA_meshface *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_GEODATA_meshface_face1_set(self,face1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_GEODATA_meshface, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_GEODATA_meshface_face1_set" "', argument " "1"" of type '" "struct _dwg_GEODATA_meshface *""'"); 
    }
    arg1 = (struct _dwg_GEODATA_meshface *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_GEODATA_meshface_face1_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->face1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_GEODATA_meshface_face1_get) {
  {
    struct _dwg_GEODATA_meshface *arg1 = (struct _dwg_GEODATA_meshface *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_GEODATA_meshface_face1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_GEODATA_meshface, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_GEODATA_meshface_face1_get" "', argument " "1"" of type '" "struct _dwg_GEODATA_meshface *""'"); 
    }
    arg1 = (struct _dwg_GEODATA_meshface *)(argp1);
    result = (BITCODE_BL) ((arg1)->face1);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_GEODATA_meshface_face2_set) {
  {
    struct _dwg_GEODATA_meshface *arg1 = (struct _dwg_GEODATA_meshface *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_GEODATA_meshface_face2_set(self,face2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_GEODATA_meshface, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_GEODATA_meshface_face2_set" "', argument " "1"" of type '" "struct _dwg_GEODATA_meshface *""'"); 
    }
    arg1 = (struct _dwg_GEODATA_meshface *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_GEODATA_meshface_face2_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->face2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_GEODATA_meshface_face2_get) {
  {
    struct _dwg_GEODATA_meshface *arg1 = (struct _dwg_GEODATA_meshface *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_GEODATA_meshface_face2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_GEODATA_meshface, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_GEODATA_meshface_face2_get" "', argument " "1"" of type '" "struct _dwg_GEODATA_meshface *""'"); 
    }
    arg1 = (struct _dwg_GEODATA_meshface *)(argp1);
    result = (BITCODE_BL) ((arg1)->face2);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_GEODATA_meshface_face3_set) {
  {
    struct _dwg_GEODATA_meshface *arg1 = (struct _dwg_GEODATA_meshface *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_GEODATA_meshface_face3_set(self,face3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_GEODATA_meshface, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_GEODATA_meshface_face3_set" "', argument " "1"" of type '" "struct _dwg_GEODATA_meshface *""'"); 
    }
    arg1 = (struct _dwg_GEODATA_meshface *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_GEODATA_meshface_face3_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->face3 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_GEODATA_meshface_face3_get) {
  {
    struct _dwg_GEODATA_meshface *arg1 = (struct _dwg_GEODATA_meshface *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_GEODATA_meshface_face3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_GEODATA_meshface, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_GEODATA_meshface_face3_get" "', argument " "1"" of type '" "struct _dwg_GEODATA_meshface *""'"); 
    }
    arg1 = (struct _dwg_GEODATA_meshface *)(argp1);
    result = (BITCODE_BL) ((arg1)->face3);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_GEODATA_meshface) {
  {
    int argvi = 0;
    struct _dwg_GEODATA_meshface *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_GEODATA_meshface();");
    }
    result = (struct _dwg_GEODATA_meshface *)calloc(1, sizeof(struct _dwg_GEODATA_meshface));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_GEODATA_meshface, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_GEODATA_meshface) {
  {
    struct _dwg_GEODATA_meshface *arg1 = (struct _dwg_GEODATA_meshface *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_GEODATA_meshface(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_GEODATA_meshface, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_GEODATA_meshface" "', argument " "1"" of type '" "struct _dwg_GEODATA_meshface *""'"); 
    }
    arg1 = (struct _dwg_GEODATA_meshface *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_parent_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_parent_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_GEODATA_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_parent_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_parent_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_class_version_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEODATA_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_class_version_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_host_block_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_host_block_set(self,host_block);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_host_block_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_GEODATA_host_block_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->host_block = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_host_block_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_host_block_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_host_block_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_H) ((arg1)->host_block);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_coord_type_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_coord_type_set(self,coord_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_coord_type_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEODATA_coord_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->coord_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_coord_type_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_coord_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_coord_type_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_BS) ((arg1)->coord_type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_design_pt_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_design_pt_set(self,design_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_design_pt_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_GEODATA_design_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->design_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_design_pt_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_design_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_design_pt_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->design_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_ref_pt_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_ref_pt_set(self,ref_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_ref_pt_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_GEODATA_ref_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ref_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_ref_pt_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_ref_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_ref_pt_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ref_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_obs_pt_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_obs_pt_set(self,obs_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_obs_pt_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_GEODATA_obs_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->obs_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_obs_pt_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_obs_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_obs_pt_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->obs_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_scale_vec_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_scale_vec_set(self,scale_vec);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_scale_vec_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_GEODATA_scale_vec_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->scale_vec = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_scale_vec_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_scale_vec_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_scale_vec_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->scale_vec);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_unit_scale_horiz_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_unit_scale_horiz_set(self,unit_scale_horiz);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_unit_scale_horiz_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEODATA_unit_scale_horiz_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->unit_scale_horiz = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_unit_scale_horiz_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_unit_scale_horiz_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_unit_scale_horiz_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_BD) ((arg1)->unit_scale_horiz);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_units_value_horiz_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_units_value_horiz_set(self,units_value_horiz);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_units_value_horiz_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEODATA_units_value_horiz_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->units_value_horiz = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_units_value_horiz_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_units_value_horiz_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_units_value_horiz_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_BL) ((arg1)->units_value_horiz);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_unit_scale_vert_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_unit_scale_vert_set(self,unit_scale_vert);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_unit_scale_vert_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEODATA_unit_scale_vert_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->unit_scale_vert = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_unit_scale_vert_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_unit_scale_vert_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_unit_scale_vert_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_BD) ((arg1)->unit_scale_vert);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_units_value_vert_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_units_value_vert_set(self,units_value_vert);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_units_value_vert_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEODATA_units_value_vert_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->units_value_vert = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_units_value_vert_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_units_value_vert_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_units_value_vert_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_BL) ((arg1)->units_value_vert);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_up_dir_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_up_dir_set(self,up_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_up_dir_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_GEODATA_up_dir_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->up_dir = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_up_dir_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_up_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_up_dir_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->up_dir);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_north_dir_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_north_dir_set(self,north_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_north_dir_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_GEODATA_north_dir_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->north_dir = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_north_dir_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_north_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_north_dir_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->north_dir);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_scale_est_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_scale_est_set(self,scale_est);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_scale_est_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEODATA_scale_est_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->scale_est = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_scale_est_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_scale_est_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_scale_est_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_BL) ((arg1)->scale_est);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_user_scale_factor_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_user_scale_factor_set(self,user_scale_factor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_user_scale_factor_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEODATA_user_scale_factor_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->user_scale_factor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_user_scale_factor_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_user_scale_factor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_user_scale_factor_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_BD) ((arg1)->user_scale_factor);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_do_sea_level_corr_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_do_sea_level_corr_set(self,do_sea_level_corr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_do_sea_level_corr_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEODATA_do_sea_level_corr_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->do_sea_level_corr = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_do_sea_level_corr_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_do_sea_level_corr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_do_sea_level_corr_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_B) ((arg1)->do_sea_level_corr);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_sea_level_elev_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_sea_level_elev_set(self,sea_level_elev);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_sea_level_elev_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEODATA_sea_level_elev_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->sea_level_elev = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_sea_level_elev_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_sea_level_elev_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_sea_level_elev_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_BD) ((arg1)->sea_level_elev);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_coord_proj_radius_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_coord_proj_radius_set(self,coord_proj_radius);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_coord_proj_radius_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEODATA_coord_proj_radius_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->coord_proj_radius = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_coord_proj_radius_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_coord_proj_radius_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_coord_proj_radius_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_BD) ((arg1)->coord_proj_radius);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_coord_system_def_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_coord_system_def_set(self,coord_system_def);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_coord_system_def_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_GEODATA_coord_system_def_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->coord_system_def);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->coord_system_def = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->coord_system_def = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_coord_system_def_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_coord_system_def_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_coord_system_def_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_TV) ((arg1)->coord_system_def);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_geo_rss_tag_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_geo_rss_tag_set(self,geo_rss_tag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_geo_rss_tag_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_GEODATA_geo_rss_tag_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->geo_rss_tag);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->geo_rss_tag = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->geo_rss_tag = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_geo_rss_tag_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_geo_rss_tag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_geo_rss_tag_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_TV) ((arg1)->geo_rss_tag);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_coord_system_datum_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_coord_system_datum_set(self,coord_system_datum);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_coord_system_datum_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_GEODATA_coord_system_datum_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->coord_system_datum);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->coord_system_datum = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->coord_system_datum = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_coord_system_datum_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_coord_system_datum_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_coord_system_datum_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_TV) ((arg1)->coord_system_datum);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_coord_system_wkt_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_coord_system_wkt_set(self,coord_system_wkt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_coord_system_wkt_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_GEODATA_coord_system_wkt_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->coord_system_wkt);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->coord_system_wkt = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->coord_system_wkt = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_coord_system_wkt_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_coord_system_wkt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_coord_system_wkt_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_TV) ((arg1)->coord_system_wkt);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_observation_from_tag_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_observation_from_tag_set(self,observation_from_tag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_observation_from_tag_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_GEODATA_observation_from_tag_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->observation_from_tag);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->observation_from_tag = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->observation_from_tag = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_observation_from_tag_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_observation_from_tag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_observation_from_tag_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_TV) ((arg1)->observation_from_tag);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_observation_to_tag_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_observation_to_tag_set(self,observation_to_tag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_observation_to_tag_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_GEODATA_observation_to_tag_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->observation_to_tag);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->observation_to_tag = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->observation_to_tag = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_observation_to_tag_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_observation_to_tag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_observation_to_tag_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_TV) ((arg1)->observation_to_tag);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_observation_coverage_tag_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_observation_coverage_tag_set(self,observation_coverage_tag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_observation_coverage_tag_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_GEODATA_observation_coverage_tag_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->observation_coverage_tag);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->observation_coverage_tag = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->observation_coverage_tag = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_observation_coverage_tag_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_observation_coverage_tag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_observation_coverage_tag_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_TV) ((arg1)->observation_coverage_tag);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_num_geomesh_pts_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_num_geomesh_pts_set(self,num_geomesh_pts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_num_geomesh_pts_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEODATA_num_geomesh_pts_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_geomesh_pts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_num_geomesh_pts_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_num_geomesh_pts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_num_geomesh_pts_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_geomesh_pts);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_geomesh_pts_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    Dwg_GEODATA_meshpt *arg2 = (Dwg_GEODATA_meshpt *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_geomesh_pts_set(self,geomesh_pts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_geomesh_pts_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_GEODATA_meshpt, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_GEODATA_geomesh_pts_set" "', argument " "2"" of type '" "Dwg_GEODATA_meshpt *""'"); 
    }
    arg2 = (Dwg_GEODATA_meshpt *)(argp2);
    if (arg1) (arg1)->geomesh_pts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_geomesh_pts_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_GEODATA_meshpt *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_geomesh_pts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_geomesh_pts_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (Dwg_GEODATA_meshpt *) ((arg1)->geomesh_pts);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_GEODATA_meshpt, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_num_geomesh_faces_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_num_geomesh_faces_set(self,num_geomesh_faces);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_num_geomesh_faces_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEODATA_num_geomesh_faces_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_geomesh_faces = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_num_geomesh_faces_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_num_geomesh_faces_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_num_geomesh_faces_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_geomesh_faces);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_geomesh_faces_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    Dwg_GEODATA_meshface *arg2 = (Dwg_GEODATA_meshface *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_geomesh_faces_set(self,geomesh_faces);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_geomesh_faces_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_GEODATA_meshface, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_GEODATA_geomesh_faces_set" "', argument " "2"" of type '" "Dwg_GEODATA_meshface *""'"); 
    }
    arg2 = (Dwg_GEODATA_meshface *)(argp2);
    if (arg1) (arg1)->geomesh_faces = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_geomesh_faces_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_GEODATA_meshface *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_geomesh_faces_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_geomesh_faces_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (Dwg_GEODATA_meshface *) ((arg1)->geomesh_faces);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_GEODATA_meshface, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_has_civil_data_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_has_civil_data_set(self,has_civil_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_has_civil_data_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEODATA_has_civil_data_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_civil_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_has_civil_data_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_has_civil_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_has_civil_data_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_B) ((arg1)->has_civil_data);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_obsolete_false_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_obsolete_false_set(self,obsolete_false);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_obsolete_false_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEODATA_obsolete_false_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->obsolete_false = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_obsolete_false_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_obsolete_false_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_obsolete_false_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_B) ((arg1)->obsolete_false);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_ref_pt2d_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_ref_pt2d_set(self,ref_pt2d);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_ref_pt2d_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_GEODATA_ref_pt2d_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->ref_pt2d = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_ref_pt2d_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_ref_pt2d_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_ref_pt2d_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->ref_pt2d);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_zero1_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_zero1_set(self,zero1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_zero1_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_GEODATA_zero1_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->zero1 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_zero1_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_zero1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_zero1_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->zero1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_zero2_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_zero2_set(self,zero2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_zero2_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_GEODATA_zero2_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->zero2 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_zero2_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_zero2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_zero2_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->zero2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_unknown1_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_unknown1_set(self,unknown1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_unknown1_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEODATA_unknown1_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_unknown1_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_unknown1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_unknown1_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown1);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_unknown2_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_unknown2_set(self,unknown2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_unknown2_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEODATA_unknown2_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_unknown2_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_unknown2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_unknown2_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown2);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_unknown_b_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_unknown_b_set(self,unknown_b);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_unknown_b_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEODATA_unknown_b_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown_b = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_unknown_b_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_unknown_b_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_unknown_b_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown_b);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_north_dir_angle_deg_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_north_dir_angle_deg_set(self,north_dir_angle_deg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_north_dir_angle_deg_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEODATA_north_dir_angle_deg_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->north_dir_angle_deg = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_north_dir_angle_deg_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_north_dir_angle_deg_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_north_dir_angle_deg_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_BD) ((arg1)->north_dir_angle_deg);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_north_dir_angle_rad_set) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_north_dir_angle_rad_set(self,north_dir_angle_rad);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_north_dir_angle_rad_set" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEODATA_north_dir_angle_rad_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->north_dir_angle_rad = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEODATA_north_dir_angle_rad_get) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEODATA_north_dir_angle_rad_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEODATA_north_dir_angle_rad_get" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    result = (BITCODE_BD) ((arg1)->north_dir_angle_rad);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_GEODATA) {
  {
    int argvi = 0;
    struct _dwg_object_GEODATA *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_GEODATA();");
    }
    result = (struct _dwg_object_GEODATA *)calloc(1, sizeof(struct _dwg_object_GEODATA));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_GEODATA, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_GEODATA) {
  {
    struct _dwg_object_GEODATA *arg1 = (struct _dwg_object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_GEODATA(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEODATA, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_GEODATA" "', argument " "1"" of type '" "struct _dwg_object_GEODATA *""'"); 
    }
    arg1 = (struct _dwg_object_GEODATA *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IDBUFFER_parent_set) {
  {
    struct _dwg_object_IDBUFFER *arg1 = (struct _dwg_object_IDBUFFER *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_IDBUFFER_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IDBUFFER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IDBUFFER_parent_set" "', argument " "1"" of type '" "struct _dwg_object_IDBUFFER *""'"); 
    }
    arg1 = (struct _dwg_object_IDBUFFER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_IDBUFFER_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IDBUFFER_parent_get) {
  {
    struct _dwg_object_IDBUFFER *arg1 = (struct _dwg_object_IDBUFFER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_IDBUFFER_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IDBUFFER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IDBUFFER_parent_get" "', argument " "1"" of type '" "struct _dwg_object_IDBUFFER *""'"); 
    }
    arg1 = (struct _dwg_object_IDBUFFER *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IDBUFFER_unknown_set) {
  {
    struct _dwg_object_IDBUFFER *arg1 = (struct _dwg_object_IDBUFFER *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_IDBUFFER_unknown_set(self,unknown);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IDBUFFER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IDBUFFER_unknown_set" "', argument " "1"" of type '" "struct _dwg_object_IDBUFFER *""'"); 
    }
    arg1 = (struct _dwg_object_IDBUFFER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_IDBUFFER_unknown_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->unknown = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IDBUFFER_unknown_get) {
  {
    struct _dwg_object_IDBUFFER *arg1 = (struct _dwg_object_IDBUFFER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_IDBUFFER_unknown_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IDBUFFER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IDBUFFER_unknown_get" "', argument " "1"" of type '" "struct _dwg_object_IDBUFFER *""'"); 
    }
    arg1 = (struct _dwg_object_IDBUFFER *)(argp1);
    result = (BITCODE_RC) ((arg1)->unknown);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IDBUFFER_num_obj_ids_set) {
  {
    struct _dwg_object_IDBUFFER *arg1 = (struct _dwg_object_IDBUFFER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_IDBUFFER_num_obj_ids_set(self,num_obj_ids);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IDBUFFER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IDBUFFER_num_obj_ids_set" "', argument " "1"" of type '" "struct _dwg_object_IDBUFFER *""'"); 
    }
    arg1 = (struct _dwg_object_IDBUFFER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_IDBUFFER_num_obj_ids_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_obj_ids = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IDBUFFER_num_obj_ids_get) {
  {
    struct _dwg_object_IDBUFFER *arg1 = (struct _dwg_object_IDBUFFER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_IDBUFFER_num_obj_ids_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IDBUFFER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IDBUFFER_num_obj_ids_get" "', argument " "1"" of type '" "struct _dwg_object_IDBUFFER *""'"); 
    }
    arg1 = (struct _dwg_object_IDBUFFER *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_obj_ids);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IDBUFFER_obj_ids_set) {
  {
    struct _dwg_object_IDBUFFER *arg1 = (struct _dwg_object_IDBUFFER *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_IDBUFFER_obj_ids_set(self,obj_ids);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IDBUFFER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IDBUFFER_obj_ids_set" "', argument " "1"" of type '" "struct _dwg_object_IDBUFFER *""'"); 
    }
    arg1 = (struct _dwg_object_IDBUFFER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_IDBUFFER_obj_ids_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->obj_ids = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IDBUFFER_obj_ids_get) {
  {
    struct _dwg_object_IDBUFFER *arg1 = (struct _dwg_object_IDBUFFER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_IDBUFFER_obj_ids_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IDBUFFER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IDBUFFER_obj_ids_get" "', argument " "1"" of type '" "struct _dwg_object_IDBUFFER *""'"); 
    }
    arg1 = (struct _dwg_object_IDBUFFER *)(argp1);
    result = (BITCODE_H *) ((arg1)->obj_ids);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_IDBUFFER) {
  {
    int argvi = 0;
    struct _dwg_object_IDBUFFER *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_IDBUFFER();");
    }
    result = (struct _dwg_object_IDBUFFER *)calloc(1, sizeof(struct _dwg_object_IDBUFFER));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_IDBUFFER, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_IDBUFFER) {
  {
    struct _dwg_object_IDBUFFER *arg1 = (struct _dwg_object_IDBUFFER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_IDBUFFER(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IDBUFFER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_IDBUFFER" "', argument " "1"" of type '" "struct _dwg_object_IDBUFFER *""'"); 
    }
    arg1 = (struct _dwg_object_IDBUFFER *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_IMAGE_parent_set) {
  {
    struct _dwg_entity_IMAGE *arg1 = (struct _dwg_entity_IMAGE *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_IMAGE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_IMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_IMAGE_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_IMAGE *""'"); 
    }
    arg1 = (struct _dwg_entity_IMAGE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_IMAGE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_IMAGE_parent_get) {
  {
    struct _dwg_entity_IMAGE *arg1 = (struct _dwg_entity_IMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_IMAGE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_IMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_IMAGE_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_IMAGE *""'"); 
    }
    arg1 = (struct _dwg_entity_IMAGE *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_IMAGE_class_version_set) {
  {
    struct _dwg_entity_IMAGE *arg1 = (struct _dwg_entity_IMAGE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_IMAGE_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_IMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_IMAGE_class_version_set" "', argument " "1"" of type '" "struct _dwg_entity_IMAGE *""'"); 
    }
    arg1 = (struct _dwg_entity_IMAGE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_IMAGE_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_IMAGE_class_version_get) {
  {
    struct _dwg_entity_IMAGE *arg1 = (struct _dwg_entity_IMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_IMAGE_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_IMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_IMAGE_class_version_get" "', argument " "1"" of type '" "struct _dwg_entity_IMAGE *""'"); 
    }
    arg1 = (struct _dwg_entity_IMAGE *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_IMAGE_pt0_set) {
  {
    struct _dwg_entity_IMAGE *arg1 = (struct _dwg_entity_IMAGE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_IMAGE_pt0_set(self,pt0);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_IMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_IMAGE_pt0_set" "', argument " "1"" of type '" "struct _dwg_entity_IMAGE *""'"); 
    }
    arg1 = (struct _dwg_entity_IMAGE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_IMAGE_pt0_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->pt0 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_IMAGE_pt0_get) {
  {
    struct _dwg_entity_IMAGE *arg1 = (struct _dwg_entity_IMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_IMAGE_pt0_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_IMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_IMAGE_pt0_get" "', argument " "1"" of type '" "struct _dwg_entity_IMAGE *""'"); 
    }
    arg1 = (struct _dwg_entity_IMAGE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->pt0);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_IMAGE_uvec_set) {
  {
    struct _dwg_entity_IMAGE *arg1 = (struct _dwg_entity_IMAGE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_IMAGE_uvec_set(self,uvec);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_IMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_IMAGE_uvec_set" "', argument " "1"" of type '" "struct _dwg_entity_IMAGE *""'"); 
    }
    arg1 = (struct _dwg_entity_IMAGE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_IMAGE_uvec_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->uvec = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_IMAGE_uvec_get) {
  {
    struct _dwg_entity_IMAGE *arg1 = (struct _dwg_entity_IMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_IMAGE_uvec_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_IMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_IMAGE_uvec_get" "', argument " "1"" of type '" "struct _dwg_entity_IMAGE *""'"); 
    }
    arg1 = (struct _dwg_entity_IMAGE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->uvec);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_IMAGE_vvec_set) {
  {
    struct _dwg_entity_IMAGE *arg1 = (struct _dwg_entity_IMAGE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_IMAGE_vvec_set(self,vvec);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_IMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_IMAGE_vvec_set" "', argument " "1"" of type '" "struct _dwg_entity_IMAGE *""'"); 
    }
    arg1 = (struct _dwg_entity_IMAGE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_IMAGE_vvec_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->vvec = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_IMAGE_vvec_get) {
  {
    struct _dwg_entity_IMAGE *arg1 = (struct _dwg_entity_IMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_IMAGE_vvec_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_IMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_IMAGE_vvec_get" "', argument " "1"" of type '" "struct _dwg_entity_IMAGE *""'"); 
    }
    arg1 = (struct _dwg_entity_IMAGE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->vvec);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_IMAGE_image_size_set) {
  {
    struct _dwg_entity_IMAGE *arg1 = (struct _dwg_entity_IMAGE *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_IMAGE_image_size_set(self,image_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_IMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_IMAGE_image_size_set" "', argument " "1"" of type '" "struct _dwg_entity_IMAGE *""'"); 
    }
    arg1 = (struct _dwg_entity_IMAGE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_IMAGE_image_size_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->image_size = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_IMAGE_image_size_get) {
  {
    struct _dwg_entity_IMAGE *arg1 = (struct _dwg_entity_IMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_IMAGE_image_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_IMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_IMAGE_image_size_get" "', argument " "1"" of type '" "struct _dwg_entity_IMAGE *""'"); 
    }
    arg1 = (struct _dwg_entity_IMAGE *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->image_size);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_IMAGE_display_props_set) {
  {
    struct _dwg_entity_IMAGE *arg1 = (struct _dwg_entity_IMAGE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_IMAGE_display_props_set(self,display_props);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_IMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_IMAGE_display_props_set" "', argument " "1"" of type '" "struct _dwg_entity_IMAGE *""'"); 
    }
    arg1 = (struct _dwg_entity_IMAGE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_IMAGE_display_props_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->display_props = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_IMAGE_display_props_get) {
  {
    struct _dwg_entity_IMAGE *arg1 = (struct _dwg_entity_IMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_IMAGE_display_props_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_IMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_IMAGE_display_props_get" "', argument " "1"" of type '" "struct _dwg_entity_IMAGE *""'"); 
    }
    arg1 = (struct _dwg_entity_IMAGE *)(argp1);
    result = (BITCODE_BS) ((arg1)->display_props);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_IMAGE_clipping_set) {
  {
    struct _dwg_entity_IMAGE *arg1 = (struct _dwg_entity_IMAGE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_IMAGE_clipping_set(self,clipping);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_IMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_IMAGE_clipping_set" "', argument " "1"" of type '" "struct _dwg_entity_IMAGE *""'"); 
    }
    arg1 = (struct _dwg_entity_IMAGE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_IMAGE_clipping_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->clipping = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_IMAGE_clipping_get) {
  {
    struct _dwg_entity_IMAGE *arg1 = (struct _dwg_entity_IMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_IMAGE_clipping_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_IMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_IMAGE_clipping_get" "', argument " "1"" of type '" "struct _dwg_entity_IMAGE *""'"); 
    }
    arg1 = (struct _dwg_entity_IMAGE *)(argp1);
    result = (BITCODE_B) ((arg1)->clipping);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_IMAGE_brightness_set) {
  {
    struct _dwg_entity_IMAGE *arg1 = (struct _dwg_entity_IMAGE *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_IMAGE_brightness_set(self,brightness);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_IMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_IMAGE_brightness_set" "', argument " "1"" of type '" "struct _dwg_entity_IMAGE *""'"); 
    }
    arg1 = (struct _dwg_entity_IMAGE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_IMAGE_brightness_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->brightness = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_IMAGE_brightness_get) {
  {
    struct _dwg_entity_IMAGE *arg1 = (struct _dwg_entity_IMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_IMAGE_brightness_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_IMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_IMAGE_brightness_get" "', argument " "1"" of type '" "struct _dwg_entity_IMAGE *""'"); 
    }
    arg1 = (struct _dwg_entity_IMAGE *)(argp1);
    result = (BITCODE_RC) ((arg1)->brightness);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_IMAGE_contrast_set) {
  {
    struct _dwg_entity_IMAGE *arg1 = (struct _dwg_entity_IMAGE *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_IMAGE_contrast_set(self,contrast);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_IMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_IMAGE_contrast_set" "', argument " "1"" of type '" "struct _dwg_entity_IMAGE *""'"); 
    }
    arg1 = (struct _dwg_entity_IMAGE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_IMAGE_contrast_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->contrast = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_IMAGE_contrast_get) {
  {
    struct _dwg_entity_IMAGE *arg1 = (struct _dwg_entity_IMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_IMAGE_contrast_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_IMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_IMAGE_contrast_get" "', argument " "1"" of type '" "struct _dwg_entity_IMAGE *""'"); 
    }
    arg1 = (struct _dwg_entity_IMAGE *)(argp1);
    result = (BITCODE_RC) ((arg1)->contrast);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_IMAGE_fade_set) {
  {
    struct _dwg_entity_IMAGE *arg1 = (struct _dwg_entity_IMAGE *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_IMAGE_fade_set(self,fade);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_IMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_IMAGE_fade_set" "', argument " "1"" of type '" "struct _dwg_entity_IMAGE *""'"); 
    }
    arg1 = (struct _dwg_entity_IMAGE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_IMAGE_fade_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->fade = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_IMAGE_fade_get) {
  {
    struct _dwg_entity_IMAGE *arg1 = (struct _dwg_entity_IMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_IMAGE_fade_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_IMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_IMAGE_fade_get" "', argument " "1"" of type '" "struct _dwg_entity_IMAGE *""'"); 
    }
    arg1 = (struct _dwg_entity_IMAGE *)(argp1);
    result = (BITCODE_RC) ((arg1)->fade);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_IMAGE_clip_mode_set) {
  {
    struct _dwg_entity_IMAGE *arg1 = (struct _dwg_entity_IMAGE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_IMAGE_clip_mode_set(self,clip_mode);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_IMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_IMAGE_clip_mode_set" "', argument " "1"" of type '" "struct _dwg_entity_IMAGE *""'"); 
    }
    arg1 = (struct _dwg_entity_IMAGE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_IMAGE_clip_mode_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->clip_mode = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_IMAGE_clip_mode_get) {
  {
    struct _dwg_entity_IMAGE *arg1 = (struct _dwg_entity_IMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_IMAGE_clip_mode_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_IMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_IMAGE_clip_mode_get" "', argument " "1"" of type '" "struct _dwg_entity_IMAGE *""'"); 
    }
    arg1 = (struct _dwg_entity_IMAGE *)(argp1);
    result = (BITCODE_B) ((arg1)->clip_mode);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_IMAGE_clip_boundary_type_set) {
  {
    struct _dwg_entity_IMAGE *arg1 = (struct _dwg_entity_IMAGE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_IMAGE_clip_boundary_type_set(self,clip_boundary_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_IMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_IMAGE_clip_boundary_type_set" "', argument " "1"" of type '" "struct _dwg_entity_IMAGE *""'"); 
    }
    arg1 = (struct _dwg_entity_IMAGE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_IMAGE_clip_boundary_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->clip_boundary_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_IMAGE_clip_boundary_type_get) {
  {
    struct _dwg_entity_IMAGE *arg1 = (struct _dwg_entity_IMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_IMAGE_clip_boundary_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_IMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_IMAGE_clip_boundary_type_get" "', argument " "1"" of type '" "struct _dwg_entity_IMAGE *""'"); 
    }
    arg1 = (struct _dwg_entity_IMAGE *)(argp1);
    result = (BITCODE_BS) ((arg1)->clip_boundary_type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_IMAGE_num_clip_verts_set) {
  {
    struct _dwg_entity_IMAGE *arg1 = (struct _dwg_entity_IMAGE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_IMAGE_num_clip_verts_set(self,num_clip_verts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_IMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_IMAGE_num_clip_verts_set" "', argument " "1"" of type '" "struct _dwg_entity_IMAGE *""'"); 
    }
    arg1 = (struct _dwg_entity_IMAGE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_IMAGE_num_clip_verts_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_clip_verts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_IMAGE_num_clip_verts_get) {
  {
    struct _dwg_entity_IMAGE *arg1 = (struct _dwg_entity_IMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_IMAGE_num_clip_verts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_IMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_IMAGE_num_clip_verts_get" "', argument " "1"" of type '" "struct _dwg_entity_IMAGE *""'"); 
    }
    arg1 = (struct _dwg_entity_IMAGE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_clip_verts);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_IMAGE_clip_verts_set) {
  {
    struct _dwg_entity_IMAGE *arg1 = (struct _dwg_entity_IMAGE *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_IMAGE_clip_verts_set(self,clip_verts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_IMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_IMAGE_clip_verts_set" "', argument " "1"" of type '" "struct _dwg_entity_IMAGE *""'"); 
    }
    arg1 = (struct _dwg_entity_IMAGE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_IMAGE_clip_verts_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->clip_verts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_IMAGE_clip_verts_get) {
  {
    struct _dwg_entity_IMAGE *arg1 = (struct _dwg_entity_IMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_IMAGE_clip_verts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_IMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_IMAGE_clip_verts_get" "', argument " "1"" of type '" "struct _dwg_entity_IMAGE *""'"); 
    }
    arg1 = (struct _dwg_entity_IMAGE *)(argp1);
    result = (BITCODE_2RD *) ((arg1)->clip_verts);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_IMAGE_imagedef_set) {
  {
    struct _dwg_entity_IMAGE *arg1 = (struct _dwg_entity_IMAGE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_IMAGE_imagedef_set(self,imagedef);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_IMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_IMAGE_imagedef_set" "', argument " "1"" of type '" "struct _dwg_entity_IMAGE *""'"); 
    }
    arg1 = (struct _dwg_entity_IMAGE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_IMAGE_imagedef_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->imagedef = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_IMAGE_imagedef_get) {
  {
    struct _dwg_entity_IMAGE *arg1 = (struct _dwg_entity_IMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_IMAGE_imagedef_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_IMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_IMAGE_imagedef_get" "', argument " "1"" of type '" "struct _dwg_entity_IMAGE *""'"); 
    }
    arg1 = (struct _dwg_entity_IMAGE *)(argp1);
    result = (BITCODE_H) ((arg1)->imagedef);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_IMAGE_imagedefreactor_set) {
  {
    struct _dwg_entity_IMAGE *arg1 = (struct _dwg_entity_IMAGE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_IMAGE_imagedefreactor_set(self,imagedefreactor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_IMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_IMAGE_imagedefreactor_set" "', argument " "1"" of type '" "struct _dwg_entity_IMAGE *""'"); 
    }
    arg1 = (struct _dwg_entity_IMAGE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_IMAGE_imagedefreactor_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->imagedefreactor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_IMAGE_imagedefreactor_get) {
  {
    struct _dwg_entity_IMAGE *arg1 = (struct _dwg_entity_IMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_IMAGE_imagedefreactor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_IMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_IMAGE_imagedefreactor_get" "', argument " "1"" of type '" "struct _dwg_entity_IMAGE *""'"); 
    }
    arg1 = (struct _dwg_entity_IMAGE *)(argp1);
    result = (BITCODE_H) ((arg1)->imagedefreactor);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_IMAGE) {
  {
    int argvi = 0;
    struct _dwg_entity_IMAGE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_IMAGE();");
    }
    result = (struct _dwg_entity_IMAGE *)calloc(1, sizeof(struct _dwg_entity_IMAGE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_IMAGE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_IMAGE) {
  {
    struct _dwg_entity_IMAGE *arg1 = (struct _dwg_entity_IMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_IMAGE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_IMAGE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_IMAGE" "', argument " "1"" of type '" "struct _dwg_entity_IMAGE *""'"); 
    }
    arg1 = (struct _dwg_entity_IMAGE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IMAGEDEF_parent_set) {
  {
    struct _dwg_object_IMAGEDEF *arg1 = (struct _dwg_object_IMAGEDEF *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_IMAGEDEF_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGEDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IMAGEDEF_parent_set" "', argument " "1"" of type '" "struct _dwg_object_IMAGEDEF *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGEDEF *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_IMAGEDEF_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IMAGEDEF_parent_get) {
  {
    struct _dwg_object_IMAGEDEF *arg1 = (struct _dwg_object_IMAGEDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_IMAGEDEF_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGEDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IMAGEDEF_parent_get" "', argument " "1"" of type '" "struct _dwg_object_IMAGEDEF *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGEDEF *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IMAGEDEF_class_version_set) {
  {
    struct _dwg_object_IMAGEDEF *arg1 = (struct _dwg_object_IMAGEDEF *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_IMAGEDEF_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGEDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IMAGEDEF_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_IMAGEDEF *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGEDEF *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_IMAGEDEF_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IMAGEDEF_class_version_get) {
  {
    struct _dwg_object_IMAGEDEF *arg1 = (struct _dwg_object_IMAGEDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_IMAGEDEF_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGEDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IMAGEDEF_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_IMAGEDEF *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGEDEF *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IMAGEDEF_image_size_set) {
  {
    struct _dwg_object_IMAGEDEF *arg1 = (struct _dwg_object_IMAGEDEF *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_IMAGEDEF_image_size_set(self,image_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGEDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IMAGEDEF_image_size_set" "', argument " "1"" of type '" "struct _dwg_object_IMAGEDEF *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGEDEF *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_IMAGEDEF_image_size_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->image_size = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IMAGEDEF_image_size_get) {
  {
    struct _dwg_object_IMAGEDEF *arg1 = (struct _dwg_object_IMAGEDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_IMAGEDEF_image_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGEDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IMAGEDEF_image_size_get" "', argument " "1"" of type '" "struct _dwg_object_IMAGEDEF *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGEDEF *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->image_size);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IMAGEDEF_file_path_set) {
  {
    struct _dwg_object_IMAGEDEF *arg1 = (struct _dwg_object_IMAGEDEF *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_IMAGEDEF_file_path_set(self,file_path);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGEDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IMAGEDEF_file_path_set" "', argument " "1"" of type '" "struct _dwg_object_IMAGEDEF *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGEDEF *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_IMAGEDEF_file_path_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->file_path);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->file_path = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->file_path = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IMAGEDEF_file_path_get) {
  {
    struct _dwg_object_IMAGEDEF *arg1 = (struct _dwg_object_IMAGEDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_IMAGEDEF_file_path_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGEDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IMAGEDEF_file_path_get" "', argument " "1"" of type '" "struct _dwg_object_IMAGEDEF *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGEDEF *)(argp1);
    result = (BITCODE_TV) ((arg1)->file_path);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IMAGEDEF_is_loaded_set) {
  {
    struct _dwg_object_IMAGEDEF *arg1 = (struct _dwg_object_IMAGEDEF *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_IMAGEDEF_is_loaded_set(self,is_loaded);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGEDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IMAGEDEF_is_loaded_set" "', argument " "1"" of type '" "struct _dwg_object_IMAGEDEF *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGEDEF *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_IMAGEDEF_is_loaded_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_loaded = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IMAGEDEF_is_loaded_get) {
  {
    struct _dwg_object_IMAGEDEF *arg1 = (struct _dwg_object_IMAGEDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_IMAGEDEF_is_loaded_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGEDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IMAGEDEF_is_loaded_get" "', argument " "1"" of type '" "struct _dwg_object_IMAGEDEF *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGEDEF *)(argp1);
    result = (BITCODE_B) ((arg1)->is_loaded);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IMAGEDEF_resunits_set) {
  {
    struct _dwg_object_IMAGEDEF *arg1 = (struct _dwg_object_IMAGEDEF *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_IMAGEDEF_resunits_set(self,resunits);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGEDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IMAGEDEF_resunits_set" "', argument " "1"" of type '" "struct _dwg_object_IMAGEDEF *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGEDEF *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_IMAGEDEF_resunits_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->resunits = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IMAGEDEF_resunits_get) {
  {
    struct _dwg_object_IMAGEDEF *arg1 = (struct _dwg_object_IMAGEDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_IMAGEDEF_resunits_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGEDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IMAGEDEF_resunits_get" "', argument " "1"" of type '" "struct _dwg_object_IMAGEDEF *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGEDEF *)(argp1);
    result = (BITCODE_RC) ((arg1)->resunits);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IMAGEDEF_pixel_size_set) {
  {
    struct _dwg_object_IMAGEDEF *arg1 = (struct _dwg_object_IMAGEDEF *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_IMAGEDEF_pixel_size_set(self,pixel_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGEDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IMAGEDEF_pixel_size_set" "', argument " "1"" of type '" "struct _dwg_object_IMAGEDEF *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGEDEF *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_IMAGEDEF_pixel_size_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->pixel_size = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IMAGEDEF_pixel_size_get) {
  {
    struct _dwg_object_IMAGEDEF *arg1 = (struct _dwg_object_IMAGEDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_IMAGEDEF_pixel_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGEDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IMAGEDEF_pixel_size_get" "', argument " "1"" of type '" "struct _dwg_object_IMAGEDEF *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGEDEF *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->pixel_size);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_IMAGEDEF) {
  {
    int argvi = 0;
    struct _dwg_object_IMAGEDEF *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_IMAGEDEF();");
    }
    result = (struct _dwg_object_IMAGEDEF *)calloc(1, sizeof(struct _dwg_object_IMAGEDEF));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_IMAGEDEF, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_IMAGEDEF) {
  {
    struct _dwg_object_IMAGEDEF *arg1 = (struct _dwg_object_IMAGEDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_IMAGEDEF(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGEDEF, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_IMAGEDEF" "', argument " "1"" of type '" "struct _dwg_object_IMAGEDEF *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGEDEF *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IMAGEDEF_REACTOR_parent_set) {
  {
    struct _dwg_object_IMAGEDEF_REACTOR *arg1 = (struct _dwg_object_IMAGEDEF_REACTOR *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_IMAGEDEF_REACTOR_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGEDEF_REACTOR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IMAGEDEF_REACTOR_parent_set" "', argument " "1"" of type '" "struct _dwg_object_IMAGEDEF_REACTOR *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGEDEF_REACTOR *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_IMAGEDEF_REACTOR_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IMAGEDEF_REACTOR_parent_get) {
  {
    struct _dwg_object_IMAGEDEF_REACTOR *arg1 = (struct _dwg_object_IMAGEDEF_REACTOR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_IMAGEDEF_REACTOR_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGEDEF_REACTOR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IMAGEDEF_REACTOR_parent_get" "', argument " "1"" of type '" "struct _dwg_object_IMAGEDEF_REACTOR *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGEDEF_REACTOR *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IMAGEDEF_REACTOR_class_version_set) {
  {
    struct _dwg_object_IMAGEDEF_REACTOR *arg1 = (struct _dwg_object_IMAGEDEF_REACTOR *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_IMAGEDEF_REACTOR_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGEDEF_REACTOR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IMAGEDEF_REACTOR_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_IMAGEDEF_REACTOR *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGEDEF_REACTOR *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_IMAGEDEF_REACTOR_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IMAGEDEF_REACTOR_class_version_get) {
  {
    struct _dwg_object_IMAGEDEF_REACTOR *arg1 = (struct _dwg_object_IMAGEDEF_REACTOR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_IMAGEDEF_REACTOR_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGEDEF_REACTOR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IMAGEDEF_REACTOR_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_IMAGEDEF_REACTOR *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGEDEF_REACTOR *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_IMAGEDEF_REACTOR) {
  {
    int argvi = 0;
    struct _dwg_object_IMAGEDEF_REACTOR *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_IMAGEDEF_REACTOR();");
    }
    result = (struct _dwg_object_IMAGEDEF_REACTOR *)calloc(1, sizeof(struct _dwg_object_IMAGEDEF_REACTOR));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_IMAGEDEF_REACTOR, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_IMAGEDEF_REACTOR) {
  {
    struct _dwg_object_IMAGEDEF_REACTOR *arg1 = (struct _dwg_object_IMAGEDEF_REACTOR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_IMAGEDEF_REACTOR(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGEDEF_REACTOR, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_IMAGEDEF_REACTOR" "', argument " "1"" of type '" "struct _dwg_object_IMAGEDEF_REACTOR *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGEDEF_REACTOR *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_INDEX_parent_set) {
  {
    struct _dwg_object_INDEX *arg1 = (struct _dwg_object_INDEX *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_INDEX_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_INDEX_parent_set" "', argument " "1"" of type '" "struct _dwg_object_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_INDEX *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_INDEX_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_INDEX_parent_get) {
  {
    struct _dwg_object_INDEX *arg1 = (struct _dwg_object_INDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_INDEX_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_INDEX_parent_get" "', argument " "1"" of type '" "struct _dwg_object_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_INDEX *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_INDEX_last_updated_set) {
  {
    struct _dwg_object_INDEX *arg1 = (struct _dwg_object_INDEX *) 0 ;
    BITCODE_TIMEBLL *arg2 = (BITCODE_TIMEBLL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_INDEX_last_updated_set(self,last_updated);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_INDEX_last_updated_set" "', argument " "1"" of type '" "struct _dwg_object_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_INDEX *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_time_bll, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_INDEX_last_updated_set" "', argument " "2"" of type '" "BITCODE_TIMEBLL *""'"); 
    }
    arg2 = (BITCODE_TIMEBLL *)(argp2);
    if (arg1) (arg1)->last_updated = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_INDEX_last_updated_get) {
  {
    struct _dwg_object_INDEX *arg1 = (struct _dwg_object_INDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TIMEBLL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_INDEX_last_updated_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_INDEX_last_updated_get" "', argument " "1"" of type '" "struct _dwg_object_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_INDEX *)(argp1);
    result = (BITCODE_TIMEBLL *)& ((arg1)->last_updated);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_time_bll, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_INDEX) {
  {
    int argvi = 0;
    struct _dwg_object_INDEX *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_INDEX();");
    }
    result = (struct _dwg_object_INDEX *)calloc(1, sizeof(struct _dwg_object_INDEX));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_INDEX, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_INDEX) {
  {
    struct _dwg_object_INDEX *arg1 = (struct _dwg_object_INDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_INDEX(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_INDEX, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_INDEX" "', argument " "1"" of type '" "struct _dwg_object_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_INDEX *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LAYER_entry_parent_set) {
  {
    struct _dwg_LAYER_entry *arg1 = (struct _dwg_LAYER_entry *) 0 ;
    struct _dwg_object_LAYER_INDEX *arg2 = (struct _dwg_object_LAYER_INDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LAYER_entry_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LAYER_entry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LAYER_entry_parent_set" "', argument " "1"" of type '" "struct _dwg_LAYER_entry *""'"); 
    }
    arg1 = (struct _dwg_LAYER_entry *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_LAYER_INDEX, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_LAYER_entry_parent_set" "', argument " "2"" of type '" "struct _dwg_object_LAYER_INDEX *""'"); 
    }
    arg2 = (struct _dwg_object_LAYER_INDEX *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LAYER_entry_parent_get) {
  {
    struct _dwg_LAYER_entry *arg1 = (struct _dwg_LAYER_entry *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_LAYER_INDEX *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LAYER_entry_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LAYER_entry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LAYER_entry_parent_get" "', argument " "1"" of type '" "struct _dwg_LAYER_entry *""'"); 
    }
    arg1 = (struct _dwg_LAYER_entry *)(argp1);
    result = (struct _dwg_object_LAYER_INDEX *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_LAYER_INDEX, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LAYER_entry_numlayers_set) {
  {
    struct _dwg_LAYER_entry *arg1 = (struct _dwg_LAYER_entry *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LAYER_entry_numlayers_set(self,numlayers);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LAYER_entry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LAYER_entry_numlayers_set" "', argument " "1"" of type '" "struct _dwg_LAYER_entry *""'"); 
    }
    arg1 = (struct _dwg_LAYER_entry *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_LAYER_entry_numlayers_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->numlayers = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LAYER_entry_numlayers_get) {
  {
    struct _dwg_LAYER_entry *arg1 = (struct _dwg_LAYER_entry *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LAYER_entry_numlayers_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LAYER_entry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LAYER_entry_numlayers_get" "', argument " "1"" of type '" "struct _dwg_LAYER_entry *""'"); 
    }
    arg1 = (struct _dwg_LAYER_entry *)(argp1);
    result = (BITCODE_BL) ((arg1)->numlayers);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LAYER_entry_name_set) {
  {
    struct _dwg_LAYER_entry *arg1 = (struct _dwg_LAYER_entry *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LAYER_entry_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LAYER_entry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LAYER_entry_name_set" "', argument " "1"" of type '" "struct _dwg_LAYER_entry *""'"); 
    }
    arg1 = (struct _dwg_LAYER_entry *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_LAYER_entry_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LAYER_entry_name_get) {
  {
    struct _dwg_LAYER_entry *arg1 = (struct _dwg_LAYER_entry *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LAYER_entry_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LAYER_entry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LAYER_entry_name_get" "', argument " "1"" of type '" "struct _dwg_LAYER_entry *""'"); 
    }
    arg1 = (struct _dwg_LAYER_entry *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LAYER_entry_handle_set) {
  {
    struct _dwg_LAYER_entry *arg1 = (struct _dwg_LAYER_entry *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LAYER_entry_handle_set(self,handle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LAYER_entry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LAYER_entry_handle_set" "', argument " "1"" of type '" "struct _dwg_LAYER_entry *""'"); 
    }
    arg1 = (struct _dwg_LAYER_entry *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_LAYER_entry_handle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->handle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LAYER_entry_handle_get) {
  {
    struct _dwg_LAYER_entry *arg1 = (struct _dwg_LAYER_entry *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LAYER_entry_handle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LAYER_entry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LAYER_entry_handle_get" "', argument " "1"" of type '" "struct _dwg_LAYER_entry *""'"); 
    }
    arg1 = (struct _dwg_LAYER_entry *)(argp1);
    result = (BITCODE_H) ((arg1)->handle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_LAYER_entry) {
  {
    int argvi = 0;
    struct _dwg_LAYER_entry *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_LAYER_entry();");
    }
    result = (struct _dwg_LAYER_entry *)calloc(1, sizeof(struct _dwg_LAYER_entry));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_LAYER_entry, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_LAYER_entry) {
  {
    struct _dwg_LAYER_entry *arg1 = (struct _dwg_LAYER_entry *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_LAYER_entry(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LAYER_entry, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_LAYER_entry" "', argument " "1"" of type '" "struct _dwg_LAYER_entry *""'"); 
    }
    arg1 = (struct _dwg_LAYER_entry *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_INDEX_parent_set) {
  {
    struct _dwg_object_LAYER_INDEX *arg1 = (struct _dwg_object_LAYER_INDEX *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_INDEX_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_INDEX_parent_set" "', argument " "1"" of type '" "struct _dwg_object_LAYER_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER_INDEX *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LAYER_INDEX_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_INDEX_parent_get) {
  {
    struct _dwg_object_LAYER_INDEX *arg1 = (struct _dwg_object_LAYER_INDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_INDEX_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_INDEX_parent_get" "', argument " "1"" of type '" "struct _dwg_object_LAYER_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER_INDEX *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_INDEX_last_updated_set) {
  {
    struct _dwg_object_LAYER_INDEX *arg1 = (struct _dwg_object_LAYER_INDEX *) 0 ;
    BITCODE_TIMEBLL *arg2 = (BITCODE_TIMEBLL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_INDEX_last_updated_set(self,last_updated);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_INDEX_last_updated_set" "', argument " "1"" of type '" "struct _dwg_object_LAYER_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER_INDEX *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_time_bll, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LAYER_INDEX_last_updated_set" "', argument " "2"" of type '" "BITCODE_TIMEBLL *""'"); 
    }
    arg2 = (BITCODE_TIMEBLL *)(argp2);
    if (arg1) (arg1)->last_updated = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_INDEX_last_updated_get) {
  {
    struct _dwg_object_LAYER_INDEX *arg1 = (struct _dwg_object_LAYER_INDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TIMEBLL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_INDEX_last_updated_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_INDEX_last_updated_get" "', argument " "1"" of type '" "struct _dwg_object_LAYER_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER_INDEX *)(argp1);
    result = (BITCODE_TIMEBLL *)& ((arg1)->last_updated);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_time_bll, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_INDEX_num_entries_set) {
  {
    struct _dwg_object_LAYER_INDEX *arg1 = (struct _dwg_object_LAYER_INDEX *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_INDEX_num_entries_set(self,num_entries);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_INDEX_num_entries_set" "', argument " "1"" of type '" "struct _dwg_object_LAYER_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER_INDEX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LAYER_INDEX_num_entries_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_entries = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_INDEX_num_entries_get) {
  {
    struct _dwg_object_LAYER_INDEX *arg1 = (struct _dwg_object_LAYER_INDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_INDEX_num_entries_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_INDEX_num_entries_get" "', argument " "1"" of type '" "struct _dwg_object_LAYER_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER_INDEX *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_entries);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_INDEX_entries_set) {
  {
    struct _dwg_object_LAYER_INDEX *arg1 = (struct _dwg_object_LAYER_INDEX *) 0 ;
    Dwg_LAYER_entry *arg2 = (Dwg_LAYER_entry *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_INDEX_entries_set(self,entries);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_INDEX_entries_set" "', argument " "1"" of type '" "struct _dwg_object_LAYER_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER_INDEX *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_LAYER_entry, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LAYER_INDEX_entries_set" "', argument " "2"" of type '" "Dwg_LAYER_entry *""'"); 
    }
    arg2 = (Dwg_LAYER_entry *)(argp2);
    if (arg1) (arg1)->entries = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYER_INDEX_entries_get) {
  {
    struct _dwg_object_LAYER_INDEX *arg1 = (struct _dwg_object_LAYER_INDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_LAYER_entry *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYER_INDEX_entries_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYER_INDEX_entries_get" "', argument " "1"" of type '" "struct _dwg_object_LAYER_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER_INDEX *)(argp1);
    result = (Dwg_LAYER_entry *) ((arg1)->entries);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_LAYER_entry, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_LAYER_INDEX) {
  {
    int argvi = 0;
    struct _dwg_object_LAYER_INDEX *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_LAYER_INDEX();");
    }
    result = (struct _dwg_object_LAYER_INDEX *)calloc(1, sizeof(struct _dwg_object_LAYER_INDEX));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_LAYER_INDEX, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_LAYER_INDEX) {
  {
    struct _dwg_object_LAYER_INDEX *arg1 = (struct _dwg_object_LAYER_INDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_LAYER_INDEX(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYER_INDEX, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_LAYER_INDEX" "', argument " "1"" of type '" "struct _dwg_object_LAYER_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_LAYER_INDEX *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LWPOLYLINE_parent_set) {
  {
    struct _dwg_entity_LWPOLYLINE *arg1 = (struct _dwg_entity_LWPOLYLINE *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LWPOLYLINE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LWPOLYLINE_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LWPOLYLINE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LWPOLYLINE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LWPOLYLINE_parent_get) {
  {
    struct _dwg_entity_LWPOLYLINE *arg1 = (struct _dwg_entity_LWPOLYLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LWPOLYLINE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LWPOLYLINE_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LWPOLYLINE *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LWPOLYLINE_flag_set) {
  {
    struct _dwg_entity_LWPOLYLINE *arg1 = (struct _dwg_entity_LWPOLYLINE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LWPOLYLINE_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LWPOLYLINE_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LWPOLYLINE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LWPOLYLINE_flag_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LWPOLYLINE_flag_get) {
  {
    struct _dwg_entity_LWPOLYLINE *arg1 = (struct _dwg_entity_LWPOLYLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LWPOLYLINE_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LWPOLYLINE_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LWPOLYLINE *)(argp1);
    result = (BITCODE_BS) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LWPOLYLINE_const_width_set) {
  {
    struct _dwg_entity_LWPOLYLINE *arg1 = (struct _dwg_entity_LWPOLYLINE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LWPOLYLINE_const_width_set(self,const_width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LWPOLYLINE_const_width_set" "', argument " "1"" of type '" "struct _dwg_entity_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LWPOLYLINE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LWPOLYLINE_const_width_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->const_width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LWPOLYLINE_const_width_get) {
  {
    struct _dwg_entity_LWPOLYLINE *arg1 = (struct _dwg_entity_LWPOLYLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LWPOLYLINE_const_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LWPOLYLINE_const_width_get" "', argument " "1"" of type '" "struct _dwg_entity_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LWPOLYLINE *)(argp1);
    result = (BITCODE_BD) ((arg1)->const_width);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LWPOLYLINE_elevation_set) {
  {
    struct _dwg_entity_LWPOLYLINE *arg1 = (struct _dwg_entity_LWPOLYLINE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LWPOLYLINE_elevation_set(self,elevation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LWPOLYLINE_elevation_set" "', argument " "1"" of type '" "struct _dwg_entity_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LWPOLYLINE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LWPOLYLINE_elevation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->elevation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LWPOLYLINE_elevation_get) {
  {
    struct _dwg_entity_LWPOLYLINE *arg1 = (struct _dwg_entity_LWPOLYLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LWPOLYLINE_elevation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LWPOLYLINE_elevation_get" "', argument " "1"" of type '" "struct _dwg_entity_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LWPOLYLINE *)(argp1);
    result = (BITCODE_BD) ((arg1)->elevation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LWPOLYLINE_thickness_set) {
  {
    struct _dwg_entity_LWPOLYLINE *arg1 = (struct _dwg_entity_LWPOLYLINE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LWPOLYLINE_thickness_set(self,thickness);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LWPOLYLINE_thickness_set" "', argument " "1"" of type '" "struct _dwg_entity_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LWPOLYLINE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LWPOLYLINE_thickness_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->thickness = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LWPOLYLINE_thickness_get) {
  {
    struct _dwg_entity_LWPOLYLINE *arg1 = (struct _dwg_entity_LWPOLYLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LWPOLYLINE_thickness_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LWPOLYLINE_thickness_get" "', argument " "1"" of type '" "struct _dwg_entity_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LWPOLYLINE *)(argp1);
    result = (BITCODE_BD) ((arg1)->thickness);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LWPOLYLINE_extrusion_set) {
  {
    struct _dwg_entity_LWPOLYLINE *arg1 = (struct _dwg_entity_LWPOLYLINE *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LWPOLYLINE_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LWPOLYLINE_extrusion_set" "', argument " "1"" of type '" "struct _dwg_entity_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LWPOLYLINE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LWPOLYLINE_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LWPOLYLINE_extrusion_get) {
  {
    struct _dwg_entity_LWPOLYLINE *arg1 = (struct _dwg_entity_LWPOLYLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LWPOLYLINE_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LWPOLYLINE_extrusion_get" "', argument " "1"" of type '" "struct _dwg_entity_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LWPOLYLINE *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LWPOLYLINE_num_points_set) {
  {
    struct _dwg_entity_LWPOLYLINE *arg1 = (struct _dwg_entity_LWPOLYLINE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LWPOLYLINE_num_points_set(self,num_points);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LWPOLYLINE_num_points_set" "', argument " "1"" of type '" "struct _dwg_entity_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LWPOLYLINE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LWPOLYLINE_num_points_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_points = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LWPOLYLINE_num_points_get) {
  {
    struct _dwg_entity_LWPOLYLINE *arg1 = (struct _dwg_entity_LWPOLYLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LWPOLYLINE_num_points_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LWPOLYLINE_num_points_get" "', argument " "1"" of type '" "struct _dwg_entity_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LWPOLYLINE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_points);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LWPOLYLINE_points_set) {
  {
    struct _dwg_entity_LWPOLYLINE *arg1 = (struct _dwg_entity_LWPOLYLINE *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LWPOLYLINE_points_set(self,points);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LWPOLYLINE_points_set" "', argument " "1"" of type '" "struct _dwg_entity_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LWPOLYLINE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LWPOLYLINE_points_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->points = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LWPOLYLINE_points_get) {
  {
    struct _dwg_entity_LWPOLYLINE *arg1 = (struct _dwg_entity_LWPOLYLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LWPOLYLINE_points_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LWPOLYLINE_points_get" "', argument " "1"" of type '" "struct _dwg_entity_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LWPOLYLINE *)(argp1);
    result = (BITCODE_2RD *) ((arg1)->points);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LWPOLYLINE_num_bulges_set) {
  {
    struct _dwg_entity_LWPOLYLINE *arg1 = (struct _dwg_entity_LWPOLYLINE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LWPOLYLINE_num_bulges_set(self,num_bulges);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LWPOLYLINE_num_bulges_set" "', argument " "1"" of type '" "struct _dwg_entity_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LWPOLYLINE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LWPOLYLINE_num_bulges_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_bulges = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LWPOLYLINE_num_bulges_get) {
  {
    struct _dwg_entity_LWPOLYLINE *arg1 = (struct _dwg_entity_LWPOLYLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LWPOLYLINE_num_bulges_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LWPOLYLINE_num_bulges_get" "', argument " "1"" of type '" "struct _dwg_entity_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LWPOLYLINE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_bulges);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LWPOLYLINE_bulges_set) {
  {
    struct _dwg_entity_LWPOLYLINE *arg1 = (struct _dwg_entity_LWPOLYLINE *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LWPOLYLINE_bulges_set(self,bulges);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LWPOLYLINE_bulges_set" "', argument " "1"" of type '" "struct _dwg_entity_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LWPOLYLINE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LWPOLYLINE_bulges_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->bulges = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LWPOLYLINE_bulges_get) {
  {
    struct _dwg_entity_LWPOLYLINE *arg1 = (struct _dwg_entity_LWPOLYLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LWPOLYLINE_bulges_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LWPOLYLINE_bulges_get" "', argument " "1"" of type '" "struct _dwg_entity_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LWPOLYLINE *)(argp1);
    result = (BITCODE_BD *) ((arg1)->bulges);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LWPOLYLINE_num_vertexids_set) {
  {
    struct _dwg_entity_LWPOLYLINE *arg1 = (struct _dwg_entity_LWPOLYLINE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LWPOLYLINE_num_vertexids_set(self,num_vertexids);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LWPOLYLINE_num_vertexids_set" "', argument " "1"" of type '" "struct _dwg_entity_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LWPOLYLINE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LWPOLYLINE_num_vertexids_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_vertexids = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LWPOLYLINE_num_vertexids_get) {
  {
    struct _dwg_entity_LWPOLYLINE *arg1 = (struct _dwg_entity_LWPOLYLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LWPOLYLINE_num_vertexids_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LWPOLYLINE_num_vertexids_get" "', argument " "1"" of type '" "struct _dwg_entity_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LWPOLYLINE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_vertexids);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LWPOLYLINE_vertexids_set) {
  {
    struct _dwg_entity_LWPOLYLINE *arg1 = (struct _dwg_entity_LWPOLYLINE *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LWPOLYLINE_vertexids_set(self,vertexids);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LWPOLYLINE_vertexids_set" "', argument " "1"" of type '" "struct _dwg_entity_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LWPOLYLINE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LWPOLYLINE_vertexids_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->vertexids = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LWPOLYLINE_vertexids_get) {
  {
    struct _dwg_entity_LWPOLYLINE *arg1 = (struct _dwg_entity_LWPOLYLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LWPOLYLINE_vertexids_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LWPOLYLINE_vertexids_get" "', argument " "1"" of type '" "struct _dwg_entity_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LWPOLYLINE *)(argp1);
    result = (BITCODE_BL *) ((arg1)->vertexids);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LWPOLYLINE_num_widths_set) {
  {
    struct _dwg_entity_LWPOLYLINE *arg1 = (struct _dwg_entity_LWPOLYLINE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LWPOLYLINE_num_widths_set(self,num_widths);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LWPOLYLINE_num_widths_set" "', argument " "1"" of type '" "struct _dwg_entity_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LWPOLYLINE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LWPOLYLINE_num_widths_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_widths = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LWPOLYLINE_num_widths_get) {
  {
    struct _dwg_entity_LWPOLYLINE *arg1 = (struct _dwg_entity_LWPOLYLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LWPOLYLINE_num_widths_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LWPOLYLINE_num_widths_get" "', argument " "1"" of type '" "struct _dwg_entity_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LWPOLYLINE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_widths);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LWPOLYLINE_widths_set) {
  {
    struct _dwg_entity_LWPOLYLINE *arg1 = (struct _dwg_entity_LWPOLYLINE *) 0 ;
    Dwg_LWPOLYLINE_width *arg2 = (Dwg_LWPOLYLINE_width *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LWPOLYLINE_widths_set(self,widths);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LWPOLYLINE_widths_set" "', argument " "1"" of type '" "struct _dwg_entity_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LWPOLYLINE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_LWPOLYLINE_width, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LWPOLYLINE_widths_set" "', argument " "2"" of type '" "Dwg_LWPOLYLINE_width *""'"); 
    }
    arg2 = (Dwg_LWPOLYLINE_width *)(argp2);
    if (arg1) (arg1)->widths = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LWPOLYLINE_widths_get) {
  {
    struct _dwg_entity_LWPOLYLINE *arg1 = (struct _dwg_entity_LWPOLYLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_LWPOLYLINE_width *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LWPOLYLINE_widths_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LWPOLYLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LWPOLYLINE_widths_get" "', argument " "1"" of type '" "struct _dwg_entity_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LWPOLYLINE *)(argp1);
    result = (Dwg_LWPOLYLINE_width *) ((arg1)->widths);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_LWPOLYLINE_width, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_LWPOLYLINE) {
  {
    int argvi = 0;
    struct _dwg_entity_LWPOLYLINE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_LWPOLYLINE();");
    }
    result = (struct _dwg_entity_LWPOLYLINE *)calloc(1, sizeof(struct _dwg_entity_LWPOLYLINE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_LWPOLYLINE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_LWPOLYLINE) {
  {
    struct _dwg_entity_LWPOLYLINE *arg1 = (struct _dwg_entity_LWPOLYLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_LWPOLYLINE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LWPOLYLINE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_LWPOLYLINE" "', argument " "1"" of type '" "struct _dwg_entity_LWPOLYLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_LWPOLYLINE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RASTERVARIABLES_parent_set) {
  {
    struct _dwg_object_RASTERVARIABLES *arg1 = (struct _dwg_object_RASTERVARIABLES *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RASTERVARIABLES_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RASTERVARIABLES, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RASTERVARIABLES_parent_set" "', argument " "1"" of type '" "struct _dwg_object_RASTERVARIABLES *""'"); 
    }
    arg1 = (struct _dwg_object_RASTERVARIABLES *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_RASTERVARIABLES_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RASTERVARIABLES_parent_get) {
  {
    struct _dwg_object_RASTERVARIABLES *arg1 = (struct _dwg_object_RASTERVARIABLES *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RASTERVARIABLES_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RASTERVARIABLES, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RASTERVARIABLES_parent_get" "', argument " "1"" of type '" "struct _dwg_object_RASTERVARIABLES *""'"); 
    }
    arg1 = (struct _dwg_object_RASTERVARIABLES *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RASTERVARIABLES_class_version_set) {
  {
    struct _dwg_object_RASTERVARIABLES *arg1 = (struct _dwg_object_RASTERVARIABLES *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RASTERVARIABLES_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RASTERVARIABLES, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RASTERVARIABLES_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_RASTERVARIABLES *""'"); 
    }
    arg1 = (struct _dwg_object_RASTERVARIABLES *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RASTERVARIABLES_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RASTERVARIABLES_class_version_get) {
  {
    struct _dwg_object_RASTERVARIABLES *arg1 = (struct _dwg_object_RASTERVARIABLES *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RASTERVARIABLES_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RASTERVARIABLES, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RASTERVARIABLES_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_RASTERVARIABLES *""'"); 
    }
    arg1 = (struct _dwg_object_RASTERVARIABLES *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RASTERVARIABLES_image_frame_set) {
  {
    struct _dwg_object_RASTERVARIABLES *arg1 = (struct _dwg_object_RASTERVARIABLES *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RASTERVARIABLES_image_frame_set(self,image_frame);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RASTERVARIABLES, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RASTERVARIABLES_image_frame_set" "', argument " "1"" of type '" "struct _dwg_object_RASTERVARIABLES *""'"); 
    }
    arg1 = (struct _dwg_object_RASTERVARIABLES *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RASTERVARIABLES_image_frame_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->image_frame = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RASTERVARIABLES_image_frame_get) {
  {
    struct _dwg_object_RASTERVARIABLES *arg1 = (struct _dwg_object_RASTERVARIABLES *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RASTERVARIABLES_image_frame_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RASTERVARIABLES, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RASTERVARIABLES_image_frame_get" "', argument " "1"" of type '" "struct _dwg_object_RASTERVARIABLES *""'"); 
    }
    arg1 = (struct _dwg_object_RASTERVARIABLES *)(argp1);
    result = (BITCODE_BS) ((arg1)->image_frame);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RASTERVARIABLES_image_quality_set) {
  {
    struct _dwg_object_RASTERVARIABLES *arg1 = (struct _dwg_object_RASTERVARIABLES *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RASTERVARIABLES_image_quality_set(self,image_quality);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RASTERVARIABLES, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RASTERVARIABLES_image_quality_set" "', argument " "1"" of type '" "struct _dwg_object_RASTERVARIABLES *""'"); 
    }
    arg1 = (struct _dwg_object_RASTERVARIABLES *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RASTERVARIABLES_image_quality_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->image_quality = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RASTERVARIABLES_image_quality_get) {
  {
    struct _dwg_object_RASTERVARIABLES *arg1 = (struct _dwg_object_RASTERVARIABLES *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RASTERVARIABLES_image_quality_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RASTERVARIABLES, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RASTERVARIABLES_image_quality_get" "', argument " "1"" of type '" "struct _dwg_object_RASTERVARIABLES *""'"); 
    }
    arg1 = (struct _dwg_object_RASTERVARIABLES *)(argp1);
    result = (BITCODE_BS) ((arg1)->image_quality);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RASTERVARIABLES_units_set) {
  {
    struct _dwg_object_RASTERVARIABLES *arg1 = (struct _dwg_object_RASTERVARIABLES *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RASTERVARIABLES_units_set(self,units);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RASTERVARIABLES, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RASTERVARIABLES_units_set" "', argument " "1"" of type '" "struct _dwg_object_RASTERVARIABLES *""'"); 
    }
    arg1 = (struct _dwg_object_RASTERVARIABLES *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RASTERVARIABLES_units_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->units = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RASTERVARIABLES_units_get) {
  {
    struct _dwg_object_RASTERVARIABLES *arg1 = (struct _dwg_object_RASTERVARIABLES *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RASTERVARIABLES_units_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RASTERVARIABLES, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RASTERVARIABLES_units_get" "', argument " "1"" of type '" "struct _dwg_object_RASTERVARIABLES *""'"); 
    }
    arg1 = (struct _dwg_object_RASTERVARIABLES *)(argp1);
    result = (BITCODE_BS) ((arg1)->units);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_RASTERVARIABLES) {
  {
    int argvi = 0;
    struct _dwg_object_RASTERVARIABLES *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_RASTERVARIABLES();");
    }
    result = (struct _dwg_object_RASTERVARIABLES *)calloc(1, sizeof(struct _dwg_object_RASTERVARIABLES));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_RASTERVARIABLES, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_RASTERVARIABLES) {
  {
    struct _dwg_object_RASTERVARIABLES *arg1 = (struct _dwg_object_RASTERVARIABLES *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_RASTERVARIABLES(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RASTERVARIABLES, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_RASTERVARIABLES" "', argument " "1"" of type '" "struct _dwg_object_RASTERVARIABLES *""'"); 
    }
    arg1 = (struct _dwg_object_RASTERVARIABLES *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SCALE_parent_set) {
  {
    struct _dwg_object_SCALE *arg1 = (struct _dwg_object_SCALE *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SCALE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SCALE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SCALE_parent_set" "', argument " "1"" of type '" "struct _dwg_object_SCALE *""'"); 
    }
    arg1 = (struct _dwg_object_SCALE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SCALE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SCALE_parent_get) {
  {
    struct _dwg_object_SCALE *arg1 = (struct _dwg_object_SCALE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SCALE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SCALE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SCALE_parent_get" "', argument " "1"" of type '" "struct _dwg_object_SCALE *""'"); 
    }
    arg1 = (struct _dwg_object_SCALE *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SCALE_flag_set) {
  {
    struct _dwg_object_SCALE *arg1 = (struct _dwg_object_SCALE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SCALE_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SCALE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SCALE_flag_set" "', argument " "1"" of type '" "struct _dwg_object_SCALE *""'"); 
    }
    arg1 = (struct _dwg_object_SCALE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SCALE_flag_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SCALE_flag_get) {
  {
    struct _dwg_object_SCALE *arg1 = (struct _dwg_object_SCALE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SCALE_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SCALE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SCALE_flag_get" "', argument " "1"" of type '" "struct _dwg_object_SCALE *""'"); 
    }
    arg1 = (struct _dwg_object_SCALE *)(argp1);
    result = (BITCODE_BS) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SCALE_name_set) {
  {
    struct _dwg_object_SCALE *arg1 = (struct _dwg_object_SCALE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SCALE_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SCALE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SCALE_name_set" "', argument " "1"" of type '" "struct _dwg_object_SCALE *""'"); 
    }
    arg1 = (struct _dwg_object_SCALE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SCALE_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SCALE_name_get) {
  {
    struct _dwg_object_SCALE *arg1 = (struct _dwg_object_SCALE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SCALE_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SCALE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SCALE_name_get" "', argument " "1"" of type '" "struct _dwg_object_SCALE *""'"); 
    }
    arg1 = (struct _dwg_object_SCALE *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SCALE_paper_units_set) {
  {
    struct _dwg_object_SCALE *arg1 = (struct _dwg_object_SCALE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SCALE_paper_units_set(self,paper_units);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SCALE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SCALE_paper_units_set" "', argument " "1"" of type '" "struct _dwg_object_SCALE *""'"); 
    }
    arg1 = (struct _dwg_object_SCALE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SCALE_paper_units_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->paper_units = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SCALE_paper_units_get) {
  {
    struct _dwg_object_SCALE *arg1 = (struct _dwg_object_SCALE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SCALE_paper_units_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SCALE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SCALE_paper_units_get" "', argument " "1"" of type '" "struct _dwg_object_SCALE *""'"); 
    }
    arg1 = (struct _dwg_object_SCALE *)(argp1);
    result = (BITCODE_BD) ((arg1)->paper_units);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SCALE_drawing_units_set) {
  {
    struct _dwg_object_SCALE *arg1 = (struct _dwg_object_SCALE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SCALE_drawing_units_set(self,drawing_units);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SCALE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SCALE_drawing_units_set" "', argument " "1"" of type '" "struct _dwg_object_SCALE *""'"); 
    }
    arg1 = (struct _dwg_object_SCALE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SCALE_drawing_units_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->drawing_units = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SCALE_drawing_units_get) {
  {
    struct _dwg_object_SCALE *arg1 = (struct _dwg_object_SCALE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SCALE_drawing_units_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SCALE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SCALE_drawing_units_get" "', argument " "1"" of type '" "struct _dwg_object_SCALE *""'"); 
    }
    arg1 = (struct _dwg_object_SCALE *)(argp1);
    result = (BITCODE_BD) ((arg1)->drawing_units);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SCALE_is_unit_scale_set) {
  {
    struct _dwg_object_SCALE *arg1 = (struct _dwg_object_SCALE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SCALE_is_unit_scale_set(self,is_unit_scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SCALE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SCALE_is_unit_scale_set" "', argument " "1"" of type '" "struct _dwg_object_SCALE *""'"); 
    }
    arg1 = (struct _dwg_object_SCALE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SCALE_is_unit_scale_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_unit_scale = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SCALE_is_unit_scale_get) {
  {
    struct _dwg_object_SCALE *arg1 = (struct _dwg_object_SCALE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SCALE_is_unit_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SCALE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SCALE_is_unit_scale_get" "', argument " "1"" of type '" "struct _dwg_object_SCALE *""'"); 
    }
    arg1 = (struct _dwg_object_SCALE *)(argp1);
    result = (BITCODE_B) ((arg1)->is_unit_scale);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_SCALE) {
  {
    int argvi = 0;
    struct _dwg_object_SCALE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_SCALE();");
    }
    result = (struct _dwg_object_SCALE *)calloc(1, sizeof(struct _dwg_object_SCALE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_SCALE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_SCALE) {
  {
    struct _dwg_object_SCALE *arg1 = (struct _dwg_object_SCALE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_SCALE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SCALE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_SCALE" "', argument " "1"" of type '" "struct _dwg_object_SCALE *""'"); 
    }
    arg1 = (struct _dwg_object_SCALE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SORTENTSTABLE_parent_set) {
  {
    struct _dwg_object_SORTENTSTABLE *arg1 = (struct _dwg_object_SORTENTSTABLE *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SORTENTSTABLE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SORTENTSTABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SORTENTSTABLE_parent_set" "', argument " "1"" of type '" "struct _dwg_object_SORTENTSTABLE *""'"); 
    }
    arg1 = (struct _dwg_object_SORTENTSTABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SORTENTSTABLE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SORTENTSTABLE_parent_get) {
  {
    struct _dwg_object_SORTENTSTABLE *arg1 = (struct _dwg_object_SORTENTSTABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SORTENTSTABLE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SORTENTSTABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SORTENTSTABLE_parent_get" "', argument " "1"" of type '" "struct _dwg_object_SORTENTSTABLE *""'"); 
    }
    arg1 = (struct _dwg_object_SORTENTSTABLE *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SORTENTSTABLE_num_ents_set) {
  {
    struct _dwg_object_SORTENTSTABLE *arg1 = (struct _dwg_object_SORTENTSTABLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SORTENTSTABLE_num_ents_set(self,num_ents);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SORTENTSTABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SORTENTSTABLE_num_ents_set" "', argument " "1"" of type '" "struct _dwg_object_SORTENTSTABLE *""'"); 
    }
    arg1 = (struct _dwg_object_SORTENTSTABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SORTENTSTABLE_num_ents_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_ents = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SORTENTSTABLE_num_ents_get) {
  {
    struct _dwg_object_SORTENTSTABLE *arg1 = (struct _dwg_object_SORTENTSTABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SORTENTSTABLE_num_ents_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SORTENTSTABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SORTENTSTABLE_num_ents_get" "', argument " "1"" of type '" "struct _dwg_object_SORTENTSTABLE *""'"); 
    }
    arg1 = (struct _dwg_object_SORTENTSTABLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_ents);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SORTENTSTABLE_sort_ents_set) {
  {
    struct _dwg_object_SORTENTSTABLE *arg1 = (struct _dwg_object_SORTENTSTABLE *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SORTENTSTABLE_sort_ents_set(self,sort_ents);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SORTENTSTABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SORTENTSTABLE_sort_ents_set" "', argument " "1"" of type '" "struct _dwg_object_SORTENTSTABLE *""'"); 
    }
    arg1 = (struct _dwg_object_SORTENTSTABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SORTENTSTABLE_sort_ents_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->sort_ents = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SORTENTSTABLE_sort_ents_get) {
  {
    struct _dwg_object_SORTENTSTABLE *arg1 = (struct _dwg_object_SORTENTSTABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SORTENTSTABLE_sort_ents_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SORTENTSTABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SORTENTSTABLE_sort_ents_get" "', argument " "1"" of type '" "struct _dwg_object_SORTENTSTABLE *""'"); 
    }
    arg1 = (struct _dwg_object_SORTENTSTABLE *)(argp1);
    result = (BITCODE_H *) ((arg1)->sort_ents);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SORTENTSTABLE_block_owner_set) {
  {
    struct _dwg_object_SORTENTSTABLE *arg1 = (struct _dwg_object_SORTENTSTABLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SORTENTSTABLE_block_owner_set(self,block_owner);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SORTENTSTABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SORTENTSTABLE_block_owner_set" "', argument " "1"" of type '" "struct _dwg_object_SORTENTSTABLE *""'"); 
    }
    arg1 = (struct _dwg_object_SORTENTSTABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SORTENTSTABLE_block_owner_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->block_owner = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SORTENTSTABLE_block_owner_get) {
  {
    struct _dwg_object_SORTENTSTABLE *arg1 = (struct _dwg_object_SORTENTSTABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SORTENTSTABLE_block_owner_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SORTENTSTABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SORTENTSTABLE_block_owner_get" "', argument " "1"" of type '" "struct _dwg_object_SORTENTSTABLE *""'"); 
    }
    arg1 = (struct _dwg_object_SORTENTSTABLE *)(argp1);
    result = (BITCODE_H) ((arg1)->block_owner);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SORTENTSTABLE_ents_set) {
  {
    struct _dwg_object_SORTENTSTABLE *arg1 = (struct _dwg_object_SORTENTSTABLE *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SORTENTSTABLE_ents_set(self,ents);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SORTENTSTABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SORTENTSTABLE_ents_set" "', argument " "1"" of type '" "struct _dwg_object_SORTENTSTABLE *""'"); 
    }
    arg1 = (struct _dwg_object_SORTENTSTABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SORTENTSTABLE_ents_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->ents = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SORTENTSTABLE_ents_get) {
  {
    struct _dwg_object_SORTENTSTABLE *arg1 = (struct _dwg_object_SORTENTSTABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SORTENTSTABLE_ents_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SORTENTSTABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SORTENTSTABLE_ents_get" "', argument " "1"" of type '" "struct _dwg_object_SORTENTSTABLE *""'"); 
    }
    arg1 = (struct _dwg_object_SORTENTSTABLE *)(argp1);
    result = (BITCODE_H *) ((arg1)->ents);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_SORTENTSTABLE) {
  {
    int argvi = 0;
    struct _dwg_object_SORTENTSTABLE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_SORTENTSTABLE();");
    }
    result = (struct _dwg_object_SORTENTSTABLE *)calloc(1, sizeof(struct _dwg_object_SORTENTSTABLE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_SORTENTSTABLE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_SORTENTSTABLE) {
  {
    struct _dwg_object_SORTENTSTABLE *arg1 = (struct _dwg_object_SORTENTSTABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_SORTENTSTABLE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SORTENTSTABLE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_SORTENTSTABLE" "', argument " "1"" of type '" "struct _dwg_object_SORTENTSTABLE *""'"); 
    }
    arg1 = (struct _dwg_object_SORTENTSTABLE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_FILTER_parent_set) {
  {
    struct _dwg_object_SPATIAL_FILTER *arg1 = (struct _dwg_object_SPATIAL_FILTER *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_FILTER_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_FILTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_FILTER_parent_set" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_FILTER *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_FILTER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SPATIAL_FILTER_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_FILTER_parent_get) {
  {
    struct _dwg_object_SPATIAL_FILTER *arg1 = (struct _dwg_object_SPATIAL_FILTER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_FILTER_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_FILTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_FILTER_parent_get" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_FILTER *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_FILTER *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_FILTER_num_clip_verts_set) {
  {
    struct _dwg_object_SPATIAL_FILTER *arg1 = (struct _dwg_object_SPATIAL_FILTER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_FILTER_num_clip_verts_set(self,num_clip_verts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_FILTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_FILTER_num_clip_verts_set" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_FILTER *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_FILTER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SPATIAL_FILTER_num_clip_verts_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_clip_verts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_FILTER_num_clip_verts_get) {
  {
    struct _dwg_object_SPATIAL_FILTER *arg1 = (struct _dwg_object_SPATIAL_FILTER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_FILTER_num_clip_verts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_FILTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_FILTER_num_clip_verts_get" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_FILTER *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_FILTER *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_clip_verts);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_FILTER_clip_verts_set) {
  {
    struct _dwg_object_SPATIAL_FILTER *arg1 = (struct _dwg_object_SPATIAL_FILTER *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_FILTER_clip_verts_set(self,clip_verts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_FILTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_FILTER_clip_verts_set" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_FILTER *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_FILTER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SPATIAL_FILTER_clip_verts_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->clip_verts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_FILTER_clip_verts_get) {
  {
    struct _dwg_object_SPATIAL_FILTER *arg1 = (struct _dwg_object_SPATIAL_FILTER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_FILTER_clip_verts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_FILTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_FILTER_clip_verts_get" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_FILTER *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_FILTER *)(argp1);
    result = (BITCODE_2RD *) ((arg1)->clip_verts);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_FILTER_extrusion_set) {
  {
    struct _dwg_object_SPATIAL_FILTER *arg1 = (struct _dwg_object_SPATIAL_FILTER *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_FILTER_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_FILTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_FILTER_extrusion_set" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_FILTER *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_FILTER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SPATIAL_FILTER_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_FILTER_extrusion_get) {
  {
    struct _dwg_object_SPATIAL_FILTER *arg1 = (struct _dwg_object_SPATIAL_FILTER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_FILTER_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_FILTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_FILTER_extrusion_get" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_FILTER *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_FILTER *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_FILTER_origin_set) {
  {
    struct _dwg_object_SPATIAL_FILTER *arg1 = (struct _dwg_object_SPATIAL_FILTER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_FILTER_origin_set(self,origin);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_FILTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_FILTER_origin_set" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_FILTER *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_FILTER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SPATIAL_FILTER_origin_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->origin = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_FILTER_origin_get) {
  {
    struct _dwg_object_SPATIAL_FILTER *arg1 = (struct _dwg_object_SPATIAL_FILTER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_FILTER_origin_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_FILTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_FILTER_origin_get" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_FILTER *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_FILTER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->origin);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_FILTER_display_boundary_on_set) {
  {
    struct _dwg_object_SPATIAL_FILTER *arg1 = (struct _dwg_object_SPATIAL_FILTER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_FILTER_display_boundary_on_set(self,display_boundary_on);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_FILTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_FILTER_display_boundary_on_set" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_FILTER *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_FILTER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SPATIAL_FILTER_display_boundary_on_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->display_boundary_on = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_FILTER_display_boundary_on_get) {
  {
    struct _dwg_object_SPATIAL_FILTER *arg1 = (struct _dwg_object_SPATIAL_FILTER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_FILTER_display_boundary_on_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_FILTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_FILTER_display_boundary_on_get" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_FILTER *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_FILTER *)(argp1);
    result = (BITCODE_BS) ((arg1)->display_boundary_on);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_FILTER_front_clip_on_set) {
  {
    struct _dwg_object_SPATIAL_FILTER *arg1 = (struct _dwg_object_SPATIAL_FILTER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_FILTER_front_clip_on_set(self,front_clip_on);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_FILTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_FILTER_front_clip_on_set" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_FILTER *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_FILTER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SPATIAL_FILTER_front_clip_on_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->front_clip_on = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_FILTER_front_clip_on_get) {
  {
    struct _dwg_object_SPATIAL_FILTER *arg1 = (struct _dwg_object_SPATIAL_FILTER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_FILTER_front_clip_on_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_FILTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_FILTER_front_clip_on_get" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_FILTER *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_FILTER *)(argp1);
    result = (BITCODE_BS) ((arg1)->front_clip_on);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_FILTER_front_clip_z_set) {
  {
    struct _dwg_object_SPATIAL_FILTER *arg1 = (struct _dwg_object_SPATIAL_FILTER *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_FILTER_front_clip_z_set(self,front_clip_z);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_FILTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_FILTER_front_clip_z_set" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_FILTER *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_FILTER *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SPATIAL_FILTER_front_clip_z_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->front_clip_z = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_FILTER_front_clip_z_get) {
  {
    struct _dwg_object_SPATIAL_FILTER *arg1 = (struct _dwg_object_SPATIAL_FILTER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_FILTER_front_clip_z_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_FILTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_FILTER_front_clip_z_get" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_FILTER *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_FILTER *)(argp1);
    result = (BITCODE_BD) ((arg1)->front_clip_z);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_FILTER_back_clip_on_set) {
  {
    struct _dwg_object_SPATIAL_FILTER *arg1 = (struct _dwg_object_SPATIAL_FILTER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_FILTER_back_clip_on_set(self,back_clip_on);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_FILTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_FILTER_back_clip_on_set" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_FILTER *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_FILTER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SPATIAL_FILTER_back_clip_on_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->back_clip_on = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_FILTER_back_clip_on_get) {
  {
    struct _dwg_object_SPATIAL_FILTER *arg1 = (struct _dwg_object_SPATIAL_FILTER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_FILTER_back_clip_on_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_FILTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_FILTER_back_clip_on_get" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_FILTER *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_FILTER *)(argp1);
    result = (BITCODE_BS) ((arg1)->back_clip_on);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_FILTER_back_clip_z_set) {
  {
    struct _dwg_object_SPATIAL_FILTER *arg1 = (struct _dwg_object_SPATIAL_FILTER *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_FILTER_back_clip_z_set(self,back_clip_z);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_FILTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_FILTER_back_clip_z_set" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_FILTER *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_FILTER *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SPATIAL_FILTER_back_clip_z_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->back_clip_z = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_FILTER_back_clip_z_get) {
  {
    struct _dwg_object_SPATIAL_FILTER *arg1 = (struct _dwg_object_SPATIAL_FILTER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_FILTER_back_clip_z_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_FILTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_FILTER_back_clip_z_get" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_FILTER *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_FILTER *)(argp1);
    result = (BITCODE_BD) ((arg1)->back_clip_z);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_FILTER_inverse_transform_set) {
  {
    struct _dwg_object_SPATIAL_FILTER *arg1 = (struct _dwg_object_SPATIAL_FILTER *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_FILTER_inverse_transform_set(self,inverse_transform);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_FILTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_FILTER_inverse_transform_set" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_FILTER *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_FILTER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SPATIAL_FILTER_inverse_transform_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->inverse_transform = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_FILTER_inverse_transform_get) {
  {
    struct _dwg_object_SPATIAL_FILTER *arg1 = (struct _dwg_object_SPATIAL_FILTER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_FILTER_inverse_transform_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_FILTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_FILTER_inverse_transform_get" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_FILTER *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_FILTER *)(argp1);
    result = (BITCODE_BD *) ((arg1)->inverse_transform);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_FILTER_transform_set) {
  {
    struct _dwg_object_SPATIAL_FILTER *arg1 = (struct _dwg_object_SPATIAL_FILTER *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_FILTER_transform_set(self,transform);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_FILTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_FILTER_transform_set" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_FILTER *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_FILTER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SPATIAL_FILTER_transform_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->transform = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_FILTER_transform_get) {
  {
    struct _dwg_object_SPATIAL_FILTER *arg1 = (struct _dwg_object_SPATIAL_FILTER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_FILTER_transform_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_FILTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_FILTER_transform_get" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_FILTER *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_FILTER *)(argp1);
    result = (BITCODE_BD *) ((arg1)->transform);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_SPATIAL_FILTER) {
  {
    int argvi = 0;
    struct _dwg_object_SPATIAL_FILTER *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_SPATIAL_FILTER();");
    }
    result = (struct _dwg_object_SPATIAL_FILTER *)calloc(1, sizeof(struct _dwg_object_SPATIAL_FILTER));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_SPATIAL_FILTER, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_SPATIAL_FILTER) {
  {
    struct _dwg_object_SPATIAL_FILTER *arg1 = (struct _dwg_object_SPATIAL_FILTER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_SPATIAL_FILTER(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_FILTER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_SPATIAL_FILTER" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_FILTER *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_FILTER *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_INDEX_parent_set) {
  {
    struct _dwg_object_SPATIAL_INDEX *arg1 = (struct _dwg_object_SPATIAL_INDEX *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_INDEX_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_INDEX_parent_set" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_INDEX *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SPATIAL_INDEX_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_INDEX_parent_get) {
  {
    struct _dwg_object_SPATIAL_INDEX *arg1 = (struct _dwg_object_SPATIAL_INDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_INDEX_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_INDEX_parent_get" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_INDEX *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_INDEX_last_updated_set) {
  {
    struct _dwg_object_SPATIAL_INDEX *arg1 = (struct _dwg_object_SPATIAL_INDEX *) 0 ;
    BITCODE_TIMEBLL *arg2 = (BITCODE_TIMEBLL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_INDEX_last_updated_set(self,last_updated);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_INDEX_last_updated_set" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_INDEX *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_time_bll, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SPATIAL_INDEX_last_updated_set" "', argument " "2"" of type '" "BITCODE_TIMEBLL *""'"); 
    }
    arg2 = (BITCODE_TIMEBLL *)(argp2);
    if (arg1) (arg1)->last_updated = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_INDEX_last_updated_get) {
  {
    struct _dwg_object_SPATIAL_INDEX *arg1 = (struct _dwg_object_SPATIAL_INDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TIMEBLL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_INDEX_last_updated_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_INDEX_last_updated_get" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_INDEX *)(argp1);
    result = (BITCODE_TIMEBLL *)& ((arg1)->last_updated);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_time_bll, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_INDEX_num1_set) {
  {
    struct _dwg_object_SPATIAL_INDEX *arg1 = (struct _dwg_object_SPATIAL_INDEX *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_INDEX_num1_set(self,num1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_INDEX_num1_set" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_INDEX *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SPATIAL_INDEX_num1_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->num1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_INDEX_num1_get) {
  {
    struct _dwg_object_SPATIAL_INDEX *arg1 = (struct _dwg_object_SPATIAL_INDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_INDEX_num1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_INDEX_num1_get" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_INDEX *)(argp1);
    result = (BITCODE_BD) ((arg1)->num1);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_INDEX_num2_set) {
  {
    struct _dwg_object_SPATIAL_INDEX *arg1 = (struct _dwg_object_SPATIAL_INDEX *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_INDEX_num2_set(self,num2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_INDEX_num2_set" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_INDEX *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SPATIAL_INDEX_num2_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->num2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_INDEX_num2_get) {
  {
    struct _dwg_object_SPATIAL_INDEX *arg1 = (struct _dwg_object_SPATIAL_INDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_INDEX_num2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_INDEX_num2_get" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_INDEX *)(argp1);
    result = (BITCODE_BD) ((arg1)->num2);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_INDEX_num3_set) {
  {
    struct _dwg_object_SPATIAL_INDEX *arg1 = (struct _dwg_object_SPATIAL_INDEX *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_INDEX_num3_set(self,num3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_INDEX_num3_set" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_INDEX *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SPATIAL_INDEX_num3_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->num3 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_INDEX_num3_get) {
  {
    struct _dwg_object_SPATIAL_INDEX *arg1 = (struct _dwg_object_SPATIAL_INDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_INDEX_num3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_INDEX_num3_get" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_INDEX *)(argp1);
    result = (BITCODE_BD) ((arg1)->num3);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_INDEX_num4_set) {
  {
    struct _dwg_object_SPATIAL_INDEX *arg1 = (struct _dwg_object_SPATIAL_INDEX *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_INDEX_num4_set(self,num4);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_INDEX_num4_set" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_INDEX *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SPATIAL_INDEX_num4_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->num4 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_INDEX_num4_get) {
  {
    struct _dwg_object_SPATIAL_INDEX *arg1 = (struct _dwg_object_SPATIAL_INDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_INDEX_num4_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_INDEX_num4_get" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_INDEX *)(argp1);
    result = (BITCODE_BD) ((arg1)->num4);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_INDEX_num5_set) {
  {
    struct _dwg_object_SPATIAL_INDEX *arg1 = (struct _dwg_object_SPATIAL_INDEX *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_INDEX_num5_set(self,num5);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_INDEX_num5_set" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_INDEX *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SPATIAL_INDEX_num5_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->num5 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_INDEX_num5_get) {
  {
    struct _dwg_object_SPATIAL_INDEX *arg1 = (struct _dwg_object_SPATIAL_INDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_INDEX_num5_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_INDEX_num5_get" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_INDEX *)(argp1);
    result = (BITCODE_BD) ((arg1)->num5);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_INDEX_num6_set) {
  {
    struct _dwg_object_SPATIAL_INDEX *arg1 = (struct _dwg_object_SPATIAL_INDEX *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_INDEX_num6_set(self,num6);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_INDEX_num6_set" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_INDEX *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SPATIAL_INDEX_num6_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->num6 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_INDEX_num6_get) {
  {
    struct _dwg_object_SPATIAL_INDEX *arg1 = (struct _dwg_object_SPATIAL_INDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_INDEX_num6_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_INDEX_num6_get" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_INDEX *)(argp1);
    result = (BITCODE_BD) ((arg1)->num6);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_INDEX_num_hdls_set) {
  {
    struct _dwg_object_SPATIAL_INDEX *arg1 = (struct _dwg_object_SPATIAL_INDEX *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_INDEX_num_hdls_set(self,num_hdls);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_INDEX_num_hdls_set" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_INDEX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SPATIAL_INDEX_num_hdls_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_hdls = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_INDEX_num_hdls_get) {
  {
    struct _dwg_object_SPATIAL_INDEX *arg1 = (struct _dwg_object_SPATIAL_INDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_INDEX_num_hdls_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_INDEX_num_hdls_get" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_INDEX *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_hdls);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_INDEX_hdls_set) {
  {
    struct _dwg_object_SPATIAL_INDEX *arg1 = (struct _dwg_object_SPATIAL_INDEX *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_INDEX_hdls_set(self,hdls);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_INDEX_hdls_set" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_INDEX *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SPATIAL_INDEX_hdls_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->hdls = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_INDEX_hdls_get) {
  {
    struct _dwg_object_SPATIAL_INDEX *arg1 = (struct _dwg_object_SPATIAL_INDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_INDEX_hdls_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_INDEX_hdls_get" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_INDEX *)(argp1);
    result = (BITCODE_H *) ((arg1)->hdls);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_INDEX_bindata_size_set) {
  {
    struct _dwg_object_SPATIAL_INDEX *arg1 = (struct _dwg_object_SPATIAL_INDEX *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_INDEX_bindata_size_set(self,bindata_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_INDEX_bindata_size_set" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_INDEX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SPATIAL_INDEX_bindata_size_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bindata_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_INDEX_bindata_size_get) {
  {
    struct _dwg_object_SPATIAL_INDEX *arg1 = (struct _dwg_object_SPATIAL_INDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_INDEX_bindata_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_INDEX_bindata_size_get" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_INDEX *)(argp1);
    result = (BITCODE_BL) ((arg1)->bindata_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_INDEX_bindata_set) {
  {
    struct _dwg_object_SPATIAL_INDEX *arg1 = (struct _dwg_object_SPATIAL_INDEX *) 0 ;
    BITCODE_TF arg2 = (BITCODE_TF) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_INDEX_bindata_set(self,bindata);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_INDEX_bindata_set" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_INDEX *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SPATIAL_INDEX_bindata_set" "', argument " "2"" of type '" "BITCODE_TF""'"); 
    }
    arg2 = (BITCODE_TF)(argp2);
    if (arg1) (arg1)->bindata = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SPATIAL_INDEX_bindata_get) {
  {
    struct _dwg_object_SPATIAL_INDEX *arg1 = (struct _dwg_object_SPATIAL_INDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TF result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SPATIAL_INDEX_bindata_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SPATIAL_INDEX_bindata_get" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_INDEX *)(argp1);
    result = (BITCODE_TF) ((arg1)->bindata);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_SPATIAL_INDEX) {
  {
    int argvi = 0;
    struct _dwg_object_SPATIAL_INDEX *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_SPATIAL_INDEX();");
    }
    result = (struct _dwg_object_SPATIAL_INDEX *)calloc(1, sizeof(struct _dwg_object_SPATIAL_INDEX));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_SPATIAL_INDEX, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_SPATIAL_INDEX) {
  {
    struct _dwg_object_SPATIAL_INDEX *arg1 = (struct _dwg_object_SPATIAL_INDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_SPATIAL_INDEX(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SPATIAL_INDEX, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_SPATIAL_INDEX" "', argument " "1"" of type '" "struct _dwg_object_SPATIAL_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_SPATIAL_INDEX *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_WIPEOUT_parent_set) {
  {
    struct _dwg_entity_WIPEOUT *arg1 = (struct _dwg_entity_WIPEOUT *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_WIPEOUT_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_WIPEOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_WIPEOUT_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_WIPEOUT *""'"); 
    }
    arg1 = (struct _dwg_entity_WIPEOUT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_WIPEOUT_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_WIPEOUT_parent_get) {
  {
    struct _dwg_entity_WIPEOUT *arg1 = (struct _dwg_entity_WIPEOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_WIPEOUT_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_WIPEOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_WIPEOUT_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_WIPEOUT *""'"); 
    }
    arg1 = (struct _dwg_entity_WIPEOUT *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_WIPEOUT_class_version_set) {
  {
    struct _dwg_entity_WIPEOUT *arg1 = (struct _dwg_entity_WIPEOUT *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_WIPEOUT_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_WIPEOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_WIPEOUT_class_version_set" "', argument " "1"" of type '" "struct _dwg_entity_WIPEOUT *""'"); 
    }
    arg1 = (struct _dwg_entity_WIPEOUT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_WIPEOUT_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_WIPEOUT_class_version_get) {
  {
    struct _dwg_entity_WIPEOUT *arg1 = (struct _dwg_entity_WIPEOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_WIPEOUT_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_WIPEOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_WIPEOUT_class_version_get" "', argument " "1"" of type '" "struct _dwg_entity_WIPEOUT *""'"); 
    }
    arg1 = (struct _dwg_entity_WIPEOUT *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_WIPEOUT_pt0_set) {
  {
    struct _dwg_entity_WIPEOUT *arg1 = (struct _dwg_entity_WIPEOUT *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_WIPEOUT_pt0_set(self,pt0);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_WIPEOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_WIPEOUT_pt0_set" "', argument " "1"" of type '" "struct _dwg_entity_WIPEOUT *""'"); 
    }
    arg1 = (struct _dwg_entity_WIPEOUT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_WIPEOUT_pt0_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->pt0 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_WIPEOUT_pt0_get) {
  {
    struct _dwg_entity_WIPEOUT *arg1 = (struct _dwg_entity_WIPEOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_WIPEOUT_pt0_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_WIPEOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_WIPEOUT_pt0_get" "', argument " "1"" of type '" "struct _dwg_entity_WIPEOUT *""'"); 
    }
    arg1 = (struct _dwg_entity_WIPEOUT *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->pt0);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_WIPEOUT_uvec_set) {
  {
    struct _dwg_entity_WIPEOUT *arg1 = (struct _dwg_entity_WIPEOUT *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_WIPEOUT_uvec_set(self,uvec);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_WIPEOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_WIPEOUT_uvec_set" "', argument " "1"" of type '" "struct _dwg_entity_WIPEOUT *""'"); 
    }
    arg1 = (struct _dwg_entity_WIPEOUT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_WIPEOUT_uvec_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->uvec = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_WIPEOUT_uvec_get) {
  {
    struct _dwg_entity_WIPEOUT *arg1 = (struct _dwg_entity_WIPEOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_WIPEOUT_uvec_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_WIPEOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_WIPEOUT_uvec_get" "', argument " "1"" of type '" "struct _dwg_entity_WIPEOUT *""'"); 
    }
    arg1 = (struct _dwg_entity_WIPEOUT *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->uvec);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_WIPEOUT_vvec_set) {
  {
    struct _dwg_entity_WIPEOUT *arg1 = (struct _dwg_entity_WIPEOUT *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_WIPEOUT_vvec_set(self,vvec);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_WIPEOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_WIPEOUT_vvec_set" "', argument " "1"" of type '" "struct _dwg_entity_WIPEOUT *""'"); 
    }
    arg1 = (struct _dwg_entity_WIPEOUT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_WIPEOUT_vvec_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->vvec = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_WIPEOUT_vvec_get) {
  {
    struct _dwg_entity_WIPEOUT *arg1 = (struct _dwg_entity_WIPEOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_WIPEOUT_vvec_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_WIPEOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_WIPEOUT_vvec_get" "', argument " "1"" of type '" "struct _dwg_entity_WIPEOUT *""'"); 
    }
    arg1 = (struct _dwg_entity_WIPEOUT *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->vvec);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_WIPEOUT_image_size_set) {
  {
    struct _dwg_entity_WIPEOUT *arg1 = (struct _dwg_entity_WIPEOUT *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_WIPEOUT_image_size_set(self,image_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_WIPEOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_WIPEOUT_image_size_set" "', argument " "1"" of type '" "struct _dwg_entity_WIPEOUT *""'"); 
    }
    arg1 = (struct _dwg_entity_WIPEOUT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_WIPEOUT_image_size_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->image_size = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_WIPEOUT_image_size_get) {
  {
    struct _dwg_entity_WIPEOUT *arg1 = (struct _dwg_entity_WIPEOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_WIPEOUT_image_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_WIPEOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_WIPEOUT_image_size_get" "', argument " "1"" of type '" "struct _dwg_entity_WIPEOUT *""'"); 
    }
    arg1 = (struct _dwg_entity_WIPEOUT *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->image_size);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_WIPEOUT_display_props_set) {
  {
    struct _dwg_entity_WIPEOUT *arg1 = (struct _dwg_entity_WIPEOUT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_WIPEOUT_display_props_set(self,display_props);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_WIPEOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_WIPEOUT_display_props_set" "', argument " "1"" of type '" "struct _dwg_entity_WIPEOUT *""'"); 
    }
    arg1 = (struct _dwg_entity_WIPEOUT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_WIPEOUT_display_props_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->display_props = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_WIPEOUT_display_props_get) {
  {
    struct _dwg_entity_WIPEOUT *arg1 = (struct _dwg_entity_WIPEOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_WIPEOUT_display_props_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_WIPEOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_WIPEOUT_display_props_get" "', argument " "1"" of type '" "struct _dwg_entity_WIPEOUT *""'"); 
    }
    arg1 = (struct _dwg_entity_WIPEOUT *)(argp1);
    result = (BITCODE_BS) ((arg1)->display_props);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_WIPEOUT_clipping_set) {
  {
    struct _dwg_entity_WIPEOUT *arg1 = (struct _dwg_entity_WIPEOUT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_WIPEOUT_clipping_set(self,clipping);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_WIPEOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_WIPEOUT_clipping_set" "', argument " "1"" of type '" "struct _dwg_entity_WIPEOUT *""'"); 
    }
    arg1 = (struct _dwg_entity_WIPEOUT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_WIPEOUT_clipping_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->clipping = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_WIPEOUT_clipping_get) {
  {
    struct _dwg_entity_WIPEOUT *arg1 = (struct _dwg_entity_WIPEOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_WIPEOUT_clipping_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_WIPEOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_WIPEOUT_clipping_get" "', argument " "1"" of type '" "struct _dwg_entity_WIPEOUT *""'"); 
    }
    arg1 = (struct _dwg_entity_WIPEOUT *)(argp1);
    result = (BITCODE_B) ((arg1)->clipping);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_WIPEOUT_brightness_set) {
  {
    struct _dwg_entity_WIPEOUT *arg1 = (struct _dwg_entity_WIPEOUT *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_WIPEOUT_brightness_set(self,brightness);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_WIPEOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_WIPEOUT_brightness_set" "', argument " "1"" of type '" "struct _dwg_entity_WIPEOUT *""'"); 
    }
    arg1 = (struct _dwg_entity_WIPEOUT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_WIPEOUT_brightness_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->brightness = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_WIPEOUT_brightness_get) {
  {
    struct _dwg_entity_WIPEOUT *arg1 = (struct _dwg_entity_WIPEOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_WIPEOUT_brightness_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_WIPEOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_WIPEOUT_brightness_get" "', argument " "1"" of type '" "struct _dwg_entity_WIPEOUT *""'"); 
    }
    arg1 = (struct _dwg_entity_WIPEOUT *)(argp1);
    result = (BITCODE_RC) ((arg1)->brightness);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_WIPEOUT_contrast_set) {
  {
    struct _dwg_entity_WIPEOUT *arg1 = (struct _dwg_entity_WIPEOUT *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_WIPEOUT_contrast_set(self,contrast);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_WIPEOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_WIPEOUT_contrast_set" "', argument " "1"" of type '" "struct _dwg_entity_WIPEOUT *""'"); 
    }
    arg1 = (struct _dwg_entity_WIPEOUT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_WIPEOUT_contrast_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->contrast = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_WIPEOUT_contrast_get) {
  {
    struct _dwg_entity_WIPEOUT *arg1 = (struct _dwg_entity_WIPEOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_WIPEOUT_contrast_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_WIPEOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_WIPEOUT_contrast_get" "', argument " "1"" of type '" "struct _dwg_entity_WIPEOUT *""'"); 
    }
    arg1 = (struct _dwg_entity_WIPEOUT *)(argp1);
    result = (BITCODE_RC) ((arg1)->contrast);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_WIPEOUT_fade_set) {
  {
    struct _dwg_entity_WIPEOUT *arg1 = (struct _dwg_entity_WIPEOUT *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_WIPEOUT_fade_set(self,fade);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_WIPEOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_WIPEOUT_fade_set" "', argument " "1"" of type '" "struct _dwg_entity_WIPEOUT *""'"); 
    }
    arg1 = (struct _dwg_entity_WIPEOUT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_WIPEOUT_fade_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->fade = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_WIPEOUT_fade_get) {
  {
    struct _dwg_entity_WIPEOUT *arg1 = (struct _dwg_entity_WIPEOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_WIPEOUT_fade_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_WIPEOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_WIPEOUT_fade_get" "', argument " "1"" of type '" "struct _dwg_entity_WIPEOUT *""'"); 
    }
    arg1 = (struct _dwg_entity_WIPEOUT *)(argp1);
    result = (BITCODE_RC) ((arg1)->fade);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_WIPEOUT_clip_mode_set) {
  {
    struct _dwg_entity_WIPEOUT *arg1 = (struct _dwg_entity_WIPEOUT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_WIPEOUT_clip_mode_set(self,clip_mode);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_WIPEOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_WIPEOUT_clip_mode_set" "', argument " "1"" of type '" "struct _dwg_entity_WIPEOUT *""'"); 
    }
    arg1 = (struct _dwg_entity_WIPEOUT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_WIPEOUT_clip_mode_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->clip_mode = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_WIPEOUT_clip_mode_get) {
  {
    struct _dwg_entity_WIPEOUT *arg1 = (struct _dwg_entity_WIPEOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_WIPEOUT_clip_mode_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_WIPEOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_WIPEOUT_clip_mode_get" "', argument " "1"" of type '" "struct _dwg_entity_WIPEOUT *""'"); 
    }
    arg1 = (struct _dwg_entity_WIPEOUT *)(argp1);
    result = (BITCODE_B) ((arg1)->clip_mode);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_WIPEOUT_clip_boundary_type_set) {
  {
    struct _dwg_entity_WIPEOUT *arg1 = (struct _dwg_entity_WIPEOUT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_WIPEOUT_clip_boundary_type_set(self,clip_boundary_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_WIPEOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_WIPEOUT_clip_boundary_type_set" "', argument " "1"" of type '" "struct _dwg_entity_WIPEOUT *""'"); 
    }
    arg1 = (struct _dwg_entity_WIPEOUT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_WIPEOUT_clip_boundary_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->clip_boundary_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_WIPEOUT_clip_boundary_type_get) {
  {
    struct _dwg_entity_WIPEOUT *arg1 = (struct _dwg_entity_WIPEOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_WIPEOUT_clip_boundary_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_WIPEOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_WIPEOUT_clip_boundary_type_get" "', argument " "1"" of type '" "struct _dwg_entity_WIPEOUT *""'"); 
    }
    arg1 = (struct _dwg_entity_WIPEOUT *)(argp1);
    result = (BITCODE_BS) ((arg1)->clip_boundary_type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_WIPEOUT_num_clip_verts_set) {
  {
    struct _dwg_entity_WIPEOUT *arg1 = (struct _dwg_entity_WIPEOUT *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_WIPEOUT_num_clip_verts_set(self,num_clip_verts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_WIPEOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_WIPEOUT_num_clip_verts_set" "', argument " "1"" of type '" "struct _dwg_entity_WIPEOUT *""'"); 
    }
    arg1 = (struct _dwg_entity_WIPEOUT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_WIPEOUT_num_clip_verts_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_clip_verts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_WIPEOUT_num_clip_verts_get) {
  {
    struct _dwg_entity_WIPEOUT *arg1 = (struct _dwg_entity_WIPEOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_WIPEOUT_num_clip_verts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_WIPEOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_WIPEOUT_num_clip_verts_get" "', argument " "1"" of type '" "struct _dwg_entity_WIPEOUT *""'"); 
    }
    arg1 = (struct _dwg_entity_WIPEOUT *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_clip_verts);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_WIPEOUT_clip_verts_set) {
  {
    struct _dwg_entity_WIPEOUT *arg1 = (struct _dwg_entity_WIPEOUT *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_WIPEOUT_clip_verts_set(self,clip_verts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_WIPEOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_WIPEOUT_clip_verts_set" "', argument " "1"" of type '" "struct _dwg_entity_WIPEOUT *""'"); 
    }
    arg1 = (struct _dwg_entity_WIPEOUT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_WIPEOUT_clip_verts_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->clip_verts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_WIPEOUT_clip_verts_get) {
  {
    struct _dwg_entity_WIPEOUT *arg1 = (struct _dwg_entity_WIPEOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_WIPEOUT_clip_verts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_WIPEOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_WIPEOUT_clip_verts_get" "', argument " "1"" of type '" "struct _dwg_entity_WIPEOUT *""'"); 
    }
    arg1 = (struct _dwg_entity_WIPEOUT *)(argp1);
    result = (BITCODE_2RD *) ((arg1)->clip_verts);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_WIPEOUT_imagedef_set) {
  {
    struct _dwg_entity_WIPEOUT *arg1 = (struct _dwg_entity_WIPEOUT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_WIPEOUT_imagedef_set(self,imagedef);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_WIPEOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_WIPEOUT_imagedef_set" "', argument " "1"" of type '" "struct _dwg_entity_WIPEOUT *""'"); 
    }
    arg1 = (struct _dwg_entity_WIPEOUT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_WIPEOUT_imagedef_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->imagedef = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_WIPEOUT_imagedef_get) {
  {
    struct _dwg_entity_WIPEOUT *arg1 = (struct _dwg_entity_WIPEOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_WIPEOUT_imagedef_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_WIPEOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_WIPEOUT_imagedef_get" "', argument " "1"" of type '" "struct _dwg_entity_WIPEOUT *""'"); 
    }
    arg1 = (struct _dwg_entity_WIPEOUT *)(argp1);
    result = (BITCODE_H) ((arg1)->imagedef);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_WIPEOUT_imagedefreactor_set) {
  {
    struct _dwg_entity_WIPEOUT *arg1 = (struct _dwg_entity_WIPEOUT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_WIPEOUT_imagedefreactor_set(self,imagedefreactor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_WIPEOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_WIPEOUT_imagedefreactor_set" "', argument " "1"" of type '" "struct _dwg_entity_WIPEOUT *""'"); 
    }
    arg1 = (struct _dwg_entity_WIPEOUT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_WIPEOUT_imagedefreactor_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->imagedefreactor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_WIPEOUT_imagedefreactor_get) {
  {
    struct _dwg_entity_WIPEOUT *arg1 = (struct _dwg_entity_WIPEOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_WIPEOUT_imagedefreactor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_WIPEOUT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_WIPEOUT_imagedefreactor_get" "', argument " "1"" of type '" "struct _dwg_entity_WIPEOUT *""'"); 
    }
    arg1 = (struct _dwg_entity_WIPEOUT *)(argp1);
    result = (BITCODE_H) ((arg1)->imagedefreactor);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_WIPEOUT) {
  {
    int argvi = 0;
    struct _dwg_entity_WIPEOUT *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_WIPEOUT();");
    }
    result = (struct _dwg_entity_WIPEOUT *)calloc(1, sizeof(struct _dwg_entity_WIPEOUT));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_WIPEOUT, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_WIPEOUT) {
  {
    struct _dwg_entity_WIPEOUT *arg1 = (struct _dwg_entity_WIPEOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_WIPEOUT(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_WIPEOUT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_WIPEOUT" "', argument " "1"" of type '" "struct _dwg_entity_WIPEOUT *""'"); 
    }
    arg1 = (struct _dwg_entity_WIPEOUT *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_WIPEOUTVARIABLES_parent_set) {
  {
    struct _dwg_object_WIPEOUTVARIABLES *arg1 = (struct _dwg_object_WIPEOUTVARIABLES *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_WIPEOUTVARIABLES_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_WIPEOUTVARIABLES, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_WIPEOUTVARIABLES_parent_set" "', argument " "1"" of type '" "struct _dwg_object_WIPEOUTVARIABLES *""'"); 
    }
    arg1 = (struct _dwg_object_WIPEOUTVARIABLES *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_WIPEOUTVARIABLES_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_WIPEOUTVARIABLES_parent_get) {
  {
    struct _dwg_object_WIPEOUTVARIABLES *arg1 = (struct _dwg_object_WIPEOUTVARIABLES *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_WIPEOUTVARIABLES_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_WIPEOUTVARIABLES, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_WIPEOUTVARIABLES_parent_get" "', argument " "1"" of type '" "struct _dwg_object_WIPEOUTVARIABLES *""'"); 
    }
    arg1 = (struct _dwg_object_WIPEOUTVARIABLES *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_WIPEOUTVARIABLES_display_frame_set) {
  {
    struct _dwg_object_WIPEOUTVARIABLES *arg1 = (struct _dwg_object_WIPEOUTVARIABLES *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_WIPEOUTVARIABLES_display_frame_set(self,display_frame);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_WIPEOUTVARIABLES, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_WIPEOUTVARIABLES_display_frame_set" "', argument " "1"" of type '" "struct _dwg_object_WIPEOUTVARIABLES *""'"); 
    }
    arg1 = (struct _dwg_object_WIPEOUTVARIABLES *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_WIPEOUTVARIABLES_display_frame_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->display_frame = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_WIPEOUTVARIABLES_display_frame_get) {
  {
    struct _dwg_object_WIPEOUTVARIABLES *arg1 = (struct _dwg_object_WIPEOUTVARIABLES *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_WIPEOUTVARIABLES_display_frame_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_WIPEOUTVARIABLES, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_WIPEOUTVARIABLES_display_frame_get" "', argument " "1"" of type '" "struct _dwg_object_WIPEOUTVARIABLES *""'"); 
    }
    arg1 = (struct _dwg_object_WIPEOUTVARIABLES *)(argp1);
    result = (BITCODE_BS) ((arg1)->display_frame);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_WIPEOUTVARIABLES) {
  {
    int argvi = 0;
    struct _dwg_object_WIPEOUTVARIABLES *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_WIPEOUTVARIABLES();");
    }
    result = (struct _dwg_object_WIPEOUTVARIABLES *)calloc(1, sizeof(struct _dwg_object_WIPEOUTVARIABLES));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_WIPEOUTVARIABLES, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_WIPEOUTVARIABLES) {
  {
    struct _dwg_object_WIPEOUTVARIABLES *arg1 = (struct _dwg_object_WIPEOUTVARIABLES *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_WIPEOUTVARIABLES(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_WIPEOUTVARIABLES, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_WIPEOUTVARIABLES" "', argument " "1"" of type '" "struct _dwg_object_WIPEOUTVARIABLES *""'"); 
    }
    arg1 = (struct _dwg_object_WIPEOUTVARIABLES *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SECTIONOBJECT_parent_set) {
  {
    struct _dwg_entity_SECTIONOBJECT *arg1 = (struct _dwg_entity_SECTIONOBJECT *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SECTIONOBJECT_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SECTIONOBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SECTIONOBJECT_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_SECTIONOBJECT *""'"); 
    }
    arg1 = (struct _dwg_entity_SECTIONOBJECT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SECTIONOBJECT_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SECTIONOBJECT_parent_get) {
  {
    struct _dwg_entity_SECTIONOBJECT *arg1 = (struct _dwg_entity_SECTIONOBJECT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SECTIONOBJECT_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SECTIONOBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SECTIONOBJECT_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_SECTIONOBJECT *""'"); 
    }
    arg1 = (struct _dwg_entity_SECTIONOBJECT *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SECTIONOBJECT_state_set) {
  {
    struct _dwg_entity_SECTIONOBJECT *arg1 = (struct _dwg_entity_SECTIONOBJECT *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SECTIONOBJECT_state_set(self,state);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SECTIONOBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SECTIONOBJECT_state_set" "', argument " "1"" of type '" "struct _dwg_entity_SECTIONOBJECT *""'"); 
    }
    arg1 = (struct _dwg_entity_SECTIONOBJECT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SECTIONOBJECT_state_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->state = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SECTIONOBJECT_state_get) {
  {
    struct _dwg_entity_SECTIONOBJECT *arg1 = (struct _dwg_entity_SECTIONOBJECT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SECTIONOBJECT_state_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SECTIONOBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SECTIONOBJECT_state_get" "', argument " "1"" of type '" "struct _dwg_entity_SECTIONOBJECT *""'"); 
    }
    arg1 = (struct _dwg_entity_SECTIONOBJECT *)(argp1);
    result = (BITCODE_BL) ((arg1)->state);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SECTIONOBJECT_flags_set) {
  {
    struct _dwg_entity_SECTIONOBJECT *arg1 = (struct _dwg_entity_SECTIONOBJECT *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SECTIONOBJECT_flags_set(self,flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SECTIONOBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SECTIONOBJECT_flags_set" "', argument " "1"" of type '" "struct _dwg_entity_SECTIONOBJECT *""'"); 
    }
    arg1 = (struct _dwg_entity_SECTIONOBJECT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SECTIONOBJECT_flags_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SECTIONOBJECT_flags_get) {
  {
    struct _dwg_entity_SECTIONOBJECT *arg1 = (struct _dwg_entity_SECTIONOBJECT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SECTIONOBJECT_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SECTIONOBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SECTIONOBJECT_flags_get" "', argument " "1"" of type '" "struct _dwg_entity_SECTIONOBJECT *""'"); 
    }
    arg1 = (struct _dwg_entity_SECTIONOBJECT *)(argp1);
    result = (BITCODE_BL) ((arg1)->flags);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SECTIONOBJECT_name_set) {
  {
    struct _dwg_entity_SECTIONOBJECT *arg1 = (struct _dwg_entity_SECTIONOBJECT *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SECTIONOBJECT_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SECTIONOBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SECTIONOBJECT_name_set" "', argument " "1"" of type '" "struct _dwg_entity_SECTIONOBJECT *""'"); 
    }
    arg1 = (struct _dwg_entity_SECTIONOBJECT *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SECTIONOBJECT_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SECTIONOBJECT_name_get) {
  {
    struct _dwg_entity_SECTIONOBJECT *arg1 = (struct _dwg_entity_SECTIONOBJECT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SECTIONOBJECT_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SECTIONOBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SECTIONOBJECT_name_get" "', argument " "1"" of type '" "struct _dwg_entity_SECTIONOBJECT *""'"); 
    }
    arg1 = (struct _dwg_entity_SECTIONOBJECT *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SECTIONOBJECT_vert_dir_set) {
  {
    struct _dwg_entity_SECTIONOBJECT *arg1 = (struct _dwg_entity_SECTIONOBJECT *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SECTIONOBJECT_vert_dir_set(self,vert_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SECTIONOBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SECTIONOBJECT_vert_dir_set" "', argument " "1"" of type '" "struct _dwg_entity_SECTIONOBJECT *""'"); 
    }
    arg1 = (struct _dwg_entity_SECTIONOBJECT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SECTIONOBJECT_vert_dir_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->vert_dir = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SECTIONOBJECT_vert_dir_get) {
  {
    struct _dwg_entity_SECTIONOBJECT *arg1 = (struct _dwg_entity_SECTIONOBJECT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SECTIONOBJECT_vert_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SECTIONOBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SECTIONOBJECT_vert_dir_get" "', argument " "1"" of type '" "struct _dwg_entity_SECTIONOBJECT *""'"); 
    }
    arg1 = (struct _dwg_entity_SECTIONOBJECT *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->vert_dir);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SECTIONOBJECT_top_height_set) {
  {
    struct _dwg_entity_SECTIONOBJECT *arg1 = (struct _dwg_entity_SECTIONOBJECT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SECTIONOBJECT_top_height_set(self,top_height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SECTIONOBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SECTIONOBJECT_top_height_set" "', argument " "1"" of type '" "struct _dwg_entity_SECTIONOBJECT *""'"); 
    }
    arg1 = (struct _dwg_entity_SECTIONOBJECT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SECTIONOBJECT_top_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->top_height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SECTIONOBJECT_top_height_get) {
  {
    struct _dwg_entity_SECTIONOBJECT *arg1 = (struct _dwg_entity_SECTIONOBJECT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SECTIONOBJECT_top_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SECTIONOBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SECTIONOBJECT_top_height_get" "', argument " "1"" of type '" "struct _dwg_entity_SECTIONOBJECT *""'"); 
    }
    arg1 = (struct _dwg_entity_SECTIONOBJECT *)(argp1);
    result = (BITCODE_BD) ((arg1)->top_height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SECTIONOBJECT_bottom_height_set) {
  {
    struct _dwg_entity_SECTIONOBJECT *arg1 = (struct _dwg_entity_SECTIONOBJECT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SECTIONOBJECT_bottom_height_set(self,bottom_height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SECTIONOBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SECTIONOBJECT_bottom_height_set" "', argument " "1"" of type '" "struct _dwg_entity_SECTIONOBJECT *""'"); 
    }
    arg1 = (struct _dwg_entity_SECTIONOBJECT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SECTIONOBJECT_bottom_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->bottom_height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SECTIONOBJECT_bottom_height_get) {
  {
    struct _dwg_entity_SECTIONOBJECT *arg1 = (struct _dwg_entity_SECTIONOBJECT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SECTIONOBJECT_bottom_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SECTIONOBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SECTIONOBJECT_bottom_height_get" "', argument " "1"" of type '" "struct _dwg_entity_SECTIONOBJECT *""'"); 
    }
    arg1 = (struct _dwg_entity_SECTIONOBJECT *)(argp1);
    result = (BITCODE_BD) ((arg1)->bottom_height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SECTIONOBJECT_indicator_alpha_set) {
  {
    struct _dwg_entity_SECTIONOBJECT *arg1 = (struct _dwg_entity_SECTIONOBJECT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SECTIONOBJECT_indicator_alpha_set(self,indicator_alpha);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SECTIONOBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SECTIONOBJECT_indicator_alpha_set" "', argument " "1"" of type '" "struct _dwg_entity_SECTIONOBJECT *""'"); 
    }
    arg1 = (struct _dwg_entity_SECTIONOBJECT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SECTIONOBJECT_indicator_alpha_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->indicator_alpha = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SECTIONOBJECT_indicator_alpha_get) {
  {
    struct _dwg_entity_SECTIONOBJECT *arg1 = (struct _dwg_entity_SECTIONOBJECT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SECTIONOBJECT_indicator_alpha_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SECTIONOBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SECTIONOBJECT_indicator_alpha_get" "', argument " "1"" of type '" "struct _dwg_entity_SECTIONOBJECT *""'"); 
    }
    arg1 = (struct _dwg_entity_SECTIONOBJECT *)(argp1);
    result = (BITCODE_BS) ((arg1)->indicator_alpha);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SECTIONOBJECT_indicator_color_set) {
  {
    struct _dwg_entity_SECTIONOBJECT *arg1 = (struct _dwg_entity_SECTIONOBJECT *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SECTIONOBJECT_indicator_color_set(self,indicator_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SECTIONOBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SECTIONOBJECT_indicator_color_set" "', argument " "1"" of type '" "struct _dwg_entity_SECTIONOBJECT *""'"); 
    }
    arg1 = (struct _dwg_entity_SECTIONOBJECT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SECTIONOBJECT_indicator_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->indicator_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SECTIONOBJECT_indicator_color_get) {
  {
    struct _dwg_entity_SECTIONOBJECT *arg1 = (struct _dwg_entity_SECTIONOBJECT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SECTIONOBJECT_indicator_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SECTIONOBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SECTIONOBJECT_indicator_color_get" "', argument " "1"" of type '" "struct _dwg_entity_SECTIONOBJECT *""'"); 
    }
    arg1 = (struct _dwg_entity_SECTIONOBJECT *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->indicator_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SECTIONOBJECT_num_verts_set) {
  {
    struct _dwg_entity_SECTIONOBJECT *arg1 = (struct _dwg_entity_SECTIONOBJECT *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SECTIONOBJECT_num_verts_set(self,num_verts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SECTIONOBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SECTIONOBJECT_num_verts_set" "', argument " "1"" of type '" "struct _dwg_entity_SECTIONOBJECT *""'"); 
    }
    arg1 = (struct _dwg_entity_SECTIONOBJECT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SECTIONOBJECT_num_verts_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_verts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SECTIONOBJECT_num_verts_get) {
  {
    struct _dwg_entity_SECTIONOBJECT *arg1 = (struct _dwg_entity_SECTIONOBJECT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SECTIONOBJECT_num_verts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SECTIONOBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SECTIONOBJECT_num_verts_get" "', argument " "1"" of type '" "struct _dwg_entity_SECTIONOBJECT *""'"); 
    }
    arg1 = (struct _dwg_entity_SECTIONOBJECT *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_verts);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SECTIONOBJECT_verts_set) {
  {
    struct _dwg_entity_SECTIONOBJECT *arg1 = (struct _dwg_entity_SECTIONOBJECT *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SECTIONOBJECT_verts_set(self,verts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SECTIONOBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SECTIONOBJECT_verts_set" "', argument " "1"" of type '" "struct _dwg_entity_SECTIONOBJECT *""'"); 
    }
    arg1 = (struct _dwg_entity_SECTIONOBJECT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SECTIONOBJECT_verts_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->verts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SECTIONOBJECT_verts_get) {
  {
    struct _dwg_entity_SECTIONOBJECT *arg1 = (struct _dwg_entity_SECTIONOBJECT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SECTIONOBJECT_verts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SECTIONOBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SECTIONOBJECT_verts_get" "', argument " "1"" of type '" "struct _dwg_entity_SECTIONOBJECT *""'"); 
    }
    arg1 = (struct _dwg_entity_SECTIONOBJECT *)(argp1);
    result = (BITCODE_3BD *) ((arg1)->verts);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SECTIONOBJECT_num_blverts_set) {
  {
    struct _dwg_entity_SECTIONOBJECT *arg1 = (struct _dwg_entity_SECTIONOBJECT *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SECTIONOBJECT_num_blverts_set(self,num_blverts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SECTIONOBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SECTIONOBJECT_num_blverts_set" "', argument " "1"" of type '" "struct _dwg_entity_SECTIONOBJECT *""'"); 
    }
    arg1 = (struct _dwg_entity_SECTIONOBJECT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SECTIONOBJECT_num_blverts_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_blverts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SECTIONOBJECT_num_blverts_get) {
  {
    struct _dwg_entity_SECTIONOBJECT *arg1 = (struct _dwg_entity_SECTIONOBJECT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SECTIONOBJECT_num_blverts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SECTIONOBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SECTIONOBJECT_num_blverts_get" "', argument " "1"" of type '" "struct _dwg_entity_SECTIONOBJECT *""'"); 
    }
    arg1 = (struct _dwg_entity_SECTIONOBJECT *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_blverts);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SECTIONOBJECT_blverts_set) {
  {
    struct _dwg_entity_SECTIONOBJECT *arg1 = (struct _dwg_entity_SECTIONOBJECT *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SECTIONOBJECT_blverts_set(self,blverts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SECTIONOBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SECTIONOBJECT_blverts_set" "', argument " "1"" of type '" "struct _dwg_entity_SECTIONOBJECT *""'"); 
    }
    arg1 = (struct _dwg_entity_SECTIONOBJECT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SECTIONOBJECT_blverts_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->blverts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SECTIONOBJECT_blverts_get) {
  {
    struct _dwg_entity_SECTIONOBJECT *arg1 = (struct _dwg_entity_SECTIONOBJECT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SECTIONOBJECT_blverts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SECTIONOBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SECTIONOBJECT_blverts_get" "', argument " "1"" of type '" "struct _dwg_entity_SECTIONOBJECT *""'"); 
    }
    arg1 = (struct _dwg_entity_SECTIONOBJECT *)(argp1);
    result = (BITCODE_3BD *) ((arg1)->blverts);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SECTIONOBJECT_section_settings_set) {
  {
    struct _dwg_entity_SECTIONOBJECT *arg1 = (struct _dwg_entity_SECTIONOBJECT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SECTIONOBJECT_section_settings_set(self,section_settings);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SECTIONOBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SECTIONOBJECT_section_settings_set" "', argument " "1"" of type '" "struct _dwg_entity_SECTIONOBJECT *""'"); 
    }
    arg1 = (struct _dwg_entity_SECTIONOBJECT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SECTIONOBJECT_section_settings_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->section_settings = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SECTIONOBJECT_section_settings_get) {
  {
    struct _dwg_entity_SECTIONOBJECT *arg1 = (struct _dwg_entity_SECTIONOBJECT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SECTIONOBJECT_section_settings_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SECTIONOBJECT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SECTIONOBJECT_section_settings_get" "', argument " "1"" of type '" "struct _dwg_entity_SECTIONOBJECT *""'"); 
    }
    arg1 = (struct _dwg_entity_SECTIONOBJECT *)(argp1);
    result = (BITCODE_H) ((arg1)->section_settings);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_SECTIONOBJECT) {
  {
    int argvi = 0;
    struct _dwg_entity_SECTIONOBJECT *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_SECTIONOBJECT();");
    }
    result = (struct _dwg_entity_SECTIONOBJECT *)calloc(1, sizeof(struct _dwg_entity_SECTIONOBJECT));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_SECTIONOBJECT, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_SECTIONOBJECT) {
  {
    struct _dwg_entity_SECTIONOBJECT *arg1 = (struct _dwg_entity_SECTIONOBJECT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_SECTIONOBJECT(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SECTIONOBJECT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_SECTIONOBJECT" "', argument " "1"" of type '" "struct _dwg_entity_SECTIONOBJECT *""'"); 
    }
    arg1 = (struct _dwg_entity_SECTIONOBJECT *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_parent_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_parent_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VISUALSTYLE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_parent_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_parent_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_description_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_description_set(self,description);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_description_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VISUALSTYLE_description_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->description);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->description = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->description = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_description_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_description_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_description_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_TV) ((arg1)->description);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_style_type_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_style_type_set(self,style_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_style_type_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_style_type_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->style_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_style_type_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_style_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_style_type_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->style_type);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_ext_lighting_model_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_ext_lighting_model_set(self,ext_lighting_model);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_ext_lighting_model_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_ext_lighting_model_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->ext_lighting_model = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_ext_lighting_model_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_ext_lighting_model_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_ext_lighting_model_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->ext_lighting_model);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_internal_only_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_internal_only_set(self,internal_only);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_internal_only_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_internal_only_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->internal_only = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_internal_only_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_internal_only_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_internal_only_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->internal_only);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_face_lighting_model_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_face_lighting_model_set(self,face_lighting_model);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_face_lighting_model_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_face_lighting_model_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->face_lighting_model = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_face_lighting_model_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_face_lighting_model_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_face_lighting_model_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->face_lighting_model);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_face_lighting_model_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_face_lighting_model_int_set(self,face_lighting_model_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_face_lighting_model_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_face_lighting_model_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->face_lighting_model_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_face_lighting_model_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_face_lighting_model_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_face_lighting_model_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->face_lighting_model_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_face_lighting_quality_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_face_lighting_quality_set(self,face_lighting_quality);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_face_lighting_quality_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_face_lighting_quality_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->face_lighting_quality = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_face_lighting_quality_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_face_lighting_quality_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_face_lighting_quality_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->face_lighting_quality);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_face_lighting_quality_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_face_lighting_quality_int_set(self,face_lighting_quality_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_face_lighting_quality_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_face_lighting_quality_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->face_lighting_quality_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_face_lighting_quality_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_face_lighting_quality_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_face_lighting_quality_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->face_lighting_quality_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_face_color_mode_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_face_color_mode_set(self,face_color_mode);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_face_color_mode_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_face_color_mode_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->face_color_mode = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_face_color_mode_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_face_color_mode_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_face_color_mode_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->face_color_mode);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_face_color_mode_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_face_color_mode_int_set(self,face_color_mode_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_face_color_mode_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_face_color_mode_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->face_color_mode_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_face_color_mode_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_face_color_mode_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_face_color_mode_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->face_color_mode_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_face_opacity_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_face_opacity_set(self,face_opacity);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_face_opacity_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_face_opacity_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->face_opacity = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_face_opacity_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_face_opacity_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_face_opacity_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->face_opacity);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_face_opacity_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_face_opacity_int_set(self,face_opacity_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_face_opacity_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_face_opacity_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->face_opacity_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_face_opacity_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_face_opacity_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_face_opacity_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->face_opacity_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_face_specular_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_face_specular_set(self,face_specular);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_face_specular_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_face_specular_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->face_specular = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_face_specular_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_face_specular_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_face_specular_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->face_specular);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_face_specular_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_face_specular_int_set(self,face_specular_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_face_specular_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_face_specular_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->face_specular_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_face_specular_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_face_specular_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_face_specular_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->face_specular_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_face_modifier_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_face_modifier_set(self,face_modifier);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_face_modifier_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_face_modifier_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->face_modifier = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_face_modifier_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_face_modifier_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_face_modifier_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->face_modifier);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_face_modifier_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_face_modifier_int_set(self,face_modifier_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_face_modifier_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_face_modifier_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->face_modifier_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_face_modifier_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_face_modifier_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_face_modifier_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->face_modifier_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_face_mono_color_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_face_mono_color_set(self,face_mono_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_face_mono_color_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VISUALSTYLE_face_mono_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->face_mono_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_face_mono_color_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_face_mono_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_face_mono_color_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->face_mono_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_face_mono_color_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_face_mono_color_int_set(self,face_mono_color_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_face_mono_color_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_face_mono_color_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->face_mono_color_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_face_mono_color_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_face_mono_color_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_face_mono_color_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->face_mono_color_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_model_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_model_set(self,edge_model);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_model_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_model_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->edge_model = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_model_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_model_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_model_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->edge_model);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_model_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_model_int_set(self,edge_model_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_model_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_model_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->edge_model_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_model_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_model_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_model_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->edge_model_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_style_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_style_set(self,edge_style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_style_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_style_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->edge_style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_style_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_style_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->edge_style);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_style_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_style_int_set(self,edge_style_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_style_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_style_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->edge_style_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_style_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_style_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_style_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->edge_style_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_intersection_color_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_intersection_color_set(self,edge_intersection_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_intersection_color_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VISUALSTYLE_edge_intersection_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->edge_intersection_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_intersection_color_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_intersection_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_intersection_color_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->edge_intersection_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_intersection_color_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_intersection_color_int_set(self,edge_intersection_color_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_intersection_color_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_intersection_color_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->edge_intersection_color_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_intersection_color_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_intersection_color_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_intersection_color_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->edge_intersection_color_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_obscured_color_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_obscured_color_set(self,edge_obscured_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_obscured_color_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VISUALSTYLE_edge_obscured_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->edge_obscured_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_obscured_color_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_obscured_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_obscured_color_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->edge_obscured_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_obscured_color_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_obscured_color_int_set(self,edge_obscured_color_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_obscured_color_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_obscured_color_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->edge_obscured_color_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_obscured_color_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_obscured_color_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_obscured_color_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->edge_obscured_color_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_obscured_ltype_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_obscured_ltype_set(self,edge_obscured_ltype);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_obscured_ltype_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_obscured_ltype_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->edge_obscured_ltype = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_obscured_ltype_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_obscured_ltype_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_obscured_ltype_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->edge_obscured_ltype);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_obscured_ltype_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_obscured_ltype_int_set(self,edge_obscured_ltype_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_obscured_ltype_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_obscured_ltype_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->edge_obscured_ltype_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_obscured_ltype_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_obscured_ltype_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_obscured_ltype_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->edge_obscured_ltype_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_intersection_ltype_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_intersection_ltype_set(self,edge_intersection_ltype);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_intersection_ltype_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_intersection_ltype_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->edge_intersection_ltype = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_intersection_ltype_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_intersection_ltype_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_intersection_ltype_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->edge_intersection_ltype);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_intersection_ltype_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_intersection_ltype_int_set(self,edge_intersection_ltype_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_intersection_ltype_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_intersection_ltype_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->edge_intersection_ltype_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_intersection_ltype_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_intersection_ltype_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_intersection_ltype_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->edge_intersection_ltype_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_crease_angle_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_crease_angle_set(self,edge_crease_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_crease_angle_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_crease_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->edge_crease_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_crease_angle_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_crease_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_crease_angle_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->edge_crease_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_crease_angle_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_crease_angle_int_set(self,edge_crease_angle_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_crease_angle_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_crease_angle_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->edge_crease_angle_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_crease_angle_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_crease_angle_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_crease_angle_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->edge_crease_angle_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_modifier_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_modifier_set(self,edge_modifier);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_modifier_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_modifier_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->edge_modifier = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_modifier_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_modifier_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_modifier_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->edge_modifier);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_modifier_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_modifier_int_set(self,edge_modifier_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_modifier_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_modifier_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->edge_modifier_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_modifier_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_modifier_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_modifier_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->edge_modifier_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_color_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_color_set(self,edge_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_color_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VISUALSTYLE_edge_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->edge_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_color_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_color_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->edge_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_color_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_color_int_set(self,edge_color_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_color_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_color_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->edge_color_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_color_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_color_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_color_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->edge_color_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_opacity_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_opacity_set(self,edge_opacity);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_opacity_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_opacity_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->edge_opacity = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_opacity_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_opacity_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_opacity_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->edge_opacity);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_opacity_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_opacity_int_set(self,edge_opacity_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_opacity_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_opacity_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->edge_opacity_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_opacity_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_opacity_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_opacity_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->edge_opacity_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_width_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_width_set(self,edge_width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_width_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_width_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->edge_width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_width_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_width_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->edge_width);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_width_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_width_int_set(self,edge_width_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_width_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_width_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->edge_width_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_width_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_width_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_width_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->edge_width_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_overhang_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_overhang_set(self,edge_overhang);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_overhang_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_overhang_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->edge_overhang = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_overhang_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_overhang_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_overhang_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->edge_overhang);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_overhang_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_overhang_int_set(self,edge_overhang_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_overhang_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_overhang_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->edge_overhang_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_overhang_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_overhang_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_overhang_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->edge_overhang_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_jitter_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_jitter_set(self,edge_jitter);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_jitter_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_jitter_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->edge_jitter = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_jitter_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_jitter_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_jitter_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->edge_jitter);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_jitter_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_jitter_int_set(self,edge_jitter_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_jitter_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_jitter_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->edge_jitter_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_jitter_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_jitter_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_jitter_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->edge_jitter_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_silhouette_color_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_silhouette_color_set(self,edge_silhouette_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_silhouette_color_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VISUALSTYLE_edge_silhouette_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->edge_silhouette_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_silhouette_color_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_silhouette_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_silhouette_color_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->edge_silhouette_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_silhouette_color_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_silhouette_color_int_set(self,edge_silhouette_color_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_silhouette_color_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_silhouette_color_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->edge_silhouette_color_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_silhouette_color_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_silhouette_color_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_silhouette_color_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->edge_silhouette_color_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_silhouette_width_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_silhouette_width_set(self,edge_silhouette_width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_silhouette_width_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_silhouette_width_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->edge_silhouette_width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_silhouette_width_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_silhouette_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_silhouette_width_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->edge_silhouette_width);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_silhouette_width_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_silhouette_width_int_set(self,edge_silhouette_width_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_silhouette_width_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_silhouette_width_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->edge_silhouette_width_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_silhouette_width_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_silhouette_width_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_silhouette_width_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->edge_silhouette_width_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_halo_gap_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_halo_gap_set(self,edge_halo_gap);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_halo_gap_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_halo_gap_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->edge_halo_gap = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_halo_gap_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_halo_gap_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_halo_gap_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->edge_halo_gap);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_halo_gap_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_halo_gap_int_set(self,edge_halo_gap_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_halo_gap_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_halo_gap_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->edge_halo_gap_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_halo_gap_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_halo_gap_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_halo_gap_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->edge_halo_gap_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_isolines_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_isolines_set(self,edge_isolines);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_isolines_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_isolines_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->edge_isolines = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_isolines_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_isolines_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_isolines_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->edge_isolines);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_isolines_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_isolines_int_set(self,edge_isolines_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_isolines_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_isolines_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->edge_isolines_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_isolines_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_isolines_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_isolines_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->edge_isolines_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_do_hide_precision_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_do_hide_precision_set(self,edge_do_hide_precision);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_do_hide_precision_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_do_hide_precision_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->edge_do_hide_precision = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_do_hide_precision_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_do_hide_precision_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_do_hide_precision_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->edge_do_hide_precision);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_do_hide_precision_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_do_hide_precision_int_set(self,edge_do_hide_precision_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_do_hide_precision_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_do_hide_precision_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->edge_do_hide_precision_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_do_hide_precision_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_do_hide_precision_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_do_hide_precision_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->edge_do_hide_precision_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_style_apply_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_style_apply_set(self,edge_style_apply);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_style_apply_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_style_apply_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->edge_style_apply = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_style_apply_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_style_apply_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_style_apply_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->edge_style_apply);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_style_apply_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_style_apply_int_set(self,edge_style_apply_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_style_apply_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_style_apply_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->edge_style_apply_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_style_apply_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_style_apply_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_style_apply_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->edge_style_apply_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_display_settings_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_display_settings_set(self,display_settings);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_display_settings_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_display_settings_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->display_settings = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_display_settings_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_display_settings_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_display_settings_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->display_settings);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_display_settings_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_display_settings_int_set(self,display_settings_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_display_settings_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_display_settings_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->display_settings_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_display_settings_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_display_settings_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_display_settings_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->display_settings_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_display_brightness_bl_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_display_brightness_bl_set(self,display_brightness_bl);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_display_brightness_bl_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_display_brightness_bl_set" "', argument " "2"" of type '" "BITCODE_BLd""'");
    } 
    arg2 = (BITCODE_BLd)(val2);
    if (arg1) (arg1)->display_brightness_bl = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_display_brightness_bl_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_display_brightness_bl_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_display_brightness_bl_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BLd) ((arg1)->display_brightness_bl);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_display_brightness_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_display_brightness_set(self,display_brightness);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_display_brightness_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_display_brightness_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->display_brightness = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_display_brightness_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_display_brightness_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_display_brightness_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->display_brightness);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_display_brightness_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_display_brightness_int_set(self,display_brightness_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_display_brightness_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_display_brightness_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->display_brightness_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_display_brightness_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_display_brightness_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_display_brightness_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->display_brightness_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_display_shadow_type_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_display_shadow_type_set(self,display_shadow_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_display_shadow_type_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_display_shadow_type_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->display_shadow_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_display_shadow_type_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_display_shadow_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_display_shadow_type_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->display_shadow_type);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_display_shadow_type_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_display_shadow_type_int_set(self,display_shadow_type_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_display_shadow_type_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_display_shadow_type_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->display_shadow_type_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_display_shadow_type_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_display_shadow_type_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_display_shadow_type_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->display_shadow_type_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bd2007_45_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bd2007_45_set(self,bd2007_45);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bd2007_45_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_bd2007_45_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->bd2007_45 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bd2007_45_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bd2007_45_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bd2007_45_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->bd2007_45);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_num_props_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_num_props_set(self,num_props);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_num_props_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_num_props_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_props = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_num_props_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_num_props_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_num_props_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_props);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop1c_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop1c_set(self,b_prop1c);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop1c_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_b_prop1c_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->b_prop1c = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop1c_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop1c_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop1c_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->b_prop1c);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop1c_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop1c_int_set(self,b_prop1c_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop1c_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_b_prop1c_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->b_prop1c_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop1c_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop1c_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop1c_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->b_prop1c_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop1d_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop1d_set(self,b_prop1d);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop1d_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_b_prop1d_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->b_prop1d = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop1d_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop1d_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop1d_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->b_prop1d);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop1d_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop1d_int_set(self,b_prop1d_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop1d_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_b_prop1d_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->b_prop1d_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop1d_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop1d_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop1d_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->b_prop1d_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop1e_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop1e_set(self,b_prop1e);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop1e_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_b_prop1e_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->b_prop1e = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop1e_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop1e_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop1e_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->b_prop1e);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop1e_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop1e_int_set(self,b_prop1e_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop1e_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_b_prop1e_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->b_prop1e_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop1e_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop1e_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop1e_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->b_prop1e_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop1f_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop1f_set(self,b_prop1f);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop1f_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_b_prop1f_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->b_prop1f = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop1f_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop1f_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop1f_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->b_prop1f);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop1f_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop1f_int_set(self,b_prop1f_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop1f_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_b_prop1f_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->b_prop1f_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop1f_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop1f_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop1f_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->b_prop1f_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop20_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop20_set(self,b_prop20);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop20_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_b_prop20_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->b_prop20 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop20_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop20_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop20_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->b_prop20);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop20_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop20_int_set(self,b_prop20_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop20_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_b_prop20_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->b_prop20_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop20_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop20_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop20_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->b_prop20_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop21_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop21_set(self,b_prop21);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop21_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_b_prop21_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->b_prop21 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop21_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop21_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop21_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->b_prop21);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop21_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop21_int_set(self,b_prop21_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop21_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_b_prop21_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->b_prop21_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop21_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop21_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop21_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->b_prop21_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop22_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop22_set(self,b_prop22);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop22_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_b_prop22_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->b_prop22 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop22_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop22_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop22_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->b_prop22);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop22_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop22_int_set(self,b_prop22_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop22_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_b_prop22_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->b_prop22_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop22_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop22_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop22_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->b_prop22_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop23_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop23_set(self,b_prop23);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop23_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_b_prop23_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->b_prop23 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop23_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop23_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop23_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->b_prop23);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop23_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop23_int_set(self,b_prop23_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop23_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_b_prop23_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->b_prop23_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop23_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop23_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop23_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->b_prop23_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop24_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop24_set(self,b_prop24);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop24_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_b_prop24_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->b_prop24 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop24_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop24_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop24_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->b_prop24);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop24_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop24_int_set(self,b_prop24_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop24_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_b_prop24_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->b_prop24_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop24_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop24_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop24_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->b_prop24_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bl_prop25_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bl_prop25_set(self,bl_prop25);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop25_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop25_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bl_prop25 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bl_prop25_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bl_prop25_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop25_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->bl_prop25);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bl_prop25_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bl_prop25_int_set(self,bl_prop25_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop25_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop25_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->bl_prop25_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bl_prop25_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bl_prop25_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop25_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->bl_prop25_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bd_prop26_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bd_prop26_set(self,bd_prop26);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bd_prop26_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_bd_prop26_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->bd_prop26 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bd_prop26_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bd_prop26_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bd_prop26_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->bd_prop26);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bd_prop26_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bd_prop26_int_set(self,bd_prop26_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bd_prop26_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_bd_prop26_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->bd_prop26_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bd_prop26_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bd_prop26_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bd_prop26_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->bd_prop26_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bd_prop27_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bd_prop27_set(self,bd_prop27);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bd_prop27_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_bd_prop27_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->bd_prop27 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bd_prop27_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bd_prop27_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bd_prop27_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->bd_prop27);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bd_prop27_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bd_prop27_int_set(self,bd_prop27_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bd_prop27_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_bd_prop27_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->bd_prop27_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bd_prop27_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bd_prop27_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bd_prop27_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->bd_prop27_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bl_prop28_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bl_prop28_set(self,bl_prop28);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop28_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop28_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bl_prop28 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bl_prop28_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bl_prop28_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop28_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->bl_prop28);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bl_prop28_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bl_prop28_int_set(self,bl_prop28_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop28_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop28_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->bl_prop28_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bl_prop28_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bl_prop28_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop28_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->bl_prop28_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_c_prop29_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_c_prop29_set(self,c_prop29);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_c_prop29_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VISUALSTYLE_c_prop29_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->c_prop29 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_c_prop29_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_c_prop29_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_c_prop29_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->c_prop29);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_c_prop29_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_c_prop29_int_set(self,c_prop29_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_c_prop29_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_c_prop29_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->c_prop29_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_c_prop29_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_c_prop29_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_c_prop29_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->c_prop29_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bl_prop2a_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bl_prop2a_set(self,bl_prop2a);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop2a_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop2a_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bl_prop2a = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bl_prop2a_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bl_prop2a_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop2a_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->bl_prop2a);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bl_prop2a_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bl_prop2a_int_set(self,bl_prop2a_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop2a_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop2a_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->bl_prop2a_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bl_prop2a_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bl_prop2a_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop2a_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->bl_prop2a_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bl_prop2b_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bl_prop2b_set(self,bl_prop2b);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop2b_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop2b_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bl_prop2b = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bl_prop2b_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bl_prop2b_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop2b_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->bl_prop2b);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bl_prop2b_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bl_prop2b_int_set(self,bl_prop2b_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop2b_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop2b_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->bl_prop2b_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bl_prop2b_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bl_prop2b_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop2b_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->bl_prop2b_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_c_prop2c_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_c_prop2c_set(self,c_prop2c);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_c_prop2c_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VISUALSTYLE_c_prop2c_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->c_prop2c = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_c_prop2c_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_c_prop2c_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_c_prop2c_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->c_prop2c);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_c_prop2c_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_c_prop2c_int_set(self,c_prop2c_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_c_prop2c_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_c_prop2c_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->c_prop2c_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_c_prop2c_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_c_prop2c_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_c_prop2c_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->c_prop2c_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop2d_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop2d_set(self,b_prop2d);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop2d_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_b_prop2d_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->b_prop2d = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop2d_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop2d_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop2d_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->b_prop2d);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop2d_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop2d_int_set(self,b_prop2d_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop2d_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_b_prop2d_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->b_prop2d_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop2d_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop2d_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop2d_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->b_prop2d_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bl_prop2e_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bl_prop2e_set(self,bl_prop2e);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop2e_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop2e_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bl_prop2e = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bl_prop2e_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bl_prop2e_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop2e_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->bl_prop2e);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bl_prop2e_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bl_prop2e_int_set(self,bl_prop2e_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop2e_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop2e_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->bl_prop2e_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bl_prop2e_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bl_prop2e_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop2e_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->bl_prop2e_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bl_prop2f_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bl_prop2f_set(self,bl_prop2f);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop2f_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop2f_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bl_prop2f = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bl_prop2f_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bl_prop2f_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop2f_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->bl_prop2f);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bl_prop2f_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bl_prop2f_int_set(self,bl_prop2f_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop2f_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop2f_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->bl_prop2f_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bl_prop2f_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bl_prop2f_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop2f_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->bl_prop2f_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bl_prop30_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bl_prop30_set(self,bl_prop30);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop30_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop30_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bl_prop30 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bl_prop30_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bl_prop30_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop30_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->bl_prop30);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bl_prop30_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bl_prop30_int_set(self,bl_prop30_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop30_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop30_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->bl_prop30_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bl_prop30_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bl_prop30_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop30_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->bl_prop30_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop31_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop31_set(self,b_prop31);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop31_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_b_prop31_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->b_prop31 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop31_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop31_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop31_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->b_prop31);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop31_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop31_int_set(self,b_prop31_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop31_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_b_prop31_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->b_prop31_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop31_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop31_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop31_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->b_prop31_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bl_prop32_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bl_prop32_set(self,bl_prop32);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop32_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop32_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bl_prop32 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bl_prop32_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bl_prop32_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop32_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->bl_prop32);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bl_prop32_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bl_prop32_int_set(self,bl_prop32_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop32_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop32_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->bl_prop32_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bl_prop32_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bl_prop32_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bl_prop32_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->bl_prop32_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_c_prop33_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_c_prop33_set(self,c_prop33);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_c_prop33_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VISUALSTYLE_c_prop33_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->c_prop33 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_c_prop33_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_c_prop33_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_c_prop33_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->c_prop33);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_c_prop33_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_c_prop33_int_set(self,c_prop33_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_c_prop33_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_c_prop33_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->c_prop33_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_c_prop33_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_c_prop33_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_c_prop33_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->c_prop33_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bd_prop34_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bd_prop34_set(self,bd_prop34);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bd_prop34_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_bd_prop34_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->bd_prop34 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bd_prop34_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bd_prop34_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bd_prop34_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->bd_prop34);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bd_prop34_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bd_prop34_int_set(self,bd_prop34_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bd_prop34_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_bd_prop34_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->bd_prop34_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bd_prop34_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bd_prop34_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bd_prop34_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->bd_prop34_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_wiggle_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_wiggle_set(self,edge_wiggle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_wiggle_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_wiggle_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->edge_wiggle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_wiggle_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_wiggle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_wiggle_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->edge_wiggle);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_wiggle_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_wiggle_int_set(self,edge_wiggle_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_wiggle_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_edge_wiggle_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->edge_wiggle_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_edge_wiggle_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_edge_wiggle_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_edge_wiggle_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->edge_wiggle_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_strokes_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_strokes_set(self,strokes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_strokes_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_VISUALSTYLE_strokes_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->strokes);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->strokes = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->strokes = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_strokes_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_strokes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_strokes_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_TV) ((arg1)->strokes);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_strokes_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_strokes_int_set(self,strokes_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_strokes_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_strokes_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->strokes_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_strokes_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_strokes_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_strokes_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->strokes_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop37_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop37_set(self,b_prop37);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop37_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_b_prop37_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->b_prop37 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop37_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop37_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop37_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->b_prop37);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop37_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop37_int_set(self,b_prop37_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop37_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_b_prop37_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->b_prop37_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_b_prop37_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_b_prop37_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_b_prop37_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->b_prop37_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bd_prop38_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bd_prop38_set(self,bd_prop38);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bd_prop38_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_bd_prop38_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->bd_prop38 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bd_prop38_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bd_prop38_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bd_prop38_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->bd_prop38);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bd_prop38_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bd_prop38_int_set(self,bd_prop38_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bd_prop38_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_bd_prop38_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->bd_prop38_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bd_prop38_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bd_prop38_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bd_prop38_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->bd_prop38_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bd_prop39_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bd_prop39_set(self,bd_prop39);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bd_prop39_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_bd_prop39_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->bd_prop39 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bd_prop39_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bd_prop39_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bd_prop39_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->bd_prop39);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bd_prop39_int_set) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bd_prop39_int_set(self,bd_prop39_int);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bd_prop39_int_set" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_VISUALSTYLE_bd_prop39_int_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->bd_prop39_int = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_VISUALSTYLE_bd_prop39_int_get) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_VISUALSTYLE_bd_prop39_int_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_VISUALSTYLE_bd_prop39_int_get" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->bd_prop39_int);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_VISUALSTYLE) {
  {
    int argvi = 0;
    struct _dwg_object_VISUALSTYLE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_VISUALSTYLE();");
    }
    result = (struct _dwg_object_VISUALSTYLE *)calloc(1, sizeof(struct _dwg_object_VISUALSTYLE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_VISUALSTYLE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_VISUALSTYLE) {
  {
    struct _dwg_object_VISUALSTYLE *arg1 = (struct _dwg_object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_VISUALSTYLE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_VISUALSTYLE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_VISUALSTYLE" "', argument " "1"" of type '" "struct _dwg_object_VISUALSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_VISUALSTYLE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LIGHTLIST_light_parent_set) {
  {
    struct _dwg_LIGHTLIST_light *arg1 = (struct _dwg_LIGHTLIST_light *) 0 ;
    struct _dwg_object_LIGHTLIST *arg2 = (struct _dwg_object_LIGHTLIST *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LIGHTLIST_light_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LIGHTLIST_light, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LIGHTLIST_light_parent_set" "', argument " "1"" of type '" "struct _dwg_LIGHTLIST_light *""'"); 
    }
    arg1 = (struct _dwg_LIGHTLIST_light *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_LIGHTLIST, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_LIGHTLIST_light_parent_set" "', argument " "2"" of type '" "struct _dwg_object_LIGHTLIST *""'"); 
    }
    arg2 = (struct _dwg_object_LIGHTLIST *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LIGHTLIST_light_parent_get) {
  {
    struct _dwg_LIGHTLIST_light *arg1 = (struct _dwg_LIGHTLIST_light *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_LIGHTLIST *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LIGHTLIST_light_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LIGHTLIST_light, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LIGHTLIST_light_parent_get" "', argument " "1"" of type '" "struct _dwg_LIGHTLIST_light *""'"); 
    }
    arg1 = (struct _dwg_LIGHTLIST_light *)(argp1);
    result = (struct _dwg_object_LIGHTLIST *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_LIGHTLIST, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LIGHTLIST_light_name_set) {
  {
    struct _dwg_LIGHTLIST_light *arg1 = (struct _dwg_LIGHTLIST_light *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LIGHTLIST_light_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LIGHTLIST_light, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LIGHTLIST_light_name_set" "', argument " "1"" of type '" "struct _dwg_LIGHTLIST_light *""'"); 
    }
    arg1 = (struct _dwg_LIGHTLIST_light *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_LIGHTLIST_light_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LIGHTLIST_light_name_get) {
  {
    struct _dwg_LIGHTLIST_light *arg1 = (struct _dwg_LIGHTLIST_light *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LIGHTLIST_light_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LIGHTLIST_light, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LIGHTLIST_light_name_get" "', argument " "1"" of type '" "struct _dwg_LIGHTLIST_light *""'"); 
    }
    arg1 = (struct _dwg_LIGHTLIST_light *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LIGHTLIST_light_handle_set) {
  {
    struct _dwg_LIGHTLIST_light *arg1 = (struct _dwg_LIGHTLIST_light *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_LIGHTLIST_light_handle_set(self,handle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LIGHTLIST_light, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LIGHTLIST_light_handle_set" "', argument " "1"" of type '" "struct _dwg_LIGHTLIST_light *""'"); 
    }
    arg1 = (struct _dwg_LIGHTLIST_light *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_LIGHTLIST_light_handle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->handle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_LIGHTLIST_light_handle_get) {
  {
    struct _dwg_LIGHTLIST_light *arg1 = (struct _dwg_LIGHTLIST_light *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_LIGHTLIST_light_handle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LIGHTLIST_light, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_LIGHTLIST_light_handle_get" "', argument " "1"" of type '" "struct _dwg_LIGHTLIST_light *""'"); 
    }
    arg1 = (struct _dwg_LIGHTLIST_light *)(argp1);
    result = (BITCODE_H) ((arg1)->handle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_LIGHTLIST_light) {
  {
    int argvi = 0;
    struct _dwg_LIGHTLIST_light *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_LIGHTLIST_light();");
    }
    result = (struct _dwg_LIGHTLIST_light *)calloc(1, sizeof(struct _dwg_LIGHTLIST_light));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_LIGHTLIST_light, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_LIGHTLIST_light) {
  {
    struct _dwg_LIGHTLIST_light *arg1 = (struct _dwg_LIGHTLIST_light *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_LIGHTLIST_light(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_LIGHTLIST_light, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_LIGHTLIST_light" "', argument " "1"" of type '" "struct _dwg_LIGHTLIST_light *""'"); 
    }
    arg1 = (struct _dwg_LIGHTLIST_light *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LIGHTLIST_parent_set) {
  {
    struct _dwg_object_LIGHTLIST *arg1 = (struct _dwg_object_LIGHTLIST *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LIGHTLIST_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LIGHTLIST, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LIGHTLIST_parent_set" "', argument " "1"" of type '" "struct _dwg_object_LIGHTLIST *""'"); 
    }
    arg1 = (struct _dwg_object_LIGHTLIST *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LIGHTLIST_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LIGHTLIST_parent_get) {
  {
    struct _dwg_object_LIGHTLIST *arg1 = (struct _dwg_object_LIGHTLIST *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LIGHTLIST_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LIGHTLIST, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LIGHTLIST_parent_get" "', argument " "1"" of type '" "struct _dwg_object_LIGHTLIST *""'"); 
    }
    arg1 = (struct _dwg_object_LIGHTLIST *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LIGHTLIST_class_version_set) {
  {
    struct _dwg_object_LIGHTLIST *arg1 = (struct _dwg_object_LIGHTLIST *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LIGHTLIST_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LIGHTLIST, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LIGHTLIST_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_LIGHTLIST *""'"); 
    }
    arg1 = (struct _dwg_object_LIGHTLIST *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LIGHTLIST_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LIGHTLIST_class_version_get) {
  {
    struct _dwg_object_LIGHTLIST *arg1 = (struct _dwg_object_LIGHTLIST *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LIGHTLIST_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LIGHTLIST, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LIGHTLIST_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_LIGHTLIST *""'"); 
    }
    arg1 = (struct _dwg_object_LIGHTLIST *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LIGHTLIST_num_lights_set) {
  {
    struct _dwg_object_LIGHTLIST *arg1 = (struct _dwg_object_LIGHTLIST *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LIGHTLIST_num_lights_set(self,num_lights);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LIGHTLIST, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LIGHTLIST_num_lights_set" "', argument " "1"" of type '" "struct _dwg_object_LIGHTLIST *""'"); 
    }
    arg1 = (struct _dwg_object_LIGHTLIST *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LIGHTLIST_num_lights_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_lights = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LIGHTLIST_num_lights_get) {
  {
    struct _dwg_object_LIGHTLIST *arg1 = (struct _dwg_object_LIGHTLIST *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LIGHTLIST_num_lights_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LIGHTLIST, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LIGHTLIST_num_lights_get" "', argument " "1"" of type '" "struct _dwg_object_LIGHTLIST *""'"); 
    }
    arg1 = (struct _dwg_object_LIGHTLIST *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_lights);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LIGHTLIST_lights_set) {
  {
    struct _dwg_object_LIGHTLIST *arg1 = (struct _dwg_object_LIGHTLIST *) 0 ;
    Dwg_LIGHTLIST_light *arg2 = (Dwg_LIGHTLIST_light *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LIGHTLIST_lights_set(self,lights);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LIGHTLIST, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LIGHTLIST_lights_set" "', argument " "1"" of type '" "struct _dwg_object_LIGHTLIST *""'"); 
    }
    arg1 = (struct _dwg_object_LIGHTLIST *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_LIGHTLIST_light, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LIGHTLIST_lights_set" "', argument " "2"" of type '" "Dwg_LIGHTLIST_light *""'"); 
    }
    arg2 = (Dwg_LIGHTLIST_light *)(argp2);
    if (arg1) (arg1)->lights = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LIGHTLIST_lights_get) {
  {
    struct _dwg_object_LIGHTLIST *arg1 = (struct _dwg_object_LIGHTLIST *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_LIGHTLIST_light *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LIGHTLIST_lights_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LIGHTLIST, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LIGHTLIST_lights_get" "', argument " "1"" of type '" "struct _dwg_object_LIGHTLIST *""'"); 
    }
    arg1 = (struct _dwg_object_LIGHTLIST *)(argp1);
    result = (Dwg_LIGHTLIST_light *) ((arg1)->lights);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_LIGHTLIST_light, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_LIGHTLIST) {
  {
    int argvi = 0;
    struct _dwg_object_LIGHTLIST *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_LIGHTLIST();");
    }
    result = (struct _dwg_object_LIGHTLIST *)calloc(1, sizeof(struct _dwg_object_LIGHTLIST));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_LIGHTLIST, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_LIGHTLIST) {
  {
    struct _dwg_object_LIGHTLIST *arg1 = (struct _dwg_object_LIGHTLIST *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_LIGHTLIST(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LIGHTLIST, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_LIGHTLIST" "', argument " "1"" of type '" "struct _dwg_object_LIGHTLIST *""'"); 
    }
    arg1 = (struct _dwg_object_LIGHTLIST *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_color_parent_set) {
  {
    struct _dwg_MATERIAL_color *arg1 = (struct _dwg_MATERIAL_color *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MATERIAL_color_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_color_parent_set" "', argument " "1"" of type '" "struct _dwg_MATERIAL_color *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_color *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MATERIAL_color_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_color_parent_get) {
  {
    struct _dwg_MATERIAL_color *arg1 = (struct _dwg_MATERIAL_color *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MATERIAL_color_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_color_parent_get" "', argument " "1"" of type '" "struct _dwg_MATERIAL_color *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_color *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_color_flag_set) {
  {
    struct _dwg_MATERIAL_color *arg1 = (struct _dwg_MATERIAL_color *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MATERIAL_color_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_color_flag_set" "', argument " "1"" of type '" "struct _dwg_MATERIAL_color *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_color *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MATERIAL_color_flag_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_color_flag_get) {
  {
    struct _dwg_MATERIAL_color *arg1 = (struct _dwg_MATERIAL_color *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MATERIAL_color_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_color_flag_get" "', argument " "1"" of type '" "struct _dwg_MATERIAL_color *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_color *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_color_factor_set) {
  {
    struct _dwg_MATERIAL_color *arg1 = (struct _dwg_MATERIAL_color *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MATERIAL_color_factor_set(self,factor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_color_factor_set" "', argument " "1"" of type '" "struct _dwg_MATERIAL_color *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_color *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MATERIAL_color_factor_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->factor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_color_factor_get) {
  {
    struct _dwg_MATERIAL_color *arg1 = (struct _dwg_MATERIAL_color *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MATERIAL_color_factor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_color_factor_get" "', argument " "1"" of type '" "struct _dwg_MATERIAL_color *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_color *)(argp1);
    result = (BITCODE_BD) ((arg1)->factor);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_color_rgb_set) {
  {
    struct _dwg_MATERIAL_color *arg1 = (struct _dwg_MATERIAL_color *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MATERIAL_color_rgb_set(self,rgb);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_color_rgb_set" "', argument " "1"" of type '" "struct _dwg_MATERIAL_color *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_color *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MATERIAL_color_rgb_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->rgb = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_color_rgb_get) {
  {
    struct _dwg_MATERIAL_color *arg1 = (struct _dwg_MATERIAL_color *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MATERIAL_color_rgb_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_color, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_color_rgb_get" "', argument " "1"" of type '" "struct _dwg_MATERIAL_color *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_color *)(argp1);
    result = (BITCODE_BL) ((arg1)->rgb);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_MATERIAL_color) {
  {
    int argvi = 0;
    struct _dwg_MATERIAL_color *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_MATERIAL_color();");
    }
    result = (struct _dwg_MATERIAL_color *)calloc(1, sizeof(struct _dwg_MATERIAL_color));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_MATERIAL_color, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_MATERIAL_color) {
  {
    struct _dwg_MATERIAL_color *arg1 = (struct _dwg_MATERIAL_color *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_MATERIAL_color(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_color, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_MATERIAL_color" "', argument " "1"" of type '" "struct _dwg_MATERIAL_color *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_color *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_mapper_parent_set) {
  {
    struct _dwg_MATERIAL_mapper *arg1 = (struct _dwg_MATERIAL_mapper *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MATERIAL_mapper_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_mapper, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_mapper_parent_set" "', argument " "1"" of type '" "struct _dwg_MATERIAL_mapper *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_mapper *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MATERIAL_mapper_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_mapper_parent_get) {
  {
    struct _dwg_MATERIAL_mapper *arg1 = (struct _dwg_MATERIAL_mapper *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MATERIAL_mapper_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_mapper, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_mapper_parent_get" "', argument " "1"" of type '" "struct _dwg_MATERIAL_mapper *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_mapper *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_mapper_blendfactor_set) {
  {
    struct _dwg_MATERIAL_mapper *arg1 = (struct _dwg_MATERIAL_mapper *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MATERIAL_mapper_blendfactor_set(self,blendfactor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_mapper, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_mapper_blendfactor_set" "', argument " "1"" of type '" "struct _dwg_MATERIAL_mapper *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_mapper *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MATERIAL_mapper_blendfactor_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->blendfactor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_mapper_blendfactor_get) {
  {
    struct _dwg_MATERIAL_mapper *arg1 = (struct _dwg_MATERIAL_mapper *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MATERIAL_mapper_blendfactor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_mapper, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_mapper_blendfactor_get" "', argument " "1"" of type '" "struct _dwg_MATERIAL_mapper *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_mapper *)(argp1);
    result = (BITCODE_BD) ((arg1)->blendfactor);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_mapper_transmatrix_set) {
  {
    struct _dwg_MATERIAL_mapper *arg1 = (struct _dwg_MATERIAL_mapper *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MATERIAL_mapper_transmatrix_set(self,transmatrix);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_mapper, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_mapper_transmatrix_set" "', argument " "1"" of type '" "struct _dwg_MATERIAL_mapper *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_mapper *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MATERIAL_mapper_transmatrix_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->transmatrix = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_mapper_transmatrix_get) {
  {
    struct _dwg_MATERIAL_mapper *arg1 = (struct _dwg_MATERIAL_mapper *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MATERIAL_mapper_transmatrix_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_mapper, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_mapper_transmatrix_get" "', argument " "1"" of type '" "struct _dwg_MATERIAL_mapper *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_mapper *)(argp1);
    result = (BITCODE_BD *) ((arg1)->transmatrix);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_mapper_filename_set) {
  {
    struct _dwg_MATERIAL_mapper *arg1 = (struct _dwg_MATERIAL_mapper *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MATERIAL_mapper_filename_set(self,filename);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_mapper, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_mapper_filename_set" "', argument " "1"" of type '" "struct _dwg_MATERIAL_mapper *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_mapper *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MATERIAL_mapper_filename_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->filename);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->filename = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->filename = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_mapper_filename_get) {
  {
    struct _dwg_MATERIAL_mapper *arg1 = (struct _dwg_MATERIAL_mapper *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MATERIAL_mapper_filename_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_mapper, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_mapper_filename_get" "', argument " "1"" of type '" "struct _dwg_MATERIAL_mapper *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_mapper *)(argp1);
    result = (BITCODE_TV) ((arg1)->filename);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_mapper_color1_set) {
  {
    struct _dwg_MATERIAL_mapper *arg1 = (struct _dwg_MATERIAL_mapper *) 0 ;
    Dwg_MATERIAL_color *arg2 = (Dwg_MATERIAL_color *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MATERIAL_mapper_color1_set(self,color1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_mapper, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_mapper_color1_set" "', argument " "1"" of type '" "struct _dwg_MATERIAL_mapper *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_mapper *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_MATERIAL_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MATERIAL_mapper_color1_set" "', argument " "2"" of type '" "Dwg_MATERIAL_color *""'"); 
    }
    arg2 = (Dwg_MATERIAL_color *)(argp2);
    if (arg1) (arg1)->color1 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_mapper_color1_get) {
  {
    struct _dwg_MATERIAL_mapper *arg1 = (struct _dwg_MATERIAL_mapper *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_MATERIAL_color *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MATERIAL_mapper_color1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_mapper, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_mapper_color1_get" "', argument " "1"" of type '" "struct _dwg_MATERIAL_mapper *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_mapper *)(argp1);
    result = (Dwg_MATERIAL_color *)& ((arg1)->color1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_MATERIAL_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_mapper_color2_set) {
  {
    struct _dwg_MATERIAL_mapper *arg1 = (struct _dwg_MATERIAL_mapper *) 0 ;
    Dwg_MATERIAL_color *arg2 = (Dwg_MATERIAL_color *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MATERIAL_mapper_color2_set(self,color2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_mapper, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_mapper_color2_set" "', argument " "1"" of type '" "struct _dwg_MATERIAL_mapper *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_mapper *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_MATERIAL_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MATERIAL_mapper_color2_set" "', argument " "2"" of type '" "Dwg_MATERIAL_color *""'"); 
    }
    arg2 = (Dwg_MATERIAL_color *)(argp2);
    if (arg1) (arg1)->color2 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_mapper_color2_get) {
  {
    struct _dwg_MATERIAL_mapper *arg1 = (struct _dwg_MATERIAL_mapper *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_MATERIAL_color *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MATERIAL_mapper_color2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_mapper, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_mapper_color2_get" "', argument " "1"" of type '" "struct _dwg_MATERIAL_mapper *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_mapper *)(argp1);
    result = (Dwg_MATERIAL_color *)& ((arg1)->color2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_MATERIAL_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_mapper_source_set) {
  {
    struct _dwg_MATERIAL_mapper *arg1 = (struct _dwg_MATERIAL_mapper *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MATERIAL_mapper_source_set(self,source);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_mapper, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_mapper_source_set" "', argument " "1"" of type '" "struct _dwg_MATERIAL_mapper *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_mapper *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MATERIAL_mapper_source_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->source = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_mapper_source_get) {
  {
    struct _dwg_MATERIAL_mapper *arg1 = (struct _dwg_MATERIAL_mapper *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MATERIAL_mapper_source_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_mapper, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_mapper_source_get" "', argument " "1"" of type '" "struct _dwg_MATERIAL_mapper *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_mapper *)(argp1);
    result = (BITCODE_RC) ((arg1)->source);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_mapper_projection_set) {
  {
    struct _dwg_MATERIAL_mapper *arg1 = (struct _dwg_MATERIAL_mapper *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MATERIAL_mapper_projection_set(self,projection);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_mapper, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_mapper_projection_set" "', argument " "1"" of type '" "struct _dwg_MATERIAL_mapper *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_mapper *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MATERIAL_mapper_projection_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->projection = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_mapper_projection_get) {
  {
    struct _dwg_MATERIAL_mapper *arg1 = (struct _dwg_MATERIAL_mapper *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MATERIAL_mapper_projection_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_mapper, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_mapper_projection_get" "', argument " "1"" of type '" "struct _dwg_MATERIAL_mapper *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_mapper *)(argp1);
    result = (BITCODE_RC) ((arg1)->projection);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_mapper_tiling_set) {
  {
    struct _dwg_MATERIAL_mapper *arg1 = (struct _dwg_MATERIAL_mapper *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MATERIAL_mapper_tiling_set(self,tiling);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_mapper, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_mapper_tiling_set" "', argument " "1"" of type '" "struct _dwg_MATERIAL_mapper *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_mapper *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MATERIAL_mapper_tiling_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->tiling = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_mapper_tiling_get) {
  {
    struct _dwg_MATERIAL_mapper *arg1 = (struct _dwg_MATERIAL_mapper *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MATERIAL_mapper_tiling_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_mapper, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_mapper_tiling_get" "', argument " "1"" of type '" "struct _dwg_MATERIAL_mapper *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_mapper *)(argp1);
    result = (BITCODE_RC) ((arg1)->tiling);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_mapper_autotransform_set) {
  {
    struct _dwg_MATERIAL_mapper *arg1 = (struct _dwg_MATERIAL_mapper *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MATERIAL_mapper_autotransform_set(self,autotransform);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_mapper, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_mapper_autotransform_set" "', argument " "1"" of type '" "struct _dwg_MATERIAL_mapper *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_mapper *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MATERIAL_mapper_autotransform_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->autotransform = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_mapper_autotransform_get) {
  {
    struct _dwg_MATERIAL_mapper *arg1 = (struct _dwg_MATERIAL_mapper *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MATERIAL_mapper_autotransform_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_mapper, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_mapper_autotransform_get" "', argument " "1"" of type '" "struct _dwg_MATERIAL_mapper *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_mapper *)(argp1);
    result = (BITCODE_RC) ((arg1)->autotransform);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_mapper_texturemode_set) {
  {
    struct _dwg_MATERIAL_mapper *arg1 = (struct _dwg_MATERIAL_mapper *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MATERIAL_mapper_texturemode_set(self,texturemode);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_mapper, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_mapper_texturemode_set" "', argument " "1"" of type '" "struct _dwg_MATERIAL_mapper *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_mapper *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MATERIAL_mapper_texturemode_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->texturemode = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_mapper_texturemode_get) {
  {
    struct _dwg_MATERIAL_mapper *arg1 = (struct _dwg_MATERIAL_mapper *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MATERIAL_mapper_texturemode_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_mapper, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_mapper_texturemode_get" "', argument " "1"" of type '" "struct _dwg_MATERIAL_mapper *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_mapper *)(argp1);
    result = (BITCODE_BS) ((arg1)->texturemode);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_MATERIAL_mapper) {
  {
    int argvi = 0;
    struct _dwg_MATERIAL_mapper *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_MATERIAL_mapper();");
    }
    result = (struct _dwg_MATERIAL_mapper *)calloc(1, sizeof(struct _dwg_MATERIAL_mapper));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_MATERIAL_mapper, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_MATERIAL_mapper) {
  {
    struct _dwg_MATERIAL_mapper *arg1 = (struct _dwg_MATERIAL_mapper *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_MATERIAL_mapper(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_mapper, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_MATERIAL_mapper" "', argument " "1"" of type '" "struct _dwg_MATERIAL_mapper *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_mapper *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_gentexture_parent_set) {
  {
    struct _dwg_MATERIAL_gentexture *arg1 = (struct _dwg_MATERIAL_gentexture *) 0 ;
    struct _dwg_object_MATERIAL *arg2 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MATERIAL_gentexture_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_gentexture, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_gentexture_parent_set" "', argument " "1"" of type '" "struct _dwg_MATERIAL_gentexture *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_gentexture *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_MATERIAL, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MATERIAL_gentexture_parent_set" "', argument " "2"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg2 = (struct _dwg_object_MATERIAL *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_gentexture_parent_get) {
  {
    struct _dwg_MATERIAL_gentexture *arg1 = (struct _dwg_MATERIAL_gentexture *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_MATERIAL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MATERIAL_gentexture_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_gentexture, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_gentexture_parent_get" "', argument " "1"" of type '" "struct _dwg_MATERIAL_gentexture *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_gentexture *)(argp1);
    result = (struct _dwg_object_MATERIAL *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_MATERIAL, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_gentexture_genprocname_set) {
  {
    struct _dwg_MATERIAL_gentexture *arg1 = (struct _dwg_MATERIAL_gentexture *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MATERIAL_gentexture_genprocname_set(self,genprocname);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_gentexture, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_gentexture_genprocname_set" "', argument " "1"" of type '" "struct _dwg_MATERIAL_gentexture *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_gentexture *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MATERIAL_gentexture_genprocname_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->genprocname);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->genprocname = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->genprocname = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_gentexture_genprocname_get) {
  {
    struct _dwg_MATERIAL_gentexture *arg1 = (struct _dwg_MATERIAL_gentexture *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MATERIAL_gentexture_genprocname_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_gentexture, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_gentexture_genprocname_get" "', argument " "1"" of type '" "struct _dwg_MATERIAL_gentexture *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_gentexture *)(argp1);
    result = (BITCODE_TV) ((arg1)->genprocname);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_gentexture_material_set) {
  {
    struct _dwg_MATERIAL_gentexture *arg1 = (struct _dwg_MATERIAL_gentexture *) 0 ;
    struct _dwg_object_MATERIAL *arg2 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MATERIAL_gentexture_material_set(self,material);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_gentexture, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_gentexture_material_set" "', argument " "1"" of type '" "struct _dwg_MATERIAL_gentexture *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_gentexture *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_MATERIAL, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MATERIAL_gentexture_material_set" "', argument " "2"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg2 = (struct _dwg_object_MATERIAL *)(argp2);
    if (arg1) (arg1)->material = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MATERIAL_gentexture_material_get) {
  {
    struct _dwg_MATERIAL_gentexture *arg1 = (struct _dwg_MATERIAL_gentexture *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_MATERIAL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MATERIAL_gentexture_material_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_gentexture, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MATERIAL_gentexture_material_get" "', argument " "1"" of type '" "struct _dwg_MATERIAL_gentexture *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_gentexture *)(argp1);
    result = (struct _dwg_object_MATERIAL *) ((arg1)->material);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_MATERIAL, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_MATERIAL_gentexture) {
  {
    int argvi = 0;
    struct _dwg_MATERIAL_gentexture *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_MATERIAL_gentexture();");
    }
    result = (struct _dwg_MATERIAL_gentexture *)calloc(1, sizeof(struct _dwg_MATERIAL_gentexture));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_MATERIAL_gentexture, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_MATERIAL_gentexture) {
  {
    struct _dwg_MATERIAL_gentexture *arg1 = (struct _dwg_MATERIAL_gentexture *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_MATERIAL_gentexture(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MATERIAL_gentexture, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_MATERIAL_gentexture" "', argument " "1"" of type '" "struct _dwg_MATERIAL_gentexture *""'"); 
    }
    arg1 = (struct _dwg_MATERIAL_gentexture *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_parent_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_parent_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MATERIAL_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_parent_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_parent_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_name_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_name_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MATERIAL_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_name_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_name_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_description_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_description_set(self,description);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_description_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MATERIAL_description_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->description);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->description = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->description = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_description_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_description_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_description_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (BITCODE_TV) ((arg1)->description);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_ambient_color_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    Dwg_MATERIAL_color *arg2 = (Dwg_MATERIAL_color *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_ambient_color_set(self,ambient_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_ambient_color_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_MATERIAL_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MATERIAL_ambient_color_set" "', argument " "2"" of type '" "Dwg_MATERIAL_color *""'"); 
    }
    arg2 = (Dwg_MATERIAL_color *)(argp2);
    if (arg1) (arg1)->ambient_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_ambient_color_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_MATERIAL_color *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_ambient_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_ambient_color_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (Dwg_MATERIAL_color *)& ((arg1)->ambient_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_MATERIAL_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_diffuse_color_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    Dwg_MATERIAL_color *arg2 = (Dwg_MATERIAL_color *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_diffuse_color_set(self,diffuse_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_diffuse_color_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_MATERIAL_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MATERIAL_diffuse_color_set" "', argument " "2"" of type '" "Dwg_MATERIAL_color *""'"); 
    }
    arg2 = (Dwg_MATERIAL_color *)(argp2);
    if (arg1) (arg1)->diffuse_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_diffuse_color_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_MATERIAL_color *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_diffuse_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_diffuse_color_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (Dwg_MATERIAL_color *)& ((arg1)->diffuse_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_MATERIAL_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_diffusemap_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    Dwg_MATERIAL_mapper *arg2 = (Dwg_MATERIAL_mapper *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_diffusemap_set(self,diffusemap);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_diffusemap_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_MATERIAL_mapper, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MATERIAL_diffusemap_set" "', argument " "2"" of type '" "Dwg_MATERIAL_mapper *""'"); 
    }
    arg2 = (Dwg_MATERIAL_mapper *)(argp2);
    if (arg1) (arg1)->diffusemap = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_diffusemap_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_MATERIAL_mapper *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_diffusemap_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_diffusemap_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (Dwg_MATERIAL_mapper *)& ((arg1)->diffusemap);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_MATERIAL_mapper, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_specular_gloss_factor_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_specular_gloss_factor_set(self,specular_gloss_factor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_specular_gloss_factor_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MATERIAL_specular_gloss_factor_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->specular_gloss_factor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_specular_gloss_factor_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_specular_gloss_factor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_specular_gloss_factor_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (BITCODE_BD) ((arg1)->specular_gloss_factor);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_specular_color_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    Dwg_MATERIAL_color *arg2 = (Dwg_MATERIAL_color *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_specular_color_set(self,specular_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_specular_color_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_MATERIAL_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MATERIAL_specular_color_set" "', argument " "2"" of type '" "Dwg_MATERIAL_color *""'"); 
    }
    arg2 = (Dwg_MATERIAL_color *)(argp2);
    if (arg1) (arg1)->specular_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_specular_color_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_MATERIAL_color *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_specular_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_specular_color_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (Dwg_MATERIAL_color *)& ((arg1)->specular_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_MATERIAL_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_specularmap_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    Dwg_MATERIAL_mapper *arg2 = (Dwg_MATERIAL_mapper *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_specularmap_set(self,specularmap);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_specularmap_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_MATERIAL_mapper, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MATERIAL_specularmap_set" "', argument " "2"" of type '" "Dwg_MATERIAL_mapper *""'"); 
    }
    arg2 = (Dwg_MATERIAL_mapper *)(argp2);
    if (arg1) (arg1)->specularmap = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_specularmap_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_MATERIAL_mapper *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_specularmap_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_specularmap_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (Dwg_MATERIAL_mapper *)& ((arg1)->specularmap);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_MATERIAL_mapper, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_reflectionmap_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    Dwg_MATERIAL_mapper *arg2 = (Dwg_MATERIAL_mapper *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_reflectionmap_set(self,reflectionmap);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_reflectionmap_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_MATERIAL_mapper, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MATERIAL_reflectionmap_set" "', argument " "2"" of type '" "Dwg_MATERIAL_mapper *""'"); 
    }
    arg2 = (Dwg_MATERIAL_mapper *)(argp2);
    if (arg1) (arg1)->reflectionmap = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_reflectionmap_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_MATERIAL_mapper *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_reflectionmap_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_reflectionmap_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (Dwg_MATERIAL_mapper *)& ((arg1)->reflectionmap);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_MATERIAL_mapper, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_opacity_percent_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_opacity_percent_set(self,opacity_percent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_opacity_percent_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MATERIAL_opacity_percent_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->opacity_percent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_opacity_percent_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_opacity_percent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_opacity_percent_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (BITCODE_BD) ((arg1)->opacity_percent);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_opacitymap_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    Dwg_MATERIAL_mapper *arg2 = (Dwg_MATERIAL_mapper *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_opacitymap_set(self,opacitymap);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_opacitymap_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_MATERIAL_mapper, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MATERIAL_opacitymap_set" "', argument " "2"" of type '" "Dwg_MATERIAL_mapper *""'"); 
    }
    arg2 = (Dwg_MATERIAL_mapper *)(argp2);
    if (arg1) (arg1)->opacitymap = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_opacitymap_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_MATERIAL_mapper *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_opacitymap_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_opacitymap_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (Dwg_MATERIAL_mapper *)& ((arg1)->opacitymap);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_MATERIAL_mapper, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_bumpmap_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    Dwg_MATERIAL_mapper *arg2 = (Dwg_MATERIAL_mapper *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_bumpmap_set(self,bumpmap);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_bumpmap_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_MATERIAL_mapper, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MATERIAL_bumpmap_set" "', argument " "2"" of type '" "Dwg_MATERIAL_mapper *""'"); 
    }
    arg2 = (Dwg_MATERIAL_mapper *)(argp2);
    if (arg1) (arg1)->bumpmap = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_bumpmap_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_MATERIAL_mapper *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_bumpmap_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_bumpmap_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (Dwg_MATERIAL_mapper *)& ((arg1)->bumpmap);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_MATERIAL_mapper, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_refraction_index_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_refraction_index_set(self,refraction_index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_refraction_index_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MATERIAL_refraction_index_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->refraction_index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_refraction_index_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_refraction_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_refraction_index_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (BITCODE_BD) ((arg1)->refraction_index);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_refractionmap_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    Dwg_MATERIAL_mapper *arg2 = (Dwg_MATERIAL_mapper *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_refractionmap_set(self,refractionmap);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_refractionmap_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_MATERIAL_mapper, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MATERIAL_refractionmap_set" "', argument " "2"" of type '" "Dwg_MATERIAL_mapper *""'"); 
    }
    arg2 = (Dwg_MATERIAL_mapper *)(argp2);
    if (arg1) (arg1)->refractionmap = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_refractionmap_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_MATERIAL_mapper *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_refractionmap_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_refractionmap_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (Dwg_MATERIAL_mapper *)& ((arg1)->refractionmap);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_MATERIAL_mapper, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_color_bleed_scale_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_color_bleed_scale_set(self,color_bleed_scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_color_bleed_scale_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MATERIAL_color_bleed_scale_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->color_bleed_scale = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_color_bleed_scale_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_color_bleed_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_color_bleed_scale_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (BITCODE_BD) ((arg1)->color_bleed_scale);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_indirect_bump_scale_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_indirect_bump_scale_set(self,indirect_bump_scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_indirect_bump_scale_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MATERIAL_indirect_bump_scale_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->indirect_bump_scale = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_indirect_bump_scale_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_indirect_bump_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_indirect_bump_scale_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (BITCODE_BD) ((arg1)->indirect_bump_scale);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_reflectance_scale_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_reflectance_scale_set(self,reflectance_scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_reflectance_scale_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MATERIAL_reflectance_scale_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->reflectance_scale = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_reflectance_scale_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_reflectance_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_reflectance_scale_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (BITCODE_BD) ((arg1)->reflectance_scale);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_transmittance_scale_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_transmittance_scale_set(self,transmittance_scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_transmittance_scale_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MATERIAL_transmittance_scale_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->transmittance_scale = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_transmittance_scale_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_transmittance_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_transmittance_scale_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (BITCODE_BD) ((arg1)->transmittance_scale);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_two_sided_material_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_two_sided_material_set(self,two_sided_material);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_two_sided_material_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MATERIAL_two_sided_material_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->two_sided_material = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_two_sided_material_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_two_sided_material_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_two_sided_material_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (BITCODE_B) ((arg1)->two_sided_material);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_luminance_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_luminance_set(self,luminance);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_luminance_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MATERIAL_luminance_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->luminance = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_luminance_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_luminance_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_luminance_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (BITCODE_BD) ((arg1)->luminance);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_luminance_mode_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_luminance_mode_set(self,luminance_mode);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_luminance_mode_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MATERIAL_luminance_mode_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->luminance_mode = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_luminance_mode_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_luminance_mode_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_luminance_mode_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (BITCODE_BS) ((arg1)->luminance_mode);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_translucence_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_translucence_set(self,translucence);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_translucence_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MATERIAL_translucence_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->translucence = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_translucence_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_translucence_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_translucence_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (BITCODE_BD) ((arg1)->translucence);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_self_illumination_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_self_illumination_set(self,self_illumination);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_self_illumination_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MATERIAL_self_illumination_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->self_illumination = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_self_illumination_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_self_illumination_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_self_illumination_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (BITCODE_BD) ((arg1)->self_illumination);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_reflectivity_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_reflectivity_set(self,reflectivity);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_reflectivity_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MATERIAL_reflectivity_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->reflectivity = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_reflectivity_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_reflectivity_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_reflectivity_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (BITCODE_BD) ((arg1)->reflectivity);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_illumination_model_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_illumination_model_set(self,illumination_model);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_illumination_model_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MATERIAL_illumination_model_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->illumination_model = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_illumination_model_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_illumination_model_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_illumination_model_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (BITCODE_BL) ((arg1)->illumination_model);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_channel_flags_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_channel_flags_set(self,channel_flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_channel_flags_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MATERIAL_channel_flags_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->channel_flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_channel_flags_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_channel_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_channel_flags_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (BITCODE_BL) ((arg1)->channel_flags);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_mode_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_mode_set(self,mode);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_mode_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MATERIAL_mode_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->mode = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_mode_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_mode_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_mode_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (BITCODE_BL) ((arg1)->mode);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_genprocname_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_genprocname_set(self,genprocname);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_genprocname_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MATERIAL_genprocname_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->genprocname);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->genprocname = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->genprocname = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_genprocname_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_genprocname_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_genprocname_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (BITCODE_TV) ((arg1)->genprocname);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_genproctype_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_genproctype_set(self,genproctype);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_genproctype_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MATERIAL_genproctype_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->genproctype = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_genproctype_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_genproctype_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_genproctype_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (BITCODE_BS) ((arg1)->genproctype);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_genprocvalbool_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_genprocvalbool_set(self,genprocvalbool);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_genprocvalbool_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MATERIAL_genprocvalbool_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->genprocvalbool = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_genprocvalbool_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_genprocvalbool_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_genprocvalbool_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (BITCODE_B) ((arg1)->genprocvalbool);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_genprocvalint_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_genprocvalint_set(self,genprocvalint);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_genprocvalint_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MATERIAL_genprocvalint_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->genprocvalint = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_genprocvalint_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_genprocvalint_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_genprocvalint_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (BITCODE_BS) ((arg1)->genprocvalint);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_genprocvalreal_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_genprocvalreal_set(self,genprocvalreal);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_genprocvalreal_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MATERIAL_genprocvalreal_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->genprocvalreal = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_genprocvalreal_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_genprocvalreal_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_genprocvalreal_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (BITCODE_BD) ((arg1)->genprocvalreal);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_genprocvaltext_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_genprocvaltext_set(self,genprocvaltext);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_genprocvaltext_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MATERIAL_genprocvaltext_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->genprocvaltext);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->genprocvaltext = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->genprocvaltext = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_genprocvaltext_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_genprocvaltext_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_genprocvaltext_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (BITCODE_TV) ((arg1)->genprocvaltext);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_genprocvalcolor_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_genprocvalcolor_set(self,genprocvalcolor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_genprocvalcolor_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MATERIAL_genprocvalcolor_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->genprocvalcolor = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_genprocvalcolor_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_genprocvalcolor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_genprocvalcolor_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->genprocvalcolor);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_genproctableend_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_genproctableend_set(self,genproctableend);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_genproctableend_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MATERIAL_genproctableend_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->genproctableend = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_genproctableend_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_genproctableend_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_genproctableend_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (BITCODE_B) ((arg1)->genproctableend);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_num_gentextures_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_num_gentextures_set(self,num_gentextures);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_num_gentextures_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MATERIAL_num_gentextures_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_gentextures = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_num_gentextures_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_num_gentextures_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_num_gentextures_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_gentextures);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_gentextures_set) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    Dwg_MATERIAL_gentexture *arg2 = (Dwg_MATERIAL_gentexture *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_gentextures_set(self,gentextures);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_gentextures_set" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_MATERIAL_gentexture, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MATERIAL_gentextures_set" "', argument " "2"" of type '" "Dwg_MATERIAL_gentexture *""'"); 
    }
    arg2 = (Dwg_MATERIAL_gentexture *)(argp2);
    if (arg1) (arg1)->gentextures = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MATERIAL_gentextures_get) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_MATERIAL_gentexture *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MATERIAL_gentextures_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MATERIAL_gentextures_get" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    result = (Dwg_MATERIAL_gentexture *) ((arg1)->gentextures);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_MATERIAL_gentexture, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_MATERIAL) {
  {
    int argvi = 0;
    struct _dwg_object_MATERIAL *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_MATERIAL();");
    }
    result = (struct _dwg_object_MATERIAL *)calloc(1, sizeof(struct _dwg_object_MATERIAL));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_MATERIAL, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_MATERIAL) {
  {
    struct _dwg_object_MATERIAL *arg1 = (struct _dwg_object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_MATERIAL(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MATERIAL, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_MATERIAL" "', argument " "1"" of type '" "struct _dwg_object_MATERIAL *""'"); 
    }
    arg1 = (struct _dwg_object_MATERIAL *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_OBJECT_PTR_parent_set) {
  {
    struct _dwg_object_OBJECT_PTR *arg1 = (struct _dwg_object_OBJECT_PTR *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_OBJECT_PTR_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_OBJECT_PTR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_OBJECT_PTR_parent_set" "', argument " "1"" of type '" "struct _dwg_object_OBJECT_PTR *""'"); 
    }
    arg1 = (struct _dwg_object_OBJECT_PTR *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_OBJECT_PTR_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_OBJECT_PTR_parent_get) {
  {
    struct _dwg_object_OBJECT_PTR *arg1 = (struct _dwg_object_OBJECT_PTR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_OBJECT_PTR_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_OBJECT_PTR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_OBJECT_PTR_parent_get" "', argument " "1"" of type '" "struct _dwg_object_OBJECT_PTR *""'"); 
    }
    arg1 = (struct _dwg_object_OBJECT_PTR *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_OBJECT_PTR) {
  {
    int argvi = 0;
    struct _dwg_object_OBJECT_PTR *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_OBJECT_PTR();");
    }
    result = (struct _dwg_object_OBJECT_PTR *)calloc(1, sizeof(struct _dwg_object_OBJECT_PTR));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_OBJECT_PTR, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_OBJECT_PTR) {
  {
    struct _dwg_object_OBJECT_PTR *arg1 = (struct _dwg_object_OBJECT_PTR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_OBJECT_PTR(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_OBJECT_PTR, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_OBJECT_PTR" "', argument " "1"" of type '" "struct _dwg_object_OBJECT_PTR *""'"); 
    }
    arg1 = (struct _dwg_object_OBJECT_PTR *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_parent_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LIGHT_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_parent_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_class_version_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_class_version_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_class_version_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_class_version_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_name_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_name_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LIGHT_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_name_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_name_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_type_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_type_set(self,type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_type_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_type_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_type_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_type_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_BL) ((arg1)->type);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_status_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_status_set(self,status);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_status_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_status_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->status = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_status_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_status_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_status_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_B) ((arg1)->status);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_light_color_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_light_color_set(self,light_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_light_color_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LIGHT_light_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->light_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_light_color_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_light_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_light_color_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->light_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_plot_glyph_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_plot_glyph_set(self,plot_glyph);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_plot_glyph_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_plot_glyph_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->plot_glyph = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_plot_glyph_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_plot_glyph_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_plot_glyph_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_B) ((arg1)->plot_glyph);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_intensity_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_intensity_set(self,intensity);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_intensity_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_intensity_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->intensity = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_intensity_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_intensity_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_intensity_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_BD) ((arg1)->intensity);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_position_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_position_set(self,position);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_position_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LIGHT_position_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->position = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_position_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_position_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_position_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->position);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_target_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_target_set(self,target);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_target_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LIGHT_target_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->target = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_target_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_target_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_target_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->target);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_attenuation_type_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_attenuation_type_set(self,attenuation_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_attenuation_type_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_attenuation_type_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->attenuation_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_attenuation_type_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_attenuation_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_attenuation_type_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_BL) ((arg1)->attenuation_type);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_use_attenuation_limits_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_use_attenuation_limits_set(self,use_attenuation_limits);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_use_attenuation_limits_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_use_attenuation_limits_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->use_attenuation_limits = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_use_attenuation_limits_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_use_attenuation_limits_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_use_attenuation_limits_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_B) ((arg1)->use_attenuation_limits);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_attenuation_start_limit_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_attenuation_start_limit_set(self,attenuation_start_limit);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_attenuation_start_limit_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_attenuation_start_limit_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->attenuation_start_limit = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_attenuation_start_limit_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_attenuation_start_limit_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_attenuation_start_limit_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_BD) ((arg1)->attenuation_start_limit);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_attenuation_end_limit_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_attenuation_end_limit_set(self,attenuation_end_limit);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_attenuation_end_limit_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_attenuation_end_limit_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->attenuation_end_limit = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_attenuation_end_limit_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_attenuation_end_limit_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_attenuation_end_limit_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_BD) ((arg1)->attenuation_end_limit);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_hotspot_angle_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_hotspot_angle_set(self,hotspot_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_hotspot_angle_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_hotspot_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->hotspot_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_hotspot_angle_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_hotspot_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_hotspot_angle_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_BD) ((arg1)->hotspot_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_falloff_angle_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_falloff_angle_set(self,falloff_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_falloff_angle_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_falloff_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->falloff_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_falloff_angle_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_falloff_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_falloff_angle_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_BD) ((arg1)->falloff_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_cast_shadows_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_cast_shadows_set(self,cast_shadows);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_cast_shadows_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_cast_shadows_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->cast_shadows = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_cast_shadows_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_cast_shadows_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_cast_shadows_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_B) ((arg1)->cast_shadows);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_shadow_type_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_shadow_type_set(self,shadow_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_shadow_type_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_shadow_type_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->shadow_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_shadow_type_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_shadow_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_shadow_type_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_BL) ((arg1)->shadow_type);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_shadow_map_size_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_shadow_map_size_set(self,shadow_map_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_shadow_map_size_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_shadow_map_size_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->shadow_map_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_shadow_map_size_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_shadow_map_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_shadow_map_size_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_BS) ((arg1)->shadow_map_size);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_shadow_map_softness_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_shadow_map_softness_set(self,shadow_map_softness);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_shadow_map_softness_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_shadow_map_softness_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->shadow_map_softness = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_shadow_map_softness_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_shadow_map_softness_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_shadow_map_softness_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_RC) ((arg1)->shadow_map_softness);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_is_photometric_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_is_photometric_set(self,is_photometric);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_is_photometric_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_is_photometric_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_photometric = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_is_photometric_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_is_photometric_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_is_photometric_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_B) ((arg1)->is_photometric);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_has_photometric_data_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_has_photometric_data_set(self,has_photometric_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_has_photometric_data_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_has_photometric_data_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_photometric_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_has_photometric_data_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_has_photometric_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_has_photometric_data_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_B) ((arg1)->has_photometric_data);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_has_webfile_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_has_webfile_set(self,has_webfile);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_has_webfile_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_has_webfile_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_webfile = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_has_webfile_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_has_webfile_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_has_webfile_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_B) ((arg1)->has_webfile);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_webfile_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_webfile_set(self,webfile);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_webfile_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LIGHT_webfile_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->webfile);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->webfile = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->webfile = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_webfile_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_webfile_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_webfile_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_TV) ((arg1)->webfile);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_physical_intensity_method_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_physical_intensity_method_set(self,physical_intensity_method);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_physical_intensity_method_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_physical_intensity_method_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->physical_intensity_method = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_physical_intensity_method_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_physical_intensity_method_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_physical_intensity_method_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_BS) ((arg1)->physical_intensity_method);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_physical_intensity_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_physical_intensity_set(self,physical_intensity);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_physical_intensity_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_physical_intensity_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->physical_intensity = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_physical_intensity_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_physical_intensity_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_physical_intensity_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_BD) ((arg1)->physical_intensity);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_illuminance_dist_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_illuminance_dist_set(self,illuminance_dist);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_illuminance_dist_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_illuminance_dist_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->illuminance_dist = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_illuminance_dist_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_illuminance_dist_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_illuminance_dist_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_BD) ((arg1)->illuminance_dist);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_lamp_color_type_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_lamp_color_type_set(self,lamp_color_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_lamp_color_type_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_lamp_color_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->lamp_color_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_lamp_color_type_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_lamp_color_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_lamp_color_type_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_BS) ((arg1)->lamp_color_type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_lamp_color_temp_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_lamp_color_temp_set(self,lamp_color_temp);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_lamp_color_temp_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_lamp_color_temp_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->lamp_color_temp = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_lamp_color_temp_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_lamp_color_temp_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_lamp_color_temp_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_BD) ((arg1)->lamp_color_temp);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_lamp_color_preset_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_lamp_color_preset_set(self,lamp_color_preset);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_lamp_color_preset_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_lamp_color_preset_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->lamp_color_preset = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_lamp_color_preset_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_lamp_color_preset_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_lamp_color_preset_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_BS) ((arg1)->lamp_color_preset);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_lamp_color_rgb_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_lamp_color_rgb_set(self,lamp_color_rgb);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_lamp_color_rgb_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_lamp_color_rgb_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->lamp_color_rgb = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_lamp_color_rgb_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_lamp_color_rgb_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_lamp_color_rgb_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_BL) ((arg1)->lamp_color_rgb);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_web_rotation_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_web_rotation_set(self,web_rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_web_rotation_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LIGHT_web_rotation_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->web_rotation = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_web_rotation_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_web_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_web_rotation_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->web_rotation);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_extlight_shape_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_extlight_shape_set(self,extlight_shape);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_extlight_shape_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_extlight_shape_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->extlight_shape = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_extlight_shape_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_extlight_shape_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_extlight_shape_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_BS) ((arg1)->extlight_shape);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_extlight_length_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_extlight_length_set(self,extlight_length);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_extlight_length_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_extlight_length_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->extlight_length = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_extlight_length_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_extlight_length_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_extlight_length_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_BD) ((arg1)->extlight_length);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_extlight_width_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_extlight_width_set(self,extlight_width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_extlight_width_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_extlight_width_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->extlight_width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_extlight_width_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_extlight_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_extlight_width_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_BD) ((arg1)->extlight_width);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_extlight_radius_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_extlight_radius_set(self,extlight_radius);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_extlight_radius_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_extlight_radius_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->extlight_radius = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_extlight_radius_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_extlight_radius_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_extlight_radius_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_BD) ((arg1)->extlight_radius);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_webfile_type_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_webfile_type_set(self,webfile_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_webfile_type_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_webfile_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->webfile_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_webfile_type_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_webfile_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_webfile_type_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_BS) ((arg1)->webfile_type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_web_symetry_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_web_symetry_set(self,web_symetry);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_web_symetry_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_web_symetry_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->web_symetry = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_web_symetry_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_web_symetry_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_web_symetry_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_BS) ((arg1)->web_symetry);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_has_target_grip_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_has_target_grip_set(self,has_target_grip);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_has_target_grip_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_has_target_grip_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->has_target_grip = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_has_target_grip_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_has_target_grip_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_has_target_grip_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_BS) ((arg1)->has_target_grip);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_web_flux_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_web_flux_set(self,web_flux);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_web_flux_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_web_flux_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->web_flux = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_web_flux_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_web_flux_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_web_flux_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_BD) ((arg1)->web_flux);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_web_angle1_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_web_angle1_set(self,web_angle1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_web_angle1_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_web_angle1_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->web_angle1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_web_angle1_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_web_angle1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_web_angle1_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_BD) ((arg1)->web_angle1);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_web_angle2_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_web_angle2_set(self,web_angle2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_web_angle2_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_web_angle2_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->web_angle2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_web_angle2_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_web_angle2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_web_angle2_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_BD) ((arg1)->web_angle2);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_web_angle3_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_web_angle3_set(self,web_angle3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_web_angle3_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_web_angle3_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->web_angle3 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_web_angle3_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_web_angle3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_web_angle3_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_BD) ((arg1)->web_angle3);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_web_angle4_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_web_angle4_set(self,web_angle4);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_web_angle4_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_web_angle4_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->web_angle4 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_web_angle4_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_web_angle4_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_web_angle4_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_BD) ((arg1)->web_angle4);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_web_angle5_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_web_angle5_set(self,web_angle5);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_web_angle5_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_web_angle5_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->web_angle5 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_web_angle5_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_web_angle5_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_web_angle5_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_BD) ((arg1)->web_angle5);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_glyph_display_type_set) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_glyph_display_type_set(self,glyph_display_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_glyph_display_type_set" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LIGHT_glyph_display_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->glyph_display_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LIGHT_glyph_display_type_get) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LIGHT_glyph_display_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LIGHT_glyph_display_type_get" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    result = (BITCODE_BS) ((arg1)->glyph_display_type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_LIGHT) {
  {
    int argvi = 0;
    struct _dwg_entity_LIGHT *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_LIGHT();");
    }
    result = (struct _dwg_entity_LIGHT *)calloc(1, sizeof(struct _dwg_entity_LIGHT));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_LIGHT, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_LIGHT) {
  {
    struct _dwg_entity_LIGHT *arg1 = (struct _dwg_entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_LIGHT(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LIGHT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_LIGHT" "', argument " "1"" of type '" "struct _dwg_entity_LIGHT *""'"); 
    }
    arg1 = (struct _dwg_entity_LIGHT *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_CAMERA_parent_set) {
  {
    struct _dwg_entity_CAMERA *arg1 = (struct _dwg_entity_CAMERA *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_CAMERA_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_CAMERA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_CAMERA_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_CAMERA *""'"); 
    }
    arg1 = (struct _dwg_entity_CAMERA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_CAMERA_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_CAMERA_parent_get) {
  {
    struct _dwg_entity_CAMERA *arg1 = (struct _dwg_entity_CAMERA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_CAMERA_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_CAMERA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_CAMERA_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_CAMERA *""'"); 
    }
    arg1 = (struct _dwg_entity_CAMERA *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_CAMERA_view_set) {
  {
    struct _dwg_entity_CAMERA *arg1 = (struct _dwg_entity_CAMERA *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_CAMERA_view_set(self,view);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_CAMERA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_CAMERA_view_set" "', argument " "1"" of type '" "struct _dwg_entity_CAMERA *""'"); 
    }
    arg1 = (struct _dwg_entity_CAMERA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_CAMERA_view_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->view = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_CAMERA_view_get) {
  {
    struct _dwg_entity_CAMERA *arg1 = (struct _dwg_entity_CAMERA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_CAMERA_view_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_CAMERA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_CAMERA_view_get" "', argument " "1"" of type '" "struct _dwg_entity_CAMERA *""'"); 
    }
    arg1 = (struct _dwg_entity_CAMERA *)(argp1);
    result = (BITCODE_H) ((arg1)->view);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_CAMERA) {
  {
    int argvi = 0;
    struct _dwg_entity_CAMERA *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_CAMERA();");
    }
    result = (struct _dwg_entity_CAMERA *)calloc(1, sizeof(struct _dwg_entity_CAMERA));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_CAMERA, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_CAMERA) {
  {
    struct _dwg_entity_CAMERA *arg1 = (struct _dwg_entity_CAMERA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_CAMERA(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_CAMERA, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_CAMERA" "', argument " "1"" of type '" "struct _dwg_entity_CAMERA *""'"); 
    }
    arg1 = (struct _dwg_entity_CAMERA *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_GEOPOSITIONMARKER_parent_set) {
  {
    struct _dwg_entity_GEOPOSITIONMARKER *arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_GEOPOSITIONMARKER_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_GEOPOSITIONMARKER *""'"); 
    }
    arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_GEOPOSITIONMARKER_parent_get) {
  {
    struct _dwg_entity_GEOPOSITIONMARKER *arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_GEOPOSITIONMARKER_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_GEOPOSITIONMARKER *""'"); 
    }
    arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_GEOPOSITIONMARKER_class_version_set) {
  {
    struct _dwg_entity_GEOPOSITIONMARKER *arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_GEOPOSITIONMARKER_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_class_version_set" "', argument " "1"" of type '" "struct _dwg_entity_GEOPOSITIONMARKER *""'"); 
    }
    arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_GEOPOSITIONMARKER_class_version_get) {
  {
    struct _dwg_entity_GEOPOSITIONMARKER *arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_GEOPOSITIONMARKER_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_class_version_get" "', argument " "1"" of type '" "struct _dwg_entity_GEOPOSITIONMARKER *""'"); 
    }
    arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_GEOPOSITIONMARKER_position_set) {
  {
    struct _dwg_entity_GEOPOSITIONMARKER *arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_GEOPOSITIONMARKER_position_set(self,position);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_position_set" "', argument " "1"" of type '" "struct _dwg_entity_GEOPOSITIONMARKER *""'"); 
    }
    arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_position_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->position = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_GEOPOSITIONMARKER_position_get) {
  {
    struct _dwg_entity_GEOPOSITIONMARKER *arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_GEOPOSITIONMARKER_position_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_position_get" "', argument " "1"" of type '" "struct _dwg_entity_GEOPOSITIONMARKER *""'"); 
    }
    arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->position);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_GEOPOSITIONMARKER_radius_set) {
  {
    struct _dwg_entity_GEOPOSITIONMARKER *arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_GEOPOSITIONMARKER_radius_set(self,radius);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_radius_set" "', argument " "1"" of type '" "struct _dwg_entity_GEOPOSITIONMARKER *""'"); 
    }
    arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_radius_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->radius = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_GEOPOSITIONMARKER_radius_get) {
  {
    struct _dwg_entity_GEOPOSITIONMARKER *arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_GEOPOSITIONMARKER_radius_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_radius_get" "', argument " "1"" of type '" "struct _dwg_entity_GEOPOSITIONMARKER *""'"); 
    }
    arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *)(argp1);
    result = (BITCODE_BD) ((arg1)->radius);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_GEOPOSITIONMARKER_landing_gap_set) {
  {
    struct _dwg_entity_GEOPOSITIONMARKER *arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_GEOPOSITIONMARKER_landing_gap_set(self,landing_gap);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_landing_gap_set" "', argument " "1"" of type '" "struct _dwg_entity_GEOPOSITIONMARKER *""'"); 
    }
    arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_landing_gap_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->landing_gap = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_GEOPOSITIONMARKER_landing_gap_get) {
  {
    struct _dwg_entity_GEOPOSITIONMARKER *arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_GEOPOSITIONMARKER_landing_gap_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_landing_gap_get" "', argument " "1"" of type '" "struct _dwg_entity_GEOPOSITIONMARKER *""'"); 
    }
    arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *)(argp1);
    result = (BITCODE_BD) ((arg1)->landing_gap);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_GEOPOSITIONMARKER_notes_set) {
  {
    struct _dwg_entity_GEOPOSITIONMARKER *arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_GEOPOSITIONMARKER_notes_set(self,notes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_notes_set" "', argument " "1"" of type '" "struct _dwg_entity_GEOPOSITIONMARKER *""'"); 
    }
    arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_notes_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->notes);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->notes = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->notes = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_GEOPOSITIONMARKER_notes_get) {
  {
    struct _dwg_entity_GEOPOSITIONMARKER *arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_GEOPOSITIONMARKER_notes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_notes_get" "', argument " "1"" of type '" "struct _dwg_entity_GEOPOSITIONMARKER *""'"); 
    }
    arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *)(argp1);
    result = (BITCODE_TV) ((arg1)->notes);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_GEOPOSITIONMARKER_text_alignment_set) {
  {
    struct _dwg_entity_GEOPOSITIONMARKER *arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_GEOPOSITIONMARKER_text_alignment_set(self,text_alignment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_text_alignment_set" "', argument " "1"" of type '" "struct _dwg_entity_GEOPOSITIONMARKER *""'"); 
    }
    arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_text_alignment_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->text_alignment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_GEOPOSITIONMARKER_text_alignment_get) {
  {
    struct _dwg_entity_GEOPOSITIONMARKER *arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_GEOPOSITIONMARKER_text_alignment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_text_alignment_get" "', argument " "1"" of type '" "struct _dwg_entity_GEOPOSITIONMARKER *""'"); 
    }
    arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *)(argp1);
    result = (BITCODE_RC) ((arg1)->text_alignment);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_GEOPOSITIONMARKER_mtext_visible_set) {
  {
    struct _dwg_entity_GEOPOSITIONMARKER *arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_GEOPOSITIONMARKER_mtext_visible_set(self,mtext_visible);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_mtext_visible_set" "', argument " "1"" of type '" "struct _dwg_entity_GEOPOSITIONMARKER *""'"); 
    }
    arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_mtext_visible_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->mtext_visible = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_GEOPOSITIONMARKER_mtext_visible_get) {
  {
    struct _dwg_entity_GEOPOSITIONMARKER *arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_GEOPOSITIONMARKER_mtext_visible_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_mtext_visible_get" "', argument " "1"" of type '" "struct _dwg_entity_GEOPOSITIONMARKER *""'"); 
    }
    arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *)(argp1);
    result = (BITCODE_B) ((arg1)->mtext_visible);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_GEOPOSITIONMARKER_enable_frame_text_set) {
  {
    struct _dwg_entity_GEOPOSITIONMARKER *arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_GEOPOSITIONMARKER_enable_frame_text_set(self,enable_frame_text);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_enable_frame_text_set" "', argument " "1"" of type '" "struct _dwg_entity_GEOPOSITIONMARKER *""'"); 
    }
    arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_enable_frame_text_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->enable_frame_text = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_GEOPOSITIONMARKER_enable_frame_text_get) {
  {
    struct _dwg_entity_GEOPOSITIONMARKER *arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_GEOPOSITIONMARKER_enable_frame_text_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_enable_frame_text_get" "', argument " "1"" of type '" "struct _dwg_entity_GEOPOSITIONMARKER *""'"); 
    }
    arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *)(argp1);
    result = (BITCODE_B) ((arg1)->enable_frame_text);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_GEOPOSITIONMARKER_ins_pt_set) {
  {
    struct _dwg_entity_GEOPOSITIONMARKER *arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_GEOPOSITIONMARKER_ins_pt_set(self,ins_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_ins_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_GEOPOSITIONMARKER *""'"); 
    }
    arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_ins_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ins_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_GEOPOSITIONMARKER_ins_pt_get) {
  {
    struct _dwg_entity_GEOPOSITIONMARKER *arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_GEOPOSITIONMARKER_ins_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_ins_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_GEOPOSITIONMARKER *""'"); 
    }
    arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ins_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_GEOPOSITIONMARKER_alignment_pt_set) {
  {
    struct _dwg_entity_GEOPOSITIONMARKER *arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_GEOPOSITIONMARKER_alignment_pt_set(self,alignment_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_alignment_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_GEOPOSITIONMARKER *""'"); 
    }
    arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_alignment_pt_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->alignment_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_GEOPOSITIONMARKER_alignment_pt_get) {
  {
    struct _dwg_entity_GEOPOSITIONMARKER *arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_GEOPOSITIONMARKER_alignment_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_alignment_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_GEOPOSITIONMARKER *""'"); 
    }
    arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->alignment_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_GEOPOSITIONMARKER_mtext_set) {
  {
    struct _dwg_entity_GEOPOSITIONMARKER *arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *) 0 ;
    Dwg_AcDbMTextObjectEmbedded *arg2 = (Dwg_AcDbMTextObjectEmbedded *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_GEOPOSITIONMARKER_mtext_set(self,mtext);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_mtext_set" "', argument " "1"" of type '" "struct _dwg_entity_GEOPOSITIONMARKER *""'"); 
    }
    arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_mtext_set" "', argument " "2"" of type '" "Dwg_AcDbMTextObjectEmbedded *""'"); 
    }
    arg2 = (Dwg_AcDbMTextObjectEmbedded *)(argp2);
    if (arg1) (arg1)->mtext = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_GEOPOSITIONMARKER_mtext_get) {
  {
    struct _dwg_entity_GEOPOSITIONMARKER *arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_AcDbMTextObjectEmbedded *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_GEOPOSITIONMARKER_mtext_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_mtext_get" "', argument " "1"" of type '" "struct _dwg_entity_GEOPOSITIONMARKER *""'"); 
    }
    arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *)(argp1);
    result = (Dwg_AcDbMTextObjectEmbedded *)& ((arg1)->mtext);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_GEOPOSITIONMARKER_is_really_locked_set) {
  {
    struct _dwg_entity_GEOPOSITIONMARKER *arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_GEOPOSITIONMARKER_is_really_locked_set(self,is_really_locked);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_is_really_locked_set" "', argument " "1"" of type '" "struct _dwg_entity_GEOPOSITIONMARKER *""'"); 
    }
    arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_is_really_locked_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_really_locked = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_GEOPOSITIONMARKER_is_really_locked_get) {
  {
    struct _dwg_entity_GEOPOSITIONMARKER *arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_GEOPOSITIONMARKER_is_really_locked_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_is_really_locked_get" "', argument " "1"" of type '" "struct _dwg_entity_GEOPOSITIONMARKER *""'"); 
    }
    arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *)(argp1);
    result = (BITCODE_B) ((arg1)->is_really_locked);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_GEOPOSITIONMARKER_annotative_data_size_set) {
  {
    struct _dwg_entity_GEOPOSITIONMARKER *arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_GEOPOSITIONMARKER_annotative_data_size_set(self,annotative_data_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_annotative_data_size_set" "', argument " "1"" of type '" "struct _dwg_entity_GEOPOSITIONMARKER *""'"); 
    }
    arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_annotative_data_size_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->annotative_data_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_GEOPOSITIONMARKER_annotative_data_size_get) {
  {
    struct _dwg_entity_GEOPOSITIONMARKER *arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_GEOPOSITIONMARKER_annotative_data_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_annotative_data_size_get" "', argument " "1"" of type '" "struct _dwg_entity_GEOPOSITIONMARKER *""'"); 
    }
    arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *)(argp1);
    result = (BITCODE_BS) ((arg1)->annotative_data_size);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_GEOPOSITIONMARKER_annotative_data_set) {
  {
    struct _dwg_entity_GEOPOSITIONMARKER *arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_GEOPOSITIONMARKER_annotative_data_set(self,annotative_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_annotative_data_set" "', argument " "1"" of type '" "struct _dwg_entity_GEOPOSITIONMARKER *""'"); 
    }
    arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_annotative_data_set" "', argument " "2"" of type '" "BITCODE_RC *""'"); 
    }
    arg2 = (BITCODE_RC *)(argp2);
    if (arg1) (arg1)->annotative_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_GEOPOSITIONMARKER_annotative_data_get) {
  {
    struct _dwg_entity_GEOPOSITIONMARKER *arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_GEOPOSITIONMARKER_annotative_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_annotative_data_get" "', argument " "1"" of type '" "struct _dwg_entity_GEOPOSITIONMARKER *""'"); 
    }
    arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *)(argp1);
    result = (BITCODE_RC *) ((arg1)->annotative_data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_GEOPOSITIONMARKER_annotative_flag_set) {
  {
    struct _dwg_entity_GEOPOSITIONMARKER *arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_GEOPOSITIONMARKER_annotative_flag_set(self,annotative_flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_annotative_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_GEOPOSITIONMARKER *""'"); 
    }
    arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_annotative_flag_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->annotative_flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_GEOPOSITIONMARKER_annotative_flag_get) {
  {
    struct _dwg_entity_GEOPOSITIONMARKER *arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_GEOPOSITIONMARKER_annotative_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_annotative_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_GEOPOSITIONMARKER *""'"); 
    }
    arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *)(argp1);
    result = (BITCODE_BS) ((arg1)->annotative_flag);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_GEOPOSITIONMARKER_annotative_style_set) {
  {
    struct _dwg_entity_GEOPOSITIONMARKER *arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_GEOPOSITIONMARKER_annotative_style_set(self,annotative_style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_annotative_style_set" "', argument " "1"" of type '" "struct _dwg_entity_GEOPOSITIONMARKER *""'"); 
    }
    arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_annotative_style_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->annotative_style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_GEOPOSITIONMARKER_annotative_style_get) {
  {
    struct _dwg_entity_GEOPOSITIONMARKER *arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_GEOPOSITIONMARKER_annotative_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_GEOPOSITIONMARKER_annotative_style_get" "', argument " "1"" of type '" "struct _dwg_entity_GEOPOSITIONMARKER *""'"); 
    }
    arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *)(argp1);
    result = (BITCODE_H) ((arg1)->annotative_style);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_GEOPOSITIONMARKER) {
  {
    int argvi = 0;
    struct _dwg_entity_GEOPOSITIONMARKER *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_GEOPOSITIONMARKER();");
    }
    result = (struct _dwg_entity_GEOPOSITIONMARKER *)calloc(1, sizeof(struct _dwg_entity_GEOPOSITIONMARKER));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_GEOPOSITIONMARKER) {
  {
    struct _dwg_entity_GEOPOSITIONMARKER *arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_GEOPOSITIONMARKER(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_GEOPOSITIONMARKER" "', argument " "1"" of type '" "struct _dwg_entity_GEOPOSITIONMARKER *""'"); 
    }
    arg1 = (struct _dwg_entity_GEOPOSITIONMARKER *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_parent_set) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_parent_set" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_GEOMAPIMAGE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_parent_get) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_parent_get" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_class_version_set) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEOMAPIMAGE_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_class_version_get) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_pt0_set) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_pt0_set(self,pt0);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_pt0_set" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_GEOMAPIMAGE_pt0_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->pt0 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_pt0_get) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_pt0_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_pt0_get" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->pt0);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_image_size_set) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_image_size_set(self,image_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_image_size_set" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_GEOMAPIMAGE_image_size_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->image_size = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_image_size_get) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_image_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_image_size_get" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->image_size);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_display_props_set) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_display_props_set(self,display_props);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_display_props_set" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEOMAPIMAGE_display_props_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->display_props = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_display_props_get) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_display_props_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_display_props_get" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    result = (BITCODE_BS) ((arg1)->display_props);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_clipping_set) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_clipping_set(self,clipping);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_clipping_set" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEOMAPIMAGE_clipping_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->clipping = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_clipping_get) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_clipping_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_clipping_get" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    result = (BITCODE_B) ((arg1)->clipping);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_brightness_set) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_brightness_set(self,brightness);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_brightness_set" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEOMAPIMAGE_brightness_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->brightness = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_brightness_get) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_brightness_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_brightness_get" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    result = (BITCODE_RC) ((arg1)->brightness);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_contrast_set) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_contrast_set(self,contrast);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_contrast_set" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEOMAPIMAGE_contrast_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->contrast = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_contrast_get) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_contrast_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_contrast_get" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    result = (BITCODE_RC) ((arg1)->contrast);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_fade_set) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_fade_set(self,fade);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_fade_set" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEOMAPIMAGE_fade_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->fade = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_fade_get) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_fade_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_fade_get" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    result = (BITCODE_RC) ((arg1)->fade);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_rotation_set) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_rotation_set(self,rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_rotation_set" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEOMAPIMAGE_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_rotation_get) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_rotation_get" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    result = (BITCODE_BD) ((arg1)->rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_image_width_set) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_image_width_set(self,image_width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_image_width_set" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEOMAPIMAGE_image_width_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->image_width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_image_width_get) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_image_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_image_width_get" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    result = (BITCODE_BD) ((arg1)->image_width);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_image_height_set) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_image_height_set(self,image_height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_image_height_set" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEOMAPIMAGE_image_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->image_height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_image_height_get) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_image_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_image_height_get" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    result = (BITCODE_BD) ((arg1)->image_height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_name_set) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_name_set" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_GEOMAPIMAGE_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_name_get) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_name_get" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_image_file_set) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_image_file_set(self,image_file);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_image_file_set" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEOMAPIMAGE_image_file_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->image_file = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_image_file_get) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_image_file_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_image_file_get" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    result = (BITCODE_BD) ((arg1)->image_file);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_image_visibility_set) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_image_visibility_set(self,image_visibility);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_image_visibility_set" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEOMAPIMAGE_image_visibility_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->image_visibility = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_image_visibility_get) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_image_visibility_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_image_visibility_get" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    result = (BITCODE_BD) ((arg1)->image_visibility);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_transparency_set) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_transparency_set(self,transparency);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_transparency_set" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEOMAPIMAGE_transparency_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->transparency = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_transparency_get) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_transparency_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_transparency_get" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    result = (BITCODE_BS) ((arg1)->transparency);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_height_set) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_height_set(self,height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_height_set" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEOMAPIMAGE_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_height_get) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_height_get" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    result = (BITCODE_BD) ((arg1)->height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_width_set) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_width_set(self,width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_width_set" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEOMAPIMAGE_width_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_width_get) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_width_get" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    result = (BITCODE_BD) ((arg1)->width);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_show_rotation_set) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_show_rotation_set(self,show_rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_show_rotation_set" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEOMAPIMAGE_show_rotation_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->show_rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_show_rotation_get) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_show_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_show_rotation_get" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    result = (BITCODE_B) ((arg1)->show_rotation);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_scale_factor_set) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_scale_factor_set(self,scale_factor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_scale_factor_set" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEOMAPIMAGE_scale_factor_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->scale_factor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_scale_factor_get) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_scale_factor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_scale_factor_get" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    result = (BITCODE_BD) ((arg1)->scale_factor);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_geoimage_brightness_set) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_geoimage_brightness_set(self,geoimage_brightness);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_geoimage_brightness_set" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEOMAPIMAGE_geoimage_brightness_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->geoimage_brightness = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_geoimage_brightness_get) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_geoimage_brightness_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_geoimage_brightness_get" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    result = (BITCODE_BS) ((arg1)->geoimage_brightness);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_geoimage_contrast_set) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_geoimage_contrast_set(self,geoimage_contrast);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_geoimage_contrast_set" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEOMAPIMAGE_geoimage_contrast_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->geoimage_contrast = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_geoimage_contrast_get) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_geoimage_contrast_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_geoimage_contrast_get" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    result = (BITCODE_BS) ((arg1)->geoimage_contrast);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_geoimage_fade_set) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_geoimage_fade_set(self,geoimage_fade);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_geoimage_fade_set" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEOMAPIMAGE_geoimage_fade_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->geoimage_fade = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_geoimage_fade_get) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_geoimage_fade_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_geoimage_fade_get" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    result = (BITCODE_BS) ((arg1)->geoimage_fade);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_geoimage_position_set) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_geoimage_position_set(self,geoimage_position);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_geoimage_position_set" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEOMAPIMAGE_geoimage_position_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->geoimage_position = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_geoimage_position_get) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_geoimage_position_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_geoimage_position_get" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    result = (BITCODE_BS) ((arg1)->geoimage_position);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_geoimage_width_set) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_geoimage_width_set(self,geoimage_width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_geoimage_width_set" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEOMAPIMAGE_geoimage_width_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->geoimage_width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_geoimage_width_get) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_geoimage_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_geoimage_width_get" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    result = (BITCODE_BS) ((arg1)->geoimage_width);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_geoimage_height_set) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_geoimage_height_set(self,geoimage_height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_geoimage_height_set" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GEOMAPIMAGE_geoimage_height_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->geoimage_height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GEOMAPIMAGE_geoimage_height_get) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GEOMAPIMAGE_geoimage_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GEOMAPIMAGE_geoimage_height_get" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    result = (BITCODE_BS) ((arg1)->geoimage_height);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_GEOMAPIMAGE) {
  {
    int argvi = 0;
    struct _dwg_object_GEOMAPIMAGE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_GEOMAPIMAGE();");
    }
    result = (struct _dwg_object_GEOMAPIMAGE *)calloc(1, sizeof(struct _dwg_object_GEOMAPIMAGE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_GEOMAPIMAGE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_GEOMAPIMAGE) {
  {
    struct _dwg_object_GEOMAPIMAGE *arg1 = (struct _dwg_object_GEOMAPIMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_GEOMAPIMAGE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_GEOMAPIMAGE" "', argument " "1"" of type '" "struct _dwg_object_GEOMAPIMAGE *""'"); 
    }
    arg1 = (struct _dwg_object_GEOMAPIMAGE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_parent_set) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_HELIX_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_parent_get) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_flag_set) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HELIX_flag_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_flag_get) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    result = (BITCODE_BS) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_scenario_set) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_scenario_set(self,scenario);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_scenario_set" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HELIX_scenario_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->scenario = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_scenario_get) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_scenario_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_scenario_get" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    result = (BITCODE_BS) ((arg1)->scenario);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_degree_set) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_degree_set(self,degree);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_degree_set" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HELIX_degree_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->degree = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_degree_get) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_degree_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_degree_get" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    result = (BITCODE_BS) ((arg1)->degree);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_splineflags_set) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_splineflags_set(self,splineflags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_splineflags_set" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HELIX_splineflags_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->splineflags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_splineflags_get) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_splineflags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_splineflags_get" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    result = (BITCODE_BL) ((arg1)->splineflags);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_knotparam_set) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_knotparam_set(self,knotparam);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_knotparam_set" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HELIX_knotparam_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->knotparam = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_knotparam_get) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_knotparam_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_knotparam_get" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    result = (BITCODE_BL) ((arg1)->knotparam);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_fit_tol_set) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_fit_tol_set(self,fit_tol);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_fit_tol_set" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HELIX_fit_tol_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->fit_tol = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_fit_tol_get) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_fit_tol_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_fit_tol_get" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    result = (BITCODE_BD) ((arg1)->fit_tol);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_beg_tan_vec_set) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_beg_tan_vec_set(self,beg_tan_vec);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_beg_tan_vec_set" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_HELIX_beg_tan_vec_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->beg_tan_vec = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_beg_tan_vec_get) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_beg_tan_vec_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_beg_tan_vec_get" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->beg_tan_vec);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_end_tan_vec_set) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_end_tan_vec_set(self,end_tan_vec);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_end_tan_vec_set" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_HELIX_end_tan_vec_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->end_tan_vec = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_end_tan_vec_get) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_end_tan_vec_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_end_tan_vec_get" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->end_tan_vec);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_rational_set) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_rational_set(self,rational);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_rational_set" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HELIX_rational_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->rational = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_rational_get) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_rational_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_rational_get" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    result = (BITCODE_B) ((arg1)->rational);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_closed_b_set) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_closed_b_set(self,closed_b);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_closed_b_set" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HELIX_closed_b_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->closed_b = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_closed_b_get) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_closed_b_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_closed_b_get" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    result = (BITCODE_B) ((arg1)->closed_b);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_periodic_set) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_periodic_set(self,periodic);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_periodic_set" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HELIX_periodic_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->periodic = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_periodic_get) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_periodic_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_periodic_get" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    result = (BITCODE_B) ((arg1)->periodic);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_weighted_set) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_weighted_set(self,weighted);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_weighted_set" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HELIX_weighted_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->weighted = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_weighted_get) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_weighted_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_weighted_get" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    result = (BITCODE_B) ((arg1)->weighted);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_knot_tol_set) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_knot_tol_set(self,knot_tol);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_knot_tol_set" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HELIX_knot_tol_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->knot_tol = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_knot_tol_get) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_knot_tol_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_knot_tol_get" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    result = (BITCODE_BD) ((arg1)->knot_tol);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_ctrl_tol_set) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_ctrl_tol_set(self,ctrl_tol);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_ctrl_tol_set" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HELIX_ctrl_tol_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->ctrl_tol = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_ctrl_tol_get) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_ctrl_tol_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_ctrl_tol_get" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    result = (BITCODE_BD) ((arg1)->ctrl_tol);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_num_fit_pts_set) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_num_fit_pts_set(self,num_fit_pts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_num_fit_pts_set" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HELIX_num_fit_pts_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_fit_pts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_num_fit_pts_get) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_num_fit_pts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_num_fit_pts_get" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_fit_pts);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_fit_pts_set) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    BITCODE_3DPOINT *arg2 = (BITCODE_3DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_fit_pts_set(self,fit_pts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_fit_pts_set" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_HELIX_fit_pts_set" "', argument " "2"" of type '" "BITCODE_3DPOINT *""'"); 
    }
    arg2 = (BITCODE_3DPOINT *)(argp2);
    if (arg1) (arg1)->fit_pts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_fit_pts_get) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_fit_pts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_fit_pts_get" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    result = (BITCODE_3DPOINT *) ((arg1)->fit_pts);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_num_knots_set) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_num_knots_set(self,num_knots);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_num_knots_set" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HELIX_num_knots_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_knots = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_num_knots_get) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_num_knots_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_num_knots_get" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_knots);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_knots_set) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_knots_set(self,knots);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_knots_set" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_HELIX_knots_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->knots = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_knots_get) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_knots_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_knots_get" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    result = (BITCODE_BD *) ((arg1)->knots);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_num_ctrl_pts_set) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_num_ctrl_pts_set(self,num_ctrl_pts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_num_ctrl_pts_set" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HELIX_num_ctrl_pts_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_ctrl_pts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_num_ctrl_pts_get) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_num_ctrl_pts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_num_ctrl_pts_get" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_ctrl_pts);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_ctrl_pts_set) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    Dwg_SPLINE_control_point *arg2 = (Dwg_SPLINE_control_point *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_ctrl_pts_set(self,ctrl_pts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_ctrl_pts_set" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_SPLINE_control_point, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_HELIX_ctrl_pts_set" "', argument " "2"" of type '" "Dwg_SPLINE_control_point *""'"); 
    }
    arg2 = (Dwg_SPLINE_control_point *)(argp2);
    if (arg1) (arg1)->ctrl_pts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_ctrl_pts_get) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_SPLINE_control_point *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_ctrl_pts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_ctrl_pts_get" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    result = (Dwg_SPLINE_control_point *) ((arg1)->ctrl_pts);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_SPLINE_control_point, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_major_version_set) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_major_version_set(self,major_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_major_version_set" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HELIX_major_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->major_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_major_version_get) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_major_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_major_version_get" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    result = (BITCODE_BL) ((arg1)->major_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_maint_version_set) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_maint_version_set(self,maint_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_maint_version_set" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HELIX_maint_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->maint_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_maint_version_get) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_maint_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_maint_version_get" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    result = (BITCODE_BL) ((arg1)->maint_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_axis_base_pt_set) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_axis_base_pt_set(self,axis_base_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_axis_base_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_HELIX_axis_base_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->axis_base_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_axis_base_pt_get) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_axis_base_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_axis_base_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->axis_base_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_start_pt_set) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_start_pt_set(self,start_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_start_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_HELIX_start_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->start_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_start_pt_get) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_start_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_start_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->start_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_axis_vector_set) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_axis_vector_set(self,axis_vector);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_axis_vector_set" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_HELIX_axis_vector_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->axis_vector = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_axis_vector_get) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_axis_vector_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_axis_vector_get" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->axis_vector);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_radius_set) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_radius_set(self,radius);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_radius_set" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HELIX_radius_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->radius = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_radius_get) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_radius_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_radius_get" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    result = (BITCODE_BD) ((arg1)->radius);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_turns_set) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_turns_set(self,turns);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_turns_set" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HELIX_turns_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->turns = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_turns_get) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_turns_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_turns_get" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    result = (BITCODE_BD) ((arg1)->turns);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_turn_height_set) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_turn_height_set(self,turn_height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_turn_height_set" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HELIX_turn_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->turn_height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_turn_height_get) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_turn_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_turn_height_get" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    result = (BITCODE_BD) ((arg1)->turn_height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_handedness_set) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_handedness_set(self,handedness);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_handedness_set" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HELIX_handedness_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->handedness = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_handedness_get) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_handedness_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_handedness_get" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    result = (BITCODE_B) ((arg1)->handedness);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_constraint_type_set) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_constraint_type_set(self,constraint_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_constraint_type_set" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_HELIX_constraint_type_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->constraint_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_HELIX_constraint_type_get) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_HELIX_constraint_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_HELIX_constraint_type_get" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    result = (BITCODE_RC) ((arg1)->constraint_type);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_HELIX) {
  {
    int argvi = 0;
    struct _dwg_entity_HELIX *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_HELIX();");
    }
    result = (struct _dwg_entity_HELIX *)calloc(1, sizeof(struct _dwg_entity_HELIX));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_HELIX, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_HELIX) {
  {
    struct _dwg_entity_HELIX *arg1 = (struct _dwg_entity_HELIX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_HELIX(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_HELIX, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_HELIX" "', argument " "1"" of type '" "struct _dwg_entity_HELIX *""'"); 
    }
    arg1 = (struct _dwg_entity_HELIX *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_parent_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_parent_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_acis_empty_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_acis_empty_set(self,acis_empty);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_acis_empty_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_acis_empty_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->acis_empty = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_acis_empty_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_acis_empty_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_acis_empty_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->acis_empty);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_unknown_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_unknown_set(self,unknown);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_unknown_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_unknown_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_unknown_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_unknown_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_unknown_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_version_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_version_set(self,version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_version_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_version_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_version_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_num_blocks_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_num_blocks_set(self,num_blocks);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_num_blocks_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_num_blocks_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_blocks = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_num_blocks_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_num_blocks_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_num_blocks_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_blocks);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_block_size_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_block_size_set(self,block_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_block_size_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_block_size_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->block_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_block_size_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_block_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_block_size_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_BL *) ((arg1)->block_size);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_encr_sat_data_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    char **arg2 = (char **) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_encr_sat_data_set(self,encr_sat_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_encr_sat_data_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_encr_sat_data_set" "', argument " "2"" of type '" "char **""'"); 
    }
    arg2 = (char **)(argp2);
    if (arg1) (arg1)->encr_sat_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_encr_sat_data_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    char **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_encr_sat_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_encr_sat_data_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (char **) ((arg1)->encr_sat_data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_sab_size_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_sab_size_set(self,sab_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_sab_size_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_sab_size_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->sab_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_sab_size_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_sab_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_sab_size_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->sab_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_acis_data_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_acis_data_set(self,acis_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_acis_data_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_acis_data_set" "', argument " "2"" of type '" "BITCODE_RC *""'"); 
    }
    arg2 = (BITCODE_RC *)(argp2);
    if (arg1) (arg1)->acis_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_acis_data_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_acis_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_acis_data_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_RC *) ((arg1)->acis_data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_wireframe_data_present_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_wireframe_data_present_set(self,wireframe_data_present);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_wireframe_data_present_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_wireframe_data_present_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->wireframe_data_present = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_wireframe_data_present_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_wireframe_data_present_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_wireframe_data_present_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->wireframe_data_present);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_point_present_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_point_present_set(self,point_present);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_point_present_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_point_present_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->point_present = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_point_present_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_point_present_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_point_present_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->point_present);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_point_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_point_set(self,point);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_point_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_point_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->point = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_point_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_point_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_point_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->point);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_isolines_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_isolines_set(self,isolines);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_isolines_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_isolines_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->isolines = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_isolines_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_isolines_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_isolines_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->isolines);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_isoline_present_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_isoline_present_set(self,isoline_present);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_isoline_present_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_isoline_present_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->isoline_present = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_isoline_present_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_isoline_present_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_isoline_present_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->isoline_present);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_num_wires_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_num_wires_set(self,num_wires);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_num_wires_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_num_wires_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_wires = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_num_wires_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_num_wires_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_num_wires_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_wires);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_wires_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    Dwg_3DSOLID_wire *arg2 = (Dwg_3DSOLID_wire *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_wires_set(self,wires);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_wires_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_3DSOLID_wire, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_wires_set" "', argument " "2"" of type '" "Dwg_3DSOLID_wire *""'"); 
    }
    arg2 = (Dwg_3DSOLID_wire *)(argp2);
    if (arg1) (arg1)->wires = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_wires_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_3DSOLID_wire *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_wires_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_wires_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (Dwg_3DSOLID_wire *) ((arg1)->wires);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_3DSOLID_wire, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_num_silhouettes_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_num_silhouettes_set(self,num_silhouettes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_num_silhouettes_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_num_silhouettes_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_silhouettes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_num_silhouettes_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_num_silhouettes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_num_silhouettes_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_silhouettes);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_silhouettes_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    Dwg_3DSOLID_silhouette *arg2 = (Dwg_3DSOLID_silhouette *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_silhouettes_set(self,silhouettes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_silhouettes_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_3DSOLID_silhouette, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_silhouettes_set" "', argument " "2"" of type '" "Dwg_3DSOLID_silhouette *""'"); 
    }
    arg2 = (Dwg_3DSOLID_silhouette *)(argp2);
    if (arg1) (arg1)->silhouettes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_silhouettes_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_3DSOLID_silhouette *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_silhouettes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_silhouettes_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (Dwg_3DSOLID_silhouette *) ((arg1)->silhouettes);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_3DSOLID_silhouette, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE__dxf_sab_converted_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE__dxf_sab_converted_set(self,_dxf_sab_converted);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE__dxf_sab_converted_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE__dxf_sab_converted_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->_dxf_sab_converted = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE__dxf_sab_converted_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE__dxf_sab_converted_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE__dxf_sab_converted_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->_dxf_sab_converted);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_acis_empty2_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_acis_empty2_set(self,acis_empty2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_acis_empty2_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_acis_empty2_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->acis_empty2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_acis_empty2_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_acis_empty2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_acis_empty2_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->acis_empty2);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_extra_acis_data_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    struct _dwg_entity_3DSOLID *arg2 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_extra_acis_data_set(self,extra_acis_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_extra_acis_data_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_3DSOLID, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_extra_acis_data_set" "', argument " "2"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg2 = (struct _dwg_entity_3DSOLID *)(argp2);
    if (arg1) (arg1)->extra_acis_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_extra_acis_data_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_entity_3DSOLID *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_extra_acis_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_extra_acis_data_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (struct _dwg_entity_3DSOLID *) ((arg1)->extra_acis_data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_3DSOLID, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_num_materials_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_num_materials_set(self,num_materials);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_num_materials_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_num_materials_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_materials = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_num_materials_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_num_materials_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_num_materials_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_materials);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_materials_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    Dwg_3DSOLID_material *arg2 = (Dwg_3DSOLID_material *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_materials_set(self,materials);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_materials_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_3DSOLID_material, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_materials_set" "', argument " "2"" of type '" "Dwg_3DSOLID_material *""'"); 
    }
    arg2 = (Dwg_3DSOLID_material *)(argp2);
    if (arg1) (arg1)->materials = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_materials_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_3DSOLID_material *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_materials_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_materials_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (Dwg_3DSOLID_material *) ((arg1)->materials);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_3DSOLID_material, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_revision_guid_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) (BITCODE_RC *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_revision_guid_set(self,revision_guid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_revision_guid_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_revision_guid_set" "', argument " "2"" of type '" "BITCODE_RC [39]""'"); 
    } 
    arg2 = (BITCODE_RC *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)39; ++ii) *(BITCODE_RC *)&arg1->revision_guid[ii] = *((BITCODE_RC *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""revision_guid""' of type '""BITCODE_RC [39]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_revision_guid_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_revision_guid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_revision_guid_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_RC *)(BITCODE_RC *) ((arg1)->revision_guid);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_revision_major_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_revision_major_set(self,revision_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_revision_major_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_revision_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->revision_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_revision_major_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_revision_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_revision_major_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->revision_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_revision_minor1_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_revision_minor1_set(self,revision_minor1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_revision_minor1_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_revision_minor1_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->revision_minor1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_revision_minor1_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_revision_minor1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_revision_minor1_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->revision_minor1);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_revision_minor2_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_revision_minor2_set(self,revision_minor2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_revision_minor2_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_revision_minor2_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->revision_minor2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_revision_minor2_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_revision_minor2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_revision_minor2_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->revision_minor2);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_revision_bytes_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) (BITCODE_RC *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_revision_bytes_set(self,revision_bytes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_revision_bytes_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_revision_bytes_set" "', argument " "2"" of type '" "BITCODE_RC [9]""'"); 
    } 
    arg2 = (BITCODE_RC *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)9; ++ii) *(BITCODE_RC *)&arg1->revision_bytes[ii] = *((BITCODE_RC *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""revision_bytes""' of type '""BITCODE_RC [9]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_revision_bytes_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_revision_bytes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_revision_bytes_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_RC *)(BITCODE_RC *) ((arg1)->revision_bytes);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_end_marker_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_end_marker_set(self,end_marker);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_end_marker_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_end_marker_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->end_marker = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_end_marker_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_end_marker_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_end_marker_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->end_marker);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_history_id_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_history_id_set(self,history_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_history_id_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_history_id_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->history_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_history_id_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_history_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_history_id_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_H) ((arg1)->history_id);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_has_revision_guid_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_has_revision_guid_set(self,has_revision_guid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_has_revision_guid_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_has_revision_guid_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_revision_guid = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_has_revision_guid_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_has_revision_guid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_has_revision_guid_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->has_revision_guid);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_acis_empty_bit_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_acis_empty_bit_set(self,acis_empty_bit);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_acis_empty_bit_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_acis_empty_bit_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->acis_empty_bit = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_acis_empty_bit_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_acis_empty_bit_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_acis_empty_bit_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->acis_empty_bit);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_modeler_format_version_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_modeler_format_version_set(self,modeler_format_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_modeler_format_version_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_modeler_format_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->modeler_format_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_modeler_format_version_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_modeler_format_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_modeler_format_version_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->modeler_format_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_bindata_size_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_bindata_size_set(self,bindata_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_bindata_size_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_bindata_size_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bindata_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_bindata_size_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_bindata_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_bindata_size_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->bindata_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_bindata_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_TF arg2 = (BITCODE_TF) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_bindata_set(self,bindata);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_bindata_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_bindata_set" "', argument " "2"" of type '" "BITCODE_TF""'"); 
    }
    arg2 = (BITCODE_TF)(argp2);
    if (arg1) (arg1)->bindata = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_bindata_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TF result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_bindata_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_bindata_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_TF) ((arg1)->bindata);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_u_isolines_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_u_isolines_set(self,u_isolines);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_u_isolines_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_u_isolines_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->u_isolines = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_u_isolines_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_u_isolines_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_u_isolines_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->u_isolines);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_v_isolines_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_v_isolines_set(self,v_isolines);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_v_isolines_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_v_isolines_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->v_isolines = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_v_isolines_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_v_isolines_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_v_isolines_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->v_isolines);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_class_version_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_class_version_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_class_version_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_class_version_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_draft_angle_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_draft_angle_set(self,draft_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_draft_angle_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_draft_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->draft_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_draft_angle_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_draft_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_draft_angle_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_BD) ((arg1)->draft_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_draft_start_distance_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_draft_start_distance_set(self,draft_start_distance);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_draft_start_distance_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_draft_start_distance_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->draft_start_distance = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_draft_start_distance_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_draft_start_distance_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_draft_start_distance_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_BD) ((arg1)->draft_start_distance);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_draft_end_distance_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_draft_end_distance_set(self,draft_end_distance);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_draft_end_distance_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_draft_end_distance_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->draft_end_distance = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_draft_end_distance_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_draft_end_distance_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_draft_end_distance_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_BD) ((arg1)->draft_end_distance);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_twist_angle_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_twist_angle_set(self,twist_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_twist_angle_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_twist_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->twist_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_twist_angle_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_twist_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_twist_angle_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_BD) ((arg1)->twist_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_scale_factor_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_scale_factor_set(self,scale_factor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_scale_factor_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_scale_factor_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->scale_factor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_scale_factor_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_scale_factor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_scale_factor_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_BD) ((arg1)->scale_factor);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_align_angle_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_align_angle_set(self,align_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_align_angle_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_align_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->align_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_align_angle_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_align_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_align_angle_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_BD) ((arg1)->align_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_transmatrix_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_transmatrix_set(self,sweep_entity_transmatrix);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_transmatrix_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_transmatrix_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->sweep_entity_transmatrix = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_transmatrix_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_transmatrix_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_transmatrix_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_BD *) ((arg1)->sweep_entity_transmatrix);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_path_entity_transmatrix_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_path_entity_transmatrix_set(self,path_entity_transmatrix);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_path_entity_transmatrix_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_path_entity_transmatrix_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->path_entity_transmatrix = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_path_entity_transmatrix_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_path_entity_transmatrix_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_path_entity_transmatrix_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_BD *) ((arg1)->path_entity_transmatrix);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_is_solid_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_is_solid_set(self,is_solid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_is_solid_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_is_solid_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_solid = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_is_solid_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_is_solid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_is_solid_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->is_solid);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_sweep_alignment_flags_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_sweep_alignment_flags_set(self,sweep_alignment_flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_sweep_alignment_flags_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_sweep_alignment_flags_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->sweep_alignment_flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_sweep_alignment_flags_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_sweep_alignment_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_sweep_alignment_flags_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->sweep_alignment_flags);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_path_flags_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_path_flags_set(self,path_flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_path_flags_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_path_flags_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->path_flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_path_flags_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_path_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_path_flags_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->path_flags);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_align_start_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_align_start_set(self,align_start);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_align_start_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_align_start_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->align_start = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_align_start_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_align_start_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_align_start_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->align_start);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_bank_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_bank_set(self,bank);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_bank_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_bank_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->bank = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_bank_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_bank_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_bank_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->bank);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_base_point_set_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_base_point_set_set(self,base_point_set);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_base_point_set_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_base_point_set_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->base_point_set = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_base_point_set_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_base_point_set_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_base_point_set_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->base_point_set);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_transform_computed_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_transform_computed_set(self,sweep_entity_transform_computed);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_transform_computed_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_transform_computed_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->sweep_entity_transform_computed = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_transform_computed_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_transform_computed_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_transform_computed_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->sweep_entity_transform_computed);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_path_entity_transform_computed_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_path_entity_transform_computed_set(self,path_entity_transform_computed);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_path_entity_transform_computed_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_path_entity_transform_computed_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->path_entity_transform_computed = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_path_entity_transform_computed_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_path_entity_transform_computed_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_path_entity_transform_computed_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->path_entity_transform_computed);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_reference_vector_for_controlling_twist_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_reference_vector_for_controlling_twist_set(self,reference_vector_for_controlling_twist);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_reference_vector_for_controlling_twist_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_reference_vector_for_controlling_twist_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->reference_vector_for_controlling_twist = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_reference_vector_for_controlling_twist_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_reference_vector_for_controlling_twist_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_reference_vector_for_controlling_twist_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->reference_vector_for_controlling_twist);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_set(self,sweep_entity);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->sweep_entity = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_H) ((arg1)->sweep_entity);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_path_entity_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_path_entity_set(self,path_entity);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_path_entity_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_path_entity_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->path_entity = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_path_entity_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_path_entity_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_path_entity_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_H) ((arg1)->path_entity);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_sweep_vector_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_sweep_vector_set(self,sweep_vector);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_sweep_vector_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_sweep_vector_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->sweep_vector = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_sweep_vector_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_sweep_vector_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_sweep_vector_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->sweep_vector);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_sweep_transmatrix_set) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_sweep_transmatrix_set(self,sweep_transmatrix);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_sweep_transmatrix_set" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_sweep_transmatrix_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->sweep_transmatrix = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_EXTRUDEDSURFACE_sweep_transmatrix_get) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_EXTRUDEDSURFACE_sweep_transmatrix_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_EXTRUDEDSURFACE_sweep_transmatrix_get" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    result = (BITCODE_BD *) ((arg1)->sweep_transmatrix);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_EXTRUDEDSURFACE) {
  {
    int argvi = 0;
    struct _dwg_entity_EXTRUDEDSURFACE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_EXTRUDEDSURFACE();");
    }
    result = (struct _dwg_entity_EXTRUDEDSURFACE *)calloc(1, sizeof(struct _dwg_entity_EXTRUDEDSURFACE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_EXTRUDEDSURFACE) {
  {
    struct _dwg_entity_EXTRUDEDSURFACE *arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_EXTRUDEDSURFACE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_EXTRUDEDSURFACE" "', argument " "1"" of type '" "struct _dwg_entity_EXTRUDEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_EXTRUDEDSURFACE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_parent_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SWEPTSURFACE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_parent_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_acis_empty_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_acis_empty_set(self,acis_empty);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_acis_empty_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_acis_empty_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->acis_empty = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_acis_empty_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_acis_empty_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_acis_empty_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->acis_empty);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_unknown_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_unknown_set(self,unknown);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_unknown_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_unknown_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_unknown_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_unknown_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_unknown_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_version_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_version_set(self,version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_version_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_version_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_version_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_num_blocks_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_num_blocks_set(self,num_blocks);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_num_blocks_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_num_blocks_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_blocks = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_num_blocks_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_num_blocks_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_num_blocks_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_blocks);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_block_size_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_block_size_set(self,block_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_block_size_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SWEPTSURFACE_block_size_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->block_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_block_size_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_block_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_block_size_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_BL *) ((arg1)->block_size);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_encr_sat_data_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    char **arg2 = (char **) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_encr_sat_data_set(self,encr_sat_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_encr_sat_data_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SWEPTSURFACE_encr_sat_data_set" "', argument " "2"" of type '" "char **""'"); 
    }
    arg2 = (char **)(argp2);
    if (arg1) (arg1)->encr_sat_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_encr_sat_data_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    char **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_encr_sat_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_encr_sat_data_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (char **) ((arg1)->encr_sat_data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_sab_size_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_sab_size_set(self,sab_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_sab_size_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_sab_size_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->sab_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_sab_size_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_sab_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_sab_size_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->sab_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_acis_data_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_acis_data_set(self,acis_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_acis_data_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SWEPTSURFACE_acis_data_set" "', argument " "2"" of type '" "BITCODE_RC *""'"); 
    }
    arg2 = (BITCODE_RC *)(argp2);
    if (arg1) (arg1)->acis_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_acis_data_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_acis_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_acis_data_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_RC *) ((arg1)->acis_data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_wireframe_data_present_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_wireframe_data_present_set(self,wireframe_data_present);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_wireframe_data_present_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_wireframe_data_present_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->wireframe_data_present = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_wireframe_data_present_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_wireframe_data_present_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_wireframe_data_present_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->wireframe_data_present);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_point_present_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_point_present_set(self,point_present);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_point_present_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_point_present_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->point_present = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_point_present_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_point_present_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_point_present_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->point_present);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_point_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_point_set(self,point);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_point_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SWEPTSURFACE_point_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->point = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_point_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_point_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_point_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->point);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_isolines_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_isolines_set(self,isolines);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_isolines_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_isolines_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->isolines = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_isolines_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_isolines_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_isolines_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->isolines);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_isoline_present_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_isoline_present_set(self,isoline_present);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_isoline_present_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_isoline_present_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->isoline_present = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_isoline_present_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_isoline_present_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_isoline_present_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->isoline_present);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_num_wires_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_num_wires_set(self,num_wires);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_num_wires_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_num_wires_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_wires = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_num_wires_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_num_wires_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_num_wires_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_wires);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_wires_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    Dwg_3DSOLID_wire *arg2 = (Dwg_3DSOLID_wire *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_wires_set(self,wires);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_wires_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_3DSOLID_wire, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SWEPTSURFACE_wires_set" "', argument " "2"" of type '" "Dwg_3DSOLID_wire *""'"); 
    }
    arg2 = (Dwg_3DSOLID_wire *)(argp2);
    if (arg1) (arg1)->wires = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_wires_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_3DSOLID_wire *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_wires_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_wires_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (Dwg_3DSOLID_wire *) ((arg1)->wires);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_3DSOLID_wire, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_num_silhouettes_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_num_silhouettes_set(self,num_silhouettes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_num_silhouettes_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_num_silhouettes_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_silhouettes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_num_silhouettes_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_num_silhouettes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_num_silhouettes_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_silhouettes);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_silhouettes_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    Dwg_3DSOLID_silhouette *arg2 = (Dwg_3DSOLID_silhouette *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_silhouettes_set(self,silhouettes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_silhouettes_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_3DSOLID_silhouette, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SWEPTSURFACE_silhouettes_set" "', argument " "2"" of type '" "Dwg_3DSOLID_silhouette *""'"); 
    }
    arg2 = (Dwg_3DSOLID_silhouette *)(argp2);
    if (arg1) (arg1)->silhouettes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_silhouettes_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_3DSOLID_silhouette *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_silhouettes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_silhouettes_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (Dwg_3DSOLID_silhouette *) ((arg1)->silhouettes);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_3DSOLID_silhouette, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE__dxf_sab_converted_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE__dxf_sab_converted_set(self,_dxf_sab_converted);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE__dxf_sab_converted_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE__dxf_sab_converted_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->_dxf_sab_converted = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE__dxf_sab_converted_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE__dxf_sab_converted_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE__dxf_sab_converted_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->_dxf_sab_converted);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_acis_empty2_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_acis_empty2_set(self,acis_empty2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_acis_empty2_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_acis_empty2_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->acis_empty2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_acis_empty2_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_acis_empty2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_acis_empty2_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->acis_empty2);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_extra_acis_data_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    struct _dwg_entity_3DSOLID *arg2 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_extra_acis_data_set(self,extra_acis_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_extra_acis_data_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_3DSOLID, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SWEPTSURFACE_extra_acis_data_set" "', argument " "2"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg2 = (struct _dwg_entity_3DSOLID *)(argp2);
    if (arg1) (arg1)->extra_acis_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_extra_acis_data_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_entity_3DSOLID *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_extra_acis_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_extra_acis_data_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (struct _dwg_entity_3DSOLID *) ((arg1)->extra_acis_data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_3DSOLID, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_num_materials_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_num_materials_set(self,num_materials);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_num_materials_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_num_materials_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_materials = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_num_materials_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_num_materials_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_num_materials_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_materials);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_materials_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    Dwg_3DSOLID_material *arg2 = (Dwg_3DSOLID_material *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_materials_set(self,materials);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_materials_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_3DSOLID_material, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SWEPTSURFACE_materials_set" "', argument " "2"" of type '" "Dwg_3DSOLID_material *""'"); 
    }
    arg2 = (Dwg_3DSOLID_material *)(argp2);
    if (arg1) (arg1)->materials = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_materials_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_3DSOLID_material *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_materials_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_materials_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (Dwg_3DSOLID_material *) ((arg1)->materials);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_3DSOLID_material, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_revision_guid_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) (BITCODE_RC *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_revision_guid_set(self,revision_guid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_revision_guid_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SWEPTSURFACE_revision_guid_set" "', argument " "2"" of type '" "BITCODE_RC [39]""'"); 
    } 
    arg2 = (BITCODE_RC *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)39; ++ii) *(BITCODE_RC *)&arg1->revision_guid[ii] = *((BITCODE_RC *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""revision_guid""' of type '""BITCODE_RC [39]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_revision_guid_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_revision_guid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_revision_guid_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_RC *)(BITCODE_RC *) ((arg1)->revision_guid);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_revision_major_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_revision_major_set(self,revision_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_revision_major_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_revision_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->revision_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_revision_major_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_revision_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_revision_major_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->revision_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_revision_minor1_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_revision_minor1_set(self,revision_minor1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_revision_minor1_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_revision_minor1_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->revision_minor1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_revision_minor1_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_revision_minor1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_revision_minor1_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->revision_minor1);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_revision_minor2_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_revision_minor2_set(self,revision_minor2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_revision_minor2_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_revision_minor2_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->revision_minor2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_revision_minor2_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_revision_minor2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_revision_minor2_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->revision_minor2);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_revision_bytes_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) (BITCODE_RC *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_revision_bytes_set(self,revision_bytes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_revision_bytes_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SWEPTSURFACE_revision_bytes_set" "', argument " "2"" of type '" "BITCODE_RC [9]""'"); 
    } 
    arg2 = (BITCODE_RC *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)9; ++ii) *(BITCODE_RC *)&arg1->revision_bytes[ii] = *((BITCODE_RC *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""revision_bytes""' of type '""BITCODE_RC [9]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_revision_bytes_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_revision_bytes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_revision_bytes_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_RC *)(BITCODE_RC *) ((arg1)->revision_bytes);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_end_marker_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_end_marker_set(self,end_marker);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_end_marker_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_end_marker_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->end_marker = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_end_marker_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_end_marker_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_end_marker_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->end_marker);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_history_id_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_history_id_set(self,history_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_history_id_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SWEPTSURFACE_history_id_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->history_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_history_id_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_history_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_history_id_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_H) ((arg1)->history_id);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_has_revision_guid_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_has_revision_guid_set(self,has_revision_guid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_has_revision_guid_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_has_revision_guid_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_revision_guid = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_has_revision_guid_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_has_revision_guid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_has_revision_guid_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->has_revision_guid);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_acis_empty_bit_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_acis_empty_bit_set(self,acis_empty_bit);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_acis_empty_bit_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_acis_empty_bit_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->acis_empty_bit = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_acis_empty_bit_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_acis_empty_bit_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_acis_empty_bit_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->acis_empty_bit);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_modeler_format_version_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_modeler_format_version_set(self,modeler_format_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_modeler_format_version_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_modeler_format_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->modeler_format_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_modeler_format_version_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_modeler_format_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_modeler_format_version_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->modeler_format_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_u_isolines_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_u_isolines_set(self,u_isolines);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_u_isolines_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_u_isolines_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->u_isolines = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_u_isolines_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_u_isolines_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_u_isolines_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->u_isolines);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_v_isolines_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_v_isolines_set(self,v_isolines);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_v_isolines_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_v_isolines_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->v_isolines = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_v_isolines_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_v_isolines_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_v_isolines_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->v_isolines);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_class_version_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_class_version_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_class_version_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_class_version_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_sweep_entity_id_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_sweep_entity_id_set(self,sweep_entity_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_sweep_entity_id_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_sweep_entity_id_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->sweep_entity_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_sweep_entity_id_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_sweep_entity_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_sweep_entity_id_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->sweep_entity_id);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_sweepdata_size_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_sweepdata_size_set(self,sweepdata_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_sweepdata_size_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_sweepdata_size_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->sweepdata_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_sweepdata_size_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_sweepdata_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_sweepdata_size_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->sweepdata_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_sweepdata_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_TF arg2 = (BITCODE_TF) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_sweepdata_set(self,sweepdata);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_sweepdata_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SWEPTSURFACE_sweepdata_set" "', argument " "2"" of type '" "BITCODE_TF""'"); 
    }
    arg2 = (BITCODE_TF)(argp2);
    if (arg1) (arg1)->sweepdata = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_sweepdata_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TF result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_sweepdata_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_sweepdata_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_TF) ((arg1)->sweepdata);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_path_entity_id_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_path_entity_id_set(self,path_entity_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_path_entity_id_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_path_entity_id_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->path_entity_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_path_entity_id_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_path_entity_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_path_entity_id_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->path_entity_id);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_pathdata_size_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_pathdata_size_set(self,pathdata_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_pathdata_size_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_pathdata_size_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->pathdata_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_pathdata_size_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_pathdata_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_pathdata_size_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->pathdata_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_pathdata_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_TF arg2 = (BITCODE_TF) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_pathdata_set(self,pathdata);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_pathdata_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SWEPTSURFACE_pathdata_set" "', argument " "2"" of type '" "BITCODE_TF""'"); 
    }
    arg2 = (BITCODE_TF)(argp2);
    if (arg1) (arg1)->pathdata = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_pathdata_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TF result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_pathdata_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_pathdata_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_TF) ((arg1)->pathdata);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_draft_angle_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_draft_angle_set(self,draft_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_draft_angle_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_draft_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->draft_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_draft_angle_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_draft_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_draft_angle_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_BD) ((arg1)->draft_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_draft_start_distance_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_draft_start_distance_set(self,draft_start_distance);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_draft_start_distance_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_draft_start_distance_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->draft_start_distance = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_draft_start_distance_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_draft_start_distance_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_draft_start_distance_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_BD) ((arg1)->draft_start_distance);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_draft_end_distance_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_draft_end_distance_set(self,draft_end_distance);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_draft_end_distance_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_draft_end_distance_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->draft_end_distance = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_draft_end_distance_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_draft_end_distance_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_draft_end_distance_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_BD) ((arg1)->draft_end_distance);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_twist_angle_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_twist_angle_set(self,twist_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_twist_angle_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_twist_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->twist_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_twist_angle_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_twist_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_twist_angle_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_BD) ((arg1)->twist_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_scale_factor_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_scale_factor_set(self,scale_factor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_scale_factor_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_scale_factor_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->scale_factor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_scale_factor_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_scale_factor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_scale_factor_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_BD) ((arg1)->scale_factor);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_align_angle_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_align_angle_set(self,align_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_align_angle_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_align_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->align_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_align_angle_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_align_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_align_angle_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_BD) ((arg1)->align_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_sweep_entity_transmatrix_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_sweep_entity_transmatrix_set(self,sweep_entity_transmatrix);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_sweep_entity_transmatrix_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SWEPTSURFACE_sweep_entity_transmatrix_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->sweep_entity_transmatrix = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_sweep_entity_transmatrix_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_sweep_entity_transmatrix_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_sweep_entity_transmatrix_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_BD *) ((arg1)->sweep_entity_transmatrix);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_path_entity_transmatrix_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_path_entity_transmatrix_set(self,path_entity_transmatrix);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_path_entity_transmatrix_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SWEPTSURFACE_path_entity_transmatrix_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->path_entity_transmatrix = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_path_entity_transmatrix_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_path_entity_transmatrix_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_path_entity_transmatrix_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_BD *) ((arg1)->path_entity_transmatrix);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_is_solid_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_is_solid_set(self,is_solid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_is_solid_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_is_solid_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_solid = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_is_solid_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_is_solid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_is_solid_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->is_solid);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_sweep_alignment_flags_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_sweep_alignment_flags_set(self,sweep_alignment_flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_sweep_alignment_flags_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_sweep_alignment_flags_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->sweep_alignment_flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_sweep_alignment_flags_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_sweep_alignment_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_sweep_alignment_flags_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->sweep_alignment_flags);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_path_flags_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_path_flags_set(self,path_flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_path_flags_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_path_flags_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->path_flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_path_flags_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_path_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_path_flags_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->path_flags);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_align_start_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_align_start_set(self,align_start);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_align_start_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_align_start_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->align_start = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_align_start_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_align_start_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_align_start_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->align_start);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_bank_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_bank_set(self,bank);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_bank_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_bank_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->bank = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_bank_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_bank_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_bank_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->bank);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_base_point_set_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_base_point_set_set(self,base_point_set);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_base_point_set_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_base_point_set_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->base_point_set = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_base_point_set_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_base_point_set_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_base_point_set_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->base_point_set);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_sweep_entity_transform_computed_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_sweep_entity_transform_computed_set(self,sweep_entity_transform_computed);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_sweep_entity_transform_computed_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_sweep_entity_transform_computed_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->sweep_entity_transform_computed = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_sweep_entity_transform_computed_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_sweep_entity_transform_computed_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_sweep_entity_transform_computed_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->sweep_entity_transform_computed);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_path_entity_transform_computed_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_path_entity_transform_computed_set(self,path_entity_transform_computed);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_path_entity_transform_computed_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_SWEPTSURFACE_path_entity_transform_computed_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->path_entity_transform_computed = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_path_entity_transform_computed_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_path_entity_transform_computed_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_path_entity_transform_computed_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->path_entity_transform_computed);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_reference_vector_for_controlling_twist_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_reference_vector_for_controlling_twist_set(self,reference_vector_for_controlling_twist);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_reference_vector_for_controlling_twist_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SWEPTSURFACE_reference_vector_for_controlling_twist_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->reference_vector_for_controlling_twist = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_reference_vector_for_controlling_twist_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_reference_vector_for_controlling_twist_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_reference_vector_for_controlling_twist_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->reference_vector_for_controlling_twist);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_sweep_entity_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_sweep_entity_set(self,sweep_entity);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_sweep_entity_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SWEPTSURFACE_sweep_entity_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->sweep_entity = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_sweep_entity_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_sweep_entity_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_sweep_entity_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_H) ((arg1)->sweep_entity);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_path_entity_set) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_path_entity_set(self,path_entity);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_path_entity_set" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_SWEPTSURFACE_path_entity_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->path_entity = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_SWEPTSURFACE_path_entity_get) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_SWEPTSURFACE_path_entity_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_SWEPTSURFACE_path_entity_get" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    result = (BITCODE_H) ((arg1)->path_entity);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_SWEPTSURFACE) {
  {
    int argvi = 0;
    struct _dwg_entity_SWEPTSURFACE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_SWEPTSURFACE();");
    }
    result = (struct _dwg_entity_SWEPTSURFACE *)calloc(1, sizeof(struct _dwg_entity_SWEPTSURFACE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_SWEPTSURFACE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_SWEPTSURFACE) {
  {
    struct _dwg_entity_SWEPTSURFACE *arg1 = (struct _dwg_entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_SWEPTSURFACE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_SWEPTSURFACE" "', argument " "1"" of type '" "struct _dwg_entity_SWEPTSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_SWEPTSURFACE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_parent_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LOFTEDSURFACE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_parent_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_acis_empty_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_acis_empty_set(self,acis_empty);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_acis_empty_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_acis_empty_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->acis_empty = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_acis_empty_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_acis_empty_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_acis_empty_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->acis_empty);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_unknown_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_unknown_set(self,unknown);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_unknown_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_unknown_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_unknown_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_unknown_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_unknown_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_version_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_version_set(self,version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_version_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_version_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_version_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_num_blocks_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_num_blocks_set(self,num_blocks);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_num_blocks_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_num_blocks_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_blocks = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_num_blocks_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_num_blocks_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_num_blocks_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_blocks);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_block_size_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_block_size_set(self,block_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_block_size_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LOFTEDSURFACE_block_size_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->block_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_block_size_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_block_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_block_size_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_BL *) ((arg1)->block_size);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_encr_sat_data_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    char **arg2 = (char **) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_encr_sat_data_set(self,encr_sat_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_encr_sat_data_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LOFTEDSURFACE_encr_sat_data_set" "', argument " "2"" of type '" "char **""'"); 
    }
    arg2 = (char **)(argp2);
    if (arg1) (arg1)->encr_sat_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_encr_sat_data_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    char **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_encr_sat_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_encr_sat_data_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (char **) ((arg1)->encr_sat_data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_sab_size_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_sab_size_set(self,sab_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_sab_size_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_sab_size_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->sab_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_sab_size_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_sab_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_sab_size_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->sab_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_acis_data_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_acis_data_set(self,acis_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_acis_data_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LOFTEDSURFACE_acis_data_set" "', argument " "2"" of type '" "BITCODE_RC *""'"); 
    }
    arg2 = (BITCODE_RC *)(argp2);
    if (arg1) (arg1)->acis_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_acis_data_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_acis_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_acis_data_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_RC *) ((arg1)->acis_data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_wireframe_data_present_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_wireframe_data_present_set(self,wireframe_data_present);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_wireframe_data_present_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_wireframe_data_present_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->wireframe_data_present = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_wireframe_data_present_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_wireframe_data_present_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_wireframe_data_present_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->wireframe_data_present);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_point_present_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_point_present_set(self,point_present);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_point_present_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_point_present_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->point_present = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_point_present_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_point_present_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_point_present_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->point_present);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_point_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_point_set(self,point);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_point_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LOFTEDSURFACE_point_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->point = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_point_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_point_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_point_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->point);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_isolines_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_isolines_set(self,isolines);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_isolines_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_isolines_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->isolines = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_isolines_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_isolines_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_isolines_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->isolines);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_isoline_present_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_isoline_present_set(self,isoline_present);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_isoline_present_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_isoline_present_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->isoline_present = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_isoline_present_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_isoline_present_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_isoline_present_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->isoline_present);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_num_wires_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_num_wires_set(self,num_wires);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_num_wires_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_num_wires_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_wires = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_num_wires_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_num_wires_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_num_wires_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_wires);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_wires_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    Dwg_3DSOLID_wire *arg2 = (Dwg_3DSOLID_wire *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_wires_set(self,wires);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_wires_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_3DSOLID_wire, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LOFTEDSURFACE_wires_set" "', argument " "2"" of type '" "Dwg_3DSOLID_wire *""'"); 
    }
    arg2 = (Dwg_3DSOLID_wire *)(argp2);
    if (arg1) (arg1)->wires = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_wires_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_3DSOLID_wire *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_wires_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_wires_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (Dwg_3DSOLID_wire *) ((arg1)->wires);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_3DSOLID_wire, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_num_silhouettes_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_num_silhouettes_set(self,num_silhouettes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_num_silhouettes_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_num_silhouettes_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_silhouettes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_num_silhouettes_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_num_silhouettes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_num_silhouettes_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_silhouettes);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_silhouettes_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    Dwg_3DSOLID_silhouette *arg2 = (Dwg_3DSOLID_silhouette *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_silhouettes_set(self,silhouettes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_silhouettes_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_3DSOLID_silhouette, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LOFTEDSURFACE_silhouettes_set" "', argument " "2"" of type '" "Dwg_3DSOLID_silhouette *""'"); 
    }
    arg2 = (Dwg_3DSOLID_silhouette *)(argp2);
    if (arg1) (arg1)->silhouettes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_silhouettes_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_3DSOLID_silhouette *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_silhouettes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_silhouettes_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (Dwg_3DSOLID_silhouette *) ((arg1)->silhouettes);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_3DSOLID_silhouette, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE__dxf_sab_converted_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE__dxf_sab_converted_set(self,_dxf_sab_converted);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE__dxf_sab_converted_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE__dxf_sab_converted_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->_dxf_sab_converted = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE__dxf_sab_converted_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE__dxf_sab_converted_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE__dxf_sab_converted_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->_dxf_sab_converted);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_acis_empty2_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_acis_empty2_set(self,acis_empty2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_acis_empty2_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_acis_empty2_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->acis_empty2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_acis_empty2_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_acis_empty2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_acis_empty2_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->acis_empty2);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_extra_acis_data_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    struct _dwg_entity_3DSOLID *arg2 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_extra_acis_data_set(self,extra_acis_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_extra_acis_data_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_3DSOLID, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LOFTEDSURFACE_extra_acis_data_set" "', argument " "2"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg2 = (struct _dwg_entity_3DSOLID *)(argp2);
    if (arg1) (arg1)->extra_acis_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_extra_acis_data_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_entity_3DSOLID *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_extra_acis_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_extra_acis_data_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (struct _dwg_entity_3DSOLID *) ((arg1)->extra_acis_data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_3DSOLID, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_num_materials_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_num_materials_set(self,num_materials);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_num_materials_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_num_materials_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_materials = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_num_materials_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_num_materials_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_num_materials_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_materials);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_materials_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    Dwg_3DSOLID_material *arg2 = (Dwg_3DSOLID_material *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_materials_set(self,materials);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_materials_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_3DSOLID_material, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LOFTEDSURFACE_materials_set" "', argument " "2"" of type '" "Dwg_3DSOLID_material *""'"); 
    }
    arg2 = (Dwg_3DSOLID_material *)(argp2);
    if (arg1) (arg1)->materials = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_materials_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_3DSOLID_material *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_materials_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_materials_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (Dwg_3DSOLID_material *) ((arg1)->materials);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_3DSOLID_material, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_revision_guid_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) (BITCODE_RC *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_revision_guid_set(self,revision_guid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_revision_guid_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LOFTEDSURFACE_revision_guid_set" "', argument " "2"" of type '" "BITCODE_RC [39]""'"); 
    } 
    arg2 = (BITCODE_RC *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)39; ++ii) *(BITCODE_RC *)&arg1->revision_guid[ii] = *((BITCODE_RC *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""revision_guid""' of type '""BITCODE_RC [39]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_revision_guid_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_revision_guid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_revision_guid_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_RC *)(BITCODE_RC *) ((arg1)->revision_guid);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_revision_major_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_revision_major_set(self,revision_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_revision_major_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_revision_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->revision_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_revision_major_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_revision_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_revision_major_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->revision_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_revision_minor1_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_revision_minor1_set(self,revision_minor1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_revision_minor1_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_revision_minor1_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->revision_minor1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_revision_minor1_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_revision_minor1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_revision_minor1_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->revision_minor1);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_revision_minor2_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_revision_minor2_set(self,revision_minor2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_revision_minor2_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_revision_minor2_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->revision_minor2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_revision_minor2_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_revision_minor2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_revision_minor2_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->revision_minor2);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_revision_bytes_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) (BITCODE_RC *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_revision_bytes_set(self,revision_bytes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_revision_bytes_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LOFTEDSURFACE_revision_bytes_set" "', argument " "2"" of type '" "BITCODE_RC [9]""'"); 
    } 
    arg2 = (BITCODE_RC *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)9; ++ii) *(BITCODE_RC *)&arg1->revision_bytes[ii] = *((BITCODE_RC *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""revision_bytes""' of type '""BITCODE_RC [9]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_revision_bytes_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_revision_bytes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_revision_bytes_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_RC *)(BITCODE_RC *) ((arg1)->revision_bytes);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_end_marker_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_end_marker_set(self,end_marker);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_end_marker_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_end_marker_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->end_marker = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_end_marker_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_end_marker_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_end_marker_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->end_marker);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_history_id_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_history_id_set(self,history_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_history_id_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LOFTEDSURFACE_history_id_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->history_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_history_id_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_history_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_history_id_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_H) ((arg1)->history_id);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_has_revision_guid_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_has_revision_guid_set(self,has_revision_guid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_has_revision_guid_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_has_revision_guid_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_revision_guid = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_has_revision_guid_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_has_revision_guid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_has_revision_guid_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->has_revision_guid);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_acis_empty_bit_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_acis_empty_bit_set(self,acis_empty_bit);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_acis_empty_bit_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_acis_empty_bit_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->acis_empty_bit = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_acis_empty_bit_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_acis_empty_bit_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_acis_empty_bit_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->acis_empty_bit);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_modeler_format_version_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_modeler_format_version_set(self,modeler_format_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_modeler_format_version_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_modeler_format_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->modeler_format_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_modeler_format_version_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_modeler_format_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_modeler_format_version_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->modeler_format_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_u_isolines_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_u_isolines_set(self,u_isolines);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_u_isolines_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_u_isolines_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->u_isolines = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_u_isolines_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_u_isolines_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_u_isolines_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->u_isolines);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_v_isolines_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_v_isolines_set(self,v_isolines);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_v_isolines_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_v_isolines_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->v_isolines = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_v_isolines_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_v_isolines_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_v_isolines_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->v_isolines);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_loft_entity_transmatrix_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_loft_entity_transmatrix_set(self,loft_entity_transmatrix);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_loft_entity_transmatrix_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LOFTEDSURFACE_loft_entity_transmatrix_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->loft_entity_transmatrix = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_loft_entity_transmatrix_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_loft_entity_transmatrix_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_loft_entity_transmatrix_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_BD *) ((arg1)->loft_entity_transmatrix);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_plane_normal_lofting_type_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_plane_normal_lofting_type_set(self,plane_normal_lofting_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_plane_normal_lofting_type_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_plane_normal_lofting_type_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->plane_normal_lofting_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_plane_normal_lofting_type_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_plane_normal_lofting_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_plane_normal_lofting_type_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->plane_normal_lofting_type);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_start_draft_angle_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_start_draft_angle_set(self,start_draft_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_start_draft_angle_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_start_draft_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->start_draft_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_start_draft_angle_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_start_draft_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_start_draft_angle_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_BD) ((arg1)->start_draft_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_end_draft_angle_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_end_draft_angle_set(self,end_draft_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_end_draft_angle_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_end_draft_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->end_draft_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_end_draft_angle_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_end_draft_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_end_draft_angle_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_BD) ((arg1)->end_draft_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_start_draft_magnitude_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_start_draft_magnitude_set(self,start_draft_magnitude);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_start_draft_magnitude_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_start_draft_magnitude_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->start_draft_magnitude = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_start_draft_magnitude_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_start_draft_magnitude_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_start_draft_magnitude_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_BD) ((arg1)->start_draft_magnitude);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_end_draft_magnitude_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_end_draft_magnitude_set(self,end_draft_magnitude);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_end_draft_magnitude_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_end_draft_magnitude_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->end_draft_magnitude = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_end_draft_magnitude_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_end_draft_magnitude_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_end_draft_magnitude_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_BD) ((arg1)->end_draft_magnitude);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_arc_length_parameterization_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_arc_length_parameterization_set(self,arc_length_parameterization);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_arc_length_parameterization_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_arc_length_parameterization_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->arc_length_parameterization = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_arc_length_parameterization_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_arc_length_parameterization_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_arc_length_parameterization_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->arc_length_parameterization);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_no_twist_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_no_twist_set(self,no_twist);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_no_twist_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_no_twist_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->no_twist = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_no_twist_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_no_twist_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_no_twist_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->no_twist);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_align_direction_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_align_direction_set(self,align_direction);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_align_direction_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_align_direction_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->align_direction = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_align_direction_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_align_direction_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_align_direction_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->align_direction);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_simple_surfaces_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_simple_surfaces_set(self,simple_surfaces);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_simple_surfaces_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_simple_surfaces_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->simple_surfaces = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_simple_surfaces_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_simple_surfaces_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_simple_surfaces_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->simple_surfaces);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_closed_surfaces_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_closed_surfaces_set(self,closed_surfaces);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_closed_surfaces_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_closed_surfaces_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->closed_surfaces = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_closed_surfaces_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_closed_surfaces_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_closed_surfaces_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->closed_surfaces);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_solid_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_solid_set(self,solid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_solid_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_solid_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->solid = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_solid_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_solid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_solid_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->solid);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_ruled_surface_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_ruled_surface_set(self,ruled_surface);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_ruled_surface_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_ruled_surface_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->ruled_surface = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_ruled_surface_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_ruled_surface_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_ruled_surface_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->ruled_surface);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_virtual_guide_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_virtual_guide_set(self,virtual_guide);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_virtual_guide_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_virtual_guide_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->virtual_guide = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_virtual_guide_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_virtual_guide_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_virtual_guide_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->virtual_guide);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_num_cross_sections_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_num_cross_sections_set(self,num_cross_sections);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_num_cross_sections_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_num_cross_sections_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_cross_sections = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_num_cross_sections_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_num_cross_sections_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_num_cross_sections_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_cross_sections);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_num_guide_curves_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_num_guide_curves_set(self,num_guide_curves);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_num_guide_curves_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LOFTEDSURFACE_num_guide_curves_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_guide_curves = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_num_guide_curves_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_num_guide_curves_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_num_guide_curves_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_guide_curves);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_cross_sections_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_cross_sections_set(self,cross_sections);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_cross_sections_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LOFTEDSURFACE_cross_sections_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->cross_sections = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_cross_sections_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_cross_sections_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_cross_sections_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_H *) ((arg1)->cross_sections);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_guide_curves_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_guide_curves_set(self,guide_curves);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_guide_curves_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LOFTEDSURFACE_guide_curves_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->guide_curves = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_guide_curves_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_guide_curves_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_guide_curves_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_H *) ((arg1)->guide_curves);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_path_curve_set) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_path_curve_set(self,path_curve);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_path_curve_set" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LOFTEDSURFACE_path_curve_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->path_curve = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOFTEDSURFACE_path_curve_get) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOFTEDSURFACE_path_curve_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOFTEDSURFACE_path_curve_get" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    result = (BITCODE_H) ((arg1)->path_curve);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_LOFTEDSURFACE) {
  {
    int argvi = 0;
    struct _dwg_entity_LOFTEDSURFACE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_LOFTEDSURFACE();");
    }
    result = (struct _dwg_entity_LOFTEDSURFACE *)calloc(1, sizeof(struct _dwg_entity_LOFTEDSURFACE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_LOFTEDSURFACE) {
  {
    struct _dwg_entity_LOFTEDSURFACE *arg1 = (struct _dwg_entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_LOFTEDSURFACE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_LOFTEDSURFACE" "', argument " "1"" of type '" "struct _dwg_entity_LOFTEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_LOFTEDSURFACE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_parent_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_NURBSURFACE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_parent_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_acis_empty_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_acis_empty_set(self,acis_empty);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_acis_empty_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_NURBSURFACE_acis_empty_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->acis_empty = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_acis_empty_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_acis_empty_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_acis_empty_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->acis_empty);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_unknown_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_unknown_set(self,unknown);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_unknown_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_NURBSURFACE_unknown_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_unknown_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_unknown_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_unknown_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_version_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_version_set(self,version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_version_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_NURBSURFACE_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_version_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_version_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_num_blocks_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_num_blocks_set(self,num_blocks);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_num_blocks_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_NURBSURFACE_num_blocks_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_blocks = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_num_blocks_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_num_blocks_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_num_blocks_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_blocks);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_block_size_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_block_size_set(self,block_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_block_size_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_NURBSURFACE_block_size_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->block_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_block_size_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_block_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_block_size_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (BITCODE_BL *) ((arg1)->block_size);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_encr_sat_data_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    char **arg2 = (char **) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_encr_sat_data_set(self,encr_sat_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_encr_sat_data_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_NURBSURFACE_encr_sat_data_set" "', argument " "2"" of type '" "char **""'"); 
    }
    arg2 = (char **)(argp2);
    if (arg1) (arg1)->encr_sat_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_encr_sat_data_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    char **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_encr_sat_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_encr_sat_data_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (char **) ((arg1)->encr_sat_data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_sab_size_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_sab_size_set(self,sab_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_sab_size_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_NURBSURFACE_sab_size_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->sab_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_sab_size_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_sab_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_sab_size_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->sab_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_acis_data_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_acis_data_set(self,acis_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_acis_data_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_NURBSURFACE_acis_data_set" "', argument " "2"" of type '" "BITCODE_RC *""'"); 
    }
    arg2 = (BITCODE_RC *)(argp2);
    if (arg1) (arg1)->acis_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_acis_data_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_acis_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_acis_data_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (BITCODE_RC *) ((arg1)->acis_data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_wireframe_data_present_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_wireframe_data_present_set(self,wireframe_data_present);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_wireframe_data_present_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_NURBSURFACE_wireframe_data_present_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->wireframe_data_present = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_wireframe_data_present_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_wireframe_data_present_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_wireframe_data_present_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->wireframe_data_present);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_point_present_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_point_present_set(self,point_present);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_point_present_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_NURBSURFACE_point_present_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->point_present = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_point_present_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_point_present_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_point_present_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->point_present);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_point_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_point_set(self,point);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_point_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_NURBSURFACE_point_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->point = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_point_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_point_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_point_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->point);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_isolines_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_isolines_set(self,isolines);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_isolines_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_NURBSURFACE_isolines_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->isolines = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_isolines_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_isolines_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_isolines_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->isolines);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_isoline_present_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_isoline_present_set(self,isoline_present);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_isoline_present_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_NURBSURFACE_isoline_present_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->isoline_present = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_isoline_present_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_isoline_present_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_isoline_present_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->isoline_present);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_num_wires_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_num_wires_set(self,num_wires);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_num_wires_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_NURBSURFACE_num_wires_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_wires = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_num_wires_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_num_wires_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_num_wires_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_wires);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_wires_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    Dwg_3DSOLID_wire *arg2 = (Dwg_3DSOLID_wire *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_wires_set(self,wires);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_wires_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_3DSOLID_wire, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_NURBSURFACE_wires_set" "', argument " "2"" of type '" "Dwg_3DSOLID_wire *""'"); 
    }
    arg2 = (Dwg_3DSOLID_wire *)(argp2);
    if (arg1) (arg1)->wires = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_wires_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_3DSOLID_wire *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_wires_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_wires_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (Dwg_3DSOLID_wire *) ((arg1)->wires);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_3DSOLID_wire, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_num_silhouettes_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_num_silhouettes_set(self,num_silhouettes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_num_silhouettes_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_NURBSURFACE_num_silhouettes_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_silhouettes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_num_silhouettes_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_num_silhouettes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_num_silhouettes_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_silhouettes);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_silhouettes_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    Dwg_3DSOLID_silhouette *arg2 = (Dwg_3DSOLID_silhouette *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_silhouettes_set(self,silhouettes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_silhouettes_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_3DSOLID_silhouette, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_NURBSURFACE_silhouettes_set" "', argument " "2"" of type '" "Dwg_3DSOLID_silhouette *""'"); 
    }
    arg2 = (Dwg_3DSOLID_silhouette *)(argp2);
    if (arg1) (arg1)->silhouettes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_silhouettes_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_3DSOLID_silhouette *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_silhouettes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_silhouettes_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (Dwg_3DSOLID_silhouette *) ((arg1)->silhouettes);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_3DSOLID_silhouette, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE__dxf_sab_converted_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE__dxf_sab_converted_set(self,_dxf_sab_converted);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE__dxf_sab_converted_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_NURBSURFACE__dxf_sab_converted_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->_dxf_sab_converted = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE__dxf_sab_converted_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE__dxf_sab_converted_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE__dxf_sab_converted_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->_dxf_sab_converted);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_acis_empty2_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_acis_empty2_set(self,acis_empty2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_acis_empty2_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_NURBSURFACE_acis_empty2_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->acis_empty2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_acis_empty2_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_acis_empty2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_acis_empty2_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->acis_empty2);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_extra_acis_data_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    struct _dwg_entity_3DSOLID *arg2 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_extra_acis_data_set(self,extra_acis_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_extra_acis_data_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_3DSOLID, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_NURBSURFACE_extra_acis_data_set" "', argument " "2"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg2 = (struct _dwg_entity_3DSOLID *)(argp2);
    if (arg1) (arg1)->extra_acis_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_extra_acis_data_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_entity_3DSOLID *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_extra_acis_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_extra_acis_data_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (struct _dwg_entity_3DSOLID *) ((arg1)->extra_acis_data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_3DSOLID, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_num_materials_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_num_materials_set(self,num_materials);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_num_materials_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_NURBSURFACE_num_materials_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_materials = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_num_materials_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_num_materials_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_num_materials_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_materials);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_materials_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    Dwg_3DSOLID_material *arg2 = (Dwg_3DSOLID_material *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_materials_set(self,materials);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_materials_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_3DSOLID_material, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_NURBSURFACE_materials_set" "', argument " "2"" of type '" "Dwg_3DSOLID_material *""'"); 
    }
    arg2 = (Dwg_3DSOLID_material *)(argp2);
    if (arg1) (arg1)->materials = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_materials_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_3DSOLID_material *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_materials_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_materials_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (Dwg_3DSOLID_material *) ((arg1)->materials);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_3DSOLID_material, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_revision_guid_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) (BITCODE_RC *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_revision_guid_set(self,revision_guid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_revision_guid_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_NURBSURFACE_revision_guid_set" "', argument " "2"" of type '" "BITCODE_RC [39]""'"); 
    } 
    arg2 = (BITCODE_RC *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)39; ++ii) *(BITCODE_RC *)&arg1->revision_guid[ii] = *((BITCODE_RC *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""revision_guid""' of type '""BITCODE_RC [39]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_revision_guid_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_revision_guid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_revision_guid_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (BITCODE_RC *)(BITCODE_RC *) ((arg1)->revision_guid);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_revision_major_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_revision_major_set(self,revision_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_revision_major_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_NURBSURFACE_revision_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->revision_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_revision_major_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_revision_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_revision_major_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->revision_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_revision_minor1_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_revision_minor1_set(self,revision_minor1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_revision_minor1_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_NURBSURFACE_revision_minor1_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->revision_minor1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_revision_minor1_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_revision_minor1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_revision_minor1_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->revision_minor1);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_revision_minor2_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_revision_minor2_set(self,revision_minor2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_revision_minor2_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_NURBSURFACE_revision_minor2_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->revision_minor2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_revision_minor2_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_revision_minor2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_revision_minor2_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->revision_minor2);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_revision_bytes_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) (BITCODE_RC *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_revision_bytes_set(self,revision_bytes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_revision_bytes_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_NURBSURFACE_revision_bytes_set" "', argument " "2"" of type '" "BITCODE_RC [9]""'"); 
    } 
    arg2 = (BITCODE_RC *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)9; ++ii) *(BITCODE_RC *)&arg1->revision_bytes[ii] = *((BITCODE_RC *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""revision_bytes""' of type '""BITCODE_RC [9]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_revision_bytes_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_revision_bytes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_revision_bytes_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (BITCODE_RC *)(BITCODE_RC *) ((arg1)->revision_bytes);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_end_marker_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_end_marker_set(self,end_marker);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_end_marker_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_NURBSURFACE_end_marker_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->end_marker = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_end_marker_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_end_marker_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_end_marker_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->end_marker);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_history_id_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_history_id_set(self,history_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_history_id_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_NURBSURFACE_history_id_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->history_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_history_id_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_history_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_history_id_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (BITCODE_H) ((arg1)->history_id);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_has_revision_guid_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_has_revision_guid_set(self,has_revision_guid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_has_revision_guid_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_NURBSURFACE_has_revision_guid_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_revision_guid = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_has_revision_guid_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_has_revision_guid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_has_revision_guid_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->has_revision_guid);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_acis_empty_bit_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_acis_empty_bit_set(self,acis_empty_bit);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_acis_empty_bit_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_NURBSURFACE_acis_empty_bit_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->acis_empty_bit = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_acis_empty_bit_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_acis_empty_bit_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_acis_empty_bit_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->acis_empty_bit);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_u_isolines_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_u_isolines_set(self,u_isolines);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_u_isolines_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_NURBSURFACE_u_isolines_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->u_isolines = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_u_isolines_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_u_isolines_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_u_isolines_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->u_isolines);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_v_isolines_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_v_isolines_set(self,v_isolines);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_v_isolines_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_NURBSURFACE_v_isolines_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->v_isolines = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_v_isolines_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_v_isolines_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_v_isolines_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->v_isolines);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_short170_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_short170_set(self,short170);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_short170_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_NURBSURFACE_short170_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->short170 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_short170_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_short170_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_short170_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->short170);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_cv_hull_display_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_cv_hull_display_set(self,cv_hull_display);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_cv_hull_display_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_NURBSURFACE_cv_hull_display_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->cv_hull_display = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_cv_hull_display_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_cv_hull_display_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_cv_hull_display_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->cv_hull_display);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_uvec1_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_uvec1_set(self,uvec1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_uvec1_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_NURBSURFACE_uvec1_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->uvec1 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_uvec1_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_uvec1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_uvec1_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->uvec1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_vvec1_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_vvec1_set(self,vvec1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_vvec1_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_NURBSURFACE_vvec1_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->vvec1 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_vvec1_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_vvec1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_vvec1_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->vvec1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_uvec2_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_uvec2_set(self,uvec2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_uvec2_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_NURBSURFACE_uvec2_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->uvec2 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_uvec2_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_uvec2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_uvec2_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->uvec2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_vvec2_set) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_vvec2_set(self,vvec2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_vvec2_set" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_NURBSURFACE_vvec2_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->vvec2 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NURBSURFACE_vvec2_get) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NURBSURFACE_vvec2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NURBSURFACE_vvec2_get" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->vvec2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_NURBSURFACE) {
  {
    int argvi = 0;
    struct _dwg_entity_NURBSURFACE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_NURBSURFACE();");
    }
    result = (struct _dwg_entity_NURBSURFACE *)calloc(1, sizeof(struct _dwg_entity_NURBSURFACE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_NURBSURFACE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_NURBSURFACE) {
  {
    struct _dwg_entity_NURBSURFACE *arg1 = (struct _dwg_entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_NURBSURFACE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NURBSURFACE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_NURBSURFACE" "', argument " "1"" of type '" "struct _dwg_entity_NURBSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_NURBSURFACE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_parent_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_PLANESURFACE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_parent_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_acis_empty_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_acis_empty_set(self,acis_empty);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_acis_empty_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_PLANESURFACE_acis_empty_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->acis_empty = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_acis_empty_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_acis_empty_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_acis_empty_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->acis_empty);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_unknown_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_unknown_set(self,unknown);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_unknown_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_PLANESURFACE_unknown_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_unknown_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_unknown_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_unknown_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_version_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_version_set(self,version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_version_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_PLANESURFACE_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_version_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_version_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_num_blocks_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_num_blocks_set(self,num_blocks);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_num_blocks_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_PLANESURFACE_num_blocks_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_blocks = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_num_blocks_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_num_blocks_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_num_blocks_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_blocks);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_block_size_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_block_size_set(self,block_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_block_size_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_PLANESURFACE_block_size_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->block_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_block_size_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_block_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_block_size_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (BITCODE_BL *) ((arg1)->block_size);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_encr_sat_data_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    char **arg2 = (char **) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_encr_sat_data_set(self,encr_sat_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_encr_sat_data_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_PLANESURFACE_encr_sat_data_set" "', argument " "2"" of type '" "char **""'"); 
    }
    arg2 = (char **)(argp2);
    if (arg1) (arg1)->encr_sat_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_encr_sat_data_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    char **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_encr_sat_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_encr_sat_data_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (char **) ((arg1)->encr_sat_data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_sab_size_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_sab_size_set(self,sab_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_sab_size_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_PLANESURFACE_sab_size_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->sab_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_sab_size_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_sab_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_sab_size_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->sab_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_acis_data_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_acis_data_set(self,acis_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_acis_data_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_PLANESURFACE_acis_data_set" "', argument " "2"" of type '" "BITCODE_RC *""'"); 
    }
    arg2 = (BITCODE_RC *)(argp2);
    if (arg1) (arg1)->acis_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_acis_data_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_acis_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_acis_data_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (BITCODE_RC *) ((arg1)->acis_data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_wireframe_data_present_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_wireframe_data_present_set(self,wireframe_data_present);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_wireframe_data_present_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_PLANESURFACE_wireframe_data_present_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->wireframe_data_present = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_wireframe_data_present_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_wireframe_data_present_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_wireframe_data_present_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->wireframe_data_present);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_point_present_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_point_present_set(self,point_present);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_point_present_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_PLANESURFACE_point_present_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->point_present = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_point_present_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_point_present_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_point_present_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->point_present);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_point_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_point_set(self,point);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_point_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_PLANESURFACE_point_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->point = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_point_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_point_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_point_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->point);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_isolines_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_isolines_set(self,isolines);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_isolines_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_PLANESURFACE_isolines_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->isolines = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_isolines_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_isolines_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_isolines_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->isolines);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_isoline_present_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_isoline_present_set(self,isoline_present);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_isoline_present_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_PLANESURFACE_isoline_present_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->isoline_present = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_isoline_present_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_isoline_present_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_isoline_present_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->isoline_present);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_num_wires_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_num_wires_set(self,num_wires);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_num_wires_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_PLANESURFACE_num_wires_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_wires = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_num_wires_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_num_wires_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_num_wires_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_wires);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_wires_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    Dwg_3DSOLID_wire *arg2 = (Dwg_3DSOLID_wire *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_wires_set(self,wires);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_wires_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_3DSOLID_wire, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_PLANESURFACE_wires_set" "', argument " "2"" of type '" "Dwg_3DSOLID_wire *""'"); 
    }
    arg2 = (Dwg_3DSOLID_wire *)(argp2);
    if (arg1) (arg1)->wires = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_wires_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_3DSOLID_wire *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_wires_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_wires_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (Dwg_3DSOLID_wire *) ((arg1)->wires);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_3DSOLID_wire, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_num_silhouettes_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_num_silhouettes_set(self,num_silhouettes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_num_silhouettes_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_PLANESURFACE_num_silhouettes_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_silhouettes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_num_silhouettes_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_num_silhouettes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_num_silhouettes_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_silhouettes);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_silhouettes_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    Dwg_3DSOLID_silhouette *arg2 = (Dwg_3DSOLID_silhouette *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_silhouettes_set(self,silhouettes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_silhouettes_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_3DSOLID_silhouette, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_PLANESURFACE_silhouettes_set" "', argument " "2"" of type '" "Dwg_3DSOLID_silhouette *""'"); 
    }
    arg2 = (Dwg_3DSOLID_silhouette *)(argp2);
    if (arg1) (arg1)->silhouettes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_silhouettes_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_3DSOLID_silhouette *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_silhouettes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_silhouettes_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (Dwg_3DSOLID_silhouette *) ((arg1)->silhouettes);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_3DSOLID_silhouette, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE__dxf_sab_converted_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE__dxf_sab_converted_set(self,_dxf_sab_converted);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE__dxf_sab_converted_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_PLANESURFACE__dxf_sab_converted_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->_dxf_sab_converted = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE__dxf_sab_converted_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE__dxf_sab_converted_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE__dxf_sab_converted_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->_dxf_sab_converted);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_acis_empty2_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_acis_empty2_set(self,acis_empty2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_acis_empty2_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_PLANESURFACE_acis_empty2_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->acis_empty2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_acis_empty2_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_acis_empty2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_acis_empty2_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->acis_empty2);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_extra_acis_data_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    struct _dwg_entity_3DSOLID *arg2 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_extra_acis_data_set(self,extra_acis_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_extra_acis_data_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_3DSOLID, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_PLANESURFACE_extra_acis_data_set" "', argument " "2"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg2 = (struct _dwg_entity_3DSOLID *)(argp2);
    if (arg1) (arg1)->extra_acis_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_extra_acis_data_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_entity_3DSOLID *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_extra_acis_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_extra_acis_data_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (struct _dwg_entity_3DSOLID *) ((arg1)->extra_acis_data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_3DSOLID, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_num_materials_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_num_materials_set(self,num_materials);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_num_materials_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_PLANESURFACE_num_materials_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_materials = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_num_materials_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_num_materials_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_num_materials_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_materials);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_materials_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    Dwg_3DSOLID_material *arg2 = (Dwg_3DSOLID_material *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_materials_set(self,materials);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_materials_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_3DSOLID_material, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_PLANESURFACE_materials_set" "', argument " "2"" of type '" "Dwg_3DSOLID_material *""'"); 
    }
    arg2 = (Dwg_3DSOLID_material *)(argp2);
    if (arg1) (arg1)->materials = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_materials_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_3DSOLID_material *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_materials_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_materials_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (Dwg_3DSOLID_material *) ((arg1)->materials);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_3DSOLID_material, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_revision_guid_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) (BITCODE_RC *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_revision_guid_set(self,revision_guid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_revision_guid_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_PLANESURFACE_revision_guid_set" "', argument " "2"" of type '" "BITCODE_RC [39]""'"); 
    } 
    arg2 = (BITCODE_RC *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)39; ++ii) *(BITCODE_RC *)&arg1->revision_guid[ii] = *((BITCODE_RC *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""revision_guid""' of type '""BITCODE_RC [39]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_revision_guid_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_revision_guid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_revision_guid_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (BITCODE_RC *)(BITCODE_RC *) ((arg1)->revision_guid);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_revision_major_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_revision_major_set(self,revision_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_revision_major_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_PLANESURFACE_revision_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->revision_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_revision_major_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_revision_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_revision_major_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->revision_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_revision_minor1_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_revision_minor1_set(self,revision_minor1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_revision_minor1_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_PLANESURFACE_revision_minor1_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->revision_minor1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_revision_minor1_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_revision_minor1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_revision_minor1_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->revision_minor1);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_revision_minor2_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_revision_minor2_set(self,revision_minor2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_revision_minor2_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_PLANESURFACE_revision_minor2_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->revision_minor2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_revision_minor2_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_revision_minor2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_revision_minor2_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->revision_minor2);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_revision_bytes_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) (BITCODE_RC *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_revision_bytes_set(self,revision_bytes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_revision_bytes_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_PLANESURFACE_revision_bytes_set" "', argument " "2"" of type '" "BITCODE_RC [9]""'"); 
    } 
    arg2 = (BITCODE_RC *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)9; ++ii) *(BITCODE_RC *)&arg1->revision_bytes[ii] = *((BITCODE_RC *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""revision_bytes""' of type '""BITCODE_RC [9]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_revision_bytes_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_revision_bytes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_revision_bytes_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (BITCODE_RC *)(BITCODE_RC *) ((arg1)->revision_bytes);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_end_marker_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_end_marker_set(self,end_marker);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_end_marker_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_PLANESURFACE_end_marker_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->end_marker = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_end_marker_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_end_marker_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_end_marker_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->end_marker);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_history_id_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_history_id_set(self,history_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_history_id_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_PLANESURFACE_history_id_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->history_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_history_id_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_history_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_history_id_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (BITCODE_H) ((arg1)->history_id);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_has_revision_guid_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_has_revision_guid_set(self,has_revision_guid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_has_revision_guid_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_PLANESURFACE_has_revision_guid_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_revision_guid = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_has_revision_guid_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_has_revision_guid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_has_revision_guid_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->has_revision_guid);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_acis_empty_bit_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_acis_empty_bit_set(self,acis_empty_bit);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_acis_empty_bit_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_PLANESURFACE_acis_empty_bit_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->acis_empty_bit = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_acis_empty_bit_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_acis_empty_bit_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_acis_empty_bit_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->acis_empty_bit);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_modeler_format_version_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_modeler_format_version_set(self,modeler_format_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_modeler_format_version_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_PLANESURFACE_modeler_format_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->modeler_format_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_modeler_format_version_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_modeler_format_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_modeler_format_version_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->modeler_format_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_u_isolines_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_u_isolines_set(self,u_isolines);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_u_isolines_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_PLANESURFACE_u_isolines_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->u_isolines = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_u_isolines_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_u_isolines_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_u_isolines_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->u_isolines);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_v_isolines_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_v_isolines_set(self,v_isolines);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_v_isolines_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_PLANESURFACE_v_isolines_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->v_isolines = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_v_isolines_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_v_isolines_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_v_isolines_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->v_isolines);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_class_version_set) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_class_version_set" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_PLANESURFACE_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_PLANESURFACE_class_version_get) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_PLANESURFACE_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_PLANESURFACE_class_version_get" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_PLANESURFACE) {
  {
    int argvi = 0;
    struct _dwg_entity_PLANESURFACE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_PLANESURFACE();");
    }
    result = (struct _dwg_entity_PLANESURFACE *)calloc(1, sizeof(struct _dwg_entity_PLANESURFACE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_PLANESURFACE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_PLANESURFACE) {
  {
    struct _dwg_entity_PLANESURFACE *arg1 = (struct _dwg_entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_PLANESURFACE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_PLANESURFACE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_PLANESURFACE" "', argument " "1"" of type '" "struct _dwg_entity_PLANESURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_PLANESURFACE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_parent_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_parent_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_acis_empty_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_acis_empty_set(self,acis_empty);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_acis_empty_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_acis_empty_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->acis_empty = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_acis_empty_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_acis_empty_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_acis_empty_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->acis_empty);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_unknown_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_unknown_set(self,unknown);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_unknown_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_unknown_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_unknown_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_unknown_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_unknown_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_version_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_version_set(self,version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_version_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_version_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_version_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_num_blocks_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_num_blocks_set(self,num_blocks);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_num_blocks_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_num_blocks_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_blocks = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_num_blocks_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_num_blocks_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_num_blocks_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_blocks);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_block_size_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_block_size_set(self,block_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_block_size_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_block_size_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->block_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_block_size_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_block_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_block_size_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_BL *) ((arg1)->block_size);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_encr_sat_data_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    char **arg2 = (char **) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_encr_sat_data_set(self,encr_sat_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_encr_sat_data_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_encr_sat_data_set" "', argument " "2"" of type '" "char **""'"); 
    }
    arg2 = (char **)(argp2);
    if (arg1) (arg1)->encr_sat_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_encr_sat_data_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    char **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_encr_sat_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_encr_sat_data_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (char **) ((arg1)->encr_sat_data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_sab_size_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_sab_size_set(self,sab_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_sab_size_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_sab_size_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->sab_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_sab_size_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_sab_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_sab_size_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->sab_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_acis_data_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_acis_data_set(self,acis_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_acis_data_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_acis_data_set" "', argument " "2"" of type '" "BITCODE_RC *""'"); 
    }
    arg2 = (BITCODE_RC *)(argp2);
    if (arg1) (arg1)->acis_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_acis_data_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_acis_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_acis_data_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_RC *) ((arg1)->acis_data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_wireframe_data_present_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_wireframe_data_present_set(self,wireframe_data_present);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_wireframe_data_present_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_wireframe_data_present_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->wireframe_data_present = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_wireframe_data_present_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_wireframe_data_present_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_wireframe_data_present_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->wireframe_data_present);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_point_present_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_point_present_set(self,point_present);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_point_present_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_point_present_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->point_present = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_point_present_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_point_present_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_point_present_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->point_present);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_point_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_point_set(self,point);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_point_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_point_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->point = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_point_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_point_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_point_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->point);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_isolines_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_isolines_set(self,isolines);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_isolines_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_isolines_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->isolines = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_isolines_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_isolines_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_isolines_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->isolines);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_isoline_present_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_isoline_present_set(self,isoline_present);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_isoline_present_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_isoline_present_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->isoline_present = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_isoline_present_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_isoline_present_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_isoline_present_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->isoline_present);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_num_wires_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_num_wires_set(self,num_wires);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_num_wires_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_num_wires_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_wires = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_num_wires_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_num_wires_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_num_wires_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_wires);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_wires_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    Dwg_3DSOLID_wire *arg2 = (Dwg_3DSOLID_wire *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_wires_set(self,wires);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_wires_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_3DSOLID_wire, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_wires_set" "', argument " "2"" of type '" "Dwg_3DSOLID_wire *""'"); 
    }
    arg2 = (Dwg_3DSOLID_wire *)(argp2);
    if (arg1) (arg1)->wires = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_wires_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_3DSOLID_wire *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_wires_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_wires_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (Dwg_3DSOLID_wire *) ((arg1)->wires);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_3DSOLID_wire, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_num_silhouettes_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_num_silhouettes_set(self,num_silhouettes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_num_silhouettes_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_num_silhouettes_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_silhouettes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_num_silhouettes_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_num_silhouettes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_num_silhouettes_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_silhouettes);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_silhouettes_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    Dwg_3DSOLID_silhouette *arg2 = (Dwg_3DSOLID_silhouette *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_silhouettes_set(self,silhouettes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_silhouettes_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_3DSOLID_silhouette, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_silhouettes_set" "', argument " "2"" of type '" "Dwg_3DSOLID_silhouette *""'"); 
    }
    arg2 = (Dwg_3DSOLID_silhouette *)(argp2);
    if (arg1) (arg1)->silhouettes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_silhouettes_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_3DSOLID_silhouette *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_silhouettes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_silhouettes_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (Dwg_3DSOLID_silhouette *) ((arg1)->silhouettes);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_3DSOLID_silhouette, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE__dxf_sab_converted_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE__dxf_sab_converted_set(self,_dxf_sab_converted);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE__dxf_sab_converted_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_REVOLVEDSURFACE__dxf_sab_converted_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->_dxf_sab_converted = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE__dxf_sab_converted_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE__dxf_sab_converted_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE__dxf_sab_converted_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->_dxf_sab_converted);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_acis_empty2_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_acis_empty2_set(self,acis_empty2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_acis_empty2_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_acis_empty2_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->acis_empty2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_acis_empty2_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_acis_empty2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_acis_empty2_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->acis_empty2);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_extra_acis_data_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    struct _dwg_entity_3DSOLID *arg2 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_extra_acis_data_set(self,extra_acis_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_extra_acis_data_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_3DSOLID, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_extra_acis_data_set" "', argument " "2"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg2 = (struct _dwg_entity_3DSOLID *)(argp2);
    if (arg1) (arg1)->extra_acis_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_extra_acis_data_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_entity_3DSOLID *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_extra_acis_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_extra_acis_data_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (struct _dwg_entity_3DSOLID *) ((arg1)->extra_acis_data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_3DSOLID, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_num_materials_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_num_materials_set(self,num_materials);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_num_materials_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_num_materials_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_materials = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_num_materials_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_num_materials_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_num_materials_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_materials);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_materials_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    Dwg_3DSOLID_material *arg2 = (Dwg_3DSOLID_material *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_materials_set(self,materials);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_materials_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_3DSOLID_material, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_materials_set" "', argument " "2"" of type '" "Dwg_3DSOLID_material *""'"); 
    }
    arg2 = (Dwg_3DSOLID_material *)(argp2);
    if (arg1) (arg1)->materials = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_materials_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_3DSOLID_material *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_materials_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_materials_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (Dwg_3DSOLID_material *) ((arg1)->materials);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_3DSOLID_material, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_revision_guid_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) (BITCODE_RC *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_revision_guid_set(self,revision_guid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_revision_guid_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_revision_guid_set" "', argument " "2"" of type '" "BITCODE_RC [39]""'"); 
    } 
    arg2 = (BITCODE_RC *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)39; ++ii) *(BITCODE_RC *)&arg1->revision_guid[ii] = *((BITCODE_RC *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""revision_guid""' of type '""BITCODE_RC [39]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_revision_guid_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_revision_guid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_revision_guid_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_RC *)(BITCODE_RC *) ((arg1)->revision_guid);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_revision_major_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_revision_major_set(self,revision_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_revision_major_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_revision_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->revision_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_revision_major_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_revision_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_revision_major_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->revision_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_revision_minor1_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_revision_minor1_set(self,revision_minor1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_revision_minor1_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_revision_minor1_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->revision_minor1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_revision_minor1_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_revision_minor1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_revision_minor1_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->revision_minor1);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_revision_minor2_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_revision_minor2_set(self,revision_minor2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_revision_minor2_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_revision_minor2_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->revision_minor2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_revision_minor2_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_revision_minor2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_revision_minor2_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->revision_minor2);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_revision_bytes_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) (BITCODE_RC *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_revision_bytes_set(self,revision_bytes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_revision_bytes_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_revision_bytes_set" "', argument " "2"" of type '" "BITCODE_RC [9]""'"); 
    } 
    arg2 = (BITCODE_RC *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)9; ++ii) *(BITCODE_RC *)&arg1->revision_bytes[ii] = *((BITCODE_RC *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""revision_bytes""' of type '""BITCODE_RC [9]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_revision_bytes_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_revision_bytes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_revision_bytes_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_RC *)(BITCODE_RC *) ((arg1)->revision_bytes);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_end_marker_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_end_marker_set(self,end_marker);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_end_marker_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_end_marker_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->end_marker = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_end_marker_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_end_marker_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_end_marker_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->end_marker);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_history_id_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_history_id_set(self,history_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_history_id_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_history_id_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->history_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_history_id_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_history_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_history_id_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_H) ((arg1)->history_id);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_has_revision_guid_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_has_revision_guid_set(self,has_revision_guid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_has_revision_guid_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_has_revision_guid_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_revision_guid = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_has_revision_guid_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_has_revision_guid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_has_revision_guid_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->has_revision_guid);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_acis_empty_bit_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_acis_empty_bit_set(self,acis_empty_bit);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_acis_empty_bit_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_acis_empty_bit_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->acis_empty_bit = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_acis_empty_bit_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_acis_empty_bit_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_acis_empty_bit_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->acis_empty_bit);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_modeler_format_version_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_modeler_format_version_set(self,modeler_format_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_modeler_format_version_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_modeler_format_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->modeler_format_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_modeler_format_version_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_modeler_format_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_modeler_format_version_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->modeler_format_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_u_isolines_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_u_isolines_set(self,u_isolines);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_u_isolines_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_u_isolines_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->u_isolines = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_u_isolines_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_u_isolines_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_u_isolines_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->u_isolines);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_v_isolines_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_v_isolines_set(self,v_isolines);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_v_isolines_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_v_isolines_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->v_isolines = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_v_isolines_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_v_isolines_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_v_isolines_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_BS) ((arg1)->v_isolines);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_class_version_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_class_version_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_class_version_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_class_version_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_id_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_id_set(self,id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_id_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_id_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_id_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_id_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_BL) ((arg1)->id);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_axis_point_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_axis_point_set(self,axis_point);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_axis_point_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_axis_point_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->axis_point = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_axis_point_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_axis_point_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_axis_point_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->axis_point);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_axis_vector_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_axis_vector_set(self,axis_vector);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_axis_vector_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_axis_vector_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->axis_vector = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_axis_vector_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_axis_vector_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_axis_vector_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->axis_vector);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_revolve_angle_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_revolve_angle_set(self,revolve_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_revolve_angle_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_revolve_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->revolve_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_revolve_angle_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_revolve_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_revolve_angle_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_BD) ((arg1)->revolve_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_start_angle_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_start_angle_set(self,start_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_start_angle_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_start_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->start_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_start_angle_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_start_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_start_angle_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_BD) ((arg1)->start_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_revolved_entity_transmatrix_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_revolved_entity_transmatrix_set(self,revolved_entity_transmatrix);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_revolved_entity_transmatrix_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_revolved_entity_transmatrix_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->revolved_entity_transmatrix = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_revolved_entity_transmatrix_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_revolved_entity_transmatrix_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_revolved_entity_transmatrix_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_BD *) ((arg1)->revolved_entity_transmatrix);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_draft_angle_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_draft_angle_set(self,draft_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_draft_angle_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_draft_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->draft_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_draft_angle_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_draft_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_draft_angle_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_BD) ((arg1)->draft_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_draft_start_distance_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_draft_start_distance_set(self,draft_start_distance);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_draft_start_distance_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_draft_start_distance_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->draft_start_distance = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_draft_start_distance_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_draft_start_distance_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_draft_start_distance_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_BD) ((arg1)->draft_start_distance);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_draft_end_distance_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_draft_end_distance_set(self,draft_end_distance);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_draft_end_distance_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_draft_end_distance_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->draft_end_distance = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_draft_end_distance_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_draft_end_distance_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_draft_end_distance_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_BD) ((arg1)->draft_end_distance);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_twist_angle_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_twist_angle_set(self,twist_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_twist_angle_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_twist_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->twist_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_twist_angle_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_twist_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_twist_angle_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_BD) ((arg1)->twist_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_solid_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_solid_set(self,solid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_solid_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_solid_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->solid = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_solid_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_solid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_solid_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->solid);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_close_to_axis_set) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_close_to_axis_set(self,close_to_axis);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_close_to_axis_set" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_REVOLVEDSURFACE_close_to_axis_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->close_to_axis = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REVOLVEDSURFACE_close_to_axis_get) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REVOLVEDSURFACE_close_to_axis_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REVOLVEDSURFACE_close_to_axis_get" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    result = (BITCODE_B) ((arg1)->close_to_axis);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_REVOLVEDSURFACE) {
  {
    int argvi = 0;
    struct _dwg_entity_REVOLVEDSURFACE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_REVOLVEDSURFACE();");
    }
    result = (struct _dwg_entity_REVOLVEDSURFACE *)calloc(1, sizeof(struct _dwg_entity_REVOLVEDSURFACE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_REVOLVEDSURFACE) {
  {
    struct _dwg_entity_REVOLVEDSURFACE *arg1 = (struct _dwg_entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_REVOLVEDSURFACE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_REVOLVEDSURFACE" "', argument " "1"" of type '" "struct _dwg_entity_REVOLVEDSURFACE *""'"); 
    }
    arg1 = (struct _dwg_entity_REVOLVEDSURFACE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MESH_edge_parent_set) {
  {
    struct _dwg_MESH_edge *arg1 = (struct _dwg_MESH_edge *) 0 ;
    struct _dwg_entity_MESH *arg2 = (struct _dwg_entity_MESH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MESH_edge_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MESH_edge, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MESH_edge_parent_set" "', argument " "1"" of type '" "struct _dwg_MESH_edge *""'"); 
    }
    arg1 = (struct _dwg_MESH_edge *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_MESH, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_MESH_edge_parent_set" "', argument " "2"" of type '" "struct _dwg_entity_MESH *""'"); 
    }
    arg2 = (struct _dwg_entity_MESH *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MESH_edge_parent_get) {
  {
    struct _dwg_MESH_edge *arg1 = (struct _dwg_MESH_edge *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_entity_MESH *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MESH_edge_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MESH_edge, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MESH_edge_parent_get" "', argument " "1"" of type '" "struct _dwg_MESH_edge *""'"); 
    }
    arg1 = (struct _dwg_MESH_edge *)(argp1);
    result = (struct _dwg_entity_MESH *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_MESH, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MESH_edge_idxfrom_set) {
  {
    struct _dwg_MESH_edge *arg1 = (struct _dwg_MESH_edge *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MESH_edge_idxfrom_set(self,idxfrom);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MESH_edge, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MESH_edge_idxfrom_set" "', argument " "1"" of type '" "struct _dwg_MESH_edge *""'"); 
    }
    arg1 = (struct _dwg_MESH_edge *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MESH_edge_idxfrom_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->idxfrom = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MESH_edge_idxfrom_get) {
  {
    struct _dwg_MESH_edge *arg1 = (struct _dwg_MESH_edge *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MESH_edge_idxfrom_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MESH_edge, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MESH_edge_idxfrom_get" "', argument " "1"" of type '" "struct _dwg_MESH_edge *""'"); 
    }
    arg1 = (struct _dwg_MESH_edge *)(argp1);
    result = (BITCODE_BL) ((arg1)->idxfrom);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MESH_edge_idxto_set) {
  {
    struct _dwg_MESH_edge *arg1 = (struct _dwg_MESH_edge *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_MESH_edge_idxto_set(self,idxto);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MESH_edge, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MESH_edge_idxto_set" "', argument " "1"" of type '" "struct _dwg_MESH_edge *""'"); 
    }
    arg1 = (struct _dwg_MESH_edge *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_MESH_edge_idxto_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->idxto = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_MESH_edge_idxto_get) {
  {
    struct _dwg_MESH_edge *arg1 = (struct _dwg_MESH_edge *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_MESH_edge_idxto_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MESH_edge, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_MESH_edge_idxto_get" "', argument " "1"" of type '" "struct _dwg_MESH_edge *""'"); 
    }
    arg1 = (struct _dwg_MESH_edge *)(argp1);
    result = (BITCODE_BL) ((arg1)->idxto);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_MESH_edge) {
  {
    int argvi = 0;
    struct _dwg_MESH_edge *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_MESH_edge();");
    }
    result = (struct _dwg_MESH_edge *)calloc(1, sizeof(struct _dwg_MESH_edge));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_MESH_edge, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_MESH_edge) {
  {
    struct _dwg_MESH_edge *arg1 = (struct _dwg_MESH_edge *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_MESH_edge(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_MESH_edge, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_MESH_edge" "', argument " "1"" of type '" "struct _dwg_MESH_edge *""'"); 
    }
    arg1 = (struct _dwg_MESH_edge *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MESH_parent_set) {
  {
    struct _dwg_entity_MESH *arg1 = (struct _dwg_entity_MESH *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MESH_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MESH_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_MESH *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MESH_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MESH_parent_get) {
  {
    struct _dwg_entity_MESH *arg1 = (struct _dwg_entity_MESH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MESH_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MESH_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_MESH *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MESH_dlevel_set) {
  {
    struct _dwg_entity_MESH *arg1 = (struct _dwg_entity_MESH *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MESH_dlevel_set(self,dlevel);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MESH_dlevel_set" "', argument " "1"" of type '" "struct _dwg_entity_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_MESH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MESH_dlevel_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->dlevel = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MESH_dlevel_get) {
  {
    struct _dwg_entity_MESH *arg1 = (struct _dwg_entity_MESH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MESH_dlevel_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MESH_dlevel_get" "', argument " "1"" of type '" "struct _dwg_entity_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_MESH *)(argp1);
    result = (BITCODE_BS) ((arg1)->dlevel);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MESH_is_watertight_set) {
  {
    struct _dwg_entity_MESH *arg1 = (struct _dwg_entity_MESH *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MESH_is_watertight_set(self,is_watertight);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MESH_is_watertight_set" "', argument " "1"" of type '" "struct _dwg_entity_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_MESH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MESH_is_watertight_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_watertight = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MESH_is_watertight_get) {
  {
    struct _dwg_entity_MESH *arg1 = (struct _dwg_entity_MESH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MESH_is_watertight_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MESH_is_watertight_get" "', argument " "1"" of type '" "struct _dwg_entity_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_MESH *)(argp1);
    result = (BITCODE_B) ((arg1)->is_watertight);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MESH_num_subdiv_vertex_set) {
  {
    struct _dwg_entity_MESH *arg1 = (struct _dwg_entity_MESH *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MESH_num_subdiv_vertex_set(self,num_subdiv_vertex);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MESH_num_subdiv_vertex_set" "', argument " "1"" of type '" "struct _dwg_entity_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_MESH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MESH_num_subdiv_vertex_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_subdiv_vertex = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MESH_num_subdiv_vertex_get) {
  {
    struct _dwg_entity_MESH *arg1 = (struct _dwg_entity_MESH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MESH_num_subdiv_vertex_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MESH_num_subdiv_vertex_get" "', argument " "1"" of type '" "struct _dwg_entity_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_MESH *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_subdiv_vertex);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MESH_subdiv_vertex_set) {
  {
    struct _dwg_entity_MESH *arg1 = (struct _dwg_entity_MESH *) 0 ;
    BITCODE_3DPOINT *arg2 = (BITCODE_3DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MESH_subdiv_vertex_set(self,subdiv_vertex);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MESH_subdiv_vertex_set" "', argument " "1"" of type '" "struct _dwg_entity_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_MESH *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MESH_subdiv_vertex_set" "', argument " "2"" of type '" "BITCODE_3DPOINT *""'"); 
    }
    arg2 = (BITCODE_3DPOINT *)(argp2);
    if (arg1) (arg1)->subdiv_vertex = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MESH_subdiv_vertex_get) {
  {
    struct _dwg_entity_MESH *arg1 = (struct _dwg_entity_MESH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MESH_subdiv_vertex_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MESH_subdiv_vertex_get" "', argument " "1"" of type '" "struct _dwg_entity_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_MESH *)(argp1);
    result = (BITCODE_3DPOINT *) ((arg1)->subdiv_vertex);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MESH_num_vertex_set) {
  {
    struct _dwg_entity_MESH *arg1 = (struct _dwg_entity_MESH *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MESH_num_vertex_set(self,num_vertex);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MESH_num_vertex_set" "', argument " "1"" of type '" "struct _dwg_entity_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_MESH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MESH_num_vertex_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_vertex = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MESH_num_vertex_get) {
  {
    struct _dwg_entity_MESH *arg1 = (struct _dwg_entity_MESH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MESH_num_vertex_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MESH_num_vertex_get" "', argument " "1"" of type '" "struct _dwg_entity_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_MESH *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_vertex);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MESH_vertex_set) {
  {
    struct _dwg_entity_MESH *arg1 = (struct _dwg_entity_MESH *) 0 ;
    BITCODE_3DPOINT *arg2 = (BITCODE_3DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MESH_vertex_set(self,vertex);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MESH_vertex_set" "', argument " "1"" of type '" "struct _dwg_entity_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_MESH *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MESH_vertex_set" "', argument " "2"" of type '" "BITCODE_3DPOINT *""'"); 
    }
    arg2 = (BITCODE_3DPOINT *)(argp2);
    if (arg1) (arg1)->vertex = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MESH_vertex_get) {
  {
    struct _dwg_entity_MESH *arg1 = (struct _dwg_entity_MESH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MESH_vertex_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MESH_vertex_get" "', argument " "1"" of type '" "struct _dwg_entity_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_MESH *)(argp1);
    result = (BITCODE_3DPOINT *) ((arg1)->vertex);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MESH_num_faces_set) {
  {
    struct _dwg_entity_MESH *arg1 = (struct _dwg_entity_MESH *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MESH_num_faces_set(self,num_faces);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MESH_num_faces_set" "', argument " "1"" of type '" "struct _dwg_entity_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_MESH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MESH_num_faces_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_faces = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MESH_num_faces_get) {
  {
    struct _dwg_entity_MESH *arg1 = (struct _dwg_entity_MESH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MESH_num_faces_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MESH_num_faces_get" "', argument " "1"" of type '" "struct _dwg_entity_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_MESH *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_faces);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MESH_faces_set) {
  {
    struct _dwg_entity_MESH *arg1 = (struct _dwg_entity_MESH *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MESH_faces_set(self,faces);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MESH_faces_set" "', argument " "1"" of type '" "struct _dwg_entity_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_MESH *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MESH_faces_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->faces = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MESH_faces_get) {
  {
    struct _dwg_entity_MESH *arg1 = (struct _dwg_entity_MESH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MESH_faces_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MESH_faces_get" "', argument " "1"" of type '" "struct _dwg_entity_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_MESH *)(argp1);
    result = (BITCODE_BL *) ((arg1)->faces);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MESH_num_edges_set) {
  {
    struct _dwg_entity_MESH *arg1 = (struct _dwg_entity_MESH *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MESH_num_edges_set(self,num_edges);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MESH_num_edges_set" "', argument " "1"" of type '" "struct _dwg_entity_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_MESH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MESH_num_edges_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_edges = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MESH_num_edges_get) {
  {
    struct _dwg_entity_MESH *arg1 = (struct _dwg_entity_MESH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MESH_num_edges_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MESH_num_edges_get" "', argument " "1"" of type '" "struct _dwg_entity_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_MESH *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_edges);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MESH_edges_set) {
  {
    struct _dwg_entity_MESH *arg1 = (struct _dwg_entity_MESH *) 0 ;
    Dwg_MESH_edge *arg2 = (Dwg_MESH_edge *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MESH_edges_set(self,edges);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MESH_edges_set" "', argument " "1"" of type '" "struct _dwg_entity_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_MESH *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_MESH_edge, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MESH_edges_set" "', argument " "2"" of type '" "Dwg_MESH_edge *""'"); 
    }
    arg2 = (Dwg_MESH_edge *)(argp2);
    if (arg1) (arg1)->edges = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MESH_edges_get) {
  {
    struct _dwg_entity_MESH *arg1 = (struct _dwg_entity_MESH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_MESH_edge *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MESH_edges_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MESH_edges_get" "', argument " "1"" of type '" "struct _dwg_entity_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_MESH *)(argp1);
    result = (Dwg_MESH_edge *) ((arg1)->edges);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_MESH_edge, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MESH_num_crease_set) {
  {
    struct _dwg_entity_MESH *arg1 = (struct _dwg_entity_MESH *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MESH_num_crease_set(self,num_crease);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MESH_num_crease_set" "', argument " "1"" of type '" "struct _dwg_entity_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_MESH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MESH_num_crease_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_crease = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MESH_num_crease_get) {
  {
    struct _dwg_entity_MESH *arg1 = (struct _dwg_entity_MESH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MESH_num_crease_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MESH_num_crease_get" "', argument " "1"" of type '" "struct _dwg_entity_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_MESH *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_crease);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MESH_crease_set) {
  {
    struct _dwg_entity_MESH *arg1 = (struct _dwg_entity_MESH *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MESH_crease_set(self,crease);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MESH_crease_set" "', argument " "1"" of type '" "struct _dwg_entity_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_MESH *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_MESH_crease_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->crease = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MESH_crease_get) {
  {
    struct _dwg_entity_MESH *arg1 = (struct _dwg_entity_MESH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MESH_crease_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MESH_crease_get" "', argument " "1"" of type '" "struct _dwg_entity_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_MESH *)(argp1);
    result = (BITCODE_BD *) ((arg1)->crease);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MESH_unknown_b1_set) {
  {
    struct _dwg_entity_MESH *arg1 = (struct _dwg_entity_MESH *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MESH_unknown_b1_set(self,unknown_b1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MESH_unknown_b1_set" "', argument " "1"" of type '" "struct _dwg_entity_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_MESH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MESH_unknown_b1_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown_b1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MESH_unknown_b1_get) {
  {
    struct _dwg_entity_MESH *arg1 = (struct _dwg_entity_MESH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MESH_unknown_b1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MESH_unknown_b1_get" "', argument " "1"" of type '" "struct _dwg_entity_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_MESH *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown_b1);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MESH_unknown_b2_set) {
  {
    struct _dwg_entity_MESH *arg1 = (struct _dwg_entity_MESH *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_MESH_unknown_b2_set(self,unknown_b2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MESH_unknown_b2_set" "', argument " "1"" of type '" "struct _dwg_entity_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_MESH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_MESH_unknown_b2_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown_b2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_MESH_unknown_b2_get) {
  {
    struct _dwg_entity_MESH *arg1 = (struct _dwg_entity_MESH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_MESH_unknown_b2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MESH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_MESH_unknown_b2_get" "', argument " "1"" of type '" "struct _dwg_entity_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_MESH *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown_b2);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_MESH) {
  {
    int argvi = 0;
    struct _dwg_entity_MESH *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_MESH();");
    }
    result = (struct _dwg_entity_MESH *)calloc(1, sizeof(struct _dwg_entity_MESH));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_MESH, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_MESH) {
  {
    struct _dwg_entity_MESH *arg1 = (struct _dwg_entity_MESH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_MESH(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_MESH, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_MESH" "', argument " "1"" of type '" "struct _dwg_entity_MESH *""'"); 
    }
    arg1 = (struct _dwg_entity_MESH *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUN_parent_set) {
  {
    struct _dwg_object_SUN *arg1 = (struct _dwg_object_SUN *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUN_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUN_parent_set" "', argument " "1"" of type '" "struct _dwg_object_SUN *""'"); 
    }
    arg1 = (struct _dwg_object_SUN *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SUN_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUN_parent_get) {
  {
    struct _dwg_object_SUN *arg1 = (struct _dwg_object_SUN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUN_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUN_parent_get" "', argument " "1"" of type '" "struct _dwg_object_SUN *""'"); 
    }
    arg1 = (struct _dwg_object_SUN *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUN_class_version_set) {
  {
    struct _dwg_object_SUN *arg1 = (struct _dwg_object_SUN *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUN_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUN_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_SUN *""'"); 
    }
    arg1 = (struct _dwg_object_SUN *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SUN_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUN_class_version_get) {
  {
    struct _dwg_object_SUN *arg1 = (struct _dwg_object_SUN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUN_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUN_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_SUN *""'"); 
    }
    arg1 = (struct _dwg_object_SUN *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUN_is_on_set) {
  {
    struct _dwg_object_SUN *arg1 = (struct _dwg_object_SUN *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUN_is_on_set(self,is_on);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUN_is_on_set" "', argument " "1"" of type '" "struct _dwg_object_SUN *""'"); 
    }
    arg1 = (struct _dwg_object_SUN *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SUN_is_on_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_on = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUN_is_on_get) {
  {
    struct _dwg_object_SUN *arg1 = (struct _dwg_object_SUN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUN_is_on_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUN_is_on_get" "', argument " "1"" of type '" "struct _dwg_object_SUN *""'"); 
    }
    arg1 = (struct _dwg_object_SUN *)(argp1);
    result = (BITCODE_B) ((arg1)->is_on);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUN_color_set) {
  {
    struct _dwg_object_SUN *arg1 = (struct _dwg_object_SUN *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUN_color_set(self,color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUN_color_set" "', argument " "1"" of type '" "struct _dwg_object_SUN *""'"); 
    }
    arg1 = (struct _dwg_object_SUN *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SUN_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUN_color_get) {
  {
    struct _dwg_object_SUN *arg1 = (struct _dwg_object_SUN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUN_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUN_color_get" "', argument " "1"" of type '" "struct _dwg_object_SUN *""'"); 
    }
    arg1 = (struct _dwg_object_SUN *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUN_intensity_set) {
  {
    struct _dwg_object_SUN *arg1 = (struct _dwg_object_SUN *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUN_intensity_set(self,intensity);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUN_intensity_set" "', argument " "1"" of type '" "struct _dwg_object_SUN *""'"); 
    }
    arg1 = (struct _dwg_object_SUN *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SUN_intensity_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->intensity = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUN_intensity_get) {
  {
    struct _dwg_object_SUN *arg1 = (struct _dwg_object_SUN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUN_intensity_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUN_intensity_get" "', argument " "1"" of type '" "struct _dwg_object_SUN *""'"); 
    }
    arg1 = (struct _dwg_object_SUN *)(argp1);
    result = (BITCODE_BD) ((arg1)->intensity);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUN_has_shadow_set) {
  {
    struct _dwg_object_SUN *arg1 = (struct _dwg_object_SUN *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUN_has_shadow_set(self,has_shadow);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUN_has_shadow_set" "', argument " "1"" of type '" "struct _dwg_object_SUN *""'"); 
    }
    arg1 = (struct _dwg_object_SUN *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SUN_has_shadow_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_shadow = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUN_has_shadow_get) {
  {
    struct _dwg_object_SUN *arg1 = (struct _dwg_object_SUN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUN_has_shadow_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUN_has_shadow_get" "', argument " "1"" of type '" "struct _dwg_object_SUN *""'"); 
    }
    arg1 = (struct _dwg_object_SUN *)(argp1);
    result = (BITCODE_B) ((arg1)->has_shadow);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUN_julian_day_set) {
  {
    struct _dwg_object_SUN *arg1 = (struct _dwg_object_SUN *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUN_julian_day_set(self,julian_day);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUN_julian_day_set" "', argument " "1"" of type '" "struct _dwg_object_SUN *""'"); 
    }
    arg1 = (struct _dwg_object_SUN *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SUN_julian_day_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->julian_day = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUN_julian_day_get) {
  {
    struct _dwg_object_SUN *arg1 = (struct _dwg_object_SUN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUN_julian_day_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUN_julian_day_get" "', argument " "1"" of type '" "struct _dwg_object_SUN *""'"); 
    }
    arg1 = (struct _dwg_object_SUN *)(argp1);
    result = (BITCODE_BL) ((arg1)->julian_day);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUN_msecs_set) {
  {
    struct _dwg_object_SUN *arg1 = (struct _dwg_object_SUN *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUN_msecs_set(self,msecs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUN_msecs_set" "', argument " "1"" of type '" "struct _dwg_object_SUN *""'"); 
    }
    arg1 = (struct _dwg_object_SUN *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SUN_msecs_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->msecs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUN_msecs_get) {
  {
    struct _dwg_object_SUN *arg1 = (struct _dwg_object_SUN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUN_msecs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUN_msecs_get" "', argument " "1"" of type '" "struct _dwg_object_SUN *""'"); 
    }
    arg1 = (struct _dwg_object_SUN *)(argp1);
    result = (BITCODE_BL) ((arg1)->msecs);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUN_is_dst_set) {
  {
    struct _dwg_object_SUN *arg1 = (struct _dwg_object_SUN *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUN_is_dst_set(self,is_dst);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUN_is_dst_set" "', argument " "1"" of type '" "struct _dwg_object_SUN *""'"); 
    }
    arg1 = (struct _dwg_object_SUN *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SUN_is_dst_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_dst = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUN_is_dst_get) {
  {
    struct _dwg_object_SUN *arg1 = (struct _dwg_object_SUN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUN_is_dst_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUN_is_dst_get" "', argument " "1"" of type '" "struct _dwg_object_SUN *""'"); 
    }
    arg1 = (struct _dwg_object_SUN *)(argp1);
    result = (BITCODE_B) ((arg1)->is_dst);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUN_shadow_type_set) {
  {
    struct _dwg_object_SUN *arg1 = (struct _dwg_object_SUN *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUN_shadow_type_set(self,shadow_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUN_shadow_type_set" "', argument " "1"" of type '" "struct _dwg_object_SUN *""'"); 
    }
    arg1 = (struct _dwg_object_SUN *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SUN_shadow_type_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->shadow_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUN_shadow_type_get) {
  {
    struct _dwg_object_SUN *arg1 = (struct _dwg_object_SUN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUN_shadow_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUN_shadow_type_get" "', argument " "1"" of type '" "struct _dwg_object_SUN *""'"); 
    }
    arg1 = (struct _dwg_object_SUN *)(argp1);
    result = (BITCODE_BL) ((arg1)->shadow_type);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUN_shadow_mapsize_set) {
  {
    struct _dwg_object_SUN *arg1 = (struct _dwg_object_SUN *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUN_shadow_mapsize_set(self,shadow_mapsize);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUN_shadow_mapsize_set" "', argument " "1"" of type '" "struct _dwg_object_SUN *""'"); 
    }
    arg1 = (struct _dwg_object_SUN *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SUN_shadow_mapsize_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->shadow_mapsize = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUN_shadow_mapsize_get) {
  {
    struct _dwg_object_SUN *arg1 = (struct _dwg_object_SUN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUN_shadow_mapsize_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUN_shadow_mapsize_get" "', argument " "1"" of type '" "struct _dwg_object_SUN *""'"); 
    }
    arg1 = (struct _dwg_object_SUN *)(argp1);
    result = (BITCODE_BS) ((arg1)->shadow_mapsize);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUN_shadow_softness_set) {
  {
    struct _dwg_object_SUN *arg1 = (struct _dwg_object_SUN *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUN_shadow_softness_set(self,shadow_softness);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUN_shadow_softness_set" "', argument " "1"" of type '" "struct _dwg_object_SUN *""'"); 
    }
    arg1 = (struct _dwg_object_SUN *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SUN_shadow_softness_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->shadow_softness = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUN_shadow_softness_get) {
  {
    struct _dwg_object_SUN *arg1 = (struct _dwg_object_SUN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUN_shadow_softness_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUN, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUN_shadow_softness_get" "', argument " "1"" of type '" "struct _dwg_object_SUN *""'"); 
    }
    arg1 = (struct _dwg_object_SUN *)(argp1);
    result = (BITCODE_RC) ((arg1)->shadow_softness);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_SUN) {
  {
    int argvi = 0;
    struct _dwg_object_SUN *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_SUN();");
    }
    result = (struct _dwg_object_SUN *)calloc(1, sizeof(struct _dwg_object_SUN));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_SUN, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_SUN) {
  {
    struct _dwg_object_SUN *arg1 = (struct _dwg_object_SUN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_SUN(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUN, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_SUN" "', argument " "1"" of type '" "struct _dwg_object_SUN *""'"); 
    }
    arg1 = (struct _dwg_object_SUN *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SUNSTUDY_Dates_julian_day_set) {
  {
    struct _dwg_SUNSTUDY_Dates *arg1 = (struct _dwg_SUNSTUDY_Dates *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SUNSTUDY_Dates_julian_day_set(self,julian_day);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SUNSTUDY_Dates, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SUNSTUDY_Dates_julian_day_set" "', argument " "1"" of type '" "struct _dwg_SUNSTUDY_Dates *""'"); 
    }
    arg1 = (struct _dwg_SUNSTUDY_Dates *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SUNSTUDY_Dates_julian_day_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->julian_day = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SUNSTUDY_Dates_julian_day_get) {
  {
    struct _dwg_SUNSTUDY_Dates *arg1 = (struct _dwg_SUNSTUDY_Dates *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SUNSTUDY_Dates_julian_day_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SUNSTUDY_Dates, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SUNSTUDY_Dates_julian_day_get" "', argument " "1"" of type '" "struct _dwg_SUNSTUDY_Dates *""'"); 
    }
    arg1 = (struct _dwg_SUNSTUDY_Dates *)(argp1);
    result = (BITCODE_BL) ((arg1)->julian_day);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SUNSTUDY_Dates_msecs_set) {
  {
    struct _dwg_SUNSTUDY_Dates *arg1 = (struct _dwg_SUNSTUDY_Dates *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SUNSTUDY_Dates_msecs_set(self,msecs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SUNSTUDY_Dates, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SUNSTUDY_Dates_msecs_set" "', argument " "1"" of type '" "struct _dwg_SUNSTUDY_Dates *""'"); 
    }
    arg1 = (struct _dwg_SUNSTUDY_Dates *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SUNSTUDY_Dates_msecs_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->msecs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SUNSTUDY_Dates_msecs_get) {
  {
    struct _dwg_SUNSTUDY_Dates *arg1 = (struct _dwg_SUNSTUDY_Dates *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SUNSTUDY_Dates_msecs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SUNSTUDY_Dates, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SUNSTUDY_Dates_msecs_get" "', argument " "1"" of type '" "struct _dwg_SUNSTUDY_Dates *""'"); 
    }
    arg1 = (struct _dwg_SUNSTUDY_Dates *)(argp1);
    result = (BITCODE_BL) ((arg1)->msecs);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_SUNSTUDY_Dates) {
  {
    int argvi = 0;
    struct _dwg_SUNSTUDY_Dates *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_SUNSTUDY_Dates();");
    }
    result = (struct _dwg_SUNSTUDY_Dates *)calloc(1, sizeof(struct _dwg_SUNSTUDY_Dates));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_SUNSTUDY_Dates, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_SUNSTUDY_Dates) {
  {
    struct _dwg_SUNSTUDY_Dates *arg1 = (struct _dwg_SUNSTUDY_Dates *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_SUNSTUDY_Dates(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SUNSTUDY_Dates, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_SUNSTUDY_Dates" "', argument " "1"" of type '" "struct _dwg_SUNSTUDY_Dates *""'"); 
    }
    arg1 = (struct _dwg_SUNSTUDY_Dates *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_parent_set) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_parent_set" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SUNSTUDY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_parent_get) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_parent_get" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_class_version_set) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SUNSTUDY_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_class_version_get) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_setup_name_set) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_setup_name_set(self,setup_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_setup_name_set" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SUNSTUDY_setup_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->setup_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->setup_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->setup_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_setup_name_get) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_setup_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_setup_name_get" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    result = (BITCODE_TV) ((arg1)->setup_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_description_set) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_description_set(self,description);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_description_set" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SUNSTUDY_description_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->description);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->description = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->description = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_description_get) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_description_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_description_get" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    result = (BITCODE_TV) ((arg1)->description);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_output_type_set) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_output_type_set(self,output_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_output_type_set" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SUNSTUDY_output_type_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->output_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_output_type_get) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_output_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_output_type_get" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    result = (BITCODE_BL) ((arg1)->output_type);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_sheet_set_name_set) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_sheet_set_name_set(self,sheet_set_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_sheet_set_name_set" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SUNSTUDY_sheet_set_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->sheet_set_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->sheet_set_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->sheet_set_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_sheet_set_name_get) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_sheet_set_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_sheet_set_name_get" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    result = (BITCODE_TV) ((arg1)->sheet_set_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_use_subset_set) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_use_subset_set(self,use_subset);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_use_subset_set" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SUNSTUDY_use_subset_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->use_subset = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_use_subset_get) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_use_subset_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_use_subset_get" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    result = (BITCODE_B) ((arg1)->use_subset);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_sheet_subset_name_set) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_sheet_subset_name_set(self,sheet_subset_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_sheet_subset_name_set" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SUNSTUDY_sheet_subset_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->sheet_subset_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->sheet_subset_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->sheet_subset_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_sheet_subset_name_get) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_sheet_subset_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_sheet_subset_name_get" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    result = (BITCODE_TV) ((arg1)->sheet_subset_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_select_dates_from_calendar_set) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_select_dates_from_calendar_set(self,select_dates_from_calendar);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_select_dates_from_calendar_set" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SUNSTUDY_select_dates_from_calendar_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->select_dates_from_calendar = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_select_dates_from_calendar_get) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_select_dates_from_calendar_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_select_dates_from_calendar_get" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    result = (BITCODE_B) ((arg1)->select_dates_from_calendar);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_num_dates_set) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_num_dates_set(self,num_dates);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_num_dates_set" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SUNSTUDY_num_dates_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_dates = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_num_dates_get) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_num_dates_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_num_dates_get" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_dates);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_dates_set) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    Dwg_SUNSTUDY_Dates *arg2 = (Dwg_SUNSTUDY_Dates *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_dates_set(self,dates);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_dates_set" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_SUNSTUDY_Dates, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SUNSTUDY_dates_set" "', argument " "2"" of type '" "Dwg_SUNSTUDY_Dates *""'"); 
    }
    arg2 = (Dwg_SUNSTUDY_Dates *)(argp2);
    if (arg1) (arg1)->dates = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_dates_get) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_SUNSTUDY_Dates *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_dates_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_dates_get" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    result = (Dwg_SUNSTUDY_Dates *) ((arg1)->dates);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_SUNSTUDY_Dates, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_select_range_of_dates_set) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_select_range_of_dates_set(self,select_range_of_dates);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_select_range_of_dates_set" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SUNSTUDY_select_range_of_dates_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->select_range_of_dates = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_select_range_of_dates_get) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_select_range_of_dates_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_select_range_of_dates_get" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    result = (BITCODE_B) ((arg1)->select_range_of_dates);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_start_time_set) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_start_time_set(self,start_time);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_start_time_set" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SUNSTUDY_start_time_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->start_time = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_start_time_get) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_start_time_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_start_time_get" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    result = (BITCODE_BL) ((arg1)->start_time);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_end_time_set) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_end_time_set(self,end_time);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_end_time_set" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SUNSTUDY_end_time_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->end_time = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_end_time_get) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_end_time_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_end_time_get" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    result = (BITCODE_BL) ((arg1)->end_time);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_interval_set) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_interval_set(self,interval);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_interval_set" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SUNSTUDY_interval_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->interval = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_interval_get) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_interval_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_interval_get" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    result = (BITCODE_BL) ((arg1)->interval);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_num_hours_set) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_num_hours_set(self,num_hours);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_num_hours_set" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SUNSTUDY_num_hours_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_hours = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_num_hours_get) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_num_hours_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_num_hours_get" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_hours);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_hours_set) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    BITCODE_B *arg2 = (BITCODE_B *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_hours_set(self,hours);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_hours_set" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SUNSTUDY_hours_set" "', argument " "2"" of type '" "BITCODE_B *""'"); 
    }
    arg2 = (BITCODE_B *)(argp2);
    if (arg1) (arg1)->hours = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_hours_get) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_hours_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_hours_get" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    result = (BITCODE_B *) ((arg1)->hours);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_shade_plot_type_set) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_shade_plot_type_set(self,shade_plot_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_shade_plot_type_set" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SUNSTUDY_shade_plot_type_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->shade_plot_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_shade_plot_type_get) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_shade_plot_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_shade_plot_type_get" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    result = (BITCODE_BL) ((arg1)->shade_plot_type);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_numvports_set) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_numvports_set(self,numvports);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_numvports_set" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SUNSTUDY_numvports_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->numvports = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_numvports_get) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_numvports_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_numvports_get" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    result = (BITCODE_BL) ((arg1)->numvports);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_numrows_set) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_numrows_set(self,numrows);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_numrows_set" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SUNSTUDY_numrows_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->numrows = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_numrows_get) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_numrows_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_numrows_get" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    result = (BITCODE_BL) ((arg1)->numrows);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_numcols_set) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_numcols_set(self,numcols);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_numcols_set" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SUNSTUDY_numcols_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->numcols = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_numcols_get) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_numcols_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_numcols_get" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    result = (BITCODE_BL) ((arg1)->numcols);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_spacing_set) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_spacing_set(self,spacing);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_spacing_set" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SUNSTUDY_spacing_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->spacing = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_spacing_get) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_spacing_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_spacing_get" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    result = (BITCODE_BD) ((arg1)->spacing);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_lock_viewports_set) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_lock_viewports_set(self,lock_viewports);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_lock_viewports_set" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SUNSTUDY_lock_viewports_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->lock_viewports = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_lock_viewports_get) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_lock_viewports_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_lock_viewports_get" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    result = (BITCODE_B) ((arg1)->lock_viewports);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_label_viewports_set) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_label_viewports_set(self,label_viewports);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_label_viewports_set" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SUNSTUDY_label_viewports_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->label_viewports = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_label_viewports_get) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_label_viewports_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_label_viewports_get" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    result = (BITCODE_B) ((arg1)->label_viewports);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_page_setup_wizard_set) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_page_setup_wizard_set(self,page_setup_wizard);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_page_setup_wizard_set" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SUNSTUDY_page_setup_wizard_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->page_setup_wizard = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_page_setup_wizard_get) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_page_setup_wizard_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_page_setup_wizard_get" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    result = (BITCODE_H) ((arg1)->page_setup_wizard);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_view_set) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_view_set(self,view);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_view_set" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SUNSTUDY_view_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->view = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_view_get) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_view_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_view_get" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    result = (BITCODE_H) ((arg1)->view);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_visualstyle_set) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_visualstyle_set(self,visualstyle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_visualstyle_set" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SUNSTUDY_visualstyle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->visualstyle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_visualstyle_get) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_visualstyle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_visualstyle_get" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    result = (BITCODE_H) ((arg1)->visualstyle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_text_style_set) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_text_style_set(self,text_style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_text_style_set" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SUNSTUDY_text_style_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->text_style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SUNSTUDY_text_style_get) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SUNSTUDY_text_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SUNSTUDY_text_style_get" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    result = (BITCODE_H) ((arg1)->text_style);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_SUNSTUDY) {
  {
    int argvi = 0;
    struct _dwg_object_SUNSTUDY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_SUNSTUDY();");
    }
    result = (struct _dwg_object_SUNSTUDY *)calloc(1, sizeof(struct _dwg_object_SUNSTUDY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_SUNSTUDY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_SUNSTUDY) {
  {
    struct _dwg_object_SUNSTUDY *arg1 = (struct _dwg_object_SUNSTUDY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_SUNSTUDY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SUNSTUDY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_SUNSTUDY" "', argument " "1"" of type '" "struct _dwg_object_SUNSTUDY *""'"); 
    }
    arg1 = (struct _dwg_object_SUNSTUDY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DATATABLE_row_parent_set) {
  {
    struct _dwg_DATATABLE_row *arg1 = (struct _dwg_DATATABLE_row *) 0 ;
    struct _dwg_DATATABLE_column *arg2 = (struct _dwg_DATATABLE_column *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DATATABLE_row_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DATATABLE_row, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DATATABLE_row_parent_set" "', argument " "1"" of type '" "struct _dwg_DATATABLE_row *""'"); 
    }
    arg1 = (struct _dwg_DATATABLE_row *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_DATATABLE_column, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_DATATABLE_row_parent_set" "', argument " "2"" of type '" "struct _dwg_DATATABLE_column *""'"); 
    }
    arg2 = (struct _dwg_DATATABLE_column *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DATATABLE_row_parent_get) {
  {
    struct _dwg_DATATABLE_row *arg1 = (struct _dwg_DATATABLE_row *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_DATATABLE_column *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DATATABLE_row_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DATATABLE_row, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DATATABLE_row_parent_get" "', argument " "1"" of type '" "struct _dwg_DATATABLE_row *""'"); 
    }
    arg1 = (struct _dwg_DATATABLE_row *)(argp1);
    result = (struct _dwg_DATATABLE_column *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_DATATABLE_column, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DATATABLE_row_value_set) {
  {
    struct _dwg_DATATABLE_row *arg1 = (struct _dwg_DATATABLE_row *) 0 ;
    Dwg_TABLE_value *arg2 = (Dwg_TABLE_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DATATABLE_row_value_set(self,value);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DATATABLE_row, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DATATABLE_row_value_set" "', argument " "1"" of type '" "struct _dwg_DATATABLE_row *""'"); 
    }
    arg1 = (struct _dwg_DATATABLE_row *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_TABLE_value, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_DATATABLE_row_value_set" "', argument " "2"" of type '" "Dwg_TABLE_value *""'"); 
    }
    arg2 = (Dwg_TABLE_value *)(argp2);
    if (arg1) (arg1)->value = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DATATABLE_row_value_get) {
  {
    struct _dwg_DATATABLE_row *arg1 = (struct _dwg_DATATABLE_row *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_TABLE_value *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DATATABLE_row_value_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DATATABLE_row, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DATATABLE_row_value_get" "', argument " "1"" of type '" "struct _dwg_DATATABLE_row *""'"); 
    }
    arg1 = (struct _dwg_DATATABLE_row *)(argp1);
    result = (Dwg_TABLE_value *)& ((arg1)->value);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_TABLE_value, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_DATATABLE_row) {
  {
    int argvi = 0;
    struct _dwg_DATATABLE_row *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_DATATABLE_row();");
    }
    result = (struct _dwg_DATATABLE_row *)calloc(1, sizeof(struct _dwg_DATATABLE_row));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_DATATABLE_row, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_DATATABLE_row) {
  {
    struct _dwg_DATATABLE_row *arg1 = (struct _dwg_DATATABLE_row *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_DATATABLE_row(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DATATABLE_row, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_DATATABLE_row" "', argument " "1"" of type '" "struct _dwg_DATATABLE_row *""'"); 
    }
    arg1 = (struct _dwg_DATATABLE_row *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DATATABLE_column_parent_set) {
  {
    struct _dwg_DATATABLE_column *arg1 = (struct _dwg_DATATABLE_column *) 0 ;
    struct _dwg_object_DATATABLE *arg2 = (struct _dwg_object_DATATABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DATATABLE_column_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DATATABLE_column, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DATATABLE_column_parent_set" "', argument " "1"" of type '" "struct _dwg_DATATABLE_column *""'"); 
    }
    arg1 = (struct _dwg_DATATABLE_column *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_DATATABLE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_DATATABLE_column_parent_set" "', argument " "2"" of type '" "struct _dwg_object_DATATABLE *""'"); 
    }
    arg2 = (struct _dwg_object_DATATABLE *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DATATABLE_column_parent_get) {
  {
    struct _dwg_DATATABLE_column *arg1 = (struct _dwg_DATATABLE_column *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_DATATABLE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DATATABLE_column_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DATATABLE_column, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DATATABLE_column_parent_get" "', argument " "1"" of type '" "struct _dwg_DATATABLE_column *""'"); 
    }
    arg1 = (struct _dwg_DATATABLE_column *)(argp1);
    result = (struct _dwg_object_DATATABLE *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DATATABLE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DATATABLE_column_type_set) {
  {
    struct _dwg_DATATABLE_column *arg1 = (struct _dwg_DATATABLE_column *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DATATABLE_column_type_set(self,type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DATATABLE_column, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DATATABLE_column_type_set" "', argument " "1"" of type '" "struct _dwg_DATATABLE_column *""'"); 
    }
    arg1 = (struct _dwg_DATATABLE_column *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_DATATABLE_column_type_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DATATABLE_column_type_get) {
  {
    struct _dwg_DATATABLE_column *arg1 = (struct _dwg_DATATABLE_column *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DATATABLE_column_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DATATABLE_column, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DATATABLE_column_type_get" "', argument " "1"" of type '" "struct _dwg_DATATABLE_column *""'"); 
    }
    arg1 = (struct _dwg_DATATABLE_column *)(argp1);
    result = (BITCODE_BL) ((arg1)->type);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DATATABLE_column_text_set) {
  {
    struct _dwg_DATATABLE_column *arg1 = (struct _dwg_DATATABLE_column *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DATATABLE_column_text_set(self,text);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DATATABLE_column, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DATATABLE_column_text_set" "', argument " "1"" of type '" "struct _dwg_DATATABLE_column *""'"); 
    }
    arg1 = (struct _dwg_DATATABLE_column *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_DATATABLE_column_text_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->text);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->text = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->text = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DATATABLE_column_text_get) {
  {
    struct _dwg_DATATABLE_column *arg1 = (struct _dwg_DATATABLE_column *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DATATABLE_column_text_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DATATABLE_column, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DATATABLE_column_text_get" "', argument " "1"" of type '" "struct _dwg_DATATABLE_column *""'"); 
    }
    arg1 = (struct _dwg_DATATABLE_column *)(argp1);
    result = (BITCODE_TV) ((arg1)->text);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DATATABLE_column_rows_set) {
  {
    struct _dwg_DATATABLE_column *arg1 = (struct _dwg_DATATABLE_column *) 0 ;
    Dwg_DATATABLE_row *arg2 = (Dwg_DATATABLE_row *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DATATABLE_column_rows_set(self,rows);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DATATABLE_column, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DATATABLE_column_rows_set" "', argument " "1"" of type '" "struct _dwg_DATATABLE_column *""'"); 
    }
    arg1 = (struct _dwg_DATATABLE_column *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_DATATABLE_row, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_DATATABLE_column_rows_set" "', argument " "2"" of type '" "Dwg_DATATABLE_row *""'"); 
    }
    arg2 = (Dwg_DATATABLE_row *)(argp2);
    if (arg1) (arg1)->rows = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DATATABLE_column_rows_get) {
  {
    struct _dwg_DATATABLE_column *arg1 = (struct _dwg_DATATABLE_column *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_DATATABLE_row *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DATATABLE_column_rows_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DATATABLE_column, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DATATABLE_column_rows_get" "', argument " "1"" of type '" "struct _dwg_DATATABLE_column *""'"); 
    }
    arg1 = (struct _dwg_DATATABLE_column *)(argp1);
    result = (Dwg_DATATABLE_row *) ((arg1)->rows);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_DATATABLE_row, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_DATATABLE_column) {
  {
    int argvi = 0;
    struct _dwg_DATATABLE_column *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_DATATABLE_column();");
    }
    result = (struct _dwg_DATATABLE_column *)calloc(1, sizeof(struct _dwg_DATATABLE_column));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_DATATABLE_column, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_DATATABLE_column) {
  {
    struct _dwg_DATATABLE_column *arg1 = (struct _dwg_DATATABLE_column *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_DATATABLE_column(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DATATABLE_column, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_DATATABLE_column" "', argument " "1"" of type '" "struct _dwg_DATATABLE_column *""'"); 
    }
    arg1 = (struct _dwg_DATATABLE_column *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATATABLE_parent_set) {
  {
    struct _dwg_object_DATATABLE *arg1 = (struct _dwg_object_DATATABLE *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DATATABLE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATATABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATATABLE_parent_set" "', argument " "1"" of type '" "struct _dwg_object_DATATABLE *""'"); 
    }
    arg1 = (struct _dwg_object_DATATABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DATATABLE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATATABLE_parent_get) {
  {
    struct _dwg_object_DATATABLE *arg1 = (struct _dwg_object_DATATABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DATATABLE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATATABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATATABLE_parent_get" "', argument " "1"" of type '" "struct _dwg_object_DATATABLE *""'"); 
    }
    arg1 = (struct _dwg_object_DATATABLE *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATATABLE_flags_set) {
  {
    struct _dwg_object_DATATABLE *arg1 = (struct _dwg_object_DATATABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DATATABLE_flags_set(self,flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATATABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATATABLE_flags_set" "', argument " "1"" of type '" "struct _dwg_object_DATATABLE *""'"); 
    }
    arg1 = (struct _dwg_object_DATATABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DATATABLE_flags_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATATABLE_flags_get) {
  {
    struct _dwg_object_DATATABLE *arg1 = (struct _dwg_object_DATATABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DATATABLE_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATATABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATATABLE_flags_get" "', argument " "1"" of type '" "struct _dwg_object_DATATABLE *""'"); 
    }
    arg1 = (struct _dwg_object_DATATABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->flags);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATATABLE_num_cols_set) {
  {
    struct _dwg_object_DATATABLE *arg1 = (struct _dwg_object_DATATABLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DATATABLE_num_cols_set(self,num_cols);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATATABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATATABLE_num_cols_set" "', argument " "1"" of type '" "struct _dwg_object_DATATABLE *""'"); 
    }
    arg1 = (struct _dwg_object_DATATABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DATATABLE_num_cols_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_cols = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATATABLE_num_cols_get) {
  {
    struct _dwg_object_DATATABLE *arg1 = (struct _dwg_object_DATATABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DATATABLE_num_cols_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATATABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATATABLE_num_cols_get" "', argument " "1"" of type '" "struct _dwg_object_DATATABLE *""'"); 
    }
    arg1 = (struct _dwg_object_DATATABLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_cols);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATATABLE_num_rows_set) {
  {
    struct _dwg_object_DATATABLE *arg1 = (struct _dwg_object_DATATABLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DATATABLE_num_rows_set(self,num_rows);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATATABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATATABLE_num_rows_set" "', argument " "1"" of type '" "struct _dwg_object_DATATABLE *""'"); 
    }
    arg1 = (struct _dwg_object_DATATABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DATATABLE_num_rows_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_rows = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATATABLE_num_rows_get) {
  {
    struct _dwg_object_DATATABLE *arg1 = (struct _dwg_object_DATATABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DATATABLE_num_rows_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATATABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATATABLE_num_rows_get" "', argument " "1"" of type '" "struct _dwg_object_DATATABLE *""'"); 
    }
    arg1 = (struct _dwg_object_DATATABLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_rows);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATATABLE_table_name_set) {
  {
    struct _dwg_object_DATATABLE *arg1 = (struct _dwg_object_DATATABLE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DATATABLE_table_name_set(self,table_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATATABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATATABLE_table_name_set" "', argument " "1"" of type '" "struct _dwg_object_DATATABLE *""'"); 
    }
    arg1 = (struct _dwg_object_DATATABLE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DATATABLE_table_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->table_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->table_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->table_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATATABLE_table_name_get) {
  {
    struct _dwg_object_DATATABLE *arg1 = (struct _dwg_object_DATATABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DATATABLE_table_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATATABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATATABLE_table_name_get" "', argument " "1"" of type '" "struct _dwg_object_DATATABLE *""'"); 
    }
    arg1 = (struct _dwg_object_DATATABLE *)(argp1);
    result = (BITCODE_TV) ((arg1)->table_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATATABLE_cols_set) {
  {
    struct _dwg_object_DATATABLE *arg1 = (struct _dwg_object_DATATABLE *) 0 ;
    Dwg_DATATABLE_column *arg2 = (Dwg_DATATABLE_column *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DATATABLE_cols_set(self,cols);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATATABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATATABLE_cols_set" "', argument " "1"" of type '" "struct _dwg_object_DATATABLE *""'"); 
    }
    arg1 = (struct _dwg_object_DATATABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_DATATABLE_column, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DATATABLE_cols_set" "', argument " "2"" of type '" "Dwg_DATATABLE_column *""'"); 
    }
    arg2 = (Dwg_DATATABLE_column *)(argp2);
    if (arg1) (arg1)->cols = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATATABLE_cols_get) {
  {
    struct _dwg_object_DATATABLE *arg1 = (struct _dwg_object_DATATABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_DATATABLE_column *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DATATABLE_cols_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATATABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATATABLE_cols_get" "', argument " "1"" of type '" "struct _dwg_object_DATATABLE *""'"); 
    }
    arg1 = (struct _dwg_object_DATATABLE *)(argp1);
    result = (Dwg_DATATABLE_column *) ((arg1)->cols);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_DATATABLE_column, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_DATATABLE) {
  {
    int argvi = 0;
    struct _dwg_object_DATATABLE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_DATATABLE();");
    }
    result = (struct _dwg_object_DATATABLE *)calloc(1, sizeof(struct _dwg_object_DATATABLE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DATATABLE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_DATATABLE) {
  {
    struct _dwg_object_DATATABLE *arg1 = (struct _dwg_object_DATATABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_DATATABLE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATATABLE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_DATATABLE" "', argument " "1"" of type '" "struct _dwg_object_DATATABLE *""'"); 
    }
    arg1 = (struct _dwg_object_DATATABLE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DATALINK_customdata_parent_set) {
  {
    struct _dwg_DATALINK_customdata *arg1 = (struct _dwg_DATALINK_customdata *) 0 ;
    struct _dwg_object_DATALINK *arg2 = (struct _dwg_object_DATALINK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DATALINK_customdata_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DATALINK_customdata, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DATALINK_customdata_parent_set" "', argument " "1"" of type '" "struct _dwg_DATALINK_customdata *""'"); 
    }
    arg1 = (struct _dwg_DATALINK_customdata *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_DATALINK, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_DATALINK_customdata_parent_set" "', argument " "2"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg2 = (struct _dwg_object_DATALINK *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DATALINK_customdata_parent_get) {
  {
    struct _dwg_DATALINK_customdata *arg1 = (struct _dwg_DATALINK_customdata *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_DATALINK *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DATALINK_customdata_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DATALINK_customdata, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DATALINK_customdata_parent_get" "', argument " "1"" of type '" "struct _dwg_DATALINK_customdata *""'"); 
    }
    arg1 = (struct _dwg_DATALINK_customdata *)(argp1);
    result = (struct _dwg_object_DATALINK *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DATALINK, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DATALINK_customdata_target_set) {
  {
    struct _dwg_DATALINK_customdata *arg1 = (struct _dwg_DATALINK_customdata *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DATALINK_customdata_target_set(self,target);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DATALINK_customdata, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DATALINK_customdata_target_set" "', argument " "1"" of type '" "struct _dwg_DATALINK_customdata *""'"); 
    }
    arg1 = (struct _dwg_DATALINK_customdata *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_DATALINK_customdata_target_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->target = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DATALINK_customdata_target_get) {
  {
    struct _dwg_DATALINK_customdata *arg1 = (struct _dwg_DATALINK_customdata *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DATALINK_customdata_target_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DATALINK_customdata, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DATALINK_customdata_target_get" "', argument " "1"" of type '" "struct _dwg_DATALINK_customdata *""'"); 
    }
    arg1 = (struct _dwg_DATALINK_customdata *)(argp1);
    result = (BITCODE_H) ((arg1)->target);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DATALINK_customdata_text_set) {
  {
    struct _dwg_DATALINK_customdata *arg1 = (struct _dwg_DATALINK_customdata *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DATALINK_customdata_text_set(self,text);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DATALINK_customdata, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DATALINK_customdata_text_set" "', argument " "1"" of type '" "struct _dwg_DATALINK_customdata *""'"); 
    }
    arg1 = (struct _dwg_DATALINK_customdata *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_DATALINK_customdata_text_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->text);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->text = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->text = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DATALINK_customdata_text_get) {
  {
    struct _dwg_DATALINK_customdata *arg1 = (struct _dwg_DATALINK_customdata *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DATALINK_customdata_text_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DATALINK_customdata, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DATALINK_customdata_text_get" "', argument " "1"" of type '" "struct _dwg_DATALINK_customdata *""'"); 
    }
    arg1 = (struct _dwg_DATALINK_customdata *)(argp1);
    result = (BITCODE_TV) ((arg1)->text);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_DATALINK_customdata) {
  {
    int argvi = 0;
    struct _dwg_DATALINK_customdata *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_DATALINK_customdata();");
    }
    result = (struct _dwg_DATALINK_customdata *)calloc(1, sizeof(struct _dwg_DATALINK_customdata));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_DATALINK_customdata, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_DATALINK_customdata) {
  {
    struct _dwg_DATALINK_customdata *arg1 = (struct _dwg_DATALINK_customdata *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_DATALINK_customdata(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DATALINK_customdata, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_DATALINK_customdata" "', argument " "1"" of type '" "struct _dwg_DATALINK_customdata *""'"); 
    }
    arg1 = (struct _dwg_DATALINK_customdata *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_parent_set) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_parent_set" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DATALINK_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_parent_get) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_parent_get" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_class_version_set) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DATALINK_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_class_version_get) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_data_adapter_set) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_data_adapter_set(self,data_adapter);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_data_adapter_set" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DATALINK_data_adapter_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->data_adapter);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->data_adapter = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->data_adapter = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_data_adapter_get) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_data_adapter_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_data_adapter_get" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    result = (BITCODE_TV) ((arg1)->data_adapter);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_description_set) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_description_set(self,description);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_description_set" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DATALINK_description_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->description);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->description = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->description = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_description_get) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_description_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_description_get" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    result = (BITCODE_TV) ((arg1)->description);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_tooltip_set) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_tooltip_set(self,tooltip);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_tooltip_set" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DATALINK_tooltip_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->tooltip);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->tooltip = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->tooltip = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_tooltip_get) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_tooltip_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_tooltip_get" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    result = (BITCODE_TV) ((arg1)->tooltip);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_connection_string_set) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_connection_string_set(self,connection_string);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_connection_string_set" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DATALINK_connection_string_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->connection_string);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->connection_string = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->connection_string = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_connection_string_get) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_connection_string_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_connection_string_get" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    result = (BITCODE_TV) ((arg1)->connection_string);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_option_set) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_option_set(self,option);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_option_set" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DATALINK_option_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->option = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_option_get) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_option_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_option_get" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    result = (BITCODE_BL) ((arg1)->option);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_update_option_set) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_update_option_set(self,update_option);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_update_option_set" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DATALINK_update_option_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->update_option = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_update_option_get) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_update_option_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_update_option_get" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    result = (BITCODE_BL) ((arg1)->update_option);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_bl92_set) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_bl92_set(self,bl92);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_bl92_set" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DATALINK_bl92_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bl92 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_bl92_get) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_bl92_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_bl92_get" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    result = (BITCODE_BL) ((arg1)->bl92);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_year_set) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_year_set(self,year);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_year_set" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DATALINK_year_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->year = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_year_get) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_year_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_year_get" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    result = (BITCODE_BS) ((arg1)->year);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_month_set) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_month_set(self,month);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_month_set" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DATALINK_month_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->month = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_month_get) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_month_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_month_get" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    result = (BITCODE_BS) ((arg1)->month);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_day_set) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_day_set(self,day);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_day_set" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DATALINK_day_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->day = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_day_get) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_day_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_day_get" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    result = (BITCODE_BS) ((arg1)->day);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_hour_set) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_hour_set(self,hour);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_hour_set" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DATALINK_hour_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->hour = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_hour_get) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_hour_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_hour_get" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    result = (BITCODE_BS) ((arg1)->hour);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_minute_set) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_minute_set(self,minute);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_minute_set" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DATALINK_minute_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->minute = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_minute_get) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_minute_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_minute_get" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    result = (BITCODE_BS) ((arg1)->minute);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_seconds_set) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_seconds_set(self,seconds);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_seconds_set" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DATALINK_seconds_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->seconds = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_seconds_get) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_seconds_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_seconds_get" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    result = (BITCODE_BS) ((arg1)->seconds);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_msec_set) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_msec_set(self,msec);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_msec_set" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DATALINK_msec_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->msec = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_msec_get) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_msec_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_msec_get" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    result = (BITCODE_BS) ((arg1)->msec);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_path_option_set) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_path_option_set(self,path_option);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_path_option_set" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DATALINK_path_option_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->path_option = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_path_option_get) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_path_option_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_path_option_get" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    result = (BITCODE_BS) ((arg1)->path_option);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_bl93_set) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_bl93_set(self,bl93);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_bl93_set" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DATALINK_bl93_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bl93 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_bl93_get) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_bl93_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_bl93_get" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    result = (BITCODE_BL) ((arg1)->bl93);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_update_status_set) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_update_status_set(self,update_status);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_update_status_set" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DATALINK_update_status_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->update_status);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->update_status = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->update_status = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_update_status_get) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_update_status_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_update_status_get" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    result = (BITCODE_TV) ((arg1)->update_status);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_num_customdata_set) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_num_customdata_set(self,num_customdata);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_num_customdata_set" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DATALINK_num_customdata_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_customdata = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_num_customdata_get) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_num_customdata_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_num_customdata_get" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_customdata);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_customdata_set) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    Dwg_DATALINK_customdata *arg2 = (Dwg_DATALINK_customdata *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_customdata_set(self,customdata);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_customdata_set" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_DATALINK_customdata, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DATALINK_customdata_set" "', argument " "2"" of type '" "Dwg_DATALINK_customdata *""'"); 
    }
    arg2 = (Dwg_DATALINK_customdata *)(argp2);
    if (arg1) (arg1)->customdata = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_customdata_get) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_DATALINK_customdata *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_customdata_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_customdata_get" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    result = (Dwg_DATALINK_customdata *) ((arg1)->customdata);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_DATALINK_customdata, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_hardowner_set) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_hardowner_set(self,hardowner);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_hardowner_set" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DATALINK_hardowner_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->hardowner = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DATALINK_hardowner_get) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DATALINK_hardowner_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DATALINK_hardowner_get" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    result = (BITCODE_H) ((arg1)->hardowner);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_DATALINK) {
  {
    int argvi = 0;
    struct _dwg_object_DATALINK *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_DATALINK();");
    }
    result = (struct _dwg_object_DATALINK *)calloc(1, sizeof(struct _dwg_object_DATALINK));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DATALINK, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_DATALINK) {
  {
    struct _dwg_object_DATALINK *arg1 = (struct _dwg_object_DATALINK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_DATALINK(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DATALINK, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_DATALINK" "', argument " "1"" of type '" "struct _dwg_object_DATALINK *""'"); 
    }
    arg1 = (struct _dwg_object_DATALINK *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMASSOC_Ref_parent_set) {
  {
    struct _dwg_DIMASSOC_Ref *arg1 = (struct _dwg_DIMASSOC_Ref *) 0 ;
    struct _dwg_object_DIMASSOC *arg2 = (struct _dwg_object_DIMASSOC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMASSOC_Ref_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMASSOC_Ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMASSOC_Ref_parent_set" "', argument " "1"" of type '" "struct _dwg_DIMASSOC_Ref *""'"); 
    }
    arg1 = (struct _dwg_DIMASSOC_Ref *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_DIMASSOC, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_DIMASSOC_Ref_parent_set" "', argument " "2"" of type '" "struct _dwg_object_DIMASSOC *""'"); 
    }
    arg2 = (struct _dwg_object_DIMASSOC *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMASSOC_Ref_parent_get) {
  {
    struct _dwg_DIMASSOC_Ref *arg1 = (struct _dwg_DIMASSOC_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_DIMASSOC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMASSOC_Ref_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMASSOC_Ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMASSOC_Ref_parent_get" "', argument " "1"" of type '" "struct _dwg_DIMASSOC_Ref *""'"); 
    }
    arg1 = (struct _dwg_DIMASSOC_Ref *)(argp1);
    result = (struct _dwg_object_DIMASSOC *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DIMASSOC, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMASSOC_Ref_classname_set) {
  {
    struct _dwg_DIMASSOC_Ref *arg1 = (struct _dwg_DIMASSOC_Ref *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMASSOC_Ref_classname_set(self,classname);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMASSOC_Ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMASSOC_Ref_classname_set" "', argument " "1"" of type '" "struct _dwg_DIMASSOC_Ref *""'"); 
    }
    arg1 = (struct _dwg_DIMASSOC_Ref *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_DIMASSOC_Ref_classname_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->classname);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->classname = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->classname = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMASSOC_Ref_classname_get) {
  {
    struct _dwg_DIMASSOC_Ref *arg1 = (struct _dwg_DIMASSOC_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMASSOC_Ref_classname_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMASSOC_Ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMASSOC_Ref_classname_get" "', argument " "1"" of type '" "struct _dwg_DIMASSOC_Ref *""'"); 
    }
    arg1 = (struct _dwg_DIMASSOC_Ref *)(argp1);
    result = (BITCODE_TV) ((arg1)->classname);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMASSOC_Ref_osnap_type_set) {
  {
    struct _dwg_DIMASSOC_Ref *arg1 = (struct _dwg_DIMASSOC_Ref *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMASSOC_Ref_osnap_type_set(self,osnap_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMASSOC_Ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMASSOC_Ref_osnap_type_set" "', argument " "1"" of type '" "struct _dwg_DIMASSOC_Ref *""'"); 
    }
    arg1 = (struct _dwg_DIMASSOC_Ref *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_DIMASSOC_Ref_osnap_type_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->osnap_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMASSOC_Ref_osnap_type_get) {
  {
    struct _dwg_DIMASSOC_Ref *arg1 = (struct _dwg_DIMASSOC_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMASSOC_Ref_osnap_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMASSOC_Ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMASSOC_Ref_osnap_type_get" "', argument " "1"" of type '" "struct _dwg_DIMASSOC_Ref *""'"); 
    }
    arg1 = (struct _dwg_DIMASSOC_Ref *)(argp1);
    result = (BITCODE_RC) ((arg1)->osnap_type);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMASSOC_Ref_osnap_dist_set) {
  {
    struct _dwg_DIMASSOC_Ref *arg1 = (struct _dwg_DIMASSOC_Ref *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMASSOC_Ref_osnap_dist_set(self,osnap_dist);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMASSOC_Ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMASSOC_Ref_osnap_dist_set" "', argument " "1"" of type '" "struct _dwg_DIMASSOC_Ref *""'"); 
    }
    arg1 = (struct _dwg_DIMASSOC_Ref *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_DIMASSOC_Ref_osnap_dist_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->osnap_dist = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMASSOC_Ref_osnap_dist_get) {
  {
    struct _dwg_DIMASSOC_Ref *arg1 = (struct _dwg_DIMASSOC_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMASSOC_Ref_osnap_dist_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMASSOC_Ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMASSOC_Ref_osnap_dist_get" "', argument " "1"" of type '" "struct _dwg_DIMASSOC_Ref *""'"); 
    }
    arg1 = (struct _dwg_DIMASSOC_Ref *)(argp1);
    result = (BITCODE_BD) ((arg1)->osnap_dist);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMASSOC_Ref_osnap_pt_set) {
  {
    struct _dwg_DIMASSOC_Ref *arg1 = (struct _dwg_DIMASSOC_Ref *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMASSOC_Ref_osnap_pt_set(self,osnap_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMASSOC_Ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMASSOC_Ref_osnap_pt_set" "', argument " "1"" of type '" "struct _dwg_DIMASSOC_Ref *""'"); 
    }
    arg1 = (struct _dwg_DIMASSOC_Ref *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_DIMASSOC_Ref_osnap_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->osnap_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMASSOC_Ref_osnap_pt_get) {
  {
    struct _dwg_DIMASSOC_Ref *arg1 = (struct _dwg_DIMASSOC_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMASSOC_Ref_osnap_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMASSOC_Ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMASSOC_Ref_osnap_pt_get" "', argument " "1"" of type '" "struct _dwg_DIMASSOC_Ref *""'"); 
    }
    arg1 = (struct _dwg_DIMASSOC_Ref *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->osnap_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMASSOC_Ref_num_xrefs_set) {
  {
    struct _dwg_DIMASSOC_Ref *arg1 = (struct _dwg_DIMASSOC_Ref *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMASSOC_Ref_num_xrefs_set(self,num_xrefs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMASSOC_Ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMASSOC_Ref_num_xrefs_set" "', argument " "1"" of type '" "struct _dwg_DIMASSOC_Ref *""'"); 
    }
    arg1 = (struct _dwg_DIMASSOC_Ref *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_DIMASSOC_Ref_num_xrefs_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_xrefs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMASSOC_Ref_num_xrefs_get) {
  {
    struct _dwg_DIMASSOC_Ref *arg1 = (struct _dwg_DIMASSOC_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMASSOC_Ref_num_xrefs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMASSOC_Ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMASSOC_Ref_num_xrefs_get" "', argument " "1"" of type '" "struct _dwg_DIMASSOC_Ref *""'"); 
    }
    arg1 = (struct _dwg_DIMASSOC_Ref *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_xrefs);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMASSOC_Ref_xrefs_set) {
  {
    struct _dwg_DIMASSOC_Ref *arg1 = (struct _dwg_DIMASSOC_Ref *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMASSOC_Ref_xrefs_set(self,xrefs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMASSOC_Ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMASSOC_Ref_xrefs_set" "', argument " "1"" of type '" "struct _dwg_DIMASSOC_Ref *""'"); 
    }
    arg1 = (struct _dwg_DIMASSOC_Ref *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_DIMASSOC_Ref_xrefs_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->xrefs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMASSOC_Ref_xrefs_get) {
  {
    struct _dwg_DIMASSOC_Ref *arg1 = (struct _dwg_DIMASSOC_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMASSOC_Ref_xrefs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMASSOC_Ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMASSOC_Ref_xrefs_get" "', argument " "1"" of type '" "struct _dwg_DIMASSOC_Ref *""'"); 
    }
    arg1 = (struct _dwg_DIMASSOC_Ref *)(argp1);
    result = (BITCODE_H *) ((arg1)->xrefs);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMASSOC_Ref_main_subent_type_set) {
  {
    struct _dwg_DIMASSOC_Ref *arg1 = (struct _dwg_DIMASSOC_Ref *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMASSOC_Ref_main_subent_type_set(self,main_subent_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMASSOC_Ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMASSOC_Ref_main_subent_type_set" "', argument " "1"" of type '" "struct _dwg_DIMASSOC_Ref *""'"); 
    }
    arg1 = (struct _dwg_DIMASSOC_Ref *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_DIMASSOC_Ref_main_subent_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->main_subent_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMASSOC_Ref_main_subent_type_get) {
  {
    struct _dwg_DIMASSOC_Ref *arg1 = (struct _dwg_DIMASSOC_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMASSOC_Ref_main_subent_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMASSOC_Ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMASSOC_Ref_main_subent_type_get" "', argument " "1"" of type '" "struct _dwg_DIMASSOC_Ref *""'"); 
    }
    arg1 = (struct _dwg_DIMASSOC_Ref *)(argp1);
    result = (BITCODE_BS) ((arg1)->main_subent_type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMASSOC_Ref_main_gsmarker_set) {
  {
    struct _dwg_DIMASSOC_Ref *arg1 = (struct _dwg_DIMASSOC_Ref *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMASSOC_Ref_main_gsmarker_set(self,main_gsmarker);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMASSOC_Ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMASSOC_Ref_main_gsmarker_set" "', argument " "1"" of type '" "struct _dwg_DIMASSOC_Ref *""'"); 
    }
    arg1 = (struct _dwg_DIMASSOC_Ref *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_DIMASSOC_Ref_main_gsmarker_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->main_gsmarker = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMASSOC_Ref_main_gsmarker_get) {
  {
    struct _dwg_DIMASSOC_Ref *arg1 = (struct _dwg_DIMASSOC_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMASSOC_Ref_main_gsmarker_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMASSOC_Ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMASSOC_Ref_main_gsmarker_get" "', argument " "1"" of type '" "struct _dwg_DIMASSOC_Ref *""'"); 
    }
    arg1 = (struct _dwg_DIMASSOC_Ref *)(argp1);
    result = (BITCODE_BL) ((arg1)->main_gsmarker);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMASSOC_Ref_num_xrefpaths_set) {
  {
    struct _dwg_DIMASSOC_Ref *arg1 = (struct _dwg_DIMASSOC_Ref *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMASSOC_Ref_num_xrefpaths_set(self,num_xrefpaths);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMASSOC_Ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMASSOC_Ref_num_xrefpaths_set" "', argument " "1"" of type '" "struct _dwg_DIMASSOC_Ref *""'"); 
    }
    arg1 = (struct _dwg_DIMASSOC_Ref *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_DIMASSOC_Ref_num_xrefpaths_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_xrefpaths = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMASSOC_Ref_num_xrefpaths_get) {
  {
    struct _dwg_DIMASSOC_Ref *arg1 = (struct _dwg_DIMASSOC_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMASSOC_Ref_num_xrefpaths_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMASSOC_Ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMASSOC_Ref_num_xrefpaths_get" "', argument " "1"" of type '" "struct _dwg_DIMASSOC_Ref *""'"); 
    }
    arg1 = (struct _dwg_DIMASSOC_Ref *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_xrefpaths);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMASSOC_Ref_xrefpaths_set) {
  {
    struct _dwg_DIMASSOC_Ref *arg1 = (struct _dwg_DIMASSOC_Ref *) 0 ;
    BITCODE_TV *arg2 = (BITCODE_TV *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMASSOC_Ref_xrefpaths_set(self,xrefpaths);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMASSOC_Ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMASSOC_Ref_xrefpaths_set" "', argument " "1"" of type '" "struct _dwg_DIMASSOC_Ref *""'"); 
    }
    arg1 = (struct _dwg_DIMASSOC_Ref *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_DIMASSOC_Ref_xrefpaths_set" "', argument " "2"" of type '" "BITCODE_TV *""'"); 
    }
    arg2 = (BITCODE_TV *)(argp2);
    if (arg1) (arg1)->xrefpaths = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMASSOC_Ref_xrefpaths_get) {
  {
    struct _dwg_DIMASSOC_Ref *arg1 = (struct _dwg_DIMASSOC_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMASSOC_Ref_xrefpaths_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMASSOC_Ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMASSOC_Ref_xrefpaths_get" "', argument " "1"" of type '" "struct _dwg_DIMASSOC_Ref *""'"); 
    }
    arg1 = (struct _dwg_DIMASSOC_Ref *)(argp1);
    result = (BITCODE_TV *) ((arg1)->xrefpaths);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMASSOC_Ref_has_lastpt_ref_set) {
  {
    struct _dwg_DIMASSOC_Ref *arg1 = (struct _dwg_DIMASSOC_Ref *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMASSOC_Ref_has_lastpt_ref_set(self,has_lastpt_ref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMASSOC_Ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMASSOC_Ref_has_lastpt_ref_set" "', argument " "1"" of type '" "struct _dwg_DIMASSOC_Ref *""'"); 
    }
    arg1 = (struct _dwg_DIMASSOC_Ref *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_DIMASSOC_Ref_has_lastpt_ref_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_lastpt_ref = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMASSOC_Ref_has_lastpt_ref_get) {
  {
    struct _dwg_DIMASSOC_Ref *arg1 = (struct _dwg_DIMASSOC_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMASSOC_Ref_has_lastpt_ref_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMASSOC_Ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMASSOC_Ref_has_lastpt_ref_get" "', argument " "1"" of type '" "struct _dwg_DIMASSOC_Ref *""'"); 
    }
    arg1 = (struct _dwg_DIMASSOC_Ref *)(argp1);
    result = (BITCODE_B) ((arg1)->has_lastpt_ref);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMASSOC_Ref_lastpt_ref_set) {
  {
    struct _dwg_DIMASSOC_Ref *arg1 = (struct _dwg_DIMASSOC_Ref *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMASSOC_Ref_lastpt_ref_set(self,lastpt_ref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMASSOC_Ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMASSOC_Ref_lastpt_ref_set" "', argument " "1"" of type '" "struct _dwg_DIMASSOC_Ref *""'"); 
    }
    arg1 = (struct _dwg_DIMASSOC_Ref *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_DIMASSOC_Ref_lastpt_ref_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->lastpt_ref = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMASSOC_Ref_lastpt_ref_get) {
  {
    struct _dwg_DIMASSOC_Ref *arg1 = (struct _dwg_DIMASSOC_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMASSOC_Ref_lastpt_ref_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMASSOC_Ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMASSOC_Ref_lastpt_ref_get" "', argument " "1"" of type '" "struct _dwg_DIMASSOC_Ref *""'"); 
    }
    arg1 = (struct _dwg_DIMASSOC_Ref *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->lastpt_ref);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMASSOC_Ref_num_intsectobj_set) {
  {
    struct _dwg_DIMASSOC_Ref *arg1 = (struct _dwg_DIMASSOC_Ref *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMASSOC_Ref_num_intsectobj_set(self,num_intsectobj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMASSOC_Ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMASSOC_Ref_num_intsectobj_set" "', argument " "1"" of type '" "struct _dwg_DIMASSOC_Ref *""'"); 
    }
    arg1 = (struct _dwg_DIMASSOC_Ref *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_DIMASSOC_Ref_num_intsectobj_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_intsectobj = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMASSOC_Ref_num_intsectobj_get) {
  {
    struct _dwg_DIMASSOC_Ref *arg1 = (struct _dwg_DIMASSOC_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMASSOC_Ref_num_intsectobj_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMASSOC_Ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMASSOC_Ref_num_intsectobj_get" "', argument " "1"" of type '" "struct _dwg_DIMASSOC_Ref *""'"); 
    }
    arg1 = (struct _dwg_DIMASSOC_Ref *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_intsectobj);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMASSOC_Ref_intsectobj_set) {
  {
    struct _dwg_DIMASSOC_Ref *arg1 = (struct _dwg_DIMASSOC_Ref *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_DIMASSOC_Ref_intsectobj_set(self,intsectobj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMASSOC_Ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMASSOC_Ref_intsectobj_set" "', argument " "1"" of type '" "struct _dwg_DIMASSOC_Ref *""'"); 
    }
    arg1 = (struct _dwg_DIMASSOC_Ref *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_DIMASSOC_Ref_intsectobj_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->intsectobj = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DIMASSOC_Ref_intsectobj_get) {
  {
    struct _dwg_DIMASSOC_Ref *arg1 = (struct _dwg_DIMASSOC_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DIMASSOC_Ref_intsectobj_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMASSOC_Ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DIMASSOC_Ref_intsectobj_get" "', argument " "1"" of type '" "struct _dwg_DIMASSOC_Ref *""'"); 
    }
    arg1 = (struct _dwg_DIMASSOC_Ref *)(argp1);
    result = (BITCODE_H *) ((arg1)->intsectobj);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_DIMASSOC_Ref) {
  {
    int argvi = 0;
    struct _dwg_DIMASSOC_Ref *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_DIMASSOC_Ref();");
    }
    result = (struct _dwg_DIMASSOC_Ref *)calloc(1, sizeof(struct _dwg_DIMASSOC_Ref));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_DIMASSOC_Ref, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_DIMASSOC_Ref) {
  {
    struct _dwg_DIMASSOC_Ref *arg1 = (struct _dwg_DIMASSOC_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_DIMASSOC_Ref(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_DIMASSOC_Ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_DIMASSOC_Ref" "', argument " "1"" of type '" "struct _dwg_DIMASSOC_Ref *""'"); 
    }
    arg1 = (struct _dwg_DIMASSOC_Ref *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMASSOC_parent_set) {
  {
    struct _dwg_object_DIMASSOC *arg1 = (struct _dwg_object_DIMASSOC *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMASSOC_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMASSOC, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMASSOC_parent_set" "', argument " "1"" of type '" "struct _dwg_object_DIMASSOC *""'"); 
    }
    arg1 = (struct _dwg_object_DIMASSOC *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DIMASSOC_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMASSOC_parent_get) {
  {
    struct _dwg_object_DIMASSOC *arg1 = (struct _dwg_object_DIMASSOC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMASSOC_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMASSOC, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMASSOC_parent_get" "', argument " "1"" of type '" "struct _dwg_object_DIMASSOC *""'"); 
    }
    arg1 = (struct _dwg_object_DIMASSOC *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMASSOC_dimensionobj_set) {
  {
    struct _dwg_object_DIMASSOC *arg1 = (struct _dwg_object_DIMASSOC *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMASSOC_dimensionobj_set(self,dimensionobj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMASSOC, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMASSOC_dimensionobj_set" "', argument " "1"" of type '" "struct _dwg_object_DIMASSOC *""'"); 
    }
    arg1 = (struct _dwg_object_DIMASSOC *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DIMASSOC_dimensionobj_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->dimensionobj = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMASSOC_dimensionobj_get) {
  {
    struct _dwg_object_DIMASSOC *arg1 = (struct _dwg_object_DIMASSOC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMASSOC_dimensionobj_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMASSOC, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMASSOC_dimensionobj_get" "', argument " "1"" of type '" "struct _dwg_object_DIMASSOC *""'"); 
    }
    arg1 = (struct _dwg_object_DIMASSOC *)(argp1);
    result = (BITCODE_H) ((arg1)->dimensionobj);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMASSOC_associativity_set) {
  {
    struct _dwg_object_DIMASSOC *arg1 = (struct _dwg_object_DIMASSOC *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMASSOC_associativity_set(self,associativity);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMASSOC, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMASSOC_associativity_set" "', argument " "1"" of type '" "struct _dwg_object_DIMASSOC *""'"); 
    }
    arg1 = (struct _dwg_object_DIMASSOC *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMASSOC_associativity_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->associativity = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMASSOC_associativity_get) {
  {
    struct _dwg_object_DIMASSOC *arg1 = (struct _dwg_object_DIMASSOC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMASSOC_associativity_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMASSOC, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMASSOC_associativity_get" "', argument " "1"" of type '" "struct _dwg_object_DIMASSOC *""'"); 
    }
    arg1 = (struct _dwg_object_DIMASSOC *)(argp1);
    result = (BITCODE_BL) ((arg1)->associativity);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMASSOC_trans_space_flag_set) {
  {
    struct _dwg_object_DIMASSOC *arg1 = (struct _dwg_object_DIMASSOC *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMASSOC_trans_space_flag_set(self,trans_space_flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMASSOC, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMASSOC_trans_space_flag_set" "', argument " "1"" of type '" "struct _dwg_object_DIMASSOC *""'"); 
    }
    arg1 = (struct _dwg_object_DIMASSOC *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMASSOC_trans_space_flag_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->trans_space_flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMASSOC_trans_space_flag_get) {
  {
    struct _dwg_object_DIMASSOC *arg1 = (struct _dwg_object_DIMASSOC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMASSOC_trans_space_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMASSOC, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMASSOC_trans_space_flag_get" "', argument " "1"" of type '" "struct _dwg_object_DIMASSOC *""'"); 
    }
    arg1 = (struct _dwg_object_DIMASSOC *)(argp1);
    result = (BITCODE_B) ((arg1)->trans_space_flag);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMASSOC_rotated_type_set) {
  {
    struct _dwg_object_DIMASSOC *arg1 = (struct _dwg_object_DIMASSOC *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMASSOC_rotated_type_set(self,rotated_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMASSOC, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMASSOC_rotated_type_set" "', argument " "1"" of type '" "struct _dwg_object_DIMASSOC *""'"); 
    }
    arg1 = (struct _dwg_object_DIMASSOC *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DIMASSOC_rotated_type_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->rotated_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMASSOC_rotated_type_get) {
  {
    struct _dwg_object_DIMASSOC *arg1 = (struct _dwg_object_DIMASSOC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMASSOC_rotated_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMASSOC, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMASSOC_rotated_type_get" "', argument " "1"" of type '" "struct _dwg_object_DIMASSOC *""'"); 
    }
    arg1 = (struct _dwg_object_DIMASSOC *)(argp1);
    result = (BITCODE_RC) ((arg1)->rotated_type);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMASSOC_ref_set) {
  {
    struct _dwg_object_DIMASSOC *arg1 = (struct _dwg_object_DIMASSOC *) 0 ;
    Dwg_DIMASSOC_Ref *arg2 = (Dwg_DIMASSOC_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DIMASSOC_ref_set(self,ref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMASSOC, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMASSOC_ref_set" "', argument " "1"" of type '" "struct _dwg_object_DIMASSOC *""'"); 
    }
    arg1 = (struct _dwg_object_DIMASSOC *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_DIMASSOC_Ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DIMASSOC_ref_set" "', argument " "2"" of type '" "Dwg_DIMASSOC_Ref *""'"); 
    }
    arg2 = (Dwg_DIMASSOC_Ref *)(argp2);
    if (arg1) (arg1)->ref = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DIMASSOC_ref_get) {
  {
    struct _dwg_object_DIMASSOC *arg1 = (struct _dwg_object_DIMASSOC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_DIMASSOC_Ref *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DIMASSOC_ref_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMASSOC, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DIMASSOC_ref_get" "', argument " "1"" of type '" "struct _dwg_object_DIMASSOC *""'"); 
    }
    arg1 = (struct _dwg_object_DIMASSOC *)(argp1);
    result = (Dwg_DIMASSOC_Ref *) ((arg1)->ref);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_DIMASSOC_Ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_DIMASSOC) {
  {
    int argvi = 0;
    struct _dwg_object_DIMASSOC *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_DIMASSOC();");
    }
    result = (struct _dwg_object_DIMASSOC *)calloc(1, sizeof(struct _dwg_object_DIMASSOC));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DIMASSOC, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_DIMASSOC) {
  {
    struct _dwg_object_DIMASSOC *arg1 = (struct _dwg_object_DIMASSOC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_DIMASSOC(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DIMASSOC, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_DIMASSOC" "', argument " "1"" of type '" "struct _dwg_object_DIMASSOC *""'"); 
    }
    arg1 = (struct _dwg_object_DIMASSOC *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACTIONBODY_parent_set) {
  {
    struct _dwg_ACTIONBODY *arg1 = (struct _dwg_ACTIONBODY *) 0 ;
    struct _dwg_object_ASSOCNETWORK *arg2 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ACTIONBODY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACTIONBODY_parent_set" "', argument " "1"" of type '" "struct _dwg_ACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCNETWORK, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_ACTIONBODY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg2 = (struct _dwg_object_ASSOCNETWORK *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACTIONBODY_parent_get) {
  {
    struct _dwg_ACTIONBODY *arg1 = (struct _dwg_ACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_ASSOCNETWORK *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ACTIONBODY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACTIONBODY_parent_get" "', argument " "1"" of type '" "struct _dwg_ACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ACTIONBODY *)(argp1);
    result = (struct _dwg_object_ASSOCNETWORK *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACTIONBODY_evaluatorid_set) {
  {
    struct _dwg_ACTIONBODY *arg1 = (struct _dwg_ACTIONBODY *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ACTIONBODY_evaluatorid_set(self,evaluatorid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACTIONBODY_evaluatorid_set" "', argument " "1"" of type '" "struct _dwg_ACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ACTIONBODY *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_ACTIONBODY_evaluatorid_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->evaluatorid);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->evaluatorid = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->evaluatorid = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACTIONBODY_evaluatorid_get) {
  {
    struct _dwg_ACTIONBODY *arg1 = (struct _dwg_ACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ACTIONBODY_evaluatorid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACTIONBODY_evaluatorid_get" "', argument " "1"" of type '" "struct _dwg_ACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ACTIONBODY *)(argp1);
    result = (BITCODE_TV) ((arg1)->evaluatorid);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACTIONBODY_expression_set) {
  {
    struct _dwg_ACTIONBODY *arg1 = (struct _dwg_ACTIONBODY *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ACTIONBODY_expression_set(self,expression);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACTIONBODY_expression_set" "', argument " "1"" of type '" "struct _dwg_ACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ACTIONBODY *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_ACTIONBODY_expression_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->expression);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->expression = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->expression = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACTIONBODY_expression_get) {
  {
    struct _dwg_ACTIONBODY *arg1 = (struct _dwg_ACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ACTIONBODY_expression_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACTIONBODY_expression_get" "', argument " "1"" of type '" "struct _dwg_ACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ACTIONBODY *)(argp1);
    result = (BITCODE_TV) ((arg1)->expression);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACTIONBODY_value_set) {
  {
    struct _dwg_ACTIONBODY *arg1 = (struct _dwg_ACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ACTIONBODY_value_set(self,value);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACTIONBODY_value_set" "', argument " "1"" of type '" "struct _dwg_ACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ACTIONBODY_value_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->value = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACTIONBODY_value_get) {
  {
    struct _dwg_ACTIONBODY *arg1 = (struct _dwg_ACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ACTIONBODY_value_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACTIONBODY_value_get" "', argument " "1"" of type '" "struct _dwg_ACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->value);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_ACTIONBODY) {
  {
    int argvi = 0;
    struct _dwg_ACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_ACTIONBODY();");
    }
    result = (struct _dwg_ACTIONBODY *)calloc(1, sizeof(struct _dwg_ACTIONBODY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ACTIONBODY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_ACTIONBODY) {
  {
    struct _dwg_ACTIONBODY *arg1 = (struct _dwg_ACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_ACTIONBODY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_ACTIONBODY" "', argument " "1"" of type '" "struct _dwg_ACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ACTIONBODY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EvalVariant_code_set) {
  {
    struct _dwg_EvalVariant *arg1 = (struct _dwg_EvalVariant *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_EvalVariant_code_set(self,code);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalVariant, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EvalVariant_code_set" "', argument " "1"" of type '" "struct _dwg_EvalVariant *""'"); 
    }
    arg1 = (struct _dwg_EvalVariant *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_EvalVariant_code_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->code = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EvalVariant_code_get) {
  {
    struct _dwg_EvalVariant *arg1 = (struct _dwg_EvalVariant *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_EvalVariant_code_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalVariant, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EvalVariant_code_get" "', argument " "1"" of type '" "struct _dwg_EvalVariant *""'"); 
    }
    arg1 = (struct _dwg_EvalVariant *)(argp1);
    result = (BITCODE_BS) ((arg1)->code);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EvalVariant_u_get) {
  {
    struct _dwg_EvalVariant *arg1 = (struct _dwg_EvalVariant *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    _dwg_EvalVariant_u *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_EvalVariant_u_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalVariant, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EvalVariant_u_get" "', argument " "1"" of type '" "struct _dwg_EvalVariant *""'"); 
    }
    arg1 = (struct _dwg_EvalVariant *)(argp1);
    result = (_dwg_EvalVariant_u *)& ((arg1)->u);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalVariant_u, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_EvalVariant) {
  {
    int argvi = 0;
    struct _dwg_EvalVariant *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_EvalVariant();");
    }
    result = (struct _dwg_EvalVariant *)calloc(1, sizeof(struct _dwg_EvalVariant));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalVariant, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_EvalVariant) {
  {
    struct _dwg_EvalVariant *arg1 = (struct _dwg_EvalVariant *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_EvalVariant(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalVariant, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_EvalVariant" "', argument " "1"" of type '" "struct _dwg_EvalVariant *""'"); 
    }
    arg1 = (struct _dwg_EvalVariant *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_EvalVariant_u_bd_set) {
  {
    _dwg_EvalVariant_u *arg1 = (_dwg_EvalVariant_u *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_EvalVariant_u_bd_set(self,bd);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalVariant_u, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_EvalVariant_u_bd_set" "', argument " "1"" of type '" "_dwg_EvalVariant_u *""'"); 
    }
    arg1 = (_dwg_EvalVariant_u *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "_dwg_EvalVariant_u_bd_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->bd = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_EvalVariant_u_bd_get) {
  {
    _dwg_EvalVariant_u *arg1 = (_dwg_EvalVariant_u *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_EvalVariant_u_bd_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalVariant_u, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_EvalVariant_u_bd_get" "', argument " "1"" of type '" "_dwg_EvalVariant_u *""'"); 
    }
    arg1 = (_dwg_EvalVariant_u *)(argp1);
    result = (BITCODE_BD) ((arg1)->bd);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_EvalVariant_u_bl_set) {
  {
    _dwg_EvalVariant_u *arg1 = (_dwg_EvalVariant_u *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_EvalVariant_u_bl_set(self,bl);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalVariant_u, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_EvalVariant_u_bl_set" "', argument " "1"" of type '" "_dwg_EvalVariant_u *""'"); 
    }
    arg1 = (_dwg_EvalVariant_u *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "_dwg_EvalVariant_u_bl_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bl = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_EvalVariant_u_bl_get) {
  {
    _dwg_EvalVariant_u *arg1 = (_dwg_EvalVariant_u *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_EvalVariant_u_bl_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalVariant_u, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_EvalVariant_u_bl_get" "', argument " "1"" of type '" "_dwg_EvalVariant_u *""'"); 
    }
    arg1 = (_dwg_EvalVariant_u *)(argp1);
    result = (BITCODE_BL) ((arg1)->bl);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_EvalVariant_u_bs_set) {
  {
    _dwg_EvalVariant_u *arg1 = (_dwg_EvalVariant_u *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_EvalVariant_u_bs_set(self,bs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalVariant_u, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_EvalVariant_u_bs_set" "', argument " "1"" of type '" "_dwg_EvalVariant_u *""'"); 
    }
    arg1 = (_dwg_EvalVariant_u *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "_dwg_EvalVariant_u_bs_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->bs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_EvalVariant_u_bs_get) {
  {
    _dwg_EvalVariant_u *arg1 = (_dwg_EvalVariant_u *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_EvalVariant_u_bs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalVariant_u, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_EvalVariant_u_bs_get" "', argument " "1"" of type '" "_dwg_EvalVariant_u *""'"); 
    }
    arg1 = (_dwg_EvalVariant_u *)(argp1);
    result = (BITCODE_BS) ((arg1)->bs);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_EvalVariant_u_rc_set) {
  {
    _dwg_EvalVariant_u *arg1 = (_dwg_EvalVariant_u *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_EvalVariant_u_rc_set(self,rc);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalVariant_u, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_EvalVariant_u_rc_set" "', argument " "1"" of type '" "_dwg_EvalVariant_u *""'"); 
    }
    arg1 = (_dwg_EvalVariant_u *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "_dwg_EvalVariant_u_rc_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->rc = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_EvalVariant_u_rc_get) {
  {
    _dwg_EvalVariant_u *arg1 = (_dwg_EvalVariant_u *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_EvalVariant_u_rc_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalVariant_u, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_EvalVariant_u_rc_get" "', argument " "1"" of type '" "_dwg_EvalVariant_u *""'"); 
    }
    arg1 = (_dwg_EvalVariant_u *)(argp1);
    result = (BITCODE_RC) ((arg1)->rc);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_EvalVariant_u_text_set) {
  {
    _dwg_EvalVariant_u *arg1 = (_dwg_EvalVariant_u *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_EvalVariant_u_text_set(self,text);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalVariant_u, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_EvalVariant_u_text_set" "', argument " "1"" of type '" "_dwg_EvalVariant_u *""'"); 
    }
    arg1 = (_dwg_EvalVariant_u *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_EvalVariant_u_text_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->text);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->text = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->text = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_EvalVariant_u_text_get) {
  {
    _dwg_EvalVariant_u *arg1 = (_dwg_EvalVariant_u *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_EvalVariant_u_text_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalVariant_u, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_EvalVariant_u_text_get" "', argument " "1"" of type '" "_dwg_EvalVariant_u *""'"); 
    }
    arg1 = (_dwg_EvalVariant_u *)(argp1);
    result = (BITCODE_TV) ((arg1)->text);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_EvalVariant_u_handle_set) {
  {
    _dwg_EvalVariant_u *arg1 = (_dwg_EvalVariant_u *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_EvalVariant_u_handle_set(self,handle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalVariant_u, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_EvalVariant_u_handle_set" "', argument " "1"" of type '" "_dwg_EvalVariant_u *""'"); 
    }
    arg1 = (_dwg_EvalVariant_u *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_EvalVariant_u_handle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->handle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_EvalVariant_u_handle_get) {
  {
    _dwg_EvalVariant_u *arg1 = (_dwg_EvalVariant_u *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_EvalVariant_u_handle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalVariant_u, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_EvalVariant_u_handle_get" "', argument " "1"" of type '" "_dwg_EvalVariant_u *""'"); 
    }
    arg1 = (_dwg_EvalVariant_u *)(argp1);
    result = (BITCODE_H) ((arg1)->handle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new__dwg_EvalVariant_u) {
  {
    int argvi = 0;
    _dwg_EvalVariant_u *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new__dwg_EvalVariant_u();");
    }
    result = (_dwg_EvalVariant_u *)calloc(1, sizeof(_dwg_EvalVariant_u));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalVariant_u, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete__dwg_EvalVariant_u) {
  {
    _dwg_EvalVariant_u *arg1 = (_dwg_EvalVariant_u *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete__dwg_EvalVariant_u(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalVariant_u, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete__dwg_EvalVariant_u" "', argument " "1"" of type '" "_dwg_EvalVariant_u *""'"); 
    }
    arg1 = (_dwg_EvalVariant_u *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_VALUEPARAM_vars_value_set) {
  {
    struct _dwg_VALUEPARAM_vars *arg1 = (struct _dwg_VALUEPARAM_vars *) 0 ;
    Dwg_EvalVariant *arg2 = (Dwg_EvalVariant *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_VALUEPARAM_vars_value_set(self,value);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_VALUEPARAM_vars, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_VALUEPARAM_vars_value_set" "', argument " "1"" of type '" "struct _dwg_VALUEPARAM_vars *""'"); 
    }
    arg1 = (struct _dwg_VALUEPARAM_vars *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalVariant, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_VALUEPARAM_vars_value_set" "', argument " "2"" of type '" "Dwg_EvalVariant *""'"); 
    }
    arg2 = (Dwg_EvalVariant *)(argp2);
    if (arg1) (arg1)->value = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_VALUEPARAM_vars_value_get) {
  {
    struct _dwg_VALUEPARAM_vars *arg1 = (struct _dwg_VALUEPARAM_vars *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalVariant *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_VALUEPARAM_vars_value_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_VALUEPARAM_vars, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_VALUEPARAM_vars_value_get" "', argument " "1"" of type '" "struct _dwg_VALUEPARAM_vars *""'"); 
    }
    arg1 = (struct _dwg_VALUEPARAM_vars *)(argp1);
    result = (Dwg_EvalVariant *)& ((arg1)->value);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalVariant, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_VALUEPARAM_vars_handle_set) {
  {
    struct _dwg_VALUEPARAM_vars *arg1 = (struct _dwg_VALUEPARAM_vars *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_VALUEPARAM_vars_handle_set(self,handle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_VALUEPARAM_vars, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_VALUEPARAM_vars_handle_set" "', argument " "1"" of type '" "struct _dwg_VALUEPARAM_vars *""'"); 
    }
    arg1 = (struct _dwg_VALUEPARAM_vars *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_VALUEPARAM_vars_handle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->handle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_VALUEPARAM_vars_handle_get) {
  {
    struct _dwg_VALUEPARAM_vars *arg1 = (struct _dwg_VALUEPARAM_vars *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_VALUEPARAM_vars_handle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_VALUEPARAM_vars, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_VALUEPARAM_vars_handle_get" "', argument " "1"" of type '" "struct _dwg_VALUEPARAM_vars *""'"); 
    }
    arg1 = (struct _dwg_VALUEPARAM_vars *)(argp1);
    result = (BITCODE_H) ((arg1)->handle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_VALUEPARAM_vars) {
  {
    int argvi = 0;
    struct _dwg_VALUEPARAM_vars *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_VALUEPARAM_vars();");
    }
    result = (struct _dwg_VALUEPARAM_vars *)calloc(1, sizeof(struct _dwg_VALUEPARAM_vars));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_VALUEPARAM_vars, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_VALUEPARAM_vars) {
  {
    struct _dwg_VALUEPARAM_vars *arg1 = (struct _dwg_VALUEPARAM_vars *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_VALUEPARAM_vars(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_VALUEPARAM_vars, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_VALUEPARAM_vars" "', argument " "1"" of type '" "struct _dwg_VALUEPARAM_vars *""'"); 
    }
    arg1 = (struct _dwg_VALUEPARAM_vars *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_VALUEPARAM_parent_set) {
  {
    struct _dwg_VALUEPARAM *arg1 = (struct _dwg_VALUEPARAM *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_VALUEPARAM_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_VALUEPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_VALUEPARAM_parent_set" "', argument " "1"" of type '" "struct _dwg_VALUEPARAM *""'"); 
    }
    arg1 = (struct _dwg_VALUEPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_VALUEPARAM_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_VALUEPARAM_parent_get) {
  {
    struct _dwg_VALUEPARAM *arg1 = (struct _dwg_VALUEPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_VALUEPARAM_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_VALUEPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_VALUEPARAM_parent_get" "', argument " "1"" of type '" "struct _dwg_VALUEPARAM *""'"); 
    }
    arg1 = (struct _dwg_VALUEPARAM *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_VALUEPARAM_class_version_set) {
  {
    struct _dwg_VALUEPARAM *arg1 = (struct _dwg_VALUEPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_VALUEPARAM_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_VALUEPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_VALUEPARAM_class_version_set" "', argument " "1"" of type '" "struct _dwg_VALUEPARAM *""'"); 
    }
    arg1 = (struct _dwg_VALUEPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_VALUEPARAM_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_VALUEPARAM_class_version_get) {
  {
    struct _dwg_VALUEPARAM *arg1 = (struct _dwg_VALUEPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_VALUEPARAM_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_VALUEPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_VALUEPARAM_class_version_get" "', argument " "1"" of type '" "struct _dwg_VALUEPARAM *""'"); 
    }
    arg1 = (struct _dwg_VALUEPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_VALUEPARAM_name_set) {
  {
    struct _dwg_VALUEPARAM *arg1 = (struct _dwg_VALUEPARAM *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_VALUEPARAM_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_VALUEPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_VALUEPARAM_name_set" "', argument " "1"" of type '" "struct _dwg_VALUEPARAM *""'"); 
    }
    arg1 = (struct _dwg_VALUEPARAM *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_VALUEPARAM_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_VALUEPARAM_name_get) {
  {
    struct _dwg_VALUEPARAM *arg1 = (struct _dwg_VALUEPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_VALUEPARAM_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_VALUEPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_VALUEPARAM_name_get" "', argument " "1"" of type '" "struct _dwg_VALUEPARAM *""'"); 
    }
    arg1 = (struct _dwg_VALUEPARAM *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_VALUEPARAM_unit_type_set) {
  {
    struct _dwg_VALUEPARAM *arg1 = (struct _dwg_VALUEPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_VALUEPARAM_unit_type_set(self,unit_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_VALUEPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_VALUEPARAM_unit_type_set" "', argument " "1"" of type '" "struct _dwg_VALUEPARAM *""'"); 
    }
    arg1 = (struct _dwg_VALUEPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_VALUEPARAM_unit_type_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unit_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_VALUEPARAM_unit_type_get) {
  {
    struct _dwg_VALUEPARAM *arg1 = (struct _dwg_VALUEPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_VALUEPARAM_unit_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_VALUEPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_VALUEPARAM_unit_type_get" "', argument " "1"" of type '" "struct _dwg_VALUEPARAM *""'"); 
    }
    arg1 = (struct _dwg_VALUEPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->unit_type);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_VALUEPARAM_num_vars_set) {
  {
    struct _dwg_VALUEPARAM *arg1 = (struct _dwg_VALUEPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_VALUEPARAM_num_vars_set(self,num_vars);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_VALUEPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_VALUEPARAM_num_vars_set" "', argument " "1"" of type '" "struct _dwg_VALUEPARAM *""'"); 
    }
    arg1 = (struct _dwg_VALUEPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_VALUEPARAM_num_vars_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_vars = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_VALUEPARAM_num_vars_get) {
  {
    struct _dwg_VALUEPARAM *arg1 = (struct _dwg_VALUEPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_VALUEPARAM_num_vars_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_VALUEPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_VALUEPARAM_num_vars_get" "', argument " "1"" of type '" "struct _dwg_VALUEPARAM *""'"); 
    }
    arg1 = (struct _dwg_VALUEPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_vars);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_VALUEPARAM_vars_set) {
  {
    struct _dwg_VALUEPARAM *arg1 = (struct _dwg_VALUEPARAM *) 0 ;
    Dwg_VALUEPARAM_vars *arg2 = (Dwg_VALUEPARAM_vars *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_VALUEPARAM_vars_set(self,vars);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_VALUEPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_VALUEPARAM_vars_set" "', argument " "1"" of type '" "struct _dwg_VALUEPARAM *""'"); 
    }
    arg1 = (struct _dwg_VALUEPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_VALUEPARAM_vars, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_VALUEPARAM_vars_set" "', argument " "2"" of type '" "Dwg_VALUEPARAM_vars *""'"); 
    }
    arg2 = (Dwg_VALUEPARAM_vars *)(argp2);
    if (arg1) (arg1)->vars = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_VALUEPARAM_vars_get) {
  {
    struct _dwg_VALUEPARAM *arg1 = (struct _dwg_VALUEPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_VALUEPARAM_vars *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_VALUEPARAM_vars_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_VALUEPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_VALUEPARAM_vars_get" "', argument " "1"" of type '" "struct _dwg_VALUEPARAM *""'"); 
    }
    arg1 = (struct _dwg_VALUEPARAM *)(argp1);
    result = (Dwg_VALUEPARAM_vars *) ((arg1)->vars);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_VALUEPARAM_vars, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_VALUEPARAM_controlled_objdep_set) {
  {
    struct _dwg_VALUEPARAM *arg1 = (struct _dwg_VALUEPARAM *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_VALUEPARAM_controlled_objdep_set(self,controlled_objdep);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_VALUEPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_VALUEPARAM_controlled_objdep_set" "', argument " "1"" of type '" "struct _dwg_VALUEPARAM *""'"); 
    }
    arg1 = (struct _dwg_VALUEPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_VALUEPARAM_controlled_objdep_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->controlled_objdep = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_VALUEPARAM_controlled_objdep_get) {
  {
    struct _dwg_VALUEPARAM *arg1 = (struct _dwg_VALUEPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_VALUEPARAM_controlled_objdep_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_VALUEPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_VALUEPARAM_controlled_objdep_get" "', argument " "1"" of type '" "struct _dwg_VALUEPARAM *""'"); 
    }
    arg1 = (struct _dwg_VALUEPARAM *)(argp1);
    result = (BITCODE_H) ((arg1)->controlled_objdep);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_VALUEPARAM) {
  {
    int argvi = 0;
    struct _dwg_VALUEPARAM *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_VALUEPARAM();");
    }
    result = (struct _dwg_VALUEPARAM *)calloc(1, sizeof(struct _dwg_VALUEPARAM));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_VALUEPARAM, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_VALUEPARAM) {
  {
    struct _dwg_VALUEPARAM *arg1 = (struct _dwg_VALUEPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_VALUEPARAM(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_VALUEPARAM, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_VALUEPARAM" "', argument " "1"" of type '" "struct _dwg_VALUEPARAM *""'"); 
    }
    arg1 = (struct _dwg_VALUEPARAM *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_parent_set) {
  {
    struct _dwg_ASSOCPARAMBASEDACTIONBODY *arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ASSOCPARAMBASEDACTIONBODY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCPARAMBASEDACTIONBODY_parent_set" "', argument " "1"" of type '" "struct _dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_ASSOCPARAMBASEDACTIONBODY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_parent_get) {
  {
    struct _dwg_ASSOCPARAMBASEDACTIONBODY *arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ASSOCPARAMBASEDACTIONBODY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCPARAMBASEDACTIONBODY_parent_get" "', argument " "1"" of type '" "struct _dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_version_set) {
  {
    struct _dwg_ASSOCPARAMBASEDACTIONBODY *arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ASSOCPARAMBASEDACTIONBODY_version_set(self,version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCPARAMBASEDACTIONBODY_version_set" "', argument " "1"" of type '" "struct _dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ASSOCPARAMBASEDACTIONBODY_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_version_get) {
  {
    struct _dwg_ASSOCPARAMBASEDACTIONBODY *arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ASSOCPARAMBASEDACTIONBODY_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCPARAMBASEDACTIONBODY_version_get" "', argument " "1"" of type '" "struct _dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_minor_set) {
  {
    struct _dwg_ASSOCPARAMBASEDACTIONBODY *arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ASSOCPARAMBASEDACTIONBODY_minor_set(self,minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCPARAMBASEDACTIONBODY_minor_set" "', argument " "1"" of type '" "struct _dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ASSOCPARAMBASEDACTIONBODY_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_minor_get) {
  {
    struct _dwg_ASSOCPARAMBASEDACTIONBODY *arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ASSOCPARAMBASEDACTIONBODY_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCPARAMBASEDACTIONBODY_minor_get" "', argument " "1"" of type '" "struct _dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_num_deps_set) {
  {
    struct _dwg_ASSOCPARAMBASEDACTIONBODY *arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ASSOCPARAMBASEDACTIONBODY_num_deps_set(self,num_deps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCPARAMBASEDACTIONBODY_num_deps_set" "', argument " "1"" of type '" "struct _dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ASSOCPARAMBASEDACTIONBODY_num_deps_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_deps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_num_deps_get) {
  {
    struct _dwg_ASSOCPARAMBASEDACTIONBODY *arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ASSOCPARAMBASEDACTIONBODY_num_deps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCPARAMBASEDACTIONBODY_num_deps_get" "', argument " "1"" of type '" "struct _dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_deps);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_deps_set) {
  {
    struct _dwg_ASSOCPARAMBASEDACTIONBODY *arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ASSOCPARAMBASEDACTIONBODY_deps_set(self,deps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCPARAMBASEDACTIONBODY_deps_set" "', argument " "1"" of type '" "struct _dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_ASSOCPARAMBASEDACTIONBODY_deps_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->deps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_deps_get) {
  {
    struct _dwg_ASSOCPARAMBASEDACTIONBODY *arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ASSOCPARAMBASEDACTIONBODY_deps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCPARAMBASEDACTIONBODY_deps_get" "', argument " "1"" of type '" "struct _dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *)(argp1);
    result = (BITCODE_H *) ((arg1)->deps);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_l4_set) {
  {
    struct _dwg_ASSOCPARAMBASEDACTIONBODY *arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ASSOCPARAMBASEDACTIONBODY_l4_set(self,l4);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCPARAMBASEDACTIONBODY_l4_set" "', argument " "1"" of type '" "struct _dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ASSOCPARAMBASEDACTIONBODY_l4_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->l4 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_l4_get) {
  {
    struct _dwg_ASSOCPARAMBASEDACTIONBODY *arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ASSOCPARAMBASEDACTIONBODY_l4_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCPARAMBASEDACTIONBODY_l4_get" "', argument " "1"" of type '" "struct _dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->l4);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_l5_set) {
  {
    struct _dwg_ASSOCPARAMBASEDACTIONBODY *arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ASSOCPARAMBASEDACTIONBODY_l5_set(self,l5);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCPARAMBASEDACTIONBODY_l5_set" "', argument " "1"" of type '" "struct _dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ASSOCPARAMBASEDACTIONBODY_l5_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->l5 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_l5_get) {
  {
    struct _dwg_ASSOCPARAMBASEDACTIONBODY *arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ASSOCPARAMBASEDACTIONBODY_l5_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCPARAMBASEDACTIONBODY_l5_get" "', argument " "1"" of type '" "struct _dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->l5);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_assocdep_set) {
  {
    struct _dwg_ASSOCPARAMBASEDACTIONBODY *arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ASSOCPARAMBASEDACTIONBODY_assocdep_set(self,assocdep);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCPARAMBASEDACTIONBODY_assocdep_set" "', argument " "1"" of type '" "struct _dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_ASSOCPARAMBASEDACTIONBODY_assocdep_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->assocdep = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_assocdep_get) {
  {
    struct _dwg_ASSOCPARAMBASEDACTIONBODY *arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ASSOCPARAMBASEDACTIONBODY_assocdep_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCPARAMBASEDACTIONBODY_assocdep_get" "', argument " "1"" of type '" "struct _dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *)(argp1);
    result = (BITCODE_H) ((arg1)->assocdep);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_num_values_set) {
  {
    struct _dwg_ASSOCPARAMBASEDACTIONBODY *arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ASSOCPARAMBASEDACTIONBODY_num_values_set(self,num_values);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCPARAMBASEDACTIONBODY_num_values_set" "', argument " "1"" of type '" "struct _dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ASSOCPARAMBASEDACTIONBODY_num_values_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_values = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_num_values_get) {
  {
    struct _dwg_ASSOCPARAMBASEDACTIONBODY *arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ASSOCPARAMBASEDACTIONBODY_num_values_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCPARAMBASEDACTIONBODY_num_values_get" "', argument " "1"" of type '" "struct _dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_values);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_values_set) {
  {
    struct _dwg_ASSOCPARAMBASEDACTIONBODY *arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    Dwg_VALUEPARAM *arg2 = (Dwg_VALUEPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ASSOCPARAMBASEDACTIONBODY_values_set(self,values);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCPARAMBASEDACTIONBODY_values_set" "', argument " "1"" of type '" "struct _dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_VALUEPARAM, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_ASSOCPARAMBASEDACTIONBODY_values_set" "', argument " "2"" of type '" "Dwg_VALUEPARAM *""'"); 
    }
    arg2 = (Dwg_VALUEPARAM *)(argp2);
    if (arg1) (arg1)->values = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_values_get) {
  {
    struct _dwg_ASSOCPARAMBASEDACTIONBODY *arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_VALUEPARAM *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ASSOCPARAMBASEDACTIONBODY_values_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCPARAMBASEDACTIONBODY_values_get" "', argument " "1"" of type '" "struct _dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *)(argp1);
    result = (Dwg_VALUEPARAM *) ((arg1)->values);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_VALUEPARAM, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_ASSOCPARAMBASEDACTIONBODY) {
  {
    int argvi = 0;
    struct _dwg_ASSOCPARAMBASEDACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_ASSOCPARAMBASEDACTIONBODY();");
    }
    result = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *)calloc(1, sizeof(struct _dwg_ASSOCPARAMBASEDACTIONBODY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_ASSOCPARAMBASEDACTIONBODY) {
  {
    struct _dwg_ASSOCPARAMBASEDACTIONBODY *arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_ASSOCPARAMBASEDACTIONBODY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_ASSOCPARAMBASEDACTIONBODY" "', argument " "1"" of type '" "struct _dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCPARAMBASEDACTIONBODY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCACTION_Deps_parent_set) {
  {
    struct _dwg_ASSOCACTION_Deps *arg1 = (struct _dwg_ASSOCACTION_Deps *) 0 ;
    struct _dwg_object_ASSOCACTION *arg2 = (struct _dwg_object_ASSOCACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ASSOCACTION_Deps_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCACTION_Deps, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCACTION_Deps_parent_set" "', argument " "1"" of type '" "struct _dwg_ASSOCACTION_Deps *""'"); 
    }
    arg1 = (struct _dwg_ASSOCACTION_Deps *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCACTION, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_ASSOCACTION_Deps_parent_set" "', argument " "2"" of type '" "struct _dwg_object_ASSOCACTION *""'"); 
    }
    arg2 = (struct _dwg_object_ASSOCACTION *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCACTION_Deps_parent_get) {
  {
    struct _dwg_ASSOCACTION_Deps *arg1 = (struct _dwg_ASSOCACTION_Deps *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_ASSOCACTION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ASSOCACTION_Deps_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCACTION_Deps, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCACTION_Deps_parent_get" "', argument " "1"" of type '" "struct _dwg_ASSOCACTION_Deps *""'"); 
    }
    arg1 = (struct _dwg_ASSOCACTION_Deps *)(argp1);
    result = (struct _dwg_object_ASSOCACTION *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCACTION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCACTION_Deps_is_owned_set) {
  {
    struct _dwg_ASSOCACTION_Deps *arg1 = (struct _dwg_ASSOCACTION_Deps *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ASSOCACTION_Deps_is_owned_set(self,is_owned);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCACTION_Deps, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCACTION_Deps_is_owned_set" "', argument " "1"" of type '" "struct _dwg_ASSOCACTION_Deps *""'"); 
    }
    arg1 = (struct _dwg_ASSOCACTION_Deps *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ASSOCACTION_Deps_is_owned_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_owned = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCACTION_Deps_is_owned_get) {
  {
    struct _dwg_ASSOCACTION_Deps *arg1 = (struct _dwg_ASSOCACTION_Deps *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ASSOCACTION_Deps_is_owned_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCACTION_Deps, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCACTION_Deps_is_owned_get" "', argument " "1"" of type '" "struct _dwg_ASSOCACTION_Deps *""'"); 
    }
    arg1 = (struct _dwg_ASSOCACTION_Deps *)(argp1);
    result = (BITCODE_B) ((arg1)->is_owned);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCACTION_Deps_dep_set) {
  {
    struct _dwg_ASSOCACTION_Deps *arg1 = (struct _dwg_ASSOCACTION_Deps *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ASSOCACTION_Deps_dep_set(self,dep);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCACTION_Deps, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCACTION_Deps_dep_set" "', argument " "1"" of type '" "struct _dwg_ASSOCACTION_Deps *""'"); 
    }
    arg1 = (struct _dwg_ASSOCACTION_Deps *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_ASSOCACTION_Deps_dep_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->dep = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCACTION_Deps_dep_get) {
  {
    struct _dwg_ASSOCACTION_Deps *arg1 = (struct _dwg_ASSOCACTION_Deps *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ASSOCACTION_Deps_dep_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCACTION_Deps, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCACTION_Deps_dep_get" "', argument " "1"" of type '" "struct _dwg_ASSOCACTION_Deps *""'"); 
    }
    arg1 = (struct _dwg_ASSOCACTION_Deps *)(argp1);
    result = (BITCODE_H) ((arg1)->dep);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_ASSOCACTION_Deps) {
  {
    int argvi = 0;
    struct _dwg_ASSOCACTION_Deps *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_ASSOCACTION_Deps();");
    }
    result = (struct _dwg_ASSOCACTION_Deps *)calloc(1, sizeof(struct _dwg_ASSOCACTION_Deps));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCACTION_Deps, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_ASSOCACTION_Deps) {
  {
    struct _dwg_ASSOCACTION_Deps *arg1 = (struct _dwg_ASSOCACTION_Deps *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_ASSOCACTION_Deps(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCACTION_Deps, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_ASSOCACTION_Deps" "', argument " "1"" of type '" "struct _dwg_ASSOCACTION_Deps *""'"); 
    }
    arg1 = (struct _dwg_ASSOCACTION_Deps *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDEPENDENCY_parent_set) {
  {
    struct _dwg_object_ASSOCDEPENDENCY *arg1 = (struct _dwg_object_ASSOCDEPENDENCY *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDEPENDENCY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDEPENDENCY_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDEPENDENCY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCDEPENDENCY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDEPENDENCY_parent_get) {
  {
    struct _dwg_object_ASSOCDEPENDENCY *arg1 = (struct _dwg_object_ASSOCDEPENDENCY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDEPENDENCY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDEPENDENCY_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDEPENDENCY *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDEPENDENCY_class_version_set) {
  {
    struct _dwg_object_ASSOCDEPENDENCY *arg1 = (struct _dwg_object_ASSOCDEPENDENCY *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDEPENDENCY_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDEPENDENCY_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDEPENDENCY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCDEPENDENCY_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDEPENDENCY_class_version_get) {
  {
    struct _dwg_object_ASSOCDEPENDENCY *arg1 = (struct _dwg_object_ASSOCDEPENDENCY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDEPENDENCY_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDEPENDENCY_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDEPENDENCY *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDEPENDENCY_status_set) {
  {
    struct _dwg_object_ASSOCDEPENDENCY *arg1 = (struct _dwg_object_ASSOCDEPENDENCY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDEPENDENCY_status_set(self,status);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDEPENDENCY_status_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDEPENDENCY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCDEPENDENCY_status_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->status = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDEPENDENCY_status_get) {
  {
    struct _dwg_object_ASSOCDEPENDENCY *arg1 = (struct _dwg_object_ASSOCDEPENDENCY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDEPENDENCY_status_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDEPENDENCY_status_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDEPENDENCY *)(argp1);
    result = (BITCODE_BL) ((arg1)->status);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDEPENDENCY_is_read_dep_set) {
  {
    struct _dwg_object_ASSOCDEPENDENCY *arg1 = (struct _dwg_object_ASSOCDEPENDENCY *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDEPENDENCY_is_read_dep_set(self,is_read_dep);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDEPENDENCY_is_read_dep_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDEPENDENCY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCDEPENDENCY_is_read_dep_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_read_dep = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDEPENDENCY_is_read_dep_get) {
  {
    struct _dwg_object_ASSOCDEPENDENCY *arg1 = (struct _dwg_object_ASSOCDEPENDENCY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDEPENDENCY_is_read_dep_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDEPENDENCY_is_read_dep_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDEPENDENCY *)(argp1);
    result = (BITCODE_B) ((arg1)->is_read_dep);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDEPENDENCY_is_write_dep_set) {
  {
    struct _dwg_object_ASSOCDEPENDENCY *arg1 = (struct _dwg_object_ASSOCDEPENDENCY *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDEPENDENCY_is_write_dep_set(self,is_write_dep);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDEPENDENCY_is_write_dep_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDEPENDENCY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCDEPENDENCY_is_write_dep_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_write_dep = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDEPENDENCY_is_write_dep_get) {
  {
    struct _dwg_object_ASSOCDEPENDENCY *arg1 = (struct _dwg_object_ASSOCDEPENDENCY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDEPENDENCY_is_write_dep_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDEPENDENCY_is_write_dep_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDEPENDENCY *)(argp1);
    result = (BITCODE_B) ((arg1)->is_write_dep);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDEPENDENCY_is_attached_to_object_set) {
  {
    struct _dwg_object_ASSOCDEPENDENCY *arg1 = (struct _dwg_object_ASSOCDEPENDENCY *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDEPENDENCY_is_attached_to_object_set(self,is_attached_to_object);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDEPENDENCY_is_attached_to_object_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDEPENDENCY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCDEPENDENCY_is_attached_to_object_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_attached_to_object = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDEPENDENCY_is_attached_to_object_get) {
  {
    struct _dwg_object_ASSOCDEPENDENCY *arg1 = (struct _dwg_object_ASSOCDEPENDENCY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDEPENDENCY_is_attached_to_object_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDEPENDENCY_is_attached_to_object_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDEPENDENCY *)(argp1);
    result = (BITCODE_B) ((arg1)->is_attached_to_object);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDEPENDENCY_is_delegating_to_owning_action_set) {
  {
    struct _dwg_object_ASSOCDEPENDENCY *arg1 = (struct _dwg_object_ASSOCDEPENDENCY *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDEPENDENCY_is_delegating_to_owning_action_set(self,is_delegating_to_owning_action);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDEPENDENCY_is_delegating_to_owning_action_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDEPENDENCY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCDEPENDENCY_is_delegating_to_owning_action_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_delegating_to_owning_action = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDEPENDENCY_is_delegating_to_owning_action_get) {
  {
    struct _dwg_object_ASSOCDEPENDENCY *arg1 = (struct _dwg_object_ASSOCDEPENDENCY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDEPENDENCY_is_delegating_to_owning_action_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDEPENDENCY_is_delegating_to_owning_action_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDEPENDENCY *)(argp1);
    result = (BITCODE_B) ((arg1)->is_delegating_to_owning_action);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDEPENDENCY_order_set) {
  {
    struct _dwg_object_ASSOCDEPENDENCY *arg1 = (struct _dwg_object_ASSOCDEPENDENCY *) 0 ;
    BITCODE_BLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDEPENDENCY_order_set(self,order);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDEPENDENCY_order_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDEPENDENCY *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCDEPENDENCY_order_set" "', argument " "2"" of type '" "BITCODE_BLd""'");
    } 
    arg2 = (BITCODE_BLd)(val2);
    if (arg1) (arg1)->order = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDEPENDENCY_order_get) {
  {
    struct _dwg_object_ASSOCDEPENDENCY *arg1 = (struct _dwg_object_ASSOCDEPENDENCY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDEPENDENCY_order_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDEPENDENCY_order_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDEPENDENCY *)(argp1);
    result = (BITCODE_BLd) ((arg1)->order);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDEPENDENCY_dep_on_set) {
  {
    struct _dwg_object_ASSOCDEPENDENCY *arg1 = (struct _dwg_object_ASSOCDEPENDENCY *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDEPENDENCY_dep_on_set(self,dep_on);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDEPENDENCY_dep_on_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDEPENDENCY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCDEPENDENCY_dep_on_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->dep_on = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDEPENDENCY_dep_on_get) {
  {
    struct _dwg_object_ASSOCDEPENDENCY *arg1 = (struct _dwg_object_ASSOCDEPENDENCY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDEPENDENCY_dep_on_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDEPENDENCY_dep_on_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDEPENDENCY *)(argp1);
    result = (BITCODE_H) ((arg1)->dep_on);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDEPENDENCY_has_name_set) {
  {
    struct _dwg_object_ASSOCDEPENDENCY *arg1 = (struct _dwg_object_ASSOCDEPENDENCY *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDEPENDENCY_has_name_set(self,has_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDEPENDENCY_has_name_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDEPENDENCY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCDEPENDENCY_has_name_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_name = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDEPENDENCY_has_name_get) {
  {
    struct _dwg_object_ASSOCDEPENDENCY *arg1 = (struct _dwg_object_ASSOCDEPENDENCY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDEPENDENCY_has_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDEPENDENCY_has_name_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDEPENDENCY *)(argp1);
    result = (BITCODE_B) ((arg1)->has_name);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDEPENDENCY_name_set) {
  {
    struct _dwg_object_ASSOCDEPENDENCY *arg1 = (struct _dwg_object_ASSOCDEPENDENCY *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDEPENDENCY_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDEPENDENCY_name_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDEPENDENCY *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCDEPENDENCY_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDEPENDENCY_name_get) {
  {
    struct _dwg_object_ASSOCDEPENDENCY *arg1 = (struct _dwg_object_ASSOCDEPENDENCY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDEPENDENCY_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDEPENDENCY_name_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDEPENDENCY *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDEPENDENCY_depbodyid_set) {
  {
    struct _dwg_object_ASSOCDEPENDENCY *arg1 = (struct _dwg_object_ASSOCDEPENDENCY *) 0 ;
    BITCODE_BLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDEPENDENCY_depbodyid_set(self,depbodyid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDEPENDENCY_depbodyid_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDEPENDENCY *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCDEPENDENCY_depbodyid_set" "', argument " "2"" of type '" "BITCODE_BLd""'");
    } 
    arg2 = (BITCODE_BLd)(val2);
    if (arg1) (arg1)->depbodyid = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDEPENDENCY_depbodyid_get) {
  {
    struct _dwg_object_ASSOCDEPENDENCY *arg1 = (struct _dwg_object_ASSOCDEPENDENCY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDEPENDENCY_depbodyid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDEPENDENCY_depbodyid_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDEPENDENCY *)(argp1);
    result = (BITCODE_BLd) ((arg1)->depbodyid);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDEPENDENCY_readdep_set) {
  {
    struct _dwg_object_ASSOCDEPENDENCY *arg1 = (struct _dwg_object_ASSOCDEPENDENCY *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDEPENDENCY_readdep_set(self,readdep);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDEPENDENCY_readdep_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDEPENDENCY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCDEPENDENCY_readdep_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->readdep = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDEPENDENCY_readdep_get) {
  {
    struct _dwg_object_ASSOCDEPENDENCY *arg1 = (struct _dwg_object_ASSOCDEPENDENCY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDEPENDENCY_readdep_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDEPENDENCY_readdep_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDEPENDENCY *)(argp1);
    result = (BITCODE_H) ((arg1)->readdep);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDEPENDENCY_dep_body_set) {
  {
    struct _dwg_object_ASSOCDEPENDENCY *arg1 = (struct _dwg_object_ASSOCDEPENDENCY *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDEPENDENCY_dep_body_set(self,dep_body);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDEPENDENCY_dep_body_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDEPENDENCY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCDEPENDENCY_dep_body_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->dep_body = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDEPENDENCY_dep_body_get) {
  {
    struct _dwg_object_ASSOCDEPENDENCY *arg1 = (struct _dwg_object_ASSOCDEPENDENCY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDEPENDENCY_dep_body_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDEPENDENCY_dep_body_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDEPENDENCY *)(argp1);
    result = (BITCODE_H) ((arg1)->dep_body);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDEPENDENCY_node_set) {
  {
    struct _dwg_object_ASSOCDEPENDENCY *arg1 = (struct _dwg_object_ASSOCDEPENDENCY *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDEPENDENCY_node_set(self,node);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDEPENDENCY_node_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDEPENDENCY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCDEPENDENCY_node_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->node = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDEPENDENCY_node_get) {
  {
    struct _dwg_object_ASSOCDEPENDENCY *arg1 = (struct _dwg_object_ASSOCDEPENDENCY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDEPENDENCY_node_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDEPENDENCY_node_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDEPENDENCY *)(argp1);
    result = (BITCODE_H) ((arg1)->node);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCDEPENDENCY) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCDEPENDENCY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCDEPENDENCY();");
    }
    result = (struct _dwg_object_ASSOCDEPENDENCY *)calloc(1, sizeof(struct _dwg_object_ASSOCDEPENDENCY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCDEPENDENCY) {
  {
    struct _dwg_object_ASSOCDEPENDENCY *arg1 = (struct _dwg_object_ASSOCDEPENDENCY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCDEPENDENCY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCDEPENDENCY" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDEPENDENCY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVALUEDEPENDENCY_parent_set) {
  {
    struct _dwg_object_ASSOCVALUEDEPENDENCY *arg1 = (struct _dwg_object_ASSOCVALUEDEPENDENCY *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVALUEDEPENDENCY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVALUEDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVALUEDEPENDENCY_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVALUEDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVALUEDEPENDENCY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCVALUEDEPENDENCY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVALUEDEPENDENCY_parent_get) {
  {
    struct _dwg_object_ASSOCVALUEDEPENDENCY *arg1 = (struct _dwg_object_ASSOCVALUEDEPENDENCY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVALUEDEPENDENCY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVALUEDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVALUEDEPENDENCY_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVALUEDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVALUEDEPENDENCY *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVALUEDEPENDENCY_assocdep_set) {
  {
    struct _dwg_object_ASSOCVALUEDEPENDENCY *arg1 = (struct _dwg_object_ASSOCVALUEDEPENDENCY *) 0 ;
    Dwg_Object_ASSOCDEPENDENCY *arg2 = (Dwg_Object_ASSOCDEPENDENCY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVALUEDEPENDENCY_assocdep_set(self,assocdep);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVALUEDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVALUEDEPENDENCY_assocdep_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVALUEDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVALUEDEPENDENCY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCVALUEDEPENDENCY_assocdep_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCDEPENDENCY *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCDEPENDENCY *)(argp2);
    if (arg1) (arg1)->assocdep = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVALUEDEPENDENCY_assocdep_get) {
  {
    struct _dwg_object_ASSOCVALUEDEPENDENCY *arg1 = (struct _dwg_object_ASSOCVALUEDEPENDENCY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCDEPENDENCY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVALUEDEPENDENCY_assocdep_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVALUEDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVALUEDEPENDENCY_assocdep_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVALUEDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVALUEDEPENDENCY *)(argp1);
    result = (Dwg_Object_ASSOCDEPENDENCY *)& ((arg1)->assocdep);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCVALUEDEPENDENCY) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCVALUEDEPENDENCY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCVALUEDEPENDENCY();");
    }
    result = (struct _dwg_object_ASSOCVALUEDEPENDENCY *)calloc(1, sizeof(struct _dwg_object_ASSOCVALUEDEPENDENCY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCVALUEDEPENDENCY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCVALUEDEPENDENCY) {
  {
    struct _dwg_object_ASSOCVALUEDEPENDENCY *arg1 = (struct _dwg_object_ASSOCVALUEDEPENDENCY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCVALUEDEPENDENCY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVALUEDEPENDENCY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCVALUEDEPENDENCY" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVALUEDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVALUEDEPENDENCY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCGEOMDEPENDENCY_parent_set) {
  {
    struct _dwg_object_ASSOCGEOMDEPENDENCY *arg1 = (struct _dwg_object_ASSOCGEOMDEPENDENCY *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCGEOMDEPENDENCY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCGEOMDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCGEOMDEPENDENCY_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCGEOMDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCGEOMDEPENDENCY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCGEOMDEPENDENCY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCGEOMDEPENDENCY_parent_get) {
  {
    struct _dwg_object_ASSOCGEOMDEPENDENCY *arg1 = (struct _dwg_object_ASSOCGEOMDEPENDENCY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCGEOMDEPENDENCY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCGEOMDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCGEOMDEPENDENCY_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCGEOMDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCGEOMDEPENDENCY *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCGEOMDEPENDENCY_assocdep_set) {
  {
    struct _dwg_object_ASSOCGEOMDEPENDENCY *arg1 = (struct _dwg_object_ASSOCGEOMDEPENDENCY *) 0 ;
    Dwg_Object_ASSOCDEPENDENCY *arg2 = (Dwg_Object_ASSOCDEPENDENCY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCGEOMDEPENDENCY_assocdep_set(self,assocdep);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCGEOMDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCGEOMDEPENDENCY_assocdep_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCGEOMDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCGEOMDEPENDENCY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCGEOMDEPENDENCY_assocdep_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCDEPENDENCY *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCDEPENDENCY *)(argp2);
    if (arg1) (arg1)->assocdep = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCGEOMDEPENDENCY_assocdep_get) {
  {
    struct _dwg_object_ASSOCGEOMDEPENDENCY *arg1 = (struct _dwg_object_ASSOCGEOMDEPENDENCY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCDEPENDENCY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCGEOMDEPENDENCY_assocdep_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCGEOMDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCGEOMDEPENDENCY_assocdep_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCGEOMDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCGEOMDEPENDENCY *)(argp1);
    result = (Dwg_Object_ASSOCDEPENDENCY *)& ((arg1)->assocdep);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCGEOMDEPENDENCY_class_version_set) {
  {
    struct _dwg_object_ASSOCGEOMDEPENDENCY *arg1 = (struct _dwg_object_ASSOCGEOMDEPENDENCY *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCGEOMDEPENDENCY_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCGEOMDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCGEOMDEPENDENCY_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCGEOMDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCGEOMDEPENDENCY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCGEOMDEPENDENCY_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCGEOMDEPENDENCY_class_version_get) {
  {
    struct _dwg_object_ASSOCGEOMDEPENDENCY *arg1 = (struct _dwg_object_ASSOCGEOMDEPENDENCY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCGEOMDEPENDENCY_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCGEOMDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCGEOMDEPENDENCY_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCGEOMDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCGEOMDEPENDENCY *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCGEOMDEPENDENCY_enabled_set) {
  {
    struct _dwg_object_ASSOCGEOMDEPENDENCY *arg1 = (struct _dwg_object_ASSOCGEOMDEPENDENCY *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCGEOMDEPENDENCY_enabled_set(self,enabled);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCGEOMDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCGEOMDEPENDENCY_enabled_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCGEOMDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCGEOMDEPENDENCY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCGEOMDEPENDENCY_enabled_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->enabled = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCGEOMDEPENDENCY_enabled_get) {
  {
    struct _dwg_object_ASSOCGEOMDEPENDENCY *arg1 = (struct _dwg_object_ASSOCGEOMDEPENDENCY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCGEOMDEPENDENCY_enabled_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCGEOMDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCGEOMDEPENDENCY_enabled_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCGEOMDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCGEOMDEPENDENCY *)(argp1);
    result = (BITCODE_B) ((arg1)->enabled);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCGEOMDEPENDENCY_classname_set) {
  {
    struct _dwg_object_ASSOCGEOMDEPENDENCY *arg1 = (struct _dwg_object_ASSOCGEOMDEPENDENCY *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCGEOMDEPENDENCY_classname_set(self,classname);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCGEOMDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCGEOMDEPENDENCY_classname_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCGEOMDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCGEOMDEPENDENCY *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCGEOMDEPENDENCY_classname_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->classname);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->classname = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->classname = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCGEOMDEPENDENCY_classname_get) {
  {
    struct _dwg_object_ASSOCGEOMDEPENDENCY *arg1 = (struct _dwg_object_ASSOCGEOMDEPENDENCY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCGEOMDEPENDENCY_classname_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCGEOMDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCGEOMDEPENDENCY_classname_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCGEOMDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCGEOMDEPENDENCY *)(argp1);
    result = (BITCODE_TV) ((arg1)->classname);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCGEOMDEPENDENCY_dependent_on_compound_object_set) {
  {
    struct _dwg_object_ASSOCGEOMDEPENDENCY *arg1 = (struct _dwg_object_ASSOCGEOMDEPENDENCY *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCGEOMDEPENDENCY_dependent_on_compound_object_set(self,dependent_on_compound_object);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCGEOMDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCGEOMDEPENDENCY_dependent_on_compound_object_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCGEOMDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCGEOMDEPENDENCY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCGEOMDEPENDENCY_dependent_on_compound_object_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->dependent_on_compound_object = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCGEOMDEPENDENCY_dependent_on_compound_object_get) {
  {
    struct _dwg_object_ASSOCGEOMDEPENDENCY *arg1 = (struct _dwg_object_ASSOCGEOMDEPENDENCY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCGEOMDEPENDENCY_dependent_on_compound_object_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCGEOMDEPENDENCY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCGEOMDEPENDENCY_dependent_on_compound_object_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCGEOMDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCGEOMDEPENDENCY *)(argp1);
    result = (BITCODE_B) ((arg1)->dependent_on_compound_object);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCGEOMDEPENDENCY) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCGEOMDEPENDENCY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCGEOMDEPENDENCY();");
    }
    result = (struct _dwg_object_ASSOCGEOMDEPENDENCY *)calloc(1, sizeof(struct _dwg_object_ASSOCGEOMDEPENDENCY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCGEOMDEPENDENCY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCGEOMDEPENDENCY) {
  {
    struct _dwg_object_ASSOCGEOMDEPENDENCY *arg1 = (struct _dwg_object_ASSOCGEOMDEPENDENCY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCGEOMDEPENDENCY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCGEOMDEPENDENCY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCGEOMDEPENDENCY" "', argument " "1"" of type '" "struct _dwg_object_ASSOCGEOMDEPENDENCY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCGEOMDEPENDENCY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCACTION_parent_set) {
  {
    struct _dwg_object_ASSOCACTION *arg1 = (struct _dwg_object_ASSOCACTION *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCACTION_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCACTION_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTION *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCACTION_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCACTION_parent_get) {
  {
    struct _dwg_object_ASSOCACTION *arg1 = (struct _dwg_object_ASSOCACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCACTION_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCACTION_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTION *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTION *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCACTION_class_version_set) {
  {
    struct _dwg_object_ASSOCACTION *arg1 = (struct _dwg_object_ASSOCACTION *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCACTION_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCACTION_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTION *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCACTION_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCACTION_class_version_get) {
  {
    struct _dwg_object_ASSOCACTION *arg1 = (struct _dwg_object_ASSOCACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCACTION_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCACTION_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTION *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTION *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCACTION_geometry_status_set) {
  {
    struct _dwg_object_ASSOCACTION *arg1 = (struct _dwg_object_ASSOCACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCACTION_geometry_status_set(self,geometry_status);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCACTION_geometry_status_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTION *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCACTION_geometry_status_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->geometry_status = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCACTION_geometry_status_get) {
  {
    struct _dwg_object_ASSOCACTION *arg1 = (struct _dwg_object_ASSOCACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCACTION_geometry_status_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCACTION_geometry_status_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTION *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->geometry_status);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCACTION_owningnetwork_set) {
  {
    struct _dwg_object_ASSOCACTION *arg1 = (struct _dwg_object_ASSOCACTION *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCACTION_owningnetwork_set(self,owningnetwork);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCACTION_owningnetwork_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTION *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCACTION_owningnetwork_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->owningnetwork = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCACTION_owningnetwork_get) {
  {
    struct _dwg_object_ASSOCACTION *arg1 = (struct _dwg_object_ASSOCACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCACTION_owningnetwork_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCACTION_owningnetwork_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTION *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTION *)(argp1);
    result = (BITCODE_H) ((arg1)->owningnetwork);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCACTION_actionbody_set) {
  {
    struct _dwg_object_ASSOCACTION *arg1 = (struct _dwg_object_ASSOCACTION *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCACTION_actionbody_set(self,actionbody);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCACTION_actionbody_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTION *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCACTION_actionbody_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->actionbody = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCACTION_actionbody_get) {
  {
    struct _dwg_object_ASSOCACTION *arg1 = (struct _dwg_object_ASSOCACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCACTION_actionbody_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCACTION_actionbody_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTION *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTION *)(argp1);
    result = (BITCODE_H) ((arg1)->actionbody);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCACTION_action_index_set) {
  {
    struct _dwg_object_ASSOCACTION *arg1 = (struct _dwg_object_ASSOCACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCACTION_action_index_set(self,action_index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCACTION_action_index_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTION *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCACTION_action_index_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->action_index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCACTION_action_index_get) {
  {
    struct _dwg_object_ASSOCACTION *arg1 = (struct _dwg_object_ASSOCACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCACTION_action_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCACTION_action_index_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTION *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->action_index);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCACTION_max_assoc_dep_index_set) {
  {
    struct _dwg_object_ASSOCACTION *arg1 = (struct _dwg_object_ASSOCACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCACTION_max_assoc_dep_index_set(self,max_assoc_dep_index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCACTION_max_assoc_dep_index_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTION *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCACTION_max_assoc_dep_index_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->max_assoc_dep_index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCACTION_max_assoc_dep_index_get) {
  {
    struct _dwg_object_ASSOCACTION *arg1 = (struct _dwg_object_ASSOCACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCACTION_max_assoc_dep_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCACTION_max_assoc_dep_index_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTION *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->max_assoc_dep_index);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCACTION_num_deps_set) {
  {
    struct _dwg_object_ASSOCACTION *arg1 = (struct _dwg_object_ASSOCACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCACTION_num_deps_set(self,num_deps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCACTION_num_deps_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTION *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCACTION_num_deps_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_deps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCACTION_num_deps_get) {
  {
    struct _dwg_object_ASSOCACTION *arg1 = (struct _dwg_object_ASSOCACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCACTION_num_deps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCACTION_num_deps_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTION *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_deps);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCACTION_deps_set) {
  {
    struct _dwg_object_ASSOCACTION *arg1 = (struct _dwg_object_ASSOCACTION *) 0 ;
    Dwg_ASSOCACTION_Deps *arg2 = (Dwg_ASSOCACTION_Deps *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCACTION_deps_set(self,deps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCACTION_deps_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTION *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCACTION_Deps, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCACTION_deps_set" "', argument " "2"" of type '" "Dwg_ASSOCACTION_Deps *""'"); 
    }
    arg2 = (Dwg_ASSOCACTION_Deps *)(argp2);
    if (arg1) (arg1)->deps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCACTION_deps_get) {
  {
    struct _dwg_object_ASSOCACTION *arg1 = (struct _dwg_object_ASSOCACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCACTION_Deps *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCACTION_deps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCACTION_deps_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTION *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTION *)(argp1);
    result = (Dwg_ASSOCACTION_Deps *) ((arg1)->deps);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCACTION_Deps, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCACTION_num_owned_params_set) {
  {
    struct _dwg_object_ASSOCACTION *arg1 = (struct _dwg_object_ASSOCACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCACTION_num_owned_params_set(self,num_owned_params);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCACTION_num_owned_params_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTION *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCACTION_num_owned_params_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_owned_params = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCACTION_num_owned_params_get) {
  {
    struct _dwg_object_ASSOCACTION *arg1 = (struct _dwg_object_ASSOCACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCACTION_num_owned_params_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCACTION_num_owned_params_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTION *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_owned_params);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCACTION_owned_params_set) {
  {
    struct _dwg_object_ASSOCACTION *arg1 = (struct _dwg_object_ASSOCACTION *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCACTION_owned_params_set(self,owned_params);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCACTION_owned_params_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTION *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCACTION_owned_params_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->owned_params = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCACTION_owned_params_get) {
  {
    struct _dwg_object_ASSOCACTION *arg1 = (struct _dwg_object_ASSOCACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCACTION_owned_params_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCACTION_owned_params_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTION *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTION *)(argp1);
    result = (BITCODE_H *) ((arg1)->owned_params);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCACTION_num_values_set) {
  {
    struct _dwg_object_ASSOCACTION *arg1 = (struct _dwg_object_ASSOCACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCACTION_num_values_set(self,num_values);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCACTION_num_values_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTION *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCACTION_num_values_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_values = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCACTION_num_values_get) {
  {
    struct _dwg_object_ASSOCACTION *arg1 = (struct _dwg_object_ASSOCACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCACTION_num_values_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCACTION_num_values_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTION *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_values);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCACTION_values_set) {
  {
    struct _dwg_object_ASSOCACTION *arg1 = (struct _dwg_object_ASSOCACTION *) 0 ;
    Dwg_VALUEPARAM *arg2 = (Dwg_VALUEPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCACTION_values_set(self,values);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCACTION_values_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTION *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_VALUEPARAM, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCACTION_values_set" "', argument " "2"" of type '" "Dwg_VALUEPARAM *""'"); 
    }
    arg2 = (Dwg_VALUEPARAM *)(argp2);
    if (arg1) (arg1)->values = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCACTION_values_get) {
  {
    struct _dwg_object_ASSOCACTION *arg1 = (struct _dwg_object_ASSOCACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_VALUEPARAM *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCACTION_values_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCACTION_values_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTION *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTION *)(argp1);
    result = (Dwg_VALUEPARAM *) ((arg1)->values);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_VALUEPARAM, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCACTION) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCACTION *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCACTION();");
    }
    result = (struct _dwg_object_ASSOCACTION *)calloc(1, sizeof(struct _dwg_object_ASSOCACTION));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCACTION, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCACTION) {
  {
    struct _dwg_object_ASSOCACTION *arg1 = (struct _dwg_object_ASSOCACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCACTION(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTION, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCACTION" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTION *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTION *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_parent_set) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCNETWORK_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_parent_get) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_class_version_set) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCNETWORK_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_class_version_get) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_geometry_status_set) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_geometry_status_set(self,geometry_status);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_geometry_status_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCNETWORK_geometry_status_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->geometry_status = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_geometry_status_get) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_geometry_status_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_geometry_status_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    result = (BITCODE_BL) ((arg1)->geometry_status);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_owningnetwork_set) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_owningnetwork_set(self,owningnetwork);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_owningnetwork_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCNETWORK_owningnetwork_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->owningnetwork = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_owningnetwork_get) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_owningnetwork_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_owningnetwork_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    result = (BITCODE_H) ((arg1)->owningnetwork);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_actionbody_set) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_actionbody_set(self,actionbody);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_actionbody_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCNETWORK_actionbody_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->actionbody = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_actionbody_get) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_actionbody_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_actionbody_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    result = (BITCODE_H) ((arg1)->actionbody);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_action_index_set) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_action_index_set(self,action_index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_action_index_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCNETWORK_action_index_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->action_index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_action_index_get) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_action_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_action_index_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    result = (BITCODE_BL) ((arg1)->action_index);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_max_assoc_dep_index_set) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_max_assoc_dep_index_set(self,max_assoc_dep_index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_max_assoc_dep_index_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCNETWORK_max_assoc_dep_index_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->max_assoc_dep_index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_max_assoc_dep_index_get) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_max_assoc_dep_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_max_assoc_dep_index_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    result = (BITCODE_BL) ((arg1)->max_assoc_dep_index);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_num_deps_set) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_num_deps_set(self,num_deps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_num_deps_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCNETWORK_num_deps_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_deps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_num_deps_get) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_num_deps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_num_deps_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_deps);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_deps_set) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    Dwg_ASSOCACTION_Deps *arg2 = (Dwg_ASSOCACTION_Deps *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_deps_set(self,deps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_deps_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCACTION_Deps, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCNETWORK_deps_set" "', argument " "2"" of type '" "Dwg_ASSOCACTION_Deps *""'"); 
    }
    arg2 = (Dwg_ASSOCACTION_Deps *)(argp2);
    if (arg1) (arg1)->deps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_deps_get) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCACTION_Deps *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_deps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_deps_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    result = (Dwg_ASSOCACTION_Deps *) ((arg1)->deps);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCACTION_Deps, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_num_owned_params_set) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_num_owned_params_set(self,num_owned_params);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_num_owned_params_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCNETWORK_num_owned_params_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_owned_params = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_num_owned_params_get) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_num_owned_params_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_num_owned_params_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_owned_params);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_owned_params_set) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_owned_params_set(self,owned_params);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_owned_params_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCNETWORK_owned_params_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->owned_params = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_owned_params_get) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_owned_params_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_owned_params_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    result = (BITCODE_H *) ((arg1)->owned_params);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_num_values_set) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_num_values_set(self,num_values);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_num_values_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCNETWORK_num_values_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_values = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_num_values_get) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_num_values_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_num_values_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_values);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_values_set) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    Dwg_VALUEPARAM *arg2 = (Dwg_VALUEPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_values_set(self,values);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_values_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_VALUEPARAM, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCNETWORK_values_set" "', argument " "2"" of type '" "Dwg_VALUEPARAM *""'"); 
    }
    arg2 = (Dwg_VALUEPARAM *)(argp2);
    if (arg1) (arg1)->values = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_values_get) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_VALUEPARAM *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_values_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_values_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    result = (Dwg_VALUEPARAM *) ((arg1)->values);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_VALUEPARAM, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_network_version_set) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_network_version_set(self,network_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_network_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCNETWORK_network_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->network_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_network_version_get) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_network_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_network_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    result = (BITCODE_BS) ((arg1)->network_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_network_action_index_set) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_network_action_index_set(self,network_action_index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_network_action_index_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCNETWORK_network_action_index_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->network_action_index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_network_action_index_get) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_network_action_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_network_action_index_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    result = (BITCODE_BL) ((arg1)->network_action_index);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_num_actions_set) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_num_actions_set(self,num_actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_num_actions_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCNETWORK_num_actions_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_num_actions_get) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_num_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_num_actions_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_actions);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_actions_set) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    Dwg_ASSOCACTION_Deps *arg2 = (Dwg_ASSOCACTION_Deps *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_actions_set(self,actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_actions_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCACTION_Deps, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCNETWORK_actions_set" "', argument " "2"" of type '" "Dwg_ASSOCACTION_Deps *""'"); 
    }
    arg2 = (Dwg_ASSOCACTION_Deps *)(argp2);
    if (arg1) (arg1)->actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_actions_get) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCACTION_Deps *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_actions_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    result = (Dwg_ASSOCACTION_Deps *) ((arg1)->actions);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCACTION_Deps, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_num_owned_actions_set) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_num_owned_actions_set(self,num_owned_actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_num_owned_actions_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCNETWORK_num_owned_actions_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_owned_actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_num_owned_actions_get) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_num_owned_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_num_owned_actions_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_owned_actions);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_owned_actions_set) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_owned_actions_set(self,owned_actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_owned_actions_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCNETWORK_owned_actions_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->owned_actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORK_owned_actions_get) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORK_owned_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORK_owned_actions_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    result = (BITCODE_H *) ((arg1)->owned_actions);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCNETWORK) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCNETWORK *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCNETWORK();");
    }
    result = (struct _dwg_object_ASSOCNETWORK *)calloc(1, sizeof(struct _dwg_object_ASSOCNETWORK));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCNETWORK, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCNETWORK) {
  {
    struct _dwg_object_ASSOCNETWORK *arg1 = (struct _dwg_object_ASSOCNETWORK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCNETWORK(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORK, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCNETWORK" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORK *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORK *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CONSTRAINTGROUPNODE_parent_set) {
  {
    struct _dwg_CONSTRAINTGROUPNODE *arg1 = (struct _dwg_CONSTRAINTGROUPNODE *) 0 ;
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg2 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CONSTRAINTGROUPNODE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CONSTRAINTGROUPNODE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CONSTRAINTGROUPNODE_parent_set" "', argument " "1"" of type '" "struct _dwg_CONSTRAINTGROUPNODE *""'"); 
    }
    arg1 = (struct _dwg_CONSTRAINTGROUPNODE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_CONSTRAINTGROUPNODE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg2 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CONSTRAINTGROUPNODE_parent_get) {
  {
    struct _dwg_CONSTRAINTGROUPNODE *arg1 = (struct _dwg_CONSTRAINTGROUPNODE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CONSTRAINTGROUPNODE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CONSTRAINTGROUPNODE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CONSTRAINTGROUPNODE_parent_get" "', argument " "1"" of type '" "struct _dwg_CONSTRAINTGROUPNODE *""'"); 
    }
    arg1 = (struct _dwg_CONSTRAINTGROUPNODE *)(argp1);
    result = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CONSTRAINTGROUPNODE_nodeid_set) {
  {
    struct _dwg_CONSTRAINTGROUPNODE *arg1 = (struct _dwg_CONSTRAINTGROUPNODE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CONSTRAINTGROUPNODE_nodeid_set(self,nodeid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CONSTRAINTGROUPNODE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CONSTRAINTGROUPNODE_nodeid_set" "', argument " "1"" of type '" "struct _dwg_CONSTRAINTGROUPNODE *""'"); 
    }
    arg1 = (struct _dwg_CONSTRAINTGROUPNODE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_CONSTRAINTGROUPNODE_nodeid_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->nodeid = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CONSTRAINTGROUPNODE_nodeid_get) {
  {
    struct _dwg_CONSTRAINTGROUPNODE *arg1 = (struct _dwg_CONSTRAINTGROUPNODE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CONSTRAINTGROUPNODE_nodeid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CONSTRAINTGROUPNODE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CONSTRAINTGROUPNODE_nodeid_get" "', argument " "1"" of type '" "struct _dwg_CONSTRAINTGROUPNODE *""'"); 
    }
    arg1 = (struct _dwg_CONSTRAINTGROUPNODE *)(argp1);
    result = (BITCODE_BL) ((arg1)->nodeid);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CONSTRAINTGROUPNODE_status_set) {
  {
    struct _dwg_CONSTRAINTGROUPNODE *arg1 = (struct _dwg_CONSTRAINTGROUPNODE *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CONSTRAINTGROUPNODE_status_set(self,status);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CONSTRAINTGROUPNODE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CONSTRAINTGROUPNODE_status_set" "', argument " "1"" of type '" "struct _dwg_CONSTRAINTGROUPNODE *""'"); 
    }
    arg1 = (struct _dwg_CONSTRAINTGROUPNODE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_CONSTRAINTGROUPNODE_status_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->status = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CONSTRAINTGROUPNODE_status_get) {
  {
    struct _dwg_CONSTRAINTGROUPNODE *arg1 = (struct _dwg_CONSTRAINTGROUPNODE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CONSTRAINTGROUPNODE_status_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CONSTRAINTGROUPNODE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CONSTRAINTGROUPNODE_status_get" "', argument " "1"" of type '" "struct _dwg_CONSTRAINTGROUPNODE *""'"); 
    }
    arg1 = (struct _dwg_CONSTRAINTGROUPNODE *)(argp1);
    result = (BITCODE_RC) ((arg1)->status);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CONSTRAINTGROUPNODE_num_connections_set) {
  {
    struct _dwg_CONSTRAINTGROUPNODE *arg1 = (struct _dwg_CONSTRAINTGROUPNODE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CONSTRAINTGROUPNODE_num_connections_set(self,num_connections);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CONSTRAINTGROUPNODE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CONSTRAINTGROUPNODE_num_connections_set" "', argument " "1"" of type '" "struct _dwg_CONSTRAINTGROUPNODE *""'"); 
    }
    arg1 = (struct _dwg_CONSTRAINTGROUPNODE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_CONSTRAINTGROUPNODE_num_connections_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_connections = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CONSTRAINTGROUPNODE_num_connections_get) {
  {
    struct _dwg_CONSTRAINTGROUPNODE *arg1 = (struct _dwg_CONSTRAINTGROUPNODE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CONSTRAINTGROUPNODE_num_connections_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CONSTRAINTGROUPNODE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CONSTRAINTGROUPNODE_num_connections_get" "', argument " "1"" of type '" "struct _dwg_CONSTRAINTGROUPNODE *""'"); 
    }
    arg1 = (struct _dwg_CONSTRAINTGROUPNODE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_connections);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CONSTRAINTGROUPNODE_connections_set) {
  {
    struct _dwg_CONSTRAINTGROUPNODE *arg1 = (struct _dwg_CONSTRAINTGROUPNODE *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CONSTRAINTGROUPNODE_connections_set(self,connections);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CONSTRAINTGROUPNODE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CONSTRAINTGROUPNODE_connections_set" "', argument " "1"" of type '" "struct _dwg_CONSTRAINTGROUPNODE *""'"); 
    }
    arg1 = (struct _dwg_CONSTRAINTGROUPNODE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_CONSTRAINTGROUPNODE_connections_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->connections = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CONSTRAINTGROUPNODE_connections_get) {
  {
    struct _dwg_CONSTRAINTGROUPNODE *arg1 = (struct _dwg_CONSTRAINTGROUPNODE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CONSTRAINTGROUPNODE_connections_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CONSTRAINTGROUPNODE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CONSTRAINTGROUPNODE_connections_get" "', argument " "1"" of type '" "struct _dwg_CONSTRAINTGROUPNODE *""'"); 
    }
    arg1 = (struct _dwg_CONSTRAINTGROUPNODE *)(argp1);
    result = (BITCODE_BL *) ((arg1)->connections);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_CONSTRAINTGROUPNODE) {
  {
    int argvi = 0;
    struct _dwg_CONSTRAINTGROUPNODE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_CONSTRAINTGROUPNODE();");
    }
    result = (struct _dwg_CONSTRAINTGROUPNODE *)calloc(1, sizeof(struct _dwg_CONSTRAINTGROUPNODE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_CONSTRAINTGROUPNODE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_CONSTRAINTGROUPNODE) {
  {
    struct _dwg_CONSTRAINTGROUPNODE *arg1 = (struct _dwg_CONSTRAINTGROUPNODE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_CONSTRAINTGROUPNODE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CONSTRAINTGROUPNODE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_CONSTRAINTGROUPNODE" "', argument " "1"" of type '" "struct _dwg_CONSTRAINTGROUPNODE *""'"); 
    }
    arg1 = (struct _dwg_CONSTRAINTGROUPNODE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_parent_set) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_parent_get) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_class_version_set) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_class_version_get) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_geometry_status_set) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_geometry_status_set(self,geometry_status);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_geometry_status_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_geometry_status_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->geometry_status = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_geometry_status_get) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_geometry_status_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_geometry_status_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    result = (BITCODE_BL) ((arg1)->geometry_status);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_owningnetwork_set) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_owningnetwork_set(self,owningnetwork);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_owningnetwork_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_owningnetwork_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->owningnetwork = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_owningnetwork_get) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_owningnetwork_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_owningnetwork_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    result = (BITCODE_H) ((arg1)->owningnetwork);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_actionbody_set) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_actionbody_set(self,actionbody);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_actionbody_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_actionbody_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->actionbody = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_actionbody_get) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_actionbody_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_actionbody_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    result = (BITCODE_H) ((arg1)->actionbody);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_action_index_set) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_action_index_set(self,action_index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_action_index_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_action_index_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->action_index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_action_index_get) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_action_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_action_index_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    result = (BITCODE_BL) ((arg1)->action_index);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_max_assoc_dep_index_set) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_max_assoc_dep_index_set(self,max_assoc_dep_index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_max_assoc_dep_index_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_max_assoc_dep_index_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->max_assoc_dep_index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_max_assoc_dep_index_get) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_max_assoc_dep_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_max_assoc_dep_index_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    result = (BITCODE_BL) ((arg1)->max_assoc_dep_index);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_deps_set) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_deps_set(self,num_deps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_deps_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_deps_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_deps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_deps_get) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_deps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_deps_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_deps);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_deps_set) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    Dwg_ASSOCACTION_Deps *arg2 = (Dwg_ASSOCACTION_Deps *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_deps_set(self,deps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_deps_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCACTION_Deps, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_deps_set" "', argument " "2"" of type '" "Dwg_ASSOCACTION_Deps *""'"); 
    }
    arg2 = (Dwg_ASSOCACTION_Deps *)(argp2);
    if (arg1) (arg1)->deps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_deps_get) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCACTION_Deps *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_deps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_deps_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    result = (Dwg_ASSOCACTION_Deps *) ((arg1)->deps);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCACTION_Deps, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_owned_params_set) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_owned_params_set(self,num_owned_params);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_owned_params_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_owned_params_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_owned_params = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_owned_params_get) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_owned_params_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_owned_params_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_owned_params);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_owned_params_set) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_owned_params_set(self,owned_params);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_owned_params_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_owned_params_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->owned_params = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_owned_params_get) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_owned_params_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_owned_params_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    result = (BITCODE_H *) ((arg1)->owned_params);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_values_set) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_values_set(self,num_values);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_values_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_values_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_values = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_values_get) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_values_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_values_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_values);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_values_set) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    Dwg_VALUEPARAM *arg2 = (Dwg_VALUEPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_values_set(self,values);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_values_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_VALUEPARAM, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_values_set" "', argument " "2"" of type '" "Dwg_VALUEPARAM *""'"); 
    }
    arg2 = (Dwg_VALUEPARAM *)(argp2);
    if (arg1) (arg1)->values = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_values_get) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_VALUEPARAM *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_values_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_values_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    result = (Dwg_VALUEPARAM *) ((arg1)->values);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_VALUEPARAM, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_version_set) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_version_set(self,version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_version_get) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    result = (BITCODE_BL) ((arg1)->version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_b1_set) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_b1_set(self,b1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_b1_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_b1_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->b1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_b1_get) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_b1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_b1_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    result = (BITCODE_B) ((arg1)->b1);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_workplane_set) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) (BITCODE_3BD *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_workplane_set(self,workplane);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_workplane_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_workplane_set" "', argument " "2"" of type '" "BITCODE_3BD [3]""'"); 
    } 
    arg2 = (BITCODE_3BD *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)3; ++ii) *(BITCODE_3BD *)&arg1->workplane[ii] = *((BITCODE_3BD *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""workplane""' of type '""BITCODE_3BD [3]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_workplane_get) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_workplane_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_workplane_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    result = (BITCODE_3BD *)(BITCODE_3BD *) ((arg1)->workplane);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_h1_set) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_h1_set(self,h1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_h1_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_h1_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->h1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_h1_get) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_h1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_h1_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    result = (BITCODE_H) ((arg1)->h1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_actions_set) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_actions_set(self,num_actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_actions_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_actions_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_actions_get) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_actions_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_actions);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_actions_set) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_actions_set(self,actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_actions_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_actions_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_actions_get) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_actions_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    result = (BITCODE_H *) ((arg1)->actions);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_nodes_set) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_nodes_set(self,num_nodes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_nodes_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_nodes_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_nodes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_nodes_get) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_nodes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_nodes_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_nodes);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_nodes_set) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    Dwg_CONSTRAINTGROUPNODE *arg2 = (Dwg_CONSTRAINTGROUPNODE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_nodes_set(self,nodes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_nodes_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_CONSTRAINTGROUPNODE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_nodes_set" "', argument " "2"" of type '" "Dwg_CONSTRAINTGROUPNODE *""'"); 
    }
    arg2 = (Dwg_CONSTRAINTGROUPNODE *)(argp2);
    if (arg1) (arg1)->nodes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_nodes_get) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_CONSTRAINTGROUPNODE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC2DCONSTRAINTGROUP_nodes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP_nodes_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    result = (Dwg_CONSTRAINTGROUPNODE *) ((arg1)->nodes);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_CONSTRAINTGROUPNODE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOC2DCONSTRAINTGROUP) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOC2DCONSTRAINTGROUP();");
    }
    result = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)calloc(1, sizeof(struct _dwg_object_ASSOC2DCONSTRAINTGROUP));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOC2DCONSTRAINTGROUP) {
  {
    struct _dwg_object_ASSOC2DCONSTRAINTGROUP *arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOC2DCONSTRAINTGROUP(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOC2DCONSTRAINTGROUP" "', argument " "1"" of type '" "struct _dwg_object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC2DCONSTRAINTGROUP *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_parent_set) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCVARIABLE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_parent_get) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_av_class_version_set) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_av_class_version_set(self,av_class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_av_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCVARIABLE_av_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->av_class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_av_class_version_get) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_av_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_av_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->av_class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_class_version_set) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCVARIABLE_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_class_version_get) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_geometry_status_set) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_geometry_status_set(self,geometry_status);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_geometry_status_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCVARIABLE_geometry_status_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->geometry_status = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_geometry_status_get) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_geometry_status_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_geometry_status_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->geometry_status);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_owningnetwork_set) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_owningnetwork_set(self,owningnetwork);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_owningnetwork_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCVARIABLE_owningnetwork_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->owningnetwork = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_owningnetwork_get) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_owningnetwork_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_owningnetwork_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    result = (BITCODE_H) ((arg1)->owningnetwork);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_actionbody_set) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_actionbody_set(self,actionbody);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_actionbody_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCVARIABLE_actionbody_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->actionbody = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_actionbody_get) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_actionbody_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_actionbody_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    result = (BITCODE_H) ((arg1)->actionbody);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_action_index_set) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_action_index_set(self,action_index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_action_index_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCVARIABLE_action_index_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->action_index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_action_index_get) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_action_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_action_index_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->action_index);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_max_assoc_dep_index_set) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_max_assoc_dep_index_set(self,max_assoc_dep_index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_max_assoc_dep_index_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCVARIABLE_max_assoc_dep_index_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->max_assoc_dep_index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_max_assoc_dep_index_get) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_max_assoc_dep_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_max_assoc_dep_index_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->max_assoc_dep_index);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_num_deps_set) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_num_deps_set(self,num_deps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_num_deps_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCVARIABLE_num_deps_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_deps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_num_deps_get) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_num_deps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_num_deps_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_deps);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_deps_set) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    Dwg_ASSOCACTION_Deps *arg2 = (Dwg_ASSOCACTION_Deps *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_deps_set(self,deps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_deps_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCACTION_Deps, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCVARIABLE_deps_set" "', argument " "2"" of type '" "Dwg_ASSOCACTION_Deps *""'"); 
    }
    arg2 = (Dwg_ASSOCACTION_Deps *)(argp2);
    if (arg1) (arg1)->deps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_deps_get) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCACTION_Deps *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_deps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_deps_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    result = (Dwg_ASSOCACTION_Deps *) ((arg1)->deps);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCACTION_Deps, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_num_owned_params_set) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_num_owned_params_set(self,num_owned_params);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_num_owned_params_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCVARIABLE_num_owned_params_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_owned_params = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_num_owned_params_get) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_num_owned_params_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_num_owned_params_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_owned_params);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_owned_params_set) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_owned_params_set(self,owned_params);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_owned_params_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCVARIABLE_owned_params_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->owned_params = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_owned_params_get) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_owned_params_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_owned_params_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    result = (BITCODE_H *) ((arg1)->owned_params);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_num_values_set) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_num_values_set(self,num_values);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_num_values_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCVARIABLE_num_values_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_values = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_num_values_get) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_num_values_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_num_values_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_values);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_values_set) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    Dwg_VALUEPARAM *arg2 = (Dwg_VALUEPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_values_set(self,values);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_values_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_VALUEPARAM, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCVARIABLE_values_set" "', argument " "2"" of type '" "Dwg_VALUEPARAM *""'"); 
    }
    arg2 = (Dwg_VALUEPARAM *)(argp2);
    if (arg1) (arg1)->values = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_values_get) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_VALUEPARAM *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_values_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_values_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    result = (Dwg_VALUEPARAM *) ((arg1)->values);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_VALUEPARAM, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_name_set) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_name_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCVARIABLE_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_name_get) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_name_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_t58_set) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_t58_set(self,t58);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_t58_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCVARIABLE_t58_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->t58);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->t58 = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->t58 = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_t58_get) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_t58_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_t58_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    result = (BITCODE_TV) ((arg1)->t58);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_evaluator_set) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_evaluator_set(self,evaluator);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_evaluator_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCVARIABLE_evaluator_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->evaluator);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->evaluator = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->evaluator = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_evaluator_get) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_evaluator_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_evaluator_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    result = (BITCODE_TV) ((arg1)->evaluator);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_desc_set) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_desc_set(self,desc);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_desc_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCVARIABLE_desc_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->desc);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->desc = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->desc = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_desc_get) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_desc_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_desc_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    result = (BITCODE_TV) ((arg1)->desc);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_value_set) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    Dwg_EvalVariant *arg2 = (Dwg_EvalVariant *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_value_set(self,value);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_value_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalVariant, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCVARIABLE_value_set" "', argument " "2"" of type '" "Dwg_EvalVariant *""'"); 
    }
    arg2 = (Dwg_EvalVariant *)(argp2);
    if (arg1) (arg1)->value = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_value_get) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalVariant *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_value_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_value_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    result = (Dwg_EvalVariant *)& ((arg1)->value);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalVariant, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_has_t78_set) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_has_t78_set(self,has_t78);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_has_t78_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCVARIABLE_has_t78_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_t78 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_has_t78_get) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_has_t78_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_has_t78_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    result = (BITCODE_B) ((arg1)->has_t78);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_t78_set) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_t78_set(self,t78);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_t78_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCVARIABLE_t78_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->t78);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->t78 = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->t78 = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_t78_get) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_t78_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_t78_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    result = (BITCODE_TV) ((arg1)->t78);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_b290_set) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_b290_set(self,b290);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_b290_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCVARIABLE_b290_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->b290 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVARIABLE_b290_get) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVARIABLE_b290_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVARIABLE_b290_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    result = (BITCODE_B) ((arg1)->b290);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCVARIABLE) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCVARIABLE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCVARIABLE();");
    }
    result = (struct _dwg_object_ASSOCVARIABLE *)calloc(1, sizeof(struct _dwg_object_ASSOCVARIABLE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCVARIABLE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCVARIABLE) {
  {
    struct _dwg_object_ASSOCVARIABLE *arg1 = (struct _dwg_object_ASSOCVARIABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCVARIABLE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCVARIABLE" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVARIABLE *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVARIABLE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EVAL_Node_parent_set) {
  {
    struct _dwg_EVAL_Node *arg1 = (struct _dwg_EVAL_Node *) 0 ;
    struct _dwg_object_EVALUATION_GRAPH *arg2 = (struct _dwg_object_EVALUATION_GRAPH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_EVAL_Node_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EVAL_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EVAL_Node_parent_set" "', argument " "1"" of type '" "struct _dwg_EVAL_Node *""'"); 
    }
    arg1 = (struct _dwg_EVAL_Node *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_EVALUATION_GRAPH, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_EVAL_Node_parent_set" "', argument " "2"" of type '" "struct _dwg_object_EVALUATION_GRAPH *""'"); 
    }
    arg2 = (struct _dwg_object_EVALUATION_GRAPH *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EVAL_Node_parent_get) {
  {
    struct _dwg_EVAL_Node *arg1 = (struct _dwg_EVAL_Node *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_EVALUATION_GRAPH *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_EVAL_Node_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EVAL_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EVAL_Node_parent_get" "', argument " "1"" of type '" "struct _dwg_EVAL_Node *""'"); 
    }
    arg1 = (struct _dwg_EVAL_Node *)(argp1);
    result = (struct _dwg_object_EVALUATION_GRAPH *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_EVALUATION_GRAPH, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EVAL_Node_id_set) {
  {
    struct _dwg_EVAL_Node *arg1 = (struct _dwg_EVAL_Node *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_EVAL_Node_id_set(self,id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EVAL_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EVAL_Node_id_set" "', argument " "1"" of type '" "struct _dwg_EVAL_Node *""'"); 
    }
    arg1 = (struct _dwg_EVAL_Node *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_EVAL_Node_id_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EVAL_Node_id_get) {
  {
    struct _dwg_EVAL_Node *arg1 = (struct _dwg_EVAL_Node *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_EVAL_Node_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EVAL_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EVAL_Node_id_get" "', argument " "1"" of type '" "struct _dwg_EVAL_Node *""'"); 
    }
    arg1 = (struct _dwg_EVAL_Node *)(argp1);
    result = (BITCODE_BL) ((arg1)->id);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EVAL_Node_edge_flags_set) {
  {
    struct _dwg_EVAL_Node *arg1 = (struct _dwg_EVAL_Node *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_EVAL_Node_edge_flags_set(self,edge_flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EVAL_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EVAL_Node_edge_flags_set" "', argument " "1"" of type '" "struct _dwg_EVAL_Node *""'"); 
    }
    arg1 = (struct _dwg_EVAL_Node *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_EVAL_Node_edge_flags_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->edge_flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EVAL_Node_edge_flags_get) {
  {
    struct _dwg_EVAL_Node *arg1 = (struct _dwg_EVAL_Node *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_EVAL_Node_edge_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EVAL_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EVAL_Node_edge_flags_get" "', argument " "1"" of type '" "struct _dwg_EVAL_Node *""'"); 
    }
    arg1 = (struct _dwg_EVAL_Node *)(argp1);
    result = (BITCODE_BL) ((arg1)->edge_flags);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EVAL_Node_nextid_set) {
  {
    struct _dwg_EVAL_Node *arg1 = (struct _dwg_EVAL_Node *) 0 ;
    BITCODE_BLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_EVAL_Node_nextid_set(self,nextid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EVAL_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EVAL_Node_nextid_set" "', argument " "1"" of type '" "struct _dwg_EVAL_Node *""'"); 
    }
    arg1 = (struct _dwg_EVAL_Node *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_EVAL_Node_nextid_set" "', argument " "2"" of type '" "BITCODE_BLd""'");
    } 
    arg2 = (BITCODE_BLd)(val2);
    if (arg1) (arg1)->nextid = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EVAL_Node_nextid_get) {
  {
    struct _dwg_EVAL_Node *arg1 = (struct _dwg_EVAL_Node *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_EVAL_Node_nextid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EVAL_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EVAL_Node_nextid_get" "', argument " "1"" of type '" "struct _dwg_EVAL_Node *""'"); 
    }
    arg1 = (struct _dwg_EVAL_Node *)(argp1);
    result = (BITCODE_BLd) ((arg1)->nextid);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EVAL_Node_evalexpr_set) {
  {
    struct _dwg_EVAL_Node *arg1 = (struct _dwg_EVAL_Node *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_EVAL_Node_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EVAL_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EVAL_Node_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_EVAL_Node *""'"); 
    }
    arg1 = (struct _dwg_EVAL_Node *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_EVAL_Node_evalexpr_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->evalexpr = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EVAL_Node_evalexpr_get) {
  {
    struct _dwg_EVAL_Node *arg1 = (struct _dwg_EVAL_Node *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_EVAL_Node_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EVAL_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EVAL_Node_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_EVAL_Node *""'"); 
    }
    arg1 = (struct _dwg_EVAL_Node *)(argp1);
    result = (BITCODE_H) ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EVAL_Node_node_set) {
  {
    struct _dwg_EVAL_Node *arg1 = (struct _dwg_EVAL_Node *) 0 ;
    BITCODE_BLd *arg2 = (BITCODE_BLd *) (BITCODE_BLd *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_EVAL_Node_node_set(self,node);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EVAL_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EVAL_Node_node_set" "', argument " "1"" of type '" "struct _dwg_EVAL_Node *""'"); 
    }
    arg1 = (struct _dwg_EVAL_Node *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_int, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_EVAL_Node_node_set" "', argument " "2"" of type '" "BITCODE_BLd [4]""'"); 
    } 
    arg2 = (BITCODE_BLd *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)4; ++ii) *(BITCODE_BLd *)&arg1->node[ii] = *((BITCODE_BLd *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""node""' of type '""BITCODE_BLd [4]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EVAL_Node_node_get) {
  {
    struct _dwg_EVAL_Node *arg1 = (struct _dwg_EVAL_Node *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_EVAL_Node_node_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EVAL_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EVAL_Node_node_get" "', argument " "1"" of type '" "struct _dwg_EVAL_Node *""'"); 
    }
    arg1 = (struct _dwg_EVAL_Node *)(argp1);
    result = (BITCODE_BLd *)(BITCODE_BLd *) ((arg1)->node);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EVAL_Node_active_cycles_set) {
  {
    struct _dwg_EVAL_Node *arg1 = (struct _dwg_EVAL_Node *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_EVAL_Node_active_cycles_set(self,active_cycles);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EVAL_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EVAL_Node_active_cycles_set" "', argument " "1"" of type '" "struct _dwg_EVAL_Node *""'"); 
    }
    arg1 = (struct _dwg_EVAL_Node *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_EVAL_Node_active_cycles_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->active_cycles = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EVAL_Node_active_cycles_get) {
  {
    struct _dwg_EVAL_Node *arg1 = (struct _dwg_EVAL_Node *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_EVAL_Node_active_cycles_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EVAL_Node, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EVAL_Node_active_cycles_get" "', argument " "1"" of type '" "struct _dwg_EVAL_Node *""'"); 
    }
    arg1 = (struct _dwg_EVAL_Node *)(argp1);
    result = (BITCODE_B) ((arg1)->active_cycles);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_EVAL_Node) {
  {
    int argvi = 0;
    struct _dwg_EVAL_Node *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_EVAL_Node();");
    }
    result = (struct _dwg_EVAL_Node *)calloc(1, sizeof(struct _dwg_EVAL_Node));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EVAL_Node, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_EVAL_Node) {
  {
    struct _dwg_EVAL_Node *arg1 = (struct _dwg_EVAL_Node *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_EVAL_Node(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EVAL_Node, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_EVAL_Node" "', argument " "1"" of type '" "struct _dwg_EVAL_Node *""'"); 
    }
    arg1 = (struct _dwg_EVAL_Node *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EVAL_Edge_parent_set) {
  {
    struct _dwg_EVAL_Edge *arg1 = (struct _dwg_EVAL_Edge *) 0 ;
    struct _dwg_object_EVALUATION_GRAPH *arg2 = (struct _dwg_object_EVALUATION_GRAPH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_EVAL_Edge_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EVAL_Edge, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EVAL_Edge_parent_set" "', argument " "1"" of type '" "struct _dwg_EVAL_Edge *""'"); 
    }
    arg1 = (struct _dwg_EVAL_Edge *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_EVALUATION_GRAPH, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_EVAL_Edge_parent_set" "', argument " "2"" of type '" "struct _dwg_object_EVALUATION_GRAPH *""'"); 
    }
    arg2 = (struct _dwg_object_EVALUATION_GRAPH *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EVAL_Edge_parent_get) {
  {
    struct _dwg_EVAL_Edge *arg1 = (struct _dwg_EVAL_Edge *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_EVALUATION_GRAPH *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_EVAL_Edge_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EVAL_Edge, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EVAL_Edge_parent_get" "', argument " "1"" of type '" "struct _dwg_EVAL_Edge *""'"); 
    }
    arg1 = (struct _dwg_EVAL_Edge *)(argp1);
    result = (struct _dwg_object_EVALUATION_GRAPH *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_EVALUATION_GRAPH, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EVAL_Edge_id_set) {
  {
    struct _dwg_EVAL_Edge *arg1 = (struct _dwg_EVAL_Edge *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_EVAL_Edge_id_set(self,id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EVAL_Edge, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EVAL_Edge_id_set" "', argument " "1"" of type '" "struct _dwg_EVAL_Edge *""'"); 
    }
    arg1 = (struct _dwg_EVAL_Edge *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_EVAL_Edge_id_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EVAL_Edge_id_get) {
  {
    struct _dwg_EVAL_Edge *arg1 = (struct _dwg_EVAL_Edge *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_EVAL_Edge_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EVAL_Edge, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EVAL_Edge_id_get" "', argument " "1"" of type '" "struct _dwg_EVAL_Edge *""'"); 
    }
    arg1 = (struct _dwg_EVAL_Edge *)(argp1);
    result = (BITCODE_BL) ((arg1)->id);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EVAL_Edge_nextid_set) {
  {
    struct _dwg_EVAL_Edge *arg1 = (struct _dwg_EVAL_Edge *) 0 ;
    BITCODE_BLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_EVAL_Edge_nextid_set(self,nextid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EVAL_Edge, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EVAL_Edge_nextid_set" "', argument " "1"" of type '" "struct _dwg_EVAL_Edge *""'"); 
    }
    arg1 = (struct _dwg_EVAL_Edge *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_EVAL_Edge_nextid_set" "', argument " "2"" of type '" "BITCODE_BLd""'");
    } 
    arg2 = (BITCODE_BLd)(val2);
    if (arg1) (arg1)->nextid = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EVAL_Edge_nextid_get) {
  {
    struct _dwg_EVAL_Edge *arg1 = (struct _dwg_EVAL_Edge *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_EVAL_Edge_nextid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EVAL_Edge, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EVAL_Edge_nextid_get" "', argument " "1"" of type '" "struct _dwg_EVAL_Edge *""'"); 
    }
    arg1 = (struct _dwg_EVAL_Edge *)(argp1);
    result = (BITCODE_BLd) ((arg1)->nextid);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EVAL_Edge_e1_set) {
  {
    struct _dwg_EVAL_Edge *arg1 = (struct _dwg_EVAL_Edge *) 0 ;
    BITCODE_BLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_EVAL_Edge_e1_set(self,e1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EVAL_Edge, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EVAL_Edge_e1_set" "', argument " "1"" of type '" "struct _dwg_EVAL_Edge *""'"); 
    }
    arg1 = (struct _dwg_EVAL_Edge *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_EVAL_Edge_e1_set" "', argument " "2"" of type '" "BITCODE_BLd""'");
    } 
    arg2 = (BITCODE_BLd)(val2);
    if (arg1) (arg1)->e1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EVAL_Edge_e1_get) {
  {
    struct _dwg_EVAL_Edge *arg1 = (struct _dwg_EVAL_Edge *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_EVAL_Edge_e1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EVAL_Edge, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EVAL_Edge_e1_get" "', argument " "1"" of type '" "struct _dwg_EVAL_Edge *""'"); 
    }
    arg1 = (struct _dwg_EVAL_Edge *)(argp1);
    result = (BITCODE_BLd) ((arg1)->e1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EVAL_Edge_e2_set) {
  {
    struct _dwg_EVAL_Edge *arg1 = (struct _dwg_EVAL_Edge *) 0 ;
    BITCODE_BLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_EVAL_Edge_e2_set(self,e2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EVAL_Edge, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EVAL_Edge_e2_set" "', argument " "1"" of type '" "struct _dwg_EVAL_Edge *""'"); 
    }
    arg1 = (struct _dwg_EVAL_Edge *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_EVAL_Edge_e2_set" "', argument " "2"" of type '" "BITCODE_BLd""'");
    } 
    arg2 = (BITCODE_BLd)(val2);
    if (arg1) (arg1)->e2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EVAL_Edge_e2_get) {
  {
    struct _dwg_EVAL_Edge *arg1 = (struct _dwg_EVAL_Edge *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_EVAL_Edge_e2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EVAL_Edge, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EVAL_Edge_e2_get" "', argument " "1"" of type '" "struct _dwg_EVAL_Edge *""'"); 
    }
    arg1 = (struct _dwg_EVAL_Edge *)(argp1);
    result = (BITCODE_BLd) ((arg1)->e2);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EVAL_Edge_e3_set) {
  {
    struct _dwg_EVAL_Edge *arg1 = (struct _dwg_EVAL_Edge *) 0 ;
    BITCODE_BLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_EVAL_Edge_e3_set(self,e3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EVAL_Edge, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EVAL_Edge_e3_set" "', argument " "1"" of type '" "struct _dwg_EVAL_Edge *""'"); 
    }
    arg1 = (struct _dwg_EVAL_Edge *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_EVAL_Edge_e3_set" "', argument " "2"" of type '" "BITCODE_BLd""'");
    } 
    arg2 = (BITCODE_BLd)(val2);
    if (arg1) (arg1)->e3 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EVAL_Edge_e3_get) {
  {
    struct _dwg_EVAL_Edge *arg1 = (struct _dwg_EVAL_Edge *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_EVAL_Edge_e3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EVAL_Edge, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EVAL_Edge_e3_get" "', argument " "1"" of type '" "struct _dwg_EVAL_Edge *""'"); 
    }
    arg1 = (struct _dwg_EVAL_Edge *)(argp1);
    result = (BITCODE_BLd) ((arg1)->e3);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EVAL_Edge_out_edge_set) {
  {
    struct _dwg_EVAL_Edge *arg1 = (struct _dwg_EVAL_Edge *) 0 ;
    BITCODE_BLd *arg2 = (BITCODE_BLd *) (BITCODE_BLd *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_EVAL_Edge_out_edge_set(self,out_edge);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EVAL_Edge, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EVAL_Edge_out_edge_set" "', argument " "1"" of type '" "struct _dwg_EVAL_Edge *""'"); 
    }
    arg1 = (struct _dwg_EVAL_Edge *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_int, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_EVAL_Edge_out_edge_set" "', argument " "2"" of type '" "BITCODE_BLd [5]""'"); 
    } 
    arg2 = (BITCODE_BLd *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)5; ++ii) *(BITCODE_BLd *)&arg1->out_edge[ii] = *((BITCODE_BLd *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""out_edge""' of type '""BITCODE_BLd [5]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EVAL_Edge_out_edge_get) {
  {
    struct _dwg_EVAL_Edge *arg1 = (struct _dwg_EVAL_Edge *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_EVAL_Edge_out_edge_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EVAL_Edge, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EVAL_Edge_out_edge_get" "', argument " "1"" of type '" "struct _dwg_EVAL_Edge *""'"); 
    }
    arg1 = (struct _dwg_EVAL_Edge *)(argp1);
    result = (BITCODE_BLd *)(BITCODE_BLd *) ((arg1)->out_edge);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_EVAL_Edge) {
  {
    int argvi = 0;
    struct _dwg_EVAL_Edge *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_EVAL_Edge();");
    }
    result = (struct _dwg_EVAL_Edge *)calloc(1, sizeof(struct _dwg_EVAL_Edge));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EVAL_Edge, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_EVAL_Edge) {
  {
    struct _dwg_EVAL_Edge *arg1 = (struct _dwg_EVAL_Edge *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_EVAL_Edge(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EVAL_Edge, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_EVAL_Edge" "', argument " "1"" of type '" "struct _dwg_EVAL_Edge *""'"); 
    }
    arg1 = (struct _dwg_EVAL_Edge *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_EVALUATION_GRAPH_parent_set) {
  {
    struct _dwg_object_EVALUATION_GRAPH *arg1 = (struct _dwg_object_EVALUATION_GRAPH *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_EVALUATION_GRAPH_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_EVALUATION_GRAPH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_EVALUATION_GRAPH_parent_set" "', argument " "1"" of type '" "struct _dwg_object_EVALUATION_GRAPH *""'"); 
    }
    arg1 = (struct _dwg_object_EVALUATION_GRAPH *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_EVALUATION_GRAPH_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_EVALUATION_GRAPH_parent_get) {
  {
    struct _dwg_object_EVALUATION_GRAPH *arg1 = (struct _dwg_object_EVALUATION_GRAPH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_EVALUATION_GRAPH_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_EVALUATION_GRAPH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_EVALUATION_GRAPH_parent_get" "', argument " "1"" of type '" "struct _dwg_object_EVALUATION_GRAPH *""'"); 
    }
    arg1 = (struct _dwg_object_EVALUATION_GRAPH *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_EVALUATION_GRAPH_major_set) {
  {
    struct _dwg_object_EVALUATION_GRAPH *arg1 = (struct _dwg_object_EVALUATION_GRAPH *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_EVALUATION_GRAPH_major_set(self,major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_EVALUATION_GRAPH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_EVALUATION_GRAPH_major_set" "', argument " "1"" of type '" "struct _dwg_object_EVALUATION_GRAPH *""'"); 
    }
    arg1 = (struct _dwg_object_EVALUATION_GRAPH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_EVALUATION_GRAPH_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_EVALUATION_GRAPH_major_get) {
  {
    struct _dwg_object_EVALUATION_GRAPH *arg1 = (struct _dwg_object_EVALUATION_GRAPH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_EVALUATION_GRAPH_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_EVALUATION_GRAPH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_EVALUATION_GRAPH_major_get" "', argument " "1"" of type '" "struct _dwg_object_EVALUATION_GRAPH *""'"); 
    }
    arg1 = (struct _dwg_object_EVALUATION_GRAPH *)(argp1);
    result = (BITCODE_BL) ((arg1)->major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_EVALUATION_GRAPH_minor_set) {
  {
    struct _dwg_object_EVALUATION_GRAPH *arg1 = (struct _dwg_object_EVALUATION_GRAPH *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_EVALUATION_GRAPH_minor_set(self,minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_EVALUATION_GRAPH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_EVALUATION_GRAPH_minor_set" "', argument " "1"" of type '" "struct _dwg_object_EVALUATION_GRAPH *""'"); 
    }
    arg1 = (struct _dwg_object_EVALUATION_GRAPH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_EVALUATION_GRAPH_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_EVALUATION_GRAPH_minor_get) {
  {
    struct _dwg_object_EVALUATION_GRAPH *arg1 = (struct _dwg_object_EVALUATION_GRAPH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_EVALUATION_GRAPH_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_EVALUATION_GRAPH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_EVALUATION_GRAPH_minor_get" "', argument " "1"" of type '" "struct _dwg_object_EVALUATION_GRAPH *""'"); 
    }
    arg1 = (struct _dwg_object_EVALUATION_GRAPH *)(argp1);
    result = (BITCODE_BL) ((arg1)->minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_EVALUATION_GRAPH_first_nodeid_set) {
  {
    struct _dwg_object_EVALUATION_GRAPH *arg1 = (struct _dwg_object_EVALUATION_GRAPH *) 0 ;
    BITCODE_BLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_EVALUATION_GRAPH_first_nodeid_set(self,first_nodeid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_EVALUATION_GRAPH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_EVALUATION_GRAPH_first_nodeid_set" "', argument " "1"" of type '" "struct _dwg_object_EVALUATION_GRAPH *""'"); 
    }
    arg1 = (struct _dwg_object_EVALUATION_GRAPH *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_EVALUATION_GRAPH_first_nodeid_set" "', argument " "2"" of type '" "BITCODE_BLd""'");
    } 
    arg2 = (BITCODE_BLd)(val2);
    if (arg1) (arg1)->first_nodeid = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_EVALUATION_GRAPH_first_nodeid_get) {
  {
    struct _dwg_object_EVALUATION_GRAPH *arg1 = (struct _dwg_object_EVALUATION_GRAPH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_EVALUATION_GRAPH_first_nodeid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_EVALUATION_GRAPH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_EVALUATION_GRAPH_first_nodeid_get" "', argument " "1"" of type '" "struct _dwg_object_EVALUATION_GRAPH *""'"); 
    }
    arg1 = (struct _dwg_object_EVALUATION_GRAPH *)(argp1);
    result = (BITCODE_BLd) ((arg1)->first_nodeid);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_EVALUATION_GRAPH_first_nodeid_copy_set) {
  {
    struct _dwg_object_EVALUATION_GRAPH *arg1 = (struct _dwg_object_EVALUATION_GRAPH *) 0 ;
    BITCODE_BLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_EVALUATION_GRAPH_first_nodeid_copy_set(self,first_nodeid_copy);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_EVALUATION_GRAPH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_EVALUATION_GRAPH_first_nodeid_copy_set" "', argument " "1"" of type '" "struct _dwg_object_EVALUATION_GRAPH *""'"); 
    }
    arg1 = (struct _dwg_object_EVALUATION_GRAPH *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_EVALUATION_GRAPH_first_nodeid_copy_set" "', argument " "2"" of type '" "BITCODE_BLd""'");
    } 
    arg2 = (BITCODE_BLd)(val2);
    if (arg1) (arg1)->first_nodeid_copy = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_EVALUATION_GRAPH_first_nodeid_copy_get) {
  {
    struct _dwg_object_EVALUATION_GRAPH *arg1 = (struct _dwg_object_EVALUATION_GRAPH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_EVALUATION_GRAPH_first_nodeid_copy_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_EVALUATION_GRAPH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_EVALUATION_GRAPH_first_nodeid_copy_get" "', argument " "1"" of type '" "struct _dwg_object_EVALUATION_GRAPH *""'"); 
    }
    arg1 = (struct _dwg_object_EVALUATION_GRAPH *)(argp1);
    result = (BITCODE_BLd) ((arg1)->first_nodeid_copy);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_EVALUATION_GRAPH_num_nodes_set) {
  {
    struct _dwg_object_EVALUATION_GRAPH *arg1 = (struct _dwg_object_EVALUATION_GRAPH *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_EVALUATION_GRAPH_num_nodes_set(self,num_nodes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_EVALUATION_GRAPH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_EVALUATION_GRAPH_num_nodes_set" "', argument " "1"" of type '" "struct _dwg_object_EVALUATION_GRAPH *""'"); 
    }
    arg1 = (struct _dwg_object_EVALUATION_GRAPH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_EVALUATION_GRAPH_num_nodes_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_nodes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_EVALUATION_GRAPH_num_nodes_get) {
  {
    struct _dwg_object_EVALUATION_GRAPH *arg1 = (struct _dwg_object_EVALUATION_GRAPH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_EVALUATION_GRAPH_num_nodes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_EVALUATION_GRAPH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_EVALUATION_GRAPH_num_nodes_get" "', argument " "1"" of type '" "struct _dwg_object_EVALUATION_GRAPH *""'"); 
    }
    arg1 = (struct _dwg_object_EVALUATION_GRAPH *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_nodes);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_EVALUATION_GRAPH_nodes_set) {
  {
    struct _dwg_object_EVALUATION_GRAPH *arg1 = (struct _dwg_object_EVALUATION_GRAPH *) 0 ;
    Dwg_EVAL_Node *arg2 = (Dwg_EVAL_Node *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_EVALUATION_GRAPH_nodes_set(self,nodes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_EVALUATION_GRAPH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_EVALUATION_GRAPH_nodes_set" "', argument " "1"" of type '" "struct _dwg_object_EVALUATION_GRAPH *""'"); 
    }
    arg1 = (struct _dwg_object_EVALUATION_GRAPH *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EVAL_Node, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_EVALUATION_GRAPH_nodes_set" "', argument " "2"" of type '" "Dwg_EVAL_Node *""'"); 
    }
    arg2 = (Dwg_EVAL_Node *)(argp2);
    if (arg1) (arg1)->nodes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_EVALUATION_GRAPH_nodes_get) {
  {
    struct _dwg_object_EVALUATION_GRAPH *arg1 = (struct _dwg_object_EVALUATION_GRAPH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EVAL_Node *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_EVALUATION_GRAPH_nodes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_EVALUATION_GRAPH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_EVALUATION_GRAPH_nodes_get" "', argument " "1"" of type '" "struct _dwg_object_EVALUATION_GRAPH *""'"); 
    }
    arg1 = (struct _dwg_object_EVALUATION_GRAPH *)(argp1);
    result = (Dwg_EVAL_Node *) ((arg1)->nodes);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EVAL_Node, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_EVALUATION_GRAPH_has_graph_set) {
  {
    struct _dwg_object_EVALUATION_GRAPH *arg1 = (struct _dwg_object_EVALUATION_GRAPH *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_EVALUATION_GRAPH_has_graph_set(self,has_graph);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_EVALUATION_GRAPH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_EVALUATION_GRAPH_has_graph_set" "', argument " "1"" of type '" "struct _dwg_object_EVALUATION_GRAPH *""'"); 
    }
    arg1 = (struct _dwg_object_EVALUATION_GRAPH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_EVALUATION_GRAPH_has_graph_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_graph = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_EVALUATION_GRAPH_has_graph_get) {
  {
    struct _dwg_object_EVALUATION_GRAPH *arg1 = (struct _dwg_object_EVALUATION_GRAPH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_EVALUATION_GRAPH_has_graph_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_EVALUATION_GRAPH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_EVALUATION_GRAPH_has_graph_get" "', argument " "1"" of type '" "struct _dwg_object_EVALUATION_GRAPH *""'"); 
    }
    arg1 = (struct _dwg_object_EVALUATION_GRAPH *)(argp1);
    result = (BITCODE_B) ((arg1)->has_graph);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_EVALUATION_GRAPH_num_edges_set) {
  {
    struct _dwg_object_EVALUATION_GRAPH *arg1 = (struct _dwg_object_EVALUATION_GRAPH *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_EVALUATION_GRAPH_num_edges_set(self,num_edges);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_EVALUATION_GRAPH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_EVALUATION_GRAPH_num_edges_set" "', argument " "1"" of type '" "struct _dwg_object_EVALUATION_GRAPH *""'"); 
    }
    arg1 = (struct _dwg_object_EVALUATION_GRAPH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_EVALUATION_GRAPH_num_edges_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_edges = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_EVALUATION_GRAPH_num_edges_get) {
  {
    struct _dwg_object_EVALUATION_GRAPH *arg1 = (struct _dwg_object_EVALUATION_GRAPH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_EVALUATION_GRAPH_num_edges_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_EVALUATION_GRAPH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_EVALUATION_GRAPH_num_edges_get" "', argument " "1"" of type '" "struct _dwg_object_EVALUATION_GRAPH *""'"); 
    }
    arg1 = (struct _dwg_object_EVALUATION_GRAPH *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_edges);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_EVALUATION_GRAPH_edges_set) {
  {
    struct _dwg_object_EVALUATION_GRAPH *arg1 = (struct _dwg_object_EVALUATION_GRAPH *) 0 ;
    Dwg_EVAL_Edge *arg2 = (Dwg_EVAL_Edge *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_EVALUATION_GRAPH_edges_set(self,edges);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_EVALUATION_GRAPH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_EVALUATION_GRAPH_edges_set" "', argument " "1"" of type '" "struct _dwg_object_EVALUATION_GRAPH *""'"); 
    }
    arg1 = (struct _dwg_object_EVALUATION_GRAPH *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EVAL_Edge, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_EVALUATION_GRAPH_edges_set" "', argument " "2"" of type '" "Dwg_EVAL_Edge *""'"); 
    }
    arg2 = (Dwg_EVAL_Edge *)(argp2);
    if (arg1) (arg1)->edges = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_EVALUATION_GRAPH_edges_get) {
  {
    struct _dwg_object_EVALUATION_GRAPH *arg1 = (struct _dwg_object_EVALUATION_GRAPH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EVAL_Edge *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_EVALUATION_GRAPH_edges_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_EVALUATION_GRAPH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_EVALUATION_GRAPH_edges_get" "', argument " "1"" of type '" "struct _dwg_object_EVALUATION_GRAPH *""'"); 
    }
    arg1 = (struct _dwg_object_EVALUATION_GRAPH *)(argp1);
    result = (Dwg_EVAL_Edge *) ((arg1)->edges);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EVAL_Edge, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_EVALUATION_GRAPH) {
  {
    int argvi = 0;
    struct _dwg_object_EVALUATION_GRAPH *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_EVALUATION_GRAPH();");
    }
    result = (struct _dwg_object_EVALUATION_GRAPH *)calloc(1, sizeof(struct _dwg_object_EVALUATION_GRAPH));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_EVALUATION_GRAPH, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_EVALUATION_GRAPH) {
  {
    struct _dwg_object_EVALUATION_GRAPH *arg1 = (struct _dwg_object_EVALUATION_GRAPH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_EVALUATION_GRAPH(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_EVALUATION_GRAPH, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_EVALUATION_GRAPH" "', argument " "1"" of type '" "struct _dwg_object_EVALUATION_GRAPH *""'"); 
    }
    arg1 = (struct _dwg_object_EVALUATION_GRAPH *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_parent_set) {
  {
    struct _dwg_object_DYNAMICBLOCKPURGEPREVENTER *arg1 = (struct _dwg_object_DYNAMICBLOCKPURGEPREVENTER *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DYNAMICBLOCKPURGEPREVENTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_parent_set" "', argument " "1"" of type '" "struct _dwg_object_DYNAMICBLOCKPURGEPREVENTER *""'"); 
    }
    arg1 = (struct _dwg_object_DYNAMICBLOCKPURGEPREVENTER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_parent_get) {
  {
    struct _dwg_object_DYNAMICBLOCKPURGEPREVENTER *arg1 = (struct _dwg_object_DYNAMICBLOCKPURGEPREVENTER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DYNAMICBLOCKPURGEPREVENTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_parent_get" "', argument " "1"" of type '" "struct _dwg_object_DYNAMICBLOCKPURGEPREVENTER *""'"); 
    }
    arg1 = (struct _dwg_object_DYNAMICBLOCKPURGEPREVENTER *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_flag_set) {
  {
    struct _dwg_object_DYNAMICBLOCKPURGEPREVENTER *arg1 = (struct _dwg_object_DYNAMICBLOCKPURGEPREVENTER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DYNAMICBLOCKPURGEPREVENTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_flag_set" "', argument " "1"" of type '" "struct _dwg_object_DYNAMICBLOCKPURGEPREVENTER *""'"); 
    }
    arg1 = (struct _dwg_object_DYNAMICBLOCKPURGEPREVENTER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_flag_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_flag_get) {
  {
    struct _dwg_object_DYNAMICBLOCKPURGEPREVENTER *arg1 = (struct _dwg_object_DYNAMICBLOCKPURGEPREVENTER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DYNAMICBLOCKPURGEPREVENTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_flag_get" "', argument " "1"" of type '" "struct _dwg_object_DYNAMICBLOCKPURGEPREVENTER *""'"); 
    }
    arg1 = (struct _dwg_object_DYNAMICBLOCKPURGEPREVENTER *)(argp1);
    result = (BITCODE_BS) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_block_set) {
  {
    struct _dwg_object_DYNAMICBLOCKPURGEPREVENTER *arg1 = (struct _dwg_object_DYNAMICBLOCKPURGEPREVENTER *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_block_set(self,block);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DYNAMICBLOCKPURGEPREVENTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_block_set" "', argument " "1"" of type '" "struct _dwg_object_DYNAMICBLOCKPURGEPREVENTER *""'"); 
    }
    arg1 = (struct _dwg_object_DYNAMICBLOCKPURGEPREVENTER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_block_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->block = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_block_get) {
  {
    struct _dwg_object_DYNAMICBLOCKPURGEPREVENTER *arg1 = (struct _dwg_object_DYNAMICBLOCKPURGEPREVENTER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_block_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DYNAMICBLOCKPURGEPREVENTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_block_get" "', argument " "1"" of type '" "struct _dwg_object_DYNAMICBLOCKPURGEPREVENTER *""'"); 
    }
    arg1 = (struct _dwg_object_DYNAMICBLOCKPURGEPREVENTER *)(argp1);
    result = (BITCODE_H) ((arg1)->block);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_DYNAMICBLOCKPURGEPREVENTER) {
  {
    int argvi = 0;
    struct _dwg_object_DYNAMICBLOCKPURGEPREVENTER *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_DYNAMICBLOCKPURGEPREVENTER();");
    }
    result = (struct _dwg_object_DYNAMICBLOCKPURGEPREVENTER *)calloc(1, sizeof(struct _dwg_object_DYNAMICBLOCKPURGEPREVENTER));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DYNAMICBLOCKPURGEPREVENTER, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_DYNAMICBLOCKPURGEPREVENTER) {
  {
    struct _dwg_object_DYNAMICBLOCKPURGEPREVENTER *arg1 = (struct _dwg_object_DYNAMICBLOCKPURGEPREVENTER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_DYNAMICBLOCKPURGEPREVENTER(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DYNAMICBLOCKPURGEPREVENTER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_DYNAMICBLOCKPURGEPREVENTER" "', argument " "1"" of type '" "struct _dwg_object_DYNAMICBLOCKPURGEPREVENTER *""'"); 
    }
    arg1 = (struct _dwg_object_DYNAMICBLOCKPURGEPREVENTER *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PERSUBENTMGR_parent_set) {
  {
    struct _dwg_object_PERSUBENTMGR *arg1 = (struct _dwg_object_PERSUBENTMGR *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PERSUBENTMGR_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PERSUBENTMGR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PERSUBENTMGR_parent_set" "', argument " "1"" of type '" "struct _dwg_object_PERSUBENTMGR *""'"); 
    }
    arg1 = (struct _dwg_object_PERSUBENTMGR *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_PERSUBENTMGR_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PERSUBENTMGR_parent_get) {
  {
    struct _dwg_object_PERSUBENTMGR *arg1 = (struct _dwg_object_PERSUBENTMGR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PERSUBENTMGR_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PERSUBENTMGR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PERSUBENTMGR_parent_get" "', argument " "1"" of type '" "struct _dwg_object_PERSUBENTMGR *""'"); 
    }
    arg1 = (struct _dwg_object_PERSUBENTMGR *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PERSUBENTMGR_class_version_set) {
  {
    struct _dwg_object_PERSUBENTMGR *arg1 = (struct _dwg_object_PERSUBENTMGR *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PERSUBENTMGR_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PERSUBENTMGR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PERSUBENTMGR_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_PERSUBENTMGR *""'"); 
    }
    arg1 = (struct _dwg_object_PERSUBENTMGR *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_PERSUBENTMGR_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PERSUBENTMGR_class_version_get) {
  {
    struct _dwg_object_PERSUBENTMGR *arg1 = (struct _dwg_object_PERSUBENTMGR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PERSUBENTMGR_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PERSUBENTMGR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PERSUBENTMGR_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_PERSUBENTMGR *""'"); 
    }
    arg1 = (struct _dwg_object_PERSUBENTMGR *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PERSUBENTMGR_unknown_0_set) {
  {
    struct _dwg_object_PERSUBENTMGR *arg1 = (struct _dwg_object_PERSUBENTMGR *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PERSUBENTMGR_unknown_0_set(self,unknown_0);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PERSUBENTMGR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PERSUBENTMGR_unknown_0_set" "', argument " "1"" of type '" "struct _dwg_object_PERSUBENTMGR *""'"); 
    }
    arg1 = (struct _dwg_object_PERSUBENTMGR *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_PERSUBENTMGR_unknown_0_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_0 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PERSUBENTMGR_unknown_0_get) {
  {
    struct _dwg_object_PERSUBENTMGR *arg1 = (struct _dwg_object_PERSUBENTMGR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PERSUBENTMGR_unknown_0_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PERSUBENTMGR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PERSUBENTMGR_unknown_0_get" "', argument " "1"" of type '" "struct _dwg_object_PERSUBENTMGR *""'"); 
    }
    arg1 = (struct _dwg_object_PERSUBENTMGR *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_0);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PERSUBENTMGR_unknown_2_set) {
  {
    struct _dwg_object_PERSUBENTMGR *arg1 = (struct _dwg_object_PERSUBENTMGR *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PERSUBENTMGR_unknown_2_set(self,unknown_2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PERSUBENTMGR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PERSUBENTMGR_unknown_2_set" "', argument " "1"" of type '" "struct _dwg_object_PERSUBENTMGR *""'"); 
    }
    arg1 = (struct _dwg_object_PERSUBENTMGR *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_PERSUBENTMGR_unknown_2_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PERSUBENTMGR_unknown_2_get) {
  {
    struct _dwg_object_PERSUBENTMGR *arg1 = (struct _dwg_object_PERSUBENTMGR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PERSUBENTMGR_unknown_2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PERSUBENTMGR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PERSUBENTMGR_unknown_2_get" "', argument " "1"" of type '" "struct _dwg_object_PERSUBENTMGR *""'"); 
    }
    arg1 = (struct _dwg_object_PERSUBENTMGR *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_2);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PERSUBENTMGR_numassocsteps_set) {
  {
    struct _dwg_object_PERSUBENTMGR *arg1 = (struct _dwg_object_PERSUBENTMGR *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PERSUBENTMGR_numassocsteps_set(self,numassocsteps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PERSUBENTMGR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PERSUBENTMGR_numassocsteps_set" "', argument " "1"" of type '" "struct _dwg_object_PERSUBENTMGR *""'"); 
    }
    arg1 = (struct _dwg_object_PERSUBENTMGR *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_PERSUBENTMGR_numassocsteps_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->numassocsteps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PERSUBENTMGR_numassocsteps_get) {
  {
    struct _dwg_object_PERSUBENTMGR *arg1 = (struct _dwg_object_PERSUBENTMGR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PERSUBENTMGR_numassocsteps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PERSUBENTMGR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PERSUBENTMGR_numassocsteps_get" "', argument " "1"" of type '" "struct _dwg_object_PERSUBENTMGR *""'"); 
    }
    arg1 = (struct _dwg_object_PERSUBENTMGR *)(argp1);
    result = (BITCODE_BL) ((arg1)->numassocsteps);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PERSUBENTMGR_numassocsubents_set) {
  {
    struct _dwg_object_PERSUBENTMGR *arg1 = (struct _dwg_object_PERSUBENTMGR *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PERSUBENTMGR_numassocsubents_set(self,numassocsubents);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PERSUBENTMGR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PERSUBENTMGR_numassocsubents_set" "', argument " "1"" of type '" "struct _dwg_object_PERSUBENTMGR *""'"); 
    }
    arg1 = (struct _dwg_object_PERSUBENTMGR *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_PERSUBENTMGR_numassocsubents_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->numassocsubents = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PERSUBENTMGR_numassocsubents_get) {
  {
    struct _dwg_object_PERSUBENTMGR *arg1 = (struct _dwg_object_PERSUBENTMGR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PERSUBENTMGR_numassocsubents_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PERSUBENTMGR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PERSUBENTMGR_numassocsubents_get" "', argument " "1"" of type '" "struct _dwg_object_PERSUBENTMGR *""'"); 
    }
    arg1 = (struct _dwg_object_PERSUBENTMGR *)(argp1);
    result = (BITCODE_BL) ((arg1)->numassocsubents);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PERSUBENTMGR_num_steps_set) {
  {
    struct _dwg_object_PERSUBENTMGR *arg1 = (struct _dwg_object_PERSUBENTMGR *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PERSUBENTMGR_num_steps_set(self,num_steps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PERSUBENTMGR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PERSUBENTMGR_num_steps_set" "', argument " "1"" of type '" "struct _dwg_object_PERSUBENTMGR *""'"); 
    }
    arg1 = (struct _dwg_object_PERSUBENTMGR *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_PERSUBENTMGR_num_steps_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_steps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PERSUBENTMGR_num_steps_get) {
  {
    struct _dwg_object_PERSUBENTMGR *arg1 = (struct _dwg_object_PERSUBENTMGR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PERSUBENTMGR_num_steps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PERSUBENTMGR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PERSUBENTMGR_num_steps_get" "', argument " "1"" of type '" "struct _dwg_object_PERSUBENTMGR *""'"); 
    }
    arg1 = (struct _dwg_object_PERSUBENTMGR *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_steps);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PERSUBENTMGR_steps_set) {
  {
    struct _dwg_object_PERSUBENTMGR *arg1 = (struct _dwg_object_PERSUBENTMGR *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PERSUBENTMGR_steps_set(self,steps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PERSUBENTMGR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PERSUBENTMGR_steps_set" "', argument " "1"" of type '" "struct _dwg_object_PERSUBENTMGR *""'"); 
    }
    arg1 = (struct _dwg_object_PERSUBENTMGR *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_PERSUBENTMGR_steps_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->steps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PERSUBENTMGR_steps_get) {
  {
    struct _dwg_object_PERSUBENTMGR *arg1 = (struct _dwg_object_PERSUBENTMGR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PERSUBENTMGR_steps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PERSUBENTMGR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PERSUBENTMGR_steps_get" "', argument " "1"" of type '" "struct _dwg_object_PERSUBENTMGR *""'"); 
    }
    arg1 = (struct _dwg_object_PERSUBENTMGR *)(argp1);
    result = (BITCODE_BL *) ((arg1)->steps);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PERSUBENTMGR_num_subents_set) {
  {
    struct _dwg_object_PERSUBENTMGR *arg1 = (struct _dwg_object_PERSUBENTMGR *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PERSUBENTMGR_num_subents_set(self,num_subents);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PERSUBENTMGR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PERSUBENTMGR_num_subents_set" "', argument " "1"" of type '" "struct _dwg_object_PERSUBENTMGR *""'"); 
    }
    arg1 = (struct _dwg_object_PERSUBENTMGR *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_PERSUBENTMGR_num_subents_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_subents = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PERSUBENTMGR_num_subents_get) {
  {
    struct _dwg_object_PERSUBENTMGR *arg1 = (struct _dwg_object_PERSUBENTMGR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PERSUBENTMGR_num_subents_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PERSUBENTMGR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PERSUBENTMGR_num_subents_get" "', argument " "1"" of type '" "struct _dwg_object_PERSUBENTMGR *""'"); 
    }
    arg1 = (struct _dwg_object_PERSUBENTMGR *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_subents);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PERSUBENTMGR_subents_set) {
  {
    struct _dwg_object_PERSUBENTMGR *arg1 = (struct _dwg_object_PERSUBENTMGR *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PERSUBENTMGR_subents_set(self,subents);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PERSUBENTMGR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PERSUBENTMGR_subents_set" "', argument " "1"" of type '" "struct _dwg_object_PERSUBENTMGR *""'"); 
    }
    arg1 = (struct _dwg_object_PERSUBENTMGR *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_PERSUBENTMGR_subents_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->subents = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PERSUBENTMGR_subents_get) {
  {
    struct _dwg_object_PERSUBENTMGR *arg1 = (struct _dwg_object_PERSUBENTMGR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PERSUBENTMGR_subents_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PERSUBENTMGR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PERSUBENTMGR_subents_get" "', argument " "1"" of type '" "struct _dwg_object_PERSUBENTMGR *""'"); 
    }
    arg1 = (struct _dwg_object_PERSUBENTMGR *)(argp1);
    result = (BITCODE_BL *) ((arg1)->subents);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_PERSUBENTMGR) {
  {
    int argvi = 0;
    struct _dwg_object_PERSUBENTMGR *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_PERSUBENTMGR();");
    }
    result = (struct _dwg_object_PERSUBENTMGR *)calloc(1, sizeof(struct _dwg_object_PERSUBENTMGR));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_PERSUBENTMGR, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_PERSUBENTMGR) {
  {
    struct _dwg_object_PERSUBENTMGR *arg1 = (struct _dwg_object_PERSUBENTMGR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_PERSUBENTMGR(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PERSUBENTMGR, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_PERSUBENTMGR" "', argument " "1"" of type '" "struct _dwg_object_PERSUBENTMGR *""'"); 
    }
    arg1 = (struct _dwg_object_PERSUBENTMGR *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_parent_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_parent_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_class_version_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_class_version_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_3_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_3_set(self,unknown_3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_3_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_3_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_3 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_3_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_3_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_3);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_0_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_0_set(self,unknown_0);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_0_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_0_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_0 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_0_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_0_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_0_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_0);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_2_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_2_set(self,unknown_2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_2_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_2_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_2_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_2_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_2);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_num_steps_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_num_steps_set(self,num_steps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_num_steps_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_num_steps_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_steps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_num_steps_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_num_steps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_num_steps_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_steps);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_num_subents_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_num_subents_set(self,num_subents);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_num_subents_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_num_subents_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_subents = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_num_subents_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_num_subents_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_num_subents_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_subents);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_steps_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_steps_set(self,steps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_steps_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_steps_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->steps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_steps_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_steps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_steps_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL *) ((arg1)->steps);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_subents_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_subents_set(self,subents);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_subents_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_subents_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->subents = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_subents_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_subents_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_subents_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL *) ((arg1)->subents);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl6_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl6_set(self,unknown_bl6);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl6_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl6_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl6 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl6_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl6_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl6_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl6);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl6a_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl6a_set(self,unknown_bl6a);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl6a_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl6a_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl6a = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl6a_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl6a_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl6a_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl6a);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl7a_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl7a_set(self,unknown_bl7a);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl7a_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl7a_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl7a = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl7a_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl7a_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl7a_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl7a);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl7_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl7_set(self,unknown_bl7);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl7_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl7_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl7 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl7_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl7_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl7_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl7);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl8_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl8_set(self,unknown_bl8);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl8_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl8_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl8 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl8_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl8_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl8_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl8);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl9_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl9_set(self,unknown_bl9);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl9_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl9_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl9 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl9_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl9_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl9_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl9);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl10_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl10_set(self,unknown_bl10);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl10_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl10_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl10 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl10_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl10_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl10_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl10);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl11_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl11_set(self,unknown_bl11);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl11_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl11_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl11 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl11_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl11_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl11_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl11);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl12_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl12_set(self,unknown_bl12);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl12_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl12_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl12 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl12_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl12_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl12_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl12);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl13_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl13_set(self,unknown_bl13);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl13_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl13_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl13 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl13_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl13_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl13_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl13);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl14_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl14_set(self,unknown_bl14);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl14_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl14_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl14 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl14_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl14_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl14_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl14);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl15_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl15_set(self,unknown_bl15);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl15_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl15_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl15 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl15_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl15_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl15_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl15);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl16_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl16_set(self,unknown_bl16);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl16_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl16_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl16 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl16_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl16_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl16_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl16);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl17_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl17_set(self,unknown_bl17);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl17_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl17_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl17 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl17_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl17_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl17_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl17);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl18_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl18_set(self,unknown_bl18);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl18_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl18_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl18 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl18_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl18_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl18_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl18);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl19_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl19_set(self,unknown_bl19);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl19_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl19_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl19 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl19_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl19_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl19_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl19);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl20_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl20_set(self,unknown_bl20);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl20_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl20_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl20 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl20_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl20_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl20_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl20);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl21_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl21_set(self,unknown_bl21);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl21_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl21_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl21 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl21_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl21_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl21_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl21);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl22_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl22_set(self,unknown_bl22);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl22_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl22_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl22 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl22_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl22_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl22_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl22);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl23_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl23_set(self,unknown_bl23);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl23_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl23_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl23 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl23_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl23_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl23_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl23);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl24_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl24_set(self,unknown_bl24);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl24_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl24_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl24 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl24_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl24_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl24_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl24);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl25_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl25_set(self,unknown_bl25);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl25_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl25_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl25 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl25_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl25_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl25_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl25);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl26_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl26_set(self,unknown_bl26);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl26_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl26_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl26 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl26_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl26_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl26_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl26);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl27_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl27_set(self,unknown_bl27);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl27_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl27_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl27 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl27_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl27_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl27_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl27);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl28_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl28_set(self,unknown_bl28);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl28_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl28_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl28 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl28_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl28_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl28_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl28);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl29_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl29_set(self,unknown_bl29);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl29_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl29_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl29 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl29_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl29_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl29_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl29);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl30_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl30_set(self,unknown_bl30);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl30_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl30_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl30 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl30_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl30_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl30_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl30);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl31_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl31_set(self,unknown_bl31);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl31_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl31_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl31 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl31_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl31_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl31_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl31);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl32_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl32_set(self,unknown_bl32);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl32_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl32_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl32 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl32_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl32_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl32_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl32);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl33_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl33_set(self,unknown_bl33);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl33_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl33_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl33 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl33_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl33_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl33_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl33);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl34_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl34_set(self,unknown_bl34);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl34_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl34_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl34 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl34_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl34_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl34_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl34);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl35_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl35_set(self,unknown_bl35);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl35_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl35_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl35 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl35_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl35_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl35_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl35);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl36_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl36_set(self,unknown_bl36);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl36_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl36_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl36 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl36_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl36_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl36_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl36);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_b37_set) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_b37_set(self,unknown_b37);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_b37_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_b37_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown_b37 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_b37_get) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_b37_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_b37_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown_b37);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCPERSSUBENTMANAGER) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCPERSSUBENTMANAGER();");
    }
    result = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)calloc(1, sizeof(struct _dwg_object_ASSOCPERSSUBENTMANAGER));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCPERSSUBENTMANAGER) {
  {
    struct _dwg_object_ASSOCPERSSUBENTMANAGER *arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCPERSSUBENTMANAGER(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCPERSSUBENTMANAGER" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPERSSUBENTMANAGER *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCACTIONPARAM_parent_set) {
  {
    struct _dwg_object_ASSOCACTIONPARAM *arg1 = (struct _dwg_object_ASSOCACTIONPARAM *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCACTIONPARAM_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCACTIONPARAM_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCACTIONPARAM_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCACTIONPARAM_parent_get) {
  {
    struct _dwg_object_ASSOCACTIONPARAM *arg1 = (struct _dwg_object_ASSOCACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCACTIONPARAM_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCACTIONPARAM_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTIONPARAM *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCACTIONPARAM_is_r2013_set) {
  {
    struct _dwg_object_ASSOCACTIONPARAM *arg1 = (struct _dwg_object_ASSOCACTIONPARAM *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCACTIONPARAM_is_r2013_set(self,is_r2013);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCACTIONPARAM_is_r2013_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCACTIONPARAM_is_r2013_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->is_r2013 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCACTIONPARAM_is_r2013_get) {
  {
    struct _dwg_object_ASSOCACTIONPARAM *arg1 = (struct _dwg_object_ASSOCACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCACTIONPARAM_is_r2013_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCACTIONPARAM_is_r2013_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTIONPARAM *)(argp1);
    result = (BITCODE_BS) ((arg1)->is_r2013);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCACTIONPARAM_aap_version_set) {
  {
    struct _dwg_object_ASSOCACTIONPARAM *arg1 = (struct _dwg_object_ASSOCACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCACTIONPARAM_aap_version_set(self,aap_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCACTIONPARAM_aap_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCACTIONPARAM_aap_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->aap_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCACTIONPARAM_aap_version_get) {
  {
    struct _dwg_object_ASSOCACTIONPARAM *arg1 = (struct _dwg_object_ASSOCACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCACTIONPARAM_aap_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCACTIONPARAM_aap_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->aap_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCACTIONPARAM_name_set) {
  {
    struct _dwg_object_ASSOCACTIONPARAM *arg1 = (struct _dwg_object_ASSOCACTIONPARAM *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCACTIONPARAM_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCACTIONPARAM_name_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTIONPARAM *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCACTIONPARAM_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCACTIONPARAM_name_get) {
  {
    struct _dwg_object_ASSOCACTIONPARAM *arg1 = (struct _dwg_object_ASSOCACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCACTIONPARAM_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCACTIONPARAM_name_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTIONPARAM *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCACTIONPARAM) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCACTIONPARAM *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCACTIONPARAM();");
    }
    result = (struct _dwg_object_ASSOCACTIONPARAM *)calloc(1, sizeof(struct _dwg_object_ASSOCACTIONPARAM));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCACTIONPARAM, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCACTIONPARAM) {
  {
    struct _dwg_object_ASSOCACTIONPARAM *arg1 = (struct _dwg_object_ASSOCACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCACTIONPARAM(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCACTIONPARAM, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCACTIONPARAM" "', argument " "1"" of type '" "struct _dwg_object_ASSOCACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCACTIONPARAM *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_parent_set) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_parent_get) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_is_r2013_set) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_is_r2013_set(self,is_r2013);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_is_r2013_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_is_r2013_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->is_r2013 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_is_r2013_get) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_is_r2013_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_is_r2013_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    result = (BITCODE_BS) ((arg1)->is_r2013);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_aap_version_set) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_aap_version_set(self,aap_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_aap_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_aap_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->aap_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_aap_version_get) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_aap_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_aap_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->aap_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_name_set) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_name_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_name_get) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_name_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_class_version_set) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_class_version_get) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_bs1_set) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_bs1_set(self,bs1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_bs1_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_bs1_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->bs1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_bs1_get) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_bs1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_bs1_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    result = (BITCODE_BS) ((arg1)->bs1);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_num_params_set) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_num_params_set(self,num_params);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_num_params_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_num_params_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_params = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_num_params_get) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_num_params_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_num_params_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_params);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_params_set) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_params_set(self,params);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_params_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_params_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->params = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_params_get) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_params_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_params_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    result = (BITCODE_H *) ((arg1)->params);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_has_child_param_set) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_has_child_param_set(self,has_child_param);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_has_child_param_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_has_child_param_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_child_param = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_has_child_param_get) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_has_child_param_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_has_child_param_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    result = (BITCODE_B) ((arg1)->has_child_param);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_status_set) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_status_set(self,child_status);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_status_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_status_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->child_status = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_status_get) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_status_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_status_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    result = (BITCODE_BS) ((arg1)->child_status);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_id_set) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_id_set(self,child_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_id_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_id_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->child_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_id_get) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_id_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->child_id);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_param_set) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_param_set(self,child_param);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_param_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_param_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->child_param = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_param_get) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_param_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_param_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    result = (BITCODE_H) ((arg1)->child_param);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_h330_2_set) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_h330_2_set(self,h330_2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_h330_2_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_h330_2_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->h330_2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_h330_2_get) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_h330_2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_h330_2_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    result = (BITCODE_H) ((arg1)->h330_2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_bl2_set) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_bl2_set(self,bl2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_bl2_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_bl2_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bl2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_bl2_get) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_bl2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_bl2_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->bl2);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_h330_3_set) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_h330_3_set(self,h330_3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_h330_3_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_h330_3_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->h330_3 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_h330_3_get) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_h330_3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_h330_3_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    result = (BITCODE_H) ((arg1)->h330_3);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_status_set) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_status_set(self,status);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_status_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_status_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->status = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_status_get) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_status_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_status_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    result = (BITCODE_BS) ((arg1)->status);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_osnap_mode_set) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_osnap_mode_set(self,osnap_mode);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_osnap_mode_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_osnap_mode_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->osnap_mode = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_osnap_mode_get) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_osnap_mode_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_osnap_mode_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    result = (BITCODE_RC) ((arg1)->osnap_mode);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_param_set) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_param_set(self,param);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_param_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_param_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->param = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_param_get) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_param_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_param_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    result = (BITCODE_BD) ((arg1)->param);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM();");
    }
    result = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)calloc(1, sizeof(struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM) {
  {
    struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_parent_set) {
  {
    struct _dwg_object_ASSOCPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPOINTREFACTIONPARAM_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_parent_get) {
  {
    struct _dwg_object_ASSOCPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPOINTREFACTIONPARAM_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_is_r2013_set) {
  {
    struct _dwg_object_ASSOCPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPOINTREFACTIONPARAM_is_r2013_set(self,is_r2013);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_is_r2013_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_is_r2013_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->is_r2013 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_is_r2013_get) {
  {
    struct _dwg_object_ASSOCPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPOINTREFACTIONPARAM_is_r2013_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_is_r2013_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *)(argp1);
    result = (BITCODE_BS) ((arg1)->is_r2013);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_aap_version_set) {
  {
    struct _dwg_object_ASSOCPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPOINTREFACTIONPARAM_aap_version_set(self,aap_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_aap_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_aap_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->aap_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_aap_version_get) {
  {
    struct _dwg_object_ASSOCPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPOINTREFACTIONPARAM_aap_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_aap_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->aap_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_name_set) {
  {
    struct _dwg_object_ASSOCPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPOINTREFACTIONPARAM_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_name_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_name_get) {
  {
    struct _dwg_object_ASSOCPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPOINTREFACTIONPARAM_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_name_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_class_version_set) {
  {
    struct _dwg_object_ASSOCPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPOINTREFACTIONPARAM_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_class_version_get) {
  {
    struct _dwg_object_ASSOCPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPOINTREFACTIONPARAM_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_bs1_set) {
  {
    struct _dwg_object_ASSOCPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPOINTREFACTIONPARAM_bs1_set(self,bs1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_bs1_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_bs1_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->bs1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_bs1_get) {
  {
    struct _dwg_object_ASSOCPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPOINTREFACTIONPARAM_bs1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_bs1_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *)(argp1);
    result = (BITCODE_BS) ((arg1)->bs1);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_num_params_set) {
  {
    struct _dwg_object_ASSOCPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPOINTREFACTIONPARAM_num_params_set(self,num_params);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_num_params_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_num_params_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_params = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_num_params_get) {
  {
    struct _dwg_object_ASSOCPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPOINTREFACTIONPARAM_num_params_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_num_params_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_params);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_params_set) {
  {
    struct _dwg_object_ASSOCPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPOINTREFACTIONPARAM_params_set(self,params);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_params_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_params_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->params = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_params_get) {
  {
    struct _dwg_object_ASSOCPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPOINTREFACTIONPARAM_params_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_params_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *)(argp1);
    result = (BITCODE_H *) ((arg1)->params);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_has_child_param_set) {
  {
    struct _dwg_object_ASSOCPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPOINTREFACTIONPARAM_has_child_param_set(self,has_child_param);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_has_child_param_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_has_child_param_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_child_param = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_has_child_param_get) {
  {
    struct _dwg_object_ASSOCPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPOINTREFACTIONPARAM_has_child_param_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_has_child_param_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *)(argp1);
    result = (BITCODE_B) ((arg1)->has_child_param);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_status_set) {
  {
    struct _dwg_object_ASSOCPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_status_set(self,child_status);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_status_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_status_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->child_status = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_status_get) {
  {
    struct _dwg_object_ASSOCPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_status_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_status_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *)(argp1);
    result = (BITCODE_BS) ((arg1)->child_status);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_id_set) {
  {
    struct _dwg_object_ASSOCPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_id_set(self,child_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_id_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_id_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->child_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_id_get) {
  {
    struct _dwg_object_ASSOCPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_id_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->child_id);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_param_set) {
  {
    struct _dwg_object_ASSOCPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_param_set(self,child_param);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_param_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_param_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->child_param = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_param_get) {
  {
    struct _dwg_object_ASSOCPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_param_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_param_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *)(argp1);
    result = (BITCODE_H) ((arg1)->child_param);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_h330_2_set) {
  {
    struct _dwg_object_ASSOCPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPOINTREFACTIONPARAM_h330_2_set(self,h330_2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_h330_2_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_h330_2_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->h330_2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_h330_2_get) {
  {
    struct _dwg_object_ASSOCPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPOINTREFACTIONPARAM_h330_2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_h330_2_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *)(argp1);
    result = (BITCODE_H) ((arg1)->h330_2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_bl2_set) {
  {
    struct _dwg_object_ASSOCPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPOINTREFACTIONPARAM_bl2_set(self,bl2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_bl2_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_bl2_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bl2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_bl2_get) {
  {
    struct _dwg_object_ASSOCPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPOINTREFACTIONPARAM_bl2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_bl2_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->bl2);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_h330_3_set) {
  {
    struct _dwg_object_ASSOCPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPOINTREFACTIONPARAM_h330_3_set(self,h330_3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_h330_3_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_h330_3_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->h330_3 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_h330_3_get) {
  {
    struct _dwg_object_ASSOCPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPOINTREFACTIONPARAM_h330_3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM_h330_3_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *)(argp1);
    result = (BITCODE_H) ((arg1)->h330_3);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCPOINTREFACTIONPARAM) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCPOINTREFACTIONPARAM *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCPOINTREFACTIONPARAM();");
    }
    result = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *)calloc(1, sizeof(struct _dwg_object_ASSOCPOINTREFACTIONPARAM));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCPOINTREFACTIONPARAM) {
  {
    struct _dwg_object_ASSOCPOINTREFACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCPOINTREFACTIONPARAM(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCPOINTREFACTIONPARAM" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPOINTREFACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPOINTREFACTIONPARAM *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_parent_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_parent_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_is_r2013_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_is_r2013_set(self,is_r2013);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_is_r2013_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_is_r2013_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->is_r2013 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_is_r2013_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_is_r2013_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_is_r2013_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (BITCODE_BS) ((arg1)->is_r2013);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_aap_version_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_aap_version_set(self,aap_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_aap_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_aap_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->aap_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_aap_version_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_aap_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_aap_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->aap_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_name_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_name_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_name_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_name_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_asdap_class_version_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_asdap_class_version_set(self,asdap_class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_asdap_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_asdap_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->asdap_class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_asdap_class_version_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_asdap_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_asdap_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->asdap_class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_dep_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_dep_set(self,dep);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_dep_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_dep_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->dep = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_dep_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_dep_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_dep_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (BITCODE_H) ((arg1)->dep);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_class_version_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_class_version_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty_set(self,acis_empty);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->acis_empty = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (BITCODE_B) ((arg1)->acis_empty);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_unknown_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_unknown_set(self,unknown);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_unknown_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_unknown_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_unknown_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_unknown_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_unknown_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_version_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_version_set(self,version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_version_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (BITCODE_BS) ((arg1)->version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_num_blocks_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_num_blocks_set(self,num_blocks);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_num_blocks_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_num_blocks_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_blocks = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_num_blocks_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_num_blocks_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_num_blocks_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_blocks);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_block_size_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_block_size_set(self,block_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_block_size_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_block_size_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->block_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_block_size_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_block_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_block_size_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (BITCODE_BL *) ((arg1)->block_size);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_encr_sat_data_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    char **arg2 = (char **) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_encr_sat_data_set(self,encr_sat_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_encr_sat_data_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_encr_sat_data_set" "', argument " "2"" of type '" "char **""'"); 
    }
    arg2 = (char **)(argp2);
    if (arg1) (arg1)->encr_sat_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_encr_sat_data_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    char **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_encr_sat_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_encr_sat_data_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (char **) ((arg1)->encr_sat_data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_sab_size_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_sab_size_set(self,sab_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_sab_size_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_sab_size_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->sab_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_sab_size_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_sab_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_sab_size_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->sab_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_data_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_data_set(self,acis_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_data_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_data_set" "', argument " "2"" of type '" "BITCODE_RC *""'"); 
    }
    arg2 = (BITCODE_RC *)(argp2);
    if (arg1) (arg1)->acis_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_data_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_data_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (BITCODE_RC *) ((arg1)->acis_data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_wireframe_data_present_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_wireframe_data_present_set(self,wireframe_data_present);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_wireframe_data_present_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_wireframe_data_present_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->wireframe_data_present = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_wireframe_data_present_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_wireframe_data_present_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_wireframe_data_present_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (BITCODE_B) ((arg1)->wireframe_data_present);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_point_present_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_point_present_set(self,point_present);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_point_present_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_point_present_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->point_present = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_point_present_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_point_present_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_point_present_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (BITCODE_B) ((arg1)->point_present);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_point_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_point_set(self,point);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_point_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_point_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->point = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_point_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_point_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_point_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->point);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_isolines_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_isolines_set(self,isolines);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_isolines_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_isolines_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->isolines = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_isolines_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_isolines_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_isolines_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->isolines);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_isoline_present_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_isoline_present_set(self,isoline_present);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_isoline_present_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_isoline_present_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->isoline_present = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_isoline_present_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_isoline_present_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_isoline_present_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (BITCODE_B) ((arg1)->isoline_present);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_num_wires_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_num_wires_set(self,num_wires);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_num_wires_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_num_wires_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_wires = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_num_wires_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_num_wires_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_num_wires_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_wires);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_wires_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    Dwg_3DSOLID_wire *arg2 = (Dwg_3DSOLID_wire *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_wires_set(self,wires);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_wires_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_3DSOLID_wire, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_wires_set" "', argument " "2"" of type '" "Dwg_3DSOLID_wire *""'"); 
    }
    arg2 = (Dwg_3DSOLID_wire *)(argp2);
    if (arg1) (arg1)->wires = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_wires_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_3DSOLID_wire *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_wires_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_wires_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (Dwg_3DSOLID_wire *) ((arg1)->wires);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_3DSOLID_wire, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_num_silhouettes_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_num_silhouettes_set(self,num_silhouettes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_num_silhouettes_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_num_silhouettes_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_silhouettes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_num_silhouettes_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_num_silhouettes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_num_silhouettes_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_silhouettes);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_silhouettes_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    Dwg_3DSOLID_silhouette *arg2 = (Dwg_3DSOLID_silhouette *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_silhouettes_set(self,silhouettes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_silhouettes_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_3DSOLID_silhouette, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_silhouettes_set" "', argument " "2"" of type '" "Dwg_3DSOLID_silhouette *""'"); 
    }
    arg2 = (Dwg_3DSOLID_silhouette *)(argp2);
    if (arg1) (arg1)->silhouettes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_silhouettes_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_3DSOLID_silhouette *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_silhouettes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_silhouettes_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (Dwg_3DSOLID_silhouette *) ((arg1)->silhouettes);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_3DSOLID_silhouette, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM__dxf_sab_converted_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM__dxf_sab_converted_set(self,_dxf_sab_converted);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM__dxf_sab_converted_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM__dxf_sab_converted_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->_dxf_sab_converted = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM__dxf_sab_converted_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM__dxf_sab_converted_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM__dxf_sab_converted_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (BITCODE_B) ((arg1)->_dxf_sab_converted);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty2_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty2_set(self,acis_empty2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty2_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty2_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->acis_empty2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty2_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty2_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (BITCODE_B) ((arg1)->acis_empty2);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_extra_acis_data_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    struct _dwg_entity_3DSOLID *arg2 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_extra_acis_data_set(self,extra_acis_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_extra_acis_data_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_3DSOLID, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_extra_acis_data_set" "', argument " "2"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg2 = (struct _dwg_entity_3DSOLID *)(argp2);
    if (arg1) (arg1)->extra_acis_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_extra_acis_data_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_entity_3DSOLID *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_extra_acis_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_extra_acis_data_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (struct _dwg_entity_3DSOLID *) ((arg1)->extra_acis_data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_3DSOLID, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_num_materials_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_num_materials_set(self,num_materials);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_num_materials_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_num_materials_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_materials = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_num_materials_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_num_materials_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_num_materials_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_materials);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_materials_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    Dwg_3DSOLID_material *arg2 = (Dwg_3DSOLID_material *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_materials_set(self,materials);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_materials_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_3DSOLID_material, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_materials_set" "', argument " "2"" of type '" "Dwg_3DSOLID_material *""'"); 
    }
    arg2 = (Dwg_3DSOLID_material *)(argp2);
    if (arg1) (arg1)->materials = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_materials_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_3DSOLID_material *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_materials_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_materials_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (Dwg_3DSOLID_material *) ((arg1)->materials);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_3DSOLID_material, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_guid_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) (BITCODE_RC *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_guid_set(self,revision_guid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_guid_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_guid_set" "', argument " "2"" of type '" "BITCODE_RC [39]""'"); 
    } 
    arg2 = (BITCODE_RC *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)39; ++ii) *(BITCODE_RC *)&arg1->revision_guid[ii] = *((BITCODE_RC *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""revision_guid""' of type '""BITCODE_RC [39]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_guid_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_guid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_guid_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (BITCODE_RC *)(BITCODE_RC *) ((arg1)->revision_guid);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_major_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_major_set(self,revision_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_major_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->revision_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_major_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_major_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->revision_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_minor1_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_minor1_set(self,revision_minor1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_minor1_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_minor1_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->revision_minor1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_minor1_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_minor1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_minor1_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (BITCODE_BS) ((arg1)->revision_minor1);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_minor2_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_minor2_set(self,revision_minor2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_minor2_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_minor2_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->revision_minor2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_minor2_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_minor2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_minor2_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (BITCODE_BS) ((arg1)->revision_minor2);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_bytes_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) (BITCODE_RC *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_bytes_set(self,revision_bytes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_bytes_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_bytes_set" "', argument " "2"" of type '" "BITCODE_RC [9]""'"); 
    } 
    arg2 = (BITCODE_RC *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)9; ++ii) *(BITCODE_RC *)&arg1->revision_bytes[ii] = *((BITCODE_RC *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""revision_bytes""' of type '""BITCODE_RC [9]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_bytes_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_bytes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_bytes_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (BITCODE_RC *)(BITCODE_RC *) ((arg1)->revision_bytes);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_end_marker_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_end_marker_set(self,end_marker);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_end_marker_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_end_marker_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->end_marker = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_end_marker_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_end_marker_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_end_marker_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->end_marker);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_history_id_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_history_id_set(self,history_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_history_id_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_history_id_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->history_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_history_id_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_history_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_history_id_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (BITCODE_H) ((arg1)->history_id);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_has_revision_guid_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_has_revision_guid_set(self,has_revision_guid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_has_revision_guid_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_has_revision_guid_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_revision_guid = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_has_revision_guid_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_has_revision_guid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_has_revision_guid_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (BITCODE_B) ((arg1)->has_revision_guid);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty_bit_set) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty_bit_set(self,acis_empty_bit);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty_bit_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty_bit_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->acis_empty_bit = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty_bit_get) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty_bit_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty_bit_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    result = (BITCODE_B) ((arg1)->acis_empty_bit);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCASMBODYACTIONPARAM) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCASMBODYACTIONPARAM();");
    }
    result = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)calloc(1, sizeof(struct _dwg_object_ASSOCASMBODYACTIONPARAM));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCASMBODYACTIONPARAM) {
  {
    struct _dwg_object_ASSOCASMBODYACTIONPARAM *arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCASMBODYACTIONPARAM(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCASMBODYACTIONPARAM" "', argument " "1"" of type '" "struct _dwg_object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCASMBODYACTIONPARAM *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_parent_set) {
  {
    struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_parent_get) {
  {
    struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_is_r2013_set) {
  {
    struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_is_r2013_set(self,is_r2013);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_is_r2013_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_is_r2013_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->is_r2013 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_is_r2013_get) {
  {
    struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_is_r2013_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_is_r2013_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *)(argp1);
    result = (BITCODE_BS) ((arg1)->is_r2013);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_aap_version_set) {
  {
    struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_aap_version_set(self,aap_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_aap_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_aap_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->aap_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_aap_version_get) {
  {
    struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_aap_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_aap_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->aap_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_name_set) {
  {
    struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_name_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_name_get) {
  {
    struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_name_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_class_version_set) {
  {
    struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_class_version_get) {
  {
    struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_bs1_set) {
  {
    struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_bs1_set(self,bs1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_bs1_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_bs1_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->bs1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_bs1_get) {
  {
    struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_bs1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_bs1_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *)(argp1);
    result = (BITCODE_BS) ((arg1)->bs1);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_num_params_set) {
  {
    struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_num_params_set(self,num_params);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_num_params_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_num_params_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_params = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_num_params_get) {
  {
    struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_num_params_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_num_params_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_params);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_params_set) {
  {
    struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_params_set(self,params);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_params_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_params_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->params = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_params_get) {
  {
    struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_params_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_params_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *)(argp1);
    result = (BITCODE_H *) ((arg1)->params);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_has_child_param_set) {
  {
    struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_has_child_param_set(self,has_child_param);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_has_child_param_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_has_child_param_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_child_param = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_has_child_param_get) {
  {
    struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_has_child_param_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_has_child_param_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *)(argp1);
    result = (BITCODE_B) ((arg1)->has_child_param);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_status_set) {
  {
    struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_status_set(self,child_status);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_status_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_status_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->child_status = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_status_get) {
  {
    struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_status_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_status_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *)(argp1);
    result = (BITCODE_BS) ((arg1)->child_status);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_id_set) {
  {
    struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_id_set(self,child_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_id_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_id_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->child_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_id_get) {
  {
    struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_id_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->child_id);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_param_set) {
  {
    struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_param_set(self,child_param);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_param_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_param_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->child_param = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_param_get) {
  {
    struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_param_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_param_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *)(argp1);
    result = (BITCODE_H) ((arg1)->child_param);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_h330_2_set) {
  {
    struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_h330_2_set(self,h330_2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_h330_2_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_h330_2_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->h330_2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_h330_2_get) {
  {
    struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_h330_2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_h330_2_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *)(argp1);
    result = (BITCODE_H) ((arg1)->h330_2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_bl2_set) {
  {
    struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_bl2_set(self,bl2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_bl2_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_bl2_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bl2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_bl2_get) {
  {
    struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_bl2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_bl2_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->bl2);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_h330_3_set) {
  {
    struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_h330_3_set(self,h330_3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_h330_3_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_h330_3_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->h330_3 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_h330_3_get) {
  {
    struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_h330_3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_h330_3_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *)(argp1);
    result = (BITCODE_H) ((arg1)->h330_3);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM();");
    }
    result = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *)calloc(1, sizeof(struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM) {
  {
    struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM" "', argument " "1"" of type '" "struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOBJECTACTIONPARAM_parent_set) {
  {
    struct _dwg_object_ASSOCOBJECTACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOBJECTACTIONPARAM *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOBJECTACTIONPARAM_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOBJECTACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOBJECTACTIONPARAM_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOBJECTACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOBJECTACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCOBJECTACTIONPARAM_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOBJECTACTIONPARAM_parent_get) {
  {
    struct _dwg_object_ASSOCOBJECTACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOBJECTACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOBJECTACTIONPARAM_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOBJECTACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOBJECTACTIONPARAM_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOBJECTACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOBJECTACTIONPARAM *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOBJECTACTIONPARAM_is_r2013_set) {
  {
    struct _dwg_object_ASSOCOBJECTACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOBJECTACTIONPARAM *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOBJECTACTIONPARAM_is_r2013_set(self,is_r2013);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOBJECTACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOBJECTACTIONPARAM_is_r2013_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOBJECTACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOBJECTACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCOBJECTACTIONPARAM_is_r2013_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->is_r2013 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOBJECTACTIONPARAM_is_r2013_get) {
  {
    struct _dwg_object_ASSOCOBJECTACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOBJECTACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOBJECTACTIONPARAM_is_r2013_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOBJECTACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOBJECTACTIONPARAM_is_r2013_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOBJECTACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOBJECTACTIONPARAM *)(argp1);
    result = (BITCODE_BS) ((arg1)->is_r2013);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOBJECTACTIONPARAM_aap_version_set) {
  {
    struct _dwg_object_ASSOCOBJECTACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOBJECTACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOBJECTACTIONPARAM_aap_version_set(self,aap_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOBJECTACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOBJECTACTIONPARAM_aap_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOBJECTACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOBJECTACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCOBJECTACTIONPARAM_aap_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->aap_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOBJECTACTIONPARAM_aap_version_get) {
  {
    struct _dwg_object_ASSOCOBJECTACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOBJECTACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOBJECTACTIONPARAM_aap_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOBJECTACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOBJECTACTIONPARAM_aap_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOBJECTACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOBJECTACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->aap_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOBJECTACTIONPARAM_name_set) {
  {
    struct _dwg_object_ASSOCOBJECTACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOBJECTACTIONPARAM *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOBJECTACTIONPARAM_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOBJECTACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOBJECTACTIONPARAM_name_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOBJECTACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOBJECTACTIONPARAM *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCOBJECTACTIONPARAM_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOBJECTACTIONPARAM_name_get) {
  {
    struct _dwg_object_ASSOCOBJECTACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOBJECTACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOBJECTACTIONPARAM_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOBJECTACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOBJECTACTIONPARAM_name_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOBJECTACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOBJECTACTIONPARAM *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOBJECTACTIONPARAM_asdap_class_version_set) {
  {
    struct _dwg_object_ASSOCOBJECTACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOBJECTACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOBJECTACTIONPARAM_asdap_class_version_set(self,asdap_class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOBJECTACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOBJECTACTIONPARAM_asdap_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOBJECTACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOBJECTACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCOBJECTACTIONPARAM_asdap_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->asdap_class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOBJECTACTIONPARAM_asdap_class_version_get) {
  {
    struct _dwg_object_ASSOCOBJECTACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOBJECTACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOBJECTACTIONPARAM_asdap_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOBJECTACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOBJECTACTIONPARAM_asdap_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOBJECTACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOBJECTACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->asdap_class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOBJECTACTIONPARAM_dep_set) {
  {
    struct _dwg_object_ASSOCOBJECTACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOBJECTACTIONPARAM *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOBJECTACTIONPARAM_dep_set(self,dep);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOBJECTACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOBJECTACTIONPARAM_dep_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOBJECTACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOBJECTACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCOBJECTACTIONPARAM_dep_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->dep = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOBJECTACTIONPARAM_dep_get) {
  {
    struct _dwg_object_ASSOCOBJECTACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOBJECTACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOBJECTACTIONPARAM_dep_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOBJECTACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOBJECTACTIONPARAM_dep_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOBJECTACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOBJECTACTIONPARAM *)(argp1);
    result = (BITCODE_H) ((arg1)->dep);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOBJECTACTIONPARAM_class_version_set) {
  {
    struct _dwg_object_ASSOCOBJECTACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOBJECTACTIONPARAM *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOBJECTACTIONPARAM_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOBJECTACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOBJECTACTIONPARAM_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOBJECTACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOBJECTACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCOBJECTACTIONPARAM_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOBJECTACTIONPARAM_class_version_get) {
  {
    struct _dwg_object_ASSOCOBJECTACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOBJECTACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOBJECTACTIONPARAM_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOBJECTACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOBJECTACTIONPARAM_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOBJECTACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOBJECTACTIONPARAM *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCOBJECTACTIONPARAM) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCOBJECTACTIONPARAM *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCOBJECTACTIONPARAM();");
    }
    result = (struct _dwg_object_ASSOCOBJECTACTIONPARAM *)calloc(1, sizeof(struct _dwg_object_ASSOCOBJECTACTIONPARAM));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCOBJECTACTIONPARAM, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCOBJECTACTIONPARAM) {
  {
    struct _dwg_object_ASSOCOBJECTACTIONPARAM *arg1 = (struct _dwg_object_ASSOCOBJECTACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCOBJECTACTIONPARAM(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOBJECTACTIONPARAM, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCOBJECTACTIONPARAM" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOBJECTACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOBJECTACTIONPARAM *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_parent_set) {
  {
    struct _dwg_object_ASSOCEDGEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGEACTIONPARAM_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGEACTIONPARAM_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCEDGEACTIONPARAM_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_parent_get) {
  {
    struct _dwg_object_ASSOCEDGEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGEACTIONPARAM_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGEACTIONPARAM_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_is_r2013_set) {
  {
    struct _dwg_object_ASSOCEDGEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGEACTIONPARAM_is_r2013_set(self,is_r2013);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGEACTIONPARAM_is_r2013_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCEDGEACTIONPARAM_is_r2013_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->is_r2013 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_is_r2013_get) {
  {
    struct _dwg_object_ASSOCEDGEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGEACTIONPARAM_is_r2013_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGEACTIONPARAM_is_r2013_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *)(argp1);
    result = (BITCODE_BS) ((arg1)->is_r2013);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_aap_version_set) {
  {
    struct _dwg_object_ASSOCEDGEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGEACTIONPARAM_aap_version_set(self,aap_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGEACTIONPARAM_aap_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCEDGEACTIONPARAM_aap_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->aap_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_aap_version_get) {
  {
    struct _dwg_object_ASSOCEDGEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGEACTIONPARAM_aap_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGEACTIONPARAM_aap_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->aap_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_name_set) {
  {
    struct _dwg_object_ASSOCEDGEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGEACTIONPARAM_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGEACTIONPARAM_name_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCEDGEACTIONPARAM_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_name_get) {
  {
    struct _dwg_object_ASSOCEDGEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGEACTIONPARAM_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGEACTIONPARAM_name_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_asdap_class_version_set) {
  {
    struct _dwg_object_ASSOCEDGEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGEACTIONPARAM_asdap_class_version_set(self,asdap_class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGEACTIONPARAM_asdap_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCEDGEACTIONPARAM_asdap_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->asdap_class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_asdap_class_version_get) {
  {
    struct _dwg_object_ASSOCEDGEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGEACTIONPARAM_asdap_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGEACTIONPARAM_asdap_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->asdap_class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_dep_set) {
  {
    struct _dwg_object_ASSOCEDGEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGEACTIONPARAM_dep_set(self,dep);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGEACTIONPARAM_dep_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCEDGEACTIONPARAM_dep_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->dep = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_dep_get) {
  {
    struct _dwg_object_ASSOCEDGEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGEACTIONPARAM_dep_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGEACTIONPARAM_dep_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *)(argp1);
    result = (BITCODE_H) ((arg1)->dep);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_class_version_set) {
  {
    struct _dwg_object_ASSOCEDGEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGEACTIONPARAM_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGEACTIONPARAM_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCEDGEACTIONPARAM_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_class_version_get) {
  {
    struct _dwg_object_ASSOCEDGEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGEACTIONPARAM_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGEACTIONPARAM_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_param_set) {
  {
    struct _dwg_object_ASSOCEDGEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGEACTIONPARAM_param_set(self,param);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGEACTIONPARAM_param_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCEDGEACTIONPARAM_param_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->param = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_param_get) {
  {
    struct _dwg_object_ASSOCEDGEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGEACTIONPARAM_param_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGEACTIONPARAM_param_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *)(argp1);
    result = (BITCODE_H) ((arg1)->param);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_has_action_set) {
  {
    struct _dwg_object_ASSOCEDGEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGEACTIONPARAM_has_action_set(self,has_action);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGEACTIONPARAM_has_action_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCEDGEACTIONPARAM_has_action_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_action = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_has_action_get) {
  {
    struct _dwg_object_ASSOCEDGEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGEACTIONPARAM_has_action_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGEACTIONPARAM_has_action_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *)(argp1);
    result = (BITCODE_B) ((arg1)->has_action);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_action_type_set) {
  {
    struct _dwg_object_ASSOCEDGEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGEACTIONPARAM_action_type_set(self,action_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGEACTIONPARAM_action_type_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCEDGEACTIONPARAM_action_type_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->action_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_action_type_get) {
  {
    struct _dwg_object_ASSOCEDGEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGEACTIONPARAM_action_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGEACTIONPARAM_action_type_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->action_type);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_subent_set) {
  {
    struct _dwg_object_ASSOCEDGEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGEACTIONPARAM_subent_set(self,subent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGEACTIONPARAM_subent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCEDGEACTIONPARAM_subent_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->subent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_subent_get) {
  {
    struct _dwg_object_ASSOCEDGEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGEACTIONPARAM_subent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGEACTIONPARAM_subent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *)(argp1);
    result = (BITCODE_H) ((arg1)->subent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCEDGEACTIONPARAM) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCEDGEACTIONPARAM *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCEDGEACTIONPARAM();");
    }
    result = (struct _dwg_object_ASSOCEDGEACTIONPARAM *)calloc(1, sizeof(struct _dwg_object_ASSOCEDGEACTIONPARAM));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCEDGEACTIONPARAM, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCEDGEACTIONPARAM) {
  {
    struct _dwg_object_ASSOCEDGEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCEDGEACTIONPARAM(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGEACTIONPARAM, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCEDGEACTIONPARAM" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGEACTIONPARAM *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCFACEACTIONPARAM_parent_set) {
  {
    struct _dwg_object_ASSOCFACEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCFACEACTIONPARAM *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCFACEACTIONPARAM_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFACEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCFACEACTIONPARAM_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFACEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFACEACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCFACEACTIONPARAM_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCFACEACTIONPARAM_parent_get) {
  {
    struct _dwg_object_ASSOCFACEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCFACEACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCFACEACTIONPARAM_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFACEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCFACEACTIONPARAM_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFACEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFACEACTIONPARAM *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCFACEACTIONPARAM_is_r2013_set) {
  {
    struct _dwg_object_ASSOCFACEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCFACEACTIONPARAM *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCFACEACTIONPARAM_is_r2013_set(self,is_r2013);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFACEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCFACEACTIONPARAM_is_r2013_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFACEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFACEACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCFACEACTIONPARAM_is_r2013_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->is_r2013 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCFACEACTIONPARAM_is_r2013_get) {
  {
    struct _dwg_object_ASSOCFACEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCFACEACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCFACEACTIONPARAM_is_r2013_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFACEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCFACEACTIONPARAM_is_r2013_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFACEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFACEACTIONPARAM *)(argp1);
    result = (BITCODE_BS) ((arg1)->is_r2013);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCFACEACTIONPARAM_aap_version_set) {
  {
    struct _dwg_object_ASSOCFACEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCFACEACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCFACEACTIONPARAM_aap_version_set(self,aap_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFACEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCFACEACTIONPARAM_aap_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFACEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFACEACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCFACEACTIONPARAM_aap_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->aap_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCFACEACTIONPARAM_aap_version_get) {
  {
    struct _dwg_object_ASSOCFACEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCFACEACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCFACEACTIONPARAM_aap_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFACEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCFACEACTIONPARAM_aap_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFACEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFACEACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->aap_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCFACEACTIONPARAM_name_set) {
  {
    struct _dwg_object_ASSOCFACEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCFACEACTIONPARAM *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCFACEACTIONPARAM_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFACEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCFACEACTIONPARAM_name_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFACEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFACEACTIONPARAM *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCFACEACTIONPARAM_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCFACEACTIONPARAM_name_get) {
  {
    struct _dwg_object_ASSOCFACEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCFACEACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCFACEACTIONPARAM_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFACEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCFACEACTIONPARAM_name_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFACEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFACEACTIONPARAM *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCFACEACTIONPARAM_asdap_class_version_set) {
  {
    struct _dwg_object_ASSOCFACEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCFACEACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCFACEACTIONPARAM_asdap_class_version_set(self,asdap_class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFACEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCFACEACTIONPARAM_asdap_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFACEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFACEACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCFACEACTIONPARAM_asdap_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->asdap_class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCFACEACTIONPARAM_asdap_class_version_get) {
  {
    struct _dwg_object_ASSOCFACEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCFACEACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCFACEACTIONPARAM_asdap_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFACEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCFACEACTIONPARAM_asdap_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFACEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFACEACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->asdap_class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCFACEACTIONPARAM_dep_set) {
  {
    struct _dwg_object_ASSOCFACEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCFACEACTIONPARAM *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCFACEACTIONPARAM_dep_set(self,dep);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFACEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCFACEACTIONPARAM_dep_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFACEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFACEACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCFACEACTIONPARAM_dep_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->dep = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCFACEACTIONPARAM_dep_get) {
  {
    struct _dwg_object_ASSOCFACEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCFACEACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCFACEACTIONPARAM_dep_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFACEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCFACEACTIONPARAM_dep_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFACEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFACEACTIONPARAM *)(argp1);
    result = (BITCODE_H) ((arg1)->dep);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCFACEACTIONPARAM_class_version_set) {
  {
    struct _dwg_object_ASSOCFACEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCFACEACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCFACEACTIONPARAM_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFACEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCFACEACTIONPARAM_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFACEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFACEACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCFACEACTIONPARAM_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCFACEACTIONPARAM_class_version_get) {
  {
    struct _dwg_object_ASSOCFACEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCFACEACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCFACEACTIONPARAM_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFACEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCFACEACTIONPARAM_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFACEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFACEACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCFACEACTIONPARAM_index_set) {
  {
    struct _dwg_object_ASSOCFACEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCFACEACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCFACEACTIONPARAM_index_set(self,index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFACEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCFACEACTIONPARAM_index_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFACEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFACEACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCFACEACTIONPARAM_index_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCFACEACTIONPARAM_index_get) {
  {
    struct _dwg_object_ASSOCFACEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCFACEACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCFACEACTIONPARAM_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFACEACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCFACEACTIONPARAM_index_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFACEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFACEACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->index);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCFACEACTIONPARAM) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCFACEACTIONPARAM *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCFACEACTIONPARAM();");
    }
    result = (struct _dwg_object_ASSOCFACEACTIONPARAM *)calloc(1, sizeof(struct _dwg_object_ASSOCFACEACTIONPARAM));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCFACEACTIONPARAM, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCFACEACTIONPARAM) {
  {
    struct _dwg_object_ASSOCFACEACTIONPARAM *arg1 = (struct _dwg_object_ASSOCFACEACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCFACEACTIONPARAM(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFACEACTIONPARAM, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCFACEACTIONPARAM" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFACEACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFACEACTIONPARAM *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATHACTIONPARAM_parent_set) {
  {
    struct _dwg_object_ASSOCPATHACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATHACTIONPARAM_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATHACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATHACTIONPARAM_parent_get) {
  {
    struct _dwg_object_ASSOCPATHACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATHACTIONPARAM_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATHACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATHACTIONPARAM_is_r2013_set) {
  {
    struct _dwg_object_ASSOCPATHACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATHACTIONPARAM_is_r2013_set(self,is_r2013);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_is_r2013_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATHACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_is_r2013_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->is_r2013 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATHACTIONPARAM_is_r2013_get) {
  {
    struct _dwg_object_ASSOCPATHACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATHACTIONPARAM_is_r2013_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_is_r2013_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATHACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *)(argp1);
    result = (BITCODE_BS) ((arg1)->is_r2013);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATHACTIONPARAM_aap_version_set) {
  {
    struct _dwg_object_ASSOCPATHACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATHACTIONPARAM_aap_version_set(self,aap_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_aap_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATHACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_aap_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->aap_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATHACTIONPARAM_aap_version_get) {
  {
    struct _dwg_object_ASSOCPATHACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATHACTIONPARAM_aap_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_aap_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATHACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->aap_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATHACTIONPARAM_name_set) {
  {
    struct _dwg_object_ASSOCPATHACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATHACTIONPARAM_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_name_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATHACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATHACTIONPARAM_name_get) {
  {
    struct _dwg_object_ASSOCPATHACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATHACTIONPARAM_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_name_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATHACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATHACTIONPARAM_class_version_set) {
  {
    struct _dwg_object_ASSOCPATHACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATHACTIONPARAM_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATHACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATHACTIONPARAM_class_version_get) {
  {
    struct _dwg_object_ASSOCPATHACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATHACTIONPARAM_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATHACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATHACTIONPARAM_bs1_set) {
  {
    struct _dwg_object_ASSOCPATHACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATHACTIONPARAM_bs1_set(self,bs1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_bs1_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATHACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_bs1_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->bs1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATHACTIONPARAM_bs1_get) {
  {
    struct _dwg_object_ASSOCPATHACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATHACTIONPARAM_bs1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_bs1_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATHACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *)(argp1);
    result = (BITCODE_BS) ((arg1)->bs1);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATHACTIONPARAM_num_params_set) {
  {
    struct _dwg_object_ASSOCPATHACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATHACTIONPARAM_num_params_set(self,num_params);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_num_params_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATHACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_num_params_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_params = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATHACTIONPARAM_num_params_get) {
  {
    struct _dwg_object_ASSOCPATHACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATHACTIONPARAM_num_params_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_num_params_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATHACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_params);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATHACTIONPARAM_params_set) {
  {
    struct _dwg_object_ASSOCPATHACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATHACTIONPARAM_params_set(self,params);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_params_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATHACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_params_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->params = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATHACTIONPARAM_params_get) {
  {
    struct _dwg_object_ASSOCPATHACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATHACTIONPARAM_params_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_params_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATHACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *)(argp1);
    result = (BITCODE_H *) ((arg1)->params);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATHACTIONPARAM_has_child_param_set) {
  {
    struct _dwg_object_ASSOCPATHACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATHACTIONPARAM_has_child_param_set(self,has_child_param);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_has_child_param_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATHACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_has_child_param_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_child_param = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATHACTIONPARAM_has_child_param_get) {
  {
    struct _dwg_object_ASSOCPATHACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATHACTIONPARAM_has_child_param_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_has_child_param_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATHACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *)(argp1);
    result = (BITCODE_B) ((arg1)->has_child_param);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATHACTIONPARAM_child_status_set) {
  {
    struct _dwg_object_ASSOCPATHACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATHACTIONPARAM_child_status_set(self,child_status);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_child_status_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATHACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_child_status_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->child_status = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATHACTIONPARAM_child_status_get) {
  {
    struct _dwg_object_ASSOCPATHACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATHACTIONPARAM_child_status_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_child_status_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATHACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *)(argp1);
    result = (BITCODE_BS) ((arg1)->child_status);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATHACTIONPARAM_child_id_set) {
  {
    struct _dwg_object_ASSOCPATHACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATHACTIONPARAM_child_id_set(self,child_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_child_id_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATHACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_child_id_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->child_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATHACTIONPARAM_child_id_get) {
  {
    struct _dwg_object_ASSOCPATHACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATHACTIONPARAM_child_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_child_id_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATHACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->child_id);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATHACTIONPARAM_child_param_set) {
  {
    struct _dwg_object_ASSOCPATHACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATHACTIONPARAM_child_param_set(self,child_param);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_child_param_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATHACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_child_param_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->child_param = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATHACTIONPARAM_child_param_get) {
  {
    struct _dwg_object_ASSOCPATHACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATHACTIONPARAM_child_param_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_child_param_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATHACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *)(argp1);
    result = (BITCODE_H) ((arg1)->child_param);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATHACTIONPARAM_h330_2_set) {
  {
    struct _dwg_object_ASSOCPATHACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATHACTIONPARAM_h330_2_set(self,h330_2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_h330_2_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATHACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_h330_2_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->h330_2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATHACTIONPARAM_h330_2_get) {
  {
    struct _dwg_object_ASSOCPATHACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATHACTIONPARAM_h330_2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_h330_2_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATHACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *)(argp1);
    result = (BITCODE_H) ((arg1)->h330_2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATHACTIONPARAM_bl2_set) {
  {
    struct _dwg_object_ASSOCPATHACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATHACTIONPARAM_bl2_set(self,bl2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_bl2_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATHACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_bl2_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bl2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATHACTIONPARAM_bl2_get) {
  {
    struct _dwg_object_ASSOCPATHACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATHACTIONPARAM_bl2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_bl2_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATHACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->bl2);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATHACTIONPARAM_h330_3_set) {
  {
    struct _dwg_object_ASSOCPATHACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATHACTIONPARAM_h330_3_set(self,h330_3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_h330_3_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATHACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_h330_3_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->h330_3 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATHACTIONPARAM_h330_3_get) {
  {
    struct _dwg_object_ASSOCPATHACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATHACTIONPARAM_h330_3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_h330_3_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATHACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *)(argp1);
    result = (BITCODE_H) ((arg1)->h330_3);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATHACTIONPARAM_version_set) {
  {
    struct _dwg_object_ASSOCPATHACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATHACTIONPARAM_version_set(self,version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATHACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATHACTIONPARAM_version_get) {
  {
    struct _dwg_object_ASSOCPATHACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATHACTIONPARAM_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATHACTIONPARAM_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATHACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCPATHACTIONPARAM) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCPATHACTIONPARAM *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCPATHACTIONPARAM();");
    }
    result = (struct _dwg_object_ASSOCPATHACTIONPARAM *)calloc(1, sizeof(struct _dwg_object_ASSOCPATHACTIONPARAM));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCPATHACTIONPARAM) {
  {
    struct _dwg_object_ASSOCPATHACTIONPARAM *arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCPATHACTIONPARAM(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCPATHACTIONPARAM" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATHACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATHACTIONPARAM *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVERTEXACTIONPARAM_parent_set) {
  {
    struct _dwg_object_ASSOCVERTEXACTIONPARAM *arg1 = (struct _dwg_object_ASSOCVERTEXACTIONPARAM *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVERTEXACTIONPARAM_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVERTEXACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVERTEXACTIONPARAM_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVERTEXACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVERTEXACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCVERTEXACTIONPARAM_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVERTEXACTIONPARAM_parent_get) {
  {
    struct _dwg_object_ASSOCVERTEXACTIONPARAM *arg1 = (struct _dwg_object_ASSOCVERTEXACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVERTEXACTIONPARAM_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVERTEXACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVERTEXACTIONPARAM_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVERTEXACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVERTEXACTIONPARAM *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVERTEXACTIONPARAM_is_r2013_set) {
  {
    struct _dwg_object_ASSOCVERTEXACTIONPARAM *arg1 = (struct _dwg_object_ASSOCVERTEXACTIONPARAM *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVERTEXACTIONPARAM_is_r2013_set(self,is_r2013);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVERTEXACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVERTEXACTIONPARAM_is_r2013_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVERTEXACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVERTEXACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCVERTEXACTIONPARAM_is_r2013_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->is_r2013 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVERTEXACTIONPARAM_is_r2013_get) {
  {
    struct _dwg_object_ASSOCVERTEXACTIONPARAM *arg1 = (struct _dwg_object_ASSOCVERTEXACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVERTEXACTIONPARAM_is_r2013_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVERTEXACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVERTEXACTIONPARAM_is_r2013_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVERTEXACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVERTEXACTIONPARAM *)(argp1);
    result = (BITCODE_BS) ((arg1)->is_r2013);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVERTEXACTIONPARAM_aap_version_set) {
  {
    struct _dwg_object_ASSOCVERTEXACTIONPARAM *arg1 = (struct _dwg_object_ASSOCVERTEXACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVERTEXACTIONPARAM_aap_version_set(self,aap_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVERTEXACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVERTEXACTIONPARAM_aap_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVERTEXACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVERTEXACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCVERTEXACTIONPARAM_aap_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->aap_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVERTEXACTIONPARAM_aap_version_get) {
  {
    struct _dwg_object_ASSOCVERTEXACTIONPARAM *arg1 = (struct _dwg_object_ASSOCVERTEXACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVERTEXACTIONPARAM_aap_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVERTEXACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVERTEXACTIONPARAM_aap_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVERTEXACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVERTEXACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->aap_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVERTEXACTIONPARAM_name_set) {
  {
    struct _dwg_object_ASSOCVERTEXACTIONPARAM *arg1 = (struct _dwg_object_ASSOCVERTEXACTIONPARAM *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVERTEXACTIONPARAM_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVERTEXACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVERTEXACTIONPARAM_name_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVERTEXACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVERTEXACTIONPARAM *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCVERTEXACTIONPARAM_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVERTEXACTIONPARAM_name_get) {
  {
    struct _dwg_object_ASSOCVERTEXACTIONPARAM *arg1 = (struct _dwg_object_ASSOCVERTEXACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVERTEXACTIONPARAM_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVERTEXACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVERTEXACTIONPARAM_name_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVERTEXACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVERTEXACTIONPARAM *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVERTEXACTIONPARAM_asdap_class_version_set) {
  {
    struct _dwg_object_ASSOCVERTEXACTIONPARAM *arg1 = (struct _dwg_object_ASSOCVERTEXACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVERTEXACTIONPARAM_asdap_class_version_set(self,asdap_class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVERTEXACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVERTEXACTIONPARAM_asdap_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVERTEXACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVERTEXACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCVERTEXACTIONPARAM_asdap_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->asdap_class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVERTEXACTIONPARAM_asdap_class_version_get) {
  {
    struct _dwg_object_ASSOCVERTEXACTIONPARAM *arg1 = (struct _dwg_object_ASSOCVERTEXACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVERTEXACTIONPARAM_asdap_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVERTEXACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVERTEXACTIONPARAM_asdap_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVERTEXACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVERTEXACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->asdap_class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVERTEXACTIONPARAM_dep_set) {
  {
    struct _dwg_object_ASSOCVERTEXACTIONPARAM *arg1 = (struct _dwg_object_ASSOCVERTEXACTIONPARAM *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVERTEXACTIONPARAM_dep_set(self,dep);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVERTEXACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVERTEXACTIONPARAM_dep_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVERTEXACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVERTEXACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCVERTEXACTIONPARAM_dep_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->dep = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVERTEXACTIONPARAM_dep_get) {
  {
    struct _dwg_object_ASSOCVERTEXACTIONPARAM *arg1 = (struct _dwg_object_ASSOCVERTEXACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVERTEXACTIONPARAM_dep_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVERTEXACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVERTEXACTIONPARAM_dep_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVERTEXACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVERTEXACTIONPARAM *)(argp1);
    result = (BITCODE_H) ((arg1)->dep);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVERTEXACTIONPARAM_class_version_set) {
  {
    struct _dwg_object_ASSOCVERTEXACTIONPARAM *arg1 = (struct _dwg_object_ASSOCVERTEXACTIONPARAM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVERTEXACTIONPARAM_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVERTEXACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVERTEXACTIONPARAM_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVERTEXACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVERTEXACTIONPARAM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCVERTEXACTIONPARAM_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVERTEXACTIONPARAM_class_version_get) {
  {
    struct _dwg_object_ASSOCVERTEXACTIONPARAM *arg1 = (struct _dwg_object_ASSOCVERTEXACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVERTEXACTIONPARAM_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVERTEXACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVERTEXACTIONPARAM_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVERTEXACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVERTEXACTIONPARAM *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVERTEXACTIONPARAM_pt_set) {
  {
    struct _dwg_object_ASSOCVERTEXACTIONPARAM *arg1 = (struct _dwg_object_ASSOCVERTEXACTIONPARAM *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVERTEXACTIONPARAM_pt_set(self,pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVERTEXACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVERTEXACTIONPARAM_pt_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVERTEXACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVERTEXACTIONPARAM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCVERTEXACTIONPARAM_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCVERTEXACTIONPARAM_pt_get) {
  {
    struct _dwg_object_ASSOCVERTEXACTIONPARAM *arg1 = (struct _dwg_object_ASSOCVERTEXACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCVERTEXACTIONPARAM_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVERTEXACTIONPARAM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCVERTEXACTIONPARAM_pt_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVERTEXACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVERTEXACTIONPARAM *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCVERTEXACTIONPARAM) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCVERTEXACTIONPARAM *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCVERTEXACTIONPARAM();");
    }
    result = (struct _dwg_object_ASSOCVERTEXACTIONPARAM *)calloc(1, sizeof(struct _dwg_object_ASSOCVERTEXACTIONPARAM));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCVERTEXACTIONPARAM, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCVERTEXACTIONPARAM) {
  {
    struct _dwg_object_ASSOCVERTEXACTIONPARAM *arg1 = (struct _dwg_object_ASSOCVERTEXACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCVERTEXACTIONPARAM(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCVERTEXACTIONPARAM, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCVERTEXACTIONPARAM" "', argument " "1"" of type '" "struct _dwg_object_ASSOCVERTEXACTIONPARAM *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCVERTEXACTIONPARAM *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCARRAYITEM_parent_set) {
  {
    struct _dwg_ASSOCARRAYITEM *arg1 = (struct _dwg_ASSOCARRAYITEM *) 0 ;
    struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *arg2 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ASSOCARRAYITEM_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCARRAYITEM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCARRAYITEM_parent_set" "', argument " "1"" of type '" "struct _dwg_ASSOCARRAYITEM *""'"); 
    }
    arg1 = (struct _dwg_ASSOCARRAYITEM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_ASSOCARRAYITEM_parent_set" "', argument " "2"" of type '" "struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *""'"); 
    }
    arg2 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCARRAYITEM_parent_get) {
  {
    struct _dwg_ASSOCARRAYITEM *arg1 = (struct _dwg_ASSOCARRAYITEM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ASSOCARRAYITEM_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCARRAYITEM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCARRAYITEM_parent_get" "', argument " "1"" of type '" "struct _dwg_ASSOCARRAYITEM *""'"); 
    }
    arg1 = (struct _dwg_ASSOCARRAYITEM *)(argp1);
    result = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCARRAYITEM_class_version_set) {
  {
    struct _dwg_ASSOCARRAYITEM *arg1 = (struct _dwg_ASSOCARRAYITEM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ASSOCARRAYITEM_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCARRAYITEM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCARRAYITEM_class_version_set" "', argument " "1"" of type '" "struct _dwg_ASSOCARRAYITEM *""'"); 
    }
    arg1 = (struct _dwg_ASSOCARRAYITEM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ASSOCARRAYITEM_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCARRAYITEM_class_version_get) {
  {
    struct _dwg_ASSOCARRAYITEM *arg1 = (struct _dwg_ASSOCARRAYITEM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ASSOCARRAYITEM_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCARRAYITEM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCARRAYITEM_class_version_get" "', argument " "1"" of type '" "struct _dwg_ASSOCARRAYITEM *""'"); 
    }
    arg1 = (struct _dwg_ASSOCARRAYITEM *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCARRAYITEM_itemloc_set) {
  {
    struct _dwg_ASSOCARRAYITEM *arg1 = (struct _dwg_ASSOCARRAYITEM *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) (BITCODE_BL *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ASSOCARRAYITEM_itemloc_set(self,itemloc);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCARRAYITEM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCARRAYITEM_itemloc_set" "', argument " "1"" of type '" "struct _dwg_ASSOCARRAYITEM *""'"); 
    }
    arg1 = (struct _dwg_ASSOCARRAYITEM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_ASSOCARRAYITEM_itemloc_set" "', argument " "2"" of type '" "BITCODE_BL [3]""'"); 
    } 
    arg2 = (BITCODE_BL *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)3; ++ii) *(BITCODE_BL *)&arg1->itemloc[ii] = *((BITCODE_BL *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""itemloc""' of type '""BITCODE_BL [3]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCARRAYITEM_itemloc_get) {
  {
    struct _dwg_ASSOCARRAYITEM *arg1 = (struct _dwg_ASSOCARRAYITEM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ASSOCARRAYITEM_itemloc_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCARRAYITEM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCARRAYITEM_itemloc_get" "', argument " "1"" of type '" "struct _dwg_ASSOCARRAYITEM *""'"); 
    }
    arg1 = (struct _dwg_ASSOCARRAYITEM *)(argp1);
    result = (BITCODE_BL *)(BITCODE_BL *) ((arg1)->itemloc);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCARRAYITEM_flags_set) {
  {
    struct _dwg_ASSOCARRAYITEM *arg1 = (struct _dwg_ASSOCARRAYITEM *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ASSOCARRAYITEM_flags_set(self,flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCARRAYITEM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCARRAYITEM_flags_set" "', argument " "1"" of type '" "struct _dwg_ASSOCARRAYITEM *""'"); 
    }
    arg1 = (struct _dwg_ASSOCARRAYITEM *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ASSOCARRAYITEM_flags_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCARRAYITEM_flags_get) {
  {
    struct _dwg_ASSOCARRAYITEM *arg1 = (struct _dwg_ASSOCARRAYITEM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ASSOCARRAYITEM_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCARRAYITEM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCARRAYITEM_flags_get" "', argument " "1"" of type '" "struct _dwg_ASSOCARRAYITEM *""'"); 
    }
    arg1 = (struct _dwg_ASSOCARRAYITEM *)(argp1);
    result = (BITCODE_BL) ((arg1)->flags);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCARRAYITEM_is_default_transmatrix_set) {
  {
    struct _dwg_ASSOCARRAYITEM *arg1 = (struct _dwg_ASSOCARRAYITEM *) 0 ;
    int arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ASSOCARRAYITEM_is_default_transmatrix_set(self,is_default_transmatrix);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCARRAYITEM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCARRAYITEM_is_default_transmatrix_set" "', argument " "1"" of type '" "struct _dwg_ASSOCARRAYITEM *""'"); 
    }
    arg1 = (struct _dwg_ASSOCARRAYITEM *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ASSOCARRAYITEM_is_default_transmatrix_set" "', argument " "2"" of type '" "int""'");
    } 
    arg2 = (int)(val2);
    if (arg1) (arg1)->is_default_transmatrix = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCARRAYITEM_is_default_transmatrix_get) {
  {
    struct _dwg_ASSOCARRAYITEM *arg1 = (struct _dwg_ASSOCARRAYITEM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ASSOCARRAYITEM_is_default_transmatrix_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCARRAYITEM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCARRAYITEM_is_default_transmatrix_get" "', argument " "1"" of type '" "struct _dwg_ASSOCARRAYITEM *""'"); 
    }
    arg1 = (struct _dwg_ASSOCARRAYITEM *)(argp1);
    result = (int) ((arg1)->is_default_transmatrix);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCARRAYITEM_x_dir_set) {
  {
    struct _dwg_ASSOCARRAYITEM *arg1 = (struct _dwg_ASSOCARRAYITEM *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ASSOCARRAYITEM_x_dir_set(self,x_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCARRAYITEM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCARRAYITEM_x_dir_set" "', argument " "1"" of type '" "struct _dwg_ASSOCARRAYITEM *""'"); 
    }
    arg1 = (struct _dwg_ASSOCARRAYITEM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_ASSOCARRAYITEM_x_dir_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->x_dir = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCARRAYITEM_x_dir_get) {
  {
    struct _dwg_ASSOCARRAYITEM *arg1 = (struct _dwg_ASSOCARRAYITEM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ASSOCARRAYITEM_x_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCARRAYITEM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCARRAYITEM_x_dir_get" "', argument " "1"" of type '" "struct _dwg_ASSOCARRAYITEM *""'"); 
    }
    arg1 = (struct _dwg_ASSOCARRAYITEM *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->x_dir);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCARRAYITEM_transmatrix_set) {
  {
    struct _dwg_ASSOCARRAYITEM *arg1 = (struct _dwg_ASSOCARRAYITEM *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ASSOCARRAYITEM_transmatrix_set(self,transmatrix);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCARRAYITEM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCARRAYITEM_transmatrix_set" "', argument " "1"" of type '" "struct _dwg_ASSOCARRAYITEM *""'"); 
    }
    arg1 = (struct _dwg_ASSOCARRAYITEM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_ASSOCARRAYITEM_transmatrix_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->transmatrix = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCARRAYITEM_transmatrix_get) {
  {
    struct _dwg_ASSOCARRAYITEM *arg1 = (struct _dwg_ASSOCARRAYITEM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ASSOCARRAYITEM_transmatrix_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCARRAYITEM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCARRAYITEM_transmatrix_get" "', argument " "1"" of type '" "struct _dwg_ASSOCARRAYITEM *""'"); 
    }
    arg1 = (struct _dwg_ASSOCARRAYITEM *)(argp1);
    result = (BITCODE_BD *) ((arg1)->transmatrix);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCARRAYITEM_rel_transform_set) {
  {
    struct _dwg_ASSOCARRAYITEM *arg1 = (struct _dwg_ASSOCARRAYITEM *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ASSOCARRAYITEM_rel_transform_set(self,rel_transform);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCARRAYITEM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCARRAYITEM_rel_transform_set" "', argument " "1"" of type '" "struct _dwg_ASSOCARRAYITEM *""'"); 
    }
    arg1 = (struct _dwg_ASSOCARRAYITEM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_ASSOCARRAYITEM_rel_transform_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->rel_transform = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCARRAYITEM_rel_transform_get) {
  {
    struct _dwg_ASSOCARRAYITEM *arg1 = (struct _dwg_ASSOCARRAYITEM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ASSOCARRAYITEM_rel_transform_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCARRAYITEM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCARRAYITEM_rel_transform_get" "', argument " "1"" of type '" "struct _dwg_ASSOCARRAYITEM *""'"); 
    }
    arg1 = (struct _dwg_ASSOCARRAYITEM *)(argp1);
    result = (BITCODE_BD *) ((arg1)->rel_transform);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCARRAYITEM_has_h1_set) {
  {
    struct _dwg_ASSOCARRAYITEM *arg1 = (struct _dwg_ASSOCARRAYITEM *) 0 ;
    int arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ASSOCARRAYITEM_has_h1_set(self,has_h1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCARRAYITEM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCARRAYITEM_has_h1_set" "', argument " "1"" of type '" "struct _dwg_ASSOCARRAYITEM *""'"); 
    }
    arg1 = (struct _dwg_ASSOCARRAYITEM *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ASSOCARRAYITEM_has_h1_set" "', argument " "2"" of type '" "int""'");
    } 
    arg2 = (int)(val2);
    if (arg1) (arg1)->has_h1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCARRAYITEM_has_h1_get) {
  {
    struct _dwg_ASSOCARRAYITEM *arg1 = (struct _dwg_ASSOCARRAYITEM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ASSOCARRAYITEM_has_h1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCARRAYITEM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCARRAYITEM_has_h1_get" "', argument " "1"" of type '" "struct _dwg_ASSOCARRAYITEM *""'"); 
    }
    arg1 = (struct _dwg_ASSOCARRAYITEM *)(argp1);
    result = (int) ((arg1)->has_h1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCARRAYITEM_h1_set) {
  {
    struct _dwg_ASSOCARRAYITEM *arg1 = (struct _dwg_ASSOCARRAYITEM *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ASSOCARRAYITEM_h1_set(self,h1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCARRAYITEM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCARRAYITEM_h1_set" "', argument " "1"" of type '" "struct _dwg_ASSOCARRAYITEM *""'"); 
    }
    arg1 = (struct _dwg_ASSOCARRAYITEM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_ASSOCARRAYITEM_h1_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->h1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCARRAYITEM_h1_get) {
  {
    struct _dwg_ASSOCARRAYITEM *arg1 = (struct _dwg_ASSOCARRAYITEM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ASSOCARRAYITEM_h1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCARRAYITEM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCARRAYITEM_h1_get" "', argument " "1"" of type '" "struct _dwg_ASSOCARRAYITEM *""'"); 
    }
    arg1 = (struct _dwg_ASSOCARRAYITEM *)(argp1);
    result = (BITCODE_H) ((arg1)->h1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCARRAYITEM_h2_set) {
  {
    struct _dwg_ASSOCARRAYITEM *arg1 = (struct _dwg_ASSOCARRAYITEM *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ASSOCARRAYITEM_h2_set(self,h2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCARRAYITEM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCARRAYITEM_h2_set" "', argument " "1"" of type '" "struct _dwg_ASSOCARRAYITEM *""'"); 
    }
    arg1 = (struct _dwg_ASSOCARRAYITEM *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_ASSOCARRAYITEM_h2_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->h2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCARRAYITEM_h2_get) {
  {
    struct _dwg_ASSOCARRAYITEM *arg1 = (struct _dwg_ASSOCARRAYITEM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ASSOCARRAYITEM_h2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCARRAYITEM, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCARRAYITEM_h2_get" "', argument " "1"" of type '" "struct _dwg_ASSOCARRAYITEM *""'"); 
    }
    arg1 = (struct _dwg_ASSOCARRAYITEM *)(argp1);
    result = (BITCODE_H) ((arg1)->h2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_ASSOCARRAYITEM) {
  {
    int argvi = 0;
    struct _dwg_ASSOCARRAYITEM *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_ASSOCARRAYITEM();");
    }
    result = (struct _dwg_ASSOCARRAYITEM *)calloc(1, sizeof(struct _dwg_ASSOCARRAYITEM));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCARRAYITEM, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_ASSOCARRAYITEM) {
  {
    struct _dwg_ASSOCARRAYITEM *arg1 = (struct _dwg_ASSOCARRAYITEM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_ASSOCARRAYITEM(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCARRAYITEM, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_ASSOCARRAYITEM" "', argument " "1"" of type '" "struct _dwg_ASSOCARRAYITEM *""'"); 
    }
    arg1 = (struct _dwg_ASSOCARRAYITEM *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYPARAMETERS_parent_set) {
  {
    struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *arg1 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYPARAMETERS_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYPARAMETERS_parent_set" "', argument " "1"" of type '" "struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *""'"); 
    }
    arg1 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCARRAYPARAMETERS_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYPARAMETERS_parent_get) {
  {
    struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *arg1 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYPARAMETERS_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYPARAMETERS_parent_get" "', argument " "1"" of type '" "struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *""'"); 
    }
    arg1 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYPARAMETERS_aap_version_set) {
  {
    struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *arg1 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYPARAMETERS_aap_version_set(self,aap_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYPARAMETERS_aap_version_set" "', argument " "1"" of type '" "struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *""'"); 
    }
    arg1 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCARRAYPARAMETERS_aap_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->aap_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYPARAMETERS_aap_version_get) {
  {
    struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *arg1 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYPARAMETERS_aap_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYPARAMETERS_aap_version_get" "', argument " "1"" of type '" "struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *""'"); 
    }
    arg1 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *)(argp1);
    result = (BITCODE_BL) ((arg1)->aap_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYPARAMETERS_num_items_set) {
  {
    struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *arg1 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYPARAMETERS_num_items_set(self,num_items);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYPARAMETERS_num_items_set" "', argument " "1"" of type '" "struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *""'"); 
    }
    arg1 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCARRAYPARAMETERS_num_items_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_items = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYPARAMETERS_num_items_get) {
  {
    struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *arg1 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYPARAMETERS_num_items_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYPARAMETERS_num_items_get" "', argument " "1"" of type '" "struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *""'"); 
    }
    arg1 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_items);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYPARAMETERS_classname_set) {
  {
    struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *arg1 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYPARAMETERS_classname_set(self,classname);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYPARAMETERS_classname_set" "', argument " "1"" of type '" "struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *""'"); 
    }
    arg1 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCARRAYPARAMETERS_classname_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->classname);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->classname = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->classname = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYPARAMETERS_classname_get) {
  {
    struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *arg1 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYPARAMETERS_classname_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYPARAMETERS_classname_get" "', argument " "1"" of type '" "struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *""'"); 
    }
    arg1 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *)(argp1);
    result = (BITCODE_TV) ((arg1)->classname);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYPARAMETERS_items_set) {
  {
    struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *arg1 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *) 0 ;
    Dwg_ASSOCARRAYITEM *arg2 = (Dwg_ASSOCARRAYITEM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYPARAMETERS_items_set(self,items);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYPARAMETERS_items_set" "', argument " "1"" of type '" "struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *""'"); 
    }
    arg1 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCARRAYITEM, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCARRAYPARAMETERS_items_set" "', argument " "2"" of type '" "Dwg_ASSOCARRAYITEM *""'"); 
    }
    arg2 = (Dwg_ASSOCARRAYITEM *)(argp2);
    if (arg1) (arg1)->items = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYPARAMETERS_items_get) {
  {
    struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *arg1 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCARRAYITEM *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYPARAMETERS_items_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYPARAMETERS_items_get" "', argument " "1"" of type '" "struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *""'"); 
    }
    arg1 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *)(argp1);
    result = (Dwg_ASSOCARRAYITEM *) ((arg1)->items);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCARRAYITEM, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYPARAMETERS_numitems_set) {
  {
    struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *arg1 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYPARAMETERS_numitems_set(self,numitems);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYPARAMETERS_numitems_set" "', argument " "1"" of type '" "struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *""'"); 
    }
    arg1 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCARRAYPARAMETERS_numitems_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->numitems = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYPARAMETERS_numitems_get) {
  {
    struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *arg1 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYPARAMETERS_numitems_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYPARAMETERS_numitems_get" "', argument " "1"" of type '" "struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *""'"); 
    }
    arg1 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *)(argp1);
    result = (BITCODE_BL) ((arg1)->numitems);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYPARAMETERS_numrows_set) {
  {
    struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *arg1 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYPARAMETERS_numrows_set(self,numrows);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYPARAMETERS_numrows_set" "', argument " "1"" of type '" "struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *""'"); 
    }
    arg1 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCARRAYPARAMETERS_numrows_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->numrows = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYPARAMETERS_numrows_get) {
  {
    struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *arg1 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYPARAMETERS_numrows_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYPARAMETERS_numrows_get" "', argument " "1"" of type '" "struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *""'"); 
    }
    arg1 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *)(argp1);
    result = (BITCODE_BL) ((arg1)->numrows);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYPARAMETERS_numlevels_set) {
  {
    struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *arg1 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYPARAMETERS_numlevels_set(self,numlevels);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYPARAMETERS_numlevels_set" "', argument " "1"" of type '" "struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *""'"); 
    }
    arg1 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCARRAYPARAMETERS_numlevels_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->numlevels = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYPARAMETERS_numlevels_get) {
  {
    struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *arg1 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYPARAMETERS_numlevels_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYPARAMETERS_numlevels_get" "', argument " "1"" of type '" "struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *""'"); 
    }
    arg1 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *)(argp1);
    result = (BITCODE_BL) ((arg1)->numlevels);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCARRAYPARAMETERS) {
  {
    int argvi = 0;
    struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCARRAYPARAMETERS();");
    }
    result = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *)calloc(1, sizeof(struct _dwg_abstractobject_ASSOCARRAYPARAMETERS));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCARRAYPARAMETERS) {
  {
    struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *arg1 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCARRAYPARAMETERS(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCARRAYPARAMETERS" "', argument " "1"" of type '" "struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *""'"); 
    }
    arg1 = (struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_parent_set) {
  {
    struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *arg1 = (struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_parent_get) {
  {
    struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *arg1 = (struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_aab_version_set) {
  {
    struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *arg1 = (struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_aab_version_set(self,aab_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_aab_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_aab_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->aab_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_aab_version_get) {
  {
    struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *arg1 = (struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_aab_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_aab_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->aab_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_class_version_set) {
  {
    struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *arg1 = (struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_class_version_get) {
  {
    struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *arg1 = (struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_entity_set) {
  {
    struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *arg1 = (struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_entity_set(self,entity);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_entity_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_entity_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->entity = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_entity_get) {
  {
    struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *arg1 = (struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_entity_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_entity_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *)(argp1);
    result = (BITCODE_H) ((arg1)->entity);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY();");
    }
    result = (struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *)calloc(1, sizeof(struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY) {
  {
    struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *arg1 = (struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY" "', argument " "1"" of type '" "struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCSURFACEACTIONBODY_parent_set) {
  {
    struct _dwg_ASSOCSURFACEACTIONBODY *arg1 = (struct _dwg_ASSOCSURFACEACTIONBODY *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ASSOCSURFACEACTIONBODY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCSURFACEACTIONBODY_parent_set" "', argument " "1"" of type '" "struct _dwg_ASSOCSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_ASSOCSURFACEACTIONBODY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCSURFACEACTIONBODY_parent_get) {
  {
    struct _dwg_ASSOCSURFACEACTIONBODY *arg1 = (struct _dwg_ASSOCSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ASSOCSURFACEACTIONBODY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCSURFACEACTIONBODY_parent_get" "', argument " "1"" of type '" "struct _dwg_ASSOCSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCSURFACEACTIONBODY *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCSURFACEACTIONBODY_version_set) {
  {
    struct _dwg_ASSOCSURFACEACTIONBODY *arg1 = (struct _dwg_ASSOCSURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ASSOCSURFACEACTIONBODY_version_set(self,version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCSURFACEACTIONBODY_version_set" "', argument " "1"" of type '" "struct _dwg_ASSOCSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ASSOCSURFACEACTIONBODY_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCSURFACEACTIONBODY_version_get) {
  {
    struct _dwg_ASSOCSURFACEACTIONBODY *arg1 = (struct _dwg_ASSOCSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ASSOCSURFACEACTIONBODY_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCSURFACEACTIONBODY_version_get" "', argument " "1"" of type '" "struct _dwg_ASSOCSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCSURFACEACTIONBODY_is_semi_assoc_set) {
  {
    struct _dwg_ASSOCSURFACEACTIONBODY *arg1 = (struct _dwg_ASSOCSURFACEACTIONBODY *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ASSOCSURFACEACTIONBODY_is_semi_assoc_set(self,is_semi_assoc);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCSURFACEACTIONBODY_is_semi_assoc_set" "', argument " "1"" of type '" "struct _dwg_ASSOCSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ASSOCSURFACEACTIONBODY_is_semi_assoc_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_semi_assoc = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCSURFACEACTIONBODY_is_semi_assoc_get) {
  {
    struct _dwg_ASSOCSURFACEACTIONBODY *arg1 = (struct _dwg_ASSOCSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ASSOCSURFACEACTIONBODY_is_semi_assoc_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCSURFACEACTIONBODY_is_semi_assoc_get" "', argument " "1"" of type '" "struct _dwg_ASSOCSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_B) ((arg1)->is_semi_assoc);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCSURFACEACTIONBODY_l2_set) {
  {
    struct _dwg_ASSOCSURFACEACTIONBODY *arg1 = (struct _dwg_ASSOCSURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ASSOCSURFACEACTIONBODY_l2_set(self,l2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCSURFACEACTIONBODY_l2_set" "', argument " "1"" of type '" "struct _dwg_ASSOCSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ASSOCSURFACEACTIONBODY_l2_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->l2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCSURFACEACTIONBODY_l2_get) {
  {
    struct _dwg_ASSOCSURFACEACTIONBODY *arg1 = (struct _dwg_ASSOCSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ASSOCSURFACEACTIONBODY_l2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCSURFACEACTIONBODY_l2_get" "', argument " "1"" of type '" "struct _dwg_ASSOCSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->l2);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCSURFACEACTIONBODY_is_semi_ovr_set) {
  {
    struct _dwg_ASSOCSURFACEACTIONBODY *arg1 = (struct _dwg_ASSOCSURFACEACTIONBODY *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ASSOCSURFACEACTIONBODY_is_semi_ovr_set(self,is_semi_ovr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCSURFACEACTIONBODY_is_semi_ovr_set" "', argument " "1"" of type '" "struct _dwg_ASSOCSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ASSOCSURFACEACTIONBODY_is_semi_ovr_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_semi_ovr = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCSURFACEACTIONBODY_is_semi_ovr_get) {
  {
    struct _dwg_ASSOCSURFACEACTIONBODY *arg1 = (struct _dwg_ASSOCSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ASSOCSURFACEACTIONBODY_is_semi_ovr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCSURFACEACTIONBODY_is_semi_ovr_get" "', argument " "1"" of type '" "struct _dwg_ASSOCSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_B) ((arg1)->is_semi_ovr);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCSURFACEACTIONBODY_grip_status_set) {
  {
    struct _dwg_ASSOCSURFACEACTIONBODY *arg1 = (struct _dwg_ASSOCSURFACEACTIONBODY *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ASSOCSURFACEACTIONBODY_grip_status_set(self,grip_status);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCSURFACEACTIONBODY_grip_status_set" "', argument " "1"" of type '" "struct _dwg_ASSOCSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ASSOCSURFACEACTIONBODY_grip_status_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->grip_status = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCSURFACEACTIONBODY_grip_status_get) {
  {
    struct _dwg_ASSOCSURFACEACTIONBODY *arg1 = (struct _dwg_ASSOCSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ASSOCSURFACEACTIONBODY_grip_status_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCSURFACEACTIONBODY_grip_status_get" "', argument " "1"" of type '" "struct _dwg_ASSOCSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BS) ((arg1)->grip_status);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCSURFACEACTIONBODY_assocdep_set) {
  {
    struct _dwg_ASSOCSURFACEACTIONBODY *arg1 = (struct _dwg_ASSOCSURFACEACTIONBODY *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ASSOCSURFACEACTIONBODY_assocdep_set(self,assocdep);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCSURFACEACTIONBODY_assocdep_set" "', argument " "1"" of type '" "struct _dwg_ASSOCSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_ASSOCSURFACEACTIONBODY_assocdep_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->assocdep = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCSURFACEACTIONBODY_assocdep_get) {
  {
    struct _dwg_ASSOCSURFACEACTIONBODY *arg1 = (struct _dwg_ASSOCSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ASSOCSURFACEACTIONBODY_assocdep_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCSURFACEACTIONBODY_assocdep_get" "', argument " "1"" of type '" "struct _dwg_ASSOCSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_H) ((arg1)->assocdep);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_ASSOCSURFACEACTIONBODY) {
  {
    int argvi = 0;
    struct _dwg_ASSOCSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_ASSOCSURFACEACTIONBODY();");
    }
    result = (struct _dwg_ASSOCSURFACEACTIONBODY *)calloc(1, sizeof(struct _dwg_ASSOCSURFACEACTIONBODY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_ASSOCSURFACEACTIONBODY) {
  {
    struct _dwg_ASSOCSURFACEACTIONBODY *arg1 = (struct _dwg_ASSOCSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_ASSOCSURFACEACTIONBODY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_ASSOCSURFACEACTIONBODY" "', argument " "1"" of type '" "struct _dwg_ASSOCSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_ASSOCSURFACEACTIONBODY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_parent_set) {
  {
    struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEXTENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_parent_get) {
  {
    struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEXTENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_aab_version_set) {
  {
    struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_aab_version_set(self,aab_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEXTENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_aab_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_aab_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->aab_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_aab_version_get) {
  {
    struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_aab_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEXTENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_aab_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->aab_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_pab_set) {
  {
    struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *) 0 ;
    Dwg_ASSOCPARAMBASEDACTIONBODY *arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_pab_set(self,pab);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEXTENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_pab_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_pab_set" "', argument " "2"" of type '" "Dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *)(argp2);
    if (arg1) (arg1)->pab = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_pab_get) {
  {
    struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCPARAMBASEDACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_pab_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEXTENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_pab_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *)(argp1);
    result = (Dwg_ASSOCPARAMBASEDACTIONBODY *)& ((arg1)->pab);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_sab_set) {
  {
    struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *) 0 ;
    Dwg_ASSOCSURFACEACTIONBODY *arg2 = (Dwg_ASSOCSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_sab_set(self,sab);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEXTENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_sab_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_sab_set" "', argument " "2"" of type '" "Dwg_ASSOCSURFACEACTIONBODY *""'"); 
    }
    arg2 = (Dwg_ASSOCSURFACEACTIONBODY *)(argp2);
    if (arg1) (arg1)->sab = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_sab_get) {
  {
    struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_sab_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEXTENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_sab_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *)(argp1);
    result = (Dwg_ASSOCSURFACEACTIONBODY *)& ((arg1)->sab);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_pbsab_status_set) {
  {
    struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_pbsab_status_set(self,pbsab_status);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEXTENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_pbsab_status_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_pbsab_status_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->pbsab_status = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_pbsab_status_get) {
  {
    struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_pbsab_status_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEXTENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_pbsab_status_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->pbsab_status);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_class_version_set) {
  {
    struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEXTENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_class_version_get) {
  {
    struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEXTENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_option_set) {
  {
    struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_option_set(self,option);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEXTENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_option_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_option_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->option = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_option_get) {
  {
    struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_option_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEXTENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_option_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_RC) ((arg1)->option);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY();");
    }
    result = (struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *)calloc(1, sizeof(struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCEXTENDSURFACEACTIONBODY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY) {
  {
    struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEXTENDSURFACEACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_parent_set) {
  {
    struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_parent_get) {
  {
    struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_aab_version_set) {
  {
    struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_aab_version_set(self,aab_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_aab_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_aab_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->aab_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_aab_version_get) {
  {
    struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_aab_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_aab_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->aab_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_pab_set) {
  {
    struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *) 0 ;
    Dwg_ASSOCPARAMBASEDACTIONBODY *arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_pab_set(self,pab);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_pab_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_pab_set" "', argument " "2"" of type '" "Dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *)(argp2);
    if (arg1) (arg1)->pab = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_pab_get) {
  {
    struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCPARAMBASEDACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_pab_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_pab_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *)(argp1);
    result = (Dwg_ASSOCPARAMBASEDACTIONBODY *)& ((arg1)->pab);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_sab_set) {
  {
    struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *) 0 ;
    Dwg_ASSOCSURFACEACTIONBODY *arg2 = (Dwg_ASSOCSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_sab_set(self,sab);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_sab_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_sab_set" "', argument " "2"" of type '" "Dwg_ASSOCSURFACEACTIONBODY *""'"); 
    }
    arg2 = (Dwg_ASSOCSURFACEACTIONBODY *)(argp2);
    if (arg1) (arg1)->sab = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_sab_get) {
  {
    struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_sab_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_sab_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *)(argp1);
    result = (Dwg_ASSOCSURFACEACTIONBODY *)& ((arg1)->sab);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_pbsab_status_set) {
  {
    struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_pbsab_status_set(self,pbsab_status);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_pbsab_status_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_pbsab_status_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->pbsab_status = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_pbsab_status_get) {
  {
    struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_pbsab_status_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_pbsab_status_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->pbsab_status);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_class_version_set) {
  {
    struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_class_version_get) {
  {
    struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY();");
    }
    result = (struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *)calloc(1, sizeof(struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY) {
  {
    struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPLANESURFACEACTIONBODY_parent_set) {
  {
    struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPLANESURFACEACTIONBODY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPLANESURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPLANESURFACEACTIONBODY_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCPLANESURFACEACTIONBODY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPLANESURFACEACTIONBODY_parent_get) {
  {
    struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPLANESURFACEACTIONBODY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPLANESURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPLANESURFACEACTIONBODY_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPLANESURFACEACTIONBODY_aab_version_set) {
  {
    struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPLANESURFACEACTIONBODY_aab_version_set(self,aab_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPLANESURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPLANESURFACEACTIONBODY_aab_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPLANESURFACEACTIONBODY_aab_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->aab_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPLANESURFACEACTIONBODY_aab_version_get) {
  {
    struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPLANESURFACEACTIONBODY_aab_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPLANESURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPLANESURFACEACTIONBODY_aab_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->aab_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPLANESURFACEACTIONBODY_pab_set) {
  {
    struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *) 0 ;
    Dwg_ASSOCPARAMBASEDACTIONBODY *arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPLANESURFACEACTIONBODY_pab_set(self,pab);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPLANESURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPLANESURFACEACTIONBODY_pab_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCPLANESURFACEACTIONBODY_pab_set" "', argument " "2"" of type '" "Dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *)(argp2);
    if (arg1) (arg1)->pab = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPLANESURFACEACTIONBODY_pab_get) {
  {
    struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCPARAMBASEDACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPLANESURFACEACTIONBODY_pab_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPLANESURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPLANESURFACEACTIONBODY_pab_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *)(argp1);
    result = (Dwg_ASSOCPARAMBASEDACTIONBODY *)& ((arg1)->pab);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPLANESURFACEACTIONBODY_sab_set) {
  {
    struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *) 0 ;
    Dwg_ASSOCSURFACEACTIONBODY *arg2 = (Dwg_ASSOCSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPLANESURFACEACTIONBODY_sab_set(self,sab);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPLANESURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPLANESURFACEACTIONBODY_sab_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCPLANESURFACEACTIONBODY_sab_set" "', argument " "2"" of type '" "Dwg_ASSOCSURFACEACTIONBODY *""'"); 
    }
    arg2 = (Dwg_ASSOCSURFACEACTIONBODY *)(argp2);
    if (arg1) (arg1)->sab = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPLANESURFACEACTIONBODY_sab_get) {
  {
    struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPLANESURFACEACTIONBODY_sab_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPLANESURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPLANESURFACEACTIONBODY_sab_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *)(argp1);
    result = (Dwg_ASSOCSURFACEACTIONBODY *)& ((arg1)->sab);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPLANESURFACEACTIONBODY_pbsab_status_set) {
  {
    struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPLANESURFACEACTIONBODY_pbsab_status_set(self,pbsab_status);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPLANESURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPLANESURFACEACTIONBODY_pbsab_status_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPLANESURFACEACTIONBODY_pbsab_status_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->pbsab_status = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPLANESURFACEACTIONBODY_pbsab_status_get) {
  {
    struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPLANESURFACEACTIONBODY_pbsab_status_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPLANESURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPLANESURFACEACTIONBODY_pbsab_status_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->pbsab_status);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPLANESURFACEACTIONBODY_class_version_set) {
  {
    struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPLANESURFACEACTIONBODY_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPLANESURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPLANESURFACEACTIONBODY_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPLANESURFACEACTIONBODY_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPLANESURFACEACTIONBODY_class_version_get) {
  {
    struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPLANESURFACEACTIONBODY_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPLANESURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPLANESURFACEACTIONBODY_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCPLANESURFACEACTIONBODY) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCPLANESURFACEACTIONBODY();");
    }
    result = (struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *)calloc(1, sizeof(struct _dwg_object_ASSOCPLANESURFACEACTIONBODY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCPLANESURFACEACTIONBODY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCPLANESURFACEACTIONBODY) {
  {
    struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCPLANESURFACEACTIONBODY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPLANESURFACEACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCPLANESURFACEACTIONBODY" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_parent_set) {
  {
    struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCLOFTEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_parent_get) {
  {
    struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCLOFTEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_aab_version_set) {
  {
    struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_aab_version_set(self,aab_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCLOFTEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_aab_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_aab_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->aab_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_aab_version_get) {
  {
    struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_aab_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCLOFTEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_aab_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->aab_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_pab_set) {
  {
    struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *) 0 ;
    Dwg_ASSOCPARAMBASEDACTIONBODY *arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_pab_set(self,pab);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCLOFTEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_pab_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_pab_set" "', argument " "2"" of type '" "Dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *)(argp2);
    if (arg1) (arg1)->pab = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_pab_get) {
  {
    struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCPARAMBASEDACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_pab_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCLOFTEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_pab_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *)(argp1);
    result = (Dwg_ASSOCPARAMBASEDACTIONBODY *)& ((arg1)->pab);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_sab_set) {
  {
    struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *) 0 ;
    Dwg_ASSOCSURFACEACTIONBODY *arg2 = (Dwg_ASSOCSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_sab_set(self,sab);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCLOFTEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_sab_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_sab_set" "', argument " "2"" of type '" "Dwg_ASSOCSURFACEACTIONBODY *""'"); 
    }
    arg2 = (Dwg_ASSOCSURFACEACTIONBODY *)(argp2);
    if (arg1) (arg1)->sab = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_sab_get) {
  {
    struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_sab_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCLOFTEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_sab_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *)(argp1);
    result = (Dwg_ASSOCSURFACEACTIONBODY *)& ((arg1)->sab);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_pbsab_status_set) {
  {
    struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_pbsab_status_set(self,pbsab_status);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCLOFTEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_pbsab_status_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_pbsab_status_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->pbsab_status = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_pbsab_status_get) {
  {
    struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_pbsab_status_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCLOFTEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_pbsab_status_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->pbsab_status);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_class_version_set) {
  {
    struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCLOFTEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_class_version_get) {
  {
    struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCLOFTEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY();");
    }
    result = (struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *)calloc(1, sizeof(struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCLOFTEDSURFACEACTIONBODY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY) {
  {
    struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCLOFTEDSURFACEACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY" "', argument " "1"" of type '" "struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_parent_set) {
  {
    struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORKSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_parent_get) {
  {
    struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORKSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_aab_version_set) {
  {
    struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_aab_version_set(self,aab_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORKSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_aab_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_aab_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->aab_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_aab_version_get) {
  {
    struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_aab_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORKSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_aab_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->aab_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_pab_set) {
  {
    struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *) 0 ;
    Dwg_ASSOCPARAMBASEDACTIONBODY *arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_pab_set(self,pab);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORKSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_pab_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_pab_set" "', argument " "2"" of type '" "Dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *)(argp2);
    if (arg1) (arg1)->pab = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_pab_get) {
  {
    struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCPARAMBASEDACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_pab_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORKSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_pab_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *)(argp1);
    result = (Dwg_ASSOCPARAMBASEDACTIONBODY *)& ((arg1)->pab);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_sab_set) {
  {
    struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *) 0 ;
    Dwg_ASSOCSURFACEACTIONBODY *arg2 = (Dwg_ASSOCSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_sab_set(self,sab);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORKSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_sab_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_sab_set" "', argument " "2"" of type '" "Dwg_ASSOCSURFACEACTIONBODY *""'"); 
    }
    arg2 = (Dwg_ASSOCSURFACEACTIONBODY *)(argp2);
    if (arg1) (arg1)->sab = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_sab_get) {
  {
    struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_sab_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORKSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_sab_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *)(argp1);
    result = (Dwg_ASSOCSURFACEACTIONBODY *)& ((arg1)->sab);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_pbsab_status_set) {
  {
    struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_pbsab_status_set(self,pbsab_status);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORKSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_pbsab_status_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_pbsab_status_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->pbsab_status = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_pbsab_status_get) {
  {
    struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_pbsab_status_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORKSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_pbsab_status_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->pbsab_status);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_class_version_set) {
  {
    struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORKSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_class_version_get) {
  {
    struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORKSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY();");
    }
    result = (struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *)calloc(1, sizeof(struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCNETWORKSURFACEACTIONBODY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY) {
  {
    struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCNETWORKSURFACEACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY" "', argument " "1"" of type '" "struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_parent_set) {
  {
    struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOFFSETSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_parent_get) {
  {
    struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOFFSETSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_aab_version_set) {
  {
    struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_aab_version_set(self,aab_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOFFSETSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_aab_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_aab_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->aab_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_aab_version_get) {
  {
    struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_aab_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOFFSETSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_aab_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->aab_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_pab_set) {
  {
    struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *) 0 ;
    Dwg_ASSOCPARAMBASEDACTIONBODY *arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_pab_set(self,pab);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOFFSETSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_pab_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_pab_set" "', argument " "2"" of type '" "Dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *)(argp2);
    if (arg1) (arg1)->pab = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_pab_get) {
  {
    struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCPARAMBASEDACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_pab_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOFFSETSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_pab_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *)(argp1);
    result = (Dwg_ASSOCPARAMBASEDACTIONBODY *)& ((arg1)->pab);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_sab_set) {
  {
    struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *) 0 ;
    Dwg_ASSOCSURFACEACTIONBODY *arg2 = (Dwg_ASSOCSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_sab_set(self,sab);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOFFSETSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_sab_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_sab_set" "', argument " "2"" of type '" "Dwg_ASSOCSURFACEACTIONBODY *""'"); 
    }
    arg2 = (Dwg_ASSOCSURFACEACTIONBODY *)(argp2);
    if (arg1) (arg1)->sab = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_sab_get) {
  {
    struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_sab_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOFFSETSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_sab_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *)(argp1);
    result = (Dwg_ASSOCSURFACEACTIONBODY *)& ((arg1)->sab);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_pbsab_status_set) {
  {
    struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_pbsab_status_set(self,pbsab_status);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOFFSETSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_pbsab_status_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_pbsab_status_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->pbsab_status = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_pbsab_status_get) {
  {
    struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_pbsab_status_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOFFSETSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_pbsab_status_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->pbsab_status);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_class_version_set) {
  {
    struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOFFSETSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_class_version_get) {
  {
    struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOFFSETSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_b1_set) {
  {
    struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_b1_set(self,b1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOFFSETSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_b1_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_b1_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->b1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_b1_get) {
  {
    struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_b1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOFFSETSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_b1_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_B) ((arg1)->b1);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY();");
    }
    result = (struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *)calloc(1, sizeof(struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCOFFSETSURFACEACTIONBODY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY) {
  {
    struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCOFFSETSURFACEACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY" "', argument " "1"" of type '" "struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_parent_set) {
  {
    struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_parent_get) {
  {
    struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_aab_version_set) {
  {
    struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_aab_version_set(self,aab_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_aab_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_aab_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->aab_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_aab_version_get) {
  {
    struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_aab_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_aab_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->aab_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_pab_set) {
  {
    struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *) 0 ;
    Dwg_ASSOCPARAMBASEDACTIONBODY *arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_pab_set(self,pab);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_pab_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_pab_set" "', argument " "2"" of type '" "Dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *)(argp2);
    if (arg1) (arg1)->pab = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_pab_get) {
  {
    struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCPARAMBASEDACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_pab_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_pab_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *)(argp1);
    result = (Dwg_ASSOCPARAMBASEDACTIONBODY *)& ((arg1)->pab);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_sab_set) {
  {
    struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *) 0 ;
    Dwg_ASSOCSURFACEACTIONBODY *arg2 = (Dwg_ASSOCSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_sab_set(self,sab);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_sab_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_sab_set" "', argument " "2"" of type '" "Dwg_ASSOCSURFACEACTIONBODY *""'"); 
    }
    arg2 = (Dwg_ASSOCSURFACEACTIONBODY *)(argp2);
    if (arg1) (arg1)->sab = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_sab_get) {
  {
    struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_sab_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_sab_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *)(argp1);
    result = (Dwg_ASSOCSURFACEACTIONBODY *)& ((arg1)->sab);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_pbsab_status_set) {
  {
    struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_pbsab_status_set(self,pbsab_status);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_pbsab_status_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_pbsab_status_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->pbsab_status = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_pbsab_status_get) {
  {
    struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_pbsab_status_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_pbsab_status_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->pbsab_status);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_class_version_set) {
  {
    struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_class_version_get) {
  {
    struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY();");
    }
    result = (struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *)calloc(1, sizeof(struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY) {
  {
    struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY" "', argument " "1"" of type '" "struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_parent_set) {
  {
    struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCSWEPTSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_parent_get) {
  {
    struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCSWEPTSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_aab_version_set) {
  {
    struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_aab_version_set(self,aab_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCSWEPTSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_aab_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_aab_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->aab_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_aab_version_get) {
  {
    struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_aab_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCSWEPTSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_aab_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->aab_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_pab_set) {
  {
    struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *) 0 ;
    Dwg_ASSOCPARAMBASEDACTIONBODY *arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_pab_set(self,pab);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCSWEPTSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_pab_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_pab_set" "', argument " "2"" of type '" "Dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *)(argp2);
    if (arg1) (arg1)->pab = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_pab_get) {
  {
    struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCPARAMBASEDACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_pab_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCSWEPTSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_pab_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *)(argp1);
    result = (Dwg_ASSOCPARAMBASEDACTIONBODY *)& ((arg1)->pab);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_sab_set) {
  {
    struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *) 0 ;
    Dwg_ASSOCSURFACEACTIONBODY *arg2 = (Dwg_ASSOCSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_sab_set(self,sab);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCSWEPTSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_sab_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_sab_set" "', argument " "2"" of type '" "Dwg_ASSOCSURFACEACTIONBODY *""'"); 
    }
    arg2 = (Dwg_ASSOCSURFACEACTIONBODY *)(argp2);
    if (arg1) (arg1)->sab = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_sab_get) {
  {
    struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_sab_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCSWEPTSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_sab_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *)(argp1);
    result = (Dwg_ASSOCSURFACEACTIONBODY *)& ((arg1)->sab);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_pbsab_status_set) {
  {
    struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_pbsab_status_set(self,pbsab_status);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCSWEPTSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_pbsab_status_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_pbsab_status_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->pbsab_status = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_pbsab_status_get) {
  {
    struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_pbsab_status_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCSWEPTSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_pbsab_status_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->pbsab_status);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_class_version_set) {
  {
    struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCSWEPTSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_class_version_get) {
  {
    struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCSWEPTSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY();");
    }
    result = (struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *)calloc(1, sizeof(struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCSWEPTSURFACEACTIONBODY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY) {
  {
    struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCSWEPTSURFACEACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY" "', argument " "1"" of type '" "struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_parent_set) {
  {
    struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *arg1 = (struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGECHAMFERACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_parent_get) {
  {
    struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *arg1 = (struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGECHAMFERACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_aab_version_set) {
  {
    struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *arg1 = (struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_aab_version_set(self,aab_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGECHAMFERACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_aab_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_aab_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->aab_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_aab_version_get) {
  {
    struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *arg1 = (struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_aab_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGECHAMFERACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_aab_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->aab_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_pab_set) {
  {
    struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *arg1 = (struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *) 0 ;
    Dwg_ASSOCPARAMBASEDACTIONBODY *arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_pab_set(self,pab);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGECHAMFERACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_pab_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_pab_set" "', argument " "2"" of type '" "Dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *)(argp2);
    if (arg1) (arg1)->pab = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_pab_get) {
  {
    struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *arg1 = (struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCPARAMBASEDACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_pab_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGECHAMFERACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_pab_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *)(argp1);
    result = (Dwg_ASSOCPARAMBASEDACTIONBODY *)& ((arg1)->pab);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_sab_set) {
  {
    struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *arg1 = (struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *) 0 ;
    Dwg_ASSOCSURFACEACTIONBODY *arg2 = (Dwg_ASSOCSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_sab_set(self,sab);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGECHAMFERACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_sab_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_sab_set" "', argument " "2"" of type '" "Dwg_ASSOCSURFACEACTIONBODY *""'"); 
    }
    arg2 = (Dwg_ASSOCSURFACEACTIONBODY *)(argp2);
    if (arg1) (arg1)->sab = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_sab_get) {
  {
    struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *arg1 = (struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_sab_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGECHAMFERACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_sab_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *)(argp1);
    result = (Dwg_ASSOCSURFACEACTIONBODY *)& ((arg1)->sab);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_pbsab_status_set) {
  {
    struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *arg1 = (struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_pbsab_status_set(self,pbsab_status);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGECHAMFERACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_pbsab_status_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_pbsab_status_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->pbsab_status = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_pbsab_status_get) {
  {
    struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *arg1 = (struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_pbsab_status_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGECHAMFERACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_pbsab_status_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->pbsab_status);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCEDGECHAMFERACTIONBODY) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCEDGECHAMFERACTIONBODY();");
    }
    result = (struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *)calloc(1, sizeof(struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCEDGECHAMFERACTIONBODY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCEDGECHAMFERACTIONBODY) {
  {
    struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *arg1 = (struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCEDGECHAMFERACTIONBODY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGECHAMFERACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCEDGECHAMFERACTIONBODY" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGEFILLETACTIONBODY_parent_set) {
  {
    struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *arg1 = (struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGEFILLETACTIONBODY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGEFILLETACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGEFILLETACTIONBODY_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCEDGEFILLETACTIONBODY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGEFILLETACTIONBODY_parent_get) {
  {
    struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *arg1 = (struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGEFILLETACTIONBODY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGEFILLETACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGEFILLETACTIONBODY_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGEFILLETACTIONBODY_aab_version_set) {
  {
    struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *arg1 = (struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGEFILLETACTIONBODY_aab_version_set(self,aab_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGEFILLETACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGEFILLETACTIONBODY_aab_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCEDGEFILLETACTIONBODY_aab_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->aab_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGEFILLETACTIONBODY_aab_version_get) {
  {
    struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *arg1 = (struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGEFILLETACTIONBODY_aab_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGEFILLETACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGEFILLETACTIONBODY_aab_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->aab_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGEFILLETACTIONBODY_pab_set) {
  {
    struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *arg1 = (struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *) 0 ;
    Dwg_ASSOCPARAMBASEDACTIONBODY *arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGEFILLETACTIONBODY_pab_set(self,pab);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGEFILLETACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGEFILLETACTIONBODY_pab_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCEDGEFILLETACTIONBODY_pab_set" "', argument " "2"" of type '" "Dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *)(argp2);
    if (arg1) (arg1)->pab = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGEFILLETACTIONBODY_pab_get) {
  {
    struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *arg1 = (struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCPARAMBASEDACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGEFILLETACTIONBODY_pab_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGEFILLETACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGEFILLETACTIONBODY_pab_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *)(argp1);
    result = (Dwg_ASSOCPARAMBASEDACTIONBODY *)& ((arg1)->pab);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGEFILLETACTIONBODY_sab_set) {
  {
    struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *arg1 = (struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *) 0 ;
    Dwg_ASSOCSURFACEACTIONBODY *arg2 = (Dwg_ASSOCSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGEFILLETACTIONBODY_sab_set(self,sab);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGEFILLETACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGEFILLETACTIONBODY_sab_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCEDGEFILLETACTIONBODY_sab_set" "', argument " "2"" of type '" "Dwg_ASSOCSURFACEACTIONBODY *""'"); 
    }
    arg2 = (Dwg_ASSOCSURFACEACTIONBODY *)(argp2);
    if (arg1) (arg1)->sab = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGEFILLETACTIONBODY_sab_get) {
  {
    struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *arg1 = (struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGEFILLETACTIONBODY_sab_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGEFILLETACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGEFILLETACTIONBODY_sab_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *)(argp1);
    result = (Dwg_ASSOCSURFACEACTIONBODY *)& ((arg1)->sab);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGEFILLETACTIONBODY_pbsab_status_set) {
  {
    struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *arg1 = (struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGEFILLETACTIONBODY_pbsab_status_set(self,pbsab_status);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGEFILLETACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGEFILLETACTIONBODY_pbsab_status_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCEDGEFILLETACTIONBODY_pbsab_status_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->pbsab_status = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCEDGEFILLETACTIONBODY_pbsab_status_get) {
  {
    struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *arg1 = (struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCEDGEFILLETACTIONBODY_pbsab_status_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGEFILLETACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCEDGEFILLETACTIONBODY_pbsab_status_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->pbsab_status);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCEDGEFILLETACTIONBODY) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCEDGEFILLETACTIONBODY();");
    }
    result = (struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *)calloc(1, sizeof(struct _dwg_object_ASSOCEDGEFILLETACTIONBODY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCEDGEFILLETACTIONBODY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCEDGEFILLETACTIONBODY) {
  {
    struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *arg1 = (struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCEDGEFILLETACTIONBODY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCEDGEFILLETACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCEDGEFILLETACTIONBODY" "', argument " "1"" of type '" "struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_parent_set) {
  {
    struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCTRIMSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_parent_get) {
  {
    struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCTRIMSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_aab_version_set) {
  {
    struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_aab_version_set(self,aab_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCTRIMSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_aab_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_aab_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->aab_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_aab_version_get) {
  {
    struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_aab_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCTRIMSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_aab_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->aab_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_pab_set) {
  {
    struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *) 0 ;
    Dwg_ASSOCPARAMBASEDACTIONBODY *arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_pab_set(self,pab);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCTRIMSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_pab_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_pab_set" "', argument " "2"" of type '" "Dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *)(argp2);
    if (arg1) (arg1)->pab = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_pab_get) {
  {
    struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCPARAMBASEDACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_pab_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCTRIMSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_pab_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *)(argp1);
    result = (Dwg_ASSOCPARAMBASEDACTIONBODY *)& ((arg1)->pab);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_sab_set) {
  {
    struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *) 0 ;
    Dwg_ASSOCSURFACEACTIONBODY *arg2 = (Dwg_ASSOCSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_sab_set(self,sab);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCTRIMSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_sab_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_sab_set" "', argument " "2"" of type '" "Dwg_ASSOCSURFACEACTIONBODY *""'"); 
    }
    arg2 = (Dwg_ASSOCSURFACEACTIONBODY *)(argp2);
    if (arg1) (arg1)->sab = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_sab_get) {
  {
    struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_sab_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCTRIMSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_sab_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *)(argp1);
    result = (Dwg_ASSOCSURFACEACTIONBODY *)& ((arg1)->sab);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_pbsab_status_set) {
  {
    struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_pbsab_status_set(self,pbsab_status);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCTRIMSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_pbsab_status_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_pbsab_status_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->pbsab_status = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_pbsab_status_get) {
  {
    struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_pbsab_status_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCTRIMSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_pbsab_status_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->pbsab_status);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_class_version_set) {
  {
    struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCTRIMSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_class_version_get) {
  {
    struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCTRIMSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_b1_set) {
  {
    struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_b1_set(self,b1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCTRIMSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_b1_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_b1_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->b1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_b1_get) {
  {
    struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_b1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCTRIMSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_b1_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_B) ((arg1)->b1);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_b2_set) {
  {
    struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_b2_set(self,b2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCTRIMSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_b2_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_b2_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->b2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_b2_get) {
  {
    struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_b2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCTRIMSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_b2_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_B) ((arg1)->b2);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_distance_set) {
  {
    struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_distance_set(self,distance);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCTRIMSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_distance_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_distance_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->distance = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_distance_get) {
  {
    struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_distance_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCTRIMSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_distance_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BD) ((arg1)->distance);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY();");
    }
    result = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *)calloc(1, sizeof(struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCTRIMSURFACEACTIONBODY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY) {
  {
    struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCTRIMSURFACEACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY" "', argument " "1"" of type '" "struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_parent_set) {
  {
    struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_parent_get) {
  {
    struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_aab_version_set) {
  {
    struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_aab_version_set(self,aab_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_aab_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_aab_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->aab_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_aab_version_get) {
  {
    struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_aab_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_aab_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->aab_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_pab_set) {
  {
    struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *) 0 ;
    Dwg_ASSOCPARAMBASEDACTIONBODY *arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_pab_set(self,pab);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_pab_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_pab_set" "', argument " "2"" of type '" "Dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *)(argp2);
    if (arg1) (arg1)->pab = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_pab_get) {
  {
    struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCPARAMBASEDACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_pab_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_pab_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *)(argp1);
    result = (Dwg_ASSOCPARAMBASEDACTIONBODY *)& ((arg1)->pab);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_sab_set) {
  {
    struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *) 0 ;
    Dwg_ASSOCSURFACEACTIONBODY *arg2 = (Dwg_ASSOCSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_sab_set(self,sab);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_sab_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_sab_set" "', argument " "2"" of type '" "Dwg_ASSOCSURFACEACTIONBODY *""'"); 
    }
    arg2 = (Dwg_ASSOCSURFACEACTIONBODY *)(argp2);
    if (arg1) (arg1)->sab = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_sab_get) {
  {
    struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_sab_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_sab_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *)(argp1);
    result = (Dwg_ASSOCSURFACEACTIONBODY *)& ((arg1)->sab);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_pbsab_status_set) {
  {
    struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_pbsab_status_set(self,pbsab_status);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_pbsab_status_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_pbsab_status_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->pbsab_status = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_pbsab_status_get) {
  {
    struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_pbsab_status_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_pbsab_status_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->pbsab_status);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_class_version_set) {
  {
    struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_class_version_get) {
  {
    struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b1_set) {
  {
    struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b1_set(self,b1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b1_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b1_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->b1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b1_get) {
  {
    struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b1_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_B) ((arg1)->b1);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b2_set) {
  {
    struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b2_set(self,b2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b2_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b2_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->b2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b2_get) {
  {
    struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b2_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_B) ((arg1)->b2);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b3_set) {
  {
    struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b3_set(self,b3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b3_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b3_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->b3 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b3_get) {
  {
    struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b3_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_B) ((arg1)->b3);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b4_set) {
  {
    struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b4_set(self,b4);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b4_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b4_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->b4 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b4_get) {
  {
    struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b4_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b4_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_B) ((arg1)->b4);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b5_set) {
  {
    struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b5_set(self,b5);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b5_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b5_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->b5 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b5_get) {
  {
    struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b5_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b5_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_B) ((arg1)->b5);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_blend_options_set) {
  {
    struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_blend_options_set(self,blend_options);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_blend_options_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_blend_options_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->blend_options = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_blend_options_get) {
  {
    struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_blend_options_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_blend_options_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BS) ((arg1)->blend_options);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_bs2_set) {
  {
    struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_bs2_set(self,bs2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_bs2_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_bs2_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->bs2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_bs2_get) {
  {
    struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_bs2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_bs2_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BS) ((arg1)->bs2);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY();");
    }
    result = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *)calloc(1, sizeof(struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY) {
  {
    struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY" "', argument " "1"" of type '" "struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_parent_set) {
  {
    struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFILLETSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_parent_get) {
  {
    struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFILLETSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_aab_version_set) {
  {
    struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_aab_version_set(self,aab_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFILLETSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_aab_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_aab_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->aab_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_aab_version_get) {
  {
    struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_aab_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFILLETSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_aab_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->aab_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pab_set) {
  {
    struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *) 0 ;
    Dwg_ASSOCPARAMBASEDACTIONBODY *arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pab_set(self,pab);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFILLETSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pab_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pab_set" "', argument " "2"" of type '" "Dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *)(argp2);
    if (arg1) (arg1)->pab = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pab_get) {
  {
    struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCPARAMBASEDACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pab_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFILLETSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pab_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *)(argp1);
    result = (Dwg_ASSOCPARAMBASEDACTIONBODY *)& ((arg1)->pab);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_sab_set) {
  {
    struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *) 0 ;
    Dwg_ASSOCSURFACEACTIONBODY *arg2 = (Dwg_ASSOCSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_sab_set(self,sab);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFILLETSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_sab_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_sab_set" "', argument " "2"" of type '" "Dwg_ASSOCSURFACEACTIONBODY *""'"); 
    }
    arg2 = (Dwg_ASSOCSURFACEACTIONBODY *)(argp2);
    if (arg1) (arg1)->sab = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_sab_get) {
  {
    struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_sab_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFILLETSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_sab_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *)(argp1);
    result = (Dwg_ASSOCSURFACEACTIONBODY *)& ((arg1)->sab);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pbsab_status_set) {
  {
    struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pbsab_status_set(self,pbsab_status);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFILLETSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pbsab_status_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pbsab_status_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->pbsab_status = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pbsab_status_get) {
  {
    struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pbsab_status_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFILLETSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pbsab_status_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->pbsab_status);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_class_version_set) {
  {
    struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFILLETSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_class_version_get) {
  {
    struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFILLETSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_status_set) {
  {
    struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_status_set(self,status);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFILLETSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_status_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_status_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->status = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_status_get) {
  {
    struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_status_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFILLETSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_status_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BS) ((arg1)->status);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pt1_set) {
  {
    struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pt1_set(self,pt1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFILLETSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pt1_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pt1_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->pt1 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pt1_get) {
  {
    struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pt1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFILLETSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pt1_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->pt1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pt2_set) {
  {
    struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pt2_set(self,pt2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFILLETSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pt2_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pt2_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->pt2 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pt2_get) {
  {
    struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pt2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFILLETSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pt2_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->pt2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY();");
    }
    result = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *)calloc(1, sizeof(struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCFILLETSURFACEACTIONBODY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY) {
  {
    struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCFILLETSURFACEACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY" "', argument " "1"" of type '" "struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_parent_set) {
  {
    struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATCHSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_parent_get) {
  {
    struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATCHSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_aab_version_set) {
  {
    struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_aab_version_set(self,aab_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATCHSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_aab_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_aab_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->aab_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_aab_version_get) {
  {
    struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_aab_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATCHSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_aab_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->aab_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_pab_set) {
  {
    struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *) 0 ;
    Dwg_ASSOCPARAMBASEDACTIONBODY *arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_pab_set(self,pab);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATCHSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_pab_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_pab_set" "', argument " "2"" of type '" "Dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *)(argp2);
    if (arg1) (arg1)->pab = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_pab_get) {
  {
    struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCPARAMBASEDACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_pab_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATCHSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_pab_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *)(argp1);
    result = (Dwg_ASSOCPARAMBASEDACTIONBODY *)& ((arg1)->pab);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_sab_set) {
  {
    struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *) 0 ;
    Dwg_ASSOCSURFACEACTIONBODY *arg2 = (Dwg_ASSOCSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_sab_set(self,sab);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATCHSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_sab_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_sab_set" "', argument " "2"" of type '" "Dwg_ASSOCSURFACEACTIONBODY *""'"); 
    }
    arg2 = (Dwg_ASSOCSURFACEACTIONBODY *)(argp2);
    if (arg1) (arg1)->sab = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_sab_get) {
  {
    struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_sab_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATCHSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_sab_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *)(argp1);
    result = (Dwg_ASSOCSURFACEACTIONBODY *)& ((arg1)->sab);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_pbsab_status_set) {
  {
    struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_pbsab_status_set(self,pbsab_status);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATCHSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_pbsab_status_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_pbsab_status_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->pbsab_status = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_pbsab_status_get) {
  {
    struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_pbsab_status_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATCHSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_pbsab_status_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->pbsab_status);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_class_version_set) {
  {
    struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATCHSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_class_version_get) {
  {
    struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATCHSURFACEACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY();");
    }
    result = (struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *)calloc(1, sizeof(struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCPATCHSURFACEACTIONBODY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY) {
  {
    struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *arg1 = (struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCPATCHSURFACEACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY" "', argument " "1"" of type '" "struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCACTIONBODY_action_parent_set) {
  {
    struct _dwg_ASSOCACTIONBODY_action *arg1 = (struct _dwg_ASSOCACTIONBODY_action *) 0 ;
    struct _dwg_object_ASSOCMLEADERACTIONBODY *arg2 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ASSOCACTIONBODY_action_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCACTIONBODY_action, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCACTIONBODY_action_parent_set" "', argument " "1"" of type '" "struct _dwg_ASSOCACTIONBODY_action *""'"); 
    }
    arg1 = (struct _dwg_ASSOCACTIONBODY_action *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCMLEADERACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_ASSOCACTIONBODY_action_parent_set" "', argument " "2"" of type '" "struct _dwg_object_ASSOCMLEADERACTIONBODY *""'"); 
    }
    arg2 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCACTIONBODY_action_parent_get) {
  {
    struct _dwg_ASSOCACTIONBODY_action *arg1 = (struct _dwg_ASSOCACTIONBODY_action *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_ASSOCMLEADERACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ASSOCACTIONBODY_action_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCACTIONBODY_action, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCACTIONBODY_action_parent_get" "', argument " "1"" of type '" "struct _dwg_ASSOCACTIONBODY_action *""'"); 
    }
    arg1 = (struct _dwg_ASSOCACTIONBODY_action *)(argp1);
    result = (struct _dwg_object_ASSOCMLEADERACTIONBODY *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCMLEADERACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCACTIONBODY_action_depid_set) {
  {
    struct _dwg_ASSOCACTIONBODY_action *arg1 = (struct _dwg_ASSOCACTIONBODY_action *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ASSOCACTIONBODY_action_depid_set(self,depid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCACTIONBODY_action, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCACTIONBODY_action_depid_set" "', argument " "1"" of type '" "struct _dwg_ASSOCACTIONBODY_action *""'"); 
    }
    arg1 = (struct _dwg_ASSOCACTIONBODY_action *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ASSOCACTIONBODY_action_depid_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->depid = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCACTIONBODY_action_depid_get) {
  {
    struct _dwg_ASSOCACTIONBODY_action *arg1 = (struct _dwg_ASSOCACTIONBODY_action *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ASSOCACTIONBODY_action_depid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCACTIONBODY_action, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCACTIONBODY_action_depid_get" "', argument " "1"" of type '" "struct _dwg_ASSOCACTIONBODY_action *""'"); 
    }
    arg1 = (struct _dwg_ASSOCACTIONBODY_action *)(argp1);
    result = (BITCODE_BL) ((arg1)->depid);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCACTIONBODY_action_dep_set) {
  {
    struct _dwg_ASSOCACTIONBODY_action *arg1 = (struct _dwg_ASSOCACTIONBODY_action *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ASSOCACTIONBODY_action_dep_set(self,dep);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCACTIONBODY_action, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCACTIONBODY_action_dep_set" "', argument " "1"" of type '" "struct _dwg_ASSOCACTIONBODY_action *""'"); 
    }
    arg1 = (struct _dwg_ASSOCACTIONBODY_action *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_ASSOCACTIONBODY_action_dep_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->dep = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ASSOCACTIONBODY_action_dep_get) {
  {
    struct _dwg_ASSOCACTIONBODY_action *arg1 = (struct _dwg_ASSOCACTIONBODY_action *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ASSOCACTIONBODY_action_dep_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCACTIONBODY_action, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ASSOCACTIONBODY_action_dep_get" "', argument " "1"" of type '" "struct _dwg_ASSOCACTIONBODY_action *""'"); 
    }
    arg1 = (struct _dwg_ASSOCACTIONBODY_action *)(argp1);
    result = (BITCODE_H) ((arg1)->dep);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_ASSOCACTIONBODY_action) {
  {
    int argvi = 0;
    struct _dwg_ASSOCACTIONBODY_action *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_ASSOCACTIONBODY_action();");
    }
    result = (struct _dwg_ASSOCACTIONBODY_action *)calloc(1, sizeof(struct _dwg_ASSOCACTIONBODY_action));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCACTIONBODY_action, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_ASSOCACTIONBODY_action) {
  {
    struct _dwg_ASSOCACTIONBODY_action *arg1 = (struct _dwg_ASSOCACTIONBODY_action *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_ASSOCACTIONBODY_action(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ASSOCACTIONBODY_action, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_ASSOCACTIONBODY_action" "', argument " "1"" of type '" "struct _dwg_ASSOCACTIONBODY_action *""'"); 
    }
    arg1 = (struct _dwg_ASSOCACTIONBODY_action *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_parent_set) {
  {
    struct _dwg_object_ASSOCMLEADERACTIONBODY *arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCMLEADERACTIONBODY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCMLEADERACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCMLEADERACTIONBODY_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCMLEADERACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCMLEADERACTIONBODY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_parent_get) {
  {
    struct _dwg_object_ASSOCMLEADERACTIONBODY *arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCMLEADERACTIONBODY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCMLEADERACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCMLEADERACTIONBODY_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCMLEADERACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_aaab_version_set) {
  {
    struct _dwg_object_ASSOCMLEADERACTIONBODY *arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCMLEADERACTIONBODY_aaab_version_set(self,aaab_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCMLEADERACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCMLEADERACTIONBODY_aaab_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCMLEADERACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCMLEADERACTIONBODY_aaab_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->aaab_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_aaab_version_get) {
  {
    struct _dwg_object_ASSOCMLEADERACTIONBODY *arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCMLEADERACTIONBODY_aaab_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCMLEADERACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCMLEADERACTIONBODY_aaab_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCMLEADERACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *)(argp1);
    result = (BITCODE_BS) ((arg1)->aaab_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_assoc_dep_set) {
  {
    struct _dwg_object_ASSOCMLEADERACTIONBODY *arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCMLEADERACTIONBODY_assoc_dep_set(self,assoc_dep);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCMLEADERACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCMLEADERACTIONBODY_assoc_dep_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCMLEADERACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCMLEADERACTIONBODY_assoc_dep_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->assoc_dep = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_assoc_dep_get) {
  {
    struct _dwg_object_ASSOCMLEADERACTIONBODY *arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCMLEADERACTIONBODY_assoc_dep_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCMLEADERACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCMLEADERACTIONBODY_assoc_dep_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCMLEADERACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *)(argp1);
    result = (BITCODE_H) ((arg1)->assoc_dep);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_aab_version_set) {
  {
    struct _dwg_object_ASSOCMLEADERACTIONBODY *arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCMLEADERACTIONBODY_aab_version_set(self,aab_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCMLEADERACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCMLEADERACTIONBODY_aab_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCMLEADERACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCMLEADERACTIONBODY_aab_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->aab_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_aab_version_get) {
  {
    struct _dwg_object_ASSOCMLEADERACTIONBODY *arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCMLEADERACTIONBODY_aab_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCMLEADERACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCMLEADERACTIONBODY_aab_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCMLEADERACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *)(argp1);
    result = (BITCODE_BS) ((arg1)->aab_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_actionbody_set) {
  {
    struct _dwg_object_ASSOCMLEADERACTIONBODY *arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCMLEADERACTIONBODY_actionbody_set(self,actionbody);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCMLEADERACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCMLEADERACTIONBODY_actionbody_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCMLEADERACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCMLEADERACTIONBODY_actionbody_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->actionbody = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_actionbody_get) {
  {
    struct _dwg_object_ASSOCMLEADERACTIONBODY *arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCMLEADERACTIONBODY_actionbody_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCMLEADERACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCMLEADERACTIONBODY_actionbody_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCMLEADERACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *)(argp1);
    result = (BITCODE_H) ((arg1)->actionbody);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_pab_set) {
  {
    struct _dwg_object_ASSOCMLEADERACTIONBODY *arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *) 0 ;
    Dwg_ASSOCPARAMBASEDACTIONBODY *arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCMLEADERACTIONBODY_pab_set(self,pab);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCMLEADERACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCMLEADERACTIONBODY_pab_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCMLEADERACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCMLEADERACTIONBODY_pab_set" "', argument " "2"" of type '" "Dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *)(argp2);
    if (arg1) (arg1)->pab = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_pab_get) {
  {
    struct _dwg_object_ASSOCMLEADERACTIONBODY *arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCPARAMBASEDACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCMLEADERACTIONBODY_pab_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCMLEADERACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCMLEADERACTIONBODY_pab_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCMLEADERACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *)(argp1);
    result = (Dwg_ASSOCPARAMBASEDACTIONBODY *)& ((arg1)->pab);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_class_version_set) {
  {
    struct _dwg_object_ASSOCMLEADERACTIONBODY *arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCMLEADERACTIONBODY_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCMLEADERACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCMLEADERACTIONBODY_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCMLEADERACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCMLEADERACTIONBODY_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_class_version_get) {
  {
    struct _dwg_object_ASSOCMLEADERACTIONBODY *arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCMLEADERACTIONBODY_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCMLEADERACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCMLEADERACTIONBODY_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCMLEADERACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_num_actions_set) {
  {
    struct _dwg_object_ASSOCMLEADERACTIONBODY *arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCMLEADERACTIONBODY_num_actions_set(self,num_actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCMLEADERACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCMLEADERACTIONBODY_num_actions_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCMLEADERACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCMLEADERACTIONBODY_num_actions_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_num_actions_get) {
  {
    struct _dwg_object_ASSOCMLEADERACTIONBODY *arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCMLEADERACTIONBODY_num_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCMLEADERACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCMLEADERACTIONBODY_num_actions_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCMLEADERACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_actions);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_actions_set) {
  {
    struct _dwg_object_ASSOCMLEADERACTIONBODY *arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *) 0 ;
    Dwg_ASSOCACTIONBODY_action *arg2 = (Dwg_ASSOCACTIONBODY_action *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCMLEADERACTIONBODY_actions_set(self,actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCMLEADERACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCMLEADERACTIONBODY_actions_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCMLEADERACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCACTIONBODY_action, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCMLEADERACTIONBODY_actions_set" "', argument " "2"" of type '" "Dwg_ASSOCACTIONBODY_action *""'"); 
    }
    arg2 = (Dwg_ASSOCACTIONBODY_action *)(argp2);
    if (arg1) (arg1)->actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_actions_get) {
  {
    struct _dwg_object_ASSOCMLEADERACTIONBODY *arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCACTIONBODY_action *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCMLEADERACTIONBODY_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCMLEADERACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCMLEADERACTIONBODY_actions_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCMLEADERACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *)(argp1);
    result = (Dwg_ASSOCACTIONBODY_action *) ((arg1)->actions);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCACTIONBODY_action, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCMLEADERACTIONBODY) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCMLEADERACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCMLEADERACTIONBODY();");
    }
    result = (struct _dwg_object_ASSOCMLEADERACTIONBODY *)calloc(1, sizeof(struct _dwg_object_ASSOCMLEADERACTIONBODY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCMLEADERACTIONBODY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCMLEADERACTIONBODY) {
  {
    struct _dwg_object_ASSOCMLEADERACTIONBODY *arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCMLEADERACTIONBODY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCMLEADERACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCMLEADERACTIONBODY" "', argument " "1"" of type '" "struct _dwg_object_ASSOCMLEADERACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCMLEADERACTIONBODY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_parent_set) {
  {
    struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCALIGNEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_parent_get) {
  {
    struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCALIGNEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_aaab_version_set) {
  {
    struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_aaab_version_set(self,aaab_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCALIGNEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_aaab_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_aaab_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->aaab_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_aaab_version_get) {
  {
    struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_aaab_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCALIGNEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_aaab_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *)(argp1);
    result = (BITCODE_BS) ((arg1)->aaab_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_assoc_dep_set) {
  {
    struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_assoc_dep_set(self,assoc_dep);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCALIGNEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_assoc_dep_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_assoc_dep_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->assoc_dep = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_assoc_dep_get) {
  {
    struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_assoc_dep_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCALIGNEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_assoc_dep_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *)(argp1);
    result = (BITCODE_H) ((arg1)->assoc_dep);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_aab_version_set) {
  {
    struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_aab_version_set(self,aab_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCALIGNEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_aab_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_aab_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->aab_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_aab_version_get) {
  {
    struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_aab_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCALIGNEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_aab_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *)(argp1);
    result = (BITCODE_BS) ((arg1)->aab_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_actionbody_set) {
  {
    struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_actionbody_set(self,actionbody);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCALIGNEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_actionbody_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_actionbody_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->actionbody = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_actionbody_get) {
  {
    struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_actionbody_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCALIGNEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_actionbody_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *)(argp1);
    result = (BITCODE_H) ((arg1)->actionbody);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_pab_set) {
  {
    struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *) 0 ;
    Dwg_ASSOCPARAMBASEDACTIONBODY *arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_pab_set(self,pab);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCALIGNEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_pab_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_pab_set" "', argument " "2"" of type '" "Dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *)(argp2);
    if (arg1) (arg1)->pab = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_pab_get) {
  {
    struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCPARAMBASEDACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_pab_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCALIGNEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_pab_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *)(argp1);
    result = (Dwg_ASSOCPARAMBASEDACTIONBODY *)& ((arg1)->pab);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_class_version_set) {
  {
    struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCALIGNEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_class_version_get) {
  {
    struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCALIGNEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_r_node_set) {
  {
    struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_r_node_set(self,r_node);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCALIGNEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_r_node_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_r_node_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->r_node = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_r_node_get) {
  {
    struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_r_node_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCALIGNEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_r_node_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *)(argp1);
    result = (BITCODE_H) ((arg1)->r_node);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_d_node_set) {
  {
    struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_d_node_set(self,d_node);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCALIGNEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_d_node_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_d_node_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->d_node = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_d_node_get) {
  {
    struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_d_node_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCALIGNEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_d_node_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *)(argp1);
    result = (BITCODE_H) ((arg1)->d_node);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY();");
    }
    result = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *)calloc(1, sizeof(struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCALIGNEDDIMACTIONBODY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY) {
  {
    struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCALIGNEDDIMACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY" "', argument " "1"" of type '" "struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_parent_set) {
  {
    struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC3POINTANGULARDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_parent_get) {
  {
    struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC3POINTANGULARDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_aaab_version_set) {
  {
    struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_aaab_version_set(self,aaab_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC3POINTANGULARDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_aaab_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_aaab_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->aaab_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_aaab_version_get) {
  {
    struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_aaab_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC3POINTANGULARDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_aaab_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *)(argp1);
    result = (BITCODE_BS) ((arg1)->aaab_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_assoc_dep_set) {
  {
    struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_assoc_dep_set(self,assoc_dep);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC3POINTANGULARDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_assoc_dep_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_assoc_dep_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->assoc_dep = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_assoc_dep_get) {
  {
    struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_assoc_dep_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC3POINTANGULARDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_assoc_dep_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *)(argp1);
    result = (BITCODE_H) ((arg1)->assoc_dep);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_aab_version_set) {
  {
    struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_aab_version_set(self,aab_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC3POINTANGULARDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_aab_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_aab_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->aab_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_aab_version_get) {
  {
    struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_aab_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC3POINTANGULARDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_aab_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *)(argp1);
    result = (BITCODE_BS) ((arg1)->aab_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_actionbody_set) {
  {
    struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_actionbody_set(self,actionbody);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC3POINTANGULARDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_actionbody_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_actionbody_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->actionbody = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_actionbody_get) {
  {
    struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_actionbody_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC3POINTANGULARDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_actionbody_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *)(argp1);
    result = (BITCODE_H) ((arg1)->actionbody);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_pab_set) {
  {
    struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *) 0 ;
    Dwg_ASSOCPARAMBASEDACTIONBODY *arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_pab_set(self,pab);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC3POINTANGULARDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_pab_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_pab_set" "', argument " "2"" of type '" "Dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *)(argp2);
    if (arg1) (arg1)->pab = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_pab_get) {
  {
    struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCPARAMBASEDACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_pab_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC3POINTANGULARDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_pab_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *)(argp1);
    result = (Dwg_ASSOCPARAMBASEDACTIONBODY *)& ((arg1)->pab);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_class_version_set) {
  {
    struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC3POINTANGULARDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_class_version_get) {
  {
    struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC3POINTANGULARDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_r_node_set) {
  {
    struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_r_node_set(self,r_node);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC3POINTANGULARDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_r_node_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_r_node_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->r_node = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_r_node_get) {
  {
    struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_r_node_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC3POINTANGULARDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_r_node_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *)(argp1);
    result = (BITCODE_H) ((arg1)->r_node);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_d_node_set) {
  {
    struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_d_node_set(self,d_node);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC3POINTANGULARDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_d_node_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_d_node_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->d_node = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_d_node_get) {
  {
    struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_d_node_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC3POINTANGULARDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_d_node_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *)(argp1);
    result = (BITCODE_H) ((arg1)->d_node);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_assocdep_set) {
  {
    struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_assocdep_set(self,assocdep);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC3POINTANGULARDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_assocdep_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_assocdep_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->assocdep = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_assocdep_get) {
  {
    struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_assocdep_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC3POINTANGULARDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_assocdep_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *)(argp1);
    result = (BITCODE_H) ((arg1)->assocdep);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY();");
    }
    result = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *)calloc(1, sizeof(struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOC3POINTANGULARDIMACTIONBODY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY) {
  {
    struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOC3POINTANGULARDIMACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY" "', argument " "1"" of type '" "struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_parent_set) {
  {
    struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCORDINATEDIMACTIONBODY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCORDINATEDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCORDINATEDIMACTIONBODY_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCORDINATEDIMACTIONBODY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_parent_get) {
  {
    struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCORDINATEDIMACTIONBODY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCORDINATEDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCORDINATEDIMACTIONBODY_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_aaab_version_set) {
  {
    struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCORDINATEDIMACTIONBODY_aaab_version_set(self,aaab_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCORDINATEDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCORDINATEDIMACTIONBODY_aaab_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCORDINATEDIMACTIONBODY_aaab_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->aaab_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_aaab_version_get) {
  {
    struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCORDINATEDIMACTIONBODY_aaab_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCORDINATEDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCORDINATEDIMACTIONBODY_aaab_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *)(argp1);
    result = (BITCODE_BS) ((arg1)->aaab_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_assoc_dep_set) {
  {
    struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCORDINATEDIMACTIONBODY_assoc_dep_set(self,assoc_dep);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCORDINATEDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCORDINATEDIMACTIONBODY_assoc_dep_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCORDINATEDIMACTIONBODY_assoc_dep_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->assoc_dep = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_assoc_dep_get) {
  {
    struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCORDINATEDIMACTIONBODY_assoc_dep_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCORDINATEDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCORDINATEDIMACTIONBODY_assoc_dep_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *)(argp1);
    result = (BITCODE_H) ((arg1)->assoc_dep);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_aab_version_set) {
  {
    struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCORDINATEDIMACTIONBODY_aab_version_set(self,aab_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCORDINATEDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCORDINATEDIMACTIONBODY_aab_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCORDINATEDIMACTIONBODY_aab_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->aab_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_aab_version_get) {
  {
    struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCORDINATEDIMACTIONBODY_aab_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCORDINATEDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCORDINATEDIMACTIONBODY_aab_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *)(argp1);
    result = (BITCODE_BS) ((arg1)->aab_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_actionbody_set) {
  {
    struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCORDINATEDIMACTIONBODY_actionbody_set(self,actionbody);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCORDINATEDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCORDINATEDIMACTIONBODY_actionbody_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCORDINATEDIMACTIONBODY_actionbody_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->actionbody = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_actionbody_get) {
  {
    struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCORDINATEDIMACTIONBODY_actionbody_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCORDINATEDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCORDINATEDIMACTIONBODY_actionbody_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *)(argp1);
    result = (BITCODE_H) ((arg1)->actionbody);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_pab_set) {
  {
    struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *) 0 ;
    Dwg_ASSOCPARAMBASEDACTIONBODY *arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCORDINATEDIMACTIONBODY_pab_set(self,pab);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCORDINATEDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCORDINATEDIMACTIONBODY_pab_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCORDINATEDIMACTIONBODY_pab_set" "', argument " "2"" of type '" "Dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *)(argp2);
    if (arg1) (arg1)->pab = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_pab_get) {
  {
    struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCPARAMBASEDACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCORDINATEDIMACTIONBODY_pab_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCORDINATEDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCORDINATEDIMACTIONBODY_pab_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *)(argp1);
    result = (Dwg_ASSOCPARAMBASEDACTIONBODY *)& ((arg1)->pab);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_class_version_set) {
  {
    struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCORDINATEDIMACTIONBODY_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCORDINATEDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCORDINATEDIMACTIONBODY_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCORDINATEDIMACTIONBODY_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_class_version_get) {
  {
    struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCORDINATEDIMACTIONBODY_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCORDINATEDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCORDINATEDIMACTIONBODY_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_r_node_set) {
  {
    struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCORDINATEDIMACTIONBODY_r_node_set(self,r_node);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCORDINATEDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCORDINATEDIMACTIONBODY_r_node_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCORDINATEDIMACTIONBODY_r_node_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->r_node = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_r_node_get) {
  {
    struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCORDINATEDIMACTIONBODY_r_node_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCORDINATEDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCORDINATEDIMACTIONBODY_r_node_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *)(argp1);
    result = (BITCODE_H) ((arg1)->r_node);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_d_node_set) {
  {
    struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCORDINATEDIMACTIONBODY_d_node_set(self,d_node);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCORDINATEDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCORDINATEDIMACTIONBODY_d_node_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCORDINATEDIMACTIONBODY_d_node_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->d_node = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_d_node_get) {
  {
    struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCORDINATEDIMACTIONBODY_d_node_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCORDINATEDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCORDINATEDIMACTIONBODY_d_node_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *)(argp1);
    result = (BITCODE_H) ((arg1)->d_node);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCORDINATEDIMACTIONBODY) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCORDINATEDIMACTIONBODY();");
    }
    result = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *)calloc(1, sizeof(struct _dwg_object_ASSOCORDINATEDIMACTIONBODY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCORDINATEDIMACTIONBODY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCORDINATEDIMACTIONBODY) {
  {
    struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCORDINATEDIMACTIONBODY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCORDINATEDIMACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCORDINATEDIMACTIONBODY" "', argument " "1"" of type '" "struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_parent_set) {
  {
    struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCROTATEDDIMACTIONBODY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCROTATEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCROTATEDDIMACTIONBODY_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCROTATEDDIMACTIONBODY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_parent_get) {
  {
    struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCROTATEDDIMACTIONBODY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCROTATEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCROTATEDDIMACTIONBODY_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_aaab_version_set) {
  {
    struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCROTATEDDIMACTIONBODY_aaab_version_set(self,aaab_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCROTATEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCROTATEDDIMACTIONBODY_aaab_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCROTATEDDIMACTIONBODY_aaab_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->aaab_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_aaab_version_get) {
  {
    struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCROTATEDDIMACTIONBODY_aaab_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCROTATEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCROTATEDDIMACTIONBODY_aaab_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *)(argp1);
    result = (BITCODE_BS) ((arg1)->aaab_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_assoc_dep_set) {
  {
    struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCROTATEDDIMACTIONBODY_assoc_dep_set(self,assoc_dep);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCROTATEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCROTATEDDIMACTIONBODY_assoc_dep_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCROTATEDDIMACTIONBODY_assoc_dep_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->assoc_dep = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_assoc_dep_get) {
  {
    struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCROTATEDDIMACTIONBODY_assoc_dep_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCROTATEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCROTATEDDIMACTIONBODY_assoc_dep_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *)(argp1);
    result = (BITCODE_H) ((arg1)->assoc_dep);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_aab_version_set) {
  {
    struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCROTATEDDIMACTIONBODY_aab_version_set(self,aab_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCROTATEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCROTATEDDIMACTIONBODY_aab_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCROTATEDDIMACTIONBODY_aab_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->aab_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_aab_version_get) {
  {
    struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCROTATEDDIMACTIONBODY_aab_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCROTATEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCROTATEDDIMACTIONBODY_aab_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *)(argp1);
    result = (BITCODE_BS) ((arg1)->aab_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_actionbody_set) {
  {
    struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCROTATEDDIMACTIONBODY_actionbody_set(self,actionbody);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCROTATEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCROTATEDDIMACTIONBODY_actionbody_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCROTATEDDIMACTIONBODY_actionbody_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->actionbody = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_actionbody_get) {
  {
    struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCROTATEDDIMACTIONBODY_actionbody_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCROTATEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCROTATEDDIMACTIONBODY_actionbody_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *)(argp1);
    result = (BITCODE_H) ((arg1)->actionbody);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_pab_set) {
  {
    struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *) 0 ;
    Dwg_ASSOCPARAMBASEDACTIONBODY *arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCROTATEDDIMACTIONBODY_pab_set(self,pab);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCROTATEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCROTATEDDIMACTIONBODY_pab_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCROTATEDDIMACTIONBODY_pab_set" "', argument " "2"" of type '" "Dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *)(argp2);
    if (arg1) (arg1)->pab = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_pab_get) {
  {
    struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCPARAMBASEDACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCROTATEDDIMACTIONBODY_pab_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCROTATEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCROTATEDDIMACTIONBODY_pab_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *)(argp1);
    result = (Dwg_ASSOCPARAMBASEDACTIONBODY *)& ((arg1)->pab);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_class_version_set) {
  {
    struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCROTATEDDIMACTIONBODY_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCROTATEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCROTATEDDIMACTIONBODY_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCROTATEDDIMACTIONBODY_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_class_version_get) {
  {
    struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCROTATEDDIMACTIONBODY_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCROTATEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCROTATEDDIMACTIONBODY_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_r_node_set) {
  {
    struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCROTATEDDIMACTIONBODY_r_node_set(self,r_node);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCROTATEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCROTATEDDIMACTIONBODY_r_node_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCROTATEDDIMACTIONBODY_r_node_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->r_node = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_r_node_get) {
  {
    struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCROTATEDDIMACTIONBODY_r_node_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCROTATEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCROTATEDDIMACTIONBODY_r_node_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *)(argp1);
    result = (BITCODE_H) ((arg1)->r_node);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_d_node_set) {
  {
    struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCROTATEDDIMACTIONBODY_d_node_set(self,d_node);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCROTATEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCROTATEDDIMACTIONBODY_d_node_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCROTATEDDIMACTIONBODY_d_node_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->d_node = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_d_node_get) {
  {
    struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCROTATEDDIMACTIONBODY_d_node_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCROTATEDDIMACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCROTATEDDIMACTIONBODY_d_node_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *)(argp1);
    result = (BITCODE_H) ((arg1)->d_node);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCROTATEDDIMACTIONBODY) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCROTATEDDIMACTIONBODY();");
    }
    result = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *)calloc(1, sizeof(struct _dwg_object_ASSOCROTATEDDIMACTIONBODY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCROTATEDDIMACTIONBODY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCROTATEDDIMACTIONBODY) {
  {
    struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCROTATEDDIMACTIONBODY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCROTATEDDIMACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCROTATEDDIMACTIONBODY" "', argument " "1"" of type '" "struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDIMDEPENDENCYBODY_parent_set) {
  {
    struct _dwg_object_ASSOCDIMDEPENDENCYBODY *arg1 = (struct _dwg_object_ASSOCDIMDEPENDENCYBODY *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDIMDEPENDENCYBODY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDIMDEPENDENCYBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDIMDEPENDENCYBODY_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDIMDEPENDENCYBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDIMDEPENDENCYBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCDIMDEPENDENCYBODY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDIMDEPENDENCYBODY_parent_get) {
  {
    struct _dwg_object_ASSOCDIMDEPENDENCYBODY *arg1 = (struct _dwg_object_ASSOCDIMDEPENDENCYBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDIMDEPENDENCYBODY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDIMDEPENDENCYBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDIMDEPENDENCYBODY_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDIMDEPENDENCYBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDIMDEPENDENCYBODY *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDIMDEPENDENCYBODY_adb_version_set) {
  {
    struct _dwg_object_ASSOCDIMDEPENDENCYBODY *arg1 = (struct _dwg_object_ASSOCDIMDEPENDENCYBODY *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDIMDEPENDENCYBODY_adb_version_set(self,adb_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDIMDEPENDENCYBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDIMDEPENDENCYBODY_adb_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDIMDEPENDENCYBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDIMDEPENDENCYBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCDIMDEPENDENCYBODY_adb_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->adb_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDIMDEPENDENCYBODY_adb_version_get) {
  {
    struct _dwg_object_ASSOCDIMDEPENDENCYBODY *arg1 = (struct _dwg_object_ASSOCDIMDEPENDENCYBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDIMDEPENDENCYBODY_adb_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDIMDEPENDENCYBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDIMDEPENDENCYBODY_adb_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDIMDEPENDENCYBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDIMDEPENDENCYBODY *)(argp1);
    result = (BITCODE_BS) ((arg1)->adb_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDIMDEPENDENCYBODY_dimbase_version_set) {
  {
    struct _dwg_object_ASSOCDIMDEPENDENCYBODY *arg1 = (struct _dwg_object_ASSOCDIMDEPENDENCYBODY *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDIMDEPENDENCYBODY_dimbase_version_set(self,dimbase_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDIMDEPENDENCYBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDIMDEPENDENCYBODY_dimbase_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDIMDEPENDENCYBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDIMDEPENDENCYBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCDIMDEPENDENCYBODY_dimbase_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->dimbase_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDIMDEPENDENCYBODY_dimbase_version_get) {
  {
    struct _dwg_object_ASSOCDIMDEPENDENCYBODY *arg1 = (struct _dwg_object_ASSOCDIMDEPENDENCYBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDIMDEPENDENCYBODY_dimbase_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDIMDEPENDENCYBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDIMDEPENDENCYBODY_dimbase_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDIMDEPENDENCYBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDIMDEPENDENCYBODY *)(argp1);
    result = (BITCODE_BS) ((arg1)->dimbase_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDIMDEPENDENCYBODY_name_set) {
  {
    struct _dwg_object_ASSOCDIMDEPENDENCYBODY *arg1 = (struct _dwg_object_ASSOCDIMDEPENDENCYBODY *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDIMDEPENDENCYBODY_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDIMDEPENDENCYBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDIMDEPENDENCYBODY_name_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDIMDEPENDENCYBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDIMDEPENDENCYBODY *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCDIMDEPENDENCYBODY_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDIMDEPENDENCYBODY_name_get) {
  {
    struct _dwg_object_ASSOCDIMDEPENDENCYBODY *arg1 = (struct _dwg_object_ASSOCDIMDEPENDENCYBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDIMDEPENDENCYBODY_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDIMDEPENDENCYBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDIMDEPENDENCYBODY_name_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDIMDEPENDENCYBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDIMDEPENDENCYBODY *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDIMDEPENDENCYBODY_class_version_set) {
  {
    struct _dwg_object_ASSOCDIMDEPENDENCYBODY *arg1 = (struct _dwg_object_ASSOCDIMDEPENDENCYBODY *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDIMDEPENDENCYBODY_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDIMDEPENDENCYBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDIMDEPENDENCYBODY_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDIMDEPENDENCYBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDIMDEPENDENCYBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCDIMDEPENDENCYBODY_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCDIMDEPENDENCYBODY_class_version_get) {
  {
    struct _dwg_object_ASSOCDIMDEPENDENCYBODY *arg1 = (struct _dwg_object_ASSOCDIMDEPENDENCYBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCDIMDEPENDENCYBODY_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDIMDEPENDENCYBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCDIMDEPENDENCYBODY_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDIMDEPENDENCYBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDIMDEPENDENCYBODY *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCDIMDEPENDENCYBODY) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCDIMDEPENDENCYBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCDIMDEPENDENCYBODY();");
    }
    result = (struct _dwg_object_ASSOCDIMDEPENDENCYBODY *)calloc(1, sizeof(struct _dwg_object_ASSOCDIMDEPENDENCYBODY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCDIMDEPENDENCYBODY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCDIMDEPENDENCYBODY) {
  {
    struct _dwg_object_ASSOCDIMDEPENDENCYBODY *arg1 = (struct _dwg_object_ASSOCDIMDEPENDENCYBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCDIMDEPENDENCYBODY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCDIMDEPENDENCYBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCDIMDEPENDENCYBODY" "', argument " "1"" of type '" "struct _dwg_object_ASSOCDIMDEPENDENCYBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCDIMDEPENDENCYBODY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPARAMDEPENDENCYBODY_parent_set) {
  {
    struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *arg1 = (struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPARAMDEPENDENCYBODY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPARAMDEPENDENCYBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPARAMDEPENDENCYBODY_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPARAMDEPENDENCYBODY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPARAMDEPENDENCYBODY_parent_get) {
  {
    struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *arg1 = (struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPARAMDEPENDENCYBODY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPARAMDEPENDENCYBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPARAMDEPENDENCYBODY_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPARAMDEPENDENCYBODY_adb_version_set) {
  {
    struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *arg1 = (struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPARAMDEPENDENCYBODY_adb_version_set(self,adb_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPARAMDEPENDENCYBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPARAMDEPENDENCYBODY_adb_version_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPARAMDEPENDENCYBODY_adb_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->adb_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPARAMDEPENDENCYBODY_adb_version_get) {
  {
    struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *arg1 = (struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPARAMDEPENDENCYBODY_adb_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPARAMDEPENDENCYBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPARAMDEPENDENCYBODY_adb_version_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *)(argp1);
    result = (BITCODE_BS) ((arg1)->adb_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPARAMDEPENDENCYBODY_dimbase_version_set) {
  {
    struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *arg1 = (struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPARAMDEPENDENCYBODY_dimbase_version_set(self,dimbase_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPARAMDEPENDENCYBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPARAMDEPENDENCYBODY_dimbase_version_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPARAMDEPENDENCYBODY_dimbase_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->dimbase_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPARAMDEPENDENCYBODY_dimbase_version_get) {
  {
    struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *arg1 = (struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPARAMDEPENDENCYBODY_dimbase_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPARAMDEPENDENCYBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPARAMDEPENDENCYBODY_dimbase_version_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *)(argp1);
    result = (BITCODE_BS) ((arg1)->dimbase_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPARAMDEPENDENCYBODY_name_set) {
  {
    struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *arg1 = (struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPARAMDEPENDENCYBODY_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPARAMDEPENDENCYBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPARAMDEPENDENCYBODY_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPARAMDEPENDENCYBODY_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPARAMDEPENDENCYBODY_name_get) {
  {
    struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *arg1 = (struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPARAMDEPENDENCYBODY_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPARAMDEPENDENCYBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPARAMDEPENDENCYBODY_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPARAMDEPENDENCYBODY_class_version_set) {
  {
    struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *arg1 = (struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPARAMDEPENDENCYBODY_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPARAMDEPENDENCYBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPARAMDEPENDENCYBODY_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPARAMDEPENDENCYBODY_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPARAMDEPENDENCYBODY_class_version_get) {
  {
    struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *arg1 = (struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPARAMDEPENDENCYBODY_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPARAMDEPENDENCYBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPARAMDEPENDENCYBODY_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKPARAMDEPENDENCYBODY) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKPARAMDEPENDENCYBODY();");
    }
    result = (struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *)calloc(1, sizeof(struct _dwg_object_BLOCKPARAMDEPENDENCYBODY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKPARAMDEPENDENCYBODY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKPARAMDEPENDENCYBODY) {
  {
    struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *arg1 = (struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKPARAMDEPENDENCYBODY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPARAMDEPENDENCYBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKPARAMDEPENDENCYBODY" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ARRAYITEMLOCATOR_parent_set) {
  {
    struct _dwg_ARRAYITEMLOCATOR *arg1 = (struct _dwg_ARRAYITEMLOCATOR *) 0 ;
    struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *arg2 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ARRAYITEMLOCATOR_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ARRAYITEMLOCATOR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ARRAYITEMLOCATOR_parent_set" "', argument " "1"" of type '" "struct _dwg_ARRAYITEMLOCATOR *""'"); 
    }
    arg1 = (struct _dwg_ARRAYITEMLOCATOR *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCARRAYMODIFYACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_ARRAYITEMLOCATOR_parent_set" "', argument " "2"" of type '" "struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *""'"); 
    }
    arg2 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ARRAYITEMLOCATOR_parent_get) {
  {
    struct _dwg_ARRAYITEMLOCATOR *arg1 = (struct _dwg_ARRAYITEMLOCATOR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ARRAYITEMLOCATOR_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ARRAYITEMLOCATOR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ARRAYITEMLOCATOR_parent_get" "', argument " "1"" of type '" "struct _dwg_ARRAYITEMLOCATOR *""'"); 
    }
    arg1 = (struct _dwg_ARRAYITEMLOCATOR *)(argp1);
    result = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCARRAYMODIFYACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ARRAYITEMLOCATOR_itemloc_set) {
  {
    struct _dwg_ARRAYITEMLOCATOR *arg1 = (struct _dwg_ARRAYITEMLOCATOR *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) (BITCODE_BL *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ARRAYITEMLOCATOR_itemloc_set(self,itemloc);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ARRAYITEMLOCATOR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ARRAYITEMLOCATOR_itemloc_set" "', argument " "1"" of type '" "struct _dwg_ARRAYITEMLOCATOR *""'"); 
    }
    arg1 = (struct _dwg_ARRAYITEMLOCATOR *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_ARRAYITEMLOCATOR_itemloc_set" "', argument " "2"" of type '" "BITCODE_BL [3]""'"); 
    } 
    arg2 = (BITCODE_BL *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)3; ++ii) *(BITCODE_BL *)&arg1->itemloc[ii] = *((BITCODE_BL *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""itemloc""' of type '""BITCODE_BL [3]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ARRAYITEMLOCATOR_itemloc_get) {
  {
    struct _dwg_ARRAYITEMLOCATOR *arg1 = (struct _dwg_ARRAYITEMLOCATOR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ARRAYITEMLOCATOR_itemloc_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ARRAYITEMLOCATOR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ARRAYITEMLOCATOR_itemloc_get" "', argument " "1"" of type '" "struct _dwg_ARRAYITEMLOCATOR *""'"); 
    }
    arg1 = (struct _dwg_ARRAYITEMLOCATOR *)(argp1);
    result = (BITCODE_BL *)(BITCODE_BL *) ((arg1)->itemloc);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_ARRAYITEMLOCATOR) {
  {
    int argvi = 0;
    struct _dwg_ARRAYITEMLOCATOR *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_ARRAYITEMLOCATOR();");
    }
    result = (struct _dwg_ARRAYITEMLOCATOR *)calloc(1, sizeof(struct _dwg_ARRAYITEMLOCATOR));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ARRAYITEMLOCATOR, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_ARRAYITEMLOCATOR) {
  {
    struct _dwg_ARRAYITEMLOCATOR *arg1 = (struct _dwg_ARRAYITEMLOCATOR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_ARRAYITEMLOCATOR(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ARRAYITEMLOCATOR, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_ARRAYITEMLOCATOR" "', argument " "1"" of type '" "struct _dwg_ARRAYITEMLOCATOR *""'"); 
    }
    arg1 = (struct _dwg_ARRAYITEMLOCATOR *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYACTIONBODY_parent_set) {
  {
    struct _dwg_object_ASSOCARRAYACTIONBODY *arg1 = (struct _dwg_object_ASSOCARRAYACTIONBODY *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYACTIONBODY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCARRAYACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYACTIONBODY_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCARRAYACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCARRAYACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCARRAYACTIONBODY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYACTIONBODY_parent_get) {
  {
    struct _dwg_object_ASSOCARRAYACTIONBODY *arg1 = (struct _dwg_object_ASSOCARRAYACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYACTIONBODY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCARRAYACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYACTIONBODY_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCARRAYACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCARRAYACTIONBODY *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYACTIONBODY_aab_version_set) {
  {
    struct _dwg_object_ASSOCARRAYACTIONBODY *arg1 = (struct _dwg_object_ASSOCARRAYACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYACTIONBODY_aab_version_set(self,aab_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCARRAYACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYACTIONBODY_aab_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCARRAYACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCARRAYACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCARRAYACTIONBODY_aab_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->aab_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYACTIONBODY_aab_version_get) {
  {
    struct _dwg_object_ASSOCARRAYACTIONBODY *arg1 = (struct _dwg_object_ASSOCARRAYACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYACTIONBODY_aab_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCARRAYACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYACTIONBODY_aab_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCARRAYACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCARRAYACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->aab_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYACTIONBODY_pab_set) {
  {
    struct _dwg_object_ASSOCARRAYACTIONBODY *arg1 = (struct _dwg_object_ASSOCARRAYACTIONBODY *) 0 ;
    Dwg_ASSOCPARAMBASEDACTIONBODY *arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYACTIONBODY_pab_set(self,pab);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCARRAYACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYACTIONBODY_pab_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCARRAYACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCARRAYACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCARRAYACTIONBODY_pab_set" "', argument " "2"" of type '" "Dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *)(argp2);
    if (arg1) (arg1)->pab = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYACTIONBODY_pab_get) {
  {
    struct _dwg_object_ASSOCARRAYACTIONBODY *arg1 = (struct _dwg_object_ASSOCARRAYACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCPARAMBASEDACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYACTIONBODY_pab_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCARRAYACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYACTIONBODY_pab_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCARRAYACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCARRAYACTIONBODY *)(argp1);
    result = (Dwg_ASSOCPARAMBASEDACTIONBODY *)& ((arg1)->pab);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYACTIONBODY_aaab_version_set) {
  {
    struct _dwg_object_ASSOCARRAYACTIONBODY *arg1 = (struct _dwg_object_ASSOCARRAYACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYACTIONBODY_aaab_version_set(self,aaab_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCARRAYACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYACTIONBODY_aaab_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCARRAYACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCARRAYACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCARRAYACTIONBODY_aaab_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->aaab_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYACTIONBODY_aaab_version_get) {
  {
    struct _dwg_object_ASSOCARRAYACTIONBODY *arg1 = (struct _dwg_object_ASSOCARRAYACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYACTIONBODY_aaab_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCARRAYACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYACTIONBODY_aaab_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCARRAYACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCARRAYACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->aaab_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYACTIONBODY_paramblock_set) {
  {
    struct _dwg_object_ASSOCARRAYACTIONBODY *arg1 = (struct _dwg_object_ASSOCARRAYACTIONBODY *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYACTIONBODY_paramblock_set(self,paramblock);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCARRAYACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYACTIONBODY_paramblock_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCARRAYACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCARRAYACTIONBODY *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCARRAYACTIONBODY_paramblock_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->paramblock);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->paramblock = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->paramblock = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYACTIONBODY_paramblock_get) {
  {
    struct _dwg_object_ASSOCARRAYACTIONBODY *arg1 = (struct _dwg_object_ASSOCARRAYACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYACTIONBODY_paramblock_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCARRAYACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYACTIONBODY_paramblock_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCARRAYACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCARRAYACTIONBODY *)(argp1);
    result = (BITCODE_TV) ((arg1)->paramblock);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYACTIONBODY_transmatrix_set) {
  {
    struct _dwg_object_ASSOCARRAYACTIONBODY *arg1 = (struct _dwg_object_ASSOCARRAYACTIONBODY *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYACTIONBODY_transmatrix_set(self,transmatrix);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCARRAYACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYACTIONBODY_transmatrix_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCARRAYACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCARRAYACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCARRAYACTIONBODY_transmatrix_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->transmatrix = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYACTIONBODY_transmatrix_get) {
  {
    struct _dwg_object_ASSOCARRAYACTIONBODY *arg1 = (struct _dwg_object_ASSOCARRAYACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYACTIONBODY_transmatrix_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCARRAYACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYACTIONBODY_transmatrix_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCARRAYACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCARRAYACTIONBODY *)(argp1);
    result = (BITCODE_BD *) ((arg1)->transmatrix);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCARRAYACTIONBODY) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCARRAYACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCARRAYACTIONBODY();");
    }
    result = (struct _dwg_object_ASSOCARRAYACTIONBODY *)calloc(1, sizeof(struct _dwg_object_ASSOCARRAYACTIONBODY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCARRAYACTIONBODY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCARRAYACTIONBODY) {
  {
    struct _dwg_object_ASSOCARRAYACTIONBODY *arg1 = (struct _dwg_object_ASSOCARRAYACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCARRAYACTIONBODY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCARRAYACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCARRAYACTIONBODY" "', argument " "1"" of type '" "struct _dwg_object_ASSOCARRAYACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCARRAYACTIONBODY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_parent_set) {
  {
    struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCARRAYMODIFYACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_parent_get) {
  {
    struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCARRAYMODIFYACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_aab_version_set) {
  {
    struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_aab_version_set(self,aab_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCARRAYMODIFYACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_aab_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_aab_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->aab_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_aab_version_get) {
  {
    struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_aab_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCARRAYMODIFYACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_aab_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->aab_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_pab_set) {
  {
    struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *) 0 ;
    Dwg_ASSOCPARAMBASEDACTIONBODY *arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_pab_set(self,pab);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCARRAYMODIFYACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_pab_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_pab_set" "', argument " "2"" of type '" "Dwg_ASSOCPARAMBASEDACTIONBODY *""'"); 
    }
    arg2 = (Dwg_ASSOCPARAMBASEDACTIONBODY *)(argp2);
    if (arg1) (arg1)->pab = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_pab_get) {
  {
    struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ASSOCPARAMBASEDACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_pab_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCARRAYMODIFYACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_pab_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *)(argp1);
    result = (Dwg_ASSOCPARAMBASEDACTIONBODY *)& ((arg1)->pab);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_aaab_version_set) {
  {
    struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_aaab_version_set(self,aaab_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCARRAYMODIFYACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_aaab_version_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_aaab_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->aaab_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_aaab_version_get) {
  {
    struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_aaab_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCARRAYMODIFYACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_aaab_version_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->aaab_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_paramblock_set) {
  {
    struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_paramblock_set(self,paramblock);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCARRAYMODIFYACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_paramblock_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_paramblock_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->paramblock);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->paramblock = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->paramblock = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_paramblock_get) {
  {
    struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_paramblock_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCARRAYMODIFYACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_paramblock_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *)(argp1);
    result = (BITCODE_TV) ((arg1)->paramblock);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_transmatrix_set) {
  {
    struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_transmatrix_set(self,transmatrix);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCARRAYMODIFYACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_transmatrix_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_transmatrix_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->transmatrix = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_transmatrix_get) {
  {
    struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_transmatrix_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCARRAYMODIFYACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_transmatrix_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *)(argp1);
    result = (BITCODE_BD *) ((arg1)->transmatrix);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_status_set) {
  {
    struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_status_set(self,status);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCARRAYMODIFYACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_status_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_status_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->status = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_status_get) {
  {
    struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_status_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCARRAYMODIFYACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_status_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *)(argp1);
    result = (BITCODE_BS) ((arg1)->status);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_num_items_set) {
  {
    struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_num_items_set(self,num_items);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCARRAYMODIFYACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_num_items_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_num_items_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_items = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_num_items_get) {
  {
    struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_num_items_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCARRAYMODIFYACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_num_items_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_items);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_items_set) {
  {
    struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *) 0 ;
    Dwg_ARRAYITEMLOCATOR *arg2 = (Dwg_ARRAYITEMLOCATOR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_items_set(self,items);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCARRAYMODIFYACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_items_set" "', argument " "1"" of type '" "struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ARRAYITEMLOCATOR, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_items_set" "', argument " "2"" of type '" "Dwg_ARRAYITEMLOCATOR *""'"); 
    }
    arg2 = (Dwg_ARRAYITEMLOCATOR *)(argp2);
    if (arg1) (arg1)->items = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_items_get) {
  {
    struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ARRAYITEMLOCATOR *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_items_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCARRAYMODIFYACTIONBODY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_items_get" "', argument " "1"" of type '" "struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *)(argp1);
    result = (Dwg_ARRAYITEMLOCATOR *) ((arg1)->items);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ARRAYITEMLOCATOR, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY) {
  {
    int argvi = 0;
    struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY();");
    }
    result = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *)calloc(1, sizeof(struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCARRAYMODIFYACTIONBODY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY) {
  {
    struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ASSOCARRAYMODIFYACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY" "', argument " "1"" of type '" "struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *""'"); 
    }
    arg1 = (struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EvalExpr_parentid_set) {
  {
    struct _dwg_EvalExpr *arg1 = (struct _dwg_EvalExpr *) 0 ;
    BITCODE_BLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_EvalExpr_parentid_set(self,parentid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EvalExpr_parentid_set" "', argument " "1"" of type '" "struct _dwg_EvalExpr *""'"); 
    }
    arg1 = (struct _dwg_EvalExpr *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_EvalExpr_parentid_set" "', argument " "2"" of type '" "BITCODE_BLd""'");
    } 
    arg2 = (BITCODE_BLd)(val2);
    if (arg1) (arg1)->parentid = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EvalExpr_parentid_get) {
  {
    struct _dwg_EvalExpr *arg1 = (struct _dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_EvalExpr_parentid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EvalExpr_parentid_get" "', argument " "1"" of type '" "struct _dwg_EvalExpr *""'"); 
    }
    arg1 = (struct _dwg_EvalExpr *)(argp1);
    result = (BITCODE_BLd) ((arg1)->parentid);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EvalExpr_major_set) {
  {
    struct _dwg_EvalExpr *arg1 = (struct _dwg_EvalExpr *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_EvalExpr_major_set(self,major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EvalExpr_major_set" "', argument " "1"" of type '" "struct _dwg_EvalExpr *""'"); 
    }
    arg1 = (struct _dwg_EvalExpr *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_EvalExpr_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EvalExpr_major_get) {
  {
    struct _dwg_EvalExpr *arg1 = (struct _dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_EvalExpr_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EvalExpr_major_get" "', argument " "1"" of type '" "struct _dwg_EvalExpr *""'"); 
    }
    arg1 = (struct _dwg_EvalExpr *)(argp1);
    result = (BITCODE_BL) ((arg1)->major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EvalExpr_minor_set) {
  {
    struct _dwg_EvalExpr *arg1 = (struct _dwg_EvalExpr *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_EvalExpr_minor_set(self,minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EvalExpr_minor_set" "', argument " "1"" of type '" "struct _dwg_EvalExpr *""'"); 
    }
    arg1 = (struct _dwg_EvalExpr *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_EvalExpr_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EvalExpr_minor_get) {
  {
    struct _dwg_EvalExpr *arg1 = (struct _dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_EvalExpr_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EvalExpr_minor_get" "', argument " "1"" of type '" "struct _dwg_EvalExpr *""'"); 
    }
    arg1 = (struct _dwg_EvalExpr *)(argp1);
    result = (BITCODE_BL) ((arg1)->minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EvalExpr_value_code_set) {
  {
    struct _dwg_EvalExpr *arg1 = (struct _dwg_EvalExpr *) 0 ;
    BITCODE_BSd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_EvalExpr_value_code_set(self,value_code);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EvalExpr_value_code_set" "', argument " "1"" of type '" "struct _dwg_EvalExpr *""'"); 
    }
    arg1 = (struct _dwg_EvalExpr *)(argp1);
    ecode2 = SWIG_AsVal_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_EvalExpr_value_code_set" "', argument " "2"" of type '" "BITCODE_BSd""'");
    } 
    arg2 = (BITCODE_BSd)(val2);
    if (arg1) (arg1)->value_code = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EvalExpr_value_code_get) {
  {
    struct _dwg_EvalExpr *arg1 = (struct _dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BSd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_EvalExpr_value_code_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EvalExpr_value_code_get" "', argument " "1"" of type '" "struct _dwg_EvalExpr *""'"); 
    }
    arg1 = (struct _dwg_EvalExpr *)(argp1);
    result = (BITCODE_BSd) ((arg1)->value_code);
    ST(argvi) = SWIG_From_short  SWIG_PERL_CALL_ARGS_1((short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EvalExpr_value_get) {
  {
    struct _dwg_EvalExpr *arg1 = (struct _dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    _dwg_EvalExpr_value *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_EvalExpr_value_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EvalExpr_value_get" "', argument " "1"" of type '" "struct _dwg_EvalExpr *""'"); 
    }
    arg1 = (struct _dwg_EvalExpr *)(argp1);
    result = (_dwg_EvalExpr_value *)& ((arg1)->value);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr_value, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EvalExpr_nodeid_set) {
  {
    struct _dwg_EvalExpr *arg1 = (struct _dwg_EvalExpr *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_EvalExpr_nodeid_set(self,nodeid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EvalExpr_nodeid_set" "', argument " "1"" of type '" "struct _dwg_EvalExpr *""'"); 
    }
    arg1 = (struct _dwg_EvalExpr *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_EvalExpr_nodeid_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->nodeid = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_EvalExpr_nodeid_get) {
  {
    struct _dwg_EvalExpr *arg1 = (struct _dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_EvalExpr_nodeid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_EvalExpr_nodeid_get" "', argument " "1"" of type '" "struct _dwg_EvalExpr *""'"); 
    }
    arg1 = (struct _dwg_EvalExpr *)(argp1);
    result = (BITCODE_BL) ((arg1)->nodeid);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_EvalExpr) {
  {
    int argvi = 0;
    struct _dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_EvalExpr();");
    }
    result = (struct _dwg_EvalExpr *)calloc(1, sizeof(struct _dwg_EvalExpr));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_EvalExpr) {
  {
    struct _dwg_EvalExpr *arg1 = (struct _dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_EvalExpr(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalExpr, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_EvalExpr" "', argument " "1"" of type '" "struct _dwg_EvalExpr *""'"); 
    }
    arg1 = (struct _dwg_EvalExpr *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_EvalExpr_value_num40_set) {
  {
    _dwg_EvalExpr_value *arg1 = (_dwg_EvalExpr_value *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_EvalExpr_value_num40_set(self,num40);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalExpr_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_EvalExpr_value_num40_set" "', argument " "1"" of type '" "_dwg_EvalExpr_value *""'"); 
    }
    arg1 = (_dwg_EvalExpr_value *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "_dwg_EvalExpr_value_num40_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->num40 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_EvalExpr_value_num40_get) {
  {
    _dwg_EvalExpr_value *arg1 = (_dwg_EvalExpr_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_EvalExpr_value_num40_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalExpr_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_EvalExpr_value_num40_get" "', argument " "1"" of type '" "_dwg_EvalExpr_value *""'"); 
    }
    arg1 = (_dwg_EvalExpr_value *)(argp1);
    result = (BITCODE_BD) ((arg1)->num40);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_EvalExpr_value_pt2d_set) {
  {
    _dwg_EvalExpr_value *arg1 = (_dwg_EvalExpr_value *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_EvalExpr_value_pt2d_set(self,pt2d);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalExpr_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_EvalExpr_value_pt2d_set" "', argument " "1"" of type '" "_dwg_EvalExpr_value *""'"); 
    }
    arg1 = (_dwg_EvalExpr_value *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_EvalExpr_value_pt2d_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->pt2d = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_EvalExpr_value_pt2d_get) {
  {
    _dwg_EvalExpr_value *arg1 = (_dwg_EvalExpr_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_EvalExpr_value_pt2d_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalExpr_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_EvalExpr_value_pt2d_get" "', argument " "1"" of type '" "_dwg_EvalExpr_value *""'"); 
    }
    arg1 = (_dwg_EvalExpr_value *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->pt2d);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_EvalExpr_value_pt3d_set) {
  {
    _dwg_EvalExpr_value *arg1 = (_dwg_EvalExpr_value *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_EvalExpr_value_pt3d_set(self,pt3d);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalExpr_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_EvalExpr_value_pt3d_set" "', argument " "1"" of type '" "_dwg_EvalExpr_value *""'"); 
    }
    arg1 = (_dwg_EvalExpr_value *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_EvalExpr_value_pt3d_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->pt3d = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_EvalExpr_value_pt3d_get) {
  {
    _dwg_EvalExpr_value *arg1 = (_dwg_EvalExpr_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_EvalExpr_value_pt3d_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalExpr_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_EvalExpr_value_pt3d_get" "', argument " "1"" of type '" "_dwg_EvalExpr_value *""'"); 
    }
    arg1 = (_dwg_EvalExpr_value *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->pt3d);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_EvalExpr_value_text1_set) {
  {
    _dwg_EvalExpr_value *arg1 = (_dwg_EvalExpr_value *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_EvalExpr_value_text1_set(self,text1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalExpr_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_EvalExpr_value_text1_set" "', argument " "1"" of type '" "_dwg_EvalExpr_value *""'"); 
    }
    arg1 = (_dwg_EvalExpr_value *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_EvalExpr_value_text1_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->text1);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->text1 = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->text1 = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_EvalExpr_value_text1_get) {
  {
    _dwg_EvalExpr_value *arg1 = (_dwg_EvalExpr_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_EvalExpr_value_text1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalExpr_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_EvalExpr_value_text1_get" "', argument " "1"" of type '" "_dwg_EvalExpr_value *""'"); 
    }
    arg1 = (_dwg_EvalExpr_value *)(argp1);
    result = (BITCODE_TV) ((arg1)->text1);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_EvalExpr_value_long90_set) {
  {
    _dwg_EvalExpr_value *arg1 = (_dwg_EvalExpr_value *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_EvalExpr_value_long90_set(self,long90);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalExpr_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_EvalExpr_value_long90_set" "', argument " "1"" of type '" "_dwg_EvalExpr_value *""'"); 
    }
    arg1 = (_dwg_EvalExpr_value *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "_dwg_EvalExpr_value_long90_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->long90 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_EvalExpr_value_long90_get) {
  {
    _dwg_EvalExpr_value *arg1 = (_dwg_EvalExpr_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_EvalExpr_value_long90_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalExpr_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_EvalExpr_value_long90_get" "', argument " "1"" of type '" "_dwg_EvalExpr_value *""'"); 
    }
    arg1 = (_dwg_EvalExpr_value *)(argp1);
    result = (BITCODE_BL) ((arg1)->long90);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_EvalExpr_value_handle91_set) {
  {
    _dwg_EvalExpr_value *arg1 = (_dwg_EvalExpr_value *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_EvalExpr_value_handle91_set(self,handle91);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalExpr_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_EvalExpr_value_handle91_set" "', argument " "1"" of type '" "_dwg_EvalExpr_value *""'"); 
    }
    arg1 = (_dwg_EvalExpr_value *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_EvalExpr_value_handle91_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->handle91 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_EvalExpr_value_handle91_get) {
  {
    _dwg_EvalExpr_value *arg1 = (_dwg_EvalExpr_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_EvalExpr_value_handle91_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalExpr_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_EvalExpr_value_handle91_get" "', argument " "1"" of type '" "_dwg_EvalExpr_value *""'"); 
    }
    arg1 = (_dwg_EvalExpr_value *)(argp1);
    result = (BITCODE_H) ((arg1)->handle91);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_EvalExpr_value_short70_set) {
  {
    _dwg_EvalExpr_value *arg1 = (_dwg_EvalExpr_value *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_EvalExpr_value_short70_set(self,short70);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalExpr_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_EvalExpr_value_short70_set" "', argument " "1"" of type '" "_dwg_EvalExpr_value *""'"); 
    }
    arg1 = (_dwg_EvalExpr_value *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "_dwg_EvalExpr_value_short70_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->short70 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_EvalExpr_value_short70_get) {
  {
    _dwg_EvalExpr_value *arg1 = (_dwg_EvalExpr_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_EvalExpr_value_short70_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalExpr_value, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_EvalExpr_value_short70_get" "', argument " "1"" of type '" "_dwg_EvalExpr_value *""'"); 
    }
    arg1 = (_dwg_EvalExpr_value *)(argp1);
    result = (BITCODE_BS) ((arg1)->short70);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new__dwg_EvalExpr_value) {
  {
    int argvi = 0;
    _dwg_EvalExpr_value *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new__dwg_EvalExpr_value();");
    }
    result = (_dwg_EvalExpr_value *)calloc(1, sizeof(_dwg_EvalExpr_value));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr_value, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete__dwg_EvalExpr_value) {
  {
    _dwg_EvalExpr_value *arg1 = (_dwg_EvalExpr_value *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete__dwg_EvalExpr_value(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_EvalExpr_value, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete__dwg_EvalExpr_value" "', argument " "1"" of type '" "_dwg_EvalExpr_value *""'"); 
    }
    arg1 = (_dwg_EvalExpr_value *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACSH_SubentMaterial_major_set) {
  {
    struct _dwg_ACSH_SubentMaterial *arg1 = (struct _dwg_ACSH_SubentMaterial *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ACSH_SubentMaterial_major_set(self,major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACSH_SubentMaterial, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACSH_SubentMaterial_major_set" "', argument " "1"" of type '" "struct _dwg_ACSH_SubentMaterial *""'"); 
    }
    arg1 = (struct _dwg_ACSH_SubentMaterial *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ACSH_SubentMaterial_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACSH_SubentMaterial_major_get) {
  {
    struct _dwg_ACSH_SubentMaterial *arg1 = (struct _dwg_ACSH_SubentMaterial *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ACSH_SubentMaterial_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACSH_SubentMaterial, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACSH_SubentMaterial_major_get" "', argument " "1"" of type '" "struct _dwg_ACSH_SubentMaterial *""'"); 
    }
    arg1 = (struct _dwg_ACSH_SubentMaterial *)(argp1);
    result = (BITCODE_BL) ((arg1)->major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACSH_SubentMaterial_minor_set) {
  {
    struct _dwg_ACSH_SubentMaterial *arg1 = (struct _dwg_ACSH_SubentMaterial *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ACSH_SubentMaterial_minor_set(self,minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACSH_SubentMaterial, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACSH_SubentMaterial_minor_set" "', argument " "1"" of type '" "struct _dwg_ACSH_SubentMaterial *""'"); 
    }
    arg1 = (struct _dwg_ACSH_SubentMaterial *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ACSH_SubentMaterial_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACSH_SubentMaterial_minor_get) {
  {
    struct _dwg_ACSH_SubentMaterial *arg1 = (struct _dwg_ACSH_SubentMaterial *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ACSH_SubentMaterial_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACSH_SubentMaterial, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACSH_SubentMaterial_minor_get" "', argument " "1"" of type '" "struct _dwg_ACSH_SubentMaterial *""'"); 
    }
    arg1 = (struct _dwg_ACSH_SubentMaterial *)(argp1);
    result = (BITCODE_BL) ((arg1)->minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACSH_SubentMaterial_reflectance_set) {
  {
    struct _dwg_ACSH_SubentMaterial *arg1 = (struct _dwg_ACSH_SubentMaterial *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ACSH_SubentMaterial_reflectance_set(self,reflectance);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACSH_SubentMaterial, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACSH_SubentMaterial_reflectance_set" "', argument " "1"" of type '" "struct _dwg_ACSH_SubentMaterial *""'"); 
    }
    arg1 = (struct _dwg_ACSH_SubentMaterial *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ACSH_SubentMaterial_reflectance_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->reflectance = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACSH_SubentMaterial_reflectance_get) {
  {
    struct _dwg_ACSH_SubentMaterial *arg1 = (struct _dwg_ACSH_SubentMaterial *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ACSH_SubentMaterial_reflectance_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACSH_SubentMaterial, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACSH_SubentMaterial_reflectance_get" "', argument " "1"" of type '" "struct _dwg_ACSH_SubentMaterial *""'"); 
    }
    arg1 = (struct _dwg_ACSH_SubentMaterial *)(argp1);
    result = (BITCODE_BL) ((arg1)->reflectance);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACSH_SubentMaterial_displacement_set) {
  {
    struct _dwg_ACSH_SubentMaterial *arg1 = (struct _dwg_ACSH_SubentMaterial *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ACSH_SubentMaterial_displacement_set(self,displacement);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACSH_SubentMaterial, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACSH_SubentMaterial_displacement_set" "', argument " "1"" of type '" "struct _dwg_ACSH_SubentMaterial *""'"); 
    }
    arg1 = (struct _dwg_ACSH_SubentMaterial *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ACSH_SubentMaterial_displacement_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->displacement = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACSH_SubentMaterial_displacement_get) {
  {
    struct _dwg_ACSH_SubentMaterial *arg1 = (struct _dwg_ACSH_SubentMaterial *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ACSH_SubentMaterial_displacement_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACSH_SubentMaterial, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACSH_SubentMaterial_displacement_get" "', argument " "1"" of type '" "struct _dwg_ACSH_SubentMaterial *""'"); 
    }
    arg1 = (struct _dwg_ACSH_SubentMaterial *)(argp1);
    result = (BITCODE_BL) ((arg1)->displacement);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_ACSH_SubentMaterial) {
  {
    int argvi = 0;
    struct _dwg_ACSH_SubentMaterial *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_ACSH_SubentMaterial();");
    }
    result = (struct _dwg_ACSH_SubentMaterial *)calloc(1, sizeof(struct _dwg_ACSH_SubentMaterial));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ACSH_SubentMaterial, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_ACSH_SubentMaterial) {
  {
    struct _dwg_ACSH_SubentMaterial *arg1 = (struct _dwg_ACSH_SubentMaterial *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_ACSH_SubentMaterial(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACSH_SubentMaterial, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_ACSH_SubentMaterial" "', argument " "1"" of type '" "struct _dwg_ACSH_SubentMaterial *""'"); 
    }
    arg1 = (struct _dwg_ACSH_SubentMaterial *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACSH_SubentColor_major_set) {
  {
    struct _dwg_ACSH_SubentColor *arg1 = (struct _dwg_ACSH_SubentColor *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ACSH_SubentColor_major_set(self,major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACSH_SubentColor, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACSH_SubentColor_major_set" "', argument " "1"" of type '" "struct _dwg_ACSH_SubentColor *""'"); 
    }
    arg1 = (struct _dwg_ACSH_SubentColor *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ACSH_SubentColor_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACSH_SubentColor_major_get) {
  {
    struct _dwg_ACSH_SubentColor *arg1 = (struct _dwg_ACSH_SubentColor *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ACSH_SubentColor_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACSH_SubentColor, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACSH_SubentColor_major_get" "', argument " "1"" of type '" "struct _dwg_ACSH_SubentColor *""'"); 
    }
    arg1 = (struct _dwg_ACSH_SubentColor *)(argp1);
    result = (BITCODE_BL) ((arg1)->major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACSH_SubentColor_minor_set) {
  {
    struct _dwg_ACSH_SubentColor *arg1 = (struct _dwg_ACSH_SubentColor *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ACSH_SubentColor_minor_set(self,minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACSH_SubentColor, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACSH_SubentColor_minor_set" "', argument " "1"" of type '" "struct _dwg_ACSH_SubentColor *""'"); 
    }
    arg1 = (struct _dwg_ACSH_SubentColor *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ACSH_SubentColor_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACSH_SubentColor_minor_get) {
  {
    struct _dwg_ACSH_SubentColor *arg1 = (struct _dwg_ACSH_SubentColor *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ACSH_SubentColor_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACSH_SubentColor, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACSH_SubentColor_minor_get" "', argument " "1"" of type '" "struct _dwg_ACSH_SubentColor *""'"); 
    }
    arg1 = (struct _dwg_ACSH_SubentColor *)(argp1);
    result = (BITCODE_BL) ((arg1)->minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACSH_SubentColor_transparency_set) {
  {
    struct _dwg_ACSH_SubentColor *arg1 = (struct _dwg_ACSH_SubentColor *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ACSH_SubentColor_transparency_set(self,transparency);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACSH_SubentColor, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACSH_SubentColor_transparency_set" "', argument " "1"" of type '" "struct _dwg_ACSH_SubentColor *""'"); 
    }
    arg1 = (struct _dwg_ACSH_SubentColor *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ACSH_SubentColor_transparency_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->transparency = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACSH_SubentColor_transparency_get) {
  {
    struct _dwg_ACSH_SubentColor *arg1 = (struct _dwg_ACSH_SubentColor *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ACSH_SubentColor_transparency_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACSH_SubentColor, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACSH_SubentColor_transparency_get" "', argument " "1"" of type '" "struct _dwg_ACSH_SubentColor *""'"); 
    }
    arg1 = (struct _dwg_ACSH_SubentColor *)(argp1);
    result = (BITCODE_BL) ((arg1)->transparency);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACSH_SubentColor_bl93_set) {
  {
    struct _dwg_ACSH_SubentColor *arg1 = (struct _dwg_ACSH_SubentColor *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ACSH_SubentColor_bl93_set(self,bl93);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACSH_SubentColor, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACSH_SubentColor_bl93_set" "', argument " "1"" of type '" "struct _dwg_ACSH_SubentColor *""'"); 
    }
    arg1 = (struct _dwg_ACSH_SubentColor *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ACSH_SubentColor_bl93_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bl93 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACSH_SubentColor_bl93_get) {
  {
    struct _dwg_ACSH_SubentColor *arg1 = (struct _dwg_ACSH_SubentColor *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ACSH_SubentColor_bl93_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACSH_SubentColor, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACSH_SubentColor_bl93_get" "', argument " "1"" of type '" "struct _dwg_ACSH_SubentColor *""'"); 
    }
    arg1 = (struct _dwg_ACSH_SubentColor *)(argp1);
    result = (BITCODE_BL) ((arg1)->bl93);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACSH_SubentColor_is_face_variable_set) {
  {
    struct _dwg_ACSH_SubentColor *arg1 = (struct _dwg_ACSH_SubentColor *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ACSH_SubentColor_is_face_variable_set(self,is_face_variable);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACSH_SubentColor, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACSH_SubentColor_is_face_variable_set" "', argument " "1"" of type '" "struct _dwg_ACSH_SubentColor *""'"); 
    }
    arg1 = (struct _dwg_ACSH_SubentColor *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ACSH_SubentColor_is_face_variable_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_face_variable = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACSH_SubentColor_is_face_variable_get) {
  {
    struct _dwg_ACSH_SubentColor *arg1 = (struct _dwg_ACSH_SubentColor *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ACSH_SubentColor_is_face_variable_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACSH_SubentColor, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACSH_SubentColor_is_face_variable_get" "', argument " "1"" of type '" "struct _dwg_ACSH_SubentColor *""'"); 
    }
    arg1 = (struct _dwg_ACSH_SubentColor *)(argp1);
    result = (BITCODE_B) ((arg1)->is_face_variable);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_ACSH_SubentColor) {
  {
    int argvi = 0;
    struct _dwg_ACSH_SubentColor *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_ACSH_SubentColor();");
    }
    result = (struct _dwg_ACSH_SubentColor *)calloc(1, sizeof(struct _dwg_ACSH_SubentColor));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ACSH_SubentColor, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_ACSH_SubentColor) {
  {
    struct _dwg_ACSH_SubentColor *arg1 = (struct _dwg_ACSH_SubentColor *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_ACSH_SubentColor(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACSH_SubentColor, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_ACSH_SubentColor" "', argument " "1"" of type '" "struct _dwg_ACSH_SubentColor *""'"); 
    }
    arg1 = (struct _dwg_ACSH_SubentColor *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACSH_HistoryNode_major_set) {
  {
    struct _dwg_ACSH_HistoryNode *arg1 = (struct _dwg_ACSH_HistoryNode *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ACSH_HistoryNode_major_set(self,major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACSH_HistoryNode_major_set" "', argument " "1"" of type '" "struct _dwg_ACSH_HistoryNode *""'"); 
    }
    arg1 = (struct _dwg_ACSH_HistoryNode *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ACSH_HistoryNode_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACSH_HistoryNode_major_get) {
  {
    struct _dwg_ACSH_HistoryNode *arg1 = (struct _dwg_ACSH_HistoryNode *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ACSH_HistoryNode_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACSH_HistoryNode_major_get" "', argument " "1"" of type '" "struct _dwg_ACSH_HistoryNode *""'"); 
    }
    arg1 = (struct _dwg_ACSH_HistoryNode *)(argp1);
    result = (BITCODE_BL) ((arg1)->major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACSH_HistoryNode_minor_set) {
  {
    struct _dwg_ACSH_HistoryNode *arg1 = (struct _dwg_ACSH_HistoryNode *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ACSH_HistoryNode_minor_set(self,minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACSH_HistoryNode_minor_set" "', argument " "1"" of type '" "struct _dwg_ACSH_HistoryNode *""'"); 
    }
    arg1 = (struct _dwg_ACSH_HistoryNode *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ACSH_HistoryNode_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACSH_HistoryNode_minor_get) {
  {
    struct _dwg_ACSH_HistoryNode *arg1 = (struct _dwg_ACSH_HistoryNode *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ACSH_HistoryNode_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACSH_HistoryNode_minor_get" "', argument " "1"" of type '" "struct _dwg_ACSH_HistoryNode *""'"); 
    }
    arg1 = (struct _dwg_ACSH_HistoryNode *)(argp1);
    result = (BITCODE_BL) ((arg1)->minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACSH_HistoryNode_trans_set) {
  {
    struct _dwg_ACSH_HistoryNode *arg1 = (struct _dwg_ACSH_HistoryNode *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ACSH_HistoryNode_trans_set(self,trans);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACSH_HistoryNode_trans_set" "', argument " "1"" of type '" "struct _dwg_ACSH_HistoryNode *""'"); 
    }
    arg1 = (struct _dwg_ACSH_HistoryNode *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_ACSH_HistoryNode_trans_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->trans = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACSH_HistoryNode_trans_get) {
  {
    struct _dwg_ACSH_HistoryNode *arg1 = (struct _dwg_ACSH_HistoryNode *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ACSH_HistoryNode_trans_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACSH_HistoryNode_trans_get" "', argument " "1"" of type '" "struct _dwg_ACSH_HistoryNode *""'"); 
    }
    arg1 = (struct _dwg_ACSH_HistoryNode *)(argp1);
    result = (BITCODE_BD *) ((arg1)->trans);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACSH_HistoryNode_color_set) {
  {
    struct _dwg_ACSH_HistoryNode *arg1 = (struct _dwg_ACSH_HistoryNode *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ACSH_HistoryNode_color_set(self,color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACSH_HistoryNode_color_set" "', argument " "1"" of type '" "struct _dwg_ACSH_HistoryNode *""'"); 
    }
    arg1 = (struct _dwg_ACSH_HistoryNode *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_ACSH_HistoryNode_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACSH_HistoryNode_color_get) {
  {
    struct _dwg_ACSH_HistoryNode *arg1 = (struct _dwg_ACSH_HistoryNode *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ACSH_HistoryNode_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACSH_HistoryNode_color_get" "', argument " "1"" of type '" "struct _dwg_ACSH_HistoryNode *""'"); 
    }
    arg1 = (struct _dwg_ACSH_HistoryNode *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACSH_HistoryNode_step_id_set) {
  {
    struct _dwg_ACSH_HistoryNode *arg1 = (struct _dwg_ACSH_HistoryNode *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ACSH_HistoryNode_step_id_set(self,step_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACSH_HistoryNode_step_id_set" "', argument " "1"" of type '" "struct _dwg_ACSH_HistoryNode *""'"); 
    }
    arg1 = (struct _dwg_ACSH_HistoryNode *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ACSH_HistoryNode_step_id_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->step_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACSH_HistoryNode_step_id_get) {
  {
    struct _dwg_ACSH_HistoryNode *arg1 = (struct _dwg_ACSH_HistoryNode *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ACSH_HistoryNode_step_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACSH_HistoryNode_step_id_get" "', argument " "1"" of type '" "struct _dwg_ACSH_HistoryNode *""'"); 
    }
    arg1 = (struct _dwg_ACSH_HistoryNode *)(argp1);
    result = (BITCODE_BL) ((arg1)->step_id);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACSH_HistoryNode_material_set) {
  {
    struct _dwg_ACSH_HistoryNode *arg1 = (struct _dwg_ACSH_HistoryNode *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ACSH_HistoryNode_material_set(self,material);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACSH_HistoryNode_material_set" "', argument " "1"" of type '" "struct _dwg_ACSH_HistoryNode *""'"); 
    }
    arg1 = (struct _dwg_ACSH_HistoryNode *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_ACSH_HistoryNode_material_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->material = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ACSH_HistoryNode_material_get) {
  {
    struct _dwg_ACSH_HistoryNode *arg1 = (struct _dwg_ACSH_HistoryNode *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ACSH_HistoryNode_material_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ACSH_HistoryNode_material_get" "', argument " "1"" of type '" "struct _dwg_ACSH_HistoryNode *""'"); 
    }
    arg1 = (struct _dwg_ACSH_HistoryNode *)(argp1);
    result = (BITCODE_H) ((arg1)->material);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_ACSH_HistoryNode) {
  {
    int argvi = 0;
    struct _dwg_ACSH_HistoryNode *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_ACSH_HistoryNode();");
    }
    result = (struct _dwg_ACSH_HistoryNode *)calloc(1, sizeof(struct _dwg_ACSH_HistoryNode));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ACSH_HistoryNode, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_ACSH_HistoryNode) {
  {
    struct _dwg_ACSH_HistoryNode *arg1 = (struct _dwg_ACSH_HistoryNode *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_ACSH_HistoryNode(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ACSH_HistoryNode, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_ACSH_HistoryNode" "', argument " "1"" of type '" "struct _dwg_ACSH_HistoryNode *""'"); 
    }
    arg1 = (struct _dwg_ACSH_HistoryNode *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_HISTORY_CLASS_parent_set) {
  {
    struct _dwg_object_ACSH_HISTORY_CLASS *arg1 = (struct _dwg_object_ACSH_HISTORY_CLASS *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_HISTORY_CLASS_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_HISTORY_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_HISTORY_CLASS_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_HISTORY_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_HISTORY_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_HISTORY_CLASS_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_HISTORY_CLASS_parent_get) {
  {
    struct _dwg_object_ACSH_HISTORY_CLASS *arg1 = (struct _dwg_object_ACSH_HISTORY_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_HISTORY_CLASS_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_HISTORY_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_HISTORY_CLASS_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_HISTORY_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_HISTORY_CLASS *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_HISTORY_CLASS_major_set) {
  {
    struct _dwg_object_ACSH_HISTORY_CLASS *arg1 = (struct _dwg_object_ACSH_HISTORY_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_HISTORY_CLASS_major_set(self,major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_HISTORY_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_HISTORY_CLASS_major_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_HISTORY_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_HISTORY_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_HISTORY_CLASS_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_HISTORY_CLASS_major_get) {
  {
    struct _dwg_object_ACSH_HISTORY_CLASS *arg1 = (struct _dwg_object_ACSH_HISTORY_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_HISTORY_CLASS_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_HISTORY_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_HISTORY_CLASS_major_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_HISTORY_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_HISTORY_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_HISTORY_CLASS_minor_set) {
  {
    struct _dwg_object_ACSH_HISTORY_CLASS *arg1 = (struct _dwg_object_ACSH_HISTORY_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_HISTORY_CLASS_minor_set(self,minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_HISTORY_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_HISTORY_CLASS_minor_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_HISTORY_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_HISTORY_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_HISTORY_CLASS_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_HISTORY_CLASS_minor_get) {
  {
    struct _dwg_object_ACSH_HISTORY_CLASS *arg1 = (struct _dwg_object_ACSH_HISTORY_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_HISTORY_CLASS_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_HISTORY_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_HISTORY_CLASS_minor_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_HISTORY_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_HISTORY_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_HISTORY_CLASS_owner_set) {
  {
    struct _dwg_object_ACSH_HISTORY_CLASS *arg1 = (struct _dwg_object_ACSH_HISTORY_CLASS *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_HISTORY_CLASS_owner_set(self,owner);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_HISTORY_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_HISTORY_CLASS_owner_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_HISTORY_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_HISTORY_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_HISTORY_CLASS_owner_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->owner = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_HISTORY_CLASS_owner_get) {
  {
    struct _dwg_object_ACSH_HISTORY_CLASS *arg1 = (struct _dwg_object_ACSH_HISTORY_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_HISTORY_CLASS_owner_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_HISTORY_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_HISTORY_CLASS_owner_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_HISTORY_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_HISTORY_CLASS *)(argp1);
    result = (BITCODE_H) ((arg1)->owner);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_HISTORY_CLASS_h_nodeid_set) {
  {
    struct _dwg_object_ACSH_HISTORY_CLASS *arg1 = (struct _dwg_object_ACSH_HISTORY_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_HISTORY_CLASS_h_nodeid_set(self,h_nodeid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_HISTORY_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_HISTORY_CLASS_h_nodeid_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_HISTORY_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_HISTORY_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_HISTORY_CLASS_h_nodeid_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->h_nodeid = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_HISTORY_CLASS_h_nodeid_get) {
  {
    struct _dwg_object_ACSH_HISTORY_CLASS *arg1 = (struct _dwg_object_ACSH_HISTORY_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_HISTORY_CLASS_h_nodeid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_HISTORY_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_HISTORY_CLASS_h_nodeid_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_HISTORY_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_HISTORY_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->h_nodeid);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_HISTORY_CLASS_show_history_set) {
  {
    struct _dwg_object_ACSH_HISTORY_CLASS *arg1 = (struct _dwg_object_ACSH_HISTORY_CLASS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_HISTORY_CLASS_show_history_set(self,show_history);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_HISTORY_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_HISTORY_CLASS_show_history_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_HISTORY_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_HISTORY_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_HISTORY_CLASS_show_history_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->show_history = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_HISTORY_CLASS_show_history_get) {
  {
    struct _dwg_object_ACSH_HISTORY_CLASS *arg1 = (struct _dwg_object_ACSH_HISTORY_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_HISTORY_CLASS_show_history_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_HISTORY_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_HISTORY_CLASS_show_history_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_HISTORY_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_HISTORY_CLASS *)(argp1);
    result = (BITCODE_B) ((arg1)->show_history);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_HISTORY_CLASS_record_history_set) {
  {
    struct _dwg_object_ACSH_HISTORY_CLASS *arg1 = (struct _dwg_object_ACSH_HISTORY_CLASS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_HISTORY_CLASS_record_history_set(self,record_history);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_HISTORY_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_HISTORY_CLASS_record_history_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_HISTORY_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_HISTORY_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_HISTORY_CLASS_record_history_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->record_history = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_HISTORY_CLASS_record_history_get) {
  {
    struct _dwg_object_ACSH_HISTORY_CLASS *arg1 = (struct _dwg_object_ACSH_HISTORY_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_HISTORY_CLASS_record_history_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_HISTORY_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_HISTORY_CLASS_record_history_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_HISTORY_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_HISTORY_CLASS *)(argp1);
    result = (BITCODE_B) ((arg1)->record_history);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ACSH_HISTORY_CLASS) {
  {
    int argvi = 0;
    struct _dwg_object_ACSH_HISTORY_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ACSH_HISTORY_CLASS();");
    }
    result = (struct _dwg_object_ACSH_HISTORY_CLASS *)calloc(1, sizeof(struct _dwg_object_ACSH_HISTORY_CLASS));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_HISTORY_CLASS, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ACSH_HISTORY_CLASS) {
  {
    struct _dwg_object_ACSH_HISTORY_CLASS *arg1 = (struct _dwg_object_ACSH_HISTORY_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ACSH_HISTORY_CLASS(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_HISTORY_CLASS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ACSH_HISTORY_CLASS" "', argument " "1"" of type '" "struct _dwg_object_ACSH_HISTORY_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_HISTORY_CLASS *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BOX_CLASS_parent_set) {
  {
    struct _dwg_object_ACSH_BOX_CLASS *arg1 = (struct _dwg_object_ACSH_BOX_CLASS *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BOX_CLASS_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BOX_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BOX_CLASS_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BOX_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BOX_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_BOX_CLASS_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BOX_CLASS_parent_get) {
  {
    struct _dwg_object_ACSH_BOX_CLASS *arg1 = (struct _dwg_object_ACSH_BOX_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BOX_CLASS_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BOX_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BOX_CLASS_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BOX_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BOX_CLASS *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BOX_CLASS_evalexpr_set) {
  {
    struct _dwg_object_ACSH_BOX_CLASS *arg1 = (struct _dwg_object_ACSH_BOX_CLASS *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BOX_CLASS_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BOX_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BOX_CLASS_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BOX_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BOX_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_BOX_CLASS_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BOX_CLASS_evalexpr_get) {
  {
    struct _dwg_object_ACSH_BOX_CLASS *arg1 = (struct _dwg_object_ACSH_BOX_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BOX_CLASS_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BOX_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BOX_CLASS_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BOX_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BOX_CLASS *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BOX_CLASS_history_node_set) {
  {
    struct _dwg_object_ACSH_BOX_CLASS *arg1 = (struct _dwg_object_ACSH_BOX_CLASS *) 0 ;
    Dwg_ACSH_HistoryNode *arg2 = (Dwg_ACSH_HistoryNode *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BOX_CLASS_history_node_set(self,history_node);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BOX_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BOX_CLASS_history_node_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BOX_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BOX_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_BOX_CLASS_history_node_set" "', argument " "2"" of type '" "Dwg_ACSH_HistoryNode *""'"); 
    }
    arg2 = (Dwg_ACSH_HistoryNode *)(argp2);
    if (arg1) (arg1)->history_node = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BOX_CLASS_history_node_get) {
  {
    struct _dwg_object_ACSH_BOX_CLASS *arg1 = (struct _dwg_object_ACSH_BOX_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ACSH_HistoryNode *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BOX_CLASS_history_node_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BOX_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BOX_CLASS_history_node_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BOX_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BOX_CLASS *)(argp1);
    result = (Dwg_ACSH_HistoryNode *)& ((arg1)->history_node);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BOX_CLASS_major_set) {
  {
    struct _dwg_object_ACSH_BOX_CLASS *arg1 = (struct _dwg_object_ACSH_BOX_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BOX_CLASS_major_set(self,major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BOX_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BOX_CLASS_major_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BOX_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BOX_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_BOX_CLASS_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BOX_CLASS_major_get) {
  {
    struct _dwg_object_ACSH_BOX_CLASS *arg1 = (struct _dwg_object_ACSH_BOX_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BOX_CLASS_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BOX_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BOX_CLASS_major_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BOX_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BOX_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BOX_CLASS_minor_set) {
  {
    struct _dwg_object_ACSH_BOX_CLASS *arg1 = (struct _dwg_object_ACSH_BOX_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BOX_CLASS_minor_set(self,minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BOX_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BOX_CLASS_minor_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BOX_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BOX_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_BOX_CLASS_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BOX_CLASS_minor_get) {
  {
    struct _dwg_object_ACSH_BOX_CLASS *arg1 = (struct _dwg_object_ACSH_BOX_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BOX_CLASS_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BOX_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BOX_CLASS_minor_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BOX_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BOX_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BOX_CLASS_length_set) {
  {
    struct _dwg_object_ACSH_BOX_CLASS *arg1 = (struct _dwg_object_ACSH_BOX_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BOX_CLASS_length_set(self,length);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BOX_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BOX_CLASS_length_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BOX_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BOX_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_BOX_CLASS_length_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->length = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BOX_CLASS_length_get) {
  {
    struct _dwg_object_ACSH_BOX_CLASS *arg1 = (struct _dwg_object_ACSH_BOX_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BOX_CLASS_length_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BOX_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BOX_CLASS_length_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BOX_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BOX_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->length);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BOX_CLASS_width_set) {
  {
    struct _dwg_object_ACSH_BOX_CLASS *arg1 = (struct _dwg_object_ACSH_BOX_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BOX_CLASS_width_set(self,width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BOX_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BOX_CLASS_width_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BOX_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BOX_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_BOX_CLASS_width_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BOX_CLASS_width_get) {
  {
    struct _dwg_object_ACSH_BOX_CLASS *arg1 = (struct _dwg_object_ACSH_BOX_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BOX_CLASS_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BOX_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BOX_CLASS_width_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BOX_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BOX_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->width);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BOX_CLASS_height_set) {
  {
    struct _dwg_object_ACSH_BOX_CLASS *arg1 = (struct _dwg_object_ACSH_BOX_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BOX_CLASS_height_set(self,height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BOX_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BOX_CLASS_height_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BOX_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BOX_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_BOX_CLASS_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BOX_CLASS_height_get) {
  {
    struct _dwg_object_ACSH_BOX_CLASS *arg1 = (struct _dwg_object_ACSH_BOX_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BOX_CLASS_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BOX_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BOX_CLASS_height_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BOX_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BOX_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ACSH_BOX_CLASS) {
  {
    int argvi = 0;
    struct _dwg_object_ACSH_BOX_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ACSH_BOX_CLASS();");
    }
    result = (struct _dwg_object_ACSH_BOX_CLASS *)calloc(1, sizeof(struct _dwg_object_ACSH_BOX_CLASS));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_BOX_CLASS, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ACSH_BOX_CLASS) {
  {
    struct _dwg_object_ACSH_BOX_CLASS *arg1 = (struct _dwg_object_ACSH_BOX_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ACSH_BOX_CLASS(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BOX_CLASS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ACSH_BOX_CLASS" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BOX_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BOX_CLASS *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_WEDGE_CLASS_parent_set) {
  {
    struct _dwg_object_ACSH_WEDGE_CLASS *arg1 = (struct _dwg_object_ACSH_WEDGE_CLASS *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_WEDGE_CLASS_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_WEDGE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_WEDGE_CLASS_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_WEDGE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_WEDGE_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_WEDGE_CLASS_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_WEDGE_CLASS_parent_get) {
  {
    struct _dwg_object_ACSH_WEDGE_CLASS *arg1 = (struct _dwg_object_ACSH_WEDGE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_WEDGE_CLASS_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_WEDGE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_WEDGE_CLASS_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_WEDGE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_WEDGE_CLASS *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_WEDGE_CLASS_evalexpr_set) {
  {
    struct _dwg_object_ACSH_WEDGE_CLASS *arg1 = (struct _dwg_object_ACSH_WEDGE_CLASS *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_WEDGE_CLASS_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_WEDGE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_WEDGE_CLASS_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_WEDGE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_WEDGE_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_WEDGE_CLASS_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_WEDGE_CLASS_evalexpr_get) {
  {
    struct _dwg_object_ACSH_WEDGE_CLASS *arg1 = (struct _dwg_object_ACSH_WEDGE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_WEDGE_CLASS_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_WEDGE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_WEDGE_CLASS_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_WEDGE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_WEDGE_CLASS *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_WEDGE_CLASS_history_node_set) {
  {
    struct _dwg_object_ACSH_WEDGE_CLASS *arg1 = (struct _dwg_object_ACSH_WEDGE_CLASS *) 0 ;
    Dwg_ACSH_HistoryNode *arg2 = (Dwg_ACSH_HistoryNode *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_WEDGE_CLASS_history_node_set(self,history_node);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_WEDGE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_WEDGE_CLASS_history_node_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_WEDGE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_WEDGE_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_WEDGE_CLASS_history_node_set" "', argument " "2"" of type '" "Dwg_ACSH_HistoryNode *""'"); 
    }
    arg2 = (Dwg_ACSH_HistoryNode *)(argp2);
    if (arg1) (arg1)->history_node = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_WEDGE_CLASS_history_node_get) {
  {
    struct _dwg_object_ACSH_WEDGE_CLASS *arg1 = (struct _dwg_object_ACSH_WEDGE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ACSH_HistoryNode *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_WEDGE_CLASS_history_node_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_WEDGE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_WEDGE_CLASS_history_node_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_WEDGE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_WEDGE_CLASS *)(argp1);
    result = (Dwg_ACSH_HistoryNode *)& ((arg1)->history_node);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_WEDGE_CLASS_major_set) {
  {
    struct _dwg_object_ACSH_WEDGE_CLASS *arg1 = (struct _dwg_object_ACSH_WEDGE_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_WEDGE_CLASS_major_set(self,major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_WEDGE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_WEDGE_CLASS_major_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_WEDGE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_WEDGE_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_WEDGE_CLASS_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_WEDGE_CLASS_major_get) {
  {
    struct _dwg_object_ACSH_WEDGE_CLASS *arg1 = (struct _dwg_object_ACSH_WEDGE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_WEDGE_CLASS_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_WEDGE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_WEDGE_CLASS_major_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_WEDGE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_WEDGE_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_WEDGE_CLASS_minor_set) {
  {
    struct _dwg_object_ACSH_WEDGE_CLASS *arg1 = (struct _dwg_object_ACSH_WEDGE_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_WEDGE_CLASS_minor_set(self,minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_WEDGE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_WEDGE_CLASS_minor_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_WEDGE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_WEDGE_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_WEDGE_CLASS_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_WEDGE_CLASS_minor_get) {
  {
    struct _dwg_object_ACSH_WEDGE_CLASS *arg1 = (struct _dwg_object_ACSH_WEDGE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_WEDGE_CLASS_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_WEDGE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_WEDGE_CLASS_minor_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_WEDGE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_WEDGE_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_WEDGE_CLASS_length_set) {
  {
    struct _dwg_object_ACSH_WEDGE_CLASS *arg1 = (struct _dwg_object_ACSH_WEDGE_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_WEDGE_CLASS_length_set(self,length);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_WEDGE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_WEDGE_CLASS_length_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_WEDGE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_WEDGE_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_WEDGE_CLASS_length_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->length = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_WEDGE_CLASS_length_get) {
  {
    struct _dwg_object_ACSH_WEDGE_CLASS *arg1 = (struct _dwg_object_ACSH_WEDGE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_WEDGE_CLASS_length_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_WEDGE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_WEDGE_CLASS_length_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_WEDGE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_WEDGE_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->length);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_WEDGE_CLASS_width_set) {
  {
    struct _dwg_object_ACSH_WEDGE_CLASS *arg1 = (struct _dwg_object_ACSH_WEDGE_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_WEDGE_CLASS_width_set(self,width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_WEDGE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_WEDGE_CLASS_width_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_WEDGE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_WEDGE_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_WEDGE_CLASS_width_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_WEDGE_CLASS_width_get) {
  {
    struct _dwg_object_ACSH_WEDGE_CLASS *arg1 = (struct _dwg_object_ACSH_WEDGE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_WEDGE_CLASS_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_WEDGE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_WEDGE_CLASS_width_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_WEDGE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_WEDGE_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->width);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_WEDGE_CLASS_height_set) {
  {
    struct _dwg_object_ACSH_WEDGE_CLASS *arg1 = (struct _dwg_object_ACSH_WEDGE_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_WEDGE_CLASS_height_set(self,height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_WEDGE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_WEDGE_CLASS_height_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_WEDGE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_WEDGE_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_WEDGE_CLASS_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_WEDGE_CLASS_height_get) {
  {
    struct _dwg_object_ACSH_WEDGE_CLASS *arg1 = (struct _dwg_object_ACSH_WEDGE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_WEDGE_CLASS_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_WEDGE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_WEDGE_CLASS_height_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_WEDGE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_WEDGE_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ACSH_WEDGE_CLASS) {
  {
    int argvi = 0;
    struct _dwg_object_ACSH_WEDGE_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ACSH_WEDGE_CLASS();");
    }
    result = (struct _dwg_object_ACSH_WEDGE_CLASS *)calloc(1, sizeof(struct _dwg_object_ACSH_WEDGE_CLASS));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_WEDGE_CLASS, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ACSH_WEDGE_CLASS) {
  {
    struct _dwg_object_ACSH_WEDGE_CLASS *arg1 = (struct _dwg_object_ACSH_WEDGE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ACSH_WEDGE_CLASS(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_WEDGE_CLASS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ACSH_WEDGE_CLASS" "', argument " "1"" of type '" "struct _dwg_object_ACSH_WEDGE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_WEDGE_CLASS *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BOOLEAN_CLASS_parent_set) {
  {
    struct _dwg_object_ACSH_BOOLEAN_CLASS *arg1 = (struct _dwg_object_ACSH_BOOLEAN_CLASS *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BOOLEAN_CLASS_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BOOLEAN_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BOOLEAN_CLASS_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BOOLEAN_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BOOLEAN_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_BOOLEAN_CLASS_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BOOLEAN_CLASS_parent_get) {
  {
    struct _dwg_object_ACSH_BOOLEAN_CLASS *arg1 = (struct _dwg_object_ACSH_BOOLEAN_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BOOLEAN_CLASS_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BOOLEAN_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BOOLEAN_CLASS_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BOOLEAN_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BOOLEAN_CLASS *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BOOLEAN_CLASS_evalexpr_set) {
  {
    struct _dwg_object_ACSH_BOOLEAN_CLASS *arg1 = (struct _dwg_object_ACSH_BOOLEAN_CLASS *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BOOLEAN_CLASS_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BOOLEAN_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BOOLEAN_CLASS_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BOOLEAN_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BOOLEAN_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_BOOLEAN_CLASS_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BOOLEAN_CLASS_evalexpr_get) {
  {
    struct _dwg_object_ACSH_BOOLEAN_CLASS *arg1 = (struct _dwg_object_ACSH_BOOLEAN_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BOOLEAN_CLASS_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BOOLEAN_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BOOLEAN_CLASS_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BOOLEAN_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BOOLEAN_CLASS *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BOOLEAN_CLASS_history_node_set) {
  {
    struct _dwg_object_ACSH_BOOLEAN_CLASS *arg1 = (struct _dwg_object_ACSH_BOOLEAN_CLASS *) 0 ;
    Dwg_ACSH_HistoryNode *arg2 = (Dwg_ACSH_HistoryNode *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BOOLEAN_CLASS_history_node_set(self,history_node);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BOOLEAN_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BOOLEAN_CLASS_history_node_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BOOLEAN_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BOOLEAN_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_BOOLEAN_CLASS_history_node_set" "', argument " "2"" of type '" "Dwg_ACSH_HistoryNode *""'"); 
    }
    arg2 = (Dwg_ACSH_HistoryNode *)(argp2);
    if (arg1) (arg1)->history_node = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BOOLEAN_CLASS_history_node_get) {
  {
    struct _dwg_object_ACSH_BOOLEAN_CLASS *arg1 = (struct _dwg_object_ACSH_BOOLEAN_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ACSH_HistoryNode *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BOOLEAN_CLASS_history_node_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BOOLEAN_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BOOLEAN_CLASS_history_node_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BOOLEAN_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BOOLEAN_CLASS *)(argp1);
    result = (Dwg_ACSH_HistoryNode *)& ((arg1)->history_node);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BOOLEAN_CLASS_major_set) {
  {
    struct _dwg_object_ACSH_BOOLEAN_CLASS *arg1 = (struct _dwg_object_ACSH_BOOLEAN_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BOOLEAN_CLASS_major_set(self,major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BOOLEAN_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BOOLEAN_CLASS_major_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BOOLEAN_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BOOLEAN_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_BOOLEAN_CLASS_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BOOLEAN_CLASS_major_get) {
  {
    struct _dwg_object_ACSH_BOOLEAN_CLASS *arg1 = (struct _dwg_object_ACSH_BOOLEAN_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BOOLEAN_CLASS_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BOOLEAN_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BOOLEAN_CLASS_major_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BOOLEAN_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BOOLEAN_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BOOLEAN_CLASS_minor_set) {
  {
    struct _dwg_object_ACSH_BOOLEAN_CLASS *arg1 = (struct _dwg_object_ACSH_BOOLEAN_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BOOLEAN_CLASS_minor_set(self,minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BOOLEAN_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BOOLEAN_CLASS_minor_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BOOLEAN_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BOOLEAN_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_BOOLEAN_CLASS_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BOOLEAN_CLASS_minor_get) {
  {
    struct _dwg_object_ACSH_BOOLEAN_CLASS *arg1 = (struct _dwg_object_ACSH_BOOLEAN_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BOOLEAN_CLASS_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BOOLEAN_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BOOLEAN_CLASS_minor_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BOOLEAN_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BOOLEAN_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BOOLEAN_CLASS_operation_set) {
  {
    struct _dwg_object_ACSH_BOOLEAN_CLASS *arg1 = (struct _dwg_object_ACSH_BOOLEAN_CLASS *) 0 ;
    BITCODE_RCd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    signed char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BOOLEAN_CLASS_operation_set(self,operation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BOOLEAN_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BOOLEAN_CLASS_operation_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BOOLEAN_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BOOLEAN_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_signed_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_BOOLEAN_CLASS_operation_set" "', argument " "2"" of type '" "BITCODE_RCd""'");
    } 
    arg2 = (BITCODE_RCd)(val2);
    if (arg1) (arg1)->operation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BOOLEAN_CLASS_operation_get) {
  {
    struct _dwg_object_ACSH_BOOLEAN_CLASS *arg1 = (struct _dwg_object_ACSH_BOOLEAN_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RCd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BOOLEAN_CLASS_operation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BOOLEAN_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BOOLEAN_CLASS_operation_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BOOLEAN_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BOOLEAN_CLASS *)(argp1);
    result = (BITCODE_RCd) ((arg1)->operation);
    ST(argvi) = SWIG_From_signed_SS_char  SWIG_PERL_CALL_ARGS_1((signed char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BOOLEAN_CLASS_operand1_set) {
  {
    struct _dwg_object_ACSH_BOOLEAN_CLASS *arg1 = (struct _dwg_object_ACSH_BOOLEAN_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BOOLEAN_CLASS_operand1_set(self,operand1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BOOLEAN_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BOOLEAN_CLASS_operand1_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BOOLEAN_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BOOLEAN_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_BOOLEAN_CLASS_operand1_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->operand1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BOOLEAN_CLASS_operand1_get) {
  {
    struct _dwg_object_ACSH_BOOLEAN_CLASS *arg1 = (struct _dwg_object_ACSH_BOOLEAN_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BOOLEAN_CLASS_operand1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BOOLEAN_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BOOLEAN_CLASS_operand1_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BOOLEAN_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BOOLEAN_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->operand1);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BOOLEAN_CLASS_operand2_set) {
  {
    struct _dwg_object_ACSH_BOOLEAN_CLASS *arg1 = (struct _dwg_object_ACSH_BOOLEAN_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BOOLEAN_CLASS_operand2_set(self,operand2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BOOLEAN_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BOOLEAN_CLASS_operand2_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BOOLEAN_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BOOLEAN_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_BOOLEAN_CLASS_operand2_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->operand2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BOOLEAN_CLASS_operand2_get) {
  {
    struct _dwg_object_ACSH_BOOLEAN_CLASS *arg1 = (struct _dwg_object_ACSH_BOOLEAN_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BOOLEAN_CLASS_operand2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BOOLEAN_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BOOLEAN_CLASS_operand2_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BOOLEAN_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BOOLEAN_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->operand2);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ACSH_BOOLEAN_CLASS) {
  {
    int argvi = 0;
    struct _dwg_object_ACSH_BOOLEAN_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ACSH_BOOLEAN_CLASS();");
    }
    result = (struct _dwg_object_ACSH_BOOLEAN_CLASS *)calloc(1, sizeof(struct _dwg_object_ACSH_BOOLEAN_CLASS));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_BOOLEAN_CLASS, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ACSH_BOOLEAN_CLASS) {
  {
    struct _dwg_object_ACSH_BOOLEAN_CLASS *arg1 = (struct _dwg_object_ACSH_BOOLEAN_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ACSH_BOOLEAN_CLASS(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BOOLEAN_CLASS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ACSH_BOOLEAN_CLASS" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BOOLEAN_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BOOLEAN_CLASS *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_parent_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_BREP_CLASS_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_parent_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_acis_empty_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_acis_empty_set(self,acis_empty);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_acis_empty_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_BREP_CLASS_acis_empty_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->acis_empty = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_acis_empty_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_acis_empty_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_acis_empty_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (BITCODE_B) ((arg1)->acis_empty);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_unknown_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_unknown_set(self,unknown);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_unknown_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_BREP_CLASS_unknown_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_unknown_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_unknown_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_unknown_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_version_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_version_set(self,version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_version_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_BREP_CLASS_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_version_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_version_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (BITCODE_BS) ((arg1)->version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_num_blocks_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_num_blocks_set(self,num_blocks);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_num_blocks_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_BREP_CLASS_num_blocks_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_blocks = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_num_blocks_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_num_blocks_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_num_blocks_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_blocks);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_block_size_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_block_size_set(self,block_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_block_size_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_BREP_CLASS_block_size_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->block_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_block_size_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_block_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_block_size_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (BITCODE_BL *) ((arg1)->block_size);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_encr_sat_data_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    char **arg2 = (char **) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_encr_sat_data_set(self,encr_sat_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_encr_sat_data_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_BREP_CLASS_encr_sat_data_set" "', argument " "2"" of type '" "char **""'"); 
    }
    arg2 = (char **)(argp2);
    if (arg1) (arg1)->encr_sat_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_encr_sat_data_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    char **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_encr_sat_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_encr_sat_data_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (char **) ((arg1)->encr_sat_data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_sab_size_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_sab_size_set(self,sab_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_sab_size_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_BREP_CLASS_sab_size_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->sab_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_sab_size_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_sab_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_sab_size_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->sab_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_acis_data_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_acis_data_set(self,acis_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_acis_data_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_BREP_CLASS_acis_data_set" "', argument " "2"" of type '" "BITCODE_RC *""'"); 
    }
    arg2 = (BITCODE_RC *)(argp2);
    if (arg1) (arg1)->acis_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_acis_data_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_acis_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_acis_data_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (BITCODE_RC *) ((arg1)->acis_data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_wireframe_data_present_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_wireframe_data_present_set(self,wireframe_data_present);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_wireframe_data_present_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_BREP_CLASS_wireframe_data_present_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->wireframe_data_present = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_wireframe_data_present_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_wireframe_data_present_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_wireframe_data_present_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (BITCODE_B) ((arg1)->wireframe_data_present);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_point_present_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_point_present_set(self,point_present);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_point_present_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_BREP_CLASS_point_present_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->point_present = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_point_present_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_point_present_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_point_present_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (BITCODE_B) ((arg1)->point_present);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_point_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_point_set(self,point);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_point_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_BREP_CLASS_point_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->point = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_point_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_point_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_point_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->point);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_isolines_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_isolines_set(self,isolines);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_isolines_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_BREP_CLASS_isolines_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->isolines = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_isolines_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_isolines_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_isolines_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->isolines);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_isoline_present_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_isoline_present_set(self,isoline_present);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_isoline_present_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_BREP_CLASS_isoline_present_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->isoline_present = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_isoline_present_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_isoline_present_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_isoline_present_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (BITCODE_B) ((arg1)->isoline_present);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_num_wires_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_num_wires_set(self,num_wires);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_num_wires_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_BREP_CLASS_num_wires_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_wires = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_num_wires_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_num_wires_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_num_wires_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_wires);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_wires_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    Dwg_3DSOLID_wire *arg2 = (Dwg_3DSOLID_wire *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_wires_set(self,wires);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_wires_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_3DSOLID_wire, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_BREP_CLASS_wires_set" "', argument " "2"" of type '" "Dwg_3DSOLID_wire *""'"); 
    }
    arg2 = (Dwg_3DSOLID_wire *)(argp2);
    if (arg1) (arg1)->wires = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_wires_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_3DSOLID_wire *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_wires_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_wires_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (Dwg_3DSOLID_wire *) ((arg1)->wires);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_3DSOLID_wire, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_num_silhouettes_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_num_silhouettes_set(self,num_silhouettes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_num_silhouettes_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_BREP_CLASS_num_silhouettes_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_silhouettes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_num_silhouettes_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_num_silhouettes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_num_silhouettes_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_silhouettes);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_silhouettes_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    Dwg_3DSOLID_silhouette *arg2 = (Dwg_3DSOLID_silhouette *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_silhouettes_set(self,silhouettes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_silhouettes_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_3DSOLID_silhouette, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_BREP_CLASS_silhouettes_set" "', argument " "2"" of type '" "Dwg_3DSOLID_silhouette *""'"); 
    }
    arg2 = (Dwg_3DSOLID_silhouette *)(argp2);
    if (arg1) (arg1)->silhouettes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_silhouettes_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_3DSOLID_silhouette *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_silhouettes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_silhouettes_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (Dwg_3DSOLID_silhouette *) ((arg1)->silhouettes);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_3DSOLID_silhouette, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS__dxf_sab_converted_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS__dxf_sab_converted_set(self,_dxf_sab_converted);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS__dxf_sab_converted_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_BREP_CLASS__dxf_sab_converted_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->_dxf_sab_converted = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS__dxf_sab_converted_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS__dxf_sab_converted_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS__dxf_sab_converted_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (BITCODE_B) ((arg1)->_dxf_sab_converted);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_acis_empty2_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_acis_empty2_set(self,acis_empty2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_acis_empty2_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_BREP_CLASS_acis_empty2_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->acis_empty2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_acis_empty2_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_acis_empty2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_acis_empty2_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (BITCODE_B) ((arg1)->acis_empty2);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_extra_acis_data_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    struct _dwg_entity_3DSOLID *arg2 = (struct _dwg_entity_3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_extra_acis_data_set(self,extra_acis_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_extra_acis_data_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_3DSOLID, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_BREP_CLASS_extra_acis_data_set" "', argument " "2"" of type '" "struct _dwg_entity_3DSOLID *""'"); 
    }
    arg2 = (struct _dwg_entity_3DSOLID *)(argp2);
    if (arg1) (arg1)->extra_acis_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_extra_acis_data_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_entity_3DSOLID *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_extra_acis_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_extra_acis_data_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (struct _dwg_entity_3DSOLID *) ((arg1)->extra_acis_data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_3DSOLID, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_num_materials_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_num_materials_set(self,num_materials);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_num_materials_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_BREP_CLASS_num_materials_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_materials = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_num_materials_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_num_materials_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_num_materials_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_materials);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_materials_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    Dwg_3DSOLID_material *arg2 = (Dwg_3DSOLID_material *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_materials_set(self,materials);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_materials_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_3DSOLID_material, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_BREP_CLASS_materials_set" "', argument " "2"" of type '" "Dwg_3DSOLID_material *""'"); 
    }
    arg2 = (Dwg_3DSOLID_material *)(argp2);
    if (arg1) (arg1)->materials = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_materials_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_3DSOLID_material *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_materials_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_materials_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (Dwg_3DSOLID_material *) ((arg1)->materials);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_3DSOLID_material, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_revision_guid_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) (BITCODE_RC *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_revision_guid_set(self,revision_guid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_revision_guid_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_BREP_CLASS_revision_guid_set" "', argument " "2"" of type '" "BITCODE_RC [39]""'"); 
    } 
    arg2 = (BITCODE_RC *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)39; ++ii) *(BITCODE_RC *)&arg1->revision_guid[ii] = *((BITCODE_RC *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""revision_guid""' of type '""BITCODE_RC [39]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_revision_guid_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_revision_guid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_revision_guid_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (BITCODE_RC *)(BITCODE_RC *) ((arg1)->revision_guid);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_revision_major_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_revision_major_set(self,revision_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_revision_major_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_BREP_CLASS_revision_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->revision_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_revision_major_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_revision_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_revision_major_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->revision_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_revision_minor1_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_revision_minor1_set(self,revision_minor1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_revision_minor1_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_BREP_CLASS_revision_minor1_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->revision_minor1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_revision_minor1_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_revision_minor1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_revision_minor1_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (BITCODE_BS) ((arg1)->revision_minor1);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_revision_minor2_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_revision_minor2_set(self,revision_minor2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_revision_minor2_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_BREP_CLASS_revision_minor2_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->revision_minor2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_revision_minor2_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_revision_minor2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_revision_minor2_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (BITCODE_BS) ((arg1)->revision_minor2);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_revision_bytes_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) (BITCODE_RC *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_revision_bytes_set(self,revision_bytes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_revision_bytes_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_BREP_CLASS_revision_bytes_set" "', argument " "2"" of type '" "BITCODE_RC [9]""'"); 
    } 
    arg2 = (BITCODE_RC *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)9; ++ii) *(BITCODE_RC *)&arg1->revision_bytes[ii] = *((BITCODE_RC *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""revision_bytes""' of type '""BITCODE_RC [9]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_revision_bytes_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_revision_bytes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_revision_bytes_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (BITCODE_RC *)(BITCODE_RC *) ((arg1)->revision_bytes);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_end_marker_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_end_marker_set(self,end_marker);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_end_marker_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_BREP_CLASS_end_marker_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->end_marker = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_end_marker_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_end_marker_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_end_marker_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->end_marker);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_history_id_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_history_id_set(self,history_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_history_id_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_BREP_CLASS_history_id_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->history_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_history_id_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_history_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_history_id_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (BITCODE_H) ((arg1)->history_id);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_has_revision_guid_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_has_revision_guid_set(self,has_revision_guid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_has_revision_guid_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_BREP_CLASS_has_revision_guid_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_revision_guid = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_has_revision_guid_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_has_revision_guid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_has_revision_guid_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (BITCODE_B) ((arg1)->has_revision_guid);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_acis_empty_bit_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_acis_empty_bit_set(self,acis_empty_bit);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_acis_empty_bit_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_BREP_CLASS_acis_empty_bit_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->acis_empty_bit = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_acis_empty_bit_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_acis_empty_bit_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_acis_empty_bit_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (BITCODE_B) ((arg1)->acis_empty_bit);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_evalexpr_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_BREP_CLASS_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_evalexpr_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_history_node_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    Dwg_ACSH_HistoryNode *arg2 = (Dwg_ACSH_HistoryNode *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_history_node_set(self,history_node);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_history_node_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_BREP_CLASS_history_node_set" "', argument " "2"" of type '" "Dwg_ACSH_HistoryNode *""'"); 
    }
    arg2 = (Dwg_ACSH_HistoryNode *)(argp2);
    if (arg1) (arg1)->history_node = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_history_node_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ACSH_HistoryNode *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_history_node_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_history_node_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (Dwg_ACSH_HistoryNode *)& ((arg1)->history_node);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_major_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_major_set(self,major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_major_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_BREP_CLASS_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_major_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_major_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_minor_set) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_minor_set(self,minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_minor_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_BREP_CLASS_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_BREP_CLASS_minor_get) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_BREP_CLASS_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_BREP_CLASS_minor_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ACSH_BREP_CLASS) {
  {
    int argvi = 0;
    struct _dwg_object_ACSH_BREP_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ACSH_BREP_CLASS();");
    }
    result = (struct _dwg_object_ACSH_BREP_CLASS *)calloc(1, sizeof(struct _dwg_object_ACSH_BREP_CLASS));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ACSH_BREP_CLASS) {
  {
    struct _dwg_object_ACSH_BREP_CLASS *arg1 = (struct _dwg_object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ACSH_BREP_CLASS(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ACSH_BREP_CLASS" "', argument " "1"" of type '" "struct _dwg_object_ACSH_BREP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_BREP_CLASS *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_parent_set) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_parent_get) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_evalexpr_set) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_evalexpr_get) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_history_node_set) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    Dwg_ACSH_HistoryNode *arg2 = (Dwg_ACSH_HistoryNode *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_history_node_set(self,history_node);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_history_node_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_history_node_set" "', argument " "2"" of type '" "Dwg_ACSH_HistoryNode *""'"); 
    }
    arg2 = (Dwg_ACSH_HistoryNode *)(argp2);
    if (arg1) (arg1)->history_node = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_history_node_get) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ACSH_HistoryNode *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_history_node_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_history_node_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    result = (Dwg_ACSH_HistoryNode *)& ((arg1)->history_node);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_major_set) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_major_set(self,major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_major_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_major_get) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_major_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_minor_set) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_minor_set(self,minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_minor_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_minor_get) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_minor_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_direction_set) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_direction_set(self,direction);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_direction_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_direction_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->direction = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_direction_get) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_direction_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_direction_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->direction);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_bl92_set) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_bl92_set(self,bl92);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_bl92_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_bl92_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bl92 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_bl92_get) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_bl92_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_bl92_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->bl92);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_shsw_text_size_set) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_shsw_text_size_set(self,shsw_text_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_shsw_text_size_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_shsw_text_size_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->shsw_text_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_shsw_text_size_get) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_shsw_text_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_shsw_text_size_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->shsw_text_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_shsw_text_set) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    BITCODE_TF arg2 = (BITCODE_TF) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_shsw_text_set(self,shsw_text);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_shsw_text_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_shsw_text_set" "', argument " "2"" of type '" "BITCODE_TF""'"); 
    }
    arg2 = (BITCODE_TF)(argp2);
    if (arg1) (arg1)->shsw_text = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_shsw_text_get) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TF result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_shsw_text_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_shsw_text_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    result = (BITCODE_TF) ((arg1)->shsw_text);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_shsw_bl93_set) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_shsw_bl93_set(self,shsw_bl93);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_shsw_bl93_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_shsw_bl93_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->shsw_bl93 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_shsw_bl93_get) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_shsw_bl93_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_shsw_bl93_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->shsw_bl93);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_shsw_text2_size_set) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_shsw_text2_size_set(self,shsw_text2_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_shsw_text2_size_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_shsw_text2_size_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->shsw_text2_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_shsw_text2_size_get) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_shsw_text2_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_shsw_text2_size_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->shsw_text2_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_shsw_text2_set) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    BITCODE_TF arg2 = (BITCODE_TF) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_shsw_text2_set(self,shsw_text2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_shsw_text2_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_shsw_text2_set" "', argument " "2"" of type '" "BITCODE_TF""'"); 
    }
    arg2 = (BITCODE_TF)(argp2);
    if (arg1) (arg1)->shsw_text2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_shsw_text2_get) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TF result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_shsw_text2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_shsw_text2_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    result = (BITCODE_TF) ((arg1)->shsw_text2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_draft_angle_set) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_draft_angle_set(self,draft_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_draft_angle_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_draft_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->draft_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_draft_angle_get) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_draft_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_draft_angle_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->draft_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_start_draft_dist_set) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_start_draft_dist_set(self,start_draft_dist);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_start_draft_dist_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_start_draft_dist_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->start_draft_dist = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_start_draft_dist_get) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_start_draft_dist_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_start_draft_dist_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->start_draft_dist);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_end_draft_dist_set) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_end_draft_dist_set(self,end_draft_dist);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_end_draft_dist_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_end_draft_dist_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->end_draft_dist = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_end_draft_dist_get) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_end_draft_dist_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_end_draft_dist_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->end_draft_dist);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_scale_factor_set) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_scale_factor_set(self,scale_factor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_scale_factor_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_scale_factor_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->scale_factor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_scale_factor_get) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_scale_factor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_scale_factor_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->scale_factor);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_twist_angle_set) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_twist_angle_set(self,twist_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_twist_angle_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_twist_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->twist_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_twist_angle_get) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_twist_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_twist_angle_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->twist_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_align_angle_set) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_align_angle_set(self,align_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_align_angle_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_align_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->align_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_align_angle_get) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_align_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_align_angle_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->align_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_sweepentity_transform_set) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_sweepentity_transform_set(self,sweepentity_transform);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_sweepentity_transform_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_sweepentity_transform_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->sweepentity_transform = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_sweepentity_transform_get) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_sweepentity_transform_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_sweepentity_transform_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    result = (BITCODE_BD *) ((arg1)->sweepentity_transform);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_pathentity_transform_set) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_pathentity_transform_set(self,pathentity_transform);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_pathentity_transform_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_pathentity_transform_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->pathentity_transform = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_pathentity_transform_get) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_pathentity_transform_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_pathentity_transform_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    result = (BITCODE_BD *) ((arg1)->pathentity_transform);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_align_option_set) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_align_option_set(self,align_option);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_align_option_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_align_option_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->align_option = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_align_option_get) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_align_option_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_align_option_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    result = (BITCODE_RC) ((arg1)->align_option);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_miter_option_set) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_miter_option_set(self,miter_option);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_miter_option_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_miter_option_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->miter_option = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_miter_option_get) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_miter_option_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_miter_option_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    result = (BITCODE_RC) ((arg1)->miter_option);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_has_align_start_set) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_has_align_start_set(self,has_align_start);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_has_align_start_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_has_align_start_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_align_start = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_has_align_start_get) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_has_align_start_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_has_align_start_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    result = (BITCODE_B) ((arg1)->has_align_start);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_bank_set) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_bank_set(self,bank);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_bank_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_bank_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->bank = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_bank_get) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_bank_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_bank_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    result = (BITCODE_B) ((arg1)->bank);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_check_intersections_set) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_check_intersections_set(self,check_intersections);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_check_intersections_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_check_intersections_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->check_intersections = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_check_intersections_get) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_check_intersections_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_check_intersections_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    result = (BITCODE_B) ((arg1)->check_intersections);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_shsw_b294_set) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_shsw_b294_set(self,shsw_b294);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_shsw_b294_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_shsw_b294_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->shsw_b294 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_shsw_b294_get) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_shsw_b294_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_shsw_b294_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    result = (BITCODE_B) ((arg1)->shsw_b294);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_shsw_b295_set) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_shsw_b295_set(self,shsw_b295);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_shsw_b295_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_shsw_b295_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->shsw_b295 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_shsw_b295_get) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_shsw_b295_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_shsw_b295_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    result = (BITCODE_B) ((arg1)->shsw_b295);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_shsw_b296_set) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_shsw_b296_set(self,shsw_b296);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_shsw_b296_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_shsw_b296_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->shsw_b296 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_shsw_b296_get) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_shsw_b296_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_shsw_b296_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    result = (BITCODE_B) ((arg1)->shsw_b296);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_pt2_set) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_pt2_set(self,pt2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_pt2_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_pt2_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->pt2 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SWEEP_CLASS_pt2_get) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SWEEP_CLASS_pt2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SWEEP_CLASS_pt2_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->pt2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ACSH_SWEEP_CLASS) {
  {
    int argvi = 0;
    struct _dwg_object_ACSH_SWEEP_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ACSH_SWEEP_CLASS();");
    }
    result = (struct _dwg_object_ACSH_SWEEP_CLASS *)calloc(1, sizeof(struct _dwg_object_ACSH_SWEEP_CLASS));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ACSH_SWEEP_CLASS) {
  {
    struct _dwg_object_ACSH_SWEEP_CLASS *arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ACSH_SWEEP_CLASS(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ACSH_SWEEP_CLASS" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SWEEP_CLASS *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_parent_set) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_parent_get) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_evalexpr_set) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_evalexpr_get) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_history_node_set) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    Dwg_ACSH_HistoryNode *arg2 = (Dwg_ACSH_HistoryNode *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_history_node_set(self,history_node);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_history_node_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_history_node_set" "', argument " "2"" of type '" "Dwg_ACSH_HistoryNode *""'"); 
    }
    arg2 = (Dwg_ACSH_HistoryNode *)(argp2);
    if (arg1) (arg1)->history_node = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_history_node_get) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ACSH_HistoryNode *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_history_node_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_history_node_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    result = (Dwg_ACSH_HistoryNode *)& ((arg1)->history_node);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_major_set) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_major_set(self,major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_major_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_major_get) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_major_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_minor_set) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_minor_set(self,minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_minor_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_minor_get) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_minor_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_direction_set) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_direction_set(self,direction);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_direction_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_direction_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->direction = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_direction_get) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_direction_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_direction_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->direction);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_bl92_set) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_bl92_set(self,bl92);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_bl92_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_bl92_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bl92 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_bl92_get) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_bl92_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_bl92_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->bl92);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text_size_set) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text_size_set(self,shsw_text_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text_size_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text_size_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->shsw_text_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text_size_get) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text_size_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->shsw_text_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text_set) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    BITCODE_TF arg2 = (BITCODE_TF) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text_set(self,shsw_text);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text_set" "', argument " "2"" of type '" "BITCODE_TF""'"); 
    }
    arg2 = (BITCODE_TF)(argp2);
    if (arg1) (arg1)->shsw_text = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text_get) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TF result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    result = (BITCODE_TF) ((arg1)->shsw_text);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_bl93_set) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_bl93_set(self,shsw_bl93);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_bl93_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_bl93_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->shsw_bl93 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_bl93_get) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_bl93_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_bl93_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->shsw_bl93);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text2_size_set) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text2_size_set(self,shsw_text2_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text2_size_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text2_size_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->shsw_text2_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text2_size_get) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text2_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text2_size_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->shsw_text2_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text2_set) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    BITCODE_TF arg2 = (BITCODE_TF) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text2_set(self,shsw_text2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text2_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text2_set" "', argument " "2"" of type '" "BITCODE_TF""'"); 
    }
    arg2 = (BITCODE_TF)(argp2);
    if (arg1) (arg1)->shsw_text2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text2_get) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TF result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text2_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    result = (BITCODE_TF) ((arg1)->shsw_text2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_draft_angle_set) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_draft_angle_set(self,draft_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_draft_angle_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_draft_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->draft_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_draft_angle_get) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_draft_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_draft_angle_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->draft_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_start_draft_dist_set) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_start_draft_dist_set(self,start_draft_dist);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_start_draft_dist_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_start_draft_dist_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->start_draft_dist = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_start_draft_dist_get) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_start_draft_dist_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_start_draft_dist_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->start_draft_dist);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_end_draft_dist_set) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_end_draft_dist_set(self,end_draft_dist);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_end_draft_dist_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_end_draft_dist_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->end_draft_dist = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_end_draft_dist_get) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_end_draft_dist_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_end_draft_dist_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->end_draft_dist);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_scale_factor_set) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_scale_factor_set(self,scale_factor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_scale_factor_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_scale_factor_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->scale_factor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_scale_factor_get) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_scale_factor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_scale_factor_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->scale_factor);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_twist_angle_set) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_twist_angle_set(self,twist_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_twist_angle_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_twist_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->twist_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_twist_angle_get) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_twist_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_twist_angle_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->twist_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_align_angle_set) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_align_angle_set(self,align_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_align_angle_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_align_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->align_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_align_angle_get) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_align_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_align_angle_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->align_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_sweepentity_transform_set) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_sweepentity_transform_set(self,sweepentity_transform);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_sweepentity_transform_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_sweepentity_transform_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->sweepentity_transform = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_sweepentity_transform_get) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_sweepentity_transform_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_sweepentity_transform_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    result = (BITCODE_BD *) ((arg1)->sweepentity_transform);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_pathentity_transform_set) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_pathentity_transform_set(self,pathentity_transform);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_pathentity_transform_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_pathentity_transform_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->pathentity_transform = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_pathentity_transform_get) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_pathentity_transform_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_pathentity_transform_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    result = (BITCODE_BD *) ((arg1)->pathentity_transform);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_align_option_set) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_align_option_set(self,align_option);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_align_option_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_align_option_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->align_option = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_align_option_get) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_align_option_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_align_option_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    result = (BITCODE_RC) ((arg1)->align_option);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_miter_option_set) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_miter_option_set(self,miter_option);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_miter_option_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_miter_option_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->miter_option = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_miter_option_get) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_miter_option_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_miter_option_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    result = (BITCODE_RC) ((arg1)->miter_option);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_has_align_start_set) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_has_align_start_set(self,has_align_start);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_has_align_start_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_has_align_start_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_align_start = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_has_align_start_get) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_has_align_start_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_has_align_start_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    result = (BITCODE_B) ((arg1)->has_align_start);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_bank_set) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_bank_set(self,bank);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_bank_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_bank_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->bank = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_bank_get) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_bank_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_bank_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    result = (BITCODE_B) ((arg1)->bank);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_check_intersections_set) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_check_intersections_set(self,check_intersections);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_check_intersections_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_check_intersections_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->check_intersections = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_check_intersections_get) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_check_intersections_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_check_intersections_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    result = (BITCODE_B) ((arg1)->check_intersections);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b294_set) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b294_set(self,shsw_b294);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b294_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b294_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->shsw_b294 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b294_get) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b294_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b294_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    result = (BITCODE_B) ((arg1)->shsw_b294);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b295_set) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b295_set(self,shsw_b295);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b295_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b295_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->shsw_b295 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b295_get) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b295_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b295_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    result = (BITCODE_B) ((arg1)->shsw_b295);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b296_set) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b296_set(self,shsw_b296);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b296_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b296_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->shsw_b296 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b296_get) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b296_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b296_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    result = (BITCODE_B) ((arg1)->shsw_b296);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_pt2_set) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_pt2_set(self,pt2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_pt2_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_pt2_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->pt2 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_pt2_get) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_EXTRUSION_CLASS_pt2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_EXTRUSION_CLASS_pt2_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->pt2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ACSH_EXTRUSION_CLASS) {
  {
    int argvi = 0;
    struct _dwg_object_ACSH_EXTRUSION_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ACSH_EXTRUSION_CLASS();");
    }
    result = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)calloc(1, sizeof(struct _dwg_object_ACSH_EXTRUSION_CLASS));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ACSH_EXTRUSION_CLASS) {
  {
    struct _dwg_object_ACSH_EXTRUSION_CLASS *arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ACSH_EXTRUSION_CLASS(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ACSH_EXTRUSION_CLASS" "', argument " "1"" of type '" "struct _dwg_object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_EXTRUSION_CLASS *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_LOFT_CLASS_parent_set) {
  {
    struct _dwg_object_ACSH_LOFT_CLASS *arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_LOFT_CLASS_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_LOFT_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_LOFT_CLASS_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_LOFT_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_LOFT_CLASS_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_LOFT_CLASS_parent_get) {
  {
    struct _dwg_object_ACSH_LOFT_CLASS *arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_LOFT_CLASS_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_LOFT_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_LOFT_CLASS_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_LOFT_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_LOFT_CLASS_evalexpr_set) {
  {
    struct _dwg_object_ACSH_LOFT_CLASS *arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_LOFT_CLASS_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_LOFT_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_LOFT_CLASS_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_LOFT_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_LOFT_CLASS_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_LOFT_CLASS_evalexpr_get) {
  {
    struct _dwg_object_ACSH_LOFT_CLASS *arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_LOFT_CLASS_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_LOFT_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_LOFT_CLASS_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_LOFT_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_LOFT_CLASS_history_node_set) {
  {
    struct _dwg_object_ACSH_LOFT_CLASS *arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *) 0 ;
    Dwg_ACSH_HistoryNode *arg2 = (Dwg_ACSH_HistoryNode *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_LOFT_CLASS_history_node_set(self,history_node);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_LOFT_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_LOFT_CLASS_history_node_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_LOFT_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_LOFT_CLASS_history_node_set" "', argument " "2"" of type '" "Dwg_ACSH_HistoryNode *""'"); 
    }
    arg2 = (Dwg_ACSH_HistoryNode *)(argp2);
    if (arg1) (arg1)->history_node = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_LOFT_CLASS_history_node_get) {
  {
    struct _dwg_object_ACSH_LOFT_CLASS *arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ACSH_HistoryNode *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_LOFT_CLASS_history_node_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_LOFT_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_LOFT_CLASS_history_node_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_LOFT_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *)(argp1);
    result = (Dwg_ACSH_HistoryNode *)& ((arg1)->history_node);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_LOFT_CLASS_major_set) {
  {
    struct _dwg_object_ACSH_LOFT_CLASS *arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_LOFT_CLASS_major_set(self,major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_LOFT_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_LOFT_CLASS_major_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_LOFT_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_LOFT_CLASS_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_LOFT_CLASS_major_get) {
  {
    struct _dwg_object_ACSH_LOFT_CLASS *arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_LOFT_CLASS_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_LOFT_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_LOFT_CLASS_major_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_LOFT_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_LOFT_CLASS_minor_set) {
  {
    struct _dwg_object_ACSH_LOFT_CLASS *arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_LOFT_CLASS_minor_set(self,minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_LOFT_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_LOFT_CLASS_minor_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_LOFT_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_LOFT_CLASS_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_LOFT_CLASS_minor_get) {
  {
    struct _dwg_object_ACSH_LOFT_CLASS *arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_LOFT_CLASS_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_LOFT_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_LOFT_CLASS_minor_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_LOFT_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_LOFT_CLASS_num_crosssects_set) {
  {
    struct _dwg_object_ACSH_LOFT_CLASS *arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_LOFT_CLASS_num_crosssects_set(self,num_crosssects);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_LOFT_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_LOFT_CLASS_num_crosssects_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_LOFT_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_LOFT_CLASS_num_crosssects_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_crosssects = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_LOFT_CLASS_num_crosssects_get) {
  {
    struct _dwg_object_ACSH_LOFT_CLASS *arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_LOFT_CLASS_num_crosssects_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_LOFT_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_LOFT_CLASS_num_crosssects_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_LOFT_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_crosssects);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_LOFT_CLASS_crosssects_set) {
  {
    struct _dwg_object_ACSH_LOFT_CLASS *arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_LOFT_CLASS_crosssects_set(self,crosssects);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_LOFT_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_LOFT_CLASS_crosssects_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_LOFT_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_LOFT_CLASS_crosssects_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->crosssects = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_LOFT_CLASS_crosssects_get) {
  {
    struct _dwg_object_ACSH_LOFT_CLASS *arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_LOFT_CLASS_crosssects_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_LOFT_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_LOFT_CLASS_crosssects_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_LOFT_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *)(argp1);
    result = (BITCODE_H *) ((arg1)->crosssects);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_LOFT_CLASS_num_guides_set) {
  {
    struct _dwg_object_ACSH_LOFT_CLASS *arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_LOFT_CLASS_num_guides_set(self,num_guides);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_LOFT_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_LOFT_CLASS_num_guides_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_LOFT_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_LOFT_CLASS_num_guides_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_guides = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_LOFT_CLASS_num_guides_get) {
  {
    struct _dwg_object_ACSH_LOFT_CLASS *arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_LOFT_CLASS_num_guides_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_LOFT_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_LOFT_CLASS_num_guides_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_LOFT_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_guides);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_LOFT_CLASS_guides_set) {
  {
    struct _dwg_object_ACSH_LOFT_CLASS *arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_LOFT_CLASS_guides_set(self,guides);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_LOFT_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_LOFT_CLASS_guides_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_LOFT_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_LOFT_CLASS_guides_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->guides = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_LOFT_CLASS_guides_get) {
  {
    struct _dwg_object_ACSH_LOFT_CLASS *arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_LOFT_CLASS_guides_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_LOFT_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_LOFT_CLASS_guides_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_LOFT_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *)(argp1);
    result = (BITCODE_H *) ((arg1)->guides);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ACSH_LOFT_CLASS) {
  {
    int argvi = 0;
    struct _dwg_object_ACSH_LOFT_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ACSH_LOFT_CLASS();");
    }
    result = (struct _dwg_object_ACSH_LOFT_CLASS *)calloc(1, sizeof(struct _dwg_object_ACSH_LOFT_CLASS));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_LOFT_CLASS, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ACSH_LOFT_CLASS) {
  {
    struct _dwg_object_ACSH_LOFT_CLASS *arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ACSH_LOFT_CLASS(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_LOFT_CLASS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ACSH_LOFT_CLASS" "', argument " "1"" of type '" "struct _dwg_object_ACSH_LOFT_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_LOFT_CLASS *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_FILLET_CLASS_parent_set) {
  {
    struct _dwg_object_ACSH_FILLET_CLASS *arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_FILLET_CLASS_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_FILLET_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_FILLET_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_FILLET_CLASS_parent_get) {
  {
    struct _dwg_object_ACSH_FILLET_CLASS *arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_FILLET_CLASS_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_FILLET_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_FILLET_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_FILLET_CLASS_evalexpr_set) {
  {
    struct _dwg_object_ACSH_FILLET_CLASS *arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_FILLET_CLASS_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_FILLET_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_FILLET_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_FILLET_CLASS_evalexpr_get) {
  {
    struct _dwg_object_ACSH_FILLET_CLASS *arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_FILLET_CLASS_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_FILLET_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_FILLET_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_FILLET_CLASS_history_node_set) {
  {
    struct _dwg_object_ACSH_FILLET_CLASS *arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *) 0 ;
    Dwg_ACSH_HistoryNode *arg2 = (Dwg_ACSH_HistoryNode *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_FILLET_CLASS_history_node_set(self,history_node);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_FILLET_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_history_node_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_FILLET_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_history_node_set" "', argument " "2"" of type '" "Dwg_ACSH_HistoryNode *""'"); 
    }
    arg2 = (Dwg_ACSH_HistoryNode *)(argp2);
    if (arg1) (arg1)->history_node = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_FILLET_CLASS_history_node_get) {
  {
    struct _dwg_object_ACSH_FILLET_CLASS *arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ACSH_HistoryNode *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_FILLET_CLASS_history_node_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_FILLET_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_history_node_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_FILLET_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *)(argp1);
    result = (Dwg_ACSH_HistoryNode *)& ((arg1)->history_node);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_FILLET_CLASS_major_set) {
  {
    struct _dwg_object_ACSH_FILLET_CLASS *arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_FILLET_CLASS_major_set(self,major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_FILLET_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_major_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_FILLET_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_FILLET_CLASS_major_get) {
  {
    struct _dwg_object_ACSH_FILLET_CLASS *arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_FILLET_CLASS_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_FILLET_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_major_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_FILLET_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_FILLET_CLASS_minor_set) {
  {
    struct _dwg_object_ACSH_FILLET_CLASS *arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_FILLET_CLASS_minor_set(self,minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_FILLET_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_minor_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_FILLET_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_FILLET_CLASS_minor_get) {
  {
    struct _dwg_object_ACSH_FILLET_CLASS *arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_FILLET_CLASS_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_FILLET_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_minor_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_FILLET_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_FILLET_CLASS_bl92_set) {
  {
    struct _dwg_object_ACSH_FILLET_CLASS *arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_FILLET_CLASS_bl92_set(self,bl92);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_FILLET_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_bl92_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_FILLET_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_bl92_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bl92 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_FILLET_CLASS_bl92_get) {
  {
    struct _dwg_object_ACSH_FILLET_CLASS *arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_FILLET_CLASS_bl92_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_FILLET_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_bl92_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_FILLET_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->bl92);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_FILLET_CLASS_num_edges_set) {
  {
    struct _dwg_object_ACSH_FILLET_CLASS *arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_FILLET_CLASS_num_edges_set(self,num_edges);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_FILLET_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_num_edges_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_FILLET_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_num_edges_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_edges = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_FILLET_CLASS_num_edges_get) {
  {
    struct _dwg_object_ACSH_FILLET_CLASS *arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_FILLET_CLASS_num_edges_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_FILLET_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_num_edges_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_FILLET_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_edges);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_FILLET_CLASS_edges_set) {
  {
    struct _dwg_object_ACSH_FILLET_CLASS *arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_FILLET_CLASS_edges_set(self,edges);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_FILLET_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_edges_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_FILLET_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_edges_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->edges = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_FILLET_CLASS_edges_get) {
  {
    struct _dwg_object_ACSH_FILLET_CLASS *arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_FILLET_CLASS_edges_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_FILLET_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_edges_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_FILLET_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *)(argp1);
    result = (BITCODE_BL *) ((arg1)->edges);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_FILLET_CLASS_num_radiuses_set) {
  {
    struct _dwg_object_ACSH_FILLET_CLASS *arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_FILLET_CLASS_num_radiuses_set(self,num_radiuses);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_FILLET_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_num_radiuses_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_FILLET_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_num_radiuses_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_radiuses = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_FILLET_CLASS_num_radiuses_get) {
  {
    struct _dwg_object_ACSH_FILLET_CLASS *arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_FILLET_CLASS_num_radiuses_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_FILLET_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_num_radiuses_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_FILLET_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_radiuses);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_FILLET_CLASS_num_startsetbacks_set) {
  {
    struct _dwg_object_ACSH_FILLET_CLASS *arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_FILLET_CLASS_num_startsetbacks_set(self,num_startsetbacks);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_FILLET_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_num_startsetbacks_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_FILLET_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_num_startsetbacks_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_startsetbacks = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_FILLET_CLASS_num_startsetbacks_get) {
  {
    struct _dwg_object_ACSH_FILLET_CLASS *arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_FILLET_CLASS_num_startsetbacks_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_FILLET_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_num_startsetbacks_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_FILLET_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_startsetbacks);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_FILLET_CLASS_num_endsetbacks_set) {
  {
    struct _dwg_object_ACSH_FILLET_CLASS *arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_FILLET_CLASS_num_endsetbacks_set(self,num_endsetbacks);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_FILLET_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_num_endsetbacks_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_FILLET_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_num_endsetbacks_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_endsetbacks = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_FILLET_CLASS_num_endsetbacks_get) {
  {
    struct _dwg_object_ACSH_FILLET_CLASS *arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_FILLET_CLASS_num_endsetbacks_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_FILLET_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_num_endsetbacks_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_FILLET_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_endsetbacks);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_FILLET_CLASS_radiuses_set) {
  {
    struct _dwg_object_ACSH_FILLET_CLASS *arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_FILLET_CLASS_radiuses_set(self,radiuses);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_FILLET_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_radiuses_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_FILLET_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_radiuses_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->radiuses = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_FILLET_CLASS_radiuses_get) {
  {
    struct _dwg_object_ACSH_FILLET_CLASS *arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_FILLET_CLASS_radiuses_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_FILLET_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_radiuses_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_FILLET_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *)(argp1);
    result = (BITCODE_BD *) ((arg1)->radiuses);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_FILLET_CLASS_startsetbacks_set) {
  {
    struct _dwg_object_ACSH_FILLET_CLASS *arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_FILLET_CLASS_startsetbacks_set(self,startsetbacks);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_FILLET_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_startsetbacks_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_FILLET_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_startsetbacks_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->startsetbacks = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_FILLET_CLASS_startsetbacks_get) {
  {
    struct _dwg_object_ACSH_FILLET_CLASS *arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_FILLET_CLASS_startsetbacks_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_FILLET_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_startsetbacks_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_FILLET_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *)(argp1);
    result = (BITCODE_BD *) ((arg1)->startsetbacks);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_FILLET_CLASS_endsetbacks_set) {
  {
    struct _dwg_object_ACSH_FILLET_CLASS *arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_FILLET_CLASS_endsetbacks_set(self,endsetbacks);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_FILLET_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_endsetbacks_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_FILLET_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_endsetbacks_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->endsetbacks = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_FILLET_CLASS_endsetbacks_get) {
  {
    struct _dwg_object_ACSH_FILLET_CLASS *arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_FILLET_CLASS_endsetbacks_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_FILLET_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_FILLET_CLASS_endsetbacks_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_FILLET_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *)(argp1);
    result = (BITCODE_BD *) ((arg1)->endsetbacks);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ACSH_FILLET_CLASS) {
  {
    int argvi = 0;
    struct _dwg_object_ACSH_FILLET_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ACSH_FILLET_CLASS();");
    }
    result = (struct _dwg_object_ACSH_FILLET_CLASS *)calloc(1, sizeof(struct _dwg_object_ACSH_FILLET_CLASS));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_FILLET_CLASS, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ACSH_FILLET_CLASS) {
  {
    struct _dwg_object_ACSH_FILLET_CLASS *arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ACSH_FILLET_CLASS(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_FILLET_CLASS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ACSH_FILLET_CLASS" "', argument " "1"" of type '" "struct _dwg_object_ACSH_FILLET_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_FILLET_CLASS *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CHAMFER_CLASS_parent_set) {
  {
    struct _dwg_object_ACSH_CHAMFER_CLASS *arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CHAMFER_CLASS_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CHAMFER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CHAMFER_CLASS_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CHAMFER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_CHAMFER_CLASS_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CHAMFER_CLASS_parent_get) {
  {
    struct _dwg_object_ACSH_CHAMFER_CLASS *arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CHAMFER_CLASS_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CHAMFER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CHAMFER_CLASS_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CHAMFER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CHAMFER_CLASS_evalexpr_set) {
  {
    struct _dwg_object_ACSH_CHAMFER_CLASS *arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CHAMFER_CLASS_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CHAMFER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CHAMFER_CLASS_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CHAMFER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_CHAMFER_CLASS_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CHAMFER_CLASS_evalexpr_get) {
  {
    struct _dwg_object_ACSH_CHAMFER_CLASS *arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CHAMFER_CLASS_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CHAMFER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CHAMFER_CLASS_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CHAMFER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CHAMFER_CLASS_history_node_set) {
  {
    struct _dwg_object_ACSH_CHAMFER_CLASS *arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *) 0 ;
    Dwg_ACSH_HistoryNode *arg2 = (Dwg_ACSH_HistoryNode *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CHAMFER_CLASS_history_node_set(self,history_node);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CHAMFER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CHAMFER_CLASS_history_node_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CHAMFER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_CHAMFER_CLASS_history_node_set" "', argument " "2"" of type '" "Dwg_ACSH_HistoryNode *""'"); 
    }
    arg2 = (Dwg_ACSH_HistoryNode *)(argp2);
    if (arg1) (arg1)->history_node = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CHAMFER_CLASS_history_node_get) {
  {
    struct _dwg_object_ACSH_CHAMFER_CLASS *arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ACSH_HistoryNode *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CHAMFER_CLASS_history_node_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CHAMFER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CHAMFER_CLASS_history_node_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CHAMFER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *)(argp1);
    result = (Dwg_ACSH_HistoryNode *)& ((arg1)->history_node);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CHAMFER_CLASS_major_set) {
  {
    struct _dwg_object_ACSH_CHAMFER_CLASS *arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CHAMFER_CLASS_major_set(self,major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CHAMFER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CHAMFER_CLASS_major_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CHAMFER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_CHAMFER_CLASS_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CHAMFER_CLASS_major_get) {
  {
    struct _dwg_object_ACSH_CHAMFER_CLASS *arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CHAMFER_CLASS_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CHAMFER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CHAMFER_CLASS_major_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CHAMFER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CHAMFER_CLASS_minor_set) {
  {
    struct _dwg_object_ACSH_CHAMFER_CLASS *arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CHAMFER_CLASS_minor_set(self,minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CHAMFER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CHAMFER_CLASS_minor_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CHAMFER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_CHAMFER_CLASS_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CHAMFER_CLASS_minor_get) {
  {
    struct _dwg_object_ACSH_CHAMFER_CLASS *arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CHAMFER_CLASS_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CHAMFER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CHAMFER_CLASS_minor_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CHAMFER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CHAMFER_CLASS_bl92_set) {
  {
    struct _dwg_object_ACSH_CHAMFER_CLASS *arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CHAMFER_CLASS_bl92_set(self,bl92);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CHAMFER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CHAMFER_CLASS_bl92_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CHAMFER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_CHAMFER_CLASS_bl92_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bl92 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CHAMFER_CLASS_bl92_get) {
  {
    struct _dwg_object_ACSH_CHAMFER_CLASS *arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CHAMFER_CLASS_bl92_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CHAMFER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CHAMFER_CLASS_bl92_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CHAMFER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->bl92);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CHAMFER_CLASS_base_dist_set) {
  {
    struct _dwg_object_ACSH_CHAMFER_CLASS *arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CHAMFER_CLASS_base_dist_set(self,base_dist);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CHAMFER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CHAMFER_CLASS_base_dist_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CHAMFER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_CHAMFER_CLASS_base_dist_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->base_dist = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CHAMFER_CLASS_base_dist_get) {
  {
    struct _dwg_object_ACSH_CHAMFER_CLASS *arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CHAMFER_CLASS_base_dist_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CHAMFER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CHAMFER_CLASS_base_dist_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CHAMFER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->base_dist);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CHAMFER_CLASS_other_dist_set) {
  {
    struct _dwg_object_ACSH_CHAMFER_CLASS *arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CHAMFER_CLASS_other_dist_set(self,other_dist);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CHAMFER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CHAMFER_CLASS_other_dist_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CHAMFER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_CHAMFER_CLASS_other_dist_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->other_dist = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CHAMFER_CLASS_other_dist_get) {
  {
    struct _dwg_object_ACSH_CHAMFER_CLASS *arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CHAMFER_CLASS_other_dist_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CHAMFER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CHAMFER_CLASS_other_dist_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CHAMFER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->other_dist);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CHAMFER_CLASS_num_edges_set) {
  {
    struct _dwg_object_ACSH_CHAMFER_CLASS *arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CHAMFER_CLASS_num_edges_set(self,num_edges);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CHAMFER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CHAMFER_CLASS_num_edges_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CHAMFER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_CHAMFER_CLASS_num_edges_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_edges = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CHAMFER_CLASS_num_edges_get) {
  {
    struct _dwg_object_ACSH_CHAMFER_CLASS *arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CHAMFER_CLASS_num_edges_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CHAMFER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CHAMFER_CLASS_num_edges_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CHAMFER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_edges);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CHAMFER_CLASS_edges_set) {
  {
    struct _dwg_object_ACSH_CHAMFER_CLASS *arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CHAMFER_CLASS_edges_set(self,edges);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CHAMFER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CHAMFER_CLASS_edges_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CHAMFER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_CHAMFER_CLASS_edges_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->edges = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CHAMFER_CLASS_edges_get) {
  {
    struct _dwg_object_ACSH_CHAMFER_CLASS *arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CHAMFER_CLASS_edges_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CHAMFER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CHAMFER_CLASS_edges_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CHAMFER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *)(argp1);
    result = (BITCODE_BL *) ((arg1)->edges);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CHAMFER_CLASS_bl95_set) {
  {
    struct _dwg_object_ACSH_CHAMFER_CLASS *arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CHAMFER_CLASS_bl95_set(self,bl95);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CHAMFER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CHAMFER_CLASS_bl95_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CHAMFER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_CHAMFER_CLASS_bl95_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bl95 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CHAMFER_CLASS_bl95_get) {
  {
    struct _dwg_object_ACSH_CHAMFER_CLASS *arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CHAMFER_CLASS_bl95_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CHAMFER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CHAMFER_CLASS_bl95_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CHAMFER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->bl95);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ACSH_CHAMFER_CLASS) {
  {
    int argvi = 0;
    struct _dwg_object_ACSH_CHAMFER_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ACSH_CHAMFER_CLASS();");
    }
    result = (struct _dwg_object_ACSH_CHAMFER_CLASS *)calloc(1, sizeof(struct _dwg_object_ACSH_CHAMFER_CLASS));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_CHAMFER_CLASS, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ACSH_CHAMFER_CLASS) {
  {
    struct _dwg_object_ACSH_CHAMFER_CLASS *arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ACSH_CHAMFER_CLASS(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CHAMFER_CLASS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ACSH_CHAMFER_CLASS" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CHAMFER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CHAMFER_CLASS *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CYLINDER_CLASS_parent_set) {
  {
    struct _dwg_object_ACSH_CYLINDER_CLASS *arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CYLINDER_CLASS_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CYLINDER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CYLINDER_CLASS_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CYLINDER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_CYLINDER_CLASS_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CYLINDER_CLASS_parent_get) {
  {
    struct _dwg_object_ACSH_CYLINDER_CLASS *arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CYLINDER_CLASS_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CYLINDER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CYLINDER_CLASS_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CYLINDER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CYLINDER_CLASS_evalexpr_set) {
  {
    struct _dwg_object_ACSH_CYLINDER_CLASS *arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CYLINDER_CLASS_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CYLINDER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CYLINDER_CLASS_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CYLINDER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_CYLINDER_CLASS_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CYLINDER_CLASS_evalexpr_get) {
  {
    struct _dwg_object_ACSH_CYLINDER_CLASS *arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CYLINDER_CLASS_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CYLINDER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CYLINDER_CLASS_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CYLINDER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CYLINDER_CLASS_history_node_set) {
  {
    struct _dwg_object_ACSH_CYLINDER_CLASS *arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *) 0 ;
    Dwg_ACSH_HistoryNode *arg2 = (Dwg_ACSH_HistoryNode *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CYLINDER_CLASS_history_node_set(self,history_node);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CYLINDER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CYLINDER_CLASS_history_node_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CYLINDER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_CYLINDER_CLASS_history_node_set" "', argument " "2"" of type '" "Dwg_ACSH_HistoryNode *""'"); 
    }
    arg2 = (Dwg_ACSH_HistoryNode *)(argp2);
    if (arg1) (arg1)->history_node = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CYLINDER_CLASS_history_node_get) {
  {
    struct _dwg_object_ACSH_CYLINDER_CLASS *arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ACSH_HistoryNode *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CYLINDER_CLASS_history_node_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CYLINDER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CYLINDER_CLASS_history_node_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CYLINDER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *)(argp1);
    result = (Dwg_ACSH_HistoryNode *)& ((arg1)->history_node);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CYLINDER_CLASS_major_set) {
  {
    struct _dwg_object_ACSH_CYLINDER_CLASS *arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CYLINDER_CLASS_major_set(self,major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CYLINDER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CYLINDER_CLASS_major_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CYLINDER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_CYLINDER_CLASS_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CYLINDER_CLASS_major_get) {
  {
    struct _dwg_object_ACSH_CYLINDER_CLASS *arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CYLINDER_CLASS_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CYLINDER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CYLINDER_CLASS_major_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CYLINDER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CYLINDER_CLASS_minor_set) {
  {
    struct _dwg_object_ACSH_CYLINDER_CLASS *arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CYLINDER_CLASS_minor_set(self,minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CYLINDER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CYLINDER_CLASS_minor_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CYLINDER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_CYLINDER_CLASS_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CYLINDER_CLASS_minor_get) {
  {
    struct _dwg_object_ACSH_CYLINDER_CLASS *arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CYLINDER_CLASS_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CYLINDER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CYLINDER_CLASS_minor_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CYLINDER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CYLINDER_CLASS_height_set) {
  {
    struct _dwg_object_ACSH_CYLINDER_CLASS *arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CYLINDER_CLASS_height_set(self,height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CYLINDER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CYLINDER_CLASS_height_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CYLINDER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_CYLINDER_CLASS_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CYLINDER_CLASS_height_get) {
  {
    struct _dwg_object_ACSH_CYLINDER_CLASS *arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CYLINDER_CLASS_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CYLINDER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CYLINDER_CLASS_height_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CYLINDER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CYLINDER_CLASS_major_radius_set) {
  {
    struct _dwg_object_ACSH_CYLINDER_CLASS *arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CYLINDER_CLASS_major_radius_set(self,major_radius);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CYLINDER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CYLINDER_CLASS_major_radius_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CYLINDER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_CYLINDER_CLASS_major_radius_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->major_radius = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CYLINDER_CLASS_major_radius_get) {
  {
    struct _dwg_object_ACSH_CYLINDER_CLASS *arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CYLINDER_CLASS_major_radius_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CYLINDER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CYLINDER_CLASS_major_radius_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CYLINDER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->major_radius);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CYLINDER_CLASS_minor_radius_set) {
  {
    struct _dwg_object_ACSH_CYLINDER_CLASS *arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CYLINDER_CLASS_minor_radius_set(self,minor_radius);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CYLINDER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CYLINDER_CLASS_minor_radius_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CYLINDER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_CYLINDER_CLASS_minor_radius_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->minor_radius = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CYLINDER_CLASS_minor_radius_get) {
  {
    struct _dwg_object_ACSH_CYLINDER_CLASS *arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CYLINDER_CLASS_minor_radius_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CYLINDER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CYLINDER_CLASS_minor_radius_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CYLINDER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->minor_radius);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CYLINDER_CLASS_x_radius_set) {
  {
    struct _dwg_object_ACSH_CYLINDER_CLASS *arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CYLINDER_CLASS_x_radius_set(self,x_radius);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CYLINDER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CYLINDER_CLASS_x_radius_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CYLINDER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_CYLINDER_CLASS_x_radius_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->x_radius = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CYLINDER_CLASS_x_radius_get) {
  {
    struct _dwg_object_ACSH_CYLINDER_CLASS *arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CYLINDER_CLASS_x_radius_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CYLINDER_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CYLINDER_CLASS_x_radius_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CYLINDER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->x_radius);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ACSH_CYLINDER_CLASS) {
  {
    int argvi = 0;
    struct _dwg_object_ACSH_CYLINDER_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ACSH_CYLINDER_CLASS();");
    }
    result = (struct _dwg_object_ACSH_CYLINDER_CLASS *)calloc(1, sizeof(struct _dwg_object_ACSH_CYLINDER_CLASS));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_CYLINDER_CLASS, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ACSH_CYLINDER_CLASS) {
  {
    struct _dwg_object_ACSH_CYLINDER_CLASS *arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ACSH_CYLINDER_CLASS(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CYLINDER_CLASS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ACSH_CYLINDER_CLASS" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CYLINDER_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CYLINDER_CLASS *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CONE_CLASS_parent_set) {
  {
    struct _dwg_object_ACSH_CONE_CLASS *arg1 = (struct _dwg_object_ACSH_CONE_CLASS *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CONE_CLASS_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CONE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CONE_CLASS_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CONE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CONE_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_CONE_CLASS_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CONE_CLASS_parent_get) {
  {
    struct _dwg_object_ACSH_CONE_CLASS *arg1 = (struct _dwg_object_ACSH_CONE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CONE_CLASS_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CONE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CONE_CLASS_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CONE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CONE_CLASS *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CONE_CLASS_evalexpr_set) {
  {
    struct _dwg_object_ACSH_CONE_CLASS *arg1 = (struct _dwg_object_ACSH_CONE_CLASS *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CONE_CLASS_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CONE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CONE_CLASS_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CONE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CONE_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_CONE_CLASS_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CONE_CLASS_evalexpr_get) {
  {
    struct _dwg_object_ACSH_CONE_CLASS *arg1 = (struct _dwg_object_ACSH_CONE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CONE_CLASS_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CONE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CONE_CLASS_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CONE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CONE_CLASS *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CONE_CLASS_history_node_set) {
  {
    struct _dwg_object_ACSH_CONE_CLASS *arg1 = (struct _dwg_object_ACSH_CONE_CLASS *) 0 ;
    Dwg_ACSH_HistoryNode *arg2 = (Dwg_ACSH_HistoryNode *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CONE_CLASS_history_node_set(self,history_node);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CONE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CONE_CLASS_history_node_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CONE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CONE_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_CONE_CLASS_history_node_set" "', argument " "2"" of type '" "Dwg_ACSH_HistoryNode *""'"); 
    }
    arg2 = (Dwg_ACSH_HistoryNode *)(argp2);
    if (arg1) (arg1)->history_node = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CONE_CLASS_history_node_get) {
  {
    struct _dwg_object_ACSH_CONE_CLASS *arg1 = (struct _dwg_object_ACSH_CONE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ACSH_HistoryNode *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CONE_CLASS_history_node_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CONE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CONE_CLASS_history_node_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CONE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CONE_CLASS *)(argp1);
    result = (Dwg_ACSH_HistoryNode *)& ((arg1)->history_node);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CONE_CLASS_major_set) {
  {
    struct _dwg_object_ACSH_CONE_CLASS *arg1 = (struct _dwg_object_ACSH_CONE_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CONE_CLASS_major_set(self,major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CONE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CONE_CLASS_major_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CONE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CONE_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_CONE_CLASS_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CONE_CLASS_major_get) {
  {
    struct _dwg_object_ACSH_CONE_CLASS *arg1 = (struct _dwg_object_ACSH_CONE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CONE_CLASS_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CONE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CONE_CLASS_major_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CONE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CONE_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CONE_CLASS_minor_set) {
  {
    struct _dwg_object_ACSH_CONE_CLASS *arg1 = (struct _dwg_object_ACSH_CONE_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CONE_CLASS_minor_set(self,minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CONE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CONE_CLASS_minor_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CONE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CONE_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_CONE_CLASS_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CONE_CLASS_minor_get) {
  {
    struct _dwg_object_ACSH_CONE_CLASS *arg1 = (struct _dwg_object_ACSH_CONE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CONE_CLASS_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CONE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CONE_CLASS_minor_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CONE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CONE_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CONE_CLASS_height_set) {
  {
    struct _dwg_object_ACSH_CONE_CLASS *arg1 = (struct _dwg_object_ACSH_CONE_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CONE_CLASS_height_set(self,height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CONE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CONE_CLASS_height_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CONE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CONE_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_CONE_CLASS_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CONE_CLASS_height_get) {
  {
    struct _dwg_object_ACSH_CONE_CLASS *arg1 = (struct _dwg_object_ACSH_CONE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CONE_CLASS_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CONE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CONE_CLASS_height_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CONE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CONE_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CONE_CLASS_major_radius_set) {
  {
    struct _dwg_object_ACSH_CONE_CLASS *arg1 = (struct _dwg_object_ACSH_CONE_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CONE_CLASS_major_radius_set(self,major_radius);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CONE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CONE_CLASS_major_radius_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CONE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CONE_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_CONE_CLASS_major_radius_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->major_radius = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CONE_CLASS_major_radius_get) {
  {
    struct _dwg_object_ACSH_CONE_CLASS *arg1 = (struct _dwg_object_ACSH_CONE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CONE_CLASS_major_radius_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CONE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CONE_CLASS_major_radius_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CONE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CONE_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->major_radius);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CONE_CLASS_minor_radius_set) {
  {
    struct _dwg_object_ACSH_CONE_CLASS *arg1 = (struct _dwg_object_ACSH_CONE_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CONE_CLASS_minor_radius_set(self,minor_radius);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CONE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CONE_CLASS_minor_radius_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CONE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CONE_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_CONE_CLASS_minor_radius_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->minor_radius = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CONE_CLASS_minor_radius_get) {
  {
    struct _dwg_object_ACSH_CONE_CLASS *arg1 = (struct _dwg_object_ACSH_CONE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CONE_CLASS_minor_radius_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CONE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CONE_CLASS_minor_radius_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CONE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CONE_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->minor_radius);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CONE_CLASS_x_radius_set) {
  {
    struct _dwg_object_ACSH_CONE_CLASS *arg1 = (struct _dwg_object_ACSH_CONE_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CONE_CLASS_x_radius_set(self,x_radius);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CONE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CONE_CLASS_x_radius_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CONE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CONE_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_CONE_CLASS_x_radius_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->x_radius = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_CONE_CLASS_x_radius_get) {
  {
    struct _dwg_object_ACSH_CONE_CLASS *arg1 = (struct _dwg_object_ACSH_CONE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_CONE_CLASS_x_radius_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CONE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_CONE_CLASS_x_radius_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CONE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CONE_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->x_radius);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ACSH_CONE_CLASS) {
  {
    int argvi = 0;
    struct _dwg_object_ACSH_CONE_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ACSH_CONE_CLASS();");
    }
    result = (struct _dwg_object_ACSH_CONE_CLASS *)calloc(1, sizeof(struct _dwg_object_ACSH_CONE_CLASS));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_CONE_CLASS, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ACSH_CONE_CLASS) {
  {
    struct _dwg_object_ACSH_CONE_CLASS *arg1 = (struct _dwg_object_ACSH_CONE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ACSH_CONE_CLASS(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_CONE_CLASS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ACSH_CONE_CLASS" "', argument " "1"" of type '" "struct _dwg_object_ACSH_CONE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_CONE_CLASS *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_PYRAMID_CLASS_parent_set) {
  {
    struct _dwg_object_ACSH_PYRAMID_CLASS *arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_PYRAMID_CLASS_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_PYRAMID_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_PYRAMID_CLASS_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_PYRAMID_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_PYRAMID_CLASS_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_PYRAMID_CLASS_parent_get) {
  {
    struct _dwg_object_ACSH_PYRAMID_CLASS *arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_PYRAMID_CLASS_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_PYRAMID_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_PYRAMID_CLASS_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_PYRAMID_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_PYRAMID_CLASS_evalexpr_set) {
  {
    struct _dwg_object_ACSH_PYRAMID_CLASS *arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_PYRAMID_CLASS_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_PYRAMID_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_PYRAMID_CLASS_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_PYRAMID_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_PYRAMID_CLASS_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_PYRAMID_CLASS_evalexpr_get) {
  {
    struct _dwg_object_ACSH_PYRAMID_CLASS *arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_PYRAMID_CLASS_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_PYRAMID_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_PYRAMID_CLASS_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_PYRAMID_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_PYRAMID_CLASS_history_node_set) {
  {
    struct _dwg_object_ACSH_PYRAMID_CLASS *arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *) 0 ;
    Dwg_ACSH_HistoryNode *arg2 = (Dwg_ACSH_HistoryNode *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_PYRAMID_CLASS_history_node_set(self,history_node);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_PYRAMID_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_PYRAMID_CLASS_history_node_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_PYRAMID_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_PYRAMID_CLASS_history_node_set" "', argument " "2"" of type '" "Dwg_ACSH_HistoryNode *""'"); 
    }
    arg2 = (Dwg_ACSH_HistoryNode *)(argp2);
    if (arg1) (arg1)->history_node = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_PYRAMID_CLASS_history_node_get) {
  {
    struct _dwg_object_ACSH_PYRAMID_CLASS *arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ACSH_HistoryNode *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_PYRAMID_CLASS_history_node_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_PYRAMID_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_PYRAMID_CLASS_history_node_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_PYRAMID_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *)(argp1);
    result = (Dwg_ACSH_HistoryNode *)& ((arg1)->history_node);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_PYRAMID_CLASS_major_set) {
  {
    struct _dwg_object_ACSH_PYRAMID_CLASS *arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_PYRAMID_CLASS_major_set(self,major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_PYRAMID_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_PYRAMID_CLASS_major_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_PYRAMID_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_PYRAMID_CLASS_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_PYRAMID_CLASS_major_get) {
  {
    struct _dwg_object_ACSH_PYRAMID_CLASS *arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_PYRAMID_CLASS_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_PYRAMID_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_PYRAMID_CLASS_major_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_PYRAMID_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_PYRAMID_CLASS_minor_set) {
  {
    struct _dwg_object_ACSH_PYRAMID_CLASS *arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_PYRAMID_CLASS_minor_set(self,minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_PYRAMID_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_PYRAMID_CLASS_minor_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_PYRAMID_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_PYRAMID_CLASS_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_PYRAMID_CLASS_minor_get) {
  {
    struct _dwg_object_ACSH_PYRAMID_CLASS *arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_PYRAMID_CLASS_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_PYRAMID_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_PYRAMID_CLASS_minor_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_PYRAMID_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_PYRAMID_CLASS_height_set) {
  {
    struct _dwg_object_ACSH_PYRAMID_CLASS *arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_PYRAMID_CLASS_height_set(self,height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_PYRAMID_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_PYRAMID_CLASS_height_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_PYRAMID_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_PYRAMID_CLASS_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_PYRAMID_CLASS_height_get) {
  {
    struct _dwg_object_ACSH_PYRAMID_CLASS *arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_PYRAMID_CLASS_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_PYRAMID_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_PYRAMID_CLASS_height_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_PYRAMID_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_PYRAMID_CLASS_sides_set) {
  {
    struct _dwg_object_ACSH_PYRAMID_CLASS *arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_PYRAMID_CLASS_sides_set(self,sides);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_PYRAMID_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_PYRAMID_CLASS_sides_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_PYRAMID_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_PYRAMID_CLASS_sides_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->sides = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_PYRAMID_CLASS_sides_get) {
  {
    struct _dwg_object_ACSH_PYRAMID_CLASS *arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_PYRAMID_CLASS_sides_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_PYRAMID_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_PYRAMID_CLASS_sides_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_PYRAMID_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->sides);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_PYRAMID_CLASS_radius_set) {
  {
    struct _dwg_object_ACSH_PYRAMID_CLASS *arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_PYRAMID_CLASS_radius_set(self,radius);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_PYRAMID_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_PYRAMID_CLASS_radius_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_PYRAMID_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_PYRAMID_CLASS_radius_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->radius = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_PYRAMID_CLASS_radius_get) {
  {
    struct _dwg_object_ACSH_PYRAMID_CLASS *arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_PYRAMID_CLASS_radius_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_PYRAMID_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_PYRAMID_CLASS_radius_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_PYRAMID_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->radius);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_PYRAMID_CLASS_topradius_set) {
  {
    struct _dwg_object_ACSH_PYRAMID_CLASS *arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_PYRAMID_CLASS_topradius_set(self,topradius);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_PYRAMID_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_PYRAMID_CLASS_topradius_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_PYRAMID_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_PYRAMID_CLASS_topradius_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->topradius = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_PYRAMID_CLASS_topradius_get) {
  {
    struct _dwg_object_ACSH_PYRAMID_CLASS *arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_PYRAMID_CLASS_topradius_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_PYRAMID_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_PYRAMID_CLASS_topradius_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_PYRAMID_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->topradius);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ACSH_PYRAMID_CLASS) {
  {
    int argvi = 0;
    struct _dwg_object_ACSH_PYRAMID_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ACSH_PYRAMID_CLASS();");
    }
    result = (struct _dwg_object_ACSH_PYRAMID_CLASS *)calloc(1, sizeof(struct _dwg_object_ACSH_PYRAMID_CLASS));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_PYRAMID_CLASS, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ACSH_PYRAMID_CLASS) {
  {
    struct _dwg_object_ACSH_PYRAMID_CLASS *arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ACSH_PYRAMID_CLASS(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_PYRAMID_CLASS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ACSH_PYRAMID_CLASS" "', argument " "1"" of type '" "struct _dwg_object_ACSH_PYRAMID_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_PYRAMID_CLASS *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SPHERE_CLASS_parent_set) {
  {
    struct _dwg_object_ACSH_SPHERE_CLASS *arg1 = (struct _dwg_object_ACSH_SPHERE_CLASS *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SPHERE_CLASS_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SPHERE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SPHERE_CLASS_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SPHERE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SPHERE_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_SPHERE_CLASS_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SPHERE_CLASS_parent_get) {
  {
    struct _dwg_object_ACSH_SPHERE_CLASS *arg1 = (struct _dwg_object_ACSH_SPHERE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SPHERE_CLASS_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SPHERE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SPHERE_CLASS_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SPHERE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SPHERE_CLASS *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SPHERE_CLASS_evalexpr_set) {
  {
    struct _dwg_object_ACSH_SPHERE_CLASS *arg1 = (struct _dwg_object_ACSH_SPHERE_CLASS *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SPHERE_CLASS_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SPHERE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SPHERE_CLASS_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SPHERE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SPHERE_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_SPHERE_CLASS_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SPHERE_CLASS_evalexpr_get) {
  {
    struct _dwg_object_ACSH_SPHERE_CLASS *arg1 = (struct _dwg_object_ACSH_SPHERE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SPHERE_CLASS_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SPHERE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SPHERE_CLASS_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SPHERE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SPHERE_CLASS *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SPHERE_CLASS_history_node_set) {
  {
    struct _dwg_object_ACSH_SPHERE_CLASS *arg1 = (struct _dwg_object_ACSH_SPHERE_CLASS *) 0 ;
    Dwg_ACSH_HistoryNode *arg2 = (Dwg_ACSH_HistoryNode *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SPHERE_CLASS_history_node_set(self,history_node);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SPHERE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SPHERE_CLASS_history_node_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SPHERE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SPHERE_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_SPHERE_CLASS_history_node_set" "', argument " "2"" of type '" "Dwg_ACSH_HistoryNode *""'"); 
    }
    arg2 = (Dwg_ACSH_HistoryNode *)(argp2);
    if (arg1) (arg1)->history_node = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SPHERE_CLASS_history_node_get) {
  {
    struct _dwg_object_ACSH_SPHERE_CLASS *arg1 = (struct _dwg_object_ACSH_SPHERE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ACSH_HistoryNode *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SPHERE_CLASS_history_node_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SPHERE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SPHERE_CLASS_history_node_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SPHERE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SPHERE_CLASS *)(argp1);
    result = (Dwg_ACSH_HistoryNode *)& ((arg1)->history_node);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SPHERE_CLASS_major_set) {
  {
    struct _dwg_object_ACSH_SPHERE_CLASS *arg1 = (struct _dwg_object_ACSH_SPHERE_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SPHERE_CLASS_major_set(self,major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SPHERE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SPHERE_CLASS_major_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SPHERE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SPHERE_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_SPHERE_CLASS_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SPHERE_CLASS_major_get) {
  {
    struct _dwg_object_ACSH_SPHERE_CLASS *arg1 = (struct _dwg_object_ACSH_SPHERE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SPHERE_CLASS_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SPHERE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SPHERE_CLASS_major_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SPHERE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SPHERE_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SPHERE_CLASS_minor_set) {
  {
    struct _dwg_object_ACSH_SPHERE_CLASS *arg1 = (struct _dwg_object_ACSH_SPHERE_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SPHERE_CLASS_minor_set(self,minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SPHERE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SPHERE_CLASS_minor_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SPHERE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SPHERE_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_SPHERE_CLASS_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SPHERE_CLASS_minor_get) {
  {
    struct _dwg_object_ACSH_SPHERE_CLASS *arg1 = (struct _dwg_object_ACSH_SPHERE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SPHERE_CLASS_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SPHERE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SPHERE_CLASS_minor_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SPHERE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SPHERE_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SPHERE_CLASS_radius_set) {
  {
    struct _dwg_object_ACSH_SPHERE_CLASS *arg1 = (struct _dwg_object_ACSH_SPHERE_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SPHERE_CLASS_radius_set(self,radius);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SPHERE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SPHERE_CLASS_radius_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SPHERE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SPHERE_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_SPHERE_CLASS_radius_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->radius = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_SPHERE_CLASS_radius_get) {
  {
    struct _dwg_object_ACSH_SPHERE_CLASS *arg1 = (struct _dwg_object_ACSH_SPHERE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_SPHERE_CLASS_radius_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SPHERE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_SPHERE_CLASS_radius_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SPHERE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SPHERE_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->radius);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ACSH_SPHERE_CLASS) {
  {
    int argvi = 0;
    struct _dwg_object_ACSH_SPHERE_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ACSH_SPHERE_CLASS();");
    }
    result = (struct _dwg_object_ACSH_SPHERE_CLASS *)calloc(1, sizeof(struct _dwg_object_ACSH_SPHERE_CLASS));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_SPHERE_CLASS, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ACSH_SPHERE_CLASS) {
  {
    struct _dwg_object_ACSH_SPHERE_CLASS *arg1 = (struct _dwg_object_ACSH_SPHERE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ACSH_SPHERE_CLASS(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_SPHERE_CLASS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ACSH_SPHERE_CLASS" "', argument " "1"" of type '" "struct _dwg_object_ACSH_SPHERE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_SPHERE_CLASS *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_TORUS_CLASS_parent_set) {
  {
    struct _dwg_object_ACSH_TORUS_CLASS *arg1 = (struct _dwg_object_ACSH_TORUS_CLASS *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_TORUS_CLASS_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_TORUS_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_TORUS_CLASS_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_TORUS_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_TORUS_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_TORUS_CLASS_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_TORUS_CLASS_parent_get) {
  {
    struct _dwg_object_ACSH_TORUS_CLASS *arg1 = (struct _dwg_object_ACSH_TORUS_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_TORUS_CLASS_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_TORUS_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_TORUS_CLASS_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_TORUS_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_TORUS_CLASS *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_TORUS_CLASS_evalexpr_set) {
  {
    struct _dwg_object_ACSH_TORUS_CLASS *arg1 = (struct _dwg_object_ACSH_TORUS_CLASS *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_TORUS_CLASS_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_TORUS_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_TORUS_CLASS_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_TORUS_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_TORUS_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_TORUS_CLASS_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_TORUS_CLASS_evalexpr_get) {
  {
    struct _dwg_object_ACSH_TORUS_CLASS *arg1 = (struct _dwg_object_ACSH_TORUS_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_TORUS_CLASS_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_TORUS_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_TORUS_CLASS_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_TORUS_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_TORUS_CLASS *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_TORUS_CLASS_history_node_set) {
  {
    struct _dwg_object_ACSH_TORUS_CLASS *arg1 = (struct _dwg_object_ACSH_TORUS_CLASS *) 0 ;
    Dwg_ACSH_HistoryNode *arg2 = (Dwg_ACSH_HistoryNode *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_TORUS_CLASS_history_node_set(self,history_node);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_TORUS_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_TORUS_CLASS_history_node_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_TORUS_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_TORUS_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_TORUS_CLASS_history_node_set" "', argument " "2"" of type '" "Dwg_ACSH_HistoryNode *""'"); 
    }
    arg2 = (Dwg_ACSH_HistoryNode *)(argp2);
    if (arg1) (arg1)->history_node = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_TORUS_CLASS_history_node_get) {
  {
    struct _dwg_object_ACSH_TORUS_CLASS *arg1 = (struct _dwg_object_ACSH_TORUS_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ACSH_HistoryNode *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_TORUS_CLASS_history_node_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_TORUS_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_TORUS_CLASS_history_node_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_TORUS_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_TORUS_CLASS *)(argp1);
    result = (Dwg_ACSH_HistoryNode *)& ((arg1)->history_node);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_TORUS_CLASS_major_set) {
  {
    struct _dwg_object_ACSH_TORUS_CLASS *arg1 = (struct _dwg_object_ACSH_TORUS_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_TORUS_CLASS_major_set(self,major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_TORUS_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_TORUS_CLASS_major_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_TORUS_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_TORUS_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_TORUS_CLASS_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_TORUS_CLASS_major_get) {
  {
    struct _dwg_object_ACSH_TORUS_CLASS *arg1 = (struct _dwg_object_ACSH_TORUS_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_TORUS_CLASS_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_TORUS_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_TORUS_CLASS_major_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_TORUS_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_TORUS_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_TORUS_CLASS_minor_set) {
  {
    struct _dwg_object_ACSH_TORUS_CLASS *arg1 = (struct _dwg_object_ACSH_TORUS_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_TORUS_CLASS_minor_set(self,minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_TORUS_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_TORUS_CLASS_minor_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_TORUS_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_TORUS_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_TORUS_CLASS_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_TORUS_CLASS_minor_get) {
  {
    struct _dwg_object_ACSH_TORUS_CLASS *arg1 = (struct _dwg_object_ACSH_TORUS_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_TORUS_CLASS_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_TORUS_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_TORUS_CLASS_minor_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_TORUS_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_TORUS_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_TORUS_CLASS_major_radius_set) {
  {
    struct _dwg_object_ACSH_TORUS_CLASS *arg1 = (struct _dwg_object_ACSH_TORUS_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_TORUS_CLASS_major_radius_set(self,major_radius);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_TORUS_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_TORUS_CLASS_major_radius_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_TORUS_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_TORUS_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_TORUS_CLASS_major_radius_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->major_radius = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_TORUS_CLASS_major_radius_get) {
  {
    struct _dwg_object_ACSH_TORUS_CLASS *arg1 = (struct _dwg_object_ACSH_TORUS_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_TORUS_CLASS_major_radius_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_TORUS_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_TORUS_CLASS_major_radius_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_TORUS_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_TORUS_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->major_radius);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_TORUS_CLASS_minor_radius_set) {
  {
    struct _dwg_object_ACSH_TORUS_CLASS *arg1 = (struct _dwg_object_ACSH_TORUS_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_TORUS_CLASS_minor_radius_set(self,minor_radius);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_TORUS_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_TORUS_CLASS_minor_radius_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_TORUS_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_TORUS_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_TORUS_CLASS_minor_radius_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->minor_radius = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_TORUS_CLASS_minor_radius_get) {
  {
    struct _dwg_object_ACSH_TORUS_CLASS *arg1 = (struct _dwg_object_ACSH_TORUS_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_TORUS_CLASS_minor_radius_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_TORUS_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_TORUS_CLASS_minor_radius_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_TORUS_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_TORUS_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->minor_radius);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ACSH_TORUS_CLASS) {
  {
    int argvi = 0;
    struct _dwg_object_ACSH_TORUS_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ACSH_TORUS_CLASS();");
    }
    result = (struct _dwg_object_ACSH_TORUS_CLASS *)calloc(1, sizeof(struct _dwg_object_ACSH_TORUS_CLASS));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_TORUS_CLASS, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ACSH_TORUS_CLASS) {
  {
    struct _dwg_object_ACSH_TORUS_CLASS *arg1 = (struct _dwg_object_ACSH_TORUS_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ACSH_TORUS_CLASS(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_TORUS_CLASS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ACSH_TORUS_CLASS" "', argument " "1"" of type '" "struct _dwg_object_ACSH_TORUS_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_TORUS_CLASS *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_REVOLVE_CLASS_parent_set) {
  {
    struct _dwg_object_ACSH_REVOLVE_CLASS *arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_REVOLVE_CLASS_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_REVOLVE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_REVOLVE_CLASS_parent_get) {
  {
    struct _dwg_object_ACSH_REVOLVE_CLASS *arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_REVOLVE_CLASS_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_REVOLVE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_REVOLVE_CLASS_evalexpr_set) {
  {
    struct _dwg_object_ACSH_REVOLVE_CLASS *arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_REVOLVE_CLASS_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_REVOLVE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_REVOLVE_CLASS_evalexpr_get) {
  {
    struct _dwg_object_ACSH_REVOLVE_CLASS *arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_REVOLVE_CLASS_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_REVOLVE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_REVOLVE_CLASS_history_node_set) {
  {
    struct _dwg_object_ACSH_REVOLVE_CLASS *arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *) 0 ;
    Dwg_ACSH_HistoryNode *arg2 = (Dwg_ACSH_HistoryNode *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_REVOLVE_CLASS_history_node_set(self,history_node);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_history_node_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_REVOLVE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_history_node_set" "', argument " "2"" of type '" "Dwg_ACSH_HistoryNode *""'"); 
    }
    arg2 = (Dwg_ACSH_HistoryNode *)(argp2);
    if (arg1) (arg1)->history_node = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_REVOLVE_CLASS_history_node_get) {
  {
    struct _dwg_object_ACSH_REVOLVE_CLASS *arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ACSH_HistoryNode *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_REVOLVE_CLASS_history_node_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_history_node_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_REVOLVE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *)(argp1);
    result = (Dwg_ACSH_HistoryNode *)& ((arg1)->history_node);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ACSH_HistoryNode, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_REVOLVE_CLASS_major_set) {
  {
    struct _dwg_object_ACSH_REVOLVE_CLASS *arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_REVOLVE_CLASS_major_set(self,major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_major_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_REVOLVE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_REVOLVE_CLASS_major_get) {
  {
    struct _dwg_object_ACSH_REVOLVE_CLASS *arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_REVOLVE_CLASS_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_major_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_REVOLVE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_REVOLVE_CLASS_minor_set) {
  {
    struct _dwg_object_ACSH_REVOLVE_CLASS *arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_REVOLVE_CLASS_minor_set(self,minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_minor_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_REVOLVE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_REVOLVE_CLASS_minor_get) {
  {
    struct _dwg_object_ACSH_REVOLVE_CLASS *arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_REVOLVE_CLASS_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_minor_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_REVOLVE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *)(argp1);
    result = (BITCODE_BL) ((arg1)->minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_REVOLVE_CLASS_axis_pt_set) {
  {
    struct _dwg_object_ACSH_REVOLVE_CLASS *arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_REVOLVE_CLASS_axis_pt_set(self,axis_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_axis_pt_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_REVOLVE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_axis_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->axis_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_REVOLVE_CLASS_axis_pt_get) {
  {
    struct _dwg_object_ACSH_REVOLVE_CLASS *arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_REVOLVE_CLASS_axis_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_axis_pt_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_REVOLVE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->axis_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_REVOLVE_CLASS_direction_set) {
  {
    struct _dwg_object_ACSH_REVOLVE_CLASS *arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_REVOLVE_CLASS_direction_set(self,direction);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_direction_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_REVOLVE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_direction_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->direction = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_REVOLVE_CLASS_direction_get) {
  {
    struct _dwg_object_ACSH_REVOLVE_CLASS *arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_REVOLVE_CLASS_direction_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_direction_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_REVOLVE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->direction);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_REVOLVE_CLASS_revolve_angle_set) {
  {
    struct _dwg_object_ACSH_REVOLVE_CLASS *arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_REVOLVE_CLASS_revolve_angle_set(self,revolve_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_revolve_angle_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_REVOLVE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_revolve_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->revolve_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_REVOLVE_CLASS_revolve_angle_get) {
  {
    struct _dwg_object_ACSH_REVOLVE_CLASS *arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_REVOLVE_CLASS_revolve_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_revolve_angle_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_REVOLVE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->revolve_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_REVOLVE_CLASS_start_angle_set) {
  {
    struct _dwg_object_ACSH_REVOLVE_CLASS *arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_REVOLVE_CLASS_start_angle_set(self,start_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_start_angle_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_REVOLVE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_start_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->start_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_REVOLVE_CLASS_start_angle_get) {
  {
    struct _dwg_object_ACSH_REVOLVE_CLASS *arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_REVOLVE_CLASS_start_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_start_angle_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_REVOLVE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->start_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_REVOLVE_CLASS_draft_angle_set) {
  {
    struct _dwg_object_ACSH_REVOLVE_CLASS *arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_REVOLVE_CLASS_draft_angle_set(self,draft_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_draft_angle_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_REVOLVE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_draft_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->draft_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_REVOLVE_CLASS_draft_angle_get) {
  {
    struct _dwg_object_ACSH_REVOLVE_CLASS *arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_REVOLVE_CLASS_draft_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_draft_angle_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_REVOLVE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->draft_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_REVOLVE_CLASS_bd44_set) {
  {
    struct _dwg_object_ACSH_REVOLVE_CLASS *arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_REVOLVE_CLASS_bd44_set(self,bd44);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_bd44_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_REVOLVE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_bd44_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->bd44 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_REVOLVE_CLASS_bd44_get) {
  {
    struct _dwg_object_ACSH_REVOLVE_CLASS *arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_REVOLVE_CLASS_bd44_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_bd44_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_REVOLVE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->bd44);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_REVOLVE_CLASS_bd45_set) {
  {
    struct _dwg_object_ACSH_REVOLVE_CLASS *arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_REVOLVE_CLASS_bd45_set(self,bd45);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_bd45_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_REVOLVE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_bd45_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->bd45 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_REVOLVE_CLASS_bd45_get) {
  {
    struct _dwg_object_ACSH_REVOLVE_CLASS *arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_REVOLVE_CLASS_bd45_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_bd45_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_REVOLVE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->bd45);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_REVOLVE_CLASS_twist_angle_set) {
  {
    struct _dwg_object_ACSH_REVOLVE_CLASS *arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_REVOLVE_CLASS_twist_angle_set(self,twist_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_twist_angle_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_REVOLVE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_twist_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->twist_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_REVOLVE_CLASS_twist_angle_get) {
  {
    struct _dwg_object_ACSH_REVOLVE_CLASS *arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_REVOLVE_CLASS_twist_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_twist_angle_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_REVOLVE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *)(argp1);
    result = (BITCODE_BD) ((arg1)->twist_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_REVOLVE_CLASS_b290_set) {
  {
    struct _dwg_object_ACSH_REVOLVE_CLASS *arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_REVOLVE_CLASS_b290_set(self,b290);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_b290_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_REVOLVE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_b290_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->b290 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_REVOLVE_CLASS_b290_get) {
  {
    struct _dwg_object_ACSH_REVOLVE_CLASS *arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_REVOLVE_CLASS_b290_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_b290_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_REVOLVE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *)(argp1);
    result = (BITCODE_B) ((arg1)->b290);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_REVOLVE_CLASS_is_close_to_axis_set) {
  {
    struct _dwg_object_ACSH_REVOLVE_CLASS *arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_REVOLVE_CLASS_is_close_to_axis_set(self,is_close_to_axis);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_is_close_to_axis_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_REVOLVE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_is_close_to_axis_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_close_to_axis = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_REVOLVE_CLASS_is_close_to_axis_get) {
  {
    struct _dwg_object_ACSH_REVOLVE_CLASS *arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_REVOLVE_CLASS_is_close_to_axis_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_is_close_to_axis_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_REVOLVE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *)(argp1);
    result = (BITCODE_B) ((arg1)->is_close_to_axis);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_REVOLVE_CLASS_sweep_entity_set) {
  {
    struct _dwg_object_ACSH_REVOLVE_CLASS *arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_REVOLVE_CLASS_sweep_entity_set(self,sweep_entity);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_sweep_entity_set" "', argument " "1"" of type '" "struct _dwg_object_ACSH_REVOLVE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_sweep_entity_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->sweep_entity = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACSH_REVOLVE_CLASS_sweep_entity_get) {
  {
    struct _dwg_object_ACSH_REVOLVE_CLASS *arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACSH_REVOLVE_CLASS_sweep_entity_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACSH_REVOLVE_CLASS_sweep_entity_get" "', argument " "1"" of type '" "struct _dwg_object_ACSH_REVOLVE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *)(argp1);
    result = (BITCODE_H) ((arg1)->sweep_entity);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ACSH_REVOLVE_CLASS) {
  {
    int argvi = 0;
    struct _dwg_object_ACSH_REVOLVE_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ACSH_REVOLVE_CLASS();");
    }
    result = (struct _dwg_object_ACSH_REVOLVE_CLASS *)calloc(1, sizeof(struct _dwg_object_ACSH_REVOLVE_CLASS));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ACSH_REVOLVE_CLASS) {
  {
    struct _dwg_object_ACSH_REVOLVE_CLASS *arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ACSH_REVOLVE_CLASS(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ACSH_REVOLVE_CLASS" "', argument " "1"" of type '" "struct _dwg_object_ACSH_REVOLVE_CLASS *""'"); 
    }
    arg1 = (struct _dwg_object_ACSH_REVOLVE_CLASS *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NAVISWORKSMODEL_parent_set) {
  {
    struct _dwg_entity_NAVISWORKSMODEL *arg1 = (struct _dwg_entity_NAVISWORKSMODEL *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NAVISWORKSMODEL_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NAVISWORKSMODEL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NAVISWORKSMODEL_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_NAVISWORKSMODEL *""'"); 
    }
    arg1 = (struct _dwg_entity_NAVISWORKSMODEL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_NAVISWORKSMODEL_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NAVISWORKSMODEL_parent_get) {
  {
    struct _dwg_entity_NAVISWORKSMODEL *arg1 = (struct _dwg_entity_NAVISWORKSMODEL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NAVISWORKSMODEL_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NAVISWORKSMODEL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NAVISWORKSMODEL_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_NAVISWORKSMODEL *""'"); 
    }
    arg1 = (struct _dwg_entity_NAVISWORKSMODEL *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NAVISWORKSMODEL_flags_set) {
  {
    struct _dwg_entity_NAVISWORKSMODEL *arg1 = (struct _dwg_entity_NAVISWORKSMODEL *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NAVISWORKSMODEL_flags_set(self,flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NAVISWORKSMODEL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NAVISWORKSMODEL_flags_set" "', argument " "1"" of type '" "struct _dwg_entity_NAVISWORKSMODEL *""'"); 
    }
    arg1 = (struct _dwg_entity_NAVISWORKSMODEL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_NAVISWORKSMODEL_flags_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NAVISWORKSMODEL_flags_get) {
  {
    struct _dwg_entity_NAVISWORKSMODEL *arg1 = (struct _dwg_entity_NAVISWORKSMODEL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NAVISWORKSMODEL_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NAVISWORKSMODEL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NAVISWORKSMODEL_flags_get" "', argument " "1"" of type '" "struct _dwg_entity_NAVISWORKSMODEL *""'"); 
    }
    arg1 = (struct _dwg_entity_NAVISWORKSMODEL *)(argp1);
    result = (BITCODE_BS) ((arg1)->flags);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NAVISWORKSMODEL_definition_set) {
  {
    struct _dwg_entity_NAVISWORKSMODEL *arg1 = (struct _dwg_entity_NAVISWORKSMODEL *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NAVISWORKSMODEL_definition_set(self,definition);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NAVISWORKSMODEL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NAVISWORKSMODEL_definition_set" "', argument " "1"" of type '" "struct _dwg_entity_NAVISWORKSMODEL *""'"); 
    }
    arg1 = (struct _dwg_entity_NAVISWORKSMODEL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_NAVISWORKSMODEL_definition_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->definition = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NAVISWORKSMODEL_definition_get) {
  {
    struct _dwg_entity_NAVISWORKSMODEL *arg1 = (struct _dwg_entity_NAVISWORKSMODEL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NAVISWORKSMODEL_definition_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NAVISWORKSMODEL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NAVISWORKSMODEL_definition_get" "', argument " "1"" of type '" "struct _dwg_entity_NAVISWORKSMODEL *""'"); 
    }
    arg1 = (struct _dwg_entity_NAVISWORKSMODEL *)(argp1);
    result = (BITCODE_H) ((arg1)->definition);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NAVISWORKSMODEL_transmatrix_set) {
  {
    struct _dwg_entity_NAVISWORKSMODEL *arg1 = (struct _dwg_entity_NAVISWORKSMODEL *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NAVISWORKSMODEL_transmatrix_set(self,transmatrix);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NAVISWORKSMODEL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NAVISWORKSMODEL_transmatrix_set" "', argument " "1"" of type '" "struct _dwg_entity_NAVISWORKSMODEL *""'"); 
    }
    arg1 = (struct _dwg_entity_NAVISWORKSMODEL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_NAVISWORKSMODEL_transmatrix_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->transmatrix = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NAVISWORKSMODEL_transmatrix_get) {
  {
    struct _dwg_entity_NAVISWORKSMODEL *arg1 = (struct _dwg_entity_NAVISWORKSMODEL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NAVISWORKSMODEL_transmatrix_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NAVISWORKSMODEL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NAVISWORKSMODEL_transmatrix_get" "', argument " "1"" of type '" "struct _dwg_entity_NAVISWORKSMODEL *""'"); 
    }
    arg1 = (struct _dwg_entity_NAVISWORKSMODEL *)(argp1);
    result = (BITCODE_BD *) ((arg1)->transmatrix);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NAVISWORKSMODEL_unitfactor_set) {
  {
    struct _dwg_entity_NAVISWORKSMODEL *arg1 = (struct _dwg_entity_NAVISWORKSMODEL *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_NAVISWORKSMODEL_unitfactor_set(self,unitfactor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NAVISWORKSMODEL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NAVISWORKSMODEL_unitfactor_set" "', argument " "1"" of type '" "struct _dwg_entity_NAVISWORKSMODEL *""'"); 
    }
    arg1 = (struct _dwg_entity_NAVISWORKSMODEL *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_NAVISWORKSMODEL_unitfactor_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->unitfactor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_NAVISWORKSMODEL_unitfactor_get) {
  {
    struct _dwg_entity_NAVISWORKSMODEL *arg1 = (struct _dwg_entity_NAVISWORKSMODEL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_NAVISWORKSMODEL_unitfactor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NAVISWORKSMODEL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_NAVISWORKSMODEL_unitfactor_get" "', argument " "1"" of type '" "struct _dwg_entity_NAVISWORKSMODEL *""'"); 
    }
    arg1 = (struct _dwg_entity_NAVISWORKSMODEL *)(argp1);
    result = (BITCODE_BD) ((arg1)->unitfactor);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_NAVISWORKSMODEL) {
  {
    int argvi = 0;
    struct _dwg_entity_NAVISWORKSMODEL *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_NAVISWORKSMODEL();");
    }
    result = (struct _dwg_entity_NAVISWORKSMODEL *)calloc(1, sizeof(struct _dwg_entity_NAVISWORKSMODEL));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_NAVISWORKSMODEL, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_NAVISWORKSMODEL) {
  {
    struct _dwg_entity_NAVISWORKSMODEL *arg1 = (struct _dwg_entity_NAVISWORKSMODEL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_NAVISWORKSMODEL(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_NAVISWORKSMODEL, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_NAVISWORKSMODEL" "', argument " "1"" of type '" "struct _dwg_entity_NAVISWORKSMODEL *""'"); 
    }
    arg1 = (struct _dwg_entity_NAVISWORKSMODEL *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_NAVISWORKSMODELDEF_parent_set) {
  {
    struct _dwg_object_NAVISWORKSMODELDEF *arg1 = (struct _dwg_object_NAVISWORKSMODELDEF *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_NAVISWORKSMODELDEF_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_NAVISWORKSMODELDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_NAVISWORKSMODELDEF_parent_set" "', argument " "1"" of type '" "struct _dwg_object_NAVISWORKSMODELDEF *""'"); 
    }
    arg1 = (struct _dwg_object_NAVISWORKSMODELDEF *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_NAVISWORKSMODELDEF_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_NAVISWORKSMODELDEF_parent_get) {
  {
    struct _dwg_object_NAVISWORKSMODELDEF *arg1 = (struct _dwg_object_NAVISWORKSMODELDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_NAVISWORKSMODELDEF_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_NAVISWORKSMODELDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_NAVISWORKSMODELDEF_parent_get" "', argument " "1"" of type '" "struct _dwg_object_NAVISWORKSMODELDEF *""'"); 
    }
    arg1 = (struct _dwg_object_NAVISWORKSMODELDEF *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_NAVISWORKSMODELDEF_flags_set) {
  {
    struct _dwg_object_NAVISWORKSMODELDEF *arg1 = (struct _dwg_object_NAVISWORKSMODELDEF *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_NAVISWORKSMODELDEF_flags_set(self,flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_NAVISWORKSMODELDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_NAVISWORKSMODELDEF_flags_set" "', argument " "1"" of type '" "struct _dwg_object_NAVISWORKSMODELDEF *""'"); 
    }
    arg1 = (struct _dwg_object_NAVISWORKSMODELDEF *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_NAVISWORKSMODELDEF_flags_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_NAVISWORKSMODELDEF_flags_get) {
  {
    struct _dwg_object_NAVISWORKSMODELDEF *arg1 = (struct _dwg_object_NAVISWORKSMODELDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_NAVISWORKSMODELDEF_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_NAVISWORKSMODELDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_NAVISWORKSMODELDEF_flags_get" "', argument " "1"" of type '" "struct _dwg_object_NAVISWORKSMODELDEF *""'"); 
    }
    arg1 = (struct _dwg_object_NAVISWORKSMODELDEF *)(argp1);
    result = (BITCODE_BS) ((arg1)->flags);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_NAVISWORKSMODELDEF_path_set) {
  {
    struct _dwg_object_NAVISWORKSMODELDEF *arg1 = (struct _dwg_object_NAVISWORKSMODELDEF *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_NAVISWORKSMODELDEF_path_set(self,path);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_NAVISWORKSMODELDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_NAVISWORKSMODELDEF_path_set" "', argument " "1"" of type '" "struct _dwg_object_NAVISWORKSMODELDEF *""'"); 
    }
    arg1 = (struct _dwg_object_NAVISWORKSMODELDEF *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_NAVISWORKSMODELDEF_path_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->path);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->path = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->path = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_NAVISWORKSMODELDEF_path_get) {
  {
    struct _dwg_object_NAVISWORKSMODELDEF *arg1 = (struct _dwg_object_NAVISWORKSMODELDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_NAVISWORKSMODELDEF_path_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_NAVISWORKSMODELDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_NAVISWORKSMODELDEF_path_get" "', argument " "1"" of type '" "struct _dwg_object_NAVISWORKSMODELDEF *""'"); 
    }
    arg1 = (struct _dwg_object_NAVISWORKSMODELDEF *)(argp1);
    result = (BITCODE_TV) ((arg1)->path);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_NAVISWORKSMODELDEF_status_set) {
  {
    struct _dwg_object_NAVISWORKSMODELDEF *arg1 = (struct _dwg_object_NAVISWORKSMODELDEF *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_NAVISWORKSMODELDEF_status_set(self,status);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_NAVISWORKSMODELDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_NAVISWORKSMODELDEF_status_set" "', argument " "1"" of type '" "struct _dwg_object_NAVISWORKSMODELDEF *""'"); 
    }
    arg1 = (struct _dwg_object_NAVISWORKSMODELDEF *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_NAVISWORKSMODELDEF_status_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->status = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_NAVISWORKSMODELDEF_status_get) {
  {
    struct _dwg_object_NAVISWORKSMODELDEF *arg1 = (struct _dwg_object_NAVISWORKSMODELDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_NAVISWORKSMODELDEF_status_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_NAVISWORKSMODELDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_NAVISWORKSMODELDEF_status_get" "', argument " "1"" of type '" "struct _dwg_object_NAVISWORKSMODELDEF *""'"); 
    }
    arg1 = (struct _dwg_object_NAVISWORKSMODELDEF *)(argp1);
    result = (BITCODE_B) ((arg1)->status);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_NAVISWORKSMODELDEF_min_extent_set) {
  {
    struct _dwg_object_NAVISWORKSMODELDEF *arg1 = (struct _dwg_object_NAVISWORKSMODELDEF *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_NAVISWORKSMODELDEF_min_extent_set(self,min_extent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_NAVISWORKSMODELDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_NAVISWORKSMODELDEF_min_extent_set" "', argument " "1"" of type '" "struct _dwg_object_NAVISWORKSMODELDEF *""'"); 
    }
    arg1 = (struct _dwg_object_NAVISWORKSMODELDEF *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_NAVISWORKSMODELDEF_min_extent_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->min_extent = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_NAVISWORKSMODELDEF_min_extent_get) {
  {
    struct _dwg_object_NAVISWORKSMODELDEF *arg1 = (struct _dwg_object_NAVISWORKSMODELDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_NAVISWORKSMODELDEF_min_extent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_NAVISWORKSMODELDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_NAVISWORKSMODELDEF_min_extent_get" "', argument " "1"" of type '" "struct _dwg_object_NAVISWORKSMODELDEF *""'"); 
    }
    arg1 = (struct _dwg_object_NAVISWORKSMODELDEF *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->min_extent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_NAVISWORKSMODELDEF_max_extent_set) {
  {
    struct _dwg_object_NAVISWORKSMODELDEF *arg1 = (struct _dwg_object_NAVISWORKSMODELDEF *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_NAVISWORKSMODELDEF_max_extent_set(self,max_extent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_NAVISWORKSMODELDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_NAVISWORKSMODELDEF_max_extent_set" "', argument " "1"" of type '" "struct _dwg_object_NAVISWORKSMODELDEF *""'"); 
    }
    arg1 = (struct _dwg_object_NAVISWORKSMODELDEF *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_NAVISWORKSMODELDEF_max_extent_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->max_extent = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_NAVISWORKSMODELDEF_max_extent_get) {
  {
    struct _dwg_object_NAVISWORKSMODELDEF *arg1 = (struct _dwg_object_NAVISWORKSMODELDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_NAVISWORKSMODELDEF_max_extent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_NAVISWORKSMODELDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_NAVISWORKSMODELDEF_max_extent_get" "', argument " "1"" of type '" "struct _dwg_object_NAVISWORKSMODELDEF *""'"); 
    }
    arg1 = (struct _dwg_object_NAVISWORKSMODELDEF *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->max_extent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_NAVISWORKSMODELDEF_host_drawing_visibility_set) {
  {
    struct _dwg_object_NAVISWORKSMODELDEF *arg1 = (struct _dwg_object_NAVISWORKSMODELDEF *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_NAVISWORKSMODELDEF_host_drawing_visibility_set(self,host_drawing_visibility);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_NAVISWORKSMODELDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_NAVISWORKSMODELDEF_host_drawing_visibility_set" "', argument " "1"" of type '" "struct _dwg_object_NAVISWORKSMODELDEF *""'"); 
    }
    arg1 = (struct _dwg_object_NAVISWORKSMODELDEF *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_NAVISWORKSMODELDEF_host_drawing_visibility_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->host_drawing_visibility = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_NAVISWORKSMODELDEF_host_drawing_visibility_get) {
  {
    struct _dwg_object_NAVISWORKSMODELDEF *arg1 = (struct _dwg_object_NAVISWORKSMODELDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_NAVISWORKSMODELDEF_host_drawing_visibility_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_NAVISWORKSMODELDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_NAVISWORKSMODELDEF_host_drawing_visibility_get" "', argument " "1"" of type '" "struct _dwg_object_NAVISWORKSMODELDEF *""'"); 
    }
    arg1 = (struct _dwg_object_NAVISWORKSMODELDEF *)(argp1);
    result = (BITCODE_B) ((arg1)->host_drawing_visibility);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_NAVISWORKSMODELDEF) {
  {
    int argvi = 0;
    struct _dwg_object_NAVISWORKSMODELDEF *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_NAVISWORKSMODELDEF();");
    }
    result = (struct _dwg_object_NAVISWORKSMODELDEF *)calloc(1, sizeof(struct _dwg_object_NAVISWORKSMODELDEF));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_NAVISWORKSMODELDEF, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_NAVISWORKSMODELDEF) {
  {
    struct _dwg_object_NAVISWORKSMODELDEF *arg1 = (struct _dwg_object_NAVISWORKSMODELDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_NAVISWORKSMODELDEF(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_NAVISWORKSMODELDEF, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_NAVISWORKSMODELDEF" "', argument " "1"" of type '" "struct _dwg_object_NAVISWORKSMODELDEF *""'"); 
    }
    arg1 = (struct _dwg_object_NAVISWORKSMODELDEF *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERSETTINGS_parent_set) {
  {
    struct _dwg_object_RENDERSETTINGS *arg1 = (struct _dwg_object_RENDERSETTINGS *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERSETTINGS_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERSETTINGS_parent_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERSETTINGS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_RENDERSETTINGS_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERSETTINGS_parent_get) {
  {
    struct _dwg_object_RENDERSETTINGS *arg1 = (struct _dwg_object_RENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERSETTINGS_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERSETTINGS_parent_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERSETTINGS *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERSETTINGS_class_version_set) {
  {
    struct _dwg_object_RENDERSETTINGS *arg1 = (struct _dwg_object_RENDERSETTINGS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERSETTINGS_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERSETTINGS_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERSETTINGS_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERSETTINGS_class_version_get) {
  {
    struct _dwg_object_RENDERSETTINGS *arg1 = (struct _dwg_object_RENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERSETTINGS_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERSETTINGS_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERSETTINGS *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERSETTINGS_name_set) {
  {
    struct _dwg_object_RENDERSETTINGS *arg1 = (struct _dwg_object_RENDERSETTINGS *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERSETTINGS_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERSETTINGS_name_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERSETTINGS *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_RENDERSETTINGS_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERSETTINGS_name_get) {
  {
    struct _dwg_object_RENDERSETTINGS *arg1 = (struct _dwg_object_RENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERSETTINGS_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERSETTINGS_name_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERSETTINGS *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERSETTINGS_fog_enabled_set) {
  {
    struct _dwg_object_RENDERSETTINGS *arg1 = (struct _dwg_object_RENDERSETTINGS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERSETTINGS_fog_enabled_set(self,fog_enabled);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERSETTINGS_fog_enabled_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERSETTINGS_fog_enabled_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->fog_enabled = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERSETTINGS_fog_enabled_get) {
  {
    struct _dwg_object_RENDERSETTINGS *arg1 = (struct _dwg_object_RENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERSETTINGS_fog_enabled_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERSETTINGS_fog_enabled_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERSETTINGS *)(argp1);
    result = (BITCODE_B) ((arg1)->fog_enabled);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERSETTINGS_fog_background_enabled_set) {
  {
    struct _dwg_object_RENDERSETTINGS *arg1 = (struct _dwg_object_RENDERSETTINGS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERSETTINGS_fog_background_enabled_set(self,fog_background_enabled);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERSETTINGS_fog_background_enabled_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERSETTINGS_fog_background_enabled_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->fog_background_enabled = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERSETTINGS_fog_background_enabled_get) {
  {
    struct _dwg_object_RENDERSETTINGS *arg1 = (struct _dwg_object_RENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERSETTINGS_fog_background_enabled_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERSETTINGS_fog_background_enabled_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERSETTINGS *)(argp1);
    result = (BITCODE_B) ((arg1)->fog_background_enabled);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERSETTINGS_backfaces_enabled_set) {
  {
    struct _dwg_object_RENDERSETTINGS *arg1 = (struct _dwg_object_RENDERSETTINGS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERSETTINGS_backfaces_enabled_set(self,backfaces_enabled);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERSETTINGS_backfaces_enabled_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERSETTINGS_backfaces_enabled_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->backfaces_enabled = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERSETTINGS_backfaces_enabled_get) {
  {
    struct _dwg_object_RENDERSETTINGS *arg1 = (struct _dwg_object_RENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERSETTINGS_backfaces_enabled_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERSETTINGS_backfaces_enabled_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERSETTINGS *)(argp1);
    result = (BITCODE_B) ((arg1)->backfaces_enabled);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERSETTINGS_environ_image_enabled_set) {
  {
    struct _dwg_object_RENDERSETTINGS *arg1 = (struct _dwg_object_RENDERSETTINGS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERSETTINGS_environ_image_enabled_set(self,environ_image_enabled);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERSETTINGS_environ_image_enabled_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERSETTINGS_environ_image_enabled_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->environ_image_enabled = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERSETTINGS_environ_image_enabled_get) {
  {
    struct _dwg_object_RENDERSETTINGS *arg1 = (struct _dwg_object_RENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERSETTINGS_environ_image_enabled_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERSETTINGS_environ_image_enabled_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERSETTINGS *)(argp1);
    result = (BITCODE_B) ((arg1)->environ_image_enabled);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERSETTINGS_environ_image_filename_set) {
  {
    struct _dwg_object_RENDERSETTINGS *arg1 = (struct _dwg_object_RENDERSETTINGS *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERSETTINGS_environ_image_filename_set(self,environ_image_filename);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERSETTINGS_environ_image_filename_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERSETTINGS *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_RENDERSETTINGS_environ_image_filename_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->environ_image_filename);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->environ_image_filename = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->environ_image_filename = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERSETTINGS_environ_image_filename_get) {
  {
    struct _dwg_object_RENDERSETTINGS *arg1 = (struct _dwg_object_RENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERSETTINGS_environ_image_filename_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERSETTINGS_environ_image_filename_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERSETTINGS *)(argp1);
    result = (BITCODE_TV) ((arg1)->environ_image_filename);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERSETTINGS_description_set) {
  {
    struct _dwg_object_RENDERSETTINGS *arg1 = (struct _dwg_object_RENDERSETTINGS *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERSETTINGS_description_set(self,description);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERSETTINGS_description_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERSETTINGS *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_RENDERSETTINGS_description_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->description);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->description = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->description = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERSETTINGS_description_get) {
  {
    struct _dwg_object_RENDERSETTINGS *arg1 = (struct _dwg_object_RENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERSETTINGS_description_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERSETTINGS_description_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERSETTINGS *)(argp1);
    result = (BITCODE_TV) ((arg1)->description);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERSETTINGS_display_index_set) {
  {
    struct _dwg_object_RENDERSETTINGS *arg1 = (struct _dwg_object_RENDERSETTINGS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERSETTINGS_display_index_set(self,display_index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERSETTINGS_display_index_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERSETTINGS_display_index_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->display_index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERSETTINGS_display_index_get) {
  {
    struct _dwg_object_RENDERSETTINGS *arg1 = (struct _dwg_object_RENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERSETTINGS_display_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERSETTINGS_display_index_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERSETTINGS *)(argp1);
    result = (BITCODE_BL) ((arg1)->display_index);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERSETTINGS_has_predefined_set) {
  {
    struct _dwg_object_RENDERSETTINGS *arg1 = (struct _dwg_object_RENDERSETTINGS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERSETTINGS_has_predefined_set(self,has_predefined);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERSETTINGS_has_predefined_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERSETTINGS_has_predefined_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_predefined = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERSETTINGS_has_predefined_get) {
  {
    struct _dwg_object_RENDERSETTINGS *arg1 = (struct _dwg_object_RENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERSETTINGS_has_predefined_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERSETTINGS_has_predefined_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERSETTINGS *)(argp1);
    result = (BITCODE_B) ((arg1)->has_predefined);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_RENDERSETTINGS) {
  {
    int argvi = 0;
    struct _dwg_object_RENDERSETTINGS *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_RENDERSETTINGS();");
    }
    result = (struct _dwg_object_RENDERSETTINGS *)calloc(1, sizeof(struct _dwg_object_RENDERSETTINGS));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_RENDERSETTINGS, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_RENDERSETTINGS) {
  {
    struct _dwg_object_RENDERSETTINGS *arg1 = (struct _dwg_object_RENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_RENDERSETTINGS(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERSETTINGS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_RENDERSETTINGS" "', argument " "1"" of type '" "struct _dwg_object_RENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERSETTINGS *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_parent_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_parent_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_parent_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_parent_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_class_version_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_class_version_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_name_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_name_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_name_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_name_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_fog_enabled_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_fog_enabled_set(self,fog_enabled);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_fog_enabled_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_fog_enabled_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->fog_enabled = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_fog_enabled_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_fog_enabled_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_fog_enabled_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_B) ((arg1)->fog_enabled);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_fog_background_enabled_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_fog_background_enabled_set(self,fog_background_enabled);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_fog_background_enabled_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_fog_background_enabled_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->fog_background_enabled = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_fog_background_enabled_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_fog_background_enabled_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_fog_background_enabled_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_B) ((arg1)->fog_background_enabled);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_backfaces_enabled_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_backfaces_enabled_set(self,backfaces_enabled);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_backfaces_enabled_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_backfaces_enabled_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->backfaces_enabled = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_backfaces_enabled_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_backfaces_enabled_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_backfaces_enabled_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_B) ((arg1)->backfaces_enabled);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_environ_image_enabled_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_environ_image_enabled_set(self,environ_image_enabled);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_environ_image_enabled_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_environ_image_enabled_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->environ_image_enabled = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_environ_image_enabled_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_environ_image_enabled_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_environ_image_enabled_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_B) ((arg1)->environ_image_enabled);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_environ_image_filename_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_environ_image_filename_set(self,environ_image_filename);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_environ_image_filename_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_environ_image_filename_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->environ_image_filename);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->environ_image_filename = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->environ_image_filename = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_environ_image_filename_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_environ_image_filename_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_environ_image_filename_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_TV) ((arg1)->environ_image_filename);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_description_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_description_set(self,description);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_description_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_description_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->description);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->description = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->description = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_description_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_description_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_description_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_TV) ((arg1)->description);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_display_index_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_display_index_set(self,display_index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_display_index_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_display_index_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->display_index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_display_index_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_display_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_display_index_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_BL) ((arg1)->display_index);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_has_predefined_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_has_predefined_set(self,has_predefined);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_has_predefined_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_has_predefined_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_predefined = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_has_predefined_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_has_predefined_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_has_predefined_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_B) ((arg1)->has_predefined);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_mr_version_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_mr_version_set(self,mr_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_mr_version_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_mr_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->mr_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_mr_version_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_mr_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_mr_version_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_BL) ((arg1)->mr_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling1_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_sampling1_set(self,sampling1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_sampling1_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_sampling1_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->sampling1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling1_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_sampling1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_sampling1_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_BL) ((arg1)->sampling1);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling2_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_sampling2_set(self,sampling2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_sampling2_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_sampling2_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->sampling2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling2_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_sampling2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_sampling2_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_BL) ((arg1)->sampling2);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_mr_filter_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_mr_filter_set(self,sampling_mr_filter);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_mr_filter_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_mr_filter_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->sampling_mr_filter = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_mr_filter_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_mr_filter_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_mr_filter_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_BS) ((arg1)->sampling_mr_filter);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_filter1_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_filter1_set(self,sampling_filter1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_filter1_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_filter1_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->sampling_filter1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_filter1_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_filter1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_filter1_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_BD) ((arg1)->sampling_filter1);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_filter2_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_filter2_set(self,sampling_filter2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_filter2_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_filter2_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->sampling_filter2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_filter2_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_filter2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_filter2_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_BD) ((arg1)->sampling_filter2);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color1_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color1_set(self,sampling_contrast_color1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color1_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color1_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->sampling_contrast_color1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color1_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color1_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_BD) ((arg1)->sampling_contrast_color1);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color2_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color2_set(self,sampling_contrast_color2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color2_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color2_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->sampling_contrast_color2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color2_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color2_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_BD) ((arg1)->sampling_contrast_color2);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color3_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color3_set(self,sampling_contrast_color3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color3_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color3_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->sampling_contrast_color3 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color3_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color3_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_BD) ((arg1)->sampling_contrast_color3);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color4_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color4_set(self,sampling_contrast_color4);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color4_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color4_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->sampling_contrast_color4 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color4_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color4_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color4_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_BD) ((arg1)->sampling_contrast_color4);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_shadow_mode_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_shadow_mode_set(self,shadow_mode);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_shadow_mode_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_shadow_mode_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->shadow_mode = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_shadow_mode_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_shadow_mode_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_shadow_mode_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_BS) ((arg1)->shadow_mode);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_shadow_maps_enabled_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_shadow_maps_enabled_set(self,shadow_maps_enabled);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_shadow_maps_enabled_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_shadow_maps_enabled_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->shadow_maps_enabled = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_shadow_maps_enabled_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_shadow_maps_enabled_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_shadow_maps_enabled_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_B) ((arg1)->shadow_maps_enabled);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_ray_tracing_enabled_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_ray_tracing_enabled_set(self,ray_tracing_enabled);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_ray_tracing_enabled_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_ray_tracing_enabled_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->ray_tracing_enabled = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_ray_tracing_enabled_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_ray_tracing_enabled_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_ray_tracing_enabled_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_B) ((arg1)->ray_tracing_enabled);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth1_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth1_set(self,ray_trace_depth1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth1_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth1_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->ray_trace_depth1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth1_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth1_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_BL) ((arg1)->ray_trace_depth1);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth2_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth2_set(self,ray_trace_depth2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth2_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth2_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->ray_trace_depth2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth2_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth2_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_BL) ((arg1)->ray_trace_depth2);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth3_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth3_set(self,ray_trace_depth3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth3_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth3_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->ray_trace_depth3 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth3_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth3_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_BL) ((arg1)->ray_trace_depth3);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_global_illumination_enabled_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_global_illumination_enabled_set(self,global_illumination_enabled);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_global_illumination_enabled_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_global_illumination_enabled_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->global_illumination_enabled = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_global_illumination_enabled_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_global_illumination_enabled_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_global_illumination_enabled_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_B) ((arg1)->global_illumination_enabled);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_count_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_count_set(self,gi_sample_count);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_count_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_count_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->gi_sample_count = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_count_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_count_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_count_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_BL) ((arg1)->gi_sample_count);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_radius_enabled_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_radius_enabled_set(self,gi_sample_radius_enabled);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_radius_enabled_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_radius_enabled_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->gi_sample_radius_enabled = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_radius_enabled_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_radius_enabled_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_radius_enabled_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_B) ((arg1)->gi_sample_radius_enabled);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_radius_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_radius_set(self,gi_sample_radius);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_radius_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_radius_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->gi_sample_radius = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_radius_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_radius_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_radius_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_BD) ((arg1)->gi_sample_radius);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_gi_photons_per_light_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_gi_photons_per_light_set(self,gi_photons_per_light);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_gi_photons_per_light_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_gi_photons_per_light_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->gi_photons_per_light = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_gi_photons_per_light_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_gi_photons_per_light_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_gi_photons_per_light_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_BL) ((arg1)->gi_photons_per_light);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth1_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth1_set(self,photon_trace_depth1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth1_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth1_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->photon_trace_depth1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth1_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth1_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_BL) ((arg1)->photon_trace_depth1);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth2_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth2_set(self,photon_trace_depth2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth2_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth2_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->photon_trace_depth2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth2_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth2_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_BL) ((arg1)->photon_trace_depth2);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth3_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth3_set(self,photon_trace_depth3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth3_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth3_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->photon_trace_depth3 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth3_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth3_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_BL) ((arg1)->photon_trace_depth3);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_final_gathering_enabled_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_final_gathering_enabled_set(self,final_gathering_enabled);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_final_gathering_enabled_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_final_gathering_enabled_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->final_gathering_enabled = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_final_gathering_enabled_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_final_gathering_enabled_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_final_gathering_enabled_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_B) ((arg1)->final_gathering_enabled);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_fg_ray_count_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_fg_ray_count_set(self,fg_ray_count);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_fg_ray_count_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_fg_ray_count_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->fg_ray_count = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_fg_ray_count_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_fg_ray_count_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_fg_ray_count_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_BL) ((arg1)->fg_ray_count);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state1_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state1_set(self,fg_sample_radius_state1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state1_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state1_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->fg_sample_radius_state1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state1_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state1_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_B) ((arg1)->fg_sample_radius_state1);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state2_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state2_set(self,fg_sample_radius_state2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state2_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state2_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->fg_sample_radius_state2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state2_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state2_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_B) ((arg1)->fg_sample_radius_state2);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state3_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state3_set(self,fg_sample_radius_state3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state3_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state3_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->fg_sample_radius_state3 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state3_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state3_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_B) ((arg1)->fg_sample_radius_state3);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius1_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius1_set(self,fg_sample_radius1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius1_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius1_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->fg_sample_radius1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius1_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius1_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_BD) ((arg1)->fg_sample_radius1);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius2_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius2_set(self,fg_sample_radius2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius2_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius2_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->fg_sample_radius2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius2_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius2_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_BD) ((arg1)->fg_sample_radius2);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_light_luminance_scale_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_light_luminance_scale_set(self,light_luminance_scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_light_luminance_scale_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_light_luminance_scale_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->light_luminance_scale = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_light_luminance_scale_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_light_luminance_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_light_luminance_scale_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_BD) ((arg1)->light_luminance_scale);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_mode_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_mode_set(self,diagnostics_mode);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_mode_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_mode_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->diagnostics_mode = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_mode_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_mode_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_mode_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_BS) ((arg1)->diagnostics_mode);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_grid_mode_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_grid_mode_set(self,diagnostics_grid_mode);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_grid_mode_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_grid_mode_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->diagnostics_grid_mode = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_grid_mode_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_grid_mode_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_grid_mode_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_BS) ((arg1)->diagnostics_grid_mode);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_grid_float_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_grid_float_set(self,diagnostics_grid_float);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_grid_float_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_grid_float_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->diagnostics_grid_float = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_grid_float_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_grid_float_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_grid_float_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_BD) ((arg1)->diagnostics_grid_float);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_photon_mode_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_photon_mode_set(self,diagnostics_photon_mode);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_photon_mode_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_photon_mode_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->diagnostics_photon_mode = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_photon_mode_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_photon_mode_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_photon_mode_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_BS) ((arg1)->diagnostics_photon_mode);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_bsp_mode_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_bsp_mode_set(self,diagnostics_bsp_mode);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_bsp_mode_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_bsp_mode_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->diagnostics_bsp_mode = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_bsp_mode_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_bsp_mode_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_bsp_mode_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_BS) ((arg1)->diagnostics_bsp_mode);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_export_mi_enabled_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_export_mi_enabled_set(self,export_mi_enabled);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_export_mi_enabled_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_export_mi_enabled_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->export_mi_enabled = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_export_mi_enabled_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_export_mi_enabled_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_export_mi_enabled_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_B) ((arg1)->export_mi_enabled);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_mr_description_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_mr_description_set(self,mr_description);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_mr_description_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_mr_description_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->mr_description);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->mr_description = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->mr_description = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_mr_description_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_mr_description_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_mr_description_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_TV) ((arg1)->mr_description);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_tile_size_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_tile_size_set(self,tile_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_tile_size_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_tile_size_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->tile_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_tile_size_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_tile_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_tile_size_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_BL) ((arg1)->tile_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_tile_order_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_tile_order_set(self,tile_order);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_tile_order_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_tile_order_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->tile_order = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_tile_order_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_tile_order_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_tile_order_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_BS) ((arg1)->tile_order);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_memory_limit_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_memory_limit_set(self,memory_limit);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_memory_limit_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_memory_limit_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->memory_limit = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_memory_limit_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_memory_limit_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_memory_limit_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_BL) ((arg1)->memory_limit);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_samples_mode_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_samples_mode_set(self,diagnostics_samples_mode);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_samples_mode_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_samples_mode_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->diagnostics_samples_mode = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_samples_mode_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_samples_mode_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_samples_mode_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_B) ((arg1)->diagnostics_samples_mode);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_energy_multiplier_set) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_energy_multiplier_set(self,energy_multiplier);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_energy_multiplier_set" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_energy_multiplier_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->energy_multiplier = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_energy_multiplier_get) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MENTALRAYRENDERSETTINGS_energy_multiplier_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MENTALRAYRENDERSETTINGS_energy_multiplier_get" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    result = (BITCODE_BD) ((arg1)->energy_multiplier);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_MENTALRAYRENDERSETTINGS) {
  {
    int argvi = 0;
    struct _dwg_object_MENTALRAYRENDERSETTINGS *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_MENTALRAYRENDERSETTINGS();");
    }
    result = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)calloc(1, sizeof(struct _dwg_object_MENTALRAYRENDERSETTINGS));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_MENTALRAYRENDERSETTINGS) {
  {
    struct _dwg_object_MENTALRAYRENDERSETTINGS *arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_MENTALRAYRENDERSETTINGS(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_MENTALRAYRENDERSETTINGS" "', argument " "1"" of type '" "struct _dwg_object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_MENTALRAYRENDERSETTINGS *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_parent_set) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_parent_set" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_parent_get) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_parent_get" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_class_version_set) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_class_version_get) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_name_set) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_name_set" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_name_get) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_name_get" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_fog_enabled_set) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_fog_enabled_set(self,fog_enabled);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_fog_enabled_set" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_fog_enabled_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->fog_enabled = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_fog_enabled_get) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_fog_enabled_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_fog_enabled_get" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    result = (BITCODE_B) ((arg1)->fog_enabled);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_fog_background_enabled_set) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_fog_background_enabled_set(self,fog_background_enabled);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_fog_background_enabled_set" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_fog_background_enabled_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->fog_background_enabled = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_fog_background_enabled_get) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_fog_background_enabled_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_fog_background_enabled_get" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    result = (BITCODE_B) ((arg1)->fog_background_enabled);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_backfaces_enabled_set) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_backfaces_enabled_set(self,backfaces_enabled);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_backfaces_enabled_set" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_backfaces_enabled_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->backfaces_enabled = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_backfaces_enabled_get) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_backfaces_enabled_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_backfaces_enabled_get" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    result = (BITCODE_B) ((arg1)->backfaces_enabled);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_environ_image_enabled_set) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_environ_image_enabled_set(self,environ_image_enabled);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_environ_image_enabled_set" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_environ_image_enabled_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->environ_image_enabled = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_environ_image_enabled_get) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_environ_image_enabled_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_environ_image_enabled_get" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    result = (BITCODE_B) ((arg1)->environ_image_enabled);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_environ_image_filename_set) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_environ_image_filename_set(self,environ_image_filename);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_environ_image_filename_set" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_environ_image_filename_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->environ_image_filename);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->environ_image_filename = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->environ_image_filename = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_environ_image_filename_get) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_environ_image_filename_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_environ_image_filename_get" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    result = (BITCODE_TV) ((arg1)->environ_image_filename);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_description_set) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_description_set(self,description);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_description_set" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_description_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->description);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->description = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->description = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_description_get) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_description_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_description_get" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    result = (BITCODE_TV) ((arg1)->description);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_display_index_set) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_display_index_set(self,display_index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_display_index_set" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_display_index_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->display_index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_display_index_get) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_display_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_display_index_get" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    result = (BITCODE_BL) ((arg1)->display_index);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_has_predefined_set) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_has_predefined_set(self,has_predefined);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_has_predefined_set" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_has_predefined_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_predefined = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_has_predefined_get) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_has_predefined_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_has_predefined_get" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    result = (BITCODE_B) ((arg1)->has_predefined);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_rapidrt_version_set) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_rapidrt_version_set(self,rapidrt_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_rapidrt_version_set" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_rapidrt_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->rapidrt_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_rapidrt_version_get) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_rapidrt_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_rapidrt_version_get" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    result = (BITCODE_BL) ((arg1)->rapidrt_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_render_target_set) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_render_target_set(self,render_target);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_render_target_set" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_render_target_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->render_target = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_render_target_get) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_render_target_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_render_target_get" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    result = (BITCODE_BL) ((arg1)->render_target);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_render_level_set) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_render_level_set(self,render_level);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_render_level_set" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_render_level_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->render_level = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_render_level_get) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_render_level_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_render_level_get" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    result = (BITCODE_BL) ((arg1)->render_level);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_render_time_set) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_render_time_set(self,render_time);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_render_time_set" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_render_time_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->render_time = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_render_time_get) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_render_time_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_render_time_get" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    result = (BITCODE_BL) ((arg1)->render_time);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_lighting_model_set) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_lighting_model_set(self,lighting_model);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_lighting_model_set" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_lighting_model_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->lighting_model = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_lighting_model_get) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_lighting_model_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_lighting_model_get" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    result = (BITCODE_BL) ((arg1)->lighting_model);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_filter_type_set) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_filter_type_set(self,filter_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_filter_type_set" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_filter_type_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->filter_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_filter_type_get) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_filter_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_filter_type_get" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    result = (BITCODE_BL) ((arg1)->filter_type);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_filter_width_set) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_filter_width_set(self,filter_width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_filter_width_set" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_filter_width_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->filter_width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_filter_width_get) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_filter_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_filter_width_get" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    result = (BITCODE_BD) ((arg1)->filter_width);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_filter_height_set) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_filter_height_set(self,filter_height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_filter_height_set" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_filter_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->filter_height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_filter_height_get) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RAPIDRTRENDERSETTINGS_filter_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RAPIDRTRENDERSETTINGS_filter_height_get" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    result = (BITCODE_BD) ((arg1)->filter_height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_RAPIDRTRENDERSETTINGS) {
  {
    int argvi = 0;
    struct _dwg_object_RAPIDRTRENDERSETTINGS *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_RAPIDRTRENDERSETTINGS();");
    }
    result = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)calloc(1, sizeof(struct _dwg_object_RAPIDRTRENDERSETTINGS));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_RAPIDRTRENDERSETTINGS) {
  {
    struct _dwg_object_RAPIDRTRENDERSETTINGS *arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_RAPIDRTRENDERSETTINGS(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_RAPIDRTRENDERSETTINGS" "', argument " "1"" of type '" "struct _dwg_object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_RAPIDRTRENDERSETTINGS *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENVIRONMENT_parent_set) {
  {
    struct _dwg_object_RENDERENVIRONMENT *arg1 = (struct _dwg_object_RENDERENVIRONMENT *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENVIRONMENT_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENVIRONMENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENVIRONMENT_parent_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERENVIRONMENT *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENVIRONMENT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_RENDERENVIRONMENT_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENVIRONMENT_parent_get) {
  {
    struct _dwg_object_RENDERENVIRONMENT *arg1 = (struct _dwg_object_RENDERENVIRONMENT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENVIRONMENT_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENVIRONMENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENVIRONMENT_parent_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERENVIRONMENT *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENVIRONMENT *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENVIRONMENT_class_version_set) {
  {
    struct _dwg_object_RENDERENVIRONMENT *arg1 = (struct _dwg_object_RENDERENVIRONMENT *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENVIRONMENT_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENVIRONMENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENVIRONMENT_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERENVIRONMENT *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENVIRONMENT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERENVIRONMENT_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENVIRONMENT_class_version_get) {
  {
    struct _dwg_object_RENDERENVIRONMENT *arg1 = (struct _dwg_object_RENDERENVIRONMENT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENVIRONMENT_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENVIRONMENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENVIRONMENT_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERENVIRONMENT *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENVIRONMENT *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENVIRONMENT_fog_enabled_set) {
  {
    struct _dwg_object_RENDERENVIRONMENT *arg1 = (struct _dwg_object_RENDERENVIRONMENT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENVIRONMENT_fog_enabled_set(self,fog_enabled);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENVIRONMENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENVIRONMENT_fog_enabled_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERENVIRONMENT *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENVIRONMENT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERENVIRONMENT_fog_enabled_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->fog_enabled = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENVIRONMENT_fog_enabled_get) {
  {
    struct _dwg_object_RENDERENVIRONMENT *arg1 = (struct _dwg_object_RENDERENVIRONMENT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENVIRONMENT_fog_enabled_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENVIRONMENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENVIRONMENT_fog_enabled_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERENVIRONMENT *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENVIRONMENT *)(argp1);
    result = (BITCODE_B) ((arg1)->fog_enabled);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENVIRONMENT_fog_background_enabled_set) {
  {
    struct _dwg_object_RENDERENVIRONMENT *arg1 = (struct _dwg_object_RENDERENVIRONMENT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENVIRONMENT_fog_background_enabled_set(self,fog_background_enabled);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENVIRONMENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENVIRONMENT_fog_background_enabled_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERENVIRONMENT *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENVIRONMENT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERENVIRONMENT_fog_background_enabled_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->fog_background_enabled = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENVIRONMENT_fog_background_enabled_get) {
  {
    struct _dwg_object_RENDERENVIRONMENT *arg1 = (struct _dwg_object_RENDERENVIRONMENT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENVIRONMENT_fog_background_enabled_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENVIRONMENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENVIRONMENT_fog_background_enabled_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERENVIRONMENT *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENVIRONMENT *)(argp1);
    result = (BITCODE_B) ((arg1)->fog_background_enabled);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENVIRONMENT_fog_color_r_set) {
  {
    struct _dwg_object_RENDERENVIRONMENT *arg1 = (struct _dwg_object_RENDERENVIRONMENT *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENVIRONMENT_fog_color_r_set(self,fog_color_r);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENVIRONMENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENVIRONMENT_fog_color_r_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERENVIRONMENT *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENVIRONMENT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERENVIRONMENT_fog_color_r_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->fog_color_r = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENVIRONMENT_fog_color_r_get) {
  {
    struct _dwg_object_RENDERENVIRONMENT *arg1 = (struct _dwg_object_RENDERENVIRONMENT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENVIRONMENT_fog_color_r_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENVIRONMENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENVIRONMENT_fog_color_r_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERENVIRONMENT *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENVIRONMENT *)(argp1);
    result = (BITCODE_RC) ((arg1)->fog_color_r);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENVIRONMENT_fog_color_g_set) {
  {
    struct _dwg_object_RENDERENVIRONMENT *arg1 = (struct _dwg_object_RENDERENVIRONMENT *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENVIRONMENT_fog_color_g_set(self,fog_color_g);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENVIRONMENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENVIRONMENT_fog_color_g_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERENVIRONMENT *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENVIRONMENT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERENVIRONMENT_fog_color_g_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->fog_color_g = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENVIRONMENT_fog_color_g_get) {
  {
    struct _dwg_object_RENDERENVIRONMENT *arg1 = (struct _dwg_object_RENDERENVIRONMENT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENVIRONMENT_fog_color_g_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENVIRONMENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENVIRONMENT_fog_color_g_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERENVIRONMENT *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENVIRONMENT *)(argp1);
    result = (BITCODE_RC) ((arg1)->fog_color_g);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENVIRONMENT_fog_color_b_set) {
  {
    struct _dwg_object_RENDERENVIRONMENT *arg1 = (struct _dwg_object_RENDERENVIRONMENT *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENVIRONMENT_fog_color_b_set(self,fog_color_b);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENVIRONMENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENVIRONMENT_fog_color_b_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERENVIRONMENT *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENVIRONMENT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERENVIRONMENT_fog_color_b_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->fog_color_b = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENVIRONMENT_fog_color_b_get) {
  {
    struct _dwg_object_RENDERENVIRONMENT *arg1 = (struct _dwg_object_RENDERENVIRONMENT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENVIRONMENT_fog_color_b_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENVIRONMENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENVIRONMENT_fog_color_b_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERENVIRONMENT *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENVIRONMENT *)(argp1);
    result = (BITCODE_RC) ((arg1)->fog_color_b);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENVIRONMENT_fog_density_near_set) {
  {
    struct _dwg_object_RENDERENVIRONMENT *arg1 = (struct _dwg_object_RENDERENVIRONMENT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENVIRONMENT_fog_density_near_set(self,fog_density_near);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENVIRONMENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENVIRONMENT_fog_density_near_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERENVIRONMENT *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENVIRONMENT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERENVIRONMENT_fog_density_near_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->fog_density_near = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENVIRONMENT_fog_density_near_get) {
  {
    struct _dwg_object_RENDERENVIRONMENT *arg1 = (struct _dwg_object_RENDERENVIRONMENT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENVIRONMENT_fog_density_near_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENVIRONMENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENVIRONMENT_fog_density_near_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERENVIRONMENT *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENVIRONMENT *)(argp1);
    result = (BITCODE_BD) ((arg1)->fog_density_near);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENVIRONMENT_fog_density_far_set) {
  {
    struct _dwg_object_RENDERENVIRONMENT *arg1 = (struct _dwg_object_RENDERENVIRONMENT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENVIRONMENT_fog_density_far_set(self,fog_density_far);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENVIRONMENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENVIRONMENT_fog_density_far_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERENVIRONMENT *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENVIRONMENT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERENVIRONMENT_fog_density_far_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->fog_density_far = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENVIRONMENT_fog_density_far_get) {
  {
    struct _dwg_object_RENDERENVIRONMENT *arg1 = (struct _dwg_object_RENDERENVIRONMENT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENVIRONMENT_fog_density_far_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENVIRONMENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENVIRONMENT_fog_density_far_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERENVIRONMENT *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENVIRONMENT *)(argp1);
    result = (BITCODE_BD) ((arg1)->fog_density_far);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENVIRONMENT_fog_distance_near_set) {
  {
    struct _dwg_object_RENDERENVIRONMENT *arg1 = (struct _dwg_object_RENDERENVIRONMENT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENVIRONMENT_fog_distance_near_set(self,fog_distance_near);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENVIRONMENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENVIRONMENT_fog_distance_near_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERENVIRONMENT *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENVIRONMENT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERENVIRONMENT_fog_distance_near_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->fog_distance_near = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENVIRONMENT_fog_distance_near_get) {
  {
    struct _dwg_object_RENDERENVIRONMENT *arg1 = (struct _dwg_object_RENDERENVIRONMENT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENVIRONMENT_fog_distance_near_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENVIRONMENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENVIRONMENT_fog_distance_near_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERENVIRONMENT *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENVIRONMENT *)(argp1);
    result = (BITCODE_BD) ((arg1)->fog_distance_near);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENVIRONMENT_fog_distance_far_set) {
  {
    struct _dwg_object_RENDERENVIRONMENT *arg1 = (struct _dwg_object_RENDERENVIRONMENT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENVIRONMENT_fog_distance_far_set(self,fog_distance_far);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENVIRONMENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENVIRONMENT_fog_distance_far_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERENVIRONMENT *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENVIRONMENT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERENVIRONMENT_fog_distance_far_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->fog_distance_far = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENVIRONMENT_fog_distance_far_get) {
  {
    struct _dwg_object_RENDERENVIRONMENT *arg1 = (struct _dwg_object_RENDERENVIRONMENT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENVIRONMENT_fog_distance_far_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENVIRONMENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENVIRONMENT_fog_distance_far_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERENVIRONMENT *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENVIRONMENT *)(argp1);
    result = (BITCODE_BD) ((arg1)->fog_distance_far);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENVIRONMENT_environ_image_enabled_set) {
  {
    struct _dwg_object_RENDERENVIRONMENT *arg1 = (struct _dwg_object_RENDERENVIRONMENT *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENVIRONMENT_environ_image_enabled_set(self,environ_image_enabled);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENVIRONMENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENVIRONMENT_environ_image_enabled_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERENVIRONMENT *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENVIRONMENT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERENVIRONMENT_environ_image_enabled_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->environ_image_enabled = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENVIRONMENT_environ_image_enabled_get) {
  {
    struct _dwg_object_RENDERENVIRONMENT *arg1 = (struct _dwg_object_RENDERENVIRONMENT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENVIRONMENT_environ_image_enabled_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENVIRONMENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENVIRONMENT_environ_image_enabled_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERENVIRONMENT *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENVIRONMENT *)(argp1);
    result = (BITCODE_B) ((arg1)->environ_image_enabled);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENVIRONMENT_environ_image_filename_set) {
  {
    struct _dwg_object_RENDERENVIRONMENT *arg1 = (struct _dwg_object_RENDERENVIRONMENT *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENVIRONMENT_environ_image_filename_set(self,environ_image_filename);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENVIRONMENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENVIRONMENT_environ_image_filename_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERENVIRONMENT *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENVIRONMENT *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_RENDERENVIRONMENT_environ_image_filename_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->environ_image_filename);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->environ_image_filename = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->environ_image_filename = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENVIRONMENT_environ_image_filename_get) {
  {
    struct _dwg_object_RENDERENVIRONMENT *arg1 = (struct _dwg_object_RENDERENVIRONMENT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENVIRONMENT_environ_image_filename_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENVIRONMENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENVIRONMENT_environ_image_filename_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERENVIRONMENT *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENVIRONMENT *)(argp1);
    result = (BITCODE_TV) ((arg1)->environ_image_filename);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_RENDERENVIRONMENT) {
  {
    int argvi = 0;
    struct _dwg_object_RENDERENVIRONMENT *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_RENDERENVIRONMENT();");
    }
    result = (struct _dwg_object_RENDERENVIRONMENT *)calloc(1, sizeof(struct _dwg_object_RENDERENVIRONMENT));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_RENDERENVIRONMENT, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_RENDERENVIRONMENT) {
  {
    struct _dwg_object_RENDERENVIRONMENT *arg1 = (struct _dwg_object_RENDERENVIRONMENT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_RENDERENVIRONMENT(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENVIRONMENT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_RENDERENVIRONMENT" "', argument " "1"" of type '" "struct _dwg_object_RENDERENVIRONMENT *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENVIRONMENT *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERGLOBAL_parent_set) {
  {
    struct _dwg_object_RENDERGLOBAL *arg1 = (struct _dwg_object_RENDERGLOBAL *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERGLOBAL_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERGLOBAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERGLOBAL_parent_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERGLOBAL *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERGLOBAL *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_RENDERGLOBAL_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERGLOBAL_parent_get) {
  {
    struct _dwg_object_RENDERGLOBAL *arg1 = (struct _dwg_object_RENDERGLOBAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERGLOBAL_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERGLOBAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERGLOBAL_parent_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERGLOBAL *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERGLOBAL *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERGLOBAL_class_version_set) {
  {
    struct _dwg_object_RENDERGLOBAL *arg1 = (struct _dwg_object_RENDERGLOBAL *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERGLOBAL_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERGLOBAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERGLOBAL_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERGLOBAL *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERGLOBAL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERGLOBAL_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERGLOBAL_class_version_get) {
  {
    struct _dwg_object_RENDERGLOBAL *arg1 = (struct _dwg_object_RENDERGLOBAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERGLOBAL_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERGLOBAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERGLOBAL_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERGLOBAL *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERGLOBAL *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERGLOBAL_procedure_set) {
  {
    struct _dwg_object_RENDERGLOBAL *arg1 = (struct _dwg_object_RENDERGLOBAL *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERGLOBAL_procedure_set(self,procedure);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERGLOBAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERGLOBAL_procedure_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERGLOBAL *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERGLOBAL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERGLOBAL_procedure_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->procedure = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERGLOBAL_procedure_get) {
  {
    struct _dwg_object_RENDERGLOBAL *arg1 = (struct _dwg_object_RENDERGLOBAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERGLOBAL_procedure_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERGLOBAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERGLOBAL_procedure_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERGLOBAL *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERGLOBAL *)(argp1);
    result = (BITCODE_BL) ((arg1)->procedure);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERGLOBAL_destination_set) {
  {
    struct _dwg_object_RENDERGLOBAL *arg1 = (struct _dwg_object_RENDERGLOBAL *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERGLOBAL_destination_set(self,destination);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERGLOBAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERGLOBAL_destination_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERGLOBAL *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERGLOBAL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERGLOBAL_destination_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->destination = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERGLOBAL_destination_get) {
  {
    struct _dwg_object_RENDERGLOBAL *arg1 = (struct _dwg_object_RENDERGLOBAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERGLOBAL_destination_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERGLOBAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERGLOBAL_destination_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERGLOBAL *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERGLOBAL *)(argp1);
    result = (BITCODE_BL) ((arg1)->destination);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERGLOBAL_save_enabled_set) {
  {
    struct _dwg_object_RENDERGLOBAL *arg1 = (struct _dwg_object_RENDERGLOBAL *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERGLOBAL_save_enabled_set(self,save_enabled);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERGLOBAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERGLOBAL_save_enabled_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERGLOBAL *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERGLOBAL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERGLOBAL_save_enabled_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->save_enabled = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERGLOBAL_save_enabled_get) {
  {
    struct _dwg_object_RENDERGLOBAL *arg1 = (struct _dwg_object_RENDERGLOBAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERGLOBAL_save_enabled_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERGLOBAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERGLOBAL_save_enabled_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERGLOBAL *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERGLOBAL *)(argp1);
    result = (BITCODE_B) ((arg1)->save_enabled);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERGLOBAL_save_filename_set) {
  {
    struct _dwg_object_RENDERGLOBAL *arg1 = (struct _dwg_object_RENDERGLOBAL *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERGLOBAL_save_filename_set(self,save_filename);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERGLOBAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERGLOBAL_save_filename_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERGLOBAL *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERGLOBAL *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_RENDERGLOBAL_save_filename_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->save_filename);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->save_filename = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->save_filename = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERGLOBAL_save_filename_get) {
  {
    struct _dwg_object_RENDERGLOBAL *arg1 = (struct _dwg_object_RENDERGLOBAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERGLOBAL_save_filename_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERGLOBAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERGLOBAL_save_filename_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERGLOBAL *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERGLOBAL *)(argp1);
    result = (BITCODE_TV) ((arg1)->save_filename);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERGLOBAL_image_width_set) {
  {
    struct _dwg_object_RENDERGLOBAL *arg1 = (struct _dwg_object_RENDERGLOBAL *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERGLOBAL_image_width_set(self,image_width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERGLOBAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERGLOBAL_image_width_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERGLOBAL *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERGLOBAL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERGLOBAL_image_width_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->image_width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERGLOBAL_image_width_get) {
  {
    struct _dwg_object_RENDERGLOBAL *arg1 = (struct _dwg_object_RENDERGLOBAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERGLOBAL_image_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERGLOBAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERGLOBAL_image_width_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERGLOBAL *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERGLOBAL *)(argp1);
    result = (BITCODE_BL) ((arg1)->image_width);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERGLOBAL_image_height_set) {
  {
    struct _dwg_object_RENDERGLOBAL *arg1 = (struct _dwg_object_RENDERGLOBAL *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERGLOBAL_image_height_set(self,image_height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERGLOBAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERGLOBAL_image_height_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERGLOBAL *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERGLOBAL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERGLOBAL_image_height_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->image_height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERGLOBAL_image_height_get) {
  {
    struct _dwg_object_RENDERGLOBAL *arg1 = (struct _dwg_object_RENDERGLOBAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERGLOBAL_image_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERGLOBAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERGLOBAL_image_height_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERGLOBAL *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERGLOBAL *)(argp1);
    result = (BITCODE_BL) ((arg1)->image_height);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERGLOBAL_predef_presets_first_set) {
  {
    struct _dwg_object_RENDERGLOBAL *arg1 = (struct _dwg_object_RENDERGLOBAL *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERGLOBAL_predef_presets_first_set(self,predef_presets_first);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERGLOBAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERGLOBAL_predef_presets_first_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERGLOBAL *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERGLOBAL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERGLOBAL_predef_presets_first_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->predef_presets_first = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERGLOBAL_predef_presets_first_get) {
  {
    struct _dwg_object_RENDERGLOBAL *arg1 = (struct _dwg_object_RENDERGLOBAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERGLOBAL_predef_presets_first_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERGLOBAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERGLOBAL_predef_presets_first_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERGLOBAL *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERGLOBAL *)(argp1);
    result = (BITCODE_B) ((arg1)->predef_presets_first);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERGLOBAL_highlevel_info_set) {
  {
    struct _dwg_object_RENDERGLOBAL *arg1 = (struct _dwg_object_RENDERGLOBAL *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERGLOBAL_highlevel_info_set(self,highlevel_info);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERGLOBAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERGLOBAL_highlevel_info_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERGLOBAL *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERGLOBAL *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERGLOBAL_highlevel_info_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->highlevel_info = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERGLOBAL_highlevel_info_get) {
  {
    struct _dwg_object_RENDERGLOBAL *arg1 = (struct _dwg_object_RENDERGLOBAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERGLOBAL_highlevel_info_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERGLOBAL, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERGLOBAL_highlevel_info_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERGLOBAL *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERGLOBAL *)(argp1);
    result = (BITCODE_B) ((arg1)->highlevel_info);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_RENDERGLOBAL) {
  {
    int argvi = 0;
    struct _dwg_object_RENDERGLOBAL *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_RENDERGLOBAL();");
    }
    result = (struct _dwg_object_RENDERGLOBAL *)calloc(1, sizeof(struct _dwg_object_RENDERGLOBAL));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_RENDERGLOBAL, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_RENDERGLOBAL) {
  {
    struct _dwg_object_RENDERGLOBAL *arg1 = (struct _dwg_object_RENDERGLOBAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_RENDERGLOBAL(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERGLOBAL, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_RENDERGLOBAL" "', argument " "1"" of type '" "struct _dwg_object_RENDERGLOBAL *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERGLOBAL *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_parent_set) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_parent_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_RENDERENTRY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_parent_get) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_parent_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_class_version_set) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERENTRY_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_class_version_get) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_image_file_name_set) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_image_file_name_set(self,image_file_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_image_file_name_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_RENDERENTRY_image_file_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->image_file_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->image_file_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->image_file_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_image_file_name_get) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_image_file_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_image_file_name_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    result = (BITCODE_TV) ((arg1)->image_file_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_preset_name_set) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_preset_name_set(self,preset_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_preset_name_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_RENDERENTRY_preset_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->preset_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->preset_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->preset_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_preset_name_get) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_preset_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_preset_name_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    result = (BITCODE_TV) ((arg1)->preset_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_view_name_set) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_view_name_set(self,view_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_view_name_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_RENDERENTRY_view_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->view_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->view_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->view_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_view_name_get) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_view_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_view_name_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    result = (BITCODE_TV) ((arg1)->view_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_dimension_x_set) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_dimension_x_set(self,dimension_x);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_dimension_x_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERENTRY_dimension_x_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->dimension_x = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_dimension_x_get) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_dimension_x_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_dimension_x_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    result = (BITCODE_BL) ((arg1)->dimension_x);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_dimension_y_set) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_dimension_y_set(self,dimension_y);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_dimension_y_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERENTRY_dimension_y_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->dimension_y = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_dimension_y_get) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_dimension_y_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_dimension_y_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    result = (BITCODE_BL) ((arg1)->dimension_y);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_start_year_set) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_start_year_set(self,start_year);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_start_year_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERENTRY_start_year_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->start_year = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_start_year_get) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_start_year_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_start_year_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    result = (BITCODE_BS) ((arg1)->start_year);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_start_month_set) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_start_month_set(self,start_month);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_start_month_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERENTRY_start_month_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->start_month = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_start_month_get) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_start_month_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_start_month_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    result = (BITCODE_BS) ((arg1)->start_month);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_start_day_set) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_start_day_set(self,start_day);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_start_day_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERENTRY_start_day_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->start_day = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_start_day_get) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_start_day_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_start_day_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    result = (BITCODE_BS) ((arg1)->start_day);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_start_minute_set) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_start_minute_set(self,start_minute);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_start_minute_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERENTRY_start_minute_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->start_minute = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_start_minute_get) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_start_minute_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_start_minute_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    result = (BITCODE_BS) ((arg1)->start_minute);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_start_second_set) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_start_second_set(self,start_second);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_start_second_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERENTRY_start_second_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->start_second = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_start_second_get) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_start_second_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_start_second_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    result = (BITCODE_BS) ((arg1)->start_second);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_start_msec_set) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_start_msec_set(self,start_msec);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_start_msec_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERENTRY_start_msec_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->start_msec = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_start_msec_get) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_start_msec_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_start_msec_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    result = (BITCODE_BS) ((arg1)->start_msec);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_render_time_set) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_render_time_set(self,render_time);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_render_time_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERENTRY_render_time_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->render_time = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_render_time_get) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_render_time_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_render_time_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    result = (BITCODE_BD) ((arg1)->render_time);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_memory_amount_set) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_memory_amount_set(self,memory_amount);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_memory_amount_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERENTRY_memory_amount_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->memory_amount = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_memory_amount_get) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_memory_amount_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_memory_amount_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    result = (BITCODE_BL) ((arg1)->memory_amount);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_material_count_set) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_material_count_set(self,material_count);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_material_count_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERENTRY_material_count_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->material_count = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_material_count_get) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_material_count_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_material_count_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    result = (BITCODE_BL) ((arg1)->material_count);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_light_count_set) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_light_count_set(self,light_count);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_light_count_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERENTRY_light_count_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->light_count = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_light_count_get) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_light_count_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_light_count_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    result = (BITCODE_BL) ((arg1)->light_count);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_triangle_count_set) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_triangle_count_set(self,triangle_count);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_triangle_count_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERENTRY_triangle_count_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->triangle_count = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_triangle_count_get) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_triangle_count_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_triangle_count_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    result = (BITCODE_BL) ((arg1)->triangle_count);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_display_index_set) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_display_index_set(self,display_index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_display_index_set" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RENDERENTRY_display_index_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->display_index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RENDERENTRY_display_index_get) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RENDERENTRY_display_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RENDERENTRY_display_index_get" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    result = (BITCODE_BL) ((arg1)->display_index);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_RENDERENTRY) {
  {
    int argvi = 0;
    struct _dwg_object_RENDERENTRY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_RENDERENTRY();");
    }
    result = (struct _dwg_object_RENDERENTRY *)calloc(1, sizeof(struct _dwg_object_RENDERENTRY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_RENDERENTRY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_RENDERENTRY) {
  {
    struct _dwg_object_RENDERENTRY *arg1 = (struct _dwg_object_RENDERENTRY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_RENDERENTRY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RENDERENTRY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_RENDERENTRY" "', argument " "1"" of type '" "struct _dwg_object_RENDERENTRY *""'"); 
    }
    arg1 = (struct _dwg_object_RENDERENTRY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MOTIONPATH_parent_set) {
  {
    struct _dwg_object_MOTIONPATH *arg1 = (struct _dwg_object_MOTIONPATH *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MOTIONPATH_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MOTIONPATH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MOTIONPATH_parent_set" "', argument " "1"" of type '" "struct _dwg_object_MOTIONPATH *""'"); 
    }
    arg1 = (struct _dwg_object_MOTIONPATH *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MOTIONPATH_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MOTIONPATH_parent_get) {
  {
    struct _dwg_object_MOTIONPATH *arg1 = (struct _dwg_object_MOTIONPATH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MOTIONPATH_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MOTIONPATH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MOTIONPATH_parent_get" "', argument " "1"" of type '" "struct _dwg_object_MOTIONPATH *""'"); 
    }
    arg1 = (struct _dwg_object_MOTIONPATH *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MOTIONPATH_class_version_set) {
  {
    struct _dwg_object_MOTIONPATH *arg1 = (struct _dwg_object_MOTIONPATH *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MOTIONPATH_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MOTIONPATH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MOTIONPATH_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_MOTIONPATH *""'"); 
    }
    arg1 = (struct _dwg_object_MOTIONPATH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MOTIONPATH_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MOTIONPATH_class_version_get) {
  {
    struct _dwg_object_MOTIONPATH *arg1 = (struct _dwg_object_MOTIONPATH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MOTIONPATH_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MOTIONPATH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MOTIONPATH_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_MOTIONPATH *""'"); 
    }
    arg1 = (struct _dwg_object_MOTIONPATH *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MOTIONPATH_camera_path_set) {
  {
    struct _dwg_object_MOTIONPATH *arg1 = (struct _dwg_object_MOTIONPATH *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MOTIONPATH_camera_path_set(self,camera_path);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MOTIONPATH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MOTIONPATH_camera_path_set" "', argument " "1"" of type '" "struct _dwg_object_MOTIONPATH *""'"); 
    }
    arg1 = (struct _dwg_object_MOTIONPATH *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MOTIONPATH_camera_path_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->camera_path = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MOTIONPATH_camera_path_get) {
  {
    struct _dwg_object_MOTIONPATH *arg1 = (struct _dwg_object_MOTIONPATH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MOTIONPATH_camera_path_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MOTIONPATH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MOTIONPATH_camera_path_get" "', argument " "1"" of type '" "struct _dwg_object_MOTIONPATH *""'"); 
    }
    arg1 = (struct _dwg_object_MOTIONPATH *)(argp1);
    result = (BITCODE_H) ((arg1)->camera_path);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MOTIONPATH_target_path_set) {
  {
    struct _dwg_object_MOTIONPATH *arg1 = (struct _dwg_object_MOTIONPATH *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MOTIONPATH_target_path_set(self,target_path);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MOTIONPATH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MOTIONPATH_target_path_set" "', argument " "1"" of type '" "struct _dwg_object_MOTIONPATH *""'"); 
    }
    arg1 = (struct _dwg_object_MOTIONPATH *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MOTIONPATH_target_path_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->target_path = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MOTIONPATH_target_path_get) {
  {
    struct _dwg_object_MOTIONPATH *arg1 = (struct _dwg_object_MOTIONPATH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MOTIONPATH_target_path_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MOTIONPATH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MOTIONPATH_target_path_get" "', argument " "1"" of type '" "struct _dwg_object_MOTIONPATH *""'"); 
    }
    arg1 = (struct _dwg_object_MOTIONPATH *)(argp1);
    result = (BITCODE_H) ((arg1)->target_path);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MOTIONPATH_viewtable_set) {
  {
    struct _dwg_object_MOTIONPATH *arg1 = (struct _dwg_object_MOTIONPATH *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MOTIONPATH_viewtable_set(self,viewtable);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MOTIONPATH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MOTIONPATH_viewtable_set" "', argument " "1"" of type '" "struct _dwg_object_MOTIONPATH *""'"); 
    }
    arg1 = (struct _dwg_object_MOTIONPATH *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MOTIONPATH_viewtable_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->viewtable = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MOTIONPATH_viewtable_get) {
  {
    struct _dwg_object_MOTIONPATH *arg1 = (struct _dwg_object_MOTIONPATH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MOTIONPATH_viewtable_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MOTIONPATH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MOTIONPATH_viewtable_get" "', argument " "1"" of type '" "struct _dwg_object_MOTIONPATH *""'"); 
    }
    arg1 = (struct _dwg_object_MOTIONPATH *)(argp1);
    result = (BITCODE_H) ((arg1)->viewtable);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MOTIONPATH_frames_set) {
  {
    struct _dwg_object_MOTIONPATH *arg1 = (struct _dwg_object_MOTIONPATH *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MOTIONPATH_frames_set(self,frames);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MOTIONPATH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MOTIONPATH_frames_set" "', argument " "1"" of type '" "struct _dwg_object_MOTIONPATH *""'"); 
    }
    arg1 = (struct _dwg_object_MOTIONPATH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MOTIONPATH_frames_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->frames = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MOTIONPATH_frames_get) {
  {
    struct _dwg_object_MOTIONPATH *arg1 = (struct _dwg_object_MOTIONPATH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MOTIONPATH_frames_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MOTIONPATH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MOTIONPATH_frames_get" "', argument " "1"" of type '" "struct _dwg_object_MOTIONPATH *""'"); 
    }
    arg1 = (struct _dwg_object_MOTIONPATH *)(argp1);
    result = (BITCODE_BS) ((arg1)->frames);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MOTIONPATH_frame_rate_set) {
  {
    struct _dwg_object_MOTIONPATH *arg1 = (struct _dwg_object_MOTIONPATH *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MOTIONPATH_frame_rate_set(self,frame_rate);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MOTIONPATH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MOTIONPATH_frame_rate_set" "', argument " "1"" of type '" "struct _dwg_object_MOTIONPATH *""'"); 
    }
    arg1 = (struct _dwg_object_MOTIONPATH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MOTIONPATH_frame_rate_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->frame_rate = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MOTIONPATH_frame_rate_get) {
  {
    struct _dwg_object_MOTIONPATH *arg1 = (struct _dwg_object_MOTIONPATH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MOTIONPATH_frame_rate_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MOTIONPATH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MOTIONPATH_frame_rate_get" "', argument " "1"" of type '" "struct _dwg_object_MOTIONPATH *""'"); 
    }
    arg1 = (struct _dwg_object_MOTIONPATH *)(argp1);
    result = (BITCODE_BS) ((arg1)->frame_rate);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MOTIONPATH_corner_decel_set) {
  {
    struct _dwg_object_MOTIONPATH *arg1 = (struct _dwg_object_MOTIONPATH *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MOTIONPATH_corner_decel_set(self,corner_decel);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MOTIONPATH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MOTIONPATH_corner_decel_set" "', argument " "1"" of type '" "struct _dwg_object_MOTIONPATH *""'"); 
    }
    arg1 = (struct _dwg_object_MOTIONPATH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MOTIONPATH_corner_decel_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->corner_decel = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MOTIONPATH_corner_decel_get) {
  {
    struct _dwg_object_MOTIONPATH *arg1 = (struct _dwg_object_MOTIONPATH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MOTIONPATH_corner_decel_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MOTIONPATH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MOTIONPATH_corner_decel_get" "', argument " "1"" of type '" "struct _dwg_object_MOTIONPATH *""'"); 
    }
    arg1 = (struct _dwg_object_MOTIONPATH *)(argp1);
    result = (BITCODE_B) ((arg1)->corner_decel);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_MOTIONPATH) {
  {
    int argvi = 0;
    struct _dwg_object_MOTIONPATH *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_MOTIONPATH();");
    }
    result = (struct _dwg_object_MOTIONPATH *)calloc(1, sizeof(struct _dwg_object_MOTIONPATH));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_MOTIONPATH, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_MOTIONPATH) {
  {
    struct _dwg_object_MOTIONPATH *arg1 = (struct _dwg_object_MOTIONPATH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_MOTIONPATH(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MOTIONPATH, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_MOTIONPATH" "', argument " "1"" of type '" "struct _dwg_object_MOTIONPATH *""'"); 
    }
    arg1 = (struct _dwg_object_MOTIONPATH *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_CURVEPATH_parent_set) {
  {
    struct _dwg_object_CURVEPATH *arg1 = (struct _dwg_object_CURVEPATH *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_CURVEPATH_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_CURVEPATH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_CURVEPATH_parent_set" "', argument " "1"" of type '" "struct _dwg_object_CURVEPATH *""'"); 
    }
    arg1 = (struct _dwg_object_CURVEPATH *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_CURVEPATH_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_CURVEPATH_parent_get) {
  {
    struct _dwg_object_CURVEPATH *arg1 = (struct _dwg_object_CURVEPATH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_CURVEPATH_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_CURVEPATH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_CURVEPATH_parent_get" "', argument " "1"" of type '" "struct _dwg_object_CURVEPATH *""'"); 
    }
    arg1 = (struct _dwg_object_CURVEPATH *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_CURVEPATH_class_version_set) {
  {
    struct _dwg_object_CURVEPATH *arg1 = (struct _dwg_object_CURVEPATH *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_CURVEPATH_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_CURVEPATH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_CURVEPATH_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_CURVEPATH *""'"); 
    }
    arg1 = (struct _dwg_object_CURVEPATH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_CURVEPATH_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_CURVEPATH_class_version_get) {
  {
    struct _dwg_object_CURVEPATH *arg1 = (struct _dwg_object_CURVEPATH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_CURVEPATH_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_CURVEPATH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_CURVEPATH_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_CURVEPATH *""'"); 
    }
    arg1 = (struct _dwg_object_CURVEPATH *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_CURVEPATH_entity_set) {
  {
    struct _dwg_object_CURVEPATH *arg1 = (struct _dwg_object_CURVEPATH *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_CURVEPATH_entity_set(self,entity);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_CURVEPATH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_CURVEPATH_entity_set" "', argument " "1"" of type '" "struct _dwg_object_CURVEPATH *""'"); 
    }
    arg1 = (struct _dwg_object_CURVEPATH *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_CURVEPATH_entity_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->entity = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_CURVEPATH_entity_get) {
  {
    struct _dwg_object_CURVEPATH *arg1 = (struct _dwg_object_CURVEPATH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_CURVEPATH_entity_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_CURVEPATH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_CURVEPATH_entity_get" "', argument " "1"" of type '" "struct _dwg_object_CURVEPATH *""'"); 
    }
    arg1 = (struct _dwg_object_CURVEPATH *)(argp1);
    result = (BITCODE_H) ((arg1)->entity);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_CURVEPATH) {
  {
    int argvi = 0;
    struct _dwg_object_CURVEPATH *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_CURVEPATH();");
    }
    result = (struct _dwg_object_CURVEPATH *)calloc(1, sizeof(struct _dwg_object_CURVEPATH));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_CURVEPATH, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_CURVEPATH) {
  {
    struct _dwg_object_CURVEPATH *arg1 = (struct _dwg_object_CURVEPATH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_CURVEPATH(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_CURVEPATH, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_CURVEPATH" "', argument " "1"" of type '" "struct _dwg_object_CURVEPATH *""'"); 
    }
    arg1 = (struct _dwg_object_CURVEPATH *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTPATH_parent_set) {
  {
    struct _dwg_object_POINTPATH *arg1 = (struct _dwg_object_POINTPATH *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_POINTPATH_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTPATH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTPATH_parent_set" "', argument " "1"" of type '" "struct _dwg_object_POINTPATH *""'"); 
    }
    arg1 = (struct _dwg_object_POINTPATH *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_POINTPATH_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTPATH_parent_get) {
  {
    struct _dwg_object_POINTPATH *arg1 = (struct _dwg_object_POINTPATH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_POINTPATH_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTPATH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTPATH_parent_get" "', argument " "1"" of type '" "struct _dwg_object_POINTPATH *""'"); 
    }
    arg1 = (struct _dwg_object_POINTPATH *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTPATH_class_version_set) {
  {
    struct _dwg_object_POINTPATH *arg1 = (struct _dwg_object_POINTPATH *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_POINTPATH_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTPATH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTPATH_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_POINTPATH *""'"); 
    }
    arg1 = (struct _dwg_object_POINTPATH *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_POINTPATH_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTPATH_class_version_get) {
  {
    struct _dwg_object_POINTPATH *arg1 = (struct _dwg_object_POINTPATH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_POINTPATH_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTPATH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTPATH_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_POINTPATH *""'"); 
    }
    arg1 = (struct _dwg_object_POINTPATH *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTPATH_point_set) {
  {
    struct _dwg_object_POINTPATH *arg1 = (struct _dwg_object_POINTPATH *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_POINTPATH_point_set(self,point);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTPATH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTPATH_point_set" "', argument " "1"" of type '" "struct _dwg_object_POINTPATH *""'"); 
    }
    arg1 = (struct _dwg_object_POINTPATH *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_POINTPATH_point_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->point = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTPATH_point_get) {
  {
    struct _dwg_object_POINTPATH *arg1 = (struct _dwg_object_POINTPATH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_POINTPATH_point_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTPATH, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTPATH_point_get" "', argument " "1"" of type '" "struct _dwg_object_POINTPATH *""'"); 
    }
    arg1 = (struct _dwg_object_POINTPATH *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->point);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_POINTPATH) {
  {
    int argvi = 0;
    struct _dwg_object_POINTPATH *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_POINTPATH();");
    }
    result = (struct _dwg_object_POINTPATH *)calloc(1, sizeof(struct _dwg_object_POINTPATH));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_POINTPATH, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_POINTPATH) {
  {
    struct _dwg_object_POINTPATH *arg1 = (struct _dwg_object_POINTPATH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_POINTPATH(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTPATH, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_POINTPATH" "', argument " "1"" of type '" "struct _dwg_object_POINTPATH *""'"); 
    }
    arg1 = (struct _dwg_object_POINTPATH *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TVDEVICEPROPERTIES_parent_set) {
  {
    struct _dwg_object_TVDEVICEPROPERTIES *arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TVDEVICEPROPERTIES_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TVDEVICEPROPERTIES, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TVDEVICEPROPERTIES_parent_set" "', argument " "1"" of type '" "struct _dwg_object_TVDEVICEPROPERTIES *""'"); 
    }
    arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_TVDEVICEPROPERTIES_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TVDEVICEPROPERTIES_parent_get) {
  {
    struct _dwg_object_TVDEVICEPROPERTIES *arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TVDEVICEPROPERTIES_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TVDEVICEPROPERTIES, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TVDEVICEPROPERTIES_parent_get" "', argument " "1"" of type '" "struct _dwg_object_TVDEVICEPROPERTIES *""'"); 
    }
    arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TVDEVICEPROPERTIES_flags_set) {
  {
    struct _dwg_object_TVDEVICEPROPERTIES *arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TVDEVICEPROPERTIES_flags_set(self,flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TVDEVICEPROPERTIES, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TVDEVICEPROPERTIES_flags_set" "', argument " "1"" of type '" "struct _dwg_object_TVDEVICEPROPERTIES *""'"); 
    }
    arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_TVDEVICEPROPERTIES_flags_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TVDEVICEPROPERTIES_flags_get) {
  {
    struct _dwg_object_TVDEVICEPROPERTIES *arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TVDEVICEPROPERTIES_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TVDEVICEPROPERTIES, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TVDEVICEPROPERTIES_flags_get" "', argument " "1"" of type '" "struct _dwg_object_TVDEVICEPROPERTIES *""'"); 
    }
    arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *)(argp1);
    result = (BITCODE_BL) ((arg1)->flags);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TVDEVICEPROPERTIES_max_regen_threads_set) {
  {
    struct _dwg_object_TVDEVICEPROPERTIES *arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TVDEVICEPROPERTIES_max_regen_threads_set(self,max_regen_threads);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TVDEVICEPROPERTIES, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TVDEVICEPROPERTIES_max_regen_threads_set" "', argument " "1"" of type '" "struct _dwg_object_TVDEVICEPROPERTIES *""'"); 
    }
    arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_TVDEVICEPROPERTIES_max_regen_threads_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->max_regen_threads = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TVDEVICEPROPERTIES_max_regen_threads_get) {
  {
    struct _dwg_object_TVDEVICEPROPERTIES *arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TVDEVICEPROPERTIES_max_regen_threads_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TVDEVICEPROPERTIES, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TVDEVICEPROPERTIES_max_regen_threads_get" "', argument " "1"" of type '" "struct _dwg_object_TVDEVICEPROPERTIES *""'"); 
    }
    arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *)(argp1);
    result = (BITCODE_BS) ((arg1)->max_regen_threads);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TVDEVICEPROPERTIES_use_lut_palette_set) {
  {
    struct _dwg_object_TVDEVICEPROPERTIES *arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TVDEVICEPROPERTIES_use_lut_palette_set(self,use_lut_palette);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TVDEVICEPROPERTIES, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TVDEVICEPROPERTIES_use_lut_palette_set" "', argument " "1"" of type '" "struct _dwg_object_TVDEVICEPROPERTIES *""'"); 
    }
    arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_TVDEVICEPROPERTIES_use_lut_palette_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->use_lut_palette = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TVDEVICEPROPERTIES_use_lut_palette_get) {
  {
    struct _dwg_object_TVDEVICEPROPERTIES *arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TVDEVICEPROPERTIES_use_lut_palette_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TVDEVICEPROPERTIES, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TVDEVICEPROPERTIES_use_lut_palette_get" "', argument " "1"" of type '" "struct _dwg_object_TVDEVICEPROPERTIES *""'"); 
    }
    arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *)(argp1);
    result = (BITCODE_BL) ((arg1)->use_lut_palette);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TVDEVICEPROPERTIES_alt_hlt_set) {
  {
    struct _dwg_object_TVDEVICEPROPERTIES *arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *) 0 ;
    BITCODE_BLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TVDEVICEPROPERTIES_alt_hlt_set(self,alt_hlt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TVDEVICEPROPERTIES, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TVDEVICEPROPERTIES_alt_hlt_set" "', argument " "1"" of type '" "struct _dwg_object_TVDEVICEPROPERTIES *""'"); 
    }
    arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_TVDEVICEPROPERTIES_alt_hlt_set" "', argument " "2"" of type '" "BITCODE_BLL""'");
    } 
    arg2 = (BITCODE_BLL)(val2);
    if (arg1) (arg1)->alt_hlt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TVDEVICEPROPERTIES_alt_hlt_get) {
  {
    struct _dwg_object_TVDEVICEPROPERTIES *arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TVDEVICEPROPERTIES_alt_hlt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TVDEVICEPROPERTIES, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TVDEVICEPROPERTIES_alt_hlt_get" "', argument " "1"" of type '" "struct _dwg_object_TVDEVICEPROPERTIES *""'"); 
    }
    arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *)(argp1);
    result = (BITCODE_BLL) ((arg1)->alt_hlt);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TVDEVICEPROPERTIES_alt_hltcolor_set) {
  {
    struct _dwg_object_TVDEVICEPROPERTIES *arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *) 0 ;
    BITCODE_BLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TVDEVICEPROPERTIES_alt_hltcolor_set(self,alt_hltcolor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TVDEVICEPROPERTIES, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TVDEVICEPROPERTIES_alt_hltcolor_set" "', argument " "1"" of type '" "struct _dwg_object_TVDEVICEPROPERTIES *""'"); 
    }
    arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_TVDEVICEPROPERTIES_alt_hltcolor_set" "', argument " "2"" of type '" "BITCODE_BLL""'");
    } 
    arg2 = (BITCODE_BLL)(val2);
    if (arg1) (arg1)->alt_hltcolor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TVDEVICEPROPERTIES_alt_hltcolor_get) {
  {
    struct _dwg_object_TVDEVICEPROPERTIES *arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TVDEVICEPROPERTIES_alt_hltcolor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TVDEVICEPROPERTIES, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TVDEVICEPROPERTIES_alt_hltcolor_get" "', argument " "1"" of type '" "struct _dwg_object_TVDEVICEPROPERTIES *""'"); 
    }
    arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *)(argp1);
    result = (BITCODE_BLL) ((arg1)->alt_hltcolor);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TVDEVICEPROPERTIES_geom_shader_usage_set) {
  {
    struct _dwg_object_TVDEVICEPROPERTIES *arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *) 0 ;
    BITCODE_BLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TVDEVICEPROPERTIES_geom_shader_usage_set(self,geom_shader_usage);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TVDEVICEPROPERTIES, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TVDEVICEPROPERTIES_geom_shader_usage_set" "', argument " "1"" of type '" "struct _dwg_object_TVDEVICEPROPERTIES *""'"); 
    }
    arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_TVDEVICEPROPERTIES_geom_shader_usage_set" "', argument " "2"" of type '" "BITCODE_BLL""'");
    } 
    arg2 = (BITCODE_BLL)(val2);
    if (arg1) (arg1)->geom_shader_usage = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TVDEVICEPROPERTIES_geom_shader_usage_get) {
  {
    struct _dwg_object_TVDEVICEPROPERTIES *arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TVDEVICEPROPERTIES_geom_shader_usage_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TVDEVICEPROPERTIES, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TVDEVICEPROPERTIES_geom_shader_usage_get" "', argument " "1"" of type '" "struct _dwg_object_TVDEVICEPROPERTIES *""'"); 
    }
    arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *)(argp1);
    result = (BITCODE_BLL) ((arg1)->geom_shader_usage);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TVDEVICEPROPERTIES_blending_mode_set) {
  {
    struct _dwg_object_TVDEVICEPROPERTIES *arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TVDEVICEPROPERTIES_blending_mode_set(self,blending_mode);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TVDEVICEPROPERTIES, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TVDEVICEPROPERTIES_blending_mode_set" "', argument " "1"" of type '" "struct _dwg_object_TVDEVICEPROPERTIES *""'"); 
    }
    arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_TVDEVICEPROPERTIES_blending_mode_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->blending_mode = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TVDEVICEPROPERTIES_blending_mode_get) {
  {
    struct _dwg_object_TVDEVICEPROPERTIES *arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TVDEVICEPROPERTIES_blending_mode_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TVDEVICEPROPERTIES, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TVDEVICEPROPERTIES_blending_mode_get" "', argument " "1"" of type '" "struct _dwg_object_TVDEVICEPROPERTIES *""'"); 
    }
    arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *)(argp1);
    result = (BITCODE_BL) ((arg1)->blending_mode);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TVDEVICEPROPERTIES_antialiasing_level_set) {
  {
    struct _dwg_object_TVDEVICEPROPERTIES *arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TVDEVICEPROPERTIES_antialiasing_level_set(self,antialiasing_level);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TVDEVICEPROPERTIES, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TVDEVICEPROPERTIES_antialiasing_level_set" "', argument " "1"" of type '" "struct _dwg_object_TVDEVICEPROPERTIES *""'"); 
    }
    arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_TVDEVICEPROPERTIES_antialiasing_level_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->antialiasing_level = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TVDEVICEPROPERTIES_antialiasing_level_get) {
  {
    struct _dwg_object_TVDEVICEPROPERTIES *arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TVDEVICEPROPERTIES_antialiasing_level_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TVDEVICEPROPERTIES, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TVDEVICEPROPERTIES_antialiasing_level_get" "', argument " "1"" of type '" "struct _dwg_object_TVDEVICEPROPERTIES *""'"); 
    }
    arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *)(argp1);
    result = (BITCODE_BD) ((arg1)->antialiasing_level);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TVDEVICEPROPERTIES_bd2_set) {
  {
    struct _dwg_object_TVDEVICEPROPERTIES *arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TVDEVICEPROPERTIES_bd2_set(self,bd2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TVDEVICEPROPERTIES, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TVDEVICEPROPERTIES_bd2_set" "', argument " "1"" of type '" "struct _dwg_object_TVDEVICEPROPERTIES *""'"); 
    }
    arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_TVDEVICEPROPERTIES_bd2_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->bd2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TVDEVICEPROPERTIES_bd2_get) {
  {
    struct _dwg_object_TVDEVICEPROPERTIES *arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TVDEVICEPROPERTIES_bd2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TVDEVICEPROPERTIES, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TVDEVICEPROPERTIES_bd2_get" "', argument " "1"" of type '" "struct _dwg_object_TVDEVICEPROPERTIES *""'"); 
    }
    arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *)(argp1);
    result = (BITCODE_BD) ((arg1)->bd2);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_TVDEVICEPROPERTIES) {
  {
    int argvi = 0;
    struct _dwg_object_TVDEVICEPROPERTIES *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_TVDEVICEPROPERTIES();");
    }
    result = (struct _dwg_object_TVDEVICEPROPERTIES *)calloc(1, sizeof(struct _dwg_object_TVDEVICEPROPERTIES));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_TVDEVICEPROPERTIES, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_TVDEVICEPROPERTIES) {
  {
    struct _dwg_object_TVDEVICEPROPERTIES *arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_TVDEVICEPROPERTIES(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TVDEVICEPROPERTIES, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_TVDEVICEPROPERTIES" "', argument " "1"" of type '" "struct _dwg_object_TVDEVICEPROPERTIES *""'"); 
    }
    arg1 = (struct _dwg_object_TVDEVICEPROPERTIES *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SKYLIGHT_BACKGROUND_parent_set) {
  {
    struct _dwg_object_SKYLIGHT_BACKGROUND *arg1 = (struct _dwg_object_SKYLIGHT_BACKGROUND *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SKYLIGHT_BACKGROUND_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SKYLIGHT_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SKYLIGHT_BACKGROUND_parent_set" "', argument " "1"" of type '" "struct _dwg_object_SKYLIGHT_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_SKYLIGHT_BACKGROUND *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SKYLIGHT_BACKGROUND_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SKYLIGHT_BACKGROUND_parent_get) {
  {
    struct _dwg_object_SKYLIGHT_BACKGROUND *arg1 = (struct _dwg_object_SKYLIGHT_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SKYLIGHT_BACKGROUND_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SKYLIGHT_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SKYLIGHT_BACKGROUND_parent_get" "', argument " "1"" of type '" "struct _dwg_object_SKYLIGHT_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_SKYLIGHT_BACKGROUND *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SKYLIGHT_BACKGROUND_class_version_set) {
  {
    struct _dwg_object_SKYLIGHT_BACKGROUND *arg1 = (struct _dwg_object_SKYLIGHT_BACKGROUND *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SKYLIGHT_BACKGROUND_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SKYLIGHT_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SKYLIGHT_BACKGROUND_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_SKYLIGHT_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_SKYLIGHT_BACKGROUND *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SKYLIGHT_BACKGROUND_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SKYLIGHT_BACKGROUND_class_version_get) {
  {
    struct _dwg_object_SKYLIGHT_BACKGROUND *arg1 = (struct _dwg_object_SKYLIGHT_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SKYLIGHT_BACKGROUND_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SKYLIGHT_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SKYLIGHT_BACKGROUND_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_SKYLIGHT_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_SKYLIGHT_BACKGROUND *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SKYLIGHT_BACKGROUND_sunid_set) {
  {
    struct _dwg_object_SKYLIGHT_BACKGROUND *arg1 = (struct _dwg_object_SKYLIGHT_BACKGROUND *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SKYLIGHT_BACKGROUND_sunid_set(self,sunid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SKYLIGHT_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SKYLIGHT_BACKGROUND_sunid_set" "', argument " "1"" of type '" "struct _dwg_object_SKYLIGHT_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_SKYLIGHT_BACKGROUND *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SKYLIGHT_BACKGROUND_sunid_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->sunid = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SKYLIGHT_BACKGROUND_sunid_get) {
  {
    struct _dwg_object_SKYLIGHT_BACKGROUND *arg1 = (struct _dwg_object_SKYLIGHT_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SKYLIGHT_BACKGROUND_sunid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SKYLIGHT_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SKYLIGHT_BACKGROUND_sunid_get" "', argument " "1"" of type '" "struct _dwg_object_SKYLIGHT_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_SKYLIGHT_BACKGROUND *)(argp1);
    result = (BITCODE_H) ((arg1)->sunid);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_SKYLIGHT_BACKGROUND) {
  {
    int argvi = 0;
    struct _dwg_object_SKYLIGHT_BACKGROUND *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_SKYLIGHT_BACKGROUND();");
    }
    result = (struct _dwg_object_SKYLIGHT_BACKGROUND *)calloc(1, sizeof(struct _dwg_object_SKYLIGHT_BACKGROUND));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_SKYLIGHT_BACKGROUND, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_SKYLIGHT_BACKGROUND) {
  {
    struct _dwg_object_SKYLIGHT_BACKGROUND *arg1 = (struct _dwg_object_SKYLIGHT_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_SKYLIGHT_BACKGROUND(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SKYLIGHT_BACKGROUND, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_SKYLIGHT_BACKGROUND" "', argument " "1"" of type '" "struct _dwg_object_SKYLIGHT_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_SKYLIGHT_BACKGROUND *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SOLID_BACKGROUND_parent_set) {
  {
    struct _dwg_object_SOLID_BACKGROUND *arg1 = (struct _dwg_object_SOLID_BACKGROUND *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SOLID_BACKGROUND_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SOLID_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SOLID_BACKGROUND_parent_set" "', argument " "1"" of type '" "struct _dwg_object_SOLID_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_SOLID_BACKGROUND *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SOLID_BACKGROUND_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SOLID_BACKGROUND_parent_get) {
  {
    struct _dwg_object_SOLID_BACKGROUND *arg1 = (struct _dwg_object_SOLID_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SOLID_BACKGROUND_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SOLID_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SOLID_BACKGROUND_parent_get" "', argument " "1"" of type '" "struct _dwg_object_SOLID_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_SOLID_BACKGROUND *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SOLID_BACKGROUND_class_version_set) {
  {
    struct _dwg_object_SOLID_BACKGROUND *arg1 = (struct _dwg_object_SOLID_BACKGROUND *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SOLID_BACKGROUND_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SOLID_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SOLID_BACKGROUND_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_SOLID_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_SOLID_BACKGROUND *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SOLID_BACKGROUND_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SOLID_BACKGROUND_class_version_get) {
  {
    struct _dwg_object_SOLID_BACKGROUND *arg1 = (struct _dwg_object_SOLID_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SOLID_BACKGROUND_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SOLID_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SOLID_BACKGROUND_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_SOLID_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_SOLID_BACKGROUND *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SOLID_BACKGROUND_color_set) {
  {
    struct _dwg_object_SOLID_BACKGROUND *arg1 = (struct _dwg_object_SOLID_BACKGROUND *) 0 ;
    BITCODE_BLx arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SOLID_BACKGROUND_color_set(self,color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SOLID_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SOLID_BACKGROUND_color_set" "', argument " "1"" of type '" "struct _dwg_object_SOLID_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_SOLID_BACKGROUND *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SOLID_BACKGROUND_color_set" "', argument " "2"" of type '" "BITCODE_BLx""'");
    } 
    arg2 = (BITCODE_BLx)(val2);
    if (arg1) (arg1)->color = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SOLID_BACKGROUND_color_get) {
  {
    struct _dwg_object_SOLID_BACKGROUND *arg1 = (struct _dwg_object_SOLID_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLx result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SOLID_BACKGROUND_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SOLID_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SOLID_BACKGROUND_color_get" "', argument " "1"" of type '" "struct _dwg_object_SOLID_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_SOLID_BACKGROUND *)(argp1);
    result = (BITCODE_BLx) ((arg1)->color);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_SOLID_BACKGROUND) {
  {
    int argvi = 0;
    struct _dwg_object_SOLID_BACKGROUND *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_SOLID_BACKGROUND();");
    }
    result = (struct _dwg_object_SOLID_BACKGROUND *)calloc(1, sizeof(struct _dwg_object_SOLID_BACKGROUND));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_SOLID_BACKGROUND, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_SOLID_BACKGROUND) {
  {
    struct _dwg_object_SOLID_BACKGROUND *arg1 = (struct _dwg_object_SOLID_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_SOLID_BACKGROUND(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SOLID_BACKGROUND, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_SOLID_BACKGROUND" "', argument " "1"" of type '" "struct _dwg_object_SOLID_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_SOLID_BACKGROUND *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IMAGE_BACKGROUND_parent_set) {
  {
    struct _dwg_object_IMAGE_BACKGROUND *arg1 = (struct _dwg_object_IMAGE_BACKGROUND *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_IMAGE_BACKGROUND_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGE_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IMAGE_BACKGROUND_parent_set" "', argument " "1"" of type '" "struct _dwg_object_IMAGE_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGE_BACKGROUND *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_IMAGE_BACKGROUND_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IMAGE_BACKGROUND_parent_get) {
  {
    struct _dwg_object_IMAGE_BACKGROUND *arg1 = (struct _dwg_object_IMAGE_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_IMAGE_BACKGROUND_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGE_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IMAGE_BACKGROUND_parent_get" "', argument " "1"" of type '" "struct _dwg_object_IMAGE_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGE_BACKGROUND *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IMAGE_BACKGROUND_class_version_set) {
  {
    struct _dwg_object_IMAGE_BACKGROUND *arg1 = (struct _dwg_object_IMAGE_BACKGROUND *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_IMAGE_BACKGROUND_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGE_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IMAGE_BACKGROUND_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_IMAGE_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGE_BACKGROUND *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_IMAGE_BACKGROUND_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IMAGE_BACKGROUND_class_version_get) {
  {
    struct _dwg_object_IMAGE_BACKGROUND *arg1 = (struct _dwg_object_IMAGE_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_IMAGE_BACKGROUND_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGE_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IMAGE_BACKGROUND_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_IMAGE_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGE_BACKGROUND *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IMAGE_BACKGROUND_filename_set) {
  {
    struct _dwg_object_IMAGE_BACKGROUND *arg1 = (struct _dwg_object_IMAGE_BACKGROUND *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_IMAGE_BACKGROUND_filename_set(self,filename);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGE_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IMAGE_BACKGROUND_filename_set" "', argument " "1"" of type '" "struct _dwg_object_IMAGE_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGE_BACKGROUND *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_IMAGE_BACKGROUND_filename_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->filename);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->filename = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->filename = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IMAGE_BACKGROUND_filename_get) {
  {
    struct _dwg_object_IMAGE_BACKGROUND *arg1 = (struct _dwg_object_IMAGE_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_IMAGE_BACKGROUND_filename_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGE_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IMAGE_BACKGROUND_filename_get" "', argument " "1"" of type '" "struct _dwg_object_IMAGE_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGE_BACKGROUND *)(argp1);
    result = (BITCODE_TV) ((arg1)->filename);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IMAGE_BACKGROUND_fit_to_screen_set) {
  {
    struct _dwg_object_IMAGE_BACKGROUND *arg1 = (struct _dwg_object_IMAGE_BACKGROUND *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_IMAGE_BACKGROUND_fit_to_screen_set(self,fit_to_screen);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGE_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IMAGE_BACKGROUND_fit_to_screen_set" "', argument " "1"" of type '" "struct _dwg_object_IMAGE_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGE_BACKGROUND *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_IMAGE_BACKGROUND_fit_to_screen_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->fit_to_screen = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IMAGE_BACKGROUND_fit_to_screen_get) {
  {
    struct _dwg_object_IMAGE_BACKGROUND *arg1 = (struct _dwg_object_IMAGE_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_IMAGE_BACKGROUND_fit_to_screen_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGE_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IMAGE_BACKGROUND_fit_to_screen_get" "', argument " "1"" of type '" "struct _dwg_object_IMAGE_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGE_BACKGROUND *)(argp1);
    result = (BITCODE_B) ((arg1)->fit_to_screen);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IMAGE_BACKGROUND_maintain_aspect_ratio_set) {
  {
    struct _dwg_object_IMAGE_BACKGROUND *arg1 = (struct _dwg_object_IMAGE_BACKGROUND *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_IMAGE_BACKGROUND_maintain_aspect_ratio_set(self,maintain_aspect_ratio);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGE_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IMAGE_BACKGROUND_maintain_aspect_ratio_set" "', argument " "1"" of type '" "struct _dwg_object_IMAGE_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGE_BACKGROUND *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_IMAGE_BACKGROUND_maintain_aspect_ratio_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->maintain_aspect_ratio = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IMAGE_BACKGROUND_maintain_aspect_ratio_get) {
  {
    struct _dwg_object_IMAGE_BACKGROUND *arg1 = (struct _dwg_object_IMAGE_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_IMAGE_BACKGROUND_maintain_aspect_ratio_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGE_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IMAGE_BACKGROUND_maintain_aspect_ratio_get" "', argument " "1"" of type '" "struct _dwg_object_IMAGE_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGE_BACKGROUND *)(argp1);
    result = (BITCODE_B) ((arg1)->maintain_aspect_ratio);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IMAGE_BACKGROUND_use_tiling_set) {
  {
    struct _dwg_object_IMAGE_BACKGROUND *arg1 = (struct _dwg_object_IMAGE_BACKGROUND *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_IMAGE_BACKGROUND_use_tiling_set(self,use_tiling);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGE_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IMAGE_BACKGROUND_use_tiling_set" "', argument " "1"" of type '" "struct _dwg_object_IMAGE_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGE_BACKGROUND *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_IMAGE_BACKGROUND_use_tiling_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->use_tiling = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IMAGE_BACKGROUND_use_tiling_get) {
  {
    struct _dwg_object_IMAGE_BACKGROUND *arg1 = (struct _dwg_object_IMAGE_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_IMAGE_BACKGROUND_use_tiling_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGE_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IMAGE_BACKGROUND_use_tiling_get" "', argument " "1"" of type '" "struct _dwg_object_IMAGE_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGE_BACKGROUND *)(argp1);
    result = (BITCODE_B) ((arg1)->use_tiling);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IMAGE_BACKGROUND_offset_set) {
  {
    struct _dwg_object_IMAGE_BACKGROUND *arg1 = (struct _dwg_object_IMAGE_BACKGROUND *) 0 ;
    BITCODE_2BD *arg2 = (BITCODE_2BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_IMAGE_BACKGROUND_offset_set(self,offset);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGE_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IMAGE_BACKGROUND_offset_set" "', argument " "1"" of type '" "struct _dwg_object_IMAGE_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGE_BACKGROUND *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_IMAGE_BACKGROUND_offset_set" "', argument " "2"" of type '" "BITCODE_2BD *""'"); 
    }
    arg2 = (BITCODE_2BD *)(argp2);
    if (arg1) (arg1)->offset = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IMAGE_BACKGROUND_offset_get) {
  {
    struct _dwg_object_IMAGE_BACKGROUND *arg1 = (struct _dwg_object_IMAGE_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_IMAGE_BACKGROUND_offset_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGE_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IMAGE_BACKGROUND_offset_get" "', argument " "1"" of type '" "struct _dwg_object_IMAGE_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGE_BACKGROUND *)(argp1);
    result = (BITCODE_2BD *)& ((arg1)->offset);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IMAGE_BACKGROUND_scale_set) {
  {
    struct _dwg_object_IMAGE_BACKGROUND *arg1 = (struct _dwg_object_IMAGE_BACKGROUND *) 0 ;
    BITCODE_2BD *arg2 = (BITCODE_2BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_IMAGE_BACKGROUND_scale_set(self,scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGE_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IMAGE_BACKGROUND_scale_set" "', argument " "1"" of type '" "struct _dwg_object_IMAGE_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGE_BACKGROUND *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_IMAGE_BACKGROUND_scale_set" "', argument " "2"" of type '" "BITCODE_2BD *""'"); 
    }
    arg2 = (BITCODE_2BD *)(argp2);
    if (arg1) (arg1)->scale = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IMAGE_BACKGROUND_scale_get) {
  {
    struct _dwg_object_IMAGE_BACKGROUND *arg1 = (struct _dwg_object_IMAGE_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_IMAGE_BACKGROUND_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGE_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IMAGE_BACKGROUND_scale_get" "', argument " "1"" of type '" "struct _dwg_object_IMAGE_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGE_BACKGROUND *)(argp1);
    result = (BITCODE_2BD *)& ((arg1)->scale);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_IMAGE_BACKGROUND) {
  {
    int argvi = 0;
    struct _dwg_object_IMAGE_BACKGROUND *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_IMAGE_BACKGROUND();");
    }
    result = (struct _dwg_object_IMAGE_BACKGROUND *)calloc(1, sizeof(struct _dwg_object_IMAGE_BACKGROUND));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_IMAGE_BACKGROUND, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_IMAGE_BACKGROUND) {
  {
    struct _dwg_object_IMAGE_BACKGROUND *arg1 = (struct _dwg_object_IMAGE_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_IMAGE_BACKGROUND(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IMAGE_BACKGROUND, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_IMAGE_BACKGROUND" "', argument " "1"" of type '" "struct _dwg_object_IMAGE_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_IMAGE_BACKGROUND *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IBL_BACKGROUND_parent_set) {
  {
    struct _dwg_object_IBL_BACKGROUND *arg1 = (struct _dwg_object_IBL_BACKGROUND *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_IBL_BACKGROUND_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IBL_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IBL_BACKGROUND_parent_set" "', argument " "1"" of type '" "struct _dwg_object_IBL_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_IBL_BACKGROUND *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_IBL_BACKGROUND_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IBL_BACKGROUND_parent_get) {
  {
    struct _dwg_object_IBL_BACKGROUND *arg1 = (struct _dwg_object_IBL_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_IBL_BACKGROUND_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IBL_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IBL_BACKGROUND_parent_get" "', argument " "1"" of type '" "struct _dwg_object_IBL_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_IBL_BACKGROUND *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IBL_BACKGROUND_class_version_set) {
  {
    struct _dwg_object_IBL_BACKGROUND *arg1 = (struct _dwg_object_IBL_BACKGROUND *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_IBL_BACKGROUND_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IBL_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IBL_BACKGROUND_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_IBL_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_IBL_BACKGROUND *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_IBL_BACKGROUND_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IBL_BACKGROUND_class_version_get) {
  {
    struct _dwg_object_IBL_BACKGROUND *arg1 = (struct _dwg_object_IBL_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_IBL_BACKGROUND_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IBL_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IBL_BACKGROUND_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_IBL_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_IBL_BACKGROUND *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IBL_BACKGROUND_enable_set) {
  {
    struct _dwg_object_IBL_BACKGROUND *arg1 = (struct _dwg_object_IBL_BACKGROUND *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_IBL_BACKGROUND_enable_set(self,enable);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IBL_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IBL_BACKGROUND_enable_set" "', argument " "1"" of type '" "struct _dwg_object_IBL_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_IBL_BACKGROUND *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_IBL_BACKGROUND_enable_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->enable = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IBL_BACKGROUND_enable_get) {
  {
    struct _dwg_object_IBL_BACKGROUND *arg1 = (struct _dwg_object_IBL_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_IBL_BACKGROUND_enable_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IBL_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IBL_BACKGROUND_enable_get" "', argument " "1"" of type '" "struct _dwg_object_IBL_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_IBL_BACKGROUND *)(argp1);
    result = (BITCODE_B) ((arg1)->enable);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IBL_BACKGROUND_name_set) {
  {
    struct _dwg_object_IBL_BACKGROUND *arg1 = (struct _dwg_object_IBL_BACKGROUND *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_IBL_BACKGROUND_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IBL_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IBL_BACKGROUND_name_set" "', argument " "1"" of type '" "struct _dwg_object_IBL_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_IBL_BACKGROUND *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_IBL_BACKGROUND_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IBL_BACKGROUND_name_get) {
  {
    struct _dwg_object_IBL_BACKGROUND *arg1 = (struct _dwg_object_IBL_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_IBL_BACKGROUND_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IBL_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IBL_BACKGROUND_name_get" "', argument " "1"" of type '" "struct _dwg_object_IBL_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_IBL_BACKGROUND *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IBL_BACKGROUND_rotation_set) {
  {
    struct _dwg_object_IBL_BACKGROUND *arg1 = (struct _dwg_object_IBL_BACKGROUND *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_IBL_BACKGROUND_rotation_set(self,rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IBL_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IBL_BACKGROUND_rotation_set" "', argument " "1"" of type '" "struct _dwg_object_IBL_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_IBL_BACKGROUND *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_IBL_BACKGROUND_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IBL_BACKGROUND_rotation_get) {
  {
    struct _dwg_object_IBL_BACKGROUND *arg1 = (struct _dwg_object_IBL_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_IBL_BACKGROUND_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IBL_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IBL_BACKGROUND_rotation_get" "', argument " "1"" of type '" "struct _dwg_object_IBL_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_IBL_BACKGROUND *)(argp1);
    result = (BITCODE_BD) ((arg1)->rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IBL_BACKGROUND_display_image_set) {
  {
    struct _dwg_object_IBL_BACKGROUND *arg1 = (struct _dwg_object_IBL_BACKGROUND *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_IBL_BACKGROUND_display_image_set(self,display_image);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IBL_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IBL_BACKGROUND_display_image_set" "', argument " "1"" of type '" "struct _dwg_object_IBL_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_IBL_BACKGROUND *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_IBL_BACKGROUND_display_image_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->display_image = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IBL_BACKGROUND_display_image_get) {
  {
    struct _dwg_object_IBL_BACKGROUND *arg1 = (struct _dwg_object_IBL_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_IBL_BACKGROUND_display_image_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IBL_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IBL_BACKGROUND_display_image_get" "', argument " "1"" of type '" "struct _dwg_object_IBL_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_IBL_BACKGROUND *)(argp1);
    result = (BITCODE_B) ((arg1)->display_image);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IBL_BACKGROUND_secondary_background_set) {
  {
    struct _dwg_object_IBL_BACKGROUND *arg1 = (struct _dwg_object_IBL_BACKGROUND *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_IBL_BACKGROUND_secondary_background_set(self,secondary_background);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IBL_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IBL_BACKGROUND_secondary_background_set" "', argument " "1"" of type '" "struct _dwg_object_IBL_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_IBL_BACKGROUND *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_IBL_BACKGROUND_secondary_background_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->secondary_background = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_IBL_BACKGROUND_secondary_background_get) {
  {
    struct _dwg_object_IBL_BACKGROUND *arg1 = (struct _dwg_object_IBL_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_IBL_BACKGROUND_secondary_background_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IBL_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_IBL_BACKGROUND_secondary_background_get" "', argument " "1"" of type '" "struct _dwg_object_IBL_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_IBL_BACKGROUND *)(argp1);
    result = (BITCODE_H) ((arg1)->secondary_background);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_IBL_BACKGROUND) {
  {
    int argvi = 0;
    struct _dwg_object_IBL_BACKGROUND *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_IBL_BACKGROUND();");
    }
    result = (struct _dwg_object_IBL_BACKGROUND *)calloc(1, sizeof(struct _dwg_object_IBL_BACKGROUND));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_IBL_BACKGROUND, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_IBL_BACKGROUND) {
  {
    struct _dwg_object_IBL_BACKGROUND *arg1 = (struct _dwg_object_IBL_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_IBL_BACKGROUND(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_IBL_BACKGROUND, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_IBL_BACKGROUND" "', argument " "1"" of type '" "struct _dwg_object_IBL_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_IBL_BACKGROUND *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GRADIENT_BACKGROUND_parent_set) {
  {
    struct _dwg_object_GRADIENT_BACKGROUND *arg1 = (struct _dwg_object_GRADIENT_BACKGROUND *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GRADIENT_BACKGROUND_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GRADIENT_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GRADIENT_BACKGROUND_parent_set" "', argument " "1"" of type '" "struct _dwg_object_GRADIENT_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_GRADIENT_BACKGROUND *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_GRADIENT_BACKGROUND_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GRADIENT_BACKGROUND_parent_get) {
  {
    struct _dwg_object_GRADIENT_BACKGROUND *arg1 = (struct _dwg_object_GRADIENT_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GRADIENT_BACKGROUND_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GRADIENT_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GRADIENT_BACKGROUND_parent_get" "', argument " "1"" of type '" "struct _dwg_object_GRADIENT_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_GRADIENT_BACKGROUND *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GRADIENT_BACKGROUND_class_version_set) {
  {
    struct _dwg_object_GRADIENT_BACKGROUND *arg1 = (struct _dwg_object_GRADIENT_BACKGROUND *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GRADIENT_BACKGROUND_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GRADIENT_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GRADIENT_BACKGROUND_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_GRADIENT_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_GRADIENT_BACKGROUND *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GRADIENT_BACKGROUND_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GRADIENT_BACKGROUND_class_version_get) {
  {
    struct _dwg_object_GRADIENT_BACKGROUND *arg1 = (struct _dwg_object_GRADIENT_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GRADIENT_BACKGROUND_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GRADIENT_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GRADIENT_BACKGROUND_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_GRADIENT_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_GRADIENT_BACKGROUND *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GRADIENT_BACKGROUND_color_top_set) {
  {
    struct _dwg_object_GRADIENT_BACKGROUND *arg1 = (struct _dwg_object_GRADIENT_BACKGROUND *) 0 ;
    BITCODE_BLx arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GRADIENT_BACKGROUND_color_top_set(self,color_top);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GRADIENT_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GRADIENT_BACKGROUND_color_top_set" "', argument " "1"" of type '" "struct _dwg_object_GRADIENT_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_GRADIENT_BACKGROUND *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GRADIENT_BACKGROUND_color_top_set" "', argument " "2"" of type '" "BITCODE_BLx""'");
    } 
    arg2 = (BITCODE_BLx)(val2);
    if (arg1) (arg1)->color_top = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GRADIENT_BACKGROUND_color_top_get) {
  {
    struct _dwg_object_GRADIENT_BACKGROUND *arg1 = (struct _dwg_object_GRADIENT_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLx result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GRADIENT_BACKGROUND_color_top_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GRADIENT_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GRADIENT_BACKGROUND_color_top_get" "', argument " "1"" of type '" "struct _dwg_object_GRADIENT_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_GRADIENT_BACKGROUND *)(argp1);
    result = (BITCODE_BLx) ((arg1)->color_top);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GRADIENT_BACKGROUND_color_middle_set) {
  {
    struct _dwg_object_GRADIENT_BACKGROUND *arg1 = (struct _dwg_object_GRADIENT_BACKGROUND *) 0 ;
    BITCODE_BLx arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GRADIENT_BACKGROUND_color_middle_set(self,color_middle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GRADIENT_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GRADIENT_BACKGROUND_color_middle_set" "', argument " "1"" of type '" "struct _dwg_object_GRADIENT_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_GRADIENT_BACKGROUND *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GRADIENT_BACKGROUND_color_middle_set" "', argument " "2"" of type '" "BITCODE_BLx""'");
    } 
    arg2 = (BITCODE_BLx)(val2);
    if (arg1) (arg1)->color_middle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GRADIENT_BACKGROUND_color_middle_get) {
  {
    struct _dwg_object_GRADIENT_BACKGROUND *arg1 = (struct _dwg_object_GRADIENT_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLx result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GRADIENT_BACKGROUND_color_middle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GRADIENT_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GRADIENT_BACKGROUND_color_middle_get" "', argument " "1"" of type '" "struct _dwg_object_GRADIENT_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_GRADIENT_BACKGROUND *)(argp1);
    result = (BITCODE_BLx) ((arg1)->color_middle);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GRADIENT_BACKGROUND_color_bottom_set) {
  {
    struct _dwg_object_GRADIENT_BACKGROUND *arg1 = (struct _dwg_object_GRADIENT_BACKGROUND *) 0 ;
    BITCODE_BLx arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GRADIENT_BACKGROUND_color_bottom_set(self,color_bottom);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GRADIENT_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GRADIENT_BACKGROUND_color_bottom_set" "', argument " "1"" of type '" "struct _dwg_object_GRADIENT_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_GRADIENT_BACKGROUND *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GRADIENT_BACKGROUND_color_bottom_set" "', argument " "2"" of type '" "BITCODE_BLx""'");
    } 
    arg2 = (BITCODE_BLx)(val2);
    if (arg1) (arg1)->color_bottom = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GRADIENT_BACKGROUND_color_bottom_get) {
  {
    struct _dwg_object_GRADIENT_BACKGROUND *arg1 = (struct _dwg_object_GRADIENT_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLx result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GRADIENT_BACKGROUND_color_bottom_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GRADIENT_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GRADIENT_BACKGROUND_color_bottom_get" "', argument " "1"" of type '" "struct _dwg_object_GRADIENT_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_GRADIENT_BACKGROUND *)(argp1);
    result = (BITCODE_BLx) ((arg1)->color_bottom);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GRADIENT_BACKGROUND_horizon_set) {
  {
    struct _dwg_object_GRADIENT_BACKGROUND *arg1 = (struct _dwg_object_GRADIENT_BACKGROUND *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GRADIENT_BACKGROUND_horizon_set(self,horizon);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GRADIENT_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GRADIENT_BACKGROUND_horizon_set" "', argument " "1"" of type '" "struct _dwg_object_GRADIENT_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_GRADIENT_BACKGROUND *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GRADIENT_BACKGROUND_horizon_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->horizon = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GRADIENT_BACKGROUND_horizon_get) {
  {
    struct _dwg_object_GRADIENT_BACKGROUND *arg1 = (struct _dwg_object_GRADIENT_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GRADIENT_BACKGROUND_horizon_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GRADIENT_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GRADIENT_BACKGROUND_horizon_get" "', argument " "1"" of type '" "struct _dwg_object_GRADIENT_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_GRADIENT_BACKGROUND *)(argp1);
    result = (BITCODE_BD) ((arg1)->horizon);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GRADIENT_BACKGROUND_height_set) {
  {
    struct _dwg_object_GRADIENT_BACKGROUND *arg1 = (struct _dwg_object_GRADIENT_BACKGROUND *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GRADIENT_BACKGROUND_height_set(self,height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GRADIENT_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GRADIENT_BACKGROUND_height_set" "', argument " "1"" of type '" "struct _dwg_object_GRADIENT_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_GRADIENT_BACKGROUND *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GRADIENT_BACKGROUND_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GRADIENT_BACKGROUND_height_get) {
  {
    struct _dwg_object_GRADIENT_BACKGROUND *arg1 = (struct _dwg_object_GRADIENT_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GRADIENT_BACKGROUND_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GRADIENT_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GRADIENT_BACKGROUND_height_get" "', argument " "1"" of type '" "struct _dwg_object_GRADIENT_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_GRADIENT_BACKGROUND *)(argp1);
    result = (BITCODE_BD) ((arg1)->height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GRADIENT_BACKGROUND_rotation_set) {
  {
    struct _dwg_object_GRADIENT_BACKGROUND *arg1 = (struct _dwg_object_GRADIENT_BACKGROUND *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GRADIENT_BACKGROUND_rotation_set(self,rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GRADIENT_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GRADIENT_BACKGROUND_rotation_set" "', argument " "1"" of type '" "struct _dwg_object_GRADIENT_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_GRADIENT_BACKGROUND *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GRADIENT_BACKGROUND_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GRADIENT_BACKGROUND_rotation_get) {
  {
    struct _dwg_object_GRADIENT_BACKGROUND *arg1 = (struct _dwg_object_GRADIENT_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GRADIENT_BACKGROUND_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GRADIENT_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GRADIENT_BACKGROUND_rotation_get" "', argument " "1"" of type '" "struct _dwg_object_GRADIENT_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_GRADIENT_BACKGROUND *)(argp1);
    result = (BITCODE_BD) ((arg1)->rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_GRADIENT_BACKGROUND) {
  {
    int argvi = 0;
    struct _dwg_object_GRADIENT_BACKGROUND *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_GRADIENT_BACKGROUND();");
    }
    result = (struct _dwg_object_GRADIENT_BACKGROUND *)calloc(1, sizeof(struct _dwg_object_GRADIENT_BACKGROUND));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_GRADIENT_BACKGROUND, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_GRADIENT_BACKGROUND) {
  {
    struct _dwg_object_GRADIENT_BACKGROUND *arg1 = (struct _dwg_object_GRADIENT_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_GRADIENT_BACKGROUND(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GRADIENT_BACKGROUND, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_GRADIENT_BACKGROUND" "', argument " "1"" of type '" "struct _dwg_object_GRADIENT_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_GRADIENT_BACKGROUND *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GROUND_PLANE_BACKGROUND_parent_set) {
  {
    struct _dwg_object_GROUND_PLANE_BACKGROUND *arg1 = (struct _dwg_object_GROUND_PLANE_BACKGROUND *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GROUND_PLANE_BACKGROUND_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GROUND_PLANE_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GROUND_PLANE_BACKGROUND_parent_set" "', argument " "1"" of type '" "struct _dwg_object_GROUND_PLANE_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_GROUND_PLANE_BACKGROUND *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_GROUND_PLANE_BACKGROUND_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GROUND_PLANE_BACKGROUND_parent_get) {
  {
    struct _dwg_object_GROUND_PLANE_BACKGROUND *arg1 = (struct _dwg_object_GROUND_PLANE_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GROUND_PLANE_BACKGROUND_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GROUND_PLANE_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GROUND_PLANE_BACKGROUND_parent_get" "', argument " "1"" of type '" "struct _dwg_object_GROUND_PLANE_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_GROUND_PLANE_BACKGROUND *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GROUND_PLANE_BACKGROUND_class_version_set) {
  {
    struct _dwg_object_GROUND_PLANE_BACKGROUND *arg1 = (struct _dwg_object_GROUND_PLANE_BACKGROUND *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GROUND_PLANE_BACKGROUND_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GROUND_PLANE_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GROUND_PLANE_BACKGROUND_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_GROUND_PLANE_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_GROUND_PLANE_BACKGROUND *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GROUND_PLANE_BACKGROUND_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GROUND_PLANE_BACKGROUND_class_version_get) {
  {
    struct _dwg_object_GROUND_PLANE_BACKGROUND *arg1 = (struct _dwg_object_GROUND_PLANE_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GROUND_PLANE_BACKGROUND_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GROUND_PLANE_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GROUND_PLANE_BACKGROUND_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_GROUND_PLANE_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_GROUND_PLANE_BACKGROUND *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GROUND_PLANE_BACKGROUND_color_sky_zenith_set) {
  {
    struct _dwg_object_GROUND_PLANE_BACKGROUND *arg1 = (struct _dwg_object_GROUND_PLANE_BACKGROUND *) 0 ;
    BITCODE_BLx arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GROUND_PLANE_BACKGROUND_color_sky_zenith_set(self,color_sky_zenith);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GROUND_PLANE_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GROUND_PLANE_BACKGROUND_color_sky_zenith_set" "', argument " "1"" of type '" "struct _dwg_object_GROUND_PLANE_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_GROUND_PLANE_BACKGROUND *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GROUND_PLANE_BACKGROUND_color_sky_zenith_set" "', argument " "2"" of type '" "BITCODE_BLx""'");
    } 
    arg2 = (BITCODE_BLx)(val2);
    if (arg1) (arg1)->color_sky_zenith = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GROUND_PLANE_BACKGROUND_color_sky_zenith_get) {
  {
    struct _dwg_object_GROUND_PLANE_BACKGROUND *arg1 = (struct _dwg_object_GROUND_PLANE_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLx result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GROUND_PLANE_BACKGROUND_color_sky_zenith_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GROUND_PLANE_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GROUND_PLANE_BACKGROUND_color_sky_zenith_get" "', argument " "1"" of type '" "struct _dwg_object_GROUND_PLANE_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_GROUND_PLANE_BACKGROUND *)(argp1);
    result = (BITCODE_BLx) ((arg1)->color_sky_zenith);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GROUND_PLANE_BACKGROUND_color_sky_horizon_set) {
  {
    struct _dwg_object_GROUND_PLANE_BACKGROUND *arg1 = (struct _dwg_object_GROUND_PLANE_BACKGROUND *) 0 ;
    BITCODE_BLx arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GROUND_PLANE_BACKGROUND_color_sky_horizon_set(self,color_sky_horizon);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GROUND_PLANE_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GROUND_PLANE_BACKGROUND_color_sky_horizon_set" "', argument " "1"" of type '" "struct _dwg_object_GROUND_PLANE_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_GROUND_PLANE_BACKGROUND *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GROUND_PLANE_BACKGROUND_color_sky_horizon_set" "', argument " "2"" of type '" "BITCODE_BLx""'");
    } 
    arg2 = (BITCODE_BLx)(val2);
    if (arg1) (arg1)->color_sky_horizon = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GROUND_PLANE_BACKGROUND_color_sky_horizon_get) {
  {
    struct _dwg_object_GROUND_PLANE_BACKGROUND *arg1 = (struct _dwg_object_GROUND_PLANE_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLx result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GROUND_PLANE_BACKGROUND_color_sky_horizon_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GROUND_PLANE_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GROUND_PLANE_BACKGROUND_color_sky_horizon_get" "', argument " "1"" of type '" "struct _dwg_object_GROUND_PLANE_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_GROUND_PLANE_BACKGROUND *)(argp1);
    result = (BITCODE_BLx) ((arg1)->color_sky_horizon);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GROUND_PLANE_BACKGROUND_color_underground_horizon_set) {
  {
    struct _dwg_object_GROUND_PLANE_BACKGROUND *arg1 = (struct _dwg_object_GROUND_PLANE_BACKGROUND *) 0 ;
    BITCODE_BLx arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GROUND_PLANE_BACKGROUND_color_underground_horizon_set(self,color_underground_horizon);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GROUND_PLANE_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GROUND_PLANE_BACKGROUND_color_underground_horizon_set" "', argument " "1"" of type '" "struct _dwg_object_GROUND_PLANE_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_GROUND_PLANE_BACKGROUND *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GROUND_PLANE_BACKGROUND_color_underground_horizon_set" "', argument " "2"" of type '" "BITCODE_BLx""'");
    } 
    arg2 = (BITCODE_BLx)(val2);
    if (arg1) (arg1)->color_underground_horizon = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GROUND_PLANE_BACKGROUND_color_underground_horizon_get) {
  {
    struct _dwg_object_GROUND_PLANE_BACKGROUND *arg1 = (struct _dwg_object_GROUND_PLANE_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLx result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GROUND_PLANE_BACKGROUND_color_underground_horizon_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GROUND_PLANE_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GROUND_PLANE_BACKGROUND_color_underground_horizon_get" "', argument " "1"" of type '" "struct _dwg_object_GROUND_PLANE_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_GROUND_PLANE_BACKGROUND *)(argp1);
    result = (BITCODE_BLx) ((arg1)->color_underground_horizon);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GROUND_PLANE_BACKGROUND_color_underground_azimuth_set) {
  {
    struct _dwg_object_GROUND_PLANE_BACKGROUND *arg1 = (struct _dwg_object_GROUND_PLANE_BACKGROUND *) 0 ;
    BITCODE_BLx arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GROUND_PLANE_BACKGROUND_color_underground_azimuth_set(self,color_underground_azimuth);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GROUND_PLANE_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GROUND_PLANE_BACKGROUND_color_underground_azimuth_set" "', argument " "1"" of type '" "struct _dwg_object_GROUND_PLANE_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_GROUND_PLANE_BACKGROUND *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GROUND_PLANE_BACKGROUND_color_underground_azimuth_set" "', argument " "2"" of type '" "BITCODE_BLx""'");
    } 
    arg2 = (BITCODE_BLx)(val2);
    if (arg1) (arg1)->color_underground_azimuth = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GROUND_PLANE_BACKGROUND_color_underground_azimuth_get) {
  {
    struct _dwg_object_GROUND_PLANE_BACKGROUND *arg1 = (struct _dwg_object_GROUND_PLANE_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLx result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GROUND_PLANE_BACKGROUND_color_underground_azimuth_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GROUND_PLANE_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GROUND_PLANE_BACKGROUND_color_underground_azimuth_get" "', argument " "1"" of type '" "struct _dwg_object_GROUND_PLANE_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_GROUND_PLANE_BACKGROUND *)(argp1);
    result = (BITCODE_BLx) ((arg1)->color_underground_azimuth);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GROUND_PLANE_BACKGROUND_color_near_set) {
  {
    struct _dwg_object_GROUND_PLANE_BACKGROUND *arg1 = (struct _dwg_object_GROUND_PLANE_BACKGROUND *) 0 ;
    BITCODE_BLx arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GROUND_PLANE_BACKGROUND_color_near_set(self,color_near);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GROUND_PLANE_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GROUND_PLANE_BACKGROUND_color_near_set" "', argument " "1"" of type '" "struct _dwg_object_GROUND_PLANE_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_GROUND_PLANE_BACKGROUND *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GROUND_PLANE_BACKGROUND_color_near_set" "', argument " "2"" of type '" "BITCODE_BLx""'");
    } 
    arg2 = (BITCODE_BLx)(val2);
    if (arg1) (arg1)->color_near = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GROUND_PLANE_BACKGROUND_color_near_get) {
  {
    struct _dwg_object_GROUND_PLANE_BACKGROUND *arg1 = (struct _dwg_object_GROUND_PLANE_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLx result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GROUND_PLANE_BACKGROUND_color_near_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GROUND_PLANE_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GROUND_PLANE_BACKGROUND_color_near_get" "', argument " "1"" of type '" "struct _dwg_object_GROUND_PLANE_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_GROUND_PLANE_BACKGROUND *)(argp1);
    result = (BITCODE_BLx) ((arg1)->color_near);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GROUND_PLANE_BACKGROUND_color_far_set) {
  {
    struct _dwg_object_GROUND_PLANE_BACKGROUND *arg1 = (struct _dwg_object_GROUND_PLANE_BACKGROUND *) 0 ;
    BITCODE_BLx arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_GROUND_PLANE_BACKGROUND_color_far_set(self,color_far);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GROUND_PLANE_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GROUND_PLANE_BACKGROUND_color_far_set" "', argument " "1"" of type '" "struct _dwg_object_GROUND_PLANE_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_GROUND_PLANE_BACKGROUND *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_GROUND_PLANE_BACKGROUND_color_far_set" "', argument " "2"" of type '" "BITCODE_BLx""'");
    } 
    arg2 = (BITCODE_BLx)(val2);
    if (arg1) (arg1)->color_far = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_GROUND_PLANE_BACKGROUND_color_far_get) {
  {
    struct _dwg_object_GROUND_PLANE_BACKGROUND *arg1 = (struct _dwg_object_GROUND_PLANE_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLx result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_GROUND_PLANE_BACKGROUND_color_far_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GROUND_PLANE_BACKGROUND, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_GROUND_PLANE_BACKGROUND_color_far_get" "', argument " "1"" of type '" "struct _dwg_object_GROUND_PLANE_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_GROUND_PLANE_BACKGROUND *)(argp1);
    result = (BITCODE_BLx) ((arg1)->color_far);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_GROUND_PLANE_BACKGROUND) {
  {
    int argvi = 0;
    struct _dwg_object_GROUND_PLANE_BACKGROUND *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_GROUND_PLANE_BACKGROUND();");
    }
    result = (struct _dwg_object_GROUND_PLANE_BACKGROUND *)calloc(1, sizeof(struct _dwg_object_GROUND_PLANE_BACKGROUND));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_GROUND_PLANE_BACKGROUND, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_GROUND_PLANE_BACKGROUND) {
  {
    struct _dwg_object_GROUND_PLANE_BACKGROUND *arg1 = (struct _dwg_object_GROUND_PLANE_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_GROUND_PLANE_BACKGROUND(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_GROUND_PLANE_BACKGROUND, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_GROUND_PLANE_BACKGROUND" "', argument " "1"" of type '" "struct _dwg_object_GROUND_PLANE_BACKGROUND *""'"); 
    }
    arg1 = (struct _dwg_object_GROUND_PLANE_BACKGROUND *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_parent_set) {
  {
    struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ANNOTSCALEOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_parent_get) {
  {
    struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ANNOTSCALEOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_class_version_set) {
  {
    struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ANNOTSCALEOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_class_version_get) {
  {
    struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ANNOTSCALEOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_is_default_set) {
  {
    struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_is_default_set(self,is_default);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ANNOTSCALEOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_is_default_set" "', argument " "1"" of type '" "struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_is_default_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_default = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_is_default_get) {
  {
    struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_is_default_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ANNOTSCALEOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_is_default_get" "', argument " "1"" of type '" "struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_B) ((arg1)->is_default);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_scale_set) {
  {
    struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_scale_set(self,scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ANNOTSCALEOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_scale_set" "', argument " "1"" of type '" "struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_scale_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->scale = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_scale_get) {
  {
    struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ANNOTSCALEOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_scale_get" "', argument " "1"" of type '" "struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_H) ((arg1)->scale);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA) {
  {
    int argvi = 0;
    struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA();");
    }
    result = (struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *)calloc(1, sizeof(struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ANNOTSCALEOBJECTCONTEXTDATA, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA) {
  {
    struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ANNOTSCALEOBJECTCONTEXTDATA, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA" "', argument " "1"" of type '" "struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CONTEXTDATA_dict_parent_set) {
  {
    struct _dwg_CONTEXTDATA_dict *arg1 = (struct _dwg_CONTEXTDATA_dict *) 0 ;
    struct _dwg_CONTEXTDATA_submgr *arg2 = (struct _dwg_CONTEXTDATA_submgr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CONTEXTDATA_dict_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CONTEXTDATA_dict, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CONTEXTDATA_dict_parent_set" "', argument " "1"" of type '" "struct _dwg_CONTEXTDATA_dict *""'"); 
    }
    arg1 = (struct _dwg_CONTEXTDATA_dict *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_CONTEXTDATA_submgr, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_CONTEXTDATA_dict_parent_set" "', argument " "2"" of type '" "struct _dwg_CONTEXTDATA_submgr *""'"); 
    }
    arg2 = (struct _dwg_CONTEXTDATA_submgr *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CONTEXTDATA_dict_parent_get) {
  {
    struct _dwg_CONTEXTDATA_dict *arg1 = (struct _dwg_CONTEXTDATA_dict *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_CONTEXTDATA_submgr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CONTEXTDATA_dict_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CONTEXTDATA_dict, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CONTEXTDATA_dict_parent_get" "', argument " "1"" of type '" "struct _dwg_CONTEXTDATA_dict *""'"); 
    }
    arg1 = (struct _dwg_CONTEXTDATA_dict *)(argp1);
    result = (struct _dwg_CONTEXTDATA_submgr *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_CONTEXTDATA_submgr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CONTEXTDATA_dict_text_set) {
  {
    struct _dwg_CONTEXTDATA_dict *arg1 = (struct _dwg_CONTEXTDATA_dict *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CONTEXTDATA_dict_text_set(self,text);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CONTEXTDATA_dict, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CONTEXTDATA_dict_text_set" "', argument " "1"" of type '" "struct _dwg_CONTEXTDATA_dict *""'"); 
    }
    arg1 = (struct _dwg_CONTEXTDATA_dict *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_CONTEXTDATA_dict_text_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->text);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->text = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->text = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CONTEXTDATA_dict_text_get) {
  {
    struct _dwg_CONTEXTDATA_dict *arg1 = (struct _dwg_CONTEXTDATA_dict *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CONTEXTDATA_dict_text_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CONTEXTDATA_dict, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CONTEXTDATA_dict_text_get" "', argument " "1"" of type '" "struct _dwg_CONTEXTDATA_dict *""'"); 
    }
    arg1 = (struct _dwg_CONTEXTDATA_dict *)(argp1);
    result = (BITCODE_TV) ((arg1)->text);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CONTEXTDATA_dict_itemhandle_set) {
  {
    struct _dwg_CONTEXTDATA_dict *arg1 = (struct _dwg_CONTEXTDATA_dict *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CONTEXTDATA_dict_itemhandle_set(self,itemhandle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CONTEXTDATA_dict, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CONTEXTDATA_dict_itemhandle_set" "', argument " "1"" of type '" "struct _dwg_CONTEXTDATA_dict *""'"); 
    }
    arg1 = (struct _dwg_CONTEXTDATA_dict *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_CONTEXTDATA_dict_itemhandle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->itemhandle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CONTEXTDATA_dict_itemhandle_get) {
  {
    struct _dwg_CONTEXTDATA_dict *arg1 = (struct _dwg_CONTEXTDATA_dict *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CONTEXTDATA_dict_itemhandle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CONTEXTDATA_dict, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CONTEXTDATA_dict_itemhandle_get" "', argument " "1"" of type '" "struct _dwg_CONTEXTDATA_dict *""'"); 
    }
    arg1 = (struct _dwg_CONTEXTDATA_dict *)(argp1);
    result = (BITCODE_H) ((arg1)->itemhandle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_CONTEXTDATA_dict) {
  {
    int argvi = 0;
    struct _dwg_CONTEXTDATA_dict *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_CONTEXTDATA_dict();");
    }
    result = (struct _dwg_CONTEXTDATA_dict *)calloc(1, sizeof(struct _dwg_CONTEXTDATA_dict));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_CONTEXTDATA_dict, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_CONTEXTDATA_dict) {
  {
    struct _dwg_CONTEXTDATA_dict *arg1 = (struct _dwg_CONTEXTDATA_dict *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_CONTEXTDATA_dict(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CONTEXTDATA_dict, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_CONTEXTDATA_dict" "', argument " "1"" of type '" "struct _dwg_CONTEXTDATA_dict *""'"); 
    }
    arg1 = (struct _dwg_CONTEXTDATA_dict *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CONTEXTDATA_submgr_parent_set) {
  {
    struct _dwg_CONTEXTDATA_submgr *arg1 = (struct _dwg_CONTEXTDATA_submgr *) 0 ;
    struct _dwg_object_CONTEXTDATAMANAGER *arg2 = (struct _dwg_object_CONTEXTDATAMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CONTEXTDATA_submgr_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CONTEXTDATA_submgr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CONTEXTDATA_submgr_parent_set" "', argument " "1"" of type '" "struct _dwg_CONTEXTDATA_submgr *""'"); 
    }
    arg1 = (struct _dwg_CONTEXTDATA_submgr *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_CONTEXTDATAMANAGER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_CONTEXTDATA_submgr_parent_set" "', argument " "2"" of type '" "struct _dwg_object_CONTEXTDATAMANAGER *""'"); 
    }
    arg2 = (struct _dwg_object_CONTEXTDATAMANAGER *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CONTEXTDATA_submgr_parent_get) {
  {
    struct _dwg_CONTEXTDATA_submgr *arg1 = (struct _dwg_CONTEXTDATA_submgr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_CONTEXTDATAMANAGER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CONTEXTDATA_submgr_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CONTEXTDATA_submgr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CONTEXTDATA_submgr_parent_get" "', argument " "1"" of type '" "struct _dwg_CONTEXTDATA_submgr *""'"); 
    }
    arg1 = (struct _dwg_CONTEXTDATA_submgr *)(argp1);
    result = (struct _dwg_object_CONTEXTDATAMANAGER *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_CONTEXTDATAMANAGER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CONTEXTDATA_submgr_handle_set) {
  {
    struct _dwg_CONTEXTDATA_submgr *arg1 = (struct _dwg_CONTEXTDATA_submgr *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CONTEXTDATA_submgr_handle_set(self,handle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CONTEXTDATA_submgr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CONTEXTDATA_submgr_handle_set" "', argument " "1"" of type '" "struct _dwg_CONTEXTDATA_submgr *""'"); 
    }
    arg1 = (struct _dwg_CONTEXTDATA_submgr *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_CONTEXTDATA_submgr_handle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->handle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CONTEXTDATA_submgr_handle_get) {
  {
    struct _dwg_CONTEXTDATA_submgr *arg1 = (struct _dwg_CONTEXTDATA_submgr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CONTEXTDATA_submgr_handle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CONTEXTDATA_submgr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CONTEXTDATA_submgr_handle_get" "', argument " "1"" of type '" "struct _dwg_CONTEXTDATA_submgr *""'"); 
    }
    arg1 = (struct _dwg_CONTEXTDATA_submgr *)(argp1);
    result = (BITCODE_H) ((arg1)->handle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CONTEXTDATA_submgr_num_entries_set) {
  {
    struct _dwg_CONTEXTDATA_submgr *arg1 = (struct _dwg_CONTEXTDATA_submgr *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CONTEXTDATA_submgr_num_entries_set(self,num_entries);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CONTEXTDATA_submgr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CONTEXTDATA_submgr_num_entries_set" "', argument " "1"" of type '" "struct _dwg_CONTEXTDATA_submgr *""'"); 
    }
    arg1 = (struct _dwg_CONTEXTDATA_submgr *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_CONTEXTDATA_submgr_num_entries_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_entries = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CONTEXTDATA_submgr_num_entries_get) {
  {
    struct _dwg_CONTEXTDATA_submgr *arg1 = (struct _dwg_CONTEXTDATA_submgr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CONTEXTDATA_submgr_num_entries_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CONTEXTDATA_submgr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CONTEXTDATA_submgr_num_entries_get" "', argument " "1"" of type '" "struct _dwg_CONTEXTDATA_submgr *""'"); 
    }
    arg1 = (struct _dwg_CONTEXTDATA_submgr *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_entries);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CONTEXTDATA_submgr_entries_set) {
  {
    struct _dwg_CONTEXTDATA_submgr *arg1 = (struct _dwg_CONTEXTDATA_submgr *) 0 ;
    Dwg_CONTEXTDATA_dict *arg2 = (Dwg_CONTEXTDATA_dict *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_CONTEXTDATA_submgr_entries_set(self,entries);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CONTEXTDATA_submgr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CONTEXTDATA_submgr_entries_set" "', argument " "1"" of type '" "struct _dwg_CONTEXTDATA_submgr *""'"); 
    }
    arg1 = (struct _dwg_CONTEXTDATA_submgr *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_CONTEXTDATA_dict, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_CONTEXTDATA_submgr_entries_set" "', argument " "2"" of type '" "Dwg_CONTEXTDATA_dict *""'"); 
    }
    arg2 = (Dwg_CONTEXTDATA_dict *)(argp2);
    if (arg1) (arg1)->entries = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_CONTEXTDATA_submgr_entries_get) {
  {
    struct _dwg_CONTEXTDATA_submgr *arg1 = (struct _dwg_CONTEXTDATA_submgr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_CONTEXTDATA_dict *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_CONTEXTDATA_submgr_entries_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CONTEXTDATA_submgr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_CONTEXTDATA_submgr_entries_get" "', argument " "1"" of type '" "struct _dwg_CONTEXTDATA_submgr *""'"); 
    }
    arg1 = (struct _dwg_CONTEXTDATA_submgr *)(argp1);
    result = (Dwg_CONTEXTDATA_dict *) ((arg1)->entries);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_CONTEXTDATA_dict, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_CONTEXTDATA_submgr) {
  {
    int argvi = 0;
    struct _dwg_CONTEXTDATA_submgr *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_CONTEXTDATA_submgr();");
    }
    result = (struct _dwg_CONTEXTDATA_submgr *)calloc(1, sizeof(struct _dwg_CONTEXTDATA_submgr));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_CONTEXTDATA_submgr, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_CONTEXTDATA_submgr) {
  {
    struct _dwg_CONTEXTDATA_submgr *arg1 = (struct _dwg_CONTEXTDATA_submgr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_CONTEXTDATA_submgr(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_CONTEXTDATA_submgr, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_CONTEXTDATA_submgr" "', argument " "1"" of type '" "struct _dwg_CONTEXTDATA_submgr *""'"); 
    }
    arg1 = (struct _dwg_CONTEXTDATA_submgr *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_CONTEXTDATAMANAGER_parent_set) {
  {
    struct _dwg_object_CONTEXTDATAMANAGER *arg1 = (struct _dwg_object_CONTEXTDATAMANAGER *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_CONTEXTDATAMANAGER_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_CONTEXTDATAMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_CONTEXTDATAMANAGER_parent_set" "', argument " "1"" of type '" "struct _dwg_object_CONTEXTDATAMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_CONTEXTDATAMANAGER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_CONTEXTDATAMANAGER_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_CONTEXTDATAMANAGER_parent_get) {
  {
    struct _dwg_object_CONTEXTDATAMANAGER *arg1 = (struct _dwg_object_CONTEXTDATAMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_CONTEXTDATAMANAGER_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_CONTEXTDATAMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_CONTEXTDATAMANAGER_parent_get" "', argument " "1"" of type '" "struct _dwg_object_CONTEXTDATAMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_CONTEXTDATAMANAGER *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_CONTEXTDATAMANAGER_objectcontext_set) {
  {
    struct _dwg_object_CONTEXTDATAMANAGER *arg1 = (struct _dwg_object_CONTEXTDATAMANAGER *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_CONTEXTDATAMANAGER_objectcontext_set(self,objectcontext);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_CONTEXTDATAMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_CONTEXTDATAMANAGER_objectcontext_set" "', argument " "1"" of type '" "struct _dwg_object_CONTEXTDATAMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_CONTEXTDATAMANAGER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_CONTEXTDATAMANAGER_objectcontext_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->objectcontext = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_CONTEXTDATAMANAGER_objectcontext_get) {
  {
    struct _dwg_object_CONTEXTDATAMANAGER *arg1 = (struct _dwg_object_CONTEXTDATAMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_CONTEXTDATAMANAGER_objectcontext_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_CONTEXTDATAMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_CONTEXTDATAMANAGER_objectcontext_get" "', argument " "1"" of type '" "struct _dwg_object_CONTEXTDATAMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_CONTEXTDATAMANAGER *)(argp1);
    result = (BITCODE_H) ((arg1)->objectcontext);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_CONTEXTDATAMANAGER_num_submgrs_set) {
  {
    struct _dwg_object_CONTEXTDATAMANAGER *arg1 = (struct _dwg_object_CONTEXTDATAMANAGER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_CONTEXTDATAMANAGER_num_submgrs_set(self,num_submgrs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_CONTEXTDATAMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_CONTEXTDATAMANAGER_num_submgrs_set" "', argument " "1"" of type '" "struct _dwg_object_CONTEXTDATAMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_CONTEXTDATAMANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_CONTEXTDATAMANAGER_num_submgrs_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_submgrs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_CONTEXTDATAMANAGER_num_submgrs_get) {
  {
    struct _dwg_object_CONTEXTDATAMANAGER *arg1 = (struct _dwg_object_CONTEXTDATAMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_CONTEXTDATAMANAGER_num_submgrs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_CONTEXTDATAMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_CONTEXTDATAMANAGER_num_submgrs_get" "', argument " "1"" of type '" "struct _dwg_object_CONTEXTDATAMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_CONTEXTDATAMANAGER *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_submgrs);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_CONTEXTDATAMANAGER_submgrs_set) {
  {
    struct _dwg_object_CONTEXTDATAMANAGER *arg1 = (struct _dwg_object_CONTEXTDATAMANAGER *) 0 ;
    Dwg_CONTEXTDATA_submgr *arg2 = (Dwg_CONTEXTDATA_submgr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_CONTEXTDATAMANAGER_submgrs_set(self,submgrs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_CONTEXTDATAMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_CONTEXTDATAMANAGER_submgrs_set" "', argument " "1"" of type '" "struct _dwg_object_CONTEXTDATAMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_CONTEXTDATAMANAGER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_CONTEXTDATA_submgr, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_CONTEXTDATAMANAGER_submgrs_set" "', argument " "2"" of type '" "Dwg_CONTEXTDATA_submgr *""'"); 
    }
    arg2 = (Dwg_CONTEXTDATA_submgr *)(argp2);
    if (arg1) (arg1)->submgrs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_CONTEXTDATAMANAGER_submgrs_get) {
  {
    struct _dwg_object_CONTEXTDATAMANAGER *arg1 = (struct _dwg_object_CONTEXTDATAMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_CONTEXTDATA_submgr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_CONTEXTDATAMANAGER_submgrs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_CONTEXTDATAMANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_CONTEXTDATAMANAGER_submgrs_get" "', argument " "1"" of type '" "struct _dwg_object_CONTEXTDATAMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_CONTEXTDATAMANAGER *)(argp1);
    result = (Dwg_CONTEXTDATA_submgr *) ((arg1)->submgrs);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_CONTEXTDATA_submgr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_CONTEXTDATAMANAGER) {
  {
    int argvi = 0;
    struct _dwg_object_CONTEXTDATAMANAGER *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_CONTEXTDATAMANAGER();");
    }
    result = (struct _dwg_object_CONTEXTDATAMANAGER *)calloc(1, sizeof(struct _dwg_object_CONTEXTDATAMANAGER));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_CONTEXTDATAMANAGER, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_CONTEXTDATAMANAGER) {
  {
    struct _dwg_object_CONTEXTDATAMANAGER *arg1 = (struct _dwg_object_CONTEXTDATAMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_CONTEXTDATAMANAGER(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_CONTEXTDATAMANAGER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_CONTEXTDATAMANAGER" "', argument " "1"" of type '" "struct _dwg_object_CONTEXTDATAMANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_CONTEXTDATAMANAGER *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TEXTOBJECTCONTEXTDATA_parent_set) {
  {
    struct _dwg_object_TEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_TEXTOBJECTCONTEXTDATA *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TEXTOBJECTCONTEXTDATA_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TEXTOBJECTCONTEXTDATA_parent_set" "', argument " "1"" of type '" "struct _dwg_object_TEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_TEXTOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_TEXTOBJECTCONTEXTDATA_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TEXTOBJECTCONTEXTDATA_parent_get) {
  {
    struct _dwg_object_TEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_TEXTOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TEXTOBJECTCONTEXTDATA_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TEXTOBJECTCONTEXTDATA_parent_get" "', argument " "1"" of type '" "struct _dwg_object_TEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_TEXTOBJECTCONTEXTDATA *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TEXTOBJECTCONTEXTDATA_class_version_set) {
  {
    struct _dwg_object_TEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_TEXTOBJECTCONTEXTDATA *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TEXTOBJECTCONTEXTDATA_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TEXTOBJECTCONTEXTDATA_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_TEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_TEXTOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_TEXTOBJECTCONTEXTDATA_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TEXTOBJECTCONTEXTDATA_class_version_get) {
  {
    struct _dwg_object_TEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_TEXTOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TEXTOBJECTCONTEXTDATA_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TEXTOBJECTCONTEXTDATA_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_TEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_TEXTOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TEXTOBJECTCONTEXTDATA_is_default_set) {
  {
    struct _dwg_object_TEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_TEXTOBJECTCONTEXTDATA *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TEXTOBJECTCONTEXTDATA_is_default_set(self,is_default);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TEXTOBJECTCONTEXTDATA_is_default_set" "', argument " "1"" of type '" "struct _dwg_object_TEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_TEXTOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_TEXTOBJECTCONTEXTDATA_is_default_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_default = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TEXTOBJECTCONTEXTDATA_is_default_get) {
  {
    struct _dwg_object_TEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_TEXTOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TEXTOBJECTCONTEXTDATA_is_default_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TEXTOBJECTCONTEXTDATA_is_default_get" "', argument " "1"" of type '" "struct _dwg_object_TEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_TEXTOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_B) ((arg1)->is_default);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TEXTOBJECTCONTEXTDATA_scale_set) {
  {
    struct _dwg_object_TEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_TEXTOBJECTCONTEXTDATA *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TEXTOBJECTCONTEXTDATA_scale_set(self,scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TEXTOBJECTCONTEXTDATA_scale_set" "', argument " "1"" of type '" "struct _dwg_object_TEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_TEXTOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_TEXTOBJECTCONTEXTDATA_scale_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->scale = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TEXTOBJECTCONTEXTDATA_scale_get) {
  {
    struct _dwg_object_TEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_TEXTOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TEXTOBJECTCONTEXTDATA_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TEXTOBJECTCONTEXTDATA_scale_get" "', argument " "1"" of type '" "struct _dwg_object_TEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_TEXTOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_H) ((arg1)->scale);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TEXTOBJECTCONTEXTDATA_horizontal_mode_set) {
  {
    struct _dwg_object_TEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_TEXTOBJECTCONTEXTDATA *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TEXTOBJECTCONTEXTDATA_horizontal_mode_set(self,horizontal_mode);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TEXTOBJECTCONTEXTDATA_horizontal_mode_set" "', argument " "1"" of type '" "struct _dwg_object_TEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_TEXTOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_TEXTOBJECTCONTEXTDATA_horizontal_mode_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->horizontal_mode = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TEXTOBJECTCONTEXTDATA_horizontal_mode_get) {
  {
    struct _dwg_object_TEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_TEXTOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TEXTOBJECTCONTEXTDATA_horizontal_mode_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TEXTOBJECTCONTEXTDATA_horizontal_mode_get" "', argument " "1"" of type '" "struct _dwg_object_TEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_TEXTOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_BS) ((arg1)->horizontal_mode);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TEXTOBJECTCONTEXTDATA_rotation_set) {
  {
    struct _dwg_object_TEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_TEXTOBJECTCONTEXTDATA *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TEXTOBJECTCONTEXTDATA_rotation_set(self,rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TEXTOBJECTCONTEXTDATA_rotation_set" "', argument " "1"" of type '" "struct _dwg_object_TEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_TEXTOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_TEXTOBJECTCONTEXTDATA_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TEXTOBJECTCONTEXTDATA_rotation_get) {
  {
    struct _dwg_object_TEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_TEXTOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TEXTOBJECTCONTEXTDATA_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TEXTOBJECTCONTEXTDATA_rotation_get" "', argument " "1"" of type '" "struct _dwg_object_TEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_TEXTOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_BD) ((arg1)->rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TEXTOBJECTCONTEXTDATA_ins_pt_set) {
  {
    struct _dwg_object_TEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_TEXTOBJECTCONTEXTDATA *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TEXTOBJECTCONTEXTDATA_ins_pt_set(self,ins_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TEXTOBJECTCONTEXTDATA_ins_pt_set" "', argument " "1"" of type '" "struct _dwg_object_TEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_TEXTOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_TEXTOBJECTCONTEXTDATA_ins_pt_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->ins_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TEXTOBJECTCONTEXTDATA_ins_pt_get) {
  {
    struct _dwg_object_TEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_TEXTOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TEXTOBJECTCONTEXTDATA_ins_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TEXTOBJECTCONTEXTDATA_ins_pt_get" "', argument " "1"" of type '" "struct _dwg_object_TEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_TEXTOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->ins_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TEXTOBJECTCONTEXTDATA_alignment_pt_set) {
  {
    struct _dwg_object_TEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_TEXTOBJECTCONTEXTDATA *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_TEXTOBJECTCONTEXTDATA_alignment_pt_set(self,alignment_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TEXTOBJECTCONTEXTDATA_alignment_pt_set" "', argument " "1"" of type '" "struct _dwg_object_TEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_TEXTOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_TEXTOBJECTCONTEXTDATA_alignment_pt_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->alignment_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_TEXTOBJECTCONTEXTDATA_alignment_pt_get) {
  {
    struct _dwg_object_TEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_TEXTOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_TEXTOBJECTCONTEXTDATA_alignment_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_TEXTOBJECTCONTEXTDATA_alignment_pt_get" "', argument " "1"" of type '" "struct _dwg_object_TEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_TEXTOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->alignment_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_TEXTOBJECTCONTEXTDATA) {
  {
    int argvi = 0;
    struct _dwg_object_TEXTOBJECTCONTEXTDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_TEXTOBJECTCONTEXTDATA();");
    }
    result = (struct _dwg_object_TEXTOBJECTCONTEXTDATA *)calloc(1, sizeof(struct _dwg_object_TEXTOBJECTCONTEXTDATA));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_TEXTOBJECTCONTEXTDATA, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_TEXTOBJECTCONTEXTDATA) {
  {
    struct _dwg_object_TEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_TEXTOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_TEXTOBJECTCONTEXTDATA(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_TEXTOBJECTCONTEXTDATA, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_TEXTOBJECTCONTEXTDATA" "', argument " "1"" of type '" "struct _dwg_object_TEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_TEXTOBJECTCONTEXTDATA *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_parent_set) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_parent_set" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_parent_get) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_parent_get" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_class_version_set) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_class_version_get) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_is_default_set) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_is_default_set(self,is_default);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_is_default_set" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_is_default_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_default = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_is_default_get) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_is_default_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_is_default_get" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_B) ((arg1)->is_default);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_scale_set) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_scale_set(self,scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_scale_set" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_scale_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->scale = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_scale_get) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_scale_get" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_H) ((arg1)->scale);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_attachment_set) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_attachment_set(self,attachment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_attachment_set" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_attachment_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->attachment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_attachment_get) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_attachment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_attachment_get" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_BL) ((arg1)->attachment);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_ins_pt_set) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_ins_pt_set(self,ins_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_ins_pt_set" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_ins_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ins_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_ins_pt_get) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_ins_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_ins_pt_get" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ins_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_x_axis_dir_set) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_x_axis_dir_set(self,x_axis_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_x_axis_dir_set" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_x_axis_dir_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->x_axis_dir = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_x_axis_dir_get) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_x_axis_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_x_axis_dir_get" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->x_axis_dir);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_rect_height_set) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_rect_height_set(self,rect_height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_rect_height_set" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_rect_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->rect_height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_rect_height_get) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_rect_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_rect_height_get" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_BD) ((arg1)->rect_height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_rect_width_set) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_rect_width_set(self,rect_width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_rect_width_set" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_rect_width_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->rect_width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_rect_width_get) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_rect_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_rect_width_get" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_BD) ((arg1)->rect_width);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_extents_width_set) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_extents_width_set(self,extents_width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_extents_width_set" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_extents_width_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->extents_width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_extents_width_get) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_extents_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_extents_width_get" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_BD) ((arg1)->extents_width);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_extents_height_set) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_extents_height_set(self,extents_height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_extents_height_set" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_extents_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->extents_height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_extents_height_get) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_extents_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_extents_height_get" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_BD) ((arg1)->extents_height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_type_set) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_type_set(self,column_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_type_set" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_type_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->column_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_type_get) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_type_get" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_BL) ((arg1)->column_type);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_width_set) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_width_set(self,column_width);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_width_set" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_width_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->column_width = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_width_get) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_width_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_width_get" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_BD) ((arg1)->column_width);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_gutter_set) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_gutter_set(self,gutter);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_gutter_set" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_gutter_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->gutter = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_gutter_get) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_gutter_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_gutter_get" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_BD) ((arg1)->gutter);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_auto_height_set) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_auto_height_set(self,auto_height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_auto_height_set" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_auto_height_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->auto_height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_auto_height_get) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_auto_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_auto_height_get" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_B) ((arg1)->auto_height);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_flow_reversed_set) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_flow_reversed_set(self,flow_reversed);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_flow_reversed_set" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_flow_reversed_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->flow_reversed = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_flow_reversed_get) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_flow_reversed_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_flow_reversed_get" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_B) ((arg1)->flow_reversed);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_num_column_heights_set) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_num_column_heights_set(self,num_column_heights);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_num_column_heights_set" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_num_column_heights_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_column_heights = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_num_column_heights_get) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_num_column_heights_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_num_column_heights_get" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_column_heights);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_heights_set) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_heights_set(self,column_heights);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_heights_set" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_heights_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->column_heights = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_heights_get) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_heights_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_heights_get" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_BD *) ((arg1)->column_heights);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_MTEXTOBJECTCONTEXTDATA) {
  {
    int argvi = 0;
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_MTEXTOBJECTCONTEXTDATA();");
    }
    result = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)calloc(1, sizeof(struct _dwg_object_MTEXTOBJECTCONTEXTDATA));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_MTEXTOBJECTCONTEXTDATA) {
  {
    struct _dwg_object_MTEXTOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_MTEXTOBJECTCONTEXTDATA(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_MTEXTOBJECTCONTEXTDATA" "', argument " "1"" of type '" "struct _dwg_object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTOBJECTCONTEXTDATA *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_OCD_Dimension_b293_set) {
  {
    struct _dwg_OCD_Dimension *arg1 = (struct _dwg_OCD_Dimension *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_OCD_Dimension_b293_set(self,b293);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_OCD_Dimension, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_OCD_Dimension_b293_set" "', argument " "1"" of type '" "struct _dwg_OCD_Dimension *""'"); 
    }
    arg1 = (struct _dwg_OCD_Dimension *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_OCD_Dimension_b293_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->b293 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_OCD_Dimension_b293_get) {
  {
    struct _dwg_OCD_Dimension *arg1 = (struct _dwg_OCD_Dimension *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_OCD_Dimension_b293_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_OCD_Dimension, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_OCD_Dimension_b293_get" "', argument " "1"" of type '" "struct _dwg_OCD_Dimension *""'"); 
    }
    arg1 = (struct _dwg_OCD_Dimension *)(argp1);
    result = (BITCODE_B) ((arg1)->b293);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_OCD_Dimension_def_pt_set) {
  {
    struct _dwg_OCD_Dimension *arg1 = (struct _dwg_OCD_Dimension *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_OCD_Dimension_def_pt_set(self,def_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_OCD_Dimension, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_OCD_Dimension_def_pt_set" "', argument " "1"" of type '" "struct _dwg_OCD_Dimension *""'"); 
    }
    arg1 = (struct _dwg_OCD_Dimension *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_OCD_Dimension_def_pt_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->def_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_OCD_Dimension_def_pt_get) {
  {
    struct _dwg_OCD_Dimension *arg1 = (struct _dwg_OCD_Dimension *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_OCD_Dimension_def_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_OCD_Dimension, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_OCD_Dimension_def_pt_get" "', argument " "1"" of type '" "struct _dwg_OCD_Dimension *""'"); 
    }
    arg1 = (struct _dwg_OCD_Dimension *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->def_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_OCD_Dimension_is_def_textloc_set) {
  {
    struct _dwg_OCD_Dimension *arg1 = (struct _dwg_OCD_Dimension *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_OCD_Dimension_is_def_textloc_set(self,is_def_textloc);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_OCD_Dimension, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_OCD_Dimension_is_def_textloc_set" "', argument " "1"" of type '" "struct _dwg_OCD_Dimension *""'"); 
    }
    arg1 = (struct _dwg_OCD_Dimension *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_OCD_Dimension_is_def_textloc_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_def_textloc = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_OCD_Dimension_is_def_textloc_get) {
  {
    struct _dwg_OCD_Dimension *arg1 = (struct _dwg_OCD_Dimension *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_OCD_Dimension_is_def_textloc_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_OCD_Dimension, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_OCD_Dimension_is_def_textloc_get" "', argument " "1"" of type '" "struct _dwg_OCD_Dimension *""'"); 
    }
    arg1 = (struct _dwg_OCD_Dimension *)(argp1);
    result = (BITCODE_B) ((arg1)->is_def_textloc);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_OCD_Dimension_text_rotation_set) {
  {
    struct _dwg_OCD_Dimension *arg1 = (struct _dwg_OCD_Dimension *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_OCD_Dimension_text_rotation_set(self,text_rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_OCD_Dimension, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_OCD_Dimension_text_rotation_set" "', argument " "1"" of type '" "struct _dwg_OCD_Dimension *""'"); 
    }
    arg1 = (struct _dwg_OCD_Dimension *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_OCD_Dimension_text_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->text_rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_OCD_Dimension_text_rotation_get) {
  {
    struct _dwg_OCD_Dimension *arg1 = (struct _dwg_OCD_Dimension *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_OCD_Dimension_text_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_OCD_Dimension, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_OCD_Dimension_text_rotation_get" "', argument " "1"" of type '" "struct _dwg_OCD_Dimension *""'"); 
    }
    arg1 = (struct _dwg_OCD_Dimension *)(argp1);
    result = (BITCODE_BD) ((arg1)->text_rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_OCD_Dimension_block_set) {
  {
    struct _dwg_OCD_Dimension *arg1 = (struct _dwg_OCD_Dimension *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_OCD_Dimension_block_set(self,block);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_OCD_Dimension, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_OCD_Dimension_block_set" "', argument " "1"" of type '" "struct _dwg_OCD_Dimension *""'"); 
    }
    arg1 = (struct _dwg_OCD_Dimension *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_OCD_Dimension_block_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->block = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_OCD_Dimension_block_get) {
  {
    struct _dwg_OCD_Dimension *arg1 = (struct _dwg_OCD_Dimension *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_OCD_Dimension_block_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_OCD_Dimension, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_OCD_Dimension_block_get" "', argument " "1"" of type '" "struct _dwg_OCD_Dimension *""'"); 
    }
    arg1 = (struct _dwg_OCD_Dimension *)(argp1);
    result = (BITCODE_H) ((arg1)->block);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_OCD_Dimension_dimtofl_set) {
  {
    struct _dwg_OCD_Dimension *arg1 = (struct _dwg_OCD_Dimension *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_OCD_Dimension_dimtofl_set(self,dimtofl);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_OCD_Dimension, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_OCD_Dimension_dimtofl_set" "', argument " "1"" of type '" "struct _dwg_OCD_Dimension *""'"); 
    }
    arg1 = (struct _dwg_OCD_Dimension *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_OCD_Dimension_dimtofl_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->dimtofl = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_OCD_Dimension_dimtofl_get) {
  {
    struct _dwg_OCD_Dimension *arg1 = (struct _dwg_OCD_Dimension *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_OCD_Dimension_dimtofl_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_OCD_Dimension, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_OCD_Dimension_dimtofl_get" "', argument " "1"" of type '" "struct _dwg_OCD_Dimension *""'"); 
    }
    arg1 = (struct _dwg_OCD_Dimension *)(argp1);
    result = (BITCODE_B) ((arg1)->dimtofl);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_OCD_Dimension_dimosxd_set) {
  {
    struct _dwg_OCD_Dimension *arg1 = (struct _dwg_OCD_Dimension *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_OCD_Dimension_dimosxd_set(self,dimosxd);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_OCD_Dimension, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_OCD_Dimension_dimosxd_set" "', argument " "1"" of type '" "struct _dwg_OCD_Dimension *""'"); 
    }
    arg1 = (struct _dwg_OCD_Dimension *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_OCD_Dimension_dimosxd_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->dimosxd = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_OCD_Dimension_dimosxd_get) {
  {
    struct _dwg_OCD_Dimension *arg1 = (struct _dwg_OCD_Dimension *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_OCD_Dimension_dimosxd_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_OCD_Dimension, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_OCD_Dimension_dimosxd_get" "', argument " "1"" of type '" "struct _dwg_OCD_Dimension *""'"); 
    }
    arg1 = (struct _dwg_OCD_Dimension *)(argp1);
    result = (BITCODE_B) ((arg1)->dimosxd);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_OCD_Dimension_dimatfit_set) {
  {
    struct _dwg_OCD_Dimension *arg1 = (struct _dwg_OCD_Dimension *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_OCD_Dimension_dimatfit_set(self,dimatfit);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_OCD_Dimension, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_OCD_Dimension_dimatfit_set" "', argument " "1"" of type '" "struct _dwg_OCD_Dimension *""'"); 
    }
    arg1 = (struct _dwg_OCD_Dimension *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_OCD_Dimension_dimatfit_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->dimatfit = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_OCD_Dimension_dimatfit_get) {
  {
    struct _dwg_OCD_Dimension *arg1 = (struct _dwg_OCD_Dimension *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_OCD_Dimension_dimatfit_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_OCD_Dimension, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_OCD_Dimension_dimatfit_get" "', argument " "1"" of type '" "struct _dwg_OCD_Dimension *""'"); 
    }
    arg1 = (struct _dwg_OCD_Dimension *)(argp1);
    result = (BITCODE_B) ((arg1)->dimatfit);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_OCD_Dimension_dimtix_set) {
  {
    struct _dwg_OCD_Dimension *arg1 = (struct _dwg_OCD_Dimension *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_OCD_Dimension_dimtix_set(self,dimtix);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_OCD_Dimension, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_OCD_Dimension_dimtix_set" "', argument " "1"" of type '" "struct _dwg_OCD_Dimension *""'"); 
    }
    arg1 = (struct _dwg_OCD_Dimension *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_OCD_Dimension_dimtix_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->dimtix = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_OCD_Dimension_dimtix_get) {
  {
    struct _dwg_OCD_Dimension *arg1 = (struct _dwg_OCD_Dimension *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_OCD_Dimension_dimtix_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_OCD_Dimension, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_OCD_Dimension_dimtix_get" "', argument " "1"" of type '" "struct _dwg_OCD_Dimension *""'"); 
    }
    arg1 = (struct _dwg_OCD_Dimension *)(argp1);
    result = (BITCODE_B) ((arg1)->dimtix);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_OCD_Dimension_dimtmove_set) {
  {
    struct _dwg_OCD_Dimension *arg1 = (struct _dwg_OCD_Dimension *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_OCD_Dimension_dimtmove_set(self,dimtmove);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_OCD_Dimension, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_OCD_Dimension_dimtmove_set" "', argument " "1"" of type '" "struct _dwg_OCD_Dimension *""'"); 
    }
    arg1 = (struct _dwg_OCD_Dimension *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_OCD_Dimension_dimtmove_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->dimtmove = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_OCD_Dimension_dimtmove_get) {
  {
    struct _dwg_OCD_Dimension *arg1 = (struct _dwg_OCD_Dimension *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_OCD_Dimension_dimtmove_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_OCD_Dimension, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_OCD_Dimension_dimtmove_get" "', argument " "1"" of type '" "struct _dwg_OCD_Dimension *""'"); 
    }
    arg1 = (struct _dwg_OCD_Dimension *)(argp1);
    result = (BITCODE_B) ((arg1)->dimtmove);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_OCD_Dimension_override_code_set) {
  {
    struct _dwg_OCD_Dimension *arg1 = (struct _dwg_OCD_Dimension *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_OCD_Dimension_override_code_set(self,override_code);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_OCD_Dimension, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_OCD_Dimension_override_code_set" "', argument " "1"" of type '" "struct _dwg_OCD_Dimension *""'"); 
    }
    arg1 = (struct _dwg_OCD_Dimension *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_OCD_Dimension_override_code_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->override_code = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_OCD_Dimension_override_code_get) {
  {
    struct _dwg_OCD_Dimension *arg1 = (struct _dwg_OCD_Dimension *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_OCD_Dimension_override_code_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_OCD_Dimension, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_OCD_Dimension_override_code_get" "', argument " "1"" of type '" "struct _dwg_OCD_Dimension *""'"); 
    }
    arg1 = (struct _dwg_OCD_Dimension *)(argp1);
    result = (BITCODE_RC) ((arg1)->override_code);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_OCD_Dimension_has_arrow2_set) {
  {
    struct _dwg_OCD_Dimension *arg1 = (struct _dwg_OCD_Dimension *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_OCD_Dimension_has_arrow2_set(self,has_arrow2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_OCD_Dimension, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_OCD_Dimension_has_arrow2_set" "', argument " "1"" of type '" "struct _dwg_OCD_Dimension *""'"); 
    }
    arg1 = (struct _dwg_OCD_Dimension *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_OCD_Dimension_has_arrow2_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_arrow2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_OCD_Dimension_has_arrow2_get) {
  {
    struct _dwg_OCD_Dimension *arg1 = (struct _dwg_OCD_Dimension *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_OCD_Dimension_has_arrow2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_OCD_Dimension, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_OCD_Dimension_has_arrow2_get" "', argument " "1"" of type '" "struct _dwg_OCD_Dimension *""'"); 
    }
    arg1 = (struct _dwg_OCD_Dimension *)(argp1);
    result = (BITCODE_B) ((arg1)->has_arrow2);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_OCD_Dimension_flip_arrow2_set) {
  {
    struct _dwg_OCD_Dimension *arg1 = (struct _dwg_OCD_Dimension *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_OCD_Dimension_flip_arrow2_set(self,flip_arrow2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_OCD_Dimension, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_OCD_Dimension_flip_arrow2_set" "', argument " "1"" of type '" "struct _dwg_OCD_Dimension *""'"); 
    }
    arg1 = (struct _dwg_OCD_Dimension *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_OCD_Dimension_flip_arrow2_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->flip_arrow2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_OCD_Dimension_flip_arrow2_get) {
  {
    struct _dwg_OCD_Dimension *arg1 = (struct _dwg_OCD_Dimension *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_OCD_Dimension_flip_arrow2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_OCD_Dimension, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_OCD_Dimension_flip_arrow2_get" "', argument " "1"" of type '" "struct _dwg_OCD_Dimension *""'"); 
    }
    arg1 = (struct _dwg_OCD_Dimension *)(argp1);
    result = (BITCODE_B) ((arg1)->flip_arrow2);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_OCD_Dimension_flip_arrow1_set) {
  {
    struct _dwg_OCD_Dimension *arg1 = (struct _dwg_OCD_Dimension *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_OCD_Dimension_flip_arrow1_set(self,flip_arrow1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_OCD_Dimension, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_OCD_Dimension_flip_arrow1_set" "', argument " "1"" of type '" "struct _dwg_OCD_Dimension *""'"); 
    }
    arg1 = (struct _dwg_OCD_Dimension *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_OCD_Dimension_flip_arrow1_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->flip_arrow1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_OCD_Dimension_flip_arrow1_get) {
  {
    struct _dwg_OCD_Dimension *arg1 = (struct _dwg_OCD_Dimension *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_OCD_Dimension_flip_arrow1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_OCD_Dimension, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_OCD_Dimension_flip_arrow1_get" "', argument " "1"" of type '" "struct _dwg_OCD_Dimension *""'"); 
    }
    arg1 = (struct _dwg_OCD_Dimension *)(argp1);
    result = (BITCODE_B) ((arg1)->flip_arrow1);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_OCD_Dimension) {
  {
    int argvi = 0;
    struct _dwg_OCD_Dimension *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_OCD_Dimension();");
    }
    result = (struct _dwg_OCD_Dimension *)calloc(1, sizeof(struct _dwg_OCD_Dimension));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_OCD_Dimension, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_OCD_Dimension) {
  {
    struct _dwg_OCD_Dimension *arg1 = (struct _dwg_OCD_Dimension *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_OCD_Dimension(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_OCD_Dimension, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_OCD_Dimension" "', argument " "1"" of type '" "struct _dwg_OCD_Dimension *""'"); 
    }
    arg1 = (struct _dwg_OCD_Dimension *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ALDIMOBJECTCONTEXTDATA_parent_set) {
  {
    struct _dwg_object_ALDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ALDIMOBJECTCONTEXTDATA *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ALDIMOBJECTCONTEXTDATA_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ALDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ALDIMOBJECTCONTEXTDATA_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ALDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ALDIMOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ALDIMOBJECTCONTEXTDATA_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ALDIMOBJECTCONTEXTDATA_parent_get) {
  {
    struct _dwg_object_ALDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ALDIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ALDIMOBJECTCONTEXTDATA_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ALDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ALDIMOBJECTCONTEXTDATA_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ALDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ALDIMOBJECTCONTEXTDATA *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ALDIMOBJECTCONTEXTDATA_class_version_set) {
  {
    struct _dwg_object_ALDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ALDIMOBJECTCONTEXTDATA *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ALDIMOBJECTCONTEXTDATA_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ALDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ALDIMOBJECTCONTEXTDATA_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ALDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ALDIMOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ALDIMOBJECTCONTEXTDATA_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ALDIMOBJECTCONTEXTDATA_class_version_get) {
  {
    struct _dwg_object_ALDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ALDIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ALDIMOBJECTCONTEXTDATA_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ALDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ALDIMOBJECTCONTEXTDATA_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ALDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ALDIMOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ALDIMOBJECTCONTEXTDATA_is_default_set) {
  {
    struct _dwg_object_ALDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ALDIMOBJECTCONTEXTDATA *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ALDIMOBJECTCONTEXTDATA_is_default_set(self,is_default);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ALDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ALDIMOBJECTCONTEXTDATA_is_default_set" "', argument " "1"" of type '" "struct _dwg_object_ALDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ALDIMOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ALDIMOBJECTCONTEXTDATA_is_default_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_default = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ALDIMOBJECTCONTEXTDATA_is_default_get) {
  {
    struct _dwg_object_ALDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ALDIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ALDIMOBJECTCONTEXTDATA_is_default_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ALDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ALDIMOBJECTCONTEXTDATA_is_default_get" "', argument " "1"" of type '" "struct _dwg_object_ALDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ALDIMOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_B) ((arg1)->is_default);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ALDIMOBJECTCONTEXTDATA_scale_set) {
  {
    struct _dwg_object_ALDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ALDIMOBJECTCONTEXTDATA *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ALDIMOBJECTCONTEXTDATA_scale_set(self,scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ALDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ALDIMOBJECTCONTEXTDATA_scale_set" "', argument " "1"" of type '" "struct _dwg_object_ALDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ALDIMOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ALDIMOBJECTCONTEXTDATA_scale_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->scale = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ALDIMOBJECTCONTEXTDATA_scale_get) {
  {
    struct _dwg_object_ALDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ALDIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ALDIMOBJECTCONTEXTDATA_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ALDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ALDIMOBJECTCONTEXTDATA_scale_get" "', argument " "1"" of type '" "struct _dwg_object_ALDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ALDIMOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_H) ((arg1)->scale);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ALDIMOBJECTCONTEXTDATA_dimension_set) {
  {
    struct _dwg_object_ALDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ALDIMOBJECTCONTEXTDATA *) 0 ;
    Dwg_OCD_Dimension *arg2 = (Dwg_OCD_Dimension *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ALDIMOBJECTCONTEXTDATA_dimension_set(self,dimension);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ALDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ALDIMOBJECTCONTEXTDATA_dimension_set" "', argument " "1"" of type '" "struct _dwg_object_ALDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ALDIMOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_OCD_Dimension, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ALDIMOBJECTCONTEXTDATA_dimension_set" "', argument " "2"" of type '" "Dwg_OCD_Dimension *""'"); 
    }
    arg2 = (Dwg_OCD_Dimension *)(argp2);
    if (arg1) (arg1)->dimension = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ALDIMOBJECTCONTEXTDATA_dimension_get) {
  {
    struct _dwg_object_ALDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ALDIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_OCD_Dimension *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ALDIMOBJECTCONTEXTDATA_dimension_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ALDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ALDIMOBJECTCONTEXTDATA_dimension_get" "', argument " "1"" of type '" "struct _dwg_object_ALDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ALDIMOBJECTCONTEXTDATA *)(argp1);
    result = (Dwg_OCD_Dimension *)& ((arg1)->dimension);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_OCD_Dimension, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ALDIMOBJECTCONTEXTDATA_dimline_pt_set) {
  {
    struct _dwg_object_ALDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ALDIMOBJECTCONTEXTDATA *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ALDIMOBJECTCONTEXTDATA_dimline_pt_set(self,dimline_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ALDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ALDIMOBJECTCONTEXTDATA_dimline_pt_set" "', argument " "1"" of type '" "struct _dwg_object_ALDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ALDIMOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ALDIMOBJECTCONTEXTDATA_dimline_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->dimline_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ALDIMOBJECTCONTEXTDATA_dimline_pt_get) {
  {
    struct _dwg_object_ALDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ALDIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ALDIMOBJECTCONTEXTDATA_dimline_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ALDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ALDIMOBJECTCONTEXTDATA_dimline_pt_get" "', argument " "1"" of type '" "struct _dwg_object_ALDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ALDIMOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->dimline_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ALDIMOBJECTCONTEXTDATA) {
  {
    int argvi = 0;
    struct _dwg_object_ALDIMOBJECTCONTEXTDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ALDIMOBJECTCONTEXTDATA();");
    }
    result = (struct _dwg_object_ALDIMOBJECTCONTEXTDATA *)calloc(1, sizeof(struct _dwg_object_ALDIMOBJECTCONTEXTDATA));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ALDIMOBJECTCONTEXTDATA, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ALDIMOBJECTCONTEXTDATA) {
  {
    struct _dwg_object_ALDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ALDIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ALDIMOBJECTCONTEXTDATA(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ALDIMOBJECTCONTEXTDATA, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ALDIMOBJECTCONTEXTDATA" "', argument " "1"" of type '" "struct _dwg_object_ALDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ALDIMOBJECTCONTEXTDATA *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ANGDIMOBJECTCONTEXTDATA_parent_set) {
  {
    struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ANGDIMOBJECTCONTEXTDATA_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ANGDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ANGDIMOBJECTCONTEXTDATA_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ANGDIMOBJECTCONTEXTDATA_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ANGDIMOBJECTCONTEXTDATA_parent_get) {
  {
    struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ANGDIMOBJECTCONTEXTDATA_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ANGDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ANGDIMOBJECTCONTEXTDATA_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ANGDIMOBJECTCONTEXTDATA_class_version_set) {
  {
    struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ANGDIMOBJECTCONTEXTDATA_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ANGDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ANGDIMOBJECTCONTEXTDATA_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ANGDIMOBJECTCONTEXTDATA_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ANGDIMOBJECTCONTEXTDATA_class_version_get) {
  {
    struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ANGDIMOBJECTCONTEXTDATA_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ANGDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ANGDIMOBJECTCONTEXTDATA_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ANGDIMOBJECTCONTEXTDATA_is_default_set) {
  {
    struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ANGDIMOBJECTCONTEXTDATA_is_default_set(self,is_default);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ANGDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ANGDIMOBJECTCONTEXTDATA_is_default_set" "', argument " "1"" of type '" "struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ANGDIMOBJECTCONTEXTDATA_is_default_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_default = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ANGDIMOBJECTCONTEXTDATA_is_default_get) {
  {
    struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ANGDIMOBJECTCONTEXTDATA_is_default_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ANGDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ANGDIMOBJECTCONTEXTDATA_is_default_get" "', argument " "1"" of type '" "struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_B) ((arg1)->is_default);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ANGDIMOBJECTCONTEXTDATA_scale_set) {
  {
    struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ANGDIMOBJECTCONTEXTDATA_scale_set(self,scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ANGDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ANGDIMOBJECTCONTEXTDATA_scale_set" "', argument " "1"" of type '" "struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ANGDIMOBJECTCONTEXTDATA_scale_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->scale = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ANGDIMOBJECTCONTEXTDATA_scale_get) {
  {
    struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ANGDIMOBJECTCONTEXTDATA_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ANGDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ANGDIMOBJECTCONTEXTDATA_scale_get" "', argument " "1"" of type '" "struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_H) ((arg1)->scale);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ANGDIMOBJECTCONTEXTDATA_dimension_set) {
  {
    struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *) 0 ;
    Dwg_OCD_Dimension *arg2 = (Dwg_OCD_Dimension *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ANGDIMOBJECTCONTEXTDATA_dimension_set(self,dimension);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ANGDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ANGDIMOBJECTCONTEXTDATA_dimension_set" "', argument " "1"" of type '" "struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_OCD_Dimension, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ANGDIMOBJECTCONTEXTDATA_dimension_set" "', argument " "2"" of type '" "Dwg_OCD_Dimension *""'"); 
    }
    arg2 = (Dwg_OCD_Dimension *)(argp2);
    if (arg1) (arg1)->dimension = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ANGDIMOBJECTCONTEXTDATA_dimension_get) {
  {
    struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_OCD_Dimension *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ANGDIMOBJECTCONTEXTDATA_dimension_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ANGDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ANGDIMOBJECTCONTEXTDATA_dimension_get" "', argument " "1"" of type '" "struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *)(argp1);
    result = (Dwg_OCD_Dimension *)& ((arg1)->dimension);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_OCD_Dimension, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ANGDIMOBJECTCONTEXTDATA_arc_pt_set) {
  {
    struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ANGDIMOBJECTCONTEXTDATA_arc_pt_set(self,arc_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ANGDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ANGDIMOBJECTCONTEXTDATA_arc_pt_set" "', argument " "1"" of type '" "struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ANGDIMOBJECTCONTEXTDATA_arc_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->arc_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ANGDIMOBJECTCONTEXTDATA_arc_pt_get) {
  {
    struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ANGDIMOBJECTCONTEXTDATA_arc_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ANGDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ANGDIMOBJECTCONTEXTDATA_arc_pt_get" "', argument " "1"" of type '" "struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->arc_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ANGDIMOBJECTCONTEXTDATA) {
  {
    int argvi = 0;
    struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ANGDIMOBJECTCONTEXTDATA();");
    }
    result = (struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *)calloc(1, sizeof(struct _dwg_object_ANGDIMOBJECTCONTEXTDATA));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ANGDIMOBJECTCONTEXTDATA, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ANGDIMOBJECTCONTEXTDATA) {
  {
    struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ANGDIMOBJECTCONTEXTDATA(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ANGDIMOBJECTCONTEXTDATA, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ANGDIMOBJECTCONTEXTDATA" "', argument " "1"" of type '" "struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DMDIMOBJECTCONTEXTDATA_parent_set) {
  {
    struct _dwg_object_DMDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_DMDIMOBJECTCONTEXTDATA *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DMDIMOBJECTCONTEXTDATA_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DMDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DMDIMOBJECTCONTEXTDATA_parent_set" "', argument " "1"" of type '" "struct _dwg_object_DMDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_DMDIMOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DMDIMOBJECTCONTEXTDATA_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DMDIMOBJECTCONTEXTDATA_parent_get) {
  {
    struct _dwg_object_DMDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_DMDIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DMDIMOBJECTCONTEXTDATA_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DMDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DMDIMOBJECTCONTEXTDATA_parent_get" "', argument " "1"" of type '" "struct _dwg_object_DMDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_DMDIMOBJECTCONTEXTDATA *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DMDIMOBJECTCONTEXTDATA_class_version_set) {
  {
    struct _dwg_object_DMDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_DMDIMOBJECTCONTEXTDATA *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DMDIMOBJECTCONTEXTDATA_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DMDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DMDIMOBJECTCONTEXTDATA_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_DMDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_DMDIMOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DMDIMOBJECTCONTEXTDATA_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DMDIMOBJECTCONTEXTDATA_class_version_get) {
  {
    struct _dwg_object_DMDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_DMDIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DMDIMOBJECTCONTEXTDATA_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DMDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DMDIMOBJECTCONTEXTDATA_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_DMDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_DMDIMOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DMDIMOBJECTCONTEXTDATA_is_default_set) {
  {
    struct _dwg_object_DMDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_DMDIMOBJECTCONTEXTDATA *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DMDIMOBJECTCONTEXTDATA_is_default_set(self,is_default);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DMDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DMDIMOBJECTCONTEXTDATA_is_default_set" "', argument " "1"" of type '" "struct _dwg_object_DMDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_DMDIMOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DMDIMOBJECTCONTEXTDATA_is_default_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_default = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DMDIMOBJECTCONTEXTDATA_is_default_get) {
  {
    struct _dwg_object_DMDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_DMDIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DMDIMOBJECTCONTEXTDATA_is_default_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DMDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DMDIMOBJECTCONTEXTDATA_is_default_get" "', argument " "1"" of type '" "struct _dwg_object_DMDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_DMDIMOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_B) ((arg1)->is_default);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DMDIMOBJECTCONTEXTDATA_scale_set) {
  {
    struct _dwg_object_DMDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_DMDIMOBJECTCONTEXTDATA *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DMDIMOBJECTCONTEXTDATA_scale_set(self,scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DMDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DMDIMOBJECTCONTEXTDATA_scale_set" "', argument " "1"" of type '" "struct _dwg_object_DMDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_DMDIMOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DMDIMOBJECTCONTEXTDATA_scale_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->scale = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DMDIMOBJECTCONTEXTDATA_scale_get) {
  {
    struct _dwg_object_DMDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_DMDIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DMDIMOBJECTCONTEXTDATA_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DMDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DMDIMOBJECTCONTEXTDATA_scale_get" "', argument " "1"" of type '" "struct _dwg_object_DMDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_DMDIMOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_H) ((arg1)->scale);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DMDIMOBJECTCONTEXTDATA_dimension_set) {
  {
    struct _dwg_object_DMDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_DMDIMOBJECTCONTEXTDATA *) 0 ;
    Dwg_OCD_Dimension *arg2 = (Dwg_OCD_Dimension *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DMDIMOBJECTCONTEXTDATA_dimension_set(self,dimension);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DMDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DMDIMOBJECTCONTEXTDATA_dimension_set" "', argument " "1"" of type '" "struct _dwg_object_DMDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_DMDIMOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_OCD_Dimension, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DMDIMOBJECTCONTEXTDATA_dimension_set" "', argument " "2"" of type '" "Dwg_OCD_Dimension *""'"); 
    }
    arg2 = (Dwg_OCD_Dimension *)(argp2);
    if (arg1) (arg1)->dimension = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DMDIMOBJECTCONTEXTDATA_dimension_get) {
  {
    struct _dwg_object_DMDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_DMDIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_OCD_Dimension *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DMDIMOBJECTCONTEXTDATA_dimension_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DMDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DMDIMOBJECTCONTEXTDATA_dimension_get" "', argument " "1"" of type '" "struct _dwg_object_DMDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_DMDIMOBJECTCONTEXTDATA *)(argp1);
    result = (Dwg_OCD_Dimension *)& ((arg1)->dimension);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_OCD_Dimension, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DMDIMOBJECTCONTEXTDATA_first_arc_pt_set) {
  {
    struct _dwg_object_DMDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_DMDIMOBJECTCONTEXTDATA *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DMDIMOBJECTCONTEXTDATA_first_arc_pt_set(self,first_arc_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DMDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DMDIMOBJECTCONTEXTDATA_first_arc_pt_set" "', argument " "1"" of type '" "struct _dwg_object_DMDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_DMDIMOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DMDIMOBJECTCONTEXTDATA_first_arc_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->first_arc_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DMDIMOBJECTCONTEXTDATA_first_arc_pt_get) {
  {
    struct _dwg_object_DMDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_DMDIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DMDIMOBJECTCONTEXTDATA_first_arc_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DMDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DMDIMOBJECTCONTEXTDATA_first_arc_pt_get" "', argument " "1"" of type '" "struct _dwg_object_DMDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_DMDIMOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->first_arc_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DMDIMOBJECTCONTEXTDATA_def_pt_set) {
  {
    struct _dwg_object_DMDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_DMDIMOBJECTCONTEXTDATA *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DMDIMOBJECTCONTEXTDATA_def_pt_set(self,def_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DMDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DMDIMOBJECTCONTEXTDATA_def_pt_set" "', argument " "1"" of type '" "struct _dwg_object_DMDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_DMDIMOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DMDIMOBJECTCONTEXTDATA_def_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DMDIMOBJECTCONTEXTDATA_def_pt_get) {
  {
    struct _dwg_object_DMDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_DMDIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DMDIMOBJECTCONTEXTDATA_def_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DMDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DMDIMOBJECTCONTEXTDATA_def_pt_get" "', argument " "1"" of type '" "struct _dwg_object_DMDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_DMDIMOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_DMDIMOBJECTCONTEXTDATA) {
  {
    int argvi = 0;
    struct _dwg_object_DMDIMOBJECTCONTEXTDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_DMDIMOBJECTCONTEXTDATA();");
    }
    result = (struct _dwg_object_DMDIMOBJECTCONTEXTDATA *)calloc(1, sizeof(struct _dwg_object_DMDIMOBJECTCONTEXTDATA));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DMDIMOBJECTCONTEXTDATA, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_DMDIMOBJECTCONTEXTDATA) {
  {
    struct _dwg_object_DMDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_DMDIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_DMDIMOBJECTCONTEXTDATA(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DMDIMOBJECTCONTEXTDATA, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_DMDIMOBJECTCONTEXTDATA" "', argument " "1"" of type '" "struct _dwg_object_DMDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_DMDIMOBJECTCONTEXTDATA *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ORDDIMOBJECTCONTEXTDATA_parent_set) {
  {
    struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ORDDIMOBJECTCONTEXTDATA_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ORDDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ORDDIMOBJECTCONTEXTDATA_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ORDDIMOBJECTCONTEXTDATA_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ORDDIMOBJECTCONTEXTDATA_parent_get) {
  {
    struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ORDDIMOBJECTCONTEXTDATA_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ORDDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ORDDIMOBJECTCONTEXTDATA_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ORDDIMOBJECTCONTEXTDATA_class_version_set) {
  {
    struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ORDDIMOBJECTCONTEXTDATA_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ORDDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ORDDIMOBJECTCONTEXTDATA_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ORDDIMOBJECTCONTEXTDATA_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ORDDIMOBJECTCONTEXTDATA_class_version_get) {
  {
    struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ORDDIMOBJECTCONTEXTDATA_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ORDDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ORDDIMOBJECTCONTEXTDATA_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ORDDIMOBJECTCONTEXTDATA_is_default_set) {
  {
    struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ORDDIMOBJECTCONTEXTDATA_is_default_set(self,is_default);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ORDDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ORDDIMOBJECTCONTEXTDATA_is_default_set" "', argument " "1"" of type '" "struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ORDDIMOBJECTCONTEXTDATA_is_default_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_default = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ORDDIMOBJECTCONTEXTDATA_is_default_get) {
  {
    struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ORDDIMOBJECTCONTEXTDATA_is_default_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ORDDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ORDDIMOBJECTCONTEXTDATA_is_default_get" "', argument " "1"" of type '" "struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_B) ((arg1)->is_default);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ORDDIMOBJECTCONTEXTDATA_scale_set) {
  {
    struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ORDDIMOBJECTCONTEXTDATA_scale_set(self,scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ORDDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ORDDIMOBJECTCONTEXTDATA_scale_set" "', argument " "1"" of type '" "struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ORDDIMOBJECTCONTEXTDATA_scale_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->scale = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ORDDIMOBJECTCONTEXTDATA_scale_get) {
  {
    struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ORDDIMOBJECTCONTEXTDATA_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ORDDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ORDDIMOBJECTCONTEXTDATA_scale_get" "', argument " "1"" of type '" "struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_H) ((arg1)->scale);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ORDDIMOBJECTCONTEXTDATA_dimension_set) {
  {
    struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *) 0 ;
    Dwg_OCD_Dimension *arg2 = (Dwg_OCD_Dimension *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ORDDIMOBJECTCONTEXTDATA_dimension_set(self,dimension);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ORDDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ORDDIMOBJECTCONTEXTDATA_dimension_set" "', argument " "1"" of type '" "struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_OCD_Dimension, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ORDDIMOBJECTCONTEXTDATA_dimension_set" "', argument " "2"" of type '" "Dwg_OCD_Dimension *""'"); 
    }
    arg2 = (Dwg_OCD_Dimension *)(argp2);
    if (arg1) (arg1)->dimension = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ORDDIMOBJECTCONTEXTDATA_dimension_get) {
  {
    struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_OCD_Dimension *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ORDDIMOBJECTCONTEXTDATA_dimension_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ORDDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ORDDIMOBJECTCONTEXTDATA_dimension_get" "', argument " "1"" of type '" "struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *)(argp1);
    result = (Dwg_OCD_Dimension *)& ((arg1)->dimension);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_OCD_Dimension, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ORDDIMOBJECTCONTEXTDATA_feature_location_pt_set) {
  {
    struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ORDDIMOBJECTCONTEXTDATA_feature_location_pt_set(self,feature_location_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ORDDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ORDDIMOBJECTCONTEXTDATA_feature_location_pt_set" "', argument " "1"" of type '" "struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ORDDIMOBJECTCONTEXTDATA_feature_location_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->feature_location_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ORDDIMOBJECTCONTEXTDATA_feature_location_pt_get) {
  {
    struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ORDDIMOBJECTCONTEXTDATA_feature_location_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ORDDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ORDDIMOBJECTCONTEXTDATA_feature_location_pt_get" "', argument " "1"" of type '" "struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->feature_location_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ORDDIMOBJECTCONTEXTDATA_leader_endpt_set) {
  {
    struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ORDDIMOBJECTCONTEXTDATA_leader_endpt_set(self,leader_endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ORDDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ORDDIMOBJECTCONTEXTDATA_leader_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ORDDIMOBJECTCONTEXTDATA_leader_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->leader_endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ORDDIMOBJECTCONTEXTDATA_leader_endpt_get) {
  {
    struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ORDDIMOBJECTCONTEXTDATA_leader_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ORDDIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ORDDIMOBJECTCONTEXTDATA_leader_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->leader_endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ORDDIMOBJECTCONTEXTDATA) {
  {
    int argvi = 0;
    struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ORDDIMOBJECTCONTEXTDATA();");
    }
    result = (struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *)calloc(1, sizeof(struct _dwg_object_ORDDIMOBJECTCONTEXTDATA));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ORDDIMOBJECTCONTEXTDATA, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ORDDIMOBJECTCONTEXTDATA) {
  {
    struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ORDDIMOBJECTCONTEXTDATA(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ORDDIMOBJECTCONTEXTDATA, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ORDDIMOBJECTCONTEXTDATA" "', argument " "1"" of type '" "struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RADIMOBJECTCONTEXTDATA_parent_set) {
  {
    struct _dwg_object_RADIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_RADIMOBJECTCONTEXTDATA *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RADIMOBJECTCONTEXTDATA_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RADIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RADIMOBJECTCONTEXTDATA_parent_set" "', argument " "1"" of type '" "struct _dwg_object_RADIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_RADIMOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_RADIMOBJECTCONTEXTDATA_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RADIMOBJECTCONTEXTDATA_parent_get) {
  {
    struct _dwg_object_RADIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_RADIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RADIMOBJECTCONTEXTDATA_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RADIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RADIMOBJECTCONTEXTDATA_parent_get" "', argument " "1"" of type '" "struct _dwg_object_RADIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_RADIMOBJECTCONTEXTDATA *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RADIMOBJECTCONTEXTDATA_class_version_set) {
  {
    struct _dwg_object_RADIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_RADIMOBJECTCONTEXTDATA *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RADIMOBJECTCONTEXTDATA_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RADIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RADIMOBJECTCONTEXTDATA_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_RADIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_RADIMOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RADIMOBJECTCONTEXTDATA_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RADIMOBJECTCONTEXTDATA_class_version_get) {
  {
    struct _dwg_object_RADIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_RADIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RADIMOBJECTCONTEXTDATA_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RADIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RADIMOBJECTCONTEXTDATA_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_RADIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_RADIMOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RADIMOBJECTCONTEXTDATA_is_default_set) {
  {
    struct _dwg_object_RADIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_RADIMOBJECTCONTEXTDATA *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RADIMOBJECTCONTEXTDATA_is_default_set(self,is_default);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RADIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RADIMOBJECTCONTEXTDATA_is_default_set" "', argument " "1"" of type '" "struct _dwg_object_RADIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_RADIMOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RADIMOBJECTCONTEXTDATA_is_default_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_default = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RADIMOBJECTCONTEXTDATA_is_default_get) {
  {
    struct _dwg_object_RADIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_RADIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RADIMOBJECTCONTEXTDATA_is_default_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RADIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RADIMOBJECTCONTEXTDATA_is_default_get" "', argument " "1"" of type '" "struct _dwg_object_RADIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_RADIMOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_B) ((arg1)->is_default);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RADIMOBJECTCONTEXTDATA_scale_set) {
  {
    struct _dwg_object_RADIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_RADIMOBJECTCONTEXTDATA *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RADIMOBJECTCONTEXTDATA_scale_set(self,scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RADIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RADIMOBJECTCONTEXTDATA_scale_set" "', argument " "1"" of type '" "struct _dwg_object_RADIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_RADIMOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_RADIMOBJECTCONTEXTDATA_scale_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->scale = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RADIMOBJECTCONTEXTDATA_scale_get) {
  {
    struct _dwg_object_RADIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_RADIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RADIMOBJECTCONTEXTDATA_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RADIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RADIMOBJECTCONTEXTDATA_scale_get" "', argument " "1"" of type '" "struct _dwg_object_RADIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_RADIMOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_H) ((arg1)->scale);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RADIMOBJECTCONTEXTDATA_dimension_set) {
  {
    struct _dwg_object_RADIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_RADIMOBJECTCONTEXTDATA *) 0 ;
    Dwg_OCD_Dimension *arg2 = (Dwg_OCD_Dimension *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RADIMOBJECTCONTEXTDATA_dimension_set(self,dimension);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RADIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RADIMOBJECTCONTEXTDATA_dimension_set" "', argument " "1"" of type '" "struct _dwg_object_RADIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_RADIMOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_OCD_Dimension, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_RADIMOBJECTCONTEXTDATA_dimension_set" "', argument " "2"" of type '" "Dwg_OCD_Dimension *""'"); 
    }
    arg2 = (Dwg_OCD_Dimension *)(argp2);
    if (arg1) (arg1)->dimension = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RADIMOBJECTCONTEXTDATA_dimension_get) {
  {
    struct _dwg_object_RADIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_RADIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_OCD_Dimension *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RADIMOBJECTCONTEXTDATA_dimension_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RADIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RADIMOBJECTCONTEXTDATA_dimension_get" "', argument " "1"" of type '" "struct _dwg_object_RADIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_RADIMOBJECTCONTEXTDATA *)(argp1);
    result = (Dwg_OCD_Dimension *)& ((arg1)->dimension);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_OCD_Dimension, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RADIMOBJECTCONTEXTDATA_first_arc_pt_set) {
  {
    struct _dwg_object_RADIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_RADIMOBJECTCONTEXTDATA *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RADIMOBJECTCONTEXTDATA_first_arc_pt_set(self,first_arc_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RADIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RADIMOBJECTCONTEXTDATA_first_arc_pt_set" "', argument " "1"" of type '" "struct _dwg_object_RADIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_RADIMOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_RADIMOBJECTCONTEXTDATA_first_arc_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->first_arc_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RADIMOBJECTCONTEXTDATA_first_arc_pt_get) {
  {
    struct _dwg_object_RADIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_RADIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RADIMOBJECTCONTEXTDATA_first_arc_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RADIMOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RADIMOBJECTCONTEXTDATA_first_arc_pt_get" "', argument " "1"" of type '" "struct _dwg_object_RADIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_RADIMOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->first_arc_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_RADIMOBJECTCONTEXTDATA) {
  {
    int argvi = 0;
    struct _dwg_object_RADIMOBJECTCONTEXTDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_RADIMOBJECTCONTEXTDATA();");
    }
    result = (struct _dwg_object_RADIMOBJECTCONTEXTDATA *)calloc(1, sizeof(struct _dwg_object_RADIMOBJECTCONTEXTDATA));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_RADIMOBJECTCONTEXTDATA, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_RADIMOBJECTCONTEXTDATA) {
  {
    struct _dwg_object_RADIMOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_RADIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_RADIMOBJECTCONTEXTDATA(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RADIMOBJECTCONTEXTDATA, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_RADIMOBJECTCONTEXTDATA" "', argument " "1"" of type '" "struct _dwg_object_RADIMOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_RADIMOBJECTCONTEXTDATA *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RADIMLGOBJECTCONTEXTDATA_parent_set) {
  {
    struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RADIMLGOBJECTCONTEXTDATA_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RADIMLGOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RADIMLGOBJECTCONTEXTDATA_parent_set" "', argument " "1"" of type '" "struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_RADIMLGOBJECTCONTEXTDATA_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RADIMLGOBJECTCONTEXTDATA_parent_get) {
  {
    struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RADIMLGOBJECTCONTEXTDATA_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RADIMLGOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RADIMLGOBJECTCONTEXTDATA_parent_get" "', argument " "1"" of type '" "struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RADIMLGOBJECTCONTEXTDATA_class_version_set) {
  {
    struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RADIMLGOBJECTCONTEXTDATA_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RADIMLGOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RADIMLGOBJECTCONTEXTDATA_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RADIMLGOBJECTCONTEXTDATA_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RADIMLGOBJECTCONTEXTDATA_class_version_get) {
  {
    struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RADIMLGOBJECTCONTEXTDATA_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RADIMLGOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RADIMLGOBJECTCONTEXTDATA_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RADIMLGOBJECTCONTEXTDATA_is_default_set) {
  {
    struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RADIMLGOBJECTCONTEXTDATA_is_default_set(self,is_default);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RADIMLGOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RADIMLGOBJECTCONTEXTDATA_is_default_set" "', argument " "1"" of type '" "struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_RADIMLGOBJECTCONTEXTDATA_is_default_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_default = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RADIMLGOBJECTCONTEXTDATA_is_default_get) {
  {
    struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RADIMLGOBJECTCONTEXTDATA_is_default_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RADIMLGOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RADIMLGOBJECTCONTEXTDATA_is_default_get" "', argument " "1"" of type '" "struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_B) ((arg1)->is_default);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RADIMLGOBJECTCONTEXTDATA_scale_set) {
  {
    struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RADIMLGOBJECTCONTEXTDATA_scale_set(self,scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RADIMLGOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RADIMLGOBJECTCONTEXTDATA_scale_set" "', argument " "1"" of type '" "struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_RADIMLGOBJECTCONTEXTDATA_scale_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->scale = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RADIMLGOBJECTCONTEXTDATA_scale_get) {
  {
    struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RADIMLGOBJECTCONTEXTDATA_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RADIMLGOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RADIMLGOBJECTCONTEXTDATA_scale_get" "', argument " "1"" of type '" "struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_H) ((arg1)->scale);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RADIMLGOBJECTCONTEXTDATA_dimension_set) {
  {
    struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *) 0 ;
    Dwg_OCD_Dimension *arg2 = (Dwg_OCD_Dimension *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RADIMLGOBJECTCONTEXTDATA_dimension_set(self,dimension);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RADIMLGOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RADIMLGOBJECTCONTEXTDATA_dimension_set" "', argument " "1"" of type '" "struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_OCD_Dimension, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_RADIMLGOBJECTCONTEXTDATA_dimension_set" "', argument " "2"" of type '" "Dwg_OCD_Dimension *""'"); 
    }
    arg2 = (Dwg_OCD_Dimension *)(argp2);
    if (arg1) (arg1)->dimension = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RADIMLGOBJECTCONTEXTDATA_dimension_get) {
  {
    struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_OCD_Dimension *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RADIMLGOBJECTCONTEXTDATA_dimension_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RADIMLGOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RADIMLGOBJECTCONTEXTDATA_dimension_get" "', argument " "1"" of type '" "struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *)(argp1);
    result = (Dwg_OCD_Dimension *)& ((arg1)->dimension);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_OCD_Dimension, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RADIMLGOBJECTCONTEXTDATA_ovr_center_set) {
  {
    struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RADIMLGOBJECTCONTEXTDATA_ovr_center_set(self,ovr_center);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RADIMLGOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RADIMLGOBJECTCONTEXTDATA_ovr_center_set" "', argument " "1"" of type '" "struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_RADIMLGOBJECTCONTEXTDATA_ovr_center_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ovr_center = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RADIMLGOBJECTCONTEXTDATA_ovr_center_get) {
  {
    struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RADIMLGOBJECTCONTEXTDATA_ovr_center_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RADIMLGOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RADIMLGOBJECTCONTEXTDATA_ovr_center_get" "', argument " "1"" of type '" "struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ovr_center);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RADIMLGOBJECTCONTEXTDATA_jog_point_set) {
  {
    struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_RADIMLGOBJECTCONTEXTDATA_jog_point_set(self,jog_point);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RADIMLGOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RADIMLGOBJECTCONTEXTDATA_jog_point_set" "', argument " "1"" of type '" "struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_RADIMLGOBJECTCONTEXTDATA_jog_point_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->jog_point = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_RADIMLGOBJECTCONTEXTDATA_jog_point_get) {
  {
    struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_RADIMLGOBJECTCONTEXTDATA_jog_point_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RADIMLGOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_RADIMLGOBJECTCONTEXTDATA_jog_point_get" "', argument " "1"" of type '" "struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->jog_point);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_RADIMLGOBJECTCONTEXTDATA) {
  {
    int argvi = 0;
    struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_RADIMLGOBJECTCONTEXTDATA();");
    }
    result = (struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *)calloc(1, sizeof(struct _dwg_object_RADIMLGOBJECTCONTEXTDATA));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_RADIMLGOBJECTCONTEXTDATA, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_RADIMLGOBJECTCONTEXTDATA) {
  {
    struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_RADIMLGOBJECTCONTEXTDATA(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_RADIMLGOBJECTCONTEXTDATA, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_RADIMLGOBJECTCONTEXTDATA" "', argument " "1"" of type '" "struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_parent_set) {
  {
    struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_parent_set" "', argument " "1"" of type '" "struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_parent_get) {
  {
    struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_parent_get" "', argument " "1"" of type '" "struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_class_version_set) {
  {
    struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_class_version_get) {
  {
    struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_is_default_set) {
  {
    struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_is_default_set(self,is_default);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_is_default_set" "', argument " "1"" of type '" "struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_is_default_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_default = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_is_default_get) {
  {
    struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_is_default_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_is_default_get" "', argument " "1"" of type '" "struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_B) ((arg1)->is_default);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_scale_set) {
  {
    struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_scale_set(self,scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_scale_set" "', argument " "1"" of type '" "struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_scale_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->scale = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_scale_get) {
  {
    struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_scale_get" "', argument " "1"" of type '" "struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_H) ((arg1)->scale);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_horizontal_mode_set) {
  {
    struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_horizontal_mode_set(self,horizontal_mode);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_horizontal_mode_set" "', argument " "1"" of type '" "struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_horizontal_mode_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->horizontal_mode = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_horizontal_mode_get) {
  {
    struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_horizontal_mode_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_horizontal_mode_get" "', argument " "1"" of type '" "struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_BS) ((arg1)->horizontal_mode);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_rotation_set) {
  {
    struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_rotation_set(self,rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_rotation_set" "', argument " "1"" of type '" "struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_rotation_get) {
  {
    struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_rotation_get" "', argument " "1"" of type '" "struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_BD) ((arg1)->rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_ins_pt_set) {
  {
    struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_ins_pt_set(self,ins_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_ins_pt_set" "', argument " "1"" of type '" "struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_ins_pt_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->ins_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_ins_pt_get) {
  {
    struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_ins_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_ins_pt_get" "', argument " "1"" of type '" "struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->ins_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_alignment_pt_set) {
  {
    struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_alignment_pt_set(self,alignment_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_alignment_pt_set" "', argument " "1"" of type '" "struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_alignment_pt_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->alignment_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_alignment_pt_get) {
  {
    struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_alignment_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_alignment_pt_get" "', argument " "1"" of type '" "struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_2RD *)& ((arg1)->alignment_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_enable_context_set) {
  {
    struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_enable_context_set(self,enable_context);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_enable_context_set" "', argument " "1"" of type '" "struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_enable_context_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->enable_context = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_enable_context_get) {
  {
    struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_enable_context_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_enable_context_get" "', argument " "1"" of type '" "struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_B) ((arg1)->enable_context);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_context_set) {
  {
    struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *) 0 ;
    Dwg_Object_SCALE *arg2 = (Dwg_Object_SCALE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_context_set(self,context);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_context_set" "', argument " "1"" of type '" "struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_SCALE, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_context_set" "', argument " "2"" of type '" "Dwg_Object_SCALE *""'"); 
    }
    arg2 = (Dwg_Object_SCALE *)(argp2);
    if (arg1) (arg1)->context = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_context_get) {
  {
    struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_SCALE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_context_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_context_get" "', argument " "1"" of type '" "struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *)(argp1);
    result = (Dwg_Object_SCALE *)& ((arg1)->context);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_SCALE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA) {
  {
    int argvi = 0;
    struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA();");
    }
    result = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *)calloc(1, sizeof(struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA) {
  {
    struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA" "', argument " "1"" of type '" "struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADEROBJECTCONTEXTDATA_parent_set) {
  {
    struct _dwg_object_MLEADEROBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MLEADEROBJECTCONTEXTDATA *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADEROBJECTCONTEXTDATA_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADEROBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADEROBJECTCONTEXTDATA_parent_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADEROBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADEROBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MLEADEROBJECTCONTEXTDATA_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADEROBJECTCONTEXTDATA_parent_get) {
  {
    struct _dwg_object_MLEADEROBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MLEADEROBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADEROBJECTCONTEXTDATA_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADEROBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADEROBJECTCONTEXTDATA_parent_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADEROBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADEROBJECTCONTEXTDATA *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADEROBJECTCONTEXTDATA_class_version_set) {
  {
    struct _dwg_object_MLEADEROBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MLEADEROBJECTCONTEXTDATA *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADEROBJECTCONTEXTDATA_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADEROBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADEROBJECTCONTEXTDATA_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADEROBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADEROBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADEROBJECTCONTEXTDATA_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADEROBJECTCONTEXTDATA_class_version_get) {
  {
    struct _dwg_object_MLEADEROBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MLEADEROBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADEROBJECTCONTEXTDATA_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADEROBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADEROBJECTCONTEXTDATA_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADEROBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADEROBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADEROBJECTCONTEXTDATA_is_default_set) {
  {
    struct _dwg_object_MLEADEROBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MLEADEROBJECTCONTEXTDATA *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADEROBJECTCONTEXTDATA_is_default_set(self,is_default);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADEROBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADEROBJECTCONTEXTDATA_is_default_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADEROBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADEROBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_MLEADEROBJECTCONTEXTDATA_is_default_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_default = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADEROBJECTCONTEXTDATA_is_default_get) {
  {
    struct _dwg_object_MLEADEROBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MLEADEROBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADEROBJECTCONTEXTDATA_is_default_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADEROBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADEROBJECTCONTEXTDATA_is_default_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADEROBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADEROBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_B) ((arg1)->is_default);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADEROBJECTCONTEXTDATA_scale_set) {
  {
    struct _dwg_object_MLEADEROBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MLEADEROBJECTCONTEXTDATA *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_MLEADEROBJECTCONTEXTDATA_scale_set(self,scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADEROBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADEROBJECTCONTEXTDATA_scale_set" "', argument " "1"" of type '" "struct _dwg_object_MLEADEROBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADEROBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_MLEADEROBJECTCONTEXTDATA_scale_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->scale = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_MLEADEROBJECTCONTEXTDATA_scale_get) {
  {
    struct _dwg_object_MLEADEROBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MLEADEROBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_MLEADEROBJECTCONTEXTDATA_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADEROBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_MLEADEROBJECTCONTEXTDATA_scale_get" "', argument " "1"" of type '" "struct _dwg_object_MLEADEROBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADEROBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_H) ((arg1)->scale);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_MLEADEROBJECTCONTEXTDATA) {
  {
    int argvi = 0;
    struct _dwg_object_MLEADEROBJECTCONTEXTDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_MLEADEROBJECTCONTEXTDATA();");
    }
    result = (struct _dwg_object_MLEADEROBJECTCONTEXTDATA *)calloc(1, sizeof(struct _dwg_object_MLEADEROBJECTCONTEXTDATA));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_MLEADEROBJECTCONTEXTDATA, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_MLEADEROBJECTCONTEXTDATA) {
  {
    struct _dwg_object_MLEADEROBJECTCONTEXTDATA *arg1 = (struct _dwg_object_MLEADEROBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_MLEADEROBJECTCONTEXTDATA(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_MLEADEROBJECTCONTEXTDATA, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_MLEADEROBJECTCONTEXTDATA" "', argument " "1"" of type '" "struct _dwg_object_MLEADEROBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_MLEADEROBJECTCONTEXTDATA *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_parent_set) {
  {
    struct _dwg_object_LEADEROBJECTCONTEXTDATA *arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LEADEROBJECTCONTEXTDATA_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LEADEROBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LEADEROBJECTCONTEXTDATA_parent_set" "', argument " "1"" of type '" "struct _dwg_object_LEADEROBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LEADEROBJECTCONTEXTDATA_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_parent_get) {
  {
    struct _dwg_object_LEADEROBJECTCONTEXTDATA *arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LEADEROBJECTCONTEXTDATA_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LEADEROBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LEADEROBJECTCONTEXTDATA_parent_get" "', argument " "1"" of type '" "struct _dwg_object_LEADEROBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_class_version_set) {
  {
    struct _dwg_object_LEADEROBJECTCONTEXTDATA *arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LEADEROBJECTCONTEXTDATA_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LEADEROBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LEADEROBJECTCONTEXTDATA_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_LEADEROBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LEADEROBJECTCONTEXTDATA_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_class_version_get) {
  {
    struct _dwg_object_LEADEROBJECTCONTEXTDATA *arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LEADEROBJECTCONTEXTDATA_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LEADEROBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LEADEROBJECTCONTEXTDATA_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_LEADEROBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_is_default_set) {
  {
    struct _dwg_object_LEADEROBJECTCONTEXTDATA *arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LEADEROBJECTCONTEXTDATA_is_default_set(self,is_default);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LEADEROBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LEADEROBJECTCONTEXTDATA_is_default_set" "', argument " "1"" of type '" "struct _dwg_object_LEADEROBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LEADEROBJECTCONTEXTDATA_is_default_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_default = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_is_default_get) {
  {
    struct _dwg_object_LEADEROBJECTCONTEXTDATA *arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LEADEROBJECTCONTEXTDATA_is_default_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LEADEROBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LEADEROBJECTCONTEXTDATA_is_default_get" "', argument " "1"" of type '" "struct _dwg_object_LEADEROBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_B) ((arg1)->is_default);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_scale_set) {
  {
    struct _dwg_object_LEADEROBJECTCONTEXTDATA *arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LEADEROBJECTCONTEXTDATA_scale_set(self,scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LEADEROBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LEADEROBJECTCONTEXTDATA_scale_set" "', argument " "1"" of type '" "struct _dwg_object_LEADEROBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LEADEROBJECTCONTEXTDATA_scale_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->scale = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_scale_get) {
  {
    struct _dwg_object_LEADEROBJECTCONTEXTDATA *arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LEADEROBJECTCONTEXTDATA_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LEADEROBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LEADEROBJECTCONTEXTDATA_scale_get" "', argument " "1"" of type '" "struct _dwg_object_LEADEROBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_H) ((arg1)->scale);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_num_points_set) {
  {
    struct _dwg_object_LEADEROBJECTCONTEXTDATA *arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LEADEROBJECTCONTEXTDATA_num_points_set(self,num_points);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LEADEROBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LEADEROBJECTCONTEXTDATA_num_points_set" "', argument " "1"" of type '" "struct _dwg_object_LEADEROBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LEADEROBJECTCONTEXTDATA_num_points_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_points = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_num_points_get) {
  {
    struct _dwg_object_LEADEROBJECTCONTEXTDATA *arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LEADEROBJECTCONTEXTDATA_num_points_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LEADEROBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LEADEROBJECTCONTEXTDATA_num_points_get" "', argument " "1"" of type '" "struct _dwg_object_LEADEROBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_points);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_points_set) {
  {
    struct _dwg_object_LEADEROBJECTCONTEXTDATA *arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *) 0 ;
    BITCODE_3DPOINT *arg2 = (BITCODE_3DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LEADEROBJECTCONTEXTDATA_points_set(self,points);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LEADEROBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LEADEROBJECTCONTEXTDATA_points_set" "', argument " "1"" of type '" "struct _dwg_object_LEADEROBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LEADEROBJECTCONTEXTDATA_points_set" "', argument " "2"" of type '" "BITCODE_3DPOINT *""'"); 
    }
    arg2 = (BITCODE_3DPOINT *)(argp2);
    if (arg1) (arg1)->points = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_points_get) {
  {
    struct _dwg_object_LEADEROBJECTCONTEXTDATA *arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LEADEROBJECTCONTEXTDATA_points_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LEADEROBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LEADEROBJECTCONTEXTDATA_points_get" "', argument " "1"" of type '" "struct _dwg_object_LEADEROBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_3DPOINT *) ((arg1)->points);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_b290_set) {
  {
    struct _dwg_object_LEADEROBJECTCONTEXTDATA *arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LEADEROBJECTCONTEXTDATA_b290_set(self,b290);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LEADEROBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LEADEROBJECTCONTEXTDATA_b290_set" "', argument " "1"" of type '" "struct _dwg_object_LEADEROBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LEADEROBJECTCONTEXTDATA_b290_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->b290 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_b290_get) {
  {
    struct _dwg_object_LEADEROBJECTCONTEXTDATA *arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LEADEROBJECTCONTEXTDATA_b290_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LEADEROBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LEADEROBJECTCONTEXTDATA_b290_get" "', argument " "1"" of type '" "struct _dwg_object_LEADEROBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_B) ((arg1)->b290);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_x_direction_set) {
  {
    struct _dwg_object_LEADEROBJECTCONTEXTDATA *arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *) 0 ;
    BITCODE_3DPOINT *arg2 = (BITCODE_3DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LEADEROBJECTCONTEXTDATA_x_direction_set(self,x_direction);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LEADEROBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LEADEROBJECTCONTEXTDATA_x_direction_set" "', argument " "1"" of type '" "struct _dwg_object_LEADEROBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LEADEROBJECTCONTEXTDATA_x_direction_set" "', argument " "2"" of type '" "BITCODE_3DPOINT *""'"); 
    }
    arg2 = (BITCODE_3DPOINT *)(argp2);
    if (arg1) (arg1)->x_direction = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_x_direction_get) {
  {
    struct _dwg_object_LEADEROBJECTCONTEXTDATA *arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LEADEROBJECTCONTEXTDATA_x_direction_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LEADEROBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LEADEROBJECTCONTEXTDATA_x_direction_get" "', argument " "1"" of type '" "struct _dwg_object_LEADEROBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_3DPOINT *)& ((arg1)->x_direction);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_inspt_offset_set) {
  {
    struct _dwg_object_LEADEROBJECTCONTEXTDATA *arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *) 0 ;
    BITCODE_3DPOINT *arg2 = (BITCODE_3DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LEADEROBJECTCONTEXTDATA_inspt_offset_set(self,inspt_offset);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LEADEROBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LEADEROBJECTCONTEXTDATA_inspt_offset_set" "', argument " "1"" of type '" "struct _dwg_object_LEADEROBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LEADEROBJECTCONTEXTDATA_inspt_offset_set" "', argument " "2"" of type '" "BITCODE_3DPOINT *""'"); 
    }
    arg2 = (BITCODE_3DPOINT *)(argp2);
    if (arg1) (arg1)->inspt_offset = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_inspt_offset_get) {
  {
    struct _dwg_object_LEADEROBJECTCONTEXTDATA *arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LEADEROBJECTCONTEXTDATA_inspt_offset_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LEADEROBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LEADEROBJECTCONTEXTDATA_inspt_offset_get" "', argument " "1"" of type '" "struct _dwg_object_LEADEROBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_3DPOINT *)& ((arg1)->inspt_offset);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_endptproj_set) {
  {
    struct _dwg_object_LEADEROBJECTCONTEXTDATA *arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *) 0 ;
    BITCODE_3DPOINT *arg2 = (BITCODE_3DPOINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LEADEROBJECTCONTEXTDATA_endptproj_set(self,endptproj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LEADEROBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LEADEROBJECTCONTEXTDATA_endptproj_set" "', argument " "1"" of type '" "struct _dwg_object_LEADEROBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LEADEROBJECTCONTEXTDATA_endptproj_set" "', argument " "2"" of type '" "BITCODE_3DPOINT *""'"); 
    }
    arg2 = (BITCODE_3DPOINT *)(argp2);
    if (arg1) (arg1)->endptproj = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_endptproj_get) {
  {
    struct _dwg_object_LEADEROBJECTCONTEXTDATA *arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3DPOINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LEADEROBJECTCONTEXTDATA_endptproj_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LEADEROBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LEADEROBJECTCONTEXTDATA_endptproj_get" "', argument " "1"" of type '" "struct _dwg_object_LEADEROBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_3DPOINT *)& ((arg1)->endptproj);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_LEADEROBJECTCONTEXTDATA) {
  {
    int argvi = 0;
    struct _dwg_object_LEADEROBJECTCONTEXTDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_LEADEROBJECTCONTEXTDATA();");
    }
    result = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *)calloc(1, sizeof(struct _dwg_object_LEADEROBJECTCONTEXTDATA));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_LEADEROBJECTCONTEXTDATA, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_LEADEROBJECTCONTEXTDATA) {
  {
    struct _dwg_object_LEADEROBJECTCONTEXTDATA *arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_LEADEROBJECTCONTEXTDATA(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LEADEROBJECTCONTEXTDATA, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_LEADEROBJECTCONTEXTDATA" "', argument " "1"" of type '" "struct _dwg_object_LEADEROBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_LEADEROBJECTCONTEXTDATA *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLKREFOBJECTCONTEXTDATA_parent_set) {
  {
    struct _dwg_object_BLKREFOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_BLKREFOBJECTCONTEXTDATA *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLKREFOBJECTCONTEXTDATA_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLKREFOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLKREFOBJECTCONTEXTDATA_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLKREFOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_BLKREFOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLKREFOBJECTCONTEXTDATA_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLKREFOBJECTCONTEXTDATA_parent_get) {
  {
    struct _dwg_object_BLKREFOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_BLKREFOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLKREFOBJECTCONTEXTDATA_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLKREFOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLKREFOBJECTCONTEXTDATA_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLKREFOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_BLKREFOBJECTCONTEXTDATA *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLKREFOBJECTCONTEXTDATA_class_version_set) {
  {
    struct _dwg_object_BLKREFOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_BLKREFOBJECTCONTEXTDATA *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLKREFOBJECTCONTEXTDATA_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLKREFOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLKREFOBJECTCONTEXTDATA_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_BLKREFOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_BLKREFOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLKREFOBJECTCONTEXTDATA_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLKREFOBJECTCONTEXTDATA_class_version_get) {
  {
    struct _dwg_object_BLKREFOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_BLKREFOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLKREFOBJECTCONTEXTDATA_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLKREFOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLKREFOBJECTCONTEXTDATA_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_BLKREFOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_BLKREFOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLKREFOBJECTCONTEXTDATA_is_default_set) {
  {
    struct _dwg_object_BLKREFOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_BLKREFOBJECTCONTEXTDATA *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLKREFOBJECTCONTEXTDATA_is_default_set(self,is_default);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLKREFOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLKREFOBJECTCONTEXTDATA_is_default_set" "', argument " "1"" of type '" "struct _dwg_object_BLKREFOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_BLKREFOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLKREFOBJECTCONTEXTDATA_is_default_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_default = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLKREFOBJECTCONTEXTDATA_is_default_get) {
  {
    struct _dwg_object_BLKREFOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_BLKREFOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLKREFOBJECTCONTEXTDATA_is_default_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLKREFOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLKREFOBJECTCONTEXTDATA_is_default_get" "', argument " "1"" of type '" "struct _dwg_object_BLKREFOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_BLKREFOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_B) ((arg1)->is_default);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLKREFOBJECTCONTEXTDATA_scale_set) {
  {
    struct _dwg_object_BLKREFOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_BLKREFOBJECTCONTEXTDATA *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLKREFOBJECTCONTEXTDATA_scale_set(self,scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLKREFOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLKREFOBJECTCONTEXTDATA_scale_set" "', argument " "1"" of type '" "struct _dwg_object_BLKREFOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_BLKREFOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLKREFOBJECTCONTEXTDATA_scale_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->scale = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLKREFOBJECTCONTEXTDATA_scale_get) {
  {
    struct _dwg_object_BLKREFOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_BLKREFOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLKREFOBJECTCONTEXTDATA_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLKREFOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLKREFOBJECTCONTEXTDATA_scale_get" "', argument " "1"" of type '" "struct _dwg_object_BLKREFOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_BLKREFOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_H) ((arg1)->scale);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLKREFOBJECTCONTEXTDATA_rotation_set) {
  {
    struct _dwg_object_BLKREFOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_BLKREFOBJECTCONTEXTDATA *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLKREFOBJECTCONTEXTDATA_rotation_set(self,rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLKREFOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLKREFOBJECTCONTEXTDATA_rotation_set" "', argument " "1"" of type '" "struct _dwg_object_BLKREFOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_BLKREFOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLKREFOBJECTCONTEXTDATA_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLKREFOBJECTCONTEXTDATA_rotation_get) {
  {
    struct _dwg_object_BLKREFOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_BLKREFOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLKREFOBJECTCONTEXTDATA_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLKREFOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLKREFOBJECTCONTEXTDATA_rotation_get" "', argument " "1"" of type '" "struct _dwg_object_BLKREFOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_BLKREFOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_BD) ((arg1)->rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLKREFOBJECTCONTEXTDATA_ins_pt_set) {
  {
    struct _dwg_object_BLKREFOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_BLKREFOBJECTCONTEXTDATA *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLKREFOBJECTCONTEXTDATA_ins_pt_set(self,ins_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLKREFOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLKREFOBJECTCONTEXTDATA_ins_pt_set" "', argument " "1"" of type '" "struct _dwg_object_BLKREFOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_BLKREFOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLKREFOBJECTCONTEXTDATA_ins_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ins_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLKREFOBJECTCONTEXTDATA_ins_pt_get) {
  {
    struct _dwg_object_BLKREFOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_BLKREFOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLKREFOBJECTCONTEXTDATA_ins_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLKREFOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLKREFOBJECTCONTEXTDATA_ins_pt_get" "', argument " "1"" of type '" "struct _dwg_object_BLKREFOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_BLKREFOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ins_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLKREFOBJECTCONTEXTDATA_scale_factor_set) {
  {
    struct _dwg_object_BLKREFOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_BLKREFOBJECTCONTEXTDATA *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLKREFOBJECTCONTEXTDATA_scale_factor_set(self,scale_factor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLKREFOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLKREFOBJECTCONTEXTDATA_scale_factor_set" "', argument " "1"" of type '" "struct _dwg_object_BLKREFOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_BLKREFOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLKREFOBJECTCONTEXTDATA_scale_factor_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->scale_factor = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLKREFOBJECTCONTEXTDATA_scale_factor_get) {
  {
    struct _dwg_object_BLKREFOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_BLKREFOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLKREFOBJECTCONTEXTDATA_scale_factor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLKREFOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLKREFOBJECTCONTEXTDATA_scale_factor_get" "', argument " "1"" of type '" "struct _dwg_object_BLKREFOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_BLKREFOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->scale_factor);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLKREFOBJECTCONTEXTDATA) {
  {
    int argvi = 0;
    struct _dwg_object_BLKREFOBJECTCONTEXTDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLKREFOBJECTCONTEXTDATA();");
    }
    result = (struct _dwg_object_BLKREFOBJECTCONTEXTDATA *)calloc(1, sizeof(struct _dwg_object_BLKREFOBJECTCONTEXTDATA));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLKREFOBJECTCONTEXTDATA, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLKREFOBJECTCONTEXTDATA) {
  {
    struct _dwg_object_BLKREFOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_BLKREFOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLKREFOBJECTCONTEXTDATA(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLKREFOBJECTCONTEXTDATA, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLKREFOBJECTCONTEXTDATA" "', argument " "1"" of type '" "struct _dwg_object_BLKREFOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_BLKREFOBJECTCONTEXTDATA *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FCFOBJECTCONTEXTDATA_parent_set) {
  {
    struct _dwg_object_FCFOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_FCFOBJECTCONTEXTDATA *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_FCFOBJECTCONTEXTDATA_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FCFOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FCFOBJECTCONTEXTDATA_parent_set" "', argument " "1"" of type '" "struct _dwg_object_FCFOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_FCFOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_FCFOBJECTCONTEXTDATA_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FCFOBJECTCONTEXTDATA_parent_get) {
  {
    struct _dwg_object_FCFOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_FCFOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_FCFOBJECTCONTEXTDATA_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FCFOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FCFOBJECTCONTEXTDATA_parent_get" "', argument " "1"" of type '" "struct _dwg_object_FCFOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_FCFOBJECTCONTEXTDATA *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FCFOBJECTCONTEXTDATA_class_version_set) {
  {
    struct _dwg_object_FCFOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_FCFOBJECTCONTEXTDATA *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_FCFOBJECTCONTEXTDATA_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FCFOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FCFOBJECTCONTEXTDATA_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_FCFOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_FCFOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_FCFOBJECTCONTEXTDATA_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FCFOBJECTCONTEXTDATA_class_version_get) {
  {
    struct _dwg_object_FCFOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_FCFOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_FCFOBJECTCONTEXTDATA_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FCFOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FCFOBJECTCONTEXTDATA_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_FCFOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_FCFOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FCFOBJECTCONTEXTDATA_is_default_set) {
  {
    struct _dwg_object_FCFOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_FCFOBJECTCONTEXTDATA *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_FCFOBJECTCONTEXTDATA_is_default_set(self,is_default);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FCFOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FCFOBJECTCONTEXTDATA_is_default_set" "', argument " "1"" of type '" "struct _dwg_object_FCFOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_FCFOBJECTCONTEXTDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_FCFOBJECTCONTEXTDATA_is_default_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_default = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FCFOBJECTCONTEXTDATA_is_default_get) {
  {
    struct _dwg_object_FCFOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_FCFOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_FCFOBJECTCONTEXTDATA_is_default_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FCFOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FCFOBJECTCONTEXTDATA_is_default_get" "', argument " "1"" of type '" "struct _dwg_object_FCFOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_FCFOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_B) ((arg1)->is_default);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FCFOBJECTCONTEXTDATA_scale_set) {
  {
    struct _dwg_object_FCFOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_FCFOBJECTCONTEXTDATA *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_FCFOBJECTCONTEXTDATA_scale_set(self,scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FCFOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FCFOBJECTCONTEXTDATA_scale_set" "', argument " "1"" of type '" "struct _dwg_object_FCFOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_FCFOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_FCFOBJECTCONTEXTDATA_scale_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->scale = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FCFOBJECTCONTEXTDATA_scale_get) {
  {
    struct _dwg_object_FCFOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_FCFOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_FCFOBJECTCONTEXTDATA_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FCFOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FCFOBJECTCONTEXTDATA_scale_get" "', argument " "1"" of type '" "struct _dwg_object_FCFOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_FCFOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_H) ((arg1)->scale);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FCFOBJECTCONTEXTDATA_location_set) {
  {
    struct _dwg_object_FCFOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_FCFOBJECTCONTEXTDATA *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_FCFOBJECTCONTEXTDATA_location_set(self,location);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FCFOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FCFOBJECTCONTEXTDATA_location_set" "', argument " "1"" of type '" "struct _dwg_object_FCFOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_FCFOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_FCFOBJECTCONTEXTDATA_location_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->location = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FCFOBJECTCONTEXTDATA_location_get) {
  {
    struct _dwg_object_FCFOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_FCFOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_FCFOBJECTCONTEXTDATA_location_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FCFOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FCFOBJECTCONTEXTDATA_location_get" "', argument " "1"" of type '" "struct _dwg_object_FCFOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_FCFOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->location);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FCFOBJECTCONTEXTDATA_horiz_dir_set) {
  {
    struct _dwg_object_FCFOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_FCFOBJECTCONTEXTDATA *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_FCFOBJECTCONTEXTDATA_horiz_dir_set(self,horiz_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FCFOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FCFOBJECTCONTEXTDATA_horiz_dir_set" "', argument " "1"" of type '" "struct _dwg_object_FCFOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_FCFOBJECTCONTEXTDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_FCFOBJECTCONTEXTDATA_horiz_dir_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->horiz_dir = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_FCFOBJECTCONTEXTDATA_horiz_dir_get) {
  {
    struct _dwg_object_FCFOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_FCFOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_FCFOBJECTCONTEXTDATA_horiz_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FCFOBJECTCONTEXTDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_FCFOBJECTCONTEXTDATA_horiz_dir_get" "', argument " "1"" of type '" "struct _dwg_object_FCFOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_FCFOBJECTCONTEXTDATA *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->horiz_dir);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_FCFOBJECTCONTEXTDATA) {
  {
    int argvi = 0;
    struct _dwg_object_FCFOBJECTCONTEXTDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_FCFOBJECTCONTEXTDATA();");
    }
    result = (struct _dwg_object_FCFOBJECTCONTEXTDATA *)calloc(1, sizeof(struct _dwg_object_FCFOBJECTCONTEXTDATA));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_FCFOBJECTCONTEXTDATA, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_FCFOBJECTCONTEXTDATA) {
  {
    struct _dwg_object_FCFOBJECTCONTEXTDATA *arg1 = (struct _dwg_object_FCFOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_FCFOBJECTCONTEXTDATA(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_FCFOBJECTCONTEXTDATA, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_FCFOBJECTCONTEXTDATA" "', argument " "1"" of type '" "struct _dwg_object_FCFOBJECTCONTEXTDATA *""'"); 
    }
    arg1 = (struct _dwg_object_FCFOBJECTCONTEXTDATA *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_parent_set) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_parent_set" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DETAILVIEWSTYLE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_parent_get) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_parent_get" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_mdoc_class_version_set) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_mdoc_class_version_set(self,mdoc_class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_mdoc_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DETAILVIEWSTYLE_mdoc_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->mdoc_class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_mdoc_class_version_get) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_mdoc_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_mdoc_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->mdoc_class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_desc_set) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_desc_set(self,desc);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_desc_set" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DETAILVIEWSTYLE_desc_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->desc);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->desc = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->desc = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_desc_get) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_desc_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_desc_get" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    result = (BITCODE_TV) ((arg1)->desc);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_is_modified_for_recompute_set) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_is_modified_for_recompute_set(self,is_modified_for_recompute);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_is_modified_for_recompute_set" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DETAILVIEWSTYLE_is_modified_for_recompute_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_modified_for_recompute = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_is_modified_for_recompute_get) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_is_modified_for_recompute_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_is_modified_for_recompute_get" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->is_modified_for_recompute);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_display_name_set) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_display_name_set(self,display_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_display_name_set" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DETAILVIEWSTYLE_display_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->display_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->display_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->display_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_display_name_get) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_display_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_display_name_get" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    result = (BITCODE_TV) ((arg1)->display_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_viewstyle_flags_set) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_viewstyle_flags_set(self,viewstyle_flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_viewstyle_flags_set" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DETAILVIEWSTYLE_viewstyle_flags_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->viewstyle_flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_viewstyle_flags_get) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_viewstyle_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_viewstyle_flags_get" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->viewstyle_flags);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_class_version_set) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DETAILVIEWSTYLE_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_class_version_get) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_flags_set) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_flags_set(self,flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_flags_set" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DETAILVIEWSTYLE_flags_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_flags_get) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_flags_get" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->flags);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_identifier_style_set) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_identifier_style_set(self,identifier_style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_identifier_style_set" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DETAILVIEWSTYLE_identifier_style_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->identifier_style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_identifier_style_get) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_identifier_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_identifier_style_get" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    result = (BITCODE_H) ((arg1)->identifier_style);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_identifier_color_set) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_identifier_color_set(self,identifier_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_identifier_color_set" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DETAILVIEWSTYLE_identifier_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->identifier_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_identifier_color_get) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_identifier_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_identifier_color_get" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->identifier_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_identifier_height_set) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_identifier_height_set(self,identifier_height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_identifier_height_set" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DETAILVIEWSTYLE_identifier_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->identifier_height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_identifier_height_get) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_identifier_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_identifier_height_get" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->identifier_height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_identifier_exclude_characters_set) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_identifier_exclude_characters_set(self,identifier_exclude_characters);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_identifier_exclude_characters_set" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DETAILVIEWSTYLE_identifier_exclude_characters_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->identifier_exclude_characters);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->identifier_exclude_characters = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->identifier_exclude_characters = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_identifier_exclude_characters_get) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_identifier_exclude_characters_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_identifier_exclude_characters_get" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    result = (BITCODE_TV) ((arg1)->identifier_exclude_characters);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_identifier_offset_set) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_identifier_offset_set(self,identifier_offset);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_identifier_offset_set" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DETAILVIEWSTYLE_identifier_offset_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->identifier_offset = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_identifier_offset_get) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_identifier_offset_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_identifier_offset_get" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->identifier_offset);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_identifier_placement_set) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_identifier_placement_set(self,identifier_placement);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_identifier_placement_set" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DETAILVIEWSTYLE_identifier_placement_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->identifier_placement = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_identifier_placement_get) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_identifier_placement_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_identifier_placement_get" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    result = (BITCODE_RC) ((arg1)->identifier_placement);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_set) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_set(self,arrow_symbol);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_set" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->arrow_symbol = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_get) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_get" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    result = (BITCODE_H) ((arg1)->arrow_symbol);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_color_set) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_color_set(self,arrow_symbol_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_color_set" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->arrow_symbol_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_color_get) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_color_get" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->arrow_symbol_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_size_set) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_size_set(self,arrow_symbol_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_size_set" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_size_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->arrow_symbol_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_size_get) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_size_get" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->arrow_symbol_size);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_boundary_ltype_set) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_boundary_ltype_set(self,boundary_ltype);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_boundary_ltype_set" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DETAILVIEWSTYLE_boundary_ltype_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->boundary_ltype = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_boundary_ltype_get) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_boundary_ltype_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_boundary_ltype_get" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    result = (BITCODE_H) ((arg1)->boundary_ltype);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_boundary_linewt_set) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    BITCODE_BLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_boundary_linewt_set(self,boundary_linewt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_boundary_linewt_set" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DETAILVIEWSTYLE_boundary_linewt_set" "', argument " "2"" of type '" "BITCODE_BLd""'");
    } 
    arg2 = (BITCODE_BLd)(val2);
    if (arg1) (arg1)->boundary_linewt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_boundary_linewt_get) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_boundary_linewt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_boundary_linewt_get" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    result = (BITCODE_BLd) ((arg1)->boundary_linewt);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_boundary_line_color_set) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_boundary_line_color_set(self,boundary_line_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_boundary_line_color_set" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DETAILVIEWSTYLE_boundary_line_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->boundary_line_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_boundary_line_color_get) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_boundary_line_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_boundary_line_color_get" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->boundary_line_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_style_set) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_style_set(self,viewlabel_text_style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_style_set" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_style_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->viewlabel_text_style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_style_get) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_style_get" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    result = (BITCODE_H) ((arg1)->viewlabel_text_style);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_color_set) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_color_set(self,viewlabel_text_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_color_set" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->viewlabel_text_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_color_get) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_color_get" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->viewlabel_text_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_height_set) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_height_set(self,viewlabel_text_height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_height_set" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->viewlabel_text_height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_height_get) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_height_get" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->viewlabel_text_height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_viewlabel_attachment_set) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_viewlabel_attachment_set(self,viewlabel_attachment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_viewlabel_attachment_set" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DETAILVIEWSTYLE_viewlabel_attachment_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->viewlabel_attachment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_viewlabel_attachment_get) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_viewlabel_attachment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_viewlabel_attachment_get" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->viewlabel_attachment);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_viewlabel_offset_set) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_viewlabel_offset_set(self,viewlabel_offset);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_viewlabel_offset_set" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DETAILVIEWSTYLE_viewlabel_offset_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->viewlabel_offset = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_viewlabel_offset_get) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_viewlabel_offset_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_viewlabel_offset_get" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->viewlabel_offset);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_viewlabel_alignment_set) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_viewlabel_alignment_set(self,viewlabel_alignment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_viewlabel_alignment_set" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DETAILVIEWSTYLE_viewlabel_alignment_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->viewlabel_alignment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_viewlabel_alignment_get) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_viewlabel_alignment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_viewlabel_alignment_get" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->viewlabel_alignment);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_viewlabel_pattern_set) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_viewlabel_pattern_set(self,viewlabel_pattern);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_viewlabel_pattern_set" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DETAILVIEWSTYLE_viewlabel_pattern_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->viewlabel_pattern);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->viewlabel_pattern = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->viewlabel_pattern = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_viewlabel_pattern_get) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_viewlabel_pattern_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_viewlabel_pattern_get" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    result = (BITCODE_TV) ((arg1)->viewlabel_pattern);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_connection_ltype_set) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_connection_ltype_set(self,connection_ltype);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_connection_ltype_set" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DETAILVIEWSTYLE_connection_ltype_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->connection_ltype = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_connection_ltype_get) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_connection_ltype_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_connection_ltype_get" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    result = (BITCODE_H) ((arg1)->connection_ltype);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_connection_linewt_set) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    BITCODE_BLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_connection_linewt_set(self,connection_linewt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_connection_linewt_set" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DETAILVIEWSTYLE_connection_linewt_set" "', argument " "2"" of type '" "BITCODE_BLd""'");
    } 
    arg2 = (BITCODE_BLd)(val2);
    if (arg1) (arg1)->connection_linewt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_connection_linewt_get) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_connection_linewt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_connection_linewt_get" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    result = (BITCODE_BLd) ((arg1)->connection_linewt);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_connection_line_color_set) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_connection_line_color_set(self,connection_line_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_connection_line_color_set" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DETAILVIEWSTYLE_connection_line_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->connection_line_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_connection_line_color_get) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_connection_line_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_connection_line_color_get" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->connection_line_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_borderline_ltype_set) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_borderline_ltype_set(self,borderline_ltype);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_borderline_ltype_set" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DETAILVIEWSTYLE_borderline_ltype_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->borderline_ltype = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_borderline_ltype_get) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_borderline_ltype_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_borderline_ltype_get" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    result = (BITCODE_H) ((arg1)->borderline_ltype);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_borderline_linewt_set) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    BITCODE_BLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_borderline_linewt_set(self,borderline_linewt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_borderline_linewt_set" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DETAILVIEWSTYLE_borderline_linewt_set" "', argument " "2"" of type '" "BITCODE_BLd""'");
    } 
    arg2 = (BITCODE_BLd)(val2);
    if (arg1) (arg1)->borderline_linewt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_borderline_linewt_get) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_borderline_linewt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_borderline_linewt_get" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    result = (BITCODE_BLd) ((arg1)->borderline_linewt);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_borderline_color_set) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_borderline_color_set(self,borderline_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_borderline_color_set" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DETAILVIEWSTYLE_borderline_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->borderline_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_borderline_color_get) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_borderline_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_borderline_color_get" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->borderline_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_model_edge_set) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_model_edge_set(self,model_edge);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_model_edge_set" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_DETAILVIEWSTYLE_model_edge_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->model_edge = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DETAILVIEWSTYLE_model_edge_get) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DETAILVIEWSTYLE_model_edge_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DETAILVIEWSTYLE_model_edge_get" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    result = (BITCODE_RC) ((arg1)->model_edge);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_DETAILVIEWSTYLE) {
  {
    int argvi = 0;
    struct _dwg_object_DETAILVIEWSTYLE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_DETAILVIEWSTYLE();");
    }
    result = (struct _dwg_object_DETAILVIEWSTYLE *)calloc(1, sizeof(struct _dwg_object_DETAILVIEWSTYLE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_DETAILVIEWSTYLE) {
  {
    struct _dwg_object_DETAILVIEWSTYLE *arg1 = (struct _dwg_object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_DETAILVIEWSTYLE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_DETAILVIEWSTYLE" "', argument " "1"" of type '" "struct _dwg_object_DETAILVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_DETAILVIEWSTYLE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_parent_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_parent_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_parent_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_parent_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_mdoc_class_version_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_mdoc_class_version_set(self,mdoc_class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_mdoc_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_mdoc_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->mdoc_class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_mdoc_class_version_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_mdoc_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_mdoc_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->mdoc_class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_desc_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_desc_set(self,desc);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_desc_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_desc_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->desc);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->desc = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->desc = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_desc_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_desc_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_desc_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_TV) ((arg1)->desc);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_is_modified_for_recompute_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_is_modified_for_recompute_set(self,is_modified_for_recompute);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_is_modified_for_recompute_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_is_modified_for_recompute_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_modified_for_recompute = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_is_modified_for_recompute_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_is_modified_for_recompute_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_is_modified_for_recompute_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->is_modified_for_recompute);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_display_name_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_display_name_set(self,display_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_display_name_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_display_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->display_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->display_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->display_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_display_name_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_display_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_display_name_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_TV) ((arg1)->display_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_viewstyle_flags_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_viewstyle_flags_set(self,viewstyle_flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_viewstyle_flags_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_viewstyle_flags_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->viewstyle_flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_viewstyle_flags_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_viewstyle_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_viewstyle_flags_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->viewstyle_flags);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_class_version_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_class_version_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_flags_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_flags_set(self,flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_flags_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_flags_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_flags_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_flags_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->flags);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_identifier_style_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_identifier_style_set(self,identifier_style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_identifier_style_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_identifier_style_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->identifier_style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_identifier_style_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_identifier_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_identifier_style_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_H) ((arg1)->identifier_style);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_identifier_color_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_identifier_color_set(self,identifier_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_identifier_color_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_identifier_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->identifier_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_identifier_color_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_identifier_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_identifier_color_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->identifier_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_identifier_height_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_identifier_height_set(self,identifier_height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_identifier_height_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_identifier_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->identifier_height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_identifier_height_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_identifier_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_identifier_height_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->identifier_height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_arrow_start_symbol_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_arrow_start_symbol_set(self,arrow_start_symbol);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_arrow_start_symbol_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_arrow_start_symbol_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->arrow_start_symbol = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_arrow_start_symbol_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_arrow_start_symbol_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_arrow_start_symbol_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_H) ((arg1)->arrow_start_symbol);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_arrow_end_symbol_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_arrow_end_symbol_set(self,arrow_end_symbol);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_arrow_end_symbol_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_arrow_end_symbol_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->arrow_end_symbol = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_arrow_end_symbol_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_arrow_end_symbol_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_arrow_end_symbol_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_H) ((arg1)->arrow_end_symbol);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_color_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_color_set(self,arrow_symbol_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_color_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->arrow_symbol_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_color_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_color_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->arrow_symbol_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_size_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_size_set(self,arrow_symbol_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_size_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_size_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->arrow_symbol_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_size_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_size_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->arrow_symbol_size);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_identifier_exclude_characters_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_identifier_exclude_characters_set(self,identifier_exclude_characters);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_identifier_exclude_characters_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_identifier_exclude_characters_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->identifier_exclude_characters);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->identifier_exclude_characters = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->identifier_exclude_characters = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_identifier_exclude_characters_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_identifier_exclude_characters_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_identifier_exclude_characters_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_TV) ((arg1)->identifier_exclude_characters);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_identifier_position_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_BLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_identifier_position_set(self,identifier_position);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_identifier_position_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_identifier_position_set" "', argument " "2"" of type '" "BITCODE_BLd""'");
    } 
    arg2 = (BITCODE_BLd)(val2);
    if (arg1) (arg1)->identifier_position = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_identifier_position_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_identifier_position_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_identifier_position_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_BLd) ((arg1)->identifier_position);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_identifier_offset_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_identifier_offset_set(self,identifier_offset);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_identifier_offset_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_identifier_offset_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->identifier_offset = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_identifier_offset_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_identifier_offset_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_identifier_offset_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->identifier_offset);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_arrow_position_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_BLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_arrow_position_set(self,arrow_position);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_arrow_position_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_arrow_position_set" "', argument " "2"" of type '" "BITCODE_BLd""'");
    } 
    arg2 = (BITCODE_BLd)(val2);
    if (arg1) (arg1)->arrow_position = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_arrow_position_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_arrow_position_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_arrow_position_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_BLd) ((arg1)->arrow_position);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_extension_length_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_extension_length_set(self,arrow_symbol_extension_length);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_extension_length_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_extension_length_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->arrow_symbol_extension_length = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_extension_length_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_extension_length_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_extension_length_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->arrow_symbol_extension_length);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_plane_ltype_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_plane_ltype_set(self,plane_ltype);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_plane_ltype_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_plane_ltype_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->plane_ltype = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_plane_ltype_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_plane_ltype_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_plane_ltype_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_H) ((arg1)->plane_ltype);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_plane_linewt_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_BLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_plane_linewt_set(self,plane_linewt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_plane_linewt_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_plane_linewt_set" "', argument " "2"" of type '" "BITCODE_BLd""'");
    } 
    arg2 = (BITCODE_BLd)(val2);
    if (arg1) (arg1)->plane_linewt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_plane_linewt_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_plane_linewt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_plane_linewt_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_BLd) ((arg1)->plane_linewt);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_plane_line_color_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_plane_line_color_set(self,plane_line_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_plane_line_color_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_plane_line_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->plane_line_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_plane_line_color_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_plane_line_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_plane_line_color_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->plane_line_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_bend_ltype_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_bend_ltype_set(self,bend_ltype);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_bend_ltype_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_bend_ltype_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->bend_ltype = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_bend_ltype_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_bend_ltype_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_bend_ltype_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_H) ((arg1)->bend_ltype);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_bend_linewt_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_BLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_bend_linewt_set(self,bend_linewt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_bend_linewt_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_bend_linewt_set" "', argument " "2"" of type '" "BITCODE_BLd""'");
    } 
    arg2 = (BITCODE_BLd)(val2);
    if (arg1) (arg1)->bend_linewt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_bend_linewt_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_bend_linewt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_bend_linewt_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_BLd) ((arg1)->bend_linewt);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_bend_line_color_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_bend_line_color_set(self,bend_line_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_bend_line_color_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_bend_line_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->bend_line_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_bend_line_color_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_bend_line_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_bend_line_color_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->bend_line_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_bend_line_length_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_bend_line_length_set(self,bend_line_length);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_bend_line_length_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_bend_line_length_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->bend_line_length = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_bend_line_length_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_bend_line_length_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_bend_line_length_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->bend_line_length);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_end_line_overshoot_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_end_line_overshoot_set(self,end_line_overshoot);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_end_line_overshoot_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_end_line_overshoot_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->end_line_overshoot = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_end_line_overshoot_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_end_line_overshoot_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_end_line_overshoot_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->end_line_overshoot);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_end_line_length_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_end_line_length_set(self,end_line_length);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_end_line_length_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_end_line_length_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->end_line_length = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_end_line_length_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_end_line_length_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_end_line_length_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->end_line_length);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_style_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_style_set(self,viewlabel_text_style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_style_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_style_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->viewlabel_text_style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_style_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_style_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_H) ((arg1)->viewlabel_text_style);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_color_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_color_set(self,viewlabel_text_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_color_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->viewlabel_text_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_color_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_color_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->viewlabel_text_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_height_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_height_set(self,viewlabel_text_height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_height_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->viewlabel_text_height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_height_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_height_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->viewlabel_text_height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_viewlabel_attachment_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_viewlabel_attachment_set(self,viewlabel_attachment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_viewlabel_attachment_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_viewlabel_attachment_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->viewlabel_attachment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_viewlabel_attachment_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_viewlabel_attachment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_viewlabel_attachment_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->viewlabel_attachment);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_viewlabel_offset_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_viewlabel_offset_set(self,viewlabel_offset);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_viewlabel_offset_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_viewlabel_offset_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->viewlabel_offset = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_viewlabel_offset_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_viewlabel_offset_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_viewlabel_offset_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->viewlabel_offset);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_viewlabel_alignment_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_viewlabel_alignment_set(self,viewlabel_alignment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_viewlabel_alignment_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_viewlabel_alignment_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->viewlabel_alignment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_viewlabel_alignment_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_viewlabel_alignment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_viewlabel_alignment_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->viewlabel_alignment);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_viewlabel_pattern_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_viewlabel_pattern_set(self,viewlabel_pattern);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_viewlabel_pattern_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_viewlabel_pattern_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->viewlabel_pattern);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->viewlabel_pattern = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->viewlabel_pattern = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_viewlabel_pattern_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_viewlabel_pattern_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_viewlabel_pattern_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_TV) ((arg1)->viewlabel_pattern);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_hatch_color_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_hatch_color_set(self,hatch_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_hatch_color_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_hatch_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->hatch_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_hatch_color_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_hatch_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_hatch_color_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->hatch_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_hatch_bg_color_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_hatch_bg_color_set(self,hatch_bg_color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_hatch_bg_color_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_hatch_bg_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->hatch_bg_color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_hatch_bg_color_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_hatch_bg_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_hatch_bg_color_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->hatch_bg_color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_hatch_pattern_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_hatch_pattern_set(self,hatch_pattern);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_hatch_pattern_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_hatch_pattern_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->hatch_pattern);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->hatch_pattern = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->hatch_pattern = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_hatch_pattern_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_hatch_pattern_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_hatch_pattern_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_TV) ((arg1)->hatch_pattern);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_hatch_scale_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_hatch_scale_set(self,hatch_scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_hatch_scale_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_hatch_scale_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->hatch_scale = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_hatch_scale_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_hatch_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_hatch_scale_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_BD) ((arg1)->hatch_scale);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_hatch_transparency_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_BLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_hatch_transparency_set(self,hatch_transparency);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_hatch_transparency_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_hatch_transparency_set" "', argument " "2"" of type '" "BITCODE_BLd""'");
    } 
    arg2 = (BITCODE_BLd)(val2);
    if (arg1) (arg1)->hatch_transparency = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_hatch_transparency_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_hatch_transparency_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_hatch_transparency_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_BLd) ((arg1)->hatch_transparency);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_unknown_b1_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_unknown_b1_set(self,unknown_b1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_unknown_b1_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_unknown_b1_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown_b1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_unknown_b1_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_unknown_b1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_unknown_b1_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown_b1);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_unknown_b2_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_unknown_b2_set(self,unknown_b2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_unknown_b2_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_unknown_b2_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown_b2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_unknown_b2_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_unknown_b2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_unknown_b2_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown_b2);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_num_hatch_angles_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_num_hatch_angles_set(self,num_hatch_angles);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_num_hatch_angles_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_num_hatch_angles_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_hatch_angles = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_num_hatch_angles_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_num_hatch_angles_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_num_hatch_angles_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_hatch_angles);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_hatch_angles_set) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_hatch_angles_set(self,hatch_angles);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_hatch_angles_set" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_hatch_angles_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->hatch_angles = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTIONVIEWSTYLE_hatch_angles_get) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTIONVIEWSTYLE_hatch_angles_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTIONVIEWSTYLE_hatch_angles_get" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    result = (BITCODE_BD *) ((arg1)->hatch_angles);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_SECTIONVIEWSTYLE) {
  {
    int argvi = 0;
    struct _dwg_object_SECTIONVIEWSTYLE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_SECTIONVIEWSTYLE();");
    }
    result = (struct _dwg_object_SECTIONVIEWSTYLE *)calloc(1, sizeof(struct _dwg_object_SECTIONVIEWSTYLE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_SECTIONVIEWSTYLE) {
  {
    struct _dwg_object_SECTIONVIEWSTYLE *arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_SECTIONVIEWSTYLE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_SECTIONVIEWSTYLE" "', argument " "1"" of type '" "struct _dwg_object_SECTIONVIEWSTYLE *""'"); 
    }
    arg1 = (struct _dwg_object_SECTIONVIEWSTYLE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTION_MANAGER_parent_set) {
  {
    struct _dwg_object_SECTION_MANAGER *arg1 = (struct _dwg_object_SECTION_MANAGER *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTION_MANAGER_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTION_MANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTION_MANAGER_parent_set" "', argument " "1"" of type '" "struct _dwg_object_SECTION_MANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_SECTION_MANAGER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SECTION_MANAGER_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTION_MANAGER_parent_get) {
  {
    struct _dwg_object_SECTION_MANAGER *arg1 = (struct _dwg_object_SECTION_MANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTION_MANAGER_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTION_MANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTION_MANAGER_parent_get" "', argument " "1"" of type '" "struct _dwg_object_SECTION_MANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_SECTION_MANAGER *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTION_MANAGER_is_live_set) {
  {
    struct _dwg_object_SECTION_MANAGER *arg1 = (struct _dwg_object_SECTION_MANAGER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTION_MANAGER_is_live_set(self,is_live);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTION_MANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTION_MANAGER_is_live_set" "', argument " "1"" of type '" "struct _dwg_object_SECTION_MANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_SECTION_MANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SECTION_MANAGER_is_live_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_live = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTION_MANAGER_is_live_get) {
  {
    struct _dwg_object_SECTION_MANAGER *arg1 = (struct _dwg_object_SECTION_MANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTION_MANAGER_is_live_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTION_MANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTION_MANAGER_is_live_get" "', argument " "1"" of type '" "struct _dwg_object_SECTION_MANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_SECTION_MANAGER *)(argp1);
    result = (BITCODE_B) ((arg1)->is_live);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTION_MANAGER_num_sections_set) {
  {
    struct _dwg_object_SECTION_MANAGER *arg1 = (struct _dwg_object_SECTION_MANAGER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTION_MANAGER_num_sections_set(self,num_sections);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTION_MANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTION_MANAGER_num_sections_set" "', argument " "1"" of type '" "struct _dwg_object_SECTION_MANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_SECTION_MANAGER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SECTION_MANAGER_num_sections_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_sections = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTION_MANAGER_num_sections_get) {
  {
    struct _dwg_object_SECTION_MANAGER *arg1 = (struct _dwg_object_SECTION_MANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTION_MANAGER_num_sections_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTION_MANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTION_MANAGER_num_sections_get" "', argument " "1"" of type '" "struct _dwg_object_SECTION_MANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_SECTION_MANAGER *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_sections);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTION_MANAGER_sections_set) {
  {
    struct _dwg_object_SECTION_MANAGER *arg1 = (struct _dwg_object_SECTION_MANAGER *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTION_MANAGER_sections_set(self,sections);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTION_MANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTION_MANAGER_sections_set" "', argument " "1"" of type '" "struct _dwg_object_SECTION_MANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_SECTION_MANAGER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SECTION_MANAGER_sections_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->sections = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTION_MANAGER_sections_get) {
  {
    struct _dwg_object_SECTION_MANAGER *arg1 = (struct _dwg_object_SECTION_MANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTION_MANAGER_sections_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTION_MANAGER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTION_MANAGER_sections_get" "', argument " "1"" of type '" "struct _dwg_object_SECTION_MANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_SECTION_MANAGER *)(argp1);
    result = (BITCODE_H *) ((arg1)->sections);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_SECTION_MANAGER) {
  {
    int argvi = 0;
    struct _dwg_object_SECTION_MANAGER *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_SECTION_MANAGER();");
    }
    result = (struct _dwg_object_SECTION_MANAGER *)calloc(1, sizeof(struct _dwg_object_SECTION_MANAGER));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_SECTION_MANAGER, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_SECTION_MANAGER) {
  {
    struct _dwg_object_SECTION_MANAGER *arg1 = (struct _dwg_object_SECTION_MANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_SECTION_MANAGER(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTION_MANAGER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_SECTION_MANAGER" "', argument " "1"" of type '" "struct _dwg_object_SECTION_MANAGER *""'"); 
    }
    arg1 = (struct _dwg_object_SECTION_MANAGER *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_geometrysettings_parent_set) {
  {
    struct _dwg_SECTION_geometrysettings *arg1 = (struct _dwg_SECTION_geometrysettings *) 0 ;
    struct _dwg_SECTION_typesettings *arg2 = (struct _dwg_SECTION_typesettings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SECTION_geometrysettings_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_geometrysettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_geometrysettings_parent_set" "', argument " "1"" of type '" "struct _dwg_SECTION_geometrysettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_geometrysettings *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_SECTION_typesettings, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_SECTION_geometrysettings_parent_set" "', argument " "2"" of type '" "struct _dwg_SECTION_typesettings *""'"); 
    }
    arg2 = (struct _dwg_SECTION_typesettings *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_geometrysettings_parent_get) {
  {
    struct _dwg_SECTION_geometrysettings *arg1 = (struct _dwg_SECTION_geometrysettings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_SECTION_typesettings *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SECTION_geometrysettings_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_geometrysettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_geometrysettings_parent_get" "', argument " "1"" of type '" "struct _dwg_SECTION_geometrysettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_geometrysettings *)(argp1);
    result = (struct _dwg_SECTION_typesettings *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_SECTION_typesettings, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_geometrysettings_num_geoms_set) {
  {
    struct _dwg_SECTION_geometrysettings *arg1 = (struct _dwg_SECTION_geometrysettings *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SECTION_geometrysettings_num_geoms_set(self,num_geoms);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_geometrysettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_geometrysettings_num_geoms_set" "', argument " "1"" of type '" "struct _dwg_SECTION_geometrysettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_geometrysettings *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SECTION_geometrysettings_num_geoms_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_geoms = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_geometrysettings_num_geoms_get) {
  {
    struct _dwg_SECTION_geometrysettings *arg1 = (struct _dwg_SECTION_geometrysettings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SECTION_geometrysettings_num_geoms_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_geometrysettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_geometrysettings_num_geoms_get" "', argument " "1"" of type '" "struct _dwg_SECTION_geometrysettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_geometrysettings *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_geoms);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_geometrysettings_hexindex_set) {
  {
    struct _dwg_SECTION_geometrysettings *arg1 = (struct _dwg_SECTION_geometrysettings *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SECTION_geometrysettings_hexindex_set(self,hexindex);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_geometrysettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_geometrysettings_hexindex_set" "', argument " "1"" of type '" "struct _dwg_SECTION_geometrysettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_geometrysettings *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SECTION_geometrysettings_hexindex_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->hexindex = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_geometrysettings_hexindex_get) {
  {
    struct _dwg_SECTION_geometrysettings *arg1 = (struct _dwg_SECTION_geometrysettings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SECTION_geometrysettings_hexindex_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_geometrysettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_geometrysettings_hexindex_get" "', argument " "1"" of type '" "struct _dwg_SECTION_geometrysettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_geometrysettings *)(argp1);
    result = (BITCODE_BL) ((arg1)->hexindex);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_geometrysettings_flags_set) {
  {
    struct _dwg_SECTION_geometrysettings *arg1 = (struct _dwg_SECTION_geometrysettings *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SECTION_geometrysettings_flags_set(self,flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_geometrysettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_geometrysettings_flags_set" "', argument " "1"" of type '" "struct _dwg_SECTION_geometrysettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_geometrysettings *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SECTION_geometrysettings_flags_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_geometrysettings_flags_get) {
  {
    struct _dwg_SECTION_geometrysettings *arg1 = (struct _dwg_SECTION_geometrysettings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SECTION_geometrysettings_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_geometrysettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_geometrysettings_flags_get" "', argument " "1"" of type '" "struct _dwg_SECTION_geometrysettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_geometrysettings *)(argp1);
    result = (BITCODE_BL) ((arg1)->flags);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_geometrysettings_color_set) {
  {
    struct _dwg_SECTION_geometrysettings *arg1 = (struct _dwg_SECTION_geometrysettings *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SECTION_geometrysettings_color_set(self,color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_geometrysettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_geometrysettings_color_set" "', argument " "1"" of type '" "struct _dwg_SECTION_geometrysettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_geometrysettings *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_SECTION_geometrysettings_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_geometrysettings_color_get) {
  {
    struct _dwg_SECTION_geometrysettings *arg1 = (struct _dwg_SECTION_geometrysettings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SECTION_geometrysettings_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_geometrysettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_geometrysettings_color_get" "', argument " "1"" of type '" "struct _dwg_SECTION_geometrysettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_geometrysettings *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_geometrysettings_layer_set) {
  {
    struct _dwg_SECTION_geometrysettings *arg1 = (struct _dwg_SECTION_geometrysettings *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SECTION_geometrysettings_layer_set(self,layer);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_geometrysettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_geometrysettings_layer_set" "', argument " "1"" of type '" "struct _dwg_SECTION_geometrysettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_geometrysettings *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_SECTION_geometrysettings_layer_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->layer);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->layer = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->layer = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_geometrysettings_layer_get) {
  {
    struct _dwg_SECTION_geometrysettings *arg1 = (struct _dwg_SECTION_geometrysettings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SECTION_geometrysettings_layer_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_geometrysettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_geometrysettings_layer_get" "', argument " "1"" of type '" "struct _dwg_SECTION_geometrysettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_geometrysettings *)(argp1);
    result = (BITCODE_TV) ((arg1)->layer);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_geometrysettings_ltype_set) {
  {
    struct _dwg_SECTION_geometrysettings *arg1 = (struct _dwg_SECTION_geometrysettings *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SECTION_geometrysettings_ltype_set(self,ltype);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_geometrysettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_geometrysettings_ltype_set" "', argument " "1"" of type '" "struct _dwg_SECTION_geometrysettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_geometrysettings *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_SECTION_geometrysettings_ltype_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->ltype);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->ltype = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->ltype = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_geometrysettings_ltype_get) {
  {
    struct _dwg_SECTION_geometrysettings *arg1 = (struct _dwg_SECTION_geometrysettings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SECTION_geometrysettings_ltype_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_geometrysettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_geometrysettings_ltype_get" "', argument " "1"" of type '" "struct _dwg_SECTION_geometrysettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_geometrysettings *)(argp1);
    result = (BITCODE_TV) ((arg1)->ltype);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_geometrysettings_ltype_scale_set) {
  {
    struct _dwg_SECTION_geometrysettings *arg1 = (struct _dwg_SECTION_geometrysettings *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SECTION_geometrysettings_ltype_scale_set(self,ltype_scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_geometrysettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_geometrysettings_ltype_scale_set" "', argument " "1"" of type '" "struct _dwg_SECTION_geometrysettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_geometrysettings *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SECTION_geometrysettings_ltype_scale_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->ltype_scale = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_geometrysettings_ltype_scale_get) {
  {
    struct _dwg_SECTION_geometrysettings *arg1 = (struct _dwg_SECTION_geometrysettings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SECTION_geometrysettings_ltype_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_geometrysettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_geometrysettings_ltype_scale_get" "', argument " "1"" of type '" "struct _dwg_SECTION_geometrysettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_geometrysettings *)(argp1);
    result = (BITCODE_BD) ((arg1)->ltype_scale);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_geometrysettings_plotstyle_set) {
  {
    struct _dwg_SECTION_geometrysettings *arg1 = (struct _dwg_SECTION_geometrysettings *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SECTION_geometrysettings_plotstyle_set(self,plotstyle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_geometrysettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_geometrysettings_plotstyle_set" "', argument " "1"" of type '" "struct _dwg_SECTION_geometrysettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_geometrysettings *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_SECTION_geometrysettings_plotstyle_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->plotstyle);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->plotstyle = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->plotstyle = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_geometrysettings_plotstyle_get) {
  {
    struct _dwg_SECTION_geometrysettings *arg1 = (struct _dwg_SECTION_geometrysettings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SECTION_geometrysettings_plotstyle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_geometrysettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_geometrysettings_plotstyle_get" "', argument " "1"" of type '" "struct _dwg_SECTION_geometrysettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_geometrysettings *)(argp1);
    result = (BITCODE_TV) ((arg1)->plotstyle);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_geometrysettings_linewt_set) {
  {
    struct _dwg_SECTION_geometrysettings *arg1 = (struct _dwg_SECTION_geometrysettings *) 0 ;
    BITCODE_BLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SECTION_geometrysettings_linewt_set(self,linewt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_geometrysettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_geometrysettings_linewt_set" "', argument " "1"" of type '" "struct _dwg_SECTION_geometrysettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_geometrysettings *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SECTION_geometrysettings_linewt_set" "', argument " "2"" of type '" "BITCODE_BLd""'");
    } 
    arg2 = (BITCODE_BLd)(val2);
    if (arg1) (arg1)->linewt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_geometrysettings_linewt_get) {
  {
    struct _dwg_SECTION_geometrysettings *arg1 = (struct _dwg_SECTION_geometrysettings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SECTION_geometrysettings_linewt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_geometrysettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_geometrysettings_linewt_get" "', argument " "1"" of type '" "struct _dwg_SECTION_geometrysettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_geometrysettings *)(argp1);
    result = (BITCODE_BLd) ((arg1)->linewt);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_geometrysettings_face_transparency_set) {
  {
    struct _dwg_SECTION_geometrysettings *arg1 = (struct _dwg_SECTION_geometrysettings *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SECTION_geometrysettings_face_transparency_set(self,face_transparency);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_geometrysettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_geometrysettings_face_transparency_set" "', argument " "1"" of type '" "struct _dwg_SECTION_geometrysettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_geometrysettings *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SECTION_geometrysettings_face_transparency_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->face_transparency = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_geometrysettings_face_transparency_get) {
  {
    struct _dwg_SECTION_geometrysettings *arg1 = (struct _dwg_SECTION_geometrysettings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SECTION_geometrysettings_face_transparency_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_geometrysettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_geometrysettings_face_transparency_get" "', argument " "1"" of type '" "struct _dwg_SECTION_geometrysettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_geometrysettings *)(argp1);
    result = (BITCODE_BS) ((arg1)->face_transparency);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_geometrysettings_edge_transparency_set) {
  {
    struct _dwg_SECTION_geometrysettings *arg1 = (struct _dwg_SECTION_geometrysettings *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SECTION_geometrysettings_edge_transparency_set(self,edge_transparency);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_geometrysettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_geometrysettings_edge_transparency_set" "', argument " "1"" of type '" "struct _dwg_SECTION_geometrysettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_geometrysettings *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SECTION_geometrysettings_edge_transparency_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->edge_transparency = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_geometrysettings_edge_transparency_get) {
  {
    struct _dwg_SECTION_geometrysettings *arg1 = (struct _dwg_SECTION_geometrysettings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SECTION_geometrysettings_edge_transparency_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_geometrysettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_geometrysettings_edge_transparency_get" "', argument " "1"" of type '" "struct _dwg_SECTION_geometrysettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_geometrysettings *)(argp1);
    result = (BITCODE_BS) ((arg1)->edge_transparency);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_geometrysettings_hatch_type_set) {
  {
    struct _dwg_SECTION_geometrysettings *arg1 = (struct _dwg_SECTION_geometrysettings *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SECTION_geometrysettings_hatch_type_set(self,hatch_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_geometrysettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_geometrysettings_hatch_type_set" "', argument " "1"" of type '" "struct _dwg_SECTION_geometrysettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_geometrysettings *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SECTION_geometrysettings_hatch_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->hatch_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_geometrysettings_hatch_type_get) {
  {
    struct _dwg_SECTION_geometrysettings *arg1 = (struct _dwg_SECTION_geometrysettings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SECTION_geometrysettings_hatch_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_geometrysettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_geometrysettings_hatch_type_get" "', argument " "1"" of type '" "struct _dwg_SECTION_geometrysettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_geometrysettings *)(argp1);
    result = (BITCODE_BS) ((arg1)->hatch_type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_geometrysettings_hatch_pattern_set) {
  {
    struct _dwg_SECTION_geometrysettings *arg1 = (struct _dwg_SECTION_geometrysettings *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SECTION_geometrysettings_hatch_pattern_set(self,hatch_pattern);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_geometrysettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_geometrysettings_hatch_pattern_set" "', argument " "1"" of type '" "struct _dwg_SECTION_geometrysettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_geometrysettings *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_SECTION_geometrysettings_hatch_pattern_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->hatch_pattern);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->hatch_pattern = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->hatch_pattern = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_geometrysettings_hatch_pattern_get) {
  {
    struct _dwg_SECTION_geometrysettings *arg1 = (struct _dwg_SECTION_geometrysettings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SECTION_geometrysettings_hatch_pattern_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_geometrysettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_geometrysettings_hatch_pattern_get" "', argument " "1"" of type '" "struct _dwg_SECTION_geometrysettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_geometrysettings *)(argp1);
    result = (BITCODE_TV) ((arg1)->hatch_pattern);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_geometrysettings_hatch_angle_set) {
  {
    struct _dwg_SECTION_geometrysettings *arg1 = (struct _dwg_SECTION_geometrysettings *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SECTION_geometrysettings_hatch_angle_set(self,hatch_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_geometrysettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_geometrysettings_hatch_angle_set" "', argument " "1"" of type '" "struct _dwg_SECTION_geometrysettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_geometrysettings *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SECTION_geometrysettings_hatch_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->hatch_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_geometrysettings_hatch_angle_get) {
  {
    struct _dwg_SECTION_geometrysettings *arg1 = (struct _dwg_SECTION_geometrysettings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SECTION_geometrysettings_hatch_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_geometrysettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_geometrysettings_hatch_angle_get" "', argument " "1"" of type '" "struct _dwg_SECTION_geometrysettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_geometrysettings *)(argp1);
    result = (BITCODE_BD) ((arg1)->hatch_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_geometrysettings_hatch_spacing_set) {
  {
    struct _dwg_SECTION_geometrysettings *arg1 = (struct _dwg_SECTION_geometrysettings *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SECTION_geometrysettings_hatch_spacing_set(self,hatch_spacing);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_geometrysettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_geometrysettings_hatch_spacing_set" "', argument " "1"" of type '" "struct _dwg_SECTION_geometrysettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_geometrysettings *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SECTION_geometrysettings_hatch_spacing_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->hatch_spacing = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_geometrysettings_hatch_spacing_get) {
  {
    struct _dwg_SECTION_geometrysettings *arg1 = (struct _dwg_SECTION_geometrysettings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SECTION_geometrysettings_hatch_spacing_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_geometrysettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_geometrysettings_hatch_spacing_get" "', argument " "1"" of type '" "struct _dwg_SECTION_geometrysettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_geometrysettings *)(argp1);
    result = (BITCODE_BD) ((arg1)->hatch_spacing);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_geometrysettings_hatch_scale_set) {
  {
    struct _dwg_SECTION_geometrysettings *arg1 = (struct _dwg_SECTION_geometrysettings *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SECTION_geometrysettings_hatch_scale_set(self,hatch_scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_geometrysettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_geometrysettings_hatch_scale_set" "', argument " "1"" of type '" "struct _dwg_SECTION_geometrysettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_geometrysettings *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SECTION_geometrysettings_hatch_scale_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->hatch_scale = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_geometrysettings_hatch_scale_get) {
  {
    struct _dwg_SECTION_geometrysettings *arg1 = (struct _dwg_SECTION_geometrysettings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SECTION_geometrysettings_hatch_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_geometrysettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_geometrysettings_hatch_scale_get" "', argument " "1"" of type '" "struct _dwg_SECTION_geometrysettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_geometrysettings *)(argp1);
    result = (BITCODE_BD) ((arg1)->hatch_scale);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_SECTION_geometrysettings) {
  {
    int argvi = 0;
    struct _dwg_SECTION_geometrysettings *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_SECTION_geometrysettings();");
    }
    result = (struct _dwg_SECTION_geometrysettings *)calloc(1, sizeof(struct _dwg_SECTION_geometrysettings));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_SECTION_geometrysettings, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_SECTION_geometrysettings) {
  {
    struct _dwg_SECTION_geometrysettings *arg1 = (struct _dwg_SECTION_geometrysettings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_SECTION_geometrysettings(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_geometrysettings, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_SECTION_geometrysettings" "', argument " "1"" of type '" "struct _dwg_SECTION_geometrysettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_geometrysettings *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_typesettings_parent_set) {
  {
    struct _dwg_SECTION_typesettings *arg1 = (struct _dwg_SECTION_typesettings *) 0 ;
    struct _dwg_object_SECTION_SETTINGS *arg2 = (struct _dwg_object_SECTION_SETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SECTION_typesettings_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_typesettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_typesettings_parent_set" "', argument " "1"" of type '" "struct _dwg_SECTION_typesettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_typesettings *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_SECTION_SETTINGS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_SECTION_typesettings_parent_set" "', argument " "2"" of type '" "struct _dwg_object_SECTION_SETTINGS *""'"); 
    }
    arg2 = (struct _dwg_object_SECTION_SETTINGS *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_typesettings_parent_get) {
  {
    struct _dwg_SECTION_typesettings *arg1 = (struct _dwg_SECTION_typesettings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_SECTION_SETTINGS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SECTION_typesettings_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_typesettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_typesettings_parent_get" "', argument " "1"" of type '" "struct _dwg_SECTION_typesettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_typesettings *)(argp1);
    result = (struct _dwg_object_SECTION_SETTINGS *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_SECTION_SETTINGS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_typesettings_type_set) {
  {
    struct _dwg_SECTION_typesettings *arg1 = (struct _dwg_SECTION_typesettings *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SECTION_typesettings_type_set(self,type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_typesettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_typesettings_type_set" "', argument " "1"" of type '" "struct _dwg_SECTION_typesettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_typesettings *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SECTION_typesettings_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_typesettings_type_get) {
  {
    struct _dwg_SECTION_typesettings *arg1 = (struct _dwg_SECTION_typesettings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SECTION_typesettings_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_typesettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_typesettings_type_get" "', argument " "1"" of type '" "struct _dwg_SECTION_typesettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_typesettings *)(argp1);
    result = (BITCODE_BS) ((arg1)->type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_typesettings_generation_set) {
  {
    struct _dwg_SECTION_typesettings *arg1 = (struct _dwg_SECTION_typesettings *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SECTION_typesettings_generation_set(self,generation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_typesettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_typesettings_generation_set" "', argument " "1"" of type '" "struct _dwg_SECTION_typesettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_typesettings *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SECTION_typesettings_generation_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->generation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_typesettings_generation_get) {
  {
    struct _dwg_SECTION_typesettings *arg1 = (struct _dwg_SECTION_typesettings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SECTION_typesettings_generation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_typesettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_typesettings_generation_get" "', argument " "1"" of type '" "struct _dwg_SECTION_typesettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_typesettings *)(argp1);
    result = (BITCODE_BS) ((arg1)->generation);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_typesettings_num_sources_set) {
  {
    struct _dwg_SECTION_typesettings *arg1 = (struct _dwg_SECTION_typesettings *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SECTION_typesettings_num_sources_set(self,num_sources);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_typesettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_typesettings_num_sources_set" "', argument " "1"" of type '" "struct _dwg_SECTION_typesettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_typesettings *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SECTION_typesettings_num_sources_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_sources = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_typesettings_num_sources_get) {
  {
    struct _dwg_SECTION_typesettings *arg1 = (struct _dwg_SECTION_typesettings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SECTION_typesettings_num_sources_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_typesettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_typesettings_num_sources_get" "', argument " "1"" of type '" "struct _dwg_SECTION_typesettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_typesettings *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_sources);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_typesettings_sources_set) {
  {
    struct _dwg_SECTION_typesettings *arg1 = (struct _dwg_SECTION_typesettings *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SECTION_typesettings_sources_set(self,sources);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_typesettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_typesettings_sources_set" "', argument " "1"" of type '" "struct _dwg_SECTION_typesettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_typesettings *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_SECTION_typesettings_sources_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->sources = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_typesettings_sources_get) {
  {
    struct _dwg_SECTION_typesettings *arg1 = (struct _dwg_SECTION_typesettings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SECTION_typesettings_sources_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_typesettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_typesettings_sources_get" "', argument " "1"" of type '" "struct _dwg_SECTION_typesettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_typesettings *)(argp1);
    result = (BITCODE_H *) ((arg1)->sources);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_typesettings_destblock_set) {
  {
    struct _dwg_SECTION_typesettings *arg1 = (struct _dwg_SECTION_typesettings *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SECTION_typesettings_destblock_set(self,destblock);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_typesettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_typesettings_destblock_set" "', argument " "1"" of type '" "struct _dwg_SECTION_typesettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_typesettings *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_SECTION_typesettings_destblock_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->destblock = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_typesettings_destblock_get) {
  {
    struct _dwg_SECTION_typesettings *arg1 = (struct _dwg_SECTION_typesettings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SECTION_typesettings_destblock_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_typesettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_typesettings_destblock_get" "', argument " "1"" of type '" "struct _dwg_SECTION_typesettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_typesettings *)(argp1);
    result = (BITCODE_H) ((arg1)->destblock);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_typesettings_destfile_set) {
  {
    struct _dwg_SECTION_typesettings *arg1 = (struct _dwg_SECTION_typesettings *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SECTION_typesettings_destfile_set(self,destfile);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_typesettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_typesettings_destfile_set" "', argument " "1"" of type '" "struct _dwg_SECTION_typesettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_typesettings *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_SECTION_typesettings_destfile_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->destfile);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->destfile = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->destfile = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_typesettings_destfile_get) {
  {
    struct _dwg_SECTION_typesettings *arg1 = (struct _dwg_SECTION_typesettings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SECTION_typesettings_destfile_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_typesettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_typesettings_destfile_get" "', argument " "1"" of type '" "struct _dwg_SECTION_typesettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_typesettings *)(argp1);
    result = (BITCODE_TV) ((arg1)->destfile);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_typesettings_num_geom_set) {
  {
    struct _dwg_SECTION_typesettings *arg1 = (struct _dwg_SECTION_typesettings *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SECTION_typesettings_num_geom_set(self,num_geom);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_typesettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_typesettings_num_geom_set" "', argument " "1"" of type '" "struct _dwg_SECTION_typesettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_typesettings *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SECTION_typesettings_num_geom_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_geom = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_typesettings_num_geom_get) {
  {
    struct _dwg_SECTION_typesettings *arg1 = (struct _dwg_SECTION_typesettings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SECTION_typesettings_num_geom_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_typesettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_typesettings_num_geom_get" "', argument " "1"" of type '" "struct _dwg_SECTION_typesettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_typesettings *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_geom);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_typesettings_geom_set) {
  {
    struct _dwg_SECTION_typesettings *arg1 = (struct _dwg_SECTION_typesettings *) 0 ;
    Dwg_SECTION_geometrysettings *arg2 = (Dwg_SECTION_geometrysettings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SECTION_typesettings_geom_set(self,geom);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_typesettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_typesettings_geom_set" "', argument " "1"" of type '" "struct _dwg_SECTION_typesettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_typesettings *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_SECTION_geometrysettings, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_SECTION_typesettings_geom_set" "', argument " "2"" of type '" "Dwg_SECTION_geometrysettings *""'"); 
    }
    arg2 = (Dwg_SECTION_geometrysettings *)(argp2);
    if (arg1) (arg1)->geom = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SECTION_typesettings_geom_get) {
  {
    struct _dwg_SECTION_typesettings *arg1 = (struct _dwg_SECTION_typesettings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_SECTION_geometrysettings *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SECTION_typesettings_geom_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_typesettings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SECTION_typesettings_geom_get" "', argument " "1"" of type '" "struct _dwg_SECTION_typesettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_typesettings *)(argp1);
    result = (Dwg_SECTION_geometrysettings *) ((arg1)->geom);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_SECTION_geometrysettings, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_SECTION_typesettings) {
  {
    int argvi = 0;
    struct _dwg_SECTION_typesettings *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_SECTION_typesettings();");
    }
    result = (struct _dwg_SECTION_typesettings *)calloc(1, sizeof(struct _dwg_SECTION_typesettings));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_SECTION_typesettings, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_SECTION_typesettings) {
  {
    struct _dwg_SECTION_typesettings *arg1 = (struct _dwg_SECTION_typesettings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_SECTION_typesettings(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SECTION_typesettings, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_SECTION_typesettings" "', argument " "1"" of type '" "struct _dwg_SECTION_typesettings *""'"); 
    }
    arg1 = (struct _dwg_SECTION_typesettings *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTION_SETTINGS_parent_set) {
  {
    struct _dwg_object_SECTION_SETTINGS *arg1 = (struct _dwg_object_SECTION_SETTINGS *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTION_SETTINGS_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTION_SETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTION_SETTINGS_parent_set" "', argument " "1"" of type '" "struct _dwg_object_SECTION_SETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_SECTION_SETTINGS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SECTION_SETTINGS_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTION_SETTINGS_parent_get) {
  {
    struct _dwg_object_SECTION_SETTINGS *arg1 = (struct _dwg_object_SECTION_SETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTION_SETTINGS_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTION_SETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTION_SETTINGS_parent_get" "', argument " "1"" of type '" "struct _dwg_object_SECTION_SETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_SECTION_SETTINGS *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTION_SETTINGS_curr_type_set) {
  {
    struct _dwg_object_SECTION_SETTINGS *arg1 = (struct _dwg_object_SECTION_SETTINGS *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTION_SETTINGS_curr_type_set(self,curr_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTION_SETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTION_SETTINGS_curr_type_set" "', argument " "1"" of type '" "struct _dwg_object_SECTION_SETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_SECTION_SETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SECTION_SETTINGS_curr_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->curr_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTION_SETTINGS_curr_type_get) {
  {
    struct _dwg_object_SECTION_SETTINGS *arg1 = (struct _dwg_object_SECTION_SETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTION_SETTINGS_curr_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTION_SETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTION_SETTINGS_curr_type_get" "', argument " "1"" of type '" "struct _dwg_object_SECTION_SETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_SECTION_SETTINGS *)(argp1);
    result = (BITCODE_BS) ((arg1)->curr_type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTION_SETTINGS_num_types_set) {
  {
    struct _dwg_object_SECTION_SETTINGS *arg1 = (struct _dwg_object_SECTION_SETTINGS *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTION_SETTINGS_num_types_set(self,num_types);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTION_SETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTION_SETTINGS_num_types_set" "', argument " "1"" of type '" "struct _dwg_object_SECTION_SETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_SECTION_SETTINGS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_SECTION_SETTINGS_num_types_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_types = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTION_SETTINGS_num_types_get) {
  {
    struct _dwg_object_SECTION_SETTINGS *arg1 = (struct _dwg_object_SECTION_SETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTION_SETTINGS_num_types_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTION_SETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTION_SETTINGS_num_types_get" "', argument " "1"" of type '" "struct _dwg_object_SECTION_SETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_SECTION_SETTINGS *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_types);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTION_SETTINGS_types_set) {
  {
    struct _dwg_object_SECTION_SETTINGS *arg1 = (struct _dwg_object_SECTION_SETTINGS *) 0 ;
    Dwg_SECTION_typesettings *arg2 = (Dwg_SECTION_typesettings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_SECTION_SETTINGS_types_set(self,types);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTION_SETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTION_SETTINGS_types_set" "', argument " "1"" of type '" "struct _dwg_object_SECTION_SETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_SECTION_SETTINGS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_SECTION_typesettings, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_SECTION_SETTINGS_types_set" "', argument " "2"" of type '" "Dwg_SECTION_typesettings *""'"); 
    }
    arg2 = (Dwg_SECTION_typesettings *)(argp2);
    if (arg1) (arg1)->types = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_SECTION_SETTINGS_types_get) {
  {
    struct _dwg_object_SECTION_SETTINGS *arg1 = (struct _dwg_object_SECTION_SETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_SECTION_typesettings *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_SECTION_SETTINGS_types_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTION_SETTINGS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_SECTION_SETTINGS_types_get" "', argument " "1"" of type '" "struct _dwg_object_SECTION_SETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_SECTION_SETTINGS *)(argp1);
    result = (Dwg_SECTION_typesettings *) ((arg1)->types);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_SECTION_typesettings, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_SECTION_SETTINGS) {
  {
    int argvi = 0;
    struct _dwg_object_SECTION_SETTINGS *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_SECTION_SETTINGS();");
    }
    result = (struct _dwg_object_SECTION_SETTINGS *)calloc(1, sizeof(struct _dwg_object_SECTION_SETTINGS));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_SECTION_SETTINGS, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_SECTION_SETTINGS) {
  {
    struct _dwg_object_SECTION_SETTINGS *arg1 = (struct _dwg_object_SECTION_SETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_SECTION_SETTINGS(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_SECTION_SETTINGS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_SECTION_SETTINGS" "', argument " "1"" of type '" "struct _dwg_object_SECTION_SETTINGS *""'"); 
    }
    arg1 = (struct _dwg_object_SECTION_SETTINGS *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYERFILTER_parent_set) {
  {
    struct _dwg_object_LAYERFILTER *arg1 = (struct _dwg_object_LAYERFILTER *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYERFILTER_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYERFILTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYERFILTER_parent_set" "', argument " "1"" of type '" "struct _dwg_object_LAYERFILTER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYERFILTER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LAYERFILTER_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYERFILTER_parent_get) {
  {
    struct _dwg_object_LAYERFILTER *arg1 = (struct _dwg_object_LAYERFILTER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYERFILTER_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYERFILTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYERFILTER_parent_get" "', argument " "1"" of type '" "struct _dwg_object_LAYERFILTER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYERFILTER *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYERFILTER_num_names_set) {
  {
    struct _dwg_object_LAYERFILTER *arg1 = (struct _dwg_object_LAYERFILTER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYERFILTER_num_names_set(self,num_names);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYERFILTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYERFILTER_num_names_set" "', argument " "1"" of type '" "struct _dwg_object_LAYERFILTER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYERFILTER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_LAYERFILTER_num_names_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_names = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYERFILTER_num_names_get) {
  {
    struct _dwg_object_LAYERFILTER *arg1 = (struct _dwg_object_LAYERFILTER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYERFILTER_num_names_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYERFILTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYERFILTER_num_names_get" "', argument " "1"" of type '" "struct _dwg_object_LAYERFILTER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYERFILTER *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_names);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYERFILTER_names_set) {
  {
    struct _dwg_object_LAYERFILTER *arg1 = (struct _dwg_object_LAYERFILTER *) 0 ;
    BITCODE_TV *arg2 = (BITCODE_TV *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_LAYERFILTER_names_set(self,names);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYERFILTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYERFILTER_names_set" "', argument " "1"" of type '" "struct _dwg_object_LAYERFILTER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYERFILTER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_LAYERFILTER_names_set" "', argument " "2"" of type '" "BITCODE_TV *""'"); 
    }
    arg2 = (BITCODE_TV *)(argp2);
    if (arg1) (arg1)->names = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_LAYERFILTER_names_get) {
  {
    struct _dwg_object_LAYERFILTER *arg1 = (struct _dwg_object_LAYERFILTER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_LAYERFILTER_names_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYERFILTER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_LAYERFILTER_names_get" "', argument " "1"" of type '" "struct _dwg_object_LAYERFILTER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYERFILTER *)(argp1);
    result = (BITCODE_TV *) ((arg1)->names);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_LAYERFILTER) {
  {
    int argvi = 0;
    struct _dwg_object_LAYERFILTER *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_LAYERFILTER();");
    }
    result = (struct _dwg_object_LAYERFILTER *)calloc(1, sizeof(struct _dwg_object_LAYERFILTER));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_LAYERFILTER, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_LAYERFILTER) {
  {
    struct _dwg_object_LAYERFILTER *arg1 = (struct _dwg_object_LAYERFILTER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_LAYERFILTER(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_LAYERFILTER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_LAYERFILTER" "', argument " "1"" of type '" "struct _dwg_object_LAYERFILTER *""'"); 
    }
    arg1 = (struct _dwg_object_LAYERFILTER *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_parent_set) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_parent_get) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_text_size_set) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    BITCODE_D2T arg2 = (BITCODE_D2T) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_text_size_set(self,text_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_text_size_set" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_text_size_set" "', argument " "2"" of type '" "BITCODE_D2T""'");
    }
    arg2 = (BITCODE_D2T)(buf2);
    free((char*)arg1->text_size);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->text_size = (BITCODE_D2T)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->text_size = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_text_size_get) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_D2T result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_text_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_text_size_get" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    result = (BITCODE_D2T) ((arg1)->text_size);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_xscale_set) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    BITCODE_D2T arg2 = (BITCODE_D2T) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_xscale_set(self,xscale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_xscale_set" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_xscale_set" "', argument " "2"" of type '" "BITCODE_D2T""'");
    }
    arg2 = (BITCODE_D2T)(buf2);
    free((char*)arg1->xscale);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->xscale = (BITCODE_D2T)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->xscale = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_xscale_get) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_D2T result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_xscale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_xscale_get" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    result = (BITCODE_D2T) ((arg1)->xscale);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_char_spacing_set) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    BITCODE_D2T arg2 = (BITCODE_D2T) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_char_spacing_set(self,char_spacing);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_char_spacing_set" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_char_spacing_set" "', argument " "2"" of type '" "BITCODE_D2T""'");
    }
    arg2 = (BITCODE_D2T)(buf2);
    free((char*)arg1->char_spacing);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->char_spacing = (BITCODE_D2T)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->char_spacing = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_char_spacing_get) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_D2T result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_char_spacing_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_char_spacing_get" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    result = (BITCODE_D2T) ((arg1)->char_spacing);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_style_set) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_style_set(self,style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_style_set" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_style_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->style);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->style = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->style = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_style_get) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_style_get" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    result = (BITCODE_TV) ((arg1)->style);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_text_value_set) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_text_value_set(self,text_value);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_text_value_set" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_text_value_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->text_value);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->text_value = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->text_value = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_text_value_get) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_text_value_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_text_value_get" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    result = (BITCODE_TV) ((arg1)->text_value);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_font_name_set) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_font_name_set(self,font_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_font_name_set" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_font_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->font_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->font_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->font_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_font_name_get) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_font_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_font_name_get" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    result = (BITCODE_TV) ((arg1)->font_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_bigfont_name_set) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_bigfont_name_set(self,bigfont_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_bigfont_name_set" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_bigfont_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->bigfont_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->bigfont_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->bigfont_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_bigfont_name_get) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_bigfont_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_bigfont_name_get" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    result = (BITCODE_TV) ((arg1)->bigfont_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_offset_from_arc_set) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    BITCODE_D2T arg2 = (BITCODE_D2T) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_offset_from_arc_set(self,offset_from_arc);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_offset_from_arc_set" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_offset_from_arc_set" "', argument " "2"" of type '" "BITCODE_D2T""'");
    }
    arg2 = (BITCODE_D2T)(buf2);
    free((char*)arg1->offset_from_arc);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->offset_from_arc = (BITCODE_D2T)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->offset_from_arc = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_offset_from_arc_get) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_D2T result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_offset_from_arc_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_offset_from_arc_get" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    result = (BITCODE_D2T) ((arg1)->offset_from_arc);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_right_offset_set) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    BITCODE_D2T arg2 = (BITCODE_D2T) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_right_offset_set(self,right_offset);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_right_offset_set" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_right_offset_set" "', argument " "2"" of type '" "BITCODE_D2T""'");
    }
    arg2 = (BITCODE_D2T)(buf2);
    free((char*)arg1->right_offset);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->right_offset = (BITCODE_D2T)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->right_offset = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_right_offset_get) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_D2T result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_right_offset_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_right_offset_get" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    result = (BITCODE_D2T) ((arg1)->right_offset);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_left_offset_set) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    BITCODE_D2T arg2 = (BITCODE_D2T) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_left_offset_set(self,left_offset);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_left_offset_set" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_left_offset_set" "', argument " "2"" of type '" "BITCODE_D2T""'");
    }
    arg2 = (BITCODE_D2T)(buf2);
    free((char*)arg1->left_offset);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->left_offset = (BITCODE_D2T)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->left_offset = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_left_offset_get) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_D2T result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_left_offset_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_left_offset_get" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    result = (BITCODE_D2T) ((arg1)->left_offset);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_center_set) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_center_set(self,center);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_center_set" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_center_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->center = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_center_get) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_center_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_center_get" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->center);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_radius_set) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_radius_set(self,radius);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_radius_set" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_radius_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->radius = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_radius_get) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_radius_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_radius_get" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    result = (BITCODE_BD) ((arg1)->radius);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_start_angle_set) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_start_angle_set(self,start_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_start_angle_set" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_start_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->start_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_start_angle_get) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_start_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_start_angle_get" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    result = (BITCODE_BD) ((arg1)->start_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_end_angle_set) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_end_angle_set(self,end_angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_end_angle_set" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_end_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->end_angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_end_angle_get) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_end_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_end_angle_get" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    result = (BITCODE_BD) ((arg1)->end_angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_extrusion_set) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_extrusion_set" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_extrusion_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_extrusion_get) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_extrusion_get" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_color_set) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_color_set(self,color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_color_set" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_color_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->color = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_color_get) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_color_get" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    result = (BITCODE_BL) ((arg1)->color);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_is_reverse_set) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_is_reverse_set(self,is_reverse);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_is_reverse_set" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_is_reverse_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->is_reverse = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_is_reverse_get) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_is_reverse_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_is_reverse_get" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    result = (BITCODE_BS) ((arg1)->is_reverse);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_text_direction_set) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_text_direction_set(self,text_direction);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_text_direction_set" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_text_direction_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->text_direction = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_text_direction_get) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_text_direction_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_text_direction_get" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    result = (BITCODE_BS) ((arg1)->text_direction);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_alignment_set) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_alignment_set(self,alignment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_alignment_set" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_alignment_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->alignment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_alignment_get) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_alignment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_alignment_get" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    result = (BITCODE_BS) ((arg1)->alignment);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_text_position_set) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_text_position_set(self,text_position);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_text_position_set" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_text_position_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->text_position = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_text_position_get) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_text_position_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_text_position_get" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    result = (BITCODE_BS) ((arg1)->text_position);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_is_bold_set) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_is_bold_set(self,is_bold);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_is_bold_set" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_is_bold_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->is_bold = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_is_bold_get) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_is_bold_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_is_bold_get" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    result = (BITCODE_BS) ((arg1)->is_bold);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_is_italic_set) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_is_italic_set(self,is_italic);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_is_italic_set" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_is_italic_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->is_italic = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_is_italic_get) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_is_italic_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_is_italic_get" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    result = (BITCODE_BS) ((arg1)->is_italic);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_is_underlined_set) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_is_underlined_set(self,is_underlined);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_is_underlined_set" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_is_underlined_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->is_underlined = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_is_underlined_get) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_is_underlined_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_is_underlined_get" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    result = (BITCODE_BS) ((arg1)->is_underlined);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_character_set_set) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_character_set_set(self,character_set);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_character_set_set" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_character_set_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->character_set = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_character_set_get) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_character_set_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_character_set_get" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    result = (BITCODE_BS) ((arg1)->character_set);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_pitch_and_family_set) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_pitch_and_family_set(self,pitch_and_family);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_pitch_and_family_set" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_pitch_and_family_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->pitch_and_family = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_pitch_and_family_get) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_pitch_and_family_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_pitch_and_family_get" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    result = (BITCODE_BS) ((arg1)->pitch_and_family);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_is_shx_set) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_is_shx_set(self,is_shx);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_is_shx_set" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_is_shx_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->is_shx = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_is_shx_get) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_is_shx_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_is_shx_get" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    result = (BITCODE_BS) ((arg1)->is_shx);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_wizard_flag_set) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_wizard_flag_set(self,wizard_flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_wizard_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_wizard_flag_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->wizard_flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_wizard_flag_get) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_wizard_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_wizard_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    result = (BITCODE_BS) ((arg1)->wizard_flag);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_arc_handle_set) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_arc_handle_set(self,arc_handle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_arc_handle_set" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_arc_handle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->arc_handle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ARCALIGNEDTEXT_arc_handle_get) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ARCALIGNEDTEXT_arc_handle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ARCALIGNEDTEXT_arc_handle_get" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    result = (BITCODE_H) ((arg1)->arc_handle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_ARCALIGNEDTEXT) {
  {
    int argvi = 0;
    struct _dwg_entity_ARCALIGNEDTEXT *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_ARCALIGNEDTEXT();");
    }
    result = (struct _dwg_entity_ARCALIGNEDTEXT *)calloc(1, sizeof(struct _dwg_entity_ARCALIGNEDTEXT));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_ARCALIGNEDTEXT) {
  {
    struct _dwg_entity_ARCALIGNEDTEXT *arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_ARCALIGNEDTEXT(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_ARCALIGNEDTEXT" "', argument " "1"" of type '" "struct _dwg_entity_ARCALIGNEDTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_ARCALIGNEDTEXT *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_RTEXT_parent_set) {
  {
    struct _dwg_entity_RTEXT *arg1 = (struct _dwg_entity_RTEXT *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_RTEXT_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_RTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_RTEXT_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_RTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_RTEXT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_RTEXT_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_RTEXT_parent_get) {
  {
    struct _dwg_entity_RTEXT *arg1 = (struct _dwg_entity_RTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_RTEXT_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_RTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_RTEXT_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_RTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_RTEXT *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_RTEXT_pt_set) {
  {
    struct _dwg_entity_RTEXT *arg1 = (struct _dwg_entity_RTEXT *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_RTEXT_pt_set(self,pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_RTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_RTEXT_pt_set" "', argument " "1"" of type '" "struct _dwg_entity_RTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_RTEXT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_RTEXT_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_RTEXT_pt_get) {
  {
    struct _dwg_entity_RTEXT *arg1 = (struct _dwg_entity_RTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_RTEXT_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_RTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_RTEXT_pt_get" "', argument " "1"" of type '" "struct _dwg_entity_RTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_RTEXT *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_RTEXT_extrusion_set) {
  {
    struct _dwg_entity_RTEXT *arg1 = (struct _dwg_entity_RTEXT *) 0 ;
    BITCODE_BE *arg2 = (BITCODE_BE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_RTEXT_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_RTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_RTEXT_extrusion_set" "', argument " "1"" of type '" "struct _dwg_entity_RTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_RTEXT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_RTEXT_extrusion_set" "', argument " "2"" of type '" "BITCODE_BE *""'"); 
    }
    arg2 = (BITCODE_BE *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_RTEXT_extrusion_get) {
  {
    struct _dwg_entity_RTEXT *arg1 = (struct _dwg_entity_RTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_RTEXT_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_RTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_RTEXT_extrusion_get" "', argument " "1"" of type '" "struct _dwg_entity_RTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_RTEXT *)(argp1);
    result = (BITCODE_BE *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_RTEXT_rotation_set) {
  {
    struct _dwg_entity_RTEXT *arg1 = (struct _dwg_entity_RTEXT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_RTEXT_rotation_set(self,rotation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_RTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_RTEXT_rotation_set" "', argument " "1"" of type '" "struct _dwg_entity_RTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_RTEXT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_RTEXT_rotation_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->rotation = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_RTEXT_rotation_get) {
  {
    struct _dwg_entity_RTEXT *arg1 = (struct _dwg_entity_RTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_RTEXT_rotation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_RTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_RTEXT_rotation_get" "', argument " "1"" of type '" "struct _dwg_entity_RTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_RTEXT *)(argp1);
    result = (BITCODE_BD) ((arg1)->rotation);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_RTEXT_height_set) {
  {
    struct _dwg_entity_RTEXT *arg1 = (struct _dwg_entity_RTEXT *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_RTEXT_height_set(self,height);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_RTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_RTEXT_height_set" "', argument " "1"" of type '" "struct _dwg_entity_RTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_RTEXT *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_RTEXT_height_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->height = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_RTEXT_height_get) {
  {
    struct _dwg_entity_RTEXT *arg1 = (struct _dwg_entity_RTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_RTEXT_height_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_RTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_RTEXT_height_get" "', argument " "1"" of type '" "struct _dwg_entity_RTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_RTEXT *)(argp1);
    result = (BITCODE_BD) ((arg1)->height);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_RTEXT_flags_set) {
  {
    struct _dwg_entity_RTEXT *arg1 = (struct _dwg_entity_RTEXT *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_RTEXT_flags_set(self,flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_RTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_RTEXT_flags_set" "', argument " "1"" of type '" "struct _dwg_entity_RTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_RTEXT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_RTEXT_flags_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_RTEXT_flags_get) {
  {
    struct _dwg_entity_RTEXT *arg1 = (struct _dwg_entity_RTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_RTEXT_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_RTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_RTEXT_flags_get" "', argument " "1"" of type '" "struct _dwg_entity_RTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_RTEXT *)(argp1);
    result = (BITCODE_BS) ((arg1)->flags);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_RTEXT_text_value_set) {
  {
    struct _dwg_entity_RTEXT *arg1 = (struct _dwg_entity_RTEXT *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_RTEXT_text_value_set(self,text_value);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_RTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_RTEXT_text_value_set" "', argument " "1"" of type '" "struct _dwg_entity_RTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_RTEXT *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_RTEXT_text_value_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->text_value);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->text_value = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->text_value = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_RTEXT_text_value_get) {
  {
    struct _dwg_entity_RTEXT *arg1 = (struct _dwg_entity_RTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_RTEXT_text_value_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_RTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_RTEXT_text_value_get" "', argument " "1"" of type '" "struct _dwg_entity_RTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_RTEXT *)(argp1);
    result = (BITCODE_TV) ((arg1)->text_value);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_RTEXT_style_set) {
  {
    struct _dwg_entity_RTEXT *arg1 = (struct _dwg_entity_RTEXT *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_RTEXT_style_set(self,style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_RTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_RTEXT_style_set" "', argument " "1"" of type '" "struct _dwg_entity_RTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_RTEXT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_RTEXT_style_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->style = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_RTEXT_style_get) {
  {
    struct _dwg_entity_RTEXT *arg1 = (struct _dwg_entity_RTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_RTEXT_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_RTEXT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_RTEXT_style_get" "', argument " "1"" of type '" "struct _dwg_entity_RTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_RTEXT *)(argp1);
    result = (BITCODE_H) ((arg1)->style);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_RTEXT) {
  {
    int argvi = 0;
    struct _dwg_entity_RTEXT *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_RTEXT();");
    }
    result = (struct _dwg_entity_RTEXT *)calloc(1, sizeof(struct _dwg_entity_RTEXT));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_RTEXT, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_RTEXT) {
  {
    struct _dwg_entity_RTEXT *arg1 = (struct _dwg_entity_RTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_RTEXT(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_RTEXT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_RTEXT" "', argument " "1"" of type '" "struct _dwg_entity_RTEXT *""'"); 
    }
    arg1 = (struct _dwg_entity_RTEXT *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LAYOUTPRINTCONFIG_parent_set) {
  {
    struct _dwg_entity_LAYOUTPRINTCONFIG *arg1 = (struct _dwg_entity_LAYOUTPRINTCONFIG *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LAYOUTPRINTCONFIG_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LAYOUTPRINTCONFIG, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LAYOUTPRINTCONFIG_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_LAYOUTPRINTCONFIG *""'"); 
    }
    arg1 = (struct _dwg_entity_LAYOUTPRINTCONFIG *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LAYOUTPRINTCONFIG_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LAYOUTPRINTCONFIG_parent_get) {
  {
    struct _dwg_entity_LAYOUTPRINTCONFIG *arg1 = (struct _dwg_entity_LAYOUTPRINTCONFIG *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LAYOUTPRINTCONFIG_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LAYOUTPRINTCONFIG, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LAYOUTPRINTCONFIG_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_LAYOUTPRINTCONFIG *""'"); 
    }
    arg1 = (struct _dwg_entity_LAYOUTPRINTCONFIG *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LAYOUTPRINTCONFIG_class_version_set) {
  {
    struct _dwg_entity_LAYOUTPRINTCONFIG *arg1 = (struct _dwg_entity_LAYOUTPRINTCONFIG *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LAYOUTPRINTCONFIG_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LAYOUTPRINTCONFIG, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LAYOUTPRINTCONFIG_class_version_set" "', argument " "1"" of type '" "struct _dwg_entity_LAYOUTPRINTCONFIG *""'"); 
    }
    arg1 = (struct _dwg_entity_LAYOUTPRINTCONFIG *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LAYOUTPRINTCONFIG_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LAYOUTPRINTCONFIG_class_version_get) {
  {
    struct _dwg_entity_LAYOUTPRINTCONFIG *arg1 = (struct _dwg_entity_LAYOUTPRINTCONFIG *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LAYOUTPRINTCONFIG_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LAYOUTPRINTCONFIG, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LAYOUTPRINTCONFIG_class_version_get" "', argument " "1"" of type '" "struct _dwg_entity_LAYOUTPRINTCONFIG *""'"); 
    }
    arg1 = (struct _dwg_entity_LAYOUTPRINTCONFIG *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LAYOUTPRINTCONFIG_flag_set) {
  {
    struct _dwg_entity_LAYOUTPRINTCONFIG *arg1 = (struct _dwg_entity_LAYOUTPRINTCONFIG *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LAYOUTPRINTCONFIG_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LAYOUTPRINTCONFIG, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LAYOUTPRINTCONFIG_flag_set" "', argument " "1"" of type '" "struct _dwg_entity_LAYOUTPRINTCONFIG *""'"); 
    }
    arg1 = (struct _dwg_entity_LAYOUTPRINTCONFIG *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_LAYOUTPRINTCONFIG_flag_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LAYOUTPRINTCONFIG_flag_get) {
  {
    struct _dwg_entity_LAYOUTPRINTCONFIG *arg1 = (struct _dwg_entity_LAYOUTPRINTCONFIG *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LAYOUTPRINTCONFIG_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LAYOUTPRINTCONFIG, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LAYOUTPRINTCONFIG_flag_get" "', argument " "1"" of type '" "struct _dwg_entity_LAYOUTPRINTCONFIG *""'"); 
    }
    arg1 = (struct _dwg_entity_LAYOUTPRINTCONFIG *)(argp1);
    result = (BITCODE_BS) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_LAYOUTPRINTCONFIG) {
  {
    int argvi = 0;
    struct _dwg_entity_LAYOUTPRINTCONFIG *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_LAYOUTPRINTCONFIG();");
    }
    result = (struct _dwg_entity_LAYOUTPRINTCONFIG *)calloc(1, sizeof(struct _dwg_entity_LAYOUTPRINTCONFIG));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_LAYOUTPRINTCONFIG, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_LAYOUTPRINTCONFIG) {
  {
    struct _dwg_entity_LAYOUTPRINTCONFIG *arg1 = (struct _dwg_entity_LAYOUTPRINTCONFIG *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_LAYOUTPRINTCONFIG(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LAYOUTPRINTCONFIG, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_LAYOUTPRINTCONFIG" "', argument " "1"" of type '" "struct _dwg_entity_LAYOUTPRINTCONFIG *""'"); 
    }
    arg1 = (struct _dwg_entity_LAYOUTPRINTCONFIG *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACMECOMMANDHISTORY_parent_set) {
  {
    struct _dwg_object_ACMECOMMANDHISTORY *arg1 = (struct _dwg_object_ACMECOMMANDHISTORY *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACMECOMMANDHISTORY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACMECOMMANDHISTORY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACMECOMMANDHISTORY_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ACMECOMMANDHISTORY *""'"); 
    }
    arg1 = (struct _dwg_object_ACMECOMMANDHISTORY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACMECOMMANDHISTORY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACMECOMMANDHISTORY_parent_get) {
  {
    struct _dwg_object_ACMECOMMANDHISTORY *arg1 = (struct _dwg_object_ACMECOMMANDHISTORY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACMECOMMANDHISTORY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACMECOMMANDHISTORY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACMECOMMANDHISTORY_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ACMECOMMANDHISTORY *""'"); 
    }
    arg1 = (struct _dwg_object_ACMECOMMANDHISTORY *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACMECOMMANDHISTORY_class_version_set) {
  {
    struct _dwg_object_ACMECOMMANDHISTORY *arg1 = (struct _dwg_object_ACMECOMMANDHISTORY *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACMECOMMANDHISTORY_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACMECOMMANDHISTORY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACMECOMMANDHISTORY_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ACMECOMMANDHISTORY *""'"); 
    }
    arg1 = (struct _dwg_object_ACMECOMMANDHISTORY *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACMECOMMANDHISTORY_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACMECOMMANDHISTORY_class_version_get) {
  {
    struct _dwg_object_ACMECOMMANDHISTORY *arg1 = (struct _dwg_object_ACMECOMMANDHISTORY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACMECOMMANDHISTORY_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACMECOMMANDHISTORY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACMECOMMANDHISTORY_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ACMECOMMANDHISTORY *""'"); 
    }
    arg1 = (struct _dwg_object_ACMECOMMANDHISTORY *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ACMECOMMANDHISTORY) {
  {
    int argvi = 0;
    struct _dwg_object_ACMECOMMANDHISTORY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ACMECOMMANDHISTORY();");
    }
    result = (struct _dwg_object_ACMECOMMANDHISTORY *)calloc(1, sizeof(struct _dwg_object_ACMECOMMANDHISTORY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACMECOMMANDHISTORY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ACMECOMMANDHISTORY) {
  {
    struct _dwg_object_ACMECOMMANDHISTORY *arg1 = (struct _dwg_object_ACMECOMMANDHISTORY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ACMECOMMANDHISTORY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACMECOMMANDHISTORY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ACMECOMMANDHISTORY" "', argument " "1"" of type '" "struct _dwg_object_ACMECOMMANDHISTORY *""'"); 
    }
    arg1 = (struct _dwg_object_ACMECOMMANDHISTORY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACMESCOPE_parent_set) {
  {
    struct _dwg_object_ACMESCOPE *arg1 = (struct _dwg_object_ACMESCOPE *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACMESCOPE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACMESCOPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACMESCOPE_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ACMESCOPE *""'"); 
    }
    arg1 = (struct _dwg_object_ACMESCOPE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACMESCOPE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACMESCOPE_parent_get) {
  {
    struct _dwg_object_ACMESCOPE *arg1 = (struct _dwg_object_ACMESCOPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACMESCOPE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACMESCOPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACMESCOPE_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ACMESCOPE *""'"); 
    }
    arg1 = (struct _dwg_object_ACMESCOPE *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACMESCOPE_class_version_set) {
  {
    struct _dwg_object_ACMESCOPE *arg1 = (struct _dwg_object_ACMESCOPE *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACMESCOPE_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACMESCOPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACMESCOPE_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ACMESCOPE *""'"); 
    }
    arg1 = (struct _dwg_object_ACMESCOPE *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACMESCOPE_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACMESCOPE_class_version_get) {
  {
    struct _dwg_object_ACMESCOPE *arg1 = (struct _dwg_object_ACMESCOPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACMESCOPE_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACMESCOPE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACMESCOPE_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ACMESCOPE *""'"); 
    }
    arg1 = (struct _dwg_object_ACMESCOPE *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ACMESCOPE) {
  {
    int argvi = 0;
    struct _dwg_object_ACMESCOPE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ACMESCOPE();");
    }
    result = (struct _dwg_object_ACMESCOPE *)calloc(1, sizeof(struct _dwg_object_ACMESCOPE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACMESCOPE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ACMESCOPE) {
  {
    struct _dwg_object_ACMESCOPE *arg1 = (struct _dwg_object_ACMESCOPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ACMESCOPE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACMESCOPE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ACMESCOPE" "', argument " "1"" of type '" "struct _dwg_object_ACMESCOPE *""'"); 
    }
    arg1 = (struct _dwg_object_ACMESCOPE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACMESTATEMGR_parent_set) {
  {
    struct _dwg_object_ACMESTATEMGR *arg1 = (struct _dwg_object_ACMESTATEMGR *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACMESTATEMGR_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACMESTATEMGR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACMESTATEMGR_parent_set" "', argument " "1"" of type '" "struct _dwg_object_ACMESTATEMGR *""'"); 
    }
    arg1 = (struct _dwg_object_ACMESTATEMGR *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_ACMESTATEMGR_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACMESTATEMGR_parent_get) {
  {
    struct _dwg_object_ACMESTATEMGR *arg1 = (struct _dwg_object_ACMESTATEMGR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACMESTATEMGR_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACMESTATEMGR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACMESTATEMGR_parent_get" "', argument " "1"" of type '" "struct _dwg_object_ACMESTATEMGR *""'"); 
    }
    arg1 = (struct _dwg_object_ACMESTATEMGR *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACMESTATEMGR_class_version_set) {
  {
    struct _dwg_object_ACMESTATEMGR *arg1 = (struct _dwg_object_ACMESTATEMGR *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_ACMESTATEMGR_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACMESTATEMGR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACMESTATEMGR_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_ACMESTATEMGR *""'"); 
    }
    arg1 = (struct _dwg_object_ACMESTATEMGR *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_ACMESTATEMGR_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_ACMESTATEMGR_class_version_get) {
  {
    struct _dwg_object_ACMESTATEMGR *arg1 = (struct _dwg_object_ACMESTATEMGR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_ACMESTATEMGR_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACMESTATEMGR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_ACMESTATEMGR_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_ACMESTATEMGR *""'"); 
    }
    arg1 = (struct _dwg_object_ACMESTATEMGR *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_ACMESTATEMGR) {
  {
    int argvi = 0;
    struct _dwg_object_ACMESTATEMGR *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_ACMESTATEMGR();");
    }
    result = (struct _dwg_object_ACMESTATEMGR *)calloc(1, sizeof(struct _dwg_object_ACMESTATEMGR));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACMESTATEMGR, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_ACMESTATEMGR) {
  {
    struct _dwg_object_ACMESTATEMGR *arg1 = (struct _dwg_object_ACMESTATEMGR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_ACMESTATEMGR(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ACMESTATEMGR, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_ACMESTATEMGR" "', argument " "1"" of type '" "struct _dwg_object_ACMESTATEMGR *""'"); 
    }
    arg1 = (struct _dwg_object_ACMESTATEMGR *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_CSACDOCUMENTOPTIONS_parent_set) {
  {
    struct _dwg_object_CSACDOCUMENTOPTIONS *arg1 = (struct _dwg_object_CSACDOCUMENTOPTIONS *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_CSACDOCUMENTOPTIONS_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_CSACDOCUMENTOPTIONS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_CSACDOCUMENTOPTIONS_parent_set" "', argument " "1"" of type '" "struct _dwg_object_CSACDOCUMENTOPTIONS *""'"); 
    }
    arg1 = (struct _dwg_object_CSACDOCUMENTOPTIONS *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_CSACDOCUMENTOPTIONS_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_CSACDOCUMENTOPTIONS_parent_get) {
  {
    struct _dwg_object_CSACDOCUMENTOPTIONS *arg1 = (struct _dwg_object_CSACDOCUMENTOPTIONS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_CSACDOCUMENTOPTIONS_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_CSACDOCUMENTOPTIONS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_CSACDOCUMENTOPTIONS_parent_get" "', argument " "1"" of type '" "struct _dwg_object_CSACDOCUMENTOPTIONS *""'"); 
    }
    arg1 = (struct _dwg_object_CSACDOCUMENTOPTIONS *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_CSACDOCUMENTOPTIONS_class_version_set) {
  {
    struct _dwg_object_CSACDOCUMENTOPTIONS *arg1 = (struct _dwg_object_CSACDOCUMENTOPTIONS *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_CSACDOCUMENTOPTIONS_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_CSACDOCUMENTOPTIONS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_CSACDOCUMENTOPTIONS_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_CSACDOCUMENTOPTIONS *""'"); 
    }
    arg1 = (struct _dwg_object_CSACDOCUMENTOPTIONS *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_CSACDOCUMENTOPTIONS_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_CSACDOCUMENTOPTIONS_class_version_get) {
  {
    struct _dwg_object_CSACDOCUMENTOPTIONS *arg1 = (struct _dwg_object_CSACDOCUMENTOPTIONS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_CSACDOCUMENTOPTIONS_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_CSACDOCUMENTOPTIONS, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_CSACDOCUMENTOPTIONS_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_CSACDOCUMENTOPTIONS *""'"); 
    }
    arg1 = (struct _dwg_object_CSACDOCUMENTOPTIONS *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_CSACDOCUMENTOPTIONS) {
  {
    int argvi = 0;
    struct _dwg_object_CSACDOCUMENTOPTIONS *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_CSACDOCUMENTOPTIONS();");
    }
    result = (struct _dwg_object_CSACDOCUMENTOPTIONS *)calloc(1, sizeof(struct _dwg_object_CSACDOCUMENTOPTIONS));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_CSACDOCUMENTOPTIONS, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_CSACDOCUMENTOPTIONS) {
  {
    struct _dwg_object_CSACDOCUMENTOPTIONS *arg1 = (struct _dwg_object_CSACDOCUMENTOPTIONS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_CSACDOCUMENTOPTIONS(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_CSACDOCUMENTOPTIONS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_CSACDOCUMENTOPTIONS" "', argument " "1"" of type '" "struct _dwg_object_CSACDOCUMENTOPTIONS *""'"); 
    }
    arg1 = (struct _dwg_object_CSACDOCUMENTOPTIONS *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKPARAMETER_connection_code_set) {
  {
    struct _dwg_BLOCKPARAMETER_connection *arg1 = (struct _dwg_BLOCKPARAMETER_connection *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_BLOCKPARAMETER_connection_code_set(self,code);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKPARAMETER_connection, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKPARAMETER_connection_code_set" "', argument " "1"" of type '" "struct _dwg_BLOCKPARAMETER_connection *""'"); 
    }
    arg1 = (struct _dwg_BLOCKPARAMETER_connection *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_BLOCKPARAMETER_connection_code_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->code = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKPARAMETER_connection_code_get) {
  {
    struct _dwg_BLOCKPARAMETER_connection *arg1 = (struct _dwg_BLOCKPARAMETER_connection *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_BLOCKPARAMETER_connection_code_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKPARAMETER_connection, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKPARAMETER_connection_code_get" "', argument " "1"" of type '" "struct _dwg_BLOCKPARAMETER_connection *""'"); 
    }
    arg1 = (struct _dwg_BLOCKPARAMETER_connection *)(argp1);
    result = (BITCODE_BL) ((arg1)->code);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKPARAMETER_connection_name_set) {
  {
    struct _dwg_BLOCKPARAMETER_connection *arg1 = (struct _dwg_BLOCKPARAMETER_connection *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_BLOCKPARAMETER_connection_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKPARAMETER_connection, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKPARAMETER_connection_name_set" "', argument " "1"" of type '" "struct _dwg_BLOCKPARAMETER_connection *""'"); 
    }
    arg1 = (struct _dwg_BLOCKPARAMETER_connection *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_BLOCKPARAMETER_connection_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKPARAMETER_connection_name_get) {
  {
    struct _dwg_BLOCKPARAMETER_connection *arg1 = (struct _dwg_BLOCKPARAMETER_connection *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_BLOCKPARAMETER_connection_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKPARAMETER_connection, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKPARAMETER_connection_name_get" "', argument " "1"" of type '" "struct _dwg_BLOCKPARAMETER_connection *""'"); 
    }
    arg1 = (struct _dwg_BLOCKPARAMETER_connection *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_BLOCKPARAMETER_connection) {
  {
    int argvi = 0;
    struct _dwg_BLOCKPARAMETER_connection *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_BLOCKPARAMETER_connection();");
    }
    result = (struct _dwg_BLOCKPARAMETER_connection *)calloc(1, sizeof(struct _dwg_BLOCKPARAMETER_connection));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_connection, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_BLOCKPARAMETER_connection) {
  {
    struct _dwg_BLOCKPARAMETER_connection *arg1 = (struct _dwg_BLOCKPARAMETER_connection *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_BLOCKPARAMETER_connection(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKPARAMETER_connection, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_BLOCKPARAMETER_connection" "', argument " "1"" of type '" "struct _dwg_BLOCKPARAMETER_connection *""'"); 
    }
    arg1 = (struct _dwg_BLOCKPARAMETER_connection *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKPARAMETER_PropInfo_num_connections_set) {
  {
    struct _dwg_BLOCKPARAMETER_PropInfo *arg1 = (struct _dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_BLOCKPARAMETER_PropInfo_num_connections_set(self,num_connections);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKPARAMETER_PropInfo_num_connections_set" "', argument " "1"" of type '" "struct _dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg1 = (struct _dwg_BLOCKPARAMETER_PropInfo *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_BLOCKPARAMETER_PropInfo_num_connections_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_connections = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKPARAMETER_PropInfo_num_connections_get) {
  {
    struct _dwg_BLOCKPARAMETER_PropInfo *arg1 = (struct _dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_BLOCKPARAMETER_PropInfo_num_connections_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKPARAMETER_PropInfo_num_connections_get" "', argument " "1"" of type '" "struct _dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg1 = (struct _dwg_BLOCKPARAMETER_PropInfo *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_connections);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKPARAMETER_PropInfo_connections_set) {
  {
    struct _dwg_BLOCKPARAMETER_PropInfo *arg1 = (struct _dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    Dwg_BLOCKPARAMETER_connection *arg2 = (Dwg_BLOCKPARAMETER_connection *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_BLOCKPARAMETER_PropInfo_connections_set(self,connections);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKPARAMETER_PropInfo_connections_set" "', argument " "1"" of type '" "struct _dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg1 = (struct _dwg_BLOCKPARAMETER_PropInfo *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_connection, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_BLOCKPARAMETER_PropInfo_connections_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_connection *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_connection *)(argp2);
    if (arg1) (arg1)->connections = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKPARAMETER_PropInfo_connections_get) {
  {
    struct _dwg_BLOCKPARAMETER_PropInfo *arg1 = (struct _dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_connection *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_BLOCKPARAMETER_PropInfo_connections_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKPARAMETER_PropInfo_connections_get" "', argument " "1"" of type '" "struct _dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg1 = (struct _dwg_BLOCKPARAMETER_PropInfo *)(argp1);
    result = (Dwg_BLOCKPARAMETER_connection *) ((arg1)->connections);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_connection, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_BLOCKPARAMETER_PropInfo) {
  {
    int argvi = 0;
    struct _dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_BLOCKPARAMETER_PropInfo();");
    }
    result = (struct _dwg_BLOCKPARAMETER_PropInfo *)calloc(1, sizeof(struct _dwg_BLOCKPARAMETER_PropInfo));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_BLOCKPARAMETER_PropInfo) {
  {
    struct _dwg_BLOCKPARAMETER_PropInfo *arg1 = (struct _dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_BLOCKPARAMETER_PropInfo(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_BLOCKPARAMETER_PropInfo" "', argument " "1"" of type '" "struct _dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg1 = (struct _dwg_BLOCKPARAMETER_PropInfo *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKPARAMVALUESET_desc_set) {
  {
    struct _dwg_BLOCKPARAMVALUESET *arg1 = (struct _dwg_BLOCKPARAMVALUESET *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_BLOCKPARAMVALUESET_desc_set(self,desc);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKPARAMVALUESET_desc_set" "', argument " "1"" of type '" "struct _dwg_BLOCKPARAMVALUESET *""'"); 
    }
    arg1 = (struct _dwg_BLOCKPARAMVALUESET *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_BLOCKPARAMVALUESET_desc_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->desc);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->desc = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->desc = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKPARAMVALUESET_desc_get) {
  {
    struct _dwg_BLOCKPARAMVALUESET *arg1 = (struct _dwg_BLOCKPARAMVALUESET *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_BLOCKPARAMVALUESET_desc_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKPARAMVALUESET_desc_get" "', argument " "1"" of type '" "struct _dwg_BLOCKPARAMVALUESET *""'"); 
    }
    arg1 = (struct _dwg_BLOCKPARAMVALUESET *)(argp1);
    result = (BITCODE_TV) ((arg1)->desc);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKPARAMVALUESET_flags_set) {
  {
    struct _dwg_BLOCKPARAMVALUESET *arg1 = (struct _dwg_BLOCKPARAMVALUESET *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_BLOCKPARAMVALUESET_flags_set(self,flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKPARAMVALUESET_flags_set" "', argument " "1"" of type '" "struct _dwg_BLOCKPARAMVALUESET *""'"); 
    }
    arg1 = (struct _dwg_BLOCKPARAMVALUESET *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_BLOCKPARAMVALUESET_flags_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKPARAMVALUESET_flags_get) {
  {
    struct _dwg_BLOCKPARAMVALUESET *arg1 = (struct _dwg_BLOCKPARAMVALUESET *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_BLOCKPARAMVALUESET_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKPARAMVALUESET_flags_get" "', argument " "1"" of type '" "struct _dwg_BLOCKPARAMVALUESET *""'"); 
    }
    arg1 = (struct _dwg_BLOCKPARAMVALUESET *)(argp1);
    result = (BITCODE_BL) ((arg1)->flags);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKPARAMVALUESET_minimum_set) {
  {
    struct _dwg_BLOCKPARAMVALUESET *arg1 = (struct _dwg_BLOCKPARAMVALUESET *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_BLOCKPARAMVALUESET_minimum_set(self,minimum);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKPARAMVALUESET_minimum_set" "', argument " "1"" of type '" "struct _dwg_BLOCKPARAMVALUESET *""'"); 
    }
    arg1 = (struct _dwg_BLOCKPARAMVALUESET *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_BLOCKPARAMVALUESET_minimum_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->minimum = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKPARAMVALUESET_minimum_get) {
  {
    struct _dwg_BLOCKPARAMVALUESET *arg1 = (struct _dwg_BLOCKPARAMVALUESET *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_BLOCKPARAMVALUESET_minimum_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKPARAMVALUESET_minimum_get" "', argument " "1"" of type '" "struct _dwg_BLOCKPARAMVALUESET *""'"); 
    }
    arg1 = (struct _dwg_BLOCKPARAMVALUESET *)(argp1);
    result = (BITCODE_BD) ((arg1)->minimum);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKPARAMVALUESET_maximum_set) {
  {
    struct _dwg_BLOCKPARAMVALUESET *arg1 = (struct _dwg_BLOCKPARAMVALUESET *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_BLOCKPARAMVALUESET_maximum_set(self,maximum);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKPARAMVALUESET_maximum_set" "', argument " "1"" of type '" "struct _dwg_BLOCKPARAMVALUESET *""'"); 
    }
    arg1 = (struct _dwg_BLOCKPARAMVALUESET *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_BLOCKPARAMVALUESET_maximum_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->maximum = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKPARAMVALUESET_maximum_get) {
  {
    struct _dwg_BLOCKPARAMVALUESET *arg1 = (struct _dwg_BLOCKPARAMVALUESET *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_BLOCKPARAMVALUESET_maximum_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKPARAMVALUESET_maximum_get" "', argument " "1"" of type '" "struct _dwg_BLOCKPARAMVALUESET *""'"); 
    }
    arg1 = (struct _dwg_BLOCKPARAMVALUESET *)(argp1);
    result = (BITCODE_BD) ((arg1)->maximum);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKPARAMVALUESET_increment_set) {
  {
    struct _dwg_BLOCKPARAMVALUESET *arg1 = (struct _dwg_BLOCKPARAMVALUESET *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_BLOCKPARAMVALUESET_increment_set(self,increment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKPARAMVALUESET_increment_set" "', argument " "1"" of type '" "struct _dwg_BLOCKPARAMVALUESET *""'"); 
    }
    arg1 = (struct _dwg_BLOCKPARAMVALUESET *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_BLOCKPARAMVALUESET_increment_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->increment = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKPARAMVALUESET_increment_get) {
  {
    struct _dwg_BLOCKPARAMVALUESET *arg1 = (struct _dwg_BLOCKPARAMVALUESET *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_BLOCKPARAMVALUESET_increment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKPARAMVALUESET_increment_get" "', argument " "1"" of type '" "struct _dwg_BLOCKPARAMVALUESET *""'"); 
    }
    arg1 = (struct _dwg_BLOCKPARAMVALUESET *)(argp1);
    result = (BITCODE_BD) ((arg1)->increment);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKPARAMVALUESET_num_valuelist_set) {
  {
    struct _dwg_BLOCKPARAMVALUESET *arg1 = (struct _dwg_BLOCKPARAMVALUESET *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_BLOCKPARAMVALUESET_num_valuelist_set(self,num_valuelist);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKPARAMVALUESET_num_valuelist_set" "', argument " "1"" of type '" "struct _dwg_BLOCKPARAMVALUESET *""'"); 
    }
    arg1 = (struct _dwg_BLOCKPARAMVALUESET *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_BLOCKPARAMVALUESET_num_valuelist_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_valuelist = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKPARAMVALUESET_num_valuelist_get) {
  {
    struct _dwg_BLOCKPARAMVALUESET *arg1 = (struct _dwg_BLOCKPARAMVALUESET *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_BLOCKPARAMVALUESET_num_valuelist_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKPARAMVALUESET_num_valuelist_get" "', argument " "1"" of type '" "struct _dwg_BLOCKPARAMVALUESET *""'"); 
    }
    arg1 = (struct _dwg_BLOCKPARAMVALUESET *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_valuelist);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKPARAMVALUESET_valuelist_set) {
  {
    struct _dwg_BLOCKPARAMVALUESET *arg1 = (struct _dwg_BLOCKPARAMVALUESET *) 0 ;
    BITCODE_BD *arg2 = (BITCODE_BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_BLOCKPARAMVALUESET_valuelist_set(self,valuelist);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKPARAMVALUESET_valuelist_set" "', argument " "1"" of type '" "struct _dwg_BLOCKPARAMVALUESET *""'"); 
    }
    arg1 = (struct _dwg_BLOCKPARAMVALUESET *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_double, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_BLOCKPARAMVALUESET_valuelist_set" "', argument " "2"" of type '" "BITCODE_BD *""'"); 
    }
    arg2 = (BITCODE_BD *)(argp2);
    if (arg1) (arg1)->valuelist = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKPARAMVALUESET_valuelist_get) {
  {
    struct _dwg_BLOCKPARAMVALUESET *arg1 = (struct _dwg_BLOCKPARAMVALUESET *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_BLOCKPARAMVALUESET_valuelist_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKPARAMVALUESET_valuelist_get" "', argument " "1"" of type '" "struct _dwg_BLOCKPARAMVALUESET *""'"); 
    }
    arg1 = (struct _dwg_BLOCKPARAMVALUESET *)(argp1);
    result = (BITCODE_BD *) ((arg1)->valuelist);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_double, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_BLOCKPARAMVALUESET) {
  {
    int argvi = 0;
    struct _dwg_BLOCKPARAMVALUESET *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_BLOCKPARAMVALUESET();");
    }
    result = (struct _dwg_BLOCKPARAMVALUESET *)calloc(1, sizeof(struct _dwg_BLOCKPARAMVALUESET));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_BLOCKPARAMVALUESET) {
  {
    struct _dwg_BLOCKPARAMVALUESET *arg1 = (struct _dwg_BLOCKPARAMVALUESET *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_BLOCKPARAMVALUESET(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_BLOCKPARAMVALUESET" "', argument " "1"" of type '" "struct _dwg_BLOCKPARAMVALUESET *""'"); 
    }
    arg1 = (struct _dwg_BLOCKPARAMVALUESET *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKACTION_connectionpts_code_set) {
  {
    struct _dwg_BLOCKACTION_connectionpts *arg1 = (struct _dwg_BLOCKACTION_connectionpts *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_BLOCKACTION_connectionpts_code_set(self,code);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKACTION_connectionpts, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKACTION_connectionpts_code_set" "', argument " "1"" of type '" "struct _dwg_BLOCKACTION_connectionpts *""'"); 
    }
    arg1 = (struct _dwg_BLOCKACTION_connectionpts *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_BLOCKACTION_connectionpts_code_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->code = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKACTION_connectionpts_code_get) {
  {
    struct _dwg_BLOCKACTION_connectionpts *arg1 = (struct _dwg_BLOCKACTION_connectionpts *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_BLOCKACTION_connectionpts_code_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKACTION_connectionpts, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKACTION_connectionpts_code_get" "', argument " "1"" of type '" "struct _dwg_BLOCKACTION_connectionpts *""'"); 
    }
    arg1 = (struct _dwg_BLOCKACTION_connectionpts *)(argp1);
    result = (BITCODE_BL) ((arg1)->code);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKACTION_connectionpts_name_set) {
  {
    struct _dwg_BLOCKACTION_connectionpts *arg1 = (struct _dwg_BLOCKACTION_connectionpts *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_BLOCKACTION_connectionpts_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKACTION_connectionpts, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKACTION_connectionpts_name_set" "', argument " "1"" of type '" "struct _dwg_BLOCKACTION_connectionpts *""'"); 
    }
    arg1 = (struct _dwg_BLOCKACTION_connectionpts *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_BLOCKACTION_connectionpts_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKACTION_connectionpts_name_get) {
  {
    struct _dwg_BLOCKACTION_connectionpts *arg1 = (struct _dwg_BLOCKACTION_connectionpts *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_BLOCKACTION_connectionpts_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKACTION_connectionpts, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKACTION_connectionpts_name_get" "', argument " "1"" of type '" "struct _dwg_BLOCKACTION_connectionpts *""'"); 
    }
    arg1 = (struct _dwg_BLOCKACTION_connectionpts *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_BLOCKACTION_connectionpts) {
  {
    int argvi = 0;
    struct _dwg_BLOCKACTION_connectionpts *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_BLOCKACTION_connectionpts();");
    }
    result = (struct _dwg_BLOCKACTION_connectionpts *)calloc(1, sizeof(struct _dwg_BLOCKACTION_connectionpts));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKACTION_connectionpts, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_BLOCKACTION_connectionpts) {
  {
    struct _dwg_BLOCKACTION_connectionpts *arg1 = (struct _dwg_BLOCKACTION_connectionpts *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_BLOCKACTION_connectionpts(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKACTION_connectionpts, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_BLOCKACTION_connectionpts" "', argument " "1"" of type '" "struct _dwg_BLOCKACTION_connectionpts *""'"); 
    }
    arg1 = (struct _dwg_BLOCKACTION_connectionpts *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKVISIBILITYPARAMETER_state_parent_set) {
  {
    struct _dwg_BLOCKVISIBILITYPARAMETER_state *arg1 = (struct _dwg_BLOCKVISIBILITYPARAMETER_state *) 0 ;
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg2 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_BLOCKVISIBILITYPARAMETER_state_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKVISIBILITYPARAMETER_state, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKVISIBILITYPARAMETER_state_parent_set" "', argument " "1"" of type '" "struct _dwg_BLOCKVISIBILITYPARAMETER_state *""'"); 
    }
    arg1 = (struct _dwg_BLOCKVISIBILITYPARAMETER_state *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_BLOCKVISIBILITYPARAMETER_state_parent_set" "', argument " "2"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg2 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKVISIBILITYPARAMETER_state_parent_get) {
  {
    struct _dwg_BLOCKVISIBILITYPARAMETER_state *arg1 = (struct _dwg_BLOCKVISIBILITYPARAMETER_state *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_BLOCKVISIBILITYPARAMETER_state_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKVISIBILITYPARAMETER_state, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKVISIBILITYPARAMETER_state_parent_get" "', argument " "1"" of type '" "struct _dwg_BLOCKVISIBILITYPARAMETER_state *""'"); 
    }
    arg1 = (struct _dwg_BLOCKVISIBILITYPARAMETER_state *)(argp1);
    result = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKVISIBILITYPARAMETER_state_name_set) {
  {
    struct _dwg_BLOCKVISIBILITYPARAMETER_state *arg1 = (struct _dwg_BLOCKVISIBILITYPARAMETER_state *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_BLOCKVISIBILITYPARAMETER_state_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKVISIBILITYPARAMETER_state, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKVISIBILITYPARAMETER_state_name_set" "', argument " "1"" of type '" "struct _dwg_BLOCKVISIBILITYPARAMETER_state *""'"); 
    }
    arg1 = (struct _dwg_BLOCKVISIBILITYPARAMETER_state *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_BLOCKVISIBILITYPARAMETER_state_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKVISIBILITYPARAMETER_state_name_get) {
  {
    struct _dwg_BLOCKVISIBILITYPARAMETER_state *arg1 = (struct _dwg_BLOCKVISIBILITYPARAMETER_state *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_BLOCKVISIBILITYPARAMETER_state_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKVISIBILITYPARAMETER_state, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKVISIBILITYPARAMETER_state_name_get" "', argument " "1"" of type '" "struct _dwg_BLOCKVISIBILITYPARAMETER_state *""'"); 
    }
    arg1 = (struct _dwg_BLOCKVISIBILITYPARAMETER_state *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKVISIBILITYPARAMETER_state_num_blocks_set) {
  {
    struct _dwg_BLOCKVISIBILITYPARAMETER_state *arg1 = (struct _dwg_BLOCKVISIBILITYPARAMETER_state *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_BLOCKVISIBILITYPARAMETER_state_num_blocks_set(self,num_blocks);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKVISIBILITYPARAMETER_state, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKVISIBILITYPARAMETER_state_num_blocks_set" "', argument " "1"" of type '" "struct _dwg_BLOCKVISIBILITYPARAMETER_state *""'"); 
    }
    arg1 = (struct _dwg_BLOCKVISIBILITYPARAMETER_state *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_BLOCKVISIBILITYPARAMETER_state_num_blocks_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_blocks = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKVISIBILITYPARAMETER_state_num_blocks_get) {
  {
    struct _dwg_BLOCKVISIBILITYPARAMETER_state *arg1 = (struct _dwg_BLOCKVISIBILITYPARAMETER_state *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_BLOCKVISIBILITYPARAMETER_state_num_blocks_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKVISIBILITYPARAMETER_state, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKVISIBILITYPARAMETER_state_num_blocks_get" "', argument " "1"" of type '" "struct _dwg_BLOCKVISIBILITYPARAMETER_state *""'"); 
    }
    arg1 = (struct _dwg_BLOCKVISIBILITYPARAMETER_state *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_blocks);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKVISIBILITYPARAMETER_state_blocks_set) {
  {
    struct _dwg_BLOCKVISIBILITYPARAMETER_state *arg1 = (struct _dwg_BLOCKVISIBILITYPARAMETER_state *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_BLOCKVISIBILITYPARAMETER_state_blocks_set(self,blocks);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKVISIBILITYPARAMETER_state, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKVISIBILITYPARAMETER_state_blocks_set" "', argument " "1"" of type '" "struct _dwg_BLOCKVISIBILITYPARAMETER_state *""'"); 
    }
    arg1 = (struct _dwg_BLOCKVISIBILITYPARAMETER_state *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_BLOCKVISIBILITYPARAMETER_state_blocks_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->blocks = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKVISIBILITYPARAMETER_state_blocks_get) {
  {
    struct _dwg_BLOCKVISIBILITYPARAMETER_state *arg1 = (struct _dwg_BLOCKVISIBILITYPARAMETER_state *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_BLOCKVISIBILITYPARAMETER_state_blocks_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKVISIBILITYPARAMETER_state, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKVISIBILITYPARAMETER_state_blocks_get" "', argument " "1"" of type '" "struct _dwg_BLOCKVISIBILITYPARAMETER_state *""'"); 
    }
    arg1 = (struct _dwg_BLOCKVISIBILITYPARAMETER_state *)(argp1);
    result = (BITCODE_H *) ((arg1)->blocks);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKVISIBILITYPARAMETER_state_num_params_set) {
  {
    struct _dwg_BLOCKVISIBILITYPARAMETER_state *arg1 = (struct _dwg_BLOCKVISIBILITYPARAMETER_state *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_BLOCKVISIBILITYPARAMETER_state_num_params_set(self,num_params);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKVISIBILITYPARAMETER_state, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKVISIBILITYPARAMETER_state_num_params_set" "', argument " "1"" of type '" "struct _dwg_BLOCKVISIBILITYPARAMETER_state *""'"); 
    }
    arg1 = (struct _dwg_BLOCKVISIBILITYPARAMETER_state *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_BLOCKVISIBILITYPARAMETER_state_num_params_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_params = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKVISIBILITYPARAMETER_state_num_params_get) {
  {
    struct _dwg_BLOCKVISIBILITYPARAMETER_state *arg1 = (struct _dwg_BLOCKVISIBILITYPARAMETER_state *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_BLOCKVISIBILITYPARAMETER_state_num_params_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKVISIBILITYPARAMETER_state, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKVISIBILITYPARAMETER_state_num_params_get" "', argument " "1"" of type '" "struct _dwg_BLOCKVISIBILITYPARAMETER_state *""'"); 
    }
    arg1 = (struct _dwg_BLOCKVISIBILITYPARAMETER_state *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_params);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKVISIBILITYPARAMETER_state_params_set) {
  {
    struct _dwg_BLOCKVISIBILITYPARAMETER_state *arg1 = (struct _dwg_BLOCKVISIBILITYPARAMETER_state *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_BLOCKVISIBILITYPARAMETER_state_params_set(self,params);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKVISIBILITYPARAMETER_state, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKVISIBILITYPARAMETER_state_params_set" "', argument " "1"" of type '" "struct _dwg_BLOCKVISIBILITYPARAMETER_state *""'"); 
    }
    arg1 = (struct _dwg_BLOCKVISIBILITYPARAMETER_state *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_BLOCKVISIBILITYPARAMETER_state_params_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->params = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKVISIBILITYPARAMETER_state_params_get) {
  {
    struct _dwg_BLOCKVISIBILITYPARAMETER_state *arg1 = (struct _dwg_BLOCKVISIBILITYPARAMETER_state *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_BLOCKVISIBILITYPARAMETER_state_params_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKVISIBILITYPARAMETER_state, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKVISIBILITYPARAMETER_state_params_get" "', argument " "1"" of type '" "struct _dwg_BLOCKVISIBILITYPARAMETER_state *""'"); 
    }
    arg1 = (struct _dwg_BLOCKVISIBILITYPARAMETER_state *)(argp1);
    result = (BITCODE_H *) ((arg1)->params);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_BLOCKVISIBILITYPARAMETER_state) {
  {
    int argvi = 0;
    struct _dwg_BLOCKVISIBILITYPARAMETER_state *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_BLOCKVISIBILITYPARAMETER_state();");
    }
    result = (struct _dwg_BLOCKVISIBILITYPARAMETER_state *)calloc(1, sizeof(struct _dwg_BLOCKVISIBILITYPARAMETER_state));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKVISIBILITYPARAMETER_state, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_BLOCKVISIBILITYPARAMETER_state) {
  {
    struct _dwg_BLOCKVISIBILITYPARAMETER_state *arg1 = (struct _dwg_BLOCKVISIBILITYPARAMETER_state *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_BLOCKVISIBILITYPARAMETER_state(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKVISIBILITYPARAMETER_state, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_BLOCKVISIBILITYPARAMETER_state" "', argument " "1"" of type '" "struct _dwg_BLOCKVISIBILITYPARAMETER_state *""'"); 
    }
    arg1 = (struct _dwg_BLOCKVISIBILITYPARAMETER_state *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_parent_set) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_parent_get) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_evalexpr_set) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_evalexpr_get) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_name_set) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_name_get) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_be_major_set) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_be_major_set(self,be_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_be_major_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_be_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_be_major_get) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_be_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_be_major_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_be_minor_set) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_be_minor_set(self,be_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_be_minor_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_be_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_be_minor_get) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_be_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_be_minor_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_eed1071_set) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_eed1071_set(self,eed1071);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_eed1071_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_eed1071_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->eed1071 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_eed1071_get) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_eed1071_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_eed1071_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->eed1071);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_show_properties_set) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_show_properties_set(self,show_properties);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_show_properties_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_show_properties_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->show_properties = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_show_properties_get) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_show_properties_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_show_properties_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->show_properties);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_chain_actions_set) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_chain_actions_set(self,chain_actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_chain_actions_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_chain_actions_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->chain_actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_chain_actions_get) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_chain_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_chain_actions_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->chain_actions);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_def_pt_set) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_def_pt_set(self,def_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_def_pt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_def_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_def_pt_get) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_def_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_def_pt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_num_propinfos_set) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_num_propinfos_set(self,num_propinfos);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_num_propinfos_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_num_propinfos_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_propinfos = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_num_propinfos_get) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_num_propinfos_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_num_propinfos_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_propinfos);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_prop1_set) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_prop1_set(self,prop1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_prop1_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_prop1_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop1 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_prop1_get) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_prop1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_prop1_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_prop2_set) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_prop2_set(self,prop2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_prop2_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_prop2_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop2 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_prop2_get) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_prop2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_prop2_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_is_initialized_set) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_is_initialized_set(self,is_initialized);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_is_initialized_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_is_initialized_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_initialized = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_is_initialized_get) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_is_initialized_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_is_initialized_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->is_initialized);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_unknown_bool_set) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_unknown_bool_set(self,unknown_bool);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_unknown_bool_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_unknown_bool_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown_bool = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_unknown_bool_get) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_unknown_bool_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_unknown_bool_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown_bool);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_blockvisi_name_set) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_blockvisi_name_set(self,blockvisi_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_blockvisi_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_blockvisi_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->blockvisi_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->blockvisi_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->blockvisi_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_blockvisi_name_get) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_blockvisi_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_blockvisi_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->blockvisi_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_blockvisi_desc_set) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_blockvisi_desc_set(self,blockvisi_desc);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_blockvisi_desc_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_blockvisi_desc_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->blockvisi_desc);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->blockvisi_desc = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->blockvisi_desc = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_blockvisi_desc_get) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_blockvisi_desc_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_blockvisi_desc_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->blockvisi_desc);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_num_blocks_set) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_num_blocks_set(self,num_blocks);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_num_blocks_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_num_blocks_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_blocks = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_num_blocks_get) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_num_blocks_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_num_blocks_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_blocks);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_blocks_set) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_blocks_set(self,blocks);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_blocks_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_blocks_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->blocks = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_blocks_get) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_blocks_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_blocks_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    result = (BITCODE_H *) ((arg1)->blocks);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_num_states_set) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_num_states_set(self,num_states);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_num_states_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_num_states_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_states = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_num_states_get) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_num_states_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_num_states_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_states);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_states_set) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    Dwg_BLOCKVISIBILITYPARAMETER_state *arg2 = (Dwg_BLOCKVISIBILITYPARAMETER_state *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_states_set(self,states);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_states_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKVISIBILITYPARAMETER_state, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_states_set" "', argument " "2"" of type '" "Dwg_BLOCKVISIBILITYPARAMETER_state *""'"); 
    }
    arg2 = (Dwg_BLOCKVISIBILITYPARAMETER_state *)(argp2);
    if (arg1) (arg1)->states = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_states_get) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKVISIBILITYPARAMETER_state *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYPARAMETER_states_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYPARAMETER_states_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    result = (Dwg_BLOCKVISIBILITYPARAMETER_state *) ((arg1)->states);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKVISIBILITYPARAMETER_state, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKVISIBILITYPARAMETER) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKVISIBILITYPARAMETER();");
    }
    result = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)calloc(1, sizeof(struct _dwg_object_BLOCKVISIBILITYPARAMETER));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKVISIBILITYPARAMETER) {
  {
    struct _dwg_object_BLOCKVISIBILITYPARAMETER *arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKVISIBILITYPARAMETER(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKVISIBILITYPARAMETER" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYPARAMETER *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYGRIP_parent_set) {
  {
    struct _dwg_object_BLOCKVISIBILITYGRIP *arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYGRIP_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYGRIP_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKVISIBILITYGRIP_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYGRIP_parent_get) {
  {
    struct _dwg_object_BLOCKVISIBILITYGRIP *arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYGRIP_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYGRIP_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYGRIP_evalexpr_set) {
  {
    struct _dwg_object_BLOCKVISIBILITYGRIP *arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYGRIP_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYGRIP_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKVISIBILITYGRIP_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYGRIP_evalexpr_get) {
  {
    struct _dwg_object_BLOCKVISIBILITYGRIP *arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYGRIP_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYGRIP_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYGRIP_name_set) {
  {
    struct _dwg_object_BLOCKVISIBILITYGRIP *arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYGRIP_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYGRIP_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKVISIBILITYGRIP_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYGRIP_name_get) {
  {
    struct _dwg_object_BLOCKVISIBILITYGRIP *arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYGRIP_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYGRIP_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYGRIP_be_major_set) {
  {
    struct _dwg_object_BLOCKVISIBILITYGRIP *arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYGRIP_be_major_set(self,be_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYGRIP_be_major_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKVISIBILITYGRIP_be_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYGRIP_be_major_get) {
  {
    struct _dwg_object_BLOCKVISIBILITYGRIP *arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYGRIP_be_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYGRIP_be_major_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYGRIP_be_minor_set) {
  {
    struct _dwg_object_BLOCKVISIBILITYGRIP *arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYGRIP_be_minor_set(self,be_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYGRIP_be_minor_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKVISIBILITYGRIP_be_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYGRIP_be_minor_get) {
  {
    struct _dwg_object_BLOCKVISIBILITYGRIP *arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYGRIP_be_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYGRIP_be_minor_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYGRIP_eed1071_set) {
  {
    struct _dwg_object_BLOCKVISIBILITYGRIP *arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYGRIP_eed1071_set(self,eed1071);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYGRIP_eed1071_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKVISIBILITYGRIP_eed1071_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->eed1071 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYGRIP_eed1071_get) {
  {
    struct _dwg_object_BLOCKVISIBILITYGRIP *arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYGRIP_eed1071_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYGRIP_eed1071_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->eed1071);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYGRIP_bg_bl91_set) {
  {
    struct _dwg_object_BLOCKVISIBILITYGRIP *arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYGRIP_bg_bl91_set(self,bg_bl91);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYGRIP_bg_bl91_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKVISIBILITYGRIP_bg_bl91_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bg_bl91 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYGRIP_bg_bl91_get) {
  {
    struct _dwg_object_BLOCKVISIBILITYGRIP *arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYGRIP_bg_bl91_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYGRIP_bg_bl91_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->bg_bl91);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYGRIP_bg_bl92_set) {
  {
    struct _dwg_object_BLOCKVISIBILITYGRIP *arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYGRIP_bg_bl92_set(self,bg_bl92);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYGRIP_bg_bl92_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKVISIBILITYGRIP_bg_bl92_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bg_bl92 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYGRIP_bg_bl92_get) {
  {
    struct _dwg_object_BLOCKVISIBILITYGRIP *arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYGRIP_bg_bl92_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYGRIP_bg_bl92_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->bg_bl92);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYGRIP_bg_location_set) {
  {
    struct _dwg_object_BLOCKVISIBILITYGRIP *arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYGRIP_bg_location_set(self,bg_location);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYGRIP_bg_location_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKVISIBILITYGRIP_bg_location_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->bg_location = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYGRIP_bg_location_get) {
  {
    struct _dwg_object_BLOCKVISIBILITYGRIP *arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYGRIP_bg_location_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYGRIP_bg_location_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->bg_location);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYGRIP_bg_insert_cycling_set) {
  {
    struct _dwg_object_BLOCKVISIBILITYGRIP *arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYGRIP_bg_insert_cycling_set(self,bg_insert_cycling);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYGRIP_bg_insert_cycling_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKVISIBILITYGRIP_bg_insert_cycling_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->bg_insert_cycling = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYGRIP_bg_insert_cycling_get) {
  {
    struct _dwg_object_BLOCKVISIBILITYGRIP *arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYGRIP_bg_insert_cycling_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYGRIP_bg_insert_cycling_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *)(argp1);
    result = (BITCODE_B) ((arg1)->bg_insert_cycling);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYGRIP_bg_insert_cycling_weight_set) {
  {
    struct _dwg_object_BLOCKVISIBILITYGRIP *arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *) 0 ;
    BITCODE_BLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYGRIP_bg_insert_cycling_weight_set(self,bg_insert_cycling_weight);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYGRIP_bg_insert_cycling_weight_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKVISIBILITYGRIP_bg_insert_cycling_weight_set" "', argument " "2"" of type '" "BITCODE_BLd""'");
    } 
    arg2 = (BITCODE_BLd)(val2);
    if (arg1) (arg1)->bg_insert_cycling_weight = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVISIBILITYGRIP_bg_insert_cycling_weight_get) {
  {
    struct _dwg_object_BLOCKVISIBILITYGRIP *arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVISIBILITYGRIP_bg_insert_cycling_weight_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVISIBILITYGRIP_bg_insert_cycling_weight_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *)(argp1);
    result = (BITCODE_BLd) ((arg1)->bg_insert_cycling_weight);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKVISIBILITYGRIP) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKVISIBILITYGRIP *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKVISIBILITYGRIP();");
    }
    result = (struct _dwg_object_BLOCKVISIBILITYGRIP *)calloc(1, sizeof(struct _dwg_object_BLOCKVISIBILITYGRIP));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKVISIBILITYGRIP, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKVISIBILITYGRIP) {
  {
    struct _dwg_object_BLOCKVISIBILITYGRIP *arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKVISIBILITYGRIP(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYGRIP, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKVISIBILITYGRIP" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVISIBILITYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVISIBILITYGRIP *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_parent_set) {
  {
    struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *arg1 = (struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKGRIPLOCATIONCOMPONENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_parent_get) {
  {
    struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *arg1 = (struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKGRIPLOCATIONCOMPONENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_evalexpr_set) {
  {
    struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *arg1 = (struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKGRIPLOCATIONCOMPONENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_evalexpr_get) {
  {
    struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *arg1 = (struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKGRIPLOCATIONCOMPONENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_grip_type_set) {
  {
    struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *arg1 = (struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_grip_type_set(self,grip_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKGRIPLOCATIONCOMPONENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_grip_type_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_grip_type_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->grip_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_grip_type_get) {
  {
    struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *arg1 = (struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_grip_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKGRIPLOCATIONCOMPONENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_grip_type_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *)(argp1);
    result = (BITCODE_BL) ((arg1)->grip_type);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_grip_expr_set) {
  {
    struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *arg1 = (struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_grip_expr_set(self,grip_expr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKGRIPLOCATIONCOMPONENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_grip_expr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_grip_expr_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->grip_expr);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->grip_expr = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->grip_expr = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_grip_expr_get) {
  {
    struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *arg1 = (struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_grip_expr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKGRIPLOCATIONCOMPONENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_grip_expr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *)(argp1);
    result = (BITCODE_TV) ((arg1)->grip_expr);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT();");
    }
    result = (struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *)calloc(1, sizeof(struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKGRIPLOCATIONCOMPONENT, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT) {
  {
    struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *arg1 = (struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKGRIPLOCATIONCOMPONENT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT" "', argument " "1"" of type '" "struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BREAKDATA_parent_set) {
  {
    struct _dwg_object_BREAKDATA *arg1 = (struct _dwg_object_BREAKDATA *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BREAKDATA_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BREAKDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BREAKDATA_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BREAKDATA *""'"); 
    }
    arg1 = (struct _dwg_object_BREAKDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BREAKDATA_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BREAKDATA_parent_get) {
  {
    struct _dwg_object_BREAKDATA *arg1 = (struct _dwg_object_BREAKDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BREAKDATA_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BREAKDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BREAKDATA_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BREAKDATA *""'"); 
    }
    arg1 = (struct _dwg_object_BREAKDATA *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BREAKDATA_num_pointrefs_set) {
  {
    struct _dwg_object_BREAKDATA *arg1 = (struct _dwg_object_BREAKDATA *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BREAKDATA_num_pointrefs_set(self,num_pointrefs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BREAKDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BREAKDATA_num_pointrefs_set" "', argument " "1"" of type '" "struct _dwg_object_BREAKDATA *""'"); 
    }
    arg1 = (struct _dwg_object_BREAKDATA *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BREAKDATA_num_pointrefs_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_pointrefs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BREAKDATA_num_pointrefs_get) {
  {
    struct _dwg_object_BREAKDATA *arg1 = (struct _dwg_object_BREAKDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BREAKDATA_num_pointrefs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BREAKDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BREAKDATA_num_pointrefs_get" "', argument " "1"" of type '" "struct _dwg_object_BREAKDATA *""'"); 
    }
    arg1 = (struct _dwg_object_BREAKDATA *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_pointrefs);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BREAKDATA_pointrefs_set) {
  {
    struct _dwg_object_BREAKDATA *arg1 = (struct _dwg_object_BREAKDATA *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BREAKDATA_pointrefs_set(self,pointrefs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BREAKDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BREAKDATA_pointrefs_set" "', argument " "1"" of type '" "struct _dwg_object_BREAKDATA *""'"); 
    }
    arg1 = (struct _dwg_object_BREAKDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BREAKDATA_pointrefs_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->pointrefs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BREAKDATA_pointrefs_get) {
  {
    struct _dwg_object_BREAKDATA *arg1 = (struct _dwg_object_BREAKDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BREAKDATA_pointrefs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BREAKDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BREAKDATA_pointrefs_get" "', argument " "1"" of type '" "struct _dwg_object_BREAKDATA *""'"); 
    }
    arg1 = (struct _dwg_object_BREAKDATA *)(argp1);
    result = (BITCODE_H *) ((arg1)->pointrefs);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BREAKDATA_dimref_set) {
  {
    struct _dwg_object_BREAKDATA *arg1 = (struct _dwg_object_BREAKDATA *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BREAKDATA_dimref_set(self,dimref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BREAKDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BREAKDATA_dimref_set" "', argument " "1"" of type '" "struct _dwg_object_BREAKDATA *""'"); 
    }
    arg1 = (struct _dwg_object_BREAKDATA *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BREAKDATA_dimref_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->dimref = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BREAKDATA_dimref_get) {
  {
    struct _dwg_object_BREAKDATA *arg1 = (struct _dwg_object_BREAKDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BREAKDATA_dimref_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BREAKDATA, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BREAKDATA_dimref_get" "', argument " "1"" of type '" "struct _dwg_object_BREAKDATA *""'"); 
    }
    arg1 = (struct _dwg_object_BREAKDATA *)(argp1);
    result = (BITCODE_H) ((arg1)->dimref);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BREAKDATA) {
  {
    int argvi = 0;
    struct _dwg_object_BREAKDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BREAKDATA();");
    }
    result = (struct _dwg_object_BREAKDATA *)calloc(1, sizeof(struct _dwg_object_BREAKDATA));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BREAKDATA, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BREAKDATA) {
  {
    struct _dwg_object_BREAKDATA *arg1 = (struct _dwg_object_BREAKDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BREAKDATA(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BREAKDATA, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BREAKDATA" "', argument " "1"" of type '" "struct _dwg_object_BREAKDATA *""'"); 
    }
    arg1 = (struct _dwg_object_BREAKDATA *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BREAKPOINTREF_parent_set) {
  {
    struct _dwg_object_BREAKPOINTREF *arg1 = (struct _dwg_object_BREAKPOINTREF *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BREAKPOINTREF_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BREAKPOINTREF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BREAKPOINTREF_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BREAKPOINTREF *""'"); 
    }
    arg1 = (struct _dwg_object_BREAKPOINTREF *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BREAKPOINTREF_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BREAKPOINTREF_parent_get) {
  {
    struct _dwg_object_BREAKPOINTREF *arg1 = (struct _dwg_object_BREAKPOINTREF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BREAKPOINTREF_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BREAKPOINTREF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BREAKPOINTREF_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BREAKPOINTREF *""'"); 
    }
    arg1 = (struct _dwg_object_BREAKPOINTREF *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BREAKPOINTREF) {
  {
    int argvi = 0;
    struct _dwg_object_BREAKPOINTREF *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BREAKPOINTREF();");
    }
    result = (struct _dwg_object_BREAKPOINTREF *)calloc(1, sizeof(struct _dwg_object_BREAKPOINTREF));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BREAKPOINTREF, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BREAKPOINTREF) {
  {
    struct _dwg_object_BREAKPOINTREF *arg1 = (struct _dwg_object_BREAKPOINTREF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BREAKPOINTREF(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BREAKPOINTREF, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BREAKPOINTREF" "', argument " "1"" of type '" "struct _dwg_object_BREAKPOINTREF *""'"); 
    }
    arg1 = (struct _dwg_object_BREAKPOINTREF *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_FLIPGRIPENTITY_parent_set) {
  {
    struct _dwg_entity_FLIPGRIPENTITY *arg1 = (struct _dwg_entity_FLIPGRIPENTITY *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_FLIPGRIPENTITY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_FLIPGRIPENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_FLIPGRIPENTITY_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_FLIPGRIPENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_FLIPGRIPENTITY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_FLIPGRIPENTITY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_FLIPGRIPENTITY_parent_get) {
  {
    struct _dwg_entity_FLIPGRIPENTITY *arg1 = (struct _dwg_entity_FLIPGRIPENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_FLIPGRIPENTITY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_FLIPGRIPENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_FLIPGRIPENTITY_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_FLIPGRIPENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_FLIPGRIPENTITY *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_FLIPGRIPENTITY) {
  {
    int argvi = 0;
    struct _dwg_entity_FLIPGRIPENTITY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_FLIPGRIPENTITY();");
    }
    result = (struct _dwg_entity_FLIPGRIPENTITY *)calloc(1, sizeof(struct _dwg_entity_FLIPGRIPENTITY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_FLIPGRIPENTITY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_FLIPGRIPENTITY) {
  {
    struct _dwg_entity_FLIPGRIPENTITY *arg1 = (struct _dwg_entity_FLIPGRIPENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_FLIPGRIPENTITY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_FLIPGRIPENTITY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_FLIPGRIPENTITY" "', argument " "1"" of type '" "struct _dwg_entity_FLIPGRIPENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_FLIPGRIPENTITY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LINEARGRIPENTITY_parent_set) {
  {
    struct _dwg_entity_LINEARGRIPENTITY *arg1 = (struct _dwg_entity_LINEARGRIPENTITY *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LINEARGRIPENTITY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LINEARGRIPENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LINEARGRIPENTITY_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_LINEARGRIPENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_LINEARGRIPENTITY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LINEARGRIPENTITY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LINEARGRIPENTITY_parent_get) {
  {
    struct _dwg_entity_LINEARGRIPENTITY *arg1 = (struct _dwg_entity_LINEARGRIPENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LINEARGRIPENTITY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LINEARGRIPENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LINEARGRIPENTITY_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_LINEARGRIPENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_LINEARGRIPENTITY *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_LINEARGRIPENTITY) {
  {
    int argvi = 0;
    struct _dwg_entity_LINEARGRIPENTITY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_LINEARGRIPENTITY();");
    }
    result = (struct _dwg_entity_LINEARGRIPENTITY *)calloc(1, sizeof(struct _dwg_entity_LINEARGRIPENTITY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_LINEARGRIPENTITY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_LINEARGRIPENTITY) {
  {
    struct _dwg_entity_LINEARGRIPENTITY *arg1 = (struct _dwg_entity_LINEARGRIPENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_LINEARGRIPENTITY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LINEARGRIPENTITY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_LINEARGRIPENTITY" "', argument " "1"" of type '" "struct _dwg_entity_LINEARGRIPENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_LINEARGRIPENTITY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLARGRIPENTITY_parent_set) {
  {
    struct _dwg_entity_POLARGRIPENTITY *arg1 = (struct _dwg_entity_POLARGRIPENTITY *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POLARGRIPENTITY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLARGRIPENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLARGRIPENTITY_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_POLARGRIPENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_POLARGRIPENTITY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POLARGRIPENTITY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POLARGRIPENTITY_parent_get) {
  {
    struct _dwg_entity_POLARGRIPENTITY *arg1 = (struct _dwg_entity_POLARGRIPENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POLARGRIPENTITY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLARGRIPENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POLARGRIPENTITY_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_POLARGRIPENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_POLARGRIPENTITY *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_POLARGRIPENTITY) {
  {
    int argvi = 0;
    struct _dwg_entity_POLARGRIPENTITY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_POLARGRIPENTITY();");
    }
    result = (struct _dwg_entity_POLARGRIPENTITY *)calloc(1, sizeof(struct _dwg_entity_POLARGRIPENTITY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_POLARGRIPENTITY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_POLARGRIPENTITY) {
  {
    struct _dwg_entity_POLARGRIPENTITY *arg1 = (struct _dwg_entity_POLARGRIPENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_POLARGRIPENTITY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POLARGRIPENTITY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_POLARGRIPENTITY" "', argument " "1"" of type '" "struct _dwg_entity_POLARGRIPENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_POLARGRIPENTITY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ROTATIONGRIPENTITY_parent_set) {
  {
    struct _dwg_entity_ROTATIONGRIPENTITY *arg1 = (struct _dwg_entity_ROTATIONGRIPENTITY *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ROTATIONGRIPENTITY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ROTATIONGRIPENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ROTATIONGRIPENTITY_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_ROTATIONGRIPENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_ROTATIONGRIPENTITY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ROTATIONGRIPENTITY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ROTATIONGRIPENTITY_parent_get) {
  {
    struct _dwg_entity_ROTATIONGRIPENTITY *arg1 = (struct _dwg_entity_ROTATIONGRIPENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ROTATIONGRIPENTITY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ROTATIONGRIPENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ROTATIONGRIPENTITY_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_ROTATIONGRIPENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_ROTATIONGRIPENTITY *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_ROTATIONGRIPENTITY) {
  {
    int argvi = 0;
    struct _dwg_entity_ROTATIONGRIPENTITY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_ROTATIONGRIPENTITY();");
    }
    result = (struct _dwg_entity_ROTATIONGRIPENTITY *)calloc(1, sizeof(struct _dwg_entity_ROTATIONGRIPENTITY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_ROTATIONGRIPENTITY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_ROTATIONGRIPENTITY) {
  {
    struct _dwg_entity_ROTATIONGRIPENTITY *arg1 = (struct _dwg_entity_ROTATIONGRIPENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_ROTATIONGRIPENTITY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ROTATIONGRIPENTITY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_ROTATIONGRIPENTITY" "', argument " "1"" of type '" "struct _dwg_entity_ROTATIONGRIPENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_ROTATIONGRIPENTITY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VISIBILITYGRIPENTITY_parent_set) {
  {
    struct _dwg_entity_VISIBILITYGRIPENTITY *arg1 = (struct _dwg_entity_VISIBILITYGRIPENTITY *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VISIBILITYGRIPENTITY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VISIBILITYGRIPENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VISIBILITYGRIPENTITY_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_VISIBILITYGRIPENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_VISIBILITYGRIPENTITY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_VISIBILITYGRIPENTITY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VISIBILITYGRIPENTITY_parent_get) {
  {
    struct _dwg_entity_VISIBILITYGRIPENTITY *arg1 = (struct _dwg_entity_VISIBILITYGRIPENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VISIBILITYGRIPENTITY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VISIBILITYGRIPENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VISIBILITYGRIPENTITY_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_VISIBILITYGRIPENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_VISIBILITYGRIPENTITY *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_VISIBILITYGRIPENTITY) {
  {
    int argvi = 0;
    struct _dwg_entity_VISIBILITYGRIPENTITY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_VISIBILITYGRIPENTITY();");
    }
    result = (struct _dwg_entity_VISIBILITYGRIPENTITY *)calloc(1, sizeof(struct _dwg_entity_VISIBILITYGRIPENTITY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_VISIBILITYGRIPENTITY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_VISIBILITYGRIPENTITY) {
  {
    struct _dwg_entity_VISIBILITYGRIPENTITY *arg1 = (struct _dwg_entity_VISIBILITYGRIPENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_VISIBILITYGRIPENTITY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VISIBILITYGRIPENTITY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_VISIBILITYGRIPENTITY" "', argument " "1"" of type '" "struct _dwg_entity_VISIBILITYGRIPENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_VISIBILITYGRIPENTITY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_XYGRIPENTITY_parent_set) {
  {
    struct _dwg_entity_XYGRIPENTITY *arg1 = (struct _dwg_entity_XYGRIPENTITY *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_XYGRIPENTITY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_XYGRIPENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_XYGRIPENTITY_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_XYGRIPENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_XYGRIPENTITY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_XYGRIPENTITY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_XYGRIPENTITY_parent_get) {
  {
    struct _dwg_entity_XYGRIPENTITY *arg1 = (struct _dwg_entity_XYGRIPENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_XYGRIPENTITY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_XYGRIPENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_XYGRIPENTITY_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_XYGRIPENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_XYGRIPENTITY *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_XYGRIPENTITY) {
  {
    int argvi = 0;
    struct _dwg_entity_XYGRIPENTITY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_XYGRIPENTITY();");
    }
    result = (struct _dwg_entity_XYGRIPENTITY *)calloc(1, sizeof(struct _dwg_entity_XYGRIPENTITY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_XYGRIPENTITY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_XYGRIPENTITY) {
  {
    struct _dwg_entity_XYGRIPENTITY *arg1 = (struct _dwg_entity_XYGRIPENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_XYGRIPENTITY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_XYGRIPENTITY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_XYGRIPENTITY" "', argument " "1"" of type '" "struct _dwg_entity_XYGRIPENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_XYGRIPENTITY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ALIGNMENTPARAMETERENTITY_parent_set) {
  {
    struct _dwg_entity_ALIGNMENTPARAMETERENTITY *arg1 = (struct _dwg_entity_ALIGNMENTPARAMETERENTITY *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ALIGNMENTPARAMETERENTITY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ALIGNMENTPARAMETERENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ALIGNMENTPARAMETERENTITY_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_ALIGNMENTPARAMETERENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_ALIGNMENTPARAMETERENTITY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ALIGNMENTPARAMETERENTITY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ALIGNMENTPARAMETERENTITY_parent_get) {
  {
    struct _dwg_entity_ALIGNMENTPARAMETERENTITY *arg1 = (struct _dwg_entity_ALIGNMENTPARAMETERENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ALIGNMENTPARAMETERENTITY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ALIGNMENTPARAMETERENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ALIGNMENTPARAMETERENTITY_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_ALIGNMENTPARAMETERENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_ALIGNMENTPARAMETERENTITY *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_ALIGNMENTPARAMETERENTITY) {
  {
    int argvi = 0;
    struct _dwg_entity_ALIGNMENTPARAMETERENTITY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_ALIGNMENTPARAMETERENTITY();");
    }
    result = (struct _dwg_entity_ALIGNMENTPARAMETERENTITY *)calloc(1, sizeof(struct _dwg_entity_ALIGNMENTPARAMETERENTITY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_ALIGNMENTPARAMETERENTITY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_ALIGNMENTPARAMETERENTITY) {
  {
    struct _dwg_entity_ALIGNMENTPARAMETERENTITY *arg1 = (struct _dwg_entity_ALIGNMENTPARAMETERENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_ALIGNMENTPARAMETERENTITY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ALIGNMENTPARAMETERENTITY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_ALIGNMENTPARAMETERENTITY" "', argument " "1"" of type '" "struct _dwg_entity_ALIGNMENTPARAMETERENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_ALIGNMENTPARAMETERENTITY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_BASEPOINTPARAMETERENTITY_parent_set) {
  {
    struct _dwg_entity_BASEPOINTPARAMETERENTITY *arg1 = (struct _dwg_entity_BASEPOINTPARAMETERENTITY *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_BASEPOINTPARAMETERENTITY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_BASEPOINTPARAMETERENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_BASEPOINTPARAMETERENTITY_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_BASEPOINTPARAMETERENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_BASEPOINTPARAMETERENTITY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_BASEPOINTPARAMETERENTITY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_BASEPOINTPARAMETERENTITY_parent_get) {
  {
    struct _dwg_entity_BASEPOINTPARAMETERENTITY *arg1 = (struct _dwg_entity_BASEPOINTPARAMETERENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_BASEPOINTPARAMETERENTITY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_BASEPOINTPARAMETERENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_BASEPOINTPARAMETERENTITY_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_BASEPOINTPARAMETERENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_BASEPOINTPARAMETERENTITY *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_BASEPOINTPARAMETERENTITY) {
  {
    int argvi = 0;
    struct _dwg_entity_BASEPOINTPARAMETERENTITY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_BASEPOINTPARAMETERENTITY();");
    }
    result = (struct _dwg_entity_BASEPOINTPARAMETERENTITY *)calloc(1, sizeof(struct _dwg_entity_BASEPOINTPARAMETERENTITY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_BASEPOINTPARAMETERENTITY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_BASEPOINTPARAMETERENTITY) {
  {
    struct _dwg_entity_BASEPOINTPARAMETERENTITY *arg1 = (struct _dwg_entity_BASEPOINTPARAMETERENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_BASEPOINTPARAMETERENTITY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_BASEPOINTPARAMETERENTITY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_BASEPOINTPARAMETERENTITY" "', argument " "1"" of type '" "struct _dwg_entity_BASEPOINTPARAMETERENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_BASEPOINTPARAMETERENTITY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_FLIPPARAMETERENTITY_parent_set) {
  {
    struct _dwg_entity_FLIPPARAMETERENTITY *arg1 = (struct _dwg_entity_FLIPPARAMETERENTITY *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_FLIPPARAMETERENTITY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_FLIPPARAMETERENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_FLIPPARAMETERENTITY_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_FLIPPARAMETERENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_FLIPPARAMETERENTITY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_FLIPPARAMETERENTITY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_FLIPPARAMETERENTITY_parent_get) {
  {
    struct _dwg_entity_FLIPPARAMETERENTITY *arg1 = (struct _dwg_entity_FLIPPARAMETERENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_FLIPPARAMETERENTITY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_FLIPPARAMETERENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_FLIPPARAMETERENTITY_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_FLIPPARAMETERENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_FLIPPARAMETERENTITY *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_FLIPPARAMETERENTITY) {
  {
    int argvi = 0;
    struct _dwg_entity_FLIPPARAMETERENTITY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_FLIPPARAMETERENTITY();");
    }
    result = (struct _dwg_entity_FLIPPARAMETERENTITY *)calloc(1, sizeof(struct _dwg_entity_FLIPPARAMETERENTITY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_FLIPPARAMETERENTITY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_FLIPPARAMETERENTITY) {
  {
    struct _dwg_entity_FLIPPARAMETERENTITY *arg1 = (struct _dwg_entity_FLIPPARAMETERENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_FLIPPARAMETERENTITY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_FLIPPARAMETERENTITY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_FLIPPARAMETERENTITY" "', argument " "1"" of type '" "struct _dwg_entity_FLIPPARAMETERENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_FLIPPARAMETERENTITY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LINEARPARAMETERENTITY_parent_set) {
  {
    struct _dwg_entity_LINEARPARAMETERENTITY *arg1 = (struct _dwg_entity_LINEARPARAMETERENTITY *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LINEARPARAMETERENTITY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LINEARPARAMETERENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LINEARPARAMETERENTITY_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_LINEARPARAMETERENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_LINEARPARAMETERENTITY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LINEARPARAMETERENTITY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LINEARPARAMETERENTITY_parent_get) {
  {
    struct _dwg_entity_LINEARPARAMETERENTITY *arg1 = (struct _dwg_entity_LINEARPARAMETERENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LINEARPARAMETERENTITY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LINEARPARAMETERENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LINEARPARAMETERENTITY_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_LINEARPARAMETERENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_LINEARPARAMETERENTITY *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_LINEARPARAMETERENTITY) {
  {
    int argvi = 0;
    struct _dwg_entity_LINEARPARAMETERENTITY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_LINEARPARAMETERENTITY();");
    }
    result = (struct _dwg_entity_LINEARPARAMETERENTITY *)calloc(1, sizeof(struct _dwg_entity_LINEARPARAMETERENTITY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_LINEARPARAMETERENTITY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_LINEARPARAMETERENTITY) {
  {
    struct _dwg_entity_LINEARPARAMETERENTITY *arg1 = (struct _dwg_entity_LINEARPARAMETERENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_LINEARPARAMETERENTITY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LINEARPARAMETERENTITY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_LINEARPARAMETERENTITY" "', argument " "1"" of type '" "struct _dwg_entity_LINEARPARAMETERENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_LINEARPARAMETERENTITY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTPARAMETERENTITY_parent_set) {
  {
    struct _dwg_entity_POINTPARAMETERENTITY *arg1 = (struct _dwg_entity_POINTPARAMETERENTITY *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTPARAMETERENTITY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTPARAMETERENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTPARAMETERENTITY_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTPARAMETERENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTPARAMETERENTITY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POINTPARAMETERENTITY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTPARAMETERENTITY_parent_get) {
  {
    struct _dwg_entity_POINTPARAMETERENTITY *arg1 = (struct _dwg_entity_POINTPARAMETERENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTPARAMETERENTITY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTPARAMETERENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTPARAMETERENTITY_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTPARAMETERENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTPARAMETERENTITY *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_POINTPARAMETERENTITY) {
  {
    int argvi = 0;
    struct _dwg_entity_POINTPARAMETERENTITY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_POINTPARAMETERENTITY();");
    }
    result = (struct _dwg_entity_POINTPARAMETERENTITY *)calloc(1, sizeof(struct _dwg_entity_POINTPARAMETERENTITY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_POINTPARAMETERENTITY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_POINTPARAMETERENTITY) {
  {
    struct _dwg_entity_POINTPARAMETERENTITY *arg1 = (struct _dwg_entity_POINTPARAMETERENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_POINTPARAMETERENTITY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTPARAMETERENTITY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_POINTPARAMETERENTITY" "', argument " "1"" of type '" "struct _dwg_entity_POINTPARAMETERENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTPARAMETERENTITY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ROTATIONPARAMETERENTITY_parent_set) {
  {
    struct _dwg_entity_ROTATIONPARAMETERENTITY *arg1 = (struct _dwg_entity_ROTATIONPARAMETERENTITY *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ROTATIONPARAMETERENTITY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ROTATIONPARAMETERENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ROTATIONPARAMETERENTITY_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_ROTATIONPARAMETERENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_ROTATIONPARAMETERENTITY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ROTATIONPARAMETERENTITY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ROTATIONPARAMETERENTITY_parent_get) {
  {
    struct _dwg_entity_ROTATIONPARAMETERENTITY *arg1 = (struct _dwg_entity_ROTATIONPARAMETERENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ROTATIONPARAMETERENTITY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ROTATIONPARAMETERENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ROTATIONPARAMETERENTITY_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_ROTATIONPARAMETERENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_ROTATIONPARAMETERENTITY *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_ROTATIONPARAMETERENTITY) {
  {
    int argvi = 0;
    struct _dwg_entity_ROTATIONPARAMETERENTITY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_ROTATIONPARAMETERENTITY();");
    }
    result = (struct _dwg_entity_ROTATIONPARAMETERENTITY *)calloc(1, sizeof(struct _dwg_entity_ROTATIONPARAMETERENTITY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_ROTATIONPARAMETERENTITY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_ROTATIONPARAMETERENTITY) {
  {
    struct _dwg_entity_ROTATIONPARAMETERENTITY *arg1 = (struct _dwg_entity_ROTATIONPARAMETERENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_ROTATIONPARAMETERENTITY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ROTATIONPARAMETERENTITY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_ROTATIONPARAMETERENTITY" "', argument " "1"" of type '" "struct _dwg_entity_ROTATIONPARAMETERENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_ROTATIONPARAMETERENTITY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VISIBILITYPARAMETERENTITY_parent_set) {
  {
    struct _dwg_entity_VISIBILITYPARAMETERENTITY *arg1 = (struct _dwg_entity_VISIBILITYPARAMETERENTITY *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_VISIBILITYPARAMETERENTITY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VISIBILITYPARAMETERENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VISIBILITYPARAMETERENTITY_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_VISIBILITYPARAMETERENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_VISIBILITYPARAMETERENTITY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_VISIBILITYPARAMETERENTITY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_VISIBILITYPARAMETERENTITY_parent_get) {
  {
    struct _dwg_entity_VISIBILITYPARAMETERENTITY *arg1 = (struct _dwg_entity_VISIBILITYPARAMETERENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_VISIBILITYPARAMETERENTITY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VISIBILITYPARAMETERENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_VISIBILITYPARAMETERENTITY_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_VISIBILITYPARAMETERENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_VISIBILITYPARAMETERENTITY *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_VISIBILITYPARAMETERENTITY) {
  {
    int argvi = 0;
    struct _dwg_entity_VISIBILITYPARAMETERENTITY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_VISIBILITYPARAMETERENTITY();");
    }
    result = (struct _dwg_entity_VISIBILITYPARAMETERENTITY *)calloc(1, sizeof(struct _dwg_entity_VISIBILITYPARAMETERENTITY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_VISIBILITYPARAMETERENTITY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_VISIBILITYPARAMETERENTITY) {
  {
    struct _dwg_entity_VISIBILITYPARAMETERENTITY *arg1 = (struct _dwg_entity_VISIBILITYPARAMETERENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_VISIBILITYPARAMETERENTITY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_VISIBILITYPARAMETERENTITY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_VISIBILITYPARAMETERENTITY" "', argument " "1"" of type '" "struct _dwg_entity_VISIBILITYPARAMETERENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_VISIBILITYPARAMETERENTITY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_XYPARAMETERENTITY_parent_set) {
  {
    struct _dwg_entity_XYPARAMETERENTITY *arg1 = (struct _dwg_entity_XYPARAMETERENTITY *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_XYPARAMETERENTITY_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_XYPARAMETERENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_XYPARAMETERENTITY_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_XYPARAMETERENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_XYPARAMETERENTITY *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_XYPARAMETERENTITY_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_XYPARAMETERENTITY_parent_get) {
  {
    struct _dwg_entity_XYPARAMETERENTITY *arg1 = (struct _dwg_entity_XYPARAMETERENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_XYPARAMETERENTITY_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_XYPARAMETERENTITY, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_XYPARAMETERENTITY_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_XYPARAMETERENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_XYPARAMETERENTITY *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_XYPARAMETERENTITY) {
  {
    int argvi = 0;
    struct _dwg_entity_XYPARAMETERENTITY *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_XYPARAMETERENTITY();");
    }
    result = (struct _dwg_entity_XYPARAMETERENTITY *)calloc(1, sizeof(struct _dwg_entity_XYPARAMETERENTITY));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_XYPARAMETERENTITY, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_XYPARAMETERENTITY) {
  {
    struct _dwg_entity_XYPARAMETERENTITY *arg1 = (struct _dwg_entity_XYPARAMETERENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_XYPARAMETERENTITY(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_XYPARAMETERENTITY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_XYPARAMETERENTITY" "', argument " "1"" of type '" "struct _dwg_entity_XYPARAMETERENTITY *""'"); 
    }
    arg1 = (struct _dwg_entity_XYPARAMETERENTITY *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTGRIP_parent_set) {
  {
    struct _dwg_object_BLOCKALIGNMENTGRIP *arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTGRIP_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTGRIP_parent_get) {
  {
    struct _dwg_object_BLOCKALIGNMENTGRIP *arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTGRIP_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTGRIP_evalexpr_set) {
  {
    struct _dwg_object_BLOCKALIGNMENTGRIP *arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTGRIP_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTGRIP_evalexpr_get) {
  {
    struct _dwg_object_BLOCKALIGNMENTGRIP *arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTGRIP_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTGRIP_name_set) {
  {
    struct _dwg_object_BLOCKALIGNMENTGRIP *arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTGRIP_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTGRIP_name_get) {
  {
    struct _dwg_object_BLOCKALIGNMENTGRIP *arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTGRIP_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTGRIP_be_major_set) {
  {
    struct _dwg_object_BLOCKALIGNMENTGRIP *arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTGRIP_be_major_set(self,be_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_be_major_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_be_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTGRIP_be_major_get) {
  {
    struct _dwg_object_BLOCKALIGNMENTGRIP *arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTGRIP_be_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_be_major_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTGRIP_be_minor_set) {
  {
    struct _dwg_object_BLOCKALIGNMENTGRIP *arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTGRIP_be_minor_set(self,be_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_be_minor_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_be_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTGRIP_be_minor_get) {
  {
    struct _dwg_object_BLOCKALIGNMENTGRIP *arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTGRIP_be_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_be_minor_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTGRIP_eed1071_set) {
  {
    struct _dwg_object_BLOCKALIGNMENTGRIP *arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTGRIP_eed1071_set(self,eed1071);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_eed1071_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_eed1071_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->eed1071 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTGRIP_eed1071_get) {
  {
    struct _dwg_object_BLOCKALIGNMENTGRIP *arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTGRIP_eed1071_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_eed1071_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->eed1071);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTGRIP_bg_bl91_set) {
  {
    struct _dwg_object_BLOCKALIGNMENTGRIP *arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTGRIP_bg_bl91_set(self,bg_bl91);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_bg_bl91_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_bg_bl91_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bg_bl91 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTGRIP_bg_bl91_get) {
  {
    struct _dwg_object_BLOCKALIGNMENTGRIP *arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTGRIP_bg_bl91_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_bg_bl91_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->bg_bl91);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTGRIP_bg_bl92_set) {
  {
    struct _dwg_object_BLOCKALIGNMENTGRIP *arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTGRIP_bg_bl92_set(self,bg_bl92);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_bg_bl92_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_bg_bl92_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bg_bl92 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTGRIP_bg_bl92_get) {
  {
    struct _dwg_object_BLOCKALIGNMENTGRIP *arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTGRIP_bg_bl92_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_bg_bl92_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->bg_bl92);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTGRIP_bg_location_set) {
  {
    struct _dwg_object_BLOCKALIGNMENTGRIP *arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTGRIP_bg_location_set(self,bg_location);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_bg_location_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_bg_location_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->bg_location = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTGRIP_bg_location_get) {
  {
    struct _dwg_object_BLOCKALIGNMENTGRIP *arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTGRIP_bg_location_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_bg_location_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->bg_location);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTGRIP_bg_insert_cycling_set) {
  {
    struct _dwg_object_BLOCKALIGNMENTGRIP *arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTGRIP_bg_insert_cycling_set(self,bg_insert_cycling);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_bg_insert_cycling_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_bg_insert_cycling_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->bg_insert_cycling = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTGRIP_bg_insert_cycling_get) {
  {
    struct _dwg_object_BLOCKALIGNMENTGRIP *arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTGRIP_bg_insert_cycling_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_bg_insert_cycling_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *)(argp1);
    result = (BITCODE_B) ((arg1)->bg_insert_cycling);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTGRIP_bg_insert_cycling_weight_set) {
  {
    struct _dwg_object_BLOCKALIGNMENTGRIP *arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *) 0 ;
    BITCODE_BLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTGRIP_bg_insert_cycling_weight_set(self,bg_insert_cycling_weight);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_bg_insert_cycling_weight_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_bg_insert_cycling_weight_set" "', argument " "2"" of type '" "BITCODE_BLd""'");
    } 
    arg2 = (BITCODE_BLd)(val2);
    if (arg1) (arg1)->bg_insert_cycling_weight = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTGRIP_bg_insert_cycling_weight_get) {
  {
    struct _dwg_object_BLOCKALIGNMENTGRIP *arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTGRIP_bg_insert_cycling_weight_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_bg_insert_cycling_weight_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *)(argp1);
    result = (BITCODE_BLd) ((arg1)->bg_insert_cycling_weight);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTGRIP_orientation_set) {
  {
    struct _dwg_object_BLOCKALIGNMENTGRIP *arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTGRIP_orientation_set(self,orientation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_orientation_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_orientation_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->orientation = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTGRIP_orientation_get) {
  {
    struct _dwg_object_BLOCKALIGNMENTGRIP *arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTGRIP_orientation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTGRIP_orientation_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->orientation);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKALIGNMENTGRIP) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKALIGNMENTGRIP *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKALIGNMENTGRIP();");
    }
    result = (struct _dwg_object_BLOCKALIGNMENTGRIP *)calloc(1, sizeof(struct _dwg_object_BLOCKALIGNMENTGRIP));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKALIGNMENTGRIP, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKALIGNMENTGRIP) {
  {
    struct _dwg_object_BLOCKALIGNMENTGRIP *arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKALIGNMENTGRIP(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTGRIP, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKALIGNMENTGRIP" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTGRIP *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_parent_set) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_parent_get) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_evalexpr_set) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_evalexpr_get) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_name_set) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_name_get) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_be_major_set) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_be_major_set(self,be_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_be_major_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_be_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_be_major_get) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_be_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_be_major_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_be_minor_set) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_be_minor_set(self,be_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_be_minor_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_be_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_be_minor_get) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_be_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_be_minor_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_eed1071_set) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_eed1071_set(self,eed1071);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_eed1071_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_eed1071_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->eed1071 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_eed1071_get) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_eed1071_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_eed1071_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->eed1071);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_show_properties_set) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_show_properties_set(self,show_properties);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_show_properties_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_show_properties_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->show_properties = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_show_properties_get) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_show_properties_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_show_properties_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->show_properties);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_chain_actions_set) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_chain_actions_set(self,chain_actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_chain_actions_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_chain_actions_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->chain_actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_chain_actions_get) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_chain_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_chain_actions_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->chain_actions);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_def_basept_set) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_def_basept_set(self,def_basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_def_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_def_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_def_basept_get) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_def_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_def_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_def_endpt_set) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_def_endpt_set(self,def_endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_def_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_def_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_def_endpt_get) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_def_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_def_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_prop1_set) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_prop1_set(self,prop1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_prop1_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_prop1_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop1 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_prop1_get) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_prop1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_prop1_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_prop2_set) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_prop2_set(self,prop2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_prop2_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_prop2_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop2 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_prop2_get) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_prop2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_prop2_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_prop3_set) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_prop3_set(self,prop3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_prop3_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_prop3_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop3 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_prop3_get) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_prop3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_prop3_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop3);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_prop4_set) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_prop4_set(self,prop4);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_prop4_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_prop4_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop4 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_prop4_get) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_prop4_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_prop4_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop4);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_prop_states_set) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_prop_states_set(self,prop_states);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_prop_states_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_prop_states_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->prop_states = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_prop_states_get) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_prop_states_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_prop_states_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    result = (BITCODE_BL *) ((arg1)->prop_states);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_parameter_base_location_set) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_parameter_base_location_set(self,parameter_base_location);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_parameter_base_location_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_parameter_base_location_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->parameter_base_location = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_parameter_base_location_get) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_parameter_base_location_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_parameter_base_location_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    result = (BITCODE_BS) ((arg1)->parameter_base_location);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_upd_basept_set) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_upd_basept_set(self,upd_basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_upd_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_upd_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->upd_basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_upd_basept_get) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_upd_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_upd_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->upd_basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_basept_set) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_basept_set(self,basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_basept_get) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_upd_endpt_set) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_upd_endpt_set(self,upd_endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_upd_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_upd_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->upd_endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_upd_endpt_get) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_upd_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_upd_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->upd_endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_endpt_set) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_endpt_set(self,endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_endpt_get) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_align_perpendicular_set) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_align_perpendicular_set(self,align_perpendicular);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_align_perpendicular_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_align_perpendicular_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->align_perpendicular = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_align_perpendicular_get) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNMENTPARAMETER_align_perpendicular_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNMENTPARAMETER_align_perpendicular_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->align_perpendicular);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKALIGNMENTPARAMETER) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKALIGNMENTPARAMETER();");
    }
    result = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)calloc(1, sizeof(struct _dwg_object_BLOCKALIGNMENTPARAMETER));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKALIGNMENTPARAMETER) {
  {
    struct _dwg_object_BLOCKALIGNMENTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKALIGNMENTPARAMETER(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKALIGNMENTPARAMETER" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNMENTPARAMETER *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_parent_set) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_parent_get) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_evalexpr_set) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_evalexpr_get) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_name_set) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_name_get) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_be_major_set) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_be_major_set(self,be_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_be_major_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_be_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_be_major_get) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_be_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_be_major_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_be_minor_set) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_be_minor_set(self,be_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_be_minor_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_be_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_be_minor_get) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_be_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_be_minor_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_eed1071_set) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_eed1071_set(self,eed1071);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_eed1071_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_eed1071_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->eed1071 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_eed1071_get) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_eed1071_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_eed1071_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->eed1071);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_show_properties_set) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_show_properties_set(self,show_properties);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_show_properties_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_show_properties_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->show_properties = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_show_properties_get) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_show_properties_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_show_properties_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->show_properties);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_chain_actions_set) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_chain_actions_set(self,chain_actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_chain_actions_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_chain_actions_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->chain_actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_chain_actions_get) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_chain_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_chain_actions_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->chain_actions);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_def_basept_set) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_def_basept_set(self,def_basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_def_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_def_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_def_basept_get) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_def_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_def_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_def_endpt_set) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_def_endpt_set(self,def_endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_def_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_def_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_def_endpt_get) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_def_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_def_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop1_set) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop1_set(self,prop1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop1_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop1_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop1 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop1_get) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop1_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop2_set) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop2_set(self,prop2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop2_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop2_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop2 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop2_get) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop2_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop3_set) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop3_set(self,prop3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop3_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop3_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop3 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop3_get) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop3_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop3);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop4_set) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop4_set(self,prop4);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop4_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop4_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop4 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop4_get) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop4_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop4_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop4);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop_states_set) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop_states_set(self,prop_states);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop_states_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop_states_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->prop_states = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop_states_get) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop_states_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop_states_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BL *) ((arg1)->prop_states);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_parameter_base_location_set) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_parameter_base_location_set(self,parameter_base_location);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_parameter_base_location_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_parameter_base_location_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->parameter_base_location = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_parameter_base_location_get) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_parameter_base_location_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_parameter_base_location_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BS) ((arg1)->parameter_base_location);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_upd_basept_set) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_upd_basept_set(self,upd_basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_upd_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_upd_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->upd_basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_upd_basept_get) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_upd_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_upd_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->upd_basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_basept_set) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_basept_set(self,basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_basept_get) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_upd_endpt_set) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_upd_endpt_set(self,upd_endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_upd_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_upd_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->upd_endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_upd_endpt_get) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_upd_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_upd_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->upd_endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_endpt_set) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_endpt_set(self,endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_endpt_get) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_dependency_set) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_dependency_set(self,dependency);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_dependency_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_dependency_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->dependency = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_dependency_get) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_dependency_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_dependency_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_H) ((arg1)->dependency);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_center_pt_set) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_center_pt_set(self,center_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_center_pt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_center_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->center_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_center_pt_get) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_center_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_center_pt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->center_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_end_pt_set) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_end_pt_set(self,end_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_end_pt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_end_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->end_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_end_pt_get) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_end_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_end_pt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->end_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_expr_name_set) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_expr_name_set(self,expr_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_expr_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_expr_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->expr_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->expr_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->expr_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_expr_name_get) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_expr_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_expr_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->expr_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_expr_description_set) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_expr_description_set(self,expr_description);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_expr_description_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_expr_description_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->expr_description);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->expr_description = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->expr_description = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_expr_description_get) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_expr_description_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_expr_description_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->expr_description);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_angle_set) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_angle_set(self,angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_angle_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_angle_get) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_angle_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BD) ((arg1)->angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_orientation_on_both_grips_set) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_orientation_on_both_grips_set(self,orientation_on_both_grips);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_orientation_on_both_grips_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_orientation_on_both_grips_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->orientation_on_both_grips = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_orientation_on_both_grips_get) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_orientation_on_both_grips_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_orientation_on_both_grips_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->orientation_on_both_grips);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_value_set_set) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMVALUESET *arg2 = (Dwg_BLOCKPARAMVALUESET *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_value_set_set(self,value_set);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_value_set_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_value_set_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMVALUESET *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMVALUESET *)(argp2);
    if (arg1) (arg1)->value_set = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_value_set_get) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMVALUESET *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_value_set_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_value_set_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMVALUESET *)& ((arg1)->value_set);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER();");
    }
    result = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)calloc(1, sizeof(struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER) {
  {
    struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER" "', argument " "1"" of type '" "struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_parent_set) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_parent_get) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_evalexpr_set) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_evalexpr_get) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_name_set) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_name_get) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_be_major_set) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_be_major_set(self,be_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_be_major_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_be_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_be_major_get) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_be_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_be_major_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_be_minor_set) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_be_minor_set(self,be_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_be_minor_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_be_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_be_minor_get) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_be_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_be_minor_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_eed1071_set) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_eed1071_set(self,eed1071);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_eed1071_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_eed1071_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->eed1071 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_eed1071_get) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_eed1071_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_eed1071_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->eed1071);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_show_properties_set) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_show_properties_set(self,show_properties);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_show_properties_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_show_properties_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->show_properties = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_show_properties_get) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_show_properties_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_show_properties_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->show_properties);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_chain_actions_set) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_chain_actions_set(self,chain_actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_chain_actions_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_chain_actions_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->chain_actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_chain_actions_get) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_chain_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_chain_actions_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->chain_actions);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_def_basept_set) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_def_basept_set(self,def_basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_def_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_def_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_def_basept_get) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_def_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_def_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_def_endpt_set) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_def_endpt_set(self,def_endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_def_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_def_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_def_endpt_get) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_def_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_def_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop1_set) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop1_set(self,prop1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop1_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop1_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop1 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop1_get) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop1_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop2_set) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop2_set(self,prop2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop2_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop2_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop2 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop2_get) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop2_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop3_set) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop3_set(self,prop3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop3_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop3_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop3 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop3_get) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop3_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop3);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop4_set) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop4_set(self,prop4);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop4_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop4_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop4 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop4_get) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop4_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop4_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop4);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop_states_set) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop_states_set(self,prop_states);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop_states_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop_states_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->prop_states = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop_states_get) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop_states_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop_states_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BL *) ((arg1)->prop_states);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_parameter_base_location_set) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_parameter_base_location_set(self,parameter_base_location);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_parameter_base_location_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_parameter_base_location_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->parameter_base_location = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_parameter_base_location_get) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_parameter_base_location_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_parameter_base_location_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BS) ((arg1)->parameter_base_location);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_upd_basept_set) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_upd_basept_set(self,upd_basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_upd_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_upd_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->upd_basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_upd_basept_get) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_upd_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_upd_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->upd_basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_basept_set) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_basept_set(self,basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_basept_get) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_upd_endpt_set) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_upd_endpt_set(self,upd_endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_upd_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_upd_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->upd_endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_upd_endpt_get) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_upd_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_upd_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->upd_endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_endpt_set) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_endpt_set(self,endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_endpt_get) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_dependency_set) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_dependency_set(self,dependency);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_dependency_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_dependency_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->dependency = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_dependency_get) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_dependency_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_dependency_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_H) ((arg1)->dependency);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_expr_name_set) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_expr_name_set(self,expr_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_expr_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_expr_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->expr_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->expr_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->expr_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_expr_name_get) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_expr_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_expr_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->expr_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_expr_description_set) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_expr_description_set(self,expr_description);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_expr_description_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_expr_description_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->expr_description);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->expr_description = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->expr_description = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_expr_description_get) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_expr_description_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_expr_description_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->expr_description);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_distance_set) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_distance_set(self,distance);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_distance_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_distance_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->distance = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_distance_get) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_distance_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_distance_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BD) ((arg1)->distance);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_orientation_on_both_grips_set) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_orientation_on_both_grips_set(self,orientation_on_both_grips);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_orientation_on_both_grips_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_orientation_on_both_grips_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->orientation_on_both_grips = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_orientation_on_both_grips_get) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_orientation_on_both_grips_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_orientation_on_both_grips_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->orientation_on_both_grips);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_value_set_set) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMVALUESET *arg2 = (Dwg_BLOCKPARAMVALUESET *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_value_set_set(self,value_set);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_value_set_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_value_set_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMVALUESET *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMVALUESET *)(argp2);
    if (arg1) (arg1)->value_set = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_value_set_get) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMVALUESET *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_value_set_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_value_set_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMVALUESET *)& ((arg1)->value_set);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER();");
    }
    result = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)calloc(1, sizeof(struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER) {
  {
    struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER" "', argument " "1"" of type '" "struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_parent_set) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_parent_get) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_evalexpr_set) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_evalexpr_get) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_name_set) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_name_get) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_be_major_set) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_be_major_set(self,be_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_be_major_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_be_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_be_major_get) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_be_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_be_major_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_be_minor_set) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_be_minor_set(self,be_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_be_minor_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_be_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_be_minor_get) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_be_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_be_minor_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_eed1071_set) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_eed1071_set(self,eed1071);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_eed1071_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_eed1071_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->eed1071 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_eed1071_get) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_eed1071_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_eed1071_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->eed1071);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_show_properties_set) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_show_properties_set(self,show_properties);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_show_properties_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_show_properties_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->show_properties = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_show_properties_get) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_show_properties_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_show_properties_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->show_properties);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_chain_actions_set) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_chain_actions_set(self,chain_actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_chain_actions_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_chain_actions_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->chain_actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_chain_actions_get) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_chain_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_chain_actions_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->chain_actions);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_def_basept_set) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_def_basept_set(self,def_basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_def_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_def_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_def_basept_get) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_def_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_def_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_def_endpt_set) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_def_endpt_set(self,def_endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_def_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_def_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_def_endpt_get) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_def_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_def_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop1_set) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop1_set(self,prop1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop1_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop1_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop1 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop1_get) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop1_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop2_set) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop2_set(self,prop2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop2_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop2_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop2 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop2_get) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop2_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop3_set) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop3_set(self,prop3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop3_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop3_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop3 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop3_get) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop3_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop3);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop4_set) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop4_set(self,prop4);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop4_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop4_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop4 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop4_get) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop4_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop4_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop4);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop_states_set) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop_states_set(self,prop_states);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop_states_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop_states_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->prop_states = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop_states_get) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop_states_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop_states_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BL *) ((arg1)->prop_states);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_parameter_base_location_set) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_parameter_base_location_set(self,parameter_base_location);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_parameter_base_location_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_parameter_base_location_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->parameter_base_location = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_parameter_base_location_get) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_parameter_base_location_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_parameter_base_location_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BS) ((arg1)->parameter_base_location);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_upd_basept_set) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_upd_basept_set(self,upd_basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_upd_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_upd_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->upd_basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_upd_basept_get) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_upd_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_upd_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->upd_basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_basept_set) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_basept_set(self,basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_basept_get) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_upd_endpt_set) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_upd_endpt_set(self,upd_endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_upd_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_upd_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->upd_endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_upd_endpt_get) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_upd_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_upd_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->upd_endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_endpt_set) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_endpt_set(self,endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_endpt_get) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_dependency_set) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_dependency_set(self,dependency);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_dependency_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_dependency_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->dependency = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_dependency_get) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_dependency_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_dependency_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_H) ((arg1)->dependency);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_expr_name_set) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_expr_name_set(self,expr_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_expr_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_expr_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->expr_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->expr_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->expr_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_expr_name_get) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_expr_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_expr_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->expr_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_expr_description_set) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_expr_description_set(self,expr_description);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_expr_description_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_expr_description_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->expr_description);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->expr_description = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->expr_description = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_expr_description_get) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_expr_description_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_expr_description_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->expr_description);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_distance_set) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_distance_set(self,distance);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_distance_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_distance_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->distance = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_distance_get) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_distance_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_distance_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BD) ((arg1)->distance);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_value_set_set) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMVALUESET *arg2 = (Dwg_BLOCKPARAMVALUESET *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_value_set_set(self,value_set);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_value_set_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_value_set_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMVALUESET *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMVALUESET *)(argp2);
    if (arg1) (arg1)->value_set = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_value_set_get) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMVALUESET *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_value_set_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_value_set_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMVALUESET *)& ((arg1)->value_set);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER();");
    }
    result = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)calloc(1, sizeof(struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER) {
  {
    struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER" "', argument " "1"" of type '" "struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKARRAYACTION_parent_set) {
  {
    struct _dwg_object_BLOCKARRAYACTION *arg1 = (struct _dwg_object_BLOCKARRAYACTION *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKARRAYACTION_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKARRAYACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKARRAYACTION_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKARRAYACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKARRAYACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKARRAYACTION_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKARRAYACTION_parent_get) {
  {
    struct _dwg_object_BLOCKARRAYACTION *arg1 = (struct _dwg_object_BLOCKARRAYACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKARRAYACTION_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKARRAYACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKARRAYACTION_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKARRAYACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKARRAYACTION *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKARRAYACTION_evalexpr_set) {
  {
    struct _dwg_object_BLOCKARRAYACTION *arg1 = (struct _dwg_object_BLOCKARRAYACTION *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKARRAYACTION_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKARRAYACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKARRAYACTION_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKARRAYACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKARRAYACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKARRAYACTION_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKARRAYACTION_evalexpr_get) {
  {
    struct _dwg_object_BLOCKARRAYACTION *arg1 = (struct _dwg_object_BLOCKARRAYACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKARRAYACTION_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKARRAYACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKARRAYACTION_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKARRAYACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKARRAYACTION *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKARRAYACTION_name_set) {
  {
    struct _dwg_object_BLOCKARRAYACTION *arg1 = (struct _dwg_object_BLOCKARRAYACTION *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKARRAYACTION_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKARRAYACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKARRAYACTION_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKARRAYACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKARRAYACTION *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKARRAYACTION_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKARRAYACTION_name_get) {
  {
    struct _dwg_object_BLOCKARRAYACTION *arg1 = (struct _dwg_object_BLOCKARRAYACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKARRAYACTION_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKARRAYACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKARRAYACTION_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKARRAYACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKARRAYACTION *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKARRAYACTION_be_major_set) {
  {
    struct _dwg_object_BLOCKARRAYACTION *arg1 = (struct _dwg_object_BLOCKARRAYACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKARRAYACTION_be_major_set(self,be_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKARRAYACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKARRAYACTION_be_major_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKARRAYACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKARRAYACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKARRAYACTION_be_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKARRAYACTION_be_major_get) {
  {
    struct _dwg_object_BLOCKARRAYACTION *arg1 = (struct _dwg_object_BLOCKARRAYACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKARRAYACTION_be_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKARRAYACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKARRAYACTION_be_major_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKARRAYACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKARRAYACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKARRAYACTION_be_minor_set) {
  {
    struct _dwg_object_BLOCKARRAYACTION *arg1 = (struct _dwg_object_BLOCKARRAYACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKARRAYACTION_be_minor_set(self,be_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKARRAYACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKARRAYACTION_be_minor_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKARRAYACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKARRAYACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKARRAYACTION_be_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKARRAYACTION_be_minor_get) {
  {
    struct _dwg_object_BLOCKARRAYACTION *arg1 = (struct _dwg_object_BLOCKARRAYACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKARRAYACTION_be_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKARRAYACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKARRAYACTION_be_minor_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKARRAYACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKARRAYACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKARRAYACTION_eed1071_set) {
  {
    struct _dwg_object_BLOCKARRAYACTION *arg1 = (struct _dwg_object_BLOCKARRAYACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKARRAYACTION_eed1071_set(self,eed1071);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKARRAYACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKARRAYACTION_eed1071_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKARRAYACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKARRAYACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKARRAYACTION_eed1071_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->eed1071 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKARRAYACTION_eed1071_get) {
  {
    struct _dwg_object_BLOCKARRAYACTION *arg1 = (struct _dwg_object_BLOCKARRAYACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKARRAYACTION_eed1071_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKARRAYACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKARRAYACTION_eed1071_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKARRAYACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKARRAYACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->eed1071);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKARRAYACTION_display_location_set) {
  {
    struct _dwg_object_BLOCKARRAYACTION *arg1 = (struct _dwg_object_BLOCKARRAYACTION *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKARRAYACTION_display_location_set(self,display_location);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKARRAYACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKARRAYACTION_display_location_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKARRAYACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKARRAYACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKARRAYACTION_display_location_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->display_location = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKARRAYACTION_display_location_get) {
  {
    struct _dwg_object_BLOCKARRAYACTION *arg1 = (struct _dwg_object_BLOCKARRAYACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKARRAYACTION_display_location_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKARRAYACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKARRAYACTION_display_location_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKARRAYACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKARRAYACTION *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->display_location);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKARRAYACTION_num_actions_set) {
  {
    struct _dwg_object_BLOCKARRAYACTION *arg1 = (struct _dwg_object_BLOCKARRAYACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKARRAYACTION_num_actions_set(self,num_actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKARRAYACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKARRAYACTION_num_actions_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKARRAYACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKARRAYACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKARRAYACTION_num_actions_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKARRAYACTION_num_actions_get) {
  {
    struct _dwg_object_BLOCKARRAYACTION *arg1 = (struct _dwg_object_BLOCKARRAYACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKARRAYACTION_num_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKARRAYACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKARRAYACTION_num_actions_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKARRAYACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKARRAYACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_actions);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKARRAYACTION_actions_set) {
  {
    struct _dwg_object_BLOCKARRAYACTION *arg1 = (struct _dwg_object_BLOCKARRAYACTION *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKARRAYACTION_actions_set(self,actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKARRAYACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKARRAYACTION_actions_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKARRAYACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKARRAYACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKARRAYACTION_actions_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKARRAYACTION_actions_get) {
  {
    struct _dwg_object_BLOCKARRAYACTION *arg1 = (struct _dwg_object_BLOCKARRAYACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKARRAYACTION_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKARRAYACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKARRAYACTION_actions_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKARRAYACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKARRAYACTION *)(argp1);
    result = (BITCODE_BL *) ((arg1)->actions);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKARRAYACTION_num_deps_set) {
  {
    struct _dwg_object_BLOCKARRAYACTION *arg1 = (struct _dwg_object_BLOCKARRAYACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKARRAYACTION_num_deps_set(self,num_deps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKARRAYACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKARRAYACTION_num_deps_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKARRAYACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKARRAYACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKARRAYACTION_num_deps_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_deps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKARRAYACTION_num_deps_get) {
  {
    struct _dwg_object_BLOCKARRAYACTION *arg1 = (struct _dwg_object_BLOCKARRAYACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKARRAYACTION_num_deps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKARRAYACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKARRAYACTION_num_deps_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKARRAYACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKARRAYACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_deps);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKARRAYACTION_deps_set) {
  {
    struct _dwg_object_BLOCKARRAYACTION *arg1 = (struct _dwg_object_BLOCKARRAYACTION *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKARRAYACTION_deps_set(self,deps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKARRAYACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKARRAYACTION_deps_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKARRAYACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKARRAYACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKARRAYACTION_deps_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->deps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKARRAYACTION_deps_get) {
  {
    struct _dwg_object_BLOCKARRAYACTION *arg1 = (struct _dwg_object_BLOCKARRAYACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKARRAYACTION_deps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKARRAYACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKARRAYACTION_deps_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKARRAYACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKARRAYACTION *)(argp1);
    result = (BITCODE_H *) ((arg1)->deps);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKARRAYACTION_conn_pts_set) {
  {
    struct _dwg_object_BLOCKARRAYACTION *arg1 = (struct _dwg_object_BLOCKARRAYACTION *) 0 ;
    Dwg_BLOCKACTION_connectionpts *arg2 = (Dwg_BLOCKACTION_connectionpts *) (Dwg_BLOCKACTION_connectionpts *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKARRAYACTION_conn_pts_set(self,conn_pts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKARRAYACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKARRAYACTION_conn_pts_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKARRAYACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKARRAYACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKACTION_connectionpts, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKARRAYACTION_conn_pts_set" "', argument " "2"" of type '" "Dwg_BLOCKACTION_connectionpts [4]""'"); 
    } 
    arg2 = (Dwg_BLOCKACTION_connectionpts *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)4; ++ii) *(Dwg_BLOCKACTION_connectionpts *)&arg1->conn_pts[ii] = *((Dwg_BLOCKACTION_connectionpts *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""conn_pts""' of type '""Dwg_BLOCKACTION_connectionpts [4]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKARRAYACTION_conn_pts_get) {
  {
    struct _dwg_object_BLOCKARRAYACTION *arg1 = (struct _dwg_object_BLOCKARRAYACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKACTION_connectionpts *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKARRAYACTION_conn_pts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKARRAYACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKARRAYACTION_conn_pts_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKARRAYACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKARRAYACTION *)(argp1);
    result = (Dwg_BLOCKACTION_connectionpts *)(Dwg_BLOCKACTION_connectionpts *) ((arg1)->conn_pts);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKACTION_connectionpts, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKARRAYACTION_column_offset_set) {
  {
    struct _dwg_object_BLOCKARRAYACTION *arg1 = (struct _dwg_object_BLOCKARRAYACTION *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKARRAYACTION_column_offset_set(self,column_offset);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKARRAYACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKARRAYACTION_column_offset_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKARRAYACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKARRAYACTION *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKARRAYACTION_column_offset_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->column_offset = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKARRAYACTION_column_offset_get) {
  {
    struct _dwg_object_BLOCKARRAYACTION *arg1 = (struct _dwg_object_BLOCKARRAYACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKARRAYACTION_column_offset_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKARRAYACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKARRAYACTION_column_offset_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKARRAYACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKARRAYACTION *)(argp1);
    result = (BITCODE_BD) ((arg1)->column_offset);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKARRAYACTION_row_offset_set) {
  {
    struct _dwg_object_BLOCKARRAYACTION *arg1 = (struct _dwg_object_BLOCKARRAYACTION *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKARRAYACTION_row_offset_set(self,row_offset);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKARRAYACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKARRAYACTION_row_offset_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKARRAYACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKARRAYACTION *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKARRAYACTION_row_offset_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->row_offset = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKARRAYACTION_row_offset_get) {
  {
    struct _dwg_object_BLOCKARRAYACTION *arg1 = (struct _dwg_object_BLOCKARRAYACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKARRAYACTION_row_offset_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKARRAYACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKARRAYACTION_row_offset_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKARRAYACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKARRAYACTION *)(argp1);
    result = (BITCODE_BD) ((arg1)->row_offset);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKARRAYACTION) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKARRAYACTION *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKARRAYACTION();");
    }
    result = (struct _dwg_object_BLOCKARRAYACTION *)calloc(1, sizeof(struct _dwg_object_BLOCKARRAYACTION));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKARRAYACTION, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKARRAYACTION) {
  {
    struct _dwg_object_BLOCKARRAYACTION *arg1 = (struct _dwg_object_BLOCKARRAYACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKARRAYACTION(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKARRAYACTION, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKARRAYACTION" "', argument " "1"" of type '" "struct _dwg_object_BLOCKARRAYACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKARRAYACTION *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_parent_set) {
  {
    struct _dwg_object_BLOCKBASEPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKBASEPOINTPARAMETER_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKBASEPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKBASEPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_parent_get) {
  {
    struct _dwg_object_BLOCKBASEPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKBASEPOINTPARAMETER_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKBASEPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKBASEPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_evalexpr_set) {
  {
    struct _dwg_object_BLOCKBASEPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKBASEPOINTPARAMETER_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKBASEPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKBASEPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_evalexpr_get) {
  {
    struct _dwg_object_BLOCKBASEPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKBASEPOINTPARAMETER_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKBASEPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKBASEPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_name_set) {
  {
    struct _dwg_object_BLOCKBASEPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKBASEPOINTPARAMETER_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKBASEPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKBASEPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_name_get) {
  {
    struct _dwg_object_BLOCKBASEPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKBASEPOINTPARAMETER_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKBASEPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKBASEPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_be_major_set) {
  {
    struct _dwg_object_BLOCKBASEPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKBASEPOINTPARAMETER_be_major_set(self,be_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKBASEPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_be_major_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKBASEPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_be_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_be_major_get) {
  {
    struct _dwg_object_BLOCKBASEPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKBASEPOINTPARAMETER_be_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKBASEPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_be_major_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKBASEPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_be_minor_set) {
  {
    struct _dwg_object_BLOCKBASEPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKBASEPOINTPARAMETER_be_minor_set(self,be_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKBASEPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_be_minor_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKBASEPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_be_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_be_minor_get) {
  {
    struct _dwg_object_BLOCKBASEPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKBASEPOINTPARAMETER_be_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKBASEPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_be_minor_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKBASEPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_eed1071_set) {
  {
    struct _dwg_object_BLOCKBASEPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKBASEPOINTPARAMETER_eed1071_set(self,eed1071);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKBASEPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_eed1071_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKBASEPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_eed1071_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->eed1071 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_eed1071_get) {
  {
    struct _dwg_object_BLOCKBASEPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKBASEPOINTPARAMETER_eed1071_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKBASEPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_eed1071_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKBASEPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->eed1071);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_show_properties_set) {
  {
    struct _dwg_object_BLOCKBASEPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKBASEPOINTPARAMETER_show_properties_set(self,show_properties);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKBASEPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_show_properties_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKBASEPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_show_properties_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->show_properties = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_show_properties_get) {
  {
    struct _dwg_object_BLOCKBASEPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKBASEPOINTPARAMETER_show_properties_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKBASEPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_show_properties_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKBASEPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->show_properties);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_chain_actions_set) {
  {
    struct _dwg_object_BLOCKBASEPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKBASEPOINTPARAMETER_chain_actions_set(self,chain_actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKBASEPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_chain_actions_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKBASEPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_chain_actions_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->chain_actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_chain_actions_get) {
  {
    struct _dwg_object_BLOCKBASEPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKBASEPOINTPARAMETER_chain_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKBASEPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_chain_actions_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKBASEPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->chain_actions);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_def_pt_set) {
  {
    struct _dwg_object_BLOCKBASEPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKBASEPOINTPARAMETER_def_pt_set(self,def_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKBASEPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_def_pt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKBASEPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_def_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_def_pt_get) {
  {
    struct _dwg_object_BLOCKBASEPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKBASEPOINTPARAMETER_def_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKBASEPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_def_pt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKBASEPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_num_propinfos_set) {
  {
    struct _dwg_object_BLOCKBASEPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKBASEPOINTPARAMETER_num_propinfos_set(self,num_propinfos);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKBASEPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_num_propinfos_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKBASEPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_num_propinfos_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_propinfos = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_num_propinfos_get) {
  {
    struct _dwg_object_BLOCKBASEPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKBASEPOINTPARAMETER_num_propinfos_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKBASEPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_num_propinfos_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKBASEPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_propinfos);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_prop1_set) {
  {
    struct _dwg_object_BLOCKBASEPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKBASEPOINTPARAMETER_prop1_set(self,prop1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKBASEPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_prop1_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKBASEPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_prop1_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop1 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_prop1_get) {
  {
    struct _dwg_object_BLOCKBASEPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKBASEPOINTPARAMETER_prop1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKBASEPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_prop1_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKBASEPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_prop2_set) {
  {
    struct _dwg_object_BLOCKBASEPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKBASEPOINTPARAMETER_prop2_set(self,prop2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKBASEPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_prop2_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKBASEPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_prop2_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop2 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_prop2_get) {
  {
    struct _dwg_object_BLOCKBASEPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKBASEPOINTPARAMETER_prop2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKBASEPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_prop2_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKBASEPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_pt_set) {
  {
    struct _dwg_object_BLOCKBASEPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKBASEPOINTPARAMETER_pt_set(self,pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKBASEPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_pt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKBASEPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_pt_get) {
  {
    struct _dwg_object_BLOCKBASEPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKBASEPOINTPARAMETER_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKBASEPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_pt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKBASEPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_base_pt_set) {
  {
    struct _dwg_object_BLOCKBASEPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKBASEPOINTPARAMETER_base_pt_set(self,base_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKBASEPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_base_pt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKBASEPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_base_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->base_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_base_pt_get) {
  {
    struct _dwg_object_BLOCKBASEPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKBASEPOINTPARAMETER_base_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKBASEPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKBASEPOINTPARAMETER_base_pt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKBASEPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->base_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKBASEPOINTPARAMETER) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKBASEPOINTPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKBASEPOINTPARAMETER();");
    }
    result = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *)calloc(1, sizeof(struct _dwg_object_BLOCKBASEPOINTPARAMETER));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKBASEPOINTPARAMETER, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKBASEPOINTPARAMETER) {
  {
    struct _dwg_object_BLOCKBASEPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKBASEPOINTPARAMETER(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKBASEPOINTPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKBASEPOINTPARAMETER" "', argument " "1"" of type '" "struct _dwg_object_BLOCKBASEPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKBASEPOINTPARAMETER *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPACTION_parent_set) {
  {
    struct _dwg_object_BLOCKFLIPACTION *arg1 = (struct _dwg_object_BLOCKFLIPACTION *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPACTION_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPACTION_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKFLIPACTION_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPACTION_parent_get) {
  {
    struct _dwg_object_BLOCKFLIPACTION *arg1 = (struct _dwg_object_BLOCKFLIPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPACTION_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPACTION_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPACTION *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPACTION_evalexpr_set) {
  {
    struct _dwg_object_BLOCKFLIPACTION *arg1 = (struct _dwg_object_BLOCKFLIPACTION *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPACTION_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPACTION_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKFLIPACTION_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPACTION_evalexpr_get) {
  {
    struct _dwg_object_BLOCKFLIPACTION *arg1 = (struct _dwg_object_BLOCKFLIPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPACTION_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPACTION_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPACTION *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPACTION_name_set) {
  {
    struct _dwg_object_BLOCKFLIPACTION *arg1 = (struct _dwg_object_BLOCKFLIPACTION *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPACTION_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPACTION_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPACTION *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKFLIPACTION_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPACTION_name_get) {
  {
    struct _dwg_object_BLOCKFLIPACTION *arg1 = (struct _dwg_object_BLOCKFLIPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPACTION_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPACTION_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPACTION *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPACTION_be_major_set) {
  {
    struct _dwg_object_BLOCKFLIPACTION *arg1 = (struct _dwg_object_BLOCKFLIPACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPACTION_be_major_set(self,be_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPACTION_be_major_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKFLIPACTION_be_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPACTION_be_major_get) {
  {
    struct _dwg_object_BLOCKFLIPACTION *arg1 = (struct _dwg_object_BLOCKFLIPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPACTION_be_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPACTION_be_major_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPACTION_be_minor_set) {
  {
    struct _dwg_object_BLOCKFLIPACTION *arg1 = (struct _dwg_object_BLOCKFLIPACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPACTION_be_minor_set(self,be_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPACTION_be_minor_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKFLIPACTION_be_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPACTION_be_minor_get) {
  {
    struct _dwg_object_BLOCKFLIPACTION *arg1 = (struct _dwg_object_BLOCKFLIPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPACTION_be_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPACTION_be_minor_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPACTION_eed1071_set) {
  {
    struct _dwg_object_BLOCKFLIPACTION *arg1 = (struct _dwg_object_BLOCKFLIPACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPACTION_eed1071_set(self,eed1071);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPACTION_eed1071_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKFLIPACTION_eed1071_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->eed1071 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPACTION_eed1071_get) {
  {
    struct _dwg_object_BLOCKFLIPACTION *arg1 = (struct _dwg_object_BLOCKFLIPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPACTION_eed1071_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPACTION_eed1071_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->eed1071);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPACTION_display_location_set) {
  {
    struct _dwg_object_BLOCKFLIPACTION *arg1 = (struct _dwg_object_BLOCKFLIPACTION *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPACTION_display_location_set(self,display_location);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPACTION_display_location_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKFLIPACTION_display_location_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->display_location = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPACTION_display_location_get) {
  {
    struct _dwg_object_BLOCKFLIPACTION *arg1 = (struct _dwg_object_BLOCKFLIPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPACTION_display_location_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPACTION_display_location_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPACTION *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->display_location);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPACTION_num_actions_set) {
  {
    struct _dwg_object_BLOCKFLIPACTION *arg1 = (struct _dwg_object_BLOCKFLIPACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPACTION_num_actions_set(self,num_actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPACTION_num_actions_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKFLIPACTION_num_actions_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPACTION_num_actions_get) {
  {
    struct _dwg_object_BLOCKFLIPACTION *arg1 = (struct _dwg_object_BLOCKFLIPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPACTION_num_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPACTION_num_actions_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_actions);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPACTION_actions_set) {
  {
    struct _dwg_object_BLOCKFLIPACTION *arg1 = (struct _dwg_object_BLOCKFLIPACTION *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPACTION_actions_set(self,actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPACTION_actions_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKFLIPACTION_actions_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPACTION_actions_get) {
  {
    struct _dwg_object_BLOCKFLIPACTION *arg1 = (struct _dwg_object_BLOCKFLIPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPACTION_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPACTION_actions_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPACTION *)(argp1);
    result = (BITCODE_BL *) ((arg1)->actions);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPACTION_num_deps_set) {
  {
    struct _dwg_object_BLOCKFLIPACTION *arg1 = (struct _dwg_object_BLOCKFLIPACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPACTION_num_deps_set(self,num_deps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPACTION_num_deps_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKFLIPACTION_num_deps_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_deps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPACTION_num_deps_get) {
  {
    struct _dwg_object_BLOCKFLIPACTION *arg1 = (struct _dwg_object_BLOCKFLIPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPACTION_num_deps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPACTION_num_deps_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_deps);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPACTION_deps_set) {
  {
    struct _dwg_object_BLOCKFLIPACTION *arg1 = (struct _dwg_object_BLOCKFLIPACTION *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPACTION_deps_set(self,deps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPACTION_deps_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKFLIPACTION_deps_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->deps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPACTION_deps_get) {
  {
    struct _dwg_object_BLOCKFLIPACTION *arg1 = (struct _dwg_object_BLOCKFLIPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPACTION_deps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPACTION_deps_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPACTION *)(argp1);
    result = (BITCODE_H *) ((arg1)->deps);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPACTION_conn_pts_set) {
  {
    struct _dwg_object_BLOCKFLIPACTION *arg1 = (struct _dwg_object_BLOCKFLIPACTION *) 0 ;
    Dwg_BLOCKACTION_connectionpts *arg2 = (Dwg_BLOCKACTION_connectionpts *) (Dwg_BLOCKACTION_connectionpts *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPACTION_conn_pts_set(self,conn_pts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPACTION_conn_pts_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKACTION_connectionpts, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKFLIPACTION_conn_pts_set" "', argument " "2"" of type '" "Dwg_BLOCKACTION_connectionpts [4]""'"); 
    } 
    arg2 = (Dwg_BLOCKACTION_connectionpts *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)4; ++ii) *(Dwg_BLOCKACTION_connectionpts *)&arg1->conn_pts[ii] = *((Dwg_BLOCKACTION_connectionpts *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""conn_pts""' of type '""Dwg_BLOCKACTION_connectionpts [4]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPACTION_conn_pts_get) {
  {
    struct _dwg_object_BLOCKFLIPACTION *arg1 = (struct _dwg_object_BLOCKFLIPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKACTION_connectionpts *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPACTION_conn_pts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPACTION_conn_pts_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPACTION *)(argp1);
    result = (Dwg_BLOCKACTION_connectionpts *)(Dwg_BLOCKACTION_connectionpts *) ((arg1)->conn_pts);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKACTION_connectionpts, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPACTION_action_offset_x_set) {
  {
    struct _dwg_object_BLOCKFLIPACTION *arg1 = (struct _dwg_object_BLOCKFLIPACTION *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPACTION_action_offset_x_set(self,action_offset_x);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPACTION_action_offset_x_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPACTION *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKFLIPACTION_action_offset_x_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->action_offset_x = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPACTION_action_offset_x_get) {
  {
    struct _dwg_object_BLOCKFLIPACTION *arg1 = (struct _dwg_object_BLOCKFLIPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPACTION_action_offset_x_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPACTION_action_offset_x_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPACTION *)(argp1);
    result = (BITCODE_BD) ((arg1)->action_offset_x);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPACTION_action_offset_y_set) {
  {
    struct _dwg_object_BLOCKFLIPACTION *arg1 = (struct _dwg_object_BLOCKFLIPACTION *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPACTION_action_offset_y_set(self,action_offset_y);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPACTION_action_offset_y_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPACTION *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKFLIPACTION_action_offset_y_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->action_offset_y = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPACTION_action_offset_y_get) {
  {
    struct _dwg_object_BLOCKFLIPACTION *arg1 = (struct _dwg_object_BLOCKFLIPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPACTION_action_offset_y_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPACTION_action_offset_y_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPACTION *)(argp1);
    result = (BITCODE_BD) ((arg1)->action_offset_y);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPACTION_angle_offset_set) {
  {
    struct _dwg_object_BLOCKFLIPACTION *arg1 = (struct _dwg_object_BLOCKFLIPACTION *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPACTION_angle_offset_set(self,angle_offset);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPACTION_angle_offset_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPACTION *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKFLIPACTION_angle_offset_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->angle_offset = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPACTION_angle_offset_get) {
  {
    struct _dwg_object_BLOCKFLIPACTION *arg1 = (struct _dwg_object_BLOCKFLIPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPACTION_angle_offset_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPACTION_angle_offset_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPACTION *)(argp1);
    result = (BITCODE_BD) ((arg1)->angle_offset);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKFLIPACTION) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKFLIPACTION *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKFLIPACTION();");
    }
    result = (struct _dwg_object_BLOCKFLIPACTION *)calloc(1, sizeof(struct _dwg_object_BLOCKFLIPACTION));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKFLIPACTION) {
  {
    struct _dwg_object_BLOCKFLIPACTION *arg1 = (struct _dwg_object_BLOCKFLIPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKFLIPACTION(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKFLIPACTION" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPACTION *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPGRIP_parent_set) {
  {
    struct _dwg_object_BLOCKFLIPGRIP *arg1 = (struct _dwg_object_BLOCKFLIPGRIP *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPGRIP_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPGRIP_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPGRIP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKFLIPGRIP_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPGRIP_parent_get) {
  {
    struct _dwg_object_BLOCKFLIPGRIP *arg1 = (struct _dwg_object_BLOCKFLIPGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPGRIP_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPGRIP_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPGRIP *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPGRIP_evalexpr_set) {
  {
    struct _dwg_object_BLOCKFLIPGRIP *arg1 = (struct _dwg_object_BLOCKFLIPGRIP *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPGRIP_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPGRIP_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPGRIP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKFLIPGRIP_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPGRIP_evalexpr_get) {
  {
    struct _dwg_object_BLOCKFLIPGRIP *arg1 = (struct _dwg_object_BLOCKFLIPGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPGRIP_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPGRIP_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPGRIP *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPGRIP_name_set) {
  {
    struct _dwg_object_BLOCKFLIPGRIP *arg1 = (struct _dwg_object_BLOCKFLIPGRIP *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPGRIP_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPGRIP_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPGRIP *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKFLIPGRIP_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPGRIP_name_get) {
  {
    struct _dwg_object_BLOCKFLIPGRIP *arg1 = (struct _dwg_object_BLOCKFLIPGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPGRIP_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPGRIP_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPGRIP *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPGRIP_be_major_set) {
  {
    struct _dwg_object_BLOCKFLIPGRIP *arg1 = (struct _dwg_object_BLOCKFLIPGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPGRIP_be_major_set(self,be_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPGRIP_be_major_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKFLIPGRIP_be_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPGRIP_be_major_get) {
  {
    struct _dwg_object_BLOCKFLIPGRIP *arg1 = (struct _dwg_object_BLOCKFLIPGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPGRIP_be_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPGRIP_be_major_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPGRIP_be_minor_set) {
  {
    struct _dwg_object_BLOCKFLIPGRIP *arg1 = (struct _dwg_object_BLOCKFLIPGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPGRIP_be_minor_set(self,be_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPGRIP_be_minor_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKFLIPGRIP_be_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPGRIP_be_minor_get) {
  {
    struct _dwg_object_BLOCKFLIPGRIP *arg1 = (struct _dwg_object_BLOCKFLIPGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPGRIP_be_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPGRIP_be_minor_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPGRIP_eed1071_set) {
  {
    struct _dwg_object_BLOCKFLIPGRIP *arg1 = (struct _dwg_object_BLOCKFLIPGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPGRIP_eed1071_set(self,eed1071);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPGRIP_eed1071_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKFLIPGRIP_eed1071_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->eed1071 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPGRIP_eed1071_get) {
  {
    struct _dwg_object_BLOCKFLIPGRIP *arg1 = (struct _dwg_object_BLOCKFLIPGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPGRIP_eed1071_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPGRIP_eed1071_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->eed1071);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPGRIP_bg_bl91_set) {
  {
    struct _dwg_object_BLOCKFLIPGRIP *arg1 = (struct _dwg_object_BLOCKFLIPGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPGRIP_bg_bl91_set(self,bg_bl91);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPGRIP_bg_bl91_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKFLIPGRIP_bg_bl91_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bg_bl91 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPGRIP_bg_bl91_get) {
  {
    struct _dwg_object_BLOCKFLIPGRIP *arg1 = (struct _dwg_object_BLOCKFLIPGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPGRIP_bg_bl91_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPGRIP_bg_bl91_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->bg_bl91);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPGRIP_bg_bl92_set) {
  {
    struct _dwg_object_BLOCKFLIPGRIP *arg1 = (struct _dwg_object_BLOCKFLIPGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPGRIP_bg_bl92_set(self,bg_bl92);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPGRIP_bg_bl92_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKFLIPGRIP_bg_bl92_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bg_bl92 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPGRIP_bg_bl92_get) {
  {
    struct _dwg_object_BLOCKFLIPGRIP *arg1 = (struct _dwg_object_BLOCKFLIPGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPGRIP_bg_bl92_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPGRIP_bg_bl92_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->bg_bl92);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPGRIP_bg_location_set) {
  {
    struct _dwg_object_BLOCKFLIPGRIP *arg1 = (struct _dwg_object_BLOCKFLIPGRIP *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPGRIP_bg_location_set(self,bg_location);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPGRIP_bg_location_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPGRIP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKFLIPGRIP_bg_location_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->bg_location = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPGRIP_bg_location_get) {
  {
    struct _dwg_object_BLOCKFLIPGRIP *arg1 = (struct _dwg_object_BLOCKFLIPGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPGRIP_bg_location_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPGRIP_bg_location_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPGRIP *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->bg_location);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPGRIP_bg_insert_cycling_set) {
  {
    struct _dwg_object_BLOCKFLIPGRIP *arg1 = (struct _dwg_object_BLOCKFLIPGRIP *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPGRIP_bg_insert_cycling_set(self,bg_insert_cycling);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPGRIP_bg_insert_cycling_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKFLIPGRIP_bg_insert_cycling_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->bg_insert_cycling = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPGRIP_bg_insert_cycling_get) {
  {
    struct _dwg_object_BLOCKFLIPGRIP *arg1 = (struct _dwg_object_BLOCKFLIPGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPGRIP_bg_insert_cycling_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPGRIP_bg_insert_cycling_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPGRIP *)(argp1);
    result = (BITCODE_B) ((arg1)->bg_insert_cycling);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPGRIP_bg_insert_cycling_weight_set) {
  {
    struct _dwg_object_BLOCKFLIPGRIP *arg1 = (struct _dwg_object_BLOCKFLIPGRIP *) 0 ;
    BITCODE_BLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPGRIP_bg_insert_cycling_weight_set(self,bg_insert_cycling_weight);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPGRIP_bg_insert_cycling_weight_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPGRIP *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKFLIPGRIP_bg_insert_cycling_weight_set" "', argument " "2"" of type '" "BITCODE_BLd""'");
    } 
    arg2 = (BITCODE_BLd)(val2);
    if (arg1) (arg1)->bg_insert_cycling_weight = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPGRIP_bg_insert_cycling_weight_get) {
  {
    struct _dwg_object_BLOCKFLIPGRIP *arg1 = (struct _dwg_object_BLOCKFLIPGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPGRIP_bg_insert_cycling_weight_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPGRIP_bg_insert_cycling_weight_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPGRIP *)(argp1);
    result = (BITCODE_BLd) ((arg1)->bg_insert_cycling_weight);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPGRIP_combined_state_set) {
  {
    struct _dwg_object_BLOCKFLIPGRIP *arg1 = (struct _dwg_object_BLOCKFLIPGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPGRIP_combined_state_set(self,combined_state);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPGRIP_combined_state_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKFLIPGRIP_combined_state_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->combined_state = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPGRIP_combined_state_get) {
  {
    struct _dwg_object_BLOCKFLIPGRIP *arg1 = (struct _dwg_object_BLOCKFLIPGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPGRIP_combined_state_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPGRIP_combined_state_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->combined_state);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPGRIP_orientation_set) {
  {
    struct _dwg_object_BLOCKFLIPGRIP *arg1 = (struct _dwg_object_BLOCKFLIPGRIP *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPGRIP_orientation_set(self,orientation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPGRIP_orientation_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPGRIP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKFLIPGRIP_orientation_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->orientation = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPGRIP_orientation_get) {
  {
    struct _dwg_object_BLOCKFLIPGRIP *arg1 = (struct _dwg_object_BLOCKFLIPGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPGRIP_orientation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPGRIP_orientation_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPGRIP *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->orientation);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPGRIP_upd_state_set) {
  {
    struct _dwg_object_BLOCKFLIPGRIP *arg1 = (struct _dwg_object_BLOCKFLIPGRIP *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPGRIP_upd_state_set(self,upd_state);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPGRIP_upd_state_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKFLIPGRIP_upd_state_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->upd_state = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPGRIP_upd_state_get) {
  {
    struct _dwg_object_BLOCKFLIPGRIP *arg1 = (struct _dwg_object_BLOCKFLIPGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPGRIP_upd_state_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPGRIP_upd_state_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPGRIP *)(argp1);
    result = (BITCODE_BS) ((arg1)->upd_state);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPGRIP_state_set) {
  {
    struct _dwg_object_BLOCKFLIPGRIP *arg1 = (struct _dwg_object_BLOCKFLIPGRIP *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPGRIP_state_set(self,state);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPGRIP_state_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKFLIPGRIP_state_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->state = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPGRIP_state_get) {
  {
    struct _dwg_object_BLOCKFLIPGRIP *arg1 = (struct _dwg_object_BLOCKFLIPGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPGRIP_state_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPGRIP_state_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPGRIP *)(argp1);
    result = (BITCODE_BS) ((arg1)->state);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKFLIPGRIP) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKFLIPGRIP *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKFLIPGRIP();");
    }
    result = (struct _dwg_object_BLOCKFLIPGRIP *)calloc(1, sizeof(struct _dwg_object_BLOCKFLIPGRIP));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKFLIPGRIP) {
  {
    struct _dwg_object_BLOCKFLIPGRIP *arg1 = (struct _dwg_object_BLOCKFLIPGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKFLIPGRIP(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKFLIPGRIP" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPGRIP *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_parent_set) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_parent_get) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_evalexpr_set) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_evalexpr_get) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_name_set) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_name_get) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_be_major_set) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_be_major_set(self,be_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_be_major_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_be_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_be_major_get) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_be_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_be_major_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_be_minor_set) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_be_minor_set(self,be_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_be_minor_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_be_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_be_minor_get) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_be_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_be_minor_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_eed1071_set) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_eed1071_set(self,eed1071);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_eed1071_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_eed1071_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->eed1071 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_eed1071_get) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_eed1071_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_eed1071_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->eed1071);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_show_properties_set) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_show_properties_set(self,show_properties);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_show_properties_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_show_properties_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->show_properties = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_show_properties_get) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_show_properties_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_show_properties_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->show_properties);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_chain_actions_set) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_chain_actions_set(self,chain_actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_chain_actions_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_chain_actions_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->chain_actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_chain_actions_get) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_chain_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_chain_actions_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->chain_actions);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_def_basept_set) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_def_basept_set(self,def_basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_def_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_def_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_def_basept_get) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_def_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_def_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_def_endpt_set) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_def_endpt_set(self,def_endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_def_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_def_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_def_endpt_get) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_def_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_def_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_prop1_set) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_prop1_set(self,prop1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_prop1_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_prop1_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop1 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_prop1_get) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_prop1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_prop1_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_prop2_set) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_prop2_set(self,prop2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_prop2_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_prop2_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop2 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_prop2_get) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_prop2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_prop2_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_prop3_set) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_prop3_set(self,prop3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_prop3_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_prop3_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop3 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_prop3_get) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_prop3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_prop3_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop3);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_prop4_set) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_prop4_set(self,prop4);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_prop4_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_prop4_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop4 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_prop4_get) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_prop4_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_prop4_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop4);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_prop_states_set) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_prop_states_set(self,prop_states);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_prop_states_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_prop_states_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->prop_states = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_prop_states_get) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_prop_states_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_prop_states_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    result = (BITCODE_BL *) ((arg1)->prop_states);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_parameter_base_location_set) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_parameter_base_location_set(self,parameter_base_location);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_parameter_base_location_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_parameter_base_location_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->parameter_base_location = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_parameter_base_location_get) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_parameter_base_location_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_parameter_base_location_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    result = (BITCODE_BS) ((arg1)->parameter_base_location);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_upd_basept_set) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_upd_basept_set(self,upd_basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_upd_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_upd_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->upd_basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_upd_basept_get) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_upd_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_upd_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->upd_basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_basept_set) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_basept_set(self,basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_basept_get) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_upd_endpt_set) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_upd_endpt_set(self,upd_endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_upd_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_upd_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->upd_endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_upd_endpt_get) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_upd_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_upd_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->upd_endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_endpt_set) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_endpt_set(self,endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_endpt_get) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_flip_label_set) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_flip_label_set(self,flip_label);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_flip_label_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_flip_label_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->flip_label);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->flip_label = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->flip_label = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_flip_label_get) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_flip_label_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_flip_label_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->flip_label);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_flip_label_desc_set) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_flip_label_desc_set(self,flip_label_desc);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_flip_label_desc_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_flip_label_desc_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->flip_label_desc);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->flip_label_desc = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->flip_label_desc = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_flip_label_desc_get) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_flip_label_desc_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_flip_label_desc_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->flip_label_desc);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_base_state_label_set) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_base_state_label_set(self,base_state_label);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_base_state_label_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_base_state_label_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->base_state_label);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->base_state_label = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->base_state_label = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_base_state_label_get) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_base_state_label_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_base_state_label_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->base_state_label);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_flipped_state_label_set) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_flipped_state_label_set(self,flipped_state_label);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_flipped_state_label_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_flipped_state_label_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->flipped_state_label);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->flipped_state_label = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->flipped_state_label = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_flipped_state_label_get) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_flipped_state_label_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_flipped_state_label_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->flipped_state_label);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_def_label_pt_set) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_def_label_pt_set(self,def_label_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_def_label_pt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_def_label_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_label_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_def_label_pt_get) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_def_label_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_def_label_pt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_label_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_bl96_set) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_bl96_set(self,bl96);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_bl96_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_bl96_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bl96 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_bl96_get) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_bl96_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_bl96_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->bl96);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_tooltip_set) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_tooltip_set(self,tooltip);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_tooltip_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_tooltip_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->tooltip);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->tooltip = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->tooltip = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKFLIPPARAMETER_tooltip_get) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKFLIPPARAMETER_tooltip_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKFLIPPARAMETER_tooltip_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->tooltip);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKFLIPPARAMETER) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKFLIPPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKFLIPPARAMETER();");
    }
    result = (struct _dwg_object_BLOCKFLIPPARAMETER *)calloc(1, sizeof(struct _dwg_object_BLOCKFLIPPARAMETER));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKFLIPPARAMETER) {
  {
    struct _dwg_object_BLOCKFLIPPARAMETER *arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKFLIPPARAMETER(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKFLIPPARAMETER" "', argument " "1"" of type '" "struct _dwg_object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKFLIPPARAMETER *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_parent_set) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_parent_get) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_evalexpr_set) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_evalexpr_get) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_name_set) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_name_get) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_be_major_set) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_be_major_set(self,be_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_be_major_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_be_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_be_major_get) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_be_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_be_major_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_be_minor_set) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_be_minor_set(self,be_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_be_minor_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_be_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_be_minor_get) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_be_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_be_minor_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_eed1071_set) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_eed1071_set(self,eed1071);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_eed1071_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_eed1071_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->eed1071 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_eed1071_get) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_eed1071_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_eed1071_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->eed1071);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_show_properties_set) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_show_properties_set(self,show_properties);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_show_properties_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_show_properties_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->show_properties = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_show_properties_get) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_show_properties_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_show_properties_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->show_properties);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_chain_actions_set) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_chain_actions_set(self,chain_actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_chain_actions_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_chain_actions_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->chain_actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_chain_actions_get) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_chain_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_chain_actions_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->chain_actions);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_def_basept_set) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_def_basept_set(self,def_basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_def_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_def_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_def_basept_get) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_def_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_def_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_def_endpt_set) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_def_endpt_set(self,def_endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_def_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_def_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_def_endpt_get) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_def_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_def_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop1_set) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop1_set(self,prop1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop1_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop1_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop1 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop1_get) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop1_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop2_set) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop2_set(self,prop2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop2_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop2_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop2 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop2_get) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop2_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop3_set) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop3_set(self,prop3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop3_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop3_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop3 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop3_get) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop3_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop3);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop4_set) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop4_set(self,prop4);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop4_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop4_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop4 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop4_get) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop4_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop4_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop4);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop_states_set) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop_states_set(self,prop_states);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop_states_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop_states_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->prop_states = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop_states_get) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop_states_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop_states_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BL *) ((arg1)->prop_states);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_parameter_base_location_set) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_parameter_base_location_set(self,parameter_base_location);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_parameter_base_location_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_parameter_base_location_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->parameter_base_location = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_parameter_base_location_get) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_parameter_base_location_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_parameter_base_location_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BS) ((arg1)->parameter_base_location);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_upd_basept_set) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_upd_basept_set(self,upd_basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_upd_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_upd_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->upd_basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_upd_basept_get) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_upd_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_upd_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->upd_basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_basept_set) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_basept_set(self,basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_basept_get) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_upd_endpt_set) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_upd_endpt_set(self,upd_endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_upd_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_upd_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->upd_endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_upd_endpt_get) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_upd_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_upd_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->upd_endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_endpt_set) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_endpt_set(self,endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_endpt_get) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_dependency_set) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_dependency_set(self,dependency);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_dependency_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_dependency_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->dependency = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_dependency_get) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_dependency_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_dependency_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_H) ((arg1)->dependency);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_expr_name_set) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_expr_name_set(self,expr_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_expr_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_expr_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->expr_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->expr_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->expr_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_expr_name_get) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_expr_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_expr_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->expr_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_expr_description_set) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_expr_description_set(self,expr_description);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_expr_description_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_expr_description_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->expr_description);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->expr_description = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->expr_description = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_expr_description_get) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_expr_description_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_expr_description_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->expr_description);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_value_set) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_value_set(self,value);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_value_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_value_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->value = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_value_get) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_value_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_value_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BD) ((arg1)->value);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_value_set_set) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMVALUESET *arg2 = (Dwg_BLOCKPARAMVALUESET *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_value_set_set(self,value_set);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_value_set_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_value_set_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMVALUESET *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMVALUESET *)(argp2);
    if (arg1) (arg1)->value_set = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_value_set_get) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMVALUESET *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_value_set_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_value_set_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMVALUESET *)& ((arg1)->value_set);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER();");
    }
    result = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)calloc(1, sizeof(struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER) {
  {
    struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER" "', argument " "1"" of type '" "struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_parent_set) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_parent_get) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_evalexpr_set) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_evalexpr_get) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_name_set) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_name_get) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_be_major_set) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_be_major_set(self,be_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_be_major_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_be_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_be_major_get) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_be_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_be_major_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_be_minor_set) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_be_minor_set(self,be_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_be_minor_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_be_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_be_minor_get) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_be_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_be_minor_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_eed1071_set) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_eed1071_set(self,eed1071);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_eed1071_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_eed1071_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->eed1071 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_eed1071_get) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_eed1071_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_eed1071_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->eed1071);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_show_properties_set) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_show_properties_set(self,show_properties);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_show_properties_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_show_properties_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->show_properties = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_show_properties_get) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_show_properties_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_show_properties_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->show_properties);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_chain_actions_set) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_chain_actions_set(self,chain_actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_chain_actions_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_chain_actions_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->chain_actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_chain_actions_get) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_chain_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_chain_actions_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->chain_actions);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_def_basept_set) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_def_basept_set(self,def_basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_def_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_def_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_def_basept_get) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_def_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_def_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_def_endpt_set) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_def_endpt_set(self,def_endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_def_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_def_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_def_endpt_get) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_def_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_def_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop1_set) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop1_set(self,prop1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop1_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop1_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop1 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop1_get) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop1_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop2_set) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop2_set(self,prop2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop2_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop2_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop2 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop2_get) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop2_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop3_set) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop3_set(self,prop3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop3_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop3_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop3 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop3_get) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop3_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop3);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop4_set) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop4_set(self,prop4);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop4_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop4_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop4 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop4_get) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop4_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop4_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop4);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop_states_set) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop_states_set(self,prop_states);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop_states_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop_states_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->prop_states = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop_states_get) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop_states_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop_states_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BL *) ((arg1)->prop_states);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_parameter_base_location_set) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_parameter_base_location_set(self,parameter_base_location);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_parameter_base_location_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_parameter_base_location_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->parameter_base_location = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_parameter_base_location_get) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_parameter_base_location_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_parameter_base_location_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BS) ((arg1)->parameter_base_location);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_upd_basept_set) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_upd_basept_set(self,upd_basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_upd_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_upd_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->upd_basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_upd_basept_get) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_upd_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_upd_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->upd_basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_basept_set) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_basept_set(self,basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_basept_get) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_upd_endpt_set) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_upd_endpt_set(self,upd_endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_upd_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_upd_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->upd_endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_upd_endpt_get) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_upd_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_upd_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->upd_endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_endpt_set) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_endpt_set(self,endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_endpt_get) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_dependency_set) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_dependency_set(self,dependency);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_dependency_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_dependency_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->dependency = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_dependency_get) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_dependency_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_dependency_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_H) ((arg1)->dependency);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_expr_name_set) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_expr_name_set(self,expr_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_expr_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_expr_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->expr_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->expr_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->expr_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_expr_name_get) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_expr_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_expr_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->expr_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_expr_description_set) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_expr_description_set(self,expr_description);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_expr_description_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_expr_description_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->expr_description);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->expr_description = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->expr_description = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_expr_description_get) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_expr_description_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_expr_description_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->expr_description);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_value_set) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_value_set(self,value);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_value_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_value_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->value = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_value_get) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_value_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_value_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BD) ((arg1)->value);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_value_set_set) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMVALUESET *arg2 = (Dwg_BLOCKPARAMVALUESET *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_value_set_set(self,value_set);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_value_set_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_value_set_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMVALUESET *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMVALUESET *)(argp2);
    if (arg1) (arg1)->value_set = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_value_set_get) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMVALUESET *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_value_set_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_value_set_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMVALUESET *)& ((arg1)->value_set);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER();");
    }
    result = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)calloc(1, sizeof(struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER) {
  {
    struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_parent_set) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_parent_get) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_evalexpr_set) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_evalexpr_get) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_name_set) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_name_get) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_be_major_set) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_be_major_set(self,be_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_be_major_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_be_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_be_major_get) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_be_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_be_major_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_be_minor_set) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_be_minor_set(self,be_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_be_minor_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_be_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_be_minor_get) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_be_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_be_minor_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_eed1071_set) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_eed1071_set(self,eed1071);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_eed1071_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_eed1071_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->eed1071 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_eed1071_get) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_eed1071_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_eed1071_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->eed1071);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_show_properties_set) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_show_properties_set(self,show_properties);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_show_properties_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_show_properties_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->show_properties = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_show_properties_get) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_show_properties_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_show_properties_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->show_properties);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_chain_actions_set) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_chain_actions_set(self,chain_actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_chain_actions_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_chain_actions_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->chain_actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_chain_actions_get) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_chain_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_chain_actions_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->chain_actions);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_def_basept_set) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_def_basept_set(self,def_basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_def_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_def_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_def_basept_get) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_def_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_def_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_def_endpt_set) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_def_endpt_set(self,def_endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_def_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_def_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_def_endpt_get) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_def_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_def_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop1_set) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop1_set(self,prop1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop1_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop1_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop1 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop1_get) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop1_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop2_set) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop2_set(self,prop2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop2_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop2_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop2 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop2_get) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop2_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop3_set) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop3_set(self,prop3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop3_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop3_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop3 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop3_get) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop3_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop3);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop4_set) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop4_set(self,prop4);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop4_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop4_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop4 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop4_get) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop4_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop4_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop4);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop_states_set) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop_states_set(self,prop_states);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop_states_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop_states_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->prop_states = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop_states_get) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop_states_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop_states_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BL *) ((arg1)->prop_states);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_parameter_base_location_set) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_parameter_base_location_set(self,parameter_base_location);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_parameter_base_location_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_parameter_base_location_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->parameter_base_location = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_parameter_base_location_get) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_parameter_base_location_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_parameter_base_location_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BS) ((arg1)->parameter_base_location);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_upd_basept_set) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_upd_basept_set(self,upd_basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_upd_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_upd_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->upd_basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_upd_basept_get) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_upd_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_upd_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->upd_basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_basept_set) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_basept_set(self,basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_basept_get) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_upd_endpt_set) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_upd_endpt_set(self,upd_endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_upd_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_upd_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->upd_endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_upd_endpt_get) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_upd_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_upd_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->upd_endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_endpt_set) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_endpt_set(self,endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_endpt_get) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_dependency_set) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_dependency_set(self,dependency);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_dependency_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_dependency_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->dependency = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_dependency_get) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_dependency_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_dependency_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_H) ((arg1)->dependency);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_expr_name_set) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_expr_name_set(self,expr_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_expr_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_expr_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->expr_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->expr_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->expr_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_expr_name_get) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_expr_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_expr_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->expr_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_expr_description_set) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_expr_description_set(self,expr_description);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_expr_description_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_expr_description_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->expr_description);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->expr_description = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->expr_description = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_expr_description_get) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_expr_description_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_expr_description_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->expr_description);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_value_set) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_value_set(self,value);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_value_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_value_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->value = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_value_get) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_value_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_value_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BD) ((arg1)->value);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_value_set_set) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMVALUESET *arg2 = (Dwg_BLOCKPARAMVALUESET *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_value_set_set(self,value_set);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_value_set_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_value_set_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMVALUESET *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMVALUESET *)(argp2);
    if (arg1) (arg1)->value_set = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_value_set_get) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMVALUESET *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_value_set_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_value_set_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMVALUESET *)& ((arg1)->value_set);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER();");
    }
    result = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)calloc(1, sizeof(struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER) {
  {
    struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER" "', argument " "1"" of type '" "struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_parent_set) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_parent_get) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_evalexpr_set) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_evalexpr_get) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_name_set) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_name_get) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_be_major_set) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_be_major_set(self,be_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_be_major_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_be_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_be_major_get) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_be_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_be_major_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_be_minor_set) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_be_minor_set(self,be_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_be_minor_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_be_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_be_minor_get) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_be_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_be_minor_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_eed1071_set) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_eed1071_set(self,eed1071);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_eed1071_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_eed1071_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->eed1071 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_eed1071_get) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_eed1071_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_eed1071_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->eed1071);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_show_properties_set) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_show_properties_set(self,show_properties);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_show_properties_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_show_properties_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->show_properties = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_show_properties_get) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_show_properties_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_show_properties_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->show_properties);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_chain_actions_set) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_chain_actions_set(self,chain_actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_chain_actions_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_chain_actions_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->chain_actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_chain_actions_get) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_chain_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_chain_actions_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->chain_actions);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_def_basept_set) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_def_basept_set(self,def_basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_def_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_def_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_def_basept_get) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_def_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_def_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_def_endpt_set) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_def_endpt_set(self,def_endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_def_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_def_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_def_endpt_get) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_def_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_def_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop1_set) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop1_set(self,prop1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop1_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop1_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop1 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop1_get) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop1_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop2_set) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop2_set(self,prop2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop2_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop2_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop2 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop2_get) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop2_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop3_set) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop3_set(self,prop3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop3_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop3_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop3 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop3_get) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop3_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop3);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop4_set) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop4_set(self,prop4);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop4_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop4_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop4 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop4_get) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop4_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop4_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop4);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop_states_set) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop_states_set(self,prop_states);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop_states_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop_states_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->prop_states = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop_states_get) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop_states_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop_states_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BL *) ((arg1)->prop_states);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_parameter_base_location_set) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_parameter_base_location_set(self,parameter_base_location);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_parameter_base_location_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_parameter_base_location_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->parameter_base_location = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_parameter_base_location_get) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_parameter_base_location_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_parameter_base_location_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BS) ((arg1)->parameter_base_location);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_upd_basept_set) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_upd_basept_set(self,upd_basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_upd_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_upd_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->upd_basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_upd_basept_get) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_upd_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_upd_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->upd_basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_basept_set) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_basept_set(self,basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_basept_get) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_upd_endpt_set) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_upd_endpt_set(self,upd_endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_upd_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_upd_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->upd_endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_upd_endpt_get) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_upd_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_upd_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->upd_endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_endpt_set) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_endpt_set(self,endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_endpt_get) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_dependency_set) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_dependency_set(self,dependency);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_dependency_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_dependency_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->dependency = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_dependency_get) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_dependency_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_dependency_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_H) ((arg1)->dependency);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_expr_name_set) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_expr_name_set(self,expr_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_expr_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_expr_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->expr_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->expr_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->expr_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_expr_name_get) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_expr_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_expr_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->expr_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_expr_description_set) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_expr_description_set(self,expr_description);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_expr_description_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_expr_description_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->expr_description);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->expr_description = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->expr_description = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_expr_description_get) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_expr_description_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_expr_description_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->expr_description);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_value_set) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_value_set(self,value);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_value_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_value_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->value = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_value_get) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_value_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_value_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    result = (BITCODE_BD) ((arg1)->value);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_value_set_set) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMVALUESET *arg2 = (Dwg_BLOCKPARAMVALUESET *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_value_set_set(self,value_set);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_value_set_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_value_set_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMVALUESET *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMVALUESET *)(argp2);
    if (arg1) (arg1)->value_set = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_value_set_get) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMVALUESET *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_value_set_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_value_set_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMVALUESET *)& ((arg1)->value_set);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER();");
    }
    result = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)calloc(1, sizeof(struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER) {
  {
    struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER" "', argument " "1"" of type '" "struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARGRIP_parent_set) {
  {
    struct _dwg_object_BLOCKLINEARGRIP *arg1 = (struct _dwg_object_BLOCKLINEARGRIP *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARGRIP_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARGRIP_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARGRIP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARGRIP_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARGRIP_parent_get) {
  {
    struct _dwg_object_BLOCKLINEARGRIP *arg1 = (struct _dwg_object_BLOCKLINEARGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARGRIP_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARGRIP_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARGRIP *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARGRIP_evalexpr_set) {
  {
    struct _dwg_object_BLOCKLINEARGRIP *arg1 = (struct _dwg_object_BLOCKLINEARGRIP *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARGRIP_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARGRIP_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARGRIP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARGRIP_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARGRIP_evalexpr_get) {
  {
    struct _dwg_object_BLOCKLINEARGRIP *arg1 = (struct _dwg_object_BLOCKLINEARGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARGRIP_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARGRIP_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARGRIP *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARGRIP_name_set) {
  {
    struct _dwg_object_BLOCKLINEARGRIP *arg1 = (struct _dwg_object_BLOCKLINEARGRIP *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARGRIP_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARGRIP_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARGRIP *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARGRIP_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARGRIP_name_get) {
  {
    struct _dwg_object_BLOCKLINEARGRIP *arg1 = (struct _dwg_object_BLOCKLINEARGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARGRIP_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARGRIP_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARGRIP *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARGRIP_be_major_set) {
  {
    struct _dwg_object_BLOCKLINEARGRIP *arg1 = (struct _dwg_object_BLOCKLINEARGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARGRIP_be_major_set(self,be_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARGRIP_be_major_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLINEARGRIP_be_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARGRIP_be_major_get) {
  {
    struct _dwg_object_BLOCKLINEARGRIP *arg1 = (struct _dwg_object_BLOCKLINEARGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARGRIP_be_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARGRIP_be_major_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARGRIP_be_minor_set) {
  {
    struct _dwg_object_BLOCKLINEARGRIP *arg1 = (struct _dwg_object_BLOCKLINEARGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARGRIP_be_minor_set(self,be_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARGRIP_be_minor_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLINEARGRIP_be_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARGRIP_be_minor_get) {
  {
    struct _dwg_object_BLOCKLINEARGRIP *arg1 = (struct _dwg_object_BLOCKLINEARGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARGRIP_be_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARGRIP_be_minor_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARGRIP_eed1071_set) {
  {
    struct _dwg_object_BLOCKLINEARGRIP *arg1 = (struct _dwg_object_BLOCKLINEARGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARGRIP_eed1071_set(self,eed1071);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARGRIP_eed1071_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLINEARGRIP_eed1071_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->eed1071 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARGRIP_eed1071_get) {
  {
    struct _dwg_object_BLOCKLINEARGRIP *arg1 = (struct _dwg_object_BLOCKLINEARGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARGRIP_eed1071_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARGRIP_eed1071_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->eed1071);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARGRIP_bg_bl91_set) {
  {
    struct _dwg_object_BLOCKLINEARGRIP *arg1 = (struct _dwg_object_BLOCKLINEARGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARGRIP_bg_bl91_set(self,bg_bl91);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARGRIP_bg_bl91_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLINEARGRIP_bg_bl91_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bg_bl91 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARGRIP_bg_bl91_get) {
  {
    struct _dwg_object_BLOCKLINEARGRIP *arg1 = (struct _dwg_object_BLOCKLINEARGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARGRIP_bg_bl91_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARGRIP_bg_bl91_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->bg_bl91);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARGRIP_bg_bl92_set) {
  {
    struct _dwg_object_BLOCKLINEARGRIP *arg1 = (struct _dwg_object_BLOCKLINEARGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARGRIP_bg_bl92_set(self,bg_bl92);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARGRIP_bg_bl92_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLINEARGRIP_bg_bl92_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bg_bl92 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARGRIP_bg_bl92_get) {
  {
    struct _dwg_object_BLOCKLINEARGRIP *arg1 = (struct _dwg_object_BLOCKLINEARGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARGRIP_bg_bl92_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARGRIP_bg_bl92_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->bg_bl92);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARGRIP_bg_location_set) {
  {
    struct _dwg_object_BLOCKLINEARGRIP *arg1 = (struct _dwg_object_BLOCKLINEARGRIP *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARGRIP_bg_location_set(self,bg_location);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARGRIP_bg_location_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARGRIP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARGRIP_bg_location_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->bg_location = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARGRIP_bg_location_get) {
  {
    struct _dwg_object_BLOCKLINEARGRIP *arg1 = (struct _dwg_object_BLOCKLINEARGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARGRIP_bg_location_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARGRIP_bg_location_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARGRIP *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->bg_location);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARGRIP_bg_insert_cycling_set) {
  {
    struct _dwg_object_BLOCKLINEARGRIP *arg1 = (struct _dwg_object_BLOCKLINEARGRIP *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARGRIP_bg_insert_cycling_set(self,bg_insert_cycling);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARGRIP_bg_insert_cycling_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLINEARGRIP_bg_insert_cycling_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->bg_insert_cycling = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARGRIP_bg_insert_cycling_get) {
  {
    struct _dwg_object_BLOCKLINEARGRIP *arg1 = (struct _dwg_object_BLOCKLINEARGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARGRIP_bg_insert_cycling_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARGRIP_bg_insert_cycling_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARGRIP *)(argp1);
    result = (BITCODE_B) ((arg1)->bg_insert_cycling);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARGRIP_bg_insert_cycling_weight_set) {
  {
    struct _dwg_object_BLOCKLINEARGRIP *arg1 = (struct _dwg_object_BLOCKLINEARGRIP *) 0 ;
    BITCODE_BLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARGRIP_bg_insert_cycling_weight_set(self,bg_insert_cycling_weight);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARGRIP_bg_insert_cycling_weight_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARGRIP *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLINEARGRIP_bg_insert_cycling_weight_set" "', argument " "2"" of type '" "BITCODE_BLd""'");
    } 
    arg2 = (BITCODE_BLd)(val2);
    if (arg1) (arg1)->bg_insert_cycling_weight = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARGRIP_bg_insert_cycling_weight_get) {
  {
    struct _dwg_object_BLOCKLINEARGRIP *arg1 = (struct _dwg_object_BLOCKLINEARGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARGRIP_bg_insert_cycling_weight_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARGRIP_bg_insert_cycling_weight_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARGRIP *)(argp1);
    result = (BITCODE_BLd) ((arg1)->bg_insert_cycling_weight);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARGRIP_orientation_set) {
  {
    struct _dwg_object_BLOCKLINEARGRIP *arg1 = (struct _dwg_object_BLOCKLINEARGRIP *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARGRIP_orientation_set(self,orientation);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARGRIP_orientation_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARGRIP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARGRIP_orientation_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->orientation = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARGRIP_orientation_get) {
  {
    struct _dwg_object_BLOCKLINEARGRIP *arg1 = (struct _dwg_object_BLOCKLINEARGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARGRIP_orientation_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARGRIP_orientation_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARGRIP *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->orientation);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKLINEARGRIP) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKLINEARGRIP *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKLINEARGRIP();");
    }
    result = (struct _dwg_object_BLOCKLINEARGRIP *)calloc(1, sizeof(struct _dwg_object_BLOCKLINEARGRIP));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKLINEARGRIP, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKLINEARGRIP) {
  {
    struct _dwg_object_BLOCKLINEARGRIP *arg1 = (struct _dwg_object_BLOCKLINEARGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKLINEARGRIP(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARGRIP, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKLINEARGRIP" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARGRIP *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_parent_set) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_parent_get) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_evalexpr_set) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_evalexpr_get) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_name_set) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_name_get) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_be_major_set) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_be_major_set(self,be_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_be_major_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_be_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_be_major_get) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_be_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_be_major_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_be_minor_set) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_be_minor_set(self,be_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_be_minor_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_be_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_be_minor_get) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_be_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_be_minor_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_eed1071_set) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_eed1071_set(self,eed1071);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_eed1071_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_eed1071_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->eed1071 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_eed1071_get) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_eed1071_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_eed1071_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->eed1071);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_show_properties_set) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_show_properties_set(self,show_properties);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_show_properties_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_show_properties_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->show_properties = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_show_properties_get) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_show_properties_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_show_properties_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->show_properties);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_chain_actions_set) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_chain_actions_set(self,chain_actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_chain_actions_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_chain_actions_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->chain_actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_chain_actions_get) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_chain_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_chain_actions_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->chain_actions);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_def_basept_set) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_def_basept_set(self,def_basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_def_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_def_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_def_basept_get) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_def_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_def_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_def_endpt_set) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_def_endpt_set(self,def_endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_def_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_def_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_def_endpt_get) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_def_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_def_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_prop1_set) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_prop1_set(self,prop1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_prop1_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_prop1_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop1 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_prop1_get) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_prop1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_prop1_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_prop2_set) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_prop2_set(self,prop2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_prop2_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_prop2_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop2 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_prop2_get) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_prop2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_prop2_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_prop3_set) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_prop3_set(self,prop3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_prop3_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_prop3_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop3 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_prop3_get) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_prop3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_prop3_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop3);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_prop4_set) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_prop4_set(self,prop4);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_prop4_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_prop4_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop4 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_prop4_get) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_prop4_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_prop4_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop4);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_prop_states_set) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_prop_states_set(self,prop_states);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_prop_states_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_prop_states_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->prop_states = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_prop_states_get) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_prop_states_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_prop_states_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    result = (BITCODE_BL *) ((arg1)->prop_states);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_parameter_base_location_set) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_parameter_base_location_set(self,parameter_base_location);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_parameter_base_location_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_parameter_base_location_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->parameter_base_location = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_parameter_base_location_get) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_parameter_base_location_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_parameter_base_location_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    result = (BITCODE_BS) ((arg1)->parameter_base_location);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_upd_basept_set) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_upd_basept_set(self,upd_basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_upd_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_upd_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->upd_basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_upd_basept_get) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_upd_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_upd_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->upd_basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_basept_set) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_basept_set(self,basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_basept_get) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_upd_endpt_set) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_upd_endpt_set(self,upd_endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_upd_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_upd_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->upd_endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_upd_endpt_get) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_upd_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_upd_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->upd_endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_endpt_set) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_endpt_set(self,endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_endpt_get) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_distance_name_set) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_distance_name_set(self,distance_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_distance_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_distance_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->distance_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->distance_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->distance_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_distance_name_get) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_distance_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_distance_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->distance_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_distance_desc_set) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_distance_desc_set(self,distance_desc);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_distance_desc_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_distance_desc_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->distance_desc);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->distance_desc = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->distance_desc = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_distance_desc_get) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_distance_desc_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_distance_desc_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->distance_desc);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_distance_set) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_distance_set(self,distance);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_distance_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_distance_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->distance = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_distance_get) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_distance_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_distance_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    result = (BITCODE_BD) ((arg1)->distance);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_value_set_set) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    Dwg_BLOCKPARAMVALUESET *arg2 = (Dwg_BLOCKPARAMVALUESET *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_value_set_set(self,value_set);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_value_set_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_value_set_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMVALUESET *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMVALUESET *)(argp2);
    if (arg1) (arg1)->value_set = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLINEARPARAMETER_value_set_get) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMVALUESET *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLINEARPARAMETER_value_set_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLINEARPARAMETER_value_set_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMVALUESET *)& ((arg1)->value_set);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKLINEARPARAMETER) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKLINEARPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKLINEARPARAMETER();");
    }
    result = (struct _dwg_object_BLOCKLINEARPARAMETER *)calloc(1, sizeof(struct _dwg_object_BLOCKLINEARPARAMETER));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKLINEARPARAMETER) {
  {
    struct _dwg_object_BLOCKLINEARPARAMETER *arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKLINEARPARAMETER(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKLINEARPARAMETER" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLINEARPARAMETER *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKLOOKUPACTION_lut_parent_set) {
  {
    struct _dwg_BLOCKLOOKUPACTION_lut *arg1 = (struct _dwg_BLOCKLOOKUPACTION_lut *) 0 ;
    struct _dwg_object_BLOCKLOOKUPACTION *arg2 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_BLOCKLOOKUPACTION_lut_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKLOOKUPACTION_lut, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKLOOKUPACTION_lut_parent_set" "', argument " "1"" of type '" "struct _dwg_BLOCKLOOKUPACTION_lut *""'"); 
    }
    arg1 = (struct _dwg_BLOCKLOOKUPACTION_lut *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_BLOCKLOOKUPACTION_lut_parent_set" "', argument " "2"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg2 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKLOOKUPACTION_lut_parent_get) {
  {
    struct _dwg_BLOCKLOOKUPACTION_lut *arg1 = (struct _dwg_BLOCKLOOKUPACTION_lut *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_BLOCKLOOKUPACTION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_BLOCKLOOKUPACTION_lut_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKLOOKUPACTION_lut, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKLOOKUPACTION_lut_parent_get" "', argument " "1"" of type '" "struct _dwg_BLOCKLOOKUPACTION_lut *""'"); 
    }
    arg1 = (struct _dwg_BLOCKLOOKUPACTION_lut *)(argp1);
    result = (struct _dwg_object_BLOCKLOOKUPACTION *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKLOOKUPACTION_lut_conn_pts_set) {
  {
    struct _dwg_BLOCKLOOKUPACTION_lut *arg1 = (struct _dwg_BLOCKLOOKUPACTION_lut *) 0 ;
    Dwg_BLOCKACTION_connectionpts *arg2 = (Dwg_BLOCKACTION_connectionpts *) (Dwg_BLOCKACTION_connectionpts *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_BLOCKLOOKUPACTION_lut_conn_pts_set(self,conn_pts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKLOOKUPACTION_lut, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKLOOKUPACTION_lut_conn_pts_set" "', argument " "1"" of type '" "struct _dwg_BLOCKLOOKUPACTION_lut *""'"); 
    }
    arg1 = (struct _dwg_BLOCKLOOKUPACTION_lut *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKACTION_connectionpts, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_BLOCKLOOKUPACTION_lut_conn_pts_set" "', argument " "2"" of type '" "Dwg_BLOCKACTION_connectionpts [3]""'"); 
    } 
    arg2 = (Dwg_BLOCKACTION_connectionpts *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)3; ++ii) *(Dwg_BLOCKACTION_connectionpts *)&arg1->conn_pts[ii] = *((Dwg_BLOCKACTION_connectionpts *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""conn_pts""' of type '""Dwg_BLOCKACTION_connectionpts [3]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKLOOKUPACTION_lut_conn_pts_get) {
  {
    struct _dwg_BLOCKLOOKUPACTION_lut *arg1 = (struct _dwg_BLOCKLOOKUPACTION_lut *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKACTION_connectionpts *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_BLOCKLOOKUPACTION_lut_conn_pts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKLOOKUPACTION_lut, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKLOOKUPACTION_lut_conn_pts_get" "', argument " "1"" of type '" "struct _dwg_BLOCKLOOKUPACTION_lut *""'"); 
    }
    arg1 = (struct _dwg_BLOCKLOOKUPACTION_lut *)(argp1);
    result = (Dwg_BLOCKACTION_connectionpts *)(Dwg_BLOCKACTION_connectionpts *) ((arg1)->conn_pts);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKACTION_connectionpts, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKLOOKUPACTION_lut_b282_set) {
  {
    struct _dwg_BLOCKLOOKUPACTION_lut *arg1 = (struct _dwg_BLOCKLOOKUPACTION_lut *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_BLOCKLOOKUPACTION_lut_b282_set(self,b282);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKLOOKUPACTION_lut, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKLOOKUPACTION_lut_b282_set" "', argument " "1"" of type '" "struct _dwg_BLOCKLOOKUPACTION_lut *""'"); 
    }
    arg1 = (struct _dwg_BLOCKLOOKUPACTION_lut *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_BLOCKLOOKUPACTION_lut_b282_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->b282 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKLOOKUPACTION_lut_b282_get) {
  {
    struct _dwg_BLOCKLOOKUPACTION_lut *arg1 = (struct _dwg_BLOCKLOOKUPACTION_lut *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_BLOCKLOOKUPACTION_lut_b282_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKLOOKUPACTION_lut, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKLOOKUPACTION_lut_b282_get" "', argument " "1"" of type '" "struct _dwg_BLOCKLOOKUPACTION_lut *""'"); 
    }
    arg1 = (struct _dwg_BLOCKLOOKUPACTION_lut *)(argp1);
    result = (BITCODE_B) ((arg1)->b282);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKLOOKUPACTION_lut_b281_set) {
  {
    struct _dwg_BLOCKLOOKUPACTION_lut *arg1 = (struct _dwg_BLOCKLOOKUPACTION_lut *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_BLOCKLOOKUPACTION_lut_b281_set(self,b281);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKLOOKUPACTION_lut, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKLOOKUPACTION_lut_b281_set" "', argument " "1"" of type '" "struct _dwg_BLOCKLOOKUPACTION_lut *""'"); 
    }
    arg1 = (struct _dwg_BLOCKLOOKUPACTION_lut *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_BLOCKLOOKUPACTION_lut_b281_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->b281 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKLOOKUPACTION_lut_b281_get) {
  {
    struct _dwg_BLOCKLOOKUPACTION_lut *arg1 = (struct _dwg_BLOCKLOOKUPACTION_lut *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_BLOCKLOOKUPACTION_lut_b281_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKLOOKUPACTION_lut, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKLOOKUPACTION_lut_b281_get" "', argument " "1"" of type '" "struct _dwg_BLOCKLOOKUPACTION_lut *""'"); 
    }
    arg1 = (struct _dwg_BLOCKLOOKUPACTION_lut *)(argp1);
    result = (BITCODE_B) ((arg1)->b281);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_BLOCKLOOKUPACTION_lut) {
  {
    int argvi = 0;
    struct _dwg_BLOCKLOOKUPACTION_lut *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_BLOCKLOOKUPACTION_lut();");
    }
    result = (struct _dwg_BLOCKLOOKUPACTION_lut *)calloc(1, sizeof(struct _dwg_BLOCKLOOKUPACTION_lut));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKLOOKUPACTION_lut, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_BLOCKLOOKUPACTION_lut) {
  {
    struct _dwg_BLOCKLOOKUPACTION_lut *arg1 = (struct _dwg_BLOCKLOOKUPACTION_lut *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_BLOCKLOOKUPACTION_lut(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKLOOKUPACTION_lut, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_BLOCKLOOKUPACTION_lut" "', argument " "1"" of type '" "struct _dwg_BLOCKLOOKUPACTION_lut *""'"); 
    }
    arg1 = (struct _dwg_BLOCKLOOKUPACTION_lut *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPACTION_parent_set) {
  {
    struct _dwg_object_BLOCKLOOKUPACTION *arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPACTION_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPACTION_parent_get) {
  {
    struct _dwg_object_BLOCKLOOKUPACTION *arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPACTION_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPACTION_evalexpr_set) {
  {
    struct _dwg_object_BLOCKLOOKUPACTION *arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPACTION_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPACTION_evalexpr_get) {
  {
    struct _dwg_object_BLOCKLOOKUPACTION *arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPACTION_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPACTION_name_set) {
  {
    struct _dwg_object_BLOCKLOOKUPACTION *arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPACTION_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPACTION_name_get) {
  {
    struct _dwg_object_BLOCKLOOKUPACTION *arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPACTION_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPACTION_be_major_set) {
  {
    struct _dwg_object_BLOCKLOOKUPACTION *arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPACTION_be_major_set(self,be_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_be_major_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_be_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPACTION_be_major_get) {
  {
    struct _dwg_object_BLOCKLOOKUPACTION *arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPACTION_be_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_be_major_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPACTION_be_minor_set) {
  {
    struct _dwg_object_BLOCKLOOKUPACTION *arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPACTION_be_minor_set(self,be_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_be_minor_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_be_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPACTION_be_minor_get) {
  {
    struct _dwg_object_BLOCKLOOKUPACTION *arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPACTION_be_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_be_minor_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPACTION_eed1071_set) {
  {
    struct _dwg_object_BLOCKLOOKUPACTION *arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPACTION_eed1071_set(self,eed1071);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_eed1071_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_eed1071_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->eed1071 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPACTION_eed1071_get) {
  {
    struct _dwg_object_BLOCKLOOKUPACTION *arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPACTION_eed1071_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_eed1071_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->eed1071);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPACTION_display_location_set) {
  {
    struct _dwg_object_BLOCKLOOKUPACTION *arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPACTION_display_location_set(self,display_location);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_display_location_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_display_location_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->display_location = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPACTION_display_location_get) {
  {
    struct _dwg_object_BLOCKLOOKUPACTION *arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPACTION_display_location_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_display_location_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->display_location);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPACTION_num_actions_set) {
  {
    struct _dwg_object_BLOCKLOOKUPACTION *arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPACTION_num_actions_set(self,num_actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_num_actions_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_num_actions_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPACTION_num_actions_get) {
  {
    struct _dwg_object_BLOCKLOOKUPACTION *arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPACTION_num_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_num_actions_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_actions);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPACTION_actions_set) {
  {
    struct _dwg_object_BLOCKLOOKUPACTION *arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPACTION_actions_set(self,actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_actions_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_actions_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPACTION_actions_get) {
  {
    struct _dwg_object_BLOCKLOOKUPACTION *arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPACTION_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_actions_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp1);
    result = (BITCODE_BL *) ((arg1)->actions);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPACTION_num_deps_set) {
  {
    struct _dwg_object_BLOCKLOOKUPACTION *arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPACTION_num_deps_set(self,num_deps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_num_deps_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_num_deps_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_deps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPACTION_num_deps_get) {
  {
    struct _dwg_object_BLOCKLOOKUPACTION *arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPACTION_num_deps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_num_deps_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_deps);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPACTION_deps_set) {
  {
    struct _dwg_object_BLOCKLOOKUPACTION *arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPACTION_deps_set(self,deps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_deps_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_deps_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->deps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPACTION_deps_get) {
  {
    struct _dwg_object_BLOCKLOOKUPACTION *arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPACTION_deps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_deps_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp1);
    result = (BITCODE_H *) ((arg1)->deps);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPACTION_numelems_set) {
  {
    struct _dwg_object_BLOCKLOOKUPACTION *arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPACTION_numelems_set(self,numelems);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_numelems_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_numelems_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->numelems = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPACTION_numelems_get) {
  {
    struct _dwg_object_BLOCKLOOKUPACTION *arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPACTION_numelems_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_numelems_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->numelems);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPACTION_numrows_set) {
  {
    struct _dwg_object_BLOCKLOOKUPACTION *arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPACTION_numrows_set(self,numrows);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_numrows_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_numrows_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->numrows = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPACTION_numrows_get) {
  {
    struct _dwg_object_BLOCKLOOKUPACTION *arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPACTION_numrows_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_numrows_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->numrows);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPACTION_numcols_set) {
  {
    struct _dwg_object_BLOCKLOOKUPACTION *arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPACTION_numcols_set(self,numcols);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_numcols_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_numcols_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->numcols = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPACTION_numcols_get) {
  {
    struct _dwg_object_BLOCKLOOKUPACTION *arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPACTION_numcols_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_numcols_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->numcols);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPACTION_lut_set) {
  {
    struct _dwg_object_BLOCKLOOKUPACTION *arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    Dwg_BLOCKLOOKUPACTION_lut *arg2 = (Dwg_BLOCKLOOKUPACTION_lut *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPACTION_lut_set(self,lut);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_lut_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKLOOKUPACTION_lut, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_lut_set" "', argument " "2"" of type '" "Dwg_BLOCKLOOKUPACTION_lut *""'"); 
    }
    arg2 = (Dwg_BLOCKLOOKUPACTION_lut *)(argp2);
    if (arg1) (arg1)->lut = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPACTION_lut_get) {
  {
    struct _dwg_object_BLOCKLOOKUPACTION *arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKLOOKUPACTION_lut *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPACTION_lut_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_lut_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp1);
    result = (Dwg_BLOCKLOOKUPACTION_lut *) ((arg1)->lut);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKLOOKUPACTION_lut, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPACTION_exprs_set) {
  {
    struct _dwg_object_BLOCKLOOKUPACTION *arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    BITCODE_TV *arg2 = (BITCODE_TV *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPACTION_exprs_set(self,exprs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_exprs_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_exprs_set" "', argument " "2"" of type '" "BITCODE_TV *""'"); 
    }
    arg2 = (BITCODE_TV *)(argp2);
    if (arg1) (arg1)->exprs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPACTION_exprs_get) {
  {
    struct _dwg_object_BLOCKLOOKUPACTION *arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPACTION_exprs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_exprs_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp1);
    result = (BITCODE_TV *) ((arg1)->exprs);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPACTION_b280_set) {
  {
    struct _dwg_object_BLOCKLOOKUPACTION *arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPACTION_b280_set(self,b280);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_b280_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_b280_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->b280 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPACTION_b280_get) {
  {
    struct _dwg_object_BLOCKLOOKUPACTION *arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPACTION_b280_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPACTION_b280_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp1);
    result = (BITCODE_B) ((arg1)->b280);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKLOOKUPACTION) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKLOOKUPACTION *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKLOOKUPACTION();");
    }
    result = (struct _dwg_object_BLOCKLOOKUPACTION *)calloc(1, sizeof(struct _dwg_object_BLOCKLOOKUPACTION));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKLOOKUPACTION) {
  {
    struct _dwg_object_BLOCKLOOKUPACTION *arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKLOOKUPACTION(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKLOOKUPACTION" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPACTION *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPGRIP_parent_set) {
  {
    struct _dwg_object_BLOCKLOOKUPGRIP *arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPGRIP_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPGRIP_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLOOKUPGRIP_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPGRIP_parent_get) {
  {
    struct _dwg_object_BLOCKLOOKUPGRIP *arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPGRIP_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPGRIP_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPGRIP_evalexpr_set) {
  {
    struct _dwg_object_BLOCKLOOKUPGRIP *arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPGRIP_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPGRIP_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLOOKUPGRIP_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPGRIP_evalexpr_get) {
  {
    struct _dwg_object_BLOCKLOOKUPGRIP *arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPGRIP_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPGRIP_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPGRIP_name_set) {
  {
    struct _dwg_object_BLOCKLOOKUPGRIP *arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPGRIP_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPGRIP_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLOOKUPGRIP_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPGRIP_name_get) {
  {
    struct _dwg_object_BLOCKLOOKUPGRIP *arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPGRIP_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPGRIP_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPGRIP_be_major_set) {
  {
    struct _dwg_object_BLOCKLOOKUPGRIP *arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPGRIP_be_major_set(self,be_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPGRIP_be_major_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLOOKUPGRIP_be_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPGRIP_be_major_get) {
  {
    struct _dwg_object_BLOCKLOOKUPGRIP *arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPGRIP_be_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPGRIP_be_major_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPGRIP_be_minor_set) {
  {
    struct _dwg_object_BLOCKLOOKUPGRIP *arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPGRIP_be_minor_set(self,be_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPGRIP_be_minor_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLOOKUPGRIP_be_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPGRIP_be_minor_get) {
  {
    struct _dwg_object_BLOCKLOOKUPGRIP *arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPGRIP_be_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPGRIP_be_minor_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPGRIP_eed1071_set) {
  {
    struct _dwg_object_BLOCKLOOKUPGRIP *arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPGRIP_eed1071_set(self,eed1071);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPGRIP_eed1071_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLOOKUPGRIP_eed1071_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->eed1071 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPGRIP_eed1071_get) {
  {
    struct _dwg_object_BLOCKLOOKUPGRIP *arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPGRIP_eed1071_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPGRIP_eed1071_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->eed1071);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPGRIP_bg_bl91_set) {
  {
    struct _dwg_object_BLOCKLOOKUPGRIP *arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPGRIP_bg_bl91_set(self,bg_bl91);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPGRIP_bg_bl91_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLOOKUPGRIP_bg_bl91_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bg_bl91 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPGRIP_bg_bl91_get) {
  {
    struct _dwg_object_BLOCKLOOKUPGRIP *arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPGRIP_bg_bl91_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPGRIP_bg_bl91_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->bg_bl91);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPGRIP_bg_bl92_set) {
  {
    struct _dwg_object_BLOCKLOOKUPGRIP *arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPGRIP_bg_bl92_set(self,bg_bl92);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPGRIP_bg_bl92_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLOOKUPGRIP_bg_bl92_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bg_bl92 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPGRIP_bg_bl92_get) {
  {
    struct _dwg_object_BLOCKLOOKUPGRIP *arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPGRIP_bg_bl92_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPGRIP_bg_bl92_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->bg_bl92);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPGRIP_bg_location_set) {
  {
    struct _dwg_object_BLOCKLOOKUPGRIP *arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPGRIP_bg_location_set(self,bg_location);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPGRIP_bg_location_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLOOKUPGRIP_bg_location_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->bg_location = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPGRIP_bg_location_get) {
  {
    struct _dwg_object_BLOCKLOOKUPGRIP *arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPGRIP_bg_location_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPGRIP_bg_location_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->bg_location);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPGRIP_bg_insert_cycling_set) {
  {
    struct _dwg_object_BLOCKLOOKUPGRIP *arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPGRIP_bg_insert_cycling_set(self,bg_insert_cycling);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPGRIP_bg_insert_cycling_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLOOKUPGRIP_bg_insert_cycling_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->bg_insert_cycling = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPGRIP_bg_insert_cycling_get) {
  {
    struct _dwg_object_BLOCKLOOKUPGRIP *arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPGRIP_bg_insert_cycling_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPGRIP_bg_insert_cycling_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *)(argp1);
    result = (BITCODE_B) ((arg1)->bg_insert_cycling);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPGRIP_bg_insert_cycling_weight_set) {
  {
    struct _dwg_object_BLOCKLOOKUPGRIP *arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *) 0 ;
    BITCODE_BLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPGRIP_bg_insert_cycling_weight_set(self,bg_insert_cycling_weight);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPGRIP_bg_insert_cycling_weight_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLOOKUPGRIP_bg_insert_cycling_weight_set" "', argument " "2"" of type '" "BITCODE_BLd""'");
    } 
    arg2 = (BITCODE_BLd)(val2);
    if (arg1) (arg1)->bg_insert_cycling_weight = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPGRIP_bg_insert_cycling_weight_get) {
  {
    struct _dwg_object_BLOCKLOOKUPGRIP *arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPGRIP_bg_insert_cycling_weight_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPGRIP_bg_insert_cycling_weight_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *)(argp1);
    result = (BITCODE_BLd) ((arg1)->bg_insert_cycling_weight);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKLOOKUPGRIP) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKLOOKUPGRIP *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKLOOKUPGRIP();");
    }
    result = (struct _dwg_object_BLOCKLOOKUPGRIP *)calloc(1, sizeof(struct _dwg_object_BLOCKLOOKUPGRIP));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKLOOKUPGRIP, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKLOOKUPGRIP) {
  {
    struct _dwg_object_BLOCKLOOKUPGRIP *arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKLOOKUPGRIP(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPGRIP, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKLOOKUPGRIP" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPGRIP *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_parent_set) {
  {
    struct _dwg_object_BLOCKLOOKUPPARAMETER *arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPPARAMETER_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_parent_get) {
  {
    struct _dwg_object_BLOCKLOOKUPPARAMETER *arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPPARAMETER_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_evalexpr_set) {
  {
    struct _dwg_object_BLOCKLOOKUPPARAMETER *arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPPARAMETER_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_evalexpr_get) {
  {
    struct _dwg_object_BLOCKLOOKUPPARAMETER *arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPPARAMETER_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_name_set) {
  {
    struct _dwg_object_BLOCKLOOKUPPARAMETER *arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPPARAMETER_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_name_get) {
  {
    struct _dwg_object_BLOCKLOOKUPPARAMETER *arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPPARAMETER_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_be_major_set) {
  {
    struct _dwg_object_BLOCKLOOKUPPARAMETER *arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPPARAMETER_be_major_set(self,be_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_be_major_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_be_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_be_major_get) {
  {
    struct _dwg_object_BLOCKLOOKUPPARAMETER *arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPPARAMETER_be_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_be_major_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_be_minor_set) {
  {
    struct _dwg_object_BLOCKLOOKUPPARAMETER *arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPPARAMETER_be_minor_set(self,be_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_be_minor_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_be_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_be_minor_get) {
  {
    struct _dwg_object_BLOCKLOOKUPPARAMETER *arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPPARAMETER_be_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_be_minor_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_eed1071_set) {
  {
    struct _dwg_object_BLOCKLOOKUPPARAMETER *arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPPARAMETER_eed1071_set(self,eed1071);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_eed1071_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_eed1071_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->eed1071 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_eed1071_get) {
  {
    struct _dwg_object_BLOCKLOOKUPPARAMETER *arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPPARAMETER_eed1071_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_eed1071_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->eed1071);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_show_properties_set) {
  {
    struct _dwg_object_BLOCKLOOKUPPARAMETER *arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPPARAMETER_show_properties_set(self,show_properties);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_show_properties_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_show_properties_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->show_properties = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_show_properties_get) {
  {
    struct _dwg_object_BLOCKLOOKUPPARAMETER *arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPPARAMETER_show_properties_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_show_properties_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->show_properties);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_chain_actions_set) {
  {
    struct _dwg_object_BLOCKLOOKUPPARAMETER *arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPPARAMETER_chain_actions_set(self,chain_actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_chain_actions_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_chain_actions_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->chain_actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_chain_actions_get) {
  {
    struct _dwg_object_BLOCKLOOKUPPARAMETER *arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPPARAMETER_chain_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_chain_actions_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->chain_actions);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_def_pt_set) {
  {
    struct _dwg_object_BLOCKLOOKUPPARAMETER *arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPPARAMETER_def_pt_set(self,def_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_def_pt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_def_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_def_pt_get) {
  {
    struct _dwg_object_BLOCKLOOKUPPARAMETER *arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPPARAMETER_def_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_def_pt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_num_propinfos_set) {
  {
    struct _dwg_object_BLOCKLOOKUPPARAMETER *arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPPARAMETER_num_propinfos_set(self,num_propinfos);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_num_propinfos_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_num_propinfos_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_propinfos = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_num_propinfos_get) {
  {
    struct _dwg_object_BLOCKLOOKUPPARAMETER *arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPPARAMETER_num_propinfos_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_num_propinfos_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_propinfos);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_prop1_set) {
  {
    struct _dwg_object_BLOCKLOOKUPPARAMETER *arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPPARAMETER_prop1_set(self,prop1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_prop1_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_prop1_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop1 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_prop1_get) {
  {
    struct _dwg_object_BLOCKLOOKUPPARAMETER *arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPPARAMETER_prop1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_prop1_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_prop2_set) {
  {
    struct _dwg_object_BLOCKLOOKUPPARAMETER *arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPPARAMETER_prop2_set(self,prop2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_prop2_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_prop2_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop2 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_prop2_get) {
  {
    struct _dwg_object_BLOCKLOOKUPPARAMETER *arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPPARAMETER_prop2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_prop2_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_lookup_name_set) {
  {
    struct _dwg_object_BLOCKLOOKUPPARAMETER *arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPPARAMETER_lookup_name_set(self,lookup_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_lookup_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_lookup_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->lookup_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->lookup_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->lookup_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_lookup_name_get) {
  {
    struct _dwg_object_BLOCKLOOKUPPARAMETER *arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPPARAMETER_lookup_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_lookup_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->lookup_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_lookup_desc_set) {
  {
    struct _dwg_object_BLOCKLOOKUPPARAMETER *arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPPARAMETER_lookup_desc_set(self,lookup_desc);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_lookup_desc_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_lookup_desc_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->lookup_desc);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->lookup_desc = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->lookup_desc = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_lookup_desc_get) {
  {
    struct _dwg_object_BLOCKLOOKUPPARAMETER *arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPPARAMETER_lookup_desc_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_lookup_desc_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->lookup_desc);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_index_set) {
  {
    struct _dwg_object_BLOCKLOOKUPPARAMETER *arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPPARAMETER_index_set(self,index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_index_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_index_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_index_get) {
  {
    struct _dwg_object_BLOCKLOOKUPPARAMETER *arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPPARAMETER_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_index_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->index);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_unknown_t_set) {
  {
    struct _dwg_object_BLOCKLOOKUPPARAMETER *arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPPARAMETER_unknown_t_set(self,unknown_t);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_unknown_t_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_unknown_t_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->unknown_t);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->unknown_t = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->unknown_t = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_unknown_t_get) {
  {
    struct _dwg_object_BLOCKLOOKUPPARAMETER *arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKLOOKUPPARAMETER_unknown_t_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKLOOKUPPARAMETER_unknown_t_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->unknown_t);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKLOOKUPPARAMETER) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKLOOKUPPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKLOOKUPPARAMETER();");
    }
    result = (struct _dwg_object_BLOCKLOOKUPPARAMETER *)calloc(1, sizeof(struct _dwg_object_BLOCKLOOKUPPARAMETER));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKLOOKUPPARAMETER) {
  {
    struct _dwg_object_BLOCKLOOKUPPARAMETER *arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKLOOKUPPARAMETER(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKLOOKUPPARAMETER" "', argument " "1"" of type '" "struct _dwg_object_BLOCKLOOKUPPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKLOOKUPPARAMETER *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKMOVEACTION_parent_set) {
  {
    struct _dwg_object_BLOCKMOVEACTION *arg1 = (struct _dwg_object_BLOCKMOVEACTION *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKMOVEACTION_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKMOVEACTION_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKMOVEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKMOVEACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKMOVEACTION_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKMOVEACTION_parent_get) {
  {
    struct _dwg_object_BLOCKMOVEACTION *arg1 = (struct _dwg_object_BLOCKMOVEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKMOVEACTION_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKMOVEACTION_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKMOVEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKMOVEACTION *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKMOVEACTION_evalexpr_set) {
  {
    struct _dwg_object_BLOCKMOVEACTION *arg1 = (struct _dwg_object_BLOCKMOVEACTION *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKMOVEACTION_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKMOVEACTION_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKMOVEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKMOVEACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKMOVEACTION_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKMOVEACTION_evalexpr_get) {
  {
    struct _dwg_object_BLOCKMOVEACTION *arg1 = (struct _dwg_object_BLOCKMOVEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKMOVEACTION_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKMOVEACTION_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKMOVEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKMOVEACTION *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKMOVEACTION_name_set) {
  {
    struct _dwg_object_BLOCKMOVEACTION *arg1 = (struct _dwg_object_BLOCKMOVEACTION *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKMOVEACTION_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKMOVEACTION_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKMOVEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKMOVEACTION *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKMOVEACTION_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKMOVEACTION_name_get) {
  {
    struct _dwg_object_BLOCKMOVEACTION *arg1 = (struct _dwg_object_BLOCKMOVEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKMOVEACTION_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKMOVEACTION_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKMOVEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKMOVEACTION *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKMOVEACTION_be_major_set) {
  {
    struct _dwg_object_BLOCKMOVEACTION *arg1 = (struct _dwg_object_BLOCKMOVEACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKMOVEACTION_be_major_set(self,be_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKMOVEACTION_be_major_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKMOVEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKMOVEACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKMOVEACTION_be_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKMOVEACTION_be_major_get) {
  {
    struct _dwg_object_BLOCKMOVEACTION *arg1 = (struct _dwg_object_BLOCKMOVEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKMOVEACTION_be_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKMOVEACTION_be_major_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKMOVEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKMOVEACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKMOVEACTION_be_minor_set) {
  {
    struct _dwg_object_BLOCKMOVEACTION *arg1 = (struct _dwg_object_BLOCKMOVEACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKMOVEACTION_be_minor_set(self,be_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKMOVEACTION_be_minor_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKMOVEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKMOVEACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKMOVEACTION_be_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKMOVEACTION_be_minor_get) {
  {
    struct _dwg_object_BLOCKMOVEACTION *arg1 = (struct _dwg_object_BLOCKMOVEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKMOVEACTION_be_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKMOVEACTION_be_minor_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKMOVEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKMOVEACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKMOVEACTION_eed1071_set) {
  {
    struct _dwg_object_BLOCKMOVEACTION *arg1 = (struct _dwg_object_BLOCKMOVEACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKMOVEACTION_eed1071_set(self,eed1071);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKMOVEACTION_eed1071_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKMOVEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKMOVEACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKMOVEACTION_eed1071_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->eed1071 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKMOVEACTION_eed1071_get) {
  {
    struct _dwg_object_BLOCKMOVEACTION *arg1 = (struct _dwg_object_BLOCKMOVEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKMOVEACTION_eed1071_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKMOVEACTION_eed1071_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKMOVEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKMOVEACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->eed1071);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKMOVEACTION_display_location_set) {
  {
    struct _dwg_object_BLOCKMOVEACTION *arg1 = (struct _dwg_object_BLOCKMOVEACTION *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKMOVEACTION_display_location_set(self,display_location);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKMOVEACTION_display_location_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKMOVEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKMOVEACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKMOVEACTION_display_location_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->display_location = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKMOVEACTION_display_location_get) {
  {
    struct _dwg_object_BLOCKMOVEACTION *arg1 = (struct _dwg_object_BLOCKMOVEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKMOVEACTION_display_location_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKMOVEACTION_display_location_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKMOVEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKMOVEACTION *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->display_location);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKMOVEACTION_num_actions_set) {
  {
    struct _dwg_object_BLOCKMOVEACTION *arg1 = (struct _dwg_object_BLOCKMOVEACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKMOVEACTION_num_actions_set(self,num_actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKMOVEACTION_num_actions_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKMOVEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKMOVEACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKMOVEACTION_num_actions_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKMOVEACTION_num_actions_get) {
  {
    struct _dwg_object_BLOCKMOVEACTION *arg1 = (struct _dwg_object_BLOCKMOVEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKMOVEACTION_num_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKMOVEACTION_num_actions_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKMOVEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKMOVEACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_actions);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKMOVEACTION_actions_set) {
  {
    struct _dwg_object_BLOCKMOVEACTION *arg1 = (struct _dwg_object_BLOCKMOVEACTION *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKMOVEACTION_actions_set(self,actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKMOVEACTION_actions_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKMOVEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKMOVEACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKMOVEACTION_actions_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKMOVEACTION_actions_get) {
  {
    struct _dwg_object_BLOCKMOVEACTION *arg1 = (struct _dwg_object_BLOCKMOVEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKMOVEACTION_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKMOVEACTION_actions_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKMOVEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKMOVEACTION *)(argp1);
    result = (BITCODE_BL *) ((arg1)->actions);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKMOVEACTION_num_deps_set) {
  {
    struct _dwg_object_BLOCKMOVEACTION *arg1 = (struct _dwg_object_BLOCKMOVEACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKMOVEACTION_num_deps_set(self,num_deps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKMOVEACTION_num_deps_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKMOVEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKMOVEACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKMOVEACTION_num_deps_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_deps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKMOVEACTION_num_deps_get) {
  {
    struct _dwg_object_BLOCKMOVEACTION *arg1 = (struct _dwg_object_BLOCKMOVEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKMOVEACTION_num_deps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKMOVEACTION_num_deps_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKMOVEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKMOVEACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_deps);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKMOVEACTION_deps_set) {
  {
    struct _dwg_object_BLOCKMOVEACTION *arg1 = (struct _dwg_object_BLOCKMOVEACTION *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKMOVEACTION_deps_set(self,deps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKMOVEACTION_deps_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKMOVEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKMOVEACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKMOVEACTION_deps_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->deps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKMOVEACTION_deps_get) {
  {
    struct _dwg_object_BLOCKMOVEACTION *arg1 = (struct _dwg_object_BLOCKMOVEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKMOVEACTION_deps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKMOVEACTION_deps_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKMOVEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKMOVEACTION *)(argp1);
    result = (BITCODE_H *) ((arg1)->deps);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKMOVEACTION_conn_pts_set) {
  {
    struct _dwg_object_BLOCKMOVEACTION *arg1 = (struct _dwg_object_BLOCKMOVEACTION *) 0 ;
    Dwg_BLOCKACTION_connectionpts *arg2 = (Dwg_BLOCKACTION_connectionpts *) (Dwg_BLOCKACTION_connectionpts *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKMOVEACTION_conn_pts_set(self,conn_pts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKMOVEACTION_conn_pts_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKMOVEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKMOVEACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKACTION_connectionpts, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKMOVEACTION_conn_pts_set" "', argument " "2"" of type '" "Dwg_BLOCKACTION_connectionpts [2]""'"); 
    } 
    arg2 = (Dwg_BLOCKACTION_connectionpts *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)2; ++ii) *(Dwg_BLOCKACTION_connectionpts *)&arg1->conn_pts[ii] = *((Dwg_BLOCKACTION_connectionpts *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""conn_pts""' of type '""Dwg_BLOCKACTION_connectionpts [2]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKMOVEACTION_conn_pts_get) {
  {
    struct _dwg_object_BLOCKMOVEACTION *arg1 = (struct _dwg_object_BLOCKMOVEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKACTION_connectionpts *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKMOVEACTION_conn_pts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKMOVEACTION_conn_pts_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKMOVEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKMOVEACTION *)(argp1);
    result = (Dwg_BLOCKACTION_connectionpts *)(Dwg_BLOCKACTION_connectionpts *) ((arg1)->conn_pts);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKACTION_connectionpts, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKMOVEACTION_action_offset_x_set) {
  {
    struct _dwg_object_BLOCKMOVEACTION *arg1 = (struct _dwg_object_BLOCKMOVEACTION *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKMOVEACTION_action_offset_x_set(self,action_offset_x);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKMOVEACTION_action_offset_x_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKMOVEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKMOVEACTION *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKMOVEACTION_action_offset_x_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->action_offset_x = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKMOVEACTION_action_offset_x_get) {
  {
    struct _dwg_object_BLOCKMOVEACTION *arg1 = (struct _dwg_object_BLOCKMOVEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKMOVEACTION_action_offset_x_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKMOVEACTION_action_offset_x_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKMOVEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKMOVEACTION *)(argp1);
    result = (BITCODE_BD) ((arg1)->action_offset_x);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKMOVEACTION_action_offset_y_set) {
  {
    struct _dwg_object_BLOCKMOVEACTION *arg1 = (struct _dwg_object_BLOCKMOVEACTION *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKMOVEACTION_action_offset_y_set(self,action_offset_y);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKMOVEACTION_action_offset_y_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKMOVEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKMOVEACTION *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKMOVEACTION_action_offset_y_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->action_offset_y = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKMOVEACTION_action_offset_y_get) {
  {
    struct _dwg_object_BLOCKMOVEACTION *arg1 = (struct _dwg_object_BLOCKMOVEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKMOVEACTION_action_offset_y_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKMOVEACTION_action_offset_y_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKMOVEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKMOVEACTION *)(argp1);
    result = (BITCODE_BD) ((arg1)->action_offset_y);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKMOVEACTION_angle_offset_set) {
  {
    struct _dwg_object_BLOCKMOVEACTION *arg1 = (struct _dwg_object_BLOCKMOVEACTION *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKMOVEACTION_angle_offset_set(self,angle_offset);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKMOVEACTION_angle_offset_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKMOVEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKMOVEACTION *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKMOVEACTION_angle_offset_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->angle_offset = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKMOVEACTION_angle_offset_get) {
  {
    struct _dwg_object_BLOCKMOVEACTION *arg1 = (struct _dwg_object_BLOCKMOVEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKMOVEACTION_angle_offset_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKMOVEACTION_angle_offset_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKMOVEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKMOVEACTION *)(argp1);
    result = (BITCODE_BD) ((arg1)->angle_offset);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKMOVEACTION) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKMOVEACTION *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKMOVEACTION();");
    }
    result = (struct _dwg_object_BLOCKMOVEACTION *)calloc(1, sizeof(struct _dwg_object_BLOCKMOVEACTION));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKMOVEACTION) {
  {
    struct _dwg_object_BLOCKMOVEACTION *arg1 = (struct _dwg_object_BLOCKMOVEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKMOVEACTION(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKMOVEACTION" "', argument " "1"" of type '" "struct _dwg_object_BLOCKMOVEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKMOVEACTION *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOINTPARAMETER_parent_set) {
  {
    struct _dwg_object_BLOCKPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOINTPARAMETER_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOINTPARAMETER_parent_get) {
  {
    struct _dwg_object_BLOCKPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOINTPARAMETER_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOINTPARAMETER_evalexpr_set) {
  {
    struct _dwg_object_BLOCKPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOINTPARAMETER_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOINTPARAMETER_evalexpr_get) {
  {
    struct _dwg_object_BLOCKPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOINTPARAMETER_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOINTPARAMETER_name_set) {
  {
    struct _dwg_object_BLOCKPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOINTPARAMETER_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOINTPARAMETER_name_get) {
  {
    struct _dwg_object_BLOCKPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOINTPARAMETER_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOINTPARAMETER_be_major_set) {
  {
    struct _dwg_object_BLOCKPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOINTPARAMETER_be_major_set(self,be_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_be_major_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_be_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOINTPARAMETER_be_major_get) {
  {
    struct _dwg_object_BLOCKPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOINTPARAMETER_be_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_be_major_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOINTPARAMETER_be_minor_set) {
  {
    struct _dwg_object_BLOCKPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOINTPARAMETER_be_minor_set(self,be_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_be_minor_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_be_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOINTPARAMETER_be_minor_get) {
  {
    struct _dwg_object_BLOCKPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOINTPARAMETER_be_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_be_minor_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOINTPARAMETER_eed1071_set) {
  {
    struct _dwg_object_BLOCKPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOINTPARAMETER_eed1071_set(self,eed1071);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_eed1071_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_eed1071_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->eed1071 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOINTPARAMETER_eed1071_get) {
  {
    struct _dwg_object_BLOCKPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOINTPARAMETER_eed1071_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_eed1071_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->eed1071);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOINTPARAMETER_show_properties_set) {
  {
    struct _dwg_object_BLOCKPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOINTPARAMETER_show_properties_set(self,show_properties);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_show_properties_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_show_properties_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->show_properties = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOINTPARAMETER_show_properties_get) {
  {
    struct _dwg_object_BLOCKPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOINTPARAMETER_show_properties_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_show_properties_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->show_properties);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOINTPARAMETER_chain_actions_set) {
  {
    struct _dwg_object_BLOCKPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOINTPARAMETER_chain_actions_set(self,chain_actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_chain_actions_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_chain_actions_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->chain_actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOINTPARAMETER_chain_actions_get) {
  {
    struct _dwg_object_BLOCKPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOINTPARAMETER_chain_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_chain_actions_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->chain_actions);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOINTPARAMETER_def_pt_set) {
  {
    struct _dwg_object_BLOCKPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOINTPARAMETER_def_pt_set(self,def_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_def_pt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_def_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOINTPARAMETER_def_pt_get) {
  {
    struct _dwg_object_BLOCKPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOINTPARAMETER_def_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_def_pt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOINTPARAMETER_num_propinfos_set) {
  {
    struct _dwg_object_BLOCKPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOINTPARAMETER_num_propinfos_set(self,num_propinfos);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_num_propinfos_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_num_propinfos_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_propinfos = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOINTPARAMETER_num_propinfos_get) {
  {
    struct _dwg_object_BLOCKPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOINTPARAMETER_num_propinfos_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_num_propinfos_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_propinfos);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOINTPARAMETER_prop1_set) {
  {
    struct _dwg_object_BLOCKPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOINTPARAMETER_prop1_set(self,prop1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_prop1_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_prop1_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop1 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOINTPARAMETER_prop1_get) {
  {
    struct _dwg_object_BLOCKPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOINTPARAMETER_prop1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_prop1_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOINTPARAMETER_prop2_set) {
  {
    struct _dwg_object_BLOCKPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOINTPARAMETER_prop2_set(self,prop2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_prop2_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_prop2_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop2 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOINTPARAMETER_prop2_get) {
  {
    struct _dwg_object_BLOCKPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOINTPARAMETER_prop2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_prop2_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOINTPARAMETER_position_name_set) {
  {
    struct _dwg_object_BLOCKPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOINTPARAMETER_position_name_set(self,position_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_position_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_position_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->position_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->position_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->position_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOINTPARAMETER_position_name_get) {
  {
    struct _dwg_object_BLOCKPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOINTPARAMETER_position_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_position_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->position_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOINTPARAMETER_position_desc_set) {
  {
    struct _dwg_object_BLOCKPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOINTPARAMETER_position_desc_set(self,position_desc);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_position_desc_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_position_desc_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->position_desc);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->position_desc = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->position_desc = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOINTPARAMETER_position_desc_get) {
  {
    struct _dwg_object_BLOCKPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOINTPARAMETER_position_desc_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_position_desc_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->position_desc);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOINTPARAMETER_def_label_pt_set) {
  {
    struct _dwg_object_BLOCKPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOINTPARAMETER_def_label_pt_set(self,def_label_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_def_label_pt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_def_label_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_label_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOINTPARAMETER_def_label_pt_get) {
  {
    struct _dwg_object_BLOCKPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOINTPARAMETER_def_label_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOINTPARAMETER_def_label_pt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_label_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKPOINTPARAMETER) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKPOINTPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKPOINTPARAMETER();");
    }
    result = (struct _dwg_object_BLOCKPOINTPARAMETER *)calloc(1, sizeof(struct _dwg_object_BLOCKPOINTPARAMETER));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKPOINTPARAMETER) {
  {
    struct _dwg_object_BLOCKPOINTPARAMETER *arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKPOINTPARAMETER(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKPOINTPARAMETER" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOINTPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOINTPARAMETER *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARGRIP_parent_set) {
  {
    struct _dwg_object_BLOCKPOLARGRIP *arg1 = (struct _dwg_object_BLOCKPOLARGRIP *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARGRIP_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARGRIP_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARGRIP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOLARGRIP_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARGRIP_parent_get) {
  {
    struct _dwg_object_BLOCKPOLARGRIP *arg1 = (struct _dwg_object_BLOCKPOLARGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARGRIP_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARGRIP_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARGRIP *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARGRIP_evalexpr_set) {
  {
    struct _dwg_object_BLOCKPOLARGRIP *arg1 = (struct _dwg_object_BLOCKPOLARGRIP *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARGRIP_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARGRIP_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARGRIP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOLARGRIP_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARGRIP_evalexpr_get) {
  {
    struct _dwg_object_BLOCKPOLARGRIP *arg1 = (struct _dwg_object_BLOCKPOLARGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARGRIP_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARGRIP_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARGRIP *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARGRIP_name_set) {
  {
    struct _dwg_object_BLOCKPOLARGRIP *arg1 = (struct _dwg_object_BLOCKPOLARGRIP *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARGRIP_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARGRIP_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARGRIP *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOLARGRIP_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARGRIP_name_get) {
  {
    struct _dwg_object_BLOCKPOLARGRIP *arg1 = (struct _dwg_object_BLOCKPOLARGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARGRIP_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARGRIP_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARGRIP *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARGRIP_be_major_set) {
  {
    struct _dwg_object_BLOCKPOLARGRIP *arg1 = (struct _dwg_object_BLOCKPOLARGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARGRIP_be_major_set(self,be_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARGRIP_be_major_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPOLARGRIP_be_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARGRIP_be_major_get) {
  {
    struct _dwg_object_BLOCKPOLARGRIP *arg1 = (struct _dwg_object_BLOCKPOLARGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARGRIP_be_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARGRIP_be_major_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARGRIP_be_minor_set) {
  {
    struct _dwg_object_BLOCKPOLARGRIP *arg1 = (struct _dwg_object_BLOCKPOLARGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARGRIP_be_minor_set(self,be_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARGRIP_be_minor_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPOLARGRIP_be_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARGRIP_be_minor_get) {
  {
    struct _dwg_object_BLOCKPOLARGRIP *arg1 = (struct _dwg_object_BLOCKPOLARGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARGRIP_be_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARGRIP_be_minor_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARGRIP_eed1071_set) {
  {
    struct _dwg_object_BLOCKPOLARGRIP *arg1 = (struct _dwg_object_BLOCKPOLARGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARGRIP_eed1071_set(self,eed1071);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARGRIP_eed1071_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPOLARGRIP_eed1071_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->eed1071 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARGRIP_eed1071_get) {
  {
    struct _dwg_object_BLOCKPOLARGRIP *arg1 = (struct _dwg_object_BLOCKPOLARGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARGRIP_eed1071_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARGRIP_eed1071_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->eed1071);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARGRIP_bg_bl91_set) {
  {
    struct _dwg_object_BLOCKPOLARGRIP *arg1 = (struct _dwg_object_BLOCKPOLARGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARGRIP_bg_bl91_set(self,bg_bl91);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARGRIP_bg_bl91_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPOLARGRIP_bg_bl91_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bg_bl91 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARGRIP_bg_bl91_get) {
  {
    struct _dwg_object_BLOCKPOLARGRIP *arg1 = (struct _dwg_object_BLOCKPOLARGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARGRIP_bg_bl91_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARGRIP_bg_bl91_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->bg_bl91);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARGRIP_bg_bl92_set) {
  {
    struct _dwg_object_BLOCKPOLARGRIP *arg1 = (struct _dwg_object_BLOCKPOLARGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARGRIP_bg_bl92_set(self,bg_bl92);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARGRIP_bg_bl92_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPOLARGRIP_bg_bl92_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bg_bl92 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARGRIP_bg_bl92_get) {
  {
    struct _dwg_object_BLOCKPOLARGRIP *arg1 = (struct _dwg_object_BLOCKPOLARGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARGRIP_bg_bl92_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARGRIP_bg_bl92_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->bg_bl92);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARGRIP_bg_location_set) {
  {
    struct _dwg_object_BLOCKPOLARGRIP *arg1 = (struct _dwg_object_BLOCKPOLARGRIP *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARGRIP_bg_location_set(self,bg_location);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARGRIP_bg_location_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARGRIP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOLARGRIP_bg_location_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->bg_location = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARGRIP_bg_location_get) {
  {
    struct _dwg_object_BLOCKPOLARGRIP *arg1 = (struct _dwg_object_BLOCKPOLARGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARGRIP_bg_location_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARGRIP_bg_location_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARGRIP *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->bg_location);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARGRIP_bg_insert_cycling_set) {
  {
    struct _dwg_object_BLOCKPOLARGRIP *arg1 = (struct _dwg_object_BLOCKPOLARGRIP *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARGRIP_bg_insert_cycling_set(self,bg_insert_cycling);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARGRIP_bg_insert_cycling_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPOLARGRIP_bg_insert_cycling_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->bg_insert_cycling = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARGRIP_bg_insert_cycling_get) {
  {
    struct _dwg_object_BLOCKPOLARGRIP *arg1 = (struct _dwg_object_BLOCKPOLARGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARGRIP_bg_insert_cycling_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARGRIP_bg_insert_cycling_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARGRIP *)(argp1);
    result = (BITCODE_B) ((arg1)->bg_insert_cycling);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARGRIP_bg_insert_cycling_weight_set) {
  {
    struct _dwg_object_BLOCKPOLARGRIP *arg1 = (struct _dwg_object_BLOCKPOLARGRIP *) 0 ;
    BITCODE_BLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARGRIP_bg_insert_cycling_weight_set(self,bg_insert_cycling_weight);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARGRIP_bg_insert_cycling_weight_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARGRIP *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPOLARGRIP_bg_insert_cycling_weight_set" "', argument " "2"" of type '" "BITCODE_BLd""'");
    } 
    arg2 = (BITCODE_BLd)(val2);
    if (arg1) (arg1)->bg_insert_cycling_weight = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARGRIP_bg_insert_cycling_weight_get) {
  {
    struct _dwg_object_BLOCKPOLARGRIP *arg1 = (struct _dwg_object_BLOCKPOLARGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARGRIP_bg_insert_cycling_weight_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARGRIP_bg_insert_cycling_weight_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARGRIP *)(argp1);
    result = (BITCODE_BLd) ((arg1)->bg_insert_cycling_weight);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKPOLARGRIP) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKPOLARGRIP *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKPOLARGRIP();");
    }
    result = (struct _dwg_object_BLOCKPOLARGRIP *)calloc(1, sizeof(struct _dwg_object_BLOCKPOLARGRIP));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKPOLARGRIP, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKPOLARGRIP) {
  {
    struct _dwg_object_BLOCKPOLARGRIP *arg1 = (struct _dwg_object_BLOCKPOLARGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKPOLARGRIP(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARGRIP, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKPOLARGRIP" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARGRIP *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_parent_set) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_parent_get) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_evalexpr_set) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_evalexpr_get) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_name_set) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_name_get) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_be_major_set) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_be_major_set(self,be_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_be_major_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_be_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_be_major_get) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_be_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_be_major_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_be_minor_set) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_be_minor_set(self,be_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_be_minor_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_be_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_be_minor_get) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_be_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_be_minor_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_eed1071_set) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_eed1071_set(self,eed1071);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_eed1071_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_eed1071_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->eed1071 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_eed1071_get) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_eed1071_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_eed1071_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->eed1071);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_show_properties_set) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_show_properties_set(self,show_properties);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_show_properties_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_show_properties_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->show_properties = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_show_properties_get) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_show_properties_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_show_properties_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->show_properties);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_chain_actions_set) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_chain_actions_set(self,chain_actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_chain_actions_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_chain_actions_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->chain_actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_chain_actions_get) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_chain_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_chain_actions_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->chain_actions);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_def_basept_set) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_def_basept_set(self,def_basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_def_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_def_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_def_basept_get) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_def_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_def_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_def_endpt_set) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_def_endpt_set(self,def_endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_def_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_def_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_def_endpt_get) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_def_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_def_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_prop1_set) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_prop1_set(self,prop1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_prop1_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_prop1_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop1 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_prop1_get) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_prop1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_prop1_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_prop2_set) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_prop2_set(self,prop2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_prop2_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_prop2_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop2 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_prop2_get) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_prop2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_prop2_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_prop3_set) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_prop3_set(self,prop3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_prop3_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_prop3_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop3 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_prop3_get) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_prop3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_prop3_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop3);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_prop4_set) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_prop4_set(self,prop4);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_prop4_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_prop4_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop4 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_prop4_get) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_prop4_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_prop4_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop4);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_prop_states_set) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_prop_states_set(self,prop_states);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_prop_states_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_prop_states_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->prop_states = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_prop_states_get) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_prop_states_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_prop_states_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    result = (BITCODE_BL *) ((arg1)->prop_states);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_parameter_base_location_set) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_parameter_base_location_set(self,parameter_base_location);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_parameter_base_location_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_parameter_base_location_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->parameter_base_location = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_parameter_base_location_get) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_parameter_base_location_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_parameter_base_location_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    result = (BITCODE_BS) ((arg1)->parameter_base_location);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_upd_basept_set) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_upd_basept_set(self,upd_basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_upd_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_upd_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->upd_basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_upd_basept_get) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_upd_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_upd_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->upd_basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_basept_set) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_basept_set(self,basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_basept_get) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_upd_endpt_set) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_upd_endpt_set(self,upd_endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_upd_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_upd_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->upd_endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_upd_endpt_get) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_upd_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_upd_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->upd_endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_endpt_set) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_endpt_set(self,endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_endpt_get) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_angle_name_set) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_angle_name_set(self,angle_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_angle_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_angle_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->angle_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->angle_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->angle_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_angle_name_get) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_angle_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_angle_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->angle_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_angle_desc_set) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_angle_desc_set(self,angle_desc);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_angle_desc_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_angle_desc_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->angle_desc);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->angle_desc = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->angle_desc = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_angle_desc_get) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_angle_desc_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_angle_desc_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->angle_desc);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_distance_name_set) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_distance_name_set(self,distance_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_distance_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_distance_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->distance_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->distance_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->distance_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_distance_name_get) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_distance_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_distance_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->distance_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_distance_desc_set) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_distance_desc_set(self,distance_desc);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_distance_desc_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_distance_desc_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->distance_desc);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->distance_desc = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->distance_desc = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_distance_desc_get) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_distance_desc_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_distance_desc_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->distance_desc);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_offset_set) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_offset_set(self,offset);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_offset_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_offset_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->offset = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_offset_get) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_offset_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_offset_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    result = (BITCODE_BD) ((arg1)->offset);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_angle_value_set_set) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    Dwg_BLOCKPARAMVALUESET *arg2 = (Dwg_BLOCKPARAMVALUESET *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_angle_value_set_set(self,angle_value_set);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_angle_value_set_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_angle_value_set_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMVALUESET *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMVALUESET *)(argp2);
    if (arg1) (arg1)->angle_value_set = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_angle_value_set_get) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMVALUESET *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_angle_value_set_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_angle_value_set_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMVALUESET *)& ((arg1)->angle_value_set);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_distance_value_set_set) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    Dwg_BLOCKPARAMVALUESET *arg2 = (Dwg_BLOCKPARAMVALUESET *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_distance_value_set_set(self,distance_value_set);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_distance_value_set_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_distance_value_set_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMVALUESET *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMVALUESET *)(argp2);
    if (arg1) (arg1)->distance_value_set = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARPARAMETER_distance_value_set_get) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMVALUESET *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARPARAMETER_distance_value_set_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARPARAMETER_distance_value_set_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMVALUESET *)& ((arg1)->distance_value_set);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKPOLARPARAMETER) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKPOLARPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKPOLARPARAMETER();");
    }
    result = (struct _dwg_object_BLOCKPOLARPARAMETER *)calloc(1, sizeof(struct _dwg_object_BLOCKPOLARPARAMETER));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKPOLARPARAMETER) {
  {
    struct _dwg_object_BLOCKPOLARPARAMETER *arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKPOLARPARAMETER(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKPOLARPARAMETER" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARPARAMETER *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_parent_set) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_parent_get) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_evalexpr_set) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_evalexpr_get) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_name_set) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_name_get) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_be_major_set) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_be_major_set(self,be_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_be_major_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_be_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_be_major_get) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_be_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_be_major_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_be_minor_set) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_be_minor_set(self,be_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_be_minor_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_be_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_be_minor_get) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_be_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_be_minor_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_eed1071_set) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_eed1071_set(self,eed1071);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_eed1071_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_eed1071_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->eed1071 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_eed1071_get) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_eed1071_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_eed1071_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->eed1071);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_display_location_set) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_display_location_set(self,display_location);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_display_location_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_display_location_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->display_location = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_display_location_get) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_display_location_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_display_location_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->display_location);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_num_actions_set) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_num_actions_set(self,num_actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_num_actions_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_num_actions_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_num_actions_get) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_num_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_num_actions_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_actions);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_actions_set) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_actions_set(self,actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_actions_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_actions_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_actions_get) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_actions_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    result = (BITCODE_BL *) ((arg1)->actions);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_num_deps_set) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_num_deps_set(self,num_deps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_num_deps_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_num_deps_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_deps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_num_deps_get) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_num_deps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_num_deps_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_deps);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_deps_set) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_deps_set(self,deps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_deps_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_deps_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->deps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_deps_get) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_deps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_deps_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    result = (BITCODE_H *) ((arg1)->deps);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_conn_pts_set) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    Dwg_BLOCKACTION_connectionpts *arg2 = (Dwg_BLOCKACTION_connectionpts *) (Dwg_BLOCKACTION_connectionpts *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_conn_pts_set(self,conn_pts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_conn_pts_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKACTION_connectionpts, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_conn_pts_set" "', argument " "2"" of type '" "Dwg_BLOCKACTION_connectionpts [6]""'"); 
    } 
    arg2 = (Dwg_BLOCKACTION_connectionpts *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)6; ++ii) *(Dwg_BLOCKACTION_connectionpts *)&arg1->conn_pts[ii] = *((Dwg_BLOCKACTION_connectionpts *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""conn_pts""' of type '""Dwg_BLOCKACTION_connectionpts [6]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_conn_pts_get) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKACTION_connectionpts *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_conn_pts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_conn_pts_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    result = (Dwg_BLOCKACTION_connectionpts *)(Dwg_BLOCKACTION_connectionpts *) ((arg1)->conn_pts);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKACTION_connectionpts, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_num_pts_set) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_num_pts_set(self,num_pts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_num_pts_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_num_pts_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_pts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_num_pts_get) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_num_pts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_num_pts_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_pts);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_pts_set) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_pts_set(self,pts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_pts_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_pts_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->pts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_pts_get) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_pts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_pts_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    result = (BITCODE_2RD *) ((arg1)->pts);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_num_hdls_set) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_num_hdls_set(self,num_hdls);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_num_hdls_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_num_hdls_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_hdls = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_num_hdls_get) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_num_hdls_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_num_hdls_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_hdls);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_hdls_set) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_hdls_set(self,hdls);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_hdls_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_hdls_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->hdls = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_hdls_get) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_hdls_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_hdls_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    result = (BITCODE_H *) ((arg1)->hdls);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_shorts_set) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    BITCODE_BS *arg2 = (BITCODE_BS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_shorts_set(self,shorts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_shorts_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_short, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_shorts_set" "', argument " "2"" of type '" "BITCODE_BS *""'"); 
    }
    arg2 = (BITCODE_BS *)(argp2);
    if (arg1) (arg1)->shorts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_shorts_get) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_shorts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_shorts_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    result = (BITCODE_BS *) ((arg1)->shorts);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_short, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_num_codes_set) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_num_codes_set(self,num_codes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_num_codes_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_num_codes_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_codes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_num_codes_get) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_num_codes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_num_codes_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_codes);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_codes_set) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_codes_set(self,codes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_codes_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_codes_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->codes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_codes_get) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPOLARSTRETCHACTION_codes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPOLARSTRETCHACTION_codes_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    result = (BITCODE_BL *) ((arg1)->codes);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKPOLARSTRETCHACTION) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKPOLARSTRETCHACTION();");
    }
    result = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)calloc(1, sizeof(struct _dwg_object_BLOCKPOLARSTRETCHACTION));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKPOLARSTRETCHACTION) {
  {
    struct _dwg_object_BLOCKPOLARSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKPOLARSTRETCHACTION(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKPOLARSTRETCHACTION" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPOLARSTRETCHACTION *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPROPERTIESTABLE_parent_set) {
  {
    struct _dwg_object_BLOCKPROPERTIESTABLE *arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLE *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPROPERTIESTABLE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLE_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPROPERTIESTABLE *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPROPERTIESTABLE_parent_get) {
  {
    struct _dwg_object_BLOCKPROPERTIESTABLE *arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPROPERTIESTABLE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLE_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPROPERTIESTABLE *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLE *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKPROPERTIESTABLE) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKPROPERTIESTABLE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKPROPERTIESTABLE();");
    }
    result = (struct _dwg_object_BLOCKPROPERTIESTABLE *)calloc(1, sizeof(struct _dwg_object_BLOCKPROPERTIESTABLE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKPROPERTIESTABLE) {
  {
    struct _dwg_object_BLOCKPROPERTIESTABLE *arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKPROPERTIESTABLE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKPROPERTIESTABLE" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPROPERTIESTABLE *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_parent_set) {
  {
    struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPROPERTIESTABLEGRIP_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLEGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLEGRIP_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLEGRIP_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_parent_get) {
  {
    struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPROPERTIESTABLEGRIP_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLEGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLEGRIP_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_evalexpr_set) {
  {
    struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPROPERTIESTABLEGRIP_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLEGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLEGRIP_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLEGRIP_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_evalexpr_get) {
  {
    struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPROPERTIESTABLEGRIP_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLEGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLEGRIP_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_name_set) {
  {
    struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPROPERTIESTABLEGRIP_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLEGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLEGRIP_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLEGRIP_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_name_get) {
  {
    struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPROPERTIESTABLEGRIP_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLEGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLEGRIP_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_be_major_set) {
  {
    struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPROPERTIESTABLEGRIP_be_major_set(self,be_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLEGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLEGRIP_be_major_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLEGRIP_be_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_be_major_get) {
  {
    struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPROPERTIESTABLEGRIP_be_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLEGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLEGRIP_be_major_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_be_minor_set) {
  {
    struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPROPERTIESTABLEGRIP_be_minor_set(self,be_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLEGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLEGRIP_be_minor_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLEGRIP_be_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_be_minor_get) {
  {
    struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPROPERTIESTABLEGRIP_be_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLEGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLEGRIP_be_minor_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_eed1071_set) {
  {
    struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPROPERTIESTABLEGRIP_eed1071_set(self,eed1071);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLEGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLEGRIP_eed1071_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLEGRIP_eed1071_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->eed1071 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_eed1071_get) {
  {
    struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPROPERTIESTABLEGRIP_eed1071_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLEGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLEGRIP_eed1071_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->eed1071);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_bl91_set) {
  {
    struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_bl91_set(self,bg_bl91);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLEGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_bl91_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_bl91_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bg_bl91 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_bl91_get) {
  {
    struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_bl91_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLEGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_bl91_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->bg_bl91);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_bl92_set) {
  {
    struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_bl92_set(self,bg_bl92);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLEGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_bl92_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_bl92_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bg_bl92 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_bl92_get) {
  {
    struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_bl92_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLEGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_bl92_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->bg_bl92);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_location_set) {
  {
    struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_location_set(self,bg_location);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLEGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_location_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_location_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->bg_location = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_location_get) {
  {
    struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_location_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLEGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_location_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->bg_location);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_insert_cycling_set) {
  {
    struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_insert_cycling_set(self,bg_insert_cycling);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLEGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_insert_cycling_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_insert_cycling_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->bg_insert_cycling = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_insert_cycling_get) {
  {
    struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_insert_cycling_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLEGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_insert_cycling_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *)(argp1);
    result = (BITCODE_B) ((arg1)->bg_insert_cycling);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_insert_cycling_weight_set) {
  {
    struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *) 0 ;
    BITCODE_BLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_insert_cycling_weight_set(self,bg_insert_cycling_weight);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLEGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_insert_cycling_weight_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_insert_cycling_weight_set" "', argument " "2"" of type '" "BITCODE_BLd""'");
    } 
    arg2 = (BITCODE_BLd)(val2);
    if (arg1) (arg1)->bg_insert_cycling_weight = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_insert_cycling_weight_get) {
  {
    struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_insert_cycling_weight_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLEGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_insert_cycling_weight_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *)(argp1);
    result = (BITCODE_BLd) ((arg1)->bg_insert_cycling_weight);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKPROPERTIESTABLEGRIP) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKPROPERTIESTABLEGRIP();");
    }
    result = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *)calloc(1, sizeof(struct _dwg_object_BLOCKPROPERTIESTABLEGRIP));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLEGRIP, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKPROPERTIESTABLEGRIP) {
  {
    struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKPROPERTIESTABLEGRIP(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLEGRIP, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKPROPERTIESTABLEGRIP" "', argument " "1"" of type '" "struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKREPRESENTATION_parent_set) {
  {
    struct _dwg_object_BLOCKREPRESENTATION *arg1 = (struct _dwg_object_BLOCKREPRESENTATION *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKREPRESENTATION_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKREPRESENTATION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKREPRESENTATION_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKREPRESENTATION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKREPRESENTATION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKREPRESENTATION_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKREPRESENTATION_parent_get) {
  {
    struct _dwg_object_BLOCKREPRESENTATION *arg1 = (struct _dwg_object_BLOCKREPRESENTATION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKREPRESENTATION_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKREPRESENTATION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKREPRESENTATION_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKREPRESENTATION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKREPRESENTATION *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKREPRESENTATION_flag_set) {
  {
    struct _dwg_object_BLOCKREPRESENTATION *arg1 = (struct _dwg_object_BLOCKREPRESENTATION *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKREPRESENTATION_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKREPRESENTATION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKREPRESENTATION_flag_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKREPRESENTATION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKREPRESENTATION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKREPRESENTATION_flag_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKREPRESENTATION_flag_get) {
  {
    struct _dwg_object_BLOCKREPRESENTATION *arg1 = (struct _dwg_object_BLOCKREPRESENTATION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKREPRESENTATION_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKREPRESENTATION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKREPRESENTATION_flag_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKREPRESENTATION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKREPRESENTATION *)(argp1);
    result = (BITCODE_BS) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKREPRESENTATION_block_set) {
  {
    struct _dwg_object_BLOCKREPRESENTATION *arg1 = (struct _dwg_object_BLOCKREPRESENTATION *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKREPRESENTATION_block_set(self,block);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKREPRESENTATION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKREPRESENTATION_block_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKREPRESENTATION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKREPRESENTATION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKREPRESENTATION_block_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->block = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKREPRESENTATION_block_get) {
  {
    struct _dwg_object_BLOCKREPRESENTATION *arg1 = (struct _dwg_object_BLOCKREPRESENTATION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKREPRESENTATION_block_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKREPRESENTATION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKREPRESENTATION_block_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKREPRESENTATION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKREPRESENTATION *)(argp1);
    result = (BITCODE_H) ((arg1)->block);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKREPRESENTATION) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKREPRESENTATION *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKREPRESENTATION();");
    }
    result = (struct _dwg_object_BLOCKREPRESENTATION *)calloc(1, sizeof(struct _dwg_object_BLOCKREPRESENTATION));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKREPRESENTATION, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKREPRESENTATION) {
  {
    struct _dwg_object_BLOCKREPRESENTATION *arg1 = (struct _dwg_object_BLOCKREPRESENTATION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKREPRESENTATION(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKREPRESENTATION, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKREPRESENTATION" "', argument " "1"" of type '" "struct _dwg_object_BLOCKREPRESENTATION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKREPRESENTATION *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATEACTION_parent_set) {
  {
    struct _dwg_object_BLOCKROTATEACTION *arg1 = (struct _dwg_object_BLOCKROTATEACTION *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATEACTION_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATEACTION_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATEACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKROTATEACTION_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATEACTION_parent_get) {
  {
    struct _dwg_object_BLOCKROTATEACTION *arg1 = (struct _dwg_object_BLOCKROTATEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATEACTION_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATEACTION_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATEACTION *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATEACTION_evalexpr_set) {
  {
    struct _dwg_object_BLOCKROTATEACTION *arg1 = (struct _dwg_object_BLOCKROTATEACTION *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATEACTION_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATEACTION_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATEACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKROTATEACTION_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATEACTION_evalexpr_get) {
  {
    struct _dwg_object_BLOCKROTATEACTION *arg1 = (struct _dwg_object_BLOCKROTATEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATEACTION_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATEACTION_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATEACTION *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATEACTION_name_set) {
  {
    struct _dwg_object_BLOCKROTATEACTION *arg1 = (struct _dwg_object_BLOCKROTATEACTION *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATEACTION_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATEACTION_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATEACTION *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKROTATEACTION_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATEACTION_name_get) {
  {
    struct _dwg_object_BLOCKROTATEACTION *arg1 = (struct _dwg_object_BLOCKROTATEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATEACTION_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATEACTION_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATEACTION *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATEACTION_be_major_set) {
  {
    struct _dwg_object_BLOCKROTATEACTION *arg1 = (struct _dwg_object_BLOCKROTATEACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATEACTION_be_major_set(self,be_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATEACTION_be_major_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATEACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKROTATEACTION_be_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATEACTION_be_major_get) {
  {
    struct _dwg_object_BLOCKROTATEACTION *arg1 = (struct _dwg_object_BLOCKROTATEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATEACTION_be_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATEACTION_be_major_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATEACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATEACTION_be_minor_set) {
  {
    struct _dwg_object_BLOCKROTATEACTION *arg1 = (struct _dwg_object_BLOCKROTATEACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATEACTION_be_minor_set(self,be_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATEACTION_be_minor_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATEACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKROTATEACTION_be_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATEACTION_be_minor_get) {
  {
    struct _dwg_object_BLOCKROTATEACTION *arg1 = (struct _dwg_object_BLOCKROTATEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATEACTION_be_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATEACTION_be_minor_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATEACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATEACTION_eed1071_set) {
  {
    struct _dwg_object_BLOCKROTATEACTION *arg1 = (struct _dwg_object_BLOCKROTATEACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATEACTION_eed1071_set(self,eed1071);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATEACTION_eed1071_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATEACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKROTATEACTION_eed1071_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->eed1071 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATEACTION_eed1071_get) {
  {
    struct _dwg_object_BLOCKROTATEACTION *arg1 = (struct _dwg_object_BLOCKROTATEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATEACTION_eed1071_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATEACTION_eed1071_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATEACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->eed1071);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATEACTION_display_location_set) {
  {
    struct _dwg_object_BLOCKROTATEACTION *arg1 = (struct _dwg_object_BLOCKROTATEACTION *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATEACTION_display_location_set(self,display_location);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATEACTION_display_location_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATEACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKROTATEACTION_display_location_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->display_location = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATEACTION_display_location_get) {
  {
    struct _dwg_object_BLOCKROTATEACTION *arg1 = (struct _dwg_object_BLOCKROTATEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATEACTION_display_location_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATEACTION_display_location_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATEACTION *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->display_location);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATEACTION_num_actions_set) {
  {
    struct _dwg_object_BLOCKROTATEACTION *arg1 = (struct _dwg_object_BLOCKROTATEACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATEACTION_num_actions_set(self,num_actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATEACTION_num_actions_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATEACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKROTATEACTION_num_actions_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATEACTION_num_actions_get) {
  {
    struct _dwg_object_BLOCKROTATEACTION *arg1 = (struct _dwg_object_BLOCKROTATEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATEACTION_num_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATEACTION_num_actions_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATEACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_actions);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATEACTION_actions_set) {
  {
    struct _dwg_object_BLOCKROTATEACTION *arg1 = (struct _dwg_object_BLOCKROTATEACTION *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATEACTION_actions_set(self,actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATEACTION_actions_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATEACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKROTATEACTION_actions_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATEACTION_actions_get) {
  {
    struct _dwg_object_BLOCKROTATEACTION *arg1 = (struct _dwg_object_BLOCKROTATEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATEACTION_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATEACTION_actions_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATEACTION *)(argp1);
    result = (BITCODE_BL *) ((arg1)->actions);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATEACTION_num_deps_set) {
  {
    struct _dwg_object_BLOCKROTATEACTION *arg1 = (struct _dwg_object_BLOCKROTATEACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATEACTION_num_deps_set(self,num_deps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATEACTION_num_deps_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATEACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKROTATEACTION_num_deps_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_deps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATEACTION_num_deps_get) {
  {
    struct _dwg_object_BLOCKROTATEACTION *arg1 = (struct _dwg_object_BLOCKROTATEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATEACTION_num_deps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATEACTION_num_deps_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATEACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_deps);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATEACTION_deps_set) {
  {
    struct _dwg_object_BLOCKROTATEACTION *arg1 = (struct _dwg_object_BLOCKROTATEACTION *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATEACTION_deps_set(self,deps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATEACTION_deps_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATEACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKROTATEACTION_deps_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->deps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATEACTION_deps_get) {
  {
    struct _dwg_object_BLOCKROTATEACTION *arg1 = (struct _dwg_object_BLOCKROTATEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATEACTION_deps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATEACTION_deps_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATEACTION *)(argp1);
    result = (BITCODE_H *) ((arg1)->deps);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATEACTION_offset_set) {
  {
    struct _dwg_object_BLOCKROTATEACTION *arg1 = (struct _dwg_object_BLOCKROTATEACTION *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATEACTION_offset_set(self,offset);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATEACTION_offset_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATEACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKROTATEACTION_offset_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->offset = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATEACTION_offset_get) {
  {
    struct _dwg_object_BLOCKROTATEACTION *arg1 = (struct _dwg_object_BLOCKROTATEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATEACTION_offset_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATEACTION_offset_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATEACTION *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->offset);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATEACTION_conn_pts_set) {
  {
    struct _dwg_object_BLOCKROTATEACTION *arg1 = (struct _dwg_object_BLOCKROTATEACTION *) 0 ;
    Dwg_BLOCKACTION_connectionpts *arg2 = (Dwg_BLOCKACTION_connectionpts *) (Dwg_BLOCKACTION_connectionpts *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATEACTION_conn_pts_set(self,conn_pts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATEACTION_conn_pts_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATEACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKACTION_connectionpts, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKROTATEACTION_conn_pts_set" "', argument " "2"" of type '" "Dwg_BLOCKACTION_connectionpts [3]""'"); 
    } 
    arg2 = (Dwg_BLOCKACTION_connectionpts *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)3; ++ii) *(Dwg_BLOCKACTION_connectionpts *)&arg1->conn_pts[ii] = *((Dwg_BLOCKACTION_connectionpts *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""conn_pts""' of type '""Dwg_BLOCKACTION_connectionpts [3]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATEACTION_conn_pts_get) {
  {
    struct _dwg_object_BLOCKROTATEACTION *arg1 = (struct _dwg_object_BLOCKROTATEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKACTION_connectionpts *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATEACTION_conn_pts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATEACTION_conn_pts_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATEACTION *)(argp1);
    result = (Dwg_BLOCKACTION_connectionpts *)(Dwg_BLOCKACTION_connectionpts *) ((arg1)->conn_pts);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKACTION_connectionpts, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATEACTION_dependent_set) {
  {
    struct _dwg_object_BLOCKROTATEACTION *arg1 = (struct _dwg_object_BLOCKROTATEACTION *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATEACTION_dependent_set(self,dependent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATEACTION_dependent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATEACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKROTATEACTION_dependent_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->dependent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATEACTION_dependent_get) {
  {
    struct _dwg_object_BLOCKROTATEACTION *arg1 = (struct _dwg_object_BLOCKROTATEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATEACTION_dependent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATEACTION_dependent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATEACTION *)(argp1);
    result = (BITCODE_B) ((arg1)->dependent);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATEACTION_base_pt_set) {
  {
    struct _dwg_object_BLOCKROTATEACTION *arg1 = (struct _dwg_object_BLOCKROTATEACTION *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATEACTION_base_pt_set(self,base_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATEACTION_base_pt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATEACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKROTATEACTION_base_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->base_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATEACTION_base_pt_get) {
  {
    struct _dwg_object_BLOCKROTATEACTION *arg1 = (struct _dwg_object_BLOCKROTATEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATEACTION_base_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATEACTION_base_pt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATEACTION *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->base_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKROTATEACTION) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKROTATEACTION *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKROTATEACTION();");
    }
    result = (struct _dwg_object_BLOCKROTATEACTION *)calloc(1, sizeof(struct _dwg_object_BLOCKROTATEACTION));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKROTATEACTION) {
  {
    struct _dwg_object_BLOCKROTATEACTION *arg1 = (struct _dwg_object_BLOCKROTATEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKROTATEACTION(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKROTATEACTION" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATEACTION *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONGRIP_parent_set) {
  {
    struct _dwg_object_BLOCKROTATIONGRIP *arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONGRIP_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONGRIP_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKROTATIONGRIP_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONGRIP_parent_get) {
  {
    struct _dwg_object_BLOCKROTATIONGRIP *arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONGRIP_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONGRIP_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONGRIP_evalexpr_set) {
  {
    struct _dwg_object_BLOCKROTATIONGRIP *arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONGRIP_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONGRIP_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKROTATIONGRIP_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONGRIP_evalexpr_get) {
  {
    struct _dwg_object_BLOCKROTATIONGRIP *arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONGRIP_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONGRIP_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONGRIP_name_set) {
  {
    struct _dwg_object_BLOCKROTATIONGRIP *arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONGRIP_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONGRIP_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKROTATIONGRIP_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONGRIP_name_get) {
  {
    struct _dwg_object_BLOCKROTATIONGRIP *arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONGRIP_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONGRIP_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONGRIP_be_major_set) {
  {
    struct _dwg_object_BLOCKROTATIONGRIP *arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONGRIP_be_major_set(self,be_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONGRIP_be_major_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKROTATIONGRIP_be_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONGRIP_be_major_get) {
  {
    struct _dwg_object_BLOCKROTATIONGRIP *arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONGRIP_be_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONGRIP_be_major_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONGRIP_be_minor_set) {
  {
    struct _dwg_object_BLOCKROTATIONGRIP *arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONGRIP_be_minor_set(self,be_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONGRIP_be_minor_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKROTATIONGRIP_be_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONGRIP_be_minor_get) {
  {
    struct _dwg_object_BLOCKROTATIONGRIP *arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONGRIP_be_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONGRIP_be_minor_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONGRIP_eed1071_set) {
  {
    struct _dwg_object_BLOCKROTATIONGRIP *arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONGRIP_eed1071_set(self,eed1071);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONGRIP_eed1071_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKROTATIONGRIP_eed1071_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->eed1071 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONGRIP_eed1071_get) {
  {
    struct _dwg_object_BLOCKROTATIONGRIP *arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONGRIP_eed1071_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONGRIP_eed1071_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->eed1071);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONGRIP_bg_bl91_set) {
  {
    struct _dwg_object_BLOCKROTATIONGRIP *arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONGRIP_bg_bl91_set(self,bg_bl91);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONGRIP_bg_bl91_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKROTATIONGRIP_bg_bl91_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bg_bl91 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONGRIP_bg_bl91_get) {
  {
    struct _dwg_object_BLOCKROTATIONGRIP *arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONGRIP_bg_bl91_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONGRIP_bg_bl91_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->bg_bl91);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONGRIP_bg_bl92_set) {
  {
    struct _dwg_object_BLOCKROTATIONGRIP *arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONGRIP_bg_bl92_set(self,bg_bl92);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONGRIP_bg_bl92_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKROTATIONGRIP_bg_bl92_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bg_bl92 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONGRIP_bg_bl92_get) {
  {
    struct _dwg_object_BLOCKROTATIONGRIP *arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONGRIP_bg_bl92_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONGRIP_bg_bl92_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->bg_bl92);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONGRIP_bg_location_set) {
  {
    struct _dwg_object_BLOCKROTATIONGRIP *arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONGRIP_bg_location_set(self,bg_location);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONGRIP_bg_location_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKROTATIONGRIP_bg_location_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->bg_location = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONGRIP_bg_location_get) {
  {
    struct _dwg_object_BLOCKROTATIONGRIP *arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONGRIP_bg_location_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONGRIP_bg_location_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->bg_location);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONGRIP_bg_insert_cycling_set) {
  {
    struct _dwg_object_BLOCKROTATIONGRIP *arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONGRIP_bg_insert_cycling_set(self,bg_insert_cycling);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONGRIP_bg_insert_cycling_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKROTATIONGRIP_bg_insert_cycling_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->bg_insert_cycling = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONGRIP_bg_insert_cycling_get) {
  {
    struct _dwg_object_BLOCKROTATIONGRIP *arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONGRIP_bg_insert_cycling_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONGRIP_bg_insert_cycling_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *)(argp1);
    result = (BITCODE_B) ((arg1)->bg_insert_cycling);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONGRIP_bg_insert_cycling_weight_set) {
  {
    struct _dwg_object_BLOCKROTATIONGRIP *arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *) 0 ;
    BITCODE_BLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONGRIP_bg_insert_cycling_weight_set(self,bg_insert_cycling_weight);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONGRIP_bg_insert_cycling_weight_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKROTATIONGRIP_bg_insert_cycling_weight_set" "', argument " "2"" of type '" "BITCODE_BLd""'");
    } 
    arg2 = (BITCODE_BLd)(val2);
    if (arg1) (arg1)->bg_insert_cycling_weight = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONGRIP_bg_insert_cycling_weight_get) {
  {
    struct _dwg_object_BLOCKROTATIONGRIP *arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONGRIP_bg_insert_cycling_weight_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONGRIP_bg_insert_cycling_weight_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *)(argp1);
    result = (BITCODE_BLd) ((arg1)->bg_insert_cycling_weight);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKROTATIONGRIP) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKROTATIONGRIP *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKROTATIONGRIP();");
    }
    result = (struct _dwg_object_BLOCKROTATIONGRIP *)calloc(1, sizeof(struct _dwg_object_BLOCKROTATIONGRIP));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKROTATIONGRIP, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKROTATIONGRIP) {
  {
    struct _dwg_object_BLOCKROTATIONGRIP *arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKROTATIONGRIP(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONGRIP, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKROTATIONGRIP" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONGRIP *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_parent_set) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_parent_get) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_evalexpr_set) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_evalexpr_get) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_name_set) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_name_get) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_be_major_set) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_be_major_set(self,be_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_be_major_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_be_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_be_major_get) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_be_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_be_major_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_be_minor_set) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_be_minor_set(self,be_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_be_minor_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_be_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_be_minor_get) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_be_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_be_minor_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_eed1071_set) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_eed1071_set(self,eed1071);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_eed1071_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_eed1071_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->eed1071 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_eed1071_get) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_eed1071_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_eed1071_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->eed1071);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_show_properties_set) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_show_properties_set(self,show_properties);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_show_properties_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_show_properties_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->show_properties = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_show_properties_get) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_show_properties_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_show_properties_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->show_properties);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_chain_actions_set) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_chain_actions_set(self,chain_actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_chain_actions_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_chain_actions_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->chain_actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_chain_actions_get) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_chain_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_chain_actions_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->chain_actions);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_def_basept_set) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_def_basept_set(self,def_basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_def_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_def_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_def_basept_get) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_def_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_def_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_def_endpt_set) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_def_endpt_set(self,def_endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_def_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_def_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_def_endpt_get) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_def_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_def_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_prop1_set) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_prop1_set(self,prop1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_prop1_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_prop1_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop1 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_prop1_get) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_prop1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_prop1_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_prop2_set) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_prop2_set(self,prop2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_prop2_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_prop2_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop2 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_prop2_get) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_prop2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_prop2_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_prop3_set) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_prop3_set(self,prop3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_prop3_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_prop3_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop3 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_prop3_get) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_prop3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_prop3_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop3);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_prop4_set) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_prop4_set(self,prop4);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_prop4_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_prop4_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop4 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_prop4_get) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_prop4_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_prop4_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop4);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_prop_states_set) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_prop_states_set(self,prop_states);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_prop_states_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_prop_states_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->prop_states = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_prop_states_get) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_prop_states_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_prop_states_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    result = (BITCODE_BL *) ((arg1)->prop_states);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_parameter_base_location_set) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_parameter_base_location_set(self,parameter_base_location);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_parameter_base_location_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_parameter_base_location_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->parameter_base_location = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_parameter_base_location_get) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_parameter_base_location_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_parameter_base_location_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    result = (BITCODE_BS) ((arg1)->parameter_base_location);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_upd_basept_set) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_upd_basept_set(self,upd_basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_upd_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_upd_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->upd_basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_upd_basept_get) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_upd_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_upd_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->upd_basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_basept_set) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_basept_set(self,basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_basept_get) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_upd_endpt_set) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_upd_endpt_set(self,upd_endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_upd_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_upd_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->upd_endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_upd_endpt_get) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_upd_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_upd_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->upd_endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_endpt_set) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_endpt_set(self,endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_endpt_get) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_def_base_angle_pt_set) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_def_base_angle_pt_set(self,def_base_angle_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_def_base_angle_pt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_def_base_angle_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_base_angle_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_def_base_angle_pt_get) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_def_base_angle_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_def_base_angle_pt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_base_angle_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_angle_name_set) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_angle_name_set(self,angle_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_angle_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_angle_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->angle_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->angle_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->angle_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_angle_name_get) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_angle_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_angle_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->angle_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_angle_desc_set) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_angle_desc_set(self,angle_desc);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_angle_desc_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_angle_desc_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->angle_desc);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->angle_desc = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->angle_desc = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_angle_desc_get) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_angle_desc_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_angle_desc_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->angle_desc);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_angle_set) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_angle_set(self,angle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_angle_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_angle_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->angle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_angle_get) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_angle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_angle_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    result = (BITCODE_BD) ((arg1)->angle);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_angle_value_set_set) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    Dwg_BLOCKPARAMVALUESET *arg2 = (Dwg_BLOCKPARAMVALUESET *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_angle_value_set_set(self,angle_value_set);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_angle_value_set_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_angle_value_set_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMVALUESET *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMVALUESET *)(argp2);
    if (arg1) (arg1)->angle_value_set = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKROTATIONPARAMETER_angle_value_set_get) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMVALUESET *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKROTATIONPARAMETER_angle_value_set_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKROTATIONPARAMETER_angle_value_set_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMVALUESET *)& ((arg1)->angle_value_set);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKROTATIONPARAMETER) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKROTATIONPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKROTATIONPARAMETER();");
    }
    result = (struct _dwg_object_BLOCKROTATIONPARAMETER *)calloc(1, sizeof(struct _dwg_object_BLOCKROTATIONPARAMETER));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKROTATIONPARAMETER) {
  {
    struct _dwg_object_BLOCKROTATIONPARAMETER *arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKROTATIONPARAMETER(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKROTATIONPARAMETER" "', argument " "1"" of type '" "struct _dwg_object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKROTATIONPARAMETER *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSCALEACTION_parent_set) {
  {
    struct _dwg_object_BLOCKSCALEACTION *arg1 = (struct _dwg_object_BLOCKSCALEACTION *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSCALEACTION_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSCALEACTION_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSCALEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSCALEACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKSCALEACTION_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSCALEACTION_parent_get) {
  {
    struct _dwg_object_BLOCKSCALEACTION *arg1 = (struct _dwg_object_BLOCKSCALEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSCALEACTION_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSCALEACTION_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSCALEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSCALEACTION *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSCALEACTION_evalexpr_set) {
  {
    struct _dwg_object_BLOCKSCALEACTION *arg1 = (struct _dwg_object_BLOCKSCALEACTION *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSCALEACTION_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSCALEACTION_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSCALEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSCALEACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKSCALEACTION_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSCALEACTION_evalexpr_get) {
  {
    struct _dwg_object_BLOCKSCALEACTION *arg1 = (struct _dwg_object_BLOCKSCALEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSCALEACTION_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSCALEACTION_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSCALEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSCALEACTION *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSCALEACTION_name_set) {
  {
    struct _dwg_object_BLOCKSCALEACTION *arg1 = (struct _dwg_object_BLOCKSCALEACTION *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSCALEACTION_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSCALEACTION_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSCALEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSCALEACTION *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKSCALEACTION_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSCALEACTION_name_get) {
  {
    struct _dwg_object_BLOCKSCALEACTION *arg1 = (struct _dwg_object_BLOCKSCALEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSCALEACTION_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSCALEACTION_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSCALEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSCALEACTION *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSCALEACTION_be_major_set) {
  {
    struct _dwg_object_BLOCKSCALEACTION *arg1 = (struct _dwg_object_BLOCKSCALEACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSCALEACTION_be_major_set(self,be_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSCALEACTION_be_major_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSCALEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSCALEACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKSCALEACTION_be_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSCALEACTION_be_major_get) {
  {
    struct _dwg_object_BLOCKSCALEACTION *arg1 = (struct _dwg_object_BLOCKSCALEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSCALEACTION_be_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSCALEACTION_be_major_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSCALEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSCALEACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSCALEACTION_be_minor_set) {
  {
    struct _dwg_object_BLOCKSCALEACTION *arg1 = (struct _dwg_object_BLOCKSCALEACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSCALEACTION_be_minor_set(self,be_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSCALEACTION_be_minor_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSCALEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSCALEACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKSCALEACTION_be_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSCALEACTION_be_minor_get) {
  {
    struct _dwg_object_BLOCKSCALEACTION *arg1 = (struct _dwg_object_BLOCKSCALEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSCALEACTION_be_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSCALEACTION_be_minor_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSCALEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSCALEACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSCALEACTION_eed1071_set) {
  {
    struct _dwg_object_BLOCKSCALEACTION *arg1 = (struct _dwg_object_BLOCKSCALEACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSCALEACTION_eed1071_set(self,eed1071);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSCALEACTION_eed1071_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSCALEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSCALEACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKSCALEACTION_eed1071_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->eed1071 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSCALEACTION_eed1071_get) {
  {
    struct _dwg_object_BLOCKSCALEACTION *arg1 = (struct _dwg_object_BLOCKSCALEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSCALEACTION_eed1071_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSCALEACTION_eed1071_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSCALEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSCALEACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->eed1071);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSCALEACTION_display_location_set) {
  {
    struct _dwg_object_BLOCKSCALEACTION *arg1 = (struct _dwg_object_BLOCKSCALEACTION *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSCALEACTION_display_location_set(self,display_location);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSCALEACTION_display_location_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSCALEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSCALEACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKSCALEACTION_display_location_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->display_location = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSCALEACTION_display_location_get) {
  {
    struct _dwg_object_BLOCKSCALEACTION *arg1 = (struct _dwg_object_BLOCKSCALEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSCALEACTION_display_location_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSCALEACTION_display_location_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSCALEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSCALEACTION *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->display_location);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSCALEACTION_num_actions_set) {
  {
    struct _dwg_object_BLOCKSCALEACTION *arg1 = (struct _dwg_object_BLOCKSCALEACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSCALEACTION_num_actions_set(self,num_actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSCALEACTION_num_actions_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSCALEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSCALEACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKSCALEACTION_num_actions_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSCALEACTION_num_actions_get) {
  {
    struct _dwg_object_BLOCKSCALEACTION *arg1 = (struct _dwg_object_BLOCKSCALEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSCALEACTION_num_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSCALEACTION_num_actions_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSCALEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSCALEACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_actions);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSCALEACTION_actions_set) {
  {
    struct _dwg_object_BLOCKSCALEACTION *arg1 = (struct _dwg_object_BLOCKSCALEACTION *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSCALEACTION_actions_set(self,actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSCALEACTION_actions_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSCALEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSCALEACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKSCALEACTION_actions_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSCALEACTION_actions_get) {
  {
    struct _dwg_object_BLOCKSCALEACTION *arg1 = (struct _dwg_object_BLOCKSCALEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSCALEACTION_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSCALEACTION_actions_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSCALEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSCALEACTION *)(argp1);
    result = (BITCODE_BL *) ((arg1)->actions);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSCALEACTION_num_deps_set) {
  {
    struct _dwg_object_BLOCKSCALEACTION *arg1 = (struct _dwg_object_BLOCKSCALEACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSCALEACTION_num_deps_set(self,num_deps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSCALEACTION_num_deps_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSCALEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSCALEACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKSCALEACTION_num_deps_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_deps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSCALEACTION_num_deps_get) {
  {
    struct _dwg_object_BLOCKSCALEACTION *arg1 = (struct _dwg_object_BLOCKSCALEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSCALEACTION_num_deps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSCALEACTION_num_deps_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSCALEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSCALEACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_deps);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSCALEACTION_deps_set) {
  {
    struct _dwg_object_BLOCKSCALEACTION *arg1 = (struct _dwg_object_BLOCKSCALEACTION *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSCALEACTION_deps_set(self,deps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSCALEACTION_deps_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSCALEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSCALEACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKSCALEACTION_deps_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->deps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSCALEACTION_deps_get) {
  {
    struct _dwg_object_BLOCKSCALEACTION *arg1 = (struct _dwg_object_BLOCKSCALEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSCALEACTION_deps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSCALEACTION_deps_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSCALEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSCALEACTION *)(argp1);
    result = (BITCODE_H *) ((arg1)->deps);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSCALEACTION_offset_set) {
  {
    struct _dwg_object_BLOCKSCALEACTION *arg1 = (struct _dwg_object_BLOCKSCALEACTION *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSCALEACTION_offset_set(self,offset);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSCALEACTION_offset_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSCALEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSCALEACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKSCALEACTION_offset_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->offset = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSCALEACTION_offset_get) {
  {
    struct _dwg_object_BLOCKSCALEACTION *arg1 = (struct _dwg_object_BLOCKSCALEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSCALEACTION_offset_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSCALEACTION_offset_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSCALEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSCALEACTION *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->offset);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSCALEACTION_conn_pts_set) {
  {
    struct _dwg_object_BLOCKSCALEACTION *arg1 = (struct _dwg_object_BLOCKSCALEACTION *) 0 ;
    Dwg_BLOCKACTION_connectionpts *arg2 = (Dwg_BLOCKACTION_connectionpts *) (Dwg_BLOCKACTION_connectionpts *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSCALEACTION_conn_pts_set(self,conn_pts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSCALEACTION_conn_pts_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSCALEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSCALEACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKACTION_connectionpts, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKSCALEACTION_conn_pts_set" "', argument " "2"" of type '" "Dwg_BLOCKACTION_connectionpts [5]""'"); 
    } 
    arg2 = (Dwg_BLOCKACTION_connectionpts *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)5; ++ii) *(Dwg_BLOCKACTION_connectionpts *)&arg1->conn_pts[ii] = *((Dwg_BLOCKACTION_connectionpts *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""conn_pts""' of type '""Dwg_BLOCKACTION_connectionpts [5]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSCALEACTION_conn_pts_get) {
  {
    struct _dwg_object_BLOCKSCALEACTION *arg1 = (struct _dwg_object_BLOCKSCALEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKACTION_connectionpts *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSCALEACTION_conn_pts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSCALEACTION_conn_pts_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSCALEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSCALEACTION *)(argp1);
    result = (Dwg_BLOCKACTION_connectionpts *)(Dwg_BLOCKACTION_connectionpts *) ((arg1)->conn_pts);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKACTION_connectionpts, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSCALEACTION_dependent_set) {
  {
    struct _dwg_object_BLOCKSCALEACTION *arg1 = (struct _dwg_object_BLOCKSCALEACTION *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSCALEACTION_dependent_set(self,dependent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSCALEACTION_dependent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSCALEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSCALEACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKSCALEACTION_dependent_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->dependent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSCALEACTION_dependent_get) {
  {
    struct _dwg_object_BLOCKSCALEACTION *arg1 = (struct _dwg_object_BLOCKSCALEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSCALEACTION_dependent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSCALEACTION_dependent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSCALEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSCALEACTION *)(argp1);
    result = (BITCODE_B) ((arg1)->dependent);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSCALEACTION_base_pt_set) {
  {
    struct _dwg_object_BLOCKSCALEACTION *arg1 = (struct _dwg_object_BLOCKSCALEACTION *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSCALEACTION_base_pt_set(self,base_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSCALEACTION_base_pt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSCALEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSCALEACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKSCALEACTION_base_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->base_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSCALEACTION_base_pt_get) {
  {
    struct _dwg_object_BLOCKSCALEACTION *arg1 = (struct _dwg_object_BLOCKSCALEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSCALEACTION_base_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSCALEACTION_base_pt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSCALEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSCALEACTION *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->base_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKSCALEACTION) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKSCALEACTION *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKSCALEACTION();");
    }
    result = (struct _dwg_object_BLOCKSCALEACTION *)calloc(1, sizeof(struct _dwg_object_BLOCKSCALEACTION));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKSCALEACTION) {
  {
    struct _dwg_object_BLOCKSCALEACTION *arg1 = (struct _dwg_object_BLOCKSCALEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKSCALEACTION(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKSCALEACTION" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSCALEACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSCALEACTION *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKSTRETCHACTION_handles_parent_set) {
  {
    struct _dwg_BLOCKSTRETCHACTION_handles *arg1 = (struct _dwg_BLOCKSTRETCHACTION_handles *) 0 ;
    struct _dwg_object_BLOCKSTRETCHACTION *arg2 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_BLOCKSTRETCHACTION_handles_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKSTRETCHACTION_handles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKSTRETCHACTION_handles_parent_set" "', argument " "1"" of type '" "struct _dwg_BLOCKSTRETCHACTION_handles *""'"); 
    }
    arg1 = (struct _dwg_BLOCKSTRETCHACTION_handles *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_BLOCKSTRETCHACTION_handles_parent_set" "', argument " "2"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg2 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKSTRETCHACTION_handles_parent_get) {
  {
    struct _dwg_BLOCKSTRETCHACTION_handles *arg1 = (struct _dwg_BLOCKSTRETCHACTION_handles *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_BLOCKSTRETCHACTION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_BLOCKSTRETCHACTION_handles_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKSTRETCHACTION_handles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKSTRETCHACTION_handles_parent_get" "', argument " "1"" of type '" "struct _dwg_BLOCKSTRETCHACTION_handles *""'"); 
    }
    arg1 = (struct _dwg_BLOCKSTRETCHACTION_handles *)(argp1);
    result = (struct _dwg_object_BLOCKSTRETCHACTION *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKSTRETCHACTION_handles_hdl_set) {
  {
    struct _dwg_BLOCKSTRETCHACTION_handles *arg1 = (struct _dwg_BLOCKSTRETCHACTION_handles *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_BLOCKSTRETCHACTION_handles_hdl_set(self,hdl);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKSTRETCHACTION_handles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKSTRETCHACTION_handles_hdl_set" "', argument " "1"" of type '" "struct _dwg_BLOCKSTRETCHACTION_handles *""'"); 
    }
    arg1 = (struct _dwg_BLOCKSTRETCHACTION_handles *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_BLOCKSTRETCHACTION_handles_hdl_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->hdl = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKSTRETCHACTION_handles_hdl_get) {
  {
    struct _dwg_BLOCKSTRETCHACTION_handles *arg1 = (struct _dwg_BLOCKSTRETCHACTION_handles *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_BLOCKSTRETCHACTION_handles_hdl_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKSTRETCHACTION_handles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKSTRETCHACTION_handles_hdl_get" "', argument " "1"" of type '" "struct _dwg_BLOCKSTRETCHACTION_handles *""'"); 
    }
    arg1 = (struct _dwg_BLOCKSTRETCHACTION_handles *)(argp1);
    result = (BITCODE_H) ((arg1)->hdl);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKSTRETCHACTION_handles_num_indexes_set) {
  {
    struct _dwg_BLOCKSTRETCHACTION_handles *arg1 = (struct _dwg_BLOCKSTRETCHACTION_handles *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_BLOCKSTRETCHACTION_handles_num_indexes_set(self,num_indexes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKSTRETCHACTION_handles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKSTRETCHACTION_handles_num_indexes_set" "', argument " "1"" of type '" "struct _dwg_BLOCKSTRETCHACTION_handles *""'"); 
    }
    arg1 = (struct _dwg_BLOCKSTRETCHACTION_handles *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_BLOCKSTRETCHACTION_handles_num_indexes_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_indexes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKSTRETCHACTION_handles_num_indexes_get) {
  {
    struct _dwg_BLOCKSTRETCHACTION_handles *arg1 = (struct _dwg_BLOCKSTRETCHACTION_handles *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_BLOCKSTRETCHACTION_handles_num_indexes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKSTRETCHACTION_handles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKSTRETCHACTION_handles_num_indexes_get" "', argument " "1"" of type '" "struct _dwg_BLOCKSTRETCHACTION_handles *""'"); 
    }
    arg1 = (struct _dwg_BLOCKSTRETCHACTION_handles *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_indexes);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKSTRETCHACTION_handles_indexes_set) {
  {
    struct _dwg_BLOCKSTRETCHACTION_handles *arg1 = (struct _dwg_BLOCKSTRETCHACTION_handles *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_BLOCKSTRETCHACTION_handles_indexes_set(self,indexes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKSTRETCHACTION_handles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKSTRETCHACTION_handles_indexes_set" "', argument " "1"" of type '" "struct _dwg_BLOCKSTRETCHACTION_handles *""'"); 
    }
    arg1 = (struct _dwg_BLOCKSTRETCHACTION_handles *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_BLOCKSTRETCHACTION_handles_indexes_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->indexes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKSTRETCHACTION_handles_indexes_get) {
  {
    struct _dwg_BLOCKSTRETCHACTION_handles *arg1 = (struct _dwg_BLOCKSTRETCHACTION_handles *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_BLOCKSTRETCHACTION_handles_indexes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKSTRETCHACTION_handles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKSTRETCHACTION_handles_indexes_get" "', argument " "1"" of type '" "struct _dwg_BLOCKSTRETCHACTION_handles *""'"); 
    }
    arg1 = (struct _dwg_BLOCKSTRETCHACTION_handles *)(argp1);
    result = (BITCODE_BL *) ((arg1)->indexes);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_BLOCKSTRETCHACTION_handles) {
  {
    int argvi = 0;
    struct _dwg_BLOCKSTRETCHACTION_handles *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_BLOCKSTRETCHACTION_handles();");
    }
    result = (struct _dwg_BLOCKSTRETCHACTION_handles *)calloc(1, sizeof(struct _dwg_BLOCKSTRETCHACTION_handles));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKSTRETCHACTION_handles, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_BLOCKSTRETCHACTION_handles) {
  {
    struct _dwg_BLOCKSTRETCHACTION_handles *arg1 = (struct _dwg_BLOCKSTRETCHACTION_handles *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_BLOCKSTRETCHACTION_handles(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKSTRETCHACTION_handles, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_BLOCKSTRETCHACTION_handles" "', argument " "1"" of type '" "struct _dwg_BLOCKSTRETCHACTION_handles *""'"); 
    }
    arg1 = (struct _dwg_BLOCKSTRETCHACTION_handles *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKSTRETCHACTION_codes_parent_set) {
  {
    struct _dwg_BLOCKSTRETCHACTION_codes *arg1 = (struct _dwg_BLOCKSTRETCHACTION_codes *) 0 ;
    struct _dwg_object_BLOCKSTRETCHACTION *arg2 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_BLOCKSTRETCHACTION_codes_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKSTRETCHACTION_codes, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKSTRETCHACTION_codes_parent_set" "', argument " "1"" of type '" "struct _dwg_BLOCKSTRETCHACTION_codes *""'"); 
    }
    arg1 = (struct _dwg_BLOCKSTRETCHACTION_codes *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_BLOCKSTRETCHACTION_codes_parent_set" "', argument " "2"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg2 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKSTRETCHACTION_codes_parent_get) {
  {
    struct _dwg_BLOCKSTRETCHACTION_codes *arg1 = (struct _dwg_BLOCKSTRETCHACTION_codes *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_BLOCKSTRETCHACTION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_BLOCKSTRETCHACTION_codes_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKSTRETCHACTION_codes, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKSTRETCHACTION_codes_parent_get" "', argument " "1"" of type '" "struct _dwg_BLOCKSTRETCHACTION_codes *""'"); 
    }
    arg1 = (struct _dwg_BLOCKSTRETCHACTION_codes *)(argp1);
    result = (struct _dwg_object_BLOCKSTRETCHACTION *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKSTRETCHACTION_codes_bl95_set) {
  {
    struct _dwg_BLOCKSTRETCHACTION_codes *arg1 = (struct _dwg_BLOCKSTRETCHACTION_codes *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_BLOCKSTRETCHACTION_codes_bl95_set(self,bl95);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKSTRETCHACTION_codes, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKSTRETCHACTION_codes_bl95_set" "', argument " "1"" of type '" "struct _dwg_BLOCKSTRETCHACTION_codes *""'"); 
    }
    arg1 = (struct _dwg_BLOCKSTRETCHACTION_codes *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_BLOCKSTRETCHACTION_codes_bl95_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bl95 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKSTRETCHACTION_codes_bl95_get) {
  {
    struct _dwg_BLOCKSTRETCHACTION_codes *arg1 = (struct _dwg_BLOCKSTRETCHACTION_codes *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_BLOCKSTRETCHACTION_codes_bl95_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKSTRETCHACTION_codes, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKSTRETCHACTION_codes_bl95_get" "', argument " "1"" of type '" "struct _dwg_BLOCKSTRETCHACTION_codes *""'"); 
    }
    arg1 = (struct _dwg_BLOCKSTRETCHACTION_codes *)(argp1);
    result = (BITCODE_BL) ((arg1)->bl95);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKSTRETCHACTION_codes_num_indexes_set) {
  {
    struct _dwg_BLOCKSTRETCHACTION_codes *arg1 = (struct _dwg_BLOCKSTRETCHACTION_codes *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_BLOCKSTRETCHACTION_codes_num_indexes_set(self,num_indexes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKSTRETCHACTION_codes, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKSTRETCHACTION_codes_num_indexes_set" "', argument " "1"" of type '" "struct _dwg_BLOCKSTRETCHACTION_codes *""'"); 
    }
    arg1 = (struct _dwg_BLOCKSTRETCHACTION_codes *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_BLOCKSTRETCHACTION_codes_num_indexes_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_indexes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKSTRETCHACTION_codes_num_indexes_get) {
  {
    struct _dwg_BLOCKSTRETCHACTION_codes *arg1 = (struct _dwg_BLOCKSTRETCHACTION_codes *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_BLOCKSTRETCHACTION_codes_num_indexes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKSTRETCHACTION_codes, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKSTRETCHACTION_codes_num_indexes_get" "', argument " "1"" of type '" "struct _dwg_BLOCKSTRETCHACTION_codes *""'"); 
    }
    arg1 = (struct _dwg_BLOCKSTRETCHACTION_codes *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_indexes);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKSTRETCHACTION_codes_indexes_set) {
  {
    struct _dwg_BLOCKSTRETCHACTION_codes *arg1 = (struct _dwg_BLOCKSTRETCHACTION_codes *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_BLOCKSTRETCHACTION_codes_indexes_set(self,indexes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKSTRETCHACTION_codes, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKSTRETCHACTION_codes_indexes_set" "', argument " "1"" of type '" "struct _dwg_BLOCKSTRETCHACTION_codes *""'"); 
    }
    arg1 = (struct _dwg_BLOCKSTRETCHACTION_codes *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_BLOCKSTRETCHACTION_codes_indexes_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->indexes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_BLOCKSTRETCHACTION_codes_indexes_get) {
  {
    struct _dwg_BLOCKSTRETCHACTION_codes *arg1 = (struct _dwg_BLOCKSTRETCHACTION_codes *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_BLOCKSTRETCHACTION_codes_indexes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKSTRETCHACTION_codes, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_BLOCKSTRETCHACTION_codes_indexes_get" "', argument " "1"" of type '" "struct _dwg_BLOCKSTRETCHACTION_codes *""'"); 
    }
    arg1 = (struct _dwg_BLOCKSTRETCHACTION_codes *)(argp1);
    result = (BITCODE_BL *) ((arg1)->indexes);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_BLOCKSTRETCHACTION_codes) {
  {
    int argvi = 0;
    struct _dwg_BLOCKSTRETCHACTION_codes *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_BLOCKSTRETCHACTION_codes();");
    }
    result = (struct _dwg_BLOCKSTRETCHACTION_codes *)calloc(1, sizeof(struct _dwg_BLOCKSTRETCHACTION_codes));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKSTRETCHACTION_codes, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_BLOCKSTRETCHACTION_codes) {
  {
    struct _dwg_BLOCKSTRETCHACTION_codes *arg1 = (struct _dwg_BLOCKSTRETCHACTION_codes *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_BLOCKSTRETCHACTION_codes(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_BLOCKSTRETCHACTION_codes, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_BLOCKSTRETCHACTION_codes" "', argument " "1"" of type '" "struct _dwg_BLOCKSTRETCHACTION_codes *""'"); 
    }
    arg1 = (struct _dwg_BLOCKSTRETCHACTION_codes *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_parent_set) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_parent_get) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_evalexpr_set) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_evalexpr_get) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_name_set) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_name_get) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_be_major_set) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_be_major_set(self,be_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_be_major_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_be_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_be_major_get) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_be_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_be_major_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_be_minor_set) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_be_minor_set(self,be_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_be_minor_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_be_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_be_minor_get) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_be_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_be_minor_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_eed1071_set) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_eed1071_set(self,eed1071);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_eed1071_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_eed1071_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->eed1071 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_eed1071_get) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_eed1071_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_eed1071_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->eed1071);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_display_location_set) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_display_location_set(self,display_location);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_display_location_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_display_location_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->display_location = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_display_location_get) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_display_location_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_display_location_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->display_location);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_num_actions_set) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_num_actions_set(self,num_actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_num_actions_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_num_actions_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_num_actions_get) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_num_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_num_actions_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_actions);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_actions_set) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_actions_set(self,actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_actions_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_actions_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_actions_get) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_actions_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    result = (BITCODE_BL *) ((arg1)->actions);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_num_deps_set) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_num_deps_set(self,num_deps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_num_deps_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_num_deps_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_deps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_num_deps_get) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_num_deps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_num_deps_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_deps);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_deps_set) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_deps_set(self,deps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_deps_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_deps_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->deps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_deps_get) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_deps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_deps_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    result = (BITCODE_H *) ((arg1)->deps);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_conn_pts_set) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    Dwg_BLOCKACTION_connectionpts *arg2 = (Dwg_BLOCKACTION_connectionpts *) (Dwg_BLOCKACTION_connectionpts *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_conn_pts_set(self,conn_pts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_conn_pts_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKACTION_connectionpts, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_conn_pts_set" "', argument " "2"" of type '" "Dwg_BLOCKACTION_connectionpts [2]""'"); 
    } 
    arg2 = (Dwg_BLOCKACTION_connectionpts *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)2; ++ii) *(Dwg_BLOCKACTION_connectionpts *)&arg1->conn_pts[ii] = *((Dwg_BLOCKACTION_connectionpts *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""conn_pts""' of type '""Dwg_BLOCKACTION_connectionpts [2]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_conn_pts_get) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKACTION_connectionpts *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_conn_pts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_conn_pts_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    result = (Dwg_BLOCKACTION_connectionpts *)(Dwg_BLOCKACTION_connectionpts *) ((arg1)->conn_pts);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKACTION_connectionpts, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_num_pts_set) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_num_pts_set(self,num_pts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_num_pts_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_num_pts_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_pts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_num_pts_get) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_num_pts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_num_pts_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_pts);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_pts_set) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_pts_set(self,pts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_pts_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_pts_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->pts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_pts_get) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_pts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_pts_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    result = (BITCODE_2RD *) ((arg1)->pts);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_num_hdls_set) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_num_hdls_set(self,num_hdls);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_num_hdls_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_num_hdls_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_hdls = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_num_hdls_get) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_num_hdls_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_num_hdls_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_hdls);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_hdls_set) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    Dwg_BLOCKSTRETCHACTION_handles *arg2 = (Dwg_BLOCKSTRETCHACTION_handles *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_hdls_set(self,hdls);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_hdls_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKSTRETCHACTION_handles, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_hdls_set" "', argument " "2"" of type '" "Dwg_BLOCKSTRETCHACTION_handles *""'"); 
    }
    arg2 = (Dwg_BLOCKSTRETCHACTION_handles *)(argp2);
    if (arg1) (arg1)->hdls = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_hdls_get) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKSTRETCHACTION_handles *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_hdls_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_hdls_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    result = (Dwg_BLOCKSTRETCHACTION_handles *) ((arg1)->hdls);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKSTRETCHACTION_handles, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_num_codes_set) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_num_codes_set(self,num_codes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_num_codes_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_num_codes_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_codes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_num_codes_get) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_num_codes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_num_codes_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_codes);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_codes_set) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    Dwg_BLOCKSTRETCHACTION_codes *arg2 = (Dwg_BLOCKSTRETCHACTION_codes *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_codes_set(self,codes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_codes_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKSTRETCHACTION_codes, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_codes_set" "', argument " "2"" of type '" "Dwg_BLOCKSTRETCHACTION_codes *""'"); 
    }
    arg2 = (Dwg_BLOCKSTRETCHACTION_codes *)(argp2);
    if (arg1) (arg1)->codes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_codes_get) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKSTRETCHACTION_codes *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_codes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_codes_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    result = (Dwg_BLOCKSTRETCHACTION_codes *) ((arg1)->codes);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKSTRETCHACTION_codes, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_action_offset_x_set) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_action_offset_x_set(self,action_offset_x);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_action_offset_x_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_action_offset_x_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->action_offset_x = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_action_offset_x_get) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_action_offset_x_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_action_offset_x_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    result = (BITCODE_BD) ((arg1)->action_offset_x);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_action_offset_y_set) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_action_offset_y_set(self,action_offset_y);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_action_offset_y_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_action_offset_y_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->action_offset_y = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_action_offset_y_get) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_action_offset_y_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_action_offset_y_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    result = (BITCODE_BD) ((arg1)->action_offset_y);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_angle_offset_set) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_angle_offset_set(self,angle_offset);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_angle_offset_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_angle_offset_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->angle_offset = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKSTRETCHACTION_angle_offset_get) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKSTRETCHACTION_angle_offset_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKSTRETCHACTION_angle_offset_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    result = (BITCODE_BD) ((arg1)->angle_offset);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKSTRETCHACTION) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKSTRETCHACTION *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKSTRETCHACTION();");
    }
    result = (struct _dwg_object_BLOCKSTRETCHACTION *)calloc(1, sizeof(struct _dwg_object_BLOCKSTRETCHACTION));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKSTRETCHACTION) {
  {
    struct _dwg_object_BLOCKSTRETCHACTION *arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKSTRETCHACTION(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKSTRETCHACTION" "', argument " "1"" of type '" "struct _dwg_object_BLOCKSTRETCHACTION *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKSTRETCHACTION *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKUSERPARAMETER_parent_set) {
  {
    struct _dwg_object_BLOCKUSERPARAMETER *arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKUSERPARAMETER_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKUSERPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKUSERPARAMETER_parent_get) {
  {
    struct _dwg_object_BLOCKUSERPARAMETER *arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKUSERPARAMETER_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKUSERPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKUSERPARAMETER_evalexpr_set) {
  {
    struct _dwg_object_BLOCKUSERPARAMETER *arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKUSERPARAMETER_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKUSERPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKUSERPARAMETER_evalexpr_get) {
  {
    struct _dwg_object_BLOCKUSERPARAMETER *arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKUSERPARAMETER_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKUSERPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKUSERPARAMETER_name_set) {
  {
    struct _dwg_object_BLOCKUSERPARAMETER *arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKUSERPARAMETER_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKUSERPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKUSERPARAMETER_name_get) {
  {
    struct _dwg_object_BLOCKUSERPARAMETER *arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKUSERPARAMETER_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKUSERPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKUSERPARAMETER_be_major_set) {
  {
    struct _dwg_object_BLOCKUSERPARAMETER *arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKUSERPARAMETER_be_major_set(self,be_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_be_major_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKUSERPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_be_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKUSERPARAMETER_be_major_get) {
  {
    struct _dwg_object_BLOCKUSERPARAMETER *arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKUSERPARAMETER_be_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_be_major_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKUSERPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKUSERPARAMETER_be_minor_set) {
  {
    struct _dwg_object_BLOCKUSERPARAMETER *arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKUSERPARAMETER_be_minor_set(self,be_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_be_minor_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKUSERPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_be_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKUSERPARAMETER_be_minor_get) {
  {
    struct _dwg_object_BLOCKUSERPARAMETER *arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKUSERPARAMETER_be_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_be_minor_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKUSERPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKUSERPARAMETER_eed1071_set) {
  {
    struct _dwg_object_BLOCKUSERPARAMETER *arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKUSERPARAMETER_eed1071_set(self,eed1071);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_eed1071_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKUSERPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_eed1071_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->eed1071 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKUSERPARAMETER_eed1071_get) {
  {
    struct _dwg_object_BLOCKUSERPARAMETER *arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKUSERPARAMETER_eed1071_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_eed1071_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKUSERPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->eed1071);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKUSERPARAMETER_show_properties_set) {
  {
    struct _dwg_object_BLOCKUSERPARAMETER *arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKUSERPARAMETER_show_properties_set(self,show_properties);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_show_properties_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKUSERPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_show_properties_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->show_properties = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKUSERPARAMETER_show_properties_get) {
  {
    struct _dwg_object_BLOCKUSERPARAMETER *arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKUSERPARAMETER_show_properties_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_show_properties_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKUSERPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->show_properties);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKUSERPARAMETER_chain_actions_set) {
  {
    struct _dwg_object_BLOCKUSERPARAMETER *arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKUSERPARAMETER_chain_actions_set(self,chain_actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_chain_actions_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKUSERPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_chain_actions_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->chain_actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKUSERPARAMETER_chain_actions_get) {
  {
    struct _dwg_object_BLOCKUSERPARAMETER *arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKUSERPARAMETER_chain_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_chain_actions_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKUSERPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->chain_actions);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKUSERPARAMETER_def_pt_set) {
  {
    struct _dwg_object_BLOCKUSERPARAMETER *arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKUSERPARAMETER_def_pt_set(self,def_pt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_def_pt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKUSERPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_def_pt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_pt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKUSERPARAMETER_def_pt_get) {
  {
    struct _dwg_object_BLOCKUSERPARAMETER *arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKUSERPARAMETER_def_pt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_def_pt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKUSERPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_pt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKUSERPARAMETER_num_propinfos_set) {
  {
    struct _dwg_object_BLOCKUSERPARAMETER *arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKUSERPARAMETER_num_propinfos_set(self,num_propinfos);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_num_propinfos_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKUSERPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_num_propinfos_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_propinfos = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKUSERPARAMETER_num_propinfos_get) {
  {
    struct _dwg_object_BLOCKUSERPARAMETER *arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKUSERPARAMETER_num_propinfos_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_num_propinfos_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKUSERPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_propinfos);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKUSERPARAMETER_prop1_set) {
  {
    struct _dwg_object_BLOCKUSERPARAMETER *arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKUSERPARAMETER_prop1_set(self,prop1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_prop1_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKUSERPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_prop1_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop1 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKUSERPARAMETER_prop1_get) {
  {
    struct _dwg_object_BLOCKUSERPARAMETER *arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKUSERPARAMETER_prop1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_prop1_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKUSERPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKUSERPARAMETER_prop2_set) {
  {
    struct _dwg_object_BLOCKUSERPARAMETER *arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKUSERPARAMETER_prop2_set(self,prop2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_prop2_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKUSERPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_prop2_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop2 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKUSERPARAMETER_prop2_get) {
  {
    struct _dwg_object_BLOCKUSERPARAMETER *arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKUSERPARAMETER_prop2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_prop2_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKUSERPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKUSERPARAMETER_flag_set) {
  {
    struct _dwg_object_BLOCKUSERPARAMETER *arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKUSERPARAMETER_flag_set(self,flag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_flag_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKUSERPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_flag_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->flag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKUSERPARAMETER_flag_get) {
  {
    struct _dwg_object_BLOCKUSERPARAMETER *arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKUSERPARAMETER_flag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_flag_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKUSERPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *)(argp1);
    result = (BITCODE_BS) ((arg1)->flag);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKUSERPARAMETER_assocvariable_set) {
  {
    struct _dwg_object_BLOCKUSERPARAMETER *arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKUSERPARAMETER_assocvariable_set(self,assocvariable);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_assocvariable_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKUSERPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_assocvariable_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->assocvariable = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKUSERPARAMETER_assocvariable_get) {
  {
    struct _dwg_object_BLOCKUSERPARAMETER *arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKUSERPARAMETER_assocvariable_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_assocvariable_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKUSERPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *)(argp1);
    result = (BITCODE_H) ((arg1)->assocvariable);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKUSERPARAMETER_expr_set) {
  {
    struct _dwg_object_BLOCKUSERPARAMETER *arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKUSERPARAMETER_expr_set(self,expr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_expr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKUSERPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_expr_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->expr);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->expr = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->expr = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKUSERPARAMETER_expr_get) {
  {
    struct _dwg_object_BLOCKUSERPARAMETER *arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKUSERPARAMETER_expr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_expr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKUSERPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->expr);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKUSERPARAMETER_value_set) {
  {
    struct _dwg_object_BLOCKUSERPARAMETER *arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *) 0 ;
    Dwg_EvalVariant *arg2 = (Dwg_EvalVariant *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKUSERPARAMETER_value_set(self,value);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_value_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKUSERPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalVariant, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_value_set" "', argument " "2"" of type '" "Dwg_EvalVariant *""'"); 
    }
    arg2 = (Dwg_EvalVariant *)(argp2);
    if (arg1) (arg1)->value = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKUSERPARAMETER_value_get) {
  {
    struct _dwg_object_BLOCKUSERPARAMETER *arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalVariant *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKUSERPARAMETER_value_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_value_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKUSERPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *)(argp1);
    result = (Dwg_EvalVariant *)& ((arg1)->value);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalVariant, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKUSERPARAMETER_type_set) {
  {
    struct _dwg_object_BLOCKUSERPARAMETER *arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKUSERPARAMETER_type_set(self,type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_type_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKUSERPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKUSERPARAMETER_type_get) {
  {
    struct _dwg_object_BLOCKUSERPARAMETER *arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKUSERPARAMETER_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKUSERPARAMETER_type_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKUSERPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *)(argp1);
    result = (BITCODE_BS) ((arg1)->type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKUSERPARAMETER) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKUSERPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKUSERPARAMETER();");
    }
    result = (struct _dwg_object_BLOCKUSERPARAMETER *)calloc(1, sizeof(struct _dwg_object_BLOCKUSERPARAMETER));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKUSERPARAMETER) {
  {
    struct _dwg_object_BLOCKUSERPARAMETER *arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKUSERPARAMETER(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKUSERPARAMETER" "', argument " "1"" of type '" "struct _dwg_object_BLOCKUSERPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKUSERPARAMETER *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYGRIP_parent_set) {
  {
    struct _dwg_object_BLOCKXYGRIP *arg1 = (struct _dwg_object_BLOCKXYGRIP *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYGRIP_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYGRIP_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYGRIP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKXYGRIP_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYGRIP_parent_get) {
  {
    struct _dwg_object_BLOCKXYGRIP *arg1 = (struct _dwg_object_BLOCKXYGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYGRIP_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYGRIP_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYGRIP *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYGRIP_evalexpr_set) {
  {
    struct _dwg_object_BLOCKXYGRIP *arg1 = (struct _dwg_object_BLOCKXYGRIP *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYGRIP_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYGRIP_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYGRIP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKXYGRIP_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYGRIP_evalexpr_get) {
  {
    struct _dwg_object_BLOCKXYGRIP *arg1 = (struct _dwg_object_BLOCKXYGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYGRIP_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYGRIP_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYGRIP *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYGRIP_name_set) {
  {
    struct _dwg_object_BLOCKXYGRIP *arg1 = (struct _dwg_object_BLOCKXYGRIP *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYGRIP_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYGRIP_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYGRIP *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKXYGRIP_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYGRIP_name_get) {
  {
    struct _dwg_object_BLOCKXYGRIP *arg1 = (struct _dwg_object_BLOCKXYGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYGRIP_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYGRIP_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYGRIP *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYGRIP_be_major_set) {
  {
    struct _dwg_object_BLOCKXYGRIP *arg1 = (struct _dwg_object_BLOCKXYGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYGRIP_be_major_set(self,be_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYGRIP_be_major_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKXYGRIP_be_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYGRIP_be_major_get) {
  {
    struct _dwg_object_BLOCKXYGRIP *arg1 = (struct _dwg_object_BLOCKXYGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYGRIP_be_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYGRIP_be_major_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYGRIP_be_minor_set) {
  {
    struct _dwg_object_BLOCKXYGRIP *arg1 = (struct _dwg_object_BLOCKXYGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYGRIP_be_minor_set(self,be_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYGRIP_be_minor_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKXYGRIP_be_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYGRIP_be_minor_get) {
  {
    struct _dwg_object_BLOCKXYGRIP *arg1 = (struct _dwg_object_BLOCKXYGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYGRIP_be_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYGRIP_be_minor_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYGRIP_eed1071_set) {
  {
    struct _dwg_object_BLOCKXYGRIP *arg1 = (struct _dwg_object_BLOCKXYGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYGRIP_eed1071_set(self,eed1071);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYGRIP_eed1071_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKXYGRIP_eed1071_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->eed1071 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYGRIP_eed1071_get) {
  {
    struct _dwg_object_BLOCKXYGRIP *arg1 = (struct _dwg_object_BLOCKXYGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYGRIP_eed1071_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYGRIP_eed1071_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->eed1071);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYGRIP_bg_bl91_set) {
  {
    struct _dwg_object_BLOCKXYGRIP *arg1 = (struct _dwg_object_BLOCKXYGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYGRIP_bg_bl91_set(self,bg_bl91);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYGRIP_bg_bl91_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKXYGRIP_bg_bl91_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bg_bl91 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYGRIP_bg_bl91_get) {
  {
    struct _dwg_object_BLOCKXYGRIP *arg1 = (struct _dwg_object_BLOCKXYGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYGRIP_bg_bl91_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYGRIP_bg_bl91_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->bg_bl91);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYGRIP_bg_bl92_set) {
  {
    struct _dwg_object_BLOCKXYGRIP *arg1 = (struct _dwg_object_BLOCKXYGRIP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYGRIP_bg_bl92_set(self,bg_bl92);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYGRIP_bg_bl92_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKXYGRIP_bg_bl92_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->bg_bl92 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYGRIP_bg_bl92_get) {
  {
    struct _dwg_object_BLOCKXYGRIP *arg1 = (struct _dwg_object_BLOCKXYGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYGRIP_bg_bl92_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYGRIP_bg_bl92_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYGRIP *)(argp1);
    result = (BITCODE_BL) ((arg1)->bg_bl92);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYGRIP_bg_location_set) {
  {
    struct _dwg_object_BLOCKXYGRIP *arg1 = (struct _dwg_object_BLOCKXYGRIP *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYGRIP_bg_location_set(self,bg_location);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYGRIP_bg_location_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYGRIP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKXYGRIP_bg_location_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->bg_location = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYGRIP_bg_location_get) {
  {
    struct _dwg_object_BLOCKXYGRIP *arg1 = (struct _dwg_object_BLOCKXYGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYGRIP_bg_location_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYGRIP_bg_location_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYGRIP *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->bg_location);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYGRIP_bg_insert_cycling_set) {
  {
    struct _dwg_object_BLOCKXYGRIP *arg1 = (struct _dwg_object_BLOCKXYGRIP *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYGRIP_bg_insert_cycling_set(self,bg_insert_cycling);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYGRIP_bg_insert_cycling_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYGRIP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKXYGRIP_bg_insert_cycling_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->bg_insert_cycling = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYGRIP_bg_insert_cycling_get) {
  {
    struct _dwg_object_BLOCKXYGRIP *arg1 = (struct _dwg_object_BLOCKXYGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYGRIP_bg_insert_cycling_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYGRIP_bg_insert_cycling_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYGRIP *)(argp1);
    result = (BITCODE_B) ((arg1)->bg_insert_cycling);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYGRIP_bg_insert_cycling_weight_set) {
  {
    struct _dwg_object_BLOCKXYGRIP *arg1 = (struct _dwg_object_BLOCKXYGRIP *) 0 ;
    BITCODE_BLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYGRIP_bg_insert_cycling_weight_set(self,bg_insert_cycling_weight);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYGRIP_bg_insert_cycling_weight_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYGRIP *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKXYGRIP_bg_insert_cycling_weight_set" "', argument " "2"" of type '" "BITCODE_BLd""'");
    } 
    arg2 = (BITCODE_BLd)(val2);
    if (arg1) (arg1)->bg_insert_cycling_weight = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYGRIP_bg_insert_cycling_weight_get) {
  {
    struct _dwg_object_BLOCKXYGRIP *arg1 = (struct _dwg_object_BLOCKXYGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYGRIP_bg_insert_cycling_weight_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYGRIP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYGRIP_bg_insert_cycling_weight_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYGRIP *)(argp1);
    result = (BITCODE_BLd) ((arg1)->bg_insert_cycling_weight);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKXYGRIP) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKXYGRIP *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKXYGRIP();");
    }
    result = (struct _dwg_object_BLOCKXYGRIP *)calloc(1, sizeof(struct _dwg_object_BLOCKXYGRIP));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKXYGRIP, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKXYGRIP) {
  {
    struct _dwg_object_BLOCKXYGRIP *arg1 = (struct _dwg_object_BLOCKXYGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKXYGRIP(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYGRIP, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKXYGRIP" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYGRIP *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYGRIP *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_parent_set) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_parent_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKXYPARAMETER_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_parent_get) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_parent_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_evalexpr_set) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKXYPARAMETER_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_evalexpr_get) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_name_set) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_name_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKXYPARAMETER_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_name_get) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_name_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_be_major_set) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_be_major_set(self,be_major);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_be_major_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKXYPARAMETER_be_major_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_major = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_be_major_get) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_be_major_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_be_major_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_major);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_be_minor_set) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_be_minor_set(self,be_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_be_minor_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKXYPARAMETER_be_minor_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->be_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_be_minor_get) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_be_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_be_minor_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->be_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_eed1071_set) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_eed1071_set(self,eed1071);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_eed1071_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKXYPARAMETER_eed1071_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->eed1071 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_eed1071_get) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_eed1071_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_eed1071_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    result = (BITCODE_BL) ((arg1)->eed1071);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_show_properties_set) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_show_properties_set(self,show_properties);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_show_properties_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKXYPARAMETER_show_properties_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->show_properties = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_show_properties_get) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_show_properties_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_show_properties_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->show_properties);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_chain_actions_set) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_chain_actions_set(self,chain_actions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_chain_actions_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKXYPARAMETER_chain_actions_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->chain_actions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_chain_actions_get) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_chain_actions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_chain_actions_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    result = (BITCODE_B) ((arg1)->chain_actions);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_def_basept_set) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_def_basept_set(self,def_basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_def_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKXYPARAMETER_def_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_def_basept_get) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_def_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_def_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_def_endpt_set) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_def_endpt_set(self,def_endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_def_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKXYPARAMETER_def_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->def_endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_def_endpt_get) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_def_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_def_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->def_endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_prop1_set) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_prop1_set(self,prop1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_prop1_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKXYPARAMETER_prop1_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop1 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_prop1_get) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_prop1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_prop1_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_prop2_set) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_prop2_set(self,prop2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_prop2_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKXYPARAMETER_prop2_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop2 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_prop2_get) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_prop2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_prop2_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_prop3_set) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_prop3_set(self,prop3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_prop3_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKXYPARAMETER_prop3_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop3 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_prop3_get) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_prop3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_prop3_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop3);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_prop4_set) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    Dwg_BLOCKPARAMETER_PropInfo *arg2 = (Dwg_BLOCKPARAMETER_PropInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_prop4_set(self,prop4);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_prop4_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKXYPARAMETER_prop4_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMETER_PropInfo *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMETER_PropInfo *)(argp2);
    if (arg1) (arg1)->prop4 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_prop4_get) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMETER_PropInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_prop4_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_prop4_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMETER_PropInfo *)& ((arg1)->prop4);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_prop_states_set) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    BITCODE_BL *arg2 = (BITCODE_BL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_prop_states_set(self,prop_states);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_prop_states_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKXYPARAMETER_prop_states_set" "', argument " "2"" of type '" "BITCODE_BL *""'"); 
    }
    arg2 = (BITCODE_BL *)(argp2);
    if (arg1) (arg1)->prop_states = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_prop_states_get) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_prop_states_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_prop_states_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    result = (BITCODE_BL *) ((arg1)->prop_states);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_parameter_base_location_set) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_parameter_base_location_set(self,parameter_base_location);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_parameter_base_location_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKXYPARAMETER_parameter_base_location_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->parameter_base_location = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_parameter_base_location_get) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_parameter_base_location_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_parameter_base_location_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    result = (BITCODE_BS) ((arg1)->parameter_base_location);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_upd_basept_set) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_upd_basept_set(self,upd_basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_upd_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKXYPARAMETER_upd_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->upd_basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_upd_basept_get) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_upd_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_upd_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->upd_basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_basept_set) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_basept_set(self,basept);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_basept_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKXYPARAMETER_basept_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->basept = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_basept_get) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_basept_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_basept_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->basept);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_upd_endpt_set) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_upd_endpt_set(self,upd_endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_upd_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKXYPARAMETER_upd_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->upd_endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_upd_endpt_get) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_upd_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_upd_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->upd_endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_endpt_set) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_endpt_set(self,endpt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_endpt_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKXYPARAMETER_endpt_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->endpt = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_endpt_get) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_endpt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_endpt_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->endpt);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_x_label_set) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_x_label_set(self,x_label);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_x_label_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKXYPARAMETER_x_label_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->x_label);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->x_label = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->x_label = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_x_label_get) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_x_label_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_x_label_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->x_label);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_x_label_desc_set) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_x_label_desc_set(self,x_label_desc);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_x_label_desc_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKXYPARAMETER_x_label_desc_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->x_label_desc);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->x_label_desc = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->x_label_desc = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_x_label_desc_get) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_x_label_desc_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_x_label_desc_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->x_label_desc);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_y_label_set) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_y_label_set(self,y_label);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_y_label_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKXYPARAMETER_y_label_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->y_label);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->y_label = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->y_label = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_y_label_get) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_y_label_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_y_label_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->y_label);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_y_label_desc_set) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_y_label_desc_set(self,y_label_desc);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_y_label_desc_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKXYPARAMETER_y_label_desc_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->y_label_desc);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->y_label_desc = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->y_label_desc = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_y_label_desc_get) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_y_label_desc_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_y_label_desc_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    result = (BITCODE_TV) ((arg1)->y_label_desc);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_x_value_set) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_x_value_set(self,x_value);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_x_value_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKXYPARAMETER_x_value_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->x_value = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_x_value_get) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_x_value_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_x_value_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    result = (BITCODE_BD) ((arg1)->x_value);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_y_value_set) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_y_value_set(self,y_value);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_y_value_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_BLOCKXYPARAMETER_y_value_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->y_value = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_y_value_get) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_y_value_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_y_value_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    result = (BITCODE_BD) ((arg1)->y_value);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_x_value_set_set) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    Dwg_BLOCKPARAMVALUESET *arg2 = (Dwg_BLOCKPARAMVALUESET *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_x_value_set_set(self,x_value_set);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_x_value_set_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKXYPARAMETER_x_value_set_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMVALUESET *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMVALUESET *)(argp2);
    if (arg1) (arg1)->x_value_set = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_x_value_set_get) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMVALUESET *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_x_value_set_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_x_value_set_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMVALUESET *)& ((arg1)->x_value_set);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_y_value_set_set) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    Dwg_BLOCKPARAMVALUESET *arg2 = (Dwg_BLOCKPARAMVALUESET *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_y_value_set_set(self,y_value_set);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_y_value_set_set" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_BLOCKXYPARAMETER_y_value_set_set" "', argument " "2"" of type '" "Dwg_BLOCKPARAMVALUESET *""'"); 
    }
    arg2 = (Dwg_BLOCKPARAMVALUESET *)(argp2);
    if (arg1) (arg1)->y_value_set = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_BLOCKXYPARAMETER_y_value_set_get) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_BLOCKPARAMVALUESET *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_BLOCKXYPARAMETER_y_value_set_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_BLOCKXYPARAMETER_y_value_set_get" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    result = (Dwg_BLOCKPARAMVALUESET *)& ((arg1)->y_value_set);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_BLOCKXYPARAMETER) {
  {
    int argvi = 0;
    struct _dwg_object_BLOCKXYPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_BLOCKXYPARAMETER();");
    }
    result = (struct _dwg_object_BLOCKXYPARAMETER *)calloc(1, sizeof(struct _dwg_object_BLOCKXYPARAMETER));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_BLOCKXYPARAMETER) {
  {
    struct _dwg_object_BLOCKXYPARAMETER *arg1 = (struct _dwg_object_BLOCKXYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_BLOCKXYPARAMETER(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_BLOCKXYPARAMETER" "', argument " "1"" of type '" "struct _dwg_object_BLOCKXYPARAMETER *""'"); 
    }
    arg1 = (struct _dwg_object_BLOCKXYPARAMETER *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DYNAMICBLOCKPROXYNODE_parent_set) {
  {
    struct _dwg_object_DYNAMICBLOCKPROXYNODE *arg1 = (struct _dwg_object_DYNAMICBLOCKPROXYNODE *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DYNAMICBLOCKPROXYNODE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DYNAMICBLOCKPROXYNODE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DYNAMICBLOCKPROXYNODE_parent_set" "', argument " "1"" of type '" "struct _dwg_object_DYNAMICBLOCKPROXYNODE *""'"); 
    }
    arg1 = (struct _dwg_object_DYNAMICBLOCKPROXYNODE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DYNAMICBLOCKPROXYNODE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DYNAMICBLOCKPROXYNODE_parent_get) {
  {
    struct _dwg_object_DYNAMICBLOCKPROXYNODE *arg1 = (struct _dwg_object_DYNAMICBLOCKPROXYNODE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DYNAMICBLOCKPROXYNODE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DYNAMICBLOCKPROXYNODE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DYNAMICBLOCKPROXYNODE_parent_get" "', argument " "1"" of type '" "struct _dwg_object_DYNAMICBLOCKPROXYNODE *""'"); 
    }
    arg1 = (struct _dwg_object_DYNAMICBLOCKPROXYNODE *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DYNAMICBLOCKPROXYNODE_evalexpr_set) {
  {
    struct _dwg_object_DYNAMICBLOCKPROXYNODE *arg1 = (struct _dwg_object_DYNAMICBLOCKPROXYNODE *) 0 ;
    Dwg_EvalExpr *arg2 = (Dwg_EvalExpr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_DYNAMICBLOCKPROXYNODE_evalexpr_set(self,evalexpr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DYNAMICBLOCKPROXYNODE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DYNAMICBLOCKPROXYNODE_evalexpr_set" "', argument " "1"" of type '" "struct _dwg_object_DYNAMICBLOCKPROXYNODE *""'"); 
    }
    arg1 = (struct _dwg_object_DYNAMICBLOCKPROXYNODE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_EvalExpr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_DYNAMICBLOCKPROXYNODE_evalexpr_set" "', argument " "2"" of type '" "Dwg_EvalExpr *""'"); 
    }
    arg2 = (Dwg_EvalExpr *)(argp2);
    if (arg1) (arg1)->evalexpr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_DYNAMICBLOCKPROXYNODE_evalexpr_get) {
  {
    struct _dwg_object_DYNAMICBLOCKPROXYNODE *arg1 = (struct _dwg_object_DYNAMICBLOCKPROXYNODE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_EvalExpr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_DYNAMICBLOCKPROXYNODE_evalexpr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DYNAMICBLOCKPROXYNODE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_DYNAMICBLOCKPROXYNODE_evalexpr_get" "', argument " "1"" of type '" "struct _dwg_object_DYNAMICBLOCKPROXYNODE *""'"); 
    }
    arg1 = (struct _dwg_object_DYNAMICBLOCKPROXYNODE *)(argp1);
    result = (Dwg_EvalExpr *)& ((arg1)->evalexpr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_EvalExpr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_DYNAMICBLOCKPROXYNODE) {
  {
    int argvi = 0;
    struct _dwg_object_DYNAMICBLOCKPROXYNODE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_DYNAMICBLOCKPROXYNODE();");
    }
    result = (struct _dwg_object_DYNAMICBLOCKPROXYNODE *)calloc(1, sizeof(struct _dwg_object_DYNAMICBLOCKPROXYNODE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DYNAMICBLOCKPROXYNODE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_DYNAMICBLOCKPROXYNODE) {
  {
    struct _dwg_object_DYNAMICBLOCKPROXYNODE *arg1 = (struct _dwg_object_DYNAMICBLOCKPROXYNODE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_DYNAMICBLOCKPROXYNODE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_DYNAMICBLOCKPROXYNODE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_DYNAMICBLOCKPROXYNODE" "', argument " "1"" of type '" "struct _dwg_object_DYNAMICBLOCKPROXYNODE *""'"); 
    }
    arg1 = (struct _dwg_object_DYNAMICBLOCKPROXYNODE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUD_IntensityStyle_parent_set) {
  {
    struct _dwg_POINTCLOUD_IntensityStyle *arg1 = (struct _dwg_POINTCLOUD_IntensityStyle *) 0 ;
    struct _dwg_entity_POINTCLOUD *arg2 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_POINTCLOUD_IntensityStyle_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUD_IntensityStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUD_IntensityStyle_parent_set" "', argument " "1"" of type '" "struct _dwg_POINTCLOUD_IntensityStyle *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUD_IntensityStyle *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_POINTCLOUD, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_POINTCLOUD_IntensityStyle_parent_set" "', argument " "2"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg2 = (struct _dwg_entity_POINTCLOUD *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUD_IntensityStyle_parent_get) {
  {
    struct _dwg_POINTCLOUD_IntensityStyle *arg1 = (struct _dwg_POINTCLOUD_IntensityStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_entity_POINTCLOUD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_POINTCLOUD_IntensityStyle_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUD_IntensityStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUD_IntensityStyle_parent_get" "', argument " "1"" of type '" "struct _dwg_POINTCLOUD_IntensityStyle *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUD_IntensityStyle *)(argp1);
    result = (struct _dwg_entity_POINTCLOUD *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUD_IntensityStyle_min_intensity_set) {
  {
    struct _dwg_POINTCLOUD_IntensityStyle *arg1 = (struct _dwg_POINTCLOUD_IntensityStyle *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_POINTCLOUD_IntensityStyle_min_intensity_set(self,min_intensity);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUD_IntensityStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUD_IntensityStyle_min_intensity_set" "', argument " "1"" of type '" "struct _dwg_POINTCLOUD_IntensityStyle *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUD_IntensityStyle *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_POINTCLOUD_IntensityStyle_min_intensity_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->min_intensity = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUD_IntensityStyle_min_intensity_get) {
  {
    struct _dwg_POINTCLOUD_IntensityStyle *arg1 = (struct _dwg_POINTCLOUD_IntensityStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_POINTCLOUD_IntensityStyle_min_intensity_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUD_IntensityStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUD_IntensityStyle_min_intensity_get" "', argument " "1"" of type '" "struct _dwg_POINTCLOUD_IntensityStyle *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUD_IntensityStyle *)(argp1);
    result = (BITCODE_BD) ((arg1)->min_intensity);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUD_IntensityStyle_max_intensity_set) {
  {
    struct _dwg_POINTCLOUD_IntensityStyle *arg1 = (struct _dwg_POINTCLOUD_IntensityStyle *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_POINTCLOUD_IntensityStyle_max_intensity_set(self,max_intensity);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUD_IntensityStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUD_IntensityStyle_max_intensity_set" "', argument " "1"" of type '" "struct _dwg_POINTCLOUD_IntensityStyle *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUD_IntensityStyle *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_POINTCLOUD_IntensityStyle_max_intensity_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->max_intensity = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUD_IntensityStyle_max_intensity_get) {
  {
    struct _dwg_POINTCLOUD_IntensityStyle *arg1 = (struct _dwg_POINTCLOUD_IntensityStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_POINTCLOUD_IntensityStyle_max_intensity_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUD_IntensityStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUD_IntensityStyle_max_intensity_get" "', argument " "1"" of type '" "struct _dwg_POINTCLOUD_IntensityStyle *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUD_IntensityStyle *)(argp1);
    result = (BITCODE_BD) ((arg1)->max_intensity);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUD_IntensityStyle_intensity_low_treshold_set) {
  {
    struct _dwg_POINTCLOUD_IntensityStyle *arg1 = (struct _dwg_POINTCLOUD_IntensityStyle *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_POINTCLOUD_IntensityStyle_intensity_low_treshold_set(self,intensity_low_treshold);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUD_IntensityStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUD_IntensityStyle_intensity_low_treshold_set" "', argument " "1"" of type '" "struct _dwg_POINTCLOUD_IntensityStyle *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUD_IntensityStyle *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_POINTCLOUD_IntensityStyle_intensity_low_treshold_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->intensity_low_treshold = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUD_IntensityStyle_intensity_low_treshold_get) {
  {
    struct _dwg_POINTCLOUD_IntensityStyle *arg1 = (struct _dwg_POINTCLOUD_IntensityStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_POINTCLOUD_IntensityStyle_intensity_low_treshold_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUD_IntensityStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUD_IntensityStyle_intensity_low_treshold_get" "', argument " "1"" of type '" "struct _dwg_POINTCLOUD_IntensityStyle *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUD_IntensityStyle *)(argp1);
    result = (BITCODE_BD) ((arg1)->intensity_low_treshold);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUD_IntensityStyle_intensity_high_treshold_set) {
  {
    struct _dwg_POINTCLOUD_IntensityStyle *arg1 = (struct _dwg_POINTCLOUD_IntensityStyle *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_POINTCLOUD_IntensityStyle_intensity_high_treshold_set(self,intensity_high_treshold);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUD_IntensityStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUD_IntensityStyle_intensity_high_treshold_set" "', argument " "1"" of type '" "struct _dwg_POINTCLOUD_IntensityStyle *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUD_IntensityStyle *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_POINTCLOUD_IntensityStyle_intensity_high_treshold_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->intensity_high_treshold = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUD_IntensityStyle_intensity_high_treshold_get) {
  {
    struct _dwg_POINTCLOUD_IntensityStyle *arg1 = (struct _dwg_POINTCLOUD_IntensityStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_POINTCLOUD_IntensityStyle_intensity_high_treshold_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUD_IntensityStyle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUD_IntensityStyle_intensity_high_treshold_get" "', argument " "1"" of type '" "struct _dwg_POINTCLOUD_IntensityStyle *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUD_IntensityStyle *)(argp1);
    result = (BITCODE_BD) ((arg1)->intensity_high_treshold);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_POINTCLOUD_IntensityStyle) {
  {
    int argvi = 0;
    struct _dwg_POINTCLOUD_IntensityStyle *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_POINTCLOUD_IntensityStyle();");
    }
    result = (struct _dwg_POINTCLOUD_IntensityStyle *)calloc(1, sizeof(struct _dwg_POINTCLOUD_IntensityStyle));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_POINTCLOUD_IntensityStyle, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_POINTCLOUD_IntensityStyle) {
  {
    struct _dwg_POINTCLOUD_IntensityStyle *arg1 = (struct _dwg_POINTCLOUD_IntensityStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_POINTCLOUD_IntensityStyle(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUD_IntensityStyle, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_POINTCLOUD_IntensityStyle" "', argument " "1"" of type '" "struct _dwg_POINTCLOUD_IntensityStyle *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUD_IntensityStyle *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUD_Clippings_parent_set) {
  {
    struct _dwg_POINTCLOUD_Clippings *arg1 = (struct _dwg_POINTCLOUD_Clippings *) 0 ;
    struct _dwg_entity_POINTCLOUD *arg2 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_POINTCLOUD_Clippings_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUD_Clippings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUD_Clippings_parent_set" "', argument " "1"" of type '" "struct _dwg_POINTCLOUD_Clippings *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUD_Clippings *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_POINTCLOUD, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_POINTCLOUD_Clippings_parent_set" "', argument " "2"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg2 = (struct _dwg_entity_POINTCLOUD *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUD_Clippings_parent_get) {
  {
    struct _dwg_POINTCLOUD_Clippings *arg1 = (struct _dwg_POINTCLOUD_Clippings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_entity_POINTCLOUD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_POINTCLOUD_Clippings_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUD_Clippings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUD_Clippings_parent_get" "', argument " "1"" of type '" "struct _dwg_POINTCLOUD_Clippings *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUD_Clippings *)(argp1);
    result = (struct _dwg_entity_POINTCLOUD *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUD_Clippings_is_inverted_set) {
  {
    struct _dwg_POINTCLOUD_Clippings *arg1 = (struct _dwg_POINTCLOUD_Clippings *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_POINTCLOUD_Clippings_is_inverted_set(self,is_inverted);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUD_Clippings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUD_Clippings_is_inverted_set" "', argument " "1"" of type '" "struct _dwg_POINTCLOUD_Clippings *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUD_Clippings *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_POINTCLOUD_Clippings_is_inverted_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_inverted = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUD_Clippings_is_inverted_get) {
  {
    struct _dwg_POINTCLOUD_Clippings *arg1 = (struct _dwg_POINTCLOUD_Clippings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_POINTCLOUD_Clippings_is_inverted_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUD_Clippings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUD_Clippings_is_inverted_get" "', argument " "1"" of type '" "struct _dwg_POINTCLOUD_Clippings *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUD_Clippings *)(argp1);
    result = (BITCODE_B) ((arg1)->is_inverted);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUD_Clippings_type_set) {
  {
    struct _dwg_POINTCLOUD_Clippings *arg1 = (struct _dwg_POINTCLOUD_Clippings *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_POINTCLOUD_Clippings_type_set(self,type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUD_Clippings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUD_Clippings_type_set" "', argument " "1"" of type '" "struct _dwg_POINTCLOUD_Clippings *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUD_Clippings *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_POINTCLOUD_Clippings_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUD_Clippings_type_get) {
  {
    struct _dwg_POINTCLOUD_Clippings *arg1 = (struct _dwg_POINTCLOUD_Clippings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_POINTCLOUD_Clippings_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUD_Clippings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUD_Clippings_type_get" "', argument " "1"" of type '" "struct _dwg_POINTCLOUD_Clippings *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUD_Clippings *)(argp1);
    result = (BITCODE_BS) ((arg1)->type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUD_Clippings_num_vertices_set) {
  {
    struct _dwg_POINTCLOUD_Clippings *arg1 = (struct _dwg_POINTCLOUD_Clippings *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_POINTCLOUD_Clippings_num_vertices_set(self,num_vertices);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUD_Clippings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUD_Clippings_num_vertices_set" "', argument " "1"" of type '" "struct _dwg_POINTCLOUD_Clippings *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUD_Clippings *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_POINTCLOUD_Clippings_num_vertices_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_vertices = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUD_Clippings_num_vertices_get) {
  {
    struct _dwg_POINTCLOUD_Clippings *arg1 = (struct _dwg_POINTCLOUD_Clippings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_POINTCLOUD_Clippings_num_vertices_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUD_Clippings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUD_Clippings_num_vertices_get" "', argument " "1"" of type '" "struct _dwg_POINTCLOUD_Clippings *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUD_Clippings *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_vertices);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUD_Clippings_vertices_set) {
  {
    struct _dwg_POINTCLOUD_Clippings *arg1 = (struct _dwg_POINTCLOUD_Clippings *) 0 ;
    BITCODE_2RD *arg2 = (BITCODE_2RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_POINTCLOUD_Clippings_vertices_set(self,vertices);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUD_Clippings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUD_Clippings_vertices_set" "', argument " "1"" of type '" "struct _dwg_POINTCLOUD_Clippings *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUD_Clippings *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_2rd, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_POINTCLOUD_Clippings_vertices_set" "', argument " "2"" of type '" "BITCODE_2RD *""'"); 
    }
    arg2 = (BITCODE_2RD *)(argp2);
    if (arg1) (arg1)->vertices = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUD_Clippings_vertices_get) {
  {
    struct _dwg_POINTCLOUD_Clippings *arg1 = (struct _dwg_POINTCLOUD_Clippings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_2RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_POINTCLOUD_Clippings_vertices_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUD_Clippings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUD_Clippings_vertices_get" "', argument " "1"" of type '" "struct _dwg_POINTCLOUD_Clippings *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUD_Clippings *)(argp1);
    result = (BITCODE_2RD *) ((arg1)->vertices);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_2rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUD_Clippings_z_min_set) {
  {
    struct _dwg_POINTCLOUD_Clippings *arg1 = (struct _dwg_POINTCLOUD_Clippings *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_POINTCLOUD_Clippings_z_min_set(self,z_min);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUD_Clippings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUD_Clippings_z_min_set" "', argument " "1"" of type '" "struct _dwg_POINTCLOUD_Clippings *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUD_Clippings *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_POINTCLOUD_Clippings_z_min_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->z_min = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUD_Clippings_z_min_get) {
  {
    struct _dwg_POINTCLOUD_Clippings *arg1 = (struct _dwg_POINTCLOUD_Clippings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_POINTCLOUD_Clippings_z_min_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUD_Clippings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUD_Clippings_z_min_get" "', argument " "1"" of type '" "struct _dwg_POINTCLOUD_Clippings *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUD_Clippings *)(argp1);
    result = (BITCODE_BD) ((arg1)->z_min);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUD_Clippings_z_max_set) {
  {
    struct _dwg_POINTCLOUD_Clippings *arg1 = (struct _dwg_POINTCLOUD_Clippings *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_POINTCLOUD_Clippings_z_max_set(self,z_max);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUD_Clippings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUD_Clippings_z_max_set" "', argument " "1"" of type '" "struct _dwg_POINTCLOUD_Clippings *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUD_Clippings *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_POINTCLOUD_Clippings_z_max_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->z_max = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUD_Clippings_z_max_get) {
  {
    struct _dwg_POINTCLOUD_Clippings *arg1 = (struct _dwg_POINTCLOUD_Clippings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_POINTCLOUD_Clippings_z_max_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUD_Clippings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUD_Clippings_z_max_get" "', argument " "1"" of type '" "struct _dwg_POINTCLOUD_Clippings *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUD_Clippings *)(argp1);
    result = (BITCODE_BD) ((arg1)->z_max);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_POINTCLOUD_Clippings) {
  {
    int argvi = 0;
    struct _dwg_POINTCLOUD_Clippings *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_POINTCLOUD_Clippings();");
    }
    result = (struct _dwg_POINTCLOUD_Clippings *)calloc(1, sizeof(struct _dwg_POINTCLOUD_Clippings));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_POINTCLOUD_Clippings, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_POINTCLOUD_Clippings) {
  {
    struct _dwg_POINTCLOUD_Clippings *arg1 = (struct _dwg_POINTCLOUD_Clippings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_POINTCLOUD_Clippings(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUD_Clippings, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_POINTCLOUD_Clippings" "', argument " "1"" of type '" "struct _dwg_POINTCLOUD_Clippings *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUD_Clippings *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUDEX_Croppings_parent_set) {
  {
    struct _dwg_POINTCLOUDEX_Croppings *arg1 = (struct _dwg_POINTCLOUDEX_Croppings *) 0 ;
    struct _dwg_entity_POINTCLOUDEX *arg2 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_POINTCLOUDEX_Croppings_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUDEX_Croppings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUDEX_Croppings_parent_set" "', argument " "1"" of type '" "struct _dwg_POINTCLOUDEX_Croppings *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUDEX_Croppings *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_POINTCLOUDEX_Croppings_parent_set" "', argument " "2"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg2 = (struct _dwg_entity_POINTCLOUDEX *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUDEX_Croppings_parent_get) {
  {
    struct _dwg_POINTCLOUDEX_Croppings *arg1 = (struct _dwg_POINTCLOUDEX_Croppings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_entity_POINTCLOUDEX *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_POINTCLOUDEX_Croppings_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUDEX_Croppings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUDEX_Croppings_parent_get" "', argument " "1"" of type '" "struct _dwg_POINTCLOUDEX_Croppings *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUDEX_Croppings *)(argp1);
    result = (struct _dwg_entity_POINTCLOUDEX *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUDEX_Croppings_type_set) {
  {
    struct _dwg_POINTCLOUDEX_Croppings *arg1 = (struct _dwg_POINTCLOUDEX_Croppings *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_POINTCLOUDEX_Croppings_type_set(self,type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUDEX_Croppings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUDEX_Croppings_type_set" "', argument " "1"" of type '" "struct _dwg_POINTCLOUDEX_Croppings *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUDEX_Croppings *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_POINTCLOUDEX_Croppings_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUDEX_Croppings_type_get) {
  {
    struct _dwg_POINTCLOUDEX_Croppings *arg1 = (struct _dwg_POINTCLOUDEX_Croppings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_POINTCLOUDEX_Croppings_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUDEX_Croppings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUDEX_Croppings_type_get" "', argument " "1"" of type '" "struct _dwg_POINTCLOUDEX_Croppings *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUDEX_Croppings *)(argp1);
    result = (BITCODE_BS) ((arg1)->type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUDEX_Croppings_is_inside_set) {
  {
    struct _dwg_POINTCLOUDEX_Croppings *arg1 = (struct _dwg_POINTCLOUDEX_Croppings *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_POINTCLOUDEX_Croppings_is_inside_set(self,is_inside);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUDEX_Croppings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUDEX_Croppings_is_inside_set" "', argument " "1"" of type '" "struct _dwg_POINTCLOUDEX_Croppings *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUDEX_Croppings *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_POINTCLOUDEX_Croppings_is_inside_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_inside = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUDEX_Croppings_is_inside_get) {
  {
    struct _dwg_POINTCLOUDEX_Croppings *arg1 = (struct _dwg_POINTCLOUDEX_Croppings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_POINTCLOUDEX_Croppings_is_inside_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUDEX_Croppings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUDEX_Croppings_is_inside_get" "', argument " "1"" of type '" "struct _dwg_POINTCLOUDEX_Croppings *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUDEX_Croppings *)(argp1);
    result = (BITCODE_B) ((arg1)->is_inside);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUDEX_Croppings_is_inverted_set) {
  {
    struct _dwg_POINTCLOUDEX_Croppings *arg1 = (struct _dwg_POINTCLOUDEX_Croppings *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_POINTCLOUDEX_Croppings_is_inverted_set(self,is_inverted);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUDEX_Croppings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUDEX_Croppings_is_inverted_set" "', argument " "1"" of type '" "struct _dwg_POINTCLOUDEX_Croppings *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUDEX_Croppings *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_POINTCLOUDEX_Croppings_is_inverted_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_inverted = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUDEX_Croppings_is_inverted_get) {
  {
    struct _dwg_POINTCLOUDEX_Croppings *arg1 = (struct _dwg_POINTCLOUDEX_Croppings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_POINTCLOUDEX_Croppings_is_inverted_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUDEX_Croppings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUDEX_Croppings_is_inverted_get" "', argument " "1"" of type '" "struct _dwg_POINTCLOUDEX_Croppings *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUDEX_Croppings *)(argp1);
    result = (BITCODE_B) ((arg1)->is_inverted);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUDEX_Croppings_crop_plane_set) {
  {
    struct _dwg_POINTCLOUDEX_Croppings *arg1 = (struct _dwg_POINTCLOUDEX_Croppings *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_POINTCLOUDEX_Croppings_crop_plane_set(self,crop_plane);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUDEX_Croppings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUDEX_Croppings_crop_plane_set" "', argument " "1"" of type '" "struct _dwg_POINTCLOUDEX_Croppings *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUDEX_Croppings *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_POINTCLOUDEX_Croppings_crop_plane_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->crop_plane = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUDEX_Croppings_crop_plane_get) {
  {
    struct _dwg_POINTCLOUDEX_Croppings *arg1 = (struct _dwg_POINTCLOUDEX_Croppings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_POINTCLOUDEX_Croppings_crop_plane_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUDEX_Croppings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUDEX_Croppings_crop_plane_get" "', argument " "1"" of type '" "struct _dwg_POINTCLOUDEX_Croppings *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUDEX_Croppings *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->crop_plane);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUDEX_Croppings_crop_x_dir_set) {
  {
    struct _dwg_POINTCLOUDEX_Croppings *arg1 = (struct _dwg_POINTCLOUDEX_Croppings *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_POINTCLOUDEX_Croppings_crop_x_dir_set(self,crop_x_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUDEX_Croppings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUDEX_Croppings_crop_x_dir_set" "', argument " "1"" of type '" "struct _dwg_POINTCLOUDEX_Croppings *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUDEX_Croppings *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_POINTCLOUDEX_Croppings_crop_x_dir_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->crop_x_dir = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUDEX_Croppings_crop_x_dir_get) {
  {
    struct _dwg_POINTCLOUDEX_Croppings *arg1 = (struct _dwg_POINTCLOUDEX_Croppings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_POINTCLOUDEX_Croppings_crop_x_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUDEX_Croppings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUDEX_Croppings_crop_x_dir_get" "', argument " "1"" of type '" "struct _dwg_POINTCLOUDEX_Croppings *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUDEX_Croppings *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->crop_x_dir);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUDEX_Croppings_crop_y_dir_set) {
  {
    struct _dwg_POINTCLOUDEX_Croppings *arg1 = (struct _dwg_POINTCLOUDEX_Croppings *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_POINTCLOUDEX_Croppings_crop_y_dir_set(self,crop_y_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUDEX_Croppings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUDEX_Croppings_crop_y_dir_set" "', argument " "1"" of type '" "struct _dwg_POINTCLOUDEX_Croppings *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUDEX_Croppings *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_POINTCLOUDEX_Croppings_crop_y_dir_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->crop_y_dir = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUDEX_Croppings_crop_y_dir_get) {
  {
    struct _dwg_POINTCLOUDEX_Croppings *arg1 = (struct _dwg_POINTCLOUDEX_Croppings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_POINTCLOUDEX_Croppings_crop_y_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUDEX_Croppings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUDEX_Croppings_crop_y_dir_get" "', argument " "1"" of type '" "struct _dwg_POINTCLOUDEX_Croppings *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUDEX_Croppings *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->crop_y_dir);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUDEX_Croppings_num_pts_set) {
  {
    struct _dwg_POINTCLOUDEX_Croppings *arg1 = (struct _dwg_POINTCLOUDEX_Croppings *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_POINTCLOUDEX_Croppings_num_pts_set(self,num_pts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUDEX_Croppings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUDEX_Croppings_num_pts_set" "', argument " "1"" of type '" "struct _dwg_POINTCLOUDEX_Croppings *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUDEX_Croppings *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_POINTCLOUDEX_Croppings_num_pts_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_pts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUDEX_Croppings_num_pts_get) {
  {
    struct _dwg_POINTCLOUDEX_Croppings *arg1 = (struct _dwg_POINTCLOUDEX_Croppings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_POINTCLOUDEX_Croppings_num_pts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUDEX_Croppings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUDEX_Croppings_num_pts_get" "', argument " "1"" of type '" "struct _dwg_POINTCLOUDEX_Croppings *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUDEX_Croppings *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_pts);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUDEX_Croppings_pts_set) {
  {
    struct _dwg_POINTCLOUDEX_Croppings *arg1 = (struct _dwg_POINTCLOUDEX_Croppings *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_POINTCLOUDEX_Croppings_pts_set(self,pts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUDEX_Croppings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUDEX_Croppings_pts_set" "', argument " "1"" of type '" "struct _dwg_POINTCLOUDEX_Croppings *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUDEX_Croppings *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_POINTCLOUDEX_Croppings_pts_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->pts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUDEX_Croppings_pts_get) {
  {
    struct _dwg_POINTCLOUDEX_Croppings *arg1 = (struct _dwg_POINTCLOUDEX_Croppings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_POINTCLOUDEX_Croppings_pts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUDEX_Croppings, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUDEX_Croppings_pts_get" "', argument " "1"" of type '" "struct _dwg_POINTCLOUDEX_Croppings *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUDEX_Croppings *)(argp1);
    result = (BITCODE_3BD *) ((arg1)->pts);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_POINTCLOUDEX_Croppings) {
  {
    int argvi = 0;
    struct _dwg_POINTCLOUDEX_Croppings *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_POINTCLOUDEX_Croppings();");
    }
    result = (struct _dwg_POINTCLOUDEX_Croppings *)calloc(1, sizeof(struct _dwg_POINTCLOUDEX_Croppings));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_POINTCLOUDEX_Croppings, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_POINTCLOUDEX_Croppings) {
  {
    struct _dwg_POINTCLOUDEX_Croppings *arg1 = (struct _dwg_POINTCLOUDEX_Croppings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_POINTCLOUDEX_Croppings(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUDEX_Croppings, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_POINTCLOUDEX_Croppings" "', argument " "1"" of type '" "struct _dwg_POINTCLOUDEX_Croppings *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUDEX_Croppings *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_parent_set) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POINTCLOUD_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_parent_get) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_class_version_set) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_class_version_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POINTCLOUD_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_class_version_get) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_class_version_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_origin_set) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_origin_set(self,origin);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_origin_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POINTCLOUD_origin_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->origin = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_origin_get) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_origin_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_origin_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->origin);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_saved_filename_set) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_saved_filename_set(self,saved_filename);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_saved_filename_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POINTCLOUD_saved_filename_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->saved_filename);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->saved_filename = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->saved_filename = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_saved_filename_get) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_saved_filename_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_saved_filename_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    result = (BITCODE_TV) ((arg1)->saved_filename);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_num_source_files_set) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_num_source_files_set(self,num_source_files);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_num_source_files_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POINTCLOUD_num_source_files_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_source_files = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_num_source_files_get) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_num_source_files_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_num_source_files_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_source_files);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_source_files_set) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    BITCODE_TV *arg2 = (BITCODE_TV *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_source_files_set(self,source_files);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_source_files_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POINTCLOUD_source_files_set" "', argument " "2"" of type '" "BITCODE_TV *""'"); 
    }
    arg2 = (BITCODE_TV *)(argp2);
    if (arg1) (arg1)->source_files = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_source_files_get) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_source_files_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_source_files_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    result = (BITCODE_TV *) ((arg1)->source_files);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_extents_min_set) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_extents_min_set(self,extents_min);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_extents_min_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POINTCLOUD_extents_min_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->extents_min = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_extents_min_get) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_extents_min_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_extents_min_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->extents_min);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_extents_max_set) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_extents_max_set(self,extents_max);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_extents_max_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POINTCLOUD_extents_max_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->extents_max = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_extents_max_get) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_extents_max_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_extents_max_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->extents_max);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_numpoints_set) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    BITCODE_RLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_numpoints_set(self,numpoints);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_numpoints_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POINTCLOUD_numpoints_set" "', argument " "2"" of type '" "BITCODE_RLL""'");
    } 
    arg2 = (BITCODE_RLL)(val2);
    if (arg1) (arg1)->numpoints = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_numpoints_get) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_numpoints_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_numpoints_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    result = (BITCODE_RLL) ((arg1)->numpoints);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_ucs_name_set) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_ucs_name_set(self,ucs_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_ucs_name_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POINTCLOUD_ucs_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->ucs_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->ucs_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->ucs_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_ucs_name_get) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_ucs_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_ucs_name_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    result = (BITCODE_TV) ((arg1)->ucs_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_ucs_origin_set) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_ucs_origin_set(self,ucs_origin);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_ucs_origin_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POINTCLOUD_ucs_origin_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ucs_origin = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_ucs_origin_get) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_ucs_origin_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_ucs_origin_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ucs_origin);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_ucs_x_dir_set) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_ucs_x_dir_set(self,ucs_x_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_ucs_x_dir_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POINTCLOUD_ucs_x_dir_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ucs_x_dir = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_ucs_x_dir_get) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_ucs_x_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_ucs_x_dir_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ucs_x_dir);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_ucs_y_dir_set) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_ucs_y_dir_set(self,ucs_y_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_ucs_y_dir_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POINTCLOUD_ucs_y_dir_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ucs_y_dir = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_ucs_y_dir_get) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_ucs_y_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_ucs_y_dir_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ucs_y_dir);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_ucs_z_dir_set) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_ucs_z_dir_set(self,ucs_z_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_ucs_z_dir_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POINTCLOUD_ucs_z_dir_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ucs_z_dir = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_ucs_z_dir_get) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_ucs_z_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_ucs_z_dir_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ucs_z_dir);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_pointclouddef_set) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_pointclouddef_set(self,pointclouddef);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_pointclouddef_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POINTCLOUD_pointclouddef_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->pointclouddef = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_pointclouddef_get) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_pointclouddef_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_pointclouddef_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    result = (BITCODE_H) ((arg1)->pointclouddef);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_reactor_set) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_reactor_set(self,reactor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_reactor_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POINTCLOUD_reactor_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->reactor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_reactor_get) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_reactor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_reactor_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    result = (BITCODE_H) ((arg1)->reactor);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_show_intensity_set) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_show_intensity_set(self,show_intensity);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_show_intensity_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POINTCLOUD_show_intensity_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->show_intensity = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_show_intensity_get) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_show_intensity_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_show_intensity_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    result = (BITCODE_B) ((arg1)->show_intensity);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_intensity_scheme_set) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_intensity_scheme_set(self,intensity_scheme);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_intensity_scheme_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POINTCLOUD_intensity_scheme_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->intensity_scheme = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_intensity_scheme_get) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_intensity_scheme_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_intensity_scheme_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    result = (BITCODE_BS) ((arg1)->intensity_scheme);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_intensity_style_set) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    Dwg_POINTCLOUD_IntensityStyle *arg2 = (Dwg_POINTCLOUD_IntensityStyle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_intensity_style_set(self,intensity_style);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_intensity_style_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_POINTCLOUD_IntensityStyle, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POINTCLOUD_intensity_style_set" "', argument " "2"" of type '" "Dwg_POINTCLOUD_IntensityStyle *""'"); 
    }
    arg2 = (Dwg_POINTCLOUD_IntensityStyle *)(argp2);
    if (arg1) (arg1)->intensity_style = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_intensity_style_get) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_POINTCLOUD_IntensityStyle *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_intensity_style_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_intensity_style_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    result = (Dwg_POINTCLOUD_IntensityStyle *)& ((arg1)->intensity_style);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_POINTCLOUD_IntensityStyle, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_show_clipping_set) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_show_clipping_set(self,show_clipping);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_show_clipping_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POINTCLOUD_show_clipping_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->show_clipping = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_show_clipping_get) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_show_clipping_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_show_clipping_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    result = (BITCODE_B) ((arg1)->show_clipping);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_num_clippings_set) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_num_clippings_set(self,num_clippings);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_num_clippings_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POINTCLOUD_num_clippings_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_clippings = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_num_clippings_get) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_num_clippings_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_num_clippings_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_clippings);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_clippings_set) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    Dwg_POINTCLOUD_Clippings *arg2 = (Dwg_POINTCLOUD_Clippings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_clippings_set(self,clippings);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_clippings_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_POINTCLOUD_Clippings, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POINTCLOUD_clippings_set" "', argument " "2"" of type '" "Dwg_POINTCLOUD_Clippings *""'"); 
    }
    arg2 = (Dwg_POINTCLOUD_Clippings *)(argp2);
    if (arg1) (arg1)->clippings = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUD_clippings_get) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_POINTCLOUD_Clippings *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUD_clippings_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUD_clippings_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    result = (Dwg_POINTCLOUD_Clippings *) ((arg1)->clippings);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_POINTCLOUD_Clippings, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_POINTCLOUD) {
  {
    int argvi = 0;
    struct _dwg_entity_POINTCLOUD *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_POINTCLOUD();");
    }
    result = (struct _dwg_entity_POINTCLOUD *)calloc(1, sizeof(struct _dwg_entity_POINTCLOUD));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_POINTCLOUD, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_POINTCLOUD) {
  {
    struct _dwg_entity_POINTCLOUD *arg1 = (struct _dwg_entity_POINTCLOUD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_POINTCLOUD(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUD, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_POINTCLOUD" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUD *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUD *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_parent_set) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POINTCLOUDEX_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_parent_get) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_class_version_set) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_class_version_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POINTCLOUDEX_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_class_version_get) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_class_version_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_extents_min_set) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_extents_min_set(self,extents_min);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_extents_min_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POINTCLOUDEX_extents_min_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->extents_min = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_extents_min_get) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_extents_min_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_extents_min_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->extents_min);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_extents_max_set) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_extents_max_set(self,extents_max);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_extents_max_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POINTCLOUDEX_extents_max_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->extents_max = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_extents_max_get) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_extents_max_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_extents_max_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->extents_max);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_ucs_origin_set) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_ucs_origin_set(self,ucs_origin);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_ucs_origin_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POINTCLOUDEX_ucs_origin_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ucs_origin = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_ucs_origin_get) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_ucs_origin_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_ucs_origin_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ucs_origin);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_ucs_x_dir_set) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_ucs_x_dir_set(self,ucs_x_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_ucs_x_dir_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POINTCLOUDEX_ucs_x_dir_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ucs_x_dir = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_ucs_x_dir_get) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_ucs_x_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_ucs_x_dir_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ucs_x_dir);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_ucs_y_dir_set) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_ucs_y_dir_set(self,ucs_y_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_ucs_y_dir_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POINTCLOUDEX_ucs_y_dir_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ucs_y_dir = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_ucs_y_dir_get) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_ucs_y_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_ucs_y_dir_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ucs_y_dir);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_ucs_z_dir_set) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_ucs_z_dir_set(self,ucs_z_dir);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_ucs_z_dir_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POINTCLOUDEX_ucs_z_dir_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->ucs_z_dir = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_ucs_z_dir_get) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_ucs_z_dir_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_ucs_z_dir_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->ucs_z_dir);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_is_locked_set) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_is_locked_set(self,is_locked);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_is_locked_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POINTCLOUDEX_is_locked_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_locked = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_is_locked_get) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_is_locked_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_is_locked_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    result = (BITCODE_B) ((arg1)->is_locked);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_pointclouddefex_set) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_pointclouddefex_set(self,pointclouddefex);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_pointclouddefex_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POINTCLOUDEX_pointclouddefex_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->pointclouddefex = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_pointclouddefex_get) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_pointclouddefex_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_pointclouddefex_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    result = (BITCODE_H) ((arg1)->pointclouddefex);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_reactor_set) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_reactor_set(self,reactor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_reactor_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POINTCLOUDEX_reactor_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->reactor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_reactor_get) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_reactor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_reactor_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    result = (BITCODE_H) ((arg1)->reactor);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_name_set) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_name_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POINTCLOUDEX_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_name_get) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_name_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_show_intensity_set) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_show_intensity_set(self,show_intensity);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_show_intensity_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POINTCLOUDEX_show_intensity_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->show_intensity = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_show_intensity_get) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_show_intensity_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_show_intensity_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    result = (BITCODE_B) ((arg1)->show_intensity);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_stylization_type_set) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_stylization_type_set(self,stylization_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_stylization_type_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POINTCLOUDEX_stylization_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->stylization_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_stylization_type_get) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_stylization_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_stylization_type_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    result = (BITCODE_BS) ((arg1)->stylization_type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_intensity_colorscheme_set) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_intensity_colorscheme_set(self,intensity_colorscheme);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_intensity_colorscheme_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POINTCLOUDEX_intensity_colorscheme_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->intensity_colorscheme);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->intensity_colorscheme = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->intensity_colorscheme = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_intensity_colorscheme_get) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_intensity_colorscheme_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_intensity_colorscheme_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    result = (BITCODE_TV) ((arg1)->intensity_colorscheme);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_cur_colorscheme_set) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_cur_colorscheme_set(self,cur_colorscheme);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_cur_colorscheme_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POINTCLOUDEX_cur_colorscheme_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->cur_colorscheme);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->cur_colorscheme = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->cur_colorscheme = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_cur_colorscheme_get) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_cur_colorscheme_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_cur_colorscheme_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    result = (BITCODE_TV) ((arg1)->cur_colorscheme);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_classification_colorscheme_set) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_classification_colorscheme_set(self,classification_colorscheme);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_classification_colorscheme_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POINTCLOUDEX_classification_colorscheme_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->classification_colorscheme);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->classification_colorscheme = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->classification_colorscheme = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_classification_colorscheme_get) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_classification_colorscheme_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_classification_colorscheme_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    result = (BITCODE_TV) ((arg1)->classification_colorscheme);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_elevation_min_set) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_elevation_min_set(self,elevation_min);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_elevation_min_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POINTCLOUDEX_elevation_min_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->elevation_min = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_elevation_min_get) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_elevation_min_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_elevation_min_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    result = (BITCODE_BD) ((arg1)->elevation_min);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_elevation_max_set) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_elevation_max_set(self,elevation_max);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_elevation_max_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POINTCLOUDEX_elevation_max_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->elevation_max = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_elevation_max_get) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_elevation_max_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_elevation_max_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    result = (BITCODE_BD) ((arg1)->elevation_max);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_intensity_min_set) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_intensity_min_set(self,intensity_min);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_intensity_min_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POINTCLOUDEX_intensity_min_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->intensity_min = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_intensity_min_get) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_intensity_min_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_intensity_min_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    result = (BITCODE_BL) ((arg1)->intensity_min);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_intensity_max_set) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_intensity_max_set(self,intensity_max);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_intensity_max_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POINTCLOUDEX_intensity_max_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->intensity_max = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_intensity_max_get) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_intensity_max_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_intensity_max_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    result = (BITCODE_BL) ((arg1)->intensity_max);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_intensity_out_of_range_behavior_set) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_intensity_out_of_range_behavior_set(self,intensity_out_of_range_behavior);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_intensity_out_of_range_behavior_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POINTCLOUDEX_intensity_out_of_range_behavior_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->intensity_out_of_range_behavior = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_intensity_out_of_range_behavior_get) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_intensity_out_of_range_behavior_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_intensity_out_of_range_behavior_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    result = (BITCODE_BS) ((arg1)->intensity_out_of_range_behavior);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_elevation_out_of_range_behavior_set) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_elevation_out_of_range_behavior_set(self,elevation_out_of_range_behavior);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_elevation_out_of_range_behavior_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POINTCLOUDEX_elevation_out_of_range_behavior_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->elevation_out_of_range_behavior = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_elevation_out_of_range_behavior_get) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_elevation_out_of_range_behavior_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_elevation_out_of_range_behavior_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    result = (BITCODE_BS) ((arg1)->elevation_out_of_range_behavior);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_elevation_apply_to_fixed_range_set) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_elevation_apply_to_fixed_range_set(self,elevation_apply_to_fixed_range);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_elevation_apply_to_fixed_range_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POINTCLOUDEX_elevation_apply_to_fixed_range_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->elevation_apply_to_fixed_range = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_elevation_apply_to_fixed_range_get) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_elevation_apply_to_fixed_range_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_elevation_apply_to_fixed_range_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    result = (BITCODE_B) ((arg1)->elevation_apply_to_fixed_range);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_intensity_as_gradient_set) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_intensity_as_gradient_set(self,intensity_as_gradient);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_intensity_as_gradient_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POINTCLOUDEX_intensity_as_gradient_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->intensity_as_gradient = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_intensity_as_gradient_get) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_intensity_as_gradient_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_intensity_as_gradient_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    result = (BITCODE_B) ((arg1)->intensity_as_gradient);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_elevation_as_gradient_set) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_elevation_as_gradient_set(self,elevation_as_gradient);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_elevation_as_gradient_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POINTCLOUDEX_elevation_as_gradient_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->elevation_as_gradient = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_elevation_as_gradient_get) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_elevation_as_gradient_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_elevation_as_gradient_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    result = (BITCODE_B) ((arg1)->elevation_as_gradient);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_show_cropping_set) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_show_cropping_set(self,show_cropping);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_show_cropping_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POINTCLOUDEX_show_cropping_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->show_cropping = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_show_cropping_get) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_show_cropping_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_show_cropping_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    result = (BITCODE_B) ((arg1)->show_cropping);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_unknown_bl0_set) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_unknown_bl0_set(self,unknown_bl0);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_unknown_bl0_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POINTCLOUDEX_unknown_bl0_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl0 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_unknown_bl0_get) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_unknown_bl0_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_unknown_bl0_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl0);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_unknown_bl1_set) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_unknown_bl1_set(self,unknown_bl1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_unknown_bl1_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POINTCLOUDEX_unknown_bl1_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_unknown_bl1_get) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_unknown_bl1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_unknown_bl1_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl1);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_num_croppings_set) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_num_croppings_set(self,num_croppings);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_num_croppings_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_POINTCLOUDEX_num_croppings_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_croppings = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_num_croppings_get) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_num_croppings_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_num_croppings_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_croppings);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_croppings_set) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    Dwg_POINTCLOUDEX_Croppings *arg2 = (Dwg_POINTCLOUDEX_Croppings *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_croppings_set(self,croppings);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_croppings_set" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_POINTCLOUDEX_Croppings, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_POINTCLOUDEX_croppings_set" "', argument " "2"" of type '" "Dwg_POINTCLOUDEX_Croppings *""'"); 
    }
    arg2 = (Dwg_POINTCLOUDEX_Croppings *)(argp2);
    if (arg1) (arg1)->croppings = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_POINTCLOUDEX_croppings_get) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_POINTCLOUDEX_Croppings *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_POINTCLOUDEX_croppings_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_POINTCLOUDEX_croppings_get" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    result = (Dwg_POINTCLOUDEX_Croppings *) ((arg1)->croppings);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_POINTCLOUDEX_Croppings, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_POINTCLOUDEX) {
  {
    int argvi = 0;
    struct _dwg_entity_POINTCLOUDEX *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_POINTCLOUDEX();");
    }
    result = (struct _dwg_entity_POINTCLOUDEX *)calloc(1, sizeof(struct _dwg_entity_POINTCLOUDEX));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_POINTCLOUDEX, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_POINTCLOUDEX) {
  {
    struct _dwg_entity_POINTCLOUDEX *arg1 = (struct _dwg_entity_POINTCLOUDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_POINTCLOUDEX(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_POINTCLOUDEX" "', argument " "1"" of type '" "struct _dwg_entity_POINTCLOUDEX *""'"); 
    }
    arg1 = (struct _dwg_entity_POINTCLOUDEX *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEF_parent_set) {
  {
    struct _dwg_object_POINTCLOUDDEF *arg1 = (struct _dwg_object_POINTCLOUDDEF *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEF_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEF_parent_set" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEF *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEF *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_POINTCLOUDDEF_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEF_parent_get) {
  {
    struct _dwg_object_POINTCLOUDDEF *arg1 = (struct _dwg_object_POINTCLOUDDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEF_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEF_parent_get" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEF *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEF *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEF_class_version_set) {
  {
    struct _dwg_object_POINTCLOUDDEF *arg1 = (struct _dwg_object_POINTCLOUDDEF *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEF_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEF_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEF *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEF *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_POINTCLOUDDEF_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEF_class_version_get) {
  {
    struct _dwg_object_POINTCLOUDDEF *arg1 = (struct _dwg_object_POINTCLOUDDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEF_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEF_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEF *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEF *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEF_source_filename_set) {
  {
    struct _dwg_object_POINTCLOUDDEF *arg1 = (struct _dwg_object_POINTCLOUDDEF *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEF_source_filename_set(self,source_filename);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEF_source_filename_set" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEF *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEF *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_POINTCLOUDDEF_source_filename_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->source_filename);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->source_filename = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->source_filename = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEF_source_filename_get) {
  {
    struct _dwg_object_POINTCLOUDDEF *arg1 = (struct _dwg_object_POINTCLOUDDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEF_source_filename_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEF_source_filename_get" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEF *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEF *)(argp1);
    result = (BITCODE_TV) ((arg1)->source_filename);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEF_is_loaded_set) {
  {
    struct _dwg_object_POINTCLOUDDEF *arg1 = (struct _dwg_object_POINTCLOUDDEF *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEF_is_loaded_set(self,is_loaded);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEF_is_loaded_set" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEF *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEF *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_POINTCLOUDDEF_is_loaded_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_loaded = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEF_is_loaded_get) {
  {
    struct _dwg_object_POINTCLOUDDEF *arg1 = (struct _dwg_object_POINTCLOUDDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEF_is_loaded_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEF_is_loaded_get" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEF *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEF *)(argp1);
    result = (BITCODE_B) ((arg1)->is_loaded);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEF_numpoints_set) {
  {
    struct _dwg_object_POINTCLOUDDEF *arg1 = (struct _dwg_object_POINTCLOUDDEF *) 0 ;
    BITCODE_RLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEF_numpoints_set(self,numpoints);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEF_numpoints_set" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEF *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEF *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_POINTCLOUDDEF_numpoints_set" "', argument " "2"" of type '" "BITCODE_RLL""'");
    } 
    arg2 = (BITCODE_RLL)(val2);
    if (arg1) (arg1)->numpoints = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEF_numpoints_get) {
  {
    struct _dwg_object_POINTCLOUDDEF *arg1 = (struct _dwg_object_POINTCLOUDDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEF_numpoints_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEF_numpoints_get" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEF *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEF *)(argp1);
    result = (BITCODE_RLL) ((arg1)->numpoints);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEF_extents_min_set) {
  {
    struct _dwg_object_POINTCLOUDDEF *arg1 = (struct _dwg_object_POINTCLOUDDEF *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEF_extents_min_set(self,extents_min);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEF_extents_min_set" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEF *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEF *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_POINTCLOUDDEF_extents_min_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->extents_min = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEF_extents_min_get) {
  {
    struct _dwg_object_POINTCLOUDDEF *arg1 = (struct _dwg_object_POINTCLOUDDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEF_extents_min_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEF_extents_min_get" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEF *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEF *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->extents_min);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEF_extents_max_set) {
  {
    struct _dwg_object_POINTCLOUDDEF *arg1 = (struct _dwg_object_POINTCLOUDDEF *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEF_extents_max_set(self,extents_max);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEF_extents_max_set" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEF *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEF *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_POINTCLOUDDEF_extents_max_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->extents_max = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEF_extents_max_get) {
  {
    struct _dwg_object_POINTCLOUDDEF *arg1 = (struct _dwg_object_POINTCLOUDDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEF_extents_max_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEF, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEF_extents_max_get" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEF *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEF *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->extents_max);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_POINTCLOUDDEF) {
  {
    int argvi = 0;
    struct _dwg_object_POINTCLOUDDEF *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_POINTCLOUDDEF();");
    }
    result = (struct _dwg_object_POINTCLOUDDEF *)calloc(1, sizeof(struct _dwg_object_POINTCLOUDDEF));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_POINTCLOUDDEF, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_POINTCLOUDDEF) {
  {
    struct _dwg_object_POINTCLOUDDEF *arg1 = (struct _dwg_object_POINTCLOUDDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_POINTCLOUDDEF(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEF, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_POINTCLOUDDEF" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEF *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEF *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEFEX_parent_set) {
  {
    struct _dwg_object_POINTCLOUDDEFEX *arg1 = (struct _dwg_object_POINTCLOUDDEFEX *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEFEX_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEFEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEFEX_parent_set" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEFEX *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEFEX *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_POINTCLOUDDEFEX_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEFEX_parent_get) {
  {
    struct _dwg_object_POINTCLOUDDEFEX *arg1 = (struct _dwg_object_POINTCLOUDDEFEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEFEX_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEFEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEFEX_parent_get" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEFEX *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEFEX *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEFEX_class_version_set) {
  {
    struct _dwg_object_POINTCLOUDDEFEX *arg1 = (struct _dwg_object_POINTCLOUDDEFEX *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEFEX_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEFEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEFEX_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEFEX *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEFEX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_POINTCLOUDDEFEX_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEFEX_class_version_get) {
  {
    struct _dwg_object_POINTCLOUDDEFEX *arg1 = (struct _dwg_object_POINTCLOUDDEFEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEFEX_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEFEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEFEX_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEFEX *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEFEX *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEFEX_source_filename_set) {
  {
    struct _dwg_object_POINTCLOUDDEFEX *arg1 = (struct _dwg_object_POINTCLOUDDEFEX *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEFEX_source_filename_set(self,source_filename);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEFEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEFEX_source_filename_set" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEFEX *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEFEX *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_POINTCLOUDDEFEX_source_filename_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->source_filename);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->source_filename = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->source_filename = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEFEX_source_filename_get) {
  {
    struct _dwg_object_POINTCLOUDDEFEX *arg1 = (struct _dwg_object_POINTCLOUDDEFEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEFEX_source_filename_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEFEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEFEX_source_filename_get" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEFEX *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEFEX *)(argp1);
    result = (BITCODE_TV) ((arg1)->source_filename);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEFEX_is_loaded_set) {
  {
    struct _dwg_object_POINTCLOUDDEFEX *arg1 = (struct _dwg_object_POINTCLOUDDEFEX *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEFEX_is_loaded_set(self,is_loaded);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEFEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEFEX_is_loaded_set" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEFEX *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEFEX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_POINTCLOUDDEFEX_is_loaded_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_loaded = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEFEX_is_loaded_get) {
  {
    struct _dwg_object_POINTCLOUDDEFEX *arg1 = (struct _dwg_object_POINTCLOUDDEFEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEFEX_is_loaded_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEFEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEFEX_is_loaded_get" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEFEX *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEFEX *)(argp1);
    result = (BITCODE_B) ((arg1)->is_loaded);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEFEX_numpoints_set) {
  {
    struct _dwg_object_POINTCLOUDDEFEX *arg1 = (struct _dwg_object_POINTCLOUDDEFEX *) 0 ;
    BITCODE_RLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEFEX_numpoints_set(self,numpoints);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEFEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEFEX_numpoints_set" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEFEX *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEFEX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_POINTCLOUDDEFEX_numpoints_set" "', argument " "2"" of type '" "BITCODE_RLL""'");
    } 
    arg2 = (BITCODE_RLL)(val2);
    if (arg1) (arg1)->numpoints = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEFEX_numpoints_get) {
  {
    struct _dwg_object_POINTCLOUDDEFEX *arg1 = (struct _dwg_object_POINTCLOUDDEFEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEFEX_numpoints_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEFEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEFEX_numpoints_get" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEFEX *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEFEX *)(argp1);
    result = (BITCODE_RLL) ((arg1)->numpoints);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEFEX_extents_min_set) {
  {
    struct _dwg_object_POINTCLOUDDEFEX *arg1 = (struct _dwg_object_POINTCLOUDDEFEX *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEFEX_extents_min_set(self,extents_min);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEFEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEFEX_extents_min_set" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEFEX *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEFEX *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_POINTCLOUDDEFEX_extents_min_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->extents_min = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEFEX_extents_min_get) {
  {
    struct _dwg_object_POINTCLOUDDEFEX *arg1 = (struct _dwg_object_POINTCLOUDDEFEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEFEX_extents_min_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEFEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEFEX_extents_min_get" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEFEX *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEFEX *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->extents_min);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEFEX_extents_max_set) {
  {
    struct _dwg_object_POINTCLOUDDEFEX *arg1 = (struct _dwg_object_POINTCLOUDDEFEX *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEFEX_extents_max_set(self,extents_max);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEFEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEFEX_extents_max_set" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEFEX *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEFEX *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_POINTCLOUDDEFEX_extents_max_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->extents_max = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEFEX_extents_max_get) {
  {
    struct _dwg_object_POINTCLOUDDEFEX *arg1 = (struct _dwg_object_POINTCLOUDDEFEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEFEX_extents_max_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEFEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEFEX_extents_max_get" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEFEX *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEFEX *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->extents_max);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_POINTCLOUDDEFEX) {
  {
    int argvi = 0;
    struct _dwg_object_POINTCLOUDDEFEX *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_POINTCLOUDDEFEX();");
    }
    result = (struct _dwg_object_POINTCLOUDDEFEX *)calloc(1, sizeof(struct _dwg_object_POINTCLOUDDEFEX));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_POINTCLOUDDEFEX, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_POINTCLOUDDEFEX) {
  {
    struct _dwg_object_POINTCLOUDDEFEX *arg1 = (struct _dwg_object_POINTCLOUDDEFEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_POINTCLOUDDEFEX(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEFEX, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_POINTCLOUDDEFEX" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEFEX *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEFEX *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEF_REACTOR_parent_set) {
  {
    struct _dwg_object_POINTCLOUDDEF_REACTOR *arg1 = (struct _dwg_object_POINTCLOUDDEF_REACTOR *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEF_REACTOR_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEF_REACTOR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEF_REACTOR_parent_set" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEF_REACTOR *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEF_REACTOR *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_POINTCLOUDDEF_REACTOR_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEF_REACTOR_parent_get) {
  {
    struct _dwg_object_POINTCLOUDDEF_REACTOR *arg1 = (struct _dwg_object_POINTCLOUDDEF_REACTOR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEF_REACTOR_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEF_REACTOR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEF_REACTOR_parent_get" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEF_REACTOR *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEF_REACTOR *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEF_REACTOR_class_version_set) {
  {
    struct _dwg_object_POINTCLOUDDEF_REACTOR *arg1 = (struct _dwg_object_POINTCLOUDDEF_REACTOR *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEF_REACTOR_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEF_REACTOR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEF_REACTOR_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEF_REACTOR *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEF_REACTOR *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_POINTCLOUDDEF_REACTOR_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEF_REACTOR_class_version_get) {
  {
    struct _dwg_object_POINTCLOUDDEF_REACTOR *arg1 = (struct _dwg_object_POINTCLOUDDEF_REACTOR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEF_REACTOR_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEF_REACTOR, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEF_REACTOR_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEF_REACTOR *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEF_REACTOR *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_POINTCLOUDDEF_REACTOR) {
  {
    int argvi = 0;
    struct _dwg_object_POINTCLOUDDEF_REACTOR *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_POINTCLOUDDEF_REACTOR();");
    }
    result = (struct _dwg_object_POINTCLOUDDEF_REACTOR *)calloc(1, sizeof(struct _dwg_object_POINTCLOUDDEF_REACTOR));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_POINTCLOUDDEF_REACTOR, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_POINTCLOUDDEF_REACTOR) {
  {
    struct _dwg_object_POINTCLOUDDEF_REACTOR *arg1 = (struct _dwg_object_POINTCLOUDDEF_REACTOR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_POINTCLOUDDEF_REACTOR(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEF_REACTOR, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_POINTCLOUDDEF_REACTOR" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEF_REACTOR *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEF_REACTOR *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEF_REACTOR_EX_parent_set) {
  {
    struct _dwg_object_POINTCLOUDDEF_REACTOR_EX *arg1 = (struct _dwg_object_POINTCLOUDDEF_REACTOR_EX *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEF_REACTOR_EX_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEF_REACTOR_EX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEF_REACTOR_EX_parent_set" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEF_REACTOR_EX *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEF_REACTOR_EX *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_POINTCLOUDDEF_REACTOR_EX_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEF_REACTOR_EX_parent_get) {
  {
    struct _dwg_object_POINTCLOUDDEF_REACTOR_EX *arg1 = (struct _dwg_object_POINTCLOUDDEF_REACTOR_EX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEF_REACTOR_EX_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEF_REACTOR_EX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEF_REACTOR_EX_parent_get" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEF_REACTOR_EX *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEF_REACTOR_EX *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEF_REACTOR_EX_class_version_set) {
  {
    struct _dwg_object_POINTCLOUDDEF_REACTOR_EX *arg1 = (struct _dwg_object_POINTCLOUDDEF_REACTOR_EX *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEF_REACTOR_EX_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEF_REACTOR_EX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEF_REACTOR_EX_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEF_REACTOR_EX *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEF_REACTOR_EX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_POINTCLOUDDEF_REACTOR_EX_class_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDDEF_REACTOR_EX_class_version_get) {
  {
    struct _dwg_object_POINTCLOUDDEF_REACTOR_EX *arg1 = (struct _dwg_object_POINTCLOUDDEF_REACTOR_EX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDDEF_REACTOR_EX_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEF_REACTOR_EX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDDEF_REACTOR_EX_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEF_REACTOR_EX *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEF_REACTOR_EX *)(argp1);
    result = (BITCODE_BL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_POINTCLOUDDEF_REACTOR_EX) {
  {
    int argvi = 0;
    struct _dwg_object_POINTCLOUDDEF_REACTOR_EX *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_POINTCLOUDDEF_REACTOR_EX();");
    }
    result = (struct _dwg_object_POINTCLOUDDEF_REACTOR_EX *)calloc(1, sizeof(struct _dwg_object_POINTCLOUDDEF_REACTOR_EX));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_POINTCLOUDDEF_REACTOR_EX, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_POINTCLOUDDEF_REACTOR_EX) {
  {
    struct _dwg_object_POINTCLOUDDEF_REACTOR_EX *arg1 = (struct _dwg_object_POINTCLOUDDEF_REACTOR_EX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_POINTCLOUDDEF_REACTOR_EX(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDDEF_REACTOR_EX, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_POINTCLOUDDEF_REACTOR_EX" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDDEF_REACTOR_EX *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDDEF_REACTOR_EX *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ColorRamp_parent_set) {
  {
    struct _dwg_ColorRamp *arg1 = (struct _dwg_ColorRamp *) 0 ;
    struct _dwg_POINTCLOUDCOLORMAP_Ramp *arg2 = (struct _dwg_POINTCLOUDCOLORMAP_Ramp *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ColorRamp_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ColorRamp, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ColorRamp_parent_set" "', argument " "1"" of type '" "struct _dwg_ColorRamp *""'"); 
    }
    arg1 = (struct _dwg_ColorRamp *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_POINTCLOUDCOLORMAP_Ramp, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_ColorRamp_parent_set" "', argument " "2"" of type '" "struct _dwg_POINTCLOUDCOLORMAP_Ramp *""'"); 
    }
    arg2 = (struct _dwg_POINTCLOUDCOLORMAP_Ramp *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ColorRamp_parent_get) {
  {
    struct _dwg_ColorRamp *arg1 = (struct _dwg_ColorRamp *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_POINTCLOUDCOLORMAP_Ramp *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ColorRamp_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ColorRamp, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ColorRamp_parent_get" "', argument " "1"" of type '" "struct _dwg_ColorRamp *""'"); 
    }
    arg1 = (struct _dwg_ColorRamp *)(argp1);
    result = (struct _dwg_POINTCLOUDCOLORMAP_Ramp *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_POINTCLOUDCOLORMAP_Ramp, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ColorRamp_colorscheme_set) {
  {
    struct _dwg_ColorRamp *arg1 = (struct _dwg_ColorRamp *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ColorRamp_colorscheme_set(self,colorscheme);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ColorRamp, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ColorRamp_colorscheme_set" "', argument " "1"" of type '" "struct _dwg_ColorRamp *""'"); 
    }
    arg1 = (struct _dwg_ColorRamp *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_ColorRamp_colorscheme_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->colorscheme);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->colorscheme = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->colorscheme = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ColorRamp_colorscheme_get) {
  {
    struct _dwg_ColorRamp *arg1 = (struct _dwg_ColorRamp *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ColorRamp_colorscheme_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ColorRamp, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ColorRamp_colorscheme_get" "', argument " "1"" of type '" "struct _dwg_ColorRamp *""'"); 
    }
    arg1 = (struct _dwg_ColorRamp *)(argp1);
    result = (BITCODE_TV) ((arg1)->colorscheme);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ColorRamp_unknown_bl_set) {
  {
    struct _dwg_ColorRamp *arg1 = (struct _dwg_ColorRamp *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ColorRamp_unknown_bl_set(self,unknown_bl);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ColorRamp, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ColorRamp_unknown_bl_set" "', argument " "1"" of type '" "struct _dwg_ColorRamp *""'"); 
    }
    arg1 = (struct _dwg_ColorRamp *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ColorRamp_unknown_bl_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_bl = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ColorRamp_unknown_bl_get) {
  {
    struct _dwg_ColorRamp *arg1 = (struct _dwg_ColorRamp *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ColorRamp_unknown_bl_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ColorRamp, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ColorRamp_unknown_bl_get" "', argument " "1"" of type '" "struct _dwg_ColorRamp *""'"); 
    }
    arg1 = (struct _dwg_ColorRamp *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_bl);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ColorRamp_unknown_b_set) {
  {
    struct _dwg_ColorRamp *arg1 = (struct _dwg_ColorRamp *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ColorRamp_unknown_b_set(self,unknown_b);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ColorRamp, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ColorRamp_unknown_b_set" "', argument " "1"" of type '" "struct _dwg_ColorRamp *""'"); 
    }
    arg1 = (struct _dwg_ColorRamp *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ColorRamp_unknown_b_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->unknown_b = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ColorRamp_unknown_b_get) {
  {
    struct _dwg_ColorRamp *arg1 = (struct _dwg_ColorRamp *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ColorRamp_unknown_b_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ColorRamp, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ColorRamp_unknown_b_get" "', argument " "1"" of type '" "struct _dwg_ColorRamp *""'"); 
    }
    arg1 = (struct _dwg_ColorRamp *)(argp1);
    result = (BITCODE_B) ((arg1)->unknown_b);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_ColorRamp) {
  {
    int argvi = 0;
    struct _dwg_ColorRamp *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_ColorRamp();");
    }
    result = (struct _dwg_ColorRamp *)calloc(1, sizeof(struct _dwg_ColorRamp));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ColorRamp, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_ColorRamp) {
  {
    struct _dwg_ColorRamp *arg1 = (struct _dwg_ColorRamp *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_ColorRamp(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_ColorRamp, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_ColorRamp" "', argument " "1"" of type '" "struct _dwg_ColorRamp *""'"); 
    }
    arg1 = (struct _dwg_ColorRamp *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUDCOLORMAP_Ramp_parent_set) {
  {
    struct _dwg_POINTCLOUDCOLORMAP_Ramp *arg1 = (struct _dwg_POINTCLOUDCOLORMAP_Ramp *) 0 ;
    struct _dwg_object_POINTCLOUDCOLORMAP *arg2 = (struct _dwg_object_POINTCLOUDCOLORMAP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_POINTCLOUDCOLORMAP_Ramp_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUDCOLORMAP_Ramp, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUDCOLORMAP_Ramp_parent_set" "', argument " "1"" of type '" "struct _dwg_POINTCLOUDCOLORMAP_Ramp *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUDCOLORMAP_Ramp *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_POINTCLOUDCOLORMAP, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_POINTCLOUDCOLORMAP_Ramp_parent_set" "', argument " "2"" of type '" "struct _dwg_object_POINTCLOUDCOLORMAP *""'"); 
    }
    arg2 = (struct _dwg_object_POINTCLOUDCOLORMAP *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUDCOLORMAP_Ramp_parent_get) {
  {
    struct _dwg_POINTCLOUDCOLORMAP_Ramp *arg1 = (struct _dwg_POINTCLOUDCOLORMAP_Ramp *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_POINTCLOUDCOLORMAP *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_POINTCLOUDCOLORMAP_Ramp_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUDCOLORMAP_Ramp, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUDCOLORMAP_Ramp_parent_get" "', argument " "1"" of type '" "struct _dwg_POINTCLOUDCOLORMAP_Ramp *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUDCOLORMAP_Ramp *)(argp1);
    result = (struct _dwg_object_POINTCLOUDCOLORMAP *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_POINTCLOUDCOLORMAP, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUDCOLORMAP_Ramp_class_version_set) {
  {
    struct _dwg_POINTCLOUDCOLORMAP_Ramp *arg1 = (struct _dwg_POINTCLOUDCOLORMAP_Ramp *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_POINTCLOUDCOLORMAP_Ramp_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUDCOLORMAP_Ramp, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUDCOLORMAP_Ramp_class_version_set" "', argument " "1"" of type '" "struct _dwg_POINTCLOUDCOLORMAP_Ramp *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUDCOLORMAP_Ramp *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_POINTCLOUDCOLORMAP_Ramp_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUDCOLORMAP_Ramp_class_version_get) {
  {
    struct _dwg_POINTCLOUDCOLORMAP_Ramp *arg1 = (struct _dwg_POINTCLOUDCOLORMAP_Ramp *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_POINTCLOUDCOLORMAP_Ramp_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUDCOLORMAP_Ramp, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUDCOLORMAP_Ramp_class_version_get" "', argument " "1"" of type '" "struct _dwg_POINTCLOUDCOLORMAP_Ramp *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUDCOLORMAP_Ramp *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUDCOLORMAP_Ramp_num_ramps_set) {
  {
    struct _dwg_POINTCLOUDCOLORMAP_Ramp *arg1 = (struct _dwg_POINTCLOUDCOLORMAP_Ramp *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_POINTCLOUDCOLORMAP_Ramp_num_ramps_set(self,num_ramps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUDCOLORMAP_Ramp, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUDCOLORMAP_Ramp_num_ramps_set" "', argument " "1"" of type '" "struct _dwg_POINTCLOUDCOLORMAP_Ramp *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUDCOLORMAP_Ramp *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_POINTCLOUDCOLORMAP_Ramp_num_ramps_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_ramps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUDCOLORMAP_Ramp_num_ramps_get) {
  {
    struct _dwg_POINTCLOUDCOLORMAP_Ramp *arg1 = (struct _dwg_POINTCLOUDCOLORMAP_Ramp *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_POINTCLOUDCOLORMAP_Ramp_num_ramps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUDCOLORMAP_Ramp, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUDCOLORMAP_Ramp_num_ramps_get" "', argument " "1"" of type '" "struct _dwg_POINTCLOUDCOLORMAP_Ramp *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUDCOLORMAP_Ramp *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_ramps);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUDCOLORMAP_Ramp_ramps_set) {
  {
    struct _dwg_POINTCLOUDCOLORMAP_Ramp *arg1 = (struct _dwg_POINTCLOUDCOLORMAP_Ramp *) 0 ;
    Dwg_ColorRamp *arg2 = (Dwg_ColorRamp *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_POINTCLOUDCOLORMAP_Ramp_ramps_set(self,ramps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUDCOLORMAP_Ramp, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUDCOLORMAP_Ramp_ramps_set" "', argument " "1"" of type '" "struct _dwg_POINTCLOUDCOLORMAP_Ramp *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUDCOLORMAP_Ramp *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_ColorRamp, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_POINTCLOUDCOLORMAP_Ramp_ramps_set" "', argument " "2"" of type '" "Dwg_ColorRamp *""'"); 
    }
    arg2 = (Dwg_ColorRamp *)(argp2);
    if (arg1) (arg1)->ramps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_POINTCLOUDCOLORMAP_Ramp_ramps_get) {
  {
    struct _dwg_POINTCLOUDCOLORMAP_Ramp *arg1 = (struct _dwg_POINTCLOUDCOLORMAP_Ramp *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ColorRamp *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_POINTCLOUDCOLORMAP_Ramp_ramps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUDCOLORMAP_Ramp, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_POINTCLOUDCOLORMAP_Ramp_ramps_get" "', argument " "1"" of type '" "struct _dwg_POINTCLOUDCOLORMAP_Ramp *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUDCOLORMAP_Ramp *)(argp1);
    result = (Dwg_ColorRamp *) ((arg1)->ramps);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_ColorRamp, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_POINTCLOUDCOLORMAP_Ramp) {
  {
    int argvi = 0;
    struct _dwg_POINTCLOUDCOLORMAP_Ramp *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_POINTCLOUDCOLORMAP_Ramp();");
    }
    result = (struct _dwg_POINTCLOUDCOLORMAP_Ramp *)calloc(1, sizeof(struct _dwg_POINTCLOUDCOLORMAP_Ramp));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_POINTCLOUDCOLORMAP_Ramp, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_POINTCLOUDCOLORMAP_Ramp) {
  {
    struct _dwg_POINTCLOUDCOLORMAP_Ramp *arg1 = (struct _dwg_POINTCLOUDCOLORMAP_Ramp *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_POINTCLOUDCOLORMAP_Ramp(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_POINTCLOUDCOLORMAP_Ramp, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_POINTCLOUDCOLORMAP_Ramp" "', argument " "1"" of type '" "struct _dwg_POINTCLOUDCOLORMAP_Ramp *""'"); 
    }
    arg1 = (struct _dwg_POINTCLOUDCOLORMAP_Ramp *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDCOLORMAP_parent_set) {
  {
    struct _dwg_object_POINTCLOUDCOLORMAP *arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDCOLORMAP_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDCOLORMAP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDCOLORMAP_parent_set" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDCOLORMAP *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_POINTCLOUDCOLORMAP_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDCOLORMAP_parent_get) {
  {
    struct _dwg_object_POINTCLOUDCOLORMAP *arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDCOLORMAP_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDCOLORMAP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDCOLORMAP_parent_get" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDCOLORMAP *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDCOLORMAP_class_version_set) {
  {
    struct _dwg_object_POINTCLOUDCOLORMAP *arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDCOLORMAP_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDCOLORMAP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDCOLORMAP_class_version_set" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDCOLORMAP *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_POINTCLOUDCOLORMAP_class_version_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDCOLORMAP_class_version_get) {
  {
    struct _dwg_object_POINTCLOUDCOLORMAP *arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDCOLORMAP_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDCOLORMAP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDCOLORMAP_class_version_get" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDCOLORMAP *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *)(argp1);
    result = (BITCODE_BS) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDCOLORMAP_def_intensity_colorscheme_set) {
  {
    struct _dwg_object_POINTCLOUDCOLORMAP *arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDCOLORMAP_def_intensity_colorscheme_set(self,def_intensity_colorscheme);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDCOLORMAP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDCOLORMAP_def_intensity_colorscheme_set" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDCOLORMAP *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_POINTCLOUDCOLORMAP_def_intensity_colorscheme_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->def_intensity_colorscheme);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->def_intensity_colorscheme = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->def_intensity_colorscheme = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDCOLORMAP_def_intensity_colorscheme_get) {
  {
    struct _dwg_object_POINTCLOUDCOLORMAP *arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDCOLORMAP_def_intensity_colorscheme_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDCOLORMAP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDCOLORMAP_def_intensity_colorscheme_get" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDCOLORMAP *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *)(argp1);
    result = (BITCODE_TV) ((arg1)->def_intensity_colorscheme);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDCOLORMAP_def_elevation_colorscheme_set) {
  {
    struct _dwg_object_POINTCLOUDCOLORMAP *arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDCOLORMAP_def_elevation_colorscheme_set(self,def_elevation_colorscheme);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDCOLORMAP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDCOLORMAP_def_elevation_colorscheme_set" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDCOLORMAP *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_POINTCLOUDCOLORMAP_def_elevation_colorscheme_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->def_elevation_colorscheme);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->def_elevation_colorscheme = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->def_elevation_colorscheme = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDCOLORMAP_def_elevation_colorscheme_get) {
  {
    struct _dwg_object_POINTCLOUDCOLORMAP *arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDCOLORMAP_def_elevation_colorscheme_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDCOLORMAP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDCOLORMAP_def_elevation_colorscheme_get" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDCOLORMAP *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *)(argp1);
    result = (BITCODE_TV) ((arg1)->def_elevation_colorscheme);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDCOLORMAP_def_classification_colorscheme_set) {
  {
    struct _dwg_object_POINTCLOUDCOLORMAP *arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDCOLORMAP_def_classification_colorscheme_set(self,def_classification_colorscheme);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDCOLORMAP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDCOLORMAP_def_classification_colorscheme_set" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDCOLORMAP *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_POINTCLOUDCOLORMAP_def_classification_colorscheme_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->def_classification_colorscheme);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->def_classification_colorscheme = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->def_classification_colorscheme = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDCOLORMAP_def_classification_colorscheme_get) {
  {
    struct _dwg_object_POINTCLOUDCOLORMAP *arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDCOLORMAP_def_classification_colorscheme_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDCOLORMAP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDCOLORMAP_def_classification_colorscheme_get" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDCOLORMAP *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *)(argp1);
    result = (BITCODE_TV) ((arg1)->def_classification_colorscheme);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDCOLORMAP_num_colorramps_set) {
  {
    struct _dwg_object_POINTCLOUDCOLORMAP *arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDCOLORMAP_num_colorramps_set(self,num_colorramps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDCOLORMAP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDCOLORMAP_num_colorramps_set" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDCOLORMAP *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_POINTCLOUDCOLORMAP_num_colorramps_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_colorramps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDCOLORMAP_num_colorramps_get) {
  {
    struct _dwg_object_POINTCLOUDCOLORMAP *arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDCOLORMAP_num_colorramps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDCOLORMAP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDCOLORMAP_num_colorramps_get" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDCOLORMAP *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_colorramps);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDCOLORMAP_colorramps_set) {
  {
    struct _dwg_object_POINTCLOUDCOLORMAP *arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *) 0 ;
    Dwg_POINTCLOUDCOLORMAP_Ramp *arg2 = (Dwg_POINTCLOUDCOLORMAP_Ramp *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDCOLORMAP_colorramps_set(self,colorramps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDCOLORMAP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDCOLORMAP_colorramps_set" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDCOLORMAP *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_POINTCLOUDCOLORMAP_Ramp, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_POINTCLOUDCOLORMAP_colorramps_set" "', argument " "2"" of type '" "Dwg_POINTCLOUDCOLORMAP_Ramp *""'"); 
    }
    arg2 = (Dwg_POINTCLOUDCOLORMAP_Ramp *)(argp2);
    if (arg1) (arg1)->colorramps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDCOLORMAP_colorramps_get) {
  {
    struct _dwg_object_POINTCLOUDCOLORMAP *arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_POINTCLOUDCOLORMAP_Ramp *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDCOLORMAP_colorramps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDCOLORMAP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDCOLORMAP_colorramps_get" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDCOLORMAP *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *)(argp1);
    result = (Dwg_POINTCLOUDCOLORMAP_Ramp *) ((arg1)->colorramps);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_POINTCLOUDCOLORMAP_Ramp, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDCOLORMAP_num_classification_colorramps_set) {
  {
    struct _dwg_object_POINTCLOUDCOLORMAP *arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDCOLORMAP_num_classification_colorramps_set(self,num_classification_colorramps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDCOLORMAP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDCOLORMAP_num_classification_colorramps_set" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDCOLORMAP *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_POINTCLOUDCOLORMAP_num_classification_colorramps_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_classification_colorramps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDCOLORMAP_num_classification_colorramps_get) {
  {
    struct _dwg_object_POINTCLOUDCOLORMAP *arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDCOLORMAP_num_classification_colorramps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDCOLORMAP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDCOLORMAP_num_classification_colorramps_get" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDCOLORMAP *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_classification_colorramps);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDCOLORMAP_classification_colorramps_set) {
  {
    struct _dwg_object_POINTCLOUDCOLORMAP *arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *) 0 ;
    Dwg_POINTCLOUDCOLORMAP_Ramp *arg2 = (Dwg_POINTCLOUDCOLORMAP_Ramp *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDCOLORMAP_classification_colorramps_set(self,classification_colorramps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDCOLORMAP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDCOLORMAP_classification_colorramps_set" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDCOLORMAP *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_POINTCLOUDCOLORMAP_Ramp, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_POINTCLOUDCOLORMAP_classification_colorramps_set" "', argument " "2"" of type '" "Dwg_POINTCLOUDCOLORMAP_Ramp *""'"); 
    }
    arg2 = (Dwg_POINTCLOUDCOLORMAP_Ramp *)(argp2);
    if (arg1) (arg1)->classification_colorramps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_POINTCLOUDCOLORMAP_classification_colorramps_get) {
  {
    struct _dwg_object_POINTCLOUDCOLORMAP *arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_POINTCLOUDCOLORMAP_Ramp *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_POINTCLOUDCOLORMAP_classification_colorramps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDCOLORMAP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_POINTCLOUDCOLORMAP_classification_colorramps_get" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDCOLORMAP *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *)(argp1);
    result = (Dwg_POINTCLOUDCOLORMAP_Ramp *) ((arg1)->classification_colorramps);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_POINTCLOUDCOLORMAP_Ramp, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_POINTCLOUDCOLORMAP) {
  {
    int argvi = 0;
    struct _dwg_object_POINTCLOUDCOLORMAP *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_POINTCLOUDCOLORMAP();");
    }
    result = (struct _dwg_object_POINTCLOUDCOLORMAP *)calloc(1, sizeof(struct _dwg_object_POINTCLOUDCOLORMAP));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_POINTCLOUDCOLORMAP, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_POINTCLOUDCOLORMAP) {
  {
    struct _dwg_object_POINTCLOUDCOLORMAP *arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_POINTCLOUDCOLORMAP(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_POINTCLOUDCOLORMAP, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_POINTCLOUDCOLORMAP" "', argument " "1"" of type '" "struct _dwg_object_POINTCLOUDCOLORMAP *""'"); 
    }
    arg1 = (struct _dwg_object_POINTCLOUDCOLORMAP *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_COMPOUNDOBJECTID_parent_set) {
  {
    struct _dwg_COMPOUNDOBJECTID *arg1 = (struct _dwg_COMPOUNDOBJECTID *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_COMPOUNDOBJECTID_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_COMPOUNDOBJECTID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_COMPOUNDOBJECTID_parent_set" "', argument " "1"" of type '" "struct _dwg_COMPOUNDOBJECTID *""'"); 
    }
    arg1 = (struct _dwg_COMPOUNDOBJECTID *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_COMPOUNDOBJECTID_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_COMPOUNDOBJECTID_parent_get) {
  {
    struct _dwg_COMPOUNDOBJECTID *arg1 = (struct _dwg_COMPOUNDOBJECTID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_COMPOUNDOBJECTID_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_COMPOUNDOBJECTID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_COMPOUNDOBJECTID_parent_get" "', argument " "1"" of type '" "struct _dwg_COMPOUNDOBJECTID *""'"); 
    }
    arg1 = (struct _dwg_COMPOUNDOBJECTID *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_COMPOUNDOBJECTID_has_object_set) {
  {
    struct _dwg_COMPOUNDOBJECTID *arg1 = (struct _dwg_COMPOUNDOBJECTID *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_COMPOUNDOBJECTID_has_object_set(self,has_object);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_COMPOUNDOBJECTID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_COMPOUNDOBJECTID_has_object_set" "', argument " "1"" of type '" "struct _dwg_COMPOUNDOBJECTID *""'"); 
    }
    arg1 = (struct _dwg_COMPOUNDOBJECTID *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_COMPOUNDOBJECTID_has_object_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_object = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_COMPOUNDOBJECTID_has_object_get) {
  {
    struct _dwg_COMPOUNDOBJECTID *arg1 = (struct _dwg_COMPOUNDOBJECTID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_COMPOUNDOBJECTID_has_object_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_COMPOUNDOBJECTID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_COMPOUNDOBJECTID_has_object_get" "', argument " "1"" of type '" "struct _dwg_COMPOUNDOBJECTID *""'"); 
    }
    arg1 = (struct _dwg_COMPOUNDOBJECTID *)(argp1);
    result = (BITCODE_B) ((arg1)->has_object);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_COMPOUNDOBJECTID_name_set) {
  {
    struct _dwg_COMPOUNDOBJECTID *arg1 = (struct _dwg_COMPOUNDOBJECTID *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_COMPOUNDOBJECTID_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_COMPOUNDOBJECTID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_COMPOUNDOBJECTID_name_set" "', argument " "1"" of type '" "struct _dwg_COMPOUNDOBJECTID *""'"); 
    }
    arg1 = (struct _dwg_COMPOUNDOBJECTID *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_COMPOUNDOBJECTID_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_COMPOUNDOBJECTID_name_get) {
  {
    struct _dwg_COMPOUNDOBJECTID *arg1 = (struct _dwg_COMPOUNDOBJECTID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_COMPOUNDOBJECTID_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_COMPOUNDOBJECTID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_COMPOUNDOBJECTID_name_get" "', argument " "1"" of type '" "struct _dwg_COMPOUNDOBJECTID *""'"); 
    }
    arg1 = (struct _dwg_COMPOUNDOBJECTID *)(argp1);
    result = (BITCODE_TV) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_COMPOUNDOBJECTID_object_set) {
  {
    struct _dwg_COMPOUNDOBJECTID *arg1 = (struct _dwg_COMPOUNDOBJECTID *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_COMPOUNDOBJECTID_object_set(self,object);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_COMPOUNDOBJECTID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_COMPOUNDOBJECTID_object_set" "', argument " "1"" of type '" "struct _dwg_COMPOUNDOBJECTID *""'"); 
    }
    arg1 = (struct _dwg_COMPOUNDOBJECTID *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_COMPOUNDOBJECTID_object_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->object = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_COMPOUNDOBJECTID_object_get) {
  {
    struct _dwg_COMPOUNDOBJECTID *arg1 = (struct _dwg_COMPOUNDOBJECTID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_COMPOUNDOBJECTID_object_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_COMPOUNDOBJECTID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_COMPOUNDOBJECTID_object_get" "', argument " "1"" of type '" "struct _dwg_COMPOUNDOBJECTID *""'"); 
    }
    arg1 = (struct _dwg_COMPOUNDOBJECTID *)(argp1);
    result = (BITCODE_H) ((arg1)->object);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_COMPOUNDOBJECTID) {
  {
    int argvi = 0;
    struct _dwg_COMPOUNDOBJECTID *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_COMPOUNDOBJECTID();");
    }
    result = (struct _dwg_COMPOUNDOBJECTID *)calloc(1, sizeof(struct _dwg_COMPOUNDOBJECTID));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_COMPOUNDOBJECTID, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_COMPOUNDOBJECTID) {
  {
    struct _dwg_COMPOUNDOBJECTID *arg1 = (struct _dwg_COMPOUNDOBJECTID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_COMPOUNDOBJECTID(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_COMPOUNDOBJECTID, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_COMPOUNDOBJECTID" "', argument " "1"" of type '" "struct _dwg_COMPOUNDOBJECTID *""'"); 
    }
    arg1 = (struct _dwg_COMPOUNDOBJECTID *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PARTIAL_VIEWING_INDEX_Entry_parent_set) {
  {
    struct _dwg_PARTIAL_VIEWING_INDEX_Entry *arg1 = (struct _dwg_PARTIAL_VIEWING_INDEX_Entry *) 0 ;
    struct _dwg_object_PARTIAL_VIEWING_INDEX *arg2 = (struct _dwg_object_PARTIAL_VIEWING_INDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_PARTIAL_VIEWING_INDEX_Entry_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PARTIAL_VIEWING_INDEX_Entry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PARTIAL_VIEWING_INDEX_Entry_parent_set" "', argument " "1"" of type '" "struct _dwg_PARTIAL_VIEWING_INDEX_Entry *""'"); 
    }
    arg1 = (struct _dwg_PARTIAL_VIEWING_INDEX_Entry *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_PARTIAL_VIEWING_INDEX, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_PARTIAL_VIEWING_INDEX_Entry_parent_set" "', argument " "2"" of type '" "struct _dwg_object_PARTIAL_VIEWING_INDEX *""'"); 
    }
    arg2 = (struct _dwg_object_PARTIAL_VIEWING_INDEX *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PARTIAL_VIEWING_INDEX_Entry_parent_get) {
  {
    struct _dwg_PARTIAL_VIEWING_INDEX_Entry *arg1 = (struct _dwg_PARTIAL_VIEWING_INDEX_Entry *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_PARTIAL_VIEWING_INDEX *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_PARTIAL_VIEWING_INDEX_Entry_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PARTIAL_VIEWING_INDEX_Entry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PARTIAL_VIEWING_INDEX_Entry_parent_get" "', argument " "1"" of type '" "struct _dwg_PARTIAL_VIEWING_INDEX_Entry *""'"); 
    }
    arg1 = (struct _dwg_PARTIAL_VIEWING_INDEX_Entry *)(argp1);
    result = (struct _dwg_object_PARTIAL_VIEWING_INDEX *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_PARTIAL_VIEWING_INDEX, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PARTIAL_VIEWING_INDEX_Entry_extents_min_set) {
  {
    struct _dwg_PARTIAL_VIEWING_INDEX_Entry *arg1 = (struct _dwg_PARTIAL_VIEWING_INDEX_Entry *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_PARTIAL_VIEWING_INDEX_Entry_extents_min_set(self,extents_min);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PARTIAL_VIEWING_INDEX_Entry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PARTIAL_VIEWING_INDEX_Entry_extents_min_set" "', argument " "1"" of type '" "struct _dwg_PARTIAL_VIEWING_INDEX_Entry *""'"); 
    }
    arg1 = (struct _dwg_PARTIAL_VIEWING_INDEX_Entry *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_PARTIAL_VIEWING_INDEX_Entry_extents_min_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->extents_min = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PARTIAL_VIEWING_INDEX_Entry_extents_min_get) {
  {
    struct _dwg_PARTIAL_VIEWING_INDEX_Entry *arg1 = (struct _dwg_PARTIAL_VIEWING_INDEX_Entry *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_PARTIAL_VIEWING_INDEX_Entry_extents_min_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PARTIAL_VIEWING_INDEX_Entry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PARTIAL_VIEWING_INDEX_Entry_extents_min_get" "', argument " "1"" of type '" "struct _dwg_PARTIAL_VIEWING_INDEX_Entry *""'"); 
    }
    arg1 = (struct _dwg_PARTIAL_VIEWING_INDEX_Entry *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->extents_min);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PARTIAL_VIEWING_INDEX_Entry_extents_max_set) {
  {
    struct _dwg_PARTIAL_VIEWING_INDEX_Entry *arg1 = (struct _dwg_PARTIAL_VIEWING_INDEX_Entry *) 0 ;
    BITCODE_3BD *arg2 = (BITCODE_3BD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_PARTIAL_VIEWING_INDEX_Entry_extents_max_set(self,extents_max);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PARTIAL_VIEWING_INDEX_Entry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PARTIAL_VIEWING_INDEX_Entry_extents_max_set" "', argument " "1"" of type '" "struct _dwg_PARTIAL_VIEWING_INDEX_Entry *""'"); 
    }
    arg1 = (struct _dwg_PARTIAL_VIEWING_INDEX_Entry *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3bd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_PARTIAL_VIEWING_INDEX_Entry_extents_max_set" "', argument " "2"" of type '" "BITCODE_3BD *""'"); 
    }
    arg2 = (BITCODE_3BD *)(argp2);
    if (arg1) (arg1)->extents_max = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PARTIAL_VIEWING_INDEX_Entry_extents_max_get) {
  {
    struct _dwg_PARTIAL_VIEWING_INDEX_Entry *arg1 = (struct _dwg_PARTIAL_VIEWING_INDEX_Entry *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3BD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_PARTIAL_VIEWING_INDEX_Entry_extents_max_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PARTIAL_VIEWING_INDEX_Entry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PARTIAL_VIEWING_INDEX_Entry_extents_max_get" "', argument " "1"" of type '" "struct _dwg_PARTIAL_VIEWING_INDEX_Entry *""'"); 
    }
    arg1 = (struct _dwg_PARTIAL_VIEWING_INDEX_Entry *)(argp1);
    result = (BITCODE_3BD *)& ((arg1)->extents_max);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3bd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PARTIAL_VIEWING_INDEX_Entry_object_set) {
  {
    struct _dwg_PARTIAL_VIEWING_INDEX_Entry *arg1 = (struct _dwg_PARTIAL_VIEWING_INDEX_Entry *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_PARTIAL_VIEWING_INDEX_Entry_object_set(self,object);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PARTIAL_VIEWING_INDEX_Entry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PARTIAL_VIEWING_INDEX_Entry_object_set" "', argument " "1"" of type '" "struct _dwg_PARTIAL_VIEWING_INDEX_Entry *""'"); 
    }
    arg1 = (struct _dwg_PARTIAL_VIEWING_INDEX_Entry *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_PARTIAL_VIEWING_INDEX_Entry_object_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->object = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_PARTIAL_VIEWING_INDEX_Entry_object_get) {
  {
    struct _dwg_PARTIAL_VIEWING_INDEX_Entry *arg1 = (struct _dwg_PARTIAL_VIEWING_INDEX_Entry *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_PARTIAL_VIEWING_INDEX_Entry_object_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PARTIAL_VIEWING_INDEX_Entry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_PARTIAL_VIEWING_INDEX_Entry_object_get" "', argument " "1"" of type '" "struct _dwg_PARTIAL_VIEWING_INDEX_Entry *""'"); 
    }
    arg1 = (struct _dwg_PARTIAL_VIEWING_INDEX_Entry *)(argp1);
    result = (BITCODE_H) ((arg1)->object);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_PARTIAL_VIEWING_INDEX_Entry) {
  {
    int argvi = 0;
    struct _dwg_PARTIAL_VIEWING_INDEX_Entry *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_PARTIAL_VIEWING_INDEX_Entry();");
    }
    result = (struct _dwg_PARTIAL_VIEWING_INDEX_Entry *)calloc(1, sizeof(struct _dwg_PARTIAL_VIEWING_INDEX_Entry));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_PARTIAL_VIEWING_INDEX_Entry, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_PARTIAL_VIEWING_INDEX_Entry) {
  {
    struct _dwg_PARTIAL_VIEWING_INDEX_Entry *arg1 = (struct _dwg_PARTIAL_VIEWING_INDEX_Entry *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_PARTIAL_VIEWING_INDEX_Entry(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_PARTIAL_VIEWING_INDEX_Entry, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_PARTIAL_VIEWING_INDEX_Entry" "', argument " "1"" of type '" "struct _dwg_PARTIAL_VIEWING_INDEX_Entry *""'"); 
    }
    arg1 = (struct _dwg_PARTIAL_VIEWING_INDEX_Entry *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PARTIAL_VIEWING_INDEX_parent_set) {
  {
    struct _dwg_object_PARTIAL_VIEWING_INDEX *arg1 = (struct _dwg_object_PARTIAL_VIEWING_INDEX *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PARTIAL_VIEWING_INDEX_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PARTIAL_VIEWING_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PARTIAL_VIEWING_INDEX_parent_set" "', argument " "1"" of type '" "struct _dwg_object_PARTIAL_VIEWING_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_PARTIAL_VIEWING_INDEX *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_PARTIAL_VIEWING_INDEX_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PARTIAL_VIEWING_INDEX_parent_get) {
  {
    struct _dwg_object_PARTIAL_VIEWING_INDEX *arg1 = (struct _dwg_object_PARTIAL_VIEWING_INDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PARTIAL_VIEWING_INDEX_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PARTIAL_VIEWING_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PARTIAL_VIEWING_INDEX_parent_get" "', argument " "1"" of type '" "struct _dwg_object_PARTIAL_VIEWING_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_PARTIAL_VIEWING_INDEX *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PARTIAL_VIEWING_INDEX_num_entries_set) {
  {
    struct _dwg_object_PARTIAL_VIEWING_INDEX *arg1 = (struct _dwg_object_PARTIAL_VIEWING_INDEX *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PARTIAL_VIEWING_INDEX_num_entries_set(self,num_entries);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PARTIAL_VIEWING_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PARTIAL_VIEWING_INDEX_num_entries_set" "', argument " "1"" of type '" "struct _dwg_object_PARTIAL_VIEWING_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_PARTIAL_VIEWING_INDEX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_PARTIAL_VIEWING_INDEX_num_entries_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_entries = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PARTIAL_VIEWING_INDEX_num_entries_get) {
  {
    struct _dwg_object_PARTIAL_VIEWING_INDEX *arg1 = (struct _dwg_object_PARTIAL_VIEWING_INDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PARTIAL_VIEWING_INDEX_num_entries_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PARTIAL_VIEWING_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PARTIAL_VIEWING_INDEX_num_entries_get" "', argument " "1"" of type '" "struct _dwg_object_PARTIAL_VIEWING_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_PARTIAL_VIEWING_INDEX *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_entries);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PARTIAL_VIEWING_INDEX_has_entries_set) {
  {
    struct _dwg_object_PARTIAL_VIEWING_INDEX *arg1 = (struct _dwg_object_PARTIAL_VIEWING_INDEX *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PARTIAL_VIEWING_INDEX_has_entries_set(self,has_entries);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PARTIAL_VIEWING_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PARTIAL_VIEWING_INDEX_has_entries_set" "', argument " "1"" of type '" "struct _dwg_object_PARTIAL_VIEWING_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_PARTIAL_VIEWING_INDEX *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_PARTIAL_VIEWING_INDEX_has_entries_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_entries = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PARTIAL_VIEWING_INDEX_has_entries_get) {
  {
    struct _dwg_object_PARTIAL_VIEWING_INDEX *arg1 = (struct _dwg_object_PARTIAL_VIEWING_INDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PARTIAL_VIEWING_INDEX_has_entries_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PARTIAL_VIEWING_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PARTIAL_VIEWING_INDEX_has_entries_get" "', argument " "1"" of type '" "struct _dwg_object_PARTIAL_VIEWING_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_PARTIAL_VIEWING_INDEX *)(argp1);
    result = (BITCODE_B) ((arg1)->has_entries);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PARTIAL_VIEWING_INDEX_entries_set) {
  {
    struct _dwg_object_PARTIAL_VIEWING_INDEX *arg1 = (struct _dwg_object_PARTIAL_VIEWING_INDEX *) 0 ;
    Dwg_PARTIAL_VIEWING_INDEX_Entry *arg2 = (Dwg_PARTIAL_VIEWING_INDEX_Entry *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_PARTIAL_VIEWING_INDEX_entries_set(self,entries);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PARTIAL_VIEWING_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PARTIAL_VIEWING_INDEX_entries_set" "', argument " "1"" of type '" "struct _dwg_object_PARTIAL_VIEWING_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_PARTIAL_VIEWING_INDEX *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_PARTIAL_VIEWING_INDEX_Entry, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_PARTIAL_VIEWING_INDEX_entries_set" "', argument " "2"" of type '" "Dwg_PARTIAL_VIEWING_INDEX_Entry *""'"); 
    }
    arg2 = (Dwg_PARTIAL_VIEWING_INDEX_Entry *)(argp2);
    if (arg1) (arg1)->entries = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_PARTIAL_VIEWING_INDEX_entries_get) {
  {
    struct _dwg_object_PARTIAL_VIEWING_INDEX *arg1 = (struct _dwg_object_PARTIAL_VIEWING_INDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_PARTIAL_VIEWING_INDEX_Entry *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_PARTIAL_VIEWING_INDEX_entries_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PARTIAL_VIEWING_INDEX, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_PARTIAL_VIEWING_INDEX_entries_get" "', argument " "1"" of type '" "struct _dwg_object_PARTIAL_VIEWING_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_PARTIAL_VIEWING_INDEX *)(argp1);
    result = (Dwg_PARTIAL_VIEWING_INDEX_Entry *) ((arg1)->entries);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_PARTIAL_VIEWING_INDEX_Entry, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_PARTIAL_VIEWING_INDEX) {
  {
    int argvi = 0;
    struct _dwg_object_PARTIAL_VIEWING_INDEX *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_PARTIAL_VIEWING_INDEX();");
    }
    result = (struct _dwg_object_PARTIAL_VIEWING_INDEX *)calloc(1, sizeof(struct _dwg_object_PARTIAL_VIEWING_INDEX));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_PARTIAL_VIEWING_INDEX, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_PARTIAL_VIEWING_INDEX) {
  {
    struct _dwg_object_PARTIAL_VIEWING_INDEX *arg1 = (struct _dwg_object_PARTIAL_VIEWING_INDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_PARTIAL_VIEWING_INDEX(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_PARTIAL_VIEWING_INDEX, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_PARTIAL_VIEWING_INDEX" "', argument " "1"" of type '" "struct _dwg_object_PARTIAL_VIEWING_INDEX *""'"); 
    }
    arg1 = (struct _dwg_object_PARTIAL_VIEWING_INDEX *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_UNKNOWN_ENT_parent_set) {
  {
    struct _dwg_entity_UNKNOWN_ENT *arg1 = (struct _dwg_entity_UNKNOWN_ENT *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_UNKNOWN_ENT_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_UNKNOWN_ENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_UNKNOWN_ENT_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_UNKNOWN_ENT *""'"); 
    }
    arg1 = (struct _dwg_entity_UNKNOWN_ENT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_UNKNOWN_ENT_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_UNKNOWN_ENT_parent_get) {
  {
    struct _dwg_entity_UNKNOWN_ENT *arg1 = (struct _dwg_entity_UNKNOWN_ENT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_UNKNOWN_ENT_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_UNKNOWN_ENT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_UNKNOWN_ENT_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_UNKNOWN_ENT *""'"); 
    }
    arg1 = (struct _dwg_entity_UNKNOWN_ENT *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_UNKNOWN_ENT) {
  {
    int argvi = 0;
    struct _dwg_entity_UNKNOWN_ENT *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_UNKNOWN_ENT();");
    }
    result = (struct _dwg_entity_UNKNOWN_ENT *)calloc(1, sizeof(struct _dwg_entity_UNKNOWN_ENT));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_UNKNOWN_ENT, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_UNKNOWN_ENT) {
  {
    struct _dwg_entity_UNKNOWN_ENT *arg1 = (struct _dwg_entity_UNKNOWN_ENT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_UNKNOWN_ENT(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_UNKNOWN_ENT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_UNKNOWN_ENT" "', argument " "1"" of type '" "struct _dwg_entity_UNKNOWN_ENT *""'"); 
    }
    arg1 = (struct _dwg_entity_UNKNOWN_ENT *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UNKNOWN_OBJ_parent_set) {
  {
    struct _dwg_object_UNKNOWN_OBJ *arg1 = (struct _dwg_object_UNKNOWN_OBJ *) 0 ;
    struct _dwg_object_object *arg2 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_UNKNOWN_OBJ_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UNKNOWN_OBJ, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UNKNOWN_OBJ_parent_set" "', argument " "1"" of type '" "struct _dwg_object_UNKNOWN_OBJ *""'"); 
    }
    arg1 = (struct _dwg_object_UNKNOWN_OBJ *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_UNKNOWN_OBJ_parent_set" "', argument " "2"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg2 = (struct _dwg_object_object *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_UNKNOWN_OBJ_parent_get) {
  {
    struct _dwg_object_UNKNOWN_OBJ *arg1 = (struct _dwg_object_UNKNOWN_OBJ *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_UNKNOWN_OBJ_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UNKNOWN_OBJ, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_UNKNOWN_OBJ_parent_get" "', argument " "1"" of type '" "struct _dwg_object_UNKNOWN_OBJ *""'"); 
    }
    arg1 = (struct _dwg_object_UNKNOWN_OBJ *)(argp1);
    result = (struct _dwg_object_object *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_UNKNOWN_OBJ) {
  {
    int argvi = 0;
    struct _dwg_object_UNKNOWN_OBJ *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_UNKNOWN_OBJ();");
    }
    result = (struct _dwg_object_UNKNOWN_OBJ *)calloc(1, sizeof(struct _dwg_object_UNKNOWN_OBJ));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_UNKNOWN_OBJ, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_UNKNOWN_OBJ) {
  {
    struct _dwg_object_UNKNOWN_OBJ *arg1 = (struct _dwg_object_UNKNOWN_OBJ *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_UNKNOWN_OBJ(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_UNKNOWN_OBJ, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_UNKNOWN_OBJ" "', argument " "1"" of type '" "struct _dwg_object_UNKNOWN_OBJ *""'"); 
    }
    arg1 = (struct _dwg_object_UNKNOWN_OBJ *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REPEAT_parent_set) {
  {
    struct _dwg_entity_REPEAT *arg1 = (struct _dwg_entity_REPEAT *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_REPEAT_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REPEAT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REPEAT_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_REPEAT *""'"); 
    }
    arg1 = (struct _dwg_entity_REPEAT *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_REPEAT_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_REPEAT_parent_get) {
  {
    struct _dwg_entity_REPEAT *arg1 = (struct _dwg_entity_REPEAT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_REPEAT_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REPEAT, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_REPEAT_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_REPEAT *""'"); 
    }
    arg1 = (struct _dwg_entity_REPEAT *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_REPEAT) {
  {
    int argvi = 0;
    struct _dwg_entity_REPEAT *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_REPEAT();");
    }
    result = (struct _dwg_entity_REPEAT *)calloc(1, sizeof(struct _dwg_entity_REPEAT));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_REPEAT, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_REPEAT) {
  {
    struct _dwg_entity_REPEAT *arg1 = (struct _dwg_entity_REPEAT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_REPEAT(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_REPEAT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_REPEAT" "', argument " "1"" of type '" "struct _dwg_entity_REPEAT *""'"); 
    }
    arg1 = (struct _dwg_entity_REPEAT *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ENDREP_parent_set) {
  {
    struct _dwg_entity_ENDREP *arg1 = (struct _dwg_entity_ENDREP *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ENDREP_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ENDREP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ENDREP_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_ENDREP *""'"); 
    }
    arg1 = (struct _dwg_entity_ENDREP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_ENDREP_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ENDREP_parent_get) {
  {
    struct _dwg_entity_ENDREP *arg1 = (struct _dwg_entity_ENDREP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ENDREP_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ENDREP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ENDREP_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_ENDREP *""'"); 
    }
    arg1 = (struct _dwg_entity_ENDREP *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ENDREP_numcols_set) {
  {
    struct _dwg_entity_ENDREP *arg1 = (struct _dwg_entity_ENDREP *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ENDREP_numcols_set(self,numcols);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ENDREP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ENDREP_numcols_set" "', argument " "1"" of type '" "struct _dwg_entity_ENDREP *""'"); 
    }
    arg1 = (struct _dwg_entity_ENDREP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ENDREP_numcols_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->numcols = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ENDREP_numcols_get) {
  {
    struct _dwg_entity_ENDREP *arg1 = (struct _dwg_entity_ENDREP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ENDREP_numcols_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ENDREP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ENDREP_numcols_get" "', argument " "1"" of type '" "struct _dwg_entity_ENDREP *""'"); 
    }
    arg1 = (struct _dwg_entity_ENDREP *)(argp1);
    result = (BITCODE_RS) ((arg1)->numcols);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ENDREP_numrows_set) {
  {
    struct _dwg_entity_ENDREP *arg1 = (struct _dwg_entity_ENDREP *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ENDREP_numrows_set(self,numrows);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ENDREP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ENDREP_numrows_set" "', argument " "1"" of type '" "struct _dwg_entity_ENDREP *""'"); 
    }
    arg1 = (struct _dwg_entity_ENDREP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ENDREP_numrows_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->numrows = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ENDREP_numrows_get) {
  {
    struct _dwg_entity_ENDREP *arg1 = (struct _dwg_entity_ENDREP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ENDREP_numrows_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ENDREP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ENDREP_numrows_get" "', argument " "1"" of type '" "struct _dwg_entity_ENDREP *""'"); 
    }
    arg1 = (struct _dwg_entity_ENDREP *)(argp1);
    result = (BITCODE_RS) ((arg1)->numrows);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ENDREP_colspacing_set) {
  {
    struct _dwg_entity_ENDREP *arg1 = (struct _dwg_entity_ENDREP *) 0 ;
    BITCODE_RD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ENDREP_colspacing_set(self,colspacing);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ENDREP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ENDREP_colspacing_set" "', argument " "1"" of type '" "struct _dwg_entity_ENDREP *""'"); 
    }
    arg1 = (struct _dwg_entity_ENDREP *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ENDREP_colspacing_set" "', argument " "2"" of type '" "BITCODE_RD""'");
    } 
    arg2 = (BITCODE_RD)(val2);
    if (arg1) (arg1)->colspacing = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ENDREP_colspacing_get) {
  {
    struct _dwg_entity_ENDREP *arg1 = (struct _dwg_entity_ENDREP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ENDREP_colspacing_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ENDREP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ENDREP_colspacing_get" "', argument " "1"" of type '" "struct _dwg_entity_ENDREP *""'"); 
    }
    arg1 = (struct _dwg_entity_ENDREP *)(argp1);
    result = (BITCODE_RD) ((arg1)->colspacing);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ENDREP_rowspacing_set) {
  {
    struct _dwg_entity_ENDREP *arg1 = (struct _dwg_entity_ENDREP *) 0 ;
    BITCODE_RD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_ENDREP_rowspacing_set(self,rowspacing);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ENDREP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ENDREP_rowspacing_set" "', argument " "1"" of type '" "struct _dwg_entity_ENDREP *""'"); 
    }
    arg1 = (struct _dwg_entity_ENDREP *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_ENDREP_rowspacing_set" "', argument " "2"" of type '" "BITCODE_RD""'");
    } 
    arg2 = (BITCODE_RD)(val2);
    if (arg1) (arg1)->rowspacing = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_ENDREP_rowspacing_get) {
  {
    struct _dwg_entity_ENDREP *arg1 = (struct _dwg_entity_ENDREP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_ENDREP_rowspacing_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ENDREP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_ENDREP_rowspacing_get" "', argument " "1"" of type '" "struct _dwg_entity_ENDREP *""'"); 
    }
    arg1 = (struct _dwg_entity_ENDREP *)(argp1);
    result = (BITCODE_RD) ((arg1)->rowspacing);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_ENDREP) {
  {
    int argvi = 0;
    struct _dwg_entity_ENDREP *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_ENDREP();");
    }
    result = (struct _dwg_entity_ENDREP *)calloc(1, sizeof(struct _dwg_entity_ENDREP));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_ENDREP, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_ENDREP) {
  {
    struct _dwg_entity_ENDREP *arg1 = (struct _dwg_entity_ENDREP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_ENDREP(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_ENDREP, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_ENDREP" "', argument " "1"" of type '" "struct _dwg_entity_ENDREP *""'"); 
    }
    arg1 = (struct _dwg_entity_ENDREP *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOAD_parent_set) {
  {
    struct _dwg_entity_LOAD *arg1 = (struct _dwg_entity_LOAD *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOAD_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOAD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOAD_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_LOAD *""'"); 
    }
    arg1 = (struct _dwg_entity_LOAD *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LOAD_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOAD_parent_get) {
  {
    struct _dwg_entity_LOAD *arg1 = (struct _dwg_entity_LOAD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOAD_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOAD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOAD_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_LOAD *""'"); 
    }
    arg1 = (struct _dwg_entity_LOAD *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOAD_file_name_set) {
  {
    struct _dwg_entity_LOAD *arg1 = (struct _dwg_entity_LOAD *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_LOAD_file_name_set(self,file_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOAD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOAD_file_name_set" "', argument " "1"" of type '" "struct _dwg_entity_LOAD *""'"); 
    }
    arg1 = (struct _dwg_entity_LOAD *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_LOAD_file_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->file_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->file_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->file_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_LOAD_file_name_get) {
  {
    struct _dwg_entity_LOAD *arg1 = (struct _dwg_entity_LOAD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_LOAD_file_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOAD, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_LOAD_file_name_get" "', argument " "1"" of type '" "struct _dwg_entity_LOAD *""'"); 
    }
    arg1 = (struct _dwg_entity_LOAD *)(argp1);
    result = (BITCODE_TV) ((arg1)->file_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_LOAD) {
  {
    int argvi = 0;
    struct _dwg_entity_LOAD *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_LOAD();");
    }
    result = (struct _dwg_entity_LOAD *)calloc(1, sizeof(struct _dwg_entity_LOAD));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_LOAD, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_LOAD) {
  {
    struct _dwg_entity_LOAD *arg1 = (struct _dwg_entity_LOAD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_LOAD(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_LOAD, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_LOAD" "', argument " "1"" of type '" "struct _dwg_entity_LOAD *""'"); 
    }
    arg1 = (struct _dwg_entity_LOAD *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_JUMP_parent_set) {
  {
    struct _dwg_entity_JUMP *arg1 = (struct _dwg_entity_JUMP *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_JUMP_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_JUMP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_JUMP_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_JUMP *""'"); 
    }
    arg1 = (struct _dwg_entity_JUMP *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity_JUMP_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_JUMP_parent_get) {
  {
    struct _dwg_entity_JUMP *arg1 = (struct _dwg_entity_JUMP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_JUMP_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_JUMP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_JUMP_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_JUMP *""'"); 
    }
    arg1 = (struct _dwg_entity_JUMP *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_JUMP_jump_address_raw_set) {
  {
    struct _dwg_entity_JUMP *arg1 = (struct _dwg_entity_JUMP *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_JUMP_jump_address_raw_set(self,jump_address_raw);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_JUMP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_JUMP_jump_address_raw_set" "', argument " "1"" of type '" "struct _dwg_entity_JUMP *""'"); 
    }
    arg1 = (struct _dwg_entity_JUMP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_JUMP_jump_address_raw_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->jump_address_raw = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_JUMP_jump_address_raw_get) {
  {
    struct _dwg_entity_JUMP *arg1 = (struct _dwg_entity_JUMP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_JUMP_jump_address_raw_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_JUMP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_JUMP_jump_address_raw_get" "', argument " "1"" of type '" "struct _dwg_entity_JUMP *""'"); 
    }
    arg1 = (struct _dwg_entity_JUMP *)(argp1);
    result = (BITCODE_RL) ((arg1)->jump_address_raw);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_JUMP_jump_address_set) {
  {
    struct _dwg_entity_JUMP *arg1 = (struct _dwg_entity_JUMP *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_JUMP_jump_address_set(self,jump_address);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_JUMP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_JUMP_jump_address_set" "', argument " "1"" of type '" "struct _dwg_entity_JUMP *""'"); 
    }
    arg1 = (struct _dwg_entity_JUMP *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_JUMP_jump_address_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->jump_address = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_JUMP_jump_address_get) {
  {
    struct _dwg_entity_JUMP *arg1 = (struct _dwg_entity_JUMP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_JUMP_jump_address_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_JUMP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_JUMP_jump_address_get" "', argument " "1"" of type '" "struct _dwg_entity_JUMP *""'"); 
    }
    arg1 = (struct _dwg_entity_JUMP *)(argp1);
    result = (BITCODE_RL) ((arg1)->jump_address);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_JUMP_jump_entity_section_set) {
  {
    struct _dwg_entity_JUMP *arg1 = (struct _dwg_entity_JUMP *) 0 ;
    Dwg_Entity_Sections arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity_JUMP_jump_entity_section_set(self,jump_entity_section);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_JUMP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_JUMP_jump_entity_section_set" "', argument " "1"" of type '" "struct _dwg_entity_JUMP *""'"); 
    }
    arg1 = (struct _dwg_entity_JUMP *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity_JUMP_jump_entity_section_set" "', argument " "2"" of type '" "Dwg_Entity_Sections""'");
    } 
    arg2 = (Dwg_Entity_Sections)(val2);
    if (arg1) (arg1)->jump_entity_section = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity_JUMP_jump_entity_section_get) {
  {
    struct _dwg_entity_JUMP *arg1 = (struct _dwg_entity_JUMP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_Sections result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity_JUMP_jump_entity_section_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_JUMP, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity_JUMP_jump_entity_section_get" "', argument " "1"" of type '" "struct _dwg_entity_JUMP *""'"); 
    }
    arg1 = (struct _dwg_entity_JUMP *)(argp1);
    result = (Dwg_Entity_Sections) ((arg1)->jump_entity_section);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity_JUMP) {
  {
    int argvi = 0;
    struct _dwg_entity_JUMP *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity_JUMP();");
    }
    result = (struct _dwg_entity_JUMP *)calloc(1, sizeof(struct _dwg_entity_JUMP));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_JUMP, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity_JUMP) {
  {
    struct _dwg_entity_JUMP *arg1 = (struct _dwg_entity_JUMP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity_JUMP(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_JUMP, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity_JUMP" "', argument " "1"" of type '" "struct _dwg_entity_JUMP *""'"); 
    }
    arg1 = (struct _dwg_entity_JUMP *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DLINE_parent_set) {
  {
    struct _dwg_entity_3DLINE *arg1 = (struct _dwg_entity_3DLINE *) 0 ;
    struct _dwg_object_entity *arg2 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DLINE_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DLINE_parent_set" "', argument " "1"" of type '" "struct _dwg_entity_3DLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_3DLINE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity__3DLINE_parent_set" "', argument " "2"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg2 = (struct _dwg_object_entity *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DLINE_parent_get) {
  {
    struct _dwg_entity_3DLINE *arg1 = (struct _dwg_entity_3DLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DLINE_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DLINE_parent_get" "', argument " "1"" of type '" "struct _dwg_entity_3DLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_3DLINE *)(argp1);
    result = (struct _dwg_object_entity *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DLINE_start_set) {
  {
    struct _dwg_entity_3DLINE *arg1 = (struct _dwg_entity_3DLINE *) 0 ;
    BITCODE_3RD *arg2 = (BITCODE_3RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DLINE_start_set(self,start);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DLINE_start_set" "', argument " "1"" of type '" "struct _dwg_entity_3DLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_3DLINE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity__3DLINE_start_set" "', argument " "2"" of type '" "BITCODE_3RD *""'"); 
    }
    arg2 = (BITCODE_3RD *)(argp2);
    if (arg1) (arg1)->start = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DLINE_start_get) {
  {
    struct _dwg_entity_3DLINE *arg1 = (struct _dwg_entity_3DLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DLINE_start_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DLINE_start_get" "', argument " "1"" of type '" "struct _dwg_entity_3DLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_3DLINE *)(argp1);
    result = (BITCODE_3RD *)& ((arg1)->start);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DLINE_end_set) {
  {
    struct _dwg_entity_3DLINE *arg1 = (struct _dwg_entity_3DLINE *) 0 ;
    BITCODE_3RD *arg2 = (BITCODE_3RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DLINE_end_set(self,end);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DLINE_end_set" "', argument " "1"" of type '" "struct _dwg_entity_3DLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_3DLINE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity__3DLINE_end_set" "', argument " "2"" of type '" "BITCODE_3RD *""'"); 
    }
    arg2 = (BITCODE_3RD *)(argp2);
    if (arg1) (arg1)->end = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DLINE_end_get) {
  {
    struct _dwg_entity_3DLINE *arg1 = (struct _dwg_entity_3DLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DLINE_end_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DLINE_end_get" "', argument " "1"" of type '" "struct _dwg_entity_3DLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_3DLINE *)(argp1);
    result = (BITCODE_3RD *)& ((arg1)->end);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DLINE_extrusion_set) {
  {
    struct _dwg_entity_3DLINE *arg1 = (struct _dwg_entity_3DLINE *) 0 ;
    BITCODE_3RD *arg2 = (BITCODE_3RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DLINE_extrusion_set(self,extrusion);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DLINE_extrusion_set" "', argument " "1"" of type '" "struct _dwg_entity_3DLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_3DLINE *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Entity__3DLINE_extrusion_set" "', argument " "2"" of type '" "BITCODE_3RD *""'"); 
    }
    arg2 = (BITCODE_3RD *)(argp2);
    if (arg1) (arg1)->extrusion = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DLINE_extrusion_get) {
  {
    struct _dwg_entity_3DLINE *arg1 = (struct _dwg_entity_3DLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DLINE_extrusion_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DLINE_extrusion_get" "', argument " "1"" of type '" "struct _dwg_entity_3DLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_3DLINE *)(argp1);
    result = (BITCODE_3RD *)& ((arg1)->extrusion);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DLINE_thickness_set) {
  {
    struct _dwg_entity_3DLINE *arg1 = (struct _dwg_entity_3DLINE *) 0 ;
    BITCODE_RD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Entity__3DLINE_thickness_set(self,thickness);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DLINE_thickness_set" "', argument " "1"" of type '" "struct _dwg_entity_3DLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_3DLINE *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Entity__3DLINE_thickness_set" "', argument " "2"" of type '" "BITCODE_RD""'");
    } 
    arg2 = (BITCODE_RD)(val2);
    if (arg1) (arg1)->thickness = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Entity__3DLINE_thickness_get) {
  {
    struct _dwg_entity_3DLINE *arg1 = (struct _dwg_entity_3DLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Entity__3DLINE_thickness_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DLINE, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Entity__3DLINE_thickness_get" "', argument " "1"" of type '" "struct _dwg_entity_3DLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_3DLINE *)(argp1);
    result = (BITCODE_RD) ((arg1)->thickness);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Entity__3DLINE) {
  {
    int argvi = 0;
    struct _dwg_entity_3DLINE *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Entity__3DLINE();");
    }
    result = (struct _dwg_entity_3DLINE *)calloc(1, sizeof(struct _dwg_entity_3DLINE));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_3DLINE, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Entity__3DLINE) {
  {
    struct _dwg_entity_3DLINE *arg1 = (struct _dwg_entity_3DLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Entity__3DLINE(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DLINE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Entity__3DLINE" "', argument " "1"" of type '" "struct _dwg_entity_3DLINE *""'"); 
    }
    arg1 = (struct _dwg_entity_3DLINE *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Eed_Data_u_set) {
  {
    struct _dwg_entity_eed_data *arg1 = (struct _dwg_entity_eed_data *) 0 ;
    union eed_data_t *arg2 = (union eed_data_t *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Eed_Data_u_set(self,u);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_eed_data, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Eed_Data_u_set" "', argument " "1"" of type '" "struct _dwg_entity_eed_data *""'"); 
    }
    arg1 = (struct _dwg_entity_eed_data *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_eed_data_t, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Eed_Data_u_set" "', argument " "2"" of type '" "union eed_data_t *""'"); 
    }
    arg2 = (union eed_data_t *)(argp2);
    if (arg1) (arg1)->u = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Eed_Data_u_get) {
  {
    struct _dwg_entity_eed_data *arg1 = (struct _dwg_entity_eed_data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    union eed_data_t *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Eed_Data_u_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_eed_data, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Eed_Data_u_get" "', argument " "1"" of type '" "struct _dwg_entity_eed_data *""'"); 
    }
    arg1 = (struct _dwg_entity_eed_data *)(argp1);
    result = (union eed_data_t *)& ((arg1)->u);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_eed_data_t, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Eed_Data_code_set) {
  {
    struct _dwg_entity_eed_data *arg1 = (struct _dwg_entity_eed_data *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Eed_Data_code_set(self,code);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_eed_data, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Eed_Data_code_set" "', argument " "1"" of type '" "struct _dwg_entity_eed_data *""'"); 
    }
    arg1 = (struct _dwg_entity_eed_data *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Eed_Data_code_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->code = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Eed_Data_code_get) {
  {
    struct _dwg_entity_eed_data *arg1 = (struct _dwg_entity_eed_data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Eed_Data_code_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_eed_data, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Eed_Data_code_get" "', argument " "1"" of type '" "struct _dwg_entity_eed_data *""'"); 
    }
    arg1 = (struct _dwg_entity_eed_data *)(argp1);
    result = (BITCODE_RC) ((arg1)->code);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Eed_Data) {
  {
    int argvi = 0;
    struct _dwg_entity_eed_data *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Eed_Data();");
    }
    result = (struct _dwg_entity_eed_data *)calloc(1, sizeof(struct _dwg_entity_eed_data));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_eed_data, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Eed_Data) {
  {
    struct _dwg_entity_eed_data *arg1 = (struct _dwg_entity_eed_data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Eed_Data(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_eed_data, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Eed_Data" "', argument " "1"" of type '" "struct _dwg_entity_eed_data *""'"); 
    }
    arg1 = (struct _dwg_entity_eed_data *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_0_get) {
  {
    union eed_data_t *arg1 = (union eed_data_t *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    eed_data_t_eed_0 *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: eed_data_t_eed_0_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_0_get" "', argument " "1"" of type '" "union eed_data_t *""'"); 
    }
    arg1 = (union eed_data_t *)(argp1);
    result = (eed_data_t_eed_0 *)& ((arg1)->eed_0);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_eed_data_t_eed_0, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_0_r2007_get) {
  {
    union eed_data_t *arg1 = (union eed_data_t *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    eed_data_t_eed_0_r2007 *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: eed_data_t_eed_0_r2007_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_0_r2007_get" "', argument " "1"" of type '" "union eed_data_t *""'"); 
    }
    arg1 = (union eed_data_t *)(argp1);
    result = (eed_data_t_eed_0_r2007 *)& ((arg1)->eed_0_r2007);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_eed_data_t_eed_0_r2007, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_1_get) {
  {
    union eed_data_t *arg1 = (union eed_data_t *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    eed_data_t_eed_1 *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: eed_data_t_eed_1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_1_get" "', argument " "1"" of type '" "union eed_data_t *""'"); 
    }
    arg1 = (union eed_data_t *)(argp1);
    result = (eed_data_t_eed_1 *)& ((arg1)->eed_1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_eed_data_t_eed_1, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_2_get) {
  {
    union eed_data_t *arg1 = (union eed_data_t *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    eed_data_t_eed_2 *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: eed_data_t_eed_2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_2_get" "', argument " "1"" of type '" "union eed_data_t *""'"); 
    }
    arg1 = (union eed_data_t *)(argp1);
    result = (eed_data_t_eed_2 *)& ((arg1)->eed_2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_eed_data_t_eed_2, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_3_get) {
  {
    union eed_data_t *arg1 = (union eed_data_t *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    eed_data_t_eed_3 *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: eed_data_t_eed_3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_3_get" "', argument " "1"" of type '" "union eed_data_t *""'"); 
    }
    arg1 = (union eed_data_t *)(argp1);
    result = (eed_data_t_eed_3 *)& ((arg1)->eed_3);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_eed_data_t_eed_3, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_4_get) {
  {
    union eed_data_t *arg1 = (union eed_data_t *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    eed_data_t_eed_4 *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: eed_data_t_eed_4_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_4_get" "', argument " "1"" of type '" "union eed_data_t *""'"); 
    }
    arg1 = (union eed_data_t *)(argp1);
    result = (eed_data_t_eed_4 *)& ((arg1)->eed_4);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_eed_data_t_eed_4, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_5_get) {
  {
    union eed_data_t *arg1 = (union eed_data_t *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    eed_data_t_eed_5 *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: eed_data_t_eed_5_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_5_get" "', argument " "1"" of type '" "union eed_data_t *""'"); 
    }
    arg1 = (union eed_data_t *)(argp1);
    result = (eed_data_t_eed_5 *)& ((arg1)->eed_5);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_eed_data_t_eed_5, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_10_get) {
  {
    union eed_data_t *arg1 = (union eed_data_t *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    eed_data_t_eed_10 *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: eed_data_t_eed_10_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_10_get" "', argument " "1"" of type '" "union eed_data_t *""'"); 
    }
    arg1 = (union eed_data_t *)(argp1);
    result = (eed_data_t_eed_10 *)& ((arg1)->eed_10);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_eed_data_t_eed_10, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_40_get) {
  {
    union eed_data_t *arg1 = (union eed_data_t *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    eed_data_t_eed_40 *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: eed_data_t_eed_40_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_40_get" "', argument " "1"" of type '" "union eed_data_t *""'"); 
    }
    arg1 = (union eed_data_t *)(argp1);
    result = (eed_data_t_eed_40 *)& ((arg1)->eed_40);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_eed_data_t_eed_40, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_70_get) {
  {
    union eed_data_t *arg1 = (union eed_data_t *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    eed_data_t_eed_70 *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: eed_data_t_eed_70_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_70_get" "', argument " "1"" of type '" "union eed_data_t *""'"); 
    }
    arg1 = (union eed_data_t *)(argp1);
    result = (eed_data_t_eed_70 *)& ((arg1)->eed_70);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_eed_data_t_eed_70, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_71_get) {
  {
    union eed_data_t *arg1 = (union eed_data_t *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    eed_data_t_eed_71 *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: eed_data_t_eed_71_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_71_get" "', argument " "1"" of type '" "union eed_data_t *""'"); 
    }
    arg1 = (union eed_data_t *)(argp1);
    result = (eed_data_t_eed_71 *)& ((arg1)->eed_71);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_eed_data_t_eed_71, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_eed_data_t) {
  {
    int argvi = 0;
    union eed_data_t *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_eed_data_t();");
    }
    result = (union eed_data_t *)calloc(1, sizeof(union eed_data_t));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_eed_data_t, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_eed_data_t) {
  {
    union eed_data_t *arg1 = (union eed_data_t *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_eed_data_t(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_eed_data_t" "', argument " "1"" of type '" "union eed_data_t *""'"); 
    }
    arg1 = (union eed_data_t *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_71_rl_set) {
  {
    eed_data_t_eed_71 *arg1 = (eed_data_t_eed_71 *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: eed_data_t_eed_71_rl_set(self,rl);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_71, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_71_rl_set" "', argument " "1"" of type '" "eed_data_t_eed_71 *""'"); 
    }
    arg1 = (eed_data_t_eed_71 *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "eed_data_t_eed_71_rl_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->rl = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_71_rl_get) {
  {
    eed_data_t_eed_71 *arg1 = (eed_data_t_eed_71 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: eed_data_t_eed_71_rl_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_71, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_71_rl_get" "', argument " "1"" of type '" "eed_data_t_eed_71 *""'"); 
    }
    arg1 = (eed_data_t_eed_71 *)(argp1);
    result = (BITCODE_RL) ((arg1)->rl);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_eed_data_t_eed_71) {
  {
    int argvi = 0;
    eed_data_t_eed_71 *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_eed_data_t_eed_71();");
    }
    result = (eed_data_t_eed_71 *)calloc(1, sizeof(eed_data_t_eed_71));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_eed_data_t_eed_71, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_eed_data_t_eed_71) {
  {
    eed_data_t_eed_71 *arg1 = (eed_data_t_eed_71 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_eed_data_t_eed_71(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_71, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_eed_data_t_eed_71" "', argument " "1"" of type '" "eed_data_t_eed_71 *""'"); 
    }
    arg1 = (eed_data_t_eed_71 *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_70_rs_set) {
  {
    eed_data_t_eed_70 *arg1 = (eed_data_t_eed_70 *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: eed_data_t_eed_70_rs_set(self,rs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_70, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_70_rs_set" "', argument " "1"" of type '" "eed_data_t_eed_70 *""'"); 
    }
    arg1 = (eed_data_t_eed_70 *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "eed_data_t_eed_70_rs_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->rs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_70_rs_get) {
  {
    eed_data_t_eed_70 *arg1 = (eed_data_t_eed_70 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: eed_data_t_eed_70_rs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_70, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_70_rs_get" "', argument " "1"" of type '" "eed_data_t_eed_70 *""'"); 
    }
    arg1 = (eed_data_t_eed_70 *)(argp1);
    result = (BITCODE_RS) ((arg1)->rs);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_eed_data_t_eed_70) {
  {
    int argvi = 0;
    eed_data_t_eed_70 *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_eed_data_t_eed_70();");
    }
    result = (eed_data_t_eed_70 *)calloc(1, sizeof(eed_data_t_eed_70));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_eed_data_t_eed_70, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_eed_data_t_eed_70) {
  {
    eed_data_t_eed_70 *arg1 = (eed_data_t_eed_70 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_eed_data_t_eed_70(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_70, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_eed_data_t_eed_70" "', argument " "1"" of type '" "eed_data_t_eed_70 *""'"); 
    }
    arg1 = (eed_data_t_eed_70 *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_40_real_set) {
  {
    eed_data_t_eed_40 *arg1 = (eed_data_t_eed_40 *) 0 ;
    BITCODE_RD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: eed_data_t_eed_40_real_set(self,real);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_40, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_40_real_set" "', argument " "1"" of type '" "eed_data_t_eed_40 *""'"); 
    }
    arg1 = (eed_data_t_eed_40 *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "eed_data_t_eed_40_real_set" "', argument " "2"" of type '" "BITCODE_RD""'");
    } 
    arg2 = (BITCODE_RD)(val2);
    if (arg1) (arg1)->real = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_40_real_get) {
  {
    eed_data_t_eed_40 *arg1 = (eed_data_t_eed_40 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: eed_data_t_eed_40_real_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_40, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_40_real_get" "', argument " "1"" of type '" "eed_data_t_eed_40 *""'"); 
    }
    arg1 = (eed_data_t_eed_40 *)(argp1);
    result = (BITCODE_RD) ((arg1)->real);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_eed_data_t_eed_40) {
  {
    int argvi = 0;
    eed_data_t_eed_40 *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_eed_data_t_eed_40();");
    }
    result = (eed_data_t_eed_40 *)calloc(1, sizeof(eed_data_t_eed_40));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_eed_data_t_eed_40, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_eed_data_t_eed_40) {
  {
    eed_data_t_eed_40 *arg1 = (eed_data_t_eed_40 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_eed_data_t_eed_40(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_40, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_eed_data_t_eed_40" "', argument " "1"" of type '" "eed_data_t_eed_40 *""'"); 
    }
    arg1 = (eed_data_t_eed_40 *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_10_point_set) {
  {
    eed_data_t_eed_10 *arg1 = (eed_data_t_eed_10 *) 0 ;
    BITCODE_3RD *arg2 = (BITCODE_3RD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: eed_data_t_eed_10_point_set(self,point);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_10, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_10_point_set" "', argument " "1"" of type '" "eed_data_t_eed_10 *""'"); 
    }
    arg1 = (eed_data_t_eed_10 *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_bitcode_3rd, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "eed_data_t_eed_10_point_set" "', argument " "2"" of type '" "BITCODE_3RD *""'"); 
    }
    arg2 = (BITCODE_3RD *)(argp2);
    if (arg1) (arg1)->point = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_10_point_get) {
  {
    eed_data_t_eed_10 *arg1 = (eed_data_t_eed_10 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_3RD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: eed_data_t_eed_10_point_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_10, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_10_point_get" "', argument " "1"" of type '" "eed_data_t_eed_10 *""'"); 
    }
    arg1 = (eed_data_t_eed_10 *)(argp1);
    result = (BITCODE_3RD *)& ((arg1)->point);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_bitcode_3rd, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_eed_data_t_eed_10) {
  {
    int argvi = 0;
    eed_data_t_eed_10 *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_eed_data_t_eed_10();");
    }
    result = (eed_data_t_eed_10 *)calloc(1, sizeof(eed_data_t_eed_10));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_eed_data_t_eed_10, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_eed_data_t_eed_10) {
  {
    eed_data_t_eed_10 *arg1 = (eed_data_t_eed_10 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_eed_data_t_eed_10(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_10, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_eed_data_t_eed_10" "', argument " "1"" of type '" "eed_data_t_eed_10 *""'"); 
    }
    arg1 = (eed_data_t_eed_10 *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_5_entity_set) {
  {
    eed_data_t_eed_5 *arg1 = (eed_data_t_eed_5 *) 0 ;
    BITCODE_RLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: eed_data_t_eed_5_entity_set(self,entity);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_5, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_5_entity_set" "', argument " "1"" of type '" "eed_data_t_eed_5 *""'"); 
    }
    arg1 = (eed_data_t_eed_5 *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "eed_data_t_eed_5_entity_set" "', argument " "2"" of type '" "BITCODE_RLL""'");
    } 
    arg2 = (BITCODE_RLL)(val2);
    if (arg1) (arg1)->entity = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_5_entity_get) {
  {
    eed_data_t_eed_5 *arg1 = (eed_data_t_eed_5 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: eed_data_t_eed_5_entity_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_5, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_5_entity_get" "', argument " "1"" of type '" "eed_data_t_eed_5 *""'"); 
    }
    arg1 = (eed_data_t_eed_5 *)(argp1);
    result = (BITCODE_RLL) ((arg1)->entity);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_eed_data_t_eed_5) {
  {
    int argvi = 0;
    eed_data_t_eed_5 *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_eed_data_t_eed_5();");
    }
    result = (eed_data_t_eed_5 *)calloc(1, sizeof(eed_data_t_eed_5));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_eed_data_t_eed_5, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_eed_data_t_eed_5) {
  {
    eed_data_t_eed_5 *arg1 = (eed_data_t_eed_5 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_eed_data_t_eed_5(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_5, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_eed_data_t_eed_5" "', argument " "1"" of type '" "eed_data_t_eed_5 *""'"); 
    }
    arg1 = (eed_data_t_eed_5 *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_4_length_set) {
  {
    eed_data_t_eed_4 *arg1 = (eed_data_t_eed_4 *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: eed_data_t_eed_4_length_set(self,length);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_4, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_4_length_set" "', argument " "1"" of type '" "eed_data_t_eed_4 *""'"); 
    }
    arg1 = (eed_data_t_eed_4 *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "eed_data_t_eed_4_length_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->length = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_4_length_get) {
  {
    eed_data_t_eed_4 *arg1 = (eed_data_t_eed_4 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: eed_data_t_eed_4_length_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_4, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_4_length_get" "', argument " "1"" of type '" "eed_data_t_eed_4 *""'"); 
    }
    arg1 = (eed_data_t_eed_4 *)(argp1);
    result = (BITCODE_RC) ((arg1)->length);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_4_data_get) {
  {
    eed_data_t_eed_4 *arg1 = (eed_data_t_eed_4 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    unsigned char *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: eed_data_t_eed_4_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_4, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_4_data_get" "', argument " "1"" of type '" "eed_data_t_eed_4 *""'"); 
    }
    arg1 = (eed_data_t_eed_4 *)(argp1);
    result = (unsigned char *)(unsigned char *) ((arg1)->data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_eed_data_t_eed_4) {
  {
    int argvi = 0;
    eed_data_t_eed_4 *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_eed_data_t_eed_4();");
    }
    result = (eed_data_t_eed_4 *)calloc(1, sizeof(eed_data_t_eed_4));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_eed_data_t_eed_4, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_eed_data_t_eed_4) {
  {
    eed_data_t_eed_4 *arg1 = (eed_data_t_eed_4 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_eed_data_t_eed_4(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_4, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_eed_data_t_eed_4" "', argument " "1"" of type '" "eed_data_t_eed_4 *""'"); 
    }
    arg1 = (eed_data_t_eed_4 *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_3_layer_set) {
  {
    eed_data_t_eed_3 *arg1 = (eed_data_t_eed_3 *) 0 ;
    BITCODE_RLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: eed_data_t_eed_3_layer_set(self,layer);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_3, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_3_layer_set" "', argument " "1"" of type '" "eed_data_t_eed_3 *""'"); 
    }
    arg1 = (eed_data_t_eed_3 *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "eed_data_t_eed_3_layer_set" "', argument " "2"" of type '" "BITCODE_RLL""'");
    } 
    arg2 = (BITCODE_RLL)(val2);
    if (arg1) (arg1)->layer = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_3_layer_get) {
  {
    eed_data_t_eed_3 *arg1 = (eed_data_t_eed_3 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: eed_data_t_eed_3_layer_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_3, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_3_layer_get" "', argument " "1"" of type '" "eed_data_t_eed_3 *""'"); 
    }
    arg1 = (eed_data_t_eed_3 *)(argp1);
    result = (BITCODE_RLL) ((arg1)->layer);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_eed_data_t_eed_3) {
  {
    int argvi = 0;
    eed_data_t_eed_3 *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_eed_data_t_eed_3();");
    }
    result = (eed_data_t_eed_3 *)calloc(1, sizeof(eed_data_t_eed_3));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_eed_data_t_eed_3, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_eed_data_t_eed_3) {
  {
    eed_data_t_eed_3 *arg1 = (eed_data_t_eed_3 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_eed_data_t_eed_3(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_3, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_eed_data_t_eed_3" "', argument " "1"" of type '" "eed_data_t_eed_3 *""'"); 
    }
    arg1 = (eed_data_t_eed_3 *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_2_close_set) {
  {
    eed_data_t_eed_2 *arg1 = (eed_data_t_eed_2 *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: eed_data_t_eed_2_close_set(self,close);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_2, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_2_close_set" "', argument " "1"" of type '" "eed_data_t_eed_2 *""'"); 
    }
    arg1 = (eed_data_t_eed_2 *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "eed_data_t_eed_2_close_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->close = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_2_close_get) {
  {
    eed_data_t_eed_2 *arg1 = (eed_data_t_eed_2 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: eed_data_t_eed_2_close_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_2, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_2_close_get" "', argument " "1"" of type '" "eed_data_t_eed_2 *""'"); 
    }
    arg1 = (eed_data_t_eed_2 *)(argp1);
    result = (BITCODE_RC) ((arg1)->close);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_eed_data_t_eed_2) {
  {
    int argvi = 0;
    eed_data_t_eed_2 *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_eed_data_t_eed_2();");
    }
    result = (eed_data_t_eed_2 *)calloc(1, sizeof(eed_data_t_eed_2));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_eed_data_t_eed_2, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_eed_data_t_eed_2) {
  {
    eed_data_t_eed_2 *arg1 = (eed_data_t_eed_2 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_eed_data_t_eed_2(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_2, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_eed_data_t_eed_2" "', argument " "1"" of type '" "eed_data_t_eed_2 *""'"); 
    }
    arg1 = (eed_data_t_eed_2 *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_1_appid_index_set) {
  {
    eed_data_t_eed_1 *arg1 = (eed_data_t_eed_1 *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: eed_data_t_eed_1_appid_index_set(self,appid_index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_1, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_1_appid_index_set" "', argument " "1"" of type '" "eed_data_t_eed_1 *""'"); 
    }
    arg1 = (eed_data_t_eed_1 *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "eed_data_t_eed_1_appid_index_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->appid_index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_1_appid_index_get) {
  {
    eed_data_t_eed_1 *arg1 = (eed_data_t_eed_1 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: eed_data_t_eed_1_appid_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_1, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_1_appid_index_get" "', argument " "1"" of type '" "eed_data_t_eed_1 *""'"); 
    }
    arg1 = (eed_data_t_eed_1 *)(argp1);
    result = (BITCODE_RS) ((arg1)->appid_index);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_1_invalid_set) {
  {
    eed_data_t_eed_1 *arg1 = (eed_data_t_eed_1 *) 0 ;
    char *arg2 = (char *) (char *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    char temp2[0] ;
    int res2 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: eed_data_t_eed_1_invalid_set(self,invalid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_1, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_1_invalid_set" "', argument " "1"" of type '" "eed_data_t_eed_1 *""'"); 
    }
    arg1 = (eed_data_t_eed_1 *)(argp1);
    res2 = SWIG_AsCharArray(ST(1), temp2, 0);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "eed_data_t_eed_1_invalid_set" "', argument " "2"" of type '" "char [0]""'");
    }
    arg2 = (char *)(temp2);
    if (arg2) memcpy(arg1->invalid,arg2,0*sizeof(char));
    else memset(arg1->invalid,0,0*sizeof(char));
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_1_invalid_get) {
  {
    eed_data_t_eed_1 *arg1 = (eed_data_t_eed_1 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    char *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: eed_data_t_eed_1_invalid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_1, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_1_invalid_get" "', argument " "1"" of type '" "eed_data_t_eed_1 *""'"); 
    }
    arg1 = (eed_data_t_eed_1 *)(argp1);
    result = (char *)(char *) ((arg1)->invalid);
    {
      size_t size = SWIG_strnlen(result, 0);
      
      
      
      ST(argvi) = SWIG_FromCharPtrAndSize(result, size); argvi++ ;
    }
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_eed_data_t_eed_1) {
  {
    int argvi = 0;
    eed_data_t_eed_1 *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_eed_data_t_eed_1();");
    }
    result = (eed_data_t_eed_1 *)calloc(1, sizeof(eed_data_t_eed_1));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_eed_data_t_eed_1, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_eed_data_t_eed_1) {
  {
    eed_data_t_eed_1 *arg1 = (eed_data_t_eed_1 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_eed_data_t_eed_1(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_1, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_eed_data_t_eed_1" "', argument " "1"" of type '" "eed_data_t_eed_1 *""'"); 
    }
    arg1 = (eed_data_t_eed_1 *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_0_r2007_length_set) {
  {
    eed_data_t_eed_0_r2007 *arg1 = (eed_data_t_eed_0_r2007 *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: eed_data_t_eed_0_r2007_length_set(self,length);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_0_r2007, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_0_r2007_length_set" "', argument " "1"" of type '" "eed_data_t_eed_0_r2007 *""'"); 
    }
    arg1 = (eed_data_t_eed_0_r2007 *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "eed_data_t_eed_0_r2007_length_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->length = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_0_r2007_length_get) {
  {
    eed_data_t_eed_0_r2007 *arg1 = (eed_data_t_eed_0_r2007 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: eed_data_t_eed_0_r2007_length_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_0_r2007, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_0_r2007_length_get" "', argument " "1"" of type '" "eed_data_t_eed_0_r2007 *""'"); 
    }
    arg1 = (eed_data_t_eed_0_r2007 *)(argp1);
    result = (BITCODE_RS) ((arg1)->length);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_0_r2007__padding_set) {
  {
    eed_data_t_eed_0_r2007 *arg1 = (eed_data_t_eed_0_r2007 *) 0 ;
    unsigned short arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: eed_data_t_eed_0_r2007__padding_set(self,_padding);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_0_r2007, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_0_r2007__padding_set" "', argument " "1"" of type '" "eed_data_t_eed_0_r2007 *""'"); 
    }
    arg1 = (eed_data_t_eed_0_r2007 *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "eed_data_t_eed_0_r2007__padding_set" "', argument " "2"" of type '" "unsigned short""'");
    } 
    arg2 = (unsigned short)(val2);
    if (arg1) (arg1)->_padding = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_0_r2007__padding_get) {
  {
    eed_data_t_eed_0_r2007 *arg1 = (eed_data_t_eed_0_r2007 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    unsigned short result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: eed_data_t_eed_0_r2007__padding_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_0_r2007, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_0_r2007__padding_get" "', argument " "1"" of type '" "eed_data_t_eed_0_r2007 *""'"); 
    }
    arg1 = (eed_data_t_eed_0_r2007 *)(argp1);
    result = (unsigned short) ((arg1)->_padding);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_0_r2007_is_tu_set) {
  {
    eed_data_t_eed_0_r2007 *arg1 = (eed_data_t_eed_0_r2007 *) 0 ;
    unsigned short arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: eed_data_t_eed_0_r2007_is_tu_set(self,is_tu);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_0_r2007, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_0_r2007_is_tu_set" "', argument " "1"" of type '" "eed_data_t_eed_0_r2007 *""'"); 
    }
    arg1 = (eed_data_t_eed_0_r2007 *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "eed_data_t_eed_0_r2007_is_tu_set" "', argument " "2"" of type '" "unsigned short""'");
    } 
    arg2 = (unsigned short)(val2);
    if (arg1) (arg1)->is_tu = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_0_r2007_is_tu_get) {
  {
    eed_data_t_eed_0_r2007 *arg1 = (eed_data_t_eed_0_r2007 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    unsigned short result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: eed_data_t_eed_0_r2007_is_tu_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_0_r2007, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_0_r2007_is_tu_get" "', argument " "1"" of type '" "eed_data_t_eed_0_r2007 *""'"); 
    }
    arg1 = (eed_data_t_eed_0_r2007 *)(argp1);
    result = (unsigned short) ((arg1)->is_tu);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_0_r2007_string_set) {
  {
    eed_data_t_eed_0_r2007 *arg1 = (eed_data_t_eed_0_r2007 *) 0 ;
    dwg_wchar_t *arg2 = (dwg_wchar_t *) (dwg_wchar_t *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: eed_data_t_eed_0_r2007_string_set(self,string);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_0_r2007, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_0_r2007_string_set" "', argument " "1"" of type '" "eed_data_t_eed_0_r2007 *""'"); 
    }
    arg1 = (eed_data_t_eed_0_r2007 *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_short, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "eed_data_t_eed_0_r2007_string_set" "', argument " "2"" of type '" "dwg_wchar_t [0]""'"); 
    } 
    arg2 = (dwg_wchar_t *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)0; ++ii) *(dwg_wchar_t *)&arg1->string[ii] = *((dwg_wchar_t *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""string""' of type '""dwg_wchar_t [0]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_0_r2007_string_get) {
  {
    eed_data_t_eed_0_r2007 *arg1 = (eed_data_t_eed_0_r2007 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dwg_wchar_t *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: eed_data_t_eed_0_r2007_string_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_0_r2007, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_0_r2007_string_get" "', argument " "1"" of type '" "eed_data_t_eed_0_r2007 *""'"); 
    }
    arg1 = (eed_data_t_eed_0_r2007 *)(argp1);
    result = (dwg_wchar_t *)(dwg_wchar_t *) ((arg1)->string);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_short, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_eed_data_t_eed_0_r2007) {
  {
    int argvi = 0;
    eed_data_t_eed_0_r2007 *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_eed_data_t_eed_0_r2007();");
    }
    result = (eed_data_t_eed_0_r2007 *)calloc(1, sizeof(eed_data_t_eed_0_r2007));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_eed_data_t_eed_0_r2007, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_eed_data_t_eed_0_r2007) {
  {
    eed_data_t_eed_0_r2007 *arg1 = (eed_data_t_eed_0_r2007 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_eed_data_t_eed_0_r2007(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_0_r2007, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_eed_data_t_eed_0_r2007" "', argument " "1"" of type '" "eed_data_t_eed_0_r2007 *""'"); 
    }
    arg1 = (eed_data_t_eed_0_r2007 *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_0_length_set) {
  {
    eed_data_t_eed_0 *arg1 = (eed_data_t_eed_0 *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: eed_data_t_eed_0_length_set(self,length);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_0, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_0_length_set" "', argument " "1"" of type '" "eed_data_t_eed_0 *""'"); 
    }
    arg1 = (eed_data_t_eed_0 *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "eed_data_t_eed_0_length_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->length = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_0_length_get) {
  {
    eed_data_t_eed_0 *arg1 = (eed_data_t_eed_0 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: eed_data_t_eed_0_length_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_0, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_0_length_get" "', argument " "1"" of type '" "eed_data_t_eed_0 *""'"); 
    }
    arg1 = (eed_data_t_eed_0 *)(argp1);
    result = (BITCODE_RS) ((arg1)->length);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_0_codepage_set) {
  {
    eed_data_t_eed_0 *arg1 = (eed_data_t_eed_0 *) 0 ;
    unsigned short arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: eed_data_t_eed_0_codepage_set(self,codepage);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_0, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_0_codepage_set" "', argument " "1"" of type '" "eed_data_t_eed_0 *""'"); 
    }
    arg1 = (eed_data_t_eed_0 *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "eed_data_t_eed_0_codepage_set" "', argument " "2"" of type '" "unsigned short""'");
    } 
    arg2 = (unsigned short)(val2);
    if (arg1) (arg1)->codepage = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_0_codepage_get) {
  {
    eed_data_t_eed_0 *arg1 = (eed_data_t_eed_0 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    unsigned short result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: eed_data_t_eed_0_codepage_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_0, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_0_codepage_get" "', argument " "1"" of type '" "eed_data_t_eed_0 *""'"); 
    }
    arg1 = (eed_data_t_eed_0 *)(argp1);
    result = (unsigned short) ((arg1)->codepage);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_0_is_tu_set) {
  {
    eed_data_t_eed_0 *arg1 = (eed_data_t_eed_0 *) 0 ;
    unsigned short arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: eed_data_t_eed_0_is_tu_set(self,is_tu);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_0, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_0_is_tu_set" "', argument " "1"" of type '" "eed_data_t_eed_0 *""'"); 
    }
    arg1 = (eed_data_t_eed_0 *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "eed_data_t_eed_0_is_tu_set" "', argument " "2"" of type '" "unsigned short""'");
    } 
    arg2 = (unsigned short)(val2);
    if (arg1) (arg1)->is_tu = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_0_is_tu_get) {
  {
    eed_data_t_eed_0 *arg1 = (eed_data_t_eed_0 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    unsigned short result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: eed_data_t_eed_0_is_tu_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_0, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_0_is_tu_get" "', argument " "1"" of type '" "eed_data_t_eed_0 *""'"); 
    }
    arg1 = (eed_data_t_eed_0 *)(argp1);
    result = (unsigned short) ((arg1)->is_tu);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_0_string_set) {
  {
    eed_data_t_eed_0 *arg1 = (eed_data_t_eed_0 *) 0 ;
    char *arg2 = (char *) (char *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    char temp2[0] ;
    int res2 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: eed_data_t_eed_0_string_set(self,string);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_0, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_0_string_set" "', argument " "1"" of type '" "eed_data_t_eed_0 *""'"); 
    }
    arg1 = (eed_data_t_eed_0 *)(argp1);
    res2 = SWIG_AsCharArray(ST(1), temp2, 0);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "eed_data_t_eed_0_string_set" "', argument " "2"" of type '" "char [0]""'");
    }
    arg2 = (char *)(temp2);
    if (arg2) memcpy(arg1->string,arg2,0*sizeof(char));
    else memset(arg1->string,0,0*sizeof(char));
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_eed_data_t_eed_0_string_get) {
  {
    eed_data_t_eed_0 *arg1 = (eed_data_t_eed_0 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    char *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: eed_data_t_eed_0_string_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_0, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "eed_data_t_eed_0_string_get" "', argument " "1"" of type '" "eed_data_t_eed_0 *""'"); 
    }
    arg1 = (eed_data_t_eed_0 *)(argp1);
    result = (char *)(char *) ((arg1)->string);
    {
      size_t size = SWIG_strnlen(result, 0);
      
      
      
      ST(argvi) = SWIG_FromCharPtrAndSize(result, size); argvi++ ;
    }
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_eed_data_t_eed_0) {
  {
    int argvi = 0;
    eed_data_t_eed_0 *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_eed_data_t_eed_0();");
    }
    result = (eed_data_t_eed_0 *)calloc(1, sizeof(eed_data_t_eed_0));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_eed_data_t_eed_0, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_eed_data_t_eed_0) {
  {
    eed_data_t_eed_0 *arg1 = (eed_data_t_eed_0 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_eed_data_t_eed_0(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_eed_data_t_eed_0, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_eed_data_t_eed_0" "', argument " "1"" of type '" "eed_data_t_eed_0 *""'"); 
    }
    arg1 = (eed_data_t_eed_0 *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Eed_size_set) {
  {
    struct _dwg_entity_eed *arg1 = (struct _dwg_entity_eed *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Eed_size_set(self,size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_eed, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Eed_size_set" "', argument " "1"" of type '" "struct _dwg_entity_eed *""'"); 
    }
    arg1 = (struct _dwg_entity_eed *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Eed_size_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Eed_size_get) {
  {
    struct _dwg_entity_eed *arg1 = (struct _dwg_entity_eed *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Eed_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_eed, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Eed_size_get" "', argument " "1"" of type '" "struct _dwg_entity_eed *""'"); 
    }
    arg1 = (struct _dwg_entity_eed *)(argp1);
    result = (BITCODE_BS) ((arg1)->size);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Eed_handle_set) {
  {
    struct _dwg_entity_eed *arg1 = (struct _dwg_entity_eed *) 0 ;
    Dwg_Handle *arg2 = (Dwg_Handle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Eed_handle_set(self,handle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_eed, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Eed_handle_set" "', argument " "1"" of type '" "struct _dwg_entity_eed *""'"); 
    }
    arg1 = (struct _dwg_entity_eed *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_handle, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Eed_handle_set" "', argument " "2"" of type '" "Dwg_Handle *""'"); 
    }
    arg2 = (Dwg_Handle *)(argp2);
    if (arg1) (arg1)->handle = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Eed_handle_get) {
  {
    struct _dwg_entity_eed *arg1 = (struct _dwg_entity_eed *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Handle *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Eed_handle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_eed, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Eed_handle_get" "', argument " "1"" of type '" "struct _dwg_entity_eed *""'"); 
    }
    arg1 = (struct _dwg_entity_eed *)(argp1);
    result = (Dwg_Handle *)& ((arg1)->handle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_handle, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Eed_data_set) {
  {
    struct _dwg_entity_eed *arg1 = (struct _dwg_entity_eed *) 0 ;
    Dwg_Eed_Data *arg2 = (Dwg_Eed_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Eed_data_set(self,data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_eed, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Eed_data_set" "', argument " "1"" of type '" "struct _dwg_entity_eed *""'"); 
    }
    arg1 = (struct _dwg_entity_eed *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_eed_data, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Eed_data_set" "', argument " "2"" of type '" "Dwg_Eed_Data *""'"); 
    }
    arg2 = (Dwg_Eed_Data *)(argp2);
    if (arg1) (arg1)->data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Eed_data_get) {
  {
    struct _dwg_entity_eed *arg1 = (struct _dwg_entity_eed *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Eed_Data *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Eed_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_eed, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Eed_data_get" "', argument " "1"" of type '" "struct _dwg_entity_eed *""'"); 
    }
    arg1 = (struct _dwg_entity_eed *)(argp1);
    result = (Dwg_Eed_Data *) ((arg1)->data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_eed_data, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Eed_raw_set) {
  {
    struct _dwg_entity_eed *arg1 = (struct _dwg_entity_eed *) 0 ;
    BITCODE_TF arg2 = (BITCODE_TF) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Eed_raw_set(self,raw);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_eed, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Eed_raw_set" "', argument " "1"" of type '" "struct _dwg_entity_eed *""'"); 
    }
    arg1 = (struct _dwg_entity_eed *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Eed_raw_set" "', argument " "2"" of type '" "BITCODE_TF""'"); 
    }
    arg2 = (BITCODE_TF)(argp2);
    if (arg1) (arg1)->raw = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Eed_raw_get) {
  {
    struct _dwg_entity_eed *arg1 = (struct _dwg_entity_eed *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TF result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Eed_raw_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_eed, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Eed_raw_get" "', argument " "1"" of type '" "struct _dwg_entity_eed *""'"); 
    }
    arg1 = (struct _dwg_entity_eed *)(argp1);
    result = (BITCODE_TF) ((arg1)->raw);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Eed) {
  {
    int argvi = 0;
    struct _dwg_entity_eed *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Eed();");
    }
    result = (struct _dwg_entity_eed *)calloc(1, sizeof(struct _dwg_entity_eed));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_eed, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Eed) {
  {
    struct _dwg_entity_eed *arg1 = (struct _dwg_entity_eed *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Eed(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_eed, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Eed" "', argument " "1"" of type '" "struct _dwg_entity_eed *""'"); 
    }
    arg1 = (struct _dwg_entity_eed *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_objid_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_objid_set(self,objid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_objid_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Entity_objid_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->objid = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_objid_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_objid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_objid_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_BL) ((arg1)->objid);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_tio_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    _dwg_object_entity_tio *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_tio_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_tio_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (_dwg_object_entity_tio *)& ((arg1)->tio);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity_tio, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_dwg_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    struct _dwg_struct *arg2 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_dwg_set(self,dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_dwg_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_struct, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_Entity_dwg_set" "', argument " "2"" of type '" "struct _dwg_struct *""'"); 
    }
    arg2 = (struct _dwg_struct *)(argp2);
    if (arg1) (arg1)->dwg = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_dwg_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_struct *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_dwg_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_dwg_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (struct _dwg_struct *) ((arg1)->dwg);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_struct, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_num_eed_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_num_eed_set(self,num_eed);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_num_eed_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Entity_num_eed_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_eed = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_num_eed_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_num_eed_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_num_eed_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_eed);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_eed_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    Dwg_Eed *arg2 = (Dwg_Eed *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_eed_set(self,eed);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_eed_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_eed, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_Entity_eed_set" "', argument " "2"" of type '" "Dwg_Eed *""'"); 
    }
    arg2 = (Dwg_Eed *)(argp2);
    if (arg1) (arg1)->eed = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_eed_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Eed *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_eed_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_eed_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (Dwg_Eed *) ((arg1)->eed);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_eed, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_preview_exists_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_preview_exists_set(self,preview_exists);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_preview_exists_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Entity_preview_exists_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->preview_exists = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_preview_exists_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_preview_exists_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_preview_exists_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_B) ((arg1)->preview_exists);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_preview_is_proxy_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_preview_is_proxy_set(self,preview_is_proxy);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_preview_is_proxy_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Entity_preview_is_proxy_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->preview_is_proxy = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_preview_is_proxy_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_preview_is_proxy_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_preview_is_proxy_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_B) ((arg1)->preview_is_proxy);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_preview_size_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_BLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_preview_size_set(self,preview_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_preview_size_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Entity_preview_size_set" "', argument " "2"" of type '" "BITCODE_BLL""'");
    } 
    arg2 = (BITCODE_BLL)(val2);
    if (arg1) (arg1)->preview_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_preview_size_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_preview_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_preview_size_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_BLL) ((arg1)->preview_size);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_preview_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_TF arg2 = (BITCODE_TF) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_preview_set(self,preview);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_preview_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_Entity_preview_set" "', argument " "2"" of type '" "BITCODE_TF""'"); 
    }
    arg2 = (BITCODE_TF)(argp2);
    if (arg1) (arg1)->preview = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_preview_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TF result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_preview_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_preview_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_TF) ((arg1)->preview);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_entmode_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_BB arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_entmode_set(self,entmode);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_entmode_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Entity_entmode_set" "', argument " "2"" of type '" "BITCODE_BB""'");
    } 
    arg2 = (BITCODE_BB)(val2);
    if (arg1) (arg1)->entmode = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_entmode_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BB result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_entmode_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_entmode_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_BB) ((arg1)->entmode);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_num_reactors_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_num_reactors_set(self,num_reactors);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_num_reactors_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Entity_num_reactors_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_reactors = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_num_reactors_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_num_reactors_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_num_reactors_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_reactors);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_is_xdic_missing_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_is_xdic_missing_set(self,is_xdic_missing);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_is_xdic_missing_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Entity_is_xdic_missing_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_xdic_missing = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_is_xdic_missing_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_is_xdic_missing_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_is_xdic_missing_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_B) ((arg1)->is_xdic_missing);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_isbylayerlt_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_isbylayerlt_set(self,isbylayerlt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_isbylayerlt_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Entity_isbylayerlt_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->isbylayerlt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_isbylayerlt_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_isbylayerlt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_isbylayerlt_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_B) ((arg1)->isbylayerlt);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_nolinks_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_nolinks_set(self,nolinks);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_nolinks_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Entity_nolinks_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->nolinks = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_nolinks_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_nolinks_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_nolinks_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_B) ((arg1)->nolinks);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_has_ds_data_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_has_ds_data_set(self,has_ds_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_has_ds_data_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Entity_has_ds_data_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_ds_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_has_ds_data_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_has_ds_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_has_ds_data_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_B) ((arg1)->has_ds_data);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_color_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_CMC *arg2 = (BITCODE_CMC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_color_set(self,color);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_color_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_color, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_Entity_color_set" "', argument " "2"" of type '" "BITCODE_CMC *""'"); 
    }
    arg2 = (BITCODE_CMC *)(argp2);
    if (arg1) (arg1)->color = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_color_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_CMC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_color_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_color_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_CMC *)& ((arg1)->color);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_color, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_ltype_scale_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_BD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_ltype_scale_set(self,ltype_scale);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_ltype_scale_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Entity_ltype_scale_set" "', argument " "2"" of type '" "BITCODE_BD""'");
    } 
    arg2 = (BITCODE_BD)(val2);
    if (arg1) (arg1)->ltype_scale = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_ltype_scale_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_ltype_scale_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_ltype_scale_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_BD) ((arg1)->ltype_scale);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_ltype_flags_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_BB arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_ltype_flags_set(self,ltype_flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_ltype_flags_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Entity_ltype_flags_set" "', argument " "2"" of type '" "BITCODE_BB""'");
    } 
    arg2 = (BITCODE_BB)(val2);
    if (arg1) (arg1)->ltype_flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_ltype_flags_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BB result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_ltype_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_ltype_flags_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_BB) ((arg1)->ltype_flags);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_plotstyle_flags_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_BB arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_plotstyle_flags_set(self,plotstyle_flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_plotstyle_flags_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Entity_plotstyle_flags_set" "', argument " "2"" of type '" "BITCODE_BB""'");
    } 
    arg2 = (BITCODE_BB)(val2);
    if (arg1) (arg1)->plotstyle_flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_plotstyle_flags_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BB result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_plotstyle_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_plotstyle_flags_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_BB) ((arg1)->plotstyle_flags);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_material_flags_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_BB arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_material_flags_set(self,material_flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_material_flags_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Entity_material_flags_set" "', argument " "2"" of type '" "BITCODE_BB""'");
    } 
    arg2 = (BITCODE_BB)(val2);
    if (arg1) (arg1)->material_flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_material_flags_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BB result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_material_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_material_flags_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_BB) ((arg1)->material_flags);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_shadow_flags_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_shadow_flags_set(self,shadow_flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_shadow_flags_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Entity_shadow_flags_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->shadow_flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_shadow_flags_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_shadow_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_shadow_flags_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_RC) ((arg1)->shadow_flags);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_has_full_visualstyle_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_has_full_visualstyle_set(self,has_full_visualstyle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_has_full_visualstyle_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Entity_has_full_visualstyle_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_full_visualstyle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_has_full_visualstyle_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_has_full_visualstyle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_has_full_visualstyle_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_B) ((arg1)->has_full_visualstyle);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_has_face_visualstyle_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_has_face_visualstyle_set(self,has_face_visualstyle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_has_face_visualstyle_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Entity_has_face_visualstyle_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_face_visualstyle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_has_face_visualstyle_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_has_face_visualstyle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_has_face_visualstyle_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_B) ((arg1)->has_face_visualstyle);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_has_edge_visualstyle_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_has_edge_visualstyle_set(self,has_edge_visualstyle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_has_edge_visualstyle_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Entity_has_edge_visualstyle_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_edge_visualstyle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_has_edge_visualstyle_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_has_edge_visualstyle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_has_edge_visualstyle_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_B) ((arg1)->has_edge_visualstyle);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_invisible_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_invisible_set(self,invisible);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_invisible_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Entity_invisible_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->invisible = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_invisible_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_invisible_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_invisible_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_BS) ((arg1)->invisible);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_linewt_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_linewt_set(self,linewt);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_linewt_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Entity_linewt_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->linewt = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_linewt_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_linewt_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_linewt_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_RC) ((arg1)->linewt);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_flag_r11_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_flag_r11_set(self,flag_r11);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_flag_r11_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Entity_flag_r11_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->flag_r11 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_flag_r11_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_flag_r11_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_flag_r11_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_RC) ((arg1)->flag_r11);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_opts_r11_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_opts_r11_set(self,opts_r11);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_opts_r11_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Entity_opts_r11_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->opts_r11 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_opts_r11_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_opts_r11_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_opts_r11_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_RS) ((arg1)->opts_r11);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_extra_r11_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_extra_r11_set(self,extra_r11);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_extra_r11_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Entity_extra_r11_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->extra_r11 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_extra_r11_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_extra_r11_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_extra_r11_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_RS) ((arg1)->extra_r11);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_color_r11_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_RCd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    signed char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_color_r11_set(self,color_r11);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_color_r11_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    ecode2 = SWIG_AsVal_signed_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Entity_color_r11_set" "', argument " "2"" of type '" "BITCODE_RCd""'");
    } 
    arg2 = (BITCODE_RCd)(val2);
    if (arg1) (arg1)->color_r11 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_color_r11_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RCd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_color_r11_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_color_r11_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_RCd) ((arg1)->color_r11);
    ST(argvi) = SWIG_From_signed_SS_char  SWIG_PERL_CALL_ARGS_1((signed char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_elevation_r11_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_RD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_elevation_r11_set(self,elevation_r11);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_elevation_r11_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Entity_elevation_r11_set" "', argument " "2"" of type '" "BITCODE_RD""'");
    } 
    arg2 = (BITCODE_RD)(val2);
    if (arg1) (arg1)->elevation_r11 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_elevation_r11_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_elevation_r11_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_elevation_r11_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_RD) ((arg1)->elevation_r11);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_thickness_r11_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_RD arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    double val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_thickness_r11_set(self,thickness_r11);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_thickness_r11_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    ecode2 = SWIG_AsVal_double SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Entity_thickness_r11_set" "', argument " "2"" of type '" "BITCODE_RD""'");
    } 
    arg2 = (BITCODE_RD)(val2);
    if (arg1) (arg1)->thickness_r11 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_thickness_r11_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RD result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_thickness_r11_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_thickness_r11_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_RD) ((arg1)->thickness_r11);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_viewport_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_viewport_set(self,viewport);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_viewport_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_Entity_viewport_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->viewport = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_viewport_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_viewport_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_viewport_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_H) ((arg1)->viewport);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity___iterator_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity___iterator_set(self,__iterator);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity___iterator_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Entity___iterator_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->__iterator = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity___iterator_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity___iterator_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity___iterator_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_BL) ((arg1)->__iterator);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_ownerhandle_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_ownerhandle_set(self,ownerhandle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_ownerhandle_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_Entity_ownerhandle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->ownerhandle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_ownerhandle_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_ownerhandle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_ownerhandle_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_H) ((arg1)->ownerhandle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_reactors_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_reactors_set(self,reactors);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_reactors_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_Entity_reactors_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->reactors = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_reactors_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_reactors_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_reactors_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_H *) ((arg1)->reactors);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_xdicobjhandle_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_xdicobjhandle_set(self,xdicobjhandle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_xdicobjhandle_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_Entity_xdicobjhandle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->xdicobjhandle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_xdicobjhandle_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_xdicobjhandle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_xdicobjhandle_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_H) ((arg1)->xdicobjhandle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_prev_entity_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_prev_entity_set(self,prev_entity);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_prev_entity_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_Entity_prev_entity_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->prev_entity = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_prev_entity_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_prev_entity_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_prev_entity_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_H) ((arg1)->prev_entity);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_next_entity_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_next_entity_set(self,next_entity);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_next_entity_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_Entity_next_entity_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->next_entity = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_next_entity_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_next_entity_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_next_entity_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_H) ((arg1)->next_entity);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_layer_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_layer_set(self,layer);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_layer_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_Entity_layer_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->layer = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_layer_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_layer_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_layer_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_H) ((arg1)->layer);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_ltype_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_ltype_set(self,ltype);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_ltype_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_Entity_ltype_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->ltype = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_ltype_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_ltype_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_ltype_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_H) ((arg1)->ltype);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_material_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_material_set(self,material);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_material_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_Entity_material_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->material = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_material_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_material_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_material_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_H) ((arg1)->material);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_shadow_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_shadow_set(self,shadow);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_shadow_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_Entity_shadow_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->shadow = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_shadow_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_shadow_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_shadow_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_H) ((arg1)->shadow);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_plotstyle_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_plotstyle_set(self,plotstyle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_plotstyle_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_Entity_plotstyle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->plotstyle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_plotstyle_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_plotstyle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_plotstyle_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_H) ((arg1)->plotstyle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_full_visualstyle_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_full_visualstyle_set(self,full_visualstyle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_full_visualstyle_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_Entity_full_visualstyle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->full_visualstyle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_full_visualstyle_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_full_visualstyle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_full_visualstyle_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_H) ((arg1)->full_visualstyle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_face_visualstyle_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_face_visualstyle_set(self,face_visualstyle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_face_visualstyle_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_Entity_face_visualstyle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->face_visualstyle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_face_visualstyle_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_face_visualstyle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_face_visualstyle_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_H) ((arg1)->face_visualstyle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_edge_visualstyle_set) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Entity_edge_visualstyle_set(self,edge_visualstyle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_edge_visualstyle_set" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_Entity_edge_visualstyle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->edge_visualstyle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Entity_edge_visualstyle_get) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Entity_edge_visualstyle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Entity_edge_visualstyle_get" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    result = (BITCODE_H) ((arg1)->edge_visualstyle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_Entity) {
  {
    int argvi = 0;
    struct _dwg_object_entity *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_Entity();");
    }
    result = (struct _dwg_object_entity *)calloc(1, sizeof(struct _dwg_object_entity));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_Entity) {
  {
    struct _dwg_object_entity *arg1 = (struct _dwg_object_entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_Entity(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_Entity" "', argument " "1"" of type '" "struct _dwg_object_entity *""'"); 
    }
    arg1 = (struct _dwg_object_entity *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_UNUSED_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_UNUSED *arg2 = (Dwg_Entity_UNUSED *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_UNUSED_set(self,UNUSED);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_UNUSED_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_UNUSED_set" "', argument " "2"" of type '" "Dwg_Entity_UNUSED *""'"); 
    }
    arg2 = (Dwg_Entity_UNUSED *)(argp2);
    if (arg1) (arg1)->UNUSED = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_UNUSED_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_UNUSED *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_UNUSED_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_UNUSED_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_UNUSED *) ((arg1)->UNUSED);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_DIMENSION_common_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_DIMENSION_common *arg2 = (Dwg_DIMENSION_common *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_DIMENSION_common_set(self,DIMENSION_common);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_DIMENSION_common_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_DIMENSION_common, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_DIMENSION_common_set" "', argument " "2"" of type '" "Dwg_DIMENSION_common *""'"); 
    }
    arg2 = (Dwg_DIMENSION_common *)(argp2);
    if (arg1) (arg1)->DIMENSION_common = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_DIMENSION_common_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_DIMENSION_common *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_DIMENSION_common_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_DIMENSION_common_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_DIMENSION_common *) ((arg1)->DIMENSION_common);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_DIMENSION_common, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio__3DFACE_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity__3DFACE *arg2 = (Dwg_Entity__3DFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio__3DFACE_set(self,_3DFACE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio__3DFACE_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_3DFACE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio__3DFACE_set" "', argument " "2"" of type '" "Dwg_Entity__3DFACE *""'"); 
    }
    arg2 = (Dwg_Entity__3DFACE *)(argp2);
    if (arg1) (arg1)->_3DFACE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio__3DFACE_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity__3DFACE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio__3DFACE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio__3DFACE_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity__3DFACE *) ((arg1)->_3DFACE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_3DFACE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio__3DSOLID_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity__3DSOLID *arg2 = (Dwg_Entity__3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio__3DSOLID_set(self,_3DSOLID);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio__3DSOLID_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_3DSOLID, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio__3DSOLID_set" "', argument " "2"" of type '" "Dwg_Entity__3DSOLID *""'"); 
    }
    arg2 = (Dwg_Entity__3DSOLID *)(argp2);
    if (arg1) (arg1)->_3DSOLID = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio__3DSOLID_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity__3DSOLID *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio__3DSOLID_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio__3DSOLID_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity__3DSOLID *) ((arg1)->_3DSOLID);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_3DSOLID, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_ARC_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_ARC *arg2 = (Dwg_Entity_ARC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_ARC_set(self,ARC);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_ARC_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_ARC, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_ARC_set" "', argument " "2"" of type '" "Dwg_Entity_ARC *""'"); 
    }
    arg2 = (Dwg_Entity_ARC *)(argp2);
    if (arg1) (arg1)->ARC = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_ARC_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_ARC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_ARC_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_ARC_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_ARC *) ((arg1)->ARC);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_ARC, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_ATTDEF_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_ATTDEF *arg2 = (Dwg_Entity_ATTDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_ATTDEF_set(self,ATTDEF);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_ATTDEF_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_ATTDEF, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_ATTDEF_set" "', argument " "2"" of type '" "Dwg_Entity_ATTDEF *""'"); 
    }
    arg2 = (Dwg_Entity_ATTDEF *)(argp2);
    if (arg1) (arg1)->ATTDEF = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_ATTDEF_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_ATTDEF *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_ATTDEF_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_ATTDEF_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_ATTDEF *) ((arg1)->ATTDEF);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_ATTDEF, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_ATTRIB_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_ATTRIB *arg2 = (Dwg_Entity_ATTRIB *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_ATTRIB_set(self,ATTRIB);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_ATTRIB_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_ATTRIB, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_ATTRIB_set" "', argument " "2"" of type '" "Dwg_Entity_ATTRIB *""'"); 
    }
    arg2 = (Dwg_Entity_ATTRIB *)(argp2);
    if (arg1) (arg1)->ATTRIB = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_ATTRIB_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_ATTRIB *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_ATTRIB_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_ATTRIB_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_ATTRIB *) ((arg1)->ATTRIB);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_ATTRIB, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_BLOCK_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_BLOCK *arg2 = (Dwg_Entity_BLOCK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_BLOCK_set(self,BLOCK);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_BLOCK_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_BLOCK, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_BLOCK_set" "', argument " "2"" of type '" "Dwg_Entity_BLOCK *""'"); 
    }
    arg2 = (Dwg_Entity_BLOCK *)(argp2);
    if (arg1) (arg1)->BLOCK = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_BLOCK_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_BLOCK *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_BLOCK_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_BLOCK_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_BLOCK *) ((arg1)->BLOCK);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_BLOCK, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_BODY_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_BODY *arg2 = (Dwg_Entity_BODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_BODY_set(self,BODY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_BODY_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_3DSOLID, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_BODY_set" "', argument " "2"" of type '" "Dwg_Entity_BODY *""'"); 
    }
    arg2 = (Dwg_Entity_BODY *)(argp2);
    if (arg1) (arg1)->BODY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_BODY_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_BODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_BODY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_BODY_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_BODY *) ((arg1)->BODY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_3DSOLID, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_CIRCLE_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_CIRCLE *arg2 = (Dwg_Entity_CIRCLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_CIRCLE_set(self,CIRCLE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_CIRCLE_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_CIRCLE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_CIRCLE_set" "', argument " "2"" of type '" "Dwg_Entity_CIRCLE *""'"); 
    }
    arg2 = (Dwg_Entity_CIRCLE *)(argp2);
    if (arg1) (arg1)->CIRCLE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_CIRCLE_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_CIRCLE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_CIRCLE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_CIRCLE_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_CIRCLE *) ((arg1)->CIRCLE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_CIRCLE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_DIMENSION_ALIGNED_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_DIMENSION_ALIGNED *arg2 = (Dwg_Entity_DIMENSION_ALIGNED *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_DIMENSION_ALIGNED_set(self,DIMENSION_ALIGNED);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_DIMENSION_ALIGNED_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_DIMENSION_ALIGNED_set" "', argument " "2"" of type '" "Dwg_Entity_DIMENSION_ALIGNED *""'"); 
    }
    arg2 = (Dwg_Entity_DIMENSION_ALIGNED *)(argp2);
    if (arg1) (arg1)->DIMENSION_ALIGNED = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_DIMENSION_ALIGNED_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_DIMENSION_ALIGNED *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_DIMENSION_ALIGNED_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_DIMENSION_ALIGNED_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_DIMENSION_ALIGNED *) ((arg1)->DIMENSION_ALIGNED);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_DIMENSION_ANG2LN_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_DIMENSION_ANG2LN *arg2 = (Dwg_Entity_DIMENSION_ANG2LN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_DIMENSION_ANG2LN_set(self,DIMENSION_ANG2LN);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_DIMENSION_ANG2LN_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_DIMENSION_ANG2LN_set" "', argument " "2"" of type '" "Dwg_Entity_DIMENSION_ANG2LN *""'"); 
    }
    arg2 = (Dwg_Entity_DIMENSION_ANG2LN *)(argp2);
    if (arg1) (arg1)->DIMENSION_ANG2LN = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_DIMENSION_ANG2LN_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_DIMENSION_ANG2LN *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_DIMENSION_ANG2LN_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_DIMENSION_ANG2LN_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_DIMENSION_ANG2LN *) ((arg1)->DIMENSION_ANG2LN);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_DIMENSION_ANG3PT_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_DIMENSION_ANG3PT *arg2 = (Dwg_Entity_DIMENSION_ANG3PT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_DIMENSION_ANG3PT_set(self,DIMENSION_ANG3PT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_DIMENSION_ANG3PT_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_DIMENSION_ANG3PT_set" "', argument " "2"" of type '" "Dwg_Entity_DIMENSION_ANG3PT *""'"); 
    }
    arg2 = (Dwg_Entity_DIMENSION_ANG3PT *)(argp2);
    if (arg1) (arg1)->DIMENSION_ANG3PT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_DIMENSION_ANG3PT_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_DIMENSION_ANG3PT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_DIMENSION_ANG3PT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_DIMENSION_ANG3PT_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_DIMENSION_ANG3PT *) ((arg1)->DIMENSION_ANG3PT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_DIMENSION_DIAMETER_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_DIMENSION_DIAMETER *arg2 = (Dwg_Entity_DIMENSION_DIAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_DIMENSION_DIAMETER_set(self,DIMENSION_DIAMETER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_DIMENSION_DIAMETER_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_DIMENSION_DIAMETER_set" "', argument " "2"" of type '" "Dwg_Entity_DIMENSION_DIAMETER *""'"); 
    }
    arg2 = (Dwg_Entity_DIMENSION_DIAMETER *)(argp2);
    if (arg1) (arg1)->DIMENSION_DIAMETER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_DIMENSION_DIAMETER_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_DIMENSION_DIAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_DIMENSION_DIAMETER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_DIMENSION_DIAMETER_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_DIMENSION_DIAMETER *) ((arg1)->DIMENSION_DIAMETER);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_DIMENSION_LINEAR_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_DIMENSION_LINEAR *arg2 = (Dwg_Entity_DIMENSION_LINEAR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_DIMENSION_LINEAR_set(self,DIMENSION_LINEAR);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_DIMENSION_LINEAR_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_DIMENSION_LINEAR_set" "', argument " "2"" of type '" "Dwg_Entity_DIMENSION_LINEAR *""'"); 
    }
    arg2 = (Dwg_Entity_DIMENSION_LINEAR *)(argp2);
    if (arg1) (arg1)->DIMENSION_LINEAR = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_DIMENSION_LINEAR_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_DIMENSION_LINEAR *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_DIMENSION_LINEAR_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_DIMENSION_LINEAR_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_DIMENSION_LINEAR *) ((arg1)->DIMENSION_LINEAR);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_DIMENSION_ORDINATE_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_DIMENSION_ORDINATE *arg2 = (Dwg_Entity_DIMENSION_ORDINATE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_DIMENSION_ORDINATE_set(self,DIMENSION_ORDINATE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_DIMENSION_ORDINATE_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_DIMENSION_ORDINATE_set" "', argument " "2"" of type '" "Dwg_Entity_DIMENSION_ORDINATE *""'"); 
    }
    arg2 = (Dwg_Entity_DIMENSION_ORDINATE *)(argp2);
    if (arg1) (arg1)->DIMENSION_ORDINATE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_DIMENSION_ORDINATE_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_DIMENSION_ORDINATE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_DIMENSION_ORDINATE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_DIMENSION_ORDINATE_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_DIMENSION_ORDINATE *) ((arg1)->DIMENSION_ORDINATE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_DIMENSION_RADIUS_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_DIMENSION_RADIUS *arg2 = (Dwg_Entity_DIMENSION_RADIUS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_DIMENSION_RADIUS_set(self,DIMENSION_RADIUS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_DIMENSION_RADIUS_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_DIMENSION_RADIUS_set" "', argument " "2"" of type '" "Dwg_Entity_DIMENSION_RADIUS *""'"); 
    }
    arg2 = (Dwg_Entity_DIMENSION_RADIUS *)(argp2);
    if (arg1) (arg1)->DIMENSION_RADIUS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_DIMENSION_RADIUS_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_DIMENSION_RADIUS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_DIMENSION_RADIUS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_DIMENSION_RADIUS_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_DIMENSION_RADIUS *) ((arg1)->DIMENSION_RADIUS);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_ELLIPSE_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_ELLIPSE *arg2 = (Dwg_Entity_ELLIPSE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_ELLIPSE_set(self,ELLIPSE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_ELLIPSE_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_ELLIPSE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_ELLIPSE_set" "', argument " "2"" of type '" "Dwg_Entity_ELLIPSE *""'"); 
    }
    arg2 = (Dwg_Entity_ELLIPSE *)(argp2);
    if (arg1) (arg1)->ELLIPSE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_ELLIPSE_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_ELLIPSE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_ELLIPSE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_ELLIPSE_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_ELLIPSE *) ((arg1)->ELLIPSE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_ELLIPSE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_ENDBLK_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_ENDBLK *arg2 = (Dwg_Entity_ENDBLK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_ENDBLK_set(self,ENDBLK);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_ENDBLK_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_ENDBLK, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_ENDBLK_set" "', argument " "2"" of type '" "Dwg_Entity_ENDBLK *""'"); 
    }
    arg2 = (Dwg_Entity_ENDBLK *)(argp2);
    if (arg1) (arg1)->ENDBLK = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_ENDBLK_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_ENDBLK *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_ENDBLK_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_ENDBLK_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_ENDBLK *) ((arg1)->ENDBLK);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_ENDBLK, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_INSERT_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_INSERT *arg2 = (Dwg_Entity_INSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_INSERT_set(self,INSERT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_INSERT_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_INSERT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_INSERT_set" "', argument " "2"" of type '" "Dwg_Entity_INSERT *""'"); 
    }
    arg2 = (Dwg_Entity_INSERT *)(argp2);
    if (arg1) (arg1)->INSERT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_INSERT_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_INSERT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_INSERT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_INSERT_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_INSERT *) ((arg1)->INSERT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_INSERT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_LEADER_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_LEADER *arg2 = (Dwg_Entity_LEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_LEADER_set(self,LEADER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_LEADER_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_LEADER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_LEADER_set" "', argument " "2"" of type '" "Dwg_Entity_LEADER *""'"); 
    }
    arg2 = (Dwg_Entity_LEADER *)(argp2);
    if (arg1) (arg1)->LEADER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_LEADER_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_LEADER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_LEADER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_LEADER_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_LEADER *) ((arg1)->LEADER);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_LEADER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_LINE_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_LINE *arg2 = (Dwg_Entity_LINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_LINE_set(self,LINE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_LINE_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_LINE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_LINE_set" "', argument " "2"" of type '" "Dwg_Entity_LINE *""'"); 
    }
    arg2 = (Dwg_Entity_LINE *)(argp2);
    if (arg1) (arg1)->LINE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_LINE_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_LINE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_LINE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_LINE_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_LINE *) ((arg1)->LINE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_LINE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_LOAD_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_LOAD *arg2 = (Dwg_Entity_LOAD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_LOAD_set(self,LOAD);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_LOAD_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_LOAD, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_LOAD_set" "', argument " "2"" of type '" "Dwg_Entity_LOAD *""'"); 
    }
    arg2 = (Dwg_Entity_LOAD *)(argp2);
    if (arg1) (arg1)->LOAD = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_LOAD_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_LOAD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_LOAD_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_LOAD_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_LOAD *) ((arg1)->LOAD);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_LOAD, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_MINSERT_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_MINSERT *arg2 = (Dwg_Entity_MINSERT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_MINSERT_set(self,MINSERT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_MINSERT_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_MINSERT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_MINSERT_set" "', argument " "2"" of type '" "Dwg_Entity_MINSERT *""'"); 
    }
    arg2 = (Dwg_Entity_MINSERT *)(argp2);
    if (arg1) (arg1)->MINSERT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_MINSERT_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_MINSERT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_MINSERT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_MINSERT_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_MINSERT *) ((arg1)->MINSERT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_MINSERT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_MLINE_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_MLINE *arg2 = (Dwg_Entity_MLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_MLINE_set(self,MLINE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_MLINE_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_MLINE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_MLINE_set" "', argument " "2"" of type '" "Dwg_Entity_MLINE *""'"); 
    }
    arg2 = (Dwg_Entity_MLINE *)(argp2);
    if (arg1) (arg1)->MLINE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_MLINE_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_MLINE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_MLINE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_MLINE_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_MLINE *) ((arg1)->MLINE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_MLINE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_MTEXT_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_MTEXT *arg2 = (Dwg_Entity_MTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_MTEXT_set(self,MTEXT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_MTEXT_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_MTEXT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_MTEXT_set" "', argument " "2"" of type '" "Dwg_Entity_MTEXT *""'"); 
    }
    arg2 = (Dwg_Entity_MTEXT *)(argp2);
    if (arg1) (arg1)->MTEXT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_MTEXT_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_MTEXT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_MTEXT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_MTEXT_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_MTEXT *) ((arg1)->MTEXT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_MTEXT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_OLEFRAME_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_OLEFRAME *arg2 = (Dwg_Entity_OLEFRAME *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_OLEFRAME_set(self,OLEFRAME);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_OLEFRAME_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_OLEFRAME, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_OLEFRAME_set" "', argument " "2"" of type '" "Dwg_Entity_OLEFRAME *""'"); 
    }
    arg2 = (Dwg_Entity_OLEFRAME *)(argp2);
    if (arg1) (arg1)->OLEFRAME = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_OLEFRAME_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_OLEFRAME *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_OLEFRAME_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_OLEFRAME_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_OLEFRAME *) ((arg1)->OLEFRAME);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_OLEFRAME, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_POINT_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_POINT *arg2 = (Dwg_Entity_POINT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_POINT_set(self,POINT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_POINT_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_POINT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_POINT_set" "', argument " "2"" of type '" "Dwg_Entity_POINT *""'"); 
    }
    arg2 = (Dwg_Entity_POINT *)(argp2);
    if (arg1) (arg1)->POINT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_POINT_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_POINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_POINT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_POINT_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_POINT *) ((arg1)->POINT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_POINT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_POLYLINE_2D_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_POLYLINE_2D *arg2 = (Dwg_Entity_POLYLINE_2D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_POLYLINE_2D_set(self,POLYLINE_2D);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_POLYLINE_2D_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_POLYLINE_2D, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_POLYLINE_2D_set" "', argument " "2"" of type '" "Dwg_Entity_POLYLINE_2D *""'"); 
    }
    arg2 = (Dwg_Entity_POLYLINE_2D *)(argp2);
    if (arg1) (arg1)->POLYLINE_2D = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_POLYLINE_2D_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_POLYLINE_2D *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_POLYLINE_2D_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_POLYLINE_2D_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_POLYLINE_2D *) ((arg1)->POLYLINE_2D);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_POLYLINE_3D_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_POLYLINE_3D *arg2 = (Dwg_Entity_POLYLINE_3D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_POLYLINE_3D_set(self,POLYLINE_3D);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_POLYLINE_3D_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_POLYLINE_3D, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_POLYLINE_3D_set" "', argument " "2"" of type '" "Dwg_Entity_POLYLINE_3D *""'"); 
    }
    arg2 = (Dwg_Entity_POLYLINE_3D *)(argp2);
    if (arg1) (arg1)->POLYLINE_3D = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_POLYLINE_3D_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_POLYLINE_3D *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_POLYLINE_3D_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_POLYLINE_3D_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_POLYLINE_3D *) ((arg1)->POLYLINE_3D);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_POLYLINE_3D, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_POLYLINE_MESH_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_POLYLINE_MESH *arg2 = (Dwg_Entity_POLYLINE_MESH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_POLYLINE_MESH_set(self,POLYLINE_MESH);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_POLYLINE_MESH_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_POLYLINE_MESH, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_POLYLINE_MESH_set" "', argument " "2"" of type '" "Dwg_Entity_POLYLINE_MESH *""'"); 
    }
    arg2 = (Dwg_Entity_POLYLINE_MESH *)(argp2);
    if (arg1) (arg1)->POLYLINE_MESH = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_POLYLINE_MESH_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_POLYLINE_MESH *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_POLYLINE_MESH_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_POLYLINE_MESH_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_POLYLINE_MESH *) ((arg1)->POLYLINE_MESH);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_POLYLINE_MESH, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_POLYLINE_PFACE_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_POLYLINE_PFACE *arg2 = (Dwg_Entity_POLYLINE_PFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_POLYLINE_PFACE_set(self,POLYLINE_PFACE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_POLYLINE_PFACE_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_POLYLINE_PFACE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_POLYLINE_PFACE_set" "', argument " "2"" of type '" "Dwg_Entity_POLYLINE_PFACE *""'"); 
    }
    arg2 = (Dwg_Entity_POLYLINE_PFACE *)(argp2);
    if (arg1) (arg1)->POLYLINE_PFACE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_POLYLINE_PFACE_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_POLYLINE_PFACE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_POLYLINE_PFACE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_POLYLINE_PFACE_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_POLYLINE_PFACE *) ((arg1)->POLYLINE_PFACE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_POLYLINE_PFACE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_PROXY_ENTITY_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_PROXY_ENTITY *arg2 = (Dwg_Entity_PROXY_ENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_PROXY_ENTITY_set(self,PROXY_ENTITY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_PROXY_ENTITY_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_PROXY_ENTITY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_PROXY_ENTITY_set" "', argument " "2"" of type '" "Dwg_Entity_PROXY_ENTITY *""'"); 
    }
    arg2 = (Dwg_Entity_PROXY_ENTITY *)(argp2);
    if (arg1) (arg1)->PROXY_ENTITY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_PROXY_ENTITY_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_PROXY_ENTITY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_PROXY_ENTITY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_PROXY_ENTITY_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_PROXY_ENTITY *) ((arg1)->PROXY_ENTITY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_PROXY_ENTITY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_RAY_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_RAY *arg2 = (Dwg_Entity_RAY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_RAY_set(self,RAY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_RAY_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_RAY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_RAY_set" "', argument " "2"" of type '" "Dwg_Entity_RAY *""'"); 
    }
    arg2 = (Dwg_Entity_RAY *)(argp2);
    if (arg1) (arg1)->RAY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_RAY_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_RAY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_RAY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_RAY_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_RAY *) ((arg1)->RAY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_RAY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_REGION_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_REGION *arg2 = (Dwg_Entity_REGION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_REGION_set(self,REGION);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_REGION_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_3DSOLID, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_REGION_set" "', argument " "2"" of type '" "Dwg_Entity_REGION *""'"); 
    }
    arg2 = (Dwg_Entity_REGION *)(argp2);
    if (arg1) (arg1)->REGION = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_REGION_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_REGION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_REGION_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_REGION_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_REGION *) ((arg1)->REGION);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_3DSOLID, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_SEQEND_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_SEQEND *arg2 = (Dwg_Entity_SEQEND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_SEQEND_set(self,SEQEND);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_SEQEND_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_SEQEND, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_SEQEND_set" "', argument " "2"" of type '" "Dwg_Entity_SEQEND *""'"); 
    }
    arg2 = (Dwg_Entity_SEQEND *)(argp2);
    if (arg1) (arg1)->SEQEND = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_SEQEND_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_SEQEND *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_SEQEND_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_SEQEND_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_SEQEND *) ((arg1)->SEQEND);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_SEQEND, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_SHAPE_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_SHAPE *arg2 = (Dwg_Entity_SHAPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_SHAPE_set(self,SHAPE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_SHAPE_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_SHAPE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_SHAPE_set" "', argument " "2"" of type '" "Dwg_Entity_SHAPE *""'"); 
    }
    arg2 = (Dwg_Entity_SHAPE *)(argp2);
    if (arg1) (arg1)->SHAPE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_SHAPE_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_SHAPE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_SHAPE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_SHAPE_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_SHAPE *) ((arg1)->SHAPE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_SHAPE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_SOLID_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_SOLID *arg2 = (Dwg_Entity_SOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_SOLID_set(self,SOLID);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_SOLID_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_SOLID, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_SOLID_set" "', argument " "2"" of type '" "Dwg_Entity_SOLID *""'"); 
    }
    arg2 = (Dwg_Entity_SOLID *)(argp2);
    if (arg1) (arg1)->SOLID = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_SOLID_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_SOLID *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_SOLID_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_SOLID_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_SOLID *) ((arg1)->SOLID);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_SOLID, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_SPLINE_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_SPLINE *arg2 = (Dwg_Entity_SPLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_SPLINE_set(self,SPLINE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_SPLINE_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_SPLINE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_SPLINE_set" "', argument " "2"" of type '" "Dwg_Entity_SPLINE *""'"); 
    }
    arg2 = (Dwg_Entity_SPLINE *)(argp2);
    if (arg1) (arg1)->SPLINE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_SPLINE_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_SPLINE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_SPLINE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_SPLINE_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_SPLINE *) ((arg1)->SPLINE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_SPLINE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_TEXT_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_TEXT *arg2 = (Dwg_Entity_TEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_TEXT_set(self,TEXT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_TEXT_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_TEXT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_TEXT_set" "', argument " "2"" of type '" "Dwg_Entity_TEXT *""'"); 
    }
    arg2 = (Dwg_Entity_TEXT *)(argp2);
    if (arg1) (arg1)->TEXT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_TEXT_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_TEXT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_TEXT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_TEXT_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_TEXT *) ((arg1)->TEXT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_TEXT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_TOLERANCE_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_TOLERANCE *arg2 = (Dwg_Entity_TOLERANCE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_TOLERANCE_set(self,TOLERANCE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_TOLERANCE_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_TOLERANCE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_TOLERANCE_set" "', argument " "2"" of type '" "Dwg_Entity_TOLERANCE *""'"); 
    }
    arg2 = (Dwg_Entity_TOLERANCE *)(argp2);
    if (arg1) (arg1)->TOLERANCE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_TOLERANCE_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_TOLERANCE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_TOLERANCE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_TOLERANCE_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_TOLERANCE *) ((arg1)->TOLERANCE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_TOLERANCE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_TRACE_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_TRACE *arg2 = (Dwg_Entity_TRACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_TRACE_set(self,TRACE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_TRACE_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_TRACE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_TRACE_set" "', argument " "2"" of type '" "Dwg_Entity_TRACE *""'"); 
    }
    arg2 = (Dwg_Entity_TRACE *)(argp2);
    if (arg1) (arg1)->TRACE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_TRACE_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_TRACE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_TRACE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_TRACE_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_TRACE *) ((arg1)->TRACE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_TRACE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_UNKNOWN_ENT_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_UNKNOWN_ENT *arg2 = (Dwg_Entity_UNKNOWN_ENT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_UNKNOWN_ENT_set(self,UNKNOWN_ENT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_UNKNOWN_ENT_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_UNKNOWN_ENT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_UNKNOWN_ENT_set" "', argument " "2"" of type '" "Dwg_Entity_UNKNOWN_ENT *""'"); 
    }
    arg2 = (Dwg_Entity_UNKNOWN_ENT *)(argp2);
    if (arg1) (arg1)->UNKNOWN_ENT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_UNKNOWN_ENT_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_UNKNOWN_ENT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_UNKNOWN_ENT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_UNKNOWN_ENT_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_UNKNOWN_ENT *) ((arg1)->UNKNOWN_ENT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_UNKNOWN_ENT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_VERTEX_2D_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_VERTEX_2D *arg2 = (Dwg_Entity_VERTEX_2D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_VERTEX_2D_set(self,VERTEX_2D);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_VERTEX_2D_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_VERTEX_2D, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_VERTEX_2D_set" "', argument " "2"" of type '" "Dwg_Entity_VERTEX_2D *""'"); 
    }
    arg2 = (Dwg_Entity_VERTEX_2D *)(argp2);
    if (arg1) (arg1)->VERTEX_2D = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_VERTEX_2D_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_VERTEX_2D *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_VERTEX_2D_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_VERTEX_2D_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_VERTEX_2D *) ((arg1)->VERTEX_2D);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_VERTEX_2D, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_VERTEX_3D_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_VERTEX_3D *arg2 = (Dwg_Entity_VERTEX_3D *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_VERTEX_3D_set(self,VERTEX_3D);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_VERTEX_3D_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_VERTEX_3D, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_VERTEX_3D_set" "', argument " "2"" of type '" "Dwg_Entity_VERTEX_3D *""'"); 
    }
    arg2 = (Dwg_Entity_VERTEX_3D *)(argp2);
    if (arg1) (arg1)->VERTEX_3D = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_VERTEX_3D_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_VERTEX_3D *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_VERTEX_3D_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_VERTEX_3D_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_VERTEX_3D *) ((arg1)->VERTEX_3D);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_VERTEX_3D, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_VERTEX_MESH_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_VERTEX_MESH *arg2 = (Dwg_Entity_VERTEX_MESH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_VERTEX_MESH_set(self,VERTEX_MESH);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_VERTEX_MESH_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_VERTEX_3D, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_VERTEX_MESH_set" "', argument " "2"" of type '" "Dwg_Entity_VERTEX_MESH *""'"); 
    }
    arg2 = (Dwg_Entity_VERTEX_MESH *)(argp2);
    if (arg1) (arg1)->VERTEX_MESH = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_VERTEX_MESH_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_VERTEX_MESH *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_VERTEX_MESH_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_VERTEX_MESH_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_VERTEX_MESH *) ((arg1)->VERTEX_MESH);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_VERTEX_3D, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_VERTEX_PFACE_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_VERTEX_PFACE *arg2 = (Dwg_Entity_VERTEX_PFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_VERTEX_PFACE_set(self,VERTEX_PFACE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_VERTEX_PFACE_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_VERTEX_3D, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_VERTEX_PFACE_set" "', argument " "2"" of type '" "Dwg_Entity_VERTEX_PFACE *""'"); 
    }
    arg2 = (Dwg_Entity_VERTEX_PFACE *)(argp2);
    if (arg1) (arg1)->VERTEX_PFACE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_VERTEX_PFACE_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_VERTEX_PFACE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_VERTEX_PFACE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_VERTEX_PFACE_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_VERTEX_PFACE *) ((arg1)->VERTEX_PFACE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_VERTEX_3D, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_VERTEX_PFACE_FACE_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_VERTEX_PFACE_FACE *arg2 = (Dwg_Entity_VERTEX_PFACE_FACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_VERTEX_PFACE_FACE_set(self,VERTEX_PFACE_FACE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_VERTEX_PFACE_FACE_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_VERTEX_PFACE_FACE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_VERTEX_PFACE_FACE_set" "', argument " "2"" of type '" "Dwg_Entity_VERTEX_PFACE_FACE *""'"); 
    }
    arg2 = (Dwg_Entity_VERTEX_PFACE_FACE *)(argp2);
    if (arg1) (arg1)->VERTEX_PFACE_FACE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_VERTEX_PFACE_FACE_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_VERTEX_PFACE_FACE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_VERTEX_PFACE_FACE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_VERTEX_PFACE_FACE_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_VERTEX_PFACE_FACE *) ((arg1)->VERTEX_PFACE_FACE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_VERTEX_PFACE_FACE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_VIEWPORT_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_VIEWPORT *arg2 = (Dwg_Entity_VIEWPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_VIEWPORT_set(self,VIEWPORT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_VIEWPORT_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_VIEWPORT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_VIEWPORT_set" "', argument " "2"" of type '" "Dwg_Entity_VIEWPORT *""'"); 
    }
    arg2 = (Dwg_Entity_VIEWPORT *)(argp2);
    if (arg1) (arg1)->VIEWPORT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_VIEWPORT_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_VIEWPORT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_VIEWPORT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_VIEWPORT_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_VIEWPORT *) ((arg1)->VIEWPORT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_VIEWPORT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_XLINE_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_XLINE *arg2 = (Dwg_Entity_XLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_XLINE_set(self,XLINE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_XLINE_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_RAY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_XLINE_set" "', argument " "2"" of type '" "Dwg_Entity_XLINE *""'"); 
    }
    arg2 = (Dwg_Entity_XLINE *)(argp2);
    if (arg1) (arg1)->XLINE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_XLINE_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_XLINE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_XLINE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_XLINE_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_XLINE *) ((arg1)->XLINE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_RAY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio__3DLINE_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity__3DLINE *arg2 = (Dwg_Entity__3DLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio__3DLINE_set(self,_3DLINE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio__3DLINE_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_3DLINE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio__3DLINE_set" "', argument " "2"" of type '" "Dwg_Entity__3DLINE *""'"); 
    }
    arg2 = (Dwg_Entity__3DLINE *)(argp2);
    if (arg1) (arg1)->_3DLINE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio__3DLINE_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity__3DLINE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio__3DLINE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio__3DLINE_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity__3DLINE *) ((arg1)->_3DLINE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_3DLINE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_CAMERA_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_CAMERA *arg2 = (Dwg_Entity_CAMERA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_CAMERA_set(self,CAMERA);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_CAMERA_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_CAMERA, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_CAMERA_set" "', argument " "2"" of type '" "Dwg_Entity_CAMERA *""'"); 
    }
    arg2 = (Dwg_Entity_CAMERA *)(argp2);
    if (arg1) (arg1)->CAMERA = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_CAMERA_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_CAMERA *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_CAMERA_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_CAMERA_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_CAMERA *) ((arg1)->CAMERA);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_CAMERA, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_DGNUNDERLAY_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_DGNUNDERLAY *arg2 = (Dwg_Entity_DGNUNDERLAY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_DGNUNDERLAY_set(self,DGNUNDERLAY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_DGNUNDERLAY_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_abstractentity_UNDERLAY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_DGNUNDERLAY_set" "', argument " "2"" of type '" "Dwg_Entity_DGNUNDERLAY *""'"); 
    }
    arg2 = (Dwg_Entity_DGNUNDERLAY *)(argp2);
    if (arg1) (arg1)->DGNUNDERLAY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_DGNUNDERLAY_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_DGNUNDERLAY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_DGNUNDERLAY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_DGNUNDERLAY_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_DGNUNDERLAY *) ((arg1)->DGNUNDERLAY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_abstractentity_UNDERLAY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_DWFUNDERLAY_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_DWFUNDERLAY *arg2 = (Dwg_Entity_DWFUNDERLAY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_DWFUNDERLAY_set(self,DWFUNDERLAY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_DWFUNDERLAY_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_abstractentity_UNDERLAY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_DWFUNDERLAY_set" "', argument " "2"" of type '" "Dwg_Entity_DWFUNDERLAY *""'"); 
    }
    arg2 = (Dwg_Entity_DWFUNDERLAY *)(argp2);
    if (arg1) (arg1)->DWFUNDERLAY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_DWFUNDERLAY_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_DWFUNDERLAY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_DWFUNDERLAY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_DWFUNDERLAY_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_DWFUNDERLAY *) ((arg1)->DWFUNDERLAY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_abstractentity_UNDERLAY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_ENDREP_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_ENDREP *arg2 = (Dwg_Entity_ENDREP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_ENDREP_set(self,ENDREP);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_ENDREP_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_ENDREP, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_ENDREP_set" "', argument " "2"" of type '" "Dwg_Entity_ENDREP *""'"); 
    }
    arg2 = (Dwg_Entity_ENDREP *)(argp2);
    if (arg1) (arg1)->ENDREP = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_ENDREP_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_ENDREP *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_ENDREP_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_ENDREP_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_ENDREP *) ((arg1)->ENDREP);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_ENDREP, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_HATCH_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_HATCH *arg2 = (Dwg_Entity_HATCH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_HATCH_set(self,HATCH);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_HATCH_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_HATCH, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_HATCH_set" "', argument " "2"" of type '" "Dwg_Entity_HATCH *""'"); 
    }
    arg2 = (Dwg_Entity_HATCH *)(argp2);
    if (arg1) (arg1)->HATCH = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_HATCH_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_HATCH *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_HATCH_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_HATCH_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_HATCH *) ((arg1)->HATCH);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_HATCH, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_IMAGE_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_IMAGE *arg2 = (Dwg_Entity_IMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_IMAGE_set(self,IMAGE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_IMAGE_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_IMAGE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_IMAGE_set" "', argument " "2"" of type '" "Dwg_Entity_IMAGE *""'"); 
    }
    arg2 = (Dwg_Entity_IMAGE *)(argp2);
    if (arg1) (arg1)->IMAGE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_IMAGE_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_IMAGE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_IMAGE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_IMAGE_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_IMAGE *) ((arg1)->IMAGE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_IMAGE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_JUMP_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_JUMP *arg2 = (Dwg_Entity_JUMP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_JUMP_set(self,JUMP);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_JUMP_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_JUMP, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_JUMP_set" "', argument " "2"" of type '" "Dwg_Entity_JUMP *""'"); 
    }
    arg2 = (Dwg_Entity_JUMP *)(argp2);
    if (arg1) (arg1)->JUMP = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_JUMP_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_JUMP *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_JUMP_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_JUMP_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_JUMP *) ((arg1)->JUMP);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_JUMP, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_LIGHT_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_LIGHT *arg2 = (Dwg_Entity_LIGHT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_LIGHT_set(self,LIGHT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_LIGHT_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_LIGHT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_LIGHT_set" "', argument " "2"" of type '" "Dwg_Entity_LIGHT *""'"); 
    }
    arg2 = (Dwg_Entity_LIGHT *)(argp2);
    if (arg1) (arg1)->LIGHT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_LIGHT_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_LIGHT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_LIGHT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_LIGHT_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_LIGHT *) ((arg1)->LIGHT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_LIGHT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_LWPOLYLINE_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_LWPOLYLINE *arg2 = (Dwg_Entity_LWPOLYLINE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_LWPOLYLINE_set(self,LWPOLYLINE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_LWPOLYLINE_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_LWPOLYLINE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_LWPOLYLINE_set" "', argument " "2"" of type '" "Dwg_Entity_LWPOLYLINE *""'"); 
    }
    arg2 = (Dwg_Entity_LWPOLYLINE *)(argp2);
    if (arg1) (arg1)->LWPOLYLINE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_LWPOLYLINE_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_LWPOLYLINE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_LWPOLYLINE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_LWPOLYLINE_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_LWPOLYLINE *) ((arg1)->LWPOLYLINE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_LWPOLYLINE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_MESH_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_MESH *arg2 = (Dwg_Entity_MESH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_MESH_set(self,MESH);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_MESH_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_MESH, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_MESH_set" "', argument " "2"" of type '" "Dwg_Entity_MESH *""'"); 
    }
    arg2 = (Dwg_Entity_MESH *)(argp2);
    if (arg1) (arg1)->MESH = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_MESH_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_MESH *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_MESH_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_MESH_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_MESH *) ((arg1)->MESH);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_MESH, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_MULTILEADER_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_MULTILEADER *arg2 = (Dwg_Entity_MULTILEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_MULTILEADER_set(self,MULTILEADER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_MULTILEADER_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_MULTILEADER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_MULTILEADER_set" "', argument " "2"" of type '" "Dwg_Entity_MULTILEADER *""'"); 
    }
    arg2 = (Dwg_Entity_MULTILEADER *)(argp2);
    if (arg1) (arg1)->MULTILEADER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_MULTILEADER_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_MULTILEADER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_MULTILEADER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_MULTILEADER_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_MULTILEADER *) ((arg1)->MULTILEADER);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_MULTILEADER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_OLE2FRAME_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_OLE2FRAME *arg2 = (Dwg_Entity_OLE2FRAME *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_OLE2FRAME_set(self,OLE2FRAME);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_OLE2FRAME_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_OLE2FRAME, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_OLE2FRAME_set" "', argument " "2"" of type '" "Dwg_Entity_OLE2FRAME *""'"); 
    }
    arg2 = (Dwg_Entity_OLE2FRAME *)(argp2);
    if (arg1) (arg1)->OLE2FRAME = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_OLE2FRAME_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_OLE2FRAME *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_OLE2FRAME_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_OLE2FRAME_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_OLE2FRAME *) ((arg1)->OLE2FRAME);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_OLE2FRAME, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_PDFUNDERLAY_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_PDFUNDERLAY *arg2 = (Dwg_Entity_PDFUNDERLAY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_PDFUNDERLAY_set(self,PDFUNDERLAY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_PDFUNDERLAY_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_abstractentity_UNDERLAY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_PDFUNDERLAY_set" "', argument " "2"" of type '" "Dwg_Entity_PDFUNDERLAY *""'"); 
    }
    arg2 = (Dwg_Entity_PDFUNDERLAY *)(argp2);
    if (arg1) (arg1)->PDFUNDERLAY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_PDFUNDERLAY_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_PDFUNDERLAY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_PDFUNDERLAY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_PDFUNDERLAY_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_PDFUNDERLAY *) ((arg1)->PDFUNDERLAY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_abstractentity_UNDERLAY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_REPEAT_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_REPEAT *arg2 = (Dwg_Entity_REPEAT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_REPEAT_set(self,REPEAT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_REPEAT_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_REPEAT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_REPEAT_set" "', argument " "2"" of type '" "Dwg_Entity_REPEAT *""'"); 
    }
    arg2 = (Dwg_Entity_REPEAT *)(argp2);
    if (arg1) (arg1)->REPEAT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_REPEAT_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_REPEAT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_REPEAT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_REPEAT_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_REPEAT *) ((arg1)->REPEAT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_REPEAT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_SECTIONOBJECT_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_SECTIONOBJECT *arg2 = (Dwg_Entity_SECTIONOBJECT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_SECTIONOBJECT_set(self,SECTIONOBJECT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_SECTIONOBJECT_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_SECTIONOBJECT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_SECTIONOBJECT_set" "', argument " "2"" of type '" "Dwg_Entity_SECTIONOBJECT *""'"); 
    }
    arg2 = (Dwg_Entity_SECTIONOBJECT *)(argp2);
    if (arg1) (arg1)->SECTIONOBJECT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_SECTIONOBJECT_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_SECTIONOBJECT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_SECTIONOBJECT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_SECTIONOBJECT_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_SECTIONOBJECT *) ((arg1)->SECTIONOBJECT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_SECTIONOBJECT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_WIPEOUT_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_WIPEOUT *arg2 = (Dwg_Entity_WIPEOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_WIPEOUT_set(self,WIPEOUT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_WIPEOUT_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_WIPEOUT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_WIPEOUT_set" "', argument " "2"" of type '" "Dwg_Entity_WIPEOUT *""'"); 
    }
    arg2 = (Dwg_Entity_WIPEOUT *)(argp2);
    if (arg1) (arg1)->WIPEOUT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_WIPEOUT_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_WIPEOUT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_WIPEOUT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_WIPEOUT_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_WIPEOUT *) ((arg1)->WIPEOUT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_WIPEOUT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_ARC_DIMENSION_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_ARC_DIMENSION *arg2 = (Dwg_Entity_ARC_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_ARC_DIMENSION_set(self,ARC_DIMENSION);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_ARC_DIMENSION_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_ARC_DIMENSION, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_ARC_DIMENSION_set" "', argument " "2"" of type '" "Dwg_Entity_ARC_DIMENSION *""'"); 
    }
    arg2 = (Dwg_Entity_ARC_DIMENSION *)(argp2);
    if (arg1) (arg1)->ARC_DIMENSION = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_ARC_DIMENSION_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_ARC_DIMENSION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_ARC_DIMENSION_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_ARC_DIMENSION_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_ARC_DIMENSION *) ((arg1)->ARC_DIMENSION);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_HELIX_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_HELIX *arg2 = (Dwg_Entity_HELIX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_HELIX_set(self,HELIX);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_HELIX_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_HELIX, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_HELIX_set" "', argument " "2"" of type '" "Dwg_Entity_HELIX *""'"); 
    }
    arg2 = (Dwg_Entity_HELIX *)(argp2);
    if (arg1) (arg1)->HELIX = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_HELIX_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_HELIX *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_HELIX_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_HELIX_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_HELIX *) ((arg1)->HELIX);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_HELIX, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_LARGE_RADIAL_DIMENSION_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_LARGE_RADIAL_DIMENSION *arg2 = (Dwg_Entity_LARGE_RADIAL_DIMENSION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_LARGE_RADIAL_DIMENSION_set(self,LARGE_RADIAL_DIMENSION);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_LARGE_RADIAL_DIMENSION_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_LARGE_RADIAL_DIMENSION_set" "', argument " "2"" of type '" "Dwg_Entity_LARGE_RADIAL_DIMENSION *""'"); 
    }
    arg2 = (Dwg_Entity_LARGE_RADIAL_DIMENSION *)(argp2);
    if (arg1) (arg1)->LARGE_RADIAL_DIMENSION = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_LARGE_RADIAL_DIMENSION_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_LARGE_RADIAL_DIMENSION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_LARGE_RADIAL_DIMENSION_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_LARGE_RADIAL_DIMENSION_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_LARGE_RADIAL_DIMENSION *) ((arg1)->LARGE_RADIAL_DIMENSION);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_LAYOUTPRINTCONFIG_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_LAYOUTPRINTCONFIG *arg2 = (Dwg_Entity_LAYOUTPRINTCONFIG *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_LAYOUTPRINTCONFIG_set(self,LAYOUTPRINTCONFIG);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_LAYOUTPRINTCONFIG_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_LAYOUTPRINTCONFIG, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_LAYOUTPRINTCONFIG_set" "', argument " "2"" of type '" "Dwg_Entity_LAYOUTPRINTCONFIG *""'"); 
    }
    arg2 = (Dwg_Entity_LAYOUTPRINTCONFIG *)(argp2);
    if (arg1) (arg1)->LAYOUTPRINTCONFIG = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_LAYOUTPRINTCONFIG_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_LAYOUTPRINTCONFIG *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_LAYOUTPRINTCONFIG_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_LAYOUTPRINTCONFIG_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_LAYOUTPRINTCONFIG *) ((arg1)->LAYOUTPRINTCONFIG);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_LAYOUTPRINTCONFIG, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_PLANESURFACE_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_PLANESURFACE *arg2 = (Dwg_Entity_PLANESURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_PLANESURFACE_set(self,PLANESURFACE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_PLANESURFACE_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_PLANESURFACE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_PLANESURFACE_set" "', argument " "2"" of type '" "Dwg_Entity_PLANESURFACE *""'"); 
    }
    arg2 = (Dwg_Entity_PLANESURFACE *)(argp2);
    if (arg1) (arg1)->PLANESURFACE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_PLANESURFACE_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_PLANESURFACE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_PLANESURFACE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_PLANESURFACE_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_PLANESURFACE *) ((arg1)->PLANESURFACE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_POINTCLOUD_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_POINTCLOUD *arg2 = (Dwg_Entity_POINTCLOUD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_POINTCLOUD_set(self,POINTCLOUD);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_POINTCLOUD_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_POINTCLOUD, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_POINTCLOUD_set" "', argument " "2"" of type '" "Dwg_Entity_POINTCLOUD *""'"); 
    }
    arg2 = (Dwg_Entity_POINTCLOUD *)(argp2);
    if (arg1) (arg1)->POINTCLOUD = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_POINTCLOUD_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_POINTCLOUD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_POINTCLOUD_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_POINTCLOUD_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_POINTCLOUD *) ((arg1)->POINTCLOUD);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_POINTCLOUDEX_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_POINTCLOUDEX *arg2 = (Dwg_Entity_POINTCLOUDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_POINTCLOUDEX_set(self,POINTCLOUDEX);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_POINTCLOUDEX_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_POINTCLOUDEX, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_POINTCLOUDEX_set" "', argument " "2"" of type '" "Dwg_Entity_POINTCLOUDEX *""'"); 
    }
    arg2 = (Dwg_Entity_POINTCLOUDEX *)(argp2);
    if (arg1) (arg1)->POINTCLOUDEX = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_POINTCLOUDEX_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_POINTCLOUDEX *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_POINTCLOUDEX_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_POINTCLOUDEX_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_POINTCLOUDEX *) ((arg1)->POINTCLOUDEX);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_ALIGNMENTPARAMETERENTITY_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_ALIGNMENTPARAMETERENTITY *arg2 = (Dwg_Entity_ALIGNMENTPARAMETERENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_ALIGNMENTPARAMETERENTITY_set(self,ALIGNMENTPARAMETERENTITY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_ALIGNMENTPARAMETERENTITY_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_ALIGNMENTPARAMETERENTITY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_ALIGNMENTPARAMETERENTITY_set" "', argument " "2"" of type '" "Dwg_Entity_ALIGNMENTPARAMETERENTITY *""'"); 
    }
    arg2 = (Dwg_Entity_ALIGNMENTPARAMETERENTITY *)(argp2);
    if (arg1) (arg1)->ALIGNMENTPARAMETERENTITY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_ALIGNMENTPARAMETERENTITY_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_ALIGNMENTPARAMETERENTITY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_ALIGNMENTPARAMETERENTITY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_ALIGNMENTPARAMETERENTITY_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_ALIGNMENTPARAMETERENTITY *) ((arg1)->ALIGNMENTPARAMETERENTITY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_ALIGNMENTPARAMETERENTITY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_ARCALIGNEDTEXT_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_ARCALIGNEDTEXT *arg2 = (Dwg_Entity_ARCALIGNEDTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_ARCALIGNEDTEXT_set(self,ARCALIGNEDTEXT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_ARCALIGNEDTEXT_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_ARCALIGNEDTEXT_set" "', argument " "2"" of type '" "Dwg_Entity_ARCALIGNEDTEXT *""'"); 
    }
    arg2 = (Dwg_Entity_ARCALIGNEDTEXT *)(argp2);
    if (arg1) (arg1)->ARCALIGNEDTEXT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_ARCALIGNEDTEXT_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_ARCALIGNEDTEXT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_ARCALIGNEDTEXT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_ARCALIGNEDTEXT_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_ARCALIGNEDTEXT *) ((arg1)->ARCALIGNEDTEXT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_BASEPOINTPARAMETERENTITY_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_BASEPOINTPARAMETERENTITY *arg2 = (Dwg_Entity_BASEPOINTPARAMETERENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_BASEPOINTPARAMETERENTITY_set(self,BASEPOINTPARAMETERENTITY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_BASEPOINTPARAMETERENTITY_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_BASEPOINTPARAMETERENTITY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_BASEPOINTPARAMETERENTITY_set" "', argument " "2"" of type '" "Dwg_Entity_BASEPOINTPARAMETERENTITY *""'"); 
    }
    arg2 = (Dwg_Entity_BASEPOINTPARAMETERENTITY *)(argp2);
    if (arg1) (arg1)->BASEPOINTPARAMETERENTITY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_BASEPOINTPARAMETERENTITY_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_BASEPOINTPARAMETERENTITY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_BASEPOINTPARAMETERENTITY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_BASEPOINTPARAMETERENTITY_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_BASEPOINTPARAMETERENTITY *) ((arg1)->BASEPOINTPARAMETERENTITY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_BASEPOINTPARAMETERENTITY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_EXTRUDEDSURFACE_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_EXTRUDEDSURFACE *arg2 = (Dwg_Entity_EXTRUDEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_EXTRUDEDSURFACE_set(self,EXTRUDEDSURFACE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_EXTRUDEDSURFACE_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_EXTRUDEDSURFACE_set" "', argument " "2"" of type '" "Dwg_Entity_EXTRUDEDSURFACE *""'"); 
    }
    arg2 = (Dwg_Entity_EXTRUDEDSURFACE *)(argp2);
    if (arg1) (arg1)->EXTRUDEDSURFACE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_EXTRUDEDSURFACE_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_EXTRUDEDSURFACE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_EXTRUDEDSURFACE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_EXTRUDEDSURFACE_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_EXTRUDEDSURFACE *) ((arg1)->EXTRUDEDSURFACE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_FLIPGRIPENTITY_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_FLIPGRIPENTITY *arg2 = (Dwg_Entity_FLIPGRIPENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_FLIPGRIPENTITY_set(self,FLIPGRIPENTITY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_FLIPGRIPENTITY_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_FLIPGRIPENTITY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_FLIPGRIPENTITY_set" "', argument " "2"" of type '" "Dwg_Entity_FLIPGRIPENTITY *""'"); 
    }
    arg2 = (Dwg_Entity_FLIPGRIPENTITY *)(argp2);
    if (arg1) (arg1)->FLIPGRIPENTITY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_FLIPGRIPENTITY_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_FLIPGRIPENTITY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_FLIPGRIPENTITY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_FLIPGRIPENTITY_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_FLIPGRIPENTITY *) ((arg1)->FLIPGRIPENTITY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_FLIPGRIPENTITY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_FLIPPARAMETERENTITY_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_FLIPPARAMETERENTITY *arg2 = (Dwg_Entity_FLIPPARAMETERENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_FLIPPARAMETERENTITY_set(self,FLIPPARAMETERENTITY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_FLIPPARAMETERENTITY_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_FLIPPARAMETERENTITY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_FLIPPARAMETERENTITY_set" "', argument " "2"" of type '" "Dwg_Entity_FLIPPARAMETERENTITY *""'"); 
    }
    arg2 = (Dwg_Entity_FLIPPARAMETERENTITY *)(argp2);
    if (arg1) (arg1)->FLIPPARAMETERENTITY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_FLIPPARAMETERENTITY_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_FLIPPARAMETERENTITY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_FLIPPARAMETERENTITY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_FLIPPARAMETERENTITY_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_FLIPPARAMETERENTITY *) ((arg1)->FLIPPARAMETERENTITY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_FLIPPARAMETERENTITY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_GEOPOSITIONMARKER_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_GEOPOSITIONMARKER *arg2 = (Dwg_Entity_GEOPOSITIONMARKER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_GEOPOSITIONMARKER_set(self,GEOPOSITIONMARKER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_GEOPOSITIONMARKER_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_GEOPOSITIONMARKER_set" "', argument " "2"" of type '" "Dwg_Entity_GEOPOSITIONMARKER *""'"); 
    }
    arg2 = (Dwg_Entity_GEOPOSITIONMARKER *)(argp2);
    if (arg1) (arg1)->GEOPOSITIONMARKER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_GEOPOSITIONMARKER_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_GEOPOSITIONMARKER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_GEOPOSITIONMARKER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_GEOPOSITIONMARKER_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_GEOPOSITIONMARKER *) ((arg1)->GEOPOSITIONMARKER);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_LINEARGRIPENTITY_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_LINEARGRIPENTITY *arg2 = (Dwg_Entity_LINEARGRIPENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_LINEARGRIPENTITY_set(self,LINEARGRIPENTITY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_LINEARGRIPENTITY_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_LINEARGRIPENTITY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_LINEARGRIPENTITY_set" "', argument " "2"" of type '" "Dwg_Entity_LINEARGRIPENTITY *""'"); 
    }
    arg2 = (Dwg_Entity_LINEARGRIPENTITY *)(argp2);
    if (arg1) (arg1)->LINEARGRIPENTITY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_LINEARGRIPENTITY_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_LINEARGRIPENTITY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_LINEARGRIPENTITY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_LINEARGRIPENTITY_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_LINEARGRIPENTITY *) ((arg1)->LINEARGRIPENTITY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_LINEARGRIPENTITY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_LINEARPARAMETERENTITY_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_LINEARPARAMETERENTITY *arg2 = (Dwg_Entity_LINEARPARAMETERENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_LINEARPARAMETERENTITY_set(self,LINEARPARAMETERENTITY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_LINEARPARAMETERENTITY_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_LINEARPARAMETERENTITY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_LINEARPARAMETERENTITY_set" "', argument " "2"" of type '" "Dwg_Entity_LINEARPARAMETERENTITY *""'"); 
    }
    arg2 = (Dwg_Entity_LINEARPARAMETERENTITY *)(argp2);
    if (arg1) (arg1)->LINEARPARAMETERENTITY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_LINEARPARAMETERENTITY_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_LINEARPARAMETERENTITY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_LINEARPARAMETERENTITY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_LINEARPARAMETERENTITY_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_LINEARPARAMETERENTITY *) ((arg1)->LINEARPARAMETERENTITY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_LINEARPARAMETERENTITY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_LOFTEDSURFACE_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_LOFTEDSURFACE *arg2 = (Dwg_Entity_LOFTEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_LOFTEDSURFACE_set(self,LOFTEDSURFACE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_LOFTEDSURFACE_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_LOFTEDSURFACE_set" "', argument " "2"" of type '" "Dwg_Entity_LOFTEDSURFACE *""'"); 
    }
    arg2 = (Dwg_Entity_LOFTEDSURFACE *)(argp2);
    if (arg1) (arg1)->LOFTEDSURFACE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_LOFTEDSURFACE_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_LOFTEDSURFACE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_LOFTEDSURFACE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_LOFTEDSURFACE_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_LOFTEDSURFACE *) ((arg1)->LOFTEDSURFACE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_MPOLYGON_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_MPOLYGON *arg2 = (Dwg_Entity_MPOLYGON *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_MPOLYGON_set(self,MPOLYGON);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_MPOLYGON_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_MPOLYGON, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_MPOLYGON_set" "', argument " "2"" of type '" "Dwg_Entity_MPOLYGON *""'"); 
    }
    arg2 = (Dwg_Entity_MPOLYGON *)(argp2);
    if (arg1) (arg1)->MPOLYGON = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_MPOLYGON_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_MPOLYGON *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_MPOLYGON_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_MPOLYGON_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_MPOLYGON *) ((arg1)->MPOLYGON);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_MPOLYGON, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_NAVISWORKSMODEL_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_NAVISWORKSMODEL *arg2 = (Dwg_Entity_NAVISWORKSMODEL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_NAVISWORKSMODEL_set(self,NAVISWORKSMODEL);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_NAVISWORKSMODEL_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_NAVISWORKSMODEL, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_NAVISWORKSMODEL_set" "', argument " "2"" of type '" "Dwg_Entity_NAVISWORKSMODEL *""'"); 
    }
    arg2 = (Dwg_Entity_NAVISWORKSMODEL *)(argp2);
    if (arg1) (arg1)->NAVISWORKSMODEL = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_NAVISWORKSMODEL_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_NAVISWORKSMODEL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_NAVISWORKSMODEL_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_NAVISWORKSMODEL_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_NAVISWORKSMODEL *) ((arg1)->NAVISWORKSMODEL);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_NAVISWORKSMODEL, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_NURBSURFACE_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_NURBSURFACE *arg2 = (Dwg_Entity_NURBSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_NURBSURFACE_set(self,NURBSURFACE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_NURBSURFACE_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_NURBSURFACE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_NURBSURFACE_set" "', argument " "2"" of type '" "Dwg_Entity_NURBSURFACE *""'"); 
    }
    arg2 = (Dwg_Entity_NURBSURFACE *)(argp2);
    if (arg1) (arg1)->NURBSURFACE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_NURBSURFACE_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_NURBSURFACE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_NURBSURFACE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_NURBSURFACE_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_NURBSURFACE *) ((arg1)->NURBSURFACE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_NURBSURFACE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_POINTPARAMETERENTITY_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_POINTPARAMETERENTITY *arg2 = (Dwg_Entity_POINTPARAMETERENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_POINTPARAMETERENTITY_set(self,POINTPARAMETERENTITY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_POINTPARAMETERENTITY_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_POINTPARAMETERENTITY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_POINTPARAMETERENTITY_set" "', argument " "2"" of type '" "Dwg_Entity_POINTPARAMETERENTITY *""'"); 
    }
    arg2 = (Dwg_Entity_POINTPARAMETERENTITY *)(argp2);
    if (arg1) (arg1)->POINTPARAMETERENTITY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_POINTPARAMETERENTITY_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_POINTPARAMETERENTITY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_POINTPARAMETERENTITY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_POINTPARAMETERENTITY_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_POINTPARAMETERENTITY *) ((arg1)->POINTPARAMETERENTITY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_POINTPARAMETERENTITY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_POLARGRIPENTITY_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_POLARGRIPENTITY *arg2 = (Dwg_Entity_POLARGRIPENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_POLARGRIPENTITY_set(self,POLARGRIPENTITY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_POLARGRIPENTITY_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_POLARGRIPENTITY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_POLARGRIPENTITY_set" "', argument " "2"" of type '" "Dwg_Entity_POLARGRIPENTITY *""'"); 
    }
    arg2 = (Dwg_Entity_POLARGRIPENTITY *)(argp2);
    if (arg1) (arg1)->POLARGRIPENTITY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_POLARGRIPENTITY_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_POLARGRIPENTITY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_POLARGRIPENTITY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_POLARGRIPENTITY_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_POLARGRIPENTITY *) ((arg1)->POLARGRIPENTITY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_POLARGRIPENTITY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_REVOLVEDSURFACE_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_REVOLVEDSURFACE *arg2 = (Dwg_Entity_REVOLVEDSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_REVOLVEDSURFACE_set(self,REVOLVEDSURFACE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_REVOLVEDSURFACE_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_REVOLVEDSURFACE_set" "', argument " "2"" of type '" "Dwg_Entity_REVOLVEDSURFACE *""'"); 
    }
    arg2 = (Dwg_Entity_REVOLVEDSURFACE *)(argp2);
    if (arg1) (arg1)->REVOLVEDSURFACE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_REVOLVEDSURFACE_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_REVOLVEDSURFACE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_REVOLVEDSURFACE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_REVOLVEDSURFACE_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_REVOLVEDSURFACE *) ((arg1)->REVOLVEDSURFACE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_ROTATIONGRIPENTITY_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_ROTATIONGRIPENTITY *arg2 = (Dwg_Entity_ROTATIONGRIPENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_ROTATIONGRIPENTITY_set(self,ROTATIONGRIPENTITY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_ROTATIONGRIPENTITY_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_ROTATIONGRIPENTITY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_ROTATIONGRIPENTITY_set" "', argument " "2"" of type '" "Dwg_Entity_ROTATIONGRIPENTITY *""'"); 
    }
    arg2 = (Dwg_Entity_ROTATIONGRIPENTITY *)(argp2);
    if (arg1) (arg1)->ROTATIONGRIPENTITY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_ROTATIONGRIPENTITY_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_ROTATIONGRIPENTITY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_ROTATIONGRIPENTITY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_ROTATIONGRIPENTITY_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_ROTATIONGRIPENTITY *) ((arg1)->ROTATIONGRIPENTITY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_ROTATIONGRIPENTITY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_ROTATIONPARAMETERENTITY_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_ROTATIONPARAMETERENTITY *arg2 = (Dwg_Entity_ROTATIONPARAMETERENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_ROTATIONPARAMETERENTITY_set(self,ROTATIONPARAMETERENTITY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_ROTATIONPARAMETERENTITY_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_ROTATIONPARAMETERENTITY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_ROTATIONPARAMETERENTITY_set" "', argument " "2"" of type '" "Dwg_Entity_ROTATIONPARAMETERENTITY *""'"); 
    }
    arg2 = (Dwg_Entity_ROTATIONPARAMETERENTITY *)(argp2);
    if (arg1) (arg1)->ROTATIONPARAMETERENTITY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_ROTATIONPARAMETERENTITY_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_ROTATIONPARAMETERENTITY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_ROTATIONPARAMETERENTITY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_ROTATIONPARAMETERENTITY_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_ROTATIONPARAMETERENTITY *) ((arg1)->ROTATIONPARAMETERENTITY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_ROTATIONPARAMETERENTITY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_RTEXT_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_RTEXT *arg2 = (Dwg_Entity_RTEXT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_RTEXT_set(self,RTEXT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_RTEXT_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_RTEXT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_RTEXT_set" "', argument " "2"" of type '" "Dwg_Entity_RTEXT *""'"); 
    }
    arg2 = (Dwg_Entity_RTEXT *)(argp2);
    if (arg1) (arg1)->RTEXT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_RTEXT_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_RTEXT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_RTEXT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_RTEXT_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_RTEXT *) ((arg1)->RTEXT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_RTEXT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_SWEPTSURFACE_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_SWEPTSURFACE *arg2 = (Dwg_Entity_SWEPTSURFACE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_SWEPTSURFACE_set(self,SWEPTSURFACE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_SWEPTSURFACE_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_SWEPTSURFACE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_SWEPTSURFACE_set" "', argument " "2"" of type '" "Dwg_Entity_SWEPTSURFACE *""'"); 
    }
    arg2 = (Dwg_Entity_SWEPTSURFACE *)(argp2);
    if (arg1) (arg1)->SWEPTSURFACE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_SWEPTSURFACE_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_SWEPTSURFACE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_SWEPTSURFACE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_SWEPTSURFACE_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_SWEPTSURFACE *) ((arg1)->SWEPTSURFACE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_SWEPTSURFACE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_TABLE_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_TABLE *arg2 = (Dwg_Entity_TABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_TABLE_set(self,TABLE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_TABLE_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_TABLE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_TABLE_set" "', argument " "2"" of type '" "Dwg_Entity_TABLE *""'"); 
    }
    arg2 = (Dwg_Entity_TABLE *)(argp2);
    if (arg1) (arg1)->TABLE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_TABLE_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_TABLE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_TABLE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_TABLE_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_TABLE *) ((arg1)->TABLE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_TABLE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_VISIBILITYGRIPENTITY_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_VISIBILITYGRIPENTITY *arg2 = (Dwg_Entity_VISIBILITYGRIPENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_VISIBILITYGRIPENTITY_set(self,VISIBILITYGRIPENTITY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_VISIBILITYGRIPENTITY_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_VISIBILITYGRIPENTITY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_VISIBILITYGRIPENTITY_set" "', argument " "2"" of type '" "Dwg_Entity_VISIBILITYGRIPENTITY *""'"); 
    }
    arg2 = (Dwg_Entity_VISIBILITYGRIPENTITY *)(argp2);
    if (arg1) (arg1)->VISIBILITYGRIPENTITY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_VISIBILITYGRIPENTITY_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_VISIBILITYGRIPENTITY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_VISIBILITYGRIPENTITY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_VISIBILITYGRIPENTITY_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_VISIBILITYGRIPENTITY *) ((arg1)->VISIBILITYGRIPENTITY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_VISIBILITYGRIPENTITY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_VISIBILITYPARAMETERENTITY_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_VISIBILITYPARAMETERENTITY *arg2 = (Dwg_Entity_VISIBILITYPARAMETERENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_VISIBILITYPARAMETERENTITY_set(self,VISIBILITYPARAMETERENTITY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_VISIBILITYPARAMETERENTITY_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_VISIBILITYPARAMETERENTITY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_VISIBILITYPARAMETERENTITY_set" "', argument " "2"" of type '" "Dwg_Entity_VISIBILITYPARAMETERENTITY *""'"); 
    }
    arg2 = (Dwg_Entity_VISIBILITYPARAMETERENTITY *)(argp2);
    if (arg1) (arg1)->VISIBILITYPARAMETERENTITY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_VISIBILITYPARAMETERENTITY_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_VISIBILITYPARAMETERENTITY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_VISIBILITYPARAMETERENTITY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_VISIBILITYPARAMETERENTITY_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_VISIBILITYPARAMETERENTITY *) ((arg1)->VISIBILITYPARAMETERENTITY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_VISIBILITYPARAMETERENTITY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_XYGRIPENTITY_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_XYGRIPENTITY *arg2 = (Dwg_Entity_XYGRIPENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_XYGRIPENTITY_set(self,XYGRIPENTITY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_XYGRIPENTITY_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_XYGRIPENTITY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_XYGRIPENTITY_set" "', argument " "2"" of type '" "Dwg_Entity_XYGRIPENTITY *""'"); 
    }
    arg2 = (Dwg_Entity_XYGRIPENTITY *)(argp2);
    if (arg1) (arg1)->XYGRIPENTITY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_XYGRIPENTITY_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_XYGRIPENTITY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_XYGRIPENTITY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_XYGRIPENTITY_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_XYGRIPENTITY *) ((arg1)->XYGRIPENTITY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_XYGRIPENTITY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_XYPARAMETERENTITY_set) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    Dwg_Entity_XYPARAMETERENTITY *arg2 = (Dwg_Entity_XYPARAMETERENTITY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_XYPARAMETERENTITY_set(self,XYPARAMETERENTITY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_XYPARAMETERENTITY_set" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_XYPARAMETERENTITY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_entity_tio_XYPARAMETERENTITY_set" "', argument " "2"" of type '" "Dwg_Entity_XYPARAMETERENTITY *""'"); 
    }
    arg2 = (Dwg_Entity_XYPARAMETERENTITY *)(argp2);
    if (arg1) (arg1)->XYPARAMETERENTITY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_entity_tio_XYPARAMETERENTITY_get) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_XYPARAMETERENTITY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_entity_tio_XYPARAMETERENTITY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_entity_tio_XYPARAMETERENTITY_get" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    result = (Dwg_Entity_XYPARAMETERENTITY *) ((arg1)->XYPARAMETERENTITY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_XYPARAMETERENTITY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new__dwg_object_entity_tio) {
  {
    int argvi = 0;
    _dwg_object_entity_tio *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new__dwg_object_entity_tio();");
    }
    result = (_dwg_object_entity_tio *)calloc(1, sizeof(_dwg_object_entity_tio));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity_tio, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete__dwg_object_entity_tio) {
  {
    _dwg_object_entity_tio *arg1 = (_dwg_object_entity_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete__dwg_object_entity_tio(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity_tio, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete__dwg_object_entity_tio" "', argument " "1"" of type '" "_dwg_object_entity_tio *""'"); 
    }
    arg1 = (_dwg_object_entity_tio *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Object_objid_set) {
  {
    struct _dwg_object_object *arg1 = (struct _dwg_object_object *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Object_objid_set(self,objid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Object_objid_set" "', argument " "1"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg1 = (struct _dwg_object_object *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Object_objid_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->objid = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Object_objid_get) {
  {
    struct _dwg_object_object *arg1 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Object_objid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Object_objid_get" "', argument " "1"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg1 = (struct _dwg_object_object *)(argp1);
    result = (BITCODE_BL) ((arg1)->objid);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Object_tio_get) {
  {
    struct _dwg_object_object *arg1 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    _dwg_object_object_tio *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Object_tio_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Object_tio_get" "', argument " "1"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg1 = (struct _dwg_object_object *)(argp1);
    result = (_dwg_object_object_tio *)& ((arg1)->tio);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object_tio, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Object_dwg_set) {
  {
    struct _dwg_object_object *arg1 = (struct _dwg_object_object *) 0 ;
    struct _dwg_struct *arg2 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Object_dwg_set(self,dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Object_dwg_set" "', argument " "1"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg1 = (struct _dwg_object_object *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_struct, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_Object_dwg_set" "', argument " "2"" of type '" "struct _dwg_struct *""'"); 
    }
    arg2 = (struct _dwg_struct *)(argp2);
    if (arg1) (arg1)->dwg = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Object_dwg_get) {
  {
    struct _dwg_object_object *arg1 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_struct *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Object_dwg_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Object_dwg_get" "', argument " "1"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg1 = (struct _dwg_object_object *)(argp1);
    result = (struct _dwg_struct *) ((arg1)->dwg);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_struct, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Object_num_eed_set) {
  {
    struct _dwg_object_object *arg1 = (struct _dwg_object_object *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Object_num_eed_set(self,num_eed);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Object_num_eed_set" "', argument " "1"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg1 = (struct _dwg_object_object *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Object_num_eed_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_eed = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Object_num_eed_get) {
  {
    struct _dwg_object_object *arg1 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Object_num_eed_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Object_num_eed_get" "', argument " "1"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg1 = (struct _dwg_object_object *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_eed);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Object_eed_set) {
  {
    struct _dwg_object_object *arg1 = (struct _dwg_object_object *) 0 ;
    Dwg_Eed *arg2 = (Dwg_Eed *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Object_eed_set(self,eed);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Object_eed_set" "', argument " "1"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg1 = (struct _dwg_object_object *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_entity_eed, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_Object_eed_set" "', argument " "2"" of type '" "Dwg_Eed *""'"); 
    }
    arg2 = (Dwg_Eed *)(argp2);
    if (arg1) (arg1)->eed = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Object_eed_get) {
  {
    struct _dwg_object_object *arg1 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Eed *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Object_eed_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Object_eed_get" "', argument " "1"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg1 = (struct _dwg_object_object *)(argp1);
    result = (Dwg_Eed *) ((arg1)->eed);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_eed, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Object_ownerhandle_set) {
  {
    struct _dwg_object_object *arg1 = (struct _dwg_object_object *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Object_ownerhandle_set(self,ownerhandle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Object_ownerhandle_set" "', argument " "1"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg1 = (struct _dwg_object_object *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_Object_ownerhandle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->ownerhandle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Object_ownerhandle_get) {
  {
    struct _dwg_object_object *arg1 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Object_ownerhandle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Object_ownerhandle_get" "', argument " "1"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg1 = (struct _dwg_object_object *)(argp1);
    result = (BITCODE_H) ((arg1)->ownerhandle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Object_num_reactors_set) {
  {
    struct _dwg_object_object *arg1 = (struct _dwg_object_object *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Object_num_reactors_set(self,num_reactors);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Object_num_reactors_set" "', argument " "1"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg1 = (struct _dwg_object_object *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Object_num_reactors_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_reactors = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Object_num_reactors_get) {
  {
    struct _dwg_object_object *arg1 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Object_num_reactors_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Object_num_reactors_get" "', argument " "1"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg1 = (struct _dwg_object_object *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_reactors);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Object_reactors_set) {
  {
    struct _dwg_object_object *arg1 = (struct _dwg_object_object *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Object_reactors_set(self,reactors);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Object_reactors_set" "', argument " "1"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg1 = (struct _dwg_object_object *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_Object_reactors_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->reactors = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Object_reactors_get) {
  {
    struct _dwg_object_object *arg1 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Object_reactors_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Object_reactors_get" "', argument " "1"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg1 = (struct _dwg_object_object *)(argp1);
    result = (BITCODE_H *) ((arg1)->reactors);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Object_xdicobjhandle_set) {
  {
    struct _dwg_object_object *arg1 = (struct _dwg_object_object *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Object_xdicobjhandle_set(self,xdicobjhandle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Object_xdicobjhandle_set" "', argument " "1"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg1 = (struct _dwg_object_object *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_Object_xdicobjhandle_set" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    if (arg1) (arg1)->xdicobjhandle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Object_xdicobjhandle_get) {
  {
    struct _dwg_object_object *arg1 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Object_xdicobjhandle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Object_xdicobjhandle_get" "', argument " "1"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg1 = (struct _dwg_object_object *)(argp1);
    result = (BITCODE_H) ((arg1)->xdicobjhandle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Object_is_xdic_missing_set) {
  {
    struct _dwg_object_object *arg1 = (struct _dwg_object_object *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Object_is_xdic_missing_set(self,is_xdic_missing);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Object_is_xdic_missing_set" "', argument " "1"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg1 = (struct _dwg_object_object *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Object_is_xdic_missing_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_xdic_missing = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Object_is_xdic_missing_get) {
  {
    struct _dwg_object_object *arg1 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Object_is_xdic_missing_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Object_is_xdic_missing_get" "', argument " "1"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg1 = (struct _dwg_object_object *)(argp1);
    result = (BITCODE_B) ((arg1)->is_xdic_missing);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Object_has_ds_data_set) {
  {
    struct _dwg_object_object *arg1 = (struct _dwg_object_object *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Object_has_ds_data_set(self,has_ds_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Object_has_ds_data_set" "', argument " "1"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg1 = (struct _dwg_object_object *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_Object_has_ds_data_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_ds_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Object_has_ds_data_get) {
  {
    struct _dwg_object_object *arg1 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Object_has_ds_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Object_has_ds_data_get" "', argument " "1"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg1 = (struct _dwg_object_object *)(argp1);
    result = (BITCODE_B) ((arg1)->has_ds_data);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Object_handleref_set) {
  {
    struct _dwg_object_object *arg1 = (struct _dwg_object_object *) 0 ;
    Dwg_Handle *arg2 = (Dwg_Handle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_Object_handleref_set(self,handleref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Object_handleref_set" "', argument " "1"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg1 = (struct _dwg_object_object *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_handle, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_Object_handleref_set" "', argument " "2"" of type '" "Dwg_Handle *""'"); 
    }
    arg2 = (Dwg_Handle *)(argp2);
    if (arg1) (arg1)->handleref = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_Object_handleref_get) {
  {
    struct _dwg_object_object *arg1 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Handle *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_Object_handleref_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_Object_handleref_get" "', argument " "1"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg1 = (struct _dwg_object_object *)(argp1);
    result = (Dwg_Handle *) ((arg1)->handleref);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_handle, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object_Object) {
  {
    int argvi = 0;
    struct _dwg_object_object *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object_Object();");
    }
    result = (struct _dwg_object_object *)calloc(1, sizeof(struct _dwg_object_object));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object_Object) {
  {
    struct _dwg_object_object *arg1 = (struct _dwg_object_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object_Object(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object_Object" "', argument " "1"" of type '" "struct _dwg_object_object *""'"); 
    }
    arg1 = (struct _dwg_object_object *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_APPID_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_APPID *arg2 = (Dwg_Object_APPID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_APPID_set(self,APPID);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_APPID_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_APPID, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_APPID_set" "', argument " "2"" of type '" "Dwg_Object_APPID *""'"); 
    }
    arg2 = (Dwg_Object_APPID *)(argp2);
    if (arg1) (arg1)->APPID = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_APPID_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_APPID *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_APPID_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_APPID_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_APPID *) ((arg1)->APPID);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_APPID, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_APPID_CONTROL_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_APPID_CONTROL *arg2 = (Dwg_Object_APPID_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_APPID_CONTROL_set(self,APPID_CONTROL);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_APPID_CONTROL_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_APPID_CONTROL, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_APPID_CONTROL_set" "', argument " "2"" of type '" "Dwg_Object_APPID_CONTROL *""'"); 
    }
    arg2 = (Dwg_Object_APPID_CONTROL *)(argp2);
    if (arg1) (arg1)->APPID_CONTROL = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_APPID_CONTROL_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_APPID_CONTROL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_APPID_CONTROL_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_APPID_CONTROL_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_APPID_CONTROL *) ((arg1)->APPID_CONTROL);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_APPID_CONTROL, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCK_CONTROL_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCK_CONTROL *arg2 = (Dwg_Object_BLOCK_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCK_CONTROL_set(self,BLOCK_CONTROL);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCK_CONTROL_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCK_CONTROL, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCK_CONTROL_set" "', argument " "2"" of type '" "Dwg_Object_BLOCK_CONTROL *""'"); 
    }
    arg2 = (Dwg_Object_BLOCK_CONTROL *)(argp2);
    if (arg1) (arg1)->BLOCK_CONTROL = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCK_CONTROL_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCK_CONTROL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCK_CONTROL_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCK_CONTROL_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCK_CONTROL *) ((arg1)->BLOCK_CONTROL);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCK_CONTROL, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCK_HEADER_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCK_HEADER *arg2 = (Dwg_Object_BLOCK_HEADER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCK_HEADER_set(self,BLOCK_HEADER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCK_HEADER_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCK_HEADER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCK_HEADER_set" "', argument " "2"" of type '" "Dwg_Object_BLOCK_HEADER *""'"); 
    }
    arg2 = (Dwg_Object_BLOCK_HEADER *)(argp2);
    if (arg1) (arg1)->BLOCK_HEADER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCK_HEADER_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCK_HEADER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCK_HEADER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCK_HEADER_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCK_HEADER *) ((arg1)->BLOCK_HEADER);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_DICTIONARY_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_DICTIONARY *arg2 = (Dwg_Object_DICTIONARY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_DICTIONARY_set(self,DICTIONARY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_DICTIONARY_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_DICTIONARY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_DICTIONARY_set" "', argument " "2"" of type '" "Dwg_Object_DICTIONARY *""'"); 
    }
    arg2 = (Dwg_Object_DICTIONARY *)(argp2);
    if (arg1) (arg1)->DICTIONARY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_DICTIONARY_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DICTIONARY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_DICTIONARY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_DICTIONARY_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_DICTIONARY *) ((arg1)->DICTIONARY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DICTIONARY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_DIMSTYLE_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_DIMSTYLE *arg2 = (Dwg_Object_DIMSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_DIMSTYLE_set(self,DIMSTYLE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_DIMSTYLE_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_DIMSTYLE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_DIMSTYLE_set" "', argument " "2"" of type '" "Dwg_Object_DIMSTYLE *""'"); 
    }
    arg2 = (Dwg_Object_DIMSTYLE *)(argp2);
    if (arg1) (arg1)->DIMSTYLE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_DIMSTYLE_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DIMSTYLE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_DIMSTYLE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_DIMSTYLE_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_DIMSTYLE *) ((arg1)->DIMSTYLE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DIMSTYLE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_DIMSTYLE_CONTROL_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_DIMSTYLE_CONTROL *arg2 = (Dwg_Object_DIMSTYLE_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_DIMSTYLE_CONTROL_set(self,DIMSTYLE_CONTROL);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_DIMSTYLE_CONTROL_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_DIMSTYLE_CONTROL, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_DIMSTYLE_CONTROL_set" "', argument " "2"" of type '" "Dwg_Object_DIMSTYLE_CONTROL *""'"); 
    }
    arg2 = (Dwg_Object_DIMSTYLE_CONTROL *)(argp2);
    if (arg1) (arg1)->DIMSTYLE_CONTROL = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_DIMSTYLE_CONTROL_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DIMSTYLE_CONTROL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_DIMSTYLE_CONTROL_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_DIMSTYLE_CONTROL_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_DIMSTYLE_CONTROL *) ((arg1)->DIMSTYLE_CONTROL);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DIMSTYLE_CONTROL, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_DUMMY_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_DUMMY *arg2 = (Dwg_Object_DUMMY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_DUMMY_set(self,DUMMY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_DUMMY_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_DUMMY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_DUMMY_set" "', argument " "2"" of type '" "Dwg_Object_DUMMY *""'"); 
    }
    arg2 = (Dwg_Object_DUMMY *)(argp2);
    if (arg1) (arg1)->DUMMY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_DUMMY_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DUMMY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_DUMMY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_DUMMY_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_DUMMY *) ((arg1)->DUMMY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DUMMY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_LAYER_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_LAYER *arg2 = (Dwg_Object_LAYER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_LAYER_set(self,LAYER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_LAYER_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_LAYER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_LAYER_set" "', argument " "2"" of type '" "Dwg_Object_LAYER *""'"); 
    }
    arg2 = (Dwg_Object_LAYER *)(argp2);
    if (arg1) (arg1)->LAYER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_LAYER_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_LAYER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_LAYER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_LAYER_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_LAYER *) ((arg1)->LAYER);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_LAYER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_LAYER_CONTROL_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_LAYER_CONTROL *arg2 = (Dwg_Object_LAYER_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_LAYER_CONTROL_set(self,LAYER_CONTROL);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_LAYER_CONTROL_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_LAYER_CONTROL, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_LAYER_CONTROL_set" "', argument " "2"" of type '" "Dwg_Object_LAYER_CONTROL *""'"); 
    }
    arg2 = (Dwg_Object_LAYER_CONTROL *)(argp2);
    if (arg1) (arg1)->LAYER_CONTROL = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_LAYER_CONTROL_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_LAYER_CONTROL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_LAYER_CONTROL_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_LAYER_CONTROL_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_LAYER_CONTROL *) ((arg1)->LAYER_CONTROL);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_LAYER_CONTROL, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_LONG_TRANSACTION_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_LONG_TRANSACTION *arg2 = (Dwg_Object_LONG_TRANSACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_LONG_TRANSACTION_set(self,LONG_TRANSACTION);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_LONG_TRANSACTION_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_LONG_TRANSACTION, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_LONG_TRANSACTION_set" "', argument " "2"" of type '" "Dwg_Object_LONG_TRANSACTION *""'"); 
    }
    arg2 = (Dwg_Object_LONG_TRANSACTION *)(argp2);
    if (arg1) (arg1)->LONG_TRANSACTION = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_LONG_TRANSACTION_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_LONG_TRANSACTION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_LONG_TRANSACTION_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_LONG_TRANSACTION_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_LONG_TRANSACTION *) ((arg1)->LONG_TRANSACTION);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_LONG_TRANSACTION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_LTYPE_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_LTYPE *arg2 = (Dwg_Object_LTYPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_LTYPE_set(self,LTYPE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_LTYPE_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_LTYPE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_LTYPE_set" "', argument " "2"" of type '" "Dwg_Object_LTYPE *""'"); 
    }
    arg2 = (Dwg_Object_LTYPE *)(argp2);
    if (arg1) (arg1)->LTYPE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_LTYPE_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_LTYPE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_LTYPE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_LTYPE_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_LTYPE *) ((arg1)->LTYPE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_LTYPE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_LTYPE_CONTROL_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_LTYPE_CONTROL *arg2 = (Dwg_Object_LTYPE_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_LTYPE_CONTROL_set(self,LTYPE_CONTROL);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_LTYPE_CONTROL_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_LTYPE_CONTROL, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_LTYPE_CONTROL_set" "', argument " "2"" of type '" "Dwg_Object_LTYPE_CONTROL *""'"); 
    }
    arg2 = (Dwg_Object_LTYPE_CONTROL *)(argp2);
    if (arg1) (arg1)->LTYPE_CONTROL = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_LTYPE_CONTROL_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_LTYPE_CONTROL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_LTYPE_CONTROL_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_LTYPE_CONTROL_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_LTYPE_CONTROL *) ((arg1)->LTYPE_CONTROL);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_LTYPE_CONTROL, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_MLINESTYLE_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_MLINESTYLE *arg2 = (Dwg_Object_MLINESTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_MLINESTYLE_set(self,MLINESTYLE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_MLINESTYLE_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_MLINESTYLE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_MLINESTYLE_set" "', argument " "2"" of type '" "Dwg_Object_MLINESTYLE *""'"); 
    }
    arg2 = (Dwg_Object_MLINESTYLE *)(argp2);
    if (arg1) (arg1)->MLINESTYLE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_MLINESTYLE_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_MLINESTYLE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_MLINESTYLE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_MLINESTYLE_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_MLINESTYLE *) ((arg1)->MLINESTYLE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_MLINESTYLE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_STYLE_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_STYLE *arg2 = (Dwg_Object_STYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_STYLE_set(self,STYLE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_STYLE_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_STYLE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_STYLE_set" "', argument " "2"" of type '" "Dwg_Object_STYLE *""'"); 
    }
    arg2 = (Dwg_Object_STYLE *)(argp2);
    if (arg1) (arg1)->STYLE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_STYLE_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_STYLE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_STYLE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_STYLE_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_STYLE *) ((arg1)->STYLE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_STYLE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_STYLE_CONTROL_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_STYLE_CONTROL *arg2 = (Dwg_Object_STYLE_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_STYLE_CONTROL_set(self,STYLE_CONTROL);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_STYLE_CONTROL_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_STYLE_CONTROL, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_STYLE_CONTROL_set" "', argument " "2"" of type '" "Dwg_Object_STYLE_CONTROL *""'"); 
    }
    arg2 = (Dwg_Object_STYLE_CONTROL *)(argp2);
    if (arg1) (arg1)->STYLE_CONTROL = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_STYLE_CONTROL_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_STYLE_CONTROL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_STYLE_CONTROL_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_STYLE_CONTROL_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_STYLE_CONTROL *) ((arg1)->STYLE_CONTROL);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_STYLE_CONTROL, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_UCS_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_UCS *arg2 = (Dwg_Object_UCS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_UCS_set(self,UCS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_UCS_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_UCS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_UCS_set" "', argument " "2"" of type '" "Dwg_Object_UCS *""'"); 
    }
    arg2 = (Dwg_Object_UCS *)(argp2);
    if (arg1) (arg1)->UCS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_UCS_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_UCS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_UCS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_UCS_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_UCS *) ((arg1)->UCS);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_UCS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_UCS_CONTROL_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_UCS_CONTROL *arg2 = (Dwg_Object_UCS_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_UCS_CONTROL_set(self,UCS_CONTROL);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_UCS_CONTROL_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_UCS_CONTROL, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_UCS_CONTROL_set" "', argument " "2"" of type '" "Dwg_Object_UCS_CONTROL *""'"); 
    }
    arg2 = (Dwg_Object_UCS_CONTROL *)(argp2);
    if (arg1) (arg1)->UCS_CONTROL = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_UCS_CONTROL_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_UCS_CONTROL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_UCS_CONTROL_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_UCS_CONTROL_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_UCS_CONTROL *) ((arg1)->UCS_CONTROL);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_UCS_CONTROL, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_UNKNOWN_OBJ_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_UNKNOWN_OBJ *arg2 = (Dwg_Object_UNKNOWN_OBJ *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_UNKNOWN_OBJ_set(self,UNKNOWN_OBJ);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_UNKNOWN_OBJ_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_UNKNOWN_OBJ, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_UNKNOWN_OBJ_set" "', argument " "2"" of type '" "Dwg_Object_UNKNOWN_OBJ *""'"); 
    }
    arg2 = (Dwg_Object_UNKNOWN_OBJ *)(argp2);
    if (arg1) (arg1)->UNKNOWN_OBJ = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_UNKNOWN_OBJ_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_UNKNOWN_OBJ *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_UNKNOWN_OBJ_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_UNKNOWN_OBJ_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_UNKNOWN_OBJ *) ((arg1)->UNKNOWN_OBJ);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_UNKNOWN_OBJ, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_VIEW_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_VIEW *arg2 = (Dwg_Object_VIEW *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_VIEW_set(self,VIEW);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_VIEW_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_VIEW, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_VIEW_set" "', argument " "2"" of type '" "Dwg_Object_VIEW *""'"); 
    }
    arg2 = (Dwg_Object_VIEW *)(argp2);
    if (arg1) (arg1)->VIEW = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_VIEW_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_VIEW *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_VIEW_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_VIEW_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_VIEW *) ((arg1)->VIEW);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_VIEW, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_VIEW_CONTROL_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_VIEW_CONTROL *arg2 = (Dwg_Object_VIEW_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_VIEW_CONTROL_set(self,VIEW_CONTROL);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_VIEW_CONTROL_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_VIEW_CONTROL, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_VIEW_CONTROL_set" "', argument " "2"" of type '" "Dwg_Object_VIEW_CONTROL *""'"); 
    }
    arg2 = (Dwg_Object_VIEW_CONTROL *)(argp2);
    if (arg1) (arg1)->VIEW_CONTROL = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_VIEW_CONTROL_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_VIEW_CONTROL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_VIEW_CONTROL_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_VIEW_CONTROL_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_VIEW_CONTROL *) ((arg1)->VIEW_CONTROL);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_VIEW_CONTROL, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_VPORT_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_VPORT *arg2 = (Dwg_Object_VPORT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_VPORT_set(self,VPORT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_VPORT_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_VPORT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_VPORT_set" "', argument " "2"" of type '" "Dwg_Object_VPORT *""'"); 
    }
    arg2 = (Dwg_Object_VPORT *)(argp2);
    if (arg1) (arg1)->VPORT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_VPORT_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_VPORT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_VPORT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_VPORT_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_VPORT *) ((arg1)->VPORT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_VPORT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_VPORT_CONTROL_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_VPORT_CONTROL *arg2 = (Dwg_Object_VPORT_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_VPORT_CONTROL_set(self,VPORT_CONTROL);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_VPORT_CONTROL_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_VPORT_CONTROL, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_VPORT_CONTROL_set" "', argument " "2"" of type '" "Dwg_Object_VPORT_CONTROL *""'"); 
    }
    arg2 = (Dwg_Object_VPORT_CONTROL *)(argp2);
    if (arg1) (arg1)->VPORT_CONTROL = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_VPORT_CONTROL_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_VPORT_CONTROL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_VPORT_CONTROL_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_VPORT_CONTROL_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_VPORT_CONTROL *) ((arg1)->VPORT_CONTROL);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_VPORT_CONTROL, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_VX_CONTROL_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_VX_CONTROL *arg2 = (Dwg_Object_VX_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_VX_CONTROL_set(self,VX_CONTROL);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_VX_CONTROL_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_VX_CONTROL, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_VX_CONTROL_set" "', argument " "2"" of type '" "Dwg_Object_VX_CONTROL *""'"); 
    }
    arg2 = (Dwg_Object_VX_CONTROL *)(argp2);
    if (arg1) (arg1)->VX_CONTROL = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_VX_CONTROL_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_VX_CONTROL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_VX_CONTROL_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_VX_CONTROL_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_VX_CONTROL *) ((arg1)->VX_CONTROL);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_VX_CONTROL, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_VX_TABLE_RECORD_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_VX_TABLE_RECORD *arg2 = (Dwg_Object_VX_TABLE_RECORD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_VX_TABLE_RECORD_set(self,VX_TABLE_RECORD);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_VX_TABLE_RECORD_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_VX_TABLE_RECORD, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_VX_TABLE_RECORD_set" "', argument " "2"" of type '" "Dwg_Object_VX_TABLE_RECORD *""'"); 
    }
    arg2 = (Dwg_Object_VX_TABLE_RECORD *)(argp2);
    if (arg1) (arg1)->VX_TABLE_RECORD = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_VX_TABLE_RECORD_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_VX_TABLE_RECORD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_VX_TABLE_RECORD_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_VX_TABLE_RECORD_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_VX_TABLE_RECORD *) ((arg1)->VX_TABLE_RECORD);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_VX_TABLE_RECORD, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACSH_BOOLEAN_CLASS_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ACSH_BOOLEAN_CLASS *arg2 = (Dwg_Object_ACSH_BOOLEAN_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACSH_BOOLEAN_CLASS_set(self,ACSH_BOOLEAN_CLASS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACSH_BOOLEAN_CLASS_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ACSH_BOOLEAN_CLASS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ACSH_BOOLEAN_CLASS_set" "', argument " "2"" of type '" "Dwg_Object_ACSH_BOOLEAN_CLASS *""'"); 
    }
    arg2 = (Dwg_Object_ACSH_BOOLEAN_CLASS *)(argp2);
    if (arg1) (arg1)->ACSH_BOOLEAN_CLASS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACSH_BOOLEAN_CLASS_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_BOOLEAN_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACSH_BOOLEAN_CLASS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACSH_BOOLEAN_CLASS_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ACSH_BOOLEAN_CLASS *) ((arg1)->ACSH_BOOLEAN_CLASS);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_BOOLEAN_CLASS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACSH_BOX_CLASS_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ACSH_BOX_CLASS *arg2 = (Dwg_Object_ACSH_BOX_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACSH_BOX_CLASS_set(self,ACSH_BOX_CLASS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACSH_BOX_CLASS_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ACSH_BOX_CLASS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ACSH_BOX_CLASS_set" "', argument " "2"" of type '" "Dwg_Object_ACSH_BOX_CLASS *""'"); 
    }
    arg2 = (Dwg_Object_ACSH_BOX_CLASS *)(argp2);
    if (arg1) (arg1)->ACSH_BOX_CLASS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACSH_BOX_CLASS_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_BOX_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACSH_BOX_CLASS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACSH_BOX_CLASS_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ACSH_BOX_CLASS *) ((arg1)->ACSH_BOX_CLASS);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_BOX_CLASS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACSH_CONE_CLASS_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ACSH_CONE_CLASS *arg2 = (Dwg_Object_ACSH_CONE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACSH_CONE_CLASS_set(self,ACSH_CONE_CLASS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACSH_CONE_CLASS_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ACSH_CONE_CLASS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ACSH_CONE_CLASS_set" "', argument " "2"" of type '" "Dwg_Object_ACSH_CONE_CLASS *""'"); 
    }
    arg2 = (Dwg_Object_ACSH_CONE_CLASS *)(argp2);
    if (arg1) (arg1)->ACSH_CONE_CLASS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACSH_CONE_CLASS_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_CONE_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACSH_CONE_CLASS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACSH_CONE_CLASS_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ACSH_CONE_CLASS *) ((arg1)->ACSH_CONE_CLASS);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_CONE_CLASS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACSH_CYLINDER_CLASS_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ACSH_CYLINDER_CLASS *arg2 = (Dwg_Object_ACSH_CYLINDER_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACSH_CYLINDER_CLASS_set(self,ACSH_CYLINDER_CLASS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACSH_CYLINDER_CLASS_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ACSH_CYLINDER_CLASS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ACSH_CYLINDER_CLASS_set" "', argument " "2"" of type '" "Dwg_Object_ACSH_CYLINDER_CLASS *""'"); 
    }
    arg2 = (Dwg_Object_ACSH_CYLINDER_CLASS *)(argp2);
    if (arg1) (arg1)->ACSH_CYLINDER_CLASS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACSH_CYLINDER_CLASS_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_CYLINDER_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACSH_CYLINDER_CLASS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACSH_CYLINDER_CLASS_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ACSH_CYLINDER_CLASS *) ((arg1)->ACSH_CYLINDER_CLASS);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_CYLINDER_CLASS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACSH_FILLET_CLASS_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ACSH_FILLET_CLASS *arg2 = (Dwg_Object_ACSH_FILLET_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACSH_FILLET_CLASS_set(self,ACSH_FILLET_CLASS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACSH_FILLET_CLASS_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ACSH_FILLET_CLASS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ACSH_FILLET_CLASS_set" "', argument " "2"" of type '" "Dwg_Object_ACSH_FILLET_CLASS *""'"); 
    }
    arg2 = (Dwg_Object_ACSH_FILLET_CLASS *)(argp2);
    if (arg1) (arg1)->ACSH_FILLET_CLASS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACSH_FILLET_CLASS_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_FILLET_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACSH_FILLET_CLASS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACSH_FILLET_CLASS_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ACSH_FILLET_CLASS *) ((arg1)->ACSH_FILLET_CLASS);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_FILLET_CLASS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACSH_HISTORY_CLASS_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ACSH_HISTORY_CLASS *arg2 = (Dwg_Object_ACSH_HISTORY_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACSH_HISTORY_CLASS_set(self,ACSH_HISTORY_CLASS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACSH_HISTORY_CLASS_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ACSH_HISTORY_CLASS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ACSH_HISTORY_CLASS_set" "', argument " "2"" of type '" "Dwg_Object_ACSH_HISTORY_CLASS *""'"); 
    }
    arg2 = (Dwg_Object_ACSH_HISTORY_CLASS *)(argp2);
    if (arg1) (arg1)->ACSH_HISTORY_CLASS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACSH_HISTORY_CLASS_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_HISTORY_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACSH_HISTORY_CLASS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACSH_HISTORY_CLASS_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ACSH_HISTORY_CLASS *) ((arg1)->ACSH_HISTORY_CLASS);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_HISTORY_CLASS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACSH_SPHERE_CLASS_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ACSH_SPHERE_CLASS *arg2 = (Dwg_Object_ACSH_SPHERE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACSH_SPHERE_CLASS_set(self,ACSH_SPHERE_CLASS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACSH_SPHERE_CLASS_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ACSH_SPHERE_CLASS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ACSH_SPHERE_CLASS_set" "', argument " "2"" of type '" "Dwg_Object_ACSH_SPHERE_CLASS *""'"); 
    }
    arg2 = (Dwg_Object_ACSH_SPHERE_CLASS *)(argp2);
    if (arg1) (arg1)->ACSH_SPHERE_CLASS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACSH_SPHERE_CLASS_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_SPHERE_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACSH_SPHERE_CLASS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACSH_SPHERE_CLASS_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ACSH_SPHERE_CLASS *) ((arg1)->ACSH_SPHERE_CLASS);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_SPHERE_CLASS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACSH_TORUS_CLASS_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ACSH_TORUS_CLASS *arg2 = (Dwg_Object_ACSH_TORUS_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACSH_TORUS_CLASS_set(self,ACSH_TORUS_CLASS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACSH_TORUS_CLASS_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ACSH_TORUS_CLASS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ACSH_TORUS_CLASS_set" "', argument " "2"" of type '" "Dwg_Object_ACSH_TORUS_CLASS *""'"); 
    }
    arg2 = (Dwg_Object_ACSH_TORUS_CLASS *)(argp2);
    if (arg1) (arg1)->ACSH_TORUS_CLASS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACSH_TORUS_CLASS_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_TORUS_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACSH_TORUS_CLASS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACSH_TORUS_CLASS_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ACSH_TORUS_CLASS *) ((arg1)->ACSH_TORUS_CLASS);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_TORUS_CLASS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACSH_WEDGE_CLASS_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ACSH_WEDGE_CLASS *arg2 = (Dwg_Object_ACSH_WEDGE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACSH_WEDGE_CLASS_set(self,ACSH_WEDGE_CLASS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACSH_WEDGE_CLASS_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ACSH_WEDGE_CLASS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ACSH_WEDGE_CLASS_set" "', argument " "2"" of type '" "Dwg_Object_ACSH_WEDGE_CLASS *""'"); 
    }
    arg2 = (Dwg_Object_ACSH_WEDGE_CLASS *)(argp2);
    if (arg1) (arg1)->ACSH_WEDGE_CLASS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACSH_WEDGE_CLASS_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_WEDGE_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACSH_WEDGE_CLASS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACSH_WEDGE_CLASS_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ACSH_WEDGE_CLASS *) ((arg1)->ACSH_WEDGE_CLASS);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_WEDGE_CLASS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCGEOMDEPENDENCY_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCGEOMDEPENDENCY *arg2 = (Dwg_Object_ASSOCGEOMDEPENDENCY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCGEOMDEPENDENCY_set(self,ASSOCGEOMDEPENDENCY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCGEOMDEPENDENCY_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCGEOMDEPENDENCY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCGEOMDEPENDENCY_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCGEOMDEPENDENCY *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCGEOMDEPENDENCY *)(argp2);
    if (arg1) (arg1)->ASSOCGEOMDEPENDENCY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCGEOMDEPENDENCY_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCGEOMDEPENDENCY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCGEOMDEPENDENCY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCGEOMDEPENDENCY_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCGEOMDEPENDENCY *) ((arg1)->ASSOCGEOMDEPENDENCY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCGEOMDEPENDENCY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCNETWORK_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCNETWORK *arg2 = (Dwg_Object_ASSOCNETWORK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCNETWORK_set(self,ASSOCNETWORK);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCNETWORK_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCNETWORK, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCNETWORK_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCNETWORK *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCNETWORK *)(argp2);
    if (arg1) (arg1)->ASSOCNETWORK = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCNETWORK_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCNETWORK *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCNETWORK_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCNETWORK_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCNETWORK *) ((arg1)->ASSOCNETWORK);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKALIGNMENTGRIP_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKALIGNMENTGRIP *arg2 = (Dwg_Object_BLOCKALIGNMENTGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKALIGNMENTGRIP_set(self,BLOCKALIGNMENTGRIP);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKALIGNMENTGRIP_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTGRIP, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKALIGNMENTGRIP_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKALIGNMENTGRIP *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKALIGNMENTGRIP *)(argp2);
    if (arg1) (arg1)->BLOCKALIGNMENTGRIP = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKALIGNMENTGRIP_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKALIGNMENTGRIP *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKALIGNMENTGRIP_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKALIGNMENTGRIP_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKALIGNMENTGRIP *) ((arg1)->BLOCKALIGNMENTGRIP);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKALIGNMENTGRIP, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKALIGNMENTPARAMETER_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKALIGNMENTPARAMETER *arg2 = (Dwg_Object_BLOCKALIGNMENTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKALIGNMENTPARAMETER_set(self,BLOCKALIGNMENTPARAMETER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKALIGNMENTPARAMETER_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKALIGNMENTPARAMETER_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKALIGNMENTPARAMETER *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKALIGNMENTPARAMETER *)(argp2);
    if (arg1) (arg1)->BLOCKALIGNMENTPARAMETER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKALIGNMENTPARAMETER_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKALIGNMENTPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKALIGNMENTPARAMETER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKALIGNMENTPARAMETER_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKALIGNMENTPARAMETER *) ((arg1)->BLOCKALIGNMENTPARAMETER);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKBASEPOINTPARAMETER_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKBASEPOINTPARAMETER *arg2 = (Dwg_Object_BLOCKBASEPOINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKBASEPOINTPARAMETER_set(self,BLOCKBASEPOINTPARAMETER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKBASEPOINTPARAMETER_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKBASEPOINTPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKBASEPOINTPARAMETER_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKBASEPOINTPARAMETER *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKBASEPOINTPARAMETER *)(argp2);
    if (arg1) (arg1)->BLOCKBASEPOINTPARAMETER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKBASEPOINTPARAMETER_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKBASEPOINTPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKBASEPOINTPARAMETER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKBASEPOINTPARAMETER_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKBASEPOINTPARAMETER *) ((arg1)->BLOCKBASEPOINTPARAMETER);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKBASEPOINTPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKFLIPACTION_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKFLIPACTION *arg2 = (Dwg_Object_BLOCKFLIPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKFLIPACTION_set(self,BLOCKFLIPACTION);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKFLIPACTION_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKFLIPACTION_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKFLIPACTION *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKFLIPACTION *)(argp2);
    if (arg1) (arg1)->BLOCKFLIPACTION = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKFLIPACTION_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKFLIPACTION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKFLIPACTION_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKFLIPACTION_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKFLIPACTION *) ((arg1)->BLOCKFLIPACTION);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKFLIPGRIP_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKFLIPGRIP *arg2 = (Dwg_Object_BLOCKFLIPGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKFLIPGRIP_set(self,BLOCKFLIPGRIP);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKFLIPGRIP_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKFLIPGRIP_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKFLIPGRIP *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKFLIPGRIP *)(argp2);
    if (arg1) (arg1)->BLOCKFLIPGRIP = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKFLIPGRIP_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKFLIPGRIP *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKFLIPGRIP_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKFLIPGRIP_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKFLIPGRIP *) ((arg1)->BLOCKFLIPGRIP);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKFLIPPARAMETER_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKFLIPPARAMETER *arg2 = (Dwg_Object_BLOCKFLIPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKFLIPPARAMETER_set(self,BLOCKFLIPPARAMETER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKFLIPPARAMETER_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKFLIPPARAMETER_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKFLIPPARAMETER *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKFLIPPARAMETER *)(argp2);
    if (arg1) (arg1)->BLOCKFLIPPARAMETER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKFLIPPARAMETER_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKFLIPPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKFLIPPARAMETER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKFLIPPARAMETER_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKFLIPPARAMETER *) ((arg1)->BLOCKFLIPPARAMETER);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKGRIPLOCATIONCOMPONENT_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT *arg2 = (Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKGRIPLOCATIONCOMPONENT_set(self,BLOCKGRIPLOCATIONCOMPONENT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKGRIPLOCATIONCOMPONENT_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKGRIPLOCATIONCOMPONENT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKGRIPLOCATIONCOMPONENT_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT *)(argp2);
    if (arg1) (arg1)->BLOCKGRIPLOCATIONCOMPONENT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKGRIPLOCATIONCOMPONENT_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKGRIPLOCATIONCOMPONENT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKGRIPLOCATIONCOMPONENT_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT *) ((arg1)->BLOCKGRIPLOCATIONCOMPONENT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKGRIPLOCATIONCOMPONENT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKLINEARGRIP_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKLINEARGRIP *arg2 = (Dwg_Object_BLOCKLINEARGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKLINEARGRIP_set(self,BLOCKLINEARGRIP);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKLINEARGRIP_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKLINEARGRIP, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKLINEARGRIP_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKLINEARGRIP *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKLINEARGRIP *)(argp2);
    if (arg1) (arg1)->BLOCKLINEARGRIP = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKLINEARGRIP_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKLINEARGRIP *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKLINEARGRIP_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKLINEARGRIP_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKLINEARGRIP *) ((arg1)->BLOCKLINEARGRIP);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKLINEARGRIP, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKLOOKUPGRIP_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKLOOKUPGRIP *arg2 = (Dwg_Object_BLOCKLOOKUPGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKLOOKUPGRIP_set(self,BLOCKLOOKUPGRIP);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKLOOKUPGRIP_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKLOOKUPGRIP, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKLOOKUPGRIP_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKLOOKUPGRIP *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKLOOKUPGRIP *)(argp2);
    if (arg1) (arg1)->BLOCKLOOKUPGRIP = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKLOOKUPGRIP_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKLOOKUPGRIP *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKLOOKUPGRIP_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKLOOKUPGRIP_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKLOOKUPGRIP *) ((arg1)->BLOCKLOOKUPGRIP);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKLOOKUPGRIP, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKMOVEACTION_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKMOVEACTION *arg2 = (Dwg_Object_BLOCKMOVEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKMOVEACTION_set(self,BLOCKMOVEACTION);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKMOVEACTION_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKMOVEACTION_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKMOVEACTION *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKMOVEACTION *)(argp2);
    if (arg1) (arg1)->BLOCKMOVEACTION = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKMOVEACTION_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKMOVEACTION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKMOVEACTION_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKMOVEACTION_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKMOVEACTION *) ((arg1)->BLOCKMOVEACTION);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKROTATEACTION_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKROTATEACTION *arg2 = (Dwg_Object_BLOCKROTATEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKROTATEACTION_set(self,BLOCKROTATEACTION);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKROTATEACTION_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKROTATEACTION_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKROTATEACTION *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKROTATEACTION *)(argp2);
    if (arg1) (arg1)->BLOCKROTATEACTION = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKROTATEACTION_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKROTATEACTION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKROTATEACTION_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKROTATEACTION_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKROTATEACTION *) ((arg1)->BLOCKROTATEACTION);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKROTATIONGRIP_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKROTATIONGRIP *arg2 = (Dwg_Object_BLOCKROTATIONGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKROTATIONGRIP_set(self,BLOCKROTATIONGRIP);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKROTATIONGRIP_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKROTATIONGRIP, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKROTATIONGRIP_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKROTATIONGRIP *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKROTATIONGRIP *)(argp2);
    if (arg1) (arg1)->BLOCKROTATIONGRIP = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKROTATIONGRIP_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKROTATIONGRIP *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKROTATIONGRIP_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKROTATIONGRIP_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKROTATIONGRIP *) ((arg1)->BLOCKROTATIONGRIP);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKROTATIONGRIP, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKSCALEACTION_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKSCALEACTION *arg2 = (Dwg_Object_BLOCKSCALEACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKSCALEACTION_set(self,BLOCKSCALEACTION);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKSCALEACTION_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKSCALEACTION_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKSCALEACTION *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKSCALEACTION *)(argp2);
    if (arg1) (arg1)->BLOCKSCALEACTION = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKSCALEACTION_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKSCALEACTION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKSCALEACTION_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKSCALEACTION_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKSCALEACTION *) ((arg1)->BLOCKSCALEACTION);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKVISIBILITYGRIP_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKVISIBILITYGRIP *arg2 = (Dwg_Object_BLOCKVISIBILITYGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKVISIBILITYGRIP_set(self,BLOCKVISIBILITYGRIP);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKVISIBILITYGRIP_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYGRIP, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKVISIBILITYGRIP_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKVISIBILITYGRIP *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKVISIBILITYGRIP *)(argp2);
    if (arg1) (arg1)->BLOCKVISIBILITYGRIP = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKVISIBILITYGRIP_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKVISIBILITYGRIP *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKVISIBILITYGRIP_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKVISIBILITYGRIP_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKVISIBILITYGRIP *) ((arg1)->BLOCKVISIBILITYGRIP);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKVISIBILITYGRIP, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_CELLSTYLEMAP_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_CELLSTYLEMAP *arg2 = (Dwg_Object_CELLSTYLEMAP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_CELLSTYLEMAP_set(self,CELLSTYLEMAP);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_CELLSTYLEMAP_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_CELLSTYLEMAP, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_CELLSTYLEMAP_set" "', argument " "2"" of type '" "Dwg_Object_CELLSTYLEMAP *""'"); 
    }
    arg2 = (Dwg_Object_CELLSTYLEMAP *)(argp2);
    if (arg1) (arg1)->CELLSTYLEMAP = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_CELLSTYLEMAP_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_CELLSTYLEMAP *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_CELLSTYLEMAP_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_CELLSTYLEMAP_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_CELLSTYLEMAP *) ((arg1)->CELLSTYLEMAP);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_CELLSTYLEMAP, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_DETAILVIEWSTYLE_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_DETAILVIEWSTYLE *arg2 = (Dwg_Object_DETAILVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_DETAILVIEWSTYLE_set(self,DETAILVIEWSTYLE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_DETAILVIEWSTYLE_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_DETAILVIEWSTYLE_set" "', argument " "2"" of type '" "Dwg_Object_DETAILVIEWSTYLE *""'"); 
    }
    arg2 = (Dwg_Object_DETAILVIEWSTYLE *)(argp2);
    if (arg1) (arg1)->DETAILVIEWSTYLE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_DETAILVIEWSTYLE_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DETAILVIEWSTYLE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_DETAILVIEWSTYLE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_DETAILVIEWSTYLE_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_DETAILVIEWSTYLE *) ((arg1)->DETAILVIEWSTYLE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_DICTIONARYVAR_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_DICTIONARYVAR *arg2 = (Dwg_Object_DICTIONARYVAR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_DICTIONARYVAR_set(self,DICTIONARYVAR);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_DICTIONARYVAR_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_DICTIONARYVAR, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_DICTIONARYVAR_set" "', argument " "2"" of type '" "Dwg_Object_DICTIONARYVAR *""'"); 
    }
    arg2 = (Dwg_Object_DICTIONARYVAR *)(argp2);
    if (arg1) (arg1)->DICTIONARYVAR = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_DICTIONARYVAR_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DICTIONARYVAR *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_DICTIONARYVAR_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_DICTIONARYVAR_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_DICTIONARYVAR *) ((arg1)->DICTIONARYVAR);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DICTIONARYVAR, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_DICTIONARYWDFLT_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_DICTIONARYWDFLT *arg2 = (Dwg_Object_DICTIONARYWDFLT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_DICTIONARYWDFLT_set(self,DICTIONARYWDFLT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_DICTIONARYWDFLT_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_DICTIONARYWDFLT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_DICTIONARYWDFLT_set" "', argument " "2"" of type '" "Dwg_Object_DICTIONARYWDFLT *""'"); 
    }
    arg2 = (Dwg_Object_DICTIONARYWDFLT *)(argp2);
    if (arg1) (arg1)->DICTIONARYWDFLT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_DICTIONARYWDFLT_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DICTIONARYWDFLT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_DICTIONARYWDFLT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_DICTIONARYWDFLT_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_DICTIONARYWDFLT *) ((arg1)->DICTIONARYWDFLT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DICTIONARYWDFLT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_DYNAMICBLOCKPURGEPREVENTER_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_DYNAMICBLOCKPURGEPREVENTER *arg2 = (Dwg_Object_DYNAMICBLOCKPURGEPREVENTER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_DYNAMICBLOCKPURGEPREVENTER_set(self,DYNAMICBLOCKPURGEPREVENTER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_DYNAMICBLOCKPURGEPREVENTER_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_DYNAMICBLOCKPURGEPREVENTER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_DYNAMICBLOCKPURGEPREVENTER_set" "', argument " "2"" of type '" "Dwg_Object_DYNAMICBLOCKPURGEPREVENTER *""'"); 
    }
    arg2 = (Dwg_Object_DYNAMICBLOCKPURGEPREVENTER *)(argp2);
    if (arg1) (arg1)->DYNAMICBLOCKPURGEPREVENTER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_DYNAMICBLOCKPURGEPREVENTER_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DYNAMICBLOCKPURGEPREVENTER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_DYNAMICBLOCKPURGEPREVENTER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_DYNAMICBLOCKPURGEPREVENTER_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_DYNAMICBLOCKPURGEPREVENTER *) ((arg1)->DYNAMICBLOCKPURGEPREVENTER);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DYNAMICBLOCKPURGEPREVENTER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_FIELD_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_FIELD *arg2 = (Dwg_Object_FIELD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_FIELD_set(self,FIELD);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_FIELD_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_FIELD, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_FIELD_set" "', argument " "2"" of type '" "Dwg_Object_FIELD *""'"); 
    }
    arg2 = (Dwg_Object_FIELD *)(argp2);
    if (arg1) (arg1)->FIELD = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_FIELD_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_FIELD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_FIELD_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_FIELD_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_FIELD *) ((arg1)->FIELD);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_FIELD, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_FIELDLIST_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_FIELDLIST *arg2 = (Dwg_Object_FIELDLIST *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_FIELDLIST_set(self,FIELDLIST);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_FIELDLIST_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_FIELDLIST, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_FIELDLIST_set" "', argument " "2"" of type '" "Dwg_Object_FIELDLIST *""'"); 
    }
    arg2 = (Dwg_Object_FIELDLIST *)(argp2);
    if (arg1) (arg1)->FIELDLIST = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_FIELDLIST_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_FIELDLIST *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_FIELDLIST_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_FIELDLIST_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_FIELDLIST *) ((arg1)->FIELDLIST);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_FIELDLIST, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_GEODATA_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_GEODATA *arg2 = (Dwg_Object_GEODATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_GEODATA_set(self,GEODATA);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_GEODATA_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_GEODATA, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_GEODATA_set" "', argument " "2"" of type '" "Dwg_Object_GEODATA *""'"); 
    }
    arg2 = (Dwg_Object_GEODATA *)(argp2);
    if (arg1) (arg1)->GEODATA = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_GEODATA_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_GEODATA *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_GEODATA_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_GEODATA_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_GEODATA *) ((arg1)->GEODATA);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_GEODATA, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_GROUP_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_GROUP *arg2 = (Dwg_Object_GROUP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_GROUP_set(self,GROUP);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_GROUP_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_GROUP, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_GROUP_set" "', argument " "2"" of type '" "Dwg_Object_GROUP *""'"); 
    }
    arg2 = (Dwg_Object_GROUP *)(argp2);
    if (arg1) (arg1)->GROUP = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_GROUP_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_GROUP *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_GROUP_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_GROUP_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_GROUP *) ((arg1)->GROUP);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_GROUP, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_IDBUFFER_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_IDBUFFER *arg2 = (Dwg_Object_IDBUFFER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_IDBUFFER_set(self,IDBUFFER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_IDBUFFER_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_IDBUFFER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_IDBUFFER_set" "', argument " "2"" of type '" "Dwg_Object_IDBUFFER *""'"); 
    }
    arg2 = (Dwg_Object_IDBUFFER *)(argp2);
    if (arg1) (arg1)->IDBUFFER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_IDBUFFER_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_IDBUFFER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_IDBUFFER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_IDBUFFER_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_IDBUFFER *) ((arg1)->IDBUFFER);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_IDBUFFER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_IMAGEDEF_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_IMAGEDEF *arg2 = (Dwg_Object_IMAGEDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_IMAGEDEF_set(self,IMAGEDEF);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_IMAGEDEF_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_IMAGEDEF, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_IMAGEDEF_set" "', argument " "2"" of type '" "Dwg_Object_IMAGEDEF *""'"); 
    }
    arg2 = (Dwg_Object_IMAGEDEF *)(argp2);
    if (arg1) (arg1)->IMAGEDEF = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_IMAGEDEF_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_IMAGEDEF *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_IMAGEDEF_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_IMAGEDEF_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_IMAGEDEF *) ((arg1)->IMAGEDEF);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_IMAGEDEF, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_IMAGEDEF_REACTOR_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_IMAGEDEF_REACTOR *arg2 = (Dwg_Object_IMAGEDEF_REACTOR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_IMAGEDEF_REACTOR_set(self,IMAGEDEF_REACTOR);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_IMAGEDEF_REACTOR_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_IMAGEDEF_REACTOR, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_IMAGEDEF_REACTOR_set" "', argument " "2"" of type '" "Dwg_Object_IMAGEDEF_REACTOR *""'"); 
    }
    arg2 = (Dwg_Object_IMAGEDEF_REACTOR *)(argp2);
    if (arg1) (arg1)->IMAGEDEF_REACTOR = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_IMAGEDEF_REACTOR_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_IMAGEDEF_REACTOR *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_IMAGEDEF_REACTOR_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_IMAGEDEF_REACTOR_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_IMAGEDEF_REACTOR *) ((arg1)->IMAGEDEF_REACTOR);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_IMAGEDEF_REACTOR, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_INDEX_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_INDEX *arg2 = (Dwg_Object_INDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_INDEX_set(self,INDEX);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_INDEX_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_INDEX, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_INDEX_set" "', argument " "2"" of type '" "Dwg_Object_INDEX *""'"); 
    }
    arg2 = (Dwg_Object_INDEX *)(argp2);
    if (arg1) (arg1)->INDEX = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_INDEX_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_INDEX *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_INDEX_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_INDEX_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_INDEX *) ((arg1)->INDEX);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_INDEX, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_LAYERFILTER_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_LAYERFILTER *arg2 = (Dwg_Object_LAYERFILTER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_LAYERFILTER_set(self,LAYERFILTER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_LAYERFILTER_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_LAYERFILTER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_LAYERFILTER_set" "', argument " "2"" of type '" "Dwg_Object_LAYERFILTER *""'"); 
    }
    arg2 = (Dwg_Object_LAYERFILTER *)(argp2);
    if (arg1) (arg1)->LAYERFILTER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_LAYERFILTER_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_LAYERFILTER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_LAYERFILTER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_LAYERFILTER_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_LAYERFILTER *) ((arg1)->LAYERFILTER);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_LAYERFILTER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_LAYER_INDEX_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_LAYER_INDEX *arg2 = (Dwg_Object_LAYER_INDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_LAYER_INDEX_set(self,LAYER_INDEX);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_LAYER_INDEX_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_LAYER_INDEX, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_LAYER_INDEX_set" "', argument " "2"" of type '" "Dwg_Object_LAYER_INDEX *""'"); 
    }
    arg2 = (Dwg_Object_LAYER_INDEX *)(argp2);
    if (arg1) (arg1)->LAYER_INDEX = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_LAYER_INDEX_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_LAYER_INDEX *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_LAYER_INDEX_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_LAYER_INDEX_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_LAYER_INDEX *) ((arg1)->LAYER_INDEX);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_LAYER_INDEX, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_LAYOUT_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_LAYOUT *arg2 = (Dwg_Object_LAYOUT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_LAYOUT_set(self,LAYOUT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_LAYOUT_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_LAYOUT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_LAYOUT_set" "', argument " "2"" of type '" "Dwg_Object_LAYOUT *""'"); 
    }
    arg2 = (Dwg_Object_LAYOUT *)(argp2);
    if (arg1) (arg1)->LAYOUT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_LAYOUT_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_LAYOUT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_LAYOUT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_LAYOUT_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_LAYOUT *) ((arg1)->LAYOUT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_LAYOUT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_PLACEHOLDER_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_PLACEHOLDER *arg2 = (Dwg_Object_PLACEHOLDER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_PLACEHOLDER_set(self,PLACEHOLDER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_PLACEHOLDER_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_PLACEHOLDER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_PLACEHOLDER_set" "', argument " "2"" of type '" "Dwg_Object_PLACEHOLDER *""'"); 
    }
    arg2 = (Dwg_Object_PLACEHOLDER *)(argp2);
    if (arg1) (arg1)->PLACEHOLDER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_PLACEHOLDER_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_PLACEHOLDER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_PLACEHOLDER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_PLACEHOLDER_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_PLACEHOLDER *) ((arg1)->PLACEHOLDER);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_PLACEHOLDER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_PLOTSETTINGS_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_PLOTSETTINGS *arg2 = (Dwg_Object_PLOTSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_PLOTSETTINGS_set(self,PLOTSETTINGS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_PLOTSETTINGS_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_PLOTSETTINGS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_PLOTSETTINGS_set" "', argument " "2"" of type '" "Dwg_Object_PLOTSETTINGS *""'"); 
    }
    arg2 = (Dwg_Object_PLOTSETTINGS *)(argp2);
    if (arg1) (arg1)->PLOTSETTINGS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_PLOTSETTINGS_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_PLOTSETTINGS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_PLOTSETTINGS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_PLOTSETTINGS_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_PLOTSETTINGS *) ((arg1)->PLOTSETTINGS);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_RASTERVARIABLES_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_RASTERVARIABLES *arg2 = (Dwg_Object_RASTERVARIABLES *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_RASTERVARIABLES_set(self,RASTERVARIABLES);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_RASTERVARIABLES_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_RASTERVARIABLES, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_RASTERVARIABLES_set" "', argument " "2"" of type '" "Dwg_Object_RASTERVARIABLES *""'"); 
    }
    arg2 = (Dwg_Object_RASTERVARIABLES *)(argp2);
    if (arg1) (arg1)->RASTERVARIABLES = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_RASTERVARIABLES_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_RASTERVARIABLES *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_RASTERVARIABLES_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_RASTERVARIABLES_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_RASTERVARIABLES *) ((arg1)->RASTERVARIABLES);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_RASTERVARIABLES, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_RENDERENVIRONMENT_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_RENDERENVIRONMENT *arg2 = (Dwg_Object_RENDERENVIRONMENT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_RENDERENVIRONMENT_set(self,RENDERENVIRONMENT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_RENDERENVIRONMENT_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_RENDERENVIRONMENT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_RENDERENVIRONMENT_set" "', argument " "2"" of type '" "Dwg_Object_RENDERENVIRONMENT *""'"); 
    }
    arg2 = (Dwg_Object_RENDERENVIRONMENT *)(argp2);
    if (arg1) (arg1)->RENDERENVIRONMENT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_RENDERENVIRONMENT_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_RENDERENVIRONMENT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_RENDERENVIRONMENT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_RENDERENVIRONMENT_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_RENDERENVIRONMENT *) ((arg1)->RENDERENVIRONMENT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_RENDERENVIRONMENT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_SCALE_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_SCALE *arg2 = (Dwg_Object_SCALE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_SCALE_set(self,SCALE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_SCALE_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_SCALE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_SCALE_set" "', argument " "2"" of type '" "Dwg_Object_SCALE *""'"); 
    }
    arg2 = (Dwg_Object_SCALE *)(argp2);
    if (arg1) (arg1)->SCALE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_SCALE_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_SCALE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_SCALE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_SCALE_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_SCALE *) ((arg1)->SCALE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_SCALE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_SECTIONVIEWSTYLE_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_SECTIONVIEWSTYLE *arg2 = (Dwg_Object_SECTIONVIEWSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_SECTIONVIEWSTYLE_set(self,SECTIONVIEWSTYLE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_SECTIONVIEWSTYLE_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_SECTIONVIEWSTYLE_set" "', argument " "2"" of type '" "Dwg_Object_SECTIONVIEWSTYLE *""'"); 
    }
    arg2 = (Dwg_Object_SECTIONVIEWSTYLE *)(argp2);
    if (arg1) (arg1)->SECTIONVIEWSTYLE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_SECTIONVIEWSTYLE_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_SECTIONVIEWSTYLE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_SECTIONVIEWSTYLE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_SECTIONVIEWSTYLE_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_SECTIONVIEWSTYLE *) ((arg1)->SECTIONVIEWSTYLE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_SECTION_MANAGER_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_SECTION_MANAGER *arg2 = (Dwg_Object_SECTION_MANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_SECTION_MANAGER_set(self,SECTION_MANAGER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_SECTION_MANAGER_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_SECTION_MANAGER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_SECTION_MANAGER_set" "', argument " "2"" of type '" "Dwg_Object_SECTION_MANAGER *""'"); 
    }
    arg2 = (Dwg_Object_SECTION_MANAGER *)(argp2);
    if (arg1) (arg1)->SECTION_MANAGER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_SECTION_MANAGER_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_SECTION_MANAGER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_SECTION_MANAGER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_SECTION_MANAGER_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_SECTION_MANAGER *) ((arg1)->SECTION_MANAGER);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_SECTION_MANAGER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_SORTENTSTABLE_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_SORTENTSTABLE *arg2 = (Dwg_Object_SORTENTSTABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_SORTENTSTABLE_set(self,SORTENTSTABLE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_SORTENTSTABLE_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_SORTENTSTABLE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_SORTENTSTABLE_set" "', argument " "2"" of type '" "Dwg_Object_SORTENTSTABLE *""'"); 
    }
    arg2 = (Dwg_Object_SORTENTSTABLE *)(argp2);
    if (arg1) (arg1)->SORTENTSTABLE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_SORTENTSTABLE_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_SORTENTSTABLE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_SORTENTSTABLE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_SORTENTSTABLE_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_SORTENTSTABLE *) ((arg1)->SORTENTSTABLE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_SORTENTSTABLE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_SPATIAL_FILTER_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_SPATIAL_FILTER *arg2 = (Dwg_Object_SPATIAL_FILTER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_SPATIAL_FILTER_set(self,SPATIAL_FILTER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_SPATIAL_FILTER_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_SPATIAL_FILTER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_SPATIAL_FILTER_set" "', argument " "2"" of type '" "Dwg_Object_SPATIAL_FILTER *""'"); 
    }
    arg2 = (Dwg_Object_SPATIAL_FILTER *)(argp2);
    if (arg1) (arg1)->SPATIAL_FILTER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_SPATIAL_FILTER_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_SPATIAL_FILTER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_SPATIAL_FILTER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_SPATIAL_FILTER_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_SPATIAL_FILTER *) ((arg1)->SPATIAL_FILTER);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_SPATIAL_FILTER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_SUN_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_SUN *arg2 = (Dwg_Object_SUN *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_SUN_set(self,SUN);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_SUN_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_SUN, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_SUN_set" "', argument " "2"" of type '" "Dwg_Object_SUN *""'"); 
    }
    arg2 = (Dwg_Object_SUN *)(argp2);
    if (arg1) (arg1)->SUN = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_SUN_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_SUN *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_SUN_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_SUN_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_SUN *) ((arg1)->SUN);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_SUN, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_TABLEGEOMETRY_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_TABLEGEOMETRY *arg2 = (Dwg_Object_TABLEGEOMETRY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_TABLEGEOMETRY_set(self,TABLEGEOMETRY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_TABLEGEOMETRY_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_TABLEGEOMETRY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_TABLEGEOMETRY_set" "', argument " "2"" of type '" "Dwg_Object_TABLEGEOMETRY *""'"); 
    }
    arg2 = (Dwg_Object_TABLEGEOMETRY *)(argp2);
    if (arg1) (arg1)->TABLEGEOMETRY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_TABLEGEOMETRY_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_TABLEGEOMETRY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_TABLEGEOMETRY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_TABLEGEOMETRY_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_TABLEGEOMETRY *) ((arg1)->TABLEGEOMETRY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_TABLEGEOMETRY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_VBA_PROJECT_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_VBA_PROJECT *arg2 = (Dwg_Object_VBA_PROJECT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_VBA_PROJECT_set(self,VBA_PROJECT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_VBA_PROJECT_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_VBA_PROJECT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_VBA_PROJECT_set" "', argument " "2"" of type '" "Dwg_Object_VBA_PROJECT *""'"); 
    }
    arg2 = (Dwg_Object_VBA_PROJECT *)(argp2);
    if (arg1) (arg1)->VBA_PROJECT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_VBA_PROJECT_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_VBA_PROJECT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_VBA_PROJECT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_VBA_PROJECT_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_VBA_PROJECT *) ((arg1)->VBA_PROJECT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_VBA_PROJECT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_VISUALSTYLE_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_VISUALSTYLE *arg2 = (Dwg_Object_VISUALSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_VISUALSTYLE_set(self,VISUALSTYLE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_VISUALSTYLE_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_VISUALSTYLE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_VISUALSTYLE_set" "', argument " "2"" of type '" "Dwg_Object_VISUALSTYLE *""'"); 
    }
    arg2 = (Dwg_Object_VISUALSTYLE *)(argp2);
    if (arg1) (arg1)->VISUALSTYLE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_VISUALSTYLE_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_VISUALSTYLE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_VISUALSTYLE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_VISUALSTYLE_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_VISUALSTYLE *) ((arg1)->VISUALSTYLE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_WIPEOUTVARIABLES_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_WIPEOUTVARIABLES *arg2 = (Dwg_Object_WIPEOUTVARIABLES *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_WIPEOUTVARIABLES_set(self,WIPEOUTVARIABLES);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_WIPEOUTVARIABLES_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_WIPEOUTVARIABLES, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_WIPEOUTVARIABLES_set" "', argument " "2"" of type '" "Dwg_Object_WIPEOUTVARIABLES *""'"); 
    }
    arg2 = (Dwg_Object_WIPEOUTVARIABLES *)(argp2);
    if (arg1) (arg1)->WIPEOUTVARIABLES = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_WIPEOUTVARIABLES_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_WIPEOUTVARIABLES *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_WIPEOUTVARIABLES_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_WIPEOUTVARIABLES_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_WIPEOUTVARIABLES *) ((arg1)->WIPEOUTVARIABLES);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_WIPEOUTVARIABLES, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_XRECORD_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_XRECORD *arg2 = (Dwg_Object_XRECORD *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_XRECORD_set(self,XRECORD);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_XRECORD_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_XRECORD, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_XRECORD_set" "', argument " "2"" of type '" "Dwg_Object_XRECORD *""'"); 
    }
    arg2 = (Dwg_Object_XRECORD *)(argp2);
    if (arg1) (arg1)->XRECORD = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_XRECORD_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_XRECORD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_XRECORD_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_XRECORD_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_XRECORD *) ((arg1)->XRECORD);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_XRECORD, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_PDFDEFINITION_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_PDFDEFINITION *arg2 = (Dwg_Object_PDFDEFINITION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_PDFDEFINITION_set(self,PDFDEFINITION);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_PDFDEFINITION_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_abstractobject_UNDERLAYDEFINITION, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_PDFDEFINITION_set" "', argument " "2"" of type '" "Dwg_Object_PDFDEFINITION *""'"); 
    }
    arg2 = (Dwg_Object_PDFDEFINITION *)(argp2);
    if (arg1) (arg1)->PDFDEFINITION = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_PDFDEFINITION_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_PDFDEFINITION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_PDFDEFINITION_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_PDFDEFINITION_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_PDFDEFINITION *) ((arg1)->PDFDEFINITION);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_abstractobject_UNDERLAYDEFINITION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_DGNDEFINITION_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_DGNDEFINITION *arg2 = (Dwg_Object_DGNDEFINITION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_DGNDEFINITION_set(self,DGNDEFINITION);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_DGNDEFINITION_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_abstractobject_UNDERLAYDEFINITION, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_DGNDEFINITION_set" "', argument " "2"" of type '" "Dwg_Object_DGNDEFINITION *""'"); 
    }
    arg2 = (Dwg_Object_DGNDEFINITION *)(argp2);
    if (arg1) (arg1)->DGNDEFINITION = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_DGNDEFINITION_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DGNDEFINITION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_DGNDEFINITION_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_DGNDEFINITION_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_DGNDEFINITION *) ((arg1)->DGNDEFINITION);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_abstractobject_UNDERLAYDEFINITION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_DWFDEFINITION_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_DWFDEFINITION *arg2 = (Dwg_Object_DWFDEFINITION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_DWFDEFINITION_set(self,DWFDEFINITION);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_DWFDEFINITION_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_abstractobject_UNDERLAYDEFINITION, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_DWFDEFINITION_set" "', argument " "2"" of type '" "Dwg_Object_DWFDEFINITION *""'"); 
    }
    arg2 = (Dwg_Object_DWFDEFINITION *)(argp2);
    if (arg1) (arg1)->DWFDEFINITION = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_DWFDEFINITION_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DWFDEFINITION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_DWFDEFINITION_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_DWFDEFINITION_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_DWFDEFINITION *) ((arg1)->DWFDEFINITION);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_abstractobject_UNDERLAYDEFINITION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACSH_BREP_CLASS_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ACSH_BREP_CLASS *arg2 = (Dwg_Object_ACSH_BREP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACSH_BREP_CLASS_set(self,ACSH_BREP_CLASS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACSH_BREP_CLASS_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ACSH_BREP_CLASS_set" "', argument " "2"" of type '" "Dwg_Object_ACSH_BREP_CLASS *""'"); 
    }
    arg2 = (Dwg_Object_ACSH_BREP_CLASS *)(argp2);
    if (arg1) (arg1)->ACSH_BREP_CLASS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACSH_BREP_CLASS_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_BREP_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACSH_BREP_CLASS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACSH_BREP_CLASS_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ACSH_BREP_CLASS *) ((arg1)->ACSH_BREP_CLASS);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACSH_CHAMFER_CLASS_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ACSH_CHAMFER_CLASS *arg2 = (Dwg_Object_ACSH_CHAMFER_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACSH_CHAMFER_CLASS_set(self,ACSH_CHAMFER_CLASS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACSH_CHAMFER_CLASS_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ACSH_CHAMFER_CLASS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ACSH_CHAMFER_CLASS_set" "', argument " "2"" of type '" "Dwg_Object_ACSH_CHAMFER_CLASS *""'"); 
    }
    arg2 = (Dwg_Object_ACSH_CHAMFER_CLASS *)(argp2);
    if (arg1) (arg1)->ACSH_CHAMFER_CLASS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACSH_CHAMFER_CLASS_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_CHAMFER_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACSH_CHAMFER_CLASS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACSH_CHAMFER_CLASS_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ACSH_CHAMFER_CLASS *) ((arg1)->ACSH_CHAMFER_CLASS);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_CHAMFER_CLASS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACSH_PYRAMID_CLASS_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ACSH_PYRAMID_CLASS *arg2 = (Dwg_Object_ACSH_PYRAMID_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACSH_PYRAMID_CLASS_set(self,ACSH_PYRAMID_CLASS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACSH_PYRAMID_CLASS_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ACSH_PYRAMID_CLASS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ACSH_PYRAMID_CLASS_set" "', argument " "2"" of type '" "Dwg_Object_ACSH_PYRAMID_CLASS *""'"); 
    }
    arg2 = (Dwg_Object_ACSH_PYRAMID_CLASS *)(argp2);
    if (arg1) (arg1)->ACSH_PYRAMID_CLASS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACSH_PYRAMID_CLASS_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_PYRAMID_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACSH_PYRAMID_CLASS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACSH_PYRAMID_CLASS_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ACSH_PYRAMID_CLASS *) ((arg1)->ACSH_PYRAMID_CLASS);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_PYRAMID_CLASS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ALDIMOBJECTCONTEXTDATA_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ALDIMOBJECTCONTEXTDATA *arg2 = (Dwg_Object_ALDIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ALDIMOBJECTCONTEXTDATA_set(self,ALDIMOBJECTCONTEXTDATA);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ALDIMOBJECTCONTEXTDATA_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ALDIMOBJECTCONTEXTDATA, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ALDIMOBJECTCONTEXTDATA_set" "', argument " "2"" of type '" "Dwg_Object_ALDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg2 = (Dwg_Object_ALDIMOBJECTCONTEXTDATA *)(argp2);
    if (arg1) (arg1)->ALDIMOBJECTCONTEXTDATA = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ALDIMOBJECTCONTEXTDATA_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ALDIMOBJECTCONTEXTDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ALDIMOBJECTCONTEXTDATA_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ALDIMOBJECTCONTEXTDATA_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ALDIMOBJECTCONTEXTDATA *) ((arg1)->ALDIMOBJECTCONTEXTDATA);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ALDIMOBJECTCONTEXTDATA, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOC2DCONSTRAINTGROUP_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOC2DCONSTRAINTGROUP *arg2 = (Dwg_Object_ASSOC2DCONSTRAINTGROUP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOC2DCONSTRAINTGROUP_set(self,ASSOC2DCONSTRAINTGROUP);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOC2DCONSTRAINTGROUP_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOC2DCONSTRAINTGROUP_set" "', argument " "2"" of type '" "Dwg_Object_ASSOC2DCONSTRAINTGROUP *""'"); 
    }
    arg2 = (Dwg_Object_ASSOC2DCONSTRAINTGROUP *)(argp2);
    if (arg1) (arg1)->ASSOC2DCONSTRAINTGROUP = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOC2DCONSTRAINTGROUP_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOC2DCONSTRAINTGROUP *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOC2DCONSTRAINTGROUP_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOC2DCONSTRAINTGROUP_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOC2DCONSTRAINTGROUP *) ((arg1)->ASSOC2DCONSTRAINTGROUP);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCACTION_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCACTION *arg2 = (Dwg_Object_ASSOCACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCACTION_set(self,ASSOCACTION);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCACTION_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCACTION, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCACTION_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCACTION *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCACTION *)(argp2);
    if (arg1) (arg1)->ASSOCACTION = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCACTION_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCACTION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCACTION_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCACTION_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCACTION *) ((arg1)->ASSOCACTION);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCACTION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCACTIONPARAM_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCACTIONPARAM *arg2 = (Dwg_Object_ASSOCACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCACTIONPARAM_set(self,ASSOCACTIONPARAM);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCACTIONPARAM_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCACTIONPARAM, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCACTIONPARAM_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCACTIONPARAM *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCACTIONPARAM *)(argp2);
    if (arg1) (arg1)->ASSOCACTIONPARAM = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCACTIONPARAM_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCACTIONPARAM *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCACTIONPARAM_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCACTIONPARAM_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCACTIONPARAM *) ((arg1)->ASSOCACTIONPARAM);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCACTIONPARAM, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCARRAYACTIONBODY_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCARRAYACTIONBODY *arg2 = (Dwg_Object_ASSOCARRAYACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCARRAYACTIONBODY_set(self,ASSOCARRAYACTIONBODY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCARRAYACTIONBODY_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCARRAYACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCARRAYACTIONBODY_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCARRAYACTIONBODY *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCARRAYACTIONBODY *)(argp2);
    if (arg1) (arg1)->ASSOCARRAYACTIONBODY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCARRAYACTIONBODY_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCARRAYACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCARRAYACTIONBODY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCARRAYACTIONBODY_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCARRAYACTIONBODY *) ((arg1)->ASSOCARRAYACTIONBODY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCARRAYACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCASMBODYACTIONPARAM_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCASMBODYACTIONPARAM *arg2 = (Dwg_Object_ASSOCASMBODYACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCASMBODYACTIONPARAM_set(self,ASSOCASMBODYACTIONPARAM);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCASMBODYACTIONPARAM_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCASMBODYACTIONPARAM_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCASMBODYACTIONPARAM *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCASMBODYACTIONPARAM *)(argp2);
    if (arg1) (arg1)->ASSOCASMBODYACTIONPARAM = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCASMBODYACTIONPARAM_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCASMBODYACTIONPARAM *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCASMBODYACTIONPARAM_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCASMBODYACTIONPARAM_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCASMBODYACTIONPARAM *) ((arg1)->ASSOCASMBODYACTIONPARAM);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCBLENDSURFACEACTIONBODY_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCBLENDSURFACEACTIONBODY *arg2 = (Dwg_Object_ASSOCBLENDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCBLENDSURFACEACTIONBODY_set(self,ASSOCBLENDSURFACEACTIONBODY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCBLENDSURFACEACTIONBODY_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCBLENDSURFACEACTIONBODY_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCBLENDSURFACEACTIONBODY *)(argp2);
    if (arg1) (arg1)->ASSOCBLENDSURFACEACTIONBODY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCBLENDSURFACEACTIONBODY_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCBLENDSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCBLENDSURFACEACTIONBODY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCBLENDSURFACEACTIONBODY_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCBLENDSURFACEACTIONBODY *) ((arg1)->ASSOCBLENDSURFACEACTIONBODY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCCOMPOUNDACTIONPARAM_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCCOMPOUNDACTIONPARAM *arg2 = (Dwg_Object_ASSOCCOMPOUNDACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCCOMPOUNDACTIONPARAM_set(self,ASSOCCOMPOUNDACTIONPARAM);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCCOMPOUNDACTIONPARAM_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCCOMPOUNDACTIONPARAM_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCCOMPOUNDACTIONPARAM *)(argp2);
    if (arg1) (arg1)->ASSOCCOMPOUNDACTIONPARAM = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCCOMPOUNDACTIONPARAM_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCCOMPOUNDACTIONPARAM *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCCOMPOUNDACTIONPARAM_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCCOMPOUNDACTIONPARAM_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCCOMPOUNDACTIONPARAM *) ((arg1)->ASSOCCOMPOUNDACTIONPARAM);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCDEPENDENCY_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCDEPENDENCY *arg2 = (Dwg_Object_ASSOCDEPENDENCY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCDEPENDENCY_set(self,ASSOCDEPENDENCY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCDEPENDENCY_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCDEPENDENCY_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCDEPENDENCY *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCDEPENDENCY *)(argp2);
    if (arg1) (arg1)->ASSOCDEPENDENCY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCDEPENDENCY_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCDEPENDENCY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCDEPENDENCY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCDEPENDENCY_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCDEPENDENCY *) ((arg1)->ASSOCDEPENDENCY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCDIMDEPENDENCYBODY_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCDIMDEPENDENCYBODY *arg2 = (Dwg_Object_ASSOCDIMDEPENDENCYBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCDIMDEPENDENCYBODY_set(self,ASSOCDIMDEPENDENCYBODY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCDIMDEPENDENCYBODY_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCDIMDEPENDENCYBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCDIMDEPENDENCYBODY_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCDIMDEPENDENCYBODY *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCDIMDEPENDENCYBODY *)(argp2);
    if (arg1) (arg1)->ASSOCDIMDEPENDENCYBODY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCDIMDEPENDENCYBODY_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCDIMDEPENDENCYBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCDIMDEPENDENCYBODY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCDIMDEPENDENCYBODY_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCDIMDEPENDENCYBODY *) ((arg1)->ASSOCDIMDEPENDENCYBODY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCDIMDEPENDENCYBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCEXTENDSURFACEACTIONBODY_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY *arg2 = (Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCEXTENDSURFACEACTIONBODY_set(self,ASSOCEXTENDSURFACEACTIONBODY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCEXTENDSURFACEACTIONBODY_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCEXTENDSURFACEACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCEXTENDSURFACEACTIONBODY_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY *)(argp2);
    if (arg1) (arg1)->ASSOCEXTENDSURFACEACTIONBODY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCEXTENDSURFACEACTIONBODY_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCEXTENDSURFACEACTIONBODY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCEXTENDSURFACEACTIONBODY_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY *) ((arg1)->ASSOCEXTENDSURFACEACTIONBODY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCEXTENDSURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCEXTRUDEDSURFACEACTIONBODY_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY *arg2 = (Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCEXTRUDEDSURFACEACTIONBODY_set(self,ASSOCEXTRUDEDSURFACEACTIONBODY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCEXTRUDEDSURFACEACTIONBODY_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCEXTRUDEDSURFACEACTIONBODY_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY *)(argp2);
    if (arg1) (arg1)->ASSOCEXTRUDEDSURFACEACTIONBODY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCEXTRUDEDSURFACEACTIONBODY_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCEXTRUDEDSURFACEACTIONBODY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCEXTRUDEDSURFACEACTIONBODY_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY *) ((arg1)->ASSOCEXTRUDEDSURFACEACTIONBODY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCFACEACTIONPARAM_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCFACEACTIONPARAM *arg2 = (Dwg_Object_ASSOCFACEACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCFACEACTIONPARAM_set(self,ASSOCFACEACTIONPARAM);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCFACEACTIONPARAM_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCFACEACTIONPARAM, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCFACEACTIONPARAM_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCFACEACTIONPARAM *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCFACEACTIONPARAM *)(argp2);
    if (arg1) (arg1)->ASSOCFACEACTIONPARAM = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCFACEACTIONPARAM_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCFACEACTIONPARAM *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCFACEACTIONPARAM_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCFACEACTIONPARAM_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCFACEACTIONPARAM *) ((arg1)->ASSOCFACEACTIONPARAM);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCFACEACTIONPARAM, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCFILLETSURFACEACTIONBODY_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCFILLETSURFACEACTIONBODY *arg2 = (Dwg_Object_ASSOCFILLETSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCFILLETSURFACEACTIONBODY_set(self,ASSOCFILLETSURFACEACTIONBODY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCFILLETSURFACEACTIONBODY_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCFILLETSURFACEACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCFILLETSURFACEACTIONBODY_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCFILLETSURFACEACTIONBODY *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCFILLETSURFACEACTIONBODY *)(argp2);
    if (arg1) (arg1)->ASSOCFILLETSURFACEACTIONBODY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCFILLETSURFACEACTIONBODY_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCFILLETSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCFILLETSURFACEACTIONBODY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCFILLETSURFACEACTIONBODY_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCFILLETSURFACEACTIONBODY *) ((arg1)->ASSOCFILLETSURFACEACTIONBODY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCFILLETSURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCLOFTEDSURFACEACTIONBODY_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY *arg2 = (Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCLOFTEDSURFACEACTIONBODY_set(self,ASSOCLOFTEDSURFACEACTIONBODY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCLOFTEDSURFACEACTIONBODY_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCLOFTEDSURFACEACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCLOFTEDSURFACEACTIONBODY_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY *)(argp2);
    if (arg1) (arg1)->ASSOCLOFTEDSURFACEACTIONBODY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCLOFTEDSURFACEACTIONBODY_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCLOFTEDSURFACEACTIONBODY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCLOFTEDSURFACEACTIONBODY_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY *) ((arg1)->ASSOCLOFTEDSURFACEACTIONBODY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCLOFTEDSURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCNETWORKSURFACEACTIONBODY_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY *arg2 = (Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCNETWORKSURFACEACTIONBODY_set(self,ASSOCNETWORKSURFACEACTIONBODY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCNETWORKSURFACEACTIONBODY_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCNETWORKSURFACEACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCNETWORKSURFACEACTIONBODY_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY *)(argp2);
    if (arg1) (arg1)->ASSOCNETWORKSURFACEACTIONBODY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCNETWORKSURFACEACTIONBODY_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCNETWORKSURFACEACTIONBODY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCNETWORKSURFACEACTIONBODY_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY *) ((arg1)->ASSOCNETWORKSURFACEACTIONBODY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCNETWORKSURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCOBJECTACTIONPARAM_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCOBJECTACTIONPARAM *arg2 = (Dwg_Object_ASSOCOBJECTACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCOBJECTACTIONPARAM_set(self,ASSOCOBJECTACTIONPARAM);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCOBJECTACTIONPARAM_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCOBJECTACTIONPARAM, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCOBJECTACTIONPARAM_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCOBJECTACTIONPARAM *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCOBJECTACTIONPARAM *)(argp2);
    if (arg1) (arg1)->ASSOCOBJECTACTIONPARAM = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCOBJECTACTIONPARAM_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCOBJECTACTIONPARAM *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCOBJECTACTIONPARAM_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCOBJECTACTIONPARAM_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCOBJECTACTIONPARAM *) ((arg1)->ASSOCOBJECTACTIONPARAM);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCOBJECTACTIONPARAM, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCOFFSETSURFACEACTIONBODY_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY *arg2 = (Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCOFFSETSURFACEACTIONBODY_set(self,ASSOCOFFSETSURFACEACTIONBODY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCOFFSETSURFACEACTIONBODY_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCOFFSETSURFACEACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCOFFSETSURFACEACTIONBODY_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY *)(argp2);
    if (arg1) (arg1)->ASSOCOFFSETSURFACEACTIONBODY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCOFFSETSURFACEACTIONBODY_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCOFFSETSURFACEACTIONBODY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCOFFSETSURFACEACTIONBODY_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY *) ((arg1)->ASSOCOFFSETSURFACEACTIONBODY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCOFFSETSURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCOSNAPPOINTREFACTIONPARAM_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM *arg2 = (Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCOSNAPPOINTREFACTIONPARAM_set(self,ASSOCOSNAPPOINTREFACTIONPARAM);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCOSNAPPOINTREFACTIONPARAM_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCOSNAPPOINTREFACTIONPARAM_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM *)(argp2);
    if (arg1) (arg1)->ASSOCOSNAPPOINTREFACTIONPARAM = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCOSNAPPOINTREFACTIONPARAM_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCOSNAPPOINTREFACTIONPARAM_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCOSNAPPOINTREFACTIONPARAM_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM *) ((arg1)->ASSOCOSNAPPOINTREFACTIONPARAM);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCPATCHSURFACEACTIONBODY_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCPATCHSURFACEACTIONBODY *arg2 = (Dwg_Object_ASSOCPATCHSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCPATCHSURFACEACTIONBODY_set(self,ASSOCPATCHSURFACEACTIONBODY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCPATCHSURFACEACTIONBODY_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCPATCHSURFACEACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCPATCHSURFACEACTIONBODY_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCPATCHSURFACEACTIONBODY *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCPATCHSURFACEACTIONBODY *)(argp2);
    if (arg1) (arg1)->ASSOCPATCHSURFACEACTIONBODY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCPATCHSURFACEACTIONBODY_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCPATCHSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCPATCHSURFACEACTIONBODY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCPATCHSURFACEACTIONBODY_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCPATCHSURFACEACTIONBODY *) ((arg1)->ASSOCPATCHSURFACEACTIONBODY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCPATCHSURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCPATHACTIONPARAM_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCPATHACTIONPARAM *arg2 = (Dwg_Object_ASSOCPATHACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCPATHACTIONPARAM_set(self,ASSOCPATHACTIONPARAM);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCPATHACTIONPARAM_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCPATHACTIONPARAM_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCPATHACTIONPARAM *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCPATHACTIONPARAM *)(argp2);
    if (arg1) (arg1)->ASSOCPATHACTIONPARAM = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCPATHACTIONPARAM_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCPATHACTIONPARAM *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCPATHACTIONPARAM_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCPATHACTIONPARAM_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCPATHACTIONPARAM *) ((arg1)->ASSOCPATHACTIONPARAM);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCPLANESURFACEACTIONBODY_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCPLANESURFACEACTIONBODY *arg2 = (Dwg_Object_ASSOCPLANESURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCPLANESURFACEACTIONBODY_set(self,ASSOCPLANESURFACEACTIONBODY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCPLANESURFACEACTIONBODY_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCPLANESURFACEACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCPLANESURFACEACTIONBODY_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCPLANESURFACEACTIONBODY *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCPLANESURFACEACTIONBODY *)(argp2);
    if (arg1) (arg1)->ASSOCPLANESURFACEACTIONBODY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCPLANESURFACEACTIONBODY_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCPLANESURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCPLANESURFACEACTIONBODY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCPLANESURFACEACTIONBODY_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCPLANESURFACEACTIONBODY *) ((arg1)->ASSOCPLANESURFACEACTIONBODY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCPLANESURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCPOINTREFACTIONPARAM_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCPOINTREFACTIONPARAM *arg2 = (Dwg_Object_ASSOCPOINTREFACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCPOINTREFACTIONPARAM_set(self,ASSOCPOINTREFACTIONPARAM);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCPOINTREFACTIONPARAM_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCPOINTREFACTIONPARAM_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCPOINTREFACTIONPARAM *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCPOINTREFACTIONPARAM *)(argp2);
    if (arg1) (arg1)->ASSOCPOINTREFACTIONPARAM = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCPOINTREFACTIONPARAM_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCPOINTREFACTIONPARAM *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCPOINTREFACTIONPARAM_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCPOINTREFACTIONPARAM_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCPOINTREFACTIONPARAM *) ((arg1)->ASSOCPOINTREFACTIONPARAM);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCREVOLVEDSURFACEACTIONBODY_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY *arg2 = (Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCREVOLVEDSURFACEACTIONBODY_set(self,ASSOCREVOLVEDSURFACEACTIONBODY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCREVOLVEDSURFACEACTIONBODY_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCREVOLVEDSURFACEACTIONBODY_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY *)(argp2);
    if (arg1) (arg1)->ASSOCREVOLVEDSURFACEACTIONBODY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCREVOLVEDSURFACEACTIONBODY_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCREVOLVEDSURFACEACTIONBODY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCREVOLVEDSURFACEACTIONBODY_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY *) ((arg1)->ASSOCREVOLVEDSURFACEACTIONBODY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCTRIMSURFACEACTIONBODY_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCTRIMSURFACEACTIONBODY *arg2 = (Dwg_Object_ASSOCTRIMSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCTRIMSURFACEACTIONBODY_set(self,ASSOCTRIMSURFACEACTIONBODY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCTRIMSURFACEACTIONBODY_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCTRIMSURFACEACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCTRIMSURFACEACTIONBODY_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCTRIMSURFACEACTIONBODY *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCTRIMSURFACEACTIONBODY *)(argp2);
    if (arg1) (arg1)->ASSOCTRIMSURFACEACTIONBODY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCTRIMSURFACEACTIONBODY_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCTRIMSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCTRIMSURFACEACTIONBODY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCTRIMSURFACEACTIONBODY_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCTRIMSURFACEACTIONBODY *) ((arg1)->ASSOCTRIMSURFACEACTIONBODY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCTRIMSURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCVALUEDEPENDENCY_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCVALUEDEPENDENCY *arg2 = (Dwg_Object_ASSOCVALUEDEPENDENCY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCVALUEDEPENDENCY_set(self,ASSOCVALUEDEPENDENCY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCVALUEDEPENDENCY_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCVALUEDEPENDENCY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCVALUEDEPENDENCY_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCVALUEDEPENDENCY *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCVALUEDEPENDENCY *)(argp2);
    if (arg1) (arg1)->ASSOCVALUEDEPENDENCY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCVALUEDEPENDENCY_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCVALUEDEPENDENCY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCVALUEDEPENDENCY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCVALUEDEPENDENCY_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCVALUEDEPENDENCY *) ((arg1)->ASSOCVALUEDEPENDENCY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCVALUEDEPENDENCY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCVARIABLE_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCVARIABLE *arg2 = (Dwg_Object_ASSOCVARIABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCVARIABLE_set(self,ASSOCVARIABLE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCVARIABLE_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCVARIABLE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCVARIABLE_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCVARIABLE *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCVARIABLE *)(argp2);
    if (arg1) (arg1)->ASSOCVARIABLE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCVARIABLE_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCVARIABLE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCVARIABLE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCVARIABLE_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCVARIABLE *) ((arg1)->ASSOCVARIABLE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCVERTEXACTIONPARAM_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCVERTEXACTIONPARAM *arg2 = (Dwg_Object_ASSOCVERTEXACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCVERTEXACTIONPARAM_set(self,ASSOCVERTEXACTIONPARAM);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCVERTEXACTIONPARAM_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCVERTEXACTIONPARAM, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCVERTEXACTIONPARAM_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCVERTEXACTIONPARAM *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCVERTEXACTIONPARAM *)(argp2);
    if (arg1) (arg1)->ASSOCVERTEXACTIONPARAM = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCVERTEXACTIONPARAM_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCVERTEXACTIONPARAM *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCVERTEXACTIONPARAM_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCVERTEXACTIONPARAM_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCVERTEXACTIONPARAM *) ((arg1)->ASSOCVERTEXACTIONPARAM);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCVERTEXACTIONPARAM, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLKREFOBJECTCONTEXTDATA_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLKREFOBJECTCONTEXTDATA *arg2 = (Dwg_Object_BLKREFOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLKREFOBJECTCONTEXTDATA_set(self,BLKREFOBJECTCONTEXTDATA);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLKREFOBJECTCONTEXTDATA_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLKREFOBJECTCONTEXTDATA, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLKREFOBJECTCONTEXTDATA_set" "', argument " "2"" of type '" "Dwg_Object_BLKREFOBJECTCONTEXTDATA *""'"); 
    }
    arg2 = (Dwg_Object_BLKREFOBJECTCONTEXTDATA *)(argp2);
    if (arg1) (arg1)->BLKREFOBJECTCONTEXTDATA = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLKREFOBJECTCONTEXTDATA_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLKREFOBJECTCONTEXTDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLKREFOBJECTCONTEXTDATA_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLKREFOBJECTCONTEXTDATA_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLKREFOBJECTCONTEXTDATA *) ((arg1)->BLKREFOBJECTCONTEXTDATA);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLKREFOBJECTCONTEXTDATA, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKALIGNEDCONSTRAINTPARAMETER_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER *arg2 = (Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKALIGNEDCONSTRAINTPARAMETER_set(self,BLOCKALIGNEDCONSTRAINTPARAMETER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKALIGNEDCONSTRAINTPARAMETER_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKALIGNEDCONSTRAINTPARAMETER_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER *)(argp2);
    if (arg1) (arg1)->BLOCKALIGNEDCONSTRAINTPARAMETER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKALIGNEDCONSTRAINTPARAMETER_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKALIGNEDCONSTRAINTPARAMETER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKALIGNEDCONSTRAINTPARAMETER_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER *) ((arg1)->BLOCKALIGNEDCONSTRAINTPARAMETER);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKANGULARCONSTRAINTPARAMETER_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER *arg2 = (Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKANGULARCONSTRAINTPARAMETER_set(self,BLOCKANGULARCONSTRAINTPARAMETER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKANGULARCONSTRAINTPARAMETER_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKANGULARCONSTRAINTPARAMETER_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER *)(argp2);
    if (arg1) (arg1)->BLOCKANGULARCONSTRAINTPARAMETER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKANGULARCONSTRAINTPARAMETER_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKANGULARCONSTRAINTPARAMETER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKANGULARCONSTRAINTPARAMETER_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER *) ((arg1)->BLOCKANGULARCONSTRAINTPARAMETER);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKARRAYACTION_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKARRAYACTION *arg2 = (Dwg_Object_BLOCKARRAYACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKARRAYACTION_set(self,BLOCKARRAYACTION);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKARRAYACTION_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKARRAYACTION, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKARRAYACTION_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKARRAYACTION *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKARRAYACTION *)(argp2);
    if (arg1) (arg1)->BLOCKARRAYACTION = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKARRAYACTION_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKARRAYACTION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKARRAYACTION_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKARRAYACTION_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKARRAYACTION *) ((arg1)->BLOCKARRAYACTION);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKARRAYACTION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKDIAMETRICCONSTRAINTPARAMETER_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER *arg2 = (Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKDIAMETRICCONSTRAINTPARAMETER_set(self,BLOCKDIAMETRICCONSTRAINTPARAMETER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKDIAMETRICCONSTRAINTPARAMETER_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKDIAMETRICCONSTRAINTPARAMETER_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)(argp2);
    if (arg1) (arg1)->BLOCKDIAMETRICCONSTRAINTPARAMETER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKDIAMETRICCONSTRAINTPARAMETER_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKDIAMETRICCONSTRAINTPARAMETER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKDIAMETRICCONSTRAINTPARAMETER_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER *) ((arg1)->BLOCKDIAMETRICCONSTRAINTPARAMETER);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKHORIZONTALCONSTRAINTPARAMETER_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER *arg2 = (Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKHORIZONTALCONSTRAINTPARAMETER_set(self,BLOCKHORIZONTALCONSTRAINTPARAMETER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKHORIZONTALCONSTRAINTPARAMETER_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKHORIZONTALCONSTRAINTPARAMETER_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)(argp2);
    if (arg1) (arg1)->BLOCKHORIZONTALCONSTRAINTPARAMETER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKHORIZONTALCONSTRAINTPARAMETER_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKHORIZONTALCONSTRAINTPARAMETER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKHORIZONTALCONSTRAINTPARAMETER_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER *) ((arg1)->BLOCKHORIZONTALCONSTRAINTPARAMETER);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKLINEARCONSTRAINTPARAMETER_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER *arg2 = (Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKLINEARCONSTRAINTPARAMETER_set(self,BLOCKLINEARCONSTRAINTPARAMETER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKLINEARCONSTRAINTPARAMETER_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKLINEARCONSTRAINTPARAMETER_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER *)(argp2);
    if (arg1) (arg1)->BLOCKLINEARCONSTRAINTPARAMETER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKLINEARCONSTRAINTPARAMETER_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKLINEARCONSTRAINTPARAMETER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKLINEARCONSTRAINTPARAMETER_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER *) ((arg1)->BLOCKLINEARCONSTRAINTPARAMETER);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKLINEARPARAMETER_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKLINEARPARAMETER *arg2 = (Dwg_Object_BLOCKLINEARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKLINEARPARAMETER_set(self,BLOCKLINEARPARAMETER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKLINEARPARAMETER_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKLINEARPARAMETER_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKLINEARPARAMETER *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKLINEARPARAMETER *)(argp2);
    if (arg1) (arg1)->BLOCKLINEARPARAMETER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKLINEARPARAMETER_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKLINEARPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKLINEARPARAMETER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKLINEARPARAMETER_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKLINEARPARAMETER *) ((arg1)->BLOCKLINEARPARAMETER);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKLOOKUPACTION_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKLOOKUPACTION *arg2 = (Dwg_Object_BLOCKLOOKUPACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKLOOKUPACTION_set(self,BLOCKLOOKUPACTION);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKLOOKUPACTION_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKLOOKUPACTION_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKLOOKUPACTION *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKLOOKUPACTION *)(argp2);
    if (arg1) (arg1)->BLOCKLOOKUPACTION = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKLOOKUPACTION_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKLOOKUPACTION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKLOOKUPACTION_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKLOOKUPACTION_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKLOOKUPACTION *) ((arg1)->BLOCKLOOKUPACTION);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKLOOKUPPARAMETER_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKLOOKUPPARAMETER *arg2 = (Dwg_Object_BLOCKLOOKUPPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKLOOKUPPARAMETER_set(self,BLOCKLOOKUPPARAMETER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKLOOKUPPARAMETER_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKLOOKUPPARAMETER_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKLOOKUPPARAMETER *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKLOOKUPPARAMETER *)(argp2);
    if (arg1) (arg1)->BLOCKLOOKUPPARAMETER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKLOOKUPPARAMETER_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKLOOKUPPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKLOOKUPPARAMETER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKLOOKUPPARAMETER_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKLOOKUPPARAMETER *) ((arg1)->BLOCKLOOKUPPARAMETER);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKPARAMDEPENDENCYBODY_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKPARAMDEPENDENCYBODY *arg2 = (Dwg_Object_BLOCKPARAMDEPENDENCYBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKPARAMDEPENDENCYBODY_set(self,BLOCKPARAMDEPENDENCYBODY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKPARAMDEPENDENCYBODY_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKPARAMDEPENDENCYBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKPARAMDEPENDENCYBODY_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKPARAMDEPENDENCYBODY *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKPARAMDEPENDENCYBODY *)(argp2);
    if (arg1) (arg1)->BLOCKPARAMDEPENDENCYBODY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKPARAMDEPENDENCYBODY_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKPARAMDEPENDENCYBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKPARAMDEPENDENCYBODY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKPARAMDEPENDENCYBODY_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKPARAMDEPENDENCYBODY *) ((arg1)->BLOCKPARAMDEPENDENCYBODY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKPARAMDEPENDENCYBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKPOINTPARAMETER_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKPOINTPARAMETER *arg2 = (Dwg_Object_BLOCKPOINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKPOINTPARAMETER_set(self,BLOCKPOINTPARAMETER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKPOINTPARAMETER_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKPOINTPARAMETER_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKPOINTPARAMETER *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKPOINTPARAMETER *)(argp2);
    if (arg1) (arg1)->BLOCKPOINTPARAMETER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKPOINTPARAMETER_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKPOINTPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKPOINTPARAMETER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKPOINTPARAMETER_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKPOINTPARAMETER *) ((arg1)->BLOCKPOINTPARAMETER);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKPOLARGRIP_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKPOLARGRIP *arg2 = (Dwg_Object_BLOCKPOLARGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKPOLARGRIP_set(self,BLOCKPOLARGRIP);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKPOLARGRIP_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKPOLARGRIP, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKPOLARGRIP_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKPOLARGRIP *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKPOLARGRIP *)(argp2);
    if (arg1) (arg1)->BLOCKPOLARGRIP = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKPOLARGRIP_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKPOLARGRIP *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKPOLARGRIP_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKPOLARGRIP_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKPOLARGRIP *) ((arg1)->BLOCKPOLARGRIP);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKPOLARGRIP, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKPOLARPARAMETER_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKPOLARPARAMETER *arg2 = (Dwg_Object_BLOCKPOLARPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKPOLARPARAMETER_set(self,BLOCKPOLARPARAMETER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKPOLARPARAMETER_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKPOLARPARAMETER_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKPOLARPARAMETER *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKPOLARPARAMETER *)(argp2);
    if (arg1) (arg1)->BLOCKPOLARPARAMETER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKPOLARPARAMETER_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKPOLARPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKPOLARPARAMETER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKPOLARPARAMETER_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKPOLARPARAMETER *) ((arg1)->BLOCKPOLARPARAMETER);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKPOLARSTRETCHACTION_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKPOLARSTRETCHACTION *arg2 = (Dwg_Object_BLOCKPOLARSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKPOLARSTRETCHACTION_set(self,BLOCKPOLARSTRETCHACTION);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKPOLARSTRETCHACTION_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKPOLARSTRETCHACTION_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKPOLARSTRETCHACTION *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKPOLARSTRETCHACTION *)(argp2);
    if (arg1) (arg1)->BLOCKPOLARSTRETCHACTION = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKPOLARSTRETCHACTION_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKPOLARSTRETCHACTION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKPOLARSTRETCHACTION_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKPOLARSTRETCHACTION_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKPOLARSTRETCHACTION *) ((arg1)->BLOCKPOLARSTRETCHACTION);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKRADIALCONSTRAINTPARAMETER_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER *arg2 = (Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKRADIALCONSTRAINTPARAMETER_set(self,BLOCKRADIALCONSTRAINTPARAMETER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKRADIALCONSTRAINTPARAMETER_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKRADIALCONSTRAINTPARAMETER_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER *)(argp2);
    if (arg1) (arg1)->BLOCKRADIALCONSTRAINTPARAMETER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKRADIALCONSTRAINTPARAMETER_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKRADIALCONSTRAINTPARAMETER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKRADIALCONSTRAINTPARAMETER_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER *) ((arg1)->BLOCKRADIALCONSTRAINTPARAMETER);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKREPRESENTATION_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKREPRESENTATION *arg2 = (Dwg_Object_BLOCKREPRESENTATION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKREPRESENTATION_set(self,BLOCKREPRESENTATION);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKREPRESENTATION_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKREPRESENTATION, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKREPRESENTATION_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKREPRESENTATION *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKREPRESENTATION *)(argp2);
    if (arg1) (arg1)->BLOCKREPRESENTATION = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKREPRESENTATION_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKREPRESENTATION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKREPRESENTATION_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKREPRESENTATION_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKREPRESENTATION *) ((arg1)->BLOCKREPRESENTATION);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKREPRESENTATION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKROTATIONPARAMETER_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKROTATIONPARAMETER *arg2 = (Dwg_Object_BLOCKROTATIONPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKROTATIONPARAMETER_set(self,BLOCKROTATIONPARAMETER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKROTATIONPARAMETER_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKROTATIONPARAMETER_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKROTATIONPARAMETER *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKROTATIONPARAMETER *)(argp2);
    if (arg1) (arg1)->BLOCKROTATIONPARAMETER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKROTATIONPARAMETER_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKROTATIONPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKROTATIONPARAMETER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKROTATIONPARAMETER_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKROTATIONPARAMETER *) ((arg1)->BLOCKROTATIONPARAMETER);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKSTRETCHACTION_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKSTRETCHACTION *arg2 = (Dwg_Object_BLOCKSTRETCHACTION *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKSTRETCHACTION_set(self,BLOCKSTRETCHACTION);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKSTRETCHACTION_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKSTRETCHACTION_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKSTRETCHACTION *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKSTRETCHACTION *)(argp2);
    if (arg1) (arg1)->BLOCKSTRETCHACTION = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKSTRETCHACTION_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKSTRETCHACTION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKSTRETCHACTION_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKSTRETCHACTION_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKSTRETCHACTION *) ((arg1)->BLOCKSTRETCHACTION);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKUSERPARAMETER_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKUSERPARAMETER *arg2 = (Dwg_Object_BLOCKUSERPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKUSERPARAMETER_set(self,BLOCKUSERPARAMETER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKUSERPARAMETER_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKUSERPARAMETER_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKUSERPARAMETER *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKUSERPARAMETER *)(argp2);
    if (arg1) (arg1)->BLOCKUSERPARAMETER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKUSERPARAMETER_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKUSERPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKUSERPARAMETER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKUSERPARAMETER_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKUSERPARAMETER *) ((arg1)->BLOCKUSERPARAMETER);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKVERTICALCONSTRAINTPARAMETER_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER *arg2 = (Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKVERTICALCONSTRAINTPARAMETER_set(self,BLOCKVERTICALCONSTRAINTPARAMETER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKVERTICALCONSTRAINTPARAMETER_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKVERTICALCONSTRAINTPARAMETER_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER *)(argp2);
    if (arg1) (arg1)->BLOCKVERTICALCONSTRAINTPARAMETER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKVERTICALCONSTRAINTPARAMETER_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKVERTICALCONSTRAINTPARAMETER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKVERTICALCONSTRAINTPARAMETER_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER *) ((arg1)->BLOCKVERTICALCONSTRAINTPARAMETER);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKVISIBILITYPARAMETER_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKVISIBILITYPARAMETER *arg2 = (Dwg_Object_BLOCKVISIBILITYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKVISIBILITYPARAMETER_set(self,BLOCKVISIBILITYPARAMETER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKVISIBILITYPARAMETER_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKVISIBILITYPARAMETER_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKVISIBILITYPARAMETER *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKVISIBILITYPARAMETER *)(argp2);
    if (arg1) (arg1)->BLOCKVISIBILITYPARAMETER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKVISIBILITYPARAMETER_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKVISIBILITYPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKVISIBILITYPARAMETER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKVISIBILITYPARAMETER_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKVISIBILITYPARAMETER *) ((arg1)->BLOCKVISIBILITYPARAMETER);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKXYGRIP_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKXYGRIP *arg2 = (Dwg_Object_BLOCKXYGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKXYGRIP_set(self,BLOCKXYGRIP);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKXYGRIP_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKXYGRIP, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKXYGRIP_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKXYGRIP *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKXYGRIP *)(argp2);
    if (arg1) (arg1)->BLOCKXYGRIP = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKXYGRIP_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKXYGRIP *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKXYGRIP_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKXYGRIP_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKXYGRIP *) ((arg1)->BLOCKXYGRIP);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKXYGRIP, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKXYPARAMETER_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKXYPARAMETER *arg2 = (Dwg_Object_BLOCKXYPARAMETER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKXYPARAMETER_set(self,BLOCKXYPARAMETER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKXYPARAMETER_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKXYPARAMETER_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKXYPARAMETER *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKXYPARAMETER *)(argp2);
    if (arg1) (arg1)->BLOCKXYPARAMETER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKXYPARAMETER_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKXYPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKXYPARAMETER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKXYPARAMETER_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKXYPARAMETER *) ((arg1)->BLOCKXYPARAMETER);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_DATALINK_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_DATALINK *arg2 = (Dwg_Object_DATALINK *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_DATALINK_set(self,DATALINK);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_DATALINK_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_DATALINK, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_DATALINK_set" "', argument " "2"" of type '" "Dwg_Object_DATALINK *""'"); 
    }
    arg2 = (Dwg_Object_DATALINK *)(argp2);
    if (arg1) (arg1)->DATALINK = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_DATALINK_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DATALINK *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_DATALINK_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_DATALINK_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_DATALINK *) ((arg1)->DATALINK);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DATALINK, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_DBCOLOR_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_DBCOLOR *arg2 = (Dwg_Object_DBCOLOR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_DBCOLOR_set(self,DBCOLOR);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_DBCOLOR_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_DBCOLOR, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_DBCOLOR_set" "', argument " "2"" of type '" "Dwg_Object_DBCOLOR *""'"); 
    }
    arg2 = (Dwg_Object_DBCOLOR *)(argp2);
    if (arg1) (arg1)->DBCOLOR = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_DBCOLOR_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DBCOLOR *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_DBCOLOR_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_DBCOLOR_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_DBCOLOR *) ((arg1)->DBCOLOR);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DBCOLOR, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_EVALUATION_GRAPH_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_EVALUATION_GRAPH *arg2 = (Dwg_Object_EVALUATION_GRAPH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_EVALUATION_GRAPH_set(self,EVALUATION_GRAPH);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_EVALUATION_GRAPH_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_EVALUATION_GRAPH, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_EVALUATION_GRAPH_set" "', argument " "2"" of type '" "Dwg_Object_EVALUATION_GRAPH *""'"); 
    }
    arg2 = (Dwg_Object_EVALUATION_GRAPH *)(argp2);
    if (arg1) (arg1)->EVALUATION_GRAPH = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_EVALUATION_GRAPH_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_EVALUATION_GRAPH *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_EVALUATION_GRAPH_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_EVALUATION_GRAPH_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_EVALUATION_GRAPH *) ((arg1)->EVALUATION_GRAPH);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_EVALUATION_GRAPH, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_FCFOBJECTCONTEXTDATA_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_FCFOBJECTCONTEXTDATA *arg2 = (Dwg_Object_FCFOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_FCFOBJECTCONTEXTDATA_set(self,FCFOBJECTCONTEXTDATA);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_FCFOBJECTCONTEXTDATA_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_FCFOBJECTCONTEXTDATA, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_FCFOBJECTCONTEXTDATA_set" "', argument " "2"" of type '" "Dwg_Object_FCFOBJECTCONTEXTDATA *""'"); 
    }
    arg2 = (Dwg_Object_FCFOBJECTCONTEXTDATA *)(argp2);
    if (arg1) (arg1)->FCFOBJECTCONTEXTDATA = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_FCFOBJECTCONTEXTDATA_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_FCFOBJECTCONTEXTDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_FCFOBJECTCONTEXTDATA_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_FCFOBJECTCONTEXTDATA_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_FCFOBJECTCONTEXTDATA *) ((arg1)->FCFOBJECTCONTEXTDATA);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_FCFOBJECTCONTEXTDATA, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_GRADIENT_BACKGROUND_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_GRADIENT_BACKGROUND *arg2 = (Dwg_Object_GRADIENT_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_GRADIENT_BACKGROUND_set(self,GRADIENT_BACKGROUND);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_GRADIENT_BACKGROUND_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_GRADIENT_BACKGROUND, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_GRADIENT_BACKGROUND_set" "', argument " "2"" of type '" "Dwg_Object_GRADIENT_BACKGROUND *""'"); 
    }
    arg2 = (Dwg_Object_GRADIENT_BACKGROUND *)(argp2);
    if (arg1) (arg1)->GRADIENT_BACKGROUND = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_GRADIENT_BACKGROUND_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_GRADIENT_BACKGROUND *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_GRADIENT_BACKGROUND_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_GRADIENT_BACKGROUND_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_GRADIENT_BACKGROUND *) ((arg1)->GRADIENT_BACKGROUND);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_GRADIENT_BACKGROUND, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_GROUND_PLANE_BACKGROUND_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_GROUND_PLANE_BACKGROUND *arg2 = (Dwg_Object_GROUND_PLANE_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_GROUND_PLANE_BACKGROUND_set(self,GROUND_PLANE_BACKGROUND);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_GROUND_PLANE_BACKGROUND_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_GROUND_PLANE_BACKGROUND, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_GROUND_PLANE_BACKGROUND_set" "', argument " "2"" of type '" "Dwg_Object_GROUND_PLANE_BACKGROUND *""'"); 
    }
    arg2 = (Dwg_Object_GROUND_PLANE_BACKGROUND *)(argp2);
    if (arg1) (arg1)->GROUND_PLANE_BACKGROUND = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_GROUND_PLANE_BACKGROUND_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_GROUND_PLANE_BACKGROUND *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_GROUND_PLANE_BACKGROUND_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_GROUND_PLANE_BACKGROUND_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_GROUND_PLANE_BACKGROUND *) ((arg1)->GROUND_PLANE_BACKGROUND);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_GROUND_PLANE_BACKGROUND, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_IBL_BACKGROUND_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_IBL_BACKGROUND *arg2 = (Dwg_Object_IBL_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_IBL_BACKGROUND_set(self,IBL_BACKGROUND);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_IBL_BACKGROUND_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_IBL_BACKGROUND, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_IBL_BACKGROUND_set" "', argument " "2"" of type '" "Dwg_Object_IBL_BACKGROUND *""'"); 
    }
    arg2 = (Dwg_Object_IBL_BACKGROUND *)(argp2);
    if (arg1) (arg1)->IBL_BACKGROUND = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_IBL_BACKGROUND_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_IBL_BACKGROUND *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_IBL_BACKGROUND_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_IBL_BACKGROUND_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_IBL_BACKGROUND *) ((arg1)->IBL_BACKGROUND);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_IBL_BACKGROUND, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_IMAGE_BACKGROUND_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_IMAGE_BACKGROUND *arg2 = (Dwg_Object_IMAGE_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_IMAGE_BACKGROUND_set(self,IMAGE_BACKGROUND);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_IMAGE_BACKGROUND_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_IMAGE_BACKGROUND, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_IMAGE_BACKGROUND_set" "', argument " "2"" of type '" "Dwg_Object_IMAGE_BACKGROUND *""'"); 
    }
    arg2 = (Dwg_Object_IMAGE_BACKGROUND *)(argp2);
    if (arg1) (arg1)->IMAGE_BACKGROUND = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_IMAGE_BACKGROUND_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_IMAGE_BACKGROUND *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_IMAGE_BACKGROUND_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_IMAGE_BACKGROUND_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_IMAGE_BACKGROUND *) ((arg1)->IMAGE_BACKGROUND);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_IMAGE_BACKGROUND, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_LEADEROBJECTCONTEXTDATA_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_LEADEROBJECTCONTEXTDATA *arg2 = (Dwg_Object_LEADEROBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_LEADEROBJECTCONTEXTDATA_set(self,LEADEROBJECTCONTEXTDATA);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_LEADEROBJECTCONTEXTDATA_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_LEADEROBJECTCONTEXTDATA, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_LEADEROBJECTCONTEXTDATA_set" "', argument " "2"" of type '" "Dwg_Object_LEADEROBJECTCONTEXTDATA *""'"); 
    }
    arg2 = (Dwg_Object_LEADEROBJECTCONTEXTDATA *)(argp2);
    if (arg1) (arg1)->LEADEROBJECTCONTEXTDATA = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_LEADEROBJECTCONTEXTDATA_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_LEADEROBJECTCONTEXTDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_LEADEROBJECTCONTEXTDATA_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_LEADEROBJECTCONTEXTDATA_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_LEADEROBJECTCONTEXTDATA *) ((arg1)->LEADEROBJECTCONTEXTDATA);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_LEADEROBJECTCONTEXTDATA, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_LIGHTLIST_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_LIGHTLIST *arg2 = (Dwg_Object_LIGHTLIST *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_LIGHTLIST_set(self,LIGHTLIST);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_LIGHTLIST_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_LIGHTLIST, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_LIGHTLIST_set" "', argument " "2"" of type '" "Dwg_Object_LIGHTLIST *""'"); 
    }
    arg2 = (Dwg_Object_LIGHTLIST *)(argp2);
    if (arg1) (arg1)->LIGHTLIST = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_LIGHTLIST_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_LIGHTLIST *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_LIGHTLIST_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_LIGHTLIST_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_LIGHTLIST *) ((arg1)->LIGHTLIST);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_LIGHTLIST, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_MATERIAL_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_MATERIAL *arg2 = (Dwg_Object_MATERIAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_MATERIAL_set(self,MATERIAL);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_MATERIAL_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_MATERIAL, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_MATERIAL_set" "', argument " "2"" of type '" "Dwg_Object_MATERIAL *""'"); 
    }
    arg2 = (Dwg_Object_MATERIAL *)(argp2);
    if (arg1) (arg1)->MATERIAL = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_MATERIAL_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_MATERIAL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_MATERIAL_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_MATERIAL_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_MATERIAL *) ((arg1)->MATERIAL);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_MATERIAL, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_MENTALRAYRENDERSETTINGS_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_MENTALRAYRENDERSETTINGS *arg2 = (Dwg_Object_MENTALRAYRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_MENTALRAYRENDERSETTINGS_set(self,MENTALRAYRENDERSETTINGS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_MENTALRAYRENDERSETTINGS_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_MENTALRAYRENDERSETTINGS_set" "', argument " "2"" of type '" "Dwg_Object_MENTALRAYRENDERSETTINGS *""'"); 
    }
    arg2 = (Dwg_Object_MENTALRAYRENDERSETTINGS *)(argp2);
    if (arg1) (arg1)->MENTALRAYRENDERSETTINGS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_MENTALRAYRENDERSETTINGS_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_MENTALRAYRENDERSETTINGS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_MENTALRAYRENDERSETTINGS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_MENTALRAYRENDERSETTINGS_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_MENTALRAYRENDERSETTINGS *) ((arg1)->MENTALRAYRENDERSETTINGS);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_MLEADERSTYLE_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_MLEADERSTYLE *arg2 = (Dwg_Object_MLEADERSTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_MLEADERSTYLE_set(self,MLEADERSTYLE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_MLEADERSTYLE_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_MLEADERSTYLE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_MLEADERSTYLE_set" "', argument " "2"" of type '" "Dwg_Object_MLEADERSTYLE *""'"); 
    }
    arg2 = (Dwg_Object_MLEADERSTYLE *)(argp2);
    if (arg1) (arg1)->MLEADERSTYLE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_MLEADERSTYLE_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_MLEADERSTYLE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_MLEADERSTYLE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_MLEADERSTYLE_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_MLEADERSTYLE *) ((arg1)->MLEADERSTYLE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_MTEXTOBJECTCONTEXTDATA_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_MTEXTOBJECTCONTEXTDATA *arg2 = (Dwg_Object_MTEXTOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_MTEXTOBJECTCONTEXTDATA_set(self,MTEXTOBJECTCONTEXTDATA);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_MTEXTOBJECTCONTEXTDATA_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_MTEXTOBJECTCONTEXTDATA_set" "', argument " "2"" of type '" "Dwg_Object_MTEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg2 = (Dwg_Object_MTEXTOBJECTCONTEXTDATA *)(argp2);
    if (arg1) (arg1)->MTEXTOBJECTCONTEXTDATA = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_MTEXTOBJECTCONTEXTDATA_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_MTEXTOBJECTCONTEXTDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_MTEXTOBJECTCONTEXTDATA_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_MTEXTOBJECTCONTEXTDATA_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_MTEXTOBJECTCONTEXTDATA *) ((arg1)->MTEXTOBJECTCONTEXTDATA);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_OBJECT_PTR_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_OBJECT_PTR *arg2 = (Dwg_Object_OBJECT_PTR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_OBJECT_PTR_set(self,OBJECT_PTR);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_OBJECT_PTR_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_OBJECT_PTR, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_OBJECT_PTR_set" "', argument " "2"" of type '" "Dwg_Object_OBJECT_PTR *""'"); 
    }
    arg2 = (Dwg_Object_OBJECT_PTR *)(argp2);
    if (arg1) (arg1)->OBJECT_PTR = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_OBJECT_PTR_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_OBJECT_PTR *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_OBJECT_PTR_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_OBJECT_PTR_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_OBJECT_PTR *) ((arg1)->OBJECT_PTR);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_OBJECT_PTR, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_PARTIAL_VIEWING_INDEX_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_PARTIAL_VIEWING_INDEX *arg2 = (Dwg_Object_PARTIAL_VIEWING_INDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_PARTIAL_VIEWING_INDEX_set(self,PARTIAL_VIEWING_INDEX);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_PARTIAL_VIEWING_INDEX_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_PARTIAL_VIEWING_INDEX, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_PARTIAL_VIEWING_INDEX_set" "', argument " "2"" of type '" "Dwg_Object_PARTIAL_VIEWING_INDEX *""'"); 
    }
    arg2 = (Dwg_Object_PARTIAL_VIEWING_INDEX *)(argp2);
    if (arg1) (arg1)->PARTIAL_VIEWING_INDEX = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_PARTIAL_VIEWING_INDEX_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_PARTIAL_VIEWING_INDEX *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_PARTIAL_VIEWING_INDEX_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_PARTIAL_VIEWING_INDEX_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_PARTIAL_VIEWING_INDEX *) ((arg1)->PARTIAL_VIEWING_INDEX);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_PARTIAL_VIEWING_INDEX, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_POINTCLOUDCOLORMAP_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_POINTCLOUDCOLORMAP *arg2 = (Dwg_Object_POINTCLOUDCOLORMAP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_POINTCLOUDCOLORMAP_set(self,POINTCLOUDCOLORMAP);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_POINTCLOUDCOLORMAP_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_POINTCLOUDCOLORMAP, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_POINTCLOUDCOLORMAP_set" "', argument " "2"" of type '" "Dwg_Object_POINTCLOUDCOLORMAP *""'"); 
    }
    arg2 = (Dwg_Object_POINTCLOUDCOLORMAP *)(argp2);
    if (arg1) (arg1)->POINTCLOUDCOLORMAP = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_POINTCLOUDCOLORMAP_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_POINTCLOUDCOLORMAP *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_POINTCLOUDCOLORMAP_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_POINTCLOUDCOLORMAP_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_POINTCLOUDCOLORMAP *) ((arg1)->POINTCLOUDCOLORMAP);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_POINTCLOUDCOLORMAP, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_POINTCLOUDDEF_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_POINTCLOUDDEF *arg2 = (Dwg_Object_POINTCLOUDDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_POINTCLOUDDEF_set(self,POINTCLOUDDEF);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_POINTCLOUDDEF_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_POINTCLOUDDEF, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_POINTCLOUDDEF_set" "', argument " "2"" of type '" "Dwg_Object_POINTCLOUDDEF *""'"); 
    }
    arg2 = (Dwg_Object_POINTCLOUDDEF *)(argp2);
    if (arg1) (arg1)->POINTCLOUDDEF = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_POINTCLOUDDEF_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_POINTCLOUDDEF *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_POINTCLOUDDEF_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_POINTCLOUDDEF_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_POINTCLOUDDEF *) ((arg1)->POINTCLOUDDEF);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_POINTCLOUDDEF, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_POINTCLOUDDEFEX_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_POINTCLOUDDEFEX *arg2 = (Dwg_Object_POINTCLOUDDEFEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_POINTCLOUDDEFEX_set(self,POINTCLOUDDEFEX);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_POINTCLOUDDEFEX_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_POINTCLOUDDEFEX, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_POINTCLOUDDEFEX_set" "', argument " "2"" of type '" "Dwg_Object_POINTCLOUDDEFEX *""'"); 
    }
    arg2 = (Dwg_Object_POINTCLOUDDEFEX *)(argp2);
    if (arg1) (arg1)->POINTCLOUDDEFEX = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_POINTCLOUDDEFEX_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_POINTCLOUDDEFEX *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_POINTCLOUDDEFEX_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_POINTCLOUDDEFEX_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_POINTCLOUDDEFEX *) ((arg1)->POINTCLOUDDEFEX);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_POINTCLOUDDEFEX, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_POINTCLOUDDEF_REACTOR_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_POINTCLOUDDEF_REACTOR *arg2 = (Dwg_Object_POINTCLOUDDEF_REACTOR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_POINTCLOUDDEF_REACTOR_set(self,POINTCLOUDDEF_REACTOR);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_POINTCLOUDDEF_REACTOR_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_POINTCLOUDDEF_REACTOR, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_POINTCLOUDDEF_REACTOR_set" "', argument " "2"" of type '" "Dwg_Object_POINTCLOUDDEF_REACTOR *""'"); 
    }
    arg2 = (Dwg_Object_POINTCLOUDDEF_REACTOR *)(argp2);
    if (arg1) (arg1)->POINTCLOUDDEF_REACTOR = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_POINTCLOUDDEF_REACTOR_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_POINTCLOUDDEF_REACTOR *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_POINTCLOUDDEF_REACTOR_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_POINTCLOUDDEF_REACTOR_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_POINTCLOUDDEF_REACTOR *) ((arg1)->POINTCLOUDDEF_REACTOR);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_POINTCLOUDDEF_REACTOR, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_POINTCLOUDDEF_REACTOR_EX_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_POINTCLOUDDEF_REACTOR_EX *arg2 = (Dwg_Object_POINTCLOUDDEF_REACTOR_EX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_POINTCLOUDDEF_REACTOR_EX_set(self,POINTCLOUDDEF_REACTOR_EX);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_POINTCLOUDDEF_REACTOR_EX_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_POINTCLOUDDEF_REACTOR_EX, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_POINTCLOUDDEF_REACTOR_EX_set" "', argument " "2"" of type '" "Dwg_Object_POINTCLOUDDEF_REACTOR_EX *""'"); 
    }
    arg2 = (Dwg_Object_POINTCLOUDDEF_REACTOR_EX *)(argp2);
    if (arg1) (arg1)->POINTCLOUDDEF_REACTOR_EX = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_POINTCLOUDDEF_REACTOR_EX_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_POINTCLOUDDEF_REACTOR_EX *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_POINTCLOUDDEF_REACTOR_EX_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_POINTCLOUDDEF_REACTOR_EX_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_POINTCLOUDDEF_REACTOR_EX *) ((arg1)->POINTCLOUDDEF_REACTOR_EX);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_POINTCLOUDDEF_REACTOR_EX, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_PROXY_OBJECT_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_PROXY_OBJECT *arg2 = (Dwg_Object_PROXY_OBJECT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_PROXY_OBJECT_set(self,PROXY_OBJECT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_PROXY_OBJECT_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_PROXY_OBJECT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_PROXY_OBJECT_set" "', argument " "2"" of type '" "Dwg_Object_PROXY_OBJECT *""'"); 
    }
    arg2 = (Dwg_Object_PROXY_OBJECT *)(argp2);
    if (arg1) (arg1)->PROXY_OBJECT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_PROXY_OBJECT_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_PROXY_OBJECT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_PROXY_OBJECT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_PROXY_OBJECT_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_PROXY_OBJECT *) ((arg1)->PROXY_OBJECT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_PROXY_OBJECT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_RAPIDRTRENDERSETTINGS_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_RAPIDRTRENDERSETTINGS *arg2 = (Dwg_Object_RAPIDRTRENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_RAPIDRTRENDERSETTINGS_set(self,RAPIDRTRENDERSETTINGS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_RAPIDRTRENDERSETTINGS_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_RAPIDRTRENDERSETTINGS_set" "', argument " "2"" of type '" "Dwg_Object_RAPIDRTRENDERSETTINGS *""'"); 
    }
    arg2 = (Dwg_Object_RAPIDRTRENDERSETTINGS *)(argp2);
    if (arg1) (arg1)->RAPIDRTRENDERSETTINGS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_RAPIDRTRENDERSETTINGS_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_RAPIDRTRENDERSETTINGS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_RAPIDRTRENDERSETTINGS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_RAPIDRTRENDERSETTINGS_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_RAPIDRTRENDERSETTINGS *) ((arg1)->RAPIDRTRENDERSETTINGS);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_RENDERENTRY_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_RENDERENTRY *arg2 = (Dwg_Object_RENDERENTRY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_RENDERENTRY_set(self,RENDERENTRY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_RENDERENTRY_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_RENDERENTRY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_RENDERENTRY_set" "', argument " "2"" of type '" "Dwg_Object_RENDERENTRY *""'"); 
    }
    arg2 = (Dwg_Object_RENDERENTRY *)(argp2);
    if (arg1) (arg1)->RENDERENTRY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_RENDERENTRY_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_RENDERENTRY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_RENDERENTRY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_RENDERENTRY_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_RENDERENTRY *) ((arg1)->RENDERENTRY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_RENDERENTRY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_RENDERGLOBAL_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_RENDERGLOBAL *arg2 = (Dwg_Object_RENDERGLOBAL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_RENDERGLOBAL_set(self,RENDERGLOBAL);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_RENDERGLOBAL_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_RENDERGLOBAL, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_RENDERGLOBAL_set" "', argument " "2"" of type '" "Dwg_Object_RENDERGLOBAL *""'"); 
    }
    arg2 = (Dwg_Object_RENDERGLOBAL *)(argp2);
    if (arg1) (arg1)->RENDERGLOBAL = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_RENDERGLOBAL_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_RENDERGLOBAL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_RENDERGLOBAL_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_RENDERGLOBAL_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_RENDERGLOBAL *) ((arg1)->RENDERGLOBAL);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_RENDERGLOBAL, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_RENDERSETTINGS_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_RENDERSETTINGS *arg2 = (Dwg_Object_RENDERSETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_RENDERSETTINGS_set(self,RENDERSETTINGS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_RENDERSETTINGS_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_RENDERSETTINGS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_RENDERSETTINGS_set" "', argument " "2"" of type '" "Dwg_Object_RENDERSETTINGS *""'"); 
    }
    arg2 = (Dwg_Object_RENDERSETTINGS *)(argp2);
    if (arg1) (arg1)->RENDERSETTINGS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_RENDERSETTINGS_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_RENDERSETTINGS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_RENDERSETTINGS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_RENDERSETTINGS_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_RENDERSETTINGS *) ((arg1)->RENDERSETTINGS);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_RENDERSETTINGS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_SECTION_SETTINGS_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_SECTION_SETTINGS *arg2 = (Dwg_Object_SECTION_SETTINGS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_SECTION_SETTINGS_set(self,SECTION_SETTINGS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_SECTION_SETTINGS_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_SECTION_SETTINGS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_SECTION_SETTINGS_set" "', argument " "2"" of type '" "Dwg_Object_SECTION_SETTINGS *""'"); 
    }
    arg2 = (Dwg_Object_SECTION_SETTINGS *)(argp2);
    if (arg1) (arg1)->SECTION_SETTINGS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_SECTION_SETTINGS_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_SECTION_SETTINGS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_SECTION_SETTINGS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_SECTION_SETTINGS_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_SECTION_SETTINGS *) ((arg1)->SECTION_SETTINGS);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_SECTION_SETTINGS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_SKYLIGHT_BACKGROUND_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_SKYLIGHT_BACKGROUND *arg2 = (Dwg_Object_SKYLIGHT_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_SKYLIGHT_BACKGROUND_set(self,SKYLIGHT_BACKGROUND);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_SKYLIGHT_BACKGROUND_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_SKYLIGHT_BACKGROUND, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_SKYLIGHT_BACKGROUND_set" "', argument " "2"" of type '" "Dwg_Object_SKYLIGHT_BACKGROUND *""'"); 
    }
    arg2 = (Dwg_Object_SKYLIGHT_BACKGROUND *)(argp2);
    if (arg1) (arg1)->SKYLIGHT_BACKGROUND = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_SKYLIGHT_BACKGROUND_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_SKYLIGHT_BACKGROUND *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_SKYLIGHT_BACKGROUND_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_SKYLIGHT_BACKGROUND_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_SKYLIGHT_BACKGROUND *) ((arg1)->SKYLIGHT_BACKGROUND);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_SKYLIGHT_BACKGROUND, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_SOLID_BACKGROUND_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_SOLID_BACKGROUND *arg2 = (Dwg_Object_SOLID_BACKGROUND *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_SOLID_BACKGROUND_set(self,SOLID_BACKGROUND);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_SOLID_BACKGROUND_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_SOLID_BACKGROUND, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_SOLID_BACKGROUND_set" "', argument " "2"" of type '" "Dwg_Object_SOLID_BACKGROUND *""'"); 
    }
    arg2 = (Dwg_Object_SOLID_BACKGROUND *)(argp2);
    if (arg1) (arg1)->SOLID_BACKGROUND = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_SOLID_BACKGROUND_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_SOLID_BACKGROUND *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_SOLID_BACKGROUND_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_SOLID_BACKGROUND_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_SOLID_BACKGROUND *) ((arg1)->SOLID_BACKGROUND);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_SOLID_BACKGROUND, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_SPATIAL_INDEX_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_SPATIAL_INDEX *arg2 = (Dwg_Object_SPATIAL_INDEX *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_SPATIAL_INDEX_set(self,SPATIAL_INDEX);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_SPATIAL_INDEX_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_SPATIAL_INDEX, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_SPATIAL_INDEX_set" "', argument " "2"" of type '" "Dwg_Object_SPATIAL_INDEX *""'"); 
    }
    arg2 = (Dwg_Object_SPATIAL_INDEX *)(argp2);
    if (arg1) (arg1)->SPATIAL_INDEX = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_SPATIAL_INDEX_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_SPATIAL_INDEX *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_SPATIAL_INDEX_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_SPATIAL_INDEX_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_SPATIAL_INDEX *) ((arg1)->SPATIAL_INDEX);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_SPATIAL_INDEX, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_TABLESTYLE_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_TABLESTYLE *arg2 = (Dwg_Object_TABLESTYLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_TABLESTYLE_set(self,TABLESTYLE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_TABLESTYLE_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_TABLESTYLE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_TABLESTYLE_set" "', argument " "2"" of type '" "Dwg_Object_TABLESTYLE *""'"); 
    }
    arg2 = (Dwg_Object_TABLESTYLE *)(argp2);
    if (arg1) (arg1)->TABLESTYLE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_TABLESTYLE_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_TABLESTYLE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_TABLESTYLE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_TABLESTYLE_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_TABLESTYLE *) ((arg1)->TABLESTYLE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_TABLESTYLE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_TEXTOBJECTCONTEXTDATA_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_TEXTOBJECTCONTEXTDATA *arg2 = (Dwg_Object_TEXTOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_TEXTOBJECTCONTEXTDATA_set(self,TEXTOBJECTCONTEXTDATA);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_TEXTOBJECTCONTEXTDATA_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_TEXTOBJECTCONTEXTDATA, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_TEXTOBJECTCONTEXTDATA_set" "', argument " "2"" of type '" "Dwg_Object_TEXTOBJECTCONTEXTDATA *""'"); 
    }
    arg2 = (Dwg_Object_TEXTOBJECTCONTEXTDATA *)(argp2);
    if (arg1) (arg1)->TEXTOBJECTCONTEXTDATA = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_TEXTOBJECTCONTEXTDATA_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_TEXTOBJECTCONTEXTDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_TEXTOBJECTCONTEXTDATA_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_TEXTOBJECTCONTEXTDATA_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_TEXTOBJECTCONTEXTDATA *) ((arg1)->TEXTOBJECTCONTEXTDATA);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_TEXTOBJECTCONTEXTDATA, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCARRAYMODIFYPARAMETERS_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCARRAYMODIFYPARAMETERS *arg2 = (Dwg_Object_ASSOCARRAYMODIFYPARAMETERS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCARRAYMODIFYPARAMETERS_set(self,ASSOCARRAYMODIFYPARAMETERS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCARRAYMODIFYPARAMETERS_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCARRAYMODIFYPARAMETERS_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCARRAYMODIFYPARAMETERS *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCARRAYMODIFYPARAMETERS *)(argp2);
    if (arg1) (arg1)->ASSOCARRAYMODIFYPARAMETERS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCARRAYMODIFYPARAMETERS_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCARRAYMODIFYPARAMETERS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCARRAYMODIFYPARAMETERS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCARRAYMODIFYPARAMETERS_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCARRAYMODIFYPARAMETERS *) ((arg1)->ASSOCARRAYMODIFYPARAMETERS);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCARRAYPATHPARAMETERS_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCARRAYPATHPARAMETERS *arg2 = (Dwg_Object_ASSOCARRAYPATHPARAMETERS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCARRAYPATHPARAMETERS_set(self,ASSOCARRAYPATHPARAMETERS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCARRAYPATHPARAMETERS_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCARRAYPATHPARAMETERS_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCARRAYPATHPARAMETERS *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCARRAYPATHPARAMETERS *)(argp2);
    if (arg1) (arg1)->ASSOCARRAYPATHPARAMETERS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCARRAYPATHPARAMETERS_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCARRAYPATHPARAMETERS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCARRAYPATHPARAMETERS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCARRAYPATHPARAMETERS_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCARRAYPATHPARAMETERS *) ((arg1)->ASSOCARRAYPATHPARAMETERS);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCARRAYPOLARPARAMETERS_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCARRAYPOLARPARAMETERS *arg2 = (Dwg_Object_ASSOCARRAYPOLARPARAMETERS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCARRAYPOLARPARAMETERS_set(self,ASSOCARRAYPOLARPARAMETERS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCARRAYPOLARPARAMETERS_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCARRAYPOLARPARAMETERS_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCARRAYPOLARPARAMETERS *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCARRAYPOLARPARAMETERS *)(argp2);
    if (arg1) (arg1)->ASSOCARRAYPOLARPARAMETERS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCARRAYPOLARPARAMETERS_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCARRAYPOLARPARAMETERS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCARRAYPOLARPARAMETERS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCARRAYPOLARPARAMETERS_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCARRAYPOLARPARAMETERS *) ((arg1)->ASSOCARRAYPOLARPARAMETERS);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCARRAYRECTANGULARPARAMETERS_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCARRAYRECTANGULARPARAMETERS *arg2 = (Dwg_Object_ASSOCARRAYRECTANGULARPARAMETERS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCARRAYRECTANGULARPARAMETERS_set(self,ASSOCARRAYRECTANGULARPARAMETERS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCARRAYRECTANGULARPARAMETERS_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCARRAYRECTANGULARPARAMETERS_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCARRAYRECTANGULARPARAMETERS *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCARRAYRECTANGULARPARAMETERS *)(argp2);
    if (arg1) (arg1)->ASSOCARRAYRECTANGULARPARAMETERS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCARRAYRECTANGULARPARAMETERS_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCARRAYRECTANGULARPARAMETERS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCARRAYRECTANGULARPARAMETERS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCARRAYRECTANGULARPARAMETERS_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCARRAYRECTANGULARPARAMETERS *) ((arg1)->ASSOCARRAYRECTANGULARPARAMETERS);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACMECOMMANDHISTORY_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ACMECOMMANDHISTORY *arg2 = (Dwg_Object_ACMECOMMANDHISTORY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACMECOMMANDHISTORY_set(self,ACMECOMMANDHISTORY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACMECOMMANDHISTORY_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ACMECOMMANDHISTORY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ACMECOMMANDHISTORY_set" "', argument " "2"" of type '" "Dwg_Object_ACMECOMMANDHISTORY *""'"); 
    }
    arg2 = (Dwg_Object_ACMECOMMANDHISTORY *)(argp2);
    if (arg1) (arg1)->ACMECOMMANDHISTORY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACMECOMMANDHISTORY_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACMECOMMANDHISTORY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACMECOMMANDHISTORY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACMECOMMANDHISTORY_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ACMECOMMANDHISTORY *) ((arg1)->ACMECOMMANDHISTORY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACMECOMMANDHISTORY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACMESCOPE_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ACMESCOPE *arg2 = (Dwg_Object_ACMESCOPE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACMESCOPE_set(self,ACMESCOPE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACMESCOPE_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ACMESCOPE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ACMESCOPE_set" "', argument " "2"" of type '" "Dwg_Object_ACMESCOPE *""'"); 
    }
    arg2 = (Dwg_Object_ACMESCOPE *)(argp2);
    if (arg1) (arg1)->ACMESCOPE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACMESCOPE_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACMESCOPE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACMESCOPE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACMESCOPE_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ACMESCOPE *) ((arg1)->ACMESCOPE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACMESCOPE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACMESTATEMGR_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ACMESTATEMGR *arg2 = (Dwg_Object_ACMESTATEMGR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACMESTATEMGR_set(self,ACMESTATEMGR);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACMESTATEMGR_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ACMESTATEMGR, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ACMESTATEMGR_set" "', argument " "2"" of type '" "Dwg_Object_ACMESTATEMGR *""'"); 
    }
    arg2 = (Dwg_Object_ACMESTATEMGR *)(argp2);
    if (arg1) (arg1)->ACMESTATEMGR = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACMESTATEMGR_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACMESTATEMGR *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACMESTATEMGR_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACMESTATEMGR_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ACMESTATEMGR *) ((arg1)->ACMESTATEMGR);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACMESTATEMGR, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACSH_EXTRUSION_CLASS_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ACSH_EXTRUSION_CLASS *arg2 = (Dwg_Object_ACSH_EXTRUSION_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACSH_EXTRUSION_CLASS_set(self,ACSH_EXTRUSION_CLASS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACSH_EXTRUSION_CLASS_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ACSH_EXTRUSION_CLASS_set" "', argument " "2"" of type '" "Dwg_Object_ACSH_EXTRUSION_CLASS *""'"); 
    }
    arg2 = (Dwg_Object_ACSH_EXTRUSION_CLASS *)(argp2);
    if (arg1) (arg1)->ACSH_EXTRUSION_CLASS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACSH_EXTRUSION_CLASS_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_EXTRUSION_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACSH_EXTRUSION_CLASS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACSH_EXTRUSION_CLASS_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ACSH_EXTRUSION_CLASS *) ((arg1)->ACSH_EXTRUSION_CLASS);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACSH_LOFT_CLASS_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ACSH_LOFT_CLASS *arg2 = (Dwg_Object_ACSH_LOFT_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACSH_LOFT_CLASS_set(self,ACSH_LOFT_CLASS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACSH_LOFT_CLASS_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ACSH_LOFT_CLASS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ACSH_LOFT_CLASS_set" "', argument " "2"" of type '" "Dwg_Object_ACSH_LOFT_CLASS *""'"); 
    }
    arg2 = (Dwg_Object_ACSH_LOFT_CLASS *)(argp2);
    if (arg1) (arg1)->ACSH_LOFT_CLASS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACSH_LOFT_CLASS_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_LOFT_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACSH_LOFT_CLASS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACSH_LOFT_CLASS_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ACSH_LOFT_CLASS *) ((arg1)->ACSH_LOFT_CLASS);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_LOFT_CLASS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACSH_REVOLVE_CLASS_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ACSH_REVOLVE_CLASS *arg2 = (Dwg_Object_ACSH_REVOLVE_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACSH_REVOLVE_CLASS_set(self,ACSH_REVOLVE_CLASS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACSH_REVOLVE_CLASS_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ACSH_REVOLVE_CLASS_set" "', argument " "2"" of type '" "Dwg_Object_ACSH_REVOLVE_CLASS *""'"); 
    }
    arg2 = (Dwg_Object_ACSH_REVOLVE_CLASS *)(argp2);
    if (arg1) (arg1)->ACSH_REVOLVE_CLASS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACSH_REVOLVE_CLASS_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_REVOLVE_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACSH_REVOLVE_CLASS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACSH_REVOLVE_CLASS_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ACSH_REVOLVE_CLASS *) ((arg1)->ACSH_REVOLVE_CLASS);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACSH_SWEEP_CLASS_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ACSH_SWEEP_CLASS *arg2 = (Dwg_Object_ACSH_SWEEP_CLASS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACSH_SWEEP_CLASS_set(self,ACSH_SWEEP_CLASS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACSH_SWEEP_CLASS_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ACSH_SWEEP_CLASS_set" "', argument " "2"" of type '" "Dwg_Object_ACSH_SWEEP_CLASS *""'"); 
    }
    arg2 = (Dwg_Object_ACSH_SWEEP_CLASS *)(argp2);
    if (arg1) (arg1)->ACSH_SWEEP_CLASS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ACSH_SWEEP_CLASS_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_SWEEP_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ACSH_SWEEP_CLASS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ACSH_SWEEP_CLASS_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ACSH_SWEEP_CLASS *) ((arg1)->ACSH_SWEEP_CLASS);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ANGDIMOBJECTCONTEXTDATA_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ANGDIMOBJECTCONTEXTDATA *arg2 = (Dwg_Object_ANGDIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ANGDIMOBJECTCONTEXTDATA_set(self,ANGDIMOBJECTCONTEXTDATA);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ANGDIMOBJECTCONTEXTDATA_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ANGDIMOBJECTCONTEXTDATA, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ANGDIMOBJECTCONTEXTDATA_set" "', argument " "2"" of type '" "Dwg_Object_ANGDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg2 = (Dwg_Object_ANGDIMOBJECTCONTEXTDATA *)(argp2);
    if (arg1) (arg1)->ANGDIMOBJECTCONTEXTDATA = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ANGDIMOBJECTCONTEXTDATA_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ANGDIMOBJECTCONTEXTDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ANGDIMOBJECTCONTEXTDATA_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ANGDIMOBJECTCONTEXTDATA_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ANGDIMOBJECTCONTEXTDATA *) ((arg1)->ANGDIMOBJECTCONTEXTDATA);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ANGDIMOBJECTCONTEXTDATA, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ANNOTSCALEOBJECTCONTEXTDATA_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA *arg2 = (Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ANNOTSCALEOBJECTCONTEXTDATA_set(self,ANNOTSCALEOBJECTCONTEXTDATA);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ANNOTSCALEOBJECTCONTEXTDATA_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ANNOTSCALEOBJECTCONTEXTDATA, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ANNOTSCALEOBJECTCONTEXTDATA_set" "', argument " "2"" of type '" "Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA *""'"); 
    }
    arg2 = (Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA *)(argp2);
    if (arg1) (arg1)->ANNOTSCALEOBJECTCONTEXTDATA = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ANNOTSCALEOBJECTCONTEXTDATA_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ANNOTSCALEOBJECTCONTEXTDATA_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ANNOTSCALEOBJECTCONTEXTDATA_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA *) ((arg1)->ANNOTSCALEOBJECTCONTEXTDATA);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ANNOTSCALEOBJECTCONTEXTDATA, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOC3POINTANGULARDIMACTIONBODY_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY *arg2 = (Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOC3POINTANGULARDIMACTIONBODY_set(self,ASSOC3POINTANGULARDIMACTIONBODY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOC3POINTANGULARDIMACTIONBODY_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOC3POINTANGULARDIMACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOC3POINTANGULARDIMACTIONBODY_set" "', argument " "2"" of type '" "Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY *""'"); 
    }
    arg2 = (Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY *)(argp2);
    if (arg1) (arg1)->ASSOC3POINTANGULARDIMACTIONBODY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOC3POINTANGULARDIMACTIONBODY_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOC3POINTANGULARDIMACTIONBODY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOC3POINTANGULARDIMACTIONBODY_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY *) ((arg1)->ASSOC3POINTANGULARDIMACTIONBODY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOC3POINTANGULARDIMACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCALIGNEDDIMACTIONBODY_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCALIGNEDDIMACTIONBODY *arg2 = (Dwg_Object_ASSOCALIGNEDDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCALIGNEDDIMACTIONBODY_set(self,ASSOCALIGNEDDIMACTIONBODY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCALIGNEDDIMACTIONBODY_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCALIGNEDDIMACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCALIGNEDDIMACTIONBODY_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCALIGNEDDIMACTIONBODY *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCALIGNEDDIMACTIONBODY *)(argp2);
    if (arg1) (arg1)->ASSOCALIGNEDDIMACTIONBODY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCALIGNEDDIMACTIONBODY_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCALIGNEDDIMACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCALIGNEDDIMACTIONBODY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCALIGNEDDIMACTIONBODY_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCALIGNEDDIMACTIONBODY *) ((arg1)->ASSOCALIGNEDDIMACTIONBODY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCALIGNEDDIMACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCARRAYMODIFYACTIONBODY_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCARRAYMODIFYACTIONBODY *arg2 = (Dwg_Object_ASSOCARRAYMODIFYACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCARRAYMODIFYACTIONBODY_set(self,ASSOCARRAYMODIFYACTIONBODY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCARRAYMODIFYACTIONBODY_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCARRAYMODIFYACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCARRAYMODIFYACTIONBODY_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCARRAYMODIFYACTIONBODY *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCARRAYMODIFYACTIONBODY *)(argp2);
    if (arg1) (arg1)->ASSOCARRAYMODIFYACTIONBODY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCARRAYMODIFYACTIONBODY_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCARRAYMODIFYACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCARRAYMODIFYACTIONBODY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCARRAYMODIFYACTIONBODY_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCARRAYMODIFYACTIONBODY *) ((arg1)->ASSOCARRAYMODIFYACTIONBODY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCARRAYMODIFYACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCEDGEACTIONPARAM_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCEDGEACTIONPARAM *arg2 = (Dwg_Object_ASSOCEDGEACTIONPARAM *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCEDGEACTIONPARAM_set(self,ASSOCEDGEACTIONPARAM);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCEDGEACTIONPARAM_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCEDGEACTIONPARAM, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCEDGEACTIONPARAM_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCEDGEACTIONPARAM *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCEDGEACTIONPARAM *)(argp2);
    if (arg1) (arg1)->ASSOCEDGEACTIONPARAM = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCEDGEACTIONPARAM_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCEDGEACTIONPARAM *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCEDGEACTIONPARAM_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCEDGEACTIONPARAM_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCEDGEACTIONPARAM *) ((arg1)->ASSOCEDGEACTIONPARAM);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCEDGEACTIONPARAM, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCEDGECHAMFERACTIONBODY_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCEDGECHAMFERACTIONBODY *arg2 = (Dwg_Object_ASSOCEDGECHAMFERACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCEDGECHAMFERACTIONBODY_set(self,ASSOCEDGECHAMFERACTIONBODY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCEDGECHAMFERACTIONBODY_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCEDGECHAMFERACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCEDGECHAMFERACTIONBODY_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCEDGECHAMFERACTIONBODY *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCEDGECHAMFERACTIONBODY *)(argp2);
    if (arg1) (arg1)->ASSOCEDGECHAMFERACTIONBODY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCEDGECHAMFERACTIONBODY_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCEDGECHAMFERACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCEDGECHAMFERACTIONBODY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCEDGECHAMFERACTIONBODY_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCEDGECHAMFERACTIONBODY *) ((arg1)->ASSOCEDGECHAMFERACTIONBODY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCEDGECHAMFERACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCEDGEFILLETACTIONBODY_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCEDGEFILLETACTIONBODY *arg2 = (Dwg_Object_ASSOCEDGEFILLETACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCEDGEFILLETACTIONBODY_set(self,ASSOCEDGEFILLETACTIONBODY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCEDGEFILLETACTIONBODY_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCEDGEFILLETACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCEDGEFILLETACTIONBODY_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCEDGEFILLETACTIONBODY *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCEDGEFILLETACTIONBODY *)(argp2);
    if (arg1) (arg1)->ASSOCEDGEFILLETACTIONBODY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCEDGEFILLETACTIONBODY_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCEDGEFILLETACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCEDGEFILLETACTIONBODY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCEDGEFILLETACTIONBODY_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCEDGEFILLETACTIONBODY *) ((arg1)->ASSOCEDGEFILLETACTIONBODY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCEDGEFILLETACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCMLEADERACTIONBODY_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCMLEADERACTIONBODY *arg2 = (Dwg_Object_ASSOCMLEADERACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCMLEADERACTIONBODY_set(self,ASSOCMLEADERACTIONBODY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCMLEADERACTIONBODY_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCMLEADERACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCMLEADERACTIONBODY_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCMLEADERACTIONBODY *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCMLEADERACTIONBODY *)(argp2);
    if (arg1) (arg1)->ASSOCMLEADERACTIONBODY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCMLEADERACTIONBODY_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCMLEADERACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCMLEADERACTIONBODY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCMLEADERACTIONBODY_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCMLEADERACTIONBODY *) ((arg1)->ASSOCMLEADERACTIONBODY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCMLEADERACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCORDINATEDIMACTIONBODY_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCORDINATEDIMACTIONBODY *arg2 = (Dwg_Object_ASSOCORDINATEDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCORDINATEDIMACTIONBODY_set(self,ASSOCORDINATEDIMACTIONBODY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCORDINATEDIMACTIONBODY_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCORDINATEDIMACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCORDINATEDIMACTIONBODY_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCORDINATEDIMACTIONBODY *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCORDINATEDIMACTIONBODY *)(argp2);
    if (arg1) (arg1)->ASSOCORDINATEDIMACTIONBODY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCORDINATEDIMACTIONBODY_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCORDINATEDIMACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCORDINATEDIMACTIONBODY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCORDINATEDIMACTIONBODY_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCORDINATEDIMACTIONBODY *) ((arg1)->ASSOCORDINATEDIMACTIONBODY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCORDINATEDIMACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCPERSSUBENTMANAGER_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCPERSSUBENTMANAGER *arg2 = (Dwg_Object_ASSOCPERSSUBENTMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCPERSSUBENTMANAGER_set(self,ASSOCPERSSUBENTMANAGER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCPERSSUBENTMANAGER_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCPERSSUBENTMANAGER_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCPERSSUBENTMANAGER *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCPERSSUBENTMANAGER *)(argp2);
    if (arg1) (arg1)->ASSOCPERSSUBENTMANAGER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCPERSSUBENTMANAGER_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCPERSSUBENTMANAGER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCPERSSUBENTMANAGER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCPERSSUBENTMANAGER_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCPERSSUBENTMANAGER *) ((arg1)->ASSOCPERSSUBENTMANAGER);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCRESTOREENTITYSTATEACTIONBODY_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY *arg2 = (Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCRESTOREENTITYSTATEACTIONBODY_set(self,ASSOCRESTOREENTITYSTATEACTIONBODY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCRESTOREENTITYSTATEACTIONBODY_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCRESTOREENTITYSTATEACTIONBODY_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY *)(argp2);
    if (arg1) (arg1)->ASSOCRESTOREENTITYSTATEACTIONBODY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCRESTOREENTITYSTATEACTIONBODY_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCRESTOREENTITYSTATEACTIONBODY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCRESTOREENTITYSTATEACTIONBODY_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY *) ((arg1)->ASSOCRESTOREENTITYSTATEACTIONBODY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCROTATEDDIMACTIONBODY_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCROTATEDDIMACTIONBODY *arg2 = (Dwg_Object_ASSOCROTATEDDIMACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCROTATEDDIMACTIONBODY_set(self,ASSOCROTATEDDIMACTIONBODY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCROTATEDDIMACTIONBODY_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCROTATEDDIMACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCROTATEDDIMACTIONBODY_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCROTATEDDIMACTIONBODY *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCROTATEDDIMACTIONBODY *)(argp2);
    if (arg1) (arg1)->ASSOCROTATEDDIMACTIONBODY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCROTATEDDIMACTIONBODY_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCROTATEDDIMACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCROTATEDDIMACTIONBODY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCROTATEDDIMACTIONBODY_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCROTATEDDIMACTIONBODY *) ((arg1)->ASSOCROTATEDDIMACTIONBODY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCROTATEDDIMACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCSWEPTSURFACEACTIONBODY_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY *arg2 = (Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCSWEPTSURFACEACTIONBODY_set(self,ASSOCSWEPTSURFACEACTIONBODY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCSWEPTSURFACEACTIONBODY_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ASSOCSWEPTSURFACEACTIONBODY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ASSOCSWEPTSURFACEACTIONBODY_set" "', argument " "2"" of type '" "Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY *""'"); 
    }
    arg2 = (Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY *)(argp2);
    if (arg1) (arg1)->ASSOCSWEPTSURFACEACTIONBODY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ASSOCSWEPTSURFACEACTIONBODY_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ASSOCSWEPTSURFACEACTIONBODY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ASSOCSWEPTSURFACEACTIONBODY_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY *) ((arg1)->ASSOCSWEPTSURFACEACTIONBODY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCSWEPTSURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKPROPERTIESTABLE_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKPROPERTIESTABLE *arg2 = (Dwg_Object_BLOCKPROPERTIESTABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKPROPERTIESTABLE_set(self,BLOCKPROPERTIESTABLE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKPROPERTIESTABLE_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKPROPERTIESTABLE_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKPROPERTIESTABLE *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKPROPERTIESTABLE *)(argp2);
    if (arg1) (arg1)->BLOCKPROPERTIESTABLE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKPROPERTIESTABLE_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKPROPERTIESTABLE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKPROPERTIESTABLE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKPROPERTIESTABLE_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKPROPERTIESTABLE *) ((arg1)->BLOCKPROPERTIESTABLE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKPROPERTIESTABLEGRIP_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BLOCKPROPERTIESTABLEGRIP *arg2 = (Dwg_Object_BLOCKPROPERTIESTABLEGRIP *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKPROPERTIESTABLEGRIP_set(self,BLOCKPROPERTIESTABLEGRIP);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKPROPERTIESTABLEGRIP_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLEGRIP, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BLOCKPROPERTIESTABLEGRIP_set" "', argument " "2"" of type '" "Dwg_Object_BLOCKPROPERTIESTABLEGRIP *""'"); 
    }
    arg2 = (Dwg_Object_BLOCKPROPERTIESTABLEGRIP *)(argp2);
    if (arg1) (arg1)->BLOCKPROPERTIESTABLEGRIP = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BLOCKPROPERTIESTABLEGRIP_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKPROPERTIESTABLEGRIP *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BLOCKPROPERTIESTABLEGRIP_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BLOCKPROPERTIESTABLEGRIP_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BLOCKPROPERTIESTABLEGRIP *) ((arg1)->BLOCKPROPERTIESTABLEGRIP);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLEGRIP, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BREAKDATA_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BREAKDATA *arg2 = (Dwg_Object_BREAKDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BREAKDATA_set(self,BREAKDATA);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BREAKDATA_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BREAKDATA, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BREAKDATA_set" "', argument " "2"" of type '" "Dwg_Object_BREAKDATA *""'"); 
    }
    arg2 = (Dwg_Object_BREAKDATA *)(argp2);
    if (arg1) (arg1)->BREAKDATA = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BREAKDATA_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BREAKDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BREAKDATA_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BREAKDATA_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BREAKDATA *) ((arg1)->BREAKDATA);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BREAKDATA, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BREAKPOINTREF_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_BREAKPOINTREF *arg2 = (Dwg_Object_BREAKPOINTREF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BREAKPOINTREF_set(self,BREAKPOINTREF);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BREAKPOINTREF_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BREAKPOINTREF, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_BREAKPOINTREF_set" "', argument " "2"" of type '" "Dwg_Object_BREAKPOINTREF *""'"); 
    }
    arg2 = (Dwg_Object_BREAKPOINTREF *)(argp2);
    if (arg1) (arg1)->BREAKPOINTREF = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_BREAKPOINTREF_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BREAKPOINTREF *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_BREAKPOINTREF_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_BREAKPOINTREF_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_BREAKPOINTREF *) ((arg1)->BREAKPOINTREF);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BREAKPOINTREF, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_CONTEXTDATAMANAGER_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_CONTEXTDATAMANAGER *arg2 = (Dwg_Object_CONTEXTDATAMANAGER *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_CONTEXTDATAMANAGER_set(self,CONTEXTDATAMANAGER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_CONTEXTDATAMANAGER_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_CONTEXTDATAMANAGER, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_CONTEXTDATAMANAGER_set" "', argument " "2"" of type '" "Dwg_Object_CONTEXTDATAMANAGER *""'"); 
    }
    arg2 = (Dwg_Object_CONTEXTDATAMANAGER *)(argp2);
    if (arg1) (arg1)->CONTEXTDATAMANAGER = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_CONTEXTDATAMANAGER_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_CONTEXTDATAMANAGER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_CONTEXTDATAMANAGER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_CONTEXTDATAMANAGER_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_CONTEXTDATAMANAGER *) ((arg1)->CONTEXTDATAMANAGER);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_CONTEXTDATAMANAGER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_CSACDOCUMENTOPTIONS_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_CSACDOCUMENTOPTIONS *arg2 = (Dwg_Object_CSACDOCUMENTOPTIONS *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_CSACDOCUMENTOPTIONS_set(self,CSACDOCUMENTOPTIONS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_CSACDOCUMENTOPTIONS_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_CSACDOCUMENTOPTIONS, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_CSACDOCUMENTOPTIONS_set" "', argument " "2"" of type '" "Dwg_Object_CSACDOCUMENTOPTIONS *""'"); 
    }
    arg2 = (Dwg_Object_CSACDOCUMENTOPTIONS *)(argp2);
    if (arg1) (arg1)->CSACDOCUMENTOPTIONS = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_CSACDOCUMENTOPTIONS_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_CSACDOCUMENTOPTIONS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_CSACDOCUMENTOPTIONS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_CSACDOCUMENTOPTIONS_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_CSACDOCUMENTOPTIONS *) ((arg1)->CSACDOCUMENTOPTIONS);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_CSACDOCUMENTOPTIONS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_CURVEPATH_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_CURVEPATH *arg2 = (Dwg_Object_CURVEPATH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_CURVEPATH_set(self,CURVEPATH);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_CURVEPATH_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_CURVEPATH, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_CURVEPATH_set" "', argument " "2"" of type '" "Dwg_Object_CURVEPATH *""'"); 
    }
    arg2 = (Dwg_Object_CURVEPATH *)(argp2);
    if (arg1) (arg1)->CURVEPATH = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_CURVEPATH_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_CURVEPATH *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_CURVEPATH_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_CURVEPATH_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_CURVEPATH *) ((arg1)->CURVEPATH);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_CURVEPATH, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_DATATABLE_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_DATATABLE *arg2 = (Dwg_Object_DATATABLE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_DATATABLE_set(self,DATATABLE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_DATATABLE_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_DATATABLE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_DATATABLE_set" "', argument " "2"" of type '" "Dwg_Object_DATATABLE *""'"); 
    }
    arg2 = (Dwg_Object_DATATABLE *)(argp2);
    if (arg1) (arg1)->DATATABLE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_DATATABLE_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DATATABLE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_DATATABLE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_DATATABLE_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_DATATABLE *) ((arg1)->DATATABLE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DATATABLE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_DIMASSOC_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_DIMASSOC *arg2 = (Dwg_Object_DIMASSOC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_DIMASSOC_set(self,DIMASSOC);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_DIMASSOC_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_DIMASSOC, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_DIMASSOC_set" "', argument " "2"" of type '" "Dwg_Object_DIMASSOC *""'"); 
    }
    arg2 = (Dwg_Object_DIMASSOC *)(argp2);
    if (arg1) (arg1)->DIMASSOC = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_DIMASSOC_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DIMASSOC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_DIMASSOC_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_DIMASSOC_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_DIMASSOC *) ((arg1)->DIMASSOC);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DIMASSOC, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_DMDIMOBJECTCONTEXTDATA_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_DMDIMOBJECTCONTEXTDATA *arg2 = (Dwg_Object_DMDIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_DMDIMOBJECTCONTEXTDATA_set(self,DMDIMOBJECTCONTEXTDATA);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_DMDIMOBJECTCONTEXTDATA_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_DMDIMOBJECTCONTEXTDATA, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_DMDIMOBJECTCONTEXTDATA_set" "', argument " "2"" of type '" "Dwg_Object_DMDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg2 = (Dwg_Object_DMDIMOBJECTCONTEXTDATA *)(argp2);
    if (arg1) (arg1)->DMDIMOBJECTCONTEXTDATA = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_DMDIMOBJECTCONTEXTDATA_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DMDIMOBJECTCONTEXTDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_DMDIMOBJECTCONTEXTDATA_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_DMDIMOBJECTCONTEXTDATA_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_DMDIMOBJECTCONTEXTDATA *) ((arg1)->DMDIMOBJECTCONTEXTDATA);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DMDIMOBJECTCONTEXTDATA, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_DYNAMICBLOCKPROXYNODE_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_DYNAMICBLOCKPROXYNODE *arg2 = (Dwg_Object_DYNAMICBLOCKPROXYNODE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_DYNAMICBLOCKPROXYNODE_set(self,DYNAMICBLOCKPROXYNODE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_DYNAMICBLOCKPROXYNODE_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_DYNAMICBLOCKPROXYNODE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_DYNAMICBLOCKPROXYNODE_set" "', argument " "2"" of type '" "Dwg_Object_DYNAMICBLOCKPROXYNODE *""'"); 
    }
    arg2 = (Dwg_Object_DYNAMICBLOCKPROXYNODE *)(argp2);
    if (arg1) (arg1)->DYNAMICBLOCKPROXYNODE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_DYNAMICBLOCKPROXYNODE_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DYNAMICBLOCKPROXYNODE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_DYNAMICBLOCKPROXYNODE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_DYNAMICBLOCKPROXYNODE_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_DYNAMICBLOCKPROXYNODE *) ((arg1)->DYNAMICBLOCKPROXYNODE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DYNAMICBLOCKPROXYNODE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_GEOMAPIMAGE_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_GEOMAPIMAGE *arg2 = (Dwg_Object_GEOMAPIMAGE *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_GEOMAPIMAGE_set(self,GEOMAPIMAGE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_GEOMAPIMAGE_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_GEOMAPIMAGE, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_GEOMAPIMAGE_set" "', argument " "2"" of type '" "Dwg_Object_GEOMAPIMAGE *""'"); 
    }
    arg2 = (Dwg_Object_GEOMAPIMAGE *)(argp2);
    if (arg1) (arg1)->GEOMAPIMAGE = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_GEOMAPIMAGE_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_GEOMAPIMAGE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_GEOMAPIMAGE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_GEOMAPIMAGE_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_GEOMAPIMAGE *) ((arg1)->GEOMAPIMAGE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_GEOMAPIMAGE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_MLEADEROBJECTCONTEXTDATA_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_MLEADEROBJECTCONTEXTDATA *arg2 = (Dwg_Object_MLEADEROBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_MLEADEROBJECTCONTEXTDATA_set(self,MLEADEROBJECTCONTEXTDATA);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_MLEADEROBJECTCONTEXTDATA_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_MLEADEROBJECTCONTEXTDATA, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_MLEADEROBJECTCONTEXTDATA_set" "', argument " "2"" of type '" "Dwg_Object_MLEADEROBJECTCONTEXTDATA *""'"); 
    }
    arg2 = (Dwg_Object_MLEADEROBJECTCONTEXTDATA *)(argp2);
    if (arg1) (arg1)->MLEADEROBJECTCONTEXTDATA = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_MLEADEROBJECTCONTEXTDATA_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_MLEADEROBJECTCONTEXTDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_MLEADEROBJECTCONTEXTDATA_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_MLEADEROBJECTCONTEXTDATA_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_MLEADEROBJECTCONTEXTDATA *) ((arg1)->MLEADEROBJECTCONTEXTDATA);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_MLEADEROBJECTCONTEXTDATA, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_MOTIONPATH_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_MOTIONPATH *arg2 = (Dwg_Object_MOTIONPATH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_MOTIONPATH_set(self,MOTIONPATH);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_MOTIONPATH_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_MOTIONPATH, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_MOTIONPATH_set" "', argument " "2"" of type '" "Dwg_Object_MOTIONPATH *""'"); 
    }
    arg2 = (Dwg_Object_MOTIONPATH *)(argp2);
    if (arg1) (arg1)->MOTIONPATH = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_MOTIONPATH_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_MOTIONPATH *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_MOTIONPATH_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_MOTIONPATH_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_MOTIONPATH *) ((arg1)->MOTIONPATH);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_MOTIONPATH, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_MTEXTATTRIBUTEOBJECTCONTEXTDATA_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *arg2 = (Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_MTEXTATTRIBUTEOBJECTCONTEXTDATA_set(self,MTEXTATTRIBUTEOBJECTCONTEXTDATA);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_MTEXTATTRIBUTEOBJECTCONTEXTDATA_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_MTEXTATTRIBUTEOBJECTCONTEXTDATA_set" "', argument " "2"" of type '" "Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *""'"); 
    }
    arg2 = (Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *)(argp2);
    if (arg1) (arg1)->MTEXTATTRIBUTEOBJECTCONTEXTDATA = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_MTEXTATTRIBUTEOBJECTCONTEXTDATA_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_MTEXTATTRIBUTEOBJECTCONTEXTDATA_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_MTEXTATTRIBUTEOBJECTCONTEXTDATA_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *) ((arg1)->MTEXTATTRIBUTEOBJECTCONTEXTDATA);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_NAVISWORKSMODELDEF_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_NAVISWORKSMODELDEF *arg2 = (Dwg_Object_NAVISWORKSMODELDEF *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_NAVISWORKSMODELDEF_set(self,NAVISWORKSMODELDEF);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_NAVISWORKSMODELDEF_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_NAVISWORKSMODELDEF, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_NAVISWORKSMODELDEF_set" "', argument " "2"" of type '" "Dwg_Object_NAVISWORKSMODELDEF *""'"); 
    }
    arg2 = (Dwg_Object_NAVISWORKSMODELDEF *)(argp2);
    if (arg1) (arg1)->NAVISWORKSMODELDEF = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_NAVISWORKSMODELDEF_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_NAVISWORKSMODELDEF *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_NAVISWORKSMODELDEF_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_NAVISWORKSMODELDEF_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_NAVISWORKSMODELDEF *) ((arg1)->NAVISWORKSMODELDEF);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_NAVISWORKSMODELDEF, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ORDDIMOBJECTCONTEXTDATA_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_ORDDIMOBJECTCONTEXTDATA *arg2 = (Dwg_Object_ORDDIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ORDDIMOBJECTCONTEXTDATA_set(self,ORDDIMOBJECTCONTEXTDATA);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ORDDIMOBJECTCONTEXTDATA_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ORDDIMOBJECTCONTEXTDATA, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_ORDDIMOBJECTCONTEXTDATA_set" "', argument " "2"" of type '" "Dwg_Object_ORDDIMOBJECTCONTEXTDATA *""'"); 
    }
    arg2 = (Dwg_Object_ORDDIMOBJECTCONTEXTDATA *)(argp2);
    if (arg1) (arg1)->ORDDIMOBJECTCONTEXTDATA = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_ORDDIMOBJECTCONTEXTDATA_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ORDDIMOBJECTCONTEXTDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_ORDDIMOBJECTCONTEXTDATA_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_ORDDIMOBJECTCONTEXTDATA_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_ORDDIMOBJECTCONTEXTDATA *) ((arg1)->ORDDIMOBJECTCONTEXTDATA);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ORDDIMOBJECTCONTEXTDATA, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_PERSUBENTMGR_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_PERSUBENTMGR *arg2 = (Dwg_Object_PERSUBENTMGR *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_PERSUBENTMGR_set(self,PERSUBENTMGR);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_PERSUBENTMGR_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_PERSUBENTMGR, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_PERSUBENTMGR_set" "', argument " "2"" of type '" "Dwg_Object_PERSUBENTMGR *""'"); 
    }
    arg2 = (Dwg_Object_PERSUBENTMGR *)(argp2);
    if (arg1) (arg1)->PERSUBENTMGR = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_PERSUBENTMGR_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_PERSUBENTMGR *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_PERSUBENTMGR_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_PERSUBENTMGR_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_PERSUBENTMGR *) ((arg1)->PERSUBENTMGR);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_PERSUBENTMGR, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_POINTPATH_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_POINTPATH *arg2 = (Dwg_Object_POINTPATH *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_POINTPATH_set(self,POINTPATH);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_POINTPATH_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_POINTPATH, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_POINTPATH_set" "', argument " "2"" of type '" "Dwg_Object_POINTPATH *""'"); 
    }
    arg2 = (Dwg_Object_POINTPATH *)(argp2);
    if (arg1) (arg1)->POINTPATH = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_POINTPATH_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_POINTPATH *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_POINTPATH_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_POINTPATH_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_POINTPATH *) ((arg1)->POINTPATH);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_POINTPATH, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_RADIMLGOBJECTCONTEXTDATA_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_RADIMLGOBJECTCONTEXTDATA *arg2 = (Dwg_Object_RADIMLGOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_RADIMLGOBJECTCONTEXTDATA_set(self,RADIMLGOBJECTCONTEXTDATA);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_RADIMLGOBJECTCONTEXTDATA_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_RADIMLGOBJECTCONTEXTDATA, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_RADIMLGOBJECTCONTEXTDATA_set" "', argument " "2"" of type '" "Dwg_Object_RADIMLGOBJECTCONTEXTDATA *""'"); 
    }
    arg2 = (Dwg_Object_RADIMLGOBJECTCONTEXTDATA *)(argp2);
    if (arg1) (arg1)->RADIMLGOBJECTCONTEXTDATA = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_RADIMLGOBJECTCONTEXTDATA_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_RADIMLGOBJECTCONTEXTDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_RADIMLGOBJECTCONTEXTDATA_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_RADIMLGOBJECTCONTEXTDATA_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_RADIMLGOBJECTCONTEXTDATA *) ((arg1)->RADIMLGOBJECTCONTEXTDATA);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_RADIMLGOBJECTCONTEXTDATA, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_RADIMOBJECTCONTEXTDATA_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_RADIMOBJECTCONTEXTDATA *arg2 = (Dwg_Object_RADIMOBJECTCONTEXTDATA *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_RADIMOBJECTCONTEXTDATA_set(self,RADIMOBJECTCONTEXTDATA);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_RADIMOBJECTCONTEXTDATA_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_RADIMOBJECTCONTEXTDATA, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_RADIMOBJECTCONTEXTDATA_set" "', argument " "2"" of type '" "Dwg_Object_RADIMOBJECTCONTEXTDATA *""'"); 
    }
    arg2 = (Dwg_Object_RADIMOBJECTCONTEXTDATA *)(argp2);
    if (arg1) (arg1)->RADIMOBJECTCONTEXTDATA = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_RADIMOBJECTCONTEXTDATA_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_RADIMOBJECTCONTEXTDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_RADIMOBJECTCONTEXTDATA_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_RADIMOBJECTCONTEXTDATA_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_RADIMOBJECTCONTEXTDATA *) ((arg1)->RADIMOBJECTCONTEXTDATA);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_RADIMOBJECTCONTEXTDATA, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_SUNSTUDY_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_SUNSTUDY *arg2 = (Dwg_Object_SUNSTUDY *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_SUNSTUDY_set(self,SUNSTUDY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_SUNSTUDY_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_SUNSTUDY, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_SUNSTUDY_set" "', argument " "2"" of type '" "Dwg_Object_SUNSTUDY *""'"); 
    }
    arg2 = (Dwg_Object_SUNSTUDY *)(argp2);
    if (arg1) (arg1)->SUNSTUDY = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_SUNSTUDY_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_SUNSTUDY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_SUNSTUDY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_SUNSTUDY_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_SUNSTUDY *) ((arg1)->SUNSTUDY);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_SUNSTUDY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_TABLECONTENT_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_TABLECONTENT *arg2 = (Dwg_Object_TABLECONTENT *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_TABLECONTENT_set(self,TABLECONTENT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_TABLECONTENT_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_TABLECONTENT, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_TABLECONTENT_set" "', argument " "2"" of type '" "Dwg_Object_TABLECONTENT *""'"); 
    }
    arg2 = (Dwg_Object_TABLECONTENT *)(argp2);
    if (arg1) (arg1)->TABLECONTENT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_TABLECONTENT_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_TABLECONTENT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_TABLECONTENT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_TABLECONTENT_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_TABLECONTENT *) ((arg1)->TABLECONTENT);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_TABLECONTENT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_TVDEVICEPROPERTIES_set) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    Dwg_Object_TVDEVICEPROPERTIES *arg2 = (Dwg_Object_TVDEVICEPROPERTIES *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_object_tio_TVDEVICEPROPERTIES_set(self,TVDEVICEPROPERTIES);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_TVDEVICEPROPERTIES_set" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_TVDEVICEPROPERTIES, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_object_tio_TVDEVICEPROPERTIES_set" "', argument " "2"" of type '" "Dwg_Object_TVDEVICEPROPERTIES *""'"); 
    }
    arg2 = (Dwg_Object_TVDEVICEPROPERTIES *)(argp2);
    if (arg1) (arg1)->TVDEVICEPROPERTIES = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_object_tio_TVDEVICEPROPERTIES_get) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_TVDEVICEPROPERTIES *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_object_tio_TVDEVICEPROPERTIES_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_object_tio_TVDEVICEPROPERTIES_get" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    result = (Dwg_Object_TVDEVICEPROPERTIES *) ((arg1)->TVDEVICEPROPERTIES);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_TVDEVICEPROPERTIES, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new__dwg_object_object_tio) {
  {
    int argvi = 0;
    _dwg_object_object_tio *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new__dwg_object_object_tio();");
    }
    result = (_dwg_object_object_tio *)calloc(1, sizeof(_dwg_object_object_tio));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object_tio, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete__dwg_object_object_tio) {
  {
    _dwg_object_object_tio *arg1 = (_dwg_object_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete__dwg_object_object_tio(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object_tio, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete__dwg_object_object_tio" "', argument " "1"" of type '" "_dwg_object_object_tio *""'"); 
    }
    arg1 = (_dwg_object_object_tio *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Class_number_set) {
  {
    struct _dwg_class *arg1 = (struct _dwg_class *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Class_number_set(self,number);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_class, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Class_number_set" "', argument " "1"" of type '" "struct _dwg_class *""'"); 
    }
    arg1 = (struct _dwg_class *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Class_number_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->number = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Class_number_get) {
  {
    struct _dwg_class *arg1 = (struct _dwg_class *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Class_number_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_class, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Class_number_get" "', argument " "1"" of type '" "struct _dwg_class *""'"); 
    }
    arg1 = (struct _dwg_class *)(argp1);
    result = (BITCODE_BS) ((arg1)->number);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Class_proxyflag_set) {
  {
    struct _dwg_class *arg1 = (struct _dwg_class *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Class_proxyflag_set(self,proxyflag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_class, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Class_proxyflag_set" "', argument " "1"" of type '" "struct _dwg_class *""'"); 
    }
    arg1 = (struct _dwg_class *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Class_proxyflag_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->proxyflag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Class_proxyflag_get) {
  {
    struct _dwg_class *arg1 = (struct _dwg_class *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Class_proxyflag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_class, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Class_proxyflag_get" "', argument " "1"" of type '" "struct _dwg_class *""'"); 
    }
    arg1 = (struct _dwg_class *)(argp1);
    result = (BITCODE_BS) ((arg1)->proxyflag);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Class_appname_set) {
  {
    struct _dwg_class *arg1 = (struct _dwg_class *) 0 ;
    char *arg2 = (char *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Class_appname_set(self,appname);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_class, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Class_appname_set" "', argument " "1"" of type '" "struct _dwg_class *""'"); 
    }
    arg1 = (struct _dwg_class *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Class_appname_set" "', argument " "2"" of type '" "char *""'");
    }
    arg2 = (char *)(buf2);
    free((char*)arg1->appname);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->appname = (char *)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->appname = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Class_appname_get) {
  {
    struct _dwg_class *arg1 = (struct _dwg_class *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    char *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Class_appname_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_class, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Class_appname_get" "', argument " "1"" of type '" "struct _dwg_class *""'"); 
    }
    arg1 = (struct _dwg_class *)(argp1);
    result = (char *) ((arg1)->appname);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Class_cppname_set) {
  {
    struct _dwg_class *arg1 = (struct _dwg_class *) 0 ;
    char *arg2 = (char *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Class_cppname_set(self,cppname);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_class, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Class_cppname_set" "', argument " "1"" of type '" "struct _dwg_class *""'"); 
    }
    arg1 = (struct _dwg_class *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Class_cppname_set" "', argument " "2"" of type '" "char *""'");
    }
    arg2 = (char *)(buf2);
    free((char*)arg1->cppname);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->cppname = (char *)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->cppname = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Class_cppname_get) {
  {
    struct _dwg_class *arg1 = (struct _dwg_class *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    char *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Class_cppname_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_class, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Class_cppname_get" "', argument " "1"" of type '" "struct _dwg_class *""'"); 
    }
    arg1 = (struct _dwg_class *)(argp1);
    result = (char *) ((arg1)->cppname);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Class_dxfname_set) {
  {
    struct _dwg_class *arg1 = (struct _dwg_class *) 0 ;
    char *arg2 = (char *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Class_dxfname_set(self,dxfname);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_class, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Class_dxfname_set" "', argument " "1"" of type '" "struct _dwg_class *""'"); 
    }
    arg1 = (struct _dwg_class *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Class_dxfname_set" "', argument " "2"" of type '" "char *""'");
    }
    arg2 = (char *)(buf2);
    free((char*)arg1->dxfname);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->dxfname = (char *)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->dxfname = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Class_dxfname_get) {
  {
    struct _dwg_class *arg1 = (struct _dwg_class *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    char *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Class_dxfname_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_class, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Class_dxfname_get" "', argument " "1"" of type '" "struct _dwg_class *""'"); 
    }
    arg1 = (struct _dwg_class *)(argp1);
    result = (char *) ((arg1)->dxfname);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Class_dxfname_u_set) {
  {
    struct _dwg_class *arg1 = (struct _dwg_class *) 0 ;
    BITCODE_TU arg2 = (BITCODE_TU) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Class_dxfname_u_set(self,dxfname_u);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_class, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Class_dxfname_u_set" "', argument " "1"" of type '" "struct _dwg_class *""'"); 
    }
    arg1 = (struct _dwg_class *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_short, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Class_dxfname_u_set" "', argument " "2"" of type '" "BITCODE_TU""'"); 
    }
    arg2 = (BITCODE_TU)(argp2);
    if (arg1) (arg1)->dxfname_u = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Class_dxfname_u_get) {
  {
    struct _dwg_class *arg1 = (struct _dwg_class *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TU result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Class_dxfname_u_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_class, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Class_dxfname_u_get" "', argument " "1"" of type '" "struct _dwg_class *""'"); 
    }
    arg1 = (struct _dwg_class *)(argp1);
    result = (BITCODE_TU) ((arg1)->dxfname_u);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_short, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Class_is_zombie_set) {
  {
    struct _dwg_class *arg1 = (struct _dwg_class *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Class_is_zombie_set(self,is_zombie);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_class, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Class_is_zombie_set" "', argument " "1"" of type '" "struct _dwg_class *""'"); 
    }
    arg1 = (struct _dwg_class *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Class_is_zombie_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->is_zombie = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Class_is_zombie_get) {
  {
    struct _dwg_class *arg1 = (struct _dwg_class *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Class_is_zombie_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_class, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Class_is_zombie_get" "', argument " "1"" of type '" "struct _dwg_class *""'"); 
    }
    arg1 = (struct _dwg_class *)(argp1);
    result = (BITCODE_B) ((arg1)->is_zombie);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Class_item_class_id_set) {
  {
    struct _dwg_class *arg1 = (struct _dwg_class *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Class_item_class_id_set(self,item_class_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_class, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Class_item_class_id_set" "', argument " "1"" of type '" "struct _dwg_class *""'"); 
    }
    arg1 = (struct _dwg_class *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Class_item_class_id_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->item_class_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Class_item_class_id_get) {
  {
    struct _dwg_class *arg1 = (struct _dwg_class *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Class_item_class_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_class, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Class_item_class_id_get" "', argument " "1"" of type '" "struct _dwg_class *""'"); 
    }
    arg1 = (struct _dwg_class *)(argp1);
    result = (BITCODE_BS) ((arg1)->item_class_id);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Class_num_instances_set) {
  {
    struct _dwg_class *arg1 = (struct _dwg_class *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Class_num_instances_set(self,num_instances);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_class, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Class_num_instances_set" "', argument " "1"" of type '" "struct _dwg_class *""'"); 
    }
    arg1 = (struct _dwg_class *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Class_num_instances_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_instances = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Class_num_instances_get) {
  {
    struct _dwg_class *arg1 = (struct _dwg_class *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Class_num_instances_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_class, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Class_num_instances_get" "', argument " "1"" of type '" "struct _dwg_class *""'"); 
    }
    arg1 = (struct _dwg_class *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_instances);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Class_dwg_version_set) {
  {
    struct _dwg_class *arg1 = (struct _dwg_class *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Class_dwg_version_set(self,dwg_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_class, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Class_dwg_version_set" "', argument " "1"" of type '" "struct _dwg_class *""'"); 
    }
    arg1 = (struct _dwg_class *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Class_dwg_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->dwg_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Class_dwg_version_get) {
  {
    struct _dwg_class *arg1 = (struct _dwg_class *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Class_dwg_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_class, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Class_dwg_version_get" "', argument " "1"" of type '" "struct _dwg_class *""'"); 
    }
    arg1 = (struct _dwg_class *)(argp1);
    result = (BITCODE_BL) ((arg1)->dwg_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Class_maint_version_set) {
  {
    struct _dwg_class *arg1 = (struct _dwg_class *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Class_maint_version_set(self,maint_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_class, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Class_maint_version_set" "', argument " "1"" of type '" "struct _dwg_class *""'"); 
    }
    arg1 = (struct _dwg_class *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Class_maint_version_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->maint_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Class_maint_version_get) {
  {
    struct _dwg_class *arg1 = (struct _dwg_class *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Class_maint_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_class, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Class_maint_version_get" "', argument " "1"" of type '" "struct _dwg_class *""'"); 
    }
    arg1 = (struct _dwg_class *)(argp1);
    result = (BITCODE_BL) ((arg1)->maint_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Class_unknown_1_set) {
  {
    struct _dwg_class *arg1 = (struct _dwg_class *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Class_unknown_1_set(self,unknown_1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_class, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Class_unknown_1_set" "', argument " "1"" of type '" "struct _dwg_class *""'"); 
    }
    arg1 = (struct _dwg_class *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Class_unknown_1_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Class_unknown_1_get) {
  {
    struct _dwg_class *arg1 = (struct _dwg_class *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Class_unknown_1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_class, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Class_unknown_1_get" "', argument " "1"" of type '" "struct _dwg_class *""'"); 
    }
    arg1 = (struct _dwg_class *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_1);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Class_unknown_2_set) {
  {
    struct _dwg_class *arg1 = (struct _dwg_class *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Class_unknown_2_set(self,unknown_2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_class, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Class_unknown_2_set" "', argument " "1"" of type '" "struct _dwg_class *""'"); 
    }
    arg1 = (struct _dwg_class *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Class_unknown_2_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->unknown_2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Class_unknown_2_get) {
  {
    struct _dwg_class *arg1 = (struct _dwg_class *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Class_unknown_2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_class, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Class_unknown_2_get" "', argument " "1"" of type '" "struct _dwg_class *""'"); 
    }
    arg1 = (struct _dwg_class *)(argp1);
    result = (BITCODE_BL) ((arg1)->unknown_2);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Class) {
  {
    int argvi = 0;
    struct _dwg_class *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Class();");
    }
    result = (struct _dwg_class *)calloc(1, sizeof(struct _dwg_class));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_class, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Class) {
  {
    struct _dwg_class *arg1 = (struct _dwg_class *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Class(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_class, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Class" "', argument " "1"" of type '" "struct _dwg_class *""'"); 
    }
    arg1 = (struct _dwg_class *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_size_set) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_size_set(self,size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_size_set" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_size_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_size_get) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_size_get" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    result = (BITCODE_RL) ((arg1)->size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_address_set) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    size_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    size_t val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_address_set(self,address);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_address_set" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    ecode2 = SWIG_AsVal_size_t SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_address_set" "', argument " "2"" of type '" "size_t""'");
    } 
    arg2 = (size_t)(val2);
    if (arg1) (arg1)->address = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_address_get) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    size_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_address_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_address_get" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    result =  ((arg1)->address);
    ST(argvi) = SWIG_From_size_t  SWIG_PERL_CALL_ARGS_1((size_t)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_type_set) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_type_set(self,type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_type_set" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_type_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_type_get) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_type_get" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    result = (BITCODE_BS) ((arg1)->type);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_index_set) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_index_set(self,index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_index_set" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_index_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_index_get) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_index_get" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    result = (BITCODE_RL) ((arg1)->index);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_fixedtype_set) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    enum DWG_OBJECT_TYPE arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_fixedtype_set(self,fixedtype);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_fixedtype_set" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_fixedtype_set" "', argument " "2"" of type '" "enum DWG_OBJECT_TYPE""'");
    } 
    arg2 = (enum DWG_OBJECT_TYPE)(val2);
    if (arg1) (arg1)->fixedtype = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_fixedtype_get) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    enum DWG_OBJECT_TYPE result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_fixedtype_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_fixedtype_get" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    result = (enum DWG_OBJECT_TYPE) ((arg1)->fixedtype);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_name_set) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    char *arg2 = (char *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_name_set" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_name_set" "', argument " "2"" of type '" "char *""'");
    }
    arg2 = (char *)(buf2);
    free((char*)arg1->name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->name = (char *)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_name_get) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    char *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_name_get" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    result = (char *) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_dxfname_set) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    char *arg2 = (char *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_dxfname_set(self,dxfname);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_dxfname_set" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_dxfname_set" "', argument " "2"" of type '" "char *""'");
    }
    arg2 = (char *)(buf2);
    free((char*)arg1->dxfname);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->dxfname = (char *)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->dxfname = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_dxfname_get) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    char *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_dxfname_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_dxfname_get" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    result = (char *) ((arg1)->dxfname);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_supertype_set) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    Dwg_Object_Supertype arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_supertype_set(self,supertype);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_supertype_set" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_supertype_set" "', argument " "2"" of type '" "Dwg_Object_Supertype""'");
    } 
    arg2 = (Dwg_Object_Supertype)(val2);
    if (arg1) (arg1)->supertype = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_supertype_get) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_Supertype result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_supertype_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_supertype_get" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    result = (Dwg_Object_Supertype) ((arg1)->supertype);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_tio_get) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    _dwg_object_tio *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_tio_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_tio_get" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    result = (_dwg_object_tio *)& ((arg1)->tio);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_tio, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_handle_set) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    Dwg_Handle *arg2 = (Dwg_Handle *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_handle_set(self,handle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_handle_set" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_handle, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_handle_set" "', argument " "2"" of type '" "Dwg_Handle *""'"); 
    }
    arg2 = (Dwg_Handle *)(argp2);
    if (arg1) (arg1)->handle = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_handle_get) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Handle *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_handle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_handle_get" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    result = (Dwg_Handle *)& ((arg1)->handle);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_handle, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_parent_set) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    struct _dwg_struct *arg2 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_parent_set" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_struct, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_parent_set" "', argument " "2"" of type '" "struct _dwg_struct *""'"); 
    }
    arg2 = (struct _dwg_struct *)(argp2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_parent_get) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _dwg_struct *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_parent_get" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    result = (struct _dwg_struct *) ((arg1)->parent);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_struct, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_klass_set) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    Dwg_Class *arg2 = (Dwg_Class *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_klass_set(self,klass);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_klass_set" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_class, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_klass_set" "', argument " "2"" of type '" "Dwg_Class *""'"); 
    }
    arg2 = (Dwg_Class *)(argp2);
    if (arg1) (arg1)->klass = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_klass_get) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Class *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_klass_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_klass_get" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    result = (Dwg_Class *) ((arg1)->klass);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_class, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_bitsize_set) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_bitsize_set(self,bitsize);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_bitsize_set" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_bitsize_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->bitsize = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_bitsize_get) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_bitsize_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_bitsize_get" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    result = (BITCODE_RL) ((arg1)->bitsize);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_bitsize_pos_set) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    size_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    size_t val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_bitsize_pos_set(self,bitsize_pos);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_bitsize_pos_set" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    ecode2 = SWIG_AsVal_size_t SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_bitsize_pos_set" "', argument " "2"" of type '" "size_t""'");
    } 
    arg2 = (size_t)(val2);
    if (arg1) (arg1)->bitsize_pos = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_bitsize_pos_get) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    size_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_bitsize_pos_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_bitsize_pos_get" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    result =  ((arg1)->bitsize_pos);
    ST(argvi) = SWIG_From_size_t  SWIG_PERL_CALL_ARGS_1((size_t)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_hdlpos_set) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    size_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    size_t val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_hdlpos_set(self,hdlpos);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_hdlpos_set" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    ecode2 = SWIG_AsVal_size_t SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_hdlpos_set" "', argument " "2"" of type '" "size_t""'");
    } 
    arg2 = (size_t)(val2);
    if (arg1) (arg1)->hdlpos = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_hdlpos_get) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    size_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_hdlpos_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_hdlpos_get" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    result =  ((arg1)->hdlpos);
    ST(argvi) = SWIG_From_size_t  SWIG_PERL_CALL_ARGS_1((size_t)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_was_bitsize_set_set) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_was_bitsize_set_set(self,was_bitsize_set);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_was_bitsize_set_set" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_was_bitsize_set_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->was_bitsize_set = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_was_bitsize_set_get) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_was_bitsize_set_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_was_bitsize_set_get" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    result = (BITCODE_B) ((arg1)->was_bitsize_set);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_has_strings_set) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    BITCODE_B arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_has_strings_set(self,has_strings);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_has_strings_set" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_has_strings_set" "', argument " "2"" of type '" "BITCODE_B""'");
    } 
    arg2 = (BITCODE_B)(val2);
    if (arg1) (arg1)->has_strings = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_has_strings_get) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_B result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_has_strings_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_has_strings_get" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    result = (BITCODE_B) ((arg1)->has_strings);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_stringstream_size_set) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_stringstream_size_set(self,stringstream_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_stringstream_size_set" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_stringstream_size_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->stringstream_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_stringstream_size_get) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_stringstream_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_stringstream_size_get" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    result = (BITCODE_RL) ((arg1)->stringstream_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_handlestream_size_set) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    BITCODE_UMC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_handlestream_size_set(self,handlestream_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_handlestream_size_set" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_handlestream_size_set" "', argument " "2"" of type '" "BITCODE_UMC""'");
    } 
    arg2 = (BITCODE_UMC)(val2);
    if (arg1) (arg1)->handlestream_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_handlestream_size_get) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_UMC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_handlestream_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_handlestream_size_get" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    result = (BITCODE_UMC) ((arg1)->handlestream_size);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_common_size_set) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    size_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    size_t val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_common_size_set(self,common_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_common_size_set" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    ecode2 = SWIG_AsVal_size_t SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_common_size_set" "', argument " "2"" of type '" "size_t""'");
    } 
    arg2 = (size_t)(val2);
    if (arg1) (arg1)->common_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_common_size_get) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    size_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_common_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_common_size_get" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    result =  ((arg1)->common_size);
    ST(argvi) = SWIG_From_size_t  SWIG_PERL_CALL_ARGS_1((size_t)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_num_unknown_bits_set) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_num_unknown_bits_set(self,num_unknown_bits);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_num_unknown_bits_set" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_num_unknown_bits_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->num_unknown_bits = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_num_unknown_bits_get) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_num_unknown_bits_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_num_unknown_bits_get" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    result = (BITCODE_RL) ((arg1)->num_unknown_bits);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_unknown_bits_set) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    BITCODE_TF arg2 = (BITCODE_TF) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_unknown_bits_set(self,unknown_bits);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_unknown_bits_set" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_unknown_bits_set" "', argument " "2"" of type '" "BITCODE_TF""'"); 
    }
    arg2 = (BITCODE_TF)(argp2);
    if (arg1) (arg1)->unknown_bits = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_unknown_bits_get) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TF result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_unknown_bits_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_unknown_bits_get" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    result = (BITCODE_TF) ((arg1)->unknown_bits);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_num_unknown_rest_set) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_num_unknown_rest_set(self,num_unknown_rest);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_num_unknown_rest_set" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Object_num_unknown_rest_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->num_unknown_rest = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_num_unknown_rest_get) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_num_unknown_rest_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_num_unknown_rest_get" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    result = (BITCODE_RL) ((arg1)->num_unknown_rest);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_unknown_rest_set) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    BITCODE_TF arg2 = (BITCODE_TF) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Object_unknown_rest_set(self,unknown_rest);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_unknown_rest_set" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Object_unknown_rest_set" "', argument " "2"" of type '" "BITCODE_TF""'"); 
    }
    arg2 = (BITCODE_TF)(argp2);
    if (arg1) (arg1)->unknown_rest = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Object_unknown_rest_get) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TF result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Object_unknown_rest_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Object_unknown_rest_get" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    result = (BITCODE_TF) ((arg1)->unknown_rest);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Object) {
  {
    int argvi = 0;
    struct _dwg_object *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Object();");
    }
    result = (struct _dwg_object *)calloc(1, sizeof(struct _dwg_object));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Object) {
  {
    struct _dwg_object *arg1 = (struct _dwg_object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Object(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Object" "', argument " "1"" of type '" "struct _dwg_object *""'"); 
    }
    arg1 = (struct _dwg_object *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_tio_entity_set) {
  {
    _dwg_object_tio *arg1 = (_dwg_object_tio *) 0 ;
    Dwg_Object_Entity *arg2 = (Dwg_Object_Entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_tio_entity_set(self,entity);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_tio_entity_set" "', argument " "1"" of type '" "_dwg_object_tio *""'"); 
    }
    arg1 = (_dwg_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_entity, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_tio_entity_set" "', argument " "2"" of type '" "Dwg_Object_Entity *""'"); 
    }
    arg2 = (Dwg_Object_Entity *)(argp2);
    if (arg1) (arg1)->entity = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_tio_entity_get) {
  {
    _dwg_object_tio *arg1 = (_dwg_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_Entity *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_tio_entity_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_tio_entity_get" "', argument " "1"" of type '" "_dwg_object_tio *""'"); 
    }
    arg1 = (_dwg_object_tio *)(argp1);
    result = (Dwg_Object_Entity *) ((arg1)->entity);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_tio_object_set) {
  {
    _dwg_object_tio *arg1 = (_dwg_object_tio *) 0 ;
    Dwg_Object_Object *arg2 = (Dwg_Object_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_object_tio_object_set(self,object);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_tio_object_set" "', argument " "1"" of type '" "_dwg_object_tio *""'"); 
    }
    arg1 = (_dwg_object_tio *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_object_tio_object_set" "', argument " "2"" of type '" "Dwg_Object_Object *""'"); 
    }
    arg2 = (Dwg_Object_Object *)(argp2);
    if (arg1) (arg1)->object = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_object_tio_object_get) {
  {
    _dwg_object_tio *arg1 = (_dwg_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_Object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_object_tio_object_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_tio, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_object_tio_object_get" "', argument " "1"" of type '" "_dwg_object_tio *""'"); 
    }
    arg1 = (_dwg_object_tio *)(argp1);
    result = (Dwg_Object_Object *) ((arg1)->object);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new__dwg_object_tio) {
  {
    int argvi = 0;
    _dwg_object_tio *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new__dwg_object_tio();");
    }
    result = (_dwg_object_tio *)calloc(1, sizeof(_dwg_object_tio));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_tio, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete__dwg_object_tio) {
  {
    _dwg_object_tio *arg1 = (_dwg_object_tio *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete__dwg_object_tio(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_tio, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete__dwg_object_tio" "', argument " "1"" of type '" "_dwg_object_tio *""'"); 
    }
    arg1 = (_dwg_object_tio *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Chain_chain_set) {
  {
    struct _dwg_chain *arg1 = (struct _dwg_chain *) 0 ;
    unsigned char *arg2 = (unsigned char *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Chain_chain_set(self,chain);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_chain, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Chain_chain_set" "', argument " "1"" of type '" "struct _dwg_chain *""'"); 
    }
    arg1 = (struct _dwg_chain *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Chain_chain_set" "', argument " "2"" of type '" "unsigned char *""'"); 
    }
    arg2 = (unsigned char *)(argp2);
    if (arg1) (arg1)->chain = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Chain_chain_get) {
  {
    struct _dwg_chain *arg1 = (struct _dwg_chain *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    unsigned char *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Chain_chain_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_chain, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Chain_chain_get" "', argument " "1"" of type '" "struct _dwg_chain *""'"); 
    }
    arg1 = (struct _dwg_chain *)(argp1);
    result = (unsigned char *) ((arg1)->chain);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Chain_size_set) {
  {
    struct _dwg_chain *arg1 = (struct _dwg_chain *) 0 ;
    size_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    size_t val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Chain_size_set(self,size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_chain, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Chain_size_set" "', argument " "1"" of type '" "struct _dwg_chain *""'"); 
    }
    arg1 = (struct _dwg_chain *)(argp1);
    ecode2 = SWIG_AsVal_size_t SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Chain_size_set" "', argument " "2"" of type '" "size_t""'");
    } 
    arg2 = (size_t)(val2);
    if (arg1) (arg1)->size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Chain_size_get) {
  {
    struct _dwg_chain *arg1 = (struct _dwg_chain *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    size_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Chain_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_chain, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Chain_size_get" "', argument " "1"" of type '" "struct _dwg_chain *""'"); 
    }
    arg1 = (struct _dwg_chain *)(argp1);
    result =  ((arg1)->size);
    ST(argvi) = SWIG_From_size_t  SWIG_PERL_CALL_ARGS_1((size_t)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Chain_byte_set) {
  {
    struct _dwg_chain *arg1 = (struct _dwg_chain *) 0 ;
    size_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    size_t val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Chain_byte_set(self,byte);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_chain, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Chain_byte_set" "', argument " "1"" of type '" "struct _dwg_chain *""'"); 
    }
    arg1 = (struct _dwg_chain *)(argp1);
    ecode2 = SWIG_AsVal_size_t SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Chain_byte_set" "', argument " "2"" of type '" "size_t""'");
    } 
    arg2 = (size_t)(val2);
    if (arg1) (arg1)->byte = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Chain_byte_get) {
  {
    struct _dwg_chain *arg1 = (struct _dwg_chain *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    size_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Chain_byte_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_chain, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Chain_byte_get" "', argument " "1"" of type '" "struct _dwg_chain *""'"); 
    }
    arg1 = (struct _dwg_chain *)(argp1);
    result =  ((arg1)->byte);
    ST(argvi) = SWIG_From_size_t  SWIG_PERL_CALL_ARGS_1((size_t)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Chain_bit_set) {
  {
    struct _dwg_chain *arg1 = (struct _dwg_chain *) 0 ;
    unsigned char arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Chain_bit_set(self,bit);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_chain, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Chain_bit_set" "', argument " "1"" of type '" "struct _dwg_chain *""'"); 
    }
    arg1 = (struct _dwg_chain *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Chain_bit_set" "', argument " "2"" of type '" "unsigned char""'");
    } 
    arg2 = (unsigned char)(val2);
    if (arg1) (arg1)->bit = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Chain_bit_get) {
  {
    struct _dwg_chain *arg1 = (struct _dwg_chain *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    unsigned char result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Chain_bit_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_chain, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Chain_bit_get" "', argument " "1"" of type '" "struct _dwg_chain *""'"); 
    }
    arg1 = (struct _dwg_chain *)(argp1);
    result = (unsigned char) ((arg1)->bit);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Chain) {
  {
    int argvi = 0;
    struct _dwg_chain *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Chain();");
    }
    result = (struct _dwg_chain *)calloc(1, sizeof(struct _dwg_chain));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_chain, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Chain) {
  {
    struct _dwg_chain *arg1 = (struct _dwg_chain *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Chain(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_chain, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Chain" "', argument " "1"" of type '" "struct _dwg_chain *""'"); 
    }
    arg1 = (struct _dwg_chain *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_number_set) {
  {
    struct _dwg_section *arg1 = (struct _dwg_section *) 0 ;
    BITCODE_RLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Section_number_set(self,number);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_section, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_number_set" "', argument " "1"" of type '" "struct _dwg_section *""'"); 
    }
    arg1 = (struct _dwg_section *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Section_number_set" "', argument " "2"" of type '" "BITCODE_RLd""'");
    } 
    arg2 = (BITCODE_RLd)(val2);
    if (arg1) (arg1)->number = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_number_get) {
  {
    struct _dwg_section *arg1 = (struct _dwg_section *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Section_number_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_section, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_number_get" "', argument " "1"" of type '" "struct _dwg_section *""'"); 
    }
    arg1 = (struct _dwg_section *)(argp1);
    result = (BITCODE_RLd) ((arg1)->number);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_size_set) {
  {
    struct _dwg_section *arg1 = (struct _dwg_section *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Section_size_set(self,size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_section, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_size_set" "', argument " "1"" of type '" "struct _dwg_section *""'"); 
    }
    arg1 = (struct _dwg_section *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Section_size_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_size_get) {
  {
    struct _dwg_section *arg1 = (struct _dwg_section *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Section_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_section, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_size_get" "', argument " "1"" of type '" "struct _dwg_section *""'"); 
    }
    arg1 = (struct _dwg_section *)(argp1);
    result = (BITCODE_RL) ((arg1)->size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_address_set) {
  {
    struct _dwg_section *arg1 = (struct _dwg_section *) 0 ;
    BITCODE_RLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Section_address_set(self,address);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_section, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_address_set" "', argument " "1"" of type '" "struct _dwg_section *""'"); 
    }
    arg1 = (struct _dwg_section *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Section_address_set" "', argument " "2"" of type '" "BITCODE_RLL""'");
    } 
    arg2 = (BITCODE_RLL)(val2);
    if (arg1) (arg1)->address = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_address_get) {
  {
    struct _dwg_section *arg1 = (struct _dwg_section *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Section_address_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_section, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_address_get" "', argument " "1"" of type '" "struct _dwg_section *""'"); 
    }
    arg1 = (struct _dwg_section *)(argp1);
    result = (BITCODE_RLL) ((arg1)->address);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_objid_r11_set) {
  {
    struct _dwg_section *arg1 = (struct _dwg_section *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Section_objid_r11_set(self,objid_r11);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_section, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_objid_r11_set" "', argument " "1"" of type '" "struct _dwg_section *""'"); 
    }
    arg1 = (struct _dwg_section *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Section_objid_r11_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->objid_r11 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_objid_r11_get) {
  {
    struct _dwg_section *arg1 = (struct _dwg_section *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Section_objid_r11_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_section, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_objid_r11_get" "', argument " "1"" of type '" "struct _dwg_section *""'"); 
    }
    arg1 = (struct _dwg_section *)(argp1);
    result = (BITCODE_RL) ((arg1)->objid_r11);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_parent_set) {
  {
    struct _dwg_section *arg1 = (struct _dwg_section *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Section_parent_set(self,parent);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_section, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_parent_set" "', argument " "1"" of type '" "struct _dwg_section *""'"); 
    }
    arg1 = (struct _dwg_section *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Section_parent_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->parent = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_parent_get) {
  {
    struct _dwg_section *arg1 = (struct _dwg_section *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Section_parent_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_section, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_parent_get" "', argument " "1"" of type '" "struct _dwg_section *""'"); 
    }
    arg1 = (struct _dwg_section *)(argp1);
    result = (BITCODE_RL) ((arg1)->parent);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_left_set) {
  {
    struct _dwg_section *arg1 = (struct _dwg_section *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Section_left_set(self,left);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_section, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_left_set" "', argument " "1"" of type '" "struct _dwg_section *""'"); 
    }
    arg1 = (struct _dwg_section *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Section_left_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->left = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_left_get) {
  {
    struct _dwg_section *arg1 = (struct _dwg_section *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Section_left_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_section, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_left_get" "', argument " "1"" of type '" "struct _dwg_section *""'"); 
    }
    arg1 = (struct _dwg_section *)(argp1);
    result = (BITCODE_RL) ((arg1)->left);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_right_set) {
  {
    struct _dwg_section *arg1 = (struct _dwg_section *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Section_right_set(self,right);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_section, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_right_set" "', argument " "1"" of type '" "struct _dwg_section *""'"); 
    }
    arg1 = (struct _dwg_section *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Section_right_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->right = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_right_get) {
  {
    struct _dwg_section *arg1 = (struct _dwg_section *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Section_right_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_section, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_right_get" "', argument " "1"" of type '" "struct _dwg_section *""'"); 
    }
    arg1 = (struct _dwg_section *)(argp1);
    result = (BITCODE_RL) ((arg1)->right);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_x00_set) {
  {
    struct _dwg_section *arg1 = (struct _dwg_section *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Section_x00_set(self,x00);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_section, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_x00_set" "', argument " "1"" of type '" "struct _dwg_section *""'"); 
    }
    arg1 = (struct _dwg_section *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Section_x00_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->x00 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_x00_get) {
  {
    struct _dwg_section *arg1 = (struct _dwg_section *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Section_x00_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_section, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_x00_get" "', argument " "1"" of type '" "struct _dwg_section *""'"); 
    }
    arg1 = (struct _dwg_section *)(argp1);
    result = (BITCODE_RL) ((arg1)->x00);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_type_set) {
  {
    struct _dwg_section *arg1 = (struct _dwg_section *) 0 ;
    Dwg_Section_Type arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Section_type_set(self,type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_section, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_type_set" "', argument " "1"" of type '" "struct _dwg_section *""'"); 
    }
    arg1 = (struct _dwg_section *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Section_type_set" "', argument " "2"" of type '" "Dwg_Section_Type""'");
    } 
    arg2 = (Dwg_Section_Type)(val2);
    if (arg1) (arg1)->type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_type_get) {
  {
    struct _dwg_section *arg1 = (struct _dwg_section *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Section_Type result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Section_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_section, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_type_get" "', argument " "1"" of type '" "struct _dwg_section *""'"); 
    }
    arg1 = (struct _dwg_section *)(argp1);
    result = (Dwg_Section_Type) ((arg1)->type);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_name_set) {
  {
    struct _dwg_section *arg1 = (struct _dwg_section *) 0 ;
    char *arg2 = (char *) (char *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    char temp2[64] ;
    int res2 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Section_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_section, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_name_set" "', argument " "1"" of type '" "struct _dwg_section *""'"); 
    }
    arg1 = (struct _dwg_section *)(argp1);
    res2 = SWIG_AsCharArray(ST(1), temp2, 64);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Section_name_set" "', argument " "2"" of type '" "char [64]""'");
    }
    arg2 = (char *)(temp2);
    if (arg2) memcpy(arg1->name,arg2,64*sizeof(char));
    else memset(arg1->name,0,64*sizeof(char));
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_name_get) {
  {
    struct _dwg_section *arg1 = (struct _dwg_section *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    char *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Section_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_section, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_name_get" "', argument " "1"" of type '" "struct _dwg_section *""'"); 
    }
    arg1 = (struct _dwg_section *)(argp1);
    result = (char *)(char *) ((arg1)->name);
    {
      size_t size = SWIG_strnlen(result, 64);
      
      
      
      ST(argvi) = SWIG_FromCharPtrAndSize(result, size); argvi++ ;
    }
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_section_type_set) {
  {
    struct _dwg_section *arg1 = (struct _dwg_section *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Section_section_type_set(self,section_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_section, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_section_type_set" "', argument " "1"" of type '" "struct _dwg_section *""'"); 
    }
    arg1 = (struct _dwg_section *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Section_section_type_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->section_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_section_type_get) {
  {
    struct _dwg_section *arg1 = (struct _dwg_section *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Section_section_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_section, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_section_type_get" "', argument " "1"" of type '" "struct _dwg_section *""'"); 
    }
    arg1 = (struct _dwg_section *)(argp1);
    result = (BITCODE_RL) ((arg1)->section_type);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_decomp_data_size_set) {
  {
    struct _dwg_section *arg1 = (struct _dwg_section *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Section_decomp_data_size_set(self,decomp_data_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_section, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_decomp_data_size_set" "', argument " "1"" of type '" "struct _dwg_section *""'"); 
    }
    arg1 = (struct _dwg_section *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Section_decomp_data_size_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->decomp_data_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_decomp_data_size_get) {
  {
    struct _dwg_section *arg1 = (struct _dwg_section *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Section_decomp_data_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_section, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_decomp_data_size_get" "', argument " "1"" of type '" "struct _dwg_section *""'"); 
    }
    arg1 = (struct _dwg_section *)(argp1);
    result = (BITCODE_RL) ((arg1)->decomp_data_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_comp_data_size_set) {
  {
    struct _dwg_section *arg1 = (struct _dwg_section *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Section_comp_data_size_set(self,comp_data_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_section, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_comp_data_size_set" "', argument " "1"" of type '" "struct _dwg_section *""'"); 
    }
    arg1 = (struct _dwg_section *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Section_comp_data_size_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->comp_data_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_comp_data_size_get) {
  {
    struct _dwg_section *arg1 = (struct _dwg_section *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Section_comp_data_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_section, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_comp_data_size_get" "', argument " "1"" of type '" "struct _dwg_section *""'"); 
    }
    arg1 = (struct _dwg_section *)(argp1);
    result = (BITCODE_RL) ((arg1)->comp_data_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_compression_type_set) {
  {
    struct _dwg_section *arg1 = (struct _dwg_section *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Section_compression_type_set(self,compression_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_section, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_compression_type_set" "', argument " "1"" of type '" "struct _dwg_section *""'"); 
    }
    arg1 = (struct _dwg_section *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Section_compression_type_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->compression_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_compression_type_get) {
  {
    struct _dwg_section *arg1 = (struct _dwg_section *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Section_compression_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_section, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_compression_type_get" "', argument " "1"" of type '" "struct _dwg_section *""'"); 
    }
    arg1 = (struct _dwg_section *)(argp1);
    result = (BITCODE_RL) ((arg1)->compression_type);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_checksum_set) {
  {
    struct _dwg_section *arg1 = (struct _dwg_section *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Section_checksum_set(self,checksum);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_section, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_checksum_set" "', argument " "1"" of type '" "struct _dwg_section *""'"); 
    }
    arg1 = (struct _dwg_section *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Section_checksum_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->checksum = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_checksum_get) {
  {
    struct _dwg_section *arg1 = (struct _dwg_section *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Section_checksum_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_section, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_checksum_get" "', argument " "1"" of type '" "struct _dwg_section *""'"); 
    }
    arg1 = (struct _dwg_section *)(argp1);
    result = (BITCODE_RL) ((arg1)->checksum);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_flags_r11_set) {
  {
    struct _dwg_section *arg1 = (struct _dwg_section *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Section_flags_r11_set(self,flags_r11);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_section, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_flags_r11_set" "', argument " "1"" of type '" "struct _dwg_section *""'"); 
    }
    arg1 = (struct _dwg_section *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Section_flags_r11_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->flags_r11 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_flags_r11_get) {
  {
    struct _dwg_section *arg1 = (struct _dwg_section *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Section_flags_r11_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_section, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_flags_r11_get" "', argument " "1"" of type '" "struct _dwg_section *""'"); 
    }
    arg1 = (struct _dwg_section *)(argp1);
    result = (BITCODE_RS) ((arg1)->flags_r11);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Section) {
  {
    int argvi = 0;
    struct _dwg_section *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Section();");
    }
    result = (struct _dwg_section *)calloc(1, sizeof(struct _dwg_section));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_section, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Section) {
  {
    struct _dwg_section *arg1 = (struct _dwg_section *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Section(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_section, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Section" "', argument " "1"" of type '" "struct _dwg_section *""'"); 
    }
    arg1 = (struct _dwg_section *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_InfoHdr_num_desc_set) {
  {
    Dwg_Section_InfoHdr *arg1 = (Dwg_Section_InfoHdr *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Section_InfoHdr_num_desc_set(self,num_desc);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_Dwg_Section_InfoHdr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_InfoHdr_num_desc_set" "', argument " "1"" of type '" "Dwg_Section_InfoHdr *""'"); 
    }
    arg1 = (Dwg_Section_InfoHdr *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Section_InfoHdr_num_desc_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->num_desc = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_InfoHdr_num_desc_get) {
  {
    Dwg_Section_InfoHdr *arg1 = (Dwg_Section_InfoHdr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Section_InfoHdr_num_desc_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_Dwg_Section_InfoHdr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_InfoHdr_num_desc_get" "', argument " "1"" of type '" "Dwg_Section_InfoHdr *""'"); 
    }
    arg1 = (Dwg_Section_InfoHdr *)(argp1);
    result = (BITCODE_RL) ((arg1)->num_desc);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_InfoHdr_compressed_set) {
  {
    Dwg_Section_InfoHdr *arg1 = (Dwg_Section_InfoHdr *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Section_InfoHdr_compressed_set(self,compressed);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_Dwg_Section_InfoHdr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_InfoHdr_compressed_set" "', argument " "1"" of type '" "Dwg_Section_InfoHdr *""'"); 
    }
    arg1 = (Dwg_Section_InfoHdr *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Section_InfoHdr_compressed_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->compressed = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_InfoHdr_compressed_get) {
  {
    Dwg_Section_InfoHdr *arg1 = (Dwg_Section_InfoHdr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Section_InfoHdr_compressed_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_Dwg_Section_InfoHdr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_InfoHdr_compressed_get" "', argument " "1"" of type '" "Dwg_Section_InfoHdr *""'"); 
    }
    arg1 = (Dwg_Section_InfoHdr *)(argp1);
    result = (BITCODE_RL) ((arg1)->compressed);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_InfoHdr_max_size_set) {
  {
    Dwg_Section_InfoHdr *arg1 = (Dwg_Section_InfoHdr *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Section_InfoHdr_max_size_set(self,max_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_Dwg_Section_InfoHdr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_InfoHdr_max_size_set" "', argument " "1"" of type '" "Dwg_Section_InfoHdr *""'"); 
    }
    arg1 = (Dwg_Section_InfoHdr *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Section_InfoHdr_max_size_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->max_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_InfoHdr_max_size_get) {
  {
    Dwg_Section_InfoHdr *arg1 = (Dwg_Section_InfoHdr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Section_InfoHdr_max_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_Dwg_Section_InfoHdr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_InfoHdr_max_size_get" "', argument " "1"" of type '" "Dwg_Section_InfoHdr *""'"); 
    }
    arg1 = (Dwg_Section_InfoHdr *)(argp1);
    result = (BITCODE_RL) ((arg1)->max_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_InfoHdr_encrypted_set) {
  {
    Dwg_Section_InfoHdr *arg1 = (Dwg_Section_InfoHdr *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Section_InfoHdr_encrypted_set(self,encrypted);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_Dwg_Section_InfoHdr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_InfoHdr_encrypted_set" "', argument " "1"" of type '" "Dwg_Section_InfoHdr *""'"); 
    }
    arg1 = (Dwg_Section_InfoHdr *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Section_InfoHdr_encrypted_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->encrypted = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_InfoHdr_encrypted_get) {
  {
    Dwg_Section_InfoHdr *arg1 = (Dwg_Section_InfoHdr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Section_InfoHdr_encrypted_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_Dwg_Section_InfoHdr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_InfoHdr_encrypted_get" "', argument " "1"" of type '" "Dwg_Section_InfoHdr *""'"); 
    }
    arg1 = (Dwg_Section_InfoHdr *)(argp1);
    result = (BITCODE_RL) ((arg1)->encrypted);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_InfoHdr_num_desc2_set) {
  {
    Dwg_Section_InfoHdr *arg1 = (Dwg_Section_InfoHdr *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Section_InfoHdr_num_desc2_set(self,num_desc2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_Dwg_Section_InfoHdr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_InfoHdr_num_desc2_set" "', argument " "1"" of type '" "Dwg_Section_InfoHdr *""'"); 
    }
    arg1 = (Dwg_Section_InfoHdr *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Section_InfoHdr_num_desc2_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->num_desc2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_InfoHdr_num_desc2_get) {
  {
    Dwg_Section_InfoHdr *arg1 = (Dwg_Section_InfoHdr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Section_InfoHdr_num_desc2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_Dwg_Section_InfoHdr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_InfoHdr_num_desc2_get" "', argument " "1"" of type '" "Dwg_Section_InfoHdr *""'"); 
    }
    arg1 = (Dwg_Section_InfoHdr *)(argp1);
    result = (BITCODE_RL) ((arg1)->num_desc2);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Section_InfoHdr) {
  {
    int argvi = 0;
    Dwg_Section_InfoHdr *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Section_InfoHdr();");
    }
    result = (Dwg_Section_InfoHdr *)calloc(1, sizeof(Dwg_Section_InfoHdr));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_Dwg_Section_InfoHdr, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Section_InfoHdr) {
  {
    Dwg_Section_InfoHdr *arg1 = (Dwg_Section_InfoHdr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Section_InfoHdr(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_Dwg_Section_InfoHdr, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Section_InfoHdr" "', argument " "1"" of type '" "Dwg_Section_InfoHdr *""'"); 
    }
    arg1 = (Dwg_Section_InfoHdr *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_Info_size_set) {
  {
    Dwg_Section_Info *arg1 = (Dwg_Section_Info *) 0 ;
    int64_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Section_Info_size_set(self,size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_Dwg_Section_Info, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_Info_size_set" "', argument " "1"" of type '" "Dwg_Section_Info *""'"); 
    }
    arg1 = (Dwg_Section_Info *)(argp1);
    ecode2 = SWIG_AsVal_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Section_Info_size_set" "', argument " "2"" of type '" "int64_t""'");
    } 
    arg2 = (int64_t)(val2);
    if (arg1) (arg1)->size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_Info_size_get) {
  {
    Dwg_Section_Info *arg1 = (Dwg_Section_Info *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int64_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Section_Info_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_Dwg_Section_Info, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_Info_size_get" "', argument " "1"" of type '" "Dwg_Section_Info *""'"); 
    }
    arg1 = (Dwg_Section_Info *)(argp1);
    result = (int64_t) ((arg1)->size);
    ST(argvi) = SWIG_From_long_SS_long  SWIG_PERL_CALL_ARGS_1((long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_Info_num_sections_set) {
  {
    Dwg_Section_Info *arg1 = (Dwg_Section_Info *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Section_Info_num_sections_set(self,num_sections);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_Dwg_Section_Info, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_Info_num_sections_set" "', argument " "1"" of type '" "Dwg_Section_Info *""'"); 
    }
    arg1 = (Dwg_Section_Info *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Section_Info_num_sections_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->num_sections = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_Info_num_sections_get) {
  {
    Dwg_Section_Info *arg1 = (Dwg_Section_Info *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Section_Info_num_sections_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_Dwg_Section_Info, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_Info_num_sections_get" "', argument " "1"" of type '" "Dwg_Section_Info *""'"); 
    }
    arg1 = (Dwg_Section_Info *)(argp1);
    result = (BITCODE_RL) ((arg1)->num_sections);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_Info_max_decomp_size_set) {
  {
    Dwg_Section_Info *arg1 = (Dwg_Section_Info *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Section_Info_max_decomp_size_set(self,max_decomp_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_Dwg_Section_Info, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_Info_max_decomp_size_set" "', argument " "1"" of type '" "Dwg_Section_Info *""'"); 
    }
    arg1 = (Dwg_Section_Info *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Section_Info_max_decomp_size_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->max_decomp_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_Info_max_decomp_size_get) {
  {
    Dwg_Section_Info *arg1 = (Dwg_Section_Info *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Section_Info_max_decomp_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_Dwg_Section_Info, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_Info_max_decomp_size_get" "', argument " "1"" of type '" "Dwg_Section_Info *""'"); 
    }
    arg1 = (Dwg_Section_Info *)(argp1);
    result = (BITCODE_RL) ((arg1)->max_decomp_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_Info_unknown_set) {
  {
    Dwg_Section_Info *arg1 = (Dwg_Section_Info *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Section_Info_unknown_set(self,unknown);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_Dwg_Section_Info, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_Info_unknown_set" "', argument " "1"" of type '" "Dwg_Section_Info *""'"); 
    }
    arg1 = (Dwg_Section_Info *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Section_Info_unknown_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->unknown = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_Info_unknown_get) {
  {
    Dwg_Section_Info *arg1 = (Dwg_Section_Info *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Section_Info_unknown_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_Dwg_Section_Info, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_Info_unknown_get" "', argument " "1"" of type '" "Dwg_Section_Info *""'"); 
    }
    arg1 = (Dwg_Section_Info *)(argp1);
    result = (BITCODE_RL) ((arg1)->unknown);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_Info_compressed_set) {
  {
    Dwg_Section_Info *arg1 = (Dwg_Section_Info *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Section_Info_compressed_set(self,compressed);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_Dwg_Section_Info, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_Info_compressed_set" "', argument " "1"" of type '" "Dwg_Section_Info *""'"); 
    }
    arg1 = (Dwg_Section_Info *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Section_Info_compressed_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->compressed = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_Info_compressed_get) {
  {
    Dwg_Section_Info *arg1 = (Dwg_Section_Info *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Section_Info_compressed_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_Dwg_Section_Info, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_Info_compressed_get" "', argument " "1"" of type '" "Dwg_Section_Info *""'"); 
    }
    arg1 = (Dwg_Section_Info *)(argp1);
    result = (BITCODE_RL) ((arg1)->compressed);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_Info_type_set) {
  {
    Dwg_Section_Info *arg1 = (Dwg_Section_Info *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Section_Info_type_set(self,type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_Dwg_Section_Info, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_Info_type_set" "', argument " "1"" of type '" "Dwg_Section_Info *""'"); 
    }
    arg1 = (Dwg_Section_Info *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Section_Info_type_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_Info_type_get) {
  {
    Dwg_Section_Info *arg1 = (Dwg_Section_Info *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Section_Info_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_Dwg_Section_Info, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_Info_type_get" "', argument " "1"" of type '" "Dwg_Section_Info *""'"); 
    }
    arg1 = (Dwg_Section_Info *)(argp1);
    result = (BITCODE_RL) ((arg1)->type);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_Info_encrypted_set) {
  {
    Dwg_Section_Info *arg1 = (Dwg_Section_Info *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Section_Info_encrypted_set(self,encrypted);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_Dwg_Section_Info, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_Info_encrypted_set" "', argument " "1"" of type '" "Dwg_Section_Info *""'"); 
    }
    arg1 = (Dwg_Section_Info *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Section_Info_encrypted_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->encrypted = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_Info_encrypted_get) {
  {
    Dwg_Section_Info *arg1 = (Dwg_Section_Info *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Section_Info_encrypted_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_Dwg_Section_Info, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_Info_encrypted_get" "', argument " "1"" of type '" "Dwg_Section_Info *""'"); 
    }
    arg1 = (Dwg_Section_Info *)(argp1);
    result = (BITCODE_RL) ((arg1)->encrypted);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_Info_name_set) {
  {
    Dwg_Section_Info *arg1 = (Dwg_Section_Info *) 0 ;
    char *arg2 = (char *) (char *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    char temp2[64] ;
    int res2 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Section_Info_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_Dwg_Section_Info, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_Info_name_set" "', argument " "1"" of type '" "Dwg_Section_Info *""'"); 
    }
    arg1 = (Dwg_Section_Info *)(argp1);
    res2 = SWIG_AsCharArray(ST(1), temp2, 64);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Section_Info_name_set" "', argument " "2"" of type '" "char [64]""'");
    }
    arg2 = (char *)(temp2);
    if (arg2) memcpy(arg1->name,arg2,64*sizeof(char));
    else memset(arg1->name,0,64*sizeof(char));
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_Info_name_get) {
  {
    Dwg_Section_Info *arg1 = (Dwg_Section_Info *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    char *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Section_Info_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_Dwg_Section_Info, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_Info_name_get" "', argument " "1"" of type '" "Dwg_Section_Info *""'"); 
    }
    arg1 = (Dwg_Section_Info *)(argp1);
    result = (char *)(char *) ((arg1)->name);
    {
      size_t size = SWIG_strnlen(result, 64);
      
      
      
      ST(argvi) = SWIG_FromCharPtrAndSize(result, size); argvi++ ;
    }
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_Info_fixedtype_set) {
  {
    Dwg_Section_Info *arg1 = (Dwg_Section_Info *) 0 ;
    Dwg_Section_Type arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Section_Info_fixedtype_set(self,fixedtype);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_Dwg_Section_Info, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_Info_fixedtype_set" "', argument " "1"" of type '" "Dwg_Section_Info *""'"); 
    }
    arg1 = (Dwg_Section_Info *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Section_Info_fixedtype_set" "', argument " "2"" of type '" "Dwg_Section_Type""'");
    } 
    arg2 = (Dwg_Section_Type)(val2);
    if (arg1) (arg1)->fixedtype = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_Info_fixedtype_get) {
  {
    Dwg_Section_Info *arg1 = (Dwg_Section_Info *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Section_Type result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Section_Info_fixedtype_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_Dwg_Section_Info, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_Info_fixedtype_get" "', argument " "1"" of type '" "Dwg_Section_Info *""'"); 
    }
    arg1 = (Dwg_Section_Info *)(argp1);
    result = (Dwg_Section_Type) ((arg1)->fixedtype);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_Info_sections_set) {
  {
    Dwg_Section_Info *arg1 = (Dwg_Section_Info *) 0 ;
    Dwg_Section **arg2 = (Dwg_Section **) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Section_Info_sections_set(self,sections);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_Dwg_Section_Info, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_Info_sections_set" "', argument " "1"" of type '" "Dwg_Section_Info *""'"); 
    }
    arg1 = (Dwg_Section_Info *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_section, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Section_Info_sections_set" "', argument " "2"" of type '" "Dwg_Section **""'"); 
    }
    arg2 = (Dwg_Section **)(argp2);
    if (arg1) (arg1)->sections = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Section_Info_sections_get) {
  {
    Dwg_Section_Info *arg1 = (Dwg_Section_Info *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Section **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Section_Info_sections_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_Dwg_Section_Info, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Section_Info_sections_get" "', argument " "1"" of type '" "Dwg_Section_Info *""'"); 
    }
    arg1 = (Dwg_Section_Info *)(argp1);
    result = (Dwg_Section **) ((arg1)->sections);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_section, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Section_Info) {
  {
    int argvi = 0;
    Dwg_Section_Info *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Section_Info();");
    }
    result = (Dwg_Section_Info *)calloc(1, sizeof(Dwg_Section_Info));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_Dwg_Section_Info, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Section_Info) {
  {
    Dwg_Section_Info *arg1 = (Dwg_Section_Info *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Section_Info(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_Dwg_Section_Info, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Section_Info" "', argument " "1"" of type '" "Dwg_Section_Info *""'"); 
    }
    arg1 = (Dwg_Section_Info *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SummaryInfo_Property_tag_set) {
  {
    struct _dwg_SummaryInfo_Property *arg1 = (struct _dwg_SummaryInfo_Property *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SummaryInfo_Property_tag_set(self,tag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SummaryInfo_Property, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SummaryInfo_Property_tag_set" "', argument " "1"" of type '" "struct _dwg_SummaryInfo_Property *""'"); 
    }
    arg1 = (struct _dwg_SummaryInfo_Property *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_SummaryInfo_Property_tag_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->tag);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->tag = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->tag = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SummaryInfo_Property_tag_get) {
  {
    struct _dwg_SummaryInfo_Property *arg1 = (struct _dwg_SummaryInfo_Property *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SummaryInfo_Property_tag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SummaryInfo_Property, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SummaryInfo_Property_tag_get" "', argument " "1"" of type '" "struct _dwg_SummaryInfo_Property *""'"); 
    }
    arg1 = (struct _dwg_SummaryInfo_Property *)(argp1);
    result = (BITCODE_TV) ((arg1)->tag);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SummaryInfo_Property_value_set) {
  {
    struct _dwg_SummaryInfo_Property *arg1 = (struct _dwg_SummaryInfo_Property *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SummaryInfo_Property_value_set(self,value);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SummaryInfo_Property, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SummaryInfo_Property_value_set" "', argument " "1"" of type '" "struct _dwg_SummaryInfo_Property *""'"); 
    }
    arg1 = (struct _dwg_SummaryInfo_Property *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_SummaryInfo_Property_value_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->value);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->value = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->value = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SummaryInfo_Property_value_get) {
  {
    struct _dwg_SummaryInfo_Property *arg1 = (struct _dwg_SummaryInfo_Property *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SummaryInfo_Property_value_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SummaryInfo_Property, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SummaryInfo_Property_value_get" "', argument " "1"" of type '" "struct _dwg_SummaryInfo_Property *""'"); 
    }
    arg1 = (struct _dwg_SummaryInfo_Property *)(argp1);
    result = (BITCODE_TV) ((arg1)->value);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_SummaryInfo_Property) {
  {
    int argvi = 0;
    struct _dwg_SummaryInfo_Property *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_SummaryInfo_Property();");
    }
    result = (struct _dwg_SummaryInfo_Property *)calloc(1, sizeof(struct _dwg_SummaryInfo_Property));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_SummaryInfo_Property, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_SummaryInfo_Property) {
  {
    struct _dwg_SummaryInfo_Property *arg1 = (struct _dwg_SummaryInfo_Property *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_SummaryInfo_Property(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_SummaryInfo_Property, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_SummaryInfo_Property" "', argument " "1"" of type '" "struct _dwg_SummaryInfo_Property *""'"); 
    }
    arg1 = (struct _dwg_SummaryInfo_Property *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FileDepList_Files_filename_set) {
  {
    struct _dwg_FileDepList_Files *arg1 = (struct _dwg_FileDepList_Files *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_FileDepList_Files_filename_set(self,filename);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FileDepList_Files, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FileDepList_Files_filename_set" "', argument " "1"" of type '" "struct _dwg_FileDepList_Files *""'"); 
    }
    arg1 = (struct _dwg_FileDepList_Files *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_FileDepList_Files_filename_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->filename);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->filename = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->filename = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FileDepList_Files_filename_get) {
  {
    struct _dwg_FileDepList_Files *arg1 = (struct _dwg_FileDepList_Files *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_FileDepList_Files_filename_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FileDepList_Files, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FileDepList_Files_filename_get" "', argument " "1"" of type '" "struct _dwg_FileDepList_Files *""'"); 
    }
    arg1 = (struct _dwg_FileDepList_Files *)(argp1);
    result = (BITCODE_TV) ((arg1)->filename);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FileDepList_Files_filepath_set) {
  {
    struct _dwg_FileDepList_Files *arg1 = (struct _dwg_FileDepList_Files *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_FileDepList_Files_filepath_set(self,filepath);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FileDepList_Files, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FileDepList_Files_filepath_set" "', argument " "1"" of type '" "struct _dwg_FileDepList_Files *""'"); 
    }
    arg1 = (struct _dwg_FileDepList_Files *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_FileDepList_Files_filepath_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->filepath);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->filepath = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->filepath = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FileDepList_Files_filepath_get) {
  {
    struct _dwg_FileDepList_Files *arg1 = (struct _dwg_FileDepList_Files *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_FileDepList_Files_filepath_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FileDepList_Files, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FileDepList_Files_filepath_get" "', argument " "1"" of type '" "struct _dwg_FileDepList_Files *""'"); 
    }
    arg1 = (struct _dwg_FileDepList_Files *)(argp1);
    result = (BITCODE_TV) ((arg1)->filepath);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FileDepList_Files_fingerprint_set) {
  {
    struct _dwg_FileDepList_Files *arg1 = (struct _dwg_FileDepList_Files *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_FileDepList_Files_fingerprint_set(self,fingerprint);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FileDepList_Files, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FileDepList_Files_fingerprint_set" "', argument " "1"" of type '" "struct _dwg_FileDepList_Files *""'"); 
    }
    arg1 = (struct _dwg_FileDepList_Files *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_FileDepList_Files_fingerprint_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->fingerprint);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->fingerprint = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->fingerprint = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FileDepList_Files_fingerprint_get) {
  {
    struct _dwg_FileDepList_Files *arg1 = (struct _dwg_FileDepList_Files *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_FileDepList_Files_fingerprint_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FileDepList_Files, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FileDepList_Files_fingerprint_get" "', argument " "1"" of type '" "struct _dwg_FileDepList_Files *""'"); 
    }
    arg1 = (struct _dwg_FileDepList_Files *)(argp1);
    result = (BITCODE_TV) ((arg1)->fingerprint);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FileDepList_Files_version_set) {
  {
    struct _dwg_FileDepList_Files *arg1 = (struct _dwg_FileDepList_Files *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_FileDepList_Files_version_set(self,version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FileDepList_Files, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FileDepList_Files_version_set" "', argument " "1"" of type '" "struct _dwg_FileDepList_Files *""'"); 
    }
    arg1 = (struct _dwg_FileDepList_Files *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_FileDepList_Files_version_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->version);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->version = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->version = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FileDepList_Files_version_get) {
  {
    struct _dwg_FileDepList_Files *arg1 = (struct _dwg_FileDepList_Files *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_FileDepList_Files_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FileDepList_Files, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FileDepList_Files_version_get" "', argument " "1"" of type '" "struct _dwg_FileDepList_Files *""'"); 
    }
    arg1 = (struct _dwg_FileDepList_Files *)(argp1);
    result = (BITCODE_TV) ((arg1)->version);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FileDepList_Files_feature_index_set) {
  {
    struct _dwg_FileDepList_Files *arg1 = (struct _dwg_FileDepList_Files *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_FileDepList_Files_feature_index_set(self,feature_index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FileDepList_Files, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FileDepList_Files_feature_index_set" "', argument " "1"" of type '" "struct _dwg_FileDepList_Files *""'"); 
    }
    arg1 = (struct _dwg_FileDepList_Files *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_FileDepList_Files_feature_index_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->feature_index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FileDepList_Files_feature_index_get) {
  {
    struct _dwg_FileDepList_Files *arg1 = (struct _dwg_FileDepList_Files *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_FileDepList_Files_feature_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FileDepList_Files, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FileDepList_Files_feature_index_get" "', argument " "1"" of type '" "struct _dwg_FileDepList_Files *""'"); 
    }
    arg1 = (struct _dwg_FileDepList_Files *)(argp1);
    result = (BITCODE_RL) ((arg1)->feature_index);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FileDepList_Files_timestamp_set) {
  {
    struct _dwg_FileDepList_Files *arg1 = (struct _dwg_FileDepList_Files *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_FileDepList_Files_timestamp_set(self,timestamp);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FileDepList_Files, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FileDepList_Files_timestamp_set" "', argument " "1"" of type '" "struct _dwg_FileDepList_Files *""'"); 
    }
    arg1 = (struct _dwg_FileDepList_Files *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_FileDepList_Files_timestamp_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->timestamp = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FileDepList_Files_timestamp_get) {
  {
    struct _dwg_FileDepList_Files *arg1 = (struct _dwg_FileDepList_Files *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_FileDepList_Files_timestamp_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FileDepList_Files, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FileDepList_Files_timestamp_get" "', argument " "1"" of type '" "struct _dwg_FileDepList_Files *""'"); 
    }
    arg1 = (struct _dwg_FileDepList_Files *)(argp1);
    result = (BITCODE_RL) ((arg1)->timestamp);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FileDepList_Files_filesize_set) {
  {
    struct _dwg_FileDepList_Files *arg1 = (struct _dwg_FileDepList_Files *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_FileDepList_Files_filesize_set(self,filesize);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FileDepList_Files, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FileDepList_Files_filesize_set" "', argument " "1"" of type '" "struct _dwg_FileDepList_Files *""'"); 
    }
    arg1 = (struct _dwg_FileDepList_Files *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_FileDepList_Files_filesize_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->filesize = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FileDepList_Files_filesize_get) {
  {
    struct _dwg_FileDepList_Files *arg1 = (struct _dwg_FileDepList_Files *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_FileDepList_Files_filesize_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FileDepList_Files, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FileDepList_Files_filesize_get" "', argument " "1"" of type '" "struct _dwg_FileDepList_Files *""'"); 
    }
    arg1 = (struct _dwg_FileDepList_Files *)(argp1);
    result = (BITCODE_RL) ((arg1)->filesize);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FileDepList_Files_affects_graphics_set) {
  {
    struct _dwg_FileDepList_Files *arg1 = (struct _dwg_FileDepList_Files *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_FileDepList_Files_affects_graphics_set(self,affects_graphics);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FileDepList_Files, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FileDepList_Files_affects_graphics_set" "', argument " "1"" of type '" "struct _dwg_FileDepList_Files *""'"); 
    }
    arg1 = (struct _dwg_FileDepList_Files *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_FileDepList_Files_affects_graphics_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->affects_graphics = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FileDepList_Files_affects_graphics_get) {
  {
    struct _dwg_FileDepList_Files *arg1 = (struct _dwg_FileDepList_Files *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_FileDepList_Files_affects_graphics_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FileDepList_Files, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FileDepList_Files_affects_graphics_get" "', argument " "1"" of type '" "struct _dwg_FileDepList_Files *""'"); 
    }
    arg1 = (struct _dwg_FileDepList_Files *)(argp1);
    result = (BITCODE_RS) ((arg1)->affects_graphics);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FileDepList_Files_refcount_set) {
  {
    struct _dwg_FileDepList_Files *arg1 = (struct _dwg_FileDepList_Files *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_FileDepList_Files_refcount_set(self,refcount);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FileDepList_Files, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FileDepList_Files_refcount_set" "', argument " "1"" of type '" "struct _dwg_FileDepList_Files *""'"); 
    }
    arg1 = (struct _dwg_FileDepList_Files *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_FileDepList_Files_refcount_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->refcount = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FileDepList_Files_refcount_get) {
  {
    struct _dwg_FileDepList_Files *arg1 = (struct _dwg_FileDepList_Files *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_FileDepList_Files_refcount_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FileDepList_Files, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FileDepList_Files_refcount_get" "', argument " "1"" of type '" "struct _dwg_FileDepList_Files *""'"); 
    }
    arg1 = (struct _dwg_FileDepList_Files *)(argp1);
    result = (BITCODE_RL) ((arg1)->refcount);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_FileDepList_Files) {
  {
    int argvi = 0;
    struct _dwg_FileDepList_Files *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_FileDepList_Files();");
    }
    result = (struct _dwg_FileDepList_Files *)calloc(1, sizeof(struct _dwg_FileDepList_Files));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_FileDepList_Files, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_FileDepList_Files) {
  {
    struct _dwg_FileDepList_Files *arg1 = (struct _dwg_FileDepList_Files *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_FileDepList_Files(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_FileDepList_Files, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_FileDepList_Files" "', argument " "1"" of type '" "struct _dwg_FileDepList_Files *""'"); 
    }
    arg1 = (struct _dwg_FileDepList_Files *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SegmentIndex_offset_set) {
  {
    struct _dwg_AcDs_SegmentIndex *arg1 = (struct _dwg_AcDs_SegmentIndex *) 0 ;
    BITCODE_RLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_SegmentIndex_offset_set(self,offset);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SegmentIndex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SegmentIndex_offset_set" "', argument " "1"" of type '" "struct _dwg_AcDs_SegmentIndex *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SegmentIndex *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_SegmentIndex_offset_set" "', argument " "2"" of type '" "BITCODE_RLL""'");
    } 
    arg2 = (BITCODE_RLL)(val2);
    if (arg1) (arg1)->offset = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SegmentIndex_offset_get) {
  {
    struct _dwg_AcDs_SegmentIndex *arg1 = (struct _dwg_AcDs_SegmentIndex *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_SegmentIndex_offset_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SegmentIndex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SegmentIndex_offset_get" "', argument " "1"" of type '" "struct _dwg_AcDs_SegmentIndex *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SegmentIndex *)(argp1);
    result = (BITCODE_RLL) ((arg1)->offset);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SegmentIndex_size_set) {
  {
    struct _dwg_AcDs_SegmentIndex *arg1 = (struct _dwg_AcDs_SegmentIndex *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_SegmentIndex_size_set(self,size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SegmentIndex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SegmentIndex_size_set" "', argument " "1"" of type '" "struct _dwg_AcDs_SegmentIndex *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SegmentIndex *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_SegmentIndex_size_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SegmentIndex_size_get) {
  {
    struct _dwg_AcDs_SegmentIndex *arg1 = (struct _dwg_AcDs_SegmentIndex *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_SegmentIndex_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SegmentIndex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SegmentIndex_size_get" "', argument " "1"" of type '" "struct _dwg_AcDs_SegmentIndex *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SegmentIndex *)(argp1);
    result = (BITCODE_RL) ((arg1)->size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_AcDs_SegmentIndex) {
  {
    int argvi = 0;
    struct _dwg_AcDs_SegmentIndex *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_AcDs_SegmentIndex();");
    }
    result = (struct _dwg_AcDs_SegmentIndex *)calloc(1, sizeof(struct _dwg_AcDs_SegmentIndex));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_SegmentIndex, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_AcDs_SegmentIndex) {
  {
    struct _dwg_AcDs_SegmentIndex *arg1 = (struct _dwg_AcDs_SegmentIndex *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_AcDs_SegmentIndex(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SegmentIndex, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_AcDs_SegmentIndex" "', argument " "1"" of type '" "struct _dwg_AcDs_SegmentIndex *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SegmentIndex *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataIndex_Entry_segidx_set) {
  {
    struct _dwg_AcDs_DataIndex_Entry *arg1 = (struct _dwg_AcDs_DataIndex_Entry *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_DataIndex_Entry_segidx_set(self,segidx);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataIndex_Entry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataIndex_Entry_segidx_set" "', argument " "1"" of type '" "struct _dwg_AcDs_DataIndex_Entry *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataIndex_Entry *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_DataIndex_Entry_segidx_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->segidx = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataIndex_Entry_segidx_get) {
  {
    struct _dwg_AcDs_DataIndex_Entry *arg1 = (struct _dwg_AcDs_DataIndex_Entry *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_DataIndex_Entry_segidx_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataIndex_Entry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataIndex_Entry_segidx_get" "', argument " "1"" of type '" "struct _dwg_AcDs_DataIndex_Entry *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataIndex_Entry *)(argp1);
    result = (BITCODE_RL) ((arg1)->segidx);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataIndex_Entry_offset_set) {
  {
    struct _dwg_AcDs_DataIndex_Entry *arg1 = (struct _dwg_AcDs_DataIndex_Entry *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_DataIndex_Entry_offset_set(self,offset);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataIndex_Entry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataIndex_Entry_offset_set" "', argument " "1"" of type '" "struct _dwg_AcDs_DataIndex_Entry *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataIndex_Entry *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_DataIndex_Entry_offset_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->offset = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataIndex_Entry_offset_get) {
  {
    struct _dwg_AcDs_DataIndex_Entry *arg1 = (struct _dwg_AcDs_DataIndex_Entry *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_DataIndex_Entry_offset_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataIndex_Entry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataIndex_Entry_offset_get" "', argument " "1"" of type '" "struct _dwg_AcDs_DataIndex_Entry *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataIndex_Entry *)(argp1);
    result = (BITCODE_RL) ((arg1)->offset);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataIndex_Entry_schidx_set) {
  {
    struct _dwg_AcDs_DataIndex_Entry *arg1 = (struct _dwg_AcDs_DataIndex_Entry *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_DataIndex_Entry_schidx_set(self,schidx);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataIndex_Entry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataIndex_Entry_schidx_set" "', argument " "1"" of type '" "struct _dwg_AcDs_DataIndex_Entry *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataIndex_Entry *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_DataIndex_Entry_schidx_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->schidx = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataIndex_Entry_schidx_get) {
  {
    struct _dwg_AcDs_DataIndex_Entry *arg1 = (struct _dwg_AcDs_DataIndex_Entry *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_DataIndex_Entry_schidx_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataIndex_Entry, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataIndex_Entry_schidx_get" "', argument " "1"" of type '" "struct _dwg_AcDs_DataIndex_Entry *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataIndex_Entry *)(argp1);
    result = (BITCODE_RL) ((arg1)->schidx);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_AcDs_DataIndex_Entry) {
  {
    int argvi = 0;
    struct _dwg_AcDs_DataIndex_Entry *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_AcDs_DataIndex_Entry();");
    }
    result = (struct _dwg_AcDs_DataIndex_Entry *)calloc(1, sizeof(struct _dwg_AcDs_DataIndex_Entry));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_DataIndex_Entry, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_AcDs_DataIndex_Entry) {
  {
    struct _dwg_AcDs_DataIndex_Entry *arg1 = (struct _dwg_AcDs_DataIndex_Entry *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_AcDs_DataIndex_Entry(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataIndex_Entry, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_AcDs_DataIndex_Entry" "', argument " "1"" of type '" "struct _dwg_AcDs_DataIndex_Entry *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataIndex_Entry *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataIndex_num_entries_set) {
  {
    struct _dwg_AcDs_DataIndex *arg1 = (struct _dwg_AcDs_DataIndex *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_DataIndex_num_entries_set(self,num_entries);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataIndex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataIndex_num_entries_set" "', argument " "1"" of type '" "struct _dwg_AcDs_DataIndex *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataIndex *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_DataIndex_num_entries_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->num_entries = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataIndex_num_entries_get) {
  {
    struct _dwg_AcDs_DataIndex *arg1 = (struct _dwg_AcDs_DataIndex *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_DataIndex_num_entries_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataIndex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataIndex_num_entries_get" "', argument " "1"" of type '" "struct _dwg_AcDs_DataIndex *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataIndex *)(argp1);
    result = (BITCODE_RL) ((arg1)->num_entries);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataIndex_di_unknown_set) {
  {
    struct _dwg_AcDs_DataIndex *arg1 = (struct _dwg_AcDs_DataIndex *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_DataIndex_di_unknown_set(self,di_unknown);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataIndex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataIndex_di_unknown_set" "', argument " "1"" of type '" "struct _dwg_AcDs_DataIndex *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataIndex *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_DataIndex_di_unknown_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->di_unknown = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataIndex_di_unknown_get) {
  {
    struct _dwg_AcDs_DataIndex *arg1 = (struct _dwg_AcDs_DataIndex *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_DataIndex_di_unknown_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataIndex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataIndex_di_unknown_get" "', argument " "1"" of type '" "struct _dwg_AcDs_DataIndex *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataIndex *)(argp1);
    result = (BITCODE_RL) ((arg1)->di_unknown);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataIndex_entries_set) {
  {
    struct _dwg_AcDs_DataIndex *arg1 = (struct _dwg_AcDs_DataIndex *) 0 ;
    Dwg_AcDs_DataIndex_Entry *arg2 = (Dwg_AcDs_DataIndex_Entry *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_DataIndex_entries_set(self,entries);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataIndex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataIndex_entries_set" "', argument " "1"" of type '" "struct _dwg_AcDs_DataIndex *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataIndex *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_AcDs_DataIndex_Entry, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AcDs_DataIndex_entries_set" "', argument " "2"" of type '" "Dwg_AcDs_DataIndex_Entry *""'"); 
    }
    arg2 = (Dwg_AcDs_DataIndex_Entry *)(argp2);
    if (arg1) (arg1)->entries = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataIndex_entries_get) {
  {
    struct _dwg_AcDs_DataIndex *arg1 = (struct _dwg_AcDs_DataIndex *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_AcDs_DataIndex_Entry *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_DataIndex_entries_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataIndex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataIndex_entries_get" "', argument " "1"" of type '" "struct _dwg_AcDs_DataIndex *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataIndex *)(argp1);
    result = (Dwg_AcDs_DataIndex_Entry *) ((arg1)->entries);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_DataIndex_Entry, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_AcDs_DataIndex) {
  {
    int argvi = 0;
    struct _dwg_AcDs_DataIndex *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_AcDs_DataIndex();");
    }
    result = (struct _dwg_AcDs_DataIndex *)calloc(1, sizeof(struct _dwg_AcDs_DataIndex));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_DataIndex, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_AcDs_DataIndex) {
  {
    struct _dwg_AcDs_DataIndex *arg1 = (struct _dwg_AcDs_DataIndex *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_AcDs_DataIndex(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataIndex, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_AcDs_DataIndex" "', argument " "1"" of type '" "struct _dwg_AcDs_DataIndex *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataIndex *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Data_RecordHdr_entry_size_set) {
  {
    struct _dwg_AcDs_Data_RecordHdr *arg1 = (struct _dwg_AcDs_Data_RecordHdr *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Data_RecordHdr_entry_size_set(self,entry_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Data_RecordHdr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Data_RecordHdr_entry_size_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Data_RecordHdr *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Data_RecordHdr *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_Data_RecordHdr_entry_size_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->entry_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Data_RecordHdr_entry_size_get) {
  {
    struct _dwg_AcDs_Data_RecordHdr *arg1 = (struct _dwg_AcDs_Data_RecordHdr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Data_RecordHdr_entry_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Data_RecordHdr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Data_RecordHdr_entry_size_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Data_RecordHdr *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Data_RecordHdr *)(argp1);
    result = (BITCODE_RL) ((arg1)->entry_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Data_RecordHdr_unknown_set) {
  {
    struct _dwg_AcDs_Data_RecordHdr *arg1 = (struct _dwg_AcDs_Data_RecordHdr *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Data_RecordHdr_unknown_set(self,unknown);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Data_RecordHdr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Data_RecordHdr_unknown_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Data_RecordHdr *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Data_RecordHdr *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_Data_RecordHdr_unknown_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->unknown = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Data_RecordHdr_unknown_get) {
  {
    struct _dwg_AcDs_Data_RecordHdr *arg1 = (struct _dwg_AcDs_Data_RecordHdr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Data_RecordHdr_unknown_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Data_RecordHdr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Data_RecordHdr_unknown_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Data_RecordHdr *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Data_RecordHdr *)(argp1);
    result = (BITCODE_RL) ((arg1)->unknown);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Data_RecordHdr_handle_set) {
  {
    struct _dwg_AcDs_Data_RecordHdr *arg1 = (struct _dwg_AcDs_Data_RecordHdr *) 0 ;
    BITCODE_RLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Data_RecordHdr_handle_set(self,handle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Data_RecordHdr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Data_RecordHdr_handle_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Data_RecordHdr *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Data_RecordHdr *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_Data_RecordHdr_handle_set" "', argument " "2"" of type '" "BITCODE_RLL""'");
    } 
    arg2 = (BITCODE_RLL)(val2);
    if (arg1) (arg1)->handle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Data_RecordHdr_handle_get) {
  {
    struct _dwg_AcDs_Data_RecordHdr *arg1 = (struct _dwg_AcDs_Data_RecordHdr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Data_RecordHdr_handle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Data_RecordHdr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Data_RecordHdr_handle_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Data_RecordHdr *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Data_RecordHdr *)(argp1);
    result = (BITCODE_RLL) ((arg1)->handle);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Data_RecordHdr_offset_set) {
  {
    struct _dwg_AcDs_Data_RecordHdr *arg1 = (struct _dwg_AcDs_Data_RecordHdr *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Data_RecordHdr_offset_set(self,offset);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Data_RecordHdr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Data_RecordHdr_offset_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Data_RecordHdr *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Data_RecordHdr *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_Data_RecordHdr_offset_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->offset = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Data_RecordHdr_offset_get) {
  {
    struct _dwg_AcDs_Data_RecordHdr *arg1 = (struct _dwg_AcDs_Data_RecordHdr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Data_RecordHdr_offset_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Data_RecordHdr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Data_RecordHdr_offset_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Data_RecordHdr *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Data_RecordHdr *)(argp1);
    result = (BITCODE_RL) ((arg1)->offset);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_AcDs_Data_RecordHdr) {
  {
    int argvi = 0;
    struct _dwg_AcDs_Data_RecordHdr *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_AcDs_Data_RecordHdr();");
    }
    result = (struct _dwg_AcDs_Data_RecordHdr *)calloc(1, sizeof(struct _dwg_AcDs_Data_RecordHdr));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_Data_RecordHdr, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_AcDs_Data_RecordHdr) {
  {
    struct _dwg_AcDs_Data_RecordHdr *arg1 = (struct _dwg_AcDs_Data_RecordHdr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_AcDs_Data_RecordHdr(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Data_RecordHdr, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_AcDs_Data_RecordHdr" "', argument " "1"" of type '" "struct _dwg_AcDs_Data_RecordHdr *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Data_RecordHdr *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Data_Record_data_size_set) {
  {
    struct _dwg_AcDs_Data_Record *arg1 = (struct _dwg_AcDs_Data_Record *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Data_Record_data_size_set(self,data_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Data_Record, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Data_Record_data_size_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Data_Record *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Data_Record *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_Data_Record_data_size_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->data_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Data_Record_data_size_get) {
  {
    struct _dwg_AcDs_Data_Record *arg1 = (struct _dwg_AcDs_Data_Record *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Data_Record_data_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Data_Record, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Data_Record_data_size_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Data_Record *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Data_Record *)(argp1);
    result = (BITCODE_RL) ((arg1)->data_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Data_Record_blob_set) {
  {
    struct _dwg_AcDs_Data_Record *arg1 = (struct _dwg_AcDs_Data_Record *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Data_Record_blob_set(self,blob);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Data_Record, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Data_Record_blob_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Data_Record *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Data_Record *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AcDs_Data_Record_blob_set" "', argument " "2"" of type '" "BITCODE_RC *""'"); 
    }
    arg2 = (BITCODE_RC *)(argp2);
    if (arg1) (arg1)->blob = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Data_Record_blob_get) {
  {
    struct _dwg_AcDs_Data_Record *arg1 = (struct _dwg_AcDs_Data_Record *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Data_Record_blob_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Data_Record, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Data_Record_blob_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Data_Record *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Data_Record *)(argp1);
    result = (BITCODE_RC *) ((arg1)->blob);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_AcDs_Data_Record) {
  {
    int argvi = 0;
    struct _dwg_AcDs_Data_Record *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_AcDs_Data_Record();");
    }
    result = (struct _dwg_AcDs_Data_Record *)calloc(1, sizeof(struct _dwg_AcDs_Data_Record));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_Data_Record, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_AcDs_Data_Record) {
  {
    struct _dwg_AcDs_Data_Record *arg1 = (struct _dwg_AcDs_Data_Record *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_AcDs_Data_Record(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Data_Record, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_AcDs_Data_Record" "', argument " "1"" of type '" "struct _dwg_AcDs_Data_Record *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Data_Record *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Data_record_hdrs_set) {
  {
    struct _dwg_AcDs_Data *arg1 = (struct _dwg_AcDs_Data *) 0 ;
    Dwg_AcDs_Data_RecordHdr *arg2 = (Dwg_AcDs_Data_RecordHdr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Data_record_hdrs_set(self,record_hdrs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Data, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Data_record_hdrs_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Data *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Data *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_AcDs_Data_RecordHdr, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AcDs_Data_record_hdrs_set" "', argument " "2"" of type '" "Dwg_AcDs_Data_RecordHdr *""'"); 
    }
    arg2 = (Dwg_AcDs_Data_RecordHdr *)(argp2);
    if (arg1) (arg1)->record_hdrs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Data_record_hdrs_get) {
  {
    struct _dwg_AcDs_Data *arg1 = (struct _dwg_AcDs_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_AcDs_Data_RecordHdr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Data_record_hdrs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Data, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Data_record_hdrs_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Data *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Data *)(argp1);
    result = (Dwg_AcDs_Data_RecordHdr *) ((arg1)->record_hdrs);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_Data_RecordHdr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Data_records_set) {
  {
    struct _dwg_AcDs_Data *arg1 = (struct _dwg_AcDs_Data *) 0 ;
    Dwg_AcDs_Data_Record *arg2 = (Dwg_AcDs_Data_Record *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Data_records_set(self,records);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Data, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Data_records_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Data *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Data *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_AcDs_Data_Record, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AcDs_Data_records_set" "', argument " "2"" of type '" "Dwg_AcDs_Data_Record *""'"); 
    }
    arg2 = (Dwg_AcDs_Data_Record *)(argp2);
    if (arg1) (arg1)->records = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Data_records_get) {
  {
    struct _dwg_AcDs_Data *arg1 = (struct _dwg_AcDs_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_AcDs_Data_Record *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Data_records_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Data, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Data_records_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Data *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Data *)(argp1);
    result = (Dwg_AcDs_Data_Record *) ((arg1)->records);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_Data_Record, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_AcDs_Data) {
  {
    int argvi = 0;
    struct _dwg_AcDs_Data *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_AcDs_Data();");
    }
    result = (struct _dwg_AcDs_Data *)calloc(1, sizeof(struct _dwg_AcDs_Data));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_Data, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_AcDs_Data) {
  {
    struct _dwg_AcDs_Data *arg1 = (struct _dwg_AcDs_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_AcDs_Data(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Data, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_AcDs_Data" "', argument " "1"" of type '" "struct _dwg_AcDs_Data *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Data *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlobRef_Page_segidx_set) {
  {
    struct _dwg_AcDs_DataBlobRef_Page *arg1 = (struct _dwg_AcDs_DataBlobRef_Page *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlobRef_Page_segidx_set(self,segidx);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlobRef_Page, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlobRef_Page_segidx_set" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlobRef_Page *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlobRef_Page *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_DataBlobRef_Page_segidx_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->segidx = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlobRef_Page_segidx_get) {
  {
    struct _dwg_AcDs_DataBlobRef_Page *arg1 = (struct _dwg_AcDs_DataBlobRef_Page *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlobRef_Page_segidx_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlobRef_Page, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlobRef_Page_segidx_get" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlobRef_Page *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlobRef_Page *)(argp1);
    result = (BITCODE_RL) ((arg1)->segidx);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlobRef_Page_size_set) {
  {
    struct _dwg_AcDs_DataBlobRef_Page *arg1 = (struct _dwg_AcDs_DataBlobRef_Page *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlobRef_Page_size_set(self,size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlobRef_Page, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlobRef_Page_size_set" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlobRef_Page *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlobRef_Page *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_DataBlobRef_Page_size_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlobRef_Page_size_get) {
  {
    struct _dwg_AcDs_DataBlobRef_Page *arg1 = (struct _dwg_AcDs_DataBlobRef_Page *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlobRef_Page_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlobRef_Page, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlobRef_Page_size_get" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlobRef_Page *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlobRef_Page *)(argp1);
    result = (BITCODE_RL) ((arg1)->size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_AcDs_DataBlobRef_Page) {
  {
    int argvi = 0;
    struct _dwg_AcDs_DataBlobRef_Page *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_AcDs_DataBlobRef_Page();");
    }
    result = (struct _dwg_AcDs_DataBlobRef_Page *)calloc(1, sizeof(struct _dwg_AcDs_DataBlobRef_Page));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_DataBlobRef_Page, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_AcDs_DataBlobRef_Page) {
  {
    struct _dwg_AcDs_DataBlobRef_Page *arg1 = (struct _dwg_AcDs_DataBlobRef_Page *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_AcDs_DataBlobRef_Page(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlobRef_Page, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_AcDs_DataBlobRef_Page" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlobRef_Page *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlobRef_Page *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlobRef_total_data_size_set) {
  {
    struct _dwg_AcDs_DataBlobRef *arg1 = (struct _dwg_AcDs_DataBlobRef *) 0 ;
    BITCODE_RLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlobRef_total_data_size_set(self,total_data_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlobRef, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlobRef_total_data_size_set" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlobRef *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlobRef *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_DataBlobRef_total_data_size_set" "', argument " "2"" of type '" "BITCODE_RLL""'");
    } 
    arg2 = (BITCODE_RLL)(val2);
    if (arg1) (arg1)->total_data_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlobRef_total_data_size_get) {
  {
    struct _dwg_AcDs_DataBlobRef *arg1 = (struct _dwg_AcDs_DataBlobRef *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlobRef_total_data_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlobRef, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlobRef_total_data_size_get" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlobRef *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlobRef *)(argp1);
    result = (BITCODE_RLL) ((arg1)->total_data_size);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlobRef_num_pages_set) {
  {
    struct _dwg_AcDs_DataBlobRef *arg1 = (struct _dwg_AcDs_DataBlobRef *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlobRef_num_pages_set(self,num_pages);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlobRef, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlobRef_num_pages_set" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlobRef *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlobRef *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_DataBlobRef_num_pages_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->num_pages = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlobRef_num_pages_get) {
  {
    struct _dwg_AcDs_DataBlobRef *arg1 = (struct _dwg_AcDs_DataBlobRef *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlobRef_num_pages_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlobRef, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlobRef_num_pages_get" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlobRef *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlobRef *)(argp1);
    result = (BITCODE_RL) ((arg1)->num_pages);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlobRef_record_size_set) {
  {
    struct _dwg_AcDs_DataBlobRef *arg1 = (struct _dwg_AcDs_DataBlobRef *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlobRef_record_size_set(self,record_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlobRef, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlobRef_record_size_set" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlobRef *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlobRef *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_DataBlobRef_record_size_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->record_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlobRef_record_size_get) {
  {
    struct _dwg_AcDs_DataBlobRef *arg1 = (struct _dwg_AcDs_DataBlobRef *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlobRef_record_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlobRef, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlobRef_record_size_get" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlobRef *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlobRef *)(argp1);
    result = (BITCODE_RL) ((arg1)->record_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlobRef_page_size_set) {
  {
    struct _dwg_AcDs_DataBlobRef *arg1 = (struct _dwg_AcDs_DataBlobRef *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlobRef_page_size_set(self,page_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlobRef, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlobRef_page_size_set" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlobRef *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlobRef *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_DataBlobRef_page_size_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->page_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlobRef_page_size_get) {
  {
    struct _dwg_AcDs_DataBlobRef *arg1 = (struct _dwg_AcDs_DataBlobRef *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlobRef_page_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlobRef, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlobRef_page_size_get" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlobRef *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlobRef *)(argp1);
    result = (BITCODE_RL) ((arg1)->page_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlobRef_unknown_1_set) {
  {
    struct _dwg_AcDs_DataBlobRef *arg1 = (struct _dwg_AcDs_DataBlobRef *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlobRef_unknown_1_set(self,unknown_1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlobRef, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlobRef_unknown_1_set" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlobRef *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlobRef *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_DataBlobRef_unknown_1_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->unknown_1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlobRef_unknown_1_get) {
  {
    struct _dwg_AcDs_DataBlobRef *arg1 = (struct _dwg_AcDs_DataBlobRef *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlobRef_unknown_1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlobRef, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlobRef_unknown_1_get" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlobRef *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlobRef *)(argp1);
    result = (BITCODE_RL) ((arg1)->unknown_1);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlobRef_unknown_2_set) {
  {
    struct _dwg_AcDs_DataBlobRef *arg1 = (struct _dwg_AcDs_DataBlobRef *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlobRef_unknown_2_set(self,unknown_2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlobRef, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlobRef_unknown_2_set" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlobRef *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlobRef *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_DataBlobRef_unknown_2_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->unknown_2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlobRef_unknown_2_get) {
  {
    struct _dwg_AcDs_DataBlobRef *arg1 = (struct _dwg_AcDs_DataBlobRef *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlobRef_unknown_2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlobRef, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlobRef_unknown_2_get" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlobRef *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlobRef *)(argp1);
    result = (BITCODE_RL) ((arg1)->unknown_2);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlobRef_pages_set) {
  {
    struct _dwg_AcDs_DataBlobRef *arg1 = (struct _dwg_AcDs_DataBlobRef *) 0 ;
    Dwg_AcDs_DataBlobRef_Page *arg2 = (Dwg_AcDs_DataBlobRef_Page *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlobRef_pages_set(self,pages);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlobRef, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlobRef_pages_set" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlobRef *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlobRef *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_AcDs_DataBlobRef_Page, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AcDs_DataBlobRef_pages_set" "', argument " "2"" of type '" "Dwg_AcDs_DataBlobRef_Page *""'"); 
    }
    arg2 = (Dwg_AcDs_DataBlobRef_Page *)(argp2);
    if (arg1) (arg1)->pages = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlobRef_pages_get) {
  {
    struct _dwg_AcDs_DataBlobRef *arg1 = (struct _dwg_AcDs_DataBlobRef *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_AcDs_DataBlobRef_Page *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlobRef_pages_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlobRef, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlobRef_pages_get" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlobRef *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlobRef *)(argp1);
    result = (Dwg_AcDs_DataBlobRef_Page *) ((arg1)->pages);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_DataBlobRef_Page, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_AcDs_DataBlobRef) {
  {
    int argvi = 0;
    struct _dwg_AcDs_DataBlobRef *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_AcDs_DataBlobRef();");
    }
    result = (struct _dwg_AcDs_DataBlobRef *)calloc(1, sizeof(struct _dwg_AcDs_DataBlobRef));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_DataBlobRef, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_AcDs_DataBlobRef) {
  {
    struct _dwg_AcDs_DataBlobRef *arg1 = (struct _dwg_AcDs_DataBlobRef *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_AcDs_DataBlobRef(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlobRef, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_AcDs_DataBlobRef" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlobRef *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlobRef *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlob_data_size_set) {
  {
    struct _dwg_AcDs_DataBlob *arg1 = (struct _dwg_AcDs_DataBlob *) 0 ;
    BITCODE_RLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlob_data_size_set(self,data_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlob, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlob_data_size_set" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlob *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlob *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_DataBlob_data_size_set" "', argument " "2"" of type '" "BITCODE_RLL""'");
    } 
    arg2 = (BITCODE_RLL)(val2);
    if (arg1) (arg1)->data_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlob_data_size_get) {
  {
    struct _dwg_AcDs_DataBlob *arg1 = (struct _dwg_AcDs_DataBlob *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlob_data_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlob, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlob_data_size_get" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlob *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlob *)(argp1);
    result = (BITCODE_RLL) ((arg1)->data_size);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlob_page_count_set) {
  {
    struct _dwg_AcDs_DataBlob *arg1 = (struct _dwg_AcDs_DataBlob *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlob_page_count_set(self,page_count);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlob, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlob_page_count_set" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlob *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlob *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_DataBlob_page_count_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->page_count = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlob_page_count_get) {
  {
    struct _dwg_AcDs_DataBlob *arg1 = (struct _dwg_AcDs_DataBlob *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlob_page_count_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlob, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlob_page_count_get" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlob *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlob *)(argp1);
    result = (BITCODE_RL) ((arg1)->page_count);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlob_record_size_set) {
  {
    struct _dwg_AcDs_DataBlob *arg1 = (struct _dwg_AcDs_DataBlob *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlob_record_size_set(self,record_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlob, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlob_record_size_set" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlob *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlob *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_DataBlob_record_size_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->record_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlob_record_size_get) {
  {
    struct _dwg_AcDs_DataBlob *arg1 = (struct _dwg_AcDs_DataBlob *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlob_record_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlob, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlob_record_size_get" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlob *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlob *)(argp1);
    result = (BITCODE_RL) ((arg1)->record_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlob_page_size_set) {
  {
    struct _dwg_AcDs_DataBlob *arg1 = (struct _dwg_AcDs_DataBlob *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlob_page_size_set(self,page_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlob, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlob_page_size_set" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlob *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlob *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_DataBlob_page_size_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->page_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlob_page_size_get) {
  {
    struct _dwg_AcDs_DataBlob *arg1 = (struct _dwg_AcDs_DataBlob *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlob_page_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlob, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlob_page_size_get" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlob *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlob *)(argp1);
    result = (BITCODE_RL) ((arg1)->page_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlob_unknown_1_set) {
  {
    struct _dwg_AcDs_DataBlob *arg1 = (struct _dwg_AcDs_DataBlob *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlob_unknown_1_set(self,unknown_1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlob, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlob_unknown_1_set" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlob *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlob *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_DataBlob_unknown_1_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->unknown_1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlob_unknown_1_get) {
  {
    struct _dwg_AcDs_DataBlob *arg1 = (struct _dwg_AcDs_DataBlob *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlob_unknown_1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlob, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlob_unknown_1_get" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlob *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlob *)(argp1);
    result = (BITCODE_RL) ((arg1)->unknown_1);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlob_unknown_2_set) {
  {
    struct _dwg_AcDs_DataBlob *arg1 = (struct _dwg_AcDs_DataBlob *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlob_unknown_2_set(self,unknown_2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlob, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlob_unknown_2_set" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlob *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlob *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_DataBlob_unknown_2_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->unknown_2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlob_unknown_2_get) {
  {
    struct _dwg_AcDs_DataBlob *arg1 = (struct _dwg_AcDs_DataBlob *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlob_unknown_2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlob, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlob_unknown_2_get" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlob *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlob *)(argp1);
    result = (BITCODE_RL) ((arg1)->unknown_2);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlob_ref_set) {
  {
    struct _dwg_AcDs_DataBlob *arg1 = (struct _dwg_AcDs_DataBlob *) 0 ;
    Dwg_AcDs_DataBlobRef *arg2 = (Dwg_AcDs_DataBlobRef *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlob_ref_set(self,ref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlob, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlob_ref_set" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlob *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlob *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_AcDs_DataBlobRef, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AcDs_DataBlob_ref_set" "', argument " "2"" of type '" "Dwg_AcDs_DataBlobRef *""'"); 
    }
    arg2 = (Dwg_AcDs_DataBlobRef *)(argp2);
    if (arg1) (arg1)->ref = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlob_ref_get) {
  {
    struct _dwg_AcDs_DataBlob *arg1 = (struct _dwg_AcDs_DataBlob *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_AcDs_DataBlobRef *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlob_ref_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlob, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlob_ref_get" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlob *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlob *)(argp1);
    result = (Dwg_AcDs_DataBlobRef *) ((arg1)->ref);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_DataBlobRef, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_AcDs_DataBlob) {
  {
    int argvi = 0;
    struct _dwg_AcDs_DataBlob *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_AcDs_DataBlob();");
    }
    result = (struct _dwg_AcDs_DataBlob *)calloc(1, sizeof(struct _dwg_AcDs_DataBlob));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_DataBlob, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_AcDs_DataBlob) {
  {
    struct _dwg_AcDs_DataBlob *arg1 = (struct _dwg_AcDs_DataBlob *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_AcDs_DataBlob(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlob, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_AcDs_DataBlob" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlob *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlob *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlob01_total_data_size_set) {
  {
    struct _dwg_AcDs_DataBlob01 *arg1 = (struct _dwg_AcDs_DataBlob01 *) 0 ;
    BITCODE_RLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlob01_total_data_size_set(self,total_data_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlob01, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlob01_total_data_size_set" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlob01 *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlob01 *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_DataBlob01_total_data_size_set" "', argument " "2"" of type '" "BITCODE_RLL""'");
    } 
    arg2 = (BITCODE_RLL)(val2);
    if (arg1) (arg1)->total_data_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlob01_total_data_size_get) {
  {
    struct _dwg_AcDs_DataBlob01 *arg1 = (struct _dwg_AcDs_DataBlob01 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlob01_total_data_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlob01, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlob01_total_data_size_get" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlob01 *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlob01 *)(argp1);
    result = (BITCODE_RLL) ((arg1)->total_data_size);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlob01_page_start_offset_set) {
  {
    struct _dwg_AcDs_DataBlob01 *arg1 = (struct _dwg_AcDs_DataBlob01 *) 0 ;
    BITCODE_RLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlob01_page_start_offset_set(self,page_start_offset);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlob01, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlob01_page_start_offset_set" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlob01 *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlob01 *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_DataBlob01_page_start_offset_set" "', argument " "2"" of type '" "BITCODE_RLL""'");
    } 
    arg2 = (BITCODE_RLL)(val2);
    if (arg1) (arg1)->page_start_offset = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlob01_page_start_offset_get) {
  {
    struct _dwg_AcDs_DataBlob01 *arg1 = (struct _dwg_AcDs_DataBlob01 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlob01_page_start_offset_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlob01, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlob01_page_start_offset_get" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlob01 *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlob01 *)(argp1);
    result = (BITCODE_RLL) ((arg1)->page_start_offset);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlob01_page_index_set) {
  {
    struct _dwg_AcDs_DataBlob01 *arg1 = (struct _dwg_AcDs_DataBlob01 *) 0 ;
    int32_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlob01_page_index_set(self,page_index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlob01, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlob01_page_index_set" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlob01 *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlob01 *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_DataBlob01_page_index_set" "', argument " "2"" of type '" "int32_t""'");
    } 
    arg2 = (int32_t)(val2);
    if (arg1) (arg1)->page_index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlob01_page_index_get) {
  {
    struct _dwg_AcDs_DataBlob01 *arg1 = (struct _dwg_AcDs_DataBlob01 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int32_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlob01_page_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlob01, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlob01_page_index_get" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlob01 *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlob01 *)(argp1);
    result = (int32_t) ((arg1)->page_index);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlob01_page_count_set) {
  {
    struct _dwg_AcDs_DataBlob01 *arg1 = (struct _dwg_AcDs_DataBlob01 *) 0 ;
    int32_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlob01_page_count_set(self,page_count);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlob01, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlob01_page_count_set" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlob01 *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlob01 *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_DataBlob01_page_count_set" "', argument " "2"" of type '" "int32_t""'");
    } 
    arg2 = (int32_t)(val2);
    if (arg1) (arg1)->page_count = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlob01_page_count_get) {
  {
    struct _dwg_AcDs_DataBlob01 *arg1 = (struct _dwg_AcDs_DataBlob01 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int32_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlob01_page_count_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlob01, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlob01_page_count_get" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlob01 *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlob01 *)(argp1);
    result = (int32_t) ((arg1)->page_count);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlob01_page_data_size_set) {
  {
    struct _dwg_AcDs_DataBlob01 *arg1 = (struct _dwg_AcDs_DataBlob01 *) 0 ;
    BITCODE_RLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlob01_page_data_size_set(self,page_data_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlob01, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlob01_page_data_size_set" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlob01 *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlob01 *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_DataBlob01_page_data_size_set" "', argument " "2"" of type '" "BITCODE_RLL""'");
    } 
    arg2 = (BITCODE_RLL)(val2);
    if (arg1) (arg1)->page_data_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlob01_page_data_size_get) {
  {
    struct _dwg_AcDs_DataBlob01 *arg1 = (struct _dwg_AcDs_DataBlob01 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlob01_page_data_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlob01, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlob01_page_data_size_get" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlob01 *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlob01 *)(argp1);
    result = (BITCODE_RLL) ((arg1)->page_data_size);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlob01_page_data_set) {
  {
    struct _dwg_AcDs_DataBlob01 *arg1 = (struct _dwg_AcDs_DataBlob01 *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlob01_page_data_set(self,page_data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlob01, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlob01_page_data_set" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlob01 *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlob01 *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AcDs_DataBlob01_page_data_set" "', argument " "2"" of type '" "BITCODE_RC *""'"); 
    }
    arg2 = (BITCODE_RC *)(argp2);
    if (arg1) (arg1)->page_data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_DataBlob01_page_data_get) {
  {
    struct _dwg_AcDs_DataBlob01 *arg1 = (struct _dwg_AcDs_DataBlob01 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_DataBlob01_page_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlob01, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_DataBlob01_page_data_get" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlob01 *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlob01 *)(argp1);
    result = (BITCODE_RC *) ((arg1)->page_data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_AcDs_DataBlob01) {
  {
    int argvi = 0;
    struct _dwg_AcDs_DataBlob01 *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_AcDs_DataBlob01();");
    }
    result = (struct _dwg_AcDs_DataBlob01 *)calloc(1, sizeof(struct _dwg_AcDs_DataBlob01));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_DataBlob01, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_AcDs_DataBlob01) {
  {
    struct _dwg_AcDs_DataBlob01 *arg1 = (struct _dwg_AcDs_DataBlob01 *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_AcDs_DataBlob01(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_DataBlob01, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_AcDs_DataBlob01" "', argument " "1"" of type '" "struct _dwg_AcDs_DataBlob01 *""'"); 
    }
    arg1 = (struct _dwg_AcDs_DataBlob01 *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaIndex_Prop_index_set) {
  {
    struct _dwg_AcDs_SchemaIndex_Prop *arg1 = (struct _dwg_AcDs_SchemaIndex_Prop *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaIndex_Prop_index_set(self,index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaIndex_Prop, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaIndex_Prop_index_set" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaIndex_Prop *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaIndex_Prop *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_SchemaIndex_Prop_index_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaIndex_Prop_index_get) {
  {
    struct _dwg_AcDs_SchemaIndex_Prop *arg1 = (struct _dwg_AcDs_SchemaIndex_Prop *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaIndex_Prop_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaIndex_Prop, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaIndex_Prop_index_get" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaIndex_Prop *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaIndex_Prop *)(argp1);
    result = (BITCODE_RL) ((arg1)->index);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaIndex_Prop_segidx_set) {
  {
    struct _dwg_AcDs_SchemaIndex_Prop *arg1 = (struct _dwg_AcDs_SchemaIndex_Prop *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaIndex_Prop_segidx_set(self,segidx);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaIndex_Prop, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaIndex_Prop_segidx_set" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaIndex_Prop *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaIndex_Prop *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_SchemaIndex_Prop_segidx_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->segidx = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaIndex_Prop_segidx_get) {
  {
    struct _dwg_AcDs_SchemaIndex_Prop *arg1 = (struct _dwg_AcDs_SchemaIndex_Prop *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaIndex_Prop_segidx_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaIndex_Prop, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaIndex_Prop_segidx_get" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaIndex_Prop *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaIndex_Prop *)(argp1);
    result = (BITCODE_RL) ((arg1)->segidx);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaIndex_Prop_offset_set) {
  {
    struct _dwg_AcDs_SchemaIndex_Prop *arg1 = (struct _dwg_AcDs_SchemaIndex_Prop *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaIndex_Prop_offset_set(self,offset);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaIndex_Prop, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaIndex_Prop_offset_set" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaIndex_Prop *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaIndex_Prop *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_SchemaIndex_Prop_offset_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->offset = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaIndex_Prop_offset_get) {
  {
    struct _dwg_AcDs_SchemaIndex_Prop *arg1 = (struct _dwg_AcDs_SchemaIndex_Prop *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaIndex_Prop_offset_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaIndex_Prop, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaIndex_Prop_offset_get" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaIndex_Prop *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaIndex_Prop *)(argp1);
    result = (BITCODE_RL) ((arg1)->offset);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_AcDs_SchemaIndex_Prop) {
  {
    int argvi = 0;
    struct _dwg_AcDs_SchemaIndex_Prop *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_AcDs_SchemaIndex_Prop();");
    }
    result = (struct _dwg_AcDs_SchemaIndex_Prop *)calloc(1, sizeof(struct _dwg_AcDs_SchemaIndex_Prop));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_SchemaIndex_Prop, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_AcDs_SchemaIndex_Prop) {
  {
    struct _dwg_AcDs_SchemaIndex_Prop *arg1 = (struct _dwg_AcDs_SchemaIndex_Prop *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_AcDs_SchemaIndex_Prop(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaIndex_Prop, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_AcDs_SchemaIndex_Prop" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaIndex_Prop *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaIndex_Prop *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaIndex_num_props_set) {
  {
    struct _dwg_AcDs_SchemaIndex *arg1 = (struct _dwg_AcDs_SchemaIndex *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaIndex_num_props_set(self,num_props);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaIndex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaIndex_num_props_set" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaIndex *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaIndex *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_SchemaIndex_num_props_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->num_props = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaIndex_num_props_get) {
  {
    struct _dwg_AcDs_SchemaIndex *arg1 = (struct _dwg_AcDs_SchemaIndex *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaIndex_num_props_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaIndex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaIndex_num_props_get" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaIndex *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaIndex *)(argp1);
    result = (BITCODE_RL) ((arg1)->num_props);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaIndex_si_unknown_1_set) {
  {
    struct _dwg_AcDs_SchemaIndex *arg1 = (struct _dwg_AcDs_SchemaIndex *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaIndex_si_unknown_1_set(self,si_unknown_1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaIndex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaIndex_si_unknown_1_set" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaIndex *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaIndex *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_SchemaIndex_si_unknown_1_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->si_unknown_1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaIndex_si_unknown_1_get) {
  {
    struct _dwg_AcDs_SchemaIndex *arg1 = (struct _dwg_AcDs_SchemaIndex *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaIndex_si_unknown_1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaIndex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaIndex_si_unknown_1_get" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaIndex *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaIndex *)(argp1);
    result = (BITCODE_RL) ((arg1)->si_unknown_1);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaIndex_props_set) {
  {
    struct _dwg_AcDs_SchemaIndex *arg1 = (struct _dwg_AcDs_SchemaIndex *) 0 ;
    Dwg_AcDs_SchemaIndex_Prop *arg2 = (Dwg_AcDs_SchemaIndex_Prop *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaIndex_props_set(self,props);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaIndex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaIndex_props_set" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaIndex *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaIndex *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_AcDs_SchemaIndex_Prop, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AcDs_SchemaIndex_props_set" "', argument " "2"" of type '" "Dwg_AcDs_SchemaIndex_Prop *""'"); 
    }
    arg2 = (Dwg_AcDs_SchemaIndex_Prop *)(argp2);
    if (arg1) (arg1)->props = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaIndex_props_get) {
  {
    struct _dwg_AcDs_SchemaIndex *arg1 = (struct _dwg_AcDs_SchemaIndex *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_AcDs_SchemaIndex_Prop *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaIndex_props_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaIndex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaIndex_props_get" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaIndex *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaIndex *)(argp1);
    result = (Dwg_AcDs_SchemaIndex_Prop *) ((arg1)->props);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_SchemaIndex_Prop, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaIndex_si_tag_set) {
  {
    struct _dwg_AcDs_SchemaIndex *arg1 = (struct _dwg_AcDs_SchemaIndex *) 0 ;
    BITCODE_RLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaIndex_si_tag_set(self,si_tag);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaIndex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaIndex_si_tag_set" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaIndex *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaIndex *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_SchemaIndex_si_tag_set" "', argument " "2"" of type '" "BITCODE_RLL""'");
    } 
    arg2 = (BITCODE_RLL)(val2);
    if (arg1) (arg1)->si_tag = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaIndex_si_tag_get) {
  {
    struct _dwg_AcDs_SchemaIndex *arg1 = (struct _dwg_AcDs_SchemaIndex *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaIndex_si_tag_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaIndex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaIndex_si_tag_get" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaIndex *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaIndex *)(argp1);
    result = (BITCODE_RLL) ((arg1)->si_tag);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaIndex_num_prop_entries_set) {
  {
    struct _dwg_AcDs_SchemaIndex *arg1 = (struct _dwg_AcDs_SchemaIndex *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaIndex_num_prop_entries_set(self,num_prop_entries);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaIndex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaIndex_num_prop_entries_set" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaIndex *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaIndex *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_SchemaIndex_num_prop_entries_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->num_prop_entries = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaIndex_num_prop_entries_get) {
  {
    struct _dwg_AcDs_SchemaIndex *arg1 = (struct _dwg_AcDs_SchemaIndex *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaIndex_num_prop_entries_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaIndex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaIndex_num_prop_entries_get" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaIndex *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaIndex *)(argp1);
    result = (BITCODE_RL) ((arg1)->num_prop_entries);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaIndex_si_unknown_2_set) {
  {
    struct _dwg_AcDs_SchemaIndex *arg1 = (struct _dwg_AcDs_SchemaIndex *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaIndex_si_unknown_2_set(self,si_unknown_2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaIndex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaIndex_si_unknown_2_set" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaIndex *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaIndex *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_SchemaIndex_si_unknown_2_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->si_unknown_2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaIndex_si_unknown_2_get) {
  {
    struct _dwg_AcDs_SchemaIndex *arg1 = (struct _dwg_AcDs_SchemaIndex *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaIndex_si_unknown_2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaIndex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaIndex_si_unknown_2_get" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaIndex *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaIndex *)(argp1);
    result = (BITCODE_RL) ((arg1)->si_unknown_2);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaIndex_prop_entries_set) {
  {
    struct _dwg_AcDs_SchemaIndex *arg1 = (struct _dwg_AcDs_SchemaIndex *) 0 ;
    Dwg_AcDs_SchemaIndex_Prop *arg2 = (Dwg_AcDs_SchemaIndex_Prop *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaIndex_prop_entries_set(self,prop_entries);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaIndex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaIndex_prop_entries_set" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaIndex *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaIndex *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_AcDs_SchemaIndex_Prop, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AcDs_SchemaIndex_prop_entries_set" "', argument " "2"" of type '" "Dwg_AcDs_SchemaIndex_Prop *""'"); 
    }
    arg2 = (Dwg_AcDs_SchemaIndex_Prop *)(argp2);
    if (arg1) (arg1)->prop_entries = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaIndex_prop_entries_get) {
  {
    struct _dwg_AcDs_SchemaIndex *arg1 = (struct _dwg_AcDs_SchemaIndex *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_AcDs_SchemaIndex_Prop *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaIndex_prop_entries_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaIndex, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaIndex_prop_entries_get" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaIndex *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaIndex *)(argp1);
    result = (Dwg_AcDs_SchemaIndex_Prop *) ((arg1)->prop_entries);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_SchemaIndex_Prop, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_AcDs_SchemaIndex) {
  {
    int argvi = 0;
    struct _dwg_AcDs_SchemaIndex *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_AcDs_SchemaIndex();");
    }
    result = (struct _dwg_AcDs_SchemaIndex *)calloc(1, sizeof(struct _dwg_AcDs_SchemaIndex));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_SchemaIndex, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_AcDs_SchemaIndex) {
  {
    struct _dwg_AcDs_SchemaIndex *arg1 = (struct _dwg_AcDs_SchemaIndex *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_AcDs_SchemaIndex(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaIndex, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_AcDs_SchemaIndex" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaIndex *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaIndex *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Schema_Prop_flags_set) {
  {
    struct _dwg_AcDs_Schema_Prop *arg1 = (struct _dwg_AcDs_Schema_Prop *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Schema_Prop_flags_set(self,flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Schema_Prop, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Schema_Prop_flags_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Schema_Prop *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Schema_Prop *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_Schema_Prop_flags_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Schema_Prop_flags_get) {
  {
    struct _dwg_AcDs_Schema_Prop *arg1 = (struct _dwg_AcDs_Schema_Prop *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Schema_Prop_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Schema_Prop, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Schema_Prop_flags_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Schema_Prop *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Schema_Prop *)(argp1);
    result = (BITCODE_RL) ((arg1)->flags);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Schema_Prop_namidx_set) {
  {
    struct _dwg_AcDs_Schema_Prop *arg1 = (struct _dwg_AcDs_Schema_Prop *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Schema_Prop_namidx_set(self,namidx);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Schema_Prop, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Schema_Prop_namidx_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Schema_Prop *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Schema_Prop *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_Schema_Prop_namidx_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->namidx = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Schema_Prop_namidx_get) {
  {
    struct _dwg_AcDs_Schema_Prop *arg1 = (struct _dwg_AcDs_Schema_Prop *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Schema_Prop_namidx_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Schema_Prop, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Schema_Prop_namidx_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Schema_Prop *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Schema_Prop *)(argp1);
    result = (BITCODE_RL) ((arg1)->namidx);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Schema_Prop_type_set) {
  {
    struct _dwg_AcDs_Schema_Prop *arg1 = (struct _dwg_AcDs_Schema_Prop *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Schema_Prop_type_set(self,type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Schema_Prop, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Schema_Prop_type_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Schema_Prop *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Schema_Prop *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_Schema_Prop_type_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Schema_Prop_type_get) {
  {
    struct _dwg_AcDs_Schema_Prop *arg1 = (struct _dwg_AcDs_Schema_Prop *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Schema_Prop_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Schema_Prop, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Schema_Prop_type_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Schema_Prop *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Schema_Prop *)(argp1);
    result = (BITCODE_RL) ((arg1)->type);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Schema_Prop_type_size_set) {
  {
    struct _dwg_AcDs_Schema_Prop *arg1 = (struct _dwg_AcDs_Schema_Prop *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Schema_Prop_type_size_set(self,type_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Schema_Prop, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Schema_Prop_type_size_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Schema_Prop *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Schema_Prop *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_Schema_Prop_type_size_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->type_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Schema_Prop_type_size_get) {
  {
    struct _dwg_AcDs_Schema_Prop *arg1 = (struct _dwg_AcDs_Schema_Prop *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Schema_Prop_type_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Schema_Prop, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Schema_Prop_type_size_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Schema_Prop *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Schema_Prop *)(argp1);
    result = (BITCODE_RL) ((arg1)->type_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Schema_Prop_unknown_1_set) {
  {
    struct _dwg_AcDs_Schema_Prop *arg1 = (struct _dwg_AcDs_Schema_Prop *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Schema_Prop_unknown_1_set(self,unknown_1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Schema_Prop, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Schema_Prop_unknown_1_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Schema_Prop *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Schema_Prop *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_Schema_Prop_unknown_1_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->unknown_1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Schema_Prop_unknown_1_get) {
  {
    struct _dwg_AcDs_Schema_Prop *arg1 = (struct _dwg_AcDs_Schema_Prop *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Schema_Prop_unknown_1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Schema_Prop, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Schema_Prop_unknown_1_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Schema_Prop *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Schema_Prop *)(argp1);
    result = (BITCODE_RL) ((arg1)->unknown_1);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Schema_Prop_unknown_2_set) {
  {
    struct _dwg_AcDs_Schema_Prop *arg1 = (struct _dwg_AcDs_Schema_Prop *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Schema_Prop_unknown_2_set(self,unknown_2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Schema_Prop, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Schema_Prop_unknown_2_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Schema_Prop *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Schema_Prop *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_Schema_Prop_unknown_2_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->unknown_2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Schema_Prop_unknown_2_get) {
  {
    struct _dwg_AcDs_Schema_Prop *arg1 = (struct _dwg_AcDs_Schema_Prop *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Schema_Prop_unknown_2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Schema_Prop, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Schema_Prop_unknown_2_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Schema_Prop *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Schema_Prop *)(argp1);
    result = (BITCODE_RL) ((arg1)->unknown_2);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Schema_Prop_num_values_set) {
  {
    struct _dwg_AcDs_Schema_Prop *arg1 = (struct _dwg_AcDs_Schema_Prop *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Schema_Prop_num_values_set(self,num_values);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Schema_Prop, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Schema_Prop_num_values_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Schema_Prop *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Schema_Prop *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_Schema_Prop_num_values_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->num_values = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Schema_Prop_num_values_get) {
  {
    struct _dwg_AcDs_Schema_Prop *arg1 = (struct _dwg_AcDs_Schema_Prop *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Schema_Prop_num_values_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Schema_Prop, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Schema_Prop_num_values_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Schema_Prop *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Schema_Prop *)(argp1);
    result = (BITCODE_RS) ((arg1)->num_values);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Schema_Prop_values_set) {
  {
    struct _dwg_AcDs_Schema_Prop *arg1 = (struct _dwg_AcDs_Schema_Prop *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Schema_Prop_values_set(self,values);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Schema_Prop, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Schema_Prop_values_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Schema_Prop *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Schema_Prop *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AcDs_Schema_Prop_values_set" "', argument " "2"" of type '" "BITCODE_RC *""'"); 
    }
    arg2 = (BITCODE_RC *)(argp2);
    if (arg1) (arg1)->values = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Schema_Prop_values_get) {
  {
    struct _dwg_AcDs_Schema_Prop *arg1 = (struct _dwg_AcDs_Schema_Prop *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Schema_Prop_values_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Schema_Prop, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Schema_Prop_values_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Schema_Prop *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Schema_Prop *)(argp1);
    result = (BITCODE_RC *) ((arg1)->values);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_AcDs_Schema_Prop) {
  {
    int argvi = 0;
    struct _dwg_AcDs_Schema_Prop *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_AcDs_Schema_Prop();");
    }
    result = (struct _dwg_AcDs_Schema_Prop *)calloc(1, sizeof(struct _dwg_AcDs_Schema_Prop));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_Schema_Prop, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_AcDs_Schema_Prop) {
  {
    struct _dwg_AcDs_Schema_Prop *arg1 = (struct _dwg_AcDs_Schema_Prop *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_AcDs_Schema_Prop(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Schema_Prop, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_AcDs_Schema_Prop" "', argument " "1"" of type '" "struct _dwg_AcDs_Schema_Prop *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Schema_Prop *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Schema_num_index_set) {
  {
    struct _dwg_AcDs_Schema *arg1 = (struct _dwg_AcDs_Schema *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Schema_num_index_set(self,num_index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Schema, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Schema_num_index_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Schema *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Schema *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_Schema_num_index_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->num_index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Schema_num_index_get) {
  {
    struct _dwg_AcDs_Schema *arg1 = (struct _dwg_AcDs_Schema *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Schema_num_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Schema, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Schema_num_index_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Schema *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Schema *)(argp1);
    result = (BITCODE_RS) ((arg1)->num_index);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Schema_index_set) {
  {
    struct _dwg_AcDs_Schema *arg1 = (struct _dwg_AcDs_Schema *) 0 ;
    BITCODE_RLL *arg2 = (BITCODE_RLL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Schema_index_set(self,index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Schema, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Schema_index_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Schema *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Schema *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_long_long, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AcDs_Schema_index_set" "', argument " "2"" of type '" "BITCODE_RLL *""'"); 
    }
    arg2 = (BITCODE_RLL *)(argp2);
    if (arg1) (arg1)->index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Schema_index_get) {
  {
    struct _dwg_AcDs_Schema *arg1 = (struct _dwg_AcDs_Schema *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Schema_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Schema, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Schema_index_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Schema *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Schema *)(argp1);
    result = (BITCODE_RLL *) ((arg1)->index);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_long_long, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Schema_num_props_set) {
  {
    struct _dwg_AcDs_Schema *arg1 = (struct _dwg_AcDs_Schema *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Schema_num_props_set(self,num_props);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Schema, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Schema_num_props_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Schema *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Schema *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_Schema_num_props_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->num_props = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Schema_num_props_get) {
  {
    struct _dwg_AcDs_Schema *arg1 = (struct _dwg_AcDs_Schema *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Schema_num_props_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Schema, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Schema_num_props_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Schema *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Schema *)(argp1);
    result = (BITCODE_RS) ((arg1)->num_props);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Schema_props_set) {
  {
    struct _dwg_AcDs_Schema *arg1 = (struct _dwg_AcDs_Schema *) 0 ;
    Dwg_AcDs_Schema_Prop *arg2 = (Dwg_AcDs_Schema_Prop *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Schema_props_set(self,props);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Schema, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Schema_props_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Schema *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Schema *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_AcDs_Schema_Prop, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AcDs_Schema_props_set" "', argument " "2"" of type '" "Dwg_AcDs_Schema_Prop *""'"); 
    }
    arg2 = (Dwg_AcDs_Schema_Prop *)(argp2);
    if (arg1) (arg1)->props = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Schema_props_get) {
  {
    struct _dwg_AcDs_Schema *arg1 = (struct _dwg_AcDs_Schema *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_AcDs_Schema_Prop *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Schema_props_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Schema, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Schema_props_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Schema *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Schema *)(argp1);
    result = (Dwg_AcDs_Schema_Prop *) ((arg1)->props);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_Schema_Prop, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_AcDs_Schema) {
  {
    int argvi = 0;
    struct _dwg_AcDs_Schema *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_AcDs_Schema();");
    }
    result = (struct _dwg_AcDs_Schema *)calloc(1, sizeof(struct _dwg_AcDs_Schema));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_Schema, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_AcDs_Schema) {
  {
    struct _dwg_AcDs_Schema *arg1 = (struct _dwg_AcDs_Schema *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_AcDs_Schema(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Schema, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_AcDs_Schema" "', argument " "1"" of type '" "struct _dwg_AcDs_Schema *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Schema *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaData_UProp_size_set) {
  {
    struct _dwg_AcDs_SchemaData_UProp *arg1 = (struct _dwg_AcDs_SchemaData_UProp *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaData_UProp_size_set(self,size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaData_UProp, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaData_UProp_size_set" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaData_UProp *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaData_UProp *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_SchemaData_UProp_size_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaData_UProp_size_get) {
  {
    struct _dwg_AcDs_SchemaData_UProp *arg1 = (struct _dwg_AcDs_SchemaData_UProp *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaData_UProp_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaData_UProp, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaData_UProp_size_get" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaData_UProp *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaData_UProp *)(argp1);
    result = (BITCODE_RL) ((arg1)->size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaData_UProp_flags_set) {
  {
    struct _dwg_AcDs_SchemaData_UProp *arg1 = (struct _dwg_AcDs_SchemaData_UProp *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaData_UProp_flags_set(self,flags);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaData_UProp, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaData_UProp_flags_set" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaData_UProp *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaData_UProp *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_SchemaData_UProp_flags_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->flags = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaData_UProp_flags_get) {
  {
    struct _dwg_AcDs_SchemaData_UProp *arg1 = (struct _dwg_AcDs_SchemaData_UProp *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaData_UProp_flags_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaData_UProp, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaData_UProp_flags_get" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaData_UProp *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaData_UProp *)(argp1);
    result = (BITCODE_RL) ((arg1)->flags);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_AcDs_SchemaData_UProp) {
  {
    int argvi = 0;
    struct _dwg_AcDs_SchemaData_UProp *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_AcDs_SchemaData_UProp();");
    }
    result = (struct _dwg_AcDs_SchemaData_UProp *)calloc(1, sizeof(struct _dwg_AcDs_SchemaData_UProp));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_SchemaData_UProp, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_AcDs_SchemaData_UProp) {
  {
    struct _dwg_AcDs_SchemaData_UProp *arg1 = (struct _dwg_AcDs_SchemaData_UProp *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_AcDs_SchemaData_UProp(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaData_UProp, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_AcDs_SchemaData_UProp" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaData_UProp *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaData_UProp *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaData_num_uprops_set) {
  {
    struct _dwg_AcDs_SchemaData *arg1 = (struct _dwg_AcDs_SchemaData *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaData_num_uprops_set(self,num_uprops);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaData_num_uprops_set" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaData *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaData *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_SchemaData_num_uprops_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->num_uprops = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaData_num_uprops_get) {
  {
    struct _dwg_AcDs_SchemaData *arg1 = (struct _dwg_AcDs_SchemaData *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaData_num_uprops_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaData_num_uprops_get" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaData *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaData *)(argp1);
    result = (BITCODE_RL) ((arg1)->num_uprops);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaData_uprops_set) {
  {
    struct _dwg_AcDs_SchemaData *arg1 = (struct _dwg_AcDs_SchemaData *) 0 ;
    Dwg_AcDs_SchemaData_UProp *arg2 = (Dwg_AcDs_SchemaData_UProp *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaData_uprops_set(self,uprops);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaData_uprops_set" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaData *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaData *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_AcDs_SchemaData_UProp, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AcDs_SchemaData_uprops_set" "', argument " "2"" of type '" "Dwg_AcDs_SchemaData_UProp *""'"); 
    }
    arg2 = (Dwg_AcDs_SchemaData_UProp *)(argp2);
    if (arg1) (arg1)->uprops = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaData_uprops_get) {
  {
    struct _dwg_AcDs_SchemaData *arg1 = (struct _dwg_AcDs_SchemaData *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_AcDs_SchemaData_UProp *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaData_uprops_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaData_uprops_get" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaData *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaData *)(argp1);
    result = (Dwg_AcDs_SchemaData_UProp *) ((arg1)->uprops);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_SchemaData_UProp, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaData_num_schemas_set) {
  {
    struct _dwg_AcDs_SchemaData *arg1 = (struct _dwg_AcDs_SchemaData *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaData_num_schemas_set(self,num_schemas);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaData_num_schemas_set" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaData *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaData *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_SchemaData_num_schemas_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->num_schemas = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaData_num_schemas_get) {
  {
    struct _dwg_AcDs_SchemaData *arg1 = (struct _dwg_AcDs_SchemaData *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaData_num_schemas_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaData_num_schemas_get" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaData *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaData *)(argp1);
    result = (BITCODE_RL) ((arg1)->num_schemas);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaData_schemas_set) {
  {
    struct _dwg_AcDs_SchemaData *arg1 = (struct _dwg_AcDs_SchemaData *) 0 ;
    Dwg_AcDs_Schema *arg2 = (Dwg_AcDs_Schema *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaData_schemas_set(self,schemas);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaData_schemas_set" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaData *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaData *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_AcDs_Schema, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AcDs_SchemaData_schemas_set" "', argument " "2"" of type '" "Dwg_AcDs_Schema *""'"); 
    }
    arg2 = (Dwg_AcDs_Schema *)(argp2);
    if (arg1) (arg1)->schemas = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaData_schemas_get) {
  {
    struct _dwg_AcDs_SchemaData *arg1 = (struct _dwg_AcDs_SchemaData *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_AcDs_Schema *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaData_schemas_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaData_schemas_get" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaData *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaData *)(argp1);
    result = (Dwg_AcDs_Schema *) ((arg1)->schemas);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_Schema, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaData_num_propnames_set) {
  {
    struct _dwg_AcDs_SchemaData *arg1 = (struct _dwg_AcDs_SchemaData *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaData_num_propnames_set(self,num_propnames);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaData_num_propnames_set" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaData *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaData *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_SchemaData_num_propnames_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->num_propnames = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaData_num_propnames_get) {
  {
    struct _dwg_AcDs_SchemaData *arg1 = (struct _dwg_AcDs_SchemaData *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaData_num_propnames_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaData_num_propnames_get" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaData *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaData *)(argp1);
    result = (BITCODE_RL) ((arg1)->num_propnames);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaData_propnames_set) {
  {
    struct _dwg_AcDs_SchemaData *arg1 = (struct _dwg_AcDs_SchemaData *) 0 ;
    BITCODE_TV *arg2 = (BITCODE_TV *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaData_propnames_set(self,propnames);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaData_propnames_set" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaData *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaData *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AcDs_SchemaData_propnames_set" "', argument " "2"" of type '" "BITCODE_TV *""'"); 
    }
    arg2 = (BITCODE_TV *)(argp2);
    if (arg1) (arg1)->propnames = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_SchemaData_propnames_get) {
  {
    struct _dwg_AcDs_SchemaData *arg1 = (struct _dwg_AcDs_SchemaData *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_SchemaData_propnames_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaData, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_SchemaData_propnames_get" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaData *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaData *)(argp1);
    result = (BITCODE_TV *) ((arg1)->propnames);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_AcDs_SchemaData) {
  {
    int argvi = 0;
    struct _dwg_AcDs_SchemaData *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_AcDs_SchemaData();");
    }
    result = (struct _dwg_AcDs_SchemaData *)calloc(1, sizeof(struct _dwg_AcDs_SchemaData));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_SchemaData, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_AcDs_SchemaData) {
  {
    struct _dwg_AcDs_SchemaData *arg1 = (struct _dwg_AcDs_SchemaData *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_AcDs_SchemaData(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_SchemaData, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_AcDs_SchemaData" "', argument " "1"" of type '" "struct _dwg_AcDs_SchemaData *""'"); 
    }
    arg1 = (struct _dwg_AcDs_SchemaData *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Search_IdIdx_handle_set) {
  {
    struct _dwg_AcDs_Search_IdIdx *arg1 = (struct _dwg_AcDs_Search_IdIdx *) 0 ;
    BITCODE_RLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Search_IdIdx_handle_set(self,handle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Search_IdIdx, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Search_IdIdx_handle_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Search_IdIdx *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Search_IdIdx *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_Search_IdIdx_handle_set" "', argument " "2"" of type '" "BITCODE_RLL""'");
    } 
    arg2 = (BITCODE_RLL)(val2);
    if (arg1) (arg1)->handle = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Search_IdIdx_handle_get) {
  {
    struct _dwg_AcDs_Search_IdIdx *arg1 = (struct _dwg_AcDs_Search_IdIdx *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Search_IdIdx_handle_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Search_IdIdx, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Search_IdIdx_handle_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Search_IdIdx *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Search_IdIdx *)(argp1);
    result = (BITCODE_RLL) ((arg1)->handle);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Search_IdIdx_num_ididx_set) {
  {
    struct _dwg_AcDs_Search_IdIdx *arg1 = (struct _dwg_AcDs_Search_IdIdx *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Search_IdIdx_num_ididx_set(self,num_ididx);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Search_IdIdx, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Search_IdIdx_num_ididx_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Search_IdIdx *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Search_IdIdx *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_Search_IdIdx_num_ididx_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->num_ididx = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Search_IdIdx_num_ididx_get) {
  {
    struct _dwg_AcDs_Search_IdIdx *arg1 = (struct _dwg_AcDs_Search_IdIdx *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Search_IdIdx_num_ididx_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Search_IdIdx, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Search_IdIdx_num_ididx_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Search_IdIdx *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Search_IdIdx *)(argp1);
    result = (BITCODE_RL) ((arg1)->num_ididx);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Search_IdIdx_ididx_set) {
  {
    struct _dwg_AcDs_Search_IdIdx *arg1 = (struct _dwg_AcDs_Search_IdIdx *) 0 ;
    BITCODE_RLL *arg2 = (BITCODE_RLL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Search_IdIdx_ididx_set(self,ididx);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Search_IdIdx, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Search_IdIdx_ididx_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Search_IdIdx *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Search_IdIdx *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_long_long, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AcDs_Search_IdIdx_ididx_set" "', argument " "2"" of type '" "BITCODE_RLL *""'"); 
    }
    arg2 = (BITCODE_RLL *)(argp2);
    if (arg1) (arg1)->ididx = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Search_IdIdx_ididx_get) {
  {
    struct _dwg_AcDs_Search_IdIdx *arg1 = (struct _dwg_AcDs_Search_IdIdx *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Search_IdIdx_ididx_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Search_IdIdx, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Search_IdIdx_ididx_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Search_IdIdx *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Search_IdIdx *)(argp1);
    result = (BITCODE_RLL *) ((arg1)->ididx);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_long_long, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_AcDs_Search_IdIdx) {
  {
    int argvi = 0;
    struct _dwg_AcDs_Search_IdIdx *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_AcDs_Search_IdIdx();");
    }
    result = (struct _dwg_AcDs_Search_IdIdx *)calloc(1, sizeof(struct _dwg_AcDs_Search_IdIdx));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_Search_IdIdx, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_AcDs_Search_IdIdx) {
  {
    struct _dwg_AcDs_Search_IdIdx *arg1 = (struct _dwg_AcDs_Search_IdIdx *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_AcDs_Search_IdIdx(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Search_IdIdx, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_AcDs_Search_IdIdx" "', argument " "1"" of type '" "struct _dwg_AcDs_Search_IdIdx *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Search_IdIdx *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Search_IdIdxs_num_ididx_set) {
  {
    struct _dwg_AcDs_Search_IdIdxs *arg1 = (struct _dwg_AcDs_Search_IdIdxs *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Search_IdIdxs_num_ididx_set(self,num_ididx);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Search_IdIdxs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Search_IdIdxs_num_ididx_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Search_IdIdxs *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Search_IdIdxs *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_Search_IdIdxs_num_ididx_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->num_ididx = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Search_IdIdxs_num_ididx_get) {
  {
    struct _dwg_AcDs_Search_IdIdxs *arg1 = (struct _dwg_AcDs_Search_IdIdxs *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Search_IdIdxs_num_ididx_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Search_IdIdxs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Search_IdIdxs_num_ididx_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Search_IdIdxs *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Search_IdIdxs *)(argp1);
    result = (BITCODE_RL) ((arg1)->num_ididx);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Search_IdIdxs_ididx_set) {
  {
    struct _dwg_AcDs_Search_IdIdxs *arg1 = (struct _dwg_AcDs_Search_IdIdxs *) 0 ;
    Dwg_AcDs_Search_IdIdx *arg2 = (Dwg_AcDs_Search_IdIdx *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Search_IdIdxs_ididx_set(self,ididx);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Search_IdIdxs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Search_IdIdxs_ididx_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Search_IdIdxs *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Search_IdIdxs *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_AcDs_Search_IdIdx, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AcDs_Search_IdIdxs_ididx_set" "', argument " "2"" of type '" "Dwg_AcDs_Search_IdIdx *""'"); 
    }
    arg2 = (Dwg_AcDs_Search_IdIdx *)(argp2);
    if (arg1) (arg1)->ididx = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Search_IdIdxs_ididx_get) {
  {
    struct _dwg_AcDs_Search_IdIdxs *arg1 = (struct _dwg_AcDs_Search_IdIdxs *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_AcDs_Search_IdIdx *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Search_IdIdxs_ididx_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Search_IdIdxs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Search_IdIdxs_ididx_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Search_IdIdxs *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Search_IdIdxs *)(argp1);
    result = (Dwg_AcDs_Search_IdIdx *) ((arg1)->ididx);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_Search_IdIdx, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_AcDs_Search_IdIdxs) {
  {
    int argvi = 0;
    struct _dwg_AcDs_Search_IdIdxs *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_AcDs_Search_IdIdxs();");
    }
    result = (struct _dwg_AcDs_Search_IdIdxs *)calloc(1, sizeof(struct _dwg_AcDs_Search_IdIdxs));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_Search_IdIdxs, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_AcDs_Search_IdIdxs) {
  {
    struct _dwg_AcDs_Search_IdIdxs *arg1 = (struct _dwg_AcDs_Search_IdIdxs *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_AcDs_Search_IdIdxs(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Search_IdIdxs, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_AcDs_Search_IdIdxs" "', argument " "1"" of type '" "struct _dwg_AcDs_Search_IdIdxs *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Search_IdIdxs *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Search_Data_schema_namidx_set) {
  {
    struct _dwg_AcDs_Search_Data *arg1 = (struct _dwg_AcDs_Search_Data *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Search_Data_schema_namidx_set(self,schema_namidx);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Search_Data, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Search_Data_schema_namidx_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Search_Data *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Search_Data *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_Search_Data_schema_namidx_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->schema_namidx = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Search_Data_schema_namidx_get) {
  {
    struct _dwg_AcDs_Search_Data *arg1 = (struct _dwg_AcDs_Search_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Search_Data_schema_namidx_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Search_Data, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Search_Data_schema_namidx_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Search_Data *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Search_Data *)(argp1);
    result = (BITCODE_RL) ((arg1)->schema_namidx);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Search_Data_num_sortedidx_set) {
  {
    struct _dwg_AcDs_Search_Data *arg1 = (struct _dwg_AcDs_Search_Data *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Search_Data_num_sortedidx_set(self,num_sortedidx);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Search_Data, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Search_Data_num_sortedidx_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Search_Data *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Search_Data *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_Search_Data_num_sortedidx_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->num_sortedidx = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Search_Data_num_sortedidx_get) {
  {
    struct _dwg_AcDs_Search_Data *arg1 = (struct _dwg_AcDs_Search_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Search_Data_num_sortedidx_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Search_Data, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Search_Data_num_sortedidx_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Search_Data *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Search_Data *)(argp1);
    result = (BITCODE_RL) ((arg1)->num_sortedidx);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Search_Data_sortedidx_set) {
  {
    struct _dwg_AcDs_Search_Data *arg1 = (struct _dwg_AcDs_Search_Data *) 0 ;
    BITCODE_RLLd *arg2 = (BITCODE_RLLd *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Search_Data_sortedidx_set(self,sortedidx);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Search_Data, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Search_Data_sortedidx_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Search_Data *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Search_Data *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_long_long, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AcDs_Search_Data_sortedidx_set" "', argument " "2"" of type '" "BITCODE_RLLd *""'"); 
    }
    arg2 = (BITCODE_RLLd *)(argp2);
    if (arg1) (arg1)->sortedidx = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Search_Data_sortedidx_get) {
  {
    struct _dwg_AcDs_Search_Data *arg1 = (struct _dwg_AcDs_Search_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLLd *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Search_Data_sortedidx_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Search_Data, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Search_Data_sortedidx_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Search_Data *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Search_Data *)(argp1);
    result = (BITCODE_RLLd *) ((arg1)->sortedidx);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_long_long, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Search_Data_num_ididxs_set) {
  {
    struct _dwg_AcDs_Search_Data *arg1 = (struct _dwg_AcDs_Search_Data *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Search_Data_num_ididxs_set(self,num_ididxs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Search_Data, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Search_Data_num_ididxs_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Search_Data *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Search_Data *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_Search_Data_num_ididxs_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->num_ididxs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Search_Data_num_ididxs_get) {
  {
    struct _dwg_AcDs_Search_Data *arg1 = (struct _dwg_AcDs_Search_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Search_Data_num_ididxs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Search_Data, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Search_Data_num_ididxs_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Search_Data *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Search_Data *)(argp1);
    result = (BITCODE_RL) ((arg1)->num_ididxs);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Search_Data_unknown_set) {
  {
    struct _dwg_AcDs_Search_Data *arg1 = (struct _dwg_AcDs_Search_Data *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Search_Data_unknown_set(self,unknown);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Search_Data, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Search_Data_unknown_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Search_Data *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Search_Data *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_Search_Data_unknown_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->unknown = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Search_Data_unknown_get) {
  {
    struct _dwg_AcDs_Search_Data *arg1 = (struct _dwg_AcDs_Search_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Search_Data_unknown_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Search_Data, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Search_Data_unknown_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Search_Data *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Search_Data *)(argp1);
    result = (BITCODE_RL) ((arg1)->unknown);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Search_Data_ididxs_set) {
  {
    struct _dwg_AcDs_Search_Data *arg1 = (struct _dwg_AcDs_Search_Data *) 0 ;
    Dwg_AcDs_Search_IdIdxs *arg2 = (Dwg_AcDs_Search_IdIdxs *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Search_Data_ididxs_set(self,ididxs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Search_Data, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Search_Data_ididxs_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Search_Data *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Search_Data *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_AcDs_Search_IdIdxs, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AcDs_Search_Data_ididxs_set" "', argument " "2"" of type '" "Dwg_AcDs_Search_IdIdxs *""'"); 
    }
    arg2 = (Dwg_AcDs_Search_IdIdxs *)(argp2);
    if (arg1) (arg1)->ididxs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Search_Data_ididxs_get) {
  {
    struct _dwg_AcDs_Search_Data *arg1 = (struct _dwg_AcDs_Search_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_AcDs_Search_IdIdxs *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Search_Data_ididxs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Search_Data, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Search_Data_ididxs_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Search_Data *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Search_Data *)(argp1);
    result = (Dwg_AcDs_Search_IdIdxs *) ((arg1)->ididxs);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_Search_IdIdxs, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_AcDs_Search_Data) {
  {
    int argvi = 0;
    struct _dwg_AcDs_Search_Data *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_AcDs_Search_Data();");
    }
    result = (struct _dwg_AcDs_Search_Data *)calloc(1, sizeof(struct _dwg_AcDs_Search_Data));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_Search_Data, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_AcDs_Search_Data) {
  {
    struct _dwg_AcDs_Search_Data *arg1 = (struct _dwg_AcDs_Search_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_AcDs_Search_Data(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Search_Data, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_AcDs_Search_Data" "', argument " "1"" of type '" "struct _dwg_AcDs_Search_Data *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Search_Data *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Search_num_search_set) {
  {
    struct _dwg_AcDs_Search *arg1 = (struct _dwg_AcDs_Search *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Search_num_search_set(self,num_search);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Search, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Search_num_search_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Search *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Search *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_Search_num_search_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->num_search = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Search_num_search_get) {
  {
    struct _dwg_AcDs_Search *arg1 = (struct _dwg_AcDs_Search *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Search_num_search_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Search, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Search_num_search_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Search *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Search *)(argp1);
    result = (BITCODE_RL) ((arg1)->num_search);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Search_search_set) {
  {
    struct _dwg_AcDs_Search *arg1 = (struct _dwg_AcDs_Search *) 0 ;
    Dwg_AcDs_Search_Data *arg2 = (Dwg_AcDs_Search_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Search_search_set(self,search);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Search, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Search_search_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Search *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Search *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_AcDs_Search_Data, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AcDs_Search_search_set" "', argument " "2"" of type '" "Dwg_AcDs_Search_Data *""'"); 
    }
    arg2 = (Dwg_AcDs_Search_Data *)(argp2);
    if (arg1) (arg1)->search = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Search_search_get) {
  {
    struct _dwg_AcDs_Search *arg1 = (struct _dwg_AcDs_Search *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_AcDs_Search_Data *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Search_search_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Search, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Search_search_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Search *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Search *)(argp1);
    result = (Dwg_AcDs_Search_Data *) ((arg1)->search);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_Search_Data, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_AcDs_Search) {
  {
    int argvi = 0;
    struct _dwg_AcDs_Search *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_AcDs_Search();");
    }
    result = (struct _dwg_AcDs_Search *)calloc(1, sizeof(struct _dwg_AcDs_Search));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_Search, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_AcDs_Search) {
  {
    struct _dwg_AcDs_Search *arg1 = (struct _dwg_AcDs_Search *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_AcDs_Search(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Search, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_AcDs_Search" "', argument " "1"" of type '" "struct _dwg_AcDs_Search *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Search *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Segment_signature_set) {
  {
    struct _dwg_AcDs_Segment *arg1 = (struct _dwg_AcDs_Segment *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Segment_signature_set(self,signature);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Segment, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Segment_signature_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Segment *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Segment *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_Segment_signature_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->signature = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Segment_signature_get) {
  {
    struct _dwg_AcDs_Segment *arg1 = (struct _dwg_AcDs_Segment *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Segment_signature_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Segment, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Segment_signature_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Segment *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Segment *)(argp1);
    result = (BITCODE_RS) ((arg1)->signature);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Segment_name_set) {
  {
    struct _dwg_AcDs_Segment *arg1 = (struct _dwg_AcDs_Segment *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) (BITCODE_RC *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Segment_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Segment, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Segment_name_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Segment *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Segment *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AcDs_Segment_name_set" "', argument " "2"" of type '" "BITCODE_RC [7]""'"); 
    } 
    arg2 = (BITCODE_RC *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)7; ++ii) *(BITCODE_RC *)&arg1->name[ii] = *((BITCODE_RC *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""name""' of type '""BITCODE_RC [7]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Segment_name_get) {
  {
    struct _dwg_AcDs_Segment *arg1 = (struct _dwg_AcDs_Segment *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Segment_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Segment, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Segment_name_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Segment *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Segment *)(argp1);
    result = (BITCODE_RC *)(BITCODE_RC *) ((arg1)->name);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Segment_type_set) {
  {
    struct _dwg_AcDs_Segment *arg1 = (struct _dwg_AcDs_Segment *) 0 ;
    BITCODE_RCd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    signed char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Segment_type_set(self,type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Segment, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Segment_type_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Segment *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Segment *)(argp1);
    ecode2 = SWIG_AsVal_signed_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_Segment_type_set" "', argument " "2"" of type '" "BITCODE_RCd""'");
    } 
    arg2 = (BITCODE_RCd)(val2);
    if (arg1) (arg1)->type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Segment_type_get) {
  {
    struct _dwg_AcDs_Segment *arg1 = (struct _dwg_AcDs_Segment *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RCd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Segment_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Segment, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Segment_type_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Segment *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Segment *)(argp1);
    result = (BITCODE_RCd) ((arg1)->type);
    ST(argvi) = SWIG_From_signed_SS_char  SWIG_PERL_CALL_ARGS_1((signed char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Segment_segment_idx_set) {
  {
    struct _dwg_AcDs_Segment *arg1 = (struct _dwg_AcDs_Segment *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Segment_segment_idx_set(self,segment_idx);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Segment, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Segment_segment_idx_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Segment *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Segment *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_Segment_segment_idx_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->segment_idx = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Segment_segment_idx_get) {
  {
    struct _dwg_AcDs_Segment *arg1 = (struct _dwg_AcDs_Segment *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Segment_segment_idx_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Segment, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Segment_segment_idx_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Segment *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Segment *)(argp1);
    result = (BITCODE_RL) ((arg1)->segment_idx);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Segment_is_blob01_set) {
  {
    struct _dwg_AcDs_Segment *arg1 = (struct _dwg_AcDs_Segment *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Segment_is_blob01_set(self,is_blob01);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Segment, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Segment_is_blob01_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Segment *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Segment *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_Segment_is_blob01_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->is_blob01 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Segment_is_blob01_get) {
  {
    struct _dwg_AcDs_Segment *arg1 = (struct _dwg_AcDs_Segment *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Segment_is_blob01_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Segment, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Segment_is_blob01_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Segment *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Segment *)(argp1);
    result = (BITCODE_RL) ((arg1)->is_blob01);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Segment_segsize_set) {
  {
    struct _dwg_AcDs_Segment *arg1 = (struct _dwg_AcDs_Segment *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Segment_segsize_set(self,segsize);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Segment, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Segment_segsize_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Segment *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Segment *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_Segment_segsize_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->segsize = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Segment_segsize_get) {
  {
    struct _dwg_AcDs_Segment *arg1 = (struct _dwg_AcDs_Segment *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Segment_segsize_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Segment, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Segment_segsize_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Segment *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Segment *)(argp1);
    result = (BITCODE_RL) ((arg1)->segsize);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Segment_unknown_2_set) {
  {
    struct _dwg_AcDs_Segment *arg1 = (struct _dwg_AcDs_Segment *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Segment_unknown_2_set(self,unknown_2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Segment, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Segment_unknown_2_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Segment *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Segment *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_Segment_unknown_2_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->unknown_2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Segment_unknown_2_get) {
  {
    struct _dwg_AcDs_Segment *arg1 = (struct _dwg_AcDs_Segment *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Segment_unknown_2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Segment, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Segment_unknown_2_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Segment *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Segment *)(argp1);
    result = (BITCODE_RL) ((arg1)->unknown_2);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Segment_ds_version_set) {
  {
    struct _dwg_AcDs_Segment *arg1 = (struct _dwg_AcDs_Segment *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Segment_ds_version_set(self,ds_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Segment, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Segment_ds_version_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Segment *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Segment *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_Segment_ds_version_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->ds_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Segment_ds_version_get) {
  {
    struct _dwg_AcDs_Segment *arg1 = (struct _dwg_AcDs_Segment *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Segment_ds_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Segment, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Segment_ds_version_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Segment *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Segment *)(argp1);
    result = (BITCODE_RL) ((arg1)->ds_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Segment_unknown_3_set) {
  {
    struct _dwg_AcDs_Segment *arg1 = (struct _dwg_AcDs_Segment *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Segment_unknown_3_set(self,unknown_3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Segment, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Segment_unknown_3_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Segment *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Segment *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_Segment_unknown_3_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->unknown_3 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Segment_unknown_3_get) {
  {
    struct _dwg_AcDs_Segment *arg1 = (struct _dwg_AcDs_Segment *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Segment_unknown_3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Segment, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Segment_unknown_3_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Segment *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Segment *)(argp1);
    result = (BITCODE_RL) ((arg1)->unknown_3);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Segment_data_algn_offset_set) {
  {
    struct _dwg_AcDs_Segment *arg1 = (struct _dwg_AcDs_Segment *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Segment_data_algn_offset_set(self,data_algn_offset);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Segment, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Segment_data_algn_offset_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Segment *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Segment *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_Segment_data_algn_offset_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->data_algn_offset = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Segment_data_algn_offset_get) {
  {
    struct _dwg_AcDs_Segment *arg1 = (struct _dwg_AcDs_Segment *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Segment_data_algn_offset_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Segment, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Segment_data_algn_offset_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Segment *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Segment *)(argp1);
    result = (BITCODE_RL) ((arg1)->data_algn_offset);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Segment_objdata_algn_offset_set) {
  {
    struct _dwg_AcDs_Segment *arg1 = (struct _dwg_AcDs_Segment *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Segment_objdata_algn_offset_set(self,objdata_algn_offset);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Segment, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Segment_objdata_algn_offset_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Segment *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Segment *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_Segment_objdata_algn_offset_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->objdata_algn_offset = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Segment_objdata_algn_offset_get) {
  {
    struct _dwg_AcDs_Segment *arg1 = (struct _dwg_AcDs_Segment *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Segment_objdata_algn_offset_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Segment, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Segment_objdata_algn_offset_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Segment *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Segment *)(argp1);
    result = (BITCODE_RL) ((arg1)->objdata_algn_offset);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Segment_padding_set) {
  {
    struct _dwg_AcDs_Segment *arg1 = (struct _dwg_AcDs_Segment *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) (BITCODE_RC *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_Segment_padding_set(self,padding);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Segment, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Segment_padding_set" "', argument " "1"" of type '" "struct _dwg_AcDs_Segment *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Segment *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AcDs_Segment_padding_set" "', argument " "2"" of type '" "BITCODE_RC [9]""'"); 
    } 
    arg2 = (BITCODE_RC *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)9; ++ii) *(BITCODE_RC *)&arg1->padding[ii] = *((BITCODE_RC *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""padding""' of type '""BITCODE_RC [9]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_Segment_padding_get) {
  {
    struct _dwg_AcDs_Segment *arg1 = (struct _dwg_AcDs_Segment *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_Segment_padding_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Segment, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_Segment_padding_get" "', argument " "1"" of type '" "struct _dwg_AcDs_Segment *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Segment *)(argp1);
    result = (BITCODE_RC *)(BITCODE_RC *) ((arg1)->padding);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_AcDs_Segment) {
  {
    int argvi = 0;
    struct _dwg_AcDs_Segment *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_AcDs_Segment();");
    }
    result = (struct _dwg_AcDs_Segment *)calloc(1, sizeof(struct _dwg_AcDs_Segment));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_Segment, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_AcDs_Segment) {
  {
    struct _dwg_AcDs_Segment *arg1 = (struct _dwg_AcDs_Segment *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_AcDs_Segment(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs_Segment, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_AcDs_Segment" "', argument " "1"" of type '" "struct _dwg_AcDs_Segment *""'"); 
    }
    arg1 = (struct _dwg_AcDs_Segment *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_file_signature_set) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_file_signature_set(self,file_signature);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_file_signature_set" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_file_signature_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->file_signature = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_file_signature_get) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_file_signature_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_file_signature_get" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    result = (BITCODE_RL) ((arg1)->file_signature);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_file_header_size_set) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_file_header_size_set(self,file_header_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_file_header_size_set" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_file_header_size_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->file_header_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_file_header_size_get) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_file_header_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_file_header_size_get" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    result = (BITCODE_RL) ((arg1)->file_header_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_unknown_1_set) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_unknown_1_set(self,unknown_1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_unknown_1_set" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_unknown_1_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->unknown_1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_unknown_1_get) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_unknown_1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_unknown_1_get" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    result = (BITCODE_RL) ((arg1)->unknown_1);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_version_set) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_version_set(self,version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_version_set" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_version_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_version_get) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_version_get" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    result = (BITCODE_RL) ((arg1)->version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_unknown_2_set) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_unknown_2_set(self,unknown_2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_unknown_2_set" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_unknown_2_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->unknown_2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_unknown_2_get) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_unknown_2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_unknown_2_get" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    result = (BITCODE_RL) ((arg1)->unknown_2);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_ds_version_set) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_ds_version_set(self,ds_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_ds_version_set" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_ds_version_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->ds_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_ds_version_get) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_ds_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_ds_version_get" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    result = (BITCODE_RL) ((arg1)->ds_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_segidx_offset_set) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_segidx_offset_set(self,segidx_offset);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_segidx_offset_set" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_segidx_offset_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->segidx_offset = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_segidx_offset_get) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_segidx_offset_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_segidx_offset_get" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    result = (BITCODE_RL) ((arg1)->segidx_offset);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_segidx_unknown_set) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_segidx_unknown_set(self,segidx_unknown);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_segidx_unknown_set" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_segidx_unknown_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->segidx_unknown = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_segidx_unknown_get) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_segidx_unknown_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_segidx_unknown_get" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    result = (BITCODE_RL) ((arg1)->segidx_unknown);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_num_segidx_set) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_num_segidx_set(self,num_segidx);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_num_segidx_set" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_num_segidx_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->num_segidx = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_num_segidx_get) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_num_segidx_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_num_segidx_get" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    result = (BITCODE_RL) ((arg1)->num_segidx);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_schidx_segidx_set) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_schidx_segidx_set(self,schidx_segidx);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_schidx_segidx_set" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_schidx_segidx_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->schidx_segidx = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_schidx_segidx_get) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_schidx_segidx_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_schidx_segidx_get" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    result = (BITCODE_RL) ((arg1)->schidx_segidx);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_datidx_segidx_set) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_datidx_segidx_set(self,datidx_segidx);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_datidx_segidx_set" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_datidx_segidx_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->datidx_segidx = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_datidx_segidx_get) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_datidx_segidx_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_datidx_segidx_get" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    result = (BITCODE_RL) ((arg1)->datidx_segidx);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_search_segidx_set) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_search_segidx_set(self,search_segidx);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_search_segidx_set" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_search_segidx_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->search_segidx = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_search_segidx_get) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_search_segidx_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_search_segidx_get" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    result = (BITCODE_RL) ((arg1)->search_segidx);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_prvsav_segidx_set) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_prvsav_segidx_set(self,prvsav_segidx);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_prvsav_segidx_set" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_prvsav_segidx_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->prvsav_segidx = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_prvsav_segidx_get) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_prvsav_segidx_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_prvsav_segidx_get" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    result = (BITCODE_RL) ((arg1)->prvsav_segidx);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_file_size_set) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_file_size_set(self,file_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_file_size_set" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_file_size_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->file_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_file_size_get) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_file_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_file_size_get" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    result = (BITCODE_RL) ((arg1)->file_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_total_segments_set) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_total_segments_set(self,total_segments);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_total_segments_set" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AcDs_total_segments_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->total_segments = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_total_segments_get) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_total_segments_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_total_segments_get" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    result = (BITCODE_BL) ((arg1)->total_segments);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_segidx_set) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    Dwg_AcDs_SegmentIndex *arg2 = (Dwg_AcDs_SegmentIndex *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_segidx_set(self,segidx);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_segidx_set" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_AcDs_SegmentIndex, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AcDs_segidx_set" "', argument " "2"" of type '" "Dwg_AcDs_SegmentIndex *""'"); 
    }
    arg2 = (Dwg_AcDs_SegmentIndex *)(argp2);
    if (arg1) (arg1)->segidx = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_segidx_get) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_AcDs_SegmentIndex *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_segidx_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_segidx_get" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    result = (Dwg_AcDs_SegmentIndex *) ((arg1)->segidx);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_SegmentIndex, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_datidx_set) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    Dwg_AcDs_DataIndex *arg2 = (Dwg_AcDs_DataIndex *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_datidx_set(self,datidx);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_datidx_set" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_AcDs_DataIndex, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AcDs_datidx_set" "', argument " "2"" of type '" "Dwg_AcDs_DataIndex *""'"); 
    }
    arg2 = (Dwg_AcDs_DataIndex *)(argp2);
    if (arg1) (arg1)->datidx = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_datidx_get) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_AcDs_DataIndex *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_datidx_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_datidx_get" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    result = (Dwg_AcDs_DataIndex *)& ((arg1)->datidx);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_DataIndex, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_data_set) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    Dwg_AcDs_Data *arg2 = (Dwg_AcDs_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_data_set(self,data);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_data_set" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_AcDs_Data, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AcDs_data_set" "', argument " "2"" of type '" "Dwg_AcDs_Data *""'"); 
    }
    arg2 = (Dwg_AcDs_Data *)(argp2);
    if (arg1) (arg1)->data = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_data_get) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_AcDs_Data *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_data_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_data_get" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    result = (Dwg_AcDs_Data *) ((arg1)->data);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_Data, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_blob01_set) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    Dwg_AcDs_DataBlob *arg2 = (Dwg_AcDs_DataBlob *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_blob01_set(self,blob01);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_blob01_set" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_AcDs_DataBlob, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AcDs_blob01_set" "', argument " "2"" of type '" "Dwg_AcDs_DataBlob *""'"); 
    }
    arg2 = (Dwg_AcDs_DataBlob *)(argp2);
    if (arg1) (arg1)->blob01 = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_blob01_get) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_AcDs_DataBlob *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_blob01_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_blob01_get" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    result = (Dwg_AcDs_DataBlob *)& ((arg1)->blob01);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_DataBlob, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_schidx_set) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    Dwg_AcDs_SchemaIndex *arg2 = (Dwg_AcDs_SchemaIndex *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_schidx_set(self,schidx);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_schidx_set" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_AcDs_SchemaIndex, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AcDs_schidx_set" "', argument " "2"" of type '" "Dwg_AcDs_SchemaIndex *""'"); 
    }
    arg2 = (Dwg_AcDs_SchemaIndex *)(argp2);
    if (arg1) (arg1)->schidx = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_schidx_get) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_AcDs_SchemaIndex *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_schidx_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_schidx_get" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    result = (Dwg_AcDs_SchemaIndex *)& ((arg1)->schidx);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_SchemaIndex, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_schdat_set) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    Dwg_AcDs_SchemaData *arg2 = (Dwg_AcDs_SchemaData *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_schdat_set(self,schdat);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_schdat_set" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_AcDs_SchemaData, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AcDs_schdat_set" "', argument " "2"" of type '" "Dwg_AcDs_SchemaData *""'"); 
    }
    arg2 = (Dwg_AcDs_SchemaData *)(argp2);
    if (arg1) (arg1)->schdat = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_schdat_get) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_AcDs_SchemaData *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_schdat_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_schdat_get" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    result = (Dwg_AcDs_SchemaData *)& ((arg1)->schdat);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_SchemaData, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_search_set) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    Dwg_AcDs_Search *arg2 = (Dwg_AcDs_Search *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_search_set(self,search);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_search_set" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_AcDs_Search, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AcDs_search_set" "', argument " "2"" of type '" "Dwg_AcDs_Search *""'"); 
    }
    arg2 = (Dwg_AcDs_Search *)(argp2);
    if (arg1) (arg1)->search = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_search_get) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_AcDs_Search *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_search_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_search_get" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    result = (Dwg_AcDs_Search *)& ((arg1)->search);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_Search, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_segments_set) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    Dwg_AcDs_Segment *arg2 = (Dwg_AcDs_Segment *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AcDs_segments_set(self,segments);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_segments_set" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_AcDs_Segment, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AcDs_segments_set" "', argument " "2"" of type '" "Dwg_AcDs_Segment *""'"); 
    }
    arg2 = (Dwg_AcDs_Segment *)(argp2);
    if (arg1) (arg1)->segments = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AcDs_segments_get) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_AcDs_Segment *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AcDs_segments_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AcDs_segments_get" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    result = (Dwg_AcDs_Segment *) ((arg1)->segments);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs_Segment, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_AcDs) {
  {
    int argvi = 0;
    struct _dwg_AcDs *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_AcDs();");
    }
    result = (struct _dwg_AcDs *)calloc(1, sizeof(struct _dwg_AcDs));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_AcDs) {
  {
    struct _dwg_AcDs *arg1 = (struct _dwg_AcDs *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_AcDs(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_AcDs, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_AcDs" "', argument " "1"" of type '" "struct _dwg_AcDs *""'"); 
    }
    arg1 = (struct _dwg_AcDs *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_version_set) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    Dwg_Version_Type arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_version_set(self,version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_version_set" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_version_set" "', argument " "2"" of type '" "Dwg_Version_Type""'");
    } 
    arg2 = (Dwg_Version_Type)(val2);
    if (arg1) (arg1)->version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_version_get) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Version_Type result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_version_get" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    result = (Dwg_Version_Type) ((arg1)->version);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_from_version_set) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    Dwg_Version_Type arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_from_version_set(self,from_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_from_version_set" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_from_version_set" "', argument " "2"" of type '" "Dwg_Version_Type""'");
    } 
    arg2 = (Dwg_Version_Type)(val2);
    if (arg1) (arg1)->from_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_from_version_get) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Version_Type result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_from_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_from_version_get" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    result = (Dwg_Version_Type) ((arg1)->from_version);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_is_maint_set) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_is_maint_set(self,is_maint);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_is_maint_set" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_is_maint_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->is_maint = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_is_maint_get) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_is_maint_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_is_maint_get" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    result = (BITCODE_RC) ((arg1)->is_maint);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_zero_one_or_three_set) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_zero_one_or_three_set(self,zero_one_or_three);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_zero_one_or_three_set" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_zero_one_or_three_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->zero_one_or_three = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_zero_one_or_three_get) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_zero_one_or_three_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_zero_one_or_three_get" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    result = (BITCODE_RC) ((arg1)->zero_one_or_three);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_numentity_sections_set) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_numentity_sections_set(self,numentity_sections);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_numentity_sections_set" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_numentity_sections_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->numentity_sections = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_numentity_sections_get) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_numentity_sections_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_numentity_sections_get" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    result = (BITCODE_RS) ((arg1)->numentity_sections);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_numheader_vars_set) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_numheader_vars_set(self,numheader_vars);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_numheader_vars_set" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_numheader_vars_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->numheader_vars = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_numheader_vars_get) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_numheader_vars_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_numheader_vars_get" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    result = (BITCODE_RS) ((arg1)->numheader_vars);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_thumbnail_address_set) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_thumbnail_address_set(self,thumbnail_address);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_thumbnail_address_set" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_thumbnail_address_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->thumbnail_address = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_thumbnail_address_get) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_thumbnail_address_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_thumbnail_address_get" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    result = (BITCODE_RL) ((arg1)->thumbnail_address);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_dwg_version_set) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_dwg_version_set(self,dwg_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_dwg_version_set" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_dwg_version_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->dwg_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_dwg_version_get) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_dwg_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_dwg_version_get" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    result = (BITCODE_RC) ((arg1)->dwg_version);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_maint_version_set) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_maint_version_set(self,maint_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_maint_version_set" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_maint_version_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->maint_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_maint_version_get) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_maint_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_maint_version_get" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    result = (BITCODE_RC) ((arg1)->maint_version);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_entities_start_set) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_entities_start_set(self,entities_start);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_entities_start_set" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_entities_start_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->entities_start = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_entities_start_get) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_entities_start_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_entities_start_get" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    result = (BITCODE_RL) ((arg1)->entities_start);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_entities_end_set) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_entities_end_set(self,entities_end);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_entities_end_set" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_entities_end_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->entities_end = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_entities_end_get) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_entities_end_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_entities_end_get" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    result = (BITCODE_RL) ((arg1)->entities_end);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_blocks_start_set) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_blocks_start_set(self,blocks_start);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_blocks_start_set" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_blocks_start_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->blocks_start = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_blocks_start_get) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_blocks_start_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_blocks_start_get" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    result = (BITCODE_RL) ((arg1)->blocks_start);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_blocks_size_set) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_blocks_size_set(self,blocks_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_blocks_size_set" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_blocks_size_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->blocks_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_blocks_size_get) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_blocks_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_blocks_size_get" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    result = (BITCODE_RL) ((arg1)->blocks_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_extras_start_set) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_extras_start_set(self,extras_start);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_extras_start_set" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_extras_start_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->extras_start = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_extras_start_get) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_extras_start_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_extras_start_get" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    result = (BITCODE_RL) ((arg1)->extras_start);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_extras_size_set) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_extras_size_set(self,extras_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_extras_size_set" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_extras_size_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->extras_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_extras_size_get) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_extras_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_extras_size_get" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    result = (BITCODE_RL) ((arg1)->extras_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_codepage_set) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_codepage_set(self,codepage);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_codepage_set" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_codepage_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->codepage = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_codepage_get) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_codepage_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_codepage_get" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    result = (BITCODE_RS) ((arg1)->codepage);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_unknown_0_set) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_unknown_0_set(self,unknown_0);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_unknown_0_set" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_unknown_0_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->unknown_0 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_unknown_0_get) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_unknown_0_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_unknown_0_get" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    result = (BITCODE_RC) ((arg1)->unknown_0);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_app_dwg_version_set) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_app_dwg_version_set(self,app_dwg_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_app_dwg_version_set" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_app_dwg_version_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->app_dwg_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_app_dwg_version_get) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_app_dwg_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_app_dwg_version_get" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    result = (BITCODE_RC) ((arg1)->app_dwg_version);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_app_maint_version_set) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_app_maint_version_set(self,app_maint_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_app_maint_version_set" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_app_maint_version_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->app_maint_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_app_maint_version_get) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_app_maint_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_app_maint_version_get" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    result = (BITCODE_RC) ((arg1)->app_maint_version);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_security_type_set) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_security_type_set(self,security_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_security_type_set" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_security_type_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->security_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_security_type_get) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_security_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_security_type_get" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    result = (BITCODE_RL) ((arg1)->security_type);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_rl_1c_address_set) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_rl_1c_address_set(self,rl_1c_address);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_rl_1c_address_set" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_rl_1c_address_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->rl_1c_address = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_rl_1c_address_get) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_rl_1c_address_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_rl_1c_address_get" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    result = (BITCODE_RL) ((arg1)->rl_1c_address);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_summaryinfo_address_set) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_summaryinfo_address_set(self,summaryinfo_address);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_summaryinfo_address_set" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_summaryinfo_address_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->summaryinfo_address = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_summaryinfo_address_get) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_summaryinfo_address_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_summaryinfo_address_get" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    result = (BITCODE_RL) ((arg1)->summaryinfo_address);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_vbaproj_address_set) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_vbaproj_address_set(self,vbaproj_address);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_vbaproj_address_set" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_vbaproj_address_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->vbaproj_address = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_vbaproj_address_get) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_vbaproj_address_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_vbaproj_address_get" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    result = (BITCODE_RL) ((arg1)->vbaproj_address);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_r2004_header_address_set) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_r2004_header_address_set(self,r2004_header_address);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_r2004_header_address_set" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_r2004_header_address_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->r2004_header_address = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_r2004_header_address_get) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_r2004_header_address_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_r2004_header_address_get" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    result = (BITCODE_RL) ((arg1)->r2004_header_address);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_sections_set) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_sections_set(self,sections);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_sections_set" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_sections_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->sections = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_sections_get) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_sections_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_sections_get" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    result = (BITCODE_RL) ((arg1)->sections);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_num_sections_set) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_num_sections_set(self,num_sections);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_num_sections_set" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Header_num_sections_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->num_sections = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_num_sections_get) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_num_sections_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_num_sections_get" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    result = (BITCODE_RL) ((arg1)->num_sections);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_section_set) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    Dwg_Section *arg2 = (Dwg_Section *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_section_set(self,section);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_section_set" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_section, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_section_set" "', argument " "2"" of type '" "Dwg_Section *""'"); 
    }
    arg2 = (Dwg_Section *)(argp2);
    if (arg1) (arg1)->section = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_section_get) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Section *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_section_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_section_get" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    result = (Dwg_Section *) ((arg1)->section);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_section, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_section_infohdr_set) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    Dwg_Section_InfoHdr *arg2 = (Dwg_Section_InfoHdr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_section_infohdr_set(self,section_infohdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_section_infohdr_set" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_Dwg_Section_InfoHdr, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_section_infohdr_set" "', argument " "2"" of type '" "Dwg_Section_InfoHdr *""'"); 
    }
    arg2 = (Dwg_Section_InfoHdr *)(argp2);
    if (arg1) (arg1)->section_infohdr = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_section_infohdr_get) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Section_InfoHdr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_section_infohdr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_section_infohdr_get" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    result = (Dwg_Section_InfoHdr *)& ((arg1)->section_infohdr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_Dwg_Section_InfoHdr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_section_info_set) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    Dwg_Section_Info *arg2 = (Dwg_Section_Info *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Header_section_info_set(self,section_info);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_section_info_set" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_Dwg_Section_Info, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Header_section_info_set" "', argument " "2"" of type '" "Dwg_Section_Info *""'"); 
    }
    arg2 = (Dwg_Section_Info *)(argp2);
    if (arg1) (arg1)->section_info = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Header_section_info_get) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Section_Info *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Header_section_info_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Header_section_info_get" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    result = (Dwg_Section_Info *) ((arg1)->section_info);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_Dwg_Section_Info, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Header) {
  {
    int argvi = 0;
    struct _dwg_header *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Header();");
    }
    result = (struct _dwg_header *)calloc(1, sizeof(struct _dwg_header));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_header, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Header) {
  {
    struct _dwg_header *arg1 = (struct _dwg_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Header(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_header, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Header" "', argument " "1"" of type '" "struct _dwg_header *""'"); 
    }
    arg1 = (struct _dwg_header *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_header_size_set) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    int64_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2007_Header_header_size_set(self,header_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_header_size_set" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    ecode2 = SWIG_AsVal_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2007_Header_header_size_set" "', argument " "2"" of type '" "int64_t""'");
    } 
    arg2 = (int64_t)(val2);
    if (arg1) (arg1)->header_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_header_size_get) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int64_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2007_Header_header_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_header_size_get" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    result = (int64_t) ((arg1)->header_size);
    ST(argvi) = SWIG_From_long_SS_long  SWIG_PERL_CALL_ARGS_1((long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_file_size_set) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    int64_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2007_Header_file_size_set(self,file_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_file_size_set" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    ecode2 = SWIG_AsVal_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2007_Header_file_size_set" "', argument " "2"" of type '" "int64_t""'");
    } 
    arg2 = (int64_t)(val2);
    if (arg1) (arg1)->file_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_file_size_get) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int64_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2007_Header_file_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_file_size_get" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    result = (int64_t) ((arg1)->file_size);
    ST(argvi) = SWIG_From_long_SS_long  SWIG_PERL_CALL_ARGS_1((long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_pages_map_crc_compressed_set) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    int64_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2007_Header_pages_map_crc_compressed_set(self,pages_map_crc_compressed);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_pages_map_crc_compressed_set" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    ecode2 = SWIG_AsVal_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2007_Header_pages_map_crc_compressed_set" "', argument " "2"" of type '" "int64_t""'");
    } 
    arg2 = (int64_t)(val2);
    if (arg1) (arg1)->pages_map_crc_compressed = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_pages_map_crc_compressed_get) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int64_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2007_Header_pages_map_crc_compressed_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_pages_map_crc_compressed_get" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    result = (int64_t) ((arg1)->pages_map_crc_compressed);
    ST(argvi) = SWIG_From_long_SS_long  SWIG_PERL_CALL_ARGS_1((long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_pages_map_correction_set) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    int64_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2007_Header_pages_map_correction_set(self,pages_map_correction);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_pages_map_correction_set" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    ecode2 = SWIG_AsVal_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2007_Header_pages_map_correction_set" "', argument " "2"" of type '" "int64_t""'");
    } 
    arg2 = (int64_t)(val2);
    if (arg1) (arg1)->pages_map_correction = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_pages_map_correction_get) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int64_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2007_Header_pages_map_correction_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_pages_map_correction_get" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    result = (int64_t) ((arg1)->pages_map_correction);
    ST(argvi) = SWIG_From_long_SS_long  SWIG_PERL_CALL_ARGS_1((long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_pages_map_crc_seed_set) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    int64_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2007_Header_pages_map_crc_seed_set(self,pages_map_crc_seed);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_pages_map_crc_seed_set" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    ecode2 = SWIG_AsVal_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2007_Header_pages_map_crc_seed_set" "', argument " "2"" of type '" "int64_t""'");
    } 
    arg2 = (int64_t)(val2);
    if (arg1) (arg1)->pages_map_crc_seed = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_pages_map_crc_seed_get) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int64_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2007_Header_pages_map_crc_seed_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_pages_map_crc_seed_get" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    result = (int64_t) ((arg1)->pages_map_crc_seed);
    ST(argvi) = SWIG_From_long_SS_long  SWIG_PERL_CALL_ARGS_1((long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_pages_map2_offset_set) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    int64_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2007_Header_pages_map2_offset_set(self,pages_map2_offset);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_pages_map2_offset_set" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    ecode2 = SWIG_AsVal_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2007_Header_pages_map2_offset_set" "', argument " "2"" of type '" "int64_t""'");
    } 
    arg2 = (int64_t)(val2);
    if (arg1) (arg1)->pages_map2_offset = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_pages_map2_offset_get) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int64_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2007_Header_pages_map2_offset_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_pages_map2_offset_get" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    result = (int64_t) ((arg1)->pages_map2_offset);
    ST(argvi) = SWIG_From_long_SS_long  SWIG_PERL_CALL_ARGS_1((long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_pages_map2_id_set) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    int64_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2007_Header_pages_map2_id_set(self,pages_map2_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_pages_map2_id_set" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    ecode2 = SWIG_AsVal_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2007_Header_pages_map2_id_set" "', argument " "2"" of type '" "int64_t""'");
    } 
    arg2 = (int64_t)(val2);
    if (arg1) (arg1)->pages_map2_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_pages_map2_id_get) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int64_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2007_Header_pages_map2_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_pages_map2_id_get" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    result = (int64_t) ((arg1)->pages_map2_id);
    ST(argvi) = SWIG_From_long_SS_long  SWIG_PERL_CALL_ARGS_1((long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_pages_map_offset_set) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    int64_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2007_Header_pages_map_offset_set(self,pages_map_offset);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_pages_map_offset_set" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    ecode2 = SWIG_AsVal_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2007_Header_pages_map_offset_set" "', argument " "2"" of type '" "int64_t""'");
    } 
    arg2 = (int64_t)(val2);
    if (arg1) (arg1)->pages_map_offset = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_pages_map_offset_get) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int64_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2007_Header_pages_map_offset_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_pages_map_offset_get" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    result = (int64_t) ((arg1)->pages_map_offset);
    ST(argvi) = SWIG_From_long_SS_long  SWIG_PERL_CALL_ARGS_1((long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_pages_map_id_set) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    int64_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2007_Header_pages_map_id_set(self,pages_map_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_pages_map_id_set" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    ecode2 = SWIG_AsVal_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2007_Header_pages_map_id_set" "', argument " "2"" of type '" "int64_t""'");
    } 
    arg2 = (int64_t)(val2);
    if (arg1) (arg1)->pages_map_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_pages_map_id_get) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int64_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2007_Header_pages_map_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_pages_map_id_get" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    result = (int64_t) ((arg1)->pages_map_id);
    ST(argvi) = SWIG_From_long_SS_long  SWIG_PERL_CALL_ARGS_1((long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_header2_offset_set) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    int64_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2007_Header_header2_offset_set(self,header2_offset);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_header2_offset_set" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    ecode2 = SWIG_AsVal_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2007_Header_header2_offset_set" "', argument " "2"" of type '" "int64_t""'");
    } 
    arg2 = (int64_t)(val2);
    if (arg1) (arg1)->header2_offset = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_header2_offset_get) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int64_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2007_Header_header2_offset_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_header2_offset_get" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    result = (int64_t) ((arg1)->header2_offset);
    ST(argvi) = SWIG_From_long_SS_long  SWIG_PERL_CALL_ARGS_1((long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_pages_map_size_comp_set) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    int64_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2007_Header_pages_map_size_comp_set(self,pages_map_size_comp);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_pages_map_size_comp_set" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    ecode2 = SWIG_AsVal_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2007_Header_pages_map_size_comp_set" "', argument " "2"" of type '" "int64_t""'");
    } 
    arg2 = (int64_t)(val2);
    if (arg1) (arg1)->pages_map_size_comp = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_pages_map_size_comp_get) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int64_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2007_Header_pages_map_size_comp_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_pages_map_size_comp_get" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    result = (int64_t) ((arg1)->pages_map_size_comp);
    ST(argvi) = SWIG_From_long_SS_long  SWIG_PERL_CALL_ARGS_1((long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_pages_map_size_uncomp_set) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    int64_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2007_Header_pages_map_size_uncomp_set(self,pages_map_size_uncomp);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_pages_map_size_uncomp_set" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    ecode2 = SWIG_AsVal_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2007_Header_pages_map_size_uncomp_set" "', argument " "2"" of type '" "int64_t""'");
    } 
    arg2 = (int64_t)(val2);
    if (arg1) (arg1)->pages_map_size_uncomp = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_pages_map_size_uncomp_get) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int64_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2007_Header_pages_map_size_uncomp_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_pages_map_size_uncomp_get" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    result = (int64_t) ((arg1)->pages_map_size_uncomp);
    ST(argvi) = SWIG_From_long_SS_long  SWIG_PERL_CALL_ARGS_1((long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_pages_amount_set) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    int64_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2007_Header_pages_amount_set(self,pages_amount);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_pages_amount_set" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    ecode2 = SWIG_AsVal_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2007_Header_pages_amount_set" "', argument " "2"" of type '" "int64_t""'");
    } 
    arg2 = (int64_t)(val2);
    if (arg1) (arg1)->pages_amount = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_pages_amount_get) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int64_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2007_Header_pages_amount_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_pages_amount_get" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    result = (int64_t) ((arg1)->pages_amount);
    ST(argvi) = SWIG_From_long_SS_long  SWIG_PERL_CALL_ARGS_1((long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_pages_maxid_set) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    int64_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2007_Header_pages_maxid_set(self,pages_maxid);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_pages_maxid_set" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    ecode2 = SWIG_AsVal_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2007_Header_pages_maxid_set" "', argument " "2"" of type '" "int64_t""'");
    } 
    arg2 = (int64_t)(val2);
    if (arg1) (arg1)->pages_maxid = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_pages_maxid_get) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int64_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2007_Header_pages_maxid_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_pages_maxid_get" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    result = (int64_t) ((arg1)->pages_maxid);
    ST(argvi) = SWIG_From_long_SS_long  SWIG_PERL_CALL_ARGS_1((long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_unknown1_set) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    int64_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2007_Header_unknown1_set(self,unknown1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_unknown1_set" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    ecode2 = SWIG_AsVal_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2007_Header_unknown1_set" "', argument " "2"" of type '" "int64_t""'");
    } 
    arg2 = (int64_t)(val2);
    if (arg1) (arg1)->unknown1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_unknown1_get) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int64_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2007_Header_unknown1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_unknown1_get" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    result = (int64_t) ((arg1)->unknown1);
    ST(argvi) = SWIG_From_long_SS_long  SWIG_PERL_CALL_ARGS_1((long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_unknown2_set) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    int64_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2007_Header_unknown2_set(self,unknown2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_unknown2_set" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    ecode2 = SWIG_AsVal_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2007_Header_unknown2_set" "', argument " "2"" of type '" "int64_t""'");
    } 
    arg2 = (int64_t)(val2);
    if (arg1) (arg1)->unknown2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_unknown2_get) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int64_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2007_Header_unknown2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_unknown2_get" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    result = (int64_t) ((arg1)->unknown2);
    ST(argvi) = SWIG_From_long_SS_long  SWIG_PERL_CALL_ARGS_1((long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_pages_map_crc_uncomp_set) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    int64_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2007_Header_pages_map_crc_uncomp_set(self,pages_map_crc_uncomp);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_pages_map_crc_uncomp_set" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    ecode2 = SWIG_AsVal_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2007_Header_pages_map_crc_uncomp_set" "', argument " "2"" of type '" "int64_t""'");
    } 
    arg2 = (int64_t)(val2);
    if (arg1) (arg1)->pages_map_crc_uncomp = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_pages_map_crc_uncomp_get) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int64_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2007_Header_pages_map_crc_uncomp_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_pages_map_crc_uncomp_get" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    result = (int64_t) ((arg1)->pages_map_crc_uncomp);
    ST(argvi) = SWIG_From_long_SS_long  SWIG_PERL_CALL_ARGS_1((long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_unknown3_set) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    int64_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2007_Header_unknown3_set(self,unknown3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_unknown3_set" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    ecode2 = SWIG_AsVal_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2007_Header_unknown3_set" "', argument " "2"" of type '" "int64_t""'");
    } 
    arg2 = (int64_t)(val2);
    if (arg1) (arg1)->unknown3 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_unknown3_get) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int64_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2007_Header_unknown3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_unknown3_get" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    result = (int64_t) ((arg1)->unknown3);
    ST(argvi) = SWIG_From_long_SS_long  SWIG_PERL_CALL_ARGS_1((long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_unknown4_set) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    int64_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2007_Header_unknown4_set(self,unknown4);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_unknown4_set" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    ecode2 = SWIG_AsVal_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2007_Header_unknown4_set" "', argument " "2"" of type '" "int64_t""'");
    } 
    arg2 = (int64_t)(val2);
    if (arg1) (arg1)->unknown4 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_unknown4_get) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int64_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2007_Header_unknown4_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_unknown4_get" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    result = (int64_t) ((arg1)->unknown4);
    ST(argvi) = SWIG_From_long_SS_long  SWIG_PERL_CALL_ARGS_1((long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_unknown5_set) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    int64_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2007_Header_unknown5_set(self,unknown5);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_unknown5_set" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    ecode2 = SWIG_AsVal_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2007_Header_unknown5_set" "', argument " "2"" of type '" "int64_t""'");
    } 
    arg2 = (int64_t)(val2);
    if (arg1) (arg1)->unknown5 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_unknown5_get) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int64_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2007_Header_unknown5_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_unknown5_get" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    result = (int64_t) ((arg1)->unknown5);
    ST(argvi) = SWIG_From_long_SS_long  SWIG_PERL_CALL_ARGS_1((long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_num_sections_set) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    int64_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2007_Header_num_sections_set(self,num_sections);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_num_sections_set" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    ecode2 = SWIG_AsVal_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2007_Header_num_sections_set" "', argument " "2"" of type '" "int64_t""'");
    } 
    arg2 = (int64_t)(val2);
    if (arg1) (arg1)->num_sections = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_num_sections_get) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int64_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2007_Header_num_sections_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_num_sections_get" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    result = (int64_t) ((arg1)->num_sections);
    ST(argvi) = SWIG_From_long_SS_long  SWIG_PERL_CALL_ARGS_1((long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_sections_map_crc_uncomp_set) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    int64_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2007_Header_sections_map_crc_uncomp_set(self,sections_map_crc_uncomp);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_sections_map_crc_uncomp_set" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    ecode2 = SWIG_AsVal_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2007_Header_sections_map_crc_uncomp_set" "', argument " "2"" of type '" "int64_t""'");
    } 
    arg2 = (int64_t)(val2);
    if (arg1) (arg1)->sections_map_crc_uncomp = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_sections_map_crc_uncomp_get) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int64_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2007_Header_sections_map_crc_uncomp_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_sections_map_crc_uncomp_get" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    result = (int64_t) ((arg1)->sections_map_crc_uncomp);
    ST(argvi) = SWIG_From_long_SS_long  SWIG_PERL_CALL_ARGS_1((long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_sections_map_size_comp_set) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    int64_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2007_Header_sections_map_size_comp_set(self,sections_map_size_comp);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_sections_map_size_comp_set" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    ecode2 = SWIG_AsVal_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2007_Header_sections_map_size_comp_set" "', argument " "2"" of type '" "int64_t""'");
    } 
    arg2 = (int64_t)(val2);
    if (arg1) (arg1)->sections_map_size_comp = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_sections_map_size_comp_get) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int64_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2007_Header_sections_map_size_comp_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_sections_map_size_comp_get" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    result = (int64_t) ((arg1)->sections_map_size_comp);
    ST(argvi) = SWIG_From_long_SS_long  SWIG_PERL_CALL_ARGS_1((long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_sections_map2_id_set) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    int64_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2007_Header_sections_map2_id_set(self,sections_map2_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_sections_map2_id_set" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    ecode2 = SWIG_AsVal_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2007_Header_sections_map2_id_set" "', argument " "2"" of type '" "int64_t""'");
    } 
    arg2 = (int64_t)(val2);
    if (arg1) (arg1)->sections_map2_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_sections_map2_id_get) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int64_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2007_Header_sections_map2_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_sections_map2_id_get" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    result = (int64_t) ((arg1)->sections_map2_id);
    ST(argvi) = SWIG_From_long_SS_long  SWIG_PERL_CALL_ARGS_1((long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_sections_map_id_set) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    int64_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2007_Header_sections_map_id_set(self,sections_map_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_sections_map_id_set" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    ecode2 = SWIG_AsVal_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2007_Header_sections_map_id_set" "', argument " "2"" of type '" "int64_t""'");
    } 
    arg2 = (int64_t)(val2);
    if (arg1) (arg1)->sections_map_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_sections_map_id_get) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int64_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2007_Header_sections_map_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_sections_map_id_get" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    result = (int64_t) ((arg1)->sections_map_id);
    ST(argvi) = SWIG_From_long_SS_long  SWIG_PERL_CALL_ARGS_1((long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_sections_map_size_uncomp_set) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    int64_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2007_Header_sections_map_size_uncomp_set(self,sections_map_size_uncomp);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_sections_map_size_uncomp_set" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    ecode2 = SWIG_AsVal_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2007_Header_sections_map_size_uncomp_set" "', argument " "2"" of type '" "int64_t""'");
    } 
    arg2 = (int64_t)(val2);
    if (arg1) (arg1)->sections_map_size_uncomp = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_sections_map_size_uncomp_get) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int64_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2007_Header_sections_map_size_uncomp_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_sections_map_size_uncomp_get" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    result = (int64_t) ((arg1)->sections_map_size_uncomp);
    ST(argvi) = SWIG_From_long_SS_long  SWIG_PERL_CALL_ARGS_1((long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_sections_map_crc_comp_set) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    int64_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2007_Header_sections_map_crc_comp_set(self,sections_map_crc_comp);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_sections_map_crc_comp_set" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    ecode2 = SWIG_AsVal_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2007_Header_sections_map_crc_comp_set" "', argument " "2"" of type '" "int64_t""'");
    } 
    arg2 = (int64_t)(val2);
    if (arg1) (arg1)->sections_map_crc_comp = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_sections_map_crc_comp_get) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int64_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2007_Header_sections_map_crc_comp_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_sections_map_crc_comp_get" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    result = (int64_t) ((arg1)->sections_map_crc_comp);
    ST(argvi) = SWIG_From_long_SS_long  SWIG_PERL_CALL_ARGS_1((long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_sections_map_correction_set) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    int64_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2007_Header_sections_map_correction_set(self,sections_map_correction);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_sections_map_correction_set" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    ecode2 = SWIG_AsVal_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2007_Header_sections_map_correction_set" "', argument " "2"" of type '" "int64_t""'");
    } 
    arg2 = (int64_t)(val2);
    if (arg1) (arg1)->sections_map_correction = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_sections_map_correction_get) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int64_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2007_Header_sections_map_correction_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_sections_map_correction_get" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    result = (int64_t) ((arg1)->sections_map_correction);
    ST(argvi) = SWIG_From_long_SS_long  SWIG_PERL_CALL_ARGS_1((long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_sections_map_crc_seed_set) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    int64_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2007_Header_sections_map_crc_seed_set(self,sections_map_crc_seed);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_sections_map_crc_seed_set" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    ecode2 = SWIG_AsVal_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2007_Header_sections_map_crc_seed_set" "', argument " "2"" of type '" "int64_t""'");
    } 
    arg2 = (int64_t)(val2);
    if (arg1) (arg1)->sections_map_crc_seed = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_sections_map_crc_seed_get) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int64_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2007_Header_sections_map_crc_seed_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_sections_map_crc_seed_get" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    result = (int64_t) ((arg1)->sections_map_crc_seed);
    ST(argvi) = SWIG_From_long_SS_long  SWIG_PERL_CALL_ARGS_1((long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_stream_version_set) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    int64_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2007_Header_stream_version_set(self,stream_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_stream_version_set" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    ecode2 = SWIG_AsVal_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2007_Header_stream_version_set" "', argument " "2"" of type '" "int64_t""'");
    } 
    arg2 = (int64_t)(val2);
    if (arg1) (arg1)->stream_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_stream_version_get) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int64_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2007_Header_stream_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_stream_version_get" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    result = (int64_t) ((arg1)->stream_version);
    ST(argvi) = SWIG_From_long_SS_long  SWIG_PERL_CALL_ARGS_1((long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_crc_seed_set) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    int64_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2007_Header_crc_seed_set(self,crc_seed);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_crc_seed_set" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    ecode2 = SWIG_AsVal_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2007_Header_crc_seed_set" "', argument " "2"" of type '" "int64_t""'");
    } 
    arg2 = (int64_t)(val2);
    if (arg1) (arg1)->crc_seed = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_crc_seed_get) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int64_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2007_Header_crc_seed_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_crc_seed_get" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    result = (int64_t) ((arg1)->crc_seed);
    ST(argvi) = SWIG_From_long_SS_long  SWIG_PERL_CALL_ARGS_1((long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_crc_seed_encoded_set) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    int64_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2007_Header_crc_seed_encoded_set(self,crc_seed_encoded);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_crc_seed_encoded_set" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    ecode2 = SWIG_AsVal_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2007_Header_crc_seed_encoded_set" "', argument " "2"" of type '" "int64_t""'");
    } 
    arg2 = (int64_t)(val2);
    if (arg1) (arg1)->crc_seed_encoded = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_crc_seed_encoded_get) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int64_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2007_Header_crc_seed_encoded_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_crc_seed_encoded_get" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    result = (int64_t) ((arg1)->crc_seed_encoded);
    ST(argvi) = SWIG_From_long_SS_long  SWIG_PERL_CALL_ARGS_1((long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_random_seed_set) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    int64_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2007_Header_random_seed_set(self,random_seed);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_random_seed_set" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    ecode2 = SWIG_AsVal_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2007_Header_random_seed_set" "', argument " "2"" of type '" "int64_t""'");
    } 
    arg2 = (int64_t)(val2);
    if (arg1) (arg1)->random_seed = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_random_seed_get) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int64_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2007_Header_random_seed_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_random_seed_get" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    result = (int64_t) ((arg1)->random_seed);
    ST(argvi) = SWIG_From_long_SS_long  SWIG_PERL_CALL_ARGS_1((long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_header_crc_set) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    int64_t arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2007_Header_header_crc_set(self,header_crc);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_header_crc_set" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    ecode2 = SWIG_AsVal_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2007_Header_header_crc_set" "', argument " "2"" of type '" "int64_t""'");
    } 
    arg2 = (int64_t)(val2);
    if (arg1) (arg1)->header_crc = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2007_Header_header_crc_get) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int64_t result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2007_Header_header_crc_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2007_Header_header_crc_get" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    result = (int64_t) ((arg1)->header_crc);
    ST(argvi) = SWIG_From_long_SS_long  SWIG_PERL_CALL_ARGS_1((long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_R2007_Header) {
  {
    int argvi = 0;
    struct r2007_file_header *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_R2007_Header();");
    }
    result = (struct r2007_file_header *)calloc(1, sizeof(struct r2007_file_header));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_r2007_file_header, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_R2007_Header) {
  {
    struct r2007_file_header *arg1 = (struct r2007_file_header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_R2007_Header(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_r2007_file_header, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_R2007_Header" "', argument " "1"" of type '" "struct r2007_file_header *""'"); 
    }
    arg1 = (struct r2007_file_header *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_file_ID_string_set) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) (BITCODE_RC *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2004_Header_file_ID_string_set(self,file_ID_string);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_file_ID_string_set" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_R2004_Header_file_ID_string_set" "', argument " "2"" of type '" "BITCODE_RC [12]""'"); 
    } 
    arg2 = (BITCODE_RC *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)12; ++ii) *(BITCODE_RC *)&arg1->file_ID_string[ii] = *((BITCODE_RC *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""file_ID_string""' of type '""BITCODE_RC [12]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_file_ID_string_get) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2004_Header_file_ID_string_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_file_ID_string_get" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    result = (BITCODE_RC *)(BITCODE_RC *) ((arg1)->file_ID_string);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_header_address_set) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    BITCODE_RLx arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2004_Header_header_address_set(self,header_address);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_header_address_set" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2004_Header_header_address_set" "', argument " "2"" of type '" "BITCODE_RLx""'");
    } 
    arg2 = (BITCODE_RLx)(val2);
    if (arg1) (arg1)->header_address = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_header_address_get) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLx result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2004_Header_header_address_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_header_address_get" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    result = (BITCODE_RLx) ((arg1)->header_address);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_header_size_set) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2004_Header_header_size_set(self,header_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_header_size_set" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2004_Header_header_size_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->header_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_header_size_get) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2004_Header_header_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_header_size_get" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    result = (BITCODE_RL) ((arg1)->header_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_x04_set) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2004_Header_x04_set(self,x04);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_x04_set" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2004_Header_x04_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->x04 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_x04_get) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2004_Header_x04_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_x04_get" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    result = (BITCODE_RL) ((arg1)->x04);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_root_tree_node_gap_set) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    BITCODE_RLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2004_Header_root_tree_node_gap_set(self,root_tree_node_gap);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_root_tree_node_gap_set" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2004_Header_root_tree_node_gap_set" "', argument " "2"" of type '" "BITCODE_RLd""'");
    } 
    arg2 = (BITCODE_RLd)(val2);
    if (arg1) (arg1)->root_tree_node_gap = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_root_tree_node_gap_get) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2004_Header_root_tree_node_gap_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_root_tree_node_gap_get" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    result = (BITCODE_RLd) ((arg1)->root_tree_node_gap);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_lowermost_left_tree_node_gap_set) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    BITCODE_RLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2004_Header_lowermost_left_tree_node_gap_set(self,lowermost_left_tree_node_gap);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_lowermost_left_tree_node_gap_set" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2004_Header_lowermost_left_tree_node_gap_set" "', argument " "2"" of type '" "BITCODE_RLd""'");
    } 
    arg2 = (BITCODE_RLd)(val2);
    if (arg1) (arg1)->lowermost_left_tree_node_gap = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_lowermost_left_tree_node_gap_get) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2004_Header_lowermost_left_tree_node_gap_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_lowermost_left_tree_node_gap_get" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    result = (BITCODE_RLd) ((arg1)->lowermost_left_tree_node_gap);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_lowermost_right_tree_node_gap_set) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    BITCODE_RLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2004_Header_lowermost_right_tree_node_gap_set(self,lowermost_right_tree_node_gap);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_lowermost_right_tree_node_gap_set" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2004_Header_lowermost_right_tree_node_gap_set" "', argument " "2"" of type '" "BITCODE_RLd""'");
    } 
    arg2 = (BITCODE_RLd)(val2);
    if (arg1) (arg1)->lowermost_right_tree_node_gap = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_lowermost_right_tree_node_gap_get) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2004_Header_lowermost_right_tree_node_gap_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_lowermost_right_tree_node_gap_get" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    result = (BITCODE_RLd) ((arg1)->lowermost_right_tree_node_gap);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_unknown_long_set) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2004_Header_unknown_long_set(self,unknown_long);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_unknown_long_set" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2004_Header_unknown_long_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->unknown_long = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_unknown_long_get) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2004_Header_unknown_long_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_unknown_long_get" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    result = (BITCODE_RL) ((arg1)->unknown_long);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_last_section_id_set) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2004_Header_last_section_id_set(self,last_section_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_last_section_id_set" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2004_Header_last_section_id_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->last_section_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_last_section_id_get) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2004_Header_last_section_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_last_section_id_get" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    result = (BITCODE_RL) ((arg1)->last_section_id);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_last_section_address_set) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    BITCODE_RLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2004_Header_last_section_address_set(self,last_section_address);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_last_section_address_set" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2004_Header_last_section_address_set" "', argument " "2"" of type '" "BITCODE_RLL""'");
    } 
    arg2 = (BITCODE_RLL)(val2);
    if (arg1) (arg1)->last_section_address = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_last_section_address_get) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2004_Header_last_section_address_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_last_section_address_get" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    result = (BITCODE_RLL) ((arg1)->last_section_address);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_secondheader_address_set) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    BITCODE_RLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2004_Header_secondheader_address_set(self,secondheader_address);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_secondheader_address_set" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2004_Header_secondheader_address_set" "', argument " "2"" of type '" "BITCODE_RLL""'");
    } 
    arg2 = (BITCODE_RLL)(val2);
    if (arg1) (arg1)->secondheader_address = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_secondheader_address_get) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2004_Header_secondheader_address_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_secondheader_address_get" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    result = (BITCODE_RLL) ((arg1)->secondheader_address);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_numgaps_set) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2004_Header_numgaps_set(self,numgaps);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_numgaps_set" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2004_Header_numgaps_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->numgaps = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_numgaps_get) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2004_Header_numgaps_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_numgaps_get" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    result = (BITCODE_RL) ((arg1)->numgaps);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_numsections_set) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2004_Header_numsections_set(self,numsections);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_numsections_set" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2004_Header_numsections_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->numsections = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_numsections_get) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2004_Header_numsections_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_numsections_get" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    result = (BITCODE_RL) ((arg1)->numsections);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_x20_set) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2004_Header_x20_set(self,x20);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_x20_set" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2004_Header_x20_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->x20 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_x20_get) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2004_Header_x20_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_x20_get" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    result = (BITCODE_RL) ((arg1)->x20);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_x80_set) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2004_Header_x80_set(self,x80);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_x80_set" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2004_Header_x80_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->x80 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_x80_get) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2004_Header_x80_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_x80_get" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    result = (BITCODE_RL) ((arg1)->x80);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_x40_set) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2004_Header_x40_set(self,x40);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_x40_set" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2004_Header_x40_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->x40 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_x40_get) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2004_Header_x40_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_x40_get" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    result = (BITCODE_RL) ((arg1)->x40);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_section_map_id_set) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2004_Header_section_map_id_set(self,section_map_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_section_map_id_set" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2004_Header_section_map_id_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->section_map_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_section_map_id_get) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2004_Header_section_map_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_section_map_id_get" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    result = (BITCODE_RL) ((arg1)->section_map_id);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_section_map_address_set) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    BITCODE_RLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2004_Header_section_map_address_set(self,section_map_address);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_section_map_address_set" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2004_Header_section_map_address_set" "', argument " "2"" of type '" "BITCODE_RLL""'");
    } 
    arg2 = (BITCODE_RLL)(val2);
    if (arg1) (arg1)->section_map_address = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_section_map_address_get) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2004_Header_section_map_address_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_section_map_address_get" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    result = (BITCODE_RLL) ((arg1)->section_map_address);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_section_info_id_set) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    BITCODE_RLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2004_Header_section_info_id_set(self,section_info_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_section_info_id_set" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2004_Header_section_info_id_set" "', argument " "2"" of type '" "BITCODE_RLd""'");
    } 
    arg2 = (BITCODE_RLd)(val2);
    if (arg1) (arg1)->section_info_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_section_info_id_get) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2004_Header_section_info_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_section_info_id_get" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    result = (BITCODE_RLd) ((arg1)->section_info_id);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_section_array_size_set) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    BITCODE_RLd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2004_Header_section_array_size_set(self,section_array_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_section_array_size_set" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2004_Header_section_array_size_set" "', argument " "2"" of type '" "BITCODE_RLd""'");
    } 
    arg2 = (BITCODE_RLd)(val2);
    if (arg1) (arg1)->section_array_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_section_array_size_get) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2004_Header_section_array_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_section_array_size_get" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    result = (BITCODE_RLd) ((arg1)->section_array_size);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_gap_array_size_set) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2004_Header_gap_array_size_set(self,gap_array_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_gap_array_size_set" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2004_Header_gap_array_size_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->gap_array_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_gap_array_size_get) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2004_Header_gap_array_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_gap_array_size_get" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    result = (BITCODE_RL) ((arg1)->gap_array_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_crc32_set) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    BITCODE_RLx arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2004_Header_crc32_set(self,crc32);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_crc32_set" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2004_Header_crc32_set" "', argument " "2"" of type '" "BITCODE_RLx""'");
    } 
    arg2 = (BITCODE_RLx)(val2);
    if (arg1) (arg1)->crc32 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_crc32_get) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLx result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2004_Header_crc32_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_crc32_get" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    result = (BITCODE_RLx) ((arg1)->crc32);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_padding_set) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) (BITCODE_RC *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2004_Header_padding_set(self,padding);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_padding_set" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_R2004_Header_padding_set" "', argument " "2"" of type '" "BITCODE_RC [12]""'"); 
    } 
    arg2 = (BITCODE_RC *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)12; ++ii) *(BITCODE_RC *)&arg1->padding[ii] = *((BITCODE_RC *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""padding""' of type '""BITCODE_RC [12]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_padding_get) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2004_Header_padding_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_padding_get" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    result = (BITCODE_RC *)(BITCODE_RC *) ((arg1)->padding);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_section_type_set) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2004_Header_section_type_set(self,section_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_section_type_set" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2004_Header_section_type_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->section_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_section_type_get) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2004_Header_section_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_section_type_get" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    result = (BITCODE_RL) ((arg1)->section_type);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_decomp_data_size_set) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2004_Header_decomp_data_size_set(self,decomp_data_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_decomp_data_size_set" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2004_Header_decomp_data_size_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->decomp_data_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_decomp_data_size_get) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2004_Header_decomp_data_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_decomp_data_size_get" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    result = (BITCODE_RL) ((arg1)->decomp_data_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_comp_data_size_set) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2004_Header_comp_data_size_set(self,comp_data_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_comp_data_size_set" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2004_Header_comp_data_size_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->comp_data_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_comp_data_size_get) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2004_Header_comp_data_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_comp_data_size_get" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    result = (BITCODE_RL) ((arg1)->comp_data_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_compression_type_set) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2004_Header_compression_type_set(self,compression_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_compression_type_set" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2004_Header_compression_type_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->compression_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_compression_type_get) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2004_Header_compression_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_compression_type_get" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    result = (BITCODE_RL) ((arg1)->compression_type);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_checksum_set) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    BITCODE_RLx arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_R2004_Header_checksum_set(self,checksum);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_checksum_set" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_R2004_Header_checksum_set" "', argument " "2"" of type '" "BITCODE_RLx""'");
    } 
    arg2 = (BITCODE_RLx)(val2);
    if (arg1) (arg1)->checksum = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_R2004_Header_checksum_get) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLx result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_R2004_Header_checksum_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_R2004_Header_checksum_get" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    result = (BITCODE_RLx) ((arg1)->checksum);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_R2004_Header) {
  {
    int argvi = 0;
    struct _dwg_R2004_Header *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_R2004_Header();");
    }
    result = (struct _dwg_R2004_Header *)calloc(1, sizeof(struct _dwg_R2004_Header));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_R2004_Header, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_R2004_Header) {
  {
    struct _dwg_R2004_Header *arg1 = (struct _dwg_R2004_Header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_R2004_Header(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_R2004_Header, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_R2004_Header" "', argument " "1"" of type '" "struct _dwg_R2004_Header *""'"); 
    }
    arg1 = (struct _dwg_R2004_Header *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_aux_intro_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) (BITCODE_RC *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_aux_intro_set(self,aux_intro);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_aux_intro_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AuxHeader_aux_intro_set" "', argument " "2"" of type '" "BITCODE_RC [3]""'"); 
    } 
    arg2 = (BITCODE_RC *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)3; ++ii) *(BITCODE_RC *)&arg1->aux_intro[ii] = *((BITCODE_RC *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""aux_intro""' of type '""BITCODE_RC [3]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_aux_intro_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_aux_intro_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_aux_intro_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_RC *)(BITCODE_RC *) ((arg1)->aux_intro);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_dwg_version_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_dwg_version_set(self,dwg_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_dwg_version_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AuxHeader_dwg_version_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->dwg_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_dwg_version_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_dwg_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_dwg_version_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_RS) ((arg1)->dwg_version);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_maint_version_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_maint_version_set(self,maint_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_maint_version_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AuxHeader_maint_version_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->maint_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_maint_version_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_maint_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_maint_version_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_RL) ((arg1)->maint_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_numsaves_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_numsaves_set(self,numsaves);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_numsaves_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AuxHeader_numsaves_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->numsaves = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_numsaves_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_numsaves_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_numsaves_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_RL) ((arg1)->numsaves);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_minus_1_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_minus_1_set(self,minus_1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_minus_1_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AuxHeader_minus_1_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->minus_1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_minus_1_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_minus_1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_minus_1_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_RL) ((arg1)->minus_1);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_numsaves_1_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_numsaves_1_set(self,numsaves_1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_numsaves_1_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AuxHeader_numsaves_1_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->numsaves_1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_numsaves_1_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_numsaves_1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_numsaves_1_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_RS) ((arg1)->numsaves_1);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_numsaves_2_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_numsaves_2_set(self,numsaves_2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_numsaves_2_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AuxHeader_numsaves_2_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->numsaves_2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_numsaves_2_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_numsaves_2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_numsaves_2_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_RS) ((arg1)->numsaves_2);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_zero_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_zero_set(self,zero);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_zero_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AuxHeader_zero_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->zero = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_zero_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_zero_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_zero_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_RL) ((arg1)->zero);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_dwg_version_1_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_dwg_version_1_set(self,dwg_version_1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_dwg_version_1_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AuxHeader_dwg_version_1_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->dwg_version_1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_dwg_version_1_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_dwg_version_1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_dwg_version_1_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_RS) ((arg1)->dwg_version_1);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_maint_version_1_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_maint_version_1_set(self,maint_version_1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_maint_version_1_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AuxHeader_maint_version_1_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->maint_version_1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_maint_version_1_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_maint_version_1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_maint_version_1_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_RL) ((arg1)->maint_version_1);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_dwg_version_2_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_dwg_version_2_set(self,dwg_version_2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_dwg_version_2_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AuxHeader_dwg_version_2_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->dwg_version_2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_dwg_version_2_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_dwg_version_2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_dwg_version_2_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_RS) ((arg1)->dwg_version_2);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_maint_version_2_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_maint_version_2_set(self,maint_version_2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_maint_version_2_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AuxHeader_maint_version_2_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->maint_version_2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_maint_version_2_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_maint_version_2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_maint_version_2_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_RL) ((arg1)->maint_version_2);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_unknown_6rs_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_RS *arg2 = (BITCODE_RS *) (BITCODE_RS *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_unknown_6rs_set(self,unknown_6rs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_unknown_6rs_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_short, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AuxHeader_unknown_6rs_set" "', argument " "2"" of type '" "BITCODE_RS [6]""'"); 
    } 
    arg2 = (BITCODE_RS *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)6; ++ii) *(BITCODE_RS *)&arg1->unknown_6rs[ii] = *((BITCODE_RS *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""unknown_6rs""' of type '""BITCODE_RS [6]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_unknown_6rs_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_unknown_6rs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_unknown_6rs_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_RS *)(BITCODE_RS *) ((arg1)->unknown_6rs);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_short, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_unknown_5rl_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_RL *arg2 = (BITCODE_RL *) (BITCODE_RL *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_unknown_5rl_set(self,unknown_5rl);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_unknown_5rl_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AuxHeader_unknown_5rl_set" "', argument " "2"" of type '" "BITCODE_RL [5]""'"); 
    } 
    arg2 = (BITCODE_RL *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)5; ++ii) *(BITCODE_RL *)&arg1->unknown_5rl[ii] = *((BITCODE_RL *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""unknown_5rl""' of type '""BITCODE_RL [5]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_unknown_5rl_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_unknown_5rl_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_unknown_5rl_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_RL *)(BITCODE_RL *) ((arg1)->unknown_5rl);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_TDCREATE_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_TIMERLL *arg2 = (BITCODE_TIMERLL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_TDCREATE_set(self,TDCREATE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_TDCREATE_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_time_bll, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AuxHeader_TDCREATE_set" "', argument " "2"" of type '" "BITCODE_TIMERLL *""'"); 
    }
    arg2 = (BITCODE_TIMERLL *)(argp2);
    if (arg1) (arg1)->TDCREATE = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_TDCREATE_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TIMERLL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_TDCREATE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_TDCREATE_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_TIMERLL *)& ((arg1)->TDCREATE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_time_bll, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_TDUPDATE_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_TIMERLL *arg2 = (BITCODE_TIMERLL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_TDUPDATE_set(self,TDUPDATE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_TDUPDATE_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_time_bll, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AuxHeader_TDUPDATE_set" "', argument " "2"" of type '" "BITCODE_TIMERLL *""'"); 
    }
    arg2 = (BITCODE_TIMERLL *)(argp2);
    if (arg1) (arg1)->TDUPDATE = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_TDUPDATE_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TIMERLL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_TDUPDATE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_TDUPDATE_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_TIMERLL *)& ((arg1)->TDUPDATE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_time_bll, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_HANDSEED_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_HV arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_HANDSEED_set(self,HANDSEED);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_HANDSEED_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AuxHeader_HANDSEED_set" "', argument " "2"" of type '" "BITCODE_HV""'");
    } 
    arg2 = (BITCODE_HV)(val2);
    if (arg1) (arg1)->HANDSEED = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_HANDSEED_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_HV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_HANDSEED_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_HANDSEED_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_HV) ((arg1)->HANDSEED);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_plot_stamp_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_plot_stamp_set(self,plot_stamp);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_plot_stamp_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AuxHeader_plot_stamp_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->plot_stamp = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_plot_stamp_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_plot_stamp_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_plot_stamp_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_RL) ((arg1)->plot_stamp);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_zero_1_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_zero_1_set(self,zero_1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_zero_1_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AuxHeader_zero_1_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->zero_1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_zero_1_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_zero_1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_zero_1_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_RS) ((arg1)->zero_1);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_numsaves_3_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_numsaves_3_set(self,numsaves_3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_numsaves_3_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AuxHeader_numsaves_3_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->numsaves_3 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_numsaves_3_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_numsaves_3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_numsaves_3_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_RS) ((arg1)->numsaves_3);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_zero_2_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_zero_2_set(self,zero_2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_zero_2_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AuxHeader_zero_2_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->zero_2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_zero_2_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_zero_2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_zero_2_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_RL) ((arg1)->zero_2);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_zero_3_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_zero_3_set(self,zero_3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_zero_3_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AuxHeader_zero_3_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->zero_3 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_zero_3_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_zero_3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_zero_3_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_RL) ((arg1)->zero_3);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_zero_4_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_zero_4_set(self,zero_4);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_zero_4_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AuxHeader_zero_4_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->zero_4 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_zero_4_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_zero_4_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_zero_4_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_RL) ((arg1)->zero_4);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_numsaves_4_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_numsaves_4_set(self,numsaves_4);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_numsaves_4_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AuxHeader_numsaves_4_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->numsaves_4 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_numsaves_4_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_numsaves_4_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_numsaves_4_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_RL) ((arg1)->numsaves_4);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_zero_5_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_zero_5_set(self,zero_5);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_zero_5_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AuxHeader_zero_5_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->zero_5 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_zero_5_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_zero_5_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_zero_5_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_RL) ((arg1)->zero_5);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_zero_6_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_zero_6_set(self,zero_6);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_zero_6_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AuxHeader_zero_6_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->zero_6 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_zero_6_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_zero_6_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_zero_6_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_RL) ((arg1)->zero_6);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_zero_7_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_zero_7_set(self,zero_7);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_zero_7_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AuxHeader_zero_7_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->zero_7 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_zero_7_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_zero_7_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_zero_7_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_RL) ((arg1)->zero_7);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_zero_8_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_zero_8_set(self,zero_8);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_zero_8_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AuxHeader_zero_8_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->zero_8 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_zero_8_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_zero_8_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_zero_8_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_RL) ((arg1)->zero_8);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_zero_18_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_RS *arg2 = (BITCODE_RS *) (BITCODE_RS *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_zero_18_set(self,zero_18);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_zero_18_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_short, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AuxHeader_zero_18_set" "', argument " "2"" of type '" "BITCODE_RS [3]""'"); 
    } 
    arg2 = (BITCODE_RS *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)3; ++ii) *(BITCODE_RS *)&arg1->zero_18[ii] = *((BITCODE_RS *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""zero_18""' of type '""BITCODE_RS [3]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_zero_18_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_zero_18_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_zero_18_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_RS *)(BITCODE_RS *) ((arg1)->zero_18);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_short, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_num_auxheader_variables_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_num_auxheader_variables_set(self,num_auxheader_variables);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_num_auxheader_variables_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AuxHeader_num_auxheader_variables_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->num_auxheader_variables = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_num_auxheader_variables_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_num_auxheader_variables_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_num_auxheader_variables_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_RS) ((arg1)->num_auxheader_variables);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_auxheader_size_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_auxheader_size_set(self,auxheader_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_auxheader_size_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AuxHeader_auxheader_size_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->auxheader_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_auxheader_size_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_auxheader_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_auxheader_size_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_RS) ((arg1)->auxheader_size);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_entities_start_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_entities_start_set(self,entities_start);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_entities_start_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AuxHeader_entities_start_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->entities_start = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_entities_start_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_entities_start_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_entities_start_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_RL) ((arg1)->entities_start);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_entities_end_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_entities_end_set(self,entities_end);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_entities_end_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AuxHeader_entities_end_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->entities_end = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_entities_end_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_entities_end_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_entities_end_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_RL) ((arg1)->entities_end);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_blocks_start_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_blocks_start_set(self,blocks_start);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_blocks_start_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AuxHeader_blocks_start_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->blocks_start = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_blocks_start_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_blocks_start_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_blocks_start_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_RL) ((arg1)->blocks_start);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_extras_start_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_extras_start_set(self,extras_start);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_extras_start_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AuxHeader_extras_start_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->extras_start = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_extras_start_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_extras_start_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_extras_start_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_RL) ((arg1)->extras_start);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_auxheader_address_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_RLx arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_auxheader_address_set(self,auxheader_address);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_auxheader_address_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AuxHeader_auxheader_address_set" "', argument " "2"" of type '" "BITCODE_RLx""'");
    } 
    arg2 = (BITCODE_RLx)(val2);
    if (arg1) (arg1)->auxheader_address = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_auxheader_address_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLx result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_auxheader_address_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_auxheader_address_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_RLx) ((arg1)->auxheader_address);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_num_aux_tables_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_num_aux_tables_set(self,num_aux_tables);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_num_aux_tables_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AuxHeader_num_aux_tables_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->num_aux_tables = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_num_aux_tables_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_num_aux_tables_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_num_aux_tables_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_RS) ((arg1)->num_aux_tables);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_R11_HANDLING_set) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AuxHeader_R11_HANDLING_set(self,R11_HANDLING);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_R11_HANDLING_set" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AuxHeader_R11_HANDLING_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->R11_HANDLING = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AuxHeader_R11_HANDLING_get) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AuxHeader_R11_HANDLING_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AuxHeader_R11_HANDLING_get" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    result = (BITCODE_BS) ((arg1)->R11_HANDLING);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_AuxHeader) {
  {
    int argvi = 0;
    struct _dwg_auxheader *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_AuxHeader();");
    }
    result = (struct _dwg_auxheader *)calloc(1, sizeof(struct _dwg_auxheader));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_auxheader, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_AuxHeader) {
  {
    struct _dwg_auxheader *arg1 = (struct _dwg_auxheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_AuxHeader(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_auxheader, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_AuxHeader" "', argument " "1"" of type '" "struct _dwg_auxheader *""'"); 
    }
    arg1 = (struct _dwg_auxheader *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SummaryInfo_TITLE_set) {
  {
    struct _dwg_summaryinfo *arg1 = (struct _dwg_summaryinfo *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SummaryInfo_TITLE_set(self,TITLE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_summaryinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SummaryInfo_TITLE_set" "', argument " "1"" of type '" "struct _dwg_summaryinfo *""'"); 
    }
    arg1 = (struct _dwg_summaryinfo *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_SummaryInfo_TITLE_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->TITLE);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->TITLE = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->TITLE = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SummaryInfo_TITLE_get) {
  {
    struct _dwg_summaryinfo *arg1 = (struct _dwg_summaryinfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SummaryInfo_TITLE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_summaryinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SummaryInfo_TITLE_get" "', argument " "1"" of type '" "struct _dwg_summaryinfo *""'"); 
    }
    arg1 = (struct _dwg_summaryinfo *)(argp1);
    result = (BITCODE_TV) ((arg1)->TITLE);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SummaryInfo_SUBJECT_set) {
  {
    struct _dwg_summaryinfo *arg1 = (struct _dwg_summaryinfo *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SummaryInfo_SUBJECT_set(self,SUBJECT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_summaryinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SummaryInfo_SUBJECT_set" "', argument " "1"" of type '" "struct _dwg_summaryinfo *""'"); 
    }
    arg1 = (struct _dwg_summaryinfo *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_SummaryInfo_SUBJECT_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->SUBJECT);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->SUBJECT = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->SUBJECT = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SummaryInfo_SUBJECT_get) {
  {
    struct _dwg_summaryinfo *arg1 = (struct _dwg_summaryinfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SummaryInfo_SUBJECT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_summaryinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SummaryInfo_SUBJECT_get" "', argument " "1"" of type '" "struct _dwg_summaryinfo *""'"); 
    }
    arg1 = (struct _dwg_summaryinfo *)(argp1);
    result = (BITCODE_TV) ((arg1)->SUBJECT);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SummaryInfo_AUTHOR_set) {
  {
    struct _dwg_summaryinfo *arg1 = (struct _dwg_summaryinfo *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SummaryInfo_AUTHOR_set(self,AUTHOR);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_summaryinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SummaryInfo_AUTHOR_set" "', argument " "1"" of type '" "struct _dwg_summaryinfo *""'"); 
    }
    arg1 = (struct _dwg_summaryinfo *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_SummaryInfo_AUTHOR_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->AUTHOR);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->AUTHOR = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->AUTHOR = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SummaryInfo_AUTHOR_get) {
  {
    struct _dwg_summaryinfo *arg1 = (struct _dwg_summaryinfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SummaryInfo_AUTHOR_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_summaryinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SummaryInfo_AUTHOR_get" "', argument " "1"" of type '" "struct _dwg_summaryinfo *""'"); 
    }
    arg1 = (struct _dwg_summaryinfo *)(argp1);
    result = (BITCODE_TV) ((arg1)->AUTHOR);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SummaryInfo_KEYWORDS_set) {
  {
    struct _dwg_summaryinfo *arg1 = (struct _dwg_summaryinfo *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SummaryInfo_KEYWORDS_set(self,KEYWORDS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_summaryinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SummaryInfo_KEYWORDS_set" "', argument " "1"" of type '" "struct _dwg_summaryinfo *""'"); 
    }
    arg1 = (struct _dwg_summaryinfo *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_SummaryInfo_KEYWORDS_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->KEYWORDS);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->KEYWORDS = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->KEYWORDS = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SummaryInfo_KEYWORDS_get) {
  {
    struct _dwg_summaryinfo *arg1 = (struct _dwg_summaryinfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SummaryInfo_KEYWORDS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_summaryinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SummaryInfo_KEYWORDS_get" "', argument " "1"" of type '" "struct _dwg_summaryinfo *""'"); 
    }
    arg1 = (struct _dwg_summaryinfo *)(argp1);
    result = (BITCODE_TV) ((arg1)->KEYWORDS);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SummaryInfo_COMMENTS_set) {
  {
    struct _dwg_summaryinfo *arg1 = (struct _dwg_summaryinfo *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SummaryInfo_COMMENTS_set(self,COMMENTS);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_summaryinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SummaryInfo_COMMENTS_set" "', argument " "1"" of type '" "struct _dwg_summaryinfo *""'"); 
    }
    arg1 = (struct _dwg_summaryinfo *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_SummaryInfo_COMMENTS_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->COMMENTS);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->COMMENTS = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->COMMENTS = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SummaryInfo_COMMENTS_get) {
  {
    struct _dwg_summaryinfo *arg1 = (struct _dwg_summaryinfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SummaryInfo_COMMENTS_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_summaryinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SummaryInfo_COMMENTS_get" "', argument " "1"" of type '" "struct _dwg_summaryinfo *""'"); 
    }
    arg1 = (struct _dwg_summaryinfo *)(argp1);
    result = (BITCODE_TV) ((arg1)->COMMENTS);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SummaryInfo_LASTSAVEDBY_set) {
  {
    struct _dwg_summaryinfo *arg1 = (struct _dwg_summaryinfo *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SummaryInfo_LASTSAVEDBY_set(self,LASTSAVEDBY);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_summaryinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SummaryInfo_LASTSAVEDBY_set" "', argument " "1"" of type '" "struct _dwg_summaryinfo *""'"); 
    }
    arg1 = (struct _dwg_summaryinfo *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_SummaryInfo_LASTSAVEDBY_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->LASTSAVEDBY);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->LASTSAVEDBY = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->LASTSAVEDBY = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SummaryInfo_LASTSAVEDBY_get) {
  {
    struct _dwg_summaryinfo *arg1 = (struct _dwg_summaryinfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SummaryInfo_LASTSAVEDBY_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_summaryinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SummaryInfo_LASTSAVEDBY_get" "', argument " "1"" of type '" "struct _dwg_summaryinfo *""'"); 
    }
    arg1 = (struct _dwg_summaryinfo *)(argp1);
    result = (BITCODE_TV) ((arg1)->LASTSAVEDBY);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SummaryInfo_REVISIONNUMBER_set) {
  {
    struct _dwg_summaryinfo *arg1 = (struct _dwg_summaryinfo *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SummaryInfo_REVISIONNUMBER_set(self,REVISIONNUMBER);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_summaryinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SummaryInfo_REVISIONNUMBER_set" "', argument " "1"" of type '" "struct _dwg_summaryinfo *""'"); 
    }
    arg1 = (struct _dwg_summaryinfo *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_SummaryInfo_REVISIONNUMBER_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->REVISIONNUMBER);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->REVISIONNUMBER = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->REVISIONNUMBER = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SummaryInfo_REVISIONNUMBER_get) {
  {
    struct _dwg_summaryinfo *arg1 = (struct _dwg_summaryinfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SummaryInfo_REVISIONNUMBER_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_summaryinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SummaryInfo_REVISIONNUMBER_get" "', argument " "1"" of type '" "struct _dwg_summaryinfo *""'"); 
    }
    arg1 = (struct _dwg_summaryinfo *)(argp1);
    result = (BITCODE_TV) ((arg1)->REVISIONNUMBER);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SummaryInfo_HYPERLINKBASE_set) {
  {
    struct _dwg_summaryinfo *arg1 = (struct _dwg_summaryinfo *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SummaryInfo_HYPERLINKBASE_set(self,HYPERLINKBASE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_summaryinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SummaryInfo_HYPERLINKBASE_set" "', argument " "1"" of type '" "struct _dwg_summaryinfo *""'"); 
    }
    arg1 = (struct _dwg_summaryinfo *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_SummaryInfo_HYPERLINKBASE_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->HYPERLINKBASE);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->HYPERLINKBASE = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->HYPERLINKBASE = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SummaryInfo_HYPERLINKBASE_get) {
  {
    struct _dwg_summaryinfo *arg1 = (struct _dwg_summaryinfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SummaryInfo_HYPERLINKBASE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_summaryinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SummaryInfo_HYPERLINKBASE_get" "', argument " "1"" of type '" "struct _dwg_summaryinfo *""'"); 
    }
    arg1 = (struct _dwg_summaryinfo *)(argp1);
    result = (BITCODE_TV) ((arg1)->HYPERLINKBASE);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SummaryInfo_TDINDWG_set) {
  {
    struct _dwg_summaryinfo *arg1 = (struct _dwg_summaryinfo *) 0 ;
    BITCODE_TIMERLL *arg2 = (BITCODE_TIMERLL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SummaryInfo_TDINDWG_set(self,TDINDWG);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_summaryinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SummaryInfo_TDINDWG_set" "', argument " "1"" of type '" "struct _dwg_summaryinfo *""'"); 
    }
    arg1 = (struct _dwg_summaryinfo *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_time_bll, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_SummaryInfo_TDINDWG_set" "', argument " "2"" of type '" "BITCODE_TIMERLL *""'"); 
    }
    arg2 = (BITCODE_TIMERLL *)(argp2);
    if (arg1) (arg1)->TDINDWG = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SummaryInfo_TDINDWG_get) {
  {
    struct _dwg_summaryinfo *arg1 = (struct _dwg_summaryinfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TIMERLL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SummaryInfo_TDINDWG_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_summaryinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SummaryInfo_TDINDWG_get" "', argument " "1"" of type '" "struct _dwg_summaryinfo *""'"); 
    }
    arg1 = (struct _dwg_summaryinfo *)(argp1);
    result = (BITCODE_TIMERLL *)& ((arg1)->TDINDWG);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_time_bll, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SummaryInfo_TDCREATE_set) {
  {
    struct _dwg_summaryinfo *arg1 = (struct _dwg_summaryinfo *) 0 ;
    BITCODE_TIMERLL *arg2 = (BITCODE_TIMERLL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SummaryInfo_TDCREATE_set(self,TDCREATE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_summaryinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SummaryInfo_TDCREATE_set" "', argument " "1"" of type '" "struct _dwg_summaryinfo *""'"); 
    }
    arg1 = (struct _dwg_summaryinfo *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_time_bll, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_SummaryInfo_TDCREATE_set" "', argument " "2"" of type '" "BITCODE_TIMERLL *""'"); 
    }
    arg2 = (BITCODE_TIMERLL *)(argp2);
    if (arg1) (arg1)->TDCREATE = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SummaryInfo_TDCREATE_get) {
  {
    struct _dwg_summaryinfo *arg1 = (struct _dwg_summaryinfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TIMERLL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SummaryInfo_TDCREATE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_summaryinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SummaryInfo_TDCREATE_get" "', argument " "1"" of type '" "struct _dwg_summaryinfo *""'"); 
    }
    arg1 = (struct _dwg_summaryinfo *)(argp1);
    result = (BITCODE_TIMERLL *)& ((arg1)->TDCREATE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_time_bll, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SummaryInfo_TDUPDATE_set) {
  {
    struct _dwg_summaryinfo *arg1 = (struct _dwg_summaryinfo *) 0 ;
    BITCODE_TIMERLL *arg2 = (BITCODE_TIMERLL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SummaryInfo_TDUPDATE_set(self,TDUPDATE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_summaryinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SummaryInfo_TDUPDATE_set" "', argument " "1"" of type '" "struct _dwg_summaryinfo *""'"); 
    }
    arg1 = (struct _dwg_summaryinfo *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_time_bll, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_SummaryInfo_TDUPDATE_set" "', argument " "2"" of type '" "BITCODE_TIMERLL *""'"); 
    }
    arg2 = (BITCODE_TIMERLL *)(argp2);
    if (arg1) (arg1)->TDUPDATE = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SummaryInfo_TDUPDATE_get) {
  {
    struct _dwg_summaryinfo *arg1 = (struct _dwg_summaryinfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TIMERLL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SummaryInfo_TDUPDATE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_summaryinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SummaryInfo_TDUPDATE_get" "', argument " "1"" of type '" "struct _dwg_summaryinfo *""'"); 
    }
    arg1 = (struct _dwg_summaryinfo *)(argp1);
    result = (BITCODE_TIMERLL *)& ((arg1)->TDUPDATE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_time_bll, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SummaryInfo_num_props_set) {
  {
    struct _dwg_summaryinfo *arg1 = (struct _dwg_summaryinfo *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SummaryInfo_num_props_set(self,num_props);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_summaryinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SummaryInfo_num_props_set" "', argument " "1"" of type '" "struct _dwg_summaryinfo *""'"); 
    }
    arg1 = (struct _dwg_summaryinfo *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SummaryInfo_num_props_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->num_props = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SummaryInfo_num_props_get) {
  {
    struct _dwg_summaryinfo *arg1 = (struct _dwg_summaryinfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SummaryInfo_num_props_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_summaryinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SummaryInfo_num_props_get" "', argument " "1"" of type '" "struct _dwg_summaryinfo *""'"); 
    }
    arg1 = (struct _dwg_summaryinfo *)(argp1);
    result = (BITCODE_RS) ((arg1)->num_props);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SummaryInfo_props_set) {
  {
    struct _dwg_summaryinfo *arg1 = (struct _dwg_summaryinfo *) 0 ;
    Dwg_SummaryInfo_Property *arg2 = (Dwg_SummaryInfo_Property *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SummaryInfo_props_set(self,props);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_summaryinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SummaryInfo_props_set" "', argument " "1"" of type '" "struct _dwg_summaryinfo *""'"); 
    }
    arg1 = (struct _dwg_summaryinfo *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_SummaryInfo_Property, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_SummaryInfo_props_set" "', argument " "2"" of type '" "Dwg_SummaryInfo_Property *""'"); 
    }
    arg2 = (Dwg_SummaryInfo_Property *)(argp2);
    if (arg1) (arg1)->props = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SummaryInfo_props_get) {
  {
    struct _dwg_summaryinfo *arg1 = (struct _dwg_summaryinfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_SummaryInfo_Property *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SummaryInfo_props_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_summaryinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SummaryInfo_props_get" "', argument " "1"" of type '" "struct _dwg_summaryinfo *""'"); 
    }
    arg1 = (struct _dwg_summaryinfo *)(argp1);
    result = (Dwg_SummaryInfo_Property *) ((arg1)->props);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_SummaryInfo_Property, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SummaryInfo_unknown1_set) {
  {
    struct _dwg_summaryinfo *arg1 = (struct _dwg_summaryinfo *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SummaryInfo_unknown1_set(self,unknown1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_summaryinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SummaryInfo_unknown1_set" "', argument " "1"" of type '" "struct _dwg_summaryinfo *""'"); 
    }
    arg1 = (struct _dwg_summaryinfo *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SummaryInfo_unknown1_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->unknown1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SummaryInfo_unknown1_get) {
  {
    struct _dwg_summaryinfo *arg1 = (struct _dwg_summaryinfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SummaryInfo_unknown1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_summaryinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SummaryInfo_unknown1_get" "', argument " "1"" of type '" "struct _dwg_summaryinfo *""'"); 
    }
    arg1 = (struct _dwg_summaryinfo *)(argp1);
    result = (BITCODE_RL) ((arg1)->unknown1);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SummaryInfo_unknown2_set) {
  {
    struct _dwg_summaryinfo *arg1 = (struct _dwg_summaryinfo *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SummaryInfo_unknown2_set(self,unknown2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_summaryinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SummaryInfo_unknown2_set" "', argument " "1"" of type '" "struct _dwg_summaryinfo *""'"); 
    }
    arg1 = (struct _dwg_summaryinfo *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SummaryInfo_unknown2_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->unknown2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SummaryInfo_unknown2_get) {
  {
    struct _dwg_summaryinfo *arg1 = (struct _dwg_summaryinfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SummaryInfo_unknown2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_summaryinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SummaryInfo_unknown2_get" "', argument " "1"" of type '" "struct _dwg_summaryinfo *""'"); 
    }
    arg1 = (struct _dwg_summaryinfo *)(argp1);
    result = (BITCODE_RL) ((arg1)->unknown2);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_SummaryInfo) {
  {
    int argvi = 0;
    struct _dwg_summaryinfo *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_SummaryInfo();");
    }
    result = (struct _dwg_summaryinfo *)calloc(1, sizeof(struct _dwg_summaryinfo));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_summaryinfo, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_SummaryInfo) {
  {
    struct _dwg_summaryinfo *arg1 = (struct _dwg_summaryinfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_SummaryInfo(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_summaryinfo, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_SummaryInfo" "', argument " "1"" of type '" "struct _dwg_summaryinfo *""'"); 
    }
    arg1 = (struct _dwg_summaryinfo *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AppInfo_class_version_set) {
  {
    struct _dwg_appinfo *arg1 = (struct _dwg_appinfo *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AppInfo_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_appinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AppInfo_class_version_set" "', argument " "1"" of type '" "struct _dwg_appinfo *""'"); 
    }
    arg1 = (struct _dwg_appinfo *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AppInfo_class_version_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AppInfo_class_version_get) {
  {
    struct _dwg_appinfo *arg1 = (struct _dwg_appinfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AppInfo_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_appinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AppInfo_class_version_get" "', argument " "1"" of type '" "struct _dwg_appinfo *""'"); 
    }
    arg1 = (struct _dwg_appinfo *)(argp1);
    result = (BITCODE_RL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AppInfo_num_strings_set) {
  {
    struct _dwg_appinfo *arg1 = (struct _dwg_appinfo *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AppInfo_num_strings_set(self,num_strings);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_appinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AppInfo_num_strings_set" "', argument " "1"" of type '" "struct _dwg_appinfo *""'"); 
    }
    arg1 = (struct _dwg_appinfo *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AppInfo_num_strings_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->num_strings = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AppInfo_num_strings_get) {
  {
    struct _dwg_appinfo *arg1 = (struct _dwg_appinfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AppInfo_num_strings_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_appinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AppInfo_num_strings_get" "', argument " "1"" of type '" "struct _dwg_appinfo *""'"); 
    }
    arg1 = (struct _dwg_appinfo *)(argp1);
    result = (BITCODE_RL) ((arg1)->num_strings);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AppInfo_appinfo_name_set) {
  {
    struct _dwg_appinfo *arg1 = (struct _dwg_appinfo *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AppInfo_appinfo_name_set(self,appinfo_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_appinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AppInfo_appinfo_name_set" "', argument " "1"" of type '" "struct _dwg_appinfo *""'"); 
    }
    arg1 = (struct _dwg_appinfo *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AppInfo_appinfo_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->appinfo_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->appinfo_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->appinfo_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AppInfo_appinfo_name_get) {
  {
    struct _dwg_appinfo *arg1 = (struct _dwg_appinfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AppInfo_appinfo_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_appinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AppInfo_appinfo_name_get" "', argument " "1"" of type '" "struct _dwg_appinfo *""'"); 
    }
    arg1 = (struct _dwg_appinfo *)(argp1);
    result = (BITCODE_TV) ((arg1)->appinfo_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AppInfo_version_checksum_set) {
  {
    struct _dwg_appinfo *arg1 = (struct _dwg_appinfo *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) (BITCODE_RC *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AppInfo_version_checksum_set(self,version_checksum);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_appinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AppInfo_version_checksum_set" "', argument " "1"" of type '" "struct _dwg_appinfo *""'"); 
    }
    arg1 = (struct _dwg_appinfo *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AppInfo_version_checksum_set" "', argument " "2"" of type '" "BITCODE_RC [16]""'"); 
    } 
    arg2 = (BITCODE_RC *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)16; ++ii) *(BITCODE_RC *)&arg1->version_checksum[ii] = *((BITCODE_RC *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""version_checksum""' of type '""BITCODE_RC [16]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AppInfo_version_checksum_get) {
  {
    struct _dwg_appinfo *arg1 = (struct _dwg_appinfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AppInfo_version_checksum_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_appinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AppInfo_version_checksum_get" "', argument " "1"" of type '" "struct _dwg_appinfo *""'"); 
    }
    arg1 = (struct _dwg_appinfo *)(argp1);
    result = (BITCODE_RC *)(BITCODE_RC *) ((arg1)->version_checksum);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AppInfo_comment_checksum_set) {
  {
    struct _dwg_appinfo *arg1 = (struct _dwg_appinfo *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) (BITCODE_RC *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AppInfo_comment_checksum_set(self,comment_checksum);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_appinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AppInfo_comment_checksum_set" "', argument " "1"" of type '" "struct _dwg_appinfo *""'"); 
    }
    arg1 = (struct _dwg_appinfo *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AppInfo_comment_checksum_set" "', argument " "2"" of type '" "BITCODE_RC [16]""'"); 
    } 
    arg2 = (BITCODE_RC *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)16; ++ii) *(BITCODE_RC *)&arg1->comment_checksum[ii] = *((BITCODE_RC *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""comment_checksum""' of type '""BITCODE_RC [16]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AppInfo_comment_checksum_get) {
  {
    struct _dwg_appinfo *arg1 = (struct _dwg_appinfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AppInfo_comment_checksum_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_appinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AppInfo_comment_checksum_get" "', argument " "1"" of type '" "struct _dwg_appinfo *""'"); 
    }
    arg1 = (struct _dwg_appinfo *)(argp1);
    result = (BITCODE_RC *)(BITCODE_RC *) ((arg1)->comment_checksum);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AppInfo_product_checksum_set) {
  {
    struct _dwg_appinfo *arg1 = (struct _dwg_appinfo *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) (BITCODE_RC *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AppInfo_product_checksum_set(self,product_checksum);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_appinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AppInfo_product_checksum_set" "', argument " "1"" of type '" "struct _dwg_appinfo *""'"); 
    }
    arg1 = (struct _dwg_appinfo *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AppInfo_product_checksum_set" "', argument " "2"" of type '" "BITCODE_RC [16]""'"); 
    } 
    arg2 = (BITCODE_RC *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)16; ++ii) *(BITCODE_RC *)&arg1->product_checksum[ii] = *((BITCODE_RC *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""product_checksum""' of type '""BITCODE_RC [16]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AppInfo_product_checksum_get) {
  {
    struct _dwg_appinfo *arg1 = (struct _dwg_appinfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AppInfo_product_checksum_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_appinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AppInfo_product_checksum_get" "', argument " "1"" of type '" "struct _dwg_appinfo *""'"); 
    }
    arg1 = (struct _dwg_appinfo *)(argp1);
    result = (BITCODE_RC *)(BITCODE_RC *) ((arg1)->product_checksum);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AppInfo_version_set) {
  {
    struct _dwg_appinfo *arg1 = (struct _dwg_appinfo *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AppInfo_version_set(self,version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_appinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AppInfo_version_set" "', argument " "1"" of type '" "struct _dwg_appinfo *""'"); 
    }
    arg1 = (struct _dwg_appinfo *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AppInfo_version_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->version);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->version = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->version = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AppInfo_version_get) {
  {
    struct _dwg_appinfo *arg1 = (struct _dwg_appinfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AppInfo_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_appinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AppInfo_version_get" "', argument " "1"" of type '" "struct _dwg_appinfo *""'"); 
    }
    arg1 = (struct _dwg_appinfo *)(argp1);
    result = (BITCODE_TV) ((arg1)->version);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AppInfo_comment_set) {
  {
    struct _dwg_appinfo *arg1 = (struct _dwg_appinfo *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AppInfo_comment_set(self,comment);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_appinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AppInfo_comment_set" "', argument " "1"" of type '" "struct _dwg_appinfo *""'"); 
    }
    arg1 = (struct _dwg_appinfo *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AppInfo_comment_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->comment);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->comment = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->comment = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AppInfo_comment_get) {
  {
    struct _dwg_appinfo *arg1 = (struct _dwg_appinfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AppInfo_comment_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_appinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AppInfo_comment_get" "', argument " "1"" of type '" "struct _dwg_appinfo *""'"); 
    }
    arg1 = (struct _dwg_appinfo *)(argp1);
    result = (BITCODE_TV) ((arg1)->comment);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AppInfo_product_info_set) {
  {
    struct _dwg_appinfo *arg1 = (struct _dwg_appinfo *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AppInfo_product_info_set(self,product_info);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_appinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AppInfo_product_info_set" "', argument " "1"" of type '" "struct _dwg_appinfo *""'"); 
    }
    arg1 = (struct _dwg_appinfo *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AppInfo_product_info_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->product_info);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->product_info = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->product_info = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AppInfo_product_info_get) {
  {
    struct _dwg_appinfo *arg1 = (struct _dwg_appinfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AppInfo_product_info_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_appinfo, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AppInfo_product_info_get" "', argument " "1"" of type '" "struct _dwg_appinfo *""'"); 
    }
    arg1 = (struct _dwg_appinfo *)(argp1);
    result = (BITCODE_TV) ((arg1)->product_info);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_AppInfo) {
  {
    int argvi = 0;
    struct _dwg_appinfo *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_AppInfo();");
    }
    result = (struct _dwg_appinfo *)calloc(1, sizeof(struct _dwg_appinfo));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_appinfo, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_AppInfo) {
  {
    struct _dwg_appinfo *arg1 = (struct _dwg_appinfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_AppInfo(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_appinfo, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_AppInfo" "', argument " "1"" of type '" "struct _dwg_appinfo *""'"); 
    }
    arg1 = (struct _dwg_appinfo *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FileDepList_num_features_set) {
  {
    struct _dwg_filedeplist *arg1 = (struct _dwg_filedeplist *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_FileDepList_num_features_set(self,num_features);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_filedeplist, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FileDepList_num_features_set" "', argument " "1"" of type '" "struct _dwg_filedeplist *""'"); 
    }
    arg1 = (struct _dwg_filedeplist *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_FileDepList_num_features_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->num_features = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FileDepList_num_features_get) {
  {
    struct _dwg_filedeplist *arg1 = (struct _dwg_filedeplist *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_FileDepList_num_features_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_filedeplist, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FileDepList_num_features_get" "', argument " "1"" of type '" "struct _dwg_filedeplist *""'"); 
    }
    arg1 = (struct _dwg_filedeplist *)(argp1);
    result = (BITCODE_RL) ((arg1)->num_features);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FileDepList_features_set) {
  {
    struct _dwg_filedeplist *arg1 = (struct _dwg_filedeplist *) 0 ;
    BITCODE_TV *arg2 = (BITCODE_TV *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_FileDepList_features_set(self,features);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_filedeplist, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FileDepList_features_set" "', argument " "1"" of type '" "struct _dwg_filedeplist *""'"); 
    }
    arg1 = (struct _dwg_filedeplist *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_FileDepList_features_set" "', argument " "2"" of type '" "BITCODE_TV *""'"); 
    }
    arg2 = (BITCODE_TV *)(argp2);
    if (arg1) (arg1)->features = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FileDepList_features_get) {
  {
    struct _dwg_filedeplist *arg1 = (struct _dwg_filedeplist *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_FileDepList_features_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_filedeplist, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FileDepList_features_get" "', argument " "1"" of type '" "struct _dwg_filedeplist *""'"); 
    }
    arg1 = (struct _dwg_filedeplist *)(argp1);
    result = (BITCODE_TV *) ((arg1)->features);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FileDepList_num_files_set) {
  {
    struct _dwg_filedeplist *arg1 = (struct _dwg_filedeplist *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_FileDepList_num_files_set(self,num_files);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_filedeplist, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FileDepList_num_files_set" "', argument " "1"" of type '" "struct _dwg_filedeplist *""'"); 
    }
    arg1 = (struct _dwg_filedeplist *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_FileDepList_num_files_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->num_files = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FileDepList_num_files_get) {
  {
    struct _dwg_filedeplist *arg1 = (struct _dwg_filedeplist *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_FileDepList_num_files_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_filedeplist, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FileDepList_num_files_get" "', argument " "1"" of type '" "struct _dwg_filedeplist *""'"); 
    }
    arg1 = (struct _dwg_filedeplist *)(argp1);
    result = (BITCODE_RL) ((arg1)->num_files);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FileDepList_files_set) {
  {
    struct _dwg_filedeplist *arg1 = (struct _dwg_filedeplist *) 0 ;
    Dwg_FileDepList_Files *arg2 = (Dwg_FileDepList_Files *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_FileDepList_files_set(self,files);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_filedeplist, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FileDepList_files_set" "', argument " "1"" of type '" "struct _dwg_filedeplist *""'"); 
    }
    arg1 = (struct _dwg_filedeplist *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_FileDepList_Files, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_FileDepList_files_set" "', argument " "2"" of type '" "Dwg_FileDepList_Files *""'"); 
    }
    arg2 = (Dwg_FileDepList_Files *)(argp2);
    if (arg1) (arg1)->files = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_FileDepList_files_get) {
  {
    struct _dwg_filedeplist *arg1 = (struct _dwg_filedeplist *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_FileDepList_Files *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_FileDepList_files_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_filedeplist, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_FileDepList_files_get" "', argument " "1"" of type '" "struct _dwg_filedeplist *""'"); 
    }
    arg1 = (struct _dwg_filedeplist *)(argp1);
    result = (Dwg_FileDepList_Files *) ((arg1)->files);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_FileDepList_Files, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_FileDepList) {
  {
    int argvi = 0;
    struct _dwg_filedeplist *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_FileDepList();");
    }
    result = (struct _dwg_filedeplist *)calloc(1, sizeof(struct _dwg_filedeplist));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_filedeplist, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_FileDepList) {
  {
    struct _dwg_filedeplist *arg1 = (struct _dwg_filedeplist *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_FileDepList(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_filedeplist, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_FileDepList" "', argument " "1"" of type '" "struct _dwg_filedeplist *""'"); 
    }
    arg1 = (struct _dwg_filedeplist *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Security_unknown_1_set) {
  {
    struct _dwg_security *arg1 = (struct _dwg_security *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Security_unknown_1_set(self,unknown_1);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_security, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Security_unknown_1_set" "', argument " "1"" of type '" "struct _dwg_security *""'"); 
    }
    arg1 = (struct _dwg_security *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Security_unknown_1_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->unknown_1 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Security_unknown_1_get) {
  {
    struct _dwg_security *arg1 = (struct _dwg_security *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Security_unknown_1_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_security, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Security_unknown_1_get" "', argument " "1"" of type '" "struct _dwg_security *""'"); 
    }
    arg1 = (struct _dwg_security *)(argp1);
    result = (BITCODE_RL) ((arg1)->unknown_1);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Security_unknown_2_set) {
  {
    struct _dwg_security *arg1 = (struct _dwg_security *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Security_unknown_2_set(self,unknown_2);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_security, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Security_unknown_2_set" "', argument " "1"" of type '" "struct _dwg_security *""'"); 
    }
    arg1 = (struct _dwg_security *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Security_unknown_2_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->unknown_2 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Security_unknown_2_get) {
  {
    struct _dwg_security *arg1 = (struct _dwg_security *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Security_unknown_2_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_security, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Security_unknown_2_get" "', argument " "1"" of type '" "struct _dwg_security *""'"); 
    }
    arg1 = (struct _dwg_security *)(argp1);
    result = (BITCODE_RL) ((arg1)->unknown_2);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Security_unknown_3_set) {
  {
    struct _dwg_security *arg1 = (struct _dwg_security *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Security_unknown_3_set(self,unknown_3);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_security, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Security_unknown_3_set" "', argument " "1"" of type '" "struct _dwg_security *""'"); 
    }
    arg1 = (struct _dwg_security *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Security_unknown_3_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->unknown_3 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Security_unknown_3_get) {
  {
    struct _dwg_security *arg1 = (struct _dwg_security *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Security_unknown_3_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_security, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Security_unknown_3_get" "', argument " "1"" of type '" "struct _dwg_security *""'"); 
    }
    arg1 = (struct _dwg_security *)(argp1);
    result = (BITCODE_RL) ((arg1)->unknown_3);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Security_crypto_id_set) {
  {
    struct _dwg_security *arg1 = (struct _dwg_security *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Security_crypto_id_set(self,crypto_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_security, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Security_crypto_id_set" "', argument " "1"" of type '" "struct _dwg_security *""'"); 
    }
    arg1 = (struct _dwg_security *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Security_crypto_id_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->crypto_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Security_crypto_id_get) {
  {
    struct _dwg_security *arg1 = (struct _dwg_security *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Security_crypto_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_security, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Security_crypto_id_get" "', argument " "1"" of type '" "struct _dwg_security *""'"); 
    }
    arg1 = (struct _dwg_security *)(argp1);
    result = (BITCODE_RL) ((arg1)->crypto_id);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Security_crypto_name_set) {
  {
    struct _dwg_security *arg1 = (struct _dwg_security *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Security_crypto_name_set(self,crypto_name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_security, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Security_crypto_name_set" "', argument " "1"" of type '" "struct _dwg_security *""'"); 
    }
    arg1 = (struct _dwg_security *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Security_crypto_name_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->crypto_name);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->crypto_name = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->crypto_name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Security_crypto_name_get) {
  {
    struct _dwg_security *arg1 = (struct _dwg_security *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Security_crypto_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_security, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Security_crypto_name_get" "', argument " "1"" of type '" "struct _dwg_security *""'"); 
    }
    arg1 = (struct _dwg_security *)(argp1);
    result = (BITCODE_TV) ((arg1)->crypto_name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Security_algo_id_set) {
  {
    struct _dwg_security *arg1 = (struct _dwg_security *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Security_algo_id_set(self,algo_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_security, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Security_algo_id_set" "', argument " "1"" of type '" "struct _dwg_security *""'"); 
    }
    arg1 = (struct _dwg_security *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Security_algo_id_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->algo_id = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Security_algo_id_get) {
  {
    struct _dwg_security *arg1 = (struct _dwg_security *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Security_algo_id_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_security, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Security_algo_id_get" "', argument " "1"" of type '" "struct _dwg_security *""'"); 
    }
    arg1 = (struct _dwg_security *)(argp1);
    result = (BITCODE_RL) ((arg1)->algo_id);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Security_key_len_set) {
  {
    struct _dwg_security *arg1 = (struct _dwg_security *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Security_key_len_set(self,key_len);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_security, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Security_key_len_set" "', argument " "1"" of type '" "struct _dwg_security *""'"); 
    }
    arg1 = (struct _dwg_security *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Security_key_len_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->key_len = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Security_key_len_get) {
  {
    struct _dwg_security *arg1 = (struct _dwg_security *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Security_key_len_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_security, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Security_key_len_get" "', argument " "1"" of type '" "struct _dwg_security *""'"); 
    }
    arg1 = (struct _dwg_security *)(argp1);
    result = (BITCODE_RL) ((arg1)->key_len);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Security_encr_size_set) {
  {
    struct _dwg_security *arg1 = (struct _dwg_security *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Security_encr_size_set(self,encr_size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_security, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Security_encr_size_set" "', argument " "1"" of type '" "struct _dwg_security *""'"); 
    }
    arg1 = (struct _dwg_security *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Security_encr_size_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->encr_size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Security_encr_size_get) {
  {
    struct _dwg_security *arg1 = (struct _dwg_security *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Security_encr_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_security, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Security_encr_size_get" "', argument " "1"" of type '" "struct _dwg_security *""'"); 
    }
    arg1 = (struct _dwg_security *)(argp1);
    result = (BITCODE_RL) ((arg1)->encr_size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Security_encr_buffer_set) {
  {
    struct _dwg_security *arg1 = (struct _dwg_security *) 0 ;
    BITCODE_TF arg2 = (BITCODE_TF) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Security_encr_buffer_set(self,encr_buffer);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_security, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Security_encr_buffer_set" "', argument " "1"" of type '" "struct _dwg_security *""'"); 
    }
    arg1 = (struct _dwg_security *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Security_encr_buffer_set" "', argument " "2"" of type '" "BITCODE_TF""'"); 
    }
    arg2 = (BITCODE_TF)(argp2);
    if (arg1) (arg1)->encr_buffer = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Security_encr_buffer_get) {
  {
    struct _dwg_security *arg1 = (struct _dwg_security *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TF result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Security_encr_buffer_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_security, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Security_encr_buffer_get" "', argument " "1"" of type '" "struct _dwg_security *""'"); 
    }
    arg1 = (struct _dwg_security *)(argp1);
    result = (BITCODE_TF) ((arg1)->encr_buffer);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Security) {
  {
    int argvi = 0;
    struct _dwg_security *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Security();");
    }
    result = (struct _dwg_security *)calloc(1, sizeof(struct _dwg_security));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_security, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Security) {
  {
    struct _dwg_security *arg1 = (struct _dwg_security *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Security(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_security, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Security" "', argument " "1"" of type '" "struct _dwg_security *""'"); 
    }
    arg1 = (struct _dwg_security *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_VBAProject_size_set) {
  {
    struct _dwg_vbaproject *arg1 = (struct _dwg_vbaproject *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_VBAProject_size_set(self,size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_vbaproject, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_VBAProject_size_set" "', argument " "1"" of type '" "struct _dwg_vbaproject *""'"); 
    }
    arg1 = (struct _dwg_vbaproject *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_VBAProject_size_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_VBAProject_size_get) {
  {
    struct _dwg_vbaproject *arg1 = (struct _dwg_vbaproject *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_VBAProject_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_vbaproject, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_VBAProject_size_get" "', argument " "1"" of type '" "struct _dwg_vbaproject *""'"); 
    }
    arg1 = (struct _dwg_vbaproject *)(argp1);
    result = (BITCODE_RL) ((arg1)->size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_VBAProject_unknown_bits_set) {
  {
    struct _dwg_vbaproject *arg1 = (struct _dwg_vbaproject *) 0 ;
    BITCODE_TF arg2 = (BITCODE_TF) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_VBAProject_unknown_bits_set(self,unknown_bits);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_vbaproject, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_VBAProject_unknown_bits_set" "', argument " "1"" of type '" "struct _dwg_vbaproject *""'"); 
    }
    arg1 = (struct _dwg_vbaproject *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_VBAProject_unknown_bits_set" "', argument " "2"" of type '" "BITCODE_TF""'"); 
    }
    arg2 = (BITCODE_TF)(argp2);
    if (arg1) (arg1)->unknown_bits = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_VBAProject_unknown_bits_get) {
  {
    struct _dwg_vbaproject *arg1 = (struct _dwg_vbaproject *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TF result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_VBAProject_unknown_bits_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_vbaproject, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_VBAProject_unknown_bits_get" "', argument " "1"" of type '" "struct _dwg_vbaproject *""'"); 
    }
    arg1 = (struct _dwg_vbaproject *)(argp1);
    result = (BITCODE_TF) ((arg1)->unknown_bits);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_VBAProject) {
  {
    int argvi = 0;
    struct _dwg_vbaproject *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_VBAProject();");
    }
    result = (struct _dwg_vbaproject *)calloc(1, sizeof(struct _dwg_vbaproject));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_vbaproject, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_VBAProject) {
  {
    struct _dwg_vbaproject *arg1 = (struct _dwg_vbaproject *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_VBAProject(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_vbaproject, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_VBAProject" "', argument " "1"" of type '" "struct _dwg_vbaproject *""'"); 
    }
    arg1 = (struct _dwg_vbaproject *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AppInfoHistory_size_set) {
  {
    struct _dwg_appinfohistory *arg1 = (struct _dwg_appinfohistory *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AppInfoHistory_size_set(self,size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_appinfohistory, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AppInfoHistory_size_set" "', argument " "1"" of type '" "struct _dwg_appinfohistory *""'"); 
    }
    arg1 = (struct _dwg_appinfohistory *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_AppInfoHistory_size_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AppInfoHistory_size_get) {
  {
    struct _dwg_appinfohistory *arg1 = (struct _dwg_appinfohistory *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AppInfoHistory_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_appinfohistory, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AppInfoHistory_size_get" "', argument " "1"" of type '" "struct _dwg_appinfohistory *""'"); 
    }
    arg1 = (struct _dwg_appinfohistory *)(argp1);
    result = (BITCODE_RL) ((arg1)->size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AppInfoHistory_unknown_bits_set) {
  {
    struct _dwg_appinfohistory *arg1 = (struct _dwg_appinfohistory *) 0 ;
    BITCODE_TF arg2 = (BITCODE_TF) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_AppInfoHistory_unknown_bits_set(self,unknown_bits);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_appinfohistory, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AppInfoHistory_unknown_bits_set" "', argument " "1"" of type '" "struct _dwg_appinfohistory *""'"); 
    }
    arg1 = (struct _dwg_appinfohistory *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_AppInfoHistory_unknown_bits_set" "', argument " "2"" of type '" "BITCODE_TF""'"); 
    }
    arg2 = (BITCODE_TF)(argp2);
    if (arg1) (arg1)->unknown_bits = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_AppInfoHistory_unknown_bits_get) {
  {
    struct _dwg_appinfohistory *arg1 = (struct _dwg_appinfohistory *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TF result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_AppInfoHistory_unknown_bits_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_appinfohistory, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_AppInfoHistory_unknown_bits_get" "', argument " "1"" of type '" "struct _dwg_appinfohistory *""'"); 
    }
    arg1 = (struct _dwg_appinfohistory *)(argp1);
    result = (BITCODE_TF) ((arg1)->unknown_bits);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_AppInfoHistory) {
  {
    int argvi = 0;
    struct _dwg_appinfohistory *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_AppInfoHistory();");
    }
    result = (struct _dwg_appinfohistory *)calloc(1, sizeof(struct _dwg_appinfohistory));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_appinfohistory, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_AppInfoHistory) {
  {
    struct _dwg_appinfohistory *arg1 = (struct _dwg_appinfohistory *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_AppInfoHistory(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_appinfohistory, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_AppInfoHistory" "', argument " "1"" of type '" "struct _dwg_appinfohistory *""'"); 
    }
    arg1 = (struct _dwg_appinfohistory *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_RevHistory_class_version_set) {
  {
    struct _dwg_revhistory *arg1 = (struct _dwg_revhistory *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_RevHistory_class_version_set(self,class_version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_revhistory, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_RevHistory_class_version_set" "', argument " "1"" of type '" "struct _dwg_revhistory *""'"); 
    }
    arg1 = (struct _dwg_revhistory *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_RevHistory_class_version_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->class_version = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_RevHistory_class_version_get) {
  {
    struct _dwg_revhistory *arg1 = (struct _dwg_revhistory *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_RevHistory_class_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_revhistory, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_RevHistory_class_version_get" "', argument " "1"" of type '" "struct _dwg_revhistory *""'"); 
    }
    arg1 = (struct _dwg_revhistory *)(argp1);
    result = (BITCODE_RL) ((arg1)->class_version);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_RevHistory_class_minor_set) {
  {
    struct _dwg_revhistory *arg1 = (struct _dwg_revhistory *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_RevHistory_class_minor_set(self,class_minor);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_revhistory, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_RevHistory_class_minor_set" "', argument " "1"" of type '" "struct _dwg_revhistory *""'"); 
    }
    arg1 = (struct _dwg_revhistory *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_RevHistory_class_minor_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->class_minor = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_RevHistory_class_minor_get) {
  {
    struct _dwg_revhistory *arg1 = (struct _dwg_revhistory *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_RevHistory_class_minor_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_revhistory, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_RevHistory_class_minor_get" "', argument " "1"" of type '" "struct _dwg_revhistory *""'"); 
    }
    arg1 = (struct _dwg_revhistory *)(argp1);
    result = (BITCODE_RL) ((arg1)->class_minor);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_RevHistory_num_histories_set) {
  {
    struct _dwg_revhistory *arg1 = (struct _dwg_revhistory *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_RevHistory_num_histories_set(self,num_histories);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_revhistory, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_RevHistory_num_histories_set" "', argument " "1"" of type '" "struct _dwg_revhistory *""'"); 
    }
    arg1 = (struct _dwg_revhistory *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_RevHistory_num_histories_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->num_histories = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_RevHistory_num_histories_get) {
  {
    struct _dwg_revhistory *arg1 = (struct _dwg_revhistory *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_RevHistory_num_histories_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_revhistory, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_RevHistory_num_histories_get" "', argument " "1"" of type '" "struct _dwg_revhistory *""'"); 
    }
    arg1 = (struct _dwg_revhistory *)(argp1);
    result = (BITCODE_RL) ((arg1)->num_histories);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_RevHistory_histories_set) {
  {
    struct _dwg_revhistory *arg1 = (struct _dwg_revhistory *) 0 ;
    BITCODE_RL *arg2 = (BITCODE_RL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_RevHistory_histories_set(self,histories);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_revhistory, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_RevHistory_histories_set" "', argument " "1"" of type '" "struct _dwg_revhistory *""'"); 
    }
    arg1 = (struct _dwg_revhistory *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_RevHistory_histories_set" "', argument " "2"" of type '" "BITCODE_RL *""'"); 
    }
    arg2 = (BITCODE_RL *)(argp2);
    if (arg1) (arg1)->histories = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_RevHistory_histories_get) {
  {
    struct _dwg_revhistory *arg1 = (struct _dwg_revhistory *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_RevHistory_histories_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_revhistory, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_RevHistory_histories_get" "', argument " "1"" of type '" "struct _dwg_revhistory *""'"); 
    }
    arg1 = (struct _dwg_revhistory *)(argp1);
    result = (BITCODE_RL *) ((arg1)->histories);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_int, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_RevHistory) {
  {
    int argvi = 0;
    struct _dwg_revhistory *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_RevHistory();");
    }
    result = (struct _dwg_revhistory *)calloc(1, sizeof(struct _dwg_revhistory));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_revhistory, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_RevHistory) {
  {
    struct _dwg_revhistory *arg1 = (struct _dwg_revhistory *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_RevHistory(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_revhistory, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_RevHistory" "', argument " "1"" of type '" "struct _dwg_revhistory *""'"); 
    }
    arg1 = (struct _dwg_revhistory *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ObjFreeSpace_zero_set) {
  {
    struct _dwg_objfreespace *arg1 = (struct _dwg_objfreespace *) 0 ;
    BITCODE_RLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ObjFreeSpace_zero_set(self,zero);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_objfreespace, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ObjFreeSpace_zero_set" "', argument " "1"" of type '" "struct _dwg_objfreespace *""'"); 
    }
    arg1 = (struct _dwg_objfreespace *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ObjFreeSpace_zero_set" "', argument " "2"" of type '" "BITCODE_RLL""'");
    } 
    arg2 = (BITCODE_RLL)(val2);
    if (arg1) (arg1)->zero = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ObjFreeSpace_zero_get) {
  {
    struct _dwg_objfreespace *arg1 = (struct _dwg_objfreespace *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ObjFreeSpace_zero_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_objfreespace, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ObjFreeSpace_zero_get" "', argument " "1"" of type '" "struct _dwg_objfreespace *""'"); 
    }
    arg1 = (struct _dwg_objfreespace *)(argp1);
    result = (BITCODE_RLL) ((arg1)->zero);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ObjFreeSpace_numhandles_set) {
  {
    struct _dwg_objfreespace *arg1 = (struct _dwg_objfreespace *) 0 ;
    BITCODE_RLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ObjFreeSpace_numhandles_set(self,numhandles);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_objfreespace, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ObjFreeSpace_numhandles_set" "', argument " "1"" of type '" "struct _dwg_objfreespace *""'"); 
    }
    arg1 = (struct _dwg_objfreespace *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ObjFreeSpace_numhandles_set" "', argument " "2"" of type '" "BITCODE_RLL""'");
    } 
    arg2 = (BITCODE_RLL)(val2);
    if (arg1) (arg1)->numhandles = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ObjFreeSpace_numhandles_get) {
  {
    struct _dwg_objfreespace *arg1 = (struct _dwg_objfreespace *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ObjFreeSpace_numhandles_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_objfreespace, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ObjFreeSpace_numhandles_get" "', argument " "1"" of type '" "struct _dwg_objfreespace *""'"); 
    }
    arg1 = (struct _dwg_objfreespace *)(argp1);
    result = (BITCODE_RLL) ((arg1)->numhandles);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ObjFreeSpace_TDUPDATE_set) {
  {
    struct _dwg_objfreespace *arg1 = (struct _dwg_objfreespace *) 0 ;
    BITCODE_TIMERLL *arg2 = (BITCODE_TIMERLL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ObjFreeSpace_TDUPDATE_set(self,TDUPDATE);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_objfreespace, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ObjFreeSpace_TDUPDATE_set" "', argument " "1"" of type '" "struct _dwg_objfreespace *""'"); 
    }
    arg1 = (struct _dwg_objfreespace *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_time_bll, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_ObjFreeSpace_TDUPDATE_set" "', argument " "2"" of type '" "BITCODE_TIMERLL *""'"); 
    }
    arg2 = (BITCODE_TIMERLL *)(argp2);
    if (arg1) (arg1)->TDUPDATE = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ObjFreeSpace_TDUPDATE_get) {
  {
    struct _dwg_objfreespace *arg1 = (struct _dwg_objfreespace *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TIMERLL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ObjFreeSpace_TDUPDATE_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_objfreespace, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ObjFreeSpace_TDUPDATE_get" "', argument " "1"" of type '" "struct _dwg_objfreespace *""'"); 
    }
    arg1 = (struct _dwg_objfreespace *)(argp1);
    result = (BITCODE_TIMERLL *)& ((arg1)->TDUPDATE);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_time_bll, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ObjFreeSpace_objects_address_set) {
  {
    struct _dwg_objfreespace *arg1 = (struct _dwg_objfreespace *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ObjFreeSpace_objects_address_set(self,objects_address);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_objfreespace, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ObjFreeSpace_objects_address_set" "', argument " "1"" of type '" "struct _dwg_objfreespace *""'"); 
    }
    arg1 = (struct _dwg_objfreespace *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ObjFreeSpace_objects_address_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->objects_address = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ObjFreeSpace_objects_address_get) {
  {
    struct _dwg_objfreespace *arg1 = (struct _dwg_objfreespace *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ObjFreeSpace_objects_address_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_objfreespace, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ObjFreeSpace_objects_address_get" "', argument " "1"" of type '" "struct _dwg_objfreespace *""'"); 
    }
    arg1 = (struct _dwg_objfreespace *)(argp1);
    result = (BITCODE_RL) ((arg1)->objects_address);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ObjFreeSpace_numnums_set) {
  {
    struct _dwg_objfreespace *arg1 = (struct _dwg_objfreespace *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ObjFreeSpace_numnums_set(self,numnums);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_objfreespace, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ObjFreeSpace_numnums_set" "', argument " "1"" of type '" "struct _dwg_objfreespace *""'"); 
    }
    arg1 = (struct _dwg_objfreespace *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ObjFreeSpace_numnums_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->numnums = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ObjFreeSpace_numnums_get) {
  {
    struct _dwg_objfreespace *arg1 = (struct _dwg_objfreespace *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ObjFreeSpace_numnums_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_objfreespace, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ObjFreeSpace_numnums_get" "', argument " "1"" of type '" "struct _dwg_objfreespace *""'"); 
    }
    arg1 = (struct _dwg_objfreespace *)(argp1);
    result = (BITCODE_RC) ((arg1)->numnums);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ObjFreeSpace_max32_set) {
  {
    struct _dwg_objfreespace *arg1 = (struct _dwg_objfreespace *) 0 ;
    BITCODE_RLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ObjFreeSpace_max32_set(self,max32);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_objfreespace, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ObjFreeSpace_max32_set" "', argument " "1"" of type '" "struct _dwg_objfreespace *""'"); 
    }
    arg1 = (struct _dwg_objfreespace *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ObjFreeSpace_max32_set" "', argument " "2"" of type '" "BITCODE_RLL""'");
    } 
    arg2 = (BITCODE_RLL)(val2);
    if (arg1) (arg1)->max32 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ObjFreeSpace_max32_get) {
  {
    struct _dwg_objfreespace *arg1 = (struct _dwg_objfreespace *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ObjFreeSpace_max32_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_objfreespace, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ObjFreeSpace_max32_get" "', argument " "1"" of type '" "struct _dwg_objfreespace *""'"); 
    }
    arg1 = (struct _dwg_objfreespace *)(argp1);
    result = (BITCODE_RLL) ((arg1)->max32);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ObjFreeSpace_max64_set) {
  {
    struct _dwg_objfreespace *arg1 = (struct _dwg_objfreespace *) 0 ;
    BITCODE_RLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ObjFreeSpace_max64_set(self,max64);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_objfreespace, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ObjFreeSpace_max64_set" "', argument " "1"" of type '" "struct _dwg_objfreespace *""'"); 
    }
    arg1 = (struct _dwg_objfreespace *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ObjFreeSpace_max64_set" "', argument " "2"" of type '" "BITCODE_RLL""'");
    } 
    arg2 = (BITCODE_RLL)(val2);
    if (arg1) (arg1)->max64 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ObjFreeSpace_max64_get) {
  {
    struct _dwg_objfreespace *arg1 = (struct _dwg_objfreespace *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ObjFreeSpace_max64_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_objfreespace, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ObjFreeSpace_max64_get" "', argument " "1"" of type '" "struct _dwg_objfreespace *""'"); 
    }
    arg1 = (struct _dwg_objfreespace *)(argp1);
    result = (BITCODE_RLL) ((arg1)->max64);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ObjFreeSpace_maxtbl_set) {
  {
    struct _dwg_objfreespace *arg1 = (struct _dwg_objfreespace *) 0 ;
    BITCODE_RLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ObjFreeSpace_maxtbl_set(self,maxtbl);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_objfreespace, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ObjFreeSpace_maxtbl_set" "', argument " "1"" of type '" "struct _dwg_objfreespace *""'"); 
    }
    arg1 = (struct _dwg_objfreespace *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ObjFreeSpace_maxtbl_set" "', argument " "2"" of type '" "BITCODE_RLL""'");
    } 
    arg2 = (BITCODE_RLL)(val2);
    if (arg1) (arg1)->maxtbl = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ObjFreeSpace_maxtbl_get) {
  {
    struct _dwg_objfreespace *arg1 = (struct _dwg_objfreespace *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ObjFreeSpace_maxtbl_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_objfreespace, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ObjFreeSpace_maxtbl_get" "', argument " "1"" of type '" "struct _dwg_objfreespace *""'"); 
    }
    arg1 = (struct _dwg_objfreespace *)(argp1);
    result = (BITCODE_RLL) ((arg1)->maxtbl);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ObjFreeSpace_maxrl_set) {
  {
    struct _dwg_objfreespace *arg1 = (struct _dwg_objfreespace *) 0 ;
    BITCODE_RLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ObjFreeSpace_maxrl_set(self,maxrl);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_objfreespace, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ObjFreeSpace_maxrl_set" "', argument " "1"" of type '" "struct _dwg_objfreespace *""'"); 
    }
    arg1 = (struct _dwg_objfreespace *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ObjFreeSpace_maxrl_set" "', argument " "2"" of type '" "BITCODE_RLL""'");
    } 
    arg2 = (BITCODE_RLL)(val2);
    if (arg1) (arg1)->maxrl = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ObjFreeSpace_maxrl_get) {
  {
    struct _dwg_objfreespace *arg1 = (struct _dwg_objfreespace *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ObjFreeSpace_maxrl_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_objfreespace, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ObjFreeSpace_maxrl_get" "', argument " "1"" of type '" "struct _dwg_objfreespace *""'"); 
    }
    arg1 = (struct _dwg_objfreespace *)(argp1);
    result = (BITCODE_RLL) ((arg1)->maxrl);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ObjFreeSpace_max32_hi_set) {
  {
    struct _dwg_objfreespace *arg1 = (struct _dwg_objfreespace *) 0 ;
    BITCODE_RLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ObjFreeSpace_max32_hi_set(self,max32_hi);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_objfreespace, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ObjFreeSpace_max32_hi_set" "', argument " "1"" of type '" "struct _dwg_objfreespace *""'"); 
    }
    arg1 = (struct _dwg_objfreespace *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ObjFreeSpace_max32_hi_set" "', argument " "2"" of type '" "BITCODE_RLL""'");
    } 
    arg2 = (BITCODE_RLL)(val2);
    if (arg1) (arg1)->max32_hi = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ObjFreeSpace_max32_hi_get) {
  {
    struct _dwg_objfreespace *arg1 = (struct _dwg_objfreespace *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ObjFreeSpace_max32_hi_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_objfreespace, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ObjFreeSpace_max32_hi_get" "', argument " "1"" of type '" "struct _dwg_objfreespace *""'"); 
    }
    arg1 = (struct _dwg_objfreespace *)(argp1);
    result = (BITCODE_RLL) ((arg1)->max32_hi);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ObjFreeSpace_max64_hi_set) {
  {
    struct _dwg_objfreespace *arg1 = (struct _dwg_objfreespace *) 0 ;
    BITCODE_RLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ObjFreeSpace_max64_hi_set(self,max64_hi);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_objfreespace, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ObjFreeSpace_max64_hi_set" "', argument " "1"" of type '" "struct _dwg_objfreespace *""'"); 
    }
    arg1 = (struct _dwg_objfreespace *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ObjFreeSpace_max64_hi_set" "', argument " "2"" of type '" "BITCODE_RLL""'");
    } 
    arg2 = (BITCODE_RLL)(val2);
    if (arg1) (arg1)->max64_hi = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ObjFreeSpace_max64_hi_get) {
  {
    struct _dwg_objfreespace *arg1 = (struct _dwg_objfreespace *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ObjFreeSpace_max64_hi_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_objfreespace, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ObjFreeSpace_max64_hi_get" "', argument " "1"" of type '" "struct _dwg_objfreespace *""'"); 
    }
    arg1 = (struct _dwg_objfreespace *)(argp1);
    result = (BITCODE_RLL) ((arg1)->max64_hi);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ObjFreeSpace_maxtbl_hi_set) {
  {
    struct _dwg_objfreespace *arg1 = (struct _dwg_objfreespace *) 0 ;
    BITCODE_RLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ObjFreeSpace_maxtbl_hi_set(self,maxtbl_hi);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_objfreespace, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ObjFreeSpace_maxtbl_hi_set" "', argument " "1"" of type '" "struct _dwg_objfreespace *""'"); 
    }
    arg1 = (struct _dwg_objfreespace *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ObjFreeSpace_maxtbl_hi_set" "', argument " "2"" of type '" "BITCODE_RLL""'");
    } 
    arg2 = (BITCODE_RLL)(val2);
    if (arg1) (arg1)->maxtbl_hi = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ObjFreeSpace_maxtbl_hi_get) {
  {
    struct _dwg_objfreespace *arg1 = (struct _dwg_objfreespace *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ObjFreeSpace_maxtbl_hi_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_objfreespace, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ObjFreeSpace_maxtbl_hi_get" "', argument " "1"" of type '" "struct _dwg_objfreespace *""'"); 
    }
    arg1 = (struct _dwg_objfreespace *)(argp1);
    result = (BITCODE_RLL) ((arg1)->maxtbl_hi);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ObjFreeSpace_maxrl_hi_set) {
  {
    struct _dwg_objfreespace *arg1 = (struct _dwg_objfreespace *) 0 ;
    BITCODE_RLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_ObjFreeSpace_maxrl_hi_set(self,maxrl_hi);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_objfreespace, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ObjFreeSpace_maxrl_hi_set" "', argument " "1"" of type '" "struct _dwg_objfreespace *""'"); 
    }
    arg1 = (struct _dwg_objfreespace *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_ObjFreeSpace_maxrl_hi_set" "', argument " "2"" of type '" "BITCODE_RLL""'");
    } 
    arg2 = (BITCODE_RLL)(val2);
    if (arg1) (arg1)->maxrl_hi = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_ObjFreeSpace_maxrl_hi_get) {
  {
    struct _dwg_objfreespace *arg1 = (struct _dwg_objfreespace *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_ObjFreeSpace_maxrl_hi_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_objfreespace, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_ObjFreeSpace_maxrl_hi_get" "', argument " "1"" of type '" "struct _dwg_objfreespace *""'"); 
    }
    arg1 = (struct _dwg_objfreespace *)(argp1);
    result = (BITCODE_RLL) ((arg1)->maxrl_hi);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_ObjFreeSpace) {
  {
    int argvi = 0;
    struct _dwg_objfreespace *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_ObjFreeSpace();");
    }
    result = (struct _dwg_objfreespace *)calloc(1, sizeof(struct _dwg_objfreespace));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_objfreespace, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_ObjFreeSpace) {
  {
    struct _dwg_objfreespace *arg1 = (struct _dwg_objfreespace *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_ObjFreeSpace(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_objfreespace, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_ObjFreeSpace" "', argument " "1"" of type '" "struct _dwg_objfreespace *""'"); 
    }
    arg1 = (struct _dwg_objfreespace *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Template_description_set) {
  {
    struct _dwg_template *arg1 = (struct _dwg_template *) 0 ;
    BITCODE_TV arg2 = (BITCODE_TV) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Template_description_set(self,description);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_template, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Template_description_set" "', argument " "1"" of type '" "struct _dwg_template *""'"); 
    }
    arg1 = (struct _dwg_template *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Template_description_set" "', argument " "2"" of type '" "BITCODE_TV""'");
    }
    arg2 = (BITCODE_TV)(buf2);
    free((char*)arg1->description);
    if (arg2) {
      size_t size = strlen((const char *)(arg2)) + 1;
      arg1->description = (BITCODE_TV)(char *)memcpy(malloc((size)*sizeof(char)), (const char *)(arg2), sizeof(char)*(size));
    } else {
      arg1->description = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Template_description_get) {
  {
    struct _dwg_template *arg1 = (struct _dwg_template *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_TV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Template_description_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_template, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Template_description_get" "', argument " "1"" of type '" "struct _dwg_template *""'"); 
    }
    arg1 = (struct _dwg_template *)(argp1);
    result = (BITCODE_TV) ((arg1)->description);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Template_MEASUREMENT_set) {
  {
    struct _dwg_template *arg1 = (struct _dwg_template *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Template_MEASUREMENT_set(self,MEASUREMENT);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_template, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Template_MEASUREMENT_set" "', argument " "1"" of type '" "struct _dwg_template *""'"); 
    }
    arg1 = (struct _dwg_template *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Template_MEASUREMENT_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->MEASUREMENT = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Template_MEASUREMENT_get) {
  {
    struct _dwg_template *arg1 = (struct _dwg_template *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Template_MEASUREMENT_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_template, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Template_MEASUREMENT_get" "', argument " "1"" of type '" "struct _dwg_template *""'"); 
    }
    arg1 = (struct _dwg_template *)(argp1);
    result = (BITCODE_RS) ((arg1)->MEASUREMENT);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Template) {
  {
    int argvi = 0;
    struct _dwg_template *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Template();");
    }
    result = (struct _dwg_template *)calloc(1, sizeof(struct _dwg_template));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_template, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Template) {
  {
    struct _dwg_template *arg1 = (struct _dwg_template *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Template(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_template, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Template" "', argument " "1"" of type '" "struct _dwg_template *""'"); 
    }
    arg1 = (struct _dwg_template *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_Sections_nr_set) {
  {
    struct _dwg_secondheader_sections *arg1 = (struct _dwg_secondheader_sections *) 0 ;
    BITCODE_RCd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    signed char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SecondHeader_Sections_nr_set(self,nr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader_sections, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_Sections_nr_set" "', argument " "1"" of type '" "struct _dwg_secondheader_sections *""'"); 
    }
    arg1 = (struct _dwg_secondheader_sections *)(argp1);
    ecode2 = SWIG_AsVal_signed_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SecondHeader_Sections_nr_set" "', argument " "2"" of type '" "BITCODE_RCd""'");
    } 
    arg2 = (BITCODE_RCd)(val2);
    if (arg1) (arg1)->nr = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_Sections_nr_get) {
  {
    struct _dwg_secondheader_sections *arg1 = (struct _dwg_secondheader_sections *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RCd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SecondHeader_Sections_nr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader_sections, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_Sections_nr_get" "', argument " "1"" of type '" "struct _dwg_secondheader_sections *""'"); 
    }
    arg1 = (struct _dwg_secondheader_sections *)(argp1);
    result = (BITCODE_RCd) ((arg1)->nr);
    ST(argvi) = SWIG_From_signed_SS_char  SWIG_PERL_CALL_ARGS_1((signed char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_Sections_address_set) {
  {
    struct _dwg_secondheader_sections *arg1 = (struct _dwg_secondheader_sections *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SecondHeader_Sections_address_set(self,address);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader_sections, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_Sections_address_set" "', argument " "1"" of type '" "struct _dwg_secondheader_sections *""'"); 
    }
    arg1 = (struct _dwg_secondheader_sections *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SecondHeader_Sections_address_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->address = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_Sections_address_get) {
  {
    struct _dwg_secondheader_sections *arg1 = (struct _dwg_secondheader_sections *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SecondHeader_Sections_address_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader_sections, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_Sections_address_get" "', argument " "1"" of type '" "struct _dwg_secondheader_sections *""'"); 
    }
    arg1 = (struct _dwg_secondheader_sections *)(argp1);
    result = (BITCODE_BL) ((arg1)->address);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_Sections_size_set) {
  {
    struct _dwg_secondheader_sections *arg1 = (struct _dwg_secondheader_sections *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SecondHeader_Sections_size_set(self,size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader_sections, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_Sections_size_set" "', argument " "1"" of type '" "struct _dwg_secondheader_sections *""'"); 
    }
    arg1 = (struct _dwg_secondheader_sections *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SecondHeader_Sections_size_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_Sections_size_get) {
  {
    struct _dwg_secondheader_sections *arg1 = (struct _dwg_secondheader_sections *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SecondHeader_Sections_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader_sections, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_Sections_size_get" "', argument " "1"" of type '" "struct _dwg_secondheader_sections *""'"); 
    }
    arg1 = (struct _dwg_secondheader_sections *)(argp1);
    result = (BITCODE_BL) ((arg1)->size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_SecondHeader_Sections) {
  {
    int argvi = 0;
    struct _dwg_secondheader_sections *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_SecondHeader_Sections();");
    }
    result = (struct _dwg_secondheader_sections *)calloc(1, sizeof(struct _dwg_secondheader_sections));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_secondheader_sections, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_SecondHeader_Sections) {
  {
    struct _dwg_secondheader_sections *arg1 = (struct _dwg_secondheader_sections *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_SecondHeader_Sections(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader_sections, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_SecondHeader_Sections" "', argument " "1"" of type '" "struct _dwg_secondheader_sections *""'"); 
    }
    arg1 = (struct _dwg_secondheader_sections *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_Handles_num_hdl_set) {
  {
    struct _dwg_secondheader_handles *arg1 = (struct _dwg_secondheader_handles *) 0 ;
    BITCODE_RCd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    signed char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SecondHeader_Handles_num_hdl_set(self,num_hdl);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader_handles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_Handles_num_hdl_set" "', argument " "1"" of type '" "struct _dwg_secondheader_handles *""'"); 
    }
    arg1 = (struct _dwg_secondheader_handles *)(argp1);
    ecode2 = SWIG_AsVal_signed_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SecondHeader_Handles_num_hdl_set" "', argument " "2"" of type '" "BITCODE_RCd""'");
    } 
    arg2 = (BITCODE_RCd)(val2);
    if (arg1) (arg1)->num_hdl = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_Handles_num_hdl_get) {
  {
    struct _dwg_secondheader_handles *arg1 = (struct _dwg_secondheader_handles *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RCd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SecondHeader_Handles_num_hdl_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader_handles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_Handles_num_hdl_get" "', argument " "1"" of type '" "struct _dwg_secondheader_handles *""'"); 
    }
    arg1 = (struct _dwg_secondheader_handles *)(argp1);
    result = (BITCODE_RCd) ((arg1)->num_hdl);
    ST(argvi) = SWIG_From_signed_SS_char  SWIG_PERL_CALL_ARGS_1((signed char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_Handles_nr_set) {
  {
    struct _dwg_secondheader_handles *arg1 = (struct _dwg_secondheader_handles *) 0 ;
    BITCODE_RCd arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    signed char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SecondHeader_Handles_nr_set(self,nr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader_handles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_Handles_nr_set" "', argument " "1"" of type '" "struct _dwg_secondheader_handles *""'"); 
    }
    arg1 = (struct _dwg_secondheader_handles *)(argp1);
    ecode2 = SWIG_AsVal_signed_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SecondHeader_Handles_nr_set" "', argument " "2"" of type '" "BITCODE_RCd""'");
    } 
    arg2 = (BITCODE_RCd)(val2);
    if (arg1) (arg1)->nr = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_Handles_nr_get) {
  {
    struct _dwg_secondheader_handles *arg1 = (struct _dwg_secondheader_handles *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RCd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SecondHeader_Handles_nr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader_handles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_Handles_nr_get" "', argument " "1"" of type '" "struct _dwg_secondheader_handles *""'"); 
    }
    arg1 = (struct _dwg_secondheader_handles *)(argp1);
    result = (BITCODE_RCd) ((arg1)->nr);
    ST(argvi) = SWIG_From_signed_SS_char  SWIG_PERL_CALL_ARGS_1((signed char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_Handles_hdl_set) {
  {
    struct _dwg_secondheader_handles *arg1 = (struct _dwg_secondheader_handles *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) (BITCODE_RC *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SecondHeader_Handles_hdl_set(self,hdl);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader_handles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_Handles_hdl_set" "', argument " "1"" of type '" "struct _dwg_secondheader_handles *""'"); 
    }
    arg1 = (struct _dwg_secondheader_handles *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_SecondHeader_Handles_hdl_set" "', argument " "2"" of type '" "BITCODE_RC [8]""'"); 
    } 
    arg2 = (BITCODE_RC *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)8; ++ii) *(BITCODE_RC *)&arg1->hdl[ii] = *((BITCODE_RC *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""hdl""' of type '""BITCODE_RC [8]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_Handles_hdl_get) {
  {
    struct _dwg_secondheader_handles *arg1 = (struct _dwg_secondheader_handles *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SecondHeader_Handles_hdl_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader_handles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_Handles_hdl_get" "', argument " "1"" of type '" "struct _dwg_secondheader_handles *""'"); 
    }
    arg1 = (struct _dwg_secondheader_handles *)(argp1);
    result = (BITCODE_RC *)(BITCODE_RC *) ((arg1)->hdl);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_Handles_name_set) {
  {
    struct _dwg_secondheader_handles *arg1 = (struct _dwg_secondheader_handles *) 0 ;
    char *arg2 = (char *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SecondHeader_Handles_name_set(self,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader_handles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_Handles_name_set" "', argument " "1"" of type '" "struct _dwg_secondheader_handles *""'"); 
    }
    arg1 = (struct _dwg_secondheader_handles *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_SecondHeader_Handles_name_set" "', argument " "2"" of type '" "char const *""'");
    }
    arg2 = (char *)(buf2);
    if (arg2) {
      size_t size = strlen((const char *)((const char *)(arg2))) + 1;
      arg1->name = (char const *)(char *)memcpy(malloc((size)*sizeof(char)), arg2, sizeof(char)*(size));
    } else {
      arg1->name = 0;
    }
    ST(argvi) = &PL_sv_undef;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_Handles_name_get) {
  {
    struct _dwg_secondheader_handles *arg1 = (struct _dwg_secondheader_handles *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    char *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SecondHeader_Handles_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader_handles, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_Handles_name_get" "', argument " "1"" of type '" "struct _dwg_secondheader_handles *""'"); 
    }
    arg1 = (struct _dwg_secondheader_handles *)(argp1);
    result = (char *) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_SecondHeader_Handles) {
  {
    int argvi = 0;
    struct _dwg_secondheader_handles *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_SecondHeader_Handles();");
    }
    result = (struct _dwg_secondheader_handles *)calloc(1, sizeof(struct _dwg_secondheader_handles));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_secondheader_handles, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_SecondHeader_Handles) {
  {
    struct _dwg_secondheader_handles *arg1 = (struct _dwg_secondheader_handles *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_SecondHeader_Handles(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader_handles, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_SecondHeader_Handles" "', argument " "1"" of type '" "struct _dwg_secondheader_handles *""'"); 
    }
    arg1 = (struct _dwg_secondheader_handles *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_size_set) {
  {
    struct _dwg_secondheader *arg1 = (struct _dwg_secondheader *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SecondHeader_size_set(self,size);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_size_set" "', argument " "1"" of type '" "struct _dwg_secondheader *""'"); 
    }
    arg1 = (struct _dwg_secondheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SecondHeader_size_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->size = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_size_get) {
  {
    struct _dwg_secondheader *arg1 = (struct _dwg_secondheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SecondHeader_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_size_get" "', argument " "1"" of type '" "struct _dwg_secondheader *""'"); 
    }
    arg1 = (struct _dwg_secondheader *)(argp1);
    result = (BITCODE_RL) ((arg1)->size);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_address_set) {
  {
    struct _dwg_secondheader *arg1 = (struct _dwg_secondheader *) 0 ;
    BITCODE_RL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SecondHeader_address_set(self,address);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_address_set" "', argument " "1"" of type '" "struct _dwg_secondheader *""'"); 
    }
    arg1 = (struct _dwg_secondheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SecondHeader_address_set" "', argument " "2"" of type '" "BITCODE_RL""'");
    } 
    arg2 = (BITCODE_RL)(val2);
    if (arg1) (arg1)->address = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_address_get) {
  {
    struct _dwg_secondheader *arg1 = (struct _dwg_secondheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SecondHeader_address_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_address_get" "', argument " "1"" of type '" "struct _dwg_secondheader *""'"); 
    }
    arg1 = (struct _dwg_secondheader *)(argp1);
    result = (BITCODE_RL) ((arg1)->address);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_version_set) {
  {
    struct _dwg_secondheader *arg1 = (struct _dwg_secondheader *) 0 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) (BITCODE_RC *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SecondHeader_version_set(self,version);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_version_set" "', argument " "1"" of type '" "struct _dwg_secondheader *""'"); 
    }
    arg1 = (struct _dwg_secondheader *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_SecondHeader_version_set" "', argument " "2"" of type '" "BITCODE_RC [11]""'"); 
    } 
    arg2 = (BITCODE_RC *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)11; ++ii) *(BITCODE_RC *)&arg1->version[ii] = *((BITCODE_RC *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""version""' of type '""BITCODE_RC [11]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_version_get) {
  {
    struct _dwg_secondheader *arg1 = (struct _dwg_secondheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SecondHeader_version_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_version_get" "', argument " "1"" of type '" "struct _dwg_secondheader *""'"); 
    }
    arg1 = (struct _dwg_secondheader *)(argp1);
    result = (BITCODE_RC *)(BITCODE_RC *) ((arg1)->version);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_is_maint_set) {
  {
    struct _dwg_secondheader *arg1 = (struct _dwg_secondheader *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SecondHeader_is_maint_set(self,is_maint);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_is_maint_set" "', argument " "1"" of type '" "struct _dwg_secondheader *""'"); 
    }
    arg1 = (struct _dwg_secondheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SecondHeader_is_maint_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->is_maint = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_is_maint_get) {
  {
    struct _dwg_secondheader *arg1 = (struct _dwg_secondheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SecondHeader_is_maint_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_is_maint_get" "', argument " "1"" of type '" "struct _dwg_secondheader *""'"); 
    }
    arg1 = (struct _dwg_secondheader *)(argp1);
    result = (BITCODE_RC) ((arg1)->is_maint);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_zero_one_or_three_set) {
  {
    struct _dwg_secondheader *arg1 = (struct _dwg_secondheader *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SecondHeader_zero_one_or_three_set(self,zero_one_or_three);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_zero_one_or_three_set" "', argument " "1"" of type '" "struct _dwg_secondheader *""'"); 
    }
    arg1 = (struct _dwg_secondheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SecondHeader_zero_one_or_three_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->zero_one_or_three = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_zero_one_or_three_get) {
  {
    struct _dwg_secondheader *arg1 = (struct _dwg_secondheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SecondHeader_zero_one_or_three_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_zero_one_or_three_get" "', argument " "1"" of type '" "struct _dwg_secondheader *""'"); 
    }
    arg1 = (struct _dwg_secondheader *)(argp1);
    result = (BITCODE_RC) ((arg1)->zero_one_or_three);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_dwg_versions_set) {
  {
    struct _dwg_secondheader *arg1 = (struct _dwg_secondheader *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SecondHeader_dwg_versions_set(self,dwg_versions);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_dwg_versions_set" "', argument " "1"" of type '" "struct _dwg_secondheader *""'"); 
    }
    arg1 = (struct _dwg_secondheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SecondHeader_dwg_versions_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->dwg_versions = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_dwg_versions_get) {
  {
    struct _dwg_secondheader *arg1 = (struct _dwg_secondheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SecondHeader_dwg_versions_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_dwg_versions_get" "', argument " "1"" of type '" "struct _dwg_secondheader *""'"); 
    }
    arg1 = (struct _dwg_secondheader *)(argp1);
    result = (BITCODE_BS) ((arg1)->dwg_versions);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_codepage_set) {
  {
    struct _dwg_secondheader *arg1 = (struct _dwg_secondheader *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SecondHeader_codepage_set(self,codepage);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_codepage_set" "', argument " "1"" of type '" "struct _dwg_secondheader *""'"); 
    }
    arg1 = (struct _dwg_secondheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SecondHeader_codepage_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->codepage = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_codepage_get) {
  {
    struct _dwg_secondheader *arg1 = (struct _dwg_secondheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SecondHeader_codepage_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_codepage_get" "', argument " "1"" of type '" "struct _dwg_secondheader *""'"); 
    }
    arg1 = (struct _dwg_secondheader *)(argp1);
    result = (BITCODE_RS) ((arg1)->codepage);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_num_sections_set) {
  {
    struct _dwg_secondheader *arg1 = (struct _dwg_secondheader *) 0 ;
    BITCODE_RC arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SecondHeader_num_sections_set(self,num_sections);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_num_sections_set" "', argument " "1"" of type '" "struct _dwg_secondheader *""'"); 
    }
    arg1 = (struct _dwg_secondheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SecondHeader_num_sections_set" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    if (arg1) (arg1)->num_sections = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_num_sections_get) {
  {
    struct _dwg_secondheader *arg1 = (struct _dwg_secondheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RC result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SecondHeader_num_sections_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_num_sections_get" "', argument " "1"" of type '" "struct _dwg_secondheader *""'"); 
    }
    arg1 = (struct _dwg_secondheader *)(argp1);
    result = (BITCODE_RC) ((arg1)->num_sections);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_sections_set) {
  {
    struct _dwg_secondheader *arg1 = (struct _dwg_secondheader *) 0 ;
    Dwg_SecondHeader_Sections *arg2 = (Dwg_SecondHeader_Sections *) (Dwg_SecondHeader_Sections *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SecondHeader_sections_set(self,sections);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_sections_set" "', argument " "1"" of type '" "struct _dwg_secondheader *""'"); 
    }
    arg1 = (struct _dwg_secondheader *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_secondheader_sections, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_SecondHeader_sections_set" "', argument " "2"" of type '" "Dwg_SecondHeader_Sections [7]""'"); 
    } 
    arg2 = (Dwg_SecondHeader_Sections *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)7; ++ii) *(Dwg_SecondHeader_Sections *)&arg1->sections[ii] = *((Dwg_SecondHeader_Sections *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""sections""' of type '""Dwg_SecondHeader_Sections [7]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_sections_get) {
  {
    struct _dwg_secondheader *arg1 = (struct _dwg_secondheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_SecondHeader_Sections *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SecondHeader_sections_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_sections_get" "', argument " "1"" of type '" "struct _dwg_secondheader *""'"); 
    }
    arg1 = (struct _dwg_secondheader *)(argp1);
    result = (Dwg_SecondHeader_Sections *)(Dwg_SecondHeader_Sections *) ((arg1)->sections);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_secondheader_sections, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_num_handles_set) {
  {
    struct _dwg_secondheader *arg1 = (struct _dwg_secondheader *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SecondHeader_num_handles_set(self,num_handles);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_num_handles_set" "', argument " "1"" of type '" "struct _dwg_secondheader *""'"); 
    }
    arg1 = (struct _dwg_secondheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SecondHeader_num_handles_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_handles = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_num_handles_get) {
  {
    struct _dwg_secondheader *arg1 = (struct _dwg_secondheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SecondHeader_num_handles_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_num_handles_get" "', argument " "1"" of type '" "struct _dwg_secondheader *""'"); 
    }
    arg1 = (struct _dwg_secondheader *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_handles);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_handles_set) {
  {
    struct _dwg_secondheader *arg1 = (struct _dwg_secondheader *) 0 ;
    Dwg_SecondHeader_Handles *arg2 = (Dwg_SecondHeader_Handles *) (Dwg_SecondHeader_Handles *)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SecondHeader_handles_set(self,handles);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_handles_set" "', argument " "1"" of type '" "struct _dwg_secondheader *""'"); 
    }
    arg1 = (struct _dwg_secondheader *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_secondheader_handles, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_SecondHeader_handles_set" "', argument " "2"" of type '" "Dwg_SecondHeader_Handles [14]""'"); 
    } 
    arg2 = (Dwg_SecondHeader_Handles *)(argp2);
    {
      if (arg2) {
        size_t ii = 0;
        for (; ii < (size_t)14; ++ii) *(Dwg_SecondHeader_Handles *)&arg1->handles[ii] = *((Dwg_SecondHeader_Handles *)arg2 + ii);
      } else {
        SWIG_exception_fail(SWIG_NullReferenceError, "invalid null reference " "in variable '""handles""' of type '""Dwg_SecondHeader_Handles [14]""'");
      }
    }
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_handles_get) {
  {
    struct _dwg_secondheader *arg1 = (struct _dwg_secondheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_SecondHeader_Handles *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SecondHeader_handles_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_handles_get" "', argument " "1"" of type '" "struct _dwg_secondheader *""'"); 
    }
    arg1 = (struct _dwg_secondheader *)(argp1);
    result = (Dwg_SecondHeader_Handles *)(Dwg_SecondHeader_Handles *) ((arg1)->handles);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_secondheader_handles, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_crc_set) {
  {
    struct _dwg_secondheader *arg1 = (struct _dwg_secondheader *) 0 ;
    BITCODE_RS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SecondHeader_crc_set(self,crc);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_crc_set" "', argument " "1"" of type '" "struct _dwg_secondheader *""'"); 
    }
    arg1 = (struct _dwg_secondheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SecondHeader_crc_set" "', argument " "2"" of type '" "BITCODE_RS""'");
    } 
    arg2 = (BITCODE_RS)(val2);
    if (arg1) (arg1)->crc = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_crc_get) {
  {
    struct _dwg_secondheader *arg1 = (struct _dwg_secondheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SecondHeader_crc_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_crc_get" "', argument " "1"" of type '" "struct _dwg_secondheader *""'"); 
    }
    arg1 = (struct _dwg_secondheader *)(argp1);
    result = (BITCODE_RS) ((arg1)->crc);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_junk_r14_set) {
  {
    struct _dwg_secondheader *arg1 = (struct _dwg_secondheader *) 0 ;
    BITCODE_RLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_SecondHeader_junk_r14_set(self,junk_r14);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_junk_r14_set" "', argument " "1"" of type '" "struct _dwg_secondheader *""'"); 
    }
    arg1 = (struct _dwg_secondheader *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_SecondHeader_junk_r14_set" "', argument " "2"" of type '" "BITCODE_RLL""'");
    } 
    arg2 = (BITCODE_RLL)(val2);
    if (arg1) (arg1)->junk_r14 = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_SecondHeader_junk_r14_get) {
  {
    struct _dwg_secondheader *arg1 = (struct _dwg_secondheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_SecondHeader_junk_r14_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_SecondHeader_junk_r14_get" "', argument " "1"" of type '" "struct _dwg_secondheader *""'"); 
    }
    arg1 = (struct _dwg_secondheader *)(argp1);
    result = (BITCODE_RLL) ((arg1)->junk_r14);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_SecondHeader) {
  {
    int argvi = 0;
    struct _dwg_secondheader *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_SecondHeader();");
    }
    result = (struct _dwg_secondheader *)calloc(1, sizeof(struct _dwg_secondheader));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_secondheader, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_SecondHeader) {
  {
    struct _dwg_secondheader *arg1 = (struct _dwg_secondheader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_SecondHeader(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_secondheader, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_SecondHeader" "', argument " "1"" of type '" "struct _dwg_secondheader *""'"); 
    }
    arg1 = (struct _dwg_secondheader *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_header_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    Dwg_Header *arg2 = (Dwg_Header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_header_set(self,header);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_header_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_header, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Data_header_set" "', argument " "2"" of type '" "Dwg_Header *""'"); 
    }
    arg2 = (Dwg_Header *)(argp2);
    if (arg1) (arg1)->header = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_header_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Header *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_header_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_header_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (Dwg_Header *)& ((arg1)->header);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_header, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_num_classes_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    BITCODE_BS arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned short val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_num_classes_set(self,num_classes);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_num_classes_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Data_num_classes_set" "', argument " "2"" of type '" "BITCODE_BS""'");
    } 
    arg2 = (BITCODE_BS)(val2);
    if (arg1) (arg1)->num_classes = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_num_classes_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_num_classes_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_num_classes_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (BITCODE_BS) ((arg1)->num_classes);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_dwg_class_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    Dwg_Class *arg2 = (Dwg_Class *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_dwg_class_set(self,dwg_class);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_dwg_class_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_class, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Data_dwg_class_set" "', argument " "2"" of type '" "Dwg_Class *""'"); 
    }
    arg2 = (Dwg_Class *)(argp2);
    if (arg1) (arg1)->dwg_class = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_dwg_class_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Class *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_dwg_class_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_dwg_class_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (Dwg_Class *) ((arg1)->dwg_class);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_class, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_num_objects_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_num_objects_set(self,num_objects);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_num_objects_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Data_num_objects_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_objects = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_num_objects_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_num_objects_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_num_objects_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_objects);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_num_alloced_objects_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_num_alloced_objects_set(self,num_alloced_objects);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_num_alloced_objects_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Data_num_alloced_objects_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_alloced_objects = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_num_alloced_objects_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_num_alloced_objects_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_num_alloced_objects_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_alloced_objects);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_object_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    Dwg_Object *arg2 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_object_set(self,object);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_object_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Data_object_set" "', argument " "2"" of type '" "Dwg_Object *""'"); 
    }
    arg2 = (Dwg_Object *)(argp2);
    if (arg1) (arg1)->object = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_object_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_object_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_object_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (Dwg_Object *) ((arg1)->object);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_num_entities_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_num_entities_set(self,num_entities);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_num_entities_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Data_num_entities_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_entities = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_num_entities_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_num_entities_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_num_entities_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_entities);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_num_object_refs_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_num_object_refs_set(self,num_object_refs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_num_object_refs_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Data_num_object_refs_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_object_refs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_num_object_refs_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_num_object_refs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_num_object_refs_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_object_refs);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_cur_index_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_cur_index_set(self,cur_index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_cur_index_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Data_cur_index_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->cur_index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_cur_index_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_cur_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_cur_index_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (BITCODE_BL) ((arg1)->cur_index);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_object_ref_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    Dwg_Object_Ref **arg2 = (Dwg_Object_Ref **) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_object_ref_set(self,object_ref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_object_ref_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Data_object_ref_set" "', argument " "2"" of type '" "Dwg_Object_Ref **""'"); 
    }
    arg2 = (Dwg_Object_Ref **)(argp2);
    if (arg1) (arg1)->object_ref = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_object_ref_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_Ref **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_object_ref_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_object_ref_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (Dwg_Object_Ref **) ((arg1)->object_ref);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_object_map_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    struct _inthash *arg2 = (struct _inthash *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_object_map_set(self,object_map);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_object_map_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__inthash, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Data_object_map_set" "', argument " "2"" of type '" "struct _inthash *""'"); 
    }
    arg2 = (struct _inthash *)(argp2);
    if (arg1) (arg1)->object_map = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_object_map_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct _inthash *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_object_map_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_object_map_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (struct _inthash *) ((arg1)->object_map);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__inthash, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_dirty_refs_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    int arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_dirty_refs_set(self,dirty_refs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_dirty_refs_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Data_dirty_refs_set" "', argument " "2"" of type '" "int""'");
    } 
    arg2 = (int)(val2);
    if (arg1) (arg1)->dirty_refs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_dirty_refs_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_dirty_refs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_dirty_refs_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (int) ((arg1)->dirty_refs);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_opts_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    unsigned int arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_opts_set(self,opts);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_opts_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Data_opts_set" "', argument " "2"" of type '" "unsigned int""'");
    } 
    arg2 = (unsigned int)(val2);
    if (arg1) (arg1)->opts = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_opts_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    unsigned int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_opts_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_opts_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (unsigned int) ((arg1)->opts);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_header_vars_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    Dwg_Header_Variables *arg2 = (Dwg_Header_Variables *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_header_vars_set(self,header_vars);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_header_vars_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_header_variables, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Data_header_vars_set" "', argument " "2"" of type '" "Dwg_Header_Variables *""'"); 
    }
    arg2 = (Dwg_Header_Variables *)(argp2);
    if (arg1) (arg1)->header_vars = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_header_vars_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Header_Variables *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_header_vars_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_header_vars_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (Dwg_Header_Variables *)& ((arg1)->header_vars);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_header_variables, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_thumbnail_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    Dwg_Chain *arg2 = (Dwg_Chain *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_thumbnail_set(self,thumbnail);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_thumbnail_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_chain, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Data_thumbnail_set" "', argument " "2"" of type '" "Dwg_Chain *""'"); 
    }
    arg2 = (Dwg_Chain *)(argp2);
    if (arg1) (arg1)->thumbnail = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_thumbnail_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Chain *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_thumbnail_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_thumbnail_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (Dwg_Chain *)& ((arg1)->thumbnail);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_chain, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_fhdr_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    _dwg_struct_fhdr *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_fhdr_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_fhdr_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (_dwg_struct_fhdr *)& ((arg1)->fhdr);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_struct_fhdr, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_mspace_block_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    Dwg_Object *arg2 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_mspace_block_set(self,mspace_block);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_mspace_block_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Data_mspace_block_set" "', argument " "2"" of type '" "Dwg_Object *""'"); 
    }
    arg2 = (Dwg_Object *)(argp2);
    if (arg1) (arg1)->mspace_block = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_mspace_block_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_mspace_block_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_mspace_block_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (Dwg_Object *) ((arg1)->mspace_block);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_pspace_block_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    Dwg_Object *arg2 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_pspace_block_set(self,pspace_block);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_pspace_block_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Data_pspace_block_set" "', argument " "2"" of type '" "Dwg_Object *""'"); 
    }
    arg2 = (Dwg_Object *)(argp2);
    if (arg1) (arg1)->pspace_block = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_pspace_block_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_pspace_block_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_pspace_block_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (Dwg_Object *) ((arg1)->pspace_block);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_block_control_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    Dwg_Object_BLOCK_CONTROL *arg2 = (Dwg_Object_BLOCK_CONTROL *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_block_control_set(self,block_control);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_block_control_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_BLOCK_CONTROL, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Data_block_control_set" "', argument " "2"" of type '" "Dwg_Object_BLOCK_CONTROL *""'"); 
    }
    arg2 = (Dwg_Object_BLOCK_CONTROL *)(argp2);
    if (arg1) (arg1)->block_control = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_block_control_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCK_CONTROL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_block_control_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_block_control_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (Dwg_Object_BLOCK_CONTROL *)& ((arg1)->block_control);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCK_CONTROL, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_auxheader_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    Dwg_AuxHeader *arg2 = (Dwg_AuxHeader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_auxheader_set(self,auxheader);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_auxheader_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_auxheader, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Data_auxheader_set" "', argument " "2"" of type '" "Dwg_AuxHeader *""'"); 
    }
    arg2 = (Dwg_AuxHeader *)(argp2);
    if (arg1) (arg1)->auxheader = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_auxheader_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_AuxHeader *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_auxheader_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_auxheader_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (Dwg_AuxHeader *)& ((arg1)->auxheader);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_auxheader, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_secondheader_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    Dwg_SecondHeader *arg2 = (Dwg_SecondHeader *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_secondheader_set(self,secondheader);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_secondheader_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_secondheader, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Data_secondheader_set" "', argument " "2"" of type '" "Dwg_SecondHeader *""'"); 
    }
    arg2 = (Dwg_SecondHeader *)(argp2);
    if (arg1) (arg1)->secondheader = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_secondheader_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_SecondHeader *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_secondheader_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_secondheader_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (Dwg_SecondHeader *)& ((arg1)->secondheader);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_secondheader, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_summaryinfo_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    Dwg_SummaryInfo *arg2 = (Dwg_SummaryInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_summaryinfo_set(self,summaryinfo);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_summaryinfo_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_summaryinfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Data_summaryinfo_set" "', argument " "2"" of type '" "Dwg_SummaryInfo *""'"); 
    }
    arg2 = (Dwg_SummaryInfo *)(argp2);
    if (arg1) (arg1)->summaryinfo = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_summaryinfo_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_SummaryInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_summaryinfo_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_summaryinfo_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (Dwg_SummaryInfo *)& ((arg1)->summaryinfo);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_summaryinfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_appinfo_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    Dwg_AppInfo *arg2 = (Dwg_AppInfo *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_appinfo_set(self,appinfo);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_appinfo_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_appinfo, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Data_appinfo_set" "', argument " "2"" of type '" "Dwg_AppInfo *""'"); 
    }
    arg2 = (Dwg_AppInfo *)(argp2);
    if (arg1) (arg1)->appinfo = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_appinfo_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_AppInfo *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_appinfo_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_appinfo_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (Dwg_AppInfo *)& ((arg1)->appinfo);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_appinfo, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_filedeplist_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    Dwg_FileDepList *arg2 = (Dwg_FileDepList *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_filedeplist_set(self,filedeplist);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_filedeplist_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_filedeplist, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Data_filedeplist_set" "', argument " "2"" of type '" "Dwg_FileDepList *""'"); 
    }
    arg2 = (Dwg_FileDepList *)(argp2);
    if (arg1) (arg1)->filedeplist = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_filedeplist_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_FileDepList *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_filedeplist_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_filedeplist_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (Dwg_FileDepList *)& ((arg1)->filedeplist);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_filedeplist, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_security_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    Dwg_Security *arg2 = (Dwg_Security *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_security_set(self,security);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_security_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_security, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Data_security_set" "', argument " "2"" of type '" "Dwg_Security *""'"); 
    }
    arg2 = (Dwg_Security *)(argp2);
    if (arg1) (arg1)->security = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_security_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Security *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_security_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_security_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (Dwg_Security *)& ((arg1)->security);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_security, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_vbaproject_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    Dwg_VBAProject *arg2 = (Dwg_VBAProject *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_vbaproject_set(self,vbaproject);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_vbaproject_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_vbaproject, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Data_vbaproject_set" "', argument " "2"" of type '" "Dwg_VBAProject *""'"); 
    }
    arg2 = (Dwg_VBAProject *)(argp2);
    if (arg1) (arg1)->vbaproject = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_vbaproject_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_VBAProject *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_vbaproject_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_vbaproject_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (Dwg_VBAProject *)& ((arg1)->vbaproject);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_vbaproject, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_appinfohistory_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    Dwg_AppInfoHistory *arg2 = (Dwg_AppInfoHistory *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_appinfohistory_set(self,appinfohistory);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_appinfohistory_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_appinfohistory, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Data_appinfohistory_set" "', argument " "2"" of type '" "Dwg_AppInfoHistory *""'"); 
    }
    arg2 = (Dwg_AppInfoHistory *)(argp2);
    if (arg1) (arg1)->appinfohistory = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_appinfohistory_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_AppInfoHistory *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_appinfohistory_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_appinfohistory_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (Dwg_AppInfoHistory *)& ((arg1)->appinfohistory);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_appinfohistory, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_revhistory_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    Dwg_RevHistory *arg2 = (Dwg_RevHistory *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_revhistory_set(self,revhistory);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_revhistory_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_revhistory, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Data_revhistory_set" "', argument " "2"" of type '" "Dwg_RevHistory *""'"); 
    }
    arg2 = (Dwg_RevHistory *)(argp2);
    if (arg1) (arg1)->revhistory = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_revhistory_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_RevHistory *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_revhistory_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_revhistory_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (Dwg_RevHistory *)& ((arg1)->revhistory);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_revhistory, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_objfreespace_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    Dwg_ObjFreeSpace *arg2 = (Dwg_ObjFreeSpace *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_objfreespace_set(self,objfreespace);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_objfreespace_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_objfreespace, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Data_objfreespace_set" "', argument " "2"" of type '" "Dwg_ObjFreeSpace *""'"); 
    }
    arg2 = (Dwg_ObjFreeSpace *)(argp2);
    if (arg1) (arg1)->objfreespace = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_objfreespace_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_ObjFreeSpace *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_objfreespace_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_objfreespace_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (Dwg_ObjFreeSpace *)& ((arg1)->objfreespace);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_objfreespace, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_Template_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    Dwg_Template *arg2 = (Dwg_Template *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_Template_set(self,Template);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_Template_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_template, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Data_Template_set" "', argument " "2"" of type '" "Dwg_Template *""'"); 
    }
    arg2 = (Dwg_Template *)(argp2);
    if (arg1) (arg1)->Template = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_Template_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Template *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_Template_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_Template_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (Dwg_Template *)& ((arg1)->Template);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_template, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_acds_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    Dwg_AcDs *arg2 = (Dwg_AcDs *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_acds_set(self,acds);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_acds_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_AcDs, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Data_acds_set" "', argument " "2"" of type '" "Dwg_AcDs *""'"); 
    }
    arg2 = (Dwg_AcDs *)(argp2);
    if (arg1) (arg1)->acds = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_acds_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_AcDs *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_acds_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_acds_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (Dwg_AcDs *)& ((arg1)->acds);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_AcDs, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_layout_type_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    unsigned int arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_layout_type_set(self,layout_type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_layout_type_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Data_layout_type_set" "', argument " "2"" of type '" "unsigned int""'");
    } 
    arg2 = (unsigned int)(val2);
    if (arg1) (arg1)->layout_type = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_layout_type_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    unsigned int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_layout_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_layout_type_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (unsigned int) ((arg1)->layout_type);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_num_acis_sab_hdl_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    unsigned int arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_num_acis_sab_hdl_set(self,num_acis_sab_hdl);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_num_acis_sab_hdl_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Data_num_acis_sab_hdl_set" "', argument " "2"" of type '" "unsigned int""'");
    } 
    arg2 = (unsigned int)(val2);
    if (arg1) (arg1)->num_acis_sab_hdl = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_num_acis_sab_hdl_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    unsigned int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_num_acis_sab_hdl_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_num_acis_sab_hdl_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (unsigned int) ((arg1)->num_acis_sab_hdl);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_acis_sab_hdl_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    BITCODE_H *arg2 = (BITCODE_H *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_acis_sab_hdl_set(self,acis_sab_hdl);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_acis_sab_hdl_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Data_acis_sab_hdl_set" "', argument " "2"" of type '" "BITCODE_H *""'"); 
    }
    arg2 = (BITCODE_H *)(argp2);
    if (arg1) (arg1)->acis_sab_hdl = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_acis_sab_hdl_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_H *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_acis_sab_hdl_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_acis_sab_hdl_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (BITCODE_H *) ((arg1)->acis_sab_hdl);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_next_hdl_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    BITCODE_RLL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_next_hdl_set(self,next_hdl);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_next_hdl_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Data_next_hdl_set" "', argument " "2"" of type '" "BITCODE_RLL""'");
    } 
    arg2 = (BITCODE_RLL)(val2);
    if (arg1) (arg1)->next_hdl = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_next_hdl_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_RLL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_next_hdl_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_next_hdl_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (BITCODE_RLL) ((arg1)->next_hdl);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_prev_entity_index_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_prev_entity_index_set(self,prev_entity_index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_prev_entity_index_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Data_prev_entity_index_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->prev_entity_index = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_prev_entity_index_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_prev_entity_index_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_prev_entity_index_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (BITCODE_BL) ((arg1)->prev_entity_index);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_object_ordered_ref_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    Dwg_Object_Ref **arg2 = (Dwg_Object_Ref **) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_object_ordered_ref_set(self,object_ordered_ref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_object_ordered_ref_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "Dwg_Data_object_ordered_ref_set" "', argument " "2"" of type '" "Dwg_Object_Ref **""'"); 
    }
    arg2 = (Dwg_Object_Ref **)(argp2);
    if (arg1) (arg1)->object_ordered_ref = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_object_ordered_ref_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_Ref **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_object_ordered_ref_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_object_ordered_ref_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (Dwg_Object_Ref **) ((arg1)->object_ordered_ref);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_num_object_ordered_refs_set) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_Data_num_object_ordered_refs_set(self,num_object_ordered_refs);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_num_object_ordered_refs_set" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_Data_num_object_ordered_refs_set" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    if (arg1) (arg1)->num_object_ordered_refs = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_Data_num_object_ordered_refs_get) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_Data_num_object_ordered_refs_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_Data_num_object_ordered_refs_get" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    result = (BITCODE_BL) ((arg1)->num_object_ordered_refs);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_Data) {
  {
    int argvi = 0;
    struct _dwg_struct *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_Data();");
    }
    result = (struct _dwg_struct *)calloc(1, sizeof(struct _dwg_struct));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_struct, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_Data) {
  {
    struct _dwg_struct *arg1 = (struct _dwg_struct *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_Data(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_Data" "', argument " "1"" of type '" "struct _dwg_struct *""'"); 
    }
    arg1 = (struct _dwg_struct *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_struct_fhdr_r2004_header_set) {
  {
    _dwg_struct_fhdr *arg1 = (_dwg_struct_fhdr *) 0 ;
    Dwg_R2004_Header *arg2 = (Dwg_R2004_Header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_struct_fhdr_r2004_header_set(self,r2004_header);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct_fhdr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_struct_fhdr_r2004_header_set" "', argument " "1"" of type '" "_dwg_struct_fhdr *""'"); 
    }
    arg1 = (_dwg_struct_fhdr *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_R2004_Header, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_struct_fhdr_r2004_header_set" "', argument " "2"" of type '" "Dwg_R2004_Header *""'"); 
    }
    arg2 = (Dwg_R2004_Header *)(argp2);
    if (arg1) (arg1)->r2004_header = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_struct_fhdr_r2004_header_get) {
  {
    _dwg_struct_fhdr *arg1 = (_dwg_struct_fhdr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_R2004_Header *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_struct_fhdr_r2004_header_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct_fhdr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_struct_fhdr_r2004_header_get" "', argument " "1"" of type '" "_dwg_struct_fhdr *""'"); 
    }
    arg1 = (_dwg_struct_fhdr *)(argp1);
    result = (Dwg_R2004_Header *)& ((arg1)->r2004_header);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_R2004_Header, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_struct_fhdr_r2007_file_header_set) {
  {
    _dwg_struct_fhdr *arg1 = (_dwg_struct_fhdr *) 0 ;
    Dwg_R2007_Header *arg2 = (Dwg_R2007_Header *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: _dwg_struct_fhdr_r2007_file_header_set(self,r2007_file_header);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct_fhdr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_struct_fhdr_r2007_file_header_set" "', argument " "1"" of type '" "_dwg_struct_fhdr *""'"); 
    }
    arg1 = (_dwg_struct_fhdr *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_r2007_file_header, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "_dwg_struct_fhdr_r2007_file_header_set" "', argument " "2"" of type '" "Dwg_R2007_Header *""'"); 
    }
    arg2 = (Dwg_R2007_Header *)(argp2);
    if (arg1) (arg1)->r2007_file_header = *arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap__dwg_struct_fhdr_r2007_file_header_get) {
  {
    _dwg_struct_fhdr *arg1 = (_dwg_struct_fhdr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_R2007_Header *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: _dwg_struct_fhdr_r2007_file_header_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct_fhdr, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "_dwg_struct_fhdr_r2007_file_header_get" "', argument " "1"" of type '" "_dwg_struct_fhdr *""'"); 
    }
    arg1 = (_dwg_struct_fhdr *)(argp1);
    result = (Dwg_R2007_Header *)& ((arg1)->r2007_file_header);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_r2007_file_header, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new__dwg_struct_fhdr) {
  {
    int argvi = 0;
    _dwg_struct_fhdr *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new__dwg_struct_fhdr();");
    }
    result = (_dwg_struct_fhdr *)calloc(1, sizeof(_dwg_struct_fhdr));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_struct_fhdr, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete__dwg_struct_fhdr) {
  {
    _dwg_struct_fhdr *arg1 = (_dwg_struct_fhdr *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete__dwg_struct_fhdr(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct_fhdr, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete__dwg_struct_fhdr" "', argument " "1"" of type '" "_dwg_struct_fhdr *""'"); 
    }
    arg1 = (_dwg_struct_fhdr *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_read_file) {
  {
    char *arg1 = (char *) 0 ;
    Dwg_Data *arg2 = (Dwg_Data *) 0 ;
    int res1 ;
    char *buf1 = 0 ;
    int alloc1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dwg_read_file(filename,dwg);");
    }
    res1 = SWIG_AsCharPtrAndSize(ST(0), &buf1, NULL, &alloc1);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_read_file" "', argument " "1"" of type '" "char const *""'");
    }
    arg1 = (char *)(buf1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_read_file" "', argument " "2"" of type '" "Dwg_Data *""'"); 
    }
    arg2 = (Dwg_Data *)(argp2);
    result = (int)dwg_read_file((char const *)arg1,arg2);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    if (alloc1 == SWIG_NEWOBJ) free((char*)buf1);
    
    XSRETURN(argvi);
  fail:
    if (alloc1 == SWIG_NEWOBJ) free((char*)buf1);
    
    SWIG_croak_null();
  }
}


XS(_wrap_dxf_read_file) {
  {
    char *arg1 = (char *) 0 ;
    Dwg_Data *arg2 = (Dwg_Data *) 0 ;
    int res1 ;
    char *buf1 = 0 ;
    int alloc1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dxf_read_file(filename,dwg);");
    }
    res1 = SWIG_AsCharPtrAndSize(ST(0), &buf1, NULL, &alloc1);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dxf_read_file" "', argument " "1"" of type '" "char const *""'");
    }
    arg1 = (char *)(buf1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dxf_read_file" "', argument " "2"" of type '" "Dwg_Data *""'"); 
    }
    arg2 = (Dwg_Data *)(argp2);
    result = (int)dxf_read_file((char const *)arg1,arg2);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    if (alloc1 == SWIG_NEWOBJ) free((char*)buf1);
    
    XSRETURN(argvi);
  fail:
    if (alloc1 == SWIG_NEWOBJ) free((char*)buf1);
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_write_file) {
  {
    char *arg1 = (char *) 0 ;
    Dwg_Data *arg2 = (Dwg_Data *) 0 ;
    int res1 ;
    char *buf1 = 0 ;
    int alloc1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dwg_write_file(filename,dwg);");
    }
    res1 = SWIG_AsCharPtrAndSize(ST(0), &buf1, NULL, &alloc1);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_write_file" "', argument " "1"" of type '" "char const *""'");
    }
    arg1 = (char *)(buf1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_write_file" "', argument " "2"" of type '" "Dwg_Data const *""'"); 
    }
    arg2 = (Dwg_Data *)(argp2);
    result = (int)dwg_write_file((char const *)arg1,(struct _dwg_struct const *)arg2);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    if (alloc1 == SWIG_NEWOBJ) free((char*)buf1);
    
    XSRETURN(argvi);
  fail:
    if (alloc1 == SWIG_NEWOBJ) free((char*)buf1);
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_bmp) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    BITCODE_RL *arg2 = (BITCODE_RL *) 0 ;
    BITCODE_RC *arg3 = (BITCODE_RC *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    void *argp3 = 0 ;
    int res3 = 0 ;
    int argvi = 0;
    unsigned char *result = 0 ;
    dXSARGS;
    
    if ((items < 3) || (items > 3)) {
      SWIG_croak("Usage: dwg_bmp(dwg,size,typep);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_bmp" "', argument " "1"" of type '" "Dwg_Data const *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_int, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_bmp" "', argument " "2"" of type '" "BITCODE_RL *""'"); 
    }
    arg2 = (BITCODE_RL *)(argp2);
    res3 = SWIG_ConvertPtr(ST(2), &argp3,SWIGTYPE_p_unsigned_char, 0 |  0 );
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "dwg_bmp" "', argument " "3"" of type '" "BITCODE_RC *""'"); 
    }
    arg3 = (BITCODE_RC *)(argp3);
    result = (unsigned char *)dwg_bmp((struct _dwg_struct const *)arg1,arg2,arg3);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_unsigned_char, 0 | 0); argvi++ ;
    
    
    
    XSRETURN(argvi);
  fail:
    
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dxf_cvt_lweight) {
  {
    BITCODE_BSd arg1 ;
    short val1 ;
    int ecode1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dxf_cvt_lweight(value);");
    }
    ecode1 = SWIG_AsVal_short SWIG_PERL_CALL_ARGS_2(ST(0), &val1);
    if (!SWIG_IsOK(ecode1)) {
      SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "dxf_cvt_lweight" "', argument " "1"" of type '" "BITCODE_BSd""'");
    } 
    arg1 = (BITCODE_BSd)(val1);
    result = (int)dxf_cvt_lweight(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dxf_revcvt_lweight) {
  {
    int arg1 ;
    int val1 ;
    int ecode1 = 0 ;
    int argvi = 0;
    BITCODE_BSd result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dxf_revcvt_lweight(lw);");
    }
    ecode1 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(0), &val1);
    if (!SWIG_IsOK(ecode1)) {
      SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "dxf_revcvt_lweight" "', argument " "1"" of type '" "int""'");
    } 
    arg1 = (int)(val1);
    result = (BITCODE_BSd)dxf_revcvt_lweight(arg1);
    ST(argvi) = SWIG_From_short  SWIG_PERL_CALL_ARGS_1((short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_ctrl_table) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    char *arg2 = (char *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dwg_ctrl_table(dwg,table);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_ctrl_table" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_ctrl_table" "', argument " "2"" of type '" "char const *""'");
    }
    arg2 = (char *)(buf2);
    result = (BITCODE_H)dwg_ctrl_table(arg1,(char const *)arg2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_find_tablehandle) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    char *arg2 = (char *) 0 ;
    char *arg3 = (char *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int res3 ;
    char *buf3 = 0 ;
    int alloc3 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 3) || (items > 3)) {
      SWIG_croak("Usage: dwg_find_tablehandle(dwg,name,table);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_find_tablehandle" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_find_tablehandle" "', argument " "2"" of type '" "char const *""'");
    }
    arg2 = (char *)(buf2);
    res3 = SWIG_AsCharPtrAndSize(ST(2), &buf3, NULL, &alloc3);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "dwg_find_tablehandle" "', argument " "3"" of type '" "char const *""'");
    }
    arg3 = (char *)(buf3);
    result = (BITCODE_H)dwg_find_tablehandle(arg1,(char const *)arg2,(char const *)arg3);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    if (alloc3 == SWIG_NEWOBJ) free((char*)buf3);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    if (alloc3 == SWIG_NEWOBJ) free((char*)buf3);
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_find_tablehandle_index) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    int arg2 ;
    char *arg3 = (char *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int res3 ;
    char *buf3 = 0 ;
    int alloc3 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 3) || (items > 3)) {
      SWIG_croak("Usage: dwg_find_tablehandle_index(dwg,index,table);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_find_tablehandle_index" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "dwg_find_tablehandle_index" "', argument " "2"" of type '" "int""'");
    } 
    arg2 = (int)(val2);
    res3 = SWIG_AsCharPtrAndSize(ST(2), &buf3, NULL, &alloc3);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "dwg_find_tablehandle_index" "', argument " "3"" of type '" "char const *""'");
    }
    arg3 = (char *)(buf3);
    result = (BITCODE_H)dwg_find_tablehandle_index(arg1,arg2,(char const *)arg3);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    
    if (alloc3 == SWIG_NEWOBJ) free((char*)buf3);
    XSRETURN(argvi);
  fail:
    
    
    if (alloc3 == SWIG_NEWOBJ) free((char*)buf3);
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_handle_name) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    char *arg2 = (char *) 0 ;
    BITCODE_H arg3 = (BITCODE_H) (BITCODE_H)0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    void *argp3 = 0 ;
    int res3 = 0 ;
    int argvi = 0;
    char *result = 0 ;
    dXSARGS;
    
    if ((items < 3) || (items > 3)) {
      SWIG_croak("Usage: dwg_handle_name(dwg,table,handle);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_handle_name" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_handle_name" "', argument " "2"" of type '" "char const *""'");
    }
    arg2 = (char *)(buf2);
    res3 = SWIG_ConvertPtr(ST(2), &argp3,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "dwg_handle_name" "', argument " "3"" of type '" "BITCODE_H const""'"); 
    }
    arg3 = (BITCODE_H)(argp3);
    result = (char *)dwg_handle_name(arg1,(char const *)arg2,arg3);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_find_table_control) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    char *arg2 = (char *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dwg_find_table_control(dwg,table);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_find_table_control" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_find_table_control" "', argument " "2"" of type '" "char const *""'");
    }
    arg2 = (char *)(buf2);
    result = (BITCODE_H)dwg_find_table_control(arg1,(char const *)arg2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_find_dictionary) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    char *arg2 = (char *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dwg_find_dictionary(dwg,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_find_dictionary" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_find_dictionary" "', argument " "2"" of type '" "char const *""'");
    }
    arg2 = (char *)(buf2);
    result = (BITCODE_H)dwg_find_dictionary(arg1,(char const *)arg2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_find_dicthandle) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    char *arg3 = (char *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int res3 ;
    char *buf3 = 0 ;
    int alloc3 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 3) || (items > 3)) {
      SWIG_croak("Usage: dwg_find_dicthandle(dwg,dict,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_find_dicthandle" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_find_dicthandle" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    res3 = SWIG_AsCharPtrAndSize(ST(2), &buf3, NULL, &alloc3);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "dwg_find_dicthandle" "', argument " "3"" of type '" "char const *""'");
    }
    arg3 = (char *)(buf3);
    result = (BITCODE_H)dwg_find_dicthandle(arg1,arg2,(char const *)arg3);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    
    if (alloc3 == SWIG_NEWOBJ) free((char*)buf3);
    XSRETURN(argvi);
  fail:
    
    
    if (alloc3 == SWIG_NEWOBJ) free((char*)buf3);
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_find_dicthandle_objname) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    BITCODE_H arg2 = (BITCODE_H) 0 ;
    char *arg3 = (char *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int res3 ;
    char *buf3 = 0 ;
    int alloc3 = 0 ;
    int argvi = 0;
    BITCODE_H result;
    dXSARGS;
    
    if ((items < 3) || (items > 3)) {
      SWIG_croak("Usage: dwg_find_dicthandle_objname(dwg,dict,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_find_dicthandle_objname" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_find_dicthandle_objname" "', argument " "2"" of type '" "BITCODE_H""'"); 
    }
    arg2 = (BITCODE_H)(argp2);
    res3 = SWIG_AsCharPtrAndSize(ST(2), &buf3, NULL, &alloc3);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "dwg_find_dicthandle_objname" "', argument " "3"" of type '" "char const *""'");
    }
    arg3 = (char *)(buf3);
    result = (BITCODE_H)dwg_find_dicthandle_objname(arg1,arg2,(char const *)arg3);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    
    if (alloc3 == SWIG_NEWOBJ) free((char*)buf3);
    XSRETURN(argvi);
  fail:
    
    
    if (alloc3 == SWIG_NEWOBJ) free((char*)buf3);
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_find_table_extname) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    Dwg_Object *arg2 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    char *result = 0 ;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dwg_find_table_extname(dwg,obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_find_table_extname" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_find_table_extname" "', argument " "2"" of type '" "Dwg_Object *""'"); 
    }
    arg2 = (Dwg_Object *)(argp2);
    result = (char *)dwg_find_table_extname(arg1,arg2);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_variable_dict) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    char *arg2 = (char *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    char *result = 0 ;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dwg_variable_dict(dwg,name);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_variable_dict" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_variable_dict" "', argument " "2"" of type '" "char const *""'");
    }
    arg2 = (char *)(buf2);
    result = (char *)dwg_variable_dict(arg1,(char const *)arg2);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_model_x_min) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    double result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_model_x_min(Dwg_Data const *);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_model_x_min" "', argument " "1"" of type '" "Dwg_Data const *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (double)dwg_model_x_min((struct _dwg_struct const *)arg1);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_model_x_max) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    double result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_model_x_max(Dwg_Data const *);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_model_x_max" "', argument " "1"" of type '" "Dwg_Data const *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (double)dwg_model_x_max((struct _dwg_struct const *)arg1);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_model_y_min) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    double result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_model_y_min(Dwg_Data const *);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_model_y_min" "', argument " "1"" of type '" "Dwg_Data const *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (double)dwg_model_y_min((struct _dwg_struct const *)arg1);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_model_y_max) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    double result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_model_y_max(Dwg_Data const *);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_model_y_max" "', argument " "1"" of type '" "Dwg_Data const *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (double)dwg_model_y_max((struct _dwg_struct const *)arg1);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_model_z_min) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    double result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_model_z_min(Dwg_Data const *);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_model_z_min" "', argument " "1"" of type '" "Dwg_Data const *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (double)dwg_model_z_min((struct _dwg_struct const *)arg1);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_model_z_max) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    double result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_model_z_max(Dwg_Data const *);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_model_z_max" "', argument " "1"" of type '" "Dwg_Data const *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (double)dwg_model_z_max((struct _dwg_struct const *)arg1);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_page_x_min) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    double result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_page_x_min(Dwg_Data const *);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_page_x_min" "', argument " "1"" of type '" "Dwg_Data const *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (double)dwg_page_x_min((struct _dwg_struct const *)arg1);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_page_x_max) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    double result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_page_x_max(Dwg_Data const *);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_page_x_max" "', argument " "1"" of type '" "Dwg_Data const *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (double)dwg_page_x_max((struct _dwg_struct const *)arg1);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_page_y_min) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    double result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_page_y_min(Dwg_Data const *);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_page_y_min" "', argument " "1"" of type '" "Dwg_Data const *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (double)dwg_page_y_min((struct _dwg_struct const *)arg1);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_page_y_max) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    double result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_page_y_max(Dwg_Data const *);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_page_y_max" "', argument " "1"" of type '" "Dwg_Data const *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (double)dwg_page_y_max((struct _dwg_struct const *)arg1);
    ST(argvi) = SWIG_From_double  SWIG_PERL_CALL_ARGS_1((double)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_block_control) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCK_CONTROL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_block_control(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_block_control" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCK_CONTROL *)dwg_block_control(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCK_CONTROL, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_model_space_ref) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_Ref *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_model_space_ref(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_model_space_ref" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_Ref *)dwg_model_space_ref(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_paper_space_ref) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_Ref *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_paper_space_ref(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_paper_space_ref" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_Ref *)dwg_paper_space_ref(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_model_space_object) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_model_space_object(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_model_space_object" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object *)dwg_model_space_object(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_paper_space_object) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_paper_space_object(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_paper_space_object" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object *)dwg_paper_space_object(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_get_layer_count) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    unsigned int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_get_layer_count(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_get_layer_count" "', argument " "1"" of type '" "Dwg_Data const *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (unsigned int)dwg_get_layer_count((struct _dwg_struct const *)arg1);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_get_layers) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_LAYER **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_get_layers(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_get_layers" "', argument " "1"" of type '" "Dwg_Data const *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_LAYER **)dwg_get_layers((struct _dwg_struct const *)arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_LAYER, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_get_num_objects) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_get_num_objects(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_get_num_objects" "', argument " "1"" of type '" "Dwg_Data const *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (BITCODE_BL)dwg_get_num_objects((struct _dwg_struct const *)arg1);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_get_object_num_objects) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_get_object_num_objects(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_get_object_num_objects" "', argument " "1"" of type '" "Dwg_Data const *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (BITCODE_BL)dwg_get_object_num_objects((struct _dwg_struct const *)arg1);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_class_is_entity) {
  {
    Dwg_Class *arg1 = (Dwg_Class *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_class_is_entity(klass);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_class, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_class_is_entity" "', argument " "1"" of type '" "Dwg_Class const *""'"); 
    }
    arg1 = (Dwg_Class *)(argp1);
    result = (int)dwg_class_is_entity((struct _dwg_class const *)arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_obj_is_control) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_obj_is_control(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_obj_is_control" "', argument " "1"" of type '" "Dwg_Object const *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_obj_is_control((struct _dwg_object const *)arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_obj_is_table) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_obj_is_table(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_obj_is_table" "', argument " "1"" of type '" "Dwg_Object const *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_obj_is_table((struct _dwg_object const *)arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_obj_is_subentity) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_obj_is_subentity(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_obj_is_subentity" "', argument " "1"" of type '" "Dwg_Object const *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_obj_is_subentity((struct _dwg_object const *)arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_obj_has_subentity) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_obj_has_subentity(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_obj_has_subentity" "', argument " "1"" of type '" "Dwg_Object const *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_obj_has_subentity((struct _dwg_object const *)arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_obj_is_3dsolid) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_obj_is_3dsolid(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_obj_is_3dsolid" "', argument " "1"" of type '" "Dwg_Object const *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_obj_is_3dsolid((struct _dwg_object const *)arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_obj_is_acsh) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_obj_is_acsh(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_obj_is_acsh" "', argument " "1"" of type '" "Dwg_Object const *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_obj_is_acsh((struct _dwg_object const *)arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_get_num_entities) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_get_num_entities(Dwg_Data const *);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_get_num_entities" "', argument " "1"" of type '" "Dwg_Data const *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (BITCODE_BL)dwg_get_num_entities((struct _dwg_struct const *)arg1);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_get_entities) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_Entity **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_get_entities(Dwg_Data const *);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_get_entities" "', argument " "1"" of type '" "Dwg_Data const *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_Entity **)dwg_get_entities((struct _dwg_struct const *)arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_entity, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_get_entity_layer) {
  {
    Dwg_Object_Entity *arg1 = (Dwg_Object_Entity *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_LAYER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_get_entity_layer(Dwg_Object_Entity const *);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_entity, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_get_entity_layer" "', argument " "1"" of type '" "Dwg_Object_Entity const *""'"); 
    }
    arg1 = (Dwg_Object_Entity *)(argp1);
    result = (Dwg_Object_LAYER *)dwg_get_entity_layer((struct _dwg_object_entity const *)arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_LAYER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_next_object) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_next_object(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_next_object" "', argument " "1"" of type '" "Dwg_Object const *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object *)dwg_next_object((struct _dwg_object const *)arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_next_entity) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_next_entity(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_next_entity" "', argument " "1"" of type '" "Dwg_Object const *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object *)dwg_next_entity((struct _dwg_object const *)arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_next_handle) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_HV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_next_handle(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_next_handle" "', argument " "1"" of type '" "Dwg_Data const *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (BITCODE_HV)dwg_next_handle((struct _dwg_struct const *)arg1);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_next_handseed) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    BITCODE_HV result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_next_handseed(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_next_handseed" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (BITCODE_HV)dwg_next_handseed(arg1);
    ST(argvi) = SWIG_From_unsigned_SS_long_SS_long  SWIG_PERL_CALL_ARGS_1((unsigned long long)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_ref_object) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    Dwg_Object_Ref *arg2 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    Dwg_Object *result = 0 ;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dwg_ref_object(dwg,ref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_ref_object" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_ref_object" "', argument " "2"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg2 = (Dwg_Object_Ref *)(argp2);
    result = (Dwg_Object *)dwg_ref_object(arg1,arg2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object, 0 | SWIG_SHADOW); argvi++ ;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_ref_object_relative) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    Dwg_Object_Ref *arg2 = (Dwg_Object_Ref *) 0 ;
    Dwg_Object *arg3 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    void *argp3 = 0 ;
    int res3 = 0 ;
    int argvi = 0;
    Dwg_Object *result = 0 ;
    dXSARGS;
    
    if ((items < 3) || (items > 3)) {
      SWIG_croak("Usage: dwg_ref_object_relative(dwg,ref,obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_ref_object_relative" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_ref_object_relative" "', argument " "2"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg2 = (Dwg_Object_Ref *)(argp2);
    res3 = SWIG_ConvertPtr(ST(2), &argp3,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "dwg_ref_object_relative" "', argument " "3"" of type '" "Dwg_Object const *""'"); 
    }
    arg3 = (Dwg_Object *)(argp3);
    result = (Dwg_Object *)dwg_ref_object_relative(arg1,arg2,(struct _dwg_object const *)arg3);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object, 0 | SWIG_SHADOW); argvi++ ;
    
    
    
    XSRETURN(argvi);
  fail:
    
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_ref_object_silent) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    Dwg_Object_Ref *arg2 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    Dwg_Object *result = 0 ;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dwg_ref_object_silent(dwg,ref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_ref_object_silent" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_ref_object_silent" "', argument " "2"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg2 = (Dwg_Object_Ref *)(argp2);
    result = (Dwg_Object *)dwg_ref_object_silent(arg1,arg2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object, 0 | SWIG_SHADOW); argvi++ ;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_get_first_owned_entity) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: get_first_owned_entity(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "get_first_owned_entity" "', argument " "1"" of type '" "Dwg_Object const *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object *)get_first_owned_entity((struct _dwg_object const *)arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_get_next_owned_entity) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    Dwg_Object *arg2 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    Dwg_Object *result = 0 ;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: get_next_owned_entity(hdr,current);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "get_next_owned_entity" "', argument " "1"" of type '" "Dwg_Object const *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "get_next_owned_entity" "', argument " "2"" of type '" "Dwg_Object const *""'"); 
    }
    arg2 = (Dwg_Object *)(argp2);
    result = (Dwg_Object *)get_next_owned_entity((struct _dwg_object const *)arg1,(struct _dwg_object const *)arg2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object, 0 | SWIG_SHADOW); argvi++ ;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_get_first_owned_subentity) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: get_first_owned_subentity(owner);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "get_first_owned_subentity" "', argument " "1"" of type '" "Dwg_Object const *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object *)get_first_owned_subentity((struct _dwg_object const *)arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_get_next_owned_subentity) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    Dwg_Object *arg2 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    Dwg_Object *result = 0 ;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: get_next_owned_subentity(owner,current);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "get_next_owned_subentity" "', argument " "1"" of type '" "Dwg_Object const *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "get_next_owned_subentity" "', argument " "2"" of type '" "Dwg_Object const *""'"); 
    }
    arg2 = (Dwg_Object *)(argp2);
    result = (Dwg_Object *)get_next_owned_subentity((struct _dwg_object const *)arg1,(struct _dwg_object const *)arg2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object, 0 | SWIG_SHADOW); argvi++ ;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_get_first_owned_block) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: get_first_owned_block(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "get_first_owned_block" "', argument " "1"" of type '" "Dwg_Object const *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object *)get_first_owned_block((struct _dwg_object const *)arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_get_last_owned_block) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: get_last_owned_block(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "get_last_owned_block" "', argument " "1"" of type '" "Dwg_Object const *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object *)get_last_owned_block((struct _dwg_object const *)arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_get_next_owned_block) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    Dwg_Object *arg2 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    Dwg_Object *result = 0 ;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: get_next_owned_block(hdr,current);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "get_next_owned_block" "', argument " "1"" of type '" "Dwg_Object const *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "get_next_owned_block" "', argument " "2"" of type '" "Dwg_Object const *""'"); 
    }
    arg2 = (Dwg_Object *)(argp2);
    result = (Dwg_Object *)get_next_owned_block((struct _dwg_object const *)arg1,(struct _dwg_object const *)arg2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object, 0 | SWIG_SHADOW); argvi++ ;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_get_next_owned_block_entity) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    Dwg_Object *arg2 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    Dwg_Object *result = 0 ;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: get_next_owned_block_entity(hdr,current);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "get_next_owned_block_entity" "', argument " "1"" of type '" "Dwg_Object const *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "get_next_owned_block_entity" "', argument " "2"" of type '" "Dwg_Object const *""'"); 
    }
    arg2 = (Dwg_Object *)(argp2);
    result = (Dwg_Object *)get_next_owned_block_entity((struct _dwg_object const *)arg1,(struct _dwg_object const *)arg2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object, 0 | SWIG_SHADOW); argvi++ ;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_get_first_object) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    Dwg_Object_Type arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    Dwg_Object *result = 0 ;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dwg_get_first_object(dwg,type);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_get_first_object" "', argument " "1"" of type '" "Dwg_Data const *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "dwg_get_first_object" "', argument " "2"" of type '" "Dwg_Object_Type""'");
    } 
    arg2 = (Dwg_Object_Type)(val2);
    result = (Dwg_Object *)dwg_get_first_object((struct _dwg_struct const *)arg1,arg2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object, 0 | SWIG_SHADOW); argvi++ ;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_get_next_object) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    Dwg_Object_Type arg2 ;
    BITCODE_RL arg3 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int val2 ;
    int ecode2 = 0 ;
    unsigned int val3 ;
    int ecode3 = 0 ;
    int argvi = 0;
    Dwg_Object *result = 0 ;
    dXSARGS;
    
    if ((items < 3) || (items > 3)) {
      SWIG_croak("Usage: dwg_get_next_object(dwg,type,index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_get_next_object" "', argument " "1"" of type '" "Dwg_Data const *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    ecode2 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "dwg_get_next_object" "', argument " "2"" of type '" "Dwg_Object_Type""'");
    } 
    arg2 = (Dwg_Object_Type)(val2);
    ecode3 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(2), &val3);
    if (!SWIG_IsOK(ecode3)) {
      SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "dwg_get_next_object" "', argument " "3"" of type '" "BITCODE_RL""'");
    } 
    arg3 = (BITCODE_RL)(val3);
    result = (Dwg_Object *)dwg_get_next_object((struct _dwg_struct const *)arg1,arg2,arg3);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object, 0 | SWIG_SHADOW); argvi++ ;
    
    
    
    XSRETURN(argvi);
  fail:
    
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_resolve_handle) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    BITCODE_HV arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    Dwg_Object *result = 0 ;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dwg_resolve_handle(dwg,absref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_resolve_handle" "', argument " "1"" of type '" "Dwg_Data const *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "dwg_resolve_handle" "', argument " "2"" of type '" "BITCODE_HV""'");
    } 
    arg2 = (BITCODE_HV)(val2);
    result = (Dwg_Object *)dwg_resolve_handle((struct _dwg_struct const *)arg1,arg2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object, 0 | SWIG_SHADOW); argvi++ ;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_resolve_handle_silent) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    BITCODE_HV arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    Dwg_Object *result = 0 ;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dwg_resolve_handle_silent(dwg,absref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_resolve_handle_silent" "', argument " "1"" of type '" "Dwg_Data const *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "dwg_resolve_handle_silent" "', argument " "2"" of type '" "BITCODE_HV""'");
    } 
    arg2 = (BITCODE_HV)(val2);
    result = (Dwg_Object *)dwg_resolve_handle_silent((struct _dwg_struct const *)arg1,arg2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object, 0 | SWIG_SHADOW); argvi++ ;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_resolve_handleref) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    Dwg_Object *arg2 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dwg_resolve_handleref(ref,obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_resolve_handleref" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_resolve_handleref" "', argument " "2"" of type '" "Dwg_Object const *""'"); 
    }
    arg2 = (Dwg_Object *)(argp2);
    result = (int)dwg_resolve_handleref(arg1,(struct _dwg_object const *)arg2);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_resolve_jump) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_resolve_jump(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_resolve_jump" "', argument " "1"" of type '" "Dwg_Object const *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object *)dwg_resolve_jump((struct _dwg_object const *)arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_section_type) {
  {
    char *arg1 = (char *) 0 ;
    int res1 ;
    char *buf1 = 0 ;
    int alloc1 = 0 ;
    int argvi = 0;
    Dwg_Section_Type result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_section_type(name);");
    }
    res1 = SWIG_AsCharPtrAndSize(ST(0), &buf1, NULL, &alloc1);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_section_type" "', argument " "1"" of type '" "char const *""'");
    }
    arg1 = (char *)(buf1);
    result = (Dwg_Section_Type)dwg_section_type((char const *)arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    if (alloc1 == SWIG_NEWOBJ) free((char*)buf1);
    XSRETURN(argvi);
  fail:
    if (alloc1 == SWIG_NEWOBJ) free((char*)buf1);
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_section_wtype) {
  {
    dwg_wchar_t *arg1 = (dwg_wchar_t *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Section_Type result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_section_wtype(wname);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_unsigned_short, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_section_wtype" "', argument " "1"" of type '" "dwg_wchar_t const *""'"); 
    }
    arg1 = (dwg_wchar_t *)(argp1);
    result = (Dwg_Section_Type)dwg_section_wtype((unsigned short const *)arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_section_name) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    unsigned int arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    char *result = 0 ;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dwg_section_name(dwg,sec_id);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_section_name" "', argument " "1"" of type '" "Dwg_Data const *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "dwg_section_name" "', argument " "2"" of type '" "unsigned int""'");
    } 
    arg2 = (unsigned int)(val2);
    result = (char *)dwg_section_name((struct _dwg_struct const *)arg1,arg2);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_resbuf_value_type) {
  {
    short arg1 ;
    short val1 ;
    int ecode1 = 0 ;
    int argvi = 0;
    enum RESBUF_VALUE_TYPE result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_resbuf_value_type(gc);");
    }
    ecode1 = SWIG_AsVal_short SWIG_PERL_CALL_ARGS_2(ST(0), &val1);
    if (!SWIG_IsOK(ecode1)) {
      SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "dwg_resbuf_value_type" "', argument " "1"" of type '" "short""'");
    } 
    arg1 = (short)(val1);
    result = (enum RESBUF_VALUE_TYPE)dwg_resbuf_value_type(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_free) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_free(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_free" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    dwg_free(arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_free_object) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_free_object(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_free_object" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    dwg_free_object(arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_new_ref) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_Ref *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_new_ref(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_new_ref" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_Ref *)dwg_new_ref(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_add_handle) {
  {
    Dwg_Handle *arg1 = (Dwg_Handle *) 0 ;
    BITCODE_RC arg2 ;
    BITCODE_HV arg3 ;
    Dwg_Object *arg4 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    unsigned long long val3 ;
    int ecode3 = 0 ;
    void *argp4 = 0 ;
    int res4 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 4) || (items > 4)) {
      SWIG_croak("Usage: dwg_add_handle(hdl,code,value,obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_handle, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_add_handle" "', argument " "1"" of type '" "Dwg_Handle *""'"); 
    }
    arg1 = (Dwg_Handle *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "dwg_add_handle" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    ecode3 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(2), &val3);
    if (!SWIG_IsOK(ecode3)) {
      SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "dwg_add_handle" "', argument " "3"" of type '" "BITCODE_HV""'");
    } 
    arg3 = (BITCODE_HV)(val3);
    res4 = SWIG_ConvertPtr(ST(3), &argp4,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "dwg_add_handle" "', argument " "4"" of type '" "Dwg_Object const *""'"); 
    }
    arg4 = (Dwg_Object *)(argp4);
    result = (int)dwg_add_handle(arg1,arg2,arg3,(struct _dwg_object const *)arg4);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    
    
    
    XSRETURN(argvi);
  fail:
    
    
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_add_handleref) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    BITCODE_RC arg2 ;
    BITCODE_HV arg3 ;
    Dwg_Object *arg4 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    unsigned long long val3 ;
    int ecode3 = 0 ;
    void *argp4 = 0 ;
    int res4 = 0 ;
    int argvi = 0;
    Dwg_Object_Ref *result = 0 ;
    dXSARGS;
    
    if ((items < 4) || (items > 4)) {
      SWIG_croak("Usage: dwg_add_handleref(dwg,code,value,obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_add_handleref" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "dwg_add_handleref" "', argument " "2"" of type '" "BITCODE_RC""'");
    } 
    arg2 = (BITCODE_RC)(val2);
    ecode3 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(2), &val3);
    if (!SWIG_IsOK(ecode3)) {
      SWIG_exception_fail(SWIG_ArgError(ecode3), "in method '" "dwg_add_handleref" "', argument " "3"" of type '" "BITCODE_HV""'");
    } 
    arg3 = (BITCODE_HV)(val3);
    res4 = SWIG_ConvertPtr(ST(3), &argp4,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "dwg_add_handleref" "', argument " "4"" of type '" "Dwg_Object const *""'"); 
    }
    arg4 = (Dwg_Object *)(argp4);
    result = (Dwg_Object_Ref *)dwg_add_handleref(arg1,arg2,arg3,(struct _dwg_object const *)arg4);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    
    
    
    XSRETURN(argvi);
  fail:
    
    
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_dup_handleref) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    Dwg_Object_Ref *arg2 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    Dwg_Object_Ref *result = 0 ;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dwg_dup_handleref(dwg,ref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_dup_handleref" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_dup_handleref" "', argument " "2"" of type '" "Dwg_Object_Ref const *""'"); 
    }
    arg2 = (Dwg_Object_Ref *)(argp2);
    result = (Dwg_Object_Ref *)dwg_dup_handleref(arg1,(struct _dwg_object_ref const *)arg2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_add_handleref_free) {
  {
    BITCODE_RC arg1 ;
    BITCODE_HV arg2 ;
    unsigned char val1 ;
    int ecode1 = 0 ;
    unsigned long long val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    Dwg_Object_Ref *result = 0 ;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dwg_add_handleref_free(code,absref);");
    }
    ecode1 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(0), &val1);
    if (!SWIG_IsOK(ecode1)) {
      SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "dwg_add_handleref_free" "', argument " "1"" of type '" "BITCODE_RC""'");
    } 
    arg1 = (BITCODE_RC)(val1);
    ecode2 = SWIG_AsVal_unsigned_SS_long_SS_long SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "dwg_add_handleref_free" "', argument " "2"" of type '" "BITCODE_HV""'");
    } 
    arg2 = (BITCODE_HV)(val2);
    result = (Dwg_Object_Ref *)dwg_add_handleref_free(arg1,arg2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ref, 0 | SWIG_SHADOW); argvi++ ;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_version_type) {
  {
    Dwg_Version_Type arg1 ;
    int val1 ;
    int ecode1 = 0 ;
    int argvi = 0;
    char *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_version_type(version);");
    }
    ecode1 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(0), &val1);
    if (!SWIG_IsOK(ecode1)) {
      SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "dwg_version_type" "', argument " "1"" of type '" "Dwg_Version_Type""'");
    } 
    arg1 = (Dwg_Version_Type)(val1);
    result = (char *)dwg_version_type(arg1);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_version_as) {
  {
    char *arg1 = (char *) 0 ;
    int res1 ;
    char *buf1 = 0 ;
    int alloc1 = 0 ;
    int argvi = 0;
    Dwg_Version_Type result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_version_as(version);");
    }
    res1 = SWIG_AsCharPtrAndSize(ST(0), &buf1, NULL, &alloc1);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_version_as" "', argument " "1"" of type '" "char const *""'");
    }
    arg1 = (char *)(buf1);
    result = (Dwg_Version_Type)dwg_version_as((char const *)arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    if (alloc1 == SWIG_NEWOBJ) free((char*)buf1);
    XSRETURN(argvi);
  fail:
    if (alloc1 == SWIG_NEWOBJ) free((char*)buf1);
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_version_hdr_type) {
  {
    char *arg1 = (char *) 0 ;
    int res1 ;
    char *buf1 = 0 ;
    int alloc1 = 0 ;
    int argvi = 0;
    Dwg_Version_Type result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_version_hdr_type(hdr);");
    }
    res1 = SWIG_AsCharPtrAndSize(ST(0), &buf1, NULL, &alloc1);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_version_hdr_type" "', argument " "1"" of type '" "char const *""'");
    }
    arg1 = (char *)(buf1);
    result = (Dwg_Version_Type)dwg_version_hdr_type((char const *)arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    if (alloc1 == SWIG_NEWOBJ) free((char*)buf1);
    XSRETURN(argvi);
  fail:
    if (alloc1 == SWIG_NEWOBJ) free((char*)buf1);
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_supports_eed) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_supports_eed(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_supports_eed" "', argument " "1"" of type '" "Dwg_Data const *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (int)dwg_supports_eed((struct _dwg_struct const *)arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_supports_obj) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    Dwg_Object *arg2 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dwg_supports_obj(dwg,obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_supports_obj" "', argument " "1"" of type '" "Dwg_Data const *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_supports_obj" "', argument " "2"" of type '" "Dwg_Object const *""'"); 
    }
    arg2 = (Dwg_Object *)(argp2);
    result = (int)dwg_supports_obj((struct _dwg_struct const *)arg1,(struct _dwg_object const *)arg2);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_errstrings) {
  {
    int arg1 ;
    int val1 ;
    int ecode1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_errstrings(error);");
    }
    ecode1 = SWIG_AsVal_int SWIG_PERL_CALL_ARGS_2(ST(0), &val1);
    if (!SWIG_IsOK(ecode1)) {
      SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "dwg_errstrings" "', argument " "1"" of type '" "int""'");
    } 
    arg1 = (int)(val1);
    dwg_errstrings(arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_encrypt_SAT1) {
  {
    BITCODE_BL arg1 ;
    BITCODE_RC *arg2 = (BITCODE_RC *) 0 ;
    int *arg3 = (int *) 0 ;
    unsigned int val1 ;
    int ecode1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    void *argp3 = 0 ;
    int res3 = 0 ;
    int argvi = 0;
    char *result = 0 ;
    dXSARGS;
    
    if ((items < 3) || (items > 3)) {
      SWIG_croak("Usage: dwg_encrypt_SAT1(blocksize,acis_data,acis_data_offset);");
    }
    ecode1 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(0), &val1);
    if (!SWIG_IsOK(ecode1)) {
      SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "dwg_encrypt_SAT1" "', argument " "1"" of type '" "BITCODE_BL""'");
    } 
    arg1 = (BITCODE_BL)(val1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_unsigned_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_encrypt_SAT1" "', argument " "2"" of type '" "BITCODE_RC *""'"); 
    }
    arg2 = (BITCODE_RC *)(argp2);
    res3 = SWIG_ConvertPtr(ST(2), &argp3,SWIGTYPE_p_int, 0 |  0 );
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "dwg_encrypt_SAT1" "', argument " "3"" of type '" "int *""'"); 
    }
    arg3 = (int *)(argp3);
    result = (char *)dwg_encrypt_SAT1(arg1,arg2,arg3);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    
    
    XSRETURN(argvi);
  fail:
    
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_convert_SAB_to_SAT1) {
  {
    Dwg_Entity__3DSOLID *arg1 = (Dwg_Entity__3DSOLID *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_convert_SAB_to_SAT1(_obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_entity_3DSOLID, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_convert_SAB_to_SAT1" "', argument " "1"" of type '" "Dwg_Entity__3DSOLID *""'"); 
    }
    arg1 = (Dwg_Entity__3DSOLID *)(argp1);
    result = (int)dwg_convert_SAB_to_SAT1(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_RGB_Palette_r_set) {
  {
    struct rgbpalette *arg1 = (struct rgbpalette *) 0 ;
    unsigned char arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_RGB_Palette_r_set(self,r);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_rgbpalette, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_RGB_Palette_r_set" "', argument " "1"" of type '" "struct rgbpalette *""'"); 
    }
    arg1 = (struct rgbpalette *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_RGB_Palette_r_set" "', argument " "2"" of type '" "unsigned char""'");
    } 
    arg2 = (unsigned char)(val2);
    if (arg1) (arg1)->r = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_RGB_Palette_r_get) {
  {
    struct rgbpalette *arg1 = (struct rgbpalette *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    unsigned char result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_RGB_Palette_r_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_rgbpalette, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_RGB_Palette_r_get" "', argument " "1"" of type '" "struct rgbpalette *""'"); 
    }
    arg1 = (struct rgbpalette *)(argp1);
    result = (unsigned char) ((arg1)->r);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_RGB_Palette_g_set) {
  {
    struct rgbpalette *arg1 = (struct rgbpalette *) 0 ;
    unsigned char arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_RGB_Palette_g_set(self,g);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_rgbpalette, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_RGB_Palette_g_set" "', argument " "1"" of type '" "struct rgbpalette *""'"); 
    }
    arg1 = (struct rgbpalette *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_RGB_Palette_g_set" "', argument " "2"" of type '" "unsigned char""'");
    } 
    arg2 = (unsigned char)(val2);
    if (arg1) (arg1)->g = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_RGB_Palette_g_get) {
  {
    struct rgbpalette *arg1 = (struct rgbpalette *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    unsigned char result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_RGB_Palette_g_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_rgbpalette, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_RGB_Palette_g_get" "', argument " "1"" of type '" "struct rgbpalette *""'"); 
    }
    arg1 = (struct rgbpalette *)(argp1);
    result = (unsigned char) ((arg1)->g);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_RGB_Palette_b_set) {
  {
    struct rgbpalette *arg1 = (struct rgbpalette *) 0 ;
    unsigned char arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned char val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: Dwg_RGB_Palette_b_set(self,b);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_rgbpalette, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_RGB_Palette_b_set" "', argument " "1"" of type '" "struct rgbpalette *""'"); 
    }
    arg1 = (struct rgbpalette *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_char SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "Dwg_RGB_Palette_b_set" "', argument " "2"" of type '" "unsigned char""'");
    } 
    arg2 = (unsigned char)(val2);
    if (arg1) (arg1)->b = arg2;
    ST(argvi) = &PL_sv_undef;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_RGB_Palette_b_get) {
  {
    struct rgbpalette *arg1 = (struct rgbpalette *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    unsigned char result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_RGB_Palette_b_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_rgbpalette, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_RGB_Palette_b_get" "', argument " "1"" of type '" "struct rgbpalette *""'"); 
    }
    arg1 = (struct rgbpalette *)(argp1);
    result = (unsigned char) ((arg1)->b);
    ST(argvi) = SWIG_From_unsigned_SS_char  SWIG_PERL_CALL_ARGS_1((unsigned char)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_RGB_Palette) {
  {
    int argvi = 0;
    struct rgbpalette *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_RGB_Palette();");
    }
    result = (struct rgbpalette *)calloc(1, sizeof(struct rgbpalette));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_rgbpalette, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_RGB_Palette) {
  {
    struct rgbpalette *arg1 = (struct rgbpalette *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_RGB_Palette(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_rgbpalette, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_RGB_Palette" "', argument " "1"" of type '" "struct rgbpalette *""'"); 
    }
    arg1 = (struct rgbpalette *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_rgb_palette) {
  {
    int argvi = 0;
    Dwg_RGB_Palette *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: dwg_rgb_palette();");
    }
    result = (Dwg_RGB_Palette *)dwg_rgb_palette();
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_rgbpalette, 0 | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_rgb_palette_index) {
  {
    BITCODE_BS arg1 ;
    unsigned short val1 ;
    int ecode1 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_rgb_palette_index(index);");
    }
    ecode1 = SWIG_AsVal_unsigned_SS_short SWIG_PERL_CALL_ARGS_2(ST(0), &val1);
    if (!SWIG_IsOK(ecode1)) {
      SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "dwg_rgb_palette_index" "', argument " "1"" of type '" "BITCODE_BS""'");
    } 
    arg1 = (BITCODE_BS)(val1);
    result = (BITCODE_BL)dwg_rgb_palette_index(arg1);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_find_color_index) {
  {
    BITCODE_BL arg1 ;
    unsigned int val1 ;
    int ecode1 = 0 ;
    int argvi = 0;
    BITCODE_BS result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_find_color_index(rgb);");
    }
    ecode1 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(0), &val1);
    if (!SWIG_IsOK(ecode1)) {
      SWIG_exception_fail(SWIG_ArgError(ecode1), "in method '" "dwg_find_color_index" "', argument " "1"" of type '" "BITCODE_BL""'");
    } 
    arg1 = (BITCODE_BL)(val1);
    result = (BITCODE_BS)dwg_find_color_index(arg1);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_add_object) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_add_object(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_add_object" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (int)dwg_add_object(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_name) {
  {
    char *arg1 = (char *) (char *)0 ;
    char **arg2 = (char **) 0 ;
    Dwg_Object_Type *arg3 = (Dwg_Object_Type *) 0 ;
    int *arg4 = (int *) 0 ;
    Dwg_Class_Stability *arg5 = (Dwg_Class_Stability *) 0 ;
    int res1 ;
    char *buf1 = 0 ;
    int alloc1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    void *argp3 = 0 ;
    int res3 = 0 ;
    void *argp4 = 0 ;
    int res4 = 0 ;
    void *argp5 = 0 ;
    int res5 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 5) || (items > 5)) {
      SWIG_croak("Usage: dwg_object_name(name,dxfnamep,typep,is_entp,stabilityp);");
    }
    res1 = SWIG_AsCharPtrAndSize(ST(0), &buf1, NULL, &alloc1);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_name" "', argument " "1"" of type '" "char const *const""'");
    }
    arg1 = (char *)(buf1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_p_char, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_object_name" "', argument " "2"" of type '" "char const **""'"); 
    }
    arg2 = (char **)(argp2);
    res3 = SWIG_ConvertPtr(ST(2), &argp3,SWIGTYPE_p_DWG_OBJECT_TYPE, 0 |  0 );
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "dwg_object_name" "', argument " "3"" of type '" "Dwg_Object_Type *""'"); 
    }
    arg3 = (Dwg_Object_Type *)(argp3);
    res4 = SWIG_ConvertPtr(ST(3), &argp4,SWIGTYPE_p_int, 0 |  0 );
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "dwg_object_name" "', argument " "4"" of type '" "int *""'"); 
    }
    arg4 = (int *)(argp4);
    res5 = SWIG_ConvertPtr(ST(4), &argp5,SWIGTYPE_p_DWG_CLASS_STABILITY, 0 |  0 );
    if (!SWIG_IsOK(res5)) {
      SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "dwg_object_name" "', argument " "5"" of type '" "Dwg_Class_Stability *""'"); 
    }
    arg5 = (Dwg_Class_Stability *)(argp5);
    result = (int)dwg_object_name((char const *)arg1,(char const **)arg2,arg3,arg4,arg5);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    if (alloc1 == SWIG_NEWOBJ) free((char*)buf1);
    
    
    
    
    XSRETURN(argvi);
  fail:
    if (alloc1 == SWIG_NEWOBJ) free((char*)buf1);
    
    
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup__3DFACE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup__3DFACE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup__3DFACE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup__3DFACE(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup__3DSOLID) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup__3DSOLID(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup__3DSOLID" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup__3DSOLID(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ARC) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ARC(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ARC" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ARC(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ATTDEF) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ATTDEF(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ATTDEF" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ATTDEF(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ATTRIB) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ATTRIB(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ATTRIB" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ATTRIB(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCK) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCK(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCK" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCK(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BODY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BODY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BODY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BODY(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_CIRCLE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_CIRCLE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_CIRCLE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_CIRCLE(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_DIMENSION_ALIGNED) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_DIMENSION_ALIGNED(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_DIMENSION_ALIGNED" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_DIMENSION_ALIGNED(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_DIMENSION_ANG2LN) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_DIMENSION_ANG2LN(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_DIMENSION_ANG2LN" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_DIMENSION_ANG2LN(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_DIMENSION_ANG3PT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_DIMENSION_ANG3PT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_DIMENSION_ANG3PT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_DIMENSION_ANG3PT(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_DIMENSION_DIAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_DIMENSION_DIAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_DIMENSION_DIAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_DIMENSION_DIAMETER(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_DIMENSION_LINEAR) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_DIMENSION_LINEAR(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_DIMENSION_LINEAR" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_DIMENSION_LINEAR(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_DIMENSION_ORDINATE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_DIMENSION_ORDINATE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_DIMENSION_ORDINATE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_DIMENSION_ORDINATE(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_DIMENSION_RADIUS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_DIMENSION_RADIUS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_DIMENSION_RADIUS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_DIMENSION_RADIUS(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ELLIPSE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ELLIPSE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ELLIPSE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ELLIPSE(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ENDBLK) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ENDBLK(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ENDBLK" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ENDBLK(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_INSERT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_INSERT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_INSERT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_INSERT(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_LEADER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_LEADER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_LEADER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_LEADER(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_LINE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_LINE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_LINE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_LINE(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_LOAD) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_LOAD(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_LOAD" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_LOAD(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_MINSERT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_MINSERT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_MINSERT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_MINSERT(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_MLINE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_MLINE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_MLINE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_MLINE(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_MTEXT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_MTEXT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_MTEXT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_MTEXT(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_OLEFRAME) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_OLEFRAME(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_OLEFRAME" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_OLEFRAME(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_POINT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_POINT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_POINT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_POINT(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_POLYLINE_2D) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_POLYLINE_2D(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_POLYLINE_2D" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_POLYLINE_2D(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_POLYLINE_3D) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_POLYLINE_3D(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_POLYLINE_3D" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_POLYLINE_3D(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_POLYLINE_MESH) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_POLYLINE_MESH(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_POLYLINE_MESH" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_POLYLINE_MESH(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_POLYLINE_PFACE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_POLYLINE_PFACE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_POLYLINE_PFACE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_POLYLINE_PFACE(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_PROXY_ENTITY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_PROXY_ENTITY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_PROXY_ENTITY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_PROXY_ENTITY(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_RAY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_RAY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_RAY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_RAY(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_REGION) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_REGION(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_REGION" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_REGION(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_SEQEND) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_SEQEND(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_SEQEND" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_SEQEND(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_SHAPE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_SHAPE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_SHAPE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_SHAPE(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_SOLID) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_SOLID(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_SOLID" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_SOLID(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_SPLINE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_SPLINE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_SPLINE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_SPLINE(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_TEXT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_TEXT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_TEXT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_TEXT(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_TOLERANCE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_TOLERANCE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_TOLERANCE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_TOLERANCE(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_TRACE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_TRACE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_TRACE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_TRACE(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_UNKNOWN_ENT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_UNKNOWN_ENT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_UNKNOWN_ENT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_UNKNOWN_ENT(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_VERTEX_2D) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_VERTEX_2D(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_VERTEX_2D" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_VERTEX_2D(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_VERTEX_3D) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_VERTEX_3D(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_VERTEX_3D" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_VERTEX_3D(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_VERTEX_MESH) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_VERTEX_MESH(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_VERTEX_MESH" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_VERTEX_MESH(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_VERTEX_PFACE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_VERTEX_PFACE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_VERTEX_PFACE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_VERTEX_PFACE(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_VERTEX_PFACE_FACE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_VERTEX_PFACE_FACE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_VERTEX_PFACE_FACE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_VERTEX_PFACE_FACE(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_VIEWPORT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_VIEWPORT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_VIEWPORT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_VIEWPORT(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_XLINE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_XLINE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_XLINE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_XLINE(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_APPID) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_APPID(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_APPID" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_APPID(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_APPID_CONTROL) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_APPID_CONTROL(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_APPID_CONTROL" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_APPID_CONTROL(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCK_CONTROL) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCK_CONTROL(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCK_CONTROL" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCK_CONTROL(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCK_HEADER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCK_HEADER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCK_HEADER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCK_HEADER(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_DICTIONARY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_DICTIONARY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_DICTIONARY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_DICTIONARY(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_DIMSTYLE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_DIMSTYLE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_DIMSTYLE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_DIMSTYLE(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_DIMSTYLE_CONTROL) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_DIMSTYLE_CONTROL(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_DIMSTYLE_CONTROL" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_DIMSTYLE_CONTROL(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_DUMMY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_DUMMY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_DUMMY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_DUMMY(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_LAYER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_LAYER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_LAYER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_LAYER(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_LAYER_CONTROL) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_LAYER_CONTROL(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_LAYER_CONTROL" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_LAYER_CONTROL(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_LONG_TRANSACTION) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_LONG_TRANSACTION(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_LONG_TRANSACTION" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_LONG_TRANSACTION(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_LTYPE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_LTYPE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_LTYPE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_LTYPE(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_LTYPE_CONTROL) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_LTYPE_CONTROL(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_LTYPE_CONTROL" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_LTYPE_CONTROL(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_MLINESTYLE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_MLINESTYLE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_MLINESTYLE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_MLINESTYLE(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_STYLE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_STYLE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_STYLE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_STYLE(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_STYLE_CONTROL) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_STYLE_CONTROL(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_STYLE_CONTROL" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_STYLE_CONTROL(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_UCS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_UCS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_UCS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_UCS(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_UCS_CONTROL) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_UCS_CONTROL(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_UCS_CONTROL" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_UCS_CONTROL(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_UNKNOWN_OBJ) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_UNKNOWN_OBJ(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_UNKNOWN_OBJ" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_UNKNOWN_OBJ(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_VIEW) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_VIEW(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_VIEW" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_VIEW(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_VIEW_CONTROL) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_VIEW_CONTROL(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_VIEW_CONTROL" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_VIEW_CONTROL(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_VPORT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_VPORT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_VPORT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_VPORT(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_VPORT_CONTROL) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_VPORT_CONTROL(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_VPORT_CONTROL" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_VPORT_CONTROL(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_VX_CONTROL) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_VX_CONTROL(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_VX_CONTROL" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_VX_CONTROL(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_VX_TABLE_RECORD) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_VX_TABLE_RECORD(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_VX_TABLE_RECORD" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_VX_TABLE_RECORD(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup__3DLINE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup__3DLINE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup__3DLINE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup__3DLINE(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_CAMERA) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_CAMERA(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_CAMERA" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_CAMERA(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_DGNUNDERLAY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_DGNUNDERLAY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_DGNUNDERLAY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_DGNUNDERLAY(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_DWFUNDERLAY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_DWFUNDERLAY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_DWFUNDERLAY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_DWFUNDERLAY(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ENDREP) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ENDREP(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ENDREP" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ENDREP(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_HATCH) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_HATCH(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_HATCH" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_HATCH(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_IMAGE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_IMAGE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_IMAGE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_IMAGE(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_JUMP) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_JUMP(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_JUMP" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_JUMP(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_LIGHT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_LIGHT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_LIGHT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_LIGHT(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_LWPOLYLINE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_LWPOLYLINE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_LWPOLYLINE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_LWPOLYLINE(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_MESH) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_MESH(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_MESH" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_MESH(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_MULTILEADER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_MULTILEADER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_MULTILEADER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_MULTILEADER(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_OLE2FRAME) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_OLE2FRAME(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_OLE2FRAME" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_OLE2FRAME(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_PDFUNDERLAY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_PDFUNDERLAY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_PDFUNDERLAY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_PDFUNDERLAY(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_REPEAT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_REPEAT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_REPEAT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_REPEAT(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_SECTIONOBJECT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_SECTIONOBJECT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_SECTIONOBJECT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_SECTIONOBJECT(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_WIPEOUT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_WIPEOUT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_WIPEOUT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_WIPEOUT(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ACSH_BOOLEAN_CLASS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ACSH_BOOLEAN_CLASS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ACSH_BOOLEAN_CLASS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ACSH_BOOLEAN_CLASS(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ACSH_BOX_CLASS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ACSH_BOX_CLASS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ACSH_BOX_CLASS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ACSH_BOX_CLASS(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ACSH_CONE_CLASS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ACSH_CONE_CLASS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ACSH_CONE_CLASS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ACSH_CONE_CLASS(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ACSH_CYLINDER_CLASS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ACSH_CYLINDER_CLASS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ACSH_CYLINDER_CLASS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ACSH_CYLINDER_CLASS(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ACSH_FILLET_CLASS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ACSH_FILLET_CLASS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ACSH_FILLET_CLASS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ACSH_FILLET_CLASS(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ACSH_HISTORY_CLASS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ACSH_HISTORY_CLASS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ACSH_HISTORY_CLASS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ACSH_HISTORY_CLASS(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ACSH_SPHERE_CLASS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ACSH_SPHERE_CLASS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ACSH_SPHERE_CLASS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ACSH_SPHERE_CLASS(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ACSH_TORUS_CLASS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ACSH_TORUS_CLASS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ACSH_TORUS_CLASS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ACSH_TORUS_CLASS(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ACSH_WEDGE_CLASS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ACSH_WEDGE_CLASS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ACSH_WEDGE_CLASS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ACSH_WEDGE_CLASS(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ASSOCGEOMDEPENDENCY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ASSOCGEOMDEPENDENCY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ASSOCGEOMDEPENDENCY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ASSOCGEOMDEPENDENCY(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ASSOCNETWORK) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ASSOCNETWORK(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ASSOCNETWORK" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ASSOCNETWORK(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKALIGNMENTGRIP) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKALIGNMENTGRIP(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKALIGNMENTGRIP" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKALIGNMENTGRIP(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKALIGNMENTPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKALIGNMENTPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKALIGNMENTPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKALIGNMENTPARAMETER(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKBASEPOINTPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKBASEPOINTPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKBASEPOINTPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKBASEPOINTPARAMETER(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKFLIPACTION) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKFLIPACTION(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKFLIPACTION" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKFLIPACTION(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKFLIPGRIP) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKFLIPGRIP(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKFLIPGRIP" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKFLIPGRIP(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKFLIPPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKFLIPPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKFLIPPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKFLIPPARAMETER(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKGRIPLOCATIONCOMPONENT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKGRIPLOCATIONCOMPONENT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKGRIPLOCATIONCOMPONENT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKGRIPLOCATIONCOMPONENT(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKLINEARGRIP) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKLINEARGRIP(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKLINEARGRIP" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKLINEARGRIP(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKLOOKUPGRIP) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKLOOKUPGRIP(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKLOOKUPGRIP" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKLOOKUPGRIP(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKMOVEACTION) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKMOVEACTION(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKMOVEACTION" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKMOVEACTION(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKROTATEACTION) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKROTATEACTION(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKROTATEACTION" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKROTATEACTION(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKROTATIONGRIP) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKROTATIONGRIP(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKROTATIONGRIP" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKROTATIONGRIP(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKSCALEACTION) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKSCALEACTION(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKSCALEACTION" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKSCALEACTION(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKVISIBILITYGRIP) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKVISIBILITYGRIP(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKVISIBILITYGRIP" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKVISIBILITYGRIP(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_CELLSTYLEMAP) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_CELLSTYLEMAP(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_CELLSTYLEMAP" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_CELLSTYLEMAP(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_DETAILVIEWSTYLE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_DETAILVIEWSTYLE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_DETAILVIEWSTYLE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_DETAILVIEWSTYLE(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_DICTIONARYVAR) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_DICTIONARYVAR(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_DICTIONARYVAR" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_DICTIONARYVAR(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_DICTIONARYWDFLT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_DICTIONARYWDFLT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_DICTIONARYWDFLT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_DICTIONARYWDFLT(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_DYNAMICBLOCKPURGEPREVENTER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_DYNAMICBLOCKPURGEPREVENTER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_DYNAMICBLOCKPURGEPREVENTER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_DYNAMICBLOCKPURGEPREVENTER(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_FIELD) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_FIELD(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_FIELD" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_FIELD(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_FIELDLIST) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_FIELDLIST(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_FIELDLIST" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_FIELDLIST(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_GEODATA) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_GEODATA(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_GEODATA" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_GEODATA(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_GROUP) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_GROUP(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_GROUP" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_GROUP(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_IDBUFFER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_IDBUFFER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_IDBUFFER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_IDBUFFER(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_IMAGEDEF) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_IMAGEDEF(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_IMAGEDEF" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_IMAGEDEF(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_IMAGEDEF_REACTOR) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_IMAGEDEF_REACTOR(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_IMAGEDEF_REACTOR" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_IMAGEDEF_REACTOR(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_INDEX) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_INDEX(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_INDEX" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_INDEX(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_LAYERFILTER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_LAYERFILTER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_LAYERFILTER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_LAYERFILTER(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_LAYER_INDEX) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_LAYER_INDEX(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_LAYER_INDEX" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_LAYER_INDEX(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_LAYOUT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_LAYOUT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_LAYOUT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_LAYOUT(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_PLACEHOLDER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_PLACEHOLDER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_PLACEHOLDER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_PLACEHOLDER(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_PLOTSETTINGS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_PLOTSETTINGS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_PLOTSETTINGS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_PLOTSETTINGS(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_RASTERVARIABLES) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_RASTERVARIABLES(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_RASTERVARIABLES" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_RASTERVARIABLES(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_RENDERENVIRONMENT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_RENDERENVIRONMENT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_RENDERENVIRONMENT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_RENDERENVIRONMENT(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_SCALE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_SCALE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_SCALE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_SCALE(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_SECTIONVIEWSTYLE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_SECTIONVIEWSTYLE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_SECTIONVIEWSTYLE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_SECTIONVIEWSTYLE(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_SECTION_MANAGER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_SECTION_MANAGER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_SECTION_MANAGER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_SECTION_MANAGER(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_SORTENTSTABLE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_SORTENTSTABLE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_SORTENTSTABLE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_SORTENTSTABLE(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_SPATIAL_FILTER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_SPATIAL_FILTER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_SPATIAL_FILTER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_SPATIAL_FILTER(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_SUN) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_SUN(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_SUN" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_SUN(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_TABLEGEOMETRY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_TABLEGEOMETRY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_TABLEGEOMETRY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_TABLEGEOMETRY(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_VBA_PROJECT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_VBA_PROJECT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_VBA_PROJECT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_VBA_PROJECT(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_VISUALSTYLE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_VISUALSTYLE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_VISUALSTYLE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_VISUALSTYLE(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_WIPEOUTVARIABLES) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_WIPEOUTVARIABLES(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_WIPEOUTVARIABLES" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_WIPEOUTVARIABLES(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_XRECORD) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_XRECORD(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_XRECORD" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_XRECORD(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_PDFDEFINITION) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_PDFDEFINITION(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_PDFDEFINITION" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_PDFDEFINITION(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_DGNDEFINITION) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_DGNDEFINITION(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_DGNDEFINITION" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_DGNDEFINITION(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_DWFDEFINITION) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_DWFDEFINITION(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_DWFDEFINITION" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_DWFDEFINITION(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ARC_DIMENSION) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ARC_DIMENSION(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ARC_DIMENSION" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ARC_DIMENSION(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_HELIX) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_HELIX(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_HELIX" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_HELIX(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_LARGE_RADIAL_DIMENSION) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_LARGE_RADIAL_DIMENSION(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_LARGE_RADIAL_DIMENSION" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_LARGE_RADIAL_DIMENSION(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_LAYOUTPRINTCONFIG) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_LAYOUTPRINTCONFIG(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_LAYOUTPRINTCONFIG" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_LAYOUTPRINTCONFIG(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_PLANESURFACE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_PLANESURFACE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_PLANESURFACE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_PLANESURFACE(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_POINTCLOUD) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_POINTCLOUD(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_POINTCLOUD" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_POINTCLOUD(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_POINTCLOUDEX) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_POINTCLOUDEX(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_POINTCLOUDEX" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_POINTCLOUDEX(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ACSH_BREP_CLASS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ACSH_BREP_CLASS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ACSH_BREP_CLASS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ACSH_BREP_CLASS(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ACSH_CHAMFER_CLASS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ACSH_CHAMFER_CLASS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ACSH_CHAMFER_CLASS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ACSH_CHAMFER_CLASS(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ACSH_PYRAMID_CLASS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ACSH_PYRAMID_CLASS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ACSH_PYRAMID_CLASS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ACSH_PYRAMID_CLASS(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ALDIMOBJECTCONTEXTDATA) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ALDIMOBJECTCONTEXTDATA(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ALDIMOBJECTCONTEXTDATA" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ALDIMOBJECTCONTEXTDATA(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ASSOC2DCONSTRAINTGROUP) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ASSOC2DCONSTRAINTGROUP(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ASSOC2DCONSTRAINTGROUP" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ASSOC2DCONSTRAINTGROUP(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ASSOCACTION) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ASSOCACTION(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ASSOCACTION" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ASSOCACTION(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ASSOCACTIONPARAM) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ASSOCACTIONPARAM(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ASSOCACTIONPARAM" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ASSOCACTIONPARAM(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ASSOCARRAYACTIONBODY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ASSOCARRAYACTIONBODY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ASSOCARRAYACTIONBODY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ASSOCARRAYACTIONBODY(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ASSOCASMBODYACTIONPARAM) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ASSOCASMBODYACTIONPARAM(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ASSOCASMBODYACTIONPARAM" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ASSOCASMBODYACTIONPARAM(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ASSOCBLENDSURFACEACTIONBODY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ASSOCBLENDSURFACEACTIONBODY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ASSOCBLENDSURFACEACTIONBODY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ASSOCBLENDSURFACEACTIONBODY(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ASSOCCOMPOUNDACTIONPARAM) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ASSOCCOMPOUNDACTIONPARAM(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ASSOCCOMPOUNDACTIONPARAM" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ASSOCCOMPOUNDACTIONPARAM(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ASSOCDEPENDENCY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ASSOCDEPENDENCY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ASSOCDEPENDENCY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ASSOCDEPENDENCY(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ASSOCDIMDEPENDENCYBODY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ASSOCDIMDEPENDENCYBODY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ASSOCDIMDEPENDENCYBODY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ASSOCDIMDEPENDENCYBODY(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ASSOCEXTENDSURFACEACTIONBODY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ASSOCEXTENDSURFACEACTIONBODY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ASSOCEXTENDSURFACEACTIONBODY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ASSOCEXTENDSURFACEACTIONBODY(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ASSOCEXTRUDEDSURFACEACTIONBODY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ASSOCEXTRUDEDSURFACEACTIONBODY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ASSOCEXTRUDEDSURFACEACTIONBODY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ASSOCEXTRUDEDSURFACEACTIONBODY(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ASSOCFACEACTIONPARAM) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ASSOCFACEACTIONPARAM(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ASSOCFACEACTIONPARAM" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ASSOCFACEACTIONPARAM(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ASSOCFILLETSURFACEACTIONBODY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ASSOCFILLETSURFACEACTIONBODY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ASSOCFILLETSURFACEACTIONBODY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ASSOCFILLETSURFACEACTIONBODY(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ASSOCLOFTEDSURFACEACTIONBODY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ASSOCLOFTEDSURFACEACTIONBODY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ASSOCLOFTEDSURFACEACTIONBODY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ASSOCLOFTEDSURFACEACTIONBODY(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ASSOCNETWORKSURFACEACTIONBODY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ASSOCNETWORKSURFACEACTIONBODY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ASSOCNETWORKSURFACEACTIONBODY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ASSOCNETWORKSURFACEACTIONBODY(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ASSOCOBJECTACTIONPARAM) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ASSOCOBJECTACTIONPARAM(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ASSOCOBJECTACTIONPARAM" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ASSOCOBJECTACTIONPARAM(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ASSOCOFFSETSURFACEACTIONBODY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ASSOCOFFSETSURFACEACTIONBODY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ASSOCOFFSETSURFACEACTIONBODY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ASSOCOFFSETSURFACEACTIONBODY(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ASSOCOSNAPPOINTREFACTIONPARAM) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ASSOCOSNAPPOINTREFACTIONPARAM(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ASSOCOSNAPPOINTREFACTIONPARAM" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ASSOCOSNAPPOINTREFACTIONPARAM(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ASSOCPATCHSURFACEACTIONBODY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ASSOCPATCHSURFACEACTIONBODY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ASSOCPATCHSURFACEACTIONBODY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ASSOCPATCHSURFACEACTIONBODY(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ASSOCPATHACTIONPARAM) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ASSOCPATHACTIONPARAM(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ASSOCPATHACTIONPARAM" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ASSOCPATHACTIONPARAM(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ASSOCPLANESURFACEACTIONBODY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ASSOCPLANESURFACEACTIONBODY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ASSOCPLANESURFACEACTIONBODY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ASSOCPLANESURFACEACTIONBODY(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ASSOCPOINTREFACTIONPARAM) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ASSOCPOINTREFACTIONPARAM(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ASSOCPOINTREFACTIONPARAM" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ASSOCPOINTREFACTIONPARAM(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ASSOCREVOLVEDSURFACEACTIONBODY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ASSOCREVOLVEDSURFACEACTIONBODY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ASSOCREVOLVEDSURFACEACTIONBODY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ASSOCREVOLVEDSURFACEACTIONBODY(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ASSOCTRIMSURFACEACTIONBODY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ASSOCTRIMSURFACEACTIONBODY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ASSOCTRIMSURFACEACTIONBODY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ASSOCTRIMSURFACEACTIONBODY(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ASSOCVALUEDEPENDENCY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ASSOCVALUEDEPENDENCY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ASSOCVALUEDEPENDENCY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ASSOCVALUEDEPENDENCY(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ASSOCVARIABLE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ASSOCVARIABLE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ASSOCVARIABLE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ASSOCVARIABLE(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ASSOCVERTEXACTIONPARAM) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ASSOCVERTEXACTIONPARAM(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ASSOCVERTEXACTIONPARAM" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ASSOCVERTEXACTIONPARAM(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLKREFOBJECTCONTEXTDATA) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLKREFOBJECTCONTEXTDATA(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLKREFOBJECTCONTEXTDATA" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLKREFOBJECTCONTEXTDATA(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKALIGNEDCONSTRAINTPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKALIGNEDCONSTRAINTPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKALIGNEDCONSTRAINTPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKALIGNEDCONSTRAINTPARAMETER(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKANGULARCONSTRAINTPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKANGULARCONSTRAINTPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKANGULARCONSTRAINTPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKANGULARCONSTRAINTPARAMETER(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKARRAYACTION) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKARRAYACTION(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKARRAYACTION" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKARRAYACTION(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKDIAMETRICCONSTRAINTPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKDIAMETRICCONSTRAINTPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKDIAMETRICCONSTRAINTPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKDIAMETRICCONSTRAINTPARAMETER(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKHORIZONTALCONSTRAINTPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKHORIZONTALCONSTRAINTPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKHORIZONTALCONSTRAINTPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKHORIZONTALCONSTRAINTPARAMETER(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKLINEARCONSTRAINTPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKLINEARCONSTRAINTPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKLINEARCONSTRAINTPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKLINEARCONSTRAINTPARAMETER(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKLINEARPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKLINEARPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKLINEARPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKLINEARPARAMETER(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKLOOKUPACTION) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKLOOKUPACTION(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKLOOKUPACTION" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKLOOKUPACTION(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKLOOKUPPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKLOOKUPPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKLOOKUPPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKLOOKUPPARAMETER(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKPARAMDEPENDENCYBODY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKPARAMDEPENDENCYBODY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKPARAMDEPENDENCYBODY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKPARAMDEPENDENCYBODY(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKPOINTPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKPOINTPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKPOINTPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKPOINTPARAMETER(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKPOLARGRIP) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKPOLARGRIP(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKPOLARGRIP" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKPOLARGRIP(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKPOLARPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKPOLARPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKPOLARPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKPOLARPARAMETER(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKPOLARSTRETCHACTION) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKPOLARSTRETCHACTION(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKPOLARSTRETCHACTION" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKPOLARSTRETCHACTION(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKRADIALCONSTRAINTPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKRADIALCONSTRAINTPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKRADIALCONSTRAINTPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKRADIALCONSTRAINTPARAMETER(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKREPRESENTATION) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKREPRESENTATION(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKREPRESENTATION" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKREPRESENTATION(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKROTATIONPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKROTATIONPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKROTATIONPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKROTATIONPARAMETER(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKSTRETCHACTION) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKSTRETCHACTION(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKSTRETCHACTION" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKSTRETCHACTION(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKUSERPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKUSERPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKUSERPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKUSERPARAMETER(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKVERTICALCONSTRAINTPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKVERTICALCONSTRAINTPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKVERTICALCONSTRAINTPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKVERTICALCONSTRAINTPARAMETER(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKVISIBILITYPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKVISIBILITYPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKVISIBILITYPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKVISIBILITYPARAMETER(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKXYGRIP) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKXYGRIP(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKXYGRIP" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKXYGRIP(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_BLOCKXYPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_BLOCKXYPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_BLOCKXYPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_BLOCKXYPARAMETER(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_DATALINK) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_DATALINK(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_DATALINK" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_DATALINK(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_DBCOLOR) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_DBCOLOR(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_DBCOLOR" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_DBCOLOR(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_EVALUATION_GRAPH) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_EVALUATION_GRAPH(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_EVALUATION_GRAPH" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_EVALUATION_GRAPH(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_FCFOBJECTCONTEXTDATA) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_FCFOBJECTCONTEXTDATA(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_FCFOBJECTCONTEXTDATA" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_FCFOBJECTCONTEXTDATA(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_GRADIENT_BACKGROUND) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_GRADIENT_BACKGROUND(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_GRADIENT_BACKGROUND" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_GRADIENT_BACKGROUND(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_GROUND_PLANE_BACKGROUND) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_GROUND_PLANE_BACKGROUND(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_GROUND_PLANE_BACKGROUND" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_GROUND_PLANE_BACKGROUND(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_IBL_BACKGROUND) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_IBL_BACKGROUND(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_IBL_BACKGROUND" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_IBL_BACKGROUND(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_IMAGE_BACKGROUND) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_IMAGE_BACKGROUND(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_IMAGE_BACKGROUND" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_IMAGE_BACKGROUND(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_LEADEROBJECTCONTEXTDATA) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_LEADEROBJECTCONTEXTDATA(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_LEADEROBJECTCONTEXTDATA" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_LEADEROBJECTCONTEXTDATA(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_LIGHTLIST) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_LIGHTLIST(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_LIGHTLIST" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_LIGHTLIST(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_MATERIAL) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_MATERIAL(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_MATERIAL" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_MATERIAL(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_MENTALRAYRENDERSETTINGS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_MENTALRAYRENDERSETTINGS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_MENTALRAYRENDERSETTINGS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_MENTALRAYRENDERSETTINGS(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_MLEADERSTYLE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_MLEADERSTYLE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_MLEADERSTYLE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_MLEADERSTYLE(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_MTEXTOBJECTCONTEXTDATA) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_MTEXTOBJECTCONTEXTDATA(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_MTEXTOBJECTCONTEXTDATA" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_MTEXTOBJECTCONTEXTDATA(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_OBJECT_PTR) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_OBJECT_PTR(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_OBJECT_PTR" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_OBJECT_PTR(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_PARTIAL_VIEWING_INDEX) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_PARTIAL_VIEWING_INDEX(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_PARTIAL_VIEWING_INDEX" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_PARTIAL_VIEWING_INDEX(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_POINTCLOUDCOLORMAP) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_POINTCLOUDCOLORMAP(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_POINTCLOUDCOLORMAP" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_POINTCLOUDCOLORMAP(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_POINTCLOUDDEF) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_POINTCLOUDDEF(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_POINTCLOUDDEF" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_POINTCLOUDDEF(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_POINTCLOUDDEFEX) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_POINTCLOUDDEFEX(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_POINTCLOUDDEFEX" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_POINTCLOUDDEFEX(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_POINTCLOUDDEF_REACTOR) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_POINTCLOUDDEF_REACTOR(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_POINTCLOUDDEF_REACTOR" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_POINTCLOUDDEF_REACTOR(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_POINTCLOUDDEF_REACTOR_EX) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_POINTCLOUDDEF_REACTOR_EX(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_POINTCLOUDDEF_REACTOR_EX" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_POINTCLOUDDEF_REACTOR_EX(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_PROXY_OBJECT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_PROXY_OBJECT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_PROXY_OBJECT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_PROXY_OBJECT(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_RAPIDRTRENDERSETTINGS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_RAPIDRTRENDERSETTINGS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_RAPIDRTRENDERSETTINGS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_RAPIDRTRENDERSETTINGS(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_RENDERENTRY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_RENDERENTRY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_RENDERENTRY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_RENDERENTRY(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_RENDERGLOBAL) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_RENDERGLOBAL(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_RENDERGLOBAL" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_RENDERGLOBAL(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_RENDERSETTINGS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_RENDERSETTINGS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_RENDERSETTINGS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_RENDERSETTINGS(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_SECTION_SETTINGS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_SECTION_SETTINGS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_SECTION_SETTINGS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_SECTION_SETTINGS(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_SKYLIGHT_BACKGROUND) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_SKYLIGHT_BACKGROUND(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_SKYLIGHT_BACKGROUND" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_SKYLIGHT_BACKGROUND(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_SOLID_BACKGROUND) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_SOLID_BACKGROUND(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_SOLID_BACKGROUND" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_SOLID_BACKGROUND(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_SPATIAL_INDEX) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_SPATIAL_INDEX(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_SPATIAL_INDEX" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_SPATIAL_INDEX(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_TABLESTYLE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_TABLESTYLE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_TABLESTYLE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_TABLESTYLE(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_TEXTOBJECTCONTEXTDATA) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_TEXTOBJECTCONTEXTDATA(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_TEXTOBJECTCONTEXTDATA" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_TEXTOBJECTCONTEXTDATA(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ASSOCARRAYMODIFYPARAMETERS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ASSOCARRAYMODIFYPARAMETERS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ASSOCARRAYMODIFYPARAMETERS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ASSOCARRAYMODIFYPARAMETERS(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ASSOCARRAYPATHPARAMETERS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ASSOCARRAYPATHPARAMETERS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ASSOCARRAYPATHPARAMETERS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ASSOCARRAYPATHPARAMETERS(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ASSOCARRAYPOLARPARAMETERS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ASSOCARRAYPOLARPARAMETERS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ASSOCARRAYPOLARPARAMETERS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ASSOCARRAYPOLARPARAMETERS(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_setup_ASSOCARRAYRECTANGULARPARAMETERS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_setup_ASSOCARRAYRECTANGULARPARAMETERS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_setup_ASSOCARRAYRECTANGULARPARAMETERS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_setup_ASSOCARRAYRECTANGULARPARAMETERS(arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DYNAPI_field_name_get) {
  {
    struct dwg_field_name_type_offset *arg1 = (struct dwg_field_name_type_offset *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    char *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DYNAPI_field_name_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_dwg_field_name_type_offset, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DYNAPI_field_name_get" "', argument " "1"" of type '" "struct dwg_field_name_type_offset *""'"); 
    }
    arg1 = (struct dwg_field_name_type_offset *)(argp1);
    result = (char *)(char *) ((arg1)->name);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DYNAPI_field_type_get) {
  {
    struct dwg_field_name_type_offset *arg1 = (struct dwg_field_name_type_offset *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    char *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DYNAPI_field_type_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_dwg_field_name_type_offset, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DYNAPI_field_type_get" "', argument " "1"" of type '" "struct dwg_field_name_type_offset *""'"); 
    }
    arg1 = (struct dwg_field_name_type_offset *)(argp1);
    result = (char *)(char *) ((arg1)->type);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DYNAPI_field_size_get) {
  {
    struct dwg_field_name_type_offset *arg1 = (struct dwg_field_name_type_offset *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    unsigned short result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DYNAPI_field_size_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_dwg_field_name_type_offset, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DYNAPI_field_size_get" "', argument " "1"" of type '" "struct dwg_field_name_type_offset *""'"); 
    }
    arg1 = (struct dwg_field_name_type_offset *)(argp1);
    result = (unsigned short)(unsigned short) ((arg1)->size);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DYNAPI_field_offset_get) {
  {
    struct dwg_field_name_type_offset *arg1 = (struct dwg_field_name_type_offset *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    unsigned short result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DYNAPI_field_offset_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_dwg_field_name_type_offset, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DYNAPI_field_offset_get" "', argument " "1"" of type '" "struct dwg_field_name_type_offset *""'"); 
    }
    arg1 = (struct dwg_field_name_type_offset *)(argp1);
    result = (unsigned short)(unsigned short) ((arg1)->offset);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DYNAPI_field_is_indirect_get) {
  {
    struct dwg_field_name_type_offset *arg1 = (struct dwg_field_name_type_offset *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    unsigned short result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DYNAPI_field_is_indirect_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_dwg_field_name_type_offset, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DYNAPI_field_is_indirect_get" "', argument " "1"" of type '" "struct dwg_field_name_type_offset *""'"); 
    }
    arg1 = (struct dwg_field_name_type_offset *)(argp1);
    result = (unsigned short)(unsigned short) ((arg1)->is_indirect);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DYNAPI_field_is_malloc_get) {
  {
    struct dwg_field_name_type_offset *arg1 = (struct dwg_field_name_type_offset *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    unsigned short result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DYNAPI_field_is_malloc_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_dwg_field_name_type_offset, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DYNAPI_field_is_malloc_get" "', argument " "1"" of type '" "struct dwg_field_name_type_offset *""'"); 
    }
    arg1 = (struct dwg_field_name_type_offset *)(argp1);
    result = (unsigned short)(unsigned short) ((arg1)->is_malloc);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DYNAPI_field_is_string_get) {
  {
    struct dwg_field_name_type_offset *arg1 = (struct dwg_field_name_type_offset *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    unsigned short result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DYNAPI_field_is_string_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_dwg_field_name_type_offset, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DYNAPI_field_is_string_get" "', argument " "1"" of type '" "struct dwg_field_name_type_offset *""'"); 
    }
    arg1 = (struct dwg_field_name_type_offset *)(argp1);
    result = (unsigned short)(unsigned short) ((arg1)->is_string);
    ST(argvi) = SWIG_From_unsigned_SS_short  SWIG_PERL_CALL_ARGS_1((unsigned short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_Dwg_DYNAPI_field_dxf_get) {
  {
    struct dwg_field_name_type_offset *arg1 = (struct dwg_field_name_type_offset *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    short result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: Dwg_DYNAPI_field_dxf_get(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_dwg_field_name_type_offset, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "Dwg_DYNAPI_field_dxf_get" "', argument " "1"" of type '" "struct dwg_field_name_type_offset *""'"); 
    }
    arg1 = (struct dwg_field_name_type_offset *)(argp1);
    result = (short)(short) ((arg1)->dxf);
    ST(argvi) = SWIG_From_short  SWIG_PERL_CALL_ARGS_1((short)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_new_Dwg_DYNAPI_field) {
  {
    int argvi = 0;
    struct dwg_field_name_type_offset *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: new_Dwg_DYNAPI_field();");
    }
    result = (struct dwg_field_name_type_offset *)calloc(1, sizeof(struct dwg_field_name_type_offset));
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_dwg_field_name_type_offset, SWIG_OWNER | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_delete_Dwg_DYNAPI_field) {
  {
    struct dwg_field_name_type_offset *arg1 = (struct dwg_field_name_type_offset *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: delete_Dwg_DYNAPI_field(self);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_dwg_field_name_type_offset, SWIG_POINTER_DISOWN |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "delete_Dwg_DYNAPI_field" "', argument " "1"" of type '" "struct dwg_field_name_type_offset *""'"); 
    }
    arg1 = (struct dwg_field_name_type_offset *)(argp1);
    free((char *) arg1);
    ST(argvi) = &PL_sv_undef;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_is_dwg_entity) {
  {
    char *arg1 = (char *) 0 ;
    int res1 ;
    char *buf1 = 0 ;
    int alloc1 = 0 ;
    int argvi = 0;
    bool result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: is_dwg_entity(name);");
    }
    res1 = SWIG_AsCharPtrAndSize(ST(0), &buf1, NULL, &alloc1);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "is_dwg_entity" "', argument " "1"" of type '" "char const *""'");
    }
    arg1 = (char *)(buf1);
    result = (bool)is_dwg_entity((char const *)arg1);
    ST(argvi) = SWIG_From_bool  SWIG_PERL_CALL_ARGS_1((bool)(result)); argvi++ ;
    if (alloc1 == SWIG_NEWOBJ) free((char*)buf1);
    XSRETURN(argvi);
  fail:
    if (alloc1 == SWIG_NEWOBJ) free((char*)buf1);
    SWIG_croak_null();
  }
}


XS(_wrap_is_dwg_object) {
  {
    char *arg1 = (char *) 0 ;
    int res1 ;
    char *buf1 = 0 ;
    int alloc1 = 0 ;
    int argvi = 0;
    bool result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: is_dwg_object(name);");
    }
    res1 = SWIG_AsCharPtrAndSize(ST(0), &buf1, NULL, &alloc1);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "is_dwg_object" "', argument " "1"" of type '" "char const *""'");
    }
    arg1 = (char *)(buf1);
    result = (bool)is_dwg_object((char const *)arg1);
    ST(argvi) = SWIG_From_bool  SWIG_PERL_CALL_ARGS_1((bool)(result)); argvi++ ;
    if (alloc1 == SWIG_NEWOBJ) free((char*)buf1);
    XSRETURN(argvi);
  fail:
    if (alloc1 == SWIG_NEWOBJ) free((char*)buf1);
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_dynapi_header_value) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    char *arg2 = (char *) 0 ;
    void *arg3 = (void *) 0 ;
    Dwg_DYNAPI_field *arg4 = (Dwg_DYNAPI_field *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int res3 ;
    void *argp4 = 0 ;
    int res4 = 0 ;
    int argvi = 0;
    bool result;
    dXSARGS;
    
    if ((items < 4) || (items > 4)) {
      SWIG_croak("Usage: dwg_dynapi_header_value(dwg,fieldname,out,fp);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_dynapi_header_value" "', argument " "1"" of type '" "Dwg_Data const *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_dynapi_header_value" "', argument " "2"" of type '" "char const *""'");
    }
    arg2 = (char *)(buf2);
    res3 = SWIG_ConvertPtr(ST(2),SWIG_as_voidptrptr(&arg3), 0, 0);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "dwg_dynapi_header_value" "', argument " "3"" of type '" "void *""'"); 
    }
    res4 = SWIG_ConvertPtr(ST(3), &argp4,SWIGTYPE_p_dwg_field_name_type_offset, 0 |  0 );
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "dwg_dynapi_header_value" "', argument " "4"" of type '" "Dwg_DYNAPI_field *""'"); 
    }
    arg4 = (Dwg_DYNAPI_field *)(argp4);
    result = (bool)dwg_dynapi_header_value((struct _dwg_struct const *)arg1,(char const *)arg2,arg3,arg4);
    ST(argvi) = SWIG_From_bool  SWIG_PERL_CALL_ARGS_1((bool)(result)); argvi++ ;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    
    
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_dynapi_entity_value) {
  {
    void *arg1 = (void *) 0 ;
    char *arg2 = (char *) 0 ;
    char *arg3 = (char *) 0 ;
    void *arg4 = (void *) 0 ;
    Dwg_DYNAPI_field *arg5 = (Dwg_DYNAPI_field *) 0 ;
    int res1 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int res3 ;
    char *buf3 = 0 ;
    int alloc3 = 0 ;
    int res4 ;
    void *argp5 = 0 ;
    int res5 = 0 ;
    int argvi = 0;
    bool result;
    dXSARGS;
    
    if ((items < 5) || (items > 5)) {
      SWIG_croak("Usage: dwg_dynapi_entity_value(entity,dxfname,fieldname,out,fp);");
    }
    res1 = SWIG_ConvertPtr(ST(0),SWIG_as_voidptrptr(&arg1), 0, 0);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_dynapi_entity_value" "', argument " "1"" of type '" "void *""'"); 
    }
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_dynapi_entity_value" "', argument " "2"" of type '" "char const *""'");
    }
    arg2 = (char *)(buf2);
    res3 = SWIG_AsCharPtrAndSize(ST(2), &buf3, NULL, &alloc3);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "dwg_dynapi_entity_value" "', argument " "3"" of type '" "char const *""'");
    }
    arg3 = (char *)(buf3);
    res4 = SWIG_ConvertPtr(ST(3),SWIG_as_voidptrptr(&arg4), 0, 0);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "dwg_dynapi_entity_value" "', argument " "4"" of type '" "void *""'"); 
    }
    res5 = SWIG_ConvertPtr(ST(4), &argp5,SWIGTYPE_p_dwg_field_name_type_offset, 0 |  0 );
    if (!SWIG_IsOK(res5)) {
      SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "dwg_dynapi_entity_value" "', argument " "5"" of type '" "Dwg_DYNAPI_field *""'"); 
    }
    arg5 = (Dwg_DYNAPI_field *)(argp5);
    result = (bool)dwg_dynapi_entity_value(arg1,(char const *)arg2,(char const *)arg3,arg4,arg5);
    ST(argvi) = SWIG_From_bool  SWIG_PERL_CALL_ARGS_1((bool)(result)); argvi++ ;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    if (alloc3 == SWIG_NEWOBJ) free((char*)buf3);
    
    
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    if (alloc3 == SWIG_NEWOBJ) free((char*)buf3);
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_dynapi_common_value) {
  {
    void *arg1 = (void *) 0 ;
    char *arg2 = (char *) 0 ;
    void *arg3 = (void *) 0 ;
    Dwg_DYNAPI_field *arg4 = (Dwg_DYNAPI_field *) 0 ;
    int res1 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int res3 ;
    void *argp4 = 0 ;
    int res4 = 0 ;
    int argvi = 0;
    bool result;
    dXSARGS;
    
    if ((items < 4) || (items > 4)) {
      SWIG_croak("Usage: dwg_dynapi_common_value(_obj,fieldname,out,fp);");
    }
    res1 = SWIG_ConvertPtr(ST(0),SWIG_as_voidptrptr(&arg1), 0, 0);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_dynapi_common_value" "', argument " "1"" of type '" "void *""'"); 
    }
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_dynapi_common_value" "', argument " "2"" of type '" "char const *""'");
    }
    arg2 = (char *)(buf2);
    res3 = SWIG_ConvertPtr(ST(2),SWIG_as_voidptrptr(&arg3), 0, 0);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "dwg_dynapi_common_value" "', argument " "3"" of type '" "void *""'"); 
    }
    res4 = SWIG_ConvertPtr(ST(3), &argp4,SWIGTYPE_p_dwg_field_name_type_offset, 0 |  0 );
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "dwg_dynapi_common_value" "', argument " "4"" of type '" "Dwg_DYNAPI_field *""'"); 
    }
    arg4 = (Dwg_DYNAPI_field *)(argp4);
    result = (bool)dwg_dynapi_common_value(arg1,(char const *)arg2,arg3,arg4);
    ST(argvi) = SWIG_From_bool  SWIG_PERL_CALL_ARGS_1((bool)(result)); argvi++ ;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    
    
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_dynapi_subclass_value) {
  {
    void *arg1 = (void *) 0 ;
    char *arg2 = (char *) 0 ;
    char *arg3 = (char *) 0 ;
    void *arg4 = (void *) 0 ;
    Dwg_DYNAPI_field *arg5 = (Dwg_DYNAPI_field *) 0 ;
    int res1 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int res3 ;
    char *buf3 = 0 ;
    int alloc3 = 0 ;
    int res4 ;
    void *argp5 = 0 ;
    int res5 = 0 ;
    int argvi = 0;
    bool result;
    dXSARGS;
    
    if ((items < 5) || (items > 5)) {
      SWIG_croak("Usage: dwg_dynapi_subclass_value(ptr,subclass,fieldname,out,fp);");
    }
    res1 = SWIG_ConvertPtr(ST(0),SWIG_as_voidptrptr(&arg1), 0, 0);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_dynapi_subclass_value" "', argument " "1"" of type '" "void const *""'"); 
    }
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_dynapi_subclass_value" "', argument " "2"" of type '" "char const *""'");
    }
    arg2 = (char *)(buf2);
    res3 = SWIG_AsCharPtrAndSize(ST(2), &buf3, NULL, &alloc3);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "dwg_dynapi_subclass_value" "', argument " "3"" of type '" "char const *""'");
    }
    arg3 = (char *)(buf3);
    res4 = SWIG_ConvertPtr(ST(3),SWIG_as_voidptrptr(&arg4), 0, 0);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "dwg_dynapi_subclass_value" "', argument " "4"" of type '" "void *""'"); 
    }
    res5 = SWIG_ConvertPtr(ST(4), &argp5,SWIGTYPE_p_dwg_field_name_type_offset, 0 |  0 );
    if (!SWIG_IsOK(res5)) {
      SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "dwg_dynapi_subclass_value" "', argument " "5"" of type '" "Dwg_DYNAPI_field *""'"); 
    }
    arg5 = (Dwg_DYNAPI_field *)(argp5);
    result = (bool)dwg_dynapi_subclass_value((void const *)arg1,(char const *)arg2,(char const *)arg3,arg4,arg5);
    ST(argvi) = SWIG_From_bool  SWIG_PERL_CALL_ARGS_1((bool)(result)); argvi++ ;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    if (alloc3 == SWIG_NEWOBJ) free((char*)buf3);
    
    
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    if (alloc3 == SWIG_NEWOBJ) free((char*)buf3);
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_dynapi_header_utf8text) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    char *arg2 = (char *) 0 ;
    char **arg3 = (char **) 0 ;
    int *arg4 = (int *) 0 ;
    Dwg_DYNAPI_field *arg5 = (Dwg_DYNAPI_field *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    void *argp3 = 0 ;
    int res3 = 0 ;
    void *argp4 = 0 ;
    int res4 = 0 ;
    void *argp5 = 0 ;
    int res5 = 0 ;
    int argvi = 0;
    bool result;
    dXSARGS;
    
    if ((items < 5) || (items > 5)) {
      SWIG_croak("Usage: dwg_dynapi_header_utf8text(dwg,fieldname,textp,isnew,fp);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_dynapi_header_utf8text" "', argument " "1"" of type '" "Dwg_Data const *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_dynapi_header_utf8text" "', argument " "2"" of type '" "char const *""'");
    }
    arg2 = (char *)(buf2);
    res3 = SWIG_ConvertPtr(ST(2), &argp3,SWIGTYPE_p_p_char, 0 |  0 );
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "dwg_dynapi_header_utf8text" "', argument " "3"" of type '" "char **""'"); 
    }
    arg3 = (char **)(argp3);
    res4 = SWIG_ConvertPtr(ST(3), &argp4,SWIGTYPE_p_int, 0 |  0 );
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "dwg_dynapi_header_utf8text" "', argument " "4"" of type '" "int *""'"); 
    }
    arg4 = (int *)(argp4);
    res5 = SWIG_ConvertPtr(ST(4), &argp5,SWIGTYPE_p_dwg_field_name_type_offset, 0 |  0 );
    if (!SWIG_IsOK(res5)) {
      SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "dwg_dynapi_header_utf8text" "', argument " "5"" of type '" "Dwg_DYNAPI_field *""'"); 
    }
    arg5 = (Dwg_DYNAPI_field *)(argp5);
    result = (bool)dwg_dynapi_header_utf8text((struct _dwg_struct const *)arg1,(char const *)arg2,arg3,arg4,arg5);
    ST(argvi) = SWIG_From_bool  SWIG_PERL_CALL_ARGS_1((bool)(result)); argvi++ ;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    
    
    
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_dynapi_entity_utf8text) {
  {
    void *arg1 = (void *) 0 ;
    char *arg2 = (char *) 0 ;
    char *arg3 = (char *) 0 ;
    char **arg4 = (char **) 0 ;
    int *arg5 = (int *) 0 ;
    Dwg_DYNAPI_field *arg6 = (Dwg_DYNAPI_field *) 0 ;
    int res1 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int res3 ;
    char *buf3 = 0 ;
    int alloc3 = 0 ;
    void *argp4 = 0 ;
    int res4 = 0 ;
    void *argp5 = 0 ;
    int res5 = 0 ;
    void *argp6 = 0 ;
    int res6 = 0 ;
    int argvi = 0;
    bool result;
    dXSARGS;
    
    if ((items < 6) || (items > 6)) {
      SWIG_croak("Usage: dwg_dynapi_entity_utf8text(entity,name,fieldname,textp,isnew,fp);");
    }
    res1 = SWIG_ConvertPtr(ST(0),SWIG_as_voidptrptr(&arg1), 0, 0);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_dynapi_entity_utf8text" "', argument " "1"" of type '" "void *""'"); 
    }
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_dynapi_entity_utf8text" "', argument " "2"" of type '" "char const *""'");
    }
    arg2 = (char *)(buf2);
    res3 = SWIG_AsCharPtrAndSize(ST(2), &buf3, NULL, &alloc3);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "dwg_dynapi_entity_utf8text" "', argument " "3"" of type '" "char const *""'");
    }
    arg3 = (char *)(buf3);
    res4 = SWIG_ConvertPtr(ST(3), &argp4,SWIGTYPE_p_p_char, 0 |  0 );
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "dwg_dynapi_entity_utf8text" "', argument " "4"" of type '" "char **""'"); 
    }
    arg4 = (char **)(argp4);
    res5 = SWIG_ConvertPtr(ST(4), &argp5,SWIGTYPE_p_int, 0 |  0 );
    if (!SWIG_IsOK(res5)) {
      SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "dwg_dynapi_entity_utf8text" "', argument " "5"" of type '" "int *""'"); 
    }
    arg5 = (int *)(argp5);
    res6 = SWIG_ConvertPtr(ST(5), &argp6,SWIGTYPE_p_dwg_field_name_type_offset, 0 |  0 );
    if (!SWIG_IsOK(res6)) {
      SWIG_exception_fail(SWIG_ArgError(res6), "in method '" "dwg_dynapi_entity_utf8text" "', argument " "6"" of type '" "Dwg_DYNAPI_field *""'"); 
    }
    arg6 = (Dwg_DYNAPI_field *)(argp6);
    result = (bool)dwg_dynapi_entity_utf8text(arg1,(char const *)arg2,(char const *)arg3,arg4,arg5,arg6);
    ST(argvi) = SWIG_From_bool  SWIG_PERL_CALL_ARGS_1((bool)(result)); argvi++ ;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    if (alloc3 == SWIG_NEWOBJ) free((char*)buf3);
    
    
    
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    if (alloc3 == SWIG_NEWOBJ) free((char*)buf3);
    
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_dynapi_common_utf8text) {
  {
    void *arg1 = (void *) 0 ;
    char *arg2 = (char *) 0 ;
    char **arg3 = (char **) 0 ;
    int *arg4 = (int *) 0 ;
    Dwg_DYNAPI_field *arg5 = (Dwg_DYNAPI_field *) 0 ;
    int res1 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    void *argp3 = 0 ;
    int res3 = 0 ;
    void *argp4 = 0 ;
    int res4 = 0 ;
    void *argp5 = 0 ;
    int res5 = 0 ;
    int argvi = 0;
    bool result;
    dXSARGS;
    
    if ((items < 5) || (items > 5)) {
      SWIG_croak("Usage: dwg_dynapi_common_utf8text(_obj,fieldname,textp,isnew,fp);");
    }
    res1 = SWIG_ConvertPtr(ST(0),SWIG_as_voidptrptr(&arg1), 0, 0);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_dynapi_common_utf8text" "', argument " "1"" of type '" "void *""'"); 
    }
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_dynapi_common_utf8text" "', argument " "2"" of type '" "char const *""'");
    }
    arg2 = (char *)(buf2);
    res3 = SWIG_ConvertPtr(ST(2), &argp3,SWIGTYPE_p_p_char, 0 |  0 );
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "dwg_dynapi_common_utf8text" "', argument " "3"" of type '" "char **""'"); 
    }
    arg3 = (char **)(argp3);
    res4 = SWIG_ConvertPtr(ST(3), &argp4,SWIGTYPE_p_int, 0 |  0 );
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "dwg_dynapi_common_utf8text" "', argument " "4"" of type '" "int *""'"); 
    }
    arg4 = (int *)(argp4);
    res5 = SWIG_ConvertPtr(ST(4), &argp5,SWIGTYPE_p_dwg_field_name_type_offset, 0 |  0 );
    if (!SWIG_IsOK(res5)) {
      SWIG_exception_fail(SWIG_ArgError(res5), "in method '" "dwg_dynapi_common_utf8text" "', argument " "5"" of type '" "Dwg_DYNAPI_field *""'"); 
    }
    arg5 = (Dwg_DYNAPI_field *)(argp5);
    result = (bool)dwg_dynapi_common_utf8text(arg1,(char const *)arg2,arg3,arg4,arg5);
    ST(argvi) = SWIG_From_bool  SWIG_PERL_CALL_ARGS_1((bool)(result)); argvi++ ;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    
    
    
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_dynapi_header_set_value) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    char *arg2 = (char *) 0 ;
    void *arg3 = (void *) 0 ;
    bool arg4 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int res3 ;
    bool val4 ;
    int ecode4 = 0 ;
    int argvi = 0;
    bool result;
    dXSARGS;
    
    if ((items < 4) || (items > 4)) {
      SWIG_croak("Usage: dwg_dynapi_header_set_value(dwg,fieldname,value,is_utf8);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_dynapi_header_set_value" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_dynapi_header_set_value" "', argument " "2"" of type '" "char const *""'");
    }
    arg2 = (char *)(buf2);
    res3 = SWIG_ConvertPtr(ST(2),SWIG_as_voidptrptr(&arg3), 0, 0);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "dwg_dynapi_header_set_value" "', argument " "3"" of type '" "void const *""'"); 
    }
    ecode4 = SWIG_AsVal_bool SWIG_PERL_CALL_ARGS_2(ST(3), &val4);
    if (!SWIG_IsOK(ecode4)) {
      SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "dwg_dynapi_header_set_value" "', argument " "4"" of type '" "bool""'");
    } 
    arg4 = (bool)(val4);
    result = (bool)dwg_dynapi_header_set_value(arg1,(char const *)arg2,(void const *)arg3,arg4);
    ST(argvi) = SWIG_From_bool  SWIG_PERL_CALL_ARGS_1((bool)(result)); argvi++ ;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    
    
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_dynapi_entity_set_value) {
  {
    void *arg1 = (void *) 0 ;
    char *arg2 = (char *) 0 ;
    char *arg3 = (char *) 0 ;
    void *arg4 = (void *) 0 ;
    bool arg5 ;
    int res1 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int res3 ;
    char *buf3 = 0 ;
    int alloc3 = 0 ;
    int res4 ;
    bool val5 ;
    int ecode5 = 0 ;
    int argvi = 0;
    bool result;
    dXSARGS;
    
    if ((items < 5) || (items > 5)) {
      SWIG_croak("Usage: dwg_dynapi_entity_set_value(entity,dxfname,fieldname,value,is_utf8);");
    }
    res1 = SWIG_ConvertPtr(ST(0),SWIG_as_voidptrptr(&arg1), 0, 0);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_dynapi_entity_set_value" "', argument " "1"" of type '" "void *""'"); 
    }
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_dynapi_entity_set_value" "', argument " "2"" of type '" "char const *""'");
    }
    arg2 = (char *)(buf2);
    res3 = SWIG_AsCharPtrAndSize(ST(2), &buf3, NULL, &alloc3);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "dwg_dynapi_entity_set_value" "', argument " "3"" of type '" "char const *""'");
    }
    arg3 = (char *)(buf3);
    res4 = SWIG_ConvertPtr(ST(3),SWIG_as_voidptrptr(&arg4), 0, 0);
    if (!SWIG_IsOK(res4)) {
      SWIG_exception_fail(SWIG_ArgError(res4), "in method '" "dwg_dynapi_entity_set_value" "', argument " "4"" of type '" "void const *""'"); 
    }
    ecode5 = SWIG_AsVal_bool SWIG_PERL_CALL_ARGS_2(ST(4), &val5);
    if (!SWIG_IsOK(ecode5)) {
      SWIG_exception_fail(SWIG_ArgError(ecode5), "in method '" "dwg_dynapi_entity_set_value" "', argument " "5"" of type '" "bool""'");
    } 
    arg5 = (bool)(val5);
    result = (bool)dwg_dynapi_entity_set_value(arg1,(char const *)arg2,(char const *)arg3,(void const *)arg4,arg5);
    ST(argvi) = SWIG_From_bool  SWIG_PERL_CALL_ARGS_1((bool)(result)); argvi++ ;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    if (alloc3 == SWIG_NEWOBJ) free((char*)buf3);
    
    
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    if (alloc3 == SWIG_NEWOBJ) free((char*)buf3);
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_dynapi_common_set_value) {
  {
    void *arg1 = (void *) 0 ;
    char *arg2 = (char *) 0 ;
    void *arg3 = (void *) 0 ;
    bool arg4 ;
    int res1 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int res3 ;
    bool val4 ;
    int ecode4 = 0 ;
    int argvi = 0;
    bool result;
    dXSARGS;
    
    if ((items < 4) || (items > 4)) {
      SWIG_croak("Usage: dwg_dynapi_common_set_value(_obj,fieldname,value,is_utf8);");
    }
    res1 = SWIG_ConvertPtr(ST(0),SWIG_as_voidptrptr(&arg1), 0, 0);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_dynapi_common_set_value" "', argument " "1"" of type '" "void *""'"); 
    }
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_dynapi_common_set_value" "', argument " "2"" of type '" "char const *""'");
    }
    arg2 = (char *)(buf2);
    res3 = SWIG_ConvertPtr(ST(2),SWIG_as_voidptrptr(&arg3), 0, 0);
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "dwg_dynapi_common_set_value" "', argument " "3"" of type '" "void const *""'"); 
    }
    ecode4 = SWIG_AsVal_bool SWIG_PERL_CALL_ARGS_2(ST(3), &val4);
    if (!SWIG_IsOK(ecode4)) {
      SWIG_exception_fail(SWIG_ArgError(ecode4), "in method '" "dwg_dynapi_common_set_value" "', argument " "4"" of type '" "bool""'");
    } 
    arg4 = (bool)(val4);
    result = (bool)dwg_dynapi_common_set_value(arg1,(char const *)arg2,(void const *)arg3,arg4);
    ST(argvi) = SWIG_From_bool  SWIG_PERL_CALL_ARGS_1((bool)(result)); argvi++ ;
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    
    
    XSRETURN(argvi);
  fail:
    
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_dynapi_handle_name) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    Dwg_Object_Ref *arg2 = (Dwg_Object_Ref *) 0 ;
    int *arg3 = (int *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    void *argp3 = 0 ;
    int res3 = 0 ;
    int argvi = 0;
    char *result = 0 ;
    dXSARGS;
    
    if ((items < 3) || (items > 3)) {
      SWIG_croak("Usage: dwg_dynapi_handle_name(dwg,hdl,alloced);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_dynapi_handle_name" "', argument " "1"" of type '" "Dwg_Data const *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_dynapi_handle_name" "', argument " "2"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg2 = (Dwg_Object_Ref *)(argp2);
    res3 = SWIG_ConvertPtr(ST(2), &argp3,SWIGTYPE_p_int, 0 |  0 );
    if (!SWIG_IsOK(res3)) {
      SWIG_exception_fail(SWIG_ArgError(res3), "in method '" "dwg_dynapi_handle_name" "', argument " "3"" of type '" "int *""'"); 
    }
    arg3 = (int *)(argp3);
    result = (char *)dwg_dynapi_handle_name((struct _dwg_struct const *)arg1,arg2,arg3);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    
    
    XSRETURN(argvi);
  fail:
    
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_dynapi_header_field) {
  {
    char *arg1 = (char *) 0 ;
    int res1 ;
    char *buf1 = 0 ;
    int alloc1 = 0 ;
    int argvi = 0;
    Dwg_DYNAPI_field *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_dynapi_header_field(fieldname);");
    }
    res1 = SWIG_AsCharPtrAndSize(ST(0), &buf1, NULL, &alloc1);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_dynapi_header_field" "', argument " "1"" of type '" "char const *""'");
    }
    arg1 = (char *)(buf1);
    result = (Dwg_DYNAPI_field *)dwg_dynapi_header_field((char const *)arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_dwg_field_name_type_offset, 0 | SWIG_SHADOW); argvi++ ;
    if (alloc1 == SWIG_NEWOBJ) free((char*)buf1);
    XSRETURN(argvi);
  fail:
    if (alloc1 == SWIG_NEWOBJ) free((char*)buf1);
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_dynapi_entity_field) {
  {
    char *arg1 = (char *) 0 ;
    char *arg2 = (char *) 0 ;
    int res1 ;
    char *buf1 = 0 ;
    int alloc1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    Dwg_DYNAPI_field *result = 0 ;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dwg_dynapi_entity_field(name,fieldname);");
    }
    res1 = SWIG_AsCharPtrAndSize(ST(0), &buf1, NULL, &alloc1);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_dynapi_entity_field" "', argument " "1"" of type '" "char const *""'");
    }
    arg1 = (char *)(buf1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_dynapi_entity_field" "', argument " "2"" of type '" "char const *""'");
    }
    arg2 = (char *)(buf2);
    result = (Dwg_DYNAPI_field *)dwg_dynapi_entity_field((char const *)arg1,(char const *)arg2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_dwg_field_name_type_offset, 0 | SWIG_SHADOW); argvi++ ;
    if (alloc1 == SWIG_NEWOBJ) free((char*)buf1);
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    if (alloc1 == SWIG_NEWOBJ) free((char*)buf1);
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_dynapi_subclass_field) {
  {
    char *arg1 = (char *) 0 ;
    char *arg2 = (char *) 0 ;
    int res1 ;
    char *buf1 = 0 ;
    int alloc1 = 0 ;
    int res2 ;
    char *buf2 = 0 ;
    int alloc2 = 0 ;
    int argvi = 0;
    Dwg_DYNAPI_field *result = 0 ;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dwg_dynapi_subclass_field(name,fieldname);");
    }
    res1 = SWIG_AsCharPtrAndSize(ST(0), &buf1, NULL, &alloc1);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_dynapi_subclass_field" "', argument " "1"" of type '" "char const *""'");
    }
    arg1 = (char *)(buf1);
    res2 = SWIG_AsCharPtrAndSize(ST(1), &buf2, NULL, &alloc2);
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_dynapi_subclass_field" "', argument " "2"" of type '" "char const *""'");
    }
    arg2 = (char *)(buf2);
    result = (Dwg_DYNAPI_field *)dwg_dynapi_subclass_field((char const *)arg1,(char const *)arg2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_dwg_field_name_type_offset, 0 | SWIG_SHADOW); argvi++ ;
    if (alloc1 == SWIG_NEWOBJ) free((char*)buf1);
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    XSRETURN(argvi);
  fail:
    if (alloc1 == SWIG_NEWOBJ) free((char*)buf1);
    if (alloc2 == SWIG_NEWOBJ) free((char*)buf2);
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_dynapi_common_entity_field) {
  {
    char *arg1 = (char *) 0 ;
    int res1 ;
    char *buf1 = 0 ;
    int alloc1 = 0 ;
    int argvi = 0;
    Dwg_DYNAPI_field *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_dynapi_common_entity_field(fieldname);");
    }
    res1 = SWIG_AsCharPtrAndSize(ST(0), &buf1, NULL, &alloc1);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_dynapi_common_entity_field" "', argument " "1"" of type '" "char const *""'");
    }
    arg1 = (char *)(buf1);
    result = (Dwg_DYNAPI_field *)dwg_dynapi_common_entity_field((char const *)arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_dwg_field_name_type_offset, 0 | SWIG_SHADOW); argvi++ ;
    if (alloc1 == SWIG_NEWOBJ) free((char*)buf1);
    XSRETURN(argvi);
  fail:
    if (alloc1 == SWIG_NEWOBJ) free((char*)buf1);
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_dynapi_common_object_field) {
  {
    char *arg1 = (char *) 0 ;
    int res1 ;
    char *buf1 = 0 ;
    int alloc1 = 0 ;
    int argvi = 0;
    Dwg_DYNAPI_field *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_dynapi_common_object_field(fieldname);");
    }
    res1 = SWIG_AsCharPtrAndSize(ST(0), &buf1, NULL, &alloc1);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_dynapi_common_object_field" "', argument " "1"" of type '" "char const *""'");
    }
    arg1 = (char *)(buf1);
    result = (Dwg_DYNAPI_field *)dwg_dynapi_common_object_field((char const *)arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_dwg_field_name_type_offset, 0 | SWIG_SHADOW); argvi++ ;
    if (alloc1 == SWIG_NEWOBJ) free((char*)buf1);
    XSRETURN(argvi);
  fail:
    if (alloc1 == SWIG_NEWOBJ) free((char*)buf1);
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_dynapi_entity_fields) {
  {
    char *arg1 = (char *) 0 ;
    int res1 ;
    char *buf1 = 0 ;
    int alloc1 = 0 ;
    int argvi = 0;
    Dwg_DYNAPI_field *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_dynapi_entity_fields(name);");
    }
    res1 = SWIG_AsCharPtrAndSize(ST(0), &buf1, NULL, &alloc1);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_dynapi_entity_fields" "', argument " "1"" of type '" "char const *""'");
    }
    arg1 = (char *)(buf1);
    result = (Dwg_DYNAPI_field *)dwg_dynapi_entity_fields((char const *)arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_dwg_field_name_type_offset, 0 | SWIG_SHADOW); argvi++ ;
    if (alloc1 == SWIG_NEWOBJ) free((char*)buf1);
    XSRETURN(argvi);
  fail:
    if (alloc1 == SWIG_NEWOBJ) free((char*)buf1);
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_dynapi_common_entity_fields) {
  {
    int argvi = 0;
    Dwg_DYNAPI_field *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: dwg_dynapi_common_entity_fields();");
    }
    result = (Dwg_DYNAPI_field *)dwg_dynapi_common_entity_fields();
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_dwg_field_name_type_offset, 0 | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_dynapi_common_object_fields) {
  {
    int argvi = 0;
    Dwg_DYNAPI_field *result = 0 ;
    dXSARGS;
    
    if ((items < 0) || (items > 0)) {
      SWIG_croak("Usage: dwg_dynapi_common_object_fields();");
    }
    result = (Dwg_DYNAPI_field *)dwg_dynapi_common_object_fields();
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_dwg_field_name_type_offset, 0 | SWIG_SHADOW); argvi++ ;
    XSRETURN(argvi);
  fail:
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_dynapi_subclass_fields) {
  {
    char *arg1 = (char *) 0 ;
    int res1 ;
    char *buf1 = 0 ;
    int alloc1 = 0 ;
    int argvi = 0;
    Dwg_DYNAPI_field *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_dynapi_subclass_fields(name);");
    }
    res1 = SWIG_AsCharPtrAndSize(ST(0), &buf1, NULL, &alloc1);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_dynapi_subclass_fields" "', argument " "1"" of type '" "char const *""'");
    }
    arg1 = (char *)(buf1);
    result = (Dwg_DYNAPI_field *)dwg_dynapi_subclass_fields((char const *)arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_dwg_field_name_type_offset, 0 | SWIG_SHADOW); argvi++ ;
    if (alloc1 == SWIG_NEWOBJ) free((char*)buf1);
    XSRETURN(argvi);
  fail:
    if (alloc1 == SWIG_NEWOBJ) free((char*)buf1);
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_dynapi_fields_size) {
  {
    char *arg1 = (char *) 0 ;
    int res1 ;
    char *buf1 = 0 ;
    int alloc1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_dynapi_fields_size(name);");
    }
    res1 = SWIG_AsCharPtrAndSize(ST(0), &buf1, NULL, &alloc1);
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_dynapi_fields_size" "', argument " "1"" of type '" "char const *""'");
    }
    arg1 = (char *)(buf1);
    result = (int)dwg_dynapi_fields_size((char const *)arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    if (alloc1 == SWIG_NEWOBJ) free((char*)buf1);
    XSRETURN(argvi);
  fail:
    if (alloc1 == SWIG_NEWOBJ) free((char*)buf1);
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_tablectrl_get_entries) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    int *arg2 = (int *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    Dwg_Object_Ref **result = 0 ;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dwg_object_tablectrl_get_entries(obj,error);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_tablectrl_get_entries" "', argument " "1"" of type '" "Dwg_Object const *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_int, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_object_tablectrl_get_entries" "', argument " "2"" of type '" "int *""'"); 
    }
    arg2 = (int *)(argp2);
    result = (Dwg_Object_Ref **)dwg_object_tablectrl_get_entries((struct _dwg_object const *)arg1,arg2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ref, 0 | 0); argvi++ ;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_ref_get_table_name) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    int *arg2 = (int *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    char *result = 0 ;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dwg_ref_get_table_name(ref,error);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_ref_get_table_name" "', argument " "1"" of type '" "Dwg_Object_Ref const *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_int, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_ref_get_table_name" "', argument " "2"" of type '" "int *""'"); 
    }
    arg2 = (int *)(argp2);
    result = (char *)dwg_ref_get_table_name((struct _dwg_object_ref const *)arg1,arg2);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_ent_get_layer_name) {
  {
    dwg_obj_ent *arg1 = (dwg_obj_ent *) 0 ;
    int *arg2 = (int *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    char *result = 0 ;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dwg_ent_get_layer_name(ent,error);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_dwg_obj_ent, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_ent_get_layer_name" "', argument " "1"" of type '" "dwg_obj_ent const *""'"); 
    }
    arg1 = (dwg_obj_ent *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_int, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_ent_get_layer_name" "', argument " "2"" of type '" "int *""'"); 
    }
    arg2 = (int *)(argp2);
    result = (char *)dwg_ent_get_layer_name((dwg_obj_ent const *)arg1,arg2);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_obj_layer_get_name) {
  {
    dwg_obj_layer *arg1 = (dwg_obj_layer *) 0 ;
    int *arg2 = (int *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    char *result = 0 ;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dwg_obj_layer_get_name(layer,error);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_dwg_obj_layer, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_obj_layer_get_name" "', argument " "1"" of type '" "dwg_obj_layer const *""'"); 
    }
    arg1 = (dwg_obj_layer *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_int, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_obj_layer_get_name" "', argument " "2"" of type '" "int *""'"); 
    }
    arg2 = (int *)(argp2);
    result = (char *)dwg_obj_layer_get_name((dwg_obj_layer const *)arg1,arg2);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_obj_table_get_name) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    int *arg2 = (int *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    char *result = 0 ;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dwg_obj_table_get_name(obj,error);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_obj_table_get_name" "', argument " "1"" of type '" "Dwg_Object const *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_int, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_obj_table_get_name" "', argument " "2"" of type '" "int *""'"); 
    }
    arg2 = (int *)(argp2);
    result = (char *)dwg_obj_table_get_name((struct _dwg_object const *)arg1,arg2);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_ent_to_object) {
  {
    dwg_obj_ent *arg1 = (dwg_obj_ent *) 0 ;
    int *arg2 = (int *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    Dwg_Object *result = 0 ;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dwg_ent_to_object(obj,error);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_dwg_obj_ent, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_ent_to_object" "', argument " "1"" of type '" "dwg_obj_ent const *""'"); 
    }
    arg1 = (dwg_obj_ent *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_int, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_ent_to_object" "', argument " "2"" of type '" "int *""'"); 
    }
    arg2 = (int *)(argp2);
    result = (Dwg_Object *)dwg_ent_to_object((dwg_obj_ent const *)arg1,arg2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object, 0 | SWIG_SHADOW); argvi++ ;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_object) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    int *arg2 = (int *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    Dwg_Object_Object *result = 0 ;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dwg_object_to_object(obj,error);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_object" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_int, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_object_to_object" "', argument " "2"" of type '" "int *""'"); 
    }
    arg2 = (int *)(argp2);
    result = (Dwg_Object_Object *)dwg_object_to_object(arg1,arg2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_object, 0 | SWIG_SHADOW); argvi++ ;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_obj_obj_to_object) {
  {
    Dwg_Object_Object *arg1 = (Dwg_Object_Object *) 0 ;
    int *arg2 = (int *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    Dwg_Object *result = 0 ;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dwg_obj_obj_to_object(obj,error);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_obj_obj_to_object" "', argument " "1"" of type '" "Dwg_Object_Object const *""'"); 
    }
    arg1 = (Dwg_Object_Object *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_int, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_obj_obj_to_object" "', argument " "2"" of type '" "int *""'"); 
    }
    arg2 = (int *)(argp2);
    result = (Dwg_Object *)dwg_obj_obj_to_object((struct _dwg_object_object const *)arg1,arg2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object, 0 | SWIG_SHADOW); argvi++ ;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_entity) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    int *arg2 = (int *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    Dwg_Object_Entity *result = 0 ;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dwg_object_to_entity(obj,error);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_entity" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_int, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_object_to_entity" "', argument " "2"" of type '" "int *""'"); 
    }
    arg2 = (int *)(argp2);
    result = (Dwg_Object_Entity *)dwg_object_to_entity(arg1,arg2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_entity, 0 | SWIG_SHADOW); argvi++ ;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_get_type) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_get_type(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_get_type" "', argument " "1"" of type '" "Dwg_Object const *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_object_get_type((struct _dwg_object const *)arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_get_fixedtype) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_get_fixedtype(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_get_fixedtype" "', argument " "1"" of type '" "Dwg_Object const *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (int)dwg_object_get_fixedtype((struct _dwg_object const *)arg1);
    ST(argvi) = SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_get_dxfname) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    char *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_get_dxfname(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_get_dxfname" "', argument " "1"" of type '" "Dwg_Object const *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (char *)dwg_object_get_dxfname((struct _dwg_object const *)arg1);
    ST(argvi) = SWIG_FromCharPtr((const char *)result); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_ref_get_absref) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    int *arg2 = (int *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    BITCODE_BL result;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dwg_ref_get_absref(ref,error);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_ref_get_absref" "', argument " "1"" of type '" "Dwg_Object_Ref const *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_int, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_ref_get_absref" "', argument " "2"" of type '" "int *""'"); 
    }
    arg2 = (int *)(argp2);
    result = (BITCODE_BL)dwg_ref_get_absref((struct _dwg_object_ref const *)arg1,arg2);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_ref_get_object) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    int *arg2 = (int *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    Dwg_Object *result = 0 ;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dwg_ref_get_object(ref,error);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_ref_get_object" "', argument " "1"" of type '" "Dwg_Object_Ref const *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_int, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_ref_get_object" "', argument " "2"" of type '" "int *""'"); 
    }
    arg2 = (int *)(argp2);
    result = (Dwg_Object *)dwg_ref_get_object((struct _dwg_object_ref const *)arg1,arg2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object, 0 | SWIG_SHADOW); argvi++ ;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_absref_get_object) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    Dwg_Object *result = 0 ;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dwg_absref_get_object(dwg,absref);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_absref_get_object" "', argument " "1"" of type '" "Dwg_Data const *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "dwg_absref_get_object" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    result = (Dwg_Object *)dwg_absref_get_object((struct _dwg_struct const *)arg1,arg2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object, 0 | SWIG_SHADOW); argvi++ ;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_get_num_classes) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    unsigned int result;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_get_num_classes(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_get_num_classes" "', argument " "1"" of type '" "Dwg_Data const *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (unsigned int)dwg_get_num_classes((struct _dwg_struct const *)arg1);
    ST(argvi) = SWIG_From_unsigned_SS_int  SWIG_PERL_CALL_ARGS_1((unsigned int)(result)); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_get_class) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    unsigned int arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    Dwg_Class *result = 0 ;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dwg_get_class(dwg,index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_get_class" "', argument " "1"" of type '" "Dwg_Data const *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "dwg_get_class" "', argument " "2"" of type '" "unsigned int""'");
    } 
    arg2 = (unsigned int)(val2);
    result = (Dwg_Class *)dwg_get_class((struct _dwg_struct const *)arg1,arg2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_class, 0 | SWIG_SHADOW); argvi++ ;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_get_object) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    BITCODE_BL arg2 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    unsigned int val2 ;
    int ecode2 = 0 ;
    int argvi = 0;
    Dwg_Object *result = 0 ;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dwg_get_object(dwg,index);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_get_object" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    ecode2 = SWIG_AsVal_unsigned_SS_int SWIG_PERL_CALL_ARGS_2(ST(1), &val2);
    if (!SWIG_IsOK(ecode2)) {
      SWIG_exception_fail(SWIG_ArgError(ecode2), "in method '" "dwg_get_object" "', argument " "2"" of type '" "BITCODE_BL""'");
    } 
    arg2 = (BITCODE_BL)(val2);
    result = (Dwg_Object *)dwg_get_object(arg1,arg2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object, 0 | SWIG_SHADOW); argvi++ ;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_obj_generic_to_object) {
  {
    dwg_obj_generic *arg1 = (dwg_obj_generic *) 0 ;
    int *arg2 = (int *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dwg_object *result = 0 ;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dwg_obj_generic_to_object(obj,error);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_dwg_obj_generic, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_obj_generic_to_object" "', argument " "1"" of type '" "dwg_obj_generic const *""'"); 
    }
    arg1 = (dwg_obj_generic *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_int, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_obj_generic_to_object" "', argument " "2"" of type '" "int *""'"); 
    }
    arg2 = (int *)(argp2);
    result = (dwg_object *)dwg_obj_generic_to_object((dwg_obj_generic const *)arg1,arg2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_dwg_object, 0 | 0); argvi++ ;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_obj_generic_parent) {
  {
    dwg_obj_generic *arg1 = (dwg_obj_generic *) 0 ;
    int *arg2 = (int *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    void *argp2 = 0 ;
    int res2 = 0 ;
    int argvi = 0;
    dwg_obj_obj *result = 0 ;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) {
      SWIG_croak("Usage: dwg_obj_generic_parent(obj,error);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p_dwg_obj_generic, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_obj_generic_parent" "', argument " "1"" of type '" "dwg_obj_generic const *""'"); 
    }
    arg1 = (dwg_obj_generic *)(argp1);
    res2 = SWIG_ConvertPtr(ST(1), &argp2,SWIGTYPE_p_int, 0 |  0 );
    if (!SWIG_IsOK(res2)) {
      SWIG_exception_fail(SWIG_ArgError(res2), "in method '" "dwg_obj_generic_parent" "', argument " "2"" of type '" "int *""'"); 
    }
    arg2 = (int *)(argp2);
    result = (dwg_obj_obj *)dwg_obj_generic_parent((dwg_obj_generic const *)arg1,arg2);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_dwg_obj_obj, 0 | 0); argvi++ ;
    
    
    XSRETURN(argvi);
  fail:
    
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_DIMENSION) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    struct dwg_entity_DIMENSION_common *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_DIMENSION(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_DIMENSION" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (struct dwg_entity_DIMENSION_common *)dwg_object_to_DIMENSION(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_dwg_entity_DIMENSION_common, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall__3DFACE) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity__3DFACE **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall__3DFACE(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall__3DFACE" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity__3DFACE **)dwg_getall__3DFACE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_3DFACE, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall__3DLINE) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity__3DLINE **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall__3DLINE(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall__3DLINE" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity__3DLINE **)dwg_getall__3DLINE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_3DLINE, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall__3DSOLID) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity__3DSOLID **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall__3DSOLID(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall__3DSOLID" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity__3DSOLID **)dwg_getall__3DSOLID(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_3DSOLID, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ARC) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_ARC **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ARC(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ARC" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_ARC **)dwg_getall_ARC(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_ARC, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ATTDEF) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_ATTDEF **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ATTDEF(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ATTDEF" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_ATTDEF **)dwg_getall_ATTDEF(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_ATTDEF, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ATTRIB) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_ATTRIB **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ATTRIB(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ATTRIB" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_ATTRIB **)dwg_getall_ATTRIB(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_ATTRIB, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCK) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_BLOCK **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCK(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCK" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_BLOCK **)dwg_getall_BLOCK(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_BLOCK, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BODY) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_BODY **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BODY(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BODY" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_BODY **)dwg_getall_BODY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_3DSOLID, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_CAMERA) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_CAMERA **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_CAMERA(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_CAMERA" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_CAMERA **)dwg_getall_CAMERA(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_CAMERA, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_CIRCLE) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_CIRCLE **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_CIRCLE(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_CIRCLE" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_CIRCLE **)dwg_getall_CIRCLE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_CIRCLE, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_DGNUNDERLAY) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_DGNUNDERLAY **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_DGNUNDERLAY(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_DGNUNDERLAY" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_DGNUNDERLAY **)dwg_getall_DGNUNDERLAY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_abstractentity_UNDERLAY, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_DIMENSION_ALIGNED) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_DIMENSION_ALIGNED **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_DIMENSION_ALIGNED(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_DIMENSION_ALIGNED" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_DIMENSION_ALIGNED **)dwg_getall_DIMENSION_ALIGNED(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_DIMENSION_ALIGNED, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_DIMENSION_ANG2LN) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_DIMENSION_ANG2LN **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_DIMENSION_ANG2LN(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_DIMENSION_ANG2LN" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_DIMENSION_ANG2LN **)dwg_getall_DIMENSION_ANG2LN(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_DIMENSION_ANG2LN, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_DIMENSION_ANG3PT) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_DIMENSION_ANG3PT **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_DIMENSION_ANG3PT(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_DIMENSION_ANG3PT" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_DIMENSION_ANG3PT **)dwg_getall_DIMENSION_ANG3PT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_DIMENSION_ANG3PT, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_DIMENSION_DIAMETER) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_DIMENSION_DIAMETER **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_DIMENSION_DIAMETER(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_DIMENSION_DIAMETER" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_DIMENSION_DIAMETER **)dwg_getall_DIMENSION_DIAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_DIMENSION_DIAMETER, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_DIMENSION_LINEAR) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_DIMENSION_LINEAR **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_DIMENSION_LINEAR(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_DIMENSION_LINEAR" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_DIMENSION_LINEAR **)dwg_getall_DIMENSION_LINEAR(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_DIMENSION_LINEAR, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_DIMENSION_ORDINATE) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_DIMENSION_ORDINATE **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_DIMENSION_ORDINATE(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_DIMENSION_ORDINATE" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_DIMENSION_ORDINATE **)dwg_getall_DIMENSION_ORDINATE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_DIMENSION_ORDINATE, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_DIMENSION_RADIUS) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_DIMENSION_RADIUS **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_DIMENSION_RADIUS(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_DIMENSION_RADIUS" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_DIMENSION_RADIUS **)dwg_getall_DIMENSION_RADIUS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_DIMENSION_RADIUS, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_DWFUNDERLAY) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_DWFUNDERLAY **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_DWFUNDERLAY(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_DWFUNDERLAY" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_DWFUNDERLAY **)dwg_getall_DWFUNDERLAY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_abstractentity_UNDERLAY, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ELLIPSE) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_ELLIPSE **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ELLIPSE(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ELLIPSE" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_ELLIPSE **)dwg_getall_ELLIPSE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_ELLIPSE, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ENDBLK) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_ENDBLK **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ENDBLK(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ENDBLK" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_ENDBLK **)dwg_getall_ENDBLK(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_ENDBLK, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ENDREP) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_ENDREP **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ENDREP(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ENDREP" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_ENDREP **)dwg_getall_ENDREP(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_ENDREP, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_HATCH) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_HATCH **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_HATCH(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_HATCH" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_HATCH **)dwg_getall_HATCH(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_HATCH, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_IMAGE) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_IMAGE **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_IMAGE(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_IMAGE" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_IMAGE **)dwg_getall_IMAGE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_IMAGE, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_INSERT) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_INSERT **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_INSERT(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_INSERT" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_INSERT **)dwg_getall_INSERT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_INSERT, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_JUMP) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_JUMP **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_JUMP(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_JUMP" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_JUMP **)dwg_getall_JUMP(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_JUMP, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_LEADER) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_LEADER **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_LEADER(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_LEADER" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_LEADER **)dwg_getall_LEADER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_LEADER, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_LIGHT) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_LIGHT **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_LIGHT(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_LIGHT" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_LIGHT **)dwg_getall_LIGHT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_LIGHT, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_LINE) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_LINE **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_LINE(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_LINE" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_LINE **)dwg_getall_LINE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_LINE, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_LOAD) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_LOAD **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_LOAD(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_LOAD" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_LOAD **)dwg_getall_LOAD(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_LOAD, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_LWPOLYLINE) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_LWPOLYLINE **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_LWPOLYLINE(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_LWPOLYLINE" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_LWPOLYLINE **)dwg_getall_LWPOLYLINE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_LWPOLYLINE, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_MESH) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_MESH **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_MESH(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_MESH" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_MESH **)dwg_getall_MESH(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_MESH, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_MINSERT) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_MINSERT **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_MINSERT(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_MINSERT" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_MINSERT **)dwg_getall_MINSERT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_MINSERT, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_MLINE) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_MLINE **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_MLINE(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_MLINE" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_MLINE **)dwg_getall_MLINE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_MLINE, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_MTEXT) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_MTEXT **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_MTEXT(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_MTEXT" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_MTEXT **)dwg_getall_MTEXT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_MTEXT, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_MULTILEADER) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_MULTILEADER **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_MULTILEADER(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_MULTILEADER" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_MULTILEADER **)dwg_getall_MULTILEADER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_MULTILEADER, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_OLE2FRAME) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_OLE2FRAME **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_OLE2FRAME(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_OLE2FRAME" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_OLE2FRAME **)dwg_getall_OLE2FRAME(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_OLE2FRAME, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_OLEFRAME) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_OLEFRAME **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_OLEFRAME(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_OLEFRAME" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_OLEFRAME **)dwg_getall_OLEFRAME(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_OLEFRAME, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_PDFUNDERLAY) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_PDFUNDERLAY **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_PDFUNDERLAY(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_PDFUNDERLAY" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_PDFUNDERLAY **)dwg_getall_PDFUNDERLAY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_abstractentity_UNDERLAY, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_POINT) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_POINT **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_POINT(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_POINT" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_POINT **)dwg_getall_POINT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_POINT, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_POLYLINE_2D) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_POLYLINE_2D **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_POLYLINE_2D(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_POLYLINE_2D" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_POLYLINE_2D **)dwg_getall_POLYLINE_2D(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_POLYLINE_2D, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_POLYLINE_3D) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_POLYLINE_3D **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_POLYLINE_3D(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_POLYLINE_3D" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_POLYLINE_3D **)dwg_getall_POLYLINE_3D(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_POLYLINE_3D, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_POLYLINE_MESH) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_POLYLINE_MESH **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_POLYLINE_MESH(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_POLYLINE_MESH" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_POLYLINE_MESH **)dwg_getall_POLYLINE_MESH(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_POLYLINE_MESH, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_POLYLINE_PFACE) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_POLYLINE_PFACE **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_POLYLINE_PFACE(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_POLYLINE_PFACE" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_POLYLINE_PFACE **)dwg_getall_POLYLINE_PFACE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_POLYLINE_PFACE, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_PROXY_ENTITY) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_PROXY_ENTITY **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_PROXY_ENTITY(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_PROXY_ENTITY" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_PROXY_ENTITY **)dwg_getall_PROXY_ENTITY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_PROXY_ENTITY, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_RAY) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_RAY **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_RAY(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_RAY" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_RAY **)dwg_getall_RAY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_RAY, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_REGION) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_REGION **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_REGION(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_REGION" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_REGION **)dwg_getall_REGION(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_3DSOLID, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_REPEAT) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_REPEAT **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_REPEAT(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_REPEAT" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_REPEAT **)dwg_getall_REPEAT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_REPEAT, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_SECTIONOBJECT) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_SECTIONOBJECT **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_SECTIONOBJECT(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_SECTIONOBJECT" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_SECTIONOBJECT **)dwg_getall_SECTIONOBJECT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_SECTIONOBJECT, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_SEQEND) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_SEQEND **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_SEQEND(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_SEQEND" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_SEQEND **)dwg_getall_SEQEND(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_SEQEND, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_SHAPE) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_SHAPE **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_SHAPE(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_SHAPE" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_SHAPE **)dwg_getall_SHAPE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_SHAPE, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_SOLID) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_SOLID **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_SOLID(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_SOLID" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_SOLID **)dwg_getall_SOLID(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_SOLID, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_SPLINE) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_SPLINE **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_SPLINE(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_SPLINE" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_SPLINE **)dwg_getall_SPLINE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_SPLINE, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_TEXT) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_TEXT **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_TEXT(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_TEXT" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_TEXT **)dwg_getall_TEXT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_TEXT, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_TOLERANCE) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_TOLERANCE **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_TOLERANCE(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_TOLERANCE" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_TOLERANCE **)dwg_getall_TOLERANCE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_TOLERANCE, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_TRACE) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_TRACE **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_TRACE(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_TRACE" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_TRACE **)dwg_getall_TRACE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_TRACE, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_UNKNOWN_ENT) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_UNKNOWN_ENT **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_UNKNOWN_ENT(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_UNKNOWN_ENT" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_UNKNOWN_ENT **)dwg_getall_UNKNOWN_ENT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_UNKNOWN_ENT, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_VERTEX_2D) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_VERTEX_2D **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_VERTEX_2D(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_VERTEX_2D" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_VERTEX_2D **)dwg_getall_VERTEX_2D(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_VERTEX_2D, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_VERTEX_3D) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_VERTEX_3D **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_VERTEX_3D(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_VERTEX_3D" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_VERTEX_3D **)dwg_getall_VERTEX_3D(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_VERTEX_3D, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_VERTEX_MESH) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_VERTEX_MESH **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_VERTEX_MESH(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_VERTEX_MESH" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_VERTEX_MESH **)dwg_getall_VERTEX_MESH(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_VERTEX_3D, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_VERTEX_PFACE) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_VERTEX_PFACE **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_VERTEX_PFACE(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_VERTEX_PFACE" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_VERTEX_PFACE **)dwg_getall_VERTEX_PFACE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_VERTEX_3D, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_VERTEX_PFACE_FACE) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_VERTEX_PFACE_FACE **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_VERTEX_PFACE_FACE(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_VERTEX_PFACE_FACE" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_VERTEX_PFACE_FACE **)dwg_getall_VERTEX_PFACE_FACE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_VERTEX_PFACE_FACE, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_VIEWPORT) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_VIEWPORT **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_VIEWPORT(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_VIEWPORT" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_VIEWPORT **)dwg_getall_VIEWPORT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_VIEWPORT, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_WIPEOUT) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_WIPEOUT **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_WIPEOUT(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_WIPEOUT" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_WIPEOUT **)dwg_getall_WIPEOUT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_WIPEOUT, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_XLINE) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_XLINE **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_XLINE(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_XLINE" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_XLINE **)dwg_getall_XLINE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_RAY, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ARC_DIMENSION) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_ARC_DIMENSION **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ARC_DIMENSION(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ARC_DIMENSION" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_ARC_DIMENSION **)dwg_getall_ARC_DIMENSION(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_ARC_DIMENSION, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_HELIX) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_HELIX **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_HELIX(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_HELIX" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_HELIX **)dwg_getall_HELIX(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_HELIX, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_LARGE_RADIAL_DIMENSION) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_LARGE_RADIAL_DIMENSION **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_LARGE_RADIAL_DIMENSION(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_LARGE_RADIAL_DIMENSION" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_LARGE_RADIAL_DIMENSION **)dwg_getall_LARGE_RADIAL_DIMENSION(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_LAYOUTPRINTCONFIG) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_LAYOUTPRINTCONFIG **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_LAYOUTPRINTCONFIG(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_LAYOUTPRINTCONFIG" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_LAYOUTPRINTCONFIG **)dwg_getall_LAYOUTPRINTCONFIG(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_LAYOUTPRINTCONFIG, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_PLANESURFACE) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_PLANESURFACE **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_PLANESURFACE(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_PLANESURFACE" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_PLANESURFACE **)dwg_getall_PLANESURFACE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_PLANESURFACE, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_POINTCLOUD) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_POINTCLOUD **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_POINTCLOUD(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_POINTCLOUD" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_POINTCLOUD **)dwg_getall_POINTCLOUD(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_POINTCLOUD, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_POINTCLOUDEX) {
  {
    Dwg_Object_Ref *arg1 = (Dwg_Object_Ref *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_POINTCLOUDEX **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_POINTCLOUDEX(hdr);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object_ref, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_POINTCLOUDEX" "', argument " "1"" of type '" "Dwg_Object_Ref *""'"); 
    }
    arg1 = (Dwg_Object_Ref *)(argp1);
    result = (Dwg_Entity_POINTCLOUDEX **)dwg_getall_POINTCLOUDEX(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_entity_POINTCLOUDEX, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ACSH_BOOLEAN_CLASS) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_BOOLEAN_CLASS **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ACSH_BOOLEAN_CLASS(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ACSH_BOOLEAN_CLASS" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ACSH_BOOLEAN_CLASS **)dwg_getall_ACSH_BOOLEAN_CLASS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ACSH_BOOLEAN_CLASS, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ACSH_BOX_CLASS) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_BOX_CLASS **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ACSH_BOX_CLASS(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ACSH_BOX_CLASS" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ACSH_BOX_CLASS **)dwg_getall_ACSH_BOX_CLASS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ACSH_BOX_CLASS, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ACSH_CONE_CLASS) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_CONE_CLASS **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ACSH_CONE_CLASS(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ACSH_CONE_CLASS" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ACSH_CONE_CLASS **)dwg_getall_ACSH_CONE_CLASS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ACSH_CONE_CLASS, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ACSH_CYLINDER_CLASS) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_CYLINDER_CLASS **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ACSH_CYLINDER_CLASS(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ACSH_CYLINDER_CLASS" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ACSH_CYLINDER_CLASS **)dwg_getall_ACSH_CYLINDER_CLASS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ACSH_CYLINDER_CLASS, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ACSH_FILLET_CLASS) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_FILLET_CLASS **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ACSH_FILLET_CLASS(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ACSH_FILLET_CLASS" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ACSH_FILLET_CLASS **)dwg_getall_ACSH_FILLET_CLASS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ACSH_FILLET_CLASS, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ACSH_HISTORY_CLASS) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_HISTORY_CLASS **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ACSH_HISTORY_CLASS(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ACSH_HISTORY_CLASS" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ACSH_HISTORY_CLASS **)dwg_getall_ACSH_HISTORY_CLASS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ACSH_HISTORY_CLASS, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ACSH_SPHERE_CLASS) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_SPHERE_CLASS **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ACSH_SPHERE_CLASS(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ACSH_SPHERE_CLASS" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ACSH_SPHERE_CLASS **)dwg_getall_ACSH_SPHERE_CLASS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ACSH_SPHERE_CLASS, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ACSH_TORUS_CLASS) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_TORUS_CLASS **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ACSH_TORUS_CLASS(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ACSH_TORUS_CLASS" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ACSH_TORUS_CLASS **)dwg_getall_ACSH_TORUS_CLASS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ACSH_TORUS_CLASS, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ACSH_WEDGE_CLASS) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_WEDGE_CLASS **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ACSH_WEDGE_CLASS(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ACSH_WEDGE_CLASS" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ACSH_WEDGE_CLASS **)dwg_getall_ACSH_WEDGE_CLASS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ACSH_WEDGE_CLASS, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_APPID) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_APPID **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_APPID(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_APPID" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_APPID **)dwg_getall_APPID(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_APPID, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_APPID_CONTROL) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_APPID_CONTROL **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_APPID_CONTROL(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_APPID_CONTROL" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_APPID_CONTROL **)dwg_getall_APPID_CONTROL(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_APPID_CONTROL, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ASSOCGEOMDEPENDENCY) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCGEOMDEPENDENCY **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ASSOCGEOMDEPENDENCY(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ASSOCGEOMDEPENDENCY" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ASSOCGEOMDEPENDENCY **)dwg_getall_ASSOCGEOMDEPENDENCY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ASSOCGEOMDEPENDENCY, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ASSOCNETWORK) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCNETWORK **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ASSOCNETWORK(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ASSOCNETWORK" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ASSOCNETWORK **)dwg_getall_ASSOCNETWORK(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ASSOCNETWORK, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKALIGNMENTGRIP) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKALIGNMENTGRIP **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKALIGNMENTGRIP(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKALIGNMENTGRIP" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKALIGNMENTGRIP **)dwg_getall_BLOCKALIGNMENTGRIP(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKALIGNMENTGRIP, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKALIGNMENTPARAMETER) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKALIGNMENTPARAMETER **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKALIGNMENTPARAMETER(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKALIGNMENTPARAMETER" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKALIGNMENTPARAMETER **)dwg_getall_BLOCKALIGNMENTPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKBASEPOINTPARAMETER) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKBASEPOINTPARAMETER **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKBASEPOINTPARAMETER(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKBASEPOINTPARAMETER" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKBASEPOINTPARAMETER **)dwg_getall_BLOCKBASEPOINTPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKBASEPOINTPARAMETER, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKFLIPACTION) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKFLIPACTION **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKFLIPACTION(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKFLIPACTION" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKFLIPACTION **)dwg_getall_BLOCKFLIPACTION(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKFLIPACTION, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKFLIPGRIP) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKFLIPGRIP **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKFLIPGRIP(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKFLIPGRIP" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKFLIPGRIP **)dwg_getall_BLOCKFLIPGRIP(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKFLIPGRIP, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKFLIPPARAMETER) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKFLIPPARAMETER **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKFLIPPARAMETER(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKFLIPPARAMETER" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKFLIPPARAMETER **)dwg_getall_BLOCKFLIPPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKFLIPPARAMETER, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKGRIPLOCATIONCOMPONENT) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKGRIPLOCATIONCOMPONENT(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKGRIPLOCATIONCOMPONENT" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT **)dwg_getall_BLOCKGRIPLOCATIONCOMPONENT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKGRIPLOCATIONCOMPONENT, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKLINEARGRIP) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKLINEARGRIP **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKLINEARGRIP(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKLINEARGRIP" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKLINEARGRIP **)dwg_getall_BLOCKLINEARGRIP(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKLINEARGRIP, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKLOOKUPGRIP) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKLOOKUPGRIP **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKLOOKUPGRIP(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKLOOKUPGRIP" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKLOOKUPGRIP **)dwg_getall_BLOCKLOOKUPGRIP(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKLOOKUPGRIP, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKMOVEACTION) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKMOVEACTION **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKMOVEACTION(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKMOVEACTION" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKMOVEACTION **)dwg_getall_BLOCKMOVEACTION(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKMOVEACTION, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKROTATEACTION) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKROTATEACTION **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKROTATEACTION(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKROTATEACTION" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKROTATEACTION **)dwg_getall_BLOCKROTATEACTION(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKROTATEACTION, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKROTATIONGRIP) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKROTATIONGRIP **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKROTATIONGRIP(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKROTATIONGRIP" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKROTATIONGRIP **)dwg_getall_BLOCKROTATIONGRIP(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKROTATIONGRIP, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKSCALEACTION) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKSCALEACTION **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKSCALEACTION(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKSCALEACTION" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKSCALEACTION **)dwg_getall_BLOCKSCALEACTION(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKSCALEACTION, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKVISIBILITYGRIP) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKVISIBILITYGRIP **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKVISIBILITYGRIP(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKVISIBILITYGRIP" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKVISIBILITYGRIP **)dwg_getall_BLOCKVISIBILITYGRIP(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKVISIBILITYGRIP, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCK_CONTROL) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCK_CONTROL **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCK_CONTROL(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCK_CONTROL" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCK_CONTROL **)dwg_getall_BLOCK_CONTROL(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCK_CONTROL, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCK_HEADER) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCK_HEADER **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCK_HEADER(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCK_HEADER" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCK_HEADER **)dwg_getall_BLOCK_HEADER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCK_HEADER, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_CELLSTYLEMAP) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_CELLSTYLEMAP **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_CELLSTYLEMAP(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_CELLSTYLEMAP" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_CELLSTYLEMAP **)dwg_getall_CELLSTYLEMAP(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_CELLSTYLEMAP, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_DETAILVIEWSTYLE) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DETAILVIEWSTYLE **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_DETAILVIEWSTYLE(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_DETAILVIEWSTYLE" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_DETAILVIEWSTYLE **)dwg_getall_DETAILVIEWSTYLE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_DETAILVIEWSTYLE, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_DICTIONARY) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DICTIONARY **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_DICTIONARY(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_DICTIONARY" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_DICTIONARY **)dwg_getall_DICTIONARY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_DICTIONARY, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_DICTIONARYVAR) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DICTIONARYVAR **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_DICTIONARYVAR(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_DICTIONARYVAR" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_DICTIONARYVAR **)dwg_getall_DICTIONARYVAR(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_DICTIONARYVAR, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_DICTIONARYWDFLT) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DICTIONARYWDFLT **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_DICTIONARYWDFLT(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_DICTIONARYWDFLT" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_DICTIONARYWDFLT **)dwg_getall_DICTIONARYWDFLT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_DICTIONARYWDFLT, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_DIMSTYLE) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DIMSTYLE **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_DIMSTYLE(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_DIMSTYLE" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_DIMSTYLE **)dwg_getall_DIMSTYLE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_DIMSTYLE, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_DIMSTYLE_CONTROL) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DIMSTYLE_CONTROL **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_DIMSTYLE_CONTROL(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_DIMSTYLE_CONTROL" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_DIMSTYLE_CONTROL **)dwg_getall_DIMSTYLE_CONTROL(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_DIMSTYLE_CONTROL, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_DUMMY) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DUMMY **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_DUMMY(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_DUMMY" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_DUMMY **)dwg_getall_DUMMY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_DUMMY, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_DYNAMICBLOCKPURGEPREVENTER) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DYNAMICBLOCKPURGEPREVENTER **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_DYNAMICBLOCKPURGEPREVENTER(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_DYNAMICBLOCKPURGEPREVENTER" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_DYNAMICBLOCKPURGEPREVENTER **)dwg_getall_DYNAMICBLOCKPURGEPREVENTER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_DYNAMICBLOCKPURGEPREVENTER, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_FIELD) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_FIELD **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_FIELD(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_FIELD" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_FIELD **)dwg_getall_FIELD(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_FIELD, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_FIELDLIST) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_FIELDLIST **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_FIELDLIST(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_FIELDLIST" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_FIELDLIST **)dwg_getall_FIELDLIST(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_FIELDLIST, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_GEODATA) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_GEODATA **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_GEODATA(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_GEODATA" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_GEODATA **)dwg_getall_GEODATA(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_GEODATA, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_GROUP) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_GROUP **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_GROUP(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_GROUP" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_GROUP **)dwg_getall_GROUP(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_GROUP, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_IDBUFFER) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_IDBUFFER **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_IDBUFFER(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_IDBUFFER" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_IDBUFFER **)dwg_getall_IDBUFFER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_IDBUFFER, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_IMAGEDEF) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_IMAGEDEF **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_IMAGEDEF(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_IMAGEDEF" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_IMAGEDEF **)dwg_getall_IMAGEDEF(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_IMAGEDEF, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_IMAGEDEF_REACTOR) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_IMAGEDEF_REACTOR **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_IMAGEDEF_REACTOR(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_IMAGEDEF_REACTOR" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_IMAGEDEF_REACTOR **)dwg_getall_IMAGEDEF_REACTOR(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_IMAGEDEF_REACTOR, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_INDEX) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_INDEX **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_INDEX(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_INDEX" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_INDEX **)dwg_getall_INDEX(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_INDEX, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_LAYER) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_LAYER **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_LAYER(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_LAYER" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_LAYER **)dwg_getall_LAYER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_LAYER, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_LAYERFILTER) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_LAYERFILTER **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_LAYERFILTER(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_LAYERFILTER" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_LAYERFILTER **)dwg_getall_LAYERFILTER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_LAYERFILTER, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_LAYER_CONTROL) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_LAYER_CONTROL **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_LAYER_CONTROL(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_LAYER_CONTROL" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_LAYER_CONTROL **)dwg_getall_LAYER_CONTROL(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_LAYER_CONTROL, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_LAYER_INDEX) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_LAYER_INDEX **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_LAYER_INDEX(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_LAYER_INDEX" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_LAYER_INDEX **)dwg_getall_LAYER_INDEX(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_LAYER_INDEX, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_LAYOUT) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_LAYOUT **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_LAYOUT(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_LAYOUT" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_LAYOUT **)dwg_getall_LAYOUT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_LAYOUT, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_LONG_TRANSACTION) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_LONG_TRANSACTION **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_LONG_TRANSACTION(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_LONG_TRANSACTION" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_LONG_TRANSACTION **)dwg_getall_LONG_TRANSACTION(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_LONG_TRANSACTION, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_LTYPE) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_LTYPE **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_LTYPE(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_LTYPE" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_LTYPE **)dwg_getall_LTYPE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_LTYPE, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_LTYPE_CONTROL) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_LTYPE_CONTROL **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_LTYPE_CONTROL(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_LTYPE_CONTROL" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_LTYPE_CONTROL **)dwg_getall_LTYPE_CONTROL(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_LTYPE_CONTROL, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_MLINESTYLE) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_MLINESTYLE **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_MLINESTYLE(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_MLINESTYLE" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_MLINESTYLE **)dwg_getall_MLINESTYLE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_MLINESTYLE, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_PLACEHOLDER) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_PLACEHOLDER **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_PLACEHOLDER(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_PLACEHOLDER" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_PLACEHOLDER **)dwg_getall_PLACEHOLDER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_PLACEHOLDER, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_PLOTSETTINGS) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_PLOTSETTINGS **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_PLOTSETTINGS(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_PLOTSETTINGS" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_PLOTSETTINGS **)dwg_getall_PLOTSETTINGS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_PLOTSETTINGS, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_RASTERVARIABLES) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_RASTERVARIABLES **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_RASTERVARIABLES(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_RASTERVARIABLES" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_RASTERVARIABLES **)dwg_getall_RASTERVARIABLES(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_RASTERVARIABLES, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_RENDERENVIRONMENT) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_RENDERENVIRONMENT **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_RENDERENVIRONMENT(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_RENDERENVIRONMENT" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_RENDERENVIRONMENT **)dwg_getall_RENDERENVIRONMENT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_RENDERENVIRONMENT, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_SCALE) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_SCALE **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_SCALE(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_SCALE" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_SCALE **)dwg_getall_SCALE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_SCALE, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_SECTIONVIEWSTYLE) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_SECTIONVIEWSTYLE **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_SECTIONVIEWSTYLE(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_SECTIONVIEWSTYLE" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_SECTIONVIEWSTYLE **)dwg_getall_SECTIONVIEWSTYLE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_SECTIONVIEWSTYLE, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_SECTION_MANAGER) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_SECTION_MANAGER **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_SECTION_MANAGER(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_SECTION_MANAGER" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_SECTION_MANAGER **)dwg_getall_SECTION_MANAGER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_SECTION_MANAGER, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_SORTENTSTABLE) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_SORTENTSTABLE **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_SORTENTSTABLE(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_SORTENTSTABLE" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_SORTENTSTABLE **)dwg_getall_SORTENTSTABLE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_SORTENTSTABLE, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_SPATIAL_FILTER) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_SPATIAL_FILTER **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_SPATIAL_FILTER(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_SPATIAL_FILTER" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_SPATIAL_FILTER **)dwg_getall_SPATIAL_FILTER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_SPATIAL_FILTER, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_STYLE) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_STYLE **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_STYLE(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_STYLE" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_STYLE **)dwg_getall_STYLE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_STYLE, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_STYLE_CONTROL) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_STYLE_CONTROL **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_STYLE_CONTROL(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_STYLE_CONTROL" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_STYLE_CONTROL **)dwg_getall_STYLE_CONTROL(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_STYLE_CONTROL, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_SUN) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_SUN **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_SUN(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_SUN" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_SUN **)dwg_getall_SUN(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_SUN, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_TABLEGEOMETRY) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_TABLEGEOMETRY **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_TABLEGEOMETRY(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_TABLEGEOMETRY" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_TABLEGEOMETRY **)dwg_getall_TABLEGEOMETRY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_TABLEGEOMETRY, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_UCS) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_UCS **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_UCS(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_UCS" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_UCS **)dwg_getall_UCS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_UCS, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_UCS_CONTROL) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_UCS_CONTROL **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_UCS_CONTROL(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_UCS_CONTROL" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_UCS_CONTROL **)dwg_getall_UCS_CONTROL(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_UCS_CONTROL, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_UNKNOWN_OBJ) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_UNKNOWN_OBJ **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_UNKNOWN_OBJ(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_UNKNOWN_OBJ" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_UNKNOWN_OBJ **)dwg_getall_UNKNOWN_OBJ(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_UNKNOWN_OBJ, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_VBA_PROJECT) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_VBA_PROJECT **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_VBA_PROJECT(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_VBA_PROJECT" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_VBA_PROJECT **)dwg_getall_VBA_PROJECT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_VBA_PROJECT, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_VIEW) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_VIEW **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_VIEW(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_VIEW" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_VIEW **)dwg_getall_VIEW(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_VIEW, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_VIEW_CONTROL) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_VIEW_CONTROL **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_VIEW_CONTROL(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_VIEW_CONTROL" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_VIEW_CONTROL **)dwg_getall_VIEW_CONTROL(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_VIEW_CONTROL, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_VISUALSTYLE) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_VISUALSTYLE **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_VISUALSTYLE(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_VISUALSTYLE" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_VISUALSTYLE **)dwg_getall_VISUALSTYLE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_VISUALSTYLE, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_VPORT) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_VPORT **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_VPORT(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_VPORT" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_VPORT **)dwg_getall_VPORT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_VPORT, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_VPORT_CONTROL) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_VPORT_CONTROL **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_VPORT_CONTROL(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_VPORT_CONTROL" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_VPORT_CONTROL **)dwg_getall_VPORT_CONTROL(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_VPORT_CONTROL, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_VX_CONTROL) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_VX_CONTROL **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_VX_CONTROL(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_VX_CONTROL" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_VX_CONTROL **)dwg_getall_VX_CONTROL(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_VX_CONTROL, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_VX_TABLE_RECORD) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_VX_TABLE_RECORD **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_VX_TABLE_RECORD(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_VX_TABLE_RECORD" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_VX_TABLE_RECORD **)dwg_getall_VX_TABLE_RECORD(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_VX_TABLE_RECORD, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_WIPEOUTVARIABLES) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_WIPEOUTVARIABLES **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_WIPEOUTVARIABLES(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_WIPEOUTVARIABLES" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_WIPEOUTVARIABLES **)dwg_getall_WIPEOUTVARIABLES(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_WIPEOUTVARIABLES, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_XRECORD) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_XRECORD **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_XRECORD(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_XRECORD" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_XRECORD **)dwg_getall_XRECORD(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_XRECORD, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_PDFDEFINITION) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_PDFDEFINITION **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_PDFDEFINITION(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_PDFDEFINITION" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_PDFDEFINITION **)dwg_getall_PDFDEFINITION(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_abstractobject_UNDERLAYDEFINITION, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_DGNDEFINITION) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DGNDEFINITION **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_DGNDEFINITION(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_DGNDEFINITION" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_DGNDEFINITION **)dwg_getall_DGNDEFINITION(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_abstractobject_UNDERLAYDEFINITION, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_DWFDEFINITION) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DWFDEFINITION **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_DWFDEFINITION(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_DWFDEFINITION" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_DWFDEFINITION **)dwg_getall_DWFDEFINITION(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_abstractobject_UNDERLAYDEFINITION, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ACSH_BREP_CLASS) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_BREP_CLASS **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ACSH_BREP_CLASS(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ACSH_BREP_CLASS" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ACSH_BREP_CLASS **)dwg_getall_ACSH_BREP_CLASS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ACSH_BREP_CLASS, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ACSH_CHAMFER_CLASS) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_CHAMFER_CLASS **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ACSH_CHAMFER_CLASS(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ACSH_CHAMFER_CLASS" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ACSH_CHAMFER_CLASS **)dwg_getall_ACSH_CHAMFER_CLASS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ACSH_CHAMFER_CLASS, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ACSH_PYRAMID_CLASS) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_PYRAMID_CLASS **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ACSH_PYRAMID_CLASS(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ACSH_PYRAMID_CLASS" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ACSH_PYRAMID_CLASS **)dwg_getall_ACSH_PYRAMID_CLASS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ACSH_PYRAMID_CLASS, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ALDIMOBJECTCONTEXTDATA) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ALDIMOBJECTCONTEXTDATA **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ALDIMOBJECTCONTEXTDATA(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ALDIMOBJECTCONTEXTDATA" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ALDIMOBJECTCONTEXTDATA **)dwg_getall_ALDIMOBJECTCONTEXTDATA(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ALDIMOBJECTCONTEXTDATA, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ASSOC2DCONSTRAINTGROUP) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOC2DCONSTRAINTGROUP **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ASSOC2DCONSTRAINTGROUP(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ASSOC2DCONSTRAINTGROUP" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ASSOC2DCONSTRAINTGROUP **)dwg_getall_ASSOC2DCONSTRAINTGROUP(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ASSOCACTION) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCACTION **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ASSOCACTION(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ASSOCACTION" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ASSOCACTION **)dwg_getall_ASSOCACTION(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ASSOCACTION, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ASSOCACTIONPARAM) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCACTIONPARAM **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ASSOCACTIONPARAM(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ASSOCACTIONPARAM" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ASSOCACTIONPARAM **)dwg_getall_ASSOCACTIONPARAM(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ASSOCACTIONPARAM, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ASSOCARRAYACTIONBODY) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCARRAYACTIONBODY **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ASSOCARRAYACTIONBODY(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ASSOCARRAYACTIONBODY" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ASSOCARRAYACTIONBODY **)dwg_getall_ASSOCARRAYACTIONBODY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ASSOCARRAYACTIONBODY, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ASSOCASMBODYACTIONPARAM) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCASMBODYACTIONPARAM **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ASSOCASMBODYACTIONPARAM(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ASSOCASMBODYACTIONPARAM" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ASSOCASMBODYACTIONPARAM **)dwg_getall_ASSOCASMBODYACTIONPARAM(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ASSOCBLENDSURFACEACTIONBODY) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCBLENDSURFACEACTIONBODY **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ASSOCBLENDSURFACEACTIONBODY(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ASSOCBLENDSURFACEACTIONBODY" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ASSOCBLENDSURFACEACTIONBODY **)dwg_getall_ASSOCBLENDSURFACEACTIONBODY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ASSOCCOMPOUNDACTIONPARAM) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCCOMPOUNDACTIONPARAM **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ASSOCCOMPOUNDACTIONPARAM(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ASSOCCOMPOUNDACTIONPARAM" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ASSOCCOMPOUNDACTIONPARAM **)dwg_getall_ASSOCCOMPOUNDACTIONPARAM(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ASSOCDEPENDENCY) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCDEPENDENCY **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ASSOCDEPENDENCY(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ASSOCDEPENDENCY" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ASSOCDEPENDENCY **)dwg_getall_ASSOCDEPENDENCY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ASSOCDEPENDENCY, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ASSOCDIMDEPENDENCYBODY) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCDIMDEPENDENCYBODY **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ASSOCDIMDEPENDENCYBODY(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ASSOCDIMDEPENDENCYBODY" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ASSOCDIMDEPENDENCYBODY **)dwg_getall_ASSOCDIMDEPENDENCYBODY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ASSOCDIMDEPENDENCYBODY, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ASSOCEXTENDSURFACEACTIONBODY) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ASSOCEXTENDSURFACEACTIONBODY(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ASSOCEXTENDSURFACEACTIONBODY" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY **)dwg_getall_ASSOCEXTENDSURFACEACTIONBODY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ASSOCEXTENDSURFACEACTIONBODY, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ASSOCEXTRUDEDSURFACEACTIONBODY) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ASSOCEXTRUDEDSURFACEACTIONBODY(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ASSOCEXTRUDEDSURFACEACTIONBODY" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY **)dwg_getall_ASSOCEXTRUDEDSURFACEACTIONBODY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ASSOCFACEACTIONPARAM) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCFACEACTIONPARAM **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ASSOCFACEACTIONPARAM(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ASSOCFACEACTIONPARAM" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ASSOCFACEACTIONPARAM **)dwg_getall_ASSOCFACEACTIONPARAM(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ASSOCFACEACTIONPARAM, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ASSOCFILLETSURFACEACTIONBODY) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCFILLETSURFACEACTIONBODY **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ASSOCFILLETSURFACEACTIONBODY(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ASSOCFILLETSURFACEACTIONBODY" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ASSOCFILLETSURFACEACTIONBODY **)dwg_getall_ASSOCFILLETSURFACEACTIONBODY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ASSOCFILLETSURFACEACTIONBODY, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ASSOCLOFTEDSURFACEACTIONBODY) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ASSOCLOFTEDSURFACEACTIONBODY(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ASSOCLOFTEDSURFACEACTIONBODY" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY **)dwg_getall_ASSOCLOFTEDSURFACEACTIONBODY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ASSOCLOFTEDSURFACEACTIONBODY, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ASSOCNETWORKSURFACEACTIONBODY) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ASSOCNETWORKSURFACEACTIONBODY(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ASSOCNETWORKSURFACEACTIONBODY" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY **)dwg_getall_ASSOCNETWORKSURFACEACTIONBODY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ASSOCNETWORKSURFACEACTIONBODY, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ASSOCOBJECTACTIONPARAM) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCOBJECTACTIONPARAM **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ASSOCOBJECTACTIONPARAM(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ASSOCOBJECTACTIONPARAM" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ASSOCOBJECTACTIONPARAM **)dwg_getall_ASSOCOBJECTACTIONPARAM(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ASSOCOBJECTACTIONPARAM, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ASSOCOFFSETSURFACEACTIONBODY) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ASSOCOFFSETSURFACEACTIONBODY(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ASSOCOFFSETSURFACEACTIONBODY" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY **)dwg_getall_ASSOCOFFSETSURFACEACTIONBODY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ASSOCOFFSETSURFACEACTIONBODY, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ASSOCOSNAPPOINTREFACTIONPARAM) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ASSOCOSNAPPOINTREFACTIONPARAM(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ASSOCOSNAPPOINTREFACTIONPARAM" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM **)dwg_getall_ASSOCOSNAPPOINTREFACTIONPARAM(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ASSOCPATCHSURFACEACTIONBODY) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCPATCHSURFACEACTIONBODY **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ASSOCPATCHSURFACEACTIONBODY(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ASSOCPATCHSURFACEACTIONBODY" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ASSOCPATCHSURFACEACTIONBODY **)dwg_getall_ASSOCPATCHSURFACEACTIONBODY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ASSOCPATCHSURFACEACTIONBODY, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ASSOCPATHACTIONPARAM) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCPATHACTIONPARAM **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ASSOCPATHACTIONPARAM(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ASSOCPATHACTIONPARAM" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ASSOCPATHACTIONPARAM **)dwg_getall_ASSOCPATHACTIONPARAM(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ASSOCPATHACTIONPARAM, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ASSOCPLANESURFACEACTIONBODY) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCPLANESURFACEACTIONBODY **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ASSOCPLANESURFACEACTIONBODY(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ASSOCPLANESURFACEACTIONBODY" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ASSOCPLANESURFACEACTIONBODY **)dwg_getall_ASSOCPLANESURFACEACTIONBODY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ASSOCPLANESURFACEACTIONBODY, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ASSOCPOINTREFACTIONPARAM) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCPOINTREFACTIONPARAM **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ASSOCPOINTREFACTIONPARAM(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ASSOCPOINTREFACTIONPARAM" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ASSOCPOINTREFACTIONPARAM **)dwg_getall_ASSOCPOINTREFACTIONPARAM(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ASSOCPOINTREFACTIONPARAM, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ASSOCREVOLVEDSURFACEACTIONBODY) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ASSOCREVOLVEDSURFACEACTIONBODY(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ASSOCREVOLVEDSURFACEACTIONBODY" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY **)dwg_getall_ASSOCREVOLVEDSURFACEACTIONBODY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ASSOCTRIMSURFACEACTIONBODY) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCTRIMSURFACEACTIONBODY **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ASSOCTRIMSURFACEACTIONBODY(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ASSOCTRIMSURFACEACTIONBODY" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ASSOCTRIMSURFACEACTIONBODY **)dwg_getall_ASSOCTRIMSURFACEACTIONBODY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ASSOCTRIMSURFACEACTIONBODY, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ASSOCVALUEDEPENDENCY) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCVALUEDEPENDENCY **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ASSOCVALUEDEPENDENCY(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ASSOCVALUEDEPENDENCY" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ASSOCVALUEDEPENDENCY **)dwg_getall_ASSOCVALUEDEPENDENCY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ASSOCVALUEDEPENDENCY, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ASSOCVARIABLE) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCVARIABLE **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ASSOCVARIABLE(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ASSOCVARIABLE" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ASSOCVARIABLE **)dwg_getall_ASSOCVARIABLE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ASSOCVARIABLE, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ASSOCVERTEXACTIONPARAM) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCVERTEXACTIONPARAM **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ASSOCVERTEXACTIONPARAM(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ASSOCVERTEXACTIONPARAM" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ASSOCVERTEXACTIONPARAM **)dwg_getall_ASSOCVERTEXACTIONPARAM(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_ASSOCVERTEXACTIONPARAM, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLKREFOBJECTCONTEXTDATA) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLKREFOBJECTCONTEXTDATA **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLKREFOBJECTCONTEXTDATA(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLKREFOBJECTCONTEXTDATA" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLKREFOBJECTCONTEXTDATA **)dwg_getall_BLKREFOBJECTCONTEXTDATA(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLKREFOBJECTCONTEXTDATA, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKALIGNEDCONSTRAINTPARAMETER) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKALIGNEDCONSTRAINTPARAMETER(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKALIGNEDCONSTRAINTPARAMETER" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER **)dwg_getall_BLOCKALIGNEDCONSTRAINTPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKANGULARCONSTRAINTPARAMETER) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKANGULARCONSTRAINTPARAMETER(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKANGULARCONSTRAINTPARAMETER" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER **)dwg_getall_BLOCKANGULARCONSTRAINTPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKARRAYACTION) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKARRAYACTION **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKARRAYACTION(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKARRAYACTION" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKARRAYACTION **)dwg_getall_BLOCKARRAYACTION(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKARRAYACTION, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKDIAMETRICCONSTRAINTPARAMETER) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKDIAMETRICCONSTRAINTPARAMETER(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKDIAMETRICCONSTRAINTPARAMETER" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER **)dwg_getall_BLOCKDIAMETRICCONSTRAINTPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKHORIZONTALCONSTRAINTPARAMETER) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKHORIZONTALCONSTRAINTPARAMETER(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKHORIZONTALCONSTRAINTPARAMETER" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER **)dwg_getall_BLOCKHORIZONTALCONSTRAINTPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKLINEARCONSTRAINTPARAMETER) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKLINEARCONSTRAINTPARAMETER(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKLINEARCONSTRAINTPARAMETER" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER **)dwg_getall_BLOCKLINEARCONSTRAINTPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKLINEARPARAMETER) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKLINEARPARAMETER **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKLINEARPARAMETER(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKLINEARPARAMETER" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKLINEARPARAMETER **)dwg_getall_BLOCKLINEARPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKLINEARPARAMETER, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKLOOKUPACTION) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKLOOKUPACTION **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKLOOKUPACTION(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKLOOKUPACTION" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKLOOKUPACTION **)dwg_getall_BLOCKLOOKUPACTION(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKLOOKUPACTION, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKLOOKUPPARAMETER) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKLOOKUPPARAMETER **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKLOOKUPPARAMETER(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKLOOKUPPARAMETER" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKLOOKUPPARAMETER **)dwg_getall_BLOCKLOOKUPPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKLOOKUPPARAMETER, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKPARAMDEPENDENCYBODY) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKPARAMDEPENDENCYBODY **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKPARAMDEPENDENCYBODY(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKPARAMDEPENDENCYBODY" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKPARAMDEPENDENCYBODY **)dwg_getall_BLOCKPARAMDEPENDENCYBODY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKPARAMDEPENDENCYBODY, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKPOINTPARAMETER) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKPOINTPARAMETER **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKPOINTPARAMETER(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKPOINTPARAMETER" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKPOINTPARAMETER **)dwg_getall_BLOCKPOINTPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKPOINTPARAMETER, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKPOLARGRIP) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKPOLARGRIP **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKPOLARGRIP(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKPOLARGRIP" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKPOLARGRIP **)dwg_getall_BLOCKPOLARGRIP(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKPOLARGRIP, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKPOLARPARAMETER) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKPOLARPARAMETER **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKPOLARPARAMETER(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKPOLARPARAMETER" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKPOLARPARAMETER **)dwg_getall_BLOCKPOLARPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKPOLARPARAMETER, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKPOLARSTRETCHACTION) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKPOLARSTRETCHACTION **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKPOLARSTRETCHACTION(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKPOLARSTRETCHACTION" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKPOLARSTRETCHACTION **)dwg_getall_BLOCKPOLARSTRETCHACTION(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKRADIALCONSTRAINTPARAMETER) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKRADIALCONSTRAINTPARAMETER(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKRADIALCONSTRAINTPARAMETER" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER **)dwg_getall_BLOCKRADIALCONSTRAINTPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKREPRESENTATION) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKREPRESENTATION **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKREPRESENTATION(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKREPRESENTATION" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKREPRESENTATION **)dwg_getall_BLOCKREPRESENTATION(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKREPRESENTATION, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKROTATIONPARAMETER) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKROTATIONPARAMETER **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKROTATIONPARAMETER(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKROTATIONPARAMETER" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKROTATIONPARAMETER **)dwg_getall_BLOCKROTATIONPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKROTATIONPARAMETER, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKSTRETCHACTION) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKSTRETCHACTION **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKSTRETCHACTION(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKSTRETCHACTION" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKSTRETCHACTION **)dwg_getall_BLOCKSTRETCHACTION(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKSTRETCHACTION, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKUSERPARAMETER) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKUSERPARAMETER **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKUSERPARAMETER(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKUSERPARAMETER" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKUSERPARAMETER **)dwg_getall_BLOCKUSERPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKUSERPARAMETER, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKVERTICALCONSTRAINTPARAMETER) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKVERTICALCONSTRAINTPARAMETER(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKVERTICALCONSTRAINTPARAMETER" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER **)dwg_getall_BLOCKVERTICALCONSTRAINTPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKVISIBILITYPARAMETER) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKVISIBILITYPARAMETER **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKVISIBILITYPARAMETER(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKVISIBILITYPARAMETER" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKVISIBILITYPARAMETER **)dwg_getall_BLOCKVISIBILITYPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKXYGRIP) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKXYGRIP **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKXYGRIP(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKXYGRIP" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKXYGRIP **)dwg_getall_BLOCKXYGRIP(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKXYGRIP, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_BLOCKXYPARAMETER) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKXYPARAMETER **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_BLOCKXYPARAMETER(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_BLOCKXYPARAMETER" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_BLOCKXYPARAMETER **)dwg_getall_BLOCKXYPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_BLOCKXYPARAMETER, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_DATALINK) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DATALINK **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_DATALINK(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_DATALINK" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_DATALINK **)dwg_getall_DATALINK(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_DATALINK, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_DBCOLOR) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DBCOLOR **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_DBCOLOR(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_DBCOLOR" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_DBCOLOR **)dwg_getall_DBCOLOR(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_DBCOLOR, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_EVALUATION_GRAPH) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_EVALUATION_GRAPH **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_EVALUATION_GRAPH(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_EVALUATION_GRAPH" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_EVALUATION_GRAPH **)dwg_getall_EVALUATION_GRAPH(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_EVALUATION_GRAPH, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_FCFOBJECTCONTEXTDATA) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_FCFOBJECTCONTEXTDATA **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_FCFOBJECTCONTEXTDATA(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_FCFOBJECTCONTEXTDATA" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_FCFOBJECTCONTEXTDATA **)dwg_getall_FCFOBJECTCONTEXTDATA(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_FCFOBJECTCONTEXTDATA, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_GRADIENT_BACKGROUND) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_GRADIENT_BACKGROUND **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_GRADIENT_BACKGROUND(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_GRADIENT_BACKGROUND" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_GRADIENT_BACKGROUND **)dwg_getall_GRADIENT_BACKGROUND(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_GRADIENT_BACKGROUND, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_GROUND_PLANE_BACKGROUND) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_GROUND_PLANE_BACKGROUND **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_GROUND_PLANE_BACKGROUND(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_GROUND_PLANE_BACKGROUND" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_GROUND_PLANE_BACKGROUND **)dwg_getall_GROUND_PLANE_BACKGROUND(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_GROUND_PLANE_BACKGROUND, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_IBL_BACKGROUND) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_IBL_BACKGROUND **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_IBL_BACKGROUND(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_IBL_BACKGROUND" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_IBL_BACKGROUND **)dwg_getall_IBL_BACKGROUND(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_IBL_BACKGROUND, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_IMAGE_BACKGROUND) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_IMAGE_BACKGROUND **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_IMAGE_BACKGROUND(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_IMAGE_BACKGROUND" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_IMAGE_BACKGROUND **)dwg_getall_IMAGE_BACKGROUND(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_IMAGE_BACKGROUND, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_LEADEROBJECTCONTEXTDATA) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_LEADEROBJECTCONTEXTDATA **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_LEADEROBJECTCONTEXTDATA(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_LEADEROBJECTCONTEXTDATA" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_LEADEROBJECTCONTEXTDATA **)dwg_getall_LEADEROBJECTCONTEXTDATA(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_LEADEROBJECTCONTEXTDATA, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_LIGHTLIST) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_LIGHTLIST **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_LIGHTLIST(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_LIGHTLIST" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_LIGHTLIST **)dwg_getall_LIGHTLIST(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_LIGHTLIST, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_MATERIAL) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_MATERIAL **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_MATERIAL(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_MATERIAL" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_MATERIAL **)dwg_getall_MATERIAL(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_MATERIAL, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_MENTALRAYRENDERSETTINGS) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_MENTALRAYRENDERSETTINGS **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_MENTALRAYRENDERSETTINGS(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_MENTALRAYRENDERSETTINGS" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_MENTALRAYRENDERSETTINGS **)dwg_getall_MENTALRAYRENDERSETTINGS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_MLEADERSTYLE) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_MLEADERSTYLE **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_MLEADERSTYLE(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_MLEADERSTYLE" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_MLEADERSTYLE **)dwg_getall_MLEADERSTYLE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_MLEADERSTYLE, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_MTEXTOBJECTCONTEXTDATA) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_MTEXTOBJECTCONTEXTDATA **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_MTEXTOBJECTCONTEXTDATA(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_MTEXTOBJECTCONTEXTDATA" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_MTEXTOBJECTCONTEXTDATA **)dwg_getall_MTEXTOBJECTCONTEXTDATA(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_OBJECT_PTR) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_OBJECT_PTR **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_OBJECT_PTR(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_OBJECT_PTR" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_OBJECT_PTR **)dwg_getall_OBJECT_PTR(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_OBJECT_PTR, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_PARTIAL_VIEWING_INDEX) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_PARTIAL_VIEWING_INDEX **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_PARTIAL_VIEWING_INDEX(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_PARTIAL_VIEWING_INDEX" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_PARTIAL_VIEWING_INDEX **)dwg_getall_PARTIAL_VIEWING_INDEX(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_PARTIAL_VIEWING_INDEX, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_POINTCLOUDCOLORMAP) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_POINTCLOUDCOLORMAP **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_POINTCLOUDCOLORMAP(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_POINTCLOUDCOLORMAP" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_POINTCLOUDCOLORMAP **)dwg_getall_POINTCLOUDCOLORMAP(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_POINTCLOUDCOLORMAP, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_POINTCLOUDDEF) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_POINTCLOUDDEF **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_POINTCLOUDDEF(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_POINTCLOUDDEF" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_POINTCLOUDDEF **)dwg_getall_POINTCLOUDDEF(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_POINTCLOUDDEF, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_POINTCLOUDDEFEX) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_POINTCLOUDDEFEX **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_POINTCLOUDDEFEX(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_POINTCLOUDDEFEX" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_POINTCLOUDDEFEX **)dwg_getall_POINTCLOUDDEFEX(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_POINTCLOUDDEFEX, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_POINTCLOUDDEF_REACTOR) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_POINTCLOUDDEF_REACTOR **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_POINTCLOUDDEF_REACTOR(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_POINTCLOUDDEF_REACTOR" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_POINTCLOUDDEF_REACTOR **)dwg_getall_POINTCLOUDDEF_REACTOR(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_POINTCLOUDDEF_REACTOR, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_POINTCLOUDDEF_REACTOR_EX) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_POINTCLOUDDEF_REACTOR_EX **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_POINTCLOUDDEF_REACTOR_EX(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_POINTCLOUDDEF_REACTOR_EX" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_POINTCLOUDDEF_REACTOR_EX **)dwg_getall_POINTCLOUDDEF_REACTOR_EX(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_POINTCLOUDDEF_REACTOR_EX, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_PROXY_OBJECT) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_PROXY_OBJECT **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_PROXY_OBJECT(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_PROXY_OBJECT" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_PROXY_OBJECT **)dwg_getall_PROXY_OBJECT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_PROXY_OBJECT, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_RAPIDRTRENDERSETTINGS) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_RAPIDRTRENDERSETTINGS **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_RAPIDRTRENDERSETTINGS(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_RAPIDRTRENDERSETTINGS" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_RAPIDRTRENDERSETTINGS **)dwg_getall_RAPIDRTRENDERSETTINGS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_RENDERENTRY) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_RENDERENTRY **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_RENDERENTRY(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_RENDERENTRY" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_RENDERENTRY **)dwg_getall_RENDERENTRY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_RENDERENTRY, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_RENDERGLOBAL) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_RENDERGLOBAL **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_RENDERGLOBAL(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_RENDERGLOBAL" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_RENDERGLOBAL **)dwg_getall_RENDERGLOBAL(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_RENDERGLOBAL, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_RENDERSETTINGS) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_RENDERSETTINGS **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_RENDERSETTINGS(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_RENDERSETTINGS" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_RENDERSETTINGS **)dwg_getall_RENDERSETTINGS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_RENDERSETTINGS, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_SECTION_SETTINGS) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_SECTION_SETTINGS **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_SECTION_SETTINGS(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_SECTION_SETTINGS" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_SECTION_SETTINGS **)dwg_getall_SECTION_SETTINGS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_SECTION_SETTINGS, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_SKYLIGHT_BACKGROUND) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_SKYLIGHT_BACKGROUND **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_SKYLIGHT_BACKGROUND(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_SKYLIGHT_BACKGROUND" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_SKYLIGHT_BACKGROUND **)dwg_getall_SKYLIGHT_BACKGROUND(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_SKYLIGHT_BACKGROUND, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_SOLID_BACKGROUND) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_SOLID_BACKGROUND **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_SOLID_BACKGROUND(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_SOLID_BACKGROUND" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_SOLID_BACKGROUND **)dwg_getall_SOLID_BACKGROUND(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_SOLID_BACKGROUND, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_SPATIAL_INDEX) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_SPATIAL_INDEX **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_SPATIAL_INDEX(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_SPATIAL_INDEX" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_SPATIAL_INDEX **)dwg_getall_SPATIAL_INDEX(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_SPATIAL_INDEX, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_TABLESTYLE) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_TABLESTYLE **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_TABLESTYLE(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_TABLESTYLE" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_TABLESTYLE **)dwg_getall_TABLESTYLE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_TABLESTYLE, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_TEXTOBJECTCONTEXTDATA) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_TEXTOBJECTCONTEXTDATA **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_TEXTOBJECTCONTEXTDATA(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_TEXTOBJECTCONTEXTDATA" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_TEXTOBJECTCONTEXTDATA **)dwg_getall_TEXTOBJECTCONTEXTDATA(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_object_TEXTOBJECTCONTEXTDATA, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ASSOCARRAYMODIFYPARAMETERS) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCARRAYMODIFYPARAMETERS **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ASSOCARRAYMODIFYPARAMETERS(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ASSOCARRAYMODIFYPARAMETERS" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ASSOCARRAYMODIFYPARAMETERS **)dwg_getall_ASSOCARRAYMODIFYPARAMETERS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ASSOCARRAYPATHPARAMETERS) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCARRAYPATHPARAMETERS **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ASSOCARRAYPATHPARAMETERS(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ASSOCARRAYPATHPARAMETERS" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ASSOCARRAYPATHPARAMETERS **)dwg_getall_ASSOCARRAYPATHPARAMETERS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ASSOCARRAYPOLARPARAMETERS) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCARRAYPOLARPARAMETERS **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ASSOCARRAYPOLARPARAMETERS(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ASSOCARRAYPOLARPARAMETERS" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ASSOCARRAYPOLARPARAMETERS **)dwg_getall_ASSOCARRAYPOLARPARAMETERS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_getall_ASSOCARRAYRECTANGULARPARAMETERS) {
  {
    Dwg_Data *arg1 = (Dwg_Data *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCARRAYRECTANGULARPARAMETERS **result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_getall_ASSOCARRAYRECTANGULARPARAMETERS(dwg);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_struct, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_getall_ASSOCARRAYRECTANGULARPARAMETERS" "', argument " "1"" of type '" "Dwg_Data *""'"); 
    }
    arg1 = (Dwg_Data *)(argp1);
    result = (Dwg_Object_ASSOCARRAYRECTANGULARPARAMETERS **)dwg_getall_ASSOCARRAYRECTANGULARPARAMETERS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, 0 | 0); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to__3DFACE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity__3DFACE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to__3DFACE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to__3DFACE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity__3DFACE *)dwg_object_to__3DFACE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_3DFACE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to__3DLINE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity__3DLINE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to__3DLINE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to__3DLINE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity__3DLINE *)dwg_object_to__3DLINE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_3DLINE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to__3DSOLID) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity__3DSOLID *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to__3DSOLID(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to__3DSOLID" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity__3DSOLID *)dwg_object_to__3DSOLID(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_3DSOLID, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ARC) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_ARC *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ARC(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ARC" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_ARC *)dwg_object_to_ARC(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_ARC, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ATTDEF) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_ATTDEF *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ATTDEF(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ATTDEF" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_ATTDEF *)dwg_object_to_ATTDEF(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_ATTDEF, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ATTRIB) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_ATTRIB *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ATTRIB(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ATTRIB" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_ATTRIB *)dwg_object_to_ATTRIB(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_ATTRIB, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCK) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_BLOCK *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCK(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCK" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_BLOCK *)dwg_object_to_BLOCK(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_BLOCK, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BODY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_BODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BODY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BODY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_BODY *)dwg_object_to_BODY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_3DSOLID, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_CAMERA) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_CAMERA *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_CAMERA(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_CAMERA" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_CAMERA *)dwg_object_to_CAMERA(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_CAMERA, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_CIRCLE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_CIRCLE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_CIRCLE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_CIRCLE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_CIRCLE *)dwg_object_to_CIRCLE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_CIRCLE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_DGNUNDERLAY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_DGNUNDERLAY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_DGNUNDERLAY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_DGNUNDERLAY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_DGNUNDERLAY *)dwg_object_to_DGNUNDERLAY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_abstractentity_UNDERLAY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_DIMENSION_ALIGNED) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_DIMENSION_ALIGNED *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_DIMENSION_ALIGNED(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_DIMENSION_ALIGNED" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_DIMENSION_ALIGNED *)dwg_object_to_DIMENSION_ALIGNED(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_DIMENSION_ANG2LN) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_DIMENSION_ANG2LN *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_DIMENSION_ANG2LN(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_DIMENSION_ANG2LN" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_DIMENSION_ANG2LN *)dwg_object_to_DIMENSION_ANG2LN(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_DIMENSION_ANG3PT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_DIMENSION_ANG3PT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_DIMENSION_ANG3PT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_DIMENSION_ANG3PT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_DIMENSION_ANG3PT *)dwg_object_to_DIMENSION_ANG3PT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_DIMENSION_DIAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_DIMENSION_DIAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_DIMENSION_DIAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_DIMENSION_DIAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_DIMENSION_DIAMETER *)dwg_object_to_DIMENSION_DIAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_DIMENSION_LINEAR) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_DIMENSION_LINEAR *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_DIMENSION_LINEAR(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_DIMENSION_LINEAR" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_DIMENSION_LINEAR *)dwg_object_to_DIMENSION_LINEAR(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_DIMENSION_ORDINATE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_DIMENSION_ORDINATE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_DIMENSION_ORDINATE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_DIMENSION_ORDINATE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_DIMENSION_ORDINATE *)dwg_object_to_DIMENSION_ORDINATE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_DIMENSION_RADIUS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_DIMENSION_RADIUS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_DIMENSION_RADIUS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_DIMENSION_RADIUS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_DIMENSION_RADIUS *)dwg_object_to_DIMENSION_RADIUS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_DWFUNDERLAY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_DWFUNDERLAY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_DWFUNDERLAY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_DWFUNDERLAY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_DWFUNDERLAY *)dwg_object_to_DWFUNDERLAY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_abstractentity_UNDERLAY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ELLIPSE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_ELLIPSE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ELLIPSE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ELLIPSE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_ELLIPSE *)dwg_object_to_ELLIPSE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_ELLIPSE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ENDBLK) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_ENDBLK *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ENDBLK(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ENDBLK" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_ENDBLK *)dwg_object_to_ENDBLK(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_ENDBLK, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ENDREP) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_ENDREP *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ENDREP(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ENDREP" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_ENDREP *)dwg_object_to_ENDREP(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_ENDREP, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_HATCH) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_HATCH *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_HATCH(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_HATCH" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_HATCH *)dwg_object_to_HATCH(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_HATCH, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_IMAGE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_IMAGE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_IMAGE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_IMAGE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_IMAGE *)dwg_object_to_IMAGE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_IMAGE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_INSERT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_INSERT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_INSERT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_INSERT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_INSERT *)dwg_object_to_INSERT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_INSERT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_JUMP) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_JUMP *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_JUMP(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_JUMP" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_JUMP *)dwg_object_to_JUMP(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_JUMP, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_LEADER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_LEADER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_LEADER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_LEADER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_LEADER *)dwg_object_to_LEADER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_LEADER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_LIGHT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_LIGHT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_LIGHT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_LIGHT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_LIGHT *)dwg_object_to_LIGHT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_LIGHT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_LINE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_LINE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_LINE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_LINE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_LINE *)dwg_object_to_LINE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_LINE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_LOAD) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_LOAD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_LOAD(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_LOAD" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_LOAD *)dwg_object_to_LOAD(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_LOAD, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_LWPOLYLINE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_LWPOLYLINE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_LWPOLYLINE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_LWPOLYLINE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_LWPOLYLINE *)dwg_object_to_LWPOLYLINE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_LWPOLYLINE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_MESH) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_MESH *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_MESH(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_MESH" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_MESH *)dwg_object_to_MESH(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_MESH, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_MINSERT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_MINSERT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_MINSERT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_MINSERT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_MINSERT *)dwg_object_to_MINSERT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_MINSERT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_MLINE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_MLINE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_MLINE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_MLINE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_MLINE *)dwg_object_to_MLINE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_MLINE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_MTEXT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_MTEXT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_MTEXT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_MTEXT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_MTEXT *)dwg_object_to_MTEXT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_MTEXT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_MULTILEADER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_MULTILEADER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_MULTILEADER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_MULTILEADER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_MULTILEADER *)dwg_object_to_MULTILEADER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_MULTILEADER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_OLE2FRAME) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_OLE2FRAME *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_OLE2FRAME(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_OLE2FRAME" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_OLE2FRAME *)dwg_object_to_OLE2FRAME(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_OLE2FRAME, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_OLEFRAME) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_OLEFRAME *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_OLEFRAME(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_OLEFRAME" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_OLEFRAME *)dwg_object_to_OLEFRAME(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_OLEFRAME, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_PDFUNDERLAY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_PDFUNDERLAY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_PDFUNDERLAY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_PDFUNDERLAY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_PDFUNDERLAY *)dwg_object_to_PDFUNDERLAY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_abstractentity_UNDERLAY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_POINT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_POINT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_POINT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_POINT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_POINT *)dwg_object_to_POINT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_POINT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_POLYLINE_2D) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_POLYLINE_2D *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_POLYLINE_2D(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_POLYLINE_2D" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_POLYLINE_2D *)dwg_object_to_POLYLINE_2D(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_POLYLINE_2D, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_POLYLINE_3D) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_POLYLINE_3D *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_POLYLINE_3D(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_POLYLINE_3D" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_POLYLINE_3D *)dwg_object_to_POLYLINE_3D(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_POLYLINE_3D, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_POLYLINE_MESH) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_POLYLINE_MESH *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_POLYLINE_MESH(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_POLYLINE_MESH" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_POLYLINE_MESH *)dwg_object_to_POLYLINE_MESH(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_POLYLINE_MESH, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_POLYLINE_PFACE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_POLYLINE_PFACE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_POLYLINE_PFACE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_POLYLINE_PFACE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_POLYLINE_PFACE *)dwg_object_to_POLYLINE_PFACE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_POLYLINE_PFACE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_PROXY_ENTITY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_PROXY_ENTITY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_PROXY_ENTITY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_PROXY_ENTITY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_PROXY_ENTITY *)dwg_object_to_PROXY_ENTITY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_PROXY_ENTITY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_RAY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_RAY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_RAY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_RAY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_RAY *)dwg_object_to_RAY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_RAY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_REGION) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_REGION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_REGION(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_REGION" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_REGION *)dwg_object_to_REGION(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_3DSOLID, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_REPEAT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_REPEAT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_REPEAT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_REPEAT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_REPEAT *)dwg_object_to_REPEAT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_REPEAT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_SECTIONOBJECT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_SECTIONOBJECT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_SECTIONOBJECT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_SECTIONOBJECT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_SECTIONOBJECT *)dwg_object_to_SECTIONOBJECT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_SECTIONOBJECT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_SEQEND) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_SEQEND *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_SEQEND(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_SEQEND" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_SEQEND *)dwg_object_to_SEQEND(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_SEQEND, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_SHAPE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_SHAPE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_SHAPE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_SHAPE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_SHAPE *)dwg_object_to_SHAPE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_SHAPE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_SOLID) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_SOLID *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_SOLID(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_SOLID" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_SOLID *)dwg_object_to_SOLID(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_SOLID, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_SPLINE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_SPLINE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_SPLINE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_SPLINE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_SPLINE *)dwg_object_to_SPLINE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_SPLINE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_TEXT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_TEXT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_TEXT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_TEXT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_TEXT *)dwg_object_to_TEXT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_TEXT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_TOLERANCE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_TOLERANCE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_TOLERANCE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_TOLERANCE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_TOLERANCE *)dwg_object_to_TOLERANCE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_TOLERANCE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_TRACE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_TRACE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_TRACE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_TRACE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_TRACE *)dwg_object_to_TRACE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_TRACE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_UNKNOWN_ENT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_UNKNOWN_ENT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_UNKNOWN_ENT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_UNKNOWN_ENT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_UNKNOWN_ENT *)dwg_object_to_UNKNOWN_ENT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_UNKNOWN_ENT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_VERTEX_2D) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_VERTEX_2D *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_VERTEX_2D(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_VERTEX_2D" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_VERTEX_2D *)dwg_object_to_VERTEX_2D(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_VERTEX_2D, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_VERTEX_3D) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_VERTEX_3D *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_VERTEX_3D(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_VERTEX_3D" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_VERTEX_3D *)dwg_object_to_VERTEX_3D(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_VERTEX_3D, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_VERTEX_MESH) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_VERTEX_MESH *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_VERTEX_MESH(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_VERTEX_MESH" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_VERTEX_MESH *)dwg_object_to_VERTEX_MESH(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_VERTEX_3D, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_VERTEX_PFACE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_VERTEX_PFACE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_VERTEX_PFACE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_VERTEX_PFACE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_VERTEX_PFACE *)dwg_object_to_VERTEX_PFACE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_VERTEX_3D, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_VERTEX_PFACE_FACE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_VERTEX_PFACE_FACE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_VERTEX_PFACE_FACE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_VERTEX_PFACE_FACE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_VERTEX_PFACE_FACE *)dwg_object_to_VERTEX_PFACE_FACE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_VERTEX_PFACE_FACE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_VIEWPORT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_VIEWPORT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_VIEWPORT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_VIEWPORT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_VIEWPORT *)dwg_object_to_VIEWPORT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_VIEWPORT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_WIPEOUT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_WIPEOUT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_WIPEOUT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_WIPEOUT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_WIPEOUT *)dwg_object_to_WIPEOUT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_WIPEOUT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_XLINE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_XLINE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_XLINE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_XLINE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_XLINE *)dwg_object_to_XLINE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_RAY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ARC_DIMENSION) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_ARC_DIMENSION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ARC_DIMENSION(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ARC_DIMENSION" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_ARC_DIMENSION *)dwg_object_to_ARC_DIMENSION(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_ARC_DIMENSION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_HELIX) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_HELIX *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_HELIX(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_HELIX" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_HELIX *)dwg_object_to_HELIX(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_HELIX, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_LARGE_RADIAL_DIMENSION) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_LARGE_RADIAL_DIMENSION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_LARGE_RADIAL_DIMENSION(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_LARGE_RADIAL_DIMENSION" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_LARGE_RADIAL_DIMENSION *)dwg_object_to_LARGE_RADIAL_DIMENSION(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_LAYOUTPRINTCONFIG) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_LAYOUTPRINTCONFIG *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_LAYOUTPRINTCONFIG(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_LAYOUTPRINTCONFIG" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_LAYOUTPRINTCONFIG *)dwg_object_to_LAYOUTPRINTCONFIG(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_LAYOUTPRINTCONFIG, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_PLANESURFACE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_PLANESURFACE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_PLANESURFACE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_PLANESURFACE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_PLANESURFACE *)dwg_object_to_PLANESURFACE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_PLANESURFACE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_POINTCLOUD) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_POINTCLOUD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_POINTCLOUD(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_POINTCLOUD" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_POINTCLOUD *)dwg_object_to_POINTCLOUD(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_POINTCLOUD, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_POINTCLOUDEX) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Entity_POINTCLOUDEX *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_POINTCLOUDEX(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_POINTCLOUDEX" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Entity_POINTCLOUDEX *)dwg_object_to_POINTCLOUDEX(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_entity_POINTCLOUDEX, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ACSH_BOOLEAN_CLASS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_BOOLEAN_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ACSH_BOOLEAN_CLASS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ACSH_BOOLEAN_CLASS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ACSH_BOOLEAN_CLASS *)dwg_object_to_ACSH_BOOLEAN_CLASS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_BOOLEAN_CLASS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ACSH_BOX_CLASS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_BOX_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ACSH_BOX_CLASS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ACSH_BOX_CLASS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ACSH_BOX_CLASS *)dwg_object_to_ACSH_BOX_CLASS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_BOX_CLASS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ACSH_CONE_CLASS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_CONE_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ACSH_CONE_CLASS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ACSH_CONE_CLASS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ACSH_CONE_CLASS *)dwg_object_to_ACSH_CONE_CLASS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_CONE_CLASS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ACSH_CYLINDER_CLASS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_CYLINDER_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ACSH_CYLINDER_CLASS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ACSH_CYLINDER_CLASS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ACSH_CYLINDER_CLASS *)dwg_object_to_ACSH_CYLINDER_CLASS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_CYLINDER_CLASS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ACSH_FILLET_CLASS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_FILLET_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ACSH_FILLET_CLASS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ACSH_FILLET_CLASS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ACSH_FILLET_CLASS *)dwg_object_to_ACSH_FILLET_CLASS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_FILLET_CLASS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ACSH_HISTORY_CLASS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_HISTORY_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ACSH_HISTORY_CLASS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ACSH_HISTORY_CLASS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ACSH_HISTORY_CLASS *)dwg_object_to_ACSH_HISTORY_CLASS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_HISTORY_CLASS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ACSH_SPHERE_CLASS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_SPHERE_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ACSH_SPHERE_CLASS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ACSH_SPHERE_CLASS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ACSH_SPHERE_CLASS *)dwg_object_to_ACSH_SPHERE_CLASS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_SPHERE_CLASS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ACSH_TORUS_CLASS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_TORUS_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ACSH_TORUS_CLASS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ACSH_TORUS_CLASS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ACSH_TORUS_CLASS *)dwg_object_to_ACSH_TORUS_CLASS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_TORUS_CLASS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ACSH_WEDGE_CLASS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_WEDGE_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ACSH_WEDGE_CLASS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ACSH_WEDGE_CLASS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ACSH_WEDGE_CLASS *)dwg_object_to_ACSH_WEDGE_CLASS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_WEDGE_CLASS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_APPID) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_APPID *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_APPID(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_APPID" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_APPID *)dwg_object_to_APPID(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_APPID, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_APPID_CONTROL) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_APPID_CONTROL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_APPID_CONTROL(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_APPID_CONTROL" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_APPID_CONTROL *)dwg_object_to_APPID_CONTROL(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_APPID_CONTROL, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ASSOCGEOMDEPENDENCY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCGEOMDEPENDENCY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ASSOCGEOMDEPENDENCY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ASSOCGEOMDEPENDENCY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ASSOCGEOMDEPENDENCY *)dwg_object_to_ASSOCGEOMDEPENDENCY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCGEOMDEPENDENCY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ASSOCNETWORK) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCNETWORK *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ASSOCNETWORK(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ASSOCNETWORK" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ASSOCNETWORK *)dwg_object_to_ASSOCNETWORK(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCNETWORK, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKALIGNMENTGRIP) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKALIGNMENTGRIP *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKALIGNMENTGRIP(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKALIGNMENTGRIP" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKALIGNMENTGRIP *)dwg_object_to_BLOCKALIGNMENTGRIP(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKALIGNMENTGRIP, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKALIGNMENTPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKALIGNMENTPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKALIGNMENTPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKALIGNMENTPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKALIGNMENTPARAMETER *)dwg_object_to_BLOCKALIGNMENTPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKBASEPOINTPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKBASEPOINTPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKBASEPOINTPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKBASEPOINTPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKBASEPOINTPARAMETER *)dwg_object_to_BLOCKBASEPOINTPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKBASEPOINTPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKFLIPACTION) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKFLIPACTION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKFLIPACTION(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKFLIPACTION" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKFLIPACTION *)dwg_object_to_BLOCKFLIPACTION(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKFLIPGRIP) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKFLIPGRIP *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKFLIPGRIP(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKFLIPGRIP" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKFLIPGRIP *)dwg_object_to_BLOCKFLIPGRIP(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKFLIPPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKFLIPPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKFLIPPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKFLIPPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKFLIPPARAMETER *)dwg_object_to_BLOCKFLIPPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKGRIPLOCATIONCOMPONENT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKGRIPLOCATIONCOMPONENT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKGRIPLOCATIONCOMPONENT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT *)dwg_object_to_BLOCKGRIPLOCATIONCOMPONENT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKGRIPLOCATIONCOMPONENT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKLINEARGRIP) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKLINEARGRIP *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKLINEARGRIP(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKLINEARGRIP" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKLINEARGRIP *)dwg_object_to_BLOCKLINEARGRIP(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKLINEARGRIP, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKLOOKUPGRIP) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKLOOKUPGRIP *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKLOOKUPGRIP(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKLOOKUPGRIP" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKLOOKUPGRIP *)dwg_object_to_BLOCKLOOKUPGRIP(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKLOOKUPGRIP, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKMOVEACTION) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKMOVEACTION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKMOVEACTION(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKMOVEACTION" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKMOVEACTION *)dwg_object_to_BLOCKMOVEACTION(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKROTATEACTION) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKROTATEACTION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKROTATEACTION(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKROTATEACTION" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKROTATEACTION *)dwg_object_to_BLOCKROTATEACTION(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKROTATIONGRIP) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKROTATIONGRIP *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKROTATIONGRIP(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKROTATIONGRIP" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKROTATIONGRIP *)dwg_object_to_BLOCKROTATIONGRIP(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKROTATIONGRIP, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKSCALEACTION) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKSCALEACTION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKSCALEACTION(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKSCALEACTION" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKSCALEACTION *)dwg_object_to_BLOCKSCALEACTION(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKVISIBILITYGRIP) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKVISIBILITYGRIP *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKVISIBILITYGRIP(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKVISIBILITYGRIP" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKVISIBILITYGRIP *)dwg_object_to_BLOCKVISIBILITYGRIP(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKVISIBILITYGRIP, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCK_CONTROL) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCK_CONTROL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCK_CONTROL(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCK_CONTROL" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCK_CONTROL *)dwg_object_to_BLOCK_CONTROL(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCK_CONTROL, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCK_HEADER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCK_HEADER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCK_HEADER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCK_HEADER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCK_HEADER *)dwg_object_to_BLOCK_HEADER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCK_HEADER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_CELLSTYLEMAP) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_CELLSTYLEMAP *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_CELLSTYLEMAP(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_CELLSTYLEMAP" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_CELLSTYLEMAP *)dwg_object_to_CELLSTYLEMAP(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_CELLSTYLEMAP, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_DETAILVIEWSTYLE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DETAILVIEWSTYLE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_DETAILVIEWSTYLE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_DETAILVIEWSTYLE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_DETAILVIEWSTYLE *)dwg_object_to_DETAILVIEWSTYLE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_DICTIONARY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DICTIONARY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_DICTIONARY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_DICTIONARY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_DICTIONARY *)dwg_object_to_DICTIONARY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DICTIONARY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_DICTIONARYVAR) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DICTIONARYVAR *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_DICTIONARYVAR(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_DICTIONARYVAR" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_DICTIONARYVAR *)dwg_object_to_DICTIONARYVAR(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DICTIONARYVAR, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_DICTIONARYWDFLT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DICTIONARYWDFLT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_DICTIONARYWDFLT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_DICTIONARYWDFLT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_DICTIONARYWDFLT *)dwg_object_to_DICTIONARYWDFLT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DICTIONARYWDFLT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_DIMSTYLE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DIMSTYLE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_DIMSTYLE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_DIMSTYLE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_DIMSTYLE *)dwg_object_to_DIMSTYLE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DIMSTYLE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_DIMSTYLE_CONTROL) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DIMSTYLE_CONTROL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_DIMSTYLE_CONTROL(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_DIMSTYLE_CONTROL" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_DIMSTYLE_CONTROL *)dwg_object_to_DIMSTYLE_CONTROL(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DIMSTYLE_CONTROL, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_DUMMY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DUMMY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_DUMMY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_DUMMY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_DUMMY *)dwg_object_to_DUMMY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DUMMY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_DYNAMICBLOCKPURGEPREVENTER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DYNAMICBLOCKPURGEPREVENTER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_DYNAMICBLOCKPURGEPREVENTER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_DYNAMICBLOCKPURGEPREVENTER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_DYNAMICBLOCKPURGEPREVENTER *)dwg_object_to_DYNAMICBLOCKPURGEPREVENTER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DYNAMICBLOCKPURGEPREVENTER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_FIELD) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_FIELD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_FIELD(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_FIELD" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_FIELD *)dwg_object_to_FIELD(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_FIELD, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_FIELDLIST) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_FIELDLIST *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_FIELDLIST(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_FIELDLIST" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_FIELDLIST *)dwg_object_to_FIELDLIST(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_FIELDLIST, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_GEODATA) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_GEODATA *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_GEODATA(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_GEODATA" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_GEODATA *)dwg_object_to_GEODATA(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_GEODATA, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_GROUP) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_GROUP *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_GROUP(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_GROUP" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_GROUP *)dwg_object_to_GROUP(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_GROUP, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_IDBUFFER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_IDBUFFER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_IDBUFFER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_IDBUFFER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_IDBUFFER *)dwg_object_to_IDBUFFER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_IDBUFFER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_IMAGEDEF) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_IMAGEDEF *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_IMAGEDEF(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_IMAGEDEF" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_IMAGEDEF *)dwg_object_to_IMAGEDEF(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_IMAGEDEF, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_IMAGEDEF_REACTOR) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_IMAGEDEF_REACTOR *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_IMAGEDEF_REACTOR(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_IMAGEDEF_REACTOR" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_IMAGEDEF_REACTOR *)dwg_object_to_IMAGEDEF_REACTOR(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_IMAGEDEF_REACTOR, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_INDEX) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_INDEX *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_INDEX(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_INDEX" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_INDEX *)dwg_object_to_INDEX(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_INDEX, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_LAYER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_LAYER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_LAYER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_LAYER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_LAYER *)dwg_object_to_LAYER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_LAYER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_LAYERFILTER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_LAYERFILTER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_LAYERFILTER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_LAYERFILTER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_LAYERFILTER *)dwg_object_to_LAYERFILTER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_LAYERFILTER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_LAYER_CONTROL) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_LAYER_CONTROL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_LAYER_CONTROL(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_LAYER_CONTROL" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_LAYER_CONTROL *)dwg_object_to_LAYER_CONTROL(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_LAYER_CONTROL, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_LAYER_INDEX) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_LAYER_INDEX *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_LAYER_INDEX(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_LAYER_INDEX" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_LAYER_INDEX *)dwg_object_to_LAYER_INDEX(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_LAYER_INDEX, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_LAYOUT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_LAYOUT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_LAYOUT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_LAYOUT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_LAYOUT *)dwg_object_to_LAYOUT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_LAYOUT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_LONG_TRANSACTION) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_LONG_TRANSACTION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_LONG_TRANSACTION(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_LONG_TRANSACTION" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_LONG_TRANSACTION *)dwg_object_to_LONG_TRANSACTION(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_LONG_TRANSACTION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_LTYPE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_LTYPE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_LTYPE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_LTYPE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_LTYPE *)dwg_object_to_LTYPE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_LTYPE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_LTYPE_CONTROL) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_LTYPE_CONTROL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_LTYPE_CONTROL(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_LTYPE_CONTROL" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_LTYPE_CONTROL *)dwg_object_to_LTYPE_CONTROL(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_LTYPE_CONTROL, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_MLINESTYLE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_MLINESTYLE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_MLINESTYLE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_MLINESTYLE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_MLINESTYLE *)dwg_object_to_MLINESTYLE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_MLINESTYLE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_PLACEHOLDER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_PLACEHOLDER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_PLACEHOLDER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_PLACEHOLDER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_PLACEHOLDER *)dwg_object_to_PLACEHOLDER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_PLACEHOLDER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_PLOTSETTINGS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_PLOTSETTINGS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_PLOTSETTINGS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_PLOTSETTINGS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_PLOTSETTINGS *)dwg_object_to_PLOTSETTINGS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_PLOTSETTINGS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_RASTERVARIABLES) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_RASTERVARIABLES *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_RASTERVARIABLES(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_RASTERVARIABLES" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_RASTERVARIABLES *)dwg_object_to_RASTERVARIABLES(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_RASTERVARIABLES, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_RENDERENVIRONMENT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_RENDERENVIRONMENT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_RENDERENVIRONMENT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_RENDERENVIRONMENT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_RENDERENVIRONMENT *)dwg_object_to_RENDERENVIRONMENT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_RENDERENVIRONMENT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_SCALE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_SCALE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_SCALE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_SCALE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_SCALE *)dwg_object_to_SCALE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_SCALE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_SECTIONVIEWSTYLE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_SECTIONVIEWSTYLE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_SECTIONVIEWSTYLE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_SECTIONVIEWSTYLE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_SECTIONVIEWSTYLE *)dwg_object_to_SECTIONVIEWSTYLE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_SECTION_MANAGER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_SECTION_MANAGER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_SECTION_MANAGER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_SECTION_MANAGER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_SECTION_MANAGER *)dwg_object_to_SECTION_MANAGER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_SECTION_MANAGER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_SORTENTSTABLE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_SORTENTSTABLE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_SORTENTSTABLE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_SORTENTSTABLE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_SORTENTSTABLE *)dwg_object_to_SORTENTSTABLE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_SORTENTSTABLE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_SPATIAL_FILTER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_SPATIAL_FILTER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_SPATIAL_FILTER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_SPATIAL_FILTER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_SPATIAL_FILTER *)dwg_object_to_SPATIAL_FILTER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_SPATIAL_FILTER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_STYLE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_STYLE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_STYLE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_STYLE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_STYLE *)dwg_object_to_STYLE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_STYLE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_STYLE_CONTROL) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_STYLE_CONTROL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_STYLE_CONTROL(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_STYLE_CONTROL" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_STYLE_CONTROL *)dwg_object_to_STYLE_CONTROL(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_STYLE_CONTROL, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_SUN) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_SUN *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_SUN(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_SUN" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_SUN *)dwg_object_to_SUN(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_SUN, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_TABLEGEOMETRY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_TABLEGEOMETRY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_TABLEGEOMETRY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_TABLEGEOMETRY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_TABLEGEOMETRY *)dwg_object_to_TABLEGEOMETRY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_TABLEGEOMETRY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_UCS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_UCS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_UCS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_UCS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_UCS *)dwg_object_to_UCS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_UCS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_UCS_CONTROL) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_UCS_CONTROL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_UCS_CONTROL(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_UCS_CONTROL" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_UCS_CONTROL *)dwg_object_to_UCS_CONTROL(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_UCS_CONTROL, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_UNKNOWN_OBJ) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_UNKNOWN_OBJ *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_UNKNOWN_OBJ(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_UNKNOWN_OBJ" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_UNKNOWN_OBJ *)dwg_object_to_UNKNOWN_OBJ(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_UNKNOWN_OBJ, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_VBA_PROJECT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_VBA_PROJECT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_VBA_PROJECT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_VBA_PROJECT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_VBA_PROJECT *)dwg_object_to_VBA_PROJECT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_VBA_PROJECT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_VIEW) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_VIEW *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_VIEW(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_VIEW" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_VIEW *)dwg_object_to_VIEW(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_VIEW, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_VIEW_CONTROL) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_VIEW_CONTROL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_VIEW_CONTROL(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_VIEW_CONTROL" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_VIEW_CONTROL *)dwg_object_to_VIEW_CONTROL(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_VIEW_CONTROL, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_VISUALSTYLE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_VISUALSTYLE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_VISUALSTYLE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_VISUALSTYLE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_VISUALSTYLE *)dwg_object_to_VISUALSTYLE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_VISUALSTYLE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_VPORT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_VPORT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_VPORT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_VPORT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_VPORT *)dwg_object_to_VPORT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_VPORT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_VPORT_CONTROL) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_VPORT_CONTROL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_VPORT_CONTROL(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_VPORT_CONTROL" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_VPORT_CONTROL *)dwg_object_to_VPORT_CONTROL(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_VPORT_CONTROL, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_VX_CONTROL) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_VX_CONTROL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_VX_CONTROL(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_VX_CONTROL" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_VX_CONTROL *)dwg_object_to_VX_CONTROL(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_VX_CONTROL, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_VX_TABLE_RECORD) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_VX_TABLE_RECORD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_VX_TABLE_RECORD(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_VX_TABLE_RECORD" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_VX_TABLE_RECORD *)dwg_object_to_VX_TABLE_RECORD(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_VX_TABLE_RECORD, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_WIPEOUTVARIABLES) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_WIPEOUTVARIABLES *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_WIPEOUTVARIABLES(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_WIPEOUTVARIABLES" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_WIPEOUTVARIABLES *)dwg_object_to_WIPEOUTVARIABLES(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_WIPEOUTVARIABLES, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_XRECORD) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_XRECORD *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_XRECORD(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_XRECORD" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_XRECORD *)dwg_object_to_XRECORD(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_XRECORD, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_PDFDEFINITION) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_PDFDEFINITION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_PDFDEFINITION(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_PDFDEFINITION" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_PDFDEFINITION *)dwg_object_to_PDFDEFINITION(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_abstractobject_UNDERLAYDEFINITION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_DGNDEFINITION) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DGNDEFINITION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_DGNDEFINITION(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_DGNDEFINITION" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_DGNDEFINITION *)dwg_object_to_DGNDEFINITION(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_abstractobject_UNDERLAYDEFINITION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_DWFDEFINITION) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DWFDEFINITION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_DWFDEFINITION(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_DWFDEFINITION" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_DWFDEFINITION *)dwg_object_to_DWFDEFINITION(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_abstractobject_UNDERLAYDEFINITION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ACSH_BREP_CLASS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_BREP_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ACSH_BREP_CLASS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ACSH_BREP_CLASS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ACSH_BREP_CLASS *)dwg_object_to_ACSH_BREP_CLASS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ACSH_CHAMFER_CLASS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_CHAMFER_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ACSH_CHAMFER_CLASS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ACSH_CHAMFER_CLASS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ACSH_CHAMFER_CLASS *)dwg_object_to_ACSH_CHAMFER_CLASS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_CHAMFER_CLASS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ACSH_PYRAMID_CLASS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ACSH_PYRAMID_CLASS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ACSH_PYRAMID_CLASS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ACSH_PYRAMID_CLASS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ACSH_PYRAMID_CLASS *)dwg_object_to_ACSH_PYRAMID_CLASS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ACSH_PYRAMID_CLASS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ALDIMOBJECTCONTEXTDATA) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ALDIMOBJECTCONTEXTDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ALDIMOBJECTCONTEXTDATA(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ALDIMOBJECTCONTEXTDATA" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ALDIMOBJECTCONTEXTDATA *)dwg_object_to_ALDIMOBJECTCONTEXTDATA(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ALDIMOBJECTCONTEXTDATA, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ASSOC2DCONSTRAINTGROUP) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOC2DCONSTRAINTGROUP *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ASSOC2DCONSTRAINTGROUP(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ASSOC2DCONSTRAINTGROUP" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ASSOC2DCONSTRAINTGROUP *)dwg_object_to_ASSOC2DCONSTRAINTGROUP(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ASSOCACTION) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCACTION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ASSOCACTION(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ASSOCACTION" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ASSOCACTION *)dwg_object_to_ASSOCACTION(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCACTION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ASSOCACTIONPARAM) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCACTIONPARAM *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ASSOCACTIONPARAM(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ASSOCACTIONPARAM" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ASSOCACTIONPARAM *)dwg_object_to_ASSOCACTIONPARAM(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCACTIONPARAM, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ASSOCARRAYACTIONBODY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCARRAYACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ASSOCARRAYACTIONBODY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ASSOCARRAYACTIONBODY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ASSOCARRAYACTIONBODY *)dwg_object_to_ASSOCARRAYACTIONBODY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCARRAYACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ASSOCASMBODYACTIONPARAM) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCASMBODYACTIONPARAM *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ASSOCASMBODYACTIONPARAM(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ASSOCASMBODYACTIONPARAM" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ASSOCASMBODYACTIONPARAM *)dwg_object_to_ASSOCASMBODYACTIONPARAM(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ASSOCBLENDSURFACEACTIONBODY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCBLENDSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ASSOCBLENDSURFACEACTIONBODY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ASSOCBLENDSURFACEACTIONBODY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ASSOCBLENDSURFACEACTIONBODY *)dwg_object_to_ASSOCBLENDSURFACEACTIONBODY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ASSOCCOMPOUNDACTIONPARAM) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCCOMPOUNDACTIONPARAM *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ASSOCCOMPOUNDACTIONPARAM(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ASSOCCOMPOUNDACTIONPARAM" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ASSOCCOMPOUNDACTIONPARAM *)dwg_object_to_ASSOCCOMPOUNDACTIONPARAM(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ASSOCDEPENDENCY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCDEPENDENCY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ASSOCDEPENDENCY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ASSOCDEPENDENCY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ASSOCDEPENDENCY *)dwg_object_to_ASSOCDEPENDENCY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ASSOCDIMDEPENDENCYBODY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCDIMDEPENDENCYBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ASSOCDIMDEPENDENCYBODY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ASSOCDIMDEPENDENCYBODY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ASSOCDIMDEPENDENCYBODY *)dwg_object_to_ASSOCDIMDEPENDENCYBODY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCDIMDEPENDENCYBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ASSOCEXTENDSURFACEACTIONBODY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ASSOCEXTENDSURFACEACTIONBODY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ASSOCEXTENDSURFACEACTIONBODY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY *)dwg_object_to_ASSOCEXTENDSURFACEACTIONBODY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCEXTENDSURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ASSOCEXTRUDEDSURFACEACTIONBODY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ASSOCEXTRUDEDSURFACEACTIONBODY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ASSOCEXTRUDEDSURFACEACTIONBODY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY *)dwg_object_to_ASSOCEXTRUDEDSURFACEACTIONBODY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ASSOCFACEACTIONPARAM) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCFACEACTIONPARAM *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ASSOCFACEACTIONPARAM(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ASSOCFACEACTIONPARAM" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ASSOCFACEACTIONPARAM *)dwg_object_to_ASSOCFACEACTIONPARAM(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCFACEACTIONPARAM, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ASSOCFILLETSURFACEACTIONBODY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCFILLETSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ASSOCFILLETSURFACEACTIONBODY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ASSOCFILLETSURFACEACTIONBODY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ASSOCFILLETSURFACEACTIONBODY *)dwg_object_to_ASSOCFILLETSURFACEACTIONBODY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCFILLETSURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ASSOCLOFTEDSURFACEACTIONBODY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ASSOCLOFTEDSURFACEACTIONBODY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ASSOCLOFTEDSURFACEACTIONBODY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY *)dwg_object_to_ASSOCLOFTEDSURFACEACTIONBODY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCLOFTEDSURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ASSOCNETWORKSURFACEACTIONBODY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ASSOCNETWORKSURFACEACTIONBODY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ASSOCNETWORKSURFACEACTIONBODY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY *)dwg_object_to_ASSOCNETWORKSURFACEACTIONBODY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCNETWORKSURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ASSOCOBJECTACTIONPARAM) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCOBJECTACTIONPARAM *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ASSOCOBJECTACTIONPARAM(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ASSOCOBJECTACTIONPARAM" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ASSOCOBJECTACTIONPARAM *)dwg_object_to_ASSOCOBJECTACTIONPARAM(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCOBJECTACTIONPARAM, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ASSOCOFFSETSURFACEACTIONBODY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ASSOCOFFSETSURFACEACTIONBODY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ASSOCOFFSETSURFACEACTIONBODY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY *)dwg_object_to_ASSOCOFFSETSURFACEACTIONBODY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCOFFSETSURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ASSOCOSNAPPOINTREFACTIONPARAM) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ASSOCOSNAPPOINTREFACTIONPARAM(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ASSOCOSNAPPOINTREFACTIONPARAM" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM *)dwg_object_to_ASSOCOSNAPPOINTREFACTIONPARAM(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ASSOCPATCHSURFACEACTIONBODY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCPATCHSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ASSOCPATCHSURFACEACTIONBODY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ASSOCPATCHSURFACEACTIONBODY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ASSOCPATCHSURFACEACTIONBODY *)dwg_object_to_ASSOCPATCHSURFACEACTIONBODY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCPATCHSURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ASSOCPATHACTIONPARAM) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCPATHACTIONPARAM *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ASSOCPATHACTIONPARAM(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ASSOCPATHACTIONPARAM" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ASSOCPATHACTIONPARAM *)dwg_object_to_ASSOCPATHACTIONPARAM(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ASSOCPLANESURFACEACTIONBODY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCPLANESURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ASSOCPLANESURFACEACTIONBODY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ASSOCPLANESURFACEACTIONBODY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ASSOCPLANESURFACEACTIONBODY *)dwg_object_to_ASSOCPLANESURFACEACTIONBODY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCPLANESURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ASSOCPOINTREFACTIONPARAM) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCPOINTREFACTIONPARAM *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ASSOCPOINTREFACTIONPARAM(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ASSOCPOINTREFACTIONPARAM" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ASSOCPOINTREFACTIONPARAM *)dwg_object_to_ASSOCPOINTREFACTIONPARAM(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ASSOCREVOLVEDSURFACEACTIONBODY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ASSOCREVOLVEDSURFACEACTIONBODY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ASSOCREVOLVEDSURFACEACTIONBODY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY *)dwg_object_to_ASSOCREVOLVEDSURFACEACTIONBODY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ASSOCTRIMSURFACEACTIONBODY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCTRIMSURFACEACTIONBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ASSOCTRIMSURFACEACTIONBODY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ASSOCTRIMSURFACEACTIONBODY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ASSOCTRIMSURFACEACTIONBODY *)dwg_object_to_ASSOCTRIMSURFACEACTIONBODY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCTRIMSURFACEACTIONBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ASSOCVALUEDEPENDENCY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCVALUEDEPENDENCY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ASSOCVALUEDEPENDENCY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ASSOCVALUEDEPENDENCY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ASSOCVALUEDEPENDENCY *)dwg_object_to_ASSOCVALUEDEPENDENCY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCVALUEDEPENDENCY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ASSOCVARIABLE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCVARIABLE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ASSOCVARIABLE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ASSOCVARIABLE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ASSOCVARIABLE *)dwg_object_to_ASSOCVARIABLE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCVARIABLE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ASSOCVERTEXACTIONPARAM) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCVERTEXACTIONPARAM *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ASSOCVERTEXACTIONPARAM(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ASSOCVERTEXACTIONPARAM" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ASSOCVERTEXACTIONPARAM *)dwg_object_to_ASSOCVERTEXACTIONPARAM(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_ASSOCVERTEXACTIONPARAM, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLKREFOBJECTCONTEXTDATA) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLKREFOBJECTCONTEXTDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLKREFOBJECTCONTEXTDATA(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLKREFOBJECTCONTEXTDATA" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLKREFOBJECTCONTEXTDATA *)dwg_object_to_BLKREFOBJECTCONTEXTDATA(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLKREFOBJECTCONTEXTDATA, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKALIGNEDCONSTRAINTPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKALIGNEDCONSTRAINTPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKALIGNEDCONSTRAINTPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER *)dwg_object_to_BLOCKALIGNEDCONSTRAINTPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKANGULARCONSTRAINTPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKANGULARCONSTRAINTPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKANGULARCONSTRAINTPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER *)dwg_object_to_BLOCKANGULARCONSTRAINTPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKARRAYACTION) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKARRAYACTION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKARRAYACTION(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKARRAYACTION" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKARRAYACTION *)dwg_object_to_BLOCKARRAYACTION(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKARRAYACTION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKDIAMETRICCONSTRAINTPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKDIAMETRICCONSTRAINTPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKDIAMETRICCONSTRAINTPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER *)dwg_object_to_BLOCKDIAMETRICCONSTRAINTPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKHORIZONTALCONSTRAINTPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKHORIZONTALCONSTRAINTPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKHORIZONTALCONSTRAINTPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER *)dwg_object_to_BLOCKHORIZONTALCONSTRAINTPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKLINEARCONSTRAINTPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKLINEARCONSTRAINTPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKLINEARCONSTRAINTPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER *)dwg_object_to_BLOCKLINEARCONSTRAINTPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKLINEARPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKLINEARPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKLINEARPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKLINEARPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKLINEARPARAMETER *)dwg_object_to_BLOCKLINEARPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKLOOKUPACTION) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKLOOKUPACTION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKLOOKUPACTION(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKLOOKUPACTION" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKLOOKUPACTION *)dwg_object_to_BLOCKLOOKUPACTION(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKLOOKUPPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKLOOKUPPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKLOOKUPPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKLOOKUPPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKLOOKUPPARAMETER *)dwg_object_to_BLOCKLOOKUPPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKPARAMDEPENDENCYBODY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKPARAMDEPENDENCYBODY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKPARAMDEPENDENCYBODY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKPARAMDEPENDENCYBODY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKPARAMDEPENDENCYBODY *)dwg_object_to_BLOCKPARAMDEPENDENCYBODY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKPARAMDEPENDENCYBODY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKPOINTPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKPOINTPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKPOINTPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKPOINTPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKPOINTPARAMETER *)dwg_object_to_BLOCKPOINTPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKPOLARGRIP) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKPOLARGRIP *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKPOLARGRIP(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKPOLARGRIP" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKPOLARGRIP *)dwg_object_to_BLOCKPOLARGRIP(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKPOLARGRIP, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKPOLARPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKPOLARPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKPOLARPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKPOLARPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKPOLARPARAMETER *)dwg_object_to_BLOCKPOLARPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKPOLARSTRETCHACTION) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKPOLARSTRETCHACTION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKPOLARSTRETCHACTION(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKPOLARSTRETCHACTION" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKPOLARSTRETCHACTION *)dwg_object_to_BLOCKPOLARSTRETCHACTION(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKRADIALCONSTRAINTPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKRADIALCONSTRAINTPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKRADIALCONSTRAINTPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER *)dwg_object_to_BLOCKRADIALCONSTRAINTPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKREPRESENTATION) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKREPRESENTATION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKREPRESENTATION(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKREPRESENTATION" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKREPRESENTATION *)dwg_object_to_BLOCKREPRESENTATION(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKREPRESENTATION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKROTATIONPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKROTATIONPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKROTATIONPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKROTATIONPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKROTATIONPARAMETER *)dwg_object_to_BLOCKROTATIONPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKSTRETCHACTION) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKSTRETCHACTION *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKSTRETCHACTION(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKSTRETCHACTION" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKSTRETCHACTION *)dwg_object_to_BLOCKSTRETCHACTION(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKUSERPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKUSERPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKUSERPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKUSERPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKUSERPARAMETER *)dwg_object_to_BLOCKUSERPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKVERTICALCONSTRAINTPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKVERTICALCONSTRAINTPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKVERTICALCONSTRAINTPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER *)dwg_object_to_BLOCKVERTICALCONSTRAINTPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKVISIBILITYPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKVISIBILITYPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKVISIBILITYPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKVISIBILITYPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKVISIBILITYPARAMETER *)dwg_object_to_BLOCKVISIBILITYPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKXYGRIP) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKXYGRIP *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKXYGRIP(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKXYGRIP" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKXYGRIP *)dwg_object_to_BLOCKXYGRIP(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKXYGRIP, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_BLOCKXYPARAMETER) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_BLOCKXYPARAMETER *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_BLOCKXYPARAMETER(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_BLOCKXYPARAMETER" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_BLOCKXYPARAMETER *)dwg_object_to_BLOCKXYPARAMETER(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_DATALINK) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DATALINK *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_DATALINK(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_DATALINK" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_DATALINK *)dwg_object_to_DATALINK(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DATALINK, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_DBCOLOR) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_DBCOLOR *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_DBCOLOR(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_DBCOLOR" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_DBCOLOR *)dwg_object_to_DBCOLOR(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_DBCOLOR, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_EVALUATION_GRAPH) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_EVALUATION_GRAPH *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_EVALUATION_GRAPH(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_EVALUATION_GRAPH" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_EVALUATION_GRAPH *)dwg_object_to_EVALUATION_GRAPH(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_EVALUATION_GRAPH, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_FCFOBJECTCONTEXTDATA) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_FCFOBJECTCONTEXTDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_FCFOBJECTCONTEXTDATA(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_FCFOBJECTCONTEXTDATA" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_FCFOBJECTCONTEXTDATA *)dwg_object_to_FCFOBJECTCONTEXTDATA(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_FCFOBJECTCONTEXTDATA, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_GRADIENT_BACKGROUND) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_GRADIENT_BACKGROUND *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_GRADIENT_BACKGROUND(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_GRADIENT_BACKGROUND" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_GRADIENT_BACKGROUND *)dwg_object_to_GRADIENT_BACKGROUND(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_GRADIENT_BACKGROUND, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_GROUND_PLANE_BACKGROUND) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_GROUND_PLANE_BACKGROUND *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_GROUND_PLANE_BACKGROUND(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_GROUND_PLANE_BACKGROUND" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_GROUND_PLANE_BACKGROUND *)dwg_object_to_GROUND_PLANE_BACKGROUND(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_GROUND_PLANE_BACKGROUND, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_IBL_BACKGROUND) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_IBL_BACKGROUND *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_IBL_BACKGROUND(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_IBL_BACKGROUND" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_IBL_BACKGROUND *)dwg_object_to_IBL_BACKGROUND(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_IBL_BACKGROUND, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_IMAGE_BACKGROUND) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_IMAGE_BACKGROUND *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_IMAGE_BACKGROUND(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_IMAGE_BACKGROUND" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_IMAGE_BACKGROUND *)dwg_object_to_IMAGE_BACKGROUND(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_IMAGE_BACKGROUND, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_LEADEROBJECTCONTEXTDATA) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_LEADEROBJECTCONTEXTDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_LEADEROBJECTCONTEXTDATA(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_LEADEROBJECTCONTEXTDATA" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_LEADEROBJECTCONTEXTDATA *)dwg_object_to_LEADEROBJECTCONTEXTDATA(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_LEADEROBJECTCONTEXTDATA, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_LIGHTLIST) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_LIGHTLIST *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_LIGHTLIST(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_LIGHTLIST" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_LIGHTLIST *)dwg_object_to_LIGHTLIST(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_LIGHTLIST, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_MATERIAL) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_MATERIAL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_MATERIAL(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_MATERIAL" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_MATERIAL *)dwg_object_to_MATERIAL(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_MATERIAL, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_MENTALRAYRENDERSETTINGS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_MENTALRAYRENDERSETTINGS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_MENTALRAYRENDERSETTINGS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_MENTALRAYRENDERSETTINGS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_MENTALRAYRENDERSETTINGS *)dwg_object_to_MENTALRAYRENDERSETTINGS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_MLEADERSTYLE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_MLEADERSTYLE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_MLEADERSTYLE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_MLEADERSTYLE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_MLEADERSTYLE *)dwg_object_to_MLEADERSTYLE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_MLEADERSTYLE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_MTEXTOBJECTCONTEXTDATA) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_MTEXTOBJECTCONTEXTDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_MTEXTOBJECTCONTEXTDATA(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_MTEXTOBJECTCONTEXTDATA" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_MTEXTOBJECTCONTEXTDATA *)dwg_object_to_MTEXTOBJECTCONTEXTDATA(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_OBJECT_PTR) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_OBJECT_PTR *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_OBJECT_PTR(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_OBJECT_PTR" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_OBJECT_PTR *)dwg_object_to_OBJECT_PTR(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_OBJECT_PTR, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_PARTIAL_VIEWING_INDEX) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_PARTIAL_VIEWING_INDEX *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_PARTIAL_VIEWING_INDEX(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_PARTIAL_VIEWING_INDEX" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_PARTIAL_VIEWING_INDEX *)dwg_object_to_PARTIAL_VIEWING_INDEX(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_PARTIAL_VIEWING_INDEX, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_POINTCLOUDCOLORMAP) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_POINTCLOUDCOLORMAP *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_POINTCLOUDCOLORMAP(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_POINTCLOUDCOLORMAP" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_POINTCLOUDCOLORMAP *)dwg_object_to_POINTCLOUDCOLORMAP(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_POINTCLOUDCOLORMAP, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_POINTCLOUDDEF) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_POINTCLOUDDEF *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_POINTCLOUDDEF(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_POINTCLOUDDEF" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_POINTCLOUDDEF *)dwg_object_to_POINTCLOUDDEF(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_POINTCLOUDDEF, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_POINTCLOUDDEFEX) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_POINTCLOUDDEFEX *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_POINTCLOUDDEFEX(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_POINTCLOUDDEFEX" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_POINTCLOUDDEFEX *)dwg_object_to_POINTCLOUDDEFEX(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_POINTCLOUDDEFEX, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_POINTCLOUDDEF_REACTOR) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_POINTCLOUDDEF_REACTOR *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_POINTCLOUDDEF_REACTOR(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_POINTCLOUDDEF_REACTOR" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_POINTCLOUDDEF_REACTOR *)dwg_object_to_POINTCLOUDDEF_REACTOR(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_POINTCLOUDDEF_REACTOR, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_POINTCLOUDDEF_REACTOR_EX) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_POINTCLOUDDEF_REACTOR_EX *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_POINTCLOUDDEF_REACTOR_EX(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_POINTCLOUDDEF_REACTOR_EX" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_POINTCLOUDDEF_REACTOR_EX *)dwg_object_to_POINTCLOUDDEF_REACTOR_EX(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_POINTCLOUDDEF_REACTOR_EX, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_PROXY_OBJECT) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_PROXY_OBJECT *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_PROXY_OBJECT(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_PROXY_OBJECT" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_PROXY_OBJECT *)dwg_object_to_PROXY_OBJECT(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_PROXY_OBJECT, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_RAPIDRTRENDERSETTINGS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_RAPIDRTRENDERSETTINGS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_RAPIDRTRENDERSETTINGS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_RAPIDRTRENDERSETTINGS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_RAPIDRTRENDERSETTINGS *)dwg_object_to_RAPIDRTRENDERSETTINGS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_RENDERENTRY) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_RENDERENTRY *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_RENDERENTRY(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_RENDERENTRY" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_RENDERENTRY *)dwg_object_to_RENDERENTRY(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_RENDERENTRY, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_RENDERGLOBAL) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_RENDERGLOBAL *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_RENDERGLOBAL(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_RENDERGLOBAL" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_RENDERGLOBAL *)dwg_object_to_RENDERGLOBAL(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_RENDERGLOBAL, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_RENDERSETTINGS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_RENDERSETTINGS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_RENDERSETTINGS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_RENDERSETTINGS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_RENDERSETTINGS *)dwg_object_to_RENDERSETTINGS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_RENDERSETTINGS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_SECTION_SETTINGS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_SECTION_SETTINGS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_SECTION_SETTINGS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_SECTION_SETTINGS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_SECTION_SETTINGS *)dwg_object_to_SECTION_SETTINGS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_SECTION_SETTINGS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_SKYLIGHT_BACKGROUND) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_SKYLIGHT_BACKGROUND *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_SKYLIGHT_BACKGROUND(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_SKYLIGHT_BACKGROUND" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_SKYLIGHT_BACKGROUND *)dwg_object_to_SKYLIGHT_BACKGROUND(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_SKYLIGHT_BACKGROUND, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_SOLID_BACKGROUND) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_SOLID_BACKGROUND *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_SOLID_BACKGROUND(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_SOLID_BACKGROUND" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_SOLID_BACKGROUND *)dwg_object_to_SOLID_BACKGROUND(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_SOLID_BACKGROUND, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_SPATIAL_INDEX) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_SPATIAL_INDEX *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_SPATIAL_INDEX(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_SPATIAL_INDEX" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_SPATIAL_INDEX *)dwg_object_to_SPATIAL_INDEX(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_SPATIAL_INDEX, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_TABLESTYLE) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_TABLESTYLE *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_TABLESTYLE(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_TABLESTYLE" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_TABLESTYLE *)dwg_object_to_TABLESTYLE(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_TABLESTYLE, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_TEXTOBJECTCONTEXTDATA) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_TEXTOBJECTCONTEXTDATA *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_TEXTOBJECTCONTEXTDATA(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_TEXTOBJECTCONTEXTDATA" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_TEXTOBJECTCONTEXTDATA *)dwg_object_to_TEXTOBJECTCONTEXTDATA(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_object_TEXTOBJECTCONTEXTDATA, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ASSOCARRAYMODIFYPARAMETERS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCARRAYMODIFYPARAMETERS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ASSOCARRAYMODIFYPARAMETERS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ASSOCARRAYMODIFYPARAMETERS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ASSOCARRAYMODIFYPARAMETERS *)dwg_object_to_ASSOCARRAYMODIFYPARAMETERS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ASSOCARRAYPATHPARAMETERS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCARRAYPATHPARAMETERS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ASSOCARRAYPATHPARAMETERS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ASSOCARRAYPATHPARAMETERS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ASSOCARRAYPATHPARAMETERS *)dwg_object_to_ASSOCARRAYPATHPARAMETERS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ASSOCARRAYPOLARPARAMETERS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCARRAYPOLARPARAMETERS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ASSOCARRAYPOLARPARAMETERS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ASSOCARRAYPOLARPARAMETERS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ASSOCARRAYPOLARPARAMETERS *)dwg_object_to_ASSOCARRAYPOLARPARAMETERS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}


XS(_wrap_dwg_object_to_ASSOCARRAYRECTANGULARPARAMETERS) {
  {
    Dwg_Object *arg1 = (Dwg_Object *) 0 ;
    void *argp1 = 0 ;
    int res1 = 0 ;
    int argvi = 0;
    Dwg_Object_ASSOCARRAYRECTANGULARPARAMETERS *result = 0 ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) {
      SWIG_croak("Usage: dwg_object_to_ASSOCARRAYRECTANGULARPARAMETERS(obj);");
    }
    res1 = SWIG_ConvertPtr(ST(0), &argp1,SWIGTYPE_p__dwg_object, 0 |  0 );
    if (!SWIG_IsOK(res1)) {
      SWIG_exception_fail(SWIG_ArgError(res1), "in method '" "dwg_object_to_ASSOCARRAYRECTANGULARPARAMETERS" "', argument " "1"" of type '" "Dwg_Object *""'"); 
    }
    arg1 = (Dwg_Object *)(argp1);
    result = (Dwg_Object_ASSOCARRAYRECTANGULARPARAMETERS *)dwg_object_to_ASSOCARRAYRECTANGULARPARAMETERS(arg1);
    ST(argvi) = SWIG_NewPointerObj(SWIG_as_voidptr(result), SWIGTYPE_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, 0 | SWIG_SHADOW); argvi++ ;
    
    XSRETURN(argvi);
  fail:
    
    SWIG_croak_null();
  }
}



/* -------- TYPE CONVERSION AND EQUIVALENCE RULES (BEGIN) -------- */

static swig_type_info _swigt__p_DWG_CLASS_STABILITY = {"_p_DWG_CLASS_STABILITY", "Dwg_Class_Stability *|enum DWG_CLASS_STABILITY *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_DWG_COLOR_METHOD = {"_p_DWG_COLOR_METHOD", "Dwg_Color_Method *|enum DWG_COLOR_METHOD *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_DWG_ENTITY_SECTIONS = {"_p_DWG_ENTITY_SECTIONS", "Dwg_Entity_Sections *|enum DWG_ENTITY_SECTIONS *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_DWG_ERROR = {"_p_DWG_ERROR", "Dwg_Error *|enum DWG_ERROR *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_DWG_HDL_CODE = {"_p_DWG_HDL_CODE", "Dwg_Hdl_Code *|enum DWG_HDL_CODE *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_DWG_OBJECT_SUPERTYPE = {"_p_DWG_OBJECT_SUPERTYPE", "Dwg_Object_Supertype *|enum DWG_OBJECT_SUPERTYPE *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_DWG_OBJECT_TYPE = {"_p_DWG_OBJECT_TYPE", "Dwg_Object_Type *|enum DWG_OBJECT_TYPE *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_DWG_OBJECT_TYPE_R11 = {"_p_DWG_OBJECT_TYPE_R11", "Dwg_Object_Type_r11 *|enum DWG_OBJECT_TYPE_R11 *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_DWG_SECTION_TYPE = {"_p_DWG_SECTION_TYPE", "Dwg_Section_Type *|enum DWG_SECTION_TYPE *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_DWG_SECTION_TYPE_R11 = {"_p_DWG_SECTION_TYPE_R11", "Dwg_Section_Type_r11 *|enum DWG_SECTION_TYPE_R11 *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_DWG_SECTION_TYPE_R13 = {"_p_DWG_SECTION_TYPE_R13", "Dwg_Section_Type_r13 *|enum DWG_SECTION_TYPE_R13 *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_DWG_VERSION_TYPE = {"_p_DWG_VERSION_TYPE", "Dwg_Version_Type *|enum DWG_VERSION_TYPE *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_Dwg_Section_Info = {"_p_Dwg_Section_Info", "Dwg_Section_Info *", 0, 0, (void*)"LibreDWG::Dwg_Section_Info", 0};
static swig_type_info _swigt__p_Dwg_Section_InfoHdr = {"_p_Dwg_Section_InfoHdr", "Dwg_Section_InfoHdr *", 0, 0, (void*)"LibreDWG::Dwg_Section_InfoHdr", 0};
static swig_type_info _swigt__p_RESBUF_VALUE_TYPE = {"_p_RESBUF_VALUE_TYPE", "Dwg_Resbuf_Value_Type *|enum RESBUF_VALUE_TYPE *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p__dwg_3DSOLID_material = {"_p__dwg_3DSOLID_material", "Dwg_3DSOLID_material *|struct _dwg_3DSOLID_material *|_dwg_3DSOLID_material *", 0, 0, (void*)"LibreDWG::Dwg_3DSOLID_material", 0};
static swig_type_info _swigt__p__dwg_3DSOLID_silhouette = {"_p__dwg_3DSOLID_silhouette", "Dwg_3DSOLID_silhouette *|struct _dwg_3DSOLID_silhouette *|_dwg_3DSOLID_silhouette *", 0, 0, (void*)"LibreDWG::Dwg_3DSOLID_silhouette", 0};
static swig_type_info _swigt__p__dwg_3DSOLID_wire = {"_p__dwg_3DSOLID_wire", "Dwg_3DSOLID_wire *|struct _dwg_3DSOLID_wire *|_dwg_3DSOLID_wire *", 0, 0, (void*)"LibreDWG::Dwg_3DSOLID_wire", 0};
static swig_type_info _swigt__p__dwg_ACSH_HistoryNode = {"_p__dwg_ACSH_HistoryNode", "Dwg_ACSH_HistoryNode *|struct _dwg_ACSH_HistoryNode *|_dwg_ACSH_HistoryNode *", 0, 0, (void*)"LibreDWG::Dwg_ACSH_HistoryNode", 0};
static swig_type_info _swigt__p__dwg_ACSH_SubentColor = {"_p__dwg_ACSH_SubentColor", "Dwg_ACSH_SubentColor *|struct _dwg_ACSH_SubentColor *|_dwg_ACSH_SubentColor *", 0, 0, (void*)"LibreDWG::Dwg_ACSH_SubentColor", 0};
static swig_type_info _swigt__p__dwg_ACSH_SubentMaterial = {"_p__dwg_ACSH_SubentMaterial", "Dwg_ACSH_SubentMaterial *|struct _dwg_ACSH_SubentMaterial *|_dwg_ACSH_SubentMaterial *", 0, 0, (void*)"LibreDWG::Dwg_ACSH_SubentMaterial", 0};
static swig_type_info _swigt__p__dwg_ACTIONBODY = {"_p__dwg_ACTIONBODY", "Dwg_ACTIONBODY *|struct _dwg_ACTIONBODY *|_dwg_ACTIONBODY *", 0, 0, (void*)"LibreDWG::Dwg_ACTIONBODY", 0};
static swig_type_info _swigt__p__dwg_ARRAYITEMLOCATOR = {"_p__dwg_ARRAYITEMLOCATOR", "Dwg_ARRAYITEMLOCATOR *|struct _dwg_ARRAYITEMLOCATOR *|_dwg_ARRAYITEMLOCATOR *", 0, 0, (void*)"LibreDWG::Dwg_ARRAYITEMLOCATOR", 0};
static swig_type_info _swigt__p__dwg_ASSOCACTIONBODY_action = {"_p__dwg_ASSOCACTIONBODY_action", "Dwg_ASSOCACTIONBODY_action *|struct _dwg_ASSOCACTIONBODY_action *|_dwg_ASSOCACTIONBODY_action *", 0, 0, (void*)"LibreDWG::Dwg_ASSOCACTIONBODY_action", 0};
static swig_type_info _swigt__p__dwg_ASSOCACTION_Deps = {"_p__dwg_ASSOCACTION_Deps", "Dwg_ASSOCACTION_Deps *|struct _dwg_ASSOCACTION_Deps *|_dwg_ASSOCACTION_Deps *", 0, 0, (void*)"LibreDWG::Dwg_ASSOCACTION_Deps", 0};
static swig_type_info _swigt__p__dwg_ASSOCARRAYITEM = {"_p__dwg_ASSOCARRAYITEM", "Dwg_ASSOCARRAYITEM *|struct _dwg_ASSOCARRAYITEM *|_dwg_ASSOCARRAYITEM *", 0, 0, (void*)"LibreDWG::Dwg_ASSOCARRAYITEM", 0};
static swig_type_info _swigt__p__dwg_ASSOCPARAMBASEDACTIONBODY = {"_p__dwg_ASSOCPARAMBASEDACTIONBODY", "Dwg_ASSOCPARAMBASEDACTIONBODY *|struct _dwg_ASSOCPARAMBASEDACTIONBODY *|_dwg_ASSOCPARAMBASEDACTIONBODY *", 0, 0, (void*)"LibreDWG::Dwg_ASSOCPARAMBASEDACTIONBODY", 0};
static swig_type_info _swigt__p__dwg_ASSOCSURFACEACTIONBODY = {"_p__dwg_ASSOCSURFACEACTIONBODY", "Dwg_ASSOCSURFACEACTIONBODY *|struct _dwg_ASSOCSURFACEACTIONBODY *|_dwg_ASSOCSURFACEACTIONBODY *", 0, 0, (void*)"LibreDWG::Dwg_ASSOCSURFACEACTIONBODY", 0};
static swig_type_info _swigt__p__dwg_AcDbMTextObjectEmbedded = {"_p__dwg_AcDbMTextObjectEmbedded", "Dwg_AcDbMTextObjectEmbedded *|struct _dwg_AcDbMTextObjectEmbedded *|_dwg_AcDbMTextObjectEmbedded *", 0, 0, (void*)"LibreDWG::Dwg_AcDbMTextObjectEmbedded", 0};
static swig_type_info _swigt__p__dwg_AcDs = {"_p__dwg_AcDs", "Dwg_AcDs *|struct _dwg_AcDs *|_dwg_AcDs *", 0, 0, (void*)"LibreDWG::Dwg_AcDs", 0};
static swig_type_info _swigt__p__dwg_AcDs_Data = {"_p__dwg_AcDs_Data", "Dwg_AcDs_Data *|struct _dwg_AcDs_Data *|_dwg_AcDs_Data *", 0, 0, (void*)"LibreDWG::Dwg_AcDs_Data", 0};
static swig_type_info _swigt__p__dwg_AcDs_DataBlob = {"_p__dwg_AcDs_DataBlob", "Dwg_AcDs_DataBlob *|struct _dwg_AcDs_DataBlob *|_dwg_AcDs_DataBlob *", 0, 0, (void*)"LibreDWG::Dwg_AcDs_DataBlob", 0};
static swig_type_info _swigt__p__dwg_AcDs_DataBlob01 = {"_p__dwg_AcDs_DataBlob01", "Dwg_AcDs_DataBlob01 *|struct _dwg_AcDs_DataBlob01 *|_dwg_AcDs_DataBlob01 *", 0, 0, (void*)"LibreDWG::Dwg_AcDs_DataBlob01", 0};
static swig_type_info _swigt__p__dwg_AcDs_DataBlobRef = {"_p__dwg_AcDs_DataBlobRef", "Dwg_AcDs_DataBlobRef *|struct _dwg_AcDs_DataBlobRef *|_dwg_AcDs_DataBlobRef *", 0, 0, (void*)"LibreDWG::Dwg_AcDs_DataBlobRef", 0};
static swig_type_info _swigt__p__dwg_AcDs_DataBlobRef_Page = {"_p__dwg_AcDs_DataBlobRef_Page", "Dwg_AcDs_DataBlobRef_Page *|struct _dwg_AcDs_DataBlobRef_Page *|_dwg_AcDs_DataBlobRef_Page *", 0, 0, (void*)"LibreDWG::Dwg_AcDs_DataBlobRef_Page", 0};
static swig_type_info _swigt__p__dwg_AcDs_DataIndex = {"_p__dwg_AcDs_DataIndex", "Dwg_AcDs_DataIndex *|struct _dwg_AcDs_DataIndex *|_dwg_AcDs_DataIndex *", 0, 0, (void*)"LibreDWG::Dwg_AcDs_DataIndex", 0};
static swig_type_info _swigt__p__dwg_AcDs_DataIndex_Entry = {"_p__dwg_AcDs_DataIndex_Entry", "Dwg_AcDs_DataIndex_Entry *|struct _dwg_AcDs_DataIndex_Entry *|_dwg_AcDs_DataIndex_Entry *", 0, 0, (void*)"LibreDWG::Dwg_AcDs_DataIndex_Entry", 0};
static swig_type_info _swigt__p__dwg_AcDs_Data_Record = {"_p__dwg_AcDs_Data_Record", "Dwg_AcDs_Data_Record *|struct _dwg_AcDs_Data_Record *|_dwg_AcDs_Data_Record *", 0, 0, (void*)"LibreDWG::Dwg_AcDs_Data_Record", 0};
static swig_type_info _swigt__p__dwg_AcDs_Data_RecordHdr = {"_p__dwg_AcDs_Data_RecordHdr", "Dwg_AcDs_Data_RecordHdr *|struct _dwg_AcDs_Data_RecordHdr *|_dwg_AcDs_Data_RecordHdr *", 0, 0, (void*)"LibreDWG::Dwg_AcDs_Data_RecordHdr", 0};
static swig_type_info _swigt__p__dwg_AcDs_Schema = {"_p__dwg_AcDs_Schema", "Dwg_AcDs_Schema *|struct _dwg_AcDs_Schema *|_dwg_AcDs_Schema *", 0, 0, (void*)"LibreDWG::Dwg_AcDs_Schema", 0};
static swig_type_info _swigt__p__dwg_AcDs_SchemaData = {"_p__dwg_AcDs_SchemaData", "Dwg_AcDs_SchemaData *|struct _dwg_AcDs_SchemaData *|_dwg_AcDs_SchemaData *", 0, 0, (void*)"LibreDWG::Dwg_AcDs_SchemaData", 0};
static swig_type_info _swigt__p__dwg_AcDs_SchemaData_UProp = {"_p__dwg_AcDs_SchemaData_UProp", "Dwg_AcDs_SchemaData_UProp *|struct _dwg_AcDs_SchemaData_UProp *|_dwg_AcDs_SchemaData_UProp *", 0, 0, (void*)"LibreDWG::Dwg_AcDs_SchemaData_UProp", 0};
static swig_type_info _swigt__p__dwg_AcDs_SchemaIndex = {"_p__dwg_AcDs_SchemaIndex", "Dwg_AcDs_SchemaIndex *|struct _dwg_AcDs_SchemaIndex *|_dwg_AcDs_SchemaIndex *", 0, 0, (void*)"LibreDWG::Dwg_AcDs_SchemaIndex", 0};
static swig_type_info _swigt__p__dwg_AcDs_SchemaIndex_Prop = {"_p__dwg_AcDs_SchemaIndex_Prop", "Dwg_AcDs_SchemaIndex_Prop *|struct _dwg_AcDs_SchemaIndex_Prop *|_dwg_AcDs_SchemaIndex_Prop *", 0, 0, (void*)"LibreDWG::Dwg_AcDs_SchemaIndex_Prop", 0};
static swig_type_info _swigt__p__dwg_AcDs_Schema_Prop = {"_p__dwg_AcDs_Schema_Prop", "Dwg_AcDs_Schema_Prop *|struct _dwg_AcDs_Schema_Prop *|_dwg_AcDs_Schema_Prop *", 0, 0, (void*)"LibreDWG::Dwg_AcDs_Schema_Prop", 0};
static swig_type_info _swigt__p__dwg_AcDs_Search = {"_p__dwg_AcDs_Search", "Dwg_AcDs_Search *|struct _dwg_AcDs_Search *|_dwg_AcDs_Search *", 0, 0, (void*)"LibreDWG::Dwg_AcDs_Search", 0};
static swig_type_info _swigt__p__dwg_AcDs_Search_Data = {"_p__dwg_AcDs_Search_Data", "Dwg_AcDs_Search_Data *|struct _dwg_AcDs_Search_Data *|_dwg_AcDs_Search_Data *", 0, 0, (void*)"LibreDWG::Dwg_AcDs_Search_Data", 0};
static swig_type_info _swigt__p__dwg_AcDs_Search_IdIdx = {"_p__dwg_AcDs_Search_IdIdx", "Dwg_AcDs_Search_IdIdx *|struct _dwg_AcDs_Search_IdIdx *|_dwg_AcDs_Search_IdIdx *", 0, 0, (void*)"LibreDWG::Dwg_AcDs_Search_IdIdx", 0};
static swig_type_info _swigt__p__dwg_AcDs_Search_IdIdxs = {"_p__dwg_AcDs_Search_IdIdxs", "Dwg_AcDs_Search_IdIdxs *|struct _dwg_AcDs_Search_IdIdxs *|_dwg_AcDs_Search_IdIdxs *", 0, 0, (void*)"LibreDWG::Dwg_AcDs_Search_IdIdxs", 0};
static swig_type_info _swigt__p__dwg_AcDs_Segment = {"_p__dwg_AcDs_Segment", "Dwg_AcDs_Segment *|struct _dwg_AcDs_Segment *|_dwg_AcDs_Segment *", 0, 0, (void*)"LibreDWG::Dwg_AcDs_Segment", 0};
static swig_type_info _swigt__p__dwg_AcDs_SegmentIndex = {"_p__dwg_AcDs_SegmentIndex", "Dwg_AcDs_SegmentIndex *|struct _dwg_AcDs_SegmentIndex *|_dwg_AcDs_SegmentIndex *", 0, 0, (void*)"LibreDWG::Dwg_AcDs_SegmentIndex", 0};
static swig_type_info _swigt__p__dwg_BLOCKACTION_connectionpts = {"_p__dwg_BLOCKACTION_connectionpts", "Dwg_BLOCKACTION_connectionpts *|struct _dwg_BLOCKACTION_connectionpts *|_dwg_BLOCKACTION_connectionpts *", 0, 0, (void*)"LibreDWG::Dwg_BLOCKACTION_connectionpts", 0};
static swig_type_info _swigt__p__dwg_BLOCKLOOKUPACTION_lut = {"_p__dwg_BLOCKLOOKUPACTION_lut", "Dwg_BLOCKLOOKUPACTION_lut *|struct _dwg_BLOCKLOOKUPACTION_lut *|_dwg_BLOCKLOOKUPACTION_lut *", 0, 0, (void*)"LibreDWG::Dwg_BLOCKLOOKUPACTION_lut", 0};
static swig_type_info _swigt__p__dwg_BLOCKPARAMETER_PropInfo = {"_p__dwg_BLOCKPARAMETER_PropInfo", "Dwg_BLOCKPARAMETER_PropInfo *|struct _dwg_BLOCKPARAMETER_PropInfo *|_dwg_BLOCKPARAMETER_PropInfo *", 0, 0, (void*)"LibreDWG::Dwg_BLOCKPARAMETER_PropInfo", 0};
static swig_type_info _swigt__p__dwg_BLOCKPARAMETER_connection = {"_p__dwg_BLOCKPARAMETER_connection", "Dwg_BLOCKPARAMETER_connection *|struct _dwg_BLOCKPARAMETER_connection *|_dwg_BLOCKPARAMETER_connection *", 0, 0, (void*)"LibreDWG::Dwg_BLOCKPARAMETER_connection", 0};
static swig_type_info _swigt__p__dwg_BLOCKPARAMVALUESET = {"_p__dwg_BLOCKPARAMVALUESET", "Dwg_BLOCKPARAMVALUESET *|struct _dwg_BLOCKPARAMVALUESET *|_dwg_BLOCKPARAMVALUESET *", 0, 0, (void*)"LibreDWG::Dwg_BLOCKPARAMVALUESET", 0};
static swig_type_info _swigt__p__dwg_BLOCKSTRETCHACTION_codes = {"_p__dwg_BLOCKSTRETCHACTION_codes", "Dwg_BLOCKSTRETCHACTION_codes *|struct _dwg_BLOCKSTRETCHACTION_codes *|_dwg_BLOCKSTRETCHACTION_codes *", 0, 0, (void*)"LibreDWG::Dwg_BLOCKSTRETCHACTION_codes", 0};
static swig_type_info _swigt__p__dwg_BLOCKSTRETCHACTION_handles = {"_p__dwg_BLOCKSTRETCHACTION_handles", "Dwg_BLOCKSTRETCHACTION_handles *|struct _dwg_BLOCKSTRETCHACTION_handles *|_dwg_BLOCKSTRETCHACTION_handles *", 0, 0, (void*)"LibreDWG::Dwg_BLOCKSTRETCHACTION_handles", 0};
static swig_type_info _swigt__p__dwg_BLOCKVISIBILITYPARAMETER_state = {"_p__dwg_BLOCKVISIBILITYPARAMETER_state", "Dwg_BLOCKVISIBILITYPARAMETER_state *|struct _dwg_BLOCKVISIBILITYPARAMETER_state *|_dwg_BLOCKVISIBILITYPARAMETER_state *", 0, 0, (void*)"LibreDWG::Dwg_BLOCKVISIBILITYPARAMETER_state", 0};
static swig_type_info _swigt__p__dwg_COMPOUNDOBJECTID = {"_p__dwg_COMPOUNDOBJECTID", "Dwg_COMPOUNDOBJECTID *|struct _dwg_COMPOUNDOBJECTID *|_dwg_COMPOUNDOBJECTID *", 0, 0, (void*)"LibreDWG::Dwg_COMPOUNDOBJECTID", 0};
static swig_type_info _swigt__p__dwg_CONSTRAINTGROUPNODE = {"_p__dwg_CONSTRAINTGROUPNODE", "Dwg_CONSTRAINTGROUPNODE *|struct _dwg_CONSTRAINTGROUPNODE *|_dwg_CONSTRAINTGROUPNODE *", 0, 0, (void*)"LibreDWG::Dwg_CONSTRAINTGROUPNODE", 0};
static swig_type_info _swigt__p__dwg_CONTEXTDATA_dict = {"_p__dwg_CONTEXTDATA_dict", "Dwg_CONTEXTDATA_dict *|struct _dwg_CONTEXTDATA_dict *|_dwg_CONTEXTDATA_dict *", 0, 0, (void*)"LibreDWG::Dwg_CONTEXTDATA_dict", 0};
static swig_type_info _swigt__p__dwg_CONTEXTDATA_submgr = {"_p__dwg_CONTEXTDATA_submgr", "Dwg_CONTEXTDATA_submgr *|struct _dwg_CONTEXTDATA_submgr *|_dwg_CONTEXTDATA_submgr *", 0, 0, (void*)"LibreDWG::Dwg_CONTEXTDATA_submgr", 0};
static swig_type_info _swigt__p__dwg_CellContentGeometry = {"_p__dwg_CellContentGeometry", "Dwg_CellContentGeometry *|struct _dwg_CellContentGeometry *|_dwg_CellContentGeometry *", 0, 0, (void*)"LibreDWG::Dwg_CellContentGeometry", 0};
static swig_type_info _swigt__p__dwg_CellStyle = {"_p__dwg_CellStyle", "Dwg_CellStyle *|struct _dwg_CellStyle *|_dwg_CellStyle *", 0, 0, (void*)"LibreDWG::Dwg_CellStyle", 0};
static swig_type_info _swigt__p__dwg_ColorRamp = {"_p__dwg_ColorRamp", "Dwg_ColorRamp *|struct _dwg_ColorRamp *|_dwg_ColorRamp *", 0, 0, (void*)"LibreDWG::Dwg_ColorRamp", 0};
static swig_type_info _swigt__p__dwg_ContentFormat = {"_p__dwg_ContentFormat", "Dwg_ContentFormat *|struct _dwg_ContentFormat *|_dwg_ContentFormat *", 0, 0, (void*)"LibreDWG::Dwg_ContentFormat", 0};
static swig_type_info _swigt__p__dwg_DATALINK_customdata = {"_p__dwg_DATALINK_customdata", "Dwg_DATALINK_customdata *|struct _dwg_DATALINK_customdata *|_dwg_DATALINK_customdata *", 0, 0, (void*)"LibreDWG::Dwg_DATALINK_customdata", 0};
static swig_type_info _swigt__p__dwg_DATATABLE_column = {"_p__dwg_DATATABLE_column", "Dwg_DATATABLE_column *|struct _dwg_DATATABLE_column *|_dwg_DATATABLE_column *", 0, 0, (void*)"LibreDWG::Dwg_DATATABLE_column", 0};
static swig_type_info _swigt__p__dwg_DATATABLE_row = {"_p__dwg_DATATABLE_row", "Dwg_DATATABLE_row *|struct _dwg_DATATABLE_row *|_dwg_DATATABLE_row *", 0, 0, (void*)"LibreDWG::Dwg_DATATABLE_row", 0};
static swig_type_info _swigt__p__dwg_DIMASSOC_Ref = {"_p__dwg_DIMASSOC_Ref", "Dwg_DIMASSOC_Ref *|struct _dwg_DIMASSOC_Ref *|_dwg_DIMASSOC_Ref *", 0, 0, (void*)"LibreDWG::Dwg_DIMASSOC_Ref", 0};
static swig_type_info _swigt__p__dwg_DIMENSION_common = {"_p__dwg_DIMENSION_common", "Dwg_DIMENSION_common *|struct _dwg_DIMENSION_common *|_dwg_DIMENSION_common *", 0, 0, (void*)"LibreDWG::Dwg_DIMENSION_common", 0};
static swig_type_info _swigt__p__dwg_EVAL_Edge = {"_p__dwg_EVAL_Edge", "Dwg_EVAL_Edge *|struct _dwg_EVAL_Edge *|_dwg_EVAL_Edge *", 0, 0, (void*)"LibreDWG::Dwg_EVAL_Edge", 0};
static swig_type_info _swigt__p__dwg_EVAL_Node = {"_p__dwg_EVAL_Node", "Dwg_EVAL_Node *|struct _dwg_EVAL_Node *|_dwg_EVAL_Node *", 0, 0, (void*)"LibreDWG::Dwg_EVAL_Node", 0};
static swig_type_info _swigt__p__dwg_EvalExpr = {"_p__dwg_EvalExpr", "Dwg_EvalExpr *|struct _dwg_EvalExpr *|_dwg_EvalExpr *", 0, 0, (void*)"LibreDWG::Dwg_EvalExpr", 0};
static swig_type_info _swigt__p__dwg_EvalExpr_value = {"_p__dwg_EvalExpr_value", "_dwg_EvalExpr_value *", 0, 0, (void*)"LibreDWG::_dwg_EvalExpr_value", 0};
static swig_type_info _swigt__p__dwg_EvalVariant = {"_p__dwg_EvalVariant", "Dwg_EvalVariant *|struct _dwg_EvalVariant *|_dwg_EvalVariant *", 0, 0, (void*)"LibreDWG::Dwg_EvalVariant", 0};
static swig_type_info _swigt__p__dwg_EvalVariant_u = {"_p__dwg_EvalVariant_u", "_dwg_EvalVariant_u *", 0, 0, (void*)"LibreDWG::_dwg_EvalVariant_u", 0};
static swig_type_info _swigt__p__dwg_FIELD_ChildValue = {"_p__dwg_FIELD_ChildValue", "Dwg_FIELD_ChildValue *|struct _dwg_FIELD_ChildValue *|_dwg_FIELD_ChildValue *", 0, 0, (void*)"LibreDWG::Dwg_FIELD_ChildValue", 0};
static swig_type_info _swigt__p__dwg_FileDepList_Files = {"_p__dwg_FileDepList_Files", "Dwg_FileDepList_Files *|struct _dwg_FileDepList_Files *|_dwg_FileDepList_Files *", 0, 0, (void*)"LibreDWG::Dwg_FileDepList_Files", 0};
static swig_type_info _swigt__p__dwg_FormattedTableData = {"_p__dwg_FormattedTableData", "Dwg_FormattedTableData *|struct _dwg_FormattedTableData *|_dwg_FormattedTableData *", 0, 0, (void*)"LibreDWG::Dwg_FormattedTableData", 0};
static swig_type_info _swigt__p__dwg_FormattedTableMerged = {"_p__dwg_FormattedTableMerged", "Dwg_FormattedTableMerged *|struct _dwg_FormattedTableMerged *|_dwg_FormattedTableMerged *", 0, 0, (void*)"LibreDWG::Dwg_FormattedTableMerged", 0};
static swig_type_info _swigt__p__dwg_GEODATA_meshface = {"_p__dwg_GEODATA_meshface", "Dwg_GEODATA_meshface *|struct _dwg_GEODATA_meshface *|_dwg_GEODATA_meshface *", 0, 0, (void*)"LibreDWG::Dwg_GEODATA_meshface", 0};
static swig_type_info _swigt__p__dwg_GEODATA_meshpt = {"_p__dwg_GEODATA_meshpt", "Dwg_GEODATA_meshpt *|struct _dwg_GEODATA_meshpt *|_dwg_GEODATA_meshpt *", 0, 0, (void*)"LibreDWG::Dwg_GEODATA_meshpt", 0};
static swig_type_info _swigt__p__dwg_GridFormat = {"_p__dwg_GridFormat", "Dwg_GridFormat *|struct _dwg_GridFormat *|_dwg_GridFormat *", 0, 0, (void*)"LibreDWG::Dwg_GridFormat", 0};
static swig_type_info _swigt__p__dwg_HATCH_Color = {"_p__dwg_HATCH_Color", "Dwg_HATCH_Color *|struct _dwg_HATCH_Color *|_dwg_HATCH_Color *", 0, 0, (void*)"LibreDWG::Dwg_HATCH_Color", 0};
static swig_type_info _swigt__p__dwg_HATCH_ControlPoint = {"_p__dwg_HATCH_ControlPoint", "Dwg_HATCH_ControlPoint *|struct _dwg_HATCH_ControlPoint *|_dwg_HATCH_ControlPoint *", 0, 0, (void*)"LibreDWG::Dwg_HATCH_ControlPoint", 0};
static swig_type_info _swigt__p__dwg_HATCH_DefLine = {"_p__dwg_HATCH_DefLine", "Dwg_HATCH_DefLine *|struct _dwg_HATCH_DefLine *|_dwg_HATCH_DefLine *", 0, 0, (void*)"LibreDWG::Dwg_HATCH_DefLine", 0};
static swig_type_info _swigt__p__dwg_HATCH_Path = {"_p__dwg_HATCH_Path", "Dwg_HATCH_Path *|struct _dwg_HATCH_Path *|_dwg_HATCH_Path *", 0, 0, (void*)"LibreDWG::Dwg_HATCH_Path", 0};
static swig_type_info _swigt__p__dwg_HATCH_PathSeg = {"_p__dwg_HATCH_PathSeg", "Dwg_HATCH_PathSeg *|struct _dwg_HATCH_PathSeg *|_dwg_HATCH_PathSeg *", 0, 0, (void*)"LibreDWG::Dwg_HATCH_PathSeg", 0};
static swig_type_info _swigt__p__dwg_HATCH_PolylinePath = {"_p__dwg_HATCH_PolylinePath", "Dwg_HATCH_PolylinePath *|struct _dwg_HATCH_PolylinePath *|_dwg_HATCH_PolylinePath *", 0, 0, (void*)"LibreDWG::Dwg_HATCH_PolylinePath", 0};
static swig_type_info _swigt__p__dwg_LAYER_entry = {"_p__dwg_LAYER_entry", "Dwg_LAYER_entry *|struct _dwg_LAYER_entry *|_dwg_LAYER_entry *", 0, 0, (void*)"LibreDWG::Dwg_LAYER_entry", 0};
static swig_type_info _swigt__p__dwg_LEADER_ArrowHead = {"_p__dwg_LEADER_ArrowHead", "Dwg_LEADER_ArrowHead *|struct _dwg_LEADER_ArrowHead *|_dwg_LEADER_ArrowHead *", 0, 0, (void*)"LibreDWG::Dwg_LEADER_ArrowHead", 0};
static swig_type_info _swigt__p__dwg_LEADER_BlockLabel = {"_p__dwg_LEADER_BlockLabel", "Dwg_LEADER_BlockLabel *|struct _dwg_LEADER_BlockLabel *|_dwg_LEADER_BlockLabel *", 0, 0, (void*)"LibreDWG::Dwg_LEADER_BlockLabel", 0};
static swig_type_info _swigt__p__dwg_LEADER_Break = {"_p__dwg_LEADER_Break", "Dwg_LEADER_Break *|struct _dwg_LEADER_Break *|_dwg_LEADER_Break *", 0, 0, (void*)"LibreDWG::Dwg_LEADER_Break", 0};
static swig_type_info _swigt__p__dwg_LEADER_Line = {"_p__dwg_LEADER_Line", "Dwg_LEADER_Line *|struct _dwg_LEADER_Line *|_dwg_LEADER_Line *", 0, 0, (void*)"LibreDWG::Dwg_LEADER_Line", 0};
static swig_type_info _swigt__p__dwg_LEADER_Node = {"_p__dwg_LEADER_Node", "Dwg_LEADER_Node *|struct _dwg_LEADER_Node *|_dwg_LEADER_Node *", 0, 0, (void*)"LibreDWG::Dwg_LEADER_Node", 0};
static swig_type_info _swigt__p__dwg_LIGHTLIST_light = {"_p__dwg_LIGHTLIST_light", "Dwg_LIGHTLIST_light *|struct _dwg_LIGHTLIST_light *|_dwg_LIGHTLIST_light *", 0, 0, (void*)"LibreDWG::Dwg_LIGHTLIST_light", 0};
static swig_type_info _swigt__p__dwg_LTYPE_dash = {"_p__dwg_LTYPE_dash", "Dwg_LTYPE_dash *|struct _dwg_LTYPE_dash *|_dwg_LTYPE_dash *", 0, 0, (void*)"LibreDWG::Dwg_LTYPE_dash", 0};
static swig_type_info _swigt__p__dwg_LWPOLYLINE_width = {"_p__dwg_LWPOLYLINE_width", "Dwg_LWPOLYLINE_width *|struct _dwg_LWPOLYLINE_width *|_dwg_LWPOLYLINE_width *", 0, 0, (void*)"LibreDWG::Dwg_LWPOLYLINE_width", 0};
static swig_type_info _swigt__p__dwg_LinkedData = {"_p__dwg_LinkedData", "Dwg_LinkedData *|struct _dwg_LinkedData *|_dwg_LinkedData *", 0, 0, (void*)"LibreDWG::Dwg_LinkedData", 0};
static swig_type_info _swigt__p__dwg_LinkedTableData = {"_p__dwg_LinkedTableData", "Dwg_LinkedTableData *|struct _dwg_LinkedTableData *|_dwg_LinkedTableData *", 0, 0, (void*)"LibreDWG::Dwg_LinkedTableData", 0};
static swig_type_info _swigt__p__dwg_MATERIAL_color = {"_p__dwg_MATERIAL_color", "Dwg_MATERIAL_color *|struct _dwg_MATERIAL_color *|_dwg_MATERIAL_color *", 0, 0, (void*)"LibreDWG::Dwg_MATERIAL_color", 0};
static swig_type_info _swigt__p__dwg_MATERIAL_gentexture = {"_p__dwg_MATERIAL_gentexture", "Dwg_MATERIAL_gentexture *|struct _dwg_MATERIAL_gentexture *|_dwg_MATERIAL_gentexture *", 0, 0, (void*)"LibreDWG::Dwg_MATERIAL_gentexture", 0};
static swig_type_info _swigt__p__dwg_MATERIAL_mapper = {"_p__dwg_MATERIAL_mapper", "Dwg_MATERIAL_mapper *|struct _dwg_MATERIAL_mapper *|_dwg_MATERIAL_mapper *", 0, 0, (void*)"LibreDWG::Dwg_MATERIAL_mapper", 0};
static swig_type_info _swigt__p__dwg_MESH_edge = {"_p__dwg_MESH_edge", "Dwg_MESH_edge *|struct _dwg_MESH_edge *|_dwg_MESH_edge *", 0, 0, (void*)"LibreDWG::Dwg_MESH_edge", 0};
static swig_type_info _swigt__p__dwg_MLEADER_AnnotContext = {"_p__dwg_MLEADER_AnnotContext", "Dwg_MLEADER_AnnotContext *|struct _dwg_MLEADER_AnnotContext *|_dwg_MLEADER_AnnotContext *", 0, 0, (void*)"LibreDWG::Dwg_MLEADER_AnnotContext", 0};
static swig_type_info _swigt__p__dwg_MLEADER_Content = {"_p__dwg_MLEADER_Content", "Dwg_MLEADER_Content *|union _dwg_MLEADER_Content *|_dwg_MLEADER_Content *", 0, 0, (void*)"LibreDWG::Dwg_MLEADER_Content", 0};
static swig_type_info _swigt__p__dwg_MLEADER_Content_Block = {"_p__dwg_MLEADER_Content_Block", "Dwg_MLEADER_Content_Block *|struct _dwg_MLEADER_Content_Block *|_dwg_MLEADER_Content_Block *", 0, 0, (void*)"LibreDWG::Dwg_MLEADER_Content_Block", 0};
static swig_type_info _swigt__p__dwg_MLEADER_Content_MText = {"_p__dwg_MLEADER_Content_MText", "Dwg_MLEADER_Content_MText *|struct _dwg_MLEADER_Content_MText *|_dwg_MLEADER_Content_MText *", 0, 0, (void*)"LibreDWG::Dwg_MLEADER_Content_MText", 0};
static swig_type_info _swigt__p__dwg_MLINESTYLE_line = {"_p__dwg_MLINESTYLE_line", "Dwg_MLINESTYLE_line *|struct _dwg_MLINESTYLE_line *|_dwg_MLINESTYLE_line *", 0, 0, (void*)"LibreDWG::Dwg_MLINESTYLE_line", 0};
static swig_type_info _swigt__p__dwg_MLINESTYLE_line_lt = {"_p__dwg_MLINESTYLE_line_lt", "_dwg_MLINESTYLE_line_lt *", 0, 0, (void*)"LibreDWG::_dwg_MLINESTYLE_line_lt", 0};
static swig_type_info _swigt__p__dwg_MLINE_line = {"_p__dwg_MLINE_line", "Dwg_MLINE_line *|struct _dwg_MLINE_line *|_dwg_MLINE_line *", 0, 0, (void*)"LibreDWG::Dwg_MLINE_line", 0};
static swig_type_info _swigt__p__dwg_MLINE_vertex = {"_p__dwg_MLINE_vertex", "Dwg_MLINE_vertex *|struct _dwg_MLINE_vertex *|_dwg_MLINE_vertex *", 0, 0, (void*)"LibreDWG::Dwg_MLINE_vertex", 0};
static swig_type_info _swigt__p__dwg_OCD_Dimension = {"_p__dwg_OCD_Dimension", "Dwg_OCD_Dimension *|struct _dwg_OCD_Dimension *|_dwg_OCD_Dimension *", 0, 0, (void*)"LibreDWG::Dwg_OCD_Dimension", 0};
static swig_type_info _swigt__p__dwg_PARTIAL_VIEWING_INDEX_Entry = {"_p__dwg_PARTIAL_VIEWING_INDEX_Entry", "Dwg_PARTIAL_VIEWING_INDEX_Entry *|struct _dwg_PARTIAL_VIEWING_INDEX_Entry *|_dwg_PARTIAL_VIEWING_INDEX_Entry *", 0, 0, (void*)"LibreDWG::Dwg_PARTIAL_VIEWING_INDEX_Entry", 0};
static swig_type_info _swigt__p__dwg_POINTCLOUDCOLORMAP_Ramp = {"_p__dwg_POINTCLOUDCOLORMAP_Ramp", "Dwg_POINTCLOUDCOLORMAP_Ramp *|struct _dwg_POINTCLOUDCOLORMAP_Ramp *|_dwg_POINTCLOUDCOLORMAP_Ramp *", 0, 0, (void*)"LibreDWG::Dwg_POINTCLOUDCOLORMAP_Ramp", 0};
static swig_type_info _swigt__p__dwg_POINTCLOUDEX_Croppings = {"_p__dwg_POINTCLOUDEX_Croppings", "Dwg_POINTCLOUDEX_Croppings *|struct _dwg_POINTCLOUDEX_Croppings *|_dwg_POINTCLOUDEX_Croppings *", 0, 0, (void*)"LibreDWG::Dwg_POINTCLOUDEX_Croppings", 0};
static swig_type_info _swigt__p__dwg_POINTCLOUD_Clippings = {"_p__dwg_POINTCLOUD_Clippings", "Dwg_POINTCLOUD_Clippings *|struct _dwg_POINTCLOUD_Clippings *|_dwg_POINTCLOUD_Clippings *", 0, 0, (void*)"LibreDWG::Dwg_POINTCLOUD_Clippings", 0};
static swig_type_info _swigt__p__dwg_POINTCLOUD_IntensityStyle = {"_p__dwg_POINTCLOUD_IntensityStyle", "Dwg_POINTCLOUD_IntensityStyle *|struct _dwg_POINTCLOUD_IntensityStyle *|_dwg_POINTCLOUD_IntensityStyle *", 0, 0, (void*)"LibreDWG::Dwg_POINTCLOUD_IntensityStyle", 0};
static swig_type_info _swigt__p__dwg_PROXY_LWPOLYLINE = {"_p__dwg_PROXY_LWPOLYLINE", "Dwg_PROXY_LWPOLYLINE *|struct _dwg_PROXY_LWPOLYLINE *|_dwg_PROXY_LWPOLYLINE *", 0, 0, (void*)"LibreDWG::Dwg_PROXY_LWPOLYLINE", 0};
static swig_type_info _swigt__p__dwg_R2004_Header = {"_p__dwg_R2004_Header", "Dwg_R2004_Header *|struct _dwg_R2004_Header *|_dwg_R2004_Header *", 0, 0, (void*)"LibreDWG::Dwg_R2004_Header", 0};
static swig_type_info _swigt__p__dwg_SECTION_geometrysettings = {"_p__dwg_SECTION_geometrysettings", "Dwg_SECTION_geometrysettings *|struct _dwg_SECTION_geometrysettings *|_dwg_SECTION_geometrysettings *", 0, 0, (void*)"LibreDWG::Dwg_SECTION_geometrysettings", 0};
static swig_type_info _swigt__p__dwg_SECTION_typesettings = {"_p__dwg_SECTION_typesettings", "Dwg_SECTION_typesettings *|struct _dwg_SECTION_typesettings *|_dwg_SECTION_typesettings *", 0, 0, (void*)"LibreDWG::Dwg_SECTION_typesettings", 0};
static swig_type_info _swigt__p__dwg_SPLINE_control_point = {"_p__dwg_SPLINE_control_point", "Dwg_SPLINE_control_point *|struct _dwg_SPLINE_control_point *|_dwg_SPLINE_control_point *", 0, 0, (void*)"LibreDWG::Dwg_SPLINE_control_point", 0};
static swig_type_info _swigt__p__dwg_SUNSTUDY_Dates = {"_p__dwg_SUNSTUDY_Dates", "Dwg_SUNSTUDY_Dates *|struct _dwg_SUNSTUDY_Dates *|_dwg_SUNSTUDY_Dates *", 0, 0, (void*)"LibreDWG::Dwg_SUNSTUDY_Dates", 0};
static swig_type_info _swigt__p__dwg_SummaryInfo_Property = {"_p__dwg_SummaryInfo_Property", "Dwg_SummaryInfo_Property *|struct _dwg_SummaryInfo_Property *|_dwg_SummaryInfo_Property *", 0, 0, (void*)"LibreDWG::Dwg_SummaryInfo_Property", 0};
static swig_type_info _swigt__p__dwg_TABLEGEOMETRY_Cell = {"_p__dwg_TABLEGEOMETRY_Cell", "Dwg_TABLEGEOMETRY_Cell *|struct _dwg_TABLEGEOMETRY_Cell *|_dwg_TABLEGEOMETRY_Cell *", 0, 0, (void*)"LibreDWG::Dwg_TABLEGEOMETRY_Cell", 0};
static swig_type_info _swigt__p__dwg_TABLESTYLE_CellStyle = {"_p__dwg_TABLESTYLE_CellStyle", "Dwg_TABLESTYLE_CellStyle *|struct _dwg_TABLESTYLE_CellStyle *|_dwg_TABLESTYLE_CellStyle *", 0, 0, (void*)"LibreDWG::Dwg_TABLESTYLE_CellStyle", 0};
static swig_type_info _swigt__p__dwg_TABLESTYLE_border = {"_p__dwg_TABLESTYLE_border", "Dwg_TABLESTYLE_border *|struct _dwg_TABLESTYLE_border *|_dwg_TABLESTYLE_border *", 0, 0, (void*)"LibreDWG::Dwg_TABLESTYLE_border", 0};
static swig_type_info _swigt__p__dwg_TABLESTYLE_rowstyles = {"_p__dwg_TABLESTYLE_rowstyles", "Dwg_TABLESTYLE_rowstyles *|struct _dwg_TABLESTYLE_rowstyles *|_dwg_TABLESTYLE_rowstyles *", 0, 0, (void*)"LibreDWG::Dwg_TABLESTYLE_rowstyles", 0};
static swig_type_info _swigt__p__dwg_TABLE_AttrDef = {"_p__dwg_TABLE_AttrDef", "Dwg_TABLE_AttrDef *|struct _dwg_TABLE_AttrDef *|_dwg_TABLE_AttrDef *", 0, 0, (void*)"LibreDWG::Dwg_TABLE_AttrDef", 0};
static swig_type_info _swigt__p__dwg_TABLE_BreakHeight = {"_p__dwg_TABLE_BreakHeight", "Dwg_TABLE_BreakHeight *|struct _dwg_TABLE_BreakHeight *|_dwg_TABLE_BreakHeight *", 0, 0, (void*)"LibreDWG::Dwg_TABLE_BreakHeight", 0};
static swig_type_info _swigt__p__dwg_TABLE_BreakRow = {"_p__dwg_TABLE_BreakRow", "Dwg_TABLE_BreakRow *|struct _dwg_TABLE_BreakRow *|_dwg_TABLE_BreakRow *", 0, 0, (void*)"LibreDWG::Dwg_TABLE_BreakRow", 0};
static swig_type_info _swigt__p__dwg_TABLE_Cell = {"_p__dwg_TABLE_Cell", "Dwg_TABLE_Cell *|struct _dwg_TABLE_Cell *|_dwg_TABLE_Cell *", 0, 0, (void*)"LibreDWG::Dwg_TABLE_Cell", 0};
static swig_type_info _swigt__p__dwg_TABLE_CustomDataItem = {"_p__dwg_TABLE_CustomDataItem", "Dwg_TABLE_CustomDataItem *|struct _dwg_TABLE_CustomDataItem *|_dwg_TABLE_CustomDataItem *", 0, 0, (void*)"LibreDWG::Dwg_TABLE_CustomDataItem", 0};
static swig_type_info _swigt__p__dwg_TABLE_value = {"_p__dwg_TABLE_value", "Dwg_TABLE_value *|struct _dwg_TABLE_value *|_dwg_TABLE_value *", 0, 0, (void*)"LibreDWG::Dwg_TABLE_value", 0};
static swig_type_info _swigt__p__dwg_TableCell = {"_p__dwg_TableCell", "Dwg_TableCell *|struct _dwg_TableCell *|_dwg_TableCell *", 0, 0, (void*)"LibreDWG::Dwg_TableCell", 0};
static swig_type_info _swigt__p__dwg_TableCellContent = {"_p__dwg_TableCellContent", "Dwg_TableCellContent *|struct _dwg_TableCellContent *|_dwg_TableCellContent *", 0, 0, (void*)"LibreDWG::Dwg_TableCellContent", 0};
static swig_type_info _swigt__p__dwg_TableCellContent_Attr = {"_p__dwg_TableCellContent_Attr", "Dwg_TableCellContent_Attr *|struct _dwg_TableCellContent_Attr *|_dwg_TableCellContent_Attr *", 0, 0, (void*)"LibreDWG::Dwg_TableCellContent_Attr", 0};
static swig_type_info _swigt__p__dwg_TableDataColumn = {"_p__dwg_TableDataColumn", "Dwg_TableDataColumn *|struct _dwg_TableDataColumn *|_dwg_TableDataColumn *", 0, 0, (void*)"LibreDWG::Dwg_TableDataColumn", 0};
static swig_type_info _swigt__p__dwg_TableRow = {"_p__dwg_TableRow", "Dwg_TableRow *|struct _dwg_TableRow *|_dwg_TableRow *", 0, 0, (void*)"LibreDWG::Dwg_TableRow", 0};
static swig_type_info _swigt__p__dwg_UCS_orthopts = {"_p__dwg_UCS_orthopts", "Dwg_UCS_orthopts *|struct _dwg_UCS_orthopts *|_dwg_UCS_orthopts *", 0, 0, (void*)"LibreDWG::Dwg_UCS_orthopts", 0};
static swig_type_info _swigt__p__dwg_VALUEPARAM = {"_p__dwg_VALUEPARAM", "Dwg_VALUEPARAM *|struct _dwg_VALUEPARAM *|_dwg_VALUEPARAM *", 0, 0, (void*)"LibreDWG::Dwg_VALUEPARAM", 0};
static swig_type_info _swigt__p__dwg_VALUEPARAM_vars = {"_p__dwg_VALUEPARAM_vars", "Dwg_VALUEPARAM_vars *|struct _dwg_VALUEPARAM_vars *|_dwg_VALUEPARAM_vars *", 0, 0, (void*)"LibreDWG::Dwg_VALUEPARAM_vars", 0};
static swig_type_info _swigt__p__dwg_abstractentity_UNDERLAY = {"_p__dwg_abstractentity_UNDERLAY", "Dwg_Entity_DGNUNDERLAY *|Dwg_Entity_DWFUNDERLAY *|Dwg_Entity_PDFUNDERLAY *|Dwg_Entity_UNDERLAY *|struct _dwg_abstractentity_UNDERLAY *|_dwg_abstractentity_UNDERLAY *", 0, 0, (void*)"LibreDWG::Dwg_Entity_UNDERLAY", 0};
static swig_type_info _swigt__p__dwg_abstractobject_ASSOCARRAYPARAMETERS = {"_p__dwg_abstractobject_ASSOCARRAYPARAMETERS", "Dwg_Object_ASSOCARRAYMODIFYPARAMETERS *|Dwg_Object_ASSOCARRAYPARAMETERS *|Dwg_Object_ASSOCARRAYPATHPARAMETERS *|Dwg_Object_ASSOCARRAYPOLARPARAMETERS *|Dwg_Object_ASSOCARRAYRECTANGULARPARAMETERS *|struct _dwg_abstractobject_ASSOCARRAYPARAMETERS *|_dwg_abstractobject_ASSOCARRAYPARAMETERS *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCARRAYPARAMETERS", 0};
static swig_type_info _swigt__p__dwg_abstractobject_UNDERLAYDEFINITION = {"_p__dwg_abstractobject_UNDERLAYDEFINITION", "Dwg_Object_DGNDEFINITION *|Dwg_Object_DWFDEFINITION *|Dwg_Object_PDFDEFINITION *|Dwg_Object_UNDERLAYDEFINITION *|struct _dwg_abstractobject_UNDERLAYDEFINITION *|_dwg_abstractobject_UNDERLAYDEFINITION *", 0, 0, (void*)"LibreDWG::Dwg_Object_UNDERLAYDEFINITION", 0};
static swig_type_info _swigt__p__dwg_appinfo = {"_p__dwg_appinfo", "Dwg_AppInfo *|struct _dwg_appinfo *|_dwg_appinfo *", 0, 0, (void*)"LibreDWG::Dwg_AppInfo", 0};
static swig_type_info _swigt__p__dwg_appinfohistory = {"_p__dwg_appinfohistory", "Dwg_AppInfoHistory *|struct _dwg_appinfohistory *|_dwg_appinfohistory *", 0, 0, (void*)"LibreDWG::Dwg_AppInfoHistory", 0};
static swig_type_info _swigt__p__dwg_auxheader = {"_p__dwg_auxheader", "Dwg_AuxHeader *|struct _dwg_auxheader *|_dwg_auxheader *", 0, 0, (void*)"LibreDWG::Dwg_AuxHeader", 0};
static swig_type_info _swigt__p__dwg_binary_chunk = {"_p__dwg_binary_chunk", "struct _dwg_binary_chunk *|_dwg_binary_chunk *", 0, 0, (void*)"LibreDWG::_dwg_binary_chunk", 0};
static swig_type_info _swigt__p__dwg_binary_chunk_u = {"_p__dwg_binary_chunk_u", "_dwg_binary_chunk_u *", 0, 0, (void*)"LibreDWG::_dwg_binary_chunk_u", 0};
static swig_type_info _swigt__p__dwg_bitcode_2bd = {"_p__dwg_bitcode_2bd", "BITCODE_2BD *|BITCODE_2BD_1 *|BITCODE_2DPOINT *|Dwg_Bitcode_2BD *|struct _dwg_bitcode_2bd *|_dwg_bitcode_2bd *", 0, 0, (void*)"LibreDWG::Dwg_Bitcode_2BD", 0};
static swig_type_info _swigt__p__dwg_bitcode_2rd = {"_p__dwg_bitcode_2rd", "BITCODE_2RD *|Dwg_Bitcode_2RD *|struct _dwg_bitcode_2rd *|_dwg_bitcode_2rd *", 0, 0, (void*)"LibreDWG::Dwg_Bitcode_2RD", 0};
static swig_type_info _swigt__p__dwg_bitcode_3bd = {"_p__dwg_bitcode_3bd", "BITCODE_3BD *|BITCODE_3BD_1 *|BITCODE_3DPOINT *|BITCODE_BE *|Dwg_Bitcode_3BD *|struct _dwg_bitcode_3bd *|_dwg_bitcode_3bd *", 0, 0, (void*)"LibreDWG::Dwg_Bitcode_3BD", 0};
static swig_type_info _swigt__p__dwg_bitcode_3rd = {"_p__dwg_bitcode_3rd", "BITCODE_3RD *|Dwg_Bitcode_3RD *|struct _dwg_bitcode_3rd *|_dwg_bitcode_3rd *", 0, 0, (void*)"LibreDWG::Dwg_Bitcode_3RD", 0};
static swig_type_info _swigt__p__dwg_chain = {"_p__dwg_chain", "Dwg_Chain *|struct _dwg_chain *|_dwg_chain *", 0, 0, (void*)"LibreDWG::Dwg_Chain", 0};
static swig_type_info _swigt__p__dwg_class = {"_p__dwg_class", "Dwg_Class *|struct _dwg_class *|_dwg_class *", 0, 0, (void*)"LibreDWG::Dwg_Class", 0};
static swig_type_info _swigt__p__dwg_color = {"_p__dwg_color", "BITCODE_CMC *|BITCODE_CMTC *|BITCODE_ENC *|Dwg_Color *|struct _dwg_color *|_dwg_color *", 0, 0, (void*)"LibreDWG::Dwg_Color", 0};
static swig_type_info _swigt__p__dwg_entity_3DFACE = {"_p__dwg_entity_3DFACE", "Dwg_Entity__3DFACE *|struct _dwg_entity_3DFACE *|_dwg_entity_3DFACE *", 0, 0, (void*)"LibreDWG::Dwg_Entity__3DFACE", 0};
static swig_type_info _swigt__p__dwg_entity_3DLINE = {"_p__dwg_entity_3DLINE", "Dwg_Entity__3DLINE *|struct _dwg_entity_3DLINE *|_dwg_entity_3DLINE *", 0, 0, (void*)"LibreDWG::Dwg_Entity__3DLINE", 0};
static swig_type_info _swigt__p__dwg_entity_3DSOLID = {"_p__dwg_entity_3DSOLID", "Dwg_Entity_BODY *|Dwg_Entity_REGION *|Dwg_Entity__3DSOLID *|struct _dwg_entity_3DSOLID *|_dwg_entity_3DSOLID *", 0, 0, (void*)"LibreDWG::Dwg_Entity__3DSOLID", 0};
static swig_type_info _swigt__p__dwg_entity_ALIGNMENTPARAMETERENTITY = {"_p__dwg_entity_ALIGNMENTPARAMETERENTITY", "Dwg_Entity_ALIGNMENTPARAMETERENTITY *|struct _dwg_entity_ALIGNMENTPARAMETERENTITY *|_dwg_entity_ALIGNMENTPARAMETERENTITY *", 0, 0, (void*)"LibreDWG::Dwg_Entity_ALIGNMENTPARAMETERENTITY", 0};
static swig_type_info _swigt__p__dwg_entity_ARC = {"_p__dwg_entity_ARC", "Dwg_Entity_ARC *|struct _dwg_entity_ARC *|_dwg_entity_ARC *", 0, 0, (void*)"LibreDWG::Dwg_Entity_ARC", 0};
static swig_type_info _swigt__p__dwg_entity_ARCALIGNEDTEXT = {"_p__dwg_entity_ARCALIGNEDTEXT", "Dwg_Entity_ARCALIGNEDTEXT *|struct _dwg_entity_ARCALIGNEDTEXT *|_dwg_entity_ARCALIGNEDTEXT *", 0, 0, (void*)"LibreDWG::Dwg_Entity_ARCALIGNEDTEXT", 0};
static swig_type_info _swigt__p__dwg_entity_ARC_DIMENSION = {"_p__dwg_entity_ARC_DIMENSION", "Dwg_Entity_ARC_DIMENSION *|struct _dwg_entity_ARC_DIMENSION *|_dwg_entity_ARC_DIMENSION *", 0, 0, (void*)"LibreDWG::Dwg_Entity_ARC_DIMENSION", 0};
static swig_type_info _swigt__p__dwg_entity_ATTDEF = {"_p__dwg_entity_ATTDEF", "Dwg_Entity_ATTDEF *|struct _dwg_entity_ATTDEF *|_dwg_entity_ATTDEF *", 0, 0, (void*)"LibreDWG::Dwg_Entity_ATTDEF", 0};
static swig_type_info _swigt__p__dwg_entity_ATTRIB = {"_p__dwg_entity_ATTRIB", "Dwg_Entity_ATTRIB *|struct _dwg_entity_ATTRIB *|_dwg_entity_ATTRIB *", 0, 0, (void*)"LibreDWG::Dwg_Entity_ATTRIB", 0};
static swig_type_info _swigt__p__dwg_entity_BASEPOINTPARAMETERENTITY = {"_p__dwg_entity_BASEPOINTPARAMETERENTITY", "Dwg_Entity_BASEPOINTPARAMETERENTITY *|struct _dwg_entity_BASEPOINTPARAMETERENTITY *|_dwg_entity_BASEPOINTPARAMETERENTITY *", 0, 0, (void*)"LibreDWG::Dwg_Entity_BASEPOINTPARAMETERENTITY", 0};
static swig_type_info _swigt__p__dwg_entity_BLOCK = {"_p__dwg_entity_BLOCK", "Dwg_Entity_BLOCK *|struct _dwg_entity_BLOCK *|_dwg_entity_BLOCK *", 0, 0, (void*)"LibreDWG::Dwg_Entity_BLOCK", 0};
static swig_type_info _swigt__p__dwg_entity_CAMERA = {"_p__dwg_entity_CAMERA", "Dwg_Entity_CAMERA *|struct _dwg_entity_CAMERA *|_dwg_entity_CAMERA *", 0, 0, (void*)"LibreDWG::Dwg_Entity_CAMERA", 0};
static swig_type_info _swigt__p__dwg_entity_CIRCLE = {"_p__dwg_entity_CIRCLE", "Dwg_Entity_CIRCLE *|struct _dwg_entity_CIRCLE *|_dwg_entity_CIRCLE *", 0, 0, (void*)"LibreDWG::Dwg_Entity_CIRCLE", 0};
static swig_type_info _swigt__p__dwg_entity_DIMENSION_ALIGNED = {"_p__dwg_entity_DIMENSION_ALIGNED", "Dwg_Entity_DIMENSION_ALIGNED *|struct _dwg_entity_DIMENSION_ALIGNED *|_dwg_entity_DIMENSION_ALIGNED *", 0, 0, (void*)"LibreDWG::Dwg_Entity_DIMENSION_ALIGNED", 0};
static swig_type_info _swigt__p__dwg_entity_DIMENSION_ANG2LN = {"_p__dwg_entity_DIMENSION_ANG2LN", "Dwg_Entity_DIMENSION_ANG2LN *|struct _dwg_entity_DIMENSION_ANG2LN *|_dwg_entity_DIMENSION_ANG2LN *", 0, 0, (void*)"LibreDWG::Dwg_Entity_DIMENSION_ANG2LN", 0};
static swig_type_info _swigt__p__dwg_entity_DIMENSION_ANG3PT = {"_p__dwg_entity_DIMENSION_ANG3PT", "Dwg_Entity_DIMENSION_ANG3PT *|struct _dwg_entity_DIMENSION_ANG3PT *|_dwg_entity_DIMENSION_ANG3PT *", 0, 0, (void*)"LibreDWG::Dwg_Entity_DIMENSION_ANG3PT", 0};
static swig_type_info _swigt__p__dwg_entity_DIMENSION_DIAMETER = {"_p__dwg_entity_DIMENSION_DIAMETER", "Dwg_Entity_DIMENSION_DIAMETER *|struct _dwg_entity_DIMENSION_DIAMETER *|_dwg_entity_DIMENSION_DIAMETER *", 0, 0, (void*)"LibreDWG::Dwg_Entity_DIMENSION_DIAMETER", 0};
static swig_type_info _swigt__p__dwg_entity_DIMENSION_LINEAR = {"_p__dwg_entity_DIMENSION_LINEAR", "Dwg_Entity_DIMENSION_LINEAR *|struct _dwg_entity_DIMENSION_LINEAR *|_dwg_entity_DIMENSION_LINEAR *", 0, 0, (void*)"LibreDWG::Dwg_Entity_DIMENSION_LINEAR", 0};
static swig_type_info _swigt__p__dwg_entity_DIMENSION_ORDINATE = {"_p__dwg_entity_DIMENSION_ORDINATE", "Dwg_Entity_DIMENSION_ORDINATE *|struct _dwg_entity_DIMENSION_ORDINATE *|_dwg_entity_DIMENSION_ORDINATE *", 0, 0, (void*)"LibreDWG::Dwg_Entity_DIMENSION_ORDINATE", 0};
static swig_type_info _swigt__p__dwg_entity_DIMENSION_RADIUS = {"_p__dwg_entity_DIMENSION_RADIUS", "Dwg_Entity_DIMENSION_RADIUS *|struct _dwg_entity_DIMENSION_RADIUS *|_dwg_entity_DIMENSION_RADIUS *", 0, 0, (void*)"LibreDWG::Dwg_Entity_DIMENSION_RADIUS", 0};
static swig_type_info _swigt__p__dwg_entity_ELLIPSE = {"_p__dwg_entity_ELLIPSE", "Dwg_Entity_ELLIPSE *|struct _dwg_entity_ELLIPSE *|_dwg_entity_ELLIPSE *", 0, 0, (void*)"LibreDWG::Dwg_Entity_ELLIPSE", 0};
static swig_type_info _swigt__p__dwg_entity_ENDBLK = {"_p__dwg_entity_ENDBLK", "Dwg_Entity_ENDBLK *|struct _dwg_entity_ENDBLK *|_dwg_entity_ENDBLK *", 0, 0, (void*)"LibreDWG::Dwg_Entity_ENDBLK", 0};
static swig_type_info _swigt__p__dwg_entity_ENDREP = {"_p__dwg_entity_ENDREP", "Dwg_Entity_ENDREP *|struct _dwg_entity_ENDREP *|_dwg_entity_ENDREP *", 0, 0, (void*)"LibreDWG::Dwg_Entity_ENDREP", 0};
static swig_type_info _swigt__p__dwg_entity_EXTRUDEDSURFACE = {"_p__dwg_entity_EXTRUDEDSURFACE", "Dwg_Entity_EXTRUDEDSURFACE *|struct _dwg_entity_EXTRUDEDSURFACE *|_dwg_entity_EXTRUDEDSURFACE *", 0, 0, (void*)"LibreDWG::Dwg_Entity_EXTRUDEDSURFACE", 0};
static swig_type_info _swigt__p__dwg_entity_FLIPGRIPENTITY = {"_p__dwg_entity_FLIPGRIPENTITY", "Dwg_Entity_FLIPGRIPENTITY *|struct _dwg_entity_FLIPGRIPENTITY *|_dwg_entity_FLIPGRIPENTITY *", 0, 0, (void*)"LibreDWG::Dwg_Entity_FLIPGRIPENTITY", 0};
static swig_type_info _swigt__p__dwg_entity_FLIPPARAMETERENTITY = {"_p__dwg_entity_FLIPPARAMETERENTITY", "Dwg_Entity_FLIPPARAMETERENTITY *|struct _dwg_entity_FLIPPARAMETERENTITY *|_dwg_entity_FLIPPARAMETERENTITY *", 0, 0, (void*)"LibreDWG::Dwg_Entity_FLIPPARAMETERENTITY", 0};
static swig_type_info _swigt__p__dwg_entity_GEOPOSITIONMARKER = {"_p__dwg_entity_GEOPOSITIONMARKER", "Dwg_Entity_GEOPOSITIONMARKER *|struct _dwg_entity_GEOPOSITIONMARKER *|_dwg_entity_GEOPOSITIONMARKER *", 0, 0, (void*)"LibreDWG::Dwg_Entity_GEOPOSITIONMARKER", 0};
static swig_type_info _swigt__p__dwg_entity_HATCH = {"_p__dwg_entity_HATCH", "Dwg_Entity_HATCH *|struct _dwg_entity_HATCH *|_dwg_entity_HATCH *", 0, 0, (void*)"LibreDWG::Dwg_Entity_HATCH", 0};
static swig_type_info _swigt__p__dwg_entity_HELIX = {"_p__dwg_entity_HELIX", "Dwg_Entity_HELIX *|struct _dwg_entity_HELIX *|_dwg_entity_HELIX *", 0, 0, (void*)"LibreDWG::Dwg_Entity_HELIX", 0};
static swig_type_info _swigt__p__dwg_entity_IMAGE = {"_p__dwg_entity_IMAGE", "Dwg_Entity_IMAGE *|struct _dwg_entity_IMAGE *|_dwg_entity_IMAGE *", 0, 0, (void*)"LibreDWG::Dwg_Entity_IMAGE", 0};
static swig_type_info _swigt__p__dwg_entity_INSERT = {"_p__dwg_entity_INSERT", "Dwg_Entity_INSERT *|struct _dwg_entity_INSERT *|_dwg_entity_INSERT *", 0, 0, (void*)"LibreDWG::Dwg_Entity_INSERT", 0};
static swig_type_info _swigt__p__dwg_entity_JUMP = {"_p__dwg_entity_JUMP", "Dwg_Entity_JUMP *|struct _dwg_entity_JUMP *|_dwg_entity_JUMP *", 0, 0, (void*)"LibreDWG::Dwg_Entity_JUMP", 0};
static swig_type_info _swigt__p__dwg_entity_LARGE_RADIAL_DIMENSION = {"_p__dwg_entity_LARGE_RADIAL_DIMENSION", "Dwg_Entity_LARGE_RADIAL_DIMENSION *|struct _dwg_entity_LARGE_RADIAL_DIMENSION *|_dwg_entity_LARGE_RADIAL_DIMENSION *", 0, 0, (void*)"LibreDWG::Dwg_Entity_LARGE_RADIAL_DIMENSION", 0};
static swig_type_info _swigt__p__dwg_entity_LAYOUTPRINTCONFIG = {"_p__dwg_entity_LAYOUTPRINTCONFIG", "Dwg_Entity_LAYOUTPRINTCONFIG *|struct _dwg_entity_LAYOUTPRINTCONFIG *|_dwg_entity_LAYOUTPRINTCONFIG *", 0, 0, (void*)"LibreDWG::Dwg_Entity_LAYOUTPRINTCONFIG", 0};
static swig_type_info _swigt__p__dwg_entity_LEADER = {"_p__dwg_entity_LEADER", "Dwg_Entity_LEADER *|struct _dwg_entity_LEADER *|_dwg_entity_LEADER *", 0, 0, (void*)"LibreDWG::Dwg_Entity_LEADER", 0};
static swig_type_info _swigt__p__dwg_entity_LIGHT = {"_p__dwg_entity_LIGHT", "Dwg_Entity_LIGHT *|struct _dwg_entity_LIGHT *|_dwg_entity_LIGHT *", 0, 0, (void*)"LibreDWG::Dwg_Entity_LIGHT", 0};
static swig_type_info _swigt__p__dwg_entity_LINE = {"_p__dwg_entity_LINE", "Dwg_Entity_LINE *|struct _dwg_entity_LINE *|_dwg_entity_LINE *", 0, 0, (void*)"LibreDWG::Dwg_Entity_LINE", 0};
static swig_type_info _swigt__p__dwg_entity_LINEARGRIPENTITY = {"_p__dwg_entity_LINEARGRIPENTITY", "Dwg_Entity_LINEARGRIPENTITY *|struct _dwg_entity_LINEARGRIPENTITY *|_dwg_entity_LINEARGRIPENTITY *", 0, 0, (void*)"LibreDWG::Dwg_Entity_LINEARGRIPENTITY", 0};
static swig_type_info _swigt__p__dwg_entity_LINEARPARAMETERENTITY = {"_p__dwg_entity_LINEARPARAMETERENTITY", "Dwg_Entity_LINEARPARAMETERENTITY *|struct _dwg_entity_LINEARPARAMETERENTITY *|_dwg_entity_LINEARPARAMETERENTITY *", 0, 0, (void*)"LibreDWG::Dwg_Entity_LINEARPARAMETERENTITY", 0};
static swig_type_info _swigt__p__dwg_entity_LOAD = {"_p__dwg_entity_LOAD", "Dwg_Entity_LOAD *|struct _dwg_entity_LOAD *|_dwg_entity_LOAD *", 0, 0, (void*)"LibreDWG::Dwg_Entity_LOAD", 0};
static swig_type_info _swigt__p__dwg_entity_LOFTEDSURFACE = {"_p__dwg_entity_LOFTEDSURFACE", "Dwg_Entity_LOFTEDSURFACE *|struct _dwg_entity_LOFTEDSURFACE *|_dwg_entity_LOFTEDSURFACE *", 0, 0, (void*)"LibreDWG::Dwg_Entity_LOFTEDSURFACE", 0};
static swig_type_info _swigt__p__dwg_entity_LWPOLYLINE = {"_p__dwg_entity_LWPOLYLINE", "Dwg_Entity_LWPOLYLINE *|struct _dwg_entity_LWPOLYLINE *|_dwg_entity_LWPOLYLINE *", 0, 0, (void*)"LibreDWG::Dwg_Entity_LWPOLYLINE", 0};
static swig_type_info _swigt__p__dwg_entity_MESH = {"_p__dwg_entity_MESH", "Dwg_Entity_MESH *|struct _dwg_entity_MESH *|_dwg_entity_MESH *", 0, 0, (void*)"LibreDWG::Dwg_Entity_MESH", 0};
static swig_type_info _swigt__p__dwg_entity_MINSERT = {"_p__dwg_entity_MINSERT", "Dwg_Entity_MINSERT *|struct _dwg_entity_MINSERT *|_dwg_entity_MINSERT *", 0, 0, (void*)"LibreDWG::Dwg_Entity_MINSERT", 0};
static swig_type_info _swigt__p__dwg_entity_MLINE = {"_p__dwg_entity_MLINE", "Dwg_Entity_MLINE *|struct _dwg_entity_MLINE *|_dwg_entity_MLINE *", 0, 0, (void*)"LibreDWG::Dwg_Entity_MLINE", 0};
static swig_type_info _swigt__p__dwg_entity_MPOLYGON = {"_p__dwg_entity_MPOLYGON", "Dwg_Entity_MPOLYGON *|struct _dwg_entity_MPOLYGON *|_dwg_entity_MPOLYGON *", 0, 0, (void*)"LibreDWG::Dwg_Entity_MPOLYGON", 0};
static swig_type_info _swigt__p__dwg_entity_MTEXT = {"_p__dwg_entity_MTEXT", "Dwg_Entity_MTEXT *|struct _dwg_entity_MTEXT *|_dwg_entity_MTEXT *", 0, 0, (void*)"LibreDWG::Dwg_Entity_MTEXT", 0};
static swig_type_info _swigt__p__dwg_entity_MULTILEADER = {"_p__dwg_entity_MULTILEADER", "Dwg_Entity_MULTILEADER *|struct _dwg_entity_MULTILEADER *|_dwg_entity_MULTILEADER *", 0, 0, (void*)"LibreDWG::Dwg_Entity_MULTILEADER", 0};
static swig_type_info _swigt__p__dwg_entity_NAVISWORKSMODEL = {"_p__dwg_entity_NAVISWORKSMODEL", "Dwg_Entity_NAVISWORKSMODEL *|struct _dwg_entity_NAVISWORKSMODEL *|_dwg_entity_NAVISWORKSMODEL *", 0, 0, (void*)"LibreDWG::Dwg_Entity_NAVISWORKSMODEL", 0};
static swig_type_info _swigt__p__dwg_entity_NURBSURFACE = {"_p__dwg_entity_NURBSURFACE", "Dwg_Entity_NURBSURFACE *|struct _dwg_entity_NURBSURFACE *|_dwg_entity_NURBSURFACE *", 0, 0, (void*)"LibreDWG::Dwg_Entity_NURBSURFACE", 0};
static swig_type_info _swigt__p__dwg_entity_OLE2FRAME = {"_p__dwg_entity_OLE2FRAME", "Dwg_Entity_OLE2FRAME *|struct _dwg_entity_OLE2FRAME *|_dwg_entity_OLE2FRAME *", 0, 0, (void*)"LibreDWG::Dwg_Entity_OLE2FRAME", 0};
static swig_type_info _swigt__p__dwg_entity_OLEFRAME = {"_p__dwg_entity_OLEFRAME", "Dwg_Entity_OLEFRAME *|struct _dwg_entity_OLEFRAME *|_dwg_entity_OLEFRAME *", 0, 0, (void*)"LibreDWG::Dwg_Entity_OLEFRAME", 0};
static swig_type_info _swigt__p__dwg_entity_PLANESURFACE = {"_p__dwg_entity_PLANESURFACE", "Dwg_Entity_PLANESURFACE *|struct _dwg_entity_PLANESURFACE *|_dwg_entity_PLANESURFACE *", 0, 0, (void*)"LibreDWG::Dwg_Entity_PLANESURFACE", 0};
static swig_type_info _swigt__p__dwg_entity_POINT = {"_p__dwg_entity_POINT", "Dwg_Entity_POINT *|struct _dwg_entity_POINT *|_dwg_entity_POINT *", 0, 0, (void*)"LibreDWG::Dwg_Entity_POINT", 0};
static swig_type_info _swigt__p__dwg_entity_POINTCLOUD = {"_p__dwg_entity_POINTCLOUD", "Dwg_Entity_POINTCLOUD *|struct _dwg_entity_POINTCLOUD *|_dwg_entity_POINTCLOUD *", 0, 0, (void*)"LibreDWG::Dwg_Entity_POINTCLOUD", 0};
static swig_type_info _swigt__p__dwg_entity_POINTCLOUDEX = {"_p__dwg_entity_POINTCLOUDEX", "Dwg_Entity_POINTCLOUDEX *|struct _dwg_entity_POINTCLOUDEX *|_dwg_entity_POINTCLOUDEX *", 0, 0, (void*)"LibreDWG::Dwg_Entity_POINTCLOUDEX", 0};
static swig_type_info _swigt__p__dwg_entity_POINTPARAMETERENTITY = {"_p__dwg_entity_POINTPARAMETERENTITY", "Dwg_Entity_POINTPARAMETERENTITY *|struct _dwg_entity_POINTPARAMETERENTITY *|_dwg_entity_POINTPARAMETERENTITY *", 0, 0, (void*)"LibreDWG::Dwg_Entity_POINTPARAMETERENTITY", 0};
static swig_type_info _swigt__p__dwg_entity_POLARGRIPENTITY = {"_p__dwg_entity_POLARGRIPENTITY", "Dwg_Entity_POLARGRIPENTITY *|struct _dwg_entity_POLARGRIPENTITY *|_dwg_entity_POLARGRIPENTITY *", 0, 0, (void*)"LibreDWG::Dwg_Entity_POLARGRIPENTITY", 0};
static swig_type_info _swigt__p__dwg_entity_POLYLINE_2D = {"_p__dwg_entity_POLYLINE_2D", "Dwg_Entity_POLYLINE_2D *|struct _dwg_entity_POLYLINE_2D *|_dwg_entity_POLYLINE_2D *", 0, 0, (void*)"LibreDWG::Dwg_Entity_POLYLINE_2D", 0};
static swig_type_info _swigt__p__dwg_entity_POLYLINE_3D = {"_p__dwg_entity_POLYLINE_3D", "Dwg_Entity_POLYLINE_3D *|struct _dwg_entity_POLYLINE_3D *|_dwg_entity_POLYLINE_3D *", 0, 0, (void*)"LibreDWG::Dwg_Entity_POLYLINE_3D", 0};
static swig_type_info _swigt__p__dwg_entity_POLYLINE_MESH = {"_p__dwg_entity_POLYLINE_MESH", "Dwg_Entity_POLYLINE_MESH *|struct _dwg_entity_POLYLINE_MESH *|_dwg_entity_POLYLINE_MESH *", 0, 0, (void*)"LibreDWG::Dwg_Entity_POLYLINE_MESH", 0};
static swig_type_info _swigt__p__dwg_entity_POLYLINE_PFACE = {"_p__dwg_entity_POLYLINE_PFACE", "Dwg_Entity_POLYLINE_PFACE *|struct _dwg_entity_POLYLINE_PFACE *|_dwg_entity_POLYLINE_PFACE *", 0, 0, (void*)"LibreDWG::Dwg_Entity_POLYLINE_PFACE", 0};
static swig_type_info _swigt__p__dwg_entity_PROXY_ENTITY = {"_p__dwg_entity_PROXY_ENTITY", "Dwg_Entity_PROXY_ENTITY *|struct _dwg_entity_PROXY_ENTITY *|_dwg_entity_PROXY_ENTITY *", 0, 0, (void*)"LibreDWG::Dwg_Entity_PROXY_ENTITY", 0};
static swig_type_info _swigt__p__dwg_entity_RAY = {"_p__dwg_entity_RAY", "Dwg_Entity_RAY *|Dwg_Entity_XLINE *|struct _dwg_entity_RAY *|_dwg_entity_RAY *", 0, 0, (void*)"LibreDWG::Dwg_Entity_RAY", 0};
static swig_type_info _swigt__p__dwg_entity_REPEAT = {"_p__dwg_entity_REPEAT", "Dwg_Entity_REPEAT *|struct _dwg_entity_REPEAT *|_dwg_entity_REPEAT *", 0, 0, (void*)"LibreDWG::Dwg_Entity_REPEAT", 0};
static swig_type_info _swigt__p__dwg_entity_REVOLVEDSURFACE = {"_p__dwg_entity_REVOLVEDSURFACE", "Dwg_Entity_REVOLVEDSURFACE *|struct _dwg_entity_REVOLVEDSURFACE *|_dwg_entity_REVOLVEDSURFACE *", 0, 0, (void*)"LibreDWG::Dwg_Entity_REVOLVEDSURFACE", 0};
static swig_type_info _swigt__p__dwg_entity_ROTATIONGRIPENTITY = {"_p__dwg_entity_ROTATIONGRIPENTITY", "Dwg_Entity_ROTATIONGRIPENTITY *|struct _dwg_entity_ROTATIONGRIPENTITY *|_dwg_entity_ROTATIONGRIPENTITY *", 0, 0, (void*)"LibreDWG::Dwg_Entity_ROTATIONGRIPENTITY", 0};
static swig_type_info _swigt__p__dwg_entity_ROTATIONPARAMETERENTITY = {"_p__dwg_entity_ROTATIONPARAMETERENTITY", "Dwg_Entity_ROTATIONPARAMETERENTITY *|struct _dwg_entity_ROTATIONPARAMETERENTITY *|_dwg_entity_ROTATIONPARAMETERENTITY *", 0, 0, (void*)"LibreDWG::Dwg_Entity_ROTATIONPARAMETERENTITY", 0};
static swig_type_info _swigt__p__dwg_entity_RTEXT = {"_p__dwg_entity_RTEXT", "Dwg_Entity_RTEXT *|struct _dwg_entity_RTEXT *|_dwg_entity_RTEXT *", 0, 0, (void*)"LibreDWG::Dwg_Entity_RTEXT", 0};
static swig_type_info _swigt__p__dwg_entity_SECTIONOBJECT = {"_p__dwg_entity_SECTIONOBJECT", "Dwg_Entity_SECTIONOBJECT *|struct _dwg_entity_SECTIONOBJECT *|_dwg_entity_SECTIONOBJECT *", 0, 0, (void*)"LibreDWG::Dwg_Entity_SECTIONOBJECT", 0};
static swig_type_info _swigt__p__dwg_entity_SEQEND = {"_p__dwg_entity_SEQEND", "Dwg_Entity_SEQEND *|struct _dwg_entity_SEQEND *|_dwg_entity_SEQEND *", 0, 0, (void*)"LibreDWG::Dwg_Entity_SEQEND", 0};
static swig_type_info _swigt__p__dwg_entity_SHAPE = {"_p__dwg_entity_SHAPE", "Dwg_Entity_SHAPE *|struct _dwg_entity_SHAPE *|_dwg_entity_SHAPE *", 0, 0, (void*)"LibreDWG::Dwg_Entity_SHAPE", 0};
static swig_type_info _swigt__p__dwg_entity_SOLID = {"_p__dwg_entity_SOLID", "Dwg_Entity_SOLID *|struct _dwg_entity_SOLID *|_dwg_entity_SOLID *", 0, 0, (void*)"LibreDWG::Dwg_Entity_SOLID", 0};
static swig_type_info _swigt__p__dwg_entity_SPLINE = {"_p__dwg_entity_SPLINE", "Dwg_Entity_SPLINE *|struct _dwg_entity_SPLINE *|_dwg_entity_SPLINE *", 0, 0, (void*)"LibreDWG::Dwg_Entity_SPLINE", 0};
static swig_type_info _swigt__p__dwg_entity_SWEPTSURFACE = {"_p__dwg_entity_SWEPTSURFACE", "Dwg_Entity_SWEPTSURFACE *|struct _dwg_entity_SWEPTSURFACE *|_dwg_entity_SWEPTSURFACE *", 0, 0, (void*)"LibreDWG::Dwg_Entity_SWEPTSURFACE", 0};
static swig_type_info _swigt__p__dwg_entity_TABLE = {"_p__dwg_entity_TABLE", "Dwg_Entity_TABLE *|struct _dwg_entity_TABLE *|_dwg_entity_TABLE *", 0, 0, (void*)"LibreDWG::Dwg_Entity_TABLE", 0};
static swig_type_info _swigt__p__dwg_entity_TEXT = {"_p__dwg_entity_TEXT", "Dwg_Entity_TEXT *|struct _dwg_entity_TEXT *|_dwg_entity_TEXT *", 0, 0, (void*)"LibreDWG::Dwg_Entity_TEXT", 0};
static swig_type_info _swigt__p__dwg_entity_TOLERANCE = {"_p__dwg_entity_TOLERANCE", "Dwg_Entity_TOLERANCE *|struct _dwg_entity_TOLERANCE *|_dwg_entity_TOLERANCE *", 0, 0, (void*)"LibreDWG::Dwg_Entity_TOLERANCE", 0};
static swig_type_info _swigt__p__dwg_entity_TRACE = {"_p__dwg_entity_TRACE", "Dwg_Entity_TRACE *|struct _dwg_entity_TRACE *|_dwg_entity_TRACE *", 0, 0, (void*)"LibreDWG::Dwg_Entity_TRACE", 0};
static swig_type_info _swigt__p__dwg_entity_UNKNOWN_ENT = {"_p__dwg_entity_UNKNOWN_ENT", "Dwg_Entity_UNKNOWN_ENT *|struct _dwg_entity_UNKNOWN_ENT *|_dwg_entity_UNKNOWN_ENT *", 0, 0, (void*)"LibreDWG::Dwg_Entity_UNKNOWN_ENT", 0};
static swig_type_info _swigt__p__dwg_entity_VERTEX_2D = {"_p__dwg_entity_VERTEX_2D", "Dwg_Entity_VERTEX_2D *|struct _dwg_entity_VERTEX_2D *|_dwg_entity_VERTEX_2D *", 0, 0, (void*)"LibreDWG::Dwg_Entity_VERTEX_2D", 0};
static swig_type_info _swigt__p__dwg_entity_VERTEX_3D = {"_p__dwg_entity_VERTEX_3D", "Dwg_Entity_VERTEX_3D *|Dwg_Entity_VERTEX_MESH *|Dwg_Entity_VERTEX_PFACE *|struct _dwg_entity_VERTEX_3D *|_dwg_entity_VERTEX_3D *", 0, 0, (void*)"LibreDWG::Dwg_Entity_VERTEX_3D", 0};
static swig_type_info _swigt__p__dwg_entity_VERTEX_PFACE_FACE = {"_p__dwg_entity_VERTEX_PFACE_FACE", "Dwg_Entity_VERTEX_PFACE_FACE *|struct _dwg_entity_VERTEX_PFACE_FACE *|_dwg_entity_VERTEX_PFACE_FACE *", 0, 0, (void*)"LibreDWG::Dwg_Entity_VERTEX_PFACE_FACE", 0};
static swig_type_info _swigt__p__dwg_entity_VIEWPORT = {"_p__dwg_entity_VIEWPORT", "Dwg_Entity_VIEWPORT *|struct _dwg_entity_VIEWPORT *|_dwg_entity_VIEWPORT *", 0, 0, (void*)"LibreDWG::Dwg_Entity_VIEWPORT", 0};
static swig_type_info _swigt__p__dwg_entity_VISIBILITYGRIPENTITY = {"_p__dwg_entity_VISIBILITYGRIPENTITY", "Dwg_Entity_VISIBILITYGRIPENTITY *|struct _dwg_entity_VISIBILITYGRIPENTITY *|_dwg_entity_VISIBILITYGRIPENTITY *", 0, 0, (void*)"LibreDWG::Dwg_Entity_VISIBILITYGRIPENTITY", 0};
static swig_type_info _swigt__p__dwg_entity_VISIBILITYPARAMETERENTITY = {"_p__dwg_entity_VISIBILITYPARAMETERENTITY", "Dwg_Entity_VISIBILITYPARAMETERENTITY *|struct _dwg_entity_VISIBILITYPARAMETERENTITY *|_dwg_entity_VISIBILITYPARAMETERENTITY *", 0, 0, (void*)"LibreDWG::Dwg_Entity_VISIBILITYPARAMETERENTITY", 0};
static swig_type_info _swigt__p__dwg_entity_WIPEOUT = {"_p__dwg_entity_WIPEOUT", "Dwg_Entity_WIPEOUT *|struct _dwg_entity_WIPEOUT *|_dwg_entity_WIPEOUT *", 0, 0, (void*)"LibreDWG::Dwg_Entity_WIPEOUT", 0};
static swig_type_info _swigt__p__dwg_entity_XYGRIPENTITY = {"_p__dwg_entity_XYGRIPENTITY", "Dwg_Entity_XYGRIPENTITY *|struct _dwg_entity_XYGRIPENTITY *|_dwg_entity_XYGRIPENTITY *", 0, 0, (void*)"LibreDWG::Dwg_Entity_XYGRIPENTITY", 0};
static swig_type_info _swigt__p__dwg_entity_XYPARAMETERENTITY = {"_p__dwg_entity_XYPARAMETERENTITY", "Dwg_Entity_XYPARAMETERENTITY *|struct _dwg_entity_XYPARAMETERENTITY *|_dwg_entity_XYPARAMETERENTITY *", 0, 0, (void*)"LibreDWG::Dwg_Entity_XYPARAMETERENTITY", 0};
static swig_type_info _swigt__p__dwg_entity_eed = {"_p__dwg_entity_eed", "Dwg_Eed *|struct _dwg_entity_eed *|_dwg_entity_eed *", 0, 0, (void*)"LibreDWG::Dwg_Eed", 0};
static swig_type_info _swigt__p__dwg_entity_eed_data = {"_p__dwg_entity_eed_data", "Dwg_Eed_Data *|struct _dwg_entity_eed_data *|_dwg_entity_eed_data *", 0, 0, (void*)"LibreDWG::Dwg_Eed_Data", 0};
static swig_type_info _swigt__p__dwg_filedeplist = {"_p__dwg_filedeplist", "Dwg_FileDepList *|struct _dwg_filedeplist *|_dwg_filedeplist *", 0, 0, (void*)"LibreDWG::Dwg_FileDepList", 0};
static swig_type_info _swigt__p__dwg_handle = {"_p__dwg_handle", "Dwg_Handle *|struct _dwg_handle *|_dwg_handle *", 0, 0, (void*)"LibreDWG::Dwg_Handle", 0};
static swig_type_info _swigt__p__dwg_header = {"_p__dwg_header", "Dwg_Header *|struct _dwg_header *|_dwg_header *", 0, 0, (void*)"LibreDWG::Dwg_Header", 0};
static swig_type_info _swigt__p__dwg_header_variables = {"_p__dwg_header_variables", "Dwg_Header_Variables *|struct _dwg_header_variables *|_dwg_header_variables *", 0, 0, (void*)"LibreDWG::Dwg_Header_Variables", 0};
static swig_type_info _swigt__p__dwg_object = {"_p__dwg_object", "Dwg_Object *|struct _dwg_object *|_dwg_object *", 0, 0, (void*)"LibreDWG::Dwg_Object", 0};
static swig_type_info _swigt__p__dwg_object_ACMECOMMANDHISTORY = {"_p__dwg_object_ACMECOMMANDHISTORY", "Dwg_Object_ACMECOMMANDHISTORY *|struct _dwg_object_ACMECOMMANDHISTORY *|_dwg_object_ACMECOMMANDHISTORY *", 0, 0, (void*)"LibreDWG::Dwg_Object_ACMECOMMANDHISTORY", 0};
static swig_type_info _swigt__p__dwg_object_ACMESCOPE = {"_p__dwg_object_ACMESCOPE", "Dwg_Object_ACMESCOPE *|struct _dwg_object_ACMESCOPE *|_dwg_object_ACMESCOPE *", 0, 0, (void*)"LibreDWG::Dwg_Object_ACMESCOPE", 0};
static swig_type_info _swigt__p__dwg_object_ACMESTATEMGR = {"_p__dwg_object_ACMESTATEMGR", "Dwg_Object_ACMESTATEMGR *|struct _dwg_object_ACMESTATEMGR *|_dwg_object_ACMESTATEMGR *", 0, 0, (void*)"LibreDWG::Dwg_Object_ACMESTATEMGR", 0};
static swig_type_info _swigt__p__dwg_object_ACSH_BOOLEAN_CLASS = {"_p__dwg_object_ACSH_BOOLEAN_CLASS", "Dwg_Object_ACSH_BOOLEAN_CLASS *|struct _dwg_object_ACSH_BOOLEAN_CLASS *|_dwg_object_ACSH_BOOLEAN_CLASS *", 0, 0, (void*)"LibreDWG::Dwg_Object_ACSH_BOOLEAN_CLASS", 0};
static swig_type_info _swigt__p__dwg_object_ACSH_BOX_CLASS = {"_p__dwg_object_ACSH_BOX_CLASS", "Dwg_Object_ACSH_BOX_CLASS *|struct _dwg_object_ACSH_BOX_CLASS *|_dwg_object_ACSH_BOX_CLASS *", 0, 0, (void*)"LibreDWG::Dwg_Object_ACSH_BOX_CLASS", 0};
static swig_type_info _swigt__p__dwg_object_ACSH_BREP_CLASS = {"_p__dwg_object_ACSH_BREP_CLASS", "Dwg_Object_ACSH_BREP_CLASS *|struct _dwg_object_ACSH_BREP_CLASS *|_dwg_object_ACSH_BREP_CLASS *", 0, 0, (void*)"LibreDWG::Dwg_Object_ACSH_BREP_CLASS", 0};
static swig_type_info _swigt__p__dwg_object_ACSH_CHAMFER_CLASS = {"_p__dwg_object_ACSH_CHAMFER_CLASS", "Dwg_Object_ACSH_CHAMFER_CLASS *|struct _dwg_object_ACSH_CHAMFER_CLASS *|_dwg_object_ACSH_CHAMFER_CLASS *", 0, 0, (void*)"LibreDWG::Dwg_Object_ACSH_CHAMFER_CLASS", 0};
static swig_type_info _swigt__p__dwg_object_ACSH_CONE_CLASS = {"_p__dwg_object_ACSH_CONE_CLASS", "Dwg_Object_ACSH_CONE_CLASS *|struct _dwg_object_ACSH_CONE_CLASS *|_dwg_object_ACSH_CONE_CLASS *", 0, 0, (void*)"LibreDWG::Dwg_Object_ACSH_CONE_CLASS", 0};
static swig_type_info _swigt__p__dwg_object_ACSH_CYLINDER_CLASS = {"_p__dwg_object_ACSH_CYLINDER_CLASS", "Dwg_Object_ACSH_CYLINDER_CLASS *|struct _dwg_object_ACSH_CYLINDER_CLASS *|_dwg_object_ACSH_CYLINDER_CLASS *", 0, 0, (void*)"LibreDWG::Dwg_Object_ACSH_CYLINDER_CLASS", 0};
static swig_type_info _swigt__p__dwg_object_ACSH_EXTRUSION_CLASS = {"_p__dwg_object_ACSH_EXTRUSION_CLASS", "Dwg_Object_ACSH_EXTRUSION_CLASS *|struct _dwg_object_ACSH_EXTRUSION_CLASS *|_dwg_object_ACSH_EXTRUSION_CLASS *", 0, 0, (void*)"LibreDWG::Dwg_Object_ACSH_EXTRUSION_CLASS", 0};
static swig_type_info _swigt__p__dwg_object_ACSH_FILLET_CLASS = {"_p__dwg_object_ACSH_FILLET_CLASS", "Dwg_Object_ACSH_FILLET_CLASS *|struct _dwg_object_ACSH_FILLET_CLASS *|_dwg_object_ACSH_FILLET_CLASS *", 0, 0, (void*)"LibreDWG::Dwg_Object_ACSH_FILLET_CLASS", 0};
static swig_type_info _swigt__p__dwg_object_ACSH_HISTORY_CLASS = {"_p__dwg_object_ACSH_HISTORY_CLASS", "Dwg_Object_ACSH_HISTORY_CLASS *|struct _dwg_object_ACSH_HISTORY_CLASS *|_dwg_object_ACSH_HISTORY_CLASS *", 0, 0, (void*)"LibreDWG::Dwg_Object_ACSH_HISTORY_CLASS", 0};
static swig_type_info _swigt__p__dwg_object_ACSH_LOFT_CLASS = {"_p__dwg_object_ACSH_LOFT_CLASS", "Dwg_Object_ACSH_LOFT_CLASS *|struct _dwg_object_ACSH_LOFT_CLASS *|_dwg_object_ACSH_LOFT_CLASS *", 0, 0, (void*)"LibreDWG::Dwg_Object_ACSH_LOFT_CLASS", 0};
static swig_type_info _swigt__p__dwg_object_ACSH_PYRAMID_CLASS = {"_p__dwg_object_ACSH_PYRAMID_CLASS", "Dwg_Object_ACSH_PYRAMID_CLASS *|struct _dwg_object_ACSH_PYRAMID_CLASS *|_dwg_object_ACSH_PYRAMID_CLASS *", 0, 0, (void*)"LibreDWG::Dwg_Object_ACSH_PYRAMID_CLASS", 0};
static swig_type_info _swigt__p__dwg_object_ACSH_REVOLVE_CLASS = {"_p__dwg_object_ACSH_REVOLVE_CLASS", "Dwg_Object_ACSH_REVOLVE_CLASS *|struct _dwg_object_ACSH_REVOLVE_CLASS *|_dwg_object_ACSH_REVOLVE_CLASS *", 0, 0, (void*)"LibreDWG::Dwg_Object_ACSH_REVOLVE_CLASS", 0};
static swig_type_info _swigt__p__dwg_object_ACSH_SPHERE_CLASS = {"_p__dwg_object_ACSH_SPHERE_CLASS", "Dwg_Object_ACSH_SPHERE_CLASS *|struct _dwg_object_ACSH_SPHERE_CLASS *|_dwg_object_ACSH_SPHERE_CLASS *", 0, 0, (void*)"LibreDWG::Dwg_Object_ACSH_SPHERE_CLASS", 0};
static swig_type_info _swigt__p__dwg_object_ACSH_SWEEP_CLASS = {"_p__dwg_object_ACSH_SWEEP_CLASS", "Dwg_Object_ACSH_SWEEP_CLASS *|struct _dwg_object_ACSH_SWEEP_CLASS *|_dwg_object_ACSH_SWEEP_CLASS *", 0, 0, (void*)"LibreDWG::Dwg_Object_ACSH_SWEEP_CLASS", 0};
static swig_type_info _swigt__p__dwg_object_ACSH_TORUS_CLASS = {"_p__dwg_object_ACSH_TORUS_CLASS", "Dwg_Object_ACSH_TORUS_CLASS *|struct _dwg_object_ACSH_TORUS_CLASS *|_dwg_object_ACSH_TORUS_CLASS *", 0, 0, (void*)"LibreDWG::Dwg_Object_ACSH_TORUS_CLASS", 0};
static swig_type_info _swigt__p__dwg_object_ACSH_WEDGE_CLASS = {"_p__dwg_object_ACSH_WEDGE_CLASS", "Dwg_Object_ACSH_WEDGE_CLASS *|struct _dwg_object_ACSH_WEDGE_CLASS *|_dwg_object_ACSH_WEDGE_CLASS *", 0, 0, (void*)"LibreDWG::Dwg_Object_ACSH_WEDGE_CLASS", 0};
static swig_type_info _swigt__p__dwg_object_ALDIMOBJECTCONTEXTDATA = {"_p__dwg_object_ALDIMOBJECTCONTEXTDATA", "Dwg_Object_ALDIMOBJECTCONTEXTDATA *|struct _dwg_object_ALDIMOBJECTCONTEXTDATA *|_dwg_object_ALDIMOBJECTCONTEXTDATA *", 0, 0, (void*)"LibreDWG::Dwg_Object_ALDIMOBJECTCONTEXTDATA", 0};
static swig_type_info _swigt__p__dwg_object_ANGDIMOBJECTCONTEXTDATA = {"_p__dwg_object_ANGDIMOBJECTCONTEXTDATA", "Dwg_Object_ANGDIMOBJECTCONTEXTDATA *|struct _dwg_object_ANGDIMOBJECTCONTEXTDATA *|_dwg_object_ANGDIMOBJECTCONTEXTDATA *", 0, 0, (void*)"LibreDWG::Dwg_Object_ANGDIMOBJECTCONTEXTDATA", 0};
static swig_type_info _swigt__p__dwg_object_ANNOTSCALEOBJECTCONTEXTDATA = {"_p__dwg_object_ANNOTSCALEOBJECTCONTEXTDATA", "Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA *|struct _dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *|_dwg_object_ANNOTSCALEOBJECTCONTEXTDATA *", 0, 0, (void*)"LibreDWG::Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA", 0};
static swig_type_info _swigt__p__dwg_object_APPID = {"_p__dwg_object_APPID", "Dwg_Object_APPID *|struct _dwg_object_APPID *|_dwg_object_APPID *", 0, 0, (void*)"LibreDWG::Dwg_Object_APPID", 0};
static swig_type_info _swigt__p__dwg_object_APPID_CONTROL = {"_p__dwg_object_APPID_CONTROL", "Dwg_Object_APPID_CONTROL *|struct _dwg_object_APPID_CONTROL *|_dwg_object_APPID_CONTROL *", 0, 0, (void*)"LibreDWG::Dwg_Object_APPID_CONTROL", 0};
static swig_type_info _swigt__p__dwg_object_ASSOC2DCONSTRAINTGROUP = {"_p__dwg_object_ASSOC2DCONSTRAINTGROUP", "Dwg_Object_ASSOC2DCONSTRAINTGROUP *|struct _dwg_object_ASSOC2DCONSTRAINTGROUP *|_dwg_object_ASSOC2DCONSTRAINTGROUP *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOC2DCONSTRAINTGROUP", 0};
static swig_type_info _swigt__p__dwg_object_ASSOC3POINTANGULARDIMACTIONBODY = {"_p__dwg_object_ASSOC3POINTANGULARDIMACTIONBODY", "Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY *|struct _dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *|_dwg_object_ASSOC3POINTANGULARDIMACTIONBODY *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCACTION = {"_p__dwg_object_ASSOCACTION", "Dwg_Object_ASSOCACTION *|struct _dwg_object_ASSOCACTION *|_dwg_object_ASSOCACTION *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCACTION", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCACTIONPARAM = {"_p__dwg_object_ASSOCACTIONPARAM", "Dwg_Object_ASSOCACTIONPARAM *|struct _dwg_object_ASSOCACTIONPARAM *|_dwg_object_ASSOCACTIONPARAM *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCACTIONPARAM", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCALIGNEDDIMACTIONBODY = {"_p__dwg_object_ASSOCALIGNEDDIMACTIONBODY", "Dwg_Object_ASSOCALIGNEDDIMACTIONBODY *|struct _dwg_object_ASSOCALIGNEDDIMACTIONBODY *|_dwg_object_ASSOCALIGNEDDIMACTIONBODY *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCARRAYACTIONBODY = {"_p__dwg_object_ASSOCARRAYACTIONBODY", "Dwg_Object_ASSOCARRAYACTIONBODY *|struct _dwg_object_ASSOCARRAYACTIONBODY *|_dwg_object_ASSOCARRAYACTIONBODY *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCARRAYACTIONBODY", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCARRAYMODIFYACTIONBODY = {"_p__dwg_object_ASSOCARRAYMODIFYACTIONBODY", "Dwg_Object_ASSOCARRAYMODIFYACTIONBODY *|struct _dwg_object_ASSOCARRAYMODIFYACTIONBODY *|_dwg_object_ASSOCARRAYMODIFYACTIONBODY *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCASMBODYACTIONPARAM = {"_p__dwg_object_ASSOCASMBODYACTIONPARAM", "Dwg_Object_ASSOCASMBODYACTIONPARAM *|struct _dwg_object_ASSOCASMBODYACTIONPARAM *|_dwg_object_ASSOCASMBODYACTIONPARAM *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCASMBODYACTIONPARAM", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCBLENDSURFACEACTIONBODY = {"_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY", "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY *|struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY *|_dwg_object_ASSOCBLENDSURFACEACTIONBODY *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCCOMPOUNDACTIONPARAM = {"_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM", "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM *|struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM *|_dwg_object_ASSOCCOMPOUNDACTIONPARAM *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCDEPENDENCY = {"_p__dwg_object_ASSOCDEPENDENCY", "Dwg_Object_ASSOCDEPENDENCY *|struct _dwg_object_ASSOCDEPENDENCY *|_dwg_object_ASSOCDEPENDENCY *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCDEPENDENCY", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCDIMDEPENDENCYBODY = {"_p__dwg_object_ASSOCDIMDEPENDENCYBODY", "Dwg_Object_ASSOCDIMDEPENDENCYBODY *|struct _dwg_object_ASSOCDIMDEPENDENCYBODY *|_dwg_object_ASSOCDIMDEPENDENCYBODY *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCDIMDEPENDENCYBODY", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCEDGEACTIONPARAM = {"_p__dwg_object_ASSOCEDGEACTIONPARAM", "Dwg_Object_ASSOCEDGEACTIONPARAM *|struct _dwg_object_ASSOCEDGEACTIONPARAM *|_dwg_object_ASSOCEDGEACTIONPARAM *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCEDGEACTIONPARAM", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCEDGECHAMFERACTIONBODY = {"_p__dwg_object_ASSOCEDGECHAMFERACTIONBODY", "Dwg_Object_ASSOCEDGECHAMFERACTIONBODY *|struct _dwg_object_ASSOCEDGECHAMFERACTIONBODY *|_dwg_object_ASSOCEDGECHAMFERACTIONBODY *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCEDGECHAMFERACTIONBODY", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCEDGEFILLETACTIONBODY = {"_p__dwg_object_ASSOCEDGEFILLETACTIONBODY", "Dwg_Object_ASSOCEDGEFILLETACTIONBODY *|struct _dwg_object_ASSOCEDGEFILLETACTIONBODY *|_dwg_object_ASSOCEDGEFILLETACTIONBODY *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCEDGEFILLETACTIONBODY", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCEXTENDSURFACEACTIONBODY = {"_p__dwg_object_ASSOCEXTENDSURFACEACTIONBODY", "Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY *|struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY *|_dwg_object_ASSOCEXTENDSURFACEACTIONBODY *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY = {"_p__dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY", "Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY *|struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *|_dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCFACEACTIONPARAM = {"_p__dwg_object_ASSOCFACEACTIONPARAM", "Dwg_Object_ASSOCFACEACTIONPARAM *|struct _dwg_object_ASSOCFACEACTIONPARAM *|_dwg_object_ASSOCFACEACTIONPARAM *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCFACEACTIONPARAM", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCFILLETSURFACEACTIONBODY = {"_p__dwg_object_ASSOCFILLETSURFACEACTIONBODY", "Dwg_Object_ASSOCFILLETSURFACEACTIONBODY *|struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY *|_dwg_object_ASSOCFILLETSURFACEACTIONBODY *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCGEOMDEPENDENCY = {"_p__dwg_object_ASSOCGEOMDEPENDENCY", "Dwg_Object_ASSOCGEOMDEPENDENCY *|struct _dwg_object_ASSOCGEOMDEPENDENCY *|_dwg_object_ASSOCGEOMDEPENDENCY *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCGEOMDEPENDENCY", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCLOFTEDSURFACEACTIONBODY = {"_p__dwg_object_ASSOCLOFTEDSURFACEACTIONBODY", "Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY *|struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *|_dwg_object_ASSOCLOFTEDSURFACEACTIONBODY *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCMLEADERACTIONBODY = {"_p__dwg_object_ASSOCMLEADERACTIONBODY", "Dwg_Object_ASSOCMLEADERACTIONBODY *|struct _dwg_object_ASSOCMLEADERACTIONBODY *|_dwg_object_ASSOCMLEADERACTIONBODY *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCMLEADERACTIONBODY", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCNETWORK = {"_p__dwg_object_ASSOCNETWORK", "Dwg_Object_ASSOCNETWORK *|struct _dwg_object_ASSOCNETWORK *|_dwg_object_ASSOCNETWORK *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCNETWORK", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCNETWORKSURFACEACTIONBODY = {"_p__dwg_object_ASSOCNETWORKSURFACEACTIONBODY", "Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY *|struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY *|_dwg_object_ASSOCNETWORKSURFACEACTIONBODY *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCOBJECTACTIONPARAM = {"_p__dwg_object_ASSOCOBJECTACTIONPARAM", "Dwg_Object_ASSOCOBJECTACTIONPARAM *|struct _dwg_object_ASSOCOBJECTACTIONPARAM *|_dwg_object_ASSOCOBJECTACTIONPARAM *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCOBJECTACTIONPARAM", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCOFFSETSURFACEACTIONBODY = {"_p__dwg_object_ASSOCOFFSETSURFACEACTIONBODY", "Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY *|struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY *|_dwg_object_ASSOCOFFSETSURFACEACTIONBODY *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCORDINATEDIMACTIONBODY = {"_p__dwg_object_ASSOCORDINATEDIMACTIONBODY", "Dwg_Object_ASSOCORDINATEDIMACTIONBODY *|struct _dwg_object_ASSOCORDINATEDIMACTIONBODY *|_dwg_object_ASSOCORDINATEDIMACTIONBODY *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCORDINATEDIMACTIONBODY", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM = {"_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM", "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM *|struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *|_dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCPATCHSURFACEACTIONBODY = {"_p__dwg_object_ASSOCPATCHSURFACEACTIONBODY", "Dwg_Object_ASSOCPATCHSURFACEACTIONBODY *|struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY *|_dwg_object_ASSOCPATCHSURFACEACTIONBODY *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCPATCHSURFACEACTIONBODY", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCPATHACTIONPARAM = {"_p__dwg_object_ASSOCPATHACTIONPARAM", "Dwg_Object_ASSOCPATHACTIONPARAM *|struct _dwg_object_ASSOCPATHACTIONPARAM *|_dwg_object_ASSOCPATHACTIONPARAM *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCPATHACTIONPARAM", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCPERSSUBENTMANAGER = {"_p__dwg_object_ASSOCPERSSUBENTMANAGER", "Dwg_Object_ASSOCPERSSUBENTMANAGER *|struct _dwg_object_ASSOCPERSSUBENTMANAGER *|_dwg_object_ASSOCPERSSUBENTMANAGER *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCPERSSUBENTMANAGER", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCPLANESURFACEACTIONBODY = {"_p__dwg_object_ASSOCPLANESURFACEACTIONBODY", "Dwg_Object_ASSOCPLANESURFACEACTIONBODY *|struct _dwg_object_ASSOCPLANESURFACEACTIONBODY *|_dwg_object_ASSOCPLANESURFACEACTIONBODY *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCPLANESURFACEACTIONBODY", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCPOINTREFACTIONPARAM = {"_p__dwg_object_ASSOCPOINTREFACTIONPARAM", "Dwg_Object_ASSOCPOINTREFACTIONPARAM *|struct _dwg_object_ASSOCPOINTREFACTIONPARAM *|_dwg_object_ASSOCPOINTREFACTIONPARAM *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCPOINTREFACTIONPARAM", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY = {"_p__dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY", "Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY *|struct _dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *|_dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY = {"_p__dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY", "Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY *|struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *|_dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCROTATEDDIMACTIONBODY = {"_p__dwg_object_ASSOCROTATEDDIMACTIONBODY", "Dwg_Object_ASSOCROTATEDDIMACTIONBODY *|struct _dwg_object_ASSOCROTATEDDIMACTIONBODY *|_dwg_object_ASSOCROTATEDDIMACTIONBODY *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCROTATEDDIMACTIONBODY", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCSWEPTSURFACEACTIONBODY = {"_p__dwg_object_ASSOCSWEPTSURFACEACTIONBODY", "Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY *|struct _dwg_object_ASSOCSWEPTSURFACEACTIONBODY *|_dwg_object_ASSOCSWEPTSURFACEACTIONBODY *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCTRIMSURFACEACTIONBODY = {"_p__dwg_object_ASSOCTRIMSURFACEACTIONBODY", "Dwg_Object_ASSOCTRIMSURFACEACTIONBODY *|struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY *|_dwg_object_ASSOCTRIMSURFACEACTIONBODY *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCVALUEDEPENDENCY = {"_p__dwg_object_ASSOCVALUEDEPENDENCY", "Dwg_Object_ASSOCVALUEDEPENDENCY *|struct _dwg_object_ASSOCVALUEDEPENDENCY *|_dwg_object_ASSOCVALUEDEPENDENCY *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCVALUEDEPENDENCY", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCVARIABLE = {"_p__dwg_object_ASSOCVARIABLE", "Dwg_Object_ASSOCVARIABLE *|struct _dwg_object_ASSOCVARIABLE *|_dwg_object_ASSOCVARIABLE *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCVARIABLE", 0};
static swig_type_info _swigt__p__dwg_object_ASSOCVERTEXACTIONPARAM = {"_p__dwg_object_ASSOCVERTEXACTIONPARAM", "Dwg_Object_ASSOCVERTEXACTIONPARAM *|struct _dwg_object_ASSOCVERTEXACTIONPARAM *|_dwg_object_ASSOCVERTEXACTIONPARAM *", 0, 0, (void*)"LibreDWG::Dwg_Object_ASSOCVERTEXACTIONPARAM", 0};
static swig_type_info _swigt__p__dwg_object_BLKREFOBJECTCONTEXTDATA = {"_p__dwg_object_BLKREFOBJECTCONTEXTDATA", "Dwg_Object_BLKREFOBJECTCONTEXTDATA *|struct _dwg_object_BLKREFOBJECTCONTEXTDATA *|_dwg_object_BLKREFOBJECTCONTEXTDATA *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLKREFOBJECTCONTEXTDATA", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER = {"_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER", "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER *|struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *|_dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKALIGNMENTGRIP = {"_p__dwg_object_BLOCKALIGNMENTGRIP", "Dwg_Object_BLOCKALIGNMENTGRIP *|struct _dwg_object_BLOCKALIGNMENTGRIP *|_dwg_object_BLOCKALIGNMENTGRIP *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKALIGNMENTGRIP", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKALIGNMENTPARAMETER = {"_p__dwg_object_BLOCKALIGNMENTPARAMETER", "Dwg_Object_BLOCKALIGNMENTPARAMETER *|struct _dwg_object_BLOCKALIGNMENTPARAMETER *|_dwg_object_BLOCKALIGNMENTPARAMETER *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKALIGNMENTPARAMETER", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER = {"_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER", "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER *|struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *|_dwg_object_BLOCKANGULARCONSTRAINTPARAMETER *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKARRAYACTION = {"_p__dwg_object_BLOCKARRAYACTION", "Dwg_Object_BLOCKARRAYACTION *|struct _dwg_object_BLOCKARRAYACTION *|_dwg_object_BLOCKARRAYACTION *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKARRAYACTION", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKBASEPOINTPARAMETER = {"_p__dwg_object_BLOCKBASEPOINTPARAMETER", "Dwg_Object_BLOCKBASEPOINTPARAMETER *|struct _dwg_object_BLOCKBASEPOINTPARAMETER *|_dwg_object_BLOCKBASEPOINTPARAMETER *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKBASEPOINTPARAMETER", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER = {"_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER", "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER *|struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *|_dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKFLIPACTION = {"_p__dwg_object_BLOCKFLIPACTION", "Dwg_Object_BLOCKFLIPACTION *|struct _dwg_object_BLOCKFLIPACTION *|_dwg_object_BLOCKFLIPACTION *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKFLIPACTION", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKFLIPGRIP = {"_p__dwg_object_BLOCKFLIPGRIP", "Dwg_Object_BLOCKFLIPGRIP *|struct _dwg_object_BLOCKFLIPGRIP *|_dwg_object_BLOCKFLIPGRIP *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKFLIPGRIP", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKFLIPPARAMETER = {"_p__dwg_object_BLOCKFLIPPARAMETER", "Dwg_Object_BLOCKFLIPPARAMETER *|struct _dwg_object_BLOCKFLIPPARAMETER *|_dwg_object_BLOCKFLIPPARAMETER *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKFLIPPARAMETER", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKGRIPLOCATIONCOMPONENT = {"_p__dwg_object_BLOCKGRIPLOCATIONCOMPONENT", "Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT *|struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT *|_dwg_object_BLOCKGRIPLOCATIONCOMPONENT *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER = {"_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER", "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER *|struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *|_dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER = {"_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER", "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER *|struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *|_dwg_object_BLOCKLINEARCONSTRAINTPARAMETER *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKLINEARGRIP = {"_p__dwg_object_BLOCKLINEARGRIP", "Dwg_Object_BLOCKLINEARGRIP *|struct _dwg_object_BLOCKLINEARGRIP *|_dwg_object_BLOCKLINEARGRIP *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKLINEARGRIP", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKLINEARPARAMETER = {"_p__dwg_object_BLOCKLINEARPARAMETER", "Dwg_Object_BLOCKLINEARPARAMETER *|struct _dwg_object_BLOCKLINEARPARAMETER *|_dwg_object_BLOCKLINEARPARAMETER *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKLINEARPARAMETER", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKLOOKUPACTION = {"_p__dwg_object_BLOCKLOOKUPACTION", "Dwg_Object_BLOCKLOOKUPACTION *|struct _dwg_object_BLOCKLOOKUPACTION *|_dwg_object_BLOCKLOOKUPACTION *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKLOOKUPACTION", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKLOOKUPGRIP = {"_p__dwg_object_BLOCKLOOKUPGRIP", "Dwg_Object_BLOCKLOOKUPGRIP *|struct _dwg_object_BLOCKLOOKUPGRIP *|_dwg_object_BLOCKLOOKUPGRIP *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKLOOKUPGRIP", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKLOOKUPPARAMETER = {"_p__dwg_object_BLOCKLOOKUPPARAMETER", "Dwg_Object_BLOCKLOOKUPPARAMETER *|struct _dwg_object_BLOCKLOOKUPPARAMETER *|_dwg_object_BLOCKLOOKUPPARAMETER *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKLOOKUPPARAMETER", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKMOVEACTION = {"_p__dwg_object_BLOCKMOVEACTION", "Dwg_Object_BLOCKMOVEACTION *|struct _dwg_object_BLOCKMOVEACTION *|_dwg_object_BLOCKMOVEACTION *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKMOVEACTION", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKPARAMDEPENDENCYBODY = {"_p__dwg_object_BLOCKPARAMDEPENDENCYBODY", "Dwg_Object_BLOCKPARAMDEPENDENCYBODY *|struct _dwg_object_BLOCKPARAMDEPENDENCYBODY *|_dwg_object_BLOCKPARAMDEPENDENCYBODY *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKPARAMDEPENDENCYBODY", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKPOINTPARAMETER = {"_p__dwg_object_BLOCKPOINTPARAMETER", "Dwg_Object_BLOCKPOINTPARAMETER *|struct _dwg_object_BLOCKPOINTPARAMETER *|_dwg_object_BLOCKPOINTPARAMETER *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKPOINTPARAMETER", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKPOLARGRIP = {"_p__dwg_object_BLOCKPOLARGRIP", "Dwg_Object_BLOCKPOLARGRIP *|struct _dwg_object_BLOCKPOLARGRIP *|_dwg_object_BLOCKPOLARGRIP *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKPOLARGRIP", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKPOLARPARAMETER = {"_p__dwg_object_BLOCKPOLARPARAMETER", "Dwg_Object_BLOCKPOLARPARAMETER *|struct _dwg_object_BLOCKPOLARPARAMETER *|_dwg_object_BLOCKPOLARPARAMETER *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKPOLARPARAMETER", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKPOLARSTRETCHACTION = {"_p__dwg_object_BLOCKPOLARSTRETCHACTION", "Dwg_Object_BLOCKPOLARSTRETCHACTION *|struct _dwg_object_BLOCKPOLARSTRETCHACTION *|_dwg_object_BLOCKPOLARSTRETCHACTION *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKPOLARSTRETCHACTION", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKPROPERTIESTABLE = {"_p__dwg_object_BLOCKPROPERTIESTABLE", "Dwg_Object_BLOCKPROPERTIESTABLE *|struct _dwg_object_BLOCKPROPERTIESTABLE *|_dwg_object_BLOCKPROPERTIESTABLE *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKPROPERTIESTABLE", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKPROPERTIESTABLEGRIP = {"_p__dwg_object_BLOCKPROPERTIESTABLEGRIP", "Dwg_Object_BLOCKPROPERTIESTABLEGRIP *|struct _dwg_object_BLOCKPROPERTIESTABLEGRIP *|_dwg_object_BLOCKPROPERTIESTABLEGRIP *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKPROPERTIESTABLEGRIP", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER = {"_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER", "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER *|struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *|_dwg_object_BLOCKRADIALCONSTRAINTPARAMETER *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKREPRESENTATION = {"_p__dwg_object_BLOCKREPRESENTATION", "Dwg_Object_BLOCKREPRESENTATION *|struct _dwg_object_BLOCKREPRESENTATION *|_dwg_object_BLOCKREPRESENTATION *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKREPRESENTATION", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKROTATEACTION = {"_p__dwg_object_BLOCKROTATEACTION", "Dwg_Object_BLOCKROTATEACTION *|struct _dwg_object_BLOCKROTATEACTION *|_dwg_object_BLOCKROTATEACTION *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKROTATEACTION", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKROTATIONGRIP = {"_p__dwg_object_BLOCKROTATIONGRIP", "Dwg_Object_BLOCKROTATIONGRIP *|struct _dwg_object_BLOCKROTATIONGRIP *|_dwg_object_BLOCKROTATIONGRIP *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKROTATIONGRIP", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKROTATIONPARAMETER = {"_p__dwg_object_BLOCKROTATIONPARAMETER", "Dwg_Object_BLOCKROTATIONPARAMETER *|struct _dwg_object_BLOCKROTATIONPARAMETER *|_dwg_object_BLOCKROTATIONPARAMETER *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKROTATIONPARAMETER", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKSCALEACTION = {"_p__dwg_object_BLOCKSCALEACTION", "Dwg_Object_BLOCKSCALEACTION *|struct _dwg_object_BLOCKSCALEACTION *|_dwg_object_BLOCKSCALEACTION *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKSCALEACTION", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKSTRETCHACTION = {"_p__dwg_object_BLOCKSTRETCHACTION", "Dwg_Object_BLOCKSTRETCHACTION *|struct _dwg_object_BLOCKSTRETCHACTION *|_dwg_object_BLOCKSTRETCHACTION *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKSTRETCHACTION", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKUSERPARAMETER = {"_p__dwg_object_BLOCKUSERPARAMETER", "Dwg_Object_BLOCKUSERPARAMETER *|struct _dwg_object_BLOCKUSERPARAMETER *|_dwg_object_BLOCKUSERPARAMETER *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKUSERPARAMETER", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER = {"_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER", "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER *|struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *|_dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKVISIBILITYGRIP = {"_p__dwg_object_BLOCKVISIBILITYGRIP", "Dwg_Object_BLOCKVISIBILITYGRIP *|struct _dwg_object_BLOCKVISIBILITYGRIP *|_dwg_object_BLOCKVISIBILITYGRIP *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKVISIBILITYGRIP", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKVISIBILITYPARAMETER = {"_p__dwg_object_BLOCKVISIBILITYPARAMETER", "Dwg_Object_BLOCKVISIBILITYPARAMETER *|struct _dwg_object_BLOCKVISIBILITYPARAMETER *|_dwg_object_BLOCKVISIBILITYPARAMETER *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKVISIBILITYPARAMETER", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKXYGRIP = {"_p__dwg_object_BLOCKXYGRIP", "Dwg_Object_BLOCKXYGRIP *|struct _dwg_object_BLOCKXYGRIP *|_dwg_object_BLOCKXYGRIP *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKXYGRIP", 0};
static swig_type_info _swigt__p__dwg_object_BLOCKXYPARAMETER = {"_p__dwg_object_BLOCKXYPARAMETER", "Dwg_Object_BLOCKXYPARAMETER *|struct _dwg_object_BLOCKXYPARAMETER *|_dwg_object_BLOCKXYPARAMETER *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCKXYPARAMETER", 0};
static swig_type_info _swigt__p__dwg_object_BLOCK_CONTROL = {"_p__dwg_object_BLOCK_CONTROL", "Dwg_Object_BLOCK_CONTROL *|struct _dwg_object_BLOCK_CONTROL *|_dwg_object_BLOCK_CONTROL *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCK_CONTROL", 0};
static swig_type_info _swigt__p__dwg_object_BLOCK_HEADER = {"_p__dwg_object_BLOCK_HEADER", "Dwg_Object_BLOCK_HEADER *|struct _dwg_object_BLOCK_HEADER *|_dwg_object_BLOCK_HEADER *", 0, 0, (void*)"LibreDWG::Dwg_Object_BLOCK_HEADER", 0};
static swig_type_info _swigt__p__dwg_object_BREAKDATA = {"_p__dwg_object_BREAKDATA", "Dwg_Object_BREAKDATA *|struct _dwg_object_BREAKDATA *|_dwg_object_BREAKDATA *", 0, 0, (void*)"LibreDWG::Dwg_Object_BREAKDATA", 0};
static swig_type_info _swigt__p__dwg_object_BREAKPOINTREF = {"_p__dwg_object_BREAKPOINTREF", "Dwg_Object_BREAKPOINTREF *|struct _dwg_object_BREAKPOINTREF *|_dwg_object_BREAKPOINTREF *", 0, 0, (void*)"LibreDWG::Dwg_Object_BREAKPOINTREF", 0};
static swig_type_info _swigt__p__dwg_object_CELLSTYLEMAP = {"_p__dwg_object_CELLSTYLEMAP", "Dwg_Object_CELLSTYLEMAP *|struct _dwg_object_CELLSTYLEMAP *|_dwg_object_CELLSTYLEMAP *", 0, 0, (void*)"LibreDWG::Dwg_Object_CELLSTYLEMAP", 0};
static swig_type_info _swigt__p__dwg_object_CONTEXTDATAMANAGER = {"_p__dwg_object_CONTEXTDATAMANAGER", "Dwg_Object_CONTEXTDATAMANAGER *|struct _dwg_object_CONTEXTDATAMANAGER *|_dwg_object_CONTEXTDATAMANAGER *", 0, 0, (void*)"LibreDWG::Dwg_Object_CONTEXTDATAMANAGER", 0};
static swig_type_info _swigt__p__dwg_object_CSACDOCUMENTOPTIONS = {"_p__dwg_object_CSACDOCUMENTOPTIONS", "Dwg_Object_CSACDOCUMENTOPTIONS *|struct _dwg_object_CSACDOCUMENTOPTIONS *|_dwg_object_CSACDOCUMENTOPTIONS *", 0, 0, (void*)"LibreDWG::Dwg_Object_CSACDOCUMENTOPTIONS", 0};
static swig_type_info _swigt__p__dwg_object_CURVEPATH = {"_p__dwg_object_CURVEPATH", "Dwg_Object_CURVEPATH *|struct _dwg_object_CURVEPATH *|_dwg_object_CURVEPATH *", 0, 0, (void*)"LibreDWG::Dwg_Object_CURVEPATH", 0};
static swig_type_info _swigt__p__dwg_object_DATALINK = {"_p__dwg_object_DATALINK", "Dwg_Object_DATALINK *|struct _dwg_object_DATALINK *|_dwg_object_DATALINK *", 0, 0, (void*)"LibreDWG::Dwg_Object_DATALINK", 0};
static swig_type_info _swigt__p__dwg_object_DATATABLE = {"_p__dwg_object_DATATABLE", "Dwg_Object_DATATABLE *|struct _dwg_object_DATATABLE *|_dwg_object_DATATABLE *", 0, 0, (void*)"LibreDWG::Dwg_Object_DATATABLE", 0};
static swig_type_info _swigt__p__dwg_object_DBCOLOR = {"_p__dwg_object_DBCOLOR", "Dwg_Object_DBCOLOR *|struct _dwg_object_DBCOLOR *|_dwg_object_DBCOLOR *", 0, 0, (void*)"LibreDWG::Dwg_Object_DBCOLOR", 0};
static swig_type_info _swigt__p__dwg_object_DETAILVIEWSTYLE = {"_p__dwg_object_DETAILVIEWSTYLE", "Dwg_Object_DETAILVIEWSTYLE *|struct _dwg_object_DETAILVIEWSTYLE *|_dwg_object_DETAILVIEWSTYLE *", 0, 0, (void*)"LibreDWG::Dwg_Object_DETAILVIEWSTYLE", 0};
static swig_type_info _swigt__p__dwg_object_DICTIONARY = {"_p__dwg_object_DICTIONARY", "Dwg_Object_DICTIONARY *|struct _dwg_object_DICTIONARY *|_dwg_object_DICTIONARY *", 0, 0, (void*)"LibreDWG::Dwg_Object_DICTIONARY", 0};
static swig_type_info _swigt__p__dwg_object_DICTIONARYVAR = {"_p__dwg_object_DICTIONARYVAR", "Dwg_Object_DICTIONARYVAR *|struct _dwg_object_DICTIONARYVAR *|_dwg_object_DICTIONARYVAR *", 0, 0, (void*)"LibreDWG::Dwg_Object_DICTIONARYVAR", 0};
static swig_type_info _swigt__p__dwg_object_DICTIONARYWDFLT = {"_p__dwg_object_DICTIONARYWDFLT", "Dwg_Object_DICTIONARYWDFLT *|struct _dwg_object_DICTIONARYWDFLT *|_dwg_object_DICTIONARYWDFLT *", 0, 0, (void*)"LibreDWG::Dwg_Object_DICTIONARYWDFLT", 0};
static swig_type_info _swigt__p__dwg_object_DIMASSOC = {"_p__dwg_object_DIMASSOC", "Dwg_Object_DIMASSOC *|struct _dwg_object_DIMASSOC *|_dwg_object_DIMASSOC *", 0, 0, (void*)"LibreDWG::Dwg_Object_DIMASSOC", 0};
static swig_type_info _swigt__p__dwg_object_DIMSTYLE = {"_p__dwg_object_DIMSTYLE", "Dwg_Object_DIMSTYLE *|struct _dwg_object_DIMSTYLE *|_dwg_object_DIMSTYLE *", 0, 0, (void*)"LibreDWG::Dwg_Object_DIMSTYLE", 0};
static swig_type_info _swigt__p__dwg_object_DIMSTYLE_CONTROL = {"_p__dwg_object_DIMSTYLE_CONTROL", "Dwg_Object_DIMSTYLE_CONTROL *|struct _dwg_object_DIMSTYLE_CONTROL *|_dwg_object_DIMSTYLE_CONTROL *", 0, 0, (void*)"LibreDWG::Dwg_Object_DIMSTYLE_CONTROL", 0};
static swig_type_info _swigt__p__dwg_object_DMDIMOBJECTCONTEXTDATA = {"_p__dwg_object_DMDIMOBJECTCONTEXTDATA", "Dwg_Object_DMDIMOBJECTCONTEXTDATA *|struct _dwg_object_DMDIMOBJECTCONTEXTDATA *|_dwg_object_DMDIMOBJECTCONTEXTDATA *", 0, 0, (void*)"LibreDWG::Dwg_Object_DMDIMOBJECTCONTEXTDATA", 0};
static swig_type_info _swigt__p__dwg_object_DUMMY = {"_p__dwg_object_DUMMY", "Dwg_Object_DUMMY *|struct _dwg_object_DUMMY *|_dwg_object_DUMMY *", 0, 0, (void*)"LibreDWG::Dwg_Object_DUMMY", 0};
static swig_type_info _swigt__p__dwg_object_DYNAMICBLOCKPROXYNODE = {"_p__dwg_object_DYNAMICBLOCKPROXYNODE", "Dwg_Object_DYNAMICBLOCKPROXYNODE *|struct _dwg_object_DYNAMICBLOCKPROXYNODE *|_dwg_object_DYNAMICBLOCKPROXYNODE *", 0, 0, (void*)"LibreDWG::Dwg_Object_DYNAMICBLOCKPROXYNODE", 0};
static swig_type_info _swigt__p__dwg_object_DYNAMICBLOCKPURGEPREVENTER = {"_p__dwg_object_DYNAMICBLOCKPURGEPREVENTER", "Dwg_Object_DYNAMICBLOCKPURGEPREVENTER *|struct _dwg_object_DYNAMICBLOCKPURGEPREVENTER *|_dwg_object_DYNAMICBLOCKPURGEPREVENTER *", 0, 0, (void*)"LibreDWG::Dwg_Object_DYNAMICBLOCKPURGEPREVENTER", 0};
static swig_type_info _swigt__p__dwg_object_EVALUATION_GRAPH = {"_p__dwg_object_EVALUATION_GRAPH", "Dwg_Object_EVALUATION_GRAPH *|struct _dwg_object_EVALUATION_GRAPH *|_dwg_object_EVALUATION_GRAPH *", 0, 0, (void*)"LibreDWG::Dwg_Object_EVALUATION_GRAPH", 0};
static swig_type_info _swigt__p__dwg_object_FCFOBJECTCONTEXTDATA = {"_p__dwg_object_FCFOBJECTCONTEXTDATA", "Dwg_Object_FCFOBJECTCONTEXTDATA *|struct _dwg_object_FCFOBJECTCONTEXTDATA *|_dwg_object_FCFOBJECTCONTEXTDATA *", 0, 0, (void*)"LibreDWG::Dwg_Object_FCFOBJECTCONTEXTDATA", 0};
static swig_type_info _swigt__p__dwg_object_FIELD = {"_p__dwg_object_FIELD", "Dwg_Object_FIELD *|struct _dwg_object_FIELD *|_dwg_object_FIELD *", 0, 0, (void*)"LibreDWG::Dwg_Object_FIELD", 0};
static swig_type_info _swigt__p__dwg_object_FIELDLIST = {"_p__dwg_object_FIELDLIST", "Dwg_Object_FIELDLIST *|struct _dwg_object_FIELDLIST *|_dwg_object_FIELDLIST *", 0, 0, (void*)"LibreDWG::Dwg_Object_FIELDLIST", 0};
static swig_type_info _swigt__p__dwg_object_GEODATA = {"_p__dwg_object_GEODATA", "Dwg_Object_GEODATA *|struct _dwg_object_GEODATA *|_dwg_object_GEODATA *", 0, 0, (void*)"LibreDWG::Dwg_Object_GEODATA", 0};
static swig_type_info _swigt__p__dwg_object_GEOMAPIMAGE = {"_p__dwg_object_GEOMAPIMAGE", "Dwg_Object_GEOMAPIMAGE *|struct _dwg_object_GEOMAPIMAGE *|_dwg_object_GEOMAPIMAGE *", 0, 0, (void*)"LibreDWG::Dwg_Object_GEOMAPIMAGE", 0};
static swig_type_info _swigt__p__dwg_object_GRADIENT_BACKGROUND = {"_p__dwg_object_GRADIENT_BACKGROUND", "Dwg_Object_GRADIENT_BACKGROUND *|struct _dwg_object_GRADIENT_BACKGROUND *|_dwg_object_GRADIENT_BACKGROUND *", 0, 0, (void*)"LibreDWG::Dwg_Object_GRADIENT_BACKGROUND", 0};
static swig_type_info _swigt__p__dwg_object_GROUND_PLANE_BACKGROUND = {"_p__dwg_object_GROUND_PLANE_BACKGROUND", "Dwg_Object_GROUND_PLANE_BACKGROUND *|struct _dwg_object_GROUND_PLANE_BACKGROUND *|_dwg_object_GROUND_PLANE_BACKGROUND *", 0, 0, (void*)"LibreDWG::Dwg_Object_GROUND_PLANE_BACKGROUND", 0};
static swig_type_info _swigt__p__dwg_object_GROUP = {"_p__dwg_object_GROUP", "Dwg_Object_GROUP *|struct _dwg_object_GROUP *|_dwg_object_GROUP *", 0, 0, (void*)"LibreDWG::Dwg_Object_GROUP", 0};
static swig_type_info _swigt__p__dwg_object_IBL_BACKGROUND = {"_p__dwg_object_IBL_BACKGROUND", "Dwg_Object_IBL_BACKGROUND *|struct _dwg_object_IBL_BACKGROUND *|_dwg_object_IBL_BACKGROUND *", 0, 0, (void*)"LibreDWG::Dwg_Object_IBL_BACKGROUND", 0};
static swig_type_info _swigt__p__dwg_object_IDBUFFER = {"_p__dwg_object_IDBUFFER", "Dwg_Object_IDBUFFER *|struct _dwg_object_IDBUFFER *|_dwg_object_IDBUFFER *", 0, 0, (void*)"LibreDWG::Dwg_Object_IDBUFFER", 0};
static swig_type_info _swigt__p__dwg_object_IMAGEDEF = {"_p__dwg_object_IMAGEDEF", "Dwg_Object_IMAGEDEF *|struct _dwg_object_IMAGEDEF *|_dwg_object_IMAGEDEF *", 0, 0, (void*)"LibreDWG::Dwg_Object_IMAGEDEF", 0};
static swig_type_info _swigt__p__dwg_object_IMAGEDEF_REACTOR = {"_p__dwg_object_IMAGEDEF_REACTOR", "Dwg_Object_IMAGEDEF_REACTOR *|struct _dwg_object_IMAGEDEF_REACTOR *|_dwg_object_IMAGEDEF_REACTOR *", 0, 0, (void*)"LibreDWG::Dwg_Object_IMAGEDEF_REACTOR", 0};
static swig_type_info _swigt__p__dwg_object_IMAGE_BACKGROUND = {"_p__dwg_object_IMAGE_BACKGROUND", "Dwg_Object_IMAGE_BACKGROUND *|struct _dwg_object_IMAGE_BACKGROUND *|_dwg_object_IMAGE_BACKGROUND *", 0, 0, (void*)"LibreDWG::Dwg_Object_IMAGE_BACKGROUND", 0};
static swig_type_info _swigt__p__dwg_object_INDEX = {"_p__dwg_object_INDEX", "Dwg_Object_INDEX *|struct _dwg_object_INDEX *|_dwg_object_INDEX *", 0, 0, (void*)"LibreDWG::Dwg_Object_INDEX", 0};
static swig_type_info _swigt__p__dwg_object_LAYER = {"_p__dwg_object_LAYER", "Dwg_Object_LAYER *|struct _dwg_object_LAYER *|_dwg_object_LAYER *", 0, 0, (void*)"LibreDWG::Dwg_Object_LAYER", 0};
static swig_type_info _swigt__p__dwg_object_LAYERFILTER = {"_p__dwg_object_LAYERFILTER", "Dwg_Object_LAYERFILTER *|struct _dwg_object_LAYERFILTER *|_dwg_object_LAYERFILTER *", 0, 0, (void*)"LibreDWG::Dwg_Object_LAYERFILTER", 0};
static swig_type_info _swigt__p__dwg_object_LAYER_CONTROL = {"_p__dwg_object_LAYER_CONTROL", "Dwg_Object_LAYER_CONTROL *|struct _dwg_object_LAYER_CONTROL *|_dwg_object_LAYER_CONTROL *", 0, 0, (void*)"LibreDWG::Dwg_Object_LAYER_CONTROL", 0};
static swig_type_info _swigt__p__dwg_object_LAYER_INDEX = {"_p__dwg_object_LAYER_INDEX", "Dwg_Object_LAYER_INDEX *|struct _dwg_object_LAYER_INDEX *|_dwg_object_LAYER_INDEX *", 0, 0, (void*)"LibreDWG::Dwg_Object_LAYER_INDEX", 0};
static swig_type_info _swigt__p__dwg_object_LAYOUT = {"_p__dwg_object_LAYOUT", "Dwg_Object_LAYOUT *|struct _dwg_object_LAYOUT *|_dwg_object_LAYOUT *", 0, 0, (void*)"LibreDWG::Dwg_Object_LAYOUT", 0};
static swig_type_info _swigt__p__dwg_object_LEADEROBJECTCONTEXTDATA = {"_p__dwg_object_LEADEROBJECTCONTEXTDATA", "Dwg_Object_LEADEROBJECTCONTEXTDATA *|struct _dwg_object_LEADEROBJECTCONTEXTDATA *|_dwg_object_LEADEROBJECTCONTEXTDATA *", 0, 0, (void*)"LibreDWG::Dwg_Object_LEADEROBJECTCONTEXTDATA", 0};
static swig_type_info _swigt__p__dwg_object_LIGHTLIST = {"_p__dwg_object_LIGHTLIST", "Dwg_Object_LIGHTLIST *|struct _dwg_object_LIGHTLIST *|_dwg_object_LIGHTLIST *", 0, 0, (void*)"LibreDWG::Dwg_Object_LIGHTLIST", 0};
static swig_type_info _swigt__p__dwg_object_LONG_TRANSACTION = {"_p__dwg_object_LONG_TRANSACTION", "Dwg_Object_LONG_TRANSACTION *|struct _dwg_object_LONG_TRANSACTION *|_dwg_object_LONG_TRANSACTION *", 0, 0, (void*)"LibreDWG::Dwg_Object_LONG_TRANSACTION", 0};
static swig_type_info _swigt__p__dwg_object_LTYPE = {"_p__dwg_object_LTYPE", "Dwg_Object_LTYPE *|struct _dwg_object_LTYPE *|_dwg_object_LTYPE *", 0, 0, (void*)"LibreDWG::Dwg_Object_LTYPE", 0};
static swig_type_info _swigt__p__dwg_object_LTYPE_CONTROL = {"_p__dwg_object_LTYPE_CONTROL", "Dwg_Object_LTYPE_CONTROL *|struct _dwg_object_LTYPE_CONTROL *|_dwg_object_LTYPE_CONTROL *", 0, 0, (void*)"LibreDWG::Dwg_Object_LTYPE_CONTROL", 0};
static swig_type_info _swigt__p__dwg_object_MATERIAL = {"_p__dwg_object_MATERIAL", "Dwg_Object_MATERIAL *|struct _dwg_object_MATERIAL *|_dwg_object_MATERIAL *", 0, 0, (void*)"LibreDWG::Dwg_Object_MATERIAL", 0};
static swig_type_info _swigt__p__dwg_object_MENTALRAYRENDERSETTINGS = {"_p__dwg_object_MENTALRAYRENDERSETTINGS", "Dwg_Object_MENTALRAYRENDERSETTINGS *|struct _dwg_object_MENTALRAYRENDERSETTINGS *|_dwg_object_MENTALRAYRENDERSETTINGS *", 0, 0, (void*)"LibreDWG::Dwg_Object_MENTALRAYRENDERSETTINGS", 0};
static swig_type_info _swigt__p__dwg_object_MLEADEROBJECTCONTEXTDATA = {"_p__dwg_object_MLEADEROBJECTCONTEXTDATA", "Dwg_Object_MLEADEROBJECTCONTEXTDATA *|struct _dwg_object_MLEADEROBJECTCONTEXTDATA *|_dwg_object_MLEADEROBJECTCONTEXTDATA *", 0, 0, (void*)"LibreDWG::Dwg_Object_MLEADEROBJECTCONTEXTDATA", 0};
static swig_type_info _swigt__p__dwg_object_MLEADERSTYLE = {"_p__dwg_object_MLEADERSTYLE", "Dwg_Object_MLEADERSTYLE *|struct _dwg_object_MLEADERSTYLE *|_dwg_object_MLEADERSTYLE *", 0, 0, (void*)"LibreDWG::Dwg_Object_MLEADERSTYLE", 0};
static swig_type_info _swigt__p__dwg_object_MLINESTYLE = {"_p__dwg_object_MLINESTYLE", "Dwg_Object_MLINESTYLE *|struct _dwg_object_MLINESTYLE *|_dwg_object_MLINESTYLE *", 0, 0, (void*)"LibreDWG::Dwg_Object_MLINESTYLE", 0};
static swig_type_info _swigt__p__dwg_object_MOTIONPATH = {"_p__dwg_object_MOTIONPATH", "Dwg_Object_MOTIONPATH *|struct _dwg_object_MOTIONPATH *|_dwg_object_MOTIONPATH *", 0, 0, (void*)"LibreDWG::Dwg_Object_MOTIONPATH", 0};
static swig_type_info _swigt__p__dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA = {"_p__dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA", "Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *|struct _dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *|_dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA *", 0, 0, (void*)"LibreDWG::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA", 0};
static swig_type_info _swigt__p__dwg_object_MTEXTOBJECTCONTEXTDATA = {"_p__dwg_object_MTEXTOBJECTCONTEXTDATA", "Dwg_Object_MTEXTOBJECTCONTEXTDATA *|struct _dwg_object_MTEXTOBJECTCONTEXTDATA *|_dwg_object_MTEXTOBJECTCONTEXTDATA *", 0, 0, (void*)"LibreDWG::Dwg_Object_MTEXTOBJECTCONTEXTDATA", 0};
static swig_type_info _swigt__p__dwg_object_NAVISWORKSMODELDEF = {"_p__dwg_object_NAVISWORKSMODELDEF", "Dwg_Object_NAVISWORKSMODELDEF *|struct _dwg_object_NAVISWORKSMODELDEF *|_dwg_object_NAVISWORKSMODELDEF *", 0, 0, (void*)"LibreDWG::Dwg_Object_NAVISWORKSMODELDEF", 0};
static swig_type_info _swigt__p__dwg_object_OBJECT_PTR = {"_p__dwg_object_OBJECT_PTR", "Dwg_Object_OBJECT_PTR *|struct _dwg_object_OBJECT_PTR *|_dwg_object_OBJECT_PTR *", 0, 0, (void*)"LibreDWG::Dwg_Object_OBJECT_PTR", 0};
static swig_type_info _swigt__p__dwg_object_ORDDIMOBJECTCONTEXTDATA = {"_p__dwg_object_ORDDIMOBJECTCONTEXTDATA", "Dwg_Object_ORDDIMOBJECTCONTEXTDATA *|struct _dwg_object_ORDDIMOBJECTCONTEXTDATA *|_dwg_object_ORDDIMOBJECTCONTEXTDATA *", 0, 0, (void*)"LibreDWG::Dwg_Object_ORDDIMOBJECTCONTEXTDATA", 0};
static swig_type_info _swigt__p__dwg_object_PARTIAL_VIEWING_INDEX = {"_p__dwg_object_PARTIAL_VIEWING_INDEX", "Dwg_Object_PARTIAL_VIEWING_INDEX *|struct _dwg_object_PARTIAL_VIEWING_INDEX *|_dwg_object_PARTIAL_VIEWING_INDEX *", 0, 0, (void*)"LibreDWG::Dwg_Object_PARTIAL_VIEWING_INDEX", 0};
static swig_type_info _swigt__p__dwg_object_PERSUBENTMGR = {"_p__dwg_object_PERSUBENTMGR", "Dwg_Object_PERSUBENTMGR *|struct _dwg_object_PERSUBENTMGR *|_dwg_object_PERSUBENTMGR *", 0, 0, (void*)"LibreDWG::Dwg_Object_PERSUBENTMGR", 0};
static swig_type_info _swigt__p__dwg_object_PLACEHOLDER = {"_p__dwg_object_PLACEHOLDER", "Dwg_Object_PLACEHOLDER *|struct _dwg_object_PLACEHOLDER *|_dwg_object_PLACEHOLDER *", 0, 0, (void*)"LibreDWG::Dwg_Object_PLACEHOLDER", 0};
static swig_type_info _swigt__p__dwg_object_PLOTSETTINGS = {"_p__dwg_object_PLOTSETTINGS", "Dwg_Object_PLOTSETTINGS *|struct _dwg_object_PLOTSETTINGS *|_dwg_object_PLOTSETTINGS *", 0, 0, (void*)"LibreDWG::Dwg_Object_PLOTSETTINGS", 0};
static swig_type_info _swigt__p__dwg_object_POINTCLOUDCOLORMAP = {"_p__dwg_object_POINTCLOUDCOLORMAP", "Dwg_Object_POINTCLOUDCOLORMAP *|struct _dwg_object_POINTCLOUDCOLORMAP *|_dwg_object_POINTCLOUDCOLORMAP *", 0, 0, (void*)"LibreDWG::Dwg_Object_POINTCLOUDCOLORMAP", 0};
static swig_type_info _swigt__p__dwg_object_POINTCLOUDDEF = {"_p__dwg_object_POINTCLOUDDEF", "Dwg_Object_POINTCLOUDDEF *|struct _dwg_object_POINTCLOUDDEF *|_dwg_object_POINTCLOUDDEF *", 0, 0, (void*)"LibreDWG::Dwg_Object_POINTCLOUDDEF", 0};
static swig_type_info _swigt__p__dwg_object_POINTCLOUDDEFEX = {"_p__dwg_object_POINTCLOUDDEFEX", "Dwg_Object_POINTCLOUDDEFEX *|struct _dwg_object_POINTCLOUDDEFEX *|_dwg_object_POINTCLOUDDEFEX *", 0, 0, (void*)"LibreDWG::Dwg_Object_POINTCLOUDDEFEX", 0};
static swig_type_info _swigt__p__dwg_object_POINTCLOUDDEF_REACTOR = {"_p__dwg_object_POINTCLOUDDEF_REACTOR", "Dwg_Object_POINTCLOUDDEF_REACTOR *|struct _dwg_object_POINTCLOUDDEF_REACTOR *|_dwg_object_POINTCLOUDDEF_REACTOR *", 0, 0, (void*)"LibreDWG::Dwg_Object_POINTCLOUDDEF_REACTOR", 0};
static swig_type_info _swigt__p__dwg_object_POINTCLOUDDEF_REACTOR_EX = {"_p__dwg_object_POINTCLOUDDEF_REACTOR_EX", "Dwg_Object_POINTCLOUDDEF_REACTOR_EX *|struct _dwg_object_POINTCLOUDDEF_REACTOR_EX *|_dwg_object_POINTCLOUDDEF_REACTOR_EX *", 0, 0, (void*)"LibreDWG::Dwg_Object_POINTCLOUDDEF_REACTOR_EX", 0};
static swig_type_info _swigt__p__dwg_object_POINTPATH = {"_p__dwg_object_POINTPATH", "Dwg_Object_POINTPATH *|struct _dwg_object_POINTPATH *|_dwg_object_POINTPATH *", 0, 0, (void*)"LibreDWG::Dwg_Object_POINTPATH", 0};
static swig_type_info _swigt__p__dwg_object_PROXY_OBJECT = {"_p__dwg_object_PROXY_OBJECT", "Dwg_Object_PROXY_OBJECT *|struct _dwg_object_PROXY_OBJECT *|_dwg_object_PROXY_OBJECT *", 0, 0, (void*)"LibreDWG::Dwg_Object_PROXY_OBJECT", 0};
static swig_type_info _swigt__p__dwg_object_RADIMLGOBJECTCONTEXTDATA = {"_p__dwg_object_RADIMLGOBJECTCONTEXTDATA", "Dwg_Object_RADIMLGOBJECTCONTEXTDATA *|struct _dwg_object_RADIMLGOBJECTCONTEXTDATA *|_dwg_object_RADIMLGOBJECTCONTEXTDATA *", 0, 0, (void*)"LibreDWG::Dwg_Object_RADIMLGOBJECTCONTEXTDATA", 0};
static swig_type_info _swigt__p__dwg_object_RADIMOBJECTCONTEXTDATA = {"_p__dwg_object_RADIMOBJECTCONTEXTDATA", "Dwg_Object_RADIMOBJECTCONTEXTDATA *|struct _dwg_object_RADIMOBJECTCONTEXTDATA *|_dwg_object_RADIMOBJECTCONTEXTDATA *", 0, 0, (void*)"LibreDWG::Dwg_Object_RADIMOBJECTCONTEXTDATA", 0};
static swig_type_info _swigt__p__dwg_object_RAPIDRTRENDERSETTINGS = {"_p__dwg_object_RAPIDRTRENDERSETTINGS", "Dwg_Object_RAPIDRTRENDERSETTINGS *|struct _dwg_object_RAPIDRTRENDERSETTINGS *|_dwg_object_RAPIDRTRENDERSETTINGS *", 0, 0, (void*)"LibreDWG::Dwg_Object_RAPIDRTRENDERSETTINGS", 0};
static swig_type_info _swigt__p__dwg_object_RASTERVARIABLES = {"_p__dwg_object_RASTERVARIABLES", "Dwg_Object_RASTERVARIABLES *|struct _dwg_object_RASTERVARIABLES *|_dwg_object_RASTERVARIABLES *", 0, 0, (void*)"LibreDWG::Dwg_Object_RASTERVARIABLES", 0};
static swig_type_info _swigt__p__dwg_object_RENDERENTRY = {"_p__dwg_object_RENDERENTRY", "Dwg_Object_RENDERENTRY *|struct _dwg_object_RENDERENTRY *|_dwg_object_RENDERENTRY *", 0, 0, (void*)"LibreDWG::Dwg_Object_RENDERENTRY", 0};
static swig_type_info _swigt__p__dwg_object_RENDERENVIRONMENT = {"_p__dwg_object_RENDERENVIRONMENT", "Dwg_Object_RENDERENVIRONMENT *|struct _dwg_object_RENDERENVIRONMENT *|_dwg_object_RENDERENVIRONMENT *", 0, 0, (void*)"LibreDWG::Dwg_Object_RENDERENVIRONMENT", 0};
static swig_type_info _swigt__p__dwg_object_RENDERGLOBAL = {"_p__dwg_object_RENDERGLOBAL", "Dwg_Object_RENDERGLOBAL *|struct _dwg_object_RENDERGLOBAL *|_dwg_object_RENDERGLOBAL *", 0, 0, (void*)"LibreDWG::Dwg_Object_RENDERGLOBAL", 0};
static swig_type_info _swigt__p__dwg_object_RENDERSETTINGS = {"_p__dwg_object_RENDERSETTINGS", "Dwg_Object_RENDERSETTINGS *|struct _dwg_object_RENDERSETTINGS *|_dwg_object_RENDERSETTINGS *", 0, 0, (void*)"LibreDWG::Dwg_Object_RENDERSETTINGS", 0};
static swig_type_info _swigt__p__dwg_object_SCALE = {"_p__dwg_object_SCALE", "Dwg_Object_SCALE *|struct _dwg_object_SCALE *|_dwg_object_SCALE *", 0, 0, (void*)"LibreDWG::Dwg_Object_SCALE", 0};
static swig_type_info _swigt__p__dwg_object_SECTIONVIEWSTYLE = {"_p__dwg_object_SECTIONVIEWSTYLE", "Dwg_Object_SECTIONVIEWSTYLE *|struct _dwg_object_SECTIONVIEWSTYLE *|_dwg_object_SECTIONVIEWSTYLE *", 0, 0, (void*)"LibreDWG::Dwg_Object_SECTIONVIEWSTYLE", 0};
static swig_type_info _swigt__p__dwg_object_SECTION_MANAGER = {"_p__dwg_object_SECTION_MANAGER", "Dwg_Object_SECTION_MANAGER *|struct _dwg_object_SECTION_MANAGER *|_dwg_object_SECTION_MANAGER *", 0, 0, (void*)"LibreDWG::Dwg_Object_SECTION_MANAGER", 0};
static swig_type_info _swigt__p__dwg_object_SECTION_SETTINGS = {"_p__dwg_object_SECTION_SETTINGS", "Dwg_Object_SECTION_SETTINGS *|struct _dwg_object_SECTION_SETTINGS *|_dwg_object_SECTION_SETTINGS *", 0, 0, (void*)"LibreDWG::Dwg_Object_SECTION_SETTINGS", 0};
static swig_type_info _swigt__p__dwg_object_SKYLIGHT_BACKGROUND = {"_p__dwg_object_SKYLIGHT_BACKGROUND", "Dwg_Object_SKYLIGHT_BACKGROUND *|struct _dwg_object_SKYLIGHT_BACKGROUND *|_dwg_object_SKYLIGHT_BACKGROUND *", 0, 0, (void*)"LibreDWG::Dwg_Object_SKYLIGHT_BACKGROUND", 0};
static swig_type_info _swigt__p__dwg_object_SOLID_BACKGROUND = {"_p__dwg_object_SOLID_BACKGROUND", "Dwg_Object_SOLID_BACKGROUND *|struct _dwg_object_SOLID_BACKGROUND *|_dwg_object_SOLID_BACKGROUND *", 0, 0, (void*)"LibreDWG::Dwg_Object_SOLID_BACKGROUND", 0};
static swig_type_info _swigt__p__dwg_object_SORTENTSTABLE = {"_p__dwg_object_SORTENTSTABLE", "Dwg_Object_SORTENTSTABLE *|struct _dwg_object_SORTENTSTABLE *|_dwg_object_SORTENTSTABLE *", 0, 0, (void*)"LibreDWG::Dwg_Object_SORTENTSTABLE", 0};
static swig_type_info _swigt__p__dwg_object_SPATIAL_FILTER = {"_p__dwg_object_SPATIAL_FILTER", "Dwg_Object_SPATIAL_FILTER *|struct _dwg_object_SPATIAL_FILTER *|_dwg_object_SPATIAL_FILTER *", 0, 0, (void*)"LibreDWG::Dwg_Object_SPATIAL_FILTER", 0};
static swig_type_info _swigt__p__dwg_object_SPATIAL_INDEX = {"_p__dwg_object_SPATIAL_INDEX", "Dwg_Object_SPATIAL_INDEX *|struct _dwg_object_SPATIAL_INDEX *|_dwg_object_SPATIAL_INDEX *", 0, 0, (void*)"LibreDWG::Dwg_Object_SPATIAL_INDEX", 0};
static swig_type_info _swigt__p__dwg_object_STYLE = {"_p__dwg_object_STYLE", "Dwg_Object_STYLE *|struct _dwg_object_STYLE *|_dwg_object_STYLE *", 0, 0, (void*)"LibreDWG::Dwg_Object_STYLE", 0};
static swig_type_info _swigt__p__dwg_object_STYLE_CONTROL = {"_p__dwg_object_STYLE_CONTROL", "Dwg_Object_STYLE_CONTROL *|struct _dwg_object_STYLE_CONTROL *|_dwg_object_STYLE_CONTROL *", 0, 0, (void*)"LibreDWG::Dwg_Object_STYLE_CONTROL", 0};
static swig_type_info _swigt__p__dwg_object_SUN = {"_p__dwg_object_SUN", "Dwg_Object_SUN *|struct _dwg_object_SUN *|_dwg_object_SUN *", 0, 0, (void*)"LibreDWG::Dwg_Object_SUN", 0};
static swig_type_info _swigt__p__dwg_object_SUNSTUDY = {"_p__dwg_object_SUNSTUDY", "Dwg_Object_SUNSTUDY *|struct _dwg_object_SUNSTUDY *|_dwg_object_SUNSTUDY *", 0, 0, (void*)"LibreDWG::Dwg_Object_SUNSTUDY", 0};
static swig_type_info _swigt__p__dwg_object_TABLECONTENT = {"_p__dwg_object_TABLECONTENT", "Dwg_Object_TABLECONTENT *|struct _dwg_object_TABLECONTENT *|_dwg_object_TABLECONTENT *", 0, 0, (void*)"LibreDWG::Dwg_Object_TABLECONTENT", 0};
static swig_type_info _swigt__p__dwg_object_TABLEGEOMETRY = {"_p__dwg_object_TABLEGEOMETRY", "Dwg_Object_TABLEGEOMETRY *|struct _dwg_object_TABLEGEOMETRY *|_dwg_object_TABLEGEOMETRY *", 0, 0, (void*)"LibreDWG::Dwg_Object_TABLEGEOMETRY", 0};
static swig_type_info _swigt__p__dwg_object_TABLESTYLE = {"_p__dwg_object_TABLESTYLE", "Dwg_Object_TABLESTYLE *|struct _dwg_object_TABLESTYLE *|_dwg_object_TABLESTYLE *", 0, 0, (void*)"LibreDWG::Dwg_Object_TABLESTYLE", 0};
static swig_type_info _swigt__p__dwg_object_TEXTOBJECTCONTEXTDATA = {"_p__dwg_object_TEXTOBJECTCONTEXTDATA", "Dwg_Object_TEXTOBJECTCONTEXTDATA *|struct _dwg_object_TEXTOBJECTCONTEXTDATA *|_dwg_object_TEXTOBJECTCONTEXTDATA *", 0, 0, (void*)"LibreDWG::Dwg_Object_TEXTOBJECTCONTEXTDATA", 0};
static swig_type_info _swigt__p__dwg_object_TVDEVICEPROPERTIES = {"_p__dwg_object_TVDEVICEPROPERTIES", "Dwg_Object_TVDEVICEPROPERTIES *|struct _dwg_object_TVDEVICEPROPERTIES *|_dwg_object_TVDEVICEPROPERTIES *", 0, 0, (void*)"LibreDWG::Dwg_Object_TVDEVICEPROPERTIES", 0};
static swig_type_info _swigt__p__dwg_object_UCS = {"_p__dwg_object_UCS", "Dwg_Object_UCS *|struct _dwg_object_UCS *|_dwg_object_UCS *", 0, 0, (void*)"LibreDWG::Dwg_Object_UCS", 0};
static swig_type_info _swigt__p__dwg_object_UCS_CONTROL = {"_p__dwg_object_UCS_CONTROL", "Dwg_Object_UCS_CONTROL *|struct _dwg_object_UCS_CONTROL *|_dwg_object_UCS_CONTROL *", 0, 0, (void*)"LibreDWG::Dwg_Object_UCS_CONTROL", 0};
static swig_type_info _swigt__p__dwg_object_UNKNOWN_OBJ = {"_p__dwg_object_UNKNOWN_OBJ", "Dwg_Object_UNKNOWN_OBJ *|struct _dwg_object_UNKNOWN_OBJ *|_dwg_object_UNKNOWN_OBJ *", 0, 0, (void*)"LibreDWG::Dwg_Object_UNKNOWN_OBJ", 0};
static swig_type_info _swigt__p__dwg_object_VBA_PROJECT = {"_p__dwg_object_VBA_PROJECT", "Dwg_Object_VBA_PROJECT *|struct _dwg_object_VBA_PROJECT *|_dwg_object_VBA_PROJECT *", 0, 0, (void*)"LibreDWG::Dwg_Object_VBA_PROJECT", 0};
static swig_type_info _swigt__p__dwg_object_VIEW = {"_p__dwg_object_VIEW", "Dwg_Object_VIEW *|struct _dwg_object_VIEW *|_dwg_object_VIEW *", 0, 0, (void*)"LibreDWG::Dwg_Object_VIEW", 0};
static swig_type_info _swigt__p__dwg_object_VIEW_CONTROL = {"_p__dwg_object_VIEW_CONTROL", "Dwg_Object_VIEW_CONTROL *|struct _dwg_object_VIEW_CONTROL *|_dwg_object_VIEW_CONTROL *", 0, 0, (void*)"LibreDWG::Dwg_Object_VIEW_CONTROL", 0};
static swig_type_info _swigt__p__dwg_object_VISUALSTYLE = {"_p__dwg_object_VISUALSTYLE", "Dwg_Object_VISUALSTYLE *|struct _dwg_object_VISUALSTYLE *|_dwg_object_VISUALSTYLE *", 0, 0, (void*)"LibreDWG::Dwg_Object_VISUALSTYLE", 0};
static swig_type_info _swigt__p__dwg_object_VPORT = {"_p__dwg_object_VPORT", "Dwg_Object_VPORT *|struct _dwg_object_VPORT *|_dwg_object_VPORT *", 0, 0, (void*)"LibreDWG::Dwg_Object_VPORT", 0};
static swig_type_info _swigt__p__dwg_object_VPORT_CONTROL = {"_p__dwg_object_VPORT_CONTROL", "Dwg_Object_VPORT_CONTROL *|struct _dwg_object_VPORT_CONTROL *|_dwg_object_VPORT_CONTROL *", 0, 0, (void*)"LibreDWG::Dwg_Object_VPORT_CONTROL", 0};
static swig_type_info _swigt__p__dwg_object_VX_CONTROL = {"_p__dwg_object_VX_CONTROL", "Dwg_Object_VX_CONTROL *|struct _dwg_object_VX_CONTROL *|_dwg_object_VX_CONTROL *", 0, 0, (void*)"LibreDWG::Dwg_Object_VX_CONTROL", 0};
static swig_type_info _swigt__p__dwg_object_VX_TABLE_RECORD = {"_p__dwg_object_VX_TABLE_RECORD", "Dwg_Object_VX_TABLE_RECORD *|struct _dwg_object_VX_TABLE_RECORD *|_dwg_object_VX_TABLE_RECORD *", 0, 0, (void*)"LibreDWG::Dwg_Object_VX_TABLE_RECORD", 0};
static swig_type_info _swigt__p__dwg_object_WIPEOUTVARIABLES = {"_p__dwg_object_WIPEOUTVARIABLES", "Dwg_Object_WIPEOUTVARIABLES *|struct _dwg_object_WIPEOUTVARIABLES *|_dwg_object_WIPEOUTVARIABLES *", 0, 0, (void*)"LibreDWG::Dwg_Object_WIPEOUTVARIABLES", 0};
static swig_type_info _swigt__p__dwg_object_XRECORD = {"_p__dwg_object_XRECORD", "Dwg_Object_XRECORD *|struct _dwg_object_XRECORD *|_dwg_object_XRECORD *", 0, 0, (void*)"LibreDWG::Dwg_Object_XRECORD", 0};
static swig_type_info _swigt__p__dwg_object_entity = {"_p__dwg_object_entity", "Dwg_Object_Entity *|struct _dwg_object_entity *|_dwg_object_entity *", 0, 0, (void*)"LibreDWG::Dwg_Object_Entity", 0};
static swig_type_info _swigt__p__dwg_object_entity_tio = {"_p__dwg_object_entity_tio", "_dwg_object_entity_tio *", 0, 0, (void*)"LibreDWG::_dwg_object_entity_tio", 0};
static swig_type_info _swigt__p__dwg_object_object = {"_p__dwg_object_object", "Dwg_Object_Object *|struct _dwg_object_object *|_dwg_object_object *", 0, 0, (void*)"LibreDWG::Dwg_Object_Object", 0};
static swig_type_info _swigt__p__dwg_object_object_tio = {"_p__dwg_object_object_tio", "_dwg_object_object_tio *", 0, 0, (void*)"LibreDWG::_dwg_object_object_tio", 0};
static swig_type_info _swigt__p__dwg_object_ref = {"_p__dwg_object_ref", "BITCODE_H|Dwg_Object_Ref *|struct _dwg_object_ref *|_dwg_object_ref *", 0, 0, (void*)"LibreDWG::Dwg_Object_Ref", 0};
static swig_type_info _swigt__p__dwg_object_tio = {"_p__dwg_object_tio", "_dwg_object_tio *", 0, 0, (void*)"LibreDWG::_dwg_object_tio", 0};
static swig_type_info _swigt__p__dwg_objfreespace = {"_p__dwg_objfreespace", "Dwg_ObjFreeSpace *|struct _dwg_objfreespace *|_dwg_objfreespace *", 0, 0, (void*)"LibreDWG::Dwg_ObjFreeSpace", 0};
static swig_type_info _swigt__p__dwg_resbuf = {"_p__dwg_resbuf", "Dwg_Resbuf *|struct _dwg_resbuf *|_dwg_resbuf *", 0, 0, (void*)"LibreDWG::Dwg_Resbuf", 0};
static swig_type_info _swigt__p__dwg_resbuf_value = {"_p__dwg_resbuf_value", "_dwg_resbuf_value *", 0, 0, (void*)"LibreDWG::_dwg_resbuf_value", 0};
static swig_type_info _swigt__p__dwg_revhistory = {"_p__dwg_revhistory", "Dwg_RevHistory *|struct _dwg_revhistory *|_dwg_revhistory *", 0, 0, (void*)"LibreDWG::Dwg_RevHistory", 0};
static swig_type_info _swigt__p__dwg_secondheader = {"_p__dwg_secondheader", "Dwg_SecondHeader *|struct _dwg_secondheader *|_dwg_secondheader *", 0, 0, (void*)"LibreDWG::Dwg_SecondHeader", 0};
static swig_type_info _swigt__p__dwg_secondheader_handles = {"_p__dwg_secondheader_handles", "Dwg_SecondHeader_Handles *|struct _dwg_secondheader_handles *|_dwg_secondheader_handles *", 0, 0, (void*)"LibreDWG::Dwg_SecondHeader_Handles", 0};
static swig_type_info _swigt__p__dwg_secondheader_sections = {"_p__dwg_secondheader_sections", "Dwg_SecondHeader_Sections *|struct _dwg_secondheader_sections *|_dwg_secondheader_sections *", 0, 0, (void*)"LibreDWG::Dwg_SecondHeader_Sections", 0};
static swig_type_info _swigt__p__dwg_section = {"_p__dwg_section", "Dwg_Section *|struct _dwg_section *|_dwg_section *", 0, 0, (void*)"LibreDWG::Dwg_Section", 0};
static swig_type_info _swigt__p__dwg_security = {"_p__dwg_security", "Dwg_Security *|struct _dwg_security *|_dwg_security *", 0, 0, (void*)"LibreDWG::Dwg_Security", 0};
static swig_type_info _swigt__p__dwg_struct = {"_p__dwg_struct", "Dwg_Data *|struct _dwg_struct *|_dwg_struct *", 0, 0, (void*)"LibreDWG::Dwg_Data", 0};
static swig_type_info _swigt__p__dwg_struct_fhdr = {"_p__dwg_struct_fhdr", "_dwg_struct_fhdr *", 0, 0, (void*)"LibreDWG::_dwg_struct_fhdr", 0};
static swig_type_info _swigt__p__dwg_summaryinfo = {"_p__dwg_summaryinfo", "Dwg_SummaryInfo *|struct _dwg_summaryinfo *|_dwg_summaryinfo *", 0, 0, (void*)"LibreDWG::Dwg_SummaryInfo", 0};
static swig_type_info _swigt__p__dwg_template = {"_p__dwg_template", "Dwg_Template *|struct _dwg_template *|_dwg_template *", 0, 0, (void*)"LibreDWG::Dwg_Template", 0};
static swig_type_info _swigt__p__dwg_time_bll = {"_p__dwg_time_bll", "BITCODE_TIMEBLL *|BITCODE_TIMERLL *|Dwg_Bitcode_TimeBLL *|struct _dwg_time_bll *|_dwg_time_bll *", 0, 0, (void*)"LibreDWG::Dwg_Bitcode_TimeBLL", 0};
static swig_type_info _swigt__p__dwg_vbaproject = {"_p__dwg_vbaproject", "Dwg_VBAProject *|struct _dwg_vbaproject *|_dwg_vbaproject *", 0, 0, (void*)"LibreDWG::Dwg_VBAProject", 0};
static swig_type_info _swigt__p__inthash = {"_p__inthash", "struct _inthash *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_char = {"_p_char", "char *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_double = {"_p_double", "BITCODE_BD *|BITCODE_BT *|BITCODE_DD *|BITCODE_RD *|double *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_dwg_entity_DIMENSION_common = {"_p_dwg_entity_DIMENSION_common", "struct dwg_entity_DIMENSION_common *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_dwg_field_name_type_offset = {"_p_dwg_field_name_type_offset", "Dwg_DYNAPI_field *|struct dwg_field_name_type_offset *|dwg_field_name_type_offset *", 0, 0, (void*)"LibreDWG::Dwg_DYNAPI_field", 0};
static swig_type_info _swigt__p_dwg_obj_ent = {"_p_dwg_obj_ent", "dwg_obj_ent *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_dwg_obj_generic = {"_p_dwg_obj_generic", "dwg_obj_generic *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_dwg_obj_layer = {"_p_dwg_obj_layer", "dwg_obj_layer *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_dwg_obj_obj = {"_p_dwg_obj_obj", "dwg_obj_obj *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_dwg_object = {"_p_dwg_object", "dwg_object *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_dwg_versions = {"_p_dwg_versions", "Dwg_Versions *|struct dwg_versions *|dwg_versions *", 0, 0, (void*)"LibreDWG::Dwg_Versions", 0};
static swig_type_info _swigt__p_eed_data_t = {"_p_eed_data_t", "union eed_data_t *|eed_data_t *", 0, 0, (void*)"LibreDWG::eed_data_t", 0};
static swig_type_info _swigt__p_eed_data_t_eed_0 = {"_p_eed_data_t_eed_0", "eed_data_t_eed_0 *", 0, 0, (void*)"LibreDWG::eed_data_t_eed_0", 0};
static swig_type_info _swigt__p_eed_data_t_eed_0_r2007 = {"_p_eed_data_t_eed_0_r2007", "eed_data_t_eed_0_r2007 *", 0, 0, (void*)"LibreDWG::eed_data_t_eed_0_r2007", 0};
static swig_type_info _swigt__p_eed_data_t_eed_1 = {"_p_eed_data_t_eed_1", "eed_data_t_eed_1 *", 0, 0, (void*)"LibreDWG::eed_data_t_eed_1", 0};
static swig_type_info _swigt__p_eed_data_t_eed_10 = {"_p_eed_data_t_eed_10", "eed_data_t_eed_10 *", 0, 0, (void*)"LibreDWG::eed_data_t_eed_10", 0};
static swig_type_info _swigt__p_eed_data_t_eed_2 = {"_p_eed_data_t_eed_2", "eed_data_t_eed_2 *", 0, 0, (void*)"LibreDWG::eed_data_t_eed_2", 0};
static swig_type_info _swigt__p_eed_data_t_eed_3 = {"_p_eed_data_t_eed_3", "eed_data_t_eed_3 *", 0, 0, (void*)"LibreDWG::eed_data_t_eed_3", 0};
static swig_type_info _swigt__p_eed_data_t_eed_4 = {"_p_eed_data_t_eed_4", "eed_data_t_eed_4 *", 0, 0, (void*)"LibreDWG::eed_data_t_eed_4", 0};
static swig_type_info _swigt__p_eed_data_t_eed_40 = {"_p_eed_data_t_eed_40", "eed_data_t_eed_40 *", 0, 0, (void*)"LibreDWG::eed_data_t_eed_40", 0};
static swig_type_info _swigt__p_eed_data_t_eed_5 = {"_p_eed_data_t_eed_5", "eed_data_t_eed_5 *", 0, 0, (void*)"LibreDWG::eed_data_t_eed_5", 0};
static swig_type_info _swigt__p_eed_data_t_eed_70 = {"_p_eed_data_t_eed_70", "eed_data_t_eed_70 *", 0, 0, (void*)"LibreDWG::eed_data_t_eed_70", 0};
static swig_type_info _swigt__p_eed_data_t_eed_71 = {"_p_eed_data_t_eed_71", "eed_data_t_eed_71 *", 0, 0, (void*)"LibreDWG::eed_data_t_eed_71", 0};
static swig_type_info _swigt__p_int = {"_p_int", "BITCODE_BLd *|BITCODE_MC *|BITCODE_RLd *|Dwg_Entity_UNUSED *|int32_t *|int_fast16_t *|int_fast32_t *|int_least32_t *|intptr_t *|int *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_long_long = {"_p_long_long", "BITCODE_RLLd *|int64_t *|int_fast64_t *|int_least64_t *|intmax_t *|long long *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_abstractentity_UNDERLAY = {"_p_p__dwg_abstractentity_UNDERLAY", "Dwg_Entity_DGNUNDERLAY **|Dwg_Entity_DWFUNDERLAY **|Dwg_Entity_PDFUNDERLAY **|struct _dwg_abstractentity_UNDERLAY **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_abstractobject_ASSOCARRAYPARAMETERS = {"_p_p__dwg_abstractobject_ASSOCARRAYPARAMETERS", "Dwg_Object_ASSOCARRAYMODIFYPARAMETERS **|Dwg_Object_ASSOCARRAYPATHPARAMETERS **|Dwg_Object_ASSOCARRAYPOLARPARAMETERS **|Dwg_Object_ASSOCARRAYRECTANGULARPARAMETERS **|struct _dwg_abstractobject_ASSOCARRAYPARAMETERS **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_abstractobject_UNDERLAYDEFINITION = {"_p_p__dwg_abstractobject_UNDERLAYDEFINITION", "Dwg_Object_DGNDEFINITION **|Dwg_Object_DWFDEFINITION **|Dwg_Object_PDFDEFINITION **|struct _dwg_abstractobject_UNDERLAYDEFINITION **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_3DFACE = {"_p_p__dwg_entity_3DFACE", "Dwg_Entity__3DFACE **|struct _dwg_entity_3DFACE **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_3DLINE = {"_p_p__dwg_entity_3DLINE", "Dwg_Entity__3DLINE **|struct _dwg_entity_3DLINE **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_3DSOLID = {"_p_p__dwg_entity_3DSOLID", "Dwg_Entity_BODY **|Dwg_Entity_REGION **|Dwg_Entity__3DSOLID **|struct _dwg_entity_3DSOLID **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_ARC = {"_p_p__dwg_entity_ARC", "Dwg_Entity_ARC **|struct _dwg_entity_ARC **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_ARC_DIMENSION = {"_p_p__dwg_entity_ARC_DIMENSION", "Dwg_Entity_ARC_DIMENSION **|struct _dwg_entity_ARC_DIMENSION **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_ATTDEF = {"_p_p__dwg_entity_ATTDEF", "Dwg_Entity_ATTDEF **|struct _dwg_entity_ATTDEF **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_ATTRIB = {"_p_p__dwg_entity_ATTRIB", "Dwg_Entity_ATTRIB **|struct _dwg_entity_ATTRIB **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_BLOCK = {"_p_p__dwg_entity_BLOCK", "Dwg_Entity_BLOCK **|struct _dwg_entity_BLOCK **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_CAMERA = {"_p_p__dwg_entity_CAMERA", "Dwg_Entity_CAMERA **|struct _dwg_entity_CAMERA **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_CIRCLE = {"_p_p__dwg_entity_CIRCLE", "Dwg_Entity_CIRCLE **|struct _dwg_entity_CIRCLE **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_DIMENSION_ALIGNED = {"_p_p__dwg_entity_DIMENSION_ALIGNED", "Dwg_Entity_DIMENSION_ALIGNED **|struct _dwg_entity_DIMENSION_ALIGNED **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_DIMENSION_ANG2LN = {"_p_p__dwg_entity_DIMENSION_ANG2LN", "Dwg_Entity_DIMENSION_ANG2LN **|struct _dwg_entity_DIMENSION_ANG2LN **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_DIMENSION_ANG3PT = {"_p_p__dwg_entity_DIMENSION_ANG3PT", "Dwg_Entity_DIMENSION_ANG3PT **|struct _dwg_entity_DIMENSION_ANG3PT **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_DIMENSION_DIAMETER = {"_p_p__dwg_entity_DIMENSION_DIAMETER", "Dwg_Entity_DIMENSION_DIAMETER **|struct _dwg_entity_DIMENSION_DIAMETER **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_DIMENSION_LINEAR = {"_p_p__dwg_entity_DIMENSION_LINEAR", "Dwg_Entity_DIMENSION_LINEAR **|struct _dwg_entity_DIMENSION_LINEAR **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_DIMENSION_ORDINATE = {"_p_p__dwg_entity_DIMENSION_ORDINATE", "Dwg_Entity_DIMENSION_ORDINATE **|struct _dwg_entity_DIMENSION_ORDINATE **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_DIMENSION_RADIUS = {"_p_p__dwg_entity_DIMENSION_RADIUS", "Dwg_Entity_DIMENSION_RADIUS **|struct _dwg_entity_DIMENSION_RADIUS **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_ELLIPSE = {"_p_p__dwg_entity_ELLIPSE", "Dwg_Entity_ELLIPSE **|struct _dwg_entity_ELLIPSE **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_ENDBLK = {"_p_p__dwg_entity_ENDBLK", "Dwg_Entity_ENDBLK **|struct _dwg_entity_ENDBLK **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_ENDREP = {"_p_p__dwg_entity_ENDREP", "Dwg_Entity_ENDREP **|struct _dwg_entity_ENDREP **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_HATCH = {"_p_p__dwg_entity_HATCH", "Dwg_Entity_HATCH **|struct _dwg_entity_HATCH **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_HELIX = {"_p_p__dwg_entity_HELIX", "Dwg_Entity_HELIX **|struct _dwg_entity_HELIX **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_IMAGE = {"_p_p__dwg_entity_IMAGE", "Dwg_Entity_IMAGE **|struct _dwg_entity_IMAGE **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_INSERT = {"_p_p__dwg_entity_INSERT", "Dwg_Entity_INSERT **|struct _dwg_entity_INSERT **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_JUMP = {"_p_p__dwg_entity_JUMP", "Dwg_Entity_JUMP **|struct _dwg_entity_JUMP **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_LARGE_RADIAL_DIMENSION = {"_p_p__dwg_entity_LARGE_RADIAL_DIMENSION", "Dwg_Entity_LARGE_RADIAL_DIMENSION **|struct _dwg_entity_LARGE_RADIAL_DIMENSION **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_LAYOUTPRINTCONFIG = {"_p_p__dwg_entity_LAYOUTPRINTCONFIG", "Dwg_Entity_LAYOUTPRINTCONFIG **|struct _dwg_entity_LAYOUTPRINTCONFIG **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_LEADER = {"_p_p__dwg_entity_LEADER", "Dwg_Entity_LEADER **|struct _dwg_entity_LEADER **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_LIGHT = {"_p_p__dwg_entity_LIGHT", "Dwg_Entity_LIGHT **|struct _dwg_entity_LIGHT **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_LINE = {"_p_p__dwg_entity_LINE", "Dwg_Entity_LINE **|struct _dwg_entity_LINE **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_LOAD = {"_p_p__dwg_entity_LOAD", "Dwg_Entity_LOAD **|struct _dwg_entity_LOAD **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_LWPOLYLINE = {"_p_p__dwg_entity_LWPOLYLINE", "Dwg_Entity_LWPOLYLINE **|struct _dwg_entity_LWPOLYLINE **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_MESH = {"_p_p__dwg_entity_MESH", "Dwg_Entity_MESH **|struct _dwg_entity_MESH **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_MINSERT = {"_p_p__dwg_entity_MINSERT", "Dwg_Entity_MINSERT **|struct _dwg_entity_MINSERT **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_MLINE = {"_p_p__dwg_entity_MLINE", "Dwg_Entity_MLINE **|struct _dwg_entity_MLINE **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_MTEXT = {"_p_p__dwg_entity_MTEXT", "Dwg_Entity_MTEXT **|struct _dwg_entity_MTEXT **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_MULTILEADER = {"_p_p__dwg_entity_MULTILEADER", "Dwg_Entity_MULTILEADER **|struct _dwg_entity_MULTILEADER **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_OLE2FRAME = {"_p_p__dwg_entity_OLE2FRAME", "Dwg_Entity_OLE2FRAME **|struct _dwg_entity_OLE2FRAME **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_OLEFRAME = {"_p_p__dwg_entity_OLEFRAME", "Dwg_Entity_OLEFRAME **|struct _dwg_entity_OLEFRAME **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_PLANESURFACE = {"_p_p__dwg_entity_PLANESURFACE", "Dwg_Entity_PLANESURFACE **|struct _dwg_entity_PLANESURFACE **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_POINT = {"_p_p__dwg_entity_POINT", "Dwg_Entity_POINT **|struct _dwg_entity_POINT **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_POINTCLOUD = {"_p_p__dwg_entity_POINTCLOUD", "Dwg_Entity_POINTCLOUD **|struct _dwg_entity_POINTCLOUD **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_POINTCLOUDEX = {"_p_p__dwg_entity_POINTCLOUDEX", "Dwg_Entity_POINTCLOUDEX **|struct _dwg_entity_POINTCLOUDEX **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_POLYLINE_2D = {"_p_p__dwg_entity_POLYLINE_2D", "Dwg_Entity_POLYLINE_2D **|struct _dwg_entity_POLYLINE_2D **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_POLYLINE_3D = {"_p_p__dwg_entity_POLYLINE_3D", "Dwg_Entity_POLYLINE_3D **|struct _dwg_entity_POLYLINE_3D **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_POLYLINE_MESH = {"_p_p__dwg_entity_POLYLINE_MESH", "Dwg_Entity_POLYLINE_MESH **|struct _dwg_entity_POLYLINE_MESH **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_POLYLINE_PFACE = {"_p_p__dwg_entity_POLYLINE_PFACE", "Dwg_Entity_POLYLINE_PFACE **|struct _dwg_entity_POLYLINE_PFACE **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_PROXY_ENTITY = {"_p_p__dwg_entity_PROXY_ENTITY", "Dwg_Entity_PROXY_ENTITY **|struct _dwg_entity_PROXY_ENTITY **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_RAY = {"_p_p__dwg_entity_RAY", "Dwg_Entity_RAY **|Dwg_Entity_XLINE **|struct _dwg_entity_RAY **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_REPEAT = {"_p_p__dwg_entity_REPEAT", "Dwg_Entity_REPEAT **|struct _dwg_entity_REPEAT **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_SECTIONOBJECT = {"_p_p__dwg_entity_SECTIONOBJECT", "Dwg_Entity_SECTIONOBJECT **|struct _dwg_entity_SECTIONOBJECT **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_SEQEND = {"_p_p__dwg_entity_SEQEND", "Dwg_Entity_SEQEND **|struct _dwg_entity_SEQEND **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_SHAPE = {"_p_p__dwg_entity_SHAPE", "Dwg_Entity_SHAPE **|struct _dwg_entity_SHAPE **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_SOLID = {"_p_p__dwg_entity_SOLID", "Dwg_Entity_SOLID **|struct _dwg_entity_SOLID **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_SPLINE = {"_p_p__dwg_entity_SPLINE", "Dwg_Entity_SPLINE **|struct _dwg_entity_SPLINE **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_TEXT = {"_p_p__dwg_entity_TEXT", "Dwg_Entity_TEXT **|struct _dwg_entity_TEXT **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_TOLERANCE = {"_p_p__dwg_entity_TOLERANCE", "Dwg_Entity_TOLERANCE **|struct _dwg_entity_TOLERANCE **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_TRACE = {"_p_p__dwg_entity_TRACE", "Dwg_Entity_TRACE **|struct _dwg_entity_TRACE **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_UNKNOWN_ENT = {"_p_p__dwg_entity_UNKNOWN_ENT", "Dwg_Entity_UNKNOWN_ENT **|struct _dwg_entity_UNKNOWN_ENT **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_VERTEX_2D = {"_p_p__dwg_entity_VERTEX_2D", "Dwg_Entity_VERTEX_2D **|struct _dwg_entity_VERTEX_2D **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_VERTEX_3D = {"_p_p__dwg_entity_VERTEX_3D", "Dwg_Entity_VERTEX_3D **|Dwg_Entity_VERTEX_MESH **|Dwg_Entity_VERTEX_PFACE **|struct _dwg_entity_VERTEX_3D **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_VERTEX_PFACE_FACE = {"_p_p__dwg_entity_VERTEX_PFACE_FACE", "Dwg_Entity_VERTEX_PFACE_FACE **|struct _dwg_entity_VERTEX_PFACE_FACE **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_VIEWPORT = {"_p_p__dwg_entity_VIEWPORT", "Dwg_Entity_VIEWPORT **|struct _dwg_entity_VIEWPORT **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_entity_WIPEOUT = {"_p_p__dwg_entity_WIPEOUT", "Dwg_Entity_WIPEOUT **|struct _dwg_entity_WIPEOUT **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ACSH_BOOLEAN_CLASS = {"_p_p__dwg_object_ACSH_BOOLEAN_CLASS", "Dwg_Object_ACSH_BOOLEAN_CLASS **|struct _dwg_object_ACSH_BOOLEAN_CLASS **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ACSH_BOX_CLASS = {"_p_p__dwg_object_ACSH_BOX_CLASS", "Dwg_Object_ACSH_BOX_CLASS **|struct _dwg_object_ACSH_BOX_CLASS **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ACSH_BREP_CLASS = {"_p_p__dwg_object_ACSH_BREP_CLASS", "Dwg_Object_ACSH_BREP_CLASS **|struct _dwg_object_ACSH_BREP_CLASS **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ACSH_CHAMFER_CLASS = {"_p_p__dwg_object_ACSH_CHAMFER_CLASS", "Dwg_Object_ACSH_CHAMFER_CLASS **|struct _dwg_object_ACSH_CHAMFER_CLASS **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ACSH_CONE_CLASS = {"_p_p__dwg_object_ACSH_CONE_CLASS", "Dwg_Object_ACSH_CONE_CLASS **|struct _dwg_object_ACSH_CONE_CLASS **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ACSH_CYLINDER_CLASS = {"_p_p__dwg_object_ACSH_CYLINDER_CLASS", "Dwg_Object_ACSH_CYLINDER_CLASS **|struct _dwg_object_ACSH_CYLINDER_CLASS **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ACSH_FILLET_CLASS = {"_p_p__dwg_object_ACSH_FILLET_CLASS", "Dwg_Object_ACSH_FILLET_CLASS **|struct _dwg_object_ACSH_FILLET_CLASS **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ACSH_HISTORY_CLASS = {"_p_p__dwg_object_ACSH_HISTORY_CLASS", "Dwg_Object_ACSH_HISTORY_CLASS **|struct _dwg_object_ACSH_HISTORY_CLASS **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ACSH_PYRAMID_CLASS = {"_p_p__dwg_object_ACSH_PYRAMID_CLASS", "Dwg_Object_ACSH_PYRAMID_CLASS **|struct _dwg_object_ACSH_PYRAMID_CLASS **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ACSH_SPHERE_CLASS = {"_p_p__dwg_object_ACSH_SPHERE_CLASS", "Dwg_Object_ACSH_SPHERE_CLASS **|struct _dwg_object_ACSH_SPHERE_CLASS **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ACSH_TORUS_CLASS = {"_p_p__dwg_object_ACSH_TORUS_CLASS", "Dwg_Object_ACSH_TORUS_CLASS **|struct _dwg_object_ACSH_TORUS_CLASS **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ACSH_WEDGE_CLASS = {"_p_p__dwg_object_ACSH_WEDGE_CLASS", "Dwg_Object_ACSH_WEDGE_CLASS **|struct _dwg_object_ACSH_WEDGE_CLASS **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ALDIMOBJECTCONTEXTDATA = {"_p_p__dwg_object_ALDIMOBJECTCONTEXTDATA", "Dwg_Object_ALDIMOBJECTCONTEXTDATA **|struct _dwg_object_ALDIMOBJECTCONTEXTDATA **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_APPID = {"_p_p__dwg_object_APPID", "Dwg_Object_APPID **|struct _dwg_object_APPID **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_APPID_CONTROL = {"_p_p__dwg_object_APPID_CONTROL", "Dwg_Object_APPID_CONTROL **|struct _dwg_object_APPID_CONTROL **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ASSOC2DCONSTRAINTGROUP = {"_p_p__dwg_object_ASSOC2DCONSTRAINTGROUP", "Dwg_Object_ASSOC2DCONSTRAINTGROUP **|struct _dwg_object_ASSOC2DCONSTRAINTGROUP **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ASSOCACTION = {"_p_p__dwg_object_ASSOCACTION", "Dwg_Object_ASSOCACTION **|struct _dwg_object_ASSOCACTION **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ASSOCACTIONPARAM = {"_p_p__dwg_object_ASSOCACTIONPARAM", "Dwg_Object_ASSOCACTIONPARAM **|struct _dwg_object_ASSOCACTIONPARAM **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ASSOCARRAYACTIONBODY = {"_p_p__dwg_object_ASSOCARRAYACTIONBODY", "Dwg_Object_ASSOCARRAYACTIONBODY **|struct _dwg_object_ASSOCARRAYACTIONBODY **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ASSOCASMBODYACTIONPARAM = {"_p_p__dwg_object_ASSOCASMBODYACTIONPARAM", "Dwg_Object_ASSOCASMBODYACTIONPARAM **|struct _dwg_object_ASSOCASMBODYACTIONPARAM **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY = {"_p_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY", "Dwg_Object_ASSOCBLENDSURFACEACTIONBODY **|struct _dwg_object_ASSOCBLENDSURFACEACTIONBODY **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM = {"_p_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM", "Dwg_Object_ASSOCCOMPOUNDACTIONPARAM **|struct _dwg_object_ASSOCCOMPOUNDACTIONPARAM **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ASSOCDEPENDENCY = {"_p_p__dwg_object_ASSOCDEPENDENCY", "Dwg_Object_ASSOCDEPENDENCY **|struct _dwg_object_ASSOCDEPENDENCY **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ASSOCDIMDEPENDENCYBODY = {"_p_p__dwg_object_ASSOCDIMDEPENDENCYBODY", "Dwg_Object_ASSOCDIMDEPENDENCYBODY **|struct _dwg_object_ASSOCDIMDEPENDENCYBODY **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ASSOCEXTENDSURFACEACTIONBODY = {"_p_p__dwg_object_ASSOCEXTENDSURFACEACTIONBODY", "Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY **|struct _dwg_object_ASSOCEXTENDSURFACEACTIONBODY **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY = {"_p_p__dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY", "Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY **|struct _dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ASSOCFACEACTIONPARAM = {"_p_p__dwg_object_ASSOCFACEACTIONPARAM", "Dwg_Object_ASSOCFACEACTIONPARAM **|struct _dwg_object_ASSOCFACEACTIONPARAM **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ASSOCFILLETSURFACEACTIONBODY = {"_p_p__dwg_object_ASSOCFILLETSURFACEACTIONBODY", "Dwg_Object_ASSOCFILLETSURFACEACTIONBODY **|struct _dwg_object_ASSOCFILLETSURFACEACTIONBODY **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ASSOCGEOMDEPENDENCY = {"_p_p__dwg_object_ASSOCGEOMDEPENDENCY", "Dwg_Object_ASSOCGEOMDEPENDENCY **|struct _dwg_object_ASSOCGEOMDEPENDENCY **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ASSOCLOFTEDSURFACEACTIONBODY = {"_p_p__dwg_object_ASSOCLOFTEDSURFACEACTIONBODY", "Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY **|struct _dwg_object_ASSOCLOFTEDSURFACEACTIONBODY **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ASSOCNETWORK = {"_p_p__dwg_object_ASSOCNETWORK", "Dwg_Object_ASSOCNETWORK **|struct _dwg_object_ASSOCNETWORK **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ASSOCNETWORKSURFACEACTIONBODY = {"_p_p__dwg_object_ASSOCNETWORKSURFACEACTIONBODY", "Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY **|struct _dwg_object_ASSOCNETWORKSURFACEACTIONBODY **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ASSOCOBJECTACTIONPARAM = {"_p_p__dwg_object_ASSOCOBJECTACTIONPARAM", "Dwg_Object_ASSOCOBJECTACTIONPARAM **|struct _dwg_object_ASSOCOBJECTACTIONPARAM **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ASSOCOFFSETSURFACEACTIONBODY = {"_p_p__dwg_object_ASSOCOFFSETSURFACEACTIONBODY", "Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY **|struct _dwg_object_ASSOCOFFSETSURFACEACTIONBODY **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM = {"_p_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM", "Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM **|struct _dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ASSOCPATCHSURFACEACTIONBODY = {"_p_p__dwg_object_ASSOCPATCHSURFACEACTIONBODY", "Dwg_Object_ASSOCPATCHSURFACEACTIONBODY **|struct _dwg_object_ASSOCPATCHSURFACEACTIONBODY **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ASSOCPATHACTIONPARAM = {"_p_p__dwg_object_ASSOCPATHACTIONPARAM", "Dwg_Object_ASSOCPATHACTIONPARAM **|struct _dwg_object_ASSOCPATHACTIONPARAM **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ASSOCPLANESURFACEACTIONBODY = {"_p_p__dwg_object_ASSOCPLANESURFACEACTIONBODY", "Dwg_Object_ASSOCPLANESURFACEACTIONBODY **|struct _dwg_object_ASSOCPLANESURFACEACTIONBODY **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ASSOCPOINTREFACTIONPARAM = {"_p_p__dwg_object_ASSOCPOINTREFACTIONPARAM", "Dwg_Object_ASSOCPOINTREFACTIONPARAM **|struct _dwg_object_ASSOCPOINTREFACTIONPARAM **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY = {"_p_p__dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY", "Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY **|struct _dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ASSOCTRIMSURFACEACTIONBODY = {"_p_p__dwg_object_ASSOCTRIMSURFACEACTIONBODY", "Dwg_Object_ASSOCTRIMSURFACEACTIONBODY **|struct _dwg_object_ASSOCTRIMSURFACEACTIONBODY **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ASSOCVALUEDEPENDENCY = {"_p_p__dwg_object_ASSOCVALUEDEPENDENCY", "Dwg_Object_ASSOCVALUEDEPENDENCY **|struct _dwg_object_ASSOCVALUEDEPENDENCY **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ASSOCVARIABLE = {"_p_p__dwg_object_ASSOCVARIABLE", "Dwg_Object_ASSOCVARIABLE **|struct _dwg_object_ASSOCVARIABLE **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ASSOCVERTEXACTIONPARAM = {"_p_p__dwg_object_ASSOCVERTEXACTIONPARAM", "Dwg_Object_ASSOCVERTEXACTIONPARAM **|struct _dwg_object_ASSOCVERTEXACTIONPARAM **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLKREFOBJECTCONTEXTDATA = {"_p_p__dwg_object_BLKREFOBJECTCONTEXTDATA", "Dwg_Object_BLKREFOBJECTCONTEXTDATA **|struct _dwg_object_BLKREFOBJECTCONTEXTDATA **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER = {"_p_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER", "Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER **|struct _dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKALIGNMENTGRIP = {"_p_p__dwg_object_BLOCKALIGNMENTGRIP", "Dwg_Object_BLOCKALIGNMENTGRIP **|struct _dwg_object_BLOCKALIGNMENTGRIP **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKALIGNMENTPARAMETER = {"_p_p__dwg_object_BLOCKALIGNMENTPARAMETER", "Dwg_Object_BLOCKALIGNMENTPARAMETER **|struct _dwg_object_BLOCKALIGNMENTPARAMETER **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER = {"_p_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER", "Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER **|struct _dwg_object_BLOCKANGULARCONSTRAINTPARAMETER **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKARRAYACTION = {"_p_p__dwg_object_BLOCKARRAYACTION", "Dwg_Object_BLOCKARRAYACTION **|struct _dwg_object_BLOCKARRAYACTION **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKBASEPOINTPARAMETER = {"_p_p__dwg_object_BLOCKBASEPOINTPARAMETER", "Dwg_Object_BLOCKBASEPOINTPARAMETER **|struct _dwg_object_BLOCKBASEPOINTPARAMETER **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER = {"_p_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER", "Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER **|struct _dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKFLIPACTION = {"_p_p__dwg_object_BLOCKFLIPACTION", "Dwg_Object_BLOCKFLIPACTION **|struct _dwg_object_BLOCKFLIPACTION **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKFLIPGRIP = {"_p_p__dwg_object_BLOCKFLIPGRIP", "Dwg_Object_BLOCKFLIPGRIP **|struct _dwg_object_BLOCKFLIPGRIP **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKFLIPPARAMETER = {"_p_p__dwg_object_BLOCKFLIPPARAMETER", "Dwg_Object_BLOCKFLIPPARAMETER **|struct _dwg_object_BLOCKFLIPPARAMETER **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKGRIPLOCATIONCOMPONENT = {"_p_p__dwg_object_BLOCKGRIPLOCATIONCOMPONENT", "Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT **|struct _dwg_object_BLOCKGRIPLOCATIONCOMPONENT **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER = {"_p_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER", "Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER **|struct _dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER = {"_p_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER", "Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER **|struct _dwg_object_BLOCKLINEARCONSTRAINTPARAMETER **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKLINEARGRIP = {"_p_p__dwg_object_BLOCKLINEARGRIP", "Dwg_Object_BLOCKLINEARGRIP **|struct _dwg_object_BLOCKLINEARGRIP **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKLINEARPARAMETER = {"_p_p__dwg_object_BLOCKLINEARPARAMETER", "Dwg_Object_BLOCKLINEARPARAMETER **|struct _dwg_object_BLOCKLINEARPARAMETER **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKLOOKUPACTION = {"_p_p__dwg_object_BLOCKLOOKUPACTION", "Dwg_Object_BLOCKLOOKUPACTION **|struct _dwg_object_BLOCKLOOKUPACTION **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKLOOKUPGRIP = {"_p_p__dwg_object_BLOCKLOOKUPGRIP", "Dwg_Object_BLOCKLOOKUPGRIP **|struct _dwg_object_BLOCKLOOKUPGRIP **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKLOOKUPPARAMETER = {"_p_p__dwg_object_BLOCKLOOKUPPARAMETER", "Dwg_Object_BLOCKLOOKUPPARAMETER **|struct _dwg_object_BLOCKLOOKUPPARAMETER **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKMOVEACTION = {"_p_p__dwg_object_BLOCKMOVEACTION", "Dwg_Object_BLOCKMOVEACTION **|struct _dwg_object_BLOCKMOVEACTION **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKPARAMDEPENDENCYBODY = {"_p_p__dwg_object_BLOCKPARAMDEPENDENCYBODY", "Dwg_Object_BLOCKPARAMDEPENDENCYBODY **|struct _dwg_object_BLOCKPARAMDEPENDENCYBODY **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKPOINTPARAMETER = {"_p_p__dwg_object_BLOCKPOINTPARAMETER", "Dwg_Object_BLOCKPOINTPARAMETER **|struct _dwg_object_BLOCKPOINTPARAMETER **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKPOLARGRIP = {"_p_p__dwg_object_BLOCKPOLARGRIP", "Dwg_Object_BLOCKPOLARGRIP **|struct _dwg_object_BLOCKPOLARGRIP **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKPOLARPARAMETER = {"_p_p__dwg_object_BLOCKPOLARPARAMETER", "Dwg_Object_BLOCKPOLARPARAMETER **|struct _dwg_object_BLOCKPOLARPARAMETER **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKPOLARSTRETCHACTION = {"_p_p__dwg_object_BLOCKPOLARSTRETCHACTION", "Dwg_Object_BLOCKPOLARSTRETCHACTION **|struct _dwg_object_BLOCKPOLARSTRETCHACTION **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER = {"_p_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER", "Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER **|struct _dwg_object_BLOCKRADIALCONSTRAINTPARAMETER **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKREPRESENTATION = {"_p_p__dwg_object_BLOCKREPRESENTATION", "Dwg_Object_BLOCKREPRESENTATION **|struct _dwg_object_BLOCKREPRESENTATION **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKROTATEACTION = {"_p_p__dwg_object_BLOCKROTATEACTION", "Dwg_Object_BLOCKROTATEACTION **|struct _dwg_object_BLOCKROTATEACTION **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKROTATIONGRIP = {"_p_p__dwg_object_BLOCKROTATIONGRIP", "Dwg_Object_BLOCKROTATIONGRIP **|struct _dwg_object_BLOCKROTATIONGRIP **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKROTATIONPARAMETER = {"_p_p__dwg_object_BLOCKROTATIONPARAMETER", "Dwg_Object_BLOCKROTATIONPARAMETER **|struct _dwg_object_BLOCKROTATIONPARAMETER **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKSCALEACTION = {"_p_p__dwg_object_BLOCKSCALEACTION", "Dwg_Object_BLOCKSCALEACTION **|struct _dwg_object_BLOCKSCALEACTION **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKSTRETCHACTION = {"_p_p__dwg_object_BLOCKSTRETCHACTION", "Dwg_Object_BLOCKSTRETCHACTION **|struct _dwg_object_BLOCKSTRETCHACTION **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKUSERPARAMETER = {"_p_p__dwg_object_BLOCKUSERPARAMETER", "Dwg_Object_BLOCKUSERPARAMETER **|struct _dwg_object_BLOCKUSERPARAMETER **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER = {"_p_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER", "Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER **|struct _dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKVISIBILITYGRIP = {"_p_p__dwg_object_BLOCKVISIBILITYGRIP", "Dwg_Object_BLOCKVISIBILITYGRIP **|struct _dwg_object_BLOCKVISIBILITYGRIP **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKVISIBILITYPARAMETER = {"_p_p__dwg_object_BLOCKVISIBILITYPARAMETER", "Dwg_Object_BLOCKVISIBILITYPARAMETER **|struct _dwg_object_BLOCKVISIBILITYPARAMETER **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKXYGRIP = {"_p_p__dwg_object_BLOCKXYGRIP", "Dwg_Object_BLOCKXYGRIP **|struct _dwg_object_BLOCKXYGRIP **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCKXYPARAMETER = {"_p_p__dwg_object_BLOCKXYPARAMETER", "Dwg_Object_BLOCKXYPARAMETER **|struct _dwg_object_BLOCKXYPARAMETER **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCK_CONTROL = {"_p_p__dwg_object_BLOCK_CONTROL", "Dwg_Object_BLOCK_CONTROL **|struct _dwg_object_BLOCK_CONTROL **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_BLOCK_HEADER = {"_p_p__dwg_object_BLOCK_HEADER", "Dwg_Object_BLOCK_HEADER **|struct _dwg_object_BLOCK_HEADER **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_CELLSTYLEMAP = {"_p_p__dwg_object_CELLSTYLEMAP", "Dwg_Object_CELLSTYLEMAP **|struct _dwg_object_CELLSTYLEMAP **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_DATALINK = {"_p_p__dwg_object_DATALINK", "Dwg_Object_DATALINK **|struct _dwg_object_DATALINK **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_DBCOLOR = {"_p_p__dwg_object_DBCOLOR", "Dwg_Object_DBCOLOR **|struct _dwg_object_DBCOLOR **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_DETAILVIEWSTYLE = {"_p_p__dwg_object_DETAILVIEWSTYLE", "Dwg_Object_DETAILVIEWSTYLE **|struct _dwg_object_DETAILVIEWSTYLE **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_DICTIONARY = {"_p_p__dwg_object_DICTIONARY", "Dwg_Object_DICTIONARY **|struct _dwg_object_DICTIONARY **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_DICTIONARYVAR = {"_p_p__dwg_object_DICTIONARYVAR", "Dwg_Object_DICTIONARYVAR **|struct _dwg_object_DICTIONARYVAR **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_DICTIONARYWDFLT = {"_p_p__dwg_object_DICTIONARYWDFLT", "Dwg_Object_DICTIONARYWDFLT **|struct _dwg_object_DICTIONARYWDFLT **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_DIMSTYLE = {"_p_p__dwg_object_DIMSTYLE", "Dwg_Object_DIMSTYLE **|struct _dwg_object_DIMSTYLE **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_DIMSTYLE_CONTROL = {"_p_p__dwg_object_DIMSTYLE_CONTROL", "Dwg_Object_DIMSTYLE_CONTROL **|struct _dwg_object_DIMSTYLE_CONTROL **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_DUMMY = {"_p_p__dwg_object_DUMMY", "Dwg_Object_DUMMY **|struct _dwg_object_DUMMY **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_DYNAMICBLOCKPURGEPREVENTER = {"_p_p__dwg_object_DYNAMICBLOCKPURGEPREVENTER", "Dwg_Object_DYNAMICBLOCKPURGEPREVENTER **|struct _dwg_object_DYNAMICBLOCKPURGEPREVENTER **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_EVALUATION_GRAPH = {"_p_p__dwg_object_EVALUATION_GRAPH", "Dwg_Object_EVALUATION_GRAPH **|struct _dwg_object_EVALUATION_GRAPH **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_FCFOBJECTCONTEXTDATA = {"_p_p__dwg_object_FCFOBJECTCONTEXTDATA", "Dwg_Object_FCFOBJECTCONTEXTDATA **|struct _dwg_object_FCFOBJECTCONTEXTDATA **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_FIELD = {"_p_p__dwg_object_FIELD", "Dwg_Object_FIELD **|struct _dwg_object_FIELD **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_FIELDLIST = {"_p_p__dwg_object_FIELDLIST", "Dwg_Object_FIELDLIST **|struct _dwg_object_FIELDLIST **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_GEODATA = {"_p_p__dwg_object_GEODATA", "Dwg_Object_GEODATA **|struct _dwg_object_GEODATA **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_GRADIENT_BACKGROUND = {"_p_p__dwg_object_GRADIENT_BACKGROUND", "Dwg_Object_GRADIENT_BACKGROUND **|struct _dwg_object_GRADIENT_BACKGROUND **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_GROUND_PLANE_BACKGROUND = {"_p_p__dwg_object_GROUND_PLANE_BACKGROUND", "Dwg_Object_GROUND_PLANE_BACKGROUND **|struct _dwg_object_GROUND_PLANE_BACKGROUND **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_GROUP = {"_p_p__dwg_object_GROUP", "Dwg_Object_GROUP **|struct _dwg_object_GROUP **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_IBL_BACKGROUND = {"_p_p__dwg_object_IBL_BACKGROUND", "Dwg_Object_IBL_BACKGROUND **|struct _dwg_object_IBL_BACKGROUND **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_IDBUFFER = {"_p_p__dwg_object_IDBUFFER", "Dwg_Object_IDBUFFER **|struct _dwg_object_IDBUFFER **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_IMAGEDEF = {"_p_p__dwg_object_IMAGEDEF", "Dwg_Object_IMAGEDEF **|struct _dwg_object_IMAGEDEF **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_IMAGEDEF_REACTOR = {"_p_p__dwg_object_IMAGEDEF_REACTOR", "Dwg_Object_IMAGEDEF_REACTOR **|struct _dwg_object_IMAGEDEF_REACTOR **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_IMAGE_BACKGROUND = {"_p_p__dwg_object_IMAGE_BACKGROUND", "Dwg_Object_IMAGE_BACKGROUND **|struct _dwg_object_IMAGE_BACKGROUND **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_INDEX = {"_p_p__dwg_object_INDEX", "Dwg_Object_INDEX **|struct _dwg_object_INDEX **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_LAYER = {"_p_p__dwg_object_LAYER", "Dwg_Object_LAYER **|struct _dwg_object_LAYER **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_LAYERFILTER = {"_p_p__dwg_object_LAYERFILTER", "Dwg_Object_LAYERFILTER **|struct _dwg_object_LAYERFILTER **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_LAYER_CONTROL = {"_p_p__dwg_object_LAYER_CONTROL", "Dwg_Object_LAYER_CONTROL **|struct _dwg_object_LAYER_CONTROL **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_LAYER_INDEX = {"_p_p__dwg_object_LAYER_INDEX", "Dwg_Object_LAYER_INDEX **|struct _dwg_object_LAYER_INDEX **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_LAYOUT = {"_p_p__dwg_object_LAYOUT", "Dwg_Object_LAYOUT **|struct _dwg_object_LAYOUT **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_LEADEROBJECTCONTEXTDATA = {"_p_p__dwg_object_LEADEROBJECTCONTEXTDATA", "Dwg_Object_LEADEROBJECTCONTEXTDATA **|struct _dwg_object_LEADEROBJECTCONTEXTDATA **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_LIGHTLIST = {"_p_p__dwg_object_LIGHTLIST", "Dwg_Object_LIGHTLIST **|struct _dwg_object_LIGHTLIST **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_LONG_TRANSACTION = {"_p_p__dwg_object_LONG_TRANSACTION", "Dwg_Object_LONG_TRANSACTION **|struct _dwg_object_LONG_TRANSACTION **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_LTYPE = {"_p_p__dwg_object_LTYPE", "Dwg_Object_LTYPE **|struct _dwg_object_LTYPE **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_LTYPE_CONTROL = {"_p_p__dwg_object_LTYPE_CONTROL", "Dwg_Object_LTYPE_CONTROL **|struct _dwg_object_LTYPE_CONTROL **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_MATERIAL = {"_p_p__dwg_object_MATERIAL", "Dwg_Object_MATERIAL **|struct _dwg_object_MATERIAL **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_MENTALRAYRENDERSETTINGS = {"_p_p__dwg_object_MENTALRAYRENDERSETTINGS", "Dwg_Object_MENTALRAYRENDERSETTINGS **|struct _dwg_object_MENTALRAYRENDERSETTINGS **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_MLEADERSTYLE = {"_p_p__dwg_object_MLEADERSTYLE", "Dwg_Object_MLEADERSTYLE **|struct _dwg_object_MLEADERSTYLE **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_MLINESTYLE = {"_p_p__dwg_object_MLINESTYLE", "Dwg_Object_MLINESTYLE **|struct _dwg_object_MLINESTYLE **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_MTEXTOBJECTCONTEXTDATA = {"_p_p__dwg_object_MTEXTOBJECTCONTEXTDATA", "Dwg_Object_MTEXTOBJECTCONTEXTDATA **|struct _dwg_object_MTEXTOBJECTCONTEXTDATA **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_OBJECT_PTR = {"_p_p__dwg_object_OBJECT_PTR", "Dwg_Object_OBJECT_PTR **|struct _dwg_object_OBJECT_PTR **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_PARTIAL_VIEWING_INDEX = {"_p_p__dwg_object_PARTIAL_VIEWING_INDEX", "Dwg_Object_PARTIAL_VIEWING_INDEX **|struct _dwg_object_PARTIAL_VIEWING_INDEX **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_PLACEHOLDER = {"_p_p__dwg_object_PLACEHOLDER", "Dwg_Object_PLACEHOLDER **|struct _dwg_object_PLACEHOLDER **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_PLOTSETTINGS = {"_p_p__dwg_object_PLOTSETTINGS", "Dwg_Object_PLOTSETTINGS **|struct _dwg_object_PLOTSETTINGS **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_POINTCLOUDCOLORMAP = {"_p_p__dwg_object_POINTCLOUDCOLORMAP", "Dwg_Object_POINTCLOUDCOLORMAP **|struct _dwg_object_POINTCLOUDCOLORMAP **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_POINTCLOUDDEF = {"_p_p__dwg_object_POINTCLOUDDEF", "Dwg_Object_POINTCLOUDDEF **|struct _dwg_object_POINTCLOUDDEF **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_POINTCLOUDDEFEX = {"_p_p__dwg_object_POINTCLOUDDEFEX", "Dwg_Object_POINTCLOUDDEFEX **|struct _dwg_object_POINTCLOUDDEFEX **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_POINTCLOUDDEF_REACTOR = {"_p_p__dwg_object_POINTCLOUDDEF_REACTOR", "Dwg_Object_POINTCLOUDDEF_REACTOR **|struct _dwg_object_POINTCLOUDDEF_REACTOR **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_POINTCLOUDDEF_REACTOR_EX = {"_p_p__dwg_object_POINTCLOUDDEF_REACTOR_EX", "Dwg_Object_POINTCLOUDDEF_REACTOR_EX **|struct _dwg_object_POINTCLOUDDEF_REACTOR_EX **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_PROXY_OBJECT = {"_p_p__dwg_object_PROXY_OBJECT", "Dwg_Object_PROXY_OBJECT **|struct _dwg_object_PROXY_OBJECT **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_RAPIDRTRENDERSETTINGS = {"_p_p__dwg_object_RAPIDRTRENDERSETTINGS", "Dwg_Object_RAPIDRTRENDERSETTINGS **|struct _dwg_object_RAPIDRTRENDERSETTINGS **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_RASTERVARIABLES = {"_p_p__dwg_object_RASTERVARIABLES", "Dwg_Object_RASTERVARIABLES **|struct _dwg_object_RASTERVARIABLES **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_RENDERENTRY = {"_p_p__dwg_object_RENDERENTRY", "Dwg_Object_RENDERENTRY **|struct _dwg_object_RENDERENTRY **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_RENDERENVIRONMENT = {"_p_p__dwg_object_RENDERENVIRONMENT", "Dwg_Object_RENDERENVIRONMENT **|struct _dwg_object_RENDERENVIRONMENT **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_RENDERGLOBAL = {"_p_p__dwg_object_RENDERGLOBAL", "Dwg_Object_RENDERGLOBAL **|struct _dwg_object_RENDERGLOBAL **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_RENDERSETTINGS = {"_p_p__dwg_object_RENDERSETTINGS", "Dwg_Object_RENDERSETTINGS **|struct _dwg_object_RENDERSETTINGS **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_SCALE = {"_p_p__dwg_object_SCALE", "Dwg_Object_SCALE **|struct _dwg_object_SCALE **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_SECTIONVIEWSTYLE = {"_p_p__dwg_object_SECTIONVIEWSTYLE", "Dwg_Object_SECTIONVIEWSTYLE **|struct _dwg_object_SECTIONVIEWSTYLE **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_SECTION_MANAGER = {"_p_p__dwg_object_SECTION_MANAGER", "Dwg_Object_SECTION_MANAGER **|struct _dwg_object_SECTION_MANAGER **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_SECTION_SETTINGS = {"_p_p__dwg_object_SECTION_SETTINGS", "Dwg_Object_SECTION_SETTINGS **|struct _dwg_object_SECTION_SETTINGS **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_SKYLIGHT_BACKGROUND = {"_p_p__dwg_object_SKYLIGHT_BACKGROUND", "Dwg_Object_SKYLIGHT_BACKGROUND **|struct _dwg_object_SKYLIGHT_BACKGROUND **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_SOLID_BACKGROUND = {"_p_p__dwg_object_SOLID_BACKGROUND", "Dwg_Object_SOLID_BACKGROUND **|struct _dwg_object_SOLID_BACKGROUND **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_SORTENTSTABLE = {"_p_p__dwg_object_SORTENTSTABLE", "Dwg_Object_SORTENTSTABLE **|struct _dwg_object_SORTENTSTABLE **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_SPATIAL_FILTER = {"_p_p__dwg_object_SPATIAL_FILTER", "Dwg_Object_SPATIAL_FILTER **|struct _dwg_object_SPATIAL_FILTER **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_SPATIAL_INDEX = {"_p_p__dwg_object_SPATIAL_INDEX", "Dwg_Object_SPATIAL_INDEX **|struct _dwg_object_SPATIAL_INDEX **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_STYLE = {"_p_p__dwg_object_STYLE", "Dwg_Object_STYLE **|struct _dwg_object_STYLE **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_STYLE_CONTROL = {"_p_p__dwg_object_STYLE_CONTROL", "Dwg_Object_STYLE_CONTROL **|struct _dwg_object_STYLE_CONTROL **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_SUN = {"_p_p__dwg_object_SUN", "Dwg_Object_SUN **|struct _dwg_object_SUN **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_TABLEGEOMETRY = {"_p_p__dwg_object_TABLEGEOMETRY", "Dwg_Object_TABLEGEOMETRY **|struct _dwg_object_TABLEGEOMETRY **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_TABLESTYLE = {"_p_p__dwg_object_TABLESTYLE", "Dwg_Object_TABLESTYLE **|struct _dwg_object_TABLESTYLE **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_TEXTOBJECTCONTEXTDATA = {"_p_p__dwg_object_TEXTOBJECTCONTEXTDATA", "Dwg_Object_TEXTOBJECTCONTEXTDATA **|struct _dwg_object_TEXTOBJECTCONTEXTDATA **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_UCS = {"_p_p__dwg_object_UCS", "Dwg_Object_UCS **|struct _dwg_object_UCS **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_UCS_CONTROL = {"_p_p__dwg_object_UCS_CONTROL", "Dwg_Object_UCS_CONTROL **|struct _dwg_object_UCS_CONTROL **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_UNKNOWN_OBJ = {"_p_p__dwg_object_UNKNOWN_OBJ", "Dwg_Object_UNKNOWN_OBJ **|struct _dwg_object_UNKNOWN_OBJ **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_VBA_PROJECT = {"_p_p__dwg_object_VBA_PROJECT", "Dwg_Object_VBA_PROJECT **|struct _dwg_object_VBA_PROJECT **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_VIEW = {"_p_p__dwg_object_VIEW", "Dwg_Object_VIEW **|struct _dwg_object_VIEW **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_VIEW_CONTROL = {"_p_p__dwg_object_VIEW_CONTROL", "Dwg_Object_VIEW_CONTROL **|struct _dwg_object_VIEW_CONTROL **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_VISUALSTYLE = {"_p_p__dwg_object_VISUALSTYLE", "Dwg_Object_VISUALSTYLE **|struct _dwg_object_VISUALSTYLE **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_VPORT = {"_p_p__dwg_object_VPORT", "Dwg_Object_VPORT **|struct _dwg_object_VPORT **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_VPORT_CONTROL = {"_p_p__dwg_object_VPORT_CONTROL", "Dwg_Object_VPORT_CONTROL **|struct _dwg_object_VPORT_CONTROL **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_VX_CONTROL = {"_p_p__dwg_object_VX_CONTROL", "Dwg_Object_VX_CONTROL **|struct _dwg_object_VX_CONTROL **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_VX_TABLE_RECORD = {"_p_p__dwg_object_VX_TABLE_RECORD", "Dwg_Object_VX_TABLE_RECORD **|struct _dwg_object_VX_TABLE_RECORD **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_WIPEOUTVARIABLES = {"_p_p__dwg_object_WIPEOUTVARIABLES", "Dwg_Object_WIPEOUTVARIABLES **|struct _dwg_object_WIPEOUTVARIABLES **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_XRECORD = {"_p_p__dwg_object_XRECORD", "Dwg_Object_XRECORD **|struct _dwg_object_XRECORD **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_entity = {"_p_p__dwg_object_entity", "Dwg_Object_Entity **|struct _dwg_object_entity **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_object_ref = {"_p_p__dwg_object_ref", "BITCODE_H *|Dwg_Object_Ref **|struct _dwg_object_ref **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p__dwg_section = {"_p_p__dwg_section", "Dwg_Section **|struct _dwg_section **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_p_char = {"_p_p_char", "BITCODE_D2T *|BITCODE_TV *|char **", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_r2007_file_header = {"_p_r2007_file_header", "Dwg_R2007_Header *|struct r2007_file_header *|r2007_file_header *", 0, 0, (void*)"LibreDWG::Dwg_R2007_Header", 0};
static swig_type_info _swigt__p_rgbpalette = {"_p_rgbpalette", "Dwg_RGB_Palette *|struct rgbpalette *|rgbpalette *", 0, 0, (void*)"LibreDWG::Dwg_RGB_Palette", 0};
static swig_type_info _swigt__p_short = {"_p_short", "BITCODE_BSd *|BITCODE_RSd *|int16_t *|int_least16_t *|short *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_signed_char = {"_p_signed_char", "BITCODE_RCd *|int8_t *|int_fast8_t *|int_least8_t *|signed char *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_unsigned_char = {"_p_unsigned_char", "BITCODE_3B *|BITCODE_4BITS *|BITCODE_B *|BITCODE_BB *|BITCODE_RC *|BITCODE_RCu *|BITCODE_RCx *|BITCODE_TF|uint8_t *|uint_fast8_t *|uint_least8_t *|unsigned char *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_unsigned_int = {"_p_unsigned_int", "BITCODE_BL *|BITCODE_BLx *|BITCODE_MS *|BITCODE_RL *|BITCODE_RLx *|uint32_t *|uint_fast16_t *|uint_fast32_t *|uint_least32_t *|uintptr_t *|unsigned int *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_unsigned_long_long = {"_p_unsigned_long_long", "BITCODE_BLL *|BITCODE_HV *|BITCODE_RLL *|BITCODE_RLLx *|BITCODE_UMC *|uint64_t *|uint_fast64_t *|uint_least64_t *|uintmax_t *|unsigned long long *", 0, 0, (void*)0, 0};
static swig_type_info _swigt__p_unsigned_short = {"_p_unsigned_short", "BITCODE_BS *|BITCODE_BSx *|BITCODE_RS *|BITCODE_RSx *|BITCODE_TU|dwg_wchar_t *|uint16_t *|uint_least16_t *|unsigned short *", 0, 0, (void*)0, 0};

static swig_type_info *swig_type_initial[] = {
  &_swigt__p_DWG_CLASS_STABILITY,
  &_swigt__p_DWG_COLOR_METHOD,
  &_swigt__p_DWG_ENTITY_SECTIONS,
  &_swigt__p_DWG_ERROR,
  &_swigt__p_DWG_HDL_CODE,
  &_swigt__p_DWG_OBJECT_SUPERTYPE,
  &_swigt__p_DWG_OBJECT_TYPE,
  &_swigt__p_DWG_OBJECT_TYPE_R11,
  &_swigt__p_DWG_SECTION_TYPE,
  &_swigt__p_DWG_SECTION_TYPE_R11,
  &_swigt__p_DWG_SECTION_TYPE_R13,
  &_swigt__p_DWG_VERSION_TYPE,
  &_swigt__p_Dwg_Section_Info,
  &_swigt__p_Dwg_Section_InfoHdr,
  &_swigt__p_RESBUF_VALUE_TYPE,
  &_swigt__p__dwg_3DSOLID_material,
  &_swigt__p__dwg_3DSOLID_silhouette,
  &_swigt__p__dwg_3DSOLID_wire,
  &_swigt__p__dwg_ACSH_HistoryNode,
  &_swigt__p__dwg_ACSH_SubentColor,
  &_swigt__p__dwg_ACSH_SubentMaterial,
  &_swigt__p__dwg_ACTIONBODY,
  &_swigt__p__dwg_ARRAYITEMLOCATOR,
  &_swigt__p__dwg_ASSOCACTIONBODY_action,
  &_swigt__p__dwg_ASSOCACTION_Deps,
  &_swigt__p__dwg_ASSOCARRAYITEM,
  &_swigt__p__dwg_ASSOCPARAMBASEDACTIONBODY,
  &_swigt__p__dwg_ASSOCSURFACEACTIONBODY,
  &_swigt__p__dwg_AcDbMTextObjectEmbedded,
  &_swigt__p__dwg_AcDs,
  &_swigt__p__dwg_AcDs_Data,
  &_swigt__p__dwg_AcDs_DataBlob,
  &_swigt__p__dwg_AcDs_DataBlob01,
  &_swigt__p__dwg_AcDs_DataBlobRef,
  &_swigt__p__dwg_AcDs_DataBlobRef_Page,
  &_swigt__p__dwg_AcDs_DataIndex,
  &_swigt__p__dwg_AcDs_DataIndex_Entry,
  &_swigt__p__dwg_AcDs_Data_Record,
  &_swigt__p__dwg_AcDs_Data_RecordHdr,
  &_swigt__p__dwg_AcDs_Schema,
  &_swigt__p__dwg_AcDs_SchemaData,
  &_swigt__p__dwg_AcDs_SchemaData_UProp,
  &_swigt__p__dwg_AcDs_SchemaIndex,
  &_swigt__p__dwg_AcDs_SchemaIndex_Prop,
  &_swigt__p__dwg_AcDs_Schema_Prop,
  &_swigt__p__dwg_AcDs_Search,
  &_swigt__p__dwg_AcDs_Search_Data,
  &_swigt__p__dwg_AcDs_Search_IdIdx,
  &_swigt__p__dwg_AcDs_Search_IdIdxs,
  &_swigt__p__dwg_AcDs_Segment,
  &_swigt__p__dwg_AcDs_SegmentIndex,
  &_swigt__p__dwg_BLOCKACTION_connectionpts,
  &_swigt__p__dwg_BLOCKLOOKUPACTION_lut,
  &_swigt__p__dwg_BLOCKPARAMETER_PropInfo,
  &_swigt__p__dwg_BLOCKPARAMETER_connection,
  &_swigt__p__dwg_BLOCKPARAMVALUESET,
  &_swigt__p__dwg_BLOCKSTRETCHACTION_codes,
  &_swigt__p__dwg_BLOCKSTRETCHACTION_handles,
  &_swigt__p__dwg_BLOCKVISIBILITYPARAMETER_state,
  &_swigt__p__dwg_COMPOUNDOBJECTID,
  &_swigt__p__dwg_CONSTRAINTGROUPNODE,
  &_swigt__p__dwg_CONTEXTDATA_dict,
  &_swigt__p__dwg_CONTEXTDATA_submgr,
  &_swigt__p__dwg_CellContentGeometry,
  &_swigt__p__dwg_CellStyle,
  &_swigt__p__dwg_ColorRamp,
  &_swigt__p__dwg_ContentFormat,
  &_swigt__p__dwg_DATALINK_customdata,
  &_swigt__p__dwg_DATATABLE_column,
  &_swigt__p__dwg_DATATABLE_row,
  &_swigt__p__dwg_DIMASSOC_Ref,
  &_swigt__p__dwg_DIMENSION_common,
  &_swigt__p__dwg_EVAL_Edge,
  &_swigt__p__dwg_EVAL_Node,
  &_swigt__p__dwg_EvalExpr,
  &_swigt__p__dwg_EvalExpr_value,
  &_swigt__p__dwg_EvalVariant,
  &_swigt__p__dwg_EvalVariant_u,
  &_swigt__p__dwg_FIELD_ChildValue,
  &_swigt__p__dwg_FileDepList_Files,
  &_swigt__p__dwg_FormattedTableData,
  &_swigt__p__dwg_FormattedTableMerged,
  &_swigt__p__dwg_GEODATA_meshface,
  &_swigt__p__dwg_GEODATA_meshpt,
  &_swigt__p__dwg_GridFormat,
  &_swigt__p__dwg_HATCH_Color,
  &_swigt__p__dwg_HATCH_ControlPoint,
  &_swigt__p__dwg_HATCH_DefLine,
  &_swigt__p__dwg_HATCH_Path,
  &_swigt__p__dwg_HATCH_PathSeg,
  &_swigt__p__dwg_HATCH_PolylinePath,
  &_swigt__p__dwg_LAYER_entry,
  &_swigt__p__dwg_LEADER_ArrowHead,
  &_swigt__p__dwg_LEADER_BlockLabel,
  &_swigt__p__dwg_LEADER_Break,
  &_swigt__p__dwg_LEADER_Line,
  &_swigt__p__dwg_LEADER_Node,
  &_swigt__p__dwg_LIGHTLIST_light,
  &_swigt__p__dwg_LTYPE_dash,
  &_swigt__p__dwg_LWPOLYLINE_width,
  &_swigt__p__dwg_LinkedData,
  &_swigt__p__dwg_LinkedTableData,
  &_swigt__p__dwg_MATERIAL_color,
  &_swigt__p__dwg_MATERIAL_gentexture,
  &_swigt__p__dwg_MATERIAL_mapper,
  &_swigt__p__dwg_MESH_edge,
  &_swigt__p__dwg_MLEADER_AnnotContext,
  &_swigt__p__dwg_MLEADER_Content,
  &_swigt__p__dwg_MLEADER_Content_Block,
  &_swigt__p__dwg_MLEADER_Content_MText,
  &_swigt__p__dwg_MLINESTYLE_line,
  &_swigt__p__dwg_MLINESTYLE_line_lt,
  &_swigt__p__dwg_MLINE_line,
  &_swigt__p__dwg_MLINE_vertex,
  &_swigt__p__dwg_OCD_Dimension,
  &_swigt__p__dwg_PARTIAL_VIEWING_INDEX_Entry,
  &_swigt__p__dwg_POINTCLOUDCOLORMAP_Ramp,
  &_swigt__p__dwg_POINTCLOUDEX_Croppings,
  &_swigt__p__dwg_POINTCLOUD_Clippings,
  &_swigt__p__dwg_POINTCLOUD_IntensityStyle,
  &_swigt__p__dwg_PROXY_LWPOLYLINE,
  &_swigt__p__dwg_R2004_Header,
  &_swigt__p__dwg_SECTION_geometrysettings,
  &_swigt__p__dwg_SECTION_typesettings,
  &_swigt__p__dwg_SPLINE_control_point,
  &_swigt__p__dwg_SUNSTUDY_Dates,
  &_swigt__p__dwg_SummaryInfo_Property,
  &_swigt__p__dwg_TABLEGEOMETRY_Cell,
  &_swigt__p__dwg_TABLESTYLE_CellStyle,
  &_swigt__p__dwg_TABLESTYLE_border,
  &_swigt__p__dwg_TABLESTYLE_rowstyles,
  &_swigt__p__dwg_TABLE_AttrDef,
  &_swigt__p__dwg_TABLE_BreakHeight,
  &_swigt__p__dwg_TABLE_BreakRow,
  &_swigt__p__dwg_TABLE_Cell,
  &_swigt__p__dwg_TABLE_CustomDataItem,
  &_swigt__p__dwg_TABLE_value,
  &_swigt__p__dwg_TableCell,
  &_swigt__p__dwg_TableCellContent,
  &_swigt__p__dwg_TableCellContent_Attr,
  &_swigt__p__dwg_TableDataColumn,
  &_swigt__p__dwg_TableRow,
  &_swigt__p__dwg_UCS_orthopts,
  &_swigt__p__dwg_VALUEPARAM,
  &_swigt__p__dwg_VALUEPARAM_vars,
  &_swigt__p__dwg_abstractentity_UNDERLAY,
  &_swigt__p__dwg_abstractobject_ASSOCARRAYPARAMETERS,
  &_swigt__p__dwg_abstractobject_UNDERLAYDEFINITION,
  &_swigt__p__dwg_appinfo,
  &_swigt__p__dwg_appinfohistory,
  &_swigt__p__dwg_auxheader,
  &_swigt__p__dwg_binary_chunk,
  &_swigt__p__dwg_binary_chunk_u,
  &_swigt__p__dwg_bitcode_2bd,
  &_swigt__p__dwg_bitcode_2rd,
  &_swigt__p__dwg_bitcode_3bd,
  &_swigt__p__dwg_bitcode_3rd,
  &_swigt__p__dwg_chain,
  &_swigt__p__dwg_class,
  &_swigt__p__dwg_color,
  &_swigt__p__dwg_entity_3DFACE,
  &_swigt__p__dwg_entity_3DLINE,
  &_swigt__p__dwg_entity_3DSOLID,
  &_swigt__p__dwg_entity_ALIGNMENTPARAMETERENTITY,
  &_swigt__p__dwg_entity_ARC,
  &_swigt__p__dwg_entity_ARCALIGNEDTEXT,
  &_swigt__p__dwg_entity_ARC_DIMENSION,
  &_swigt__p__dwg_entity_ATTDEF,
  &_swigt__p__dwg_entity_ATTRIB,
  &_swigt__p__dwg_entity_BASEPOINTPARAMETERENTITY,
  &_swigt__p__dwg_entity_BLOCK,
  &_swigt__p__dwg_entity_CAMERA,
  &_swigt__p__dwg_entity_CIRCLE,
  &_swigt__p__dwg_entity_DIMENSION_ALIGNED,
  &_swigt__p__dwg_entity_DIMENSION_ANG2LN,
  &_swigt__p__dwg_entity_DIMENSION_ANG3PT,
  &_swigt__p__dwg_entity_DIMENSION_DIAMETER,
  &_swigt__p__dwg_entity_DIMENSION_LINEAR,
  &_swigt__p__dwg_entity_DIMENSION_ORDINATE,
  &_swigt__p__dwg_entity_DIMENSION_RADIUS,
  &_swigt__p__dwg_entity_ELLIPSE,
  &_swigt__p__dwg_entity_ENDBLK,
  &_swigt__p__dwg_entity_ENDREP,
  &_swigt__p__dwg_entity_EXTRUDEDSURFACE,
  &_swigt__p__dwg_entity_FLIPGRIPENTITY,
  &_swigt__p__dwg_entity_FLIPPARAMETERENTITY,
  &_swigt__p__dwg_entity_GEOPOSITIONMARKER,
  &_swigt__p__dwg_entity_HATCH,
  &_swigt__p__dwg_entity_HELIX,
  &_swigt__p__dwg_entity_IMAGE,
  &_swigt__p__dwg_entity_INSERT,
  &_swigt__p__dwg_entity_JUMP,
  &_swigt__p__dwg_entity_LARGE_RADIAL_DIMENSION,
  &_swigt__p__dwg_entity_LAYOUTPRINTCONFIG,
  &_swigt__p__dwg_entity_LEADER,
  &_swigt__p__dwg_entity_LIGHT,
  &_swigt__p__dwg_entity_LINE,
  &_swigt__p__dwg_entity_LINEARGRIPENTITY,
  &_swigt__p__dwg_entity_LINEARPARAMETERENTITY,
  &_swigt__p__dwg_entity_LOAD,
  &_swigt__p__dwg_entity_LOFTEDSURFACE,
  &_swigt__p__dwg_entity_LWPOLYLINE,
  &_swigt__p__dwg_entity_MESH,
  &_swigt__p__dwg_entity_MINSERT,
  &_swigt__p__dwg_entity_MLINE,
  &_swigt__p__dwg_entity_MPOLYGON,
  &_swigt__p__dwg_entity_MTEXT,
  &_swigt__p__dwg_entity_MULTILEADER,
  &_swigt__p__dwg_entity_NAVISWORKSMODEL,
  &_swigt__p__dwg_entity_NURBSURFACE,
  &_swigt__p__dwg_entity_OLE2FRAME,
  &_swigt__p__dwg_entity_OLEFRAME,
  &_swigt__p__dwg_entity_PLANESURFACE,
  &_swigt__p__dwg_entity_POINT,
  &_swigt__p__dwg_entity_POINTCLOUD,
  &_swigt__p__dwg_entity_POINTCLOUDEX,
  &_swigt__p__dwg_entity_POINTPARAMETERENTITY,
  &_swigt__p__dwg_entity_POLARGRIPENTITY,
  &_swigt__p__dwg_entity_POLYLINE_2D,
  &_swigt__p__dwg_entity_POLYLINE_3D,
  &_swigt__p__dwg_entity_POLYLINE_MESH,
  &_swigt__p__dwg_entity_POLYLINE_PFACE,
  &_swigt__p__dwg_entity_PROXY_ENTITY,
  &_swigt__p__dwg_entity_RAY,
  &_swigt__p__dwg_entity_REPEAT,
  &_swigt__p__dwg_entity_REVOLVEDSURFACE,
  &_swigt__p__dwg_entity_ROTATIONGRIPENTITY,
  &_swigt__p__dwg_entity_ROTATIONPARAMETERENTITY,
  &_swigt__p__dwg_entity_RTEXT,
  &_swigt__p__dwg_entity_SECTIONOBJECT,
  &_swigt__p__dwg_entity_SEQEND,
  &_swigt__p__dwg_entity_SHAPE,
  &_swigt__p__dwg_entity_SOLID,
  &_swigt__p__dwg_entity_SPLINE,
  &_swigt__p__dwg_entity_SWEPTSURFACE,
  &_swigt__p__dwg_entity_TABLE,
  &_swigt__p__dwg_entity_TEXT,
  &_swigt__p__dwg_entity_TOLERANCE,
  &_swigt__p__dwg_entity_TRACE,
  &_swigt__p__dwg_entity_UNKNOWN_ENT,
  &_swigt__p__dwg_entity_VERTEX_2D,
  &_swigt__p__dwg_entity_VERTEX_3D,
  &_swigt__p__dwg_entity_VERTEX_PFACE_FACE,
  &_swigt__p__dwg_entity_VIEWPORT,
  &_swigt__p__dwg_entity_VISIBILITYGRIPENTITY,
  &_swigt__p__dwg_entity_VISIBILITYPARAMETERENTITY,
  &_swigt__p__dwg_entity_WIPEOUT,
  &_swigt__p__dwg_entity_XYGRIPENTITY,
  &_swigt__p__dwg_entity_XYPARAMETERENTITY,
  &_swigt__p__dwg_entity_eed,
  &_swigt__p__dwg_entity_eed_data,
  &_swigt__p__dwg_filedeplist,
  &_swigt__p__dwg_handle,
  &_swigt__p__dwg_header,
  &_swigt__p__dwg_header_variables,
  &_swigt__p__dwg_object,
  &_swigt__p__dwg_object_ACMECOMMANDHISTORY,
  &_swigt__p__dwg_object_ACMESCOPE,
  &_swigt__p__dwg_object_ACMESTATEMGR,
  &_swigt__p__dwg_object_ACSH_BOOLEAN_CLASS,
  &_swigt__p__dwg_object_ACSH_BOX_CLASS,
  &_swigt__p__dwg_object_ACSH_BREP_CLASS,
  &_swigt__p__dwg_object_ACSH_CHAMFER_CLASS,
  &_swigt__p__dwg_object_ACSH_CONE_CLASS,
  &_swigt__p__dwg_object_ACSH_CYLINDER_CLASS,
  &_swigt__p__dwg_object_ACSH_EXTRUSION_CLASS,
  &_swigt__p__dwg_object_ACSH_FILLET_CLASS,
  &_swigt__p__dwg_object_ACSH_HISTORY_CLASS,
  &_swigt__p__dwg_object_ACSH_LOFT_CLASS,
  &_swigt__p__dwg_object_ACSH_PYRAMID_CLASS,
  &_swigt__p__dwg_object_ACSH_REVOLVE_CLASS,
  &_swigt__p__dwg_object_ACSH_SPHERE_CLASS,
  &_swigt__p__dwg_object_ACSH_SWEEP_CLASS,
  &_swigt__p__dwg_object_ACSH_TORUS_CLASS,
  &_swigt__p__dwg_object_ACSH_WEDGE_CLASS,
  &_swigt__p__dwg_object_ALDIMOBJECTCONTEXTDATA,
  &_swigt__p__dwg_object_ANGDIMOBJECTCONTEXTDATA,
  &_swigt__p__dwg_object_ANNOTSCALEOBJECTCONTEXTDATA,
  &_swigt__p__dwg_object_APPID,
  &_swigt__p__dwg_object_APPID_CONTROL,
  &_swigt__p__dwg_object_ASSOC2DCONSTRAINTGROUP,
  &_swigt__p__dwg_object_ASSOC3POINTANGULARDIMACTIONBODY,
  &_swigt__p__dwg_object_ASSOCACTION,
  &_swigt__p__dwg_object_ASSOCACTIONPARAM,
  &_swigt__p__dwg_object_ASSOCALIGNEDDIMACTIONBODY,
  &_swigt__p__dwg_object_ASSOCARRAYACTIONBODY,
  &_swigt__p__dwg_object_ASSOCARRAYMODIFYACTIONBODY,
  &_swigt__p__dwg_object_ASSOCASMBODYACTIONPARAM,
  &_swigt__p__dwg_object_ASSOCBLENDSURFACEACTIONBODY,
  &_swigt__p__dwg_object_ASSOCCOMPOUNDACTIONPARAM,
  &_swigt__p__dwg_object_ASSOCDEPENDENCY,
  &_swigt__p__dwg_object_ASSOCDIMDEPENDENCYBODY,
  &_swigt__p__dwg_object_ASSOCEDGEACTIONPARAM,
  &_swigt__p__dwg_object_ASSOCEDGECHAMFERACTIONBODY,
  &_swigt__p__dwg_object_ASSOCEDGEFILLETACTIONBODY,
  &_swigt__p__dwg_object_ASSOCEXTENDSURFACEACTIONBODY,
  &_swigt__p__dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY,
  &_swigt__p__dwg_object_ASSOCFACEACTIONPARAM,
  &_swigt__p__dwg_object_ASSOCFILLETSURFACEACTIONBODY,
  &_swigt__p__dwg_object_ASSOCGEOMDEPENDENCY,
  &_swigt__p__dwg_object_ASSOCLOFTEDSURFACEACTIONBODY,
  &_swigt__p__dwg_object_ASSOCMLEADERACTIONBODY,
  &_swigt__p__dwg_object_ASSOCNETWORK,
  &_swigt__p__dwg_object_ASSOCNETWORKSURFACEACTIONBODY,
  &_swigt__p__dwg_object_ASSOCOBJECTACTIONPARAM,
  &_swigt__p__dwg_object_ASSOCOFFSETSURFACEACTIONBODY,
  &_swigt__p__dwg_object_ASSOCORDINATEDIMACTIONBODY,
  &_swigt__p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM,
  &_swigt__p__dwg_object_ASSOCPATCHSURFACEACTIONBODY,
  &_swigt__p__dwg_object_ASSOCPATHACTIONPARAM,
  &_swigt__p__dwg_object_ASSOCPERSSUBENTMANAGER,
  &_swigt__p__dwg_object_ASSOCPLANESURFACEACTIONBODY,
  &_swigt__p__dwg_object_ASSOCPOINTREFACTIONPARAM,
  &_swigt__p__dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY,
  &_swigt__p__dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY,
  &_swigt__p__dwg_object_ASSOCROTATEDDIMACTIONBODY,
  &_swigt__p__dwg_object_ASSOCSWEPTSURFACEACTIONBODY,
  &_swigt__p__dwg_object_ASSOCTRIMSURFACEACTIONBODY,
  &_swigt__p__dwg_object_ASSOCVALUEDEPENDENCY,
  &_swigt__p__dwg_object_ASSOCVARIABLE,
  &_swigt__p__dwg_object_ASSOCVERTEXACTIONPARAM,
  &_swigt__p__dwg_object_BLKREFOBJECTCONTEXTDATA,
  &_swigt__p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER,
  &_swigt__p__dwg_object_BLOCKALIGNMENTGRIP,
  &_swigt__p__dwg_object_BLOCKALIGNMENTPARAMETER,
  &_swigt__p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER,
  &_swigt__p__dwg_object_BLOCKARRAYACTION,
  &_swigt__p__dwg_object_BLOCKBASEPOINTPARAMETER,
  &_swigt__p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER,
  &_swigt__p__dwg_object_BLOCKFLIPACTION,
  &_swigt__p__dwg_object_BLOCKFLIPGRIP,
  &_swigt__p__dwg_object_BLOCKFLIPPARAMETER,
  &_swigt__p__dwg_object_BLOCKGRIPLOCATIONCOMPONENT,
  &_swigt__p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER,
  &_swigt__p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER,
  &_swigt__p__dwg_object_BLOCKLINEARGRIP,
  &_swigt__p__dwg_object_BLOCKLINEARPARAMETER,
  &_swigt__p__dwg_object_BLOCKLOOKUPACTION,
  &_swigt__p__dwg_object_BLOCKLOOKUPGRIP,
  &_swigt__p__dwg_object_BLOCKLOOKUPPARAMETER,
  &_swigt__p__dwg_object_BLOCKMOVEACTION,
  &_swigt__p__dwg_object_BLOCKPARAMDEPENDENCYBODY,
  &_swigt__p__dwg_object_BLOCKPOINTPARAMETER,
  &_swigt__p__dwg_object_BLOCKPOLARGRIP,
  &_swigt__p__dwg_object_BLOCKPOLARPARAMETER,
  &_swigt__p__dwg_object_BLOCKPOLARSTRETCHACTION,
  &_swigt__p__dwg_object_BLOCKPROPERTIESTABLE,
  &_swigt__p__dwg_object_BLOCKPROPERTIESTABLEGRIP,
  &_swigt__p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER,
  &_swigt__p__dwg_object_BLOCKREPRESENTATION,
  &_swigt__p__dwg_object_BLOCKROTATEACTION,
  &_swigt__p__dwg_object_BLOCKROTATIONGRIP,
  &_swigt__p__dwg_object_BLOCKROTATIONPARAMETER,
  &_swigt__p__dwg_object_BLOCKSCALEACTION,
  &_swigt__p__dwg_object_BLOCKSTRETCHACTION,
  &_swigt__p__dwg_object_BLOCKUSERPARAMETER,
  &_swigt__p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER,
  &_swigt__p__dwg_object_BLOCKVISIBILITYGRIP,
  &_swigt__p__dwg_object_BLOCKVISIBILITYPARAMETER,
  &_swigt__p__dwg_object_BLOCKXYGRIP,
  &_swigt__p__dwg_object_BLOCKXYPARAMETER,
  &_swigt__p__dwg_object_BLOCK_CONTROL,
  &_swigt__p__dwg_object_BLOCK_HEADER,
  &_swigt__p__dwg_object_BREAKDATA,
  &_swigt__p__dwg_object_BREAKPOINTREF,
  &_swigt__p__dwg_object_CELLSTYLEMAP,
  &_swigt__p__dwg_object_CONTEXTDATAMANAGER,
  &_swigt__p__dwg_object_CSACDOCUMENTOPTIONS,
  &_swigt__p__dwg_object_CURVEPATH,
  &_swigt__p__dwg_object_DATALINK,
  &_swigt__p__dwg_object_DATATABLE,
  &_swigt__p__dwg_object_DBCOLOR,
  &_swigt__p__dwg_object_DETAILVIEWSTYLE,
  &_swigt__p__dwg_object_DICTIONARY,
  &_swigt__p__dwg_object_DICTIONARYVAR,
  &_swigt__p__dwg_object_DICTIONARYWDFLT,
  &_swigt__p__dwg_object_DIMASSOC,
  &_swigt__p__dwg_object_DIMSTYLE,
  &_swigt__p__dwg_object_DIMSTYLE_CONTROL,
  &_swigt__p__dwg_object_DMDIMOBJECTCONTEXTDATA,
  &_swigt__p__dwg_object_DUMMY,
  &_swigt__p__dwg_object_DYNAMICBLOCKPROXYNODE,
  &_swigt__p__dwg_object_DYNAMICBLOCKPURGEPREVENTER,
  &_swigt__p__dwg_object_EVALUATION_GRAPH,
  &_swigt__p__dwg_object_FCFOBJECTCONTEXTDATA,
  &_swigt__p__dwg_object_FIELD,
  &_swigt__p__dwg_object_FIELDLIST,
  &_swigt__p__dwg_object_GEODATA,
  &_swigt__p__dwg_object_GEOMAPIMAGE,
  &_swigt__p__dwg_object_GRADIENT_BACKGROUND,
  &_swigt__p__dwg_object_GROUND_PLANE_BACKGROUND,
  &_swigt__p__dwg_object_GROUP,
  &_swigt__p__dwg_object_IBL_BACKGROUND,
  &_swigt__p__dwg_object_IDBUFFER,
  &_swigt__p__dwg_object_IMAGEDEF,
  &_swigt__p__dwg_object_IMAGEDEF_REACTOR,
  &_swigt__p__dwg_object_IMAGE_BACKGROUND,
  &_swigt__p__dwg_object_INDEX,
  &_swigt__p__dwg_object_LAYER,
  &_swigt__p__dwg_object_LAYERFILTER,
  &_swigt__p__dwg_object_LAYER_CONTROL,
  &_swigt__p__dwg_object_LAYER_INDEX,
  &_swigt__p__dwg_object_LAYOUT,
  &_swigt__p__dwg_object_LEADEROBJECTCONTEXTDATA,
  &_swigt__p__dwg_object_LIGHTLIST,
  &_swigt__p__dwg_object_LONG_TRANSACTION,
  &_swigt__p__dwg_object_LTYPE,
  &_swigt__p__dwg_object_LTYPE_CONTROL,
  &_swigt__p__dwg_object_MATERIAL,
  &_swigt__p__dwg_object_MENTALRAYRENDERSETTINGS,
  &_swigt__p__dwg_object_MLEADEROBJECTCONTEXTDATA,
  &_swigt__p__dwg_object_MLEADERSTYLE,
  &_swigt__p__dwg_object_MLINESTYLE,
  &_swigt__p__dwg_object_MOTIONPATH,
  &_swigt__p__dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA,
  &_swigt__p__dwg_object_MTEXTOBJECTCONTEXTDATA,
  &_swigt__p__dwg_object_NAVISWORKSMODELDEF,
  &_swigt__p__dwg_object_OBJECT_PTR,
  &_swigt__p__dwg_object_ORDDIMOBJECTCONTEXTDATA,
  &_swigt__p__dwg_object_PARTIAL_VIEWING_INDEX,
  &_swigt__p__dwg_object_PERSUBENTMGR,
  &_swigt__p__dwg_object_PLACEHOLDER,
  &_swigt__p__dwg_object_PLOTSETTINGS,
  &_swigt__p__dwg_object_POINTCLOUDCOLORMAP,
  &_swigt__p__dwg_object_POINTCLOUDDEF,
  &_swigt__p__dwg_object_POINTCLOUDDEFEX,
  &_swigt__p__dwg_object_POINTCLOUDDEF_REACTOR,
  &_swigt__p__dwg_object_POINTCLOUDDEF_REACTOR_EX,
  &_swigt__p__dwg_object_POINTPATH,
  &_swigt__p__dwg_object_PROXY_OBJECT,
  &_swigt__p__dwg_object_RADIMLGOBJECTCONTEXTDATA,
  &_swigt__p__dwg_object_RADIMOBJECTCONTEXTDATA,
  &_swigt__p__dwg_object_RAPIDRTRENDERSETTINGS,
  &_swigt__p__dwg_object_RASTERVARIABLES,
  &_swigt__p__dwg_object_RENDERENTRY,
  &_swigt__p__dwg_object_RENDERENVIRONMENT,
  &_swigt__p__dwg_object_RENDERGLOBAL,
  &_swigt__p__dwg_object_RENDERSETTINGS,
  &_swigt__p__dwg_object_SCALE,
  &_swigt__p__dwg_object_SECTIONVIEWSTYLE,
  &_swigt__p__dwg_object_SECTION_MANAGER,
  &_swigt__p__dwg_object_SECTION_SETTINGS,
  &_swigt__p__dwg_object_SKYLIGHT_BACKGROUND,
  &_swigt__p__dwg_object_SOLID_BACKGROUND,
  &_swigt__p__dwg_object_SORTENTSTABLE,
  &_swigt__p__dwg_object_SPATIAL_FILTER,
  &_swigt__p__dwg_object_SPATIAL_INDEX,
  &_swigt__p__dwg_object_STYLE,
  &_swigt__p__dwg_object_STYLE_CONTROL,
  &_swigt__p__dwg_object_SUN,
  &_swigt__p__dwg_object_SUNSTUDY,
  &_swigt__p__dwg_object_TABLECONTENT,
  &_swigt__p__dwg_object_TABLEGEOMETRY,
  &_swigt__p__dwg_object_TABLESTYLE,
  &_swigt__p__dwg_object_TEXTOBJECTCONTEXTDATA,
  &_swigt__p__dwg_object_TVDEVICEPROPERTIES,
  &_swigt__p__dwg_object_UCS,
  &_swigt__p__dwg_object_UCS_CONTROL,
  &_swigt__p__dwg_object_UNKNOWN_OBJ,
  &_swigt__p__dwg_object_VBA_PROJECT,
  &_swigt__p__dwg_object_VIEW,
  &_swigt__p__dwg_object_VIEW_CONTROL,
  &_swigt__p__dwg_object_VISUALSTYLE,
  &_swigt__p__dwg_object_VPORT,
  &_swigt__p__dwg_object_VPORT_CONTROL,
  &_swigt__p__dwg_object_VX_CONTROL,
  &_swigt__p__dwg_object_VX_TABLE_RECORD,
  &_swigt__p__dwg_object_WIPEOUTVARIABLES,
  &_swigt__p__dwg_object_XRECORD,
  &_swigt__p__dwg_object_entity,
  &_swigt__p__dwg_object_entity_tio,
  &_swigt__p__dwg_object_object,
  &_swigt__p__dwg_object_object_tio,
  &_swigt__p__dwg_object_ref,
  &_swigt__p__dwg_object_tio,
  &_swigt__p__dwg_objfreespace,
  &_swigt__p__dwg_resbuf,
  &_swigt__p__dwg_resbuf_value,
  &_swigt__p__dwg_revhistory,
  &_swigt__p__dwg_secondheader,
  &_swigt__p__dwg_secondheader_handles,
  &_swigt__p__dwg_secondheader_sections,
  &_swigt__p__dwg_section,
  &_swigt__p__dwg_security,
  &_swigt__p__dwg_struct,
  &_swigt__p__dwg_struct_fhdr,
  &_swigt__p__dwg_summaryinfo,
  &_swigt__p__dwg_template,
  &_swigt__p__dwg_time_bll,
  &_swigt__p__dwg_vbaproject,
  &_swigt__p__inthash,
  &_swigt__p_char,
  &_swigt__p_double,
  &_swigt__p_dwg_entity_DIMENSION_common,
  &_swigt__p_dwg_field_name_type_offset,
  &_swigt__p_dwg_obj_ent,
  &_swigt__p_dwg_obj_generic,
  &_swigt__p_dwg_obj_layer,
  &_swigt__p_dwg_obj_obj,
  &_swigt__p_dwg_object,
  &_swigt__p_dwg_versions,
  &_swigt__p_eed_data_t,
  &_swigt__p_eed_data_t_eed_0,
  &_swigt__p_eed_data_t_eed_0_r2007,
  &_swigt__p_eed_data_t_eed_1,
  &_swigt__p_eed_data_t_eed_10,
  &_swigt__p_eed_data_t_eed_2,
  &_swigt__p_eed_data_t_eed_3,
  &_swigt__p_eed_data_t_eed_4,
  &_swigt__p_eed_data_t_eed_40,
  &_swigt__p_eed_data_t_eed_5,
  &_swigt__p_eed_data_t_eed_70,
  &_swigt__p_eed_data_t_eed_71,
  &_swigt__p_int,
  &_swigt__p_long_long,
  &_swigt__p_p__dwg_abstractentity_UNDERLAY,
  &_swigt__p_p__dwg_abstractobject_ASSOCARRAYPARAMETERS,
  &_swigt__p_p__dwg_abstractobject_UNDERLAYDEFINITION,
  &_swigt__p_p__dwg_entity_3DFACE,
  &_swigt__p_p__dwg_entity_3DLINE,
  &_swigt__p_p__dwg_entity_3DSOLID,
  &_swigt__p_p__dwg_entity_ARC,
  &_swigt__p_p__dwg_entity_ARC_DIMENSION,
  &_swigt__p_p__dwg_entity_ATTDEF,
  &_swigt__p_p__dwg_entity_ATTRIB,
  &_swigt__p_p__dwg_entity_BLOCK,
  &_swigt__p_p__dwg_entity_CAMERA,
  &_swigt__p_p__dwg_entity_CIRCLE,
  &_swigt__p_p__dwg_entity_DIMENSION_ALIGNED,
  &_swigt__p_p__dwg_entity_DIMENSION_ANG2LN,
  &_swigt__p_p__dwg_entity_DIMENSION_ANG3PT,
  &_swigt__p_p__dwg_entity_DIMENSION_DIAMETER,
  &_swigt__p_p__dwg_entity_DIMENSION_LINEAR,
  &_swigt__p_p__dwg_entity_DIMENSION_ORDINATE,
  &_swigt__p_p__dwg_entity_DIMENSION_RADIUS,
  &_swigt__p_p__dwg_entity_ELLIPSE,
  &_swigt__p_p__dwg_entity_ENDBLK,
  &_swigt__p_p__dwg_entity_ENDREP,
  &_swigt__p_p__dwg_entity_HATCH,
  &_swigt__p_p__dwg_entity_HELIX,
  &_swigt__p_p__dwg_entity_IMAGE,
  &_swigt__p_p__dwg_entity_INSERT,
  &_swigt__p_p__dwg_entity_JUMP,
  &_swigt__p_p__dwg_entity_LARGE_RADIAL_DIMENSION,
  &_swigt__p_p__dwg_entity_LAYOUTPRINTCONFIG,
  &_swigt__p_p__dwg_entity_LEADER,
  &_swigt__p_p__dwg_entity_LIGHT,
  &_swigt__p_p__dwg_entity_LINE,
  &_swigt__p_p__dwg_entity_LOAD,
  &_swigt__p_p__dwg_entity_LWPOLYLINE,
  &_swigt__p_p__dwg_entity_MESH,
  &_swigt__p_p__dwg_entity_MINSERT,
  &_swigt__p_p__dwg_entity_MLINE,
  &_swigt__p_p__dwg_entity_MTEXT,
  &_swigt__p_p__dwg_entity_MULTILEADER,
  &_swigt__p_p__dwg_entity_OLE2FRAME,
  &_swigt__p_p__dwg_entity_OLEFRAME,
  &_swigt__p_p__dwg_entity_PLANESURFACE,
  &_swigt__p_p__dwg_entity_POINT,
  &_swigt__p_p__dwg_entity_POINTCLOUD,
  &_swigt__p_p__dwg_entity_POINTCLOUDEX,
  &_swigt__p_p__dwg_entity_POLYLINE_2D,
  &_swigt__p_p__dwg_entity_POLYLINE_3D,
  &_swigt__p_p__dwg_entity_POLYLINE_MESH,
  &_swigt__p_p__dwg_entity_POLYLINE_PFACE,
  &_swigt__p_p__dwg_entity_PROXY_ENTITY,
  &_swigt__p_p__dwg_entity_RAY,
  &_swigt__p_p__dwg_entity_REPEAT,
  &_swigt__p_p__dwg_entity_SECTIONOBJECT,
  &_swigt__p_p__dwg_entity_SEQEND,
  &_swigt__p_p__dwg_entity_SHAPE,
  &_swigt__p_p__dwg_entity_SOLID,
  &_swigt__p_p__dwg_entity_SPLINE,
  &_swigt__p_p__dwg_entity_TEXT,
  &_swigt__p_p__dwg_entity_TOLERANCE,
  &_swigt__p_p__dwg_entity_TRACE,
  &_swigt__p_p__dwg_entity_UNKNOWN_ENT,
  &_swigt__p_p__dwg_entity_VERTEX_2D,
  &_swigt__p_p__dwg_entity_VERTEX_3D,
  &_swigt__p_p__dwg_entity_VERTEX_PFACE_FACE,
  &_swigt__p_p__dwg_entity_VIEWPORT,
  &_swigt__p_p__dwg_entity_WIPEOUT,
  &_swigt__p_p__dwg_object_ACSH_BOOLEAN_CLASS,
  &_swigt__p_p__dwg_object_ACSH_BOX_CLASS,
  &_swigt__p_p__dwg_object_ACSH_BREP_CLASS,
  &_swigt__p_p__dwg_object_ACSH_CHAMFER_CLASS,
  &_swigt__p_p__dwg_object_ACSH_CONE_CLASS,
  &_swigt__p_p__dwg_object_ACSH_CYLINDER_CLASS,
  &_swigt__p_p__dwg_object_ACSH_FILLET_CLASS,
  &_swigt__p_p__dwg_object_ACSH_HISTORY_CLASS,
  &_swigt__p_p__dwg_object_ACSH_PYRAMID_CLASS,
  &_swigt__p_p__dwg_object_ACSH_SPHERE_CLASS,
  &_swigt__p_p__dwg_object_ACSH_TORUS_CLASS,
  &_swigt__p_p__dwg_object_ACSH_WEDGE_CLASS,
  &_swigt__p_p__dwg_object_ALDIMOBJECTCONTEXTDATA,
  &_swigt__p_p__dwg_object_APPID,
  &_swigt__p_p__dwg_object_APPID_CONTROL,
  &_swigt__p_p__dwg_object_ASSOC2DCONSTRAINTGROUP,
  &_swigt__p_p__dwg_object_ASSOCACTION,
  &_swigt__p_p__dwg_object_ASSOCACTIONPARAM,
  &_swigt__p_p__dwg_object_ASSOCARRAYACTIONBODY,
  &_swigt__p_p__dwg_object_ASSOCASMBODYACTIONPARAM,
  &_swigt__p_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY,
  &_swigt__p_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM,
  &_swigt__p_p__dwg_object_ASSOCDEPENDENCY,
  &_swigt__p_p__dwg_object_ASSOCDIMDEPENDENCYBODY,
  &_swigt__p_p__dwg_object_ASSOCEXTENDSURFACEACTIONBODY,
  &_swigt__p_p__dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY,
  &_swigt__p_p__dwg_object_ASSOCFACEACTIONPARAM,
  &_swigt__p_p__dwg_object_ASSOCFILLETSURFACEACTIONBODY,
  &_swigt__p_p__dwg_object_ASSOCGEOMDEPENDENCY,
  &_swigt__p_p__dwg_object_ASSOCLOFTEDSURFACEACTIONBODY,
  &_swigt__p_p__dwg_object_ASSOCNETWORK,
  &_swigt__p_p__dwg_object_ASSOCNETWORKSURFACEACTIONBODY,
  &_swigt__p_p__dwg_object_ASSOCOBJECTACTIONPARAM,
  &_swigt__p_p__dwg_object_ASSOCOFFSETSURFACEACTIONBODY,
  &_swigt__p_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM,
  &_swigt__p_p__dwg_object_ASSOCPATCHSURFACEACTIONBODY,
  &_swigt__p_p__dwg_object_ASSOCPATHACTIONPARAM,
  &_swigt__p_p__dwg_object_ASSOCPLANESURFACEACTIONBODY,
  &_swigt__p_p__dwg_object_ASSOCPOINTREFACTIONPARAM,
  &_swigt__p_p__dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY,
  &_swigt__p_p__dwg_object_ASSOCTRIMSURFACEACTIONBODY,
  &_swigt__p_p__dwg_object_ASSOCVALUEDEPENDENCY,
  &_swigt__p_p__dwg_object_ASSOCVARIABLE,
  &_swigt__p_p__dwg_object_ASSOCVERTEXACTIONPARAM,
  &_swigt__p_p__dwg_object_BLKREFOBJECTCONTEXTDATA,
  &_swigt__p_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER,
  &_swigt__p_p__dwg_object_BLOCKALIGNMENTGRIP,
  &_swigt__p_p__dwg_object_BLOCKALIGNMENTPARAMETER,
  &_swigt__p_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER,
  &_swigt__p_p__dwg_object_BLOCKARRAYACTION,
  &_swigt__p_p__dwg_object_BLOCKBASEPOINTPARAMETER,
  &_swigt__p_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER,
  &_swigt__p_p__dwg_object_BLOCKFLIPACTION,
  &_swigt__p_p__dwg_object_BLOCKFLIPGRIP,
  &_swigt__p_p__dwg_object_BLOCKFLIPPARAMETER,
  &_swigt__p_p__dwg_object_BLOCKGRIPLOCATIONCOMPONENT,
  &_swigt__p_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER,
  &_swigt__p_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER,
  &_swigt__p_p__dwg_object_BLOCKLINEARGRIP,
  &_swigt__p_p__dwg_object_BLOCKLINEARPARAMETER,
  &_swigt__p_p__dwg_object_BLOCKLOOKUPACTION,
  &_swigt__p_p__dwg_object_BLOCKLOOKUPGRIP,
  &_swigt__p_p__dwg_object_BLOCKLOOKUPPARAMETER,
  &_swigt__p_p__dwg_object_BLOCKMOVEACTION,
  &_swigt__p_p__dwg_object_BLOCKPARAMDEPENDENCYBODY,
  &_swigt__p_p__dwg_object_BLOCKPOINTPARAMETER,
  &_swigt__p_p__dwg_object_BLOCKPOLARGRIP,
  &_swigt__p_p__dwg_object_BLOCKPOLARPARAMETER,
  &_swigt__p_p__dwg_object_BLOCKPOLARSTRETCHACTION,
  &_swigt__p_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER,
  &_swigt__p_p__dwg_object_BLOCKREPRESENTATION,
  &_swigt__p_p__dwg_object_BLOCKROTATEACTION,
  &_swigt__p_p__dwg_object_BLOCKROTATIONGRIP,
  &_swigt__p_p__dwg_object_BLOCKROTATIONPARAMETER,
  &_swigt__p_p__dwg_object_BLOCKSCALEACTION,
  &_swigt__p_p__dwg_object_BLOCKSTRETCHACTION,
  &_swigt__p_p__dwg_object_BLOCKUSERPARAMETER,
  &_swigt__p_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER,
  &_swigt__p_p__dwg_object_BLOCKVISIBILITYGRIP,
  &_swigt__p_p__dwg_object_BLOCKVISIBILITYPARAMETER,
  &_swigt__p_p__dwg_object_BLOCKXYGRIP,
  &_swigt__p_p__dwg_object_BLOCKXYPARAMETER,
  &_swigt__p_p__dwg_object_BLOCK_CONTROL,
  &_swigt__p_p__dwg_object_BLOCK_HEADER,
  &_swigt__p_p__dwg_object_CELLSTYLEMAP,
  &_swigt__p_p__dwg_object_DATALINK,
  &_swigt__p_p__dwg_object_DBCOLOR,
  &_swigt__p_p__dwg_object_DETAILVIEWSTYLE,
  &_swigt__p_p__dwg_object_DICTIONARY,
  &_swigt__p_p__dwg_object_DICTIONARYVAR,
  &_swigt__p_p__dwg_object_DICTIONARYWDFLT,
  &_swigt__p_p__dwg_object_DIMSTYLE,
  &_swigt__p_p__dwg_object_DIMSTYLE_CONTROL,
  &_swigt__p_p__dwg_object_DUMMY,
  &_swigt__p_p__dwg_object_DYNAMICBLOCKPURGEPREVENTER,
  &_swigt__p_p__dwg_object_EVALUATION_GRAPH,
  &_swigt__p_p__dwg_object_FCFOBJECTCONTEXTDATA,
  &_swigt__p_p__dwg_object_FIELD,
  &_swigt__p_p__dwg_object_FIELDLIST,
  &_swigt__p_p__dwg_object_GEODATA,
  &_swigt__p_p__dwg_object_GRADIENT_BACKGROUND,
  &_swigt__p_p__dwg_object_GROUND_PLANE_BACKGROUND,
  &_swigt__p_p__dwg_object_GROUP,
  &_swigt__p_p__dwg_object_IBL_BACKGROUND,
  &_swigt__p_p__dwg_object_IDBUFFER,
  &_swigt__p_p__dwg_object_IMAGEDEF,
  &_swigt__p_p__dwg_object_IMAGEDEF_REACTOR,
  &_swigt__p_p__dwg_object_IMAGE_BACKGROUND,
  &_swigt__p_p__dwg_object_INDEX,
  &_swigt__p_p__dwg_object_LAYER,
  &_swigt__p_p__dwg_object_LAYERFILTER,
  &_swigt__p_p__dwg_object_LAYER_CONTROL,
  &_swigt__p_p__dwg_object_LAYER_INDEX,
  &_swigt__p_p__dwg_object_LAYOUT,
  &_swigt__p_p__dwg_object_LEADEROBJECTCONTEXTDATA,
  &_swigt__p_p__dwg_object_LIGHTLIST,
  &_swigt__p_p__dwg_object_LONG_TRANSACTION,
  &_swigt__p_p__dwg_object_LTYPE,
  &_swigt__p_p__dwg_object_LTYPE_CONTROL,
  &_swigt__p_p__dwg_object_MATERIAL,
  &_swigt__p_p__dwg_object_MENTALRAYRENDERSETTINGS,
  &_swigt__p_p__dwg_object_MLEADERSTYLE,
  &_swigt__p_p__dwg_object_MLINESTYLE,
  &_swigt__p_p__dwg_object_MTEXTOBJECTCONTEXTDATA,
  &_swigt__p_p__dwg_object_OBJECT_PTR,
  &_swigt__p_p__dwg_object_PARTIAL_VIEWING_INDEX,
  &_swigt__p_p__dwg_object_PLACEHOLDER,
  &_swigt__p_p__dwg_object_PLOTSETTINGS,
  &_swigt__p_p__dwg_object_POINTCLOUDCOLORMAP,
  &_swigt__p_p__dwg_object_POINTCLOUDDEF,
  &_swigt__p_p__dwg_object_POINTCLOUDDEFEX,
  &_swigt__p_p__dwg_object_POINTCLOUDDEF_REACTOR,
  &_swigt__p_p__dwg_object_POINTCLOUDDEF_REACTOR_EX,
  &_swigt__p_p__dwg_object_PROXY_OBJECT,
  &_swigt__p_p__dwg_object_RAPIDRTRENDERSETTINGS,
  &_swigt__p_p__dwg_object_RASTERVARIABLES,
  &_swigt__p_p__dwg_object_RENDERENTRY,
  &_swigt__p_p__dwg_object_RENDERENVIRONMENT,
  &_swigt__p_p__dwg_object_RENDERGLOBAL,
  &_swigt__p_p__dwg_object_RENDERSETTINGS,
  &_swigt__p_p__dwg_object_SCALE,
  &_swigt__p_p__dwg_object_SECTIONVIEWSTYLE,
  &_swigt__p_p__dwg_object_SECTION_MANAGER,
  &_swigt__p_p__dwg_object_SECTION_SETTINGS,
  &_swigt__p_p__dwg_object_SKYLIGHT_BACKGROUND,
  &_swigt__p_p__dwg_object_SOLID_BACKGROUND,
  &_swigt__p_p__dwg_object_SORTENTSTABLE,
  &_swigt__p_p__dwg_object_SPATIAL_FILTER,
  &_swigt__p_p__dwg_object_SPATIAL_INDEX,
  &_swigt__p_p__dwg_object_STYLE,
  &_swigt__p_p__dwg_object_STYLE_CONTROL,
  &_swigt__p_p__dwg_object_SUN,
  &_swigt__p_p__dwg_object_TABLEGEOMETRY,
  &_swigt__p_p__dwg_object_TABLESTYLE,
  &_swigt__p_p__dwg_object_TEXTOBJECTCONTEXTDATA,
  &_swigt__p_p__dwg_object_UCS,
  &_swigt__p_p__dwg_object_UCS_CONTROL,
  &_swigt__p_p__dwg_object_UNKNOWN_OBJ,
  &_swigt__p_p__dwg_object_VBA_PROJECT,
  &_swigt__p_p__dwg_object_VIEW,
  &_swigt__p_p__dwg_object_VIEW_CONTROL,
  &_swigt__p_p__dwg_object_VISUALSTYLE,
  &_swigt__p_p__dwg_object_VPORT,
  &_swigt__p_p__dwg_object_VPORT_CONTROL,
  &_swigt__p_p__dwg_object_VX_CONTROL,
  &_swigt__p_p__dwg_object_VX_TABLE_RECORD,
  &_swigt__p_p__dwg_object_WIPEOUTVARIABLES,
  &_swigt__p_p__dwg_object_XRECORD,
  &_swigt__p_p__dwg_object_entity,
  &_swigt__p_p__dwg_object_ref,
  &_swigt__p_p__dwg_section,
  &_swigt__p_p_char,
  &_swigt__p_r2007_file_header,
  &_swigt__p_rgbpalette,
  &_swigt__p_short,
  &_swigt__p_signed_char,
  &_swigt__p_unsigned_char,
  &_swigt__p_unsigned_int,
  &_swigt__p_unsigned_long_long,
  &_swigt__p_unsigned_short,
};

static swig_cast_info _swigc__p_DWG_CLASS_STABILITY[] = {  {&_swigt__p_DWG_CLASS_STABILITY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_DWG_COLOR_METHOD[] = {  {&_swigt__p_DWG_COLOR_METHOD, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_DWG_ENTITY_SECTIONS[] = {  {&_swigt__p_DWG_ENTITY_SECTIONS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_DWG_ERROR[] = {  {&_swigt__p_DWG_ERROR, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_DWG_HDL_CODE[] = {  {&_swigt__p_DWG_HDL_CODE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_DWG_OBJECT_SUPERTYPE[] = {  {&_swigt__p_DWG_OBJECT_SUPERTYPE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_DWG_OBJECT_TYPE[] = {  {&_swigt__p_DWG_OBJECT_TYPE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_DWG_OBJECT_TYPE_R11[] = {  {&_swigt__p_DWG_OBJECT_TYPE_R11, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_DWG_SECTION_TYPE[] = {  {&_swigt__p_DWG_SECTION_TYPE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_DWG_SECTION_TYPE_R11[] = {  {&_swigt__p_DWG_SECTION_TYPE_R11, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_DWG_SECTION_TYPE_R13[] = {  {&_swigt__p_DWG_SECTION_TYPE_R13, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_DWG_VERSION_TYPE[] = {  {&_swigt__p_DWG_VERSION_TYPE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_Dwg_Section_Info[] = {  {&_swigt__p_Dwg_Section_Info, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_Dwg_Section_InfoHdr[] = {  {&_swigt__p_Dwg_Section_InfoHdr, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_RESBUF_VALUE_TYPE[] = {  {&_swigt__p_RESBUF_VALUE_TYPE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_3DSOLID_material[] = {  {&_swigt__p__dwg_3DSOLID_material, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_3DSOLID_silhouette[] = {  {&_swigt__p__dwg_3DSOLID_silhouette, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_3DSOLID_wire[] = {  {&_swigt__p__dwg_3DSOLID_wire, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_ACSH_HistoryNode[] = {  {&_swigt__p__dwg_ACSH_HistoryNode, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_ACSH_SubentColor[] = {  {&_swigt__p__dwg_ACSH_SubentColor, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_ACSH_SubentMaterial[] = {  {&_swigt__p__dwg_ACSH_SubentMaterial, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_ACTIONBODY[] = {  {&_swigt__p__dwg_ACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_ARRAYITEMLOCATOR[] = {  {&_swigt__p__dwg_ARRAYITEMLOCATOR, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_ASSOCACTIONBODY_action[] = {  {&_swigt__p__dwg_ASSOCACTIONBODY_action, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_ASSOCACTION_Deps[] = {  {&_swigt__p__dwg_ASSOCACTION_Deps, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_ASSOCARRAYITEM[] = {  {&_swigt__p__dwg_ASSOCARRAYITEM, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_ASSOCPARAMBASEDACTIONBODY[] = {  {&_swigt__p__dwg_ASSOCPARAMBASEDACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_ASSOCSURFACEACTIONBODY[] = {  {&_swigt__p__dwg_ASSOCSURFACEACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_AcDbMTextObjectEmbedded[] = {  {&_swigt__p__dwg_AcDbMTextObjectEmbedded, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_AcDs[] = {  {&_swigt__p__dwg_AcDs, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_AcDs_Data[] = {  {&_swigt__p__dwg_AcDs_Data, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_AcDs_DataBlob[] = {  {&_swigt__p__dwg_AcDs_DataBlob, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_AcDs_DataBlob01[] = {  {&_swigt__p__dwg_AcDs_DataBlob01, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_AcDs_DataBlobRef[] = {  {&_swigt__p__dwg_AcDs_DataBlobRef, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_AcDs_DataBlobRef_Page[] = {  {&_swigt__p__dwg_AcDs_DataBlobRef_Page, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_AcDs_DataIndex[] = {  {&_swigt__p__dwg_AcDs_DataIndex, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_AcDs_DataIndex_Entry[] = {  {&_swigt__p__dwg_AcDs_DataIndex_Entry, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_AcDs_Data_Record[] = {  {&_swigt__p__dwg_AcDs_Data_Record, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_AcDs_Data_RecordHdr[] = {  {&_swigt__p__dwg_AcDs_Data_RecordHdr, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_AcDs_Schema[] = {  {&_swigt__p__dwg_AcDs_Schema, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_AcDs_SchemaData[] = {  {&_swigt__p__dwg_AcDs_SchemaData, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_AcDs_SchemaData_UProp[] = {  {&_swigt__p__dwg_AcDs_SchemaData_UProp, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_AcDs_SchemaIndex[] = {  {&_swigt__p__dwg_AcDs_SchemaIndex, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_AcDs_SchemaIndex_Prop[] = {  {&_swigt__p__dwg_AcDs_SchemaIndex_Prop, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_AcDs_Schema_Prop[] = {  {&_swigt__p__dwg_AcDs_Schema_Prop, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_AcDs_Search[] = {  {&_swigt__p__dwg_AcDs_Search, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_AcDs_Search_Data[] = {  {&_swigt__p__dwg_AcDs_Search_Data, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_AcDs_Search_IdIdx[] = {  {&_swigt__p__dwg_AcDs_Search_IdIdx, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_AcDs_Search_IdIdxs[] = {  {&_swigt__p__dwg_AcDs_Search_IdIdxs, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_AcDs_Segment[] = {  {&_swigt__p__dwg_AcDs_Segment, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_AcDs_SegmentIndex[] = {  {&_swigt__p__dwg_AcDs_SegmentIndex, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_BLOCKACTION_connectionpts[] = {  {&_swigt__p__dwg_BLOCKACTION_connectionpts, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_BLOCKLOOKUPACTION_lut[] = {  {&_swigt__p__dwg_BLOCKLOOKUPACTION_lut, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_BLOCKPARAMETER_PropInfo[] = {  {&_swigt__p__dwg_BLOCKPARAMETER_PropInfo, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_BLOCKPARAMETER_connection[] = {  {&_swigt__p__dwg_BLOCKPARAMETER_connection, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_BLOCKPARAMVALUESET[] = {  {&_swigt__p__dwg_BLOCKPARAMVALUESET, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_BLOCKSTRETCHACTION_codes[] = {  {&_swigt__p__dwg_BLOCKSTRETCHACTION_codes, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_BLOCKSTRETCHACTION_handles[] = {  {&_swigt__p__dwg_BLOCKSTRETCHACTION_handles, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_BLOCKVISIBILITYPARAMETER_state[] = {  {&_swigt__p__dwg_BLOCKVISIBILITYPARAMETER_state, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_COMPOUNDOBJECTID[] = {  {&_swigt__p__dwg_COMPOUNDOBJECTID, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_CONSTRAINTGROUPNODE[] = {  {&_swigt__p__dwg_CONSTRAINTGROUPNODE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_CONTEXTDATA_dict[] = {  {&_swigt__p__dwg_CONTEXTDATA_dict, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_CONTEXTDATA_submgr[] = {  {&_swigt__p__dwg_CONTEXTDATA_submgr, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_CellContentGeometry[] = {  {&_swigt__p__dwg_CellContentGeometry, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_CellStyle[] = {  {&_swigt__p__dwg_CellStyle, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_ColorRamp[] = {  {&_swigt__p__dwg_ColorRamp, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_ContentFormat[] = {  {&_swigt__p__dwg_ContentFormat, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_DATALINK_customdata[] = {  {&_swigt__p__dwg_DATALINK_customdata, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_DATATABLE_column[] = {  {&_swigt__p__dwg_DATATABLE_column, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_DATATABLE_row[] = {  {&_swigt__p__dwg_DATATABLE_row, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_DIMASSOC_Ref[] = {  {&_swigt__p__dwg_DIMASSOC_Ref, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_DIMENSION_common[] = {  {&_swigt__p__dwg_DIMENSION_common, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_EVAL_Edge[] = {  {&_swigt__p__dwg_EVAL_Edge, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_EVAL_Node[] = {  {&_swigt__p__dwg_EVAL_Node, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_EvalExpr[] = {  {&_swigt__p__dwg_EvalExpr, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_EvalExpr_value[] = {  {&_swigt__p__dwg_EvalExpr_value, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_EvalVariant[] = {  {&_swigt__p__dwg_EvalVariant, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_EvalVariant_u[] = {  {&_swigt__p__dwg_EvalVariant_u, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_FIELD_ChildValue[] = {  {&_swigt__p__dwg_FIELD_ChildValue, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_FileDepList_Files[] = {  {&_swigt__p__dwg_FileDepList_Files, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_FormattedTableData[] = {  {&_swigt__p__dwg_FormattedTableData, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_FormattedTableMerged[] = {  {&_swigt__p__dwg_FormattedTableMerged, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_GEODATA_meshface[] = {  {&_swigt__p__dwg_GEODATA_meshface, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_GEODATA_meshpt[] = {  {&_swigt__p__dwg_GEODATA_meshpt, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_GridFormat[] = {  {&_swigt__p__dwg_GridFormat, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_HATCH_Color[] = {  {&_swigt__p__dwg_HATCH_Color, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_HATCH_ControlPoint[] = {  {&_swigt__p__dwg_HATCH_ControlPoint, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_HATCH_DefLine[] = {  {&_swigt__p__dwg_HATCH_DefLine, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_HATCH_Path[] = {  {&_swigt__p__dwg_HATCH_Path, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_HATCH_PathSeg[] = {  {&_swigt__p__dwg_HATCH_PathSeg, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_HATCH_PolylinePath[] = {  {&_swigt__p__dwg_HATCH_PolylinePath, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_LAYER_entry[] = {  {&_swigt__p__dwg_LAYER_entry, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_LEADER_ArrowHead[] = {  {&_swigt__p__dwg_LEADER_ArrowHead, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_LEADER_BlockLabel[] = {  {&_swigt__p__dwg_LEADER_BlockLabel, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_LEADER_Break[] = {  {&_swigt__p__dwg_LEADER_Break, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_LEADER_Line[] = {  {&_swigt__p__dwg_LEADER_Line, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_LEADER_Node[] = {  {&_swigt__p__dwg_LEADER_Node, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_LIGHTLIST_light[] = {  {&_swigt__p__dwg_LIGHTLIST_light, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_LTYPE_dash[] = {  {&_swigt__p__dwg_LTYPE_dash, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_LWPOLYLINE_width[] = {  {&_swigt__p__dwg_LWPOLYLINE_width, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_LinkedData[] = {  {&_swigt__p__dwg_LinkedData, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_LinkedTableData[] = {  {&_swigt__p__dwg_LinkedTableData, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_MATERIAL_color[] = {  {&_swigt__p__dwg_MATERIAL_color, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_MATERIAL_gentexture[] = {  {&_swigt__p__dwg_MATERIAL_gentexture, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_MATERIAL_mapper[] = {  {&_swigt__p__dwg_MATERIAL_mapper, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_MESH_edge[] = {  {&_swigt__p__dwg_MESH_edge, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_MLEADER_AnnotContext[] = {  {&_swigt__p__dwg_MLEADER_AnnotContext, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_MLEADER_Content[] = {  {&_swigt__p__dwg_MLEADER_Content, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_MLEADER_Content_Block[] = {  {&_swigt__p__dwg_MLEADER_Content_Block, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_MLEADER_Content_MText[] = {  {&_swigt__p__dwg_MLEADER_Content_MText, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_MLINESTYLE_line[] = {  {&_swigt__p__dwg_MLINESTYLE_line, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_MLINESTYLE_line_lt[] = {  {&_swigt__p__dwg_MLINESTYLE_line_lt, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_MLINE_line[] = {  {&_swigt__p__dwg_MLINE_line, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_MLINE_vertex[] = {  {&_swigt__p__dwg_MLINE_vertex, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_OCD_Dimension[] = {  {&_swigt__p__dwg_OCD_Dimension, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_PARTIAL_VIEWING_INDEX_Entry[] = {  {&_swigt__p__dwg_PARTIAL_VIEWING_INDEX_Entry, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_POINTCLOUDCOLORMAP_Ramp[] = {  {&_swigt__p__dwg_POINTCLOUDCOLORMAP_Ramp, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_POINTCLOUDEX_Croppings[] = {  {&_swigt__p__dwg_POINTCLOUDEX_Croppings, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_POINTCLOUD_Clippings[] = {  {&_swigt__p__dwg_POINTCLOUD_Clippings, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_POINTCLOUD_IntensityStyle[] = {  {&_swigt__p__dwg_POINTCLOUD_IntensityStyle, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_PROXY_LWPOLYLINE[] = {  {&_swigt__p__dwg_PROXY_LWPOLYLINE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_R2004_Header[] = {  {&_swigt__p__dwg_R2004_Header, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_SECTION_geometrysettings[] = {  {&_swigt__p__dwg_SECTION_geometrysettings, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_SECTION_typesettings[] = {  {&_swigt__p__dwg_SECTION_typesettings, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_SPLINE_control_point[] = {  {&_swigt__p__dwg_SPLINE_control_point, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_SUNSTUDY_Dates[] = {  {&_swigt__p__dwg_SUNSTUDY_Dates, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_SummaryInfo_Property[] = {  {&_swigt__p__dwg_SummaryInfo_Property, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_TABLEGEOMETRY_Cell[] = {  {&_swigt__p__dwg_TABLEGEOMETRY_Cell, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_TABLESTYLE_CellStyle[] = {  {&_swigt__p__dwg_TABLESTYLE_CellStyle, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_TABLESTYLE_border[] = {  {&_swigt__p__dwg_TABLESTYLE_border, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_TABLESTYLE_rowstyles[] = {  {&_swigt__p__dwg_TABLESTYLE_rowstyles, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_TABLE_AttrDef[] = {  {&_swigt__p__dwg_TABLE_AttrDef, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_TABLE_BreakHeight[] = {  {&_swigt__p__dwg_TABLE_BreakHeight, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_TABLE_BreakRow[] = {  {&_swigt__p__dwg_TABLE_BreakRow, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_TABLE_Cell[] = {  {&_swigt__p__dwg_TABLE_Cell, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_TABLE_CustomDataItem[] = {  {&_swigt__p__dwg_TABLE_CustomDataItem, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_TABLE_value[] = {  {&_swigt__p__dwg_TABLE_value, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_TableCell[] = {  {&_swigt__p__dwg_TableCell, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_TableCellContent[] = {  {&_swigt__p__dwg_TableCellContent, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_TableCellContent_Attr[] = {  {&_swigt__p__dwg_TableCellContent_Attr, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_TableDataColumn[] = {  {&_swigt__p__dwg_TableDataColumn, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_TableRow[] = {  {&_swigt__p__dwg_TableRow, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_UCS_orthopts[] = {  {&_swigt__p__dwg_UCS_orthopts, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_VALUEPARAM[] = {  {&_swigt__p__dwg_VALUEPARAM, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_VALUEPARAM_vars[] = {  {&_swigt__p__dwg_VALUEPARAM_vars, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_abstractentity_UNDERLAY[] = {  {&_swigt__p__dwg_abstractentity_UNDERLAY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_abstractobject_ASSOCARRAYPARAMETERS[] = {  {&_swigt__p__dwg_abstractobject_ASSOCARRAYPARAMETERS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_abstractobject_UNDERLAYDEFINITION[] = {  {&_swigt__p__dwg_abstractobject_UNDERLAYDEFINITION, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_appinfo[] = {  {&_swigt__p__dwg_appinfo, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_appinfohistory[] = {  {&_swigt__p__dwg_appinfohistory, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_auxheader[] = {  {&_swigt__p__dwg_auxheader, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_binary_chunk[] = {  {&_swigt__p__dwg_binary_chunk, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_binary_chunk_u[] = {  {&_swigt__p__dwg_binary_chunk_u, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_bitcode_2bd[] = {  {&_swigt__p__dwg_bitcode_2bd, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_bitcode_2rd[] = {  {&_swigt__p__dwg_bitcode_2rd, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_bitcode_3bd[] = {  {&_swigt__p__dwg_bitcode_3bd, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_bitcode_3rd[] = {  {&_swigt__p__dwg_bitcode_3rd, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_chain[] = {  {&_swigt__p__dwg_chain, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_class[] = {  {&_swigt__p__dwg_class, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_color[] = {  {&_swigt__p__dwg_color, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_3DFACE[] = {  {&_swigt__p__dwg_entity_3DFACE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_3DLINE[] = {  {&_swigt__p__dwg_entity_3DLINE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_3DSOLID[] = {  {&_swigt__p__dwg_entity_3DSOLID, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_ALIGNMENTPARAMETERENTITY[] = {  {&_swigt__p__dwg_entity_ALIGNMENTPARAMETERENTITY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_ARC[] = {  {&_swigt__p__dwg_entity_ARC, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_ARCALIGNEDTEXT[] = {  {&_swigt__p__dwg_entity_ARCALIGNEDTEXT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_ARC_DIMENSION[] = {  {&_swigt__p__dwg_entity_ARC_DIMENSION, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_ATTDEF[] = {  {&_swigt__p__dwg_entity_ATTDEF, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_ATTRIB[] = {  {&_swigt__p__dwg_entity_ATTRIB, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_BASEPOINTPARAMETERENTITY[] = {  {&_swigt__p__dwg_entity_BASEPOINTPARAMETERENTITY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_BLOCK[] = {  {&_swigt__p__dwg_entity_BLOCK, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_CAMERA[] = {  {&_swigt__p__dwg_entity_CAMERA, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_CIRCLE[] = {  {&_swigt__p__dwg_entity_CIRCLE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_DIMENSION_ALIGNED[] = {  {&_swigt__p__dwg_entity_DIMENSION_ALIGNED, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_DIMENSION_ANG2LN[] = {  {&_swigt__p__dwg_entity_DIMENSION_ANG2LN, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_DIMENSION_ANG3PT[] = {  {&_swigt__p__dwg_entity_DIMENSION_ANG3PT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_DIMENSION_DIAMETER[] = {  {&_swigt__p__dwg_entity_DIMENSION_DIAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_DIMENSION_LINEAR[] = {  {&_swigt__p__dwg_entity_DIMENSION_LINEAR, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_DIMENSION_ORDINATE[] = {  {&_swigt__p__dwg_entity_DIMENSION_ORDINATE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_DIMENSION_RADIUS[] = {  {&_swigt__p__dwg_entity_DIMENSION_RADIUS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_ELLIPSE[] = {  {&_swigt__p__dwg_entity_ELLIPSE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_ENDBLK[] = {  {&_swigt__p__dwg_entity_ENDBLK, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_ENDREP[] = {  {&_swigt__p__dwg_entity_ENDREP, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_EXTRUDEDSURFACE[] = {  {&_swigt__p__dwg_entity_EXTRUDEDSURFACE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_FLIPGRIPENTITY[] = {  {&_swigt__p__dwg_entity_FLIPGRIPENTITY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_FLIPPARAMETERENTITY[] = {  {&_swigt__p__dwg_entity_FLIPPARAMETERENTITY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_GEOPOSITIONMARKER[] = {  {&_swigt__p__dwg_entity_GEOPOSITIONMARKER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_HATCH[] = {  {&_swigt__p__dwg_entity_HATCH, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_HELIX[] = {  {&_swigt__p__dwg_entity_HELIX, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_IMAGE[] = {  {&_swigt__p__dwg_entity_IMAGE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_INSERT[] = {  {&_swigt__p__dwg_entity_INSERT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_JUMP[] = {  {&_swigt__p__dwg_entity_JUMP, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_LARGE_RADIAL_DIMENSION[] = {  {&_swigt__p__dwg_entity_LARGE_RADIAL_DIMENSION, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_LAYOUTPRINTCONFIG[] = {  {&_swigt__p__dwg_entity_LAYOUTPRINTCONFIG, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_LEADER[] = {  {&_swigt__p__dwg_entity_LEADER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_LIGHT[] = {  {&_swigt__p__dwg_entity_LIGHT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_LINE[] = {  {&_swigt__p__dwg_entity_LINE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_LINEARGRIPENTITY[] = {  {&_swigt__p__dwg_entity_LINEARGRIPENTITY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_LINEARPARAMETERENTITY[] = {  {&_swigt__p__dwg_entity_LINEARPARAMETERENTITY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_LOAD[] = {  {&_swigt__p__dwg_entity_LOAD, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_LOFTEDSURFACE[] = {  {&_swigt__p__dwg_entity_LOFTEDSURFACE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_LWPOLYLINE[] = {  {&_swigt__p__dwg_entity_LWPOLYLINE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_MESH[] = {  {&_swigt__p__dwg_entity_MESH, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_MINSERT[] = {  {&_swigt__p__dwg_entity_MINSERT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_MLINE[] = {  {&_swigt__p__dwg_entity_MLINE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_MPOLYGON[] = {  {&_swigt__p__dwg_entity_MPOLYGON, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_MTEXT[] = {  {&_swigt__p__dwg_entity_MTEXT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_MULTILEADER[] = {  {&_swigt__p__dwg_entity_MULTILEADER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_NAVISWORKSMODEL[] = {  {&_swigt__p__dwg_entity_NAVISWORKSMODEL, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_NURBSURFACE[] = {  {&_swigt__p__dwg_entity_NURBSURFACE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_OLE2FRAME[] = {  {&_swigt__p__dwg_entity_OLE2FRAME, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_OLEFRAME[] = {  {&_swigt__p__dwg_entity_OLEFRAME, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_PLANESURFACE[] = {  {&_swigt__p__dwg_entity_PLANESURFACE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_POINT[] = {  {&_swigt__p__dwg_entity_POINT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_POINTCLOUD[] = {  {&_swigt__p__dwg_entity_POINTCLOUD, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_POINTCLOUDEX[] = {  {&_swigt__p__dwg_entity_POINTCLOUDEX, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_POINTPARAMETERENTITY[] = {  {&_swigt__p__dwg_entity_POINTPARAMETERENTITY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_POLARGRIPENTITY[] = {  {&_swigt__p__dwg_entity_POLARGRIPENTITY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_POLYLINE_2D[] = {  {&_swigt__p__dwg_entity_POLYLINE_2D, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_POLYLINE_3D[] = {  {&_swigt__p__dwg_entity_POLYLINE_3D, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_POLYLINE_MESH[] = {  {&_swigt__p__dwg_entity_POLYLINE_MESH, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_POLYLINE_PFACE[] = {  {&_swigt__p__dwg_entity_POLYLINE_PFACE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_PROXY_ENTITY[] = {  {&_swigt__p__dwg_entity_PROXY_ENTITY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_RAY[] = {  {&_swigt__p__dwg_entity_RAY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_REPEAT[] = {  {&_swigt__p__dwg_entity_REPEAT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_REVOLVEDSURFACE[] = {  {&_swigt__p__dwg_entity_REVOLVEDSURFACE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_ROTATIONGRIPENTITY[] = {  {&_swigt__p__dwg_entity_ROTATIONGRIPENTITY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_ROTATIONPARAMETERENTITY[] = {  {&_swigt__p__dwg_entity_ROTATIONPARAMETERENTITY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_RTEXT[] = {  {&_swigt__p__dwg_entity_RTEXT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_SECTIONOBJECT[] = {  {&_swigt__p__dwg_entity_SECTIONOBJECT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_SEQEND[] = {  {&_swigt__p__dwg_entity_SEQEND, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_SHAPE[] = {  {&_swigt__p__dwg_entity_SHAPE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_SOLID[] = {  {&_swigt__p__dwg_entity_SOLID, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_SPLINE[] = {  {&_swigt__p__dwg_entity_SPLINE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_SWEPTSURFACE[] = {  {&_swigt__p__dwg_entity_SWEPTSURFACE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_TABLE[] = {  {&_swigt__p__dwg_entity_TABLE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_TEXT[] = {  {&_swigt__p__dwg_entity_TEXT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_TOLERANCE[] = {  {&_swigt__p__dwg_entity_TOLERANCE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_TRACE[] = {  {&_swigt__p__dwg_entity_TRACE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_UNKNOWN_ENT[] = {  {&_swigt__p__dwg_entity_UNKNOWN_ENT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_VERTEX_2D[] = {  {&_swigt__p__dwg_entity_VERTEX_2D, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_VERTEX_3D[] = {  {&_swigt__p__dwg_entity_VERTEX_3D, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_VERTEX_PFACE_FACE[] = {  {&_swigt__p__dwg_entity_VERTEX_PFACE_FACE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_VIEWPORT[] = {  {&_swigt__p__dwg_entity_VIEWPORT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_VISIBILITYGRIPENTITY[] = {  {&_swigt__p__dwg_entity_VISIBILITYGRIPENTITY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_VISIBILITYPARAMETERENTITY[] = {  {&_swigt__p__dwg_entity_VISIBILITYPARAMETERENTITY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_WIPEOUT[] = {  {&_swigt__p__dwg_entity_WIPEOUT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_XYGRIPENTITY[] = {  {&_swigt__p__dwg_entity_XYGRIPENTITY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_XYPARAMETERENTITY[] = {  {&_swigt__p__dwg_entity_XYPARAMETERENTITY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_eed[] = {  {&_swigt__p__dwg_entity_eed, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_entity_eed_data[] = {  {&_swigt__p__dwg_entity_eed_data, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_filedeplist[] = {  {&_swigt__p__dwg_filedeplist, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_handle[] = {  {&_swigt__p__dwg_handle, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_header[] = {  {&_swigt__p__dwg_header, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_header_variables[] = {  {&_swigt__p__dwg_header_variables, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object[] = {  {&_swigt__p__dwg_object, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ACMECOMMANDHISTORY[] = {  {&_swigt__p__dwg_object_ACMECOMMANDHISTORY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ACMESCOPE[] = {  {&_swigt__p__dwg_object_ACMESCOPE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ACMESTATEMGR[] = {  {&_swigt__p__dwg_object_ACMESTATEMGR, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ACSH_BOOLEAN_CLASS[] = {  {&_swigt__p__dwg_object_ACSH_BOOLEAN_CLASS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ACSH_BOX_CLASS[] = {  {&_swigt__p__dwg_object_ACSH_BOX_CLASS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ACSH_BREP_CLASS[] = {  {&_swigt__p__dwg_object_ACSH_BREP_CLASS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ACSH_CHAMFER_CLASS[] = {  {&_swigt__p__dwg_object_ACSH_CHAMFER_CLASS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ACSH_CONE_CLASS[] = {  {&_swigt__p__dwg_object_ACSH_CONE_CLASS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ACSH_CYLINDER_CLASS[] = {  {&_swigt__p__dwg_object_ACSH_CYLINDER_CLASS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ACSH_EXTRUSION_CLASS[] = {  {&_swigt__p__dwg_object_ACSH_EXTRUSION_CLASS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ACSH_FILLET_CLASS[] = {  {&_swigt__p__dwg_object_ACSH_FILLET_CLASS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ACSH_HISTORY_CLASS[] = {  {&_swigt__p__dwg_object_ACSH_HISTORY_CLASS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ACSH_LOFT_CLASS[] = {  {&_swigt__p__dwg_object_ACSH_LOFT_CLASS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ACSH_PYRAMID_CLASS[] = {  {&_swigt__p__dwg_object_ACSH_PYRAMID_CLASS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ACSH_REVOLVE_CLASS[] = {  {&_swigt__p__dwg_object_ACSH_REVOLVE_CLASS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ACSH_SPHERE_CLASS[] = {  {&_swigt__p__dwg_object_ACSH_SPHERE_CLASS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ACSH_SWEEP_CLASS[] = {  {&_swigt__p__dwg_object_ACSH_SWEEP_CLASS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ACSH_TORUS_CLASS[] = {  {&_swigt__p__dwg_object_ACSH_TORUS_CLASS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ACSH_WEDGE_CLASS[] = {  {&_swigt__p__dwg_object_ACSH_WEDGE_CLASS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ALDIMOBJECTCONTEXTDATA[] = {  {&_swigt__p__dwg_object_ALDIMOBJECTCONTEXTDATA, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ANGDIMOBJECTCONTEXTDATA[] = {  {&_swigt__p__dwg_object_ANGDIMOBJECTCONTEXTDATA, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ANNOTSCALEOBJECTCONTEXTDATA[] = {  {&_swigt__p__dwg_object_ANNOTSCALEOBJECTCONTEXTDATA, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_APPID[] = {  {&_swigt__p__dwg_object_APPID, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_APPID_CONTROL[] = {  {&_swigt__p__dwg_object_APPID_CONTROL, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOC2DCONSTRAINTGROUP[] = {  {&_swigt__p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOC3POINTANGULARDIMACTIONBODY[] = {  {&_swigt__p__dwg_object_ASSOC3POINTANGULARDIMACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCACTION[] = {  {&_swigt__p__dwg_object_ASSOCACTION, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCACTIONPARAM[] = {  {&_swigt__p__dwg_object_ASSOCACTIONPARAM, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCALIGNEDDIMACTIONBODY[] = {  {&_swigt__p__dwg_object_ASSOCALIGNEDDIMACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCARRAYACTIONBODY[] = {  {&_swigt__p__dwg_object_ASSOCARRAYACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCARRAYMODIFYACTIONBODY[] = {  {&_swigt__p__dwg_object_ASSOCARRAYMODIFYACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCASMBODYACTIONPARAM[] = {  {&_swigt__p__dwg_object_ASSOCASMBODYACTIONPARAM, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCBLENDSURFACEACTIONBODY[] = {  {&_swigt__p__dwg_object_ASSOCBLENDSURFACEACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCCOMPOUNDACTIONPARAM[] = {  {&_swigt__p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCDEPENDENCY[] = {  {&_swigt__p__dwg_object_ASSOCDEPENDENCY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCDIMDEPENDENCYBODY[] = {  {&_swigt__p__dwg_object_ASSOCDIMDEPENDENCYBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCEDGEACTIONPARAM[] = {  {&_swigt__p__dwg_object_ASSOCEDGEACTIONPARAM, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCEDGECHAMFERACTIONBODY[] = {  {&_swigt__p__dwg_object_ASSOCEDGECHAMFERACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCEDGEFILLETACTIONBODY[] = {  {&_swigt__p__dwg_object_ASSOCEDGEFILLETACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCEXTENDSURFACEACTIONBODY[] = {  {&_swigt__p__dwg_object_ASSOCEXTENDSURFACEACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY[] = {  {&_swigt__p__dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCFACEACTIONPARAM[] = {  {&_swigt__p__dwg_object_ASSOCFACEACTIONPARAM, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCFILLETSURFACEACTIONBODY[] = {  {&_swigt__p__dwg_object_ASSOCFILLETSURFACEACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCGEOMDEPENDENCY[] = {  {&_swigt__p__dwg_object_ASSOCGEOMDEPENDENCY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCLOFTEDSURFACEACTIONBODY[] = {  {&_swigt__p__dwg_object_ASSOCLOFTEDSURFACEACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCMLEADERACTIONBODY[] = {  {&_swigt__p__dwg_object_ASSOCMLEADERACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCNETWORK[] = {  {&_swigt__p__dwg_object_ASSOCNETWORK, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCNETWORKSURFACEACTIONBODY[] = {  {&_swigt__p__dwg_object_ASSOCNETWORKSURFACEACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCOBJECTACTIONPARAM[] = {  {&_swigt__p__dwg_object_ASSOCOBJECTACTIONPARAM, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCOFFSETSURFACEACTIONBODY[] = {  {&_swigt__p__dwg_object_ASSOCOFFSETSURFACEACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCORDINATEDIMACTIONBODY[] = {  {&_swigt__p__dwg_object_ASSOCORDINATEDIMACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM[] = {  {&_swigt__p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCPATCHSURFACEACTIONBODY[] = {  {&_swigt__p__dwg_object_ASSOCPATCHSURFACEACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCPATHACTIONPARAM[] = {  {&_swigt__p__dwg_object_ASSOCPATHACTIONPARAM, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCPERSSUBENTMANAGER[] = {  {&_swigt__p__dwg_object_ASSOCPERSSUBENTMANAGER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCPLANESURFACEACTIONBODY[] = {  {&_swigt__p__dwg_object_ASSOCPLANESURFACEACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCPOINTREFACTIONPARAM[] = {  {&_swigt__p__dwg_object_ASSOCPOINTREFACTIONPARAM, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY[] = {  {&_swigt__p__dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY[] = {  {&_swigt__p__dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCROTATEDDIMACTIONBODY[] = {  {&_swigt__p__dwg_object_ASSOCROTATEDDIMACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCSWEPTSURFACEACTIONBODY[] = {  {&_swigt__p__dwg_object_ASSOCSWEPTSURFACEACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCTRIMSURFACEACTIONBODY[] = {  {&_swigt__p__dwg_object_ASSOCTRIMSURFACEACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCVALUEDEPENDENCY[] = {  {&_swigt__p__dwg_object_ASSOCVALUEDEPENDENCY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCVARIABLE[] = {  {&_swigt__p__dwg_object_ASSOCVARIABLE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ASSOCVERTEXACTIONPARAM[] = {  {&_swigt__p__dwg_object_ASSOCVERTEXACTIONPARAM, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLKREFOBJECTCONTEXTDATA[] = {  {&_swigt__p__dwg_object_BLKREFOBJECTCONTEXTDATA, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER[] = {  {&_swigt__p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKALIGNMENTGRIP[] = {  {&_swigt__p__dwg_object_BLOCKALIGNMENTGRIP, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKALIGNMENTPARAMETER[] = {  {&_swigt__p__dwg_object_BLOCKALIGNMENTPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER[] = {  {&_swigt__p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKARRAYACTION[] = {  {&_swigt__p__dwg_object_BLOCKARRAYACTION, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKBASEPOINTPARAMETER[] = {  {&_swigt__p__dwg_object_BLOCKBASEPOINTPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER[] = {  {&_swigt__p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKFLIPACTION[] = {  {&_swigt__p__dwg_object_BLOCKFLIPACTION, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKFLIPGRIP[] = {  {&_swigt__p__dwg_object_BLOCKFLIPGRIP, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKFLIPPARAMETER[] = {  {&_swigt__p__dwg_object_BLOCKFLIPPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKGRIPLOCATIONCOMPONENT[] = {  {&_swigt__p__dwg_object_BLOCKGRIPLOCATIONCOMPONENT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER[] = {  {&_swigt__p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER[] = {  {&_swigt__p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKLINEARGRIP[] = {  {&_swigt__p__dwg_object_BLOCKLINEARGRIP, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKLINEARPARAMETER[] = {  {&_swigt__p__dwg_object_BLOCKLINEARPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKLOOKUPACTION[] = {  {&_swigt__p__dwg_object_BLOCKLOOKUPACTION, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKLOOKUPGRIP[] = {  {&_swigt__p__dwg_object_BLOCKLOOKUPGRIP, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKLOOKUPPARAMETER[] = {  {&_swigt__p__dwg_object_BLOCKLOOKUPPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKMOVEACTION[] = {  {&_swigt__p__dwg_object_BLOCKMOVEACTION, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKPARAMDEPENDENCYBODY[] = {  {&_swigt__p__dwg_object_BLOCKPARAMDEPENDENCYBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKPOINTPARAMETER[] = {  {&_swigt__p__dwg_object_BLOCKPOINTPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKPOLARGRIP[] = {  {&_swigt__p__dwg_object_BLOCKPOLARGRIP, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKPOLARPARAMETER[] = {  {&_swigt__p__dwg_object_BLOCKPOLARPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKPOLARSTRETCHACTION[] = {  {&_swigt__p__dwg_object_BLOCKPOLARSTRETCHACTION, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKPROPERTIESTABLE[] = {  {&_swigt__p__dwg_object_BLOCKPROPERTIESTABLE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKPROPERTIESTABLEGRIP[] = {  {&_swigt__p__dwg_object_BLOCKPROPERTIESTABLEGRIP, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER[] = {  {&_swigt__p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKREPRESENTATION[] = {  {&_swigt__p__dwg_object_BLOCKREPRESENTATION, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKROTATEACTION[] = {  {&_swigt__p__dwg_object_BLOCKROTATEACTION, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKROTATIONGRIP[] = {  {&_swigt__p__dwg_object_BLOCKROTATIONGRIP, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKROTATIONPARAMETER[] = {  {&_swigt__p__dwg_object_BLOCKROTATIONPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKSCALEACTION[] = {  {&_swigt__p__dwg_object_BLOCKSCALEACTION, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKSTRETCHACTION[] = {  {&_swigt__p__dwg_object_BLOCKSTRETCHACTION, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKUSERPARAMETER[] = {  {&_swigt__p__dwg_object_BLOCKUSERPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER[] = {  {&_swigt__p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKVISIBILITYGRIP[] = {  {&_swigt__p__dwg_object_BLOCKVISIBILITYGRIP, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKVISIBILITYPARAMETER[] = {  {&_swigt__p__dwg_object_BLOCKVISIBILITYPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKXYGRIP[] = {  {&_swigt__p__dwg_object_BLOCKXYGRIP, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCKXYPARAMETER[] = {  {&_swigt__p__dwg_object_BLOCKXYPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCK_CONTROL[] = {  {&_swigt__p__dwg_object_BLOCK_CONTROL, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BLOCK_HEADER[] = {  {&_swigt__p__dwg_object_BLOCK_HEADER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BREAKDATA[] = {  {&_swigt__p__dwg_object_BREAKDATA, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_BREAKPOINTREF[] = {  {&_swigt__p__dwg_object_BREAKPOINTREF, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_CELLSTYLEMAP[] = {  {&_swigt__p__dwg_object_CELLSTYLEMAP, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_CONTEXTDATAMANAGER[] = {  {&_swigt__p__dwg_object_CONTEXTDATAMANAGER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_CSACDOCUMENTOPTIONS[] = {  {&_swigt__p__dwg_object_CSACDOCUMENTOPTIONS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_CURVEPATH[] = {  {&_swigt__p__dwg_object_CURVEPATH, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_DATALINK[] = {  {&_swigt__p__dwg_object_DATALINK, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_DATATABLE[] = {  {&_swigt__p__dwg_object_DATATABLE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_DBCOLOR[] = {  {&_swigt__p__dwg_object_DBCOLOR, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_DETAILVIEWSTYLE[] = {  {&_swigt__p__dwg_object_DETAILVIEWSTYLE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_DICTIONARY[] = {  {&_swigt__p__dwg_object_DICTIONARY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_DICTIONARYVAR[] = {  {&_swigt__p__dwg_object_DICTIONARYVAR, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_DICTIONARYWDFLT[] = {  {&_swigt__p__dwg_object_DICTIONARYWDFLT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_DIMASSOC[] = {  {&_swigt__p__dwg_object_DIMASSOC, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_DIMSTYLE[] = {  {&_swigt__p__dwg_object_DIMSTYLE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_DIMSTYLE_CONTROL[] = {  {&_swigt__p__dwg_object_DIMSTYLE_CONTROL, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_DMDIMOBJECTCONTEXTDATA[] = {  {&_swigt__p__dwg_object_DMDIMOBJECTCONTEXTDATA, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_DUMMY[] = {  {&_swigt__p__dwg_object_DUMMY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_DYNAMICBLOCKPROXYNODE[] = {  {&_swigt__p__dwg_object_DYNAMICBLOCKPROXYNODE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_DYNAMICBLOCKPURGEPREVENTER[] = {  {&_swigt__p__dwg_object_DYNAMICBLOCKPURGEPREVENTER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_EVALUATION_GRAPH[] = {  {&_swigt__p__dwg_object_EVALUATION_GRAPH, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_FCFOBJECTCONTEXTDATA[] = {  {&_swigt__p__dwg_object_FCFOBJECTCONTEXTDATA, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_FIELD[] = {  {&_swigt__p__dwg_object_FIELD, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_FIELDLIST[] = {  {&_swigt__p__dwg_object_FIELDLIST, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_GEODATA[] = {  {&_swigt__p__dwg_object_GEODATA, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_GEOMAPIMAGE[] = {  {&_swigt__p__dwg_object_GEOMAPIMAGE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_GRADIENT_BACKGROUND[] = {  {&_swigt__p__dwg_object_GRADIENT_BACKGROUND, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_GROUND_PLANE_BACKGROUND[] = {  {&_swigt__p__dwg_object_GROUND_PLANE_BACKGROUND, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_GROUP[] = {  {&_swigt__p__dwg_object_GROUP, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_IBL_BACKGROUND[] = {  {&_swigt__p__dwg_object_IBL_BACKGROUND, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_IDBUFFER[] = {  {&_swigt__p__dwg_object_IDBUFFER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_IMAGEDEF[] = {  {&_swigt__p__dwg_object_IMAGEDEF, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_IMAGEDEF_REACTOR[] = {  {&_swigt__p__dwg_object_IMAGEDEF_REACTOR, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_IMAGE_BACKGROUND[] = {  {&_swigt__p__dwg_object_IMAGE_BACKGROUND, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_INDEX[] = {  {&_swigt__p__dwg_object_INDEX, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_LAYER[] = {  {&_swigt__p__dwg_object_LAYER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_LAYERFILTER[] = {  {&_swigt__p__dwg_object_LAYERFILTER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_LAYER_CONTROL[] = {  {&_swigt__p__dwg_object_LAYER_CONTROL, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_LAYER_INDEX[] = {  {&_swigt__p__dwg_object_LAYER_INDEX, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_LAYOUT[] = {  {&_swigt__p__dwg_object_LAYOUT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_LEADEROBJECTCONTEXTDATA[] = {  {&_swigt__p__dwg_object_LEADEROBJECTCONTEXTDATA, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_LIGHTLIST[] = {  {&_swigt__p__dwg_object_LIGHTLIST, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_LONG_TRANSACTION[] = {  {&_swigt__p__dwg_object_LONG_TRANSACTION, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_LTYPE[] = {  {&_swigt__p__dwg_object_LTYPE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_LTYPE_CONTROL[] = {  {&_swigt__p__dwg_object_LTYPE_CONTROL, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_MATERIAL[] = {  {&_swigt__p__dwg_object_MATERIAL, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_MENTALRAYRENDERSETTINGS[] = {  {&_swigt__p__dwg_object_MENTALRAYRENDERSETTINGS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_MLEADEROBJECTCONTEXTDATA[] = {  {&_swigt__p__dwg_object_MLEADEROBJECTCONTEXTDATA, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_MLEADERSTYLE[] = {  {&_swigt__p__dwg_object_MLEADERSTYLE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_MLINESTYLE[] = {  {&_swigt__p__dwg_object_MLINESTYLE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_MOTIONPATH[] = {  {&_swigt__p__dwg_object_MOTIONPATH, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA[] = {  {&_swigt__p__dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_MTEXTOBJECTCONTEXTDATA[] = {  {&_swigt__p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_NAVISWORKSMODELDEF[] = {  {&_swigt__p__dwg_object_NAVISWORKSMODELDEF, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_OBJECT_PTR[] = {  {&_swigt__p__dwg_object_OBJECT_PTR, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ORDDIMOBJECTCONTEXTDATA[] = {  {&_swigt__p__dwg_object_ORDDIMOBJECTCONTEXTDATA, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_PARTIAL_VIEWING_INDEX[] = {  {&_swigt__p__dwg_object_PARTIAL_VIEWING_INDEX, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_PERSUBENTMGR[] = {  {&_swigt__p__dwg_object_PERSUBENTMGR, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_PLACEHOLDER[] = {  {&_swigt__p__dwg_object_PLACEHOLDER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_PLOTSETTINGS[] = {  {&_swigt__p__dwg_object_PLOTSETTINGS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_POINTCLOUDCOLORMAP[] = {  {&_swigt__p__dwg_object_POINTCLOUDCOLORMAP, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_POINTCLOUDDEF[] = {  {&_swigt__p__dwg_object_POINTCLOUDDEF, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_POINTCLOUDDEFEX[] = {  {&_swigt__p__dwg_object_POINTCLOUDDEFEX, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_POINTCLOUDDEF_REACTOR[] = {  {&_swigt__p__dwg_object_POINTCLOUDDEF_REACTOR, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_POINTCLOUDDEF_REACTOR_EX[] = {  {&_swigt__p__dwg_object_POINTCLOUDDEF_REACTOR_EX, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_POINTPATH[] = {  {&_swigt__p__dwg_object_POINTPATH, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_PROXY_OBJECT[] = {  {&_swigt__p__dwg_object_PROXY_OBJECT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_RADIMLGOBJECTCONTEXTDATA[] = {  {&_swigt__p__dwg_object_RADIMLGOBJECTCONTEXTDATA, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_RADIMOBJECTCONTEXTDATA[] = {  {&_swigt__p__dwg_object_RADIMOBJECTCONTEXTDATA, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_RAPIDRTRENDERSETTINGS[] = {  {&_swigt__p__dwg_object_RAPIDRTRENDERSETTINGS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_RASTERVARIABLES[] = {  {&_swigt__p__dwg_object_RASTERVARIABLES, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_RENDERENTRY[] = {  {&_swigt__p__dwg_object_RENDERENTRY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_RENDERENVIRONMENT[] = {  {&_swigt__p__dwg_object_RENDERENVIRONMENT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_RENDERGLOBAL[] = {  {&_swigt__p__dwg_object_RENDERGLOBAL, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_RENDERSETTINGS[] = {  {&_swigt__p__dwg_object_RENDERSETTINGS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_SCALE[] = {  {&_swigt__p__dwg_object_SCALE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_SECTIONVIEWSTYLE[] = {  {&_swigt__p__dwg_object_SECTIONVIEWSTYLE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_SECTION_MANAGER[] = {  {&_swigt__p__dwg_object_SECTION_MANAGER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_SECTION_SETTINGS[] = {  {&_swigt__p__dwg_object_SECTION_SETTINGS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_SKYLIGHT_BACKGROUND[] = {  {&_swigt__p__dwg_object_SKYLIGHT_BACKGROUND, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_SOLID_BACKGROUND[] = {  {&_swigt__p__dwg_object_SOLID_BACKGROUND, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_SORTENTSTABLE[] = {  {&_swigt__p__dwg_object_SORTENTSTABLE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_SPATIAL_FILTER[] = {  {&_swigt__p__dwg_object_SPATIAL_FILTER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_SPATIAL_INDEX[] = {  {&_swigt__p__dwg_object_SPATIAL_INDEX, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_STYLE[] = {  {&_swigt__p__dwg_object_STYLE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_STYLE_CONTROL[] = {  {&_swigt__p__dwg_object_STYLE_CONTROL, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_SUN[] = {  {&_swigt__p__dwg_object_SUN, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_SUNSTUDY[] = {  {&_swigt__p__dwg_object_SUNSTUDY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_TABLECONTENT[] = {  {&_swigt__p__dwg_object_TABLECONTENT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_TABLEGEOMETRY[] = {  {&_swigt__p__dwg_object_TABLEGEOMETRY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_TABLESTYLE[] = {  {&_swigt__p__dwg_object_TABLESTYLE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_TEXTOBJECTCONTEXTDATA[] = {  {&_swigt__p__dwg_object_TEXTOBJECTCONTEXTDATA, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_TVDEVICEPROPERTIES[] = {  {&_swigt__p__dwg_object_TVDEVICEPROPERTIES, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_UCS[] = {  {&_swigt__p__dwg_object_UCS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_UCS_CONTROL[] = {  {&_swigt__p__dwg_object_UCS_CONTROL, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_UNKNOWN_OBJ[] = {  {&_swigt__p__dwg_object_UNKNOWN_OBJ, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_VBA_PROJECT[] = {  {&_swigt__p__dwg_object_VBA_PROJECT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_VIEW[] = {  {&_swigt__p__dwg_object_VIEW, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_VIEW_CONTROL[] = {  {&_swigt__p__dwg_object_VIEW_CONTROL, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_VISUALSTYLE[] = {  {&_swigt__p__dwg_object_VISUALSTYLE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_VPORT[] = {  {&_swigt__p__dwg_object_VPORT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_VPORT_CONTROL[] = {  {&_swigt__p__dwg_object_VPORT_CONTROL, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_VX_CONTROL[] = {  {&_swigt__p__dwg_object_VX_CONTROL, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_VX_TABLE_RECORD[] = {  {&_swigt__p__dwg_object_VX_TABLE_RECORD, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_WIPEOUTVARIABLES[] = {  {&_swigt__p__dwg_object_WIPEOUTVARIABLES, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_XRECORD[] = {  {&_swigt__p__dwg_object_XRECORD, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_entity[] = {  {&_swigt__p__dwg_object_entity, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_entity_tio[] = {  {&_swigt__p__dwg_object_entity_tio, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_object[] = {  {&_swigt__p__dwg_object_object, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_object_tio[] = {  {&_swigt__p__dwg_object_object_tio, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_ref[] = {  {&_swigt__p__dwg_object_ref, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_object_tio[] = {  {&_swigt__p__dwg_object_tio, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_objfreespace[] = {  {&_swigt__p__dwg_objfreespace, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_resbuf[] = {  {&_swigt__p__dwg_resbuf, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_resbuf_value[] = {  {&_swigt__p__dwg_resbuf_value, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_revhistory[] = {  {&_swigt__p__dwg_revhistory, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_secondheader[] = {  {&_swigt__p__dwg_secondheader, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_secondheader_handles[] = {  {&_swigt__p__dwg_secondheader_handles, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_secondheader_sections[] = {  {&_swigt__p__dwg_secondheader_sections, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_section[] = {  {&_swigt__p__dwg_section, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_security[] = {  {&_swigt__p__dwg_security, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_struct[] = {  {&_swigt__p__dwg_struct, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_struct_fhdr[] = {  {&_swigt__p__dwg_struct_fhdr, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_summaryinfo[] = {  {&_swigt__p__dwg_summaryinfo, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_template[] = {  {&_swigt__p__dwg_template, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_time_bll[] = {  {&_swigt__p__dwg_time_bll, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__dwg_vbaproject[] = {  {&_swigt__p__dwg_vbaproject, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p__inthash[] = {  {&_swigt__p__inthash, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_char[] = {  {&_swigt__p_char, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_double[] = {  {&_swigt__p_double, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_dwg_entity_DIMENSION_common[] = {  {&_swigt__p_dwg_entity_DIMENSION_common, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_dwg_field_name_type_offset[] = {  {&_swigt__p_dwg_field_name_type_offset, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_dwg_obj_ent[] = {  {&_swigt__p_dwg_obj_ent, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_dwg_obj_generic[] = {  {&_swigt__p_dwg_obj_generic, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_dwg_obj_layer[] = {  {&_swigt__p_dwg_obj_layer, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_dwg_obj_obj[] = {  {&_swigt__p_dwg_obj_obj, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_dwg_object[] = {  {&_swigt__p_dwg_object, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_dwg_versions[] = {  {&_swigt__p_dwg_versions, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_eed_data_t[] = {  {&_swigt__p_eed_data_t, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_eed_data_t_eed_0[] = {  {&_swigt__p_eed_data_t_eed_0, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_eed_data_t_eed_0_r2007[] = {  {&_swigt__p_eed_data_t_eed_0_r2007, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_eed_data_t_eed_1[] = {  {&_swigt__p_eed_data_t_eed_1, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_eed_data_t_eed_10[] = {  {&_swigt__p_eed_data_t_eed_10, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_eed_data_t_eed_2[] = {  {&_swigt__p_eed_data_t_eed_2, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_eed_data_t_eed_3[] = {  {&_swigt__p_eed_data_t_eed_3, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_eed_data_t_eed_4[] = {  {&_swigt__p_eed_data_t_eed_4, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_eed_data_t_eed_40[] = {  {&_swigt__p_eed_data_t_eed_40, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_eed_data_t_eed_5[] = {  {&_swigt__p_eed_data_t_eed_5, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_eed_data_t_eed_70[] = {  {&_swigt__p_eed_data_t_eed_70, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_eed_data_t_eed_71[] = {  {&_swigt__p_eed_data_t_eed_71, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_int[] = {  {&_swigt__p_int, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_long_long[] = {  {&_swigt__p_long_long, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_abstractentity_UNDERLAY[] = {  {&_swigt__p_p__dwg_abstractentity_UNDERLAY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_abstractobject_ASSOCARRAYPARAMETERS[] = {  {&_swigt__p_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_abstractobject_UNDERLAYDEFINITION[] = {  {&_swigt__p_p__dwg_abstractobject_UNDERLAYDEFINITION, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_3DFACE[] = {  {&_swigt__p_p__dwg_entity_3DFACE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_3DLINE[] = {  {&_swigt__p_p__dwg_entity_3DLINE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_3DSOLID[] = {  {&_swigt__p_p__dwg_entity_3DSOLID, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_ARC[] = {  {&_swigt__p_p__dwg_entity_ARC, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_ARC_DIMENSION[] = {  {&_swigt__p_p__dwg_entity_ARC_DIMENSION, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_ATTDEF[] = {  {&_swigt__p_p__dwg_entity_ATTDEF, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_ATTRIB[] = {  {&_swigt__p_p__dwg_entity_ATTRIB, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_BLOCK[] = {  {&_swigt__p_p__dwg_entity_BLOCK, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_CAMERA[] = {  {&_swigt__p_p__dwg_entity_CAMERA, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_CIRCLE[] = {  {&_swigt__p_p__dwg_entity_CIRCLE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_DIMENSION_ALIGNED[] = {  {&_swigt__p_p__dwg_entity_DIMENSION_ALIGNED, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_DIMENSION_ANG2LN[] = {  {&_swigt__p_p__dwg_entity_DIMENSION_ANG2LN, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_DIMENSION_ANG3PT[] = {  {&_swigt__p_p__dwg_entity_DIMENSION_ANG3PT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_DIMENSION_DIAMETER[] = {  {&_swigt__p_p__dwg_entity_DIMENSION_DIAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_DIMENSION_LINEAR[] = {  {&_swigt__p_p__dwg_entity_DIMENSION_LINEAR, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_DIMENSION_ORDINATE[] = {  {&_swigt__p_p__dwg_entity_DIMENSION_ORDINATE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_DIMENSION_RADIUS[] = {  {&_swigt__p_p__dwg_entity_DIMENSION_RADIUS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_ELLIPSE[] = {  {&_swigt__p_p__dwg_entity_ELLIPSE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_ENDBLK[] = {  {&_swigt__p_p__dwg_entity_ENDBLK, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_ENDREP[] = {  {&_swigt__p_p__dwg_entity_ENDREP, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_HATCH[] = {  {&_swigt__p_p__dwg_entity_HATCH, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_HELIX[] = {  {&_swigt__p_p__dwg_entity_HELIX, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_IMAGE[] = {  {&_swigt__p_p__dwg_entity_IMAGE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_INSERT[] = {  {&_swigt__p_p__dwg_entity_INSERT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_JUMP[] = {  {&_swigt__p_p__dwg_entity_JUMP, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_LARGE_RADIAL_DIMENSION[] = {  {&_swigt__p_p__dwg_entity_LARGE_RADIAL_DIMENSION, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_LAYOUTPRINTCONFIG[] = {  {&_swigt__p_p__dwg_entity_LAYOUTPRINTCONFIG, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_LEADER[] = {  {&_swigt__p_p__dwg_entity_LEADER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_LIGHT[] = {  {&_swigt__p_p__dwg_entity_LIGHT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_LINE[] = {  {&_swigt__p_p__dwg_entity_LINE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_LOAD[] = {  {&_swigt__p_p__dwg_entity_LOAD, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_LWPOLYLINE[] = {  {&_swigt__p_p__dwg_entity_LWPOLYLINE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_MESH[] = {  {&_swigt__p_p__dwg_entity_MESH, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_MINSERT[] = {  {&_swigt__p_p__dwg_entity_MINSERT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_MLINE[] = {  {&_swigt__p_p__dwg_entity_MLINE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_MTEXT[] = {  {&_swigt__p_p__dwg_entity_MTEXT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_MULTILEADER[] = {  {&_swigt__p_p__dwg_entity_MULTILEADER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_OLE2FRAME[] = {  {&_swigt__p_p__dwg_entity_OLE2FRAME, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_OLEFRAME[] = {  {&_swigt__p_p__dwg_entity_OLEFRAME, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_PLANESURFACE[] = {  {&_swigt__p_p__dwg_entity_PLANESURFACE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_POINT[] = {  {&_swigt__p_p__dwg_entity_POINT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_POINTCLOUD[] = {  {&_swigt__p_p__dwg_entity_POINTCLOUD, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_POINTCLOUDEX[] = {  {&_swigt__p_p__dwg_entity_POINTCLOUDEX, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_POLYLINE_2D[] = {  {&_swigt__p_p__dwg_entity_POLYLINE_2D, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_POLYLINE_3D[] = {  {&_swigt__p_p__dwg_entity_POLYLINE_3D, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_POLYLINE_MESH[] = {  {&_swigt__p_p__dwg_entity_POLYLINE_MESH, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_POLYLINE_PFACE[] = {  {&_swigt__p_p__dwg_entity_POLYLINE_PFACE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_PROXY_ENTITY[] = {  {&_swigt__p_p__dwg_entity_PROXY_ENTITY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_RAY[] = {  {&_swigt__p_p__dwg_entity_RAY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_REPEAT[] = {  {&_swigt__p_p__dwg_entity_REPEAT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_SECTIONOBJECT[] = {  {&_swigt__p_p__dwg_entity_SECTIONOBJECT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_SEQEND[] = {  {&_swigt__p_p__dwg_entity_SEQEND, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_SHAPE[] = {  {&_swigt__p_p__dwg_entity_SHAPE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_SOLID[] = {  {&_swigt__p_p__dwg_entity_SOLID, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_SPLINE[] = {  {&_swigt__p_p__dwg_entity_SPLINE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_TEXT[] = {  {&_swigt__p_p__dwg_entity_TEXT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_TOLERANCE[] = {  {&_swigt__p_p__dwg_entity_TOLERANCE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_TRACE[] = {  {&_swigt__p_p__dwg_entity_TRACE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_UNKNOWN_ENT[] = {  {&_swigt__p_p__dwg_entity_UNKNOWN_ENT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_VERTEX_2D[] = {  {&_swigt__p_p__dwg_entity_VERTEX_2D, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_VERTEX_3D[] = {  {&_swigt__p_p__dwg_entity_VERTEX_3D, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_VERTEX_PFACE_FACE[] = {  {&_swigt__p_p__dwg_entity_VERTEX_PFACE_FACE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_VIEWPORT[] = {  {&_swigt__p_p__dwg_entity_VIEWPORT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_entity_WIPEOUT[] = {  {&_swigt__p_p__dwg_entity_WIPEOUT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ACSH_BOOLEAN_CLASS[] = {  {&_swigt__p_p__dwg_object_ACSH_BOOLEAN_CLASS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ACSH_BOX_CLASS[] = {  {&_swigt__p_p__dwg_object_ACSH_BOX_CLASS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ACSH_BREP_CLASS[] = {  {&_swigt__p_p__dwg_object_ACSH_BREP_CLASS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ACSH_CHAMFER_CLASS[] = {  {&_swigt__p_p__dwg_object_ACSH_CHAMFER_CLASS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ACSH_CONE_CLASS[] = {  {&_swigt__p_p__dwg_object_ACSH_CONE_CLASS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ACSH_CYLINDER_CLASS[] = {  {&_swigt__p_p__dwg_object_ACSH_CYLINDER_CLASS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ACSH_FILLET_CLASS[] = {  {&_swigt__p_p__dwg_object_ACSH_FILLET_CLASS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ACSH_HISTORY_CLASS[] = {  {&_swigt__p_p__dwg_object_ACSH_HISTORY_CLASS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ACSH_PYRAMID_CLASS[] = {  {&_swigt__p_p__dwg_object_ACSH_PYRAMID_CLASS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ACSH_SPHERE_CLASS[] = {  {&_swigt__p_p__dwg_object_ACSH_SPHERE_CLASS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ACSH_TORUS_CLASS[] = {  {&_swigt__p_p__dwg_object_ACSH_TORUS_CLASS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ACSH_WEDGE_CLASS[] = {  {&_swigt__p_p__dwg_object_ACSH_WEDGE_CLASS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ALDIMOBJECTCONTEXTDATA[] = {  {&_swigt__p_p__dwg_object_ALDIMOBJECTCONTEXTDATA, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_APPID[] = {  {&_swigt__p_p__dwg_object_APPID, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_APPID_CONTROL[] = {  {&_swigt__p_p__dwg_object_APPID_CONTROL, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ASSOC2DCONSTRAINTGROUP[] = {  {&_swigt__p_p__dwg_object_ASSOC2DCONSTRAINTGROUP, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ASSOCACTION[] = {  {&_swigt__p_p__dwg_object_ASSOCACTION, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ASSOCACTIONPARAM[] = {  {&_swigt__p_p__dwg_object_ASSOCACTIONPARAM, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ASSOCARRAYACTIONBODY[] = {  {&_swigt__p_p__dwg_object_ASSOCARRAYACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ASSOCASMBODYACTIONPARAM[] = {  {&_swigt__p_p__dwg_object_ASSOCASMBODYACTIONPARAM, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY[] = {  {&_swigt__p_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM[] = {  {&_swigt__p_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ASSOCDEPENDENCY[] = {  {&_swigt__p_p__dwg_object_ASSOCDEPENDENCY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ASSOCDIMDEPENDENCYBODY[] = {  {&_swigt__p_p__dwg_object_ASSOCDIMDEPENDENCYBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ASSOCEXTENDSURFACEACTIONBODY[] = {  {&_swigt__p_p__dwg_object_ASSOCEXTENDSURFACEACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY[] = {  {&_swigt__p_p__dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ASSOCFACEACTIONPARAM[] = {  {&_swigt__p_p__dwg_object_ASSOCFACEACTIONPARAM, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ASSOCFILLETSURFACEACTIONBODY[] = {  {&_swigt__p_p__dwg_object_ASSOCFILLETSURFACEACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ASSOCGEOMDEPENDENCY[] = {  {&_swigt__p_p__dwg_object_ASSOCGEOMDEPENDENCY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ASSOCLOFTEDSURFACEACTIONBODY[] = {  {&_swigt__p_p__dwg_object_ASSOCLOFTEDSURFACEACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ASSOCNETWORK[] = {  {&_swigt__p_p__dwg_object_ASSOCNETWORK, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ASSOCNETWORKSURFACEACTIONBODY[] = {  {&_swigt__p_p__dwg_object_ASSOCNETWORKSURFACEACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ASSOCOBJECTACTIONPARAM[] = {  {&_swigt__p_p__dwg_object_ASSOCOBJECTACTIONPARAM, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ASSOCOFFSETSURFACEACTIONBODY[] = {  {&_swigt__p_p__dwg_object_ASSOCOFFSETSURFACEACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM[] = {  {&_swigt__p_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ASSOCPATCHSURFACEACTIONBODY[] = {  {&_swigt__p_p__dwg_object_ASSOCPATCHSURFACEACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ASSOCPATHACTIONPARAM[] = {  {&_swigt__p_p__dwg_object_ASSOCPATHACTIONPARAM, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ASSOCPLANESURFACEACTIONBODY[] = {  {&_swigt__p_p__dwg_object_ASSOCPLANESURFACEACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ASSOCPOINTREFACTIONPARAM[] = {  {&_swigt__p_p__dwg_object_ASSOCPOINTREFACTIONPARAM, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY[] = {  {&_swigt__p_p__dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ASSOCTRIMSURFACEACTIONBODY[] = {  {&_swigt__p_p__dwg_object_ASSOCTRIMSURFACEACTIONBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ASSOCVALUEDEPENDENCY[] = {  {&_swigt__p_p__dwg_object_ASSOCVALUEDEPENDENCY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ASSOCVARIABLE[] = {  {&_swigt__p_p__dwg_object_ASSOCVARIABLE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ASSOCVERTEXACTIONPARAM[] = {  {&_swigt__p_p__dwg_object_ASSOCVERTEXACTIONPARAM, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLKREFOBJECTCONTEXTDATA[] = {  {&_swigt__p_p__dwg_object_BLKREFOBJECTCONTEXTDATA, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER[] = {  {&_swigt__p_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKALIGNMENTGRIP[] = {  {&_swigt__p_p__dwg_object_BLOCKALIGNMENTGRIP, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKALIGNMENTPARAMETER[] = {  {&_swigt__p_p__dwg_object_BLOCKALIGNMENTPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER[] = {  {&_swigt__p_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKARRAYACTION[] = {  {&_swigt__p_p__dwg_object_BLOCKARRAYACTION, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKBASEPOINTPARAMETER[] = {  {&_swigt__p_p__dwg_object_BLOCKBASEPOINTPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER[] = {  {&_swigt__p_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKFLIPACTION[] = {  {&_swigt__p_p__dwg_object_BLOCKFLIPACTION, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKFLIPGRIP[] = {  {&_swigt__p_p__dwg_object_BLOCKFLIPGRIP, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKFLIPPARAMETER[] = {  {&_swigt__p_p__dwg_object_BLOCKFLIPPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKGRIPLOCATIONCOMPONENT[] = {  {&_swigt__p_p__dwg_object_BLOCKGRIPLOCATIONCOMPONENT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER[] = {  {&_swigt__p_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER[] = {  {&_swigt__p_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKLINEARGRIP[] = {  {&_swigt__p_p__dwg_object_BLOCKLINEARGRIP, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKLINEARPARAMETER[] = {  {&_swigt__p_p__dwg_object_BLOCKLINEARPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKLOOKUPACTION[] = {  {&_swigt__p_p__dwg_object_BLOCKLOOKUPACTION, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKLOOKUPGRIP[] = {  {&_swigt__p_p__dwg_object_BLOCKLOOKUPGRIP, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKLOOKUPPARAMETER[] = {  {&_swigt__p_p__dwg_object_BLOCKLOOKUPPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKMOVEACTION[] = {  {&_swigt__p_p__dwg_object_BLOCKMOVEACTION, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKPARAMDEPENDENCYBODY[] = {  {&_swigt__p_p__dwg_object_BLOCKPARAMDEPENDENCYBODY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKPOINTPARAMETER[] = {  {&_swigt__p_p__dwg_object_BLOCKPOINTPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKPOLARGRIP[] = {  {&_swigt__p_p__dwg_object_BLOCKPOLARGRIP, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKPOLARPARAMETER[] = {  {&_swigt__p_p__dwg_object_BLOCKPOLARPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKPOLARSTRETCHACTION[] = {  {&_swigt__p_p__dwg_object_BLOCKPOLARSTRETCHACTION, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER[] = {  {&_swigt__p_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKREPRESENTATION[] = {  {&_swigt__p_p__dwg_object_BLOCKREPRESENTATION, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKROTATEACTION[] = {  {&_swigt__p_p__dwg_object_BLOCKROTATEACTION, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKROTATIONGRIP[] = {  {&_swigt__p_p__dwg_object_BLOCKROTATIONGRIP, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKROTATIONPARAMETER[] = {  {&_swigt__p_p__dwg_object_BLOCKROTATIONPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKSCALEACTION[] = {  {&_swigt__p_p__dwg_object_BLOCKSCALEACTION, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKSTRETCHACTION[] = {  {&_swigt__p_p__dwg_object_BLOCKSTRETCHACTION, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKUSERPARAMETER[] = {  {&_swigt__p_p__dwg_object_BLOCKUSERPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER[] = {  {&_swigt__p_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKVISIBILITYGRIP[] = {  {&_swigt__p_p__dwg_object_BLOCKVISIBILITYGRIP, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKVISIBILITYPARAMETER[] = {  {&_swigt__p_p__dwg_object_BLOCKVISIBILITYPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKXYGRIP[] = {  {&_swigt__p_p__dwg_object_BLOCKXYGRIP, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCKXYPARAMETER[] = {  {&_swigt__p_p__dwg_object_BLOCKXYPARAMETER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCK_CONTROL[] = {  {&_swigt__p_p__dwg_object_BLOCK_CONTROL, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_BLOCK_HEADER[] = {  {&_swigt__p_p__dwg_object_BLOCK_HEADER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_CELLSTYLEMAP[] = {  {&_swigt__p_p__dwg_object_CELLSTYLEMAP, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_DATALINK[] = {  {&_swigt__p_p__dwg_object_DATALINK, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_DBCOLOR[] = {  {&_swigt__p_p__dwg_object_DBCOLOR, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_DETAILVIEWSTYLE[] = {  {&_swigt__p_p__dwg_object_DETAILVIEWSTYLE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_DICTIONARY[] = {  {&_swigt__p_p__dwg_object_DICTIONARY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_DICTIONARYVAR[] = {  {&_swigt__p_p__dwg_object_DICTIONARYVAR, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_DICTIONARYWDFLT[] = {  {&_swigt__p_p__dwg_object_DICTIONARYWDFLT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_DIMSTYLE[] = {  {&_swigt__p_p__dwg_object_DIMSTYLE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_DIMSTYLE_CONTROL[] = {  {&_swigt__p_p__dwg_object_DIMSTYLE_CONTROL, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_DUMMY[] = {  {&_swigt__p_p__dwg_object_DUMMY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_DYNAMICBLOCKPURGEPREVENTER[] = {  {&_swigt__p_p__dwg_object_DYNAMICBLOCKPURGEPREVENTER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_EVALUATION_GRAPH[] = {  {&_swigt__p_p__dwg_object_EVALUATION_GRAPH, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_FCFOBJECTCONTEXTDATA[] = {  {&_swigt__p_p__dwg_object_FCFOBJECTCONTEXTDATA, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_FIELD[] = {  {&_swigt__p_p__dwg_object_FIELD, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_FIELDLIST[] = {  {&_swigt__p_p__dwg_object_FIELDLIST, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_GEODATA[] = {  {&_swigt__p_p__dwg_object_GEODATA, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_GRADIENT_BACKGROUND[] = {  {&_swigt__p_p__dwg_object_GRADIENT_BACKGROUND, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_GROUND_PLANE_BACKGROUND[] = {  {&_swigt__p_p__dwg_object_GROUND_PLANE_BACKGROUND, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_GROUP[] = {  {&_swigt__p_p__dwg_object_GROUP, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_IBL_BACKGROUND[] = {  {&_swigt__p_p__dwg_object_IBL_BACKGROUND, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_IDBUFFER[] = {  {&_swigt__p_p__dwg_object_IDBUFFER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_IMAGEDEF[] = {  {&_swigt__p_p__dwg_object_IMAGEDEF, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_IMAGEDEF_REACTOR[] = {  {&_swigt__p_p__dwg_object_IMAGEDEF_REACTOR, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_IMAGE_BACKGROUND[] = {  {&_swigt__p_p__dwg_object_IMAGE_BACKGROUND, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_INDEX[] = {  {&_swigt__p_p__dwg_object_INDEX, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_LAYER[] = {  {&_swigt__p_p__dwg_object_LAYER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_LAYERFILTER[] = {  {&_swigt__p_p__dwg_object_LAYERFILTER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_LAYER_CONTROL[] = {  {&_swigt__p_p__dwg_object_LAYER_CONTROL, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_LAYER_INDEX[] = {  {&_swigt__p_p__dwg_object_LAYER_INDEX, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_LAYOUT[] = {  {&_swigt__p_p__dwg_object_LAYOUT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_LEADEROBJECTCONTEXTDATA[] = {  {&_swigt__p_p__dwg_object_LEADEROBJECTCONTEXTDATA, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_LIGHTLIST[] = {  {&_swigt__p_p__dwg_object_LIGHTLIST, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_LONG_TRANSACTION[] = {  {&_swigt__p_p__dwg_object_LONG_TRANSACTION, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_LTYPE[] = {  {&_swigt__p_p__dwg_object_LTYPE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_LTYPE_CONTROL[] = {  {&_swigt__p_p__dwg_object_LTYPE_CONTROL, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_MATERIAL[] = {  {&_swigt__p_p__dwg_object_MATERIAL, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_MENTALRAYRENDERSETTINGS[] = {  {&_swigt__p_p__dwg_object_MENTALRAYRENDERSETTINGS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_MLEADERSTYLE[] = {  {&_swigt__p_p__dwg_object_MLEADERSTYLE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_MLINESTYLE[] = {  {&_swigt__p_p__dwg_object_MLINESTYLE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_MTEXTOBJECTCONTEXTDATA[] = {  {&_swigt__p_p__dwg_object_MTEXTOBJECTCONTEXTDATA, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_OBJECT_PTR[] = {  {&_swigt__p_p__dwg_object_OBJECT_PTR, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_PARTIAL_VIEWING_INDEX[] = {  {&_swigt__p_p__dwg_object_PARTIAL_VIEWING_INDEX, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_PLACEHOLDER[] = {  {&_swigt__p_p__dwg_object_PLACEHOLDER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_PLOTSETTINGS[] = {  {&_swigt__p_p__dwg_object_PLOTSETTINGS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_POINTCLOUDCOLORMAP[] = {  {&_swigt__p_p__dwg_object_POINTCLOUDCOLORMAP, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_POINTCLOUDDEF[] = {  {&_swigt__p_p__dwg_object_POINTCLOUDDEF, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_POINTCLOUDDEFEX[] = {  {&_swigt__p_p__dwg_object_POINTCLOUDDEFEX, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_POINTCLOUDDEF_REACTOR[] = {  {&_swigt__p_p__dwg_object_POINTCLOUDDEF_REACTOR, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_POINTCLOUDDEF_REACTOR_EX[] = {  {&_swigt__p_p__dwg_object_POINTCLOUDDEF_REACTOR_EX, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_PROXY_OBJECT[] = {  {&_swigt__p_p__dwg_object_PROXY_OBJECT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_RAPIDRTRENDERSETTINGS[] = {  {&_swigt__p_p__dwg_object_RAPIDRTRENDERSETTINGS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_RASTERVARIABLES[] = {  {&_swigt__p_p__dwg_object_RASTERVARIABLES, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_RENDERENTRY[] = {  {&_swigt__p_p__dwg_object_RENDERENTRY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_RENDERENVIRONMENT[] = {  {&_swigt__p_p__dwg_object_RENDERENVIRONMENT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_RENDERGLOBAL[] = {  {&_swigt__p_p__dwg_object_RENDERGLOBAL, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_RENDERSETTINGS[] = {  {&_swigt__p_p__dwg_object_RENDERSETTINGS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_SCALE[] = {  {&_swigt__p_p__dwg_object_SCALE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_SECTIONVIEWSTYLE[] = {  {&_swigt__p_p__dwg_object_SECTIONVIEWSTYLE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_SECTION_MANAGER[] = {  {&_swigt__p_p__dwg_object_SECTION_MANAGER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_SECTION_SETTINGS[] = {  {&_swigt__p_p__dwg_object_SECTION_SETTINGS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_SKYLIGHT_BACKGROUND[] = {  {&_swigt__p_p__dwg_object_SKYLIGHT_BACKGROUND, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_SOLID_BACKGROUND[] = {  {&_swigt__p_p__dwg_object_SOLID_BACKGROUND, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_SORTENTSTABLE[] = {  {&_swigt__p_p__dwg_object_SORTENTSTABLE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_SPATIAL_FILTER[] = {  {&_swigt__p_p__dwg_object_SPATIAL_FILTER, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_SPATIAL_INDEX[] = {  {&_swigt__p_p__dwg_object_SPATIAL_INDEX, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_STYLE[] = {  {&_swigt__p_p__dwg_object_STYLE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_STYLE_CONTROL[] = {  {&_swigt__p_p__dwg_object_STYLE_CONTROL, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_SUN[] = {  {&_swigt__p_p__dwg_object_SUN, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_TABLEGEOMETRY[] = {  {&_swigt__p_p__dwg_object_TABLEGEOMETRY, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_TABLESTYLE[] = {  {&_swigt__p_p__dwg_object_TABLESTYLE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_TEXTOBJECTCONTEXTDATA[] = {  {&_swigt__p_p__dwg_object_TEXTOBJECTCONTEXTDATA, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_UCS[] = {  {&_swigt__p_p__dwg_object_UCS, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_UCS_CONTROL[] = {  {&_swigt__p_p__dwg_object_UCS_CONTROL, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_UNKNOWN_OBJ[] = {  {&_swigt__p_p__dwg_object_UNKNOWN_OBJ, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_VBA_PROJECT[] = {  {&_swigt__p_p__dwg_object_VBA_PROJECT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_VIEW[] = {  {&_swigt__p_p__dwg_object_VIEW, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_VIEW_CONTROL[] = {  {&_swigt__p_p__dwg_object_VIEW_CONTROL, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_VISUALSTYLE[] = {  {&_swigt__p_p__dwg_object_VISUALSTYLE, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_VPORT[] = {  {&_swigt__p_p__dwg_object_VPORT, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_VPORT_CONTROL[] = {  {&_swigt__p_p__dwg_object_VPORT_CONTROL, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_VX_CONTROL[] = {  {&_swigt__p_p__dwg_object_VX_CONTROL, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_VX_TABLE_RECORD[] = {  {&_swigt__p_p__dwg_object_VX_TABLE_RECORD, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_WIPEOUTVARIABLES[] = {  {&_swigt__p_p__dwg_object_WIPEOUTVARIABLES, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_XRECORD[] = {  {&_swigt__p_p__dwg_object_XRECORD, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_entity[] = {  {&_swigt__p_p__dwg_object_entity, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_object_ref[] = {  {&_swigt__p_p__dwg_object_ref, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p__dwg_section[] = {  {&_swigt__p_p__dwg_section, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_p_char[] = {  {&_swigt__p_p_char, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_r2007_file_header[] = {  {&_swigt__p_r2007_file_header, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_rgbpalette[] = {  {&_swigt__p_rgbpalette, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_short[] = {  {&_swigt__p_short, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_signed_char[] = {  {&_swigt__p_signed_char, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_unsigned_char[] = {  {&_swigt__p_unsigned_char, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_unsigned_int[] = {  {&_swigt__p_unsigned_int, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_unsigned_long_long[] = {  {&_swigt__p_unsigned_long_long, 0, 0, 0},{0, 0, 0, 0}};
static swig_cast_info _swigc__p_unsigned_short[] = {  {&_swigt__p_unsigned_short, 0, 0, 0},{0, 0, 0, 0}};

static swig_cast_info *swig_cast_initial[] = {
  _swigc__p_DWG_CLASS_STABILITY,
  _swigc__p_DWG_COLOR_METHOD,
  _swigc__p_DWG_ENTITY_SECTIONS,
  _swigc__p_DWG_ERROR,
  _swigc__p_DWG_HDL_CODE,
  _swigc__p_DWG_OBJECT_SUPERTYPE,
  _swigc__p_DWG_OBJECT_TYPE,
  _swigc__p_DWG_OBJECT_TYPE_R11,
  _swigc__p_DWG_SECTION_TYPE,
  _swigc__p_DWG_SECTION_TYPE_R11,
  _swigc__p_DWG_SECTION_TYPE_R13,
  _swigc__p_DWG_VERSION_TYPE,
  _swigc__p_Dwg_Section_Info,
  _swigc__p_Dwg_Section_InfoHdr,
  _swigc__p_RESBUF_VALUE_TYPE,
  _swigc__p__dwg_3DSOLID_material,
  _swigc__p__dwg_3DSOLID_silhouette,
  _swigc__p__dwg_3DSOLID_wire,
  _swigc__p__dwg_ACSH_HistoryNode,
  _swigc__p__dwg_ACSH_SubentColor,
  _swigc__p__dwg_ACSH_SubentMaterial,
  _swigc__p__dwg_ACTIONBODY,
  _swigc__p__dwg_ARRAYITEMLOCATOR,
  _swigc__p__dwg_ASSOCACTIONBODY_action,
  _swigc__p__dwg_ASSOCACTION_Deps,
  _swigc__p__dwg_ASSOCARRAYITEM,
  _swigc__p__dwg_ASSOCPARAMBASEDACTIONBODY,
  _swigc__p__dwg_ASSOCSURFACEACTIONBODY,
  _swigc__p__dwg_AcDbMTextObjectEmbedded,
  _swigc__p__dwg_AcDs,
  _swigc__p__dwg_AcDs_Data,
  _swigc__p__dwg_AcDs_DataBlob,
  _swigc__p__dwg_AcDs_DataBlob01,
  _swigc__p__dwg_AcDs_DataBlobRef,
  _swigc__p__dwg_AcDs_DataBlobRef_Page,
  _swigc__p__dwg_AcDs_DataIndex,
  _swigc__p__dwg_AcDs_DataIndex_Entry,
  _swigc__p__dwg_AcDs_Data_Record,
  _swigc__p__dwg_AcDs_Data_RecordHdr,
  _swigc__p__dwg_AcDs_Schema,
  _swigc__p__dwg_AcDs_SchemaData,
  _swigc__p__dwg_AcDs_SchemaData_UProp,
  _swigc__p__dwg_AcDs_SchemaIndex,
  _swigc__p__dwg_AcDs_SchemaIndex_Prop,
  _swigc__p__dwg_AcDs_Schema_Prop,
  _swigc__p__dwg_AcDs_Search,
  _swigc__p__dwg_AcDs_Search_Data,
  _swigc__p__dwg_AcDs_Search_IdIdx,
  _swigc__p__dwg_AcDs_Search_IdIdxs,
  _swigc__p__dwg_AcDs_Segment,
  _swigc__p__dwg_AcDs_SegmentIndex,
  _swigc__p__dwg_BLOCKACTION_connectionpts,
  _swigc__p__dwg_BLOCKLOOKUPACTION_lut,
  _swigc__p__dwg_BLOCKPARAMETER_PropInfo,
  _swigc__p__dwg_BLOCKPARAMETER_connection,
  _swigc__p__dwg_BLOCKPARAMVALUESET,
  _swigc__p__dwg_BLOCKSTRETCHACTION_codes,
  _swigc__p__dwg_BLOCKSTRETCHACTION_handles,
  _swigc__p__dwg_BLOCKVISIBILITYPARAMETER_state,
  _swigc__p__dwg_COMPOUNDOBJECTID,
  _swigc__p__dwg_CONSTRAINTGROUPNODE,
  _swigc__p__dwg_CONTEXTDATA_dict,
  _swigc__p__dwg_CONTEXTDATA_submgr,
  _swigc__p__dwg_CellContentGeometry,
  _swigc__p__dwg_CellStyle,
  _swigc__p__dwg_ColorRamp,
  _swigc__p__dwg_ContentFormat,
  _swigc__p__dwg_DATALINK_customdata,
  _swigc__p__dwg_DATATABLE_column,
  _swigc__p__dwg_DATATABLE_row,
  _swigc__p__dwg_DIMASSOC_Ref,
  _swigc__p__dwg_DIMENSION_common,
  _swigc__p__dwg_EVAL_Edge,
  _swigc__p__dwg_EVAL_Node,
  _swigc__p__dwg_EvalExpr,
  _swigc__p__dwg_EvalExpr_value,
  _swigc__p__dwg_EvalVariant,
  _swigc__p__dwg_EvalVariant_u,
  _swigc__p__dwg_FIELD_ChildValue,
  _swigc__p__dwg_FileDepList_Files,
  _swigc__p__dwg_FormattedTableData,
  _swigc__p__dwg_FormattedTableMerged,
  _swigc__p__dwg_GEODATA_meshface,
  _swigc__p__dwg_GEODATA_meshpt,
  _swigc__p__dwg_GridFormat,
  _swigc__p__dwg_HATCH_Color,
  _swigc__p__dwg_HATCH_ControlPoint,
  _swigc__p__dwg_HATCH_DefLine,
  _swigc__p__dwg_HATCH_Path,
  _swigc__p__dwg_HATCH_PathSeg,
  _swigc__p__dwg_HATCH_PolylinePath,
  _swigc__p__dwg_LAYER_entry,
  _swigc__p__dwg_LEADER_ArrowHead,
  _swigc__p__dwg_LEADER_BlockLabel,
  _swigc__p__dwg_LEADER_Break,
  _swigc__p__dwg_LEADER_Line,
  _swigc__p__dwg_LEADER_Node,
  _swigc__p__dwg_LIGHTLIST_light,
  _swigc__p__dwg_LTYPE_dash,
  _swigc__p__dwg_LWPOLYLINE_width,
  _swigc__p__dwg_LinkedData,
  _swigc__p__dwg_LinkedTableData,
  _swigc__p__dwg_MATERIAL_color,
  _swigc__p__dwg_MATERIAL_gentexture,
  _swigc__p__dwg_MATERIAL_mapper,
  _swigc__p__dwg_MESH_edge,
  _swigc__p__dwg_MLEADER_AnnotContext,
  _swigc__p__dwg_MLEADER_Content,
  _swigc__p__dwg_MLEADER_Content_Block,
  _swigc__p__dwg_MLEADER_Content_MText,
  _swigc__p__dwg_MLINESTYLE_line,
  _swigc__p__dwg_MLINESTYLE_line_lt,
  _swigc__p__dwg_MLINE_line,
  _swigc__p__dwg_MLINE_vertex,
  _swigc__p__dwg_OCD_Dimension,
  _swigc__p__dwg_PARTIAL_VIEWING_INDEX_Entry,
  _swigc__p__dwg_POINTCLOUDCOLORMAP_Ramp,
  _swigc__p__dwg_POINTCLOUDEX_Croppings,
  _swigc__p__dwg_POINTCLOUD_Clippings,
  _swigc__p__dwg_POINTCLOUD_IntensityStyle,
  _swigc__p__dwg_PROXY_LWPOLYLINE,
  _swigc__p__dwg_R2004_Header,
  _swigc__p__dwg_SECTION_geometrysettings,
  _swigc__p__dwg_SECTION_typesettings,
  _swigc__p__dwg_SPLINE_control_point,
  _swigc__p__dwg_SUNSTUDY_Dates,
  _swigc__p__dwg_SummaryInfo_Property,
  _swigc__p__dwg_TABLEGEOMETRY_Cell,
  _swigc__p__dwg_TABLESTYLE_CellStyle,
  _swigc__p__dwg_TABLESTYLE_border,
  _swigc__p__dwg_TABLESTYLE_rowstyles,
  _swigc__p__dwg_TABLE_AttrDef,
  _swigc__p__dwg_TABLE_BreakHeight,
  _swigc__p__dwg_TABLE_BreakRow,
  _swigc__p__dwg_TABLE_Cell,
  _swigc__p__dwg_TABLE_CustomDataItem,
  _swigc__p__dwg_TABLE_value,
  _swigc__p__dwg_TableCell,
  _swigc__p__dwg_TableCellContent,
  _swigc__p__dwg_TableCellContent_Attr,
  _swigc__p__dwg_TableDataColumn,
  _swigc__p__dwg_TableRow,
  _swigc__p__dwg_UCS_orthopts,
  _swigc__p__dwg_VALUEPARAM,
  _swigc__p__dwg_VALUEPARAM_vars,
  _swigc__p__dwg_abstractentity_UNDERLAY,
  _swigc__p__dwg_abstractobject_ASSOCARRAYPARAMETERS,
  _swigc__p__dwg_abstractobject_UNDERLAYDEFINITION,
  _swigc__p__dwg_appinfo,
  _swigc__p__dwg_appinfohistory,
  _swigc__p__dwg_auxheader,
  _swigc__p__dwg_binary_chunk,
  _swigc__p__dwg_binary_chunk_u,
  _swigc__p__dwg_bitcode_2bd,
  _swigc__p__dwg_bitcode_2rd,
  _swigc__p__dwg_bitcode_3bd,
  _swigc__p__dwg_bitcode_3rd,
  _swigc__p__dwg_chain,
  _swigc__p__dwg_class,
  _swigc__p__dwg_color,
  _swigc__p__dwg_entity_3DFACE,
  _swigc__p__dwg_entity_3DLINE,
  _swigc__p__dwg_entity_3DSOLID,
  _swigc__p__dwg_entity_ALIGNMENTPARAMETERENTITY,
  _swigc__p__dwg_entity_ARC,
  _swigc__p__dwg_entity_ARCALIGNEDTEXT,
  _swigc__p__dwg_entity_ARC_DIMENSION,
  _swigc__p__dwg_entity_ATTDEF,
  _swigc__p__dwg_entity_ATTRIB,
  _swigc__p__dwg_entity_BASEPOINTPARAMETERENTITY,
  _swigc__p__dwg_entity_BLOCK,
  _swigc__p__dwg_entity_CAMERA,
  _swigc__p__dwg_entity_CIRCLE,
  _swigc__p__dwg_entity_DIMENSION_ALIGNED,
  _swigc__p__dwg_entity_DIMENSION_ANG2LN,
  _swigc__p__dwg_entity_DIMENSION_ANG3PT,
  _swigc__p__dwg_entity_DIMENSION_DIAMETER,
  _swigc__p__dwg_entity_DIMENSION_LINEAR,
  _swigc__p__dwg_entity_DIMENSION_ORDINATE,
  _swigc__p__dwg_entity_DIMENSION_RADIUS,
  _swigc__p__dwg_entity_ELLIPSE,
  _swigc__p__dwg_entity_ENDBLK,
  _swigc__p__dwg_entity_ENDREP,
  _swigc__p__dwg_entity_EXTRUDEDSURFACE,
  _swigc__p__dwg_entity_FLIPGRIPENTITY,
  _swigc__p__dwg_entity_FLIPPARAMETERENTITY,
  _swigc__p__dwg_entity_GEOPOSITIONMARKER,
  _swigc__p__dwg_entity_HATCH,
  _swigc__p__dwg_entity_HELIX,
  _swigc__p__dwg_entity_IMAGE,
  _swigc__p__dwg_entity_INSERT,
  _swigc__p__dwg_entity_JUMP,
  _swigc__p__dwg_entity_LARGE_RADIAL_DIMENSION,
  _swigc__p__dwg_entity_LAYOUTPRINTCONFIG,
  _swigc__p__dwg_entity_LEADER,
  _swigc__p__dwg_entity_LIGHT,
  _swigc__p__dwg_entity_LINE,
  _swigc__p__dwg_entity_LINEARGRIPENTITY,
  _swigc__p__dwg_entity_LINEARPARAMETERENTITY,
  _swigc__p__dwg_entity_LOAD,
  _swigc__p__dwg_entity_LOFTEDSURFACE,
  _swigc__p__dwg_entity_LWPOLYLINE,
  _swigc__p__dwg_entity_MESH,
  _swigc__p__dwg_entity_MINSERT,
  _swigc__p__dwg_entity_MLINE,
  _swigc__p__dwg_entity_MPOLYGON,
  _swigc__p__dwg_entity_MTEXT,
  _swigc__p__dwg_entity_MULTILEADER,
  _swigc__p__dwg_entity_NAVISWORKSMODEL,
  _swigc__p__dwg_entity_NURBSURFACE,
  _swigc__p__dwg_entity_OLE2FRAME,
  _swigc__p__dwg_entity_OLEFRAME,
  _swigc__p__dwg_entity_PLANESURFACE,
  _swigc__p__dwg_entity_POINT,
  _swigc__p__dwg_entity_POINTCLOUD,
  _swigc__p__dwg_entity_POINTCLOUDEX,
  _swigc__p__dwg_entity_POINTPARAMETERENTITY,
  _swigc__p__dwg_entity_POLARGRIPENTITY,
  _swigc__p__dwg_entity_POLYLINE_2D,
  _swigc__p__dwg_entity_POLYLINE_3D,
  _swigc__p__dwg_entity_POLYLINE_MESH,
  _swigc__p__dwg_entity_POLYLINE_PFACE,
  _swigc__p__dwg_entity_PROXY_ENTITY,
  _swigc__p__dwg_entity_RAY,
  _swigc__p__dwg_entity_REPEAT,
  _swigc__p__dwg_entity_REVOLVEDSURFACE,
  _swigc__p__dwg_entity_ROTATIONGRIPENTITY,
  _swigc__p__dwg_entity_ROTATIONPARAMETERENTITY,
  _swigc__p__dwg_entity_RTEXT,
  _swigc__p__dwg_entity_SECTIONOBJECT,
  _swigc__p__dwg_entity_SEQEND,
  _swigc__p__dwg_entity_SHAPE,
  _swigc__p__dwg_entity_SOLID,
  _swigc__p__dwg_entity_SPLINE,
  _swigc__p__dwg_entity_SWEPTSURFACE,
  _swigc__p__dwg_entity_TABLE,
  _swigc__p__dwg_entity_TEXT,
  _swigc__p__dwg_entity_TOLERANCE,
  _swigc__p__dwg_entity_TRACE,
  _swigc__p__dwg_entity_UNKNOWN_ENT,
  _swigc__p__dwg_entity_VERTEX_2D,
  _swigc__p__dwg_entity_VERTEX_3D,
  _swigc__p__dwg_entity_VERTEX_PFACE_FACE,
  _swigc__p__dwg_entity_VIEWPORT,
  _swigc__p__dwg_entity_VISIBILITYGRIPENTITY,
  _swigc__p__dwg_entity_VISIBILITYPARAMETERENTITY,
  _swigc__p__dwg_entity_WIPEOUT,
  _swigc__p__dwg_entity_XYGRIPENTITY,
  _swigc__p__dwg_entity_XYPARAMETERENTITY,
  _swigc__p__dwg_entity_eed,
  _swigc__p__dwg_entity_eed_data,
  _swigc__p__dwg_filedeplist,
  _swigc__p__dwg_handle,
  _swigc__p__dwg_header,
  _swigc__p__dwg_header_variables,
  _swigc__p__dwg_object,
  _swigc__p__dwg_object_ACMECOMMANDHISTORY,
  _swigc__p__dwg_object_ACMESCOPE,
  _swigc__p__dwg_object_ACMESTATEMGR,
  _swigc__p__dwg_object_ACSH_BOOLEAN_CLASS,
  _swigc__p__dwg_object_ACSH_BOX_CLASS,
  _swigc__p__dwg_object_ACSH_BREP_CLASS,
  _swigc__p__dwg_object_ACSH_CHAMFER_CLASS,
  _swigc__p__dwg_object_ACSH_CONE_CLASS,
  _swigc__p__dwg_object_ACSH_CYLINDER_CLASS,
  _swigc__p__dwg_object_ACSH_EXTRUSION_CLASS,
  _swigc__p__dwg_object_ACSH_FILLET_CLASS,
  _swigc__p__dwg_object_ACSH_HISTORY_CLASS,
  _swigc__p__dwg_object_ACSH_LOFT_CLASS,
  _swigc__p__dwg_object_ACSH_PYRAMID_CLASS,
  _swigc__p__dwg_object_ACSH_REVOLVE_CLASS,
  _swigc__p__dwg_object_ACSH_SPHERE_CLASS,
  _swigc__p__dwg_object_ACSH_SWEEP_CLASS,
  _swigc__p__dwg_object_ACSH_TORUS_CLASS,
  _swigc__p__dwg_object_ACSH_WEDGE_CLASS,
  _swigc__p__dwg_object_ALDIMOBJECTCONTEXTDATA,
  _swigc__p__dwg_object_ANGDIMOBJECTCONTEXTDATA,
  _swigc__p__dwg_object_ANNOTSCALEOBJECTCONTEXTDATA,
  _swigc__p__dwg_object_APPID,
  _swigc__p__dwg_object_APPID_CONTROL,
  _swigc__p__dwg_object_ASSOC2DCONSTRAINTGROUP,
  _swigc__p__dwg_object_ASSOC3POINTANGULARDIMACTIONBODY,
  _swigc__p__dwg_object_ASSOCACTION,
  _swigc__p__dwg_object_ASSOCACTIONPARAM,
  _swigc__p__dwg_object_ASSOCALIGNEDDIMACTIONBODY,
  _swigc__p__dwg_object_ASSOCARRAYACTIONBODY,
  _swigc__p__dwg_object_ASSOCARRAYMODIFYACTIONBODY,
  _swigc__p__dwg_object_ASSOCASMBODYACTIONPARAM,
  _swigc__p__dwg_object_ASSOCBLENDSURFACEACTIONBODY,
  _swigc__p__dwg_object_ASSOCCOMPOUNDACTIONPARAM,
  _swigc__p__dwg_object_ASSOCDEPENDENCY,
  _swigc__p__dwg_object_ASSOCDIMDEPENDENCYBODY,
  _swigc__p__dwg_object_ASSOCEDGEACTIONPARAM,
  _swigc__p__dwg_object_ASSOCEDGECHAMFERACTIONBODY,
  _swigc__p__dwg_object_ASSOCEDGEFILLETACTIONBODY,
  _swigc__p__dwg_object_ASSOCEXTENDSURFACEACTIONBODY,
  _swigc__p__dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY,
  _swigc__p__dwg_object_ASSOCFACEACTIONPARAM,
  _swigc__p__dwg_object_ASSOCFILLETSURFACEACTIONBODY,
  _swigc__p__dwg_object_ASSOCGEOMDEPENDENCY,
  _swigc__p__dwg_object_ASSOCLOFTEDSURFACEACTIONBODY,
  _swigc__p__dwg_object_ASSOCMLEADERACTIONBODY,
  _swigc__p__dwg_object_ASSOCNETWORK,
  _swigc__p__dwg_object_ASSOCNETWORKSURFACEACTIONBODY,
  _swigc__p__dwg_object_ASSOCOBJECTACTIONPARAM,
  _swigc__p__dwg_object_ASSOCOFFSETSURFACEACTIONBODY,
  _swigc__p__dwg_object_ASSOCORDINATEDIMACTIONBODY,
  _swigc__p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM,
  _swigc__p__dwg_object_ASSOCPATCHSURFACEACTIONBODY,
  _swigc__p__dwg_object_ASSOCPATHACTIONPARAM,
  _swigc__p__dwg_object_ASSOCPERSSUBENTMANAGER,
  _swigc__p__dwg_object_ASSOCPLANESURFACEACTIONBODY,
  _swigc__p__dwg_object_ASSOCPOINTREFACTIONPARAM,
  _swigc__p__dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY,
  _swigc__p__dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY,
  _swigc__p__dwg_object_ASSOCROTATEDDIMACTIONBODY,
  _swigc__p__dwg_object_ASSOCSWEPTSURFACEACTIONBODY,
  _swigc__p__dwg_object_ASSOCTRIMSURFACEACTIONBODY,
  _swigc__p__dwg_object_ASSOCVALUEDEPENDENCY,
  _swigc__p__dwg_object_ASSOCVARIABLE,
  _swigc__p__dwg_object_ASSOCVERTEXACTIONPARAM,
  _swigc__p__dwg_object_BLKREFOBJECTCONTEXTDATA,
  _swigc__p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER,
  _swigc__p__dwg_object_BLOCKALIGNMENTGRIP,
  _swigc__p__dwg_object_BLOCKALIGNMENTPARAMETER,
  _swigc__p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER,
  _swigc__p__dwg_object_BLOCKARRAYACTION,
  _swigc__p__dwg_object_BLOCKBASEPOINTPARAMETER,
  _swigc__p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER,
  _swigc__p__dwg_object_BLOCKFLIPACTION,
  _swigc__p__dwg_object_BLOCKFLIPGRIP,
  _swigc__p__dwg_object_BLOCKFLIPPARAMETER,
  _swigc__p__dwg_object_BLOCKGRIPLOCATIONCOMPONENT,
  _swigc__p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER,
  _swigc__p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER,
  _swigc__p__dwg_object_BLOCKLINEARGRIP,
  _swigc__p__dwg_object_BLOCKLINEARPARAMETER,
  _swigc__p__dwg_object_BLOCKLOOKUPACTION,
  _swigc__p__dwg_object_BLOCKLOOKUPGRIP,
  _swigc__p__dwg_object_BLOCKLOOKUPPARAMETER,
  _swigc__p__dwg_object_BLOCKMOVEACTION,
  _swigc__p__dwg_object_BLOCKPARAMDEPENDENCYBODY,
  _swigc__p__dwg_object_BLOCKPOINTPARAMETER,
  _swigc__p__dwg_object_BLOCKPOLARGRIP,
  _swigc__p__dwg_object_BLOCKPOLARPARAMETER,
  _swigc__p__dwg_object_BLOCKPOLARSTRETCHACTION,
  _swigc__p__dwg_object_BLOCKPROPERTIESTABLE,
  _swigc__p__dwg_object_BLOCKPROPERTIESTABLEGRIP,
  _swigc__p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER,
  _swigc__p__dwg_object_BLOCKREPRESENTATION,
  _swigc__p__dwg_object_BLOCKROTATEACTION,
  _swigc__p__dwg_object_BLOCKROTATIONGRIP,
  _swigc__p__dwg_object_BLOCKROTATIONPARAMETER,
  _swigc__p__dwg_object_BLOCKSCALEACTION,
  _swigc__p__dwg_object_BLOCKSTRETCHACTION,
  _swigc__p__dwg_object_BLOCKUSERPARAMETER,
  _swigc__p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER,
  _swigc__p__dwg_object_BLOCKVISIBILITYGRIP,
  _swigc__p__dwg_object_BLOCKVISIBILITYPARAMETER,
  _swigc__p__dwg_object_BLOCKXYGRIP,
  _swigc__p__dwg_object_BLOCKXYPARAMETER,
  _swigc__p__dwg_object_BLOCK_CONTROL,
  _swigc__p__dwg_object_BLOCK_HEADER,
  _swigc__p__dwg_object_BREAKDATA,
  _swigc__p__dwg_object_BREAKPOINTREF,
  _swigc__p__dwg_object_CELLSTYLEMAP,
  _swigc__p__dwg_object_CONTEXTDATAMANAGER,
  _swigc__p__dwg_object_CSACDOCUMENTOPTIONS,
  _swigc__p__dwg_object_CURVEPATH,
  _swigc__p__dwg_object_DATALINK,
  _swigc__p__dwg_object_DATATABLE,
  _swigc__p__dwg_object_DBCOLOR,
  _swigc__p__dwg_object_DETAILVIEWSTYLE,
  _swigc__p__dwg_object_DICTIONARY,
  _swigc__p__dwg_object_DICTIONARYVAR,
  _swigc__p__dwg_object_DICTIONARYWDFLT,
  _swigc__p__dwg_object_DIMASSOC,
  _swigc__p__dwg_object_DIMSTYLE,
  _swigc__p__dwg_object_DIMSTYLE_CONTROL,
  _swigc__p__dwg_object_DMDIMOBJECTCONTEXTDATA,
  _swigc__p__dwg_object_DUMMY,
  _swigc__p__dwg_object_DYNAMICBLOCKPROXYNODE,
  _swigc__p__dwg_object_DYNAMICBLOCKPURGEPREVENTER,
  _swigc__p__dwg_object_EVALUATION_GRAPH,
  _swigc__p__dwg_object_FCFOBJECTCONTEXTDATA,
  _swigc__p__dwg_object_FIELD,
  _swigc__p__dwg_object_FIELDLIST,
  _swigc__p__dwg_object_GEODATA,
  _swigc__p__dwg_object_GEOMAPIMAGE,
  _swigc__p__dwg_object_GRADIENT_BACKGROUND,
  _swigc__p__dwg_object_GROUND_PLANE_BACKGROUND,
  _swigc__p__dwg_object_GROUP,
  _swigc__p__dwg_object_IBL_BACKGROUND,
  _swigc__p__dwg_object_IDBUFFER,
  _swigc__p__dwg_object_IMAGEDEF,
  _swigc__p__dwg_object_IMAGEDEF_REACTOR,
  _swigc__p__dwg_object_IMAGE_BACKGROUND,
  _swigc__p__dwg_object_INDEX,
  _swigc__p__dwg_object_LAYER,
  _swigc__p__dwg_object_LAYERFILTER,
  _swigc__p__dwg_object_LAYER_CONTROL,
  _swigc__p__dwg_object_LAYER_INDEX,
  _swigc__p__dwg_object_LAYOUT,
  _swigc__p__dwg_object_LEADEROBJECTCONTEXTDATA,
  _swigc__p__dwg_object_LIGHTLIST,
  _swigc__p__dwg_object_LONG_TRANSACTION,
  _swigc__p__dwg_object_LTYPE,
  _swigc__p__dwg_object_LTYPE_CONTROL,
  _swigc__p__dwg_object_MATERIAL,
  _swigc__p__dwg_object_MENTALRAYRENDERSETTINGS,
  _swigc__p__dwg_object_MLEADEROBJECTCONTEXTDATA,
  _swigc__p__dwg_object_MLEADERSTYLE,
  _swigc__p__dwg_object_MLINESTYLE,
  _swigc__p__dwg_object_MOTIONPATH,
  _swigc__p__dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA,
  _swigc__p__dwg_object_MTEXTOBJECTCONTEXTDATA,
  _swigc__p__dwg_object_NAVISWORKSMODELDEF,
  _swigc__p__dwg_object_OBJECT_PTR,
  _swigc__p__dwg_object_ORDDIMOBJECTCONTEXTDATA,
  _swigc__p__dwg_object_PARTIAL_VIEWING_INDEX,
  _swigc__p__dwg_object_PERSUBENTMGR,
  _swigc__p__dwg_object_PLACEHOLDER,
  _swigc__p__dwg_object_PLOTSETTINGS,
  _swigc__p__dwg_object_POINTCLOUDCOLORMAP,
  _swigc__p__dwg_object_POINTCLOUDDEF,
  _swigc__p__dwg_object_POINTCLOUDDEFEX,
  _swigc__p__dwg_object_POINTCLOUDDEF_REACTOR,
  _swigc__p__dwg_object_POINTCLOUDDEF_REACTOR_EX,
  _swigc__p__dwg_object_POINTPATH,
  _swigc__p__dwg_object_PROXY_OBJECT,
  _swigc__p__dwg_object_RADIMLGOBJECTCONTEXTDATA,
  _swigc__p__dwg_object_RADIMOBJECTCONTEXTDATA,
  _swigc__p__dwg_object_RAPIDRTRENDERSETTINGS,
  _swigc__p__dwg_object_RASTERVARIABLES,
  _swigc__p__dwg_object_RENDERENTRY,
  _swigc__p__dwg_object_RENDERENVIRONMENT,
  _swigc__p__dwg_object_RENDERGLOBAL,
  _swigc__p__dwg_object_RENDERSETTINGS,
  _swigc__p__dwg_object_SCALE,
  _swigc__p__dwg_object_SECTIONVIEWSTYLE,
  _swigc__p__dwg_object_SECTION_MANAGER,
  _swigc__p__dwg_object_SECTION_SETTINGS,
  _swigc__p__dwg_object_SKYLIGHT_BACKGROUND,
  _swigc__p__dwg_object_SOLID_BACKGROUND,
  _swigc__p__dwg_object_SORTENTSTABLE,
  _swigc__p__dwg_object_SPATIAL_FILTER,
  _swigc__p__dwg_object_SPATIAL_INDEX,
  _swigc__p__dwg_object_STYLE,
  _swigc__p__dwg_object_STYLE_CONTROL,
  _swigc__p__dwg_object_SUN,
  _swigc__p__dwg_object_SUNSTUDY,
  _swigc__p__dwg_object_TABLECONTENT,
  _swigc__p__dwg_object_TABLEGEOMETRY,
  _swigc__p__dwg_object_TABLESTYLE,
  _swigc__p__dwg_object_TEXTOBJECTCONTEXTDATA,
  _swigc__p__dwg_object_TVDEVICEPROPERTIES,
  _swigc__p__dwg_object_UCS,
  _swigc__p__dwg_object_UCS_CONTROL,
  _swigc__p__dwg_object_UNKNOWN_OBJ,
  _swigc__p__dwg_object_VBA_PROJECT,
  _swigc__p__dwg_object_VIEW,
  _swigc__p__dwg_object_VIEW_CONTROL,
  _swigc__p__dwg_object_VISUALSTYLE,
  _swigc__p__dwg_object_VPORT,
  _swigc__p__dwg_object_VPORT_CONTROL,
  _swigc__p__dwg_object_VX_CONTROL,
  _swigc__p__dwg_object_VX_TABLE_RECORD,
  _swigc__p__dwg_object_WIPEOUTVARIABLES,
  _swigc__p__dwg_object_XRECORD,
  _swigc__p__dwg_object_entity,
  _swigc__p__dwg_object_entity_tio,
  _swigc__p__dwg_object_object,
  _swigc__p__dwg_object_object_tio,
  _swigc__p__dwg_object_ref,
  _swigc__p__dwg_object_tio,
  _swigc__p__dwg_objfreespace,
  _swigc__p__dwg_resbuf,
  _swigc__p__dwg_resbuf_value,
  _swigc__p__dwg_revhistory,
  _swigc__p__dwg_secondheader,
  _swigc__p__dwg_secondheader_handles,
  _swigc__p__dwg_secondheader_sections,
  _swigc__p__dwg_section,
  _swigc__p__dwg_security,
  _swigc__p__dwg_struct,
  _swigc__p__dwg_struct_fhdr,
  _swigc__p__dwg_summaryinfo,
  _swigc__p__dwg_template,
  _swigc__p__dwg_time_bll,
  _swigc__p__dwg_vbaproject,
  _swigc__p__inthash,
  _swigc__p_char,
  _swigc__p_double,
  _swigc__p_dwg_entity_DIMENSION_common,
  _swigc__p_dwg_field_name_type_offset,
  _swigc__p_dwg_obj_ent,
  _swigc__p_dwg_obj_generic,
  _swigc__p_dwg_obj_layer,
  _swigc__p_dwg_obj_obj,
  _swigc__p_dwg_object,
  _swigc__p_dwg_versions,
  _swigc__p_eed_data_t,
  _swigc__p_eed_data_t_eed_0,
  _swigc__p_eed_data_t_eed_0_r2007,
  _swigc__p_eed_data_t_eed_1,
  _swigc__p_eed_data_t_eed_10,
  _swigc__p_eed_data_t_eed_2,
  _swigc__p_eed_data_t_eed_3,
  _swigc__p_eed_data_t_eed_4,
  _swigc__p_eed_data_t_eed_40,
  _swigc__p_eed_data_t_eed_5,
  _swigc__p_eed_data_t_eed_70,
  _swigc__p_eed_data_t_eed_71,
  _swigc__p_int,
  _swigc__p_long_long,
  _swigc__p_p__dwg_abstractentity_UNDERLAY,
  _swigc__p_p__dwg_abstractobject_ASSOCARRAYPARAMETERS,
  _swigc__p_p__dwg_abstractobject_UNDERLAYDEFINITION,
  _swigc__p_p__dwg_entity_3DFACE,
  _swigc__p_p__dwg_entity_3DLINE,
  _swigc__p_p__dwg_entity_3DSOLID,
  _swigc__p_p__dwg_entity_ARC,
  _swigc__p_p__dwg_entity_ARC_DIMENSION,
  _swigc__p_p__dwg_entity_ATTDEF,
  _swigc__p_p__dwg_entity_ATTRIB,
  _swigc__p_p__dwg_entity_BLOCK,
  _swigc__p_p__dwg_entity_CAMERA,
  _swigc__p_p__dwg_entity_CIRCLE,
  _swigc__p_p__dwg_entity_DIMENSION_ALIGNED,
  _swigc__p_p__dwg_entity_DIMENSION_ANG2LN,
  _swigc__p_p__dwg_entity_DIMENSION_ANG3PT,
  _swigc__p_p__dwg_entity_DIMENSION_DIAMETER,
  _swigc__p_p__dwg_entity_DIMENSION_LINEAR,
  _swigc__p_p__dwg_entity_DIMENSION_ORDINATE,
  _swigc__p_p__dwg_entity_DIMENSION_RADIUS,
  _swigc__p_p__dwg_entity_ELLIPSE,
  _swigc__p_p__dwg_entity_ENDBLK,
  _swigc__p_p__dwg_entity_ENDREP,
  _swigc__p_p__dwg_entity_HATCH,
  _swigc__p_p__dwg_entity_HELIX,
  _swigc__p_p__dwg_entity_IMAGE,
  _swigc__p_p__dwg_entity_INSERT,
  _swigc__p_p__dwg_entity_JUMP,
  _swigc__p_p__dwg_entity_LARGE_RADIAL_DIMENSION,
  _swigc__p_p__dwg_entity_LAYOUTPRINTCONFIG,
  _swigc__p_p__dwg_entity_LEADER,
  _swigc__p_p__dwg_entity_LIGHT,
  _swigc__p_p__dwg_entity_LINE,
  _swigc__p_p__dwg_entity_LOAD,
  _swigc__p_p__dwg_entity_LWPOLYLINE,
  _swigc__p_p__dwg_entity_MESH,
  _swigc__p_p__dwg_entity_MINSERT,
  _swigc__p_p__dwg_entity_MLINE,
  _swigc__p_p__dwg_entity_MTEXT,
  _swigc__p_p__dwg_entity_MULTILEADER,
  _swigc__p_p__dwg_entity_OLE2FRAME,
  _swigc__p_p__dwg_entity_OLEFRAME,
  _swigc__p_p__dwg_entity_PLANESURFACE,
  _swigc__p_p__dwg_entity_POINT,
  _swigc__p_p__dwg_entity_POINTCLOUD,
  _swigc__p_p__dwg_entity_POINTCLOUDEX,
  _swigc__p_p__dwg_entity_POLYLINE_2D,
  _swigc__p_p__dwg_entity_POLYLINE_3D,
  _swigc__p_p__dwg_entity_POLYLINE_MESH,
  _swigc__p_p__dwg_entity_POLYLINE_PFACE,
  _swigc__p_p__dwg_entity_PROXY_ENTITY,
  _swigc__p_p__dwg_entity_RAY,
  _swigc__p_p__dwg_entity_REPEAT,
  _swigc__p_p__dwg_entity_SECTIONOBJECT,
  _swigc__p_p__dwg_entity_SEQEND,
  _swigc__p_p__dwg_entity_SHAPE,
  _swigc__p_p__dwg_entity_SOLID,
  _swigc__p_p__dwg_entity_SPLINE,
  _swigc__p_p__dwg_entity_TEXT,
  _swigc__p_p__dwg_entity_TOLERANCE,
  _swigc__p_p__dwg_entity_TRACE,
  _swigc__p_p__dwg_entity_UNKNOWN_ENT,
  _swigc__p_p__dwg_entity_VERTEX_2D,
  _swigc__p_p__dwg_entity_VERTEX_3D,
  _swigc__p_p__dwg_entity_VERTEX_PFACE_FACE,
  _swigc__p_p__dwg_entity_VIEWPORT,
  _swigc__p_p__dwg_entity_WIPEOUT,
  _swigc__p_p__dwg_object_ACSH_BOOLEAN_CLASS,
  _swigc__p_p__dwg_object_ACSH_BOX_CLASS,
  _swigc__p_p__dwg_object_ACSH_BREP_CLASS,
  _swigc__p_p__dwg_object_ACSH_CHAMFER_CLASS,
  _swigc__p_p__dwg_object_ACSH_CONE_CLASS,
  _swigc__p_p__dwg_object_ACSH_CYLINDER_CLASS,
  _swigc__p_p__dwg_object_ACSH_FILLET_CLASS,
  _swigc__p_p__dwg_object_ACSH_HISTORY_CLASS,
  _swigc__p_p__dwg_object_ACSH_PYRAMID_CLASS,
  _swigc__p_p__dwg_object_ACSH_SPHERE_CLASS,
  _swigc__p_p__dwg_object_ACSH_TORUS_CLASS,
  _swigc__p_p__dwg_object_ACSH_WEDGE_CLASS,
  _swigc__p_p__dwg_object_ALDIMOBJECTCONTEXTDATA,
  _swigc__p_p__dwg_object_APPID,
  _swigc__p_p__dwg_object_APPID_CONTROL,
  _swigc__p_p__dwg_object_ASSOC2DCONSTRAINTGROUP,
  _swigc__p_p__dwg_object_ASSOCACTION,
  _swigc__p_p__dwg_object_ASSOCACTIONPARAM,
  _swigc__p_p__dwg_object_ASSOCARRAYACTIONBODY,
  _swigc__p_p__dwg_object_ASSOCASMBODYACTIONPARAM,
  _swigc__p_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY,
  _swigc__p_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM,
  _swigc__p_p__dwg_object_ASSOCDEPENDENCY,
  _swigc__p_p__dwg_object_ASSOCDIMDEPENDENCYBODY,
  _swigc__p_p__dwg_object_ASSOCEXTENDSURFACEACTIONBODY,
  _swigc__p_p__dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY,
  _swigc__p_p__dwg_object_ASSOCFACEACTIONPARAM,
  _swigc__p_p__dwg_object_ASSOCFILLETSURFACEACTIONBODY,
  _swigc__p_p__dwg_object_ASSOCGEOMDEPENDENCY,
  _swigc__p_p__dwg_object_ASSOCLOFTEDSURFACEACTIONBODY,
  _swigc__p_p__dwg_object_ASSOCNETWORK,
  _swigc__p_p__dwg_object_ASSOCNETWORKSURFACEACTIONBODY,
  _swigc__p_p__dwg_object_ASSOCOBJECTACTIONPARAM,
  _swigc__p_p__dwg_object_ASSOCOFFSETSURFACEACTIONBODY,
  _swigc__p_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM,
  _swigc__p_p__dwg_object_ASSOCPATCHSURFACEACTIONBODY,
  _swigc__p_p__dwg_object_ASSOCPATHACTIONPARAM,
  _swigc__p_p__dwg_object_ASSOCPLANESURFACEACTIONBODY,
  _swigc__p_p__dwg_object_ASSOCPOINTREFACTIONPARAM,
  _swigc__p_p__dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY,
  _swigc__p_p__dwg_object_ASSOCTRIMSURFACEACTIONBODY,
  _swigc__p_p__dwg_object_ASSOCVALUEDEPENDENCY,
  _swigc__p_p__dwg_object_ASSOCVARIABLE,
  _swigc__p_p__dwg_object_ASSOCVERTEXACTIONPARAM,
  _swigc__p_p__dwg_object_BLKREFOBJECTCONTEXTDATA,
  _swigc__p_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER,
  _swigc__p_p__dwg_object_BLOCKALIGNMENTGRIP,
  _swigc__p_p__dwg_object_BLOCKALIGNMENTPARAMETER,
  _swigc__p_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER,
  _swigc__p_p__dwg_object_BLOCKARRAYACTION,
  _swigc__p_p__dwg_object_BLOCKBASEPOINTPARAMETER,
  _swigc__p_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER,
  _swigc__p_p__dwg_object_BLOCKFLIPACTION,
  _swigc__p_p__dwg_object_BLOCKFLIPGRIP,
  _swigc__p_p__dwg_object_BLOCKFLIPPARAMETER,
  _swigc__p_p__dwg_object_BLOCKGRIPLOCATIONCOMPONENT,
  _swigc__p_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER,
  _swigc__p_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER,
  _swigc__p_p__dwg_object_BLOCKLINEARGRIP,
  _swigc__p_p__dwg_object_BLOCKLINEARPARAMETER,
  _swigc__p_p__dwg_object_BLOCKLOOKUPACTION,
  _swigc__p_p__dwg_object_BLOCKLOOKUPGRIP,
  _swigc__p_p__dwg_object_BLOCKLOOKUPPARAMETER,
  _swigc__p_p__dwg_object_BLOCKMOVEACTION,
  _swigc__p_p__dwg_object_BLOCKPARAMDEPENDENCYBODY,
  _swigc__p_p__dwg_object_BLOCKPOINTPARAMETER,
  _swigc__p_p__dwg_object_BLOCKPOLARGRIP,
  _swigc__p_p__dwg_object_BLOCKPOLARPARAMETER,
  _swigc__p_p__dwg_object_BLOCKPOLARSTRETCHACTION,
  _swigc__p_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER,
  _swigc__p_p__dwg_object_BLOCKREPRESENTATION,
  _swigc__p_p__dwg_object_BLOCKROTATEACTION,
  _swigc__p_p__dwg_object_BLOCKROTATIONGRIP,
  _swigc__p_p__dwg_object_BLOCKROTATIONPARAMETER,
  _swigc__p_p__dwg_object_BLOCKSCALEACTION,
  _swigc__p_p__dwg_object_BLOCKSTRETCHACTION,
  _swigc__p_p__dwg_object_BLOCKUSERPARAMETER,
  _swigc__p_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER,
  _swigc__p_p__dwg_object_BLOCKVISIBILITYGRIP,
  _swigc__p_p__dwg_object_BLOCKVISIBILITYPARAMETER,
  _swigc__p_p__dwg_object_BLOCKXYGRIP,
  _swigc__p_p__dwg_object_BLOCKXYPARAMETER,
  _swigc__p_p__dwg_object_BLOCK_CONTROL,
  _swigc__p_p__dwg_object_BLOCK_HEADER,
  _swigc__p_p__dwg_object_CELLSTYLEMAP,
  _swigc__p_p__dwg_object_DATALINK,
  _swigc__p_p__dwg_object_DBCOLOR,
  _swigc__p_p__dwg_object_DETAILVIEWSTYLE,
  _swigc__p_p__dwg_object_DICTIONARY,
  _swigc__p_p__dwg_object_DICTIONARYVAR,
  _swigc__p_p__dwg_object_DICTIONARYWDFLT,
  _swigc__p_p__dwg_object_DIMSTYLE,
  _swigc__p_p__dwg_object_DIMSTYLE_CONTROL,
  _swigc__p_p__dwg_object_DUMMY,
  _swigc__p_p__dwg_object_DYNAMICBLOCKPURGEPREVENTER,
  _swigc__p_p__dwg_object_EVALUATION_GRAPH,
  _swigc__p_p__dwg_object_FCFOBJECTCONTEXTDATA,
  _swigc__p_p__dwg_object_FIELD,
  _swigc__p_p__dwg_object_FIELDLIST,
  _swigc__p_p__dwg_object_GEODATA,
  _swigc__p_p__dwg_object_GRADIENT_BACKGROUND,
  _swigc__p_p__dwg_object_GROUND_PLANE_BACKGROUND,
  _swigc__p_p__dwg_object_GROUP,
  _swigc__p_p__dwg_object_IBL_BACKGROUND,
  _swigc__p_p__dwg_object_IDBUFFER,
  _swigc__p_p__dwg_object_IMAGEDEF,
  _swigc__p_p__dwg_object_IMAGEDEF_REACTOR,
  _swigc__p_p__dwg_object_IMAGE_BACKGROUND,
  _swigc__p_p__dwg_object_INDEX,
  _swigc__p_p__dwg_object_LAYER,
  _swigc__p_p__dwg_object_LAYERFILTER,
  _swigc__p_p__dwg_object_LAYER_CONTROL,
  _swigc__p_p__dwg_object_LAYER_INDEX,
  _swigc__p_p__dwg_object_LAYOUT,
  _swigc__p_p__dwg_object_LEADEROBJECTCONTEXTDATA,
  _swigc__p_p__dwg_object_LIGHTLIST,
  _swigc__p_p__dwg_object_LONG_TRANSACTION,
  _swigc__p_p__dwg_object_LTYPE,
  _swigc__p_p__dwg_object_LTYPE_CONTROL,
  _swigc__p_p__dwg_object_MATERIAL,
  _swigc__p_p__dwg_object_MENTALRAYRENDERSETTINGS,
  _swigc__p_p__dwg_object_MLEADERSTYLE,
  _swigc__p_p__dwg_object_MLINESTYLE,
  _swigc__p_p__dwg_object_MTEXTOBJECTCONTEXTDATA,
  _swigc__p_p__dwg_object_OBJECT_PTR,
  _swigc__p_p__dwg_object_PARTIAL_VIEWING_INDEX,
  _swigc__p_p__dwg_object_PLACEHOLDER,
  _swigc__p_p__dwg_object_PLOTSETTINGS,
  _swigc__p_p__dwg_object_POINTCLOUDCOLORMAP,
  _swigc__p_p__dwg_object_POINTCLOUDDEF,
  _swigc__p_p__dwg_object_POINTCLOUDDEFEX,
  _swigc__p_p__dwg_object_POINTCLOUDDEF_REACTOR,
  _swigc__p_p__dwg_object_POINTCLOUDDEF_REACTOR_EX,
  _swigc__p_p__dwg_object_PROXY_OBJECT,
  _swigc__p_p__dwg_object_RAPIDRTRENDERSETTINGS,
  _swigc__p_p__dwg_object_RASTERVARIABLES,
  _swigc__p_p__dwg_object_RENDERENTRY,
  _swigc__p_p__dwg_object_RENDERENVIRONMENT,
  _swigc__p_p__dwg_object_RENDERGLOBAL,
  _swigc__p_p__dwg_object_RENDERSETTINGS,
  _swigc__p_p__dwg_object_SCALE,
  _swigc__p_p__dwg_object_SECTIONVIEWSTYLE,
  _swigc__p_p__dwg_object_SECTION_MANAGER,
  _swigc__p_p__dwg_object_SECTION_SETTINGS,
  _swigc__p_p__dwg_object_SKYLIGHT_BACKGROUND,
  _swigc__p_p__dwg_object_SOLID_BACKGROUND,
  _swigc__p_p__dwg_object_SORTENTSTABLE,
  _swigc__p_p__dwg_object_SPATIAL_FILTER,
  _swigc__p_p__dwg_object_SPATIAL_INDEX,
  _swigc__p_p__dwg_object_STYLE,
  _swigc__p_p__dwg_object_STYLE_CONTROL,
  _swigc__p_p__dwg_object_SUN,
  _swigc__p_p__dwg_object_TABLEGEOMETRY,
  _swigc__p_p__dwg_object_TABLESTYLE,
  _swigc__p_p__dwg_object_TEXTOBJECTCONTEXTDATA,
  _swigc__p_p__dwg_object_UCS,
  _swigc__p_p__dwg_object_UCS_CONTROL,
  _swigc__p_p__dwg_object_UNKNOWN_OBJ,
  _swigc__p_p__dwg_object_VBA_PROJECT,
  _swigc__p_p__dwg_object_VIEW,
  _swigc__p_p__dwg_object_VIEW_CONTROL,
  _swigc__p_p__dwg_object_VISUALSTYLE,
  _swigc__p_p__dwg_object_VPORT,
  _swigc__p_p__dwg_object_VPORT_CONTROL,
  _swigc__p_p__dwg_object_VX_CONTROL,
  _swigc__p_p__dwg_object_VX_TABLE_RECORD,
  _swigc__p_p__dwg_object_WIPEOUTVARIABLES,
  _swigc__p_p__dwg_object_XRECORD,
  _swigc__p_p__dwg_object_entity,
  _swigc__p_p__dwg_object_ref,
  _swigc__p_p__dwg_section,
  _swigc__p_p_char,
  _swigc__p_r2007_file_header,
  _swigc__p_rgbpalette,
  _swigc__p_short,
  _swigc__p_signed_char,
  _swigc__p_unsigned_char,
  _swigc__p_unsigned_int,
  _swigc__p_unsigned_long_long,
  _swigc__p_unsigned_short,
};


/* -------- TYPE CONVERSION AND EQUIVALENCE RULES (END) -------- */

static swig_constant_info swig_constants[] = {
{0,0,0,0,0,0}
};
#ifdef __cplusplus
}
#endif
static swig_variable_info swig_variables[] = {
{0,0,0,0}
};
static swig_command_info swig_commands[] = {
{"LibreDWGc::new_Dwg_Object_Array", _wrap_new_Dwg_Object_Array},
{"LibreDWGc::delete_Dwg_Object_Array", _wrap_delete_Dwg_Object_Array},
{"LibreDWGc::Dwg_Object_Array_getitem", _wrap_Dwg_Object_Array_getitem},
{"LibreDWGc::Dwg_Object_Array_setitem", _wrap_Dwg_Object_Array_setitem},
{"LibreDWGc::Dwg_Bitcode_TimeBLL_days_set", _wrap_Dwg_Bitcode_TimeBLL_days_set},
{"LibreDWGc::Dwg_Bitcode_TimeBLL_days_get", _wrap_Dwg_Bitcode_TimeBLL_days_get},
{"LibreDWGc::Dwg_Bitcode_TimeBLL_ms_set", _wrap_Dwg_Bitcode_TimeBLL_ms_set},
{"LibreDWGc::Dwg_Bitcode_TimeBLL_ms_get", _wrap_Dwg_Bitcode_TimeBLL_ms_get},
{"LibreDWGc::Dwg_Bitcode_TimeBLL_value_set", _wrap_Dwg_Bitcode_TimeBLL_value_set},
{"LibreDWGc::Dwg_Bitcode_TimeBLL_value_get", _wrap_Dwg_Bitcode_TimeBLL_value_get},
{"LibreDWGc::new_Dwg_Bitcode_TimeBLL", _wrap_new_Dwg_Bitcode_TimeBLL},
{"LibreDWGc::delete_Dwg_Bitcode_TimeBLL", _wrap_delete_Dwg_Bitcode_TimeBLL},
{"LibreDWGc::Dwg_Bitcode_2RD_x_set", _wrap_Dwg_Bitcode_2RD_x_set},
{"LibreDWGc::Dwg_Bitcode_2RD_x_get", _wrap_Dwg_Bitcode_2RD_x_get},
{"LibreDWGc::Dwg_Bitcode_2RD_y_set", _wrap_Dwg_Bitcode_2RD_y_set},
{"LibreDWGc::Dwg_Bitcode_2RD_y_get", _wrap_Dwg_Bitcode_2RD_y_get},
{"LibreDWGc::new_Dwg_Bitcode_2RD", _wrap_new_Dwg_Bitcode_2RD},
{"LibreDWGc::delete_Dwg_Bitcode_2RD", _wrap_delete_Dwg_Bitcode_2RD},
{"LibreDWGc::Dwg_Bitcode_2BD_x_set", _wrap_Dwg_Bitcode_2BD_x_set},
{"LibreDWGc::Dwg_Bitcode_2BD_x_get", _wrap_Dwg_Bitcode_2BD_x_get},
{"LibreDWGc::Dwg_Bitcode_2BD_y_set", _wrap_Dwg_Bitcode_2BD_y_set},
{"LibreDWGc::Dwg_Bitcode_2BD_y_get", _wrap_Dwg_Bitcode_2BD_y_get},
{"LibreDWGc::new_Dwg_Bitcode_2BD", _wrap_new_Dwg_Bitcode_2BD},
{"LibreDWGc::delete_Dwg_Bitcode_2BD", _wrap_delete_Dwg_Bitcode_2BD},
{"LibreDWGc::Dwg_Bitcode_3RD_x_set", _wrap_Dwg_Bitcode_3RD_x_set},
{"LibreDWGc::Dwg_Bitcode_3RD_x_get", _wrap_Dwg_Bitcode_3RD_x_get},
{"LibreDWGc::Dwg_Bitcode_3RD_y_set", _wrap_Dwg_Bitcode_3RD_y_set},
{"LibreDWGc::Dwg_Bitcode_3RD_y_get", _wrap_Dwg_Bitcode_3RD_y_get},
{"LibreDWGc::Dwg_Bitcode_3RD_z_set", _wrap_Dwg_Bitcode_3RD_z_set},
{"LibreDWGc::Dwg_Bitcode_3RD_z_get", _wrap_Dwg_Bitcode_3RD_z_get},
{"LibreDWGc::new_Dwg_Bitcode_3RD", _wrap_new_Dwg_Bitcode_3RD},
{"LibreDWGc::delete_Dwg_Bitcode_3RD", _wrap_delete_Dwg_Bitcode_3RD},
{"LibreDWGc::Dwg_Bitcode_3BD_x_set", _wrap_Dwg_Bitcode_3BD_x_set},
{"LibreDWGc::Dwg_Bitcode_3BD_x_get", _wrap_Dwg_Bitcode_3BD_x_get},
{"LibreDWGc::Dwg_Bitcode_3BD_y_set", _wrap_Dwg_Bitcode_3BD_y_set},
{"LibreDWGc::Dwg_Bitcode_3BD_y_get", _wrap_Dwg_Bitcode_3BD_y_get},
{"LibreDWGc::Dwg_Bitcode_3BD_z_set", _wrap_Dwg_Bitcode_3BD_z_set},
{"LibreDWGc::Dwg_Bitcode_3BD_z_get", _wrap_Dwg_Bitcode_3BD_z_get},
{"LibreDWGc::new_Dwg_Bitcode_3BD", _wrap_new_Dwg_Bitcode_3BD},
{"LibreDWGc::delete_Dwg_Bitcode_3BD", _wrap_delete_Dwg_Bitcode_3BD},
{"LibreDWGc::Dwg_Versions_r_set", _wrap_Dwg_Versions_r_set},
{"LibreDWGc::Dwg_Versions_r_get", _wrap_Dwg_Versions_r_get},
{"LibreDWGc::Dwg_Versions_type_get", _wrap_Dwg_Versions_type_get},
{"LibreDWGc::Dwg_Versions_hdr_get", _wrap_Dwg_Versions_hdr_get},
{"LibreDWGc::Dwg_Versions_desc_get", _wrap_Dwg_Versions_desc_get},
{"LibreDWGc::Dwg_Versions_dwg_version_set", _wrap_Dwg_Versions_dwg_version_set},
{"LibreDWGc::Dwg_Versions_dwg_version_get", _wrap_Dwg_Versions_dwg_version_get},
{"LibreDWGc::new_Dwg_Versions", _wrap_new_Dwg_Versions},
{"LibreDWGc::delete_Dwg_Versions", _wrap_delete_Dwg_Versions},
{"LibreDWGc::Dwg_Handle_code_set", _wrap_Dwg_Handle_code_set},
{"LibreDWGc::Dwg_Handle_code_get", _wrap_Dwg_Handle_code_get},
{"LibreDWGc::Dwg_Handle_size_set", _wrap_Dwg_Handle_size_set},
{"LibreDWGc::Dwg_Handle_size_get", _wrap_Dwg_Handle_size_get},
{"LibreDWGc::Dwg_Handle_value_set", _wrap_Dwg_Handle_value_set},
{"LibreDWGc::Dwg_Handle_value_get", _wrap_Dwg_Handle_value_get},
{"LibreDWGc::Dwg_Handle_is_global_set", _wrap_Dwg_Handle_is_global_set},
{"LibreDWGc::Dwg_Handle_is_global_get", _wrap_Dwg_Handle_is_global_get},
{"LibreDWGc::new_Dwg_Handle", _wrap_new_Dwg_Handle},
{"LibreDWGc::delete_Dwg_Handle", _wrap_delete_Dwg_Handle},
{"LibreDWGc::Dwg_Object_Ref_obj_set", _wrap_Dwg_Object_Ref_obj_set},
{"LibreDWGc::Dwg_Object_Ref_obj_get", _wrap_Dwg_Object_Ref_obj_get},
{"LibreDWGc::Dwg_Object_Ref_handleref_set", _wrap_Dwg_Object_Ref_handleref_set},
{"LibreDWGc::Dwg_Object_Ref_handleref_get", _wrap_Dwg_Object_Ref_handleref_get},
{"LibreDWGc::Dwg_Object_Ref_absolute_ref_set", _wrap_Dwg_Object_Ref_absolute_ref_set},
{"LibreDWGc::Dwg_Object_Ref_absolute_ref_get", _wrap_Dwg_Object_Ref_absolute_ref_get},
{"LibreDWGc::Dwg_Object_Ref_r11_idx_set", _wrap_Dwg_Object_Ref_r11_idx_set},
{"LibreDWGc::Dwg_Object_Ref_r11_idx_get", _wrap_Dwg_Object_Ref_r11_idx_get},
{"LibreDWGc::new_Dwg_Object_Ref", _wrap_new_Dwg_Object_Ref},
{"LibreDWGc::delete_Dwg_Object_Ref", _wrap_delete_Dwg_Object_Ref},
{"LibreDWGc::Dwg_Color_index_set", _wrap_Dwg_Color_index_set},
{"LibreDWGc::Dwg_Color_index_get", _wrap_Dwg_Color_index_get},
{"LibreDWGc::Dwg_Color_flag_set", _wrap_Dwg_Color_flag_set},
{"LibreDWGc::Dwg_Color_flag_get", _wrap_Dwg_Color_flag_get},
{"LibreDWGc::Dwg_Color_raw_set", _wrap_Dwg_Color_raw_set},
{"LibreDWGc::Dwg_Color_raw_get", _wrap_Dwg_Color_raw_get},
{"LibreDWGc::Dwg_Color_rgb_set", _wrap_Dwg_Color_rgb_set},
{"LibreDWGc::Dwg_Color_rgb_get", _wrap_Dwg_Color_rgb_get},
{"LibreDWGc::Dwg_Color_method_set", _wrap_Dwg_Color_method_set},
{"LibreDWGc::Dwg_Color_method_get", _wrap_Dwg_Color_method_get},
{"LibreDWGc::Dwg_Color_name_set", _wrap_Dwg_Color_name_set},
{"LibreDWGc::Dwg_Color_name_get", _wrap_Dwg_Color_name_get},
{"LibreDWGc::Dwg_Color_book_name_set", _wrap_Dwg_Color_book_name_set},
{"LibreDWGc::Dwg_Color_book_name_get", _wrap_Dwg_Color_book_name_get},
{"LibreDWGc::Dwg_Color_handle_set", _wrap_Dwg_Color_handle_set},
{"LibreDWGc::Dwg_Color_handle_get", _wrap_Dwg_Color_handle_get},
{"LibreDWGc::Dwg_Color_alpha_raw_set", _wrap_Dwg_Color_alpha_raw_set},
{"LibreDWGc::Dwg_Color_alpha_raw_get", _wrap_Dwg_Color_alpha_raw_get},
{"LibreDWGc::Dwg_Color_alpha_type_set", _wrap_Dwg_Color_alpha_type_set},
{"LibreDWGc::Dwg_Color_alpha_type_get", _wrap_Dwg_Color_alpha_type_get},
{"LibreDWGc::Dwg_Color_alpha_set", _wrap_Dwg_Color_alpha_set},
{"LibreDWGc::Dwg_Color_alpha_get", _wrap_Dwg_Color_alpha_get},
{"LibreDWGc::new_Dwg_Color", _wrap_new_Dwg_Color},
{"LibreDWGc::delete_Dwg_Color", _wrap_delete_Dwg_Color},
{"LibreDWGc::dwg_color_method_name", _wrap_dwg_color_method_name},
{"LibreDWGc::_dwg_binary_chunk_size_set", _wrap__dwg_binary_chunk_size_set},
{"LibreDWGc::_dwg_binary_chunk_size_get", _wrap__dwg_binary_chunk_size_get},
{"LibreDWGc::_dwg_binary_chunk_codepage_set", _wrap__dwg_binary_chunk_codepage_set},
{"LibreDWGc::_dwg_binary_chunk_codepage_get", _wrap__dwg_binary_chunk_codepage_get},
{"LibreDWGc::_dwg_binary_chunk_is_tu_set", _wrap__dwg_binary_chunk_is_tu_set},
{"LibreDWGc::_dwg_binary_chunk_is_tu_get", _wrap__dwg_binary_chunk_is_tu_get},
{"LibreDWGc::_dwg_binary_chunk_u_get", _wrap__dwg_binary_chunk_u_get},
{"LibreDWGc::new__dwg_binary_chunk", _wrap_new__dwg_binary_chunk},
{"LibreDWGc::delete__dwg_binary_chunk", _wrap_delete__dwg_binary_chunk},
{"LibreDWGc::_dwg_binary_chunk_u_data_set", _wrap__dwg_binary_chunk_u_data_set},
{"LibreDWGc::_dwg_binary_chunk_u_data_get", _wrap__dwg_binary_chunk_u_data_get},
{"LibreDWGc::_dwg_binary_chunk_u_wdata_set", _wrap__dwg_binary_chunk_u_wdata_set},
{"LibreDWGc::_dwg_binary_chunk_u_wdata_get", _wrap__dwg_binary_chunk_u_wdata_get},
{"LibreDWGc::new__dwg_binary_chunk_u", _wrap_new__dwg_binary_chunk_u},
{"LibreDWGc::delete__dwg_binary_chunk_u", _wrap_delete__dwg_binary_chunk_u},
{"LibreDWGc::Dwg_Resbuf_type_set", _wrap_Dwg_Resbuf_type_set},
{"LibreDWGc::Dwg_Resbuf_type_get", _wrap_Dwg_Resbuf_type_get},
{"LibreDWGc::Dwg_Resbuf_value_get", _wrap_Dwg_Resbuf_value_get},
{"LibreDWGc::Dwg_Resbuf_nextrb_set", _wrap_Dwg_Resbuf_nextrb_set},
{"LibreDWGc::Dwg_Resbuf_nextrb_get", _wrap_Dwg_Resbuf_nextrb_get},
{"LibreDWGc::new_Dwg_Resbuf", _wrap_new_Dwg_Resbuf},
{"LibreDWGc::delete_Dwg_Resbuf", _wrap_delete_Dwg_Resbuf},
{"LibreDWGc::_dwg_resbuf_value_pt_set", _wrap__dwg_resbuf_value_pt_set},
{"LibreDWGc::_dwg_resbuf_value_pt_get", _wrap__dwg_resbuf_value_pt_get},
{"LibreDWGc::_dwg_resbuf_value_i8_set", _wrap__dwg_resbuf_value_i8_set},
{"LibreDWGc::_dwg_resbuf_value_i8_get", _wrap__dwg_resbuf_value_i8_get},
{"LibreDWGc::_dwg_resbuf_value_i16_set", _wrap__dwg_resbuf_value_i16_set},
{"LibreDWGc::_dwg_resbuf_value_i16_get", _wrap__dwg_resbuf_value_i16_get},
{"LibreDWGc::_dwg_resbuf_value_i32_set", _wrap__dwg_resbuf_value_i32_set},
{"LibreDWGc::_dwg_resbuf_value_i32_get", _wrap__dwg_resbuf_value_i32_get},
{"LibreDWGc::_dwg_resbuf_value_i64_set", _wrap__dwg_resbuf_value_i64_set},
{"LibreDWGc::_dwg_resbuf_value_i64_get", _wrap__dwg_resbuf_value_i64_get},
{"LibreDWGc::_dwg_resbuf_value_absref_set", _wrap__dwg_resbuf_value_absref_set},
{"LibreDWGc::_dwg_resbuf_value_absref_get", _wrap__dwg_resbuf_value_absref_get},
{"LibreDWGc::_dwg_resbuf_value_dbl_set", _wrap__dwg_resbuf_value_dbl_set},
{"LibreDWGc::_dwg_resbuf_value_dbl_get", _wrap__dwg_resbuf_value_dbl_get},
{"LibreDWGc::_dwg_resbuf_value_str_set", _wrap__dwg_resbuf_value_str_set},
{"LibreDWGc::_dwg_resbuf_value_str_get", _wrap__dwg_resbuf_value_str_get},
{"LibreDWGc::new__dwg_resbuf_value", _wrap_new__dwg_resbuf_value},
{"LibreDWGc::delete__dwg_resbuf_value", _wrap_delete__dwg_resbuf_value},
{"LibreDWGc::Dwg_Header_Variables_size_set", _wrap_Dwg_Header_Variables_size_set},
{"LibreDWGc::Dwg_Header_Variables_size_get", _wrap_Dwg_Header_Variables_size_get},
{"LibreDWGc::Dwg_Header_Variables_bitsize_hi_set", _wrap_Dwg_Header_Variables_bitsize_hi_set},
{"LibreDWGc::Dwg_Header_Variables_bitsize_hi_get", _wrap_Dwg_Header_Variables_bitsize_hi_get},
{"LibreDWGc::Dwg_Header_Variables_bitsize_set", _wrap_Dwg_Header_Variables_bitsize_set},
{"LibreDWGc::Dwg_Header_Variables_bitsize_get", _wrap_Dwg_Header_Variables_bitsize_get},
{"LibreDWGc::Dwg_Header_Variables_ACADMAINTVER_set", _wrap_Dwg_Header_Variables_ACADMAINTVER_set},
{"LibreDWGc::Dwg_Header_Variables_ACADMAINTVER_get", _wrap_Dwg_Header_Variables_ACADMAINTVER_get},
{"LibreDWGc::Dwg_Header_Variables_REQUIREDVERSIONS_set", _wrap_Dwg_Header_Variables_REQUIREDVERSIONS_set},
{"LibreDWGc::Dwg_Header_Variables_REQUIREDVERSIONS_get", _wrap_Dwg_Header_Variables_REQUIREDVERSIONS_get},
{"LibreDWGc::Dwg_Header_Variables_DWGCODEPAGE_set", _wrap_Dwg_Header_Variables_DWGCODEPAGE_set},
{"LibreDWGc::Dwg_Header_Variables_DWGCODEPAGE_get", _wrap_Dwg_Header_Variables_DWGCODEPAGE_get},
{"LibreDWGc::Dwg_Header_Variables_codepage_set", _wrap_Dwg_Header_Variables_codepage_set},
{"LibreDWGc::Dwg_Header_Variables_codepage_get", _wrap_Dwg_Header_Variables_codepage_get},
{"LibreDWGc::Dwg_Header_Variables_unit1_ratio_set", _wrap_Dwg_Header_Variables_unit1_ratio_set},
{"LibreDWGc::Dwg_Header_Variables_unit1_ratio_get", _wrap_Dwg_Header_Variables_unit1_ratio_get},
{"LibreDWGc::Dwg_Header_Variables_unit2_ratio_set", _wrap_Dwg_Header_Variables_unit2_ratio_set},
{"LibreDWGc::Dwg_Header_Variables_unit2_ratio_get", _wrap_Dwg_Header_Variables_unit2_ratio_get},
{"LibreDWGc::Dwg_Header_Variables_unit3_ratio_set", _wrap_Dwg_Header_Variables_unit3_ratio_set},
{"LibreDWGc::Dwg_Header_Variables_unit3_ratio_get", _wrap_Dwg_Header_Variables_unit3_ratio_get},
{"LibreDWGc::Dwg_Header_Variables_unit4_ratio_set", _wrap_Dwg_Header_Variables_unit4_ratio_set},
{"LibreDWGc::Dwg_Header_Variables_unit4_ratio_get", _wrap_Dwg_Header_Variables_unit4_ratio_get},
{"LibreDWGc::Dwg_Header_Variables_unit1_name_set", _wrap_Dwg_Header_Variables_unit1_name_set},
{"LibreDWGc::Dwg_Header_Variables_unit1_name_get", _wrap_Dwg_Header_Variables_unit1_name_get},
{"LibreDWGc::Dwg_Header_Variables_unit2_name_set", _wrap_Dwg_Header_Variables_unit2_name_set},
{"LibreDWGc::Dwg_Header_Variables_unit2_name_get", _wrap_Dwg_Header_Variables_unit2_name_get},
{"LibreDWGc::Dwg_Header_Variables_unit3_name_set", _wrap_Dwg_Header_Variables_unit3_name_set},
{"LibreDWGc::Dwg_Header_Variables_unit3_name_get", _wrap_Dwg_Header_Variables_unit3_name_get},
{"LibreDWGc::Dwg_Header_Variables_unit4_name_set", _wrap_Dwg_Header_Variables_unit4_name_set},
{"LibreDWGc::Dwg_Header_Variables_unit4_name_get", _wrap_Dwg_Header_Variables_unit4_name_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_8_set", _wrap_Dwg_Header_Variables_unknown_8_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_8_get", _wrap_Dwg_Header_Variables_unknown_8_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_9_set", _wrap_Dwg_Header_Variables_unknown_9_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_9_get", _wrap_Dwg_Header_Variables_unknown_9_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_10_set", _wrap_Dwg_Header_Variables_unknown_10_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_10_get", _wrap_Dwg_Header_Variables_unknown_10_get},
{"LibreDWGc::Dwg_Header_Variables_VX_TABLE_RECORD_set", _wrap_Dwg_Header_Variables_VX_TABLE_RECORD_set},
{"LibreDWGc::Dwg_Header_Variables_VX_TABLE_RECORD_get", _wrap_Dwg_Header_Variables_VX_TABLE_RECORD_get},
{"LibreDWGc::Dwg_Header_Variables_DIMASO_set", _wrap_Dwg_Header_Variables_DIMASO_set},
{"LibreDWGc::Dwg_Header_Variables_DIMASO_get", _wrap_Dwg_Header_Variables_DIMASO_get},
{"LibreDWGc::Dwg_Header_Variables_DIMSHO_set", _wrap_Dwg_Header_Variables_DIMSHO_set},
{"LibreDWGc::Dwg_Header_Variables_DIMSHO_get", _wrap_Dwg_Header_Variables_DIMSHO_get},
{"LibreDWGc::Dwg_Header_Variables_DIMSAV_set", _wrap_Dwg_Header_Variables_DIMSAV_set},
{"LibreDWGc::Dwg_Header_Variables_DIMSAV_get", _wrap_Dwg_Header_Variables_DIMSAV_get},
{"LibreDWGc::Dwg_Header_Variables_PLINEGEN_set", _wrap_Dwg_Header_Variables_PLINEGEN_set},
{"LibreDWGc::Dwg_Header_Variables_PLINEGEN_get", _wrap_Dwg_Header_Variables_PLINEGEN_get},
{"LibreDWGc::Dwg_Header_Variables_ORTHOMODE_set", _wrap_Dwg_Header_Variables_ORTHOMODE_set},
{"LibreDWGc::Dwg_Header_Variables_ORTHOMODE_get", _wrap_Dwg_Header_Variables_ORTHOMODE_get},
{"LibreDWGc::Dwg_Header_Variables_REGENMODE_set", _wrap_Dwg_Header_Variables_REGENMODE_set},
{"LibreDWGc::Dwg_Header_Variables_REGENMODE_get", _wrap_Dwg_Header_Variables_REGENMODE_get},
{"LibreDWGc::Dwg_Header_Variables_FILLMODE_set", _wrap_Dwg_Header_Variables_FILLMODE_set},
{"LibreDWGc::Dwg_Header_Variables_FILLMODE_get", _wrap_Dwg_Header_Variables_FILLMODE_get},
{"LibreDWGc::Dwg_Header_Variables_QTEXTMODE_set", _wrap_Dwg_Header_Variables_QTEXTMODE_set},
{"LibreDWGc::Dwg_Header_Variables_QTEXTMODE_get", _wrap_Dwg_Header_Variables_QTEXTMODE_get},
{"LibreDWGc::Dwg_Header_Variables_PSLTSCALE_set", _wrap_Dwg_Header_Variables_PSLTSCALE_set},
{"LibreDWGc::Dwg_Header_Variables_PSLTSCALE_get", _wrap_Dwg_Header_Variables_PSLTSCALE_get},
{"LibreDWGc::Dwg_Header_Variables_LIMCHECK_set", _wrap_Dwg_Header_Variables_LIMCHECK_set},
{"LibreDWGc::Dwg_Header_Variables_LIMCHECK_get", _wrap_Dwg_Header_Variables_LIMCHECK_get},
{"LibreDWGc::Dwg_Header_Variables_MENUEXT_set", _wrap_Dwg_Header_Variables_MENUEXT_set},
{"LibreDWGc::Dwg_Header_Variables_MENUEXT_get", _wrap_Dwg_Header_Variables_MENUEXT_get},
{"LibreDWGc::Dwg_Header_Variables_BLIPMODE_set", _wrap_Dwg_Header_Variables_BLIPMODE_set},
{"LibreDWGc::Dwg_Header_Variables_BLIPMODE_get", _wrap_Dwg_Header_Variables_BLIPMODE_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_11_set", _wrap_Dwg_Header_Variables_unknown_11_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_11_get", _wrap_Dwg_Header_Variables_unknown_11_get},
{"LibreDWGc::Dwg_Header_Variables_USRTIMER_set", _wrap_Dwg_Header_Variables_USRTIMER_set},
{"LibreDWGc::Dwg_Header_Variables_USRTIMER_get", _wrap_Dwg_Header_Variables_USRTIMER_get},
{"LibreDWGc::Dwg_Header_Variables_FASTZOOM_set", _wrap_Dwg_Header_Variables_FASTZOOM_set},
{"LibreDWGc::Dwg_Header_Variables_FASTZOOM_get", _wrap_Dwg_Header_Variables_FASTZOOM_get},
{"LibreDWGc::Dwg_Header_Variables_FLATLAND_set", _wrap_Dwg_Header_Variables_FLATLAND_set},
{"LibreDWGc::Dwg_Header_Variables_FLATLAND_get", _wrap_Dwg_Header_Variables_FLATLAND_get},
{"LibreDWGc::Dwg_Header_Variables_VIEWMODE_set", _wrap_Dwg_Header_Variables_VIEWMODE_set},
{"LibreDWGc::Dwg_Header_Variables_VIEWMODE_get", _wrap_Dwg_Header_Variables_VIEWMODE_get},
{"LibreDWGc::Dwg_Header_Variables_SKPOLY_set", _wrap_Dwg_Header_Variables_SKPOLY_set},
{"LibreDWGc::Dwg_Header_Variables_SKPOLY_get", _wrap_Dwg_Header_Variables_SKPOLY_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_mon_set", _wrap_Dwg_Header_Variables_unknown_mon_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_mon_get", _wrap_Dwg_Header_Variables_unknown_mon_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_day_set", _wrap_Dwg_Header_Variables_unknown_day_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_day_get", _wrap_Dwg_Header_Variables_unknown_day_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_year_set", _wrap_Dwg_Header_Variables_unknown_year_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_year_get", _wrap_Dwg_Header_Variables_unknown_year_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_hour_set", _wrap_Dwg_Header_Variables_unknown_hour_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_hour_get", _wrap_Dwg_Header_Variables_unknown_hour_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_min_set", _wrap_Dwg_Header_Variables_unknown_min_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_min_get", _wrap_Dwg_Header_Variables_unknown_min_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_sec_set", _wrap_Dwg_Header_Variables_unknown_sec_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_sec_get", _wrap_Dwg_Header_Variables_unknown_sec_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_ms_set", _wrap_Dwg_Header_Variables_unknown_ms_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_ms_get", _wrap_Dwg_Header_Variables_unknown_ms_get},
{"LibreDWGc::Dwg_Header_Variables_ANGDIR_set", _wrap_Dwg_Header_Variables_ANGDIR_set},
{"LibreDWGc::Dwg_Header_Variables_ANGDIR_get", _wrap_Dwg_Header_Variables_ANGDIR_get},
{"LibreDWGc::Dwg_Header_Variables_SPLFRAME_set", _wrap_Dwg_Header_Variables_SPLFRAME_set},
{"LibreDWGc::Dwg_Header_Variables_SPLFRAME_get", _wrap_Dwg_Header_Variables_SPLFRAME_get},
{"LibreDWGc::Dwg_Header_Variables_ATTREQ_set", _wrap_Dwg_Header_Variables_ATTREQ_set},
{"LibreDWGc::Dwg_Header_Variables_ATTREQ_get", _wrap_Dwg_Header_Variables_ATTREQ_get},
{"LibreDWGc::Dwg_Header_Variables_ATTDIA_set", _wrap_Dwg_Header_Variables_ATTDIA_set},
{"LibreDWGc::Dwg_Header_Variables_ATTDIA_get", _wrap_Dwg_Header_Variables_ATTDIA_get},
{"LibreDWGc::Dwg_Header_Variables_MIRRTEXT_set", _wrap_Dwg_Header_Variables_MIRRTEXT_set},
{"LibreDWGc::Dwg_Header_Variables_MIRRTEXT_get", _wrap_Dwg_Header_Variables_MIRRTEXT_get},
{"LibreDWGc::Dwg_Header_Variables_WORLDVIEW_set", _wrap_Dwg_Header_Variables_WORLDVIEW_set},
{"LibreDWGc::Dwg_Header_Variables_WORLDVIEW_get", _wrap_Dwg_Header_Variables_WORLDVIEW_get},
{"LibreDWGc::Dwg_Header_Variables_WIREFRAME_set", _wrap_Dwg_Header_Variables_WIREFRAME_set},
{"LibreDWGc::Dwg_Header_Variables_WIREFRAME_get", _wrap_Dwg_Header_Variables_WIREFRAME_get},
{"LibreDWGc::Dwg_Header_Variables_TILEMODE_set", _wrap_Dwg_Header_Variables_TILEMODE_set},
{"LibreDWGc::Dwg_Header_Variables_TILEMODE_get", _wrap_Dwg_Header_Variables_TILEMODE_get},
{"LibreDWGc::Dwg_Header_Variables_PLIMCHECK_set", _wrap_Dwg_Header_Variables_PLIMCHECK_set},
{"LibreDWGc::Dwg_Header_Variables_PLIMCHECK_get", _wrap_Dwg_Header_Variables_PLIMCHECK_get},
{"LibreDWGc::Dwg_Header_Variables_VISRETAIN_set", _wrap_Dwg_Header_Variables_VISRETAIN_set},
{"LibreDWGc::Dwg_Header_Variables_VISRETAIN_get", _wrap_Dwg_Header_Variables_VISRETAIN_get},
{"LibreDWGc::Dwg_Header_Variables_DELOBJ_set", _wrap_Dwg_Header_Variables_DELOBJ_set},
{"LibreDWGc::Dwg_Header_Variables_DELOBJ_get", _wrap_Dwg_Header_Variables_DELOBJ_get},
{"LibreDWGc::Dwg_Header_Variables_DISPSILH_set", _wrap_Dwg_Header_Variables_DISPSILH_set},
{"LibreDWGc::Dwg_Header_Variables_DISPSILH_get", _wrap_Dwg_Header_Variables_DISPSILH_get},
{"LibreDWGc::Dwg_Header_Variables_PELLIPSE_set", _wrap_Dwg_Header_Variables_PELLIPSE_set},
{"LibreDWGc::Dwg_Header_Variables_PELLIPSE_get", _wrap_Dwg_Header_Variables_PELLIPSE_get},
{"LibreDWGc::Dwg_Header_Variables_SAVEIMAGES_set", _wrap_Dwg_Header_Variables_SAVEIMAGES_set},
{"LibreDWGc::Dwg_Header_Variables_SAVEIMAGES_get", _wrap_Dwg_Header_Variables_SAVEIMAGES_get},
{"LibreDWGc::Dwg_Header_Variables_PROXYGRAPHICS_set", _wrap_Dwg_Header_Variables_PROXYGRAPHICS_set},
{"LibreDWGc::Dwg_Header_Variables_PROXYGRAPHICS_get", _wrap_Dwg_Header_Variables_PROXYGRAPHICS_get},
{"LibreDWGc::Dwg_Header_Variables_MEASUREMENT_set", _wrap_Dwg_Header_Variables_MEASUREMENT_set},
{"LibreDWGc::Dwg_Header_Variables_MEASUREMENT_get", _wrap_Dwg_Header_Variables_MEASUREMENT_get},
{"LibreDWGc::Dwg_Header_Variables_DRAGMODE_set", _wrap_Dwg_Header_Variables_DRAGMODE_set},
{"LibreDWGc::Dwg_Header_Variables_DRAGMODE_get", _wrap_Dwg_Header_Variables_DRAGMODE_get},
{"LibreDWGc::Dwg_Header_Variables_TREEDEPTH_set", _wrap_Dwg_Header_Variables_TREEDEPTH_set},
{"LibreDWGc::Dwg_Header_Variables_TREEDEPTH_get", _wrap_Dwg_Header_Variables_TREEDEPTH_get},
{"LibreDWGc::Dwg_Header_Variables_LUNITS_set", _wrap_Dwg_Header_Variables_LUNITS_set},
{"LibreDWGc::Dwg_Header_Variables_LUNITS_get", _wrap_Dwg_Header_Variables_LUNITS_get},
{"LibreDWGc::Dwg_Header_Variables_LUPREC_set", _wrap_Dwg_Header_Variables_LUPREC_set},
{"LibreDWGc::Dwg_Header_Variables_LUPREC_get", _wrap_Dwg_Header_Variables_LUPREC_get},
{"LibreDWGc::Dwg_Header_Variables_AUNITS_set", _wrap_Dwg_Header_Variables_AUNITS_set},
{"LibreDWGc::Dwg_Header_Variables_AUNITS_get", _wrap_Dwg_Header_Variables_AUNITS_get},
{"LibreDWGc::Dwg_Header_Variables_AUPREC_set", _wrap_Dwg_Header_Variables_AUPREC_set},
{"LibreDWGc::Dwg_Header_Variables_AUPREC_get", _wrap_Dwg_Header_Variables_AUPREC_get},
{"LibreDWGc::Dwg_Header_Variables_ATTMODE_set", _wrap_Dwg_Header_Variables_ATTMODE_set},
{"LibreDWGc::Dwg_Header_Variables_ATTMODE_get", _wrap_Dwg_Header_Variables_ATTMODE_get},
{"LibreDWGc::Dwg_Header_Variables_COORDS_set", _wrap_Dwg_Header_Variables_COORDS_set},
{"LibreDWGc::Dwg_Header_Variables_COORDS_get", _wrap_Dwg_Header_Variables_COORDS_get},
{"LibreDWGc::Dwg_Header_Variables_PDMODE_set", _wrap_Dwg_Header_Variables_PDMODE_set},
{"LibreDWGc::Dwg_Header_Variables_PDMODE_get", _wrap_Dwg_Header_Variables_PDMODE_get},
{"LibreDWGc::Dwg_Header_Variables_PICKSTYLE_set", _wrap_Dwg_Header_Variables_PICKSTYLE_set},
{"LibreDWGc::Dwg_Header_Variables_PICKSTYLE_get", _wrap_Dwg_Header_Variables_PICKSTYLE_get},
{"LibreDWGc::Dwg_Header_Variables_OSMODE_set", _wrap_Dwg_Header_Variables_OSMODE_set},
{"LibreDWGc::Dwg_Header_Variables_OSMODE_get", _wrap_Dwg_Header_Variables_OSMODE_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_12_set", _wrap_Dwg_Header_Variables_unknown_12_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_12_get", _wrap_Dwg_Header_Variables_unknown_12_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_13_set", _wrap_Dwg_Header_Variables_unknown_13_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_13_get", _wrap_Dwg_Header_Variables_unknown_13_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_14_set", _wrap_Dwg_Header_Variables_unknown_14_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_14_get", _wrap_Dwg_Header_Variables_unknown_14_get},
{"LibreDWGc::Dwg_Header_Variables_USERI1_set", _wrap_Dwg_Header_Variables_USERI1_set},
{"LibreDWGc::Dwg_Header_Variables_USERI1_get", _wrap_Dwg_Header_Variables_USERI1_get},
{"LibreDWGc::Dwg_Header_Variables_USERI2_set", _wrap_Dwg_Header_Variables_USERI2_set},
{"LibreDWGc::Dwg_Header_Variables_USERI2_get", _wrap_Dwg_Header_Variables_USERI2_get},
{"LibreDWGc::Dwg_Header_Variables_USERI3_set", _wrap_Dwg_Header_Variables_USERI3_set},
{"LibreDWGc::Dwg_Header_Variables_USERI3_get", _wrap_Dwg_Header_Variables_USERI3_get},
{"LibreDWGc::Dwg_Header_Variables_USERI4_set", _wrap_Dwg_Header_Variables_USERI4_set},
{"LibreDWGc::Dwg_Header_Variables_USERI4_get", _wrap_Dwg_Header_Variables_USERI4_get},
{"LibreDWGc::Dwg_Header_Variables_USERI5_set", _wrap_Dwg_Header_Variables_USERI5_set},
{"LibreDWGc::Dwg_Header_Variables_USERI5_get", _wrap_Dwg_Header_Variables_USERI5_get},
{"LibreDWGc::Dwg_Header_Variables_SPLINESEGS_set", _wrap_Dwg_Header_Variables_SPLINESEGS_set},
{"LibreDWGc::Dwg_Header_Variables_SPLINESEGS_get", _wrap_Dwg_Header_Variables_SPLINESEGS_get},
{"LibreDWGc::Dwg_Header_Variables_SURFU_set", _wrap_Dwg_Header_Variables_SURFU_set},
{"LibreDWGc::Dwg_Header_Variables_SURFU_get", _wrap_Dwg_Header_Variables_SURFU_get},
{"LibreDWGc::Dwg_Header_Variables_SURFV_set", _wrap_Dwg_Header_Variables_SURFV_set},
{"LibreDWGc::Dwg_Header_Variables_SURFV_get", _wrap_Dwg_Header_Variables_SURFV_get},
{"LibreDWGc::Dwg_Header_Variables_SURFTYPE_set", _wrap_Dwg_Header_Variables_SURFTYPE_set},
{"LibreDWGc::Dwg_Header_Variables_SURFTYPE_get", _wrap_Dwg_Header_Variables_SURFTYPE_get},
{"LibreDWGc::Dwg_Header_Variables_SURFTAB1_set", _wrap_Dwg_Header_Variables_SURFTAB1_set},
{"LibreDWGc::Dwg_Header_Variables_SURFTAB1_get", _wrap_Dwg_Header_Variables_SURFTAB1_get},
{"LibreDWGc::Dwg_Header_Variables_SURFTAB2_set", _wrap_Dwg_Header_Variables_SURFTAB2_set},
{"LibreDWGc::Dwg_Header_Variables_SURFTAB2_get", _wrap_Dwg_Header_Variables_SURFTAB2_get},
{"LibreDWGc::Dwg_Header_Variables_SPLINETYPE_set", _wrap_Dwg_Header_Variables_SPLINETYPE_set},
{"LibreDWGc::Dwg_Header_Variables_SPLINETYPE_get", _wrap_Dwg_Header_Variables_SPLINETYPE_get},
{"LibreDWGc::Dwg_Header_Variables_SHADEDGE_set", _wrap_Dwg_Header_Variables_SHADEDGE_set},
{"LibreDWGc::Dwg_Header_Variables_SHADEDGE_get", _wrap_Dwg_Header_Variables_SHADEDGE_get},
{"LibreDWGc::Dwg_Header_Variables_SHADEDIF_set", _wrap_Dwg_Header_Variables_SHADEDIF_set},
{"LibreDWGc::Dwg_Header_Variables_SHADEDIF_get", _wrap_Dwg_Header_Variables_SHADEDIF_get},
{"LibreDWGc::Dwg_Header_Variables_UNITMODE_set", _wrap_Dwg_Header_Variables_UNITMODE_set},
{"LibreDWGc::Dwg_Header_Variables_UNITMODE_get", _wrap_Dwg_Header_Variables_UNITMODE_get},
{"LibreDWGc::Dwg_Header_Variables_MAXACTVP_set", _wrap_Dwg_Header_Variables_MAXACTVP_set},
{"LibreDWGc::Dwg_Header_Variables_MAXACTVP_get", _wrap_Dwg_Header_Variables_MAXACTVP_get},
{"LibreDWGc::Dwg_Header_Variables_ISOLINES_set", _wrap_Dwg_Header_Variables_ISOLINES_set},
{"LibreDWGc::Dwg_Header_Variables_ISOLINES_get", _wrap_Dwg_Header_Variables_ISOLINES_get},
{"LibreDWGc::Dwg_Header_Variables_CMLJUST_set", _wrap_Dwg_Header_Variables_CMLJUST_set},
{"LibreDWGc::Dwg_Header_Variables_CMLJUST_get", _wrap_Dwg_Header_Variables_CMLJUST_get},
{"LibreDWGc::Dwg_Header_Variables_TEXTQLTY_set", _wrap_Dwg_Header_Variables_TEXTQLTY_set},
{"LibreDWGc::Dwg_Header_Variables_TEXTQLTY_get", _wrap_Dwg_Header_Variables_TEXTQLTY_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_14b_set", _wrap_Dwg_Header_Variables_unknown_14b_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_14b_get", _wrap_Dwg_Header_Variables_unknown_14b_get},
{"LibreDWGc::Dwg_Header_Variables_LTSCALE_set", _wrap_Dwg_Header_Variables_LTSCALE_set},
{"LibreDWGc::Dwg_Header_Variables_LTSCALE_get", _wrap_Dwg_Header_Variables_LTSCALE_get},
{"LibreDWGc::Dwg_Header_Variables_TEXTSIZE_set", _wrap_Dwg_Header_Variables_TEXTSIZE_set},
{"LibreDWGc::Dwg_Header_Variables_TEXTSIZE_get", _wrap_Dwg_Header_Variables_TEXTSIZE_get},
{"LibreDWGc::Dwg_Header_Variables_TRACEWID_set", _wrap_Dwg_Header_Variables_TRACEWID_set},
{"LibreDWGc::Dwg_Header_Variables_TRACEWID_get", _wrap_Dwg_Header_Variables_TRACEWID_get},
{"LibreDWGc::Dwg_Header_Variables_SKETCHINC_set", _wrap_Dwg_Header_Variables_SKETCHINC_set},
{"LibreDWGc::Dwg_Header_Variables_SKETCHINC_get", _wrap_Dwg_Header_Variables_SKETCHINC_get},
{"LibreDWGc::Dwg_Header_Variables_FILLETRAD_set", _wrap_Dwg_Header_Variables_FILLETRAD_set},
{"LibreDWGc::Dwg_Header_Variables_FILLETRAD_get", _wrap_Dwg_Header_Variables_FILLETRAD_get},
{"LibreDWGc::Dwg_Header_Variables_THICKNESS_set", _wrap_Dwg_Header_Variables_THICKNESS_set},
{"LibreDWGc::Dwg_Header_Variables_THICKNESS_get", _wrap_Dwg_Header_Variables_THICKNESS_get},
{"LibreDWGc::Dwg_Header_Variables_ANGBASE_set", _wrap_Dwg_Header_Variables_ANGBASE_set},
{"LibreDWGc::Dwg_Header_Variables_ANGBASE_get", _wrap_Dwg_Header_Variables_ANGBASE_get},
{"LibreDWGc::Dwg_Header_Variables_PDSIZE_set", _wrap_Dwg_Header_Variables_PDSIZE_set},
{"LibreDWGc::Dwg_Header_Variables_PDSIZE_get", _wrap_Dwg_Header_Variables_PDSIZE_get},
{"LibreDWGc::Dwg_Header_Variables_PLINEWID_set", _wrap_Dwg_Header_Variables_PLINEWID_set},
{"LibreDWGc::Dwg_Header_Variables_PLINEWID_get", _wrap_Dwg_Header_Variables_PLINEWID_get},
{"LibreDWGc::Dwg_Header_Variables_USERR1_set", _wrap_Dwg_Header_Variables_USERR1_set},
{"LibreDWGc::Dwg_Header_Variables_USERR1_get", _wrap_Dwg_Header_Variables_USERR1_get},
{"LibreDWGc::Dwg_Header_Variables_USERR2_set", _wrap_Dwg_Header_Variables_USERR2_set},
{"LibreDWGc::Dwg_Header_Variables_USERR2_get", _wrap_Dwg_Header_Variables_USERR2_get},
{"LibreDWGc::Dwg_Header_Variables_USERR3_set", _wrap_Dwg_Header_Variables_USERR3_set},
{"LibreDWGc::Dwg_Header_Variables_USERR3_get", _wrap_Dwg_Header_Variables_USERR3_get},
{"LibreDWGc::Dwg_Header_Variables_USERR4_set", _wrap_Dwg_Header_Variables_USERR4_set},
{"LibreDWGc::Dwg_Header_Variables_USERR4_get", _wrap_Dwg_Header_Variables_USERR4_get},
{"LibreDWGc::Dwg_Header_Variables_USERR5_set", _wrap_Dwg_Header_Variables_USERR5_set},
{"LibreDWGc::Dwg_Header_Variables_USERR5_get", _wrap_Dwg_Header_Variables_USERR5_get},
{"LibreDWGc::Dwg_Header_Variables_CHAMFERA_set", _wrap_Dwg_Header_Variables_CHAMFERA_set},
{"LibreDWGc::Dwg_Header_Variables_CHAMFERA_get", _wrap_Dwg_Header_Variables_CHAMFERA_get},
{"LibreDWGc::Dwg_Header_Variables_CHAMFERB_set", _wrap_Dwg_Header_Variables_CHAMFERB_set},
{"LibreDWGc::Dwg_Header_Variables_CHAMFERB_get", _wrap_Dwg_Header_Variables_CHAMFERB_get},
{"LibreDWGc::Dwg_Header_Variables_CHAMFERC_set", _wrap_Dwg_Header_Variables_CHAMFERC_set},
{"LibreDWGc::Dwg_Header_Variables_CHAMFERC_get", _wrap_Dwg_Header_Variables_CHAMFERC_get},
{"LibreDWGc::Dwg_Header_Variables_CHAMFERD_set", _wrap_Dwg_Header_Variables_CHAMFERD_set},
{"LibreDWGc::Dwg_Header_Variables_CHAMFERD_get", _wrap_Dwg_Header_Variables_CHAMFERD_get},
{"LibreDWGc::Dwg_Header_Variables_FACETRES_set", _wrap_Dwg_Header_Variables_FACETRES_set},
{"LibreDWGc::Dwg_Header_Variables_FACETRES_get", _wrap_Dwg_Header_Variables_FACETRES_get},
{"LibreDWGc::Dwg_Header_Variables_CMLSCALE_set", _wrap_Dwg_Header_Variables_CMLSCALE_set},
{"LibreDWGc::Dwg_Header_Variables_CMLSCALE_get", _wrap_Dwg_Header_Variables_CMLSCALE_get},
{"LibreDWGc::Dwg_Header_Variables_CELTSCALE_set", _wrap_Dwg_Header_Variables_CELTSCALE_set},
{"LibreDWGc::Dwg_Header_Variables_CELTSCALE_get", _wrap_Dwg_Header_Variables_CELTSCALE_get},
{"LibreDWGc::Dwg_Header_Variables_VIEWTWIST_set", _wrap_Dwg_Header_Variables_VIEWTWIST_set},
{"LibreDWGc::Dwg_Header_Variables_VIEWTWIST_get", _wrap_Dwg_Header_Variables_VIEWTWIST_get},
{"LibreDWGc::Dwg_Header_Variables_MENU_set", _wrap_Dwg_Header_Variables_MENU_set},
{"LibreDWGc::Dwg_Header_Variables_MENU_get", _wrap_Dwg_Header_Variables_MENU_get},
{"LibreDWGc::Dwg_Header_Variables_TDCREATE_set", _wrap_Dwg_Header_Variables_TDCREATE_set},
{"LibreDWGc::Dwg_Header_Variables_TDCREATE_get", _wrap_Dwg_Header_Variables_TDCREATE_get},
{"LibreDWGc::Dwg_Header_Variables_TDUPDATE_set", _wrap_Dwg_Header_Variables_TDUPDATE_set},
{"LibreDWGc::Dwg_Header_Variables_TDUPDATE_get", _wrap_Dwg_Header_Variables_TDUPDATE_get},
{"LibreDWGc::Dwg_Header_Variables_TDUCREATE_set", _wrap_Dwg_Header_Variables_TDUCREATE_set},
{"LibreDWGc::Dwg_Header_Variables_TDUCREATE_get", _wrap_Dwg_Header_Variables_TDUCREATE_get},
{"LibreDWGc::Dwg_Header_Variables_TDUUPDATE_set", _wrap_Dwg_Header_Variables_TDUUPDATE_set},
{"LibreDWGc::Dwg_Header_Variables_TDUUPDATE_get", _wrap_Dwg_Header_Variables_TDUUPDATE_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_15_set", _wrap_Dwg_Header_Variables_unknown_15_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_15_get", _wrap_Dwg_Header_Variables_unknown_15_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_16_set", _wrap_Dwg_Header_Variables_unknown_16_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_16_get", _wrap_Dwg_Header_Variables_unknown_16_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_17_set", _wrap_Dwg_Header_Variables_unknown_17_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_17_get", _wrap_Dwg_Header_Variables_unknown_17_get},
{"LibreDWGc::Dwg_Header_Variables_TDINDWG_set", _wrap_Dwg_Header_Variables_TDINDWG_set},
{"LibreDWGc::Dwg_Header_Variables_TDINDWG_get", _wrap_Dwg_Header_Variables_TDINDWG_get},
{"LibreDWGc::Dwg_Header_Variables_TDUSRTIMER_set", _wrap_Dwg_Header_Variables_TDUSRTIMER_set},
{"LibreDWGc::Dwg_Header_Variables_TDUSRTIMER_get", _wrap_Dwg_Header_Variables_TDUSRTIMER_get},
{"LibreDWGc::Dwg_Header_Variables_CECOLOR_set", _wrap_Dwg_Header_Variables_CECOLOR_set},
{"LibreDWGc::Dwg_Header_Variables_CECOLOR_get", _wrap_Dwg_Header_Variables_CECOLOR_get},
{"LibreDWGc::Dwg_Header_Variables_HANDLING_set", _wrap_Dwg_Header_Variables_HANDLING_set},
{"LibreDWGc::Dwg_Header_Variables_HANDLING_get", _wrap_Dwg_Header_Variables_HANDLING_get},
{"LibreDWGc::Dwg_Header_Variables_HANDSEED_set", _wrap_Dwg_Header_Variables_HANDSEED_set},
{"LibreDWGc::Dwg_Header_Variables_HANDSEED_get", _wrap_Dwg_Header_Variables_HANDSEED_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_4f2_set", _wrap_Dwg_Header_Variables_unknown_4f2_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_4f2_get", _wrap_Dwg_Header_Variables_unknown_4f2_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_5_set", _wrap_Dwg_Header_Variables_unknown_5_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_5_get", _wrap_Dwg_Header_Variables_unknown_5_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_6_set", _wrap_Dwg_Header_Variables_unknown_6_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_6_get", _wrap_Dwg_Header_Variables_unknown_6_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_6a_set", _wrap_Dwg_Header_Variables_unknown_6a_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_6a_get", _wrap_Dwg_Header_Variables_unknown_6a_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_6b_set", _wrap_Dwg_Header_Variables_unknown_6b_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_6b_get", _wrap_Dwg_Header_Variables_unknown_6b_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_6c_set", _wrap_Dwg_Header_Variables_unknown_6c_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_6c_get", _wrap_Dwg_Header_Variables_unknown_6c_get},
{"LibreDWGc::Dwg_Header_Variables_CLAYER_set", _wrap_Dwg_Header_Variables_CLAYER_set},
{"LibreDWGc::Dwg_Header_Variables_CLAYER_get", _wrap_Dwg_Header_Variables_CLAYER_get},
{"LibreDWGc::Dwg_Header_Variables_TEXTSTYLE_set", _wrap_Dwg_Header_Variables_TEXTSTYLE_set},
{"LibreDWGc::Dwg_Header_Variables_TEXTSTYLE_get", _wrap_Dwg_Header_Variables_TEXTSTYLE_get},
{"LibreDWGc::Dwg_Header_Variables_CELTYPE_set", _wrap_Dwg_Header_Variables_CELTYPE_set},
{"LibreDWGc::Dwg_Header_Variables_CELTYPE_get", _wrap_Dwg_Header_Variables_CELTYPE_get},
{"LibreDWGc::Dwg_Header_Variables_CMATERIAL_set", _wrap_Dwg_Header_Variables_CMATERIAL_set},
{"LibreDWGc::Dwg_Header_Variables_CMATERIAL_get", _wrap_Dwg_Header_Variables_CMATERIAL_get},
{"LibreDWGc::Dwg_Header_Variables_DIMSTYLE_set", _wrap_Dwg_Header_Variables_DIMSTYLE_set},
{"LibreDWGc::Dwg_Header_Variables_DIMSTYLE_get", _wrap_Dwg_Header_Variables_DIMSTYLE_get},
{"LibreDWGc::Dwg_Header_Variables_CMLSTYLE_set", _wrap_Dwg_Header_Variables_CMLSTYLE_set},
{"LibreDWGc::Dwg_Header_Variables_CMLSTYLE_get", _wrap_Dwg_Header_Variables_CMLSTYLE_get},
{"LibreDWGc::Dwg_Header_Variables_PSVPSCALE_set", _wrap_Dwg_Header_Variables_PSVPSCALE_set},
{"LibreDWGc::Dwg_Header_Variables_PSVPSCALE_get", _wrap_Dwg_Header_Variables_PSVPSCALE_get},
{"LibreDWGc::Dwg_Header_Variables_PINSBASE_set", _wrap_Dwg_Header_Variables_PINSBASE_set},
{"LibreDWGc::Dwg_Header_Variables_PINSBASE_get", _wrap_Dwg_Header_Variables_PINSBASE_get},
{"LibreDWGc::Dwg_Header_Variables_PEXTMIN_set", _wrap_Dwg_Header_Variables_PEXTMIN_set},
{"LibreDWGc::Dwg_Header_Variables_PEXTMIN_get", _wrap_Dwg_Header_Variables_PEXTMIN_get},
{"LibreDWGc::Dwg_Header_Variables_PEXTMAX_set", _wrap_Dwg_Header_Variables_PEXTMAX_set},
{"LibreDWGc::Dwg_Header_Variables_PEXTMAX_get", _wrap_Dwg_Header_Variables_PEXTMAX_get},
{"LibreDWGc::Dwg_Header_Variables_PLIMMIN_set", _wrap_Dwg_Header_Variables_PLIMMIN_set},
{"LibreDWGc::Dwg_Header_Variables_PLIMMIN_get", _wrap_Dwg_Header_Variables_PLIMMIN_get},
{"LibreDWGc::Dwg_Header_Variables_PLIMMAX_set", _wrap_Dwg_Header_Variables_PLIMMAX_set},
{"LibreDWGc::Dwg_Header_Variables_PLIMMAX_get", _wrap_Dwg_Header_Variables_PLIMMAX_get},
{"LibreDWGc::Dwg_Header_Variables_PELEVATION_set", _wrap_Dwg_Header_Variables_PELEVATION_set},
{"LibreDWGc::Dwg_Header_Variables_PELEVATION_get", _wrap_Dwg_Header_Variables_PELEVATION_get},
{"LibreDWGc::Dwg_Header_Variables_PUCSORG_set", _wrap_Dwg_Header_Variables_PUCSORG_set},
{"LibreDWGc::Dwg_Header_Variables_PUCSORG_get", _wrap_Dwg_Header_Variables_PUCSORG_get},
{"LibreDWGc::Dwg_Header_Variables_PUCSXDIR_set", _wrap_Dwg_Header_Variables_PUCSXDIR_set},
{"LibreDWGc::Dwg_Header_Variables_PUCSXDIR_get", _wrap_Dwg_Header_Variables_PUCSXDIR_get},
{"LibreDWGc::Dwg_Header_Variables_PUCSYDIR_set", _wrap_Dwg_Header_Variables_PUCSYDIR_set},
{"LibreDWGc::Dwg_Header_Variables_PUCSYDIR_get", _wrap_Dwg_Header_Variables_PUCSYDIR_get},
{"LibreDWGc::Dwg_Header_Variables_PUCSNAME_set", _wrap_Dwg_Header_Variables_PUCSNAME_set},
{"LibreDWGc::Dwg_Header_Variables_PUCSNAME_get", _wrap_Dwg_Header_Variables_PUCSNAME_get},
{"LibreDWGc::Dwg_Header_Variables_PUCSBASE_set", _wrap_Dwg_Header_Variables_PUCSBASE_set},
{"LibreDWGc::Dwg_Header_Variables_PUCSBASE_get", _wrap_Dwg_Header_Variables_PUCSBASE_get},
{"LibreDWGc::Dwg_Header_Variables_PUCSORTHOREF_set", _wrap_Dwg_Header_Variables_PUCSORTHOREF_set},
{"LibreDWGc::Dwg_Header_Variables_PUCSORTHOREF_get", _wrap_Dwg_Header_Variables_PUCSORTHOREF_get},
{"LibreDWGc::Dwg_Header_Variables_PUCSORTHOVIEW_set", _wrap_Dwg_Header_Variables_PUCSORTHOVIEW_set},
{"LibreDWGc::Dwg_Header_Variables_PUCSORTHOVIEW_get", _wrap_Dwg_Header_Variables_PUCSORTHOVIEW_get},
{"LibreDWGc::Dwg_Header_Variables_PUCSORGTOP_set", _wrap_Dwg_Header_Variables_PUCSORGTOP_set},
{"LibreDWGc::Dwg_Header_Variables_PUCSORGTOP_get", _wrap_Dwg_Header_Variables_PUCSORGTOP_get},
{"LibreDWGc::Dwg_Header_Variables_PUCSORGBOTTOM_set", _wrap_Dwg_Header_Variables_PUCSORGBOTTOM_set},
{"LibreDWGc::Dwg_Header_Variables_PUCSORGBOTTOM_get", _wrap_Dwg_Header_Variables_PUCSORGBOTTOM_get},
{"LibreDWGc::Dwg_Header_Variables_PUCSORGLEFT_set", _wrap_Dwg_Header_Variables_PUCSORGLEFT_set},
{"LibreDWGc::Dwg_Header_Variables_PUCSORGLEFT_get", _wrap_Dwg_Header_Variables_PUCSORGLEFT_get},
{"LibreDWGc::Dwg_Header_Variables_PUCSORGRIGHT_set", _wrap_Dwg_Header_Variables_PUCSORGRIGHT_set},
{"LibreDWGc::Dwg_Header_Variables_PUCSORGRIGHT_get", _wrap_Dwg_Header_Variables_PUCSORGRIGHT_get},
{"LibreDWGc::Dwg_Header_Variables_PUCSORGFRONT_set", _wrap_Dwg_Header_Variables_PUCSORGFRONT_set},
{"LibreDWGc::Dwg_Header_Variables_PUCSORGFRONT_get", _wrap_Dwg_Header_Variables_PUCSORGFRONT_get},
{"LibreDWGc::Dwg_Header_Variables_PUCSORGBACK_set", _wrap_Dwg_Header_Variables_PUCSORGBACK_set},
{"LibreDWGc::Dwg_Header_Variables_PUCSORGBACK_get", _wrap_Dwg_Header_Variables_PUCSORGBACK_get},
{"LibreDWGc::Dwg_Header_Variables_INSBASE_set", _wrap_Dwg_Header_Variables_INSBASE_set},
{"LibreDWGc::Dwg_Header_Variables_INSBASE_get", _wrap_Dwg_Header_Variables_INSBASE_get},
{"LibreDWGc::Dwg_Header_Variables_EXTMIN_set", _wrap_Dwg_Header_Variables_EXTMIN_set},
{"LibreDWGc::Dwg_Header_Variables_EXTMIN_get", _wrap_Dwg_Header_Variables_EXTMIN_get},
{"LibreDWGc::Dwg_Header_Variables_EXTMAX_set", _wrap_Dwg_Header_Variables_EXTMAX_set},
{"LibreDWGc::Dwg_Header_Variables_EXTMAX_get", _wrap_Dwg_Header_Variables_EXTMAX_get},
{"LibreDWGc::Dwg_Header_Variables_VIEWDIR_set", _wrap_Dwg_Header_Variables_VIEWDIR_set},
{"LibreDWGc::Dwg_Header_Variables_VIEWDIR_get", _wrap_Dwg_Header_Variables_VIEWDIR_get},
{"LibreDWGc::Dwg_Header_Variables_TARGET_set", _wrap_Dwg_Header_Variables_TARGET_set},
{"LibreDWGc::Dwg_Header_Variables_TARGET_get", _wrap_Dwg_Header_Variables_TARGET_get},
{"LibreDWGc::Dwg_Header_Variables_LIMMIN_set", _wrap_Dwg_Header_Variables_LIMMIN_set},
{"LibreDWGc::Dwg_Header_Variables_LIMMIN_get", _wrap_Dwg_Header_Variables_LIMMIN_get},
{"LibreDWGc::Dwg_Header_Variables_LIMMAX_set", _wrap_Dwg_Header_Variables_LIMMAX_set},
{"LibreDWGc::Dwg_Header_Variables_LIMMAX_get", _wrap_Dwg_Header_Variables_LIMMAX_get},
{"LibreDWGc::Dwg_Header_Variables_VIEWCTR_set", _wrap_Dwg_Header_Variables_VIEWCTR_set},
{"LibreDWGc::Dwg_Header_Variables_VIEWCTR_get", _wrap_Dwg_Header_Variables_VIEWCTR_get},
{"LibreDWGc::Dwg_Header_Variables_ELEVATION_set", _wrap_Dwg_Header_Variables_ELEVATION_set},
{"LibreDWGc::Dwg_Header_Variables_ELEVATION_get", _wrap_Dwg_Header_Variables_ELEVATION_get},
{"LibreDWGc::Dwg_Header_Variables_VIEWSIZE_set", _wrap_Dwg_Header_Variables_VIEWSIZE_set},
{"LibreDWGc::Dwg_Header_Variables_VIEWSIZE_get", _wrap_Dwg_Header_Variables_VIEWSIZE_get},
{"LibreDWGc::Dwg_Header_Variables_SNAPMODE_set", _wrap_Dwg_Header_Variables_SNAPMODE_set},
{"LibreDWGc::Dwg_Header_Variables_SNAPMODE_get", _wrap_Dwg_Header_Variables_SNAPMODE_get},
{"LibreDWGc::Dwg_Header_Variables_SNAPUNIT_set", _wrap_Dwg_Header_Variables_SNAPUNIT_set},
{"LibreDWGc::Dwg_Header_Variables_SNAPUNIT_get", _wrap_Dwg_Header_Variables_SNAPUNIT_get},
{"LibreDWGc::Dwg_Header_Variables_SNAPBASE_set", _wrap_Dwg_Header_Variables_SNAPBASE_set},
{"LibreDWGc::Dwg_Header_Variables_SNAPBASE_get", _wrap_Dwg_Header_Variables_SNAPBASE_get},
{"LibreDWGc::Dwg_Header_Variables_SNAPANG_set", _wrap_Dwg_Header_Variables_SNAPANG_set},
{"LibreDWGc::Dwg_Header_Variables_SNAPANG_get", _wrap_Dwg_Header_Variables_SNAPANG_get},
{"LibreDWGc::Dwg_Header_Variables_SNAPSTYLE_set", _wrap_Dwg_Header_Variables_SNAPSTYLE_set},
{"LibreDWGc::Dwg_Header_Variables_SNAPSTYLE_get", _wrap_Dwg_Header_Variables_SNAPSTYLE_get},
{"LibreDWGc::Dwg_Header_Variables_SNAPISOPAIR_set", _wrap_Dwg_Header_Variables_SNAPISOPAIR_set},
{"LibreDWGc::Dwg_Header_Variables_SNAPISOPAIR_get", _wrap_Dwg_Header_Variables_SNAPISOPAIR_get},
{"LibreDWGc::Dwg_Header_Variables_GRIDMODE_set", _wrap_Dwg_Header_Variables_GRIDMODE_set},
{"LibreDWGc::Dwg_Header_Variables_GRIDMODE_get", _wrap_Dwg_Header_Variables_GRIDMODE_get},
{"LibreDWGc::Dwg_Header_Variables_GRIDUNIT_set", _wrap_Dwg_Header_Variables_GRIDUNIT_set},
{"LibreDWGc::Dwg_Header_Variables_GRIDUNIT_get", _wrap_Dwg_Header_Variables_GRIDUNIT_get},
{"LibreDWGc::Dwg_Header_Variables_AXISMODE_set", _wrap_Dwg_Header_Variables_AXISMODE_set},
{"LibreDWGc::Dwg_Header_Variables_AXISMODE_get", _wrap_Dwg_Header_Variables_AXISMODE_get},
{"LibreDWGc::Dwg_Header_Variables_AXISUNIT_set", _wrap_Dwg_Header_Variables_AXISUNIT_set},
{"LibreDWGc::Dwg_Header_Variables_AXISUNIT_get", _wrap_Dwg_Header_Variables_AXISUNIT_get},
{"LibreDWGc::Dwg_Header_Variables_VPOINTX_set", _wrap_Dwg_Header_Variables_VPOINTX_set},
{"LibreDWGc::Dwg_Header_Variables_VPOINTX_get", _wrap_Dwg_Header_Variables_VPOINTX_get},
{"LibreDWGc::Dwg_Header_Variables_VPOINTY_set", _wrap_Dwg_Header_Variables_VPOINTY_set},
{"LibreDWGc::Dwg_Header_Variables_VPOINTY_get", _wrap_Dwg_Header_Variables_VPOINTY_get},
{"LibreDWGc::Dwg_Header_Variables_VPOINTZ_set", _wrap_Dwg_Header_Variables_VPOINTZ_set},
{"LibreDWGc::Dwg_Header_Variables_VPOINTZ_get", _wrap_Dwg_Header_Variables_VPOINTZ_get},
{"LibreDWGc::Dwg_Header_Variables_VPOINTXALT_set", _wrap_Dwg_Header_Variables_VPOINTXALT_set},
{"LibreDWGc::Dwg_Header_Variables_VPOINTXALT_get", _wrap_Dwg_Header_Variables_VPOINTXALT_get},
{"LibreDWGc::Dwg_Header_Variables_VPOINTYALT_set", _wrap_Dwg_Header_Variables_VPOINTYALT_set},
{"LibreDWGc::Dwg_Header_Variables_VPOINTYALT_get", _wrap_Dwg_Header_Variables_VPOINTYALT_get},
{"LibreDWGc::Dwg_Header_Variables_VPOINTZALT_set", _wrap_Dwg_Header_Variables_VPOINTZALT_set},
{"LibreDWGc::Dwg_Header_Variables_VPOINTZALT_get", _wrap_Dwg_Header_Variables_VPOINTZALT_get},
{"LibreDWGc::Dwg_Header_Variables_flag_3d_set", _wrap_Dwg_Header_Variables_flag_3d_set},
{"LibreDWGc::Dwg_Header_Variables_flag_3d_get", _wrap_Dwg_Header_Variables_flag_3d_get},
{"LibreDWGc::Dwg_Header_Variables_UCSORG_set", _wrap_Dwg_Header_Variables_UCSORG_set},
{"LibreDWGc::Dwg_Header_Variables_UCSORG_get", _wrap_Dwg_Header_Variables_UCSORG_get},
{"LibreDWGc::Dwg_Header_Variables_UCSXDIR_set", _wrap_Dwg_Header_Variables_UCSXDIR_set},
{"LibreDWGc::Dwg_Header_Variables_UCSXDIR_get", _wrap_Dwg_Header_Variables_UCSXDIR_get},
{"LibreDWGc::Dwg_Header_Variables_UCSYDIR_set", _wrap_Dwg_Header_Variables_UCSYDIR_set},
{"LibreDWGc::Dwg_Header_Variables_UCSYDIR_get", _wrap_Dwg_Header_Variables_UCSYDIR_get},
{"LibreDWGc::Dwg_Header_Variables_UCSNAME_set", _wrap_Dwg_Header_Variables_UCSNAME_set},
{"LibreDWGc::Dwg_Header_Variables_UCSNAME_get", _wrap_Dwg_Header_Variables_UCSNAME_get},
{"LibreDWGc::Dwg_Header_Variables_UCSBASE_set", _wrap_Dwg_Header_Variables_UCSBASE_set},
{"LibreDWGc::Dwg_Header_Variables_UCSBASE_get", _wrap_Dwg_Header_Variables_UCSBASE_get},
{"LibreDWGc::Dwg_Header_Variables_UCSORTHOVIEW_set", _wrap_Dwg_Header_Variables_UCSORTHOVIEW_set},
{"LibreDWGc::Dwg_Header_Variables_UCSORTHOVIEW_get", _wrap_Dwg_Header_Variables_UCSORTHOVIEW_get},
{"LibreDWGc::Dwg_Header_Variables_UCSORTHOREF_set", _wrap_Dwg_Header_Variables_UCSORTHOREF_set},
{"LibreDWGc::Dwg_Header_Variables_UCSORTHOREF_get", _wrap_Dwg_Header_Variables_UCSORTHOREF_get},
{"LibreDWGc::Dwg_Header_Variables_UCSORGTOP_set", _wrap_Dwg_Header_Variables_UCSORGTOP_set},
{"LibreDWGc::Dwg_Header_Variables_UCSORGTOP_get", _wrap_Dwg_Header_Variables_UCSORGTOP_get},
{"LibreDWGc::Dwg_Header_Variables_UCSORGBOTTOM_set", _wrap_Dwg_Header_Variables_UCSORGBOTTOM_set},
{"LibreDWGc::Dwg_Header_Variables_UCSORGBOTTOM_get", _wrap_Dwg_Header_Variables_UCSORGBOTTOM_get},
{"LibreDWGc::Dwg_Header_Variables_UCSORGLEFT_set", _wrap_Dwg_Header_Variables_UCSORGLEFT_set},
{"LibreDWGc::Dwg_Header_Variables_UCSORGLEFT_get", _wrap_Dwg_Header_Variables_UCSORGLEFT_get},
{"LibreDWGc::Dwg_Header_Variables_UCSORGRIGHT_set", _wrap_Dwg_Header_Variables_UCSORGRIGHT_set},
{"LibreDWGc::Dwg_Header_Variables_UCSORGRIGHT_get", _wrap_Dwg_Header_Variables_UCSORGRIGHT_get},
{"LibreDWGc::Dwg_Header_Variables_UCSORGFRONT_set", _wrap_Dwg_Header_Variables_UCSORGFRONT_set},
{"LibreDWGc::Dwg_Header_Variables_UCSORGFRONT_get", _wrap_Dwg_Header_Variables_UCSORGFRONT_get},
{"LibreDWGc::Dwg_Header_Variables_UCSORGBACK_set", _wrap_Dwg_Header_Variables_UCSORGBACK_set},
{"LibreDWGc::Dwg_Header_Variables_UCSORGBACK_get", _wrap_Dwg_Header_Variables_UCSORGBACK_get},
{"LibreDWGc::Dwg_Header_Variables_DIMPOST_set", _wrap_Dwg_Header_Variables_DIMPOST_set},
{"LibreDWGc::Dwg_Header_Variables_DIMPOST_get", _wrap_Dwg_Header_Variables_DIMPOST_get},
{"LibreDWGc::Dwg_Header_Variables_DIMAPOST_set", _wrap_Dwg_Header_Variables_DIMAPOST_set},
{"LibreDWGc::Dwg_Header_Variables_DIMAPOST_get", _wrap_Dwg_Header_Variables_DIMAPOST_get},
{"LibreDWGc::Dwg_Header_Variables_DIMTOL_set", _wrap_Dwg_Header_Variables_DIMTOL_set},
{"LibreDWGc::Dwg_Header_Variables_DIMTOL_get", _wrap_Dwg_Header_Variables_DIMTOL_get},
{"LibreDWGc::Dwg_Header_Variables_DIMLIM_set", _wrap_Dwg_Header_Variables_DIMLIM_set},
{"LibreDWGc::Dwg_Header_Variables_DIMLIM_get", _wrap_Dwg_Header_Variables_DIMLIM_get},
{"LibreDWGc::Dwg_Header_Variables_DIMTIH_set", _wrap_Dwg_Header_Variables_DIMTIH_set},
{"LibreDWGc::Dwg_Header_Variables_DIMTIH_get", _wrap_Dwg_Header_Variables_DIMTIH_get},
{"LibreDWGc::Dwg_Header_Variables_DIMTOH_set", _wrap_Dwg_Header_Variables_DIMTOH_set},
{"LibreDWGc::Dwg_Header_Variables_DIMTOH_get", _wrap_Dwg_Header_Variables_DIMTOH_get},
{"LibreDWGc::Dwg_Header_Variables_DIMSE1_set", _wrap_Dwg_Header_Variables_DIMSE1_set},
{"LibreDWGc::Dwg_Header_Variables_DIMSE1_get", _wrap_Dwg_Header_Variables_DIMSE1_get},
{"LibreDWGc::Dwg_Header_Variables_DIMSE2_set", _wrap_Dwg_Header_Variables_DIMSE2_set},
{"LibreDWGc::Dwg_Header_Variables_DIMSE2_get", _wrap_Dwg_Header_Variables_DIMSE2_get},
{"LibreDWGc::Dwg_Header_Variables_DIMALT_set", _wrap_Dwg_Header_Variables_DIMALT_set},
{"LibreDWGc::Dwg_Header_Variables_DIMALT_get", _wrap_Dwg_Header_Variables_DIMALT_get},
{"LibreDWGc::Dwg_Header_Variables_DIMTOFL_set", _wrap_Dwg_Header_Variables_DIMTOFL_set},
{"LibreDWGc::Dwg_Header_Variables_DIMTOFL_get", _wrap_Dwg_Header_Variables_DIMTOFL_get},
{"LibreDWGc::Dwg_Header_Variables_DIMSAH_set", _wrap_Dwg_Header_Variables_DIMSAH_set},
{"LibreDWGc::Dwg_Header_Variables_DIMSAH_get", _wrap_Dwg_Header_Variables_DIMSAH_get},
{"LibreDWGc::Dwg_Header_Variables_DIMTIX_set", _wrap_Dwg_Header_Variables_DIMTIX_set},
{"LibreDWGc::Dwg_Header_Variables_DIMTIX_get", _wrap_Dwg_Header_Variables_DIMTIX_get},
{"LibreDWGc::Dwg_Header_Variables_DIMSOXD_set", _wrap_Dwg_Header_Variables_DIMSOXD_set},
{"LibreDWGc::Dwg_Header_Variables_DIMSOXD_get", _wrap_Dwg_Header_Variables_DIMSOXD_get},
{"LibreDWGc::Dwg_Header_Variables_DIMALTD_set", _wrap_Dwg_Header_Variables_DIMALTD_set},
{"LibreDWGc::Dwg_Header_Variables_DIMALTD_get", _wrap_Dwg_Header_Variables_DIMALTD_get},
{"LibreDWGc::Dwg_Header_Variables_DIMZIN_set", _wrap_Dwg_Header_Variables_DIMZIN_set},
{"LibreDWGc::Dwg_Header_Variables_DIMZIN_get", _wrap_Dwg_Header_Variables_DIMZIN_get},
{"LibreDWGc::Dwg_Header_Variables_DIMSD1_set", _wrap_Dwg_Header_Variables_DIMSD1_set},
{"LibreDWGc::Dwg_Header_Variables_DIMSD1_get", _wrap_Dwg_Header_Variables_DIMSD1_get},
{"LibreDWGc::Dwg_Header_Variables_DIMSD2_set", _wrap_Dwg_Header_Variables_DIMSD2_set},
{"LibreDWGc::Dwg_Header_Variables_DIMSD2_get", _wrap_Dwg_Header_Variables_DIMSD2_get},
{"LibreDWGc::Dwg_Header_Variables_DIMTOLJ_set", _wrap_Dwg_Header_Variables_DIMTOLJ_set},
{"LibreDWGc::Dwg_Header_Variables_DIMTOLJ_get", _wrap_Dwg_Header_Variables_DIMTOLJ_get},
{"LibreDWGc::Dwg_Header_Variables_DIMJUST_set", _wrap_Dwg_Header_Variables_DIMJUST_set},
{"LibreDWGc::Dwg_Header_Variables_DIMJUST_get", _wrap_Dwg_Header_Variables_DIMJUST_get},
{"LibreDWGc::Dwg_Header_Variables_DIMFIT_set", _wrap_Dwg_Header_Variables_DIMFIT_set},
{"LibreDWGc::Dwg_Header_Variables_DIMFIT_get", _wrap_Dwg_Header_Variables_DIMFIT_get},
{"LibreDWGc::Dwg_Header_Variables_DIMUPT_set", _wrap_Dwg_Header_Variables_DIMUPT_set},
{"LibreDWGc::Dwg_Header_Variables_DIMUPT_get", _wrap_Dwg_Header_Variables_DIMUPT_get},
{"LibreDWGc::Dwg_Header_Variables_DIMTZIN_set", _wrap_Dwg_Header_Variables_DIMTZIN_set},
{"LibreDWGc::Dwg_Header_Variables_DIMTZIN_get", _wrap_Dwg_Header_Variables_DIMTZIN_get},
{"LibreDWGc::Dwg_Header_Variables_DIMTAD_set", _wrap_Dwg_Header_Variables_DIMTAD_set},
{"LibreDWGc::Dwg_Header_Variables_DIMTAD_get", _wrap_Dwg_Header_Variables_DIMTAD_get},
{"LibreDWGc::Dwg_Header_Variables_DIMUNIT_set", _wrap_Dwg_Header_Variables_DIMUNIT_set},
{"LibreDWGc::Dwg_Header_Variables_DIMUNIT_get", _wrap_Dwg_Header_Variables_DIMUNIT_get},
{"LibreDWGc::Dwg_Header_Variables_DIMAUNIT_set", _wrap_Dwg_Header_Variables_DIMAUNIT_set},
{"LibreDWGc::Dwg_Header_Variables_DIMAUNIT_get", _wrap_Dwg_Header_Variables_DIMAUNIT_get},
{"LibreDWGc::Dwg_Header_Variables_DIMDEC_set", _wrap_Dwg_Header_Variables_DIMDEC_set},
{"LibreDWGc::Dwg_Header_Variables_DIMDEC_get", _wrap_Dwg_Header_Variables_DIMDEC_get},
{"LibreDWGc::Dwg_Header_Variables_DIMTDEC_set", _wrap_Dwg_Header_Variables_DIMTDEC_set},
{"LibreDWGc::Dwg_Header_Variables_DIMTDEC_get", _wrap_Dwg_Header_Variables_DIMTDEC_get},
{"LibreDWGc::Dwg_Header_Variables_DIMALTU_set", _wrap_Dwg_Header_Variables_DIMALTU_set},
{"LibreDWGc::Dwg_Header_Variables_DIMALTU_get", _wrap_Dwg_Header_Variables_DIMALTU_get},
{"LibreDWGc::Dwg_Header_Variables_DIMALTTD_set", _wrap_Dwg_Header_Variables_DIMALTTD_set},
{"LibreDWGc::Dwg_Header_Variables_DIMALTTD_get", _wrap_Dwg_Header_Variables_DIMALTTD_get},
{"LibreDWGc::Dwg_Header_Variables_DIMTXSTY_set", _wrap_Dwg_Header_Variables_DIMTXSTY_set},
{"LibreDWGc::Dwg_Header_Variables_DIMTXSTY_get", _wrap_Dwg_Header_Variables_DIMTXSTY_get},
{"LibreDWGc::Dwg_Header_Variables_DIMSCALE_set", _wrap_Dwg_Header_Variables_DIMSCALE_set},
{"LibreDWGc::Dwg_Header_Variables_DIMSCALE_get", _wrap_Dwg_Header_Variables_DIMSCALE_get},
{"LibreDWGc::Dwg_Header_Variables_DIMARROW_set", _wrap_Dwg_Header_Variables_DIMARROW_set},
{"LibreDWGc::Dwg_Header_Variables_DIMARROW_get", _wrap_Dwg_Header_Variables_DIMARROW_get},
{"LibreDWGc::Dwg_Header_Variables_DIMASZ_set", _wrap_Dwg_Header_Variables_DIMASZ_set},
{"LibreDWGc::Dwg_Header_Variables_DIMASZ_get", _wrap_Dwg_Header_Variables_DIMASZ_get},
{"LibreDWGc::Dwg_Header_Variables_DIMEXO_set", _wrap_Dwg_Header_Variables_DIMEXO_set},
{"LibreDWGc::Dwg_Header_Variables_DIMEXO_get", _wrap_Dwg_Header_Variables_DIMEXO_get},
{"LibreDWGc::Dwg_Header_Variables_DIMDLI_set", _wrap_Dwg_Header_Variables_DIMDLI_set},
{"LibreDWGc::Dwg_Header_Variables_DIMDLI_get", _wrap_Dwg_Header_Variables_DIMDLI_get},
{"LibreDWGc::Dwg_Header_Variables_DIMEXE_set", _wrap_Dwg_Header_Variables_DIMEXE_set},
{"LibreDWGc::Dwg_Header_Variables_DIMEXE_get", _wrap_Dwg_Header_Variables_DIMEXE_get},
{"LibreDWGc::Dwg_Header_Variables_DIMRND_set", _wrap_Dwg_Header_Variables_DIMRND_set},
{"LibreDWGc::Dwg_Header_Variables_DIMRND_get", _wrap_Dwg_Header_Variables_DIMRND_get},
{"LibreDWGc::Dwg_Header_Variables_DIMDLE_set", _wrap_Dwg_Header_Variables_DIMDLE_set},
{"LibreDWGc::Dwg_Header_Variables_DIMDLE_get", _wrap_Dwg_Header_Variables_DIMDLE_get},
{"LibreDWGc::Dwg_Header_Variables_DIMTP_set", _wrap_Dwg_Header_Variables_DIMTP_set},
{"LibreDWGc::Dwg_Header_Variables_DIMTP_get", _wrap_Dwg_Header_Variables_DIMTP_get},
{"LibreDWGc::Dwg_Header_Variables_DIMTM_set", _wrap_Dwg_Header_Variables_DIMTM_set},
{"LibreDWGc::Dwg_Header_Variables_DIMTM_get", _wrap_Dwg_Header_Variables_DIMTM_get},
{"LibreDWGc::Dwg_Header_Variables_DIMFXL_set", _wrap_Dwg_Header_Variables_DIMFXL_set},
{"LibreDWGc::Dwg_Header_Variables_DIMFXL_get", _wrap_Dwg_Header_Variables_DIMFXL_get},
{"LibreDWGc::Dwg_Header_Variables_DIMJOGANG_set", _wrap_Dwg_Header_Variables_DIMJOGANG_set},
{"LibreDWGc::Dwg_Header_Variables_DIMJOGANG_get", _wrap_Dwg_Header_Variables_DIMJOGANG_get},
{"LibreDWGc::Dwg_Header_Variables_DIMTFILL_set", _wrap_Dwg_Header_Variables_DIMTFILL_set},
{"LibreDWGc::Dwg_Header_Variables_DIMTFILL_get", _wrap_Dwg_Header_Variables_DIMTFILL_get},
{"LibreDWGc::Dwg_Header_Variables_DIMTFILLCLR_set", _wrap_Dwg_Header_Variables_DIMTFILLCLR_set},
{"LibreDWGc::Dwg_Header_Variables_DIMTFILLCLR_get", _wrap_Dwg_Header_Variables_DIMTFILLCLR_get},
{"LibreDWGc::Dwg_Header_Variables_DIMAZIN_set", _wrap_Dwg_Header_Variables_DIMAZIN_set},
{"LibreDWGc::Dwg_Header_Variables_DIMAZIN_get", _wrap_Dwg_Header_Variables_DIMAZIN_get},
{"LibreDWGc::Dwg_Header_Variables_DIMARCSYM_set", _wrap_Dwg_Header_Variables_DIMARCSYM_set},
{"LibreDWGc::Dwg_Header_Variables_DIMARCSYM_get", _wrap_Dwg_Header_Variables_DIMARCSYM_get},
{"LibreDWGc::Dwg_Header_Variables_DIMTXT_set", _wrap_Dwg_Header_Variables_DIMTXT_set},
{"LibreDWGc::Dwg_Header_Variables_DIMTXT_get", _wrap_Dwg_Header_Variables_DIMTXT_get},
{"LibreDWGc::Dwg_Header_Variables_DIMCEN_set", _wrap_Dwg_Header_Variables_DIMCEN_set},
{"LibreDWGc::Dwg_Header_Variables_DIMCEN_get", _wrap_Dwg_Header_Variables_DIMCEN_get},
{"LibreDWGc::Dwg_Header_Variables_DIMTSZ_set", _wrap_Dwg_Header_Variables_DIMTSZ_set},
{"LibreDWGc::Dwg_Header_Variables_DIMTSZ_get", _wrap_Dwg_Header_Variables_DIMTSZ_get},
{"LibreDWGc::Dwg_Header_Variables_DIMALTF_set", _wrap_Dwg_Header_Variables_DIMALTF_set},
{"LibreDWGc::Dwg_Header_Variables_DIMALTF_get", _wrap_Dwg_Header_Variables_DIMALTF_get},
{"LibreDWGc::Dwg_Header_Variables_DIMLFAC_set", _wrap_Dwg_Header_Variables_DIMLFAC_set},
{"LibreDWGc::Dwg_Header_Variables_DIMLFAC_get", _wrap_Dwg_Header_Variables_DIMLFAC_get},
{"LibreDWGc::Dwg_Header_Variables_DIMTVP_set", _wrap_Dwg_Header_Variables_DIMTVP_set},
{"LibreDWGc::Dwg_Header_Variables_DIMTVP_get", _wrap_Dwg_Header_Variables_DIMTVP_get},
{"LibreDWGc::Dwg_Header_Variables_DIMTFAC_set", _wrap_Dwg_Header_Variables_DIMTFAC_set},
{"LibreDWGc::Dwg_Header_Variables_DIMTFAC_get", _wrap_Dwg_Header_Variables_DIMTFAC_get},
{"LibreDWGc::Dwg_Header_Variables_DIMGAP_set", _wrap_Dwg_Header_Variables_DIMGAP_set},
{"LibreDWGc::Dwg_Header_Variables_DIMGAP_get", _wrap_Dwg_Header_Variables_DIMGAP_get},
{"LibreDWGc::Dwg_Header_Variables_DIMPOST_T_set", _wrap_Dwg_Header_Variables_DIMPOST_T_set},
{"LibreDWGc::Dwg_Header_Variables_DIMPOST_T_get", _wrap_Dwg_Header_Variables_DIMPOST_T_get},
{"LibreDWGc::Dwg_Header_Variables_DIMAPOST_T_set", _wrap_Dwg_Header_Variables_DIMAPOST_T_set},
{"LibreDWGc::Dwg_Header_Variables_DIMAPOST_T_get", _wrap_Dwg_Header_Variables_DIMAPOST_T_get},
{"LibreDWGc::Dwg_Header_Variables_DIMBLK_T_set", _wrap_Dwg_Header_Variables_DIMBLK_T_set},
{"LibreDWGc::Dwg_Header_Variables_DIMBLK_T_get", _wrap_Dwg_Header_Variables_DIMBLK_T_get},
{"LibreDWGc::Dwg_Header_Variables_DIMBLK1_T_set", _wrap_Dwg_Header_Variables_DIMBLK1_T_set},
{"LibreDWGc::Dwg_Header_Variables_DIMBLK1_T_get", _wrap_Dwg_Header_Variables_DIMBLK1_T_get},
{"LibreDWGc::Dwg_Header_Variables_DIMBLK2_T_set", _wrap_Dwg_Header_Variables_DIMBLK2_T_set},
{"LibreDWGc::Dwg_Header_Variables_DIMBLK2_T_get", _wrap_Dwg_Header_Variables_DIMBLK2_T_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_string_set", _wrap_Dwg_Header_Variables_unknown_string_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_string_get", _wrap_Dwg_Header_Variables_unknown_string_get},
{"LibreDWGc::Dwg_Header_Variables_DIMALTRND_set", _wrap_Dwg_Header_Variables_DIMALTRND_set},
{"LibreDWGc::Dwg_Header_Variables_DIMALTRND_get", _wrap_Dwg_Header_Variables_DIMALTRND_get},
{"LibreDWGc::Dwg_Header_Variables_DIMCLRD_C_set", _wrap_Dwg_Header_Variables_DIMCLRD_C_set},
{"LibreDWGc::Dwg_Header_Variables_DIMCLRD_C_get", _wrap_Dwg_Header_Variables_DIMCLRD_C_get},
{"LibreDWGc::Dwg_Header_Variables_DIMCLRE_C_set", _wrap_Dwg_Header_Variables_DIMCLRE_C_set},
{"LibreDWGc::Dwg_Header_Variables_DIMCLRE_C_get", _wrap_Dwg_Header_Variables_DIMCLRE_C_get},
{"LibreDWGc::Dwg_Header_Variables_DIMCLRT_C_set", _wrap_Dwg_Header_Variables_DIMCLRT_C_set},
{"LibreDWGc::Dwg_Header_Variables_DIMCLRT_C_get", _wrap_Dwg_Header_Variables_DIMCLRT_C_get},
{"LibreDWGc::Dwg_Header_Variables_DIMCLRD_set", _wrap_Dwg_Header_Variables_DIMCLRD_set},
{"LibreDWGc::Dwg_Header_Variables_DIMCLRD_get", _wrap_Dwg_Header_Variables_DIMCLRD_get},
{"LibreDWGc::Dwg_Header_Variables_DIMCLRE_set", _wrap_Dwg_Header_Variables_DIMCLRE_set},
{"LibreDWGc::Dwg_Header_Variables_DIMCLRE_get", _wrap_Dwg_Header_Variables_DIMCLRE_get},
{"LibreDWGc::Dwg_Header_Variables_DIMCLRT_set", _wrap_Dwg_Header_Variables_DIMCLRT_set},
{"LibreDWGc::Dwg_Header_Variables_DIMCLRT_get", _wrap_Dwg_Header_Variables_DIMCLRT_get},
{"LibreDWGc::Dwg_Header_Variables_DIMADEC_set", _wrap_Dwg_Header_Variables_DIMADEC_set},
{"LibreDWGc::Dwg_Header_Variables_DIMADEC_get", _wrap_Dwg_Header_Variables_DIMADEC_get},
{"LibreDWGc::Dwg_Header_Variables_DIMFRAC_set", _wrap_Dwg_Header_Variables_DIMFRAC_set},
{"LibreDWGc::Dwg_Header_Variables_DIMFRAC_get", _wrap_Dwg_Header_Variables_DIMFRAC_get},
{"LibreDWGc::Dwg_Header_Variables_DIMLUNIT_set", _wrap_Dwg_Header_Variables_DIMLUNIT_set},
{"LibreDWGc::Dwg_Header_Variables_DIMLUNIT_get", _wrap_Dwg_Header_Variables_DIMLUNIT_get},
{"LibreDWGc::Dwg_Header_Variables_DIMDSEP_set", _wrap_Dwg_Header_Variables_DIMDSEP_set},
{"LibreDWGc::Dwg_Header_Variables_DIMDSEP_get", _wrap_Dwg_Header_Variables_DIMDSEP_get},
{"LibreDWGc::Dwg_Header_Variables_DIMTMOVE_set", _wrap_Dwg_Header_Variables_DIMTMOVE_set},
{"LibreDWGc::Dwg_Header_Variables_DIMTMOVE_get", _wrap_Dwg_Header_Variables_DIMTMOVE_get},
{"LibreDWGc::Dwg_Header_Variables_DIMALTZ_set", _wrap_Dwg_Header_Variables_DIMALTZ_set},
{"LibreDWGc::Dwg_Header_Variables_DIMALTZ_get", _wrap_Dwg_Header_Variables_DIMALTZ_get},
{"LibreDWGc::Dwg_Header_Variables_DIMALTTZ_set", _wrap_Dwg_Header_Variables_DIMALTTZ_set},
{"LibreDWGc::Dwg_Header_Variables_DIMALTTZ_get", _wrap_Dwg_Header_Variables_DIMALTTZ_get},
{"LibreDWGc::Dwg_Header_Variables_DIMATFIT_set", _wrap_Dwg_Header_Variables_DIMATFIT_set},
{"LibreDWGc::Dwg_Header_Variables_DIMATFIT_get", _wrap_Dwg_Header_Variables_DIMATFIT_get},
{"LibreDWGc::Dwg_Header_Variables_DIMFXLON_set", _wrap_Dwg_Header_Variables_DIMFXLON_set},
{"LibreDWGc::Dwg_Header_Variables_DIMFXLON_get", _wrap_Dwg_Header_Variables_DIMFXLON_get},
{"LibreDWGc::Dwg_Header_Variables_DIMTXTDIRECTION_set", _wrap_Dwg_Header_Variables_DIMTXTDIRECTION_set},
{"LibreDWGc::Dwg_Header_Variables_DIMTXTDIRECTION_get", _wrap_Dwg_Header_Variables_DIMTXTDIRECTION_get},
{"LibreDWGc::Dwg_Header_Variables_DIMALTMZF_set", _wrap_Dwg_Header_Variables_DIMALTMZF_set},
{"LibreDWGc::Dwg_Header_Variables_DIMALTMZF_get", _wrap_Dwg_Header_Variables_DIMALTMZF_get},
{"LibreDWGc::Dwg_Header_Variables_DIMALTMZS_set", _wrap_Dwg_Header_Variables_DIMALTMZS_set},
{"LibreDWGc::Dwg_Header_Variables_DIMALTMZS_get", _wrap_Dwg_Header_Variables_DIMALTMZS_get},
{"LibreDWGc::Dwg_Header_Variables_DIMMZF_set", _wrap_Dwg_Header_Variables_DIMMZF_set},
{"LibreDWGc::Dwg_Header_Variables_DIMMZF_get", _wrap_Dwg_Header_Variables_DIMMZF_get},
{"LibreDWGc::Dwg_Header_Variables_DIMMZS_set", _wrap_Dwg_Header_Variables_DIMMZS_set},
{"LibreDWGc::Dwg_Header_Variables_DIMMZS_get", _wrap_Dwg_Header_Variables_DIMMZS_get},
{"LibreDWGc::Dwg_Header_Variables_DIMLDRBLK_set", _wrap_Dwg_Header_Variables_DIMLDRBLK_set},
{"LibreDWGc::Dwg_Header_Variables_DIMLDRBLK_get", _wrap_Dwg_Header_Variables_DIMLDRBLK_get},
{"LibreDWGc::Dwg_Header_Variables_DIMBLK_set", _wrap_Dwg_Header_Variables_DIMBLK_set},
{"LibreDWGc::Dwg_Header_Variables_DIMBLK_get", _wrap_Dwg_Header_Variables_DIMBLK_get},
{"LibreDWGc::Dwg_Header_Variables_DIMBLK1_set", _wrap_Dwg_Header_Variables_DIMBLK1_set},
{"LibreDWGc::Dwg_Header_Variables_DIMBLK1_get", _wrap_Dwg_Header_Variables_DIMBLK1_get},
{"LibreDWGc::Dwg_Header_Variables_DIMBLK2_set", _wrap_Dwg_Header_Variables_DIMBLK2_set},
{"LibreDWGc::Dwg_Header_Variables_DIMBLK2_get", _wrap_Dwg_Header_Variables_DIMBLK2_get},
{"LibreDWGc::Dwg_Header_Variables_DIMLTYPE_set", _wrap_Dwg_Header_Variables_DIMLTYPE_set},
{"LibreDWGc::Dwg_Header_Variables_DIMLTYPE_get", _wrap_Dwg_Header_Variables_DIMLTYPE_get},
{"LibreDWGc::Dwg_Header_Variables_DIMLTEX1_set", _wrap_Dwg_Header_Variables_DIMLTEX1_set},
{"LibreDWGc::Dwg_Header_Variables_DIMLTEX1_get", _wrap_Dwg_Header_Variables_DIMLTEX1_get},
{"LibreDWGc::Dwg_Header_Variables_DIMLTEX2_set", _wrap_Dwg_Header_Variables_DIMLTEX2_set},
{"LibreDWGc::Dwg_Header_Variables_DIMLTEX2_get", _wrap_Dwg_Header_Variables_DIMLTEX2_get},
{"LibreDWGc::Dwg_Header_Variables_DIMLWD_set", _wrap_Dwg_Header_Variables_DIMLWD_set},
{"LibreDWGc::Dwg_Header_Variables_DIMLWD_get", _wrap_Dwg_Header_Variables_DIMLWD_get},
{"LibreDWGc::Dwg_Header_Variables_DIMLWE_set", _wrap_Dwg_Header_Variables_DIMLWE_set},
{"LibreDWGc::Dwg_Header_Variables_DIMLWE_get", _wrap_Dwg_Header_Variables_DIMLWE_get},
{"LibreDWGc::Dwg_Header_Variables_BLOCK_CONTROL_OBJECT_set", _wrap_Dwg_Header_Variables_BLOCK_CONTROL_OBJECT_set},
{"LibreDWGc::Dwg_Header_Variables_BLOCK_CONTROL_OBJECT_get", _wrap_Dwg_Header_Variables_BLOCK_CONTROL_OBJECT_get},
{"LibreDWGc::Dwg_Header_Variables_LAYER_CONTROL_OBJECT_set", _wrap_Dwg_Header_Variables_LAYER_CONTROL_OBJECT_set},
{"LibreDWGc::Dwg_Header_Variables_LAYER_CONTROL_OBJECT_get", _wrap_Dwg_Header_Variables_LAYER_CONTROL_OBJECT_get},
{"LibreDWGc::Dwg_Header_Variables_STYLE_CONTROL_OBJECT_set", _wrap_Dwg_Header_Variables_STYLE_CONTROL_OBJECT_set},
{"LibreDWGc::Dwg_Header_Variables_STYLE_CONTROL_OBJECT_get", _wrap_Dwg_Header_Variables_STYLE_CONTROL_OBJECT_get},
{"LibreDWGc::Dwg_Header_Variables_LTYPE_CONTROL_OBJECT_set", _wrap_Dwg_Header_Variables_LTYPE_CONTROL_OBJECT_set},
{"LibreDWGc::Dwg_Header_Variables_LTYPE_CONTROL_OBJECT_get", _wrap_Dwg_Header_Variables_LTYPE_CONTROL_OBJECT_get},
{"LibreDWGc::Dwg_Header_Variables_VIEW_CONTROL_OBJECT_set", _wrap_Dwg_Header_Variables_VIEW_CONTROL_OBJECT_set},
{"LibreDWGc::Dwg_Header_Variables_VIEW_CONTROL_OBJECT_get", _wrap_Dwg_Header_Variables_VIEW_CONTROL_OBJECT_get},
{"LibreDWGc::Dwg_Header_Variables_UCS_CONTROL_OBJECT_set", _wrap_Dwg_Header_Variables_UCS_CONTROL_OBJECT_set},
{"LibreDWGc::Dwg_Header_Variables_UCS_CONTROL_OBJECT_get", _wrap_Dwg_Header_Variables_UCS_CONTROL_OBJECT_get},
{"LibreDWGc::Dwg_Header_Variables_VPORT_CONTROL_OBJECT_set", _wrap_Dwg_Header_Variables_VPORT_CONTROL_OBJECT_set},
{"LibreDWGc::Dwg_Header_Variables_VPORT_CONTROL_OBJECT_get", _wrap_Dwg_Header_Variables_VPORT_CONTROL_OBJECT_get},
{"LibreDWGc::Dwg_Header_Variables_APPID_CONTROL_OBJECT_set", _wrap_Dwg_Header_Variables_APPID_CONTROL_OBJECT_set},
{"LibreDWGc::Dwg_Header_Variables_APPID_CONTROL_OBJECT_get", _wrap_Dwg_Header_Variables_APPID_CONTROL_OBJECT_get},
{"LibreDWGc::Dwg_Header_Variables_DIMSTYLE_CONTROL_OBJECT_set", _wrap_Dwg_Header_Variables_DIMSTYLE_CONTROL_OBJECT_set},
{"LibreDWGc::Dwg_Header_Variables_DIMSTYLE_CONTROL_OBJECT_get", _wrap_Dwg_Header_Variables_DIMSTYLE_CONTROL_OBJECT_get},
{"LibreDWGc::Dwg_Header_Variables_VX_CONTROL_OBJECT_set", _wrap_Dwg_Header_Variables_VX_CONTROL_OBJECT_set},
{"LibreDWGc::Dwg_Header_Variables_VX_CONTROL_OBJECT_get", _wrap_Dwg_Header_Variables_VX_CONTROL_OBJECT_get},
{"LibreDWGc::Dwg_Header_Variables_DICTIONARY_ACAD_GROUP_set", _wrap_Dwg_Header_Variables_DICTIONARY_ACAD_GROUP_set},
{"LibreDWGc::Dwg_Header_Variables_DICTIONARY_ACAD_GROUP_get", _wrap_Dwg_Header_Variables_DICTIONARY_ACAD_GROUP_get},
{"LibreDWGc::Dwg_Header_Variables_DICTIONARY_ACAD_MLINESTYLE_set", _wrap_Dwg_Header_Variables_DICTIONARY_ACAD_MLINESTYLE_set},
{"LibreDWGc::Dwg_Header_Variables_DICTIONARY_ACAD_MLINESTYLE_get", _wrap_Dwg_Header_Variables_DICTIONARY_ACAD_MLINESTYLE_get},
{"LibreDWGc::Dwg_Header_Variables_DICTIONARY_NAMED_OBJECT_set", _wrap_Dwg_Header_Variables_DICTIONARY_NAMED_OBJECT_set},
{"LibreDWGc::Dwg_Header_Variables_DICTIONARY_NAMED_OBJECT_get", _wrap_Dwg_Header_Variables_DICTIONARY_NAMED_OBJECT_get},
{"LibreDWGc::Dwg_Header_Variables_TSTACKALIGN_set", _wrap_Dwg_Header_Variables_TSTACKALIGN_set},
{"LibreDWGc::Dwg_Header_Variables_TSTACKALIGN_get", _wrap_Dwg_Header_Variables_TSTACKALIGN_get},
{"LibreDWGc::Dwg_Header_Variables_TSTACKSIZE_set", _wrap_Dwg_Header_Variables_TSTACKSIZE_set},
{"LibreDWGc::Dwg_Header_Variables_TSTACKSIZE_get", _wrap_Dwg_Header_Variables_TSTACKSIZE_get},
{"LibreDWGc::Dwg_Header_Variables_HYPERLINKBASE_set", _wrap_Dwg_Header_Variables_HYPERLINKBASE_set},
{"LibreDWGc::Dwg_Header_Variables_HYPERLINKBASE_get", _wrap_Dwg_Header_Variables_HYPERLINKBASE_get},
{"LibreDWGc::Dwg_Header_Variables_STYLESHEET_set", _wrap_Dwg_Header_Variables_STYLESHEET_set},
{"LibreDWGc::Dwg_Header_Variables_STYLESHEET_get", _wrap_Dwg_Header_Variables_STYLESHEET_get},
{"LibreDWGc::Dwg_Header_Variables_DICTIONARY_LAYOUT_set", _wrap_Dwg_Header_Variables_DICTIONARY_LAYOUT_set},
{"LibreDWGc::Dwg_Header_Variables_DICTIONARY_LAYOUT_get", _wrap_Dwg_Header_Variables_DICTIONARY_LAYOUT_get},
{"LibreDWGc::Dwg_Header_Variables_DICTIONARY_PLOTSETTINGS_set", _wrap_Dwg_Header_Variables_DICTIONARY_PLOTSETTINGS_set},
{"LibreDWGc::Dwg_Header_Variables_DICTIONARY_PLOTSETTINGS_get", _wrap_Dwg_Header_Variables_DICTIONARY_PLOTSETTINGS_get},
{"LibreDWGc::Dwg_Header_Variables_DICTIONARY_PLOTSTYLENAME_set", _wrap_Dwg_Header_Variables_DICTIONARY_PLOTSTYLENAME_set},
{"LibreDWGc::Dwg_Header_Variables_DICTIONARY_PLOTSTYLENAME_get", _wrap_Dwg_Header_Variables_DICTIONARY_PLOTSTYLENAME_get},
{"LibreDWGc::Dwg_Header_Variables_DICTIONARY_MATERIAL_set", _wrap_Dwg_Header_Variables_DICTIONARY_MATERIAL_set},
{"LibreDWGc::Dwg_Header_Variables_DICTIONARY_MATERIAL_get", _wrap_Dwg_Header_Variables_DICTIONARY_MATERIAL_get},
{"LibreDWGc::Dwg_Header_Variables_DICTIONARY_COLOR_set", _wrap_Dwg_Header_Variables_DICTIONARY_COLOR_set},
{"LibreDWGc::Dwg_Header_Variables_DICTIONARY_COLOR_get", _wrap_Dwg_Header_Variables_DICTIONARY_COLOR_get},
{"LibreDWGc::Dwg_Header_Variables_DICTIONARY_VISUALSTYLE_set", _wrap_Dwg_Header_Variables_DICTIONARY_VISUALSTYLE_set},
{"LibreDWGc::Dwg_Header_Variables_DICTIONARY_VISUALSTYLE_get", _wrap_Dwg_Header_Variables_DICTIONARY_VISUALSTYLE_get},
{"LibreDWGc::Dwg_Header_Variables_DICTIONARY_LIGHTLIST_set", _wrap_Dwg_Header_Variables_DICTIONARY_LIGHTLIST_set},
{"LibreDWGc::Dwg_Header_Variables_DICTIONARY_LIGHTLIST_get", _wrap_Dwg_Header_Variables_DICTIONARY_LIGHTLIST_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_20_set", _wrap_Dwg_Header_Variables_unknown_20_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_20_get", _wrap_Dwg_Header_Variables_unknown_20_get},
{"LibreDWGc::Dwg_Header_Variables_FLAGS_set", _wrap_Dwg_Header_Variables_FLAGS_set},
{"LibreDWGc::Dwg_Header_Variables_FLAGS_get", _wrap_Dwg_Header_Variables_FLAGS_get},
{"LibreDWGc::Dwg_Header_Variables_CELWEIGHT_set", _wrap_Dwg_Header_Variables_CELWEIGHT_set},
{"LibreDWGc::Dwg_Header_Variables_CELWEIGHT_get", _wrap_Dwg_Header_Variables_CELWEIGHT_get},
{"LibreDWGc::Dwg_Header_Variables_ENDCAPS_set", _wrap_Dwg_Header_Variables_ENDCAPS_set},
{"LibreDWGc::Dwg_Header_Variables_ENDCAPS_get", _wrap_Dwg_Header_Variables_ENDCAPS_get},
{"LibreDWGc::Dwg_Header_Variables_JOINSTYLE_set", _wrap_Dwg_Header_Variables_JOINSTYLE_set},
{"LibreDWGc::Dwg_Header_Variables_JOINSTYLE_get", _wrap_Dwg_Header_Variables_JOINSTYLE_get},
{"LibreDWGc::Dwg_Header_Variables_LWDISPLAY_set", _wrap_Dwg_Header_Variables_LWDISPLAY_set},
{"LibreDWGc::Dwg_Header_Variables_LWDISPLAY_get", _wrap_Dwg_Header_Variables_LWDISPLAY_get},
{"LibreDWGc::Dwg_Header_Variables_XEDIT_set", _wrap_Dwg_Header_Variables_XEDIT_set},
{"LibreDWGc::Dwg_Header_Variables_XEDIT_get", _wrap_Dwg_Header_Variables_XEDIT_get},
{"LibreDWGc::Dwg_Header_Variables_EXTNAMES_set", _wrap_Dwg_Header_Variables_EXTNAMES_set},
{"LibreDWGc::Dwg_Header_Variables_EXTNAMES_get", _wrap_Dwg_Header_Variables_EXTNAMES_get},
{"LibreDWGc::Dwg_Header_Variables_PSTYLEMODE_set", _wrap_Dwg_Header_Variables_PSTYLEMODE_set},
{"LibreDWGc::Dwg_Header_Variables_PSTYLEMODE_get", _wrap_Dwg_Header_Variables_PSTYLEMODE_get},
{"LibreDWGc::Dwg_Header_Variables_OLESTARTUP_set", _wrap_Dwg_Header_Variables_OLESTARTUP_set},
{"LibreDWGc::Dwg_Header_Variables_OLESTARTUP_get", _wrap_Dwg_Header_Variables_OLESTARTUP_get},
{"LibreDWGc::Dwg_Header_Variables_INSUNITS_set", _wrap_Dwg_Header_Variables_INSUNITS_set},
{"LibreDWGc::Dwg_Header_Variables_INSUNITS_get", _wrap_Dwg_Header_Variables_INSUNITS_get},
{"LibreDWGc::Dwg_Header_Variables_CEPSNTYPE_set", _wrap_Dwg_Header_Variables_CEPSNTYPE_set},
{"LibreDWGc::Dwg_Header_Variables_CEPSNTYPE_get", _wrap_Dwg_Header_Variables_CEPSNTYPE_get},
{"LibreDWGc::Dwg_Header_Variables_CPSNID_set", _wrap_Dwg_Header_Variables_CPSNID_set},
{"LibreDWGc::Dwg_Header_Variables_CPSNID_get", _wrap_Dwg_Header_Variables_CPSNID_get},
{"LibreDWGc::Dwg_Header_Variables_FINGERPRINTGUID_set", _wrap_Dwg_Header_Variables_FINGERPRINTGUID_set},
{"LibreDWGc::Dwg_Header_Variables_FINGERPRINTGUID_get", _wrap_Dwg_Header_Variables_FINGERPRINTGUID_get},
{"LibreDWGc::Dwg_Header_Variables_VERSIONGUID_set", _wrap_Dwg_Header_Variables_VERSIONGUID_set},
{"LibreDWGc::Dwg_Header_Variables_VERSIONGUID_get", _wrap_Dwg_Header_Variables_VERSIONGUID_get},
{"LibreDWGc::Dwg_Header_Variables_SORTENTS_set", _wrap_Dwg_Header_Variables_SORTENTS_set},
{"LibreDWGc::Dwg_Header_Variables_SORTENTS_get", _wrap_Dwg_Header_Variables_SORTENTS_get},
{"LibreDWGc::Dwg_Header_Variables_INDEXCTL_set", _wrap_Dwg_Header_Variables_INDEXCTL_set},
{"LibreDWGc::Dwg_Header_Variables_INDEXCTL_get", _wrap_Dwg_Header_Variables_INDEXCTL_get},
{"LibreDWGc::Dwg_Header_Variables_HIDETEXT_set", _wrap_Dwg_Header_Variables_HIDETEXT_set},
{"LibreDWGc::Dwg_Header_Variables_HIDETEXT_get", _wrap_Dwg_Header_Variables_HIDETEXT_get},
{"LibreDWGc::Dwg_Header_Variables_XCLIPFRAME_set", _wrap_Dwg_Header_Variables_XCLIPFRAME_set},
{"LibreDWGc::Dwg_Header_Variables_XCLIPFRAME_get", _wrap_Dwg_Header_Variables_XCLIPFRAME_get},
{"LibreDWGc::Dwg_Header_Variables_DIMASSOC_set", _wrap_Dwg_Header_Variables_DIMASSOC_set},
{"LibreDWGc::Dwg_Header_Variables_DIMASSOC_get", _wrap_Dwg_Header_Variables_DIMASSOC_get},
{"LibreDWGc::Dwg_Header_Variables_HALOGAP_set", _wrap_Dwg_Header_Variables_HALOGAP_set},
{"LibreDWGc::Dwg_Header_Variables_HALOGAP_get", _wrap_Dwg_Header_Variables_HALOGAP_get},
{"LibreDWGc::Dwg_Header_Variables_OBSCOLOR_set", _wrap_Dwg_Header_Variables_OBSCOLOR_set},
{"LibreDWGc::Dwg_Header_Variables_OBSCOLOR_get", _wrap_Dwg_Header_Variables_OBSCOLOR_get},
{"LibreDWGc::Dwg_Header_Variables_INTERSECTIONCOLOR_set", _wrap_Dwg_Header_Variables_INTERSECTIONCOLOR_set},
{"LibreDWGc::Dwg_Header_Variables_INTERSECTIONCOLOR_get", _wrap_Dwg_Header_Variables_INTERSECTIONCOLOR_get},
{"LibreDWGc::Dwg_Header_Variables_OBSLTYPE_set", _wrap_Dwg_Header_Variables_OBSLTYPE_set},
{"LibreDWGc::Dwg_Header_Variables_OBSLTYPE_get", _wrap_Dwg_Header_Variables_OBSLTYPE_get},
{"LibreDWGc::Dwg_Header_Variables_INTERSECTIONDISPLAY_set", _wrap_Dwg_Header_Variables_INTERSECTIONDISPLAY_set},
{"LibreDWGc::Dwg_Header_Variables_INTERSECTIONDISPLAY_get", _wrap_Dwg_Header_Variables_INTERSECTIONDISPLAY_get},
{"LibreDWGc::Dwg_Header_Variables_PROJECTNAME_set", _wrap_Dwg_Header_Variables_PROJECTNAME_set},
{"LibreDWGc::Dwg_Header_Variables_PROJECTNAME_get", _wrap_Dwg_Header_Variables_PROJECTNAME_get},
{"LibreDWGc::Dwg_Header_Variables_BLOCK_RECORD_PSPACE_set", _wrap_Dwg_Header_Variables_BLOCK_RECORD_PSPACE_set},
{"LibreDWGc::Dwg_Header_Variables_BLOCK_RECORD_PSPACE_get", _wrap_Dwg_Header_Variables_BLOCK_RECORD_PSPACE_get},
{"LibreDWGc::Dwg_Header_Variables_BLOCK_RECORD_MSPACE_set", _wrap_Dwg_Header_Variables_BLOCK_RECORD_MSPACE_set},
{"LibreDWGc::Dwg_Header_Variables_BLOCK_RECORD_MSPACE_get", _wrap_Dwg_Header_Variables_BLOCK_RECORD_MSPACE_get},
{"LibreDWGc::Dwg_Header_Variables_LTYPE_BYLAYER_set", _wrap_Dwg_Header_Variables_LTYPE_BYLAYER_set},
{"LibreDWGc::Dwg_Header_Variables_LTYPE_BYLAYER_get", _wrap_Dwg_Header_Variables_LTYPE_BYLAYER_get},
{"LibreDWGc::Dwg_Header_Variables_LTYPE_BYBLOCK_set", _wrap_Dwg_Header_Variables_LTYPE_BYBLOCK_set},
{"LibreDWGc::Dwg_Header_Variables_LTYPE_BYBLOCK_get", _wrap_Dwg_Header_Variables_LTYPE_BYBLOCK_get},
{"LibreDWGc::Dwg_Header_Variables_LTYPE_CONTINUOUS_set", _wrap_Dwg_Header_Variables_LTYPE_CONTINUOUS_set},
{"LibreDWGc::Dwg_Header_Variables_LTYPE_CONTINUOUS_get", _wrap_Dwg_Header_Variables_LTYPE_CONTINUOUS_get},
{"LibreDWGc::Dwg_Header_Variables_CAMERADISPLAY_set", _wrap_Dwg_Header_Variables_CAMERADISPLAY_set},
{"LibreDWGc::Dwg_Header_Variables_CAMERADISPLAY_get", _wrap_Dwg_Header_Variables_CAMERADISPLAY_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_21_set", _wrap_Dwg_Header_Variables_unknown_21_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_21_get", _wrap_Dwg_Header_Variables_unknown_21_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_22_set", _wrap_Dwg_Header_Variables_unknown_22_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_22_get", _wrap_Dwg_Header_Variables_unknown_22_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_23_set", _wrap_Dwg_Header_Variables_unknown_23_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_23_get", _wrap_Dwg_Header_Variables_unknown_23_get},
{"LibreDWGc::Dwg_Header_Variables_STEPSPERSEC_set", _wrap_Dwg_Header_Variables_STEPSPERSEC_set},
{"LibreDWGc::Dwg_Header_Variables_STEPSPERSEC_get", _wrap_Dwg_Header_Variables_STEPSPERSEC_get},
{"LibreDWGc::Dwg_Header_Variables_STEPSIZE_set", _wrap_Dwg_Header_Variables_STEPSIZE_set},
{"LibreDWGc::Dwg_Header_Variables_STEPSIZE_get", _wrap_Dwg_Header_Variables_STEPSIZE_get},
{"LibreDWGc::Dwg_Header_Variables__3DDWFPREC_set", _wrap_Dwg_Header_Variables__3DDWFPREC_set},
{"LibreDWGc::Dwg_Header_Variables__3DDWFPREC_get", _wrap_Dwg_Header_Variables__3DDWFPREC_get},
{"LibreDWGc::Dwg_Header_Variables_LENSLENGTH_set", _wrap_Dwg_Header_Variables_LENSLENGTH_set},
{"LibreDWGc::Dwg_Header_Variables_LENSLENGTH_get", _wrap_Dwg_Header_Variables_LENSLENGTH_get},
{"LibreDWGc::Dwg_Header_Variables_CAMERAHEIGHT_set", _wrap_Dwg_Header_Variables_CAMERAHEIGHT_set},
{"LibreDWGc::Dwg_Header_Variables_CAMERAHEIGHT_get", _wrap_Dwg_Header_Variables_CAMERAHEIGHT_get},
{"LibreDWGc::Dwg_Header_Variables_SOLIDHIST_set", _wrap_Dwg_Header_Variables_SOLIDHIST_set},
{"LibreDWGc::Dwg_Header_Variables_SOLIDHIST_get", _wrap_Dwg_Header_Variables_SOLIDHIST_get},
{"LibreDWGc::Dwg_Header_Variables_SHOWHIST_set", _wrap_Dwg_Header_Variables_SHOWHIST_set},
{"LibreDWGc::Dwg_Header_Variables_SHOWHIST_get", _wrap_Dwg_Header_Variables_SHOWHIST_get},
{"LibreDWGc::Dwg_Header_Variables_PSOLWIDTH_set", _wrap_Dwg_Header_Variables_PSOLWIDTH_set},
{"LibreDWGc::Dwg_Header_Variables_PSOLWIDTH_get", _wrap_Dwg_Header_Variables_PSOLWIDTH_get},
{"LibreDWGc::Dwg_Header_Variables_PSOLHEIGHT_set", _wrap_Dwg_Header_Variables_PSOLHEIGHT_set},
{"LibreDWGc::Dwg_Header_Variables_PSOLHEIGHT_get", _wrap_Dwg_Header_Variables_PSOLHEIGHT_get},
{"LibreDWGc::Dwg_Header_Variables_LOFTANG1_set", _wrap_Dwg_Header_Variables_LOFTANG1_set},
{"LibreDWGc::Dwg_Header_Variables_LOFTANG1_get", _wrap_Dwg_Header_Variables_LOFTANG1_get},
{"LibreDWGc::Dwg_Header_Variables_LOFTANG2_set", _wrap_Dwg_Header_Variables_LOFTANG2_set},
{"LibreDWGc::Dwg_Header_Variables_LOFTANG2_get", _wrap_Dwg_Header_Variables_LOFTANG2_get},
{"LibreDWGc::Dwg_Header_Variables_LOFTMAG1_set", _wrap_Dwg_Header_Variables_LOFTMAG1_set},
{"LibreDWGc::Dwg_Header_Variables_LOFTMAG1_get", _wrap_Dwg_Header_Variables_LOFTMAG1_get},
{"LibreDWGc::Dwg_Header_Variables_LOFTMAG2_set", _wrap_Dwg_Header_Variables_LOFTMAG2_set},
{"LibreDWGc::Dwg_Header_Variables_LOFTMAG2_get", _wrap_Dwg_Header_Variables_LOFTMAG2_get},
{"LibreDWGc::Dwg_Header_Variables_LOFTPARAM_set", _wrap_Dwg_Header_Variables_LOFTPARAM_set},
{"LibreDWGc::Dwg_Header_Variables_LOFTPARAM_get", _wrap_Dwg_Header_Variables_LOFTPARAM_get},
{"LibreDWGc::Dwg_Header_Variables_LOFTNORMALS_set", _wrap_Dwg_Header_Variables_LOFTNORMALS_set},
{"LibreDWGc::Dwg_Header_Variables_LOFTNORMALS_get", _wrap_Dwg_Header_Variables_LOFTNORMALS_get},
{"LibreDWGc::Dwg_Header_Variables_LATITUDE_set", _wrap_Dwg_Header_Variables_LATITUDE_set},
{"LibreDWGc::Dwg_Header_Variables_LATITUDE_get", _wrap_Dwg_Header_Variables_LATITUDE_get},
{"LibreDWGc::Dwg_Header_Variables_LONGITUDE_set", _wrap_Dwg_Header_Variables_LONGITUDE_set},
{"LibreDWGc::Dwg_Header_Variables_LONGITUDE_get", _wrap_Dwg_Header_Variables_LONGITUDE_get},
{"LibreDWGc::Dwg_Header_Variables_NORTHDIRECTION_set", _wrap_Dwg_Header_Variables_NORTHDIRECTION_set},
{"LibreDWGc::Dwg_Header_Variables_NORTHDIRECTION_get", _wrap_Dwg_Header_Variables_NORTHDIRECTION_get},
{"LibreDWGc::Dwg_Header_Variables_TIMEZONE_set", _wrap_Dwg_Header_Variables_TIMEZONE_set},
{"LibreDWGc::Dwg_Header_Variables_TIMEZONE_get", _wrap_Dwg_Header_Variables_TIMEZONE_get},
{"LibreDWGc::Dwg_Header_Variables_LIGHTGLYPHDISPLAY_set", _wrap_Dwg_Header_Variables_LIGHTGLYPHDISPLAY_set},
{"LibreDWGc::Dwg_Header_Variables_LIGHTGLYPHDISPLAY_get", _wrap_Dwg_Header_Variables_LIGHTGLYPHDISPLAY_get},
{"LibreDWGc::Dwg_Header_Variables_TILEMODELIGHTSYNCH_set", _wrap_Dwg_Header_Variables_TILEMODELIGHTSYNCH_set},
{"LibreDWGc::Dwg_Header_Variables_TILEMODELIGHTSYNCH_get", _wrap_Dwg_Header_Variables_TILEMODELIGHTSYNCH_get},
{"LibreDWGc::Dwg_Header_Variables_DWFFRAME_set", _wrap_Dwg_Header_Variables_DWFFRAME_set},
{"LibreDWGc::Dwg_Header_Variables_DWFFRAME_get", _wrap_Dwg_Header_Variables_DWFFRAME_get},
{"LibreDWGc::Dwg_Header_Variables_DGNFRAME_set", _wrap_Dwg_Header_Variables_DGNFRAME_set},
{"LibreDWGc::Dwg_Header_Variables_DGNFRAME_get", _wrap_Dwg_Header_Variables_DGNFRAME_get},
{"LibreDWGc::Dwg_Header_Variables_REALWORLDSCALE_set", _wrap_Dwg_Header_Variables_REALWORLDSCALE_set},
{"LibreDWGc::Dwg_Header_Variables_REALWORLDSCALE_get", _wrap_Dwg_Header_Variables_REALWORLDSCALE_get},
{"LibreDWGc::Dwg_Header_Variables_INTERFERECOLOR_set", _wrap_Dwg_Header_Variables_INTERFERECOLOR_set},
{"LibreDWGc::Dwg_Header_Variables_INTERFERECOLOR_get", _wrap_Dwg_Header_Variables_INTERFERECOLOR_get},
{"LibreDWGc::Dwg_Header_Variables_INTERFEREOBJVS_set", _wrap_Dwg_Header_Variables_INTERFEREOBJVS_set},
{"LibreDWGc::Dwg_Header_Variables_INTERFEREOBJVS_get", _wrap_Dwg_Header_Variables_INTERFEREOBJVS_get},
{"LibreDWGc::Dwg_Header_Variables_INTERFEREVPVS_set", _wrap_Dwg_Header_Variables_INTERFEREVPVS_set},
{"LibreDWGc::Dwg_Header_Variables_INTERFEREVPVS_get", _wrap_Dwg_Header_Variables_INTERFEREVPVS_get},
{"LibreDWGc::Dwg_Header_Variables_DRAGVS_set", _wrap_Dwg_Header_Variables_DRAGVS_set},
{"LibreDWGc::Dwg_Header_Variables_DRAGVS_get", _wrap_Dwg_Header_Variables_DRAGVS_get},
{"LibreDWGc::Dwg_Header_Variables_CSHADOW_set", _wrap_Dwg_Header_Variables_CSHADOW_set},
{"LibreDWGc::Dwg_Header_Variables_CSHADOW_get", _wrap_Dwg_Header_Variables_CSHADOW_get},
{"LibreDWGc::Dwg_Header_Variables_SHADOWPLANELOCATION_set", _wrap_Dwg_Header_Variables_SHADOWPLANELOCATION_set},
{"LibreDWGc::Dwg_Header_Variables_SHADOWPLANELOCATION_get", _wrap_Dwg_Header_Variables_SHADOWPLANELOCATION_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_54_set", _wrap_Dwg_Header_Variables_unknown_54_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_54_get", _wrap_Dwg_Header_Variables_unknown_54_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_55_set", _wrap_Dwg_Header_Variables_unknown_55_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_55_get", _wrap_Dwg_Header_Variables_unknown_55_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_56_set", _wrap_Dwg_Header_Variables_unknown_56_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_56_get", _wrap_Dwg_Header_Variables_unknown_56_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_57_set", _wrap_Dwg_Header_Variables_unknown_57_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_57_get", _wrap_Dwg_Header_Variables_unknown_57_get},
{"LibreDWGc::Dwg_Header_Variables_dwg_size_set", _wrap_Dwg_Header_Variables_dwg_size_set},
{"LibreDWGc::Dwg_Header_Variables_dwg_size_get", _wrap_Dwg_Header_Variables_dwg_size_get},
{"LibreDWGc::Dwg_Header_Variables_numentities_set", _wrap_Dwg_Header_Variables_numentities_set},
{"LibreDWGc::Dwg_Header_Variables_numentities_get", _wrap_Dwg_Header_Variables_numentities_get},
{"LibreDWGc::Dwg_Header_Variables_circle_zoom_percent_set", _wrap_Dwg_Header_Variables_circle_zoom_percent_set},
{"LibreDWGc::Dwg_Header_Variables_circle_zoom_percent_get", _wrap_Dwg_Header_Variables_circle_zoom_percent_get},
{"LibreDWGc::Dwg_Header_Variables_FRONTZ_set", _wrap_Dwg_Header_Variables_FRONTZ_set},
{"LibreDWGc::Dwg_Header_Variables_FRONTZ_get", _wrap_Dwg_Header_Variables_FRONTZ_get},
{"LibreDWGc::Dwg_Header_Variables_BACKZ_set", _wrap_Dwg_Header_Variables_BACKZ_set},
{"LibreDWGc::Dwg_Header_Variables_BACKZ_get", _wrap_Dwg_Header_Variables_BACKZ_get},
{"LibreDWGc::Dwg_Header_Variables_UCSICON_set", _wrap_Dwg_Header_Variables_UCSICON_set},
{"LibreDWGc::Dwg_Header_Variables_UCSICON_get", _wrap_Dwg_Header_Variables_UCSICON_get},
{"LibreDWGc::Dwg_Header_Variables_oldCECOLOR_hi_set", _wrap_Dwg_Header_Variables_oldCECOLOR_hi_set},
{"LibreDWGc::Dwg_Header_Variables_oldCECOLOR_hi_get", _wrap_Dwg_Header_Variables_oldCECOLOR_hi_get},
{"LibreDWGc::Dwg_Header_Variables_oldCECOLOR_lo_set", _wrap_Dwg_Header_Variables_oldCECOLOR_lo_set},
{"LibreDWGc::Dwg_Header_Variables_oldCECOLOR_lo_get", _wrap_Dwg_Header_Variables_oldCECOLOR_lo_get},
{"LibreDWGc::Dwg_Header_Variables_layer_colors_set", _wrap_Dwg_Header_Variables_layer_colors_set},
{"LibreDWGc::Dwg_Header_Variables_layer_colors_get", _wrap_Dwg_Header_Variables_layer_colors_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_51e_set", _wrap_Dwg_Header_Variables_unknown_51e_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_51e_get", _wrap_Dwg_Header_Variables_unknown_51e_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_520_set", _wrap_Dwg_Header_Variables_unknown_520_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_520_get", _wrap_Dwg_Header_Variables_unknown_520_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_52c_set", _wrap_Dwg_Header_Variables_unknown_52c_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_52c_get", _wrap_Dwg_Header_Variables_unknown_52c_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_52e_set", _wrap_Dwg_Header_Variables_unknown_52e_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_52e_get", _wrap_Dwg_Header_Variables_unknown_52e_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_530_set", _wrap_Dwg_Header_Variables_unknown_530_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_530_get", _wrap_Dwg_Header_Variables_unknown_530_get},
{"LibreDWGc::Dwg_Header_Variables_unknown_59_set", _wrap_Dwg_Header_Variables_unknown_59_set},
{"LibreDWGc::Dwg_Header_Variables_unknown_59_get", _wrap_Dwg_Header_Variables_unknown_59_get},
{"LibreDWGc::Dwg_Header_Variables_aspect_ratio_set", _wrap_Dwg_Header_Variables_aspect_ratio_set},
{"LibreDWGc::Dwg_Header_Variables_aspect_ratio_get", _wrap_Dwg_Header_Variables_aspect_ratio_get},
{"LibreDWGc::new_Dwg_Header_Variables", _wrap_new_Dwg_Header_Variables},
{"LibreDWGc::delete_Dwg_Header_Variables", _wrap_delete_Dwg_Header_Variables},
{"LibreDWGc::Dwg_Entity_TEXT_parent_set", _wrap_Dwg_Entity_TEXT_parent_set},
{"LibreDWGc::Dwg_Entity_TEXT_parent_get", _wrap_Dwg_Entity_TEXT_parent_get},
{"LibreDWGc::Dwg_Entity_TEXT_dataflags_set", _wrap_Dwg_Entity_TEXT_dataflags_set},
{"LibreDWGc::Dwg_Entity_TEXT_dataflags_get", _wrap_Dwg_Entity_TEXT_dataflags_get},
{"LibreDWGc::Dwg_Entity_TEXT_elevation_set", _wrap_Dwg_Entity_TEXT_elevation_set},
{"LibreDWGc::Dwg_Entity_TEXT_elevation_get", _wrap_Dwg_Entity_TEXT_elevation_get},
{"LibreDWGc::Dwg_Entity_TEXT_ins_pt_set", _wrap_Dwg_Entity_TEXT_ins_pt_set},
{"LibreDWGc::Dwg_Entity_TEXT_ins_pt_get", _wrap_Dwg_Entity_TEXT_ins_pt_get},
{"LibreDWGc::Dwg_Entity_TEXT_alignment_pt_set", _wrap_Dwg_Entity_TEXT_alignment_pt_set},
{"LibreDWGc::Dwg_Entity_TEXT_alignment_pt_get", _wrap_Dwg_Entity_TEXT_alignment_pt_get},
{"LibreDWGc::Dwg_Entity_TEXT_extrusion_set", _wrap_Dwg_Entity_TEXT_extrusion_set},
{"LibreDWGc::Dwg_Entity_TEXT_extrusion_get", _wrap_Dwg_Entity_TEXT_extrusion_get},
{"LibreDWGc::Dwg_Entity_TEXT_thickness_set", _wrap_Dwg_Entity_TEXT_thickness_set},
{"LibreDWGc::Dwg_Entity_TEXT_thickness_get", _wrap_Dwg_Entity_TEXT_thickness_get},
{"LibreDWGc::Dwg_Entity_TEXT_oblique_angle_set", _wrap_Dwg_Entity_TEXT_oblique_angle_set},
{"LibreDWGc::Dwg_Entity_TEXT_oblique_angle_get", _wrap_Dwg_Entity_TEXT_oblique_angle_get},
{"LibreDWGc::Dwg_Entity_TEXT_rotation_set", _wrap_Dwg_Entity_TEXT_rotation_set},
{"LibreDWGc::Dwg_Entity_TEXT_rotation_get", _wrap_Dwg_Entity_TEXT_rotation_get},
{"LibreDWGc::Dwg_Entity_TEXT_height_set", _wrap_Dwg_Entity_TEXT_height_set},
{"LibreDWGc::Dwg_Entity_TEXT_height_get", _wrap_Dwg_Entity_TEXT_height_get},
{"LibreDWGc::Dwg_Entity_TEXT_width_factor_set", _wrap_Dwg_Entity_TEXT_width_factor_set},
{"LibreDWGc::Dwg_Entity_TEXT_width_factor_get", _wrap_Dwg_Entity_TEXT_width_factor_get},
{"LibreDWGc::Dwg_Entity_TEXT_text_value_set", _wrap_Dwg_Entity_TEXT_text_value_set},
{"LibreDWGc::Dwg_Entity_TEXT_text_value_get", _wrap_Dwg_Entity_TEXT_text_value_get},
{"LibreDWGc::Dwg_Entity_TEXT_generation_set", _wrap_Dwg_Entity_TEXT_generation_set},
{"LibreDWGc::Dwg_Entity_TEXT_generation_get", _wrap_Dwg_Entity_TEXT_generation_get},
{"LibreDWGc::Dwg_Entity_TEXT_horiz_alignment_set", _wrap_Dwg_Entity_TEXT_horiz_alignment_set},
{"LibreDWGc::Dwg_Entity_TEXT_horiz_alignment_get", _wrap_Dwg_Entity_TEXT_horiz_alignment_get},
{"LibreDWGc::Dwg_Entity_TEXT_vert_alignment_set", _wrap_Dwg_Entity_TEXT_vert_alignment_set},
{"LibreDWGc::Dwg_Entity_TEXT_vert_alignment_get", _wrap_Dwg_Entity_TEXT_vert_alignment_get},
{"LibreDWGc::Dwg_Entity_TEXT_style_set", _wrap_Dwg_Entity_TEXT_style_set},
{"LibreDWGc::Dwg_Entity_TEXT_style_get", _wrap_Dwg_Entity_TEXT_style_get},
{"LibreDWGc::new_Dwg_Entity_TEXT", _wrap_new_Dwg_Entity_TEXT},
{"LibreDWGc::delete_Dwg_Entity_TEXT", _wrap_delete_Dwg_Entity_TEXT},
{"LibreDWGc::Dwg_AcDbMTextObjectEmbedded_attachment_set", _wrap_Dwg_AcDbMTextObjectEmbedded_attachment_set},
{"LibreDWGc::Dwg_AcDbMTextObjectEmbedded_attachment_get", _wrap_Dwg_AcDbMTextObjectEmbedded_attachment_get},
{"LibreDWGc::Dwg_AcDbMTextObjectEmbedded_ins_pt_set", _wrap_Dwg_AcDbMTextObjectEmbedded_ins_pt_set},
{"LibreDWGc::Dwg_AcDbMTextObjectEmbedded_ins_pt_get", _wrap_Dwg_AcDbMTextObjectEmbedded_ins_pt_get},
{"LibreDWGc::Dwg_AcDbMTextObjectEmbedded_x_axis_dir_set", _wrap_Dwg_AcDbMTextObjectEmbedded_x_axis_dir_set},
{"LibreDWGc::Dwg_AcDbMTextObjectEmbedded_x_axis_dir_get", _wrap_Dwg_AcDbMTextObjectEmbedded_x_axis_dir_get},
{"LibreDWGc::Dwg_AcDbMTextObjectEmbedded_rect_height_set", _wrap_Dwg_AcDbMTextObjectEmbedded_rect_height_set},
{"LibreDWGc::Dwg_AcDbMTextObjectEmbedded_rect_height_get", _wrap_Dwg_AcDbMTextObjectEmbedded_rect_height_get},
{"LibreDWGc::Dwg_AcDbMTextObjectEmbedded_rect_width_set", _wrap_Dwg_AcDbMTextObjectEmbedded_rect_width_set},
{"LibreDWGc::Dwg_AcDbMTextObjectEmbedded_rect_width_get", _wrap_Dwg_AcDbMTextObjectEmbedded_rect_width_get},
{"LibreDWGc::Dwg_AcDbMTextObjectEmbedded_extents_width_set", _wrap_Dwg_AcDbMTextObjectEmbedded_extents_width_set},
{"LibreDWGc::Dwg_AcDbMTextObjectEmbedded_extents_width_get", _wrap_Dwg_AcDbMTextObjectEmbedded_extents_width_get},
{"LibreDWGc::Dwg_AcDbMTextObjectEmbedded_extents_height_set", _wrap_Dwg_AcDbMTextObjectEmbedded_extents_height_set},
{"LibreDWGc::Dwg_AcDbMTextObjectEmbedded_extents_height_get", _wrap_Dwg_AcDbMTextObjectEmbedded_extents_height_get},
{"LibreDWGc::Dwg_AcDbMTextObjectEmbedded_column_type_set", _wrap_Dwg_AcDbMTextObjectEmbedded_column_type_set},
{"LibreDWGc::Dwg_AcDbMTextObjectEmbedded_column_type_get", _wrap_Dwg_AcDbMTextObjectEmbedded_column_type_get},
{"LibreDWGc::Dwg_AcDbMTextObjectEmbedded_column_width_set", _wrap_Dwg_AcDbMTextObjectEmbedded_column_width_set},
{"LibreDWGc::Dwg_AcDbMTextObjectEmbedded_column_width_get", _wrap_Dwg_AcDbMTextObjectEmbedded_column_width_get},
{"LibreDWGc::Dwg_AcDbMTextObjectEmbedded_gutter_set", _wrap_Dwg_AcDbMTextObjectEmbedded_gutter_set},
{"LibreDWGc::Dwg_AcDbMTextObjectEmbedded_gutter_get", _wrap_Dwg_AcDbMTextObjectEmbedded_gutter_get},
{"LibreDWGc::Dwg_AcDbMTextObjectEmbedded_auto_height_set", _wrap_Dwg_AcDbMTextObjectEmbedded_auto_height_set},
{"LibreDWGc::Dwg_AcDbMTextObjectEmbedded_auto_height_get", _wrap_Dwg_AcDbMTextObjectEmbedded_auto_height_get},
{"LibreDWGc::Dwg_AcDbMTextObjectEmbedded_flow_reversed_set", _wrap_Dwg_AcDbMTextObjectEmbedded_flow_reversed_set},
{"LibreDWGc::Dwg_AcDbMTextObjectEmbedded_flow_reversed_get", _wrap_Dwg_AcDbMTextObjectEmbedded_flow_reversed_get},
{"LibreDWGc::Dwg_AcDbMTextObjectEmbedded_num_column_heights_set", _wrap_Dwg_AcDbMTextObjectEmbedded_num_column_heights_set},
{"LibreDWGc::Dwg_AcDbMTextObjectEmbedded_num_column_heights_get", _wrap_Dwg_AcDbMTextObjectEmbedded_num_column_heights_get},
{"LibreDWGc::Dwg_AcDbMTextObjectEmbedded_column_heights_set", _wrap_Dwg_AcDbMTextObjectEmbedded_column_heights_set},
{"LibreDWGc::Dwg_AcDbMTextObjectEmbedded_column_heights_get", _wrap_Dwg_AcDbMTextObjectEmbedded_column_heights_get},
{"LibreDWGc::new_Dwg_AcDbMTextObjectEmbedded", _wrap_new_Dwg_AcDbMTextObjectEmbedded},
{"LibreDWGc::delete_Dwg_AcDbMTextObjectEmbedded", _wrap_delete_Dwg_AcDbMTextObjectEmbedded},
{"LibreDWGc::Dwg_Entity_ATTRIB_parent_set", _wrap_Dwg_Entity_ATTRIB_parent_set},
{"LibreDWGc::Dwg_Entity_ATTRIB_parent_get", _wrap_Dwg_Entity_ATTRIB_parent_get},
{"LibreDWGc::Dwg_Entity_ATTRIB_elevation_set", _wrap_Dwg_Entity_ATTRIB_elevation_set},
{"LibreDWGc::Dwg_Entity_ATTRIB_elevation_get", _wrap_Dwg_Entity_ATTRIB_elevation_get},
{"LibreDWGc::Dwg_Entity_ATTRIB_ins_pt_set", _wrap_Dwg_Entity_ATTRIB_ins_pt_set},
{"LibreDWGc::Dwg_Entity_ATTRIB_ins_pt_get", _wrap_Dwg_Entity_ATTRIB_ins_pt_get},
{"LibreDWGc::Dwg_Entity_ATTRIB_alignment_pt_set", _wrap_Dwg_Entity_ATTRIB_alignment_pt_set},
{"LibreDWGc::Dwg_Entity_ATTRIB_alignment_pt_get", _wrap_Dwg_Entity_ATTRIB_alignment_pt_get},
{"LibreDWGc::Dwg_Entity_ATTRIB_extrusion_set", _wrap_Dwg_Entity_ATTRIB_extrusion_set},
{"LibreDWGc::Dwg_Entity_ATTRIB_extrusion_get", _wrap_Dwg_Entity_ATTRIB_extrusion_get},
{"LibreDWGc::Dwg_Entity_ATTRIB_thickness_set", _wrap_Dwg_Entity_ATTRIB_thickness_set},
{"LibreDWGc::Dwg_Entity_ATTRIB_thickness_get", _wrap_Dwg_Entity_ATTRIB_thickness_get},
{"LibreDWGc::Dwg_Entity_ATTRIB_oblique_angle_set", _wrap_Dwg_Entity_ATTRIB_oblique_angle_set},
{"LibreDWGc::Dwg_Entity_ATTRIB_oblique_angle_get", _wrap_Dwg_Entity_ATTRIB_oblique_angle_get},
{"LibreDWGc::Dwg_Entity_ATTRIB_rotation_set", _wrap_Dwg_Entity_ATTRIB_rotation_set},
{"LibreDWGc::Dwg_Entity_ATTRIB_rotation_get", _wrap_Dwg_Entity_ATTRIB_rotation_get},
{"LibreDWGc::Dwg_Entity_ATTRIB_height_set", _wrap_Dwg_Entity_ATTRIB_height_set},
{"LibreDWGc::Dwg_Entity_ATTRIB_height_get", _wrap_Dwg_Entity_ATTRIB_height_get},
{"LibreDWGc::Dwg_Entity_ATTRIB_width_factor_set", _wrap_Dwg_Entity_ATTRIB_width_factor_set},
{"LibreDWGc::Dwg_Entity_ATTRIB_width_factor_get", _wrap_Dwg_Entity_ATTRIB_width_factor_get},
{"LibreDWGc::Dwg_Entity_ATTRIB_text_value_set", _wrap_Dwg_Entity_ATTRIB_text_value_set},
{"LibreDWGc::Dwg_Entity_ATTRIB_text_value_get", _wrap_Dwg_Entity_ATTRIB_text_value_get},
{"LibreDWGc::Dwg_Entity_ATTRIB_generation_set", _wrap_Dwg_Entity_ATTRIB_generation_set},
{"LibreDWGc::Dwg_Entity_ATTRIB_generation_get", _wrap_Dwg_Entity_ATTRIB_generation_get},
{"LibreDWGc::Dwg_Entity_ATTRIB_horiz_alignment_set", _wrap_Dwg_Entity_ATTRIB_horiz_alignment_set},
{"LibreDWGc::Dwg_Entity_ATTRIB_horiz_alignment_get", _wrap_Dwg_Entity_ATTRIB_horiz_alignment_get},
{"LibreDWGc::Dwg_Entity_ATTRIB_vert_alignment_set", _wrap_Dwg_Entity_ATTRIB_vert_alignment_set},
{"LibreDWGc::Dwg_Entity_ATTRIB_vert_alignment_get", _wrap_Dwg_Entity_ATTRIB_vert_alignment_get},
{"LibreDWGc::Dwg_Entity_ATTRIB_dataflags_set", _wrap_Dwg_Entity_ATTRIB_dataflags_set},
{"LibreDWGc::Dwg_Entity_ATTRIB_dataflags_get", _wrap_Dwg_Entity_ATTRIB_dataflags_get},
{"LibreDWGc::Dwg_Entity_ATTRIB_is_locked_in_block_set", _wrap_Dwg_Entity_ATTRIB_is_locked_in_block_set},
{"LibreDWGc::Dwg_Entity_ATTRIB_is_locked_in_block_get", _wrap_Dwg_Entity_ATTRIB_is_locked_in_block_get},
{"LibreDWGc::Dwg_Entity_ATTRIB_keep_duplicate_records_set", _wrap_Dwg_Entity_ATTRIB_keep_duplicate_records_set},
{"LibreDWGc::Dwg_Entity_ATTRIB_keep_duplicate_records_get", _wrap_Dwg_Entity_ATTRIB_keep_duplicate_records_get},
{"LibreDWGc::Dwg_Entity_ATTRIB_tag_set", _wrap_Dwg_Entity_ATTRIB_tag_set},
{"LibreDWGc::Dwg_Entity_ATTRIB_tag_get", _wrap_Dwg_Entity_ATTRIB_tag_get},
{"LibreDWGc::Dwg_Entity_ATTRIB_field_length_set", _wrap_Dwg_Entity_ATTRIB_field_length_set},
{"LibreDWGc::Dwg_Entity_ATTRIB_field_length_get", _wrap_Dwg_Entity_ATTRIB_field_length_get},
{"LibreDWGc::Dwg_Entity_ATTRIB_flags_set", _wrap_Dwg_Entity_ATTRIB_flags_set},
{"LibreDWGc::Dwg_Entity_ATTRIB_flags_get", _wrap_Dwg_Entity_ATTRIB_flags_get},
{"LibreDWGc::Dwg_Entity_ATTRIB_lock_position_flag_set", _wrap_Dwg_Entity_ATTRIB_lock_position_flag_set},
{"LibreDWGc::Dwg_Entity_ATTRIB_lock_position_flag_get", _wrap_Dwg_Entity_ATTRIB_lock_position_flag_get},
{"LibreDWGc::Dwg_Entity_ATTRIB_style_set", _wrap_Dwg_Entity_ATTRIB_style_set},
{"LibreDWGc::Dwg_Entity_ATTRIB_style_get", _wrap_Dwg_Entity_ATTRIB_style_get},
{"LibreDWGc::Dwg_Entity_ATTRIB_mtext_type_set", _wrap_Dwg_Entity_ATTRIB_mtext_type_set},
{"LibreDWGc::Dwg_Entity_ATTRIB_mtext_type_get", _wrap_Dwg_Entity_ATTRIB_mtext_type_get},
{"LibreDWGc::Dwg_Entity_ATTRIB_mtext_set", _wrap_Dwg_Entity_ATTRIB_mtext_set},
{"LibreDWGc::Dwg_Entity_ATTRIB_mtext_get", _wrap_Dwg_Entity_ATTRIB_mtext_get},
{"LibreDWGc::Dwg_Entity_ATTRIB_is_really_locked_set", _wrap_Dwg_Entity_ATTRIB_is_really_locked_set},
{"LibreDWGc::Dwg_Entity_ATTRIB_is_really_locked_get", _wrap_Dwg_Entity_ATTRIB_is_really_locked_get},
{"LibreDWGc::Dwg_Entity_ATTRIB_annotative_data_size_set", _wrap_Dwg_Entity_ATTRIB_annotative_data_size_set},
{"LibreDWGc::Dwg_Entity_ATTRIB_annotative_data_size_get", _wrap_Dwg_Entity_ATTRIB_annotative_data_size_get},
{"LibreDWGc::Dwg_Entity_ATTRIB_annotative_data_set", _wrap_Dwg_Entity_ATTRIB_annotative_data_set},
{"LibreDWGc::Dwg_Entity_ATTRIB_annotative_data_get", _wrap_Dwg_Entity_ATTRIB_annotative_data_get},
{"LibreDWGc::Dwg_Entity_ATTRIB_annotative_flag_set", _wrap_Dwg_Entity_ATTRIB_annotative_flag_set},
{"LibreDWGc::Dwg_Entity_ATTRIB_annotative_flag_get", _wrap_Dwg_Entity_ATTRIB_annotative_flag_get},
{"LibreDWGc::Dwg_Entity_ATTRIB_annotative_style_set", _wrap_Dwg_Entity_ATTRIB_annotative_style_set},
{"LibreDWGc::Dwg_Entity_ATTRIB_annotative_style_get", _wrap_Dwg_Entity_ATTRIB_annotative_style_get},
{"LibreDWGc::new_Dwg_Entity_ATTRIB", _wrap_new_Dwg_Entity_ATTRIB},
{"LibreDWGc::delete_Dwg_Entity_ATTRIB", _wrap_delete_Dwg_Entity_ATTRIB},
{"LibreDWGc::Dwg_Entity_ATTDEF_parent_set", _wrap_Dwg_Entity_ATTDEF_parent_set},
{"LibreDWGc::Dwg_Entity_ATTDEF_parent_get", _wrap_Dwg_Entity_ATTDEF_parent_get},
{"LibreDWGc::Dwg_Entity_ATTDEF_elevation_set", _wrap_Dwg_Entity_ATTDEF_elevation_set},
{"LibreDWGc::Dwg_Entity_ATTDEF_elevation_get", _wrap_Dwg_Entity_ATTDEF_elevation_get},
{"LibreDWGc::Dwg_Entity_ATTDEF_ins_pt_set", _wrap_Dwg_Entity_ATTDEF_ins_pt_set},
{"LibreDWGc::Dwg_Entity_ATTDEF_ins_pt_get", _wrap_Dwg_Entity_ATTDEF_ins_pt_get},
{"LibreDWGc::Dwg_Entity_ATTDEF_alignment_pt_set", _wrap_Dwg_Entity_ATTDEF_alignment_pt_set},
{"LibreDWGc::Dwg_Entity_ATTDEF_alignment_pt_get", _wrap_Dwg_Entity_ATTDEF_alignment_pt_get},
{"LibreDWGc::Dwg_Entity_ATTDEF_extrusion_set", _wrap_Dwg_Entity_ATTDEF_extrusion_set},
{"LibreDWGc::Dwg_Entity_ATTDEF_extrusion_get", _wrap_Dwg_Entity_ATTDEF_extrusion_get},
{"LibreDWGc::Dwg_Entity_ATTDEF_thickness_set", _wrap_Dwg_Entity_ATTDEF_thickness_set},
{"LibreDWGc::Dwg_Entity_ATTDEF_thickness_get", _wrap_Dwg_Entity_ATTDEF_thickness_get},
{"LibreDWGc::Dwg_Entity_ATTDEF_oblique_angle_set", _wrap_Dwg_Entity_ATTDEF_oblique_angle_set},
{"LibreDWGc::Dwg_Entity_ATTDEF_oblique_angle_get", _wrap_Dwg_Entity_ATTDEF_oblique_angle_get},
{"LibreDWGc::Dwg_Entity_ATTDEF_rotation_set", _wrap_Dwg_Entity_ATTDEF_rotation_set},
{"LibreDWGc::Dwg_Entity_ATTDEF_rotation_get", _wrap_Dwg_Entity_ATTDEF_rotation_get},
{"LibreDWGc::Dwg_Entity_ATTDEF_height_set", _wrap_Dwg_Entity_ATTDEF_height_set},
{"LibreDWGc::Dwg_Entity_ATTDEF_height_get", _wrap_Dwg_Entity_ATTDEF_height_get},
{"LibreDWGc::Dwg_Entity_ATTDEF_width_factor_set", _wrap_Dwg_Entity_ATTDEF_width_factor_set},
{"LibreDWGc::Dwg_Entity_ATTDEF_width_factor_get", _wrap_Dwg_Entity_ATTDEF_width_factor_get},
{"LibreDWGc::Dwg_Entity_ATTDEF_default_value_set", _wrap_Dwg_Entity_ATTDEF_default_value_set},
{"LibreDWGc::Dwg_Entity_ATTDEF_default_value_get", _wrap_Dwg_Entity_ATTDEF_default_value_get},
{"LibreDWGc::Dwg_Entity_ATTDEF_generation_set", _wrap_Dwg_Entity_ATTDEF_generation_set},
{"LibreDWGc::Dwg_Entity_ATTDEF_generation_get", _wrap_Dwg_Entity_ATTDEF_generation_get},
{"LibreDWGc::Dwg_Entity_ATTDEF_horiz_alignment_set", _wrap_Dwg_Entity_ATTDEF_horiz_alignment_set},
{"LibreDWGc::Dwg_Entity_ATTDEF_horiz_alignment_get", _wrap_Dwg_Entity_ATTDEF_horiz_alignment_get},
{"LibreDWGc::Dwg_Entity_ATTDEF_vert_alignment_set", _wrap_Dwg_Entity_ATTDEF_vert_alignment_set},
{"LibreDWGc::Dwg_Entity_ATTDEF_vert_alignment_get", _wrap_Dwg_Entity_ATTDEF_vert_alignment_get},
{"LibreDWGc::Dwg_Entity_ATTDEF_dataflags_set", _wrap_Dwg_Entity_ATTDEF_dataflags_set},
{"LibreDWGc::Dwg_Entity_ATTDEF_dataflags_get", _wrap_Dwg_Entity_ATTDEF_dataflags_get},
{"LibreDWGc::Dwg_Entity_ATTDEF_is_locked_in_block_set", _wrap_Dwg_Entity_ATTDEF_is_locked_in_block_set},
{"LibreDWGc::Dwg_Entity_ATTDEF_is_locked_in_block_get", _wrap_Dwg_Entity_ATTDEF_is_locked_in_block_get},
{"LibreDWGc::Dwg_Entity_ATTDEF_keep_duplicate_records_set", _wrap_Dwg_Entity_ATTDEF_keep_duplicate_records_set},
{"LibreDWGc::Dwg_Entity_ATTDEF_keep_duplicate_records_get", _wrap_Dwg_Entity_ATTDEF_keep_duplicate_records_get},
{"LibreDWGc::Dwg_Entity_ATTDEF_tag_set", _wrap_Dwg_Entity_ATTDEF_tag_set},
{"LibreDWGc::Dwg_Entity_ATTDEF_tag_get", _wrap_Dwg_Entity_ATTDEF_tag_get},
{"LibreDWGc::Dwg_Entity_ATTDEF_field_length_set", _wrap_Dwg_Entity_ATTDEF_field_length_set},
{"LibreDWGc::Dwg_Entity_ATTDEF_field_length_get", _wrap_Dwg_Entity_ATTDEF_field_length_get},
{"LibreDWGc::Dwg_Entity_ATTDEF_flags_set", _wrap_Dwg_Entity_ATTDEF_flags_set},
{"LibreDWGc::Dwg_Entity_ATTDEF_flags_get", _wrap_Dwg_Entity_ATTDEF_flags_get},
{"LibreDWGc::Dwg_Entity_ATTDEF_lock_position_flag_set", _wrap_Dwg_Entity_ATTDEF_lock_position_flag_set},
{"LibreDWGc::Dwg_Entity_ATTDEF_lock_position_flag_get", _wrap_Dwg_Entity_ATTDEF_lock_position_flag_get},
{"LibreDWGc::Dwg_Entity_ATTDEF_style_set", _wrap_Dwg_Entity_ATTDEF_style_set},
{"LibreDWGc::Dwg_Entity_ATTDEF_style_get", _wrap_Dwg_Entity_ATTDEF_style_get},
{"LibreDWGc::Dwg_Entity_ATTDEF_mtext_type_set", _wrap_Dwg_Entity_ATTDEF_mtext_type_set},
{"LibreDWGc::Dwg_Entity_ATTDEF_mtext_type_get", _wrap_Dwg_Entity_ATTDEF_mtext_type_get},
{"LibreDWGc::Dwg_Entity_ATTDEF_mtext_set", _wrap_Dwg_Entity_ATTDEF_mtext_set},
{"LibreDWGc::Dwg_Entity_ATTDEF_mtext_get", _wrap_Dwg_Entity_ATTDEF_mtext_get},
{"LibreDWGc::Dwg_Entity_ATTDEF_is_really_locked_set", _wrap_Dwg_Entity_ATTDEF_is_really_locked_set},
{"LibreDWGc::Dwg_Entity_ATTDEF_is_really_locked_get", _wrap_Dwg_Entity_ATTDEF_is_really_locked_get},
{"LibreDWGc::Dwg_Entity_ATTDEF_annotative_data_size_set", _wrap_Dwg_Entity_ATTDEF_annotative_data_size_set},
{"LibreDWGc::Dwg_Entity_ATTDEF_annotative_data_size_get", _wrap_Dwg_Entity_ATTDEF_annotative_data_size_get},
{"LibreDWGc::Dwg_Entity_ATTDEF_annotative_data_set", _wrap_Dwg_Entity_ATTDEF_annotative_data_set},
{"LibreDWGc::Dwg_Entity_ATTDEF_annotative_data_get", _wrap_Dwg_Entity_ATTDEF_annotative_data_get},
{"LibreDWGc::Dwg_Entity_ATTDEF_annotative_flag_set", _wrap_Dwg_Entity_ATTDEF_annotative_flag_set},
{"LibreDWGc::Dwg_Entity_ATTDEF_annotative_flag_get", _wrap_Dwg_Entity_ATTDEF_annotative_flag_get},
{"LibreDWGc::Dwg_Entity_ATTDEF_annotative_style_set", _wrap_Dwg_Entity_ATTDEF_annotative_style_set},
{"LibreDWGc::Dwg_Entity_ATTDEF_annotative_style_get", _wrap_Dwg_Entity_ATTDEF_annotative_style_get},
{"LibreDWGc::Dwg_Entity_ATTDEF_prompt_set", _wrap_Dwg_Entity_ATTDEF_prompt_set},
{"LibreDWGc::Dwg_Entity_ATTDEF_prompt_get", _wrap_Dwg_Entity_ATTDEF_prompt_get},
{"LibreDWGc::new_Dwg_Entity_ATTDEF", _wrap_new_Dwg_Entity_ATTDEF},
{"LibreDWGc::delete_Dwg_Entity_ATTDEF", _wrap_delete_Dwg_Entity_ATTDEF},
{"LibreDWGc::Dwg_Entity_BLOCK_parent_set", _wrap_Dwg_Entity_BLOCK_parent_set},
{"LibreDWGc::Dwg_Entity_BLOCK_parent_get", _wrap_Dwg_Entity_BLOCK_parent_get},
{"LibreDWGc::Dwg_Entity_BLOCK_name_set", _wrap_Dwg_Entity_BLOCK_name_set},
{"LibreDWGc::Dwg_Entity_BLOCK_name_get", _wrap_Dwg_Entity_BLOCK_name_get},
{"LibreDWGc::Dwg_Entity_BLOCK_xref_pname_set", _wrap_Dwg_Entity_BLOCK_xref_pname_set},
{"LibreDWGc::Dwg_Entity_BLOCK_xref_pname_get", _wrap_Dwg_Entity_BLOCK_xref_pname_get},
{"LibreDWGc::Dwg_Entity_BLOCK_base_pt_set", _wrap_Dwg_Entity_BLOCK_base_pt_set},
{"LibreDWGc::Dwg_Entity_BLOCK_base_pt_get", _wrap_Dwg_Entity_BLOCK_base_pt_get},
{"LibreDWGc::new_Dwg_Entity_BLOCK", _wrap_new_Dwg_Entity_BLOCK},
{"LibreDWGc::delete_Dwg_Entity_BLOCK", _wrap_delete_Dwg_Entity_BLOCK},
{"LibreDWGc::Dwg_Entity_ENDBLK_parent_set", _wrap_Dwg_Entity_ENDBLK_parent_set},
{"LibreDWGc::Dwg_Entity_ENDBLK_parent_get", _wrap_Dwg_Entity_ENDBLK_parent_get},
{"LibreDWGc::new_Dwg_Entity_ENDBLK", _wrap_new_Dwg_Entity_ENDBLK},
{"LibreDWGc::delete_Dwg_Entity_ENDBLK", _wrap_delete_Dwg_Entity_ENDBLK},
{"LibreDWGc::Dwg_Entity_SEQEND_parent_set", _wrap_Dwg_Entity_SEQEND_parent_set},
{"LibreDWGc::Dwg_Entity_SEQEND_parent_get", _wrap_Dwg_Entity_SEQEND_parent_get},
{"LibreDWGc::Dwg_Entity_SEQEND_begin_addr_r11_set", _wrap_Dwg_Entity_SEQEND_begin_addr_r11_set},
{"LibreDWGc::Dwg_Entity_SEQEND_begin_addr_r11_get", _wrap_Dwg_Entity_SEQEND_begin_addr_r11_get},
{"LibreDWGc::new_Dwg_Entity_SEQEND", _wrap_new_Dwg_Entity_SEQEND},
{"LibreDWGc::delete_Dwg_Entity_SEQEND", _wrap_delete_Dwg_Entity_SEQEND},
{"LibreDWGc::Dwg_Entity_INSERT_parent_set", _wrap_Dwg_Entity_INSERT_parent_set},
{"LibreDWGc::Dwg_Entity_INSERT_parent_get", _wrap_Dwg_Entity_INSERT_parent_get},
{"LibreDWGc::Dwg_Entity_INSERT_ins_pt_set", _wrap_Dwg_Entity_INSERT_ins_pt_set},
{"LibreDWGc::Dwg_Entity_INSERT_ins_pt_get", _wrap_Dwg_Entity_INSERT_ins_pt_get},
{"LibreDWGc::Dwg_Entity_INSERT_scale_flag_set", _wrap_Dwg_Entity_INSERT_scale_flag_set},
{"LibreDWGc::Dwg_Entity_INSERT_scale_flag_get", _wrap_Dwg_Entity_INSERT_scale_flag_get},
{"LibreDWGc::Dwg_Entity_INSERT_scale_set", _wrap_Dwg_Entity_INSERT_scale_set},
{"LibreDWGc::Dwg_Entity_INSERT_scale_get", _wrap_Dwg_Entity_INSERT_scale_get},
{"LibreDWGc::Dwg_Entity_INSERT_rotation_set", _wrap_Dwg_Entity_INSERT_rotation_set},
{"LibreDWGc::Dwg_Entity_INSERT_rotation_get", _wrap_Dwg_Entity_INSERT_rotation_get},
{"LibreDWGc::Dwg_Entity_INSERT_extrusion_set", _wrap_Dwg_Entity_INSERT_extrusion_set},
{"LibreDWGc::Dwg_Entity_INSERT_extrusion_get", _wrap_Dwg_Entity_INSERT_extrusion_get},
{"LibreDWGc::Dwg_Entity_INSERT_has_attribs_set", _wrap_Dwg_Entity_INSERT_has_attribs_set},
{"LibreDWGc::Dwg_Entity_INSERT_has_attribs_get", _wrap_Dwg_Entity_INSERT_has_attribs_get},
{"LibreDWGc::Dwg_Entity_INSERT_num_owned_set", _wrap_Dwg_Entity_INSERT_num_owned_set},
{"LibreDWGc::Dwg_Entity_INSERT_num_owned_get", _wrap_Dwg_Entity_INSERT_num_owned_get},
{"LibreDWGc::Dwg_Entity_INSERT_block_header_set", _wrap_Dwg_Entity_INSERT_block_header_set},
{"LibreDWGc::Dwg_Entity_INSERT_block_header_get", _wrap_Dwg_Entity_INSERT_block_header_get},
{"LibreDWGc::Dwg_Entity_INSERT_first_attrib_set", _wrap_Dwg_Entity_INSERT_first_attrib_set},
{"LibreDWGc::Dwg_Entity_INSERT_first_attrib_get", _wrap_Dwg_Entity_INSERT_first_attrib_get},
{"LibreDWGc::Dwg_Entity_INSERT_last_attrib_set", _wrap_Dwg_Entity_INSERT_last_attrib_set},
{"LibreDWGc::Dwg_Entity_INSERT_last_attrib_get", _wrap_Dwg_Entity_INSERT_last_attrib_get},
{"LibreDWGc::Dwg_Entity_INSERT_attribs_set", _wrap_Dwg_Entity_INSERT_attribs_set},
{"LibreDWGc::Dwg_Entity_INSERT_attribs_get", _wrap_Dwg_Entity_INSERT_attribs_get},
{"LibreDWGc::Dwg_Entity_INSERT_seqend_set", _wrap_Dwg_Entity_INSERT_seqend_set},
{"LibreDWGc::Dwg_Entity_INSERT_seqend_get", _wrap_Dwg_Entity_INSERT_seqend_get},
{"LibreDWGc::Dwg_Entity_INSERT_num_cols_set", _wrap_Dwg_Entity_INSERT_num_cols_set},
{"LibreDWGc::Dwg_Entity_INSERT_num_cols_get", _wrap_Dwg_Entity_INSERT_num_cols_get},
{"LibreDWGc::Dwg_Entity_INSERT_num_rows_set", _wrap_Dwg_Entity_INSERT_num_rows_set},
{"LibreDWGc::Dwg_Entity_INSERT_num_rows_get", _wrap_Dwg_Entity_INSERT_num_rows_get},
{"LibreDWGc::Dwg_Entity_INSERT_col_spacing_set", _wrap_Dwg_Entity_INSERT_col_spacing_set},
{"LibreDWGc::Dwg_Entity_INSERT_col_spacing_get", _wrap_Dwg_Entity_INSERT_col_spacing_get},
{"LibreDWGc::Dwg_Entity_INSERT_row_spacing_set", _wrap_Dwg_Entity_INSERT_row_spacing_set},
{"LibreDWGc::Dwg_Entity_INSERT_row_spacing_get", _wrap_Dwg_Entity_INSERT_row_spacing_get},
{"LibreDWGc::Dwg_Entity_INSERT_block_name_set", _wrap_Dwg_Entity_INSERT_block_name_set},
{"LibreDWGc::Dwg_Entity_INSERT_block_name_get", _wrap_Dwg_Entity_INSERT_block_name_get},
{"LibreDWGc::new_Dwg_Entity_INSERT", _wrap_new_Dwg_Entity_INSERT},
{"LibreDWGc::delete_Dwg_Entity_INSERT", _wrap_delete_Dwg_Entity_INSERT},
{"LibreDWGc::Dwg_Entity_MINSERT_parent_set", _wrap_Dwg_Entity_MINSERT_parent_set},
{"LibreDWGc::Dwg_Entity_MINSERT_parent_get", _wrap_Dwg_Entity_MINSERT_parent_get},
{"LibreDWGc::Dwg_Entity_MINSERT_ins_pt_set", _wrap_Dwg_Entity_MINSERT_ins_pt_set},
{"LibreDWGc::Dwg_Entity_MINSERT_ins_pt_get", _wrap_Dwg_Entity_MINSERT_ins_pt_get},
{"LibreDWGc::Dwg_Entity_MINSERT_scale_flag_set", _wrap_Dwg_Entity_MINSERT_scale_flag_set},
{"LibreDWGc::Dwg_Entity_MINSERT_scale_flag_get", _wrap_Dwg_Entity_MINSERT_scale_flag_get},
{"LibreDWGc::Dwg_Entity_MINSERT_scale_set", _wrap_Dwg_Entity_MINSERT_scale_set},
{"LibreDWGc::Dwg_Entity_MINSERT_scale_get", _wrap_Dwg_Entity_MINSERT_scale_get},
{"LibreDWGc::Dwg_Entity_MINSERT_rotation_set", _wrap_Dwg_Entity_MINSERT_rotation_set},
{"LibreDWGc::Dwg_Entity_MINSERT_rotation_get", _wrap_Dwg_Entity_MINSERT_rotation_get},
{"LibreDWGc::Dwg_Entity_MINSERT_extrusion_set", _wrap_Dwg_Entity_MINSERT_extrusion_set},
{"LibreDWGc::Dwg_Entity_MINSERT_extrusion_get", _wrap_Dwg_Entity_MINSERT_extrusion_get},
{"LibreDWGc::Dwg_Entity_MINSERT_has_attribs_set", _wrap_Dwg_Entity_MINSERT_has_attribs_set},
{"LibreDWGc::Dwg_Entity_MINSERT_has_attribs_get", _wrap_Dwg_Entity_MINSERT_has_attribs_get},
{"LibreDWGc::Dwg_Entity_MINSERT_num_owned_set", _wrap_Dwg_Entity_MINSERT_num_owned_set},
{"LibreDWGc::Dwg_Entity_MINSERT_num_owned_get", _wrap_Dwg_Entity_MINSERT_num_owned_get},
{"LibreDWGc::Dwg_Entity_MINSERT_num_cols_set", _wrap_Dwg_Entity_MINSERT_num_cols_set},
{"LibreDWGc::Dwg_Entity_MINSERT_num_cols_get", _wrap_Dwg_Entity_MINSERT_num_cols_get},
{"LibreDWGc::Dwg_Entity_MINSERT_num_rows_set", _wrap_Dwg_Entity_MINSERT_num_rows_set},
{"LibreDWGc::Dwg_Entity_MINSERT_num_rows_get", _wrap_Dwg_Entity_MINSERT_num_rows_get},
{"LibreDWGc::Dwg_Entity_MINSERT_col_spacing_set", _wrap_Dwg_Entity_MINSERT_col_spacing_set},
{"LibreDWGc::Dwg_Entity_MINSERT_col_spacing_get", _wrap_Dwg_Entity_MINSERT_col_spacing_get},
{"LibreDWGc::Dwg_Entity_MINSERT_row_spacing_set", _wrap_Dwg_Entity_MINSERT_row_spacing_set},
{"LibreDWGc::Dwg_Entity_MINSERT_row_spacing_get", _wrap_Dwg_Entity_MINSERT_row_spacing_get},
{"LibreDWGc::Dwg_Entity_MINSERT_block_header_set", _wrap_Dwg_Entity_MINSERT_block_header_set},
{"LibreDWGc::Dwg_Entity_MINSERT_block_header_get", _wrap_Dwg_Entity_MINSERT_block_header_get},
{"LibreDWGc::Dwg_Entity_MINSERT_first_attrib_set", _wrap_Dwg_Entity_MINSERT_first_attrib_set},
{"LibreDWGc::Dwg_Entity_MINSERT_first_attrib_get", _wrap_Dwg_Entity_MINSERT_first_attrib_get},
{"LibreDWGc::Dwg_Entity_MINSERT_last_attrib_set", _wrap_Dwg_Entity_MINSERT_last_attrib_set},
{"LibreDWGc::Dwg_Entity_MINSERT_last_attrib_get", _wrap_Dwg_Entity_MINSERT_last_attrib_get},
{"LibreDWGc::Dwg_Entity_MINSERT_attribs_set", _wrap_Dwg_Entity_MINSERT_attribs_set},
{"LibreDWGc::Dwg_Entity_MINSERT_attribs_get", _wrap_Dwg_Entity_MINSERT_attribs_get},
{"LibreDWGc::Dwg_Entity_MINSERT_seqend_set", _wrap_Dwg_Entity_MINSERT_seqend_set},
{"LibreDWGc::Dwg_Entity_MINSERT_seqend_get", _wrap_Dwg_Entity_MINSERT_seqend_get},
{"LibreDWGc::new_Dwg_Entity_MINSERT", _wrap_new_Dwg_Entity_MINSERT},
{"LibreDWGc::delete_Dwg_Entity_MINSERT", _wrap_delete_Dwg_Entity_MINSERT},
{"LibreDWGc::Dwg_Entity_VERTEX_2D_parent_set", _wrap_Dwg_Entity_VERTEX_2D_parent_set},
{"LibreDWGc::Dwg_Entity_VERTEX_2D_parent_get", _wrap_Dwg_Entity_VERTEX_2D_parent_get},
{"LibreDWGc::Dwg_Entity_VERTEX_2D_flag_set", _wrap_Dwg_Entity_VERTEX_2D_flag_set},
{"LibreDWGc::Dwg_Entity_VERTEX_2D_flag_get", _wrap_Dwg_Entity_VERTEX_2D_flag_get},
{"LibreDWGc::Dwg_Entity_VERTEX_2D_point_set", _wrap_Dwg_Entity_VERTEX_2D_point_set},
{"LibreDWGc::Dwg_Entity_VERTEX_2D_point_get", _wrap_Dwg_Entity_VERTEX_2D_point_get},
{"LibreDWGc::Dwg_Entity_VERTEX_2D_start_width_set", _wrap_Dwg_Entity_VERTEX_2D_start_width_set},
{"LibreDWGc::Dwg_Entity_VERTEX_2D_start_width_get", _wrap_Dwg_Entity_VERTEX_2D_start_width_get},
{"LibreDWGc::Dwg_Entity_VERTEX_2D_end_width_set", _wrap_Dwg_Entity_VERTEX_2D_end_width_set},
{"LibreDWGc::Dwg_Entity_VERTEX_2D_end_width_get", _wrap_Dwg_Entity_VERTEX_2D_end_width_get},
{"LibreDWGc::Dwg_Entity_VERTEX_2D_id_set", _wrap_Dwg_Entity_VERTEX_2D_id_set},
{"LibreDWGc::Dwg_Entity_VERTEX_2D_id_get", _wrap_Dwg_Entity_VERTEX_2D_id_get},
{"LibreDWGc::Dwg_Entity_VERTEX_2D_bulge_set", _wrap_Dwg_Entity_VERTEX_2D_bulge_set},
{"LibreDWGc::Dwg_Entity_VERTEX_2D_bulge_get", _wrap_Dwg_Entity_VERTEX_2D_bulge_get},
{"LibreDWGc::Dwg_Entity_VERTEX_2D_tangent_dir_set", _wrap_Dwg_Entity_VERTEX_2D_tangent_dir_set},
{"LibreDWGc::Dwg_Entity_VERTEX_2D_tangent_dir_get", _wrap_Dwg_Entity_VERTEX_2D_tangent_dir_get},
{"LibreDWGc::new_Dwg_Entity_VERTEX_2D", _wrap_new_Dwg_Entity_VERTEX_2D},
{"LibreDWGc::delete_Dwg_Entity_VERTEX_2D", _wrap_delete_Dwg_Entity_VERTEX_2D},
{"LibreDWGc::Dwg_Entity_VERTEX_3D_parent_set", _wrap_Dwg_Entity_VERTEX_3D_parent_set},
{"LibreDWGc::Dwg_Entity_VERTEX_3D_parent_get", _wrap_Dwg_Entity_VERTEX_3D_parent_get},
{"LibreDWGc::Dwg_Entity_VERTEX_3D_flag_set", _wrap_Dwg_Entity_VERTEX_3D_flag_set},
{"LibreDWGc::Dwg_Entity_VERTEX_3D_flag_get", _wrap_Dwg_Entity_VERTEX_3D_flag_get},
{"LibreDWGc::Dwg_Entity_VERTEX_3D_point_set", _wrap_Dwg_Entity_VERTEX_3D_point_set},
{"LibreDWGc::Dwg_Entity_VERTEX_3D_point_get", _wrap_Dwg_Entity_VERTEX_3D_point_get},
{"LibreDWGc::new_Dwg_Entity_VERTEX_3D", _wrap_new_Dwg_Entity_VERTEX_3D},
{"LibreDWGc::delete_Dwg_Entity_VERTEX_3D", _wrap_delete_Dwg_Entity_VERTEX_3D},
{"LibreDWGc::Dwg_Entity_VERTEX_PFACE_FACE_parent_set", _wrap_Dwg_Entity_VERTEX_PFACE_FACE_parent_set},
{"LibreDWGc::Dwg_Entity_VERTEX_PFACE_FACE_parent_get", _wrap_Dwg_Entity_VERTEX_PFACE_FACE_parent_get},
{"LibreDWGc::Dwg_Entity_VERTEX_PFACE_FACE_flag_set", _wrap_Dwg_Entity_VERTEX_PFACE_FACE_flag_set},
{"LibreDWGc::Dwg_Entity_VERTEX_PFACE_FACE_flag_get", _wrap_Dwg_Entity_VERTEX_PFACE_FACE_flag_get},
{"LibreDWGc::Dwg_Entity_VERTEX_PFACE_FACE_vertind_set", _wrap_Dwg_Entity_VERTEX_PFACE_FACE_vertind_set},
{"LibreDWGc::Dwg_Entity_VERTEX_PFACE_FACE_vertind_get", _wrap_Dwg_Entity_VERTEX_PFACE_FACE_vertind_get},
{"LibreDWGc::new_Dwg_Entity_VERTEX_PFACE_FACE", _wrap_new_Dwg_Entity_VERTEX_PFACE_FACE},
{"LibreDWGc::delete_Dwg_Entity_VERTEX_PFACE_FACE", _wrap_delete_Dwg_Entity_VERTEX_PFACE_FACE},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_parent_set", _wrap_Dwg_Entity_POLYLINE_2D_parent_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_parent_get", _wrap_Dwg_Entity_POLYLINE_2D_parent_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_has_vertex_set", _wrap_Dwg_Entity_POLYLINE_2D_has_vertex_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_has_vertex_get", _wrap_Dwg_Entity_POLYLINE_2D_has_vertex_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_num_owned_set", _wrap_Dwg_Entity_POLYLINE_2D_num_owned_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_num_owned_get", _wrap_Dwg_Entity_POLYLINE_2D_num_owned_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_first_vertex_set", _wrap_Dwg_Entity_POLYLINE_2D_first_vertex_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_first_vertex_get", _wrap_Dwg_Entity_POLYLINE_2D_first_vertex_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_last_vertex_set", _wrap_Dwg_Entity_POLYLINE_2D_last_vertex_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_last_vertex_get", _wrap_Dwg_Entity_POLYLINE_2D_last_vertex_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_vertex_set", _wrap_Dwg_Entity_POLYLINE_2D_vertex_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_vertex_get", _wrap_Dwg_Entity_POLYLINE_2D_vertex_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_seqend_set", _wrap_Dwg_Entity_POLYLINE_2D_seqend_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_seqend_get", _wrap_Dwg_Entity_POLYLINE_2D_seqend_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_flag_set", _wrap_Dwg_Entity_POLYLINE_2D_flag_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_flag_get", _wrap_Dwg_Entity_POLYLINE_2D_flag_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_curve_type_set", _wrap_Dwg_Entity_POLYLINE_2D_curve_type_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_curve_type_get", _wrap_Dwg_Entity_POLYLINE_2D_curve_type_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_start_width_set", _wrap_Dwg_Entity_POLYLINE_2D_start_width_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_start_width_get", _wrap_Dwg_Entity_POLYLINE_2D_start_width_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_end_width_set", _wrap_Dwg_Entity_POLYLINE_2D_end_width_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_end_width_get", _wrap_Dwg_Entity_POLYLINE_2D_end_width_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_thickness_set", _wrap_Dwg_Entity_POLYLINE_2D_thickness_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_thickness_get", _wrap_Dwg_Entity_POLYLINE_2D_thickness_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_elevation_set", _wrap_Dwg_Entity_POLYLINE_2D_elevation_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_elevation_get", _wrap_Dwg_Entity_POLYLINE_2D_elevation_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_extrusion_set", _wrap_Dwg_Entity_POLYLINE_2D_extrusion_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_extrusion_get", _wrap_Dwg_Entity_POLYLINE_2D_extrusion_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_extra_r11_size_set", _wrap_Dwg_Entity_POLYLINE_2D_extra_r11_size_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_extra_r11_size_get", _wrap_Dwg_Entity_POLYLINE_2D_extra_r11_size_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_extra_r11_text_set", _wrap_Dwg_Entity_POLYLINE_2D_extra_r11_text_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_extra_r11_text_get", _wrap_Dwg_Entity_POLYLINE_2D_extra_r11_text_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_num_m_verts_set", _wrap_Dwg_Entity_POLYLINE_2D_num_m_verts_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_num_m_verts_get", _wrap_Dwg_Entity_POLYLINE_2D_num_m_verts_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_num_n_verts_set", _wrap_Dwg_Entity_POLYLINE_2D_num_n_verts_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_2D_num_n_verts_get", _wrap_Dwg_Entity_POLYLINE_2D_num_n_verts_get},
{"LibreDWGc::new_Dwg_Entity_POLYLINE_2D", _wrap_new_Dwg_Entity_POLYLINE_2D},
{"LibreDWGc::delete_Dwg_Entity_POLYLINE_2D", _wrap_delete_Dwg_Entity_POLYLINE_2D},
{"LibreDWGc::Dwg_Entity_POLYLINE_3D_parent_set", _wrap_Dwg_Entity_POLYLINE_3D_parent_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_3D_parent_get", _wrap_Dwg_Entity_POLYLINE_3D_parent_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_3D_has_vertex_set", _wrap_Dwg_Entity_POLYLINE_3D_has_vertex_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_3D_has_vertex_get", _wrap_Dwg_Entity_POLYLINE_3D_has_vertex_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_3D_num_owned_set", _wrap_Dwg_Entity_POLYLINE_3D_num_owned_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_3D_num_owned_get", _wrap_Dwg_Entity_POLYLINE_3D_num_owned_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_3D_first_vertex_set", _wrap_Dwg_Entity_POLYLINE_3D_first_vertex_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_3D_first_vertex_get", _wrap_Dwg_Entity_POLYLINE_3D_first_vertex_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_3D_last_vertex_set", _wrap_Dwg_Entity_POLYLINE_3D_last_vertex_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_3D_last_vertex_get", _wrap_Dwg_Entity_POLYLINE_3D_last_vertex_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_3D_vertex_set", _wrap_Dwg_Entity_POLYLINE_3D_vertex_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_3D_vertex_get", _wrap_Dwg_Entity_POLYLINE_3D_vertex_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_3D_seqend_set", _wrap_Dwg_Entity_POLYLINE_3D_seqend_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_3D_seqend_get", _wrap_Dwg_Entity_POLYLINE_3D_seqend_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_3D_curve_type_set", _wrap_Dwg_Entity_POLYLINE_3D_curve_type_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_3D_curve_type_get", _wrap_Dwg_Entity_POLYLINE_3D_curve_type_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_3D_start_width_set", _wrap_Dwg_Entity_POLYLINE_3D_start_width_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_3D_start_width_get", _wrap_Dwg_Entity_POLYLINE_3D_start_width_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_3D_end_width_set", _wrap_Dwg_Entity_POLYLINE_3D_end_width_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_3D_end_width_get", _wrap_Dwg_Entity_POLYLINE_3D_end_width_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_3D_flag_set", _wrap_Dwg_Entity_POLYLINE_3D_flag_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_3D_flag_get", _wrap_Dwg_Entity_POLYLINE_3D_flag_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_3D_extrusion_set", _wrap_Dwg_Entity_POLYLINE_3D_extrusion_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_3D_extrusion_get", _wrap_Dwg_Entity_POLYLINE_3D_extrusion_get},
{"LibreDWGc::new_Dwg_Entity_POLYLINE_3D", _wrap_new_Dwg_Entity_POLYLINE_3D},
{"LibreDWGc::delete_Dwg_Entity_POLYLINE_3D", _wrap_delete_Dwg_Entity_POLYLINE_3D},
{"LibreDWGc::Dwg_Entity_ARC_parent_set", _wrap_Dwg_Entity_ARC_parent_set},
{"LibreDWGc::Dwg_Entity_ARC_parent_get", _wrap_Dwg_Entity_ARC_parent_get},
{"LibreDWGc::Dwg_Entity_ARC_center_set", _wrap_Dwg_Entity_ARC_center_set},
{"LibreDWGc::Dwg_Entity_ARC_center_get", _wrap_Dwg_Entity_ARC_center_get},
{"LibreDWGc::Dwg_Entity_ARC_radius_set", _wrap_Dwg_Entity_ARC_radius_set},
{"LibreDWGc::Dwg_Entity_ARC_radius_get", _wrap_Dwg_Entity_ARC_radius_get},
{"LibreDWGc::Dwg_Entity_ARC_thickness_set", _wrap_Dwg_Entity_ARC_thickness_set},
{"LibreDWGc::Dwg_Entity_ARC_thickness_get", _wrap_Dwg_Entity_ARC_thickness_get},
{"LibreDWGc::Dwg_Entity_ARC_extrusion_set", _wrap_Dwg_Entity_ARC_extrusion_set},
{"LibreDWGc::Dwg_Entity_ARC_extrusion_get", _wrap_Dwg_Entity_ARC_extrusion_get},
{"LibreDWGc::Dwg_Entity_ARC_start_angle_set", _wrap_Dwg_Entity_ARC_start_angle_set},
{"LibreDWGc::Dwg_Entity_ARC_start_angle_get", _wrap_Dwg_Entity_ARC_start_angle_get},
{"LibreDWGc::Dwg_Entity_ARC_end_angle_set", _wrap_Dwg_Entity_ARC_end_angle_set},
{"LibreDWGc::Dwg_Entity_ARC_end_angle_get", _wrap_Dwg_Entity_ARC_end_angle_get},
{"LibreDWGc::new_Dwg_Entity_ARC", _wrap_new_Dwg_Entity_ARC},
{"LibreDWGc::delete_Dwg_Entity_ARC", _wrap_delete_Dwg_Entity_ARC},
{"LibreDWGc::Dwg_Entity_CIRCLE_parent_set", _wrap_Dwg_Entity_CIRCLE_parent_set},
{"LibreDWGc::Dwg_Entity_CIRCLE_parent_get", _wrap_Dwg_Entity_CIRCLE_parent_get},
{"LibreDWGc::Dwg_Entity_CIRCLE_center_set", _wrap_Dwg_Entity_CIRCLE_center_set},
{"LibreDWGc::Dwg_Entity_CIRCLE_center_get", _wrap_Dwg_Entity_CIRCLE_center_get},
{"LibreDWGc::Dwg_Entity_CIRCLE_radius_set", _wrap_Dwg_Entity_CIRCLE_radius_set},
{"LibreDWGc::Dwg_Entity_CIRCLE_radius_get", _wrap_Dwg_Entity_CIRCLE_radius_get},
{"LibreDWGc::Dwg_Entity_CIRCLE_thickness_set", _wrap_Dwg_Entity_CIRCLE_thickness_set},
{"LibreDWGc::Dwg_Entity_CIRCLE_thickness_get", _wrap_Dwg_Entity_CIRCLE_thickness_get},
{"LibreDWGc::Dwg_Entity_CIRCLE_extrusion_set", _wrap_Dwg_Entity_CIRCLE_extrusion_set},
{"LibreDWGc::Dwg_Entity_CIRCLE_extrusion_get", _wrap_Dwg_Entity_CIRCLE_extrusion_get},
{"LibreDWGc::new_Dwg_Entity_CIRCLE", _wrap_new_Dwg_Entity_CIRCLE},
{"LibreDWGc::delete_Dwg_Entity_CIRCLE", _wrap_delete_Dwg_Entity_CIRCLE},
{"LibreDWGc::Dwg_Entity_LINE_parent_set", _wrap_Dwg_Entity_LINE_parent_set},
{"LibreDWGc::Dwg_Entity_LINE_parent_get", _wrap_Dwg_Entity_LINE_parent_get},
{"LibreDWGc::Dwg_Entity_LINE_z_is_zero_set", _wrap_Dwg_Entity_LINE_z_is_zero_set},
{"LibreDWGc::Dwg_Entity_LINE_z_is_zero_get", _wrap_Dwg_Entity_LINE_z_is_zero_get},
{"LibreDWGc::Dwg_Entity_LINE_start_set", _wrap_Dwg_Entity_LINE_start_set},
{"LibreDWGc::Dwg_Entity_LINE_start_get", _wrap_Dwg_Entity_LINE_start_get},
{"LibreDWGc::Dwg_Entity_LINE_end_set", _wrap_Dwg_Entity_LINE_end_set},
{"LibreDWGc::Dwg_Entity_LINE_end_get", _wrap_Dwg_Entity_LINE_end_get},
{"LibreDWGc::Dwg_Entity_LINE_thickness_set", _wrap_Dwg_Entity_LINE_thickness_set},
{"LibreDWGc::Dwg_Entity_LINE_thickness_get", _wrap_Dwg_Entity_LINE_thickness_get},
{"LibreDWGc::Dwg_Entity_LINE_extrusion_set", _wrap_Dwg_Entity_LINE_extrusion_set},
{"LibreDWGc::Dwg_Entity_LINE_extrusion_get", _wrap_Dwg_Entity_LINE_extrusion_get},
{"LibreDWGc::Dwg_Entity_LINE_unknown_r11_set", _wrap_Dwg_Entity_LINE_unknown_r11_set},
{"LibreDWGc::Dwg_Entity_LINE_unknown_r11_get", _wrap_Dwg_Entity_LINE_unknown_r11_get},
{"LibreDWGc::new_Dwg_Entity_LINE", _wrap_new_Dwg_Entity_LINE},
{"LibreDWGc::delete_Dwg_Entity_LINE", _wrap_delete_Dwg_Entity_LINE},
{"LibreDWGc::Dwg_DIMENSION_common_parent_set", _wrap_Dwg_DIMENSION_common_parent_set},
{"LibreDWGc::Dwg_DIMENSION_common_parent_get", _wrap_Dwg_DIMENSION_common_parent_get},
{"LibreDWGc::Dwg_DIMENSION_common_class_version_set", _wrap_Dwg_DIMENSION_common_class_version_set},
{"LibreDWGc::Dwg_DIMENSION_common_class_version_get", _wrap_Dwg_DIMENSION_common_class_version_get},
{"LibreDWGc::Dwg_DIMENSION_common_extrusion_set", _wrap_Dwg_DIMENSION_common_extrusion_set},
{"LibreDWGc::Dwg_DIMENSION_common_extrusion_get", _wrap_Dwg_DIMENSION_common_extrusion_get},
{"LibreDWGc::Dwg_DIMENSION_common_def_pt_set", _wrap_Dwg_DIMENSION_common_def_pt_set},
{"LibreDWGc::Dwg_DIMENSION_common_def_pt_get", _wrap_Dwg_DIMENSION_common_def_pt_get},
{"LibreDWGc::Dwg_DIMENSION_common_text_midpt_set", _wrap_Dwg_DIMENSION_common_text_midpt_set},
{"LibreDWGc::Dwg_DIMENSION_common_text_midpt_get", _wrap_Dwg_DIMENSION_common_text_midpt_get},
{"LibreDWGc::Dwg_DIMENSION_common_elevation_set", _wrap_Dwg_DIMENSION_common_elevation_set},
{"LibreDWGc::Dwg_DIMENSION_common_elevation_get", _wrap_Dwg_DIMENSION_common_elevation_get},
{"LibreDWGc::Dwg_DIMENSION_common_flag_set", _wrap_Dwg_DIMENSION_common_flag_set},
{"LibreDWGc::Dwg_DIMENSION_common_flag_get", _wrap_Dwg_DIMENSION_common_flag_get},
{"LibreDWGc::Dwg_DIMENSION_common_flag1_set", _wrap_Dwg_DIMENSION_common_flag1_set},
{"LibreDWGc::Dwg_DIMENSION_common_flag1_get", _wrap_Dwg_DIMENSION_common_flag1_get},
{"LibreDWGc::Dwg_DIMENSION_common_user_text_set", _wrap_Dwg_DIMENSION_common_user_text_set},
{"LibreDWGc::Dwg_DIMENSION_common_user_text_get", _wrap_Dwg_DIMENSION_common_user_text_get},
{"LibreDWGc::Dwg_DIMENSION_common_text_rotation_set", _wrap_Dwg_DIMENSION_common_text_rotation_set},
{"LibreDWGc::Dwg_DIMENSION_common_text_rotation_get", _wrap_Dwg_DIMENSION_common_text_rotation_get},
{"LibreDWGc::Dwg_DIMENSION_common_horiz_dir_set", _wrap_Dwg_DIMENSION_common_horiz_dir_set},
{"LibreDWGc::Dwg_DIMENSION_common_horiz_dir_get", _wrap_Dwg_DIMENSION_common_horiz_dir_get},
{"LibreDWGc::Dwg_DIMENSION_common_ins_scale_set", _wrap_Dwg_DIMENSION_common_ins_scale_set},
{"LibreDWGc::Dwg_DIMENSION_common_ins_scale_get", _wrap_Dwg_DIMENSION_common_ins_scale_get},
{"LibreDWGc::Dwg_DIMENSION_common_ins_rotation_set", _wrap_Dwg_DIMENSION_common_ins_rotation_set},
{"LibreDWGc::Dwg_DIMENSION_common_ins_rotation_get", _wrap_Dwg_DIMENSION_common_ins_rotation_get},
{"LibreDWGc::Dwg_DIMENSION_common_attachment_set", _wrap_Dwg_DIMENSION_common_attachment_set},
{"LibreDWGc::Dwg_DIMENSION_common_attachment_get", _wrap_Dwg_DIMENSION_common_attachment_get},
{"LibreDWGc::Dwg_DIMENSION_common_lspace_style_set", _wrap_Dwg_DIMENSION_common_lspace_style_set},
{"LibreDWGc::Dwg_DIMENSION_common_lspace_style_get", _wrap_Dwg_DIMENSION_common_lspace_style_get},
{"LibreDWGc::Dwg_DIMENSION_common_lspace_factor_set", _wrap_Dwg_DIMENSION_common_lspace_factor_set},
{"LibreDWGc::Dwg_DIMENSION_common_lspace_factor_get", _wrap_Dwg_DIMENSION_common_lspace_factor_get},
{"LibreDWGc::Dwg_DIMENSION_common_act_measurement_set", _wrap_Dwg_DIMENSION_common_act_measurement_set},
{"LibreDWGc::Dwg_DIMENSION_common_act_measurement_get", _wrap_Dwg_DIMENSION_common_act_measurement_get},
{"LibreDWGc::Dwg_DIMENSION_common_unknown_set", _wrap_Dwg_DIMENSION_common_unknown_set},
{"LibreDWGc::Dwg_DIMENSION_common_unknown_get", _wrap_Dwg_DIMENSION_common_unknown_get},
{"LibreDWGc::Dwg_DIMENSION_common_flip_arrow1_set", _wrap_Dwg_DIMENSION_common_flip_arrow1_set},
{"LibreDWGc::Dwg_DIMENSION_common_flip_arrow1_get", _wrap_Dwg_DIMENSION_common_flip_arrow1_get},
{"LibreDWGc::Dwg_DIMENSION_common_flip_arrow2_set", _wrap_Dwg_DIMENSION_common_flip_arrow2_set},
{"LibreDWGc::Dwg_DIMENSION_common_flip_arrow2_get", _wrap_Dwg_DIMENSION_common_flip_arrow2_get},
{"LibreDWGc::Dwg_DIMENSION_common_clone_ins_pt_set", _wrap_Dwg_DIMENSION_common_clone_ins_pt_set},
{"LibreDWGc::Dwg_DIMENSION_common_clone_ins_pt_get", _wrap_Dwg_DIMENSION_common_clone_ins_pt_get},
{"LibreDWGc::Dwg_DIMENSION_common_dimstyle_set", _wrap_Dwg_DIMENSION_common_dimstyle_set},
{"LibreDWGc::Dwg_DIMENSION_common_dimstyle_get", _wrap_Dwg_DIMENSION_common_dimstyle_get},
{"LibreDWGc::Dwg_DIMENSION_common_block_set", _wrap_Dwg_DIMENSION_common_block_set},
{"LibreDWGc::Dwg_DIMENSION_common_block_get", _wrap_Dwg_DIMENSION_common_block_get},
{"LibreDWGc::new_Dwg_DIMENSION_common", _wrap_new_Dwg_DIMENSION_common},
{"LibreDWGc::delete_Dwg_DIMENSION_common", _wrap_delete_Dwg_DIMENSION_common},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_parent_set", _wrap_Dwg_Entity_DIMENSION_ORDINATE_parent_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_parent_get", _wrap_Dwg_Entity_DIMENSION_ORDINATE_parent_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_class_version_set", _wrap_Dwg_Entity_DIMENSION_ORDINATE_class_version_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_class_version_get", _wrap_Dwg_Entity_DIMENSION_ORDINATE_class_version_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_extrusion_set", _wrap_Dwg_Entity_DIMENSION_ORDINATE_extrusion_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_extrusion_get", _wrap_Dwg_Entity_DIMENSION_ORDINATE_extrusion_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_def_pt_set", _wrap_Dwg_Entity_DIMENSION_ORDINATE_def_pt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_def_pt_get", _wrap_Dwg_Entity_DIMENSION_ORDINATE_def_pt_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_text_midpt_set", _wrap_Dwg_Entity_DIMENSION_ORDINATE_text_midpt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_text_midpt_get", _wrap_Dwg_Entity_DIMENSION_ORDINATE_text_midpt_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_elevation_set", _wrap_Dwg_Entity_DIMENSION_ORDINATE_elevation_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_elevation_get", _wrap_Dwg_Entity_DIMENSION_ORDINATE_elevation_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_flag_set", _wrap_Dwg_Entity_DIMENSION_ORDINATE_flag_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_flag_get", _wrap_Dwg_Entity_DIMENSION_ORDINATE_flag_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_flag1_set", _wrap_Dwg_Entity_DIMENSION_ORDINATE_flag1_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_flag1_get", _wrap_Dwg_Entity_DIMENSION_ORDINATE_flag1_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_user_text_set", _wrap_Dwg_Entity_DIMENSION_ORDINATE_user_text_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_user_text_get", _wrap_Dwg_Entity_DIMENSION_ORDINATE_user_text_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_text_rotation_set", _wrap_Dwg_Entity_DIMENSION_ORDINATE_text_rotation_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_text_rotation_get", _wrap_Dwg_Entity_DIMENSION_ORDINATE_text_rotation_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_horiz_dir_set", _wrap_Dwg_Entity_DIMENSION_ORDINATE_horiz_dir_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_horiz_dir_get", _wrap_Dwg_Entity_DIMENSION_ORDINATE_horiz_dir_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_ins_scale_set", _wrap_Dwg_Entity_DIMENSION_ORDINATE_ins_scale_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_ins_scale_get", _wrap_Dwg_Entity_DIMENSION_ORDINATE_ins_scale_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_ins_rotation_set", _wrap_Dwg_Entity_DIMENSION_ORDINATE_ins_rotation_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_ins_rotation_get", _wrap_Dwg_Entity_DIMENSION_ORDINATE_ins_rotation_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_attachment_set", _wrap_Dwg_Entity_DIMENSION_ORDINATE_attachment_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_attachment_get", _wrap_Dwg_Entity_DIMENSION_ORDINATE_attachment_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_lspace_style_set", _wrap_Dwg_Entity_DIMENSION_ORDINATE_lspace_style_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_lspace_style_get", _wrap_Dwg_Entity_DIMENSION_ORDINATE_lspace_style_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_lspace_factor_set", _wrap_Dwg_Entity_DIMENSION_ORDINATE_lspace_factor_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_lspace_factor_get", _wrap_Dwg_Entity_DIMENSION_ORDINATE_lspace_factor_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_act_measurement_set", _wrap_Dwg_Entity_DIMENSION_ORDINATE_act_measurement_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_act_measurement_get", _wrap_Dwg_Entity_DIMENSION_ORDINATE_act_measurement_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_unknown_set", _wrap_Dwg_Entity_DIMENSION_ORDINATE_unknown_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_unknown_get", _wrap_Dwg_Entity_DIMENSION_ORDINATE_unknown_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_flip_arrow1_set", _wrap_Dwg_Entity_DIMENSION_ORDINATE_flip_arrow1_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_flip_arrow1_get", _wrap_Dwg_Entity_DIMENSION_ORDINATE_flip_arrow1_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_flip_arrow2_set", _wrap_Dwg_Entity_DIMENSION_ORDINATE_flip_arrow2_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_flip_arrow2_get", _wrap_Dwg_Entity_DIMENSION_ORDINATE_flip_arrow2_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_clone_ins_pt_set", _wrap_Dwg_Entity_DIMENSION_ORDINATE_clone_ins_pt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_clone_ins_pt_get", _wrap_Dwg_Entity_DIMENSION_ORDINATE_clone_ins_pt_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_dimstyle_set", _wrap_Dwg_Entity_DIMENSION_ORDINATE_dimstyle_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_dimstyle_get", _wrap_Dwg_Entity_DIMENSION_ORDINATE_dimstyle_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_block_set", _wrap_Dwg_Entity_DIMENSION_ORDINATE_block_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_block_get", _wrap_Dwg_Entity_DIMENSION_ORDINATE_block_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_feature_location_pt_set", _wrap_Dwg_Entity_DIMENSION_ORDINATE_feature_location_pt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_feature_location_pt_get", _wrap_Dwg_Entity_DIMENSION_ORDINATE_feature_location_pt_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_leader_endpt_set", _wrap_Dwg_Entity_DIMENSION_ORDINATE_leader_endpt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_leader_endpt_get", _wrap_Dwg_Entity_DIMENSION_ORDINATE_leader_endpt_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_flag2_set", _wrap_Dwg_Entity_DIMENSION_ORDINATE_flag2_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_flag2_get", _wrap_Dwg_Entity_DIMENSION_ORDINATE_flag2_get},
{"LibreDWGc::new_Dwg_Entity_DIMENSION_ORDINATE", _wrap_new_Dwg_Entity_DIMENSION_ORDINATE},
{"LibreDWGc::delete_Dwg_Entity_DIMENSION_ORDINATE", _wrap_delete_Dwg_Entity_DIMENSION_ORDINATE},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_parent_set", _wrap_Dwg_Entity_DIMENSION_LINEAR_parent_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_parent_get", _wrap_Dwg_Entity_DIMENSION_LINEAR_parent_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_class_version_set", _wrap_Dwg_Entity_DIMENSION_LINEAR_class_version_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_class_version_get", _wrap_Dwg_Entity_DIMENSION_LINEAR_class_version_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_extrusion_set", _wrap_Dwg_Entity_DIMENSION_LINEAR_extrusion_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_extrusion_get", _wrap_Dwg_Entity_DIMENSION_LINEAR_extrusion_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_def_pt_set", _wrap_Dwg_Entity_DIMENSION_LINEAR_def_pt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_def_pt_get", _wrap_Dwg_Entity_DIMENSION_LINEAR_def_pt_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_text_midpt_set", _wrap_Dwg_Entity_DIMENSION_LINEAR_text_midpt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_text_midpt_get", _wrap_Dwg_Entity_DIMENSION_LINEAR_text_midpt_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_elevation_set", _wrap_Dwg_Entity_DIMENSION_LINEAR_elevation_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_elevation_get", _wrap_Dwg_Entity_DIMENSION_LINEAR_elevation_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_flag_set", _wrap_Dwg_Entity_DIMENSION_LINEAR_flag_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_flag_get", _wrap_Dwg_Entity_DIMENSION_LINEAR_flag_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_flag1_set", _wrap_Dwg_Entity_DIMENSION_LINEAR_flag1_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_flag1_get", _wrap_Dwg_Entity_DIMENSION_LINEAR_flag1_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_user_text_set", _wrap_Dwg_Entity_DIMENSION_LINEAR_user_text_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_user_text_get", _wrap_Dwg_Entity_DIMENSION_LINEAR_user_text_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_text_rotation_set", _wrap_Dwg_Entity_DIMENSION_LINEAR_text_rotation_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_text_rotation_get", _wrap_Dwg_Entity_DIMENSION_LINEAR_text_rotation_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_horiz_dir_set", _wrap_Dwg_Entity_DIMENSION_LINEAR_horiz_dir_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_horiz_dir_get", _wrap_Dwg_Entity_DIMENSION_LINEAR_horiz_dir_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_ins_scale_set", _wrap_Dwg_Entity_DIMENSION_LINEAR_ins_scale_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_ins_scale_get", _wrap_Dwg_Entity_DIMENSION_LINEAR_ins_scale_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_ins_rotation_set", _wrap_Dwg_Entity_DIMENSION_LINEAR_ins_rotation_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_ins_rotation_get", _wrap_Dwg_Entity_DIMENSION_LINEAR_ins_rotation_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_attachment_set", _wrap_Dwg_Entity_DIMENSION_LINEAR_attachment_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_attachment_get", _wrap_Dwg_Entity_DIMENSION_LINEAR_attachment_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_lspace_style_set", _wrap_Dwg_Entity_DIMENSION_LINEAR_lspace_style_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_lspace_style_get", _wrap_Dwg_Entity_DIMENSION_LINEAR_lspace_style_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_lspace_factor_set", _wrap_Dwg_Entity_DIMENSION_LINEAR_lspace_factor_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_lspace_factor_get", _wrap_Dwg_Entity_DIMENSION_LINEAR_lspace_factor_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_act_measurement_set", _wrap_Dwg_Entity_DIMENSION_LINEAR_act_measurement_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_act_measurement_get", _wrap_Dwg_Entity_DIMENSION_LINEAR_act_measurement_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_unknown_set", _wrap_Dwg_Entity_DIMENSION_LINEAR_unknown_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_unknown_get", _wrap_Dwg_Entity_DIMENSION_LINEAR_unknown_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_flip_arrow1_set", _wrap_Dwg_Entity_DIMENSION_LINEAR_flip_arrow1_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_flip_arrow1_get", _wrap_Dwg_Entity_DIMENSION_LINEAR_flip_arrow1_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_flip_arrow2_set", _wrap_Dwg_Entity_DIMENSION_LINEAR_flip_arrow2_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_flip_arrow2_get", _wrap_Dwg_Entity_DIMENSION_LINEAR_flip_arrow2_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_clone_ins_pt_set", _wrap_Dwg_Entity_DIMENSION_LINEAR_clone_ins_pt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_clone_ins_pt_get", _wrap_Dwg_Entity_DIMENSION_LINEAR_clone_ins_pt_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_dimstyle_set", _wrap_Dwg_Entity_DIMENSION_LINEAR_dimstyle_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_dimstyle_get", _wrap_Dwg_Entity_DIMENSION_LINEAR_dimstyle_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_block_set", _wrap_Dwg_Entity_DIMENSION_LINEAR_block_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_block_get", _wrap_Dwg_Entity_DIMENSION_LINEAR_block_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_xline1_pt_set", _wrap_Dwg_Entity_DIMENSION_LINEAR_xline1_pt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_xline1_pt_get", _wrap_Dwg_Entity_DIMENSION_LINEAR_xline1_pt_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_xline2_pt_set", _wrap_Dwg_Entity_DIMENSION_LINEAR_xline2_pt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_xline2_pt_get", _wrap_Dwg_Entity_DIMENSION_LINEAR_xline2_pt_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_oblique_angle_set", _wrap_Dwg_Entity_DIMENSION_LINEAR_oblique_angle_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_oblique_angle_get", _wrap_Dwg_Entity_DIMENSION_LINEAR_oblique_angle_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_dim_rotation_set", _wrap_Dwg_Entity_DIMENSION_LINEAR_dim_rotation_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_dim_rotation_get", _wrap_Dwg_Entity_DIMENSION_LINEAR_dim_rotation_get},
{"LibreDWGc::new_Dwg_Entity_DIMENSION_LINEAR", _wrap_new_Dwg_Entity_DIMENSION_LINEAR},
{"LibreDWGc::delete_Dwg_Entity_DIMENSION_LINEAR", _wrap_delete_Dwg_Entity_DIMENSION_LINEAR},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_parent_set", _wrap_Dwg_Entity_DIMENSION_ALIGNED_parent_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_parent_get", _wrap_Dwg_Entity_DIMENSION_ALIGNED_parent_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_class_version_set", _wrap_Dwg_Entity_DIMENSION_ALIGNED_class_version_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_class_version_get", _wrap_Dwg_Entity_DIMENSION_ALIGNED_class_version_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_extrusion_set", _wrap_Dwg_Entity_DIMENSION_ALIGNED_extrusion_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_extrusion_get", _wrap_Dwg_Entity_DIMENSION_ALIGNED_extrusion_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_def_pt_set", _wrap_Dwg_Entity_DIMENSION_ALIGNED_def_pt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_def_pt_get", _wrap_Dwg_Entity_DIMENSION_ALIGNED_def_pt_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_text_midpt_set", _wrap_Dwg_Entity_DIMENSION_ALIGNED_text_midpt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_text_midpt_get", _wrap_Dwg_Entity_DIMENSION_ALIGNED_text_midpt_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_elevation_set", _wrap_Dwg_Entity_DIMENSION_ALIGNED_elevation_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_elevation_get", _wrap_Dwg_Entity_DIMENSION_ALIGNED_elevation_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_flag_set", _wrap_Dwg_Entity_DIMENSION_ALIGNED_flag_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_flag_get", _wrap_Dwg_Entity_DIMENSION_ALIGNED_flag_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_flag1_set", _wrap_Dwg_Entity_DIMENSION_ALIGNED_flag1_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_flag1_get", _wrap_Dwg_Entity_DIMENSION_ALIGNED_flag1_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_user_text_set", _wrap_Dwg_Entity_DIMENSION_ALIGNED_user_text_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_user_text_get", _wrap_Dwg_Entity_DIMENSION_ALIGNED_user_text_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_text_rotation_set", _wrap_Dwg_Entity_DIMENSION_ALIGNED_text_rotation_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_text_rotation_get", _wrap_Dwg_Entity_DIMENSION_ALIGNED_text_rotation_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_horiz_dir_set", _wrap_Dwg_Entity_DIMENSION_ALIGNED_horiz_dir_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_horiz_dir_get", _wrap_Dwg_Entity_DIMENSION_ALIGNED_horiz_dir_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_ins_scale_set", _wrap_Dwg_Entity_DIMENSION_ALIGNED_ins_scale_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_ins_scale_get", _wrap_Dwg_Entity_DIMENSION_ALIGNED_ins_scale_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_ins_rotation_set", _wrap_Dwg_Entity_DIMENSION_ALIGNED_ins_rotation_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_ins_rotation_get", _wrap_Dwg_Entity_DIMENSION_ALIGNED_ins_rotation_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_attachment_set", _wrap_Dwg_Entity_DIMENSION_ALIGNED_attachment_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_attachment_get", _wrap_Dwg_Entity_DIMENSION_ALIGNED_attachment_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_lspace_style_set", _wrap_Dwg_Entity_DIMENSION_ALIGNED_lspace_style_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_lspace_style_get", _wrap_Dwg_Entity_DIMENSION_ALIGNED_lspace_style_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_lspace_factor_set", _wrap_Dwg_Entity_DIMENSION_ALIGNED_lspace_factor_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_lspace_factor_get", _wrap_Dwg_Entity_DIMENSION_ALIGNED_lspace_factor_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_act_measurement_set", _wrap_Dwg_Entity_DIMENSION_ALIGNED_act_measurement_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_act_measurement_get", _wrap_Dwg_Entity_DIMENSION_ALIGNED_act_measurement_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_unknown_set", _wrap_Dwg_Entity_DIMENSION_ALIGNED_unknown_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_unknown_get", _wrap_Dwg_Entity_DIMENSION_ALIGNED_unknown_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_flip_arrow1_set", _wrap_Dwg_Entity_DIMENSION_ALIGNED_flip_arrow1_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_flip_arrow1_get", _wrap_Dwg_Entity_DIMENSION_ALIGNED_flip_arrow1_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_flip_arrow2_set", _wrap_Dwg_Entity_DIMENSION_ALIGNED_flip_arrow2_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_flip_arrow2_get", _wrap_Dwg_Entity_DIMENSION_ALIGNED_flip_arrow2_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_clone_ins_pt_set", _wrap_Dwg_Entity_DIMENSION_ALIGNED_clone_ins_pt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_clone_ins_pt_get", _wrap_Dwg_Entity_DIMENSION_ALIGNED_clone_ins_pt_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_dimstyle_set", _wrap_Dwg_Entity_DIMENSION_ALIGNED_dimstyle_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_dimstyle_get", _wrap_Dwg_Entity_DIMENSION_ALIGNED_dimstyle_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_block_set", _wrap_Dwg_Entity_DIMENSION_ALIGNED_block_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_block_get", _wrap_Dwg_Entity_DIMENSION_ALIGNED_block_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_xline1_pt_set", _wrap_Dwg_Entity_DIMENSION_ALIGNED_xline1_pt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_xline1_pt_get", _wrap_Dwg_Entity_DIMENSION_ALIGNED_xline1_pt_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_xline2_pt_set", _wrap_Dwg_Entity_DIMENSION_ALIGNED_xline2_pt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_xline2_pt_get", _wrap_Dwg_Entity_DIMENSION_ALIGNED_xline2_pt_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_oblique_angle_set", _wrap_Dwg_Entity_DIMENSION_ALIGNED_oblique_angle_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_oblique_angle_get", _wrap_Dwg_Entity_DIMENSION_ALIGNED_oblique_angle_get},
{"LibreDWGc::new_Dwg_Entity_DIMENSION_ALIGNED", _wrap_new_Dwg_Entity_DIMENSION_ALIGNED},
{"LibreDWGc::delete_Dwg_Entity_DIMENSION_ALIGNED", _wrap_delete_Dwg_Entity_DIMENSION_ALIGNED},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_parent_set", _wrap_Dwg_Entity_DIMENSION_ANG3PT_parent_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_parent_get", _wrap_Dwg_Entity_DIMENSION_ANG3PT_parent_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_class_version_set", _wrap_Dwg_Entity_DIMENSION_ANG3PT_class_version_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_class_version_get", _wrap_Dwg_Entity_DIMENSION_ANG3PT_class_version_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_extrusion_set", _wrap_Dwg_Entity_DIMENSION_ANG3PT_extrusion_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_extrusion_get", _wrap_Dwg_Entity_DIMENSION_ANG3PT_extrusion_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_def_pt_set", _wrap_Dwg_Entity_DIMENSION_ANG3PT_def_pt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_def_pt_get", _wrap_Dwg_Entity_DIMENSION_ANG3PT_def_pt_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_text_midpt_set", _wrap_Dwg_Entity_DIMENSION_ANG3PT_text_midpt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_text_midpt_get", _wrap_Dwg_Entity_DIMENSION_ANG3PT_text_midpt_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_elevation_set", _wrap_Dwg_Entity_DIMENSION_ANG3PT_elevation_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_elevation_get", _wrap_Dwg_Entity_DIMENSION_ANG3PT_elevation_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_flag_set", _wrap_Dwg_Entity_DIMENSION_ANG3PT_flag_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_flag_get", _wrap_Dwg_Entity_DIMENSION_ANG3PT_flag_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_flag1_set", _wrap_Dwg_Entity_DIMENSION_ANG3PT_flag1_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_flag1_get", _wrap_Dwg_Entity_DIMENSION_ANG3PT_flag1_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_user_text_set", _wrap_Dwg_Entity_DIMENSION_ANG3PT_user_text_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_user_text_get", _wrap_Dwg_Entity_DIMENSION_ANG3PT_user_text_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_text_rotation_set", _wrap_Dwg_Entity_DIMENSION_ANG3PT_text_rotation_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_text_rotation_get", _wrap_Dwg_Entity_DIMENSION_ANG3PT_text_rotation_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_horiz_dir_set", _wrap_Dwg_Entity_DIMENSION_ANG3PT_horiz_dir_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_horiz_dir_get", _wrap_Dwg_Entity_DIMENSION_ANG3PT_horiz_dir_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_ins_scale_set", _wrap_Dwg_Entity_DIMENSION_ANG3PT_ins_scale_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_ins_scale_get", _wrap_Dwg_Entity_DIMENSION_ANG3PT_ins_scale_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_ins_rotation_set", _wrap_Dwg_Entity_DIMENSION_ANG3PT_ins_rotation_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_ins_rotation_get", _wrap_Dwg_Entity_DIMENSION_ANG3PT_ins_rotation_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_attachment_set", _wrap_Dwg_Entity_DIMENSION_ANG3PT_attachment_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_attachment_get", _wrap_Dwg_Entity_DIMENSION_ANG3PT_attachment_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_lspace_style_set", _wrap_Dwg_Entity_DIMENSION_ANG3PT_lspace_style_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_lspace_style_get", _wrap_Dwg_Entity_DIMENSION_ANG3PT_lspace_style_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_lspace_factor_set", _wrap_Dwg_Entity_DIMENSION_ANG3PT_lspace_factor_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_lspace_factor_get", _wrap_Dwg_Entity_DIMENSION_ANG3PT_lspace_factor_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_act_measurement_set", _wrap_Dwg_Entity_DIMENSION_ANG3PT_act_measurement_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_act_measurement_get", _wrap_Dwg_Entity_DIMENSION_ANG3PT_act_measurement_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_unknown_set", _wrap_Dwg_Entity_DIMENSION_ANG3PT_unknown_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_unknown_get", _wrap_Dwg_Entity_DIMENSION_ANG3PT_unknown_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_flip_arrow1_set", _wrap_Dwg_Entity_DIMENSION_ANG3PT_flip_arrow1_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_flip_arrow1_get", _wrap_Dwg_Entity_DIMENSION_ANG3PT_flip_arrow1_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_flip_arrow2_set", _wrap_Dwg_Entity_DIMENSION_ANG3PT_flip_arrow2_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_flip_arrow2_get", _wrap_Dwg_Entity_DIMENSION_ANG3PT_flip_arrow2_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_clone_ins_pt_set", _wrap_Dwg_Entity_DIMENSION_ANG3PT_clone_ins_pt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_clone_ins_pt_get", _wrap_Dwg_Entity_DIMENSION_ANG3PT_clone_ins_pt_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_dimstyle_set", _wrap_Dwg_Entity_DIMENSION_ANG3PT_dimstyle_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_dimstyle_get", _wrap_Dwg_Entity_DIMENSION_ANG3PT_dimstyle_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_block_set", _wrap_Dwg_Entity_DIMENSION_ANG3PT_block_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_block_get", _wrap_Dwg_Entity_DIMENSION_ANG3PT_block_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_xline1_pt_set", _wrap_Dwg_Entity_DIMENSION_ANG3PT_xline1_pt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_xline1_pt_get", _wrap_Dwg_Entity_DIMENSION_ANG3PT_xline1_pt_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_xline2_pt_set", _wrap_Dwg_Entity_DIMENSION_ANG3PT_xline2_pt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_xline2_pt_get", _wrap_Dwg_Entity_DIMENSION_ANG3PT_xline2_pt_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_center_pt_set", _wrap_Dwg_Entity_DIMENSION_ANG3PT_center_pt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_center_pt_get", _wrap_Dwg_Entity_DIMENSION_ANG3PT_center_pt_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_xline2end_pt_set", _wrap_Dwg_Entity_DIMENSION_ANG3PT_xline2end_pt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_xline2end_pt_get", _wrap_Dwg_Entity_DIMENSION_ANG3PT_xline2end_pt_get},
{"LibreDWGc::new_Dwg_Entity_DIMENSION_ANG3PT", _wrap_new_Dwg_Entity_DIMENSION_ANG3PT},
{"LibreDWGc::delete_Dwg_Entity_DIMENSION_ANG3PT", _wrap_delete_Dwg_Entity_DIMENSION_ANG3PT},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_parent_set", _wrap_Dwg_Entity_DIMENSION_ANG2LN_parent_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_parent_get", _wrap_Dwg_Entity_DIMENSION_ANG2LN_parent_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_class_version_set", _wrap_Dwg_Entity_DIMENSION_ANG2LN_class_version_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_class_version_get", _wrap_Dwg_Entity_DIMENSION_ANG2LN_class_version_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_extrusion_set", _wrap_Dwg_Entity_DIMENSION_ANG2LN_extrusion_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_extrusion_get", _wrap_Dwg_Entity_DIMENSION_ANG2LN_extrusion_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_def_pt_set", _wrap_Dwg_Entity_DIMENSION_ANG2LN_def_pt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_def_pt_get", _wrap_Dwg_Entity_DIMENSION_ANG2LN_def_pt_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_text_midpt_set", _wrap_Dwg_Entity_DIMENSION_ANG2LN_text_midpt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_text_midpt_get", _wrap_Dwg_Entity_DIMENSION_ANG2LN_text_midpt_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_elevation_set", _wrap_Dwg_Entity_DIMENSION_ANG2LN_elevation_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_elevation_get", _wrap_Dwg_Entity_DIMENSION_ANG2LN_elevation_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_flag_set", _wrap_Dwg_Entity_DIMENSION_ANG2LN_flag_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_flag_get", _wrap_Dwg_Entity_DIMENSION_ANG2LN_flag_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_flag1_set", _wrap_Dwg_Entity_DIMENSION_ANG2LN_flag1_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_flag1_get", _wrap_Dwg_Entity_DIMENSION_ANG2LN_flag1_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_user_text_set", _wrap_Dwg_Entity_DIMENSION_ANG2LN_user_text_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_user_text_get", _wrap_Dwg_Entity_DIMENSION_ANG2LN_user_text_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_text_rotation_set", _wrap_Dwg_Entity_DIMENSION_ANG2LN_text_rotation_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_text_rotation_get", _wrap_Dwg_Entity_DIMENSION_ANG2LN_text_rotation_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_horiz_dir_set", _wrap_Dwg_Entity_DIMENSION_ANG2LN_horiz_dir_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_horiz_dir_get", _wrap_Dwg_Entity_DIMENSION_ANG2LN_horiz_dir_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_ins_scale_set", _wrap_Dwg_Entity_DIMENSION_ANG2LN_ins_scale_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_ins_scale_get", _wrap_Dwg_Entity_DIMENSION_ANG2LN_ins_scale_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_ins_rotation_set", _wrap_Dwg_Entity_DIMENSION_ANG2LN_ins_rotation_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_ins_rotation_get", _wrap_Dwg_Entity_DIMENSION_ANG2LN_ins_rotation_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_attachment_set", _wrap_Dwg_Entity_DIMENSION_ANG2LN_attachment_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_attachment_get", _wrap_Dwg_Entity_DIMENSION_ANG2LN_attachment_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_lspace_style_set", _wrap_Dwg_Entity_DIMENSION_ANG2LN_lspace_style_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_lspace_style_get", _wrap_Dwg_Entity_DIMENSION_ANG2LN_lspace_style_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_lspace_factor_set", _wrap_Dwg_Entity_DIMENSION_ANG2LN_lspace_factor_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_lspace_factor_get", _wrap_Dwg_Entity_DIMENSION_ANG2LN_lspace_factor_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_act_measurement_set", _wrap_Dwg_Entity_DIMENSION_ANG2LN_act_measurement_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_act_measurement_get", _wrap_Dwg_Entity_DIMENSION_ANG2LN_act_measurement_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_unknown_set", _wrap_Dwg_Entity_DIMENSION_ANG2LN_unknown_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_unknown_get", _wrap_Dwg_Entity_DIMENSION_ANG2LN_unknown_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_flip_arrow1_set", _wrap_Dwg_Entity_DIMENSION_ANG2LN_flip_arrow1_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_flip_arrow1_get", _wrap_Dwg_Entity_DIMENSION_ANG2LN_flip_arrow1_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_flip_arrow2_set", _wrap_Dwg_Entity_DIMENSION_ANG2LN_flip_arrow2_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_flip_arrow2_get", _wrap_Dwg_Entity_DIMENSION_ANG2LN_flip_arrow2_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_clone_ins_pt_set", _wrap_Dwg_Entity_DIMENSION_ANG2LN_clone_ins_pt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_clone_ins_pt_get", _wrap_Dwg_Entity_DIMENSION_ANG2LN_clone_ins_pt_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_dimstyle_set", _wrap_Dwg_Entity_DIMENSION_ANG2LN_dimstyle_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_dimstyle_get", _wrap_Dwg_Entity_DIMENSION_ANG2LN_dimstyle_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_block_set", _wrap_Dwg_Entity_DIMENSION_ANG2LN_block_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_block_get", _wrap_Dwg_Entity_DIMENSION_ANG2LN_block_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_xline1start_pt_set", _wrap_Dwg_Entity_DIMENSION_ANG2LN_xline1start_pt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_xline1start_pt_get", _wrap_Dwg_Entity_DIMENSION_ANG2LN_xline1start_pt_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_xline1end_pt_set", _wrap_Dwg_Entity_DIMENSION_ANG2LN_xline1end_pt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_xline1end_pt_get", _wrap_Dwg_Entity_DIMENSION_ANG2LN_xline1end_pt_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_xline2start_pt_set", _wrap_Dwg_Entity_DIMENSION_ANG2LN_xline2start_pt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_xline2start_pt_get", _wrap_Dwg_Entity_DIMENSION_ANG2LN_xline2start_pt_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_xline2end_pt_set", _wrap_Dwg_Entity_DIMENSION_ANG2LN_xline2end_pt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_xline2end_pt_get", _wrap_Dwg_Entity_DIMENSION_ANG2LN_xline2end_pt_get},
{"LibreDWGc::new_Dwg_Entity_DIMENSION_ANG2LN", _wrap_new_Dwg_Entity_DIMENSION_ANG2LN},
{"LibreDWGc::delete_Dwg_Entity_DIMENSION_ANG2LN", _wrap_delete_Dwg_Entity_DIMENSION_ANG2LN},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_parent_set", _wrap_Dwg_Entity_DIMENSION_RADIUS_parent_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_parent_get", _wrap_Dwg_Entity_DIMENSION_RADIUS_parent_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_class_version_set", _wrap_Dwg_Entity_DIMENSION_RADIUS_class_version_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_class_version_get", _wrap_Dwg_Entity_DIMENSION_RADIUS_class_version_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_extrusion_set", _wrap_Dwg_Entity_DIMENSION_RADIUS_extrusion_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_extrusion_get", _wrap_Dwg_Entity_DIMENSION_RADIUS_extrusion_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_def_pt_set", _wrap_Dwg_Entity_DIMENSION_RADIUS_def_pt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_def_pt_get", _wrap_Dwg_Entity_DIMENSION_RADIUS_def_pt_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_text_midpt_set", _wrap_Dwg_Entity_DIMENSION_RADIUS_text_midpt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_text_midpt_get", _wrap_Dwg_Entity_DIMENSION_RADIUS_text_midpt_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_elevation_set", _wrap_Dwg_Entity_DIMENSION_RADIUS_elevation_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_elevation_get", _wrap_Dwg_Entity_DIMENSION_RADIUS_elevation_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_flag_set", _wrap_Dwg_Entity_DIMENSION_RADIUS_flag_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_flag_get", _wrap_Dwg_Entity_DIMENSION_RADIUS_flag_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_flag1_set", _wrap_Dwg_Entity_DIMENSION_RADIUS_flag1_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_flag1_get", _wrap_Dwg_Entity_DIMENSION_RADIUS_flag1_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_user_text_set", _wrap_Dwg_Entity_DIMENSION_RADIUS_user_text_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_user_text_get", _wrap_Dwg_Entity_DIMENSION_RADIUS_user_text_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_text_rotation_set", _wrap_Dwg_Entity_DIMENSION_RADIUS_text_rotation_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_text_rotation_get", _wrap_Dwg_Entity_DIMENSION_RADIUS_text_rotation_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_horiz_dir_set", _wrap_Dwg_Entity_DIMENSION_RADIUS_horiz_dir_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_horiz_dir_get", _wrap_Dwg_Entity_DIMENSION_RADIUS_horiz_dir_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_ins_scale_set", _wrap_Dwg_Entity_DIMENSION_RADIUS_ins_scale_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_ins_scale_get", _wrap_Dwg_Entity_DIMENSION_RADIUS_ins_scale_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_ins_rotation_set", _wrap_Dwg_Entity_DIMENSION_RADIUS_ins_rotation_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_ins_rotation_get", _wrap_Dwg_Entity_DIMENSION_RADIUS_ins_rotation_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_attachment_set", _wrap_Dwg_Entity_DIMENSION_RADIUS_attachment_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_attachment_get", _wrap_Dwg_Entity_DIMENSION_RADIUS_attachment_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_lspace_style_set", _wrap_Dwg_Entity_DIMENSION_RADIUS_lspace_style_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_lspace_style_get", _wrap_Dwg_Entity_DIMENSION_RADIUS_lspace_style_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_lspace_factor_set", _wrap_Dwg_Entity_DIMENSION_RADIUS_lspace_factor_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_lspace_factor_get", _wrap_Dwg_Entity_DIMENSION_RADIUS_lspace_factor_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_act_measurement_set", _wrap_Dwg_Entity_DIMENSION_RADIUS_act_measurement_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_act_measurement_get", _wrap_Dwg_Entity_DIMENSION_RADIUS_act_measurement_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_unknown_set", _wrap_Dwg_Entity_DIMENSION_RADIUS_unknown_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_unknown_get", _wrap_Dwg_Entity_DIMENSION_RADIUS_unknown_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_flip_arrow1_set", _wrap_Dwg_Entity_DIMENSION_RADIUS_flip_arrow1_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_flip_arrow1_get", _wrap_Dwg_Entity_DIMENSION_RADIUS_flip_arrow1_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_flip_arrow2_set", _wrap_Dwg_Entity_DIMENSION_RADIUS_flip_arrow2_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_flip_arrow2_get", _wrap_Dwg_Entity_DIMENSION_RADIUS_flip_arrow2_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_clone_ins_pt_set", _wrap_Dwg_Entity_DIMENSION_RADIUS_clone_ins_pt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_clone_ins_pt_get", _wrap_Dwg_Entity_DIMENSION_RADIUS_clone_ins_pt_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_dimstyle_set", _wrap_Dwg_Entity_DIMENSION_RADIUS_dimstyle_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_dimstyle_get", _wrap_Dwg_Entity_DIMENSION_RADIUS_dimstyle_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_block_set", _wrap_Dwg_Entity_DIMENSION_RADIUS_block_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_block_get", _wrap_Dwg_Entity_DIMENSION_RADIUS_block_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_first_arc_pt_set", _wrap_Dwg_Entity_DIMENSION_RADIUS_first_arc_pt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_first_arc_pt_get", _wrap_Dwg_Entity_DIMENSION_RADIUS_first_arc_pt_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_leader_len_set", _wrap_Dwg_Entity_DIMENSION_RADIUS_leader_len_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_leader_len_get", _wrap_Dwg_Entity_DIMENSION_RADIUS_leader_len_get},
{"LibreDWGc::new_Dwg_Entity_DIMENSION_RADIUS", _wrap_new_Dwg_Entity_DIMENSION_RADIUS},
{"LibreDWGc::delete_Dwg_Entity_DIMENSION_RADIUS", _wrap_delete_Dwg_Entity_DIMENSION_RADIUS},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_parent_set", _wrap_Dwg_Entity_DIMENSION_DIAMETER_parent_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_parent_get", _wrap_Dwg_Entity_DIMENSION_DIAMETER_parent_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_class_version_set", _wrap_Dwg_Entity_DIMENSION_DIAMETER_class_version_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_class_version_get", _wrap_Dwg_Entity_DIMENSION_DIAMETER_class_version_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_extrusion_set", _wrap_Dwg_Entity_DIMENSION_DIAMETER_extrusion_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_extrusion_get", _wrap_Dwg_Entity_DIMENSION_DIAMETER_extrusion_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_def_pt_set", _wrap_Dwg_Entity_DIMENSION_DIAMETER_def_pt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_def_pt_get", _wrap_Dwg_Entity_DIMENSION_DIAMETER_def_pt_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_text_midpt_set", _wrap_Dwg_Entity_DIMENSION_DIAMETER_text_midpt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_text_midpt_get", _wrap_Dwg_Entity_DIMENSION_DIAMETER_text_midpt_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_elevation_set", _wrap_Dwg_Entity_DIMENSION_DIAMETER_elevation_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_elevation_get", _wrap_Dwg_Entity_DIMENSION_DIAMETER_elevation_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_flag_set", _wrap_Dwg_Entity_DIMENSION_DIAMETER_flag_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_flag_get", _wrap_Dwg_Entity_DIMENSION_DIAMETER_flag_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_flag1_set", _wrap_Dwg_Entity_DIMENSION_DIAMETER_flag1_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_flag1_get", _wrap_Dwg_Entity_DIMENSION_DIAMETER_flag1_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_user_text_set", _wrap_Dwg_Entity_DIMENSION_DIAMETER_user_text_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_user_text_get", _wrap_Dwg_Entity_DIMENSION_DIAMETER_user_text_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_text_rotation_set", _wrap_Dwg_Entity_DIMENSION_DIAMETER_text_rotation_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_text_rotation_get", _wrap_Dwg_Entity_DIMENSION_DIAMETER_text_rotation_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_horiz_dir_set", _wrap_Dwg_Entity_DIMENSION_DIAMETER_horiz_dir_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_horiz_dir_get", _wrap_Dwg_Entity_DIMENSION_DIAMETER_horiz_dir_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_ins_scale_set", _wrap_Dwg_Entity_DIMENSION_DIAMETER_ins_scale_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_ins_scale_get", _wrap_Dwg_Entity_DIMENSION_DIAMETER_ins_scale_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_ins_rotation_set", _wrap_Dwg_Entity_DIMENSION_DIAMETER_ins_rotation_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_ins_rotation_get", _wrap_Dwg_Entity_DIMENSION_DIAMETER_ins_rotation_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_attachment_set", _wrap_Dwg_Entity_DIMENSION_DIAMETER_attachment_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_attachment_get", _wrap_Dwg_Entity_DIMENSION_DIAMETER_attachment_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_lspace_style_set", _wrap_Dwg_Entity_DIMENSION_DIAMETER_lspace_style_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_lspace_style_get", _wrap_Dwg_Entity_DIMENSION_DIAMETER_lspace_style_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_lspace_factor_set", _wrap_Dwg_Entity_DIMENSION_DIAMETER_lspace_factor_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_lspace_factor_get", _wrap_Dwg_Entity_DIMENSION_DIAMETER_lspace_factor_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_act_measurement_set", _wrap_Dwg_Entity_DIMENSION_DIAMETER_act_measurement_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_act_measurement_get", _wrap_Dwg_Entity_DIMENSION_DIAMETER_act_measurement_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_unknown_set", _wrap_Dwg_Entity_DIMENSION_DIAMETER_unknown_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_unknown_get", _wrap_Dwg_Entity_DIMENSION_DIAMETER_unknown_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_flip_arrow1_set", _wrap_Dwg_Entity_DIMENSION_DIAMETER_flip_arrow1_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_flip_arrow1_get", _wrap_Dwg_Entity_DIMENSION_DIAMETER_flip_arrow1_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_flip_arrow2_set", _wrap_Dwg_Entity_DIMENSION_DIAMETER_flip_arrow2_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_flip_arrow2_get", _wrap_Dwg_Entity_DIMENSION_DIAMETER_flip_arrow2_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_clone_ins_pt_set", _wrap_Dwg_Entity_DIMENSION_DIAMETER_clone_ins_pt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_clone_ins_pt_get", _wrap_Dwg_Entity_DIMENSION_DIAMETER_clone_ins_pt_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_dimstyle_set", _wrap_Dwg_Entity_DIMENSION_DIAMETER_dimstyle_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_dimstyle_get", _wrap_Dwg_Entity_DIMENSION_DIAMETER_dimstyle_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_block_set", _wrap_Dwg_Entity_DIMENSION_DIAMETER_block_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_block_get", _wrap_Dwg_Entity_DIMENSION_DIAMETER_block_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_first_arc_pt_set", _wrap_Dwg_Entity_DIMENSION_DIAMETER_first_arc_pt_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_first_arc_pt_get", _wrap_Dwg_Entity_DIMENSION_DIAMETER_first_arc_pt_get},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_leader_len_set", _wrap_Dwg_Entity_DIMENSION_DIAMETER_leader_len_set},
{"LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_leader_len_get", _wrap_Dwg_Entity_DIMENSION_DIAMETER_leader_len_get},
{"LibreDWGc::new_Dwg_Entity_DIMENSION_DIAMETER", _wrap_new_Dwg_Entity_DIMENSION_DIAMETER},
{"LibreDWGc::delete_Dwg_Entity_DIMENSION_DIAMETER", _wrap_delete_Dwg_Entity_DIMENSION_DIAMETER},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_parent_set", _wrap_Dwg_Entity_ARC_DIMENSION_parent_set},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_parent_get", _wrap_Dwg_Entity_ARC_DIMENSION_parent_get},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_class_version_set", _wrap_Dwg_Entity_ARC_DIMENSION_class_version_set},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_class_version_get", _wrap_Dwg_Entity_ARC_DIMENSION_class_version_get},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_extrusion_set", _wrap_Dwg_Entity_ARC_DIMENSION_extrusion_set},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_extrusion_get", _wrap_Dwg_Entity_ARC_DIMENSION_extrusion_get},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_def_pt_set", _wrap_Dwg_Entity_ARC_DIMENSION_def_pt_set},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_def_pt_get", _wrap_Dwg_Entity_ARC_DIMENSION_def_pt_get},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_text_midpt_set", _wrap_Dwg_Entity_ARC_DIMENSION_text_midpt_set},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_text_midpt_get", _wrap_Dwg_Entity_ARC_DIMENSION_text_midpt_get},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_elevation_set", _wrap_Dwg_Entity_ARC_DIMENSION_elevation_set},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_elevation_get", _wrap_Dwg_Entity_ARC_DIMENSION_elevation_get},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_flag_set", _wrap_Dwg_Entity_ARC_DIMENSION_flag_set},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_flag_get", _wrap_Dwg_Entity_ARC_DIMENSION_flag_get},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_flag1_set", _wrap_Dwg_Entity_ARC_DIMENSION_flag1_set},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_flag1_get", _wrap_Dwg_Entity_ARC_DIMENSION_flag1_get},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_user_text_set", _wrap_Dwg_Entity_ARC_DIMENSION_user_text_set},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_user_text_get", _wrap_Dwg_Entity_ARC_DIMENSION_user_text_get},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_text_rotation_set", _wrap_Dwg_Entity_ARC_DIMENSION_text_rotation_set},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_text_rotation_get", _wrap_Dwg_Entity_ARC_DIMENSION_text_rotation_get},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_horiz_dir_set", _wrap_Dwg_Entity_ARC_DIMENSION_horiz_dir_set},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_horiz_dir_get", _wrap_Dwg_Entity_ARC_DIMENSION_horiz_dir_get},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_ins_scale_set", _wrap_Dwg_Entity_ARC_DIMENSION_ins_scale_set},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_ins_scale_get", _wrap_Dwg_Entity_ARC_DIMENSION_ins_scale_get},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_ins_rotation_set", _wrap_Dwg_Entity_ARC_DIMENSION_ins_rotation_set},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_ins_rotation_get", _wrap_Dwg_Entity_ARC_DIMENSION_ins_rotation_get},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_attachment_set", _wrap_Dwg_Entity_ARC_DIMENSION_attachment_set},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_attachment_get", _wrap_Dwg_Entity_ARC_DIMENSION_attachment_get},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_lspace_style_set", _wrap_Dwg_Entity_ARC_DIMENSION_lspace_style_set},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_lspace_style_get", _wrap_Dwg_Entity_ARC_DIMENSION_lspace_style_get},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_lspace_factor_set", _wrap_Dwg_Entity_ARC_DIMENSION_lspace_factor_set},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_lspace_factor_get", _wrap_Dwg_Entity_ARC_DIMENSION_lspace_factor_get},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_act_measurement_set", _wrap_Dwg_Entity_ARC_DIMENSION_act_measurement_set},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_act_measurement_get", _wrap_Dwg_Entity_ARC_DIMENSION_act_measurement_get},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_unknown_set", _wrap_Dwg_Entity_ARC_DIMENSION_unknown_set},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_unknown_get", _wrap_Dwg_Entity_ARC_DIMENSION_unknown_get},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_flip_arrow1_set", _wrap_Dwg_Entity_ARC_DIMENSION_flip_arrow1_set},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_flip_arrow1_get", _wrap_Dwg_Entity_ARC_DIMENSION_flip_arrow1_get},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_flip_arrow2_set", _wrap_Dwg_Entity_ARC_DIMENSION_flip_arrow2_set},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_flip_arrow2_get", _wrap_Dwg_Entity_ARC_DIMENSION_flip_arrow2_get},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_clone_ins_pt_set", _wrap_Dwg_Entity_ARC_DIMENSION_clone_ins_pt_set},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_clone_ins_pt_get", _wrap_Dwg_Entity_ARC_DIMENSION_clone_ins_pt_get},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_dimstyle_set", _wrap_Dwg_Entity_ARC_DIMENSION_dimstyle_set},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_dimstyle_get", _wrap_Dwg_Entity_ARC_DIMENSION_dimstyle_get},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_block_set", _wrap_Dwg_Entity_ARC_DIMENSION_block_set},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_block_get", _wrap_Dwg_Entity_ARC_DIMENSION_block_get},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_xline1_pt_set", _wrap_Dwg_Entity_ARC_DIMENSION_xline1_pt_set},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_xline1_pt_get", _wrap_Dwg_Entity_ARC_DIMENSION_xline1_pt_get},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_xline2_pt_set", _wrap_Dwg_Entity_ARC_DIMENSION_xline2_pt_set},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_xline2_pt_get", _wrap_Dwg_Entity_ARC_DIMENSION_xline2_pt_get},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_center_pt_set", _wrap_Dwg_Entity_ARC_DIMENSION_center_pt_set},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_center_pt_get", _wrap_Dwg_Entity_ARC_DIMENSION_center_pt_get},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_is_partial_set", _wrap_Dwg_Entity_ARC_DIMENSION_is_partial_set},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_is_partial_get", _wrap_Dwg_Entity_ARC_DIMENSION_is_partial_get},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_arc_start_param_set", _wrap_Dwg_Entity_ARC_DIMENSION_arc_start_param_set},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_arc_start_param_get", _wrap_Dwg_Entity_ARC_DIMENSION_arc_start_param_get},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_arc_end_param_set", _wrap_Dwg_Entity_ARC_DIMENSION_arc_end_param_set},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_arc_end_param_get", _wrap_Dwg_Entity_ARC_DIMENSION_arc_end_param_get},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_has_leader_set", _wrap_Dwg_Entity_ARC_DIMENSION_has_leader_set},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_has_leader_get", _wrap_Dwg_Entity_ARC_DIMENSION_has_leader_get},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_leader1_pt_set", _wrap_Dwg_Entity_ARC_DIMENSION_leader1_pt_set},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_leader1_pt_get", _wrap_Dwg_Entity_ARC_DIMENSION_leader1_pt_get},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_leader2_pt_set", _wrap_Dwg_Entity_ARC_DIMENSION_leader2_pt_set},
{"LibreDWGc::Dwg_Entity_ARC_DIMENSION_leader2_pt_get", _wrap_Dwg_Entity_ARC_DIMENSION_leader2_pt_get},
{"LibreDWGc::new_Dwg_Entity_ARC_DIMENSION", _wrap_new_Dwg_Entity_ARC_DIMENSION},
{"LibreDWGc::delete_Dwg_Entity_ARC_DIMENSION", _wrap_delete_Dwg_Entity_ARC_DIMENSION},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_parent_set", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_parent_set},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_parent_get", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_parent_get},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_class_version_set", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_class_version_set},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_class_version_get", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_class_version_get},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_extrusion_set", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_extrusion_set},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_extrusion_get", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_extrusion_get},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_def_pt_set", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_def_pt_set},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_def_pt_get", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_def_pt_get},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_text_midpt_set", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_text_midpt_set},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_text_midpt_get", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_text_midpt_get},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_elevation_set", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_elevation_set},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_elevation_get", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_elevation_get},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_flag_set", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_flag_set},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_flag_get", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_flag_get},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_flag1_set", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_flag1_set},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_flag1_get", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_flag1_get},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_user_text_set", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_user_text_set},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_user_text_get", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_user_text_get},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_text_rotation_set", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_text_rotation_set},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_text_rotation_get", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_text_rotation_get},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_horiz_dir_set", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_horiz_dir_set},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_horiz_dir_get", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_horiz_dir_get},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_ins_scale_set", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_ins_scale_set},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_ins_scale_get", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_ins_scale_get},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_ins_rotation_set", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_ins_rotation_set},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_ins_rotation_get", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_ins_rotation_get},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_attachment_set", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_attachment_set},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_attachment_get", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_attachment_get},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_lspace_style_set", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_lspace_style_set},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_lspace_style_get", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_lspace_style_get},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_lspace_factor_set", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_lspace_factor_set},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_lspace_factor_get", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_lspace_factor_get},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_act_measurement_set", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_act_measurement_set},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_act_measurement_get", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_act_measurement_get},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_unknown_set", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_unknown_set},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_unknown_get", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_unknown_get},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_flip_arrow1_set", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_flip_arrow1_set},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_flip_arrow1_get", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_flip_arrow1_get},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_flip_arrow2_set", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_flip_arrow2_set},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_flip_arrow2_get", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_flip_arrow2_get},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_clone_ins_pt_set", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_clone_ins_pt_set},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_clone_ins_pt_get", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_clone_ins_pt_get},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_dimstyle_set", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_dimstyle_set},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_dimstyle_get", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_dimstyle_get},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_block_set", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_block_set},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_block_get", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_block_get},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_jog_angle_set", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_jog_angle_set},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_jog_angle_get", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_jog_angle_get},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_jog_pt_set", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_jog_pt_set},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_jog_pt_get", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_jog_pt_get},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_ovr_center_set", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_ovr_center_set},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_ovr_center_get", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_ovr_center_get},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_chord_pt_set", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_chord_pt_set},
{"LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_chord_pt_get", _wrap_Dwg_Entity_LARGE_RADIAL_DIMENSION_chord_pt_get},
{"LibreDWGc::new_Dwg_Entity_LARGE_RADIAL_DIMENSION", _wrap_new_Dwg_Entity_LARGE_RADIAL_DIMENSION},
{"LibreDWGc::delete_Dwg_Entity_LARGE_RADIAL_DIMENSION", _wrap_delete_Dwg_Entity_LARGE_RADIAL_DIMENSION},
{"LibreDWGc::Dwg_Entity_POINT_parent_set", _wrap_Dwg_Entity_POINT_parent_set},
{"LibreDWGc::Dwg_Entity_POINT_parent_get", _wrap_Dwg_Entity_POINT_parent_get},
{"LibreDWGc::Dwg_Entity_POINT_x_set", _wrap_Dwg_Entity_POINT_x_set},
{"LibreDWGc::Dwg_Entity_POINT_x_get", _wrap_Dwg_Entity_POINT_x_get},
{"LibreDWGc::Dwg_Entity_POINT_y_set", _wrap_Dwg_Entity_POINT_y_set},
{"LibreDWGc::Dwg_Entity_POINT_y_get", _wrap_Dwg_Entity_POINT_y_get},
{"LibreDWGc::Dwg_Entity_POINT_z_set", _wrap_Dwg_Entity_POINT_z_set},
{"LibreDWGc::Dwg_Entity_POINT_z_get", _wrap_Dwg_Entity_POINT_z_get},
{"LibreDWGc::Dwg_Entity_POINT_thickness_set", _wrap_Dwg_Entity_POINT_thickness_set},
{"LibreDWGc::Dwg_Entity_POINT_thickness_get", _wrap_Dwg_Entity_POINT_thickness_get},
{"LibreDWGc::Dwg_Entity_POINT_extrusion_set", _wrap_Dwg_Entity_POINT_extrusion_set},
{"LibreDWGc::Dwg_Entity_POINT_extrusion_get", _wrap_Dwg_Entity_POINT_extrusion_get},
{"LibreDWGc::Dwg_Entity_POINT_x_ang_set", _wrap_Dwg_Entity_POINT_x_ang_set},
{"LibreDWGc::Dwg_Entity_POINT_x_ang_get", _wrap_Dwg_Entity_POINT_x_ang_get},
{"LibreDWGc::new_Dwg_Entity_POINT", _wrap_new_Dwg_Entity_POINT},
{"LibreDWGc::delete_Dwg_Entity_POINT", _wrap_delete_Dwg_Entity_POINT},
{"LibreDWGc::Dwg_Entity__3DFACE_parent_set", _wrap_Dwg_Entity__3DFACE_parent_set},
{"LibreDWGc::Dwg_Entity__3DFACE_parent_get", _wrap_Dwg_Entity__3DFACE_parent_get},
{"LibreDWGc::Dwg_Entity__3DFACE_has_no_flags_set", _wrap_Dwg_Entity__3DFACE_has_no_flags_set},
{"LibreDWGc::Dwg_Entity__3DFACE_has_no_flags_get", _wrap_Dwg_Entity__3DFACE_has_no_flags_get},
{"LibreDWGc::Dwg_Entity__3DFACE_z_is_zero_set", _wrap_Dwg_Entity__3DFACE_z_is_zero_set},
{"LibreDWGc::Dwg_Entity__3DFACE_z_is_zero_get", _wrap_Dwg_Entity__3DFACE_z_is_zero_get},
{"LibreDWGc::Dwg_Entity__3DFACE_corner1_set", _wrap_Dwg_Entity__3DFACE_corner1_set},
{"LibreDWGc::Dwg_Entity__3DFACE_corner1_get", _wrap_Dwg_Entity__3DFACE_corner1_get},
{"LibreDWGc::Dwg_Entity__3DFACE_corner2_set", _wrap_Dwg_Entity__3DFACE_corner2_set},
{"LibreDWGc::Dwg_Entity__3DFACE_corner2_get", _wrap_Dwg_Entity__3DFACE_corner2_get},
{"LibreDWGc::Dwg_Entity__3DFACE_corner3_set", _wrap_Dwg_Entity__3DFACE_corner3_set},
{"LibreDWGc::Dwg_Entity__3DFACE_corner3_get", _wrap_Dwg_Entity__3DFACE_corner3_get},
{"LibreDWGc::Dwg_Entity__3DFACE_corner4_set", _wrap_Dwg_Entity__3DFACE_corner4_set},
{"LibreDWGc::Dwg_Entity__3DFACE_corner4_get", _wrap_Dwg_Entity__3DFACE_corner4_get},
{"LibreDWGc::Dwg_Entity__3DFACE_invis_flags_set", _wrap_Dwg_Entity__3DFACE_invis_flags_set},
{"LibreDWGc::Dwg_Entity__3DFACE_invis_flags_get", _wrap_Dwg_Entity__3DFACE_invis_flags_get},
{"LibreDWGc::new_Dwg_Entity__3DFACE", _wrap_new_Dwg_Entity__3DFACE},
{"LibreDWGc::delete_Dwg_Entity__3DFACE", _wrap_delete_Dwg_Entity__3DFACE},
{"LibreDWGc::Dwg_Entity_POLYLINE_PFACE_parent_set", _wrap_Dwg_Entity_POLYLINE_PFACE_parent_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_PFACE_parent_get", _wrap_Dwg_Entity_POLYLINE_PFACE_parent_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_PFACE_has_vertex_set", _wrap_Dwg_Entity_POLYLINE_PFACE_has_vertex_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_PFACE_has_vertex_get", _wrap_Dwg_Entity_POLYLINE_PFACE_has_vertex_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_PFACE_num_owned_set", _wrap_Dwg_Entity_POLYLINE_PFACE_num_owned_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_PFACE_num_owned_get", _wrap_Dwg_Entity_POLYLINE_PFACE_num_owned_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_PFACE_first_vertex_set", _wrap_Dwg_Entity_POLYLINE_PFACE_first_vertex_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_PFACE_first_vertex_get", _wrap_Dwg_Entity_POLYLINE_PFACE_first_vertex_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_PFACE_last_vertex_set", _wrap_Dwg_Entity_POLYLINE_PFACE_last_vertex_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_PFACE_last_vertex_get", _wrap_Dwg_Entity_POLYLINE_PFACE_last_vertex_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_PFACE_vertex_set", _wrap_Dwg_Entity_POLYLINE_PFACE_vertex_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_PFACE_vertex_get", _wrap_Dwg_Entity_POLYLINE_PFACE_vertex_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_PFACE_seqend_set", _wrap_Dwg_Entity_POLYLINE_PFACE_seqend_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_PFACE_seqend_get", _wrap_Dwg_Entity_POLYLINE_PFACE_seqend_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_PFACE_flag_set", _wrap_Dwg_Entity_POLYLINE_PFACE_flag_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_PFACE_flag_get", _wrap_Dwg_Entity_POLYLINE_PFACE_flag_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_PFACE_numverts_set", _wrap_Dwg_Entity_POLYLINE_PFACE_numverts_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_PFACE_numverts_get", _wrap_Dwg_Entity_POLYLINE_PFACE_numverts_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_PFACE_numfaces_set", _wrap_Dwg_Entity_POLYLINE_PFACE_numfaces_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_PFACE_numfaces_get", _wrap_Dwg_Entity_POLYLINE_PFACE_numfaces_get},
{"LibreDWGc::new_Dwg_Entity_POLYLINE_PFACE", _wrap_new_Dwg_Entity_POLYLINE_PFACE},
{"LibreDWGc::delete_Dwg_Entity_POLYLINE_PFACE", _wrap_delete_Dwg_Entity_POLYLINE_PFACE},
{"LibreDWGc::Dwg_Entity_POLYLINE_MESH_parent_set", _wrap_Dwg_Entity_POLYLINE_MESH_parent_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_MESH_parent_get", _wrap_Dwg_Entity_POLYLINE_MESH_parent_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_MESH_has_vertex_set", _wrap_Dwg_Entity_POLYLINE_MESH_has_vertex_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_MESH_has_vertex_get", _wrap_Dwg_Entity_POLYLINE_MESH_has_vertex_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_MESH_num_owned_set", _wrap_Dwg_Entity_POLYLINE_MESH_num_owned_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_MESH_num_owned_get", _wrap_Dwg_Entity_POLYLINE_MESH_num_owned_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_MESH_first_vertex_set", _wrap_Dwg_Entity_POLYLINE_MESH_first_vertex_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_MESH_first_vertex_get", _wrap_Dwg_Entity_POLYLINE_MESH_first_vertex_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_MESH_last_vertex_set", _wrap_Dwg_Entity_POLYLINE_MESH_last_vertex_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_MESH_last_vertex_get", _wrap_Dwg_Entity_POLYLINE_MESH_last_vertex_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_MESH_vertex_set", _wrap_Dwg_Entity_POLYLINE_MESH_vertex_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_MESH_vertex_get", _wrap_Dwg_Entity_POLYLINE_MESH_vertex_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_MESH_seqend_set", _wrap_Dwg_Entity_POLYLINE_MESH_seqend_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_MESH_seqend_get", _wrap_Dwg_Entity_POLYLINE_MESH_seqend_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_MESH_flag_set", _wrap_Dwg_Entity_POLYLINE_MESH_flag_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_MESH_flag_get", _wrap_Dwg_Entity_POLYLINE_MESH_flag_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_MESH_curve_type_set", _wrap_Dwg_Entity_POLYLINE_MESH_curve_type_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_MESH_curve_type_get", _wrap_Dwg_Entity_POLYLINE_MESH_curve_type_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_MESH_num_m_verts_set", _wrap_Dwg_Entity_POLYLINE_MESH_num_m_verts_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_MESH_num_m_verts_get", _wrap_Dwg_Entity_POLYLINE_MESH_num_m_verts_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_MESH_num_n_verts_set", _wrap_Dwg_Entity_POLYLINE_MESH_num_n_verts_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_MESH_num_n_verts_get", _wrap_Dwg_Entity_POLYLINE_MESH_num_n_verts_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_MESH_m_density_set", _wrap_Dwg_Entity_POLYLINE_MESH_m_density_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_MESH_m_density_get", _wrap_Dwg_Entity_POLYLINE_MESH_m_density_get},
{"LibreDWGc::Dwg_Entity_POLYLINE_MESH_n_density_set", _wrap_Dwg_Entity_POLYLINE_MESH_n_density_set},
{"LibreDWGc::Dwg_Entity_POLYLINE_MESH_n_density_get", _wrap_Dwg_Entity_POLYLINE_MESH_n_density_get},
{"LibreDWGc::new_Dwg_Entity_POLYLINE_MESH", _wrap_new_Dwg_Entity_POLYLINE_MESH},
{"LibreDWGc::delete_Dwg_Entity_POLYLINE_MESH", _wrap_delete_Dwg_Entity_POLYLINE_MESH},
{"LibreDWGc::Dwg_Entity_SOLID_parent_set", _wrap_Dwg_Entity_SOLID_parent_set},
{"LibreDWGc::Dwg_Entity_SOLID_parent_get", _wrap_Dwg_Entity_SOLID_parent_get},
{"LibreDWGc::Dwg_Entity_SOLID_thickness_set", _wrap_Dwg_Entity_SOLID_thickness_set},
{"LibreDWGc::Dwg_Entity_SOLID_thickness_get", _wrap_Dwg_Entity_SOLID_thickness_get},
{"LibreDWGc::Dwg_Entity_SOLID_elevation_set", _wrap_Dwg_Entity_SOLID_elevation_set},
{"LibreDWGc::Dwg_Entity_SOLID_elevation_get", _wrap_Dwg_Entity_SOLID_elevation_get},
{"LibreDWGc::Dwg_Entity_SOLID_corner1_set", _wrap_Dwg_Entity_SOLID_corner1_set},
{"LibreDWGc::Dwg_Entity_SOLID_corner1_get", _wrap_Dwg_Entity_SOLID_corner1_get},
{"LibreDWGc::Dwg_Entity_SOLID_corner2_set", _wrap_Dwg_Entity_SOLID_corner2_set},
{"LibreDWGc::Dwg_Entity_SOLID_corner2_get", _wrap_Dwg_Entity_SOLID_corner2_get},
{"LibreDWGc::Dwg_Entity_SOLID_corner3_set", _wrap_Dwg_Entity_SOLID_corner3_set},
{"LibreDWGc::Dwg_Entity_SOLID_corner3_get", _wrap_Dwg_Entity_SOLID_corner3_get},
{"LibreDWGc::Dwg_Entity_SOLID_corner4_set", _wrap_Dwg_Entity_SOLID_corner4_set},
{"LibreDWGc::Dwg_Entity_SOLID_corner4_get", _wrap_Dwg_Entity_SOLID_corner4_get},
{"LibreDWGc::Dwg_Entity_SOLID_extrusion_set", _wrap_Dwg_Entity_SOLID_extrusion_set},
{"LibreDWGc::Dwg_Entity_SOLID_extrusion_get", _wrap_Dwg_Entity_SOLID_extrusion_get},
{"LibreDWGc::new_Dwg_Entity_SOLID", _wrap_new_Dwg_Entity_SOLID},
{"LibreDWGc::delete_Dwg_Entity_SOLID", _wrap_delete_Dwg_Entity_SOLID},
{"LibreDWGc::Dwg_Entity_TRACE_parent_set", _wrap_Dwg_Entity_TRACE_parent_set},
{"LibreDWGc::Dwg_Entity_TRACE_parent_get", _wrap_Dwg_Entity_TRACE_parent_get},
{"LibreDWGc::Dwg_Entity_TRACE_thickness_set", _wrap_Dwg_Entity_TRACE_thickness_set},
{"LibreDWGc::Dwg_Entity_TRACE_thickness_get", _wrap_Dwg_Entity_TRACE_thickness_get},
{"LibreDWGc::Dwg_Entity_TRACE_elevation_set", _wrap_Dwg_Entity_TRACE_elevation_set},
{"LibreDWGc::Dwg_Entity_TRACE_elevation_get", _wrap_Dwg_Entity_TRACE_elevation_get},
{"LibreDWGc::Dwg_Entity_TRACE_corner1_set", _wrap_Dwg_Entity_TRACE_corner1_set},
{"LibreDWGc::Dwg_Entity_TRACE_corner1_get", _wrap_Dwg_Entity_TRACE_corner1_get},
{"LibreDWGc::Dwg_Entity_TRACE_corner2_set", _wrap_Dwg_Entity_TRACE_corner2_set},
{"LibreDWGc::Dwg_Entity_TRACE_corner2_get", _wrap_Dwg_Entity_TRACE_corner2_get},
{"LibreDWGc::Dwg_Entity_TRACE_corner3_set", _wrap_Dwg_Entity_TRACE_corner3_set},
{"LibreDWGc::Dwg_Entity_TRACE_corner3_get", _wrap_Dwg_Entity_TRACE_corner3_get},
{"LibreDWGc::Dwg_Entity_TRACE_corner4_set", _wrap_Dwg_Entity_TRACE_corner4_set},
{"LibreDWGc::Dwg_Entity_TRACE_corner4_get", _wrap_Dwg_Entity_TRACE_corner4_get},
{"LibreDWGc::Dwg_Entity_TRACE_extrusion_set", _wrap_Dwg_Entity_TRACE_extrusion_set},
{"LibreDWGc::Dwg_Entity_TRACE_extrusion_get", _wrap_Dwg_Entity_TRACE_extrusion_get},
{"LibreDWGc::new_Dwg_Entity_TRACE", _wrap_new_Dwg_Entity_TRACE},
{"LibreDWGc::delete_Dwg_Entity_TRACE", _wrap_delete_Dwg_Entity_TRACE},
{"LibreDWGc::Dwg_Entity_SHAPE_parent_set", _wrap_Dwg_Entity_SHAPE_parent_set},
{"LibreDWGc::Dwg_Entity_SHAPE_parent_get", _wrap_Dwg_Entity_SHAPE_parent_get},
{"LibreDWGc::Dwg_Entity_SHAPE_ins_pt_set", _wrap_Dwg_Entity_SHAPE_ins_pt_set},
{"LibreDWGc::Dwg_Entity_SHAPE_ins_pt_get", _wrap_Dwg_Entity_SHAPE_ins_pt_get},
{"LibreDWGc::Dwg_Entity_SHAPE_scale_set", _wrap_Dwg_Entity_SHAPE_scale_set},
{"LibreDWGc::Dwg_Entity_SHAPE_scale_get", _wrap_Dwg_Entity_SHAPE_scale_get},
{"LibreDWGc::Dwg_Entity_SHAPE_rotation_set", _wrap_Dwg_Entity_SHAPE_rotation_set},
{"LibreDWGc::Dwg_Entity_SHAPE_rotation_get", _wrap_Dwg_Entity_SHAPE_rotation_get},
{"LibreDWGc::Dwg_Entity_SHAPE_width_factor_set", _wrap_Dwg_Entity_SHAPE_width_factor_set},
{"LibreDWGc::Dwg_Entity_SHAPE_width_factor_get", _wrap_Dwg_Entity_SHAPE_width_factor_get},
{"LibreDWGc::Dwg_Entity_SHAPE_oblique_angle_set", _wrap_Dwg_Entity_SHAPE_oblique_angle_set},
{"LibreDWGc::Dwg_Entity_SHAPE_oblique_angle_get", _wrap_Dwg_Entity_SHAPE_oblique_angle_get},
{"LibreDWGc::Dwg_Entity_SHAPE_thickness_set", _wrap_Dwg_Entity_SHAPE_thickness_set},
{"LibreDWGc::Dwg_Entity_SHAPE_thickness_get", _wrap_Dwg_Entity_SHAPE_thickness_get},
{"LibreDWGc::Dwg_Entity_SHAPE_style_id_set", _wrap_Dwg_Entity_SHAPE_style_id_set},
{"LibreDWGc::Dwg_Entity_SHAPE_style_id_get", _wrap_Dwg_Entity_SHAPE_style_id_get},
{"LibreDWGc::Dwg_Entity_SHAPE_extrusion_set", _wrap_Dwg_Entity_SHAPE_extrusion_set},
{"LibreDWGc::Dwg_Entity_SHAPE_extrusion_get", _wrap_Dwg_Entity_SHAPE_extrusion_get},
{"LibreDWGc::Dwg_Entity_SHAPE_style_set", _wrap_Dwg_Entity_SHAPE_style_set},
{"LibreDWGc::Dwg_Entity_SHAPE_style_get", _wrap_Dwg_Entity_SHAPE_style_get},
{"LibreDWGc::new_Dwg_Entity_SHAPE", _wrap_new_Dwg_Entity_SHAPE},
{"LibreDWGc::delete_Dwg_Entity_SHAPE", _wrap_delete_Dwg_Entity_SHAPE},
{"LibreDWGc::Dwg_Entity_VIEWPORT_parent_set", _wrap_Dwg_Entity_VIEWPORT_parent_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_parent_get", _wrap_Dwg_Entity_VIEWPORT_parent_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_center_set", _wrap_Dwg_Entity_VIEWPORT_center_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_center_get", _wrap_Dwg_Entity_VIEWPORT_center_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_width_set", _wrap_Dwg_Entity_VIEWPORT_width_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_width_get", _wrap_Dwg_Entity_VIEWPORT_width_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_height_set", _wrap_Dwg_Entity_VIEWPORT_height_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_height_get", _wrap_Dwg_Entity_VIEWPORT_height_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_on_off_set", _wrap_Dwg_Entity_VIEWPORT_on_off_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_on_off_get", _wrap_Dwg_Entity_VIEWPORT_on_off_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_id_set", _wrap_Dwg_Entity_VIEWPORT_id_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_id_get", _wrap_Dwg_Entity_VIEWPORT_id_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_view_target_set", _wrap_Dwg_Entity_VIEWPORT_view_target_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_view_target_get", _wrap_Dwg_Entity_VIEWPORT_view_target_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_VIEWDIR_set", _wrap_Dwg_Entity_VIEWPORT_VIEWDIR_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_VIEWDIR_get", _wrap_Dwg_Entity_VIEWPORT_VIEWDIR_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_twist_angle_set", _wrap_Dwg_Entity_VIEWPORT_twist_angle_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_twist_angle_get", _wrap_Dwg_Entity_VIEWPORT_twist_angle_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_VIEWSIZE_set", _wrap_Dwg_Entity_VIEWPORT_VIEWSIZE_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_VIEWSIZE_get", _wrap_Dwg_Entity_VIEWPORT_VIEWSIZE_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_lens_length_set", _wrap_Dwg_Entity_VIEWPORT_lens_length_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_lens_length_get", _wrap_Dwg_Entity_VIEWPORT_lens_length_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_front_clip_z_set", _wrap_Dwg_Entity_VIEWPORT_front_clip_z_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_front_clip_z_get", _wrap_Dwg_Entity_VIEWPORT_front_clip_z_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_back_clip_z_set", _wrap_Dwg_Entity_VIEWPORT_back_clip_z_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_back_clip_z_get", _wrap_Dwg_Entity_VIEWPORT_back_clip_z_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_SNAPANG_set", _wrap_Dwg_Entity_VIEWPORT_SNAPANG_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_SNAPANG_get", _wrap_Dwg_Entity_VIEWPORT_SNAPANG_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_VIEWCTR_set", _wrap_Dwg_Entity_VIEWPORT_VIEWCTR_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_VIEWCTR_get", _wrap_Dwg_Entity_VIEWPORT_VIEWCTR_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_SNAPBASE_set", _wrap_Dwg_Entity_VIEWPORT_SNAPBASE_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_SNAPBASE_get", _wrap_Dwg_Entity_VIEWPORT_SNAPBASE_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_SNAPUNIT_set", _wrap_Dwg_Entity_VIEWPORT_SNAPUNIT_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_SNAPUNIT_get", _wrap_Dwg_Entity_VIEWPORT_SNAPUNIT_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_GRIDUNIT_set", _wrap_Dwg_Entity_VIEWPORT_GRIDUNIT_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_GRIDUNIT_get", _wrap_Dwg_Entity_VIEWPORT_GRIDUNIT_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_circle_zoom_set", _wrap_Dwg_Entity_VIEWPORT_circle_zoom_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_circle_zoom_get", _wrap_Dwg_Entity_VIEWPORT_circle_zoom_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_grid_major_set", _wrap_Dwg_Entity_VIEWPORT_grid_major_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_grid_major_get", _wrap_Dwg_Entity_VIEWPORT_grid_major_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_num_frozen_layers_set", _wrap_Dwg_Entity_VIEWPORT_num_frozen_layers_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_num_frozen_layers_get", _wrap_Dwg_Entity_VIEWPORT_num_frozen_layers_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_status_flag_set", _wrap_Dwg_Entity_VIEWPORT_status_flag_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_status_flag_get", _wrap_Dwg_Entity_VIEWPORT_status_flag_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_style_sheet_set", _wrap_Dwg_Entity_VIEWPORT_style_sheet_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_style_sheet_get", _wrap_Dwg_Entity_VIEWPORT_style_sheet_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_render_mode_set", _wrap_Dwg_Entity_VIEWPORT_render_mode_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_render_mode_get", _wrap_Dwg_Entity_VIEWPORT_render_mode_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_ucs_at_origin_set", _wrap_Dwg_Entity_VIEWPORT_ucs_at_origin_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_ucs_at_origin_get", _wrap_Dwg_Entity_VIEWPORT_ucs_at_origin_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_UCSVP_set", _wrap_Dwg_Entity_VIEWPORT_UCSVP_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_UCSVP_get", _wrap_Dwg_Entity_VIEWPORT_UCSVP_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_ucsorg_set", _wrap_Dwg_Entity_VIEWPORT_ucsorg_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_ucsorg_get", _wrap_Dwg_Entity_VIEWPORT_ucsorg_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_ucsxdir_set", _wrap_Dwg_Entity_VIEWPORT_ucsxdir_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_ucsxdir_get", _wrap_Dwg_Entity_VIEWPORT_ucsxdir_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_ucsydir_set", _wrap_Dwg_Entity_VIEWPORT_ucsydir_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_ucsydir_get", _wrap_Dwg_Entity_VIEWPORT_ucsydir_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_ucs_elevation_set", _wrap_Dwg_Entity_VIEWPORT_ucs_elevation_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_ucs_elevation_get", _wrap_Dwg_Entity_VIEWPORT_ucs_elevation_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_UCSORTHOVIEW_set", _wrap_Dwg_Entity_VIEWPORT_UCSORTHOVIEW_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_UCSORTHOVIEW_get", _wrap_Dwg_Entity_VIEWPORT_UCSORTHOVIEW_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_shadeplot_mode_set", _wrap_Dwg_Entity_VIEWPORT_shadeplot_mode_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_shadeplot_mode_get", _wrap_Dwg_Entity_VIEWPORT_shadeplot_mode_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_use_default_lights_set", _wrap_Dwg_Entity_VIEWPORT_use_default_lights_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_use_default_lights_get", _wrap_Dwg_Entity_VIEWPORT_use_default_lights_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_default_lighting_type_set", _wrap_Dwg_Entity_VIEWPORT_default_lighting_type_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_default_lighting_type_get", _wrap_Dwg_Entity_VIEWPORT_default_lighting_type_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_brightness_set", _wrap_Dwg_Entity_VIEWPORT_brightness_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_brightness_get", _wrap_Dwg_Entity_VIEWPORT_brightness_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_contrast_set", _wrap_Dwg_Entity_VIEWPORT_contrast_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_contrast_get", _wrap_Dwg_Entity_VIEWPORT_contrast_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_ambient_color_set", _wrap_Dwg_Entity_VIEWPORT_ambient_color_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_ambient_color_get", _wrap_Dwg_Entity_VIEWPORT_ambient_color_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_vport_entity_header_set", _wrap_Dwg_Entity_VIEWPORT_vport_entity_header_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_vport_entity_header_get", _wrap_Dwg_Entity_VIEWPORT_vport_entity_header_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_frozen_layers_set", _wrap_Dwg_Entity_VIEWPORT_frozen_layers_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_frozen_layers_get", _wrap_Dwg_Entity_VIEWPORT_frozen_layers_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_clip_boundary_set", _wrap_Dwg_Entity_VIEWPORT_clip_boundary_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_clip_boundary_get", _wrap_Dwg_Entity_VIEWPORT_clip_boundary_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_named_ucs_set", _wrap_Dwg_Entity_VIEWPORT_named_ucs_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_named_ucs_get", _wrap_Dwg_Entity_VIEWPORT_named_ucs_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_base_ucs_set", _wrap_Dwg_Entity_VIEWPORT_base_ucs_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_base_ucs_get", _wrap_Dwg_Entity_VIEWPORT_base_ucs_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_background_set", _wrap_Dwg_Entity_VIEWPORT_background_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_background_get", _wrap_Dwg_Entity_VIEWPORT_background_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_visualstyle_set", _wrap_Dwg_Entity_VIEWPORT_visualstyle_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_visualstyle_get", _wrap_Dwg_Entity_VIEWPORT_visualstyle_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_shadeplot_set", _wrap_Dwg_Entity_VIEWPORT_shadeplot_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_shadeplot_get", _wrap_Dwg_Entity_VIEWPORT_shadeplot_get},
{"LibreDWGc::Dwg_Entity_VIEWPORT_sun_set", _wrap_Dwg_Entity_VIEWPORT_sun_set},
{"LibreDWGc::Dwg_Entity_VIEWPORT_sun_get", _wrap_Dwg_Entity_VIEWPORT_sun_get},
{"LibreDWGc::new_Dwg_Entity_VIEWPORT", _wrap_new_Dwg_Entity_VIEWPORT},
{"LibreDWGc::delete_Dwg_Entity_VIEWPORT", _wrap_delete_Dwg_Entity_VIEWPORT},
{"LibreDWGc::Dwg_Entity_ELLIPSE_parent_set", _wrap_Dwg_Entity_ELLIPSE_parent_set},
{"LibreDWGc::Dwg_Entity_ELLIPSE_parent_get", _wrap_Dwg_Entity_ELLIPSE_parent_get},
{"LibreDWGc::Dwg_Entity_ELLIPSE_center_set", _wrap_Dwg_Entity_ELLIPSE_center_set},
{"LibreDWGc::Dwg_Entity_ELLIPSE_center_get", _wrap_Dwg_Entity_ELLIPSE_center_get},
{"LibreDWGc::Dwg_Entity_ELLIPSE_sm_axis_set", _wrap_Dwg_Entity_ELLIPSE_sm_axis_set},
{"LibreDWGc::Dwg_Entity_ELLIPSE_sm_axis_get", _wrap_Dwg_Entity_ELLIPSE_sm_axis_get},
{"LibreDWGc::Dwg_Entity_ELLIPSE_extrusion_set", _wrap_Dwg_Entity_ELLIPSE_extrusion_set},
{"LibreDWGc::Dwg_Entity_ELLIPSE_extrusion_get", _wrap_Dwg_Entity_ELLIPSE_extrusion_get},
{"LibreDWGc::Dwg_Entity_ELLIPSE_axis_ratio_set", _wrap_Dwg_Entity_ELLIPSE_axis_ratio_set},
{"LibreDWGc::Dwg_Entity_ELLIPSE_axis_ratio_get", _wrap_Dwg_Entity_ELLIPSE_axis_ratio_get},
{"LibreDWGc::Dwg_Entity_ELLIPSE_start_angle_set", _wrap_Dwg_Entity_ELLIPSE_start_angle_set},
{"LibreDWGc::Dwg_Entity_ELLIPSE_start_angle_get", _wrap_Dwg_Entity_ELLIPSE_start_angle_get},
{"LibreDWGc::Dwg_Entity_ELLIPSE_end_angle_set", _wrap_Dwg_Entity_ELLIPSE_end_angle_set},
{"LibreDWGc::Dwg_Entity_ELLIPSE_end_angle_get", _wrap_Dwg_Entity_ELLIPSE_end_angle_get},
{"LibreDWGc::new_Dwg_Entity_ELLIPSE", _wrap_new_Dwg_Entity_ELLIPSE},
{"LibreDWGc::delete_Dwg_Entity_ELLIPSE", _wrap_delete_Dwg_Entity_ELLIPSE},
{"LibreDWGc::Dwg_SPLINE_control_point_parent_set", _wrap_Dwg_SPLINE_control_point_parent_set},
{"LibreDWGc::Dwg_SPLINE_control_point_parent_get", _wrap_Dwg_SPLINE_control_point_parent_get},
{"LibreDWGc::Dwg_SPLINE_control_point_x_set", _wrap_Dwg_SPLINE_control_point_x_set},
{"LibreDWGc::Dwg_SPLINE_control_point_x_get", _wrap_Dwg_SPLINE_control_point_x_get},
{"LibreDWGc::Dwg_SPLINE_control_point_y_set", _wrap_Dwg_SPLINE_control_point_y_set},
{"LibreDWGc::Dwg_SPLINE_control_point_y_get", _wrap_Dwg_SPLINE_control_point_y_get},
{"LibreDWGc::Dwg_SPLINE_control_point_z_set", _wrap_Dwg_SPLINE_control_point_z_set},
{"LibreDWGc::Dwg_SPLINE_control_point_z_get", _wrap_Dwg_SPLINE_control_point_z_get},
{"LibreDWGc::Dwg_SPLINE_control_point_w_set", _wrap_Dwg_SPLINE_control_point_w_set},
{"LibreDWGc::Dwg_SPLINE_control_point_w_get", _wrap_Dwg_SPLINE_control_point_w_get},
{"LibreDWGc::new_Dwg_SPLINE_control_point", _wrap_new_Dwg_SPLINE_control_point},
{"LibreDWGc::delete_Dwg_SPLINE_control_point", _wrap_delete_Dwg_SPLINE_control_point},
{"LibreDWGc::Dwg_Entity_SPLINE_parent_set", _wrap_Dwg_Entity_SPLINE_parent_set},
{"LibreDWGc::Dwg_Entity_SPLINE_parent_get", _wrap_Dwg_Entity_SPLINE_parent_get},
{"LibreDWGc::Dwg_Entity_SPLINE_flag_set", _wrap_Dwg_Entity_SPLINE_flag_set},
{"LibreDWGc::Dwg_Entity_SPLINE_flag_get", _wrap_Dwg_Entity_SPLINE_flag_get},
{"LibreDWGc::Dwg_Entity_SPLINE_scenario_set", _wrap_Dwg_Entity_SPLINE_scenario_set},
{"LibreDWGc::Dwg_Entity_SPLINE_scenario_get", _wrap_Dwg_Entity_SPLINE_scenario_get},
{"LibreDWGc::Dwg_Entity_SPLINE_degree_set", _wrap_Dwg_Entity_SPLINE_degree_set},
{"LibreDWGc::Dwg_Entity_SPLINE_degree_get", _wrap_Dwg_Entity_SPLINE_degree_get},
{"LibreDWGc::Dwg_Entity_SPLINE_splineflags_set", _wrap_Dwg_Entity_SPLINE_splineflags_set},
{"LibreDWGc::Dwg_Entity_SPLINE_splineflags_get", _wrap_Dwg_Entity_SPLINE_splineflags_get},
{"LibreDWGc::Dwg_Entity_SPLINE_knotparam_set", _wrap_Dwg_Entity_SPLINE_knotparam_set},
{"LibreDWGc::Dwg_Entity_SPLINE_knotparam_get", _wrap_Dwg_Entity_SPLINE_knotparam_get},
{"LibreDWGc::Dwg_Entity_SPLINE_fit_tol_set", _wrap_Dwg_Entity_SPLINE_fit_tol_set},
{"LibreDWGc::Dwg_Entity_SPLINE_fit_tol_get", _wrap_Dwg_Entity_SPLINE_fit_tol_get},
{"LibreDWGc::Dwg_Entity_SPLINE_beg_tan_vec_set", _wrap_Dwg_Entity_SPLINE_beg_tan_vec_set},
{"LibreDWGc::Dwg_Entity_SPLINE_beg_tan_vec_get", _wrap_Dwg_Entity_SPLINE_beg_tan_vec_get},
{"LibreDWGc::Dwg_Entity_SPLINE_end_tan_vec_set", _wrap_Dwg_Entity_SPLINE_end_tan_vec_set},
{"LibreDWGc::Dwg_Entity_SPLINE_end_tan_vec_get", _wrap_Dwg_Entity_SPLINE_end_tan_vec_get},
{"LibreDWGc::Dwg_Entity_SPLINE_closed_b_set", _wrap_Dwg_Entity_SPLINE_closed_b_set},
{"LibreDWGc::Dwg_Entity_SPLINE_closed_b_get", _wrap_Dwg_Entity_SPLINE_closed_b_get},
{"LibreDWGc::Dwg_Entity_SPLINE_periodic_set", _wrap_Dwg_Entity_SPLINE_periodic_set},
{"LibreDWGc::Dwg_Entity_SPLINE_periodic_get", _wrap_Dwg_Entity_SPLINE_periodic_get},
{"LibreDWGc::Dwg_Entity_SPLINE_rational_set", _wrap_Dwg_Entity_SPLINE_rational_set},
{"LibreDWGc::Dwg_Entity_SPLINE_rational_get", _wrap_Dwg_Entity_SPLINE_rational_get},
{"LibreDWGc::Dwg_Entity_SPLINE_weighted_set", _wrap_Dwg_Entity_SPLINE_weighted_set},
{"LibreDWGc::Dwg_Entity_SPLINE_weighted_get", _wrap_Dwg_Entity_SPLINE_weighted_get},
{"LibreDWGc::Dwg_Entity_SPLINE_knot_tol_set", _wrap_Dwg_Entity_SPLINE_knot_tol_set},
{"LibreDWGc::Dwg_Entity_SPLINE_knot_tol_get", _wrap_Dwg_Entity_SPLINE_knot_tol_get},
{"LibreDWGc::Dwg_Entity_SPLINE_ctrl_tol_set", _wrap_Dwg_Entity_SPLINE_ctrl_tol_set},
{"LibreDWGc::Dwg_Entity_SPLINE_ctrl_tol_get", _wrap_Dwg_Entity_SPLINE_ctrl_tol_get},
{"LibreDWGc::Dwg_Entity_SPLINE_num_fit_pts_set", _wrap_Dwg_Entity_SPLINE_num_fit_pts_set},
{"LibreDWGc::Dwg_Entity_SPLINE_num_fit_pts_get", _wrap_Dwg_Entity_SPLINE_num_fit_pts_get},
{"LibreDWGc::Dwg_Entity_SPLINE_fit_pts_set", _wrap_Dwg_Entity_SPLINE_fit_pts_set},
{"LibreDWGc::Dwg_Entity_SPLINE_fit_pts_get", _wrap_Dwg_Entity_SPLINE_fit_pts_get},
{"LibreDWGc::Dwg_Entity_SPLINE_num_knots_set", _wrap_Dwg_Entity_SPLINE_num_knots_set},
{"LibreDWGc::Dwg_Entity_SPLINE_num_knots_get", _wrap_Dwg_Entity_SPLINE_num_knots_get},
{"LibreDWGc::Dwg_Entity_SPLINE_knots_set", _wrap_Dwg_Entity_SPLINE_knots_set},
{"LibreDWGc::Dwg_Entity_SPLINE_knots_get", _wrap_Dwg_Entity_SPLINE_knots_get},
{"LibreDWGc::Dwg_Entity_SPLINE_num_ctrl_pts_set", _wrap_Dwg_Entity_SPLINE_num_ctrl_pts_set},
{"LibreDWGc::Dwg_Entity_SPLINE_num_ctrl_pts_get", _wrap_Dwg_Entity_SPLINE_num_ctrl_pts_get},
{"LibreDWGc::Dwg_Entity_SPLINE_ctrl_pts_set", _wrap_Dwg_Entity_SPLINE_ctrl_pts_set},
{"LibreDWGc::Dwg_Entity_SPLINE_ctrl_pts_get", _wrap_Dwg_Entity_SPLINE_ctrl_pts_get},
{"LibreDWGc::new_Dwg_Entity_SPLINE", _wrap_new_Dwg_Entity_SPLINE},
{"LibreDWGc::delete_Dwg_Entity_SPLINE", _wrap_delete_Dwg_Entity_SPLINE},
{"LibreDWGc::Dwg_3DSOLID_wire_parent_set", _wrap_Dwg_3DSOLID_wire_parent_set},
{"LibreDWGc::Dwg_3DSOLID_wire_parent_get", _wrap_Dwg_3DSOLID_wire_parent_get},
{"LibreDWGc::Dwg_3DSOLID_wire_type_set", _wrap_Dwg_3DSOLID_wire_type_set},
{"LibreDWGc::Dwg_3DSOLID_wire_type_get", _wrap_Dwg_3DSOLID_wire_type_get},
{"LibreDWGc::Dwg_3DSOLID_wire_selection_marker_set", _wrap_Dwg_3DSOLID_wire_selection_marker_set},
{"LibreDWGc::Dwg_3DSOLID_wire_selection_marker_get", _wrap_Dwg_3DSOLID_wire_selection_marker_get},
{"LibreDWGc::Dwg_3DSOLID_wire_color_set", _wrap_Dwg_3DSOLID_wire_color_set},
{"LibreDWGc::Dwg_3DSOLID_wire_color_get", _wrap_Dwg_3DSOLID_wire_color_get},
{"LibreDWGc::Dwg_3DSOLID_wire_acis_index_set", _wrap_Dwg_3DSOLID_wire_acis_index_set},
{"LibreDWGc::Dwg_3DSOLID_wire_acis_index_get", _wrap_Dwg_3DSOLID_wire_acis_index_get},
{"LibreDWGc::Dwg_3DSOLID_wire_num_points_set", _wrap_Dwg_3DSOLID_wire_num_points_set},
{"LibreDWGc::Dwg_3DSOLID_wire_num_points_get", _wrap_Dwg_3DSOLID_wire_num_points_get},
{"LibreDWGc::Dwg_3DSOLID_wire_points_set", _wrap_Dwg_3DSOLID_wire_points_set},
{"LibreDWGc::Dwg_3DSOLID_wire_points_get", _wrap_Dwg_3DSOLID_wire_points_get},
{"LibreDWGc::Dwg_3DSOLID_wire_transform_present_set", _wrap_Dwg_3DSOLID_wire_transform_present_set},
{"LibreDWGc::Dwg_3DSOLID_wire_transform_present_get", _wrap_Dwg_3DSOLID_wire_transform_present_get},
{"LibreDWGc::Dwg_3DSOLID_wire_axis_x_set", _wrap_Dwg_3DSOLID_wire_axis_x_set},
{"LibreDWGc::Dwg_3DSOLID_wire_axis_x_get", _wrap_Dwg_3DSOLID_wire_axis_x_get},
{"LibreDWGc::Dwg_3DSOLID_wire_axis_y_set", _wrap_Dwg_3DSOLID_wire_axis_y_set},
{"LibreDWGc::Dwg_3DSOLID_wire_axis_y_get", _wrap_Dwg_3DSOLID_wire_axis_y_get},
{"LibreDWGc::Dwg_3DSOLID_wire_axis_z_set", _wrap_Dwg_3DSOLID_wire_axis_z_set},
{"LibreDWGc::Dwg_3DSOLID_wire_axis_z_get", _wrap_Dwg_3DSOLID_wire_axis_z_get},
{"LibreDWGc::Dwg_3DSOLID_wire_translation_set", _wrap_Dwg_3DSOLID_wire_translation_set},
{"LibreDWGc::Dwg_3DSOLID_wire_translation_get", _wrap_Dwg_3DSOLID_wire_translation_get},
{"LibreDWGc::Dwg_3DSOLID_wire_scale_set", _wrap_Dwg_3DSOLID_wire_scale_set},
{"LibreDWGc::Dwg_3DSOLID_wire_scale_get", _wrap_Dwg_3DSOLID_wire_scale_get},
{"LibreDWGc::Dwg_3DSOLID_wire_has_rotation_set", _wrap_Dwg_3DSOLID_wire_has_rotation_set},
{"LibreDWGc::Dwg_3DSOLID_wire_has_rotation_get", _wrap_Dwg_3DSOLID_wire_has_rotation_get},
{"LibreDWGc::Dwg_3DSOLID_wire_has_reflection_set", _wrap_Dwg_3DSOLID_wire_has_reflection_set},
{"LibreDWGc::Dwg_3DSOLID_wire_has_reflection_get", _wrap_Dwg_3DSOLID_wire_has_reflection_get},
{"LibreDWGc::Dwg_3DSOLID_wire_has_shear_set", _wrap_Dwg_3DSOLID_wire_has_shear_set},
{"LibreDWGc::Dwg_3DSOLID_wire_has_shear_get", _wrap_Dwg_3DSOLID_wire_has_shear_get},
{"LibreDWGc::new_Dwg_3DSOLID_wire", _wrap_new_Dwg_3DSOLID_wire},
{"LibreDWGc::delete_Dwg_3DSOLID_wire", _wrap_delete_Dwg_3DSOLID_wire},
{"LibreDWGc::Dwg_3DSOLID_silhouette_parent_set", _wrap_Dwg_3DSOLID_silhouette_parent_set},
{"LibreDWGc::Dwg_3DSOLID_silhouette_parent_get", _wrap_Dwg_3DSOLID_silhouette_parent_get},
{"LibreDWGc::Dwg_3DSOLID_silhouette_vp_id_set", _wrap_Dwg_3DSOLID_silhouette_vp_id_set},
{"LibreDWGc::Dwg_3DSOLID_silhouette_vp_id_get", _wrap_Dwg_3DSOLID_silhouette_vp_id_get},
{"LibreDWGc::Dwg_3DSOLID_silhouette_vp_target_set", _wrap_Dwg_3DSOLID_silhouette_vp_target_set},
{"LibreDWGc::Dwg_3DSOLID_silhouette_vp_target_get", _wrap_Dwg_3DSOLID_silhouette_vp_target_get},
{"LibreDWGc::Dwg_3DSOLID_silhouette_vp_dir_from_target_set", _wrap_Dwg_3DSOLID_silhouette_vp_dir_from_target_set},
{"LibreDWGc::Dwg_3DSOLID_silhouette_vp_dir_from_target_get", _wrap_Dwg_3DSOLID_silhouette_vp_dir_from_target_get},
{"LibreDWGc::Dwg_3DSOLID_silhouette_vp_up_dir_set", _wrap_Dwg_3DSOLID_silhouette_vp_up_dir_set},
{"LibreDWGc::Dwg_3DSOLID_silhouette_vp_up_dir_get", _wrap_Dwg_3DSOLID_silhouette_vp_up_dir_get},
{"LibreDWGc::Dwg_3DSOLID_silhouette_vp_perspective_set", _wrap_Dwg_3DSOLID_silhouette_vp_perspective_set},
{"LibreDWGc::Dwg_3DSOLID_silhouette_vp_perspective_get", _wrap_Dwg_3DSOLID_silhouette_vp_perspective_get},
{"LibreDWGc::Dwg_3DSOLID_silhouette_has_wires_set", _wrap_Dwg_3DSOLID_silhouette_has_wires_set},
{"LibreDWGc::Dwg_3DSOLID_silhouette_has_wires_get", _wrap_Dwg_3DSOLID_silhouette_has_wires_get},
{"LibreDWGc::Dwg_3DSOLID_silhouette_num_wires_set", _wrap_Dwg_3DSOLID_silhouette_num_wires_set},
{"LibreDWGc::Dwg_3DSOLID_silhouette_num_wires_get", _wrap_Dwg_3DSOLID_silhouette_num_wires_get},
{"LibreDWGc::Dwg_3DSOLID_silhouette_wires_set", _wrap_Dwg_3DSOLID_silhouette_wires_set},
{"LibreDWGc::Dwg_3DSOLID_silhouette_wires_get", _wrap_Dwg_3DSOLID_silhouette_wires_get},
{"LibreDWGc::new_Dwg_3DSOLID_silhouette", _wrap_new_Dwg_3DSOLID_silhouette},
{"LibreDWGc::delete_Dwg_3DSOLID_silhouette", _wrap_delete_Dwg_3DSOLID_silhouette},
{"LibreDWGc::Dwg_3DSOLID_material_parent_set", _wrap_Dwg_3DSOLID_material_parent_set},
{"LibreDWGc::Dwg_3DSOLID_material_parent_get", _wrap_Dwg_3DSOLID_material_parent_get},
{"LibreDWGc::Dwg_3DSOLID_material_array_index_set", _wrap_Dwg_3DSOLID_material_array_index_set},
{"LibreDWGc::Dwg_3DSOLID_material_array_index_get", _wrap_Dwg_3DSOLID_material_array_index_get},
{"LibreDWGc::Dwg_3DSOLID_material_mat_absref_set", _wrap_Dwg_3DSOLID_material_mat_absref_set},
{"LibreDWGc::Dwg_3DSOLID_material_mat_absref_get", _wrap_Dwg_3DSOLID_material_mat_absref_get},
{"LibreDWGc::Dwg_3DSOLID_material_material_handle_set", _wrap_Dwg_3DSOLID_material_material_handle_set},
{"LibreDWGc::Dwg_3DSOLID_material_material_handle_get", _wrap_Dwg_3DSOLID_material_material_handle_get},
{"LibreDWGc::new_Dwg_3DSOLID_material", _wrap_new_Dwg_3DSOLID_material},
{"LibreDWGc::delete_Dwg_3DSOLID_material", _wrap_delete_Dwg_3DSOLID_material},
{"LibreDWGc::Dwg_Entity__3DSOLID_parent_set", _wrap_Dwg_Entity__3DSOLID_parent_set},
{"LibreDWGc::Dwg_Entity__3DSOLID_parent_get", _wrap_Dwg_Entity__3DSOLID_parent_get},
{"LibreDWGc::Dwg_Entity__3DSOLID_acis_empty_set", _wrap_Dwg_Entity__3DSOLID_acis_empty_set},
{"LibreDWGc::Dwg_Entity__3DSOLID_acis_empty_get", _wrap_Dwg_Entity__3DSOLID_acis_empty_get},
{"LibreDWGc::Dwg_Entity__3DSOLID_unknown_set", _wrap_Dwg_Entity__3DSOLID_unknown_set},
{"LibreDWGc::Dwg_Entity__3DSOLID_unknown_get", _wrap_Dwg_Entity__3DSOLID_unknown_get},
{"LibreDWGc::Dwg_Entity__3DSOLID_version_set", _wrap_Dwg_Entity__3DSOLID_version_set},
{"LibreDWGc::Dwg_Entity__3DSOLID_version_get", _wrap_Dwg_Entity__3DSOLID_version_get},
{"LibreDWGc::Dwg_Entity__3DSOLID_num_blocks_set", _wrap_Dwg_Entity__3DSOLID_num_blocks_set},
{"LibreDWGc::Dwg_Entity__3DSOLID_num_blocks_get", _wrap_Dwg_Entity__3DSOLID_num_blocks_get},
{"LibreDWGc::Dwg_Entity__3DSOLID_block_size_set", _wrap_Dwg_Entity__3DSOLID_block_size_set},
{"LibreDWGc::Dwg_Entity__3DSOLID_block_size_get", _wrap_Dwg_Entity__3DSOLID_block_size_get},
{"LibreDWGc::Dwg_Entity__3DSOLID_encr_sat_data_set", _wrap_Dwg_Entity__3DSOLID_encr_sat_data_set},
{"LibreDWGc::Dwg_Entity__3DSOLID_encr_sat_data_get", _wrap_Dwg_Entity__3DSOLID_encr_sat_data_get},
{"LibreDWGc::Dwg_Entity__3DSOLID_sab_size_set", _wrap_Dwg_Entity__3DSOLID_sab_size_set},
{"LibreDWGc::Dwg_Entity__3DSOLID_sab_size_get", _wrap_Dwg_Entity__3DSOLID_sab_size_get},
{"LibreDWGc::Dwg_Entity__3DSOLID_acis_data_set", _wrap_Dwg_Entity__3DSOLID_acis_data_set},
{"LibreDWGc::Dwg_Entity__3DSOLID_acis_data_get", _wrap_Dwg_Entity__3DSOLID_acis_data_get},
{"LibreDWGc::Dwg_Entity__3DSOLID_wireframe_data_present_set", _wrap_Dwg_Entity__3DSOLID_wireframe_data_present_set},
{"LibreDWGc::Dwg_Entity__3DSOLID_wireframe_data_present_get", _wrap_Dwg_Entity__3DSOLID_wireframe_data_present_get},
{"LibreDWGc::Dwg_Entity__3DSOLID_point_present_set", _wrap_Dwg_Entity__3DSOLID_point_present_set},
{"LibreDWGc::Dwg_Entity__3DSOLID_point_present_get", _wrap_Dwg_Entity__3DSOLID_point_present_get},
{"LibreDWGc::Dwg_Entity__3DSOLID_point_set", _wrap_Dwg_Entity__3DSOLID_point_set},
{"LibreDWGc::Dwg_Entity__3DSOLID_point_get", _wrap_Dwg_Entity__3DSOLID_point_get},
{"LibreDWGc::Dwg_Entity__3DSOLID_isolines_set", _wrap_Dwg_Entity__3DSOLID_isolines_set},
{"LibreDWGc::Dwg_Entity__3DSOLID_isolines_get", _wrap_Dwg_Entity__3DSOLID_isolines_get},
{"LibreDWGc::Dwg_Entity__3DSOLID_isoline_present_set", _wrap_Dwg_Entity__3DSOLID_isoline_present_set},
{"LibreDWGc::Dwg_Entity__3DSOLID_isoline_present_get", _wrap_Dwg_Entity__3DSOLID_isoline_present_get},
{"LibreDWGc::Dwg_Entity__3DSOLID_num_wires_set", _wrap_Dwg_Entity__3DSOLID_num_wires_set},
{"LibreDWGc::Dwg_Entity__3DSOLID_num_wires_get", _wrap_Dwg_Entity__3DSOLID_num_wires_get},
{"LibreDWGc::Dwg_Entity__3DSOLID_wires_set", _wrap_Dwg_Entity__3DSOLID_wires_set},
{"LibreDWGc::Dwg_Entity__3DSOLID_wires_get", _wrap_Dwg_Entity__3DSOLID_wires_get},
{"LibreDWGc::Dwg_Entity__3DSOLID_num_silhouettes_set", _wrap_Dwg_Entity__3DSOLID_num_silhouettes_set},
{"LibreDWGc::Dwg_Entity__3DSOLID_num_silhouettes_get", _wrap_Dwg_Entity__3DSOLID_num_silhouettes_get},
{"LibreDWGc::Dwg_Entity__3DSOLID_silhouettes_set", _wrap_Dwg_Entity__3DSOLID_silhouettes_set},
{"LibreDWGc::Dwg_Entity__3DSOLID_silhouettes_get", _wrap_Dwg_Entity__3DSOLID_silhouettes_get},
{"LibreDWGc::Dwg_Entity__3DSOLID__dxf_sab_converted_set", _wrap_Dwg_Entity__3DSOLID__dxf_sab_converted_set},
{"LibreDWGc::Dwg_Entity__3DSOLID__dxf_sab_converted_get", _wrap_Dwg_Entity__3DSOLID__dxf_sab_converted_get},
{"LibreDWGc::Dwg_Entity__3DSOLID_acis_empty2_set", _wrap_Dwg_Entity__3DSOLID_acis_empty2_set},
{"LibreDWGc::Dwg_Entity__3DSOLID_acis_empty2_get", _wrap_Dwg_Entity__3DSOLID_acis_empty2_get},
{"LibreDWGc::Dwg_Entity__3DSOLID_extra_acis_data_set", _wrap_Dwg_Entity__3DSOLID_extra_acis_data_set},
{"LibreDWGc::Dwg_Entity__3DSOLID_extra_acis_data_get", _wrap_Dwg_Entity__3DSOLID_extra_acis_data_get},
{"LibreDWGc::Dwg_Entity__3DSOLID_num_materials_set", _wrap_Dwg_Entity__3DSOLID_num_materials_set},
{"LibreDWGc::Dwg_Entity__3DSOLID_num_materials_get", _wrap_Dwg_Entity__3DSOLID_num_materials_get},
{"LibreDWGc::Dwg_Entity__3DSOLID_materials_set", _wrap_Dwg_Entity__3DSOLID_materials_set},
{"LibreDWGc::Dwg_Entity__3DSOLID_materials_get", _wrap_Dwg_Entity__3DSOLID_materials_get},
{"LibreDWGc::Dwg_Entity__3DSOLID_revision_guid_set", _wrap_Dwg_Entity__3DSOLID_revision_guid_set},
{"LibreDWGc::Dwg_Entity__3DSOLID_revision_guid_get", _wrap_Dwg_Entity__3DSOLID_revision_guid_get},
{"LibreDWGc::Dwg_Entity__3DSOLID_revision_major_set", _wrap_Dwg_Entity__3DSOLID_revision_major_set},
{"LibreDWGc::Dwg_Entity__3DSOLID_revision_major_get", _wrap_Dwg_Entity__3DSOLID_revision_major_get},
{"LibreDWGc::Dwg_Entity__3DSOLID_revision_minor1_set", _wrap_Dwg_Entity__3DSOLID_revision_minor1_set},
{"LibreDWGc::Dwg_Entity__3DSOLID_revision_minor1_get", _wrap_Dwg_Entity__3DSOLID_revision_minor1_get},
{"LibreDWGc::Dwg_Entity__3DSOLID_revision_minor2_set", _wrap_Dwg_Entity__3DSOLID_revision_minor2_set},
{"LibreDWGc::Dwg_Entity__3DSOLID_revision_minor2_get", _wrap_Dwg_Entity__3DSOLID_revision_minor2_get},
{"LibreDWGc::Dwg_Entity__3DSOLID_revision_bytes_set", _wrap_Dwg_Entity__3DSOLID_revision_bytes_set},
{"LibreDWGc::Dwg_Entity__3DSOLID_revision_bytes_get", _wrap_Dwg_Entity__3DSOLID_revision_bytes_get},
{"LibreDWGc::Dwg_Entity__3DSOLID_end_marker_set", _wrap_Dwg_Entity__3DSOLID_end_marker_set},
{"LibreDWGc::Dwg_Entity__3DSOLID_end_marker_get", _wrap_Dwg_Entity__3DSOLID_end_marker_get},
{"LibreDWGc::Dwg_Entity__3DSOLID_history_id_set", _wrap_Dwg_Entity__3DSOLID_history_id_set},
{"LibreDWGc::Dwg_Entity__3DSOLID_history_id_get", _wrap_Dwg_Entity__3DSOLID_history_id_get},
{"LibreDWGc::Dwg_Entity__3DSOLID_has_revision_guid_set", _wrap_Dwg_Entity__3DSOLID_has_revision_guid_set},
{"LibreDWGc::Dwg_Entity__3DSOLID_has_revision_guid_get", _wrap_Dwg_Entity__3DSOLID_has_revision_guid_get},
{"LibreDWGc::Dwg_Entity__3DSOLID_acis_empty_bit_set", _wrap_Dwg_Entity__3DSOLID_acis_empty_bit_set},
{"LibreDWGc::Dwg_Entity__3DSOLID_acis_empty_bit_get", _wrap_Dwg_Entity__3DSOLID_acis_empty_bit_get},
{"LibreDWGc::new_Dwg_Entity__3DSOLID", _wrap_new_Dwg_Entity__3DSOLID},
{"LibreDWGc::delete_Dwg_Entity__3DSOLID", _wrap_delete_Dwg_Entity__3DSOLID},
{"LibreDWGc::Dwg_Entity_RAY_parent_set", _wrap_Dwg_Entity_RAY_parent_set},
{"LibreDWGc::Dwg_Entity_RAY_parent_get", _wrap_Dwg_Entity_RAY_parent_get},
{"LibreDWGc::Dwg_Entity_RAY_point_set", _wrap_Dwg_Entity_RAY_point_set},
{"LibreDWGc::Dwg_Entity_RAY_point_get", _wrap_Dwg_Entity_RAY_point_get},
{"LibreDWGc::Dwg_Entity_RAY_vector_set", _wrap_Dwg_Entity_RAY_vector_set},
{"LibreDWGc::Dwg_Entity_RAY_vector_get", _wrap_Dwg_Entity_RAY_vector_get},
{"LibreDWGc::new_Dwg_Entity_RAY", _wrap_new_Dwg_Entity_RAY},
{"LibreDWGc::delete_Dwg_Entity_RAY", _wrap_delete_Dwg_Entity_RAY},
{"LibreDWGc::Dwg_Object_DICTIONARY_parent_set", _wrap_Dwg_Object_DICTIONARY_parent_set},
{"LibreDWGc::Dwg_Object_DICTIONARY_parent_get", _wrap_Dwg_Object_DICTIONARY_parent_get},
{"LibreDWGc::Dwg_Object_DICTIONARY_numitems_set", _wrap_Dwg_Object_DICTIONARY_numitems_set},
{"LibreDWGc::Dwg_Object_DICTIONARY_numitems_get", _wrap_Dwg_Object_DICTIONARY_numitems_get},
{"LibreDWGc::Dwg_Object_DICTIONARY_is_hardowner_set", _wrap_Dwg_Object_DICTIONARY_is_hardowner_set},
{"LibreDWGc::Dwg_Object_DICTIONARY_is_hardowner_get", _wrap_Dwg_Object_DICTIONARY_is_hardowner_get},
{"LibreDWGc::Dwg_Object_DICTIONARY_cloning_set", _wrap_Dwg_Object_DICTIONARY_cloning_set},
{"LibreDWGc::Dwg_Object_DICTIONARY_cloning_get", _wrap_Dwg_Object_DICTIONARY_cloning_get},
{"LibreDWGc::Dwg_Object_DICTIONARY_texts_set", _wrap_Dwg_Object_DICTIONARY_texts_set},
{"LibreDWGc::Dwg_Object_DICTIONARY_texts_get", _wrap_Dwg_Object_DICTIONARY_texts_get},
{"LibreDWGc::Dwg_Object_DICTIONARY_itemhandles_set", _wrap_Dwg_Object_DICTIONARY_itemhandles_set},
{"LibreDWGc::Dwg_Object_DICTIONARY_itemhandles_get", _wrap_Dwg_Object_DICTIONARY_itemhandles_get},
{"LibreDWGc::new_Dwg_Object_DICTIONARY", _wrap_new_Dwg_Object_DICTIONARY},
{"LibreDWGc::delete_Dwg_Object_DICTIONARY", _wrap_delete_Dwg_Object_DICTIONARY},
{"LibreDWGc::Dwg_Object_DICTIONARYWDFLT_parent_set", _wrap_Dwg_Object_DICTIONARYWDFLT_parent_set},
{"LibreDWGc::Dwg_Object_DICTIONARYWDFLT_parent_get", _wrap_Dwg_Object_DICTIONARYWDFLT_parent_get},
{"LibreDWGc::Dwg_Object_DICTIONARYWDFLT_numitems_set", _wrap_Dwg_Object_DICTIONARYWDFLT_numitems_set},
{"LibreDWGc::Dwg_Object_DICTIONARYWDFLT_numitems_get", _wrap_Dwg_Object_DICTIONARYWDFLT_numitems_get},
{"LibreDWGc::Dwg_Object_DICTIONARYWDFLT_is_hardowner_set", _wrap_Dwg_Object_DICTIONARYWDFLT_is_hardowner_set},
{"LibreDWGc::Dwg_Object_DICTIONARYWDFLT_is_hardowner_get", _wrap_Dwg_Object_DICTIONARYWDFLT_is_hardowner_get},
{"LibreDWGc::Dwg_Object_DICTIONARYWDFLT_cloning_set", _wrap_Dwg_Object_DICTIONARYWDFLT_cloning_set},
{"LibreDWGc::Dwg_Object_DICTIONARYWDFLT_cloning_get", _wrap_Dwg_Object_DICTIONARYWDFLT_cloning_get},
{"LibreDWGc::Dwg_Object_DICTIONARYWDFLT_texts_set", _wrap_Dwg_Object_DICTIONARYWDFLT_texts_set},
{"LibreDWGc::Dwg_Object_DICTIONARYWDFLT_texts_get", _wrap_Dwg_Object_DICTIONARYWDFLT_texts_get},
{"LibreDWGc::Dwg_Object_DICTIONARYWDFLT_itemhandles_set", _wrap_Dwg_Object_DICTIONARYWDFLT_itemhandles_set},
{"LibreDWGc::Dwg_Object_DICTIONARYWDFLT_itemhandles_get", _wrap_Dwg_Object_DICTIONARYWDFLT_itemhandles_get},
{"LibreDWGc::Dwg_Object_DICTIONARYWDFLT_defaultid_set", _wrap_Dwg_Object_DICTIONARYWDFLT_defaultid_set},
{"LibreDWGc::Dwg_Object_DICTIONARYWDFLT_defaultid_get", _wrap_Dwg_Object_DICTIONARYWDFLT_defaultid_get},
{"LibreDWGc::new_Dwg_Object_DICTIONARYWDFLT", _wrap_new_Dwg_Object_DICTIONARYWDFLT},
{"LibreDWGc::delete_Dwg_Object_DICTIONARYWDFLT", _wrap_delete_Dwg_Object_DICTIONARYWDFLT},
{"LibreDWGc::Dwg_Entity_OLEFRAME_parent_set", _wrap_Dwg_Entity_OLEFRAME_parent_set},
{"LibreDWGc::Dwg_Entity_OLEFRAME_parent_get", _wrap_Dwg_Entity_OLEFRAME_parent_get},
{"LibreDWGc::Dwg_Entity_OLEFRAME_flag_set", _wrap_Dwg_Entity_OLEFRAME_flag_set},
{"LibreDWGc::Dwg_Entity_OLEFRAME_flag_get", _wrap_Dwg_Entity_OLEFRAME_flag_get},
{"LibreDWGc::Dwg_Entity_OLEFRAME_mode_set", _wrap_Dwg_Entity_OLEFRAME_mode_set},
{"LibreDWGc::Dwg_Entity_OLEFRAME_mode_get", _wrap_Dwg_Entity_OLEFRAME_mode_get},
{"LibreDWGc::Dwg_Entity_OLEFRAME_data_size_set", _wrap_Dwg_Entity_OLEFRAME_data_size_set},
{"LibreDWGc::Dwg_Entity_OLEFRAME_data_size_get", _wrap_Dwg_Entity_OLEFRAME_data_size_get},
{"LibreDWGc::Dwg_Entity_OLEFRAME_data_set", _wrap_Dwg_Entity_OLEFRAME_data_set},
{"LibreDWGc::Dwg_Entity_OLEFRAME_data_get", _wrap_Dwg_Entity_OLEFRAME_data_get},
{"LibreDWGc::new_Dwg_Entity_OLEFRAME", _wrap_new_Dwg_Entity_OLEFRAME},
{"LibreDWGc::delete_Dwg_Entity_OLEFRAME", _wrap_delete_Dwg_Entity_OLEFRAME},
{"LibreDWGc::Dwg_Entity_MTEXT_parent_set", _wrap_Dwg_Entity_MTEXT_parent_set},
{"LibreDWGc::Dwg_Entity_MTEXT_parent_get", _wrap_Dwg_Entity_MTEXT_parent_get},
{"LibreDWGc::Dwg_Entity_MTEXT_ins_pt_set", _wrap_Dwg_Entity_MTEXT_ins_pt_set},
{"LibreDWGc::Dwg_Entity_MTEXT_ins_pt_get", _wrap_Dwg_Entity_MTEXT_ins_pt_get},
{"LibreDWGc::Dwg_Entity_MTEXT_extrusion_set", _wrap_Dwg_Entity_MTEXT_extrusion_set},
{"LibreDWGc::Dwg_Entity_MTEXT_extrusion_get", _wrap_Dwg_Entity_MTEXT_extrusion_get},
{"LibreDWGc::Dwg_Entity_MTEXT_x_axis_dir_set", _wrap_Dwg_Entity_MTEXT_x_axis_dir_set},
{"LibreDWGc::Dwg_Entity_MTEXT_x_axis_dir_get", _wrap_Dwg_Entity_MTEXT_x_axis_dir_get},
{"LibreDWGc::Dwg_Entity_MTEXT_rect_height_set", _wrap_Dwg_Entity_MTEXT_rect_height_set},
{"LibreDWGc::Dwg_Entity_MTEXT_rect_height_get", _wrap_Dwg_Entity_MTEXT_rect_height_get},
{"LibreDWGc::Dwg_Entity_MTEXT_rect_width_set", _wrap_Dwg_Entity_MTEXT_rect_width_set},
{"LibreDWGc::Dwg_Entity_MTEXT_rect_width_get", _wrap_Dwg_Entity_MTEXT_rect_width_get},
{"LibreDWGc::Dwg_Entity_MTEXT_text_height_set", _wrap_Dwg_Entity_MTEXT_text_height_set},
{"LibreDWGc::Dwg_Entity_MTEXT_text_height_get", _wrap_Dwg_Entity_MTEXT_text_height_get},
{"LibreDWGc::Dwg_Entity_MTEXT_attachment_set", _wrap_Dwg_Entity_MTEXT_attachment_set},
{"LibreDWGc::Dwg_Entity_MTEXT_attachment_get", _wrap_Dwg_Entity_MTEXT_attachment_get},
{"LibreDWGc::Dwg_Entity_MTEXT_flow_dir_set", _wrap_Dwg_Entity_MTEXT_flow_dir_set},
{"LibreDWGc::Dwg_Entity_MTEXT_flow_dir_get", _wrap_Dwg_Entity_MTEXT_flow_dir_get},
{"LibreDWGc::Dwg_Entity_MTEXT_extents_width_set", _wrap_Dwg_Entity_MTEXT_extents_width_set},
{"LibreDWGc::Dwg_Entity_MTEXT_extents_width_get", _wrap_Dwg_Entity_MTEXT_extents_width_get},
{"LibreDWGc::Dwg_Entity_MTEXT_extents_height_set", _wrap_Dwg_Entity_MTEXT_extents_height_set},
{"LibreDWGc::Dwg_Entity_MTEXT_extents_height_get", _wrap_Dwg_Entity_MTEXT_extents_height_get},
{"LibreDWGc::Dwg_Entity_MTEXT_text_set", _wrap_Dwg_Entity_MTEXT_text_set},
{"LibreDWGc::Dwg_Entity_MTEXT_text_get", _wrap_Dwg_Entity_MTEXT_text_get},
{"LibreDWGc::Dwg_Entity_MTEXT_style_set", _wrap_Dwg_Entity_MTEXT_style_set},
{"LibreDWGc::Dwg_Entity_MTEXT_style_get", _wrap_Dwg_Entity_MTEXT_style_get},
{"LibreDWGc::Dwg_Entity_MTEXT_linespace_style_set", _wrap_Dwg_Entity_MTEXT_linespace_style_set},
{"LibreDWGc::Dwg_Entity_MTEXT_linespace_style_get", _wrap_Dwg_Entity_MTEXT_linespace_style_get},
{"LibreDWGc::Dwg_Entity_MTEXT_linespace_factor_set", _wrap_Dwg_Entity_MTEXT_linespace_factor_set},
{"LibreDWGc::Dwg_Entity_MTEXT_linespace_factor_get", _wrap_Dwg_Entity_MTEXT_linespace_factor_get},
{"LibreDWGc::Dwg_Entity_MTEXT_unknown_b0_set", _wrap_Dwg_Entity_MTEXT_unknown_b0_set},
{"LibreDWGc::Dwg_Entity_MTEXT_unknown_b0_get", _wrap_Dwg_Entity_MTEXT_unknown_b0_get},
{"LibreDWGc::Dwg_Entity_MTEXT_bg_fill_flag_set", _wrap_Dwg_Entity_MTEXT_bg_fill_flag_set},
{"LibreDWGc::Dwg_Entity_MTEXT_bg_fill_flag_get", _wrap_Dwg_Entity_MTEXT_bg_fill_flag_get},
{"LibreDWGc::Dwg_Entity_MTEXT_bg_fill_scale_set", _wrap_Dwg_Entity_MTEXT_bg_fill_scale_set},
{"LibreDWGc::Dwg_Entity_MTEXT_bg_fill_scale_get", _wrap_Dwg_Entity_MTEXT_bg_fill_scale_get},
{"LibreDWGc::Dwg_Entity_MTEXT_bg_fill_color_set", _wrap_Dwg_Entity_MTEXT_bg_fill_color_set},
{"LibreDWGc::Dwg_Entity_MTEXT_bg_fill_color_get", _wrap_Dwg_Entity_MTEXT_bg_fill_color_get},
{"LibreDWGc::Dwg_Entity_MTEXT_bg_fill_trans_set", _wrap_Dwg_Entity_MTEXT_bg_fill_trans_set},
{"LibreDWGc::Dwg_Entity_MTEXT_bg_fill_trans_get", _wrap_Dwg_Entity_MTEXT_bg_fill_trans_get},
{"LibreDWGc::Dwg_Entity_MTEXT_is_not_annotative_set", _wrap_Dwg_Entity_MTEXT_is_not_annotative_set},
{"LibreDWGc::Dwg_Entity_MTEXT_is_not_annotative_get", _wrap_Dwg_Entity_MTEXT_is_not_annotative_get},
{"LibreDWGc::Dwg_Entity_MTEXT_class_version_set", _wrap_Dwg_Entity_MTEXT_class_version_set},
{"LibreDWGc::Dwg_Entity_MTEXT_class_version_get", _wrap_Dwg_Entity_MTEXT_class_version_get},
{"LibreDWGc::Dwg_Entity_MTEXT_default_flag_set", _wrap_Dwg_Entity_MTEXT_default_flag_set},
{"LibreDWGc::Dwg_Entity_MTEXT_default_flag_get", _wrap_Dwg_Entity_MTEXT_default_flag_get},
{"LibreDWGc::Dwg_Entity_MTEXT_appid_set", _wrap_Dwg_Entity_MTEXT_appid_set},
{"LibreDWGc::Dwg_Entity_MTEXT_appid_get", _wrap_Dwg_Entity_MTEXT_appid_get},
{"LibreDWGc::Dwg_Entity_MTEXT_ignore_attachment_set", _wrap_Dwg_Entity_MTEXT_ignore_attachment_set},
{"LibreDWGc::Dwg_Entity_MTEXT_ignore_attachment_get", _wrap_Dwg_Entity_MTEXT_ignore_attachment_get},
{"LibreDWGc::Dwg_Entity_MTEXT_column_type_set", _wrap_Dwg_Entity_MTEXT_column_type_set},
{"LibreDWGc::Dwg_Entity_MTEXT_column_type_get", _wrap_Dwg_Entity_MTEXT_column_type_get},
{"LibreDWGc::Dwg_Entity_MTEXT_numfragments_set", _wrap_Dwg_Entity_MTEXT_numfragments_set},
{"LibreDWGc::Dwg_Entity_MTEXT_numfragments_get", _wrap_Dwg_Entity_MTEXT_numfragments_get},
{"LibreDWGc::Dwg_Entity_MTEXT_column_width_set", _wrap_Dwg_Entity_MTEXT_column_width_set},
{"LibreDWGc::Dwg_Entity_MTEXT_column_width_get", _wrap_Dwg_Entity_MTEXT_column_width_get},
{"LibreDWGc::Dwg_Entity_MTEXT_gutter_set", _wrap_Dwg_Entity_MTEXT_gutter_set},
{"LibreDWGc::Dwg_Entity_MTEXT_gutter_get", _wrap_Dwg_Entity_MTEXT_gutter_get},
{"LibreDWGc::Dwg_Entity_MTEXT_auto_height_set", _wrap_Dwg_Entity_MTEXT_auto_height_set},
{"LibreDWGc::Dwg_Entity_MTEXT_auto_height_get", _wrap_Dwg_Entity_MTEXT_auto_height_get},
{"LibreDWGc::Dwg_Entity_MTEXT_flow_reversed_set", _wrap_Dwg_Entity_MTEXT_flow_reversed_set},
{"LibreDWGc::Dwg_Entity_MTEXT_flow_reversed_get", _wrap_Dwg_Entity_MTEXT_flow_reversed_get},
{"LibreDWGc::Dwg_Entity_MTEXT_num_column_heights_set", _wrap_Dwg_Entity_MTEXT_num_column_heights_set},
{"LibreDWGc::Dwg_Entity_MTEXT_num_column_heights_get", _wrap_Dwg_Entity_MTEXT_num_column_heights_get},
{"LibreDWGc::Dwg_Entity_MTEXT_column_heights_set", _wrap_Dwg_Entity_MTEXT_column_heights_set},
{"LibreDWGc::Dwg_Entity_MTEXT_column_heights_get", _wrap_Dwg_Entity_MTEXT_column_heights_get},
{"LibreDWGc::new_Dwg_Entity_MTEXT", _wrap_new_Dwg_Entity_MTEXT},
{"LibreDWGc::delete_Dwg_Entity_MTEXT", _wrap_delete_Dwg_Entity_MTEXT},
{"LibreDWGc::Dwg_Entity_LEADER_parent_set", _wrap_Dwg_Entity_LEADER_parent_set},
{"LibreDWGc::Dwg_Entity_LEADER_parent_get", _wrap_Dwg_Entity_LEADER_parent_get},
{"LibreDWGc::Dwg_Entity_LEADER_unknown_bit_1_set", _wrap_Dwg_Entity_LEADER_unknown_bit_1_set},
{"LibreDWGc::Dwg_Entity_LEADER_unknown_bit_1_get", _wrap_Dwg_Entity_LEADER_unknown_bit_1_get},
{"LibreDWGc::Dwg_Entity_LEADER_path_type_set", _wrap_Dwg_Entity_LEADER_path_type_set},
{"LibreDWGc::Dwg_Entity_LEADER_path_type_get", _wrap_Dwg_Entity_LEADER_path_type_get},
{"LibreDWGc::Dwg_Entity_LEADER_annot_type_set", _wrap_Dwg_Entity_LEADER_annot_type_set},
{"LibreDWGc::Dwg_Entity_LEADER_annot_type_get", _wrap_Dwg_Entity_LEADER_annot_type_get},
{"LibreDWGc::Dwg_Entity_LEADER_num_points_set", _wrap_Dwg_Entity_LEADER_num_points_set},
{"LibreDWGc::Dwg_Entity_LEADER_num_points_get", _wrap_Dwg_Entity_LEADER_num_points_get},
{"LibreDWGc::Dwg_Entity_LEADER_points_set", _wrap_Dwg_Entity_LEADER_points_set},
{"LibreDWGc::Dwg_Entity_LEADER_points_get", _wrap_Dwg_Entity_LEADER_points_get},
{"LibreDWGc::Dwg_Entity_LEADER_origin_set", _wrap_Dwg_Entity_LEADER_origin_set},
{"LibreDWGc::Dwg_Entity_LEADER_origin_get", _wrap_Dwg_Entity_LEADER_origin_get},
{"LibreDWGc::Dwg_Entity_LEADER_extrusion_set", _wrap_Dwg_Entity_LEADER_extrusion_set},
{"LibreDWGc::Dwg_Entity_LEADER_extrusion_get", _wrap_Dwg_Entity_LEADER_extrusion_get},
{"LibreDWGc::Dwg_Entity_LEADER_x_direction_set", _wrap_Dwg_Entity_LEADER_x_direction_set},
{"LibreDWGc::Dwg_Entity_LEADER_x_direction_get", _wrap_Dwg_Entity_LEADER_x_direction_get},
{"LibreDWGc::Dwg_Entity_LEADER_inspt_offset_set", _wrap_Dwg_Entity_LEADER_inspt_offset_set},
{"LibreDWGc::Dwg_Entity_LEADER_inspt_offset_get", _wrap_Dwg_Entity_LEADER_inspt_offset_get},
{"LibreDWGc::Dwg_Entity_LEADER_endptproj_set", _wrap_Dwg_Entity_LEADER_endptproj_set},
{"LibreDWGc::Dwg_Entity_LEADER_endptproj_get", _wrap_Dwg_Entity_LEADER_endptproj_get},
{"LibreDWGc::Dwg_Entity_LEADER_dimgap_set", _wrap_Dwg_Entity_LEADER_dimgap_set},
{"LibreDWGc::Dwg_Entity_LEADER_dimgap_get", _wrap_Dwg_Entity_LEADER_dimgap_get},
{"LibreDWGc::Dwg_Entity_LEADER_box_height_set", _wrap_Dwg_Entity_LEADER_box_height_set},
{"LibreDWGc::Dwg_Entity_LEADER_box_height_get", _wrap_Dwg_Entity_LEADER_box_height_get},
{"LibreDWGc::Dwg_Entity_LEADER_box_width_set", _wrap_Dwg_Entity_LEADER_box_width_set},
{"LibreDWGc::Dwg_Entity_LEADER_box_width_get", _wrap_Dwg_Entity_LEADER_box_width_get},
{"LibreDWGc::Dwg_Entity_LEADER_hookline_dir_set", _wrap_Dwg_Entity_LEADER_hookline_dir_set},
{"LibreDWGc::Dwg_Entity_LEADER_hookline_dir_get", _wrap_Dwg_Entity_LEADER_hookline_dir_get},
{"LibreDWGc::Dwg_Entity_LEADER_hookline_on_set", _wrap_Dwg_Entity_LEADER_hookline_on_set},
{"LibreDWGc::Dwg_Entity_LEADER_hookline_on_get", _wrap_Dwg_Entity_LEADER_hookline_on_get},
{"LibreDWGc::Dwg_Entity_LEADER_arrowhead_on_set", _wrap_Dwg_Entity_LEADER_arrowhead_on_set},
{"LibreDWGc::Dwg_Entity_LEADER_arrowhead_on_get", _wrap_Dwg_Entity_LEADER_arrowhead_on_get},
{"LibreDWGc::Dwg_Entity_LEADER_arrowhead_type_set", _wrap_Dwg_Entity_LEADER_arrowhead_type_set},
{"LibreDWGc::Dwg_Entity_LEADER_arrowhead_type_get", _wrap_Dwg_Entity_LEADER_arrowhead_type_get},
{"LibreDWGc::Dwg_Entity_LEADER_dimasz_set", _wrap_Dwg_Entity_LEADER_dimasz_set},
{"LibreDWGc::Dwg_Entity_LEADER_dimasz_get", _wrap_Dwg_Entity_LEADER_dimasz_get},
{"LibreDWGc::Dwg_Entity_LEADER_unknown_bit_2_set", _wrap_Dwg_Entity_LEADER_unknown_bit_2_set},
{"LibreDWGc::Dwg_Entity_LEADER_unknown_bit_2_get", _wrap_Dwg_Entity_LEADER_unknown_bit_2_get},
{"LibreDWGc::Dwg_Entity_LEADER_unknown_bit_3_set", _wrap_Dwg_Entity_LEADER_unknown_bit_3_set},
{"LibreDWGc::Dwg_Entity_LEADER_unknown_bit_3_get", _wrap_Dwg_Entity_LEADER_unknown_bit_3_get},
{"LibreDWGc::Dwg_Entity_LEADER_unknown_short_1_set", _wrap_Dwg_Entity_LEADER_unknown_short_1_set},
{"LibreDWGc::Dwg_Entity_LEADER_unknown_short_1_get", _wrap_Dwg_Entity_LEADER_unknown_short_1_get},
{"LibreDWGc::Dwg_Entity_LEADER_byblock_color_set", _wrap_Dwg_Entity_LEADER_byblock_color_set},
{"LibreDWGc::Dwg_Entity_LEADER_byblock_color_get", _wrap_Dwg_Entity_LEADER_byblock_color_get},
{"LibreDWGc::Dwg_Entity_LEADER_unknown_bit_4_set", _wrap_Dwg_Entity_LEADER_unknown_bit_4_set},
{"LibreDWGc::Dwg_Entity_LEADER_unknown_bit_4_get", _wrap_Dwg_Entity_LEADER_unknown_bit_4_get},
{"LibreDWGc::Dwg_Entity_LEADER_unknown_bit_5_set", _wrap_Dwg_Entity_LEADER_unknown_bit_5_set},
{"LibreDWGc::Dwg_Entity_LEADER_unknown_bit_5_get", _wrap_Dwg_Entity_LEADER_unknown_bit_5_get},
{"LibreDWGc::Dwg_Entity_LEADER_associated_annotation_set", _wrap_Dwg_Entity_LEADER_associated_annotation_set},
{"LibreDWGc::Dwg_Entity_LEADER_associated_annotation_get", _wrap_Dwg_Entity_LEADER_associated_annotation_get},
{"LibreDWGc::Dwg_Entity_LEADER_dimstyle_set", _wrap_Dwg_Entity_LEADER_dimstyle_set},
{"LibreDWGc::Dwg_Entity_LEADER_dimstyle_get", _wrap_Dwg_Entity_LEADER_dimstyle_get},
{"LibreDWGc::new_Dwg_Entity_LEADER", _wrap_new_Dwg_Entity_LEADER},
{"LibreDWGc::delete_Dwg_Entity_LEADER", _wrap_delete_Dwg_Entity_LEADER},
{"LibreDWGc::Dwg_Entity_TOLERANCE_parent_set", _wrap_Dwg_Entity_TOLERANCE_parent_set},
{"LibreDWGc::Dwg_Entity_TOLERANCE_parent_get", _wrap_Dwg_Entity_TOLERANCE_parent_get},
{"LibreDWGc::Dwg_Entity_TOLERANCE_unknown_short_set", _wrap_Dwg_Entity_TOLERANCE_unknown_short_set},
{"LibreDWGc::Dwg_Entity_TOLERANCE_unknown_short_get", _wrap_Dwg_Entity_TOLERANCE_unknown_short_get},
{"LibreDWGc::Dwg_Entity_TOLERANCE_height_set", _wrap_Dwg_Entity_TOLERANCE_height_set},
{"LibreDWGc::Dwg_Entity_TOLERANCE_height_get", _wrap_Dwg_Entity_TOLERANCE_height_get},
{"LibreDWGc::Dwg_Entity_TOLERANCE_dimgap_set", _wrap_Dwg_Entity_TOLERANCE_dimgap_set},
{"LibreDWGc::Dwg_Entity_TOLERANCE_dimgap_get", _wrap_Dwg_Entity_TOLERANCE_dimgap_get},
{"LibreDWGc::Dwg_Entity_TOLERANCE_ins_pt_set", _wrap_Dwg_Entity_TOLERANCE_ins_pt_set},
{"LibreDWGc::Dwg_Entity_TOLERANCE_ins_pt_get", _wrap_Dwg_Entity_TOLERANCE_ins_pt_get},
{"LibreDWGc::Dwg_Entity_TOLERANCE_x_direction_set", _wrap_Dwg_Entity_TOLERANCE_x_direction_set},
{"LibreDWGc::Dwg_Entity_TOLERANCE_x_direction_get", _wrap_Dwg_Entity_TOLERANCE_x_direction_get},
{"LibreDWGc::Dwg_Entity_TOLERANCE_extrusion_set", _wrap_Dwg_Entity_TOLERANCE_extrusion_set},
{"LibreDWGc::Dwg_Entity_TOLERANCE_extrusion_get", _wrap_Dwg_Entity_TOLERANCE_extrusion_get},
{"LibreDWGc::Dwg_Entity_TOLERANCE_text_value_set", _wrap_Dwg_Entity_TOLERANCE_text_value_set},
{"LibreDWGc::Dwg_Entity_TOLERANCE_text_value_get", _wrap_Dwg_Entity_TOLERANCE_text_value_get},
{"LibreDWGc::Dwg_Entity_TOLERANCE_dimstyle_set", _wrap_Dwg_Entity_TOLERANCE_dimstyle_set},
{"LibreDWGc::Dwg_Entity_TOLERANCE_dimstyle_get", _wrap_Dwg_Entity_TOLERANCE_dimstyle_get},
{"LibreDWGc::new_Dwg_Entity_TOLERANCE", _wrap_new_Dwg_Entity_TOLERANCE},
{"LibreDWGc::delete_Dwg_Entity_TOLERANCE", _wrap_delete_Dwg_Entity_TOLERANCE},
{"LibreDWGc::Dwg_MLINE_line_parent_set", _wrap_Dwg_MLINE_line_parent_set},
{"LibreDWGc::Dwg_MLINE_line_parent_get", _wrap_Dwg_MLINE_line_parent_get},
{"LibreDWGc::Dwg_MLINE_line_num_segparms_set", _wrap_Dwg_MLINE_line_num_segparms_set},
{"LibreDWGc::Dwg_MLINE_line_num_segparms_get", _wrap_Dwg_MLINE_line_num_segparms_get},
{"LibreDWGc::Dwg_MLINE_line_segparms_set", _wrap_Dwg_MLINE_line_segparms_set},
{"LibreDWGc::Dwg_MLINE_line_segparms_get", _wrap_Dwg_MLINE_line_segparms_get},
{"LibreDWGc::Dwg_MLINE_line_num_areafillparms_set", _wrap_Dwg_MLINE_line_num_areafillparms_set},
{"LibreDWGc::Dwg_MLINE_line_num_areafillparms_get", _wrap_Dwg_MLINE_line_num_areafillparms_get},
{"LibreDWGc::Dwg_MLINE_line_areafillparms_set", _wrap_Dwg_MLINE_line_areafillparms_set},
{"LibreDWGc::Dwg_MLINE_line_areafillparms_get", _wrap_Dwg_MLINE_line_areafillparms_get},
{"LibreDWGc::new_Dwg_MLINE_line", _wrap_new_Dwg_MLINE_line},
{"LibreDWGc::delete_Dwg_MLINE_line", _wrap_delete_Dwg_MLINE_line},
{"LibreDWGc::Dwg_MLINE_vertex_parent_set", _wrap_Dwg_MLINE_vertex_parent_set},
{"LibreDWGc::Dwg_MLINE_vertex_parent_get", _wrap_Dwg_MLINE_vertex_parent_get},
{"LibreDWGc::Dwg_MLINE_vertex_vertex_set", _wrap_Dwg_MLINE_vertex_vertex_set},
{"LibreDWGc::Dwg_MLINE_vertex_vertex_get", _wrap_Dwg_MLINE_vertex_vertex_get},
{"LibreDWGc::Dwg_MLINE_vertex_vertex_direction_set", _wrap_Dwg_MLINE_vertex_vertex_direction_set},
{"LibreDWGc::Dwg_MLINE_vertex_vertex_direction_get", _wrap_Dwg_MLINE_vertex_vertex_direction_get},
{"LibreDWGc::Dwg_MLINE_vertex_miter_direction_set", _wrap_Dwg_MLINE_vertex_miter_direction_set},
{"LibreDWGc::Dwg_MLINE_vertex_miter_direction_get", _wrap_Dwg_MLINE_vertex_miter_direction_get},
{"LibreDWGc::Dwg_MLINE_vertex_num_lines_set", _wrap_Dwg_MLINE_vertex_num_lines_set},
{"LibreDWGc::Dwg_MLINE_vertex_num_lines_get", _wrap_Dwg_MLINE_vertex_num_lines_get},
{"LibreDWGc::Dwg_MLINE_vertex_lines_set", _wrap_Dwg_MLINE_vertex_lines_set},
{"LibreDWGc::Dwg_MLINE_vertex_lines_get", _wrap_Dwg_MLINE_vertex_lines_get},
{"LibreDWGc::new_Dwg_MLINE_vertex", _wrap_new_Dwg_MLINE_vertex},
{"LibreDWGc::delete_Dwg_MLINE_vertex", _wrap_delete_Dwg_MLINE_vertex},
{"LibreDWGc::Dwg_Entity_MLINE_parent_set", _wrap_Dwg_Entity_MLINE_parent_set},
{"LibreDWGc::Dwg_Entity_MLINE_parent_get", _wrap_Dwg_Entity_MLINE_parent_get},
{"LibreDWGc::Dwg_Entity_MLINE_scale_set", _wrap_Dwg_Entity_MLINE_scale_set},
{"LibreDWGc::Dwg_Entity_MLINE_scale_get", _wrap_Dwg_Entity_MLINE_scale_get},
{"LibreDWGc::Dwg_Entity_MLINE_justification_set", _wrap_Dwg_Entity_MLINE_justification_set},
{"LibreDWGc::Dwg_Entity_MLINE_justification_get", _wrap_Dwg_Entity_MLINE_justification_get},
{"LibreDWGc::Dwg_Entity_MLINE_base_point_set", _wrap_Dwg_Entity_MLINE_base_point_set},
{"LibreDWGc::Dwg_Entity_MLINE_base_point_get", _wrap_Dwg_Entity_MLINE_base_point_get},
{"LibreDWGc::Dwg_Entity_MLINE_extrusion_set", _wrap_Dwg_Entity_MLINE_extrusion_set},
{"LibreDWGc::Dwg_Entity_MLINE_extrusion_get", _wrap_Dwg_Entity_MLINE_extrusion_get},
{"LibreDWGc::Dwg_Entity_MLINE_flags_set", _wrap_Dwg_Entity_MLINE_flags_set},
{"LibreDWGc::Dwg_Entity_MLINE_flags_get", _wrap_Dwg_Entity_MLINE_flags_get},
{"LibreDWGc::Dwg_Entity_MLINE_num_lines_set", _wrap_Dwg_Entity_MLINE_num_lines_set},
{"LibreDWGc::Dwg_Entity_MLINE_num_lines_get", _wrap_Dwg_Entity_MLINE_num_lines_get},
{"LibreDWGc::Dwg_Entity_MLINE_num_verts_set", _wrap_Dwg_Entity_MLINE_num_verts_set},
{"LibreDWGc::Dwg_Entity_MLINE_num_verts_get", _wrap_Dwg_Entity_MLINE_num_verts_get},
{"LibreDWGc::Dwg_Entity_MLINE_verts_set", _wrap_Dwg_Entity_MLINE_verts_set},
{"LibreDWGc::Dwg_Entity_MLINE_verts_get", _wrap_Dwg_Entity_MLINE_verts_get},
{"LibreDWGc::Dwg_Entity_MLINE_mlinestyle_set", _wrap_Dwg_Entity_MLINE_mlinestyle_set},
{"LibreDWGc::Dwg_Entity_MLINE_mlinestyle_get", _wrap_Dwg_Entity_MLINE_mlinestyle_get},
{"LibreDWGc::new_Dwg_Entity_MLINE", _wrap_new_Dwg_Entity_MLINE},
{"LibreDWGc::delete_Dwg_Entity_MLINE", _wrap_delete_Dwg_Entity_MLINE},
{"LibreDWGc::Dwg_Object_BLOCK_CONTROL_parent_set", _wrap_Dwg_Object_BLOCK_CONTROL_parent_set},
{"LibreDWGc::Dwg_Object_BLOCK_CONTROL_parent_get", _wrap_Dwg_Object_BLOCK_CONTROL_parent_get},
{"LibreDWGc::Dwg_Object_BLOCK_CONTROL_num_entries_set", _wrap_Dwg_Object_BLOCK_CONTROL_num_entries_set},
{"LibreDWGc::Dwg_Object_BLOCK_CONTROL_num_entries_get", _wrap_Dwg_Object_BLOCK_CONTROL_num_entries_get},
{"LibreDWGc::Dwg_Object_BLOCK_CONTROL_entries_set", _wrap_Dwg_Object_BLOCK_CONTROL_entries_set},
{"LibreDWGc::Dwg_Object_BLOCK_CONTROL_entries_get", _wrap_Dwg_Object_BLOCK_CONTROL_entries_get},
{"LibreDWGc::Dwg_Object_BLOCK_CONTROL_flags_r11_set", _wrap_Dwg_Object_BLOCK_CONTROL_flags_r11_set},
{"LibreDWGc::Dwg_Object_BLOCK_CONTROL_flags_r11_get", _wrap_Dwg_Object_BLOCK_CONTROL_flags_r11_get},
{"LibreDWGc::Dwg_Object_BLOCK_CONTROL_model_space_set", _wrap_Dwg_Object_BLOCK_CONTROL_model_space_set},
{"LibreDWGc::Dwg_Object_BLOCK_CONTROL_model_space_get", _wrap_Dwg_Object_BLOCK_CONTROL_model_space_get},
{"LibreDWGc::Dwg_Object_BLOCK_CONTROL_paper_space_set", _wrap_Dwg_Object_BLOCK_CONTROL_paper_space_set},
{"LibreDWGc::Dwg_Object_BLOCK_CONTROL_paper_space_get", _wrap_Dwg_Object_BLOCK_CONTROL_paper_space_get},
{"LibreDWGc::new_Dwg_Object_BLOCK_CONTROL", _wrap_new_Dwg_Object_BLOCK_CONTROL},
{"LibreDWGc::delete_Dwg_Object_BLOCK_CONTROL", _wrap_delete_Dwg_Object_BLOCK_CONTROL},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_parent_set", _wrap_Dwg_Object_BLOCK_HEADER_parent_set},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_parent_get", _wrap_Dwg_Object_BLOCK_HEADER_parent_get},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_flag_set", _wrap_Dwg_Object_BLOCK_HEADER_flag_set},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_flag_get", _wrap_Dwg_Object_BLOCK_HEADER_flag_get},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_name_set", _wrap_Dwg_Object_BLOCK_HEADER_name_set},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_name_get", _wrap_Dwg_Object_BLOCK_HEADER_name_get},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_used_set", _wrap_Dwg_Object_BLOCK_HEADER_used_set},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_used_get", _wrap_Dwg_Object_BLOCK_HEADER_used_get},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_is_xref_ref_set", _wrap_Dwg_Object_BLOCK_HEADER_is_xref_ref_set},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_is_xref_ref_get", _wrap_Dwg_Object_BLOCK_HEADER_is_xref_ref_get},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_is_xref_resolved_set", _wrap_Dwg_Object_BLOCK_HEADER_is_xref_resolved_set},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_is_xref_resolved_get", _wrap_Dwg_Object_BLOCK_HEADER_is_xref_resolved_get},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_is_xref_dep_set", _wrap_Dwg_Object_BLOCK_HEADER_is_xref_dep_set},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_is_xref_dep_get", _wrap_Dwg_Object_BLOCK_HEADER_is_xref_dep_get},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_xref_set", _wrap_Dwg_Object_BLOCK_HEADER_xref_set},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_xref_get", _wrap_Dwg_Object_BLOCK_HEADER_xref_get},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER___iterator_set", _wrap_Dwg_Object_BLOCK_HEADER___iterator_set},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER___iterator_get", _wrap_Dwg_Object_BLOCK_HEADER___iterator_get},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_anonymous_set", _wrap_Dwg_Object_BLOCK_HEADER_anonymous_set},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_anonymous_get", _wrap_Dwg_Object_BLOCK_HEADER_anonymous_get},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_hasattrs_set", _wrap_Dwg_Object_BLOCK_HEADER_hasattrs_set},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_hasattrs_get", _wrap_Dwg_Object_BLOCK_HEADER_hasattrs_get},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_blkisxref_set", _wrap_Dwg_Object_BLOCK_HEADER_blkisxref_set},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_blkisxref_get", _wrap_Dwg_Object_BLOCK_HEADER_blkisxref_get},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_xrefoverlaid_set", _wrap_Dwg_Object_BLOCK_HEADER_xrefoverlaid_set},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_xrefoverlaid_get", _wrap_Dwg_Object_BLOCK_HEADER_xrefoverlaid_get},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_xref_loaded_set", _wrap_Dwg_Object_BLOCK_HEADER_xref_loaded_set},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_xref_loaded_get", _wrap_Dwg_Object_BLOCK_HEADER_xref_loaded_get},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_num_owned_set", _wrap_Dwg_Object_BLOCK_HEADER_num_owned_set},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_num_owned_get", _wrap_Dwg_Object_BLOCK_HEADER_num_owned_get},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_base_pt_set", _wrap_Dwg_Object_BLOCK_HEADER_base_pt_set},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_base_pt_get", _wrap_Dwg_Object_BLOCK_HEADER_base_pt_get},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_xref_pname_set", _wrap_Dwg_Object_BLOCK_HEADER_xref_pname_set},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_xref_pname_get", _wrap_Dwg_Object_BLOCK_HEADER_xref_pname_get},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_num_inserts_set", _wrap_Dwg_Object_BLOCK_HEADER_num_inserts_set},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_num_inserts_get", _wrap_Dwg_Object_BLOCK_HEADER_num_inserts_get},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_description_set", _wrap_Dwg_Object_BLOCK_HEADER_description_set},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_description_get", _wrap_Dwg_Object_BLOCK_HEADER_description_get},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_preview_size_set", _wrap_Dwg_Object_BLOCK_HEADER_preview_size_set},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_preview_size_get", _wrap_Dwg_Object_BLOCK_HEADER_preview_size_get},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_preview_set", _wrap_Dwg_Object_BLOCK_HEADER_preview_set},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_preview_get", _wrap_Dwg_Object_BLOCK_HEADER_preview_get},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_insert_units_set", _wrap_Dwg_Object_BLOCK_HEADER_insert_units_set},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_insert_units_get", _wrap_Dwg_Object_BLOCK_HEADER_insert_units_get},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_explodable_set", _wrap_Dwg_Object_BLOCK_HEADER_explodable_set},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_explodable_get", _wrap_Dwg_Object_BLOCK_HEADER_explodable_get},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_block_scaling_set", _wrap_Dwg_Object_BLOCK_HEADER_block_scaling_set},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_block_scaling_get", _wrap_Dwg_Object_BLOCK_HEADER_block_scaling_get},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_block_entity_set", _wrap_Dwg_Object_BLOCK_HEADER_block_entity_set},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_block_entity_get", _wrap_Dwg_Object_BLOCK_HEADER_block_entity_get},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_first_entity_set", _wrap_Dwg_Object_BLOCK_HEADER_first_entity_set},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_first_entity_get", _wrap_Dwg_Object_BLOCK_HEADER_first_entity_get},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_last_entity_set", _wrap_Dwg_Object_BLOCK_HEADER_last_entity_set},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_last_entity_get", _wrap_Dwg_Object_BLOCK_HEADER_last_entity_get},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_entities_set", _wrap_Dwg_Object_BLOCK_HEADER_entities_set},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_entities_get", _wrap_Dwg_Object_BLOCK_HEADER_entities_get},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_endblk_entity_set", _wrap_Dwg_Object_BLOCK_HEADER_endblk_entity_set},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_endblk_entity_get", _wrap_Dwg_Object_BLOCK_HEADER_endblk_entity_get},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_inserts_set", _wrap_Dwg_Object_BLOCK_HEADER_inserts_set},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_inserts_get", _wrap_Dwg_Object_BLOCK_HEADER_inserts_get},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_layout_set", _wrap_Dwg_Object_BLOCK_HEADER_layout_set},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_layout_get", _wrap_Dwg_Object_BLOCK_HEADER_layout_get},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_flag2_set", _wrap_Dwg_Object_BLOCK_HEADER_flag2_set},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_flag2_get", _wrap_Dwg_Object_BLOCK_HEADER_flag2_get},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_unknown_r11_set", _wrap_Dwg_Object_BLOCK_HEADER_unknown_r11_set},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_unknown_r11_get", _wrap_Dwg_Object_BLOCK_HEADER_unknown_r11_get},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_block_offset_r11_set", _wrap_Dwg_Object_BLOCK_HEADER_block_offset_r11_set},
{"LibreDWGc::Dwg_Object_BLOCK_HEADER_block_offset_r11_get", _wrap_Dwg_Object_BLOCK_HEADER_block_offset_r11_get},
{"LibreDWGc::new_Dwg_Object_BLOCK_HEADER", _wrap_new_Dwg_Object_BLOCK_HEADER},
{"LibreDWGc::delete_Dwg_Object_BLOCK_HEADER", _wrap_delete_Dwg_Object_BLOCK_HEADER},
{"LibreDWGc::Dwg_Object_LAYER_CONTROL_parent_set", _wrap_Dwg_Object_LAYER_CONTROL_parent_set},
{"LibreDWGc::Dwg_Object_LAYER_CONTROL_parent_get", _wrap_Dwg_Object_LAYER_CONTROL_parent_get},
{"LibreDWGc::Dwg_Object_LAYER_CONTROL_num_entries_set", _wrap_Dwg_Object_LAYER_CONTROL_num_entries_set},
{"LibreDWGc::Dwg_Object_LAYER_CONTROL_num_entries_get", _wrap_Dwg_Object_LAYER_CONTROL_num_entries_get},
{"LibreDWGc::Dwg_Object_LAYER_CONTROL_entries_set", _wrap_Dwg_Object_LAYER_CONTROL_entries_set},
{"LibreDWGc::Dwg_Object_LAYER_CONTROL_entries_get", _wrap_Dwg_Object_LAYER_CONTROL_entries_get},
{"LibreDWGc::Dwg_Object_LAYER_CONTROL_flags_r11_set", _wrap_Dwg_Object_LAYER_CONTROL_flags_r11_set},
{"LibreDWGc::Dwg_Object_LAYER_CONTROL_flags_r11_get", _wrap_Dwg_Object_LAYER_CONTROL_flags_r11_get},
{"LibreDWGc::new_Dwg_Object_LAYER_CONTROL", _wrap_new_Dwg_Object_LAYER_CONTROL},
{"LibreDWGc::delete_Dwg_Object_LAYER_CONTROL", _wrap_delete_Dwg_Object_LAYER_CONTROL},
{"LibreDWGc::Dwg_Object_LAYER_parent_set", _wrap_Dwg_Object_LAYER_parent_set},
{"LibreDWGc::Dwg_Object_LAYER_parent_get", _wrap_Dwg_Object_LAYER_parent_get},
{"LibreDWGc::Dwg_Object_LAYER_flag_set", _wrap_Dwg_Object_LAYER_flag_set},
{"LibreDWGc::Dwg_Object_LAYER_flag_get", _wrap_Dwg_Object_LAYER_flag_get},
{"LibreDWGc::Dwg_Object_LAYER_name_set", _wrap_Dwg_Object_LAYER_name_set},
{"LibreDWGc::Dwg_Object_LAYER_name_get", _wrap_Dwg_Object_LAYER_name_get},
{"LibreDWGc::Dwg_Object_LAYER_used_set", _wrap_Dwg_Object_LAYER_used_set},
{"LibreDWGc::Dwg_Object_LAYER_used_get", _wrap_Dwg_Object_LAYER_used_get},
{"LibreDWGc::Dwg_Object_LAYER_is_xref_ref_set", _wrap_Dwg_Object_LAYER_is_xref_ref_set},
{"LibreDWGc::Dwg_Object_LAYER_is_xref_ref_get", _wrap_Dwg_Object_LAYER_is_xref_ref_get},
{"LibreDWGc::Dwg_Object_LAYER_is_xref_resolved_set", _wrap_Dwg_Object_LAYER_is_xref_resolved_set},
{"LibreDWGc::Dwg_Object_LAYER_is_xref_resolved_get", _wrap_Dwg_Object_LAYER_is_xref_resolved_get},
{"LibreDWGc::Dwg_Object_LAYER_is_xref_dep_set", _wrap_Dwg_Object_LAYER_is_xref_dep_set},
{"LibreDWGc::Dwg_Object_LAYER_is_xref_dep_get", _wrap_Dwg_Object_LAYER_is_xref_dep_get},
{"LibreDWGc::Dwg_Object_LAYER_xref_set", _wrap_Dwg_Object_LAYER_xref_set},
{"LibreDWGc::Dwg_Object_LAYER_xref_get", _wrap_Dwg_Object_LAYER_xref_get},
{"LibreDWGc::Dwg_Object_LAYER_flag0_set", _wrap_Dwg_Object_LAYER_flag0_set},
{"LibreDWGc::Dwg_Object_LAYER_flag0_get", _wrap_Dwg_Object_LAYER_flag0_get},
{"LibreDWGc::Dwg_Object_LAYER_frozen_set", _wrap_Dwg_Object_LAYER_frozen_set},
{"LibreDWGc::Dwg_Object_LAYER_frozen_get", _wrap_Dwg_Object_LAYER_frozen_get},
{"LibreDWGc::Dwg_Object_LAYER_off_set", _wrap_Dwg_Object_LAYER_off_set},
{"LibreDWGc::Dwg_Object_LAYER_off_get", _wrap_Dwg_Object_LAYER_off_get},
{"LibreDWGc::Dwg_Object_LAYER_frozen_in_new_set", _wrap_Dwg_Object_LAYER_frozen_in_new_set},
{"LibreDWGc::Dwg_Object_LAYER_frozen_in_new_get", _wrap_Dwg_Object_LAYER_frozen_in_new_get},
{"LibreDWGc::Dwg_Object_LAYER_locked_set", _wrap_Dwg_Object_LAYER_locked_set},
{"LibreDWGc::Dwg_Object_LAYER_locked_get", _wrap_Dwg_Object_LAYER_locked_get},
{"LibreDWGc::Dwg_Object_LAYER_plotflag_set", _wrap_Dwg_Object_LAYER_plotflag_set},
{"LibreDWGc::Dwg_Object_LAYER_plotflag_get", _wrap_Dwg_Object_LAYER_plotflag_get},
{"LibreDWGc::Dwg_Object_LAYER_linewt_set", _wrap_Dwg_Object_LAYER_linewt_set},
{"LibreDWGc::Dwg_Object_LAYER_linewt_get", _wrap_Dwg_Object_LAYER_linewt_get},
{"LibreDWGc::Dwg_Object_LAYER_color_set", _wrap_Dwg_Object_LAYER_color_set},
{"LibreDWGc::Dwg_Object_LAYER_color_get", _wrap_Dwg_Object_LAYER_color_get},
{"LibreDWGc::Dwg_Object_LAYER_plotstyle_set", _wrap_Dwg_Object_LAYER_plotstyle_set},
{"LibreDWGc::Dwg_Object_LAYER_plotstyle_get", _wrap_Dwg_Object_LAYER_plotstyle_get},
{"LibreDWGc::Dwg_Object_LAYER_material_set", _wrap_Dwg_Object_LAYER_material_set},
{"LibreDWGc::Dwg_Object_LAYER_material_get", _wrap_Dwg_Object_LAYER_material_get},
{"LibreDWGc::Dwg_Object_LAYER_ltype_set", _wrap_Dwg_Object_LAYER_ltype_set},
{"LibreDWGc::Dwg_Object_LAYER_ltype_get", _wrap_Dwg_Object_LAYER_ltype_get},
{"LibreDWGc::Dwg_Object_LAYER_visualstyle_set", _wrap_Dwg_Object_LAYER_visualstyle_set},
{"LibreDWGc::Dwg_Object_LAYER_visualstyle_get", _wrap_Dwg_Object_LAYER_visualstyle_get},
{"LibreDWGc::new_Dwg_Object_LAYER", _wrap_new_Dwg_Object_LAYER},
{"LibreDWGc::delete_Dwg_Object_LAYER", _wrap_delete_Dwg_Object_LAYER},
{"LibreDWGc::Dwg_Object_STYLE_CONTROL_parent_set", _wrap_Dwg_Object_STYLE_CONTROL_parent_set},
{"LibreDWGc::Dwg_Object_STYLE_CONTROL_parent_get", _wrap_Dwg_Object_STYLE_CONTROL_parent_get},
{"LibreDWGc::Dwg_Object_STYLE_CONTROL_num_entries_set", _wrap_Dwg_Object_STYLE_CONTROL_num_entries_set},
{"LibreDWGc::Dwg_Object_STYLE_CONTROL_num_entries_get", _wrap_Dwg_Object_STYLE_CONTROL_num_entries_get},
{"LibreDWGc::Dwg_Object_STYLE_CONTROL_entries_set", _wrap_Dwg_Object_STYLE_CONTROL_entries_set},
{"LibreDWGc::Dwg_Object_STYLE_CONTROL_entries_get", _wrap_Dwg_Object_STYLE_CONTROL_entries_get},
{"LibreDWGc::Dwg_Object_STYLE_CONTROL_flags_r11_set", _wrap_Dwg_Object_STYLE_CONTROL_flags_r11_set},
{"LibreDWGc::Dwg_Object_STYLE_CONTROL_flags_r11_get", _wrap_Dwg_Object_STYLE_CONTROL_flags_r11_get},
{"LibreDWGc::new_Dwg_Object_STYLE_CONTROL", _wrap_new_Dwg_Object_STYLE_CONTROL},
{"LibreDWGc::delete_Dwg_Object_STYLE_CONTROL", _wrap_delete_Dwg_Object_STYLE_CONTROL},
{"LibreDWGc::Dwg_Object_STYLE_parent_set", _wrap_Dwg_Object_STYLE_parent_set},
{"LibreDWGc::Dwg_Object_STYLE_parent_get", _wrap_Dwg_Object_STYLE_parent_get},
{"LibreDWGc::Dwg_Object_STYLE_flag_set", _wrap_Dwg_Object_STYLE_flag_set},
{"LibreDWGc::Dwg_Object_STYLE_flag_get", _wrap_Dwg_Object_STYLE_flag_get},
{"LibreDWGc::Dwg_Object_STYLE_name_set", _wrap_Dwg_Object_STYLE_name_set},
{"LibreDWGc::Dwg_Object_STYLE_name_get", _wrap_Dwg_Object_STYLE_name_get},
{"LibreDWGc::Dwg_Object_STYLE_used_set", _wrap_Dwg_Object_STYLE_used_set},
{"LibreDWGc::Dwg_Object_STYLE_used_get", _wrap_Dwg_Object_STYLE_used_get},
{"LibreDWGc::Dwg_Object_STYLE_is_xref_ref_set", _wrap_Dwg_Object_STYLE_is_xref_ref_set},
{"LibreDWGc::Dwg_Object_STYLE_is_xref_ref_get", _wrap_Dwg_Object_STYLE_is_xref_ref_get},
{"LibreDWGc::Dwg_Object_STYLE_is_xref_resolved_set", _wrap_Dwg_Object_STYLE_is_xref_resolved_set},
{"LibreDWGc::Dwg_Object_STYLE_is_xref_resolved_get", _wrap_Dwg_Object_STYLE_is_xref_resolved_get},
{"LibreDWGc::Dwg_Object_STYLE_is_xref_dep_set", _wrap_Dwg_Object_STYLE_is_xref_dep_set},
{"LibreDWGc::Dwg_Object_STYLE_is_xref_dep_get", _wrap_Dwg_Object_STYLE_is_xref_dep_get},
{"LibreDWGc::Dwg_Object_STYLE_xref_set", _wrap_Dwg_Object_STYLE_xref_set},
{"LibreDWGc::Dwg_Object_STYLE_xref_get", _wrap_Dwg_Object_STYLE_xref_get},
{"LibreDWGc::Dwg_Object_STYLE_is_shape_set", _wrap_Dwg_Object_STYLE_is_shape_set},
{"LibreDWGc::Dwg_Object_STYLE_is_shape_get", _wrap_Dwg_Object_STYLE_is_shape_get},
{"LibreDWGc::Dwg_Object_STYLE_is_vertical_set", _wrap_Dwg_Object_STYLE_is_vertical_set},
{"LibreDWGc::Dwg_Object_STYLE_is_vertical_get", _wrap_Dwg_Object_STYLE_is_vertical_get},
{"LibreDWGc::Dwg_Object_STYLE_text_size_set", _wrap_Dwg_Object_STYLE_text_size_set},
{"LibreDWGc::Dwg_Object_STYLE_text_size_get", _wrap_Dwg_Object_STYLE_text_size_get},
{"LibreDWGc::Dwg_Object_STYLE_width_factor_set", _wrap_Dwg_Object_STYLE_width_factor_set},
{"LibreDWGc::Dwg_Object_STYLE_width_factor_get", _wrap_Dwg_Object_STYLE_width_factor_get},
{"LibreDWGc::Dwg_Object_STYLE_oblique_angle_set", _wrap_Dwg_Object_STYLE_oblique_angle_set},
{"LibreDWGc::Dwg_Object_STYLE_oblique_angle_get", _wrap_Dwg_Object_STYLE_oblique_angle_get},
{"LibreDWGc::Dwg_Object_STYLE_generation_set", _wrap_Dwg_Object_STYLE_generation_set},
{"LibreDWGc::Dwg_Object_STYLE_generation_get", _wrap_Dwg_Object_STYLE_generation_get},
{"LibreDWGc::Dwg_Object_STYLE_last_height_set", _wrap_Dwg_Object_STYLE_last_height_set},
{"LibreDWGc::Dwg_Object_STYLE_last_height_get", _wrap_Dwg_Object_STYLE_last_height_get},
{"LibreDWGc::Dwg_Object_STYLE_font_file_set", _wrap_Dwg_Object_STYLE_font_file_set},
{"LibreDWGc::Dwg_Object_STYLE_font_file_get", _wrap_Dwg_Object_STYLE_font_file_get},
{"LibreDWGc::Dwg_Object_STYLE_bigfont_file_set", _wrap_Dwg_Object_STYLE_bigfont_file_set},
{"LibreDWGc::Dwg_Object_STYLE_bigfont_file_get", _wrap_Dwg_Object_STYLE_bigfont_file_get},
{"LibreDWGc::new_Dwg_Object_STYLE", _wrap_new_Dwg_Object_STYLE},
{"LibreDWGc::delete_Dwg_Object_STYLE", _wrap_delete_Dwg_Object_STYLE},
{"LibreDWGc::Dwg_Object_LTYPE_CONTROL_parent_set", _wrap_Dwg_Object_LTYPE_CONTROL_parent_set},
{"LibreDWGc::Dwg_Object_LTYPE_CONTROL_parent_get", _wrap_Dwg_Object_LTYPE_CONTROL_parent_get},
{"LibreDWGc::Dwg_Object_LTYPE_CONTROL_num_entries_set", _wrap_Dwg_Object_LTYPE_CONTROL_num_entries_set},
{"LibreDWGc::Dwg_Object_LTYPE_CONTROL_num_entries_get", _wrap_Dwg_Object_LTYPE_CONTROL_num_entries_get},
{"LibreDWGc::Dwg_Object_LTYPE_CONTROL_entries_set", _wrap_Dwg_Object_LTYPE_CONTROL_entries_set},
{"LibreDWGc::Dwg_Object_LTYPE_CONTROL_entries_get", _wrap_Dwg_Object_LTYPE_CONTROL_entries_get},
{"LibreDWGc::Dwg_Object_LTYPE_CONTROL_flags_r11_set", _wrap_Dwg_Object_LTYPE_CONTROL_flags_r11_set},
{"LibreDWGc::Dwg_Object_LTYPE_CONTROL_flags_r11_get", _wrap_Dwg_Object_LTYPE_CONTROL_flags_r11_get},
{"LibreDWGc::Dwg_Object_LTYPE_CONTROL_bylayer_set", _wrap_Dwg_Object_LTYPE_CONTROL_bylayer_set},
{"LibreDWGc::Dwg_Object_LTYPE_CONTROL_bylayer_get", _wrap_Dwg_Object_LTYPE_CONTROL_bylayer_get},
{"LibreDWGc::Dwg_Object_LTYPE_CONTROL_byblock_set", _wrap_Dwg_Object_LTYPE_CONTROL_byblock_set},
{"LibreDWGc::Dwg_Object_LTYPE_CONTROL_byblock_get", _wrap_Dwg_Object_LTYPE_CONTROL_byblock_get},
{"LibreDWGc::new_Dwg_Object_LTYPE_CONTROL", _wrap_new_Dwg_Object_LTYPE_CONTROL},
{"LibreDWGc::delete_Dwg_Object_LTYPE_CONTROL", _wrap_delete_Dwg_Object_LTYPE_CONTROL},
{"LibreDWGc::Dwg_LTYPE_dash_parent_set", _wrap_Dwg_LTYPE_dash_parent_set},
{"LibreDWGc::Dwg_LTYPE_dash_parent_get", _wrap_Dwg_LTYPE_dash_parent_get},
{"LibreDWGc::Dwg_LTYPE_dash_length_set", _wrap_Dwg_LTYPE_dash_length_set},
{"LibreDWGc::Dwg_LTYPE_dash_length_get", _wrap_Dwg_LTYPE_dash_length_get},
{"LibreDWGc::Dwg_LTYPE_dash_complex_shapecode_set", _wrap_Dwg_LTYPE_dash_complex_shapecode_set},
{"LibreDWGc::Dwg_LTYPE_dash_complex_shapecode_get", _wrap_Dwg_LTYPE_dash_complex_shapecode_get},
{"LibreDWGc::Dwg_LTYPE_dash_style_set", _wrap_Dwg_LTYPE_dash_style_set},
{"LibreDWGc::Dwg_LTYPE_dash_style_get", _wrap_Dwg_LTYPE_dash_style_get},
{"LibreDWGc::Dwg_LTYPE_dash_x_offset_set", _wrap_Dwg_LTYPE_dash_x_offset_set},
{"LibreDWGc::Dwg_LTYPE_dash_x_offset_get", _wrap_Dwg_LTYPE_dash_x_offset_get},
{"LibreDWGc::Dwg_LTYPE_dash_y_offset_set", _wrap_Dwg_LTYPE_dash_y_offset_set},
{"LibreDWGc::Dwg_LTYPE_dash_y_offset_get", _wrap_Dwg_LTYPE_dash_y_offset_get},
{"LibreDWGc::Dwg_LTYPE_dash_scale_set", _wrap_Dwg_LTYPE_dash_scale_set},
{"LibreDWGc::Dwg_LTYPE_dash_scale_get", _wrap_Dwg_LTYPE_dash_scale_get},
{"LibreDWGc::Dwg_LTYPE_dash_rotation_set", _wrap_Dwg_LTYPE_dash_rotation_set},
{"LibreDWGc::Dwg_LTYPE_dash_rotation_get", _wrap_Dwg_LTYPE_dash_rotation_get},
{"LibreDWGc::Dwg_LTYPE_dash_shape_flag_set", _wrap_Dwg_LTYPE_dash_shape_flag_set},
{"LibreDWGc::Dwg_LTYPE_dash_shape_flag_get", _wrap_Dwg_LTYPE_dash_shape_flag_get},
{"LibreDWGc::Dwg_LTYPE_dash_text_set", _wrap_Dwg_LTYPE_dash_text_set},
{"LibreDWGc::Dwg_LTYPE_dash_text_get", _wrap_Dwg_LTYPE_dash_text_get},
{"LibreDWGc::new_Dwg_LTYPE_dash", _wrap_new_Dwg_LTYPE_dash},
{"LibreDWGc::delete_Dwg_LTYPE_dash", _wrap_delete_Dwg_LTYPE_dash},
{"LibreDWGc::Dwg_Object_LTYPE_parent_set", _wrap_Dwg_Object_LTYPE_parent_set},
{"LibreDWGc::Dwg_Object_LTYPE_parent_get", _wrap_Dwg_Object_LTYPE_parent_get},
{"LibreDWGc::Dwg_Object_LTYPE_flag_set", _wrap_Dwg_Object_LTYPE_flag_set},
{"LibreDWGc::Dwg_Object_LTYPE_flag_get", _wrap_Dwg_Object_LTYPE_flag_get},
{"LibreDWGc::Dwg_Object_LTYPE_name_set", _wrap_Dwg_Object_LTYPE_name_set},
{"LibreDWGc::Dwg_Object_LTYPE_name_get", _wrap_Dwg_Object_LTYPE_name_get},
{"LibreDWGc::Dwg_Object_LTYPE_used_set", _wrap_Dwg_Object_LTYPE_used_set},
{"LibreDWGc::Dwg_Object_LTYPE_used_get", _wrap_Dwg_Object_LTYPE_used_get},
{"LibreDWGc::Dwg_Object_LTYPE_is_xref_ref_set", _wrap_Dwg_Object_LTYPE_is_xref_ref_set},
{"LibreDWGc::Dwg_Object_LTYPE_is_xref_ref_get", _wrap_Dwg_Object_LTYPE_is_xref_ref_get},
{"LibreDWGc::Dwg_Object_LTYPE_is_xref_resolved_set", _wrap_Dwg_Object_LTYPE_is_xref_resolved_set},
{"LibreDWGc::Dwg_Object_LTYPE_is_xref_resolved_get", _wrap_Dwg_Object_LTYPE_is_xref_resolved_get},
{"LibreDWGc::Dwg_Object_LTYPE_is_xref_dep_set", _wrap_Dwg_Object_LTYPE_is_xref_dep_set},
{"LibreDWGc::Dwg_Object_LTYPE_is_xref_dep_get", _wrap_Dwg_Object_LTYPE_is_xref_dep_get},
{"LibreDWGc::Dwg_Object_LTYPE_xref_set", _wrap_Dwg_Object_LTYPE_xref_set},
{"LibreDWGc::Dwg_Object_LTYPE_xref_get", _wrap_Dwg_Object_LTYPE_xref_get},
{"LibreDWGc::Dwg_Object_LTYPE_description_set", _wrap_Dwg_Object_LTYPE_description_set},
{"LibreDWGc::Dwg_Object_LTYPE_description_get", _wrap_Dwg_Object_LTYPE_description_get},
{"LibreDWGc::Dwg_Object_LTYPE_pattern_len_set", _wrap_Dwg_Object_LTYPE_pattern_len_set},
{"LibreDWGc::Dwg_Object_LTYPE_pattern_len_get", _wrap_Dwg_Object_LTYPE_pattern_len_get},
{"LibreDWGc::Dwg_Object_LTYPE_alignment_set", _wrap_Dwg_Object_LTYPE_alignment_set},
{"LibreDWGc::Dwg_Object_LTYPE_alignment_get", _wrap_Dwg_Object_LTYPE_alignment_get},
{"LibreDWGc::Dwg_Object_LTYPE_numdashes_set", _wrap_Dwg_Object_LTYPE_numdashes_set},
{"LibreDWGc::Dwg_Object_LTYPE_numdashes_get", _wrap_Dwg_Object_LTYPE_numdashes_get},
{"LibreDWGc::Dwg_Object_LTYPE_dashes_set", _wrap_Dwg_Object_LTYPE_dashes_set},
{"LibreDWGc::Dwg_Object_LTYPE_dashes_get", _wrap_Dwg_Object_LTYPE_dashes_get},
{"LibreDWGc::Dwg_Object_LTYPE_dashes_r11_set", _wrap_Dwg_Object_LTYPE_dashes_r11_set},
{"LibreDWGc::Dwg_Object_LTYPE_dashes_r11_get", _wrap_Dwg_Object_LTYPE_dashes_r11_get},
{"LibreDWGc::Dwg_Object_LTYPE_has_strings_area_set", _wrap_Dwg_Object_LTYPE_has_strings_area_set},
{"LibreDWGc::Dwg_Object_LTYPE_has_strings_area_get", _wrap_Dwg_Object_LTYPE_has_strings_area_get},
{"LibreDWGc::Dwg_Object_LTYPE_strings_area_set", _wrap_Dwg_Object_LTYPE_strings_area_set},
{"LibreDWGc::Dwg_Object_LTYPE_strings_area_get", _wrap_Dwg_Object_LTYPE_strings_area_get},
{"LibreDWGc::Dwg_Object_LTYPE_unknown_r11_set", _wrap_Dwg_Object_LTYPE_unknown_r11_set},
{"LibreDWGc::Dwg_Object_LTYPE_unknown_r11_get", _wrap_Dwg_Object_LTYPE_unknown_r11_get},
{"LibreDWGc::new_Dwg_Object_LTYPE", _wrap_new_Dwg_Object_LTYPE},
{"LibreDWGc::delete_Dwg_Object_LTYPE", _wrap_delete_Dwg_Object_LTYPE},
{"LibreDWGc::Dwg_Object_VIEW_CONTROL_parent_set", _wrap_Dwg_Object_VIEW_CONTROL_parent_set},
{"LibreDWGc::Dwg_Object_VIEW_CONTROL_parent_get", _wrap_Dwg_Object_VIEW_CONTROL_parent_get},
{"LibreDWGc::Dwg_Object_VIEW_CONTROL_num_entries_set", _wrap_Dwg_Object_VIEW_CONTROL_num_entries_set},
{"LibreDWGc::Dwg_Object_VIEW_CONTROL_num_entries_get", _wrap_Dwg_Object_VIEW_CONTROL_num_entries_get},
{"LibreDWGc::Dwg_Object_VIEW_CONTROL_entries_set", _wrap_Dwg_Object_VIEW_CONTROL_entries_set},
{"LibreDWGc::Dwg_Object_VIEW_CONTROL_entries_get", _wrap_Dwg_Object_VIEW_CONTROL_entries_get},
{"LibreDWGc::Dwg_Object_VIEW_CONTROL_flags_r11_set", _wrap_Dwg_Object_VIEW_CONTROL_flags_r11_set},
{"LibreDWGc::Dwg_Object_VIEW_CONTROL_flags_r11_get", _wrap_Dwg_Object_VIEW_CONTROL_flags_r11_get},
{"LibreDWGc::new_Dwg_Object_VIEW_CONTROL", _wrap_new_Dwg_Object_VIEW_CONTROL},
{"LibreDWGc::delete_Dwg_Object_VIEW_CONTROL", _wrap_delete_Dwg_Object_VIEW_CONTROL},
{"LibreDWGc::Dwg_Object_VIEW_parent_set", _wrap_Dwg_Object_VIEW_parent_set},
{"LibreDWGc::Dwg_Object_VIEW_parent_get", _wrap_Dwg_Object_VIEW_parent_get},
{"LibreDWGc::Dwg_Object_VIEW_flag_set", _wrap_Dwg_Object_VIEW_flag_set},
{"LibreDWGc::Dwg_Object_VIEW_flag_get", _wrap_Dwg_Object_VIEW_flag_get},
{"LibreDWGc::Dwg_Object_VIEW_name_set", _wrap_Dwg_Object_VIEW_name_set},
{"LibreDWGc::Dwg_Object_VIEW_name_get", _wrap_Dwg_Object_VIEW_name_get},
{"LibreDWGc::Dwg_Object_VIEW_used_set", _wrap_Dwg_Object_VIEW_used_set},
{"LibreDWGc::Dwg_Object_VIEW_used_get", _wrap_Dwg_Object_VIEW_used_get},
{"LibreDWGc::Dwg_Object_VIEW_is_xref_ref_set", _wrap_Dwg_Object_VIEW_is_xref_ref_set},
{"LibreDWGc::Dwg_Object_VIEW_is_xref_ref_get", _wrap_Dwg_Object_VIEW_is_xref_ref_get},
{"LibreDWGc::Dwg_Object_VIEW_is_xref_resolved_set", _wrap_Dwg_Object_VIEW_is_xref_resolved_set},
{"LibreDWGc::Dwg_Object_VIEW_is_xref_resolved_get", _wrap_Dwg_Object_VIEW_is_xref_resolved_get},
{"LibreDWGc::Dwg_Object_VIEW_is_xref_dep_set", _wrap_Dwg_Object_VIEW_is_xref_dep_set},
{"LibreDWGc::Dwg_Object_VIEW_is_xref_dep_get", _wrap_Dwg_Object_VIEW_is_xref_dep_get},
{"LibreDWGc::Dwg_Object_VIEW_xref_set", _wrap_Dwg_Object_VIEW_xref_set},
{"LibreDWGc::Dwg_Object_VIEW_xref_get", _wrap_Dwg_Object_VIEW_xref_get},
{"LibreDWGc::Dwg_Object_VIEW_VIEWSIZE_set", _wrap_Dwg_Object_VIEW_VIEWSIZE_set},
{"LibreDWGc::Dwg_Object_VIEW_VIEWSIZE_get", _wrap_Dwg_Object_VIEW_VIEWSIZE_get},
{"LibreDWGc::Dwg_Object_VIEW_view_width_set", _wrap_Dwg_Object_VIEW_view_width_set},
{"LibreDWGc::Dwg_Object_VIEW_view_width_get", _wrap_Dwg_Object_VIEW_view_width_get},
{"LibreDWGc::Dwg_Object_VIEW_aspect_ratio_set", _wrap_Dwg_Object_VIEW_aspect_ratio_set},
{"LibreDWGc::Dwg_Object_VIEW_aspect_ratio_get", _wrap_Dwg_Object_VIEW_aspect_ratio_get},
{"LibreDWGc::Dwg_Object_VIEW_VIEWCTR_set", _wrap_Dwg_Object_VIEW_VIEWCTR_set},
{"LibreDWGc::Dwg_Object_VIEW_VIEWCTR_get", _wrap_Dwg_Object_VIEW_VIEWCTR_get},
{"LibreDWGc::Dwg_Object_VIEW_view_target_set", _wrap_Dwg_Object_VIEW_view_target_set},
{"LibreDWGc::Dwg_Object_VIEW_view_target_get", _wrap_Dwg_Object_VIEW_view_target_get},
{"LibreDWGc::Dwg_Object_VIEW_VIEWDIR_set", _wrap_Dwg_Object_VIEW_VIEWDIR_set},
{"LibreDWGc::Dwg_Object_VIEW_VIEWDIR_get", _wrap_Dwg_Object_VIEW_VIEWDIR_get},
{"LibreDWGc::Dwg_Object_VIEW_twist_angle_set", _wrap_Dwg_Object_VIEW_twist_angle_set},
{"LibreDWGc::Dwg_Object_VIEW_twist_angle_get", _wrap_Dwg_Object_VIEW_twist_angle_get},
{"LibreDWGc::Dwg_Object_VIEW_lens_length_set", _wrap_Dwg_Object_VIEW_lens_length_set},
{"LibreDWGc::Dwg_Object_VIEW_lens_length_get", _wrap_Dwg_Object_VIEW_lens_length_get},
{"LibreDWGc::Dwg_Object_VIEW_front_clip_z_set", _wrap_Dwg_Object_VIEW_front_clip_z_set},
{"LibreDWGc::Dwg_Object_VIEW_front_clip_z_get", _wrap_Dwg_Object_VIEW_front_clip_z_get},
{"LibreDWGc::Dwg_Object_VIEW_back_clip_z_set", _wrap_Dwg_Object_VIEW_back_clip_z_set},
{"LibreDWGc::Dwg_Object_VIEW_back_clip_z_get", _wrap_Dwg_Object_VIEW_back_clip_z_get},
{"LibreDWGc::Dwg_Object_VIEW_VIEWMODE_set", _wrap_Dwg_Object_VIEW_VIEWMODE_set},
{"LibreDWGc::Dwg_Object_VIEW_VIEWMODE_get", _wrap_Dwg_Object_VIEW_VIEWMODE_get},
{"LibreDWGc::Dwg_Object_VIEW_render_mode_set", _wrap_Dwg_Object_VIEW_render_mode_set},
{"LibreDWGc::Dwg_Object_VIEW_render_mode_get", _wrap_Dwg_Object_VIEW_render_mode_get},
{"LibreDWGc::Dwg_Object_VIEW_use_default_lights_set", _wrap_Dwg_Object_VIEW_use_default_lights_set},
{"LibreDWGc::Dwg_Object_VIEW_use_default_lights_get", _wrap_Dwg_Object_VIEW_use_default_lights_get},
{"LibreDWGc::Dwg_Object_VIEW_default_lightning_type_set", _wrap_Dwg_Object_VIEW_default_lightning_type_set},
{"LibreDWGc::Dwg_Object_VIEW_default_lightning_type_get", _wrap_Dwg_Object_VIEW_default_lightning_type_get},
{"LibreDWGc::Dwg_Object_VIEW_brightness_set", _wrap_Dwg_Object_VIEW_brightness_set},
{"LibreDWGc::Dwg_Object_VIEW_brightness_get", _wrap_Dwg_Object_VIEW_brightness_get},
{"LibreDWGc::Dwg_Object_VIEW_contrast_set", _wrap_Dwg_Object_VIEW_contrast_set},
{"LibreDWGc::Dwg_Object_VIEW_contrast_get", _wrap_Dwg_Object_VIEW_contrast_get},
{"LibreDWGc::Dwg_Object_VIEW_ambient_color_set", _wrap_Dwg_Object_VIEW_ambient_color_set},
{"LibreDWGc::Dwg_Object_VIEW_ambient_color_get", _wrap_Dwg_Object_VIEW_ambient_color_get},
{"LibreDWGc::Dwg_Object_VIEW_is_pspace_set", _wrap_Dwg_Object_VIEW_is_pspace_set},
{"LibreDWGc::Dwg_Object_VIEW_is_pspace_get", _wrap_Dwg_Object_VIEW_is_pspace_get},
{"LibreDWGc::Dwg_Object_VIEW_associated_ucs_set", _wrap_Dwg_Object_VIEW_associated_ucs_set},
{"LibreDWGc::Dwg_Object_VIEW_associated_ucs_get", _wrap_Dwg_Object_VIEW_associated_ucs_get},
{"LibreDWGc::Dwg_Object_VIEW_ucsorg_set", _wrap_Dwg_Object_VIEW_ucsorg_set},
{"LibreDWGc::Dwg_Object_VIEW_ucsorg_get", _wrap_Dwg_Object_VIEW_ucsorg_get},
{"LibreDWGc::Dwg_Object_VIEW_ucsxdir_set", _wrap_Dwg_Object_VIEW_ucsxdir_set},
{"LibreDWGc::Dwg_Object_VIEW_ucsxdir_get", _wrap_Dwg_Object_VIEW_ucsxdir_get},
{"LibreDWGc::Dwg_Object_VIEW_ucsydir_set", _wrap_Dwg_Object_VIEW_ucsydir_set},
{"LibreDWGc::Dwg_Object_VIEW_ucsydir_get", _wrap_Dwg_Object_VIEW_ucsydir_get},
{"LibreDWGc::Dwg_Object_VIEW_ucs_elevation_set", _wrap_Dwg_Object_VIEW_ucs_elevation_set},
{"LibreDWGc::Dwg_Object_VIEW_ucs_elevation_get", _wrap_Dwg_Object_VIEW_ucs_elevation_get},
{"LibreDWGc::Dwg_Object_VIEW_UCSORTHOVIEW_set", _wrap_Dwg_Object_VIEW_UCSORTHOVIEW_set},
{"LibreDWGc::Dwg_Object_VIEW_UCSORTHOVIEW_get", _wrap_Dwg_Object_VIEW_UCSORTHOVIEW_get},
{"LibreDWGc::Dwg_Object_VIEW_is_camera_plottable_set", _wrap_Dwg_Object_VIEW_is_camera_plottable_set},
{"LibreDWGc::Dwg_Object_VIEW_is_camera_plottable_get", _wrap_Dwg_Object_VIEW_is_camera_plottable_get},
{"LibreDWGc::Dwg_Object_VIEW_background_set", _wrap_Dwg_Object_VIEW_background_set},
{"LibreDWGc::Dwg_Object_VIEW_background_get", _wrap_Dwg_Object_VIEW_background_get},
{"LibreDWGc::Dwg_Object_VIEW_visualstyle_set", _wrap_Dwg_Object_VIEW_visualstyle_set},
{"LibreDWGc::Dwg_Object_VIEW_visualstyle_get", _wrap_Dwg_Object_VIEW_visualstyle_get},
{"LibreDWGc::Dwg_Object_VIEW_sun_set", _wrap_Dwg_Object_VIEW_sun_set},
{"LibreDWGc::Dwg_Object_VIEW_sun_get", _wrap_Dwg_Object_VIEW_sun_get},
{"LibreDWGc::Dwg_Object_VIEW_base_ucs_set", _wrap_Dwg_Object_VIEW_base_ucs_set},
{"LibreDWGc::Dwg_Object_VIEW_base_ucs_get", _wrap_Dwg_Object_VIEW_base_ucs_get},
{"LibreDWGc::Dwg_Object_VIEW_named_ucs_set", _wrap_Dwg_Object_VIEW_named_ucs_set},
{"LibreDWGc::Dwg_Object_VIEW_named_ucs_get", _wrap_Dwg_Object_VIEW_named_ucs_get},
{"LibreDWGc::Dwg_Object_VIEW_livesection_set", _wrap_Dwg_Object_VIEW_livesection_set},
{"LibreDWGc::Dwg_Object_VIEW_livesection_get", _wrap_Dwg_Object_VIEW_livesection_get},
{"LibreDWGc::Dwg_Object_VIEW_flag_3d_set", _wrap_Dwg_Object_VIEW_flag_3d_set},
{"LibreDWGc::Dwg_Object_VIEW_flag_3d_get", _wrap_Dwg_Object_VIEW_flag_3d_get},
{"LibreDWGc::Dwg_Object_VIEW_unknown_r2_set", _wrap_Dwg_Object_VIEW_unknown_r2_set},
{"LibreDWGc::Dwg_Object_VIEW_unknown_r2_get", _wrap_Dwg_Object_VIEW_unknown_r2_get},
{"LibreDWGc::Dwg_Object_VIEW_unknown_r11_set", _wrap_Dwg_Object_VIEW_unknown_r11_set},
{"LibreDWGc::Dwg_Object_VIEW_unknown_r11_get", _wrap_Dwg_Object_VIEW_unknown_r11_get},
{"LibreDWGc::new_Dwg_Object_VIEW", _wrap_new_Dwg_Object_VIEW},
{"LibreDWGc::delete_Dwg_Object_VIEW", _wrap_delete_Dwg_Object_VIEW},
{"LibreDWGc::Dwg_Object_UCS_CONTROL_parent_set", _wrap_Dwg_Object_UCS_CONTROL_parent_set},
{"LibreDWGc::Dwg_Object_UCS_CONTROL_parent_get", _wrap_Dwg_Object_UCS_CONTROL_parent_get},
{"LibreDWGc::Dwg_Object_UCS_CONTROL_num_entries_set", _wrap_Dwg_Object_UCS_CONTROL_num_entries_set},
{"LibreDWGc::Dwg_Object_UCS_CONTROL_num_entries_get", _wrap_Dwg_Object_UCS_CONTROL_num_entries_get},
{"LibreDWGc::Dwg_Object_UCS_CONTROL_entries_set", _wrap_Dwg_Object_UCS_CONTROL_entries_set},
{"LibreDWGc::Dwg_Object_UCS_CONTROL_entries_get", _wrap_Dwg_Object_UCS_CONTROL_entries_get},
{"LibreDWGc::Dwg_Object_UCS_CONTROL_flags_r11_set", _wrap_Dwg_Object_UCS_CONTROL_flags_r11_set},
{"LibreDWGc::Dwg_Object_UCS_CONTROL_flags_r11_get", _wrap_Dwg_Object_UCS_CONTROL_flags_r11_get},
{"LibreDWGc::new_Dwg_Object_UCS_CONTROL", _wrap_new_Dwg_Object_UCS_CONTROL},
{"LibreDWGc::delete_Dwg_Object_UCS_CONTROL", _wrap_delete_Dwg_Object_UCS_CONTROL},
{"LibreDWGc::Dwg_UCS_orthopts_parent_set", _wrap_Dwg_UCS_orthopts_parent_set},
{"LibreDWGc::Dwg_UCS_orthopts_parent_get", _wrap_Dwg_UCS_orthopts_parent_get},
{"LibreDWGc::Dwg_UCS_orthopts_type_set", _wrap_Dwg_UCS_orthopts_type_set},
{"LibreDWGc::Dwg_UCS_orthopts_type_get", _wrap_Dwg_UCS_orthopts_type_get},
{"LibreDWGc::Dwg_UCS_orthopts_pt_set", _wrap_Dwg_UCS_orthopts_pt_set},
{"LibreDWGc::Dwg_UCS_orthopts_pt_get", _wrap_Dwg_UCS_orthopts_pt_get},
{"LibreDWGc::new_Dwg_UCS_orthopts", _wrap_new_Dwg_UCS_orthopts},
{"LibreDWGc::delete_Dwg_UCS_orthopts", _wrap_delete_Dwg_UCS_orthopts},
{"LibreDWGc::Dwg_Object_UCS_parent_set", _wrap_Dwg_Object_UCS_parent_set},
{"LibreDWGc::Dwg_Object_UCS_parent_get", _wrap_Dwg_Object_UCS_parent_get},
{"LibreDWGc::Dwg_Object_UCS_flag_set", _wrap_Dwg_Object_UCS_flag_set},
{"LibreDWGc::Dwg_Object_UCS_flag_get", _wrap_Dwg_Object_UCS_flag_get},
{"LibreDWGc::Dwg_Object_UCS_name_set", _wrap_Dwg_Object_UCS_name_set},
{"LibreDWGc::Dwg_Object_UCS_name_get", _wrap_Dwg_Object_UCS_name_get},
{"LibreDWGc::Dwg_Object_UCS_used_set", _wrap_Dwg_Object_UCS_used_set},
{"LibreDWGc::Dwg_Object_UCS_used_get", _wrap_Dwg_Object_UCS_used_get},
{"LibreDWGc::Dwg_Object_UCS_is_xref_ref_set", _wrap_Dwg_Object_UCS_is_xref_ref_set},
{"LibreDWGc::Dwg_Object_UCS_is_xref_ref_get", _wrap_Dwg_Object_UCS_is_xref_ref_get},
{"LibreDWGc::Dwg_Object_UCS_is_xref_resolved_set", _wrap_Dwg_Object_UCS_is_xref_resolved_set},
{"LibreDWGc::Dwg_Object_UCS_is_xref_resolved_get", _wrap_Dwg_Object_UCS_is_xref_resolved_get},
{"LibreDWGc::Dwg_Object_UCS_is_xref_dep_set", _wrap_Dwg_Object_UCS_is_xref_dep_set},
{"LibreDWGc::Dwg_Object_UCS_is_xref_dep_get", _wrap_Dwg_Object_UCS_is_xref_dep_get},
{"LibreDWGc::Dwg_Object_UCS_xref_set", _wrap_Dwg_Object_UCS_xref_set},
{"LibreDWGc::Dwg_Object_UCS_xref_get", _wrap_Dwg_Object_UCS_xref_get},
{"LibreDWGc::Dwg_Object_UCS_ucsorg_set", _wrap_Dwg_Object_UCS_ucsorg_set},
{"LibreDWGc::Dwg_Object_UCS_ucsorg_get", _wrap_Dwg_Object_UCS_ucsorg_get},
{"LibreDWGc::Dwg_Object_UCS_ucsxdir_set", _wrap_Dwg_Object_UCS_ucsxdir_set},
{"LibreDWGc::Dwg_Object_UCS_ucsxdir_get", _wrap_Dwg_Object_UCS_ucsxdir_get},
{"LibreDWGc::Dwg_Object_UCS_ucsydir_set", _wrap_Dwg_Object_UCS_ucsydir_set},
{"LibreDWGc::Dwg_Object_UCS_ucsydir_get", _wrap_Dwg_Object_UCS_ucsydir_get},
{"LibreDWGc::Dwg_Object_UCS_ucs_elevation_set", _wrap_Dwg_Object_UCS_ucs_elevation_set},
{"LibreDWGc::Dwg_Object_UCS_ucs_elevation_get", _wrap_Dwg_Object_UCS_ucs_elevation_get},
{"LibreDWGc::Dwg_Object_UCS_UCSORTHOVIEW_set", _wrap_Dwg_Object_UCS_UCSORTHOVIEW_set},
{"LibreDWGc::Dwg_Object_UCS_UCSORTHOVIEW_get", _wrap_Dwg_Object_UCS_UCSORTHOVIEW_get},
{"LibreDWGc::Dwg_Object_UCS_base_ucs_set", _wrap_Dwg_Object_UCS_base_ucs_set},
{"LibreDWGc::Dwg_Object_UCS_base_ucs_get", _wrap_Dwg_Object_UCS_base_ucs_get},
{"LibreDWGc::Dwg_Object_UCS_named_ucs_set", _wrap_Dwg_Object_UCS_named_ucs_set},
{"LibreDWGc::Dwg_Object_UCS_named_ucs_get", _wrap_Dwg_Object_UCS_named_ucs_get},
{"LibreDWGc::Dwg_Object_UCS_num_orthopts_set", _wrap_Dwg_Object_UCS_num_orthopts_set},
{"LibreDWGc::Dwg_Object_UCS_num_orthopts_get", _wrap_Dwg_Object_UCS_num_orthopts_get},
{"LibreDWGc::Dwg_Object_UCS_orthopts_set", _wrap_Dwg_Object_UCS_orthopts_set},
{"LibreDWGc::Dwg_Object_UCS_orthopts_get", _wrap_Dwg_Object_UCS_orthopts_get},
{"LibreDWGc::new_Dwg_Object_UCS", _wrap_new_Dwg_Object_UCS},
{"LibreDWGc::delete_Dwg_Object_UCS", _wrap_delete_Dwg_Object_UCS},
{"LibreDWGc::Dwg_Object_VPORT_CONTROL_parent_set", _wrap_Dwg_Object_VPORT_CONTROL_parent_set},
{"LibreDWGc::Dwg_Object_VPORT_CONTROL_parent_get", _wrap_Dwg_Object_VPORT_CONTROL_parent_get},
{"LibreDWGc::Dwg_Object_VPORT_CONTROL_num_entries_set", _wrap_Dwg_Object_VPORT_CONTROL_num_entries_set},
{"LibreDWGc::Dwg_Object_VPORT_CONTROL_num_entries_get", _wrap_Dwg_Object_VPORT_CONTROL_num_entries_get},
{"LibreDWGc::Dwg_Object_VPORT_CONTROL_entries_set", _wrap_Dwg_Object_VPORT_CONTROL_entries_set},
{"LibreDWGc::Dwg_Object_VPORT_CONTROL_entries_get", _wrap_Dwg_Object_VPORT_CONTROL_entries_get},
{"LibreDWGc::Dwg_Object_VPORT_CONTROL_flags_r11_set", _wrap_Dwg_Object_VPORT_CONTROL_flags_r11_set},
{"LibreDWGc::Dwg_Object_VPORT_CONTROL_flags_r11_get", _wrap_Dwg_Object_VPORT_CONTROL_flags_r11_get},
{"LibreDWGc::new_Dwg_Object_VPORT_CONTROL", _wrap_new_Dwg_Object_VPORT_CONTROL},
{"LibreDWGc::delete_Dwg_Object_VPORT_CONTROL", _wrap_delete_Dwg_Object_VPORT_CONTROL},
{"LibreDWGc::Dwg_Object_VPORT_parent_set", _wrap_Dwg_Object_VPORT_parent_set},
{"LibreDWGc::Dwg_Object_VPORT_parent_get", _wrap_Dwg_Object_VPORT_parent_get},
{"LibreDWGc::Dwg_Object_VPORT_flag_set", _wrap_Dwg_Object_VPORT_flag_set},
{"LibreDWGc::Dwg_Object_VPORT_flag_get", _wrap_Dwg_Object_VPORT_flag_get},
{"LibreDWGc::Dwg_Object_VPORT_name_set", _wrap_Dwg_Object_VPORT_name_set},
{"LibreDWGc::Dwg_Object_VPORT_name_get", _wrap_Dwg_Object_VPORT_name_get},
{"LibreDWGc::Dwg_Object_VPORT_used_set", _wrap_Dwg_Object_VPORT_used_set},
{"LibreDWGc::Dwg_Object_VPORT_used_get", _wrap_Dwg_Object_VPORT_used_get},
{"LibreDWGc::Dwg_Object_VPORT_is_xref_ref_set", _wrap_Dwg_Object_VPORT_is_xref_ref_set},
{"LibreDWGc::Dwg_Object_VPORT_is_xref_ref_get", _wrap_Dwg_Object_VPORT_is_xref_ref_get},
{"LibreDWGc::Dwg_Object_VPORT_is_xref_resolved_set", _wrap_Dwg_Object_VPORT_is_xref_resolved_set},
{"LibreDWGc::Dwg_Object_VPORT_is_xref_resolved_get", _wrap_Dwg_Object_VPORT_is_xref_resolved_get},
{"LibreDWGc::Dwg_Object_VPORT_is_xref_dep_set", _wrap_Dwg_Object_VPORT_is_xref_dep_set},
{"LibreDWGc::Dwg_Object_VPORT_is_xref_dep_get", _wrap_Dwg_Object_VPORT_is_xref_dep_get},
{"LibreDWGc::Dwg_Object_VPORT_xref_set", _wrap_Dwg_Object_VPORT_xref_set},
{"LibreDWGc::Dwg_Object_VPORT_xref_get", _wrap_Dwg_Object_VPORT_xref_get},
{"LibreDWGc::Dwg_Object_VPORT_VIEWSIZE_set", _wrap_Dwg_Object_VPORT_VIEWSIZE_set},
{"LibreDWGc::Dwg_Object_VPORT_VIEWSIZE_get", _wrap_Dwg_Object_VPORT_VIEWSIZE_get},
{"LibreDWGc::Dwg_Object_VPORT_view_width_set", _wrap_Dwg_Object_VPORT_view_width_set},
{"LibreDWGc::Dwg_Object_VPORT_view_width_get", _wrap_Dwg_Object_VPORT_view_width_get},
{"LibreDWGc::Dwg_Object_VPORT_aspect_ratio_set", _wrap_Dwg_Object_VPORT_aspect_ratio_set},
{"LibreDWGc::Dwg_Object_VPORT_aspect_ratio_get", _wrap_Dwg_Object_VPORT_aspect_ratio_get},
{"LibreDWGc::Dwg_Object_VPORT_VIEWCTR_set", _wrap_Dwg_Object_VPORT_VIEWCTR_set},
{"LibreDWGc::Dwg_Object_VPORT_VIEWCTR_get", _wrap_Dwg_Object_VPORT_VIEWCTR_get},
{"LibreDWGc::Dwg_Object_VPORT_view_target_set", _wrap_Dwg_Object_VPORT_view_target_set},
{"LibreDWGc::Dwg_Object_VPORT_view_target_get", _wrap_Dwg_Object_VPORT_view_target_get},
{"LibreDWGc::Dwg_Object_VPORT_VIEWDIR_set", _wrap_Dwg_Object_VPORT_VIEWDIR_set},
{"LibreDWGc::Dwg_Object_VPORT_VIEWDIR_get", _wrap_Dwg_Object_VPORT_VIEWDIR_get},
{"LibreDWGc::Dwg_Object_VPORT_view_twist_set", _wrap_Dwg_Object_VPORT_view_twist_set},
{"LibreDWGc::Dwg_Object_VPORT_view_twist_get", _wrap_Dwg_Object_VPORT_view_twist_get},
{"LibreDWGc::Dwg_Object_VPORT_lens_length_set", _wrap_Dwg_Object_VPORT_lens_length_set},
{"LibreDWGc::Dwg_Object_VPORT_lens_length_get", _wrap_Dwg_Object_VPORT_lens_length_get},
{"LibreDWGc::Dwg_Object_VPORT_front_clip_z_set", _wrap_Dwg_Object_VPORT_front_clip_z_set},
{"LibreDWGc::Dwg_Object_VPORT_front_clip_z_get", _wrap_Dwg_Object_VPORT_front_clip_z_get},
{"LibreDWGc::Dwg_Object_VPORT_back_clip_z_set", _wrap_Dwg_Object_VPORT_back_clip_z_set},
{"LibreDWGc::Dwg_Object_VPORT_back_clip_z_get", _wrap_Dwg_Object_VPORT_back_clip_z_get},
{"LibreDWGc::Dwg_Object_VPORT_VIEWMODE_set", _wrap_Dwg_Object_VPORT_VIEWMODE_set},
{"LibreDWGc::Dwg_Object_VPORT_VIEWMODE_get", _wrap_Dwg_Object_VPORT_VIEWMODE_get},
{"LibreDWGc::Dwg_Object_VPORT_render_mode_set", _wrap_Dwg_Object_VPORT_render_mode_set},
{"LibreDWGc::Dwg_Object_VPORT_render_mode_get", _wrap_Dwg_Object_VPORT_render_mode_get},
{"LibreDWGc::Dwg_Object_VPORT_use_default_lights_set", _wrap_Dwg_Object_VPORT_use_default_lights_set},
{"LibreDWGc::Dwg_Object_VPORT_use_default_lights_get", _wrap_Dwg_Object_VPORT_use_default_lights_get},
{"LibreDWGc::Dwg_Object_VPORT_default_lightning_type_set", _wrap_Dwg_Object_VPORT_default_lightning_type_set},
{"LibreDWGc::Dwg_Object_VPORT_default_lightning_type_get", _wrap_Dwg_Object_VPORT_default_lightning_type_get},
{"LibreDWGc::Dwg_Object_VPORT_brightness_set", _wrap_Dwg_Object_VPORT_brightness_set},
{"LibreDWGc::Dwg_Object_VPORT_brightness_get", _wrap_Dwg_Object_VPORT_brightness_get},
{"LibreDWGc::Dwg_Object_VPORT_contrast_set", _wrap_Dwg_Object_VPORT_contrast_set},
{"LibreDWGc::Dwg_Object_VPORT_contrast_get", _wrap_Dwg_Object_VPORT_contrast_get},
{"LibreDWGc::Dwg_Object_VPORT_ambient_color_set", _wrap_Dwg_Object_VPORT_ambient_color_set},
{"LibreDWGc::Dwg_Object_VPORT_ambient_color_get", _wrap_Dwg_Object_VPORT_ambient_color_get},
{"LibreDWGc::Dwg_Object_VPORT_lower_left_set", _wrap_Dwg_Object_VPORT_lower_left_set},
{"LibreDWGc::Dwg_Object_VPORT_lower_left_get", _wrap_Dwg_Object_VPORT_lower_left_get},
{"LibreDWGc::Dwg_Object_VPORT_upper_right_set", _wrap_Dwg_Object_VPORT_upper_right_set},
{"LibreDWGc::Dwg_Object_VPORT_upper_right_get", _wrap_Dwg_Object_VPORT_upper_right_get},
{"LibreDWGc::Dwg_Object_VPORT_UCSFOLLOW_set", _wrap_Dwg_Object_VPORT_UCSFOLLOW_set},
{"LibreDWGc::Dwg_Object_VPORT_UCSFOLLOW_get", _wrap_Dwg_Object_VPORT_UCSFOLLOW_get},
{"LibreDWGc::Dwg_Object_VPORT_circle_zoom_set", _wrap_Dwg_Object_VPORT_circle_zoom_set},
{"LibreDWGc::Dwg_Object_VPORT_circle_zoom_get", _wrap_Dwg_Object_VPORT_circle_zoom_get},
{"LibreDWGc::Dwg_Object_VPORT_FASTZOOM_set", _wrap_Dwg_Object_VPORT_FASTZOOM_set},
{"LibreDWGc::Dwg_Object_VPORT_FASTZOOM_get", _wrap_Dwg_Object_VPORT_FASTZOOM_get},
{"LibreDWGc::Dwg_Object_VPORT_UCSICON_set", _wrap_Dwg_Object_VPORT_UCSICON_set},
{"LibreDWGc::Dwg_Object_VPORT_UCSICON_get", _wrap_Dwg_Object_VPORT_UCSICON_get},
{"LibreDWGc::Dwg_Object_VPORT_GRIDMODE_set", _wrap_Dwg_Object_VPORT_GRIDMODE_set},
{"LibreDWGc::Dwg_Object_VPORT_GRIDMODE_get", _wrap_Dwg_Object_VPORT_GRIDMODE_get},
{"LibreDWGc::Dwg_Object_VPORT_GRIDUNIT_set", _wrap_Dwg_Object_VPORT_GRIDUNIT_set},
{"LibreDWGc::Dwg_Object_VPORT_GRIDUNIT_get", _wrap_Dwg_Object_VPORT_GRIDUNIT_get},
{"LibreDWGc::Dwg_Object_VPORT_SNAPMODE_set", _wrap_Dwg_Object_VPORT_SNAPMODE_set},
{"LibreDWGc::Dwg_Object_VPORT_SNAPMODE_get", _wrap_Dwg_Object_VPORT_SNAPMODE_get},
{"LibreDWGc::Dwg_Object_VPORT_SNAPSTYLE_set", _wrap_Dwg_Object_VPORT_SNAPSTYLE_set},
{"LibreDWGc::Dwg_Object_VPORT_SNAPSTYLE_get", _wrap_Dwg_Object_VPORT_SNAPSTYLE_get},
{"LibreDWGc::Dwg_Object_VPORT_SNAPISOPAIR_set", _wrap_Dwg_Object_VPORT_SNAPISOPAIR_set},
{"LibreDWGc::Dwg_Object_VPORT_SNAPISOPAIR_get", _wrap_Dwg_Object_VPORT_SNAPISOPAIR_get},
{"LibreDWGc::Dwg_Object_VPORT_SNAPANG_set", _wrap_Dwg_Object_VPORT_SNAPANG_set},
{"LibreDWGc::Dwg_Object_VPORT_SNAPANG_get", _wrap_Dwg_Object_VPORT_SNAPANG_get},
{"LibreDWGc::Dwg_Object_VPORT_SNAPBASE_set", _wrap_Dwg_Object_VPORT_SNAPBASE_set},
{"LibreDWGc::Dwg_Object_VPORT_SNAPBASE_get", _wrap_Dwg_Object_VPORT_SNAPBASE_get},
{"LibreDWGc::Dwg_Object_VPORT_SNAPUNIT_set", _wrap_Dwg_Object_VPORT_SNAPUNIT_set},
{"LibreDWGc::Dwg_Object_VPORT_SNAPUNIT_get", _wrap_Dwg_Object_VPORT_SNAPUNIT_get},
{"LibreDWGc::Dwg_Object_VPORT_ucs_at_origin_set", _wrap_Dwg_Object_VPORT_ucs_at_origin_set},
{"LibreDWGc::Dwg_Object_VPORT_ucs_at_origin_get", _wrap_Dwg_Object_VPORT_ucs_at_origin_get},
{"LibreDWGc::Dwg_Object_VPORT_UCSVP_set", _wrap_Dwg_Object_VPORT_UCSVP_set},
{"LibreDWGc::Dwg_Object_VPORT_UCSVP_get", _wrap_Dwg_Object_VPORT_UCSVP_get},
{"LibreDWGc::Dwg_Object_VPORT_ucsorg_set", _wrap_Dwg_Object_VPORT_ucsorg_set},
{"LibreDWGc::Dwg_Object_VPORT_ucsorg_get", _wrap_Dwg_Object_VPORT_ucsorg_get},
{"LibreDWGc::Dwg_Object_VPORT_ucsxdir_set", _wrap_Dwg_Object_VPORT_ucsxdir_set},
{"LibreDWGc::Dwg_Object_VPORT_ucsxdir_get", _wrap_Dwg_Object_VPORT_ucsxdir_get},
{"LibreDWGc::Dwg_Object_VPORT_ucsydir_set", _wrap_Dwg_Object_VPORT_ucsydir_set},
{"LibreDWGc::Dwg_Object_VPORT_ucsydir_get", _wrap_Dwg_Object_VPORT_ucsydir_get},
{"LibreDWGc::Dwg_Object_VPORT_ucs_elevation_set", _wrap_Dwg_Object_VPORT_ucs_elevation_set},
{"LibreDWGc::Dwg_Object_VPORT_ucs_elevation_get", _wrap_Dwg_Object_VPORT_ucs_elevation_get},
{"LibreDWGc::Dwg_Object_VPORT_UCSORTHOVIEW_set", _wrap_Dwg_Object_VPORT_UCSORTHOVIEW_set},
{"LibreDWGc::Dwg_Object_VPORT_UCSORTHOVIEW_get", _wrap_Dwg_Object_VPORT_UCSORTHOVIEW_get},
{"LibreDWGc::Dwg_Object_VPORT_grid_flags_set", _wrap_Dwg_Object_VPORT_grid_flags_set},
{"LibreDWGc::Dwg_Object_VPORT_grid_flags_get", _wrap_Dwg_Object_VPORT_grid_flags_get},
{"LibreDWGc::Dwg_Object_VPORT_grid_major_set", _wrap_Dwg_Object_VPORT_grid_major_set},
{"LibreDWGc::Dwg_Object_VPORT_grid_major_get", _wrap_Dwg_Object_VPORT_grid_major_get},
{"LibreDWGc::Dwg_Object_VPORT_background_set", _wrap_Dwg_Object_VPORT_background_set},
{"LibreDWGc::Dwg_Object_VPORT_background_get", _wrap_Dwg_Object_VPORT_background_get},
{"LibreDWGc::Dwg_Object_VPORT_visualstyle_set", _wrap_Dwg_Object_VPORT_visualstyle_set},
{"LibreDWGc::Dwg_Object_VPORT_visualstyle_get", _wrap_Dwg_Object_VPORT_visualstyle_get},
{"LibreDWGc::Dwg_Object_VPORT_sun_set", _wrap_Dwg_Object_VPORT_sun_set},
{"LibreDWGc::Dwg_Object_VPORT_sun_get", _wrap_Dwg_Object_VPORT_sun_get},
{"LibreDWGc::Dwg_Object_VPORT_named_ucs_set", _wrap_Dwg_Object_VPORT_named_ucs_set},
{"LibreDWGc::Dwg_Object_VPORT_named_ucs_get", _wrap_Dwg_Object_VPORT_named_ucs_get},
{"LibreDWGc::Dwg_Object_VPORT_base_ucs_set", _wrap_Dwg_Object_VPORT_base_ucs_set},
{"LibreDWGc::Dwg_Object_VPORT_base_ucs_get", _wrap_Dwg_Object_VPORT_base_ucs_get},
{"LibreDWGc::new_Dwg_Object_VPORT", _wrap_new_Dwg_Object_VPORT},
{"LibreDWGc::delete_Dwg_Object_VPORT", _wrap_delete_Dwg_Object_VPORT},
{"LibreDWGc::Dwg_Object_APPID_CONTROL_parent_set", _wrap_Dwg_Object_APPID_CONTROL_parent_set},
{"LibreDWGc::Dwg_Object_APPID_CONTROL_parent_get", _wrap_Dwg_Object_APPID_CONTROL_parent_get},
{"LibreDWGc::Dwg_Object_APPID_CONTROL_num_entries_set", _wrap_Dwg_Object_APPID_CONTROL_num_entries_set},
{"LibreDWGc::Dwg_Object_APPID_CONTROL_num_entries_get", _wrap_Dwg_Object_APPID_CONTROL_num_entries_get},
{"LibreDWGc::Dwg_Object_APPID_CONTROL_entries_set", _wrap_Dwg_Object_APPID_CONTROL_entries_set},
{"LibreDWGc::Dwg_Object_APPID_CONTROL_entries_get", _wrap_Dwg_Object_APPID_CONTROL_entries_get},
{"LibreDWGc::Dwg_Object_APPID_CONTROL_flags_r11_set", _wrap_Dwg_Object_APPID_CONTROL_flags_r11_set},
{"LibreDWGc::Dwg_Object_APPID_CONTROL_flags_r11_get", _wrap_Dwg_Object_APPID_CONTROL_flags_r11_get},
{"LibreDWGc::new_Dwg_Object_APPID_CONTROL", _wrap_new_Dwg_Object_APPID_CONTROL},
{"LibreDWGc::delete_Dwg_Object_APPID_CONTROL", _wrap_delete_Dwg_Object_APPID_CONTROL},
{"LibreDWGc::Dwg_Object_APPID_parent_set", _wrap_Dwg_Object_APPID_parent_set},
{"LibreDWGc::Dwg_Object_APPID_parent_get", _wrap_Dwg_Object_APPID_parent_get},
{"LibreDWGc::Dwg_Object_APPID_flag_set", _wrap_Dwg_Object_APPID_flag_set},
{"LibreDWGc::Dwg_Object_APPID_flag_get", _wrap_Dwg_Object_APPID_flag_get},
{"LibreDWGc::Dwg_Object_APPID_name_set", _wrap_Dwg_Object_APPID_name_set},
{"LibreDWGc::Dwg_Object_APPID_name_get", _wrap_Dwg_Object_APPID_name_get},
{"LibreDWGc::Dwg_Object_APPID_used_set", _wrap_Dwg_Object_APPID_used_set},
{"LibreDWGc::Dwg_Object_APPID_used_get", _wrap_Dwg_Object_APPID_used_get},
{"LibreDWGc::Dwg_Object_APPID_is_xref_ref_set", _wrap_Dwg_Object_APPID_is_xref_ref_set},
{"LibreDWGc::Dwg_Object_APPID_is_xref_ref_get", _wrap_Dwg_Object_APPID_is_xref_ref_get},
{"LibreDWGc::Dwg_Object_APPID_is_xref_resolved_set", _wrap_Dwg_Object_APPID_is_xref_resolved_set},
{"LibreDWGc::Dwg_Object_APPID_is_xref_resolved_get", _wrap_Dwg_Object_APPID_is_xref_resolved_get},
{"LibreDWGc::Dwg_Object_APPID_is_xref_dep_set", _wrap_Dwg_Object_APPID_is_xref_dep_set},
{"LibreDWGc::Dwg_Object_APPID_is_xref_dep_get", _wrap_Dwg_Object_APPID_is_xref_dep_get},
{"LibreDWGc::Dwg_Object_APPID_xref_set", _wrap_Dwg_Object_APPID_xref_set},
{"LibreDWGc::Dwg_Object_APPID_xref_get", _wrap_Dwg_Object_APPID_xref_get},
{"LibreDWGc::Dwg_Object_APPID_unknown_set", _wrap_Dwg_Object_APPID_unknown_set},
{"LibreDWGc::Dwg_Object_APPID_unknown_get", _wrap_Dwg_Object_APPID_unknown_get},
{"LibreDWGc::new_Dwg_Object_APPID", _wrap_new_Dwg_Object_APPID},
{"LibreDWGc::delete_Dwg_Object_APPID", _wrap_delete_Dwg_Object_APPID},
{"LibreDWGc::Dwg_Object_DIMSTYLE_CONTROL_parent_set", _wrap_Dwg_Object_DIMSTYLE_CONTROL_parent_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_CONTROL_parent_get", _wrap_Dwg_Object_DIMSTYLE_CONTROL_parent_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_CONTROL_num_entries_set", _wrap_Dwg_Object_DIMSTYLE_CONTROL_num_entries_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_CONTROL_num_entries_get", _wrap_Dwg_Object_DIMSTYLE_CONTROL_num_entries_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_CONTROL_entries_set", _wrap_Dwg_Object_DIMSTYLE_CONTROL_entries_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_CONTROL_entries_get", _wrap_Dwg_Object_DIMSTYLE_CONTROL_entries_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_CONTROL_flags_r11_set", _wrap_Dwg_Object_DIMSTYLE_CONTROL_flags_r11_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_CONTROL_flags_r11_get", _wrap_Dwg_Object_DIMSTYLE_CONTROL_flags_r11_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_CONTROL_num_morehandles_set", _wrap_Dwg_Object_DIMSTYLE_CONTROL_num_morehandles_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_CONTROL_num_morehandles_get", _wrap_Dwg_Object_DIMSTYLE_CONTROL_num_morehandles_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_CONTROL_morehandles_set", _wrap_Dwg_Object_DIMSTYLE_CONTROL_morehandles_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_CONTROL_morehandles_get", _wrap_Dwg_Object_DIMSTYLE_CONTROL_morehandles_get},
{"LibreDWGc::new_Dwg_Object_DIMSTYLE_CONTROL", _wrap_new_Dwg_Object_DIMSTYLE_CONTROL},
{"LibreDWGc::delete_Dwg_Object_DIMSTYLE_CONTROL", _wrap_delete_Dwg_Object_DIMSTYLE_CONTROL},
{"LibreDWGc::Dwg_Object_DIMSTYLE_parent_set", _wrap_Dwg_Object_DIMSTYLE_parent_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_parent_get", _wrap_Dwg_Object_DIMSTYLE_parent_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_flag_set", _wrap_Dwg_Object_DIMSTYLE_flag_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_flag_get", _wrap_Dwg_Object_DIMSTYLE_flag_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_name_set", _wrap_Dwg_Object_DIMSTYLE_name_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_name_get", _wrap_Dwg_Object_DIMSTYLE_name_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_used_set", _wrap_Dwg_Object_DIMSTYLE_used_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_used_get", _wrap_Dwg_Object_DIMSTYLE_used_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_is_xref_ref_set", _wrap_Dwg_Object_DIMSTYLE_is_xref_ref_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_is_xref_ref_get", _wrap_Dwg_Object_DIMSTYLE_is_xref_ref_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_is_xref_resolved_set", _wrap_Dwg_Object_DIMSTYLE_is_xref_resolved_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_is_xref_resolved_get", _wrap_Dwg_Object_DIMSTYLE_is_xref_resolved_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_is_xref_dep_set", _wrap_Dwg_Object_DIMSTYLE_is_xref_dep_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_is_xref_dep_get", _wrap_Dwg_Object_DIMSTYLE_is_xref_dep_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_xref_set", _wrap_Dwg_Object_DIMSTYLE_xref_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_xref_get", _wrap_Dwg_Object_DIMSTYLE_xref_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTOL_set", _wrap_Dwg_Object_DIMSTYLE_DIMTOL_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTOL_get", _wrap_Dwg_Object_DIMSTYLE_DIMTOL_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMLIM_set", _wrap_Dwg_Object_DIMSTYLE_DIMLIM_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMLIM_get", _wrap_Dwg_Object_DIMSTYLE_DIMLIM_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTIH_set", _wrap_Dwg_Object_DIMSTYLE_DIMTIH_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTIH_get", _wrap_Dwg_Object_DIMSTYLE_DIMTIH_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTOH_set", _wrap_Dwg_Object_DIMSTYLE_DIMTOH_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTOH_get", _wrap_Dwg_Object_DIMSTYLE_DIMTOH_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMSE1_set", _wrap_Dwg_Object_DIMSTYLE_DIMSE1_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMSE1_get", _wrap_Dwg_Object_DIMSTYLE_DIMSE1_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMSE2_set", _wrap_Dwg_Object_DIMSTYLE_DIMSE2_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMSE2_get", _wrap_Dwg_Object_DIMSTYLE_DIMSE2_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMALT_set", _wrap_Dwg_Object_DIMSTYLE_DIMALT_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMALT_get", _wrap_Dwg_Object_DIMSTYLE_DIMALT_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTOFL_set", _wrap_Dwg_Object_DIMSTYLE_DIMTOFL_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTOFL_get", _wrap_Dwg_Object_DIMSTYLE_DIMTOFL_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMSAH_set", _wrap_Dwg_Object_DIMSTYLE_DIMSAH_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMSAH_get", _wrap_Dwg_Object_DIMSTYLE_DIMSAH_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTIX_set", _wrap_Dwg_Object_DIMSTYLE_DIMTIX_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTIX_get", _wrap_Dwg_Object_DIMSTYLE_DIMTIX_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMSOXD_set", _wrap_Dwg_Object_DIMSTYLE_DIMSOXD_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMSOXD_get", _wrap_Dwg_Object_DIMSTYLE_DIMSOXD_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTD_set", _wrap_Dwg_Object_DIMSTYLE_DIMALTD_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTD_get", _wrap_Dwg_Object_DIMSTYLE_DIMALTD_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMZIN_set", _wrap_Dwg_Object_DIMSTYLE_DIMZIN_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMZIN_get", _wrap_Dwg_Object_DIMSTYLE_DIMZIN_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMSD1_set", _wrap_Dwg_Object_DIMSTYLE_DIMSD1_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMSD1_get", _wrap_Dwg_Object_DIMSTYLE_DIMSD1_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMSD2_set", _wrap_Dwg_Object_DIMSTYLE_DIMSD2_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMSD2_get", _wrap_Dwg_Object_DIMSTYLE_DIMSD2_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTOLJ_set", _wrap_Dwg_Object_DIMSTYLE_DIMTOLJ_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTOLJ_get", _wrap_Dwg_Object_DIMSTYLE_DIMTOLJ_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMJUST_set", _wrap_Dwg_Object_DIMSTYLE_DIMJUST_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMJUST_get", _wrap_Dwg_Object_DIMSTYLE_DIMJUST_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMFIT_set", _wrap_Dwg_Object_DIMSTYLE_DIMFIT_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMFIT_get", _wrap_Dwg_Object_DIMSTYLE_DIMFIT_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMUPT_set", _wrap_Dwg_Object_DIMSTYLE_DIMUPT_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMUPT_get", _wrap_Dwg_Object_DIMSTYLE_DIMUPT_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTZIN_set", _wrap_Dwg_Object_DIMSTYLE_DIMTZIN_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTZIN_get", _wrap_Dwg_Object_DIMSTYLE_DIMTZIN_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTZ_set", _wrap_Dwg_Object_DIMSTYLE_DIMALTZ_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTZ_get", _wrap_Dwg_Object_DIMSTYLE_DIMALTZ_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTTZ_set", _wrap_Dwg_Object_DIMSTYLE_DIMALTTZ_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTTZ_get", _wrap_Dwg_Object_DIMSTYLE_DIMALTTZ_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTAD_set", _wrap_Dwg_Object_DIMSTYLE_DIMTAD_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTAD_get", _wrap_Dwg_Object_DIMSTYLE_DIMTAD_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMUNIT_set", _wrap_Dwg_Object_DIMSTYLE_DIMUNIT_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMUNIT_get", _wrap_Dwg_Object_DIMSTYLE_DIMUNIT_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMAUNIT_set", _wrap_Dwg_Object_DIMSTYLE_DIMAUNIT_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMAUNIT_get", _wrap_Dwg_Object_DIMSTYLE_DIMAUNIT_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMDEC_set", _wrap_Dwg_Object_DIMSTYLE_DIMDEC_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMDEC_get", _wrap_Dwg_Object_DIMSTYLE_DIMDEC_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTDEC_set", _wrap_Dwg_Object_DIMSTYLE_DIMTDEC_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTDEC_get", _wrap_Dwg_Object_DIMSTYLE_DIMTDEC_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTU_set", _wrap_Dwg_Object_DIMSTYLE_DIMALTU_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTU_get", _wrap_Dwg_Object_DIMSTYLE_DIMALTU_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTTD_set", _wrap_Dwg_Object_DIMSTYLE_DIMALTTD_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTTD_get", _wrap_Dwg_Object_DIMSTYLE_DIMALTTD_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMSCALE_set", _wrap_Dwg_Object_DIMSTYLE_DIMSCALE_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMSCALE_get", _wrap_Dwg_Object_DIMSTYLE_DIMSCALE_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMASZ_set", _wrap_Dwg_Object_DIMSTYLE_DIMASZ_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMASZ_get", _wrap_Dwg_Object_DIMSTYLE_DIMASZ_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMEXO_set", _wrap_Dwg_Object_DIMSTYLE_DIMEXO_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMEXO_get", _wrap_Dwg_Object_DIMSTYLE_DIMEXO_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMDLI_set", _wrap_Dwg_Object_DIMSTYLE_DIMDLI_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMDLI_get", _wrap_Dwg_Object_DIMSTYLE_DIMDLI_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMEXE_set", _wrap_Dwg_Object_DIMSTYLE_DIMEXE_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMEXE_get", _wrap_Dwg_Object_DIMSTYLE_DIMEXE_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMRND_set", _wrap_Dwg_Object_DIMSTYLE_DIMRND_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMRND_get", _wrap_Dwg_Object_DIMSTYLE_DIMRND_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMDLE_set", _wrap_Dwg_Object_DIMSTYLE_DIMDLE_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMDLE_get", _wrap_Dwg_Object_DIMSTYLE_DIMDLE_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTP_set", _wrap_Dwg_Object_DIMSTYLE_DIMTP_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTP_get", _wrap_Dwg_Object_DIMSTYLE_DIMTP_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTM_set", _wrap_Dwg_Object_DIMSTYLE_DIMTM_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTM_get", _wrap_Dwg_Object_DIMSTYLE_DIMTM_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMFXL_set", _wrap_Dwg_Object_DIMSTYLE_DIMFXL_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMFXL_get", _wrap_Dwg_Object_DIMSTYLE_DIMFXL_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMJOGANG_set", _wrap_Dwg_Object_DIMSTYLE_DIMJOGANG_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMJOGANG_get", _wrap_Dwg_Object_DIMSTYLE_DIMJOGANG_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTFILL_set", _wrap_Dwg_Object_DIMSTYLE_DIMTFILL_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTFILL_get", _wrap_Dwg_Object_DIMSTYLE_DIMTFILL_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTFILLCLR_set", _wrap_Dwg_Object_DIMSTYLE_DIMTFILLCLR_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTFILLCLR_get", _wrap_Dwg_Object_DIMSTYLE_DIMTFILLCLR_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMAZIN_set", _wrap_Dwg_Object_DIMSTYLE_DIMAZIN_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMAZIN_get", _wrap_Dwg_Object_DIMSTYLE_DIMAZIN_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMARCSYM_set", _wrap_Dwg_Object_DIMSTYLE_DIMARCSYM_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMARCSYM_get", _wrap_Dwg_Object_DIMSTYLE_DIMARCSYM_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTXT_set", _wrap_Dwg_Object_DIMSTYLE_DIMTXT_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTXT_get", _wrap_Dwg_Object_DIMSTYLE_DIMTXT_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMCEN_set", _wrap_Dwg_Object_DIMSTYLE_DIMCEN_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMCEN_get", _wrap_Dwg_Object_DIMSTYLE_DIMCEN_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTSZ_set", _wrap_Dwg_Object_DIMSTYLE_DIMTSZ_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTSZ_get", _wrap_Dwg_Object_DIMSTYLE_DIMTSZ_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTF_set", _wrap_Dwg_Object_DIMSTYLE_DIMALTF_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTF_get", _wrap_Dwg_Object_DIMSTYLE_DIMALTF_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMLFAC_set", _wrap_Dwg_Object_DIMSTYLE_DIMLFAC_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMLFAC_get", _wrap_Dwg_Object_DIMSTYLE_DIMLFAC_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTVP_set", _wrap_Dwg_Object_DIMSTYLE_DIMTVP_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTVP_get", _wrap_Dwg_Object_DIMSTYLE_DIMTVP_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTFAC_set", _wrap_Dwg_Object_DIMSTYLE_DIMTFAC_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTFAC_get", _wrap_Dwg_Object_DIMSTYLE_DIMTFAC_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMGAP_set", _wrap_Dwg_Object_DIMSTYLE_DIMGAP_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMGAP_get", _wrap_Dwg_Object_DIMSTYLE_DIMGAP_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMPOST_set", _wrap_Dwg_Object_DIMSTYLE_DIMPOST_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMPOST_get", _wrap_Dwg_Object_DIMSTYLE_DIMPOST_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMAPOST_set", _wrap_Dwg_Object_DIMSTYLE_DIMAPOST_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMAPOST_get", _wrap_Dwg_Object_DIMSTYLE_DIMAPOST_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMBLK_T_set", _wrap_Dwg_Object_DIMSTYLE_DIMBLK_T_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMBLK_T_get", _wrap_Dwg_Object_DIMSTYLE_DIMBLK_T_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMBLK1_T_set", _wrap_Dwg_Object_DIMSTYLE_DIMBLK1_T_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMBLK1_T_get", _wrap_Dwg_Object_DIMSTYLE_DIMBLK1_T_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMBLK2_T_set", _wrap_Dwg_Object_DIMSTYLE_DIMBLK2_T_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMBLK2_T_get", _wrap_Dwg_Object_DIMSTYLE_DIMBLK2_T_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTRND_set", _wrap_Dwg_Object_DIMSTYLE_DIMALTRND_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTRND_get", _wrap_Dwg_Object_DIMSTYLE_DIMALTRND_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMCLRD_N_set", _wrap_Dwg_Object_DIMSTYLE_DIMCLRD_N_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMCLRD_N_get", _wrap_Dwg_Object_DIMSTYLE_DIMCLRD_N_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMCLRE_N_set", _wrap_Dwg_Object_DIMSTYLE_DIMCLRE_N_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMCLRE_N_get", _wrap_Dwg_Object_DIMSTYLE_DIMCLRE_N_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMCLRT_N_set", _wrap_Dwg_Object_DIMSTYLE_DIMCLRT_N_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMCLRT_N_get", _wrap_Dwg_Object_DIMSTYLE_DIMCLRT_N_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMCLRD_set", _wrap_Dwg_Object_DIMSTYLE_DIMCLRD_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMCLRD_get", _wrap_Dwg_Object_DIMSTYLE_DIMCLRD_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMCLRE_set", _wrap_Dwg_Object_DIMSTYLE_DIMCLRE_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMCLRE_get", _wrap_Dwg_Object_DIMSTYLE_DIMCLRE_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMCLRT_set", _wrap_Dwg_Object_DIMSTYLE_DIMCLRT_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMCLRT_get", _wrap_Dwg_Object_DIMSTYLE_DIMCLRT_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMADEC_set", _wrap_Dwg_Object_DIMSTYLE_DIMADEC_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMADEC_get", _wrap_Dwg_Object_DIMSTYLE_DIMADEC_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMFRAC_set", _wrap_Dwg_Object_DIMSTYLE_DIMFRAC_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMFRAC_get", _wrap_Dwg_Object_DIMSTYLE_DIMFRAC_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMLUNIT_set", _wrap_Dwg_Object_DIMSTYLE_DIMLUNIT_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMLUNIT_get", _wrap_Dwg_Object_DIMSTYLE_DIMLUNIT_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMDSEP_set", _wrap_Dwg_Object_DIMSTYLE_DIMDSEP_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMDSEP_get", _wrap_Dwg_Object_DIMSTYLE_DIMDSEP_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTMOVE_set", _wrap_Dwg_Object_DIMSTYLE_DIMTMOVE_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTMOVE_get", _wrap_Dwg_Object_DIMSTYLE_DIMTMOVE_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMATFIT_set", _wrap_Dwg_Object_DIMSTYLE_DIMATFIT_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMATFIT_get", _wrap_Dwg_Object_DIMSTYLE_DIMATFIT_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMFXLON_set", _wrap_Dwg_Object_DIMSTYLE_DIMFXLON_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMFXLON_get", _wrap_Dwg_Object_DIMSTYLE_DIMFXLON_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTXTDIRECTION_set", _wrap_Dwg_Object_DIMSTYLE_DIMTXTDIRECTION_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTXTDIRECTION_get", _wrap_Dwg_Object_DIMSTYLE_DIMTXTDIRECTION_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTMZF_set", _wrap_Dwg_Object_DIMSTYLE_DIMALTMZF_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTMZF_get", _wrap_Dwg_Object_DIMSTYLE_DIMALTMZF_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTMZS_set", _wrap_Dwg_Object_DIMSTYLE_DIMALTMZS_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTMZS_get", _wrap_Dwg_Object_DIMSTYLE_DIMALTMZS_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMMZF_set", _wrap_Dwg_Object_DIMSTYLE_DIMMZF_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMMZF_get", _wrap_Dwg_Object_DIMSTYLE_DIMMZF_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMMZS_set", _wrap_Dwg_Object_DIMSTYLE_DIMMZS_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMMZS_get", _wrap_Dwg_Object_DIMSTYLE_DIMMZS_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMLWD_set", _wrap_Dwg_Object_DIMSTYLE_DIMLWD_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMLWD_get", _wrap_Dwg_Object_DIMSTYLE_DIMLWD_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMLWE_set", _wrap_Dwg_Object_DIMSTYLE_DIMLWE_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMLWE_get", _wrap_Dwg_Object_DIMSTYLE_DIMLWE_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_flag0_set", _wrap_Dwg_Object_DIMSTYLE_flag0_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_flag0_get", _wrap_Dwg_Object_DIMSTYLE_flag0_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTXSTY_set", _wrap_Dwg_Object_DIMSTYLE_DIMTXSTY_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMTXSTY_get", _wrap_Dwg_Object_DIMSTYLE_DIMTXSTY_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMLDRBLK_set", _wrap_Dwg_Object_DIMSTYLE_DIMLDRBLK_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMLDRBLK_get", _wrap_Dwg_Object_DIMSTYLE_DIMLDRBLK_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMBLK_set", _wrap_Dwg_Object_DIMSTYLE_DIMBLK_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMBLK_get", _wrap_Dwg_Object_DIMSTYLE_DIMBLK_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMBLK1_set", _wrap_Dwg_Object_DIMSTYLE_DIMBLK1_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMBLK1_get", _wrap_Dwg_Object_DIMSTYLE_DIMBLK1_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMBLK2_set", _wrap_Dwg_Object_DIMSTYLE_DIMBLK2_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMBLK2_get", _wrap_Dwg_Object_DIMSTYLE_DIMBLK2_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMLTYPE_set", _wrap_Dwg_Object_DIMSTYLE_DIMLTYPE_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMLTYPE_get", _wrap_Dwg_Object_DIMSTYLE_DIMLTYPE_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMLTEX1_set", _wrap_Dwg_Object_DIMSTYLE_DIMLTEX1_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMLTEX1_get", _wrap_Dwg_Object_DIMSTYLE_DIMLTEX1_get},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMLTEX2_set", _wrap_Dwg_Object_DIMSTYLE_DIMLTEX2_set},
{"LibreDWGc::Dwg_Object_DIMSTYLE_DIMLTEX2_get", _wrap_Dwg_Object_DIMSTYLE_DIMLTEX2_get},
{"LibreDWGc::new_Dwg_Object_DIMSTYLE", _wrap_new_Dwg_Object_DIMSTYLE},
{"LibreDWGc::delete_Dwg_Object_DIMSTYLE", _wrap_delete_Dwg_Object_DIMSTYLE},
{"LibreDWGc::Dwg_Object_VX_CONTROL_parent_set", _wrap_Dwg_Object_VX_CONTROL_parent_set},
{"LibreDWGc::Dwg_Object_VX_CONTROL_parent_get", _wrap_Dwg_Object_VX_CONTROL_parent_get},
{"LibreDWGc::Dwg_Object_VX_CONTROL_num_entries_set", _wrap_Dwg_Object_VX_CONTROL_num_entries_set},
{"LibreDWGc::Dwg_Object_VX_CONTROL_num_entries_get", _wrap_Dwg_Object_VX_CONTROL_num_entries_get},
{"LibreDWGc::Dwg_Object_VX_CONTROL_entries_set", _wrap_Dwg_Object_VX_CONTROL_entries_set},
{"LibreDWGc::Dwg_Object_VX_CONTROL_entries_get", _wrap_Dwg_Object_VX_CONTROL_entries_get},
{"LibreDWGc::Dwg_Object_VX_CONTROL_flags_r11_set", _wrap_Dwg_Object_VX_CONTROL_flags_r11_set},
{"LibreDWGc::Dwg_Object_VX_CONTROL_flags_r11_get", _wrap_Dwg_Object_VX_CONTROL_flags_r11_get},
{"LibreDWGc::new_Dwg_Object_VX_CONTROL", _wrap_new_Dwg_Object_VX_CONTROL},
{"LibreDWGc::delete_Dwg_Object_VX_CONTROL", _wrap_delete_Dwg_Object_VX_CONTROL},
{"LibreDWGc::Dwg_Object_VX_TABLE_RECORD_parent_set", _wrap_Dwg_Object_VX_TABLE_RECORD_parent_set},
{"LibreDWGc::Dwg_Object_VX_TABLE_RECORD_parent_get", _wrap_Dwg_Object_VX_TABLE_RECORD_parent_get},
{"LibreDWGc::Dwg_Object_VX_TABLE_RECORD_flag_set", _wrap_Dwg_Object_VX_TABLE_RECORD_flag_set},
{"LibreDWGc::Dwg_Object_VX_TABLE_RECORD_flag_get", _wrap_Dwg_Object_VX_TABLE_RECORD_flag_get},
{"LibreDWGc::Dwg_Object_VX_TABLE_RECORD_name_set", _wrap_Dwg_Object_VX_TABLE_RECORD_name_set},
{"LibreDWGc::Dwg_Object_VX_TABLE_RECORD_name_get", _wrap_Dwg_Object_VX_TABLE_RECORD_name_get},
{"LibreDWGc::Dwg_Object_VX_TABLE_RECORD_used_set", _wrap_Dwg_Object_VX_TABLE_RECORD_used_set},
{"LibreDWGc::Dwg_Object_VX_TABLE_RECORD_used_get", _wrap_Dwg_Object_VX_TABLE_RECORD_used_get},
{"LibreDWGc::Dwg_Object_VX_TABLE_RECORD_is_xref_ref_set", _wrap_Dwg_Object_VX_TABLE_RECORD_is_xref_ref_set},
{"LibreDWGc::Dwg_Object_VX_TABLE_RECORD_is_xref_ref_get", _wrap_Dwg_Object_VX_TABLE_RECORD_is_xref_ref_get},
{"LibreDWGc::Dwg_Object_VX_TABLE_RECORD_is_xref_resolved_set", _wrap_Dwg_Object_VX_TABLE_RECORD_is_xref_resolved_set},
{"LibreDWGc::Dwg_Object_VX_TABLE_RECORD_is_xref_resolved_get", _wrap_Dwg_Object_VX_TABLE_RECORD_is_xref_resolved_get},
{"LibreDWGc::Dwg_Object_VX_TABLE_RECORD_is_xref_dep_set", _wrap_Dwg_Object_VX_TABLE_RECORD_is_xref_dep_set},
{"LibreDWGc::Dwg_Object_VX_TABLE_RECORD_is_xref_dep_get", _wrap_Dwg_Object_VX_TABLE_RECORD_is_xref_dep_get},
{"LibreDWGc::Dwg_Object_VX_TABLE_RECORD_xref_set", _wrap_Dwg_Object_VX_TABLE_RECORD_xref_set},
{"LibreDWGc::Dwg_Object_VX_TABLE_RECORD_xref_get", _wrap_Dwg_Object_VX_TABLE_RECORD_xref_get},
{"LibreDWGc::Dwg_Object_VX_TABLE_RECORD_is_on_set", _wrap_Dwg_Object_VX_TABLE_RECORD_is_on_set},
{"LibreDWGc::Dwg_Object_VX_TABLE_RECORD_is_on_get", _wrap_Dwg_Object_VX_TABLE_RECORD_is_on_get},
{"LibreDWGc::Dwg_Object_VX_TABLE_RECORD_viewport_set", _wrap_Dwg_Object_VX_TABLE_RECORD_viewport_set},
{"LibreDWGc::Dwg_Object_VX_TABLE_RECORD_viewport_get", _wrap_Dwg_Object_VX_TABLE_RECORD_viewport_get},
{"LibreDWGc::Dwg_Object_VX_TABLE_RECORD_prev_entry_set", _wrap_Dwg_Object_VX_TABLE_RECORD_prev_entry_set},
{"LibreDWGc::Dwg_Object_VX_TABLE_RECORD_prev_entry_get", _wrap_Dwg_Object_VX_TABLE_RECORD_prev_entry_get},
{"LibreDWGc::Dwg_Object_VX_TABLE_RECORD_vport_entity_address_set", _wrap_Dwg_Object_VX_TABLE_RECORD_vport_entity_address_set},
{"LibreDWGc::Dwg_Object_VX_TABLE_RECORD_vport_entity_address_get", _wrap_Dwg_Object_VX_TABLE_RECORD_vport_entity_address_get},
{"LibreDWGc::Dwg_Object_VX_TABLE_RECORD_r11_viewport_index_set", _wrap_Dwg_Object_VX_TABLE_RECORD_r11_viewport_index_set},
{"LibreDWGc::Dwg_Object_VX_TABLE_RECORD_r11_viewport_index_get", _wrap_Dwg_Object_VX_TABLE_RECORD_r11_viewport_index_get},
{"LibreDWGc::Dwg_Object_VX_TABLE_RECORD_r11_prev_entry_index_set", _wrap_Dwg_Object_VX_TABLE_RECORD_r11_prev_entry_index_set},
{"LibreDWGc::Dwg_Object_VX_TABLE_RECORD_r11_prev_entry_index_get", _wrap_Dwg_Object_VX_TABLE_RECORD_r11_prev_entry_index_get},
{"LibreDWGc::new_Dwg_Object_VX_TABLE_RECORD", _wrap_new_Dwg_Object_VX_TABLE_RECORD},
{"LibreDWGc::delete_Dwg_Object_VX_TABLE_RECORD", _wrap_delete_Dwg_Object_VX_TABLE_RECORD},
{"LibreDWGc::Dwg_Object_GROUP_parent_set", _wrap_Dwg_Object_GROUP_parent_set},
{"LibreDWGc::Dwg_Object_GROUP_parent_get", _wrap_Dwg_Object_GROUP_parent_get},
{"LibreDWGc::Dwg_Object_GROUP_name_set", _wrap_Dwg_Object_GROUP_name_set},
{"LibreDWGc::Dwg_Object_GROUP_name_get", _wrap_Dwg_Object_GROUP_name_get},
{"LibreDWGc::Dwg_Object_GROUP_unnamed_set", _wrap_Dwg_Object_GROUP_unnamed_set},
{"LibreDWGc::Dwg_Object_GROUP_unnamed_get", _wrap_Dwg_Object_GROUP_unnamed_get},
{"LibreDWGc::Dwg_Object_GROUP_selectable_set", _wrap_Dwg_Object_GROUP_selectable_set},
{"LibreDWGc::Dwg_Object_GROUP_selectable_get", _wrap_Dwg_Object_GROUP_selectable_get},
{"LibreDWGc::Dwg_Object_GROUP_num_groups_set", _wrap_Dwg_Object_GROUP_num_groups_set},
{"LibreDWGc::Dwg_Object_GROUP_num_groups_get", _wrap_Dwg_Object_GROUP_num_groups_get},
{"LibreDWGc::Dwg_Object_GROUP_groups_set", _wrap_Dwg_Object_GROUP_groups_set},
{"LibreDWGc::Dwg_Object_GROUP_groups_get", _wrap_Dwg_Object_GROUP_groups_get},
{"LibreDWGc::new_Dwg_Object_GROUP", _wrap_new_Dwg_Object_GROUP},
{"LibreDWGc::delete_Dwg_Object_GROUP", _wrap_delete_Dwg_Object_GROUP},
{"LibreDWGc::Dwg_MLINESTYLE_line_parent_set", _wrap_Dwg_MLINESTYLE_line_parent_set},
{"LibreDWGc::Dwg_MLINESTYLE_line_parent_get", _wrap_Dwg_MLINESTYLE_line_parent_get},
{"LibreDWGc::Dwg_MLINESTYLE_line_offset_set", _wrap_Dwg_MLINESTYLE_line_offset_set},
{"LibreDWGc::Dwg_MLINESTYLE_line_offset_get", _wrap_Dwg_MLINESTYLE_line_offset_get},
{"LibreDWGc::Dwg_MLINESTYLE_line_color_set", _wrap_Dwg_MLINESTYLE_line_color_set},
{"LibreDWGc::Dwg_MLINESTYLE_line_color_get", _wrap_Dwg_MLINESTYLE_line_color_get},
{"LibreDWGc::Dwg_MLINESTYLE_line_lt_get", _wrap_Dwg_MLINESTYLE_line_lt_get},
{"LibreDWGc::new_Dwg_MLINESTYLE_line", _wrap_new_Dwg_MLINESTYLE_line},
{"LibreDWGc::delete_Dwg_MLINESTYLE_line", _wrap_delete_Dwg_MLINESTYLE_line},
{"LibreDWGc::_dwg_MLINESTYLE_line_lt_index_set", _wrap__dwg_MLINESTYLE_line_lt_index_set},
{"LibreDWGc::_dwg_MLINESTYLE_line_lt_index_get", _wrap__dwg_MLINESTYLE_line_lt_index_get},
{"LibreDWGc::_dwg_MLINESTYLE_line_lt_ltype_set", _wrap__dwg_MLINESTYLE_line_lt_ltype_set},
{"LibreDWGc::_dwg_MLINESTYLE_line_lt_ltype_get", _wrap__dwg_MLINESTYLE_line_lt_ltype_get},
{"LibreDWGc::new__dwg_MLINESTYLE_line_lt", _wrap_new__dwg_MLINESTYLE_line_lt},
{"LibreDWGc::delete__dwg_MLINESTYLE_line_lt", _wrap_delete__dwg_MLINESTYLE_line_lt},
{"LibreDWGc::Dwg_Object_MLINESTYLE_parent_set", _wrap_Dwg_Object_MLINESTYLE_parent_set},
{"LibreDWGc::Dwg_Object_MLINESTYLE_parent_get", _wrap_Dwg_Object_MLINESTYLE_parent_get},
{"LibreDWGc::Dwg_Object_MLINESTYLE_name_set", _wrap_Dwg_Object_MLINESTYLE_name_set},
{"LibreDWGc::Dwg_Object_MLINESTYLE_name_get", _wrap_Dwg_Object_MLINESTYLE_name_get},
{"LibreDWGc::Dwg_Object_MLINESTYLE_description_set", _wrap_Dwg_Object_MLINESTYLE_description_set},
{"LibreDWGc::Dwg_Object_MLINESTYLE_description_get", _wrap_Dwg_Object_MLINESTYLE_description_get},
{"LibreDWGc::Dwg_Object_MLINESTYLE_flag_set", _wrap_Dwg_Object_MLINESTYLE_flag_set},
{"LibreDWGc::Dwg_Object_MLINESTYLE_flag_get", _wrap_Dwg_Object_MLINESTYLE_flag_get},
{"LibreDWGc::Dwg_Object_MLINESTYLE_fill_color_set", _wrap_Dwg_Object_MLINESTYLE_fill_color_set},
{"LibreDWGc::Dwg_Object_MLINESTYLE_fill_color_get", _wrap_Dwg_Object_MLINESTYLE_fill_color_get},
{"LibreDWGc::Dwg_Object_MLINESTYLE_start_angle_set", _wrap_Dwg_Object_MLINESTYLE_start_angle_set},
{"LibreDWGc::Dwg_Object_MLINESTYLE_start_angle_get", _wrap_Dwg_Object_MLINESTYLE_start_angle_get},
{"LibreDWGc::Dwg_Object_MLINESTYLE_end_angle_set", _wrap_Dwg_Object_MLINESTYLE_end_angle_set},
{"LibreDWGc::Dwg_Object_MLINESTYLE_end_angle_get", _wrap_Dwg_Object_MLINESTYLE_end_angle_get},
{"LibreDWGc::Dwg_Object_MLINESTYLE_num_lines_set", _wrap_Dwg_Object_MLINESTYLE_num_lines_set},
{"LibreDWGc::Dwg_Object_MLINESTYLE_num_lines_get", _wrap_Dwg_Object_MLINESTYLE_num_lines_get},
{"LibreDWGc::Dwg_Object_MLINESTYLE_lines_set", _wrap_Dwg_Object_MLINESTYLE_lines_set},
{"LibreDWGc::Dwg_Object_MLINESTYLE_lines_get", _wrap_Dwg_Object_MLINESTYLE_lines_get},
{"LibreDWGc::new_Dwg_Object_MLINESTYLE", _wrap_new_Dwg_Object_MLINESTYLE},
{"LibreDWGc::delete_Dwg_Object_MLINESTYLE", _wrap_delete_Dwg_Object_MLINESTYLE},
{"LibreDWGc::Dwg_Entity_OLE2FRAME_parent_set", _wrap_Dwg_Entity_OLE2FRAME_parent_set},
{"LibreDWGc::Dwg_Entity_OLE2FRAME_parent_get", _wrap_Dwg_Entity_OLE2FRAME_parent_get},
{"LibreDWGc::Dwg_Entity_OLE2FRAME_type_set", _wrap_Dwg_Entity_OLE2FRAME_type_set},
{"LibreDWGc::Dwg_Entity_OLE2FRAME_type_get", _wrap_Dwg_Entity_OLE2FRAME_type_get},
{"LibreDWGc::Dwg_Entity_OLE2FRAME_mode_set", _wrap_Dwg_Entity_OLE2FRAME_mode_set},
{"LibreDWGc::Dwg_Entity_OLE2FRAME_mode_get", _wrap_Dwg_Entity_OLE2FRAME_mode_get},
{"LibreDWGc::Dwg_Entity_OLE2FRAME_lock_aspect_set", _wrap_Dwg_Entity_OLE2FRAME_lock_aspect_set},
{"LibreDWGc::Dwg_Entity_OLE2FRAME_lock_aspect_get", _wrap_Dwg_Entity_OLE2FRAME_lock_aspect_get},
{"LibreDWGc::Dwg_Entity_OLE2FRAME_data_size_set", _wrap_Dwg_Entity_OLE2FRAME_data_size_set},
{"LibreDWGc::Dwg_Entity_OLE2FRAME_data_size_get", _wrap_Dwg_Entity_OLE2FRAME_data_size_get},
{"LibreDWGc::Dwg_Entity_OLE2FRAME_data_set", _wrap_Dwg_Entity_OLE2FRAME_data_set},
{"LibreDWGc::Dwg_Entity_OLE2FRAME_data_get", _wrap_Dwg_Entity_OLE2FRAME_data_get},
{"LibreDWGc::Dwg_Entity_OLE2FRAME_oleversion_set", _wrap_Dwg_Entity_OLE2FRAME_oleversion_set},
{"LibreDWGc::Dwg_Entity_OLE2FRAME_oleversion_get", _wrap_Dwg_Entity_OLE2FRAME_oleversion_get},
{"LibreDWGc::Dwg_Entity_OLE2FRAME_oleclient_set", _wrap_Dwg_Entity_OLE2FRAME_oleclient_set},
{"LibreDWGc::Dwg_Entity_OLE2FRAME_oleclient_get", _wrap_Dwg_Entity_OLE2FRAME_oleclient_get},
{"LibreDWGc::Dwg_Entity_OLE2FRAME_pt1_set", _wrap_Dwg_Entity_OLE2FRAME_pt1_set},
{"LibreDWGc::Dwg_Entity_OLE2FRAME_pt1_get", _wrap_Dwg_Entity_OLE2FRAME_pt1_get},
{"LibreDWGc::Dwg_Entity_OLE2FRAME_pt2_set", _wrap_Dwg_Entity_OLE2FRAME_pt2_set},
{"LibreDWGc::Dwg_Entity_OLE2FRAME_pt2_get", _wrap_Dwg_Entity_OLE2FRAME_pt2_get},
{"LibreDWGc::new_Dwg_Entity_OLE2FRAME", _wrap_new_Dwg_Entity_OLE2FRAME},
{"LibreDWGc::delete_Dwg_Entity_OLE2FRAME", _wrap_delete_Dwg_Entity_OLE2FRAME},
{"LibreDWGc::Dwg_Object_DUMMY_parent_set", _wrap_Dwg_Object_DUMMY_parent_set},
{"LibreDWGc::Dwg_Object_DUMMY_parent_get", _wrap_Dwg_Object_DUMMY_parent_get},
{"LibreDWGc::new_Dwg_Object_DUMMY", _wrap_new_Dwg_Object_DUMMY},
{"LibreDWGc::delete_Dwg_Object_DUMMY", _wrap_delete_Dwg_Object_DUMMY},
{"LibreDWGc::Dwg_Object_LONG_TRANSACTION_parent_set", _wrap_Dwg_Object_LONG_TRANSACTION_parent_set},
{"LibreDWGc::Dwg_Object_LONG_TRANSACTION_parent_get", _wrap_Dwg_Object_LONG_TRANSACTION_parent_get},
{"LibreDWGc::new_Dwg_Object_LONG_TRANSACTION", _wrap_new_Dwg_Object_LONG_TRANSACTION},
{"LibreDWGc::delete_Dwg_Object_LONG_TRANSACTION", _wrap_delete_Dwg_Object_LONG_TRANSACTION},
{"LibreDWGc::Dwg_LWPOLYLINE_width_start_set", _wrap_Dwg_LWPOLYLINE_width_start_set},
{"LibreDWGc::Dwg_LWPOLYLINE_width_start_get", _wrap_Dwg_LWPOLYLINE_width_start_get},
{"LibreDWGc::Dwg_LWPOLYLINE_width_end_set", _wrap_Dwg_LWPOLYLINE_width_end_set},
{"LibreDWGc::Dwg_LWPOLYLINE_width_end_get", _wrap_Dwg_LWPOLYLINE_width_end_get},
{"LibreDWGc::new_Dwg_LWPOLYLINE_width", _wrap_new_Dwg_LWPOLYLINE_width},
{"LibreDWGc::delete_Dwg_LWPOLYLINE_width", _wrap_delete_Dwg_LWPOLYLINE_width},
{"LibreDWGc::Dwg_PROXY_LWPOLYLINE_parent_set", _wrap_Dwg_PROXY_LWPOLYLINE_parent_set},
{"LibreDWGc::Dwg_PROXY_LWPOLYLINE_parent_get", _wrap_Dwg_PROXY_LWPOLYLINE_parent_get},
{"LibreDWGc::Dwg_PROXY_LWPOLYLINE_data_size_set", _wrap_Dwg_PROXY_LWPOLYLINE_data_size_set},
{"LibreDWGc::Dwg_PROXY_LWPOLYLINE_data_size_get", _wrap_Dwg_PROXY_LWPOLYLINE_data_size_get},
{"LibreDWGc::Dwg_PROXY_LWPOLYLINE_flags_set", _wrap_Dwg_PROXY_LWPOLYLINE_flags_set},
{"LibreDWGc::Dwg_PROXY_LWPOLYLINE_flags_get", _wrap_Dwg_PROXY_LWPOLYLINE_flags_get},
{"LibreDWGc::Dwg_PROXY_LWPOLYLINE_const_width_set", _wrap_Dwg_PROXY_LWPOLYLINE_const_width_set},
{"LibreDWGc::Dwg_PROXY_LWPOLYLINE_const_width_get", _wrap_Dwg_PROXY_LWPOLYLINE_const_width_get},
{"LibreDWGc::Dwg_PROXY_LWPOLYLINE_elevation_set", _wrap_Dwg_PROXY_LWPOLYLINE_elevation_set},
{"LibreDWGc::Dwg_PROXY_LWPOLYLINE_elevation_get", _wrap_Dwg_PROXY_LWPOLYLINE_elevation_get},
{"LibreDWGc::Dwg_PROXY_LWPOLYLINE_thickness_set", _wrap_Dwg_PROXY_LWPOLYLINE_thickness_set},
{"LibreDWGc::Dwg_PROXY_LWPOLYLINE_thickness_get", _wrap_Dwg_PROXY_LWPOLYLINE_thickness_get},
{"LibreDWGc::Dwg_PROXY_LWPOLYLINE_extrusion_set", _wrap_Dwg_PROXY_LWPOLYLINE_extrusion_set},
{"LibreDWGc::Dwg_PROXY_LWPOLYLINE_extrusion_get", _wrap_Dwg_PROXY_LWPOLYLINE_extrusion_get},
{"LibreDWGc::Dwg_PROXY_LWPOLYLINE_num_points_set", _wrap_Dwg_PROXY_LWPOLYLINE_num_points_set},
{"LibreDWGc::Dwg_PROXY_LWPOLYLINE_num_points_get", _wrap_Dwg_PROXY_LWPOLYLINE_num_points_get},
{"LibreDWGc::Dwg_PROXY_LWPOLYLINE_points_set", _wrap_Dwg_PROXY_LWPOLYLINE_points_set},
{"LibreDWGc::Dwg_PROXY_LWPOLYLINE_points_get", _wrap_Dwg_PROXY_LWPOLYLINE_points_get},
{"LibreDWGc::Dwg_PROXY_LWPOLYLINE_num_bulges_set", _wrap_Dwg_PROXY_LWPOLYLINE_num_bulges_set},
{"LibreDWGc::Dwg_PROXY_LWPOLYLINE_num_bulges_get", _wrap_Dwg_PROXY_LWPOLYLINE_num_bulges_get},
{"LibreDWGc::Dwg_PROXY_LWPOLYLINE_bulges_set", _wrap_Dwg_PROXY_LWPOLYLINE_bulges_set},
{"LibreDWGc::Dwg_PROXY_LWPOLYLINE_bulges_get", _wrap_Dwg_PROXY_LWPOLYLINE_bulges_get},
{"LibreDWGc::Dwg_PROXY_LWPOLYLINE_num_widths_set", _wrap_Dwg_PROXY_LWPOLYLINE_num_widths_set},
{"LibreDWGc::Dwg_PROXY_LWPOLYLINE_num_widths_get", _wrap_Dwg_PROXY_LWPOLYLINE_num_widths_get},
{"LibreDWGc::Dwg_PROXY_LWPOLYLINE_widths_set", _wrap_Dwg_PROXY_LWPOLYLINE_widths_set},
{"LibreDWGc::Dwg_PROXY_LWPOLYLINE_widths_get", _wrap_Dwg_PROXY_LWPOLYLINE_widths_get},
{"LibreDWGc::Dwg_PROXY_LWPOLYLINE_unknown_1_set", _wrap_Dwg_PROXY_LWPOLYLINE_unknown_1_set},
{"LibreDWGc::Dwg_PROXY_LWPOLYLINE_unknown_1_get", _wrap_Dwg_PROXY_LWPOLYLINE_unknown_1_get},
{"LibreDWGc::Dwg_PROXY_LWPOLYLINE_unknown_2_set", _wrap_Dwg_PROXY_LWPOLYLINE_unknown_2_set},
{"LibreDWGc::Dwg_PROXY_LWPOLYLINE_unknown_2_get", _wrap_Dwg_PROXY_LWPOLYLINE_unknown_2_get},
{"LibreDWGc::Dwg_PROXY_LWPOLYLINE_unknown_3_set", _wrap_Dwg_PROXY_LWPOLYLINE_unknown_3_set},
{"LibreDWGc::Dwg_PROXY_LWPOLYLINE_unknown_3_get", _wrap_Dwg_PROXY_LWPOLYLINE_unknown_3_get},
{"LibreDWGc::new_Dwg_PROXY_LWPOLYLINE", _wrap_new_Dwg_PROXY_LWPOLYLINE},
{"LibreDWGc::delete_Dwg_PROXY_LWPOLYLINE", _wrap_delete_Dwg_PROXY_LWPOLYLINE},
{"LibreDWGc::Dwg_Entity_PROXY_ENTITY_parent_set", _wrap_Dwg_Entity_PROXY_ENTITY_parent_set},
{"LibreDWGc::Dwg_Entity_PROXY_ENTITY_parent_get", _wrap_Dwg_Entity_PROXY_ENTITY_parent_get},
{"LibreDWGc::Dwg_Entity_PROXY_ENTITY_proxy_id_set", _wrap_Dwg_Entity_PROXY_ENTITY_proxy_id_set},
{"LibreDWGc::Dwg_Entity_PROXY_ENTITY_proxy_id_get", _wrap_Dwg_Entity_PROXY_ENTITY_proxy_id_get},
{"LibreDWGc::Dwg_Entity_PROXY_ENTITY_class_id_set", _wrap_Dwg_Entity_PROXY_ENTITY_class_id_set},
{"LibreDWGc::Dwg_Entity_PROXY_ENTITY_class_id_get", _wrap_Dwg_Entity_PROXY_ENTITY_class_id_get},
{"LibreDWGc::Dwg_Entity_PROXY_ENTITY_dwg_versions_set", _wrap_Dwg_Entity_PROXY_ENTITY_dwg_versions_set},
{"LibreDWGc::Dwg_Entity_PROXY_ENTITY_dwg_versions_get", _wrap_Dwg_Entity_PROXY_ENTITY_dwg_versions_get},
{"LibreDWGc::Dwg_Entity_PROXY_ENTITY_maint_version_set", _wrap_Dwg_Entity_PROXY_ENTITY_maint_version_set},
{"LibreDWGc::Dwg_Entity_PROXY_ENTITY_maint_version_get", _wrap_Dwg_Entity_PROXY_ENTITY_maint_version_get},
{"LibreDWGc::Dwg_Entity_PROXY_ENTITY_dwg_version_set", _wrap_Dwg_Entity_PROXY_ENTITY_dwg_version_set},
{"LibreDWGc::Dwg_Entity_PROXY_ENTITY_dwg_version_get", _wrap_Dwg_Entity_PROXY_ENTITY_dwg_version_get},
{"LibreDWGc::Dwg_Entity_PROXY_ENTITY_from_dxf_set", _wrap_Dwg_Entity_PROXY_ENTITY_from_dxf_set},
{"LibreDWGc::Dwg_Entity_PROXY_ENTITY_from_dxf_get", _wrap_Dwg_Entity_PROXY_ENTITY_from_dxf_get},
{"LibreDWGc::Dwg_Entity_PROXY_ENTITY_proxy_data_size_set", _wrap_Dwg_Entity_PROXY_ENTITY_proxy_data_size_set},
{"LibreDWGc::Dwg_Entity_PROXY_ENTITY_proxy_data_size_get", _wrap_Dwg_Entity_PROXY_ENTITY_proxy_data_size_get},
{"LibreDWGc::Dwg_Entity_PROXY_ENTITY_proxy_data_set", _wrap_Dwg_Entity_PROXY_ENTITY_proxy_data_set},
{"LibreDWGc::Dwg_Entity_PROXY_ENTITY_proxy_data_get", _wrap_Dwg_Entity_PROXY_ENTITY_proxy_data_get},
{"LibreDWGc::Dwg_Entity_PROXY_ENTITY_data_numbits_set", _wrap_Dwg_Entity_PROXY_ENTITY_data_numbits_set},
{"LibreDWGc::Dwg_Entity_PROXY_ENTITY_data_numbits_get", _wrap_Dwg_Entity_PROXY_ENTITY_data_numbits_get},
{"LibreDWGc::Dwg_Entity_PROXY_ENTITY_data_size_set", _wrap_Dwg_Entity_PROXY_ENTITY_data_size_set},
{"LibreDWGc::Dwg_Entity_PROXY_ENTITY_data_size_get", _wrap_Dwg_Entity_PROXY_ENTITY_data_size_get},
{"LibreDWGc::Dwg_Entity_PROXY_ENTITY_data_set", _wrap_Dwg_Entity_PROXY_ENTITY_data_set},
{"LibreDWGc::Dwg_Entity_PROXY_ENTITY_data_get", _wrap_Dwg_Entity_PROXY_ENTITY_data_get},
{"LibreDWGc::Dwg_Entity_PROXY_ENTITY_num_objids_set", _wrap_Dwg_Entity_PROXY_ENTITY_num_objids_set},
{"LibreDWGc::Dwg_Entity_PROXY_ENTITY_num_objids_get", _wrap_Dwg_Entity_PROXY_ENTITY_num_objids_get},
{"LibreDWGc::Dwg_Entity_PROXY_ENTITY_objids_set", _wrap_Dwg_Entity_PROXY_ENTITY_objids_set},
{"LibreDWGc::Dwg_Entity_PROXY_ENTITY_objids_get", _wrap_Dwg_Entity_PROXY_ENTITY_objids_get},
{"LibreDWGc::new_Dwg_Entity_PROXY_ENTITY", _wrap_new_Dwg_Entity_PROXY_ENTITY},
{"LibreDWGc::delete_Dwg_Entity_PROXY_ENTITY", _wrap_delete_Dwg_Entity_PROXY_ENTITY},
{"LibreDWGc::Dwg_Object_PROXY_OBJECT_parent_set", _wrap_Dwg_Object_PROXY_OBJECT_parent_set},
{"LibreDWGc::Dwg_Object_PROXY_OBJECT_parent_get", _wrap_Dwg_Object_PROXY_OBJECT_parent_get},
{"LibreDWGc::Dwg_Object_PROXY_OBJECT_proxy_id_set", _wrap_Dwg_Object_PROXY_OBJECT_proxy_id_set},
{"LibreDWGc::Dwg_Object_PROXY_OBJECT_proxy_id_get", _wrap_Dwg_Object_PROXY_OBJECT_proxy_id_get},
{"LibreDWGc::Dwg_Object_PROXY_OBJECT_class_id_set", _wrap_Dwg_Object_PROXY_OBJECT_class_id_set},
{"LibreDWGc::Dwg_Object_PROXY_OBJECT_class_id_get", _wrap_Dwg_Object_PROXY_OBJECT_class_id_get},
{"LibreDWGc::Dwg_Object_PROXY_OBJECT_dwg_versions_set", _wrap_Dwg_Object_PROXY_OBJECT_dwg_versions_set},
{"LibreDWGc::Dwg_Object_PROXY_OBJECT_dwg_versions_get", _wrap_Dwg_Object_PROXY_OBJECT_dwg_versions_get},
{"LibreDWGc::Dwg_Object_PROXY_OBJECT_maint_version_set", _wrap_Dwg_Object_PROXY_OBJECT_maint_version_set},
{"LibreDWGc::Dwg_Object_PROXY_OBJECT_maint_version_get", _wrap_Dwg_Object_PROXY_OBJECT_maint_version_get},
{"LibreDWGc::Dwg_Object_PROXY_OBJECT_dwg_version_set", _wrap_Dwg_Object_PROXY_OBJECT_dwg_version_set},
{"LibreDWGc::Dwg_Object_PROXY_OBJECT_dwg_version_get", _wrap_Dwg_Object_PROXY_OBJECT_dwg_version_get},
{"LibreDWGc::Dwg_Object_PROXY_OBJECT_from_dxf_set", _wrap_Dwg_Object_PROXY_OBJECT_from_dxf_set},
{"LibreDWGc::Dwg_Object_PROXY_OBJECT_from_dxf_get", _wrap_Dwg_Object_PROXY_OBJECT_from_dxf_get},
{"LibreDWGc::Dwg_Object_PROXY_OBJECT_data_numbits_set", _wrap_Dwg_Object_PROXY_OBJECT_data_numbits_set},
{"LibreDWGc::Dwg_Object_PROXY_OBJECT_data_numbits_get", _wrap_Dwg_Object_PROXY_OBJECT_data_numbits_get},
{"LibreDWGc::Dwg_Object_PROXY_OBJECT_data_size_set", _wrap_Dwg_Object_PROXY_OBJECT_data_size_set},
{"LibreDWGc::Dwg_Object_PROXY_OBJECT_data_size_get", _wrap_Dwg_Object_PROXY_OBJECT_data_size_get},
{"LibreDWGc::Dwg_Object_PROXY_OBJECT_data_set", _wrap_Dwg_Object_PROXY_OBJECT_data_set},
{"LibreDWGc::Dwg_Object_PROXY_OBJECT_data_get", _wrap_Dwg_Object_PROXY_OBJECT_data_get},
{"LibreDWGc::Dwg_Object_PROXY_OBJECT_num_objids_set", _wrap_Dwg_Object_PROXY_OBJECT_num_objids_set},
{"LibreDWGc::Dwg_Object_PROXY_OBJECT_num_objids_get", _wrap_Dwg_Object_PROXY_OBJECT_num_objids_get},
{"LibreDWGc::Dwg_Object_PROXY_OBJECT_objids_set", _wrap_Dwg_Object_PROXY_OBJECT_objids_set},
{"LibreDWGc::Dwg_Object_PROXY_OBJECT_objids_get", _wrap_Dwg_Object_PROXY_OBJECT_objids_get},
{"LibreDWGc::new_Dwg_Object_PROXY_OBJECT", _wrap_new_Dwg_Object_PROXY_OBJECT},
{"LibreDWGc::delete_Dwg_Object_PROXY_OBJECT", _wrap_delete_Dwg_Object_PROXY_OBJECT},
{"LibreDWGc::Dwg_HATCH_Color_parent_set", _wrap_Dwg_HATCH_Color_parent_set},
{"LibreDWGc::Dwg_HATCH_Color_parent_get", _wrap_Dwg_HATCH_Color_parent_get},
{"LibreDWGc::Dwg_HATCH_Color_shift_value_set", _wrap_Dwg_HATCH_Color_shift_value_set},
{"LibreDWGc::Dwg_HATCH_Color_shift_value_get", _wrap_Dwg_HATCH_Color_shift_value_get},
{"LibreDWGc::Dwg_HATCH_Color_color_set", _wrap_Dwg_HATCH_Color_color_set},
{"LibreDWGc::Dwg_HATCH_Color_color_get", _wrap_Dwg_HATCH_Color_color_get},
{"LibreDWGc::new_Dwg_HATCH_Color", _wrap_new_Dwg_HATCH_Color},
{"LibreDWGc::delete_Dwg_HATCH_Color", _wrap_delete_Dwg_HATCH_Color},
{"LibreDWGc::Dwg_HATCH_ControlPoint_parent_set", _wrap_Dwg_HATCH_ControlPoint_parent_set},
{"LibreDWGc::Dwg_HATCH_ControlPoint_parent_get", _wrap_Dwg_HATCH_ControlPoint_parent_get},
{"LibreDWGc::Dwg_HATCH_ControlPoint_point_set", _wrap_Dwg_HATCH_ControlPoint_point_set},
{"LibreDWGc::Dwg_HATCH_ControlPoint_point_get", _wrap_Dwg_HATCH_ControlPoint_point_get},
{"LibreDWGc::Dwg_HATCH_ControlPoint_weight_set", _wrap_Dwg_HATCH_ControlPoint_weight_set},
{"LibreDWGc::Dwg_HATCH_ControlPoint_weight_get", _wrap_Dwg_HATCH_ControlPoint_weight_get},
{"LibreDWGc::new_Dwg_HATCH_ControlPoint", _wrap_new_Dwg_HATCH_ControlPoint},
{"LibreDWGc::delete_Dwg_HATCH_ControlPoint", _wrap_delete_Dwg_HATCH_ControlPoint},
{"LibreDWGc::Dwg_HATCH_PathSeg_parent_set", _wrap_Dwg_HATCH_PathSeg_parent_set},
{"LibreDWGc::Dwg_HATCH_PathSeg_parent_get", _wrap_Dwg_HATCH_PathSeg_parent_get},
{"LibreDWGc::Dwg_HATCH_PathSeg_curve_type_set", _wrap_Dwg_HATCH_PathSeg_curve_type_set},
{"LibreDWGc::Dwg_HATCH_PathSeg_curve_type_get", _wrap_Dwg_HATCH_PathSeg_curve_type_get},
{"LibreDWGc::Dwg_HATCH_PathSeg_first_endpoint_set", _wrap_Dwg_HATCH_PathSeg_first_endpoint_set},
{"LibreDWGc::Dwg_HATCH_PathSeg_first_endpoint_get", _wrap_Dwg_HATCH_PathSeg_first_endpoint_get},
{"LibreDWGc::Dwg_HATCH_PathSeg_second_endpoint_set", _wrap_Dwg_HATCH_PathSeg_second_endpoint_set},
{"LibreDWGc::Dwg_HATCH_PathSeg_second_endpoint_get", _wrap_Dwg_HATCH_PathSeg_second_endpoint_get},
{"LibreDWGc::Dwg_HATCH_PathSeg_center_set", _wrap_Dwg_HATCH_PathSeg_center_set},
{"LibreDWGc::Dwg_HATCH_PathSeg_center_get", _wrap_Dwg_HATCH_PathSeg_center_get},
{"LibreDWGc::Dwg_HATCH_PathSeg_radius_set", _wrap_Dwg_HATCH_PathSeg_radius_set},
{"LibreDWGc::Dwg_HATCH_PathSeg_radius_get", _wrap_Dwg_HATCH_PathSeg_radius_get},
{"LibreDWGc::Dwg_HATCH_PathSeg_start_angle_set", _wrap_Dwg_HATCH_PathSeg_start_angle_set},
{"LibreDWGc::Dwg_HATCH_PathSeg_start_angle_get", _wrap_Dwg_HATCH_PathSeg_start_angle_get},
{"LibreDWGc::Dwg_HATCH_PathSeg_end_angle_set", _wrap_Dwg_HATCH_PathSeg_end_angle_set},
{"LibreDWGc::Dwg_HATCH_PathSeg_end_angle_get", _wrap_Dwg_HATCH_PathSeg_end_angle_get},
{"LibreDWGc::Dwg_HATCH_PathSeg_is_ccw_set", _wrap_Dwg_HATCH_PathSeg_is_ccw_set},
{"LibreDWGc::Dwg_HATCH_PathSeg_is_ccw_get", _wrap_Dwg_HATCH_PathSeg_is_ccw_get},
{"LibreDWGc::Dwg_HATCH_PathSeg_endpoint_set", _wrap_Dwg_HATCH_PathSeg_endpoint_set},
{"LibreDWGc::Dwg_HATCH_PathSeg_endpoint_get", _wrap_Dwg_HATCH_PathSeg_endpoint_get},
{"LibreDWGc::Dwg_HATCH_PathSeg_minor_major_ratio_set", _wrap_Dwg_HATCH_PathSeg_minor_major_ratio_set},
{"LibreDWGc::Dwg_HATCH_PathSeg_minor_major_ratio_get", _wrap_Dwg_HATCH_PathSeg_minor_major_ratio_get},
{"LibreDWGc::Dwg_HATCH_PathSeg_degree_set", _wrap_Dwg_HATCH_PathSeg_degree_set},
{"LibreDWGc::Dwg_HATCH_PathSeg_degree_get", _wrap_Dwg_HATCH_PathSeg_degree_get},
{"LibreDWGc::Dwg_HATCH_PathSeg_is_rational_set", _wrap_Dwg_HATCH_PathSeg_is_rational_set},
{"LibreDWGc::Dwg_HATCH_PathSeg_is_rational_get", _wrap_Dwg_HATCH_PathSeg_is_rational_get},
{"LibreDWGc::Dwg_HATCH_PathSeg_is_periodic_set", _wrap_Dwg_HATCH_PathSeg_is_periodic_set},
{"LibreDWGc::Dwg_HATCH_PathSeg_is_periodic_get", _wrap_Dwg_HATCH_PathSeg_is_periodic_get},
{"LibreDWGc::Dwg_HATCH_PathSeg_num_knots_set", _wrap_Dwg_HATCH_PathSeg_num_knots_set},
{"LibreDWGc::Dwg_HATCH_PathSeg_num_knots_get", _wrap_Dwg_HATCH_PathSeg_num_knots_get},
{"LibreDWGc::Dwg_HATCH_PathSeg_num_control_points_set", _wrap_Dwg_HATCH_PathSeg_num_control_points_set},
{"LibreDWGc::Dwg_HATCH_PathSeg_num_control_points_get", _wrap_Dwg_HATCH_PathSeg_num_control_points_get},
{"LibreDWGc::Dwg_HATCH_PathSeg_knots_set", _wrap_Dwg_HATCH_PathSeg_knots_set},
{"LibreDWGc::Dwg_HATCH_PathSeg_knots_get", _wrap_Dwg_HATCH_PathSeg_knots_get},
{"LibreDWGc::Dwg_HATCH_PathSeg_control_points_set", _wrap_Dwg_HATCH_PathSeg_control_points_set},
{"LibreDWGc::Dwg_HATCH_PathSeg_control_points_get", _wrap_Dwg_HATCH_PathSeg_control_points_get},
{"LibreDWGc::Dwg_HATCH_PathSeg_num_fitpts_set", _wrap_Dwg_HATCH_PathSeg_num_fitpts_set},
{"LibreDWGc::Dwg_HATCH_PathSeg_num_fitpts_get", _wrap_Dwg_HATCH_PathSeg_num_fitpts_get},
{"LibreDWGc::Dwg_HATCH_PathSeg_fitpts_set", _wrap_Dwg_HATCH_PathSeg_fitpts_set},
{"LibreDWGc::Dwg_HATCH_PathSeg_fitpts_get", _wrap_Dwg_HATCH_PathSeg_fitpts_get},
{"LibreDWGc::Dwg_HATCH_PathSeg_start_tangent_set", _wrap_Dwg_HATCH_PathSeg_start_tangent_set},
{"LibreDWGc::Dwg_HATCH_PathSeg_start_tangent_get", _wrap_Dwg_HATCH_PathSeg_start_tangent_get},
{"LibreDWGc::Dwg_HATCH_PathSeg_end_tangent_set", _wrap_Dwg_HATCH_PathSeg_end_tangent_set},
{"LibreDWGc::Dwg_HATCH_PathSeg_end_tangent_get", _wrap_Dwg_HATCH_PathSeg_end_tangent_get},
{"LibreDWGc::new_Dwg_HATCH_PathSeg", _wrap_new_Dwg_HATCH_PathSeg},
{"LibreDWGc::delete_Dwg_HATCH_PathSeg", _wrap_delete_Dwg_HATCH_PathSeg},
{"LibreDWGc::Dwg_HATCH_PolylinePath_parent_set", _wrap_Dwg_HATCH_PolylinePath_parent_set},
{"LibreDWGc::Dwg_HATCH_PolylinePath_parent_get", _wrap_Dwg_HATCH_PolylinePath_parent_get},
{"LibreDWGc::Dwg_HATCH_PolylinePath_point_set", _wrap_Dwg_HATCH_PolylinePath_point_set},
{"LibreDWGc::Dwg_HATCH_PolylinePath_point_get", _wrap_Dwg_HATCH_PolylinePath_point_get},
{"LibreDWGc::Dwg_HATCH_PolylinePath_bulge_set", _wrap_Dwg_HATCH_PolylinePath_bulge_set},
{"LibreDWGc::Dwg_HATCH_PolylinePath_bulge_get", _wrap_Dwg_HATCH_PolylinePath_bulge_get},
{"LibreDWGc::new_Dwg_HATCH_PolylinePath", _wrap_new_Dwg_HATCH_PolylinePath},
{"LibreDWGc::delete_Dwg_HATCH_PolylinePath", _wrap_delete_Dwg_HATCH_PolylinePath},
{"LibreDWGc::Dwg_HATCH_Path_parent_set", _wrap_Dwg_HATCH_Path_parent_set},
{"LibreDWGc::Dwg_HATCH_Path_parent_get", _wrap_Dwg_HATCH_Path_parent_get},
{"LibreDWGc::Dwg_HATCH_Path_flag_set", _wrap_Dwg_HATCH_Path_flag_set},
{"LibreDWGc::Dwg_HATCH_Path_flag_get", _wrap_Dwg_HATCH_Path_flag_get},
{"LibreDWGc::Dwg_HATCH_Path_num_segs_or_paths_set", _wrap_Dwg_HATCH_Path_num_segs_or_paths_set},
{"LibreDWGc::Dwg_HATCH_Path_num_segs_or_paths_get", _wrap_Dwg_HATCH_Path_num_segs_or_paths_get},
{"LibreDWGc::Dwg_HATCH_Path_segs_set", _wrap_Dwg_HATCH_Path_segs_set},
{"LibreDWGc::Dwg_HATCH_Path_segs_get", _wrap_Dwg_HATCH_Path_segs_get},
{"LibreDWGc::Dwg_HATCH_Path_bulges_present_set", _wrap_Dwg_HATCH_Path_bulges_present_set},
{"LibreDWGc::Dwg_HATCH_Path_bulges_present_get", _wrap_Dwg_HATCH_Path_bulges_present_get},
{"LibreDWGc::Dwg_HATCH_Path_closed_set", _wrap_Dwg_HATCH_Path_closed_set},
{"LibreDWGc::Dwg_HATCH_Path_closed_get", _wrap_Dwg_HATCH_Path_closed_get},
{"LibreDWGc::Dwg_HATCH_Path_polyline_paths_set", _wrap_Dwg_HATCH_Path_polyline_paths_set},
{"LibreDWGc::Dwg_HATCH_Path_polyline_paths_get", _wrap_Dwg_HATCH_Path_polyline_paths_get},
{"LibreDWGc::Dwg_HATCH_Path_num_boundary_handles_set", _wrap_Dwg_HATCH_Path_num_boundary_handles_set},
{"LibreDWGc::Dwg_HATCH_Path_num_boundary_handles_get", _wrap_Dwg_HATCH_Path_num_boundary_handles_get},
{"LibreDWGc::Dwg_HATCH_Path_boundary_handles_set", _wrap_Dwg_HATCH_Path_boundary_handles_set},
{"LibreDWGc::Dwg_HATCH_Path_boundary_handles_get", _wrap_Dwg_HATCH_Path_boundary_handles_get},
{"LibreDWGc::new_Dwg_HATCH_Path", _wrap_new_Dwg_HATCH_Path},
{"LibreDWGc::delete_Dwg_HATCH_Path", _wrap_delete_Dwg_HATCH_Path},
{"LibreDWGc::Dwg_HATCH_DefLine_parent_set", _wrap_Dwg_HATCH_DefLine_parent_set},
{"LibreDWGc::Dwg_HATCH_DefLine_parent_get", _wrap_Dwg_HATCH_DefLine_parent_get},
{"LibreDWGc::Dwg_HATCH_DefLine_angle_set", _wrap_Dwg_HATCH_DefLine_angle_set},
{"LibreDWGc::Dwg_HATCH_DefLine_angle_get", _wrap_Dwg_HATCH_DefLine_angle_get},
{"LibreDWGc::Dwg_HATCH_DefLine_pt0_set", _wrap_Dwg_HATCH_DefLine_pt0_set},
{"LibreDWGc::Dwg_HATCH_DefLine_pt0_get", _wrap_Dwg_HATCH_DefLine_pt0_get},
{"LibreDWGc::Dwg_HATCH_DefLine_offset_set", _wrap_Dwg_HATCH_DefLine_offset_set},
{"LibreDWGc::Dwg_HATCH_DefLine_offset_get", _wrap_Dwg_HATCH_DefLine_offset_get},
{"LibreDWGc::Dwg_HATCH_DefLine_num_dashes_set", _wrap_Dwg_HATCH_DefLine_num_dashes_set},
{"LibreDWGc::Dwg_HATCH_DefLine_num_dashes_get", _wrap_Dwg_HATCH_DefLine_num_dashes_get},
{"LibreDWGc::Dwg_HATCH_DefLine_dashes_set", _wrap_Dwg_HATCH_DefLine_dashes_set},
{"LibreDWGc::Dwg_HATCH_DefLine_dashes_get", _wrap_Dwg_HATCH_DefLine_dashes_get},
{"LibreDWGc::new_Dwg_HATCH_DefLine", _wrap_new_Dwg_HATCH_DefLine},
{"LibreDWGc::delete_Dwg_HATCH_DefLine", _wrap_delete_Dwg_HATCH_DefLine},
{"LibreDWGc::Dwg_Entity_HATCH_parent_set", _wrap_Dwg_Entity_HATCH_parent_set},
{"LibreDWGc::Dwg_Entity_HATCH_parent_get", _wrap_Dwg_Entity_HATCH_parent_get},
{"LibreDWGc::Dwg_Entity_HATCH_is_gradient_fill_set", _wrap_Dwg_Entity_HATCH_is_gradient_fill_set},
{"LibreDWGc::Dwg_Entity_HATCH_is_gradient_fill_get", _wrap_Dwg_Entity_HATCH_is_gradient_fill_get},
{"LibreDWGc::Dwg_Entity_HATCH_reserved_set", _wrap_Dwg_Entity_HATCH_reserved_set},
{"LibreDWGc::Dwg_Entity_HATCH_reserved_get", _wrap_Dwg_Entity_HATCH_reserved_get},
{"LibreDWGc::Dwg_Entity_HATCH_gradient_angle_set", _wrap_Dwg_Entity_HATCH_gradient_angle_set},
{"LibreDWGc::Dwg_Entity_HATCH_gradient_angle_get", _wrap_Dwg_Entity_HATCH_gradient_angle_get},
{"LibreDWGc::Dwg_Entity_HATCH_gradient_shift_set", _wrap_Dwg_Entity_HATCH_gradient_shift_set},
{"LibreDWGc::Dwg_Entity_HATCH_gradient_shift_get", _wrap_Dwg_Entity_HATCH_gradient_shift_get},
{"LibreDWGc::Dwg_Entity_HATCH_single_color_gradient_set", _wrap_Dwg_Entity_HATCH_single_color_gradient_set},
{"LibreDWGc::Dwg_Entity_HATCH_single_color_gradient_get", _wrap_Dwg_Entity_HATCH_single_color_gradient_get},
{"LibreDWGc::Dwg_Entity_HATCH_gradient_tint_set", _wrap_Dwg_Entity_HATCH_gradient_tint_set},
{"LibreDWGc::Dwg_Entity_HATCH_gradient_tint_get", _wrap_Dwg_Entity_HATCH_gradient_tint_get},
{"LibreDWGc::Dwg_Entity_HATCH_num_colors_set", _wrap_Dwg_Entity_HATCH_num_colors_set},
{"LibreDWGc::Dwg_Entity_HATCH_num_colors_get", _wrap_Dwg_Entity_HATCH_num_colors_get},
{"LibreDWGc::Dwg_Entity_HATCH_colors_set", _wrap_Dwg_Entity_HATCH_colors_set},
{"LibreDWGc::Dwg_Entity_HATCH_colors_get", _wrap_Dwg_Entity_HATCH_colors_get},
{"LibreDWGc::Dwg_Entity_HATCH_gradient_name_set", _wrap_Dwg_Entity_HATCH_gradient_name_set},
{"LibreDWGc::Dwg_Entity_HATCH_gradient_name_get", _wrap_Dwg_Entity_HATCH_gradient_name_get},
{"LibreDWGc::Dwg_Entity_HATCH_elevation_set", _wrap_Dwg_Entity_HATCH_elevation_set},
{"LibreDWGc::Dwg_Entity_HATCH_elevation_get", _wrap_Dwg_Entity_HATCH_elevation_get},
{"LibreDWGc::Dwg_Entity_HATCH_extrusion_set", _wrap_Dwg_Entity_HATCH_extrusion_set},
{"LibreDWGc::Dwg_Entity_HATCH_extrusion_get", _wrap_Dwg_Entity_HATCH_extrusion_get},
{"LibreDWGc::Dwg_Entity_HATCH_name_set", _wrap_Dwg_Entity_HATCH_name_set},
{"LibreDWGc::Dwg_Entity_HATCH_name_get", _wrap_Dwg_Entity_HATCH_name_get},
{"LibreDWGc::Dwg_Entity_HATCH_is_solid_fill_set", _wrap_Dwg_Entity_HATCH_is_solid_fill_set},
{"LibreDWGc::Dwg_Entity_HATCH_is_solid_fill_get", _wrap_Dwg_Entity_HATCH_is_solid_fill_get},
{"LibreDWGc::Dwg_Entity_HATCH_is_associative_set", _wrap_Dwg_Entity_HATCH_is_associative_set},
{"LibreDWGc::Dwg_Entity_HATCH_is_associative_get", _wrap_Dwg_Entity_HATCH_is_associative_get},
{"LibreDWGc::Dwg_Entity_HATCH_num_paths_set", _wrap_Dwg_Entity_HATCH_num_paths_set},
{"LibreDWGc::Dwg_Entity_HATCH_num_paths_get", _wrap_Dwg_Entity_HATCH_num_paths_get},
{"LibreDWGc::Dwg_Entity_HATCH_paths_set", _wrap_Dwg_Entity_HATCH_paths_set},
{"LibreDWGc::Dwg_Entity_HATCH_paths_get", _wrap_Dwg_Entity_HATCH_paths_get},
{"LibreDWGc::Dwg_Entity_HATCH_style_set", _wrap_Dwg_Entity_HATCH_style_set},
{"LibreDWGc::Dwg_Entity_HATCH_style_get", _wrap_Dwg_Entity_HATCH_style_get},
{"LibreDWGc::Dwg_Entity_HATCH_pattern_type_set", _wrap_Dwg_Entity_HATCH_pattern_type_set},
{"LibreDWGc::Dwg_Entity_HATCH_pattern_type_get", _wrap_Dwg_Entity_HATCH_pattern_type_get},
{"LibreDWGc::Dwg_Entity_HATCH_angle_set", _wrap_Dwg_Entity_HATCH_angle_set},
{"LibreDWGc::Dwg_Entity_HATCH_angle_get", _wrap_Dwg_Entity_HATCH_angle_get},
{"LibreDWGc::Dwg_Entity_HATCH_scale_spacing_set", _wrap_Dwg_Entity_HATCH_scale_spacing_set},
{"LibreDWGc::Dwg_Entity_HATCH_scale_spacing_get", _wrap_Dwg_Entity_HATCH_scale_spacing_get},
{"LibreDWGc::Dwg_Entity_HATCH_double_flag_set", _wrap_Dwg_Entity_HATCH_double_flag_set},
{"LibreDWGc::Dwg_Entity_HATCH_double_flag_get", _wrap_Dwg_Entity_HATCH_double_flag_get},
{"LibreDWGc::Dwg_Entity_HATCH_num_deflines_set", _wrap_Dwg_Entity_HATCH_num_deflines_set},
{"LibreDWGc::Dwg_Entity_HATCH_num_deflines_get", _wrap_Dwg_Entity_HATCH_num_deflines_get},
{"LibreDWGc::Dwg_Entity_HATCH_deflines_set", _wrap_Dwg_Entity_HATCH_deflines_set},
{"LibreDWGc::Dwg_Entity_HATCH_deflines_get", _wrap_Dwg_Entity_HATCH_deflines_get},
{"LibreDWGc::Dwg_Entity_HATCH_has_derived_set", _wrap_Dwg_Entity_HATCH_has_derived_set},
{"LibreDWGc::Dwg_Entity_HATCH_has_derived_get", _wrap_Dwg_Entity_HATCH_has_derived_get},
{"LibreDWGc::Dwg_Entity_HATCH_pixel_size_set", _wrap_Dwg_Entity_HATCH_pixel_size_set},
{"LibreDWGc::Dwg_Entity_HATCH_pixel_size_get", _wrap_Dwg_Entity_HATCH_pixel_size_get},
{"LibreDWGc::Dwg_Entity_HATCH_num_seeds_set", _wrap_Dwg_Entity_HATCH_num_seeds_set},
{"LibreDWGc::Dwg_Entity_HATCH_num_seeds_get", _wrap_Dwg_Entity_HATCH_num_seeds_get},
{"LibreDWGc::Dwg_Entity_HATCH_seeds_set", _wrap_Dwg_Entity_HATCH_seeds_set},
{"LibreDWGc::Dwg_Entity_HATCH_seeds_get", _wrap_Dwg_Entity_HATCH_seeds_get},
{"LibreDWGc::new_Dwg_Entity_HATCH", _wrap_new_Dwg_Entity_HATCH},
{"LibreDWGc::delete_Dwg_Entity_HATCH", _wrap_delete_Dwg_Entity_HATCH},
{"LibreDWGc::Dwg_Entity_MPOLYGON_parent_set", _wrap_Dwg_Entity_MPOLYGON_parent_set},
{"LibreDWGc::Dwg_Entity_MPOLYGON_parent_get", _wrap_Dwg_Entity_MPOLYGON_parent_get},
{"LibreDWGc::Dwg_Entity_MPOLYGON_is_gradient_fill_set", _wrap_Dwg_Entity_MPOLYGON_is_gradient_fill_set},
{"LibreDWGc::Dwg_Entity_MPOLYGON_is_gradient_fill_get", _wrap_Dwg_Entity_MPOLYGON_is_gradient_fill_get},
{"LibreDWGc::Dwg_Entity_MPOLYGON_reserved_set", _wrap_Dwg_Entity_MPOLYGON_reserved_set},
{"LibreDWGc::Dwg_Entity_MPOLYGON_reserved_get", _wrap_Dwg_Entity_MPOLYGON_reserved_get},
{"LibreDWGc::Dwg_Entity_MPOLYGON_gradient_angle_set", _wrap_Dwg_Entity_MPOLYGON_gradient_angle_set},
{"LibreDWGc::Dwg_Entity_MPOLYGON_gradient_angle_get", _wrap_Dwg_Entity_MPOLYGON_gradient_angle_get},
{"LibreDWGc::Dwg_Entity_MPOLYGON_gradient_shift_set", _wrap_Dwg_Entity_MPOLYGON_gradient_shift_set},
{"LibreDWGc::Dwg_Entity_MPOLYGON_gradient_shift_get", _wrap_Dwg_Entity_MPOLYGON_gradient_shift_get},
{"LibreDWGc::Dwg_Entity_MPOLYGON_single_color_gradient_set", _wrap_Dwg_Entity_MPOLYGON_single_color_gradient_set},
{"LibreDWGc::Dwg_Entity_MPOLYGON_single_color_gradient_get", _wrap_Dwg_Entity_MPOLYGON_single_color_gradient_get},
{"LibreDWGc::Dwg_Entity_MPOLYGON_gradient_tint_set", _wrap_Dwg_Entity_MPOLYGON_gradient_tint_set},
{"LibreDWGc::Dwg_Entity_MPOLYGON_gradient_tint_get", _wrap_Dwg_Entity_MPOLYGON_gradient_tint_get},
{"LibreDWGc::Dwg_Entity_MPOLYGON_num_colors_set", _wrap_Dwg_Entity_MPOLYGON_num_colors_set},
{"LibreDWGc::Dwg_Entity_MPOLYGON_num_colors_get", _wrap_Dwg_Entity_MPOLYGON_num_colors_get},
{"LibreDWGc::Dwg_Entity_MPOLYGON_colors_set", _wrap_Dwg_Entity_MPOLYGON_colors_set},
{"LibreDWGc::Dwg_Entity_MPOLYGON_colors_get", _wrap_Dwg_Entity_MPOLYGON_colors_get},
{"LibreDWGc::Dwg_Entity_MPOLYGON_gradient_name_set", _wrap_Dwg_Entity_MPOLYGON_gradient_name_set},
{"LibreDWGc::Dwg_Entity_MPOLYGON_gradient_name_get", _wrap_Dwg_Entity_MPOLYGON_gradient_name_get},
{"LibreDWGc::Dwg_Entity_MPOLYGON_elevation_set", _wrap_Dwg_Entity_MPOLYGON_elevation_set},
{"LibreDWGc::Dwg_Entity_MPOLYGON_elevation_get", _wrap_Dwg_Entity_MPOLYGON_elevation_get},
{"LibreDWGc::Dwg_Entity_MPOLYGON_extrusion_set", _wrap_Dwg_Entity_MPOLYGON_extrusion_set},
{"LibreDWGc::Dwg_Entity_MPOLYGON_extrusion_get", _wrap_Dwg_Entity_MPOLYGON_extrusion_get},
{"LibreDWGc::Dwg_Entity_MPOLYGON_name_set", _wrap_Dwg_Entity_MPOLYGON_name_set},
{"LibreDWGc::Dwg_Entity_MPOLYGON_name_get", _wrap_Dwg_Entity_MPOLYGON_name_get},
{"LibreDWGc::Dwg_Entity_MPOLYGON_is_solid_fill_set", _wrap_Dwg_Entity_MPOLYGON_is_solid_fill_set},
{"LibreDWGc::Dwg_Entity_MPOLYGON_is_solid_fill_get", _wrap_Dwg_Entity_MPOLYGON_is_solid_fill_get},
{"LibreDWGc::Dwg_Entity_MPOLYGON_is_associative_set", _wrap_Dwg_Entity_MPOLYGON_is_associative_set},
{"LibreDWGc::Dwg_Entity_MPOLYGON_is_associative_get", _wrap_Dwg_Entity_MPOLYGON_is_associative_get},
{"LibreDWGc::Dwg_Entity_MPOLYGON_num_paths_set", _wrap_Dwg_Entity_MPOLYGON_num_paths_set},
{"LibreDWGc::Dwg_Entity_MPOLYGON_num_paths_get", _wrap_Dwg_Entity_MPOLYGON_num_paths_get},
{"LibreDWGc::Dwg_Entity_MPOLYGON_paths_set", _wrap_Dwg_Entity_MPOLYGON_paths_set},
{"LibreDWGc::Dwg_Entity_MPOLYGON_paths_get", _wrap_Dwg_Entity_MPOLYGON_paths_get},
{"LibreDWGc::Dwg_Entity_MPOLYGON_style_set", _wrap_Dwg_Entity_MPOLYGON_style_set},
{"LibreDWGc::Dwg_Entity_MPOLYGON_style_get", _wrap_Dwg_Entity_MPOLYGON_style_get},
{"LibreDWGc::Dwg_Entity_MPOLYGON_pattern_type_set", _wrap_Dwg_Entity_MPOLYGON_pattern_type_set},
{"LibreDWGc::Dwg_Entity_MPOLYGON_pattern_type_get", _wrap_Dwg_Entity_MPOLYGON_pattern_type_get},
{"LibreDWGc::Dwg_Entity_MPOLYGON_angle_set", _wrap_Dwg_Entity_MPOLYGON_angle_set},
{"LibreDWGc::Dwg_Entity_MPOLYGON_angle_get", _wrap_Dwg_Entity_MPOLYGON_angle_get},
{"LibreDWGc::Dwg_Entity_MPOLYGON_scale_spacing_set", _wrap_Dwg_Entity_MPOLYGON_scale_spacing_set},
{"LibreDWGc::Dwg_Entity_MPOLYGON_scale_spacing_get", _wrap_Dwg_Entity_MPOLYGON_scale_spacing_get},
{"LibreDWGc::Dwg_Entity_MPOLYGON_double_flag_set", _wrap_Dwg_Entity_MPOLYGON_double_flag_set},
{"LibreDWGc::Dwg_Entity_MPOLYGON_double_flag_get", _wrap_Dwg_Entity_MPOLYGON_double_flag_get},
{"LibreDWGc::Dwg_Entity_MPOLYGON_num_deflines_set", _wrap_Dwg_Entity_MPOLYGON_num_deflines_set},
{"LibreDWGc::Dwg_Entity_MPOLYGON_num_deflines_get", _wrap_Dwg_Entity_MPOLYGON_num_deflines_get},
{"LibreDWGc::Dwg_Entity_MPOLYGON_deflines_set", _wrap_Dwg_Entity_MPOLYGON_deflines_set},
{"LibreDWGc::Dwg_Entity_MPOLYGON_deflines_get", _wrap_Dwg_Entity_MPOLYGON_deflines_get},
{"LibreDWGc::Dwg_Entity_MPOLYGON_hatch_color_set", _wrap_Dwg_Entity_MPOLYGON_hatch_color_set},
{"LibreDWGc::Dwg_Entity_MPOLYGON_hatch_color_get", _wrap_Dwg_Entity_MPOLYGON_hatch_color_get},
{"LibreDWGc::Dwg_Entity_MPOLYGON_x_dir_set", _wrap_Dwg_Entity_MPOLYGON_x_dir_set},
{"LibreDWGc::Dwg_Entity_MPOLYGON_x_dir_get", _wrap_Dwg_Entity_MPOLYGON_x_dir_get},
{"LibreDWGc::Dwg_Entity_MPOLYGON_num_boundary_handles_set", _wrap_Dwg_Entity_MPOLYGON_num_boundary_handles_set},
{"LibreDWGc::Dwg_Entity_MPOLYGON_num_boundary_handles_get", _wrap_Dwg_Entity_MPOLYGON_num_boundary_handles_get},
{"LibreDWGc::new_Dwg_Entity_MPOLYGON", _wrap_new_Dwg_Entity_MPOLYGON},
{"LibreDWGc::delete_Dwg_Entity_MPOLYGON", _wrap_delete_Dwg_Entity_MPOLYGON},
{"LibreDWGc::Dwg_Object_XRECORD_parent_set", _wrap_Dwg_Object_XRECORD_parent_set},
{"LibreDWGc::Dwg_Object_XRECORD_parent_get", _wrap_Dwg_Object_XRECORD_parent_get},
{"LibreDWGc::Dwg_Object_XRECORD_cloning_set", _wrap_Dwg_Object_XRECORD_cloning_set},
{"LibreDWGc::Dwg_Object_XRECORD_cloning_get", _wrap_Dwg_Object_XRECORD_cloning_get},
{"LibreDWGc::Dwg_Object_XRECORD_xdata_size_set", _wrap_Dwg_Object_XRECORD_xdata_size_set},
{"LibreDWGc::Dwg_Object_XRECORD_xdata_size_get", _wrap_Dwg_Object_XRECORD_xdata_size_get},
{"LibreDWGc::Dwg_Object_XRECORD_num_xdata_set", _wrap_Dwg_Object_XRECORD_num_xdata_set},
{"LibreDWGc::Dwg_Object_XRECORD_num_xdata_get", _wrap_Dwg_Object_XRECORD_num_xdata_get},
{"LibreDWGc::Dwg_Object_XRECORD_xdata_set", _wrap_Dwg_Object_XRECORD_xdata_set},
{"LibreDWGc::Dwg_Object_XRECORD_xdata_get", _wrap_Dwg_Object_XRECORD_xdata_get},
{"LibreDWGc::Dwg_Object_XRECORD_num_objid_handles_set", _wrap_Dwg_Object_XRECORD_num_objid_handles_set},
{"LibreDWGc::Dwg_Object_XRECORD_num_objid_handles_get", _wrap_Dwg_Object_XRECORD_num_objid_handles_get},
{"LibreDWGc::Dwg_Object_XRECORD_objid_handles_set", _wrap_Dwg_Object_XRECORD_objid_handles_set},
{"LibreDWGc::Dwg_Object_XRECORD_objid_handles_get", _wrap_Dwg_Object_XRECORD_objid_handles_get},
{"LibreDWGc::new_Dwg_Object_XRECORD", _wrap_new_Dwg_Object_XRECORD},
{"LibreDWGc::delete_Dwg_Object_XRECORD", _wrap_delete_Dwg_Object_XRECORD},
{"LibreDWGc::Dwg_Object_PLACEHOLDER_parent_set", _wrap_Dwg_Object_PLACEHOLDER_parent_set},
{"LibreDWGc::Dwg_Object_PLACEHOLDER_parent_get", _wrap_Dwg_Object_PLACEHOLDER_parent_get},
{"LibreDWGc::new_Dwg_Object_PLACEHOLDER", _wrap_new_Dwg_Object_PLACEHOLDER},
{"LibreDWGc::delete_Dwg_Object_PLACEHOLDER", _wrap_delete_Dwg_Object_PLACEHOLDER},
{"LibreDWGc::Dwg_LEADER_Break_parent_set", _wrap_Dwg_LEADER_Break_parent_set},
{"LibreDWGc::Dwg_LEADER_Break_parent_get", _wrap_Dwg_LEADER_Break_parent_get},
{"LibreDWGc::Dwg_LEADER_Break_start_set", _wrap_Dwg_LEADER_Break_start_set},
{"LibreDWGc::Dwg_LEADER_Break_start_get", _wrap_Dwg_LEADER_Break_start_get},
{"LibreDWGc::Dwg_LEADER_Break_end_set", _wrap_Dwg_LEADER_Break_end_set},
{"LibreDWGc::Dwg_LEADER_Break_end_get", _wrap_Dwg_LEADER_Break_end_get},
{"LibreDWGc::new_Dwg_LEADER_Break", _wrap_new_Dwg_LEADER_Break},
{"LibreDWGc::delete_Dwg_LEADER_Break", _wrap_delete_Dwg_LEADER_Break},
{"LibreDWGc::Dwg_LEADER_Line_parent_set", _wrap_Dwg_LEADER_Line_parent_set},
{"LibreDWGc::Dwg_LEADER_Line_parent_get", _wrap_Dwg_LEADER_Line_parent_get},
{"LibreDWGc::Dwg_LEADER_Line_num_points_set", _wrap_Dwg_LEADER_Line_num_points_set},
{"LibreDWGc::Dwg_LEADER_Line_num_points_get", _wrap_Dwg_LEADER_Line_num_points_get},
{"LibreDWGc::Dwg_LEADER_Line_points_set", _wrap_Dwg_LEADER_Line_points_set},
{"LibreDWGc::Dwg_LEADER_Line_points_get", _wrap_Dwg_LEADER_Line_points_get},
{"LibreDWGc::Dwg_LEADER_Line_num_breaks_set", _wrap_Dwg_LEADER_Line_num_breaks_set},
{"LibreDWGc::Dwg_LEADER_Line_num_breaks_get", _wrap_Dwg_LEADER_Line_num_breaks_get},
{"LibreDWGc::Dwg_LEADER_Line_breaks_set", _wrap_Dwg_LEADER_Line_breaks_set},
{"LibreDWGc::Dwg_LEADER_Line_breaks_get", _wrap_Dwg_LEADER_Line_breaks_get},
{"LibreDWGc::Dwg_LEADER_Line_line_index_set", _wrap_Dwg_LEADER_Line_line_index_set},
{"LibreDWGc::Dwg_LEADER_Line_line_index_get", _wrap_Dwg_LEADER_Line_line_index_get},
{"LibreDWGc::Dwg_LEADER_Line_type_set", _wrap_Dwg_LEADER_Line_type_set},
{"LibreDWGc::Dwg_LEADER_Line_type_get", _wrap_Dwg_LEADER_Line_type_get},
{"LibreDWGc::Dwg_LEADER_Line_color_set", _wrap_Dwg_LEADER_Line_color_set},
{"LibreDWGc::Dwg_LEADER_Line_color_get", _wrap_Dwg_LEADER_Line_color_get},
{"LibreDWGc::Dwg_LEADER_Line_ltype_set", _wrap_Dwg_LEADER_Line_ltype_set},
{"LibreDWGc::Dwg_LEADER_Line_ltype_get", _wrap_Dwg_LEADER_Line_ltype_get},
{"LibreDWGc::Dwg_LEADER_Line_linewt_set", _wrap_Dwg_LEADER_Line_linewt_set},
{"LibreDWGc::Dwg_LEADER_Line_linewt_get", _wrap_Dwg_LEADER_Line_linewt_get},
{"LibreDWGc::Dwg_LEADER_Line_arrow_size_set", _wrap_Dwg_LEADER_Line_arrow_size_set},
{"LibreDWGc::Dwg_LEADER_Line_arrow_size_get", _wrap_Dwg_LEADER_Line_arrow_size_get},
{"LibreDWGc::Dwg_LEADER_Line_arrow_handle_set", _wrap_Dwg_LEADER_Line_arrow_handle_set},
{"LibreDWGc::Dwg_LEADER_Line_arrow_handle_get", _wrap_Dwg_LEADER_Line_arrow_handle_get},
{"LibreDWGc::Dwg_LEADER_Line_flags_set", _wrap_Dwg_LEADER_Line_flags_set},
{"LibreDWGc::Dwg_LEADER_Line_flags_get", _wrap_Dwg_LEADER_Line_flags_get},
{"LibreDWGc::new_Dwg_LEADER_Line", _wrap_new_Dwg_LEADER_Line},
{"LibreDWGc::delete_Dwg_LEADER_Line", _wrap_delete_Dwg_LEADER_Line},
{"LibreDWGc::Dwg_LEADER_ArrowHead_parent_set", _wrap_Dwg_LEADER_ArrowHead_parent_set},
{"LibreDWGc::Dwg_LEADER_ArrowHead_parent_get", _wrap_Dwg_LEADER_ArrowHead_parent_get},
{"LibreDWGc::Dwg_LEADER_ArrowHead_is_default_set", _wrap_Dwg_LEADER_ArrowHead_is_default_set},
{"LibreDWGc::Dwg_LEADER_ArrowHead_is_default_get", _wrap_Dwg_LEADER_ArrowHead_is_default_get},
{"LibreDWGc::Dwg_LEADER_ArrowHead_arrowhead_set", _wrap_Dwg_LEADER_ArrowHead_arrowhead_set},
{"LibreDWGc::Dwg_LEADER_ArrowHead_arrowhead_get", _wrap_Dwg_LEADER_ArrowHead_arrowhead_get},
{"LibreDWGc::new_Dwg_LEADER_ArrowHead", _wrap_new_Dwg_LEADER_ArrowHead},
{"LibreDWGc::delete_Dwg_LEADER_ArrowHead", _wrap_delete_Dwg_LEADER_ArrowHead},
{"LibreDWGc::Dwg_LEADER_BlockLabel_parent_set", _wrap_Dwg_LEADER_BlockLabel_parent_set},
{"LibreDWGc::Dwg_LEADER_BlockLabel_parent_get", _wrap_Dwg_LEADER_BlockLabel_parent_get},
{"LibreDWGc::Dwg_LEADER_BlockLabel_attdef_set", _wrap_Dwg_LEADER_BlockLabel_attdef_set},
{"LibreDWGc::Dwg_LEADER_BlockLabel_attdef_get", _wrap_Dwg_LEADER_BlockLabel_attdef_get},
{"LibreDWGc::Dwg_LEADER_BlockLabel_label_text_set", _wrap_Dwg_LEADER_BlockLabel_label_text_set},
{"LibreDWGc::Dwg_LEADER_BlockLabel_label_text_get", _wrap_Dwg_LEADER_BlockLabel_label_text_get},
{"LibreDWGc::Dwg_LEADER_BlockLabel_ui_index_set", _wrap_Dwg_LEADER_BlockLabel_ui_index_set},
{"LibreDWGc::Dwg_LEADER_BlockLabel_ui_index_get", _wrap_Dwg_LEADER_BlockLabel_ui_index_get},
{"LibreDWGc::Dwg_LEADER_BlockLabel_width_set", _wrap_Dwg_LEADER_BlockLabel_width_set},
{"LibreDWGc::Dwg_LEADER_BlockLabel_width_get", _wrap_Dwg_LEADER_BlockLabel_width_get},
{"LibreDWGc::new_Dwg_LEADER_BlockLabel", _wrap_new_Dwg_LEADER_BlockLabel},
{"LibreDWGc::delete_Dwg_LEADER_BlockLabel", _wrap_delete_Dwg_LEADER_BlockLabel},
{"LibreDWGc::Dwg_LEADER_Node_parent_set", _wrap_Dwg_LEADER_Node_parent_set},
{"LibreDWGc::Dwg_LEADER_Node_parent_get", _wrap_Dwg_LEADER_Node_parent_get},
{"LibreDWGc::Dwg_LEADER_Node_has_lastleaderlinepoint_set", _wrap_Dwg_LEADER_Node_has_lastleaderlinepoint_set},
{"LibreDWGc::Dwg_LEADER_Node_has_lastleaderlinepoint_get", _wrap_Dwg_LEADER_Node_has_lastleaderlinepoint_get},
{"LibreDWGc::Dwg_LEADER_Node_has_dogleg_set", _wrap_Dwg_LEADER_Node_has_dogleg_set},
{"LibreDWGc::Dwg_LEADER_Node_has_dogleg_get", _wrap_Dwg_LEADER_Node_has_dogleg_get},
{"LibreDWGc::Dwg_LEADER_Node_lastleaderlinepoint_set", _wrap_Dwg_LEADER_Node_lastleaderlinepoint_set},
{"LibreDWGc::Dwg_LEADER_Node_lastleaderlinepoint_get", _wrap_Dwg_LEADER_Node_lastleaderlinepoint_get},
{"LibreDWGc::Dwg_LEADER_Node_dogleg_vector_set", _wrap_Dwg_LEADER_Node_dogleg_vector_set},
{"LibreDWGc::Dwg_LEADER_Node_dogleg_vector_get", _wrap_Dwg_LEADER_Node_dogleg_vector_get},
{"LibreDWGc::Dwg_LEADER_Node_branch_index_set", _wrap_Dwg_LEADER_Node_branch_index_set},
{"LibreDWGc::Dwg_LEADER_Node_branch_index_get", _wrap_Dwg_LEADER_Node_branch_index_get},
{"LibreDWGc::Dwg_LEADER_Node_dogleg_length_set", _wrap_Dwg_LEADER_Node_dogleg_length_set},
{"LibreDWGc::Dwg_LEADER_Node_dogleg_length_get", _wrap_Dwg_LEADER_Node_dogleg_length_get},
{"LibreDWGc::Dwg_LEADER_Node_num_lines_set", _wrap_Dwg_LEADER_Node_num_lines_set},
{"LibreDWGc::Dwg_LEADER_Node_num_lines_get", _wrap_Dwg_LEADER_Node_num_lines_get},
{"LibreDWGc::Dwg_LEADER_Node_lines_set", _wrap_Dwg_LEADER_Node_lines_set},
{"LibreDWGc::Dwg_LEADER_Node_lines_get", _wrap_Dwg_LEADER_Node_lines_get},
{"LibreDWGc::Dwg_LEADER_Node_num_breaks_set", _wrap_Dwg_LEADER_Node_num_breaks_set},
{"LibreDWGc::Dwg_LEADER_Node_num_breaks_get", _wrap_Dwg_LEADER_Node_num_breaks_get},
{"LibreDWGc::Dwg_LEADER_Node_breaks_set", _wrap_Dwg_LEADER_Node_breaks_set},
{"LibreDWGc::Dwg_LEADER_Node_breaks_get", _wrap_Dwg_LEADER_Node_breaks_get},
{"LibreDWGc::Dwg_LEADER_Node_attach_dir_set", _wrap_Dwg_LEADER_Node_attach_dir_set},
{"LibreDWGc::Dwg_LEADER_Node_attach_dir_get", _wrap_Dwg_LEADER_Node_attach_dir_get},
{"LibreDWGc::new_Dwg_LEADER_Node", _wrap_new_Dwg_LEADER_Node},
{"LibreDWGc::delete_Dwg_LEADER_Node", _wrap_delete_Dwg_LEADER_Node},
{"LibreDWGc::Dwg_MLEADER_Content_MText_type_set", _wrap_Dwg_MLEADER_Content_MText_type_set},
{"LibreDWGc::Dwg_MLEADER_Content_MText_type_get", _wrap_Dwg_MLEADER_Content_MText_type_get},
{"LibreDWGc::Dwg_MLEADER_Content_MText_normal_set", _wrap_Dwg_MLEADER_Content_MText_normal_set},
{"LibreDWGc::Dwg_MLEADER_Content_MText_normal_get", _wrap_Dwg_MLEADER_Content_MText_normal_get},
{"LibreDWGc::Dwg_MLEADER_Content_MText_location_set", _wrap_Dwg_MLEADER_Content_MText_location_set},
{"LibreDWGc::Dwg_MLEADER_Content_MText_location_get", _wrap_Dwg_MLEADER_Content_MText_location_get},
{"LibreDWGc::Dwg_MLEADER_Content_MText_rotation_set", _wrap_Dwg_MLEADER_Content_MText_rotation_set},
{"LibreDWGc::Dwg_MLEADER_Content_MText_rotation_get", _wrap_Dwg_MLEADER_Content_MText_rotation_get},
{"LibreDWGc::Dwg_MLEADER_Content_MText_style_set", _wrap_Dwg_MLEADER_Content_MText_style_set},
{"LibreDWGc::Dwg_MLEADER_Content_MText_style_get", _wrap_Dwg_MLEADER_Content_MText_style_get},
{"LibreDWGc::Dwg_MLEADER_Content_MText_direction_set", _wrap_Dwg_MLEADER_Content_MText_direction_set},
{"LibreDWGc::Dwg_MLEADER_Content_MText_direction_get", _wrap_Dwg_MLEADER_Content_MText_direction_get},
{"LibreDWGc::Dwg_MLEADER_Content_MText_color_set", _wrap_Dwg_MLEADER_Content_MText_color_set},
{"LibreDWGc::Dwg_MLEADER_Content_MText_color_get", _wrap_Dwg_MLEADER_Content_MText_color_get},
{"LibreDWGc::Dwg_MLEADER_Content_MText_width_set", _wrap_Dwg_MLEADER_Content_MText_width_set},
{"LibreDWGc::Dwg_MLEADER_Content_MText_width_get", _wrap_Dwg_MLEADER_Content_MText_width_get},
{"LibreDWGc::Dwg_MLEADER_Content_MText_height_set", _wrap_Dwg_MLEADER_Content_MText_height_set},
{"LibreDWGc::Dwg_MLEADER_Content_MText_height_get", _wrap_Dwg_MLEADER_Content_MText_height_get},
{"LibreDWGc::Dwg_MLEADER_Content_MText_line_spacing_factor_set", _wrap_Dwg_MLEADER_Content_MText_line_spacing_factor_set},
{"LibreDWGc::Dwg_MLEADER_Content_MText_line_spacing_factor_get", _wrap_Dwg_MLEADER_Content_MText_line_spacing_factor_get},
{"LibreDWGc::Dwg_MLEADER_Content_MText_default_text_set", _wrap_Dwg_MLEADER_Content_MText_default_text_set},
{"LibreDWGc::Dwg_MLEADER_Content_MText_default_text_get", _wrap_Dwg_MLEADER_Content_MText_default_text_get},
{"LibreDWGc::Dwg_MLEADER_Content_MText_line_spacing_style_set", _wrap_Dwg_MLEADER_Content_MText_line_spacing_style_set},
{"LibreDWGc::Dwg_MLEADER_Content_MText_line_spacing_style_get", _wrap_Dwg_MLEADER_Content_MText_line_spacing_style_get},
{"LibreDWGc::Dwg_MLEADER_Content_MText_alignment_set", _wrap_Dwg_MLEADER_Content_MText_alignment_set},
{"LibreDWGc::Dwg_MLEADER_Content_MText_alignment_get", _wrap_Dwg_MLEADER_Content_MText_alignment_get},
{"LibreDWGc::Dwg_MLEADER_Content_MText_flow_set", _wrap_Dwg_MLEADER_Content_MText_flow_set},
{"LibreDWGc::Dwg_MLEADER_Content_MText_flow_get", _wrap_Dwg_MLEADER_Content_MText_flow_get},
{"LibreDWGc::Dwg_MLEADER_Content_MText_bg_color_set", _wrap_Dwg_MLEADER_Content_MText_bg_color_set},
{"LibreDWGc::Dwg_MLEADER_Content_MText_bg_color_get", _wrap_Dwg_MLEADER_Content_MText_bg_color_get},
{"LibreDWGc::Dwg_MLEADER_Content_MText_bg_scale_set", _wrap_Dwg_MLEADER_Content_MText_bg_scale_set},
{"LibreDWGc::Dwg_MLEADER_Content_MText_bg_scale_get", _wrap_Dwg_MLEADER_Content_MText_bg_scale_get},
{"LibreDWGc::Dwg_MLEADER_Content_MText_bg_transparency_set", _wrap_Dwg_MLEADER_Content_MText_bg_transparency_set},
{"LibreDWGc::Dwg_MLEADER_Content_MText_bg_transparency_get", _wrap_Dwg_MLEADER_Content_MText_bg_transparency_get},
{"LibreDWGc::Dwg_MLEADER_Content_MText_is_bg_fill_set", _wrap_Dwg_MLEADER_Content_MText_is_bg_fill_set},
{"LibreDWGc::Dwg_MLEADER_Content_MText_is_bg_fill_get", _wrap_Dwg_MLEADER_Content_MText_is_bg_fill_get},
{"LibreDWGc::Dwg_MLEADER_Content_MText_is_bg_mask_fill_set", _wrap_Dwg_MLEADER_Content_MText_is_bg_mask_fill_set},
{"LibreDWGc::Dwg_MLEADER_Content_MText_is_bg_mask_fill_get", _wrap_Dwg_MLEADER_Content_MText_is_bg_mask_fill_get},
{"LibreDWGc::Dwg_MLEADER_Content_MText_col_type_set", _wrap_Dwg_MLEADER_Content_MText_col_type_set},
{"LibreDWGc::Dwg_MLEADER_Content_MText_col_type_get", _wrap_Dwg_MLEADER_Content_MText_col_type_get},
{"LibreDWGc::Dwg_MLEADER_Content_MText_is_height_auto_set", _wrap_Dwg_MLEADER_Content_MText_is_height_auto_set},
{"LibreDWGc::Dwg_MLEADER_Content_MText_is_height_auto_get", _wrap_Dwg_MLEADER_Content_MText_is_height_auto_get},
{"LibreDWGc::Dwg_MLEADER_Content_MText_col_width_set", _wrap_Dwg_MLEADER_Content_MText_col_width_set},
{"LibreDWGc::Dwg_MLEADER_Content_MText_col_width_get", _wrap_Dwg_MLEADER_Content_MText_col_width_get},
{"LibreDWGc::Dwg_MLEADER_Content_MText_col_gutter_set", _wrap_Dwg_MLEADER_Content_MText_col_gutter_set},
{"LibreDWGc::Dwg_MLEADER_Content_MText_col_gutter_get", _wrap_Dwg_MLEADER_Content_MText_col_gutter_get},
{"LibreDWGc::Dwg_MLEADER_Content_MText_is_col_flow_reversed_set", _wrap_Dwg_MLEADER_Content_MText_is_col_flow_reversed_set},
{"LibreDWGc::Dwg_MLEADER_Content_MText_is_col_flow_reversed_get", _wrap_Dwg_MLEADER_Content_MText_is_col_flow_reversed_get},
{"LibreDWGc::Dwg_MLEADER_Content_MText_num_col_sizes_set", _wrap_Dwg_MLEADER_Content_MText_num_col_sizes_set},
{"LibreDWGc::Dwg_MLEADER_Content_MText_num_col_sizes_get", _wrap_Dwg_MLEADER_Content_MText_num_col_sizes_get},
{"LibreDWGc::Dwg_MLEADER_Content_MText_col_sizes_set", _wrap_Dwg_MLEADER_Content_MText_col_sizes_set},
{"LibreDWGc::Dwg_MLEADER_Content_MText_col_sizes_get", _wrap_Dwg_MLEADER_Content_MText_col_sizes_get},
{"LibreDWGc::Dwg_MLEADER_Content_MText_word_break_set", _wrap_Dwg_MLEADER_Content_MText_word_break_set},
{"LibreDWGc::Dwg_MLEADER_Content_MText_word_break_get", _wrap_Dwg_MLEADER_Content_MText_word_break_get},
{"LibreDWGc::Dwg_MLEADER_Content_MText_unknown_set", _wrap_Dwg_MLEADER_Content_MText_unknown_set},
{"LibreDWGc::Dwg_MLEADER_Content_MText_unknown_get", _wrap_Dwg_MLEADER_Content_MText_unknown_get},
{"LibreDWGc::new_Dwg_MLEADER_Content_MText", _wrap_new_Dwg_MLEADER_Content_MText},
{"LibreDWGc::delete_Dwg_MLEADER_Content_MText", _wrap_delete_Dwg_MLEADER_Content_MText},
{"LibreDWGc::Dwg_MLEADER_Content_Block_type_set", _wrap_Dwg_MLEADER_Content_Block_type_set},
{"LibreDWGc::Dwg_MLEADER_Content_Block_type_get", _wrap_Dwg_MLEADER_Content_Block_type_get},
{"LibreDWGc::Dwg_MLEADER_Content_Block_normal_set", _wrap_Dwg_MLEADER_Content_Block_normal_set},
{"LibreDWGc::Dwg_MLEADER_Content_Block_normal_get", _wrap_Dwg_MLEADER_Content_Block_normal_get},
{"LibreDWGc::Dwg_MLEADER_Content_Block_location_set", _wrap_Dwg_MLEADER_Content_Block_location_set},
{"LibreDWGc::Dwg_MLEADER_Content_Block_location_get", _wrap_Dwg_MLEADER_Content_Block_location_get},
{"LibreDWGc::Dwg_MLEADER_Content_Block_rotation_set", _wrap_Dwg_MLEADER_Content_Block_rotation_set},
{"LibreDWGc::Dwg_MLEADER_Content_Block_rotation_get", _wrap_Dwg_MLEADER_Content_Block_rotation_get},
{"LibreDWGc::Dwg_MLEADER_Content_Block_block_table_set", _wrap_Dwg_MLEADER_Content_Block_block_table_set},
{"LibreDWGc::Dwg_MLEADER_Content_Block_block_table_get", _wrap_Dwg_MLEADER_Content_Block_block_table_get},
{"LibreDWGc::Dwg_MLEADER_Content_Block_scale_set", _wrap_Dwg_MLEADER_Content_Block_scale_set},
{"LibreDWGc::Dwg_MLEADER_Content_Block_scale_get", _wrap_Dwg_MLEADER_Content_Block_scale_get},
{"LibreDWGc::Dwg_MLEADER_Content_Block_color_set", _wrap_Dwg_MLEADER_Content_Block_color_set},
{"LibreDWGc::Dwg_MLEADER_Content_Block_color_get", _wrap_Dwg_MLEADER_Content_Block_color_get},
{"LibreDWGc::Dwg_MLEADER_Content_Block_transform_set", _wrap_Dwg_MLEADER_Content_Block_transform_set},
{"LibreDWGc::Dwg_MLEADER_Content_Block_transform_get", _wrap_Dwg_MLEADER_Content_Block_transform_get},
{"LibreDWGc::new_Dwg_MLEADER_Content_Block", _wrap_new_Dwg_MLEADER_Content_Block},
{"LibreDWGc::delete_Dwg_MLEADER_Content_Block", _wrap_delete_Dwg_MLEADER_Content_Block},
{"LibreDWGc::Dwg_MLEADER_Content_txt_set", _wrap_Dwg_MLEADER_Content_txt_set},
{"LibreDWGc::Dwg_MLEADER_Content_txt_get", _wrap_Dwg_MLEADER_Content_txt_get},
{"LibreDWGc::Dwg_MLEADER_Content_blk_set", _wrap_Dwg_MLEADER_Content_blk_set},
{"LibreDWGc::Dwg_MLEADER_Content_blk_get", _wrap_Dwg_MLEADER_Content_blk_get},
{"LibreDWGc::new_Dwg_MLEADER_Content", _wrap_new_Dwg_MLEADER_Content},
{"LibreDWGc::delete_Dwg_MLEADER_Content", _wrap_delete_Dwg_MLEADER_Content},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_num_leaders_set", _wrap_Dwg_MLEADER_AnnotContext_num_leaders_set},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_num_leaders_get", _wrap_Dwg_MLEADER_AnnotContext_num_leaders_get},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_leaders_set", _wrap_Dwg_MLEADER_AnnotContext_leaders_set},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_leaders_get", _wrap_Dwg_MLEADER_AnnotContext_leaders_get},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_attach_dir_set", _wrap_Dwg_MLEADER_AnnotContext_attach_dir_set},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_attach_dir_get", _wrap_Dwg_MLEADER_AnnotContext_attach_dir_get},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_scale_factor_set", _wrap_Dwg_MLEADER_AnnotContext_scale_factor_set},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_scale_factor_get", _wrap_Dwg_MLEADER_AnnotContext_scale_factor_get},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_content_base_set", _wrap_Dwg_MLEADER_AnnotContext_content_base_set},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_content_base_get", _wrap_Dwg_MLEADER_AnnotContext_content_base_get},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_text_height_set", _wrap_Dwg_MLEADER_AnnotContext_text_height_set},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_text_height_get", _wrap_Dwg_MLEADER_AnnotContext_text_height_get},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_arrow_size_set", _wrap_Dwg_MLEADER_AnnotContext_arrow_size_set},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_arrow_size_get", _wrap_Dwg_MLEADER_AnnotContext_arrow_size_get},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_landing_gap_set", _wrap_Dwg_MLEADER_AnnotContext_landing_gap_set},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_landing_gap_get", _wrap_Dwg_MLEADER_AnnotContext_landing_gap_get},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_text_left_set", _wrap_Dwg_MLEADER_AnnotContext_text_left_set},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_text_left_get", _wrap_Dwg_MLEADER_AnnotContext_text_left_get},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_text_right_set", _wrap_Dwg_MLEADER_AnnotContext_text_right_set},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_text_right_get", _wrap_Dwg_MLEADER_AnnotContext_text_right_get},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_text_angletype_set", _wrap_Dwg_MLEADER_AnnotContext_text_angletype_set},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_text_angletype_get", _wrap_Dwg_MLEADER_AnnotContext_text_angletype_get},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_text_alignment_set", _wrap_Dwg_MLEADER_AnnotContext_text_alignment_set},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_text_alignment_get", _wrap_Dwg_MLEADER_AnnotContext_text_alignment_get},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_has_content_txt_set", _wrap_Dwg_MLEADER_AnnotContext_has_content_txt_set},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_has_content_txt_get", _wrap_Dwg_MLEADER_AnnotContext_has_content_txt_get},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_has_content_blk_set", _wrap_Dwg_MLEADER_AnnotContext_has_content_blk_set},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_has_content_blk_get", _wrap_Dwg_MLEADER_AnnotContext_has_content_blk_get},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_content_set", _wrap_Dwg_MLEADER_AnnotContext_content_set},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_content_get", _wrap_Dwg_MLEADER_AnnotContext_content_get},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_base_set", _wrap_Dwg_MLEADER_AnnotContext_base_set},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_base_get", _wrap_Dwg_MLEADER_AnnotContext_base_get},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_base_dir_set", _wrap_Dwg_MLEADER_AnnotContext_base_dir_set},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_base_dir_get", _wrap_Dwg_MLEADER_AnnotContext_base_dir_get},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_base_vert_set", _wrap_Dwg_MLEADER_AnnotContext_base_vert_set},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_base_vert_get", _wrap_Dwg_MLEADER_AnnotContext_base_vert_get},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_is_normal_reversed_set", _wrap_Dwg_MLEADER_AnnotContext_is_normal_reversed_set},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_is_normal_reversed_get", _wrap_Dwg_MLEADER_AnnotContext_is_normal_reversed_get},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_text_top_set", _wrap_Dwg_MLEADER_AnnotContext_text_top_set},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_text_top_get", _wrap_Dwg_MLEADER_AnnotContext_text_top_get},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_text_bottom_set", _wrap_Dwg_MLEADER_AnnotContext_text_bottom_set},
{"LibreDWGc::Dwg_MLEADER_AnnotContext_text_bottom_get", _wrap_Dwg_MLEADER_AnnotContext_text_bottom_get},
{"LibreDWGc::new_Dwg_MLEADER_AnnotContext", _wrap_new_Dwg_MLEADER_AnnotContext},
{"LibreDWGc::delete_Dwg_MLEADER_AnnotContext", _wrap_delete_Dwg_MLEADER_AnnotContext},
{"LibreDWGc::Dwg_Entity_MULTILEADER_parent_set", _wrap_Dwg_Entity_MULTILEADER_parent_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_parent_get", _wrap_Dwg_Entity_MULTILEADER_parent_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_class_version_set", _wrap_Dwg_Entity_MULTILEADER_class_version_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_class_version_get", _wrap_Dwg_Entity_MULTILEADER_class_version_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_ctx_set", _wrap_Dwg_Entity_MULTILEADER_ctx_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_ctx_get", _wrap_Dwg_Entity_MULTILEADER_ctx_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_mleaderstyle_set", _wrap_Dwg_Entity_MULTILEADER_mleaderstyle_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_mleaderstyle_get", _wrap_Dwg_Entity_MULTILEADER_mleaderstyle_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_flags_set", _wrap_Dwg_Entity_MULTILEADER_flags_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_flags_get", _wrap_Dwg_Entity_MULTILEADER_flags_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_type_set", _wrap_Dwg_Entity_MULTILEADER_type_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_type_get", _wrap_Dwg_Entity_MULTILEADER_type_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_line_color_set", _wrap_Dwg_Entity_MULTILEADER_line_color_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_line_color_get", _wrap_Dwg_Entity_MULTILEADER_line_color_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_line_ltype_set", _wrap_Dwg_Entity_MULTILEADER_line_ltype_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_line_ltype_get", _wrap_Dwg_Entity_MULTILEADER_line_ltype_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_line_linewt_set", _wrap_Dwg_Entity_MULTILEADER_line_linewt_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_line_linewt_get", _wrap_Dwg_Entity_MULTILEADER_line_linewt_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_has_landing_set", _wrap_Dwg_Entity_MULTILEADER_has_landing_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_has_landing_get", _wrap_Dwg_Entity_MULTILEADER_has_landing_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_has_dogleg_set", _wrap_Dwg_Entity_MULTILEADER_has_dogleg_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_has_dogleg_get", _wrap_Dwg_Entity_MULTILEADER_has_dogleg_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_landing_dist_set", _wrap_Dwg_Entity_MULTILEADER_landing_dist_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_landing_dist_get", _wrap_Dwg_Entity_MULTILEADER_landing_dist_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_arrow_handle_set", _wrap_Dwg_Entity_MULTILEADER_arrow_handle_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_arrow_handle_get", _wrap_Dwg_Entity_MULTILEADER_arrow_handle_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_arrow_size_set", _wrap_Dwg_Entity_MULTILEADER_arrow_size_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_arrow_size_get", _wrap_Dwg_Entity_MULTILEADER_arrow_size_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_style_content_set", _wrap_Dwg_Entity_MULTILEADER_style_content_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_style_content_get", _wrap_Dwg_Entity_MULTILEADER_style_content_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_text_style_set", _wrap_Dwg_Entity_MULTILEADER_text_style_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_text_style_get", _wrap_Dwg_Entity_MULTILEADER_text_style_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_text_left_set", _wrap_Dwg_Entity_MULTILEADER_text_left_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_text_left_get", _wrap_Dwg_Entity_MULTILEADER_text_left_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_text_right_set", _wrap_Dwg_Entity_MULTILEADER_text_right_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_text_right_get", _wrap_Dwg_Entity_MULTILEADER_text_right_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_text_angletype_set", _wrap_Dwg_Entity_MULTILEADER_text_angletype_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_text_angletype_get", _wrap_Dwg_Entity_MULTILEADER_text_angletype_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_text_alignment_set", _wrap_Dwg_Entity_MULTILEADER_text_alignment_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_text_alignment_get", _wrap_Dwg_Entity_MULTILEADER_text_alignment_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_text_color_set", _wrap_Dwg_Entity_MULTILEADER_text_color_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_text_color_get", _wrap_Dwg_Entity_MULTILEADER_text_color_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_has_text_frame_set", _wrap_Dwg_Entity_MULTILEADER_has_text_frame_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_has_text_frame_get", _wrap_Dwg_Entity_MULTILEADER_has_text_frame_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_block_style_set", _wrap_Dwg_Entity_MULTILEADER_block_style_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_block_style_get", _wrap_Dwg_Entity_MULTILEADER_block_style_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_block_color_set", _wrap_Dwg_Entity_MULTILEADER_block_color_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_block_color_get", _wrap_Dwg_Entity_MULTILEADER_block_color_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_block_scale_set", _wrap_Dwg_Entity_MULTILEADER_block_scale_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_block_scale_get", _wrap_Dwg_Entity_MULTILEADER_block_scale_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_block_rotation_set", _wrap_Dwg_Entity_MULTILEADER_block_rotation_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_block_rotation_get", _wrap_Dwg_Entity_MULTILEADER_block_rotation_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_style_attachment_set", _wrap_Dwg_Entity_MULTILEADER_style_attachment_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_style_attachment_get", _wrap_Dwg_Entity_MULTILEADER_style_attachment_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_is_annotative_set", _wrap_Dwg_Entity_MULTILEADER_is_annotative_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_is_annotative_get", _wrap_Dwg_Entity_MULTILEADER_is_annotative_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_num_arrowheads_set", _wrap_Dwg_Entity_MULTILEADER_num_arrowheads_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_num_arrowheads_get", _wrap_Dwg_Entity_MULTILEADER_num_arrowheads_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_arrowheads_set", _wrap_Dwg_Entity_MULTILEADER_arrowheads_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_arrowheads_get", _wrap_Dwg_Entity_MULTILEADER_arrowheads_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_num_blocklabels_set", _wrap_Dwg_Entity_MULTILEADER_num_blocklabels_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_num_blocklabels_get", _wrap_Dwg_Entity_MULTILEADER_num_blocklabels_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_blocklabels_set", _wrap_Dwg_Entity_MULTILEADER_blocklabels_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_blocklabels_get", _wrap_Dwg_Entity_MULTILEADER_blocklabels_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_is_neg_textdir_set", _wrap_Dwg_Entity_MULTILEADER_is_neg_textdir_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_is_neg_textdir_get", _wrap_Dwg_Entity_MULTILEADER_is_neg_textdir_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_ipe_alignment_set", _wrap_Dwg_Entity_MULTILEADER_ipe_alignment_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_ipe_alignment_get", _wrap_Dwg_Entity_MULTILEADER_ipe_alignment_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_justification_set", _wrap_Dwg_Entity_MULTILEADER_justification_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_justification_get", _wrap_Dwg_Entity_MULTILEADER_justification_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_scale_factor_set", _wrap_Dwg_Entity_MULTILEADER_scale_factor_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_scale_factor_get", _wrap_Dwg_Entity_MULTILEADER_scale_factor_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_attach_dir_set", _wrap_Dwg_Entity_MULTILEADER_attach_dir_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_attach_dir_get", _wrap_Dwg_Entity_MULTILEADER_attach_dir_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_attach_top_set", _wrap_Dwg_Entity_MULTILEADER_attach_top_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_attach_top_get", _wrap_Dwg_Entity_MULTILEADER_attach_top_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_attach_bottom_set", _wrap_Dwg_Entity_MULTILEADER_attach_bottom_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_attach_bottom_get", _wrap_Dwg_Entity_MULTILEADER_attach_bottom_get},
{"LibreDWGc::Dwg_Entity_MULTILEADER_is_text_extended_set", _wrap_Dwg_Entity_MULTILEADER_is_text_extended_set},
{"LibreDWGc::Dwg_Entity_MULTILEADER_is_text_extended_get", _wrap_Dwg_Entity_MULTILEADER_is_text_extended_get},
{"LibreDWGc::new_Dwg_Entity_MULTILEADER", _wrap_new_Dwg_Entity_MULTILEADER},
{"LibreDWGc::delete_Dwg_Entity_MULTILEADER", _wrap_delete_Dwg_Entity_MULTILEADER},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_parent_set", _wrap_Dwg_Object_MLEADERSTYLE_parent_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_parent_get", _wrap_Dwg_Object_MLEADERSTYLE_parent_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_class_version_set", _wrap_Dwg_Object_MLEADERSTYLE_class_version_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_class_version_get", _wrap_Dwg_Object_MLEADERSTYLE_class_version_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_content_type_set", _wrap_Dwg_Object_MLEADERSTYLE_content_type_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_content_type_get", _wrap_Dwg_Object_MLEADERSTYLE_content_type_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_mleader_order_set", _wrap_Dwg_Object_MLEADERSTYLE_mleader_order_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_mleader_order_get", _wrap_Dwg_Object_MLEADERSTYLE_mleader_order_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_leader_order_set", _wrap_Dwg_Object_MLEADERSTYLE_leader_order_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_leader_order_get", _wrap_Dwg_Object_MLEADERSTYLE_leader_order_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_max_points_set", _wrap_Dwg_Object_MLEADERSTYLE_max_points_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_max_points_get", _wrap_Dwg_Object_MLEADERSTYLE_max_points_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_first_seg_angle_set", _wrap_Dwg_Object_MLEADERSTYLE_first_seg_angle_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_first_seg_angle_get", _wrap_Dwg_Object_MLEADERSTYLE_first_seg_angle_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_second_seg_angle_set", _wrap_Dwg_Object_MLEADERSTYLE_second_seg_angle_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_second_seg_angle_get", _wrap_Dwg_Object_MLEADERSTYLE_second_seg_angle_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_type_set", _wrap_Dwg_Object_MLEADERSTYLE_type_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_type_get", _wrap_Dwg_Object_MLEADERSTYLE_type_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_line_color_set", _wrap_Dwg_Object_MLEADERSTYLE_line_color_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_line_color_get", _wrap_Dwg_Object_MLEADERSTYLE_line_color_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_line_type_set", _wrap_Dwg_Object_MLEADERSTYLE_line_type_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_line_type_get", _wrap_Dwg_Object_MLEADERSTYLE_line_type_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_linewt_set", _wrap_Dwg_Object_MLEADERSTYLE_linewt_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_linewt_get", _wrap_Dwg_Object_MLEADERSTYLE_linewt_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_has_landing_set", _wrap_Dwg_Object_MLEADERSTYLE_has_landing_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_has_landing_get", _wrap_Dwg_Object_MLEADERSTYLE_has_landing_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_has_dogleg_set", _wrap_Dwg_Object_MLEADERSTYLE_has_dogleg_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_has_dogleg_get", _wrap_Dwg_Object_MLEADERSTYLE_has_dogleg_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_landing_gap_set", _wrap_Dwg_Object_MLEADERSTYLE_landing_gap_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_landing_gap_get", _wrap_Dwg_Object_MLEADERSTYLE_landing_gap_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_landing_dist_set", _wrap_Dwg_Object_MLEADERSTYLE_landing_dist_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_landing_dist_get", _wrap_Dwg_Object_MLEADERSTYLE_landing_dist_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_description_set", _wrap_Dwg_Object_MLEADERSTYLE_description_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_description_get", _wrap_Dwg_Object_MLEADERSTYLE_description_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_arrow_head_set", _wrap_Dwg_Object_MLEADERSTYLE_arrow_head_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_arrow_head_get", _wrap_Dwg_Object_MLEADERSTYLE_arrow_head_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_arrow_head_size_set", _wrap_Dwg_Object_MLEADERSTYLE_arrow_head_size_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_arrow_head_size_get", _wrap_Dwg_Object_MLEADERSTYLE_arrow_head_size_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_text_default_set", _wrap_Dwg_Object_MLEADERSTYLE_text_default_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_text_default_get", _wrap_Dwg_Object_MLEADERSTYLE_text_default_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_text_style_set", _wrap_Dwg_Object_MLEADERSTYLE_text_style_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_text_style_get", _wrap_Dwg_Object_MLEADERSTYLE_text_style_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_attach_left_set", _wrap_Dwg_Object_MLEADERSTYLE_attach_left_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_attach_left_get", _wrap_Dwg_Object_MLEADERSTYLE_attach_left_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_attach_right_set", _wrap_Dwg_Object_MLEADERSTYLE_attach_right_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_attach_right_get", _wrap_Dwg_Object_MLEADERSTYLE_attach_right_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_text_angle_type_set", _wrap_Dwg_Object_MLEADERSTYLE_text_angle_type_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_text_angle_type_get", _wrap_Dwg_Object_MLEADERSTYLE_text_angle_type_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_text_align_type_set", _wrap_Dwg_Object_MLEADERSTYLE_text_align_type_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_text_align_type_get", _wrap_Dwg_Object_MLEADERSTYLE_text_align_type_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_text_color_set", _wrap_Dwg_Object_MLEADERSTYLE_text_color_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_text_color_get", _wrap_Dwg_Object_MLEADERSTYLE_text_color_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_text_height_set", _wrap_Dwg_Object_MLEADERSTYLE_text_height_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_text_height_get", _wrap_Dwg_Object_MLEADERSTYLE_text_height_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_has_text_frame_set", _wrap_Dwg_Object_MLEADERSTYLE_has_text_frame_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_has_text_frame_get", _wrap_Dwg_Object_MLEADERSTYLE_has_text_frame_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_text_always_left_set", _wrap_Dwg_Object_MLEADERSTYLE_text_always_left_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_text_always_left_get", _wrap_Dwg_Object_MLEADERSTYLE_text_always_left_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_align_space_set", _wrap_Dwg_Object_MLEADERSTYLE_align_space_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_align_space_get", _wrap_Dwg_Object_MLEADERSTYLE_align_space_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_block_set", _wrap_Dwg_Object_MLEADERSTYLE_block_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_block_get", _wrap_Dwg_Object_MLEADERSTYLE_block_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_block_color_set", _wrap_Dwg_Object_MLEADERSTYLE_block_color_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_block_color_get", _wrap_Dwg_Object_MLEADERSTYLE_block_color_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_block_scale_set", _wrap_Dwg_Object_MLEADERSTYLE_block_scale_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_block_scale_get", _wrap_Dwg_Object_MLEADERSTYLE_block_scale_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_use_block_scale_set", _wrap_Dwg_Object_MLEADERSTYLE_use_block_scale_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_use_block_scale_get", _wrap_Dwg_Object_MLEADERSTYLE_use_block_scale_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_block_rotation_set", _wrap_Dwg_Object_MLEADERSTYLE_block_rotation_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_block_rotation_get", _wrap_Dwg_Object_MLEADERSTYLE_block_rotation_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_use_block_rotation_set", _wrap_Dwg_Object_MLEADERSTYLE_use_block_rotation_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_use_block_rotation_get", _wrap_Dwg_Object_MLEADERSTYLE_use_block_rotation_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_block_connection_set", _wrap_Dwg_Object_MLEADERSTYLE_block_connection_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_block_connection_get", _wrap_Dwg_Object_MLEADERSTYLE_block_connection_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_scale_set", _wrap_Dwg_Object_MLEADERSTYLE_scale_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_scale_get", _wrap_Dwg_Object_MLEADERSTYLE_scale_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_is_changed_set", _wrap_Dwg_Object_MLEADERSTYLE_is_changed_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_is_changed_get", _wrap_Dwg_Object_MLEADERSTYLE_is_changed_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_is_annotative_set", _wrap_Dwg_Object_MLEADERSTYLE_is_annotative_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_is_annotative_get", _wrap_Dwg_Object_MLEADERSTYLE_is_annotative_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_break_size_set", _wrap_Dwg_Object_MLEADERSTYLE_break_size_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_break_size_get", _wrap_Dwg_Object_MLEADERSTYLE_break_size_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_attach_dir_set", _wrap_Dwg_Object_MLEADERSTYLE_attach_dir_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_attach_dir_get", _wrap_Dwg_Object_MLEADERSTYLE_attach_dir_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_attach_top_set", _wrap_Dwg_Object_MLEADERSTYLE_attach_top_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_attach_top_get", _wrap_Dwg_Object_MLEADERSTYLE_attach_top_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_attach_bottom_set", _wrap_Dwg_Object_MLEADERSTYLE_attach_bottom_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_attach_bottom_get", _wrap_Dwg_Object_MLEADERSTYLE_attach_bottom_get},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_text_extended_set", _wrap_Dwg_Object_MLEADERSTYLE_text_extended_set},
{"LibreDWGc::Dwg_Object_MLEADERSTYLE_text_extended_get", _wrap_Dwg_Object_MLEADERSTYLE_text_extended_get},
{"LibreDWGc::new_Dwg_Object_MLEADERSTYLE", _wrap_new_Dwg_Object_MLEADERSTYLE},
{"LibreDWGc::delete_Dwg_Object_MLEADERSTYLE", _wrap_delete_Dwg_Object_MLEADERSTYLE},
{"LibreDWGc::Dwg_Object_VBA_PROJECT_parent_set", _wrap_Dwg_Object_VBA_PROJECT_parent_set},
{"LibreDWGc::Dwg_Object_VBA_PROJECT_parent_get", _wrap_Dwg_Object_VBA_PROJECT_parent_get},
{"LibreDWGc::Dwg_Object_VBA_PROJECT_data_size_set", _wrap_Dwg_Object_VBA_PROJECT_data_size_set},
{"LibreDWGc::Dwg_Object_VBA_PROJECT_data_size_get", _wrap_Dwg_Object_VBA_PROJECT_data_size_get},
{"LibreDWGc::Dwg_Object_VBA_PROJECT_data_set", _wrap_Dwg_Object_VBA_PROJECT_data_set},
{"LibreDWGc::Dwg_Object_VBA_PROJECT_data_get", _wrap_Dwg_Object_VBA_PROJECT_data_get},
{"LibreDWGc::new_Dwg_Object_VBA_PROJECT", _wrap_new_Dwg_Object_VBA_PROJECT},
{"LibreDWGc::delete_Dwg_Object_VBA_PROJECT", _wrap_delete_Dwg_Object_VBA_PROJECT},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_parent_set", _wrap_Dwg_Object_PLOTSETTINGS_parent_set},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_parent_get", _wrap_Dwg_Object_PLOTSETTINGS_parent_get},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_printer_cfg_file_set", _wrap_Dwg_Object_PLOTSETTINGS_printer_cfg_file_set},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_printer_cfg_file_get", _wrap_Dwg_Object_PLOTSETTINGS_printer_cfg_file_get},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_paper_size_set", _wrap_Dwg_Object_PLOTSETTINGS_paper_size_set},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_paper_size_get", _wrap_Dwg_Object_PLOTSETTINGS_paper_size_get},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_canonical_media_name_set", _wrap_Dwg_Object_PLOTSETTINGS_canonical_media_name_set},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_canonical_media_name_get", _wrap_Dwg_Object_PLOTSETTINGS_canonical_media_name_get},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_plot_flags_set", _wrap_Dwg_Object_PLOTSETTINGS_plot_flags_set},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_plot_flags_get", _wrap_Dwg_Object_PLOTSETTINGS_plot_flags_get},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_plotview_set", _wrap_Dwg_Object_PLOTSETTINGS_plotview_set},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_plotview_get", _wrap_Dwg_Object_PLOTSETTINGS_plotview_get},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_plotview_name_set", _wrap_Dwg_Object_PLOTSETTINGS_plotview_name_set},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_plotview_name_get", _wrap_Dwg_Object_PLOTSETTINGS_plotview_name_get},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_left_margin_set", _wrap_Dwg_Object_PLOTSETTINGS_left_margin_set},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_left_margin_get", _wrap_Dwg_Object_PLOTSETTINGS_left_margin_get},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_bottom_margin_set", _wrap_Dwg_Object_PLOTSETTINGS_bottom_margin_set},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_bottom_margin_get", _wrap_Dwg_Object_PLOTSETTINGS_bottom_margin_get},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_right_margin_set", _wrap_Dwg_Object_PLOTSETTINGS_right_margin_set},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_right_margin_get", _wrap_Dwg_Object_PLOTSETTINGS_right_margin_get},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_top_margin_set", _wrap_Dwg_Object_PLOTSETTINGS_top_margin_set},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_top_margin_get", _wrap_Dwg_Object_PLOTSETTINGS_top_margin_get},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_paper_width_set", _wrap_Dwg_Object_PLOTSETTINGS_paper_width_set},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_paper_width_get", _wrap_Dwg_Object_PLOTSETTINGS_paper_width_get},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_paper_height_set", _wrap_Dwg_Object_PLOTSETTINGS_paper_height_set},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_paper_height_get", _wrap_Dwg_Object_PLOTSETTINGS_paper_height_get},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_plot_origin_set", _wrap_Dwg_Object_PLOTSETTINGS_plot_origin_set},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_plot_origin_get", _wrap_Dwg_Object_PLOTSETTINGS_plot_origin_get},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_plot_window_ll_set", _wrap_Dwg_Object_PLOTSETTINGS_plot_window_ll_set},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_plot_window_ll_get", _wrap_Dwg_Object_PLOTSETTINGS_plot_window_ll_get},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_plot_window_ur_set", _wrap_Dwg_Object_PLOTSETTINGS_plot_window_ur_set},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_plot_window_ur_get", _wrap_Dwg_Object_PLOTSETTINGS_plot_window_ur_get},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_plot_paper_unit_set", _wrap_Dwg_Object_PLOTSETTINGS_plot_paper_unit_set},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_plot_paper_unit_get", _wrap_Dwg_Object_PLOTSETTINGS_plot_paper_unit_get},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_plot_rotation_mode_set", _wrap_Dwg_Object_PLOTSETTINGS_plot_rotation_mode_set},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_plot_rotation_mode_get", _wrap_Dwg_Object_PLOTSETTINGS_plot_rotation_mode_get},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_plot_type_set", _wrap_Dwg_Object_PLOTSETTINGS_plot_type_set},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_plot_type_get", _wrap_Dwg_Object_PLOTSETTINGS_plot_type_get},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_paper_units_set", _wrap_Dwg_Object_PLOTSETTINGS_paper_units_set},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_paper_units_get", _wrap_Dwg_Object_PLOTSETTINGS_paper_units_get},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_drawing_units_set", _wrap_Dwg_Object_PLOTSETTINGS_drawing_units_set},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_drawing_units_get", _wrap_Dwg_Object_PLOTSETTINGS_drawing_units_get},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_stylesheet_set", _wrap_Dwg_Object_PLOTSETTINGS_stylesheet_set},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_stylesheet_get", _wrap_Dwg_Object_PLOTSETTINGS_stylesheet_get},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_std_scale_type_set", _wrap_Dwg_Object_PLOTSETTINGS_std_scale_type_set},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_std_scale_type_get", _wrap_Dwg_Object_PLOTSETTINGS_std_scale_type_get},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_std_scale_factor_set", _wrap_Dwg_Object_PLOTSETTINGS_std_scale_factor_set},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_std_scale_factor_get", _wrap_Dwg_Object_PLOTSETTINGS_std_scale_factor_get},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_paper_image_origin_set", _wrap_Dwg_Object_PLOTSETTINGS_paper_image_origin_set},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_paper_image_origin_get", _wrap_Dwg_Object_PLOTSETTINGS_paper_image_origin_get},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_shadeplot_type_set", _wrap_Dwg_Object_PLOTSETTINGS_shadeplot_type_set},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_shadeplot_type_get", _wrap_Dwg_Object_PLOTSETTINGS_shadeplot_type_get},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_shadeplot_reslevel_set", _wrap_Dwg_Object_PLOTSETTINGS_shadeplot_reslevel_set},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_shadeplot_reslevel_get", _wrap_Dwg_Object_PLOTSETTINGS_shadeplot_reslevel_get},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_shadeplot_customdpi_set", _wrap_Dwg_Object_PLOTSETTINGS_shadeplot_customdpi_set},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_shadeplot_customdpi_get", _wrap_Dwg_Object_PLOTSETTINGS_shadeplot_customdpi_get},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_shadeplot_set", _wrap_Dwg_Object_PLOTSETTINGS_shadeplot_set},
{"LibreDWGc::Dwg_Object_PLOTSETTINGS_shadeplot_get", _wrap_Dwg_Object_PLOTSETTINGS_shadeplot_get},
{"LibreDWGc::new_Dwg_Object_PLOTSETTINGS", _wrap_new_Dwg_Object_PLOTSETTINGS},
{"LibreDWGc::delete_Dwg_Object_PLOTSETTINGS", _wrap_delete_Dwg_Object_PLOTSETTINGS},
{"LibreDWGc::Dwg_Object_LAYOUT_parent_set", _wrap_Dwg_Object_LAYOUT_parent_set},
{"LibreDWGc::Dwg_Object_LAYOUT_parent_get", _wrap_Dwg_Object_LAYOUT_parent_get},
{"LibreDWGc::Dwg_Object_LAYOUT_plotsettings_set", _wrap_Dwg_Object_LAYOUT_plotsettings_set},
{"LibreDWGc::Dwg_Object_LAYOUT_plotsettings_get", _wrap_Dwg_Object_LAYOUT_plotsettings_get},
{"LibreDWGc::Dwg_Object_LAYOUT_layout_name_set", _wrap_Dwg_Object_LAYOUT_layout_name_set},
{"LibreDWGc::Dwg_Object_LAYOUT_layout_name_get", _wrap_Dwg_Object_LAYOUT_layout_name_get},
{"LibreDWGc::Dwg_Object_LAYOUT_tab_order_set", _wrap_Dwg_Object_LAYOUT_tab_order_set},
{"LibreDWGc::Dwg_Object_LAYOUT_tab_order_get", _wrap_Dwg_Object_LAYOUT_tab_order_get},
{"LibreDWGc::Dwg_Object_LAYOUT_layout_flags_set", _wrap_Dwg_Object_LAYOUT_layout_flags_set},
{"LibreDWGc::Dwg_Object_LAYOUT_layout_flags_get", _wrap_Dwg_Object_LAYOUT_layout_flags_get},
{"LibreDWGc::Dwg_Object_LAYOUT_INSBASE_set", _wrap_Dwg_Object_LAYOUT_INSBASE_set},
{"LibreDWGc::Dwg_Object_LAYOUT_INSBASE_get", _wrap_Dwg_Object_LAYOUT_INSBASE_get},
{"LibreDWGc::Dwg_Object_LAYOUT_LIMMIN_set", _wrap_Dwg_Object_LAYOUT_LIMMIN_set},
{"LibreDWGc::Dwg_Object_LAYOUT_LIMMIN_get", _wrap_Dwg_Object_LAYOUT_LIMMIN_get},
{"LibreDWGc::Dwg_Object_LAYOUT_LIMMAX_set", _wrap_Dwg_Object_LAYOUT_LIMMAX_set},
{"LibreDWGc::Dwg_Object_LAYOUT_LIMMAX_get", _wrap_Dwg_Object_LAYOUT_LIMMAX_get},
{"LibreDWGc::Dwg_Object_LAYOUT_UCSORG_set", _wrap_Dwg_Object_LAYOUT_UCSORG_set},
{"LibreDWGc::Dwg_Object_LAYOUT_UCSORG_get", _wrap_Dwg_Object_LAYOUT_UCSORG_get},
{"LibreDWGc::Dwg_Object_LAYOUT_UCSXDIR_set", _wrap_Dwg_Object_LAYOUT_UCSXDIR_set},
{"LibreDWGc::Dwg_Object_LAYOUT_UCSXDIR_get", _wrap_Dwg_Object_LAYOUT_UCSXDIR_get},
{"LibreDWGc::Dwg_Object_LAYOUT_UCSYDIR_set", _wrap_Dwg_Object_LAYOUT_UCSYDIR_set},
{"LibreDWGc::Dwg_Object_LAYOUT_UCSYDIR_get", _wrap_Dwg_Object_LAYOUT_UCSYDIR_get},
{"LibreDWGc::Dwg_Object_LAYOUT_ucs_elevation_set", _wrap_Dwg_Object_LAYOUT_ucs_elevation_set},
{"LibreDWGc::Dwg_Object_LAYOUT_ucs_elevation_get", _wrap_Dwg_Object_LAYOUT_ucs_elevation_get},
{"LibreDWGc::Dwg_Object_LAYOUT_UCSORTHOVIEW_set", _wrap_Dwg_Object_LAYOUT_UCSORTHOVIEW_set},
{"LibreDWGc::Dwg_Object_LAYOUT_UCSORTHOVIEW_get", _wrap_Dwg_Object_LAYOUT_UCSORTHOVIEW_get},
{"LibreDWGc::Dwg_Object_LAYOUT_EXTMIN_set", _wrap_Dwg_Object_LAYOUT_EXTMIN_set},
{"LibreDWGc::Dwg_Object_LAYOUT_EXTMIN_get", _wrap_Dwg_Object_LAYOUT_EXTMIN_get},
{"LibreDWGc::Dwg_Object_LAYOUT_EXTMAX_set", _wrap_Dwg_Object_LAYOUT_EXTMAX_set},
{"LibreDWGc::Dwg_Object_LAYOUT_EXTMAX_get", _wrap_Dwg_Object_LAYOUT_EXTMAX_get},
{"LibreDWGc::Dwg_Object_LAYOUT_block_header_set", _wrap_Dwg_Object_LAYOUT_block_header_set},
{"LibreDWGc::Dwg_Object_LAYOUT_block_header_get", _wrap_Dwg_Object_LAYOUT_block_header_get},
{"LibreDWGc::Dwg_Object_LAYOUT_active_viewport_set", _wrap_Dwg_Object_LAYOUT_active_viewport_set},
{"LibreDWGc::Dwg_Object_LAYOUT_active_viewport_get", _wrap_Dwg_Object_LAYOUT_active_viewport_get},
{"LibreDWGc::Dwg_Object_LAYOUT_base_ucs_set", _wrap_Dwg_Object_LAYOUT_base_ucs_set},
{"LibreDWGc::Dwg_Object_LAYOUT_base_ucs_get", _wrap_Dwg_Object_LAYOUT_base_ucs_get},
{"LibreDWGc::Dwg_Object_LAYOUT_named_ucs_set", _wrap_Dwg_Object_LAYOUT_named_ucs_set},
{"LibreDWGc::Dwg_Object_LAYOUT_named_ucs_get", _wrap_Dwg_Object_LAYOUT_named_ucs_get},
{"LibreDWGc::Dwg_Object_LAYOUT_num_viewports_set", _wrap_Dwg_Object_LAYOUT_num_viewports_set},
{"LibreDWGc::Dwg_Object_LAYOUT_num_viewports_get", _wrap_Dwg_Object_LAYOUT_num_viewports_get},
{"LibreDWGc::Dwg_Object_LAYOUT_viewports_set", _wrap_Dwg_Object_LAYOUT_viewports_set},
{"LibreDWGc::Dwg_Object_LAYOUT_viewports_get", _wrap_Dwg_Object_LAYOUT_viewports_get},
{"LibreDWGc::new_Dwg_Object_LAYOUT", _wrap_new_Dwg_Object_LAYOUT},
{"LibreDWGc::delete_Dwg_Object_LAYOUT", _wrap_delete_Dwg_Object_LAYOUT},
{"LibreDWGc::Dwg_Object_DICTIONARYVAR_parent_set", _wrap_Dwg_Object_DICTIONARYVAR_parent_set},
{"LibreDWGc::Dwg_Object_DICTIONARYVAR_parent_get", _wrap_Dwg_Object_DICTIONARYVAR_parent_get},
{"LibreDWGc::Dwg_Object_DICTIONARYVAR_schema_set", _wrap_Dwg_Object_DICTIONARYVAR_schema_set},
{"LibreDWGc::Dwg_Object_DICTIONARYVAR_schema_get", _wrap_Dwg_Object_DICTIONARYVAR_schema_get},
{"LibreDWGc::Dwg_Object_DICTIONARYVAR_strvalue_set", _wrap_Dwg_Object_DICTIONARYVAR_strvalue_set},
{"LibreDWGc::Dwg_Object_DICTIONARYVAR_strvalue_get", _wrap_Dwg_Object_DICTIONARYVAR_strvalue_get},
{"LibreDWGc::new_Dwg_Object_DICTIONARYVAR", _wrap_new_Dwg_Object_DICTIONARYVAR},
{"LibreDWGc::delete_Dwg_Object_DICTIONARYVAR", _wrap_delete_Dwg_Object_DICTIONARYVAR},
{"LibreDWGc::Dwg_TABLE_value_flags_set", _wrap_Dwg_TABLE_value_flags_set},
{"LibreDWGc::Dwg_TABLE_value_flags_get", _wrap_Dwg_TABLE_value_flags_get},
{"LibreDWGc::Dwg_TABLE_value_format_flags_set", _wrap_Dwg_TABLE_value_format_flags_set},
{"LibreDWGc::Dwg_TABLE_value_format_flags_get", _wrap_Dwg_TABLE_value_format_flags_get},
{"LibreDWGc::Dwg_TABLE_value_data_type_set", _wrap_Dwg_TABLE_value_data_type_set},
{"LibreDWGc::Dwg_TABLE_value_data_type_get", _wrap_Dwg_TABLE_value_data_type_get},
{"LibreDWGc::Dwg_TABLE_value_data_size_set", _wrap_Dwg_TABLE_value_data_size_set},
{"LibreDWGc::Dwg_TABLE_value_data_size_get", _wrap_Dwg_TABLE_value_data_size_get},
{"LibreDWGc::Dwg_TABLE_value_data_long_set", _wrap_Dwg_TABLE_value_data_long_set},
{"LibreDWGc::Dwg_TABLE_value_data_long_get", _wrap_Dwg_TABLE_value_data_long_get},
{"LibreDWGc::Dwg_TABLE_value_data_double_set", _wrap_Dwg_TABLE_value_data_double_set},
{"LibreDWGc::Dwg_TABLE_value_data_double_get", _wrap_Dwg_TABLE_value_data_double_get},
{"LibreDWGc::Dwg_TABLE_value_data_string_set", _wrap_Dwg_TABLE_value_data_string_set},
{"LibreDWGc::Dwg_TABLE_value_data_string_get", _wrap_Dwg_TABLE_value_data_string_get},
{"LibreDWGc::Dwg_TABLE_value_data_date_set", _wrap_Dwg_TABLE_value_data_date_set},
{"LibreDWGc::Dwg_TABLE_value_data_date_get", _wrap_Dwg_TABLE_value_data_date_get},
{"LibreDWGc::Dwg_TABLE_value_data_point_set", _wrap_Dwg_TABLE_value_data_point_set},
{"LibreDWGc::Dwg_TABLE_value_data_point_get", _wrap_Dwg_TABLE_value_data_point_get},
{"LibreDWGc::Dwg_TABLE_value_data_3dpoint_set", _wrap_Dwg_TABLE_value_data_3dpoint_set},
{"LibreDWGc::Dwg_TABLE_value_data_3dpoint_get", _wrap_Dwg_TABLE_value_data_3dpoint_get},
{"LibreDWGc::Dwg_TABLE_value_data_handle_set", _wrap_Dwg_TABLE_value_data_handle_set},
{"LibreDWGc::Dwg_TABLE_value_data_handle_get", _wrap_Dwg_TABLE_value_data_handle_get},
{"LibreDWGc::Dwg_TABLE_value_unit_type_set", _wrap_Dwg_TABLE_value_unit_type_set},
{"LibreDWGc::Dwg_TABLE_value_unit_type_get", _wrap_Dwg_TABLE_value_unit_type_get},
{"LibreDWGc::Dwg_TABLE_value_format_string_set", _wrap_Dwg_TABLE_value_format_string_set},
{"LibreDWGc::Dwg_TABLE_value_format_string_get", _wrap_Dwg_TABLE_value_format_string_get},
{"LibreDWGc::Dwg_TABLE_value_value_string_set", _wrap_Dwg_TABLE_value_value_string_set},
{"LibreDWGc::Dwg_TABLE_value_value_string_get", _wrap_Dwg_TABLE_value_value_string_get},
{"LibreDWGc::new_Dwg_TABLE_value", _wrap_new_Dwg_TABLE_value},
{"LibreDWGc::delete_Dwg_TABLE_value", _wrap_delete_Dwg_TABLE_value},
{"LibreDWGc::Dwg_TABLE_CustomDataItem_name_set", _wrap_Dwg_TABLE_CustomDataItem_name_set},
{"LibreDWGc::Dwg_TABLE_CustomDataItem_name_get", _wrap_Dwg_TABLE_CustomDataItem_name_get},
{"LibreDWGc::Dwg_TABLE_CustomDataItem_value_set", _wrap_Dwg_TABLE_CustomDataItem_value_set},
{"LibreDWGc::Dwg_TABLE_CustomDataItem_value_get", _wrap_Dwg_TABLE_CustomDataItem_value_get},
{"LibreDWGc::Dwg_TABLE_CustomDataItem_cell_parent_set", _wrap_Dwg_TABLE_CustomDataItem_cell_parent_set},
{"LibreDWGc::Dwg_TABLE_CustomDataItem_cell_parent_get", _wrap_Dwg_TABLE_CustomDataItem_cell_parent_get},
{"LibreDWGc::Dwg_TABLE_CustomDataItem_row_parent_set", _wrap_Dwg_TABLE_CustomDataItem_row_parent_set},
{"LibreDWGc::Dwg_TABLE_CustomDataItem_row_parent_get", _wrap_Dwg_TABLE_CustomDataItem_row_parent_get},
{"LibreDWGc::new_Dwg_TABLE_CustomDataItem", _wrap_new_Dwg_TABLE_CustomDataItem},
{"LibreDWGc::delete_Dwg_TABLE_CustomDataItem", _wrap_delete_Dwg_TABLE_CustomDataItem},
{"LibreDWGc::Dwg_TABLE_AttrDef_parent_set", _wrap_Dwg_TABLE_AttrDef_parent_set},
{"LibreDWGc::Dwg_TABLE_AttrDef_parent_get", _wrap_Dwg_TABLE_AttrDef_parent_get},
{"LibreDWGc::Dwg_TABLE_AttrDef_attdef_set", _wrap_Dwg_TABLE_AttrDef_attdef_set},
{"LibreDWGc::Dwg_TABLE_AttrDef_attdef_get", _wrap_Dwg_TABLE_AttrDef_attdef_get},
{"LibreDWGc::Dwg_TABLE_AttrDef_index_set", _wrap_Dwg_TABLE_AttrDef_index_set},
{"LibreDWGc::Dwg_TABLE_AttrDef_index_get", _wrap_Dwg_TABLE_AttrDef_index_get},
{"LibreDWGc::Dwg_TABLE_AttrDef_text_set", _wrap_Dwg_TABLE_AttrDef_text_set},
{"LibreDWGc::Dwg_TABLE_AttrDef_text_get", _wrap_Dwg_TABLE_AttrDef_text_get},
{"LibreDWGc::new_Dwg_TABLE_AttrDef", _wrap_new_Dwg_TABLE_AttrDef},
{"LibreDWGc::delete_Dwg_TABLE_AttrDef", _wrap_delete_Dwg_TABLE_AttrDef},
{"LibreDWGc::Dwg_TABLE_Cell_parent_set", _wrap_Dwg_TABLE_Cell_parent_set},
{"LibreDWGc::Dwg_TABLE_Cell_parent_get", _wrap_Dwg_TABLE_Cell_parent_get},
{"LibreDWGc::Dwg_TABLE_Cell_type_set", _wrap_Dwg_TABLE_Cell_type_set},
{"LibreDWGc::Dwg_TABLE_Cell_type_get", _wrap_Dwg_TABLE_Cell_type_get},
{"LibreDWGc::Dwg_TABLE_Cell_flags_set", _wrap_Dwg_TABLE_Cell_flags_set},
{"LibreDWGc::Dwg_TABLE_Cell_flags_get", _wrap_Dwg_TABLE_Cell_flags_get},
{"LibreDWGc::Dwg_TABLE_Cell_is_merged_value_set", _wrap_Dwg_TABLE_Cell_is_merged_value_set},
{"LibreDWGc::Dwg_TABLE_Cell_is_merged_value_get", _wrap_Dwg_TABLE_Cell_is_merged_value_get},
{"LibreDWGc::Dwg_TABLE_Cell_is_autofit_flag_set", _wrap_Dwg_TABLE_Cell_is_autofit_flag_set},
{"LibreDWGc::Dwg_TABLE_Cell_is_autofit_flag_get", _wrap_Dwg_TABLE_Cell_is_autofit_flag_get},
{"LibreDWGc::Dwg_TABLE_Cell_merged_width_flag_set", _wrap_Dwg_TABLE_Cell_merged_width_flag_set},
{"LibreDWGc::Dwg_TABLE_Cell_merged_width_flag_get", _wrap_Dwg_TABLE_Cell_merged_width_flag_get},
{"LibreDWGc::Dwg_TABLE_Cell_merged_height_flag_set", _wrap_Dwg_TABLE_Cell_merged_height_flag_set},
{"LibreDWGc::Dwg_TABLE_Cell_merged_height_flag_get", _wrap_Dwg_TABLE_Cell_merged_height_flag_get},
{"LibreDWGc::Dwg_TABLE_Cell_rotation_set", _wrap_Dwg_TABLE_Cell_rotation_set},
{"LibreDWGc::Dwg_TABLE_Cell_rotation_get", _wrap_Dwg_TABLE_Cell_rotation_get},
{"LibreDWGc::Dwg_TABLE_Cell_text_value_set", _wrap_Dwg_TABLE_Cell_text_value_set},
{"LibreDWGc::Dwg_TABLE_Cell_text_value_get", _wrap_Dwg_TABLE_Cell_text_value_get},
{"LibreDWGc::Dwg_TABLE_Cell_text_style_set", _wrap_Dwg_TABLE_Cell_text_style_set},
{"LibreDWGc::Dwg_TABLE_Cell_text_style_get", _wrap_Dwg_TABLE_Cell_text_style_get},
{"LibreDWGc::Dwg_TABLE_Cell_block_handle_set", _wrap_Dwg_TABLE_Cell_block_handle_set},
{"LibreDWGc::Dwg_TABLE_Cell_block_handle_get", _wrap_Dwg_TABLE_Cell_block_handle_get},
{"LibreDWGc::Dwg_TABLE_Cell_block_scale_set", _wrap_Dwg_TABLE_Cell_block_scale_set},
{"LibreDWGc::Dwg_TABLE_Cell_block_scale_get", _wrap_Dwg_TABLE_Cell_block_scale_get},
{"LibreDWGc::Dwg_TABLE_Cell_additional_data_flag_set", _wrap_Dwg_TABLE_Cell_additional_data_flag_set},
{"LibreDWGc::Dwg_TABLE_Cell_additional_data_flag_get", _wrap_Dwg_TABLE_Cell_additional_data_flag_get},
{"LibreDWGc::Dwg_TABLE_Cell_cell_flag_override_set", _wrap_Dwg_TABLE_Cell_cell_flag_override_set},
{"LibreDWGc::Dwg_TABLE_Cell_cell_flag_override_get", _wrap_Dwg_TABLE_Cell_cell_flag_override_get},
{"LibreDWGc::Dwg_TABLE_Cell_virtual_edge_flag_set", _wrap_Dwg_TABLE_Cell_virtual_edge_flag_set},
{"LibreDWGc::Dwg_TABLE_Cell_virtual_edge_flag_get", _wrap_Dwg_TABLE_Cell_virtual_edge_flag_get},
{"LibreDWGc::Dwg_TABLE_Cell_cell_alignment_set", _wrap_Dwg_TABLE_Cell_cell_alignment_set},
{"LibreDWGc::Dwg_TABLE_Cell_cell_alignment_get", _wrap_Dwg_TABLE_Cell_cell_alignment_get},
{"LibreDWGc::Dwg_TABLE_Cell_bg_fill_none_set", _wrap_Dwg_TABLE_Cell_bg_fill_none_set},
{"LibreDWGc::Dwg_TABLE_Cell_bg_fill_none_get", _wrap_Dwg_TABLE_Cell_bg_fill_none_get},
{"LibreDWGc::Dwg_TABLE_Cell_bg_color_set", _wrap_Dwg_TABLE_Cell_bg_color_set},
{"LibreDWGc::Dwg_TABLE_Cell_bg_color_get", _wrap_Dwg_TABLE_Cell_bg_color_get},
{"LibreDWGc::Dwg_TABLE_Cell_content_color_set", _wrap_Dwg_TABLE_Cell_content_color_set},
{"LibreDWGc::Dwg_TABLE_Cell_content_color_get", _wrap_Dwg_TABLE_Cell_content_color_get},
{"LibreDWGc::Dwg_TABLE_Cell_text_height_set", _wrap_Dwg_TABLE_Cell_text_height_set},
{"LibreDWGc::Dwg_TABLE_Cell_text_height_get", _wrap_Dwg_TABLE_Cell_text_height_get},
{"LibreDWGc::Dwg_TABLE_Cell_top_grid_color_set", _wrap_Dwg_TABLE_Cell_top_grid_color_set},
{"LibreDWGc::Dwg_TABLE_Cell_top_grid_color_get", _wrap_Dwg_TABLE_Cell_top_grid_color_get},
{"LibreDWGc::Dwg_TABLE_Cell_top_grid_linewt_set", _wrap_Dwg_TABLE_Cell_top_grid_linewt_set},
{"LibreDWGc::Dwg_TABLE_Cell_top_grid_linewt_get", _wrap_Dwg_TABLE_Cell_top_grid_linewt_get},
{"LibreDWGc::Dwg_TABLE_Cell_top_visibility_set", _wrap_Dwg_TABLE_Cell_top_visibility_set},
{"LibreDWGc::Dwg_TABLE_Cell_top_visibility_get", _wrap_Dwg_TABLE_Cell_top_visibility_get},
{"LibreDWGc::Dwg_TABLE_Cell_right_grid_color_set", _wrap_Dwg_TABLE_Cell_right_grid_color_set},
{"LibreDWGc::Dwg_TABLE_Cell_right_grid_color_get", _wrap_Dwg_TABLE_Cell_right_grid_color_get},
{"LibreDWGc::Dwg_TABLE_Cell_right_grid_linewt_set", _wrap_Dwg_TABLE_Cell_right_grid_linewt_set},
{"LibreDWGc::Dwg_TABLE_Cell_right_grid_linewt_get", _wrap_Dwg_TABLE_Cell_right_grid_linewt_get},
{"LibreDWGc::Dwg_TABLE_Cell_right_visibility_set", _wrap_Dwg_TABLE_Cell_right_visibility_set},
{"LibreDWGc::Dwg_TABLE_Cell_right_visibility_get", _wrap_Dwg_TABLE_Cell_right_visibility_get},
{"LibreDWGc::Dwg_TABLE_Cell_bottom_grid_color_set", _wrap_Dwg_TABLE_Cell_bottom_grid_color_set},
{"LibreDWGc::Dwg_TABLE_Cell_bottom_grid_color_get", _wrap_Dwg_TABLE_Cell_bottom_grid_color_get},
{"LibreDWGc::Dwg_TABLE_Cell_bottom_grid_linewt_set", _wrap_Dwg_TABLE_Cell_bottom_grid_linewt_set},
{"LibreDWGc::Dwg_TABLE_Cell_bottom_grid_linewt_get", _wrap_Dwg_TABLE_Cell_bottom_grid_linewt_get},
{"LibreDWGc::Dwg_TABLE_Cell_bottom_visibility_set", _wrap_Dwg_TABLE_Cell_bottom_visibility_set},
{"LibreDWGc::Dwg_TABLE_Cell_bottom_visibility_get", _wrap_Dwg_TABLE_Cell_bottom_visibility_get},
{"LibreDWGc::Dwg_TABLE_Cell_left_grid_color_set", _wrap_Dwg_TABLE_Cell_left_grid_color_set},
{"LibreDWGc::Dwg_TABLE_Cell_left_grid_color_get", _wrap_Dwg_TABLE_Cell_left_grid_color_get},
{"LibreDWGc::Dwg_TABLE_Cell_left_grid_linewt_set", _wrap_Dwg_TABLE_Cell_left_grid_linewt_set},
{"LibreDWGc::Dwg_TABLE_Cell_left_grid_linewt_get", _wrap_Dwg_TABLE_Cell_left_grid_linewt_get},
{"LibreDWGc::Dwg_TABLE_Cell_left_visibility_set", _wrap_Dwg_TABLE_Cell_left_visibility_set},
{"LibreDWGc::Dwg_TABLE_Cell_left_visibility_get", _wrap_Dwg_TABLE_Cell_left_visibility_get},
{"LibreDWGc::Dwg_TABLE_Cell_unknown_set", _wrap_Dwg_TABLE_Cell_unknown_set},
{"LibreDWGc::Dwg_TABLE_Cell_unknown_get", _wrap_Dwg_TABLE_Cell_unknown_get},
{"LibreDWGc::Dwg_TABLE_Cell_value_set", _wrap_Dwg_TABLE_Cell_value_set},
{"LibreDWGc::Dwg_TABLE_Cell_value_get", _wrap_Dwg_TABLE_Cell_value_get},
{"LibreDWGc::Dwg_TABLE_Cell_num_attr_defs_set", _wrap_Dwg_TABLE_Cell_num_attr_defs_set},
{"LibreDWGc::Dwg_TABLE_Cell_num_attr_defs_get", _wrap_Dwg_TABLE_Cell_num_attr_defs_get},
{"LibreDWGc::Dwg_TABLE_Cell_attr_defs_set", _wrap_Dwg_TABLE_Cell_attr_defs_set},
{"LibreDWGc::Dwg_TABLE_Cell_attr_defs_get", _wrap_Dwg_TABLE_Cell_attr_defs_get},
{"LibreDWGc::new_Dwg_TABLE_Cell", _wrap_new_Dwg_TABLE_Cell},
{"LibreDWGc::delete_Dwg_TABLE_Cell", _wrap_delete_Dwg_TABLE_Cell},
{"LibreDWGc::Dwg_TABLE_BreakHeight_parent_set", _wrap_Dwg_TABLE_BreakHeight_parent_set},
{"LibreDWGc::Dwg_TABLE_BreakHeight_parent_get", _wrap_Dwg_TABLE_BreakHeight_parent_get},
{"LibreDWGc::Dwg_TABLE_BreakHeight_position_set", _wrap_Dwg_TABLE_BreakHeight_position_set},
{"LibreDWGc::Dwg_TABLE_BreakHeight_position_get", _wrap_Dwg_TABLE_BreakHeight_position_get},
{"LibreDWGc::Dwg_TABLE_BreakHeight_height_set", _wrap_Dwg_TABLE_BreakHeight_height_set},
{"LibreDWGc::Dwg_TABLE_BreakHeight_height_get", _wrap_Dwg_TABLE_BreakHeight_height_get},
{"LibreDWGc::Dwg_TABLE_BreakHeight_flag_set", _wrap_Dwg_TABLE_BreakHeight_flag_set},
{"LibreDWGc::Dwg_TABLE_BreakHeight_flag_get", _wrap_Dwg_TABLE_BreakHeight_flag_get},
{"LibreDWGc::new_Dwg_TABLE_BreakHeight", _wrap_new_Dwg_TABLE_BreakHeight},
{"LibreDWGc::delete_Dwg_TABLE_BreakHeight", _wrap_delete_Dwg_TABLE_BreakHeight},
{"LibreDWGc::Dwg_TABLE_BreakRow_parent_set", _wrap_Dwg_TABLE_BreakRow_parent_set},
{"LibreDWGc::Dwg_TABLE_BreakRow_parent_get", _wrap_Dwg_TABLE_BreakRow_parent_get},
{"LibreDWGc::Dwg_TABLE_BreakRow_position_set", _wrap_Dwg_TABLE_BreakRow_position_set},
{"LibreDWGc::Dwg_TABLE_BreakRow_position_get", _wrap_Dwg_TABLE_BreakRow_position_get},
{"LibreDWGc::Dwg_TABLE_BreakRow_start_set", _wrap_Dwg_TABLE_BreakRow_start_set},
{"LibreDWGc::Dwg_TABLE_BreakRow_start_get", _wrap_Dwg_TABLE_BreakRow_start_get},
{"LibreDWGc::Dwg_TABLE_BreakRow_end_set", _wrap_Dwg_TABLE_BreakRow_end_set},
{"LibreDWGc::Dwg_TABLE_BreakRow_end_get", _wrap_Dwg_TABLE_BreakRow_end_get},
{"LibreDWGc::new_Dwg_TABLE_BreakRow", _wrap_new_Dwg_TABLE_BreakRow},
{"LibreDWGc::delete_Dwg_TABLE_BreakRow", _wrap_delete_Dwg_TABLE_BreakRow},
{"LibreDWGc::Dwg_LinkedData_name_set", _wrap_Dwg_LinkedData_name_set},
{"LibreDWGc::Dwg_LinkedData_name_get", _wrap_Dwg_LinkedData_name_get},
{"LibreDWGc::Dwg_LinkedData_description_set", _wrap_Dwg_LinkedData_description_set},
{"LibreDWGc::Dwg_LinkedData_description_get", _wrap_Dwg_LinkedData_description_get},
{"LibreDWGc::new_Dwg_LinkedData", _wrap_new_Dwg_LinkedData},
{"LibreDWGc::delete_Dwg_LinkedData", _wrap_delete_Dwg_LinkedData},
{"LibreDWGc::Dwg_TableCellContent_Attr_parent_set", _wrap_Dwg_TableCellContent_Attr_parent_set},
{"LibreDWGc::Dwg_TableCellContent_Attr_parent_get", _wrap_Dwg_TableCellContent_Attr_parent_get},
{"LibreDWGc::Dwg_TableCellContent_Attr_attdef_set", _wrap_Dwg_TableCellContent_Attr_attdef_set},
{"LibreDWGc::Dwg_TableCellContent_Attr_attdef_get", _wrap_Dwg_TableCellContent_Attr_attdef_get},
{"LibreDWGc::Dwg_TableCellContent_Attr_value_set", _wrap_Dwg_TableCellContent_Attr_value_set},
{"LibreDWGc::Dwg_TableCellContent_Attr_value_get", _wrap_Dwg_TableCellContent_Attr_value_get},
{"LibreDWGc::Dwg_TableCellContent_Attr_index_set", _wrap_Dwg_TableCellContent_Attr_index_set},
{"LibreDWGc::Dwg_TableCellContent_Attr_index_get", _wrap_Dwg_TableCellContent_Attr_index_get},
{"LibreDWGc::new_Dwg_TableCellContent_Attr", _wrap_new_Dwg_TableCellContent_Attr},
{"LibreDWGc::delete_Dwg_TableCellContent_Attr", _wrap_delete_Dwg_TableCellContent_Attr},
{"LibreDWGc::Dwg_ContentFormat_property_override_flags_set", _wrap_Dwg_ContentFormat_property_override_flags_set},
{"LibreDWGc::Dwg_ContentFormat_property_override_flags_get", _wrap_Dwg_ContentFormat_property_override_flags_get},
{"LibreDWGc::Dwg_ContentFormat_property_flags_set", _wrap_Dwg_ContentFormat_property_flags_set},
{"LibreDWGc::Dwg_ContentFormat_property_flags_get", _wrap_Dwg_ContentFormat_property_flags_get},
{"LibreDWGc::Dwg_ContentFormat_value_data_type_set", _wrap_Dwg_ContentFormat_value_data_type_set},
{"LibreDWGc::Dwg_ContentFormat_value_data_type_get", _wrap_Dwg_ContentFormat_value_data_type_get},
{"LibreDWGc::Dwg_ContentFormat_value_unit_type_set", _wrap_Dwg_ContentFormat_value_unit_type_set},
{"LibreDWGc::Dwg_ContentFormat_value_unit_type_get", _wrap_Dwg_ContentFormat_value_unit_type_get},
{"LibreDWGc::Dwg_ContentFormat_value_format_string_set", _wrap_Dwg_ContentFormat_value_format_string_set},
{"LibreDWGc::Dwg_ContentFormat_value_format_string_get", _wrap_Dwg_ContentFormat_value_format_string_get},
{"LibreDWGc::Dwg_ContentFormat_rotation_set", _wrap_Dwg_ContentFormat_rotation_set},
{"LibreDWGc::Dwg_ContentFormat_rotation_get", _wrap_Dwg_ContentFormat_rotation_get},
{"LibreDWGc::Dwg_ContentFormat_block_scale_set", _wrap_Dwg_ContentFormat_block_scale_set},
{"LibreDWGc::Dwg_ContentFormat_block_scale_get", _wrap_Dwg_ContentFormat_block_scale_get},
{"LibreDWGc::Dwg_ContentFormat_cell_alignment_set", _wrap_Dwg_ContentFormat_cell_alignment_set},
{"LibreDWGc::Dwg_ContentFormat_cell_alignment_get", _wrap_Dwg_ContentFormat_cell_alignment_get},
{"LibreDWGc::Dwg_ContentFormat_content_color_set", _wrap_Dwg_ContentFormat_content_color_set},
{"LibreDWGc::Dwg_ContentFormat_content_color_get", _wrap_Dwg_ContentFormat_content_color_get},
{"LibreDWGc::Dwg_ContentFormat_text_style_set", _wrap_Dwg_ContentFormat_text_style_set},
{"LibreDWGc::Dwg_ContentFormat_text_style_get", _wrap_Dwg_ContentFormat_text_style_get},
{"LibreDWGc::Dwg_ContentFormat_text_height_set", _wrap_Dwg_ContentFormat_text_height_set},
{"LibreDWGc::Dwg_ContentFormat_text_height_get", _wrap_Dwg_ContentFormat_text_height_get},
{"LibreDWGc::new_Dwg_ContentFormat", _wrap_new_Dwg_ContentFormat},
{"LibreDWGc::delete_Dwg_ContentFormat", _wrap_delete_Dwg_ContentFormat},
{"LibreDWGc::Dwg_TableCellContent_parent_set", _wrap_Dwg_TableCellContent_parent_set},
{"LibreDWGc::Dwg_TableCellContent_parent_get", _wrap_Dwg_TableCellContent_parent_get},
{"LibreDWGc::Dwg_TableCellContent_type_set", _wrap_Dwg_TableCellContent_type_set},
{"LibreDWGc::Dwg_TableCellContent_type_get", _wrap_Dwg_TableCellContent_type_get},
{"LibreDWGc::Dwg_TableCellContent_value_set", _wrap_Dwg_TableCellContent_value_set},
{"LibreDWGc::Dwg_TableCellContent_value_get", _wrap_Dwg_TableCellContent_value_get},
{"LibreDWGc::Dwg_TableCellContent_handle_set", _wrap_Dwg_TableCellContent_handle_set},
{"LibreDWGc::Dwg_TableCellContent_handle_get", _wrap_Dwg_TableCellContent_handle_get},
{"LibreDWGc::Dwg_TableCellContent_num_attrs_set", _wrap_Dwg_TableCellContent_num_attrs_set},
{"LibreDWGc::Dwg_TableCellContent_num_attrs_get", _wrap_Dwg_TableCellContent_num_attrs_get},
{"LibreDWGc::Dwg_TableCellContent_attrs_set", _wrap_Dwg_TableCellContent_attrs_set},
{"LibreDWGc::Dwg_TableCellContent_attrs_get", _wrap_Dwg_TableCellContent_attrs_get},
{"LibreDWGc::Dwg_TableCellContent_has_content_format_overrides_set", _wrap_Dwg_TableCellContent_has_content_format_overrides_set},
{"LibreDWGc::Dwg_TableCellContent_has_content_format_overrides_get", _wrap_Dwg_TableCellContent_has_content_format_overrides_get},
{"LibreDWGc::Dwg_TableCellContent_content_format_set", _wrap_Dwg_TableCellContent_content_format_set},
{"LibreDWGc::Dwg_TableCellContent_content_format_get", _wrap_Dwg_TableCellContent_content_format_get},
{"LibreDWGc::new_Dwg_TableCellContent", _wrap_new_Dwg_TableCellContent},
{"LibreDWGc::delete_Dwg_TableCellContent", _wrap_delete_Dwg_TableCellContent},
{"LibreDWGc::Dwg_CellContentGeometry_dist_top_left_set", _wrap_Dwg_CellContentGeometry_dist_top_left_set},
{"LibreDWGc::Dwg_CellContentGeometry_dist_top_left_get", _wrap_Dwg_CellContentGeometry_dist_top_left_get},
{"LibreDWGc::Dwg_CellContentGeometry_dist_center_set", _wrap_Dwg_CellContentGeometry_dist_center_set},
{"LibreDWGc::Dwg_CellContentGeometry_dist_center_get", _wrap_Dwg_CellContentGeometry_dist_center_get},
{"LibreDWGc::Dwg_CellContentGeometry_content_width_set", _wrap_Dwg_CellContentGeometry_content_width_set},
{"LibreDWGc::Dwg_CellContentGeometry_content_width_get", _wrap_Dwg_CellContentGeometry_content_width_get},
{"LibreDWGc::Dwg_CellContentGeometry_content_height_set", _wrap_Dwg_CellContentGeometry_content_height_set},
{"LibreDWGc::Dwg_CellContentGeometry_content_height_get", _wrap_Dwg_CellContentGeometry_content_height_get},
{"LibreDWGc::Dwg_CellContentGeometry_width_set", _wrap_Dwg_CellContentGeometry_width_set},
{"LibreDWGc::Dwg_CellContentGeometry_width_get", _wrap_Dwg_CellContentGeometry_width_get},
{"LibreDWGc::Dwg_CellContentGeometry_height_set", _wrap_Dwg_CellContentGeometry_height_set},
{"LibreDWGc::Dwg_CellContentGeometry_height_get", _wrap_Dwg_CellContentGeometry_height_get},
{"LibreDWGc::Dwg_CellContentGeometry_unknown_set", _wrap_Dwg_CellContentGeometry_unknown_set},
{"LibreDWGc::Dwg_CellContentGeometry_unknown_get", _wrap_Dwg_CellContentGeometry_unknown_get},
{"LibreDWGc::Dwg_CellContentGeometry_cell_parent_set", _wrap_Dwg_CellContentGeometry_cell_parent_set},
{"LibreDWGc::Dwg_CellContentGeometry_cell_parent_get", _wrap_Dwg_CellContentGeometry_cell_parent_get},
{"LibreDWGc::Dwg_CellContentGeometry_geom_parent_set", _wrap_Dwg_CellContentGeometry_geom_parent_set},
{"LibreDWGc::Dwg_CellContentGeometry_geom_parent_get", _wrap_Dwg_CellContentGeometry_geom_parent_get},
{"LibreDWGc::new_Dwg_CellContentGeometry", _wrap_new_Dwg_CellContentGeometry},
{"LibreDWGc::delete_Dwg_CellContentGeometry", _wrap_delete_Dwg_CellContentGeometry},
{"LibreDWGc::Dwg_TableCell_flag_set", _wrap_Dwg_TableCell_flag_set},
{"LibreDWGc::Dwg_TableCell_flag_get", _wrap_Dwg_TableCell_flag_get},
{"LibreDWGc::Dwg_TableCell_tooltip_set", _wrap_Dwg_TableCell_tooltip_set},
{"LibreDWGc::Dwg_TableCell_tooltip_get", _wrap_Dwg_TableCell_tooltip_get},
{"LibreDWGc::Dwg_TableCell_customdata_set", _wrap_Dwg_TableCell_customdata_set},
{"LibreDWGc::Dwg_TableCell_customdata_get", _wrap_Dwg_TableCell_customdata_get},
{"LibreDWGc::Dwg_TableCell_num_customdata_items_set", _wrap_Dwg_TableCell_num_customdata_items_set},
{"LibreDWGc::Dwg_TableCell_num_customdata_items_get", _wrap_Dwg_TableCell_num_customdata_items_get},
{"LibreDWGc::Dwg_TableCell_customdata_items_set", _wrap_Dwg_TableCell_customdata_items_set},
{"LibreDWGc::Dwg_TableCell_customdata_items_get", _wrap_Dwg_TableCell_customdata_items_get},
{"LibreDWGc::Dwg_TableCell_has_linked_data_set", _wrap_Dwg_TableCell_has_linked_data_set},
{"LibreDWGc::Dwg_TableCell_has_linked_data_get", _wrap_Dwg_TableCell_has_linked_data_get},
{"LibreDWGc::Dwg_TableCell_data_link_set", _wrap_Dwg_TableCell_data_link_set},
{"LibreDWGc::Dwg_TableCell_data_link_get", _wrap_Dwg_TableCell_data_link_get},
{"LibreDWGc::Dwg_TableCell_num_rows_set", _wrap_Dwg_TableCell_num_rows_set},
{"LibreDWGc::Dwg_TableCell_num_rows_get", _wrap_Dwg_TableCell_num_rows_get},
{"LibreDWGc::Dwg_TableCell_num_cols_set", _wrap_Dwg_TableCell_num_cols_set},
{"LibreDWGc::Dwg_TableCell_num_cols_get", _wrap_Dwg_TableCell_num_cols_get},
{"LibreDWGc::Dwg_TableCell_unknown_set", _wrap_Dwg_TableCell_unknown_set},
{"LibreDWGc::Dwg_TableCell_unknown_get", _wrap_Dwg_TableCell_unknown_get},
{"LibreDWGc::Dwg_TableCell_num_cell_contents_set", _wrap_Dwg_TableCell_num_cell_contents_set},
{"LibreDWGc::Dwg_TableCell_num_cell_contents_get", _wrap_Dwg_TableCell_num_cell_contents_get},
{"LibreDWGc::Dwg_TableCell_cell_contents_set", _wrap_Dwg_TableCell_cell_contents_set},
{"LibreDWGc::Dwg_TableCell_cell_contents_get", _wrap_Dwg_TableCell_cell_contents_get},
{"LibreDWGc::Dwg_TableCell_style_id_set", _wrap_Dwg_TableCell_style_id_set},
{"LibreDWGc::Dwg_TableCell_style_id_get", _wrap_Dwg_TableCell_style_id_get},
{"LibreDWGc::Dwg_TableCell_has_geom_data_set", _wrap_Dwg_TableCell_has_geom_data_set},
{"LibreDWGc::Dwg_TableCell_has_geom_data_get", _wrap_Dwg_TableCell_has_geom_data_get},
{"LibreDWGc::Dwg_TableCell_geom_data_flag_set", _wrap_Dwg_TableCell_geom_data_flag_set},
{"LibreDWGc::Dwg_TableCell_geom_data_flag_get", _wrap_Dwg_TableCell_geom_data_flag_get},
{"LibreDWGc::Dwg_TableCell_width_w_gap_set", _wrap_Dwg_TableCell_width_w_gap_set},
{"LibreDWGc::Dwg_TableCell_width_w_gap_get", _wrap_Dwg_TableCell_width_w_gap_get},
{"LibreDWGc::Dwg_TableCell_height_w_gap_set", _wrap_Dwg_TableCell_height_w_gap_set},
{"LibreDWGc::Dwg_TableCell_height_w_gap_get", _wrap_Dwg_TableCell_height_w_gap_get},
{"LibreDWGc::Dwg_TableCell_tablegeometry_set", _wrap_Dwg_TableCell_tablegeometry_set},
{"LibreDWGc::Dwg_TableCell_tablegeometry_get", _wrap_Dwg_TableCell_tablegeometry_get},
{"LibreDWGc::Dwg_TableCell_num_geometry_set", _wrap_Dwg_TableCell_num_geometry_set},
{"LibreDWGc::Dwg_TableCell_num_geometry_get", _wrap_Dwg_TableCell_num_geometry_get},
{"LibreDWGc::Dwg_TableCell_geometry_set", _wrap_Dwg_TableCell_geometry_set},
{"LibreDWGc::Dwg_TableCell_geometry_get", _wrap_Dwg_TableCell_geometry_get},
{"LibreDWGc::Dwg_TableCell_style_parent_set", _wrap_Dwg_TableCell_style_parent_set},
{"LibreDWGc::Dwg_TableCell_style_parent_get", _wrap_Dwg_TableCell_style_parent_get},
{"LibreDWGc::Dwg_TableCell_row_parent_set", _wrap_Dwg_TableCell_row_parent_set},
{"LibreDWGc::Dwg_TableCell_row_parent_get", _wrap_Dwg_TableCell_row_parent_get},
{"LibreDWGc::new_Dwg_TableCell", _wrap_new_Dwg_TableCell},
{"LibreDWGc::delete_Dwg_TableCell", _wrap_delete_Dwg_TableCell},
{"LibreDWGc::Dwg_GridFormat_parent_set", _wrap_Dwg_GridFormat_parent_set},
{"LibreDWGc::Dwg_GridFormat_parent_get", _wrap_Dwg_GridFormat_parent_get},
{"LibreDWGc::Dwg_GridFormat_index_mask_set", _wrap_Dwg_GridFormat_index_mask_set},
{"LibreDWGc::Dwg_GridFormat_index_mask_get", _wrap_Dwg_GridFormat_index_mask_get},
{"LibreDWGc::Dwg_GridFormat_border_overrides_set", _wrap_Dwg_GridFormat_border_overrides_set},
{"LibreDWGc::Dwg_GridFormat_border_overrides_get", _wrap_Dwg_GridFormat_border_overrides_get},
{"LibreDWGc::Dwg_GridFormat_border_type_set", _wrap_Dwg_GridFormat_border_type_set},
{"LibreDWGc::Dwg_GridFormat_border_type_get", _wrap_Dwg_GridFormat_border_type_get},
{"LibreDWGc::Dwg_GridFormat_color_set", _wrap_Dwg_GridFormat_color_set},
{"LibreDWGc::Dwg_GridFormat_color_get", _wrap_Dwg_GridFormat_color_get},
{"LibreDWGc::Dwg_GridFormat_linewt_set", _wrap_Dwg_GridFormat_linewt_set},
{"LibreDWGc::Dwg_GridFormat_linewt_get", _wrap_Dwg_GridFormat_linewt_get},
{"LibreDWGc::Dwg_GridFormat_ltype_set", _wrap_Dwg_GridFormat_ltype_set},
{"LibreDWGc::Dwg_GridFormat_ltype_get", _wrap_Dwg_GridFormat_ltype_get},
{"LibreDWGc::Dwg_GridFormat_visible_set", _wrap_Dwg_GridFormat_visible_set},
{"LibreDWGc::Dwg_GridFormat_visible_get", _wrap_Dwg_GridFormat_visible_get},
{"LibreDWGc::Dwg_GridFormat_double_line_spacing_set", _wrap_Dwg_GridFormat_double_line_spacing_set},
{"LibreDWGc::Dwg_GridFormat_double_line_spacing_get", _wrap_Dwg_GridFormat_double_line_spacing_get},
{"LibreDWGc::new_Dwg_GridFormat", _wrap_new_Dwg_GridFormat},
{"LibreDWGc::delete_Dwg_GridFormat", _wrap_delete_Dwg_GridFormat},
{"LibreDWGc::Dwg_CellStyle_type_set", _wrap_Dwg_CellStyle_type_set},
{"LibreDWGc::Dwg_CellStyle_type_get", _wrap_Dwg_CellStyle_type_get},
{"LibreDWGc::Dwg_CellStyle_data_flags_set", _wrap_Dwg_CellStyle_data_flags_set},
{"LibreDWGc::Dwg_CellStyle_data_flags_get", _wrap_Dwg_CellStyle_data_flags_get},
{"LibreDWGc::Dwg_CellStyle_property_override_flags_set", _wrap_Dwg_CellStyle_property_override_flags_set},
{"LibreDWGc::Dwg_CellStyle_property_override_flags_get", _wrap_Dwg_CellStyle_property_override_flags_get},
{"LibreDWGc::Dwg_CellStyle_merge_flags_set", _wrap_Dwg_CellStyle_merge_flags_set},
{"LibreDWGc::Dwg_CellStyle_merge_flags_get", _wrap_Dwg_CellStyle_merge_flags_get},
{"LibreDWGc::Dwg_CellStyle_bg_color_set", _wrap_Dwg_CellStyle_bg_color_set},
{"LibreDWGc::Dwg_CellStyle_bg_color_get", _wrap_Dwg_CellStyle_bg_color_get},
{"LibreDWGc::Dwg_CellStyle_content_layout_set", _wrap_Dwg_CellStyle_content_layout_set},
{"LibreDWGc::Dwg_CellStyle_content_layout_get", _wrap_Dwg_CellStyle_content_layout_get},
{"LibreDWGc::Dwg_CellStyle_content_format_set", _wrap_Dwg_CellStyle_content_format_set},
{"LibreDWGc::Dwg_CellStyle_content_format_get", _wrap_Dwg_CellStyle_content_format_get},
{"LibreDWGc::Dwg_CellStyle_margin_override_flags_set", _wrap_Dwg_CellStyle_margin_override_flags_set},
{"LibreDWGc::Dwg_CellStyle_margin_override_flags_get", _wrap_Dwg_CellStyle_margin_override_flags_get},
{"LibreDWGc::Dwg_CellStyle_vert_margin_set", _wrap_Dwg_CellStyle_vert_margin_set},
{"LibreDWGc::Dwg_CellStyle_vert_margin_get", _wrap_Dwg_CellStyle_vert_margin_get},
{"LibreDWGc::Dwg_CellStyle_horiz_margin_set", _wrap_Dwg_CellStyle_horiz_margin_set},
{"LibreDWGc::Dwg_CellStyle_horiz_margin_get", _wrap_Dwg_CellStyle_horiz_margin_get},
{"LibreDWGc::Dwg_CellStyle_bottom_margin_set", _wrap_Dwg_CellStyle_bottom_margin_set},
{"LibreDWGc::Dwg_CellStyle_bottom_margin_get", _wrap_Dwg_CellStyle_bottom_margin_get},
{"LibreDWGc::Dwg_CellStyle_right_margin_set", _wrap_Dwg_CellStyle_right_margin_set},
{"LibreDWGc::Dwg_CellStyle_right_margin_get", _wrap_Dwg_CellStyle_right_margin_get},
{"LibreDWGc::Dwg_CellStyle_margin_horiz_spacing_set", _wrap_Dwg_CellStyle_margin_horiz_spacing_set},
{"LibreDWGc::Dwg_CellStyle_margin_horiz_spacing_get", _wrap_Dwg_CellStyle_margin_horiz_spacing_get},
{"LibreDWGc::Dwg_CellStyle_margin_vert_spacing_set", _wrap_Dwg_CellStyle_margin_vert_spacing_set},
{"LibreDWGc::Dwg_CellStyle_margin_vert_spacing_get", _wrap_Dwg_CellStyle_margin_vert_spacing_get},
{"LibreDWGc::Dwg_CellStyle_num_borders_set", _wrap_Dwg_CellStyle_num_borders_set},
{"LibreDWGc::Dwg_CellStyle_num_borders_get", _wrap_Dwg_CellStyle_num_borders_get},
{"LibreDWGc::Dwg_CellStyle_borders_set", _wrap_Dwg_CellStyle_borders_set},
{"LibreDWGc::Dwg_CellStyle_borders_get", _wrap_Dwg_CellStyle_borders_get},
{"LibreDWGc::Dwg_CellStyle_tablerow_parent_set", _wrap_Dwg_CellStyle_tablerow_parent_set},
{"LibreDWGc::Dwg_CellStyle_tablerow_parent_get", _wrap_Dwg_CellStyle_tablerow_parent_get},
{"LibreDWGc::Dwg_CellStyle_tabledatacolumn_parent_set", _wrap_Dwg_CellStyle_tabledatacolumn_parent_set},
{"LibreDWGc::Dwg_CellStyle_tabledatacolumn_parent_get", _wrap_Dwg_CellStyle_tabledatacolumn_parent_get},
{"LibreDWGc::new_Dwg_CellStyle", _wrap_new_Dwg_CellStyle},
{"LibreDWGc::delete_Dwg_CellStyle", _wrap_delete_Dwg_CellStyle},
{"LibreDWGc::Dwg_TableRow_parent_set", _wrap_Dwg_TableRow_parent_set},
{"LibreDWGc::Dwg_TableRow_parent_get", _wrap_Dwg_TableRow_parent_get},
{"LibreDWGc::Dwg_TableRow_num_cells_set", _wrap_Dwg_TableRow_num_cells_set},
{"LibreDWGc::Dwg_TableRow_num_cells_get", _wrap_Dwg_TableRow_num_cells_get},
{"LibreDWGc::Dwg_TableRow_cells_set", _wrap_Dwg_TableRow_cells_set},
{"LibreDWGc::Dwg_TableRow_cells_get", _wrap_Dwg_TableRow_cells_get},
{"LibreDWGc::Dwg_TableRow_custom_data_set", _wrap_Dwg_TableRow_custom_data_set},
{"LibreDWGc::Dwg_TableRow_custom_data_get", _wrap_Dwg_TableRow_custom_data_get},
{"LibreDWGc::Dwg_TableRow_num_customdata_items_set", _wrap_Dwg_TableRow_num_customdata_items_set},
{"LibreDWGc::Dwg_TableRow_num_customdata_items_get", _wrap_Dwg_TableRow_num_customdata_items_get},
{"LibreDWGc::Dwg_TableRow_customdata_items_set", _wrap_Dwg_TableRow_customdata_items_set},
{"LibreDWGc::Dwg_TableRow_customdata_items_get", _wrap_Dwg_TableRow_customdata_items_get},
{"LibreDWGc::Dwg_TableRow_cellstyle_set", _wrap_Dwg_TableRow_cellstyle_set},
{"LibreDWGc::Dwg_TableRow_cellstyle_get", _wrap_Dwg_TableRow_cellstyle_get},
{"LibreDWGc::Dwg_TableRow_style_id_set", _wrap_Dwg_TableRow_style_id_set},
{"LibreDWGc::Dwg_TableRow_style_id_get", _wrap_Dwg_TableRow_style_id_get},
{"LibreDWGc::Dwg_TableRow_height_set", _wrap_Dwg_TableRow_height_set},
{"LibreDWGc::Dwg_TableRow_height_get", _wrap_Dwg_TableRow_height_get},
{"LibreDWGc::new_Dwg_TableRow", _wrap_new_Dwg_TableRow},
{"LibreDWGc::delete_Dwg_TableRow", _wrap_delete_Dwg_TableRow},
{"LibreDWGc::Dwg_TableDataColumn_parent_set", _wrap_Dwg_TableDataColumn_parent_set},
{"LibreDWGc::Dwg_TableDataColumn_parent_get", _wrap_Dwg_TableDataColumn_parent_get},
{"LibreDWGc::Dwg_TableDataColumn_name_set", _wrap_Dwg_TableDataColumn_name_set},
{"LibreDWGc::Dwg_TableDataColumn_name_get", _wrap_Dwg_TableDataColumn_name_get},
{"LibreDWGc::Dwg_TableDataColumn_custom_data_set", _wrap_Dwg_TableDataColumn_custom_data_set},
{"LibreDWGc::Dwg_TableDataColumn_custom_data_get", _wrap_Dwg_TableDataColumn_custom_data_get},
{"LibreDWGc::Dwg_TableDataColumn_cellstyle_set", _wrap_Dwg_TableDataColumn_cellstyle_set},
{"LibreDWGc::Dwg_TableDataColumn_cellstyle_get", _wrap_Dwg_TableDataColumn_cellstyle_get},
{"LibreDWGc::Dwg_TableDataColumn_cellstyle_id_set", _wrap_Dwg_TableDataColumn_cellstyle_id_set},
{"LibreDWGc::Dwg_TableDataColumn_cellstyle_id_get", _wrap_Dwg_TableDataColumn_cellstyle_id_get},
{"LibreDWGc::Dwg_TableDataColumn_width_set", _wrap_Dwg_TableDataColumn_width_set},
{"LibreDWGc::Dwg_TableDataColumn_width_get", _wrap_Dwg_TableDataColumn_width_get},
{"LibreDWGc::new_Dwg_TableDataColumn", _wrap_new_Dwg_TableDataColumn},
{"LibreDWGc::delete_Dwg_TableDataColumn", _wrap_delete_Dwg_TableDataColumn},
{"LibreDWGc::Dwg_LinkedTableData_num_cols_set", _wrap_Dwg_LinkedTableData_num_cols_set},
{"LibreDWGc::Dwg_LinkedTableData_num_cols_get", _wrap_Dwg_LinkedTableData_num_cols_get},
{"LibreDWGc::Dwg_LinkedTableData_cols_set", _wrap_Dwg_LinkedTableData_cols_set},
{"LibreDWGc::Dwg_LinkedTableData_cols_get", _wrap_Dwg_LinkedTableData_cols_get},
{"LibreDWGc::Dwg_LinkedTableData_num_rows_set", _wrap_Dwg_LinkedTableData_num_rows_set},
{"LibreDWGc::Dwg_LinkedTableData_num_rows_get", _wrap_Dwg_LinkedTableData_num_rows_get},
{"LibreDWGc::Dwg_LinkedTableData_rows_set", _wrap_Dwg_LinkedTableData_rows_set},
{"LibreDWGc::Dwg_LinkedTableData_rows_get", _wrap_Dwg_LinkedTableData_rows_get},
{"LibreDWGc::Dwg_LinkedTableData_num_field_refs_set", _wrap_Dwg_LinkedTableData_num_field_refs_set},
{"LibreDWGc::Dwg_LinkedTableData_num_field_refs_get", _wrap_Dwg_LinkedTableData_num_field_refs_get},
{"LibreDWGc::Dwg_LinkedTableData_field_refs_set", _wrap_Dwg_LinkedTableData_field_refs_set},
{"LibreDWGc::Dwg_LinkedTableData_field_refs_get", _wrap_Dwg_LinkedTableData_field_refs_get},
{"LibreDWGc::new_Dwg_LinkedTableData", _wrap_new_Dwg_LinkedTableData},
{"LibreDWGc::delete_Dwg_LinkedTableData", _wrap_delete_Dwg_LinkedTableData},
{"LibreDWGc::Dwg_FormattedTableMerged_parent_set", _wrap_Dwg_FormattedTableMerged_parent_set},
{"LibreDWGc::Dwg_FormattedTableMerged_parent_get", _wrap_Dwg_FormattedTableMerged_parent_get},
{"LibreDWGc::Dwg_FormattedTableMerged_top_row_set", _wrap_Dwg_FormattedTableMerged_top_row_set},
{"LibreDWGc::Dwg_FormattedTableMerged_top_row_get", _wrap_Dwg_FormattedTableMerged_top_row_get},
{"LibreDWGc::Dwg_FormattedTableMerged_left_col_set", _wrap_Dwg_FormattedTableMerged_left_col_set},
{"LibreDWGc::Dwg_FormattedTableMerged_left_col_get", _wrap_Dwg_FormattedTableMerged_left_col_get},
{"LibreDWGc::Dwg_FormattedTableMerged_bottom_row_set", _wrap_Dwg_FormattedTableMerged_bottom_row_set},
{"LibreDWGc::Dwg_FormattedTableMerged_bottom_row_get", _wrap_Dwg_FormattedTableMerged_bottom_row_get},
{"LibreDWGc::Dwg_FormattedTableMerged_right_col_set", _wrap_Dwg_FormattedTableMerged_right_col_set},
{"LibreDWGc::Dwg_FormattedTableMerged_right_col_get", _wrap_Dwg_FormattedTableMerged_right_col_get},
{"LibreDWGc::new_Dwg_FormattedTableMerged", _wrap_new_Dwg_FormattedTableMerged},
{"LibreDWGc::delete_Dwg_FormattedTableMerged", _wrap_delete_Dwg_FormattedTableMerged},
{"LibreDWGc::Dwg_FormattedTableData_parent_set", _wrap_Dwg_FormattedTableData_parent_set},
{"LibreDWGc::Dwg_FormattedTableData_parent_get", _wrap_Dwg_FormattedTableData_parent_get},
{"LibreDWGc::Dwg_FormattedTableData_cellstyle_set", _wrap_Dwg_FormattedTableData_cellstyle_set},
{"LibreDWGc::Dwg_FormattedTableData_cellstyle_get", _wrap_Dwg_FormattedTableData_cellstyle_get},
{"LibreDWGc::Dwg_FormattedTableData_num_merged_cells_set", _wrap_Dwg_FormattedTableData_num_merged_cells_set},
{"LibreDWGc::Dwg_FormattedTableData_num_merged_cells_get", _wrap_Dwg_FormattedTableData_num_merged_cells_get},
{"LibreDWGc::Dwg_FormattedTableData_merged_cells_set", _wrap_Dwg_FormattedTableData_merged_cells_set},
{"LibreDWGc::Dwg_FormattedTableData_merged_cells_get", _wrap_Dwg_FormattedTableData_merged_cells_get},
{"LibreDWGc::new_Dwg_FormattedTableData", _wrap_new_Dwg_FormattedTableData},
{"LibreDWGc::delete_Dwg_FormattedTableData", _wrap_delete_Dwg_FormattedTableData},
{"LibreDWGc::Dwg_Object_TABLECONTENT_parent_set", _wrap_Dwg_Object_TABLECONTENT_parent_set},
{"LibreDWGc::Dwg_Object_TABLECONTENT_parent_get", _wrap_Dwg_Object_TABLECONTENT_parent_get},
{"LibreDWGc::Dwg_Object_TABLECONTENT_ldata_set", _wrap_Dwg_Object_TABLECONTENT_ldata_set},
{"LibreDWGc::Dwg_Object_TABLECONTENT_ldata_get", _wrap_Dwg_Object_TABLECONTENT_ldata_get},
{"LibreDWGc::Dwg_Object_TABLECONTENT_tdata_set", _wrap_Dwg_Object_TABLECONTENT_tdata_set},
{"LibreDWGc::Dwg_Object_TABLECONTENT_tdata_get", _wrap_Dwg_Object_TABLECONTENT_tdata_get},
{"LibreDWGc::Dwg_Object_TABLECONTENT_fdata_set", _wrap_Dwg_Object_TABLECONTENT_fdata_set},
{"LibreDWGc::Dwg_Object_TABLECONTENT_fdata_get", _wrap_Dwg_Object_TABLECONTENT_fdata_get},
{"LibreDWGc::Dwg_Object_TABLECONTENT_tablestyle_set", _wrap_Dwg_Object_TABLECONTENT_tablestyle_set},
{"LibreDWGc::Dwg_Object_TABLECONTENT_tablestyle_get", _wrap_Dwg_Object_TABLECONTENT_tablestyle_get},
{"LibreDWGc::new_Dwg_Object_TABLECONTENT", _wrap_new_Dwg_Object_TABLECONTENT},
{"LibreDWGc::delete_Dwg_Object_TABLECONTENT", _wrap_delete_Dwg_Object_TABLECONTENT},
{"LibreDWGc::Dwg_Entity_TABLE_parent_set", _wrap_Dwg_Entity_TABLE_parent_set},
{"LibreDWGc::Dwg_Entity_TABLE_parent_get", _wrap_Dwg_Entity_TABLE_parent_get},
{"LibreDWGc::Dwg_Entity_TABLE_ldata_set", _wrap_Dwg_Entity_TABLE_ldata_set},
{"LibreDWGc::Dwg_Entity_TABLE_ldata_get", _wrap_Dwg_Entity_TABLE_ldata_get},
{"LibreDWGc::Dwg_Entity_TABLE_tdata_set", _wrap_Dwg_Entity_TABLE_tdata_set},
{"LibreDWGc::Dwg_Entity_TABLE_tdata_get", _wrap_Dwg_Entity_TABLE_tdata_get},
{"LibreDWGc::Dwg_Entity_TABLE_fdata_set", _wrap_Dwg_Entity_TABLE_fdata_set},
{"LibreDWGc::Dwg_Entity_TABLE_fdata_get", _wrap_Dwg_Entity_TABLE_fdata_get},
{"LibreDWGc::Dwg_Entity_TABLE_tablestyle_set", _wrap_Dwg_Entity_TABLE_tablestyle_set},
{"LibreDWGc::Dwg_Entity_TABLE_tablestyle_get", _wrap_Dwg_Entity_TABLE_tablestyle_get},
{"LibreDWGc::Dwg_Entity_TABLE_unknown_rc_set", _wrap_Dwg_Entity_TABLE_unknown_rc_set},
{"LibreDWGc::Dwg_Entity_TABLE_unknown_rc_get", _wrap_Dwg_Entity_TABLE_unknown_rc_get},
{"LibreDWGc::Dwg_Entity_TABLE_unknown_h_set", _wrap_Dwg_Entity_TABLE_unknown_h_set},
{"LibreDWGc::Dwg_Entity_TABLE_unknown_h_get", _wrap_Dwg_Entity_TABLE_unknown_h_get},
{"LibreDWGc::Dwg_Entity_TABLE_unknown_bl_set", _wrap_Dwg_Entity_TABLE_unknown_bl_set},
{"LibreDWGc::Dwg_Entity_TABLE_unknown_bl_get", _wrap_Dwg_Entity_TABLE_unknown_bl_get},
{"LibreDWGc::Dwg_Entity_TABLE_unknown_b_set", _wrap_Dwg_Entity_TABLE_unknown_b_set},
{"LibreDWGc::Dwg_Entity_TABLE_unknown_b_get", _wrap_Dwg_Entity_TABLE_unknown_b_get},
{"LibreDWGc::Dwg_Entity_TABLE_unknown_bl1_set", _wrap_Dwg_Entity_TABLE_unknown_bl1_set},
{"LibreDWGc::Dwg_Entity_TABLE_unknown_bl1_get", _wrap_Dwg_Entity_TABLE_unknown_bl1_get},
{"LibreDWGc::Dwg_Entity_TABLE_ins_pt_set", _wrap_Dwg_Entity_TABLE_ins_pt_set},
{"LibreDWGc::Dwg_Entity_TABLE_ins_pt_get", _wrap_Dwg_Entity_TABLE_ins_pt_get},
{"LibreDWGc::Dwg_Entity_TABLE_scale_set", _wrap_Dwg_Entity_TABLE_scale_set},
{"LibreDWGc::Dwg_Entity_TABLE_scale_get", _wrap_Dwg_Entity_TABLE_scale_get},
{"LibreDWGc::Dwg_Entity_TABLE_scale_flag_set", _wrap_Dwg_Entity_TABLE_scale_flag_set},
{"LibreDWGc::Dwg_Entity_TABLE_scale_flag_get", _wrap_Dwg_Entity_TABLE_scale_flag_get},
{"LibreDWGc::Dwg_Entity_TABLE_rotation_set", _wrap_Dwg_Entity_TABLE_rotation_set},
{"LibreDWGc::Dwg_Entity_TABLE_rotation_get", _wrap_Dwg_Entity_TABLE_rotation_get},
{"LibreDWGc::Dwg_Entity_TABLE_extrusion_set", _wrap_Dwg_Entity_TABLE_extrusion_set},
{"LibreDWGc::Dwg_Entity_TABLE_extrusion_get", _wrap_Dwg_Entity_TABLE_extrusion_get},
{"LibreDWGc::Dwg_Entity_TABLE_has_attribs_set", _wrap_Dwg_Entity_TABLE_has_attribs_set},
{"LibreDWGc::Dwg_Entity_TABLE_has_attribs_get", _wrap_Dwg_Entity_TABLE_has_attribs_get},
{"LibreDWGc::Dwg_Entity_TABLE_num_owned_set", _wrap_Dwg_Entity_TABLE_num_owned_set},
{"LibreDWGc::Dwg_Entity_TABLE_num_owned_get", _wrap_Dwg_Entity_TABLE_num_owned_get},
{"LibreDWGc::Dwg_Entity_TABLE_flag_for_table_value_set", _wrap_Dwg_Entity_TABLE_flag_for_table_value_set},
{"LibreDWGc::Dwg_Entity_TABLE_flag_for_table_value_get", _wrap_Dwg_Entity_TABLE_flag_for_table_value_get},
{"LibreDWGc::Dwg_Entity_TABLE_horiz_direction_set", _wrap_Dwg_Entity_TABLE_horiz_direction_set},
{"LibreDWGc::Dwg_Entity_TABLE_horiz_direction_get", _wrap_Dwg_Entity_TABLE_horiz_direction_get},
{"LibreDWGc::Dwg_Entity_TABLE_num_cols_set", _wrap_Dwg_Entity_TABLE_num_cols_set},
{"LibreDWGc::Dwg_Entity_TABLE_num_cols_get", _wrap_Dwg_Entity_TABLE_num_cols_get},
{"LibreDWGc::Dwg_Entity_TABLE_num_rows_set", _wrap_Dwg_Entity_TABLE_num_rows_set},
{"LibreDWGc::Dwg_Entity_TABLE_num_rows_get", _wrap_Dwg_Entity_TABLE_num_rows_get},
{"LibreDWGc::Dwg_Entity_TABLE_num_cells_set", _wrap_Dwg_Entity_TABLE_num_cells_set},
{"LibreDWGc::Dwg_Entity_TABLE_num_cells_get", _wrap_Dwg_Entity_TABLE_num_cells_get},
{"LibreDWGc::Dwg_Entity_TABLE_col_widths_set", _wrap_Dwg_Entity_TABLE_col_widths_set},
{"LibreDWGc::Dwg_Entity_TABLE_col_widths_get", _wrap_Dwg_Entity_TABLE_col_widths_get},
{"LibreDWGc::Dwg_Entity_TABLE_row_heights_set", _wrap_Dwg_Entity_TABLE_row_heights_set},
{"LibreDWGc::Dwg_Entity_TABLE_row_heights_get", _wrap_Dwg_Entity_TABLE_row_heights_get},
{"LibreDWGc::Dwg_Entity_TABLE_cells_set", _wrap_Dwg_Entity_TABLE_cells_set},
{"LibreDWGc::Dwg_Entity_TABLE_cells_get", _wrap_Dwg_Entity_TABLE_cells_get},
{"LibreDWGc::Dwg_Entity_TABLE_has_table_overrides_set", _wrap_Dwg_Entity_TABLE_has_table_overrides_set},
{"LibreDWGc::Dwg_Entity_TABLE_has_table_overrides_get", _wrap_Dwg_Entity_TABLE_has_table_overrides_get},
{"LibreDWGc::Dwg_Entity_TABLE_table_flag_override_set", _wrap_Dwg_Entity_TABLE_table_flag_override_set},
{"LibreDWGc::Dwg_Entity_TABLE_table_flag_override_get", _wrap_Dwg_Entity_TABLE_table_flag_override_get},
{"LibreDWGc::Dwg_Entity_TABLE_title_suppressed_set", _wrap_Dwg_Entity_TABLE_title_suppressed_set},
{"LibreDWGc::Dwg_Entity_TABLE_title_suppressed_get", _wrap_Dwg_Entity_TABLE_title_suppressed_get},
{"LibreDWGc::Dwg_Entity_TABLE_header_suppressed_set", _wrap_Dwg_Entity_TABLE_header_suppressed_set},
{"LibreDWGc::Dwg_Entity_TABLE_header_suppressed_get", _wrap_Dwg_Entity_TABLE_header_suppressed_get},
{"LibreDWGc::Dwg_Entity_TABLE_flow_direction_set", _wrap_Dwg_Entity_TABLE_flow_direction_set},
{"LibreDWGc::Dwg_Entity_TABLE_flow_direction_get", _wrap_Dwg_Entity_TABLE_flow_direction_get},
{"LibreDWGc::Dwg_Entity_TABLE_horiz_cell_margin_set", _wrap_Dwg_Entity_TABLE_horiz_cell_margin_set},
{"LibreDWGc::Dwg_Entity_TABLE_horiz_cell_margin_get", _wrap_Dwg_Entity_TABLE_horiz_cell_margin_get},
{"LibreDWGc::Dwg_Entity_TABLE_vert_cell_margin_set", _wrap_Dwg_Entity_TABLE_vert_cell_margin_set},
{"LibreDWGc::Dwg_Entity_TABLE_vert_cell_margin_get", _wrap_Dwg_Entity_TABLE_vert_cell_margin_get},
{"LibreDWGc::Dwg_Entity_TABLE_title_row_color_set", _wrap_Dwg_Entity_TABLE_title_row_color_set},
{"LibreDWGc::Dwg_Entity_TABLE_title_row_color_get", _wrap_Dwg_Entity_TABLE_title_row_color_get},
{"LibreDWGc::Dwg_Entity_TABLE_header_row_color_set", _wrap_Dwg_Entity_TABLE_header_row_color_set},
{"LibreDWGc::Dwg_Entity_TABLE_header_row_color_get", _wrap_Dwg_Entity_TABLE_header_row_color_get},
{"LibreDWGc::Dwg_Entity_TABLE_data_row_color_set", _wrap_Dwg_Entity_TABLE_data_row_color_set},
{"LibreDWGc::Dwg_Entity_TABLE_data_row_color_get", _wrap_Dwg_Entity_TABLE_data_row_color_get},
{"LibreDWGc::Dwg_Entity_TABLE_title_row_fill_none_set", _wrap_Dwg_Entity_TABLE_title_row_fill_none_set},
{"LibreDWGc::Dwg_Entity_TABLE_title_row_fill_none_get", _wrap_Dwg_Entity_TABLE_title_row_fill_none_get},
{"LibreDWGc::Dwg_Entity_TABLE_header_row_fill_none_set", _wrap_Dwg_Entity_TABLE_header_row_fill_none_set},
{"LibreDWGc::Dwg_Entity_TABLE_header_row_fill_none_get", _wrap_Dwg_Entity_TABLE_header_row_fill_none_get},
{"LibreDWGc::Dwg_Entity_TABLE_data_row_fill_none_set", _wrap_Dwg_Entity_TABLE_data_row_fill_none_set},
{"LibreDWGc::Dwg_Entity_TABLE_data_row_fill_none_get", _wrap_Dwg_Entity_TABLE_data_row_fill_none_get},
{"LibreDWGc::Dwg_Entity_TABLE_title_row_fill_color_set", _wrap_Dwg_Entity_TABLE_title_row_fill_color_set},
{"LibreDWGc::Dwg_Entity_TABLE_title_row_fill_color_get", _wrap_Dwg_Entity_TABLE_title_row_fill_color_get},
{"LibreDWGc::Dwg_Entity_TABLE_header_row_fill_color_set", _wrap_Dwg_Entity_TABLE_header_row_fill_color_set},
{"LibreDWGc::Dwg_Entity_TABLE_header_row_fill_color_get", _wrap_Dwg_Entity_TABLE_header_row_fill_color_get},
{"LibreDWGc::Dwg_Entity_TABLE_data_row_fill_color_set", _wrap_Dwg_Entity_TABLE_data_row_fill_color_set},
{"LibreDWGc::Dwg_Entity_TABLE_data_row_fill_color_get", _wrap_Dwg_Entity_TABLE_data_row_fill_color_get},
{"LibreDWGc::Dwg_Entity_TABLE_title_row_alignment_set", _wrap_Dwg_Entity_TABLE_title_row_alignment_set},
{"LibreDWGc::Dwg_Entity_TABLE_title_row_alignment_get", _wrap_Dwg_Entity_TABLE_title_row_alignment_get},
{"LibreDWGc::Dwg_Entity_TABLE_header_row_alignment_set", _wrap_Dwg_Entity_TABLE_header_row_alignment_set},
{"LibreDWGc::Dwg_Entity_TABLE_header_row_alignment_get", _wrap_Dwg_Entity_TABLE_header_row_alignment_get},
{"LibreDWGc::Dwg_Entity_TABLE_data_row_alignment_set", _wrap_Dwg_Entity_TABLE_data_row_alignment_set},
{"LibreDWGc::Dwg_Entity_TABLE_data_row_alignment_get", _wrap_Dwg_Entity_TABLE_data_row_alignment_get},
{"LibreDWGc::Dwg_Entity_TABLE_title_text_style_set", _wrap_Dwg_Entity_TABLE_title_text_style_set},
{"LibreDWGc::Dwg_Entity_TABLE_title_text_style_get", _wrap_Dwg_Entity_TABLE_title_text_style_get},
{"LibreDWGc::Dwg_Entity_TABLE_header_text_style_set", _wrap_Dwg_Entity_TABLE_header_text_style_set},
{"LibreDWGc::Dwg_Entity_TABLE_header_text_style_get", _wrap_Dwg_Entity_TABLE_header_text_style_get},
{"LibreDWGc::Dwg_Entity_TABLE_data_text_style_set", _wrap_Dwg_Entity_TABLE_data_text_style_set},
{"LibreDWGc::Dwg_Entity_TABLE_data_text_style_get", _wrap_Dwg_Entity_TABLE_data_text_style_get},
{"LibreDWGc::Dwg_Entity_TABLE_title_row_height_set", _wrap_Dwg_Entity_TABLE_title_row_height_set},
{"LibreDWGc::Dwg_Entity_TABLE_title_row_height_get", _wrap_Dwg_Entity_TABLE_title_row_height_get},
{"LibreDWGc::Dwg_Entity_TABLE_header_row_height_set", _wrap_Dwg_Entity_TABLE_header_row_height_set},
{"LibreDWGc::Dwg_Entity_TABLE_header_row_height_get", _wrap_Dwg_Entity_TABLE_header_row_height_get},
{"LibreDWGc::Dwg_Entity_TABLE_data_row_height_set", _wrap_Dwg_Entity_TABLE_data_row_height_set},
{"LibreDWGc::Dwg_Entity_TABLE_data_row_height_get", _wrap_Dwg_Entity_TABLE_data_row_height_get},
{"LibreDWGc::Dwg_Entity_TABLE_has_border_color_overrides_set", _wrap_Dwg_Entity_TABLE_has_border_color_overrides_set},
{"LibreDWGc::Dwg_Entity_TABLE_has_border_color_overrides_get", _wrap_Dwg_Entity_TABLE_has_border_color_overrides_get},
{"LibreDWGc::Dwg_Entity_TABLE_border_color_overrides_flag_set", _wrap_Dwg_Entity_TABLE_border_color_overrides_flag_set},
{"LibreDWGc::Dwg_Entity_TABLE_border_color_overrides_flag_get", _wrap_Dwg_Entity_TABLE_border_color_overrides_flag_get},
{"LibreDWGc::Dwg_Entity_TABLE_title_horiz_top_color_set", _wrap_Dwg_Entity_TABLE_title_horiz_top_color_set},
{"LibreDWGc::Dwg_Entity_TABLE_title_horiz_top_color_get", _wrap_Dwg_Entity_TABLE_title_horiz_top_color_get},
{"LibreDWGc::Dwg_Entity_TABLE_title_horiz_ins_color_set", _wrap_Dwg_Entity_TABLE_title_horiz_ins_color_set},
{"LibreDWGc::Dwg_Entity_TABLE_title_horiz_ins_color_get", _wrap_Dwg_Entity_TABLE_title_horiz_ins_color_get},
{"LibreDWGc::Dwg_Entity_TABLE_title_horiz_bottom_color_set", _wrap_Dwg_Entity_TABLE_title_horiz_bottom_color_set},
{"LibreDWGc::Dwg_Entity_TABLE_title_horiz_bottom_color_get", _wrap_Dwg_Entity_TABLE_title_horiz_bottom_color_get},
{"LibreDWGc::Dwg_Entity_TABLE_title_vert_left_color_set", _wrap_Dwg_Entity_TABLE_title_vert_left_color_set},
{"LibreDWGc::Dwg_Entity_TABLE_title_vert_left_color_get", _wrap_Dwg_Entity_TABLE_title_vert_left_color_get},
{"LibreDWGc::Dwg_Entity_TABLE_title_vert_ins_color_set", _wrap_Dwg_Entity_TABLE_title_vert_ins_color_set},
{"LibreDWGc::Dwg_Entity_TABLE_title_vert_ins_color_get", _wrap_Dwg_Entity_TABLE_title_vert_ins_color_get},
{"LibreDWGc::Dwg_Entity_TABLE_title_vert_right_color_set", _wrap_Dwg_Entity_TABLE_title_vert_right_color_set},
{"LibreDWGc::Dwg_Entity_TABLE_title_vert_right_color_get", _wrap_Dwg_Entity_TABLE_title_vert_right_color_get},
{"LibreDWGc::Dwg_Entity_TABLE_header_horiz_top_color_set", _wrap_Dwg_Entity_TABLE_header_horiz_top_color_set},
{"LibreDWGc::Dwg_Entity_TABLE_header_horiz_top_color_get", _wrap_Dwg_Entity_TABLE_header_horiz_top_color_get},
{"LibreDWGc::Dwg_Entity_TABLE_header_horiz_ins_color_set", _wrap_Dwg_Entity_TABLE_header_horiz_ins_color_set},
{"LibreDWGc::Dwg_Entity_TABLE_header_horiz_ins_color_get", _wrap_Dwg_Entity_TABLE_header_horiz_ins_color_get},
{"LibreDWGc::Dwg_Entity_TABLE_header_horiz_bottom_color_set", _wrap_Dwg_Entity_TABLE_header_horiz_bottom_color_set},
{"LibreDWGc::Dwg_Entity_TABLE_header_horiz_bottom_color_get", _wrap_Dwg_Entity_TABLE_header_horiz_bottom_color_get},
{"LibreDWGc::Dwg_Entity_TABLE_header_vert_left_color_set", _wrap_Dwg_Entity_TABLE_header_vert_left_color_set},
{"LibreDWGc::Dwg_Entity_TABLE_header_vert_left_color_get", _wrap_Dwg_Entity_TABLE_header_vert_left_color_get},
{"LibreDWGc::Dwg_Entity_TABLE_header_vert_ins_color_set", _wrap_Dwg_Entity_TABLE_header_vert_ins_color_set},
{"LibreDWGc::Dwg_Entity_TABLE_header_vert_ins_color_get", _wrap_Dwg_Entity_TABLE_header_vert_ins_color_get},
{"LibreDWGc::Dwg_Entity_TABLE_header_vert_right_color_set", _wrap_Dwg_Entity_TABLE_header_vert_right_color_set},
{"LibreDWGc::Dwg_Entity_TABLE_header_vert_right_color_get", _wrap_Dwg_Entity_TABLE_header_vert_right_color_get},
{"LibreDWGc::Dwg_Entity_TABLE_data_horiz_top_color_set", _wrap_Dwg_Entity_TABLE_data_horiz_top_color_set},
{"LibreDWGc::Dwg_Entity_TABLE_data_horiz_top_color_get", _wrap_Dwg_Entity_TABLE_data_horiz_top_color_get},
{"LibreDWGc::Dwg_Entity_TABLE_data_horiz_ins_color_set", _wrap_Dwg_Entity_TABLE_data_horiz_ins_color_set},
{"LibreDWGc::Dwg_Entity_TABLE_data_horiz_ins_color_get", _wrap_Dwg_Entity_TABLE_data_horiz_ins_color_get},
{"LibreDWGc::Dwg_Entity_TABLE_data_horiz_bottom_color_set", _wrap_Dwg_Entity_TABLE_data_horiz_bottom_color_set},
{"LibreDWGc::Dwg_Entity_TABLE_data_horiz_bottom_color_get", _wrap_Dwg_Entity_TABLE_data_horiz_bottom_color_get},
{"LibreDWGc::Dwg_Entity_TABLE_data_vert_left_color_set", _wrap_Dwg_Entity_TABLE_data_vert_left_color_set},
{"LibreDWGc::Dwg_Entity_TABLE_data_vert_left_color_get", _wrap_Dwg_Entity_TABLE_data_vert_left_color_get},
{"LibreDWGc::Dwg_Entity_TABLE_data_vert_ins_color_set", _wrap_Dwg_Entity_TABLE_data_vert_ins_color_set},
{"LibreDWGc::Dwg_Entity_TABLE_data_vert_ins_color_get", _wrap_Dwg_Entity_TABLE_data_vert_ins_color_get},
{"LibreDWGc::Dwg_Entity_TABLE_data_vert_right_color_set", _wrap_Dwg_Entity_TABLE_data_vert_right_color_set},
{"LibreDWGc::Dwg_Entity_TABLE_data_vert_right_color_get", _wrap_Dwg_Entity_TABLE_data_vert_right_color_get},
{"LibreDWGc::Dwg_Entity_TABLE_has_border_lineweight_overrides_set", _wrap_Dwg_Entity_TABLE_has_border_lineweight_overrides_set},
{"LibreDWGc::Dwg_Entity_TABLE_has_border_lineweight_overrides_get", _wrap_Dwg_Entity_TABLE_has_border_lineweight_overrides_get},
{"LibreDWGc::Dwg_Entity_TABLE_border_lineweight_overrides_flag_set", _wrap_Dwg_Entity_TABLE_border_lineweight_overrides_flag_set},
{"LibreDWGc::Dwg_Entity_TABLE_border_lineweight_overrides_flag_get", _wrap_Dwg_Entity_TABLE_border_lineweight_overrides_flag_get},
{"LibreDWGc::Dwg_Entity_TABLE_title_horiz_top_linewt_set", _wrap_Dwg_Entity_TABLE_title_horiz_top_linewt_set},
{"LibreDWGc::Dwg_Entity_TABLE_title_horiz_top_linewt_get", _wrap_Dwg_Entity_TABLE_title_horiz_top_linewt_get},
{"LibreDWGc::Dwg_Entity_TABLE_title_horiz_ins_linewt_set", _wrap_Dwg_Entity_TABLE_title_horiz_ins_linewt_set},
{"LibreDWGc::Dwg_Entity_TABLE_title_horiz_ins_linewt_get", _wrap_Dwg_Entity_TABLE_title_horiz_ins_linewt_get},
{"LibreDWGc::Dwg_Entity_TABLE_title_horiz_bottom_linewt_set", _wrap_Dwg_Entity_TABLE_title_horiz_bottom_linewt_set},
{"LibreDWGc::Dwg_Entity_TABLE_title_horiz_bottom_linewt_get", _wrap_Dwg_Entity_TABLE_title_horiz_bottom_linewt_get},
{"LibreDWGc::Dwg_Entity_TABLE_title_vert_left_linewt_set", _wrap_Dwg_Entity_TABLE_title_vert_left_linewt_set},
{"LibreDWGc::Dwg_Entity_TABLE_title_vert_left_linewt_get", _wrap_Dwg_Entity_TABLE_title_vert_left_linewt_get},
{"LibreDWGc::Dwg_Entity_TABLE_title_vert_ins_linewt_set", _wrap_Dwg_Entity_TABLE_title_vert_ins_linewt_set},
{"LibreDWGc::Dwg_Entity_TABLE_title_vert_ins_linewt_get", _wrap_Dwg_Entity_TABLE_title_vert_ins_linewt_get},
{"LibreDWGc::Dwg_Entity_TABLE_title_vert_right_linewt_set", _wrap_Dwg_Entity_TABLE_title_vert_right_linewt_set},
{"LibreDWGc::Dwg_Entity_TABLE_title_vert_right_linewt_get", _wrap_Dwg_Entity_TABLE_title_vert_right_linewt_get},
{"LibreDWGc::Dwg_Entity_TABLE_header_horiz_top_linewt_set", _wrap_Dwg_Entity_TABLE_header_horiz_top_linewt_set},
{"LibreDWGc::Dwg_Entity_TABLE_header_horiz_top_linewt_get", _wrap_Dwg_Entity_TABLE_header_horiz_top_linewt_get},
{"LibreDWGc::Dwg_Entity_TABLE_header_horiz_ins_linewt_set", _wrap_Dwg_Entity_TABLE_header_horiz_ins_linewt_set},
{"LibreDWGc::Dwg_Entity_TABLE_header_horiz_ins_linewt_get", _wrap_Dwg_Entity_TABLE_header_horiz_ins_linewt_get},
{"LibreDWGc::Dwg_Entity_TABLE_header_horiz_bottom_linewt_set", _wrap_Dwg_Entity_TABLE_header_horiz_bottom_linewt_set},
{"LibreDWGc::Dwg_Entity_TABLE_header_horiz_bottom_linewt_get", _wrap_Dwg_Entity_TABLE_header_horiz_bottom_linewt_get},
{"LibreDWGc::Dwg_Entity_TABLE_header_vert_left_linewt_set", _wrap_Dwg_Entity_TABLE_header_vert_left_linewt_set},
{"LibreDWGc::Dwg_Entity_TABLE_header_vert_left_linewt_get", _wrap_Dwg_Entity_TABLE_header_vert_left_linewt_get},
{"LibreDWGc::Dwg_Entity_TABLE_header_vert_ins_linewt_set", _wrap_Dwg_Entity_TABLE_header_vert_ins_linewt_set},
{"LibreDWGc::Dwg_Entity_TABLE_header_vert_ins_linewt_get", _wrap_Dwg_Entity_TABLE_header_vert_ins_linewt_get},
{"LibreDWGc::Dwg_Entity_TABLE_header_vert_right_linewt_set", _wrap_Dwg_Entity_TABLE_header_vert_right_linewt_set},
{"LibreDWGc::Dwg_Entity_TABLE_header_vert_right_linewt_get", _wrap_Dwg_Entity_TABLE_header_vert_right_linewt_get},
{"LibreDWGc::Dwg_Entity_TABLE_data_horiz_top_linewt_set", _wrap_Dwg_Entity_TABLE_data_horiz_top_linewt_set},
{"LibreDWGc::Dwg_Entity_TABLE_data_horiz_top_linewt_get", _wrap_Dwg_Entity_TABLE_data_horiz_top_linewt_get},
{"LibreDWGc::Dwg_Entity_TABLE_data_horiz_ins_linewt_set", _wrap_Dwg_Entity_TABLE_data_horiz_ins_linewt_set},
{"LibreDWGc::Dwg_Entity_TABLE_data_horiz_ins_linewt_get", _wrap_Dwg_Entity_TABLE_data_horiz_ins_linewt_get},
{"LibreDWGc::Dwg_Entity_TABLE_data_horiz_bottom_linewt_set", _wrap_Dwg_Entity_TABLE_data_horiz_bottom_linewt_set},
{"LibreDWGc::Dwg_Entity_TABLE_data_horiz_bottom_linewt_get", _wrap_Dwg_Entity_TABLE_data_horiz_bottom_linewt_get},
{"LibreDWGc::Dwg_Entity_TABLE_data_vert_left_linewt_set", _wrap_Dwg_Entity_TABLE_data_vert_left_linewt_set},
{"LibreDWGc::Dwg_Entity_TABLE_data_vert_left_linewt_get", _wrap_Dwg_Entity_TABLE_data_vert_left_linewt_get},
{"LibreDWGc::Dwg_Entity_TABLE_data_vert_ins_linewt_set", _wrap_Dwg_Entity_TABLE_data_vert_ins_linewt_set},
{"LibreDWGc::Dwg_Entity_TABLE_data_vert_ins_linewt_get", _wrap_Dwg_Entity_TABLE_data_vert_ins_linewt_get},
{"LibreDWGc::Dwg_Entity_TABLE_data_vert_right_linewt_set", _wrap_Dwg_Entity_TABLE_data_vert_right_linewt_set},
{"LibreDWGc::Dwg_Entity_TABLE_data_vert_right_linewt_get", _wrap_Dwg_Entity_TABLE_data_vert_right_linewt_get},
{"LibreDWGc::Dwg_Entity_TABLE_has_border_visibility_overrides_set", _wrap_Dwg_Entity_TABLE_has_border_visibility_overrides_set},
{"LibreDWGc::Dwg_Entity_TABLE_has_border_visibility_overrides_get", _wrap_Dwg_Entity_TABLE_has_border_visibility_overrides_get},
{"LibreDWGc::Dwg_Entity_TABLE_border_visibility_overrides_flag_set", _wrap_Dwg_Entity_TABLE_border_visibility_overrides_flag_set},
{"LibreDWGc::Dwg_Entity_TABLE_border_visibility_overrides_flag_get", _wrap_Dwg_Entity_TABLE_border_visibility_overrides_flag_get},
{"LibreDWGc::Dwg_Entity_TABLE_title_horiz_top_visibility_set", _wrap_Dwg_Entity_TABLE_title_horiz_top_visibility_set},
{"LibreDWGc::Dwg_Entity_TABLE_title_horiz_top_visibility_get", _wrap_Dwg_Entity_TABLE_title_horiz_top_visibility_get},
{"LibreDWGc::Dwg_Entity_TABLE_title_horiz_ins_visibility_set", _wrap_Dwg_Entity_TABLE_title_horiz_ins_visibility_set},
{"LibreDWGc::Dwg_Entity_TABLE_title_horiz_ins_visibility_get", _wrap_Dwg_Entity_TABLE_title_horiz_ins_visibility_get},
{"LibreDWGc::Dwg_Entity_TABLE_title_horiz_bottom_visibility_set", _wrap_Dwg_Entity_TABLE_title_horiz_bottom_visibility_set},
{"LibreDWGc::Dwg_Entity_TABLE_title_horiz_bottom_visibility_get", _wrap_Dwg_Entity_TABLE_title_horiz_bottom_visibility_get},
{"LibreDWGc::Dwg_Entity_TABLE_title_vert_left_visibility_set", _wrap_Dwg_Entity_TABLE_title_vert_left_visibility_set},
{"LibreDWGc::Dwg_Entity_TABLE_title_vert_left_visibility_get", _wrap_Dwg_Entity_TABLE_title_vert_left_visibility_get},
{"LibreDWGc::Dwg_Entity_TABLE_title_vert_ins_visibility_set", _wrap_Dwg_Entity_TABLE_title_vert_ins_visibility_set},
{"LibreDWGc::Dwg_Entity_TABLE_title_vert_ins_visibility_get", _wrap_Dwg_Entity_TABLE_title_vert_ins_visibility_get},
{"LibreDWGc::Dwg_Entity_TABLE_title_vert_right_visibility_set", _wrap_Dwg_Entity_TABLE_title_vert_right_visibility_set},
{"LibreDWGc::Dwg_Entity_TABLE_title_vert_right_visibility_get", _wrap_Dwg_Entity_TABLE_title_vert_right_visibility_get},
{"LibreDWGc::Dwg_Entity_TABLE_header_horiz_top_visibility_set", _wrap_Dwg_Entity_TABLE_header_horiz_top_visibility_set},
{"LibreDWGc::Dwg_Entity_TABLE_header_horiz_top_visibility_get", _wrap_Dwg_Entity_TABLE_header_horiz_top_visibility_get},
{"LibreDWGc::Dwg_Entity_TABLE_header_horiz_ins_visibility_set", _wrap_Dwg_Entity_TABLE_header_horiz_ins_visibility_set},
{"LibreDWGc::Dwg_Entity_TABLE_header_horiz_ins_visibility_get", _wrap_Dwg_Entity_TABLE_header_horiz_ins_visibility_get},
{"LibreDWGc::Dwg_Entity_TABLE_header_horiz_bottom_visibility_set", _wrap_Dwg_Entity_TABLE_header_horiz_bottom_visibility_set},
{"LibreDWGc::Dwg_Entity_TABLE_header_horiz_bottom_visibility_get", _wrap_Dwg_Entity_TABLE_header_horiz_bottom_visibility_get},
{"LibreDWGc::Dwg_Entity_TABLE_header_vert_left_visibility_set", _wrap_Dwg_Entity_TABLE_header_vert_left_visibility_set},
{"LibreDWGc::Dwg_Entity_TABLE_header_vert_left_visibility_get", _wrap_Dwg_Entity_TABLE_header_vert_left_visibility_get},
{"LibreDWGc::Dwg_Entity_TABLE_header_vert_ins_visibility_set", _wrap_Dwg_Entity_TABLE_header_vert_ins_visibility_set},
{"LibreDWGc::Dwg_Entity_TABLE_header_vert_ins_visibility_get", _wrap_Dwg_Entity_TABLE_header_vert_ins_visibility_get},
{"LibreDWGc::Dwg_Entity_TABLE_header_vert_right_visibility_set", _wrap_Dwg_Entity_TABLE_header_vert_right_visibility_set},
{"LibreDWGc::Dwg_Entity_TABLE_header_vert_right_visibility_get", _wrap_Dwg_Entity_TABLE_header_vert_right_visibility_get},
{"LibreDWGc::Dwg_Entity_TABLE_data_horiz_top_visibility_set", _wrap_Dwg_Entity_TABLE_data_horiz_top_visibility_set},
{"LibreDWGc::Dwg_Entity_TABLE_data_horiz_top_visibility_get", _wrap_Dwg_Entity_TABLE_data_horiz_top_visibility_get},
{"LibreDWGc::Dwg_Entity_TABLE_data_horiz_ins_visibility_set", _wrap_Dwg_Entity_TABLE_data_horiz_ins_visibility_set},
{"LibreDWGc::Dwg_Entity_TABLE_data_horiz_ins_visibility_get", _wrap_Dwg_Entity_TABLE_data_horiz_ins_visibility_get},
{"LibreDWGc::Dwg_Entity_TABLE_data_horiz_bottom_visibility_set", _wrap_Dwg_Entity_TABLE_data_horiz_bottom_visibility_set},
{"LibreDWGc::Dwg_Entity_TABLE_data_horiz_bottom_visibility_get", _wrap_Dwg_Entity_TABLE_data_horiz_bottom_visibility_get},
{"LibreDWGc::Dwg_Entity_TABLE_data_vert_left_visibility_set", _wrap_Dwg_Entity_TABLE_data_vert_left_visibility_set},
{"LibreDWGc::Dwg_Entity_TABLE_data_vert_left_visibility_get", _wrap_Dwg_Entity_TABLE_data_vert_left_visibility_get},
{"LibreDWGc::Dwg_Entity_TABLE_data_vert_ins_visibility_set", _wrap_Dwg_Entity_TABLE_data_vert_ins_visibility_set},
{"LibreDWGc::Dwg_Entity_TABLE_data_vert_ins_visibility_get", _wrap_Dwg_Entity_TABLE_data_vert_ins_visibility_get},
{"LibreDWGc::Dwg_Entity_TABLE_data_vert_right_visibility_set", _wrap_Dwg_Entity_TABLE_data_vert_right_visibility_set},
{"LibreDWGc::Dwg_Entity_TABLE_data_vert_right_visibility_get", _wrap_Dwg_Entity_TABLE_data_vert_right_visibility_get},
{"LibreDWGc::Dwg_Entity_TABLE_block_header_set", _wrap_Dwg_Entity_TABLE_block_header_set},
{"LibreDWGc::Dwg_Entity_TABLE_block_header_get", _wrap_Dwg_Entity_TABLE_block_header_get},
{"LibreDWGc::Dwg_Entity_TABLE_first_attrib_set", _wrap_Dwg_Entity_TABLE_first_attrib_set},
{"LibreDWGc::Dwg_Entity_TABLE_first_attrib_get", _wrap_Dwg_Entity_TABLE_first_attrib_get},
{"LibreDWGc::Dwg_Entity_TABLE_last_attrib_set", _wrap_Dwg_Entity_TABLE_last_attrib_set},
{"LibreDWGc::Dwg_Entity_TABLE_last_attrib_get", _wrap_Dwg_Entity_TABLE_last_attrib_get},
{"LibreDWGc::Dwg_Entity_TABLE_attribs_set", _wrap_Dwg_Entity_TABLE_attribs_set},
{"LibreDWGc::Dwg_Entity_TABLE_attribs_get", _wrap_Dwg_Entity_TABLE_attribs_get},
{"LibreDWGc::Dwg_Entity_TABLE_seqend_set", _wrap_Dwg_Entity_TABLE_seqend_set},
{"LibreDWGc::Dwg_Entity_TABLE_seqend_get", _wrap_Dwg_Entity_TABLE_seqend_get},
{"LibreDWGc::Dwg_Entity_TABLE_title_row_style_override_set", _wrap_Dwg_Entity_TABLE_title_row_style_override_set},
{"LibreDWGc::Dwg_Entity_TABLE_title_row_style_override_get", _wrap_Dwg_Entity_TABLE_title_row_style_override_get},
{"LibreDWGc::Dwg_Entity_TABLE_header_row_style_override_set", _wrap_Dwg_Entity_TABLE_header_row_style_override_set},
{"LibreDWGc::Dwg_Entity_TABLE_header_row_style_override_get", _wrap_Dwg_Entity_TABLE_header_row_style_override_get},
{"LibreDWGc::Dwg_Entity_TABLE_data_row_style_override_set", _wrap_Dwg_Entity_TABLE_data_row_style_override_set},
{"LibreDWGc::Dwg_Entity_TABLE_data_row_style_override_get", _wrap_Dwg_Entity_TABLE_data_row_style_override_get},
{"LibreDWGc::Dwg_Entity_TABLE_unknown_bs_set", _wrap_Dwg_Entity_TABLE_unknown_bs_set},
{"LibreDWGc::Dwg_Entity_TABLE_unknown_bs_get", _wrap_Dwg_Entity_TABLE_unknown_bs_get},
{"LibreDWGc::Dwg_Entity_TABLE_hor_dir_set", _wrap_Dwg_Entity_TABLE_hor_dir_set},
{"LibreDWGc::Dwg_Entity_TABLE_hor_dir_get", _wrap_Dwg_Entity_TABLE_hor_dir_get},
{"LibreDWGc::Dwg_Entity_TABLE_has_break_data_set", _wrap_Dwg_Entity_TABLE_has_break_data_set},
{"LibreDWGc::Dwg_Entity_TABLE_has_break_data_get", _wrap_Dwg_Entity_TABLE_has_break_data_get},
{"LibreDWGc::Dwg_Entity_TABLE_break_flag_set", _wrap_Dwg_Entity_TABLE_break_flag_set},
{"LibreDWGc::Dwg_Entity_TABLE_break_flag_get", _wrap_Dwg_Entity_TABLE_break_flag_get},
{"LibreDWGc::Dwg_Entity_TABLE_break_flow_direction_set", _wrap_Dwg_Entity_TABLE_break_flow_direction_set},
{"LibreDWGc::Dwg_Entity_TABLE_break_flow_direction_get", _wrap_Dwg_Entity_TABLE_break_flow_direction_get},
{"LibreDWGc::Dwg_Entity_TABLE_break_spacing_set", _wrap_Dwg_Entity_TABLE_break_spacing_set},
{"LibreDWGc::Dwg_Entity_TABLE_break_spacing_get", _wrap_Dwg_Entity_TABLE_break_spacing_get},
{"LibreDWGc::Dwg_Entity_TABLE_break_unknown1_set", _wrap_Dwg_Entity_TABLE_break_unknown1_set},
{"LibreDWGc::Dwg_Entity_TABLE_break_unknown1_get", _wrap_Dwg_Entity_TABLE_break_unknown1_get},
{"LibreDWGc::Dwg_Entity_TABLE_break_unknown2_set", _wrap_Dwg_Entity_TABLE_break_unknown2_set},
{"LibreDWGc::Dwg_Entity_TABLE_break_unknown2_get", _wrap_Dwg_Entity_TABLE_break_unknown2_get},
{"LibreDWGc::Dwg_Entity_TABLE_num_break_heights_set", _wrap_Dwg_Entity_TABLE_num_break_heights_set},
{"LibreDWGc::Dwg_Entity_TABLE_num_break_heights_get", _wrap_Dwg_Entity_TABLE_num_break_heights_get},
{"LibreDWGc::Dwg_Entity_TABLE_break_heights_set", _wrap_Dwg_Entity_TABLE_break_heights_set},
{"LibreDWGc::Dwg_Entity_TABLE_break_heights_get", _wrap_Dwg_Entity_TABLE_break_heights_get},
{"LibreDWGc::Dwg_Entity_TABLE_num_break_rows_set", _wrap_Dwg_Entity_TABLE_num_break_rows_set},
{"LibreDWGc::Dwg_Entity_TABLE_num_break_rows_get", _wrap_Dwg_Entity_TABLE_num_break_rows_get},
{"LibreDWGc::Dwg_Entity_TABLE_break_rows_set", _wrap_Dwg_Entity_TABLE_break_rows_set},
{"LibreDWGc::Dwg_Entity_TABLE_break_rows_get", _wrap_Dwg_Entity_TABLE_break_rows_get},
{"LibreDWGc::new_Dwg_Entity_TABLE", _wrap_new_Dwg_Entity_TABLE},
{"LibreDWGc::delete_Dwg_Entity_TABLE", _wrap_delete_Dwg_Entity_TABLE},
{"LibreDWGc::Dwg_TABLESTYLE_CellStyle_parent_set", _wrap_Dwg_TABLESTYLE_CellStyle_parent_set},
{"LibreDWGc::Dwg_TABLESTYLE_CellStyle_parent_get", _wrap_Dwg_TABLESTYLE_CellStyle_parent_get},
{"LibreDWGc::Dwg_TABLESTYLE_CellStyle_id_set", _wrap_Dwg_TABLESTYLE_CellStyle_id_set},
{"LibreDWGc::Dwg_TABLESTYLE_CellStyle_id_get", _wrap_Dwg_TABLESTYLE_CellStyle_id_get},
{"LibreDWGc::Dwg_TABLESTYLE_CellStyle_type_set", _wrap_Dwg_TABLESTYLE_CellStyle_type_set},
{"LibreDWGc::Dwg_TABLESTYLE_CellStyle_type_get", _wrap_Dwg_TABLESTYLE_CellStyle_type_get},
{"LibreDWGc::Dwg_TABLESTYLE_CellStyle_name_set", _wrap_Dwg_TABLESTYLE_CellStyle_name_set},
{"LibreDWGc::Dwg_TABLESTYLE_CellStyle_name_get", _wrap_Dwg_TABLESTYLE_CellStyle_name_get},
{"LibreDWGc::Dwg_TABLESTYLE_CellStyle_cellstyle_set", _wrap_Dwg_TABLESTYLE_CellStyle_cellstyle_set},
{"LibreDWGc::Dwg_TABLESTYLE_CellStyle_cellstyle_get", _wrap_Dwg_TABLESTYLE_CellStyle_cellstyle_get},
{"LibreDWGc::new_Dwg_TABLESTYLE_CellStyle", _wrap_new_Dwg_TABLESTYLE_CellStyle},
{"LibreDWGc::delete_Dwg_TABLESTYLE_CellStyle", _wrap_delete_Dwg_TABLESTYLE_CellStyle},
{"LibreDWGc::Dwg_TABLESTYLE_border_linewt_set", _wrap_Dwg_TABLESTYLE_border_linewt_set},
{"LibreDWGc::Dwg_TABLESTYLE_border_linewt_get", _wrap_Dwg_TABLESTYLE_border_linewt_get},
{"LibreDWGc::Dwg_TABLESTYLE_border_visible_set", _wrap_Dwg_TABLESTYLE_border_visible_set},
{"LibreDWGc::Dwg_TABLESTYLE_border_visible_get", _wrap_Dwg_TABLESTYLE_border_visible_get},
{"LibreDWGc::Dwg_TABLESTYLE_border_color_set", _wrap_Dwg_TABLESTYLE_border_color_set},
{"LibreDWGc::Dwg_TABLESTYLE_border_color_get", _wrap_Dwg_TABLESTYLE_border_color_get},
{"LibreDWGc::new_Dwg_TABLESTYLE_border", _wrap_new_Dwg_TABLESTYLE_border},
{"LibreDWGc::delete_Dwg_TABLESTYLE_border", _wrap_delete_Dwg_TABLESTYLE_border},
{"LibreDWGc::Dwg_TABLESTYLE_rowstyles_parent_set", _wrap_Dwg_TABLESTYLE_rowstyles_parent_set},
{"LibreDWGc::Dwg_TABLESTYLE_rowstyles_parent_get", _wrap_Dwg_TABLESTYLE_rowstyles_parent_get},
{"LibreDWGc::Dwg_TABLESTYLE_rowstyles_text_style_set", _wrap_Dwg_TABLESTYLE_rowstyles_text_style_set},
{"LibreDWGc::Dwg_TABLESTYLE_rowstyles_text_style_get", _wrap_Dwg_TABLESTYLE_rowstyles_text_style_get},
{"LibreDWGc::Dwg_TABLESTYLE_rowstyles_text_height_set", _wrap_Dwg_TABLESTYLE_rowstyles_text_height_set},
{"LibreDWGc::Dwg_TABLESTYLE_rowstyles_text_height_get", _wrap_Dwg_TABLESTYLE_rowstyles_text_height_get},
{"LibreDWGc::Dwg_TABLESTYLE_rowstyles_text_alignment_set", _wrap_Dwg_TABLESTYLE_rowstyles_text_alignment_set},
{"LibreDWGc::Dwg_TABLESTYLE_rowstyles_text_alignment_get", _wrap_Dwg_TABLESTYLE_rowstyles_text_alignment_get},
{"LibreDWGc::Dwg_TABLESTYLE_rowstyles_text_color_set", _wrap_Dwg_TABLESTYLE_rowstyles_text_color_set},
{"LibreDWGc::Dwg_TABLESTYLE_rowstyles_text_color_get", _wrap_Dwg_TABLESTYLE_rowstyles_text_color_get},
{"LibreDWGc::Dwg_TABLESTYLE_rowstyles_fill_color_set", _wrap_Dwg_TABLESTYLE_rowstyles_fill_color_set},
{"LibreDWGc::Dwg_TABLESTYLE_rowstyles_fill_color_get", _wrap_Dwg_TABLESTYLE_rowstyles_fill_color_get},
{"LibreDWGc::Dwg_TABLESTYLE_rowstyles_has_bgcolor_set", _wrap_Dwg_TABLESTYLE_rowstyles_has_bgcolor_set},
{"LibreDWGc::Dwg_TABLESTYLE_rowstyles_has_bgcolor_get", _wrap_Dwg_TABLESTYLE_rowstyles_has_bgcolor_get},
{"LibreDWGc::Dwg_TABLESTYLE_rowstyles_num_borders_set", _wrap_Dwg_TABLESTYLE_rowstyles_num_borders_set},
{"LibreDWGc::Dwg_TABLESTYLE_rowstyles_num_borders_get", _wrap_Dwg_TABLESTYLE_rowstyles_num_borders_get},
{"LibreDWGc::Dwg_TABLESTYLE_rowstyles_borders_set", _wrap_Dwg_TABLESTYLE_rowstyles_borders_set},
{"LibreDWGc::Dwg_TABLESTYLE_rowstyles_borders_get", _wrap_Dwg_TABLESTYLE_rowstyles_borders_get},
{"LibreDWGc::Dwg_TABLESTYLE_rowstyles_data_type_set", _wrap_Dwg_TABLESTYLE_rowstyles_data_type_set},
{"LibreDWGc::Dwg_TABLESTYLE_rowstyles_data_type_get", _wrap_Dwg_TABLESTYLE_rowstyles_data_type_get},
{"LibreDWGc::Dwg_TABLESTYLE_rowstyles_unit_type_set", _wrap_Dwg_TABLESTYLE_rowstyles_unit_type_set},
{"LibreDWGc::Dwg_TABLESTYLE_rowstyles_unit_type_get", _wrap_Dwg_TABLESTYLE_rowstyles_unit_type_get},
{"LibreDWGc::Dwg_TABLESTYLE_rowstyles_format_string_set", _wrap_Dwg_TABLESTYLE_rowstyles_format_string_set},
{"LibreDWGc::Dwg_TABLESTYLE_rowstyles_format_string_get", _wrap_Dwg_TABLESTYLE_rowstyles_format_string_get},
{"LibreDWGc::new_Dwg_TABLESTYLE_rowstyles", _wrap_new_Dwg_TABLESTYLE_rowstyles},
{"LibreDWGc::delete_Dwg_TABLESTYLE_rowstyles", _wrap_delete_Dwg_TABLESTYLE_rowstyles},
{"LibreDWGc::Dwg_Object_TABLESTYLE_parent_set", _wrap_Dwg_Object_TABLESTYLE_parent_set},
{"LibreDWGc::Dwg_Object_TABLESTYLE_parent_get", _wrap_Dwg_Object_TABLESTYLE_parent_get},
{"LibreDWGc::Dwg_Object_TABLESTYLE_class_version_set", _wrap_Dwg_Object_TABLESTYLE_class_version_set},
{"LibreDWGc::Dwg_Object_TABLESTYLE_class_version_get", _wrap_Dwg_Object_TABLESTYLE_class_version_get},
{"LibreDWGc::Dwg_Object_TABLESTYLE_name_set", _wrap_Dwg_Object_TABLESTYLE_name_set},
{"LibreDWGc::Dwg_Object_TABLESTYLE_name_get", _wrap_Dwg_Object_TABLESTYLE_name_get},
{"LibreDWGc::Dwg_Object_TABLESTYLE_flags_set", _wrap_Dwg_Object_TABLESTYLE_flags_set},
{"LibreDWGc::Dwg_Object_TABLESTYLE_flags_get", _wrap_Dwg_Object_TABLESTYLE_flags_get},
{"LibreDWGc::Dwg_Object_TABLESTYLE_flow_direction_set", _wrap_Dwg_Object_TABLESTYLE_flow_direction_set},
{"LibreDWGc::Dwg_Object_TABLESTYLE_flow_direction_get", _wrap_Dwg_Object_TABLESTYLE_flow_direction_get},
{"LibreDWGc::Dwg_Object_TABLESTYLE_horiz_cell_margin_set", _wrap_Dwg_Object_TABLESTYLE_horiz_cell_margin_set},
{"LibreDWGc::Dwg_Object_TABLESTYLE_horiz_cell_margin_get", _wrap_Dwg_Object_TABLESTYLE_horiz_cell_margin_get},
{"LibreDWGc::Dwg_Object_TABLESTYLE_vert_cell_margin_set", _wrap_Dwg_Object_TABLESTYLE_vert_cell_margin_set},
{"LibreDWGc::Dwg_Object_TABLESTYLE_vert_cell_margin_get", _wrap_Dwg_Object_TABLESTYLE_vert_cell_margin_get},
{"LibreDWGc::Dwg_Object_TABLESTYLE_is_title_suppressed_set", _wrap_Dwg_Object_TABLESTYLE_is_title_suppressed_set},
{"LibreDWGc::Dwg_Object_TABLESTYLE_is_title_suppressed_get", _wrap_Dwg_Object_TABLESTYLE_is_title_suppressed_get},
{"LibreDWGc::Dwg_Object_TABLESTYLE_is_header_suppressed_set", _wrap_Dwg_Object_TABLESTYLE_is_header_suppressed_set},
{"LibreDWGc::Dwg_Object_TABLESTYLE_is_header_suppressed_get", _wrap_Dwg_Object_TABLESTYLE_is_header_suppressed_get},
{"LibreDWGc::Dwg_Object_TABLESTYLE_unknown_rc_set", _wrap_Dwg_Object_TABLESTYLE_unknown_rc_set},
{"LibreDWGc::Dwg_Object_TABLESTYLE_unknown_rc_get", _wrap_Dwg_Object_TABLESTYLE_unknown_rc_get},
{"LibreDWGc::Dwg_Object_TABLESTYLE_unknown_bl1_set", _wrap_Dwg_Object_TABLESTYLE_unknown_bl1_set},
{"LibreDWGc::Dwg_Object_TABLESTYLE_unknown_bl1_get", _wrap_Dwg_Object_TABLESTYLE_unknown_bl1_get},
{"LibreDWGc::Dwg_Object_TABLESTYLE_unknown_bl2_set", _wrap_Dwg_Object_TABLESTYLE_unknown_bl2_set},
{"LibreDWGc::Dwg_Object_TABLESTYLE_unknown_bl2_get", _wrap_Dwg_Object_TABLESTYLE_unknown_bl2_get},
{"LibreDWGc::Dwg_Object_TABLESTYLE_cellstyle_set", _wrap_Dwg_Object_TABLESTYLE_cellstyle_set},
{"LibreDWGc::Dwg_Object_TABLESTYLE_cellstyle_get", _wrap_Dwg_Object_TABLESTYLE_cellstyle_get},
{"LibreDWGc::Dwg_Object_TABLESTYLE_sty_set", _wrap_Dwg_Object_TABLESTYLE_sty_set},
{"LibreDWGc::Dwg_Object_TABLESTYLE_sty_get", _wrap_Dwg_Object_TABLESTYLE_sty_get},
{"LibreDWGc::Dwg_Object_TABLESTYLE_numoverrides_set", _wrap_Dwg_Object_TABLESTYLE_numoverrides_set},
{"LibreDWGc::Dwg_Object_TABLESTYLE_numoverrides_get", _wrap_Dwg_Object_TABLESTYLE_numoverrides_get},
{"LibreDWGc::Dwg_Object_TABLESTYLE_unknown_bl3_set", _wrap_Dwg_Object_TABLESTYLE_unknown_bl3_set},
{"LibreDWGc::Dwg_Object_TABLESTYLE_unknown_bl3_get", _wrap_Dwg_Object_TABLESTYLE_unknown_bl3_get},
{"LibreDWGc::Dwg_Object_TABLESTYLE_ovr_set", _wrap_Dwg_Object_TABLESTYLE_ovr_set},
{"LibreDWGc::Dwg_Object_TABLESTYLE_ovr_get", _wrap_Dwg_Object_TABLESTYLE_ovr_get},
{"LibreDWGc::Dwg_Object_TABLESTYLE_num_rowstyles_set", _wrap_Dwg_Object_TABLESTYLE_num_rowstyles_set},
{"LibreDWGc::Dwg_Object_TABLESTYLE_num_rowstyles_get", _wrap_Dwg_Object_TABLESTYLE_num_rowstyles_get},
{"LibreDWGc::Dwg_Object_TABLESTYLE_rowstyles_set", _wrap_Dwg_Object_TABLESTYLE_rowstyles_set},
{"LibreDWGc::Dwg_Object_TABLESTYLE_rowstyles_get", _wrap_Dwg_Object_TABLESTYLE_rowstyles_get},
{"LibreDWGc::new_Dwg_Object_TABLESTYLE", _wrap_new_Dwg_Object_TABLESTYLE},
{"LibreDWGc::delete_Dwg_Object_TABLESTYLE", _wrap_delete_Dwg_Object_TABLESTYLE},
{"LibreDWGc::Dwg_Object_CELLSTYLEMAP_parent_set", _wrap_Dwg_Object_CELLSTYLEMAP_parent_set},
{"LibreDWGc::Dwg_Object_CELLSTYLEMAP_parent_get", _wrap_Dwg_Object_CELLSTYLEMAP_parent_get},
{"LibreDWGc::Dwg_Object_CELLSTYLEMAP_num_cells_set", _wrap_Dwg_Object_CELLSTYLEMAP_num_cells_set},
{"LibreDWGc::Dwg_Object_CELLSTYLEMAP_num_cells_get", _wrap_Dwg_Object_CELLSTYLEMAP_num_cells_get},
{"LibreDWGc::Dwg_Object_CELLSTYLEMAP_cells_set", _wrap_Dwg_Object_CELLSTYLEMAP_cells_set},
{"LibreDWGc::Dwg_Object_CELLSTYLEMAP_cells_get", _wrap_Dwg_Object_CELLSTYLEMAP_cells_get},
{"LibreDWGc::new_Dwg_Object_CELLSTYLEMAP", _wrap_new_Dwg_Object_CELLSTYLEMAP},
{"LibreDWGc::delete_Dwg_Object_CELLSTYLEMAP", _wrap_delete_Dwg_Object_CELLSTYLEMAP},
{"LibreDWGc::Dwg_TABLEGEOMETRY_Cell_parent_set", _wrap_Dwg_TABLEGEOMETRY_Cell_parent_set},
{"LibreDWGc::Dwg_TABLEGEOMETRY_Cell_parent_get", _wrap_Dwg_TABLEGEOMETRY_Cell_parent_get},
{"LibreDWGc::Dwg_TABLEGEOMETRY_Cell_geom_data_flag_set", _wrap_Dwg_TABLEGEOMETRY_Cell_geom_data_flag_set},
{"LibreDWGc::Dwg_TABLEGEOMETRY_Cell_geom_data_flag_get", _wrap_Dwg_TABLEGEOMETRY_Cell_geom_data_flag_get},
{"LibreDWGc::Dwg_TABLEGEOMETRY_Cell_width_w_gap_set", _wrap_Dwg_TABLEGEOMETRY_Cell_width_w_gap_set},
{"LibreDWGc::Dwg_TABLEGEOMETRY_Cell_width_w_gap_get", _wrap_Dwg_TABLEGEOMETRY_Cell_width_w_gap_get},
{"LibreDWGc::Dwg_TABLEGEOMETRY_Cell_height_w_gap_set", _wrap_Dwg_TABLEGEOMETRY_Cell_height_w_gap_set},
{"LibreDWGc::Dwg_TABLEGEOMETRY_Cell_height_w_gap_get", _wrap_Dwg_TABLEGEOMETRY_Cell_height_w_gap_get},
{"LibreDWGc::Dwg_TABLEGEOMETRY_Cell_tablegeometry_set", _wrap_Dwg_TABLEGEOMETRY_Cell_tablegeometry_set},
{"LibreDWGc::Dwg_TABLEGEOMETRY_Cell_tablegeometry_get", _wrap_Dwg_TABLEGEOMETRY_Cell_tablegeometry_get},
{"LibreDWGc::Dwg_TABLEGEOMETRY_Cell_num_geometry_set", _wrap_Dwg_TABLEGEOMETRY_Cell_num_geometry_set},
{"LibreDWGc::Dwg_TABLEGEOMETRY_Cell_num_geometry_get", _wrap_Dwg_TABLEGEOMETRY_Cell_num_geometry_get},
{"LibreDWGc::Dwg_TABLEGEOMETRY_Cell_geometry_set", _wrap_Dwg_TABLEGEOMETRY_Cell_geometry_set},
{"LibreDWGc::Dwg_TABLEGEOMETRY_Cell_geometry_get", _wrap_Dwg_TABLEGEOMETRY_Cell_geometry_get},
{"LibreDWGc::new_Dwg_TABLEGEOMETRY_Cell", _wrap_new_Dwg_TABLEGEOMETRY_Cell},
{"LibreDWGc::delete_Dwg_TABLEGEOMETRY_Cell", _wrap_delete_Dwg_TABLEGEOMETRY_Cell},
{"LibreDWGc::Dwg_Object_TABLEGEOMETRY_parent_set", _wrap_Dwg_Object_TABLEGEOMETRY_parent_set},
{"LibreDWGc::Dwg_Object_TABLEGEOMETRY_parent_get", _wrap_Dwg_Object_TABLEGEOMETRY_parent_get},
{"LibreDWGc::Dwg_Object_TABLEGEOMETRY_numrows_set", _wrap_Dwg_Object_TABLEGEOMETRY_numrows_set},
{"LibreDWGc::Dwg_Object_TABLEGEOMETRY_numrows_get", _wrap_Dwg_Object_TABLEGEOMETRY_numrows_get},
{"LibreDWGc::Dwg_Object_TABLEGEOMETRY_numcols_set", _wrap_Dwg_Object_TABLEGEOMETRY_numcols_set},
{"LibreDWGc::Dwg_Object_TABLEGEOMETRY_numcols_get", _wrap_Dwg_Object_TABLEGEOMETRY_numcols_get},
{"LibreDWGc::Dwg_Object_TABLEGEOMETRY_num_cells_set", _wrap_Dwg_Object_TABLEGEOMETRY_num_cells_set},
{"LibreDWGc::Dwg_Object_TABLEGEOMETRY_num_cells_get", _wrap_Dwg_Object_TABLEGEOMETRY_num_cells_get},
{"LibreDWGc::Dwg_Object_TABLEGEOMETRY_cells_set", _wrap_Dwg_Object_TABLEGEOMETRY_cells_set},
{"LibreDWGc::Dwg_Object_TABLEGEOMETRY_cells_get", _wrap_Dwg_Object_TABLEGEOMETRY_cells_get},
{"LibreDWGc::new_Dwg_Object_TABLEGEOMETRY", _wrap_new_Dwg_Object_TABLEGEOMETRY},
{"LibreDWGc::delete_Dwg_Object_TABLEGEOMETRY", _wrap_delete_Dwg_Object_TABLEGEOMETRY},
{"LibreDWGc::Dwg_Object_UNDERLAYDEFINITION_parent_set", _wrap_Dwg_Object_UNDERLAYDEFINITION_parent_set},
{"LibreDWGc::Dwg_Object_UNDERLAYDEFINITION_parent_get", _wrap_Dwg_Object_UNDERLAYDEFINITION_parent_get},
{"LibreDWGc::Dwg_Object_UNDERLAYDEFINITION_filename_set", _wrap_Dwg_Object_UNDERLAYDEFINITION_filename_set},
{"LibreDWGc::Dwg_Object_UNDERLAYDEFINITION_filename_get", _wrap_Dwg_Object_UNDERLAYDEFINITION_filename_get},
{"LibreDWGc::Dwg_Object_UNDERLAYDEFINITION_name_set", _wrap_Dwg_Object_UNDERLAYDEFINITION_name_set},
{"LibreDWGc::Dwg_Object_UNDERLAYDEFINITION_name_get", _wrap_Dwg_Object_UNDERLAYDEFINITION_name_get},
{"LibreDWGc::new_Dwg_Object_UNDERLAYDEFINITION", _wrap_new_Dwg_Object_UNDERLAYDEFINITION},
{"LibreDWGc::delete_Dwg_Object_UNDERLAYDEFINITION", _wrap_delete_Dwg_Object_UNDERLAYDEFINITION},
{"LibreDWGc::Dwg_Entity_UNDERLAY_parent_set", _wrap_Dwg_Entity_UNDERLAY_parent_set},
{"LibreDWGc::Dwg_Entity_UNDERLAY_parent_get", _wrap_Dwg_Entity_UNDERLAY_parent_get},
{"LibreDWGc::Dwg_Entity_UNDERLAY_extrusion_set", _wrap_Dwg_Entity_UNDERLAY_extrusion_set},
{"LibreDWGc::Dwg_Entity_UNDERLAY_extrusion_get", _wrap_Dwg_Entity_UNDERLAY_extrusion_get},
{"LibreDWGc::Dwg_Entity_UNDERLAY_ins_pt_set", _wrap_Dwg_Entity_UNDERLAY_ins_pt_set},
{"LibreDWGc::Dwg_Entity_UNDERLAY_ins_pt_get", _wrap_Dwg_Entity_UNDERLAY_ins_pt_get},
{"LibreDWGc::Dwg_Entity_UNDERLAY_scale_set", _wrap_Dwg_Entity_UNDERLAY_scale_set},
{"LibreDWGc::Dwg_Entity_UNDERLAY_scale_get", _wrap_Dwg_Entity_UNDERLAY_scale_get},
{"LibreDWGc::Dwg_Entity_UNDERLAY_angle_set", _wrap_Dwg_Entity_UNDERLAY_angle_set},
{"LibreDWGc::Dwg_Entity_UNDERLAY_angle_get", _wrap_Dwg_Entity_UNDERLAY_angle_get},
{"LibreDWGc::Dwg_Entity_UNDERLAY_flag_set", _wrap_Dwg_Entity_UNDERLAY_flag_set},
{"LibreDWGc::Dwg_Entity_UNDERLAY_flag_get", _wrap_Dwg_Entity_UNDERLAY_flag_get},
{"LibreDWGc::Dwg_Entity_UNDERLAY_contrast_set", _wrap_Dwg_Entity_UNDERLAY_contrast_set},
{"LibreDWGc::Dwg_Entity_UNDERLAY_contrast_get", _wrap_Dwg_Entity_UNDERLAY_contrast_get},
{"LibreDWGc::Dwg_Entity_UNDERLAY_fade_set", _wrap_Dwg_Entity_UNDERLAY_fade_set},
{"LibreDWGc::Dwg_Entity_UNDERLAY_fade_get", _wrap_Dwg_Entity_UNDERLAY_fade_get},
{"LibreDWGc::Dwg_Entity_UNDERLAY_num_clip_verts_set", _wrap_Dwg_Entity_UNDERLAY_num_clip_verts_set},
{"LibreDWGc::Dwg_Entity_UNDERLAY_num_clip_verts_get", _wrap_Dwg_Entity_UNDERLAY_num_clip_verts_get},
{"LibreDWGc::Dwg_Entity_UNDERLAY_clip_verts_set", _wrap_Dwg_Entity_UNDERLAY_clip_verts_set},
{"LibreDWGc::Dwg_Entity_UNDERLAY_clip_verts_get", _wrap_Dwg_Entity_UNDERLAY_clip_verts_get},
{"LibreDWGc::Dwg_Entity_UNDERLAY_num_clip_inverts_set", _wrap_Dwg_Entity_UNDERLAY_num_clip_inverts_set},
{"LibreDWGc::Dwg_Entity_UNDERLAY_num_clip_inverts_get", _wrap_Dwg_Entity_UNDERLAY_num_clip_inverts_get},
{"LibreDWGc::Dwg_Entity_UNDERLAY_clip_inverts_set", _wrap_Dwg_Entity_UNDERLAY_clip_inverts_set},
{"LibreDWGc::Dwg_Entity_UNDERLAY_clip_inverts_get", _wrap_Dwg_Entity_UNDERLAY_clip_inverts_get},
{"LibreDWGc::Dwg_Entity_UNDERLAY_definition_id_set", _wrap_Dwg_Entity_UNDERLAY_definition_id_set},
{"LibreDWGc::Dwg_Entity_UNDERLAY_definition_id_get", _wrap_Dwg_Entity_UNDERLAY_definition_id_get},
{"LibreDWGc::new_Dwg_Entity_UNDERLAY", _wrap_new_Dwg_Entity_UNDERLAY},
{"LibreDWGc::delete_Dwg_Entity_UNDERLAY", _wrap_delete_Dwg_Entity_UNDERLAY},
{"LibreDWGc::Dwg_Object_DBCOLOR_parent_set", _wrap_Dwg_Object_DBCOLOR_parent_set},
{"LibreDWGc::Dwg_Object_DBCOLOR_parent_get", _wrap_Dwg_Object_DBCOLOR_parent_get},
{"LibreDWGc::Dwg_Object_DBCOLOR_color_set", _wrap_Dwg_Object_DBCOLOR_color_set},
{"LibreDWGc::Dwg_Object_DBCOLOR_color_get", _wrap_Dwg_Object_DBCOLOR_color_get},
{"LibreDWGc::new_Dwg_Object_DBCOLOR", _wrap_new_Dwg_Object_DBCOLOR},
{"LibreDWGc::delete_Dwg_Object_DBCOLOR", _wrap_delete_Dwg_Object_DBCOLOR},
{"LibreDWGc::Dwg_FIELD_ChildValue_parent_set", _wrap_Dwg_FIELD_ChildValue_parent_set},
{"LibreDWGc::Dwg_FIELD_ChildValue_parent_get", _wrap_Dwg_FIELD_ChildValue_parent_get},
{"LibreDWGc::Dwg_FIELD_ChildValue_key_set", _wrap_Dwg_FIELD_ChildValue_key_set},
{"LibreDWGc::Dwg_FIELD_ChildValue_key_get", _wrap_Dwg_FIELD_ChildValue_key_get},
{"LibreDWGc::Dwg_FIELD_ChildValue_value_set", _wrap_Dwg_FIELD_ChildValue_value_set},
{"LibreDWGc::Dwg_FIELD_ChildValue_value_get", _wrap_Dwg_FIELD_ChildValue_value_get},
{"LibreDWGc::new_Dwg_FIELD_ChildValue", _wrap_new_Dwg_FIELD_ChildValue},
{"LibreDWGc::delete_Dwg_FIELD_ChildValue", _wrap_delete_Dwg_FIELD_ChildValue},
{"LibreDWGc::Dwg_Object_FIELD_parent_set", _wrap_Dwg_Object_FIELD_parent_set},
{"LibreDWGc::Dwg_Object_FIELD_parent_get", _wrap_Dwg_Object_FIELD_parent_get},
{"LibreDWGc::Dwg_Object_FIELD_id_set", _wrap_Dwg_Object_FIELD_id_set},
{"LibreDWGc::Dwg_Object_FIELD_id_get", _wrap_Dwg_Object_FIELD_id_get},
{"LibreDWGc::Dwg_Object_FIELD_code_set", _wrap_Dwg_Object_FIELD_code_set},
{"LibreDWGc::Dwg_Object_FIELD_code_get", _wrap_Dwg_Object_FIELD_code_get},
{"LibreDWGc::Dwg_Object_FIELD_num_childs_set", _wrap_Dwg_Object_FIELD_num_childs_set},
{"LibreDWGc::Dwg_Object_FIELD_num_childs_get", _wrap_Dwg_Object_FIELD_num_childs_get},
{"LibreDWGc::Dwg_Object_FIELD_childs_set", _wrap_Dwg_Object_FIELD_childs_set},
{"LibreDWGc::Dwg_Object_FIELD_childs_get", _wrap_Dwg_Object_FIELD_childs_get},
{"LibreDWGc::Dwg_Object_FIELD_num_objects_set", _wrap_Dwg_Object_FIELD_num_objects_set},
{"LibreDWGc::Dwg_Object_FIELD_num_objects_get", _wrap_Dwg_Object_FIELD_num_objects_get},
{"LibreDWGc::Dwg_Object_FIELD_objects_set", _wrap_Dwg_Object_FIELD_objects_set},
{"LibreDWGc::Dwg_Object_FIELD_objects_get", _wrap_Dwg_Object_FIELD_objects_get},
{"LibreDWGc::Dwg_Object_FIELD_format_set", _wrap_Dwg_Object_FIELD_format_set},
{"LibreDWGc::Dwg_Object_FIELD_format_get", _wrap_Dwg_Object_FIELD_format_get},
{"LibreDWGc::Dwg_Object_FIELD_evaluation_option_set", _wrap_Dwg_Object_FIELD_evaluation_option_set},
{"LibreDWGc::Dwg_Object_FIELD_evaluation_option_get", _wrap_Dwg_Object_FIELD_evaluation_option_get},
{"LibreDWGc::Dwg_Object_FIELD_filing_option_set", _wrap_Dwg_Object_FIELD_filing_option_set},
{"LibreDWGc::Dwg_Object_FIELD_filing_option_get", _wrap_Dwg_Object_FIELD_filing_option_get},
{"LibreDWGc::Dwg_Object_FIELD_field_state_set", _wrap_Dwg_Object_FIELD_field_state_set},
{"LibreDWGc::Dwg_Object_FIELD_field_state_get", _wrap_Dwg_Object_FIELD_field_state_get},
{"LibreDWGc::Dwg_Object_FIELD_evaluation_status_set", _wrap_Dwg_Object_FIELD_evaluation_status_set},
{"LibreDWGc::Dwg_Object_FIELD_evaluation_status_get", _wrap_Dwg_Object_FIELD_evaluation_status_get},
{"LibreDWGc::Dwg_Object_FIELD_evaluation_error_code_set", _wrap_Dwg_Object_FIELD_evaluation_error_code_set},
{"LibreDWGc::Dwg_Object_FIELD_evaluation_error_code_get", _wrap_Dwg_Object_FIELD_evaluation_error_code_get},
{"LibreDWGc::Dwg_Object_FIELD_evaluation_error_msg_set", _wrap_Dwg_Object_FIELD_evaluation_error_msg_set},
{"LibreDWGc::Dwg_Object_FIELD_evaluation_error_msg_get", _wrap_Dwg_Object_FIELD_evaluation_error_msg_get},
{"LibreDWGc::Dwg_Object_FIELD_value_set", _wrap_Dwg_Object_FIELD_value_set},
{"LibreDWGc::Dwg_Object_FIELD_value_get", _wrap_Dwg_Object_FIELD_value_get},
{"LibreDWGc::Dwg_Object_FIELD_value_string_set", _wrap_Dwg_Object_FIELD_value_string_set},
{"LibreDWGc::Dwg_Object_FIELD_value_string_get", _wrap_Dwg_Object_FIELD_value_string_get},
{"LibreDWGc::Dwg_Object_FIELD_value_string_length_set", _wrap_Dwg_Object_FIELD_value_string_length_set},
{"LibreDWGc::Dwg_Object_FIELD_value_string_length_get", _wrap_Dwg_Object_FIELD_value_string_length_get},
{"LibreDWGc::Dwg_Object_FIELD_num_childval_set", _wrap_Dwg_Object_FIELD_num_childval_set},
{"LibreDWGc::Dwg_Object_FIELD_num_childval_get", _wrap_Dwg_Object_FIELD_num_childval_get},
{"LibreDWGc::Dwg_Object_FIELD_childval_set", _wrap_Dwg_Object_FIELD_childval_set},
{"LibreDWGc::Dwg_Object_FIELD_childval_get", _wrap_Dwg_Object_FIELD_childval_get},
{"LibreDWGc::new_Dwg_Object_FIELD", _wrap_new_Dwg_Object_FIELD},
{"LibreDWGc::delete_Dwg_Object_FIELD", _wrap_delete_Dwg_Object_FIELD},
{"LibreDWGc::Dwg_Object_FIELDLIST_parent_set", _wrap_Dwg_Object_FIELDLIST_parent_set},
{"LibreDWGc::Dwg_Object_FIELDLIST_parent_get", _wrap_Dwg_Object_FIELDLIST_parent_get},
{"LibreDWGc::Dwg_Object_FIELDLIST_num_fields_set", _wrap_Dwg_Object_FIELDLIST_num_fields_set},
{"LibreDWGc::Dwg_Object_FIELDLIST_num_fields_get", _wrap_Dwg_Object_FIELDLIST_num_fields_get},
{"LibreDWGc::Dwg_Object_FIELDLIST_unknown_set", _wrap_Dwg_Object_FIELDLIST_unknown_set},
{"LibreDWGc::Dwg_Object_FIELDLIST_unknown_get", _wrap_Dwg_Object_FIELDLIST_unknown_get},
{"LibreDWGc::Dwg_Object_FIELDLIST_fields_set", _wrap_Dwg_Object_FIELDLIST_fields_set},
{"LibreDWGc::Dwg_Object_FIELDLIST_fields_get", _wrap_Dwg_Object_FIELDLIST_fields_get},
{"LibreDWGc::new_Dwg_Object_FIELDLIST", _wrap_new_Dwg_Object_FIELDLIST},
{"LibreDWGc::delete_Dwg_Object_FIELDLIST", _wrap_delete_Dwg_Object_FIELDLIST},
{"LibreDWGc::Dwg_GEODATA_meshpt_source_pt_set", _wrap_Dwg_GEODATA_meshpt_source_pt_set},
{"LibreDWGc::Dwg_GEODATA_meshpt_source_pt_get", _wrap_Dwg_GEODATA_meshpt_source_pt_get},
{"LibreDWGc::Dwg_GEODATA_meshpt_dest_pt_set", _wrap_Dwg_GEODATA_meshpt_dest_pt_set},
{"LibreDWGc::Dwg_GEODATA_meshpt_dest_pt_get", _wrap_Dwg_GEODATA_meshpt_dest_pt_get},
{"LibreDWGc::new_Dwg_GEODATA_meshpt", _wrap_new_Dwg_GEODATA_meshpt},
{"LibreDWGc::delete_Dwg_GEODATA_meshpt", _wrap_delete_Dwg_GEODATA_meshpt},
{"LibreDWGc::Dwg_GEODATA_meshface_face1_set", _wrap_Dwg_GEODATA_meshface_face1_set},
{"LibreDWGc::Dwg_GEODATA_meshface_face1_get", _wrap_Dwg_GEODATA_meshface_face1_get},
{"LibreDWGc::Dwg_GEODATA_meshface_face2_set", _wrap_Dwg_GEODATA_meshface_face2_set},
{"LibreDWGc::Dwg_GEODATA_meshface_face2_get", _wrap_Dwg_GEODATA_meshface_face2_get},
{"LibreDWGc::Dwg_GEODATA_meshface_face3_set", _wrap_Dwg_GEODATA_meshface_face3_set},
{"LibreDWGc::Dwg_GEODATA_meshface_face3_get", _wrap_Dwg_GEODATA_meshface_face3_get},
{"LibreDWGc::new_Dwg_GEODATA_meshface", _wrap_new_Dwg_GEODATA_meshface},
{"LibreDWGc::delete_Dwg_GEODATA_meshface", _wrap_delete_Dwg_GEODATA_meshface},
{"LibreDWGc::Dwg_Object_GEODATA_parent_set", _wrap_Dwg_Object_GEODATA_parent_set},
{"LibreDWGc::Dwg_Object_GEODATA_parent_get", _wrap_Dwg_Object_GEODATA_parent_get},
{"LibreDWGc::Dwg_Object_GEODATA_class_version_set", _wrap_Dwg_Object_GEODATA_class_version_set},
{"LibreDWGc::Dwg_Object_GEODATA_class_version_get", _wrap_Dwg_Object_GEODATA_class_version_get},
{"LibreDWGc::Dwg_Object_GEODATA_host_block_set", _wrap_Dwg_Object_GEODATA_host_block_set},
{"LibreDWGc::Dwg_Object_GEODATA_host_block_get", _wrap_Dwg_Object_GEODATA_host_block_get},
{"LibreDWGc::Dwg_Object_GEODATA_coord_type_set", _wrap_Dwg_Object_GEODATA_coord_type_set},
{"LibreDWGc::Dwg_Object_GEODATA_coord_type_get", _wrap_Dwg_Object_GEODATA_coord_type_get},
{"LibreDWGc::Dwg_Object_GEODATA_design_pt_set", _wrap_Dwg_Object_GEODATA_design_pt_set},
{"LibreDWGc::Dwg_Object_GEODATA_design_pt_get", _wrap_Dwg_Object_GEODATA_design_pt_get},
{"LibreDWGc::Dwg_Object_GEODATA_ref_pt_set", _wrap_Dwg_Object_GEODATA_ref_pt_set},
{"LibreDWGc::Dwg_Object_GEODATA_ref_pt_get", _wrap_Dwg_Object_GEODATA_ref_pt_get},
{"LibreDWGc::Dwg_Object_GEODATA_obs_pt_set", _wrap_Dwg_Object_GEODATA_obs_pt_set},
{"LibreDWGc::Dwg_Object_GEODATA_obs_pt_get", _wrap_Dwg_Object_GEODATA_obs_pt_get},
{"LibreDWGc::Dwg_Object_GEODATA_scale_vec_set", _wrap_Dwg_Object_GEODATA_scale_vec_set},
{"LibreDWGc::Dwg_Object_GEODATA_scale_vec_get", _wrap_Dwg_Object_GEODATA_scale_vec_get},
{"LibreDWGc::Dwg_Object_GEODATA_unit_scale_horiz_set", _wrap_Dwg_Object_GEODATA_unit_scale_horiz_set},
{"LibreDWGc::Dwg_Object_GEODATA_unit_scale_horiz_get", _wrap_Dwg_Object_GEODATA_unit_scale_horiz_get},
{"LibreDWGc::Dwg_Object_GEODATA_units_value_horiz_set", _wrap_Dwg_Object_GEODATA_units_value_horiz_set},
{"LibreDWGc::Dwg_Object_GEODATA_units_value_horiz_get", _wrap_Dwg_Object_GEODATA_units_value_horiz_get},
{"LibreDWGc::Dwg_Object_GEODATA_unit_scale_vert_set", _wrap_Dwg_Object_GEODATA_unit_scale_vert_set},
{"LibreDWGc::Dwg_Object_GEODATA_unit_scale_vert_get", _wrap_Dwg_Object_GEODATA_unit_scale_vert_get},
{"LibreDWGc::Dwg_Object_GEODATA_units_value_vert_set", _wrap_Dwg_Object_GEODATA_units_value_vert_set},
{"LibreDWGc::Dwg_Object_GEODATA_units_value_vert_get", _wrap_Dwg_Object_GEODATA_units_value_vert_get},
{"LibreDWGc::Dwg_Object_GEODATA_up_dir_set", _wrap_Dwg_Object_GEODATA_up_dir_set},
{"LibreDWGc::Dwg_Object_GEODATA_up_dir_get", _wrap_Dwg_Object_GEODATA_up_dir_get},
{"LibreDWGc::Dwg_Object_GEODATA_north_dir_set", _wrap_Dwg_Object_GEODATA_north_dir_set},
{"LibreDWGc::Dwg_Object_GEODATA_north_dir_get", _wrap_Dwg_Object_GEODATA_north_dir_get},
{"LibreDWGc::Dwg_Object_GEODATA_scale_est_set", _wrap_Dwg_Object_GEODATA_scale_est_set},
{"LibreDWGc::Dwg_Object_GEODATA_scale_est_get", _wrap_Dwg_Object_GEODATA_scale_est_get},
{"LibreDWGc::Dwg_Object_GEODATA_user_scale_factor_set", _wrap_Dwg_Object_GEODATA_user_scale_factor_set},
{"LibreDWGc::Dwg_Object_GEODATA_user_scale_factor_get", _wrap_Dwg_Object_GEODATA_user_scale_factor_get},
{"LibreDWGc::Dwg_Object_GEODATA_do_sea_level_corr_set", _wrap_Dwg_Object_GEODATA_do_sea_level_corr_set},
{"LibreDWGc::Dwg_Object_GEODATA_do_sea_level_corr_get", _wrap_Dwg_Object_GEODATA_do_sea_level_corr_get},
{"LibreDWGc::Dwg_Object_GEODATA_sea_level_elev_set", _wrap_Dwg_Object_GEODATA_sea_level_elev_set},
{"LibreDWGc::Dwg_Object_GEODATA_sea_level_elev_get", _wrap_Dwg_Object_GEODATA_sea_level_elev_get},
{"LibreDWGc::Dwg_Object_GEODATA_coord_proj_radius_set", _wrap_Dwg_Object_GEODATA_coord_proj_radius_set},
{"LibreDWGc::Dwg_Object_GEODATA_coord_proj_radius_get", _wrap_Dwg_Object_GEODATA_coord_proj_radius_get},
{"LibreDWGc::Dwg_Object_GEODATA_coord_system_def_set", _wrap_Dwg_Object_GEODATA_coord_system_def_set},
{"LibreDWGc::Dwg_Object_GEODATA_coord_system_def_get", _wrap_Dwg_Object_GEODATA_coord_system_def_get},
{"LibreDWGc::Dwg_Object_GEODATA_geo_rss_tag_set", _wrap_Dwg_Object_GEODATA_geo_rss_tag_set},
{"LibreDWGc::Dwg_Object_GEODATA_geo_rss_tag_get", _wrap_Dwg_Object_GEODATA_geo_rss_tag_get},
{"LibreDWGc::Dwg_Object_GEODATA_coord_system_datum_set", _wrap_Dwg_Object_GEODATA_coord_system_datum_set},
{"LibreDWGc::Dwg_Object_GEODATA_coord_system_datum_get", _wrap_Dwg_Object_GEODATA_coord_system_datum_get},
{"LibreDWGc::Dwg_Object_GEODATA_coord_system_wkt_set", _wrap_Dwg_Object_GEODATA_coord_system_wkt_set},
{"LibreDWGc::Dwg_Object_GEODATA_coord_system_wkt_get", _wrap_Dwg_Object_GEODATA_coord_system_wkt_get},
{"LibreDWGc::Dwg_Object_GEODATA_observation_from_tag_set", _wrap_Dwg_Object_GEODATA_observation_from_tag_set},
{"LibreDWGc::Dwg_Object_GEODATA_observation_from_tag_get", _wrap_Dwg_Object_GEODATA_observation_from_tag_get},
{"LibreDWGc::Dwg_Object_GEODATA_observation_to_tag_set", _wrap_Dwg_Object_GEODATA_observation_to_tag_set},
{"LibreDWGc::Dwg_Object_GEODATA_observation_to_tag_get", _wrap_Dwg_Object_GEODATA_observation_to_tag_get},
{"LibreDWGc::Dwg_Object_GEODATA_observation_coverage_tag_set", _wrap_Dwg_Object_GEODATA_observation_coverage_tag_set},
{"LibreDWGc::Dwg_Object_GEODATA_observation_coverage_tag_get", _wrap_Dwg_Object_GEODATA_observation_coverage_tag_get},
{"LibreDWGc::Dwg_Object_GEODATA_num_geomesh_pts_set", _wrap_Dwg_Object_GEODATA_num_geomesh_pts_set},
{"LibreDWGc::Dwg_Object_GEODATA_num_geomesh_pts_get", _wrap_Dwg_Object_GEODATA_num_geomesh_pts_get},
{"LibreDWGc::Dwg_Object_GEODATA_geomesh_pts_set", _wrap_Dwg_Object_GEODATA_geomesh_pts_set},
{"LibreDWGc::Dwg_Object_GEODATA_geomesh_pts_get", _wrap_Dwg_Object_GEODATA_geomesh_pts_get},
{"LibreDWGc::Dwg_Object_GEODATA_num_geomesh_faces_set", _wrap_Dwg_Object_GEODATA_num_geomesh_faces_set},
{"LibreDWGc::Dwg_Object_GEODATA_num_geomesh_faces_get", _wrap_Dwg_Object_GEODATA_num_geomesh_faces_get},
{"LibreDWGc::Dwg_Object_GEODATA_geomesh_faces_set", _wrap_Dwg_Object_GEODATA_geomesh_faces_set},
{"LibreDWGc::Dwg_Object_GEODATA_geomesh_faces_get", _wrap_Dwg_Object_GEODATA_geomesh_faces_get},
{"LibreDWGc::Dwg_Object_GEODATA_has_civil_data_set", _wrap_Dwg_Object_GEODATA_has_civil_data_set},
{"LibreDWGc::Dwg_Object_GEODATA_has_civil_data_get", _wrap_Dwg_Object_GEODATA_has_civil_data_get},
{"LibreDWGc::Dwg_Object_GEODATA_obsolete_false_set", _wrap_Dwg_Object_GEODATA_obsolete_false_set},
{"LibreDWGc::Dwg_Object_GEODATA_obsolete_false_get", _wrap_Dwg_Object_GEODATA_obsolete_false_get},
{"LibreDWGc::Dwg_Object_GEODATA_ref_pt2d_set", _wrap_Dwg_Object_GEODATA_ref_pt2d_set},
{"LibreDWGc::Dwg_Object_GEODATA_ref_pt2d_get", _wrap_Dwg_Object_GEODATA_ref_pt2d_get},
{"LibreDWGc::Dwg_Object_GEODATA_zero1_set", _wrap_Dwg_Object_GEODATA_zero1_set},
{"LibreDWGc::Dwg_Object_GEODATA_zero1_get", _wrap_Dwg_Object_GEODATA_zero1_get},
{"LibreDWGc::Dwg_Object_GEODATA_zero2_set", _wrap_Dwg_Object_GEODATA_zero2_set},
{"LibreDWGc::Dwg_Object_GEODATA_zero2_get", _wrap_Dwg_Object_GEODATA_zero2_get},
{"LibreDWGc::Dwg_Object_GEODATA_unknown1_set", _wrap_Dwg_Object_GEODATA_unknown1_set},
{"LibreDWGc::Dwg_Object_GEODATA_unknown1_get", _wrap_Dwg_Object_GEODATA_unknown1_get},
{"LibreDWGc::Dwg_Object_GEODATA_unknown2_set", _wrap_Dwg_Object_GEODATA_unknown2_set},
{"LibreDWGc::Dwg_Object_GEODATA_unknown2_get", _wrap_Dwg_Object_GEODATA_unknown2_get},
{"LibreDWGc::Dwg_Object_GEODATA_unknown_b_set", _wrap_Dwg_Object_GEODATA_unknown_b_set},
{"LibreDWGc::Dwg_Object_GEODATA_unknown_b_get", _wrap_Dwg_Object_GEODATA_unknown_b_get},
{"LibreDWGc::Dwg_Object_GEODATA_north_dir_angle_deg_set", _wrap_Dwg_Object_GEODATA_north_dir_angle_deg_set},
{"LibreDWGc::Dwg_Object_GEODATA_north_dir_angle_deg_get", _wrap_Dwg_Object_GEODATA_north_dir_angle_deg_get},
{"LibreDWGc::Dwg_Object_GEODATA_north_dir_angle_rad_set", _wrap_Dwg_Object_GEODATA_north_dir_angle_rad_set},
{"LibreDWGc::Dwg_Object_GEODATA_north_dir_angle_rad_get", _wrap_Dwg_Object_GEODATA_north_dir_angle_rad_get},
{"LibreDWGc::new_Dwg_Object_GEODATA", _wrap_new_Dwg_Object_GEODATA},
{"LibreDWGc::delete_Dwg_Object_GEODATA", _wrap_delete_Dwg_Object_GEODATA},
{"LibreDWGc::Dwg_Object_IDBUFFER_parent_set", _wrap_Dwg_Object_IDBUFFER_parent_set},
{"LibreDWGc::Dwg_Object_IDBUFFER_parent_get", _wrap_Dwg_Object_IDBUFFER_parent_get},
{"LibreDWGc::Dwg_Object_IDBUFFER_unknown_set", _wrap_Dwg_Object_IDBUFFER_unknown_set},
{"LibreDWGc::Dwg_Object_IDBUFFER_unknown_get", _wrap_Dwg_Object_IDBUFFER_unknown_get},
{"LibreDWGc::Dwg_Object_IDBUFFER_num_obj_ids_set", _wrap_Dwg_Object_IDBUFFER_num_obj_ids_set},
{"LibreDWGc::Dwg_Object_IDBUFFER_num_obj_ids_get", _wrap_Dwg_Object_IDBUFFER_num_obj_ids_get},
{"LibreDWGc::Dwg_Object_IDBUFFER_obj_ids_set", _wrap_Dwg_Object_IDBUFFER_obj_ids_set},
{"LibreDWGc::Dwg_Object_IDBUFFER_obj_ids_get", _wrap_Dwg_Object_IDBUFFER_obj_ids_get},
{"LibreDWGc::new_Dwg_Object_IDBUFFER", _wrap_new_Dwg_Object_IDBUFFER},
{"LibreDWGc::delete_Dwg_Object_IDBUFFER", _wrap_delete_Dwg_Object_IDBUFFER},
{"LibreDWGc::Dwg_Entity_IMAGE_parent_set", _wrap_Dwg_Entity_IMAGE_parent_set},
{"LibreDWGc::Dwg_Entity_IMAGE_parent_get", _wrap_Dwg_Entity_IMAGE_parent_get},
{"LibreDWGc::Dwg_Entity_IMAGE_class_version_set", _wrap_Dwg_Entity_IMAGE_class_version_set},
{"LibreDWGc::Dwg_Entity_IMAGE_class_version_get", _wrap_Dwg_Entity_IMAGE_class_version_get},
{"LibreDWGc::Dwg_Entity_IMAGE_pt0_set", _wrap_Dwg_Entity_IMAGE_pt0_set},
{"LibreDWGc::Dwg_Entity_IMAGE_pt0_get", _wrap_Dwg_Entity_IMAGE_pt0_get},
{"LibreDWGc::Dwg_Entity_IMAGE_uvec_set", _wrap_Dwg_Entity_IMAGE_uvec_set},
{"LibreDWGc::Dwg_Entity_IMAGE_uvec_get", _wrap_Dwg_Entity_IMAGE_uvec_get},
{"LibreDWGc::Dwg_Entity_IMAGE_vvec_set", _wrap_Dwg_Entity_IMAGE_vvec_set},
{"LibreDWGc::Dwg_Entity_IMAGE_vvec_get", _wrap_Dwg_Entity_IMAGE_vvec_get},
{"LibreDWGc::Dwg_Entity_IMAGE_image_size_set", _wrap_Dwg_Entity_IMAGE_image_size_set},
{"LibreDWGc::Dwg_Entity_IMAGE_image_size_get", _wrap_Dwg_Entity_IMAGE_image_size_get},
{"LibreDWGc::Dwg_Entity_IMAGE_display_props_set", _wrap_Dwg_Entity_IMAGE_display_props_set},
{"LibreDWGc::Dwg_Entity_IMAGE_display_props_get", _wrap_Dwg_Entity_IMAGE_display_props_get},
{"LibreDWGc::Dwg_Entity_IMAGE_clipping_set", _wrap_Dwg_Entity_IMAGE_clipping_set},
{"LibreDWGc::Dwg_Entity_IMAGE_clipping_get", _wrap_Dwg_Entity_IMAGE_clipping_get},
{"LibreDWGc::Dwg_Entity_IMAGE_brightness_set", _wrap_Dwg_Entity_IMAGE_brightness_set},
{"LibreDWGc::Dwg_Entity_IMAGE_brightness_get", _wrap_Dwg_Entity_IMAGE_brightness_get},
{"LibreDWGc::Dwg_Entity_IMAGE_contrast_set", _wrap_Dwg_Entity_IMAGE_contrast_set},
{"LibreDWGc::Dwg_Entity_IMAGE_contrast_get", _wrap_Dwg_Entity_IMAGE_contrast_get},
{"LibreDWGc::Dwg_Entity_IMAGE_fade_set", _wrap_Dwg_Entity_IMAGE_fade_set},
{"LibreDWGc::Dwg_Entity_IMAGE_fade_get", _wrap_Dwg_Entity_IMAGE_fade_get},
{"LibreDWGc::Dwg_Entity_IMAGE_clip_mode_set", _wrap_Dwg_Entity_IMAGE_clip_mode_set},
{"LibreDWGc::Dwg_Entity_IMAGE_clip_mode_get", _wrap_Dwg_Entity_IMAGE_clip_mode_get},
{"LibreDWGc::Dwg_Entity_IMAGE_clip_boundary_type_set", _wrap_Dwg_Entity_IMAGE_clip_boundary_type_set},
{"LibreDWGc::Dwg_Entity_IMAGE_clip_boundary_type_get", _wrap_Dwg_Entity_IMAGE_clip_boundary_type_get},
{"LibreDWGc::Dwg_Entity_IMAGE_num_clip_verts_set", _wrap_Dwg_Entity_IMAGE_num_clip_verts_set},
{"LibreDWGc::Dwg_Entity_IMAGE_num_clip_verts_get", _wrap_Dwg_Entity_IMAGE_num_clip_verts_get},
{"LibreDWGc::Dwg_Entity_IMAGE_clip_verts_set", _wrap_Dwg_Entity_IMAGE_clip_verts_set},
{"LibreDWGc::Dwg_Entity_IMAGE_clip_verts_get", _wrap_Dwg_Entity_IMAGE_clip_verts_get},
{"LibreDWGc::Dwg_Entity_IMAGE_imagedef_set", _wrap_Dwg_Entity_IMAGE_imagedef_set},
{"LibreDWGc::Dwg_Entity_IMAGE_imagedef_get", _wrap_Dwg_Entity_IMAGE_imagedef_get},
{"LibreDWGc::Dwg_Entity_IMAGE_imagedefreactor_set", _wrap_Dwg_Entity_IMAGE_imagedefreactor_set},
{"LibreDWGc::Dwg_Entity_IMAGE_imagedefreactor_get", _wrap_Dwg_Entity_IMAGE_imagedefreactor_get},
{"LibreDWGc::new_Dwg_Entity_IMAGE", _wrap_new_Dwg_Entity_IMAGE},
{"LibreDWGc::delete_Dwg_Entity_IMAGE", _wrap_delete_Dwg_Entity_IMAGE},
{"LibreDWGc::Dwg_Object_IMAGEDEF_parent_set", _wrap_Dwg_Object_IMAGEDEF_parent_set},
{"LibreDWGc::Dwg_Object_IMAGEDEF_parent_get", _wrap_Dwg_Object_IMAGEDEF_parent_get},
{"LibreDWGc::Dwg_Object_IMAGEDEF_class_version_set", _wrap_Dwg_Object_IMAGEDEF_class_version_set},
{"LibreDWGc::Dwg_Object_IMAGEDEF_class_version_get", _wrap_Dwg_Object_IMAGEDEF_class_version_get},
{"LibreDWGc::Dwg_Object_IMAGEDEF_image_size_set", _wrap_Dwg_Object_IMAGEDEF_image_size_set},
{"LibreDWGc::Dwg_Object_IMAGEDEF_image_size_get", _wrap_Dwg_Object_IMAGEDEF_image_size_get},
{"LibreDWGc::Dwg_Object_IMAGEDEF_file_path_set", _wrap_Dwg_Object_IMAGEDEF_file_path_set},
{"LibreDWGc::Dwg_Object_IMAGEDEF_file_path_get", _wrap_Dwg_Object_IMAGEDEF_file_path_get},
{"LibreDWGc::Dwg_Object_IMAGEDEF_is_loaded_set", _wrap_Dwg_Object_IMAGEDEF_is_loaded_set},
{"LibreDWGc::Dwg_Object_IMAGEDEF_is_loaded_get", _wrap_Dwg_Object_IMAGEDEF_is_loaded_get},
{"LibreDWGc::Dwg_Object_IMAGEDEF_resunits_set", _wrap_Dwg_Object_IMAGEDEF_resunits_set},
{"LibreDWGc::Dwg_Object_IMAGEDEF_resunits_get", _wrap_Dwg_Object_IMAGEDEF_resunits_get},
{"LibreDWGc::Dwg_Object_IMAGEDEF_pixel_size_set", _wrap_Dwg_Object_IMAGEDEF_pixel_size_set},
{"LibreDWGc::Dwg_Object_IMAGEDEF_pixel_size_get", _wrap_Dwg_Object_IMAGEDEF_pixel_size_get},
{"LibreDWGc::new_Dwg_Object_IMAGEDEF", _wrap_new_Dwg_Object_IMAGEDEF},
{"LibreDWGc::delete_Dwg_Object_IMAGEDEF", _wrap_delete_Dwg_Object_IMAGEDEF},
{"LibreDWGc::Dwg_Object_IMAGEDEF_REACTOR_parent_set", _wrap_Dwg_Object_IMAGEDEF_REACTOR_parent_set},
{"LibreDWGc::Dwg_Object_IMAGEDEF_REACTOR_parent_get", _wrap_Dwg_Object_IMAGEDEF_REACTOR_parent_get},
{"LibreDWGc::Dwg_Object_IMAGEDEF_REACTOR_class_version_set", _wrap_Dwg_Object_IMAGEDEF_REACTOR_class_version_set},
{"LibreDWGc::Dwg_Object_IMAGEDEF_REACTOR_class_version_get", _wrap_Dwg_Object_IMAGEDEF_REACTOR_class_version_get},
{"LibreDWGc::new_Dwg_Object_IMAGEDEF_REACTOR", _wrap_new_Dwg_Object_IMAGEDEF_REACTOR},
{"LibreDWGc::delete_Dwg_Object_IMAGEDEF_REACTOR", _wrap_delete_Dwg_Object_IMAGEDEF_REACTOR},
{"LibreDWGc::Dwg_Object_INDEX_parent_set", _wrap_Dwg_Object_INDEX_parent_set},
{"LibreDWGc::Dwg_Object_INDEX_parent_get", _wrap_Dwg_Object_INDEX_parent_get},
{"LibreDWGc::Dwg_Object_INDEX_last_updated_set", _wrap_Dwg_Object_INDEX_last_updated_set},
{"LibreDWGc::Dwg_Object_INDEX_last_updated_get", _wrap_Dwg_Object_INDEX_last_updated_get},
{"LibreDWGc::new_Dwg_Object_INDEX", _wrap_new_Dwg_Object_INDEX},
{"LibreDWGc::delete_Dwg_Object_INDEX", _wrap_delete_Dwg_Object_INDEX},
{"LibreDWGc::Dwg_LAYER_entry_parent_set", _wrap_Dwg_LAYER_entry_parent_set},
{"LibreDWGc::Dwg_LAYER_entry_parent_get", _wrap_Dwg_LAYER_entry_parent_get},
{"LibreDWGc::Dwg_LAYER_entry_numlayers_set", _wrap_Dwg_LAYER_entry_numlayers_set},
{"LibreDWGc::Dwg_LAYER_entry_numlayers_get", _wrap_Dwg_LAYER_entry_numlayers_get},
{"LibreDWGc::Dwg_LAYER_entry_name_set", _wrap_Dwg_LAYER_entry_name_set},
{"LibreDWGc::Dwg_LAYER_entry_name_get", _wrap_Dwg_LAYER_entry_name_get},
{"LibreDWGc::Dwg_LAYER_entry_handle_set", _wrap_Dwg_LAYER_entry_handle_set},
{"LibreDWGc::Dwg_LAYER_entry_handle_get", _wrap_Dwg_LAYER_entry_handle_get},
{"LibreDWGc::new_Dwg_LAYER_entry", _wrap_new_Dwg_LAYER_entry},
{"LibreDWGc::delete_Dwg_LAYER_entry", _wrap_delete_Dwg_LAYER_entry},
{"LibreDWGc::Dwg_Object_LAYER_INDEX_parent_set", _wrap_Dwg_Object_LAYER_INDEX_parent_set},
{"LibreDWGc::Dwg_Object_LAYER_INDEX_parent_get", _wrap_Dwg_Object_LAYER_INDEX_parent_get},
{"LibreDWGc::Dwg_Object_LAYER_INDEX_last_updated_set", _wrap_Dwg_Object_LAYER_INDEX_last_updated_set},
{"LibreDWGc::Dwg_Object_LAYER_INDEX_last_updated_get", _wrap_Dwg_Object_LAYER_INDEX_last_updated_get},
{"LibreDWGc::Dwg_Object_LAYER_INDEX_num_entries_set", _wrap_Dwg_Object_LAYER_INDEX_num_entries_set},
{"LibreDWGc::Dwg_Object_LAYER_INDEX_num_entries_get", _wrap_Dwg_Object_LAYER_INDEX_num_entries_get},
{"LibreDWGc::Dwg_Object_LAYER_INDEX_entries_set", _wrap_Dwg_Object_LAYER_INDEX_entries_set},
{"LibreDWGc::Dwg_Object_LAYER_INDEX_entries_get", _wrap_Dwg_Object_LAYER_INDEX_entries_get},
{"LibreDWGc::new_Dwg_Object_LAYER_INDEX", _wrap_new_Dwg_Object_LAYER_INDEX},
{"LibreDWGc::delete_Dwg_Object_LAYER_INDEX", _wrap_delete_Dwg_Object_LAYER_INDEX},
{"LibreDWGc::Dwg_Entity_LWPOLYLINE_parent_set", _wrap_Dwg_Entity_LWPOLYLINE_parent_set},
{"LibreDWGc::Dwg_Entity_LWPOLYLINE_parent_get", _wrap_Dwg_Entity_LWPOLYLINE_parent_get},
{"LibreDWGc::Dwg_Entity_LWPOLYLINE_flag_set", _wrap_Dwg_Entity_LWPOLYLINE_flag_set},
{"LibreDWGc::Dwg_Entity_LWPOLYLINE_flag_get", _wrap_Dwg_Entity_LWPOLYLINE_flag_get},
{"LibreDWGc::Dwg_Entity_LWPOLYLINE_const_width_set", _wrap_Dwg_Entity_LWPOLYLINE_const_width_set},
{"LibreDWGc::Dwg_Entity_LWPOLYLINE_const_width_get", _wrap_Dwg_Entity_LWPOLYLINE_const_width_get},
{"LibreDWGc::Dwg_Entity_LWPOLYLINE_elevation_set", _wrap_Dwg_Entity_LWPOLYLINE_elevation_set},
{"LibreDWGc::Dwg_Entity_LWPOLYLINE_elevation_get", _wrap_Dwg_Entity_LWPOLYLINE_elevation_get},
{"LibreDWGc::Dwg_Entity_LWPOLYLINE_thickness_set", _wrap_Dwg_Entity_LWPOLYLINE_thickness_set},
{"LibreDWGc::Dwg_Entity_LWPOLYLINE_thickness_get", _wrap_Dwg_Entity_LWPOLYLINE_thickness_get},
{"LibreDWGc::Dwg_Entity_LWPOLYLINE_extrusion_set", _wrap_Dwg_Entity_LWPOLYLINE_extrusion_set},
{"LibreDWGc::Dwg_Entity_LWPOLYLINE_extrusion_get", _wrap_Dwg_Entity_LWPOLYLINE_extrusion_get},
{"LibreDWGc::Dwg_Entity_LWPOLYLINE_num_points_set", _wrap_Dwg_Entity_LWPOLYLINE_num_points_set},
{"LibreDWGc::Dwg_Entity_LWPOLYLINE_num_points_get", _wrap_Dwg_Entity_LWPOLYLINE_num_points_get},
{"LibreDWGc::Dwg_Entity_LWPOLYLINE_points_set", _wrap_Dwg_Entity_LWPOLYLINE_points_set},
{"LibreDWGc::Dwg_Entity_LWPOLYLINE_points_get", _wrap_Dwg_Entity_LWPOLYLINE_points_get},
{"LibreDWGc::Dwg_Entity_LWPOLYLINE_num_bulges_set", _wrap_Dwg_Entity_LWPOLYLINE_num_bulges_set},
{"LibreDWGc::Dwg_Entity_LWPOLYLINE_num_bulges_get", _wrap_Dwg_Entity_LWPOLYLINE_num_bulges_get},
{"LibreDWGc::Dwg_Entity_LWPOLYLINE_bulges_set", _wrap_Dwg_Entity_LWPOLYLINE_bulges_set},
{"LibreDWGc::Dwg_Entity_LWPOLYLINE_bulges_get", _wrap_Dwg_Entity_LWPOLYLINE_bulges_get},
{"LibreDWGc::Dwg_Entity_LWPOLYLINE_num_vertexids_set", _wrap_Dwg_Entity_LWPOLYLINE_num_vertexids_set},
{"LibreDWGc::Dwg_Entity_LWPOLYLINE_num_vertexids_get", _wrap_Dwg_Entity_LWPOLYLINE_num_vertexids_get},
{"LibreDWGc::Dwg_Entity_LWPOLYLINE_vertexids_set", _wrap_Dwg_Entity_LWPOLYLINE_vertexids_set},
{"LibreDWGc::Dwg_Entity_LWPOLYLINE_vertexids_get", _wrap_Dwg_Entity_LWPOLYLINE_vertexids_get},
{"LibreDWGc::Dwg_Entity_LWPOLYLINE_num_widths_set", _wrap_Dwg_Entity_LWPOLYLINE_num_widths_set},
{"LibreDWGc::Dwg_Entity_LWPOLYLINE_num_widths_get", _wrap_Dwg_Entity_LWPOLYLINE_num_widths_get},
{"LibreDWGc::Dwg_Entity_LWPOLYLINE_widths_set", _wrap_Dwg_Entity_LWPOLYLINE_widths_set},
{"LibreDWGc::Dwg_Entity_LWPOLYLINE_widths_get", _wrap_Dwg_Entity_LWPOLYLINE_widths_get},
{"LibreDWGc::new_Dwg_Entity_LWPOLYLINE", _wrap_new_Dwg_Entity_LWPOLYLINE},
{"LibreDWGc::delete_Dwg_Entity_LWPOLYLINE", _wrap_delete_Dwg_Entity_LWPOLYLINE},
{"LibreDWGc::Dwg_Object_RASTERVARIABLES_parent_set", _wrap_Dwg_Object_RASTERVARIABLES_parent_set},
{"LibreDWGc::Dwg_Object_RASTERVARIABLES_parent_get", _wrap_Dwg_Object_RASTERVARIABLES_parent_get},
{"LibreDWGc::Dwg_Object_RASTERVARIABLES_class_version_set", _wrap_Dwg_Object_RASTERVARIABLES_class_version_set},
{"LibreDWGc::Dwg_Object_RASTERVARIABLES_class_version_get", _wrap_Dwg_Object_RASTERVARIABLES_class_version_get},
{"LibreDWGc::Dwg_Object_RASTERVARIABLES_image_frame_set", _wrap_Dwg_Object_RASTERVARIABLES_image_frame_set},
{"LibreDWGc::Dwg_Object_RASTERVARIABLES_image_frame_get", _wrap_Dwg_Object_RASTERVARIABLES_image_frame_get},
{"LibreDWGc::Dwg_Object_RASTERVARIABLES_image_quality_set", _wrap_Dwg_Object_RASTERVARIABLES_image_quality_set},
{"LibreDWGc::Dwg_Object_RASTERVARIABLES_image_quality_get", _wrap_Dwg_Object_RASTERVARIABLES_image_quality_get},
{"LibreDWGc::Dwg_Object_RASTERVARIABLES_units_set", _wrap_Dwg_Object_RASTERVARIABLES_units_set},
{"LibreDWGc::Dwg_Object_RASTERVARIABLES_units_get", _wrap_Dwg_Object_RASTERVARIABLES_units_get},
{"LibreDWGc::new_Dwg_Object_RASTERVARIABLES", _wrap_new_Dwg_Object_RASTERVARIABLES},
{"LibreDWGc::delete_Dwg_Object_RASTERVARIABLES", _wrap_delete_Dwg_Object_RASTERVARIABLES},
{"LibreDWGc::Dwg_Object_SCALE_parent_set", _wrap_Dwg_Object_SCALE_parent_set},
{"LibreDWGc::Dwg_Object_SCALE_parent_get", _wrap_Dwg_Object_SCALE_parent_get},
{"LibreDWGc::Dwg_Object_SCALE_flag_set", _wrap_Dwg_Object_SCALE_flag_set},
{"LibreDWGc::Dwg_Object_SCALE_flag_get", _wrap_Dwg_Object_SCALE_flag_get},
{"LibreDWGc::Dwg_Object_SCALE_name_set", _wrap_Dwg_Object_SCALE_name_set},
{"LibreDWGc::Dwg_Object_SCALE_name_get", _wrap_Dwg_Object_SCALE_name_get},
{"LibreDWGc::Dwg_Object_SCALE_paper_units_set", _wrap_Dwg_Object_SCALE_paper_units_set},
{"LibreDWGc::Dwg_Object_SCALE_paper_units_get", _wrap_Dwg_Object_SCALE_paper_units_get},
{"LibreDWGc::Dwg_Object_SCALE_drawing_units_set", _wrap_Dwg_Object_SCALE_drawing_units_set},
{"LibreDWGc::Dwg_Object_SCALE_drawing_units_get", _wrap_Dwg_Object_SCALE_drawing_units_get},
{"LibreDWGc::Dwg_Object_SCALE_is_unit_scale_set", _wrap_Dwg_Object_SCALE_is_unit_scale_set},
{"LibreDWGc::Dwg_Object_SCALE_is_unit_scale_get", _wrap_Dwg_Object_SCALE_is_unit_scale_get},
{"LibreDWGc::new_Dwg_Object_SCALE", _wrap_new_Dwg_Object_SCALE},
{"LibreDWGc::delete_Dwg_Object_SCALE", _wrap_delete_Dwg_Object_SCALE},
{"LibreDWGc::Dwg_Object_SORTENTSTABLE_parent_set", _wrap_Dwg_Object_SORTENTSTABLE_parent_set},
{"LibreDWGc::Dwg_Object_SORTENTSTABLE_parent_get", _wrap_Dwg_Object_SORTENTSTABLE_parent_get},
{"LibreDWGc::Dwg_Object_SORTENTSTABLE_num_ents_set", _wrap_Dwg_Object_SORTENTSTABLE_num_ents_set},
{"LibreDWGc::Dwg_Object_SORTENTSTABLE_num_ents_get", _wrap_Dwg_Object_SORTENTSTABLE_num_ents_get},
{"LibreDWGc::Dwg_Object_SORTENTSTABLE_sort_ents_set", _wrap_Dwg_Object_SORTENTSTABLE_sort_ents_set},
{"LibreDWGc::Dwg_Object_SORTENTSTABLE_sort_ents_get", _wrap_Dwg_Object_SORTENTSTABLE_sort_ents_get},
{"LibreDWGc::Dwg_Object_SORTENTSTABLE_block_owner_set", _wrap_Dwg_Object_SORTENTSTABLE_block_owner_set},
{"LibreDWGc::Dwg_Object_SORTENTSTABLE_block_owner_get", _wrap_Dwg_Object_SORTENTSTABLE_block_owner_get},
{"LibreDWGc::Dwg_Object_SORTENTSTABLE_ents_set", _wrap_Dwg_Object_SORTENTSTABLE_ents_set},
{"LibreDWGc::Dwg_Object_SORTENTSTABLE_ents_get", _wrap_Dwg_Object_SORTENTSTABLE_ents_get},
{"LibreDWGc::new_Dwg_Object_SORTENTSTABLE", _wrap_new_Dwg_Object_SORTENTSTABLE},
{"LibreDWGc::delete_Dwg_Object_SORTENTSTABLE", _wrap_delete_Dwg_Object_SORTENTSTABLE},
{"LibreDWGc::Dwg_Object_SPATIAL_FILTER_parent_set", _wrap_Dwg_Object_SPATIAL_FILTER_parent_set},
{"LibreDWGc::Dwg_Object_SPATIAL_FILTER_parent_get", _wrap_Dwg_Object_SPATIAL_FILTER_parent_get},
{"LibreDWGc::Dwg_Object_SPATIAL_FILTER_num_clip_verts_set", _wrap_Dwg_Object_SPATIAL_FILTER_num_clip_verts_set},
{"LibreDWGc::Dwg_Object_SPATIAL_FILTER_num_clip_verts_get", _wrap_Dwg_Object_SPATIAL_FILTER_num_clip_verts_get},
{"LibreDWGc::Dwg_Object_SPATIAL_FILTER_clip_verts_set", _wrap_Dwg_Object_SPATIAL_FILTER_clip_verts_set},
{"LibreDWGc::Dwg_Object_SPATIAL_FILTER_clip_verts_get", _wrap_Dwg_Object_SPATIAL_FILTER_clip_verts_get},
{"LibreDWGc::Dwg_Object_SPATIAL_FILTER_extrusion_set", _wrap_Dwg_Object_SPATIAL_FILTER_extrusion_set},
{"LibreDWGc::Dwg_Object_SPATIAL_FILTER_extrusion_get", _wrap_Dwg_Object_SPATIAL_FILTER_extrusion_get},
{"LibreDWGc::Dwg_Object_SPATIAL_FILTER_origin_set", _wrap_Dwg_Object_SPATIAL_FILTER_origin_set},
{"LibreDWGc::Dwg_Object_SPATIAL_FILTER_origin_get", _wrap_Dwg_Object_SPATIAL_FILTER_origin_get},
{"LibreDWGc::Dwg_Object_SPATIAL_FILTER_display_boundary_on_set", _wrap_Dwg_Object_SPATIAL_FILTER_display_boundary_on_set},
{"LibreDWGc::Dwg_Object_SPATIAL_FILTER_display_boundary_on_get", _wrap_Dwg_Object_SPATIAL_FILTER_display_boundary_on_get},
{"LibreDWGc::Dwg_Object_SPATIAL_FILTER_front_clip_on_set", _wrap_Dwg_Object_SPATIAL_FILTER_front_clip_on_set},
{"LibreDWGc::Dwg_Object_SPATIAL_FILTER_front_clip_on_get", _wrap_Dwg_Object_SPATIAL_FILTER_front_clip_on_get},
{"LibreDWGc::Dwg_Object_SPATIAL_FILTER_front_clip_z_set", _wrap_Dwg_Object_SPATIAL_FILTER_front_clip_z_set},
{"LibreDWGc::Dwg_Object_SPATIAL_FILTER_front_clip_z_get", _wrap_Dwg_Object_SPATIAL_FILTER_front_clip_z_get},
{"LibreDWGc::Dwg_Object_SPATIAL_FILTER_back_clip_on_set", _wrap_Dwg_Object_SPATIAL_FILTER_back_clip_on_set},
{"LibreDWGc::Dwg_Object_SPATIAL_FILTER_back_clip_on_get", _wrap_Dwg_Object_SPATIAL_FILTER_back_clip_on_get},
{"LibreDWGc::Dwg_Object_SPATIAL_FILTER_back_clip_z_set", _wrap_Dwg_Object_SPATIAL_FILTER_back_clip_z_set},
{"LibreDWGc::Dwg_Object_SPATIAL_FILTER_back_clip_z_get", _wrap_Dwg_Object_SPATIAL_FILTER_back_clip_z_get},
{"LibreDWGc::Dwg_Object_SPATIAL_FILTER_inverse_transform_set", _wrap_Dwg_Object_SPATIAL_FILTER_inverse_transform_set},
{"LibreDWGc::Dwg_Object_SPATIAL_FILTER_inverse_transform_get", _wrap_Dwg_Object_SPATIAL_FILTER_inverse_transform_get},
{"LibreDWGc::Dwg_Object_SPATIAL_FILTER_transform_set", _wrap_Dwg_Object_SPATIAL_FILTER_transform_set},
{"LibreDWGc::Dwg_Object_SPATIAL_FILTER_transform_get", _wrap_Dwg_Object_SPATIAL_FILTER_transform_get},
{"LibreDWGc::new_Dwg_Object_SPATIAL_FILTER", _wrap_new_Dwg_Object_SPATIAL_FILTER},
{"LibreDWGc::delete_Dwg_Object_SPATIAL_FILTER", _wrap_delete_Dwg_Object_SPATIAL_FILTER},
{"LibreDWGc::Dwg_Object_SPATIAL_INDEX_parent_set", _wrap_Dwg_Object_SPATIAL_INDEX_parent_set},
{"LibreDWGc::Dwg_Object_SPATIAL_INDEX_parent_get", _wrap_Dwg_Object_SPATIAL_INDEX_parent_get},
{"LibreDWGc::Dwg_Object_SPATIAL_INDEX_last_updated_set", _wrap_Dwg_Object_SPATIAL_INDEX_last_updated_set},
{"LibreDWGc::Dwg_Object_SPATIAL_INDEX_last_updated_get", _wrap_Dwg_Object_SPATIAL_INDEX_last_updated_get},
{"LibreDWGc::Dwg_Object_SPATIAL_INDEX_num1_set", _wrap_Dwg_Object_SPATIAL_INDEX_num1_set},
{"LibreDWGc::Dwg_Object_SPATIAL_INDEX_num1_get", _wrap_Dwg_Object_SPATIAL_INDEX_num1_get},
{"LibreDWGc::Dwg_Object_SPATIAL_INDEX_num2_set", _wrap_Dwg_Object_SPATIAL_INDEX_num2_set},
{"LibreDWGc::Dwg_Object_SPATIAL_INDEX_num2_get", _wrap_Dwg_Object_SPATIAL_INDEX_num2_get},
{"LibreDWGc::Dwg_Object_SPATIAL_INDEX_num3_set", _wrap_Dwg_Object_SPATIAL_INDEX_num3_set},
{"LibreDWGc::Dwg_Object_SPATIAL_INDEX_num3_get", _wrap_Dwg_Object_SPATIAL_INDEX_num3_get},
{"LibreDWGc::Dwg_Object_SPATIAL_INDEX_num4_set", _wrap_Dwg_Object_SPATIAL_INDEX_num4_set},
{"LibreDWGc::Dwg_Object_SPATIAL_INDEX_num4_get", _wrap_Dwg_Object_SPATIAL_INDEX_num4_get},
{"LibreDWGc::Dwg_Object_SPATIAL_INDEX_num5_set", _wrap_Dwg_Object_SPATIAL_INDEX_num5_set},
{"LibreDWGc::Dwg_Object_SPATIAL_INDEX_num5_get", _wrap_Dwg_Object_SPATIAL_INDEX_num5_get},
{"LibreDWGc::Dwg_Object_SPATIAL_INDEX_num6_set", _wrap_Dwg_Object_SPATIAL_INDEX_num6_set},
{"LibreDWGc::Dwg_Object_SPATIAL_INDEX_num6_get", _wrap_Dwg_Object_SPATIAL_INDEX_num6_get},
{"LibreDWGc::Dwg_Object_SPATIAL_INDEX_num_hdls_set", _wrap_Dwg_Object_SPATIAL_INDEX_num_hdls_set},
{"LibreDWGc::Dwg_Object_SPATIAL_INDEX_num_hdls_get", _wrap_Dwg_Object_SPATIAL_INDEX_num_hdls_get},
{"LibreDWGc::Dwg_Object_SPATIAL_INDEX_hdls_set", _wrap_Dwg_Object_SPATIAL_INDEX_hdls_set},
{"LibreDWGc::Dwg_Object_SPATIAL_INDEX_hdls_get", _wrap_Dwg_Object_SPATIAL_INDEX_hdls_get},
{"LibreDWGc::Dwg_Object_SPATIAL_INDEX_bindata_size_set", _wrap_Dwg_Object_SPATIAL_INDEX_bindata_size_set},
{"LibreDWGc::Dwg_Object_SPATIAL_INDEX_bindata_size_get", _wrap_Dwg_Object_SPATIAL_INDEX_bindata_size_get},
{"LibreDWGc::Dwg_Object_SPATIAL_INDEX_bindata_set", _wrap_Dwg_Object_SPATIAL_INDEX_bindata_set},
{"LibreDWGc::Dwg_Object_SPATIAL_INDEX_bindata_get", _wrap_Dwg_Object_SPATIAL_INDEX_bindata_get},
{"LibreDWGc::new_Dwg_Object_SPATIAL_INDEX", _wrap_new_Dwg_Object_SPATIAL_INDEX},
{"LibreDWGc::delete_Dwg_Object_SPATIAL_INDEX", _wrap_delete_Dwg_Object_SPATIAL_INDEX},
{"LibreDWGc::Dwg_Entity_WIPEOUT_parent_set", _wrap_Dwg_Entity_WIPEOUT_parent_set},
{"LibreDWGc::Dwg_Entity_WIPEOUT_parent_get", _wrap_Dwg_Entity_WIPEOUT_parent_get},
{"LibreDWGc::Dwg_Entity_WIPEOUT_class_version_set", _wrap_Dwg_Entity_WIPEOUT_class_version_set},
{"LibreDWGc::Dwg_Entity_WIPEOUT_class_version_get", _wrap_Dwg_Entity_WIPEOUT_class_version_get},
{"LibreDWGc::Dwg_Entity_WIPEOUT_pt0_set", _wrap_Dwg_Entity_WIPEOUT_pt0_set},
{"LibreDWGc::Dwg_Entity_WIPEOUT_pt0_get", _wrap_Dwg_Entity_WIPEOUT_pt0_get},
{"LibreDWGc::Dwg_Entity_WIPEOUT_uvec_set", _wrap_Dwg_Entity_WIPEOUT_uvec_set},
{"LibreDWGc::Dwg_Entity_WIPEOUT_uvec_get", _wrap_Dwg_Entity_WIPEOUT_uvec_get},
{"LibreDWGc::Dwg_Entity_WIPEOUT_vvec_set", _wrap_Dwg_Entity_WIPEOUT_vvec_set},
{"LibreDWGc::Dwg_Entity_WIPEOUT_vvec_get", _wrap_Dwg_Entity_WIPEOUT_vvec_get},
{"LibreDWGc::Dwg_Entity_WIPEOUT_image_size_set", _wrap_Dwg_Entity_WIPEOUT_image_size_set},
{"LibreDWGc::Dwg_Entity_WIPEOUT_image_size_get", _wrap_Dwg_Entity_WIPEOUT_image_size_get},
{"LibreDWGc::Dwg_Entity_WIPEOUT_display_props_set", _wrap_Dwg_Entity_WIPEOUT_display_props_set},
{"LibreDWGc::Dwg_Entity_WIPEOUT_display_props_get", _wrap_Dwg_Entity_WIPEOUT_display_props_get},
{"LibreDWGc::Dwg_Entity_WIPEOUT_clipping_set", _wrap_Dwg_Entity_WIPEOUT_clipping_set},
{"LibreDWGc::Dwg_Entity_WIPEOUT_clipping_get", _wrap_Dwg_Entity_WIPEOUT_clipping_get},
{"LibreDWGc::Dwg_Entity_WIPEOUT_brightness_set", _wrap_Dwg_Entity_WIPEOUT_brightness_set},
{"LibreDWGc::Dwg_Entity_WIPEOUT_brightness_get", _wrap_Dwg_Entity_WIPEOUT_brightness_get},
{"LibreDWGc::Dwg_Entity_WIPEOUT_contrast_set", _wrap_Dwg_Entity_WIPEOUT_contrast_set},
{"LibreDWGc::Dwg_Entity_WIPEOUT_contrast_get", _wrap_Dwg_Entity_WIPEOUT_contrast_get},
{"LibreDWGc::Dwg_Entity_WIPEOUT_fade_set", _wrap_Dwg_Entity_WIPEOUT_fade_set},
{"LibreDWGc::Dwg_Entity_WIPEOUT_fade_get", _wrap_Dwg_Entity_WIPEOUT_fade_get},
{"LibreDWGc::Dwg_Entity_WIPEOUT_clip_mode_set", _wrap_Dwg_Entity_WIPEOUT_clip_mode_set},
{"LibreDWGc::Dwg_Entity_WIPEOUT_clip_mode_get", _wrap_Dwg_Entity_WIPEOUT_clip_mode_get},
{"LibreDWGc::Dwg_Entity_WIPEOUT_clip_boundary_type_set", _wrap_Dwg_Entity_WIPEOUT_clip_boundary_type_set},
{"LibreDWGc::Dwg_Entity_WIPEOUT_clip_boundary_type_get", _wrap_Dwg_Entity_WIPEOUT_clip_boundary_type_get},
{"LibreDWGc::Dwg_Entity_WIPEOUT_num_clip_verts_set", _wrap_Dwg_Entity_WIPEOUT_num_clip_verts_set},
{"LibreDWGc::Dwg_Entity_WIPEOUT_num_clip_verts_get", _wrap_Dwg_Entity_WIPEOUT_num_clip_verts_get},
{"LibreDWGc::Dwg_Entity_WIPEOUT_clip_verts_set", _wrap_Dwg_Entity_WIPEOUT_clip_verts_set},
{"LibreDWGc::Dwg_Entity_WIPEOUT_clip_verts_get", _wrap_Dwg_Entity_WIPEOUT_clip_verts_get},
{"LibreDWGc::Dwg_Entity_WIPEOUT_imagedef_set", _wrap_Dwg_Entity_WIPEOUT_imagedef_set},
{"LibreDWGc::Dwg_Entity_WIPEOUT_imagedef_get", _wrap_Dwg_Entity_WIPEOUT_imagedef_get},
{"LibreDWGc::Dwg_Entity_WIPEOUT_imagedefreactor_set", _wrap_Dwg_Entity_WIPEOUT_imagedefreactor_set},
{"LibreDWGc::Dwg_Entity_WIPEOUT_imagedefreactor_get", _wrap_Dwg_Entity_WIPEOUT_imagedefreactor_get},
{"LibreDWGc::new_Dwg_Entity_WIPEOUT", _wrap_new_Dwg_Entity_WIPEOUT},
{"LibreDWGc::delete_Dwg_Entity_WIPEOUT", _wrap_delete_Dwg_Entity_WIPEOUT},
{"LibreDWGc::Dwg_Object_WIPEOUTVARIABLES_parent_set", _wrap_Dwg_Object_WIPEOUTVARIABLES_parent_set},
{"LibreDWGc::Dwg_Object_WIPEOUTVARIABLES_parent_get", _wrap_Dwg_Object_WIPEOUTVARIABLES_parent_get},
{"LibreDWGc::Dwg_Object_WIPEOUTVARIABLES_display_frame_set", _wrap_Dwg_Object_WIPEOUTVARIABLES_display_frame_set},
{"LibreDWGc::Dwg_Object_WIPEOUTVARIABLES_display_frame_get", _wrap_Dwg_Object_WIPEOUTVARIABLES_display_frame_get},
{"LibreDWGc::new_Dwg_Object_WIPEOUTVARIABLES", _wrap_new_Dwg_Object_WIPEOUTVARIABLES},
{"LibreDWGc::delete_Dwg_Object_WIPEOUTVARIABLES", _wrap_delete_Dwg_Object_WIPEOUTVARIABLES},
{"LibreDWGc::Dwg_Entity_SECTIONOBJECT_parent_set", _wrap_Dwg_Entity_SECTIONOBJECT_parent_set},
{"LibreDWGc::Dwg_Entity_SECTIONOBJECT_parent_get", _wrap_Dwg_Entity_SECTIONOBJECT_parent_get},
{"LibreDWGc::Dwg_Entity_SECTIONOBJECT_state_set", _wrap_Dwg_Entity_SECTIONOBJECT_state_set},
{"LibreDWGc::Dwg_Entity_SECTIONOBJECT_state_get", _wrap_Dwg_Entity_SECTIONOBJECT_state_get},
{"LibreDWGc::Dwg_Entity_SECTIONOBJECT_flags_set", _wrap_Dwg_Entity_SECTIONOBJECT_flags_set},
{"LibreDWGc::Dwg_Entity_SECTIONOBJECT_flags_get", _wrap_Dwg_Entity_SECTIONOBJECT_flags_get},
{"LibreDWGc::Dwg_Entity_SECTIONOBJECT_name_set", _wrap_Dwg_Entity_SECTIONOBJECT_name_set},
{"LibreDWGc::Dwg_Entity_SECTIONOBJECT_name_get", _wrap_Dwg_Entity_SECTIONOBJECT_name_get},
{"LibreDWGc::Dwg_Entity_SECTIONOBJECT_vert_dir_set", _wrap_Dwg_Entity_SECTIONOBJECT_vert_dir_set},
{"LibreDWGc::Dwg_Entity_SECTIONOBJECT_vert_dir_get", _wrap_Dwg_Entity_SECTIONOBJECT_vert_dir_get},
{"LibreDWGc::Dwg_Entity_SECTIONOBJECT_top_height_set", _wrap_Dwg_Entity_SECTIONOBJECT_top_height_set},
{"LibreDWGc::Dwg_Entity_SECTIONOBJECT_top_height_get", _wrap_Dwg_Entity_SECTIONOBJECT_top_height_get},
{"LibreDWGc::Dwg_Entity_SECTIONOBJECT_bottom_height_set", _wrap_Dwg_Entity_SECTIONOBJECT_bottom_height_set},
{"LibreDWGc::Dwg_Entity_SECTIONOBJECT_bottom_height_get", _wrap_Dwg_Entity_SECTIONOBJECT_bottom_height_get},
{"LibreDWGc::Dwg_Entity_SECTIONOBJECT_indicator_alpha_set", _wrap_Dwg_Entity_SECTIONOBJECT_indicator_alpha_set},
{"LibreDWGc::Dwg_Entity_SECTIONOBJECT_indicator_alpha_get", _wrap_Dwg_Entity_SECTIONOBJECT_indicator_alpha_get},
{"LibreDWGc::Dwg_Entity_SECTIONOBJECT_indicator_color_set", _wrap_Dwg_Entity_SECTIONOBJECT_indicator_color_set},
{"LibreDWGc::Dwg_Entity_SECTIONOBJECT_indicator_color_get", _wrap_Dwg_Entity_SECTIONOBJECT_indicator_color_get},
{"LibreDWGc::Dwg_Entity_SECTIONOBJECT_num_verts_set", _wrap_Dwg_Entity_SECTIONOBJECT_num_verts_set},
{"LibreDWGc::Dwg_Entity_SECTIONOBJECT_num_verts_get", _wrap_Dwg_Entity_SECTIONOBJECT_num_verts_get},
{"LibreDWGc::Dwg_Entity_SECTIONOBJECT_verts_set", _wrap_Dwg_Entity_SECTIONOBJECT_verts_set},
{"LibreDWGc::Dwg_Entity_SECTIONOBJECT_verts_get", _wrap_Dwg_Entity_SECTIONOBJECT_verts_get},
{"LibreDWGc::Dwg_Entity_SECTIONOBJECT_num_blverts_set", _wrap_Dwg_Entity_SECTIONOBJECT_num_blverts_set},
{"LibreDWGc::Dwg_Entity_SECTIONOBJECT_num_blverts_get", _wrap_Dwg_Entity_SECTIONOBJECT_num_blverts_get},
{"LibreDWGc::Dwg_Entity_SECTIONOBJECT_blverts_set", _wrap_Dwg_Entity_SECTIONOBJECT_blverts_set},
{"LibreDWGc::Dwg_Entity_SECTIONOBJECT_blverts_get", _wrap_Dwg_Entity_SECTIONOBJECT_blverts_get},
{"LibreDWGc::Dwg_Entity_SECTIONOBJECT_section_settings_set", _wrap_Dwg_Entity_SECTIONOBJECT_section_settings_set},
{"LibreDWGc::Dwg_Entity_SECTIONOBJECT_section_settings_get", _wrap_Dwg_Entity_SECTIONOBJECT_section_settings_get},
{"LibreDWGc::new_Dwg_Entity_SECTIONOBJECT", _wrap_new_Dwg_Entity_SECTIONOBJECT},
{"LibreDWGc::delete_Dwg_Entity_SECTIONOBJECT", _wrap_delete_Dwg_Entity_SECTIONOBJECT},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_parent_set", _wrap_Dwg_Object_VISUALSTYLE_parent_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_parent_get", _wrap_Dwg_Object_VISUALSTYLE_parent_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_description_set", _wrap_Dwg_Object_VISUALSTYLE_description_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_description_get", _wrap_Dwg_Object_VISUALSTYLE_description_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_style_type_set", _wrap_Dwg_Object_VISUALSTYLE_style_type_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_style_type_get", _wrap_Dwg_Object_VISUALSTYLE_style_type_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_ext_lighting_model_set", _wrap_Dwg_Object_VISUALSTYLE_ext_lighting_model_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_ext_lighting_model_get", _wrap_Dwg_Object_VISUALSTYLE_ext_lighting_model_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_internal_only_set", _wrap_Dwg_Object_VISUALSTYLE_internal_only_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_internal_only_get", _wrap_Dwg_Object_VISUALSTYLE_internal_only_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_face_lighting_model_set", _wrap_Dwg_Object_VISUALSTYLE_face_lighting_model_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_face_lighting_model_get", _wrap_Dwg_Object_VISUALSTYLE_face_lighting_model_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_face_lighting_model_int_set", _wrap_Dwg_Object_VISUALSTYLE_face_lighting_model_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_face_lighting_model_int_get", _wrap_Dwg_Object_VISUALSTYLE_face_lighting_model_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_face_lighting_quality_set", _wrap_Dwg_Object_VISUALSTYLE_face_lighting_quality_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_face_lighting_quality_get", _wrap_Dwg_Object_VISUALSTYLE_face_lighting_quality_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_face_lighting_quality_int_set", _wrap_Dwg_Object_VISUALSTYLE_face_lighting_quality_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_face_lighting_quality_int_get", _wrap_Dwg_Object_VISUALSTYLE_face_lighting_quality_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_face_color_mode_set", _wrap_Dwg_Object_VISUALSTYLE_face_color_mode_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_face_color_mode_get", _wrap_Dwg_Object_VISUALSTYLE_face_color_mode_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_face_color_mode_int_set", _wrap_Dwg_Object_VISUALSTYLE_face_color_mode_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_face_color_mode_int_get", _wrap_Dwg_Object_VISUALSTYLE_face_color_mode_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_face_opacity_set", _wrap_Dwg_Object_VISUALSTYLE_face_opacity_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_face_opacity_get", _wrap_Dwg_Object_VISUALSTYLE_face_opacity_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_face_opacity_int_set", _wrap_Dwg_Object_VISUALSTYLE_face_opacity_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_face_opacity_int_get", _wrap_Dwg_Object_VISUALSTYLE_face_opacity_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_face_specular_set", _wrap_Dwg_Object_VISUALSTYLE_face_specular_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_face_specular_get", _wrap_Dwg_Object_VISUALSTYLE_face_specular_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_face_specular_int_set", _wrap_Dwg_Object_VISUALSTYLE_face_specular_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_face_specular_int_get", _wrap_Dwg_Object_VISUALSTYLE_face_specular_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_face_modifier_set", _wrap_Dwg_Object_VISUALSTYLE_face_modifier_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_face_modifier_get", _wrap_Dwg_Object_VISUALSTYLE_face_modifier_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_face_modifier_int_set", _wrap_Dwg_Object_VISUALSTYLE_face_modifier_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_face_modifier_int_get", _wrap_Dwg_Object_VISUALSTYLE_face_modifier_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_face_mono_color_set", _wrap_Dwg_Object_VISUALSTYLE_face_mono_color_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_face_mono_color_get", _wrap_Dwg_Object_VISUALSTYLE_face_mono_color_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_face_mono_color_int_set", _wrap_Dwg_Object_VISUALSTYLE_face_mono_color_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_face_mono_color_int_get", _wrap_Dwg_Object_VISUALSTYLE_face_mono_color_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_model_set", _wrap_Dwg_Object_VISUALSTYLE_edge_model_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_model_get", _wrap_Dwg_Object_VISUALSTYLE_edge_model_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_model_int_set", _wrap_Dwg_Object_VISUALSTYLE_edge_model_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_model_int_get", _wrap_Dwg_Object_VISUALSTYLE_edge_model_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_style_set", _wrap_Dwg_Object_VISUALSTYLE_edge_style_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_style_get", _wrap_Dwg_Object_VISUALSTYLE_edge_style_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_style_int_set", _wrap_Dwg_Object_VISUALSTYLE_edge_style_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_style_int_get", _wrap_Dwg_Object_VISUALSTYLE_edge_style_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_intersection_color_set", _wrap_Dwg_Object_VISUALSTYLE_edge_intersection_color_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_intersection_color_get", _wrap_Dwg_Object_VISUALSTYLE_edge_intersection_color_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_intersection_color_int_set", _wrap_Dwg_Object_VISUALSTYLE_edge_intersection_color_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_intersection_color_int_get", _wrap_Dwg_Object_VISUALSTYLE_edge_intersection_color_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_obscured_color_set", _wrap_Dwg_Object_VISUALSTYLE_edge_obscured_color_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_obscured_color_get", _wrap_Dwg_Object_VISUALSTYLE_edge_obscured_color_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_obscured_color_int_set", _wrap_Dwg_Object_VISUALSTYLE_edge_obscured_color_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_obscured_color_int_get", _wrap_Dwg_Object_VISUALSTYLE_edge_obscured_color_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_obscured_ltype_set", _wrap_Dwg_Object_VISUALSTYLE_edge_obscured_ltype_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_obscured_ltype_get", _wrap_Dwg_Object_VISUALSTYLE_edge_obscured_ltype_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_obscured_ltype_int_set", _wrap_Dwg_Object_VISUALSTYLE_edge_obscured_ltype_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_obscured_ltype_int_get", _wrap_Dwg_Object_VISUALSTYLE_edge_obscured_ltype_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_intersection_ltype_set", _wrap_Dwg_Object_VISUALSTYLE_edge_intersection_ltype_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_intersection_ltype_get", _wrap_Dwg_Object_VISUALSTYLE_edge_intersection_ltype_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_intersection_ltype_int_set", _wrap_Dwg_Object_VISUALSTYLE_edge_intersection_ltype_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_intersection_ltype_int_get", _wrap_Dwg_Object_VISUALSTYLE_edge_intersection_ltype_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_crease_angle_set", _wrap_Dwg_Object_VISUALSTYLE_edge_crease_angle_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_crease_angle_get", _wrap_Dwg_Object_VISUALSTYLE_edge_crease_angle_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_crease_angle_int_set", _wrap_Dwg_Object_VISUALSTYLE_edge_crease_angle_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_crease_angle_int_get", _wrap_Dwg_Object_VISUALSTYLE_edge_crease_angle_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_modifier_set", _wrap_Dwg_Object_VISUALSTYLE_edge_modifier_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_modifier_get", _wrap_Dwg_Object_VISUALSTYLE_edge_modifier_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_modifier_int_set", _wrap_Dwg_Object_VISUALSTYLE_edge_modifier_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_modifier_int_get", _wrap_Dwg_Object_VISUALSTYLE_edge_modifier_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_color_set", _wrap_Dwg_Object_VISUALSTYLE_edge_color_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_color_get", _wrap_Dwg_Object_VISUALSTYLE_edge_color_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_color_int_set", _wrap_Dwg_Object_VISUALSTYLE_edge_color_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_color_int_get", _wrap_Dwg_Object_VISUALSTYLE_edge_color_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_opacity_set", _wrap_Dwg_Object_VISUALSTYLE_edge_opacity_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_opacity_get", _wrap_Dwg_Object_VISUALSTYLE_edge_opacity_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_opacity_int_set", _wrap_Dwg_Object_VISUALSTYLE_edge_opacity_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_opacity_int_get", _wrap_Dwg_Object_VISUALSTYLE_edge_opacity_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_width_set", _wrap_Dwg_Object_VISUALSTYLE_edge_width_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_width_get", _wrap_Dwg_Object_VISUALSTYLE_edge_width_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_width_int_set", _wrap_Dwg_Object_VISUALSTYLE_edge_width_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_width_int_get", _wrap_Dwg_Object_VISUALSTYLE_edge_width_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_overhang_set", _wrap_Dwg_Object_VISUALSTYLE_edge_overhang_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_overhang_get", _wrap_Dwg_Object_VISUALSTYLE_edge_overhang_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_overhang_int_set", _wrap_Dwg_Object_VISUALSTYLE_edge_overhang_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_overhang_int_get", _wrap_Dwg_Object_VISUALSTYLE_edge_overhang_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_jitter_set", _wrap_Dwg_Object_VISUALSTYLE_edge_jitter_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_jitter_get", _wrap_Dwg_Object_VISUALSTYLE_edge_jitter_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_jitter_int_set", _wrap_Dwg_Object_VISUALSTYLE_edge_jitter_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_jitter_int_get", _wrap_Dwg_Object_VISUALSTYLE_edge_jitter_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_silhouette_color_set", _wrap_Dwg_Object_VISUALSTYLE_edge_silhouette_color_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_silhouette_color_get", _wrap_Dwg_Object_VISUALSTYLE_edge_silhouette_color_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_silhouette_color_int_set", _wrap_Dwg_Object_VISUALSTYLE_edge_silhouette_color_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_silhouette_color_int_get", _wrap_Dwg_Object_VISUALSTYLE_edge_silhouette_color_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_silhouette_width_set", _wrap_Dwg_Object_VISUALSTYLE_edge_silhouette_width_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_silhouette_width_get", _wrap_Dwg_Object_VISUALSTYLE_edge_silhouette_width_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_silhouette_width_int_set", _wrap_Dwg_Object_VISUALSTYLE_edge_silhouette_width_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_silhouette_width_int_get", _wrap_Dwg_Object_VISUALSTYLE_edge_silhouette_width_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_halo_gap_set", _wrap_Dwg_Object_VISUALSTYLE_edge_halo_gap_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_halo_gap_get", _wrap_Dwg_Object_VISUALSTYLE_edge_halo_gap_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_halo_gap_int_set", _wrap_Dwg_Object_VISUALSTYLE_edge_halo_gap_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_halo_gap_int_get", _wrap_Dwg_Object_VISUALSTYLE_edge_halo_gap_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_isolines_set", _wrap_Dwg_Object_VISUALSTYLE_edge_isolines_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_isolines_get", _wrap_Dwg_Object_VISUALSTYLE_edge_isolines_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_isolines_int_set", _wrap_Dwg_Object_VISUALSTYLE_edge_isolines_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_isolines_int_get", _wrap_Dwg_Object_VISUALSTYLE_edge_isolines_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_do_hide_precision_set", _wrap_Dwg_Object_VISUALSTYLE_edge_do_hide_precision_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_do_hide_precision_get", _wrap_Dwg_Object_VISUALSTYLE_edge_do_hide_precision_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_do_hide_precision_int_set", _wrap_Dwg_Object_VISUALSTYLE_edge_do_hide_precision_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_do_hide_precision_int_get", _wrap_Dwg_Object_VISUALSTYLE_edge_do_hide_precision_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_style_apply_set", _wrap_Dwg_Object_VISUALSTYLE_edge_style_apply_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_style_apply_get", _wrap_Dwg_Object_VISUALSTYLE_edge_style_apply_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_style_apply_int_set", _wrap_Dwg_Object_VISUALSTYLE_edge_style_apply_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_style_apply_int_get", _wrap_Dwg_Object_VISUALSTYLE_edge_style_apply_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_display_settings_set", _wrap_Dwg_Object_VISUALSTYLE_display_settings_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_display_settings_get", _wrap_Dwg_Object_VISUALSTYLE_display_settings_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_display_settings_int_set", _wrap_Dwg_Object_VISUALSTYLE_display_settings_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_display_settings_int_get", _wrap_Dwg_Object_VISUALSTYLE_display_settings_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_display_brightness_bl_set", _wrap_Dwg_Object_VISUALSTYLE_display_brightness_bl_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_display_brightness_bl_get", _wrap_Dwg_Object_VISUALSTYLE_display_brightness_bl_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_display_brightness_set", _wrap_Dwg_Object_VISUALSTYLE_display_brightness_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_display_brightness_get", _wrap_Dwg_Object_VISUALSTYLE_display_brightness_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_display_brightness_int_set", _wrap_Dwg_Object_VISUALSTYLE_display_brightness_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_display_brightness_int_get", _wrap_Dwg_Object_VISUALSTYLE_display_brightness_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_display_shadow_type_set", _wrap_Dwg_Object_VISUALSTYLE_display_shadow_type_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_display_shadow_type_get", _wrap_Dwg_Object_VISUALSTYLE_display_shadow_type_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_display_shadow_type_int_set", _wrap_Dwg_Object_VISUALSTYLE_display_shadow_type_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_display_shadow_type_int_get", _wrap_Dwg_Object_VISUALSTYLE_display_shadow_type_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bd2007_45_set", _wrap_Dwg_Object_VISUALSTYLE_bd2007_45_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bd2007_45_get", _wrap_Dwg_Object_VISUALSTYLE_bd2007_45_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_num_props_set", _wrap_Dwg_Object_VISUALSTYLE_num_props_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_num_props_get", _wrap_Dwg_Object_VISUALSTYLE_num_props_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop1c_set", _wrap_Dwg_Object_VISUALSTYLE_b_prop1c_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop1c_get", _wrap_Dwg_Object_VISUALSTYLE_b_prop1c_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop1c_int_set", _wrap_Dwg_Object_VISUALSTYLE_b_prop1c_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop1c_int_get", _wrap_Dwg_Object_VISUALSTYLE_b_prop1c_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop1d_set", _wrap_Dwg_Object_VISUALSTYLE_b_prop1d_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop1d_get", _wrap_Dwg_Object_VISUALSTYLE_b_prop1d_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop1d_int_set", _wrap_Dwg_Object_VISUALSTYLE_b_prop1d_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop1d_int_get", _wrap_Dwg_Object_VISUALSTYLE_b_prop1d_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop1e_set", _wrap_Dwg_Object_VISUALSTYLE_b_prop1e_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop1e_get", _wrap_Dwg_Object_VISUALSTYLE_b_prop1e_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop1e_int_set", _wrap_Dwg_Object_VISUALSTYLE_b_prop1e_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop1e_int_get", _wrap_Dwg_Object_VISUALSTYLE_b_prop1e_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop1f_set", _wrap_Dwg_Object_VISUALSTYLE_b_prop1f_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop1f_get", _wrap_Dwg_Object_VISUALSTYLE_b_prop1f_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop1f_int_set", _wrap_Dwg_Object_VISUALSTYLE_b_prop1f_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop1f_int_get", _wrap_Dwg_Object_VISUALSTYLE_b_prop1f_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop20_set", _wrap_Dwg_Object_VISUALSTYLE_b_prop20_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop20_get", _wrap_Dwg_Object_VISUALSTYLE_b_prop20_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop20_int_set", _wrap_Dwg_Object_VISUALSTYLE_b_prop20_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop20_int_get", _wrap_Dwg_Object_VISUALSTYLE_b_prop20_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop21_set", _wrap_Dwg_Object_VISUALSTYLE_b_prop21_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop21_get", _wrap_Dwg_Object_VISUALSTYLE_b_prop21_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop21_int_set", _wrap_Dwg_Object_VISUALSTYLE_b_prop21_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop21_int_get", _wrap_Dwg_Object_VISUALSTYLE_b_prop21_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop22_set", _wrap_Dwg_Object_VISUALSTYLE_b_prop22_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop22_get", _wrap_Dwg_Object_VISUALSTYLE_b_prop22_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop22_int_set", _wrap_Dwg_Object_VISUALSTYLE_b_prop22_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop22_int_get", _wrap_Dwg_Object_VISUALSTYLE_b_prop22_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop23_set", _wrap_Dwg_Object_VISUALSTYLE_b_prop23_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop23_get", _wrap_Dwg_Object_VISUALSTYLE_b_prop23_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop23_int_set", _wrap_Dwg_Object_VISUALSTYLE_b_prop23_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop23_int_get", _wrap_Dwg_Object_VISUALSTYLE_b_prop23_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop24_set", _wrap_Dwg_Object_VISUALSTYLE_b_prop24_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop24_get", _wrap_Dwg_Object_VISUALSTYLE_b_prop24_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop24_int_set", _wrap_Dwg_Object_VISUALSTYLE_b_prop24_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop24_int_get", _wrap_Dwg_Object_VISUALSTYLE_b_prop24_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop25_set", _wrap_Dwg_Object_VISUALSTYLE_bl_prop25_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop25_get", _wrap_Dwg_Object_VISUALSTYLE_bl_prop25_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop25_int_set", _wrap_Dwg_Object_VISUALSTYLE_bl_prop25_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop25_int_get", _wrap_Dwg_Object_VISUALSTYLE_bl_prop25_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop26_set", _wrap_Dwg_Object_VISUALSTYLE_bd_prop26_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop26_get", _wrap_Dwg_Object_VISUALSTYLE_bd_prop26_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop26_int_set", _wrap_Dwg_Object_VISUALSTYLE_bd_prop26_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop26_int_get", _wrap_Dwg_Object_VISUALSTYLE_bd_prop26_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop27_set", _wrap_Dwg_Object_VISUALSTYLE_bd_prop27_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop27_get", _wrap_Dwg_Object_VISUALSTYLE_bd_prop27_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop27_int_set", _wrap_Dwg_Object_VISUALSTYLE_bd_prop27_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop27_int_get", _wrap_Dwg_Object_VISUALSTYLE_bd_prop27_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop28_set", _wrap_Dwg_Object_VISUALSTYLE_bl_prop28_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop28_get", _wrap_Dwg_Object_VISUALSTYLE_bl_prop28_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop28_int_set", _wrap_Dwg_Object_VISUALSTYLE_bl_prop28_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop28_int_get", _wrap_Dwg_Object_VISUALSTYLE_bl_prop28_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_c_prop29_set", _wrap_Dwg_Object_VISUALSTYLE_c_prop29_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_c_prop29_get", _wrap_Dwg_Object_VISUALSTYLE_c_prop29_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_c_prop29_int_set", _wrap_Dwg_Object_VISUALSTYLE_c_prop29_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_c_prop29_int_get", _wrap_Dwg_Object_VISUALSTYLE_c_prop29_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop2a_set", _wrap_Dwg_Object_VISUALSTYLE_bl_prop2a_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop2a_get", _wrap_Dwg_Object_VISUALSTYLE_bl_prop2a_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop2a_int_set", _wrap_Dwg_Object_VISUALSTYLE_bl_prop2a_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop2a_int_get", _wrap_Dwg_Object_VISUALSTYLE_bl_prop2a_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop2b_set", _wrap_Dwg_Object_VISUALSTYLE_bl_prop2b_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop2b_get", _wrap_Dwg_Object_VISUALSTYLE_bl_prop2b_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop2b_int_set", _wrap_Dwg_Object_VISUALSTYLE_bl_prop2b_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop2b_int_get", _wrap_Dwg_Object_VISUALSTYLE_bl_prop2b_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_c_prop2c_set", _wrap_Dwg_Object_VISUALSTYLE_c_prop2c_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_c_prop2c_get", _wrap_Dwg_Object_VISUALSTYLE_c_prop2c_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_c_prop2c_int_set", _wrap_Dwg_Object_VISUALSTYLE_c_prop2c_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_c_prop2c_int_get", _wrap_Dwg_Object_VISUALSTYLE_c_prop2c_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop2d_set", _wrap_Dwg_Object_VISUALSTYLE_b_prop2d_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop2d_get", _wrap_Dwg_Object_VISUALSTYLE_b_prop2d_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop2d_int_set", _wrap_Dwg_Object_VISUALSTYLE_b_prop2d_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop2d_int_get", _wrap_Dwg_Object_VISUALSTYLE_b_prop2d_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop2e_set", _wrap_Dwg_Object_VISUALSTYLE_bl_prop2e_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop2e_get", _wrap_Dwg_Object_VISUALSTYLE_bl_prop2e_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop2e_int_set", _wrap_Dwg_Object_VISUALSTYLE_bl_prop2e_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop2e_int_get", _wrap_Dwg_Object_VISUALSTYLE_bl_prop2e_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop2f_set", _wrap_Dwg_Object_VISUALSTYLE_bl_prop2f_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop2f_get", _wrap_Dwg_Object_VISUALSTYLE_bl_prop2f_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop2f_int_set", _wrap_Dwg_Object_VISUALSTYLE_bl_prop2f_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop2f_int_get", _wrap_Dwg_Object_VISUALSTYLE_bl_prop2f_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop30_set", _wrap_Dwg_Object_VISUALSTYLE_bl_prop30_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop30_get", _wrap_Dwg_Object_VISUALSTYLE_bl_prop30_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop30_int_set", _wrap_Dwg_Object_VISUALSTYLE_bl_prop30_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop30_int_get", _wrap_Dwg_Object_VISUALSTYLE_bl_prop30_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop31_set", _wrap_Dwg_Object_VISUALSTYLE_b_prop31_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop31_get", _wrap_Dwg_Object_VISUALSTYLE_b_prop31_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop31_int_set", _wrap_Dwg_Object_VISUALSTYLE_b_prop31_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop31_int_get", _wrap_Dwg_Object_VISUALSTYLE_b_prop31_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop32_set", _wrap_Dwg_Object_VISUALSTYLE_bl_prop32_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop32_get", _wrap_Dwg_Object_VISUALSTYLE_bl_prop32_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop32_int_set", _wrap_Dwg_Object_VISUALSTYLE_bl_prop32_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop32_int_get", _wrap_Dwg_Object_VISUALSTYLE_bl_prop32_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_c_prop33_set", _wrap_Dwg_Object_VISUALSTYLE_c_prop33_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_c_prop33_get", _wrap_Dwg_Object_VISUALSTYLE_c_prop33_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_c_prop33_int_set", _wrap_Dwg_Object_VISUALSTYLE_c_prop33_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_c_prop33_int_get", _wrap_Dwg_Object_VISUALSTYLE_c_prop33_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop34_set", _wrap_Dwg_Object_VISUALSTYLE_bd_prop34_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop34_get", _wrap_Dwg_Object_VISUALSTYLE_bd_prop34_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop34_int_set", _wrap_Dwg_Object_VISUALSTYLE_bd_prop34_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop34_int_get", _wrap_Dwg_Object_VISUALSTYLE_bd_prop34_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_wiggle_set", _wrap_Dwg_Object_VISUALSTYLE_edge_wiggle_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_wiggle_get", _wrap_Dwg_Object_VISUALSTYLE_edge_wiggle_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_wiggle_int_set", _wrap_Dwg_Object_VISUALSTYLE_edge_wiggle_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_edge_wiggle_int_get", _wrap_Dwg_Object_VISUALSTYLE_edge_wiggle_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_strokes_set", _wrap_Dwg_Object_VISUALSTYLE_strokes_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_strokes_get", _wrap_Dwg_Object_VISUALSTYLE_strokes_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_strokes_int_set", _wrap_Dwg_Object_VISUALSTYLE_strokes_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_strokes_int_get", _wrap_Dwg_Object_VISUALSTYLE_strokes_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop37_set", _wrap_Dwg_Object_VISUALSTYLE_b_prop37_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop37_get", _wrap_Dwg_Object_VISUALSTYLE_b_prop37_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop37_int_set", _wrap_Dwg_Object_VISUALSTYLE_b_prop37_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop37_int_get", _wrap_Dwg_Object_VISUALSTYLE_b_prop37_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop38_set", _wrap_Dwg_Object_VISUALSTYLE_bd_prop38_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop38_get", _wrap_Dwg_Object_VISUALSTYLE_bd_prop38_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop38_int_set", _wrap_Dwg_Object_VISUALSTYLE_bd_prop38_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop38_int_get", _wrap_Dwg_Object_VISUALSTYLE_bd_prop38_int_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop39_set", _wrap_Dwg_Object_VISUALSTYLE_bd_prop39_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop39_get", _wrap_Dwg_Object_VISUALSTYLE_bd_prop39_get},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop39_int_set", _wrap_Dwg_Object_VISUALSTYLE_bd_prop39_int_set},
{"LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop39_int_get", _wrap_Dwg_Object_VISUALSTYLE_bd_prop39_int_get},
{"LibreDWGc::new_Dwg_Object_VISUALSTYLE", _wrap_new_Dwg_Object_VISUALSTYLE},
{"LibreDWGc::delete_Dwg_Object_VISUALSTYLE", _wrap_delete_Dwg_Object_VISUALSTYLE},
{"LibreDWGc::Dwg_LIGHTLIST_light_parent_set", _wrap_Dwg_LIGHTLIST_light_parent_set},
{"LibreDWGc::Dwg_LIGHTLIST_light_parent_get", _wrap_Dwg_LIGHTLIST_light_parent_get},
{"LibreDWGc::Dwg_LIGHTLIST_light_name_set", _wrap_Dwg_LIGHTLIST_light_name_set},
{"LibreDWGc::Dwg_LIGHTLIST_light_name_get", _wrap_Dwg_LIGHTLIST_light_name_get},
{"LibreDWGc::Dwg_LIGHTLIST_light_handle_set", _wrap_Dwg_LIGHTLIST_light_handle_set},
{"LibreDWGc::Dwg_LIGHTLIST_light_handle_get", _wrap_Dwg_LIGHTLIST_light_handle_get},
{"LibreDWGc::new_Dwg_LIGHTLIST_light", _wrap_new_Dwg_LIGHTLIST_light},
{"LibreDWGc::delete_Dwg_LIGHTLIST_light", _wrap_delete_Dwg_LIGHTLIST_light},
{"LibreDWGc::Dwg_Object_LIGHTLIST_parent_set", _wrap_Dwg_Object_LIGHTLIST_parent_set},
{"LibreDWGc::Dwg_Object_LIGHTLIST_parent_get", _wrap_Dwg_Object_LIGHTLIST_parent_get},
{"LibreDWGc::Dwg_Object_LIGHTLIST_class_version_set", _wrap_Dwg_Object_LIGHTLIST_class_version_set},
{"LibreDWGc::Dwg_Object_LIGHTLIST_class_version_get", _wrap_Dwg_Object_LIGHTLIST_class_version_get},
{"LibreDWGc::Dwg_Object_LIGHTLIST_num_lights_set", _wrap_Dwg_Object_LIGHTLIST_num_lights_set},
{"LibreDWGc::Dwg_Object_LIGHTLIST_num_lights_get", _wrap_Dwg_Object_LIGHTLIST_num_lights_get},
{"LibreDWGc::Dwg_Object_LIGHTLIST_lights_set", _wrap_Dwg_Object_LIGHTLIST_lights_set},
{"LibreDWGc::Dwg_Object_LIGHTLIST_lights_get", _wrap_Dwg_Object_LIGHTLIST_lights_get},
{"LibreDWGc::new_Dwg_Object_LIGHTLIST", _wrap_new_Dwg_Object_LIGHTLIST},
{"LibreDWGc::delete_Dwg_Object_LIGHTLIST", _wrap_delete_Dwg_Object_LIGHTLIST},
{"LibreDWGc::Dwg_MATERIAL_color_parent_set", _wrap_Dwg_MATERIAL_color_parent_set},
{"LibreDWGc::Dwg_MATERIAL_color_parent_get", _wrap_Dwg_MATERIAL_color_parent_get},
{"LibreDWGc::Dwg_MATERIAL_color_flag_set", _wrap_Dwg_MATERIAL_color_flag_set},
{"LibreDWGc::Dwg_MATERIAL_color_flag_get", _wrap_Dwg_MATERIAL_color_flag_get},
{"LibreDWGc::Dwg_MATERIAL_color_factor_set", _wrap_Dwg_MATERIAL_color_factor_set},
{"LibreDWGc::Dwg_MATERIAL_color_factor_get", _wrap_Dwg_MATERIAL_color_factor_get},
{"LibreDWGc::Dwg_MATERIAL_color_rgb_set", _wrap_Dwg_MATERIAL_color_rgb_set},
{"LibreDWGc::Dwg_MATERIAL_color_rgb_get", _wrap_Dwg_MATERIAL_color_rgb_get},
{"LibreDWGc::new_Dwg_MATERIAL_color", _wrap_new_Dwg_MATERIAL_color},
{"LibreDWGc::delete_Dwg_MATERIAL_color", _wrap_delete_Dwg_MATERIAL_color},
{"LibreDWGc::Dwg_MATERIAL_mapper_parent_set", _wrap_Dwg_MATERIAL_mapper_parent_set},
{"LibreDWGc::Dwg_MATERIAL_mapper_parent_get", _wrap_Dwg_MATERIAL_mapper_parent_get},
{"LibreDWGc::Dwg_MATERIAL_mapper_blendfactor_set", _wrap_Dwg_MATERIAL_mapper_blendfactor_set},
{"LibreDWGc::Dwg_MATERIAL_mapper_blendfactor_get", _wrap_Dwg_MATERIAL_mapper_blendfactor_get},
{"LibreDWGc::Dwg_MATERIAL_mapper_transmatrix_set", _wrap_Dwg_MATERIAL_mapper_transmatrix_set},
{"LibreDWGc::Dwg_MATERIAL_mapper_transmatrix_get", _wrap_Dwg_MATERIAL_mapper_transmatrix_get},
{"LibreDWGc::Dwg_MATERIAL_mapper_filename_set", _wrap_Dwg_MATERIAL_mapper_filename_set},
{"LibreDWGc::Dwg_MATERIAL_mapper_filename_get", _wrap_Dwg_MATERIAL_mapper_filename_get},
{"LibreDWGc::Dwg_MATERIAL_mapper_color1_set", _wrap_Dwg_MATERIAL_mapper_color1_set},
{"LibreDWGc::Dwg_MATERIAL_mapper_color1_get", _wrap_Dwg_MATERIAL_mapper_color1_get},
{"LibreDWGc::Dwg_MATERIAL_mapper_color2_set", _wrap_Dwg_MATERIAL_mapper_color2_set},
{"LibreDWGc::Dwg_MATERIAL_mapper_color2_get", _wrap_Dwg_MATERIAL_mapper_color2_get},
{"LibreDWGc::Dwg_MATERIAL_mapper_source_set", _wrap_Dwg_MATERIAL_mapper_source_set},
{"LibreDWGc::Dwg_MATERIAL_mapper_source_get", _wrap_Dwg_MATERIAL_mapper_source_get},
{"LibreDWGc::Dwg_MATERIAL_mapper_projection_set", _wrap_Dwg_MATERIAL_mapper_projection_set},
{"LibreDWGc::Dwg_MATERIAL_mapper_projection_get", _wrap_Dwg_MATERIAL_mapper_projection_get},
{"LibreDWGc::Dwg_MATERIAL_mapper_tiling_set", _wrap_Dwg_MATERIAL_mapper_tiling_set},
{"LibreDWGc::Dwg_MATERIAL_mapper_tiling_get", _wrap_Dwg_MATERIAL_mapper_tiling_get},
{"LibreDWGc::Dwg_MATERIAL_mapper_autotransform_set", _wrap_Dwg_MATERIAL_mapper_autotransform_set},
{"LibreDWGc::Dwg_MATERIAL_mapper_autotransform_get", _wrap_Dwg_MATERIAL_mapper_autotransform_get},
{"LibreDWGc::Dwg_MATERIAL_mapper_texturemode_set", _wrap_Dwg_MATERIAL_mapper_texturemode_set},
{"LibreDWGc::Dwg_MATERIAL_mapper_texturemode_get", _wrap_Dwg_MATERIAL_mapper_texturemode_get},
{"LibreDWGc::new_Dwg_MATERIAL_mapper", _wrap_new_Dwg_MATERIAL_mapper},
{"LibreDWGc::delete_Dwg_MATERIAL_mapper", _wrap_delete_Dwg_MATERIAL_mapper},
{"LibreDWGc::Dwg_MATERIAL_gentexture_parent_set", _wrap_Dwg_MATERIAL_gentexture_parent_set},
{"LibreDWGc::Dwg_MATERIAL_gentexture_parent_get", _wrap_Dwg_MATERIAL_gentexture_parent_get},
{"LibreDWGc::Dwg_MATERIAL_gentexture_genprocname_set", _wrap_Dwg_MATERIAL_gentexture_genprocname_set},
{"LibreDWGc::Dwg_MATERIAL_gentexture_genprocname_get", _wrap_Dwg_MATERIAL_gentexture_genprocname_get},
{"LibreDWGc::Dwg_MATERIAL_gentexture_material_set", _wrap_Dwg_MATERIAL_gentexture_material_set},
{"LibreDWGc::Dwg_MATERIAL_gentexture_material_get", _wrap_Dwg_MATERIAL_gentexture_material_get},
{"LibreDWGc::new_Dwg_MATERIAL_gentexture", _wrap_new_Dwg_MATERIAL_gentexture},
{"LibreDWGc::delete_Dwg_MATERIAL_gentexture", _wrap_delete_Dwg_MATERIAL_gentexture},
{"LibreDWGc::Dwg_Object_MATERIAL_parent_set", _wrap_Dwg_Object_MATERIAL_parent_set},
{"LibreDWGc::Dwg_Object_MATERIAL_parent_get", _wrap_Dwg_Object_MATERIAL_parent_get},
{"LibreDWGc::Dwg_Object_MATERIAL_name_set", _wrap_Dwg_Object_MATERIAL_name_set},
{"LibreDWGc::Dwg_Object_MATERIAL_name_get", _wrap_Dwg_Object_MATERIAL_name_get},
{"LibreDWGc::Dwg_Object_MATERIAL_description_set", _wrap_Dwg_Object_MATERIAL_description_set},
{"LibreDWGc::Dwg_Object_MATERIAL_description_get", _wrap_Dwg_Object_MATERIAL_description_get},
{"LibreDWGc::Dwg_Object_MATERIAL_ambient_color_set", _wrap_Dwg_Object_MATERIAL_ambient_color_set},
{"LibreDWGc::Dwg_Object_MATERIAL_ambient_color_get", _wrap_Dwg_Object_MATERIAL_ambient_color_get},
{"LibreDWGc::Dwg_Object_MATERIAL_diffuse_color_set", _wrap_Dwg_Object_MATERIAL_diffuse_color_set},
{"LibreDWGc::Dwg_Object_MATERIAL_diffuse_color_get", _wrap_Dwg_Object_MATERIAL_diffuse_color_get},
{"LibreDWGc::Dwg_Object_MATERIAL_diffusemap_set", _wrap_Dwg_Object_MATERIAL_diffusemap_set},
{"LibreDWGc::Dwg_Object_MATERIAL_diffusemap_get", _wrap_Dwg_Object_MATERIAL_diffusemap_get},
{"LibreDWGc::Dwg_Object_MATERIAL_specular_gloss_factor_set", _wrap_Dwg_Object_MATERIAL_specular_gloss_factor_set},
{"LibreDWGc::Dwg_Object_MATERIAL_specular_gloss_factor_get", _wrap_Dwg_Object_MATERIAL_specular_gloss_factor_get},
{"LibreDWGc::Dwg_Object_MATERIAL_specular_color_set", _wrap_Dwg_Object_MATERIAL_specular_color_set},
{"LibreDWGc::Dwg_Object_MATERIAL_specular_color_get", _wrap_Dwg_Object_MATERIAL_specular_color_get},
{"LibreDWGc::Dwg_Object_MATERIAL_specularmap_set", _wrap_Dwg_Object_MATERIAL_specularmap_set},
{"LibreDWGc::Dwg_Object_MATERIAL_specularmap_get", _wrap_Dwg_Object_MATERIAL_specularmap_get},
{"LibreDWGc::Dwg_Object_MATERIAL_reflectionmap_set", _wrap_Dwg_Object_MATERIAL_reflectionmap_set},
{"LibreDWGc::Dwg_Object_MATERIAL_reflectionmap_get", _wrap_Dwg_Object_MATERIAL_reflectionmap_get},
{"LibreDWGc::Dwg_Object_MATERIAL_opacity_percent_set", _wrap_Dwg_Object_MATERIAL_opacity_percent_set},
{"LibreDWGc::Dwg_Object_MATERIAL_opacity_percent_get", _wrap_Dwg_Object_MATERIAL_opacity_percent_get},
{"LibreDWGc::Dwg_Object_MATERIAL_opacitymap_set", _wrap_Dwg_Object_MATERIAL_opacitymap_set},
{"LibreDWGc::Dwg_Object_MATERIAL_opacitymap_get", _wrap_Dwg_Object_MATERIAL_opacitymap_get},
{"LibreDWGc::Dwg_Object_MATERIAL_bumpmap_set", _wrap_Dwg_Object_MATERIAL_bumpmap_set},
{"LibreDWGc::Dwg_Object_MATERIAL_bumpmap_get", _wrap_Dwg_Object_MATERIAL_bumpmap_get},
{"LibreDWGc::Dwg_Object_MATERIAL_refraction_index_set", _wrap_Dwg_Object_MATERIAL_refraction_index_set},
{"LibreDWGc::Dwg_Object_MATERIAL_refraction_index_get", _wrap_Dwg_Object_MATERIAL_refraction_index_get},
{"LibreDWGc::Dwg_Object_MATERIAL_refractionmap_set", _wrap_Dwg_Object_MATERIAL_refractionmap_set},
{"LibreDWGc::Dwg_Object_MATERIAL_refractionmap_get", _wrap_Dwg_Object_MATERIAL_refractionmap_get},
{"LibreDWGc::Dwg_Object_MATERIAL_color_bleed_scale_set", _wrap_Dwg_Object_MATERIAL_color_bleed_scale_set},
{"LibreDWGc::Dwg_Object_MATERIAL_color_bleed_scale_get", _wrap_Dwg_Object_MATERIAL_color_bleed_scale_get},
{"LibreDWGc::Dwg_Object_MATERIAL_indirect_bump_scale_set", _wrap_Dwg_Object_MATERIAL_indirect_bump_scale_set},
{"LibreDWGc::Dwg_Object_MATERIAL_indirect_bump_scale_get", _wrap_Dwg_Object_MATERIAL_indirect_bump_scale_get},
{"LibreDWGc::Dwg_Object_MATERIAL_reflectance_scale_set", _wrap_Dwg_Object_MATERIAL_reflectance_scale_set},
{"LibreDWGc::Dwg_Object_MATERIAL_reflectance_scale_get", _wrap_Dwg_Object_MATERIAL_reflectance_scale_get},
{"LibreDWGc::Dwg_Object_MATERIAL_transmittance_scale_set", _wrap_Dwg_Object_MATERIAL_transmittance_scale_set},
{"LibreDWGc::Dwg_Object_MATERIAL_transmittance_scale_get", _wrap_Dwg_Object_MATERIAL_transmittance_scale_get},
{"LibreDWGc::Dwg_Object_MATERIAL_two_sided_material_set", _wrap_Dwg_Object_MATERIAL_two_sided_material_set},
{"LibreDWGc::Dwg_Object_MATERIAL_two_sided_material_get", _wrap_Dwg_Object_MATERIAL_two_sided_material_get},
{"LibreDWGc::Dwg_Object_MATERIAL_luminance_set", _wrap_Dwg_Object_MATERIAL_luminance_set},
{"LibreDWGc::Dwg_Object_MATERIAL_luminance_get", _wrap_Dwg_Object_MATERIAL_luminance_get},
{"LibreDWGc::Dwg_Object_MATERIAL_luminance_mode_set", _wrap_Dwg_Object_MATERIAL_luminance_mode_set},
{"LibreDWGc::Dwg_Object_MATERIAL_luminance_mode_get", _wrap_Dwg_Object_MATERIAL_luminance_mode_get},
{"LibreDWGc::Dwg_Object_MATERIAL_translucence_set", _wrap_Dwg_Object_MATERIAL_translucence_set},
{"LibreDWGc::Dwg_Object_MATERIAL_translucence_get", _wrap_Dwg_Object_MATERIAL_translucence_get},
{"LibreDWGc::Dwg_Object_MATERIAL_self_illumination_set", _wrap_Dwg_Object_MATERIAL_self_illumination_set},
{"LibreDWGc::Dwg_Object_MATERIAL_self_illumination_get", _wrap_Dwg_Object_MATERIAL_self_illumination_get},
{"LibreDWGc::Dwg_Object_MATERIAL_reflectivity_set", _wrap_Dwg_Object_MATERIAL_reflectivity_set},
{"LibreDWGc::Dwg_Object_MATERIAL_reflectivity_get", _wrap_Dwg_Object_MATERIAL_reflectivity_get},
{"LibreDWGc::Dwg_Object_MATERIAL_illumination_model_set", _wrap_Dwg_Object_MATERIAL_illumination_model_set},
{"LibreDWGc::Dwg_Object_MATERIAL_illumination_model_get", _wrap_Dwg_Object_MATERIAL_illumination_model_get},
{"LibreDWGc::Dwg_Object_MATERIAL_channel_flags_set", _wrap_Dwg_Object_MATERIAL_channel_flags_set},
{"LibreDWGc::Dwg_Object_MATERIAL_channel_flags_get", _wrap_Dwg_Object_MATERIAL_channel_flags_get},
{"LibreDWGc::Dwg_Object_MATERIAL_mode_set", _wrap_Dwg_Object_MATERIAL_mode_set},
{"LibreDWGc::Dwg_Object_MATERIAL_mode_get", _wrap_Dwg_Object_MATERIAL_mode_get},
{"LibreDWGc::Dwg_Object_MATERIAL_genprocname_set", _wrap_Dwg_Object_MATERIAL_genprocname_set},
{"LibreDWGc::Dwg_Object_MATERIAL_genprocname_get", _wrap_Dwg_Object_MATERIAL_genprocname_get},
{"LibreDWGc::Dwg_Object_MATERIAL_genproctype_set", _wrap_Dwg_Object_MATERIAL_genproctype_set},
{"LibreDWGc::Dwg_Object_MATERIAL_genproctype_get", _wrap_Dwg_Object_MATERIAL_genproctype_get},
{"LibreDWGc::Dwg_Object_MATERIAL_genprocvalbool_set", _wrap_Dwg_Object_MATERIAL_genprocvalbool_set},
{"LibreDWGc::Dwg_Object_MATERIAL_genprocvalbool_get", _wrap_Dwg_Object_MATERIAL_genprocvalbool_get},
{"LibreDWGc::Dwg_Object_MATERIAL_genprocvalint_set", _wrap_Dwg_Object_MATERIAL_genprocvalint_set},
{"LibreDWGc::Dwg_Object_MATERIAL_genprocvalint_get", _wrap_Dwg_Object_MATERIAL_genprocvalint_get},
{"LibreDWGc::Dwg_Object_MATERIAL_genprocvalreal_set", _wrap_Dwg_Object_MATERIAL_genprocvalreal_set},
{"LibreDWGc::Dwg_Object_MATERIAL_genprocvalreal_get", _wrap_Dwg_Object_MATERIAL_genprocvalreal_get},
{"LibreDWGc::Dwg_Object_MATERIAL_genprocvaltext_set", _wrap_Dwg_Object_MATERIAL_genprocvaltext_set},
{"LibreDWGc::Dwg_Object_MATERIAL_genprocvaltext_get", _wrap_Dwg_Object_MATERIAL_genprocvaltext_get},
{"LibreDWGc::Dwg_Object_MATERIAL_genprocvalcolor_set", _wrap_Dwg_Object_MATERIAL_genprocvalcolor_set},
{"LibreDWGc::Dwg_Object_MATERIAL_genprocvalcolor_get", _wrap_Dwg_Object_MATERIAL_genprocvalcolor_get},
{"LibreDWGc::Dwg_Object_MATERIAL_genproctableend_set", _wrap_Dwg_Object_MATERIAL_genproctableend_set},
{"LibreDWGc::Dwg_Object_MATERIAL_genproctableend_get", _wrap_Dwg_Object_MATERIAL_genproctableend_get},
{"LibreDWGc::Dwg_Object_MATERIAL_num_gentextures_set", _wrap_Dwg_Object_MATERIAL_num_gentextures_set},
{"LibreDWGc::Dwg_Object_MATERIAL_num_gentextures_get", _wrap_Dwg_Object_MATERIAL_num_gentextures_get},
{"LibreDWGc::Dwg_Object_MATERIAL_gentextures_set", _wrap_Dwg_Object_MATERIAL_gentextures_set},
{"LibreDWGc::Dwg_Object_MATERIAL_gentextures_get", _wrap_Dwg_Object_MATERIAL_gentextures_get},
{"LibreDWGc::new_Dwg_Object_MATERIAL", _wrap_new_Dwg_Object_MATERIAL},
{"LibreDWGc::delete_Dwg_Object_MATERIAL", _wrap_delete_Dwg_Object_MATERIAL},
{"LibreDWGc::Dwg_Object_OBJECT_PTR_parent_set", _wrap_Dwg_Object_OBJECT_PTR_parent_set},
{"LibreDWGc::Dwg_Object_OBJECT_PTR_parent_get", _wrap_Dwg_Object_OBJECT_PTR_parent_get},
{"LibreDWGc::new_Dwg_Object_OBJECT_PTR", _wrap_new_Dwg_Object_OBJECT_PTR},
{"LibreDWGc::delete_Dwg_Object_OBJECT_PTR", _wrap_delete_Dwg_Object_OBJECT_PTR},
{"LibreDWGc::Dwg_Entity_LIGHT_parent_set", _wrap_Dwg_Entity_LIGHT_parent_set},
{"LibreDWGc::Dwg_Entity_LIGHT_parent_get", _wrap_Dwg_Entity_LIGHT_parent_get},
{"LibreDWGc::Dwg_Entity_LIGHT_class_version_set", _wrap_Dwg_Entity_LIGHT_class_version_set},
{"LibreDWGc::Dwg_Entity_LIGHT_class_version_get", _wrap_Dwg_Entity_LIGHT_class_version_get},
{"LibreDWGc::Dwg_Entity_LIGHT_name_set", _wrap_Dwg_Entity_LIGHT_name_set},
{"LibreDWGc::Dwg_Entity_LIGHT_name_get", _wrap_Dwg_Entity_LIGHT_name_get},
{"LibreDWGc::Dwg_Entity_LIGHT_type_set", _wrap_Dwg_Entity_LIGHT_type_set},
{"LibreDWGc::Dwg_Entity_LIGHT_type_get", _wrap_Dwg_Entity_LIGHT_type_get},
{"LibreDWGc::Dwg_Entity_LIGHT_status_set", _wrap_Dwg_Entity_LIGHT_status_set},
{"LibreDWGc::Dwg_Entity_LIGHT_status_get", _wrap_Dwg_Entity_LIGHT_status_get},
{"LibreDWGc::Dwg_Entity_LIGHT_light_color_set", _wrap_Dwg_Entity_LIGHT_light_color_set},
{"LibreDWGc::Dwg_Entity_LIGHT_light_color_get", _wrap_Dwg_Entity_LIGHT_light_color_get},
{"LibreDWGc::Dwg_Entity_LIGHT_plot_glyph_set", _wrap_Dwg_Entity_LIGHT_plot_glyph_set},
{"LibreDWGc::Dwg_Entity_LIGHT_plot_glyph_get", _wrap_Dwg_Entity_LIGHT_plot_glyph_get},
{"LibreDWGc::Dwg_Entity_LIGHT_intensity_set", _wrap_Dwg_Entity_LIGHT_intensity_set},
{"LibreDWGc::Dwg_Entity_LIGHT_intensity_get", _wrap_Dwg_Entity_LIGHT_intensity_get},
{"LibreDWGc::Dwg_Entity_LIGHT_position_set", _wrap_Dwg_Entity_LIGHT_position_set},
{"LibreDWGc::Dwg_Entity_LIGHT_position_get", _wrap_Dwg_Entity_LIGHT_position_get},
{"LibreDWGc::Dwg_Entity_LIGHT_target_set", _wrap_Dwg_Entity_LIGHT_target_set},
{"LibreDWGc::Dwg_Entity_LIGHT_target_get", _wrap_Dwg_Entity_LIGHT_target_get},
{"LibreDWGc::Dwg_Entity_LIGHT_attenuation_type_set", _wrap_Dwg_Entity_LIGHT_attenuation_type_set},
{"LibreDWGc::Dwg_Entity_LIGHT_attenuation_type_get", _wrap_Dwg_Entity_LIGHT_attenuation_type_get},
{"LibreDWGc::Dwg_Entity_LIGHT_use_attenuation_limits_set", _wrap_Dwg_Entity_LIGHT_use_attenuation_limits_set},
{"LibreDWGc::Dwg_Entity_LIGHT_use_attenuation_limits_get", _wrap_Dwg_Entity_LIGHT_use_attenuation_limits_get},
{"LibreDWGc::Dwg_Entity_LIGHT_attenuation_start_limit_set", _wrap_Dwg_Entity_LIGHT_attenuation_start_limit_set},
{"LibreDWGc::Dwg_Entity_LIGHT_attenuation_start_limit_get", _wrap_Dwg_Entity_LIGHT_attenuation_start_limit_get},
{"LibreDWGc::Dwg_Entity_LIGHT_attenuation_end_limit_set", _wrap_Dwg_Entity_LIGHT_attenuation_end_limit_set},
{"LibreDWGc::Dwg_Entity_LIGHT_attenuation_end_limit_get", _wrap_Dwg_Entity_LIGHT_attenuation_end_limit_get},
{"LibreDWGc::Dwg_Entity_LIGHT_hotspot_angle_set", _wrap_Dwg_Entity_LIGHT_hotspot_angle_set},
{"LibreDWGc::Dwg_Entity_LIGHT_hotspot_angle_get", _wrap_Dwg_Entity_LIGHT_hotspot_angle_get},
{"LibreDWGc::Dwg_Entity_LIGHT_falloff_angle_set", _wrap_Dwg_Entity_LIGHT_falloff_angle_set},
{"LibreDWGc::Dwg_Entity_LIGHT_falloff_angle_get", _wrap_Dwg_Entity_LIGHT_falloff_angle_get},
{"LibreDWGc::Dwg_Entity_LIGHT_cast_shadows_set", _wrap_Dwg_Entity_LIGHT_cast_shadows_set},
{"LibreDWGc::Dwg_Entity_LIGHT_cast_shadows_get", _wrap_Dwg_Entity_LIGHT_cast_shadows_get},
{"LibreDWGc::Dwg_Entity_LIGHT_shadow_type_set", _wrap_Dwg_Entity_LIGHT_shadow_type_set},
{"LibreDWGc::Dwg_Entity_LIGHT_shadow_type_get", _wrap_Dwg_Entity_LIGHT_shadow_type_get},
{"LibreDWGc::Dwg_Entity_LIGHT_shadow_map_size_set", _wrap_Dwg_Entity_LIGHT_shadow_map_size_set},
{"LibreDWGc::Dwg_Entity_LIGHT_shadow_map_size_get", _wrap_Dwg_Entity_LIGHT_shadow_map_size_get},
{"LibreDWGc::Dwg_Entity_LIGHT_shadow_map_softness_set", _wrap_Dwg_Entity_LIGHT_shadow_map_softness_set},
{"LibreDWGc::Dwg_Entity_LIGHT_shadow_map_softness_get", _wrap_Dwg_Entity_LIGHT_shadow_map_softness_get},
{"LibreDWGc::Dwg_Entity_LIGHT_is_photometric_set", _wrap_Dwg_Entity_LIGHT_is_photometric_set},
{"LibreDWGc::Dwg_Entity_LIGHT_is_photometric_get", _wrap_Dwg_Entity_LIGHT_is_photometric_get},
{"LibreDWGc::Dwg_Entity_LIGHT_has_photometric_data_set", _wrap_Dwg_Entity_LIGHT_has_photometric_data_set},
{"LibreDWGc::Dwg_Entity_LIGHT_has_photometric_data_get", _wrap_Dwg_Entity_LIGHT_has_photometric_data_get},
{"LibreDWGc::Dwg_Entity_LIGHT_has_webfile_set", _wrap_Dwg_Entity_LIGHT_has_webfile_set},
{"LibreDWGc::Dwg_Entity_LIGHT_has_webfile_get", _wrap_Dwg_Entity_LIGHT_has_webfile_get},
{"LibreDWGc::Dwg_Entity_LIGHT_webfile_set", _wrap_Dwg_Entity_LIGHT_webfile_set},
{"LibreDWGc::Dwg_Entity_LIGHT_webfile_get", _wrap_Dwg_Entity_LIGHT_webfile_get},
{"LibreDWGc::Dwg_Entity_LIGHT_physical_intensity_method_set", _wrap_Dwg_Entity_LIGHT_physical_intensity_method_set},
{"LibreDWGc::Dwg_Entity_LIGHT_physical_intensity_method_get", _wrap_Dwg_Entity_LIGHT_physical_intensity_method_get},
{"LibreDWGc::Dwg_Entity_LIGHT_physical_intensity_set", _wrap_Dwg_Entity_LIGHT_physical_intensity_set},
{"LibreDWGc::Dwg_Entity_LIGHT_physical_intensity_get", _wrap_Dwg_Entity_LIGHT_physical_intensity_get},
{"LibreDWGc::Dwg_Entity_LIGHT_illuminance_dist_set", _wrap_Dwg_Entity_LIGHT_illuminance_dist_set},
{"LibreDWGc::Dwg_Entity_LIGHT_illuminance_dist_get", _wrap_Dwg_Entity_LIGHT_illuminance_dist_get},
{"LibreDWGc::Dwg_Entity_LIGHT_lamp_color_type_set", _wrap_Dwg_Entity_LIGHT_lamp_color_type_set},
{"LibreDWGc::Dwg_Entity_LIGHT_lamp_color_type_get", _wrap_Dwg_Entity_LIGHT_lamp_color_type_get},
{"LibreDWGc::Dwg_Entity_LIGHT_lamp_color_temp_set", _wrap_Dwg_Entity_LIGHT_lamp_color_temp_set},
{"LibreDWGc::Dwg_Entity_LIGHT_lamp_color_temp_get", _wrap_Dwg_Entity_LIGHT_lamp_color_temp_get},
{"LibreDWGc::Dwg_Entity_LIGHT_lamp_color_preset_set", _wrap_Dwg_Entity_LIGHT_lamp_color_preset_set},
{"LibreDWGc::Dwg_Entity_LIGHT_lamp_color_preset_get", _wrap_Dwg_Entity_LIGHT_lamp_color_preset_get},
{"LibreDWGc::Dwg_Entity_LIGHT_lamp_color_rgb_set", _wrap_Dwg_Entity_LIGHT_lamp_color_rgb_set},
{"LibreDWGc::Dwg_Entity_LIGHT_lamp_color_rgb_get", _wrap_Dwg_Entity_LIGHT_lamp_color_rgb_get},
{"LibreDWGc::Dwg_Entity_LIGHT_web_rotation_set", _wrap_Dwg_Entity_LIGHT_web_rotation_set},
{"LibreDWGc::Dwg_Entity_LIGHT_web_rotation_get", _wrap_Dwg_Entity_LIGHT_web_rotation_get},
{"LibreDWGc::Dwg_Entity_LIGHT_extlight_shape_set", _wrap_Dwg_Entity_LIGHT_extlight_shape_set},
{"LibreDWGc::Dwg_Entity_LIGHT_extlight_shape_get", _wrap_Dwg_Entity_LIGHT_extlight_shape_get},
{"LibreDWGc::Dwg_Entity_LIGHT_extlight_length_set", _wrap_Dwg_Entity_LIGHT_extlight_length_set},
{"LibreDWGc::Dwg_Entity_LIGHT_extlight_length_get", _wrap_Dwg_Entity_LIGHT_extlight_length_get},
{"LibreDWGc::Dwg_Entity_LIGHT_extlight_width_set", _wrap_Dwg_Entity_LIGHT_extlight_width_set},
{"LibreDWGc::Dwg_Entity_LIGHT_extlight_width_get", _wrap_Dwg_Entity_LIGHT_extlight_width_get},
{"LibreDWGc::Dwg_Entity_LIGHT_extlight_radius_set", _wrap_Dwg_Entity_LIGHT_extlight_radius_set},
{"LibreDWGc::Dwg_Entity_LIGHT_extlight_radius_get", _wrap_Dwg_Entity_LIGHT_extlight_radius_get},
{"LibreDWGc::Dwg_Entity_LIGHT_webfile_type_set", _wrap_Dwg_Entity_LIGHT_webfile_type_set},
{"LibreDWGc::Dwg_Entity_LIGHT_webfile_type_get", _wrap_Dwg_Entity_LIGHT_webfile_type_get},
{"LibreDWGc::Dwg_Entity_LIGHT_web_symetry_set", _wrap_Dwg_Entity_LIGHT_web_symetry_set},
{"LibreDWGc::Dwg_Entity_LIGHT_web_symetry_get", _wrap_Dwg_Entity_LIGHT_web_symetry_get},
{"LibreDWGc::Dwg_Entity_LIGHT_has_target_grip_set", _wrap_Dwg_Entity_LIGHT_has_target_grip_set},
{"LibreDWGc::Dwg_Entity_LIGHT_has_target_grip_get", _wrap_Dwg_Entity_LIGHT_has_target_grip_get},
{"LibreDWGc::Dwg_Entity_LIGHT_web_flux_set", _wrap_Dwg_Entity_LIGHT_web_flux_set},
{"LibreDWGc::Dwg_Entity_LIGHT_web_flux_get", _wrap_Dwg_Entity_LIGHT_web_flux_get},
{"LibreDWGc::Dwg_Entity_LIGHT_web_angle1_set", _wrap_Dwg_Entity_LIGHT_web_angle1_set},
{"LibreDWGc::Dwg_Entity_LIGHT_web_angle1_get", _wrap_Dwg_Entity_LIGHT_web_angle1_get},
{"LibreDWGc::Dwg_Entity_LIGHT_web_angle2_set", _wrap_Dwg_Entity_LIGHT_web_angle2_set},
{"LibreDWGc::Dwg_Entity_LIGHT_web_angle2_get", _wrap_Dwg_Entity_LIGHT_web_angle2_get},
{"LibreDWGc::Dwg_Entity_LIGHT_web_angle3_set", _wrap_Dwg_Entity_LIGHT_web_angle3_set},
{"LibreDWGc::Dwg_Entity_LIGHT_web_angle3_get", _wrap_Dwg_Entity_LIGHT_web_angle3_get},
{"LibreDWGc::Dwg_Entity_LIGHT_web_angle4_set", _wrap_Dwg_Entity_LIGHT_web_angle4_set},
{"LibreDWGc::Dwg_Entity_LIGHT_web_angle4_get", _wrap_Dwg_Entity_LIGHT_web_angle4_get},
{"LibreDWGc::Dwg_Entity_LIGHT_web_angle5_set", _wrap_Dwg_Entity_LIGHT_web_angle5_set},
{"LibreDWGc::Dwg_Entity_LIGHT_web_angle5_get", _wrap_Dwg_Entity_LIGHT_web_angle5_get},
{"LibreDWGc::Dwg_Entity_LIGHT_glyph_display_type_set", _wrap_Dwg_Entity_LIGHT_glyph_display_type_set},
{"LibreDWGc::Dwg_Entity_LIGHT_glyph_display_type_get", _wrap_Dwg_Entity_LIGHT_glyph_display_type_get},
{"LibreDWGc::new_Dwg_Entity_LIGHT", _wrap_new_Dwg_Entity_LIGHT},
{"LibreDWGc::delete_Dwg_Entity_LIGHT", _wrap_delete_Dwg_Entity_LIGHT},
{"LibreDWGc::Dwg_Entity_CAMERA_parent_set", _wrap_Dwg_Entity_CAMERA_parent_set},
{"LibreDWGc::Dwg_Entity_CAMERA_parent_get", _wrap_Dwg_Entity_CAMERA_parent_get},
{"LibreDWGc::Dwg_Entity_CAMERA_view_set", _wrap_Dwg_Entity_CAMERA_view_set},
{"LibreDWGc::Dwg_Entity_CAMERA_view_get", _wrap_Dwg_Entity_CAMERA_view_get},
{"LibreDWGc::new_Dwg_Entity_CAMERA", _wrap_new_Dwg_Entity_CAMERA},
{"LibreDWGc::delete_Dwg_Entity_CAMERA", _wrap_delete_Dwg_Entity_CAMERA},
{"LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_parent_set", _wrap_Dwg_Entity_GEOPOSITIONMARKER_parent_set},
{"LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_parent_get", _wrap_Dwg_Entity_GEOPOSITIONMARKER_parent_get},
{"LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_class_version_set", _wrap_Dwg_Entity_GEOPOSITIONMARKER_class_version_set},
{"LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_class_version_get", _wrap_Dwg_Entity_GEOPOSITIONMARKER_class_version_get},
{"LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_position_set", _wrap_Dwg_Entity_GEOPOSITIONMARKER_position_set},
{"LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_position_get", _wrap_Dwg_Entity_GEOPOSITIONMARKER_position_get},
{"LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_radius_set", _wrap_Dwg_Entity_GEOPOSITIONMARKER_radius_set},
{"LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_radius_get", _wrap_Dwg_Entity_GEOPOSITIONMARKER_radius_get},
{"LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_landing_gap_set", _wrap_Dwg_Entity_GEOPOSITIONMARKER_landing_gap_set},
{"LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_landing_gap_get", _wrap_Dwg_Entity_GEOPOSITIONMARKER_landing_gap_get},
{"LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_notes_set", _wrap_Dwg_Entity_GEOPOSITIONMARKER_notes_set},
{"LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_notes_get", _wrap_Dwg_Entity_GEOPOSITIONMARKER_notes_get},
{"LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_text_alignment_set", _wrap_Dwg_Entity_GEOPOSITIONMARKER_text_alignment_set},
{"LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_text_alignment_get", _wrap_Dwg_Entity_GEOPOSITIONMARKER_text_alignment_get},
{"LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_mtext_visible_set", _wrap_Dwg_Entity_GEOPOSITIONMARKER_mtext_visible_set},
{"LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_mtext_visible_get", _wrap_Dwg_Entity_GEOPOSITIONMARKER_mtext_visible_get},
{"LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_enable_frame_text_set", _wrap_Dwg_Entity_GEOPOSITIONMARKER_enable_frame_text_set},
{"LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_enable_frame_text_get", _wrap_Dwg_Entity_GEOPOSITIONMARKER_enable_frame_text_get},
{"LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_ins_pt_set", _wrap_Dwg_Entity_GEOPOSITIONMARKER_ins_pt_set},
{"LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_ins_pt_get", _wrap_Dwg_Entity_GEOPOSITIONMARKER_ins_pt_get},
{"LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_alignment_pt_set", _wrap_Dwg_Entity_GEOPOSITIONMARKER_alignment_pt_set},
{"LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_alignment_pt_get", _wrap_Dwg_Entity_GEOPOSITIONMARKER_alignment_pt_get},
{"LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_mtext_set", _wrap_Dwg_Entity_GEOPOSITIONMARKER_mtext_set},
{"LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_mtext_get", _wrap_Dwg_Entity_GEOPOSITIONMARKER_mtext_get},
{"LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_is_really_locked_set", _wrap_Dwg_Entity_GEOPOSITIONMARKER_is_really_locked_set},
{"LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_is_really_locked_get", _wrap_Dwg_Entity_GEOPOSITIONMARKER_is_really_locked_get},
{"LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_annotative_data_size_set", _wrap_Dwg_Entity_GEOPOSITIONMARKER_annotative_data_size_set},
{"LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_annotative_data_size_get", _wrap_Dwg_Entity_GEOPOSITIONMARKER_annotative_data_size_get},
{"LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_annotative_data_set", _wrap_Dwg_Entity_GEOPOSITIONMARKER_annotative_data_set},
{"LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_annotative_data_get", _wrap_Dwg_Entity_GEOPOSITIONMARKER_annotative_data_get},
{"LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_annotative_flag_set", _wrap_Dwg_Entity_GEOPOSITIONMARKER_annotative_flag_set},
{"LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_annotative_flag_get", _wrap_Dwg_Entity_GEOPOSITIONMARKER_annotative_flag_get},
{"LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_annotative_style_set", _wrap_Dwg_Entity_GEOPOSITIONMARKER_annotative_style_set},
{"LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_annotative_style_get", _wrap_Dwg_Entity_GEOPOSITIONMARKER_annotative_style_get},
{"LibreDWGc::new_Dwg_Entity_GEOPOSITIONMARKER", _wrap_new_Dwg_Entity_GEOPOSITIONMARKER},
{"LibreDWGc::delete_Dwg_Entity_GEOPOSITIONMARKER", _wrap_delete_Dwg_Entity_GEOPOSITIONMARKER},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_parent_set", _wrap_Dwg_Object_GEOMAPIMAGE_parent_set},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_parent_get", _wrap_Dwg_Object_GEOMAPIMAGE_parent_get},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_class_version_set", _wrap_Dwg_Object_GEOMAPIMAGE_class_version_set},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_class_version_get", _wrap_Dwg_Object_GEOMAPIMAGE_class_version_get},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_pt0_set", _wrap_Dwg_Object_GEOMAPIMAGE_pt0_set},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_pt0_get", _wrap_Dwg_Object_GEOMAPIMAGE_pt0_get},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_image_size_set", _wrap_Dwg_Object_GEOMAPIMAGE_image_size_set},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_image_size_get", _wrap_Dwg_Object_GEOMAPIMAGE_image_size_get},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_display_props_set", _wrap_Dwg_Object_GEOMAPIMAGE_display_props_set},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_display_props_get", _wrap_Dwg_Object_GEOMAPIMAGE_display_props_get},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_clipping_set", _wrap_Dwg_Object_GEOMAPIMAGE_clipping_set},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_clipping_get", _wrap_Dwg_Object_GEOMAPIMAGE_clipping_get},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_brightness_set", _wrap_Dwg_Object_GEOMAPIMAGE_brightness_set},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_brightness_get", _wrap_Dwg_Object_GEOMAPIMAGE_brightness_get},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_contrast_set", _wrap_Dwg_Object_GEOMAPIMAGE_contrast_set},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_contrast_get", _wrap_Dwg_Object_GEOMAPIMAGE_contrast_get},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_fade_set", _wrap_Dwg_Object_GEOMAPIMAGE_fade_set},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_fade_get", _wrap_Dwg_Object_GEOMAPIMAGE_fade_get},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_rotation_set", _wrap_Dwg_Object_GEOMAPIMAGE_rotation_set},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_rotation_get", _wrap_Dwg_Object_GEOMAPIMAGE_rotation_get},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_image_width_set", _wrap_Dwg_Object_GEOMAPIMAGE_image_width_set},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_image_width_get", _wrap_Dwg_Object_GEOMAPIMAGE_image_width_get},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_image_height_set", _wrap_Dwg_Object_GEOMAPIMAGE_image_height_set},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_image_height_get", _wrap_Dwg_Object_GEOMAPIMAGE_image_height_get},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_name_set", _wrap_Dwg_Object_GEOMAPIMAGE_name_set},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_name_get", _wrap_Dwg_Object_GEOMAPIMAGE_name_get},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_image_file_set", _wrap_Dwg_Object_GEOMAPIMAGE_image_file_set},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_image_file_get", _wrap_Dwg_Object_GEOMAPIMAGE_image_file_get},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_image_visibility_set", _wrap_Dwg_Object_GEOMAPIMAGE_image_visibility_set},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_image_visibility_get", _wrap_Dwg_Object_GEOMAPIMAGE_image_visibility_get},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_transparency_set", _wrap_Dwg_Object_GEOMAPIMAGE_transparency_set},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_transparency_get", _wrap_Dwg_Object_GEOMAPIMAGE_transparency_get},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_height_set", _wrap_Dwg_Object_GEOMAPIMAGE_height_set},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_height_get", _wrap_Dwg_Object_GEOMAPIMAGE_height_get},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_width_set", _wrap_Dwg_Object_GEOMAPIMAGE_width_set},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_width_get", _wrap_Dwg_Object_GEOMAPIMAGE_width_get},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_show_rotation_set", _wrap_Dwg_Object_GEOMAPIMAGE_show_rotation_set},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_show_rotation_get", _wrap_Dwg_Object_GEOMAPIMAGE_show_rotation_get},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_scale_factor_set", _wrap_Dwg_Object_GEOMAPIMAGE_scale_factor_set},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_scale_factor_get", _wrap_Dwg_Object_GEOMAPIMAGE_scale_factor_get},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_geoimage_brightness_set", _wrap_Dwg_Object_GEOMAPIMAGE_geoimage_brightness_set},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_geoimage_brightness_get", _wrap_Dwg_Object_GEOMAPIMAGE_geoimage_brightness_get},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_geoimage_contrast_set", _wrap_Dwg_Object_GEOMAPIMAGE_geoimage_contrast_set},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_geoimage_contrast_get", _wrap_Dwg_Object_GEOMAPIMAGE_geoimage_contrast_get},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_geoimage_fade_set", _wrap_Dwg_Object_GEOMAPIMAGE_geoimage_fade_set},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_geoimage_fade_get", _wrap_Dwg_Object_GEOMAPIMAGE_geoimage_fade_get},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_geoimage_position_set", _wrap_Dwg_Object_GEOMAPIMAGE_geoimage_position_set},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_geoimage_position_get", _wrap_Dwg_Object_GEOMAPIMAGE_geoimage_position_get},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_geoimage_width_set", _wrap_Dwg_Object_GEOMAPIMAGE_geoimage_width_set},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_geoimage_width_get", _wrap_Dwg_Object_GEOMAPIMAGE_geoimage_width_get},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_geoimage_height_set", _wrap_Dwg_Object_GEOMAPIMAGE_geoimage_height_set},
{"LibreDWGc::Dwg_Object_GEOMAPIMAGE_geoimage_height_get", _wrap_Dwg_Object_GEOMAPIMAGE_geoimage_height_get},
{"LibreDWGc::new_Dwg_Object_GEOMAPIMAGE", _wrap_new_Dwg_Object_GEOMAPIMAGE},
{"LibreDWGc::delete_Dwg_Object_GEOMAPIMAGE", _wrap_delete_Dwg_Object_GEOMAPIMAGE},
{"LibreDWGc::Dwg_Entity_HELIX_parent_set", _wrap_Dwg_Entity_HELIX_parent_set},
{"LibreDWGc::Dwg_Entity_HELIX_parent_get", _wrap_Dwg_Entity_HELIX_parent_get},
{"LibreDWGc::Dwg_Entity_HELIX_flag_set", _wrap_Dwg_Entity_HELIX_flag_set},
{"LibreDWGc::Dwg_Entity_HELIX_flag_get", _wrap_Dwg_Entity_HELIX_flag_get},
{"LibreDWGc::Dwg_Entity_HELIX_scenario_set", _wrap_Dwg_Entity_HELIX_scenario_set},
{"LibreDWGc::Dwg_Entity_HELIX_scenario_get", _wrap_Dwg_Entity_HELIX_scenario_get},
{"LibreDWGc::Dwg_Entity_HELIX_degree_set", _wrap_Dwg_Entity_HELIX_degree_set},
{"LibreDWGc::Dwg_Entity_HELIX_degree_get", _wrap_Dwg_Entity_HELIX_degree_get},
{"LibreDWGc::Dwg_Entity_HELIX_splineflags_set", _wrap_Dwg_Entity_HELIX_splineflags_set},
{"LibreDWGc::Dwg_Entity_HELIX_splineflags_get", _wrap_Dwg_Entity_HELIX_splineflags_get},
{"LibreDWGc::Dwg_Entity_HELIX_knotparam_set", _wrap_Dwg_Entity_HELIX_knotparam_set},
{"LibreDWGc::Dwg_Entity_HELIX_knotparam_get", _wrap_Dwg_Entity_HELIX_knotparam_get},
{"LibreDWGc::Dwg_Entity_HELIX_fit_tol_set", _wrap_Dwg_Entity_HELIX_fit_tol_set},
{"LibreDWGc::Dwg_Entity_HELIX_fit_tol_get", _wrap_Dwg_Entity_HELIX_fit_tol_get},
{"LibreDWGc::Dwg_Entity_HELIX_beg_tan_vec_set", _wrap_Dwg_Entity_HELIX_beg_tan_vec_set},
{"LibreDWGc::Dwg_Entity_HELIX_beg_tan_vec_get", _wrap_Dwg_Entity_HELIX_beg_tan_vec_get},
{"LibreDWGc::Dwg_Entity_HELIX_end_tan_vec_set", _wrap_Dwg_Entity_HELIX_end_tan_vec_set},
{"LibreDWGc::Dwg_Entity_HELIX_end_tan_vec_get", _wrap_Dwg_Entity_HELIX_end_tan_vec_get},
{"LibreDWGc::Dwg_Entity_HELIX_rational_set", _wrap_Dwg_Entity_HELIX_rational_set},
{"LibreDWGc::Dwg_Entity_HELIX_rational_get", _wrap_Dwg_Entity_HELIX_rational_get},
{"LibreDWGc::Dwg_Entity_HELIX_closed_b_set", _wrap_Dwg_Entity_HELIX_closed_b_set},
{"LibreDWGc::Dwg_Entity_HELIX_closed_b_get", _wrap_Dwg_Entity_HELIX_closed_b_get},
{"LibreDWGc::Dwg_Entity_HELIX_periodic_set", _wrap_Dwg_Entity_HELIX_periodic_set},
{"LibreDWGc::Dwg_Entity_HELIX_periodic_get", _wrap_Dwg_Entity_HELIX_periodic_get},
{"LibreDWGc::Dwg_Entity_HELIX_weighted_set", _wrap_Dwg_Entity_HELIX_weighted_set},
{"LibreDWGc::Dwg_Entity_HELIX_weighted_get", _wrap_Dwg_Entity_HELIX_weighted_get},
{"LibreDWGc::Dwg_Entity_HELIX_knot_tol_set", _wrap_Dwg_Entity_HELIX_knot_tol_set},
{"LibreDWGc::Dwg_Entity_HELIX_knot_tol_get", _wrap_Dwg_Entity_HELIX_knot_tol_get},
{"LibreDWGc::Dwg_Entity_HELIX_ctrl_tol_set", _wrap_Dwg_Entity_HELIX_ctrl_tol_set},
{"LibreDWGc::Dwg_Entity_HELIX_ctrl_tol_get", _wrap_Dwg_Entity_HELIX_ctrl_tol_get},
{"LibreDWGc::Dwg_Entity_HELIX_num_fit_pts_set", _wrap_Dwg_Entity_HELIX_num_fit_pts_set},
{"LibreDWGc::Dwg_Entity_HELIX_num_fit_pts_get", _wrap_Dwg_Entity_HELIX_num_fit_pts_get},
{"LibreDWGc::Dwg_Entity_HELIX_fit_pts_set", _wrap_Dwg_Entity_HELIX_fit_pts_set},
{"LibreDWGc::Dwg_Entity_HELIX_fit_pts_get", _wrap_Dwg_Entity_HELIX_fit_pts_get},
{"LibreDWGc::Dwg_Entity_HELIX_num_knots_set", _wrap_Dwg_Entity_HELIX_num_knots_set},
{"LibreDWGc::Dwg_Entity_HELIX_num_knots_get", _wrap_Dwg_Entity_HELIX_num_knots_get},
{"LibreDWGc::Dwg_Entity_HELIX_knots_set", _wrap_Dwg_Entity_HELIX_knots_set},
{"LibreDWGc::Dwg_Entity_HELIX_knots_get", _wrap_Dwg_Entity_HELIX_knots_get},
{"LibreDWGc::Dwg_Entity_HELIX_num_ctrl_pts_set", _wrap_Dwg_Entity_HELIX_num_ctrl_pts_set},
{"LibreDWGc::Dwg_Entity_HELIX_num_ctrl_pts_get", _wrap_Dwg_Entity_HELIX_num_ctrl_pts_get},
{"LibreDWGc::Dwg_Entity_HELIX_ctrl_pts_set", _wrap_Dwg_Entity_HELIX_ctrl_pts_set},
{"LibreDWGc::Dwg_Entity_HELIX_ctrl_pts_get", _wrap_Dwg_Entity_HELIX_ctrl_pts_get},
{"LibreDWGc::Dwg_Entity_HELIX_major_version_set", _wrap_Dwg_Entity_HELIX_major_version_set},
{"LibreDWGc::Dwg_Entity_HELIX_major_version_get", _wrap_Dwg_Entity_HELIX_major_version_get},
{"LibreDWGc::Dwg_Entity_HELIX_maint_version_set", _wrap_Dwg_Entity_HELIX_maint_version_set},
{"LibreDWGc::Dwg_Entity_HELIX_maint_version_get", _wrap_Dwg_Entity_HELIX_maint_version_get},
{"LibreDWGc::Dwg_Entity_HELIX_axis_base_pt_set", _wrap_Dwg_Entity_HELIX_axis_base_pt_set},
{"LibreDWGc::Dwg_Entity_HELIX_axis_base_pt_get", _wrap_Dwg_Entity_HELIX_axis_base_pt_get},
{"LibreDWGc::Dwg_Entity_HELIX_start_pt_set", _wrap_Dwg_Entity_HELIX_start_pt_set},
{"LibreDWGc::Dwg_Entity_HELIX_start_pt_get", _wrap_Dwg_Entity_HELIX_start_pt_get},
{"LibreDWGc::Dwg_Entity_HELIX_axis_vector_set", _wrap_Dwg_Entity_HELIX_axis_vector_set},
{"LibreDWGc::Dwg_Entity_HELIX_axis_vector_get", _wrap_Dwg_Entity_HELIX_axis_vector_get},
{"LibreDWGc::Dwg_Entity_HELIX_radius_set", _wrap_Dwg_Entity_HELIX_radius_set},
{"LibreDWGc::Dwg_Entity_HELIX_radius_get", _wrap_Dwg_Entity_HELIX_radius_get},
{"LibreDWGc::Dwg_Entity_HELIX_turns_set", _wrap_Dwg_Entity_HELIX_turns_set},
{"LibreDWGc::Dwg_Entity_HELIX_turns_get", _wrap_Dwg_Entity_HELIX_turns_get},
{"LibreDWGc::Dwg_Entity_HELIX_turn_height_set", _wrap_Dwg_Entity_HELIX_turn_height_set},
{"LibreDWGc::Dwg_Entity_HELIX_turn_height_get", _wrap_Dwg_Entity_HELIX_turn_height_get},
{"LibreDWGc::Dwg_Entity_HELIX_handedness_set", _wrap_Dwg_Entity_HELIX_handedness_set},
{"LibreDWGc::Dwg_Entity_HELIX_handedness_get", _wrap_Dwg_Entity_HELIX_handedness_get},
{"LibreDWGc::Dwg_Entity_HELIX_constraint_type_set", _wrap_Dwg_Entity_HELIX_constraint_type_set},
{"LibreDWGc::Dwg_Entity_HELIX_constraint_type_get", _wrap_Dwg_Entity_HELIX_constraint_type_get},
{"LibreDWGc::new_Dwg_Entity_HELIX", _wrap_new_Dwg_Entity_HELIX},
{"LibreDWGc::delete_Dwg_Entity_HELIX", _wrap_delete_Dwg_Entity_HELIX},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_parent_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_parent_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_parent_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_parent_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_acis_empty_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_acis_empty_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_acis_empty_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_acis_empty_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_unknown_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_unknown_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_unknown_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_unknown_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_version_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_version_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_version_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_version_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_num_blocks_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_num_blocks_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_num_blocks_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_num_blocks_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_block_size_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_block_size_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_block_size_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_block_size_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_encr_sat_data_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_encr_sat_data_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_encr_sat_data_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_encr_sat_data_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_sab_size_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_sab_size_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_sab_size_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_sab_size_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_acis_data_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_acis_data_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_acis_data_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_acis_data_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_wireframe_data_present_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_wireframe_data_present_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_wireframe_data_present_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_wireframe_data_present_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_point_present_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_point_present_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_point_present_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_point_present_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_point_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_point_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_point_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_point_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_isolines_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_isolines_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_isolines_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_isolines_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_isoline_present_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_isoline_present_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_isoline_present_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_isoline_present_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_num_wires_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_num_wires_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_num_wires_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_num_wires_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_wires_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_wires_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_wires_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_wires_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_num_silhouettes_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_num_silhouettes_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_num_silhouettes_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_num_silhouettes_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_silhouettes_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_silhouettes_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_silhouettes_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_silhouettes_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE__dxf_sab_converted_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE__dxf_sab_converted_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE__dxf_sab_converted_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE__dxf_sab_converted_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_acis_empty2_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_acis_empty2_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_acis_empty2_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_acis_empty2_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_extra_acis_data_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_extra_acis_data_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_extra_acis_data_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_extra_acis_data_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_num_materials_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_num_materials_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_num_materials_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_num_materials_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_materials_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_materials_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_materials_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_materials_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_revision_guid_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_revision_guid_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_revision_guid_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_revision_guid_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_revision_major_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_revision_major_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_revision_major_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_revision_major_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_revision_minor1_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_revision_minor1_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_revision_minor1_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_revision_minor1_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_revision_minor2_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_revision_minor2_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_revision_minor2_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_revision_minor2_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_revision_bytes_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_revision_bytes_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_revision_bytes_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_revision_bytes_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_end_marker_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_end_marker_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_end_marker_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_end_marker_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_history_id_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_history_id_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_history_id_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_history_id_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_has_revision_guid_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_has_revision_guid_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_has_revision_guid_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_has_revision_guid_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_acis_empty_bit_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_acis_empty_bit_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_acis_empty_bit_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_acis_empty_bit_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_modeler_format_version_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_modeler_format_version_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_modeler_format_version_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_modeler_format_version_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_bindata_size_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_bindata_size_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_bindata_size_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_bindata_size_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_bindata_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_bindata_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_bindata_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_bindata_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_u_isolines_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_u_isolines_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_u_isolines_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_u_isolines_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_v_isolines_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_v_isolines_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_v_isolines_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_v_isolines_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_class_version_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_class_version_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_class_version_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_class_version_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_draft_angle_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_draft_angle_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_draft_angle_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_draft_angle_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_draft_start_distance_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_draft_start_distance_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_draft_start_distance_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_draft_start_distance_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_draft_end_distance_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_draft_end_distance_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_draft_end_distance_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_draft_end_distance_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_twist_angle_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_twist_angle_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_twist_angle_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_twist_angle_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_scale_factor_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_scale_factor_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_scale_factor_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_scale_factor_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_align_angle_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_align_angle_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_align_angle_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_align_angle_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_transmatrix_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_transmatrix_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_transmatrix_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_transmatrix_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_path_entity_transmatrix_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_path_entity_transmatrix_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_path_entity_transmatrix_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_path_entity_transmatrix_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_is_solid_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_is_solid_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_is_solid_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_is_solid_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_sweep_alignment_flags_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_sweep_alignment_flags_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_sweep_alignment_flags_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_sweep_alignment_flags_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_path_flags_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_path_flags_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_path_flags_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_path_flags_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_align_start_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_align_start_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_align_start_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_align_start_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_bank_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_bank_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_bank_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_bank_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_base_point_set_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_base_point_set_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_base_point_set_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_base_point_set_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_transform_computed_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_transform_computed_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_transform_computed_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_transform_computed_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_path_entity_transform_computed_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_path_entity_transform_computed_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_path_entity_transform_computed_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_path_entity_transform_computed_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_reference_vector_for_controlling_twist_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_reference_vector_for_controlling_twist_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_reference_vector_for_controlling_twist_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_reference_vector_for_controlling_twist_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_path_entity_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_path_entity_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_path_entity_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_path_entity_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_sweep_vector_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_sweep_vector_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_sweep_vector_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_sweep_vector_get},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_sweep_transmatrix_set", _wrap_Dwg_Entity_EXTRUDEDSURFACE_sweep_transmatrix_set},
{"LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_sweep_transmatrix_get", _wrap_Dwg_Entity_EXTRUDEDSURFACE_sweep_transmatrix_get},
{"LibreDWGc::new_Dwg_Entity_EXTRUDEDSURFACE", _wrap_new_Dwg_Entity_EXTRUDEDSURFACE},
{"LibreDWGc::delete_Dwg_Entity_EXTRUDEDSURFACE", _wrap_delete_Dwg_Entity_EXTRUDEDSURFACE},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_parent_set", _wrap_Dwg_Entity_SWEPTSURFACE_parent_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_parent_get", _wrap_Dwg_Entity_SWEPTSURFACE_parent_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_acis_empty_set", _wrap_Dwg_Entity_SWEPTSURFACE_acis_empty_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_acis_empty_get", _wrap_Dwg_Entity_SWEPTSURFACE_acis_empty_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_unknown_set", _wrap_Dwg_Entity_SWEPTSURFACE_unknown_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_unknown_get", _wrap_Dwg_Entity_SWEPTSURFACE_unknown_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_version_set", _wrap_Dwg_Entity_SWEPTSURFACE_version_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_version_get", _wrap_Dwg_Entity_SWEPTSURFACE_version_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_num_blocks_set", _wrap_Dwg_Entity_SWEPTSURFACE_num_blocks_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_num_blocks_get", _wrap_Dwg_Entity_SWEPTSURFACE_num_blocks_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_block_size_set", _wrap_Dwg_Entity_SWEPTSURFACE_block_size_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_block_size_get", _wrap_Dwg_Entity_SWEPTSURFACE_block_size_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_encr_sat_data_set", _wrap_Dwg_Entity_SWEPTSURFACE_encr_sat_data_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_encr_sat_data_get", _wrap_Dwg_Entity_SWEPTSURFACE_encr_sat_data_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_sab_size_set", _wrap_Dwg_Entity_SWEPTSURFACE_sab_size_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_sab_size_get", _wrap_Dwg_Entity_SWEPTSURFACE_sab_size_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_acis_data_set", _wrap_Dwg_Entity_SWEPTSURFACE_acis_data_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_acis_data_get", _wrap_Dwg_Entity_SWEPTSURFACE_acis_data_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_wireframe_data_present_set", _wrap_Dwg_Entity_SWEPTSURFACE_wireframe_data_present_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_wireframe_data_present_get", _wrap_Dwg_Entity_SWEPTSURFACE_wireframe_data_present_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_point_present_set", _wrap_Dwg_Entity_SWEPTSURFACE_point_present_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_point_present_get", _wrap_Dwg_Entity_SWEPTSURFACE_point_present_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_point_set", _wrap_Dwg_Entity_SWEPTSURFACE_point_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_point_get", _wrap_Dwg_Entity_SWEPTSURFACE_point_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_isolines_set", _wrap_Dwg_Entity_SWEPTSURFACE_isolines_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_isolines_get", _wrap_Dwg_Entity_SWEPTSURFACE_isolines_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_isoline_present_set", _wrap_Dwg_Entity_SWEPTSURFACE_isoline_present_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_isoline_present_get", _wrap_Dwg_Entity_SWEPTSURFACE_isoline_present_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_num_wires_set", _wrap_Dwg_Entity_SWEPTSURFACE_num_wires_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_num_wires_get", _wrap_Dwg_Entity_SWEPTSURFACE_num_wires_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_wires_set", _wrap_Dwg_Entity_SWEPTSURFACE_wires_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_wires_get", _wrap_Dwg_Entity_SWEPTSURFACE_wires_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_num_silhouettes_set", _wrap_Dwg_Entity_SWEPTSURFACE_num_silhouettes_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_num_silhouettes_get", _wrap_Dwg_Entity_SWEPTSURFACE_num_silhouettes_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_silhouettes_set", _wrap_Dwg_Entity_SWEPTSURFACE_silhouettes_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_silhouettes_get", _wrap_Dwg_Entity_SWEPTSURFACE_silhouettes_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE__dxf_sab_converted_set", _wrap_Dwg_Entity_SWEPTSURFACE__dxf_sab_converted_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE__dxf_sab_converted_get", _wrap_Dwg_Entity_SWEPTSURFACE__dxf_sab_converted_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_acis_empty2_set", _wrap_Dwg_Entity_SWEPTSURFACE_acis_empty2_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_acis_empty2_get", _wrap_Dwg_Entity_SWEPTSURFACE_acis_empty2_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_extra_acis_data_set", _wrap_Dwg_Entity_SWEPTSURFACE_extra_acis_data_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_extra_acis_data_get", _wrap_Dwg_Entity_SWEPTSURFACE_extra_acis_data_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_num_materials_set", _wrap_Dwg_Entity_SWEPTSURFACE_num_materials_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_num_materials_get", _wrap_Dwg_Entity_SWEPTSURFACE_num_materials_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_materials_set", _wrap_Dwg_Entity_SWEPTSURFACE_materials_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_materials_get", _wrap_Dwg_Entity_SWEPTSURFACE_materials_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_revision_guid_set", _wrap_Dwg_Entity_SWEPTSURFACE_revision_guid_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_revision_guid_get", _wrap_Dwg_Entity_SWEPTSURFACE_revision_guid_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_revision_major_set", _wrap_Dwg_Entity_SWEPTSURFACE_revision_major_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_revision_major_get", _wrap_Dwg_Entity_SWEPTSURFACE_revision_major_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_revision_minor1_set", _wrap_Dwg_Entity_SWEPTSURFACE_revision_minor1_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_revision_minor1_get", _wrap_Dwg_Entity_SWEPTSURFACE_revision_minor1_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_revision_minor2_set", _wrap_Dwg_Entity_SWEPTSURFACE_revision_minor2_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_revision_minor2_get", _wrap_Dwg_Entity_SWEPTSURFACE_revision_minor2_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_revision_bytes_set", _wrap_Dwg_Entity_SWEPTSURFACE_revision_bytes_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_revision_bytes_get", _wrap_Dwg_Entity_SWEPTSURFACE_revision_bytes_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_end_marker_set", _wrap_Dwg_Entity_SWEPTSURFACE_end_marker_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_end_marker_get", _wrap_Dwg_Entity_SWEPTSURFACE_end_marker_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_history_id_set", _wrap_Dwg_Entity_SWEPTSURFACE_history_id_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_history_id_get", _wrap_Dwg_Entity_SWEPTSURFACE_history_id_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_has_revision_guid_set", _wrap_Dwg_Entity_SWEPTSURFACE_has_revision_guid_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_has_revision_guid_get", _wrap_Dwg_Entity_SWEPTSURFACE_has_revision_guid_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_acis_empty_bit_set", _wrap_Dwg_Entity_SWEPTSURFACE_acis_empty_bit_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_acis_empty_bit_get", _wrap_Dwg_Entity_SWEPTSURFACE_acis_empty_bit_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_modeler_format_version_set", _wrap_Dwg_Entity_SWEPTSURFACE_modeler_format_version_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_modeler_format_version_get", _wrap_Dwg_Entity_SWEPTSURFACE_modeler_format_version_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_u_isolines_set", _wrap_Dwg_Entity_SWEPTSURFACE_u_isolines_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_u_isolines_get", _wrap_Dwg_Entity_SWEPTSURFACE_u_isolines_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_v_isolines_set", _wrap_Dwg_Entity_SWEPTSURFACE_v_isolines_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_v_isolines_get", _wrap_Dwg_Entity_SWEPTSURFACE_v_isolines_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_class_version_set", _wrap_Dwg_Entity_SWEPTSURFACE_class_version_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_class_version_get", _wrap_Dwg_Entity_SWEPTSURFACE_class_version_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_sweep_entity_id_set", _wrap_Dwg_Entity_SWEPTSURFACE_sweep_entity_id_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_sweep_entity_id_get", _wrap_Dwg_Entity_SWEPTSURFACE_sweep_entity_id_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_sweepdata_size_set", _wrap_Dwg_Entity_SWEPTSURFACE_sweepdata_size_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_sweepdata_size_get", _wrap_Dwg_Entity_SWEPTSURFACE_sweepdata_size_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_sweepdata_set", _wrap_Dwg_Entity_SWEPTSURFACE_sweepdata_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_sweepdata_get", _wrap_Dwg_Entity_SWEPTSURFACE_sweepdata_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_path_entity_id_set", _wrap_Dwg_Entity_SWEPTSURFACE_path_entity_id_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_path_entity_id_get", _wrap_Dwg_Entity_SWEPTSURFACE_path_entity_id_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_pathdata_size_set", _wrap_Dwg_Entity_SWEPTSURFACE_pathdata_size_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_pathdata_size_get", _wrap_Dwg_Entity_SWEPTSURFACE_pathdata_size_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_pathdata_set", _wrap_Dwg_Entity_SWEPTSURFACE_pathdata_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_pathdata_get", _wrap_Dwg_Entity_SWEPTSURFACE_pathdata_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_draft_angle_set", _wrap_Dwg_Entity_SWEPTSURFACE_draft_angle_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_draft_angle_get", _wrap_Dwg_Entity_SWEPTSURFACE_draft_angle_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_draft_start_distance_set", _wrap_Dwg_Entity_SWEPTSURFACE_draft_start_distance_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_draft_start_distance_get", _wrap_Dwg_Entity_SWEPTSURFACE_draft_start_distance_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_draft_end_distance_set", _wrap_Dwg_Entity_SWEPTSURFACE_draft_end_distance_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_draft_end_distance_get", _wrap_Dwg_Entity_SWEPTSURFACE_draft_end_distance_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_twist_angle_set", _wrap_Dwg_Entity_SWEPTSURFACE_twist_angle_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_twist_angle_get", _wrap_Dwg_Entity_SWEPTSURFACE_twist_angle_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_scale_factor_set", _wrap_Dwg_Entity_SWEPTSURFACE_scale_factor_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_scale_factor_get", _wrap_Dwg_Entity_SWEPTSURFACE_scale_factor_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_align_angle_set", _wrap_Dwg_Entity_SWEPTSURFACE_align_angle_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_align_angle_get", _wrap_Dwg_Entity_SWEPTSURFACE_align_angle_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_sweep_entity_transmatrix_set", _wrap_Dwg_Entity_SWEPTSURFACE_sweep_entity_transmatrix_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_sweep_entity_transmatrix_get", _wrap_Dwg_Entity_SWEPTSURFACE_sweep_entity_transmatrix_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_path_entity_transmatrix_set", _wrap_Dwg_Entity_SWEPTSURFACE_path_entity_transmatrix_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_path_entity_transmatrix_get", _wrap_Dwg_Entity_SWEPTSURFACE_path_entity_transmatrix_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_is_solid_set", _wrap_Dwg_Entity_SWEPTSURFACE_is_solid_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_is_solid_get", _wrap_Dwg_Entity_SWEPTSURFACE_is_solid_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_sweep_alignment_flags_set", _wrap_Dwg_Entity_SWEPTSURFACE_sweep_alignment_flags_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_sweep_alignment_flags_get", _wrap_Dwg_Entity_SWEPTSURFACE_sweep_alignment_flags_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_path_flags_set", _wrap_Dwg_Entity_SWEPTSURFACE_path_flags_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_path_flags_get", _wrap_Dwg_Entity_SWEPTSURFACE_path_flags_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_align_start_set", _wrap_Dwg_Entity_SWEPTSURFACE_align_start_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_align_start_get", _wrap_Dwg_Entity_SWEPTSURFACE_align_start_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_bank_set", _wrap_Dwg_Entity_SWEPTSURFACE_bank_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_bank_get", _wrap_Dwg_Entity_SWEPTSURFACE_bank_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_base_point_set_set", _wrap_Dwg_Entity_SWEPTSURFACE_base_point_set_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_base_point_set_get", _wrap_Dwg_Entity_SWEPTSURFACE_base_point_set_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_sweep_entity_transform_computed_set", _wrap_Dwg_Entity_SWEPTSURFACE_sweep_entity_transform_computed_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_sweep_entity_transform_computed_get", _wrap_Dwg_Entity_SWEPTSURFACE_sweep_entity_transform_computed_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_path_entity_transform_computed_set", _wrap_Dwg_Entity_SWEPTSURFACE_path_entity_transform_computed_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_path_entity_transform_computed_get", _wrap_Dwg_Entity_SWEPTSURFACE_path_entity_transform_computed_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_reference_vector_for_controlling_twist_set", _wrap_Dwg_Entity_SWEPTSURFACE_reference_vector_for_controlling_twist_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_reference_vector_for_controlling_twist_get", _wrap_Dwg_Entity_SWEPTSURFACE_reference_vector_for_controlling_twist_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_sweep_entity_set", _wrap_Dwg_Entity_SWEPTSURFACE_sweep_entity_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_sweep_entity_get", _wrap_Dwg_Entity_SWEPTSURFACE_sweep_entity_get},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_path_entity_set", _wrap_Dwg_Entity_SWEPTSURFACE_path_entity_set},
{"LibreDWGc::Dwg_Entity_SWEPTSURFACE_path_entity_get", _wrap_Dwg_Entity_SWEPTSURFACE_path_entity_get},
{"LibreDWGc::new_Dwg_Entity_SWEPTSURFACE", _wrap_new_Dwg_Entity_SWEPTSURFACE},
{"LibreDWGc::delete_Dwg_Entity_SWEPTSURFACE", _wrap_delete_Dwg_Entity_SWEPTSURFACE},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_parent_set", _wrap_Dwg_Entity_LOFTEDSURFACE_parent_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_parent_get", _wrap_Dwg_Entity_LOFTEDSURFACE_parent_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_acis_empty_set", _wrap_Dwg_Entity_LOFTEDSURFACE_acis_empty_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_acis_empty_get", _wrap_Dwg_Entity_LOFTEDSURFACE_acis_empty_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_unknown_set", _wrap_Dwg_Entity_LOFTEDSURFACE_unknown_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_unknown_get", _wrap_Dwg_Entity_LOFTEDSURFACE_unknown_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_version_set", _wrap_Dwg_Entity_LOFTEDSURFACE_version_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_version_get", _wrap_Dwg_Entity_LOFTEDSURFACE_version_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_num_blocks_set", _wrap_Dwg_Entity_LOFTEDSURFACE_num_blocks_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_num_blocks_get", _wrap_Dwg_Entity_LOFTEDSURFACE_num_blocks_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_block_size_set", _wrap_Dwg_Entity_LOFTEDSURFACE_block_size_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_block_size_get", _wrap_Dwg_Entity_LOFTEDSURFACE_block_size_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_encr_sat_data_set", _wrap_Dwg_Entity_LOFTEDSURFACE_encr_sat_data_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_encr_sat_data_get", _wrap_Dwg_Entity_LOFTEDSURFACE_encr_sat_data_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_sab_size_set", _wrap_Dwg_Entity_LOFTEDSURFACE_sab_size_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_sab_size_get", _wrap_Dwg_Entity_LOFTEDSURFACE_sab_size_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_acis_data_set", _wrap_Dwg_Entity_LOFTEDSURFACE_acis_data_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_acis_data_get", _wrap_Dwg_Entity_LOFTEDSURFACE_acis_data_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_wireframe_data_present_set", _wrap_Dwg_Entity_LOFTEDSURFACE_wireframe_data_present_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_wireframe_data_present_get", _wrap_Dwg_Entity_LOFTEDSURFACE_wireframe_data_present_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_point_present_set", _wrap_Dwg_Entity_LOFTEDSURFACE_point_present_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_point_present_get", _wrap_Dwg_Entity_LOFTEDSURFACE_point_present_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_point_set", _wrap_Dwg_Entity_LOFTEDSURFACE_point_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_point_get", _wrap_Dwg_Entity_LOFTEDSURFACE_point_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_isolines_set", _wrap_Dwg_Entity_LOFTEDSURFACE_isolines_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_isolines_get", _wrap_Dwg_Entity_LOFTEDSURFACE_isolines_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_isoline_present_set", _wrap_Dwg_Entity_LOFTEDSURFACE_isoline_present_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_isoline_present_get", _wrap_Dwg_Entity_LOFTEDSURFACE_isoline_present_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_num_wires_set", _wrap_Dwg_Entity_LOFTEDSURFACE_num_wires_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_num_wires_get", _wrap_Dwg_Entity_LOFTEDSURFACE_num_wires_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_wires_set", _wrap_Dwg_Entity_LOFTEDSURFACE_wires_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_wires_get", _wrap_Dwg_Entity_LOFTEDSURFACE_wires_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_num_silhouettes_set", _wrap_Dwg_Entity_LOFTEDSURFACE_num_silhouettes_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_num_silhouettes_get", _wrap_Dwg_Entity_LOFTEDSURFACE_num_silhouettes_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_silhouettes_set", _wrap_Dwg_Entity_LOFTEDSURFACE_silhouettes_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_silhouettes_get", _wrap_Dwg_Entity_LOFTEDSURFACE_silhouettes_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE__dxf_sab_converted_set", _wrap_Dwg_Entity_LOFTEDSURFACE__dxf_sab_converted_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE__dxf_sab_converted_get", _wrap_Dwg_Entity_LOFTEDSURFACE__dxf_sab_converted_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_acis_empty2_set", _wrap_Dwg_Entity_LOFTEDSURFACE_acis_empty2_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_acis_empty2_get", _wrap_Dwg_Entity_LOFTEDSURFACE_acis_empty2_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_extra_acis_data_set", _wrap_Dwg_Entity_LOFTEDSURFACE_extra_acis_data_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_extra_acis_data_get", _wrap_Dwg_Entity_LOFTEDSURFACE_extra_acis_data_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_num_materials_set", _wrap_Dwg_Entity_LOFTEDSURFACE_num_materials_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_num_materials_get", _wrap_Dwg_Entity_LOFTEDSURFACE_num_materials_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_materials_set", _wrap_Dwg_Entity_LOFTEDSURFACE_materials_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_materials_get", _wrap_Dwg_Entity_LOFTEDSURFACE_materials_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_revision_guid_set", _wrap_Dwg_Entity_LOFTEDSURFACE_revision_guid_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_revision_guid_get", _wrap_Dwg_Entity_LOFTEDSURFACE_revision_guid_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_revision_major_set", _wrap_Dwg_Entity_LOFTEDSURFACE_revision_major_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_revision_major_get", _wrap_Dwg_Entity_LOFTEDSURFACE_revision_major_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_revision_minor1_set", _wrap_Dwg_Entity_LOFTEDSURFACE_revision_minor1_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_revision_minor1_get", _wrap_Dwg_Entity_LOFTEDSURFACE_revision_minor1_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_revision_minor2_set", _wrap_Dwg_Entity_LOFTEDSURFACE_revision_minor2_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_revision_minor2_get", _wrap_Dwg_Entity_LOFTEDSURFACE_revision_minor2_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_revision_bytes_set", _wrap_Dwg_Entity_LOFTEDSURFACE_revision_bytes_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_revision_bytes_get", _wrap_Dwg_Entity_LOFTEDSURFACE_revision_bytes_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_end_marker_set", _wrap_Dwg_Entity_LOFTEDSURFACE_end_marker_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_end_marker_get", _wrap_Dwg_Entity_LOFTEDSURFACE_end_marker_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_history_id_set", _wrap_Dwg_Entity_LOFTEDSURFACE_history_id_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_history_id_get", _wrap_Dwg_Entity_LOFTEDSURFACE_history_id_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_has_revision_guid_set", _wrap_Dwg_Entity_LOFTEDSURFACE_has_revision_guid_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_has_revision_guid_get", _wrap_Dwg_Entity_LOFTEDSURFACE_has_revision_guid_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_acis_empty_bit_set", _wrap_Dwg_Entity_LOFTEDSURFACE_acis_empty_bit_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_acis_empty_bit_get", _wrap_Dwg_Entity_LOFTEDSURFACE_acis_empty_bit_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_modeler_format_version_set", _wrap_Dwg_Entity_LOFTEDSURFACE_modeler_format_version_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_modeler_format_version_get", _wrap_Dwg_Entity_LOFTEDSURFACE_modeler_format_version_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_u_isolines_set", _wrap_Dwg_Entity_LOFTEDSURFACE_u_isolines_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_u_isolines_get", _wrap_Dwg_Entity_LOFTEDSURFACE_u_isolines_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_v_isolines_set", _wrap_Dwg_Entity_LOFTEDSURFACE_v_isolines_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_v_isolines_get", _wrap_Dwg_Entity_LOFTEDSURFACE_v_isolines_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_loft_entity_transmatrix_set", _wrap_Dwg_Entity_LOFTEDSURFACE_loft_entity_transmatrix_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_loft_entity_transmatrix_get", _wrap_Dwg_Entity_LOFTEDSURFACE_loft_entity_transmatrix_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_plane_normal_lofting_type_set", _wrap_Dwg_Entity_LOFTEDSURFACE_plane_normal_lofting_type_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_plane_normal_lofting_type_get", _wrap_Dwg_Entity_LOFTEDSURFACE_plane_normal_lofting_type_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_start_draft_angle_set", _wrap_Dwg_Entity_LOFTEDSURFACE_start_draft_angle_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_start_draft_angle_get", _wrap_Dwg_Entity_LOFTEDSURFACE_start_draft_angle_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_end_draft_angle_set", _wrap_Dwg_Entity_LOFTEDSURFACE_end_draft_angle_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_end_draft_angle_get", _wrap_Dwg_Entity_LOFTEDSURFACE_end_draft_angle_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_start_draft_magnitude_set", _wrap_Dwg_Entity_LOFTEDSURFACE_start_draft_magnitude_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_start_draft_magnitude_get", _wrap_Dwg_Entity_LOFTEDSURFACE_start_draft_magnitude_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_end_draft_magnitude_set", _wrap_Dwg_Entity_LOFTEDSURFACE_end_draft_magnitude_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_end_draft_magnitude_get", _wrap_Dwg_Entity_LOFTEDSURFACE_end_draft_magnitude_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_arc_length_parameterization_set", _wrap_Dwg_Entity_LOFTEDSURFACE_arc_length_parameterization_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_arc_length_parameterization_get", _wrap_Dwg_Entity_LOFTEDSURFACE_arc_length_parameterization_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_no_twist_set", _wrap_Dwg_Entity_LOFTEDSURFACE_no_twist_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_no_twist_get", _wrap_Dwg_Entity_LOFTEDSURFACE_no_twist_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_align_direction_set", _wrap_Dwg_Entity_LOFTEDSURFACE_align_direction_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_align_direction_get", _wrap_Dwg_Entity_LOFTEDSURFACE_align_direction_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_simple_surfaces_set", _wrap_Dwg_Entity_LOFTEDSURFACE_simple_surfaces_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_simple_surfaces_get", _wrap_Dwg_Entity_LOFTEDSURFACE_simple_surfaces_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_closed_surfaces_set", _wrap_Dwg_Entity_LOFTEDSURFACE_closed_surfaces_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_closed_surfaces_get", _wrap_Dwg_Entity_LOFTEDSURFACE_closed_surfaces_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_solid_set", _wrap_Dwg_Entity_LOFTEDSURFACE_solid_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_solid_get", _wrap_Dwg_Entity_LOFTEDSURFACE_solid_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_ruled_surface_set", _wrap_Dwg_Entity_LOFTEDSURFACE_ruled_surface_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_ruled_surface_get", _wrap_Dwg_Entity_LOFTEDSURFACE_ruled_surface_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_virtual_guide_set", _wrap_Dwg_Entity_LOFTEDSURFACE_virtual_guide_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_virtual_guide_get", _wrap_Dwg_Entity_LOFTEDSURFACE_virtual_guide_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_num_cross_sections_set", _wrap_Dwg_Entity_LOFTEDSURFACE_num_cross_sections_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_num_cross_sections_get", _wrap_Dwg_Entity_LOFTEDSURFACE_num_cross_sections_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_num_guide_curves_set", _wrap_Dwg_Entity_LOFTEDSURFACE_num_guide_curves_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_num_guide_curves_get", _wrap_Dwg_Entity_LOFTEDSURFACE_num_guide_curves_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_cross_sections_set", _wrap_Dwg_Entity_LOFTEDSURFACE_cross_sections_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_cross_sections_get", _wrap_Dwg_Entity_LOFTEDSURFACE_cross_sections_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_guide_curves_set", _wrap_Dwg_Entity_LOFTEDSURFACE_guide_curves_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_guide_curves_get", _wrap_Dwg_Entity_LOFTEDSURFACE_guide_curves_get},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_path_curve_set", _wrap_Dwg_Entity_LOFTEDSURFACE_path_curve_set},
{"LibreDWGc::Dwg_Entity_LOFTEDSURFACE_path_curve_get", _wrap_Dwg_Entity_LOFTEDSURFACE_path_curve_get},
{"LibreDWGc::new_Dwg_Entity_LOFTEDSURFACE", _wrap_new_Dwg_Entity_LOFTEDSURFACE},
{"LibreDWGc::delete_Dwg_Entity_LOFTEDSURFACE", _wrap_delete_Dwg_Entity_LOFTEDSURFACE},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_parent_set", _wrap_Dwg_Entity_NURBSURFACE_parent_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_parent_get", _wrap_Dwg_Entity_NURBSURFACE_parent_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_acis_empty_set", _wrap_Dwg_Entity_NURBSURFACE_acis_empty_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_acis_empty_get", _wrap_Dwg_Entity_NURBSURFACE_acis_empty_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_unknown_set", _wrap_Dwg_Entity_NURBSURFACE_unknown_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_unknown_get", _wrap_Dwg_Entity_NURBSURFACE_unknown_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_version_set", _wrap_Dwg_Entity_NURBSURFACE_version_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_version_get", _wrap_Dwg_Entity_NURBSURFACE_version_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_num_blocks_set", _wrap_Dwg_Entity_NURBSURFACE_num_blocks_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_num_blocks_get", _wrap_Dwg_Entity_NURBSURFACE_num_blocks_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_block_size_set", _wrap_Dwg_Entity_NURBSURFACE_block_size_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_block_size_get", _wrap_Dwg_Entity_NURBSURFACE_block_size_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_encr_sat_data_set", _wrap_Dwg_Entity_NURBSURFACE_encr_sat_data_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_encr_sat_data_get", _wrap_Dwg_Entity_NURBSURFACE_encr_sat_data_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_sab_size_set", _wrap_Dwg_Entity_NURBSURFACE_sab_size_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_sab_size_get", _wrap_Dwg_Entity_NURBSURFACE_sab_size_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_acis_data_set", _wrap_Dwg_Entity_NURBSURFACE_acis_data_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_acis_data_get", _wrap_Dwg_Entity_NURBSURFACE_acis_data_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_wireframe_data_present_set", _wrap_Dwg_Entity_NURBSURFACE_wireframe_data_present_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_wireframe_data_present_get", _wrap_Dwg_Entity_NURBSURFACE_wireframe_data_present_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_point_present_set", _wrap_Dwg_Entity_NURBSURFACE_point_present_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_point_present_get", _wrap_Dwg_Entity_NURBSURFACE_point_present_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_point_set", _wrap_Dwg_Entity_NURBSURFACE_point_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_point_get", _wrap_Dwg_Entity_NURBSURFACE_point_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_isolines_set", _wrap_Dwg_Entity_NURBSURFACE_isolines_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_isolines_get", _wrap_Dwg_Entity_NURBSURFACE_isolines_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_isoline_present_set", _wrap_Dwg_Entity_NURBSURFACE_isoline_present_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_isoline_present_get", _wrap_Dwg_Entity_NURBSURFACE_isoline_present_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_num_wires_set", _wrap_Dwg_Entity_NURBSURFACE_num_wires_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_num_wires_get", _wrap_Dwg_Entity_NURBSURFACE_num_wires_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_wires_set", _wrap_Dwg_Entity_NURBSURFACE_wires_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_wires_get", _wrap_Dwg_Entity_NURBSURFACE_wires_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_num_silhouettes_set", _wrap_Dwg_Entity_NURBSURFACE_num_silhouettes_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_num_silhouettes_get", _wrap_Dwg_Entity_NURBSURFACE_num_silhouettes_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_silhouettes_set", _wrap_Dwg_Entity_NURBSURFACE_silhouettes_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_silhouettes_get", _wrap_Dwg_Entity_NURBSURFACE_silhouettes_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE__dxf_sab_converted_set", _wrap_Dwg_Entity_NURBSURFACE__dxf_sab_converted_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE__dxf_sab_converted_get", _wrap_Dwg_Entity_NURBSURFACE__dxf_sab_converted_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_acis_empty2_set", _wrap_Dwg_Entity_NURBSURFACE_acis_empty2_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_acis_empty2_get", _wrap_Dwg_Entity_NURBSURFACE_acis_empty2_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_extra_acis_data_set", _wrap_Dwg_Entity_NURBSURFACE_extra_acis_data_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_extra_acis_data_get", _wrap_Dwg_Entity_NURBSURFACE_extra_acis_data_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_num_materials_set", _wrap_Dwg_Entity_NURBSURFACE_num_materials_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_num_materials_get", _wrap_Dwg_Entity_NURBSURFACE_num_materials_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_materials_set", _wrap_Dwg_Entity_NURBSURFACE_materials_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_materials_get", _wrap_Dwg_Entity_NURBSURFACE_materials_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_revision_guid_set", _wrap_Dwg_Entity_NURBSURFACE_revision_guid_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_revision_guid_get", _wrap_Dwg_Entity_NURBSURFACE_revision_guid_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_revision_major_set", _wrap_Dwg_Entity_NURBSURFACE_revision_major_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_revision_major_get", _wrap_Dwg_Entity_NURBSURFACE_revision_major_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_revision_minor1_set", _wrap_Dwg_Entity_NURBSURFACE_revision_minor1_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_revision_minor1_get", _wrap_Dwg_Entity_NURBSURFACE_revision_minor1_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_revision_minor2_set", _wrap_Dwg_Entity_NURBSURFACE_revision_minor2_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_revision_minor2_get", _wrap_Dwg_Entity_NURBSURFACE_revision_minor2_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_revision_bytes_set", _wrap_Dwg_Entity_NURBSURFACE_revision_bytes_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_revision_bytes_get", _wrap_Dwg_Entity_NURBSURFACE_revision_bytes_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_end_marker_set", _wrap_Dwg_Entity_NURBSURFACE_end_marker_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_end_marker_get", _wrap_Dwg_Entity_NURBSURFACE_end_marker_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_history_id_set", _wrap_Dwg_Entity_NURBSURFACE_history_id_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_history_id_get", _wrap_Dwg_Entity_NURBSURFACE_history_id_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_has_revision_guid_set", _wrap_Dwg_Entity_NURBSURFACE_has_revision_guid_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_has_revision_guid_get", _wrap_Dwg_Entity_NURBSURFACE_has_revision_guid_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_acis_empty_bit_set", _wrap_Dwg_Entity_NURBSURFACE_acis_empty_bit_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_acis_empty_bit_get", _wrap_Dwg_Entity_NURBSURFACE_acis_empty_bit_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_u_isolines_set", _wrap_Dwg_Entity_NURBSURFACE_u_isolines_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_u_isolines_get", _wrap_Dwg_Entity_NURBSURFACE_u_isolines_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_v_isolines_set", _wrap_Dwg_Entity_NURBSURFACE_v_isolines_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_v_isolines_get", _wrap_Dwg_Entity_NURBSURFACE_v_isolines_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_short170_set", _wrap_Dwg_Entity_NURBSURFACE_short170_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_short170_get", _wrap_Dwg_Entity_NURBSURFACE_short170_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_cv_hull_display_set", _wrap_Dwg_Entity_NURBSURFACE_cv_hull_display_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_cv_hull_display_get", _wrap_Dwg_Entity_NURBSURFACE_cv_hull_display_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_uvec1_set", _wrap_Dwg_Entity_NURBSURFACE_uvec1_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_uvec1_get", _wrap_Dwg_Entity_NURBSURFACE_uvec1_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_vvec1_set", _wrap_Dwg_Entity_NURBSURFACE_vvec1_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_vvec1_get", _wrap_Dwg_Entity_NURBSURFACE_vvec1_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_uvec2_set", _wrap_Dwg_Entity_NURBSURFACE_uvec2_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_uvec2_get", _wrap_Dwg_Entity_NURBSURFACE_uvec2_get},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_vvec2_set", _wrap_Dwg_Entity_NURBSURFACE_vvec2_set},
{"LibreDWGc::Dwg_Entity_NURBSURFACE_vvec2_get", _wrap_Dwg_Entity_NURBSURFACE_vvec2_get},
{"LibreDWGc::new_Dwg_Entity_NURBSURFACE", _wrap_new_Dwg_Entity_NURBSURFACE},
{"LibreDWGc::delete_Dwg_Entity_NURBSURFACE", _wrap_delete_Dwg_Entity_NURBSURFACE},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_parent_set", _wrap_Dwg_Entity_PLANESURFACE_parent_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_parent_get", _wrap_Dwg_Entity_PLANESURFACE_parent_get},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_acis_empty_set", _wrap_Dwg_Entity_PLANESURFACE_acis_empty_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_acis_empty_get", _wrap_Dwg_Entity_PLANESURFACE_acis_empty_get},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_unknown_set", _wrap_Dwg_Entity_PLANESURFACE_unknown_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_unknown_get", _wrap_Dwg_Entity_PLANESURFACE_unknown_get},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_version_set", _wrap_Dwg_Entity_PLANESURFACE_version_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_version_get", _wrap_Dwg_Entity_PLANESURFACE_version_get},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_num_blocks_set", _wrap_Dwg_Entity_PLANESURFACE_num_blocks_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_num_blocks_get", _wrap_Dwg_Entity_PLANESURFACE_num_blocks_get},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_block_size_set", _wrap_Dwg_Entity_PLANESURFACE_block_size_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_block_size_get", _wrap_Dwg_Entity_PLANESURFACE_block_size_get},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_encr_sat_data_set", _wrap_Dwg_Entity_PLANESURFACE_encr_sat_data_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_encr_sat_data_get", _wrap_Dwg_Entity_PLANESURFACE_encr_sat_data_get},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_sab_size_set", _wrap_Dwg_Entity_PLANESURFACE_sab_size_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_sab_size_get", _wrap_Dwg_Entity_PLANESURFACE_sab_size_get},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_acis_data_set", _wrap_Dwg_Entity_PLANESURFACE_acis_data_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_acis_data_get", _wrap_Dwg_Entity_PLANESURFACE_acis_data_get},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_wireframe_data_present_set", _wrap_Dwg_Entity_PLANESURFACE_wireframe_data_present_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_wireframe_data_present_get", _wrap_Dwg_Entity_PLANESURFACE_wireframe_data_present_get},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_point_present_set", _wrap_Dwg_Entity_PLANESURFACE_point_present_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_point_present_get", _wrap_Dwg_Entity_PLANESURFACE_point_present_get},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_point_set", _wrap_Dwg_Entity_PLANESURFACE_point_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_point_get", _wrap_Dwg_Entity_PLANESURFACE_point_get},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_isolines_set", _wrap_Dwg_Entity_PLANESURFACE_isolines_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_isolines_get", _wrap_Dwg_Entity_PLANESURFACE_isolines_get},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_isoline_present_set", _wrap_Dwg_Entity_PLANESURFACE_isoline_present_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_isoline_present_get", _wrap_Dwg_Entity_PLANESURFACE_isoline_present_get},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_num_wires_set", _wrap_Dwg_Entity_PLANESURFACE_num_wires_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_num_wires_get", _wrap_Dwg_Entity_PLANESURFACE_num_wires_get},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_wires_set", _wrap_Dwg_Entity_PLANESURFACE_wires_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_wires_get", _wrap_Dwg_Entity_PLANESURFACE_wires_get},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_num_silhouettes_set", _wrap_Dwg_Entity_PLANESURFACE_num_silhouettes_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_num_silhouettes_get", _wrap_Dwg_Entity_PLANESURFACE_num_silhouettes_get},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_silhouettes_set", _wrap_Dwg_Entity_PLANESURFACE_silhouettes_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_silhouettes_get", _wrap_Dwg_Entity_PLANESURFACE_silhouettes_get},
{"LibreDWGc::Dwg_Entity_PLANESURFACE__dxf_sab_converted_set", _wrap_Dwg_Entity_PLANESURFACE__dxf_sab_converted_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE__dxf_sab_converted_get", _wrap_Dwg_Entity_PLANESURFACE__dxf_sab_converted_get},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_acis_empty2_set", _wrap_Dwg_Entity_PLANESURFACE_acis_empty2_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_acis_empty2_get", _wrap_Dwg_Entity_PLANESURFACE_acis_empty2_get},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_extra_acis_data_set", _wrap_Dwg_Entity_PLANESURFACE_extra_acis_data_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_extra_acis_data_get", _wrap_Dwg_Entity_PLANESURFACE_extra_acis_data_get},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_num_materials_set", _wrap_Dwg_Entity_PLANESURFACE_num_materials_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_num_materials_get", _wrap_Dwg_Entity_PLANESURFACE_num_materials_get},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_materials_set", _wrap_Dwg_Entity_PLANESURFACE_materials_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_materials_get", _wrap_Dwg_Entity_PLANESURFACE_materials_get},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_revision_guid_set", _wrap_Dwg_Entity_PLANESURFACE_revision_guid_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_revision_guid_get", _wrap_Dwg_Entity_PLANESURFACE_revision_guid_get},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_revision_major_set", _wrap_Dwg_Entity_PLANESURFACE_revision_major_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_revision_major_get", _wrap_Dwg_Entity_PLANESURFACE_revision_major_get},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_revision_minor1_set", _wrap_Dwg_Entity_PLANESURFACE_revision_minor1_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_revision_minor1_get", _wrap_Dwg_Entity_PLANESURFACE_revision_minor1_get},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_revision_minor2_set", _wrap_Dwg_Entity_PLANESURFACE_revision_minor2_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_revision_minor2_get", _wrap_Dwg_Entity_PLANESURFACE_revision_minor2_get},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_revision_bytes_set", _wrap_Dwg_Entity_PLANESURFACE_revision_bytes_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_revision_bytes_get", _wrap_Dwg_Entity_PLANESURFACE_revision_bytes_get},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_end_marker_set", _wrap_Dwg_Entity_PLANESURFACE_end_marker_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_end_marker_get", _wrap_Dwg_Entity_PLANESURFACE_end_marker_get},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_history_id_set", _wrap_Dwg_Entity_PLANESURFACE_history_id_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_history_id_get", _wrap_Dwg_Entity_PLANESURFACE_history_id_get},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_has_revision_guid_set", _wrap_Dwg_Entity_PLANESURFACE_has_revision_guid_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_has_revision_guid_get", _wrap_Dwg_Entity_PLANESURFACE_has_revision_guid_get},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_acis_empty_bit_set", _wrap_Dwg_Entity_PLANESURFACE_acis_empty_bit_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_acis_empty_bit_get", _wrap_Dwg_Entity_PLANESURFACE_acis_empty_bit_get},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_modeler_format_version_set", _wrap_Dwg_Entity_PLANESURFACE_modeler_format_version_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_modeler_format_version_get", _wrap_Dwg_Entity_PLANESURFACE_modeler_format_version_get},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_u_isolines_set", _wrap_Dwg_Entity_PLANESURFACE_u_isolines_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_u_isolines_get", _wrap_Dwg_Entity_PLANESURFACE_u_isolines_get},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_v_isolines_set", _wrap_Dwg_Entity_PLANESURFACE_v_isolines_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_v_isolines_get", _wrap_Dwg_Entity_PLANESURFACE_v_isolines_get},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_class_version_set", _wrap_Dwg_Entity_PLANESURFACE_class_version_set},
{"LibreDWGc::Dwg_Entity_PLANESURFACE_class_version_get", _wrap_Dwg_Entity_PLANESURFACE_class_version_get},
{"LibreDWGc::new_Dwg_Entity_PLANESURFACE", _wrap_new_Dwg_Entity_PLANESURFACE},
{"LibreDWGc::delete_Dwg_Entity_PLANESURFACE", _wrap_delete_Dwg_Entity_PLANESURFACE},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_parent_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_parent_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_parent_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_parent_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_acis_empty_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_acis_empty_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_acis_empty_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_acis_empty_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_unknown_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_unknown_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_unknown_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_unknown_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_version_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_version_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_version_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_version_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_num_blocks_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_num_blocks_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_num_blocks_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_num_blocks_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_block_size_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_block_size_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_block_size_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_block_size_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_encr_sat_data_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_encr_sat_data_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_encr_sat_data_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_encr_sat_data_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_sab_size_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_sab_size_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_sab_size_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_sab_size_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_acis_data_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_acis_data_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_acis_data_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_acis_data_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_wireframe_data_present_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_wireframe_data_present_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_wireframe_data_present_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_wireframe_data_present_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_point_present_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_point_present_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_point_present_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_point_present_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_point_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_point_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_point_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_point_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_isolines_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_isolines_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_isolines_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_isolines_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_isoline_present_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_isoline_present_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_isoline_present_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_isoline_present_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_num_wires_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_num_wires_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_num_wires_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_num_wires_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_wires_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_wires_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_wires_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_wires_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_num_silhouettes_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_num_silhouettes_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_num_silhouettes_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_num_silhouettes_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_silhouettes_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_silhouettes_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_silhouettes_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_silhouettes_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE__dxf_sab_converted_set", _wrap_Dwg_Entity_REVOLVEDSURFACE__dxf_sab_converted_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE__dxf_sab_converted_get", _wrap_Dwg_Entity_REVOLVEDSURFACE__dxf_sab_converted_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_acis_empty2_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_acis_empty2_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_acis_empty2_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_acis_empty2_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_extra_acis_data_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_extra_acis_data_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_extra_acis_data_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_extra_acis_data_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_num_materials_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_num_materials_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_num_materials_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_num_materials_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_materials_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_materials_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_materials_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_materials_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_revision_guid_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_revision_guid_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_revision_guid_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_revision_guid_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_revision_major_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_revision_major_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_revision_major_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_revision_major_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_revision_minor1_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_revision_minor1_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_revision_minor1_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_revision_minor1_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_revision_minor2_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_revision_minor2_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_revision_minor2_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_revision_minor2_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_revision_bytes_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_revision_bytes_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_revision_bytes_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_revision_bytes_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_end_marker_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_end_marker_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_end_marker_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_end_marker_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_history_id_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_history_id_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_history_id_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_history_id_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_has_revision_guid_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_has_revision_guid_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_has_revision_guid_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_has_revision_guid_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_acis_empty_bit_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_acis_empty_bit_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_acis_empty_bit_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_acis_empty_bit_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_modeler_format_version_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_modeler_format_version_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_modeler_format_version_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_modeler_format_version_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_u_isolines_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_u_isolines_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_u_isolines_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_u_isolines_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_v_isolines_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_v_isolines_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_v_isolines_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_v_isolines_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_class_version_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_class_version_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_class_version_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_class_version_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_id_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_id_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_id_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_id_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_axis_point_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_axis_point_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_axis_point_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_axis_point_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_axis_vector_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_axis_vector_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_axis_vector_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_axis_vector_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_revolve_angle_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_revolve_angle_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_revolve_angle_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_revolve_angle_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_start_angle_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_start_angle_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_start_angle_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_start_angle_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_revolved_entity_transmatrix_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_revolved_entity_transmatrix_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_revolved_entity_transmatrix_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_revolved_entity_transmatrix_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_draft_angle_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_draft_angle_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_draft_angle_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_draft_angle_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_draft_start_distance_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_draft_start_distance_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_draft_start_distance_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_draft_start_distance_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_draft_end_distance_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_draft_end_distance_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_draft_end_distance_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_draft_end_distance_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_twist_angle_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_twist_angle_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_twist_angle_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_twist_angle_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_solid_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_solid_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_solid_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_solid_get},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_close_to_axis_set", _wrap_Dwg_Entity_REVOLVEDSURFACE_close_to_axis_set},
{"LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_close_to_axis_get", _wrap_Dwg_Entity_REVOLVEDSURFACE_close_to_axis_get},
{"LibreDWGc::new_Dwg_Entity_REVOLVEDSURFACE", _wrap_new_Dwg_Entity_REVOLVEDSURFACE},
{"LibreDWGc::delete_Dwg_Entity_REVOLVEDSURFACE", _wrap_delete_Dwg_Entity_REVOLVEDSURFACE},
{"LibreDWGc::Dwg_MESH_edge_parent_set", _wrap_Dwg_MESH_edge_parent_set},
{"LibreDWGc::Dwg_MESH_edge_parent_get", _wrap_Dwg_MESH_edge_parent_get},
{"LibreDWGc::Dwg_MESH_edge_idxfrom_set", _wrap_Dwg_MESH_edge_idxfrom_set},
{"LibreDWGc::Dwg_MESH_edge_idxfrom_get", _wrap_Dwg_MESH_edge_idxfrom_get},
{"LibreDWGc::Dwg_MESH_edge_idxto_set", _wrap_Dwg_MESH_edge_idxto_set},
{"LibreDWGc::Dwg_MESH_edge_idxto_get", _wrap_Dwg_MESH_edge_idxto_get},
{"LibreDWGc::new_Dwg_MESH_edge", _wrap_new_Dwg_MESH_edge},
{"LibreDWGc::delete_Dwg_MESH_edge", _wrap_delete_Dwg_MESH_edge},
{"LibreDWGc::Dwg_Entity_MESH_parent_set", _wrap_Dwg_Entity_MESH_parent_set},
{"LibreDWGc::Dwg_Entity_MESH_parent_get", _wrap_Dwg_Entity_MESH_parent_get},
{"LibreDWGc::Dwg_Entity_MESH_dlevel_set", _wrap_Dwg_Entity_MESH_dlevel_set},
{"LibreDWGc::Dwg_Entity_MESH_dlevel_get", _wrap_Dwg_Entity_MESH_dlevel_get},
{"LibreDWGc::Dwg_Entity_MESH_is_watertight_set", _wrap_Dwg_Entity_MESH_is_watertight_set},
{"LibreDWGc::Dwg_Entity_MESH_is_watertight_get", _wrap_Dwg_Entity_MESH_is_watertight_get},
{"LibreDWGc::Dwg_Entity_MESH_num_subdiv_vertex_set", _wrap_Dwg_Entity_MESH_num_subdiv_vertex_set},
{"LibreDWGc::Dwg_Entity_MESH_num_subdiv_vertex_get", _wrap_Dwg_Entity_MESH_num_subdiv_vertex_get},
{"LibreDWGc::Dwg_Entity_MESH_subdiv_vertex_set", _wrap_Dwg_Entity_MESH_subdiv_vertex_set},
{"LibreDWGc::Dwg_Entity_MESH_subdiv_vertex_get", _wrap_Dwg_Entity_MESH_subdiv_vertex_get},
{"LibreDWGc::Dwg_Entity_MESH_num_vertex_set", _wrap_Dwg_Entity_MESH_num_vertex_set},
{"LibreDWGc::Dwg_Entity_MESH_num_vertex_get", _wrap_Dwg_Entity_MESH_num_vertex_get},
{"LibreDWGc::Dwg_Entity_MESH_vertex_set", _wrap_Dwg_Entity_MESH_vertex_set},
{"LibreDWGc::Dwg_Entity_MESH_vertex_get", _wrap_Dwg_Entity_MESH_vertex_get},
{"LibreDWGc::Dwg_Entity_MESH_num_faces_set", _wrap_Dwg_Entity_MESH_num_faces_set},
{"LibreDWGc::Dwg_Entity_MESH_num_faces_get", _wrap_Dwg_Entity_MESH_num_faces_get},
{"LibreDWGc::Dwg_Entity_MESH_faces_set", _wrap_Dwg_Entity_MESH_faces_set},
{"LibreDWGc::Dwg_Entity_MESH_faces_get", _wrap_Dwg_Entity_MESH_faces_get},
{"LibreDWGc::Dwg_Entity_MESH_num_edges_set", _wrap_Dwg_Entity_MESH_num_edges_set},
{"LibreDWGc::Dwg_Entity_MESH_num_edges_get", _wrap_Dwg_Entity_MESH_num_edges_get},
{"LibreDWGc::Dwg_Entity_MESH_edges_set", _wrap_Dwg_Entity_MESH_edges_set},
{"LibreDWGc::Dwg_Entity_MESH_edges_get", _wrap_Dwg_Entity_MESH_edges_get},
{"LibreDWGc::Dwg_Entity_MESH_num_crease_set", _wrap_Dwg_Entity_MESH_num_crease_set},
{"LibreDWGc::Dwg_Entity_MESH_num_crease_get", _wrap_Dwg_Entity_MESH_num_crease_get},
{"LibreDWGc::Dwg_Entity_MESH_crease_set", _wrap_Dwg_Entity_MESH_crease_set},
{"LibreDWGc::Dwg_Entity_MESH_crease_get", _wrap_Dwg_Entity_MESH_crease_get},
{"LibreDWGc::Dwg_Entity_MESH_unknown_b1_set", _wrap_Dwg_Entity_MESH_unknown_b1_set},
{"LibreDWGc::Dwg_Entity_MESH_unknown_b1_get", _wrap_Dwg_Entity_MESH_unknown_b1_get},
{"LibreDWGc::Dwg_Entity_MESH_unknown_b2_set", _wrap_Dwg_Entity_MESH_unknown_b2_set},
{"LibreDWGc::Dwg_Entity_MESH_unknown_b2_get", _wrap_Dwg_Entity_MESH_unknown_b2_get},
{"LibreDWGc::new_Dwg_Entity_MESH", _wrap_new_Dwg_Entity_MESH},
{"LibreDWGc::delete_Dwg_Entity_MESH", _wrap_delete_Dwg_Entity_MESH},
{"LibreDWGc::Dwg_Object_SUN_parent_set", _wrap_Dwg_Object_SUN_parent_set},
{"LibreDWGc::Dwg_Object_SUN_parent_get", _wrap_Dwg_Object_SUN_parent_get},
{"LibreDWGc::Dwg_Object_SUN_class_version_set", _wrap_Dwg_Object_SUN_class_version_set},
{"LibreDWGc::Dwg_Object_SUN_class_version_get", _wrap_Dwg_Object_SUN_class_version_get},
{"LibreDWGc::Dwg_Object_SUN_is_on_set", _wrap_Dwg_Object_SUN_is_on_set},
{"LibreDWGc::Dwg_Object_SUN_is_on_get", _wrap_Dwg_Object_SUN_is_on_get},
{"LibreDWGc::Dwg_Object_SUN_color_set", _wrap_Dwg_Object_SUN_color_set},
{"LibreDWGc::Dwg_Object_SUN_color_get", _wrap_Dwg_Object_SUN_color_get},
{"LibreDWGc::Dwg_Object_SUN_intensity_set", _wrap_Dwg_Object_SUN_intensity_set},
{"LibreDWGc::Dwg_Object_SUN_intensity_get", _wrap_Dwg_Object_SUN_intensity_get},
{"LibreDWGc::Dwg_Object_SUN_has_shadow_set", _wrap_Dwg_Object_SUN_has_shadow_set},
{"LibreDWGc::Dwg_Object_SUN_has_shadow_get", _wrap_Dwg_Object_SUN_has_shadow_get},
{"LibreDWGc::Dwg_Object_SUN_julian_day_set", _wrap_Dwg_Object_SUN_julian_day_set},
{"LibreDWGc::Dwg_Object_SUN_julian_day_get", _wrap_Dwg_Object_SUN_julian_day_get},
{"LibreDWGc::Dwg_Object_SUN_msecs_set", _wrap_Dwg_Object_SUN_msecs_set},
{"LibreDWGc::Dwg_Object_SUN_msecs_get", _wrap_Dwg_Object_SUN_msecs_get},
{"LibreDWGc::Dwg_Object_SUN_is_dst_set", _wrap_Dwg_Object_SUN_is_dst_set},
{"LibreDWGc::Dwg_Object_SUN_is_dst_get", _wrap_Dwg_Object_SUN_is_dst_get},
{"LibreDWGc::Dwg_Object_SUN_shadow_type_set", _wrap_Dwg_Object_SUN_shadow_type_set},
{"LibreDWGc::Dwg_Object_SUN_shadow_type_get", _wrap_Dwg_Object_SUN_shadow_type_get},
{"LibreDWGc::Dwg_Object_SUN_shadow_mapsize_set", _wrap_Dwg_Object_SUN_shadow_mapsize_set},
{"LibreDWGc::Dwg_Object_SUN_shadow_mapsize_get", _wrap_Dwg_Object_SUN_shadow_mapsize_get},
{"LibreDWGc::Dwg_Object_SUN_shadow_softness_set", _wrap_Dwg_Object_SUN_shadow_softness_set},
{"LibreDWGc::Dwg_Object_SUN_shadow_softness_get", _wrap_Dwg_Object_SUN_shadow_softness_get},
{"LibreDWGc::new_Dwg_Object_SUN", _wrap_new_Dwg_Object_SUN},
{"LibreDWGc::delete_Dwg_Object_SUN", _wrap_delete_Dwg_Object_SUN},
{"LibreDWGc::Dwg_SUNSTUDY_Dates_julian_day_set", _wrap_Dwg_SUNSTUDY_Dates_julian_day_set},
{"LibreDWGc::Dwg_SUNSTUDY_Dates_julian_day_get", _wrap_Dwg_SUNSTUDY_Dates_julian_day_get},
{"LibreDWGc::Dwg_SUNSTUDY_Dates_msecs_set", _wrap_Dwg_SUNSTUDY_Dates_msecs_set},
{"LibreDWGc::Dwg_SUNSTUDY_Dates_msecs_get", _wrap_Dwg_SUNSTUDY_Dates_msecs_get},
{"LibreDWGc::new_Dwg_SUNSTUDY_Dates", _wrap_new_Dwg_SUNSTUDY_Dates},
{"LibreDWGc::delete_Dwg_SUNSTUDY_Dates", _wrap_delete_Dwg_SUNSTUDY_Dates},
{"LibreDWGc::Dwg_Object_SUNSTUDY_parent_set", _wrap_Dwg_Object_SUNSTUDY_parent_set},
{"LibreDWGc::Dwg_Object_SUNSTUDY_parent_get", _wrap_Dwg_Object_SUNSTUDY_parent_get},
{"LibreDWGc::Dwg_Object_SUNSTUDY_class_version_set", _wrap_Dwg_Object_SUNSTUDY_class_version_set},
{"LibreDWGc::Dwg_Object_SUNSTUDY_class_version_get", _wrap_Dwg_Object_SUNSTUDY_class_version_get},
{"LibreDWGc::Dwg_Object_SUNSTUDY_setup_name_set", _wrap_Dwg_Object_SUNSTUDY_setup_name_set},
{"LibreDWGc::Dwg_Object_SUNSTUDY_setup_name_get", _wrap_Dwg_Object_SUNSTUDY_setup_name_get},
{"LibreDWGc::Dwg_Object_SUNSTUDY_description_set", _wrap_Dwg_Object_SUNSTUDY_description_set},
{"LibreDWGc::Dwg_Object_SUNSTUDY_description_get", _wrap_Dwg_Object_SUNSTUDY_description_get},
{"LibreDWGc::Dwg_Object_SUNSTUDY_output_type_set", _wrap_Dwg_Object_SUNSTUDY_output_type_set},
{"LibreDWGc::Dwg_Object_SUNSTUDY_output_type_get", _wrap_Dwg_Object_SUNSTUDY_output_type_get},
{"LibreDWGc::Dwg_Object_SUNSTUDY_sheet_set_name_set", _wrap_Dwg_Object_SUNSTUDY_sheet_set_name_set},
{"LibreDWGc::Dwg_Object_SUNSTUDY_sheet_set_name_get", _wrap_Dwg_Object_SUNSTUDY_sheet_set_name_get},
{"LibreDWGc::Dwg_Object_SUNSTUDY_use_subset_set", _wrap_Dwg_Object_SUNSTUDY_use_subset_set},
{"LibreDWGc::Dwg_Object_SUNSTUDY_use_subset_get", _wrap_Dwg_Object_SUNSTUDY_use_subset_get},
{"LibreDWGc::Dwg_Object_SUNSTUDY_sheet_subset_name_set", _wrap_Dwg_Object_SUNSTUDY_sheet_subset_name_set},
{"LibreDWGc::Dwg_Object_SUNSTUDY_sheet_subset_name_get", _wrap_Dwg_Object_SUNSTUDY_sheet_subset_name_get},
{"LibreDWGc::Dwg_Object_SUNSTUDY_select_dates_from_calendar_set", _wrap_Dwg_Object_SUNSTUDY_select_dates_from_calendar_set},
{"LibreDWGc::Dwg_Object_SUNSTUDY_select_dates_from_calendar_get", _wrap_Dwg_Object_SUNSTUDY_select_dates_from_calendar_get},
{"LibreDWGc::Dwg_Object_SUNSTUDY_num_dates_set", _wrap_Dwg_Object_SUNSTUDY_num_dates_set},
{"LibreDWGc::Dwg_Object_SUNSTUDY_num_dates_get", _wrap_Dwg_Object_SUNSTUDY_num_dates_get},
{"LibreDWGc::Dwg_Object_SUNSTUDY_dates_set", _wrap_Dwg_Object_SUNSTUDY_dates_set},
{"LibreDWGc::Dwg_Object_SUNSTUDY_dates_get", _wrap_Dwg_Object_SUNSTUDY_dates_get},
{"LibreDWGc::Dwg_Object_SUNSTUDY_select_range_of_dates_set", _wrap_Dwg_Object_SUNSTUDY_select_range_of_dates_set},
{"LibreDWGc::Dwg_Object_SUNSTUDY_select_range_of_dates_get", _wrap_Dwg_Object_SUNSTUDY_select_range_of_dates_get},
{"LibreDWGc::Dwg_Object_SUNSTUDY_start_time_set", _wrap_Dwg_Object_SUNSTUDY_start_time_set},
{"LibreDWGc::Dwg_Object_SUNSTUDY_start_time_get", _wrap_Dwg_Object_SUNSTUDY_start_time_get},
{"LibreDWGc::Dwg_Object_SUNSTUDY_end_time_set", _wrap_Dwg_Object_SUNSTUDY_end_time_set},
{"LibreDWGc::Dwg_Object_SUNSTUDY_end_time_get", _wrap_Dwg_Object_SUNSTUDY_end_time_get},
{"LibreDWGc::Dwg_Object_SUNSTUDY_interval_set", _wrap_Dwg_Object_SUNSTUDY_interval_set},
{"LibreDWGc::Dwg_Object_SUNSTUDY_interval_get", _wrap_Dwg_Object_SUNSTUDY_interval_get},
{"LibreDWGc::Dwg_Object_SUNSTUDY_num_hours_set", _wrap_Dwg_Object_SUNSTUDY_num_hours_set},
{"LibreDWGc::Dwg_Object_SUNSTUDY_num_hours_get", _wrap_Dwg_Object_SUNSTUDY_num_hours_get},
{"LibreDWGc::Dwg_Object_SUNSTUDY_hours_set", _wrap_Dwg_Object_SUNSTUDY_hours_set},
{"LibreDWGc::Dwg_Object_SUNSTUDY_hours_get", _wrap_Dwg_Object_SUNSTUDY_hours_get},
{"LibreDWGc::Dwg_Object_SUNSTUDY_shade_plot_type_set", _wrap_Dwg_Object_SUNSTUDY_shade_plot_type_set},
{"LibreDWGc::Dwg_Object_SUNSTUDY_shade_plot_type_get", _wrap_Dwg_Object_SUNSTUDY_shade_plot_type_get},
{"LibreDWGc::Dwg_Object_SUNSTUDY_numvports_set", _wrap_Dwg_Object_SUNSTUDY_numvports_set},
{"LibreDWGc::Dwg_Object_SUNSTUDY_numvports_get", _wrap_Dwg_Object_SUNSTUDY_numvports_get},
{"LibreDWGc::Dwg_Object_SUNSTUDY_numrows_set", _wrap_Dwg_Object_SUNSTUDY_numrows_set},
{"LibreDWGc::Dwg_Object_SUNSTUDY_numrows_get", _wrap_Dwg_Object_SUNSTUDY_numrows_get},
{"LibreDWGc::Dwg_Object_SUNSTUDY_numcols_set", _wrap_Dwg_Object_SUNSTUDY_numcols_set},
{"LibreDWGc::Dwg_Object_SUNSTUDY_numcols_get", _wrap_Dwg_Object_SUNSTUDY_numcols_get},
{"LibreDWGc::Dwg_Object_SUNSTUDY_spacing_set", _wrap_Dwg_Object_SUNSTUDY_spacing_set},
{"LibreDWGc::Dwg_Object_SUNSTUDY_spacing_get", _wrap_Dwg_Object_SUNSTUDY_spacing_get},
{"LibreDWGc::Dwg_Object_SUNSTUDY_lock_viewports_set", _wrap_Dwg_Object_SUNSTUDY_lock_viewports_set},
{"LibreDWGc::Dwg_Object_SUNSTUDY_lock_viewports_get", _wrap_Dwg_Object_SUNSTUDY_lock_viewports_get},
{"LibreDWGc::Dwg_Object_SUNSTUDY_label_viewports_set", _wrap_Dwg_Object_SUNSTUDY_label_viewports_set},
{"LibreDWGc::Dwg_Object_SUNSTUDY_label_viewports_get", _wrap_Dwg_Object_SUNSTUDY_label_viewports_get},
{"LibreDWGc::Dwg_Object_SUNSTUDY_page_setup_wizard_set", _wrap_Dwg_Object_SUNSTUDY_page_setup_wizard_set},
{"LibreDWGc::Dwg_Object_SUNSTUDY_page_setup_wizard_get", _wrap_Dwg_Object_SUNSTUDY_page_setup_wizard_get},
{"LibreDWGc::Dwg_Object_SUNSTUDY_view_set", _wrap_Dwg_Object_SUNSTUDY_view_set},
{"LibreDWGc::Dwg_Object_SUNSTUDY_view_get", _wrap_Dwg_Object_SUNSTUDY_view_get},
{"LibreDWGc::Dwg_Object_SUNSTUDY_visualstyle_set", _wrap_Dwg_Object_SUNSTUDY_visualstyle_set},
{"LibreDWGc::Dwg_Object_SUNSTUDY_visualstyle_get", _wrap_Dwg_Object_SUNSTUDY_visualstyle_get},
{"LibreDWGc::Dwg_Object_SUNSTUDY_text_style_set", _wrap_Dwg_Object_SUNSTUDY_text_style_set},
{"LibreDWGc::Dwg_Object_SUNSTUDY_text_style_get", _wrap_Dwg_Object_SUNSTUDY_text_style_get},
{"LibreDWGc::new_Dwg_Object_SUNSTUDY", _wrap_new_Dwg_Object_SUNSTUDY},
{"LibreDWGc::delete_Dwg_Object_SUNSTUDY", _wrap_delete_Dwg_Object_SUNSTUDY},
{"LibreDWGc::Dwg_DATATABLE_row_parent_set", _wrap_Dwg_DATATABLE_row_parent_set},
{"LibreDWGc::Dwg_DATATABLE_row_parent_get", _wrap_Dwg_DATATABLE_row_parent_get},
{"LibreDWGc::Dwg_DATATABLE_row_value_set", _wrap_Dwg_DATATABLE_row_value_set},
{"LibreDWGc::Dwg_DATATABLE_row_value_get", _wrap_Dwg_DATATABLE_row_value_get},
{"LibreDWGc::new_Dwg_DATATABLE_row", _wrap_new_Dwg_DATATABLE_row},
{"LibreDWGc::delete_Dwg_DATATABLE_row", _wrap_delete_Dwg_DATATABLE_row},
{"LibreDWGc::Dwg_DATATABLE_column_parent_set", _wrap_Dwg_DATATABLE_column_parent_set},
{"LibreDWGc::Dwg_DATATABLE_column_parent_get", _wrap_Dwg_DATATABLE_column_parent_get},
{"LibreDWGc::Dwg_DATATABLE_column_type_set", _wrap_Dwg_DATATABLE_column_type_set},
{"LibreDWGc::Dwg_DATATABLE_column_type_get", _wrap_Dwg_DATATABLE_column_type_get},
{"LibreDWGc::Dwg_DATATABLE_column_text_set", _wrap_Dwg_DATATABLE_column_text_set},
{"LibreDWGc::Dwg_DATATABLE_column_text_get", _wrap_Dwg_DATATABLE_column_text_get},
{"LibreDWGc::Dwg_DATATABLE_column_rows_set", _wrap_Dwg_DATATABLE_column_rows_set},
{"LibreDWGc::Dwg_DATATABLE_column_rows_get", _wrap_Dwg_DATATABLE_column_rows_get},
{"LibreDWGc::new_Dwg_DATATABLE_column", _wrap_new_Dwg_DATATABLE_column},
{"LibreDWGc::delete_Dwg_DATATABLE_column", _wrap_delete_Dwg_DATATABLE_column},
{"LibreDWGc::Dwg_Object_DATATABLE_parent_set", _wrap_Dwg_Object_DATATABLE_parent_set},
{"LibreDWGc::Dwg_Object_DATATABLE_parent_get", _wrap_Dwg_Object_DATATABLE_parent_get},
{"LibreDWGc::Dwg_Object_DATATABLE_flags_set", _wrap_Dwg_Object_DATATABLE_flags_set},
{"LibreDWGc::Dwg_Object_DATATABLE_flags_get", _wrap_Dwg_Object_DATATABLE_flags_get},
{"LibreDWGc::Dwg_Object_DATATABLE_num_cols_set", _wrap_Dwg_Object_DATATABLE_num_cols_set},
{"LibreDWGc::Dwg_Object_DATATABLE_num_cols_get", _wrap_Dwg_Object_DATATABLE_num_cols_get},
{"LibreDWGc::Dwg_Object_DATATABLE_num_rows_set", _wrap_Dwg_Object_DATATABLE_num_rows_set},
{"LibreDWGc::Dwg_Object_DATATABLE_num_rows_get", _wrap_Dwg_Object_DATATABLE_num_rows_get},
{"LibreDWGc::Dwg_Object_DATATABLE_table_name_set", _wrap_Dwg_Object_DATATABLE_table_name_set},
{"LibreDWGc::Dwg_Object_DATATABLE_table_name_get", _wrap_Dwg_Object_DATATABLE_table_name_get},
{"LibreDWGc::Dwg_Object_DATATABLE_cols_set", _wrap_Dwg_Object_DATATABLE_cols_set},
{"LibreDWGc::Dwg_Object_DATATABLE_cols_get", _wrap_Dwg_Object_DATATABLE_cols_get},
{"LibreDWGc::new_Dwg_Object_DATATABLE", _wrap_new_Dwg_Object_DATATABLE},
{"LibreDWGc::delete_Dwg_Object_DATATABLE", _wrap_delete_Dwg_Object_DATATABLE},
{"LibreDWGc::Dwg_DATALINK_customdata_parent_set", _wrap_Dwg_DATALINK_customdata_parent_set},
{"LibreDWGc::Dwg_DATALINK_customdata_parent_get", _wrap_Dwg_DATALINK_customdata_parent_get},
{"LibreDWGc::Dwg_DATALINK_customdata_target_set", _wrap_Dwg_DATALINK_customdata_target_set},
{"LibreDWGc::Dwg_DATALINK_customdata_target_get", _wrap_Dwg_DATALINK_customdata_target_get},
{"LibreDWGc::Dwg_DATALINK_customdata_text_set", _wrap_Dwg_DATALINK_customdata_text_set},
{"LibreDWGc::Dwg_DATALINK_customdata_text_get", _wrap_Dwg_DATALINK_customdata_text_get},
{"LibreDWGc::new_Dwg_DATALINK_customdata", _wrap_new_Dwg_DATALINK_customdata},
{"LibreDWGc::delete_Dwg_DATALINK_customdata", _wrap_delete_Dwg_DATALINK_customdata},
{"LibreDWGc::Dwg_Object_DATALINK_parent_set", _wrap_Dwg_Object_DATALINK_parent_set},
{"LibreDWGc::Dwg_Object_DATALINK_parent_get", _wrap_Dwg_Object_DATALINK_parent_get},
{"LibreDWGc::Dwg_Object_DATALINK_class_version_set", _wrap_Dwg_Object_DATALINK_class_version_set},
{"LibreDWGc::Dwg_Object_DATALINK_class_version_get", _wrap_Dwg_Object_DATALINK_class_version_get},
{"LibreDWGc::Dwg_Object_DATALINK_data_adapter_set", _wrap_Dwg_Object_DATALINK_data_adapter_set},
{"LibreDWGc::Dwg_Object_DATALINK_data_adapter_get", _wrap_Dwg_Object_DATALINK_data_adapter_get},
{"LibreDWGc::Dwg_Object_DATALINK_description_set", _wrap_Dwg_Object_DATALINK_description_set},
{"LibreDWGc::Dwg_Object_DATALINK_description_get", _wrap_Dwg_Object_DATALINK_description_get},
{"LibreDWGc::Dwg_Object_DATALINK_tooltip_set", _wrap_Dwg_Object_DATALINK_tooltip_set},
{"LibreDWGc::Dwg_Object_DATALINK_tooltip_get", _wrap_Dwg_Object_DATALINK_tooltip_get},
{"LibreDWGc::Dwg_Object_DATALINK_connection_string_set", _wrap_Dwg_Object_DATALINK_connection_string_set},
{"LibreDWGc::Dwg_Object_DATALINK_connection_string_get", _wrap_Dwg_Object_DATALINK_connection_string_get},
{"LibreDWGc::Dwg_Object_DATALINK_option_set", _wrap_Dwg_Object_DATALINK_option_set},
{"LibreDWGc::Dwg_Object_DATALINK_option_get", _wrap_Dwg_Object_DATALINK_option_get},
{"LibreDWGc::Dwg_Object_DATALINK_update_option_set", _wrap_Dwg_Object_DATALINK_update_option_set},
{"LibreDWGc::Dwg_Object_DATALINK_update_option_get", _wrap_Dwg_Object_DATALINK_update_option_get},
{"LibreDWGc::Dwg_Object_DATALINK_bl92_set", _wrap_Dwg_Object_DATALINK_bl92_set},
{"LibreDWGc::Dwg_Object_DATALINK_bl92_get", _wrap_Dwg_Object_DATALINK_bl92_get},
{"LibreDWGc::Dwg_Object_DATALINK_year_set", _wrap_Dwg_Object_DATALINK_year_set},
{"LibreDWGc::Dwg_Object_DATALINK_year_get", _wrap_Dwg_Object_DATALINK_year_get},
{"LibreDWGc::Dwg_Object_DATALINK_month_set", _wrap_Dwg_Object_DATALINK_month_set},
{"LibreDWGc::Dwg_Object_DATALINK_month_get", _wrap_Dwg_Object_DATALINK_month_get},
{"LibreDWGc::Dwg_Object_DATALINK_day_set", _wrap_Dwg_Object_DATALINK_day_set},
{"LibreDWGc::Dwg_Object_DATALINK_day_get", _wrap_Dwg_Object_DATALINK_day_get},
{"LibreDWGc::Dwg_Object_DATALINK_hour_set", _wrap_Dwg_Object_DATALINK_hour_set},
{"LibreDWGc::Dwg_Object_DATALINK_hour_get", _wrap_Dwg_Object_DATALINK_hour_get},
{"LibreDWGc::Dwg_Object_DATALINK_minute_set", _wrap_Dwg_Object_DATALINK_minute_set},
{"LibreDWGc::Dwg_Object_DATALINK_minute_get", _wrap_Dwg_Object_DATALINK_minute_get},
{"LibreDWGc::Dwg_Object_DATALINK_seconds_set", _wrap_Dwg_Object_DATALINK_seconds_set},
{"LibreDWGc::Dwg_Object_DATALINK_seconds_get", _wrap_Dwg_Object_DATALINK_seconds_get},
{"LibreDWGc::Dwg_Object_DATALINK_msec_set", _wrap_Dwg_Object_DATALINK_msec_set},
{"LibreDWGc::Dwg_Object_DATALINK_msec_get", _wrap_Dwg_Object_DATALINK_msec_get},
{"LibreDWGc::Dwg_Object_DATALINK_path_option_set", _wrap_Dwg_Object_DATALINK_path_option_set},
{"LibreDWGc::Dwg_Object_DATALINK_path_option_get", _wrap_Dwg_Object_DATALINK_path_option_get},
{"LibreDWGc::Dwg_Object_DATALINK_bl93_set", _wrap_Dwg_Object_DATALINK_bl93_set},
{"LibreDWGc::Dwg_Object_DATALINK_bl93_get", _wrap_Dwg_Object_DATALINK_bl93_get},
{"LibreDWGc::Dwg_Object_DATALINK_update_status_set", _wrap_Dwg_Object_DATALINK_update_status_set},
{"LibreDWGc::Dwg_Object_DATALINK_update_status_get", _wrap_Dwg_Object_DATALINK_update_status_get},
{"LibreDWGc::Dwg_Object_DATALINK_num_customdata_set", _wrap_Dwg_Object_DATALINK_num_customdata_set},
{"LibreDWGc::Dwg_Object_DATALINK_num_customdata_get", _wrap_Dwg_Object_DATALINK_num_customdata_get},
{"LibreDWGc::Dwg_Object_DATALINK_customdata_set", _wrap_Dwg_Object_DATALINK_customdata_set},
{"LibreDWGc::Dwg_Object_DATALINK_customdata_get", _wrap_Dwg_Object_DATALINK_customdata_get},
{"LibreDWGc::Dwg_Object_DATALINK_hardowner_set", _wrap_Dwg_Object_DATALINK_hardowner_set},
{"LibreDWGc::Dwg_Object_DATALINK_hardowner_get", _wrap_Dwg_Object_DATALINK_hardowner_get},
{"LibreDWGc::new_Dwg_Object_DATALINK", _wrap_new_Dwg_Object_DATALINK},
{"LibreDWGc::delete_Dwg_Object_DATALINK", _wrap_delete_Dwg_Object_DATALINK},
{"LibreDWGc::Dwg_DIMASSOC_Ref_parent_set", _wrap_Dwg_DIMASSOC_Ref_parent_set},
{"LibreDWGc::Dwg_DIMASSOC_Ref_parent_get", _wrap_Dwg_DIMASSOC_Ref_parent_get},
{"LibreDWGc::Dwg_DIMASSOC_Ref_classname_set", _wrap_Dwg_DIMASSOC_Ref_classname_set},
{"LibreDWGc::Dwg_DIMASSOC_Ref_classname_get", _wrap_Dwg_DIMASSOC_Ref_classname_get},
{"LibreDWGc::Dwg_DIMASSOC_Ref_osnap_type_set", _wrap_Dwg_DIMASSOC_Ref_osnap_type_set},
{"LibreDWGc::Dwg_DIMASSOC_Ref_osnap_type_get", _wrap_Dwg_DIMASSOC_Ref_osnap_type_get},
{"LibreDWGc::Dwg_DIMASSOC_Ref_osnap_dist_set", _wrap_Dwg_DIMASSOC_Ref_osnap_dist_set},
{"LibreDWGc::Dwg_DIMASSOC_Ref_osnap_dist_get", _wrap_Dwg_DIMASSOC_Ref_osnap_dist_get},
{"LibreDWGc::Dwg_DIMASSOC_Ref_osnap_pt_set", _wrap_Dwg_DIMASSOC_Ref_osnap_pt_set},
{"LibreDWGc::Dwg_DIMASSOC_Ref_osnap_pt_get", _wrap_Dwg_DIMASSOC_Ref_osnap_pt_get},
{"LibreDWGc::Dwg_DIMASSOC_Ref_num_xrefs_set", _wrap_Dwg_DIMASSOC_Ref_num_xrefs_set},
{"LibreDWGc::Dwg_DIMASSOC_Ref_num_xrefs_get", _wrap_Dwg_DIMASSOC_Ref_num_xrefs_get},
{"LibreDWGc::Dwg_DIMASSOC_Ref_xrefs_set", _wrap_Dwg_DIMASSOC_Ref_xrefs_set},
{"LibreDWGc::Dwg_DIMASSOC_Ref_xrefs_get", _wrap_Dwg_DIMASSOC_Ref_xrefs_get},
{"LibreDWGc::Dwg_DIMASSOC_Ref_main_subent_type_set", _wrap_Dwg_DIMASSOC_Ref_main_subent_type_set},
{"LibreDWGc::Dwg_DIMASSOC_Ref_main_subent_type_get", _wrap_Dwg_DIMASSOC_Ref_main_subent_type_get},
{"LibreDWGc::Dwg_DIMASSOC_Ref_main_gsmarker_set", _wrap_Dwg_DIMASSOC_Ref_main_gsmarker_set},
{"LibreDWGc::Dwg_DIMASSOC_Ref_main_gsmarker_get", _wrap_Dwg_DIMASSOC_Ref_main_gsmarker_get},
{"LibreDWGc::Dwg_DIMASSOC_Ref_num_xrefpaths_set", _wrap_Dwg_DIMASSOC_Ref_num_xrefpaths_set},
{"LibreDWGc::Dwg_DIMASSOC_Ref_num_xrefpaths_get", _wrap_Dwg_DIMASSOC_Ref_num_xrefpaths_get},
{"LibreDWGc::Dwg_DIMASSOC_Ref_xrefpaths_set", _wrap_Dwg_DIMASSOC_Ref_xrefpaths_set},
{"LibreDWGc::Dwg_DIMASSOC_Ref_xrefpaths_get", _wrap_Dwg_DIMASSOC_Ref_xrefpaths_get},
{"LibreDWGc::Dwg_DIMASSOC_Ref_has_lastpt_ref_set", _wrap_Dwg_DIMASSOC_Ref_has_lastpt_ref_set},
{"LibreDWGc::Dwg_DIMASSOC_Ref_has_lastpt_ref_get", _wrap_Dwg_DIMASSOC_Ref_has_lastpt_ref_get},
{"LibreDWGc::Dwg_DIMASSOC_Ref_lastpt_ref_set", _wrap_Dwg_DIMASSOC_Ref_lastpt_ref_set},
{"LibreDWGc::Dwg_DIMASSOC_Ref_lastpt_ref_get", _wrap_Dwg_DIMASSOC_Ref_lastpt_ref_get},
{"LibreDWGc::Dwg_DIMASSOC_Ref_num_intsectobj_set", _wrap_Dwg_DIMASSOC_Ref_num_intsectobj_set},
{"LibreDWGc::Dwg_DIMASSOC_Ref_num_intsectobj_get", _wrap_Dwg_DIMASSOC_Ref_num_intsectobj_get},
{"LibreDWGc::Dwg_DIMASSOC_Ref_intsectobj_set", _wrap_Dwg_DIMASSOC_Ref_intsectobj_set},
{"LibreDWGc::Dwg_DIMASSOC_Ref_intsectobj_get", _wrap_Dwg_DIMASSOC_Ref_intsectobj_get},
{"LibreDWGc::new_Dwg_DIMASSOC_Ref", _wrap_new_Dwg_DIMASSOC_Ref},
{"LibreDWGc::delete_Dwg_DIMASSOC_Ref", _wrap_delete_Dwg_DIMASSOC_Ref},
{"LibreDWGc::Dwg_Object_DIMASSOC_parent_set", _wrap_Dwg_Object_DIMASSOC_parent_set},
{"LibreDWGc::Dwg_Object_DIMASSOC_parent_get", _wrap_Dwg_Object_DIMASSOC_parent_get},
{"LibreDWGc::Dwg_Object_DIMASSOC_dimensionobj_set", _wrap_Dwg_Object_DIMASSOC_dimensionobj_set},
{"LibreDWGc::Dwg_Object_DIMASSOC_dimensionobj_get", _wrap_Dwg_Object_DIMASSOC_dimensionobj_get},
{"LibreDWGc::Dwg_Object_DIMASSOC_associativity_set", _wrap_Dwg_Object_DIMASSOC_associativity_set},
{"LibreDWGc::Dwg_Object_DIMASSOC_associativity_get", _wrap_Dwg_Object_DIMASSOC_associativity_get},
{"LibreDWGc::Dwg_Object_DIMASSOC_trans_space_flag_set", _wrap_Dwg_Object_DIMASSOC_trans_space_flag_set},
{"LibreDWGc::Dwg_Object_DIMASSOC_trans_space_flag_get", _wrap_Dwg_Object_DIMASSOC_trans_space_flag_get},
{"LibreDWGc::Dwg_Object_DIMASSOC_rotated_type_set", _wrap_Dwg_Object_DIMASSOC_rotated_type_set},
{"LibreDWGc::Dwg_Object_DIMASSOC_rotated_type_get", _wrap_Dwg_Object_DIMASSOC_rotated_type_get},
{"LibreDWGc::Dwg_Object_DIMASSOC_ref_set", _wrap_Dwg_Object_DIMASSOC_ref_set},
{"LibreDWGc::Dwg_Object_DIMASSOC_ref_get", _wrap_Dwg_Object_DIMASSOC_ref_get},
{"LibreDWGc::new_Dwg_Object_DIMASSOC", _wrap_new_Dwg_Object_DIMASSOC},
{"LibreDWGc::delete_Dwg_Object_DIMASSOC", _wrap_delete_Dwg_Object_DIMASSOC},
{"LibreDWGc::Dwg_ACTIONBODY_parent_set", _wrap_Dwg_ACTIONBODY_parent_set},
{"LibreDWGc::Dwg_ACTIONBODY_parent_get", _wrap_Dwg_ACTIONBODY_parent_get},
{"LibreDWGc::Dwg_ACTIONBODY_evaluatorid_set", _wrap_Dwg_ACTIONBODY_evaluatorid_set},
{"LibreDWGc::Dwg_ACTIONBODY_evaluatorid_get", _wrap_Dwg_ACTIONBODY_evaluatorid_get},
{"LibreDWGc::Dwg_ACTIONBODY_expression_set", _wrap_Dwg_ACTIONBODY_expression_set},
{"LibreDWGc::Dwg_ACTIONBODY_expression_get", _wrap_Dwg_ACTIONBODY_expression_get},
{"LibreDWGc::Dwg_ACTIONBODY_value_set", _wrap_Dwg_ACTIONBODY_value_set},
{"LibreDWGc::Dwg_ACTIONBODY_value_get", _wrap_Dwg_ACTIONBODY_value_get},
{"LibreDWGc::new_Dwg_ACTIONBODY", _wrap_new_Dwg_ACTIONBODY},
{"LibreDWGc::delete_Dwg_ACTIONBODY", _wrap_delete_Dwg_ACTIONBODY},
{"LibreDWGc::Dwg_EvalVariant_code_set", _wrap_Dwg_EvalVariant_code_set},
{"LibreDWGc::Dwg_EvalVariant_code_get", _wrap_Dwg_EvalVariant_code_get},
{"LibreDWGc::Dwg_EvalVariant_u_get", _wrap_Dwg_EvalVariant_u_get},
{"LibreDWGc::new_Dwg_EvalVariant", _wrap_new_Dwg_EvalVariant},
{"LibreDWGc::delete_Dwg_EvalVariant", _wrap_delete_Dwg_EvalVariant},
{"LibreDWGc::_dwg_EvalVariant_u_bd_set", _wrap__dwg_EvalVariant_u_bd_set},
{"LibreDWGc::_dwg_EvalVariant_u_bd_get", _wrap__dwg_EvalVariant_u_bd_get},
{"LibreDWGc::_dwg_EvalVariant_u_bl_set", _wrap__dwg_EvalVariant_u_bl_set},
{"LibreDWGc::_dwg_EvalVariant_u_bl_get", _wrap__dwg_EvalVariant_u_bl_get},
{"LibreDWGc::_dwg_EvalVariant_u_bs_set", _wrap__dwg_EvalVariant_u_bs_set},
{"LibreDWGc::_dwg_EvalVariant_u_bs_get", _wrap__dwg_EvalVariant_u_bs_get},
{"LibreDWGc::_dwg_EvalVariant_u_rc_set", _wrap__dwg_EvalVariant_u_rc_set},
{"LibreDWGc::_dwg_EvalVariant_u_rc_get", _wrap__dwg_EvalVariant_u_rc_get},
{"LibreDWGc::_dwg_EvalVariant_u_text_set", _wrap__dwg_EvalVariant_u_text_set},
{"LibreDWGc::_dwg_EvalVariant_u_text_get", _wrap__dwg_EvalVariant_u_text_get},
{"LibreDWGc::_dwg_EvalVariant_u_handle_set", _wrap__dwg_EvalVariant_u_handle_set},
{"LibreDWGc::_dwg_EvalVariant_u_handle_get", _wrap__dwg_EvalVariant_u_handle_get},
{"LibreDWGc::new__dwg_EvalVariant_u", _wrap_new__dwg_EvalVariant_u},
{"LibreDWGc::delete__dwg_EvalVariant_u", _wrap_delete__dwg_EvalVariant_u},
{"LibreDWGc::Dwg_VALUEPARAM_vars_value_set", _wrap_Dwg_VALUEPARAM_vars_value_set},
{"LibreDWGc::Dwg_VALUEPARAM_vars_value_get", _wrap_Dwg_VALUEPARAM_vars_value_get},
{"LibreDWGc::Dwg_VALUEPARAM_vars_handle_set", _wrap_Dwg_VALUEPARAM_vars_handle_set},
{"LibreDWGc::Dwg_VALUEPARAM_vars_handle_get", _wrap_Dwg_VALUEPARAM_vars_handle_get},
{"LibreDWGc::new_Dwg_VALUEPARAM_vars", _wrap_new_Dwg_VALUEPARAM_vars},
{"LibreDWGc::delete_Dwg_VALUEPARAM_vars", _wrap_delete_Dwg_VALUEPARAM_vars},
{"LibreDWGc::Dwg_VALUEPARAM_parent_set", _wrap_Dwg_VALUEPARAM_parent_set},
{"LibreDWGc::Dwg_VALUEPARAM_parent_get", _wrap_Dwg_VALUEPARAM_parent_get},
{"LibreDWGc::Dwg_VALUEPARAM_class_version_set", _wrap_Dwg_VALUEPARAM_class_version_set},
{"LibreDWGc::Dwg_VALUEPARAM_class_version_get", _wrap_Dwg_VALUEPARAM_class_version_get},
{"LibreDWGc::Dwg_VALUEPARAM_name_set", _wrap_Dwg_VALUEPARAM_name_set},
{"LibreDWGc::Dwg_VALUEPARAM_name_get", _wrap_Dwg_VALUEPARAM_name_get},
{"LibreDWGc::Dwg_VALUEPARAM_unit_type_set", _wrap_Dwg_VALUEPARAM_unit_type_set},
{"LibreDWGc::Dwg_VALUEPARAM_unit_type_get", _wrap_Dwg_VALUEPARAM_unit_type_get},
{"LibreDWGc::Dwg_VALUEPARAM_num_vars_set", _wrap_Dwg_VALUEPARAM_num_vars_set},
{"LibreDWGc::Dwg_VALUEPARAM_num_vars_get", _wrap_Dwg_VALUEPARAM_num_vars_get},
{"LibreDWGc::Dwg_VALUEPARAM_vars_set", _wrap_Dwg_VALUEPARAM_vars_set},
{"LibreDWGc::Dwg_VALUEPARAM_vars_get", _wrap_Dwg_VALUEPARAM_vars_get},
{"LibreDWGc::Dwg_VALUEPARAM_controlled_objdep_set", _wrap_Dwg_VALUEPARAM_controlled_objdep_set},
{"LibreDWGc::Dwg_VALUEPARAM_controlled_objdep_get", _wrap_Dwg_VALUEPARAM_controlled_objdep_get},
{"LibreDWGc::new_Dwg_VALUEPARAM", _wrap_new_Dwg_VALUEPARAM},
{"LibreDWGc::delete_Dwg_VALUEPARAM", _wrap_delete_Dwg_VALUEPARAM},
{"LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_parent_set", _wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_parent_set},
{"LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_parent_get", _wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_parent_get},
{"LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_version_set", _wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_version_set},
{"LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_version_get", _wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_version_get},
{"LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_minor_set", _wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_minor_set},
{"LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_minor_get", _wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_minor_get},
{"LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_num_deps_set", _wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_num_deps_set},
{"LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_num_deps_get", _wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_num_deps_get},
{"LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_deps_set", _wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_deps_set},
{"LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_deps_get", _wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_deps_get},
{"LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_l4_set", _wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_l4_set},
{"LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_l4_get", _wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_l4_get},
{"LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_l5_set", _wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_l5_set},
{"LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_l5_get", _wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_l5_get},
{"LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_assocdep_set", _wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_assocdep_set},
{"LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_assocdep_get", _wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_assocdep_get},
{"LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_num_values_set", _wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_num_values_set},
{"LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_num_values_get", _wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_num_values_get},
{"LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_values_set", _wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_values_set},
{"LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_values_get", _wrap_Dwg_ASSOCPARAMBASEDACTIONBODY_values_get},
{"LibreDWGc::new_Dwg_ASSOCPARAMBASEDACTIONBODY", _wrap_new_Dwg_ASSOCPARAMBASEDACTIONBODY},
{"LibreDWGc::delete_Dwg_ASSOCPARAMBASEDACTIONBODY", _wrap_delete_Dwg_ASSOCPARAMBASEDACTIONBODY},
{"LibreDWGc::Dwg_ASSOCACTION_Deps_parent_set", _wrap_Dwg_ASSOCACTION_Deps_parent_set},
{"LibreDWGc::Dwg_ASSOCACTION_Deps_parent_get", _wrap_Dwg_ASSOCACTION_Deps_parent_get},
{"LibreDWGc::Dwg_ASSOCACTION_Deps_is_owned_set", _wrap_Dwg_ASSOCACTION_Deps_is_owned_set},
{"LibreDWGc::Dwg_ASSOCACTION_Deps_is_owned_get", _wrap_Dwg_ASSOCACTION_Deps_is_owned_get},
{"LibreDWGc::Dwg_ASSOCACTION_Deps_dep_set", _wrap_Dwg_ASSOCACTION_Deps_dep_set},
{"LibreDWGc::Dwg_ASSOCACTION_Deps_dep_get", _wrap_Dwg_ASSOCACTION_Deps_dep_get},
{"LibreDWGc::new_Dwg_ASSOCACTION_Deps", _wrap_new_Dwg_ASSOCACTION_Deps},
{"LibreDWGc::delete_Dwg_ASSOCACTION_Deps", _wrap_delete_Dwg_ASSOCACTION_Deps},
{"LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_parent_set", _wrap_Dwg_Object_ASSOCDEPENDENCY_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_parent_get", _wrap_Dwg_Object_ASSOCDEPENDENCY_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_class_version_set", _wrap_Dwg_Object_ASSOCDEPENDENCY_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_class_version_get", _wrap_Dwg_Object_ASSOCDEPENDENCY_class_version_get},
{"LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_status_set", _wrap_Dwg_Object_ASSOCDEPENDENCY_status_set},
{"LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_status_get", _wrap_Dwg_Object_ASSOCDEPENDENCY_status_get},
{"LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_is_read_dep_set", _wrap_Dwg_Object_ASSOCDEPENDENCY_is_read_dep_set},
{"LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_is_read_dep_get", _wrap_Dwg_Object_ASSOCDEPENDENCY_is_read_dep_get},
{"LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_is_write_dep_set", _wrap_Dwg_Object_ASSOCDEPENDENCY_is_write_dep_set},
{"LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_is_write_dep_get", _wrap_Dwg_Object_ASSOCDEPENDENCY_is_write_dep_get},
{"LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_is_attached_to_object_set", _wrap_Dwg_Object_ASSOCDEPENDENCY_is_attached_to_object_set},
{"LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_is_attached_to_object_get", _wrap_Dwg_Object_ASSOCDEPENDENCY_is_attached_to_object_get},
{"LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_is_delegating_to_owning_action_set", _wrap_Dwg_Object_ASSOCDEPENDENCY_is_delegating_to_owning_action_set},
{"LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_is_delegating_to_owning_action_get", _wrap_Dwg_Object_ASSOCDEPENDENCY_is_delegating_to_owning_action_get},
{"LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_order_set", _wrap_Dwg_Object_ASSOCDEPENDENCY_order_set},
{"LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_order_get", _wrap_Dwg_Object_ASSOCDEPENDENCY_order_get},
{"LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_dep_on_set", _wrap_Dwg_Object_ASSOCDEPENDENCY_dep_on_set},
{"LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_dep_on_get", _wrap_Dwg_Object_ASSOCDEPENDENCY_dep_on_get},
{"LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_has_name_set", _wrap_Dwg_Object_ASSOCDEPENDENCY_has_name_set},
{"LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_has_name_get", _wrap_Dwg_Object_ASSOCDEPENDENCY_has_name_get},
{"LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_name_set", _wrap_Dwg_Object_ASSOCDEPENDENCY_name_set},
{"LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_name_get", _wrap_Dwg_Object_ASSOCDEPENDENCY_name_get},
{"LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_depbodyid_set", _wrap_Dwg_Object_ASSOCDEPENDENCY_depbodyid_set},
{"LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_depbodyid_get", _wrap_Dwg_Object_ASSOCDEPENDENCY_depbodyid_get},
{"LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_readdep_set", _wrap_Dwg_Object_ASSOCDEPENDENCY_readdep_set},
{"LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_readdep_get", _wrap_Dwg_Object_ASSOCDEPENDENCY_readdep_get},
{"LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_dep_body_set", _wrap_Dwg_Object_ASSOCDEPENDENCY_dep_body_set},
{"LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_dep_body_get", _wrap_Dwg_Object_ASSOCDEPENDENCY_dep_body_get},
{"LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_node_set", _wrap_Dwg_Object_ASSOCDEPENDENCY_node_set},
{"LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_node_get", _wrap_Dwg_Object_ASSOCDEPENDENCY_node_get},
{"LibreDWGc::new_Dwg_Object_ASSOCDEPENDENCY", _wrap_new_Dwg_Object_ASSOCDEPENDENCY},
{"LibreDWGc::delete_Dwg_Object_ASSOCDEPENDENCY", _wrap_delete_Dwg_Object_ASSOCDEPENDENCY},
{"LibreDWGc::Dwg_Object_ASSOCVALUEDEPENDENCY_parent_set", _wrap_Dwg_Object_ASSOCVALUEDEPENDENCY_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCVALUEDEPENDENCY_parent_get", _wrap_Dwg_Object_ASSOCVALUEDEPENDENCY_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCVALUEDEPENDENCY_assocdep_set", _wrap_Dwg_Object_ASSOCVALUEDEPENDENCY_assocdep_set},
{"LibreDWGc::Dwg_Object_ASSOCVALUEDEPENDENCY_assocdep_get", _wrap_Dwg_Object_ASSOCVALUEDEPENDENCY_assocdep_get},
{"LibreDWGc::new_Dwg_Object_ASSOCVALUEDEPENDENCY", _wrap_new_Dwg_Object_ASSOCVALUEDEPENDENCY},
{"LibreDWGc::delete_Dwg_Object_ASSOCVALUEDEPENDENCY", _wrap_delete_Dwg_Object_ASSOCVALUEDEPENDENCY},
{"LibreDWGc::Dwg_Object_ASSOCGEOMDEPENDENCY_parent_set", _wrap_Dwg_Object_ASSOCGEOMDEPENDENCY_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCGEOMDEPENDENCY_parent_get", _wrap_Dwg_Object_ASSOCGEOMDEPENDENCY_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCGEOMDEPENDENCY_assocdep_set", _wrap_Dwg_Object_ASSOCGEOMDEPENDENCY_assocdep_set},
{"LibreDWGc::Dwg_Object_ASSOCGEOMDEPENDENCY_assocdep_get", _wrap_Dwg_Object_ASSOCGEOMDEPENDENCY_assocdep_get},
{"LibreDWGc::Dwg_Object_ASSOCGEOMDEPENDENCY_class_version_set", _wrap_Dwg_Object_ASSOCGEOMDEPENDENCY_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCGEOMDEPENDENCY_class_version_get", _wrap_Dwg_Object_ASSOCGEOMDEPENDENCY_class_version_get},
{"LibreDWGc::Dwg_Object_ASSOCGEOMDEPENDENCY_enabled_set", _wrap_Dwg_Object_ASSOCGEOMDEPENDENCY_enabled_set},
{"LibreDWGc::Dwg_Object_ASSOCGEOMDEPENDENCY_enabled_get", _wrap_Dwg_Object_ASSOCGEOMDEPENDENCY_enabled_get},
{"LibreDWGc::Dwg_Object_ASSOCGEOMDEPENDENCY_classname_set", _wrap_Dwg_Object_ASSOCGEOMDEPENDENCY_classname_set},
{"LibreDWGc::Dwg_Object_ASSOCGEOMDEPENDENCY_classname_get", _wrap_Dwg_Object_ASSOCGEOMDEPENDENCY_classname_get},
{"LibreDWGc::Dwg_Object_ASSOCGEOMDEPENDENCY_dependent_on_compound_object_set", _wrap_Dwg_Object_ASSOCGEOMDEPENDENCY_dependent_on_compound_object_set},
{"LibreDWGc::Dwg_Object_ASSOCGEOMDEPENDENCY_dependent_on_compound_object_get", _wrap_Dwg_Object_ASSOCGEOMDEPENDENCY_dependent_on_compound_object_get},
{"LibreDWGc::new_Dwg_Object_ASSOCGEOMDEPENDENCY", _wrap_new_Dwg_Object_ASSOCGEOMDEPENDENCY},
{"LibreDWGc::delete_Dwg_Object_ASSOCGEOMDEPENDENCY", _wrap_delete_Dwg_Object_ASSOCGEOMDEPENDENCY},
{"LibreDWGc::Dwg_Object_ASSOCACTION_parent_set", _wrap_Dwg_Object_ASSOCACTION_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCACTION_parent_get", _wrap_Dwg_Object_ASSOCACTION_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCACTION_class_version_set", _wrap_Dwg_Object_ASSOCACTION_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCACTION_class_version_get", _wrap_Dwg_Object_ASSOCACTION_class_version_get},
{"LibreDWGc::Dwg_Object_ASSOCACTION_geometry_status_set", _wrap_Dwg_Object_ASSOCACTION_geometry_status_set},
{"LibreDWGc::Dwg_Object_ASSOCACTION_geometry_status_get", _wrap_Dwg_Object_ASSOCACTION_geometry_status_get},
{"LibreDWGc::Dwg_Object_ASSOCACTION_owningnetwork_set", _wrap_Dwg_Object_ASSOCACTION_owningnetwork_set},
{"LibreDWGc::Dwg_Object_ASSOCACTION_owningnetwork_get", _wrap_Dwg_Object_ASSOCACTION_owningnetwork_get},
{"LibreDWGc::Dwg_Object_ASSOCACTION_actionbody_set", _wrap_Dwg_Object_ASSOCACTION_actionbody_set},
{"LibreDWGc::Dwg_Object_ASSOCACTION_actionbody_get", _wrap_Dwg_Object_ASSOCACTION_actionbody_get},
{"LibreDWGc::Dwg_Object_ASSOCACTION_action_index_set", _wrap_Dwg_Object_ASSOCACTION_action_index_set},
{"LibreDWGc::Dwg_Object_ASSOCACTION_action_index_get", _wrap_Dwg_Object_ASSOCACTION_action_index_get},
{"LibreDWGc::Dwg_Object_ASSOCACTION_max_assoc_dep_index_set", _wrap_Dwg_Object_ASSOCACTION_max_assoc_dep_index_set},
{"LibreDWGc::Dwg_Object_ASSOCACTION_max_assoc_dep_index_get", _wrap_Dwg_Object_ASSOCACTION_max_assoc_dep_index_get},
{"LibreDWGc::Dwg_Object_ASSOCACTION_num_deps_set", _wrap_Dwg_Object_ASSOCACTION_num_deps_set},
{"LibreDWGc::Dwg_Object_ASSOCACTION_num_deps_get", _wrap_Dwg_Object_ASSOCACTION_num_deps_get},
{"LibreDWGc::Dwg_Object_ASSOCACTION_deps_set", _wrap_Dwg_Object_ASSOCACTION_deps_set},
{"LibreDWGc::Dwg_Object_ASSOCACTION_deps_get", _wrap_Dwg_Object_ASSOCACTION_deps_get},
{"LibreDWGc::Dwg_Object_ASSOCACTION_num_owned_params_set", _wrap_Dwg_Object_ASSOCACTION_num_owned_params_set},
{"LibreDWGc::Dwg_Object_ASSOCACTION_num_owned_params_get", _wrap_Dwg_Object_ASSOCACTION_num_owned_params_get},
{"LibreDWGc::Dwg_Object_ASSOCACTION_owned_params_set", _wrap_Dwg_Object_ASSOCACTION_owned_params_set},
{"LibreDWGc::Dwg_Object_ASSOCACTION_owned_params_get", _wrap_Dwg_Object_ASSOCACTION_owned_params_get},
{"LibreDWGc::Dwg_Object_ASSOCACTION_num_values_set", _wrap_Dwg_Object_ASSOCACTION_num_values_set},
{"LibreDWGc::Dwg_Object_ASSOCACTION_num_values_get", _wrap_Dwg_Object_ASSOCACTION_num_values_get},
{"LibreDWGc::Dwg_Object_ASSOCACTION_values_set", _wrap_Dwg_Object_ASSOCACTION_values_set},
{"LibreDWGc::Dwg_Object_ASSOCACTION_values_get", _wrap_Dwg_Object_ASSOCACTION_values_get},
{"LibreDWGc::new_Dwg_Object_ASSOCACTION", _wrap_new_Dwg_Object_ASSOCACTION},
{"LibreDWGc::delete_Dwg_Object_ASSOCACTION", _wrap_delete_Dwg_Object_ASSOCACTION},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_parent_set", _wrap_Dwg_Object_ASSOCNETWORK_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_parent_get", _wrap_Dwg_Object_ASSOCNETWORK_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_class_version_set", _wrap_Dwg_Object_ASSOCNETWORK_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_class_version_get", _wrap_Dwg_Object_ASSOCNETWORK_class_version_get},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_geometry_status_set", _wrap_Dwg_Object_ASSOCNETWORK_geometry_status_set},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_geometry_status_get", _wrap_Dwg_Object_ASSOCNETWORK_geometry_status_get},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_owningnetwork_set", _wrap_Dwg_Object_ASSOCNETWORK_owningnetwork_set},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_owningnetwork_get", _wrap_Dwg_Object_ASSOCNETWORK_owningnetwork_get},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_actionbody_set", _wrap_Dwg_Object_ASSOCNETWORK_actionbody_set},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_actionbody_get", _wrap_Dwg_Object_ASSOCNETWORK_actionbody_get},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_action_index_set", _wrap_Dwg_Object_ASSOCNETWORK_action_index_set},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_action_index_get", _wrap_Dwg_Object_ASSOCNETWORK_action_index_get},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_max_assoc_dep_index_set", _wrap_Dwg_Object_ASSOCNETWORK_max_assoc_dep_index_set},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_max_assoc_dep_index_get", _wrap_Dwg_Object_ASSOCNETWORK_max_assoc_dep_index_get},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_num_deps_set", _wrap_Dwg_Object_ASSOCNETWORK_num_deps_set},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_num_deps_get", _wrap_Dwg_Object_ASSOCNETWORK_num_deps_get},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_deps_set", _wrap_Dwg_Object_ASSOCNETWORK_deps_set},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_deps_get", _wrap_Dwg_Object_ASSOCNETWORK_deps_get},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_num_owned_params_set", _wrap_Dwg_Object_ASSOCNETWORK_num_owned_params_set},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_num_owned_params_get", _wrap_Dwg_Object_ASSOCNETWORK_num_owned_params_get},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_owned_params_set", _wrap_Dwg_Object_ASSOCNETWORK_owned_params_set},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_owned_params_get", _wrap_Dwg_Object_ASSOCNETWORK_owned_params_get},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_num_values_set", _wrap_Dwg_Object_ASSOCNETWORK_num_values_set},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_num_values_get", _wrap_Dwg_Object_ASSOCNETWORK_num_values_get},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_values_set", _wrap_Dwg_Object_ASSOCNETWORK_values_set},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_values_get", _wrap_Dwg_Object_ASSOCNETWORK_values_get},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_network_version_set", _wrap_Dwg_Object_ASSOCNETWORK_network_version_set},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_network_version_get", _wrap_Dwg_Object_ASSOCNETWORK_network_version_get},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_network_action_index_set", _wrap_Dwg_Object_ASSOCNETWORK_network_action_index_set},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_network_action_index_get", _wrap_Dwg_Object_ASSOCNETWORK_network_action_index_get},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_num_actions_set", _wrap_Dwg_Object_ASSOCNETWORK_num_actions_set},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_num_actions_get", _wrap_Dwg_Object_ASSOCNETWORK_num_actions_get},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_actions_set", _wrap_Dwg_Object_ASSOCNETWORK_actions_set},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_actions_get", _wrap_Dwg_Object_ASSOCNETWORK_actions_get},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_num_owned_actions_set", _wrap_Dwg_Object_ASSOCNETWORK_num_owned_actions_set},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_num_owned_actions_get", _wrap_Dwg_Object_ASSOCNETWORK_num_owned_actions_get},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_owned_actions_set", _wrap_Dwg_Object_ASSOCNETWORK_owned_actions_set},
{"LibreDWGc::Dwg_Object_ASSOCNETWORK_owned_actions_get", _wrap_Dwg_Object_ASSOCNETWORK_owned_actions_get},
{"LibreDWGc::new_Dwg_Object_ASSOCNETWORK", _wrap_new_Dwg_Object_ASSOCNETWORK},
{"LibreDWGc::delete_Dwg_Object_ASSOCNETWORK", _wrap_delete_Dwg_Object_ASSOCNETWORK},
{"LibreDWGc::Dwg_CONSTRAINTGROUPNODE_parent_set", _wrap_Dwg_CONSTRAINTGROUPNODE_parent_set},
{"LibreDWGc::Dwg_CONSTRAINTGROUPNODE_parent_get", _wrap_Dwg_CONSTRAINTGROUPNODE_parent_get},
{"LibreDWGc::Dwg_CONSTRAINTGROUPNODE_nodeid_set", _wrap_Dwg_CONSTRAINTGROUPNODE_nodeid_set},
{"LibreDWGc::Dwg_CONSTRAINTGROUPNODE_nodeid_get", _wrap_Dwg_CONSTRAINTGROUPNODE_nodeid_get},
{"LibreDWGc::Dwg_CONSTRAINTGROUPNODE_status_set", _wrap_Dwg_CONSTRAINTGROUPNODE_status_set},
{"LibreDWGc::Dwg_CONSTRAINTGROUPNODE_status_get", _wrap_Dwg_CONSTRAINTGROUPNODE_status_get},
{"LibreDWGc::Dwg_CONSTRAINTGROUPNODE_num_connections_set", _wrap_Dwg_CONSTRAINTGROUPNODE_num_connections_set},
{"LibreDWGc::Dwg_CONSTRAINTGROUPNODE_num_connections_get", _wrap_Dwg_CONSTRAINTGROUPNODE_num_connections_get},
{"LibreDWGc::Dwg_CONSTRAINTGROUPNODE_connections_set", _wrap_Dwg_CONSTRAINTGROUPNODE_connections_set},
{"LibreDWGc::Dwg_CONSTRAINTGROUPNODE_connections_get", _wrap_Dwg_CONSTRAINTGROUPNODE_connections_get},
{"LibreDWGc::new_Dwg_CONSTRAINTGROUPNODE", _wrap_new_Dwg_CONSTRAINTGROUPNODE},
{"LibreDWGc::delete_Dwg_CONSTRAINTGROUPNODE", _wrap_delete_Dwg_CONSTRAINTGROUPNODE},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_parent_set", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_parent_set},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_parent_get", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_parent_get},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_class_version_set", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_class_version_get", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_class_version_get},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_geometry_status_set", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_geometry_status_set},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_geometry_status_get", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_geometry_status_get},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_owningnetwork_set", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_owningnetwork_set},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_owningnetwork_get", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_owningnetwork_get},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_actionbody_set", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_actionbody_set},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_actionbody_get", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_actionbody_get},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_action_index_set", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_action_index_set},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_action_index_get", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_action_index_get},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_max_assoc_dep_index_set", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_max_assoc_dep_index_set},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_max_assoc_dep_index_get", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_max_assoc_dep_index_get},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_deps_set", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_deps_set},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_deps_get", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_deps_get},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_deps_set", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_deps_set},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_deps_get", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_deps_get},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_owned_params_set", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_owned_params_set},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_owned_params_get", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_owned_params_get},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_owned_params_set", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_owned_params_set},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_owned_params_get", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_owned_params_get},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_values_set", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_values_set},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_values_get", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_values_get},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_values_set", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_values_set},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_values_get", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_values_get},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_version_set", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_version_set},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_version_get", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_version_get},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_b1_set", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_b1_set},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_b1_get", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_b1_get},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_workplane_set", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_workplane_set},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_workplane_get", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_workplane_get},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_h1_set", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_h1_set},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_h1_get", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_h1_get},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_actions_set", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_actions_set},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_actions_get", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_actions_get},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_actions_set", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_actions_set},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_actions_get", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_actions_get},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_nodes_set", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_nodes_set},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_nodes_get", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_nodes_get},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_nodes_set", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_nodes_set},
{"LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_nodes_get", _wrap_Dwg_Object_ASSOC2DCONSTRAINTGROUP_nodes_get},
{"LibreDWGc::new_Dwg_Object_ASSOC2DCONSTRAINTGROUP", _wrap_new_Dwg_Object_ASSOC2DCONSTRAINTGROUP},
{"LibreDWGc::delete_Dwg_Object_ASSOC2DCONSTRAINTGROUP", _wrap_delete_Dwg_Object_ASSOC2DCONSTRAINTGROUP},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_parent_set", _wrap_Dwg_Object_ASSOCVARIABLE_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_parent_get", _wrap_Dwg_Object_ASSOCVARIABLE_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_av_class_version_set", _wrap_Dwg_Object_ASSOCVARIABLE_av_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_av_class_version_get", _wrap_Dwg_Object_ASSOCVARIABLE_av_class_version_get},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_class_version_set", _wrap_Dwg_Object_ASSOCVARIABLE_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_class_version_get", _wrap_Dwg_Object_ASSOCVARIABLE_class_version_get},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_geometry_status_set", _wrap_Dwg_Object_ASSOCVARIABLE_geometry_status_set},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_geometry_status_get", _wrap_Dwg_Object_ASSOCVARIABLE_geometry_status_get},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_owningnetwork_set", _wrap_Dwg_Object_ASSOCVARIABLE_owningnetwork_set},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_owningnetwork_get", _wrap_Dwg_Object_ASSOCVARIABLE_owningnetwork_get},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_actionbody_set", _wrap_Dwg_Object_ASSOCVARIABLE_actionbody_set},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_actionbody_get", _wrap_Dwg_Object_ASSOCVARIABLE_actionbody_get},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_action_index_set", _wrap_Dwg_Object_ASSOCVARIABLE_action_index_set},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_action_index_get", _wrap_Dwg_Object_ASSOCVARIABLE_action_index_get},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_max_assoc_dep_index_set", _wrap_Dwg_Object_ASSOCVARIABLE_max_assoc_dep_index_set},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_max_assoc_dep_index_get", _wrap_Dwg_Object_ASSOCVARIABLE_max_assoc_dep_index_get},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_num_deps_set", _wrap_Dwg_Object_ASSOCVARIABLE_num_deps_set},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_num_deps_get", _wrap_Dwg_Object_ASSOCVARIABLE_num_deps_get},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_deps_set", _wrap_Dwg_Object_ASSOCVARIABLE_deps_set},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_deps_get", _wrap_Dwg_Object_ASSOCVARIABLE_deps_get},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_num_owned_params_set", _wrap_Dwg_Object_ASSOCVARIABLE_num_owned_params_set},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_num_owned_params_get", _wrap_Dwg_Object_ASSOCVARIABLE_num_owned_params_get},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_owned_params_set", _wrap_Dwg_Object_ASSOCVARIABLE_owned_params_set},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_owned_params_get", _wrap_Dwg_Object_ASSOCVARIABLE_owned_params_get},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_num_values_set", _wrap_Dwg_Object_ASSOCVARIABLE_num_values_set},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_num_values_get", _wrap_Dwg_Object_ASSOCVARIABLE_num_values_get},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_values_set", _wrap_Dwg_Object_ASSOCVARIABLE_values_set},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_values_get", _wrap_Dwg_Object_ASSOCVARIABLE_values_get},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_name_set", _wrap_Dwg_Object_ASSOCVARIABLE_name_set},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_name_get", _wrap_Dwg_Object_ASSOCVARIABLE_name_get},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_t58_set", _wrap_Dwg_Object_ASSOCVARIABLE_t58_set},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_t58_get", _wrap_Dwg_Object_ASSOCVARIABLE_t58_get},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_evaluator_set", _wrap_Dwg_Object_ASSOCVARIABLE_evaluator_set},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_evaluator_get", _wrap_Dwg_Object_ASSOCVARIABLE_evaluator_get},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_desc_set", _wrap_Dwg_Object_ASSOCVARIABLE_desc_set},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_desc_get", _wrap_Dwg_Object_ASSOCVARIABLE_desc_get},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_value_set", _wrap_Dwg_Object_ASSOCVARIABLE_value_set},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_value_get", _wrap_Dwg_Object_ASSOCVARIABLE_value_get},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_has_t78_set", _wrap_Dwg_Object_ASSOCVARIABLE_has_t78_set},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_has_t78_get", _wrap_Dwg_Object_ASSOCVARIABLE_has_t78_get},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_t78_set", _wrap_Dwg_Object_ASSOCVARIABLE_t78_set},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_t78_get", _wrap_Dwg_Object_ASSOCVARIABLE_t78_get},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_b290_set", _wrap_Dwg_Object_ASSOCVARIABLE_b290_set},
{"LibreDWGc::Dwg_Object_ASSOCVARIABLE_b290_get", _wrap_Dwg_Object_ASSOCVARIABLE_b290_get},
{"LibreDWGc::new_Dwg_Object_ASSOCVARIABLE", _wrap_new_Dwg_Object_ASSOCVARIABLE},
{"LibreDWGc::delete_Dwg_Object_ASSOCVARIABLE", _wrap_delete_Dwg_Object_ASSOCVARIABLE},
{"LibreDWGc::Dwg_EVAL_Node_parent_set", _wrap_Dwg_EVAL_Node_parent_set},
{"LibreDWGc::Dwg_EVAL_Node_parent_get", _wrap_Dwg_EVAL_Node_parent_get},
{"LibreDWGc::Dwg_EVAL_Node_id_set", _wrap_Dwg_EVAL_Node_id_set},
{"LibreDWGc::Dwg_EVAL_Node_id_get", _wrap_Dwg_EVAL_Node_id_get},
{"LibreDWGc::Dwg_EVAL_Node_edge_flags_set", _wrap_Dwg_EVAL_Node_edge_flags_set},
{"LibreDWGc::Dwg_EVAL_Node_edge_flags_get", _wrap_Dwg_EVAL_Node_edge_flags_get},
{"LibreDWGc::Dwg_EVAL_Node_nextid_set", _wrap_Dwg_EVAL_Node_nextid_set},
{"LibreDWGc::Dwg_EVAL_Node_nextid_get", _wrap_Dwg_EVAL_Node_nextid_get},
{"LibreDWGc::Dwg_EVAL_Node_evalexpr_set", _wrap_Dwg_EVAL_Node_evalexpr_set},
{"LibreDWGc::Dwg_EVAL_Node_evalexpr_get", _wrap_Dwg_EVAL_Node_evalexpr_get},
{"LibreDWGc::Dwg_EVAL_Node_node_set", _wrap_Dwg_EVAL_Node_node_set},
{"LibreDWGc::Dwg_EVAL_Node_node_get", _wrap_Dwg_EVAL_Node_node_get},
{"LibreDWGc::Dwg_EVAL_Node_active_cycles_set", _wrap_Dwg_EVAL_Node_active_cycles_set},
{"LibreDWGc::Dwg_EVAL_Node_active_cycles_get", _wrap_Dwg_EVAL_Node_active_cycles_get},
{"LibreDWGc::new_Dwg_EVAL_Node", _wrap_new_Dwg_EVAL_Node},
{"LibreDWGc::delete_Dwg_EVAL_Node", _wrap_delete_Dwg_EVAL_Node},
{"LibreDWGc::Dwg_EVAL_Edge_parent_set", _wrap_Dwg_EVAL_Edge_parent_set},
{"LibreDWGc::Dwg_EVAL_Edge_parent_get", _wrap_Dwg_EVAL_Edge_parent_get},
{"LibreDWGc::Dwg_EVAL_Edge_id_set", _wrap_Dwg_EVAL_Edge_id_set},
{"LibreDWGc::Dwg_EVAL_Edge_id_get", _wrap_Dwg_EVAL_Edge_id_get},
{"LibreDWGc::Dwg_EVAL_Edge_nextid_set", _wrap_Dwg_EVAL_Edge_nextid_set},
{"LibreDWGc::Dwg_EVAL_Edge_nextid_get", _wrap_Dwg_EVAL_Edge_nextid_get},
{"LibreDWGc::Dwg_EVAL_Edge_e1_set", _wrap_Dwg_EVAL_Edge_e1_set},
{"LibreDWGc::Dwg_EVAL_Edge_e1_get", _wrap_Dwg_EVAL_Edge_e1_get},
{"LibreDWGc::Dwg_EVAL_Edge_e2_set", _wrap_Dwg_EVAL_Edge_e2_set},
{"LibreDWGc::Dwg_EVAL_Edge_e2_get", _wrap_Dwg_EVAL_Edge_e2_get},
{"LibreDWGc::Dwg_EVAL_Edge_e3_set", _wrap_Dwg_EVAL_Edge_e3_set},
{"LibreDWGc::Dwg_EVAL_Edge_e3_get", _wrap_Dwg_EVAL_Edge_e3_get},
{"LibreDWGc::Dwg_EVAL_Edge_out_edge_set", _wrap_Dwg_EVAL_Edge_out_edge_set},
{"LibreDWGc::Dwg_EVAL_Edge_out_edge_get", _wrap_Dwg_EVAL_Edge_out_edge_get},
{"LibreDWGc::new_Dwg_EVAL_Edge", _wrap_new_Dwg_EVAL_Edge},
{"LibreDWGc::delete_Dwg_EVAL_Edge", _wrap_delete_Dwg_EVAL_Edge},
{"LibreDWGc::Dwg_Object_EVALUATION_GRAPH_parent_set", _wrap_Dwg_Object_EVALUATION_GRAPH_parent_set},
{"LibreDWGc::Dwg_Object_EVALUATION_GRAPH_parent_get", _wrap_Dwg_Object_EVALUATION_GRAPH_parent_get},
{"LibreDWGc::Dwg_Object_EVALUATION_GRAPH_major_set", _wrap_Dwg_Object_EVALUATION_GRAPH_major_set},
{"LibreDWGc::Dwg_Object_EVALUATION_GRAPH_major_get", _wrap_Dwg_Object_EVALUATION_GRAPH_major_get},
{"LibreDWGc::Dwg_Object_EVALUATION_GRAPH_minor_set", _wrap_Dwg_Object_EVALUATION_GRAPH_minor_set},
{"LibreDWGc::Dwg_Object_EVALUATION_GRAPH_minor_get", _wrap_Dwg_Object_EVALUATION_GRAPH_minor_get},
{"LibreDWGc::Dwg_Object_EVALUATION_GRAPH_first_nodeid_set", _wrap_Dwg_Object_EVALUATION_GRAPH_first_nodeid_set},
{"LibreDWGc::Dwg_Object_EVALUATION_GRAPH_first_nodeid_get", _wrap_Dwg_Object_EVALUATION_GRAPH_first_nodeid_get},
{"LibreDWGc::Dwg_Object_EVALUATION_GRAPH_first_nodeid_copy_set", _wrap_Dwg_Object_EVALUATION_GRAPH_first_nodeid_copy_set},
{"LibreDWGc::Dwg_Object_EVALUATION_GRAPH_first_nodeid_copy_get", _wrap_Dwg_Object_EVALUATION_GRAPH_first_nodeid_copy_get},
{"LibreDWGc::Dwg_Object_EVALUATION_GRAPH_num_nodes_set", _wrap_Dwg_Object_EVALUATION_GRAPH_num_nodes_set},
{"LibreDWGc::Dwg_Object_EVALUATION_GRAPH_num_nodes_get", _wrap_Dwg_Object_EVALUATION_GRAPH_num_nodes_get},
{"LibreDWGc::Dwg_Object_EVALUATION_GRAPH_nodes_set", _wrap_Dwg_Object_EVALUATION_GRAPH_nodes_set},
{"LibreDWGc::Dwg_Object_EVALUATION_GRAPH_nodes_get", _wrap_Dwg_Object_EVALUATION_GRAPH_nodes_get},
{"LibreDWGc::Dwg_Object_EVALUATION_GRAPH_has_graph_set", _wrap_Dwg_Object_EVALUATION_GRAPH_has_graph_set},
{"LibreDWGc::Dwg_Object_EVALUATION_GRAPH_has_graph_get", _wrap_Dwg_Object_EVALUATION_GRAPH_has_graph_get},
{"LibreDWGc::Dwg_Object_EVALUATION_GRAPH_num_edges_set", _wrap_Dwg_Object_EVALUATION_GRAPH_num_edges_set},
{"LibreDWGc::Dwg_Object_EVALUATION_GRAPH_num_edges_get", _wrap_Dwg_Object_EVALUATION_GRAPH_num_edges_get},
{"LibreDWGc::Dwg_Object_EVALUATION_GRAPH_edges_set", _wrap_Dwg_Object_EVALUATION_GRAPH_edges_set},
{"LibreDWGc::Dwg_Object_EVALUATION_GRAPH_edges_get", _wrap_Dwg_Object_EVALUATION_GRAPH_edges_get},
{"LibreDWGc::new_Dwg_Object_EVALUATION_GRAPH", _wrap_new_Dwg_Object_EVALUATION_GRAPH},
{"LibreDWGc::delete_Dwg_Object_EVALUATION_GRAPH", _wrap_delete_Dwg_Object_EVALUATION_GRAPH},
{"LibreDWGc::Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_parent_set", _wrap_Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_parent_set},
{"LibreDWGc::Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_parent_get", _wrap_Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_parent_get},
{"LibreDWGc::Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_flag_set", _wrap_Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_flag_set},
{"LibreDWGc::Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_flag_get", _wrap_Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_flag_get},
{"LibreDWGc::Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_block_set", _wrap_Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_block_set},
{"LibreDWGc::Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_block_get", _wrap_Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_block_get},
{"LibreDWGc::new_Dwg_Object_DYNAMICBLOCKPURGEPREVENTER", _wrap_new_Dwg_Object_DYNAMICBLOCKPURGEPREVENTER},
{"LibreDWGc::delete_Dwg_Object_DYNAMICBLOCKPURGEPREVENTER", _wrap_delete_Dwg_Object_DYNAMICBLOCKPURGEPREVENTER},
{"LibreDWGc::Dwg_Object_PERSUBENTMGR_parent_set", _wrap_Dwg_Object_PERSUBENTMGR_parent_set},
{"LibreDWGc::Dwg_Object_PERSUBENTMGR_parent_get", _wrap_Dwg_Object_PERSUBENTMGR_parent_get},
{"LibreDWGc::Dwg_Object_PERSUBENTMGR_class_version_set", _wrap_Dwg_Object_PERSUBENTMGR_class_version_set},
{"LibreDWGc::Dwg_Object_PERSUBENTMGR_class_version_get", _wrap_Dwg_Object_PERSUBENTMGR_class_version_get},
{"LibreDWGc::Dwg_Object_PERSUBENTMGR_unknown_0_set", _wrap_Dwg_Object_PERSUBENTMGR_unknown_0_set},
{"LibreDWGc::Dwg_Object_PERSUBENTMGR_unknown_0_get", _wrap_Dwg_Object_PERSUBENTMGR_unknown_0_get},
{"LibreDWGc::Dwg_Object_PERSUBENTMGR_unknown_2_set", _wrap_Dwg_Object_PERSUBENTMGR_unknown_2_set},
{"LibreDWGc::Dwg_Object_PERSUBENTMGR_unknown_2_get", _wrap_Dwg_Object_PERSUBENTMGR_unknown_2_get},
{"LibreDWGc::Dwg_Object_PERSUBENTMGR_numassocsteps_set", _wrap_Dwg_Object_PERSUBENTMGR_numassocsteps_set},
{"LibreDWGc::Dwg_Object_PERSUBENTMGR_numassocsteps_get", _wrap_Dwg_Object_PERSUBENTMGR_numassocsteps_get},
{"LibreDWGc::Dwg_Object_PERSUBENTMGR_numassocsubents_set", _wrap_Dwg_Object_PERSUBENTMGR_numassocsubents_set},
{"LibreDWGc::Dwg_Object_PERSUBENTMGR_numassocsubents_get", _wrap_Dwg_Object_PERSUBENTMGR_numassocsubents_get},
{"LibreDWGc::Dwg_Object_PERSUBENTMGR_num_steps_set", _wrap_Dwg_Object_PERSUBENTMGR_num_steps_set},
{"LibreDWGc::Dwg_Object_PERSUBENTMGR_num_steps_get", _wrap_Dwg_Object_PERSUBENTMGR_num_steps_get},
{"LibreDWGc::Dwg_Object_PERSUBENTMGR_steps_set", _wrap_Dwg_Object_PERSUBENTMGR_steps_set},
{"LibreDWGc::Dwg_Object_PERSUBENTMGR_steps_get", _wrap_Dwg_Object_PERSUBENTMGR_steps_get},
{"LibreDWGc::Dwg_Object_PERSUBENTMGR_num_subents_set", _wrap_Dwg_Object_PERSUBENTMGR_num_subents_set},
{"LibreDWGc::Dwg_Object_PERSUBENTMGR_num_subents_get", _wrap_Dwg_Object_PERSUBENTMGR_num_subents_get},
{"LibreDWGc::Dwg_Object_PERSUBENTMGR_subents_set", _wrap_Dwg_Object_PERSUBENTMGR_subents_set},
{"LibreDWGc::Dwg_Object_PERSUBENTMGR_subents_get", _wrap_Dwg_Object_PERSUBENTMGR_subents_get},
{"LibreDWGc::new_Dwg_Object_PERSUBENTMGR", _wrap_new_Dwg_Object_PERSUBENTMGR},
{"LibreDWGc::delete_Dwg_Object_PERSUBENTMGR", _wrap_delete_Dwg_Object_PERSUBENTMGR},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_parent_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_parent_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_class_version_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_class_version_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_class_version_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_3_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_3_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_3_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_3_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_0_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_0_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_0_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_0_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_2_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_2_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_2_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_2_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_num_steps_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_num_steps_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_num_steps_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_num_steps_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_num_subents_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_num_subents_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_num_subents_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_num_subents_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_steps_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_steps_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_steps_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_steps_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_subents_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_subents_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_subents_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_subents_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl6_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl6_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl6_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl6_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl6a_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl6a_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl6a_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl6a_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl7a_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl7a_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl7a_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl7a_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl7_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl7_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl7_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl7_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl8_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl8_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl8_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl8_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl9_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl9_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl9_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl9_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl10_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl10_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl10_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl10_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl11_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl11_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl11_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl11_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl12_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl12_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl12_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl12_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl13_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl13_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl13_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl13_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl14_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl14_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl14_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl14_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl15_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl15_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl15_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl15_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl16_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl16_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl16_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl16_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl17_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl17_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl17_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl17_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl18_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl18_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl18_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl18_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl19_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl19_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl19_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl19_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl20_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl20_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl20_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl20_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl21_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl21_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl21_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl21_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl22_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl22_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl22_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl22_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl23_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl23_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl23_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl23_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl24_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl24_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl24_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl24_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl25_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl25_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl25_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl25_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl26_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl26_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl26_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl26_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl27_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl27_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl27_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl27_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl28_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl28_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl28_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl28_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl29_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl29_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl29_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl29_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl30_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl30_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl30_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl30_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl31_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl31_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl31_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl31_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl32_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl32_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl32_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl32_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl33_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl33_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl33_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl33_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl34_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl34_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl34_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl34_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl35_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl35_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl35_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl35_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl36_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl36_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl36_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl36_get},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_b37_set", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_b37_set},
{"LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_b37_get", _wrap_Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_b37_get},
{"LibreDWGc::new_Dwg_Object_ASSOCPERSSUBENTMANAGER", _wrap_new_Dwg_Object_ASSOCPERSSUBENTMANAGER},
{"LibreDWGc::delete_Dwg_Object_ASSOCPERSSUBENTMANAGER", _wrap_delete_Dwg_Object_ASSOCPERSSUBENTMANAGER},
{"LibreDWGc::Dwg_Object_ASSOCACTIONPARAM_parent_set", _wrap_Dwg_Object_ASSOCACTIONPARAM_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCACTIONPARAM_parent_get", _wrap_Dwg_Object_ASSOCACTIONPARAM_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCACTIONPARAM_is_r2013_set", _wrap_Dwg_Object_ASSOCACTIONPARAM_is_r2013_set},
{"LibreDWGc::Dwg_Object_ASSOCACTIONPARAM_is_r2013_get", _wrap_Dwg_Object_ASSOCACTIONPARAM_is_r2013_get},
{"LibreDWGc::Dwg_Object_ASSOCACTIONPARAM_aap_version_set", _wrap_Dwg_Object_ASSOCACTIONPARAM_aap_version_set},
{"LibreDWGc::Dwg_Object_ASSOCACTIONPARAM_aap_version_get", _wrap_Dwg_Object_ASSOCACTIONPARAM_aap_version_get},
{"LibreDWGc::Dwg_Object_ASSOCACTIONPARAM_name_set", _wrap_Dwg_Object_ASSOCACTIONPARAM_name_set},
{"LibreDWGc::Dwg_Object_ASSOCACTIONPARAM_name_get", _wrap_Dwg_Object_ASSOCACTIONPARAM_name_get},
{"LibreDWGc::new_Dwg_Object_ASSOCACTIONPARAM", _wrap_new_Dwg_Object_ASSOCACTIONPARAM},
{"LibreDWGc::delete_Dwg_Object_ASSOCACTIONPARAM", _wrap_delete_Dwg_Object_ASSOCACTIONPARAM},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_parent_set", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_parent_get", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_is_r2013_set", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_is_r2013_set},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_is_r2013_get", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_is_r2013_get},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_aap_version_set", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_aap_version_set},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_aap_version_get", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_aap_version_get},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_name_set", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_name_set},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_name_get", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_name_get},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_class_version_set", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_class_version_get", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_class_version_get},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_bs1_set", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_bs1_set},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_bs1_get", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_bs1_get},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_num_params_set", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_num_params_set},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_num_params_get", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_num_params_get},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_params_set", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_params_set},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_params_get", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_params_get},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_has_child_param_set", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_has_child_param_set},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_has_child_param_get", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_has_child_param_get},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_status_set", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_status_set},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_status_get", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_status_get},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_id_set", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_id_set},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_id_get", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_id_get},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_param_set", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_param_set},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_param_get", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_param_get},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_h330_2_set", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_h330_2_set},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_h330_2_get", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_h330_2_get},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_bl2_set", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_bl2_set},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_bl2_get", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_bl2_get},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_h330_3_set", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_h330_3_set},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_h330_3_get", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_h330_3_get},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_status_set", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_status_set},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_status_get", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_status_get},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_osnap_mode_set", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_osnap_mode_set},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_osnap_mode_get", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_osnap_mode_get},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_param_set", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_param_set},
{"LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_param_get", _wrap_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_param_get},
{"LibreDWGc::new_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM", _wrap_new_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM},
{"LibreDWGc::delete_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM", _wrap_delete_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM},
{"LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_parent_set", _wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_parent_get", _wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_is_r2013_set", _wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_is_r2013_set},
{"LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_is_r2013_get", _wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_is_r2013_get},
{"LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_aap_version_set", _wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_aap_version_set},
{"LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_aap_version_get", _wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_aap_version_get},
{"LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_name_set", _wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_name_set},
{"LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_name_get", _wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_name_get},
{"LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_class_version_set", _wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_class_version_get", _wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_class_version_get},
{"LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_bs1_set", _wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_bs1_set},
{"LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_bs1_get", _wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_bs1_get},
{"LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_num_params_set", _wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_num_params_set},
{"LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_num_params_get", _wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_num_params_get},
{"LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_params_set", _wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_params_set},
{"LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_params_get", _wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_params_get},
{"LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_has_child_param_set", _wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_has_child_param_set},
{"LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_has_child_param_get", _wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_has_child_param_get},
{"LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_status_set", _wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_status_set},
{"LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_status_get", _wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_status_get},
{"LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_id_set", _wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_id_set},
{"LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_id_get", _wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_id_get},
{"LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_param_set", _wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_param_set},
{"LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_param_get", _wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_param_get},
{"LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_h330_2_set", _wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_h330_2_set},
{"LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_h330_2_get", _wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_h330_2_get},
{"LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_bl2_set", _wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_bl2_set},
{"LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_bl2_get", _wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_bl2_get},
{"LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_h330_3_set", _wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_h330_3_set},
{"LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_h330_3_get", _wrap_Dwg_Object_ASSOCPOINTREFACTIONPARAM_h330_3_get},
{"LibreDWGc::new_Dwg_Object_ASSOCPOINTREFACTIONPARAM", _wrap_new_Dwg_Object_ASSOCPOINTREFACTIONPARAM},
{"LibreDWGc::delete_Dwg_Object_ASSOCPOINTREFACTIONPARAM", _wrap_delete_Dwg_Object_ASSOCPOINTREFACTIONPARAM},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_parent_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_parent_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_is_r2013_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_is_r2013_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_is_r2013_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_is_r2013_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_aap_version_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_aap_version_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_aap_version_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_aap_version_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_name_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_name_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_name_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_name_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_asdap_class_version_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_asdap_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_asdap_class_version_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_asdap_class_version_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_dep_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_dep_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_dep_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_dep_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_class_version_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_class_version_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_class_version_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_unknown_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_unknown_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_unknown_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_unknown_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_version_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_version_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_version_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_version_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_num_blocks_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_num_blocks_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_num_blocks_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_num_blocks_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_block_size_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_block_size_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_block_size_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_block_size_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_encr_sat_data_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_encr_sat_data_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_encr_sat_data_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_encr_sat_data_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_sab_size_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_sab_size_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_sab_size_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_sab_size_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_data_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_data_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_data_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_data_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_wireframe_data_present_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_wireframe_data_present_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_wireframe_data_present_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_wireframe_data_present_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_point_present_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_point_present_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_point_present_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_point_present_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_point_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_point_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_point_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_point_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_isolines_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_isolines_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_isolines_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_isolines_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_isoline_present_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_isoline_present_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_isoline_present_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_isoline_present_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_num_wires_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_num_wires_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_num_wires_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_num_wires_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_wires_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_wires_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_wires_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_wires_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_num_silhouettes_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_num_silhouettes_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_num_silhouettes_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_num_silhouettes_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_silhouettes_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_silhouettes_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_silhouettes_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_silhouettes_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM__dxf_sab_converted_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM__dxf_sab_converted_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM__dxf_sab_converted_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM__dxf_sab_converted_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty2_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty2_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty2_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty2_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_extra_acis_data_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_extra_acis_data_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_extra_acis_data_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_extra_acis_data_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_num_materials_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_num_materials_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_num_materials_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_num_materials_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_materials_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_materials_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_materials_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_materials_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_guid_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_guid_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_guid_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_guid_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_major_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_major_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_major_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_major_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_minor1_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_minor1_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_minor1_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_minor1_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_minor2_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_minor2_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_minor2_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_minor2_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_bytes_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_bytes_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_bytes_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_bytes_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_end_marker_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_end_marker_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_end_marker_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_end_marker_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_history_id_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_history_id_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_history_id_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_history_id_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_has_revision_guid_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_has_revision_guid_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_has_revision_guid_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_has_revision_guid_get},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty_bit_set", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty_bit_set},
{"LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty_bit_get", _wrap_Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty_bit_get},
{"LibreDWGc::new_Dwg_Object_ASSOCASMBODYACTIONPARAM", _wrap_new_Dwg_Object_ASSOCASMBODYACTIONPARAM},
{"LibreDWGc::delete_Dwg_Object_ASSOCASMBODYACTIONPARAM", _wrap_delete_Dwg_Object_ASSOCASMBODYACTIONPARAM},
{"LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_parent_set", _wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_parent_get", _wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_is_r2013_set", _wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_is_r2013_set},
{"LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_is_r2013_get", _wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_is_r2013_get},
{"LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_aap_version_set", _wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_aap_version_set},
{"LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_aap_version_get", _wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_aap_version_get},
{"LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_name_set", _wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_name_set},
{"LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_name_get", _wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_name_get},
{"LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_class_version_set", _wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_class_version_get", _wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_class_version_get},
{"LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_bs1_set", _wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_bs1_set},
{"LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_bs1_get", _wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_bs1_get},
{"LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_num_params_set", _wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_num_params_set},
{"LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_num_params_get", _wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_num_params_get},
{"LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_params_set", _wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_params_set},
{"LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_params_get", _wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_params_get},
{"LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_has_child_param_set", _wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_has_child_param_set},
{"LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_has_child_param_get", _wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_has_child_param_get},
{"LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_status_set", _wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_status_set},
{"LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_status_get", _wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_status_get},
{"LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_id_set", _wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_id_set},
{"LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_id_get", _wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_id_get},
{"LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_param_set", _wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_param_set},
{"LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_param_get", _wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_param_get},
{"LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_h330_2_set", _wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_h330_2_set},
{"LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_h330_2_get", _wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_h330_2_get},
{"LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_bl2_set", _wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_bl2_set},
{"LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_bl2_get", _wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_bl2_get},
{"LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_h330_3_set", _wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_h330_3_set},
{"LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_h330_3_get", _wrap_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_h330_3_get},
{"LibreDWGc::new_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM", _wrap_new_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM},
{"LibreDWGc::delete_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM", _wrap_delete_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM},
{"LibreDWGc::Dwg_Object_ASSOCOBJECTACTIONPARAM_parent_set", _wrap_Dwg_Object_ASSOCOBJECTACTIONPARAM_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCOBJECTACTIONPARAM_parent_get", _wrap_Dwg_Object_ASSOCOBJECTACTIONPARAM_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCOBJECTACTIONPARAM_is_r2013_set", _wrap_Dwg_Object_ASSOCOBJECTACTIONPARAM_is_r2013_set},
{"LibreDWGc::Dwg_Object_ASSOCOBJECTACTIONPARAM_is_r2013_get", _wrap_Dwg_Object_ASSOCOBJECTACTIONPARAM_is_r2013_get},
{"LibreDWGc::Dwg_Object_ASSOCOBJECTACTIONPARAM_aap_version_set", _wrap_Dwg_Object_ASSOCOBJECTACTIONPARAM_aap_version_set},
{"LibreDWGc::Dwg_Object_ASSOCOBJECTACTIONPARAM_aap_version_get", _wrap_Dwg_Object_ASSOCOBJECTACTIONPARAM_aap_version_get},
{"LibreDWGc::Dwg_Object_ASSOCOBJECTACTIONPARAM_name_set", _wrap_Dwg_Object_ASSOCOBJECTACTIONPARAM_name_set},
{"LibreDWGc::Dwg_Object_ASSOCOBJECTACTIONPARAM_name_get", _wrap_Dwg_Object_ASSOCOBJECTACTIONPARAM_name_get},
{"LibreDWGc::Dwg_Object_ASSOCOBJECTACTIONPARAM_asdap_class_version_set", _wrap_Dwg_Object_ASSOCOBJECTACTIONPARAM_asdap_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCOBJECTACTIONPARAM_asdap_class_version_get", _wrap_Dwg_Object_ASSOCOBJECTACTIONPARAM_asdap_class_version_get},
{"LibreDWGc::Dwg_Object_ASSOCOBJECTACTIONPARAM_dep_set", _wrap_Dwg_Object_ASSOCOBJECTACTIONPARAM_dep_set},
{"LibreDWGc::Dwg_Object_ASSOCOBJECTACTIONPARAM_dep_get", _wrap_Dwg_Object_ASSOCOBJECTACTIONPARAM_dep_get},
{"LibreDWGc::Dwg_Object_ASSOCOBJECTACTIONPARAM_class_version_set", _wrap_Dwg_Object_ASSOCOBJECTACTIONPARAM_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCOBJECTACTIONPARAM_class_version_get", _wrap_Dwg_Object_ASSOCOBJECTACTIONPARAM_class_version_get},
{"LibreDWGc::new_Dwg_Object_ASSOCOBJECTACTIONPARAM", _wrap_new_Dwg_Object_ASSOCOBJECTACTIONPARAM},
{"LibreDWGc::delete_Dwg_Object_ASSOCOBJECTACTIONPARAM", _wrap_delete_Dwg_Object_ASSOCOBJECTACTIONPARAM},
{"LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_parent_set", _wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_parent_get", _wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_is_r2013_set", _wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_is_r2013_set},
{"LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_is_r2013_get", _wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_is_r2013_get},
{"LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_aap_version_set", _wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_aap_version_set},
{"LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_aap_version_get", _wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_aap_version_get},
{"LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_name_set", _wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_name_set},
{"LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_name_get", _wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_name_get},
{"LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_asdap_class_version_set", _wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_asdap_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_asdap_class_version_get", _wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_asdap_class_version_get},
{"LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_dep_set", _wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_dep_set},
{"LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_dep_get", _wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_dep_get},
{"LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_class_version_set", _wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_class_version_get", _wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_class_version_get},
{"LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_param_set", _wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_param_set},
{"LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_param_get", _wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_param_get},
{"LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_has_action_set", _wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_has_action_set},
{"LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_has_action_get", _wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_has_action_get},
{"LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_action_type_set", _wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_action_type_set},
{"LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_action_type_get", _wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_action_type_get},
{"LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_subent_set", _wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_subent_set},
{"LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_subent_get", _wrap_Dwg_Object_ASSOCEDGEACTIONPARAM_subent_get},
{"LibreDWGc::new_Dwg_Object_ASSOCEDGEACTIONPARAM", _wrap_new_Dwg_Object_ASSOCEDGEACTIONPARAM},
{"LibreDWGc::delete_Dwg_Object_ASSOCEDGEACTIONPARAM", _wrap_delete_Dwg_Object_ASSOCEDGEACTIONPARAM},
{"LibreDWGc::Dwg_Object_ASSOCFACEACTIONPARAM_parent_set", _wrap_Dwg_Object_ASSOCFACEACTIONPARAM_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCFACEACTIONPARAM_parent_get", _wrap_Dwg_Object_ASSOCFACEACTIONPARAM_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCFACEACTIONPARAM_is_r2013_set", _wrap_Dwg_Object_ASSOCFACEACTIONPARAM_is_r2013_set},
{"LibreDWGc::Dwg_Object_ASSOCFACEACTIONPARAM_is_r2013_get", _wrap_Dwg_Object_ASSOCFACEACTIONPARAM_is_r2013_get},
{"LibreDWGc::Dwg_Object_ASSOCFACEACTIONPARAM_aap_version_set", _wrap_Dwg_Object_ASSOCFACEACTIONPARAM_aap_version_set},
{"LibreDWGc::Dwg_Object_ASSOCFACEACTIONPARAM_aap_version_get", _wrap_Dwg_Object_ASSOCFACEACTIONPARAM_aap_version_get},
{"LibreDWGc::Dwg_Object_ASSOCFACEACTIONPARAM_name_set", _wrap_Dwg_Object_ASSOCFACEACTIONPARAM_name_set},
{"LibreDWGc::Dwg_Object_ASSOCFACEACTIONPARAM_name_get", _wrap_Dwg_Object_ASSOCFACEACTIONPARAM_name_get},
{"LibreDWGc::Dwg_Object_ASSOCFACEACTIONPARAM_asdap_class_version_set", _wrap_Dwg_Object_ASSOCFACEACTIONPARAM_asdap_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCFACEACTIONPARAM_asdap_class_version_get", _wrap_Dwg_Object_ASSOCFACEACTIONPARAM_asdap_class_version_get},
{"LibreDWGc::Dwg_Object_ASSOCFACEACTIONPARAM_dep_set", _wrap_Dwg_Object_ASSOCFACEACTIONPARAM_dep_set},
{"LibreDWGc::Dwg_Object_ASSOCFACEACTIONPARAM_dep_get", _wrap_Dwg_Object_ASSOCFACEACTIONPARAM_dep_get},
{"LibreDWGc::Dwg_Object_ASSOCFACEACTIONPARAM_class_version_set", _wrap_Dwg_Object_ASSOCFACEACTIONPARAM_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCFACEACTIONPARAM_class_version_get", _wrap_Dwg_Object_ASSOCFACEACTIONPARAM_class_version_get},
{"LibreDWGc::Dwg_Object_ASSOCFACEACTIONPARAM_index_set", _wrap_Dwg_Object_ASSOCFACEACTIONPARAM_index_set},
{"LibreDWGc::Dwg_Object_ASSOCFACEACTIONPARAM_index_get", _wrap_Dwg_Object_ASSOCFACEACTIONPARAM_index_get},
{"LibreDWGc::new_Dwg_Object_ASSOCFACEACTIONPARAM", _wrap_new_Dwg_Object_ASSOCFACEACTIONPARAM},
{"LibreDWGc::delete_Dwg_Object_ASSOCFACEACTIONPARAM", _wrap_delete_Dwg_Object_ASSOCFACEACTIONPARAM},
{"LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_parent_set", _wrap_Dwg_Object_ASSOCPATHACTIONPARAM_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_parent_get", _wrap_Dwg_Object_ASSOCPATHACTIONPARAM_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_is_r2013_set", _wrap_Dwg_Object_ASSOCPATHACTIONPARAM_is_r2013_set},
{"LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_is_r2013_get", _wrap_Dwg_Object_ASSOCPATHACTIONPARAM_is_r2013_get},
{"LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_aap_version_set", _wrap_Dwg_Object_ASSOCPATHACTIONPARAM_aap_version_set},
{"LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_aap_version_get", _wrap_Dwg_Object_ASSOCPATHACTIONPARAM_aap_version_get},
{"LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_name_set", _wrap_Dwg_Object_ASSOCPATHACTIONPARAM_name_set},
{"LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_name_get", _wrap_Dwg_Object_ASSOCPATHACTIONPARAM_name_get},
{"LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_class_version_set", _wrap_Dwg_Object_ASSOCPATHACTIONPARAM_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_class_version_get", _wrap_Dwg_Object_ASSOCPATHACTIONPARAM_class_version_get},
{"LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_bs1_set", _wrap_Dwg_Object_ASSOCPATHACTIONPARAM_bs1_set},
{"LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_bs1_get", _wrap_Dwg_Object_ASSOCPATHACTIONPARAM_bs1_get},
{"LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_num_params_set", _wrap_Dwg_Object_ASSOCPATHACTIONPARAM_num_params_set},
{"LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_num_params_get", _wrap_Dwg_Object_ASSOCPATHACTIONPARAM_num_params_get},
{"LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_params_set", _wrap_Dwg_Object_ASSOCPATHACTIONPARAM_params_set},
{"LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_params_get", _wrap_Dwg_Object_ASSOCPATHACTIONPARAM_params_get},
{"LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_has_child_param_set", _wrap_Dwg_Object_ASSOCPATHACTIONPARAM_has_child_param_set},
{"LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_has_child_param_get", _wrap_Dwg_Object_ASSOCPATHACTIONPARAM_has_child_param_get},
{"LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_child_status_set", _wrap_Dwg_Object_ASSOCPATHACTIONPARAM_child_status_set},
{"LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_child_status_get", _wrap_Dwg_Object_ASSOCPATHACTIONPARAM_child_status_get},
{"LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_child_id_set", _wrap_Dwg_Object_ASSOCPATHACTIONPARAM_child_id_set},
{"LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_child_id_get", _wrap_Dwg_Object_ASSOCPATHACTIONPARAM_child_id_get},
{"LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_child_param_set", _wrap_Dwg_Object_ASSOCPATHACTIONPARAM_child_param_set},
{"LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_child_param_get", _wrap_Dwg_Object_ASSOCPATHACTIONPARAM_child_param_get},
{"LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_h330_2_set", _wrap_Dwg_Object_ASSOCPATHACTIONPARAM_h330_2_set},
{"LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_h330_2_get", _wrap_Dwg_Object_ASSOCPATHACTIONPARAM_h330_2_get},
{"LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_bl2_set", _wrap_Dwg_Object_ASSOCPATHACTIONPARAM_bl2_set},
{"LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_bl2_get", _wrap_Dwg_Object_ASSOCPATHACTIONPARAM_bl2_get},
{"LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_h330_3_set", _wrap_Dwg_Object_ASSOCPATHACTIONPARAM_h330_3_set},
{"LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_h330_3_get", _wrap_Dwg_Object_ASSOCPATHACTIONPARAM_h330_3_get},
{"LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_version_set", _wrap_Dwg_Object_ASSOCPATHACTIONPARAM_version_set},
{"LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_version_get", _wrap_Dwg_Object_ASSOCPATHACTIONPARAM_version_get},
{"LibreDWGc::new_Dwg_Object_ASSOCPATHACTIONPARAM", _wrap_new_Dwg_Object_ASSOCPATHACTIONPARAM},
{"LibreDWGc::delete_Dwg_Object_ASSOCPATHACTIONPARAM", _wrap_delete_Dwg_Object_ASSOCPATHACTIONPARAM},
{"LibreDWGc::Dwg_Object_ASSOCVERTEXACTIONPARAM_parent_set", _wrap_Dwg_Object_ASSOCVERTEXACTIONPARAM_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCVERTEXACTIONPARAM_parent_get", _wrap_Dwg_Object_ASSOCVERTEXACTIONPARAM_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCVERTEXACTIONPARAM_is_r2013_set", _wrap_Dwg_Object_ASSOCVERTEXACTIONPARAM_is_r2013_set},
{"LibreDWGc::Dwg_Object_ASSOCVERTEXACTIONPARAM_is_r2013_get", _wrap_Dwg_Object_ASSOCVERTEXACTIONPARAM_is_r2013_get},
{"LibreDWGc::Dwg_Object_ASSOCVERTEXACTIONPARAM_aap_version_set", _wrap_Dwg_Object_ASSOCVERTEXACTIONPARAM_aap_version_set},
{"LibreDWGc::Dwg_Object_ASSOCVERTEXACTIONPARAM_aap_version_get", _wrap_Dwg_Object_ASSOCVERTEXACTIONPARAM_aap_version_get},
{"LibreDWGc::Dwg_Object_ASSOCVERTEXACTIONPARAM_name_set", _wrap_Dwg_Object_ASSOCVERTEXACTIONPARAM_name_set},
{"LibreDWGc::Dwg_Object_ASSOCVERTEXACTIONPARAM_name_get", _wrap_Dwg_Object_ASSOCVERTEXACTIONPARAM_name_get},
{"LibreDWGc::Dwg_Object_ASSOCVERTEXACTIONPARAM_asdap_class_version_set", _wrap_Dwg_Object_ASSOCVERTEXACTIONPARAM_asdap_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCVERTEXACTIONPARAM_asdap_class_version_get", _wrap_Dwg_Object_ASSOCVERTEXACTIONPARAM_asdap_class_version_get},
{"LibreDWGc::Dwg_Object_ASSOCVERTEXACTIONPARAM_dep_set", _wrap_Dwg_Object_ASSOCVERTEXACTIONPARAM_dep_set},
{"LibreDWGc::Dwg_Object_ASSOCVERTEXACTIONPARAM_dep_get", _wrap_Dwg_Object_ASSOCVERTEXACTIONPARAM_dep_get},
{"LibreDWGc::Dwg_Object_ASSOCVERTEXACTIONPARAM_class_version_set", _wrap_Dwg_Object_ASSOCVERTEXACTIONPARAM_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCVERTEXACTIONPARAM_class_version_get", _wrap_Dwg_Object_ASSOCVERTEXACTIONPARAM_class_version_get},
{"LibreDWGc::Dwg_Object_ASSOCVERTEXACTIONPARAM_pt_set", _wrap_Dwg_Object_ASSOCVERTEXACTIONPARAM_pt_set},
{"LibreDWGc::Dwg_Object_ASSOCVERTEXACTIONPARAM_pt_get", _wrap_Dwg_Object_ASSOCVERTEXACTIONPARAM_pt_get},
{"LibreDWGc::new_Dwg_Object_ASSOCVERTEXACTIONPARAM", _wrap_new_Dwg_Object_ASSOCVERTEXACTIONPARAM},
{"LibreDWGc::delete_Dwg_Object_ASSOCVERTEXACTIONPARAM", _wrap_delete_Dwg_Object_ASSOCVERTEXACTIONPARAM},
{"LibreDWGc::Dwg_ASSOCARRAYITEM_parent_set", _wrap_Dwg_ASSOCARRAYITEM_parent_set},
{"LibreDWGc::Dwg_ASSOCARRAYITEM_parent_get", _wrap_Dwg_ASSOCARRAYITEM_parent_get},
{"LibreDWGc::Dwg_ASSOCARRAYITEM_class_version_set", _wrap_Dwg_ASSOCARRAYITEM_class_version_set},
{"LibreDWGc::Dwg_ASSOCARRAYITEM_class_version_get", _wrap_Dwg_ASSOCARRAYITEM_class_version_get},
{"LibreDWGc::Dwg_ASSOCARRAYITEM_itemloc_set", _wrap_Dwg_ASSOCARRAYITEM_itemloc_set},
{"LibreDWGc::Dwg_ASSOCARRAYITEM_itemloc_get", _wrap_Dwg_ASSOCARRAYITEM_itemloc_get},
{"LibreDWGc::Dwg_ASSOCARRAYITEM_flags_set", _wrap_Dwg_ASSOCARRAYITEM_flags_set},
{"LibreDWGc::Dwg_ASSOCARRAYITEM_flags_get", _wrap_Dwg_ASSOCARRAYITEM_flags_get},
{"LibreDWGc::Dwg_ASSOCARRAYITEM_is_default_transmatrix_set", _wrap_Dwg_ASSOCARRAYITEM_is_default_transmatrix_set},
{"LibreDWGc::Dwg_ASSOCARRAYITEM_is_default_transmatrix_get", _wrap_Dwg_ASSOCARRAYITEM_is_default_transmatrix_get},
{"LibreDWGc::Dwg_ASSOCARRAYITEM_x_dir_set", _wrap_Dwg_ASSOCARRAYITEM_x_dir_set},
{"LibreDWGc::Dwg_ASSOCARRAYITEM_x_dir_get", _wrap_Dwg_ASSOCARRAYITEM_x_dir_get},
{"LibreDWGc::Dwg_ASSOCARRAYITEM_transmatrix_set", _wrap_Dwg_ASSOCARRAYITEM_transmatrix_set},
{"LibreDWGc::Dwg_ASSOCARRAYITEM_transmatrix_get", _wrap_Dwg_ASSOCARRAYITEM_transmatrix_get},
{"LibreDWGc::Dwg_ASSOCARRAYITEM_rel_transform_set", _wrap_Dwg_ASSOCARRAYITEM_rel_transform_set},
{"LibreDWGc::Dwg_ASSOCARRAYITEM_rel_transform_get", _wrap_Dwg_ASSOCARRAYITEM_rel_transform_get},
{"LibreDWGc::Dwg_ASSOCARRAYITEM_has_h1_set", _wrap_Dwg_ASSOCARRAYITEM_has_h1_set},
{"LibreDWGc::Dwg_ASSOCARRAYITEM_has_h1_get", _wrap_Dwg_ASSOCARRAYITEM_has_h1_get},
{"LibreDWGc::Dwg_ASSOCARRAYITEM_h1_set", _wrap_Dwg_ASSOCARRAYITEM_h1_set},
{"LibreDWGc::Dwg_ASSOCARRAYITEM_h1_get", _wrap_Dwg_ASSOCARRAYITEM_h1_get},
{"LibreDWGc::Dwg_ASSOCARRAYITEM_h2_set", _wrap_Dwg_ASSOCARRAYITEM_h2_set},
{"LibreDWGc::Dwg_ASSOCARRAYITEM_h2_get", _wrap_Dwg_ASSOCARRAYITEM_h2_get},
{"LibreDWGc::new_Dwg_ASSOCARRAYITEM", _wrap_new_Dwg_ASSOCARRAYITEM},
{"LibreDWGc::delete_Dwg_ASSOCARRAYITEM", _wrap_delete_Dwg_ASSOCARRAYITEM},
{"LibreDWGc::Dwg_Object_ASSOCARRAYPARAMETERS_parent_set", _wrap_Dwg_Object_ASSOCARRAYPARAMETERS_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCARRAYPARAMETERS_parent_get", _wrap_Dwg_Object_ASSOCARRAYPARAMETERS_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCARRAYPARAMETERS_aap_version_set", _wrap_Dwg_Object_ASSOCARRAYPARAMETERS_aap_version_set},
{"LibreDWGc::Dwg_Object_ASSOCARRAYPARAMETERS_aap_version_get", _wrap_Dwg_Object_ASSOCARRAYPARAMETERS_aap_version_get},
{"LibreDWGc::Dwg_Object_ASSOCARRAYPARAMETERS_num_items_set", _wrap_Dwg_Object_ASSOCARRAYPARAMETERS_num_items_set},
{"LibreDWGc::Dwg_Object_ASSOCARRAYPARAMETERS_num_items_get", _wrap_Dwg_Object_ASSOCARRAYPARAMETERS_num_items_get},
{"LibreDWGc::Dwg_Object_ASSOCARRAYPARAMETERS_classname_set", _wrap_Dwg_Object_ASSOCARRAYPARAMETERS_classname_set},
{"LibreDWGc::Dwg_Object_ASSOCARRAYPARAMETERS_classname_get", _wrap_Dwg_Object_ASSOCARRAYPARAMETERS_classname_get},
{"LibreDWGc::Dwg_Object_ASSOCARRAYPARAMETERS_items_set", _wrap_Dwg_Object_ASSOCARRAYPARAMETERS_items_set},
{"LibreDWGc::Dwg_Object_ASSOCARRAYPARAMETERS_items_get", _wrap_Dwg_Object_ASSOCARRAYPARAMETERS_items_get},
{"LibreDWGc::Dwg_Object_ASSOCARRAYPARAMETERS_numitems_set", _wrap_Dwg_Object_ASSOCARRAYPARAMETERS_numitems_set},
{"LibreDWGc::Dwg_Object_ASSOCARRAYPARAMETERS_numitems_get", _wrap_Dwg_Object_ASSOCARRAYPARAMETERS_numitems_get},
{"LibreDWGc::Dwg_Object_ASSOCARRAYPARAMETERS_numrows_set", _wrap_Dwg_Object_ASSOCARRAYPARAMETERS_numrows_set},
{"LibreDWGc::Dwg_Object_ASSOCARRAYPARAMETERS_numrows_get", _wrap_Dwg_Object_ASSOCARRAYPARAMETERS_numrows_get},
{"LibreDWGc::Dwg_Object_ASSOCARRAYPARAMETERS_numlevels_set", _wrap_Dwg_Object_ASSOCARRAYPARAMETERS_numlevels_set},
{"LibreDWGc::Dwg_Object_ASSOCARRAYPARAMETERS_numlevels_get", _wrap_Dwg_Object_ASSOCARRAYPARAMETERS_numlevels_get},
{"LibreDWGc::new_Dwg_Object_ASSOCARRAYPARAMETERS", _wrap_new_Dwg_Object_ASSOCARRAYPARAMETERS},
{"LibreDWGc::delete_Dwg_Object_ASSOCARRAYPARAMETERS", _wrap_delete_Dwg_Object_ASSOCARRAYPARAMETERS},
{"LibreDWGc::Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_parent_set", _wrap_Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_parent_get", _wrap_Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_aab_version_set", _wrap_Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_aab_version_set},
{"LibreDWGc::Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_aab_version_get", _wrap_Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_aab_version_get},
{"LibreDWGc::Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_class_version_set", _wrap_Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_class_version_get", _wrap_Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_class_version_get},
{"LibreDWGc::Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_entity_set", _wrap_Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_entity_set},
{"LibreDWGc::Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_entity_get", _wrap_Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_entity_get},
{"LibreDWGc::new_Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY", _wrap_new_Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY},
{"LibreDWGc::delete_Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY", _wrap_delete_Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY},
{"LibreDWGc::Dwg_ASSOCSURFACEACTIONBODY_parent_set", _wrap_Dwg_ASSOCSURFACEACTIONBODY_parent_set},
{"LibreDWGc::Dwg_ASSOCSURFACEACTIONBODY_parent_get", _wrap_Dwg_ASSOCSURFACEACTIONBODY_parent_get},
{"LibreDWGc::Dwg_ASSOCSURFACEACTIONBODY_version_set", _wrap_Dwg_ASSOCSURFACEACTIONBODY_version_set},
{"LibreDWGc::Dwg_ASSOCSURFACEACTIONBODY_version_get", _wrap_Dwg_ASSOCSURFACEACTIONBODY_version_get},
{"LibreDWGc::Dwg_ASSOCSURFACEACTIONBODY_is_semi_assoc_set", _wrap_Dwg_ASSOCSURFACEACTIONBODY_is_semi_assoc_set},
{"LibreDWGc::Dwg_ASSOCSURFACEACTIONBODY_is_semi_assoc_get", _wrap_Dwg_ASSOCSURFACEACTIONBODY_is_semi_assoc_get},
{"LibreDWGc::Dwg_ASSOCSURFACEACTIONBODY_l2_set", _wrap_Dwg_ASSOCSURFACEACTIONBODY_l2_set},
{"LibreDWGc::Dwg_ASSOCSURFACEACTIONBODY_l2_get", _wrap_Dwg_ASSOCSURFACEACTIONBODY_l2_get},
{"LibreDWGc::Dwg_ASSOCSURFACEACTIONBODY_is_semi_ovr_set", _wrap_Dwg_ASSOCSURFACEACTIONBODY_is_semi_ovr_set},
{"LibreDWGc::Dwg_ASSOCSURFACEACTIONBODY_is_semi_ovr_get", _wrap_Dwg_ASSOCSURFACEACTIONBODY_is_semi_ovr_get},
{"LibreDWGc::Dwg_ASSOCSURFACEACTIONBODY_grip_status_set", _wrap_Dwg_ASSOCSURFACEACTIONBODY_grip_status_set},
{"LibreDWGc::Dwg_ASSOCSURFACEACTIONBODY_grip_status_get", _wrap_Dwg_ASSOCSURFACEACTIONBODY_grip_status_get},
{"LibreDWGc::Dwg_ASSOCSURFACEACTIONBODY_assocdep_set", _wrap_Dwg_ASSOCSURFACEACTIONBODY_assocdep_set},
{"LibreDWGc::Dwg_ASSOCSURFACEACTIONBODY_assocdep_get", _wrap_Dwg_ASSOCSURFACEACTIONBODY_assocdep_get},
{"LibreDWGc::new_Dwg_ASSOCSURFACEACTIONBODY", _wrap_new_Dwg_ASSOCSURFACEACTIONBODY},
{"LibreDWGc::delete_Dwg_ASSOCSURFACEACTIONBODY", _wrap_delete_Dwg_ASSOCSURFACEACTIONBODY},
{"LibreDWGc::Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_parent_set", _wrap_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_parent_get", _wrap_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_aab_version_set", _wrap_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_aab_version_set},
{"LibreDWGc::Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_aab_version_get", _wrap_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_aab_version_get},
{"LibreDWGc::Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_pab_set", _wrap_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_pab_set},
{"LibreDWGc::Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_pab_get", _wrap_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_pab_get},
{"LibreDWGc::Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_sab_set", _wrap_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_sab_set},
{"LibreDWGc::Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_sab_get", _wrap_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_sab_get},
{"LibreDWGc::Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_pbsab_status_set", _wrap_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_pbsab_status_set},
{"LibreDWGc::Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_pbsab_status_get", _wrap_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_pbsab_status_get},
{"LibreDWGc::Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_class_version_set", _wrap_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_class_version_get", _wrap_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_class_version_get},
{"LibreDWGc::Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_option_set", _wrap_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_option_set},
{"LibreDWGc::Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_option_get", _wrap_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_option_get},
{"LibreDWGc::new_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY", _wrap_new_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY},
{"LibreDWGc::delete_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY", _wrap_delete_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY},
{"LibreDWGc::Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_parent_set", _wrap_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_parent_get", _wrap_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_aab_version_set", _wrap_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_aab_version_set},
{"LibreDWGc::Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_aab_version_get", _wrap_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_aab_version_get},
{"LibreDWGc::Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_pab_set", _wrap_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_pab_set},
{"LibreDWGc::Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_pab_get", _wrap_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_pab_get},
{"LibreDWGc::Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_sab_set", _wrap_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_sab_set},
{"LibreDWGc::Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_sab_get", _wrap_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_sab_get},
{"LibreDWGc::Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_pbsab_status_set", _wrap_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_pbsab_status_set},
{"LibreDWGc::Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_pbsab_status_get", _wrap_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_pbsab_status_get},
{"LibreDWGc::Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_class_version_set", _wrap_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_class_version_get", _wrap_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_class_version_get},
{"LibreDWGc::new_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY", _wrap_new_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY},
{"LibreDWGc::delete_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY", _wrap_delete_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY},
{"LibreDWGc::Dwg_Object_ASSOCPLANESURFACEACTIONBODY_parent_set", _wrap_Dwg_Object_ASSOCPLANESURFACEACTIONBODY_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCPLANESURFACEACTIONBODY_parent_get", _wrap_Dwg_Object_ASSOCPLANESURFACEACTIONBODY_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCPLANESURFACEACTIONBODY_aab_version_set", _wrap_Dwg_Object_ASSOCPLANESURFACEACTIONBODY_aab_version_set},
{"LibreDWGc::Dwg_Object_ASSOCPLANESURFACEACTIONBODY_aab_version_get", _wrap_Dwg_Object_ASSOCPLANESURFACEACTIONBODY_aab_version_get},
{"LibreDWGc::Dwg_Object_ASSOCPLANESURFACEACTIONBODY_pab_set", _wrap_Dwg_Object_ASSOCPLANESURFACEACTIONBODY_pab_set},
{"LibreDWGc::Dwg_Object_ASSOCPLANESURFACEACTIONBODY_pab_get", _wrap_Dwg_Object_ASSOCPLANESURFACEACTIONBODY_pab_get},
{"LibreDWGc::Dwg_Object_ASSOCPLANESURFACEACTIONBODY_sab_set", _wrap_Dwg_Object_ASSOCPLANESURFACEACTIONBODY_sab_set},
{"LibreDWGc::Dwg_Object_ASSOCPLANESURFACEACTIONBODY_sab_get", _wrap_Dwg_Object_ASSOCPLANESURFACEACTIONBODY_sab_get},
{"LibreDWGc::Dwg_Object_ASSOCPLANESURFACEACTIONBODY_pbsab_status_set", _wrap_Dwg_Object_ASSOCPLANESURFACEACTIONBODY_pbsab_status_set},
{"LibreDWGc::Dwg_Object_ASSOCPLANESURFACEACTIONBODY_pbsab_status_get", _wrap_Dwg_Object_ASSOCPLANESURFACEACTIONBODY_pbsab_status_get},
{"LibreDWGc::Dwg_Object_ASSOCPLANESURFACEACTIONBODY_class_version_set", _wrap_Dwg_Object_ASSOCPLANESURFACEACTIONBODY_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCPLANESURFACEACTIONBODY_class_version_get", _wrap_Dwg_Object_ASSOCPLANESURFACEACTIONBODY_class_version_get},
{"LibreDWGc::new_Dwg_Object_ASSOCPLANESURFACEACTIONBODY", _wrap_new_Dwg_Object_ASSOCPLANESURFACEACTIONBODY},
{"LibreDWGc::delete_Dwg_Object_ASSOCPLANESURFACEACTIONBODY", _wrap_delete_Dwg_Object_ASSOCPLANESURFACEACTIONBODY},
{"LibreDWGc::Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_parent_set", _wrap_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_parent_get", _wrap_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_aab_version_set", _wrap_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_aab_version_set},
{"LibreDWGc::Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_aab_version_get", _wrap_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_aab_version_get},
{"LibreDWGc::Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_pab_set", _wrap_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_pab_set},
{"LibreDWGc::Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_pab_get", _wrap_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_pab_get},
{"LibreDWGc::Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_sab_set", _wrap_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_sab_set},
{"LibreDWGc::Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_sab_get", _wrap_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_sab_get},
{"LibreDWGc::Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_pbsab_status_set", _wrap_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_pbsab_status_set},
{"LibreDWGc::Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_pbsab_status_get", _wrap_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_pbsab_status_get},
{"LibreDWGc::Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_class_version_set", _wrap_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_class_version_get", _wrap_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_class_version_get},
{"LibreDWGc::new_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY", _wrap_new_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY},
{"LibreDWGc::delete_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY", _wrap_delete_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY},
{"LibreDWGc::Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_parent_set", _wrap_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_parent_get", _wrap_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_aab_version_set", _wrap_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_aab_version_set},
{"LibreDWGc::Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_aab_version_get", _wrap_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_aab_version_get},
{"LibreDWGc::Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_pab_set", _wrap_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_pab_set},
{"LibreDWGc::Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_pab_get", _wrap_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_pab_get},
{"LibreDWGc::Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_sab_set", _wrap_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_sab_set},
{"LibreDWGc::Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_sab_get", _wrap_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_sab_get},
{"LibreDWGc::Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_pbsab_status_set", _wrap_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_pbsab_status_set},
{"LibreDWGc::Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_pbsab_status_get", _wrap_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_pbsab_status_get},
{"LibreDWGc::Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_class_version_set", _wrap_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_class_version_get", _wrap_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_class_version_get},
{"LibreDWGc::new_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY", _wrap_new_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY},
{"LibreDWGc::delete_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY", _wrap_delete_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY},
{"LibreDWGc::Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_parent_set", _wrap_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_parent_get", _wrap_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_aab_version_set", _wrap_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_aab_version_set},
{"LibreDWGc::Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_aab_version_get", _wrap_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_aab_version_get},
{"LibreDWGc::Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_pab_set", _wrap_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_pab_set},
{"LibreDWGc::Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_pab_get", _wrap_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_pab_get},
{"LibreDWGc::Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_sab_set", _wrap_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_sab_set},
{"LibreDWGc::Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_sab_get", _wrap_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_sab_get},
{"LibreDWGc::Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_pbsab_status_set", _wrap_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_pbsab_status_set},
{"LibreDWGc::Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_pbsab_status_get", _wrap_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_pbsab_status_get},
{"LibreDWGc::Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_class_version_set", _wrap_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_class_version_get", _wrap_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_class_version_get},
{"LibreDWGc::Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_b1_set", _wrap_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_b1_set},
{"LibreDWGc::Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_b1_get", _wrap_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_b1_get},
{"LibreDWGc::new_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY", _wrap_new_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY},
{"LibreDWGc::delete_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY", _wrap_delete_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY},
{"LibreDWGc::Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_parent_set", _wrap_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_parent_get", _wrap_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_aab_version_set", _wrap_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_aab_version_set},
{"LibreDWGc::Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_aab_version_get", _wrap_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_aab_version_get},
{"LibreDWGc::Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_pab_set", _wrap_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_pab_set},
{"LibreDWGc::Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_pab_get", _wrap_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_pab_get},
{"LibreDWGc::Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_sab_set", _wrap_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_sab_set},
{"LibreDWGc::Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_sab_get", _wrap_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_sab_get},
{"LibreDWGc::Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_pbsab_status_set", _wrap_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_pbsab_status_set},
{"LibreDWGc::Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_pbsab_status_get", _wrap_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_pbsab_status_get},
{"LibreDWGc::Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_class_version_set", _wrap_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_class_version_get", _wrap_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_class_version_get},
{"LibreDWGc::new_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY", _wrap_new_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY},
{"LibreDWGc::delete_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY", _wrap_delete_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY},
{"LibreDWGc::Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_parent_set", _wrap_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_parent_get", _wrap_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_aab_version_set", _wrap_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_aab_version_set},
{"LibreDWGc::Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_aab_version_get", _wrap_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_aab_version_get},
{"LibreDWGc::Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_pab_set", _wrap_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_pab_set},
{"LibreDWGc::Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_pab_get", _wrap_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_pab_get},
{"LibreDWGc::Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_sab_set", _wrap_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_sab_set},
{"LibreDWGc::Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_sab_get", _wrap_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_sab_get},
{"LibreDWGc::Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_pbsab_status_set", _wrap_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_pbsab_status_set},
{"LibreDWGc::Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_pbsab_status_get", _wrap_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_pbsab_status_get},
{"LibreDWGc::Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_class_version_set", _wrap_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_class_version_get", _wrap_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_class_version_get},
{"LibreDWGc::new_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY", _wrap_new_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY},
{"LibreDWGc::delete_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY", _wrap_delete_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY},
{"LibreDWGc::Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_parent_set", _wrap_Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_parent_get", _wrap_Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_aab_version_set", _wrap_Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_aab_version_set},
{"LibreDWGc::Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_aab_version_get", _wrap_Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_aab_version_get},
{"LibreDWGc::Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_pab_set", _wrap_Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_pab_set},
{"LibreDWGc::Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_pab_get", _wrap_Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_pab_get},
{"LibreDWGc::Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_sab_set", _wrap_Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_sab_set},
{"LibreDWGc::Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_sab_get", _wrap_Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_sab_get},
{"LibreDWGc::Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_pbsab_status_set", _wrap_Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_pbsab_status_set},
{"LibreDWGc::Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_pbsab_status_get", _wrap_Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_pbsab_status_get},
{"LibreDWGc::new_Dwg_Object_ASSOCEDGECHAMFERACTIONBODY", _wrap_new_Dwg_Object_ASSOCEDGECHAMFERACTIONBODY},
{"LibreDWGc::delete_Dwg_Object_ASSOCEDGECHAMFERACTIONBODY", _wrap_delete_Dwg_Object_ASSOCEDGECHAMFERACTIONBODY},
{"LibreDWGc::Dwg_Object_ASSOCEDGEFILLETACTIONBODY_parent_set", _wrap_Dwg_Object_ASSOCEDGEFILLETACTIONBODY_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCEDGEFILLETACTIONBODY_parent_get", _wrap_Dwg_Object_ASSOCEDGEFILLETACTIONBODY_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCEDGEFILLETACTIONBODY_aab_version_set", _wrap_Dwg_Object_ASSOCEDGEFILLETACTIONBODY_aab_version_set},
{"LibreDWGc::Dwg_Object_ASSOCEDGEFILLETACTIONBODY_aab_version_get", _wrap_Dwg_Object_ASSOCEDGEFILLETACTIONBODY_aab_version_get},
{"LibreDWGc::Dwg_Object_ASSOCEDGEFILLETACTIONBODY_pab_set", _wrap_Dwg_Object_ASSOCEDGEFILLETACTIONBODY_pab_set},
{"LibreDWGc::Dwg_Object_ASSOCEDGEFILLETACTIONBODY_pab_get", _wrap_Dwg_Object_ASSOCEDGEFILLETACTIONBODY_pab_get},
{"LibreDWGc::Dwg_Object_ASSOCEDGEFILLETACTIONBODY_sab_set", _wrap_Dwg_Object_ASSOCEDGEFILLETACTIONBODY_sab_set},
{"LibreDWGc::Dwg_Object_ASSOCEDGEFILLETACTIONBODY_sab_get", _wrap_Dwg_Object_ASSOCEDGEFILLETACTIONBODY_sab_get},
{"LibreDWGc::Dwg_Object_ASSOCEDGEFILLETACTIONBODY_pbsab_status_set", _wrap_Dwg_Object_ASSOCEDGEFILLETACTIONBODY_pbsab_status_set},
{"LibreDWGc::Dwg_Object_ASSOCEDGEFILLETACTIONBODY_pbsab_status_get", _wrap_Dwg_Object_ASSOCEDGEFILLETACTIONBODY_pbsab_status_get},
{"LibreDWGc::new_Dwg_Object_ASSOCEDGEFILLETACTIONBODY", _wrap_new_Dwg_Object_ASSOCEDGEFILLETACTIONBODY},
{"LibreDWGc::delete_Dwg_Object_ASSOCEDGEFILLETACTIONBODY", _wrap_delete_Dwg_Object_ASSOCEDGEFILLETACTIONBODY},
{"LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_parent_set", _wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_parent_get", _wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_aab_version_set", _wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_aab_version_set},
{"LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_aab_version_get", _wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_aab_version_get},
{"LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_pab_set", _wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_pab_set},
{"LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_pab_get", _wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_pab_get},
{"LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_sab_set", _wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_sab_set},
{"LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_sab_get", _wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_sab_get},
{"LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_pbsab_status_set", _wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_pbsab_status_set},
{"LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_pbsab_status_get", _wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_pbsab_status_get},
{"LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_class_version_set", _wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_class_version_get", _wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_class_version_get},
{"LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_b1_set", _wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_b1_set},
{"LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_b1_get", _wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_b1_get},
{"LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_b2_set", _wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_b2_set},
{"LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_b2_get", _wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_b2_get},
{"LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_distance_set", _wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_distance_set},
{"LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_distance_get", _wrap_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_distance_get},
{"LibreDWGc::new_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY", _wrap_new_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY},
{"LibreDWGc::delete_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY", _wrap_delete_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY},
{"LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_parent_set", _wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_parent_get", _wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_aab_version_set", _wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_aab_version_set},
{"LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_aab_version_get", _wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_aab_version_get},
{"LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_pab_set", _wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_pab_set},
{"LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_pab_get", _wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_pab_get},
{"LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_sab_set", _wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_sab_set},
{"LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_sab_get", _wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_sab_get},
{"LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_pbsab_status_set", _wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_pbsab_status_set},
{"LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_pbsab_status_get", _wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_pbsab_status_get},
{"LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_class_version_set", _wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_class_version_get", _wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_class_version_get},
{"LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b1_set", _wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b1_set},
{"LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b1_get", _wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b1_get},
{"LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b2_set", _wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b2_set},
{"LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b2_get", _wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b2_get},
{"LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b3_set", _wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b3_set},
{"LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b3_get", _wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b3_get},
{"LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b4_set", _wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b4_set},
{"LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b4_get", _wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b4_get},
{"LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b5_set", _wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b5_set},
{"LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b5_get", _wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b5_get},
{"LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_blend_options_set", _wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_blend_options_set},
{"LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_blend_options_get", _wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_blend_options_get},
{"LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_bs2_set", _wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_bs2_set},
{"LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_bs2_get", _wrap_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_bs2_get},
{"LibreDWGc::new_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY", _wrap_new_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY},
{"LibreDWGc::delete_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY", _wrap_delete_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY},
{"LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_parent_set", _wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_parent_get", _wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_aab_version_set", _wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_aab_version_set},
{"LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_aab_version_get", _wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_aab_version_get},
{"LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pab_set", _wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pab_set},
{"LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pab_get", _wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pab_get},
{"LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_sab_set", _wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_sab_set},
{"LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_sab_get", _wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_sab_get},
{"LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pbsab_status_set", _wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pbsab_status_set},
{"LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pbsab_status_get", _wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pbsab_status_get},
{"LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_class_version_set", _wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_class_version_get", _wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_class_version_get},
{"LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_status_set", _wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_status_set},
{"LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_status_get", _wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_status_get},
{"LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pt1_set", _wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pt1_set},
{"LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pt1_get", _wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pt1_get},
{"LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pt2_set", _wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pt2_set},
{"LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pt2_get", _wrap_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pt2_get},
{"LibreDWGc::new_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY", _wrap_new_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY},
{"LibreDWGc::delete_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY", _wrap_delete_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY},
{"LibreDWGc::Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_parent_set", _wrap_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_parent_get", _wrap_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_aab_version_set", _wrap_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_aab_version_set},
{"LibreDWGc::Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_aab_version_get", _wrap_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_aab_version_get},
{"LibreDWGc::Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_pab_set", _wrap_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_pab_set},
{"LibreDWGc::Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_pab_get", _wrap_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_pab_get},
{"LibreDWGc::Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_sab_set", _wrap_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_sab_set},
{"LibreDWGc::Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_sab_get", _wrap_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_sab_get},
{"LibreDWGc::Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_pbsab_status_set", _wrap_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_pbsab_status_set},
{"LibreDWGc::Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_pbsab_status_get", _wrap_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_pbsab_status_get},
{"LibreDWGc::Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_class_version_set", _wrap_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_class_version_get", _wrap_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_class_version_get},
{"LibreDWGc::new_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY", _wrap_new_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY},
{"LibreDWGc::delete_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY", _wrap_delete_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY},
{"LibreDWGc::Dwg_ASSOCACTIONBODY_action_parent_set", _wrap_Dwg_ASSOCACTIONBODY_action_parent_set},
{"LibreDWGc::Dwg_ASSOCACTIONBODY_action_parent_get", _wrap_Dwg_ASSOCACTIONBODY_action_parent_get},
{"LibreDWGc::Dwg_ASSOCACTIONBODY_action_depid_set", _wrap_Dwg_ASSOCACTIONBODY_action_depid_set},
{"LibreDWGc::Dwg_ASSOCACTIONBODY_action_depid_get", _wrap_Dwg_ASSOCACTIONBODY_action_depid_get},
{"LibreDWGc::Dwg_ASSOCACTIONBODY_action_dep_set", _wrap_Dwg_ASSOCACTIONBODY_action_dep_set},
{"LibreDWGc::Dwg_ASSOCACTIONBODY_action_dep_get", _wrap_Dwg_ASSOCACTIONBODY_action_dep_get},
{"LibreDWGc::new_Dwg_ASSOCACTIONBODY_action", _wrap_new_Dwg_ASSOCACTIONBODY_action},
{"LibreDWGc::delete_Dwg_ASSOCACTIONBODY_action", _wrap_delete_Dwg_ASSOCACTIONBODY_action},
{"LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_parent_set", _wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_parent_get", _wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_aaab_version_set", _wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_aaab_version_set},
{"LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_aaab_version_get", _wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_aaab_version_get},
{"LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_assoc_dep_set", _wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_assoc_dep_set},
{"LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_assoc_dep_get", _wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_assoc_dep_get},
{"LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_aab_version_set", _wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_aab_version_set},
{"LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_aab_version_get", _wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_aab_version_get},
{"LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_actionbody_set", _wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_actionbody_set},
{"LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_actionbody_get", _wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_actionbody_get},
{"LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_pab_set", _wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_pab_set},
{"LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_pab_get", _wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_pab_get},
{"LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_class_version_set", _wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_class_version_get", _wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_class_version_get},
{"LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_num_actions_set", _wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_num_actions_set},
{"LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_num_actions_get", _wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_num_actions_get},
{"LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_actions_set", _wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_actions_set},
{"LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_actions_get", _wrap_Dwg_Object_ASSOCMLEADERACTIONBODY_actions_get},
{"LibreDWGc::new_Dwg_Object_ASSOCMLEADERACTIONBODY", _wrap_new_Dwg_Object_ASSOCMLEADERACTIONBODY},
{"LibreDWGc::delete_Dwg_Object_ASSOCMLEADERACTIONBODY", _wrap_delete_Dwg_Object_ASSOCMLEADERACTIONBODY},
{"LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_parent_set", _wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_parent_get", _wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_aaab_version_set", _wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_aaab_version_set},
{"LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_aaab_version_get", _wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_aaab_version_get},
{"LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_assoc_dep_set", _wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_assoc_dep_set},
{"LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_assoc_dep_get", _wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_assoc_dep_get},
{"LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_aab_version_set", _wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_aab_version_set},
{"LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_aab_version_get", _wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_aab_version_get},
{"LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_actionbody_set", _wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_actionbody_set},
{"LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_actionbody_get", _wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_actionbody_get},
{"LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_pab_set", _wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_pab_set},
{"LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_pab_get", _wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_pab_get},
{"LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_class_version_set", _wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_class_version_get", _wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_class_version_get},
{"LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_r_node_set", _wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_r_node_set},
{"LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_r_node_get", _wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_r_node_get},
{"LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_d_node_set", _wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_d_node_set},
{"LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_d_node_get", _wrap_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_d_node_get},
{"LibreDWGc::new_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY", _wrap_new_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY},
{"LibreDWGc::delete_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY", _wrap_delete_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY},
{"LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_parent_set", _wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_parent_set},
{"LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_parent_get", _wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_parent_get},
{"LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_aaab_version_set", _wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_aaab_version_set},
{"LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_aaab_version_get", _wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_aaab_version_get},
{"LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_assoc_dep_set", _wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_assoc_dep_set},
{"LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_assoc_dep_get", _wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_assoc_dep_get},
{"LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_aab_version_set", _wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_aab_version_set},
{"LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_aab_version_get", _wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_aab_version_get},
{"LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_actionbody_set", _wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_actionbody_set},
{"LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_actionbody_get", _wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_actionbody_get},
{"LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_pab_set", _wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_pab_set},
{"LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_pab_get", _wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_pab_get},
{"LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_class_version_set", _wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_class_version_get", _wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_class_version_get},
{"LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_r_node_set", _wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_r_node_set},
{"LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_r_node_get", _wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_r_node_get},
{"LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_d_node_set", _wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_d_node_set},
{"LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_d_node_get", _wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_d_node_get},
{"LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_assocdep_set", _wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_assocdep_set},
{"LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_assocdep_get", _wrap_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_assocdep_get},
{"LibreDWGc::new_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY", _wrap_new_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY},
{"LibreDWGc::delete_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY", _wrap_delete_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY},
{"LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_parent_set", _wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_parent_get", _wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_aaab_version_set", _wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_aaab_version_set},
{"LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_aaab_version_get", _wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_aaab_version_get},
{"LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_assoc_dep_set", _wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_assoc_dep_set},
{"LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_assoc_dep_get", _wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_assoc_dep_get},
{"LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_aab_version_set", _wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_aab_version_set},
{"LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_aab_version_get", _wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_aab_version_get},
{"LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_actionbody_set", _wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_actionbody_set},
{"LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_actionbody_get", _wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_actionbody_get},
{"LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_pab_set", _wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_pab_set},
{"LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_pab_get", _wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_pab_get},
{"LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_class_version_set", _wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_class_version_get", _wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_class_version_get},
{"LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_r_node_set", _wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_r_node_set},
{"LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_r_node_get", _wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_r_node_get},
{"LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_d_node_set", _wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_d_node_set},
{"LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_d_node_get", _wrap_Dwg_Object_ASSOCORDINATEDIMACTIONBODY_d_node_get},
{"LibreDWGc::new_Dwg_Object_ASSOCORDINATEDIMACTIONBODY", _wrap_new_Dwg_Object_ASSOCORDINATEDIMACTIONBODY},
{"LibreDWGc::delete_Dwg_Object_ASSOCORDINATEDIMACTIONBODY", _wrap_delete_Dwg_Object_ASSOCORDINATEDIMACTIONBODY},
{"LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_parent_set", _wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_parent_get", _wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_aaab_version_set", _wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_aaab_version_set},
{"LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_aaab_version_get", _wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_aaab_version_get},
{"LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_assoc_dep_set", _wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_assoc_dep_set},
{"LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_assoc_dep_get", _wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_assoc_dep_get},
{"LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_aab_version_set", _wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_aab_version_set},
{"LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_aab_version_get", _wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_aab_version_get},
{"LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_actionbody_set", _wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_actionbody_set},
{"LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_actionbody_get", _wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_actionbody_get},
{"LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_pab_set", _wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_pab_set},
{"LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_pab_get", _wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_pab_get},
{"LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_class_version_set", _wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_class_version_get", _wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_class_version_get},
{"LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_r_node_set", _wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_r_node_set},
{"LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_r_node_get", _wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_r_node_get},
{"LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_d_node_set", _wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_d_node_set},
{"LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_d_node_get", _wrap_Dwg_Object_ASSOCROTATEDDIMACTIONBODY_d_node_get},
{"LibreDWGc::new_Dwg_Object_ASSOCROTATEDDIMACTIONBODY", _wrap_new_Dwg_Object_ASSOCROTATEDDIMACTIONBODY},
{"LibreDWGc::delete_Dwg_Object_ASSOCROTATEDDIMACTIONBODY", _wrap_delete_Dwg_Object_ASSOCROTATEDDIMACTIONBODY},
{"LibreDWGc::Dwg_Object_ASSOCDIMDEPENDENCYBODY_parent_set", _wrap_Dwg_Object_ASSOCDIMDEPENDENCYBODY_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCDIMDEPENDENCYBODY_parent_get", _wrap_Dwg_Object_ASSOCDIMDEPENDENCYBODY_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCDIMDEPENDENCYBODY_adb_version_set", _wrap_Dwg_Object_ASSOCDIMDEPENDENCYBODY_adb_version_set},
{"LibreDWGc::Dwg_Object_ASSOCDIMDEPENDENCYBODY_adb_version_get", _wrap_Dwg_Object_ASSOCDIMDEPENDENCYBODY_adb_version_get},
{"LibreDWGc::Dwg_Object_ASSOCDIMDEPENDENCYBODY_dimbase_version_set", _wrap_Dwg_Object_ASSOCDIMDEPENDENCYBODY_dimbase_version_set},
{"LibreDWGc::Dwg_Object_ASSOCDIMDEPENDENCYBODY_dimbase_version_get", _wrap_Dwg_Object_ASSOCDIMDEPENDENCYBODY_dimbase_version_get},
{"LibreDWGc::Dwg_Object_ASSOCDIMDEPENDENCYBODY_name_set", _wrap_Dwg_Object_ASSOCDIMDEPENDENCYBODY_name_set},
{"LibreDWGc::Dwg_Object_ASSOCDIMDEPENDENCYBODY_name_get", _wrap_Dwg_Object_ASSOCDIMDEPENDENCYBODY_name_get},
{"LibreDWGc::Dwg_Object_ASSOCDIMDEPENDENCYBODY_class_version_set", _wrap_Dwg_Object_ASSOCDIMDEPENDENCYBODY_class_version_set},
{"LibreDWGc::Dwg_Object_ASSOCDIMDEPENDENCYBODY_class_version_get", _wrap_Dwg_Object_ASSOCDIMDEPENDENCYBODY_class_version_get},
{"LibreDWGc::new_Dwg_Object_ASSOCDIMDEPENDENCYBODY", _wrap_new_Dwg_Object_ASSOCDIMDEPENDENCYBODY},
{"LibreDWGc::delete_Dwg_Object_ASSOCDIMDEPENDENCYBODY", _wrap_delete_Dwg_Object_ASSOCDIMDEPENDENCYBODY},
{"LibreDWGc::Dwg_Object_BLOCKPARAMDEPENDENCYBODY_parent_set", _wrap_Dwg_Object_BLOCKPARAMDEPENDENCYBODY_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKPARAMDEPENDENCYBODY_parent_get", _wrap_Dwg_Object_BLOCKPARAMDEPENDENCYBODY_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKPARAMDEPENDENCYBODY_adb_version_set", _wrap_Dwg_Object_BLOCKPARAMDEPENDENCYBODY_adb_version_set},
{"LibreDWGc::Dwg_Object_BLOCKPARAMDEPENDENCYBODY_adb_version_get", _wrap_Dwg_Object_BLOCKPARAMDEPENDENCYBODY_adb_version_get},
{"LibreDWGc::Dwg_Object_BLOCKPARAMDEPENDENCYBODY_dimbase_version_set", _wrap_Dwg_Object_BLOCKPARAMDEPENDENCYBODY_dimbase_version_set},
{"LibreDWGc::Dwg_Object_BLOCKPARAMDEPENDENCYBODY_dimbase_version_get", _wrap_Dwg_Object_BLOCKPARAMDEPENDENCYBODY_dimbase_version_get},
{"LibreDWGc::Dwg_Object_BLOCKPARAMDEPENDENCYBODY_name_set", _wrap_Dwg_Object_BLOCKPARAMDEPENDENCYBODY_name_set},
{"LibreDWGc::Dwg_Object_BLOCKPARAMDEPENDENCYBODY_name_get", _wrap_Dwg_Object_BLOCKPARAMDEPENDENCYBODY_name_get},
{"LibreDWGc::Dwg_Object_BLOCKPARAMDEPENDENCYBODY_class_version_set", _wrap_Dwg_Object_BLOCKPARAMDEPENDENCYBODY_class_version_set},
{"LibreDWGc::Dwg_Object_BLOCKPARAMDEPENDENCYBODY_class_version_get", _wrap_Dwg_Object_BLOCKPARAMDEPENDENCYBODY_class_version_get},
{"LibreDWGc::new_Dwg_Object_BLOCKPARAMDEPENDENCYBODY", _wrap_new_Dwg_Object_BLOCKPARAMDEPENDENCYBODY},
{"LibreDWGc::delete_Dwg_Object_BLOCKPARAMDEPENDENCYBODY", _wrap_delete_Dwg_Object_BLOCKPARAMDEPENDENCYBODY},
{"LibreDWGc::Dwg_ARRAYITEMLOCATOR_parent_set", _wrap_Dwg_ARRAYITEMLOCATOR_parent_set},
{"LibreDWGc::Dwg_ARRAYITEMLOCATOR_parent_get", _wrap_Dwg_ARRAYITEMLOCATOR_parent_get},
{"LibreDWGc::Dwg_ARRAYITEMLOCATOR_itemloc_set", _wrap_Dwg_ARRAYITEMLOCATOR_itemloc_set},
{"LibreDWGc::Dwg_ARRAYITEMLOCATOR_itemloc_get", _wrap_Dwg_ARRAYITEMLOCATOR_itemloc_get},
{"LibreDWGc::new_Dwg_ARRAYITEMLOCATOR", _wrap_new_Dwg_ARRAYITEMLOCATOR},
{"LibreDWGc::delete_Dwg_ARRAYITEMLOCATOR", _wrap_delete_Dwg_ARRAYITEMLOCATOR},
{"LibreDWGc::Dwg_Object_ASSOCARRAYACTIONBODY_parent_set", _wrap_Dwg_Object_ASSOCARRAYACTIONBODY_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCARRAYACTIONBODY_parent_get", _wrap_Dwg_Object_ASSOCARRAYACTIONBODY_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCARRAYACTIONBODY_aab_version_set", _wrap_Dwg_Object_ASSOCARRAYACTIONBODY_aab_version_set},
{"LibreDWGc::Dwg_Object_ASSOCARRAYACTIONBODY_aab_version_get", _wrap_Dwg_Object_ASSOCARRAYACTIONBODY_aab_version_get},
{"LibreDWGc::Dwg_Object_ASSOCARRAYACTIONBODY_pab_set", _wrap_Dwg_Object_ASSOCARRAYACTIONBODY_pab_set},
{"LibreDWGc::Dwg_Object_ASSOCARRAYACTIONBODY_pab_get", _wrap_Dwg_Object_ASSOCARRAYACTIONBODY_pab_get},
{"LibreDWGc::Dwg_Object_ASSOCARRAYACTIONBODY_aaab_version_set", _wrap_Dwg_Object_ASSOCARRAYACTIONBODY_aaab_version_set},
{"LibreDWGc::Dwg_Object_ASSOCARRAYACTIONBODY_aaab_version_get", _wrap_Dwg_Object_ASSOCARRAYACTIONBODY_aaab_version_get},
{"LibreDWGc::Dwg_Object_ASSOCARRAYACTIONBODY_paramblock_set", _wrap_Dwg_Object_ASSOCARRAYACTIONBODY_paramblock_set},
{"LibreDWGc::Dwg_Object_ASSOCARRAYACTIONBODY_paramblock_get", _wrap_Dwg_Object_ASSOCARRAYACTIONBODY_paramblock_get},
{"LibreDWGc::Dwg_Object_ASSOCARRAYACTIONBODY_transmatrix_set", _wrap_Dwg_Object_ASSOCARRAYACTIONBODY_transmatrix_set},
{"LibreDWGc::Dwg_Object_ASSOCARRAYACTIONBODY_transmatrix_get", _wrap_Dwg_Object_ASSOCARRAYACTIONBODY_transmatrix_get},
{"LibreDWGc::new_Dwg_Object_ASSOCARRAYACTIONBODY", _wrap_new_Dwg_Object_ASSOCARRAYACTIONBODY},
{"LibreDWGc::delete_Dwg_Object_ASSOCARRAYACTIONBODY", _wrap_delete_Dwg_Object_ASSOCARRAYACTIONBODY},
{"LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_parent_set", _wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_parent_set},
{"LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_parent_get", _wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_parent_get},
{"LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_aab_version_set", _wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_aab_version_set},
{"LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_aab_version_get", _wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_aab_version_get},
{"LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_pab_set", _wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_pab_set},
{"LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_pab_get", _wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_pab_get},
{"LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_aaab_version_set", _wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_aaab_version_set},
{"LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_aaab_version_get", _wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_aaab_version_get},
{"LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_paramblock_set", _wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_paramblock_set},
{"LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_paramblock_get", _wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_paramblock_get},
{"LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_transmatrix_set", _wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_transmatrix_set},
{"LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_transmatrix_get", _wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_transmatrix_get},
{"LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_status_set", _wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_status_set},
{"LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_status_get", _wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_status_get},
{"LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_num_items_set", _wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_num_items_set},
{"LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_num_items_get", _wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_num_items_get},
{"LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_items_set", _wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_items_set},
{"LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_items_get", _wrap_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_items_get},
{"LibreDWGc::new_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY", _wrap_new_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY},
{"LibreDWGc::delete_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY", _wrap_delete_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY},
{"LibreDWGc::Dwg_EvalExpr_parentid_set", _wrap_Dwg_EvalExpr_parentid_set},
{"LibreDWGc::Dwg_EvalExpr_parentid_get", _wrap_Dwg_EvalExpr_parentid_get},
{"LibreDWGc::Dwg_EvalExpr_major_set", _wrap_Dwg_EvalExpr_major_set},
{"LibreDWGc::Dwg_EvalExpr_major_get", _wrap_Dwg_EvalExpr_major_get},
{"LibreDWGc::Dwg_EvalExpr_minor_set", _wrap_Dwg_EvalExpr_minor_set},
{"LibreDWGc::Dwg_EvalExpr_minor_get", _wrap_Dwg_EvalExpr_minor_get},
{"LibreDWGc::Dwg_EvalExpr_value_code_set", _wrap_Dwg_EvalExpr_value_code_set},
{"LibreDWGc::Dwg_EvalExpr_value_code_get", _wrap_Dwg_EvalExpr_value_code_get},
{"LibreDWGc::Dwg_EvalExpr_value_get", _wrap_Dwg_EvalExpr_value_get},
{"LibreDWGc::Dwg_EvalExpr_nodeid_set", _wrap_Dwg_EvalExpr_nodeid_set},
{"LibreDWGc::Dwg_EvalExpr_nodeid_get", _wrap_Dwg_EvalExpr_nodeid_get},
{"LibreDWGc::new_Dwg_EvalExpr", _wrap_new_Dwg_EvalExpr},
{"LibreDWGc::delete_Dwg_EvalExpr", _wrap_delete_Dwg_EvalExpr},
{"LibreDWGc::_dwg_EvalExpr_value_num40_set", _wrap__dwg_EvalExpr_value_num40_set},
{"LibreDWGc::_dwg_EvalExpr_value_num40_get", _wrap__dwg_EvalExpr_value_num40_get},
{"LibreDWGc::_dwg_EvalExpr_value_pt2d_set", _wrap__dwg_EvalExpr_value_pt2d_set},
{"LibreDWGc::_dwg_EvalExpr_value_pt2d_get", _wrap__dwg_EvalExpr_value_pt2d_get},
{"LibreDWGc::_dwg_EvalExpr_value_pt3d_set", _wrap__dwg_EvalExpr_value_pt3d_set},
{"LibreDWGc::_dwg_EvalExpr_value_pt3d_get", _wrap__dwg_EvalExpr_value_pt3d_get},
{"LibreDWGc::_dwg_EvalExpr_value_text1_set", _wrap__dwg_EvalExpr_value_text1_set},
{"LibreDWGc::_dwg_EvalExpr_value_text1_get", _wrap__dwg_EvalExpr_value_text1_get},
{"LibreDWGc::_dwg_EvalExpr_value_long90_set", _wrap__dwg_EvalExpr_value_long90_set},
{"LibreDWGc::_dwg_EvalExpr_value_long90_get", _wrap__dwg_EvalExpr_value_long90_get},
{"LibreDWGc::_dwg_EvalExpr_value_handle91_set", _wrap__dwg_EvalExpr_value_handle91_set},
{"LibreDWGc::_dwg_EvalExpr_value_handle91_get", _wrap__dwg_EvalExpr_value_handle91_get},
{"LibreDWGc::_dwg_EvalExpr_value_short70_set", _wrap__dwg_EvalExpr_value_short70_set},
{"LibreDWGc::_dwg_EvalExpr_value_short70_get", _wrap__dwg_EvalExpr_value_short70_get},
{"LibreDWGc::new__dwg_EvalExpr_value", _wrap_new__dwg_EvalExpr_value},
{"LibreDWGc::delete__dwg_EvalExpr_value", _wrap_delete__dwg_EvalExpr_value},
{"LibreDWGc::Dwg_ACSH_SubentMaterial_major_set", _wrap_Dwg_ACSH_SubentMaterial_major_set},
{"LibreDWGc::Dwg_ACSH_SubentMaterial_major_get", _wrap_Dwg_ACSH_SubentMaterial_major_get},
{"LibreDWGc::Dwg_ACSH_SubentMaterial_minor_set", _wrap_Dwg_ACSH_SubentMaterial_minor_set},
{"LibreDWGc::Dwg_ACSH_SubentMaterial_minor_get", _wrap_Dwg_ACSH_SubentMaterial_minor_get},
{"LibreDWGc::Dwg_ACSH_SubentMaterial_reflectance_set", _wrap_Dwg_ACSH_SubentMaterial_reflectance_set},
{"LibreDWGc::Dwg_ACSH_SubentMaterial_reflectance_get", _wrap_Dwg_ACSH_SubentMaterial_reflectance_get},
{"LibreDWGc::Dwg_ACSH_SubentMaterial_displacement_set", _wrap_Dwg_ACSH_SubentMaterial_displacement_set},
{"LibreDWGc::Dwg_ACSH_SubentMaterial_displacement_get", _wrap_Dwg_ACSH_SubentMaterial_displacement_get},
{"LibreDWGc::new_Dwg_ACSH_SubentMaterial", _wrap_new_Dwg_ACSH_SubentMaterial},
{"LibreDWGc::delete_Dwg_ACSH_SubentMaterial", _wrap_delete_Dwg_ACSH_SubentMaterial},
{"LibreDWGc::Dwg_ACSH_SubentColor_major_set", _wrap_Dwg_ACSH_SubentColor_major_set},
{"LibreDWGc::Dwg_ACSH_SubentColor_major_get", _wrap_Dwg_ACSH_SubentColor_major_get},
{"LibreDWGc::Dwg_ACSH_SubentColor_minor_set", _wrap_Dwg_ACSH_SubentColor_minor_set},
{"LibreDWGc::Dwg_ACSH_SubentColor_minor_get", _wrap_Dwg_ACSH_SubentColor_minor_get},
{"LibreDWGc::Dwg_ACSH_SubentColor_transparency_set", _wrap_Dwg_ACSH_SubentColor_transparency_set},
{"LibreDWGc::Dwg_ACSH_SubentColor_transparency_get", _wrap_Dwg_ACSH_SubentColor_transparency_get},
{"LibreDWGc::Dwg_ACSH_SubentColor_bl93_set", _wrap_Dwg_ACSH_SubentColor_bl93_set},
{"LibreDWGc::Dwg_ACSH_SubentColor_bl93_get", _wrap_Dwg_ACSH_SubentColor_bl93_get},
{"LibreDWGc::Dwg_ACSH_SubentColor_is_face_variable_set", _wrap_Dwg_ACSH_SubentColor_is_face_variable_set},
{"LibreDWGc::Dwg_ACSH_SubentColor_is_face_variable_get", _wrap_Dwg_ACSH_SubentColor_is_face_variable_get},
{"LibreDWGc::new_Dwg_ACSH_SubentColor", _wrap_new_Dwg_ACSH_SubentColor},
{"LibreDWGc::delete_Dwg_ACSH_SubentColor", _wrap_delete_Dwg_ACSH_SubentColor},
{"LibreDWGc::Dwg_ACSH_HistoryNode_major_set", _wrap_Dwg_ACSH_HistoryNode_major_set},
{"LibreDWGc::Dwg_ACSH_HistoryNode_major_get", _wrap_Dwg_ACSH_HistoryNode_major_get},
{"LibreDWGc::Dwg_ACSH_HistoryNode_minor_set", _wrap_Dwg_ACSH_HistoryNode_minor_set},
{"LibreDWGc::Dwg_ACSH_HistoryNode_minor_get", _wrap_Dwg_ACSH_HistoryNode_minor_get},
{"LibreDWGc::Dwg_ACSH_HistoryNode_trans_set", _wrap_Dwg_ACSH_HistoryNode_trans_set},
{"LibreDWGc::Dwg_ACSH_HistoryNode_trans_get", _wrap_Dwg_ACSH_HistoryNode_trans_get},
{"LibreDWGc::Dwg_ACSH_HistoryNode_color_set", _wrap_Dwg_ACSH_HistoryNode_color_set},
{"LibreDWGc::Dwg_ACSH_HistoryNode_color_get", _wrap_Dwg_ACSH_HistoryNode_color_get},
{"LibreDWGc::Dwg_ACSH_HistoryNode_step_id_set", _wrap_Dwg_ACSH_HistoryNode_step_id_set},
{"LibreDWGc::Dwg_ACSH_HistoryNode_step_id_get", _wrap_Dwg_ACSH_HistoryNode_step_id_get},
{"LibreDWGc::Dwg_ACSH_HistoryNode_material_set", _wrap_Dwg_ACSH_HistoryNode_material_set},
{"LibreDWGc::Dwg_ACSH_HistoryNode_material_get", _wrap_Dwg_ACSH_HistoryNode_material_get},
{"LibreDWGc::new_Dwg_ACSH_HistoryNode", _wrap_new_Dwg_ACSH_HistoryNode},
{"LibreDWGc::delete_Dwg_ACSH_HistoryNode", _wrap_delete_Dwg_ACSH_HistoryNode},
{"LibreDWGc::Dwg_Object_ACSH_HISTORY_CLASS_parent_set", _wrap_Dwg_Object_ACSH_HISTORY_CLASS_parent_set},
{"LibreDWGc::Dwg_Object_ACSH_HISTORY_CLASS_parent_get", _wrap_Dwg_Object_ACSH_HISTORY_CLASS_parent_get},
{"LibreDWGc::Dwg_Object_ACSH_HISTORY_CLASS_major_set", _wrap_Dwg_Object_ACSH_HISTORY_CLASS_major_set},
{"LibreDWGc::Dwg_Object_ACSH_HISTORY_CLASS_major_get", _wrap_Dwg_Object_ACSH_HISTORY_CLASS_major_get},
{"LibreDWGc::Dwg_Object_ACSH_HISTORY_CLASS_minor_set", _wrap_Dwg_Object_ACSH_HISTORY_CLASS_minor_set},
{"LibreDWGc::Dwg_Object_ACSH_HISTORY_CLASS_minor_get", _wrap_Dwg_Object_ACSH_HISTORY_CLASS_minor_get},
{"LibreDWGc::Dwg_Object_ACSH_HISTORY_CLASS_owner_set", _wrap_Dwg_Object_ACSH_HISTORY_CLASS_owner_set},
{"LibreDWGc::Dwg_Object_ACSH_HISTORY_CLASS_owner_get", _wrap_Dwg_Object_ACSH_HISTORY_CLASS_owner_get},
{"LibreDWGc::Dwg_Object_ACSH_HISTORY_CLASS_h_nodeid_set", _wrap_Dwg_Object_ACSH_HISTORY_CLASS_h_nodeid_set},
{"LibreDWGc::Dwg_Object_ACSH_HISTORY_CLASS_h_nodeid_get", _wrap_Dwg_Object_ACSH_HISTORY_CLASS_h_nodeid_get},
{"LibreDWGc::Dwg_Object_ACSH_HISTORY_CLASS_show_history_set", _wrap_Dwg_Object_ACSH_HISTORY_CLASS_show_history_set},
{"LibreDWGc::Dwg_Object_ACSH_HISTORY_CLASS_show_history_get", _wrap_Dwg_Object_ACSH_HISTORY_CLASS_show_history_get},
{"LibreDWGc::Dwg_Object_ACSH_HISTORY_CLASS_record_history_set", _wrap_Dwg_Object_ACSH_HISTORY_CLASS_record_history_set},
{"LibreDWGc::Dwg_Object_ACSH_HISTORY_CLASS_record_history_get", _wrap_Dwg_Object_ACSH_HISTORY_CLASS_record_history_get},
{"LibreDWGc::new_Dwg_Object_ACSH_HISTORY_CLASS", _wrap_new_Dwg_Object_ACSH_HISTORY_CLASS},
{"LibreDWGc::delete_Dwg_Object_ACSH_HISTORY_CLASS", _wrap_delete_Dwg_Object_ACSH_HISTORY_CLASS},
{"LibreDWGc::Dwg_Object_ACSH_BOX_CLASS_parent_set", _wrap_Dwg_Object_ACSH_BOX_CLASS_parent_set},
{"LibreDWGc::Dwg_Object_ACSH_BOX_CLASS_parent_get", _wrap_Dwg_Object_ACSH_BOX_CLASS_parent_get},
{"LibreDWGc::Dwg_Object_ACSH_BOX_CLASS_evalexpr_set", _wrap_Dwg_Object_ACSH_BOX_CLASS_evalexpr_set},
{"LibreDWGc::Dwg_Object_ACSH_BOX_CLASS_evalexpr_get", _wrap_Dwg_Object_ACSH_BOX_CLASS_evalexpr_get},
{"LibreDWGc::Dwg_Object_ACSH_BOX_CLASS_history_node_set", _wrap_Dwg_Object_ACSH_BOX_CLASS_history_node_set},
{"LibreDWGc::Dwg_Object_ACSH_BOX_CLASS_history_node_get", _wrap_Dwg_Object_ACSH_BOX_CLASS_history_node_get},
{"LibreDWGc::Dwg_Object_ACSH_BOX_CLASS_major_set", _wrap_Dwg_Object_ACSH_BOX_CLASS_major_set},
{"LibreDWGc::Dwg_Object_ACSH_BOX_CLASS_major_get", _wrap_Dwg_Object_ACSH_BOX_CLASS_major_get},
{"LibreDWGc::Dwg_Object_ACSH_BOX_CLASS_minor_set", _wrap_Dwg_Object_ACSH_BOX_CLASS_minor_set},
{"LibreDWGc::Dwg_Object_ACSH_BOX_CLASS_minor_get", _wrap_Dwg_Object_ACSH_BOX_CLASS_minor_get},
{"LibreDWGc::Dwg_Object_ACSH_BOX_CLASS_length_set", _wrap_Dwg_Object_ACSH_BOX_CLASS_length_set},
{"LibreDWGc::Dwg_Object_ACSH_BOX_CLASS_length_get", _wrap_Dwg_Object_ACSH_BOX_CLASS_length_get},
{"LibreDWGc::Dwg_Object_ACSH_BOX_CLASS_width_set", _wrap_Dwg_Object_ACSH_BOX_CLASS_width_set},
{"LibreDWGc::Dwg_Object_ACSH_BOX_CLASS_width_get", _wrap_Dwg_Object_ACSH_BOX_CLASS_width_get},
{"LibreDWGc::Dwg_Object_ACSH_BOX_CLASS_height_set", _wrap_Dwg_Object_ACSH_BOX_CLASS_height_set},
{"LibreDWGc::Dwg_Object_ACSH_BOX_CLASS_height_get", _wrap_Dwg_Object_ACSH_BOX_CLASS_height_get},
{"LibreDWGc::new_Dwg_Object_ACSH_BOX_CLASS", _wrap_new_Dwg_Object_ACSH_BOX_CLASS},
{"LibreDWGc::delete_Dwg_Object_ACSH_BOX_CLASS", _wrap_delete_Dwg_Object_ACSH_BOX_CLASS},
{"LibreDWGc::Dwg_Object_ACSH_WEDGE_CLASS_parent_set", _wrap_Dwg_Object_ACSH_WEDGE_CLASS_parent_set},
{"LibreDWGc::Dwg_Object_ACSH_WEDGE_CLASS_parent_get", _wrap_Dwg_Object_ACSH_WEDGE_CLASS_parent_get},
{"LibreDWGc::Dwg_Object_ACSH_WEDGE_CLASS_evalexpr_set", _wrap_Dwg_Object_ACSH_WEDGE_CLASS_evalexpr_set},
{"LibreDWGc::Dwg_Object_ACSH_WEDGE_CLASS_evalexpr_get", _wrap_Dwg_Object_ACSH_WEDGE_CLASS_evalexpr_get},
{"LibreDWGc::Dwg_Object_ACSH_WEDGE_CLASS_history_node_set", _wrap_Dwg_Object_ACSH_WEDGE_CLASS_history_node_set},
{"LibreDWGc::Dwg_Object_ACSH_WEDGE_CLASS_history_node_get", _wrap_Dwg_Object_ACSH_WEDGE_CLASS_history_node_get},
{"LibreDWGc::Dwg_Object_ACSH_WEDGE_CLASS_major_set", _wrap_Dwg_Object_ACSH_WEDGE_CLASS_major_set},
{"LibreDWGc::Dwg_Object_ACSH_WEDGE_CLASS_major_get", _wrap_Dwg_Object_ACSH_WEDGE_CLASS_major_get},
{"LibreDWGc::Dwg_Object_ACSH_WEDGE_CLASS_minor_set", _wrap_Dwg_Object_ACSH_WEDGE_CLASS_minor_set},
{"LibreDWGc::Dwg_Object_ACSH_WEDGE_CLASS_minor_get", _wrap_Dwg_Object_ACSH_WEDGE_CLASS_minor_get},
{"LibreDWGc::Dwg_Object_ACSH_WEDGE_CLASS_length_set", _wrap_Dwg_Object_ACSH_WEDGE_CLASS_length_set},
{"LibreDWGc::Dwg_Object_ACSH_WEDGE_CLASS_length_get", _wrap_Dwg_Object_ACSH_WEDGE_CLASS_length_get},
{"LibreDWGc::Dwg_Object_ACSH_WEDGE_CLASS_width_set", _wrap_Dwg_Object_ACSH_WEDGE_CLASS_width_set},
{"LibreDWGc::Dwg_Object_ACSH_WEDGE_CLASS_width_get", _wrap_Dwg_Object_ACSH_WEDGE_CLASS_width_get},
{"LibreDWGc::Dwg_Object_ACSH_WEDGE_CLASS_height_set", _wrap_Dwg_Object_ACSH_WEDGE_CLASS_height_set},
{"LibreDWGc::Dwg_Object_ACSH_WEDGE_CLASS_height_get", _wrap_Dwg_Object_ACSH_WEDGE_CLASS_height_get},
{"LibreDWGc::new_Dwg_Object_ACSH_WEDGE_CLASS", _wrap_new_Dwg_Object_ACSH_WEDGE_CLASS},
{"LibreDWGc::delete_Dwg_Object_ACSH_WEDGE_CLASS", _wrap_delete_Dwg_Object_ACSH_WEDGE_CLASS},
{"LibreDWGc::Dwg_Object_ACSH_BOOLEAN_CLASS_parent_set", _wrap_Dwg_Object_ACSH_BOOLEAN_CLASS_parent_set},
{"LibreDWGc::Dwg_Object_ACSH_BOOLEAN_CLASS_parent_get", _wrap_Dwg_Object_ACSH_BOOLEAN_CLASS_parent_get},
{"LibreDWGc::Dwg_Object_ACSH_BOOLEAN_CLASS_evalexpr_set", _wrap_Dwg_Object_ACSH_BOOLEAN_CLASS_evalexpr_set},
{"LibreDWGc::Dwg_Object_ACSH_BOOLEAN_CLASS_evalexpr_get", _wrap_Dwg_Object_ACSH_BOOLEAN_CLASS_evalexpr_get},
{"LibreDWGc::Dwg_Object_ACSH_BOOLEAN_CLASS_history_node_set", _wrap_Dwg_Object_ACSH_BOOLEAN_CLASS_history_node_set},
{"LibreDWGc::Dwg_Object_ACSH_BOOLEAN_CLASS_history_node_get", _wrap_Dwg_Object_ACSH_BOOLEAN_CLASS_history_node_get},
{"LibreDWGc::Dwg_Object_ACSH_BOOLEAN_CLASS_major_set", _wrap_Dwg_Object_ACSH_BOOLEAN_CLASS_major_set},
{"LibreDWGc::Dwg_Object_ACSH_BOOLEAN_CLASS_major_get", _wrap_Dwg_Object_ACSH_BOOLEAN_CLASS_major_get},
{"LibreDWGc::Dwg_Object_ACSH_BOOLEAN_CLASS_minor_set", _wrap_Dwg_Object_ACSH_BOOLEAN_CLASS_minor_set},
{"LibreDWGc::Dwg_Object_ACSH_BOOLEAN_CLASS_minor_get", _wrap_Dwg_Object_ACSH_BOOLEAN_CLASS_minor_get},
{"LibreDWGc::Dwg_Object_ACSH_BOOLEAN_CLASS_operation_set", _wrap_Dwg_Object_ACSH_BOOLEAN_CLASS_operation_set},
{"LibreDWGc::Dwg_Object_ACSH_BOOLEAN_CLASS_operation_get", _wrap_Dwg_Object_ACSH_BOOLEAN_CLASS_operation_get},
{"LibreDWGc::Dwg_Object_ACSH_BOOLEAN_CLASS_operand1_set", _wrap_Dwg_Object_ACSH_BOOLEAN_CLASS_operand1_set},
{"LibreDWGc::Dwg_Object_ACSH_BOOLEAN_CLASS_operand1_get", _wrap_Dwg_Object_ACSH_BOOLEAN_CLASS_operand1_get},
{"LibreDWGc::Dwg_Object_ACSH_BOOLEAN_CLASS_operand2_set", _wrap_Dwg_Object_ACSH_BOOLEAN_CLASS_operand2_set},
{"LibreDWGc::Dwg_Object_ACSH_BOOLEAN_CLASS_operand2_get", _wrap_Dwg_Object_ACSH_BOOLEAN_CLASS_operand2_get},
{"LibreDWGc::new_Dwg_Object_ACSH_BOOLEAN_CLASS", _wrap_new_Dwg_Object_ACSH_BOOLEAN_CLASS},
{"LibreDWGc::delete_Dwg_Object_ACSH_BOOLEAN_CLASS", _wrap_delete_Dwg_Object_ACSH_BOOLEAN_CLASS},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_parent_set", _wrap_Dwg_Object_ACSH_BREP_CLASS_parent_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_parent_get", _wrap_Dwg_Object_ACSH_BREP_CLASS_parent_get},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_acis_empty_set", _wrap_Dwg_Object_ACSH_BREP_CLASS_acis_empty_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_acis_empty_get", _wrap_Dwg_Object_ACSH_BREP_CLASS_acis_empty_get},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_unknown_set", _wrap_Dwg_Object_ACSH_BREP_CLASS_unknown_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_unknown_get", _wrap_Dwg_Object_ACSH_BREP_CLASS_unknown_get},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_version_set", _wrap_Dwg_Object_ACSH_BREP_CLASS_version_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_version_get", _wrap_Dwg_Object_ACSH_BREP_CLASS_version_get},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_num_blocks_set", _wrap_Dwg_Object_ACSH_BREP_CLASS_num_blocks_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_num_blocks_get", _wrap_Dwg_Object_ACSH_BREP_CLASS_num_blocks_get},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_block_size_set", _wrap_Dwg_Object_ACSH_BREP_CLASS_block_size_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_block_size_get", _wrap_Dwg_Object_ACSH_BREP_CLASS_block_size_get},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_encr_sat_data_set", _wrap_Dwg_Object_ACSH_BREP_CLASS_encr_sat_data_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_encr_sat_data_get", _wrap_Dwg_Object_ACSH_BREP_CLASS_encr_sat_data_get},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_sab_size_set", _wrap_Dwg_Object_ACSH_BREP_CLASS_sab_size_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_sab_size_get", _wrap_Dwg_Object_ACSH_BREP_CLASS_sab_size_get},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_acis_data_set", _wrap_Dwg_Object_ACSH_BREP_CLASS_acis_data_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_acis_data_get", _wrap_Dwg_Object_ACSH_BREP_CLASS_acis_data_get},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_wireframe_data_present_set", _wrap_Dwg_Object_ACSH_BREP_CLASS_wireframe_data_present_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_wireframe_data_present_get", _wrap_Dwg_Object_ACSH_BREP_CLASS_wireframe_data_present_get},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_point_present_set", _wrap_Dwg_Object_ACSH_BREP_CLASS_point_present_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_point_present_get", _wrap_Dwg_Object_ACSH_BREP_CLASS_point_present_get},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_point_set", _wrap_Dwg_Object_ACSH_BREP_CLASS_point_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_point_get", _wrap_Dwg_Object_ACSH_BREP_CLASS_point_get},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_isolines_set", _wrap_Dwg_Object_ACSH_BREP_CLASS_isolines_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_isolines_get", _wrap_Dwg_Object_ACSH_BREP_CLASS_isolines_get},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_isoline_present_set", _wrap_Dwg_Object_ACSH_BREP_CLASS_isoline_present_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_isoline_present_get", _wrap_Dwg_Object_ACSH_BREP_CLASS_isoline_present_get},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_num_wires_set", _wrap_Dwg_Object_ACSH_BREP_CLASS_num_wires_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_num_wires_get", _wrap_Dwg_Object_ACSH_BREP_CLASS_num_wires_get},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_wires_set", _wrap_Dwg_Object_ACSH_BREP_CLASS_wires_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_wires_get", _wrap_Dwg_Object_ACSH_BREP_CLASS_wires_get},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_num_silhouettes_set", _wrap_Dwg_Object_ACSH_BREP_CLASS_num_silhouettes_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_num_silhouettes_get", _wrap_Dwg_Object_ACSH_BREP_CLASS_num_silhouettes_get},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_silhouettes_set", _wrap_Dwg_Object_ACSH_BREP_CLASS_silhouettes_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_silhouettes_get", _wrap_Dwg_Object_ACSH_BREP_CLASS_silhouettes_get},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS__dxf_sab_converted_set", _wrap_Dwg_Object_ACSH_BREP_CLASS__dxf_sab_converted_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS__dxf_sab_converted_get", _wrap_Dwg_Object_ACSH_BREP_CLASS__dxf_sab_converted_get},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_acis_empty2_set", _wrap_Dwg_Object_ACSH_BREP_CLASS_acis_empty2_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_acis_empty2_get", _wrap_Dwg_Object_ACSH_BREP_CLASS_acis_empty2_get},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_extra_acis_data_set", _wrap_Dwg_Object_ACSH_BREP_CLASS_extra_acis_data_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_extra_acis_data_get", _wrap_Dwg_Object_ACSH_BREP_CLASS_extra_acis_data_get},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_num_materials_set", _wrap_Dwg_Object_ACSH_BREP_CLASS_num_materials_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_num_materials_get", _wrap_Dwg_Object_ACSH_BREP_CLASS_num_materials_get},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_materials_set", _wrap_Dwg_Object_ACSH_BREP_CLASS_materials_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_materials_get", _wrap_Dwg_Object_ACSH_BREP_CLASS_materials_get},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_revision_guid_set", _wrap_Dwg_Object_ACSH_BREP_CLASS_revision_guid_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_revision_guid_get", _wrap_Dwg_Object_ACSH_BREP_CLASS_revision_guid_get},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_revision_major_set", _wrap_Dwg_Object_ACSH_BREP_CLASS_revision_major_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_revision_major_get", _wrap_Dwg_Object_ACSH_BREP_CLASS_revision_major_get},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_revision_minor1_set", _wrap_Dwg_Object_ACSH_BREP_CLASS_revision_minor1_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_revision_minor1_get", _wrap_Dwg_Object_ACSH_BREP_CLASS_revision_minor1_get},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_revision_minor2_set", _wrap_Dwg_Object_ACSH_BREP_CLASS_revision_minor2_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_revision_minor2_get", _wrap_Dwg_Object_ACSH_BREP_CLASS_revision_minor2_get},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_revision_bytes_set", _wrap_Dwg_Object_ACSH_BREP_CLASS_revision_bytes_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_revision_bytes_get", _wrap_Dwg_Object_ACSH_BREP_CLASS_revision_bytes_get},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_end_marker_set", _wrap_Dwg_Object_ACSH_BREP_CLASS_end_marker_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_end_marker_get", _wrap_Dwg_Object_ACSH_BREP_CLASS_end_marker_get},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_history_id_set", _wrap_Dwg_Object_ACSH_BREP_CLASS_history_id_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_history_id_get", _wrap_Dwg_Object_ACSH_BREP_CLASS_history_id_get},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_has_revision_guid_set", _wrap_Dwg_Object_ACSH_BREP_CLASS_has_revision_guid_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_has_revision_guid_get", _wrap_Dwg_Object_ACSH_BREP_CLASS_has_revision_guid_get},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_acis_empty_bit_set", _wrap_Dwg_Object_ACSH_BREP_CLASS_acis_empty_bit_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_acis_empty_bit_get", _wrap_Dwg_Object_ACSH_BREP_CLASS_acis_empty_bit_get},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_evalexpr_set", _wrap_Dwg_Object_ACSH_BREP_CLASS_evalexpr_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_evalexpr_get", _wrap_Dwg_Object_ACSH_BREP_CLASS_evalexpr_get},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_history_node_set", _wrap_Dwg_Object_ACSH_BREP_CLASS_history_node_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_history_node_get", _wrap_Dwg_Object_ACSH_BREP_CLASS_history_node_get},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_major_set", _wrap_Dwg_Object_ACSH_BREP_CLASS_major_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_major_get", _wrap_Dwg_Object_ACSH_BREP_CLASS_major_get},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_minor_set", _wrap_Dwg_Object_ACSH_BREP_CLASS_minor_set},
{"LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_minor_get", _wrap_Dwg_Object_ACSH_BREP_CLASS_minor_get},
{"LibreDWGc::new_Dwg_Object_ACSH_BREP_CLASS", _wrap_new_Dwg_Object_ACSH_BREP_CLASS},
{"LibreDWGc::delete_Dwg_Object_ACSH_BREP_CLASS", _wrap_delete_Dwg_Object_ACSH_BREP_CLASS},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_parent_set", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_parent_set},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_parent_get", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_parent_get},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_evalexpr_set", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_evalexpr_set},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_evalexpr_get", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_evalexpr_get},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_history_node_set", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_history_node_set},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_history_node_get", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_history_node_get},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_major_set", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_major_set},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_major_get", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_major_get},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_minor_set", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_minor_set},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_minor_get", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_minor_get},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_direction_set", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_direction_set},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_direction_get", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_direction_get},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_bl92_set", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_bl92_set},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_bl92_get", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_bl92_get},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_shsw_text_size_set", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_shsw_text_size_set},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_shsw_text_size_get", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_shsw_text_size_get},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_shsw_text_set", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_shsw_text_set},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_shsw_text_get", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_shsw_text_get},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_shsw_bl93_set", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_shsw_bl93_set},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_shsw_bl93_get", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_shsw_bl93_get},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_shsw_text2_size_set", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_shsw_text2_size_set},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_shsw_text2_size_get", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_shsw_text2_size_get},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_shsw_text2_set", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_shsw_text2_set},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_shsw_text2_get", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_shsw_text2_get},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_draft_angle_set", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_draft_angle_set},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_draft_angle_get", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_draft_angle_get},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_start_draft_dist_set", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_start_draft_dist_set},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_start_draft_dist_get", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_start_draft_dist_get},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_end_draft_dist_set", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_end_draft_dist_set},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_end_draft_dist_get", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_end_draft_dist_get},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_scale_factor_set", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_scale_factor_set},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_scale_factor_get", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_scale_factor_get},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_twist_angle_set", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_twist_angle_set},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_twist_angle_get", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_twist_angle_get},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_align_angle_set", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_align_angle_set},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_align_angle_get", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_align_angle_get},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_sweepentity_transform_set", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_sweepentity_transform_set},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_sweepentity_transform_get", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_sweepentity_transform_get},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_pathentity_transform_set", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_pathentity_transform_set},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_pathentity_transform_get", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_pathentity_transform_get},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_align_option_set", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_align_option_set},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_align_option_get", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_align_option_get},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_miter_option_set", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_miter_option_set},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_miter_option_get", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_miter_option_get},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_has_align_start_set", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_has_align_start_set},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_has_align_start_get", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_has_align_start_get},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_bank_set", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_bank_set},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_bank_get", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_bank_get},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_check_intersections_set", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_check_intersections_set},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_check_intersections_get", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_check_intersections_get},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_shsw_b294_set", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_shsw_b294_set},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_shsw_b294_get", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_shsw_b294_get},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_shsw_b295_set", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_shsw_b295_set},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_shsw_b295_get", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_shsw_b295_get},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_shsw_b296_set", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_shsw_b296_set},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_shsw_b296_get", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_shsw_b296_get},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_pt2_set", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_pt2_set},
{"LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_pt2_get", _wrap_Dwg_Object_ACSH_SWEEP_CLASS_pt2_get},
{"LibreDWGc::new_Dwg_Object_ACSH_SWEEP_CLASS", _wrap_new_Dwg_Object_ACSH_SWEEP_CLASS},
{"LibreDWGc::delete_Dwg_Object_ACSH_SWEEP_CLASS", _wrap_delete_Dwg_Object_ACSH_SWEEP_CLASS},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_parent_set", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_parent_set},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_parent_get", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_parent_get},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_evalexpr_set", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_evalexpr_set},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_evalexpr_get", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_evalexpr_get},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_history_node_set", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_history_node_set},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_history_node_get", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_history_node_get},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_major_set", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_major_set},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_major_get", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_major_get},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_minor_set", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_minor_set},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_minor_get", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_minor_get},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_direction_set", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_direction_set},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_direction_get", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_direction_get},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_bl92_set", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_bl92_set},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_bl92_get", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_bl92_get},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text_size_set", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text_size_set},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text_size_get", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text_size_get},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text_set", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text_set},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text_get", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text_get},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_bl93_set", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_bl93_set},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_bl93_get", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_bl93_get},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text2_size_set", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text2_size_set},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text2_size_get", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text2_size_get},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text2_set", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text2_set},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text2_get", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text2_get},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_draft_angle_set", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_draft_angle_set},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_draft_angle_get", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_draft_angle_get},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_start_draft_dist_set", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_start_draft_dist_set},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_start_draft_dist_get", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_start_draft_dist_get},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_end_draft_dist_set", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_end_draft_dist_set},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_end_draft_dist_get", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_end_draft_dist_get},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_scale_factor_set", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_scale_factor_set},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_scale_factor_get", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_scale_factor_get},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_twist_angle_set", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_twist_angle_set},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_twist_angle_get", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_twist_angle_get},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_align_angle_set", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_align_angle_set},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_align_angle_get", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_align_angle_get},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_sweepentity_transform_set", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_sweepentity_transform_set},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_sweepentity_transform_get", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_sweepentity_transform_get},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_pathentity_transform_set", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_pathentity_transform_set},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_pathentity_transform_get", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_pathentity_transform_get},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_align_option_set", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_align_option_set},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_align_option_get", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_align_option_get},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_miter_option_set", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_miter_option_set},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_miter_option_get", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_miter_option_get},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_has_align_start_set", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_has_align_start_set},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_has_align_start_get", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_has_align_start_get},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_bank_set", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_bank_set},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_bank_get", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_bank_get},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_check_intersections_set", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_check_intersections_set},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_check_intersections_get", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_check_intersections_get},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b294_set", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b294_set},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b294_get", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b294_get},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b295_set", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b295_set},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b295_get", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b295_get},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b296_set", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b296_set},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b296_get", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b296_get},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_pt2_set", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_pt2_set},
{"LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_pt2_get", _wrap_Dwg_Object_ACSH_EXTRUSION_CLASS_pt2_get},
{"LibreDWGc::new_Dwg_Object_ACSH_EXTRUSION_CLASS", _wrap_new_Dwg_Object_ACSH_EXTRUSION_CLASS},
{"LibreDWGc::delete_Dwg_Object_ACSH_EXTRUSION_CLASS", _wrap_delete_Dwg_Object_ACSH_EXTRUSION_CLASS},
{"LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_parent_set", _wrap_Dwg_Object_ACSH_LOFT_CLASS_parent_set},
{"LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_parent_get", _wrap_Dwg_Object_ACSH_LOFT_CLASS_parent_get},
{"LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_evalexpr_set", _wrap_Dwg_Object_ACSH_LOFT_CLASS_evalexpr_set},
{"LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_evalexpr_get", _wrap_Dwg_Object_ACSH_LOFT_CLASS_evalexpr_get},
{"LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_history_node_set", _wrap_Dwg_Object_ACSH_LOFT_CLASS_history_node_set},
{"LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_history_node_get", _wrap_Dwg_Object_ACSH_LOFT_CLASS_history_node_get},
{"LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_major_set", _wrap_Dwg_Object_ACSH_LOFT_CLASS_major_set},
{"LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_major_get", _wrap_Dwg_Object_ACSH_LOFT_CLASS_major_get},
{"LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_minor_set", _wrap_Dwg_Object_ACSH_LOFT_CLASS_minor_set},
{"LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_minor_get", _wrap_Dwg_Object_ACSH_LOFT_CLASS_minor_get},
{"LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_num_crosssects_set", _wrap_Dwg_Object_ACSH_LOFT_CLASS_num_crosssects_set},
{"LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_num_crosssects_get", _wrap_Dwg_Object_ACSH_LOFT_CLASS_num_crosssects_get},
{"LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_crosssects_set", _wrap_Dwg_Object_ACSH_LOFT_CLASS_crosssects_set},
{"LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_crosssects_get", _wrap_Dwg_Object_ACSH_LOFT_CLASS_crosssects_get},
{"LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_num_guides_set", _wrap_Dwg_Object_ACSH_LOFT_CLASS_num_guides_set},
{"LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_num_guides_get", _wrap_Dwg_Object_ACSH_LOFT_CLASS_num_guides_get},
{"LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_guides_set", _wrap_Dwg_Object_ACSH_LOFT_CLASS_guides_set},
{"LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_guides_get", _wrap_Dwg_Object_ACSH_LOFT_CLASS_guides_get},
{"LibreDWGc::new_Dwg_Object_ACSH_LOFT_CLASS", _wrap_new_Dwg_Object_ACSH_LOFT_CLASS},
{"LibreDWGc::delete_Dwg_Object_ACSH_LOFT_CLASS", _wrap_delete_Dwg_Object_ACSH_LOFT_CLASS},
{"LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_parent_set", _wrap_Dwg_Object_ACSH_FILLET_CLASS_parent_set},
{"LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_parent_get", _wrap_Dwg_Object_ACSH_FILLET_CLASS_parent_get},
{"LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_evalexpr_set", _wrap_Dwg_Object_ACSH_FILLET_CLASS_evalexpr_set},
{"LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_evalexpr_get", _wrap_Dwg_Object_ACSH_FILLET_CLASS_evalexpr_get},
{"LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_history_node_set", _wrap_Dwg_Object_ACSH_FILLET_CLASS_history_node_set},
{"LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_history_node_get", _wrap_Dwg_Object_ACSH_FILLET_CLASS_history_node_get},
{"LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_major_set", _wrap_Dwg_Object_ACSH_FILLET_CLASS_major_set},
{"LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_major_get", _wrap_Dwg_Object_ACSH_FILLET_CLASS_major_get},
{"LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_minor_set", _wrap_Dwg_Object_ACSH_FILLET_CLASS_minor_set},
{"LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_minor_get", _wrap_Dwg_Object_ACSH_FILLET_CLASS_minor_get},
{"LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_bl92_set", _wrap_Dwg_Object_ACSH_FILLET_CLASS_bl92_set},
{"LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_bl92_get", _wrap_Dwg_Object_ACSH_FILLET_CLASS_bl92_get},
{"LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_num_edges_set", _wrap_Dwg_Object_ACSH_FILLET_CLASS_num_edges_set},
{"LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_num_edges_get", _wrap_Dwg_Object_ACSH_FILLET_CLASS_num_edges_get},
{"LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_edges_set", _wrap_Dwg_Object_ACSH_FILLET_CLASS_edges_set},
{"LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_edges_get", _wrap_Dwg_Object_ACSH_FILLET_CLASS_edges_get},
{"LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_num_radiuses_set", _wrap_Dwg_Object_ACSH_FILLET_CLASS_num_radiuses_set},
{"LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_num_radiuses_get", _wrap_Dwg_Object_ACSH_FILLET_CLASS_num_radiuses_get},
{"LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_num_startsetbacks_set", _wrap_Dwg_Object_ACSH_FILLET_CLASS_num_startsetbacks_set},
{"LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_num_startsetbacks_get", _wrap_Dwg_Object_ACSH_FILLET_CLASS_num_startsetbacks_get},
{"LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_num_endsetbacks_set", _wrap_Dwg_Object_ACSH_FILLET_CLASS_num_endsetbacks_set},
{"LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_num_endsetbacks_get", _wrap_Dwg_Object_ACSH_FILLET_CLASS_num_endsetbacks_get},
{"LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_radiuses_set", _wrap_Dwg_Object_ACSH_FILLET_CLASS_radiuses_set},
{"LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_radiuses_get", _wrap_Dwg_Object_ACSH_FILLET_CLASS_radiuses_get},
{"LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_startsetbacks_set", _wrap_Dwg_Object_ACSH_FILLET_CLASS_startsetbacks_set},
{"LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_startsetbacks_get", _wrap_Dwg_Object_ACSH_FILLET_CLASS_startsetbacks_get},
{"LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_endsetbacks_set", _wrap_Dwg_Object_ACSH_FILLET_CLASS_endsetbacks_set},
{"LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_endsetbacks_get", _wrap_Dwg_Object_ACSH_FILLET_CLASS_endsetbacks_get},
{"LibreDWGc::new_Dwg_Object_ACSH_FILLET_CLASS", _wrap_new_Dwg_Object_ACSH_FILLET_CLASS},
{"LibreDWGc::delete_Dwg_Object_ACSH_FILLET_CLASS", _wrap_delete_Dwg_Object_ACSH_FILLET_CLASS},
{"LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_parent_set", _wrap_Dwg_Object_ACSH_CHAMFER_CLASS_parent_set},
{"LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_parent_get", _wrap_Dwg_Object_ACSH_CHAMFER_CLASS_parent_get},
{"LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_evalexpr_set", _wrap_Dwg_Object_ACSH_CHAMFER_CLASS_evalexpr_set},
{"LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_evalexpr_get", _wrap_Dwg_Object_ACSH_CHAMFER_CLASS_evalexpr_get},
{"LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_history_node_set", _wrap_Dwg_Object_ACSH_CHAMFER_CLASS_history_node_set},
{"LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_history_node_get", _wrap_Dwg_Object_ACSH_CHAMFER_CLASS_history_node_get},
{"LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_major_set", _wrap_Dwg_Object_ACSH_CHAMFER_CLASS_major_set},
{"LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_major_get", _wrap_Dwg_Object_ACSH_CHAMFER_CLASS_major_get},
{"LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_minor_set", _wrap_Dwg_Object_ACSH_CHAMFER_CLASS_minor_set},
{"LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_minor_get", _wrap_Dwg_Object_ACSH_CHAMFER_CLASS_minor_get},
{"LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_bl92_set", _wrap_Dwg_Object_ACSH_CHAMFER_CLASS_bl92_set},
{"LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_bl92_get", _wrap_Dwg_Object_ACSH_CHAMFER_CLASS_bl92_get},
{"LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_base_dist_set", _wrap_Dwg_Object_ACSH_CHAMFER_CLASS_base_dist_set},
{"LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_base_dist_get", _wrap_Dwg_Object_ACSH_CHAMFER_CLASS_base_dist_get},
{"LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_other_dist_set", _wrap_Dwg_Object_ACSH_CHAMFER_CLASS_other_dist_set},
{"LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_other_dist_get", _wrap_Dwg_Object_ACSH_CHAMFER_CLASS_other_dist_get},
{"LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_num_edges_set", _wrap_Dwg_Object_ACSH_CHAMFER_CLASS_num_edges_set},
{"LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_num_edges_get", _wrap_Dwg_Object_ACSH_CHAMFER_CLASS_num_edges_get},
{"LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_edges_set", _wrap_Dwg_Object_ACSH_CHAMFER_CLASS_edges_set},
{"LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_edges_get", _wrap_Dwg_Object_ACSH_CHAMFER_CLASS_edges_get},
{"LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_bl95_set", _wrap_Dwg_Object_ACSH_CHAMFER_CLASS_bl95_set},
{"LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_bl95_get", _wrap_Dwg_Object_ACSH_CHAMFER_CLASS_bl95_get},
{"LibreDWGc::new_Dwg_Object_ACSH_CHAMFER_CLASS", _wrap_new_Dwg_Object_ACSH_CHAMFER_CLASS},
{"LibreDWGc::delete_Dwg_Object_ACSH_CHAMFER_CLASS", _wrap_delete_Dwg_Object_ACSH_CHAMFER_CLASS},
{"LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_parent_set", _wrap_Dwg_Object_ACSH_CYLINDER_CLASS_parent_set},
{"LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_parent_get", _wrap_Dwg_Object_ACSH_CYLINDER_CLASS_parent_get},
{"LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_evalexpr_set", _wrap_Dwg_Object_ACSH_CYLINDER_CLASS_evalexpr_set},
{"LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_evalexpr_get", _wrap_Dwg_Object_ACSH_CYLINDER_CLASS_evalexpr_get},
{"LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_history_node_set", _wrap_Dwg_Object_ACSH_CYLINDER_CLASS_history_node_set},
{"LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_history_node_get", _wrap_Dwg_Object_ACSH_CYLINDER_CLASS_history_node_get},
{"LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_major_set", _wrap_Dwg_Object_ACSH_CYLINDER_CLASS_major_set},
{"LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_major_get", _wrap_Dwg_Object_ACSH_CYLINDER_CLASS_major_get},
{"LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_minor_set", _wrap_Dwg_Object_ACSH_CYLINDER_CLASS_minor_set},
{"LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_minor_get", _wrap_Dwg_Object_ACSH_CYLINDER_CLASS_minor_get},
{"LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_height_set", _wrap_Dwg_Object_ACSH_CYLINDER_CLASS_height_set},
{"LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_height_get", _wrap_Dwg_Object_ACSH_CYLINDER_CLASS_height_get},
{"LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_major_radius_set", _wrap_Dwg_Object_ACSH_CYLINDER_CLASS_major_radius_set},
{"LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_major_radius_get", _wrap_Dwg_Object_ACSH_CYLINDER_CLASS_major_radius_get},
{"LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_minor_radius_set", _wrap_Dwg_Object_ACSH_CYLINDER_CLASS_minor_radius_set},
{"LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_minor_radius_get", _wrap_Dwg_Object_ACSH_CYLINDER_CLASS_minor_radius_get},
{"LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_x_radius_set", _wrap_Dwg_Object_ACSH_CYLINDER_CLASS_x_radius_set},
{"LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_x_radius_get", _wrap_Dwg_Object_ACSH_CYLINDER_CLASS_x_radius_get},
{"LibreDWGc::new_Dwg_Object_ACSH_CYLINDER_CLASS", _wrap_new_Dwg_Object_ACSH_CYLINDER_CLASS},
{"LibreDWGc::delete_Dwg_Object_ACSH_CYLINDER_CLASS", _wrap_delete_Dwg_Object_ACSH_CYLINDER_CLASS},
{"LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_parent_set", _wrap_Dwg_Object_ACSH_CONE_CLASS_parent_set},
{"LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_parent_get", _wrap_Dwg_Object_ACSH_CONE_CLASS_parent_get},
{"LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_evalexpr_set", _wrap_Dwg_Object_ACSH_CONE_CLASS_evalexpr_set},
{"LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_evalexpr_get", _wrap_Dwg_Object_ACSH_CONE_CLASS_evalexpr_get},
{"LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_history_node_set", _wrap_Dwg_Object_ACSH_CONE_CLASS_history_node_set},
{"LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_history_node_get", _wrap_Dwg_Object_ACSH_CONE_CLASS_history_node_get},
{"LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_major_set", _wrap_Dwg_Object_ACSH_CONE_CLASS_major_set},
{"LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_major_get", _wrap_Dwg_Object_ACSH_CONE_CLASS_major_get},
{"LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_minor_set", _wrap_Dwg_Object_ACSH_CONE_CLASS_minor_set},
{"LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_minor_get", _wrap_Dwg_Object_ACSH_CONE_CLASS_minor_get},
{"LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_height_set", _wrap_Dwg_Object_ACSH_CONE_CLASS_height_set},
{"LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_height_get", _wrap_Dwg_Object_ACSH_CONE_CLASS_height_get},
{"LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_major_radius_set", _wrap_Dwg_Object_ACSH_CONE_CLASS_major_radius_set},
{"LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_major_radius_get", _wrap_Dwg_Object_ACSH_CONE_CLASS_major_radius_get},
{"LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_minor_radius_set", _wrap_Dwg_Object_ACSH_CONE_CLASS_minor_radius_set},
{"LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_minor_radius_get", _wrap_Dwg_Object_ACSH_CONE_CLASS_minor_radius_get},
{"LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_x_radius_set", _wrap_Dwg_Object_ACSH_CONE_CLASS_x_radius_set},
{"LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_x_radius_get", _wrap_Dwg_Object_ACSH_CONE_CLASS_x_radius_get},
{"LibreDWGc::new_Dwg_Object_ACSH_CONE_CLASS", _wrap_new_Dwg_Object_ACSH_CONE_CLASS},
{"LibreDWGc::delete_Dwg_Object_ACSH_CONE_CLASS", _wrap_delete_Dwg_Object_ACSH_CONE_CLASS},
{"LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_parent_set", _wrap_Dwg_Object_ACSH_PYRAMID_CLASS_parent_set},
{"LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_parent_get", _wrap_Dwg_Object_ACSH_PYRAMID_CLASS_parent_get},
{"LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_evalexpr_set", _wrap_Dwg_Object_ACSH_PYRAMID_CLASS_evalexpr_set},
{"LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_evalexpr_get", _wrap_Dwg_Object_ACSH_PYRAMID_CLASS_evalexpr_get},
{"LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_history_node_set", _wrap_Dwg_Object_ACSH_PYRAMID_CLASS_history_node_set},
{"LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_history_node_get", _wrap_Dwg_Object_ACSH_PYRAMID_CLASS_history_node_get},
{"LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_major_set", _wrap_Dwg_Object_ACSH_PYRAMID_CLASS_major_set},
{"LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_major_get", _wrap_Dwg_Object_ACSH_PYRAMID_CLASS_major_get},
{"LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_minor_set", _wrap_Dwg_Object_ACSH_PYRAMID_CLASS_minor_set},
{"LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_minor_get", _wrap_Dwg_Object_ACSH_PYRAMID_CLASS_minor_get},
{"LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_height_set", _wrap_Dwg_Object_ACSH_PYRAMID_CLASS_height_set},
{"LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_height_get", _wrap_Dwg_Object_ACSH_PYRAMID_CLASS_height_get},
{"LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_sides_set", _wrap_Dwg_Object_ACSH_PYRAMID_CLASS_sides_set},
{"LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_sides_get", _wrap_Dwg_Object_ACSH_PYRAMID_CLASS_sides_get},
{"LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_radius_set", _wrap_Dwg_Object_ACSH_PYRAMID_CLASS_radius_set},
{"LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_radius_get", _wrap_Dwg_Object_ACSH_PYRAMID_CLASS_radius_get},
{"LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_topradius_set", _wrap_Dwg_Object_ACSH_PYRAMID_CLASS_topradius_set},
{"LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_topradius_get", _wrap_Dwg_Object_ACSH_PYRAMID_CLASS_topradius_get},
{"LibreDWGc::new_Dwg_Object_ACSH_PYRAMID_CLASS", _wrap_new_Dwg_Object_ACSH_PYRAMID_CLASS},
{"LibreDWGc::delete_Dwg_Object_ACSH_PYRAMID_CLASS", _wrap_delete_Dwg_Object_ACSH_PYRAMID_CLASS},
{"LibreDWGc::Dwg_Object_ACSH_SPHERE_CLASS_parent_set", _wrap_Dwg_Object_ACSH_SPHERE_CLASS_parent_set},
{"LibreDWGc::Dwg_Object_ACSH_SPHERE_CLASS_parent_get", _wrap_Dwg_Object_ACSH_SPHERE_CLASS_parent_get},
{"LibreDWGc::Dwg_Object_ACSH_SPHERE_CLASS_evalexpr_set", _wrap_Dwg_Object_ACSH_SPHERE_CLASS_evalexpr_set},
{"LibreDWGc::Dwg_Object_ACSH_SPHERE_CLASS_evalexpr_get", _wrap_Dwg_Object_ACSH_SPHERE_CLASS_evalexpr_get},
{"LibreDWGc::Dwg_Object_ACSH_SPHERE_CLASS_history_node_set", _wrap_Dwg_Object_ACSH_SPHERE_CLASS_history_node_set},
{"LibreDWGc::Dwg_Object_ACSH_SPHERE_CLASS_history_node_get", _wrap_Dwg_Object_ACSH_SPHERE_CLASS_history_node_get},
{"LibreDWGc::Dwg_Object_ACSH_SPHERE_CLASS_major_set", _wrap_Dwg_Object_ACSH_SPHERE_CLASS_major_set},
{"LibreDWGc::Dwg_Object_ACSH_SPHERE_CLASS_major_get", _wrap_Dwg_Object_ACSH_SPHERE_CLASS_major_get},
{"LibreDWGc::Dwg_Object_ACSH_SPHERE_CLASS_minor_set", _wrap_Dwg_Object_ACSH_SPHERE_CLASS_minor_set},
{"LibreDWGc::Dwg_Object_ACSH_SPHERE_CLASS_minor_get", _wrap_Dwg_Object_ACSH_SPHERE_CLASS_minor_get},
{"LibreDWGc::Dwg_Object_ACSH_SPHERE_CLASS_radius_set", _wrap_Dwg_Object_ACSH_SPHERE_CLASS_radius_set},
{"LibreDWGc::Dwg_Object_ACSH_SPHERE_CLASS_radius_get", _wrap_Dwg_Object_ACSH_SPHERE_CLASS_radius_get},
{"LibreDWGc::new_Dwg_Object_ACSH_SPHERE_CLASS", _wrap_new_Dwg_Object_ACSH_SPHERE_CLASS},
{"LibreDWGc::delete_Dwg_Object_ACSH_SPHERE_CLASS", _wrap_delete_Dwg_Object_ACSH_SPHERE_CLASS},
{"LibreDWGc::Dwg_Object_ACSH_TORUS_CLASS_parent_set", _wrap_Dwg_Object_ACSH_TORUS_CLASS_parent_set},
{"LibreDWGc::Dwg_Object_ACSH_TORUS_CLASS_parent_get", _wrap_Dwg_Object_ACSH_TORUS_CLASS_parent_get},
{"LibreDWGc::Dwg_Object_ACSH_TORUS_CLASS_evalexpr_set", _wrap_Dwg_Object_ACSH_TORUS_CLASS_evalexpr_set},
{"LibreDWGc::Dwg_Object_ACSH_TORUS_CLASS_evalexpr_get", _wrap_Dwg_Object_ACSH_TORUS_CLASS_evalexpr_get},
{"LibreDWGc::Dwg_Object_ACSH_TORUS_CLASS_history_node_set", _wrap_Dwg_Object_ACSH_TORUS_CLASS_history_node_set},
{"LibreDWGc::Dwg_Object_ACSH_TORUS_CLASS_history_node_get", _wrap_Dwg_Object_ACSH_TORUS_CLASS_history_node_get},
{"LibreDWGc::Dwg_Object_ACSH_TORUS_CLASS_major_set", _wrap_Dwg_Object_ACSH_TORUS_CLASS_major_set},
{"LibreDWGc::Dwg_Object_ACSH_TORUS_CLASS_major_get", _wrap_Dwg_Object_ACSH_TORUS_CLASS_major_get},
{"LibreDWGc::Dwg_Object_ACSH_TORUS_CLASS_minor_set", _wrap_Dwg_Object_ACSH_TORUS_CLASS_minor_set},
{"LibreDWGc::Dwg_Object_ACSH_TORUS_CLASS_minor_get", _wrap_Dwg_Object_ACSH_TORUS_CLASS_minor_get},
{"LibreDWGc::Dwg_Object_ACSH_TORUS_CLASS_major_radius_set", _wrap_Dwg_Object_ACSH_TORUS_CLASS_major_radius_set},
{"LibreDWGc::Dwg_Object_ACSH_TORUS_CLASS_major_radius_get", _wrap_Dwg_Object_ACSH_TORUS_CLASS_major_radius_get},
{"LibreDWGc::Dwg_Object_ACSH_TORUS_CLASS_minor_radius_set", _wrap_Dwg_Object_ACSH_TORUS_CLASS_minor_radius_set},
{"LibreDWGc::Dwg_Object_ACSH_TORUS_CLASS_minor_radius_get", _wrap_Dwg_Object_ACSH_TORUS_CLASS_minor_radius_get},
{"LibreDWGc::new_Dwg_Object_ACSH_TORUS_CLASS", _wrap_new_Dwg_Object_ACSH_TORUS_CLASS},
{"LibreDWGc::delete_Dwg_Object_ACSH_TORUS_CLASS", _wrap_delete_Dwg_Object_ACSH_TORUS_CLASS},
{"LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_parent_set", _wrap_Dwg_Object_ACSH_REVOLVE_CLASS_parent_set},
{"LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_parent_get", _wrap_Dwg_Object_ACSH_REVOLVE_CLASS_parent_get},
{"LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_evalexpr_set", _wrap_Dwg_Object_ACSH_REVOLVE_CLASS_evalexpr_set},
{"LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_evalexpr_get", _wrap_Dwg_Object_ACSH_REVOLVE_CLASS_evalexpr_get},
{"LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_history_node_set", _wrap_Dwg_Object_ACSH_REVOLVE_CLASS_history_node_set},
{"LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_history_node_get", _wrap_Dwg_Object_ACSH_REVOLVE_CLASS_history_node_get},
{"LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_major_set", _wrap_Dwg_Object_ACSH_REVOLVE_CLASS_major_set},
{"LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_major_get", _wrap_Dwg_Object_ACSH_REVOLVE_CLASS_major_get},
{"LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_minor_set", _wrap_Dwg_Object_ACSH_REVOLVE_CLASS_minor_set},
{"LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_minor_get", _wrap_Dwg_Object_ACSH_REVOLVE_CLASS_minor_get},
{"LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_axis_pt_set", _wrap_Dwg_Object_ACSH_REVOLVE_CLASS_axis_pt_set},
{"LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_axis_pt_get", _wrap_Dwg_Object_ACSH_REVOLVE_CLASS_axis_pt_get},
{"LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_direction_set", _wrap_Dwg_Object_ACSH_REVOLVE_CLASS_direction_set},
{"LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_direction_get", _wrap_Dwg_Object_ACSH_REVOLVE_CLASS_direction_get},
{"LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_revolve_angle_set", _wrap_Dwg_Object_ACSH_REVOLVE_CLASS_revolve_angle_set},
{"LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_revolve_angle_get", _wrap_Dwg_Object_ACSH_REVOLVE_CLASS_revolve_angle_get},
{"LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_start_angle_set", _wrap_Dwg_Object_ACSH_REVOLVE_CLASS_start_angle_set},
{"LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_start_angle_get", _wrap_Dwg_Object_ACSH_REVOLVE_CLASS_start_angle_get},
{"LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_draft_angle_set", _wrap_Dwg_Object_ACSH_REVOLVE_CLASS_draft_angle_set},
{"LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_draft_angle_get", _wrap_Dwg_Object_ACSH_REVOLVE_CLASS_draft_angle_get},
{"LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_bd44_set", _wrap_Dwg_Object_ACSH_REVOLVE_CLASS_bd44_set},
{"LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_bd44_get", _wrap_Dwg_Object_ACSH_REVOLVE_CLASS_bd44_get},
{"LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_bd45_set", _wrap_Dwg_Object_ACSH_REVOLVE_CLASS_bd45_set},
{"LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_bd45_get", _wrap_Dwg_Object_ACSH_REVOLVE_CLASS_bd45_get},
{"LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_twist_angle_set", _wrap_Dwg_Object_ACSH_REVOLVE_CLASS_twist_angle_set},
{"LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_twist_angle_get", _wrap_Dwg_Object_ACSH_REVOLVE_CLASS_twist_angle_get},
{"LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_b290_set", _wrap_Dwg_Object_ACSH_REVOLVE_CLASS_b290_set},
{"LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_b290_get", _wrap_Dwg_Object_ACSH_REVOLVE_CLASS_b290_get},
{"LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_is_close_to_axis_set", _wrap_Dwg_Object_ACSH_REVOLVE_CLASS_is_close_to_axis_set},
{"LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_is_close_to_axis_get", _wrap_Dwg_Object_ACSH_REVOLVE_CLASS_is_close_to_axis_get},
{"LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_sweep_entity_set", _wrap_Dwg_Object_ACSH_REVOLVE_CLASS_sweep_entity_set},
{"LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_sweep_entity_get", _wrap_Dwg_Object_ACSH_REVOLVE_CLASS_sweep_entity_get},
{"LibreDWGc::new_Dwg_Object_ACSH_REVOLVE_CLASS", _wrap_new_Dwg_Object_ACSH_REVOLVE_CLASS},
{"LibreDWGc::delete_Dwg_Object_ACSH_REVOLVE_CLASS", _wrap_delete_Dwg_Object_ACSH_REVOLVE_CLASS},
{"LibreDWGc::Dwg_Entity_NAVISWORKSMODEL_parent_set", _wrap_Dwg_Entity_NAVISWORKSMODEL_parent_set},
{"LibreDWGc::Dwg_Entity_NAVISWORKSMODEL_parent_get", _wrap_Dwg_Entity_NAVISWORKSMODEL_parent_get},
{"LibreDWGc::Dwg_Entity_NAVISWORKSMODEL_flags_set", _wrap_Dwg_Entity_NAVISWORKSMODEL_flags_set},
{"LibreDWGc::Dwg_Entity_NAVISWORKSMODEL_flags_get", _wrap_Dwg_Entity_NAVISWORKSMODEL_flags_get},
{"LibreDWGc::Dwg_Entity_NAVISWORKSMODEL_definition_set", _wrap_Dwg_Entity_NAVISWORKSMODEL_definition_set},
{"LibreDWGc::Dwg_Entity_NAVISWORKSMODEL_definition_get", _wrap_Dwg_Entity_NAVISWORKSMODEL_definition_get},
{"LibreDWGc::Dwg_Entity_NAVISWORKSMODEL_transmatrix_set", _wrap_Dwg_Entity_NAVISWORKSMODEL_transmatrix_set},
{"LibreDWGc::Dwg_Entity_NAVISWORKSMODEL_transmatrix_get", _wrap_Dwg_Entity_NAVISWORKSMODEL_transmatrix_get},
{"LibreDWGc::Dwg_Entity_NAVISWORKSMODEL_unitfactor_set", _wrap_Dwg_Entity_NAVISWORKSMODEL_unitfactor_set},
{"LibreDWGc::Dwg_Entity_NAVISWORKSMODEL_unitfactor_get", _wrap_Dwg_Entity_NAVISWORKSMODEL_unitfactor_get},
{"LibreDWGc::new_Dwg_Entity_NAVISWORKSMODEL", _wrap_new_Dwg_Entity_NAVISWORKSMODEL},
{"LibreDWGc::delete_Dwg_Entity_NAVISWORKSMODEL", _wrap_delete_Dwg_Entity_NAVISWORKSMODEL},
{"LibreDWGc::Dwg_Object_NAVISWORKSMODELDEF_parent_set", _wrap_Dwg_Object_NAVISWORKSMODELDEF_parent_set},
{"LibreDWGc::Dwg_Object_NAVISWORKSMODELDEF_parent_get", _wrap_Dwg_Object_NAVISWORKSMODELDEF_parent_get},
{"LibreDWGc::Dwg_Object_NAVISWORKSMODELDEF_flags_set", _wrap_Dwg_Object_NAVISWORKSMODELDEF_flags_set},
{"LibreDWGc::Dwg_Object_NAVISWORKSMODELDEF_flags_get", _wrap_Dwg_Object_NAVISWORKSMODELDEF_flags_get},
{"LibreDWGc::Dwg_Object_NAVISWORKSMODELDEF_path_set", _wrap_Dwg_Object_NAVISWORKSMODELDEF_path_set},
{"LibreDWGc::Dwg_Object_NAVISWORKSMODELDEF_path_get", _wrap_Dwg_Object_NAVISWORKSMODELDEF_path_get},
{"LibreDWGc::Dwg_Object_NAVISWORKSMODELDEF_status_set", _wrap_Dwg_Object_NAVISWORKSMODELDEF_status_set},
{"LibreDWGc::Dwg_Object_NAVISWORKSMODELDEF_status_get", _wrap_Dwg_Object_NAVISWORKSMODELDEF_status_get},
{"LibreDWGc::Dwg_Object_NAVISWORKSMODELDEF_min_extent_set", _wrap_Dwg_Object_NAVISWORKSMODELDEF_min_extent_set},
{"LibreDWGc::Dwg_Object_NAVISWORKSMODELDEF_min_extent_get", _wrap_Dwg_Object_NAVISWORKSMODELDEF_min_extent_get},
{"LibreDWGc::Dwg_Object_NAVISWORKSMODELDEF_max_extent_set", _wrap_Dwg_Object_NAVISWORKSMODELDEF_max_extent_set},
{"LibreDWGc::Dwg_Object_NAVISWORKSMODELDEF_max_extent_get", _wrap_Dwg_Object_NAVISWORKSMODELDEF_max_extent_get},
{"LibreDWGc::Dwg_Object_NAVISWORKSMODELDEF_host_drawing_visibility_set", _wrap_Dwg_Object_NAVISWORKSMODELDEF_host_drawing_visibility_set},
{"LibreDWGc::Dwg_Object_NAVISWORKSMODELDEF_host_drawing_visibility_get", _wrap_Dwg_Object_NAVISWORKSMODELDEF_host_drawing_visibility_get},
{"LibreDWGc::new_Dwg_Object_NAVISWORKSMODELDEF", _wrap_new_Dwg_Object_NAVISWORKSMODELDEF},
{"LibreDWGc::delete_Dwg_Object_NAVISWORKSMODELDEF", _wrap_delete_Dwg_Object_NAVISWORKSMODELDEF},
{"LibreDWGc::Dwg_Object_RENDERSETTINGS_parent_set", _wrap_Dwg_Object_RENDERSETTINGS_parent_set},
{"LibreDWGc::Dwg_Object_RENDERSETTINGS_parent_get", _wrap_Dwg_Object_RENDERSETTINGS_parent_get},
{"LibreDWGc::Dwg_Object_RENDERSETTINGS_class_version_set", _wrap_Dwg_Object_RENDERSETTINGS_class_version_set},
{"LibreDWGc::Dwg_Object_RENDERSETTINGS_class_version_get", _wrap_Dwg_Object_RENDERSETTINGS_class_version_get},
{"LibreDWGc::Dwg_Object_RENDERSETTINGS_name_set", _wrap_Dwg_Object_RENDERSETTINGS_name_set},
{"LibreDWGc::Dwg_Object_RENDERSETTINGS_name_get", _wrap_Dwg_Object_RENDERSETTINGS_name_get},
{"LibreDWGc::Dwg_Object_RENDERSETTINGS_fog_enabled_set", _wrap_Dwg_Object_RENDERSETTINGS_fog_enabled_set},
{"LibreDWGc::Dwg_Object_RENDERSETTINGS_fog_enabled_get", _wrap_Dwg_Object_RENDERSETTINGS_fog_enabled_get},
{"LibreDWGc::Dwg_Object_RENDERSETTINGS_fog_background_enabled_set", _wrap_Dwg_Object_RENDERSETTINGS_fog_background_enabled_set},
{"LibreDWGc::Dwg_Object_RENDERSETTINGS_fog_background_enabled_get", _wrap_Dwg_Object_RENDERSETTINGS_fog_background_enabled_get},
{"LibreDWGc::Dwg_Object_RENDERSETTINGS_backfaces_enabled_set", _wrap_Dwg_Object_RENDERSETTINGS_backfaces_enabled_set},
{"LibreDWGc::Dwg_Object_RENDERSETTINGS_backfaces_enabled_get", _wrap_Dwg_Object_RENDERSETTINGS_backfaces_enabled_get},
{"LibreDWGc::Dwg_Object_RENDERSETTINGS_environ_image_enabled_set", _wrap_Dwg_Object_RENDERSETTINGS_environ_image_enabled_set},
{"LibreDWGc::Dwg_Object_RENDERSETTINGS_environ_image_enabled_get", _wrap_Dwg_Object_RENDERSETTINGS_environ_image_enabled_get},
{"LibreDWGc::Dwg_Object_RENDERSETTINGS_environ_image_filename_set", _wrap_Dwg_Object_RENDERSETTINGS_environ_image_filename_set},
{"LibreDWGc::Dwg_Object_RENDERSETTINGS_environ_image_filename_get", _wrap_Dwg_Object_RENDERSETTINGS_environ_image_filename_get},
{"LibreDWGc::Dwg_Object_RENDERSETTINGS_description_set", _wrap_Dwg_Object_RENDERSETTINGS_description_set},
{"LibreDWGc::Dwg_Object_RENDERSETTINGS_description_get", _wrap_Dwg_Object_RENDERSETTINGS_description_get},
{"LibreDWGc::Dwg_Object_RENDERSETTINGS_display_index_set", _wrap_Dwg_Object_RENDERSETTINGS_display_index_set},
{"LibreDWGc::Dwg_Object_RENDERSETTINGS_display_index_get", _wrap_Dwg_Object_RENDERSETTINGS_display_index_get},
{"LibreDWGc::Dwg_Object_RENDERSETTINGS_has_predefined_set", _wrap_Dwg_Object_RENDERSETTINGS_has_predefined_set},
{"LibreDWGc::Dwg_Object_RENDERSETTINGS_has_predefined_get", _wrap_Dwg_Object_RENDERSETTINGS_has_predefined_get},
{"LibreDWGc::new_Dwg_Object_RENDERSETTINGS", _wrap_new_Dwg_Object_RENDERSETTINGS},
{"LibreDWGc::delete_Dwg_Object_RENDERSETTINGS", _wrap_delete_Dwg_Object_RENDERSETTINGS},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_parent_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_parent_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_parent_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_parent_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_class_version_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_class_version_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_class_version_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_class_version_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_name_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_name_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_name_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_name_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_fog_enabled_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_fog_enabled_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_fog_enabled_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_fog_enabled_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_fog_background_enabled_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_fog_background_enabled_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_fog_background_enabled_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_fog_background_enabled_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_backfaces_enabled_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_backfaces_enabled_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_backfaces_enabled_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_backfaces_enabled_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_environ_image_enabled_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_environ_image_enabled_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_environ_image_enabled_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_environ_image_enabled_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_environ_image_filename_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_environ_image_filename_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_environ_image_filename_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_environ_image_filename_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_description_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_description_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_description_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_description_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_display_index_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_display_index_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_display_index_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_display_index_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_has_predefined_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_has_predefined_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_has_predefined_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_has_predefined_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_mr_version_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_mr_version_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_mr_version_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_mr_version_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling1_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling1_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling1_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling1_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling2_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling2_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling2_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling2_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_mr_filter_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_mr_filter_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_mr_filter_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_mr_filter_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_filter1_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_filter1_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_filter1_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_filter1_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_filter2_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_filter2_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_filter2_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_filter2_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color1_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color1_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color1_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color1_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color2_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color2_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color2_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color2_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color3_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color3_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color3_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color3_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color4_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color4_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color4_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color4_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_shadow_mode_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_shadow_mode_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_shadow_mode_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_shadow_mode_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_shadow_maps_enabled_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_shadow_maps_enabled_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_shadow_maps_enabled_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_shadow_maps_enabled_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_ray_tracing_enabled_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_ray_tracing_enabled_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_ray_tracing_enabled_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_ray_tracing_enabled_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth1_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth1_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth1_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth1_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth2_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth2_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth2_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth2_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth3_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth3_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth3_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth3_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_global_illumination_enabled_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_global_illumination_enabled_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_global_illumination_enabled_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_global_illumination_enabled_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_count_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_count_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_count_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_count_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_radius_enabled_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_radius_enabled_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_radius_enabled_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_radius_enabled_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_radius_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_radius_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_radius_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_radius_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_gi_photons_per_light_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_gi_photons_per_light_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_gi_photons_per_light_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_gi_photons_per_light_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth1_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth1_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth1_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth1_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth2_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth2_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth2_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth2_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth3_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth3_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth3_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth3_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_final_gathering_enabled_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_final_gathering_enabled_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_final_gathering_enabled_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_final_gathering_enabled_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_fg_ray_count_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_fg_ray_count_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_fg_ray_count_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_fg_ray_count_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state1_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state1_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state1_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state1_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state2_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state2_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state2_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state2_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state3_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state3_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state3_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state3_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius1_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius1_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius1_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius1_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius2_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius2_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius2_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius2_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_light_luminance_scale_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_light_luminance_scale_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_light_luminance_scale_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_light_luminance_scale_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_mode_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_mode_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_mode_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_mode_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_grid_mode_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_grid_mode_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_grid_mode_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_grid_mode_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_grid_float_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_grid_float_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_grid_float_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_grid_float_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_photon_mode_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_photon_mode_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_photon_mode_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_photon_mode_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_bsp_mode_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_bsp_mode_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_bsp_mode_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_bsp_mode_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_export_mi_enabled_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_export_mi_enabled_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_export_mi_enabled_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_export_mi_enabled_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_mr_description_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_mr_description_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_mr_description_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_mr_description_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_tile_size_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_tile_size_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_tile_size_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_tile_size_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_tile_order_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_tile_order_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_tile_order_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_tile_order_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_memory_limit_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_memory_limit_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_memory_limit_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_memory_limit_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_samples_mode_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_samples_mode_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_samples_mode_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_samples_mode_get},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_energy_multiplier_set", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_energy_multiplier_set},
{"LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_energy_multiplier_get", _wrap_Dwg_Object_MENTALRAYRENDERSETTINGS_energy_multiplier_get},
{"LibreDWGc::new_Dwg_Object_MENTALRAYRENDERSETTINGS", _wrap_new_Dwg_Object_MENTALRAYRENDERSETTINGS},
{"LibreDWGc::delete_Dwg_Object_MENTALRAYRENDERSETTINGS", _wrap_delete_Dwg_Object_MENTALRAYRENDERSETTINGS},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_parent_set", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_parent_set},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_parent_get", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_parent_get},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_class_version_set", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_class_version_set},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_class_version_get", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_class_version_get},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_name_set", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_name_set},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_name_get", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_name_get},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_fog_enabled_set", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_fog_enabled_set},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_fog_enabled_get", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_fog_enabled_get},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_fog_background_enabled_set", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_fog_background_enabled_set},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_fog_background_enabled_get", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_fog_background_enabled_get},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_backfaces_enabled_set", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_backfaces_enabled_set},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_backfaces_enabled_get", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_backfaces_enabled_get},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_environ_image_enabled_set", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_environ_image_enabled_set},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_environ_image_enabled_get", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_environ_image_enabled_get},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_environ_image_filename_set", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_environ_image_filename_set},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_environ_image_filename_get", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_environ_image_filename_get},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_description_set", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_description_set},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_description_get", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_description_get},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_display_index_set", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_display_index_set},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_display_index_get", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_display_index_get},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_has_predefined_set", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_has_predefined_set},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_has_predefined_get", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_has_predefined_get},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_rapidrt_version_set", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_rapidrt_version_set},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_rapidrt_version_get", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_rapidrt_version_get},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_render_target_set", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_render_target_set},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_render_target_get", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_render_target_get},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_render_level_set", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_render_level_set},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_render_level_get", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_render_level_get},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_render_time_set", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_render_time_set},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_render_time_get", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_render_time_get},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_lighting_model_set", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_lighting_model_set},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_lighting_model_get", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_lighting_model_get},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_filter_type_set", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_filter_type_set},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_filter_type_get", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_filter_type_get},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_filter_width_set", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_filter_width_set},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_filter_width_get", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_filter_width_get},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_filter_height_set", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_filter_height_set},
{"LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_filter_height_get", _wrap_Dwg_Object_RAPIDRTRENDERSETTINGS_filter_height_get},
{"LibreDWGc::new_Dwg_Object_RAPIDRTRENDERSETTINGS", _wrap_new_Dwg_Object_RAPIDRTRENDERSETTINGS},
{"LibreDWGc::delete_Dwg_Object_RAPIDRTRENDERSETTINGS", _wrap_delete_Dwg_Object_RAPIDRTRENDERSETTINGS},
{"LibreDWGc::Dwg_Object_RENDERENVIRONMENT_parent_set", _wrap_Dwg_Object_RENDERENVIRONMENT_parent_set},
{"LibreDWGc::Dwg_Object_RENDERENVIRONMENT_parent_get", _wrap_Dwg_Object_RENDERENVIRONMENT_parent_get},
{"LibreDWGc::Dwg_Object_RENDERENVIRONMENT_class_version_set", _wrap_Dwg_Object_RENDERENVIRONMENT_class_version_set},
{"LibreDWGc::Dwg_Object_RENDERENVIRONMENT_class_version_get", _wrap_Dwg_Object_RENDERENVIRONMENT_class_version_get},
{"LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_enabled_set", _wrap_Dwg_Object_RENDERENVIRONMENT_fog_enabled_set},
{"LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_enabled_get", _wrap_Dwg_Object_RENDERENVIRONMENT_fog_enabled_get},
{"LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_background_enabled_set", _wrap_Dwg_Object_RENDERENVIRONMENT_fog_background_enabled_set},
{"LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_background_enabled_get", _wrap_Dwg_Object_RENDERENVIRONMENT_fog_background_enabled_get},
{"LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_color_r_set", _wrap_Dwg_Object_RENDERENVIRONMENT_fog_color_r_set},
{"LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_color_r_get", _wrap_Dwg_Object_RENDERENVIRONMENT_fog_color_r_get},
{"LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_color_g_set", _wrap_Dwg_Object_RENDERENVIRONMENT_fog_color_g_set},
{"LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_color_g_get", _wrap_Dwg_Object_RENDERENVIRONMENT_fog_color_g_get},
{"LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_color_b_set", _wrap_Dwg_Object_RENDERENVIRONMENT_fog_color_b_set},
{"LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_color_b_get", _wrap_Dwg_Object_RENDERENVIRONMENT_fog_color_b_get},
{"LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_density_near_set", _wrap_Dwg_Object_RENDERENVIRONMENT_fog_density_near_set},
{"LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_density_near_get", _wrap_Dwg_Object_RENDERENVIRONMENT_fog_density_near_get},
{"LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_density_far_set", _wrap_Dwg_Object_RENDERENVIRONMENT_fog_density_far_set},
{"LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_density_far_get", _wrap_Dwg_Object_RENDERENVIRONMENT_fog_density_far_get},
{"LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_distance_near_set", _wrap_Dwg_Object_RENDERENVIRONMENT_fog_distance_near_set},
{"LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_distance_near_get", _wrap_Dwg_Object_RENDERENVIRONMENT_fog_distance_near_get},
{"LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_distance_far_set", _wrap_Dwg_Object_RENDERENVIRONMENT_fog_distance_far_set},
{"LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_distance_far_get", _wrap_Dwg_Object_RENDERENVIRONMENT_fog_distance_far_get},
{"LibreDWGc::Dwg_Object_RENDERENVIRONMENT_environ_image_enabled_set", _wrap_Dwg_Object_RENDERENVIRONMENT_environ_image_enabled_set},
{"LibreDWGc::Dwg_Object_RENDERENVIRONMENT_environ_image_enabled_get", _wrap_Dwg_Object_RENDERENVIRONMENT_environ_image_enabled_get},
{"LibreDWGc::Dwg_Object_RENDERENVIRONMENT_environ_image_filename_set", _wrap_Dwg_Object_RENDERENVIRONMENT_environ_image_filename_set},
{"LibreDWGc::Dwg_Object_RENDERENVIRONMENT_environ_image_filename_get", _wrap_Dwg_Object_RENDERENVIRONMENT_environ_image_filename_get},
{"LibreDWGc::new_Dwg_Object_RENDERENVIRONMENT", _wrap_new_Dwg_Object_RENDERENVIRONMENT},
{"LibreDWGc::delete_Dwg_Object_RENDERENVIRONMENT", _wrap_delete_Dwg_Object_RENDERENVIRONMENT},
{"LibreDWGc::Dwg_Object_RENDERGLOBAL_parent_set", _wrap_Dwg_Object_RENDERGLOBAL_parent_set},
{"LibreDWGc::Dwg_Object_RENDERGLOBAL_parent_get", _wrap_Dwg_Object_RENDERGLOBAL_parent_get},
{"LibreDWGc::Dwg_Object_RENDERGLOBAL_class_version_set", _wrap_Dwg_Object_RENDERGLOBAL_class_version_set},
{"LibreDWGc::Dwg_Object_RENDERGLOBAL_class_version_get", _wrap_Dwg_Object_RENDERGLOBAL_class_version_get},
{"LibreDWGc::Dwg_Object_RENDERGLOBAL_procedure_set", _wrap_Dwg_Object_RENDERGLOBAL_procedure_set},
{"LibreDWGc::Dwg_Object_RENDERGLOBAL_procedure_get", _wrap_Dwg_Object_RENDERGLOBAL_procedure_get},
{"LibreDWGc::Dwg_Object_RENDERGLOBAL_destination_set", _wrap_Dwg_Object_RENDERGLOBAL_destination_set},
{"LibreDWGc::Dwg_Object_RENDERGLOBAL_destination_get", _wrap_Dwg_Object_RENDERGLOBAL_destination_get},
{"LibreDWGc::Dwg_Object_RENDERGLOBAL_save_enabled_set", _wrap_Dwg_Object_RENDERGLOBAL_save_enabled_set},
{"LibreDWGc::Dwg_Object_RENDERGLOBAL_save_enabled_get", _wrap_Dwg_Object_RENDERGLOBAL_save_enabled_get},
{"LibreDWGc::Dwg_Object_RENDERGLOBAL_save_filename_set", _wrap_Dwg_Object_RENDERGLOBAL_save_filename_set},
{"LibreDWGc::Dwg_Object_RENDERGLOBAL_save_filename_get", _wrap_Dwg_Object_RENDERGLOBAL_save_filename_get},
{"LibreDWGc::Dwg_Object_RENDERGLOBAL_image_width_set", _wrap_Dwg_Object_RENDERGLOBAL_image_width_set},
{"LibreDWGc::Dwg_Object_RENDERGLOBAL_image_width_get", _wrap_Dwg_Object_RENDERGLOBAL_image_width_get},
{"LibreDWGc::Dwg_Object_RENDERGLOBAL_image_height_set", _wrap_Dwg_Object_RENDERGLOBAL_image_height_set},
{"LibreDWGc::Dwg_Object_RENDERGLOBAL_image_height_get", _wrap_Dwg_Object_RENDERGLOBAL_image_height_get},
{"LibreDWGc::Dwg_Object_RENDERGLOBAL_predef_presets_first_set", _wrap_Dwg_Object_RENDERGLOBAL_predef_presets_first_set},
{"LibreDWGc::Dwg_Object_RENDERGLOBAL_predef_presets_first_get", _wrap_Dwg_Object_RENDERGLOBAL_predef_presets_first_get},
{"LibreDWGc::Dwg_Object_RENDERGLOBAL_highlevel_info_set", _wrap_Dwg_Object_RENDERGLOBAL_highlevel_info_set},
{"LibreDWGc::Dwg_Object_RENDERGLOBAL_highlevel_info_get", _wrap_Dwg_Object_RENDERGLOBAL_highlevel_info_get},
{"LibreDWGc::new_Dwg_Object_RENDERGLOBAL", _wrap_new_Dwg_Object_RENDERGLOBAL},
{"LibreDWGc::delete_Dwg_Object_RENDERGLOBAL", _wrap_delete_Dwg_Object_RENDERGLOBAL},
{"LibreDWGc::Dwg_Object_RENDERENTRY_parent_set", _wrap_Dwg_Object_RENDERENTRY_parent_set},
{"LibreDWGc::Dwg_Object_RENDERENTRY_parent_get", _wrap_Dwg_Object_RENDERENTRY_parent_get},
{"LibreDWGc::Dwg_Object_RENDERENTRY_class_version_set", _wrap_Dwg_Object_RENDERENTRY_class_version_set},
{"LibreDWGc::Dwg_Object_RENDERENTRY_class_version_get", _wrap_Dwg_Object_RENDERENTRY_class_version_get},
{"LibreDWGc::Dwg_Object_RENDERENTRY_image_file_name_set", _wrap_Dwg_Object_RENDERENTRY_image_file_name_set},
{"LibreDWGc::Dwg_Object_RENDERENTRY_image_file_name_get", _wrap_Dwg_Object_RENDERENTRY_image_file_name_get},
{"LibreDWGc::Dwg_Object_RENDERENTRY_preset_name_set", _wrap_Dwg_Object_RENDERENTRY_preset_name_set},
{"LibreDWGc::Dwg_Object_RENDERENTRY_preset_name_get", _wrap_Dwg_Object_RENDERENTRY_preset_name_get},
{"LibreDWGc::Dwg_Object_RENDERENTRY_view_name_set", _wrap_Dwg_Object_RENDERENTRY_view_name_set},
{"LibreDWGc::Dwg_Object_RENDERENTRY_view_name_get", _wrap_Dwg_Object_RENDERENTRY_view_name_get},
{"LibreDWGc::Dwg_Object_RENDERENTRY_dimension_x_set", _wrap_Dwg_Object_RENDERENTRY_dimension_x_set},
{"LibreDWGc::Dwg_Object_RENDERENTRY_dimension_x_get", _wrap_Dwg_Object_RENDERENTRY_dimension_x_get},
{"LibreDWGc::Dwg_Object_RENDERENTRY_dimension_y_set", _wrap_Dwg_Object_RENDERENTRY_dimension_y_set},
{"LibreDWGc::Dwg_Object_RENDERENTRY_dimension_y_get", _wrap_Dwg_Object_RENDERENTRY_dimension_y_get},
{"LibreDWGc::Dwg_Object_RENDERENTRY_start_year_set", _wrap_Dwg_Object_RENDERENTRY_start_year_set},
{"LibreDWGc::Dwg_Object_RENDERENTRY_start_year_get", _wrap_Dwg_Object_RENDERENTRY_start_year_get},
{"LibreDWGc::Dwg_Object_RENDERENTRY_start_month_set", _wrap_Dwg_Object_RENDERENTRY_start_month_set},
{"LibreDWGc::Dwg_Object_RENDERENTRY_start_month_get", _wrap_Dwg_Object_RENDERENTRY_start_month_get},
{"LibreDWGc::Dwg_Object_RENDERENTRY_start_day_set", _wrap_Dwg_Object_RENDERENTRY_start_day_set},
{"LibreDWGc::Dwg_Object_RENDERENTRY_start_day_get", _wrap_Dwg_Object_RENDERENTRY_start_day_get},
{"LibreDWGc::Dwg_Object_RENDERENTRY_start_minute_set", _wrap_Dwg_Object_RENDERENTRY_start_minute_set},
{"LibreDWGc::Dwg_Object_RENDERENTRY_start_minute_get", _wrap_Dwg_Object_RENDERENTRY_start_minute_get},
{"LibreDWGc::Dwg_Object_RENDERENTRY_start_second_set", _wrap_Dwg_Object_RENDERENTRY_start_second_set},
{"LibreDWGc::Dwg_Object_RENDERENTRY_start_second_get", _wrap_Dwg_Object_RENDERENTRY_start_second_get},
{"LibreDWGc::Dwg_Object_RENDERENTRY_start_msec_set", _wrap_Dwg_Object_RENDERENTRY_start_msec_set},
{"LibreDWGc::Dwg_Object_RENDERENTRY_start_msec_get", _wrap_Dwg_Object_RENDERENTRY_start_msec_get},
{"LibreDWGc::Dwg_Object_RENDERENTRY_render_time_set", _wrap_Dwg_Object_RENDERENTRY_render_time_set},
{"LibreDWGc::Dwg_Object_RENDERENTRY_render_time_get", _wrap_Dwg_Object_RENDERENTRY_render_time_get},
{"LibreDWGc::Dwg_Object_RENDERENTRY_memory_amount_set", _wrap_Dwg_Object_RENDERENTRY_memory_amount_set},
{"LibreDWGc::Dwg_Object_RENDERENTRY_memory_amount_get", _wrap_Dwg_Object_RENDERENTRY_memory_amount_get},
{"LibreDWGc::Dwg_Object_RENDERENTRY_material_count_set", _wrap_Dwg_Object_RENDERENTRY_material_count_set},
{"LibreDWGc::Dwg_Object_RENDERENTRY_material_count_get", _wrap_Dwg_Object_RENDERENTRY_material_count_get},
{"LibreDWGc::Dwg_Object_RENDERENTRY_light_count_set", _wrap_Dwg_Object_RENDERENTRY_light_count_set},
{"LibreDWGc::Dwg_Object_RENDERENTRY_light_count_get", _wrap_Dwg_Object_RENDERENTRY_light_count_get},
{"LibreDWGc::Dwg_Object_RENDERENTRY_triangle_count_set", _wrap_Dwg_Object_RENDERENTRY_triangle_count_set},
{"LibreDWGc::Dwg_Object_RENDERENTRY_triangle_count_get", _wrap_Dwg_Object_RENDERENTRY_triangle_count_get},
{"LibreDWGc::Dwg_Object_RENDERENTRY_display_index_set", _wrap_Dwg_Object_RENDERENTRY_display_index_set},
{"LibreDWGc::Dwg_Object_RENDERENTRY_display_index_get", _wrap_Dwg_Object_RENDERENTRY_display_index_get},
{"LibreDWGc::new_Dwg_Object_RENDERENTRY", _wrap_new_Dwg_Object_RENDERENTRY},
{"LibreDWGc::delete_Dwg_Object_RENDERENTRY", _wrap_delete_Dwg_Object_RENDERENTRY},
{"LibreDWGc::Dwg_Object_MOTIONPATH_parent_set", _wrap_Dwg_Object_MOTIONPATH_parent_set},
{"LibreDWGc::Dwg_Object_MOTIONPATH_parent_get", _wrap_Dwg_Object_MOTIONPATH_parent_get},
{"LibreDWGc::Dwg_Object_MOTIONPATH_class_version_set", _wrap_Dwg_Object_MOTIONPATH_class_version_set},
{"LibreDWGc::Dwg_Object_MOTIONPATH_class_version_get", _wrap_Dwg_Object_MOTIONPATH_class_version_get},
{"LibreDWGc::Dwg_Object_MOTIONPATH_camera_path_set", _wrap_Dwg_Object_MOTIONPATH_camera_path_set},
{"LibreDWGc::Dwg_Object_MOTIONPATH_camera_path_get", _wrap_Dwg_Object_MOTIONPATH_camera_path_get},
{"LibreDWGc::Dwg_Object_MOTIONPATH_target_path_set", _wrap_Dwg_Object_MOTIONPATH_target_path_set},
{"LibreDWGc::Dwg_Object_MOTIONPATH_target_path_get", _wrap_Dwg_Object_MOTIONPATH_target_path_get},
{"LibreDWGc::Dwg_Object_MOTIONPATH_viewtable_set", _wrap_Dwg_Object_MOTIONPATH_viewtable_set},
{"LibreDWGc::Dwg_Object_MOTIONPATH_viewtable_get", _wrap_Dwg_Object_MOTIONPATH_viewtable_get},
{"LibreDWGc::Dwg_Object_MOTIONPATH_frames_set", _wrap_Dwg_Object_MOTIONPATH_frames_set},
{"LibreDWGc::Dwg_Object_MOTIONPATH_frames_get", _wrap_Dwg_Object_MOTIONPATH_frames_get},
{"LibreDWGc::Dwg_Object_MOTIONPATH_frame_rate_set", _wrap_Dwg_Object_MOTIONPATH_frame_rate_set},
{"LibreDWGc::Dwg_Object_MOTIONPATH_frame_rate_get", _wrap_Dwg_Object_MOTIONPATH_frame_rate_get},
{"LibreDWGc::Dwg_Object_MOTIONPATH_corner_decel_set", _wrap_Dwg_Object_MOTIONPATH_corner_decel_set},
{"LibreDWGc::Dwg_Object_MOTIONPATH_corner_decel_get", _wrap_Dwg_Object_MOTIONPATH_corner_decel_get},
{"LibreDWGc::new_Dwg_Object_MOTIONPATH", _wrap_new_Dwg_Object_MOTIONPATH},
{"LibreDWGc::delete_Dwg_Object_MOTIONPATH", _wrap_delete_Dwg_Object_MOTIONPATH},
{"LibreDWGc::Dwg_Object_CURVEPATH_parent_set", _wrap_Dwg_Object_CURVEPATH_parent_set},
{"LibreDWGc::Dwg_Object_CURVEPATH_parent_get", _wrap_Dwg_Object_CURVEPATH_parent_get},
{"LibreDWGc::Dwg_Object_CURVEPATH_class_version_set", _wrap_Dwg_Object_CURVEPATH_class_version_set},
{"LibreDWGc::Dwg_Object_CURVEPATH_class_version_get", _wrap_Dwg_Object_CURVEPATH_class_version_get},
{"LibreDWGc::Dwg_Object_CURVEPATH_entity_set", _wrap_Dwg_Object_CURVEPATH_entity_set},
{"LibreDWGc::Dwg_Object_CURVEPATH_entity_get", _wrap_Dwg_Object_CURVEPATH_entity_get},
{"LibreDWGc::new_Dwg_Object_CURVEPATH", _wrap_new_Dwg_Object_CURVEPATH},
{"LibreDWGc::delete_Dwg_Object_CURVEPATH", _wrap_delete_Dwg_Object_CURVEPATH},
{"LibreDWGc::Dwg_Object_POINTPATH_parent_set", _wrap_Dwg_Object_POINTPATH_parent_set},
{"LibreDWGc::Dwg_Object_POINTPATH_parent_get", _wrap_Dwg_Object_POINTPATH_parent_get},
{"LibreDWGc::Dwg_Object_POINTPATH_class_version_set", _wrap_Dwg_Object_POINTPATH_class_version_set},
{"LibreDWGc::Dwg_Object_POINTPATH_class_version_get", _wrap_Dwg_Object_POINTPATH_class_version_get},
{"LibreDWGc::Dwg_Object_POINTPATH_point_set", _wrap_Dwg_Object_POINTPATH_point_set},
{"LibreDWGc::Dwg_Object_POINTPATH_point_get", _wrap_Dwg_Object_POINTPATH_point_get},
{"LibreDWGc::new_Dwg_Object_POINTPATH", _wrap_new_Dwg_Object_POINTPATH},
{"LibreDWGc::delete_Dwg_Object_POINTPATH", _wrap_delete_Dwg_Object_POINTPATH},
{"LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_parent_set", _wrap_Dwg_Object_TVDEVICEPROPERTIES_parent_set},
{"LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_parent_get", _wrap_Dwg_Object_TVDEVICEPROPERTIES_parent_get},
{"LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_flags_set", _wrap_Dwg_Object_TVDEVICEPROPERTIES_flags_set},
{"LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_flags_get", _wrap_Dwg_Object_TVDEVICEPROPERTIES_flags_get},
{"LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_max_regen_threads_set", _wrap_Dwg_Object_TVDEVICEPROPERTIES_max_regen_threads_set},
{"LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_max_regen_threads_get", _wrap_Dwg_Object_TVDEVICEPROPERTIES_max_regen_threads_get},
{"LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_use_lut_palette_set", _wrap_Dwg_Object_TVDEVICEPROPERTIES_use_lut_palette_set},
{"LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_use_lut_palette_get", _wrap_Dwg_Object_TVDEVICEPROPERTIES_use_lut_palette_get},
{"LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_alt_hlt_set", _wrap_Dwg_Object_TVDEVICEPROPERTIES_alt_hlt_set},
{"LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_alt_hlt_get", _wrap_Dwg_Object_TVDEVICEPROPERTIES_alt_hlt_get},
{"LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_alt_hltcolor_set", _wrap_Dwg_Object_TVDEVICEPROPERTIES_alt_hltcolor_set},
{"LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_alt_hltcolor_get", _wrap_Dwg_Object_TVDEVICEPROPERTIES_alt_hltcolor_get},
{"LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_geom_shader_usage_set", _wrap_Dwg_Object_TVDEVICEPROPERTIES_geom_shader_usage_set},
{"LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_geom_shader_usage_get", _wrap_Dwg_Object_TVDEVICEPROPERTIES_geom_shader_usage_get},
{"LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_blending_mode_set", _wrap_Dwg_Object_TVDEVICEPROPERTIES_blending_mode_set},
{"LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_blending_mode_get", _wrap_Dwg_Object_TVDEVICEPROPERTIES_blending_mode_get},
{"LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_antialiasing_level_set", _wrap_Dwg_Object_TVDEVICEPROPERTIES_antialiasing_level_set},
{"LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_antialiasing_level_get", _wrap_Dwg_Object_TVDEVICEPROPERTIES_antialiasing_level_get},
{"LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_bd2_set", _wrap_Dwg_Object_TVDEVICEPROPERTIES_bd2_set},
{"LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_bd2_get", _wrap_Dwg_Object_TVDEVICEPROPERTIES_bd2_get},
{"LibreDWGc::new_Dwg_Object_TVDEVICEPROPERTIES", _wrap_new_Dwg_Object_TVDEVICEPROPERTIES},
{"LibreDWGc::delete_Dwg_Object_TVDEVICEPROPERTIES", _wrap_delete_Dwg_Object_TVDEVICEPROPERTIES},
{"LibreDWGc::Dwg_Object_SKYLIGHT_BACKGROUND_parent_set", _wrap_Dwg_Object_SKYLIGHT_BACKGROUND_parent_set},
{"LibreDWGc::Dwg_Object_SKYLIGHT_BACKGROUND_parent_get", _wrap_Dwg_Object_SKYLIGHT_BACKGROUND_parent_get},
{"LibreDWGc::Dwg_Object_SKYLIGHT_BACKGROUND_class_version_set", _wrap_Dwg_Object_SKYLIGHT_BACKGROUND_class_version_set},
{"LibreDWGc::Dwg_Object_SKYLIGHT_BACKGROUND_class_version_get", _wrap_Dwg_Object_SKYLIGHT_BACKGROUND_class_version_get},
{"LibreDWGc::Dwg_Object_SKYLIGHT_BACKGROUND_sunid_set", _wrap_Dwg_Object_SKYLIGHT_BACKGROUND_sunid_set},
{"LibreDWGc::Dwg_Object_SKYLIGHT_BACKGROUND_sunid_get", _wrap_Dwg_Object_SKYLIGHT_BACKGROUND_sunid_get},
{"LibreDWGc::new_Dwg_Object_SKYLIGHT_BACKGROUND", _wrap_new_Dwg_Object_SKYLIGHT_BACKGROUND},
{"LibreDWGc::delete_Dwg_Object_SKYLIGHT_BACKGROUND", _wrap_delete_Dwg_Object_SKYLIGHT_BACKGROUND},
{"LibreDWGc::Dwg_Object_SOLID_BACKGROUND_parent_set", _wrap_Dwg_Object_SOLID_BACKGROUND_parent_set},
{"LibreDWGc::Dwg_Object_SOLID_BACKGROUND_parent_get", _wrap_Dwg_Object_SOLID_BACKGROUND_parent_get},
{"LibreDWGc::Dwg_Object_SOLID_BACKGROUND_class_version_set", _wrap_Dwg_Object_SOLID_BACKGROUND_class_version_set},
{"LibreDWGc::Dwg_Object_SOLID_BACKGROUND_class_version_get", _wrap_Dwg_Object_SOLID_BACKGROUND_class_version_get},
{"LibreDWGc::Dwg_Object_SOLID_BACKGROUND_color_set", _wrap_Dwg_Object_SOLID_BACKGROUND_color_set},
{"LibreDWGc::Dwg_Object_SOLID_BACKGROUND_color_get", _wrap_Dwg_Object_SOLID_BACKGROUND_color_get},
{"LibreDWGc::new_Dwg_Object_SOLID_BACKGROUND", _wrap_new_Dwg_Object_SOLID_BACKGROUND},
{"LibreDWGc::delete_Dwg_Object_SOLID_BACKGROUND", _wrap_delete_Dwg_Object_SOLID_BACKGROUND},
{"LibreDWGc::Dwg_Object_IMAGE_BACKGROUND_parent_set", _wrap_Dwg_Object_IMAGE_BACKGROUND_parent_set},
{"LibreDWGc::Dwg_Object_IMAGE_BACKGROUND_parent_get", _wrap_Dwg_Object_IMAGE_BACKGROUND_parent_get},
{"LibreDWGc::Dwg_Object_IMAGE_BACKGROUND_class_version_set", _wrap_Dwg_Object_IMAGE_BACKGROUND_class_version_set},
{"LibreDWGc::Dwg_Object_IMAGE_BACKGROUND_class_version_get", _wrap_Dwg_Object_IMAGE_BACKGROUND_class_version_get},
{"LibreDWGc::Dwg_Object_IMAGE_BACKGROUND_filename_set", _wrap_Dwg_Object_IMAGE_BACKGROUND_filename_set},
{"LibreDWGc::Dwg_Object_IMAGE_BACKGROUND_filename_get", _wrap_Dwg_Object_IMAGE_BACKGROUND_filename_get},
{"LibreDWGc::Dwg_Object_IMAGE_BACKGROUND_fit_to_screen_set", _wrap_Dwg_Object_IMAGE_BACKGROUND_fit_to_screen_set},
{"LibreDWGc::Dwg_Object_IMAGE_BACKGROUND_fit_to_screen_get", _wrap_Dwg_Object_IMAGE_BACKGROUND_fit_to_screen_get},
{"LibreDWGc::Dwg_Object_IMAGE_BACKGROUND_maintain_aspect_ratio_set", _wrap_Dwg_Object_IMAGE_BACKGROUND_maintain_aspect_ratio_set},
{"LibreDWGc::Dwg_Object_IMAGE_BACKGROUND_maintain_aspect_ratio_get", _wrap_Dwg_Object_IMAGE_BACKGROUND_maintain_aspect_ratio_get},
{"LibreDWGc::Dwg_Object_IMAGE_BACKGROUND_use_tiling_set", _wrap_Dwg_Object_IMAGE_BACKGROUND_use_tiling_set},
{"LibreDWGc::Dwg_Object_IMAGE_BACKGROUND_use_tiling_get", _wrap_Dwg_Object_IMAGE_BACKGROUND_use_tiling_get},
{"LibreDWGc::Dwg_Object_IMAGE_BACKGROUND_offset_set", _wrap_Dwg_Object_IMAGE_BACKGROUND_offset_set},
{"LibreDWGc::Dwg_Object_IMAGE_BACKGROUND_offset_get", _wrap_Dwg_Object_IMAGE_BACKGROUND_offset_get},
{"LibreDWGc::Dwg_Object_IMAGE_BACKGROUND_scale_set", _wrap_Dwg_Object_IMAGE_BACKGROUND_scale_set},
{"LibreDWGc::Dwg_Object_IMAGE_BACKGROUND_scale_get", _wrap_Dwg_Object_IMAGE_BACKGROUND_scale_get},
{"LibreDWGc::new_Dwg_Object_IMAGE_BACKGROUND", _wrap_new_Dwg_Object_IMAGE_BACKGROUND},
{"LibreDWGc::delete_Dwg_Object_IMAGE_BACKGROUND", _wrap_delete_Dwg_Object_IMAGE_BACKGROUND},
{"LibreDWGc::Dwg_Object_IBL_BACKGROUND_parent_set", _wrap_Dwg_Object_IBL_BACKGROUND_parent_set},
{"LibreDWGc::Dwg_Object_IBL_BACKGROUND_parent_get", _wrap_Dwg_Object_IBL_BACKGROUND_parent_get},
{"LibreDWGc::Dwg_Object_IBL_BACKGROUND_class_version_set", _wrap_Dwg_Object_IBL_BACKGROUND_class_version_set},
{"LibreDWGc::Dwg_Object_IBL_BACKGROUND_class_version_get", _wrap_Dwg_Object_IBL_BACKGROUND_class_version_get},
{"LibreDWGc::Dwg_Object_IBL_BACKGROUND_enable_set", _wrap_Dwg_Object_IBL_BACKGROUND_enable_set},
{"LibreDWGc::Dwg_Object_IBL_BACKGROUND_enable_get", _wrap_Dwg_Object_IBL_BACKGROUND_enable_get},
{"LibreDWGc::Dwg_Object_IBL_BACKGROUND_name_set", _wrap_Dwg_Object_IBL_BACKGROUND_name_set},
{"LibreDWGc::Dwg_Object_IBL_BACKGROUND_name_get", _wrap_Dwg_Object_IBL_BACKGROUND_name_get},
{"LibreDWGc::Dwg_Object_IBL_BACKGROUND_rotation_set", _wrap_Dwg_Object_IBL_BACKGROUND_rotation_set},
{"LibreDWGc::Dwg_Object_IBL_BACKGROUND_rotation_get", _wrap_Dwg_Object_IBL_BACKGROUND_rotation_get},
{"LibreDWGc::Dwg_Object_IBL_BACKGROUND_display_image_set", _wrap_Dwg_Object_IBL_BACKGROUND_display_image_set},
{"LibreDWGc::Dwg_Object_IBL_BACKGROUND_display_image_get", _wrap_Dwg_Object_IBL_BACKGROUND_display_image_get},
{"LibreDWGc::Dwg_Object_IBL_BACKGROUND_secondary_background_set", _wrap_Dwg_Object_IBL_BACKGROUND_secondary_background_set},
{"LibreDWGc::Dwg_Object_IBL_BACKGROUND_secondary_background_get", _wrap_Dwg_Object_IBL_BACKGROUND_secondary_background_get},
{"LibreDWGc::new_Dwg_Object_IBL_BACKGROUND", _wrap_new_Dwg_Object_IBL_BACKGROUND},
{"LibreDWGc::delete_Dwg_Object_IBL_BACKGROUND", _wrap_delete_Dwg_Object_IBL_BACKGROUND},
{"LibreDWGc::Dwg_Object_GRADIENT_BACKGROUND_parent_set", _wrap_Dwg_Object_GRADIENT_BACKGROUND_parent_set},
{"LibreDWGc::Dwg_Object_GRADIENT_BACKGROUND_parent_get", _wrap_Dwg_Object_GRADIENT_BACKGROUND_parent_get},
{"LibreDWGc::Dwg_Object_GRADIENT_BACKGROUND_class_version_set", _wrap_Dwg_Object_GRADIENT_BACKGROUND_class_version_set},
{"LibreDWGc::Dwg_Object_GRADIENT_BACKGROUND_class_version_get", _wrap_Dwg_Object_GRADIENT_BACKGROUND_class_version_get},
{"LibreDWGc::Dwg_Object_GRADIENT_BACKGROUND_color_top_set", _wrap_Dwg_Object_GRADIENT_BACKGROUND_color_top_set},
{"LibreDWGc::Dwg_Object_GRADIENT_BACKGROUND_color_top_get", _wrap_Dwg_Object_GRADIENT_BACKGROUND_color_top_get},
{"LibreDWGc::Dwg_Object_GRADIENT_BACKGROUND_color_middle_set", _wrap_Dwg_Object_GRADIENT_BACKGROUND_color_middle_set},
{"LibreDWGc::Dwg_Object_GRADIENT_BACKGROUND_color_middle_get", _wrap_Dwg_Object_GRADIENT_BACKGROUND_color_middle_get},
{"LibreDWGc::Dwg_Object_GRADIENT_BACKGROUND_color_bottom_set", _wrap_Dwg_Object_GRADIENT_BACKGROUND_color_bottom_set},
{"LibreDWGc::Dwg_Object_GRADIENT_BACKGROUND_color_bottom_get", _wrap_Dwg_Object_GRADIENT_BACKGROUND_color_bottom_get},
{"LibreDWGc::Dwg_Object_GRADIENT_BACKGROUND_horizon_set", _wrap_Dwg_Object_GRADIENT_BACKGROUND_horizon_set},
{"LibreDWGc::Dwg_Object_GRADIENT_BACKGROUND_horizon_get", _wrap_Dwg_Object_GRADIENT_BACKGROUND_horizon_get},
{"LibreDWGc::Dwg_Object_GRADIENT_BACKGROUND_height_set", _wrap_Dwg_Object_GRADIENT_BACKGROUND_height_set},
{"LibreDWGc::Dwg_Object_GRADIENT_BACKGROUND_height_get", _wrap_Dwg_Object_GRADIENT_BACKGROUND_height_get},
{"LibreDWGc::Dwg_Object_GRADIENT_BACKGROUND_rotation_set", _wrap_Dwg_Object_GRADIENT_BACKGROUND_rotation_set},
{"LibreDWGc::Dwg_Object_GRADIENT_BACKGROUND_rotation_get", _wrap_Dwg_Object_GRADIENT_BACKGROUND_rotation_get},
{"LibreDWGc::new_Dwg_Object_GRADIENT_BACKGROUND", _wrap_new_Dwg_Object_GRADIENT_BACKGROUND},
{"LibreDWGc::delete_Dwg_Object_GRADIENT_BACKGROUND", _wrap_delete_Dwg_Object_GRADIENT_BACKGROUND},
{"LibreDWGc::Dwg_Object_GROUND_PLANE_BACKGROUND_parent_set", _wrap_Dwg_Object_GROUND_PLANE_BACKGROUND_parent_set},
{"LibreDWGc::Dwg_Object_GROUND_PLANE_BACKGROUND_parent_get", _wrap_Dwg_Object_GROUND_PLANE_BACKGROUND_parent_get},
{"LibreDWGc::Dwg_Object_GROUND_PLANE_BACKGROUND_class_version_set", _wrap_Dwg_Object_GROUND_PLANE_BACKGROUND_class_version_set},
{"LibreDWGc::Dwg_Object_GROUND_PLANE_BACKGROUND_class_version_get", _wrap_Dwg_Object_GROUND_PLANE_BACKGROUND_class_version_get},
{"LibreDWGc::Dwg_Object_GROUND_PLANE_BACKGROUND_color_sky_zenith_set", _wrap_Dwg_Object_GROUND_PLANE_BACKGROUND_color_sky_zenith_set},
{"LibreDWGc::Dwg_Object_GROUND_PLANE_BACKGROUND_color_sky_zenith_get", _wrap_Dwg_Object_GROUND_PLANE_BACKGROUND_color_sky_zenith_get},
{"LibreDWGc::Dwg_Object_GROUND_PLANE_BACKGROUND_color_sky_horizon_set", _wrap_Dwg_Object_GROUND_PLANE_BACKGROUND_color_sky_horizon_set},
{"LibreDWGc::Dwg_Object_GROUND_PLANE_BACKGROUND_color_sky_horizon_get", _wrap_Dwg_Object_GROUND_PLANE_BACKGROUND_color_sky_horizon_get},
{"LibreDWGc::Dwg_Object_GROUND_PLANE_BACKGROUND_color_underground_horizon_set", _wrap_Dwg_Object_GROUND_PLANE_BACKGROUND_color_underground_horizon_set},
{"LibreDWGc::Dwg_Object_GROUND_PLANE_BACKGROUND_color_underground_horizon_get", _wrap_Dwg_Object_GROUND_PLANE_BACKGROUND_color_underground_horizon_get},
{"LibreDWGc::Dwg_Object_GROUND_PLANE_BACKGROUND_color_underground_azimuth_set", _wrap_Dwg_Object_GROUND_PLANE_BACKGROUND_color_underground_azimuth_set},
{"LibreDWGc::Dwg_Object_GROUND_PLANE_BACKGROUND_color_underground_azimuth_get", _wrap_Dwg_Object_GROUND_PLANE_BACKGROUND_color_underground_azimuth_get},
{"LibreDWGc::Dwg_Object_GROUND_PLANE_BACKGROUND_color_near_set", _wrap_Dwg_Object_GROUND_PLANE_BACKGROUND_color_near_set},
{"LibreDWGc::Dwg_Object_GROUND_PLANE_BACKGROUND_color_near_get", _wrap_Dwg_Object_GROUND_PLANE_BACKGROUND_color_near_get},
{"LibreDWGc::Dwg_Object_GROUND_PLANE_BACKGROUND_color_far_set", _wrap_Dwg_Object_GROUND_PLANE_BACKGROUND_color_far_set},
{"LibreDWGc::Dwg_Object_GROUND_PLANE_BACKGROUND_color_far_get", _wrap_Dwg_Object_GROUND_PLANE_BACKGROUND_color_far_get},
{"LibreDWGc::new_Dwg_Object_GROUND_PLANE_BACKGROUND", _wrap_new_Dwg_Object_GROUND_PLANE_BACKGROUND},
{"LibreDWGc::delete_Dwg_Object_GROUND_PLANE_BACKGROUND", _wrap_delete_Dwg_Object_GROUND_PLANE_BACKGROUND},
{"LibreDWGc::Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_parent_set", _wrap_Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_parent_set},
{"LibreDWGc::Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_parent_get", _wrap_Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_parent_get},
{"LibreDWGc::Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_class_version_set", _wrap_Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_class_version_set},
{"LibreDWGc::Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_class_version_get", _wrap_Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_class_version_get},
{"LibreDWGc::Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_is_default_set", _wrap_Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_is_default_set},
{"LibreDWGc::Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_is_default_get", _wrap_Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_is_default_get},
{"LibreDWGc::Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_scale_set", _wrap_Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_scale_set},
{"LibreDWGc::Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_scale_get", _wrap_Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_scale_get},
{"LibreDWGc::new_Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA", _wrap_new_Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA},
{"LibreDWGc::delete_Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA", _wrap_delete_Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA},
{"LibreDWGc::Dwg_CONTEXTDATA_dict_parent_set", _wrap_Dwg_CONTEXTDATA_dict_parent_set},
{"LibreDWGc::Dwg_CONTEXTDATA_dict_parent_get", _wrap_Dwg_CONTEXTDATA_dict_parent_get},
{"LibreDWGc::Dwg_CONTEXTDATA_dict_text_set", _wrap_Dwg_CONTEXTDATA_dict_text_set},
{"LibreDWGc::Dwg_CONTEXTDATA_dict_text_get", _wrap_Dwg_CONTEXTDATA_dict_text_get},
{"LibreDWGc::Dwg_CONTEXTDATA_dict_itemhandle_set", _wrap_Dwg_CONTEXTDATA_dict_itemhandle_set},
{"LibreDWGc::Dwg_CONTEXTDATA_dict_itemhandle_get", _wrap_Dwg_CONTEXTDATA_dict_itemhandle_get},
{"LibreDWGc::new_Dwg_CONTEXTDATA_dict", _wrap_new_Dwg_CONTEXTDATA_dict},
{"LibreDWGc::delete_Dwg_CONTEXTDATA_dict", _wrap_delete_Dwg_CONTEXTDATA_dict},
{"LibreDWGc::Dwg_CONTEXTDATA_submgr_parent_set", _wrap_Dwg_CONTEXTDATA_submgr_parent_set},
{"LibreDWGc::Dwg_CONTEXTDATA_submgr_parent_get", _wrap_Dwg_CONTEXTDATA_submgr_parent_get},
{"LibreDWGc::Dwg_CONTEXTDATA_submgr_handle_set", _wrap_Dwg_CONTEXTDATA_submgr_handle_set},
{"LibreDWGc::Dwg_CONTEXTDATA_submgr_handle_get", _wrap_Dwg_CONTEXTDATA_submgr_handle_get},
{"LibreDWGc::Dwg_CONTEXTDATA_submgr_num_entries_set", _wrap_Dwg_CONTEXTDATA_submgr_num_entries_set},
{"LibreDWGc::Dwg_CONTEXTDATA_submgr_num_entries_get", _wrap_Dwg_CONTEXTDATA_submgr_num_entries_get},
{"LibreDWGc::Dwg_CONTEXTDATA_submgr_entries_set", _wrap_Dwg_CONTEXTDATA_submgr_entries_set},
{"LibreDWGc::Dwg_CONTEXTDATA_submgr_entries_get", _wrap_Dwg_CONTEXTDATA_submgr_entries_get},
{"LibreDWGc::new_Dwg_CONTEXTDATA_submgr", _wrap_new_Dwg_CONTEXTDATA_submgr},
{"LibreDWGc::delete_Dwg_CONTEXTDATA_submgr", _wrap_delete_Dwg_CONTEXTDATA_submgr},
{"LibreDWGc::Dwg_Object_CONTEXTDATAMANAGER_parent_set", _wrap_Dwg_Object_CONTEXTDATAMANAGER_parent_set},
{"LibreDWGc::Dwg_Object_CONTEXTDATAMANAGER_parent_get", _wrap_Dwg_Object_CONTEXTDATAMANAGER_parent_get},
{"LibreDWGc::Dwg_Object_CONTEXTDATAMANAGER_objectcontext_set", _wrap_Dwg_Object_CONTEXTDATAMANAGER_objectcontext_set},
{"LibreDWGc::Dwg_Object_CONTEXTDATAMANAGER_objectcontext_get", _wrap_Dwg_Object_CONTEXTDATAMANAGER_objectcontext_get},
{"LibreDWGc::Dwg_Object_CONTEXTDATAMANAGER_num_submgrs_set", _wrap_Dwg_Object_CONTEXTDATAMANAGER_num_submgrs_set},
{"LibreDWGc::Dwg_Object_CONTEXTDATAMANAGER_num_submgrs_get", _wrap_Dwg_Object_CONTEXTDATAMANAGER_num_submgrs_get},
{"LibreDWGc::Dwg_Object_CONTEXTDATAMANAGER_submgrs_set", _wrap_Dwg_Object_CONTEXTDATAMANAGER_submgrs_set},
{"LibreDWGc::Dwg_Object_CONTEXTDATAMANAGER_submgrs_get", _wrap_Dwg_Object_CONTEXTDATAMANAGER_submgrs_get},
{"LibreDWGc::new_Dwg_Object_CONTEXTDATAMANAGER", _wrap_new_Dwg_Object_CONTEXTDATAMANAGER},
{"LibreDWGc::delete_Dwg_Object_CONTEXTDATAMANAGER", _wrap_delete_Dwg_Object_CONTEXTDATAMANAGER},
{"LibreDWGc::Dwg_Object_TEXTOBJECTCONTEXTDATA_parent_set", _wrap_Dwg_Object_TEXTOBJECTCONTEXTDATA_parent_set},
{"LibreDWGc::Dwg_Object_TEXTOBJECTCONTEXTDATA_parent_get", _wrap_Dwg_Object_TEXTOBJECTCONTEXTDATA_parent_get},
{"LibreDWGc::Dwg_Object_TEXTOBJECTCONTEXTDATA_class_version_set", _wrap_Dwg_Object_TEXTOBJECTCONTEXTDATA_class_version_set},
{"LibreDWGc::Dwg_Object_TEXTOBJECTCONTEXTDATA_class_version_get", _wrap_Dwg_Object_TEXTOBJECTCONTEXTDATA_class_version_get},
{"LibreDWGc::Dwg_Object_TEXTOBJECTCONTEXTDATA_is_default_set", _wrap_Dwg_Object_TEXTOBJECTCONTEXTDATA_is_default_set},
{"LibreDWGc::Dwg_Object_TEXTOBJECTCONTEXTDATA_is_default_get", _wrap_Dwg_Object_TEXTOBJECTCONTEXTDATA_is_default_get},
{"LibreDWGc::Dwg_Object_TEXTOBJECTCONTEXTDATA_scale_set", _wrap_Dwg_Object_TEXTOBJECTCONTEXTDATA_scale_set},
{"LibreDWGc::Dwg_Object_TEXTOBJECTCONTEXTDATA_scale_get", _wrap_Dwg_Object_TEXTOBJECTCONTEXTDATA_scale_get},
{"LibreDWGc::Dwg_Object_TEXTOBJECTCONTEXTDATA_horizontal_mode_set", _wrap_Dwg_Object_TEXTOBJECTCONTEXTDATA_horizontal_mode_set},
{"LibreDWGc::Dwg_Object_TEXTOBJECTCONTEXTDATA_horizontal_mode_get", _wrap_Dwg_Object_TEXTOBJECTCONTEXTDATA_horizontal_mode_get},
{"LibreDWGc::Dwg_Object_TEXTOBJECTCONTEXTDATA_rotation_set", _wrap_Dwg_Object_TEXTOBJECTCONTEXTDATA_rotation_set},
{"LibreDWGc::Dwg_Object_TEXTOBJECTCONTEXTDATA_rotation_get", _wrap_Dwg_Object_TEXTOBJECTCONTEXTDATA_rotation_get},
{"LibreDWGc::Dwg_Object_TEXTOBJECTCONTEXTDATA_ins_pt_set", _wrap_Dwg_Object_TEXTOBJECTCONTEXTDATA_ins_pt_set},
{"LibreDWGc::Dwg_Object_TEXTOBJECTCONTEXTDATA_ins_pt_get", _wrap_Dwg_Object_TEXTOBJECTCONTEXTDATA_ins_pt_get},
{"LibreDWGc::Dwg_Object_TEXTOBJECTCONTEXTDATA_alignment_pt_set", _wrap_Dwg_Object_TEXTOBJECTCONTEXTDATA_alignment_pt_set},
{"LibreDWGc::Dwg_Object_TEXTOBJECTCONTEXTDATA_alignment_pt_get", _wrap_Dwg_Object_TEXTOBJECTCONTEXTDATA_alignment_pt_get},
{"LibreDWGc::new_Dwg_Object_TEXTOBJECTCONTEXTDATA", _wrap_new_Dwg_Object_TEXTOBJECTCONTEXTDATA},
{"LibreDWGc::delete_Dwg_Object_TEXTOBJECTCONTEXTDATA", _wrap_delete_Dwg_Object_TEXTOBJECTCONTEXTDATA},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_parent_set", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_parent_set},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_parent_get", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_parent_get},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_class_version_set", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_class_version_set},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_class_version_get", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_class_version_get},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_is_default_set", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_is_default_set},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_is_default_get", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_is_default_get},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_scale_set", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_scale_set},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_scale_get", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_scale_get},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_attachment_set", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_attachment_set},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_attachment_get", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_attachment_get},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_ins_pt_set", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_ins_pt_set},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_ins_pt_get", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_ins_pt_get},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_x_axis_dir_set", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_x_axis_dir_set},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_x_axis_dir_get", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_x_axis_dir_get},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_rect_height_set", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_rect_height_set},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_rect_height_get", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_rect_height_get},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_rect_width_set", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_rect_width_set},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_rect_width_get", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_rect_width_get},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_extents_width_set", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_extents_width_set},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_extents_width_get", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_extents_width_get},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_extents_height_set", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_extents_height_set},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_extents_height_get", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_extents_height_get},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_type_set", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_type_set},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_type_get", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_type_get},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_width_set", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_width_set},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_width_get", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_width_get},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_gutter_set", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_gutter_set},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_gutter_get", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_gutter_get},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_auto_height_set", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_auto_height_set},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_auto_height_get", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_auto_height_get},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_flow_reversed_set", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_flow_reversed_set},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_flow_reversed_get", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_flow_reversed_get},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_num_column_heights_set", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_num_column_heights_set},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_num_column_heights_get", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_num_column_heights_get},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_heights_set", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_heights_set},
{"LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_heights_get", _wrap_Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_heights_get},
{"LibreDWGc::new_Dwg_Object_MTEXTOBJECTCONTEXTDATA", _wrap_new_Dwg_Object_MTEXTOBJECTCONTEXTDATA},
{"LibreDWGc::delete_Dwg_Object_MTEXTOBJECTCONTEXTDATA", _wrap_delete_Dwg_Object_MTEXTOBJECTCONTEXTDATA},
{"LibreDWGc::Dwg_OCD_Dimension_b293_set", _wrap_Dwg_OCD_Dimension_b293_set},
{"LibreDWGc::Dwg_OCD_Dimension_b293_get", _wrap_Dwg_OCD_Dimension_b293_get},
{"LibreDWGc::Dwg_OCD_Dimension_def_pt_set", _wrap_Dwg_OCD_Dimension_def_pt_set},
{"LibreDWGc::Dwg_OCD_Dimension_def_pt_get", _wrap_Dwg_OCD_Dimension_def_pt_get},
{"LibreDWGc::Dwg_OCD_Dimension_is_def_textloc_set", _wrap_Dwg_OCD_Dimension_is_def_textloc_set},
{"LibreDWGc::Dwg_OCD_Dimension_is_def_textloc_get", _wrap_Dwg_OCD_Dimension_is_def_textloc_get},
{"LibreDWGc::Dwg_OCD_Dimension_text_rotation_set", _wrap_Dwg_OCD_Dimension_text_rotation_set},
{"LibreDWGc::Dwg_OCD_Dimension_text_rotation_get", _wrap_Dwg_OCD_Dimension_text_rotation_get},
{"LibreDWGc::Dwg_OCD_Dimension_block_set", _wrap_Dwg_OCD_Dimension_block_set},
{"LibreDWGc::Dwg_OCD_Dimension_block_get", _wrap_Dwg_OCD_Dimension_block_get},
{"LibreDWGc::Dwg_OCD_Dimension_dimtofl_set", _wrap_Dwg_OCD_Dimension_dimtofl_set},
{"LibreDWGc::Dwg_OCD_Dimension_dimtofl_get", _wrap_Dwg_OCD_Dimension_dimtofl_get},
{"LibreDWGc::Dwg_OCD_Dimension_dimosxd_set", _wrap_Dwg_OCD_Dimension_dimosxd_set},
{"LibreDWGc::Dwg_OCD_Dimension_dimosxd_get", _wrap_Dwg_OCD_Dimension_dimosxd_get},
{"LibreDWGc::Dwg_OCD_Dimension_dimatfit_set", _wrap_Dwg_OCD_Dimension_dimatfit_set},
{"LibreDWGc::Dwg_OCD_Dimension_dimatfit_get", _wrap_Dwg_OCD_Dimension_dimatfit_get},
{"LibreDWGc::Dwg_OCD_Dimension_dimtix_set", _wrap_Dwg_OCD_Dimension_dimtix_set},
{"LibreDWGc::Dwg_OCD_Dimension_dimtix_get", _wrap_Dwg_OCD_Dimension_dimtix_get},
{"LibreDWGc::Dwg_OCD_Dimension_dimtmove_set", _wrap_Dwg_OCD_Dimension_dimtmove_set},
{"LibreDWGc::Dwg_OCD_Dimension_dimtmove_get", _wrap_Dwg_OCD_Dimension_dimtmove_get},
{"LibreDWGc::Dwg_OCD_Dimension_override_code_set", _wrap_Dwg_OCD_Dimension_override_code_set},
{"LibreDWGc::Dwg_OCD_Dimension_override_code_get", _wrap_Dwg_OCD_Dimension_override_code_get},
{"LibreDWGc::Dwg_OCD_Dimension_has_arrow2_set", _wrap_Dwg_OCD_Dimension_has_arrow2_set},
{"LibreDWGc::Dwg_OCD_Dimension_has_arrow2_get", _wrap_Dwg_OCD_Dimension_has_arrow2_get},
{"LibreDWGc::Dwg_OCD_Dimension_flip_arrow2_set", _wrap_Dwg_OCD_Dimension_flip_arrow2_set},
{"LibreDWGc::Dwg_OCD_Dimension_flip_arrow2_get", _wrap_Dwg_OCD_Dimension_flip_arrow2_get},
{"LibreDWGc::Dwg_OCD_Dimension_flip_arrow1_set", _wrap_Dwg_OCD_Dimension_flip_arrow1_set},
{"LibreDWGc::Dwg_OCD_Dimension_flip_arrow1_get", _wrap_Dwg_OCD_Dimension_flip_arrow1_get},
{"LibreDWGc::new_Dwg_OCD_Dimension", _wrap_new_Dwg_OCD_Dimension},
{"LibreDWGc::delete_Dwg_OCD_Dimension", _wrap_delete_Dwg_OCD_Dimension},
{"LibreDWGc::Dwg_Object_ALDIMOBJECTCONTEXTDATA_parent_set", _wrap_Dwg_Object_ALDIMOBJECTCONTEXTDATA_parent_set},
{"LibreDWGc::Dwg_Object_ALDIMOBJECTCONTEXTDATA_parent_get", _wrap_Dwg_Object_ALDIMOBJECTCONTEXTDATA_parent_get},
{"LibreDWGc::Dwg_Object_ALDIMOBJECTCONTEXTDATA_class_version_set", _wrap_Dwg_Object_ALDIMOBJECTCONTEXTDATA_class_version_set},
{"LibreDWGc::Dwg_Object_ALDIMOBJECTCONTEXTDATA_class_version_get", _wrap_Dwg_Object_ALDIMOBJECTCONTEXTDATA_class_version_get},
{"LibreDWGc::Dwg_Object_ALDIMOBJECTCONTEXTDATA_is_default_set", _wrap_Dwg_Object_ALDIMOBJECTCONTEXTDATA_is_default_set},
{"LibreDWGc::Dwg_Object_ALDIMOBJECTCONTEXTDATA_is_default_get", _wrap_Dwg_Object_ALDIMOBJECTCONTEXTDATA_is_default_get},
{"LibreDWGc::Dwg_Object_ALDIMOBJECTCONTEXTDATA_scale_set", _wrap_Dwg_Object_ALDIMOBJECTCONTEXTDATA_scale_set},
{"LibreDWGc::Dwg_Object_ALDIMOBJECTCONTEXTDATA_scale_get", _wrap_Dwg_Object_ALDIMOBJECTCONTEXTDATA_scale_get},
{"LibreDWGc::Dwg_Object_ALDIMOBJECTCONTEXTDATA_dimension_set", _wrap_Dwg_Object_ALDIMOBJECTCONTEXTDATA_dimension_set},
{"LibreDWGc::Dwg_Object_ALDIMOBJECTCONTEXTDATA_dimension_get", _wrap_Dwg_Object_ALDIMOBJECTCONTEXTDATA_dimension_get},
{"LibreDWGc::Dwg_Object_ALDIMOBJECTCONTEXTDATA_dimline_pt_set", _wrap_Dwg_Object_ALDIMOBJECTCONTEXTDATA_dimline_pt_set},
{"LibreDWGc::Dwg_Object_ALDIMOBJECTCONTEXTDATA_dimline_pt_get", _wrap_Dwg_Object_ALDIMOBJECTCONTEXTDATA_dimline_pt_get},
{"LibreDWGc::new_Dwg_Object_ALDIMOBJECTCONTEXTDATA", _wrap_new_Dwg_Object_ALDIMOBJECTCONTEXTDATA},
{"LibreDWGc::delete_Dwg_Object_ALDIMOBJECTCONTEXTDATA", _wrap_delete_Dwg_Object_ALDIMOBJECTCONTEXTDATA},
{"LibreDWGc::Dwg_Object_ANGDIMOBJECTCONTEXTDATA_parent_set", _wrap_Dwg_Object_ANGDIMOBJECTCONTEXTDATA_parent_set},
{"LibreDWGc::Dwg_Object_ANGDIMOBJECTCONTEXTDATA_parent_get", _wrap_Dwg_Object_ANGDIMOBJECTCONTEXTDATA_parent_get},
{"LibreDWGc::Dwg_Object_ANGDIMOBJECTCONTEXTDATA_class_version_set", _wrap_Dwg_Object_ANGDIMOBJECTCONTEXTDATA_class_version_set},
{"LibreDWGc::Dwg_Object_ANGDIMOBJECTCONTEXTDATA_class_version_get", _wrap_Dwg_Object_ANGDIMOBJECTCONTEXTDATA_class_version_get},
{"LibreDWGc::Dwg_Object_ANGDIMOBJECTCONTEXTDATA_is_default_set", _wrap_Dwg_Object_ANGDIMOBJECTCONTEXTDATA_is_default_set},
{"LibreDWGc::Dwg_Object_ANGDIMOBJECTCONTEXTDATA_is_default_get", _wrap_Dwg_Object_ANGDIMOBJECTCONTEXTDATA_is_default_get},
{"LibreDWGc::Dwg_Object_ANGDIMOBJECTCONTEXTDATA_scale_set", _wrap_Dwg_Object_ANGDIMOBJECTCONTEXTDATA_scale_set},
{"LibreDWGc::Dwg_Object_ANGDIMOBJECTCONTEXTDATA_scale_get", _wrap_Dwg_Object_ANGDIMOBJECTCONTEXTDATA_scale_get},
{"LibreDWGc::Dwg_Object_ANGDIMOBJECTCONTEXTDATA_dimension_set", _wrap_Dwg_Object_ANGDIMOBJECTCONTEXTDATA_dimension_set},
{"LibreDWGc::Dwg_Object_ANGDIMOBJECTCONTEXTDATA_dimension_get", _wrap_Dwg_Object_ANGDIMOBJECTCONTEXTDATA_dimension_get},
{"LibreDWGc::Dwg_Object_ANGDIMOBJECTCONTEXTDATA_arc_pt_set", _wrap_Dwg_Object_ANGDIMOBJECTCONTEXTDATA_arc_pt_set},
{"LibreDWGc::Dwg_Object_ANGDIMOBJECTCONTEXTDATA_arc_pt_get", _wrap_Dwg_Object_ANGDIMOBJECTCONTEXTDATA_arc_pt_get},
{"LibreDWGc::new_Dwg_Object_ANGDIMOBJECTCONTEXTDATA", _wrap_new_Dwg_Object_ANGDIMOBJECTCONTEXTDATA},
{"LibreDWGc::delete_Dwg_Object_ANGDIMOBJECTCONTEXTDATA", _wrap_delete_Dwg_Object_ANGDIMOBJECTCONTEXTDATA},
{"LibreDWGc::Dwg_Object_DMDIMOBJECTCONTEXTDATA_parent_set", _wrap_Dwg_Object_DMDIMOBJECTCONTEXTDATA_parent_set},
{"LibreDWGc::Dwg_Object_DMDIMOBJECTCONTEXTDATA_parent_get", _wrap_Dwg_Object_DMDIMOBJECTCONTEXTDATA_parent_get},
{"LibreDWGc::Dwg_Object_DMDIMOBJECTCONTEXTDATA_class_version_set", _wrap_Dwg_Object_DMDIMOBJECTCONTEXTDATA_class_version_set},
{"LibreDWGc::Dwg_Object_DMDIMOBJECTCONTEXTDATA_class_version_get", _wrap_Dwg_Object_DMDIMOBJECTCONTEXTDATA_class_version_get},
{"LibreDWGc::Dwg_Object_DMDIMOBJECTCONTEXTDATA_is_default_set", _wrap_Dwg_Object_DMDIMOBJECTCONTEXTDATA_is_default_set},
{"LibreDWGc::Dwg_Object_DMDIMOBJECTCONTEXTDATA_is_default_get", _wrap_Dwg_Object_DMDIMOBJECTCONTEXTDATA_is_default_get},
{"LibreDWGc::Dwg_Object_DMDIMOBJECTCONTEXTDATA_scale_set", _wrap_Dwg_Object_DMDIMOBJECTCONTEXTDATA_scale_set},
{"LibreDWGc::Dwg_Object_DMDIMOBJECTCONTEXTDATA_scale_get", _wrap_Dwg_Object_DMDIMOBJECTCONTEXTDATA_scale_get},
{"LibreDWGc::Dwg_Object_DMDIMOBJECTCONTEXTDATA_dimension_set", _wrap_Dwg_Object_DMDIMOBJECTCONTEXTDATA_dimension_set},
{"LibreDWGc::Dwg_Object_DMDIMOBJECTCONTEXTDATA_dimension_get", _wrap_Dwg_Object_DMDIMOBJECTCONTEXTDATA_dimension_get},
{"LibreDWGc::Dwg_Object_DMDIMOBJECTCONTEXTDATA_first_arc_pt_set", _wrap_Dwg_Object_DMDIMOBJECTCONTEXTDATA_first_arc_pt_set},
{"LibreDWGc::Dwg_Object_DMDIMOBJECTCONTEXTDATA_first_arc_pt_get", _wrap_Dwg_Object_DMDIMOBJECTCONTEXTDATA_first_arc_pt_get},
{"LibreDWGc::Dwg_Object_DMDIMOBJECTCONTEXTDATA_def_pt_set", _wrap_Dwg_Object_DMDIMOBJECTCONTEXTDATA_def_pt_set},
{"LibreDWGc::Dwg_Object_DMDIMOBJECTCONTEXTDATA_def_pt_get", _wrap_Dwg_Object_DMDIMOBJECTCONTEXTDATA_def_pt_get},
{"LibreDWGc::new_Dwg_Object_DMDIMOBJECTCONTEXTDATA", _wrap_new_Dwg_Object_DMDIMOBJECTCONTEXTDATA},
{"LibreDWGc::delete_Dwg_Object_DMDIMOBJECTCONTEXTDATA", _wrap_delete_Dwg_Object_DMDIMOBJECTCONTEXTDATA},
{"LibreDWGc::Dwg_Object_ORDDIMOBJECTCONTEXTDATA_parent_set", _wrap_Dwg_Object_ORDDIMOBJECTCONTEXTDATA_parent_set},
{"LibreDWGc::Dwg_Object_ORDDIMOBJECTCONTEXTDATA_parent_get", _wrap_Dwg_Object_ORDDIMOBJECTCONTEXTDATA_parent_get},
{"LibreDWGc::Dwg_Object_ORDDIMOBJECTCONTEXTDATA_class_version_set", _wrap_Dwg_Object_ORDDIMOBJECTCONTEXTDATA_class_version_set},
{"LibreDWGc::Dwg_Object_ORDDIMOBJECTCONTEXTDATA_class_version_get", _wrap_Dwg_Object_ORDDIMOBJECTCONTEXTDATA_class_version_get},
{"LibreDWGc::Dwg_Object_ORDDIMOBJECTCONTEXTDATA_is_default_set", _wrap_Dwg_Object_ORDDIMOBJECTCONTEXTDATA_is_default_set},
{"LibreDWGc::Dwg_Object_ORDDIMOBJECTCONTEXTDATA_is_default_get", _wrap_Dwg_Object_ORDDIMOBJECTCONTEXTDATA_is_default_get},
{"LibreDWGc::Dwg_Object_ORDDIMOBJECTCONTEXTDATA_scale_set", _wrap_Dwg_Object_ORDDIMOBJECTCONTEXTDATA_scale_set},
{"LibreDWGc::Dwg_Object_ORDDIMOBJECTCONTEXTDATA_scale_get", _wrap_Dwg_Object_ORDDIMOBJECTCONTEXTDATA_scale_get},
{"LibreDWGc::Dwg_Object_ORDDIMOBJECTCONTEXTDATA_dimension_set", _wrap_Dwg_Object_ORDDIMOBJECTCONTEXTDATA_dimension_set},
{"LibreDWGc::Dwg_Object_ORDDIMOBJECTCONTEXTDATA_dimension_get", _wrap_Dwg_Object_ORDDIMOBJECTCONTEXTDATA_dimension_get},
{"LibreDWGc::Dwg_Object_ORDDIMOBJECTCONTEXTDATA_feature_location_pt_set", _wrap_Dwg_Object_ORDDIMOBJECTCONTEXTDATA_feature_location_pt_set},
{"LibreDWGc::Dwg_Object_ORDDIMOBJECTCONTEXTDATA_feature_location_pt_get", _wrap_Dwg_Object_ORDDIMOBJECTCONTEXTDATA_feature_location_pt_get},
{"LibreDWGc::Dwg_Object_ORDDIMOBJECTCONTEXTDATA_leader_endpt_set", _wrap_Dwg_Object_ORDDIMOBJECTCONTEXTDATA_leader_endpt_set},
{"LibreDWGc::Dwg_Object_ORDDIMOBJECTCONTEXTDATA_leader_endpt_get", _wrap_Dwg_Object_ORDDIMOBJECTCONTEXTDATA_leader_endpt_get},
{"LibreDWGc::new_Dwg_Object_ORDDIMOBJECTCONTEXTDATA", _wrap_new_Dwg_Object_ORDDIMOBJECTCONTEXTDATA},
{"LibreDWGc::delete_Dwg_Object_ORDDIMOBJECTCONTEXTDATA", _wrap_delete_Dwg_Object_ORDDIMOBJECTCONTEXTDATA},
{"LibreDWGc::Dwg_Object_RADIMOBJECTCONTEXTDATA_parent_set", _wrap_Dwg_Object_RADIMOBJECTCONTEXTDATA_parent_set},
{"LibreDWGc::Dwg_Object_RADIMOBJECTCONTEXTDATA_parent_get", _wrap_Dwg_Object_RADIMOBJECTCONTEXTDATA_parent_get},
{"LibreDWGc::Dwg_Object_RADIMOBJECTCONTEXTDATA_class_version_set", _wrap_Dwg_Object_RADIMOBJECTCONTEXTDATA_class_version_set},
{"LibreDWGc::Dwg_Object_RADIMOBJECTCONTEXTDATA_class_version_get", _wrap_Dwg_Object_RADIMOBJECTCONTEXTDATA_class_version_get},
{"LibreDWGc::Dwg_Object_RADIMOBJECTCONTEXTDATA_is_default_set", _wrap_Dwg_Object_RADIMOBJECTCONTEXTDATA_is_default_set},
{"LibreDWGc::Dwg_Object_RADIMOBJECTCONTEXTDATA_is_default_get", _wrap_Dwg_Object_RADIMOBJECTCONTEXTDATA_is_default_get},
{"LibreDWGc::Dwg_Object_RADIMOBJECTCONTEXTDATA_scale_set", _wrap_Dwg_Object_RADIMOBJECTCONTEXTDATA_scale_set},
{"LibreDWGc::Dwg_Object_RADIMOBJECTCONTEXTDATA_scale_get", _wrap_Dwg_Object_RADIMOBJECTCONTEXTDATA_scale_get},
{"LibreDWGc::Dwg_Object_RADIMOBJECTCONTEXTDATA_dimension_set", _wrap_Dwg_Object_RADIMOBJECTCONTEXTDATA_dimension_set},
{"LibreDWGc::Dwg_Object_RADIMOBJECTCONTEXTDATA_dimension_get", _wrap_Dwg_Object_RADIMOBJECTCONTEXTDATA_dimension_get},
{"LibreDWGc::Dwg_Object_RADIMOBJECTCONTEXTDATA_first_arc_pt_set", _wrap_Dwg_Object_RADIMOBJECTCONTEXTDATA_first_arc_pt_set},
{"LibreDWGc::Dwg_Object_RADIMOBJECTCONTEXTDATA_first_arc_pt_get", _wrap_Dwg_Object_RADIMOBJECTCONTEXTDATA_first_arc_pt_get},
{"LibreDWGc::new_Dwg_Object_RADIMOBJECTCONTEXTDATA", _wrap_new_Dwg_Object_RADIMOBJECTCONTEXTDATA},
{"LibreDWGc::delete_Dwg_Object_RADIMOBJECTCONTEXTDATA", _wrap_delete_Dwg_Object_RADIMOBJECTCONTEXTDATA},
{"LibreDWGc::Dwg_Object_RADIMLGOBJECTCONTEXTDATA_parent_set", _wrap_Dwg_Object_RADIMLGOBJECTCONTEXTDATA_parent_set},
{"LibreDWGc::Dwg_Object_RADIMLGOBJECTCONTEXTDATA_parent_get", _wrap_Dwg_Object_RADIMLGOBJECTCONTEXTDATA_parent_get},
{"LibreDWGc::Dwg_Object_RADIMLGOBJECTCONTEXTDATA_class_version_set", _wrap_Dwg_Object_RADIMLGOBJECTCONTEXTDATA_class_version_set},
{"LibreDWGc::Dwg_Object_RADIMLGOBJECTCONTEXTDATA_class_version_get", _wrap_Dwg_Object_RADIMLGOBJECTCONTEXTDATA_class_version_get},
{"LibreDWGc::Dwg_Object_RADIMLGOBJECTCONTEXTDATA_is_default_set", _wrap_Dwg_Object_RADIMLGOBJECTCONTEXTDATA_is_default_set},
{"LibreDWGc::Dwg_Object_RADIMLGOBJECTCONTEXTDATA_is_default_get", _wrap_Dwg_Object_RADIMLGOBJECTCONTEXTDATA_is_default_get},
{"LibreDWGc::Dwg_Object_RADIMLGOBJECTCONTEXTDATA_scale_set", _wrap_Dwg_Object_RADIMLGOBJECTCONTEXTDATA_scale_set},
{"LibreDWGc::Dwg_Object_RADIMLGOBJECTCONTEXTDATA_scale_get", _wrap_Dwg_Object_RADIMLGOBJECTCONTEXTDATA_scale_get},
{"LibreDWGc::Dwg_Object_RADIMLGOBJECTCONTEXTDATA_dimension_set", _wrap_Dwg_Object_RADIMLGOBJECTCONTEXTDATA_dimension_set},
{"LibreDWGc::Dwg_Object_RADIMLGOBJECTCONTEXTDATA_dimension_get", _wrap_Dwg_Object_RADIMLGOBJECTCONTEXTDATA_dimension_get},
{"LibreDWGc::Dwg_Object_RADIMLGOBJECTCONTEXTDATA_ovr_center_set", _wrap_Dwg_Object_RADIMLGOBJECTCONTEXTDATA_ovr_center_set},
{"LibreDWGc::Dwg_Object_RADIMLGOBJECTCONTEXTDATA_ovr_center_get", _wrap_Dwg_Object_RADIMLGOBJECTCONTEXTDATA_ovr_center_get},
{"LibreDWGc::Dwg_Object_RADIMLGOBJECTCONTEXTDATA_jog_point_set", _wrap_Dwg_Object_RADIMLGOBJECTCONTEXTDATA_jog_point_set},
{"LibreDWGc::Dwg_Object_RADIMLGOBJECTCONTEXTDATA_jog_point_get", _wrap_Dwg_Object_RADIMLGOBJECTCONTEXTDATA_jog_point_get},
{"LibreDWGc::new_Dwg_Object_RADIMLGOBJECTCONTEXTDATA", _wrap_new_Dwg_Object_RADIMLGOBJECTCONTEXTDATA},
{"LibreDWGc::delete_Dwg_Object_RADIMLGOBJECTCONTEXTDATA", _wrap_delete_Dwg_Object_RADIMLGOBJECTCONTEXTDATA},
{"LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_parent_set", _wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_parent_set},
{"LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_parent_get", _wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_parent_get},
{"LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_class_version_set", _wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_class_version_set},
{"LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_class_version_get", _wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_class_version_get},
{"LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_is_default_set", _wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_is_default_set},
{"LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_is_default_get", _wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_is_default_get},
{"LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_scale_set", _wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_scale_set},
{"LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_scale_get", _wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_scale_get},
{"LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_horizontal_mode_set", _wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_horizontal_mode_set},
{"LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_horizontal_mode_get", _wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_horizontal_mode_get},
{"LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_rotation_set", _wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_rotation_set},
{"LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_rotation_get", _wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_rotation_get},
{"LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_ins_pt_set", _wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_ins_pt_set},
{"LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_ins_pt_get", _wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_ins_pt_get},
{"LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_alignment_pt_set", _wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_alignment_pt_set},
{"LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_alignment_pt_get", _wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_alignment_pt_get},
{"LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_enable_context_set", _wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_enable_context_set},
{"LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_enable_context_get", _wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_enable_context_get},
{"LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_context_set", _wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_context_set},
{"LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_context_get", _wrap_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_context_get},
{"LibreDWGc::new_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA", _wrap_new_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA},
{"LibreDWGc::delete_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA", _wrap_delete_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA},
{"LibreDWGc::Dwg_Object_MLEADEROBJECTCONTEXTDATA_parent_set", _wrap_Dwg_Object_MLEADEROBJECTCONTEXTDATA_parent_set},
{"LibreDWGc::Dwg_Object_MLEADEROBJECTCONTEXTDATA_parent_get", _wrap_Dwg_Object_MLEADEROBJECTCONTEXTDATA_parent_get},
{"LibreDWGc::Dwg_Object_MLEADEROBJECTCONTEXTDATA_class_version_set", _wrap_Dwg_Object_MLEADEROBJECTCONTEXTDATA_class_version_set},
{"LibreDWGc::Dwg_Object_MLEADEROBJECTCONTEXTDATA_class_version_get", _wrap_Dwg_Object_MLEADEROBJECTCONTEXTDATA_class_version_get},
{"LibreDWGc::Dwg_Object_MLEADEROBJECTCONTEXTDATA_is_default_set", _wrap_Dwg_Object_MLEADEROBJECTCONTEXTDATA_is_default_set},
{"LibreDWGc::Dwg_Object_MLEADEROBJECTCONTEXTDATA_is_default_get", _wrap_Dwg_Object_MLEADEROBJECTCONTEXTDATA_is_default_get},
{"LibreDWGc::Dwg_Object_MLEADEROBJECTCONTEXTDATA_scale_set", _wrap_Dwg_Object_MLEADEROBJECTCONTEXTDATA_scale_set},
{"LibreDWGc::Dwg_Object_MLEADEROBJECTCONTEXTDATA_scale_get", _wrap_Dwg_Object_MLEADEROBJECTCONTEXTDATA_scale_get},
{"LibreDWGc::new_Dwg_Object_MLEADEROBJECTCONTEXTDATA", _wrap_new_Dwg_Object_MLEADEROBJECTCONTEXTDATA},
{"LibreDWGc::delete_Dwg_Object_MLEADEROBJECTCONTEXTDATA", _wrap_delete_Dwg_Object_MLEADEROBJECTCONTEXTDATA},
{"LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_parent_set", _wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_parent_set},
{"LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_parent_get", _wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_parent_get},
{"LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_class_version_set", _wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_class_version_set},
{"LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_class_version_get", _wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_class_version_get},
{"LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_is_default_set", _wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_is_default_set},
{"LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_is_default_get", _wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_is_default_get},
{"LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_scale_set", _wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_scale_set},
{"LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_scale_get", _wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_scale_get},
{"LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_num_points_set", _wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_num_points_set},
{"LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_num_points_get", _wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_num_points_get},
{"LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_points_set", _wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_points_set},
{"LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_points_get", _wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_points_get},
{"LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_b290_set", _wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_b290_set},
{"LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_b290_get", _wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_b290_get},
{"LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_x_direction_set", _wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_x_direction_set},
{"LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_x_direction_get", _wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_x_direction_get},
{"LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_inspt_offset_set", _wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_inspt_offset_set},
{"LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_inspt_offset_get", _wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_inspt_offset_get},
{"LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_endptproj_set", _wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_endptproj_set},
{"LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_endptproj_get", _wrap_Dwg_Object_LEADEROBJECTCONTEXTDATA_endptproj_get},
{"LibreDWGc::new_Dwg_Object_LEADEROBJECTCONTEXTDATA", _wrap_new_Dwg_Object_LEADEROBJECTCONTEXTDATA},
{"LibreDWGc::delete_Dwg_Object_LEADEROBJECTCONTEXTDATA", _wrap_delete_Dwg_Object_LEADEROBJECTCONTEXTDATA},
{"LibreDWGc::Dwg_Object_BLKREFOBJECTCONTEXTDATA_parent_set", _wrap_Dwg_Object_BLKREFOBJECTCONTEXTDATA_parent_set},
{"LibreDWGc::Dwg_Object_BLKREFOBJECTCONTEXTDATA_parent_get", _wrap_Dwg_Object_BLKREFOBJECTCONTEXTDATA_parent_get},
{"LibreDWGc::Dwg_Object_BLKREFOBJECTCONTEXTDATA_class_version_set", _wrap_Dwg_Object_BLKREFOBJECTCONTEXTDATA_class_version_set},
{"LibreDWGc::Dwg_Object_BLKREFOBJECTCONTEXTDATA_class_version_get", _wrap_Dwg_Object_BLKREFOBJECTCONTEXTDATA_class_version_get},
{"LibreDWGc::Dwg_Object_BLKREFOBJECTCONTEXTDATA_is_default_set", _wrap_Dwg_Object_BLKREFOBJECTCONTEXTDATA_is_default_set},
{"LibreDWGc::Dwg_Object_BLKREFOBJECTCONTEXTDATA_is_default_get", _wrap_Dwg_Object_BLKREFOBJECTCONTEXTDATA_is_default_get},
{"LibreDWGc::Dwg_Object_BLKREFOBJECTCONTEXTDATA_scale_set", _wrap_Dwg_Object_BLKREFOBJECTCONTEXTDATA_scale_set},
{"LibreDWGc::Dwg_Object_BLKREFOBJECTCONTEXTDATA_scale_get", _wrap_Dwg_Object_BLKREFOBJECTCONTEXTDATA_scale_get},
{"LibreDWGc::Dwg_Object_BLKREFOBJECTCONTEXTDATA_rotation_set", _wrap_Dwg_Object_BLKREFOBJECTCONTEXTDATA_rotation_set},
{"LibreDWGc::Dwg_Object_BLKREFOBJECTCONTEXTDATA_rotation_get", _wrap_Dwg_Object_BLKREFOBJECTCONTEXTDATA_rotation_get},
{"LibreDWGc::Dwg_Object_BLKREFOBJECTCONTEXTDATA_ins_pt_set", _wrap_Dwg_Object_BLKREFOBJECTCONTEXTDATA_ins_pt_set},
{"LibreDWGc::Dwg_Object_BLKREFOBJECTCONTEXTDATA_ins_pt_get", _wrap_Dwg_Object_BLKREFOBJECTCONTEXTDATA_ins_pt_get},
{"LibreDWGc::Dwg_Object_BLKREFOBJECTCONTEXTDATA_scale_factor_set", _wrap_Dwg_Object_BLKREFOBJECTCONTEXTDATA_scale_factor_set},
{"LibreDWGc::Dwg_Object_BLKREFOBJECTCONTEXTDATA_scale_factor_get", _wrap_Dwg_Object_BLKREFOBJECTCONTEXTDATA_scale_factor_get},
{"LibreDWGc::new_Dwg_Object_BLKREFOBJECTCONTEXTDATA", _wrap_new_Dwg_Object_BLKREFOBJECTCONTEXTDATA},
{"LibreDWGc::delete_Dwg_Object_BLKREFOBJECTCONTEXTDATA", _wrap_delete_Dwg_Object_BLKREFOBJECTCONTEXTDATA},
{"LibreDWGc::Dwg_Object_FCFOBJECTCONTEXTDATA_parent_set", _wrap_Dwg_Object_FCFOBJECTCONTEXTDATA_parent_set},
{"LibreDWGc::Dwg_Object_FCFOBJECTCONTEXTDATA_parent_get", _wrap_Dwg_Object_FCFOBJECTCONTEXTDATA_parent_get},
{"LibreDWGc::Dwg_Object_FCFOBJECTCONTEXTDATA_class_version_set", _wrap_Dwg_Object_FCFOBJECTCONTEXTDATA_class_version_set},
{"LibreDWGc::Dwg_Object_FCFOBJECTCONTEXTDATA_class_version_get", _wrap_Dwg_Object_FCFOBJECTCONTEXTDATA_class_version_get},
{"LibreDWGc::Dwg_Object_FCFOBJECTCONTEXTDATA_is_default_set", _wrap_Dwg_Object_FCFOBJECTCONTEXTDATA_is_default_set},
{"LibreDWGc::Dwg_Object_FCFOBJECTCONTEXTDATA_is_default_get", _wrap_Dwg_Object_FCFOBJECTCONTEXTDATA_is_default_get},
{"LibreDWGc::Dwg_Object_FCFOBJECTCONTEXTDATA_scale_set", _wrap_Dwg_Object_FCFOBJECTCONTEXTDATA_scale_set},
{"LibreDWGc::Dwg_Object_FCFOBJECTCONTEXTDATA_scale_get", _wrap_Dwg_Object_FCFOBJECTCONTEXTDATA_scale_get},
{"LibreDWGc::Dwg_Object_FCFOBJECTCONTEXTDATA_location_set", _wrap_Dwg_Object_FCFOBJECTCONTEXTDATA_location_set},
{"LibreDWGc::Dwg_Object_FCFOBJECTCONTEXTDATA_location_get", _wrap_Dwg_Object_FCFOBJECTCONTEXTDATA_location_get},
{"LibreDWGc::Dwg_Object_FCFOBJECTCONTEXTDATA_horiz_dir_set", _wrap_Dwg_Object_FCFOBJECTCONTEXTDATA_horiz_dir_set},
{"LibreDWGc::Dwg_Object_FCFOBJECTCONTEXTDATA_horiz_dir_get", _wrap_Dwg_Object_FCFOBJECTCONTEXTDATA_horiz_dir_get},
{"LibreDWGc::new_Dwg_Object_FCFOBJECTCONTEXTDATA", _wrap_new_Dwg_Object_FCFOBJECTCONTEXTDATA},
{"LibreDWGc::delete_Dwg_Object_FCFOBJECTCONTEXTDATA", _wrap_delete_Dwg_Object_FCFOBJECTCONTEXTDATA},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_parent_set", _wrap_Dwg_Object_DETAILVIEWSTYLE_parent_set},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_parent_get", _wrap_Dwg_Object_DETAILVIEWSTYLE_parent_get},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_mdoc_class_version_set", _wrap_Dwg_Object_DETAILVIEWSTYLE_mdoc_class_version_set},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_mdoc_class_version_get", _wrap_Dwg_Object_DETAILVIEWSTYLE_mdoc_class_version_get},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_desc_set", _wrap_Dwg_Object_DETAILVIEWSTYLE_desc_set},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_desc_get", _wrap_Dwg_Object_DETAILVIEWSTYLE_desc_get},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_is_modified_for_recompute_set", _wrap_Dwg_Object_DETAILVIEWSTYLE_is_modified_for_recompute_set},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_is_modified_for_recompute_get", _wrap_Dwg_Object_DETAILVIEWSTYLE_is_modified_for_recompute_get},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_display_name_set", _wrap_Dwg_Object_DETAILVIEWSTYLE_display_name_set},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_display_name_get", _wrap_Dwg_Object_DETAILVIEWSTYLE_display_name_get},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_viewstyle_flags_set", _wrap_Dwg_Object_DETAILVIEWSTYLE_viewstyle_flags_set},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_viewstyle_flags_get", _wrap_Dwg_Object_DETAILVIEWSTYLE_viewstyle_flags_get},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_class_version_set", _wrap_Dwg_Object_DETAILVIEWSTYLE_class_version_set},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_class_version_get", _wrap_Dwg_Object_DETAILVIEWSTYLE_class_version_get},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_flags_set", _wrap_Dwg_Object_DETAILVIEWSTYLE_flags_set},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_flags_get", _wrap_Dwg_Object_DETAILVIEWSTYLE_flags_get},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_identifier_style_set", _wrap_Dwg_Object_DETAILVIEWSTYLE_identifier_style_set},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_identifier_style_get", _wrap_Dwg_Object_DETAILVIEWSTYLE_identifier_style_get},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_identifier_color_set", _wrap_Dwg_Object_DETAILVIEWSTYLE_identifier_color_set},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_identifier_color_get", _wrap_Dwg_Object_DETAILVIEWSTYLE_identifier_color_get},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_identifier_height_set", _wrap_Dwg_Object_DETAILVIEWSTYLE_identifier_height_set},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_identifier_height_get", _wrap_Dwg_Object_DETAILVIEWSTYLE_identifier_height_get},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_identifier_exclude_characters_set", _wrap_Dwg_Object_DETAILVIEWSTYLE_identifier_exclude_characters_set},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_identifier_exclude_characters_get", _wrap_Dwg_Object_DETAILVIEWSTYLE_identifier_exclude_characters_get},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_identifier_offset_set", _wrap_Dwg_Object_DETAILVIEWSTYLE_identifier_offset_set},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_identifier_offset_get", _wrap_Dwg_Object_DETAILVIEWSTYLE_identifier_offset_get},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_identifier_placement_set", _wrap_Dwg_Object_DETAILVIEWSTYLE_identifier_placement_set},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_identifier_placement_get", _wrap_Dwg_Object_DETAILVIEWSTYLE_identifier_placement_get},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_set", _wrap_Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_set},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_get", _wrap_Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_get},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_color_set", _wrap_Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_color_set},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_color_get", _wrap_Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_color_get},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_size_set", _wrap_Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_size_set},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_size_get", _wrap_Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_size_get},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_boundary_ltype_set", _wrap_Dwg_Object_DETAILVIEWSTYLE_boundary_ltype_set},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_boundary_ltype_get", _wrap_Dwg_Object_DETAILVIEWSTYLE_boundary_ltype_get},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_boundary_linewt_set", _wrap_Dwg_Object_DETAILVIEWSTYLE_boundary_linewt_set},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_boundary_linewt_get", _wrap_Dwg_Object_DETAILVIEWSTYLE_boundary_linewt_get},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_boundary_line_color_set", _wrap_Dwg_Object_DETAILVIEWSTYLE_boundary_line_color_set},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_boundary_line_color_get", _wrap_Dwg_Object_DETAILVIEWSTYLE_boundary_line_color_get},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_style_set", _wrap_Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_style_set},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_style_get", _wrap_Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_style_get},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_color_set", _wrap_Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_color_set},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_color_get", _wrap_Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_color_get},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_height_set", _wrap_Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_height_set},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_height_get", _wrap_Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_height_get},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_viewlabel_attachment_set", _wrap_Dwg_Object_DETAILVIEWSTYLE_viewlabel_attachment_set},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_viewlabel_attachment_get", _wrap_Dwg_Object_DETAILVIEWSTYLE_viewlabel_attachment_get},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_viewlabel_offset_set", _wrap_Dwg_Object_DETAILVIEWSTYLE_viewlabel_offset_set},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_viewlabel_offset_get", _wrap_Dwg_Object_DETAILVIEWSTYLE_viewlabel_offset_get},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_viewlabel_alignment_set", _wrap_Dwg_Object_DETAILVIEWSTYLE_viewlabel_alignment_set},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_viewlabel_alignment_get", _wrap_Dwg_Object_DETAILVIEWSTYLE_viewlabel_alignment_get},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_viewlabel_pattern_set", _wrap_Dwg_Object_DETAILVIEWSTYLE_viewlabel_pattern_set},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_viewlabel_pattern_get", _wrap_Dwg_Object_DETAILVIEWSTYLE_viewlabel_pattern_get},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_connection_ltype_set", _wrap_Dwg_Object_DETAILVIEWSTYLE_connection_ltype_set},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_connection_ltype_get", _wrap_Dwg_Object_DETAILVIEWSTYLE_connection_ltype_get},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_connection_linewt_set", _wrap_Dwg_Object_DETAILVIEWSTYLE_connection_linewt_set},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_connection_linewt_get", _wrap_Dwg_Object_DETAILVIEWSTYLE_connection_linewt_get},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_connection_line_color_set", _wrap_Dwg_Object_DETAILVIEWSTYLE_connection_line_color_set},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_connection_line_color_get", _wrap_Dwg_Object_DETAILVIEWSTYLE_connection_line_color_get},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_borderline_ltype_set", _wrap_Dwg_Object_DETAILVIEWSTYLE_borderline_ltype_set},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_borderline_ltype_get", _wrap_Dwg_Object_DETAILVIEWSTYLE_borderline_ltype_get},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_borderline_linewt_set", _wrap_Dwg_Object_DETAILVIEWSTYLE_borderline_linewt_set},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_borderline_linewt_get", _wrap_Dwg_Object_DETAILVIEWSTYLE_borderline_linewt_get},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_borderline_color_set", _wrap_Dwg_Object_DETAILVIEWSTYLE_borderline_color_set},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_borderline_color_get", _wrap_Dwg_Object_DETAILVIEWSTYLE_borderline_color_get},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_model_edge_set", _wrap_Dwg_Object_DETAILVIEWSTYLE_model_edge_set},
{"LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_model_edge_get", _wrap_Dwg_Object_DETAILVIEWSTYLE_model_edge_get},
{"LibreDWGc::new_Dwg_Object_DETAILVIEWSTYLE", _wrap_new_Dwg_Object_DETAILVIEWSTYLE},
{"LibreDWGc::delete_Dwg_Object_DETAILVIEWSTYLE", _wrap_delete_Dwg_Object_DETAILVIEWSTYLE},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_parent_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_parent_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_parent_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_parent_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_mdoc_class_version_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_mdoc_class_version_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_mdoc_class_version_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_mdoc_class_version_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_desc_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_desc_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_desc_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_desc_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_is_modified_for_recompute_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_is_modified_for_recompute_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_is_modified_for_recompute_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_is_modified_for_recompute_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_display_name_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_display_name_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_display_name_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_display_name_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_viewstyle_flags_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_viewstyle_flags_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_viewstyle_flags_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_viewstyle_flags_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_class_version_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_class_version_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_class_version_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_class_version_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_flags_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_flags_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_flags_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_flags_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_identifier_style_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_identifier_style_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_identifier_style_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_identifier_style_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_identifier_color_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_identifier_color_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_identifier_color_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_identifier_color_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_identifier_height_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_identifier_height_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_identifier_height_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_identifier_height_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_arrow_start_symbol_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_arrow_start_symbol_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_arrow_start_symbol_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_arrow_start_symbol_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_arrow_end_symbol_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_arrow_end_symbol_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_arrow_end_symbol_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_arrow_end_symbol_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_color_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_color_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_color_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_color_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_size_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_size_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_size_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_size_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_identifier_exclude_characters_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_identifier_exclude_characters_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_identifier_exclude_characters_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_identifier_exclude_characters_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_identifier_position_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_identifier_position_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_identifier_position_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_identifier_position_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_identifier_offset_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_identifier_offset_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_identifier_offset_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_identifier_offset_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_arrow_position_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_arrow_position_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_arrow_position_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_arrow_position_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_extension_length_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_extension_length_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_extension_length_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_extension_length_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_plane_ltype_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_plane_ltype_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_plane_ltype_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_plane_ltype_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_plane_linewt_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_plane_linewt_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_plane_linewt_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_plane_linewt_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_plane_line_color_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_plane_line_color_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_plane_line_color_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_plane_line_color_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_bend_ltype_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_bend_ltype_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_bend_ltype_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_bend_ltype_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_bend_linewt_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_bend_linewt_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_bend_linewt_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_bend_linewt_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_bend_line_color_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_bend_line_color_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_bend_line_color_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_bend_line_color_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_bend_line_length_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_bend_line_length_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_bend_line_length_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_bend_line_length_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_end_line_overshoot_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_end_line_overshoot_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_end_line_overshoot_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_end_line_overshoot_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_end_line_length_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_end_line_length_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_end_line_length_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_end_line_length_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_style_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_style_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_style_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_style_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_color_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_color_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_color_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_color_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_height_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_height_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_height_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_height_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_viewlabel_attachment_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_viewlabel_attachment_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_viewlabel_attachment_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_viewlabel_attachment_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_viewlabel_offset_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_viewlabel_offset_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_viewlabel_offset_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_viewlabel_offset_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_viewlabel_alignment_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_viewlabel_alignment_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_viewlabel_alignment_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_viewlabel_alignment_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_viewlabel_pattern_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_viewlabel_pattern_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_viewlabel_pattern_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_viewlabel_pattern_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_hatch_color_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_hatch_color_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_hatch_color_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_hatch_color_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_hatch_bg_color_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_hatch_bg_color_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_hatch_bg_color_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_hatch_bg_color_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_hatch_pattern_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_hatch_pattern_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_hatch_pattern_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_hatch_pattern_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_hatch_scale_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_hatch_scale_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_hatch_scale_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_hatch_scale_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_hatch_transparency_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_hatch_transparency_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_hatch_transparency_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_hatch_transparency_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_unknown_b1_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_unknown_b1_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_unknown_b1_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_unknown_b1_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_unknown_b2_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_unknown_b2_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_unknown_b2_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_unknown_b2_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_num_hatch_angles_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_num_hatch_angles_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_num_hatch_angles_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_num_hatch_angles_get},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_hatch_angles_set", _wrap_Dwg_Object_SECTIONVIEWSTYLE_hatch_angles_set},
{"LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_hatch_angles_get", _wrap_Dwg_Object_SECTIONVIEWSTYLE_hatch_angles_get},
{"LibreDWGc::new_Dwg_Object_SECTIONVIEWSTYLE", _wrap_new_Dwg_Object_SECTIONVIEWSTYLE},
{"LibreDWGc::delete_Dwg_Object_SECTIONVIEWSTYLE", _wrap_delete_Dwg_Object_SECTIONVIEWSTYLE},
{"LibreDWGc::Dwg_Object_SECTION_MANAGER_parent_set", _wrap_Dwg_Object_SECTION_MANAGER_parent_set},
{"LibreDWGc::Dwg_Object_SECTION_MANAGER_parent_get", _wrap_Dwg_Object_SECTION_MANAGER_parent_get},
{"LibreDWGc::Dwg_Object_SECTION_MANAGER_is_live_set", _wrap_Dwg_Object_SECTION_MANAGER_is_live_set},
{"LibreDWGc::Dwg_Object_SECTION_MANAGER_is_live_get", _wrap_Dwg_Object_SECTION_MANAGER_is_live_get},
{"LibreDWGc::Dwg_Object_SECTION_MANAGER_num_sections_set", _wrap_Dwg_Object_SECTION_MANAGER_num_sections_set},
{"LibreDWGc::Dwg_Object_SECTION_MANAGER_num_sections_get", _wrap_Dwg_Object_SECTION_MANAGER_num_sections_get},
{"LibreDWGc::Dwg_Object_SECTION_MANAGER_sections_set", _wrap_Dwg_Object_SECTION_MANAGER_sections_set},
{"LibreDWGc::Dwg_Object_SECTION_MANAGER_sections_get", _wrap_Dwg_Object_SECTION_MANAGER_sections_get},
{"LibreDWGc::new_Dwg_Object_SECTION_MANAGER", _wrap_new_Dwg_Object_SECTION_MANAGER},
{"LibreDWGc::delete_Dwg_Object_SECTION_MANAGER", _wrap_delete_Dwg_Object_SECTION_MANAGER},
{"LibreDWGc::Dwg_SECTION_geometrysettings_parent_set", _wrap_Dwg_SECTION_geometrysettings_parent_set},
{"LibreDWGc::Dwg_SECTION_geometrysettings_parent_get", _wrap_Dwg_SECTION_geometrysettings_parent_get},
{"LibreDWGc::Dwg_SECTION_geometrysettings_num_geoms_set", _wrap_Dwg_SECTION_geometrysettings_num_geoms_set},
{"LibreDWGc::Dwg_SECTION_geometrysettings_num_geoms_get", _wrap_Dwg_SECTION_geometrysettings_num_geoms_get},
{"LibreDWGc::Dwg_SECTION_geometrysettings_hexindex_set", _wrap_Dwg_SECTION_geometrysettings_hexindex_set},
{"LibreDWGc::Dwg_SECTION_geometrysettings_hexindex_get", _wrap_Dwg_SECTION_geometrysettings_hexindex_get},
{"LibreDWGc::Dwg_SECTION_geometrysettings_flags_set", _wrap_Dwg_SECTION_geometrysettings_flags_set},
{"LibreDWGc::Dwg_SECTION_geometrysettings_flags_get", _wrap_Dwg_SECTION_geometrysettings_flags_get},
{"LibreDWGc::Dwg_SECTION_geometrysettings_color_set", _wrap_Dwg_SECTION_geometrysettings_color_set},
{"LibreDWGc::Dwg_SECTION_geometrysettings_color_get", _wrap_Dwg_SECTION_geometrysettings_color_get},
{"LibreDWGc::Dwg_SECTION_geometrysettings_layer_set", _wrap_Dwg_SECTION_geometrysettings_layer_set},
{"LibreDWGc::Dwg_SECTION_geometrysettings_layer_get", _wrap_Dwg_SECTION_geometrysettings_layer_get},
{"LibreDWGc::Dwg_SECTION_geometrysettings_ltype_set", _wrap_Dwg_SECTION_geometrysettings_ltype_set},
{"LibreDWGc::Dwg_SECTION_geometrysettings_ltype_get", _wrap_Dwg_SECTION_geometrysettings_ltype_get},
{"LibreDWGc::Dwg_SECTION_geometrysettings_ltype_scale_set", _wrap_Dwg_SECTION_geometrysettings_ltype_scale_set},
{"LibreDWGc::Dwg_SECTION_geometrysettings_ltype_scale_get", _wrap_Dwg_SECTION_geometrysettings_ltype_scale_get},
{"LibreDWGc::Dwg_SECTION_geometrysettings_plotstyle_set", _wrap_Dwg_SECTION_geometrysettings_plotstyle_set},
{"LibreDWGc::Dwg_SECTION_geometrysettings_plotstyle_get", _wrap_Dwg_SECTION_geometrysettings_plotstyle_get},
{"LibreDWGc::Dwg_SECTION_geometrysettings_linewt_set", _wrap_Dwg_SECTION_geometrysettings_linewt_set},
{"LibreDWGc::Dwg_SECTION_geometrysettings_linewt_get", _wrap_Dwg_SECTION_geometrysettings_linewt_get},
{"LibreDWGc::Dwg_SECTION_geometrysettings_face_transparency_set", _wrap_Dwg_SECTION_geometrysettings_face_transparency_set},
{"LibreDWGc::Dwg_SECTION_geometrysettings_face_transparency_get", _wrap_Dwg_SECTION_geometrysettings_face_transparency_get},
{"LibreDWGc::Dwg_SECTION_geometrysettings_edge_transparency_set", _wrap_Dwg_SECTION_geometrysettings_edge_transparency_set},
{"LibreDWGc::Dwg_SECTION_geometrysettings_edge_transparency_get", _wrap_Dwg_SECTION_geometrysettings_edge_transparency_get},
{"LibreDWGc::Dwg_SECTION_geometrysettings_hatch_type_set", _wrap_Dwg_SECTION_geometrysettings_hatch_type_set},
{"LibreDWGc::Dwg_SECTION_geometrysettings_hatch_type_get", _wrap_Dwg_SECTION_geometrysettings_hatch_type_get},
{"LibreDWGc::Dwg_SECTION_geometrysettings_hatch_pattern_set", _wrap_Dwg_SECTION_geometrysettings_hatch_pattern_set},
{"LibreDWGc::Dwg_SECTION_geometrysettings_hatch_pattern_get", _wrap_Dwg_SECTION_geometrysettings_hatch_pattern_get},
{"LibreDWGc::Dwg_SECTION_geometrysettings_hatch_angle_set", _wrap_Dwg_SECTION_geometrysettings_hatch_angle_set},
{"LibreDWGc::Dwg_SECTION_geometrysettings_hatch_angle_get", _wrap_Dwg_SECTION_geometrysettings_hatch_angle_get},
{"LibreDWGc::Dwg_SECTION_geometrysettings_hatch_spacing_set", _wrap_Dwg_SECTION_geometrysettings_hatch_spacing_set},
{"LibreDWGc::Dwg_SECTION_geometrysettings_hatch_spacing_get", _wrap_Dwg_SECTION_geometrysettings_hatch_spacing_get},
{"LibreDWGc::Dwg_SECTION_geometrysettings_hatch_scale_set", _wrap_Dwg_SECTION_geometrysettings_hatch_scale_set},
{"LibreDWGc::Dwg_SECTION_geometrysettings_hatch_scale_get", _wrap_Dwg_SECTION_geometrysettings_hatch_scale_get},
{"LibreDWGc::new_Dwg_SECTION_geometrysettings", _wrap_new_Dwg_SECTION_geometrysettings},
{"LibreDWGc::delete_Dwg_SECTION_geometrysettings", _wrap_delete_Dwg_SECTION_geometrysettings},
{"LibreDWGc::Dwg_SECTION_typesettings_parent_set", _wrap_Dwg_SECTION_typesettings_parent_set},
{"LibreDWGc::Dwg_SECTION_typesettings_parent_get", _wrap_Dwg_SECTION_typesettings_parent_get},
{"LibreDWGc::Dwg_SECTION_typesettings_type_set", _wrap_Dwg_SECTION_typesettings_type_set},
{"LibreDWGc::Dwg_SECTION_typesettings_type_get", _wrap_Dwg_SECTION_typesettings_type_get},
{"LibreDWGc::Dwg_SECTION_typesettings_generation_set", _wrap_Dwg_SECTION_typesettings_generation_set},
{"LibreDWGc::Dwg_SECTION_typesettings_generation_get", _wrap_Dwg_SECTION_typesettings_generation_get},
{"LibreDWGc::Dwg_SECTION_typesettings_num_sources_set", _wrap_Dwg_SECTION_typesettings_num_sources_set},
{"LibreDWGc::Dwg_SECTION_typesettings_num_sources_get", _wrap_Dwg_SECTION_typesettings_num_sources_get},
{"LibreDWGc::Dwg_SECTION_typesettings_sources_set", _wrap_Dwg_SECTION_typesettings_sources_set},
{"LibreDWGc::Dwg_SECTION_typesettings_sources_get", _wrap_Dwg_SECTION_typesettings_sources_get},
{"LibreDWGc::Dwg_SECTION_typesettings_destblock_set", _wrap_Dwg_SECTION_typesettings_destblock_set},
{"LibreDWGc::Dwg_SECTION_typesettings_destblock_get", _wrap_Dwg_SECTION_typesettings_destblock_get},
{"LibreDWGc::Dwg_SECTION_typesettings_destfile_set", _wrap_Dwg_SECTION_typesettings_destfile_set},
{"LibreDWGc::Dwg_SECTION_typesettings_destfile_get", _wrap_Dwg_SECTION_typesettings_destfile_get},
{"LibreDWGc::Dwg_SECTION_typesettings_num_geom_set", _wrap_Dwg_SECTION_typesettings_num_geom_set},
{"LibreDWGc::Dwg_SECTION_typesettings_num_geom_get", _wrap_Dwg_SECTION_typesettings_num_geom_get},
{"LibreDWGc::Dwg_SECTION_typesettings_geom_set", _wrap_Dwg_SECTION_typesettings_geom_set},
{"LibreDWGc::Dwg_SECTION_typesettings_geom_get", _wrap_Dwg_SECTION_typesettings_geom_get},
{"LibreDWGc::new_Dwg_SECTION_typesettings", _wrap_new_Dwg_SECTION_typesettings},
{"LibreDWGc::delete_Dwg_SECTION_typesettings", _wrap_delete_Dwg_SECTION_typesettings},
{"LibreDWGc::Dwg_Object_SECTION_SETTINGS_parent_set", _wrap_Dwg_Object_SECTION_SETTINGS_parent_set},
{"LibreDWGc::Dwg_Object_SECTION_SETTINGS_parent_get", _wrap_Dwg_Object_SECTION_SETTINGS_parent_get},
{"LibreDWGc::Dwg_Object_SECTION_SETTINGS_curr_type_set", _wrap_Dwg_Object_SECTION_SETTINGS_curr_type_set},
{"LibreDWGc::Dwg_Object_SECTION_SETTINGS_curr_type_get", _wrap_Dwg_Object_SECTION_SETTINGS_curr_type_get},
{"LibreDWGc::Dwg_Object_SECTION_SETTINGS_num_types_set", _wrap_Dwg_Object_SECTION_SETTINGS_num_types_set},
{"LibreDWGc::Dwg_Object_SECTION_SETTINGS_num_types_get", _wrap_Dwg_Object_SECTION_SETTINGS_num_types_get},
{"LibreDWGc::Dwg_Object_SECTION_SETTINGS_types_set", _wrap_Dwg_Object_SECTION_SETTINGS_types_set},
{"LibreDWGc::Dwg_Object_SECTION_SETTINGS_types_get", _wrap_Dwg_Object_SECTION_SETTINGS_types_get},
{"LibreDWGc::new_Dwg_Object_SECTION_SETTINGS", _wrap_new_Dwg_Object_SECTION_SETTINGS},
{"LibreDWGc::delete_Dwg_Object_SECTION_SETTINGS", _wrap_delete_Dwg_Object_SECTION_SETTINGS},
{"LibreDWGc::Dwg_Object_LAYERFILTER_parent_set", _wrap_Dwg_Object_LAYERFILTER_parent_set},
{"LibreDWGc::Dwg_Object_LAYERFILTER_parent_get", _wrap_Dwg_Object_LAYERFILTER_parent_get},
{"LibreDWGc::Dwg_Object_LAYERFILTER_num_names_set", _wrap_Dwg_Object_LAYERFILTER_num_names_set},
{"LibreDWGc::Dwg_Object_LAYERFILTER_num_names_get", _wrap_Dwg_Object_LAYERFILTER_num_names_get},
{"LibreDWGc::Dwg_Object_LAYERFILTER_names_set", _wrap_Dwg_Object_LAYERFILTER_names_set},
{"LibreDWGc::Dwg_Object_LAYERFILTER_names_get", _wrap_Dwg_Object_LAYERFILTER_names_get},
{"LibreDWGc::new_Dwg_Object_LAYERFILTER", _wrap_new_Dwg_Object_LAYERFILTER},
{"LibreDWGc::delete_Dwg_Object_LAYERFILTER", _wrap_delete_Dwg_Object_LAYERFILTER},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_parent_set", _wrap_Dwg_Entity_ARCALIGNEDTEXT_parent_set},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_parent_get", _wrap_Dwg_Entity_ARCALIGNEDTEXT_parent_get},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_text_size_set", _wrap_Dwg_Entity_ARCALIGNEDTEXT_text_size_set},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_text_size_get", _wrap_Dwg_Entity_ARCALIGNEDTEXT_text_size_get},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_xscale_set", _wrap_Dwg_Entity_ARCALIGNEDTEXT_xscale_set},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_xscale_get", _wrap_Dwg_Entity_ARCALIGNEDTEXT_xscale_get},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_char_spacing_set", _wrap_Dwg_Entity_ARCALIGNEDTEXT_char_spacing_set},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_char_spacing_get", _wrap_Dwg_Entity_ARCALIGNEDTEXT_char_spacing_get},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_style_set", _wrap_Dwg_Entity_ARCALIGNEDTEXT_style_set},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_style_get", _wrap_Dwg_Entity_ARCALIGNEDTEXT_style_get},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_text_value_set", _wrap_Dwg_Entity_ARCALIGNEDTEXT_text_value_set},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_text_value_get", _wrap_Dwg_Entity_ARCALIGNEDTEXT_text_value_get},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_font_name_set", _wrap_Dwg_Entity_ARCALIGNEDTEXT_font_name_set},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_font_name_get", _wrap_Dwg_Entity_ARCALIGNEDTEXT_font_name_get},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_bigfont_name_set", _wrap_Dwg_Entity_ARCALIGNEDTEXT_bigfont_name_set},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_bigfont_name_get", _wrap_Dwg_Entity_ARCALIGNEDTEXT_bigfont_name_get},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_offset_from_arc_set", _wrap_Dwg_Entity_ARCALIGNEDTEXT_offset_from_arc_set},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_offset_from_arc_get", _wrap_Dwg_Entity_ARCALIGNEDTEXT_offset_from_arc_get},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_right_offset_set", _wrap_Dwg_Entity_ARCALIGNEDTEXT_right_offset_set},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_right_offset_get", _wrap_Dwg_Entity_ARCALIGNEDTEXT_right_offset_get},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_left_offset_set", _wrap_Dwg_Entity_ARCALIGNEDTEXT_left_offset_set},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_left_offset_get", _wrap_Dwg_Entity_ARCALIGNEDTEXT_left_offset_get},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_center_set", _wrap_Dwg_Entity_ARCALIGNEDTEXT_center_set},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_center_get", _wrap_Dwg_Entity_ARCALIGNEDTEXT_center_get},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_radius_set", _wrap_Dwg_Entity_ARCALIGNEDTEXT_radius_set},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_radius_get", _wrap_Dwg_Entity_ARCALIGNEDTEXT_radius_get},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_start_angle_set", _wrap_Dwg_Entity_ARCALIGNEDTEXT_start_angle_set},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_start_angle_get", _wrap_Dwg_Entity_ARCALIGNEDTEXT_start_angle_get},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_end_angle_set", _wrap_Dwg_Entity_ARCALIGNEDTEXT_end_angle_set},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_end_angle_get", _wrap_Dwg_Entity_ARCALIGNEDTEXT_end_angle_get},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_extrusion_set", _wrap_Dwg_Entity_ARCALIGNEDTEXT_extrusion_set},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_extrusion_get", _wrap_Dwg_Entity_ARCALIGNEDTEXT_extrusion_get},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_color_set", _wrap_Dwg_Entity_ARCALIGNEDTEXT_color_set},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_color_get", _wrap_Dwg_Entity_ARCALIGNEDTEXT_color_get},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_is_reverse_set", _wrap_Dwg_Entity_ARCALIGNEDTEXT_is_reverse_set},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_is_reverse_get", _wrap_Dwg_Entity_ARCALIGNEDTEXT_is_reverse_get},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_text_direction_set", _wrap_Dwg_Entity_ARCALIGNEDTEXT_text_direction_set},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_text_direction_get", _wrap_Dwg_Entity_ARCALIGNEDTEXT_text_direction_get},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_alignment_set", _wrap_Dwg_Entity_ARCALIGNEDTEXT_alignment_set},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_alignment_get", _wrap_Dwg_Entity_ARCALIGNEDTEXT_alignment_get},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_text_position_set", _wrap_Dwg_Entity_ARCALIGNEDTEXT_text_position_set},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_text_position_get", _wrap_Dwg_Entity_ARCALIGNEDTEXT_text_position_get},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_is_bold_set", _wrap_Dwg_Entity_ARCALIGNEDTEXT_is_bold_set},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_is_bold_get", _wrap_Dwg_Entity_ARCALIGNEDTEXT_is_bold_get},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_is_italic_set", _wrap_Dwg_Entity_ARCALIGNEDTEXT_is_italic_set},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_is_italic_get", _wrap_Dwg_Entity_ARCALIGNEDTEXT_is_italic_get},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_is_underlined_set", _wrap_Dwg_Entity_ARCALIGNEDTEXT_is_underlined_set},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_is_underlined_get", _wrap_Dwg_Entity_ARCALIGNEDTEXT_is_underlined_get},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_character_set_set", _wrap_Dwg_Entity_ARCALIGNEDTEXT_character_set_set},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_character_set_get", _wrap_Dwg_Entity_ARCALIGNEDTEXT_character_set_get},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_pitch_and_family_set", _wrap_Dwg_Entity_ARCALIGNEDTEXT_pitch_and_family_set},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_pitch_and_family_get", _wrap_Dwg_Entity_ARCALIGNEDTEXT_pitch_and_family_get},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_is_shx_set", _wrap_Dwg_Entity_ARCALIGNEDTEXT_is_shx_set},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_is_shx_get", _wrap_Dwg_Entity_ARCALIGNEDTEXT_is_shx_get},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_wizard_flag_set", _wrap_Dwg_Entity_ARCALIGNEDTEXT_wizard_flag_set},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_wizard_flag_get", _wrap_Dwg_Entity_ARCALIGNEDTEXT_wizard_flag_get},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_arc_handle_set", _wrap_Dwg_Entity_ARCALIGNEDTEXT_arc_handle_set},
{"LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_arc_handle_get", _wrap_Dwg_Entity_ARCALIGNEDTEXT_arc_handle_get},
{"LibreDWGc::new_Dwg_Entity_ARCALIGNEDTEXT", _wrap_new_Dwg_Entity_ARCALIGNEDTEXT},
{"LibreDWGc::delete_Dwg_Entity_ARCALIGNEDTEXT", _wrap_delete_Dwg_Entity_ARCALIGNEDTEXT},
{"LibreDWGc::Dwg_Entity_RTEXT_parent_set", _wrap_Dwg_Entity_RTEXT_parent_set},
{"LibreDWGc::Dwg_Entity_RTEXT_parent_get", _wrap_Dwg_Entity_RTEXT_parent_get},
{"LibreDWGc::Dwg_Entity_RTEXT_pt_set", _wrap_Dwg_Entity_RTEXT_pt_set},
{"LibreDWGc::Dwg_Entity_RTEXT_pt_get", _wrap_Dwg_Entity_RTEXT_pt_get},
{"LibreDWGc::Dwg_Entity_RTEXT_extrusion_set", _wrap_Dwg_Entity_RTEXT_extrusion_set},
{"LibreDWGc::Dwg_Entity_RTEXT_extrusion_get", _wrap_Dwg_Entity_RTEXT_extrusion_get},
{"LibreDWGc::Dwg_Entity_RTEXT_rotation_set", _wrap_Dwg_Entity_RTEXT_rotation_set},
{"LibreDWGc::Dwg_Entity_RTEXT_rotation_get", _wrap_Dwg_Entity_RTEXT_rotation_get},
{"LibreDWGc::Dwg_Entity_RTEXT_height_set", _wrap_Dwg_Entity_RTEXT_height_set},
{"LibreDWGc::Dwg_Entity_RTEXT_height_get", _wrap_Dwg_Entity_RTEXT_height_get},
{"LibreDWGc::Dwg_Entity_RTEXT_flags_set", _wrap_Dwg_Entity_RTEXT_flags_set},
{"LibreDWGc::Dwg_Entity_RTEXT_flags_get", _wrap_Dwg_Entity_RTEXT_flags_get},
{"LibreDWGc::Dwg_Entity_RTEXT_text_value_set", _wrap_Dwg_Entity_RTEXT_text_value_set},
{"LibreDWGc::Dwg_Entity_RTEXT_text_value_get", _wrap_Dwg_Entity_RTEXT_text_value_get},
{"LibreDWGc::Dwg_Entity_RTEXT_style_set", _wrap_Dwg_Entity_RTEXT_style_set},
{"LibreDWGc::Dwg_Entity_RTEXT_style_get", _wrap_Dwg_Entity_RTEXT_style_get},
{"LibreDWGc::new_Dwg_Entity_RTEXT", _wrap_new_Dwg_Entity_RTEXT},
{"LibreDWGc::delete_Dwg_Entity_RTEXT", _wrap_delete_Dwg_Entity_RTEXT},
{"LibreDWGc::Dwg_Entity_LAYOUTPRINTCONFIG_parent_set", _wrap_Dwg_Entity_LAYOUTPRINTCONFIG_parent_set},
{"LibreDWGc::Dwg_Entity_LAYOUTPRINTCONFIG_parent_get", _wrap_Dwg_Entity_LAYOUTPRINTCONFIG_parent_get},
{"LibreDWGc::Dwg_Entity_LAYOUTPRINTCONFIG_class_version_set", _wrap_Dwg_Entity_LAYOUTPRINTCONFIG_class_version_set},
{"LibreDWGc::Dwg_Entity_LAYOUTPRINTCONFIG_class_version_get", _wrap_Dwg_Entity_LAYOUTPRINTCONFIG_class_version_get},
{"LibreDWGc::Dwg_Entity_LAYOUTPRINTCONFIG_flag_set", _wrap_Dwg_Entity_LAYOUTPRINTCONFIG_flag_set},
{"LibreDWGc::Dwg_Entity_LAYOUTPRINTCONFIG_flag_get", _wrap_Dwg_Entity_LAYOUTPRINTCONFIG_flag_get},
{"LibreDWGc::new_Dwg_Entity_LAYOUTPRINTCONFIG", _wrap_new_Dwg_Entity_LAYOUTPRINTCONFIG},
{"LibreDWGc::delete_Dwg_Entity_LAYOUTPRINTCONFIG", _wrap_delete_Dwg_Entity_LAYOUTPRINTCONFIG},
{"LibreDWGc::Dwg_Object_ACMECOMMANDHISTORY_parent_set", _wrap_Dwg_Object_ACMECOMMANDHISTORY_parent_set},
{"LibreDWGc::Dwg_Object_ACMECOMMANDHISTORY_parent_get", _wrap_Dwg_Object_ACMECOMMANDHISTORY_parent_get},
{"LibreDWGc::Dwg_Object_ACMECOMMANDHISTORY_class_version_set", _wrap_Dwg_Object_ACMECOMMANDHISTORY_class_version_set},
{"LibreDWGc::Dwg_Object_ACMECOMMANDHISTORY_class_version_get", _wrap_Dwg_Object_ACMECOMMANDHISTORY_class_version_get},
{"LibreDWGc::new_Dwg_Object_ACMECOMMANDHISTORY", _wrap_new_Dwg_Object_ACMECOMMANDHISTORY},
{"LibreDWGc::delete_Dwg_Object_ACMECOMMANDHISTORY", _wrap_delete_Dwg_Object_ACMECOMMANDHISTORY},
{"LibreDWGc::Dwg_Object_ACMESCOPE_parent_set", _wrap_Dwg_Object_ACMESCOPE_parent_set},
{"LibreDWGc::Dwg_Object_ACMESCOPE_parent_get", _wrap_Dwg_Object_ACMESCOPE_parent_get},
{"LibreDWGc::Dwg_Object_ACMESCOPE_class_version_set", _wrap_Dwg_Object_ACMESCOPE_class_version_set},
{"LibreDWGc::Dwg_Object_ACMESCOPE_class_version_get", _wrap_Dwg_Object_ACMESCOPE_class_version_get},
{"LibreDWGc::new_Dwg_Object_ACMESCOPE", _wrap_new_Dwg_Object_ACMESCOPE},
{"LibreDWGc::delete_Dwg_Object_ACMESCOPE", _wrap_delete_Dwg_Object_ACMESCOPE},
{"LibreDWGc::Dwg_Object_ACMESTATEMGR_parent_set", _wrap_Dwg_Object_ACMESTATEMGR_parent_set},
{"LibreDWGc::Dwg_Object_ACMESTATEMGR_parent_get", _wrap_Dwg_Object_ACMESTATEMGR_parent_get},
{"LibreDWGc::Dwg_Object_ACMESTATEMGR_class_version_set", _wrap_Dwg_Object_ACMESTATEMGR_class_version_set},
{"LibreDWGc::Dwg_Object_ACMESTATEMGR_class_version_get", _wrap_Dwg_Object_ACMESTATEMGR_class_version_get},
{"LibreDWGc::new_Dwg_Object_ACMESTATEMGR", _wrap_new_Dwg_Object_ACMESTATEMGR},
{"LibreDWGc::delete_Dwg_Object_ACMESTATEMGR", _wrap_delete_Dwg_Object_ACMESTATEMGR},
{"LibreDWGc::Dwg_Object_CSACDOCUMENTOPTIONS_parent_set", _wrap_Dwg_Object_CSACDOCUMENTOPTIONS_parent_set},
{"LibreDWGc::Dwg_Object_CSACDOCUMENTOPTIONS_parent_get", _wrap_Dwg_Object_CSACDOCUMENTOPTIONS_parent_get},
{"LibreDWGc::Dwg_Object_CSACDOCUMENTOPTIONS_class_version_set", _wrap_Dwg_Object_CSACDOCUMENTOPTIONS_class_version_set},
{"LibreDWGc::Dwg_Object_CSACDOCUMENTOPTIONS_class_version_get", _wrap_Dwg_Object_CSACDOCUMENTOPTIONS_class_version_get},
{"LibreDWGc::new_Dwg_Object_CSACDOCUMENTOPTIONS", _wrap_new_Dwg_Object_CSACDOCUMENTOPTIONS},
{"LibreDWGc::delete_Dwg_Object_CSACDOCUMENTOPTIONS", _wrap_delete_Dwg_Object_CSACDOCUMENTOPTIONS},
{"LibreDWGc::Dwg_BLOCKPARAMETER_connection_code_set", _wrap_Dwg_BLOCKPARAMETER_connection_code_set},
{"LibreDWGc::Dwg_BLOCKPARAMETER_connection_code_get", _wrap_Dwg_BLOCKPARAMETER_connection_code_get},
{"LibreDWGc::Dwg_BLOCKPARAMETER_connection_name_set", _wrap_Dwg_BLOCKPARAMETER_connection_name_set},
{"LibreDWGc::Dwg_BLOCKPARAMETER_connection_name_get", _wrap_Dwg_BLOCKPARAMETER_connection_name_get},
{"LibreDWGc::new_Dwg_BLOCKPARAMETER_connection", _wrap_new_Dwg_BLOCKPARAMETER_connection},
{"LibreDWGc::delete_Dwg_BLOCKPARAMETER_connection", _wrap_delete_Dwg_BLOCKPARAMETER_connection},
{"LibreDWGc::Dwg_BLOCKPARAMETER_PropInfo_num_connections_set", _wrap_Dwg_BLOCKPARAMETER_PropInfo_num_connections_set},
{"LibreDWGc::Dwg_BLOCKPARAMETER_PropInfo_num_connections_get", _wrap_Dwg_BLOCKPARAMETER_PropInfo_num_connections_get},
{"LibreDWGc::Dwg_BLOCKPARAMETER_PropInfo_connections_set", _wrap_Dwg_BLOCKPARAMETER_PropInfo_connections_set},
{"LibreDWGc::Dwg_BLOCKPARAMETER_PropInfo_connections_get", _wrap_Dwg_BLOCKPARAMETER_PropInfo_connections_get},
{"LibreDWGc::new_Dwg_BLOCKPARAMETER_PropInfo", _wrap_new_Dwg_BLOCKPARAMETER_PropInfo},
{"LibreDWGc::delete_Dwg_BLOCKPARAMETER_PropInfo", _wrap_delete_Dwg_BLOCKPARAMETER_PropInfo},
{"LibreDWGc::Dwg_BLOCKPARAMVALUESET_desc_set", _wrap_Dwg_BLOCKPARAMVALUESET_desc_set},
{"LibreDWGc::Dwg_BLOCKPARAMVALUESET_desc_get", _wrap_Dwg_BLOCKPARAMVALUESET_desc_get},
{"LibreDWGc::Dwg_BLOCKPARAMVALUESET_flags_set", _wrap_Dwg_BLOCKPARAMVALUESET_flags_set},
{"LibreDWGc::Dwg_BLOCKPARAMVALUESET_flags_get", _wrap_Dwg_BLOCKPARAMVALUESET_flags_get},
{"LibreDWGc::Dwg_BLOCKPARAMVALUESET_minimum_set", _wrap_Dwg_BLOCKPARAMVALUESET_minimum_set},
{"LibreDWGc::Dwg_BLOCKPARAMVALUESET_minimum_get", _wrap_Dwg_BLOCKPARAMVALUESET_minimum_get},
{"LibreDWGc::Dwg_BLOCKPARAMVALUESET_maximum_set", _wrap_Dwg_BLOCKPARAMVALUESET_maximum_set},
{"LibreDWGc::Dwg_BLOCKPARAMVALUESET_maximum_get", _wrap_Dwg_BLOCKPARAMVALUESET_maximum_get},
{"LibreDWGc::Dwg_BLOCKPARAMVALUESET_increment_set", _wrap_Dwg_BLOCKPARAMVALUESET_increment_set},
{"LibreDWGc::Dwg_BLOCKPARAMVALUESET_increment_get", _wrap_Dwg_BLOCKPARAMVALUESET_increment_get},
{"LibreDWGc::Dwg_BLOCKPARAMVALUESET_num_valuelist_set", _wrap_Dwg_BLOCKPARAMVALUESET_num_valuelist_set},
{"LibreDWGc::Dwg_BLOCKPARAMVALUESET_num_valuelist_get", _wrap_Dwg_BLOCKPARAMVALUESET_num_valuelist_get},
{"LibreDWGc::Dwg_BLOCKPARAMVALUESET_valuelist_set", _wrap_Dwg_BLOCKPARAMVALUESET_valuelist_set},
{"LibreDWGc::Dwg_BLOCKPARAMVALUESET_valuelist_get", _wrap_Dwg_BLOCKPARAMVALUESET_valuelist_get},
{"LibreDWGc::new_Dwg_BLOCKPARAMVALUESET", _wrap_new_Dwg_BLOCKPARAMVALUESET},
{"LibreDWGc::delete_Dwg_BLOCKPARAMVALUESET", _wrap_delete_Dwg_BLOCKPARAMVALUESET},
{"LibreDWGc::Dwg_BLOCKACTION_connectionpts_code_set", _wrap_Dwg_BLOCKACTION_connectionpts_code_set},
{"LibreDWGc::Dwg_BLOCKACTION_connectionpts_code_get", _wrap_Dwg_BLOCKACTION_connectionpts_code_get},
{"LibreDWGc::Dwg_BLOCKACTION_connectionpts_name_set", _wrap_Dwg_BLOCKACTION_connectionpts_name_set},
{"LibreDWGc::Dwg_BLOCKACTION_connectionpts_name_get", _wrap_Dwg_BLOCKACTION_connectionpts_name_get},
{"LibreDWGc::new_Dwg_BLOCKACTION_connectionpts", _wrap_new_Dwg_BLOCKACTION_connectionpts},
{"LibreDWGc::delete_Dwg_BLOCKACTION_connectionpts", _wrap_delete_Dwg_BLOCKACTION_connectionpts},
{"LibreDWGc::Dwg_BLOCKVISIBILITYPARAMETER_state_parent_set", _wrap_Dwg_BLOCKVISIBILITYPARAMETER_state_parent_set},
{"LibreDWGc::Dwg_BLOCKVISIBILITYPARAMETER_state_parent_get", _wrap_Dwg_BLOCKVISIBILITYPARAMETER_state_parent_get},
{"LibreDWGc::Dwg_BLOCKVISIBILITYPARAMETER_state_name_set", _wrap_Dwg_BLOCKVISIBILITYPARAMETER_state_name_set},
{"LibreDWGc::Dwg_BLOCKVISIBILITYPARAMETER_state_name_get", _wrap_Dwg_BLOCKVISIBILITYPARAMETER_state_name_get},
{"LibreDWGc::Dwg_BLOCKVISIBILITYPARAMETER_state_num_blocks_set", _wrap_Dwg_BLOCKVISIBILITYPARAMETER_state_num_blocks_set},
{"LibreDWGc::Dwg_BLOCKVISIBILITYPARAMETER_state_num_blocks_get", _wrap_Dwg_BLOCKVISIBILITYPARAMETER_state_num_blocks_get},
{"LibreDWGc::Dwg_BLOCKVISIBILITYPARAMETER_state_blocks_set", _wrap_Dwg_BLOCKVISIBILITYPARAMETER_state_blocks_set},
{"LibreDWGc::Dwg_BLOCKVISIBILITYPARAMETER_state_blocks_get", _wrap_Dwg_BLOCKVISIBILITYPARAMETER_state_blocks_get},
{"LibreDWGc::Dwg_BLOCKVISIBILITYPARAMETER_state_num_params_set", _wrap_Dwg_BLOCKVISIBILITYPARAMETER_state_num_params_set},
{"LibreDWGc::Dwg_BLOCKVISIBILITYPARAMETER_state_num_params_get", _wrap_Dwg_BLOCKVISIBILITYPARAMETER_state_num_params_get},
{"LibreDWGc::Dwg_BLOCKVISIBILITYPARAMETER_state_params_set", _wrap_Dwg_BLOCKVISIBILITYPARAMETER_state_params_set},
{"LibreDWGc::Dwg_BLOCKVISIBILITYPARAMETER_state_params_get", _wrap_Dwg_BLOCKVISIBILITYPARAMETER_state_params_get},
{"LibreDWGc::new_Dwg_BLOCKVISIBILITYPARAMETER_state", _wrap_new_Dwg_BLOCKVISIBILITYPARAMETER_state},
{"LibreDWGc::delete_Dwg_BLOCKVISIBILITYPARAMETER_state", _wrap_delete_Dwg_BLOCKVISIBILITYPARAMETER_state},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_parent_set", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_parent_get", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_evalexpr_set", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_evalexpr_get", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_name_set", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_name_set},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_name_get", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_name_get},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_be_major_set", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_be_major_set},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_be_major_get", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_be_major_get},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_be_minor_set", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_be_minor_set},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_be_minor_get", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_be_minor_get},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_eed1071_set", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_eed1071_set},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_eed1071_get", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_eed1071_get},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_show_properties_set", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_show_properties_set},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_show_properties_get", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_show_properties_get},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_chain_actions_set", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_chain_actions_set},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_chain_actions_get", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_chain_actions_get},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_def_pt_set", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_def_pt_set},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_def_pt_get", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_def_pt_get},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_num_propinfos_set", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_num_propinfos_set},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_num_propinfos_get", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_num_propinfos_get},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_prop1_set", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_prop1_set},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_prop1_get", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_prop1_get},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_prop2_set", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_prop2_set},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_prop2_get", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_prop2_get},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_is_initialized_set", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_is_initialized_set},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_is_initialized_get", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_is_initialized_get},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_unknown_bool_set", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_unknown_bool_set},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_unknown_bool_get", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_unknown_bool_get},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_blockvisi_name_set", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_blockvisi_name_set},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_blockvisi_name_get", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_blockvisi_name_get},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_blockvisi_desc_set", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_blockvisi_desc_set},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_blockvisi_desc_get", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_blockvisi_desc_get},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_num_blocks_set", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_num_blocks_set},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_num_blocks_get", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_num_blocks_get},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_blocks_set", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_blocks_set},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_blocks_get", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_blocks_get},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_num_states_set", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_num_states_set},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_num_states_get", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_num_states_get},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_states_set", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_states_set},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_states_get", _wrap_Dwg_Object_BLOCKVISIBILITYPARAMETER_states_get},
{"LibreDWGc::new_Dwg_Object_BLOCKVISIBILITYPARAMETER", _wrap_new_Dwg_Object_BLOCKVISIBILITYPARAMETER},
{"LibreDWGc::delete_Dwg_Object_BLOCKVISIBILITYPARAMETER", _wrap_delete_Dwg_Object_BLOCKVISIBILITYPARAMETER},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_parent_set", _wrap_Dwg_Object_BLOCKVISIBILITYGRIP_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_parent_get", _wrap_Dwg_Object_BLOCKVISIBILITYGRIP_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_evalexpr_set", _wrap_Dwg_Object_BLOCKVISIBILITYGRIP_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_evalexpr_get", _wrap_Dwg_Object_BLOCKVISIBILITYGRIP_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_name_set", _wrap_Dwg_Object_BLOCKVISIBILITYGRIP_name_set},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_name_get", _wrap_Dwg_Object_BLOCKVISIBILITYGRIP_name_get},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_be_major_set", _wrap_Dwg_Object_BLOCKVISIBILITYGRIP_be_major_set},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_be_major_get", _wrap_Dwg_Object_BLOCKVISIBILITYGRIP_be_major_get},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_be_minor_set", _wrap_Dwg_Object_BLOCKVISIBILITYGRIP_be_minor_set},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_be_minor_get", _wrap_Dwg_Object_BLOCKVISIBILITYGRIP_be_minor_get},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_eed1071_set", _wrap_Dwg_Object_BLOCKVISIBILITYGRIP_eed1071_set},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_eed1071_get", _wrap_Dwg_Object_BLOCKVISIBILITYGRIP_eed1071_get},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_bg_bl91_set", _wrap_Dwg_Object_BLOCKVISIBILITYGRIP_bg_bl91_set},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_bg_bl91_get", _wrap_Dwg_Object_BLOCKVISIBILITYGRIP_bg_bl91_get},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_bg_bl92_set", _wrap_Dwg_Object_BLOCKVISIBILITYGRIP_bg_bl92_set},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_bg_bl92_get", _wrap_Dwg_Object_BLOCKVISIBILITYGRIP_bg_bl92_get},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_bg_location_set", _wrap_Dwg_Object_BLOCKVISIBILITYGRIP_bg_location_set},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_bg_location_get", _wrap_Dwg_Object_BLOCKVISIBILITYGRIP_bg_location_get},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_bg_insert_cycling_set", _wrap_Dwg_Object_BLOCKVISIBILITYGRIP_bg_insert_cycling_set},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_bg_insert_cycling_get", _wrap_Dwg_Object_BLOCKVISIBILITYGRIP_bg_insert_cycling_get},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_bg_insert_cycling_weight_set", _wrap_Dwg_Object_BLOCKVISIBILITYGRIP_bg_insert_cycling_weight_set},
{"LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_bg_insert_cycling_weight_get", _wrap_Dwg_Object_BLOCKVISIBILITYGRIP_bg_insert_cycling_weight_get},
{"LibreDWGc::new_Dwg_Object_BLOCKVISIBILITYGRIP", _wrap_new_Dwg_Object_BLOCKVISIBILITYGRIP},
{"LibreDWGc::delete_Dwg_Object_BLOCKVISIBILITYGRIP", _wrap_delete_Dwg_Object_BLOCKVISIBILITYGRIP},
{"LibreDWGc::Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_parent_set", _wrap_Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_parent_get", _wrap_Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_evalexpr_set", _wrap_Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_evalexpr_get", _wrap_Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_grip_type_set", _wrap_Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_grip_type_set},
{"LibreDWGc::Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_grip_type_get", _wrap_Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_grip_type_get},
{"LibreDWGc::Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_grip_expr_set", _wrap_Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_grip_expr_set},
{"LibreDWGc::Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_grip_expr_get", _wrap_Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_grip_expr_get},
{"LibreDWGc::new_Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT", _wrap_new_Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT},
{"LibreDWGc::delete_Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT", _wrap_delete_Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT},
{"LibreDWGc::Dwg_Object_BREAKDATA_parent_set", _wrap_Dwg_Object_BREAKDATA_parent_set},
{"LibreDWGc::Dwg_Object_BREAKDATA_parent_get", _wrap_Dwg_Object_BREAKDATA_parent_get},
{"LibreDWGc::Dwg_Object_BREAKDATA_num_pointrefs_set", _wrap_Dwg_Object_BREAKDATA_num_pointrefs_set},
{"LibreDWGc::Dwg_Object_BREAKDATA_num_pointrefs_get", _wrap_Dwg_Object_BREAKDATA_num_pointrefs_get},
{"LibreDWGc::Dwg_Object_BREAKDATA_pointrefs_set", _wrap_Dwg_Object_BREAKDATA_pointrefs_set},
{"LibreDWGc::Dwg_Object_BREAKDATA_pointrefs_get", _wrap_Dwg_Object_BREAKDATA_pointrefs_get},
{"LibreDWGc::Dwg_Object_BREAKDATA_dimref_set", _wrap_Dwg_Object_BREAKDATA_dimref_set},
{"LibreDWGc::Dwg_Object_BREAKDATA_dimref_get", _wrap_Dwg_Object_BREAKDATA_dimref_get},
{"LibreDWGc::new_Dwg_Object_BREAKDATA", _wrap_new_Dwg_Object_BREAKDATA},
{"LibreDWGc::delete_Dwg_Object_BREAKDATA", _wrap_delete_Dwg_Object_BREAKDATA},
{"LibreDWGc::Dwg_Object_BREAKPOINTREF_parent_set", _wrap_Dwg_Object_BREAKPOINTREF_parent_set},
{"LibreDWGc::Dwg_Object_BREAKPOINTREF_parent_get", _wrap_Dwg_Object_BREAKPOINTREF_parent_get},
{"LibreDWGc::new_Dwg_Object_BREAKPOINTREF", _wrap_new_Dwg_Object_BREAKPOINTREF},
{"LibreDWGc::delete_Dwg_Object_BREAKPOINTREF", _wrap_delete_Dwg_Object_BREAKPOINTREF},
{"LibreDWGc::Dwg_Entity_FLIPGRIPENTITY_parent_set", _wrap_Dwg_Entity_FLIPGRIPENTITY_parent_set},
{"LibreDWGc::Dwg_Entity_FLIPGRIPENTITY_parent_get", _wrap_Dwg_Entity_FLIPGRIPENTITY_parent_get},
{"LibreDWGc::new_Dwg_Entity_FLIPGRIPENTITY", _wrap_new_Dwg_Entity_FLIPGRIPENTITY},
{"LibreDWGc::delete_Dwg_Entity_FLIPGRIPENTITY", _wrap_delete_Dwg_Entity_FLIPGRIPENTITY},
{"LibreDWGc::Dwg_Entity_LINEARGRIPENTITY_parent_set", _wrap_Dwg_Entity_LINEARGRIPENTITY_parent_set},
{"LibreDWGc::Dwg_Entity_LINEARGRIPENTITY_parent_get", _wrap_Dwg_Entity_LINEARGRIPENTITY_parent_get},
{"LibreDWGc::new_Dwg_Entity_LINEARGRIPENTITY", _wrap_new_Dwg_Entity_LINEARGRIPENTITY},
{"LibreDWGc::delete_Dwg_Entity_LINEARGRIPENTITY", _wrap_delete_Dwg_Entity_LINEARGRIPENTITY},
{"LibreDWGc::Dwg_Entity_POLARGRIPENTITY_parent_set", _wrap_Dwg_Entity_POLARGRIPENTITY_parent_set},
{"LibreDWGc::Dwg_Entity_POLARGRIPENTITY_parent_get", _wrap_Dwg_Entity_POLARGRIPENTITY_parent_get},
{"LibreDWGc::new_Dwg_Entity_POLARGRIPENTITY", _wrap_new_Dwg_Entity_POLARGRIPENTITY},
{"LibreDWGc::delete_Dwg_Entity_POLARGRIPENTITY", _wrap_delete_Dwg_Entity_POLARGRIPENTITY},
{"LibreDWGc::Dwg_Entity_ROTATIONGRIPENTITY_parent_set", _wrap_Dwg_Entity_ROTATIONGRIPENTITY_parent_set},
{"LibreDWGc::Dwg_Entity_ROTATIONGRIPENTITY_parent_get", _wrap_Dwg_Entity_ROTATIONGRIPENTITY_parent_get},
{"LibreDWGc::new_Dwg_Entity_ROTATIONGRIPENTITY", _wrap_new_Dwg_Entity_ROTATIONGRIPENTITY},
{"LibreDWGc::delete_Dwg_Entity_ROTATIONGRIPENTITY", _wrap_delete_Dwg_Entity_ROTATIONGRIPENTITY},
{"LibreDWGc::Dwg_Entity_VISIBILITYGRIPENTITY_parent_set", _wrap_Dwg_Entity_VISIBILITYGRIPENTITY_parent_set},
{"LibreDWGc::Dwg_Entity_VISIBILITYGRIPENTITY_parent_get", _wrap_Dwg_Entity_VISIBILITYGRIPENTITY_parent_get},
{"LibreDWGc::new_Dwg_Entity_VISIBILITYGRIPENTITY", _wrap_new_Dwg_Entity_VISIBILITYGRIPENTITY},
{"LibreDWGc::delete_Dwg_Entity_VISIBILITYGRIPENTITY", _wrap_delete_Dwg_Entity_VISIBILITYGRIPENTITY},
{"LibreDWGc::Dwg_Entity_XYGRIPENTITY_parent_set", _wrap_Dwg_Entity_XYGRIPENTITY_parent_set},
{"LibreDWGc::Dwg_Entity_XYGRIPENTITY_parent_get", _wrap_Dwg_Entity_XYGRIPENTITY_parent_get},
{"LibreDWGc::new_Dwg_Entity_XYGRIPENTITY", _wrap_new_Dwg_Entity_XYGRIPENTITY},
{"LibreDWGc::delete_Dwg_Entity_XYGRIPENTITY", _wrap_delete_Dwg_Entity_XYGRIPENTITY},
{"LibreDWGc::Dwg_Entity_ALIGNMENTPARAMETERENTITY_parent_set", _wrap_Dwg_Entity_ALIGNMENTPARAMETERENTITY_parent_set},
{"LibreDWGc::Dwg_Entity_ALIGNMENTPARAMETERENTITY_parent_get", _wrap_Dwg_Entity_ALIGNMENTPARAMETERENTITY_parent_get},
{"LibreDWGc::new_Dwg_Entity_ALIGNMENTPARAMETERENTITY", _wrap_new_Dwg_Entity_ALIGNMENTPARAMETERENTITY},
{"LibreDWGc::delete_Dwg_Entity_ALIGNMENTPARAMETERENTITY", _wrap_delete_Dwg_Entity_ALIGNMENTPARAMETERENTITY},
{"LibreDWGc::Dwg_Entity_BASEPOINTPARAMETERENTITY_parent_set", _wrap_Dwg_Entity_BASEPOINTPARAMETERENTITY_parent_set},
{"LibreDWGc::Dwg_Entity_BASEPOINTPARAMETERENTITY_parent_get", _wrap_Dwg_Entity_BASEPOINTPARAMETERENTITY_parent_get},
{"LibreDWGc::new_Dwg_Entity_BASEPOINTPARAMETERENTITY", _wrap_new_Dwg_Entity_BASEPOINTPARAMETERENTITY},
{"LibreDWGc::delete_Dwg_Entity_BASEPOINTPARAMETERENTITY", _wrap_delete_Dwg_Entity_BASEPOINTPARAMETERENTITY},
{"LibreDWGc::Dwg_Entity_FLIPPARAMETERENTITY_parent_set", _wrap_Dwg_Entity_FLIPPARAMETERENTITY_parent_set},
{"LibreDWGc::Dwg_Entity_FLIPPARAMETERENTITY_parent_get", _wrap_Dwg_Entity_FLIPPARAMETERENTITY_parent_get},
{"LibreDWGc::new_Dwg_Entity_FLIPPARAMETERENTITY", _wrap_new_Dwg_Entity_FLIPPARAMETERENTITY},
{"LibreDWGc::delete_Dwg_Entity_FLIPPARAMETERENTITY", _wrap_delete_Dwg_Entity_FLIPPARAMETERENTITY},
{"LibreDWGc::Dwg_Entity_LINEARPARAMETERENTITY_parent_set", _wrap_Dwg_Entity_LINEARPARAMETERENTITY_parent_set},
{"LibreDWGc::Dwg_Entity_LINEARPARAMETERENTITY_parent_get", _wrap_Dwg_Entity_LINEARPARAMETERENTITY_parent_get},
{"LibreDWGc::new_Dwg_Entity_LINEARPARAMETERENTITY", _wrap_new_Dwg_Entity_LINEARPARAMETERENTITY},
{"LibreDWGc::delete_Dwg_Entity_LINEARPARAMETERENTITY", _wrap_delete_Dwg_Entity_LINEARPARAMETERENTITY},
{"LibreDWGc::Dwg_Entity_POINTPARAMETERENTITY_parent_set", _wrap_Dwg_Entity_POINTPARAMETERENTITY_parent_set},
{"LibreDWGc::Dwg_Entity_POINTPARAMETERENTITY_parent_get", _wrap_Dwg_Entity_POINTPARAMETERENTITY_parent_get},
{"LibreDWGc::new_Dwg_Entity_POINTPARAMETERENTITY", _wrap_new_Dwg_Entity_POINTPARAMETERENTITY},
{"LibreDWGc::delete_Dwg_Entity_POINTPARAMETERENTITY", _wrap_delete_Dwg_Entity_POINTPARAMETERENTITY},
{"LibreDWGc::Dwg_Entity_ROTATIONPARAMETERENTITY_parent_set", _wrap_Dwg_Entity_ROTATIONPARAMETERENTITY_parent_set},
{"LibreDWGc::Dwg_Entity_ROTATIONPARAMETERENTITY_parent_get", _wrap_Dwg_Entity_ROTATIONPARAMETERENTITY_parent_get},
{"LibreDWGc::new_Dwg_Entity_ROTATIONPARAMETERENTITY", _wrap_new_Dwg_Entity_ROTATIONPARAMETERENTITY},
{"LibreDWGc::delete_Dwg_Entity_ROTATIONPARAMETERENTITY", _wrap_delete_Dwg_Entity_ROTATIONPARAMETERENTITY},
{"LibreDWGc::Dwg_Entity_VISIBILITYPARAMETERENTITY_parent_set", _wrap_Dwg_Entity_VISIBILITYPARAMETERENTITY_parent_set},
{"LibreDWGc::Dwg_Entity_VISIBILITYPARAMETERENTITY_parent_get", _wrap_Dwg_Entity_VISIBILITYPARAMETERENTITY_parent_get},
{"LibreDWGc::new_Dwg_Entity_VISIBILITYPARAMETERENTITY", _wrap_new_Dwg_Entity_VISIBILITYPARAMETERENTITY},
{"LibreDWGc::delete_Dwg_Entity_VISIBILITYPARAMETERENTITY", _wrap_delete_Dwg_Entity_VISIBILITYPARAMETERENTITY},
{"LibreDWGc::Dwg_Entity_XYPARAMETERENTITY_parent_set", _wrap_Dwg_Entity_XYPARAMETERENTITY_parent_set},
{"LibreDWGc::Dwg_Entity_XYPARAMETERENTITY_parent_get", _wrap_Dwg_Entity_XYPARAMETERENTITY_parent_get},
{"LibreDWGc::new_Dwg_Entity_XYPARAMETERENTITY", _wrap_new_Dwg_Entity_XYPARAMETERENTITY},
{"LibreDWGc::delete_Dwg_Entity_XYPARAMETERENTITY", _wrap_delete_Dwg_Entity_XYPARAMETERENTITY},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_parent_set", _wrap_Dwg_Object_BLOCKALIGNMENTGRIP_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_parent_get", _wrap_Dwg_Object_BLOCKALIGNMENTGRIP_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_evalexpr_set", _wrap_Dwg_Object_BLOCKALIGNMENTGRIP_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_evalexpr_get", _wrap_Dwg_Object_BLOCKALIGNMENTGRIP_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_name_set", _wrap_Dwg_Object_BLOCKALIGNMENTGRIP_name_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_name_get", _wrap_Dwg_Object_BLOCKALIGNMENTGRIP_name_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_be_major_set", _wrap_Dwg_Object_BLOCKALIGNMENTGRIP_be_major_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_be_major_get", _wrap_Dwg_Object_BLOCKALIGNMENTGRIP_be_major_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_be_minor_set", _wrap_Dwg_Object_BLOCKALIGNMENTGRIP_be_minor_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_be_minor_get", _wrap_Dwg_Object_BLOCKALIGNMENTGRIP_be_minor_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_eed1071_set", _wrap_Dwg_Object_BLOCKALIGNMENTGRIP_eed1071_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_eed1071_get", _wrap_Dwg_Object_BLOCKALIGNMENTGRIP_eed1071_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_bg_bl91_set", _wrap_Dwg_Object_BLOCKALIGNMENTGRIP_bg_bl91_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_bg_bl91_get", _wrap_Dwg_Object_BLOCKALIGNMENTGRIP_bg_bl91_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_bg_bl92_set", _wrap_Dwg_Object_BLOCKALIGNMENTGRIP_bg_bl92_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_bg_bl92_get", _wrap_Dwg_Object_BLOCKALIGNMENTGRIP_bg_bl92_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_bg_location_set", _wrap_Dwg_Object_BLOCKALIGNMENTGRIP_bg_location_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_bg_location_get", _wrap_Dwg_Object_BLOCKALIGNMENTGRIP_bg_location_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_bg_insert_cycling_set", _wrap_Dwg_Object_BLOCKALIGNMENTGRIP_bg_insert_cycling_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_bg_insert_cycling_get", _wrap_Dwg_Object_BLOCKALIGNMENTGRIP_bg_insert_cycling_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_bg_insert_cycling_weight_set", _wrap_Dwg_Object_BLOCKALIGNMENTGRIP_bg_insert_cycling_weight_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_bg_insert_cycling_weight_get", _wrap_Dwg_Object_BLOCKALIGNMENTGRIP_bg_insert_cycling_weight_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_orientation_set", _wrap_Dwg_Object_BLOCKALIGNMENTGRIP_orientation_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_orientation_get", _wrap_Dwg_Object_BLOCKALIGNMENTGRIP_orientation_get},
{"LibreDWGc::new_Dwg_Object_BLOCKALIGNMENTGRIP", _wrap_new_Dwg_Object_BLOCKALIGNMENTGRIP},
{"LibreDWGc::delete_Dwg_Object_BLOCKALIGNMENTGRIP", _wrap_delete_Dwg_Object_BLOCKALIGNMENTGRIP},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_parent_set", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_parent_get", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_evalexpr_set", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_evalexpr_get", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_name_set", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_name_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_name_get", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_name_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_be_major_set", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_be_major_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_be_major_get", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_be_major_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_be_minor_set", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_be_minor_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_be_minor_get", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_be_minor_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_eed1071_set", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_eed1071_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_eed1071_get", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_eed1071_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_show_properties_set", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_show_properties_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_show_properties_get", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_show_properties_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_chain_actions_set", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_chain_actions_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_chain_actions_get", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_chain_actions_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_def_basept_set", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_def_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_def_basept_get", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_def_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_def_endpt_set", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_def_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_def_endpt_get", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_def_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_prop1_set", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_prop1_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_prop1_get", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_prop1_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_prop2_set", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_prop2_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_prop2_get", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_prop2_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_prop3_set", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_prop3_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_prop3_get", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_prop3_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_prop4_set", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_prop4_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_prop4_get", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_prop4_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_prop_states_set", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_prop_states_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_prop_states_get", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_prop_states_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_parameter_base_location_set", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_parameter_base_location_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_parameter_base_location_get", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_parameter_base_location_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_upd_basept_set", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_upd_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_upd_basept_get", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_upd_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_basept_set", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_basept_get", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_upd_endpt_set", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_upd_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_upd_endpt_get", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_upd_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_endpt_set", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_endpt_get", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_align_perpendicular_set", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_align_perpendicular_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_align_perpendicular_get", _wrap_Dwg_Object_BLOCKALIGNMENTPARAMETER_align_perpendicular_get},
{"LibreDWGc::new_Dwg_Object_BLOCKALIGNMENTPARAMETER", _wrap_new_Dwg_Object_BLOCKALIGNMENTPARAMETER},
{"LibreDWGc::delete_Dwg_Object_BLOCKALIGNMENTPARAMETER", _wrap_delete_Dwg_Object_BLOCKALIGNMENTPARAMETER},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_parent_set", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_parent_get", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_evalexpr_set", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_evalexpr_get", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_name_set", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_name_set},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_name_get", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_name_get},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_be_major_set", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_be_major_set},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_be_major_get", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_be_major_get},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_be_minor_set", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_be_minor_set},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_be_minor_get", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_be_minor_get},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_eed1071_set", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_eed1071_set},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_eed1071_get", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_eed1071_get},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_show_properties_set", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_show_properties_set},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_show_properties_get", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_show_properties_get},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_chain_actions_set", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_chain_actions_set},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_chain_actions_get", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_chain_actions_get},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_def_basept_set", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_def_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_def_basept_get", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_def_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_def_endpt_set", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_def_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_def_endpt_get", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_def_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop1_set", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop1_set},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop1_get", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop1_get},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop2_set", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop2_set},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop2_get", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop2_get},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop3_set", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop3_set},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop3_get", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop3_get},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop4_set", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop4_set},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop4_get", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop4_get},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop_states_set", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop_states_set},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop_states_get", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop_states_get},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_parameter_base_location_set", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_parameter_base_location_set},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_parameter_base_location_get", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_parameter_base_location_get},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_upd_basept_set", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_upd_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_upd_basept_get", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_upd_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_basept_set", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_basept_get", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_upd_endpt_set", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_upd_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_upd_endpt_get", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_upd_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_endpt_set", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_endpt_get", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_dependency_set", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_dependency_set},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_dependency_get", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_dependency_get},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_center_pt_set", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_center_pt_set},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_center_pt_get", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_center_pt_get},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_end_pt_set", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_end_pt_set},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_end_pt_get", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_end_pt_get},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_expr_name_set", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_expr_name_set},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_expr_name_get", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_expr_name_get},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_expr_description_set", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_expr_description_set},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_expr_description_get", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_expr_description_get},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_angle_set", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_angle_set},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_angle_get", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_angle_get},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_orientation_on_both_grips_set", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_orientation_on_both_grips_set},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_orientation_on_both_grips_get", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_orientation_on_both_grips_get},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_value_set_set", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_value_set_set},
{"LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_value_set_get", _wrap_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_value_set_get},
{"LibreDWGc::new_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER", _wrap_new_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER},
{"LibreDWGc::delete_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER", _wrap_delete_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_parent_set", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_parent_get", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_evalexpr_set", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_evalexpr_get", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_name_set", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_name_set},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_name_get", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_name_get},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_be_major_set", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_be_major_set},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_be_major_get", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_be_major_get},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_be_minor_set", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_be_minor_set},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_be_minor_get", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_be_minor_get},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_eed1071_set", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_eed1071_set},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_eed1071_get", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_eed1071_get},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_show_properties_set", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_show_properties_set},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_show_properties_get", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_show_properties_get},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_chain_actions_set", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_chain_actions_set},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_chain_actions_get", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_chain_actions_get},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_def_basept_set", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_def_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_def_basept_get", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_def_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_def_endpt_set", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_def_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_def_endpt_get", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_def_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop1_set", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop1_set},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop1_get", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop1_get},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop2_set", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop2_set},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop2_get", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop2_get},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop3_set", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop3_set},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop3_get", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop3_get},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop4_set", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop4_set},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop4_get", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop4_get},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop_states_set", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop_states_set},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop_states_get", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop_states_get},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_parameter_base_location_set", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_parameter_base_location_set},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_parameter_base_location_get", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_parameter_base_location_get},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_upd_basept_set", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_upd_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_upd_basept_get", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_upd_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_basept_set", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_basept_get", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_upd_endpt_set", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_upd_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_upd_endpt_get", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_upd_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_endpt_set", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_endpt_get", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_dependency_set", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_dependency_set},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_dependency_get", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_dependency_get},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_expr_name_set", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_expr_name_set},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_expr_name_get", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_expr_name_get},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_expr_description_set", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_expr_description_set},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_expr_description_get", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_expr_description_get},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_distance_set", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_distance_set},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_distance_get", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_distance_get},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_orientation_on_both_grips_set", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_orientation_on_both_grips_set},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_orientation_on_both_grips_get", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_orientation_on_both_grips_get},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_value_set_set", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_value_set_set},
{"LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_value_set_get", _wrap_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_value_set_get},
{"LibreDWGc::new_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER", _wrap_new_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER},
{"LibreDWGc::delete_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER", _wrap_delete_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_parent_set", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_parent_get", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_evalexpr_set", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_evalexpr_get", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_name_set", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_name_set},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_name_get", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_name_get},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_be_major_set", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_be_major_set},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_be_major_get", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_be_major_get},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_be_minor_set", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_be_minor_set},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_be_minor_get", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_be_minor_get},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_eed1071_set", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_eed1071_set},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_eed1071_get", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_eed1071_get},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_show_properties_set", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_show_properties_set},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_show_properties_get", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_show_properties_get},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_chain_actions_set", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_chain_actions_set},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_chain_actions_get", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_chain_actions_get},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_def_basept_set", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_def_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_def_basept_get", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_def_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_def_endpt_set", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_def_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_def_endpt_get", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_def_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop1_set", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop1_set},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop1_get", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop1_get},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop2_set", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop2_set},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop2_get", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop2_get},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop3_set", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop3_set},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop3_get", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop3_get},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop4_set", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop4_set},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop4_get", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop4_get},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop_states_set", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop_states_set},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop_states_get", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop_states_get},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_parameter_base_location_set", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_parameter_base_location_set},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_parameter_base_location_get", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_parameter_base_location_get},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_upd_basept_set", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_upd_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_upd_basept_get", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_upd_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_basept_set", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_basept_get", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_upd_endpt_set", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_upd_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_upd_endpt_get", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_upd_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_endpt_set", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_endpt_get", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_dependency_set", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_dependency_set},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_dependency_get", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_dependency_get},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_expr_name_set", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_expr_name_set},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_expr_name_get", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_expr_name_get},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_expr_description_set", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_expr_description_set},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_expr_description_get", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_expr_description_get},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_distance_set", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_distance_set},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_distance_get", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_distance_get},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_value_set_set", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_value_set_set},
{"LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_value_set_get", _wrap_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_value_set_get},
{"LibreDWGc::new_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER", _wrap_new_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER},
{"LibreDWGc::delete_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER", _wrap_delete_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER},
{"LibreDWGc::Dwg_Object_BLOCKARRAYACTION_parent_set", _wrap_Dwg_Object_BLOCKARRAYACTION_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKARRAYACTION_parent_get", _wrap_Dwg_Object_BLOCKARRAYACTION_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKARRAYACTION_evalexpr_set", _wrap_Dwg_Object_BLOCKARRAYACTION_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKARRAYACTION_evalexpr_get", _wrap_Dwg_Object_BLOCKARRAYACTION_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKARRAYACTION_name_set", _wrap_Dwg_Object_BLOCKARRAYACTION_name_set},
{"LibreDWGc::Dwg_Object_BLOCKARRAYACTION_name_get", _wrap_Dwg_Object_BLOCKARRAYACTION_name_get},
{"LibreDWGc::Dwg_Object_BLOCKARRAYACTION_be_major_set", _wrap_Dwg_Object_BLOCKARRAYACTION_be_major_set},
{"LibreDWGc::Dwg_Object_BLOCKARRAYACTION_be_major_get", _wrap_Dwg_Object_BLOCKARRAYACTION_be_major_get},
{"LibreDWGc::Dwg_Object_BLOCKARRAYACTION_be_minor_set", _wrap_Dwg_Object_BLOCKARRAYACTION_be_minor_set},
{"LibreDWGc::Dwg_Object_BLOCKARRAYACTION_be_minor_get", _wrap_Dwg_Object_BLOCKARRAYACTION_be_minor_get},
{"LibreDWGc::Dwg_Object_BLOCKARRAYACTION_eed1071_set", _wrap_Dwg_Object_BLOCKARRAYACTION_eed1071_set},
{"LibreDWGc::Dwg_Object_BLOCKARRAYACTION_eed1071_get", _wrap_Dwg_Object_BLOCKARRAYACTION_eed1071_get},
{"LibreDWGc::Dwg_Object_BLOCKARRAYACTION_display_location_set", _wrap_Dwg_Object_BLOCKARRAYACTION_display_location_set},
{"LibreDWGc::Dwg_Object_BLOCKARRAYACTION_display_location_get", _wrap_Dwg_Object_BLOCKARRAYACTION_display_location_get},
{"LibreDWGc::Dwg_Object_BLOCKARRAYACTION_num_actions_set", _wrap_Dwg_Object_BLOCKARRAYACTION_num_actions_set},
{"LibreDWGc::Dwg_Object_BLOCKARRAYACTION_num_actions_get", _wrap_Dwg_Object_BLOCKARRAYACTION_num_actions_get},
{"LibreDWGc::Dwg_Object_BLOCKARRAYACTION_actions_set", _wrap_Dwg_Object_BLOCKARRAYACTION_actions_set},
{"LibreDWGc::Dwg_Object_BLOCKARRAYACTION_actions_get", _wrap_Dwg_Object_BLOCKARRAYACTION_actions_get},
{"LibreDWGc::Dwg_Object_BLOCKARRAYACTION_num_deps_set", _wrap_Dwg_Object_BLOCKARRAYACTION_num_deps_set},
{"LibreDWGc::Dwg_Object_BLOCKARRAYACTION_num_deps_get", _wrap_Dwg_Object_BLOCKARRAYACTION_num_deps_get},
{"LibreDWGc::Dwg_Object_BLOCKARRAYACTION_deps_set", _wrap_Dwg_Object_BLOCKARRAYACTION_deps_set},
{"LibreDWGc::Dwg_Object_BLOCKARRAYACTION_deps_get", _wrap_Dwg_Object_BLOCKARRAYACTION_deps_get},
{"LibreDWGc::Dwg_Object_BLOCKARRAYACTION_conn_pts_set", _wrap_Dwg_Object_BLOCKARRAYACTION_conn_pts_set},
{"LibreDWGc::Dwg_Object_BLOCKARRAYACTION_conn_pts_get", _wrap_Dwg_Object_BLOCKARRAYACTION_conn_pts_get},
{"LibreDWGc::Dwg_Object_BLOCKARRAYACTION_column_offset_set", _wrap_Dwg_Object_BLOCKARRAYACTION_column_offset_set},
{"LibreDWGc::Dwg_Object_BLOCKARRAYACTION_column_offset_get", _wrap_Dwg_Object_BLOCKARRAYACTION_column_offset_get},
{"LibreDWGc::Dwg_Object_BLOCKARRAYACTION_row_offset_set", _wrap_Dwg_Object_BLOCKARRAYACTION_row_offset_set},
{"LibreDWGc::Dwg_Object_BLOCKARRAYACTION_row_offset_get", _wrap_Dwg_Object_BLOCKARRAYACTION_row_offset_get},
{"LibreDWGc::new_Dwg_Object_BLOCKARRAYACTION", _wrap_new_Dwg_Object_BLOCKARRAYACTION},
{"LibreDWGc::delete_Dwg_Object_BLOCKARRAYACTION", _wrap_delete_Dwg_Object_BLOCKARRAYACTION},
{"LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_parent_set", _wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_parent_get", _wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_evalexpr_set", _wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_evalexpr_get", _wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_name_set", _wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_name_set},
{"LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_name_get", _wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_name_get},
{"LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_be_major_set", _wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_be_major_set},
{"LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_be_major_get", _wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_be_major_get},
{"LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_be_minor_set", _wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_be_minor_set},
{"LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_be_minor_get", _wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_be_minor_get},
{"LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_eed1071_set", _wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_eed1071_set},
{"LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_eed1071_get", _wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_eed1071_get},
{"LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_show_properties_set", _wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_show_properties_set},
{"LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_show_properties_get", _wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_show_properties_get},
{"LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_chain_actions_set", _wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_chain_actions_set},
{"LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_chain_actions_get", _wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_chain_actions_get},
{"LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_def_pt_set", _wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_def_pt_set},
{"LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_def_pt_get", _wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_def_pt_get},
{"LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_num_propinfos_set", _wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_num_propinfos_set},
{"LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_num_propinfos_get", _wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_num_propinfos_get},
{"LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_prop1_set", _wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_prop1_set},
{"LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_prop1_get", _wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_prop1_get},
{"LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_prop2_set", _wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_prop2_set},
{"LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_prop2_get", _wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_prop2_get},
{"LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_pt_set", _wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_pt_set},
{"LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_pt_get", _wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_pt_get},
{"LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_base_pt_set", _wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_base_pt_set},
{"LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_base_pt_get", _wrap_Dwg_Object_BLOCKBASEPOINTPARAMETER_base_pt_get},
{"LibreDWGc::new_Dwg_Object_BLOCKBASEPOINTPARAMETER", _wrap_new_Dwg_Object_BLOCKBASEPOINTPARAMETER},
{"LibreDWGc::delete_Dwg_Object_BLOCKBASEPOINTPARAMETER", _wrap_delete_Dwg_Object_BLOCKBASEPOINTPARAMETER},
{"LibreDWGc::Dwg_Object_BLOCKFLIPACTION_parent_set", _wrap_Dwg_Object_BLOCKFLIPACTION_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPACTION_parent_get", _wrap_Dwg_Object_BLOCKFLIPACTION_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPACTION_evalexpr_set", _wrap_Dwg_Object_BLOCKFLIPACTION_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPACTION_evalexpr_get", _wrap_Dwg_Object_BLOCKFLIPACTION_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPACTION_name_set", _wrap_Dwg_Object_BLOCKFLIPACTION_name_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPACTION_name_get", _wrap_Dwg_Object_BLOCKFLIPACTION_name_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPACTION_be_major_set", _wrap_Dwg_Object_BLOCKFLIPACTION_be_major_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPACTION_be_major_get", _wrap_Dwg_Object_BLOCKFLIPACTION_be_major_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPACTION_be_minor_set", _wrap_Dwg_Object_BLOCKFLIPACTION_be_minor_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPACTION_be_minor_get", _wrap_Dwg_Object_BLOCKFLIPACTION_be_minor_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPACTION_eed1071_set", _wrap_Dwg_Object_BLOCKFLIPACTION_eed1071_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPACTION_eed1071_get", _wrap_Dwg_Object_BLOCKFLIPACTION_eed1071_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPACTION_display_location_set", _wrap_Dwg_Object_BLOCKFLIPACTION_display_location_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPACTION_display_location_get", _wrap_Dwg_Object_BLOCKFLIPACTION_display_location_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPACTION_num_actions_set", _wrap_Dwg_Object_BLOCKFLIPACTION_num_actions_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPACTION_num_actions_get", _wrap_Dwg_Object_BLOCKFLIPACTION_num_actions_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPACTION_actions_set", _wrap_Dwg_Object_BLOCKFLIPACTION_actions_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPACTION_actions_get", _wrap_Dwg_Object_BLOCKFLIPACTION_actions_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPACTION_num_deps_set", _wrap_Dwg_Object_BLOCKFLIPACTION_num_deps_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPACTION_num_deps_get", _wrap_Dwg_Object_BLOCKFLIPACTION_num_deps_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPACTION_deps_set", _wrap_Dwg_Object_BLOCKFLIPACTION_deps_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPACTION_deps_get", _wrap_Dwg_Object_BLOCKFLIPACTION_deps_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPACTION_conn_pts_set", _wrap_Dwg_Object_BLOCKFLIPACTION_conn_pts_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPACTION_conn_pts_get", _wrap_Dwg_Object_BLOCKFLIPACTION_conn_pts_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPACTION_action_offset_x_set", _wrap_Dwg_Object_BLOCKFLIPACTION_action_offset_x_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPACTION_action_offset_x_get", _wrap_Dwg_Object_BLOCKFLIPACTION_action_offset_x_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPACTION_action_offset_y_set", _wrap_Dwg_Object_BLOCKFLIPACTION_action_offset_y_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPACTION_action_offset_y_get", _wrap_Dwg_Object_BLOCKFLIPACTION_action_offset_y_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPACTION_angle_offset_set", _wrap_Dwg_Object_BLOCKFLIPACTION_angle_offset_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPACTION_angle_offset_get", _wrap_Dwg_Object_BLOCKFLIPACTION_angle_offset_get},
{"LibreDWGc::new_Dwg_Object_BLOCKFLIPACTION", _wrap_new_Dwg_Object_BLOCKFLIPACTION},
{"LibreDWGc::delete_Dwg_Object_BLOCKFLIPACTION", _wrap_delete_Dwg_Object_BLOCKFLIPACTION},
{"LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_parent_set", _wrap_Dwg_Object_BLOCKFLIPGRIP_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_parent_get", _wrap_Dwg_Object_BLOCKFLIPGRIP_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_evalexpr_set", _wrap_Dwg_Object_BLOCKFLIPGRIP_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_evalexpr_get", _wrap_Dwg_Object_BLOCKFLIPGRIP_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_name_set", _wrap_Dwg_Object_BLOCKFLIPGRIP_name_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_name_get", _wrap_Dwg_Object_BLOCKFLIPGRIP_name_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_be_major_set", _wrap_Dwg_Object_BLOCKFLIPGRIP_be_major_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_be_major_get", _wrap_Dwg_Object_BLOCKFLIPGRIP_be_major_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_be_minor_set", _wrap_Dwg_Object_BLOCKFLIPGRIP_be_minor_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_be_minor_get", _wrap_Dwg_Object_BLOCKFLIPGRIP_be_minor_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_eed1071_set", _wrap_Dwg_Object_BLOCKFLIPGRIP_eed1071_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_eed1071_get", _wrap_Dwg_Object_BLOCKFLIPGRIP_eed1071_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_bg_bl91_set", _wrap_Dwg_Object_BLOCKFLIPGRIP_bg_bl91_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_bg_bl91_get", _wrap_Dwg_Object_BLOCKFLIPGRIP_bg_bl91_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_bg_bl92_set", _wrap_Dwg_Object_BLOCKFLIPGRIP_bg_bl92_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_bg_bl92_get", _wrap_Dwg_Object_BLOCKFLIPGRIP_bg_bl92_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_bg_location_set", _wrap_Dwg_Object_BLOCKFLIPGRIP_bg_location_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_bg_location_get", _wrap_Dwg_Object_BLOCKFLIPGRIP_bg_location_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_bg_insert_cycling_set", _wrap_Dwg_Object_BLOCKFLIPGRIP_bg_insert_cycling_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_bg_insert_cycling_get", _wrap_Dwg_Object_BLOCKFLIPGRIP_bg_insert_cycling_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_bg_insert_cycling_weight_set", _wrap_Dwg_Object_BLOCKFLIPGRIP_bg_insert_cycling_weight_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_bg_insert_cycling_weight_get", _wrap_Dwg_Object_BLOCKFLIPGRIP_bg_insert_cycling_weight_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_combined_state_set", _wrap_Dwg_Object_BLOCKFLIPGRIP_combined_state_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_combined_state_get", _wrap_Dwg_Object_BLOCKFLIPGRIP_combined_state_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_orientation_set", _wrap_Dwg_Object_BLOCKFLIPGRIP_orientation_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_orientation_get", _wrap_Dwg_Object_BLOCKFLIPGRIP_orientation_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_upd_state_set", _wrap_Dwg_Object_BLOCKFLIPGRIP_upd_state_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_upd_state_get", _wrap_Dwg_Object_BLOCKFLIPGRIP_upd_state_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_state_set", _wrap_Dwg_Object_BLOCKFLIPGRIP_state_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_state_get", _wrap_Dwg_Object_BLOCKFLIPGRIP_state_get},
{"LibreDWGc::new_Dwg_Object_BLOCKFLIPGRIP", _wrap_new_Dwg_Object_BLOCKFLIPGRIP},
{"LibreDWGc::delete_Dwg_Object_BLOCKFLIPGRIP", _wrap_delete_Dwg_Object_BLOCKFLIPGRIP},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_parent_set", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_parent_get", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_evalexpr_set", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_evalexpr_get", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_name_set", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_name_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_name_get", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_name_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_be_major_set", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_be_major_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_be_major_get", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_be_major_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_be_minor_set", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_be_minor_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_be_minor_get", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_be_minor_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_eed1071_set", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_eed1071_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_eed1071_get", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_eed1071_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_show_properties_set", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_show_properties_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_show_properties_get", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_show_properties_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_chain_actions_set", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_chain_actions_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_chain_actions_get", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_chain_actions_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_def_basept_set", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_def_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_def_basept_get", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_def_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_def_endpt_set", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_def_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_def_endpt_get", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_def_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_prop1_set", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_prop1_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_prop1_get", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_prop1_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_prop2_set", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_prop2_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_prop2_get", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_prop2_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_prop3_set", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_prop3_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_prop3_get", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_prop3_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_prop4_set", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_prop4_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_prop4_get", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_prop4_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_prop_states_set", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_prop_states_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_prop_states_get", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_prop_states_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_parameter_base_location_set", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_parameter_base_location_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_parameter_base_location_get", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_parameter_base_location_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_upd_basept_set", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_upd_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_upd_basept_get", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_upd_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_basept_set", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_basept_get", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_upd_endpt_set", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_upd_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_upd_endpt_get", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_upd_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_endpt_set", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_endpt_get", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_flip_label_set", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_flip_label_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_flip_label_get", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_flip_label_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_flip_label_desc_set", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_flip_label_desc_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_flip_label_desc_get", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_flip_label_desc_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_base_state_label_set", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_base_state_label_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_base_state_label_get", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_base_state_label_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_flipped_state_label_set", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_flipped_state_label_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_flipped_state_label_get", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_flipped_state_label_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_def_label_pt_set", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_def_label_pt_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_def_label_pt_get", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_def_label_pt_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_bl96_set", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_bl96_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_bl96_get", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_bl96_get},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_tooltip_set", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_tooltip_set},
{"LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_tooltip_get", _wrap_Dwg_Object_BLOCKFLIPPARAMETER_tooltip_get},
{"LibreDWGc::new_Dwg_Object_BLOCKFLIPPARAMETER", _wrap_new_Dwg_Object_BLOCKFLIPPARAMETER},
{"LibreDWGc::delete_Dwg_Object_BLOCKFLIPPARAMETER", _wrap_delete_Dwg_Object_BLOCKFLIPPARAMETER},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_parent_set", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_parent_get", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_evalexpr_set", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_evalexpr_get", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_name_set", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_name_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_name_get", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_name_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_be_major_set", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_be_major_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_be_major_get", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_be_major_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_be_minor_set", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_be_minor_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_be_minor_get", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_be_minor_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_eed1071_set", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_eed1071_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_eed1071_get", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_eed1071_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_show_properties_set", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_show_properties_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_show_properties_get", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_show_properties_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_chain_actions_set", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_chain_actions_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_chain_actions_get", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_chain_actions_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_def_basept_set", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_def_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_def_basept_get", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_def_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_def_endpt_set", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_def_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_def_endpt_get", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_def_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop1_set", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop1_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop1_get", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop1_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop2_set", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop2_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop2_get", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop2_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop3_set", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop3_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop3_get", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop3_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop4_set", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop4_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop4_get", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop4_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop_states_set", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop_states_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop_states_get", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop_states_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_parameter_base_location_set", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_parameter_base_location_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_parameter_base_location_get", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_parameter_base_location_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_upd_basept_set", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_upd_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_upd_basept_get", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_upd_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_basept_set", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_basept_get", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_upd_endpt_set", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_upd_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_upd_endpt_get", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_upd_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_endpt_set", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_endpt_get", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_dependency_set", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_dependency_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_dependency_get", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_dependency_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_expr_name_set", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_expr_name_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_expr_name_get", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_expr_name_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_expr_description_set", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_expr_description_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_expr_description_get", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_expr_description_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_value_set", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_value_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_value_get", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_value_get},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_value_set_set", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_value_set_set},
{"LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_value_set_get", _wrap_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_value_set_get},
{"LibreDWGc::new_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER", _wrap_new_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER},
{"LibreDWGc::delete_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER", _wrap_delete_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_parent_set", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_parent_get", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_evalexpr_set", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_evalexpr_get", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_name_set", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_name_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_name_get", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_name_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_be_major_set", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_be_major_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_be_major_get", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_be_major_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_be_minor_set", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_be_minor_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_be_minor_get", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_be_minor_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_eed1071_set", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_eed1071_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_eed1071_get", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_eed1071_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_show_properties_set", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_show_properties_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_show_properties_get", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_show_properties_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_chain_actions_set", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_chain_actions_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_chain_actions_get", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_chain_actions_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_def_basept_set", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_def_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_def_basept_get", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_def_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_def_endpt_set", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_def_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_def_endpt_get", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_def_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop1_set", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop1_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop1_get", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop1_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop2_set", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop2_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop2_get", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop2_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop3_set", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop3_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop3_get", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop3_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop4_set", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop4_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop4_get", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop4_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop_states_set", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop_states_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop_states_get", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop_states_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_parameter_base_location_set", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_parameter_base_location_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_parameter_base_location_get", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_parameter_base_location_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_upd_basept_set", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_upd_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_upd_basept_get", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_upd_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_basept_set", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_basept_get", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_upd_endpt_set", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_upd_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_upd_endpt_get", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_upd_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_endpt_set", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_endpt_get", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_dependency_set", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_dependency_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_dependency_get", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_dependency_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_expr_name_set", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_expr_name_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_expr_name_get", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_expr_name_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_expr_description_set", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_expr_description_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_expr_description_get", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_expr_description_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_value_set", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_value_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_value_get", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_value_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_value_set_set", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_value_set_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_value_set_get", _wrap_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_value_set_get},
{"LibreDWGc::new_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER", _wrap_new_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER},
{"LibreDWGc::delete_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER", _wrap_delete_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_parent_set", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_parent_get", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_evalexpr_set", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_evalexpr_get", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_name_set", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_name_set},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_name_get", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_name_get},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_be_major_set", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_be_major_set},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_be_major_get", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_be_major_get},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_be_minor_set", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_be_minor_set},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_be_minor_get", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_be_minor_get},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_eed1071_set", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_eed1071_set},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_eed1071_get", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_eed1071_get},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_show_properties_set", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_show_properties_set},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_show_properties_get", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_show_properties_get},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_chain_actions_set", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_chain_actions_set},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_chain_actions_get", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_chain_actions_get},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_def_basept_set", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_def_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_def_basept_get", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_def_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_def_endpt_set", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_def_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_def_endpt_get", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_def_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop1_set", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop1_set},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop1_get", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop1_get},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop2_set", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop2_set},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop2_get", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop2_get},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop3_set", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop3_set},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop3_get", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop3_get},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop4_set", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop4_set},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop4_get", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop4_get},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop_states_set", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop_states_set},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop_states_get", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop_states_get},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_parameter_base_location_set", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_parameter_base_location_set},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_parameter_base_location_get", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_parameter_base_location_get},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_upd_basept_set", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_upd_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_upd_basept_get", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_upd_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_basept_set", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_basept_get", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_upd_endpt_set", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_upd_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_upd_endpt_get", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_upd_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_endpt_set", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_endpt_get", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_dependency_set", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_dependency_set},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_dependency_get", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_dependency_get},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_expr_name_set", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_expr_name_set},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_expr_name_get", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_expr_name_get},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_expr_description_set", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_expr_description_set},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_expr_description_get", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_expr_description_get},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_value_set", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_value_set},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_value_get", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_value_get},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_value_set_set", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_value_set_set},
{"LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_value_set_get", _wrap_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_value_set_get},
{"LibreDWGc::new_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER", _wrap_new_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER},
{"LibreDWGc::delete_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER", _wrap_delete_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_parent_set", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_parent_get", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_evalexpr_set", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_evalexpr_get", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_name_set", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_name_set},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_name_get", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_name_get},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_be_major_set", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_be_major_set},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_be_major_get", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_be_major_get},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_be_minor_set", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_be_minor_set},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_be_minor_get", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_be_minor_get},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_eed1071_set", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_eed1071_set},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_eed1071_get", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_eed1071_get},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_show_properties_set", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_show_properties_set},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_show_properties_get", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_show_properties_get},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_chain_actions_set", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_chain_actions_set},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_chain_actions_get", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_chain_actions_get},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_def_basept_set", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_def_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_def_basept_get", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_def_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_def_endpt_set", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_def_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_def_endpt_get", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_def_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop1_set", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop1_set},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop1_get", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop1_get},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop2_set", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop2_set},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop2_get", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop2_get},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop3_set", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop3_set},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop3_get", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop3_get},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop4_set", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop4_set},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop4_get", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop4_get},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop_states_set", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop_states_set},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop_states_get", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop_states_get},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_parameter_base_location_set", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_parameter_base_location_set},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_parameter_base_location_get", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_parameter_base_location_get},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_upd_basept_set", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_upd_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_upd_basept_get", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_upd_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_basept_set", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_basept_get", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_upd_endpt_set", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_upd_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_upd_endpt_get", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_upd_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_endpt_set", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_endpt_get", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_dependency_set", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_dependency_set},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_dependency_get", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_dependency_get},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_expr_name_set", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_expr_name_set},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_expr_name_get", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_expr_name_get},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_expr_description_set", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_expr_description_set},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_expr_description_get", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_expr_description_get},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_value_set", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_value_set},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_value_get", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_value_get},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_value_set_set", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_value_set_set},
{"LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_value_set_get", _wrap_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_value_set_get},
{"LibreDWGc::new_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER", _wrap_new_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER},
{"LibreDWGc::delete_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER", _wrap_delete_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER},
{"LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_parent_set", _wrap_Dwg_Object_BLOCKLINEARGRIP_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_parent_get", _wrap_Dwg_Object_BLOCKLINEARGRIP_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_evalexpr_set", _wrap_Dwg_Object_BLOCKLINEARGRIP_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_evalexpr_get", _wrap_Dwg_Object_BLOCKLINEARGRIP_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_name_set", _wrap_Dwg_Object_BLOCKLINEARGRIP_name_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_name_get", _wrap_Dwg_Object_BLOCKLINEARGRIP_name_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_be_major_set", _wrap_Dwg_Object_BLOCKLINEARGRIP_be_major_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_be_major_get", _wrap_Dwg_Object_BLOCKLINEARGRIP_be_major_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_be_minor_set", _wrap_Dwg_Object_BLOCKLINEARGRIP_be_minor_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_be_minor_get", _wrap_Dwg_Object_BLOCKLINEARGRIP_be_minor_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_eed1071_set", _wrap_Dwg_Object_BLOCKLINEARGRIP_eed1071_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_eed1071_get", _wrap_Dwg_Object_BLOCKLINEARGRIP_eed1071_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_bg_bl91_set", _wrap_Dwg_Object_BLOCKLINEARGRIP_bg_bl91_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_bg_bl91_get", _wrap_Dwg_Object_BLOCKLINEARGRIP_bg_bl91_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_bg_bl92_set", _wrap_Dwg_Object_BLOCKLINEARGRIP_bg_bl92_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_bg_bl92_get", _wrap_Dwg_Object_BLOCKLINEARGRIP_bg_bl92_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_bg_location_set", _wrap_Dwg_Object_BLOCKLINEARGRIP_bg_location_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_bg_location_get", _wrap_Dwg_Object_BLOCKLINEARGRIP_bg_location_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_bg_insert_cycling_set", _wrap_Dwg_Object_BLOCKLINEARGRIP_bg_insert_cycling_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_bg_insert_cycling_get", _wrap_Dwg_Object_BLOCKLINEARGRIP_bg_insert_cycling_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_bg_insert_cycling_weight_set", _wrap_Dwg_Object_BLOCKLINEARGRIP_bg_insert_cycling_weight_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_bg_insert_cycling_weight_get", _wrap_Dwg_Object_BLOCKLINEARGRIP_bg_insert_cycling_weight_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_orientation_set", _wrap_Dwg_Object_BLOCKLINEARGRIP_orientation_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_orientation_get", _wrap_Dwg_Object_BLOCKLINEARGRIP_orientation_get},
{"LibreDWGc::new_Dwg_Object_BLOCKLINEARGRIP", _wrap_new_Dwg_Object_BLOCKLINEARGRIP},
{"LibreDWGc::delete_Dwg_Object_BLOCKLINEARGRIP", _wrap_delete_Dwg_Object_BLOCKLINEARGRIP},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_parent_set", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_parent_get", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_evalexpr_set", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_evalexpr_get", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_name_set", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_name_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_name_get", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_name_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_be_major_set", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_be_major_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_be_major_get", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_be_major_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_be_minor_set", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_be_minor_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_be_minor_get", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_be_minor_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_eed1071_set", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_eed1071_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_eed1071_get", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_eed1071_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_show_properties_set", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_show_properties_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_show_properties_get", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_show_properties_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_chain_actions_set", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_chain_actions_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_chain_actions_get", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_chain_actions_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_def_basept_set", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_def_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_def_basept_get", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_def_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_def_endpt_set", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_def_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_def_endpt_get", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_def_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_prop1_set", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_prop1_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_prop1_get", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_prop1_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_prop2_set", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_prop2_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_prop2_get", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_prop2_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_prop3_set", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_prop3_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_prop3_get", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_prop3_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_prop4_set", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_prop4_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_prop4_get", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_prop4_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_prop_states_set", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_prop_states_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_prop_states_get", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_prop_states_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_parameter_base_location_set", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_parameter_base_location_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_parameter_base_location_get", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_parameter_base_location_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_upd_basept_set", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_upd_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_upd_basept_get", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_upd_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_basept_set", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_basept_get", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_upd_endpt_set", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_upd_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_upd_endpt_get", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_upd_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_endpt_set", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_endpt_get", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_distance_name_set", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_distance_name_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_distance_name_get", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_distance_name_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_distance_desc_set", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_distance_desc_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_distance_desc_get", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_distance_desc_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_distance_set", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_distance_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_distance_get", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_distance_get},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_value_set_set", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_value_set_set},
{"LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_value_set_get", _wrap_Dwg_Object_BLOCKLINEARPARAMETER_value_set_get},
{"LibreDWGc::new_Dwg_Object_BLOCKLINEARPARAMETER", _wrap_new_Dwg_Object_BLOCKLINEARPARAMETER},
{"LibreDWGc::delete_Dwg_Object_BLOCKLINEARPARAMETER", _wrap_delete_Dwg_Object_BLOCKLINEARPARAMETER},
{"LibreDWGc::Dwg_BLOCKLOOKUPACTION_lut_parent_set", _wrap_Dwg_BLOCKLOOKUPACTION_lut_parent_set},
{"LibreDWGc::Dwg_BLOCKLOOKUPACTION_lut_parent_get", _wrap_Dwg_BLOCKLOOKUPACTION_lut_parent_get},
{"LibreDWGc::Dwg_BLOCKLOOKUPACTION_lut_conn_pts_set", _wrap_Dwg_BLOCKLOOKUPACTION_lut_conn_pts_set},
{"LibreDWGc::Dwg_BLOCKLOOKUPACTION_lut_conn_pts_get", _wrap_Dwg_BLOCKLOOKUPACTION_lut_conn_pts_get},
{"LibreDWGc::Dwg_BLOCKLOOKUPACTION_lut_b282_set", _wrap_Dwg_BLOCKLOOKUPACTION_lut_b282_set},
{"LibreDWGc::Dwg_BLOCKLOOKUPACTION_lut_b282_get", _wrap_Dwg_BLOCKLOOKUPACTION_lut_b282_get},
{"LibreDWGc::Dwg_BLOCKLOOKUPACTION_lut_b281_set", _wrap_Dwg_BLOCKLOOKUPACTION_lut_b281_set},
{"LibreDWGc::Dwg_BLOCKLOOKUPACTION_lut_b281_get", _wrap_Dwg_BLOCKLOOKUPACTION_lut_b281_get},
{"LibreDWGc::new_Dwg_BLOCKLOOKUPACTION_lut", _wrap_new_Dwg_BLOCKLOOKUPACTION_lut},
{"LibreDWGc::delete_Dwg_BLOCKLOOKUPACTION_lut", _wrap_delete_Dwg_BLOCKLOOKUPACTION_lut},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_parent_set", _wrap_Dwg_Object_BLOCKLOOKUPACTION_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_parent_get", _wrap_Dwg_Object_BLOCKLOOKUPACTION_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_evalexpr_set", _wrap_Dwg_Object_BLOCKLOOKUPACTION_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_evalexpr_get", _wrap_Dwg_Object_BLOCKLOOKUPACTION_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_name_set", _wrap_Dwg_Object_BLOCKLOOKUPACTION_name_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_name_get", _wrap_Dwg_Object_BLOCKLOOKUPACTION_name_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_be_major_set", _wrap_Dwg_Object_BLOCKLOOKUPACTION_be_major_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_be_major_get", _wrap_Dwg_Object_BLOCKLOOKUPACTION_be_major_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_be_minor_set", _wrap_Dwg_Object_BLOCKLOOKUPACTION_be_minor_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_be_minor_get", _wrap_Dwg_Object_BLOCKLOOKUPACTION_be_minor_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_eed1071_set", _wrap_Dwg_Object_BLOCKLOOKUPACTION_eed1071_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_eed1071_get", _wrap_Dwg_Object_BLOCKLOOKUPACTION_eed1071_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_display_location_set", _wrap_Dwg_Object_BLOCKLOOKUPACTION_display_location_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_display_location_get", _wrap_Dwg_Object_BLOCKLOOKUPACTION_display_location_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_num_actions_set", _wrap_Dwg_Object_BLOCKLOOKUPACTION_num_actions_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_num_actions_get", _wrap_Dwg_Object_BLOCKLOOKUPACTION_num_actions_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_actions_set", _wrap_Dwg_Object_BLOCKLOOKUPACTION_actions_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_actions_get", _wrap_Dwg_Object_BLOCKLOOKUPACTION_actions_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_num_deps_set", _wrap_Dwg_Object_BLOCKLOOKUPACTION_num_deps_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_num_deps_get", _wrap_Dwg_Object_BLOCKLOOKUPACTION_num_deps_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_deps_set", _wrap_Dwg_Object_BLOCKLOOKUPACTION_deps_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_deps_get", _wrap_Dwg_Object_BLOCKLOOKUPACTION_deps_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_numelems_set", _wrap_Dwg_Object_BLOCKLOOKUPACTION_numelems_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_numelems_get", _wrap_Dwg_Object_BLOCKLOOKUPACTION_numelems_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_numrows_set", _wrap_Dwg_Object_BLOCKLOOKUPACTION_numrows_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_numrows_get", _wrap_Dwg_Object_BLOCKLOOKUPACTION_numrows_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_numcols_set", _wrap_Dwg_Object_BLOCKLOOKUPACTION_numcols_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_numcols_get", _wrap_Dwg_Object_BLOCKLOOKUPACTION_numcols_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_lut_set", _wrap_Dwg_Object_BLOCKLOOKUPACTION_lut_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_lut_get", _wrap_Dwg_Object_BLOCKLOOKUPACTION_lut_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_exprs_set", _wrap_Dwg_Object_BLOCKLOOKUPACTION_exprs_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_exprs_get", _wrap_Dwg_Object_BLOCKLOOKUPACTION_exprs_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_b280_set", _wrap_Dwg_Object_BLOCKLOOKUPACTION_b280_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_b280_get", _wrap_Dwg_Object_BLOCKLOOKUPACTION_b280_get},
{"LibreDWGc::new_Dwg_Object_BLOCKLOOKUPACTION", _wrap_new_Dwg_Object_BLOCKLOOKUPACTION},
{"LibreDWGc::delete_Dwg_Object_BLOCKLOOKUPACTION", _wrap_delete_Dwg_Object_BLOCKLOOKUPACTION},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_parent_set", _wrap_Dwg_Object_BLOCKLOOKUPGRIP_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_parent_get", _wrap_Dwg_Object_BLOCKLOOKUPGRIP_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_evalexpr_set", _wrap_Dwg_Object_BLOCKLOOKUPGRIP_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_evalexpr_get", _wrap_Dwg_Object_BLOCKLOOKUPGRIP_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_name_set", _wrap_Dwg_Object_BLOCKLOOKUPGRIP_name_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_name_get", _wrap_Dwg_Object_BLOCKLOOKUPGRIP_name_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_be_major_set", _wrap_Dwg_Object_BLOCKLOOKUPGRIP_be_major_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_be_major_get", _wrap_Dwg_Object_BLOCKLOOKUPGRIP_be_major_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_be_minor_set", _wrap_Dwg_Object_BLOCKLOOKUPGRIP_be_minor_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_be_minor_get", _wrap_Dwg_Object_BLOCKLOOKUPGRIP_be_minor_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_eed1071_set", _wrap_Dwg_Object_BLOCKLOOKUPGRIP_eed1071_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_eed1071_get", _wrap_Dwg_Object_BLOCKLOOKUPGRIP_eed1071_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_bg_bl91_set", _wrap_Dwg_Object_BLOCKLOOKUPGRIP_bg_bl91_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_bg_bl91_get", _wrap_Dwg_Object_BLOCKLOOKUPGRIP_bg_bl91_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_bg_bl92_set", _wrap_Dwg_Object_BLOCKLOOKUPGRIP_bg_bl92_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_bg_bl92_get", _wrap_Dwg_Object_BLOCKLOOKUPGRIP_bg_bl92_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_bg_location_set", _wrap_Dwg_Object_BLOCKLOOKUPGRIP_bg_location_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_bg_location_get", _wrap_Dwg_Object_BLOCKLOOKUPGRIP_bg_location_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_bg_insert_cycling_set", _wrap_Dwg_Object_BLOCKLOOKUPGRIP_bg_insert_cycling_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_bg_insert_cycling_get", _wrap_Dwg_Object_BLOCKLOOKUPGRIP_bg_insert_cycling_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_bg_insert_cycling_weight_set", _wrap_Dwg_Object_BLOCKLOOKUPGRIP_bg_insert_cycling_weight_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_bg_insert_cycling_weight_get", _wrap_Dwg_Object_BLOCKLOOKUPGRIP_bg_insert_cycling_weight_get},
{"LibreDWGc::new_Dwg_Object_BLOCKLOOKUPGRIP", _wrap_new_Dwg_Object_BLOCKLOOKUPGRIP},
{"LibreDWGc::delete_Dwg_Object_BLOCKLOOKUPGRIP", _wrap_delete_Dwg_Object_BLOCKLOOKUPGRIP},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_parent_set", _wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_parent_get", _wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_evalexpr_set", _wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_evalexpr_get", _wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_name_set", _wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_name_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_name_get", _wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_name_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_be_major_set", _wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_be_major_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_be_major_get", _wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_be_major_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_be_minor_set", _wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_be_minor_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_be_minor_get", _wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_be_minor_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_eed1071_set", _wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_eed1071_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_eed1071_get", _wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_eed1071_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_show_properties_set", _wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_show_properties_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_show_properties_get", _wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_show_properties_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_chain_actions_set", _wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_chain_actions_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_chain_actions_get", _wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_chain_actions_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_def_pt_set", _wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_def_pt_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_def_pt_get", _wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_def_pt_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_num_propinfos_set", _wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_num_propinfos_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_num_propinfos_get", _wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_num_propinfos_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_prop1_set", _wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_prop1_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_prop1_get", _wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_prop1_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_prop2_set", _wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_prop2_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_prop2_get", _wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_prop2_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_lookup_name_set", _wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_lookup_name_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_lookup_name_get", _wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_lookup_name_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_lookup_desc_set", _wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_lookup_desc_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_lookup_desc_get", _wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_lookup_desc_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_index_set", _wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_index_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_index_get", _wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_index_get},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_unknown_t_set", _wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_unknown_t_set},
{"LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_unknown_t_get", _wrap_Dwg_Object_BLOCKLOOKUPPARAMETER_unknown_t_get},
{"LibreDWGc::new_Dwg_Object_BLOCKLOOKUPPARAMETER", _wrap_new_Dwg_Object_BLOCKLOOKUPPARAMETER},
{"LibreDWGc::delete_Dwg_Object_BLOCKLOOKUPPARAMETER", _wrap_delete_Dwg_Object_BLOCKLOOKUPPARAMETER},
{"LibreDWGc::Dwg_Object_BLOCKMOVEACTION_parent_set", _wrap_Dwg_Object_BLOCKMOVEACTION_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKMOVEACTION_parent_get", _wrap_Dwg_Object_BLOCKMOVEACTION_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKMOVEACTION_evalexpr_set", _wrap_Dwg_Object_BLOCKMOVEACTION_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKMOVEACTION_evalexpr_get", _wrap_Dwg_Object_BLOCKMOVEACTION_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKMOVEACTION_name_set", _wrap_Dwg_Object_BLOCKMOVEACTION_name_set},
{"LibreDWGc::Dwg_Object_BLOCKMOVEACTION_name_get", _wrap_Dwg_Object_BLOCKMOVEACTION_name_get},
{"LibreDWGc::Dwg_Object_BLOCKMOVEACTION_be_major_set", _wrap_Dwg_Object_BLOCKMOVEACTION_be_major_set},
{"LibreDWGc::Dwg_Object_BLOCKMOVEACTION_be_major_get", _wrap_Dwg_Object_BLOCKMOVEACTION_be_major_get},
{"LibreDWGc::Dwg_Object_BLOCKMOVEACTION_be_minor_set", _wrap_Dwg_Object_BLOCKMOVEACTION_be_minor_set},
{"LibreDWGc::Dwg_Object_BLOCKMOVEACTION_be_minor_get", _wrap_Dwg_Object_BLOCKMOVEACTION_be_minor_get},
{"LibreDWGc::Dwg_Object_BLOCKMOVEACTION_eed1071_set", _wrap_Dwg_Object_BLOCKMOVEACTION_eed1071_set},
{"LibreDWGc::Dwg_Object_BLOCKMOVEACTION_eed1071_get", _wrap_Dwg_Object_BLOCKMOVEACTION_eed1071_get},
{"LibreDWGc::Dwg_Object_BLOCKMOVEACTION_display_location_set", _wrap_Dwg_Object_BLOCKMOVEACTION_display_location_set},
{"LibreDWGc::Dwg_Object_BLOCKMOVEACTION_display_location_get", _wrap_Dwg_Object_BLOCKMOVEACTION_display_location_get},
{"LibreDWGc::Dwg_Object_BLOCKMOVEACTION_num_actions_set", _wrap_Dwg_Object_BLOCKMOVEACTION_num_actions_set},
{"LibreDWGc::Dwg_Object_BLOCKMOVEACTION_num_actions_get", _wrap_Dwg_Object_BLOCKMOVEACTION_num_actions_get},
{"LibreDWGc::Dwg_Object_BLOCKMOVEACTION_actions_set", _wrap_Dwg_Object_BLOCKMOVEACTION_actions_set},
{"LibreDWGc::Dwg_Object_BLOCKMOVEACTION_actions_get", _wrap_Dwg_Object_BLOCKMOVEACTION_actions_get},
{"LibreDWGc::Dwg_Object_BLOCKMOVEACTION_num_deps_set", _wrap_Dwg_Object_BLOCKMOVEACTION_num_deps_set},
{"LibreDWGc::Dwg_Object_BLOCKMOVEACTION_num_deps_get", _wrap_Dwg_Object_BLOCKMOVEACTION_num_deps_get},
{"LibreDWGc::Dwg_Object_BLOCKMOVEACTION_deps_set", _wrap_Dwg_Object_BLOCKMOVEACTION_deps_set},
{"LibreDWGc::Dwg_Object_BLOCKMOVEACTION_deps_get", _wrap_Dwg_Object_BLOCKMOVEACTION_deps_get},
{"LibreDWGc::Dwg_Object_BLOCKMOVEACTION_conn_pts_set", _wrap_Dwg_Object_BLOCKMOVEACTION_conn_pts_set},
{"LibreDWGc::Dwg_Object_BLOCKMOVEACTION_conn_pts_get", _wrap_Dwg_Object_BLOCKMOVEACTION_conn_pts_get},
{"LibreDWGc::Dwg_Object_BLOCKMOVEACTION_action_offset_x_set", _wrap_Dwg_Object_BLOCKMOVEACTION_action_offset_x_set},
{"LibreDWGc::Dwg_Object_BLOCKMOVEACTION_action_offset_x_get", _wrap_Dwg_Object_BLOCKMOVEACTION_action_offset_x_get},
{"LibreDWGc::Dwg_Object_BLOCKMOVEACTION_action_offset_y_set", _wrap_Dwg_Object_BLOCKMOVEACTION_action_offset_y_set},
{"LibreDWGc::Dwg_Object_BLOCKMOVEACTION_action_offset_y_get", _wrap_Dwg_Object_BLOCKMOVEACTION_action_offset_y_get},
{"LibreDWGc::Dwg_Object_BLOCKMOVEACTION_angle_offset_set", _wrap_Dwg_Object_BLOCKMOVEACTION_angle_offset_set},
{"LibreDWGc::Dwg_Object_BLOCKMOVEACTION_angle_offset_get", _wrap_Dwg_Object_BLOCKMOVEACTION_angle_offset_get},
{"LibreDWGc::new_Dwg_Object_BLOCKMOVEACTION", _wrap_new_Dwg_Object_BLOCKMOVEACTION},
{"LibreDWGc::delete_Dwg_Object_BLOCKMOVEACTION", _wrap_delete_Dwg_Object_BLOCKMOVEACTION},
{"LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_parent_set", _wrap_Dwg_Object_BLOCKPOINTPARAMETER_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_parent_get", _wrap_Dwg_Object_BLOCKPOINTPARAMETER_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_evalexpr_set", _wrap_Dwg_Object_BLOCKPOINTPARAMETER_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_evalexpr_get", _wrap_Dwg_Object_BLOCKPOINTPARAMETER_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_name_set", _wrap_Dwg_Object_BLOCKPOINTPARAMETER_name_set},
{"LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_name_get", _wrap_Dwg_Object_BLOCKPOINTPARAMETER_name_get},
{"LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_be_major_set", _wrap_Dwg_Object_BLOCKPOINTPARAMETER_be_major_set},
{"LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_be_major_get", _wrap_Dwg_Object_BLOCKPOINTPARAMETER_be_major_get},
{"LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_be_minor_set", _wrap_Dwg_Object_BLOCKPOINTPARAMETER_be_minor_set},
{"LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_be_minor_get", _wrap_Dwg_Object_BLOCKPOINTPARAMETER_be_minor_get},
{"LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_eed1071_set", _wrap_Dwg_Object_BLOCKPOINTPARAMETER_eed1071_set},
{"LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_eed1071_get", _wrap_Dwg_Object_BLOCKPOINTPARAMETER_eed1071_get},
{"LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_show_properties_set", _wrap_Dwg_Object_BLOCKPOINTPARAMETER_show_properties_set},
{"LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_show_properties_get", _wrap_Dwg_Object_BLOCKPOINTPARAMETER_show_properties_get},
{"LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_chain_actions_set", _wrap_Dwg_Object_BLOCKPOINTPARAMETER_chain_actions_set},
{"LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_chain_actions_get", _wrap_Dwg_Object_BLOCKPOINTPARAMETER_chain_actions_get},
{"LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_def_pt_set", _wrap_Dwg_Object_BLOCKPOINTPARAMETER_def_pt_set},
{"LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_def_pt_get", _wrap_Dwg_Object_BLOCKPOINTPARAMETER_def_pt_get},
{"LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_num_propinfos_set", _wrap_Dwg_Object_BLOCKPOINTPARAMETER_num_propinfos_set},
{"LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_num_propinfos_get", _wrap_Dwg_Object_BLOCKPOINTPARAMETER_num_propinfos_get},
{"LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_prop1_set", _wrap_Dwg_Object_BLOCKPOINTPARAMETER_prop1_set},
{"LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_prop1_get", _wrap_Dwg_Object_BLOCKPOINTPARAMETER_prop1_get},
{"LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_prop2_set", _wrap_Dwg_Object_BLOCKPOINTPARAMETER_prop2_set},
{"LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_prop2_get", _wrap_Dwg_Object_BLOCKPOINTPARAMETER_prop2_get},
{"LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_position_name_set", _wrap_Dwg_Object_BLOCKPOINTPARAMETER_position_name_set},
{"LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_position_name_get", _wrap_Dwg_Object_BLOCKPOINTPARAMETER_position_name_get},
{"LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_position_desc_set", _wrap_Dwg_Object_BLOCKPOINTPARAMETER_position_desc_set},
{"LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_position_desc_get", _wrap_Dwg_Object_BLOCKPOINTPARAMETER_position_desc_get},
{"LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_def_label_pt_set", _wrap_Dwg_Object_BLOCKPOINTPARAMETER_def_label_pt_set},
{"LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_def_label_pt_get", _wrap_Dwg_Object_BLOCKPOINTPARAMETER_def_label_pt_get},
{"LibreDWGc::new_Dwg_Object_BLOCKPOINTPARAMETER", _wrap_new_Dwg_Object_BLOCKPOINTPARAMETER},
{"LibreDWGc::delete_Dwg_Object_BLOCKPOINTPARAMETER", _wrap_delete_Dwg_Object_BLOCKPOINTPARAMETER},
{"LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_parent_set", _wrap_Dwg_Object_BLOCKPOLARGRIP_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_parent_get", _wrap_Dwg_Object_BLOCKPOLARGRIP_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_evalexpr_set", _wrap_Dwg_Object_BLOCKPOLARGRIP_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_evalexpr_get", _wrap_Dwg_Object_BLOCKPOLARGRIP_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_name_set", _wrap_Dwg_Object_BLOCKPOLARGRIP_name_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_name_get", _wrap_Dwg_Object_BLOCKPOLARGRIP_name_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_be_major_set", _wrap_Dwg_Object_BLOCKPOLARGRIP_be_major_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_be_major_get", _wrap_Dwg_Object_BLOCKPOLARGRIP_be_major_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_be_minor_set", _wrap_Dwg_Object_BLOCKPOLARGRIP_be_minor_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_be_minor_get", _wrap_Dwg_Object_BLOCKPOLARGRIP_be_minor_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_eed1071_set", _wrap_Dwg_Object_BLOCKPOLARGRIP_eed1071_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_eed1071_get", _wrap_Dwg_Object_BLOCKPOLARGRIP_eed1071_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_bg_bl91_set", _wrap_Dwg_Object_BLOCKPOLARGRIP_bg_bl91_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_bg_bl91_get", _wrap_Dwg_Object_BLOCKPOLARGRIP_bg_bl91_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_bg_bl92_set", _wrap_Dwg_Object_BLOCKPOLARGRIP_bg_bl92_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_bg_bl92_get", _wrap_Dwg_Object_BLOCKPOLARGRIP_bg_bl92_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_bg_location_set", _wrap_Dwg_Object_BLOCKPOLARGRIP_bg_location_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_bg_location_get", _wrap_Dwg_Object_BLOCKPOLARGRIP_bg_location_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_bg_insert_cycling_set", _wrap_Dwg_Object_BLOCKPOLARGRIP_bg_insert_cycling_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_bg_insert_cycling_get", _wrap_Dwg_Object_BLOCKPOLARGRIP_bg_insert_cycling_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_bg_insert_cycling_weight_set", _wrap_Dwg_Object_BLOCKPOLARGRIP_bg_insert_cycling_weight_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_bg_insert_cycling_weight_get", _wrap_Dwg_Object_BLOCKPOLARGRIP_bg_insert_cycling_weight_get},
{"LibreDWGc::new_Dwg_Object_BLOCKPOLARGRIP", _wrap_new_Dwg_Object_BLOCKPOLARGRIP},
{"LibreDWGc::delete_Dwg_Object_BLOCKPOLARGRIP", _wrap_delete_Dwg_Object_BLOCKPOLARGRIP},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_parent_set", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_parent_get", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_evalexpr_set", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_evalexpr_get", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_name_set", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_name_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_name_get", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_name_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_be_major_set", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_be_major_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_be_major_get", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_be_major_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_be_minor_set", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_be_minor_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_be_minor_get", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_be_minor_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_eed1071_set", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_eed1071_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_eed1071_get", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_eed1071_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_show_properties_set", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_show_properties_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_show_properties_get", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_show_properties_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_chain_actions_set", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_chain_actions_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_chain_actions_get", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_chain_actions_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_def_basept_set", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_def_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_def_basept_get", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_def_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_def_endpt_set", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_def_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_def_endpt_get", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_def_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_prop1_set", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_prop1_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_prop1_get", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_prop1_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_prop2_set", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_prop2_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_prop2_get", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_prop2_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_prop3_set", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_prop3_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_prop3_get", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_prop3_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_prop4_set", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_prop4_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_prop4_get", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_prop4_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_prop_states_set", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_prop_states_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_prop_states_get", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_prop_states_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_parameter_base_location_set", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_parameter_base_location_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_parameter_base_location_get", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_parameter_base_location_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_upd_basept_set", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_upd_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_upd_basept_get", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_upd_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_basept_set", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_basept_get", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_upd_endpt_set", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_upd_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_upd_endpt_get", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_upd_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_endpt_set", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_endpt_get", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_angle_name_set", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_angle_name_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_angle_name_get", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_angle_name_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_angle_desc_set", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_angle_desc_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_angle_desc_get", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_angle_desc_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_distance_name_set", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_distance_name_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_distance_name_get", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_distance_name_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_distance_desc_set", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_distance_desc_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_distance_desc_get", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_distance_desc_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_offset_set", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_offset_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_offset_get", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_offset_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_angle_value_set_set", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_angle_value_set_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_angle_value_set_get", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_angle_value_set_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_distance_value_set_set", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_distance_value_set_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_distance_value_set_get", _wrap_Dwg_Object_BLOCKPOLARPARAMETER_distance_value_set_get},
{"LibreDWGc::new_Dwg_Object_BLOCKPOLARPARAMETER", _wrap_new_Dwg_Object_BLOCKPOLARPARAMETER},
{"LibreDWGc::delete_Dwg_Object_BLOCKPOLARPARAMETER", _wrap_delete_Dwg_Object_BLOCKPOLARPARAMETER},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_parent_set", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_parent_get", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_evalexpr_set", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_evalexpr_get", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_name_set", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_name_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_name_get", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_name_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_be_major_set", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_be_major_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_be_major_get", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_be_major_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_be_minor_set", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_be_minor_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_be_minor_get", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_be_minor_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_eed1071_set", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_eed1071_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_eed1071_get", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_eed1071_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_display_location_set", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_display_location_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_display_location_get", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_display_location_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_num_actions_set", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_num_actions_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_num_actions_get", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_num_actions_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_actions_set", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_actions_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_actions_get", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_actions_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_num_deps_set", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_num_deps_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_num_deps_get", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_num_deps_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_deps_set", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_deps_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_deps_get", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_deps_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_conn_pts_set", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_conn_pts_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_conn_pts_get", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_conn_pts_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_num_pts_set", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_num_pts_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_num_pts_get", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_num_pts_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_pts_set", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_pts_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_pts_get", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_pts_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_num_hdls_set", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_num_hdls_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_num_hdls_get", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_num_hdls_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_hdls_set", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_hdls_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_hdls_get", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_hdls_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_shorts_set", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_shorts_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_shorts_get", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_shorts_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_num_codes_set", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_num_codes_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_num_codes_get", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_num_codes_get},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_codes_set", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_codes_set},
{"LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_codes_get", _wrap_Dwg_Object_BLOCKPOLARSTRETCHACTION_codes_get},
{"LibreDWGc::new_Dwg_Object_BLOCKPOLARSTRETCHACTION", _wrap_new_Dwg_Object_BLOCKPOLARSTRETCHACTION},
{"LibreDWGc::delete_Dwg_Object_BLOCKPOLARSTRETCHACTION", _wrap_delete_Dwg_Object_BLOCKPOLARSTRETCHACTION},
{"LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLE_parent_set", _wrap_Dwg_Object_BLOCKPROPERTIESTABLE_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLE_parent_get", _wrap_Dwg_Object_BLOCKPROPERTIESTABLE_parent_get},
{"LibreDWGc::new_Dwg_Object_BLOCKPROPERTIESTABLE", _wrap_new_Dwg_Object_BLOCKPROPERTIESTABLE},
{"LibreDWGc::delete_Dwg_Object_BLOCKPROPERTIESTABLE", _wrap_delete_Dwg_Object_BLOCKPROPERTIESTABLE},
{"LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_parent_set", _wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_parent_get", _wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_evalexpr_set", _wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_evalexpr_get", _wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_name_set", _wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_name_set},
{"LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_name_get", _wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_name_get},
{"LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_be_major_set", _wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_be_major_set},
{"LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_be_major_get", _wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_be_major_get},
{"LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_be_minor_set", _wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_be_minor_set},
{"LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_be_minor_get", _wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_be_minor_get},
{"LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_eed1071_set", _wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_eed1071_set},
{"LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_eed1071_get", _wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_eed1071_get},
{"LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_bl91_set", _wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_bl91_set},
{"LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_bl91_get", _wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_bl91_get},
{"LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_bl92_set", _wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_bl92_set},
{"LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_bl92_get", _wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_bl92_get},
{"LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_location_set", _wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_location_set},
{"LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_location_get", _wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_location_get},
{"LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_insert_cycling_set", _wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_insert_cycling_set},
{"LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_insert_cycling_get", _wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_insert_cycling_get},
{"LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_insert_cycling_weight_set", _wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_insert_cycling_weight_set},
{"LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_insert_cycling_weight_get", _wrap_Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_insert_cycling_weight_get},
{"LibreDWGc::new_Dwg_Object_BLOCKPROPERTIESTABLEGRIP", _wrap_new_Dwg_Object_BLOCKPROPERTIESTABLEGRIP},
{"LibreDWGc::delete_Dwg_Object_BLOCKPROPERTIESTABLEGRIP", _wrap_delete_Dwg_Object_BLOCKPROPERTIESTABLEGRIP},
{"LibreDWGc::Dwg_Object_BLOCKREPRESENTATION_parent_set", _wrap_Dwg_Object_BLOCKREPRESENTATION_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKREPRESENTATION_parent_get", _wrap_Dwg_Object_BLOCKREPRESENTATION_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKREPRESENTATION_flag_set", _wrap_Dwg_Object_BLOCKREPRESENTATION_flag_set},
{"LibreDWGc::Dwg_Object_BLOCKREPRESENTATION_flag_get", _wrap_Dwg_Object_BLOCKREPRESENTATION_flag_get},
{"LibreDWGc::Dwg_Object_BLOCKREPRESENTATION_block_set", _wrap_Dwg_Object_BLOCKREPRESENTATION_block_set},
{"LibreDWGc::Dwg_Object_BLOCKREPRESENTATION_block_get", _wrap_Dwg_Object_BLOCKREPRESENTATION_block_get},
{"LibreDWGc::new_Dwg_Object_BLOCKREPRESENTATION", _wrap_new_Dwg_Object_BLOCKREPRESENTATION},
{"LibreDWGc::delete_Dwg_Object_BLOCKREPRESENTATION", _wrap_delete_Dwg_Object_BLOCKREPRESENTATION},
{"LibreDWGc::Dwg_Object_BLOCKROTATEACTION_parent_set", _wrap_Dwg_Object_BLOCKROTATEACTION_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATEACTION_parent_get", _wrap_Dwg_Object_BLOCKROTATEACTION_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATEACTION_evalexpr_set", _wrap_Dwg_Object_BLOCKROTATEACTION_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATEACTION_evalexpr_get", _wrap_Dwg_Object_BLOCKROTATEACTION_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATEACTION_name_set", _wrap_Dwg_Object_BLOCKROTATEACTION_name_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATEACTION_name_get", _wrap_Dwg_Object_BLOCKROTATEACTION_name_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATEACTION_be_major_set", _wrap_Dwg_Object_BLOCKROTATEACTION_be_major_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATEACTION_be_major_get", _wrap_Dwg_Object_BLOCKROTATEACTION_be_major_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATEACTION_be_minor_set", _wrap_Dwg_Object_BLOCKROTATEACTION_be_minor_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATEACTION_be_minor_get", _wrap_Dwg_Object_BLOCKROTATEACTION_be_minor_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATEACTION_eed1071_set", _wrap_Dwg_Object_BLOCKROTATEACTION_eed1071_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATEACTION_eed1071_get", _wrap_Dwg_Object_BLOCKROTATEACTION_eed1071_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATEACTION_display_location_set", _wrap_Dwg_Object_BLOCKROTATEACTION_display_location_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATEACTION_display_location_get", _wrap_Dwg_Object_BLOCKROTATEACTION_display_location_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATEACTION_num_actions_set", _wrap_Dwg_Object_BLOCKROTATEACTION_num_actions_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATEACTION_num_actions_get", _wrap_Dwg_Object_BLOCKROTATEACTION_num_actions_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATEACTION_actions_set", _wrap_Dwg_Object_BLOCKROTATEACTION_actions_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATEACTION_actions_get", _wrap_Dwg_Object_BLOCKROTATEACTION_actions_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATEACTION_num_deps_set", _wrap_Dwg_Object_BLOCKROTATEACTION_num_deps_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATEACTION_num_deps_get", _wrap_Dwg_Object_BLOCKROTATEACTION_num_deps_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATEACTION_deps_set", _wrap_Dwg_Object_BLOCKROTATEACTION_deps_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATEACTION_deps_get", _wrap_Dwg_Object_BLOCKROTATEACTION_deps_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATEACTION_offset_set", _wrap_Dwg_Object_BLOCKROTATEACTION_offset_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATEACTION_offset_get", _wrap_Dwg_Object_BLOCKROTATEACTION_offset_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATEACTION_conn_pts_set", _wrap_Dwg_Object_BLOCKROTATEACTION_conn_pts_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATEACTION_conn_pts_get", _wrap_Dwg_Object_BLOCKROTATEACTION_conn_pts_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATEACTION_dependent_set", _wrap_Dwg_Object_BLOCKROTATEACTION_dependent_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATEACTION_dependent_get", _wrap_Dwg_Object_BLOCKROTATEACTION_dependent_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATEACTION_base_pt_set", _wrap_Dwg_Object_BLOCKROTATEACTION_base_pt_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATEACTION_base_pt_get", _wrap_Dwg_Object_BLOCKROTATEACTION_base_pt_get},
{"LibreDWGc::new_Dwg_Object_BLOCKROTATEACTION", _wrap_new_Dwg_Object_BLOCKROTATEACTION},
{"LibreDWGc::delete_Dwg_Object_BLOCKROTATEACTION", _wrap_delete_Dwg_Object_BLOCKROTATEACTION},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_parent_set", _wrap_Dwg_Object_BLOCKROTATIONGRIP_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_parent_get", _wrap_Dwg_Object_BLOCKROTATIONGRIP_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_evalexpr_set", _wrap_Dwg_Object_BLOCKROTATIONGRIP_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_evalexpr_get", _wrap_Dwg_Object_BLOCKROTATIONGRIP_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_name_set", _wrap_Dwg_Object_BLOCKROTATIONGRIP_name_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_name_get", _wrap_Dwg_Object_BLOCKROTATIONGRIP_name_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_be_major_set", _wrap_Dwg_Object_BLOCKROTATIONGRIP_be_major_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_be_major_get", _wrap_Dwg_Object_BLOCKROTATIONGRIP_be_major_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_be_minor_set", _wrap_Dwg_Object_BLOCKROTATIONGRIP_be_minor_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_be_minor_get", _wrap_Dwg_Object_BLOCKROTATIONGRIP_be_minor_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_eed1071_set", _wrap_Dwg_Object_BLOCKROTATIONGRIP_eed1071_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_eed1071_get", _wrap_Dwg_Object_BLOCKROTATIONGRIP_eed1071_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_bg_bl91_set", _wrap_Dwg_Object_BLOCKROTATIONGRIP_bg_bl91_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_bg_bl91_get", _wrap_Dwg_Object_BLOCKROTATIONGRIP_bg_bl91_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_bg_bl92_set", _wrap_Dwg_Object_BLOCKROTATIONGRIP_bg_bl92_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_bg_bl92_get", _wrap_Dwg_Object_BLOCKROTATIONGRIP_bg_bl92_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_bg_location_set", _wrap_Dwg_Object_BLOCKROTATIONGRIP_bg_location_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_bg_location_get", _wrap_Dwg_Object_BLOCKROTATIONGRIP_bg_location_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_bg_insert_cycling_set", _wrap_Dwg_Object_BLOCKROTATIONGRIP_bg_insert_cycling_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_bg_insert_cycling_get", _wrap_Dwg_Object_BLOCKROTATIONGRIP_bg_insert_cycling_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_bg_insert_cycling_weight_set", _wrap_Dwg_Object_BLOCKROTATIONGRIP_bg_insert_cycling_weight_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_bg_insert_cycling_weight_get", _wrap_Dwg_Object_BLOCKROTATIONGRIP_bg_insert_cycling_weight_get},
{"LibreDWGc::new_Dwg_Object_BLOCKROTATIONGRIP", _wrap_new_Dwg_Object_BLOCKROTATIONGRIP},
{"LibreDWGc::delete_Dwg_Object_BLOCKROTATIONGRIP", _wrap_delete_Dwg_Object_BLOCKROTATIONGRIP},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_parent_set", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_parent_get", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_evalexpr_set", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_evalexpr_get", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_name_set", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_name_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_name_get", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_name_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_be_major_set", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_be_major_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_be_major_get", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_be_major_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_be_minor_set", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_be_minor_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_be_minor_get", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_be_minor_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_eed1071_set", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_eed1071_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_eed1071_get", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_eed1071_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_show_properties_set", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_show_properties_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_show_properties_get", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_show_properties_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_chain_actions_set", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_chain_actions_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_chain_actions_get", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_chain_actions_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_def_basept_set", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_def_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_def_basept_get", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_def_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_def_endpt_set", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_def_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_def_endpt_get", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_def_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_prop1_set", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_prop1_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_prop1_get", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_prop1_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_prop2_set", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_prop2_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_prop2_get", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_prop2_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_prop3_set", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_prop3_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_prop3_get", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_prop3_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_prop4_set", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_prop4_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_prop4_get", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_prop4_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_prop_states_set", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_prop_states_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_prop_states_get", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_prop_states_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_parameter_base_location_set", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_parameter_base_location_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_parameter_base_location_get", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_parameter_base_location_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_upd_basept_set", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_upd_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_upd_basept_get", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_upd_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_basept_set", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_basept_get", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_upd_endpt_set", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_upd_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_upd_endpt_get", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_upd_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_endpt_set", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_endpt_get", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_def_base_angle_pt_set", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_def_base_angle_pt_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_def_base_angle_pt_get", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_def_base_angle_pt_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_angle_name_set", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_angle_name_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_angle_name_get", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_angle_name_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_angle_desc_set", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_angle_desc_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_angle_desc_get", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_angle_desc_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_angle_set", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_angle_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_angle_get", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_angle_get},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_angle_value_set_set", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_angle_value_set_set},
{"LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_angle_value_set_get", _wrap_Dwg_Object_BLOCKROTATIONPARAMETER_angle_value_set_get},
{"LibreDWGc::new_Dwg_Object_BLOCKROTATIONPARAMETER", _wrap_new_Dwg_Object_BLOCKROTATIONPARAMETER},
{"LibreDWGc::delete_Dwg_Object_BLOCKROTATIONPARAMETER", _wrap_delete_Dwg_Object_BLOCKROTATIONPARAMETER},
{"LibreDWGc::Dwg_Object_BLOCKSCALEACTION_parent_set", _wrap_Dwg_Object_BLOCKSCALEACTION_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKSCALEACTION_parent_get", _wrap_Dwg_Object_BLOCKSCALEACTION_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKSCALEACTION_evalexpr_set", _wrap_Dwg_Object_BLOCKSCALEACTION_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKSCALEACTION_evalexpr_get", _wrap_Dwg_Object_BLOCKSCALEACTION_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKSCALEACTION_name_set", _wrap_Dwg_Object_BLOCKSCALEACTION_name_set},
{"LibreDWGc::Dwg_Object_BLOCKSCALEACTION_name_get", _wrap_Dwg_Object_BLOCKSCALEACTION_name_get},
{"LibreDWGc::Dwg_Object_BLOCKSCALEACTION_be_major_set", _wrap_Dwg_Object_BLOCKSCALEACTION_be_major_set},
{"LibreDWGc::Dwg_Object_BLOCKSCALEACTION_be_major_get", _wrap_Dwg_Object_BLOCKSCALEACTION_be_major_get},
{"LibreDWGc::Dwg_Object_BLOCKSCALEACTION_be_minor_set", _wrap_Dwg_Object_BLOCKSCALEACTION_be_minor_set},
{"LibreDWGc::Dwg_Object_BLOCKSCALEACTION_be_minor_get", _wrap_Dwg_Object_BLOCKSCALEACTION_be_minor_get},
{"LibreDWGc::Dwg_Object_BLOCKSCALEACTION_eed1071_set", _wrap_Dwg_Object_BLOCKSCALEACTION_eed1071_set},
{"LibreDWGc::Dwg_Object_BLOCKSCALEACTION_eed1071_get", _wrap_Dwg_Object_BLOCKSCALEACTION_eed1071_get},
{"LibreDWGc::Dwg_Object_BLOCKSCALEACTION_display_location_set", _wrap_Dwg_Object_BLOCKSCALEACTION_display_location_set},
{"LibreDWGc::Dwg_Object_BLOCKSCALEACTION_display_location_get", _wrap_Dwg_Object_BLOCKSCALEACTION_display_location_get},
{"LibreDWGc::Dwg_Object_BLOCKSCALEACTION_num_actions_set", _wrap_Dwg_Object_BLOCKSCALEACTION_num_actions_set},
{"LibreDWGc::Dwg_Object_BLOCKSCALEACTION_num_actions_get", _wrap_Dwg_Object_BLOCKSCALEACTION_num_actions_get},
{"LibreDWGc::Dwg_Object_BLOCKSCALEACTION_actions_set", _wrap_Dwg_Object_BLOCKSCALEACTION_actions_set},
{"LibreDWGc::Dwg_Object_BLOCKSCALEACTION_actions_get", _wrap_Dwg_Object_BLOCKSCALEACTION_actions_get},
{"LibreDWGc::Dwg_Object_BLOCKSCALEACTION_num_deps_set", _wrap_Dwg_Object_BLOCKSCALEACTION_num_deps_set},
{"LibreDWGc::Dwg_Object_BLOCKSCALEACTION_num_deps_get", _wrap_Dwg_Object_BLOCKSCALEACTION_num_deps_get},
{"LibreDWGc::Dwg_Object_BLOCKSCALEACTION_deps_set", _wrap_Dwg_Object_BLOCKSCALEACTION_deps_set},
{"LibreDWGc::Dwg_Object_BLOCKSCALEACTION_deps_get", _wrap_Dwg_Object_BLOCKSCALEACTION_deps_get},
{"LibreDWGc::Dwg_Object_BLOCKSCALEACTION_offset_set", _wrap_Dwg_Object_BLOCKSCALEACTION_offset_set},
{"LibreDWGc::Dwg_Object_BLOCKSCALEACTION_offset_get", _wrap_Dwg_Object_BLOCKSCALEACTION_offset_get},
{"LibreDWGc::Dwg_Object_BLOCKSCALEACTION_conn_pts_set", _wrap_Dwg_Object_BLOCKSCALEACTION_conn_pts_set},
{"LibreDWGc::Dwg_Object_BLOCKSCALEACTION_conn_pts_get", _wrap_Dwg_Object_BLOCKSCALEACTION_conn_pts_get},
{"LibreDWGc::Dwg_Object_BLOCKSCALEACTION_dependent_set", _wrap_Dwg_Object_BLOCKSCALEACTION_dependent_set},
{"LibreDWGc::Dwg_Object_BLOCKSCALEACTION_dependent_get", _wrap_Dwg_Object_BLOCKSCALEACTION_dependent_get},
{"LibreDWGc::Dwg_Object_BLOCKSCALEACTION_base_pt_set", _wrap_Dwg_Object_BLOCKSCALEACTION_base_pt_set},
{"LibreDWGc::Dwg_Object_BLOCKSCALEACTION_base_pt_get", _wrap_Dwg_Object_BLOCKSCALEACTION_base_pt_get},
{"LibreDWGc::new_Dwg_Object_BLOCKSCALEACTION", _wrap_new_Dwg_Object_BLOCKSCALEACTION},
{"LibreDWGc::delete_Dwg_Object_BLOCKSCALEACTION", _wrap_delete_Dwg_Object_BLOCKSCALEACTION},
{"LibreDWGc::Dwg_BLOCKSTRETCHACTION_handles_parent_set", _wrap_Dwg_BLOCKSTRETCHACTION_handles_parent_set},
{"LibreDWGc::Dwg_BLOCKSTRETCHACTION_handles_parent_get", _wrap_Dwg_BLOCKSTRETCHACTION_handles_parent_get},
{"LibreDWGc::Dwg_BLOCKSTRETCHACTION_handles_hdl_set", _wrap_Dwg_BLOCKSTRETCHACTION_handles_hdl_set},
{"LibreDWGc::Dwg_BLOCKSTRETCHACTION_handles_hdl_get", _wrap_Dwg_BLOCKSTRETCHACTION_handles_hdl_get},
{"LibreDWGc::Dwg_BLOCKSTRETCHACTION_handles_num_indexes_set", _wrap_Dwg_BLOCKSTRETCHACTION_handles_num_indexes_set},
{"LibreDWGc::Dwg_BLOCKSTRETCHACTION_handles_num_indexes_get", _wrap_Dwg_BLOCKSTRETCHACTION_handles_num_indexes_get},
{"LibreDWGc::Dwg_BLOCKSTRETCHACTION_handles_indexes_set", _wrap_Dwg_BLOCKSTRETCHACTION_handles_indexes_set},
{"LibreDWGc::Dwg_BLOCKSTRETCHACTION_handles_indexes_get", _wrap_Dwg_BLOCKSTRETCHACTION_handles_indexes_get},
{"LibreDWGc::new_Dwg_BLOCKSTRETCHACTION_handles", _wrap_new_Dwg_BLOCKSTRETCHACTION_handles},
{"LibreDWGc::delete_Dwg_BLOCKSTRETCHACTION_handles", _wrap_delete_Dwg_BLOCKSTRETCHACTION_handles},
{"LibreDWGc::Dwg_BLOCKSTRETCHACTION_codes_parent_set", _wrap_Dwg_BLOCKSTRETCHACTION_codes_parent_set},
{"LibreDWGc::Dwg_BLOCKSTRETCHACTION_codes_parent_get", _wrap_Dwg_BLOCKSTRETCHACTION_codes_parent_get},
{"LibreDWGc::Dwg_BLOCKSTRETCHACTION_codes_bl95_set", _wrap_Dwg_BLOCKSTRETCHACTION_codes_bl95_set},
{"LibreDWGc::Dwg_BLOCKSTRETCHACTION_codes_bl95_get", _wrap_Dwg_BLOCKSTRETCHACTION_codes_bl95_get},
{"LibreDWGc::Dwg_BLOCKSTRETCHACTION_codes_num_indexes_set", _wrap_Dwg_BLOCKSTRETCHACTION_codes_num_indexes_set},
{"LibreDWGc::Dwg_BLOCKSTRETCHACTION_codes_num_indexes_get", _wrap_Dwg_BLOCKSTRETCHACTION_codes_num_indexes_get},
{"LibreDWGc::Dwg_BLOCKSTRETCHACTION_codes_indexes_set", _wrap_Dwg_BLOCKSTRETCHACTION_codes_indexes_set},
{"LibreDWGc::Dwg_BLOCKSTRETCHACTION_codes_indexes_get", _wrap_Dwg_BLOCKSTRETCHACTION_codes_indexes_get},
{"LibreDWGc::new_Dwg_BLOCKSTRETCHACTION_codes", _wrap_new_Dwg_BLOCKSTRETCHACTION_codes},
{"LibreDWGc::delete_Dwg_BLOCKSTRETCHACTION_codes", _wrap_delete_Dwg_BLOCKSTRETCHACTION_codes},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_parent_set", _wrap_Dwg_Object_BLOCKSTRETCHACTION_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_parent_get", _wrap_Dwg_Object_BLOCKSTRETCHACTION_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_evalexpr_set", _wrap_Dwg_Object_BLOCKSTRETCHACTION_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_evalexpr_get", _wrap_Dwg_Object_BLOCKSTRETCHACTION_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_name_set", _wrap_Dwg_Object_BLOCKSTRETCHACTION_name_set},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_name_get", _wrap_Dwg_Object_BLOCKSTRETCHACTION_name_get},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_be_major_set", _wrap_Dwg_Object_BLOCKSTRETCHACTION_be_major_set},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_be_major_get", _wrap_Dwg_Object_BLOCKSTRETCHACTION_be_major_get},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_be_minor_set", _wrap_Dwg_Object_BLOCKSTRETCHACTION_be_minor_set},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_be_minor_get", _wrap_Dwg_Object_BLOCKSTRETCHACTION_be_minor_get},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_eed1071_set", _wrap_Dwg_Object_BLOCKSTRETCHACTION_eed1071_set},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_eed1071_get", _wrap_Dwg_Object_BLOCKSTRETCHACTION_eed1071_get},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_display_location_set", _wrap_Dwg_Object_BLOCKSTRETCHACTION_display_location_set},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_display_location_get", _wrap_Dwg_Object_BLOCKSTRETCHACTION_display_location_get},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_num_actions_set", _wrap_Dwg_Object_BLOCKSTRETCHACTION_num_actions_set},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_num_actions_get", _wrap_Dwg_Object_BLOCKSTRETCHACTION_num_actions_get},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_actions_set", _wrap_Dwg_Object_BLOCKSTRETCHACTION_actions_set},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_actions_get", _wrap_Dwg_Object_BLOCKSTRETCHACTION_actions_get},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_num_deps_set", _wrap_Dwg_Object_BLOCKSTRETCHACTION_num_deps_set},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_num_deps_get", _wrap_Dwg_Object_BLOCKSTRETCHACTION_num_deps_get},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_deps_set", _wrap_Dwg_Object_BLOCKSTRETCHACTION_deps_set},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_deps_get", _wrap_Dwg_Object_BLOCKSTRETCHACTION_deps_get},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_conn_pts_set", _wrap_Dwg_Object_BLOCKSTRETCHACTION_conn_pts_set},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_conn_pts_get", _wrap_Dwg_Object_BLOCKSTRETCHACTION_conn_pts_get},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_num_pts_set", _wrap_Dwg_Object_BLOCKSTRETCHACTION_num_pts_set},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_num_pts_get", _wrap_Dwg_Object_BLOCKSTRETCHACTION_num_pts_get},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_pts_set", _wrap_Dwg_Object_BLOCKSTRETCHACTION_pts_set},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_pts_get", _wrap_Dwg_Object_BLOCKSTRETCHACTION_pts_get},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_num_hdls_set", _wrap_Dwg_Object_BLOCKSTRETCHACTION_num_hdls_set},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_num_hdls_get", _wrap_Dwg_Object_BLOCKSTRETCHACTION_num_hdls_get},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_hdls_set", _wrap_Dwg_Object_BLOCKSTRETCHACTION_hdls_set},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_hdls_get", _wrap_Dwg_Object_BLOCKSTRETCHACTION_hdls_get},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_num_codes_set", _wrap_Dwg_Object_BLOCKSTRETCHACTION_num_codes_set},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_num_codes_get", _wrap_Dwg_Object_BLOCKSTRETCHACTION_num_codes_get},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_codes_set", _wrap_Dwg_Object_BLOCKSTRETCHACTION_codes_set},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_codes_get", _wrap_Dwg_Object_BLOCKSTRETCHACTION_codes_get},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_action_offset_x_set", _wrap_Dwg_Object_BLOCKSTRETCHACTION_action_offset_x_set},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_action_offset_x_get", _wrap_Dwg_Object_BLOCKSTRETCHACTION_action_offset_x_get},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_action_offset_y_set", _wrap_Dwg_Object_BLOCKSTRETCHACTION_action_offset_y_set},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_action_offset_y_get", _wrap_Dwg_Object_BLOCKSTRETCHACTION_action_offset_y_get},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_angle_offset_set", _wrap_Dwg_Object_BLOCKSTRETCHACTION_angle_offset_set},
{"LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_angle_offset_get", _wrap_Dwg_Object_BLOCKSTRETCHACTION_angle_offset_get},
{"LibreDWGc::new_Dwg_Object_BLOCKSTRETCHACTION", _wrap_new_Dwg_Object_BLOCKSTRETCHACTION},
{"LibreDWGc::delete_Dwg_Object_BLOCKSTRETCHACTION", _wrap_delete_Dwg_Object_BLOCKSTRETCHACTION},
{"LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_parent_set", _wrap_Dwg_Object_BLOCKUSERPARAMETER_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_parent_get", _wrap_Dwg_Object_BLOCKUSERPARAMETER_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_evalexpr_set", _wrap_Dwg_Object_BLOCKUSERPARAMETER_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_evalexpr_get", _wrap_Dwg_Object_BLOCKUSERPARAMETER_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_name_set", _wrap_Dwg_Object_BLOCKUSERPARAMETER_name_set},
{"LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_name_get", _wrap_Dwg_Object_BLOCKUSERPARAMETER_name_get},
{"LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_be_major_set", _wrap_Dwg_Object_BLOCKUSERPARAMETER_be_major_set},
{"LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_be_major_get", _wrap_Dwg_Object_BLOCKUSERPARAMETER_be_major_get},
{"LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_be_minor_set", _wrap_Dwg_Object_BLOCKUSERPARAMETER_be_minor_set},
{"LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_be_minor_get", _wrap_Dwg_Object_BLOCKUSERPARAMETER_be_minor_get},
{"LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_eed1071_set", _wrap_Dwg_Object_BLOCKUSERPARAMETER_eed1071_set},
{"LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_eed1071_get", _wrap_Dwg_Object_BLOCKUSERPARAMETER_eed1071_get},
{"LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_show_properties_set", _wrap_Dwg_Object_BLOCKUSERPARAMETER_show_properties_set},
{"LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_show_properties_get", _wrap_Dwg_Object_BLOCKUSERPARAMETER_show_properties_get},
{"LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_chain_actions_set", _wrap_Dwg_Object_BLOCKUSERPARAMETER_chain_actions_set},
{"LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_chain_actions_get", _wrap_Dwg_Object_BLOCKUSERPARAMETER_chain_actions_get},
{"LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_def_pt_set", _wrap_Dwg_Object_BLOCKUSERPARAMETER_def_pt_set},
{"LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_def_pt_get", _wrap_Dwg_Object_BLOCKUSERPARAMETER_def_pt_get},
{"LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_num_propinfos_set", _wrap_Dwg_Object_BLOCKUSERPARAMETER_num_propinfos_set},
{"LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_num_propinfos_get", _wrap_Dwg_Object_BLOCKUSERPARAMETER_num_propinfos_get},
{"LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_prop1_set", _wrap_Dwg_Object_BLOCKUSERPARAMETER_prop1_set},
{"LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_prop1_get", _wrap_Dwg_Object_BLOCKUSERPARAMETER_prop1_get},
{"LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_prop2_set", _wrap_Dwg_Object_BLOCKUSERPARAMETER_prop2_set},
{"LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_prop2_get", _wrap_Dwg_Object_BLOCKUSERPARAMETER_prop2_get},
{"LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_flag_set", _wrap_Dwg_Object_BLOCKUSERPARAMETER_flag_set},
{"LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_flag_get", _wrap_Dwg_Object_BLOCKUSERPARAMETER_flag_get},
{"LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_assocvariable_set", _wrap_Dwg_Object_BLOCKUSERPARAMETER_assocvariable_set},
{"LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_assocvariable_get", _wrap_Dwg_Object_BLOCKUSERPARAMETER_assocvariable_get},
{"LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_expr_set", _wrap_Dwg_Object_BLOCKUSERPARAMETER_expr_set},
{"LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_expr_get", _wrap_Dwg_Object_BLOCKUSERPARAMETER_expr_get},
{"LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_value_set", _wrap_Dwg_Object_BLOCKUSERPARAMETER_value_set},
{"LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_value_get", _wrap_Dwg_Object_BLOCKUSERPARAMETER_value_get},
{"LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_type_set", _wrap_Dwg_Object_BLOCKUSERPARAMETER_type_set},
{"LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_type_get", _wrap_Dwg_Object_BLOCKUSERPARAMETER_type_get},
{"LibreDWGc::new_Dwg_Object_BLOCKUSERPARAMETER", _wrap_new_Dwg_Object_BLOCKUSERPARAMETER},
{"LibreDWGc::delete_Dwg_Object_BLOCKUSERPARAMETER", _wrap_delete_Dwg_Object_BLOCKUSERPARAMETER},
{"LibreDWGc::Dwg_Object_BLOCKXYGRIP_parent_set", _wrap_Dwg_Object_BLOCKXYGRIP_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKXYGRIP_parent_get", _wrap_Dwg_Object_BLOCKXYGRIP_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKXYGRIP_evalexpr_set", _wrap_Dwg_Object_BLOCKXYGRIP_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKXYGRIP_evalexpr_get", _wrap_Dwg_Object_BLOCKXYGRIP_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKXYGRIP_name_set", _wrap_Dwg_Object_BLOCKXYGRIP_name_set},
{"LibreDWGc::Dwg_Object_BLOCKXYGRIP_name_get", _wrap_Dwg_Object_BLOCKXYGRIP_name_get},
{"LibreDWGc::Dwg_Object_BLOCKXYGRIP_be_major_set", _wrap_Dwg_Object_BLOCKXYGRIP_be_major_set},
{"LibreDWGc::Dwg_Object_BLOCKXYGRIP_be_major_get", _wrap_Dwg_Object_BLOCKXYGRIP_be_major_get},
{"LibreDWGc::Dwg_Object_BLOCKXYGRIP_be_minor_set", _wrap_Dwg_Object_BLOCKXYGRIP_be_minor_set},
{"LibreDWGc::Dwg_Object_BLOCKXYGRIP_be_minor_get", _wrap_Dwg_Object_BLOCKXYGRIP_be_minor_get},
{"LibreDWGc::Dwg_Object_BLOCKXYGRIP_eed1071_set", _wrap_Dwg_Object_BLOCKXYGRIP_eed1071_set},
{"LibreDWGc::Dwg_Object_BLOCKXYGRIP_eed1071_get", _wrap_Dwg_Object_BLOCKXYGRIP_eed1071_get},
{"LibreDWGc::Dwg_Object_BLOCKXYGRIP_bg_bl91_set", _wrap_Dwg_Object_BLOCKXYGRIP_bg_bl91_set},
{"LibreDWGc::Dwg_Object_BLOCKXYGRIP_bg_bl91_get", _wrap_Dwg_Object_BLOCKXYGRIP_bg_bl91_get},
{"LibreDWGc::Dwg_Object_BLOCKXYGRIP_bg_bl92_set", _wrap_Dwg_Object_BLOCKXYGRIP_bg_bl92_set},
{"LibreDWGc::Dwg_Object_BLOCKXYGRIP_bg_bl92_get", _wrap_Dwg_Object_BLOCKXYGRIP_bg_bl92_get},
{"LibreDWGc::Dwg_Object_BLOCKXYGRIP_bg_location_set", _wrap_Dwg_Object_BLOCKXYGRIP_bg_location_set},
{"LibreDWGc::Dwg_Object_BLOCKXYGRIP_bg_location_get", _wrap_Dwg_Object_BLOCKXYGRIP_bg_location_get},
{"LibreDWGc::Dwg_Object_BLOCKXYGRIP_bg_insert_cycling_set", _wrap_Dwg_Object_BLOCKXYGRIP_bg_insert_cycling_set},
{"LibreDWGc::Dwg_Object_BLOCKXYGRIP_bg_insert_cycling_get", _wrap_Dwg_Object_BLOCKXYGRIP_bg_insert_cycling_get},
{"LibreDWGc::Dwg_Object_BLOCKXYGRIP_bg_insert_cycling_weight_set", _wrap_Dwg_Object_BLOCKXYGRIP_bg_insert_cycling_weight_set},
{"LibreDWGc::Dwg_Object_BLOCKXYGRIP_bg_insert_cycling_weight_get", _wrap_Dwg_Object_BLOCKXYGRIP_bg_insert_cycling_weight_get},
{"LibreDWGc::new_Dwg_Object_BLOCKXYGRIP", _wrap_new_Dwg_Object_BLOCKXYGRIP},
{"LibreDWGc::delete_Dwg_Object_BLOCKXYGRIP", _wrap_delete_Dwg_Object_BLOCKXYGRIP},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_parent_set", _wrap_Dwg_Object_BLOCKXYPARAMETER_parent_set},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_parent_get", _wrap_Dwg_Object_BLOCKXYPARAMETER_parent_get},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_evalexpr_set", _wrap_Dwg_Object_BLOCKXYPARAMETER_evalexpr_set},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_evalexpr_get", _wrap_Dwg_Object_BLOCKXYPARAMETER_evalexpr_get},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_name_set", _wrap_Dwg_Object_BLOCKXYPARAMETER_name_set},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_name_get", _wrap_Dwg_Object_BLOCKXYPARAMETER_name_get},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_be_major_set", _wrap_Dwg_Object_BLOCKXYPARAMETER_be_major_set},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_be_major_get", _wrap_Dwg_Object_BLOCKXYPARAMETER_be_major_get},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_be_minor_set", _wrap_Dwg_Object_BLOCKXYPARAMETER_be_minor_set},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_be_minor_get", _wrap_Dwg_Object_BLOCKXYPARAMETER_be_minor_get},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_eed1071_set", _wrap_Dwg_Object_BLOCKXYPARAMETER_eed1071_set},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_eed1071_get", _wrap_Dwg_Object_BLOCKXYPARAMETER_eed1071_get},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_show_properties_set", _wrap_Dwg_Object_BLOCKXYPARAMETER_show_properties_set},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_show_properties_get", _wrap_Dwg_Object_BLOCKXYPARAMETER_show_properties_get},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_chain_actions_set", _wrap_Dwg_Object_BLOCKXYPARAMETER_chain_actions_set},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_chain_actions_get", _wrap_Dwg_Object_BLOCKXYPARAMETER_chain_actions_get},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_def_basept_set", _wrap_Dwg_Object_BLOCKXYPARAMETER_def_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_def_basept_get", _wrap_Dwg_Object_BLOCKXYPARAMETER_def_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_def_endpt_set", _wrap_Dwg_Object_BLOCKXYPARAMETER_def_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_def_endpt_get", _wrap_Dwg_Object_BLOCKXYPARAMETER_def_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_prop1_set", _wrap_Dwg_Object_BLOCKXYPARAMETER_prop1_set},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_prop1_get", _wrap_Dwg_Object_BLOCKXYPARAMETER_prop1_get},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_prop2_set", _wrap_Dwg_Object_BLOCKXYPARAMETER_prop2_set},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_prop2_get", _wrap_Dwg_Object_BLOCKXYPARAMETER_prop2_get},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_prop3_set", _wrap_Dwg_Object_BLOCKXYPARAMETER_prop3_set},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_prop3_get", _wrap_Dwg_Object_BLOCKXYPARAMETER_prop3_get},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_prop4_set", _wrap_Dwg_Object_BLOCKXYPARAMETER_prop4_set},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_prop4_get", _wrap_Dwg_Object_BLOCKXYPARAMETER_prop4_get},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_prop_states_set", _wrap_Dwg_Object_BLOCKXYPARAMETER_prop_states_set},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_prop_states_get", _wrap_Dwg_Object_BLOCKXYPARAMETER_prop_states_get},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_parameter_base_location_set", _wrap_Dwg_Object_BLOCKXYPARAMETER_parameter_base_location_set},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_parameter_base_location_get", _wrap_Dwg_Object_BLOCKXYPARAMETER_parameter_base_location_get},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_upd_basept_set", _wrap_Dwg_Object_BLOCKXYPARAMETER_upd_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_upd_basept_get", _wrap_Dwg_Object_BLOCKXYPARAMETER_upd_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_basept_set", _wrap_Dwg_Object_BLOCKXYPARAMETER_basept_set},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_basept_get", _wrap_Dwg_Object_BLOCKXYPARAMETER_basept_get},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_upd_endpt_set", _wrap_Dwg_Object_BLOCKXYPARAMETER_upd_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_upd_endpt_get", _wrap_Dwg_Object_BLOCKXYPARAMETER_upd_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_endpt_set", _wrap_Dwg_Object_BLOCKXYPARAMETER_endpt_set},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_endpt_get", _wrap_Dwg_Object_BLOCKXYPARAMETER_endpt_get},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_x_label_set", _wrap_Dwg_Object_BLOCKXYPARAMETER_x_label_set},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_x_label_get", _wrap_Dwg_Object_BLOCKXYPARAMETER_x_label_get},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_x_label_desc_set", _wrap_Dwg_Object_BLOCKXYPARAMETER_x_label_desc_set},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_x_label_desc_get", _wrap_Dwg_Object_BLOCKXYPARAMETER_x_label_desc_get},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_y_label_set", _wrap_Dwg_Object_BLOCKXYPARAMETER_y_label_set},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_y_label_get", _wrap_Dwg_Object_BLOCKXYPARAMETER_y_label_get},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_y_label_desc_set", _wrap_Dwg_Object_BLOCKXYPARAMETER_y_label_desc_set},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_y_label_desc_get", _wrap_Dwg_Object_BLOCKXYPARAMETER_y_label_desc_get},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_x_value_set", _wrap_Dwg_Object_BLOCKXYPARAMETER_x_value_set},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_x_value_get", _wrap_Dwg_Object_BLOCKXYPARAMETER_x_value_get},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_y_value_set", _wrap_Dwg_Object_BLOCKXYPARAMETER_y_value_set},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_y_value_get", _wrap_Dwg_Object_BLOCKXYPARAMETER_y_value_get},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_x_value_set_set", _wrap_Dwg_Object_BLOCKXYPARAMETER_x_value_set_set},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_x_value_set_get", _wrap_Dwg_Object_BLOCKXYPARAMETER_x_value_set_get},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_y_value_set_set", _wrap_Dwg_Object_BLOCKXYPARAMETER_y_value_set_set},
{"LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_y_value_set_get", _wrap_Dwg_Object_BLOCKXYPARAMETER_y_value_set_get},
{"LibreDWGc::new_Dwg_Object_BLOCKXYPARAMETER", _wrap_new_Dwg_Object_BLOCKXYPARAMETER},
{"LibreDWGc::delete_Dwg_Object_BLOCKXYPARAMETER", _wrap_delete_Dwg_Object_BLOCKXYPARAMETER},
{"LibreDWGc::Dwg_Object_DYNAMICBLOCKPROXYNODE_parent_set", _wrap_Dwg_Object_DYNAMICBLOCKPROXYNODE_parent_set},
{"LibreDWGc::Dwg_Object_DYNAMICBLOCKPROXYNODE_parent_get", _wrap_Dwg_Object_DYNAMICBLOCKPROXYNODE_parent_get},
{"LibreDWGc::Dwg_Object_DYNAMICBLOCKPROXYNODE_evalexpr_set", _wrap_Dwg_Object_DYNAMICBLOCKPROXYNODE_evalexpr_set},
{"LibreDWGc::Dwg_Object_DYNAMICBLOCKPROXYNODE_evalexpr_get", _wrap_Dwg_Object_DYNAMICBLOCKPROXYNODE_evalexpr_get},
{"LibreDWGc::new_Dwg_Object_DYNAMICBLOCKPROXYNODE", _wrap_new_Dwg_Object_DYNAMICBLOCKPROXYNODE},
{"LibreDWGc::delete_Dwg_Object_DYNAMICBLOCKPROXYNODE", _wrap_delete_Dwg_Object_DYNAMICBLOCKPROXYNODE},
{"LibreDWGc::Dwg_POINTCLOUD_IntensityStyle_parent_set", _wrap_Dwg_POINTCLOUD_IntensityStyle_parent_set},
{"LibreDWGc::Dwg_POINTCLOUD_IntensityStyle_parent_get", _wrap_Dwg_POINTCLOUD_IntensityStyle_parent_get},
{"LibreDWGc::Dwg_POINTCLOUD_IntensityStyle_min_intensity_set", _wrap_Dwg_POINTCLOUD_IntensityStyle_min_intensity_set},
{"LibreDWGc::Dwg_POINTCLOUD_IntensityStyle_min_intensity_get", _wrap_Dwg_POINTCLOUD_IntensityStyle_min_intensity_get},
{"LibreDWGc::Dwg_POINTCLOUD_IntensityStyle_max_intensity_set", _wrap_Dwg_POINTCLOUD_IntensityStyle_max_intensity_set},
{"LibreDWGc::Dwg_POINTCLOUD_IntensityStyle_max_intensity_get", _wrap_Dwg_POINTCLOUD_IntensityStyle_max_intensity_get},
{"LibreDWGc::Dwg_POINTCLOUD_IntensityStyle_intensity_low_treshold_set", _wrap_Dwg_POINTCLOUD_IntensityStyle_intensity_low_treshold_set},
{"LibreDWGc::Dwg_POINTCLOUD_IntensityStyle_intensity_low_treshold_get", _wrap_Dwg_POINTCLOUD_IntensityStyle_intensity_low_treshold_get},
{"LibreDWGc::Dwg_POINTCLOUD_IntensityStyle_intensity_high_treshold_set", _wrap_Dwg_POINTCLOUD_IntensityStyle_intensity_high_treshold_set},
{"LibreDWGc::Dwg_POINTCLOUD_IntensityStyle_intensity_high_treshold_get", _wrap_Dwg_POINTCLOUD_IntensityStyle_intensity_high_treshold_get},
{"LibreDWGc::new_Dwg_POINTCLOUD_IntensityStyle", _wrap_new_Dwg_POINTCLOUD_IntensityStyle},
{"LibreDWGc::delete_Dwg_POINTCLOUD_IntensityStyle", _wrap_delete_Dwg_POINTCLOUD_IntensityStyle},
{"LibreDWGc::Dwg_POINTCLOUD_Clippings_parent_set", _wrap_Dwg_POINTCLOUD_Clippings_parent_set},
{"LibreDWGc::Dwg_POINTCLOUD_Clippings_parent_get", _wrap_Dwg_POINTCLOUD_Clippings_parent_get},
{"LibreDWGc::Dwg_POINTCLOUD_Clippings_is_inverted_set", _wrap_Dwg_POINTCLOUD_Clippings_is_inverted_set},
{"LibreDWGc::Dwg_POINTCLOUD_Clippings_is_inverted_get", _wrap_Dwg_POINTCLOUD_Clippings_is_inverted_get},
{"LibreDWGc::Dwg_POINTCLOUD_Clippings_type_set", _wrap_Dwg_POINTCLOUD_Clippings_type_set},
{"LibreDWGc::Dwg_POINTCLOUD_Clippings_type_get", _wrap_Dwg_POINTCLOUD_Clippings_type_get},
{"LibreDWGc::Dwg_POINTCLOUD_Clippings_num_vertices_set", _wrap_Dwg_POINTCLOUD_Clippings_num_vertices_set},
{"LibreDWGc::Dwg_POINTCLOUD_Clippings_num_vertices_get", _wrap_Dwg_POINTCLOUD_Clippings_num_vertices_get},
{"LibreDWGc::Dwg_POINTCLOUD_Clippings_vertices_set", _wrap_Dwg_POINTCLOUD_Clippings_vertices_set},
{"LibreDWGc::Dwg_POINTCLOUD_Clippings_vertices_get", _wrap_Dwg_POINTCLOUD_Clippings_vertices_get},
{"LibreDWGc::Dwg_POINTCLOUD_Clippings_z_min_set", _wrap_Dwg_POINTCLOUD_Clippings_z_min_set},
{"LibreDWGc::Dwg_POINTCLOUD_Clippings_z_min_get", _wrap_Dwg_POINTCLOUD_Clippings_z_min_get},
{"LibreDWGc::Dwg_POINTCLOUD_Clippings_z_max_set", _wrap_Dwg_POINTCLOUD_Clippings_z_max_set},
{"LibreDWGc::Dwg_POINTCLOUD_Clippings_z_max_get", _wrap_Dwg_POINTCLOUD_Clippings_z_max_get},
{"LibreDWGc::new_Dwg_POINTCLOUD_Clippings", _wrap_new_Dwg_POINTCLOUD_Clippings},
{"LibreDWGc::delete_Dwg_POINTCLOUD_Clippings", _wrap_delete_Dwg_POINTCLOUD_Clippings},
{"LibreDWGc::Dwg_POINTCLOUDEX_Croppings_parent_set", _wrap_Dwg_POINTCLOUDEX_Croppings_parent_set},
{"LibreDWGc::Dwg_POINTCLOUDEX_Croppings_parent_get", _wrap_Dwg_POINTCLOUDEX_Croppings_parent_get},
{"LibreDWGc::Dwg_POINTCLOUDEX_Croppings_type_set", _wrap_Dwg_POINTCLOUDEX_Croppings_type_set},
{"LibreDWGc::Dwg_POINTCLOUDEX_Croppings_type_get", _wrap_Dwg_POINTCLOUDEX_Croppings_type_get},
{"LibreDWGc::Dwg_POINTCLOUDEX_Croppings_is_inside_set", _wrap_Dwg_POINTCLOUDEX_Croppings_is_inside_set},
{"LibreDWGc::Dwg_POINTCLOUDEX_Croppings_is_inside_get", _wrap_Dwg_POINTCLOUDEX_Croppings_is_inside_get},
{"LibreDWGc::Dwg_POINTCLOUDEX_Croppings_is_inverted_set", _wrap_Dwg_POINTCLOUDEX_Croppings_is_inverted_set},
{"LibreDWGc::Dwg_POINTCLOUDEX_Croppings_is_inverted_get", _wrap_Dwg_POINTCLOUDEX_Croppings_is_inverted_get},
{"LibreDWGc::Dwg_POINTCLOUDEX_Croppings_crop_plane_set", _wrap_Dwg_POINTCLOUDEX_Croppings_crop_plane_set},
{"LibreDWGc::Dwg_POINTCLOUDEX_Croppings_crop_plane_get", _wrap_Dwg_POINTCLOUDEX_Croppings_crop_plane_get},
{"LibreDWGc::Dwg_POINTCLOUDEX_Croppings_crop_x_dir_set", _wrap_Dwg_POINTCLOUDEX_Croppings_crop_x_dir_set},
{"LibreDWGc::Dwg_POINTCLOUDEX_Croppings_crop_x_dir_get", _wrap_Dwg_POINTCLOUDEX_Croppings_crop_x_dir_get},
{"LibreDWGc::Dwg_POINTCLOUDEX_Croppings_crop_y_dir_set", _wrap_Dwg_POINTCLOUDEX_Croppings_crop_y_dir_set},
{"LibreDWGc::Dwg_POINTCLOUDEX_Croppings_crop_y_dir_get", _wrap_Dwg_POINTCLOUDEX_Croppings_crop_y_dir_get},
{"LibreDWGc::Dwg_POINTCLOUDEX_Croppings_num_pts_set", _wrap_Dwg_POINTCLOUDEX_Croppings_num_pts_set},
{"LibreDWGc::Dwg_POINTCLOUDEX_Croppings_num_pts_get", _wrap_Dwg_POINTCLOUDEX_Croppings_num_pts_get},
{"LibreDWGc::Dwg_POINTCLOUDEX_Croppings_pts_set", _wrap_Dwg_POINTCLOUDEX_Croppings_pts_set},
{"LibreDWGc::Dwg_POINTCLOUDEX_Croppings_pts_get", _wrap_Dwg_POINTCLOUDEX_Croppings_pts_get},
{"LibreDWGc::new_Dwg_POINTCLOUDEX_Croppings", _wrap_new_Dwg_POINTCLOUDEX_Croppings},
{"LibreDWGc::delete_Dwg_POINTCLOUDEX_Croppings", _wrap_delete_Dwg_POINTCLOUDEX_Croppings},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_parent_set", _wrap_Dwg_Entity_POINTCLOUD_parent_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_parent_get", _wrap_Dwg_Entity_POINTCLOUD_parent_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_class_version_set", _wrap_Dwg_Entity_POINTCLOUD_class_version_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_class_version_get", _wrap_Dwg_Entity_POINTCLOUD_class_version_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_origin_set", _wrap_Dwg_Entity_POINTCLOUD_origin_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_origin_get", _wrap_Dwg_Entity_POINTCLOUD_origin_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_saved_filename_set", _wrap_Dwg_Entity_POINTCLOUD_saved_filename_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_saved_filename_get", _wrap_Dwg_Entity_POINTCLOUD_saved_filename_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_num_source_files_set", _wrap_Dwg_Entity_POINTCLOUD_num_source_files_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_num_source_files_get", _wrap_Dwg_Entity_POINTCLOUD_num_source_files_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_source_files_set", _wrap_Dwg_Entity_POINTCLOUD_source_files_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_source_files_get", _wrap_Dwg_Entity_POINTCLOUD_source_files_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_extents_min_set", _wrap_Dwg_Entity_POINTCLOUD_extents_min_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_extents_min_get", _wrap_Dwg_Entity_POINTCLOUD_extents_min_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_extents_max_set", _wrap_Dwg_Entity_POINTCLOUD_extents_max_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_extents_max_get", _wrap_Dwg_Entity_POINTCLOUD_extents_max_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_numpoints_set", _wrap_Dwg_Entity_POINTCLOUD_numpoints_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_numpoints_get", _wrap_Dwg_Entity_POINTCLOUD_numpoints_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_ucs_name_set", _wrap_Dwg_Entity_POINTCLOUD_ucs_name_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_ucs_name_get", _wrap_Dwg_Entity_POINTCLOUD_ucs_name_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_ucs_origin_set", _wrap_Dwg_Entity_POINTCLOUD_ucs_origin_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_ucs_origin_get", _wrap_Dwg_Entity_POINTCLOUD_ucs_origin_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_ucs_x_dir_set", _wrap_Dwg_Entity_POINTCLOUD_ucs_x_dir_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_ucs_x_dir_get", _wrap_Dwg_Entity_POINTCLOUD_ucs_x_dir_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_ucs_y_dir_set", _wrap_Dwg_Entity_POINTCLOUD_ucs_y_dir_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_ucs_y_dir_get", _wrap_Dwg_Entity_POINTCLOUD_ucs_y_dir_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_ucs_z_dir_set", _wrap_Dwg_Entity_POINTCLOUD_ucs_z_dir_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_ucs_z_dir_get", _wrap_Dwg_Entity_POINTCLOUD_ucs_z_dir_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_pointclouddef_set", _wrap_Dwg_Entity_POINTCLOUD_pointclouddef_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_pointclouddef_get", _wrap_Dwg_Entity_POINTCLOUD_pointclouddef_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_reactor_set", _wrap_Dwg_Entity_POINTCLOUD_reactor_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_reactor_get", _wrap_Dwg_Entity_POINTCLOUD_reactor_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_show_intensity_set", _wrap_Dwg_Entity_POINTCLOUD_show_intensity_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_show_intensity_get", _wrap_Dwg_Entity_POINTCLOUD_show_intensity_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_intensity_scheme_set", _wrap_Dwg_Entity_POINTCLOUD_intensity_scheme_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_intensity_scheme_get", _wrap_Dwg_Entity_POINTCLOUD_intensity_scheme_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_intensity_style_set", _wrap_Dwg_Entity_POINTCLOUD_intensity_style_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_intensity_style_get", _wrap_Dwg_Entity_POINTCLOUD_intensity_style_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_show_clipping_set", _wrap_Dwg_Entity_POINTCLOUD_show_clipping_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_show_clipping_get", _wrap_Dwg_Entity_POINTCLOUD_show_clipping_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_num_clippings_set", _wrap_Dwg_Entity_POINTCLOUD_num_clippings_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_num_clippings_get", _wrap_Dwg_Entity_POINTCLOUD_num_clippings_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_clippings_set", _wrap_Dwg_Entity_POINTCLOUD_clippings_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUD_clippings_get", _wrap_Dwg_Entity_POINTCLOUD_clippings_get},
{"LibreDWGc::new_Dwg_Entity_POINTCLOUD", _wrap_new_Dwg_Entity_POINTCLOUD},
{"LibreDWGc::delete_Dwg_Entity_POINTCLOUD", _wrap_delete_Dwg_Entity_POINTCLOUD},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_parent_set", _wrap_Dwg_Entity_POINTCLOUDEX_parent_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_parent_get", _wrap_Dwg_Entity_POINTCLOUDEX_parent_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_class_version_set", _wrap_Dwg_Entity_POINTCLOUDEX_class_version_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_class_version_get", _wrap_Dwg_Entity_POINTCLOUDEX_class_version_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_extents_min_set", _wrap_Dwg_Entity_POINTCLOUDEX_extents_min_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_extents_min_get", _wrap_Dwg_Entity_POINTCLOUDEX_extents_min_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_extents_max_set", _wrap_Dwg_Entity_POINTCLOUDEX_extents_max_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_extents_max_get", _wrap_Dwg_Entity_POINTCLOUDEX_extents_max_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_ucs_origin_set", _wrap_Dwg_Entity_POINTCLOUDEX_ucs_origin_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_ucs_origin_get", _wrap_Dwg_Entity_POINTCLOUDEX_ucs_origin_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_ucs_x_dir_set", _wrap_Dwg_Entity_POINTCLOUDEX_ucs_x_dir_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_ucs_x_dir_get", _wrap_Dwg_Entity_POINTCLOUDEX_ucs_x_dir_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_ucs_y_dir_set", _wrap_Dwg_Entity_POINTCLOUDEX_ucs_y_dir_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_ucs_y_dir_get", _wrap_Dwg_Entity_POINTCLOUDEX_ucs_y_dir_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_ucs_z_dir_set", _wrap_Dwg_Entity_POINTCLOUDEX_ucs_z_dir_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_ucs_z_dir_get", _wrap_Dwg_Entity_POINTCLOUDEX_ucs_z_dir_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_is_locked_set", _wrap_Dwg_Entity_POINTCLOUDEX_is_locked_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_is_locked_get", _wrap_Dwg_Entity_POINTCLOUDEX_is_locked_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_pointclouddefex_set", _wrap_Dwg_Entity_POINTCLOUDEX_pointclouddefex_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_pointclouddefex_get", _wrap_Dwg_Entity_POINTCLOUDEX_pointclouddefex_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_reactor_set", _wrap_Dwg_Entity_POINTCLOUDEX_reactor_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_reactor_get", _wrap_Dwg_Entity_POINTCLOUDEX_reactor_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_name_set", _wrap_Dwg_Entity_POINTCLOUDEX_name_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_name_get", _wrap_Dwg_Entity_POINTCLOUDEX_name_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_show_intensity_set", _wrap_Dwg_Entity_POINTCLOUDEX_show_intensity_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_show_intensity_get", _wrap_Dwg_Entity_POINTCLOUDEX_show_intensity_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_stylization_type_set", _wrap_Dwg_Entity_POINTCLOUDEX_stylization_type_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_stylization_type_get", _wrap_Dwg_Entity_POINTCLOUDEX_stylization_type_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_intensity_colorscheme_set", _wrap_Dwg_Entity_POINTCLOUDEX_intensity_colorscheme_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_intensity_colorscheme_get", _wrap_Dwg_Entity_POINTCLOUDEX_intensity_colorscheme_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_cur_colorscheme_set", _wrap_Dwg_Entity_POINTCLOUDEX_cur_colorscheme_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_cur_colorscheme_get", _wrap_Dwg_Entity_POINTCLOUDEX_cur_colorscheme_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_classification_colorscheme_set", _wrap_Dwg_Entity_POINTCLOUDEX_classification_colorscheme_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_classification_colorscheme_get", _wrap_Dwg_Entity_POINTCLOUDEX_classification_colorscheme_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_elevation_min_set", _wrap_Dwg_Entity_POINTCLOUDEX_elevation_min_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_elevation_min_get", _wrap_Dwg_Entity_POINTCLOUDEX_elevation_min_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_elevation_max_set", _wrap_Dwg_Entity_POINTCLOUDEX_elevation_max_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_elevation_max_get", _wrap_Dwg_Entity_POINTCLOUDEX_elevation_max_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_intensity_min_set", _wrap_Dwg_Entity_POINTCLOUDEX_intensity_min_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_intensity_min_get", _wrap_Dwg_Entity_POINTCLOUDEX_intensity_min_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_intensity_max_set", _wrap_Dwg_Entity_POINTCLOUDEX_intensity_max_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_intensity_max_get", _wrap_Dwg_Entity_POINTCLOUDEX_intensity_max_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_intensity_out_of_range_behavior_set", _wrap_Dwg_Entity_POINTCLOUDEX_intensity_out_of_range_behavior_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_intensity_out_of_range_behavior_get", _wrap_Dwg_Entity_POINTCLOUDEX_intensity_out_of_range_behavior_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_elevation_out_of_range_behavior_set", _wrap_Dwg_Entity_POINTCLOUDEX_elevation_out_of_range_behavior_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_elevation_out_of_range_behavior_get", _wrap_Dwg_Entity_POINTCLOUDEX_elevation_out_of_range_behavior_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_elevation_apply_to_fixed_range_set", _wrap_Dwg_Entity_POINTCLOUDEX_elevation_apply_to_fixed_range_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_elevation_apply_to_fixed_range_get", _wrap_Dwg_Entity_POINTCLOUDEX_elevation_apply_to_fixed_range_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_intensity_as_gradient_set", _wrap_Dwg_Entity_POINTCLOUDEX_intensity_as_gradient_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_intensity_as_gradient_get", _wrap_Dwg_Entity_POINTCLOUDEX_intensity_as_gradient_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_elevation_as_gradient_set", _wrap_Dwg_Entity_POINTCLOUDEX_elevation_as_gradient_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_elevation_as_gradient_get", _wrap_Dwg_Entity_POINTCLOUDEX_elevation_as_gradient_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_show_cropping_set", _wrap_Dwg_Entity_POINTCLOUDEX_show_cropping_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_show_cropping_get", _wrap_Dwg_Entity_POINTCLOUDEX_show_cropping_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_unknown_bl0_set", _wrap_Dwg_Entity_POINTCLOUDEX_unknown_bl0_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_unknown_bl0_get", _wrap_Dwg_Entity_POINTCLOUDEX_unknown_bl0_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_unknown_bl1_set", _wrap_Dwg_Entity_POINTCLOUDEX_unknown_bl1_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_unknown_bl1_get", _wrap_Dwg_Entity_POINTCLOUDEX_unknown_bl1_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_num_croppings_set", _wrap_Dwg_Entity_POINTCLOUDEX_num_croppings_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_num_croppings_get", _wrap_Dwg_Entity_POINTCLOUDEX_num_croppings_get},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_croppings_set", _wrap_Dwg_Entity_POINTCLOUDEX_croppings_set},
{"LibreDWGc::Dwg_Entity_POINTCLOUDEX_croppings_get", _wrap_Dwg_Entity_POINTCLOUDEX_croppings_get},
{"LibreDWGc::new_Dwg_Entity_POINTCLOUDEX", _wrap_new_Dwg_Entity_POINTCLOUDEX},
{"LibreDWGc::delete_Dwg_Entity_POINTCLOUDEX", _wrap_delete_Dwg_Entity_POINTCLOUDEX},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEF_parent_set", _wrap_Dwg_Object_POINTCLOUDDEF_parent_set},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEF_parent_get", _wrap_Dwg_Object_POINTCLOUDDEF_parent_get},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEF_class_version_set", _wrap_Dwg_Object_POINTCLOUDDEF_class_version_set},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEF_class_version_get", _wrap_Dwg_Object_POINTCLOUDDEF_class_version_get},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEF_source_filename_set", _wrap_Dwg_Object_POINTCLOUDDEF_source_filename_set},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEF_source_filename_get", _wrap_Dwg_Object_POINTCLOUDDEF_source_filename_get},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEF_is_loaded_set", _wrap_Dwg_Object_POINTCLOUDDEF_is_loaded_set},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEF_is_loaded_get", _wrap_Dwg_Object_POINTCLOUDDEF_is_loaded_get},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEF_numpoints_set", _wrap_Dwg_Object_POINTCLOUDDEF_numpoints_set},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEF_numpoints_get", _wrap_Dwg_Object_POINTCLOUDDEF_numpoints_get},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEF_extents_min_set", _wrap_Dwg_Object_POINTCLOUDDEF_extents_min_set},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEF_extents_min_get", _wrap_Dwg_Object_POINTCLOUDDEF_extents_min_get},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEF_extents_max_set", _wrap_Dwg_Object_POINTCLOUDDEF_extents_max_set},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEF_extents_max_get", _wrap_Dwg_Object_POINTCLOUDDEF_extents_max_get},
{"LibreDWGc::new_Dwg_Object_POINTCLOUDDEF", _wrap_new_Dwg_Object_POINTCLOUDDEF},
{"LibreDWGc::delete_Dwg_Object_POINTCLOUDDEF", _wrap_delete_Dwg_Object_POINTCLOUDDEF},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEFEX_parent_set", _wrap_Dwg_Object_POINTCLOUDDEFEX_parent_set},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEFEX_parent_get", _wrap_Dwg_Object_POINTCLOUDDEFEX_parent_get},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEFEX_class_version_set", _wrap_Dwg_Object_POINTCLOUDDEFEX_class_version_set},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEFEX_class_version_get", _wrap_Dwg_Object_POINTCLOUDDEFEX_class_version_get},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEFEX_source_filename_set", _wrap_Dwg_Object_POINTCLOUDDEFEX_source_filename_set},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEFEX_source_filename_get", _wrap_Dwg_Object_POINTCLOUDDEFEX_source_filename_get},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEFEX_is_loaded_set", _wrap_Dwg_Object_POINTCLOUDDEFEX_is_loaded_set},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEFEX_is_loaded_get", _wrap_Dwg_Object_POINTCLOUDDEFEX_is_loaded_get},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEFEX_numpoints_set", _wrap_Dwg_Object_POINTCLOUDDEFEX_numpoints_set},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEFEX_numpoints_get", _wrap_Dwg_Object_POINTCLOUDDEFEX_numpoints_get},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEFEX_extents_min_set", _wrap_Dwg_Object_POINTCLOUDDEFEX_extents_min_set},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEFEX_extents_min_get", _wrap_Dwg_Object_POINTCLOUDDEFEX_extents_min_get},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEFEX_extents_max_set", _wrap_Dwg_Object_POINTCLOUDDEFEX_extents_max_set},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEFEX_extents_max_get", _wrap_Dwg_Object_POINTCLOUDDEFEX_extents_max_get},
{"LibreDWGc::new_Dwg_Object_POINTCLOUDDEFEX", _wrap_new_Dwg_Object_POINTCLOUDDEFEX},
{"LibreDWGc::delete_Dwg_Object_POINTCLOUDDEFEX", _wrap_delete_Dwg_Object_POINTCLOUDDEFEX},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEF_REACTOR_parent_set", _wrap_Dwg_Object_POINTCLOUDDEF_REACTOR_parent_set},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEF_REACTOR_parent_get", _wrap_Dwg_Object_POINTCLOUDDEF_REACTOR_parent_get},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEF_REACTOR_class_version_set", _wrap_Dwg_Object_POINTCLOUDDEF_REACTOR_class_version_set},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEF_REACTOR_class_version_get", _wrap_Dwg_Object_POINTCLOUDDEF_REACTOR_class_version_get},
{"LibreDWGc::new_Dwg_Object_POINTCLOUDDEF_REACTOR", _wrap_new_Dwg_Object_POINTCLOUDDEF_REACTOR},
{"LibreDWGc::delete_Dwg_Object_POINTCLOUDDEF_REACTOR", _wrap_delete_Dwg_Object_POINTCLOUDDEF_REACTOR},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEF_REACTOR_EX_parent_set", _wrap_Dwg_Object_POINTCLOUDDEF_REACTOR_EX_parent_set},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEF_REACTOR_EX_parent_get", _wrap_Dwg_Object_POINTCLOUDDEF_REACTOR_EX_parent_get},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEF_REACTOR_EX_class_version_set", _wrap_Dwg_Object_POINTCLOUDDEF_REACTOR_EX_class_version_set},
{"LibreDWGc::Dwg_Object_POINTCLOUDDEF_REACTOR_EX_class_version_get", _wrap_Dwg_Object_POINTCLOUDDEF_REACTOR_EX_class_version_get},
{"LibreDWGc::new_Dwg_Object_POINTCLOUDDEF_REACTOR_EX", _wrap_new_Dwg_Object_POINTCLOUDDEF_REACTOR_EX},
{"LibreDWGc::delete_Dwg_Object_POINTCLOUDDEF_REACTOR_EX", _wrap_delete_Dwg_Object_POINTCLOUDDEF_REACTOR_EX},
{"LibreDWGc::Dwg_ColorRamp_parent_set", _wrap_Dwg_ColorRamp_parent_set},
{"LibreDWGc::Dwg_ColorRamp_parent_get", _wrap_Dwg_ColorRamp_parent_get},
{"LibreDWGc::Dwg_ColorRamp_colorscheme_set", _wrap_Dwg_ColorRamp_colorscheme_set},
{"LibreDWGc::Dwg_ColorRamp_colorscheme_get", _wrap_Dwg_ColorRamp_colorscheme_get},
{"LibreDWGc::Dwg_ColorRamp_unknown_bl_set", _wrap_Dwg_ColorRamp_unknown_bl_set},
{"LibreDWGc::Dwg_ColorRamp_unknown_bl_get", _wrap_Dwg_ColorRamp_unknown_bl_get},
{"LibreDWGc::Dwg_ColorRamp_unknown_b_set", _wrap_Dwg_ColorRamp_unknown_b_set},
{"LibreDWGc::Dwg_ColorRamp_unknown_b_get", _wrap_Dwg_ColorRamp_unknown_b_get},
{"LibreDWGc::new_Dwg_ColorRamp", _wrap_new_Dwg_ColorRamp},
{"LibreDWGc::delete_Dwg_ColorRamp", _wrap_delete_Dwg_ColorRamp},
{"LibreDWGc::Dwg_POINTCLOUDCOLORMAP_Ramp_parent_set", _wrap_Dwg_POINTCLOUDCOLORMAP_Ramp_parent_set},
{"LibreDWGc::Dwg_POINTCLOUDCOLORMAP_Ramp_parent_get", _wrap_Dwg_POINTCLOUDCOLORMAP_Ramp_parent_get},
{"LibreDWGc::Dwg_POINTCLOUDCOLORMAP_Ramp_class_version_set", _wrap_Dwg_POINTCLOUDCOLORMAP_Ramp_class_version_set},
{"LibreDWGc::Dwg_POINTCLOUDCOLORMAP_Ramp_class_version_get", _wrap_Dwg_POINTCLOUDCOLORMAP_Ramp_class_version_get},
{"LibreDWGc::Dwg_POINTCLOUDCOLORMAP_Ramp_num_ramps_set", _wrap_Dwg_POINTCLOUDCOLORMAP_Ramp_num_ramps_set},
{"LibreDWGc::Dwg_POINTCLOUDCOLORMAP_Ramp_num_ramps_get", _wrap_Dwg_POINTCLOUDCOLORMAP_Ramp_num_ramps_get},
{"LibreDWGc::Dwg_POINTCLOUDCOLORMAP_Ramp_ramps_set", _wrap_Dwg_POINTCLOUDCOLORMAP_Ramp_ramps_set},
{"LibreDWGc::Dwg_POINTCLOUDCOLORMAP_Ramp_ramps_get", _wrap_Dwg_POINTCLOUDCOLORMAP_Ramp_ramps_get},
{"LibreDWGc::new_Dwg_POINTCLOUDCOLORMAP_Ramp", _wrap_new_Dwg_POINTCLOUDCOLORMAP_Ramp},
{"LibreDWGc::delete_Dwg_POINTCLOUDCOLORMAP_Ramp", _wrap_delete_Dwg_POINTCLOUDCOLORMAP_Ramp},
{"LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_parent_set", _wrap_Dwg_Object_POINTCLOUDCOLORMAP_parent_set},
{"LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_parent_get", _wrap_Dwg_Object_POINTCLOUDCOLORMAP_parent_get},
{"LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_class_version_set", _wrap_Dwg_Object_POINTCLOUDCOLORMAP_class_version_set},
{"LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_class_version_get", _wrap_Dwg_Object_POINTCLOUDCOLORMAP_class_version_get},
{"LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_def_intensity_colorscheme_set", _wrap_Dwg_Object_POINTCLOUDCOLORMAP_def_intensity_colorscheme_set},
{"LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_def_intensity_colorscheme_get", _wrap_Dwg_Object_POINTCLOUDCOLORMAP_def_intensity_colorscheme_get},
{"LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_def_elevation_colorscheme_set", _wrap_Dwg_Object_POINTCLOUDCOLORMAP_def_elevation_colorscheme_set},
{"LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_def_elevation_colorscheme_get", _wrap_Dwg_Object_POINTCLOUDCOLORMAP_def_elevation_colorscheme_get},
{"LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_def_classification_colorscheme_set", _wrap_Dwg_Object_POINTCLOUDCOLORMAP_def_classification_colorscheme_set},
{"LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_def_classification_colorscheme_get", _wrap_Dwg_Object_POINTCLOUDCOLORMAP_def_classification_colorscheme_get},
{"LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_num_colorramps_set", _wrap_Dwg_Object_POINTCLOUDCOLORMAP_num_colorramps_set},
{"LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_num_colorramps_get", _wrap_Dwg_Object_POINTCLOUDCOLORMAP_num_colorramps_get},
{"LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_colorramps_set", _wrap_Dwg_Object_POINTCLOUDCOLORMAP_colorramps_set},
{"LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_colorramps_get", _wrap_Dwg_Object_POINTCLOUDCOLORMAP_colorramps_get},
{"LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_num_classification_colorramps_set", _wrap_Dwg_Object_POINTCLOUDCOLORMAP_num_classification_colorramps_set},
{"LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_num_classification_colorramps_get", _wrap_Dwg_Object_POINTCLOUDCOLORMAP_num_classification_colorramps_get},
{"LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_classification_colorramps_set", _wrap_Dwg_Object_POINTCLOUDCOLORMAP_classification_colorramps_set},
{"LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_classification_colorramps_get", _wrap_Dwg_Object_POINTCLOUDCOLORMAP_classification_colorramps_get},
{"LibreDWGc::new_Dwg_Object_POINTCLOUDCOLORMAP", _wrap_new_Dwg_Object_POINTCLOUDCOLORMAP},
{"LibreDWGc::delete_Dwg_Object_POINTCLOUDCOLORMAP", _wrap_delete_Dwg_Object_POINTCLOUDCOLORMAP},
{"LibreDWGc::Dwg_COMPOUNDOBJECTID_parent_set", _wrap_Dwg_COMPOUNDOBJECTID_parent_set},
{"LibreDWGc::Dwg_COMPOUNDOBJECTID_parent_get", _wrap_Dwg_COMPOUNDOBJECTID_parent_get},
{"LibreDWGc::Dwg_COMPOUNDOBJECTID_has_object_set", _wrap_Dwg_COMPOUNDOBJECTID_has_object_set},
{"LibreDWGc::Dwg_COMPOUNDOBJECTID_has_object_get", _wrap_Dwg_COMPOUNDOBJECTID_has_object_get},
{"LibreDWGc::Dwg_COMPOUNDOBJECTID_name_set", _wrap_Dwg_COMPOUNDOBJECTID_name_set},
{"LibreDWGc::Dwg_COMPOUNDOBJECTID_name_get", _wrap_Dwg_COMPOUNDOBJECTID_name_get},
{"LibreDWGc::Dwg_COMPOUNDOBJECTID_object_set", _wrap_Dwg_COMPOUNDOBJECTID_object_set},
{"LibreDWGc::Dwg_COMPOUNDOBJECTID_object_get", _wrap_Dwg_COMPOUNDOBJECTID_object_get},
{"LibreDWGc::new_Dwg_COMPOUNDOBJECTID", _wrap_new_Dwg_COMPOUNDOBJECTID},
{"LibreDWGc::delete_Dwg_COMPOUNDOBJECTID", _wrap_delete_Dwg_COMPOUNDOBJECTID},
{"LibreDWGc::Dwg_PARTIAL_VIEWING_INDEX_Entry_parent_set", _wrap_Dwg_PARTIAL_VIEWING_INDEX_Entry_parent_set},
{"LibreDWGc::Dwg_PARTIAL_VIEWING_INDEX_Entry_parent_get", _wrap_Dwg_PARTIAL_VIEWING_INDEX_Entry_parent_get},
{"LibreDWGc::Dwg_PARTIAL_VIEWING_INDEX_Entry_extents_min_set", _wrap_Dwg_PARTIAL_VIEWING_INDEX_Entry_extents_min_set},
{"LibreDWGc::Dwg_PARTIAL_VIEWING_INDEX_Entry_extents_min_get", _wrap_Dwg_PARTIAL_VIEWING_INDEX_Entry_extents_min_get},
{"LibreDWGc::Dwg_PARTIAL_VIEWING_INDEX_Entry_extents_max_set", _wrap_Dwg_PARTIAL_VIEWING_INDEX_Entry_extents_max_set},
{"LibreDWGc::Dwg_PARTIAL_VIEWING_INDEX_Entry_extents_max_get", _wrap_Dwg_PARTIAL_VIEWING_INDEX_Entry_extents_max_get},
{"LibreDWGc::Dwg_PARTIAL_VIEWING_INDEX_Entry_object_set", _wrap_Dwg_PARTIAL_VIEWING_INDEX_Entry_object_set},
{"LibreDWGc::Dwg_PARTIAL_VIEWING_INDEX_Entry_object_get", _wrap_Dwg_PARTIAL_VIEWING_INDEX_Entry_object_get},
{"LibreDWGc::new_Dwg_PARTIAL_VIEWING_INDEX_Entry", _wrap_new_Dwg_PARTIAL_VIEWING_INDEX_Entry},
{"LibreDWGc::delete_Dwg_PARTIAL_VIEWING_INDEX_Entry", _wrap_delete_Dwg_PARTIAL_VIEWING_INDEX_Entry},
{"LibreDWGc::Dwg_Object_PARTIAL_VIEWING_INDEX_parent_set", _wrap_Dwg_Object_PARTIAL_VIEWING_INDEX_parent_set},
{"LibreDWGc::Dwg_Object_PARTIAL_VIEWING_INDEX_parent_get", _wrap_Dwg_Object_PARTIAL_VIEWING_INDEX_parent_get},
{"LibreDWGc::Dwg_Object_PARTIAL_VIEWING_INDEX_num_entries_set", _wrap_Dwg_Object_PARTIAL_VIEWING_INDEX_num_entries_set},
{"LibreDWGc::Dwg_Object_PARTIAL_VIEWING_INDEX_num_entries_get", _wrap_Dwg_Object_PARTIAL_VIEWING_INDEX_num_entries_get},
{"LibreDWGc::Dwg_Object_PARTIAL_VIEWING_INDEX_has_entries_set", _wrap_Dwg_Object_PARTIAL_VIEWING_INDEX_has_entries_set},
{"LibreDWGc::Dwg_Object_PARTIAL_VIEWING_INDEX_has_entries_get", _wrap_Dwg_Object_PARTIAL_VIEWING_INDEX_has_entries_get},
{"LibreDWGc::Dwg_Object_PARTIAL_VIEWING_INDEX_entries_set", _wrap_Dwg_Object_PARTIAL_VIEWING_INDEX_entries_set},
{"LibreDWGc::Dwg_Object_PARTIAL_VIEWING_INDEX_entries_get", _wrap_Dwg_Object_PARTIAL_VIEWING_INDEX_entries_get},
{"LibreDWGc::new_Dwg_Object_PARTIAL_VIEWING_INDEX", _wrap_new_Dwg_Object_PARTIAL_VIEWING_INDEX},
{"LibreDWGc::delete_Dwg_Object_PARTIAL_VIEWING_INDEX", _wrap_delete_Dwg_Object_PARTIAL_VIEWING_INDEX},
{"LibreDWGc::Dwg_Entity_UNKNOWN_ENT_parent_set", _wrap_Dwg_Entity_UNKNOWN_ENT_parent_set},
{"LibreDWGc::Dwg_Entity_UNKNOWN_ENT_parent_get", _wrap_Dwg_Entity_UNKNOWN_ENT_parent_get},
{"LibreDWGc::new_Dwg_Entity_UNKNOWN_ENT", _wrap_new_Dwg_Entity_UNKNOWN_ENT},
{"LibreDWGc::delete_Dwg_Entity_UNKNOWN_ENT", _wrap_delete_Dwg_Entity_UNKNOWN_ENT},
{"LibreDWGc::Dwg_Object_UNKNOWN_OBJ_parent_set", _wrap_Dwg_Object_UNKNOWN_OBJ_parent_set},
{"LibreDWGc::Dwg_Object_UNKNOWN_OBJ_parent_get", _wrap_Dwg_Object_UNKNOWN_OBJ_parent_get},
{"LibreDWGc::new_Dwg_Object_UNKNOWN_OBJ", _wrap_new_Dwg_Object_UNKNOWN_OBJ},
{"LibreDWGc::delete_Dwg_Object_UNKNOWN_OBJ", _wrap_delete_Dwg_Object_UNKNOWN_OBJ},
{"LibreDWGc::Dwg_Entity_REPEAT_parent_set", _wrap_Dwg_Entity_REPEAT_parent_set},
{"LibreDWGc::Dwg_Entity_REPEAT_parent_get", _wrap_Dwg_Entity_REPEAT_parent_get},
{"LibreDWGc::new_Dwg_Entity_REPEAT", _wrap_new_Dwg_Entity_REPEAT},
{"LibreDWGc::delete_Dwg_Entity_REPEAT", _wrap_delete_Dwg_Entity_REPEAT},
{"LibreDWGc::Dwg_Entity_ENDREP_parent_set", _wrap_Dwg_Entity_ENDREP_parent_set},
{"LibreDWGc::Dwg_Entity_ENDREP_parent_get", _wrap_Dwg_Entity_ENDREP_parent_get},
{"LibreDWGc::Dwg_Entity_ENDREP_numcols_set", _wrap_Dwg_Entity_ENDREP_numcols_set},
{"LibreDWGc::Dwg_Entity_ENDREP_numcols_get", _wrap_Dwg_Entity_ENDREP_numcols_get},
{"LibreDWGc::Dwg_Entity_ENDREP_numrows_set", _wrap_Dwg_Entity_ENDREP_numrows_set},
{"LibreDWGc::Dwg_Entity_ENDREP_numrows_get", _wrap_Dwg_Entity_ENDREP_numrows_get},
{"LibreDWGc::Dwg_Entity_ENDREP_colspacing_set", _wrap_Dwg_Entity_ENDREP_colspacing_set},
{"LibreDWGc::Dwg_Entity_ENDREP_colspacing_get", _wrap_Dwg_Entity_ENDREP_colspacing_get},
{"LibreDWGc::Dwg_Entity_ENDREP_rowspacing_set", _wrap_Dwg_Entity_ENDREP_rowspacing_set},
{"LibreDWGc::Dwg_Entity_ENDREP_rowspacing_get", _wrap_Dwg_Entity_ENDREP_rowspacing_get},
{"LibreDWGc::new_Dwg_Entity_ENDREP", _wrap_new_Dwg_Entity_ENDREP},
{"LibreDWGc::delete_Dwg_Entity_ENDREP", _wrap_delete_Dwg_Entity_ENDREP},
{"LibreDWGc::Dwg_Entity_LOAD_parent_set", _wrap_Dwg_Entity_LOAD_parent_set},
{"LibreDWGc::Dwg_Entity_LOAD_parent_get", _wrap_Dwg_Entity_LOAD_parent_get},
{"LibreDWGc::Dwg_Entity_LOAD_file_name_set", _wrap_Dwg_Entity_LOAD_file_name_set},
{"LibreDWGc::Dwg_Entity_LOAD_file_name_get", _wrap_Dwg_Entity_LOAD_file_name_get},
{"LibreDWGc::new_Dwg_Entity_LOAD", _wrap_new_Dwg_Entity_LOAD},
{"LibreDWGc::delete_Dwg_Entity_LOAD", _wrap_delete_Dwg_Entity_LOAD},
{"LibreDWGc::Dwg_Entity_JUMP_parent_set", _wrap_Dwg_Entity_JUMP_parent_set},
{"LibreDWGc::Dwg_Entity_JUMP_parent_get", _wrap_Dwg_Entity_JUMP_parent_get},
{"LibreDWGc::Dwg_Entity_JUMP_jump_address_raw_set", _wrap_Dwg_Entity_JUMP_jump_address_raw_set},
{"LibreDWGc::Dwg_Entity_JUMP_jump_address_raw_get", _wrap_Dwg_Entity_JUMP_jump_address_raw_get},
{"LibreDWGc::Dwg_Entity_JUMP_jump_address_set", _wrap_Dwg_Entity_JUMP_jump_address_set},
{"LibreDWGc::Dwg_Entity_JUMP_jump_address_get", _wrap_Dwg_Entity_JUMP_jump_address_get},
{"LibreDWGc::Dwg_Entity_JUMP_jump_entity_section_set", _wrap_Dwg_Entity_JUMP_jump_entity_section_set},
{"LibreDWGc::Dwg_Entity_JUMP_jump_entity_section_get", _wrap_Dwg_Entity_JUMP_jump_entity_section_get},
{"LibreDWGc::new_Dwg_Entity_JUMP", _wrap_new_Dwg_Entity_JUMP},
{"LibreDWGc::delete_Dwg_Entity_JUMP", _wrap_delete_Dwg_Entity_JUMP},
{"LibreDWGc::Dwg_Entity__3DLINE_parent_set", _wrap_Dwg_Entity__3DLINE_parent_set},
{"LibreDWGc::Dwg_Entity__3DLINE_parent_get", _wrap_Dwg_Entity__3DLINE_parent_get},
{"LibreDWGc::Dwg_Entity__3DLINE_start_set", _wrap_Dwg_Entity__3DLINE_start_set},
{"LibreDWGc::Dwg_Entity__3DLINE_start_get", _wrap_Dwg_Entity__3DLINE_start_get},
{"LibreDWGc::Dwg_Entity__3DLINE_end_set", _wrap_Dwg_Entity__3DLINE_end_set},
{"LibreDWGc::Dwg_Entity__3DLINE_end_get", _wrap_Dwg_Entity__3DLINE_end_get},
{"LibreDWGc::Dwg_Entity__3DLINE_extrusion_set", _wrap_Dwg_Entity__3DLINE_extrusion_set},
{"LibreDWGc::Dwg_Entity__3DLINE_extrusion_get", _wrap_Dwg_Entity__3DLINE_extrusion_get},
{"LibreDWGc::Dwg_Entity__3DLINE_thickness_set", _wrap_Dwg_Entity__3DLINE_thickness_set},
{"LibreDWGc::Dwg_Entity__3DLINE_thickness_get", _wrap_Dwg_Entity__3DLINE_thickness_get},
{"LibreDWGc::new_Dwg_Entity__3DLINE", _wrap_new_Dwg_Entity__3DLINE},
{"LibreDWGc::delete_Dwg_Entity__3DLINE", _wrap_delete_Dwg_Entity__3DLINE},
{"LibreDWGc::Dwg_Eed_Data_u_set", _wrap_Dwg_Eed_Data_u_set},
{"LibreDWGc::Dwg_Eed_Data_u_get", _wrap_Dwg_Eed_Data_u_get},
{"LibreDWGc::Dwg_Eed_Data_code_set", _wrap_Dwg_Eed_Data_code_set},
{"LibreDWGc::Dwg_Eed_Data_code_get", _wrap_Dwg_Eed_Data_code_get},
{"LibreDWGc::new_Dwg_Eed_Data", _wrap_new_Dwg_Eed_Data},
{"LibreDWGc::delete_Dwg_Eed_Data", _wrap_delete_Dwg_Eed_Data},
{"LibreDWGc::eed_data_t_eed_0_get", _wrap_eed_data_t_eed_0_get},
{"LibreDWGc::eed_data_t_eed_0_r2007_get", _wrap_eed_data_t_eed_0_r2007_get},
{"LibreDWGc::eed_data_t_eed_1_get", _wrap_eed_data_t_eed_1_get},
{"LibreDWGc::eed_data_t_eed_2_get", _wrap_eed_data_t_eed_2_get},
{"LibreDWGc::eed_data_t_eed_3_get", _wrap_eed_data_t_eed_3_get},
{"LibreDWGc::eed_data_t_eed_4_get", _wrap_eed_data_t_eed_4_get},
{"LibreDWGc::eed_data_t_eed_5_get", _wrap_eed_data_t_eed_5_get},
{"LibreDWGc::eed_data_t_eed_10_get", _wrap_eed_data_t_eed_10_get},
{"LibreDWGc::eed_data_t_eed_40_get", _wrap_eed_data_t_eed_40_get},
{"LibreDWGc::eed_data_t_eed_70_get", _wrap_eed_data_t_eed_70_get},
{"LibreDWGc::eed_data_t_eed_71_get", _wrap_eed_data_t_eed_71_get},
{"LibreDWGc::new_eed_data_t", _wrap_new_eed_data_t},
{"LibreDWGc::delete_eed_data_t", _wrap_delete_eed_data_t},
{"LibreDWGc::eed_data_t_eed_71_rl_set", _wrap_eed_data_t_eed_71_rl_set},
{"LibreDWGc::eed_data_t_eed_71_rl_get", _wrap_eed_data_t_eed_71_rl_get},
{"LibreDWGc::new_eed_data_t_eed_71", _wrap_new_eed_data_t_eed_71},
{"LibreDWGc::delete_eed_data_t_eed_71", _wrap_delete_eed_data_t_eed_71},
{"LibreDWGc::eed_data_t_eed_70_rs_set", _wrap_eed_data_t_eed_70_rs_set},
{"LibreDWGc::eed_data_t_eed_70_rs_get", _wrap_eed_data_t_eed_70_rs_get},
{"LibreDWGc::new_eed_data_t_eed_70", _wrap_new_eed_data_t_eed_70},
{"LibreDWGc::delete_eed_data_t_eed_70", _wrap_delete_eed_data_t_eed_70},
{"LibreDWGc::eed_data_t_eed_40_real_set", _wrap_eed_data_t_eed_40_real_set},
{"LibreDWGc::eed_data_t_eed_40_real_get", _wrap_eed_data_t_eed_40_real_get},
{"LibreDWGc::new_eed_data_t_eed_40", _wrap_new_eed_data_t_eed_40},
{"LibreDWGc::delete_eed_data_t_eed_40", _wrap_delete_eed_data_t_eed_40},
{"LibreDWGc::eed_data_t_eed_10_point_set", _wrap_eed_data_t_eed_10_point_set},
{"LibreDWGc::eed_data_t_eed_10_point_get", _wrap_eed_data_t_eed_10_point_get},
{"LibreDWGc::new_eed_data_t_eed_10", _wrap_new_eed_data_t_eed_10},
{"LibreDWGc::delete_eed_data_t_eed_10", _wrap_delete_eed_data_t_eed_10},
{"LibreDWGc::eed_data_t_eed_5_entity_set", _wrap_eed_data_t_eed_5_entity_set},
{"LibreDWGc::eed_data_t_eed_5_entity_get", _wrap_eed_data_t_eed_5_entity_get},
{"LibreDWGc::new_eed_data_t_eed_5", _wrap_new_eed_data_t_eed_5},
{"LibreDWGc::delete_eed_data_t_eed_5", _wrap_delete_eed_data_t_eed_5},
{"LibreDWGc::eed_data_t_eed_4_length_set", _wrap_eed_data_t_eed_4_length_set},
{"LibreDWGc::eed_data_t_eed_4_length_get", _wrap_eed_data_t_eed_4_length_get},
{"LibreDWGc::eed_data_t_eed_4_data_get", _wrap_eed_data_t_eed_4_data_get},
{"LibreDWGc::new_eed_data_t_eed_4", _wrap_new_eed_data_t_eed_4},
{"LibreDWGc::delete_eed_data_t_eed_4", _wrap_delete_eed_data_t_eed_4},
{"LibreDWGc::eed_data_t_eed_3_layer_set", _wrap_eed_data_t_eed_3_layer_set},
{"LibreDWGc::eed_data_t_eed_3_layer_get", _wrap_eed_data_t_eed_3_layer_get},
{"LibreDWGc::new_eed_data_t_eed_3", _wrap_new_eed_data_t_eed_3},
{"LibreDWGc::delete_eed_data_t_eed_3", _wrap_delete_eed_data_t_eed_3},
{"LibreDWGc::eed_data_t_eed_2_close_set", _wrap_eed_data_t_eed_2_close_set},
{"LibreDWGc::eed_data_t_eed_2_close_get", _wrap_eed_data_t_eed_2_close_get},
{"LibreDWGc::new_eed_data_t_eed_2", _wrap_new_eed_data_t_eed_2},
{"LibreDWGc::delete_eed_data_t_eed_2", _wrap_delete_eed_data_t_eed_2},
{"LibreDWGc::eed_data_t_eed_1_appid_index_set", _wrap_eed_data_t_eed_1_appid_index_set},
{"LibreDWGc::eed_data_t_eed_1_appid_index_get", _wrap_eed_data_t_eed_1_appid_index_get},
{"LibreDWGc::eed_data_t_eed_1_invalid_set", _wrap_eed_data_t_eed_1_invalid_set},
{"LibreDWGc::eed_data_t_eed_1_invalid_get", _wrap_eed_data_t_eed_1_invalid_get},
{"LibreDWGc::new_eed_data_t_eed_1", _wrap_new_eed_data_t_eed_1},
{"LibreDWGc::delete_eed_data_t_eed_1", _wrap_delete_eed_data_t_eed_1},
{"LibreDWGc::eed_data_t_eed_0_r2007_length_set", _wrap_eed_data_t_eed_0_r2007_length_set},
{"LibreDWGc::eed_data_t_eed_0_r2007_length_get", _wrap_eed_data_t_eed_0_r2007_length_get},
{"LibreDWGc::eed_data_t_eed_0_r2007__padding_set", _wrap_eed_data_t_eed_0_r2007__padding_set},
{"LibreDWGc::eed_data_t_eed_0_r2007__padding_get", _wrap_eed_data_t_eed_0_r2007__padding_get},
{"LibreDWGc::eed_data_t_eed_0_r2007_is_tu_set", _wrap_eed_data_t_eed_0_r2007_is_tu_set},
{"LibreDWGc::eed_data_t_eed_0_r2007_is_tu_get", _wrap_eed_data_t_eed_0_r2007_is_tu_get},
{"LibreDWGc::eed_data_t_eed_0_r2007_string_set", _wrap_eed_data_t_eed_0_r2007_string_set},
{"LibreDWGc::eed_data_t_eed_0_r2007_string_get", _wrap_eed_data_t_eed_0_r2007_string_get},
{"LibreDWGc::new_eed_data_t_eed_0_r2007", _wrap_new_eed_data_t_eed_0_r2007},
{"LibreDWGc::delete_eed_data_t_eed_0_r2007", _wrap_delete_eed_data_t_eed_0_r2007},
{"LibreDWGc::eed_data_t_eed_0_length_set", _wrap_eed_data_t_eed_0_length_set},
{"LibreDWGc::eed_data_t_eed_0_length_get", _wrap_eed_data_t_eed_0_length_get},
{"LibreDWGc::eed_data_t_eed_0_codepage_set", _wrap_eed_data_t_eed_0_codepage_set},
{"LibreDWGc::eed_data_t_eed_0_codepage_get", _wrap_eed_data_t_eed_0_codepage_get},
{"LibreDWGc::eed_data_t_eed_0_is_tu_set", _wrap_eed_data_t_eed_0_is_tu_set},
{"LibreDWGc::eed_data_t_eed_0_is_tu_get", _wrap_eed_data_t_eed_0_is_tu_get},
{"LibreDWGc::eed_data_t_eed_0_string_set", _wrap_eed_data_t_eed_0_string_set},
{"LibreDWGc::eed_data_t_eed_0_string_get", _wrap_eed_data_t_eed_0_string_get},
{"LibreDWGc::new_eed_data_t_eed_0", _wrap_new_eed_data_t_eed_0},
{"LibreDWGc::delete_eed_data_t_eed_0", _wrap_delete_eed_data_t_eed_0},
{"LibreDWGc::Dwg_Eed_size_set", _wrap_Dwg_Eed_size_set},
{"LibreDWGc::Dwg_Eed_size_get", _wrap_Dwg_Eed_size_get},
{"LibreDWGc::Dwg_Eed_handle_set", _wrap_Dwg_Eed_handle_set},
{"LibreDWGc::Dwg_Eed_handle_get", _wrap_Dwg_Eed_handle_get},
{"LibreDWGc::Dwg_Eed_data_set", _wrap_Dwg_Eed_data_set},
{"LibreDWGc::Dwg_Eed_data_get", _wrap_Dwg_Eed_data_get},
{"LibreDWGc::Dwg_Eed_raw_set", _wrap_Dwg_Eed_raw_set},
{"LibreDWGc::Dwg_Eed_raw_get", _wrap_Dwg_Eed_raw_get},
{"LibreDWGc::new_Dwg_Eed", _wrap_new_Dwg_Eed},
{"LibreDWGc::delete_Dwg_Eed", _wrap_delete_Dwg_Eed},
{"LibreDWGc::Dwg_Object_Entity_objid_set", _wrap_Dwg_Object_Entity_objid_set},
{"LibreDWGc::Dwg_Object_Entity_objid_get", _wrap_Dwg_Object_Entity_objid_get},
{"LibreDWGc::Dwg_Object_Entity_tio_get", _wrap_Dwg_Object_Entity_tio_get},
{"LibreDWGc::Dwg_Object_Entity_dwg_set", _wrap_Dwg_Object_Entity_dwg_set},
{"LibreDWGc::Dwg_Object_Entity_dwg_get", _wrap_Dwg_Object_Entity_dwg_get},
{"LibreDWGc::Dwg_Object_Entity_num_eed_set", _wrap_Dwg_Object_Entity_num_eed_set},
{"LibreDWGc::Dwg_Object_Entity_num_eed_get", _wrap_Dwg_Object_Entity_num_eed_get},
{"LibreDWGc::Dwg_Object_Entity_eed_set", _wrap_Dwg_Object_Entity_eed_set},
{"LibreDWGc::Dwg_Object_Entity_eed_get", _wrap_Dwg_Object_Entity_eed_get},
{"LibreDWGc::Dwg_Object_Entity_preview_exists_set", _wrap_Dwg_Object_Entity_preview_exists_set},
{"LibreDWGc::Dwg_Object_Entity_preview_exists_get", _wrap_Dwg_Object_Entity_preview_exists_get},
{"LibreDWGc::Dwg_Object_Entity_preview_is_proxy_set", _wrap_Dwg_Object_Entity_preview_is_proxy_set},
{"LibreDWGc::Dwg_Object_Entity_preview_is_proxy_get", _wrap_Dwg_Object_Entity_preview_is_proxy_get},
{"LibreDWGc::Dwg_Object_Entity_preview_size_set", _wrap_Dwg_Object_Entity_preview_size_set},
{"LibreDWGc::Dwg_Object_Entity_preview_size_get", _wrap_Dwg_Object_Entity_preview_size_get},
{"LibreDWGc::Dwg_Object_Entity_preview_set", _wrap_Dwg_Object_Entity_preview_set},
{"LibreDWGc::Dwg_Object_Entity_preview_get", _wrap_Dwg_Object_Entity_preview_get},
{"LibreDWGc::Dwg_Object_Entity_entmode_set", _wrap_Dwg_Object_Entity_entmode_set},
{"LibreDWGc::Dwg_Object_Entity_entmode_get", _wrap_Dwg_Object_Entity_entmode_get},
{"LibreDWGc::Dwg_Object_Entity_num_reactors_set", _wrap_Dwg_Object_Entity_num_reactors_set},
{"LibreDWGc::Dwg_Object_Entity_num_reactors_get", _wrap_Dwg_Object_Entity_num_reactors_get},
{"LibreDWGc::Dwg_Object_Entity_is_xdic_missing_set", _wrap_Dwg_Object_Entity_is_xdic_missing_set},
{"LibreDWGc::Dwg_Object_Entity_is_xdic_missing_get", _wrap_Dwg_Object_Entity_is_xdic_missing_get},
{"LibreDWGc::Dwg_Object_Entity_isbylayerlt_set", _wrap_Dwg_Object_Entity_isbylayerlt_set},
{"LibreDWGc::Dwg_Object_Entity_isbylayerlt_get", _wrap_Dwg_Object_Entity_isbylayerlt_get},
{"LibreDWGc::Dwg_Object_Entity_nolinks_set", _wrap_Dwg_Object_Entity_nolinks_set},
{"LibreDWGc::Dwg_Object_Entity_nolinks_get", _wrap_Dwg_Object_Entity_nolinks_get},
{"LibreDWGc::Dwg_Object_Entity_has_ds_data_set", _wrap_Dwg_Object_Entity_has_ds_data_set},
{"LibreDWGc::Dwg_Object_Entity_has_ds_data_get", _wrap_Dwg_Object_Entity_has_ds_data_get},
{"LibreDWGc::Dwg_Object_Entity_color_set", _wrap_Dwg_Object_Entity_color_set},
{"LibreDWGc::Dwg_Object_Entity_color_get", _wrap_Dwg_Object_Entity_color_get},
{"LibreDWGc::Dwg_Object_Entity_ltype_scale_set", _wrap_Dwg_Object_Entity_ltype_scale_set},
{"LibreDWGc::Dwg_Object_Entity_ltype_scale_get", _wrap_Dwg_Object_Entity_ltype_scale_get},
{"LibreDWGc::Dwg_Object_Entity_ltype_flags_set", _wrap_Dwg_Object_Entity_ltype_flags_set},
{"LibreDWGc::Dwg_Object_Entity_ltype_flags_get", _wrap_Dwg_Object_Entity_ltype_flags_get},
{"LibreDWGc::Dwg_Object_Entity_plotstyle_flags_set", _wrap_Dwg_Object_Entity_plotstyle_flags_set},
{"LibreDWGc::Dwg_Object_Entity_plotstyle_flags_get", _wrap_Dwg_Object_Entity_plotstyle_flags_get},
{"LibreDWGc::Dwg_Object_Entity_material_flags_set", _wrap_Dwg_Object_Entity_material_flags_set},
{"LibreDWGc::Dwg_Object_Entity_material_flags_get", _wrap_Dwg_Object_Entity_material_flags_get},
{"LibreDWGc::Dwg_Object_Entity_shadow_flags_set", _wrap_Dwg_Object_Entity_shadow_flags_set},
{"LibreDWGc::Dwg_Object_Entity_shadow_flags_get", _wrap_Dwg_Object_Entity_shadow_flags_get},
{"LibreDWGc::Dwg_Object_Entity_has_full_visualstyle_set", _wrap_Dwg_Object_Entity_has_full_visualstyle_set},
{"LibreDWGc::Dwg_Object_Entity_has_full_visualstyle_get", _wrap_Dwg_Object_Entity_has_full_visualstyle_get},
{"LibreDWGc::Dwg_Object_Entity_has_face_visualstyle_set", _wrap_Dwg_Object_Entity_has_face_visualstyle_set},
{"LibreDWGc::Dwg_Object_Entity_has_face_visualstyle_get", _wrap_Dwg_Object_Entity_has_face_visualstyle_get},
{"LibreDWGc::Dwg_Object_Entity_has_edge_visualstyle_set", _wrap_Dwg_Object_Entity_has_edge_visualstyle_set},
{"LibreDWGc::Dwg_Object_Entity_has_edge_visualstyle_get", _wrap_Dwg_Object_Entity_has_edge_visualstyle_get},
{"LibreDWGc::Dwg_Object_Entity_invisible_set", _wrap_Dwg_Object_Entity_invisible_set},
{"LibreDWGc::Dwg_Object_Entity_invisible_get", _wrap_Dwg_Object_Entity_invisible_get},
{"LibreDWGc::Dwg_Object_Entity_linewt_set", _wrap_Dwg_Object_Entity_linewt_set},
{"LibreDWGc::Dwg_Object_Entity_linewt_get", _wrap_Dwg_Object_Entity_linewt_get},
{"LibreDWGc::Dwg_Object_Entity_flag_r11_set", _wrap_Dwg_Object_Entity_flag_r11_set},
{"LibreDWGc::Dwg_Object_Entity_flag_r11_get", _wrap_Dwg_Object_Entity_flag_r11_get},
{"LibreDWGc::Dwg_Object_Entity_opts_r11_set", _wrap_Dwg_Object_Entity_opts_r11_set},
{"LibreDWGc::Dwg_Object_Entity_opts_r11_get", _wrap_Dwg_Object_Entity_opts_r11_get},
{"LibreDWGc::Dwg_Object_Entity_extra_r11_set", _wrap_Dwg_Object_Entity_extra_r11_set},
{"LibreDWGc::Dwg_Object_Entity_extra_r11_get", _wrap_Dwg_Object_Entity_extra_r11_get},
{"LibreDWGc::Dwg_Object_Entity_color_r11_set", _wrap_Dwg_Object_Entity_color_r11_set},
{"LibreDWGc::Dwg_Object_Entity_color_r11_get", _wrap_Dwg_Object_Entity_color_r11_get},
{"LibreDWGc::Dwg_Object_Entity_elevation_r11_set", _wrap_Dwg_Object_Entity_elevation_r11_set},
{"LibreDWGc::Dwg_Object_Entity_elevation_r11_get", _wrap_Dwg_Object_Entity_elevation_r11_get},
{"LibreDWGc::Dwg_Object_Entity_thickness_r11_set", _wrap_Dwg_Object_Entity_thickness_r11_set},
{"LibreDWGc::Dwg_Object_Entity_thickness_r11_get", _wrap_Dwg_Object_Entity_thickness_r11_get},
{"LibreDWGc::Dwg_Object_Entity_viewport_set", _wrap_Dwg_Object_Entity_viewport_set},
{"LibreDWGc::Dwg_Object_Entity_viewport_get", _wrap_Dwg_Object_Entity_viewport_get},
{"LibreDWGc::Dwg_Object_Entity___iterator_set", _wrap_Dwg_Object_Entity___iterator_set},
{"LibreDWGc::Dwg_Object_Entity___iterator_get", _wrap_Dwg_Object_Entity___iterator_get},
{"LibreDWGc::Dwg_Object_Entity_ownerhandle_set", _wrap_Dwg_Object_Entity_ownerhandle_set},
{"LibreDWGc::Dwg_Object_Entity_ownerhandle_get", _wrap_Dwg_Object_Entity_ownerhandle_get},
{"LibreDWGc::Dwg_Object_Entity_reactors_set", _wrap_Dwg_Object_Entity_reactors_set},
{"LibreDWGc::Dwg_Object_Entity_reactors_get", _wrap_Dwg_Object_Entity_reactors_get},
{"LibreDWGc::Dwg_Object_Entity_xdicobjhandle_set", _wrap_Dwg_Object_Entity_xdicobjhandle_set},
{"LibreDWGc::Dwg_Object_Entity_xdicobjhandle_get", _wrap_Dwg_Object_Entity_xdicobjhandle_get},
{"LibreDWGc::Dwg_Object_Entity_prev_entity_set", _wrap_Dwg_Object_Entity_prev_entity_set},
{"LibreDWGc::Dwg_Object_Entity_prev_entity_get", _wrap_Dwg_Object_Entity_prev_entity_get},
{"LibreDWGc::Dwg_Object_Entity_next_entity_set", _wrap_Dwg_Object_Entity_next_entity_set},
{"LibreDWGc::Dwg_Object_Entity_next_entity_get", _wrap_Dwg_Object_Entity_next_entity_get},
{"LibreDWGc::Dwg_Object_Entity_layer_set", _wrap_Dwg_Object_Entity_layer_set},
{"LibreDWGc::Dwg_Object_Entity_layer_get", _wrap_Dwg_Object_Entity_layer_get},
{"LibreDWGc::Dwg_Object_Entity_ltype_set", _wrap_Dwg_Object_Entity_ltype_set},
{"LibreDWGc::Dwg_Object_Entity_ltype_get", _wrap_Dwg_Object_Entity_ltype_get},
{"LibreDWGc::Dwg_Object_Entity_material_set", _wrap_Dwg_Object_Entity_material_set},
{"LibreDWGc::Dwg_Object_Entity_material_get", _wrap_Dwg_Object_Entity_material_get},
{"LibreDWGc::Dwg_Object_Entity_shadow_set", _wrap_Dwg_Object_Entity_shadow_set},
{"LibreDWGc::Dwg_Object_Entity_shadow_get", _wrap_Dwg_Object_Entity_shadow_get},
{"LibreDWGc::Dwg_Object_Entity_plotstyle_set", _wrap_Dwg_Object_Entity_plotstyle_set},
{"LibreDWGc::Dwg_Object_Entity_plotstyle_get", _wrap_Dwg_Object_Entity_plotstyle_get},
{"LibreDWGc::Dwg_Object_Entity_full_visualstyle_set", _wrap_Dwg_Object_Entity_full_visualstyle_set},
{"LibreDWGc::Dwg_Object_Entity_full_visualstyle_get", _wrap_Dwg_Object_Entity_full_visualstyle_get},
{"LibreDWGc::Dwg_Object_Entity_face_visualstyle_set", _wrap_Dwg_Object_Entity_face_visualstyle_set},
{"LibreDWGc::Dwg_Object_Entity_face_visualstyle_get", _wrap_Dwg_Object_Entity_face_visualstyle_get},
{"LibreDWGc::Dwg_Object_Entity_edge_visualstyle_set", _wrap_Dwg_Object_Entity_edge_visualstyle_set},
{"LibreDWGc::Dwg_Object_Entity_edge_visualstyle_get", _wrap_Dwg_Object_Entity_edge_visualstyle_get},
{"LibreDWGc::new_Dwg_Object_Entity", _wrap_new_Dwg_Object_Entity},
{"LibreDWGc::delete_Dwg_Object_Entity", _wrap_delete_Dwg_Object_Entity},
{"LibreDWGc::_dwg_object_entity_tio_UNUSED_set", _wrap__dwg_object_entity_tio_UNUSED_set},
{"LibreDWGc::_dwg_object_entity_tio_UNUSED_get", _wrap__dwg_object_entity_tio_UNUSED_get},
{"LibreDWGc::_dwg_object_entity_tio_DIMENSION_common_set", _wrap__dwg_object_entity_tio_DIMENSION_common_set},
{"LibreDWGc::_dwg_object_entity_tio_DIMENSION_common_get", _wrap__dwg_object_entity_tio_DIMENSION_common_get},
{"LibreDWGc::_dwg_object_entity_tio__3DFACE_set", _wrap__dwg_object_entity_tio__3DFACE_set},
{"LibreDWGc::_dwg_object_entity_tio__3DFACE_get", _wrap__dwg_object_entity_tio__3DFACE_get},
{"LibreDWGc::_dwg_object_entity_tio__3DSOLID_set", _wrap__dwg_object_entity_tio__3DSOLID_set},
{"LibreDWGc::_dwg_object_entity_tio__3DSOLID_get", _wrap__dwg_object_entity_tio__3DSOLID_get},
{"LibreDWGc::_dwg_object_entity_tio_ARC_set", _wrap__dwg_object_entity_tio_ARC_set},
{"LibreDWGc::_dwg_object_entity_tio_ARC_get", _wrap__dwg_object_entity_tio_ARC_get},
{"LibreDWGc::_dwg_object_entity_tio_ATTDEF_set", _wrap__dwg_object_entity_tio_ATTDEF_set},
{"LibreDWGc::_dwg_object_entity_tio_ATTDEF_get", _wrap__dwg_object_entity_tio_ATTDEF_get},
{"LibreDWGc::_dwg_object_entity_tio_ATTRIB_set", _wrap__dwg_object_entity_tio_ATTRIB_set},
{"LibreDWGc::_dwg_object_entity_tio_ATTRIB_get", _wrap__dwg_object_entity_tio_ATTRIB_get},
{"LibreDWGc::_dwg_object_entity_tio_BLOCK_set", _wrap__dwg_object_entity_tio_BLOCK_set},
{"LibreDWGc::_dwg_object_entity_tio_BLOCK_get", _wrap__dwg_object_entity_tio_BLOCK_get},
{"LibreDWGc::_dwg_object_entity_tio_BODY_set", _wrap__dwg_object_entity_tio_BODY_set},
{"LibreDWGc::_dwg_object_entity_tio_BODY_get", _wrap__dwg_object_entity_tio_BODY_get},
{"LibreDWGc::_dwg_object_entity_tio_CIRCLE_set", _wrap__dwg_object_entity_tio_CIRCLE_set},
{"LibreDWGc::_dwg_object_entity_tio_CIRCLE_get", _wrap__dwg_object_entity_tio_CIRCLE_get},
{"LibreDWGc::_dwg_object_entity_tio_DIMENSION_ALIGNED_set", _wrap__dwg_object_entity_tio_DIMENSION_ALIGNED_set},
{"LibreDWGc::_dwg_object_entity_tio_DIMENSION_ALIGNED_get", _wrap__dwg_object_entity_tio_DIMENSION_ALIGNED_get},
{"LibreDWGc::_dwg_object_entity_tio_DIMENSION_ANG2LN_set", _wrap__dwg_object_entity_tio_DIMENSION_ANG2LN_set},
{"LibreDWGc::_dwg_object_entity_tio_DIMENSION_ANG2LN_get", _wrap__dwg_object_entity_tio_DIMENSION_ANG2LN_get},
{"LibreDWGc::_dwg_object_entity_tio_DIMENSION_ANG3PT_set", _wrap__dwg_object_entity_tio_DIMENSION_ANG3PT_set},
{"LibreDWGc::_dwg_object_entity_tio_DIMENSION_ANG3PT_get", _wrap__dwg_object_entity_tio_DIMENSION_ANG3PT_get},
{"LibreDWGc::_dwg_object_entity_tio_DIMENSION_DIAMETER_set", _wrap__dwg_object_entity_tio_DIMENSION_DIAMETER_set},
{"LibreDWGc::_dwg_object_entity_tio_DIMENSION_DIAMETER_get", _wrap__dwg_object_entity_tio_DIMENSION_DIAMETER_get},
{"LibreDWGc::_dwg_object_entity_tio_DIMENSION_LINEAR_set", _wrap__dwg_object_entity_tio_DIMENSION_LINEAR_set},
{"LibreDWGc::_dwg_object_entity_tio_DIMENSION_LINEAR_get", _wrap__dwg_object_entity_tio_DIMENSION_LINEAR_get},
{"LibreDWGc::_dwg_object_entity_tio_DIMENSION_ORDINATE_set", _wrap__dwg_object_entity_tio_DIMENSION_ORDINATE_set},
{"LibreDWGc::_dwg_object_entity_tio_DIMENSION_ORDINATE_get", _wrap__dwg_object_entity_tio_DIMENSION_ORDINATE_get},
{"LibreDWGc::_dwg_object_entity_tio_DIMENSION_RADIUS_set", _wrap__dwg_object_entity_tio_DIMENSION_RADIUS_set},
{"LibreDWGc::_dwg_object_entity_tio_DIMENSION_RADIUS_get", _wrap__dwg_object_entity_tio_DIMENSION_RADIUS_get},
{"LibreDWGc::_dwg_object_entity_tio_ELLIPSE_set", _wrap__dwg_object_entity_tio_ELLIPSE_set},
{"LibreDWGc::_dwg_object_entity_tio_ELLIPSE_get", _wrap__dwg_object_entity_tio_ELLIPSE_get},
{"LibreDWGc::_dwg_object_entity_tio_ENDBLK_set", _wrap__dwg_object_entity_tio_ENDBLK_set},
{"LibreDWGc::_dwg_object_entity_tio_ENDBLK_get", _wrap__dwg_object_entity_tio_ENDBLK_get},
{"LibreDWGc::_dwg_object_entity_tio_INSERT_set", _wrap__dwg_object_entity_tio_INSERT_set},
{"LibreDWGc::_dwg_object_entity_tio_INSERT_get", _wrap__dwg_object_entity_tio_INSERT_get},
{"LibreDWGc::_dwg_object_entity_tio_LEADER_set", _wrap__dwg_object_entity_tio_LEADER_set},
{"LibreDWGc::_dwg_object_entity_tio_LEADER_get", _wrap__dwg_object_entity_tio_LEADER_get},
{"LibreDWGc::_dwg_object_entity_tio_LINE_set", _wrap__dwg_object_entity_tio_LINE_set},
{"LibreDWGc::_dwg_object_entity_tio_LINE_get", _wrap__dwg_object_entity_tio_LINE_get},
{"LibreDWGc::_dwg_object_entity_tio_LOAD_set", _wrap__dwg_object_entity_tio_LOAD_set},
{"LibreDWGc::_dwg_object_entity_tio_LOAD_get", _wrap__dwg_object_entity_tio_LOAD_get},
{"LibreDWGc::_dwg_object_entity_tio_MINSERT_set", _wrap__dwg_object_entity_tio_MINSERT_set},
{"LibreDWGc::_dwg_object_entity_tio_MINSERT_get", _wrap__dwg_object_entity_tio_MINSERT_get},
{"LibreDWGc::_dwg_object_entity_tio_MLINE_set", _wrap__dwg_object_entity_tio_MLINE_set},
{"LibreDWGc::_dwg_object_entity_tio_MLINE_get", _wrap__dwg_object_entity_tio_MLINE_get},
{"LibreDWGc::_dwg_object_entity_tio_MTEXT_set", _wrap__dwg_object_entity_tio_MTEXT_set},
{"LibreDWGc::_dwg_object_entity_tio_MTEXT_get", _wrap__dwg_object_entity_tio_MTEXT_get},
{"LibreDWGc::_dwg_object_entity_tio_OLEFRAME_set", _wrap__dwg_object_entity_tio_OLEFRAME_set},
{"LibreDWGc::_dwg_object_entity_tio_OLEFRAME_get", _wrap__dwg_object_entity_tio_OLEFRAME_get},
{"LibreDWGc::_dwg_object_entity_tio_POINT_set", _wrap__dwg_object_entity_tio_POINT_set},
{"LibreDWGc::_dwg_object_entity_tio_POINT_get", _wrap__dwg_object_entity_tio_POINT_get},
{"LibreDWGc::_dwg_object_entity_tio_POLYLINE_2D_set", _wrap__dwg_object_entity_tio_POLYLINE_2D_set},
{"LibreDWGc::_dwg_object_entity_tio_POLYLINE_2D_get", _wrap__dwg_object_entity_tio_POLYLINE_2D_get},
{"LibreDWGc::_dwg_object_entity_tio_POLYLINE_3D_set", _wrap__dwg_object_entity_tio_POLYLINE_3D_set},
{"LibreDWGc::_dwg_object_entity_tio_POLYLINE_3D_get", _wrap__dwg_object_entity_tio_POLYLINE_3D_get},
{"LibreDWGc::_dwg_object_entity_tio_POLYLINE_MESH_set", _wrap__dwg_object_entity_tio_POLYLINE_MESH_set},
{"LibreDWGc::_dwg_object_entity_tio_POLYLINE_MESH_get", _wrap__dwg_object_entity_tio_POLYLINE_MESH_get},
{"LibreDWGc::_dwg_object_entity_tio_POLYLINE_PFACE_set", _wrap__dwg_object_entity_tio_POLYLINE_PFACE_set},
{"LibreDWGc::_dwg_object_entity_tio_POLYLINE_PFACE_get", _wrap__dwg_object_entity_tio_POLYLINE_PFACE_get},
{"LibreDWGc::_dwg_object_entity_tio_PROXY_ENTITY_set", _wrap__dwg_object_entity_tio_PROXY_ENTITY_set},
{"LibreDWGc::_dwg_object_entity_tio_PROXY_ENTITY_get", _wrap__dwg_object_entity_tio_PROXY_ENTITY_get},
{"LibreDWGc::_dwg_object_entity_tio_RAY_set", _wrap__dwg_object_entity_tio_RAY_set},
{"LibreDWGc::_dwg_object_entity_tio_RAY_get", _wrap__dwg_object_entity_tio_RAY_get},
{"LibreDWGc::_dwg_object_entity_tio_REGION_set", _wrap__dwg_object_entity_tio_REGION_set},
{"LibreDWGc::_dwg_object_entity_tio_REGION_get", _wrap__dwg_object_entity_tio_REGION_get},
{"LibreDWGc::_dwg_object_entity_tio_SEQEND_set", _wrap__dwg_object_entity_tio_SEQEND_set},
{"LibreDWGc::_dwg_object_entity_tio_SEQEND_get", _wrap__dwg_object_entity_tio_SEQEND_get},
{"LibreDWGc::_dwg_object_entity_tio_SHAPE_set", _wrap__dwg_object_entity_tio_SHAPE_set},
{"LibreDWGc::_dwg_object_entity_tio_SHAPE_get", _wrap__dwg_object_entity_tio_SHAPE_get},
{"LibreDWGc::_dwg_object_entity_tio_SOLID_set", _wrap__dwg_object_entity_tio_SOLID_set},
{"LibreDWGc::_dwg_object_entity_tio_SOLID_get", _wrap__dwg_object_entity_tio_SOLID_get},
{"LibreDWGc::_dwg_object_entity_tio_SPLINE_set", _wrap__dwg_object_entity_tio_SPLINE_set},
{"LibreDWGc::_dwg_object_entity_tio_SPLINE_get", _wrap__dwg_object_entity_tio_SPLINE_get},
{"LibreDWGc::_dwg_object_entity_tio_TEXT_set", _wrap__dwg_object_entity_tio_TEXT_set},
{"LibreDWGc::_dwg_object_entity_tio_TEXT_get", _wrap__dwg_object_entity_tio_TEXT_get},
{"LibreDWGc::_dwg_object_entity_tio_TOLERANCE_set", _wrap__dwg_object_entity_tio_TOLERANCE_set},
{"LibreDWGc::_dwg_object_entity_tio_TOLERANCE_get", _wrap__dwg_object_entity_tio_TOLERANCE_get},
{"LibreDWGc::_dwg_object_entity_tio_TRACE_set", _wrap__dwg_object_entity_tio_TRACE_set},
{"LibreDWGc::_dwg_object_entity_tio_TRACE_get", _wrap__dwg_object_entity_tio_TRACE_get},
{"LibreDWGc::_dwg_object_entity_tio_UNKNOWN_ENT_set", _wrap__dwg_object_entity_tio_UNKNOWN_ENT_set},
{"LibreDWGc::_dwg_object_entity_tio_UNKNOWN_ENT_get", _wrap__dwg_object_entity_tio_UNKNOWN_ENT_get},
{"LibreDWGc::_dwg_object_entity_tio_VERTEX_2D_set", _wrap__dwg_object_entity_tio_VERTEX_2D_set},
{"LibreDWGc::_dwg_object_entity_tio_VERTEX_2D_get", _wrap__dwg_object_entity_tio_VERTEX_2D_get},
{"LibreDWGc::_dwg_object_entity_tio_VERTEX_3D_set", _wrap__dwg_object_entity_tio_VERTEX_3D_set},
{"LibreDWGc::_dwg_object_entity_tio_VERTEX_3D_get", _wrap__dwg_object_entity_tio_VERTEX_3D_get},
{"LibreDWGc::_dwg_object_entity_tio_VERTEX_MESH_set", _wrap__dwg_object_entity_tio_VERTEX_MESH_set},
{"LibreDWGc::_dwg_object_entity_tio_VERTEX_MESH_get", _wrap__dwg_object_entity_tio_VERTEX_MESH_get},
{"LibreDWGc::_dwg_object_entity_tio_VERTEX_PFACE_set", _wrap__dwg_object_entity_tio_VERTEX_PFACE_set},
{"LibreDWGc::_dwg_object_entity_tio_VERTEX_PFACE_get", _wrap__dwg_object_entity_tio_VERTEX_PFACE_get},
{"LibreDWGc::_dwg_object_entity_tio_VERTEX_PFACE_FACE_set", _wrap__dwg_object_entity_tio_VERTEX_PFACE_FACE_set},
{"LibreDWGc::_dwg_object_entity_tio_VERTEX_PFACE_FACE_get", _wrap__dwg_object_entity_tio_VERTEX_PFACE_FACE_get},
{"LibreDWGc::_dwg_object_entity_tio_VIEWPORT_set", _wrap__dwg_object_entity_tio_VIEWPORT_set},
{"LibreDWGc::_dwg_object_entity_tio_VIEWPORT_get", _wrap__dwg_object_entity_tio_VIEWPORT_get},
{"LibreDWGc::_dwg_object_entity_tio_XLINE_set", _wrap__dwg_object_entity_tio_XLINE_set},
{"LibreDWGc::_dwg_object_entity_tio_XLINE_get", _wrap__dwg_object_entity_tio_XLINE_get},
{"LibreDWGc::_dwg_object_entity_tio__3DLINE_set", _wrap__dwg_object_entity_tio__3DLINE_set},
{"LibreDWGc::_dwg_object_entity_tio__3DLINE_get", _wrap__dwg_object_entity_tio__3DLINE_get},
{"LibreDWGc::_dwg_object_entity_tio_CAMERA_set", _wrap__dwg_object_entity_tio_CAMERA_set},
{"LibreDWGc::_dwg_object_entity_tio_CAMERA_get", _wrap__dwg_object_entity_tio_CAMERA_get},
{"LibreDWGc::_dwg_object_entity_tio_DGNUNDERLAY_set", _wrap__dwg_object_entity_tio_DGNUNDERLAY_set},
{"LibreDWGc::_dwg_object_entity_tio_DGNUNDERLAY_get", _wrap__dwg_object_entity_tio_DGNUNDERLAY_get},
{"LibreDWGc::_dwg_object_entity_tio_DWFUNDERLAY_set", _wrap__dwg_object_entity_tio_DWFUNDERLAY_set},
{"LibreDWGc::_dwg_object_entity_tio_DWFUNDERLAY_get", _wrap__dwg_object_entity_tio_DWFUNDERLAY_get},
{"LibreDWGc::_dwg_object_entity_tio_ENDREP_set", _wrap__dwg_object_entity_tio_ENDREP_set},
{"LibreDWGc::_dwg_object_entity_tio_ENDREP_get", _wrap__dwg_object_entity_tio_ENDREP_get},
{"LibreDWGc::_dwg_object_entity_tio_HATCH_set", _wrap__dwg_object_entity_tio_HATCH_set},
{"LibreDWGc::_dwg_object_entity_tio_HATCH_get", _wrap__dwg_object_entity_tio_HATCH_get},
{"LibreDWGc::_dwg_object_entity_tio_IMAGE_set", _wrap__dwg_object_entity_tio_IMAGE_set},
{"LibreDWGc::_dwg_object_entity_tio_IMAGE_get", _wrap__dwg_object_entity_tio_IMAGE_get},
{"LibreDWGc::_dwg_object_entity_tio_JUMP_set", _wrap__dwg_object_entity_tio_JUMP_set},
{"LibreDWGc::_dwg_object_entity_tio_JUMP_get", _wrap__dwg_object_entity_tio_JUMP_get},
{"LibreDWGc::_dwg_object_entity_tio_LIGHT_set", _wrap__dwg_object_entity_tio_LIGHT_set},
{"LibreDWGc::_dwg_object_entity_tio_LIGHT_get", _wrap__dwg_object_entity_tio_LIGHT_get},
{"LibreDWGc::_dwg_object_entity_tio_LWPOLYLINE_set", _wrap__dwg_object_entity_tio_LWPOLYLINE_set},
{"LibreDWGc::_dwg_object_entity_tio_LWPOLYLINE_get", _wrap__dwg_object_entity_tio_LWPOLYLINE_get},
{"LibreDWGc::_dwg_object_entity_tio_MESH_set", _wrap__dwg_object_entity_tio_MESH_set},
{"LibreDWGc::_dwg_object_entity_tio_MESH_get", _wrap__dwg_object_entity_tio_MESH_get},
{"LibreDWGc::_dwg_object_entity_tio_MULTILEADER_set", _wrap__dwg_object_entity_tio_MULTILEADER_set},
{"LibreDWGc::_dwg_object_entity_tio_MULTILEADER_get", _wrap__dwg_object_entity_tio_MULTILEADER_get},
{"LibreDWGc::_dwg_object_entity_tio_OLE2FRAME_set", _wrap__dwg_object_entity_tio_OLE2FRAME_set},
{"LibreDWGc::_dwg_object_entity_tio_OLE2FRAME_get", _wrap__dwg_object_entity_tio_OLE2FRAME_get},
{"LibreDWGc::_dwg_object_entity_tio_PDFUNDERLAY_set", _wrap__dwg_object_entity_tio_PDFUNDERLAY_set},
{"LibreDWGc::_dwg_object_entity_tio_PDFUNDERLAY_get", _wrap__dwg_object_entity_tio_PDFUNDERLAY_get},
{"LibreDWGc::_dwg_object_entity_tio_REPEAT_set", _wrap__dwg_object_entity_tio_REPEAT_set},
{"LibreDWGc::_dwg_object_entity_tio_REPEAT_get", _wrap__dwg_object_entity_tio_REPEAT_get},
{"LibreDWGc::_dwg_object_entity_tio_SECTIONOBJECT_set", _wrap__dwg_object_entity_tio_SECTIONOBJECT_set},
{"LibreDWGc::_dwg_object_entity_tio_SECTIONOBJECT_get", _wrap__dwg_object_entity_tio_SECTIONOBJECT_get},
{"LibreDWGc::_dwg_object_entity_tio_WIPEOUT_set", _wrap__dwg_object_entity_tio_WIPEOUT_set},
{"LibreDWGc::_dwg_object_entity_tio_WIPEOUT_get", _wrap__dwg_object_entity_tio_WIPEOUT_get},
{"LibreDWGc::_dwg_object_entity_tio_ARC_DIMENSION_set", _wrap__dwg_object_entity_tio_ARC_DIMENSION_set},
{"LibreDWGc::_dwg_object_entity_tio_ARC_DIMENSION_get", _wrap__dwg_object_entity_tio_ARC_DIMENSION_get},
{"LibreDWGc::_dwg_object_entity_tio_HELIX_set", _wrap__dwg_object_entity_tio_HELIX_set},
{"LibreDWGc::_dwg_object_entity_tio_HELIX_get", _wrap__dwg_object_entity_tio_HELIX_get},
{"LibreDWGc::_dwg_object_entity_tio_LARGE_RADIAL_DIMENSION_set", _wrap__dwg_object_entity_tio_LARGE_RADIAL_DIMENSION_set},
{"LibreDWGc::_dwg_object_entity_tio_LARGE_RADIAL_DIMENSION_get", _wrap__dwg_object_entity_tio_LARGE_RADIAL_DIMENSION_get},
{"LibreDWGc::_dwg_object_entity_tio_LAYOUTPRINTCONFIG_set", _wrap__dwg_object_entity_tio_LAYOUTPRINTCONFIG_set},
{"LibreDWGc::_dwg_object_entity_tio_LAYOUTPRINTCONFIG_get", _wrap__dwg_object_entity_tio_LAYOUTPRINTCONFIG_get},
{"LibreDWGc::_dwg_object_entity_tio_PLANESURFACE_set", _wrap__dwg_object_entity_tio_PLANESURFACE_set},
{"LibreDWGc::_dwg_object_entity_tio_PLANESURFACE_get", _wrap__dwg_object_entity_tio_PLANESURFACE_get},
{"LibreDWGc::_dwg_object_entity_tio_POINTCLOUD_set", _wrap__dwg_object_entity_tio_POINTCLOUD_set},
{"LibreDWGc::_dwg_object_entity_tio_POINTCLOUD_get", _wrap__dwg_object_entity_tio_POINTCLOUD_get},
{"LibreDWGc::_dwg_object_entity_tio_POINTCLOUDEX_set", _wrap__dwg_object_entity_tio_POINTCLOUDEX_set},
{"LibreDWGc::_dwg_object_entity_tio_POINTCLOUDEX_get", _wrap__dwg_object_entity_tio_POINTCLOUDEX_get},
{"LibreDWGc::_dwg_object_entity_tio_ALIGNMENTPARAMETERENTITY_set", _wrap__dwg_object_entity_tio_ALIGNMENTPARAMETERENTITY_set},
{"LibreDWGc::_dwg_object_entity_tio_ALIGNMENTPARAMETERENTITY_get", _wrap__dwg_object_entity_tio_ALIGNMENTPARAMETERENTITY_get},
{"LibreDWGc::_dwg_object_entity_tio_ARCALIGNEDTEXT_set", _wrap__dwg_object_entity_tio_ARCALIGNEDTEXT_set},
{"LibreDWGc::_dwg_object_entity_tio_ARCALIGNEDTEXT_get", _wrap__dwg_object_entity_tio_ARCALIGNEDTEXT_get},
{"LibreDWGc::_dwg_object_entity_tio_BASEPOINTPARAMETERENTITY_set", _wrap__dwg_object_entity_tio_BASEPOINTPARAMETERENTITY_set},
{"LibreDWGc::_dwg_object_entity_tio_BASEPOINTPARAMETERENTITY_get", _wrap__dwg_object_entity_tio_BASEPOINTPARAMETERENTITY_get},
{"LibreDWGc::_dwg_object_entity_tio_EXTRUDEDSURFACE_set", _wrap__dwg_object_entity_tio_EXTRUDEDSURFACE_set},
{"LibreDWGc::_dwg_object_entity_tio_EXTRUDEDSURFACE_get", _wrap__dwg_object_entity_tio_EXTRUDEDSURFACE_get},
{"LibreDWGc::_dwg_object_entity_tio_FLIPGRIPENTITY_set", _wrap__dwg_object_entity_tio_FLIPGRIPENTITY_set},
{"LibreDWGc::_dwg_object_entity_tio_FLIPGRIPENTITY_get", _wrap__dwg_object_entity_tio_FLIPGRIPENTITY_get},
{"LibreDWGc::_dwg_object_entity_tio_FLIPPARAMETERENTITY_set", _wrap__dwg_object_entity_tio_FLIPPARAMETERENTITY_set},
{"LibreDWGc::_dwg_object_entity_tio_FLIPPARAMETERENTITY_get", _wrap__dwg_object_entity_tio_FLIPPARAMETERENTITY_get},
{"LibreDWGc::_dwg_object_entity_tio_GEOPOSITIONMARKER_set", _wrap__dwg_object_entity_tio_GEOPOSITIONMARKER_set},
{"LibreDWGc::_dwg_object_entity_tio_GEOPOSITIONMARKER_get", _wrap__dwg_object_entity_tio_GEOPOSITIONMARKER_get},
{"LibreDWGc::_dwg_object_entity_tio_LINEARGRIPENTITY_set", _wrap__dwg_object_entity_tio_LINEARGRIPENTITY_set},
{"LibreDWGc::_dwg_object_entity_tio_LINEARGRIPENTITY_get", _wrap__dwg_object_entity_tio_LINEARGRIPENTITY_get},
{"LibreDWGc::_dwg_object_entity_tio_LINEARPARAMETERENTITY_set", _wrap__dwg_object_entity_tio_LINEARPARAMETERENTITY_set},
{"LibreDWGc::_dwg_object_entity_tio_LINEARPARAMETERENTITY_get", _wrap__dwg_object_entity_tio_LINEARPARAMETERENTITY_get},
{"LibreDWGc::_dwg_object_entity_tio_LOFTEDSURFACE_set", _wrap__dwg_object_entity_tio_LOFTEDSURFACE_set},
{"LibreDWGc::_dwg_object_entity_tio_LOFTEDSURFACE_get", _wrap__dwg_object_entity_tio_LOFTEDSURFACE_get},
{"LibreDWGc::_dwg_object_entity_tio_MPOLYGON_set", _wrap__dwg_object_entity_tio_MPOLYGON_set},
{"LibreDWGc::_dwg_object_entity_tio_MPOLYGON_get", _wrap__dwg_object_entity_tio_MPOLYGON_get},
{"LibreDWGc::_dwg_object_entity_tio_NAVISWORKSMODEL_set", _wrap__dwg_object_entity_tio_NAVISWORKSMODEL_set},
{"LibreDWGc::_dwg_object_entity_tio_NAVISWORKSMODEL_get", _wrap__dwg_object_entity_tio_NAVISWORKSMODEL_get},
{"LibreDWGc::_dwg_object_entity_tio_NURBSURFACE_set", _wrap__dwg_object_entity_tio_NURBSURFACE_set},
{"LibreDWGc::_dwg_object_entity_tio_NURBSURFACE_get", _wrap__dwg_object_entity_tio_NURBSURFACE_get},
{"LibreDWGc::_dwg_object_entity_tio_POINTPARAMETERENTITY_set", _wrap__dwg_object_entity_tio_POINTPARAMETERENTITY_set},
{"LibreDWGc::_dwg_object_entity_tio_POINTPARAMETERENTITY_get", _wrap__dwg_object_entity_tio_POINTPARAMETERENTITY_get},
{"LibreDWGc::_dwg_object_entity_tio_POLARGRIPENTITY_set", _wrap__dwg_object_entity_tio_POLARGRIPENTITY_set},
{"LibreDWGc::_dwg_object_entity_tio_POLARGRIPENTITY_get", _wrap__dwg_object_entity_tio_POLARGRIPENTITY_get},
{"LibreDWGc::_dwg_object_entity_tio_REVOLVEDSURFACE_set", _wrap__dwg_object_entity_tio_REVOLVEDSURFACE_set},
{"LibreDWGc::_dwg_object_entity_tio_REVOLVEDSURFACE_get", _wrap__dwg_object_entity_tio_REVOLVEDSURFACE_get},
{"LibreDWGc::_dwg_object_entity_tio_ROTATIONGRIPENTITY_set", _wrap__dwg_object_entity_tio_ROTATIONGRIPENTITY_set},
{"LibreDWGc::_dwg_object_entity_tio_ROTATIONGRIPENTITY_get", _wrap__dwg_object_entity_tio_ROTATIONGRIPENTITY_get},
{"LibreDWGc::_dwg_object_entity_tio_ROTATIONPARAMETERENTITY_set", _wrap__dwg_object_entity_tio_ROTATIONPARAMETERENTITY_set},
{"LibreDWGc::_dwg_object_entity_tio_ROTATIONPARAMETERENTITY_get", _wrap__dwg_object_entity_tio_ROTATIONPARAMETERENTITY_get},
{"LibreDWGc::_dwg_object_entity_tio_RTEXT_set", _wrap__dwg_object_entity_tio_RTEXT_set},
{"LibreDWGc::_dwg_object_entity_tio_RTEXT_get", _wrap__dwg_object_entity_tio_RTEXT_get},
{"LibreDWGc::_dwg_object_entity_tio_SWEPTSURFACE_set", _wrap__dwg_object_entity_tio_SWEPTSURFACE_set},
{"LibreDWGc::_dwg_object_entity_tio_SWEPTSURFACE_get", _wrap__dwg_object_entity_tio_SWEPTSURFACE_get},
{"LibreDWGc::_dwg_object_entity_tio_TABLE_set", _wrap__dwg_object_entity_tio_TABLE_set},
{"LibreDWGc::_dwg_object_entity_tio_TABLE_get", _wrap__dwg_object_entity_tio_TABLE_get},
{"LibreDWGc::_dwg_object_entity_tio_VISIBILITYGRIPENTITY_set", _wrap__dwg_object_entity_tio_VISIBILITYGRIPENTITY_set},
{"LibreDWGc::_dwg_object_entity_tio_VISIBILITYGRIPENTITY_get", _wrap__dwg_object_entity_tio_VISIBILITYGRIPENTITY_get},
{"LibreDWGc::_dwg_object_entity_tio_VISIBILITYPARAMETERENTITY_set", _wrap__dwg_object_entity_tio_VISIBILITYPARAMETERENTITY_set},
{"LibreDWGc::_dwg_object_entity_tio_VISIBILITYPARAMETERENTITY_get", _wrap__dwg_object_entity_tio_VISIBILITYPARAMETERENTITY_get},
{"LibreDWGc::_dwg_object_entity_tio_XYGRIPENTITY_set", _wrap__dwg_object_entity_tio_XYGRIPENTITY_set},
{"LibreDWGc::_dwg_object_entity_tio_XYGRIPENTITY_get", _wrap__dwg_object_entity_tio_XYGRIPENTITY_get},
{"LibreDWGc::_dwg_object_entity_tio_XYPARAMETERENTITY_set", _wrap__dwg_object_entity_tio_XYPARAMETERENTITY_set},
{"LibreDWGc::_dwg_object_entity_tio_XYPARAMETERENTITY_get", _wrap__dwg_object_entity_tio_XYPARAMETERENTITY_get},
{"LibreDWGc::new__dwg_object_entity_tio", _wrap_new__dwg_object_entity_tio},
{"LibreDWGc::delete__dwg_object_entity_tio", _wrap_delete__dwg_object_entity_tio},
{"LibreDWGc::Dwg_Object_Object_objid_set", _wrap_Dwg_Object_Object_objid_set},
{"LibreDWGc::Dwg_Object_Object_objid_get", _wrap_Dwg_Object_Object_objid_get},
{"LibreDWGc::Dwg_Object_Object_tio_get", _wrap_Dwg_Object_Object_tio_get},
{"LibreDWGc::Dwg_Object_Object_dwg_set", _wrap_Dwg_Object_Object_dwg_set},
{"LibreDWGc::Dwg_Object_Object_dwg_get", _wrap_Dwg_Object_Object_dwg_get},
{"LibreDWGc::Dwg_Object_Object_num_eed_set", _wrap_Dwg_Object_Object_num_eed_set},
{"LibreDWGc::Dwg_Object_Object_num_eed_get", _wrap_Dwg_Object_Object_num_eed_get},
{"LibreDWGc::Dwg_Object_Object_eed_set", _wrap_Dwg_Object_Object_eed_set},
{"LibreDWGc::Dwg_Object_Object_eed_get", _wrap_Dwg_Object_Object_eed_get},
{"LibreDWGc::Dwg_Object_Object_ownerhandle_set", _wrap_Dwg_Object_Object_ownerhandle_set},
{"LibreDWGc::Dwg_Object_Object_ownerhandle_get", _wrap_Dwg_Object_Object_ownerhandle_get},
{"LibreDWGc::Dwg_Object_Object_num_reactors_set", _wrap_Dwg_Object_Object_num_reactors_set},
{"LibreDWGc::Dwg_Object_Object_num_reactors_get", _wrap_Dwg_Object_Object_num_reactors_get},
{"LibreDWGc::Dwg_Object_Object_reactors_set", _wrap_Dwg_Object_Object_reactors_set},
{"LibreDWGc::Dwg_Object_Object_reactors_get", _wrap_Dwg_Object_Object_reactors_get},
{"LibreDWGc::Dwg_Object_Object_xdicobjhandle_set", _wrap_Dwg_Object_Object_xdicobjhandle_set},
{"LibreDWGc::Dwg_Object_Object_xdicobjhandle_get", _wrap_Dwg_Object_Object_xdicobjhandle_get},
{"LibreDWGc::Dwg_Object_Object_is_xdic_missing_set", _wrap_Dwg_Object_Object_is_xdic_missing_set},
{"LibreDWGc::Dwg_Object_Object_is_xdic_missing_get", _wrap_Dwg_Object_Object_is_xdic_missing_get},
{"LibreDWGc::Dwg_Object_Object_has_ds_data_set", _wrap_Dwg_Object_Object_has_ds_data_set},
{"LibreDWGc::Dwg_Object_Object_has_ds_data_get", _wrap_Dwg_Object_Object_has_ds_data_get},
{"LibreDWGc::Dwg_Object_Object_handleref_set", _wrap_Dwg_Object_Object_handleref_set},
{"LibreDWGc::Dwg_Object_Object_handleref_get", _wrap_Dwg_Object_Object_handleref_get},
{"LibreDWGc::new_Dwg_Object_Object", _wrap_new_Dwg_Object_Object},
{"LibreDWGc::delete_Dwg_Object_Object", _wrap_delete_Dwg_Object_Object},
{"LibreDWGc::_dwg_object_object_tio_APPID_set", _wrap__dwg_object_object_tio_APPID_set},
{"LibreDWGc::_dwg_object_object_tio_APPID_get", _wrap__dwg_object_object_tio_APPID_get},
{"LibreDWGc::_dwg_object_object_tio_APPID_CONTROL_set", _wrap__dwg_object_object_tio_APPID_CONTROL_set},
{"LibreDWGc::_dwg_object_object_tio_APPID_CONTROL_get", _wrap__dwg_object_object_tio_APPID_CONTROL_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCK_CONTROL_set", _wrap__dwg_object_object_tio_BLOCK_CONTROL_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCK_CONTROL_get", _wrap__dwg_object_object_tio_BLOCK_CONTROL_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCK_HEADER_set", _wrap__dwg_object_object_tio_BLOCK_HEADER_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCK_HEADER_get", _wrap__dwg_object_object_tio_BLOCK_HEADER_get},
{"LibreDWGc::_dwg_object_object_tio_DICTIONARY_set", _wrap__dwg_object_object_tio_DICTIONARY_set},
{"LibreDWGc::_dwg_object_object_tio_DICTIONARY_get", _wrap__dwg_object_object_tio_DICTIONARY_get},
{"LibreDWGc::_dwg_object_object_tio_DIMSTYLE_set", _wrap__dwg_object_object_tio_DIMSTYLE_set},
{"LibreDWGc::_dwg_object_object_tio_DIMSTYLE_get", _wrap__dwg_object_object_tio_DIMSTYLE_get},
{"LibreDWGc::_dwg_object_object_tio_DIMSTYLE_CONTROL_set", _wrap__dwg_object_object_tio_DIMSTYLE_CONTROL_set},
{"LibreDWGc::_dwg_object_object_tio_DIMSTYLE_CONTROL_get", _wrap__dwg_object_object_tio_DIMSTYLE_CONTROL_get},
{"LibreDWGc::_dwg_object_object_tio_DUMMY_set", _wrap__dwg_object_object_tio_DUMMY_set},
{"LibreDWGc::_dwg_object_object_tio_DUMMY_get", _wrap__dwg_object_object_tio_DUMMY_get},
{"LibreDWGc::_dwg_object_object_tio_LAYER_set", _wrap__dwg_object_object_tio_LAYER_set},
{"LibreDWGc::_dwg_object_object_tio_LAYER_get", _wrap__dwg_object_object_tio_LAYER_get},
{"LibreDWGc::_dwg_object_object_tio_LAYER_CONTROL_set", _wrap__dwg_object_object_tio_LAYER_CONTROL_set},
{"LibreDWGc::_dwg_object_object_tio_LAYER_CONTROL_get", _wrap__dwg_object_object_tio_LAYER_CONTROL_get},
{"LibreDWGc::_dwg_object_object_tio_LONG_TRANSACTION_set", _wrap__dwg_object_object_tio_LONG_TRANSACTION_set},
{"LibreDWGc::_dwg_object_object_tio_LONG_TRANSACTION_get", _wrap__dwg_object_object_tio_LONG_TRANSACTION_get},
{"LibreDWGc::_dwg_object_object_tio_LTYPE_set", _wrap__dwg_object_object_tio_LTYPE_set},
{"LibreDWGc::_dwg_object_object_tio_LTYPE_get", _wrap__dwg_object_object_tio_LTYPE_get},
{"LibreDWGc::_dwg_object_object_tio_LTYPE_CONTROL_set", _wrap__dwg_object_object_tio_LTYPE_CONTROL_set},
{"LibreDWGc::_dwg_object_object_tio_LTYPE_CONTROL_get", _wrap__dwg_object_object_tio_LTYPE_CONTROL_get},
{"LibreDWGc::_dwg_object_object_tio_MLINESTYLE_set", _wrap__dwg_object_object_tio_MLINESTYLE_set},
{"LibreDWGc::_dwg_object_object_tio_MLINESTYLE_get", _wrap__dwg_object_object_tio_MLINESTYLE_get},
{"LibreDWGc::_dwg_object_object_tio_STYLE_set", _wrap__dwg_object_object_tio_STYLE_set},
{"LibreDWGc::_dwg_object_object_tio_STYLE_get", _wrap__dwg_object_object_tio_STYLE_get},
{"LibreDWGc::_dwg_object_object_tio_STYLE_CONTROL_set", _wrap__dwg_object_object_tio_STYLE_CONTROL_set},
{"LibreDWGc::_dwg_object_object_tio_STYLE_CONTROL_get", _wrap__dwg_object_object_tio_STYLE_CONTROL_get},
{"LibreDWGc::_dwg_object_object_tio_UCS_set", _wrap__dwg_object_object_tio_UCS_set},
{"LibreDWGc::_dwg_object_object_tio_UCS_get", _wrap__dwg_object_object_tio_UCS_get},
{"LibreDWGc::_dwg_object_object_tio_UCS_CONTROL_set", _wrap__dwg_object_object_tio_UCS_CONTROL_set},
{"LibreDWGc::_dwg_object_object_tio_UCS_CONTROL_get", _wrap__dwg_object_object_tio_UCS_CONTROL_get},
{"LibreDWGc::_dwg_object_object_tio_UNKNOWN_OBJ_set", _wrap__dwg_object_object_tio_UNKNOWN_OBJ_set},
{"LibreDWGc::_dwg_object_object_tio_UNKNOWN_OBJ_get", _wrap__dwg_object_object_tio_UNKNOWN_OBJ_get},
{"LibreDWGc::_dwg_object_object_tio_VIEW_set", _wrap__dwg_object_object_tio_VIEW_set},
{"LibreDWGc::_dwg_object_object_tio_VIEW_get", _wrap__dwg_object_object_tio_VIEW_get},
{"LibreDWGc::_dwg_object_object_tio_VIEW_CONTROL_set", _wrap__dwg_object_object_tio_VIEW_CONTROL_set},
{"LibreDWGc::_dwg_object_object_tio_VIEW_CONTROL_get", _wrap__dwg_object_object_tio_VIEW_CONTROL_get},
{"LibreDWGc::_dwg_object_object_tio_VPORT_set", _wrap__dwg_object_object_tio_VPORT_set},
{"LibreDWGc::_dwg_object_object_tio_VPORT_get", _wrap__dwg_object_object_tio_VPORT_get},
{"LibreDWGc::_dwg_object_object_tio_VPORT_CONTROL_set", _wrap__dwg_object_object_tio_VPORT_CONTROL_set},
{"LibreDWGc::_dwg_object_object_tio_VPORT_CONTROL_get", _wrap__dwg_object_object_tio_VPORT_CONTROL_get},
{"LibreDWGc::_dwg_object_object_tio_VX_CONTROL_set", _wrap__dwg_object_object_tio_VX_CONTROL_set},
{"LibreDWGc::_dwg_object_object_tio_VX_CONTROL_get", _wrap__dwg_object_object_tio_VX_CONTROL_get},
{"LibreDWGc::_dwg_object_object_tio_VX_TABLE_RECORD_set", _wrap__dwg_object_object_tio_VX_TABLE_RECORD_set},
{"LibreDWGc::_dwg_object_object_tio_VX_TABLE_RECORD_get", _wrap__dwg_object_object_tio_VX_TABLE_RECORD_get},
{"LibreDWGc::_dwg_object_object_tio_ACSH_BOOLEAN_CLASS_set", _wrap__dwg_object_object_tio_ACSH_BOOLEAN_CLASS_set},
{"LibreDWGc::_dwg_object_object_tio_ACSH_BOOLEAN_CLASS_get", _wrap__dwg_object_object_tio_ACSH_BOOLEAN_CLASS_get},
{"LibreDWGc::_dwg_object_object_tio_ACSH_BOX_CLASS_set", _wrap__dwg_object_object_tio_ACSH_BOX_CLASS_set},
{"LibreDWGc::_dwg_object_object_tio_ACSH_BOX_CLASS_get", _wrap__dwg_object_object_tio_ACSH_BOX_CLASS_get},
{"LibreDWGc::_dwg_object_object_tio_ACSH_CONE_CLASS_set", _wrap__dwg_object_object_tio_ACSH_CONE_CLASS_set},
{"LibreDWGc::_dwg_object_object_tio_ACSH_CONE_CLASS_get", _wrap__dwg_object_object_tio_ACSH_CONE_CLASS_get},
{"LibreDWGc::_dwg_object_object_tio_ACSH_CYLINDER_CLASS_set", _wrap__dwg_object_object_tio_ACSH_CYLINDER_CLASS_set},
{"LibreDWGc::_dwg_object_object_tio_ACSH_CYLINDER_CLASS_get", _wrap__dwg_object_object_tio_ACSH_CYLINDER_CLASS_get},
{"LibreDWGc::_dwg_object_object_tio_ACSH_FILLET_CLASS_set", _wrap__dwg_object_object_tio_ACSH_FILLET_CLASS_set},
{"LibreDWGc::_dwg_object_object_tio_ACSH_FILLET_CLASS_get", _wrap__dwg_object_object_tio_ACSH_FILLET_CLASS_get},
{"LibreDWGc::_dwg_object_object_tio_ACSH_HISTORY_CLASS_set", _wrap__dwg_object_object_tio_ACSH_HISTORY_CLASS_set},
{"LibreDWGc::_dwg_object_object_tio_ACSH_HISTORY_CLASS_get", _wrap__dwg_object_object_tio_ACSH_HISTORY_CLASS_get},
{"LibreDWGc::_dwg_object_object_tio_ACSH_SPHERE_CLASS_set", _wrap__dwg_object_object_tio_ACSH_SPHERE_CLASS_set},
{"LibreDWGc::_dwg_object_object_tio_ACSH_SPHERE_CLASS_get", _wrap__dwg_object_object_tio_ACSH_SPHERE_CLASS_get},
{"LibreDWGc::_dwg_object_object_tio_ACSH_TORUS_CLASS_set", _wrap__dwg_object_object_tio_ACSH_TORUS_CLASS_set},
{"LibreDWGc::_dwg_object_object_tio_ACSH_TORUS_CLASS_get", _wrap__dwg_object_object_tio_ACSH_TORUS_CLASS_get},
{"LibreDWGc::_dwg_object_object_tio_ACSH_WEDGE_CLASS_set", _wrap__dwg_object_object_tio_ACSH_WEDGE_CLASS_set},
{"LibreDWGc::_dwg_object_object_tio_ACSH_WEDGE_CLASS_get", _wrap__dwg_object_object_tio_ACSH_WEDGE_CLASS_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCGEOMDEPENDENCY_set", _wrap__dwg_object_object_tio_ASSOCGEOMDEPENDENCY_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCGEOMDEPENDENCY_get", _wrap__dwg_object_object_tio_ASSOCGEOMDEPENDENCY_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCNETWORK_set", _wrap__dwg_object_object_tio_ASSOCNETWORK_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCNETWORK_get", _wrap__dwg_object_object_tio_ASSOCNETWORK_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKALIGNMENTGRIP_set", _wrap__dwg_object_object_tio_BLOCKALIGNMENTGRIP_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKALIGNMENTGRIP_get", _wrap__dwg_object_object_tio_BLOCKALIGNMENTGRIP_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKALIGNMENTPARAMETER_set", _wrap__dwg_object_object_tio_BLOCKALIGNMENTPARAMETER_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKALIGNMENTPARAMETER_get", _wrap__dwg_object_object_tio_BLOCKALIGNMENTPARAMETER_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKBASEPOINTPARAMETER_set", _wrap__dwg_object_object_tio_BLOCKBASEPOINTPARAMETER_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKBASEPOINTPARAMETER_get", _wrap__dwg_object_object_tio_BLOCKBASEPOINTPARAMETER_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKFLIPACTION_set", _wrap__dwg_object_object_tio_BLOCKFLIPACTION_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKFLIPACTION_get", _wrap__dwg_object_object_tio_BLOCKFLIPACTION_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKFLIPGRIP_set", _wrap__dwg_object_object_tio_BLOCKFLIPGRIP_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKFLIPGRIP_get", _wrap__dwg_object_object_tio_BLOCKFLIPGRIP_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKFLIPPARAMETER_set", _wrap__dwg_object_object_tio_BLOCKFLIPPARAMETER_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKFLIPPARAMETER_get", _wrap__dwg_object_object_tio_BLOCKFLIPPARAMETER_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKGRIPLOCATIONCOMPONENT_set", _wrap__dwg_object_object_tio_BLOCKGRIPLOCATIONCOMPONENT_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKGRIPLOCATIONCOMPONENT_get", _wrap__dwg_object_object_tio_BLOCKGRIPLOCATIONCOMPONENT_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKLINEARGRIP_set", _wrap__dwg_object_object_tio_BLOCKLINEARGRIP_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKLINEARGRIP_get", _wrap__dwg_object_object_tio_BLOCKLINEARGRIP_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKLOOKUPGRIP_set", _wrap__dwg_object_object_tio_BLOCKLOOKUPGRIP_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKLOOKUPGRIP_get", _wrap__dwg_object_object_tio_BLOCKLOOKUPGRIP_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKMOVEACTION_set", _wrap__dwg_object_object_tio_BLOCKMOVEACTION_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKMOVEACTION_get", _wrap__dwg_object_object_tio_BLOCKMOVEACTION_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKROTATEACTION_set", _wrap__dwg_object_object_tio_BLOCKROTATEACTION_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKROTATEACTION_get", _wrap__dwg_object_object_tio_BLOCKROTATEACTION_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKROTATIONGRIP_set", _wrap__dwg_object_object_tio_BLOCKROTATIONGRIP_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKROTATIONGRIP_get", _wrap__dwg_object_object_tio_BLOCKROTATIONGRIP_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKSCALEACTION_set", _wrap__dwg_object_object_tio_BLOCKSCALEACTION_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKSCALEACTION_get", _wrap__dwg_object_object_tio_BLOCKSCALEACTION_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKVISIBILITYGRIP_set", _wrap__dwg_object_object_tio_BLOCKVISIBILITYGRIP_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKVISIBILITYGRIP_get", _wrap__dwg_object_object_tio_BLOCKVISIBILITYGRIP_get},
{"LibreDWGc::_dwg_object_object_tio_CELLSTYLEMAP_set", _wrap__dwg_object_object_tio_CELLSTYLEMAP_set},
{"LibreDWGc::_dwg_object_object_tio_CELLSTYLEMAP_get", _wrap__dwg_object_object_tio_CELLSTYLEMAP_get},
{"LibreDWGc::_dwg_object_object_tio_DETAILVIEWSTYLE_set", _wrap__dwg_object_object_tio_DETAILVIEWSTYLE_set},
{"LibreDWGc::_dwg_object_object_tio_DETAILVIEWSTYLE_get", _wrap__dwg_object_object_tio_DETAILVIEWSTYLE_get},
{"LibreDWGc::_dwg_object_object_tio_DICTIONARYVAR_set", _wrap__dwg_object_object_tio_DICTIONARYVAR_set},
{"LibreDWGc::_dwg_object_object_tio_DICTIONARYVAR_get", _wrap__dwg_object_object_tio_DICTIONARYVAR_get},
{"LibreDWGc::_dwg_object_object_tio_DICTIONARYWDFLT_set", _wrap__dwg_object_object_tio_DICTIONARYWDFLT_set},
{"LibreDWGc::_dwg_object_object_tio_DICTIONARYWDFLT_get", _wrap__dwg_object_object_tio_DICTIONARYWDFLT_get},
{"LibreDWGc::_dwg_object_object_tio_DYNAMICBLOCKPURGEPREVENTER_set", _wrap__dwg_object_object_tio_DYNAMICBLOCKPURGEPREVENTER_set},
{"LibreDWGc::_dwg_object_object_tio_DYNAMICBLOCKPURGEPREVENTER_get", _wrap__dwg_object_object_tio_DYNAMICBLOCKPURGEPREVENTER_get},
{"LibreDWGc::_dwg_object_object_tio_FIELD_set", _wrap__dwg_object_object_tio_FIELD_set},
{"LibreDWGc::_dwg_object_object_tio_FIELD_get", _wrap__dwg_object_object_tio_FIELD_get},
{"LibreDWGc::_dwg_object_object_tio_FIELDLIST_set", _wrap__dwg_object_object_tio_FIELDLIST_set},
{"LibreDWGc::_dwg_object_object_tio_FIELDLIST_get", _wrap__dwg_object_object_tio_FIELDLIST_get},
{"LibreDWGc::_dwg_object_object_tio_GEODATA_set", _wrap__dwg_object_object_tio_GEODATA_set},
{"LibreDWGc::_dwg_object_object_tio_GEODATA_get", _wrap__dwg_object_object_tio_GEODATA_get},
{"LibreDWGc::_dwg_object_object_tio_GROUP_set", _wrap__dwg_object_object_tio_GROUP_set},
{"LibreDWGc::_dwg_object_object_tio_GROUP_get", _wrap__dwg_object_object_tio_GROUP_get},
{"LibreDWGc::_dwg_object_object_tio_IDBUFFER_set", _wrap__dwg_object_object_tio_IDBUFFER_set},
{"LibreDWGc::_dwg_object_object_tio_IDBUFFER_get", _wrap__dwg_object_object_tio_IDBUFFER_get},
{"LibreDWGc::_dwg_object_object_tio_IMAGEDEF_set", _wrap__dwg_object_object_tio_IMAGEDEF_set},
{"LibreDWGc::_dwg_object_object_tio_IMAGEDEF_get", _wrap__dwg_object_object_tio_IMAGEDEF_get},
{"LibreDWGc::_dwg_object_object_tio_IMAGEDEF_REACTOR_set", _wrap__dwg_object_object_tio_IMAGEDEF_REACTOR_set},
{"LibreDWGc::_dwg_object_object_tio_IMAGEDEF_REACTOR_get", _wrap__dwg_object_object_tio_IMAGEDEF_REACTOR_get},
{"LibreDWGc::_dwg_object_object_tio_INDEX_set", _wrap__dwg_object_object_tio_INDEX_set},
{"LibreDWGc::_dwg_object_object_tio_INDEX_get", _wrap__dwg_object_object_tio_INDEX_get},
{"LibreDWGc::_dwg_object_object_tio_LAYERFILTER_set", _wrap__dwg_object_object_tio_LAYERFILTER_set},
{"LibreDWGc::_dwg_object_object_tio_LAYERFILTER_get", _wrap__dwg_object_object_tio_LAYERFILTER_get},
{"LibreDWGc::_dwg_object_object_tio_LAYER_INDEX_set", _wrap__dwg_object_object_tio_LAYER_INDEX_set},
{"LibreDWGc::_dwg_object_object_tio_LAYER_INDEX_get", _wrap__dwg_object_object_tio_LAYER_INDEX_get},
{"LibreDWGc::_dwg_object_object_tio_LAYOUT_set", _wrap__dwg_object_object_tio_LAYOUT_set},
{"LibreDWGc::_dwg_object_object_tio_LAYOUT_get", _wrap__dwg_object_object_tio_LAYOUT_get},
{"LibreDWGc::_dwg_object_object_tio_PLACEHOLDER_set", _wrap__dwg_object_object_tio_PLACEHOLDER_set},
{"LibreDWGc::_dwg_object_object_tio_PLACEHOLDER_get", _wrap__dwg_object_object_tio_PLACEHOLDER_get},
{"LibreDWGc::_dwg_object_object_tio_PLOTSETTINGS_set", _wrap__dwg_object_object_tio_PLOTSETTINGS_set},
{"LibreDWGc::_dwg_object_object_tio_PLOTSETTINGS_get", _wrap__dwg_object_object_tio_PLOTSETTINGS_get},
{"LibreDWGc::_dwg_object_object_tio_RASTERVARIABLES_set", _wrap__dwg_object_object_tio_RASTERVARIABLES_set},
{"LibreDWGc::_dwg_object_object_tio_RASTERVARIABLES_get", _wrap__dwg_object_object_tio_RASTERVARIABLES_get},
{"LibreDWGc::_dwg_object_object_tio_RENDERENVIRONMENT_set", _wrap__dwg_object_object_tio_RENDERENVIRONMENT_set},
{"LibreDWGc::_dwg_object_object_tio_RENDERENVIRONMENT_get", _wrap__dwg_object_object_tio_RENDERENVIRONMENT_get},
{"LibreDWGc::_dwg_object_object_tio_SCALE_set", _wrap__dwg_object_object_tio_SCALE_set},
{"LibreDWGc::_dwg_object_object_tio_SCALE_get", _wrap__dwg_object_object_tio_SCALE_get},
{"LibreDWGc::_dwg_object_object_tio_SECTIONVIEWSTYLE_set", _wrap__dwg_object_object_tio_SECTIONVIEWSTYLE_set},
{"LibreDWGc::_dwg_object_object_tio_SECTIONVIEWSTYLE_get", _wrap__dwg_object_object_tio_SECTIONVIEWSTYLE_get},
{"LibreDWGc::_dwg_object_object_tio_SECTION_MANAGER_set", _wrap__dwg_object_object_tio_SECTION_MANAGER_set},
{"LibreDWGc::_dwg_object_object_tio_SECTION_MANAGER_get", _wrap__dwg_object_object_tio_SECTION_MANAGER_get},
{"LibreDWGc::_dwg_object_object_tio_SORTENTSTABLE_set", _wrap__dwg_object_object_tio_SORTENTSTABLE_set},
{"LibreDWGc::_dwg_object_object_tio_SORTENTSTABLE_get", _wrap__dwg_object_object_tio_SORTENTSTABLE_get},
{"LibreDWGc::_dwg_object_object_tio_SPATIAL_FILTER_set", _wrap__dwg_object_object_tio_SPATIAL_FILTER_set},
{"LibreDWGc::_dwg_object_object_tio_SPATIAL_FILTER_get", _wrap__dwg_object_object_tio_SPATIAL_FILTER_get},
{"LibreDWGc::_dwg_object_object_tio_SUN_set", _wrap__dwg_object_object_tio_SUN_set},
{"LibreDWGc::_dwg_object_object_tio_SUN_get", _wrap__dwg_object_object_tio_SUN_get},
{"LibreDWGc::_dwg_object_object_tio_TABLEGEOMETRY_set", _wrap__dwg_object_object_tio_TABLEGEOMETRY_set},
{"LibreDWGc::_dwg_object_object_tio_TABLEGEOMETRY_get", _wrap__dwg_object_object_tio_TABLEGEOMETRY_get},
{"LibreDWGc::_dwg_object_object_tio_VBA_PROJECT_set", _wrap__dwg_object_object_tio_VBA_PROJECT_set},
{"LibreDWGc::_dwg_object_object_tio_VBA_PROJECT_get", _wrap__dwg_object_object_tio_VBA_PROJECT_get},
{"LibreDWGc::_dwg_object_object_tio_VISUALSTYLE_set", _wrap__dwg_object_object_tio_VISUALSTYLE_set},
{"LibreDWGc::_dwg_object_object_tio_VISUALSTYLE_get", _wrap__dwg_object_object_tio_VISUALSTYLE_get},
{"LibreDWGc::_dwg_object_object_tio_WIPEOUTVARIABLES_set", _wrap__dwg_object_object_tio_WIPEOUTVARIABLES_set},
{"LibreDWGc::_dwg_object_object_tio_WIPEOUTVARIABLES_get", _wrap__dwg_object_object_tio_WIPEOUTVARIABLES_get},
{"LibreDWGc::_dwg_object_object_tio_XRECORD_set", _wrap__dwg_object_object_tio_XRECORD_set},
{"LibreDWGc::_dwg_object_object_tio_XRECORD_get", _wrap__dwg_object_object_tio_XRECORD_get},
{"LibreDWGc::_dwg_object_object_tio_PDFDEFINITION_set", _wrap__dwg_object_object_tio_PDFDEFINITION_set},
{"LibreDWGc::_dwg_object_object_tio_PDFDEFINITION_get", _wrap__dwg_object_object_tio_PDFDEFINITION_get},
{"LibreDWGc::_dwg_object_object_tio_DGNDEFINITION_set", _wrap__dwg_object_object_tio_DGNDEFINITION_set},
{"LibreDWGc::_dwg_object_object_tio_DGNDEFINITION_get", _wrap__dwg_object_object_tio_DGNDEFINITION_get},
{"LibreDWGc::_dwg_object_object_tio_DWFDEFINITION_set", _wrap__dwg_object_object_tio_DWFDEFINITION_set},
{"LibreDWGc::_dwg_object_object_tio_DWFDEFINITION_get", _wrap__dwg_object_object_tio_DWFDEFINITION_get},
{"LibreDWGc::_dwg_object_object_tio_ACSH_BREP_CLASS_set", _wrap__dwg_object_object_tio_ACSH_BREP_CLASS_set},
{"LibreDWGc::_dwg_object_object_tio_ACSH_BREP_CLASS_get", _wrap__dwg_object_object_tio_ACSH_BREP_CLASS_get},
{"LibreDWGc::_dwg_object_object_tio_ACSH_CHAMFER_CLASS_set", _wrap__dwg_object_object_tio_ACSH_CHAMFER_CLASS_set},
{"LibreDWGc::_dwg_object_object_tio_ACSH_CHAMFER_CLASS_get", _wrap__dwg_object_object_tio_ACSH_CHAMFER_CLASS_get},
{"LibreDWGc::_dwg_object_object_tio_ACSH_PYRAMID_CLASS_set", _wrap__dwg_object_object_tio_ACSH_PYRAMID_CLASS_set},
{"LibreDWGc::_dwg_object_object_tio_ACSH_PYRAMID_CLASS_get", _wrap__dwg_object_object_tio_ACSH_PYRAMID_CLASS_get},
{"LibreDWGc::_dwg_object_object_tio_ALDIMOBJECTCONTEXTDATA_set", _wrap__dwg_object_object_tio_ALDIMOBJECTCONTEXTDATA_set},
{"LibreDWGc::_dwg_object_object_tio_ALDIMOBJECTCONTEXTDATA_get", _wrap__dwg_object_object_tio_ALDIMOBJECTCONTEXTDATA_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOC2DCONSTRAINTGROUP_set", _wrap__dwg_object_object_tio_ASSOC2DCONSTRAINTGROUP_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOC2DCONSTRAINTGROUP_get", _wrap__dwg_object_object_tio_ASSOC2DCONSTRAINTGROUP_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCACTION_set", _wrap__dwg_object_object_tio_ASSOCACTION_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCACTION_get", _wrap__dwg_object_object_tio_ASSOCACTION_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCACTIONPARAM_set", _wrap__dwg_object_object_tio_ASSOCACTIONPARAM_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCACTIONPARAM_get", _wrap__dwg_object_object_tio_ASSOCACTIONPARAM_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCARRAYACTIONBODY_set", _wrap__dwg_object_object_tio_ASSOCARRAYACTIONBODY_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCARRAYACTIONBODY_get", _wrap__dwg_object_object_tio_ASSOCARRAYACTIONBODY_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCASMBODYACTIONPARAM_set", _wrap__dwg_object_object_tio_ASSOCASMBODYACTIONPARAM_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCASMBODYACTIONPARAM_get", _wrap__dwg_object_object_tio_ASSOCASMBODYACTIONPARAM_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCBLENDSURFACEACTIONBODY_set", _wrap__dwg_object_object_tio_ASSOCBLENDSURFACEACTIONBODY_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCBLENDSURFACEACTIONBODY_get", _wrap__dwg_object_object_tio_ASSOCBLENDSURFACEACTIONBODY_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCCOMPOUNDACTIONPARAM_set", _wrap__dwg_object_object_tio_ASSOCCOMPOUNDACTIONPARAM_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCCOMPOUNDACTIONPARAM_get", _wrap__dwg_object_object_tio_ASSOCCOMPOUNDACTIONPARAM_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCDEPENDENCY_set", _wrap__dwg_object_object_tio_ASSOCDEPENDENCY_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCDEPENDENCY_get", _wrap__dwg_object_object_tio_ASSOCDEPENDENCY_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCDIMDEPENDENCYBODY_set", _wrap__dwg_object_object_tio_ASSOCDIMDEPENDENCYBODY_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCDIMDEPENDENCYBODY_get", _wrap__dwg_object_object_tio_ASSOCDIMDEPENDENCYBODY_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCEXTENDSURFACEACTIONBODY_set", _wrap__dwg_object_object_tio_ASSOCEXTENDSURFACEACTIONBODY_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCEXTENDSURFACEACTIONBODY_get", _wrap__dwg_object_object_tio_ASSOCEXTENDSURFACEACTIONBODY_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCEXTRUDEDSURFACEACTIONBODY_set", _wrap__dwg_object_object_tio_ASSOCEXTRUDEDSURFACEACTIONBODY_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCEXTRUDEDSURFACEACTIONBODY_get", _wrap__dwg_object_object_tio_ASSOCEXTRUDEDSURFACEACTIONBODY_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCFACEACTIONPARAM_set", _wrap__dwg_object_object_tio_ASSOCFACEACTIONPARAM_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCFACEACTIONPARAM_get", _wrap__dwg_object_object_tio_ASSOCFACEACTIONPARAM_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCFILLETSURFACEACTIONBODY_set", _wrap__dwg_object_object_tio_ASSOCFILLETSURFACEACTIONBODY_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCFILLETSURFACEACTIONBODY_get", _wrap__dwg_object_object_tio_ASSOCFILLETSURFACEACTIONBODY_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCLOFTEDSURFACEACTIONBODY_set", _wrap__dwg_object_object_tio_ASSOCLOFTEDSURFACEACTIONBODY_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCLOFTEDSURFACEACTIONBODY_get", _wrap__dwg_object_object_tio_ASSOCLOFTEDSURFACEACTIONBODY_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCNETWORKSURFACEACTIONBODY_set", _wrap__dwg_object_object_tio_ASSOCNETWORKSURFACEACTIONBODY_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCNETWORKSURFACEACTIONBODY_get", _wrap__dwg_object_object_tio_ASSOCNETWORKSURFACEACTIONBODY_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCOBJECTACTIONPARAM_set", _wrap__dwg_object_object_tio_ASSOCOBJECTACTIONPARAM_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCOBJECTACTIONPARAM_get", _wrap__dwg_object_object_tio_ASSOCOBJECTACTIONPARAM_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCOFFSETSURFACEACTIONBODY_set", _wrap__dwg_object_object_tio_ASSOCOFFSETSURFACEACTIONBODY_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCOFFSETSURFACEACTIONBODY_get", _wrap__dwg_object_object_tio_ASSOCOFFSETSURFACEACTIONBODY_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCOSNAPPOINTREFACTIONPARAM_set", _wrap__dwg_object_object_tio_ASSOCOSNAPPOINTREFACTIONPARAM_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCOSNAPPOINTREFACTIONPARAM_get", _wrap__dwg_object_object_tio_ASSOCOSNAPPOINTREFACTIONPARAM_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCPATCHSURFACEACTIONBODY_set", _wrap__dwg_object_object_tio_ASSOCPATCHSURFACEACTIONBODY_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCPATCHSURFACEACTIONBODY_get", _wrap__dwg_object_object_tio_ASSOCPATCHSURFACEACTIONBODY_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCPATHACTIONPARAM_set", _wrap__dwg_object_object_tio_ASSOCPATHACTIONPARAM_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCPATHACTIONPARAM_get", _wrap__dwg_object_object_tio_ASSOCPATHACTIONPARAM_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCPLANESURFACEACTIONBODY_set", _wrap__dwg_object_object_tio_ASSOCPLANESURFACEACTIONBODY_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCPLANESURFACEACTIONBODY_get", _wrap__dwg_object_object_tio_ASSOCPLANESURFACEACTIONBODY_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCPOINTREFACTIONPARAM_set", _wrap__dwg_object_object_tio_ASSOCPOINTREFACTIONPARAM_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCPOINTREFACTIONPARAM_get", _wrap__dwg_object_object_tio_ASSOCPOINTREFACTIONPARAM_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCREVOLVEDSURFACEACTIONBODY_set", _wrap__dwg_object_object_tio_ASSOCREVOLVEDSURFACEACTIONBODY_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCREVOLVEDSURFACEACTIONBODY_get", _wrap__dwg_object_object_tio_ASSOCREVOLVEDSURFACEACTIONBODY_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCTRIMSURFACEACTIONBODY_set", _wrap__dwg_object_object_tio_ASSOCTRIMSURFACEACTIONBODY_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCTRIMSURFACEACTIONBODY_get", _wrap__dwg_object_object_tio_ASSOCTRIMSURFACEACTIONBODY_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCVALUEDEPENDENCY_set", _wrap__dwg_object_object_tio_ASSOCVALUEDEPENDENCY_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCVALUEDEPENDENCY_get", _wrap__dwg_object_object_tio_ASSOCVALUEDEPENDENCY_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCVARIABLE_set", _wrap__dwg_object_object_tio_ASSOCVARIABLE_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCVARIABLE_get", _wrap__dwg_object_object_tio_ASSOCVARIABLE_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCVERTEXACTIONPARAM_set", _wrap__dwg_object_object_tio_ASSOCVERTEXACTIONPARAM_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCVERTEXACTIONPARAM_get", _wrap__dwg_object_object_tio_ASSOCVERTEXACTIONPARAM_get},
{"LibreDWGc::_dwg_object_object_tio_BLKREFOBJECTCONTEXTDATA_set", _wrap__dwg_object_object_tio_BLKREFOBJECTCONTEXTDATA_set},
{"LibreDWGc::_dwg_object_object_tio_BLKREFOBJECTCONTEXTDATA_get", _wrap__dwg_object_object_tio_BLKREFOBJECTCONTEXTDATA_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKALIGNEDCONSTRAINTPARAMETER_set", _wrap__dwg_object_object_tio_BLOCKALIGNEDCONSTRAINTPARAMETER_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKALIGNEDCONSTRAINTPARAMETER_get", _wrap__dwg_object_object_tio_BLOCKALIGNEDCONSTRAINTPARAMETER_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKANGULARCONSTRAINTPARAMETER_set", _wrap__dwg_object_object_tio_BLOCKANGULARCONSTRAINTPARAMETER_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKANGULARCONSTRAINTPARAMETER_get", _wrap__dwg_object_object_tio_BLOCKANGULARCONSTRAINTPARAMETER_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKARRAYACTION_set", _wrap__dwg_object_object_tio_BLOCKARRAYACTION_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKARRAYACTION_get", _wrap__dwg_object_object_tio_BLOCKARRAYACTION_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKDIAMETRICCONSTRAINTPARAMETER_set", _wrap__dwg_object_object_tio_BLOCKDIAMETRICCONSTRAINTPARAMETER_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKDIAMETRICCONSTRAINTPARAMETER_get", _wrap__dwg_object_object_tio_BLOCKDIAMETRICCONSTRAINTPARAMETER_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKHORIZONTALCONSTRAINTPARAMETER_set", _wrap__dwg_object_object_tio_BLOCKHORIZONTALCONSTRAINTPARAMETER_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKHORIZONTALCONSTRAINTPARAMETER_get", _wrap__dwg_object_object_tio_BLOCKHORIZONTALCONSTRAINTPARAMETER_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKLINEARCONSTRAINTPARAMETER_set", _wrap__dwg_object_object_tio_BLOCKLINEARCONSTRAINTPARAMETER_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKLINEARCONSTRAINTPARAMETER_get", _wrap__dwg_object_object_tio_BLOCKLINEARCONSTRAINTPARAMETER_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKLINEARPARAMETER_set", _wrap__dwg_object_object_tio_BLOCKLINEARPARAMETER_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKLINEARPARAMETER_get", _wrap__dwg_object_object_tio_BLOCKLINEARPARAMETER_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKLOOKUPACTION_set", _wrap__dwg_object_object_tio_BLOCKLOOKUPACTION_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKLOOKUPACTION_get", _wrap__dwg_object_object_tio_BLOCKLOOKUPACTION_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKLOOKUPPARAMETER_set", _wrap__dwg_object_object_tio_BLOCKLOOKUPPARAMETER_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKLOOKUPPARAMETER_get", _wrap__dwg_object_object_tio_BLOCKLOOKUPPARAMETER_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKPARAMDEPENDENCYBODY_set", _wrap__dwg_object_object_tio_BLOCKPARAMDEPENDENCYBODY_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKPARAMDEPENDENCYBODY_get", _wrap__dwg_object_object_tio_BLOCKPARAMDEPENDENCYBODY_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKPOINTPARAMETER_set", _wrap__dwg_object_object_tio_BLOCKPOINTPARAMETER_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKPOINTPARAMETER_get", _wrap__dwg_object_object_tio_BLOCKPOINTPARAMETER_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKPOLARGRIP_set", _wrap__dwg_object_object_tio_BLOCKPOLARGRIP_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKPOLARGRIP_get", _wrap__dwg_object_object_tio_BLOCKPOLARGRIP_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKPOLARPARAMETER_set", _wrap__dwg_object_object_tio_BLOCKPOLARPARAMETER_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKPOLARPARAMETER_get", _wrap__dwg_object_object_tio_BLOCKPOLARPARAMETER_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKPOLARSTRETCHACTION_set", _wrap__dwg_object_object_tio_BLOCKPOLARSTRETCHACTION_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKPOLARSTRETCHACTION_get", _wrap__dwg_object_object_tio_BLOCKPOLARSTRETCHACTION_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKRADIALCONSTRAINTPARAMETER_set", _wrap__dwg_object_object_tio_BLOCKRADIALCONSTRAINTPARAMETER_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKRADIALCONSTRAINTPARAMETER_get", _wrap__dwg_object_object_tio_BLOCKRADIALCONSTRAINTPARAMETER_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKREPRESENTATION_set", _wrap__dwg_object_object_tio_BLOCKREPRESENTATION_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKREPRESENTATION_get", _wrap__dwg_object_object_tio_BLOCKREPRESENTATION_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKROTATIONPARAMETER_set", _wrap__dwg_object_object_tio_BLOCKROTATIONPARAMETER_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKROTATIONPARAMETER_get", _wrap__dwg_object_object_tio_BLOCKROTATIONPARAMETER_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKSTRETCHACTION_set", _wrap__dwg_object_object_tio_BLOCKSTRETCHACTION_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKSTRETCHACTION_get", _wrap__dwg_object_object_tio_BLOCKSTRETCHACTION_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKUSERPARAMETER_set", _wrap__dwg_object_object_tio_BLOCKUSERPARAMETER_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKUSERPARAMETER_get", _wrap__dwg_object_object_tio_BLOCKUSERPARAMETER_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKVERTICALCONSTRAINTPARAMETER_set", _wrap__dwg_object_object_tio_BLOCKVERTICALCONSTRAINTPARAMETER_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKVERTICALCONSTRAINTPARAMETER_get", _wrap__dwg_object_object_tio_BLOCKVERTICALCONSTRAINTPARAMETER_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKVISIBILITYPARAMETER_set", _wrap__dwg_object_object_tio_BLOCKVISIBILITYPARAMETER_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKVISIBILITYPARAMETER_get", _wrap__dwg_object_object_tio_BLOCKVISIBILITYPARAMETER_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKXYGRIP_set", _wrap__dwg_object_object_tio_BLOCKXYGRIP_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKXYGRIP_get", _wrap__dwg_object_object_tio_BLOCKXYGRIP_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKXYPARAMETER_set", _wrap__dwg_object_object_tio_BLOCKXYPARAMETER_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKXYPARAMETER_get", _wrap__dwg_object_object_tio_BLOCKXYPARAMETER_get},
{"LibreDWGc::_dwg_object_object_tio_DATALINK_set", _wrap__dwg_object_object_tio_DATALINK_set},
{"LibreDWGc::_dwg_object_object_tio_DATALINK_get", _wrap__dwg_object_object_tio_DATALINK_get},
{"LibreDWGc::_dwg_object_object_tio_DBCOLOR_set", _wrap__dwg_object_object_tio_DBCOLOR_set},
{"LibreDWGc::_dwg_object_object_tio_DBCOLOR_get", _wrap__dwg_object_object_tio_DBCOLOR_get},
{"LibreDWGc::_dwg_object_object_tio_EVALUATION_GRAPH_set", _wrap__dwg_object_object_tio_EVALUATION_GRAPH_set},
{"LibreDWGc::_dwg_object_object_tio_EVALUATION_GRAPH_get", _wrap__dwg_object_object_tio_EVALUATION_GRAPH_get},
{"LibreDWGc::_dwg_object_object_tio_FCFOBJECTCONTEXTDATA_set", _wrap__dwg_object_object_tio_FCFOBJECTCONTEXTDATA_set},
{"LibreDWGc::_dwg_object_object_tio_FCFOBJECTCONTEXTDATA_get", _wrap__dwg_object_object_tio_FCFOBJECTCONTEXTDATA_get},
{"LibreDWGc::_dwg_object_object_tio_GRADIENT_BACKGROUND_set", _wrap__dwg_object_object_tio_GRADIENT_BACKGROUND_set},
{"LibreDWGc::_dwg_object_object_tio_GRADIENT_BACKGROUND_get", _wrap__dwg_object_object_tio_GRADIENT_BACKGROUND_get},
{"LibreDWGc::_dwg_object_object_tio_GROUND_PLANE_BACKGROUND_set", _wrap__dwg_object_object_tio_GROUND_PLANE_BACKGROUND_set},
{"LibreDWGc::_dwg_object_object_tio_GROUND_PLANE_BACKGROUND_get", _wrap__dwg_object_object_tio_GROUND_PLANE_BACKGROUND_get},
{"LibreDWGc::_dwg_object_object_tio_IBL_BACKGROUND_set", _wrap__dwg_object_object_tio_IBL_BACKGROUND_set},
{"LibreDWGc::_dwg_object_object_tio_IBL_BACKGROUND_get", _wrap__dwg_object_object_tio_IBL_BACKGROUND_get},
{"LibreDWGc::_dwg_object_object_tio_IMAGE_BACKGROUND_set", _wrap__dwg_object_object_tio_IMAGE_BACKGROUND_set},
{"LibreDWGc::_dwg_object_object_tio_IMAGE_BACKGROUND_get", _wrap__dwg_object_object_tio_IMAGE_BACKGROUND_get},
{"LibreDWGc::_dwg_object_object_tio_LEADEROBJECTCONTEXTDATA_set", _wrap__dwg_object_object_tio_LEADEROBJECTCONTEXTDATA_set},
{"LibreDWGc::_dwg_object_object_tio_LEADEROBJECTCONTEXTDATA_get", _wrap__dwg_object_object_tio_LEADEROBJECTCONTEXTDATA_get},
{"LibreDWGc::_dwg_object_object_tio_LIGHTLIST_set", _wrap__dwg_object_object_tio_LIGHTLIST_set},
{"LibreDWGc::_dwg_object_object_tio_LIGHTLIST_get", _wrap__dwg_object_object_tio_LIGHTLIST_get},
{"LibreDWGc::_dwg_object_object_tio_MATERIAL_set", _wrap__dwg_object_object_tio_MATERIAL_set},
{"LibreDWGc::_dwg_object_object_tio_MATERIAL_get", _wrap__dwg_object_object_tio_MATERIAL_get},
{"LibreDWGc::_dwg_object_object_tio_MENTALRAYRENDERSETTINGS_set", _wrap__dwg_object_object_tio_MENTALRAYRENDERSETTINGS_set},
{"LibreDWGc::_dwg_object_object_tio_MENTALRAYRENDERSETTINGS_get", _wrap__dwg_object_object_tio_MENTALRAYRENDERSETTINGS_get},
{"LibreDWGc::_dwg_object_object_tio_MLEADERSTYLE_set", _wrap__dwg_object_object_tio_MLEADERSTYLE_set},
{"LibreDWGc::_dwg_object_object_tio_MLEADERSTYLE_get", _wrap__dwg_object_object_tio_MLEADERSTYLE_get},
{"LibreDWGc::_dwg_object_object_tio_MTEXTOBJECTCONTEXTDATA_set", _wrap__dwg_object_object_tio_MTEXTOBJECTCONTEXTDATA_set},
{"LibreDWGc::_dwg_object_object_tio_MTEXTOBJECTCONTEXTDATA_get", _wrap__dwg_object_object_tio_MTEXTOBJECTCONTEXTDATA_get},
{"LibreDWGc::_dwg_object_object_tio_OBJECT_PTR_set", _wrap__dwg_object_object_tio_OBJECT_PTR_set},
{"LibreDWGc::_dwg_object_object_tio_OBJECT_PTR_get", _wrap__dwg_object_object_tio_OBJECT_PTR_get},
{"LibreDWGc::_dwg_object_object_tio_PARTIAL_VIEWING_INDEX_set", _wrap__dwg_object_object_tio_PARTIAL_VIEWING_INDEX_set},
{"LibreDWGc::_dwg_object_object_tio_PARTIAL_VIEWING_INDEX_get", _wrap__dwg_object_object_tio_PARTIAL_VIEWING_INDEX_get},
{"LibreDWGc::_dwg_object_object_tio_POINTCLOUDCOLORMAP_set", _wrap__dwg_object_object_tio_POINTCLOUDCOLORMAP_set},
{"LibreDWGc::_dwg_object_object_tio_POINTCLOUDCOLORMAP_get", _wrap__dwg_object_object_tio_POINTCLOUDCOLORMAP_get},
{"LibreDWGc::_dwg_object_object_tio_POINTCLOUDDEF_set", _wrap__dwg_object_object_tio_POINTCLOUDDEF_set},
{"LibreDWGc::_dwg_object_object_tio_POINTCLOUDDEF_get", _wrap__dwg_object_object_tio_POINTCLOUDDEF_get},
{"LibreDWGc::_dwg_object_object_tio_POINTCLOUDDEFEX_set", _wrap__dwg_object_object_tio_POINTCLOUDDEFEX_set},
{"LibreDWGc::_dwg_object_object_tio_POINTCLOUDDEFEX_get", _wrap__dwg_object_object_tio_POINTCLOUDDEFEX_get},
{"LibreDWGc::_dwg_object_object_tio_POINTCLOUDDEF_REACTOR_set", _wrap__dwg_object_object_tio_POINTCLOUDDEF_REACTOR_set},
{"LibreDWGc::_dwg_object_object_tio_POINTCLOUDDEF_REACTOR_get", _wrap__dwg_object_object_tio_POINTCLOUDDEF_REACTOR_get},
{"LibreDWGc::_dwg_object_object_tio_POINTCLOUDDEF_REACTOR_EX_set", _wrap__dwg_object_object_tio_POINTCLOUDDEF_REACTOR_EX_set},
{"LibreDWGc::_dwg_object_object_tio_POINTCLOUDDEF_REACTOR_EX_get", _wrap__dwg_object_object_tio_POINTCLOUDDEF_REACTOR_EX_get},
{"LibreDWGc::_dwg_object_object_tio_PROXY_OBJECT_set", _wrap__dwg_object_object_tio_PROXY_OBJECT_set},
{"LibreDWGc::_dwg_object_object_tio_PROXY_OBJECT_get", _wrap__dwg_object_object_tio_PROXY_OBJECT_get},
{"LibreDWGc::_dwg_object_object_tio_RAPIDRTRENDERSETTINGS_set", _wrap__dwg_object_object_tio_RAPIDRTRENDERSETTINGS_set},
{"LibreDWGc::_dwg_object_object_tio_RAPIDRTRENDERSETTINGS_get", _wrap__dwg_object_object_tio_RAPIDRTRENDERSETTINGS_get},
{"LibreDWGc::_dwg_object_object_tio_RENDERENTRY_set", _wrap__dwg_object_object_tio_RENDERENTRY_set},
{"LibreDWGc::_dwg_object_object_tio_RENDERENTRY_get", _wrap__dwg_object_object_tio_RENDERENTRY_get},
{"LibreDWGc::_dwg_object_object_tio_RENDERGLOBAL_set", _wrap__dwg_object_object_tio_RENDERGLOBAL_set},
{"LibreDWGc::_dwg_object_object_tio_RENDERGLOBAL_get", _wrap__dwg_object_object_tio_RENDERGLOBAL_get},
{"LibreDWGc::_dwg_object_object_tio_RENDERSETTINGS_set", _wrap__dwg_object_object_tio_RENDERSETTINGS_set},
{"LibreDWGc::_dwg_object_object_tio_RENDERSETTINGS_get", _wrap__dwg_object_object_tio_RENDERSETTINGS_get},
{"LibreDWGc::_dwg_object_object_tio_SECTION_SETTINGS_set", _wrap__dwg_object_object_tio_SECTION_SETTINGS_set},
{"LibreDWGc::_dwg_object_object_tio_SECTION_SETTINGS_get", _wrap__dwg_object_object_tio_SECTION_SETTINGS_get},
{"LibreDWGc::_dwg_object_object_tio_SKYLIGHT_BACKGROUND_set", _wrap__dwg_object_object_tio_SKYLIGHT_BACKGROUND_set},
{"LibreDWGc::_dwg_object_object_tio_SKYLIGHT_BACKGROUND_get", _wrap__dwg_object_object_tio_SKYLIGHT_BACKGROUND_get},
{"LibreDWGc::_dwg_object_object_tio_SOLID_BACKGROUND_set", _wrap__dwg_object_object_tio_SOLID_BACKGROUND_set},
{"LibreDWGc::_dwg_object_object_tio_SOLID_BACKGROUND_get", _wrap__dwg_object_object_tio_SOLID_BACKGROUND_get},
{"LibreDWGc::_dwg_object_object_tio_SPATIAL_INDEX_set", _wrap__dwg_object_object_tio_SPATIAL_INDEX_set},
{"LibreDWGc::_dwg_object_object_tio_SPATIAL_INDEX_get", _wrap__dwg_object_object_tio_SPATIAL_INDEX_get},
{"LibreDWGc::_dwg_object_object_tio_TABLESTYLE_set", _wrap__dwg_object_object_tio_TABLESTYLE_set},
{"LibreDWGc::_dwg_object_object_tio_TABLESTYLE_get", _wrap__dwg_object_object_tio_TABLESTYLE_get},
{"LibreDWGc::_dwg_object_object_tio_TEXTOBJECTCONTEXTDATA_set", _wrap__dwg_object_object_tio_TEXTOBJECTCONTEXTDATA_set},
{"LibreDWGc::_dwg_object_object_tio_TEXTOBJECTCONTEXTDATA_get", _wrap__dwg_object_object_tio_TEXTOBJECTCONTEXTDATA_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCARRAYMODIFYPARAMETERS_set", _wrap__dwg_object_object_tio_ASSOCARRAYMODIFYPARAMETERS_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCARRAYMODIFYPARAMETERS_get", _wrap__dwg_object_object_tio_ASSOCARRAYMODIFYPARAMETERS_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCARRAYPATHPARAMETERS_set", _wrap__dwg_object_object_tio_ASSOCARRAYPATHPARAMETERS_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCARRAYPATHPARAMETERS_get", _wrap__dwg_object_object_tio_ASSOCARRAYPATHPARAMETERS_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCARRAYPOLARPARAMETERS_set", _wrap__dwg_object_object_tio_ASSOCARRAYPOLARPARAMETERS_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCARRAYPOLARPARAMETERS_get", _wrap__dwg_object_object_tio_ASSOCARRAYPOLARPARAMETERS_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCARRAYRECTANGULARPARAMETERS_set", _wrap__dwg_object_object_tio_ASSOCARRAYRECTANGULARPARAMETERS_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCARRAYRECTANGULARPARAMETERS_get", _wrap__dwg_object_object_tio_ASSOCARRAYRECTANGULARPARAMETERS_get},
{"LibreDWGc::_dwg_object_object_tio_ACMECOMMANDHISTORY_set", _wrap__dwg_object_object_tio_ACMECOMMANDHISTORY_set},
{"LibreDWGc::_dwg_object_object_tio_ACMECOMMANDHISTORY_get", _wrap__dwg_object_object_tio_ACMECOMMANDHISTORY_get},
{"LibreDWGc::_dwg_object_object_tio_ACMESCOPE_set", _wrap__dwg_object_object_tio_ACMESCOPE_set},
{"LibreDWGc::_dwg_object_object_tio_ACMESCOPE_get", _wrap__dwg_object_object_tio_ACMESCOPE_get},
{"LibreDWGc::_dwg_object_object_tio_ACMESTATEMGR_set", _wrap__dwg_object_object_tio_ACMESTATEMGR_set},
{"LibreDWGc::_dwg_object_object_tio_ACMESTATEMGR_get", _wrap__dwg_object_object_tio_ACMESTATEMGR_get},
{"LibreDWGc::_dwg_object_object_tio_ACSH_EXTRUSION_CLASS_set", _wrap__dwg_object_object_tio_ACSH_EXTRUSION_CLASS_set},
{"LibreDWGc::_dwg_object_object_tio_ACSH_EXTRUSION_CLASS_get", _wrap__dwg_object_object_tio_ACSH_EXTRUSION_CLASS_get},
{"LibreDWGc::_dwg_object_object_tio_ACSH_LOFT_CLASS_set", _wrap__dwg_object_object_tio_ACSH_LOFT_CLASS_set},
{"LibreDWGc::_dwg_object_object_tio_ACSH_LOFT_CLASS_get", _wrap__dwg_object_object_tio_ACSH_LOFT_CLASS_get},
{"LibreDWGc::_dwg_object_object_tio_ACSH_REVOLVE_CLASS_set", _wrap__dwg_object_object_tio_ACSH_REVOLVE_CLASS_set},
{"LibreDWGc::_dwg_object_object_tio_ACSH_REVOLVE_CLASS_get", _wrap__dwg_object_object_tio_ACSH_REVOLVE_CLASS_get},
{"LibreDWGc::_dwg_object_object_tio_ACSH_SWEEP_CLASS_set", _wrap__dwg_object_object_tio_ACSH_SWEEP_CLASS_set},
{"LibreDWGc::_dwg_object_object_tio_ACSH_SWEEP_CLASS_get", _wrap__dwg_object_object_tio_ACSH_SWEEP_CLASS_get},
{"LibreDWGc::_dwg_object_object_tio_ANGDIMOBJECTCONTEXTDATA_set", _wrap__dwg_object_object_tio_ANGDIMOBJECTCONTEXTDATA_set},
{"LibreDWGc::_dwg_object_object_tio_ANGDIMOBJECTCONTEXTDATA_get", _wrap__dwg_object_object_tio_ANGDIMOBJECTCONTEXTDATA_get},
{"LibreDWGc::_dwg_object_object_tio_ANNOTSCALEOBJECTCONTEXTDATA_set", _wrap__dwg_object_object_tio_ANNOTSCALEOBJECTCONTEXTDATA_set},
{"LibreDWGc::_dwg_object_object_tio_ANNOTSCALEOBJECTCONTEXTDATA_get", _wrap__dwg_object_object_tio_ANNOTSCALEOBJECTCONTEXTDATA_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOC3POINTANGULARDIMACTIONBODY_set", _wrap__dwg_object_object_tio_ASSOC3POINTANGULARDIMACTIONBODY_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOC3POINTANGULARDIMACTIONBODY_get", _wrap__dwg_object_object_tio_ASSOC3POINTANGULARDIMACTIONBODY_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCALIGNEDDIMACTIONBODY_set", _wrap__dwg_object_object_tio_ASSOCALIGNEDDIMACTIONBODY_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCALIGNEDDIMACTIONBODY_get", _wrap__dwg_object_object_tio_ASSOCALIGNEDDIMACTIONBODY_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCARRAYMODIFYACTIONBODY_set", _wrap__dwg_object_object_tio_ASSOCARRAYMODIFYACTIONBODY_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCARRAYMODIFYACTIONBODY_get", _wrap__dwg_object_object_tio_ASSOCARRAYMODIFYACTIONBODY_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCEDGEACTIONPARAM_set", _wrap__dwg_object_object_tio_ASSOCEDGEACTIONPARAM_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCEDGEACTIONPARAM_get", _wrap__dwg_object_object_tio_ASSOCEDGEACTIONPARAM_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCEDGECHAMFERACTIONBODY_set", _wrap__dwg_object_object_tio_ASSOCEDGECHAMFERACTIONBODY_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCEDGECHAMFERACTIONBODY_get", _wrap__dwg_object_object_tio_ASSOCEDGECHAMFERACTIONBODY_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCEDGEFILLETACTIONBODY_set", _wrap__dwg_object_object_tio_ASSOCEDGEFILLETACTIONBODY_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCEDGEFILLETACTIONBODY_get", _wrap__dwg_object_object_tio_ASSOCEDGEFILLETACTIONBODY_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCMLEADERACTIONBODY_set", _wrap__dwg_object_object_tio_ASSOCMLEADERACTIONBODY_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCMLEADERACTIONBODY_get", _wrap__dwg_object_object_tio_ASSOCMLEADERACTIONBODY_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCORDINATEDIMACTIONBODY_set", _wrap__dwg_object_object_tio_ASSOCORDINATEDIMACTIONBODY_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCORDINATEDIMACTIONBODY_get", _wrap__dwg_object_object_tio_ASSOCORDINATEDIMACTIONBODY_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCPERSSUBENTMANAGER_set", _wrap__dwg_object_object_tio_ASSOCPERSSUBENTMANAGER_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCPERSSUBENTMANAGER_get", _wrap__dwg_object_object_tio_ASSOCPERSSUBENTMANAGER_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCRESTOREENTITYSTATEACTIONBODY_set", _wrap__dwg_object_object_tio_ASSOCRESTOREENTITYSTATEACTIONBODY_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCRESTOREENTITYSTATEACTIONBODY_get", _wrap__dwg_object_object_tio_ASSOCRESTOREENTITYSTATEACTIONBODY_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCROTATEDDIMACTIONBODY_set", _wrap__dwg_object_object_tio_ASSOCROTATEDDIMACTIONBODY_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCROTATEDDIMACTIONBODY_get", _wrap__dwg_object_object_tio_ASSOCROTATEDDIMACTIONBODY_get},
{"LibreDWGc::_dwg_object_object_tio_ASSOCSWEPTSURFACEACTIONBODY_set", _wrap__dwg_object_object_tio_ASSOCSWEPTSURFACEACTIONBODY_set},
{"LibreDWGc::_dwg_object_object_tio_ASSOCSWEPTSURFACEACTIONBODY_get", _wrap__dwg_object_object_tio_ASSOCSWEPTSURFACEACTIONBODY_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKPROPERTIESTABLE_set", _wrap__dwg_object_object_tio_BLOCKPROPERTIESTABLE_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKPROPERTIESTABLE_get", _wrap__dwg_object_object_tio_BLOCKPROPERTIESTABLE_get},
{"LibreDWGc::_dwg_object_object_tio_BLOCKPROPERTIESTABLEGRIP_set", _wrap__dwg_object_object_tio_BLOCKPROPERTIESTABLEGRIP_set},
{"LibreDWGc::_dwg_object_object_tio_BLOCKPROPERTIESTABLEGRIP_get", _wrap__dwg_object_object_tio_BLOCKPROPERTIESTABLEGRIP_get},
{"LibreDWGc::_dwg_object_object_tio_BREAKDATA_set", _wrap__dwg_object_object_tio_BREAKDATA_set},
{"LibreDWGc::_dwg_object_object_tio_BREAKDATA_get", _wrap__dwg_object_object_tio_BREAKDATA_get},
{"LibreDWGc::_dwg_object_object_tio_BREAKPOINTREF_set", _wrap__dwg_object_object_tio_BREAKPOINTREF_set},
{"LibreDWGc::_dwg_object_object_tio_BREAKPOINTREF_get", _wrap__dwg_object_object_tio_BREAKPOINTREF_get},
{"LibreDWGc::_dwg_object_object_tio_CONTEXTDATAMANAGER_set", _wrap__dwg_object_object_tio_CONTEXTDATAMANAGER_set},
{"LibreDWGc::_dwg_object_object_tio_CONTEXTDATAMANAGER_get", _wrap__dwg_object_object_tio_CONTEXTDATAMANAGER_get},
{"LibreDWGc::_dwg_object_object_tio_CSACDOCUMENTOPTIONS_set", _wrap__dwg_object_object_tio_CSACDOCUMENTOPTIONS_set},
{"LibreDWGc::_dwg_object_object_tio_CSACDOCUMENTOPTIONS_get", _wrap__dwg_object_object_tio_CSACDOCUMENTOPTIONS_get},
{"LibreDWGc::_dwg_object_object_tio_CURVEPATH_set", _wrap__dwg_object_object_tio_CURVEPATH_set},
{"LibreDWGc::_dwg_object_object_tio_CURVEPATH_get", _wrap__dwg_object_object_tio_CURVEPATH_get},
{"LibreDWGc::_dwg_object_object_tio_DATATABLE_set", _wrap__dwg_object_object_tio_DATATABLE_set},
{"LibreDWGc::_dwg_object_object_tio_DATATABLE_get", _wrap__dwg_object_object_tio_DATATABLE_get},
{"LibreDWGc::_dwg_object_object_tio_DIMASSOC_set", _wrap__dwg_object_object_tio_DIMASSOC_set},
{"LibreDWGc::_dwg_object_object_tio_DIMASSOC_get", _wrap__dwg_object_object_tio_DIMASSOC_get},
{"LibreDWGc::_dwg_object_object_tio_DMDIMOBJECTCONTEXTDATA_set", _wrap__dwg_object_object_tio_DMDIMOBJECTCONTEXTDATA_set},
{"LibreDWGc::_dwg_object_object_tio_DMDIMOBJECTCONTEXTDATA_get", _wrap__dwg_object_object_tio_DMDIMOBJECTCONTEXTDATA_get},
{"LibreDWGc::_dwg_object_object_tio_DYNAMICBLOCKPROXYNODE_set", _wrap__dwg_object_object_tio_DYNAMICBLOCKPROXYNODE_set},
{"LibreDWGc::_dwg_object_object_tio_DYNAMICBLOCKPROXYNODE_get", _wrap__dwg_object_object_tio_DYNAMICBLOCKPROXYNODE_get},
{"LibreDWGc::_dwg_object_object_tio_GEOMAPIMAGE_set", _wrap__dwg_object_object_tio_GEOMAPIMAGE_set},
{"LibreDWGc::_dwg_object_object_tio_GEOMAPIMAGE_get", _wrap__dwg_object_object_tio_GEOMAPIMAGE_get},
{"LibreDWGc::_dwg_object_object_tio_MLEADEROBJECTCONTEXTDATA_set", _wrap__dwg_object_object_tio_MLEADEROBJECTCONTEXTDATA_set},
{"LibreDWGc::_dwg_object_object_tio_MLEADEROBJECTCONTEXTDATA_get", _wrap__dwg_object_object_tio_MLEADEROBJECTCONTEXTDATA_get},
{"LibreDWGc::_dwg_object_object_tio_MOTIONPATH_set", _wrap__dwg_object_object_tio_MOTIONPATH_set},
{"LibreDWGc::_dwg_object_object_tio_MOTIONPATH_get", _wrap__dwg_object_object_tio_MOTIONPATH_get},
{"LibreDWGc::_dwg_object_object_tio_MTEXTATTRIBUTEOBJECTCONTEXTDATA_set", _wrap__dwg_object_object_tio_MTEXTATTRIBUTEOBJECTCONTEXTDATA_set},
{"LibreDWGc::_dwg_object_object_tio_MTEXTATTRIBUTEOBJECTCONTEXTDATA_get", _wrap__dwg_object_object_tio_MTEXTATTRIBUTEOBJECTCONTEXTDATA_get},
{"LibreDWGc::_dwg_object_object_tio_NAVISWORKSMODELDEF_set", _wrap__dwg_object_object_tio_NAVISWORKSMODELDEF_set},
{"LibreDWGc::_dwg_object_object_tio_NAVISWORKSMODELDEF_get", _wrap__dwg_object_object_tio_NAVISWORKSMODELDEF_get},
{"LibreDWGc::_dwg_object_object_tio_ORDDIMOBJECTCONTEXTDATA_set", _wrap__dwg_object_object_tio_ORDDIMOBJECTCONTEXTDATA_set},
{"LibreDWGc::_dwg_object_object_tio_ORDDIMOBJECTCONTEXTDATA_get", _wrap__dwg_object_object_tio_ORDDIMOBJECTCONTEXTDATA_get},
{"LibreDWGc::_dwg_object_object_tio_PERSUBENTMGR_set", _wrap__dwg_object_object_tio_PERSUBENTMGR_set},
{"LibreDWGc::_dwg_object_object_tio_PERSUBENTMGR_get", _wrap__dwg_object_object_tio_PERSUBENTMGR_get},
{"LibreDWGc::_dwg_object_object_tio_POINTPATH_set", _wrap__dwg_object_object_tio_POINTPATH_set},
{"LibreDWGc::_dwg_object_object_tio_POINTPATH_get", _wrap__dwg_object_object_tio_POINTPATH_get},
{"LibreDWGc::_dwg_object_object_tio_RADIMLGOBJECTCONTEXTDATA_set", _wrap__dwg_object_object_tio_RADIMLGOBJECTCONTEXTDATA_set},
{"LibreDWGc::_dwg_object_object_tio_RADIMLGOBJECTCONTEXTDATA_get", _wrap__dwg_object_object_tio_RADIMLGOBJECTCONTEXTDATA_get},
{"LibreDWGc::_dwg_object_object_tio_RADIMOBJECTCONTEXTDATA_set", _wrap__dwg_object_object_tio_RADIMOBJECTCONTEXTDATA_set},
{"LibreDWGc::_dwg_object_object_tio_RADIMOBJECTCONTEXTDATA_get", _wrap__dwg_object_object_tio_RADIMOBJECTCONTEXTDATA_get},
{"LibreDWGc::_dwg_object_object_tio_SUNSTUDY_set", _wrap__dwg_object_object_tio_SUNSTUDY_set},
{"LibreDWGc::_dwg_object_object_tio_SUNSTUDY_get", _wrap__dwg_object_object_tio_SUNSTUDY_get},
{"LibreDWGc::_dwg_object_object_tio_TABLECONTENT_set", _wrap__dwg_object_object_tio_TABLECONTENT_set},
{"LibreDWGc::_dwg_object_object_tio_TABLECONTENT_get", _wrap__dwg_object_object_tio_TABLECONTENT_get},
{"LibreDWGc::_dwg_object_object_tio_TVDEVICEPROPERTIES_set", _wrap__dwg_object_object_tio_TVDEVICEPROPERTIES_set},
{"LibreDWGc::_dwg_object_object_tio_TVDEVICEPROPERTIES_get", _wrap__dwg_object_object_tio_TVDEVICEPROPERTIES_get},
{"LibreDWGc::new__dwg_object_object_tio", _wrap_new__dwg_object_object_tio},
{"LibreDWGc::delete__dwg_object_object_tio", _wrap_delete__dwg_object_object_tio},
{"LibreDWGc::Dwg_Class_number_set", _wrap_Dwg_Class_number_set},
{"LibreDWGc::Dwg_Class_number_get", _wrap_Dwg_Class_number_get},
{"LibreDWGc::Dwg_Class_proxyflag_set", _wrap_Dwg_Class_proxyflag_set},
{"LibreDWGc::Dwg_Class_proxyflag_get", _wrap_Dwg_Class_proxyflag_get},
{"LibreDWGc::Dwg_Class_appname_set", _wrap_Dwg_Class_appname_set},
{"LibreDWGc::Dwg_Class_appname_get", _wrap_Dwg_Class_appname_get},
{"LibreDWGc::Dwg_Class_cppname_set", _wrap_Dwg_Class_cppname_set},
{"LibreDWGc::Dwg_Class_cppname_get", _wrap_Dwg_Class_cppname_get},
{"LibreDWGc::Dwg_Class_dxfname_set", _wrap_Dwg_Class_dxfname_set},
{"LibreDWGc::Dwg_Class_dxfname_get", _wrap_Dwg_Class_dxfname_get},
{"LibreDWGc::Dwg_Class_dxfname_u_set", _wrap_Dwg_Class_dxfname_u_set},
{"LibreDWGc::Dwg_Class_dxfname_u_get", _wrap_Dwg_Class_dxfname_u_get},
{"LibreDWGc::Dwg_Class_is_zombie_set", _wrap_Dwg_Class_is_zombie_set},
{"LibreDWGc::Dwg_Class_is_zombie_get", _wrap_Dwg_Class_is_zombie_get},
{"LibreDWGc::Dwg_Class_item_class_id_set", _wrap_Dwg_Class_item_class_id_set},
{"LibreDWGc::Dwg_Class_item_class_id_get", _wrap_Dwg_Class_item_class_id_get},
{"LibreDWGc::Dwg_Class_num_instances_set", _wrap_Dwg_Class_num_instances_set},
{"LibreDWGc::Dwg_Class_num_instances_get", _wrap_Dwg_Class_num_instances_get},
{"LibreDWGc::Dwg_Class_dwg_version_set", _wrap_Dwg_Class_dwg_version_set},
{"LibreDWGc::Dwg_Class_dwg_version_get", _wrap_Dwg_Class_dwg_version_get},
{"LibreDWGc::Dwg_Class_maint_version_set", _wrap_Dwg_Class_maint_version_set},
{"LibreDWGc::Dwg_Class_maint_version_get", _wrap_Dwg_Class_maint_version_get},
{"LibreDWGc::Dwg_Class_unknown_1_set", _wrap_Dwg_Class_unknown_1_set},
{"LibreDWGc::Dwg_Class_unknown_1_get", _wrap_Dwg_Class_unknown_1_get},
{"LibreDWGc::Dwg_Class_unknown_2_set", _wrap_Dwg_Class_unknown_2_set},
{"LibreDWGc::Dwg_Class_unknown_2_get", _wrap_Dwg_Class_unknown_2_get},
{"LibreDWGc::new_Dwg_Class", _wrap_new_Dwg_Class},
{"LibreDWGc::delete_Dwg_Class", _wrap_delete_Dwg_Class},
{"LibreDWGc::Dwg_Object_size_set", _wrap_Dwg_Object_size_set},
{"LibreDWGc::Dwg_Object_size_get", _wrap_Dwg_Object_size_get},
{"LibreDWGc::Dwg_Object_address_set", _wrap_Dwg_Object_address_set},
{"LibreDWGc::Dwg_Object_address_get", _wrap_Dwg_Object_address_get},
{"LibreDWGc::Dwg_Object_type_set", _wrap_Dwg_Object_type_set},
{"LibreDWGc::Dwg_Object_type_get", _wrap_Dwg_Object_type_get},
{"LibreDWGc::Dwg_Object_index_set", _wrap_Dwg_Object_index_set},
{"LibreDWGc::Dwg_Object_index_get", _wrap_Dwg_Object_index_get},
{"LibreDWGc::Dwg_Object_fixedtype_set", _wrap_Dwg_Object_fixedtype_set},
{"LibreDWGc::Dwg_Object_fixedtype_get", _wrap_Dwg_Object_fixedtype_get},
{"LibreDWGc::Dwg_Object_name_set", _wrap_Dwg_Object_name_set},
{"LibreDWGc::Dwg_Object_name_get", _wrap_Dwg_Object_name_get},
{"LibreDWGc::Dwg_Object_dxfname_set", _wrap_Dwg_Object_dxfname_set},
{"LibreDWGc::Dwg_Object_dxfname_get", _wrap_Dwg_Object_dxfname_get},
{"LibreDWGc::Dwg_Object_supertype_set", _wrap_Dwg_Object_supertype_set},
{"LibreDWGc::Dwg_Object_supertype_get", _wrap_Dwg_Object_supertype_get},
{"LibreDWGc::Dwg_Object_tio_get", _wrap_Dwg_Object_tio_get},
{"LibreDWGc::Dwg_Object_handle_set", _wrap_Dwg_Object_handle_set},
{"LibreDWGc::Dwg_Object_handle_get", _wrap_Dwg_Object_handle_get},
{"LibreDWGc::Dwg_Object_parent_set", _wrap_Dwg_Object_parent_set},
{"LibreDWGc::Dwg_Object_parent_get", _wrap_Dwg_Object_parent_get},
{"LibreDWGc::Dwg_Object_klass_set", _wrap_Dwg_Object_klass_set},
{"LibreDWGc::Dwg_Object_klass_get", _wrap_Dwg_Object_klass_get},
{"LibreDWGc::Dwg_Object_bitsize_set", _wrap_Dwg_Object_bitsize_set},
{"LibreDWGc::Dwg_Object_bitsize_get", _wrap_Dwg_Object_bitsize_get},
{"LibreDWGc::Dwg_Object_bitsize_pos_set", _wrap_Dwg_Object_bitsize_pos_set},
{"LibreDWGc::Dwg_Object_bitsize_pos_get", _wrap_Dwg_Object_bitsize_pos_get},
{"LibreDWGc::Dwg_Object_hdlpos_set", _wrap_Dwg_Object_hdlpos_set},
{"LibreDWGc::Dwg_Object_hdlpos_get", _wrap_Dwg_Object_hdlpos_get},
{"LibreDWGc::Dwg_Object_was_bitsize_set_set", _wrap_Dwg_Object_was_bitsize_set_set},
{"LibreDWGc::Dwg_Object_was_bitsize_set_get", _wrap_Dwg_Object_was_bitsize_set_get},
{"LibreDWGc::Dwg_Object_has_strings_set", _wrap_Dwg_Object_has_strings_set},
{"LibreDWGc::Dwg_Object_has_strings_get", _wrap_Dwg_Object_has_strings_get},
{"LibreDWGc::Dwg_Object_stringstream_size_set", _wrap_Dwg_Object_stringstream_size_set},
{"LibreDWGc::Dwg_Object_stringstream_size_get", _wrap_Dwg_Object_stringstream_size_get},
{"LibreDWGc::Dwg_Object_handlestream_size_set", _wrap_Dwg_Object_handlestream_size_set},
{"LibreDWGc::Dwg_Object_handlestream_size_get", _wrap_Dwg_Object_handlestream_size_get},
{"LibreDWGc::Dwg_Object_common_size_set", _wrap_Dwg_Object_common_size_set},
{"LibreDWGc::Dwg_Object_common_size_get", _wrap_Dwg_Object_common_size_get},
{"LibreDWGc::Dwg_Object_num_unknown_bits_set", _wrap_Dwg_Object_num_unknown_bits_set},
{"LibreDWGc::Dwg_Object_num_unknown_bits_get", _wrap_Dwg_Object_num_unknown_bits_get},
{"LibreDWGc::Dwg_Object_unknown_bits_set", _wrap_Dwg_Object_unknown_bits_set},
{"LibreDWGc::Dwg_Object_unknown_bits_get", _wrap_Dwg_Object_unknown_bits_get},
{"LibreDWGc::Dwg_Object_num_unknown_rest_set", _wrap_Dwg_Object_num_unknown_rest_set},
{"LibreDWGc::Dwg_Object_num_unknown_rest_get", _wrap_Dwg_Object_num_unknown_rest_get},
{"LibreDWGc::Dwg_Object_unknown_rest_set", _wrap_Dwg_Object_unknown_rest_set},
{"LibreDWGc::Dwg_Object_unknown_rest_get", _wrap_Dwg_Object_unknown_rest_get},
{"LibreDWGc::new_Dwg_Object", _wrap_new_Dwg_Object},
{"LibreDWGc::delete_Dwg_Object", _wrap_delete_Dwg_Object},
{"LibreDWGc::_dwg_object_tio_entity_set", _wrap__dwg_object_tio_entity_set},
{"LibreDWGc::_dwg_object_tio_entity_get", _wrap__dwg_object_tio_entity_get},
{"LibreDWGc::_dwg_object_tio_object_set", _wrap__dwg_object_tio_object_set},
{"LibreDWGc::_dwg_object_tio_object_get", _wrap__dwg_object_tio_object_get},
{"LibreDWGc::new__dwg_object_tio", _wrap_new__dwg_object_tio},
{"LibreDWGc::delete__dwg_object_tio", _wrap_delete__dwg_object_tio},
{"LibreDWGc::Dwg_Chain_chain_set", _wrap_Dwg_Chain_chain_set},
{"LibreDWGc::Dwg_Chain_chain_get", _wrap_Dwg_Chain_chain_get},
{"LibreDWGc::Dwg_Chain_size_set", _wrap_Dwg_Chain_size_set},
{"LibreDWGc::Dwg_Chain_size_get", _wrap_Dwg_Chain_size_get},
{"LibreDWGc::Dwg_Chain_byte_set", _wrap_Dwg_Chain_byte_set},
{"LibreDWGc::Dwg_Chain_byte_get", _wrap_Dwg_Chain_byte_get},
{"LibreDWGc::Dwg_Chain_bit_set", _wrap_Dwg_Chain_bit_set},
{"LibreDWGc::Dwg_Chain_bit_get", _wrap_Dwg_Chain_bit_get},
{"LibreDWGc::new_Dwg_Chain", _wrap_new_Dwg_Chain},
{"LibreDWGc::delete_Dwg_Chain", _wrap_delete_Dwg_Chain},
{"LibreDWGc::Dwg_Section_number_set", _wrap_Dwg_Section_number_set},
{"LibreDWGc::Dwg_Section_number_get", _wrap_Dwg_Section_number_get},
{"LibreDWGc::Dwg_Section_size_set", _wrap_Dwg_Section_size_set},
{"LibreDWGc::Dwg_Section_size_get", _wrap_Dwg_Section_size_get},
{"LibreDWGc::Dwg_Section_address_set", _wrap_Dwg_Section_address_set},
{"LibreDWGc::Dwg_Section_address_get", _wrap_Dwg_Section_address_get},
{"LibreDWGc::Dwg_Section_objid_r11_set", _wrap_Dwg_Section_objid_r11_set},
{"LibreDWGc::Dwg_Section_objid_r11_get", _wrap_Dwg_Section_objid_r11_get},
{"LibreDWGc::Dwg_Section_parent_set", _wrap_Dwg_Section_parent_set},
{"LibreDWGc::Dwg_Section_parent_get", _wrap_Dwg_Section_parent_get},
{"LibreDWGc::Dwg_Section_left_set", _wrap_Dwg_Section_left_set},
{"LibreDWGc::Dwg_Section_left_get", _wrap_Dwg_Section_left_get},
{"LibreDWGc::Dwg_Section_right_set", _wrap_Dwg_Section_right_set},
{"LibreDWGc::Dwg_Section_right_get", _wrap_Dwg_Section_right_get},
{"LibreDWGc::Dwg_Section_x00_set", _wrap_Dwg_Section_x00_set},
{"LibreDWGc::Dwg_Section_x00_get", _wrap_Dwg_Section_x00_get},
{"LibreDWGc::Dwg_Section_type_set", _wrap_Dwg_Section_type_set},
{"LibreDWGc::Dwg_Section_type_get", _wrap_Dwg_Section_type_get},
{"LibreDWGc::Dwg_Section_name_set", _wrap_Dwg_Section_name_set},
{"LibreDWGc::Dwg_Section_name_get", _wrap_Dwg_Section_name_get},
{"LibreDWGc::Dwg_Section_section_type_set", _wrap_Dwg_Section_section_type_set},
{"LibreDWGc::Dwg_Section_section_type_get", _wrap_Dwg_Section_section_type_get},
{"LibreDWGc::Dwg_Section_decomp_data_size_set", _wrap_Dwg_Section_decomp_data_size_set},
{"LibreDWGc::Dwg_Section_decomp_data_size_get", _wrap_Dwg_Section_decomp_data_size_get},
{"LibreDWGc::Dwg_Section_comp_data_size_set", _wrap_Dwg_Section_comp_data_size_set},
{"LibreDWGc::Dwg_Section_comp_data_size_get", _wrap_Dwg_Section_comp_data_size_get},
{"LibreDWGc::Dwg_Section_compression_type_set", _wrap_Dwg_Section_compression_type_set},
{"LibreDWGc::Dwg_Section_compression_type_get", _wrap_Dwg_Section_compression_type_get},
{"LibreDWGc::Dwg_Section_checksum_set", _wrap_Dwg_Section_checksum_set},
{"LibreDWGc::Dwg_Section_checksum_get", _wrap_Dwg_Section_checksum_get},
{"LibreDWGc::Dwg_Section_flags_r11_set", _wrap_Dwg_Section_flags_r11_set},
{"LibreDWGc::Dwg_Section_flags_r11_get", _wrap_Dwg_Section_flags_r11_get},
{"LibreDWGc::new_Dwg_Section", _wrap_new_Dwg_Section},
{"LibreDWGc::delete_Dwg_Section", _wrap_delete_Dwg_Section},
{"LibreDWGc::Dwg_Section_InfoHdr_num_desc_set", _wrap_Dwg_Section_InfoHdr_num_desc_set},
{"LibreDWGc::Dwg_Section_InfoHdr_num_desc_get", _wrap_Dwg_Section_InfoHdr_num_desc_get},
{"LibreDWGc::Dwg_Section_InfoHdr_compressed_set", _wrap_Dwg_Section_InfoHdr_compressed_set},
{"LibreDWGc::Dwg_Section_InfoHdr_compressed_get", _wrap_Dwg_Section_InfoHdr_compressed_get},
{"LibreDWGc::Dwg_Section_InfoHdr_max_size_set", _wrap_Dwg_Section_InfoHdr_max_size_set},
{"LibreDWGc::Dwg_Section_InfoHdr_max_size_get", _wrap_Dwg_Section_InfoHdr_max_size_get},
{"LibreDWGc::Dwg_Section_InfoHdr_encrypted_set", _wrap_Dwg_Section_InfoHdr_encrypted_set},
{"LibreDWGc::Dwg_Section_InfoHdr_encrypted_get", _wrap_Dwg_Section_InfoHdr_encrypted_get},
{"LibreDWGc::Dwg_Section_InfoHdr_num_desc2_set", _wrap_Dwg_Section_InfoHdr_num_desc2_set},
{"LibreDWGc::Dwg_Section_InfoHdr_num_desc2_get", _wrap_Dwg_Section_InfoHdr_num_desc2_get},
{"LibreDWGc::new_Dwg_Section_InfoHdr", _wrap_new_Dwg_Section_InfoHdr},
{"LibreDWGc::delete_Dwg_Section_InfoHdr", _wrap_delete_Dwg_Section_InfoHdr},
{"LibreDWGc::Dwg_Section_Info_size_set", _wrap_Dwg_Section_Info_size_set},
{"LibreDWGc::Dwg_Section_Info_size_get", _wrap_Dwg_Section_Info_size_get},
{"LibreDWGc::Dwg_Section_Info_num_sections_set", _wrap_Dwg_Section_Info_num_sections_set},
{"LibreDWGc::Dwg_Section_Info_num_sections_get", _wrap_Dwg_Section_Info_num_sections_get},
{"LibreDWGc::Dwg_Section_Info_max_decomp_size_set", _wrap_Dwg_Section_Info_max_decomp_size_set},
{"LibreDWGc::Dwg_Section_Info_max_decomp_size_get", _wrap_Dwg_Section_Info_max_decomp_size_get},
{"LibreDWGc::Dwg_Section_Info_unknown_set", _wrap_Dwg_Section_Info_unknown_set},
{"LibreDWGc::Dwg_Section_Info_unknown_get", _wrap_Dwg_Section_Info_unknown_get},
{"LibreDWGc::Dwg_Section_Info_compressed_set", _wrap_Dwg_Section_Info_compressed_set},
{"LibreDWGc::Dwg_Section_Info_compressed_get", _wrap_Dwg_Section_Info_compressed_get},
{"LibreDWGc::Dwg_Section_Info_type_set", _wrap_Dwg_Section_Info_type_set},
{"LibreDWGc::Dwg_Section_Info_type_get", _wrap_Dwg_Section_Info_type_get},
{"LibreDWGc::Dwg_Section_Info_encrypted_set", _wrap_Dwg_Section_Info_encrypted_set},
{"LibreDWGc::Dwg_Section_Info_encrypted_get", _wrap_Dwg_Section_Info_encrypted_get},
{"LibreDWGc::Dwg_Section_Info_name_set", _wrap_Dwg_Section_Info_name_set},
{"LibreDWGc::Dwg_Section_Info_name_get", _wrap_Dwg_Section_Info_name_get},
{"LibreDWGc::Dwg_Section_Info_fixedtype_set", _wrap_Dwg_Section_Info_fixedtype_set},
{"LibreDWGc::Dwg_Section_Info_fixedtype_get", _wrap_Dwg_Section_Info_fixedtype_get},
{"LibreDWGc::Dwg_Section_Info_sections_set", _wrap_Dwg_Section_Info_sections_set},
{"LibreDWGc::Dwg_Section_Info_sections_get", _wrap_Dwg_Section_Info_sections_get},
{"LibreDWGc::new_Dwg_Section_Info", _wrap_new_Dwg_Section_Info},
{"LibreDWGc::delete_Dwg_Section_Info", _wrap_delete_Dwg_Section_Info},
{"LibreDWGc::Dwg_SummaryInfo_Property_tag_set", _wrap_Dwg_SummaryInfo_Property_tag_set},
{"LibreDWGc::Dwg_SummaryInfo_Property_tag_get", _wrap_Dwg_SummaryInfo_Property_tag_get},
{"LibreDWGc::Dwg_SummaryInfo_Property_value_set", _wrap_Dwg_SummaryInfo_Property_value_set},
{"LibreDWGc::Dwg_SummaryInfo_Property_value_get", _wrap_Dwg_SummaryInfo_Property_value_get},
{"LibreDWGc::new_Dwg_SummaryInfo_Property", _wrap_new_Dwg_SummaryInfo_Property},
{"LibreDWGc::delete_Dwg_SummaryInfo_Property", _wrap_delete_Dwg_SummaryInfo_Property},
{"LibreDWGc::Dwg_FileDepList_Files_filename_set", _wrap_Dwg_FileDepList_Files_filename_set},
{"LibreDWGc::Dwg_FileDepList_Files_filename_get", _wrap_Dwg_FileDepList_Files_filename_get},
{"LibreDWGc::Dwg_FileDepList_Files_filepath_set", _wrap_Dwg_FileDepList_Files_filepath_set},
{"LibreDWGc::Dwg_FileDepList_Files_filepath_get", _wrap_Dwg_FileDepList_Files_filepath_get},
{"LibreDWGc::Dwg_FileDepList_Files_fingerprint_set", _wrap_Dwg_FileDepList_Files_fingerprint_set},
{"LibreDWGc::Dwg_FileDepList_Files_fingerprint_get", _wrap_Dwg_FileDepList_Files_fingerprint_get},
{"LibreDWGc::Dwg_FileDepList_Files_version_set", _wrap_Dwg_FileDepList_Files_version_set},
{"LibreDWGc::Dwg_FileDepList_Files_version_get", _wrap_Dwg_FileDepList_Files_version_get},
{"LibreDWGc::Dwg_FileDepList_Files_feature_index_set", _wrap_Dwg_FileDepList_Files_feature_index_set},
{"LibreDWGc::Dwg_FileDepList_Files_feature_index_get", _wrap_Dwg_FileDepList_Files_feature_index_get},
{"LibreDWGc::Dwg_FileDepList_Files_timestamp_set", _wrap_Dwg_FileDepList_Files_timestamp_set},
{"LibreDWGc::Dwg_FileDepList_Files_timestamp_get", _wrap_Dwg_FileDepList_Files_timestamp_get},
{"LibreDWGc::Dwg_FileDepList_Files_filesize_set", _wrap_Dwg_FileDepList_Files_filesize_set},
{"LibreDWGc::Dwg_FileDepList_Files_filesize_get", _wrap_Dwg_FileDepList_Files_filesize_get},
{"LibreDWGc::Dwg_FileDepList_Files_affects_graphics_set", _wrap_Dwg_FileDepList_Files_affects_graphics_set},
{"LibreDWGc::Dwg_FileDepList_Files_affects_graphics_get", _wrap_Dwg_FileDepList_Files_affects_graphics_get},
{"LibreDWGc::Dwg_FileDepList_Files_refcount_set", _wrap_Dwg_FileDepList_Files_refcount_set},
{"LibreDWGc::Dwg_FileDepList_Files_refcount_get", _wrap_Dwg_FileDepList_Files_refcount_get},
{"LibreDWGc::new_Dwg_FileDepList_Files", _wrap_new_Dwg_FileDepList_Files},
{"LibreDWGc::delete_Dwg_FileDepList_Files", _wrap_delete_Dwg_FileDepList_Files},
{"LibreDWGc::Dwg_AcDs_SegmentIndex_offset_set", _wrap_Dwg_AcDs_SegmentIndex_offset_set},
{"LibreDWGc::Dwg_AcDs_SegmentIndex_offset_get", _wrap_Dwg_AcDs_SegmentIndex_offset_get},
{"LibreDWGc::Dwg_AcDs_SegmentIndex_size_set", _wrap_Dwg_AcDs_SegmentIndex_size_set},
{"LibreDWGc::Dwg_AcDs_SegmentIndex_size_get", _wrap_Dwg_AcDs_SegmentIndex_size_get},
{"LibreDWGc::new_Dwg_AcDs_SegmentIndex", _wrap_new_Dwg_AcDs_SegmentIndex},
{"LibreDWGc::delete_Dwg_AcDs_SegmentIndex", _wrap_delete_Dwg_AcDs_SegmentIndex},
{"LibreDWGc::Dwg_AcDs_DataIndex_Entry_segidx_set", _wrap_Dwg_AcDs_DataIndex_Entry_segidx_set},
{"LibreDWGc::Dwg_AcDs_DataIndex_Entry_segidx_get", _wrap_Dwg_AcDs_DataIndex_Entry_segidx_get},
{"LibreDWGc::Dwg_AcDs_DataIndex_Entry_offset_set", _wrap_Dwg_AcDs_DataIndex_Entry_offset_set},
{"LibreDWGc::Dwg_AcDs_DataIndex_Entry_offset_get", _wrap_Dwg_AcDs_DataIndex_Entry_offset_get},
{"LibreDWGc::Dwg_AcDs_DataIndex_Entry_schidx_set", _wrap_Dwg_AcDs_DataIndex_Entry_schidx_set},
{"LibreDWGc::Dwg_AcDs_DataIndex_Entry_schidx_get", _wrap_Dwg_AcDs_DataIndex_Entry_schidx_get},
{"LibreDWGc::new_Dwg_AcDs_DataIndex_Entry", _wrap_new_Dwg_AcDs_DataIndex_Entry},
{"LibreDWGc::delete_Dwg_AcDs_DataIndex_Entry", _wrap_delete_Dwg_AcDs_DataIndex_Entry},
{"LibreDWGc::Dwg_AcDs_DataIndex_num_entries_set", _wrap_Dwg_AcDs_DataIndex_num_entries_set},
{"LibreDWGc::Dwg_AcDs_DataIndex_num_entries_get", _wrap_Dwg_AcDs_DataIndex_num_entries_get},
{"LibreDWGc::Dwg_AcDs_DataIndex_di_unknown_set", _wrap_Dwg_AcDs_DataIndex_di_unknown_set},
{"LibreDWGc::Dwg_AcDs_DataIndex_di_unknown_get", _wrap_Dwg_AcDs_DataIndex_di_unknown_get},
{"LibreDWGc::Dwg_AcDs_DataIndex_entries_set", _wrap_Dwg_AcDs_DataIndex_entries_set},
{"LibreDWGc::Dwg_AcDs_DataIndex_entries_get", _wrap_Dwg_AcDs_DataIndex_entries_get},
{"LibreDWGc::new_Dwg_AcDs_DataIndex", _wrap_new_Dwg_AcDs_DataIndex},
{"LibreDWGc::delete_Dwg_AcDs_DataIndex", _wrap_delete_Dwg_AcDs_DataIndex},
{"LibreDWGc::Dwg_AcDs_Data_RecordHdr_entry_size_set", _wrap_Dwg_AcDs_Data_RecordHdr_entry_size_set},
{"LibreDWGc::Dwg_AcDs_Data_RecordHdr_entry_size_get", _wrap_Dwg_AcDs_Data_RecordHdr_entry_size_get},
{"LibreDWGc::Dwg_AcDs_Data_RecordHdr_unknown_set", _wrap_Dwg_AcDs_Data_RecordHdr_unknown_set},
{"LibreDWGc::Dwg_AcDs_Data_RecordHdr_unknown_get", _wrap_Dwg_AcDs_Data_RecordHdr_unknown_get},
{"LibreDWGc::Dwg_AcDs_Data_RecordHdr_handle_set", _wrap_Dwg_AcDs_Data_RecordHdr_handle_set},
{"LibreDWGc::Dwg_AcDs_Data_RecordHdr_handle_get", _wrap_Dwg_AcDs_Data_RecordHdr_handle_get},
{"LibreDWGc::Dwg_AcDs_Data_RecordHdr_offset_set", _wrap_Dwg_AcDs_Data_RecordHdr_offset_set},
{"LibreDWGc::Dwg_AcDs_Data_RecordHdr_offset_get", _wrap_Dwg_AcDs_Data_RecordHdr_offset_get},
{"LibreDWGc::new_Dwg_AcDs_Data_RecordHdr", _wrap_new_Dwg_AcDs_Data_RecordHdr},
{"LibreDWGc::delete_Dwg_AcDs_Data_RecordHdr", _wrap_delete_Dwg_AcDs_Data_RecordHdr},
{"LibreDWGc::Dwg_AcDs_Data_Record_data_size_set", _wrap_Dwg_AcDs_Data_Record_data_size_set},
{"LibreDWGc::Dwg_AcDs_Data_Record_data_size_get", _wrap_Dwg_AcDs_Data_Record_data_size_get},
{"LibreDWGc::Dwg_AcDs_Data_Record_blob_set", _wrap_Dwg_AcDs_Data_Record_blob_set},
{"LibreDWGc::Dwg_AcDs_Data_Record_blob_get", _wrap_Dwg_AcDs_Data_Record_blob_get},
{"LibreDWGc::new_Dwg_AcDs_Data_Record", _wrap_new_Dwg_AcDs_Data_Record},
{"LibreDWGc::delete_Dwg_AcDs_Data_Record", _wrap_delete_Dwg_AcDs_Data_Record},
{"LibreDWGc::Dwg_AcDs_Data_record_hdrs_set", _wrap_Dwg_AcDs_Data_record_hdrs_set},
{"LibreDWGc::Dwg_AcDs_Data_record_hdrs_get", _wrap_Dwg_AcDs_Data_record_hdrs_get},
{"LibreDWGc::Dwg_AcDs_Data_records_set", _wrap_Dwg_AcDs_Data_records_set},
{"LibreDWGc::Dwg_AcDs_Data_records_get", _wrap_Dwg_AcDs_Data_records_get},
{"LibreDWGc::new_Dwg_AcDs_Data", _wrap_new_Dwg_AcDs_Data},
{"LibreDWGc::delete_Dwg_AcDs_Data", _wrap_delete_Dwg_AcDs_Data},
{"LibreDWGc::Dwg_AcDs_DataBlobRef_Page_segidx_set", _wrap_Dwg_AcDs_DataBlobRef_Page_segidx_set},
{"LibreDWGc::Dwg_AcDs_DataBlobRef_Page_segidx_get", _wrap_Dwg_AcDs_DataBlobRef_Page_segidx_get},
{"LibreDWGc::Dwg_AcDs_DataBlobRef_Page_size_set", _wrap_Dwg_AcDs_DataBlobRef_Page_size_set},
{"LibreDWGc::Dwg_AcDs_DataBlobRef_Page_size_get", _wrap_Dwg_AcDs_DataBlobRef_Page_size_get},
{"LibreDWGc::new_Dwg_AcDs_DataBlobRef_Page", _wrap_new_Dwg_AcDs_DataBlobRef_Page},
{"LibreDWGc::delete_Dwg_AcDs_DataBlobRef_Page", _wrap_delete_Dwg_AcDs_DataBlobRef_Page},
{"LibreDWGc::Dwg_AcDs_DataBlobRef_total_data_size_set", _wrap_Dwg_AcDs_DataBlobRef_total_data_size_set},
{"LibreDWGc::Dwg_AcDs_DataBlobRef_total_data_size_get", _wrap_Dwg_AcDs_DataBlobRef_total_data_size_get},
{"LibreDWGc::Dwg_AcDs_DataBlobRef_num_pages_set", _wrap_Dwg_AcDs_DataBlobRef_num_pages_set},
{"LibreDWGc::Dwg_AcDs_DataBlobRef_num_pages_get", _wrap_Dwg_AcDs_DataBlobRef_num_pages_get},
{"LibreDWGc::Dwg_AcDs_DataBlobRef_record_size_set", _wrap_Dwg_AcDs_DataBlobRef_record_size_set},
{"LibreDWGc::Dwg_AcDs_DataBlobRef_record_size_get", _wrap_Dwg_AcDs_DataBlobRef_record_size_get},
{"LibreDWGc::Dwg_AcDs_DataBlobRef_page_size_set", _wrap_Dwg_AcDs_DataBlobRef_page_size_set},
{"LibreDWGc::Dwg_AcDs_DataBlobRef_page_size_get", _wrap_Dwg_AcDs_DataBlobRef_page_size_get},
{"LibreDWGc::Dwg_AcDs_DataBlobRef_unknown_1_set", _wrap_Dwg_AcDs_DataBlobRef_unknown_1_set},
{"LibreDWGc::Dwg_AcDs_DataBlobRef_unknown_1_get", _wrap_Dwg_AcDs_DataBlobRef_unknown_1_get},
{"LibreDWGc::Dwg_AcDs_DataBlobRef_unknown_2_set", _wrap_Dwg_AcDs_DataBlobRef_unknown_2_set},
{"LibreDWGc::Dwg_AcDs_DataBlobRef_unknown_2_get", _wrap_Dwg_AcDs_DataBlobRef_unknown_2_get},
{"LibreDWGc::Dwg_AcDs_DataBlobRef_pages_set", _wrap_Dwg_AcDs_DataBlobRef_pages_set},
{"LibreDWGc::Dwg_AcDs_DataBlobRef_pages_get", _wrap_Dwg_AcDs_DataBlobRef_pages_get},
{"LibreDWGc::new_Dwg_AcDs_DataBlobRef", _wrap_new_Dwg_AcDs_DataBlobRef},
{"LibreDWGc::delete_Dwg_AcDs_DataBlobRef", _wrap_delete_Dwg_AcDs_DataBlobRef},
{"LibreDWGc::Dwg_AcDs_DataBlob_data_size_set", _wrap_Dwg_AcDs_DataBlob_data_size_set},
{"LibreDWGc::Dwg_AcDs_DataBlob_data_size_get", _wrap_Dwg_AcDs_DataBlob_data_size_get},
{"LibreDWGc::Dwg_AcDs_DataBlob_page_count_set", _wrap_Dwg_AcDs_DataBlob_page_count_set},
{"LibreDWGc::Dwg_AcDs_DataBlob_page_count_get", _wrap_Dwg_AcDs_DataBlob_page_count_get},
{"LibreDWGc::Dwg_AcDs_DataBlob_record_size_set", _wrap_Dwg_AcDs_DataBlob_record_size_set},
{"LibreDWGc::Dwg_AcDs_DataBlob_record_size_get", _wrap_Dwg_AcDs_DataBlob_record_size_get},
{"LibreDWGc::Dwg_AcDs_DataBlob_page_size_set", _wrap_Dwg_AcDs_DataBlob_page_size_set},
{"LibreDWGc::Dwg_AcDs_DataBlob_page_size_get", _wrap_Dwg_AcDs_DataBlob_page_size_get},
{"LibreDWGc::Dwg_AcDs_DataBlob_unknown_1_set", _wrap_Dwg_AcDs_DataBlob_unknown_1_set},
{"LibreDWGc::Dwg_AcDs_DataBlob_unknown_1_get", _wrap_Dwg_AcDs_DataBlob_unknown_1_get},
{"LibreDWGc::Dwg_AcDs_DataBlob_unknown_2_set", _wrap_Dwg_AcDs_DataBlob_unknown_2_set},
{"LibreDWGc::Dwg_AcDs_DataBlob_unknown_2_get", _wrap_Dwg_AcDs_DataBlob_unknown_2_get},
{"LibreDWGc::Dwg_AcDs_DataBlob_ref_set", _wrap_Dwg_AcDs_DataBlob_ref_set},
{"LibreDWGc::Dwg_AcDs_DataBlob_ref_get", _wrap_Dwg_AcDs_DataBlob_ref_get},
{"LibreDWGc::new_Dwg_AcDs_DataBlob", _wrap_new_Dwg_AcDs_DataBlob},
{"LibreDWGc::delete_Dwg_AcDs_DataBlob", _wrap_delete_Dwg_AcDs_DataBlob},
{"LibreDWGc::Dwg_AcDs_DataBlob01_total_data_size_set", _wrap_Dwg_AcDs_DataBlob01_total_data_size_set},
{"LibreDWGc::Dwg_AcDs_DataBlob01_total_data_size_get", _wrap_Dwg_AcDs_DataBlob01_total_data_size_get},
{"LibreDWGc::Dwg_AcDs_DataBlob01_page_start_offset_set", _wrap_Dwg_AcDs_DataBlob01_page_start_offset_set},
{"LibreDWGc::Dwg_AcDs_DataBlob01_page_start_offset_get", _wrap_Dwg_AcDs_DataBlob01_page_start_offset_get},
{"LibreDWGc::Dwg_AcDs_DataBlob01_page_index_set", _wrap_Dwg_AcDs_DataBlob01_page_index_set},
{"LibreDWGc::Dwg_AcDs_DataBlob01_page_index_get", _wrap_Dwg_AcDs_DataBlob01_page_index_get},
{"LibreDWGc::Dwg_AcDs_DataBlob01_page_count_set", _wrap_Dwg_AcDs_DataBlob01_page_count_set},
{"LibreDWGc::Dwg_AcDs_DataBlob01_page_count_get", _wrap_Dwg_AcDs_DataBlob01_page_count_get},
{"LibreDWGc::Dwg_AcDs_DataBlob01_page_data_size_set", _wrap_Dwg_AcDs_DataBlob01_page_data_size_set},
{"LibreDWGc::Dwg_AcDs_DataBlob01_page_data_size_get", _wrap_Dwg_AcDs_DataBlob01_page_data_size_get},
{"LibreDWGc::Dwg_AcDs_DataBlob01_page_data_set", _wrap_Dwg_AcDs_DataBlob01_page_data_set},
{"LibreDWGc::Dwg_AcDs_DataBlob01_page_data_get", _wrap_Dwg_AcDs_DataBlob01_page_data_get},
{"LibreDWGc::new_Dwg_AcDs_DataBlob01", _wrap_new_Dwg_AcDs_DataBlob01},
{"LibreDWGc::delete_Dwg_AcDs_DataBlob01", _wrap_delete_Dwg_AcDs_DataBlob01},
{"LibreDWGc::Dwg_AcDs_SchemaIndex_Prop_index_set", _wrap_Dwg_AcDs_SchemaIndex_Prop_index_set},
{"LibreDWGc::Dwg_AcDs_SchemaIndex_Prop_index_get", _wrap_Dwg_AcDs_SchemaIndex_Prop_index_get},
{"LibreDWGc::Dwg_AcDs_SchemaIndex_Prop_segidx_set", _wrap_Dwg_AcDs_SchemaIndex_Prop_segidx_set},
{"LibreDWGc::Dwg_AcDs_SchemaIndex_Prop_segidx_get", _wrap_Dwg_AcDs_SchemaIndex_Prop_segidx_get},
{"LibreDWGc::Dwg_AcDs_SchemaIndex_Prop_offset_set", _wrap_Dwg_AcDs_SchemaIndex_Prop_offset_set},
{"LibreDWGc::Dwg_AcDs_SchemaIndex_Prop_offset_get", _wrap_Dwg_AcDs_SchemaIndex_Prop_offset_get},
{"LibreDWGc::new_Dwg_AcDs_SchemaIndex_Prop", _wrap_new_Dwg_AcDs_SchemaIndex_Prop},
{"LibreDWGc::delete_Dwg_AcDs_SchemaIndex_Prop", _wrap_delete_Dwg_AcDs_SchemaIndex_Prop},
{"LibreDWGc::Dwg_AcDs_SchemaIndex_num_props_set", _wrap_Dwg_AcDs_SchemaIndex_num_props_set},
{"LibreDWGc::Dwg_AcDs_SchemaIndex_num_props_get", _wrap_Dwg_AcDs_SchemaIndex_num_props_get},
{"LibreDWGc::Dwg_AcDs_SchemaIndex_si_unknown_1_set", _wrap_Dwg_AcDs_SchemaIndex_si_unknown_1_set},
{"LibreDWGc::Dwg_AcDs_SchemaIndex_si_unknown_1_get", _wrap_Dwg_AcDs_SchemaIndex_si_unknown_1_get},
{"LibreDWGc::Dwg_AcDs_SchemaIndex_props_set", _wrap_Dwg_AcDs_SchemaIndex_props_set},
{"LibreDWGc::Dwg_AcDs_SchemaIndex_props_get", _wrap_Dwg_AcDs_SchemaIndex_props_get},
{"LibreDWGc::Dwg_AcDs_SchemaIndex_si_tag_set", _wrap_Dwg_AcDs_SchemaIndex_si_tag_set},
{"LibreDWGc::Dwg_AcDs_SchemaIndex_si_tag_get", _wrap_Dwg_AcDs_SchemaIndex_si_tag_get},
{"LibreDWGc::Dwg_AcDs_SchemaIndex_num_prop_entries_set", _wrap_Dwg_AcDs_SchemaIndex_num_prop_entries_set},
{"LibreDWGc::Dwg_AcDs_SchemaIndex_num_prop_entries_get", _wrap_Dwg_AcDs_SchemaIndex_num_prop_entries_get},
{"LibreDWGc::Dwg_AcDs_SchemaIndex_si_unknown_2_set", _wrap_Dwg_AcDs_SchemaIndex_si_unknown_2_set},
{"LibreDWGc::Dwg_AcDs_SchemaIndex_si_unknown_2_get", _wrap_Dwg_AcDs_SchemaIndex_si_unknown_2_get},
{"LibreDWGc::Dwg_AcDs_SchemaIndex_prop_entries_set", _wrap_Dwg_AcDs_SchemaIndex_prop_entries_set},
{"LibreDWGc::Dwg_AcDs_SchemaIndex_prop_entries_get", _wrap_Dwg_AcDs_SchemaIndex_prop_entries_get},
{"LibreDWGc::new_Dwg_AcDs_SchemaIndex", _wrap_new_Dwg_AcDs_SchemaIndex},
{"LibreDWGc::delete_Dwg_AcDs_SchemaIndex", _wrap_delete_Dwg_AcDs_SchemaIndex},
{"LibreDWGc::Dwg_AcDs_Schema_Prop_flags_set", _wrap_Dwg_AcDs_Schema_Prop_flags_set},
{"LibreDWGc::Dwg_AcDs_Schema_Prop_flags_get", _wrap_Dwg_AcDs_Schema_Prop_flags_get},
{"LibreDWGc::Dwg_AcDs_Schema_Prop_namidx_set", _wrap_Dwg_AcDs_Schema_Prop_namidx_set},
{"LibreDWGc::Dwg_AcDs_Schema_Prop_namidx_get", _wrap_Dwg_AcDs_Schema_Prop_namidx_get},
{"LibreDWGc::Dwg_AcDs_Schema_Prop_type_set", _wrap_Dwg_AcDs_Schema_Prop_type_set},
{"LibreDWGc::Dwg_AcDs_Schema_Prop_type_get", _wrap_Dwg_AcDs_Schema_Prop_type_get},
{"LibreDWGc::Dwg_AcDs_Schema_Prop_type_size_set", _wrap_Dwg_AcDs_Schema_Prop_type_size_set},
{"LibreDWGc::Dwg_AcDs_Schema_Prop_type_size_get", _wrap_Dwg_AcDs_Schema_Prop_type_size_get},
{"LibreDWGc::Dwg_AcDs_Schema_Prop_unknown_1_set", _wrap_Dwg_AcDs_Schema_Prop_unknown_1_set},
{"LibreDWGc::Dwg_AcDs_Schema_Prop_unknown_1_get", _wrap_Dwg_AcDs_Schema_Prop_unknown_1_get},
{"LibreDWGc::Dwg_AcDs_Schema_Prop_unknown_2_set", _wrap_Dwg_AcDs_Schema_Prop_unknown_2_set},
{"LibreDWGc::Dwg_AcDs_Schema_Prop_unknown_2_get", _wrap_Dwg_AcDs_Schema_Prop_unknown_2_get},
{"LibreDWGc::Dwg_AcDs_Schema_Prop_num_values_set", _wrap_Dwg_AcDs_Schema_Prop_num_values_set},
{"LibreDWGc::Dwg_AcDs_Schema_Prop_num_values_get", _wrap_Dwg_AcDs_Schema_Prop_num_values_get},
{"LibreDWGc::Dwg_AcDs_Schema_Prop_values_set", _wrap_Dwg_AcDs_Schema_Prop_values_set},
{"LibreDWGc::Dwg_AcDs_Schema_Prop_values_get", _wrap_Dwg_AcDs_Schema_Prop_values_get},
{"LibreDWGc::new_Dwg_AcDs_Schema_Prop", _wrap_new_Dwg_AcDs_Schema_Prop},
{"LibreDWGc::delete_Dwg_AcDs_Schema_Prop", _wrap_delete_Dwg_AcDs_Schema_Prop},
{"LibreDWGc::Dwg_AcDs_Schema_num_index_set", _wrap_Dwg_AcDs_Schema_num_index_set},
{"LibreDWGc::Dwg_AcDs_Schema_num_index_get", _wrap_Dwg_AcDs_Schema_num_index_get},
{"LibreDWGc::Dwg_AcDs_Schema_index_set", _wrap_Dwg_AcDs_Schema_index_set},
{"LibreDWGc::Dwg_AcDs_Schema_index_get", _wrap_Dwg_AcDs_Schema_index_get},
{"LibreDWGc::Dwg_AcDs_Schema_num_props_set", _wrap_Dwg_AcDs_Schema_num_props_set},
{"LibreDWGc::Dwg_AcDs_Schema_num_props_get", _wrap_Dwg_AcDs_Schema_num_props_get},
{"LibreDWGc::Dwg_AcDs_Schema_props_set", _wrap_Dwg_AcDs_Schema_props_set},
{"LibreDWGc::Dwg_AcDs_Schema_props_get", _wrap_Dwg_AcDs_Schema_props_get},
{"LibreDWGc::new_Dwg_AcDs_Schema", _wrap_new_Dwg_AcDs_Schema},
{"LibreDWGc::delete_Dwg_AcDs_Schema", _wrap_delete_Dwg_AcDs_Schema},
{"LibreDWGc::Dwg_AcDs_SchemaData_UProp_size_set", _wrap_Dwg_AcDs_SchemaData_UProp_size_set},
{"LibreDWGc::Dwg_AcDs_SchemaData_UProp_size_get", _wrap_Dwg_AcDs_SchemaData_UProp_size_get},
{"LibreDWGc::Dwg_AcDs_SchemaData_UProp_flags_set", _wrap_Dwg_AcDs_SchemaData_UProp_flags_set},
{"LibreDWGc::Dwg_AcDs_SchemaData_UProp_flags_get", _wrap_Dwg_AcDs_SchemaData_UProp_flags_get},
{"LibreDWGc::new_Dwg_AcDs_SchemaData_UProp", _wrap_new_Dwg_AcDs_SchemaData_UProp},
{"LibreDWGc::delete_Dwg_AcDs_SchemaData_UProp", _wrap_delete_Dwg_AcDs_SchemaData_UProp},
{"LibreDWGc::Dwg_AcDs_SchemaData_num_uprops_set", _wrap_Dwg_AcDs_SchemaData_num_uprops_set},
{"LibreDWGc::Dwg_AcDs_SchemaData_num_uprops_get", _wrap_Dwg_AcDs_SchemaData_num_uprops_get},
{"LibreDWGc::Dwg_AcDs_SchemaData_uprops_set", _wrap_Dwg_AcDs_SchemaData_uprops_set},
{"LibreDWGc::Dwg_AcDs_SchemaData_uprops_get", _wrap_Dwg_AcDs_SchemaData_uprops_get},
{"LibreDWGc::Dwg_AcDs_SchemaData_num_schemas_set", _wrap_Dwg_AcDs_SchemaData_num_schemas_set},
{"LibreDWGc::Dwg_AcDs_SchemaData_num_schemas_get", _wrap_Dwg_AcDs_SchemaData_num_schemas_get},
{"LibreDWGc::Dwg_AcDs_SchemaData_schemas_set", _wrap_Dwg_AcDs_SchemaData_schemas_set},
{"LibreDWGc::Dwg_AcDs_SchemaData_schemas_get", _wrap_Dwg_AcDs_SchemaData_schemas_get},
{"LibreDWGc::Dwg_AcDs_SchemaData_num_propnames_set", _wrap_Dwg_AcDs_SchemaData_num_propnames_set},
{"LibreDWGc::Dwg_AcDs_SchemaData_num_propnames_get", _wrap_Dwg_AcDs_SchemaData_num_propnames_get},
{"LibreDWGc::Dwg_AcDs_SchemaData_propnames_set", _wrap_Dwg_AcDs_SchemaData_propnames_set},
{"LibreDWGc::Dwg_AcDs_SchemaData_propnames_get", _wrap_Dwg_AcDs_SchemaData_propnames_get},
{"LibreDWGc::new_Dwg_AcDs_SchemaData", _wrap_new_Dwg_AcDs_SchemaData},
{"LibreDWGc::delete_Dwg_AcDs_SchemaData", _wrap_delete_Dwg_AcDs_SchemaData},
{"LibreDWGc::Dwg_AcDs_Search_IdIdx_handle_set", _wrap_Dwg_AcDs_Search_IdIdx_handle_set},
{"LibreDWGc::Dwg_AcDs_Search_IdIdx_handle_get", _wrap_Dwg_AcDs_Search_IdIdx_handle_get},
{"LibreDWGc::Dwg_AcDs_Search_IdIdx_num_ididx_set", _wrap_Dwg_AcDs_Search_IdIdx_num_ididx_set},
{"LibreDWGc::Dwg_AcDs_Search_IdIdx_num_ididx_get", _wrap_Dwg_AcDs_Search_IdIdx_num_ididx_get},
{"LibreDWGc::Dwg_AcDs_Search_IdIdx_ididx_set", _wrap_Dwg_AcDs_Search_IdIdx_ididx_set},
{"LibreDWGc::Dwg_AcDs_Search_IdIdx_ididx_get", _wrap_Dwg_AcDs_Search_IdIdx_ididx_get},
{"LibreDWGc::new_Dwg_AcDs_Search_IdIdx", _wrap_new_Dwg_AcDs_Search_IdIdx},
{"LibreDWGc::delete_Dwg_AcDs_Search_IdIdx", _wrap_delete_Dwg_AcDs_Search_IdIdx},
{"LibreDWGc::Dwg_AcDs_Search_IdIdxs_num_ididx_set", _wrap_Dwg_AcDs_Search_IdIdxs_num_ididx_set},
{"LibreDWGc::Dwg_AcDs_Search_IdIdxs_num_ididx_get", _wrap_Dwg_AcDs_Search_IdIdxs_num_ididx_get},
{"LibreDWGc::Dwg_AcDs_Search_IdIdxs_ididx_set", _wrap_Dwg_AcDs_Search_IdIdxs_ididx_set},
{"LibreDWGc::Dwg_AcDs_Search_IdIdxs_ididx_get", _wrap_Dwg_AcDs_Search_IdIdxs_ididx_get},
{"LibreDWGc::new_Dwg_AcDs_Search_IdIdxs", _wrap_new_Dwg_AcDs_Search_IdIdxs},
{"LibreDWGc::delete_Dwg_AcDs_Search_IdIdxs", _wrap_delete_Dwg_AcDs_Search_IdIdxs},
{"LibreDWGc::Dwg_AcDs_Search_Data_schema_namidx_set", _wrap_Dwg_AcDs_Search_Data_schema_namidx_set},
{"LibreDWGc::Dwg_AcDs_Search_Data_schema_namidx_get", _wrap_Dwg_AcDs_Search_Data_schema_namidx_get},
{"LibreDWGc::Dwg_AcDs_Search_Data_num_sortedidx_set", _wrap_Dwg_AcDs_Search_Data_num_sortedidx_set},
{"LibreDWGc::Dwg_AcDs_Search_Data_num_sortedidx_get", _wrap_Dwg_AcDs_Search_Data_num_sortedidx_get},
{"LibreDWGc::Dwg_AcDs_Search_Data_sortedidx_set", _wrap_Dwg_AcDs_Search_Data_sortedidx_set},
{"LibreDWGc::Dwg_AcDs_Search_Data_sortedidx_get", _wrap_Dwg_AcDs_Search_Data_sortedidx_get},
{"LibreDWGc::Dwg_AcDs_Search_Data_num_ididxs_set", _wrap_Dwg_AcDs_Search_Data_num_ididxs_set},
{"LibreDWGc::Dwg_AcDs_Search_Data_num_ididxs_get", _wrap_Dwg_AcDs_Search_Data_num_ididxs_get},
{"LibreDWGc::Dwg_AcDs_Search_Data_unknown_set", _wrap_Dwg_AcDs_Search_Data_unknown_set},
{"LibreDWGc::Dwg_AcDs_Search_Data_unknown_get", _wrap_Dwg_AcDs_Search_Data_unknown_get},
{"LibreDWGc::Dwg_AcDs_Search_Data_ididxs_set", _wrap_Dwg_AcDs_Search_Data_ididxs_set},
{"LibreDWGc::Dwg_AcDs_Search_Data_ididxs_get", _wrap_Dwg_AcDs_Search_Data_ididxs_get},
{"LibreDWGc::new_Dwg_AcDs_Search_Data", _wrap_new_Dwg_AcDs_Search_Data},
{"LibreDWGc::delete_Dwg_AcDs_Search_Data", _wrap_delete_Dwg_AcDs_Search_Data},
{"LibreDWGc::Dwg_AcDs_Search_num_search_set", _wrap_Dwg_AcDs_Search_num_search_set},
{"LibreDWGc::Dwg_AcDs_Search_num_search_get", _wrap_Dwg_AcDs_Search_num_search_get},
{"LibreDWGc::Dwg_AcDs_Search_search_set", _wrap_Dwg_AcDs_Search_search_set},
{"LibreDWGc::Dwg_AcDs_Search_search_get", _wrap_Dwg_AcDs_Search_search_get},
{"LibreDWGc::new_Dwg_AcDs_Search", _wrap_new_Dwg_AcDs_Search},
{"LibreDWGc::delete_Dwg_AcDs_Search", _wrap_delete_Dwg_AcDs_Search},
{"LibreDWGc::Dwg_AcDs_Segment_signature_set", _wrap_Dwg_AcDs_Segment_signature_set},
{"LibreDWGc::Dwg_AcDs_Segment_signature_get", _wrap_Dwg_AcDs_Segment_signature_get},
{"LibreDWGc::Dwg_AcDs_Segment_name_set", _wrap_Dwg_AcDs_Segment_name_set},
{"LibreDWGc::Dwg_AcDs_Segment_name_get", _wrap_Dwg_AcDs_Segment_name_get},
{"LibreDWGc::Dwg_AcDs_Segment_type_set", _wrap_Dwg_AcDs_Segment_type_set},
{"LibreDWGc::Dwg_AcDs_Segment_type_get", _wrap_Dwg_AcDs_Segment_type_get},
{"LibreDWGc::Dwg_AcDs_Segment_segment_idx_set", _wrap_Dwg_AcDs_Segment_segment_idx_set},
{"LibreDWGc::Dwg_AcDs_Segment_segment_idx_get", _wrap_Dwg_AcDs_Segment_segment_idx_get},
{"LibreDWGc::Dwg_AcDs_Segment_is_blob01_set", _wrap_Dwg_AcDs_Segment_is_blob01_set},
{"LibreDWGc::Dwg_AcDs_Segment_is_blob01_get", _wrap_Dwg_AcDs_Segment_is_blob01_get},
{"LibreDWGc::Dwg_AcDs_Segment_segsize_set", _wrap_Dwg_AcDs_Segment_segsize_set},
{"LibreDWGc::Dwg_AcDs_Segment_segsize_get", _wrap_Dwg_AcDs_Segment_segsize_get},
{"LibreDWGc::Dwg_AcDs_Segment_unknown_2_set", _wrap_Dwg_AcDs_Segment_unknown_2_set},
{"LibreDWGc::Dwg_AcDs_Segment_unknown_2_get", _wrap_Dwg_AcDs_Segment_unknown_2_get},
{"LibreDWGc::Dwg_AcDs_Segment_ds_version_set", _wrap_Dwg_AcDs_Segment_ds_version_set},
{"LibreDWGc::Dwg_AcDs_Segment_ds_version_get", _wrap_Dwg_AcDs_Segment_ds_version_get},
{"LibreDWGc::Dwg_AcDs_Segment_unknown_3_set", _wrap_Dwg_AcDs_Segment_unknown_3_set},
{"LibreDWGc::Dwg_AcDs_Segment_unknown_3_get", _wrap_Dwg_AcDs_Segment_unknown_3_get},
{"LibreDWGc::Dwg_AcDs_Segment_data_algn_offset_set", _wrap_Dwg_AcDs_Segment_data_algn_offset_set},
{"LibreDWGc::Dwg_AcDs_Segment_data_algn_offset_get", _wrap_Dwg_AcDs_Segment_data_algn_offset_get},
{"LibreDWGc::Dwg_AcDs_Segment_objdata_algn_offset_set", _wrap_Dwg_AcDs_Segment_objdata_algn_offset_set},
{"LibreDWGc::Dwg_AcDs_Segment_objdata_algn_offset_get", _wrap_Dwg_AcDs_Segment_objdata_algn_offset_get},
{"LibreDWGc::Dwg_AcDs_Segment_padding_set", _wrap_Dwg_AcDs_Segment_padding_set},
{"LibreDWGc::Dwg_AcDs_Segment_padding_get", _wrap_Dwg_AcDs_Segment_padding_get},
{"LibreDWGc::new_Dwg_AcDs_Segment", _wrap_new_Dwg_AcDs_Segment},
{"LibreDWGc::delete_Dwg_AcDs_Segment", _wrap_delete_Dwg_AcDs_Segment},
{"LibreDWGc::Dwg_AcDs_file_signature_set", _wrap_Dwg_AcDs_file_signature_set},
{"LibreDWGc::Dwg_AcDs_file_signature_get", _wrap_Dwg_AcDs_file_signature_get},
{"LibreDWGc::Dwg_AcDs_file_header_size_set", _wrap_Dwg_AcDs_file_header_size_set},
{"LibreDWGc::Dwg_AcDs_file_header_size_get", _wrap_Dwg_AcDs_file_header_size_get},
{"LibreDWGc::Dwg_AcDs_unknown_1_set", _wrap_Dwg_AcDs_unknown_1_set},
{"LibreDWGc::Dwg_AcDs_unknown_1_get", _wrap_Dwg_AcDs_unknown_1_get},
{"LibreDWGc::Dwg_AcDs_version_set", _wrap_Dwg_AcDs_version_set},
{"LibreDWGc::Dwg_AcDs_version_get", _wrap_Dwg_AcDs_version_get},
{"LibreDWGc::Dwg_AcDs_unknown_2_set", _wrap_Dwg_AcDs_unknown_2_set},
{"LibreDWGc::Dwg_AcDs_unknown_2_get", _wrap_Dwg_AcDs_unknown_2_get},
{"LibreDWGc::Dwg_AcDs_ds_version_set", _wrap_Dwg_AcDs_ds_version_set},
{"LibreDWGc::Dwg_AcDs_ds_version_get", _wrap_Dwg_AcDs_ds_version_get},
{"LibreDWGc::Dwg_AcDs_segidx_offset_set", _wrap_Dwg_AcDs_segidx_offset_set},
{"LibreDWGc::Dwg_AcDs_segidx_offset_get", _wrap_Dwg_AcDs_segidx_offset_get},
{"LibreDWGc::Dwg_AcDs_segidx_unknown_set", _wrap_Dwg_AcDs_segidx_unknown_set},
{"LibreDWGc::Dwg_AcDs_segidx_unknown_get", _wrap_Dwg_AcDs_segidx_unknown_get},
{"LibreDWGc::Dwg_AcDs_num_segidx_set", _wrap_Dwg_AcDs_num_segidx_set},
{"LibreDWGc::Dwg_AcDs_num_segidx_get", _wrap_Dwg_AcDs_num_segidx_get},
{"LibreDWGc::Dwg_AcDs_schidx_segidx_set", _wrap_Dwg_AcDs_schidx_segidx_set},
{"LibreDWGc::Dwg_AcDs_schidx_segidx_get", _wrap_Dwg_AcDs_schidx_segidx_get},
{"LibreDWGc::Dwg_AcDs_datidx_segidx_set", _wrap_Dwg_AcDs_datidx_segidx_set},
{"LibreDWGc::Dwg_AcDs_datidx_segidx_get", _wrap_Dwg_AcDs_datidx_segidx_get},
{"LibreDWGc::Dwg_AcDs_search_segidx_set", _wrap_Dwg_AcDs_search_segidx_set},
{"LibreDWGc::Dwg_AcDs_search_segidx_get", _wrap_Dwg_AcDs_search_segidx_get},
{"LibreDWGc::Dwg_AcDs_prvsav_segidx_set", _wrap_Dwg_AcDs_prvsav_segidx_set},
{"LibreDWGc::Dwg_AcDs_prvsav_segidx_get", _wrap_Dwg_AcDs_prvsav_segidx_get},
{"LibreDWGc::Dwg_AcDs_file_size_set", _wrap_Dwg_AcDs_file_size_set},
{"LibreDWGc::Dwg_AcDs_file_size_get", _wrap_Dwg_AcDs_file_size_get},
{"LibreDWGc::Dwg_AcDs_total_segments_set", _wrap_Dwg_AcDs_total_segments_set},
{"LibreDWGc::Dwg_AcDs_total_segments_get", _wrap_Dwg_AcDs_total_segments_get},
{"LibreDWGc::Dwg_AcDs_segidx_set", _wrap_Dwg_AcDs_segidx_set},
{"LibreDWGc::Dwg_AcDs_segidx_get", _wrap_Dwg_AcDs_segidx_get},
{"LibreDWGc::Dwg_AcDs_datidx_set", _wrap_Dwg_AcDs_datidx_set},
{"LibreDWGc::Dwg_AcDs_datidx_get", _wrap_Dwg_AcDs_datidx_get},
{"LibreDWGc::Dwg_AcDs_data_set", _wrap_Dwg_AcDs_data_set},
{"LibreDWGc::Dwg_AcDs_data_get", _wrap_Dwg_AcDs_data_get},
{"LibreDWGc::Dwg_AcDs_blob01_set", _wrap_Dwg_AcDs_blob01_set},
{"LibreDWGc::Dwg_AcDs_blob01_get", _wrap_Dwg_AcDs_blob01_get},
{"LibreDWGc::Dwg_AcDs_schidx_set", _wrap_Dwg_AcDs_schidx_set},
{"LibreDWGc::Dwg_AcDs_schidx_get", _wrap_Dwg_AcDs_schidx_get},
{"LibreDWGc::Dwg_AcDs_schdat_set", _wrap_Dwg_AcDs_schdat_set},
{"LibreDWGc::Dwg_AcDs_schdat_get", _wrap_Dwg_AcDs_schdat_get},
{"LibreDWGc::Dwg_AcDs_search_set", _wrap_Dwg_AcDs_search_set},
{"LibreDWGc::Dwg_AcDs_search_get", _wrap_Dwg_AcDs_search_get},
{"LibreDWGc::Dwg_AcDs_segments_set", _wrap_Dwg_AcDs_segments_set},
{"LibreDWGc::Dwg_AcDs_segments_get", _wrap_Dwg_AcDs_segments_get},
{"LibreDWGc::new_Dwg_AcDs", _wrap_new_Dwg_AcDs},
{"LibreDWGc::delete_Dwg_AcDs", _wrap_delete_Dwg_AcDs},
{"LibreDWGc::Dwg_Header_version_set", _wrap_Dwg_Header_version_set},
{"LibreDWGc::Dwg_Header_version_get", _wrap_Dwg_Header_version_get},
{"LibreDWGc::Dwg_Header_from_version_set", _wrap_Dwg_Header_from_version_set},
{"LibreDWGc::Dwg_Header_from_version_get", _wrap_Dwg_Header_from_version_get},
{"LibreDWGc::Dwg_Header_is_maint_set", _wrap_Dwg_Header_is_maint_set},
{"LibreDWGc::Dwg_Header_is_maint_get", _wrap_Dwg_Header_is_maint_get},
{"LibreDWGc::Dwg_Header_zero_one_or_three_set", _wrap_Dwg_Header_zero_one_or_three_set},
{"LibreDWGc::Dwg_Header_zero_one_or_three_get", _wrap_Dwg_Header_zero_one_or_three_get},
{"LibreDWGc::Dwg_Header_numentity_sections_set", _wrap_Dwg_Header_numentity_sections_set},
{"LibreDWGc::Dwg_Header_numentity_sections_get", _wrap_Dwg_Header_numentity_sections_get},
{"LibreDWGc::Dwg_Header_numheader_vars_set", _wrap_Dwg_Header_numheader_vars_set},
{"LibreDWGc::Dwg_Header_numheader_vars_get", _wrap_Dwg_Header_numheader_vars_get},
{"LibreDWGc::Dwg_Header_thumbnail_address_set", _wrap_Dwg_Header_thumbnail_address_set},
{"LibreDWGc::Dwg_Header_thumbnail_address_get", _wrap_Dwg_Header_thumbnail_address_get},
{"LibreDWGc::Dwg_Header_dwg_version_set", _wrap_Dwg_Header_dwg_version_set},
{"LibreDWGc::Dwg_Header_dwg_version_get", _wrap_Dwg_Header_dwg_version_get},
{"LibreDWGc::Dwg_Header_maint_version_set", _wrap_Dwg_Header_maint_version_set},
{"LibreDWGc::Dwg_Header_maint_version_get", _wrap_Dwg_Header_maint_version_get},
{"LibreDWGc::Dwg_Header_entities_start_set", _wrap_Dwg_Header_entities_start_set},
{"LibreDWGc::Dwg_Header_entities_start_get", _wrap_Dwg_Header_entities_start_get},
{"LibreDWGc::Dwg_Header_entities_end_set", _wrap_Dwg_Header_entities_end_set},
{"LibreDWGc::Dwg_Header_entities_end_get", _wrap_Dwg_Header_entities_end_get},
{"LibreDWGc::Dwg_Header_blocks_start_set", _wrap_Dwg_Header_blocks_start_set},
{"LibreDWGc::Dwg_Header_blocks_start_get", _wrap_Dwg_Header_blocks_start_get},
{"LibreDWGc::Dwg_Header_blocks_size_set", _wrap_Dwg_Header_blocks_size_set},
{"LibreDWGc::Dwg_Header_blocks_size_get", _wrap_Dwg_Header_blocks_size_get},
{"LibreDWGc::Dwg_Header_extras_start_set", _wrap_Dwg_Header_extras_start_set},
{"LibreDWGc::Dwg_Header_extras_start_get", _wrap_Dwg_Header_extras_start_get},
{"LibreDWGc::Dwg_Header_extras_size_set", _wrap_Dwg_Header_extras_size_set},
{"LibreDWGc::Dwg_Header_extras_size_get", _wrap_Dwg_Header_extras_size_get},
{"LibreDWGc::Dwg_Header_codepage_set", _wrap_Dwg_Header_codepage_set},
{"LibreDWGc::Dwg_Header_codepage_get", _wrap_Dwg_Header_codepage_get},
{"LibreDWGc::Dwg_Header_unknown_0_set", _wrap_Dwg_Header_unknown_0_set},
{"LibreDWGc::Dwg_Header_unknown_0_get", _wrap_Dwg_Header_unknown_0_get},
{"LibreDWGc::Dwg_Header_app_dwg_version_set", _wrap_Dwg_Header_app_dwg_version_set},
{"LibreDWGc::Dwg_Header_app_dwg_version_get", _wrap_Dwg_Header_app_dwg_version_get},
{"LibreDWGc::Dwg_Header_app_maint_version_set", _wrap_Dwg_Header_app_maint_version_set},
{"LibreDWGc::Dwg_Header_app_maint_version_get", _wrap_Dwg_Header_app_maint_version_get},
{"LibreDWGc::Dwg_Header_security_type_set", _wrap_Dwg_Header_security_type_set},
{"LibreDWGc::Dwg_Header_security_type_get", _wrap_Dwg_Header_security_type_get},
{"LibreDWGc::Dwg_Header_rl_1c_address_set", _wrap_Dwg_Header_rl_1c_address_set},
{"LibreDWGc::Dwg_Header_rl_1c_address_get", _wrap_Dwg_Header_rl_1c_address_get},
{"LibreDWGc::Dwg_Header_summaryinfo_address_set", _wrap_Dwg_Header_summaryinfo_address_set},
{"LibreDWGc::Dwg_Header_summaryinfo_address_get", _wrap_Dwg_Header_summaryinfo_address_get},
{"LibreDWGc::Dwg_Header_vbaproj_address_set", _wrap_Dwg_Header_vbaproj_address_set},
{"LibreDWGc::Dwg_Header_vbaproj_address_get", _wrap_Dwg_Header_vbaproj_address_get},
{"LibreDWGc::Dwg_Header_r2004_header_address_set", _wrap_Dwg_Header_r2004_header_address_set},
{"LibreDWGc::Dwg_Header_r2004_header_address_get", _wrap_Dwg_Header_r2004_header_address_get},
{"LibreDWGc::Dwg_Header_sections_set", _wrap_Dwg_Header_sections_set},
{"LibreDWGc::Dwg_Header_sections_get", _wrap_Dwg_Header_sections_get},
{"LibreDWGc::Dwg_Header_num_sections_set", _wrap_Dwg_Header_num_sections_set},
{"LibreDWGc::Dwg_Header_num_sections_get", _wrap_Dwg_Header_num_sections_get},
{"LibreDWGc::Dwg_Header_section_set", _wrap_Dwg_Header_section_set},
{"LibreDWGc::Dwg_Header_section_get", _wrap_Dwg_Header_section_get},
{"LibreDWGc::Dwg_Header_section_infohdr_set", _wrap_Dwg_Header_section_infohdr_set},
{"LibreDWGc::Dwg_Header_section_infohdr_get", _wrap_Dwg_Header_section_infohdr_get},
{"LibreDWGc::Dwg_Header_section_info_set", _wrap_Dwg_Header_section_info_set},
{"LibreDWGc::Dwg_Header_section_info_get", _wrap_Dwg_Header_section_info_get},
{"LibreDWGc::new_Dwg_Header", _wrap_new_Dwg_Header},
{"LibreDWGc::delete_Dwg_Header", _wrap_delete_Dwg_Header},
{"LibreDWGc::Dwg_R2007_Header_header_size_set", _wrap_Dwg_R2007_Header_header_size_set},
{"LibreDWGc::Dwg_R2007_Header_header_size_get", _wrap_Dwg_R2007_Header_header_size_get},
{"LibreDWGc::Dwg_R2007_Header_file_size_set", _wrap_Dwg_R2007_Header_file_size_set},
{"LibreDWGc::Dwg_R2007_Header_file_size_get", _wrap_Dwg_R2007_Header_file_size_get},
{"LibreDWGc::Dwg_R2007_Header_pages_map_crc_compressed_set", _wrap_Dwg_R2007_Header_pages_map_crc_compressed_set},
{"LibreDWGc::Dwg_R2007_Header_pages_map_crc_compressed_get", _wrap_Dwg_R2007_Header_pages_map_crc_compressed_get},
{"LibreDWGc::Dwg_R2007_Header_pages_map_correction_set", _wrap_Dwg_R2007_Header_pages_map_correction_set},
{"LibreDWGc::Dwg_R2007_Header_pages_map_correction_get", _wrap_Dwg_R2007_Header_pages_map_correction_get},
{"LibreDWGc::Dwg_R2007_Header_pages_map_crc_seed_set", _wrap_Dwg_R2007_Header_pages_map_crc_seed_set},
{"LibreDWGc::Dwg_R2007_Header_pages_map_crc_seed_get", _wrap_Dwg_R2007_Header_pages_map_crc_seed_get},
{"LibreDWGc::Dwg_R2007_Header_pages_map2_offset_set", _wrap_Dwg_R2007_Header_pages_map2_offset_set},
{"LibreDWGc::Dwg_R2007_Header_pages_map2_offset_get", _wrap_Dwg_R2007_Header_pages_map2_offset_get},
{"LibreDWGc::Dwg_R2007_Header_pages_map2_id_set", _wrap_Dwg_R2007_Header_pages_map2_id_set},
{"LibreDWGc::Dwg_R2007_Header_pages_map2_id_get", _wrap_Dwg_R2007_Header_pages_map2_id_get},
{"LibreDWGc::Dwg_R2007_Header_pages_map_offset_set", _wrap_Dwg_R2007_Header_pages_map_offset_set},
{"LibreDWGc::Dwg_R2007_Header_pages_map_offset_get", _wrap_Dwg_R2007_Header_pages_map_offset_get},
{"LibreDWGc::Dwg_R2007_Header_pages_map_id_set", _wrap_Dwg_R2007_Header_pages_map_id_set},
{"LibreDWGc::Dwg_R2007_Header_pages_map_id_get", _wrap_Dwg_R2007_Header_pages_map_id_get},
{"LibreDWGc::Dwg_R2007_Header_header2_offset_set", _wrap_Dwg_R2007_Header_header2_offset_set},
{"LibreDWGc::Dwg_R2007_Header_header2_offset_get", _wrap_Dwg_R2007_Header_header2_offset_get},
{"LibreDWGc::Dwg_R2007_Header_pages_map_size_comp_set", _wrap_Dwg_R2007_Header_pages_map_size_comp_set},
{"LibreDWGc::Dwg_R2007_Header_pages_map_size_comp_get", _wrap_Dwg_R2007_Header_pages_map_size_comp_get},
{"LibreDWGc::Dwg_R2007_Header_pages_map_size_uncomp_set", _wrap_Dwg_R2007_Header_pages_map_size_uncomp_set},
{"LibreDWGc::Dwg_R2007_Header_pages_map_size_uncomp_get", _wrap_Dwg_R2007_Header_pages_map_size_uncomp_get},
{"LibreDWGc::Dwg_R2007_Header_pages_amount_set", _wrap_Dwg_R2007_Header_pages_amount_set},
{"LibreDWGc::Dwg_R2007_Header_pages_amount_get", _wrap_Dwg_R2007_Header_pages_amount_get},
{"LibreDWGc::Dwg_R2007_Header_pages_maxid_set", _wrap_Dwg_R2007_Header_pages_maxid_set},
{"LibreDWGc::Dwg_R2007_Header_pages_maxid_get", _wrap_Dwg_R2007_Header_pages_maxid_get},
{"LibreDWGc::Dwg_R2007_Header_unknown1_set", _wrap_Dwg_R2007_Header_unknown1_set},
{"LibreDWGc::Dwg_R2007_Header_unknown1_get", _wrap_Dwg_R2007_Header_unknown1_get},
{"LibreDWGc::Dwg_R2007_Header_unknown2_set", _wrap_Dwg_R2007_Header_unknown2_set},
{"LibreDWGc::Dwg_R2007_Header_unknown2_get", _wrap_Dwg_R2007_Header_unknown2_get},
{"LibreDWGc::Dwg_R2007_Header_pages_map_crc_uncomp_set", _wrap_Dwg_R2007_Header_pages_map_crc_uncomp_set},
{"LibreDWGc::Dwg_R2007_Header_pages_map_crc_uncomp_get", _wrap_Dwg_R2007_Header_pages_map_crc_uncomp_get},
{"LibreDWGc::Dwg_R2007_Header_unknown3_set", _wrap_Dwg_R2007_Header_unknown3_set},
{"LibreDWGc::Dwg_R2007_Header_unknown3_get", _wrap_Dwg_R2007_Header_unknown3_get},
{"LibreDWGc::Dwg_R2007_Header_unknown4_set", _wrap_Dwg_R2007_Header_unknown4_set},
{"LibreDWGc::Dwg_R2007_Header_unknown4_get", _wrap_Dwg_R2007_Header_unknown4_get},
{"LibreDWGc::Dwg_R2007_Header_unknown5_set", _wrap_Dwg_R2007_Header_unknown5_set},
{"LibreDWGc::Dwg_R2007_Header_unknown5_get", _wrap_Dwg_R2007_Header_unknown5_get},
{"LibreDWGc::Dwg_R2007_Header_num_sections_set", _wrap_Dwg_R2007_Header_num_sections_set},
{"LibreDWGc::Dwg_R2007_Header_num_sections_get", _wrap_Dwg_R2007_Header_num_sections_get},
{"LibreDWGc::Dwg_R2007_Header_sections_map_crc_uncomp_set", _wrap_Dwg_R2007_Header_sections_map_crc_uncomp_set},
{"LibreDWGc::Dwg_R2007_Header_sections_map_crc_uncomp_get", _wrap_Dwg_R2007_Header_sections_map_crc_uncomp_get},
{"LibreDWGc::Dwg_R2007_Header_sections_map_size_comp_set", _wrap_Dwg_R2007_Header_sections_map_size_comp_set},
{"LibreDWGc::Dwg_R2007_Header_sections_map_size_comp_get", _wrap_Dwg_R2007_Header_sections_map_size_comp_get},
{"LibreDWGc::Dwg_R2007_Header_sections_map2_id_set", _wrap_Dwg_R2007_Header_sections_map2_id_set},
{"LibreDWGc::Dwg_R2007_Header_sections_map2_id_get", _wrap_Dwg_R2007_Header_sections_map2_id_get},
{"LibreDWGc::Dwg_R2007_Header_sections_map_id_set", _wrap_Dwg_R2007_Header_sections_map_id_set},
{"LibreDWGc::Dwg_R2007_Header_sections_map_id_get", _wrap_Dwg_R2007_Header_sections_map_id_get},
{"LibreDWGc::Dwg_R2007_Header_sections_map_size_uncomp_set", _wrap_Dwg_R2007_Header_sections_map_size_uncomp_set},
{"LibreDWGc::Dwg_R2007_Header_sections_map_size_uncomp_get", _wrap_Dwg_R2007_Header_sections_map_size_uncomp_get},
{"LibreDWGc::Dwg_R2007_Header_sections_map_crc_comp_set", _wrap_Dwg_R2007_Header_sections_map_crc_comp_set},
{"LibreDWGc::Dwg_R2007_Header_sections_map_crc_comp_get", _wrap_Dwg_R2007_Header_sections_map_crc_comp_get},
{"LibreDWGc::Dwg_R2007_Header_sections_map_correction_set", _wrap_Dwg_R2007_Header_sections_map_correction_set},
{"LibreDWGc::Dwg_R2007_Header_sections_map_correction_get", _wrap_Dwg_R2007_Header_sections_map_correction_get},
{"LibreDWGc::Dwg_R2007_Header_sections_map_crc_seed_set", _wrap_Dwg_R2007_Header_sections_map_crc_seed_set},
{"LibreDWGc::Dwg_R2007_Header_sections_map_crc_seed_get", _wrap_Dwg_R2007_Header_sections_map_crc_seed_get},
{"LibreDWGc::Dwg_R2007_Header_stream_version_set", _wrap_Dwg_R2007_Header_stream_version_set},
{"LibreDWGc::Dwg_R2007_Header_stream_version_get", _wrap_Dwg_R2007_Header_stream_version_get},
{"LibreDWGc::Dwg_R2007_Header_crc_seed_set", _wrap_Dwg_R2007_Header_crc_seed_set},
{"LibreDWGc::Dwg_R2007_Header_crc_seed_get", _wrap_Dwg_R2007_Header_crc_seed_get},
{"LibreDWGc::Dwg_R2007_Header_crc_seed_encoded_set", _wrap_Dwg_R2007_Header_crc_seed_encoded_set},
{"LibreDWGc::Dwg_R2007_Header_crc_seed_encoded_get", _wrap_Dwg_R2007_Header_crc_seed_encoded_get},
{"LibreDWGc::Dwg_R2007_Header_random_seed_set", _wrap_Dwg_R2007_Header_random_seed_set},
{"LibreDWGc::Dwg_R2007_Header_random_seed_get", _wrap_Dwg_R2007_Header_random_seed_get},
{"LibreDWGc::Dwg_R2007_Header_header_crc_set", _wrap_Dwg_R2007_Header_header_crc_set},
{"LibreDWGc::Dwg_R2007_Header_header_crc_get", _wrap_Dwg_R2007_Header_header_crc_get},
{"LibreDWGc::new_Dwg_R2007_Header", _wrap_new_Dwg_R2007_Header},
{"LibreDWGc::delete_Dwg_R2007_Header", _wrap_delete_Dwg_R2007_Header},
{"LibreDWGc::Dwg_R2004_Header_file_ID_string_set", _wrap_Dwg_R2004_Header_file_ID_string_set},
{"LibreDWGc::Dwg_R2004_Header_file_ID_string_get", _wrap_Dwg_R2004_Header_file_ID_string_get},
{"LibreDWGc::Dwg_R2004_Header_header_address_set", _wrap_Dwg_R2004_Header_header_address_set},
{"LibreDWGc::Dwg_R2004_Header_header_address_get", _wrap_Dwg_R2004_Header_header_address_get},
{"LibreDWGc::Dwg_R2004_Header_header_size_set", _wrap_Dwg_R2004_Header_header_size_set},
{"LibreDWGc::Dwg_R2004_Header_header_size_get", _wrap_Dwg_R2004_Header_header_size_get},
{"LibreDWGc::Dwg_R2004_Header_x04_set", _wrap_Dwg_R2004_Header_x04_set},
{"LibreDWGc::Dwg_R2004_Header_x04_get", _wrap_Dwg_R2004_Header_x04_get},
{"LibreDWGc::Dwg_R2004_Header_root_tree_node_gap_set", _wrap_Dwg_R2004_Header_root_tree_node_gap_set},
{"LibreDWGc::Dwg_R2004_Header_root_tree_node_gap_get", _wrap_Dwg_R2004_Header_root_tree_node_gap_get},
{"LibreDWGc::Dwg_R2004_Header_lowermost_left_tree_node_gap_set", _wrap_Dwg_R2004_Header_lowermost_left_tree_node_gap_set},
{"LibreDWGc::Dwg_R2004_Header_lowermost_left_tree_node_gap_get", _wrap_Dwg_R2004_Header_lowermost_left_tree_node_gap_get},
{"LibreDWGc::Dwg_R2004_Header_lowermost_right_tree_node_gap_set", _wrap_Dwg_R2004_Header_lowermost_right_tree_node_gap_set},
{"LibreDWGc::Dwg_R2004_Header_lowermost_right_tree_node_gap_get", _wrap_Dwg_R2004_Header_lowermost_right_tree_node_gap_get},
{"LibreDWGc::Dwg_R2004_Header_unknown_long_set", _wrap_Dwg_R2004_Header_unknown_long_set},
{"LibreDWGc::Dwg_R2004_Header_unknown_long_get", _wrap_Dwg_R2004_Header_unknown_long_get},
{"LibreDWGc::Dwg_R2004_Header_last_section_id_set", _wrap_Dwg_R2004_Header_last_section_id_set},
{"LibreDWGc::Dwg_R2004_Header_last_section_id_get", _wrap_Dwg_R2004_Header_last_section_id_get},
{"LibreDWGc::Dwg_R2004_Header_last_section_address_set", _wrap_Dwg_R2004_Header_last_section_address_set},
{"LibreDWGc::Dwg_R2004_Header_last_section_address_get", _wrap_Dwg_R2004_Header_last_section_address_get},
{"LibreDWGc::Dwg_R2004_Header_secondheader_address_set", _wrap_Dwg_R2004_Header_secondheader_address_set},
{"LibreDWGc::Dwg_R2004_Header_secondheader_address_get", _wrap_Dwg_R2004_Header_secondheader_address_get},
{"LibreDWGc::Dwg_R2004_Header_numgaps_set", _wrap_Dwg_R2004_Header_numgaps_set},
{"LibreDWGc::Dwg_R2004_Header_numgaps_get", _wrap_Dwg_R2004_Header_numgaps_get},
{"LibreDWGc::Dwg_R2004_Header_numsections_set", _wrap_Dwg_R2004_Header_numsections_set},
{"LibreDWGc::Dwg_R2004_Header_numsections_get", _wrap_Dwg_R2004_Header_numsections_get},
{"LibreDWGc::Dwg_R2004_Header_x20_set", _wrap_Dwg_R2004_Header_x20_set},
{"LibreDWGc::Dwg_R2004_Header_x20_get", _wrap_Dwg_R2004_Header_x20_get},
{"LibreDWGc::Dwg_R2004_Header_x80_set", _wrap_Dwg_R2004_Header_x80_set},
{"LibreDWGc::Dwg_R2004_Header_x80_get", _wrap_Dwg_R2004_Header_x80_get},
{"LibreDWGc::Dwg_R2004_Header_x40_set", _wrap_Dwg_R2004_Header_x40_set},
{"LibreDWGc::Dwg_R2004_Header_x40_get", _wrap_Dwg_R2004_Header_x40_get},
{"LibreDWGc::Dwg_R2004_Header_section_map_id_set", _wrap_Dwg_R2004_Header_section_map_id_set},
{"LibreDWGc::Dwg_R2004_Header_section_map_id_get", _wrap_Dwg_R2004_Header_section_map_id_get},
{"LibreDWGc::Dwg_R2004_Header_section_map_address_set", _wrap_Dwg_R2004_Header_section_map_address_set},
{"LibreDWGc::Dwg_R2004_Header_section_map_address_get", _wrap_Dwg_R2004_Header_section_map_address_get},
{"LibreDWGc::Dwg_R2004_Header_section_info_id_set", _wrap_Dwg_R2004_Header_section_info_id_set},
{"LibreDWGc::Dwg_R2004_Header_section_info_id_get", _wrap_Dwg_R2004_Header_section_info_id_get},
{"LibreDWGc::Dwg_R2004_Header_section_array_size_set", _wrap_Dwg_R2004_Header_section_array_size_set},
{"LibreDWGc::Dwg_R2004_Header_section_array_size_get", _wrap_Dwg_R2004_Header_section_array_size_get},
{"LibreDWGc::Dwg_R2004_Header_gap_array_size_set", _wrap_Dwg_R2004_Header_gap_array_size_set},
{"LibreDWGc::Dwg_R2004_Header_gap_array_size_get", _wrap_Dwg_R2004_Header_gap_array_size_get},
{"LibreDWGc::Dwg_R2004_Header_crc32_set", _wrap_Dwg_R2004_Header_crc32_set},
{"LibreDWGc::Dwg_R2004_Header_crc32_get", _wrap_Dwg_R2004_Header_crc32_get},
{"LibreDWGc::Dwg_R2004_Header_padding_set", _wrap_Dwg_R2004_Header_padding_set},
{"LibreDWGc::Dwg_R2004_Header_padding_get", _wrap_Dwg_R2004_Header_padding_get},
{"LibreDWGc::Dwg_R2004_Header_section_type_set", _wrap_Dwg_R2004_Header_section_type_set},
{"LibreDWGc::Dwg_R2004_Header_section_type_get", _wrap_Dwg_R2004_Header_section_type_get},
{"LibreDWGc::Dwg_R2004_Header_decomp_data_size_set", _wrap_Dwg_R2004_Header_decomp_data_size_set},
{"LibreDWGc::Dwg_R2004_Header_decomp_data_size_get", _wrap_Dwg_R2004_Header_decomp_data_size_get},
{"LibreDWGc::Dwg_R2004_Header_comp_data_size_set", _wrap_Dwg_R2004_Header_comp_data_size_set},
{"LibreDWGc::Dwg_R2004_Header_comp_data_size_get", _wrap_Dwg_R2004_Header_comp_data_size_get},
{"LibreDWGc::Dwg_R2004_Header_compression_type_set", _wrap_Dwg_R2004_Header_compression_type_set},
{"LibreDWGc::Dwg_R2004_Header_compression_type_get", _wrap_Dwg_R2004_Header_compression_type_get},
{"LibreDWGc::Dwg_R2004_Header_checksum_set", _wrap_Dwg_R2004_Header_checksum_set},
{"LibreDWGc::Dwg_R2004_Header_checksum_get", _wrap_Dwg_R2004_Header_checksum_get},
{"LibreDWGc::new_Dwg_R2004_Header", _wrap_new_Dwg_R2004_Header},
{"LibreDWGc::delete_Dwg_R2004_Header", _wrap_delete_Dwg_R2004_Header},
{"LibreDWGc::Dwg_AuxHeader_aux_intro_set", _wrap_Dwg_AuxHeader_aux_intro_set},
{"LibreDWGc::Dwg_AuxHeader_aux_intro_get", _wrap_Dwg_AuxHeader_aux_intro_get},
{"LibreDWGc::Dwg_AuxHeader_dwg_version_set", _wrap_Dwg_AuxHeader_dwg_version_set},
{"LibreDWGc::Dwg_AuxHeader_dwg_version_get", _wrap_Dwg_AuxHeader_dwg_version_get},
{"LibreDWGc::Dwg_AuxHeader_maint_version_set", _wrap_Dwg_AuxHeader_maint_version_set},
{"LibreDWGc::Dwg_AuxHeader_maint_version_get", _wrap_Dwg_AuxHeader_maint_version_get},
{"LibreDWGc::Dwg_AuxHeader_numsaves_set", _wrap_Dwg_AuxHeader_numsaves_set},
{"LibreDWGc::Dwg_AuxHeader_numsaves_get", _wrap_Dwg_AuxHeader_numsaves_get},
{"LibreDWGc::Dwg_AuxHeader_minus_1_set", _wrap_Dwg_AuxHeader_minus_1_set},
{"LibreDWGc::Dwg_AuxHeader_minus_1_get", _wrap_Dwg_AuxHeader_minus_1_get},
{"LibreDWGc::Dwg_AuxHeader_numsaves_1_set", _wrap_Dwg_AuxHeader_numsaves_1_set},
{"LibreDWGc::Dwg_AuxHeader_numsaves_1_get", _wrap_Dwg_AuxHeader_numsaves_1_get},
{"LibreDWGc::Dwg_AuxHeader_numsaves_2_set", _wrap_Dwg_AuxHeader_numsaves_2_set},
{"LibreDWGc::Dwg_AuxHeader_numsaves_2_get", _wrap_Dwg_AuxHeader_numsaves_2_get},
{"LibreDWGc::Dwg_AuxHeader_zero_set", _wrap_Dwg_AuxHeader_zero_set},
{"LibreDWGc::Dwg_AuxHeader_zero_get", _wrap_Dwg_AuxHeader_zero_get},
{"LibreDWGc::Dwg_AuxHeader_dwg_version_1_set", _wrap_Dwg_AuxHeader_dwg_version_1_set},
{"LibreDWGc::Dwg_AuxHeader_dwg_version_1_get", _wrap_Dwg_AuxHeader_dwg_version_1_get},
{"LibreDWGc::Dwg_AuxHeader_maint_version_1_set", _wrap_Dwg_AuxHeader_maint_version_1_set},
{"LibreDWGc::Dwg_AuxHeader_maint_version_1_get", _wrap_Dwg_AuxHeader_maint_version_1_get},
{"LibreDWGc::Dwg_AuxHeader_dwg_version_2_set", _wrap_Dwg_AuxHeader_dwg_version_2_set},
{"LibreDWGc::Dwg_AuxHeader_dwg_version_2_get", _wrap_Dwg_AuxHeader_dwg_version_2_get},
{"LibreDWGc::Dwg_AuxHeader_maint_version_2_set", _wrap_Dwg_AuxHeader_maint_version_2_set},
{"LibreDWGc::Dwg_AuxHeader_maint_version_2_get", _wrap_Dwg_AuxHeader_maint_version_2_get},
{"LibreDWGc::Dwg_AuxHeader_unknown_6rs_set", _wrap_Dwg_AuxHeader_unknown_6rs_set},
{"LibreDWGc::Dwg_AuxHeader_unknown_6rs_get", _wrap_Dwg_AuxHeader_unknown_6rs_get},
{"LibreDWGc::Dwg_AuxHeader_unknown_5rl_set", _wrap_Dwg_AuxHeader_unknown_5rl_set},
{"LibreDWGc::Dwg_AuxHeader_unknown_5rl_get", _wrap_Dwg_AuxHeader_unknown_5rl_get},
{"LibreDWGc::Dwg_AuxHeader_TDCREATE_set", _wrap_Dwg_AuxHeader_TDCREATE_set},
{"LibreDWGc::Dwg_AuxHeader_TDCREATE_get", _wrap_Dwg_AuxHeader_TDCREATE_get},
{"LibreDWGc::Dwg_AuxHeader_TDUPDATE_set", _wrap_Dwg_AuxHeader_TDUPDATE_set},
{"LibreDWGc::Dwg_AuxHeader_TDUPDATE_get", _wrap_Dwg_AuxHeader_TDUPDATE_get},
{"LibreDWGc::Dwg_AuxHeader_HANDSEED_set", _wrap_Dwg_AuxHeader_HANDSEED_set},
{"LibreDWGc::Dwg_AuxHeader_HANDSEED_get", _wrap_Dwg_AuxHeader_HANDSEED_get},
{"LibreDWGc::Dwg_AuxHeader_plot_stamp_set", _wrap_Dwg_AuxHeader_plot_stamp_set},
{"LibreDWGc::Dwg_AuxHeader_plot_stamp_get", _wrap_Dwg_AuxHeader_plot_stamp_get},
{"LibreDWGc::Dwg_AuxHeader_zero_1_set", _wrap_Dwg_AuxHeader_zero_1_set},
{"LibreDWGc::Dwg_AuxHeader_zero_1_get", _wrap_Dwg_AuxHeader_zero_1_get},
{"LibreDWGc::Dwg_AuxHeader_numsaves_3_set", _wrap_Dwg_AuxHeader_numsaves_3_set},
{"LibreDWGc::Dwg_AuxHeader_numsaves_3_get", _wrap_Dwg_AuxHeader_numsaves_3_get},
{"LibreDWGc::Dwg_AuxHeader_zero_2_set", _wrap_Dwg_AuxHeader_zero_2_set},
{"LibreDWGc::Dwg_AuxHeader_zero_2_get", _wrap_Dwg_AuxHeader_zero_2_get},
{"LibreDWGc::Dwg_AuxHeader_zero_3_set", _wrap_Dwg_AuxHeader_zero_3_set},
{"LibreDWGc::Dwg_AuxHeader_zero_3_get", _wrap_Dwg_AuxHeader_zero_3_get},
{"LibreDWGc::Dwg_AuxHeader_zero_4_set", _wrap_Dwg_AuxHeader_zero_4_set},
{"LibreDWGc::Dwg_AuxHeader_zero_4_get", _wrap_Dwg_AuxHeader_zero_4_get},
{"LibreDWGc::Dwg_AuxHeader_numsaves_4_set", _wrap_Dwg_AuxHeader_numsaves_4_set},
{"LibreDWGc::Dwg_AuxHeader_numsaves_4_get", _wrap_Dwg_AuxHeader_numsaves_4_get},
{"LibreDWGc::Dwg_AuxHeader_zero_5_set", _wrap_Dwg_AuxHeader_zero_5_set},
{"LibreDWGc::Dwg_AuxHeader_zero_5_get", _wrap_Dwg_AuxHeader_zero_5_get},
{"LibreDWGc::Dwg_AuxHeader_zero_6_set", _wrap_Dwg_AuxHeader_zero_6_set},
{"LibreDWGc::Dwg_AuxHeader_zero_6_get", _wrap_Dwg_AuxHeader_zero_6_get},
{"LibreDWGc::Dwg_AuxHeader_zero_7_set", _wrap_Dwg_AuxHeader_zero_7_set},
{"LibreDWGc::Dwg_AuxHeader_zero_7_get", _wrap_Dwg_AuxHeader_zero_7_get},
{"LibreDWGc::Dwg_AuxHeader_zero_8_set", _wrap_Dwg_AuxHeader_zero_8_set},
{"LibreDWGc::Dwg_AuxHeader_zero_8_get", _wrap_Dwg_AuxHeader_zero_8_get},
{"LibreDWGc::Dwg_AuxHeader_zero_18_set", _wrap_Dwg_AuxHeader_zero_18_set},
{"LibreDWGc::Dwg_AuxHeader_zero_18_get", _wrap_Dwg_AuxHeader_zero_18_get},
{"LibreDWGc::Dwg_AuxHeader_num_auxheader_variables_set", _wrap_Dwg_AuxHeader_num_auxheader_variables_set},
{"LibreDWGc::Dwg_AuxHeader_num_auxheader_variables_get", _wrap_Dwg_AuxHeader_num_auxheader_variables_get},
{"LibreDWGc::Dwg_AuxHeader_auxheader_size_set", _wrap_Dwg_AuxHeader_auxheader_size_set},
{"LibreDWGc::Dwg_AuxHeader_auxheader_size_get", _wrap_Dwg_AuxHeader_auxheader_size_get},
{"LibreDWGc::Dwg_AuxHeader_entities_start_set", _wrap_Dwg_AuxHeader_entities_start_set},
{"LibreDWGc::Dwg_AuxHeader_entities_start_get", _wrap_Dwg_AuxHeader_entities_start_get},
{"LibreDWGc::Dwg_AuxHeader_entities_end_set", _wrap_Dwg_AuxHeader_entities_end_set},
{"LibreDWGc::Dwg_AuxHeader_entities_end_get", _wrap_Dwg_AuxHeader_entities_end_get},
{"LibreDWGc::Dwg_AuxHeader_blocks_start_set", _wrap_Dwg_AuxHeader_blocks_start_set},
{"LibreDWGc::Dwg_AuxHeader_blocks_start_get", _wrap_Dwg_AuxHeader_blocks_start_get},
{"LibreDWGc::Dwg_AuxHeader_extras_start_set", _wrap_Dwg_AuxHeader_extras_start_set},
{"LibreDWGc::Dwg_AuxHeader_extras_start_get", _wrap_Dwg_AuxHeader_extras_start_get},
{"LibreDWGc::Dwg_AuxHeader_auxheader_address_set", _wrap_Dwg_AuxHeader_auxheader_address_set},
{"LibreDWGc::Dwg_AuxHeader_auxheader_address_get", _wrap_Dwg_AuxHeader_auxheader_address_get},
{"LibreDWGc::Dwg_AuxHeader_num_aux_tables_set", _wrap_Dwg_AuxHeader_num_aux_tables_set},
{"LibreDWGc::Dwg_AuxHeader_num_aux_tables_get", _wrap_Dwg_AuxHeader_num_aux_tables_get},
{"LibreDWGc::Dwg_AuxHeader_R11_HANDLING_set", _wrap_Dwg_AuxHeader_R11_HANDLING_set},
{"LibreDWGc::Dwg_AuxHeader_R11_HANDLING_get", _wrap_Dwg_AuxHeader_R11_HANDLING_get},
{"LibreDWGc::new_Dwg_AuxHeader", _wrap_new_Dwg_AuxHeader},
{"LibreDWGc::delete_Dwg_AuxHeader", _wrap_delete_Dwg_AuxHeader},
{"LibreDWGc::Dwg_SummaryInfo_TITLE_set", _wrap_Dwg_SummaryInfo_TITLE_set},
{"LibreDWGc::Dwg_SummaryInfo_TITLE_get", _wrap_Dwg_SummaryInfo_TITLE_get},
{"LibreDWGc::Dwg_SummaryInfo_SUBJECT_set", _wrap_Dwg_SummaryInfo_SUBJECT_set},
{"LibreDWGc::Dwg_SummaryInfo_SUBJECT_get", _wrap_Dwg_SummaryInfo_SUBJECT_get},
{"LibreDWGc::Dwg_SummaryInfo_AUTHOR_set", _wrap_Dwg_SummaryInfo_AUTHOR_set},
{"LibreDWGc::Dwg_SummaryInfo_AUTHOR_get", _wrap_Dwg_SummaryInfo_AUTHOR_get},
{"LibreDWGc::Dwg_SummaryInfo_KEYWORDS_set", _wrap_Dwg_SummaryInfo_KEYWORDS_set},
{"LibreDWGc::Dwg_SummaryInfo_KEYWORDS_get", _wrap_Dwg_SummaryInfo_KEYWORDS_get},
{"LibreDWGc::Dwg_SummaryInfo_COMMENTS_set", _wrap_Dwg_SummaryInfo_COMMENTS_set},
{"LibreDWGc::Dwg_SummaryInfo_COMMENTS_get", _wrap_Dwg_SummaryInfo_COMMENTS_get},
{"LibreDWGc::Dwg_SummaryInfo_LASTSAVEDBY_set", _wrap_Dwg_SummaryInfo_LASTSAVEDBY_set},
{"LibreDWGc::Dwg_SummaryInfo_LASTSAVEDBY_get", _wrap_Dwg_SummaryInfo_LASTSAVEDBY_get},
{"LibreDWGc::Dwg_SummaryInfo_REVISIONNUMBER_set", _wrap_Dwg_SummaryInfo_REVISIONNUMBER_set},
{"LibreDWGc::Dwg_SummaryInfo_REVISIONNUMBER_get", _wrap_Dwg_SummaryInfo_REVISIONNUMBER_get},
{"LibreDWGc::Dwg_SummaryInfo_HYPERLINKBASE_set", _wrap_Dwg_SummaryInfo_HYPERLINKBASE_set},
{"LibreDWGc::Dwg_SummaryInfo_HYPERLINKBASE_get", _wrap_Dwg_SummaryInfo_HYPERLINKBASE_get},
{"LibreDWGc::Dwg_SummaryInfo_TDINDWG_set", _wrap_Dwg_SummaryInfo_TDINDWG_set},
{"LibreDWGc::Dwg_SummaryInfo_TDINDWG_get", _wrap_Dwg_SummaryInfo_TDINDWG_get},
{"LibreDWGc::Dwg_SummaryInfo_TDCREATE_set", _wrap_Dwg_SummaryInfo_TDCREATE_set},
{"LibreDWGc::Dwg_SummaryInfo_TDCREATE_get", _wrap_Dwg_SummaryInfo_TDCREATE_get},
{"LibreDWGc::Dwg_SummaryInfo_TDUPDATE_set", _wrap_Dwg_SummaryInfo_TDUPDATE_set},
{"LibreDWGc::Dwg_SummaryInfo_TDUPDATE_get", _wrap_Dwg_SummaryInfo_TDUPDATE_get},
{"LibreDWGc::Dwg_SummaryInfo_num_props_set", _wrap_Dwg_SummaryInfo_num_props_set},
{"LibreDWGc::Dwg_SummaryInfo_num_props_get", _wrap_Dwg_SummaryInfo_num_props_get},
{"LibreDWGc::Dwg_SummaryInfo_props_set", _wrap_Dwg_SummaryInfo_props_set},
{"LibreDWGc::Dwg_SummaryInfo_props_get", _wrap_Dwg_SummaryInfo_props_get},
{"LibreDWGc::Dwg_SummaryInfo_unknown1_set", _wrap_Dwg_SummaryInfo_unknown1_set},
{"LibreDWGc::Dwg_SummaryInfo_unknown1_get", _wrap_Dwg_SummaryInfo_unknown1_get},
{"LibreDWGc::Dwg_SummaryInfo_unknown2_set", _wrap_Dwg_SummaryInfo_unknown2_set},
{"LibreDWGc::Dwg_SummaryInfo_unknown2_get", _wrap_Dwg_SummaryInfo_unknown2_get},
{"LibreDWGc::new_Dwg_SummaryInfo", _wrap_new_Dwg_SummaryInfo},
{"LibreDWGc::delete_Dwg_SummaryInfo", _wrap_delete_Dwg_SummaryInfo},
{"LibreDWGc::Dwg_AppInfo_class_version_set", _wrap_Dwg_AppInfo_class_version_set},
{"LibreDWGc::Dwg_AppInfo_class_version_get", _wrap_Dwg_AppInfo_class_version_get},
{"LibreDWGc::Dwg_AppInfo_num_strings_set", _wrap_Dwg_AppInfo_num_strings_set},
{"LibreDWGc::Dwg_AppInfo_num_strings_get", _wrap_Dwg_AppInfo_num_strings_get},
{"LibreDWGc::Dwg_AppInfo_appinfo_name_set", _wrap_Dwg_AppInfo_appinfo_name_set},
{"LibreDWGc::Dwg_AppInfo_appinfo_name_get", _wrap_Dwg_AppInfo_appinfo_name_get},
{"LibreDWGc::Dwg_AppInfo_version_checksum_set", _wrap_Dwg_AppInfo_version_checksum_set},
{"LibreDWGc::Dwg_AppInfo_version_checksum_get", _wrap_Dwg_AppInfo_version_checksum_get},
{"LibreDWGc::Dwg_AppInfo_comment_checksum_set", _wrap_Dwg_AppInfo_comment_checksum_set},
{"LibreDWGc::Dwg_AppInfo_comment_checksum_get", _wrap_Dwg_AppInfo_comment_checksum_get},
{"LibreDWGc::Dwg_AppInfo_product_checksum_set", _wrap_Dwg_AppInfo_product_checksum_set},
{"LibreDWGc::Dwg_AppInfo_product_checksum_get", _wrap_Dwg_AppInfo_product_checksum_get},
{"LibreDWGc::Dwg_AppInfo_version_set", _wrap_Dwg_AppInfo_version_set},
{"LibreDWGc::Dwg_AppInfo_version_get", _wrap_Dwg_AppInfo_version_get},
{"LibreDWGc::Dwg_AppInfo_comment_set", _wrap_Dwg_AppInfo_comment_set},
{"LibreDWGc::Dwg_AppInfo_comment_get", _wrap_Dwg_AppInfo_comment_get},
{"LibreDWGc::Dwg_AppInfo_product_info_set", _wrap_Dwg_AppInfo_product_info_set},
{"LibreDWGc::Dwg_AppInfo_product_info_get", _wrap_Dwg_AppInfo_product_info_get},
{"LibreDWGc::new_Dwg_AppInfo", _wrap_new_Dwg_AppInfo},
{"LibreDWGc::delete_Dwg_AppInfo", _wrap_delete_Dwg_AppInfo},
{"LibreDWGc::Dwg_FileDepList_num_features_set", _wrap_Dwg_FileDepList_num_features_set},
{"LibreDWGc::Dwg_FileDepList_num_features_get", _wrap_Dwg_FileDepList_num_features_get},
{"LibreDWGc::Dwg_FileDepList_features_set", _wrap_Dwg_FileDepList_features_set},
{"LibreDWGc::Dwg_FileDepList_features_get", _wrap_Dwg_FileDepList_features_get},
{"LibreDWGc::Dwg_FileDepList_num_files_set", _wrap_Dwg_FileDepList_num_files_set},
{"LibreDWGc::Dwg_FileDepList_num_files_get", _wrap_Dwg_FileDepList_num_files_get},
{"LibreDWGc::Dwg_FileDepList_files_set", _wrap_Dwg_FileDepList_files_set},
{"LibreDWGc::Dwg_FileDepList_files_get", _wrap_Dwg_FileDepList_files_get},
{"LibreDWGc::new_Dwg_FileDepList", _wrap_new_Dwg_FileDepList},
{"LibreDWGc::delete_Dwg_FileDepList", _wrap_delete_Dwg_FileDepList},
{"LibreDWGc::Dwg_Security_unknown_1_set", _wrap_Dwg_Security_unknown_1_set},
{"LibreDWGc::Dwg_Security_unknown_1_get", _wrap_Dwg_Security_unknown_1_get},
{"LibreDWGc::Dwg_Security_unknown_2_set", _wrap_Dwg_Security_unknown_2_set},
{"LibreDWGc::Dwg_Security_unknown_2_get", _wrap_Dwg_Security_unknown_2_get},
{"LibreDWGc::Dwg_Security_unknown_3_set", _wrap_Dwg_Security_unknown_3_set},
{"LibreDWGc::Dwg_Security_unknown_3_get", _wrap_Dwg_Security_unknown_3_get},
{"LibreDWGc::Dwg_Security_crypto_id_set", _wrap_Dwg_Security_crypto_id_set},
{"LibreDWGc::Dwg_Security_crypto_id_get", _wrap_Dwg_Security_crypto_id_get},
{"LibreDWGc::Dwg_Security_crypto_name_set", _wrap_Dwg_Security_crypto_name_set},
{"LibreDWGc::Dwg_Security_crypto_name_get", _wrap_Dwg_Security_crypto_name_get},
{"LibreDWGc::Dwg_Security_algo_id_set", _wrap_Dwg_Security_algo_id_set},
{"LibreDWGc::Dwg_Security_algo_id_get", _wrap_Dwg_Security_algo_id_get},
{"LibreDWGc::Dwg_Security_key_len_set", _wrap_Dwg_Security_key_len_set},
{"LibreDWGc::Dwg_Security_key_len_get", _wrap_Dwg_Security_key_len_get},
{"LibreDWGc::Dwg_Security_encr_size_set", _wrap_Dwg_Security_encr_size_set},
{"LibreDWGc::Dwg_Security_encr_size_get", _wrap_Dwg_Security_encr_size_get},
{"LibreDWGc::Dwg_Security_encr_buffer_set", _wrap_Dwg_Security_encr_buffer_set},
{"LibreDWGc::Dwg_Security_encr_buffer_get", _wrap_Dwg_Security_encr_buffer_get},
{"LibreDWGc::new_Dwg_Security", _wrap_new_Dwg_Security},
{"LibreDWGc::delete_Dwg_Security", _wrap_delete_Dwg_Security},
{"LibreDWGc::Dwg_VBAProject_size_set", _wrap_Dwg_VBAProject_size_set},
{"LibreDWGc::Dwg_VBAProject_size_get", _wrap_Dwg_VBAProject_size_get},
{"LibreDWGc::Dwg_VBAProject_unknown_bits_set", _wrap_Dwg_VBAProject_unknown_bits_set},
{"LibreDWGc::Dwg_VBAProject_unknown_bits_get", _wrap_Dwg_VBAProject_unknown_bits_get},
{"LibreDWGc::new_Dwg_VBAProject", _wrap_new_Dwg_VBAProject},
{"LibreDWGc::delete_Dwg_VBAProject", _wrap_delete_Dwg_VBAProject},
{"LibreDWGc::Dwg_AppInfoHistory_size_set", _wrap_Dwg_AppInfoHistory_size_set},
{"LibreDWGc::Dwg_AppInfoHistory_size_get", _wrap_Dwg_AppInfoHistory_size_get},
{"LibreDWGc::Dwg_AppInfoHistory_unknown_bits_set", _wrap_Dwg_AppInfoHistory_unknown_bits_set},
{"LibreDWGc::Dwg_AppInfoHistory_unknown_bits_get", _wrap_Dwg_AppInfoHistory_unknown_bits_get},
{"LibreDWGc::new_Dwg_AppInfoHistory", _wrap_new_Dwg_AppInfoHistory},
{"LibreDWGc::delete_Dwg_AppInfoHistory", _wrap_delete_Dwg_AppInfoHistory},
{"LibreDWGc::Dwg_RevHistory_class_version_set", _wrap_Dwg_RevHistory_class_version_set},
{"LibreDWGc::Dwg_RevHistory_class_version_get", _wrap_Dwg_RevHistory_class_version_get},
{"LibreDWGc::Dwg_RevHistory_class_minor_set", _wrap_Dwg_RevHistory_class_minor_set},
{"LibreDWGc::Dwg_RevHistory_class_minor_get", _wrap_Dwg_RevHistory_class_minor_get},
{"LibreDWGc::Dwg_RevHistory_num_histories_set", _wrap_Dwg_RevHistory_num_histories_set},
{"LibreDWGc::Dwg_RevHistory_num_histories_get", _wrap_Dwg_RevHistory_num_histories_get},
{"LibreDWGc::Dwg_RevHistory_histories_set", _wrap_Dwg_RevHistory_histories_set},
{"LibreDWGc::Dwg_RevHistory_histories_get", _wrap_Dwg_RevHistory_histories_get},
{"LibreDWGc::new_Dwg_RevHistory", _wrap_new_Dwg_RevHistory},
{"LibreDWGc::delete_Dwg_RevHistory", _wrap_delete_Dwg_RevHistory},
{"LibreDWGc::Dwg_ObjFreeSpace_zero_set", _wrap_Dwg_ObjFreeSpace_zero_set},
{"LibreDWGc::Dwg_ObjFreeSpace_zero_get", _wrap_Dwg_ObjFreeSpace_zero_get},
{"LibreDWGc::Dwg_ObjFreeSpace_numhandles_set", _wrap_Dwg_ObjFreeSpace_numhandles_set},
{"LibreDWGc::Dwg_ObjFreeSpace_numhandles_get", _wrap_Dwg_ObjFreeSpace_numhandles_get},
{"LibreDWGc::Dwg_ObjFreeSpace_TDUPDATE_set", _wrap_Dwg_ObjFreeSpace_TDUPDATE_set},
{"LibreDWGc::Dwg_ObjFreeSpace_TDUPDATE_get", _wrap_Dwg_ObjFreeSpace_TDUPDATE_get},
{"LibreDWGc::Dwg_ObjFreeSpace_objects_address_set", _wrap_Dwg_ObjFreeSpace_objects_address_set},
{"LibreDWGc::Dwg_ObjFreeSpace_objects_address_get", _wrap_Dwg_ObjFreeSpace_objects_address_get},
{"LibreDWGc::Dwg_ObjFreeSpace_numnums_set", _wrap_Dwg_ObjFreeSpace_numnums_set},
{"LibreDWGc::Dwg_ObjFreeSpace_numnums_get", _wrap_Dwg_ObjFreeSpace_numnums_get},
{"LibreDWGc::Dwg_ObjFreeSpace_max32_set", _wrap_Dwg_ObjFreeSpace_max32_set},
{"LibreDWGc::Dwg_ObjFreeSpace_max32_get", _wrap_Dwg_ObjFreeSpace_max32_get},
{"LibreDWGc::Dwg_ObjFreeSpace_max64_set", _wrap_Dwg_ObjFreeSpace_max64_set},
{"LibreDWGc::Dwg_ObjFreeSpace_max64_get", _wrap_Dwg_ObjFreeSpace_max64_get},
{"LibreDWGc::Dwg_ObjFreeSpace_maxtbl_set", _wrap_Dwg_ObjFreeSpace_maxtbl_set},
{"LibreDWGc::Dwg_ObjFreeSpace_maxtbl_get", _wrap_Dwg_ObjFreeSpace_maxtbl_get},
{"LibreDWGc::Dwg_ObjFreeSpace_maxrl_set", _wrap_Dwg_ObjFreeSpace_maxrl_set},
{"LibreDWGc::Dwg_ObjFreeSpace_maxrl_get", _wrap_Dwg_ObjFreeSpace_maxrl_get},
{"LibreDWGc::Dwg_ObjFreeSpace_max32_hi_set", _wrap_Dwg_ObjFreeSpace_max32_hi_set},
{"LibreDWGc::Dwg_ObjFreeSpace_max32_hi_get", _wrap_Dwg_ObjFreeSpace_max32_hi_get},
{"LibreDWGc::Dwg_ObjFreeSpace_max64_hi_set", _wrap_Dwg_ObjFreeSpace_max64_hi_set},
{"LibreDWGc::Dwg_ObjFreeSpace_max64_hi_get", _wrap_Dwg_ObjFreeSpace_max64_hi_get},
{"LibreDWGc::Dwg_ObjFreeSpace_maxtbl_hi_set", _wrap_Dwg_ObjFreeSpace_maxtbl_hi_set},
{"LibreDWGc::Dwg_ObjFreeSpace_maxtbl_hi_get", _wrap_Dwg_ObjFreeSpace_maxtbl_hi_get},
{"LibreDWGc::Dwg_ObjFreeSpace_maxrl_hi_set", _wrap_Dwg_ObjFreeSpace_maxrl_hi_set},
{"LibreDWGc::Dwg_ObjFreeSpace_maxrl_hi_get", _wrap_Dwg_ObjFreeSpace_maxrl_hi_get},
{"LibreDWGc::new_Dwg_ObjFreeSpace", _wrap_new_Dwg_ObjFreeSpace},
{"LibreDWGc::delete_Dwg_ObjFreeSpace", _wrap_delete_Dwg_ObjFreeSpace},
{"LibreDWGc::Dwg_Template_description_set", _wrap_Dwg_Template_description_set},
{"LibreDWGc::Dwg_Template_description_get", _wrap_Dwg_Template_description_get},
{"LibreDWGc::Dwg_Template_MEASUREMENT_set", _wrap_Dwg_Template_MEASUREMENT_set},
{"LibreDWGc::Dwg_Template_MEASUREMENT_get", _wrap_Dwg_Template_MEASUREMENT_get},
{"LibreDWGc::new_Dwg_Template", _wrap_new_Dwg_Template},
{"LibreDWGc::delete_Dwg_Template", _wrap_delete_Dwg_Template},
{"LibreDWGc::Dwg_SecondHeader_Sections_nr_set", _wrap_Dwg_SecondHeader_Sections_nr_set},
{"LibreDWGc::Dwg_SecondHeader_Sections_nr_get", _wrap_Dwg_SecondHeader_Sections_nr_get},
{"LibreDWGc::Dwg_SecondHeader_Sections_address_set", _wrap_Dwg_SecondHeader_Sections_address_set},
{"LibreDWGc::Dwg_SecondHeader_Sections_address_get", _wrap_Dwg_SecondHeader_Sections_address_get},
{"LibreDWGc::Dwg_SecondHeader_Sections_size_set", _wrap_Dwg_SecondHeader_Sections_size_set},
{"LibreDWGc::Dwg_SecondHeader_Sections_size_get", _wrap_Dwg_SecondHeader_Sections_size_get},
{"LibreDWGc::new_Dwg_SecondHeader_Sections", _wrap_new_Dwg_SecondHeader_Sections},
{"LibreDWGc::delete_Dwg_SecondHeader_Sections", _wrap_delete_Dwg_SecondHeader_Sections},
{"LibreDWGc::Dwg_SecondHeader_Handles_num_hdl_set", _wrap_Dwg_SecondHeader_Handles_num_hdl_set},
{"LibreDWGc::Dwg_SecondHeader_Handles_num_hdl_get", _wrap_Dwg_SecondHeader_Handles_num_hdl_get},
{"LibreDWGc::Dwg_SecondHeader_Handles_nr_set", _wrap_Dwg_SecondHeader_Handles_nr_set},
{"LibreDWGc::Dwg_SecondHeader_Handles_nr_get", _wrap_Dwg_SecondHeader_Handles_nr_get},
{"LibreDWGc::Dwg_SecondHeader_Handles_hdl_set", _wrap_Dwg_SecondHeader_Handles_hdl_set},
{"LibreDWGc::Dwg_SecondHeader_Handles_hdl_get", _wrap_Dwg_SecondHeader_Handles_hdl_get},
{"LibreDWGc::Dwg_SecondHeader_Handles_name_set", _wrap_Dwg_SecondHeader_Handles_name_set},
{"LibreDWGc::Dwg_SecondHeader_Handles_name_get", _wrap_Dwg_SecondHeader_Handles_name_get},
{"LibreDWGc::new_Dwg_SecondHeader_Handles", _wrap_new_Dwg_SecondHeader_Handles},
{"LibreDWGc::delete_Dwg_SecondHeader_Handles", _wrap_delete_Dwg_SecondHeader_Handles},
{"LibreDWGc::Dwg_SecondHeader_size_set", _wrap_Dwg_SecondHeader_size_set},
{"LibreDWGc::Dwg_SecondHeader_size_get", _wrap_Dwg_SecondHeader_size_get},
{"LibreDWGc::Dwg_SecondHeader_address_set", _wrap_Dwg_SecondHeader_address_set},
{"LibreDWGc::Dwg_SecondHeader_address_get", _wrap_Dwg_SecondHeader_address_get},
{"LibreDWGc::Dwg_SecondHeader_version_set", _wrap_Dwg_SecondHeader_version_set},
{"LibreDWGc::Dwg_SecondHeader_version_get", _wrap_Dwg_SecondHeader_version_get},
{"LibreDWGc::Dwg_SecondHeader_is_maint_set", _wrap_Dwg_SecondHeader_is_maint_set},
{"LibreDWGc::Dwg_SecondHeader_is_maint_get", _wrap_Dwg_SecondHeader_is_maint_get},
{"LibreDWGc::Dwg_SecondHeader_zero_one_or_three_set", _wrap_Dwg_SecondHeader_zero_one_or_three_set},
{"LibreDWGc::Dwg_SecondHeader_zero_one_or_three_get", _wrap_Dwg_SecondHeader_zero_one_or_three_get},
{"LibreDWGc::Dwg_SecondHeader_dwg_versions_set", _wrap_Dwg_SecondHeader_dwg_versions_set},
{"LibreDWGc::Dwg_SecondHeader_dwg_versions_get", _wrap_Dwg_SecondHeader_dwg_versions_get},
{"LibreDWGc::Dwg_SecondHeader_codepage_set", _wrap_Dwg_SecondHeader_codepage_set},
{"LibreDWGc::Dwg_SecondHeader_codepage_get", _wrap_Dwg_SecondHeader_codepage_get},
{"LibreDWGc::Dwg_SecondHeader_num_sections_set", _wrap_Dwg_SecondHeader_num_sections_set},
{"LibreDWGc::Dwg_SecondHeader_num_sections_get", _wrap_Dwg_SecondHeader_num_sections_get},
{"LibreDWGc::Dwg_SecondHeader_sections_set", _wrap_Dwg_SecondHeader_sections_set},
{"LibreDWGc::Dwg_SecondHeader_sections_get", _wrap_Dwg_SecondHeader_sections_get},
{"LibreDWGc::Dwg_SecondHeader_num_handles_set", _wrap_Dwg_SecondHeader_num_handles_set},
{"LibreDWGc::Dwg_SecondHeader_num_handles_get", _wrap_Dwg_SecondHeader_num_handles_get},
{"LibreDWGc::Dwg_SecondHeader_handles_set", _wrap_Dwg_SecondHeader_handles_set},
{"LibreDWGc::Dwg_SecondHeader_handles_get", _wrap_Dwg_SecondHeader_handles_get},
{"LibreDWGc::Dwg_SecondHeader_crc_set", _wrap_Dwg_SecondHeader_crc_set},
{"LibreDWGc::Dwg_SecondHeader_crc_get", _wrap_Dwg_SecondHeader_crc_get},
{"LibreDWGc::Dwg_SecondHeader_junk_r14_set", _wrap_Dwg_SecondHeader_junk_r14_set},
{"LibreDWGc::Dwg_SecondHeader_junk_r14_get", _wrap_Dwg_SecondHeader_junk_r14_get},
{"LibreDWGc::new_Dwg_SecondHeader", _wrap_new_Dwg_SecondHeader},
{"LibreDWGc::delete_Dwg_SecondHeader", _wrap_delete_Dwg_SecondHeader},
{"LibreDWGc::Dwg_Data_header_set", _wrap_Dwg_Data_header_set},
{"LibreDWGc::Dwg_Data_header_get", _wrap_Dwg_Data_header_get},
{"LibreDWGc::Dwg_Data_num_classes_set", _wrap_Dwg_Data_num_classes_set},
{"LibreDWGc::Dwg_Data_num_classes_get", _wrap_Dwg_Data_num_classes_get},
{"LibreDWGc::Dwg_Data_dwg_class_set", _wrap_Dwg_Data_dwg_class_set},
{"LibreDWGc::Dwg_Data_dwg_class_get", _wrap_Dwg_Data_dwg_class_get},
{"LibreDWGc::Dwg_Data_num_objects_set", _wrap_Dwg_Data_num_objects_set},
{"LibreDWGc::Dwg_Data_num_objects_get", _wrap_Dwg_Data_num_objects_get},
{"LibreDWGc::Dwg_Data_num_alloced_objects_set", _wrap_Dwg_Data_num_alloced_objects_set},
{"LibreDWGc::Dwg_Data_num_alloced_objects_get", _wrap_Dwg_Data_num_alloced_objects_get},
{"LibreDWGc::Dwg_Data_object_set", _wrap_Dwg_Data_object_set},
{"LibreDWGc::Dwg_Data_object_get", _wrap_Dwg_Data_object_get},
{"LibreDWGc::Dwg_Data_num_entities_set", _wrap_Dwg_Data_num_entities_set},
{"LibreDWGc::Dwg_Data_num_entities_get", _wrap_Dwg_Data_num_entities_get},
{"LibreDWGc::Dwg_Data_num_object_refs_set", _wrap_Dwg_Data_num_object_refs_set},
{"LibreDWGc::Dwg_Data_num_object_refs_get", _wrap_Dwg_Data_num_object_refs_get},
{"LibreDWGc::Dwg_Data_cur_index_set", _wrap_Dwg_Data_cur_index_set},
{"LibreDWGc::Dwg_Data_cur_index_get", _wrap_Dwg_Data_cur_index_get},
{"LibreDWGc::Dwg_Data_object_ref_set", _wrap_Dwg_Data_object_ref_set},
{"LibreDWGc::Dwg_Data_object_ref_get", _wrap_Dwg_Data_object_ref_get},
{"LibreDWGc::Dwg_Data_object_map_set", _wrap_Dwg_Data_object_map_set},
{"LibreDWGc::Dwg_Data_object_map_get", _wrap_Dwg_Data_object_map_get},
{"LibreDWGc::Dwg_Data_dirty_refs_set", _wrap_Dwg_Data_dirty_refs_set},
{"LibreDWGc::Dwg_Data_dirty_refs_get", _wrap_Dwg_Data_dirty_refs_get},
{"LibreDWGc::Dwg_Data_opts_set", _wrap_Dwg_Data_opts_set},
{"LibreDWGc::Dwg_Data_opts_get", _wrap_Dwg_Data_opts_get},
{"LibreDWGc::Dwg_Data_header_vars_set", _wrap_Dwg_Data_header_vars_set},
{"LibreDWGc::Dwg_Data_header_vars_get", _wrap_Dwg_Data_header_vars_get},
{"LibreDWGc::Dwg_Data_thumbnail_set", _wrap_Dwg_Data_thumbnail_set},
{"LibreDWGc::Dwg_Data_thumbnail_get", _wrap_Dwg_Data_thumbnail_get},
{"LibreDWGc::Dwg_Data_fhdr_get", _wrap_Dwg_Data_fhdr_get},
{"LibreDWGc::Dwg_Data_mspace_block_set", _wrap_Dwg_Data_mspace_block_set},
{"LibreDWGc::Dwg_Data_mspace_block_get", _wrap_Dwg_Data_mspace_block_get},
{"LibreDWGc::Dwg_Data_pspace_block_set", _wrap_Dwg_Data_pspace_block_set},
{"LibreDWGc::Dwg_Data_pspace_block_get", _wrap_Dwg_Data_pspace_block_get},
{"LibreDWGc::Dwg_Data_block_control_set", _wrap_Dwg_Data_block_control_set},
{"LibreDWGc::Dwg_Data_block_control_get", _wrap_Dwg_Data_block_control_get},
{"LibreDWGc::Dwg_Data_auxheader_set", _wrap_Dwg_Data_auxheader_set},
{"LibreDWGc::Dwg_Data_auxheader_get", _wrap_Dwg_Data_auxheader_get},
{"LibreDWGc::Dwg_Data_secondheader_set", _wrap_Dwg_Data_secondheader_set},
{"LibreDWGc::Dwg_Data_secondheader_get", _wrap_Dwg_Data_secondheader_get},
{"LibreDWGc::Dwg_Data_summaryinfo_set", _wrap_Dwg_Data_summaryinfo_set},
{"LibreDWGc::Dwg_Data_summaryinfo_get", _wrap_Dwg_Data_summaryinfo_get},
{"LibreDWGc::Dwg_Data_appinfo_set", _wrap_Dwg_Data_appinfo_set},
{"LibreDWGc::Dwg_Data_appinfo_get", _wrap_Dwg_Data_appinfo_get},
{"LibreDWGc::Dwg_Data_filedeplist_set", _wrap_Dwg_Data_filedeplist_set},
{"LibreDWGc::Dwg_Data_filedeplist_get", _wrap_Dwg_Data_filedeplist_get},
{"LibreDWGc::Dwg_Data_security_set", _wrap_Dwg_Data_security_set},
{"LibreDWGc::Dwg_Data_security_get", _wrap_Dwg_Data_security_get},
{"LibreDWGc::Dwg_Data_vbaproject_set", _wrap_Dwg_Data_vbaproject_set},
{"LibreDWGc::Dwg_Data_vbaproject_get", _wrap_Dwg_Data_vbaproject_get},
{"LibreDWGc::Dwg_Data_appinfohistory_set", _wrap_Dwg_Data_appinfohistory_set},
{"LibreDWGc::Dwg_Data_appinfohistory_get", _wrap_Dwg_Data_appinfohistory_get},
{"LibreDWGc::Dwg_Data_revhistory_set", _wrap_Dwg_Data_revhistory_set},
{"LibreDWGc::Dwg_Data_revhistory_get", _wrap_Dwg_Data_revhistory_get},
{"LibreDWGc::Dwg_Data_objfreespace_set", _wrap_Dwg_Data_objfreespace_set},
{"LibreDWGc::Dwg_Data_objfreespace_get", _wrap_Dwg_Data_objfreespace_get},
{"LibreDWGc::Dwg_Data_Template_set", _wrap_Dwg_Data_Template_set},
{"LibreDWGc::Dwg_Data_Template_get", _wrap_Dwg_Data_Template_get},
{"LibreDWGc::Dwg_Data_acds_set", _wrap_Dwg_Data_acds_set},
{"LibreDWGc::Dwg_Data_acds_get", _wrap_Dwg_Data_acds_get},
{"LibreDWGc::Dwg_Data_layout_type_set", _wrap_Dwg_Data_layout_type_set},
{"LibreDWGc::Dwg_Data_layout_type_get", _wrap_Dwg_Data_layout_type_get},
{"LibreDWGc::Dwg_Data_num_acis_sab_hdl_set", _wrap_Dwg_Data_num_acis_sab_hdl_set},
{"LibreDWGc::Dwg_Data_num_acis_sab_hdl_get", _wrap_Dwg_Data_num_acis_sab_hdl_get},
{"LibreDWGc::Dwg_Data_acis_sab_hdl_set", _wrap_Dwg_Data_acis_sab_hdl_set},
{"LibreDWGc::Dwg_Data_acis_sab_hdl_get", _wrap_Dwg_Data_acis_sab_hdl_get},
{"LibreDWGc::Dwg_Data_next_hdl_set", _wrap_Dwg_Data_next_hdl_set},
{"LibreDWGc::Dwg_Data_next_hdl_get", _wrap_Dwg_Data_next_hdl_get},
{"LibreDWGc::Dwg_Data_prev_entity_index_set", _wrap_Dwg_Data_prev_entity_index_set},
{"LibreDWGc::Dwg_Data_prev_entity_index_get", _wrap_Dwg_Data_prev_entity_index_get},
{"LibreDWGc::Dwg_Data_object_ordered_ref_set", _wrap_Dwg_Data_object_ordered_ref_set},
{"LibreDWGc::Dwg_Data_object_ordered_ref_get", _wrap_Dwg_Data_object_ordered_ref_get},
{"LibreDWGc::Dwg_Data_num_object_ordered_refs_set", _wrap_Dwg_Data_num_object_ordered_refs_set},
{"LibreDWGc::Dwg_Data_num_object_ordered_refs_get", _wrap_Dwg_Data_num_object_ordered_refs_get},
{"LibreDWGc::new_Dwg_Data", _wrap_new_Dwg_Data},
{"LibreDWGc::delete_Dwg_Data", _wrap_delete_Dwg_Data},
{"LibreDWGc::_dwg_struct_fhdr_r2004_header_set", _wrap__dwg_struct_fhdr_r2004_header_set},
{"LibreDWGc::_dwg_struct_fhdr_r2004_header_get", _wrap__dwg_struct_fhdr_r2004_header_get},
{"LibreDWGc::_dwg_struct_fhdr_r2007_file_header_set", _wrap__dwg_struct_fhdr_r2007_file_header_set},
{"LibreDWGc::_dwg_struct_fhdr_r2007_file_header_get", _wrap__dwg_struct_fhdr_r2007_file_header_get},
{"LibreDWGc::new__dwg_struct_fhdr", _wrap_new__dwg_struct_fhdr},
{"LibreDWGc::delete__dwg_struct_fhdr", _wrap_delete__dwg_struct_fhdr},
{"LibreDWGc::dwg_read_file", _wrap_dwg_read_file},
{"LibreDWGc::dxf_read_file", _wrap_dxf_read_file},
{"LibreDWGc::dwg_write_file", _wrap_dwg_write_file},
{"LibreDWGc::dwg_bmp", _wrap_dwg_bmp},
{"LibreDWGc::dxf_cvt_lweight", _wrap_dxf_cvt_lweight},
{"LibreDWGc::dxf_revcvt_lweight", _wrap_dxf_revcvt_lweight},
{"LibreDWGc::dwg_ctrl_table", _wrap_dwg_ctrl_table},
{"LibreDWGc::dwg_find_tablehandle", _wrap_dwg_find_tablehandle},
{"LibreDWGc::dwg_find_tablehandle_index", _wrap_dwg_find_tablehandle_index},
{"LibreDWGc::dwg_handle_name", _wrap_dwg_handle_name},
{"LibreDWGc::dwg_find_table_control", _wrap_dwg_find_table_control},
{"LibreDWGc::dwg_find_dictionary", _wrap_dwg_find_dictionary},
{"LibreDWGc::dwg_find_dicthandle", _wrap_dwg_find_dicthandle},
{"LibreDWGc::dwg_find_dicthandle_objname", _wrap_dwg_find_dicthandle_objname},
{"LibreDWGc::dwg_find_table_extname", _wrap_dwg_find_table_extname},
{"LibreDWGc::dwg_variable_dict", _wrap_dwg_variable_dict},
{"LibreDWGc::dwg_model_x_min", _wrap_dwg_model_x_min},
{"LibreDWGc::dwg_model_x_max", _wrap_dwg_model_x_max},
{"LibreDWGc::dwg_model_y_min", _wrap_dwg_model_y_min},
{"LibreDWGc::dwg_model_y_max", _wrap_dwg_model_y_max},
{"LibreDWGc::dwg_model_z_min", _wrap_dwg_model_z_min},
{"LibreDWGc::dwg_model_z_max", _wrap_dwg_model_z_max},
{"LibreDWGc::dwg_page_x_min", _wrap_dwg_page_x_min},
{"LibreDWGc::dwg_page_x_max", _wrap_dwg_page_x_max},
{"LibreDWGc::dwg_page_y_min", _wrap_dwg_page_y_min},
{"LibreDWGc::dwg_page_y_max", _wrap_dwg_page_y_max},
{"LibreDWGc::dwg_block_control", _wrap_dwg_block_control},
{"LibreDWGc::dwg_model_space_ref", _wrap_dwg_model_space_ref},
{"LibreDWGc::dwg_paper_space_ref", _wrap_dwg_paper_space_ref},
{"LibreDWGc::dwg_model_space_object", _wrap_dwg_model_space_object},
{"LibreDWGc::dwg_paper_space_object", _wrap_dwg_paper_space_object},
{"LibreDWGc::dwg_get_layer_count", _wrap_dwg_get_layer_count},
{"LibreDWGc::dwg_get_layers", _wrap_dwg_get_layers},
{"LibreDWGc::dwg_get_num_objects", _wrap_dwg_get_num_objects},
{"LibreDWGc::dwg_get_object_num_objects", _wrap_dwg_get_object_num_objects},
{"LibreDWGc::dwg_class_is_entity", _wrap_dwg_class_is_entity},
{"LibreDWGc::dwg_obj_is_control", _wrap_dwg_obj_is_control},
{"LibreDWGc::dwg_obj_is_table", _wrap_dwg_obj_is_table},
{"LibreDWGc::dwg_obj_is_subentity", _wrap_dwg_obj_is_subentity},
{"LibreDWGc::dwg_obj_has_subentity", _wrap_dwg_obj_has_subentity},
{"LibreDWGc::dwg_obj_is_3dsolid", _wrap_dwg_obj_is_3dsolid},
{"LibreDWGc::dwg_obj_is_acsh", _wrap_dwg_obj_is_acsh},
{"LibreDWGc::dwg_get_num_entities", _wrap_dwg_get_num_entities},
{"LibreDWGc::dwg_get_entities", _wrap_dwg_get_entities},
{"LibreDWGc::dwg_get_entity_layer", _wrap_dwg_get_entity_layer},
{"LibreDWGc::dwg_next_object", _wrap_dwg_next_object},
{"LibreDWGc::dwg_next_entity", _wrap_dwg_next_entity},
{"LibreDWGc::dwg_next_handle", _wrap_dwg_next_handle},
{"LibreDWGc::dwg_next_handseed", _wrap_dwg_next_handseed},
{"LibreDWGc::dwg_ref_object", _wrap_dwg_ref_object},
{"LibreDWGc::dwg_ref_object_relative", _wrap_dwg_ref_object_relative},
{"LibreDWGc::dwg_ref_object_silent", _wrap_dwg_ref_object_silent},
{"LibreDWGc::get_first_owned_entity", _wrap_get_first_owned_entity},
{"LibreDWGc::get_next_owned_entity", _wrap_get_next_owned_entity},
{"LibreDWGc::get_first_owned_subentity", _wrap_get_first_owned_subentity},
{"LibreDWGc::get_next_owned_subentity", _wrap_get_next_owned_subentity},
{"LibreDWGc::get_first_owned_block", _wrap_get_first_owned_block},
{"LibreDWGc::get_last_owned_block", _wrap_get_last_owned_block},
{"LibreDWGc::get_next_owned_block", _wrap_get_next_owned_block},
{"LibreDWGc::get_next_owned_block_entity", _wrap_get_next_owned_block_entity},
{"LibreDWGc::dwg_get_first_object", _wrap_dwg_get_first_object},
{"LibreDWGc::dwg_get_next_object", _wrap_dwg_get_next_object},
{"LibreDWGc::dwg_resolve_handle", _wrap_dwg_resolve_handle},
{"LibreDWGc::dwg_resolve_handle_silent", _wrap_dwg_resolve_handle_silent},
{"LibreDWGc::dwg_resolve_handleref", _wrap_dwg_resolve_handleref},
{"LibreDWGc::dwg_resolve_jump", _wrap_dwg_resolve_jump},
{"LibreDWGc::dwg_section_type", _wrap_dwg_section_type},
{"LibreDWGc::dwg_section_wtype", _wrap_dwg_section_wtype},
{"LibreDWGc::dwg_section_name", _wrap_dwg_section_name},
{"LibreDWGc::dwg_resbuf_value_type", _wrap_dwg_resbuf_value_type},
{"LibreDWGc::dwg_free", _wrap_dwg_free},
{"LibreDWGc::dwg_free_object", _wrap_dwg_free_object},
{"LibreDWGc::dwg_new_ref", _wrap_dwg_new_ref},
{"LibreDWGc::dwg_add_handle", _wrap_dwg_add_handle},
{"LibreDWGc::dwg_add_handleref", _wrap_dwg_add_handleref},
{"LibreDWGc::dwg_dup_handleref", _wrap_dwg_dup_handleref},
{"LibreDWGc::dwg_add_handleref_free", _wrap_dwg_add_handleref_free},
{"LibreDWGc::dwg_version_type", _wrap_dwg_version_type},
{"LibreDWGc::dwg_version_as", _wrap_dwg_version_as},
{"LibreDWGc::dwg_version_hdr_type", _wrap_dwg_version_hdr_type},
{"LibreDWGc::dwg_supports_eed", _wrap_dwg_supports_eed},
{"LibreDWGc::dwg_supports_obj", _wrap_dwg_supports_obj},
{"LibreDWGc::dwg_errstrings", _wrap_dwg_errstrings},
{"LibreDWGc::dwg_encrypt_SAT1", _wrap_dwg_encrypt_SAT1},
{"LibreDWGc::dwg_convert_SAB_to_SAT1", _wrap_dwg_convert_SAB_to_SAT1},
{"LibreDWGc::Dwg_RGB_Palette_r_set", _wrap_Dwg_RGB_Palette_r_set},
{"LibreDWGc::Dwg_RGB_Palette_r_get", _wrap_Dwg_RGB_Palette_r_get},
{"LibreDWGc::Dwg_RGB_Palette_g_set", _wrap_Dwg_RGB_Palette_g_set},
{"LibreDWGc::Dwg_RGB_Palette_g_get", _wrap_Dwg_RGB_Palette_g_get},
{"LibreDWGc::Dwg_RGB_Palette_b_set", _wrap_Dwg_RGB_Palette_b_set},
{"LibreDWGc::Dwg_RGB_Palette_b_get", _wrap_Dwg_RGB_Palette_b_get},
{"LibreDWGc::new_Dwg_RGB_Palette", _wrap_new_Dwg_RGB_Palette},
{"LibreDWGc::delete_Dwg_RGB_Palette", _wrap_delete_Dwg_RGB_Palette},
{"LibreDWGc::dwg_rgb_palette", _wrap_dwg_rgb_palette},
{"LibreDWGc::dwg_rgb_palette_index", _wrap_dwg_rgb_palette_index},
{"LibreDWGc::dwg_find_color_index", _wrap_dwg_find_color_index},
{"LibreDWGc::dwg_add_object", _wrap_dwg_add_object},
{"LibreDWGc::dwg_object_name", _wrap_dwg_object_name},
{"LibreDWGc::dwg_setup__3DFACE", _wrap_dwg_setup__3DFACE},
{"LibreDWGc::dwg_setup__3DSOLID", _wrap_dwg_setup__3DSOLID},
{"LibreDWGc::dwg_setup_ARC", _wrap_dwg_setup_ARC},
{"LibreDWGc::dwg_setup_ATTDEF", _wrap_dwg_setup_ATTDEF},
{"LibreDWGc::dwg_setup_ATTRIB", _wrap_dwg_setup_ATTRIB},
{"LibreDWGc::dwg_setup_BLOCK", _wrap_dwg_setup_BLOCK},
{"LibreDWGc::dwg_setup_BODY", _wrap_dwg_setup_BODY},
{"LibreDWGc::dwg_setup_CIRCLE", _wrap_dwg_setup_CIRCLE},
{"LibreDWGc::dwg_setup_DIMENSION_ALIGNED", _wrap_dwg_setup_DIMENSION_ALIGNED},
{"LibreDWGc::dwg_setup_DIMENSION_ANG2LN", _wrap_dwg_setup_DIMENSION_ANG2LN},
{"LibreDWGc::dwg_setup_DIMENSION_ANG3PT", _wrap_dwg_setup_DIMENSION_ANG3PT},
{"LibreDWGc::dwg_setup_DIMENSION_DIAMETER", _wrap_dwg_setup_DIMENSION_DIAMETER},
{"LibreDWGc::dwg_setup_DIMENSION_LINEAR", _wrap_dwg_setup_DIMENSION_LINEAR},
{"LibreDWGc::dwg_setup_DIMENSION_ORDINATE", _wrap_dwg_setup_DIMENSION_ORDINATE},
{"LibreDWGc::dwg_setup_DIMENSION_RADIUS", _wrap_dwg_setup_DIMENSION_RADIUS},
{"LibreDWGc::dwg_setup_ELLIPSE", _wrap_dwg_setup_ELLIPSE},
{"LibreDWGc::dwg_setup_ENDBLK", _wrap_dwg_setup_ENDBLK},
{"LibreDWGc::dwg_setup_INSERT", _wrap_dwg_setup_INSERT},
{"LibreDWGc::dwg_setup_LEADER", _wrap_dwg_setup_LEADER},
{"LibreDWGc::dwg_setup_LINE", _wrap_dwg_setup_LINE},
{"LibreDWGc::dwg_setup_LOAD", _wrap_dwg_setup_LOAD},
{"LibreDWGc::dwg_setup_MINSERT", _wrap_dwg_setup_MINSERT},
{"LibreDWGc::dwg_setup_MLINE", _wrap_dwg_setup_MLINE},
{"LibreDWGc::dwg_setup_MTEXT", _wrap_dwg_setup_MTEXT},
{"LibreDWGc::dwg_setup_OLEFRAME", _wrap_dwg_setup_OLEFRAME},
{"LibreDWGc::dwg_setup_POINT", _wrap_dwg_setup_POINT},
{"LibreDWGc::dwg_setup_POLYLINE_2D", _wrap_dwg_setup_POLYLINE_2D},
{"LibreDWGc::dwg_setup_POLYLINE_3D", _wrap_dwg_setup_POLYLINE_3D},
{"LibreDWGc::dwg_setup_POLYLINE_MESH", _wrap_dwg_setup_POLYLINE_MESH},
{"LibreDWGc::dwg_setup_POLYLINE_PFACE", _wrap_dwg_setup_POLYLINE_PFACE},
{"LibreDWGc::dwg_setup_PROXY_ENTITY", _wrap_dwg_setup_PROXY_ENTITY},
{"LibreDWGc::dwg_setup_RAY", _wrap_dwg_setup_RAY},
{"LibreDWGc::dwg_setup_REGION", _wrap_dwg_setup_REGION},
{"LibreDWGc::dwg_setup_SEQEND", _wrap_dwg_setup_SEQEND},
{"LibreDWGc::dwg_setup_SHAPE", _wrap_dwg_setup_SHAPE},
{"LibreDWGc::dwg_setup_SOLID", _wrap_dwg_setup_SOLID},
{"LibreDWGc::dwg_setup_SPLINE", _wrap_dwg_setup_SPLINE},
{"LibreDWGc::dwg_setup_TEXT", _wrap_dwg_setup_TEXT},
{"LibreDWGc::dwg_setup_TOLERANCE", _wrap_dwg_setup_TOLERANCE},
{"LibreDWGc::dwg_setup_TRACE", _wrap_dwg_setup_TRACE},
{"LibreDWGc::dwg_setup_UNKNOWN_ENT", _wrap_dwg_setup_UNKNOWN_ENT},
{"LibreDWGc::dwg_setup_VERTEX_2D", _wrap_dwg_setup_VERTEX_2D},
{"LibreDWGc::dwg_setup_VERTEX_3D", _wrap_dwg_setup_VERTEX_3D},
{"LibreDWGc::dwg_setup_VERTEX_MESH", _wrap_dwg_setup_VERTEX_MESH},
{"LibreDWGc::dwg_setup_VERTEX_PFACE", _wrap_dwg_setup_VERTEX_PFACE},
{"LibreDWGc::dwg_setup_VERTEX_PFACE_FACE", _wrap_dwg_setup_VERTEX_PFACE_FACE},
{"LibreDWGc::dwg_setup_VIEWPORT", _wrap_dwg_setup_VIEWPORT},
{"LibreDWGc::dwg_setup_XLINE", _wrap_dwg_setup_XLINE},
{"LibreDWGc::dwg_setup_APPID", _wrap_dwg_setup_APPID},
{"LibreDWGc::dwg_setup_APPID_CONTROL", _wrap_dwg_setup_APPID_CONTROL},
{"LibreDWGc::dwg_setup_BLOCK_CONTROL", _wrap_dwg_setup_BLOCK_CONTROL},
{"LibreDWGc::dwg_setup_BLOCK_HEADER", _wrap_dwg_setup_BLOCK_HEADER},
{"LibreDWGc::dwg_setup_DICTIONARY", _wrap_dwg_setup_DICTIONARY},
{"LibreDWGc::dwg_setup_DIMSTYLE", _wrap_dwg_setup_DIMSTYLE},
{"LibreDWGc::dwg_setup_DIMSTYLE_CONTROL", _wrap_dwg_setup_DIMSTYLE_CONTROL},
{"LibreDWGc::dwg_setup_DUMMY", _wrap_dwg_setup_DUMMY},
{"LibreDWGc::dwg_setup_LAYER", _wrap_dwg_setup_LAYER},
{"LibreDWGc::dwg_setup_LAYER_CONTROL", _wrap_dwg_setup_LAYER_CONTROL},
{"LibreDWGc::dwg_setup_LONG_TRANSACTION", _wrap_dwg_setup_LONG_TRANSACTION},
{"LibreDWGc::dwg_setup_LTYPE", _wrap_dwg_setup_LTYPE},
{"LibreDWGc::dwg_setup_LTYPE_CONTROL", _wrap_dwg_setup_LTYPE_CONTROL},
{"LibreDWGc::dwg_setup_MLINESTYLE", _wrap_dwg_setup_MLINESTYLE},
{"LibreDWGc::dwg_setup_STYLE", _wrap_dwg_setup_STYLE},
{"LibreDWGc::dwg_setup_STYLE_CONTROL", _wrap_dwg_setup_STYLE_CONTROL},
{"LibreDWGc::dwg_setup_UCS", _wrap_dwg_setup_UCS},
{"LibreDWGc::dwg_setup_UCS_CONTROL", _wrap_dwg_setup_UCS_CONTROL},
{"LibreDWGc::dwg_setup_UNKNOWN_OBJ", _wrap_dwg_setup_UNKNOWN_OBJ},
{"LibreDWGc::dwg_setup_VIEW", _wrap_dwg_setup_VIEW},
{"LibreDWGc::dwg_setup_VIEW_CONTROL", _wrap_dwg_setup_VIEW_CONTROL},
{"LibreDWGc::dwg_setup_VPORT", _wrap_dwg_setup_VPORT},
{"LibreDWGc::dwg_setup_VPORT_CONTROL", _wrap_dwg_setup_VPORT_CONTROL},
{"LibreDWGc::dwg_setup_VX_CONTROL", _wrap_dwg_setup_VX_CONTROL},
{"LibreDWGc::dwg_setup_VX_TABLE_RECORD", _wrap_dwg_setup_VX_TABLE_RECORD},
{"LibreDWGc::dwg_setup__3DLINE", _wrap_dwg_setup__3DLINE},
{"LibreDWGc::dwg_setup_CAMERA", _wrap_dwg_setup_CAMERA},
{"LibreDWGc::dwg_setup_DGNUNDERLAY", _wrap_dwg_setup_DGNUNDERLAY},
{"LibreDWGc::dwg_setup_DWFUNDERLAY", _wrap_dwg_setup_DWFUNDERLAY},
{"LibreDWGc::dwg_setup_ENDREP", _wrap_dwg_setup_ENDREP},
{"LibreDWGc::dwg_setup_HATCH", _wrap_dwg_setup_HATCH},
{"LibreDWGc::dwg_setup_IMAGE", _wrap_dwg_setup_IMAGE},
{"LibreDWGc::dwg_setup_JUMP", _wrap_dwg_setup_JUMP},
{"LibreDWGc::dwg_setup_LIGHT", _wrap_dwg_setup_LIGHT},
{"LibreDWGc::dwg_setup_LWPOLYLINE", _wrap_dwg_setup_LWPOLYLINE},
{"LibreDWGc::dwg_setup_MESH", _wrap_dwg_setup_MESH},
{"LibreDWGc::dwg_setup_MULTILEADER", _wrap_dwg_setup_MULTILEADER},
{"LibreDWGc::dwg_setup_OLE2FRAME", _wrap_dwg_setup_OLE2FRAME},
{"LibreDWGc::dwg_setup_PDFUNDERLAY", _wrap_dwg_setup_PDFUNDERLAY},
{"LibreDWGc::dwg_setup_REPEAT", _wrap_dwg_setup_REPEAT},
{"LibreDWGc::dwg_setup_SECTIONOBJECT", _wrap_dwg_setup_SECTIONOBJECT},
{"LibreDWGc::dwg_setup_WIPEOUT", _wrap_dwg_setup_WIPEOUT},
{"LibreDWGc::dwg_setup_ACSH_BOOLEAN_CLASS", _wrap_dwg_setup_ACSH_BOOLEAN_CLASS},
{"LibreDWGc::dwg_setup_ACSH_BOX_CLASS", _wrap_dwg_setup_ACSH_BOX_CLASS},
{"LibreDWGc::dwg_setup_ACSH_CONE_CLASS", _wrap_dwg_setup_ACSH_CONE_CLASS},
{"LibreDWGc::dwg_setup_ACSH_CYLINDER_CLASS", _wrap_dwg_setup_ACSH_CYLINDER_CLASS},
{"LibreDWGc::dwg_setup_ACSH_FILLET_CLASS", _wrap_dwg_setup_ACSH_FILLET_CLASS},
{"LibreDWGc::dwg_setup_ACSH_HISTORY_CLASS", _wrap_dwg_setup_ACSH_HISTORY_CLASS},
{"LibreDWGc::dwg_setup_ACSH_SPHERE_CLASS", _wrap_dwg_setup_ACSH_SPHERE_CLASS},
{"LibreDWGc::dwg_setup_ACSH_TORUS_CLASS", _wrap_dwg_setup_ACSH_TORUS_CLASS},
{"LibreDWGc::dwg_setup_ACSH_WEDGE_CLASS", _wrap_dwg_setup_ACSH_WEDGE_CLASS},
{"LibreDWGc::dwg_setup_ASSOCGEOMDEPENDENCY", _wrap_dwg_setup_ASSOCGEOMDEPENDENCY},
{"LibreDWGc::dwg_setup_ASSOCNETWORK", _wrap_dwg_setup_ASSOCNETWORK},
{"LibreDWGc::dwg_setup_BLOCKALIGNMENTGRIP", _wrap_dwg_setup_BLOCKALIGNMENTGRIP},
{"LibreDWGc::dwg_setup_BLOCKALIGNMENTPARAMETER", _wrap_dwg_setup_BLOCKALIGNMENTPARAMETER},
{"LibreDWGc::dwg_setup_BLOCKBASEPOINTPARAMETER", _wrap_dwg_setup_BLOCKBASEPOINTPARAMETER},
{"LibreDWGc::dwg_setup_BLOCKFLIPACTION", _wrap_dwg_setup_BLOCKFLIPACTION},
{"LibreDWGc::dwg_setup_BLOCKFLIPGRIP", _wrap_dwg_setup_BLOCKFLIPGRIP},
{"LibreDWGc::dwg_setup_BLOCKFLIPPARAMETER", _wrap_dwg_setup_BLOCKFLIPPARAMETER},
{"LibreDWGc::dwg_setup_BLOCKGRIPLOCATIONCOMPONENT", _wrap_dwg_setup_BLOCKGRIPLOCATIONCOMPONENT},
{"LibreDWGc::dwg_setup_BLOCKLINEARGRIP", _wrap_dwg_setup_BLOCKLINEARGRIP},
{"LibreDWGc::dwg_setup_BLOCKLOOKUPGRIP", _wrap_dwg_setup_BLOCKLOOKUPGRIP},
{"LibreDWGc::dwg_setup_BLOCKMOVEACTION", _wrap_dwg_setup_BLOCKMOVEACTION},
{"LibreDWGc::dwg_setup_BLOCKROTATEACTION", _wrap_dwg_setup_BLOCKROTATEACTION},
{"LibreDWGc::dwg_setup_BLOCKROTATIONGRIP", _wrap_dwg_setup_BLOCKROTATIONGRIP},
{"LibreDWGc::dwg_setup_BLOCKSCALEACTION", _wrap_dwg_setup_BLOCKSCALEACTION},
{"LibreDWGc::dwg_setup_BLOCKVISIBILITYGRIP", _wrap_dwg_setup_BLOCKVISIBILITYGRIP},
{"LibreDWGc::dwg_setup_CELLSTYLEMAP", _wrap_dwg_setup_CELLSTYLEMAP},
{"LibreDWGc::dwg_setup_DETAILVIEWSTYLE", _wrap_dwg_setup_DETAILVIEWSTYLE},
{"LibreDWGc::dwg_setup_DICTIONARYVAR", _wrap_dwg_setup_DICTIONARYVAR},
{"LibreDWGc::dwg_setup_DICTIONARYWDFLT", _wrap_dwg_setup_DICTIONARYWDFLT},
{"LibreDWGc::dwg_setup_DYNAMICBLOCKPURGEPREVENTER", _wrap_dwg_setup_DYNAMICBLOCKPURGEPREVENTER},
{"LibreDWGc::dwg_setup_FIELD", _wrap_dwg_setup_FIELD},
{"LibreDWGc::dwg_setup_FIELDLIST", _wrap_dwg_setup_FIELDLIST},
{"LibreDWGc::dwg_setup_GEODATA", _wrap_dwg_setup_GEODATA},
{"LibreDWGc::dwg_setup_GROUP", _wrap_dwg_setup_GROUP},
{"LibreDWGc::dwg_setup_IDBUFFER", _wrap_dwg_setup_IDBUFFER},
{"LibreDWGc::dwg_setup_IMAGEDEF", _wrap_dwg_setup_IMAGEDEF},
{"LibreDWGc::dwg_setup_IMAGEDEF_REACTOR", _wrap_dwg_setup_IMAGEDEF_REACTOR},
{"LibreDWGc::dwg_setup_INDEX", _wrap_dwg_setup_INDEX},
{"LibreDWGc::dwg_setup_LAYERFILTER", _wrap_dwg_setup_LAYERFILTER},
{"LibreDWGc::dwg_setup_LAYER_INDEX", _wrap_dwg_setup_LAYER_INDEX},
{"LibreDWGc::dwg_setup_LAYOUT", _wrap_dwg_setup_LAYOUT},
{"LibreDWGc::dwg_setup_PLACEHOLDER", _wrap_dwg_setup_PLACEHOLDER},
{"LibreDWGc::dwg_setup_PLOTSETTINGS", _wrap_dwg_setup_PLOTSETTINGS},
{"LibreDWGc::dwg_setup_RASTERVARIABLES", _wrap_dwg_setup_RASTERVARIABLES},
{"LibreDWGc::dwg_setup_RENDERENVIRONMENT", _wrap_dwg_setup_RENDERENVIRONMENT},
{"LibreDWGc::dwg_setup_SCALE", _wrap_dwg_setup_SCALE},
{"LibreDWGc::dwg_setup_SECTIONVIEWSTYLE", _wrap_dwg_setup_SECTIONVIEWSTYLE},
{"LibreDWGc::dwg_setup_SECTION_MANAGER", _wrap_dwg_setup_SECTION_MANAGER},
{"LibreDWGc::dwg_setup_SORTENTSTABLE", _wrap_dwg_setup_SORTENTSTABLE},
{"LibreDWGc::dwg_setup_SPATIAL_FILTER", _wrap_dwg_setup_SPATIAL_FILTER},
{"LibreDWGc::dwg_setup_SUN", _wrap_dwg_setup_SUN},
{"LibreDWGc::dwg_setup_TABLEGEOMETRY", _wrap_dwg_setup_TABLEGEOMETRY},
{"LibreDWGc::dwg_setup_VBA_PROJECT", _wrap_dwg_setup_VBA_PROJECT},
{"LibreDWGc::dwg_setup_VISUALSTYLE", _wrap_dwg_setup_VISUALSTYLE},
{"LibreDWGc::dwg_setup_WIPEOUTVARIABLES", _wrap_dwg_setup_WIPEOUTVARIABLES},
{"LibreDWGc::dwg_setup_XRECORD", _wrap_dwg_setup_XRECORD},
{"LibreDWGc::dwg_setup_PDFDEFINITION", _wrap_dwg_setup_PDFDEFINITION},
{"LibreDWGc::dwg_setup_DGNDEFINITION", _wrap_dwg_setup_DGNDEFINITION},
{"LibreDWGc::dwg_setup_DWFDEFINITION", _wrap_dwg_setup_DWFDEFINITION},
{"LibreDWGc::dwg_setup_ARC_DIMENSION", _wrap_dwg_setup_ARC_DIMENSION},
{"LibreDWGc::dwg_setup_HELIX", _wrap_dwg_setup_HELIX},
{"LibreDWGc::dwg_setup_LARGE_RADIAL_DIMENSION", _wrap_dwg_setup_LARGE_RADIAL_DIMENSION},
{"LibreDWGc::dwg_setup_LAYOUTPRINTCONFIG", _wrap_dwg_setup_LAYOUTPRINTCONFIG},
{"LibreDWGc::dwg_setup_PLANESURFACE", _wrap_dwg_setup_PLANESURFACE},
{"LibreDWGc::dwg_setup_POINTCLOUD", _wrap_dwg_setup_POINTCLOUD},
{"LibreDWGc::dwg_setup_POINTCLOUDEX", _wrap_dwg_setup_POINTCLOUDEX},
{"LibreDWGc::dwg_setup_ACSH_BREP_CLASS", _wrap_dwg_setup_ACSH_BREP_CLASS},
{"LibreDWGc::dwg_setup_ACSH_CHAMFER_CLASS", _wrap_dwg_setup_ACSH_CHAMFER_CLASS},
{"LibreDWGc::dwg_setup_ACSH_PYRAMID_CLASS", _wrap_dwg_setup_ACSH_PYRAMID_CLASS},
{"LibreDWGc::dwg_setup_ALDIMOBJECTCONTEXTDATA", _wrap_dwg_setup_ALDIMOBJECTCONTEXTDATA},
{"LibreDWGc::dwg_setup_ASSOC2DCONSTRAINTGROUP", _wrap_dwg_setup_ASSOC2DCONSTRAINTGROUP},
{"LibreDWGc::dwg_setup_ASSOCACTION", _wrap_dwg_setup_ASSOCACTION},
{"LibreDWGc::dwg_setup_ASSOCACTIONPARAM", _wrap_dwg_setup_ASSOCACTIONPARAM},
{"LibreDWGc::dwg_setup_ASSOCARRAYACTIONBODY", _wrap_dwg_setup_ASSOCARRAYACTIONBODY},
{"LibreDWGc::dwg_setup_ASSOCASMBODYACTIONPARAM", _wrap_dwg_setup_ASSOCASMBODYACTIONPARAM},
{"LibreDWGc::dwg_setup_ASSOCBLENDSURFACEACTIONBODY", _wrap_dwg_setup_ASSOCBLENDSURFACEACTIONBODY},
{"LibreDWGc::dwg_setup_ASSOCCOMPOUNDACTIONPARAM", _wrap_dwg_setup_ASSOCCOMPOUNDACTIONPARAM},
{"LibreDWGc::dwg_setup_ASSOCDEPENDENCY", _wrap_dwg_setup_ASSOCDEPENDENCY},
{"LibreDWGc::dwg_setup_ASSOCDIMDEPENDENCYBODY", _wrap_dwg_setup_ASSOCDIMDEPENDENCYBODY},
{"LibreDWGc::dwg_setup_ASSOCEXTENDSURFACEACTIONBODY", _wrap_dwg_setup_ASSOCEXTENDSURFACEACTIONBODY},
{"LibreDWGc::dwg_setup_ASSOCEXTRUDEDSURFACEACTIONBODY", _wrap_dwg_setup_ASSOCEXTRUDEDSURFACEACTIONBODY},
{"LibreDWGc::dwg_setup_ASSOCFACEACTIONPARAM", _wrap_dwg_setup_ASSOCFACEACTIONPARAM},
{"LibreDWGc::dwg_setup_ASSOCFILLETSURFACEACTIONBODY", _wrap_dwg_setup_ASSOCFILLETSURFACEACTIONBODY},
{"LibreDWGc::dwg_setup_ASSOCLOFTEDSURFACEACTIONBODY", _wrap_dwg_setup_ASSOCLOFTEDSURFACEACTIONBODY},
{"LibreDWGc::dwg_setup_ASSOCNETWORKSURFACEACTIONBODY", _wrap_dwg_setup_ASSOCNETWORKSURFACEACTIONBODY},
{"LibreDWGc::dwg_setup_ASSOCOBJECTACTIONPARAM", _wrap_dwg_setup_ASSOCOBJECTACTIONPARAM},
{"LibreDWGc::dwg_setup_ASSOCOFFSETSURFACEACTIONBODY", _wrap_dwg_setup_ASSOCOFFSETSURFACEACTIONBODY},
{"LibreDWGc::dwg_setup_ASSOCOSNAPPOINTREFACTIONPARAM", _wrap_dwg_setup_ASSOCOSNAPPOINTREFACTIONPARAM},
{"LibreDWGc::dwg_setup_ASSOCPATCHSURFACEACTIONBODY", _wrap_dwg_setup_ASSOCPATCHSURFACEACTIONBODY},
{"LibreDWGc::dwg_setup_ASSOCPATHACTIONPARAM", _wrap_dwg_setup_ASSOCPATHACTIONPARAM},
{"LibreDWGc::dwg_setup_ASSOCPLANESURFACEACTIONBODY", _wrap_dwg_setup_ASSOCPLANESURFACEACTIONBODY},
{"LibreDWGc::dwg_setup_ASSOCPOINTREFACTIONPARAM", _wrap_dwg_setup_ASSOCPOINTREFACTIONPARAM},
{"LibreDWGc::dwg_setup_ASSOCREVOLVEDSURFACEACTIONBODY", _wrap_dwg_setup_ASSOCREVOLVEDSURFACEACTIONBODY},
{"LibreDWGc::dwg_setup_ASSOCTRIMSURFACEACTIONBODY", _wrap_dwg_setup_ASSOCTRIMSURFACEACTIONBODY},
{"LibreDWGc::dwg_setup_ASSOCVALUEDEPENDENCY", _wrap_dwg_setup_ASSOCVALUEDEPENDENCY},
{"LibreDWGc::dwg_setup_ASSOCVARIABLE", _wrap_dwg_setup_ASSOCVARIABLE},
{"LibreDWGc::dwg_setup_ASSOCVERTEXACTIONPARAM", _wrap_dwg_setup_ASSOCVERTEXACTIONPARAM},
{"LibreDWGc::dwg_setup_BLKREFOBJECTCONTEXTDATA", _wrap_dwg_setup_BLKREFOBJECTCONTEXTDATA},
{"LibreDWGc::dwg_setup_BLOCKALIGNEDCONSTRAINTPARAMETER", _wrap_dwg_setup_BLOCKALIGNEDCONSTRAINTPARAMETER},
{"LibreDWGc::dwg_setup_BLOCKANGULARCONSTRAINTPARAMETER", _wrap_dwg_setup_BLOCKANGULARCONSTRAINTPARAMETER},
{"LibreDWGc::dwg_setup_BLOCKARRAYACTION", _wrap_dwg_setup_BLOCKARRAYACTION},
{"LibreDWGc::dwg_setup_BLOCKDIAMETRICCONSTRAINTPARAMETER", _wrap_dwg_setup_BLOCKDIAMETRICCONSTRAINTPARAMETER},
{"LibreDWGc::dwg_setup_BLOCKHORIZONTALCONSTRAINTPARAMETER", _wrap_dwg_setup_BLOCKHORIZONTALCONSTRAINTPARAMETER},
{"LibreDWGc::dwg_setup_BLOCKLINEARCONSTRAINTPARAMETER", _wrap_dwg_setup_BLOCKLINEARCONSTRAINTPARAMETER},
{"LibreDWGc::dwg_setup_BLOCKLINEARPARAMETER", _wrap_dwg_setup_BLOCKLINEARPARAMETER},
{"LibreDWGc::dwg_setup_BLOCKLOOKUPACTION", _wrap_dwg_setup_BLOCKLOOKUPACTION},
{"LibreDWGc::dwg_setup_BLOCKLOOKUPPARAMETER", _wrap_dwg_setup_BLOCKLOOKUPPARAMETER},
{"LibreDWGc::dwg_setup_BLOCKPARAMDEPENDENCYBODY", _wrap_dwg_setup_BLOCKPARAMDEPENDENCYBODY},
{"LibreDWGc::dwg_setup_BLOCKPOINTPARAMETER", _wrap_dwg_setup_BLOCKPOINTPARAMETER},
{"LibreDWGc::dwg_setup_BLOCKPOLARGRIP", _wrap_dwg_setup_BLOCKPOLARGRIP},
{"LibreDWGc::dwg_setup_BLOCKPOLARPARAMETER", _wrap_dwg_setup_BLOCKPOLARPARAMETER},
{"LibreDWGc::dwg_setup_BLOCKPOLARSTRETCHACTION", _wrap_dwg_setup_BLOCKPOLARSTRETCHACTION},
{"LibreDWGc::dwg_setup_BLOCKRADIALCONSTRAINTPARAMETER", _wrap_dwg_setup_BLOCKRADIALCONSTRAINTPARAMETER},
{"LibreDWGc::dwg_setup_BLOCKREPRESENTATION", _wrap_dwg_setup_BLOCKREPRESENTATION},
{"LibreDWGc::dwg_setup_BLOCKROTATIONPARAMETER", _wrap_dwg_setup_BLOCKROTATIONPARAMETER},
{"LibreDWGc::dwg_setup_BLOCKSTRETCHACTION", _wrap_dwg_setup_BLOCKSTRETCHACTION},
{"LibreDWGc::dwg_setup_BLOCKUSERPARAMETER", _wrap_dwg_setup_BLOCKUSERPARAMETER},
{"LibreDWGc::dwg_setup_BLOCKVERTICALCONSTRAINTPARAMETER", _wrap_dwg_setup_BLOCKVERTICALCONSTRAINTPARAMETER},
{"LibreDWGc::dwg_setup_BLOCKVISIBILITYPARAMETER", _wrap_dwg_setup_BLOCKVISIBILITYPARAMETER},
{"LibreDWGc::dwg_setup_BLOCKXYGRIP", _wrap_dwg_setup_BLOCKXYGRIP},
{"LibreDWGc::dwg_setup_BLOCKXYPARAMETER", _wrap_dwg_setup_BLOCKXYPARAMETER},
{"LibreDWGc::dwg_setup_DATALINK", _wrap_dwg_setup_DATALINK},
{"LibreDWGc::dwg_setup_DBCOLOR", _wrap_dwg_setup_DBCOLOR},
{"LibreDWGc::dwg_setup_EVALUATION_GRAPH", _wrap_dwg_setup_EVALUATION_GRAPH},
{"LibreDWGc::dwg_setup_FCFOBJECTCONTEXTDATA", _wrap_dwg_setup_FCFOBJECTCONTEXTDATA},
{"LibreDWGc::dwg_setup_GRADIENT_BACKGROUND", _wrap_dwg_setup_GRADIENT_BACKGROUND},
{"LibreDWGc::dwg_setup_GROUND_PLANE_BACKGROUND", _wrap_dwg_setup_GROUND_PLANE_BACKGROUND},
{"LibreDWGc::dwg_setup_IBL_BACKGROUND", _wrap_dwg_setup_IBL_BACKGROUND},
{"LibreDWGc::dwg_setup_IMAGE_BACKGROUND", _wrap_dwg_setup_IMAGE_BACKGROUND},
{"LibreDWGc::dwg_setup_LEADEROBJECTCONTEXTDATA", _wrap_dwg_setup_LEADEROBJECTCONTEXTDATA},
{"LibreDWGc::dwg_setup_LIGHTLIST", _wrap_dwg_setup_LIGHTLIST},
{"LibreDWGc::dwg_setup_MATERIAL", _wrap_dwg_setup_MATERIAL},
{"LibreDWGc::dwg_setup_MENTALRAYRENDERSETTINGS", _wrap_dwg_setup_MENTALRAYRENDERSETTINGS},
{"LibreDWGc::dwg_setup_MLEADERSTYLE", _wrap_dwg_setup_MLEADERSTYLE},
{"LibreDWGc::dwg_setup_MTEXTOBJECTCONTEXTDATA", _wrap_dwg_setup_MTEXTOBJECTCONTEXTDATA},
{"LibreDWGc::dwg_setup_OBJECT_PTR", _wrap_dwg_setup_OBJECT_PTR},
{"LibreDWGc::dwg_setup_PARTIAL_VIEWING_INDEX", _wrap_dwg_setup_PARTIAL_VIEWING_INDEX},
{"LibreDWGc::dwg_setup_POINTCLOUDCOLORMAP", _wrap_dwg_setup_POINTCLOUDCOLORMAP},
{"LibreDWGc::dwg_setup_POINTCLOUDDEF", _wrap_dwg_setup_POINTCLOUDDEF},
{"LibreDWGc::dwg_setup_POINTCLOUDDEFEX", _wrap_dwg_setup_POINTCLOUDDEFEX},
{"LibreDWGc::dwg_setup_POINTCLOUDDEF_REACTOR", _wrap_dwg_setup_POINTCLOUDDEF_REACTOR},
{"LibreDWGc::dwg_setup_POINTCLOUDDEF_REACTOR_EX", _wrap_dwg_setup_POINTCLOUDDEF_REACTOR_EX},
{"LibreDWGc::dwg_setup_PROXY_OBJECT", _wrap_dwg_setup_PROXY_OBJECT},
{"LibreDWGc::dwg_setup_RAPIDRTRENDERSETTINGS", _wrap_dwg_setup_RAPIDRTRENDERSETTINGS},
{"LibreDWGc::dwg_setup_RENDERENTRY", _wrap_dwg_setup_RENDERENTRY},
{"LibreDWGc::dwg_setup_RENDERGLOBAL", _wrap_dwg_setup_RENDERGLOBAL},
{"LibreDWGc::dwg_setup_RENDERSETTINGS", _wrap_dwg_setup_RENDERSETTINGS},
{"LibreDWGc::dwg_setup_SECTION_SETTINGS", _wrap_dwg_setup_SECTION_SETTINGS},
{"LibreDWGc::dwg_setup_SKYLIGHT_BACKGROUND", _wrap_dwg_setup_SKYLIGHT_BACKGROUND},
{"LibreDWGc::dwg_setup_SOLID_BACKGROUND", _wrap_dwg_setup_SOLID_BACKGROUND},
{"LibreDWGc::dwg_setup_SPATIAL_INDEX", _wrap_dwg_setup_SPATIAL_INDEX},
{"LibreDWGc::dwg_setup_TABLESTYLE", _wrap_dwg_setup_TABLESTYLE},
{"LibreDWGc::dwg_setup_TEXTOBJECTCONTEXTDATA", _wrap_dwg_setup_TEXTOBJECTCONTEXTDATA},
{"LibreDWGc::dwg_setup_ASSOCARRAYMODIFYPARAMETERS", _wrap_dwg_setup_ASSOCARRAYMODIFYPARAMETERS},
{"LibreDWGc::dwg_setup_ASSOCARRAYPATHPARAMETERS", _wrap_dwg_setup_ASSOCARRAYPATHPARAMETERS},
{"LibreDWGc::dwg_setup_ASSOCARRAYPOLARPARAMETERS", _wrap_dwg_setup_ASSOCARRAYPOLARPARAMETERS},
{"LibreDWGc::dwg_setup_ASSOCARRAYRECTANGULARPARAMETERS", _wrap_dwg_setup_ASSOCARRAYRECTANGULARPARAMETERS},
{"LibreDWGc::Dwg_DYNAPI_field_name_get", _wrap_Dwg_DYNAPI_field_name_get},
{"LibreDWGc::Dwg_DYNAPI_field_type_get", _wrap_Dwg_DYNAPI_field_type_get},
{"LibreDWGc::Dwg_DYNAPI_field_size_get", _wrap_Dwg_DYNAPI_field_size_get},
{"LibreDWGc::Dwg_DYNAPI_field_offset_get", _wrap_Dwg_DYNAPI_field_offset_get},
{"LibreDWGc::Dwg_DYNAPI_field_is_indirect_get", _wrap_Dwg_DYNAPI_field_is_indirect_get},
{"LibreDWGc::Dwg_DYNAPI_field_is_malloc_get", _wrap_Dwg_DYNAPI_field_is_malloc_get},
{"LibreDWGc::Dwg_DYNAPI_field_is_string_get", _wrap_Dwg_DYNAPI_field_is_string_get},
{"LibreDWGc::Dwg_DYNAPI_field_dxf_get", _wrap_Dwg_DYNAPI_field_dxf_get},
{"LibreDWGc::new_Dwg_DYNAPI_field", _wrap_new_Dwg_DYNAPI_field},
{"LibreDWGc::delete_Dwg_DYNAPI_field", _wrap_delete_Dwg_DYNAPI_field},
{"LibreDWGc::is_dwg_entity", _wrap_is_dwg_entity},
{"LibreDWGc::is_dwg_object", _wrap_is_dwg_object},
{"LibreDWGc::dwg_dynapi_header_value", _wrap_dwg_dynapi_header_value},
{"LibreDWGc::dwg_dynapi_entity_value", _wrap_dwg_dynapi_entity_value},
{"LibreDWGc::dwg_dynapi_common_value", _wrap_dwg_dynapi_common_value},
{"LibreDWGc::dwg_dynapi_subclass_value", _wrap_dwg_dynapi_subclass_value},
{"LibreDWGc::dwg_dynapi_header_utf8text", _wrap_dwg_dynapi_header_utf8text},
{"LibreDWGc::dwg_dynapi_entity_utf8text", _wrap_dwg_dynapi_entity_utf8text},
{"LibreDWGc::dwg_dynapi_common_utf8text", _wrap_dwg_dynapi_common_utf8text},
{"LibreDWGc::dwg_dynapi_header_set_value", _wrap_dwg_dynapi_header_set_value},
{"LibreDWGc::dwg_dynapi_entity_set_value", _wrap_dwg_dynapi_entity_set_value},
{"LibreDWGc::dwg_dynapi_common_set_value", _wrap_dwg_dynapi_common_set_value},
{"LibreDWGc::dwg_dynapi_handle_name", _wrap_dwg_dynapi_handle_name},
{"LibreDWGc::dwg_dynapi_header_field", _wrap_dwg_dynapi_header_field},
{"LibreDWGc::dwg_dynapi_entity_field", _wrap_dwg_dynapi_entity_field},
{"LibreDWGc::dwg_dynapi_subclass_field", _wrap_dwg_dynapi_subclass_field},
{"LibreDWGc::dwg_dynapi_common_entity_field", _wrap_dwg_dynapi_common_entity_field},
{"LibreDWGc::dwg_dynapi_common_object_field", _wrap_dwg_dynapi_common_object_field},
{"LibreDWGc::dwg_dynapi_entity_fields", _wrap_dwg_dynapi_entity_fields},
{"LibreDWGc::dwg_dynapi_common_entity_fields", _wrap_dwg_dynapi_common_entity_fields},
{"LibreDWGc::dwg_dynapi_common_object_fields", _wrap_dwg_dynapi_common_object_fields},
{"LibreDWGc::dwg_dynapi_subclass_fields", _wrap_dwg_dynapi_subclass_fields},
{"LibreDWGc::dwg_dynapi_fields_size", _wrap_dwg_dynapi_fields_size},
{"LibreDWGc::dwg_object_tablectrl_get_entries", _wrap_dwg_object_tablectrl_get_entries},
{"LibreDWGc::dwg_ref_get_table_name", _wrap_dwg_ref_get_table_name},
{"LibreDWGc::dwg_ent_get_layer_name", _wrap_dwg_ent_get_layer_name},
{"LibreDWGc::dwg_obj_layer_get_name", _wrap_dwg_obj_layer_get_name},
{"LibreDWGc::dwg_obj_table_get_name", _wrap_dwg_obj_table_get_name},
{"LibreDWGc::dwg_ent_to_object", _wrap_dwg_ent_to_object},
{"LibreDWGc::dwg_object_to_object", _wrap_dwg_object_to_object},
{"LibreDWGc::dwg_obj_obj_to_object", _wrap_dwg_obj_obj_to_object},
{"LibreDWGc::dwg_object_to_entity", _wrap_dwg_object_to_entity},
{"LibreDWGc::dwg_object_get_type", _wrap_dwg_object_get_type},
{"LibreDWGc::dwg_object_get_fixedtype", _wrap_dwg_object_get_fixedtype},
{"LibreDWGc::dwg_object_get_dxfname", _wrap_dwg_object_get_dxfname},
{"LibreDWGc::dwg_ref_get_absref", _wrap_dwg_ref_get_absref},
{"LibreDWGc::dwg_ref_get_object", _wrap_dwg_ref_get_object},
{"LibreDWGc::dwg_absref_get_object", _wrap_dwg_absref_get_object},
{"LibreDWGc::dwg_get_num_classes", _wrap_dwg_get_num_classes},
{"LibreDWGc::dwg_get_class", _wrap_dwg_get_class},
{"LibreDWGc::dwg_get_object", _wrap_dwg_get_object},
{"LibreDWGc::dwg_obj_generic_to_object", _wrap_dwg_obj_generic_to_object},
{"LibreDWGc::dwg_obj_generic_parent", _wrap_dwg_obj_generic_parent},
{"LibreDWGc::dwg_object_to_DIMENSION", _wrap_dwg_object_to_DIMENSION},
{"LibreDWGc::dwg_getall__3DFACE", _wrap_dwg_getall__3DFACE},
{"LibreDWGc::dwg_getall__3DLINE", _wrap_dwg_getall__3DLINE},
{"LibreDWGc::dwg_getall__3DSOLID", _wrap_dwg_getall__3DSOLID},
{"LibreDWGc::dwg_getall_ARC", _wrap_dwg_getall_ARC},
{"LibreDWGc::dwg_getall_ATTDEF", _wrap_dwg_getall_ATTDEF},
{"LibreDWGc::dwg_getall_ATTRIB", _wrap_dwg_getall_ATTRIB},
{"LibreDWGc::dwg_getall_BLOCK", _wrap_dwg_getall_BLOCK},
{"LibreDWGc::dwg_getall_BODY", _wrap_dwg_getall_BODY},
{"LibreDWGc::dwg_getall_CAMERA", _wrap_dwg_getall_CAMERA},
{"LibreDWGc::dwg_getall_CIRCLE", _wrap_dwg_getall_CIRCLE},
{"LibreDWGc::dwg_getall_DGNUNDERLAY", _wrap_dwg_getall_DGNUNDERLAY},
{"LibreDWGc::dwg_getall_DIMENSION_ALIGNED", _wrap_dwg_getall_DIMENSION_ALIGNED},
{"LibreDWGc::dwg_getall_DIMENSION_ANG2LN", _wrap_dwg_getall_DIMENSION_ANG2LN},
{"LibreDWGc::dwg_getall_DIMENSION_ANG3PT", _wrap_dwg_getall_DIMENSION_ANG3PT},
{"LibreDWGc::dwg_getall_DIMENSION_DIAMETER", _wrap_dwg_getall_DIMENSION_DIAMETER},
{"LibreDWGc::dwg_getall_DIMENSION_LINEAR", _wrap_dwg_getall_DIMENSION_LINEAR},
{"LibreDWGc::dwg_getall_DIMENSION_ORDINATE", _wrap_dwg_getall_DIMENSION_ORDINATE},
{"LibreDWGc::dwg_getall_DIMENSION_RADIUS", _wrap_dwg_getall_DIMENSION_RADIUS},
{"LibreDWGc::dwg_getall_DWFUNDERLAY", _wrap_dwg_getall_DWFUNDERLAY},
{"LibreDWGc::dwg_getall_ELLIPSE", _wrap_dwg_getall_ELLIPSE},
{"LibreDWGc::dwg_getall_ENDBLK", _wrap_dwg_getall_ENDBLK},
{"LibreDWGc::dwg_getall_ENDREP", _wrap_dwg_getall_ENDREP},
{"LibreDWGc::dwg_getall_HATCH", _wrap_dwg_getall_HATCH},
{"LibreDWGc::dwg_getall_IMAGE", _wrap_dwg_getall_IMAGE},
{"LibreDWGc::dwg_getall_INSERT", _wrap_dwg_getall_INSERT},
{"LibreDWGc::dwg_getall_JUMP", _wrap_dwg_getall_JUMP},
{"LibreDWGc::dwg_getall_LEADER", _wrap_dwg_getall_LEADER},
{"LibreDWGc::dwg_getall_LIGHT", _wrap_dwg_getall_LIGHT},
{"LibreDWGc::dwg_getall_LINE", _wrap_dwg_getall_LINE},
{"LibreDWGc::dwg_getall_LOAD", _wrap_dwg_getall_LOAD},
{"LibreDWGc::dwg_getall_LWPOLYLINE", _wrap_dwg_getall_LWPOLYLINE},
{"LibreDWGc::dwg_getall_MESH", _wrap_dwg_getall_MESH},
{"LibreDWGc::dwg_getall_MINSERT", _wrap_dwg_getall_MINSERT},
{"LibreDWGc::dwg_getall_MLINE", _wrap_dwg_getall_MLINE},
{"LibreDWGc::dwg_getall_MTEXT", _wrap_dwg_getall_MTEXT},
{"LibreDWGc::dwg_getall_MULTILEADER", _wrap_dwg_getall_MULTILEADER},
{"LibreDWGc::dwg_getall_OLE2FRAME", _wrap_dwg_getall_OLE2FRAME},
{"LibreDWGc::dwg_getall_OLEFRAME", _wrap_dwg_getall_OLEFRAME},
{"LibreDWGc::dwg_getall_PDFUNDERLAY", _wrap_dwg_getall_PDFUNDERLAY},
{"LibreDWGc::dwg_getall_POINT", _wrap_dwg_getall_POINT},
{"LibreDWGc::dwg_getall_POLYLINE_2D", _wrap_dwg_getall_POLYLINE_2D},
{"LibreDWGc::dwg_getall_POLYLINE_3D", _wrap_dwg_getall_POLYLINE_3D},
{"LibreDWGc::dwg_getall_POLYLINE_MESH", _wrap_dwg_getall_POLYLINE_MESH},
{"LibreDWGc::dwg_getall_POLYLINE_PFACE", _wrap_dwg_getall_POLYLINE_PFACE},
{"LibreDWGc::dwg_getall_PROXY_ENTITY", _wrap_dwg_getall_PROXY_ENTITY},
{"LibreDWGc::dwg_getall_RAY", _wrap_dwg_getall_RAY},
{"LibreDWGc::dwg_getall_REGION", _wrap_dwg_getall_REGION},
{"LibreDWGc::dwg_getall_REPEAT", _wrap_dwg_getall_REPEAT},
{"LibreDWGc::dwg_getall_SECTIONOBJECT", _wrap_dwg_getall_SECTIONOBJECT},
{"LibreDWGc::dwg_getall_SEQEND", _wrap_dwg_getall_SEQEND},
{"LibreDWGc::dwg_getall_SHAPE", _wrap_dwg_getall_SHAPE},
{"LibreDWGc::dwg_getall_SOLID", _wrap_dwg_getall_SOLID},
{"LibreDWGc::dwg_getall_SPLINE", _wrap_dwg_getall_SPLINE},
{"LibreDWGc::dwg_getall_TEXT", _wrap_dwg_getall_TEXT},
{"LibreDWGc::dwg_getall_TOLERANCE", _wrap_dwg_getall_TOLERANCE},
{"LibreDWGc::dwg_getall_TRACE", _wrap_dwg_getall_TRACE},
{"LibreDWGc::dwg_getall_UNKNOWN_ENT", _wrap_dwg_getall_UNKNOWN_ENT},
{"LibreDWGc::dwg_getall_VERTEX_2D", _wrap_dwg_getall_VERTEX_2D},
{"LibreDWGc::dwg_getall_VERTEX_3D", _wrap_dwg_getall_VERTEX_3D},
{"LibreDWGc::dwg_getall_VERTEX_MESH", _wrap_dwg_getall_VERTEX_MESH},
{"LibreDWGc::dwg_getall_VERTEX_PFACE", _wrap_dwg_getall_VERTEX_PFACE},
{"LibreDWGc::dwg_getall_VERTEX_PFACE_FACE", _wrap_dwg_getall_VERTEX_PFACE_FACE},
{"LibreDWGc::dwg_getall_VIEWPORT", _wrap_dwg_getall_VIEWPORT},
{"LibreDWGc::dwg_getall_WIPEOUT", _wrap_dwg_getall_WIPEOUT},
{"LibreDWGc::dwg_getall_XLINE", _wrap_dwg_getall_XLINE},
{"LibreDWGc::dwg_getall_ARC_DIMENSION", _wrap_dwg_getall_ARC_DIMENSION},
{"LibreDWGc::dwg_getall_HELIX", _wrap_dwg_getall_HELIX},
{"LibreDWGc::dwg_getall_LARGE_RADIAL_DIMENSION", _wrap_dwg_getall_LARGE_RADIAL_DIMENSION},
{"LibreDWGc::dwg_getall_LAYOUTPRINTCONFIG", _wrap_dwg_getall_LAYOUTPRINTCONFIG},
{"LibreDWGc::dwg_getall_PLANESURFACE", _wrap_dwg_getall_PLANESURFACE},
{"LibreDWGc::dwg_getall_POINTCLOUD", _wrap_dwg_getall_POINTCLOUD},
{"LibreDWGc::dwg_getall_POINTCLOUDEX", _wrap_dwg_getall_POINTCLOUDEX},
{"LibreDWGc::dwg_getall_ACSH_BOOLEAN_CLASS", _wrap_dwg_getall_ACSH_BOOLEAN_CLASS},
{"LibreDWGc::dwg_getall_ACSH_BOX_CLASS", _wrap_dwg_getall_ACSH_BOX_CLASS},
{"LibreDWGc::dwg_getall_ACSH_CONE_CLASS", _wrap_dwg_getall_ACSH_CONE_CLASS},
{"LibreDWGc::dwg_getall_ACSH_CYLINDER_CLASS", _wrap_dwg_getall_ACSH_CYLINDER_CLASS},
{"LibreDWGc::dwg_getall_ACSH_FILLET_CLASS", _wrap_dwg_getall_ACSH_FILLET_CLASS},
{"LibreDWGc::dwg_getall_ACSH_HISTORY_CLASS", _wrap_dwg_getall_ACSH_HISTORY_CLASS},
{"LibreDWGc::dwg_getall_ACSH_SPHERE_CLASS", _wrap_dwg_getall_ACSH_SPHERE_CLASS},
{"LibreDWGc::dwg_getall_ACSH_TORUS_CLASS", _wrap_dwg_getall_ACSH_TORUS_CLASS},
{"LibreDWGc::dwg_getall_ACSH_WEDGE_CLASS", _wrap_dwg_getall_ACSH_WEDGE_CLASS},
{"LibreDWGc::dwg_getall_APPID", _wrap_dwg_getall_APPID},
{"LibreDWGc::dwg_getall_APPID_CONTROL", _wrap_dwg_getall_APPID_CONTROL},
{"LibreDWGc::dwg_getall_ASSOCGEOMDEPENDENCY", _wrap_dwg_getall_ASSOCGEOMDEPENDENCY},
{"LibreDWGc::dwg_getall_ASSOCNETWORK", _wrap_dwg_getall_ASSOCNETWORK},
{"LibreDWGc::dwg_getall_BLOCKALIGNMENTGRIP", _wrap_dwg_getall_BLOCKALIGNMENTGRIP},
{"LibreDWGc::dwg_getall_BLOCKALIGNMENTPARAMETER", _wrap_dwg_getall_BLOCKALIGNMENTPARAMETER},
{"LibreDWGc::dwg_getall_BLOCKBASEPOINTPARAMETER", _wrap_dwg_getall_BLOCKBASEPOINTPARAMETER},
{"LibreDWGc::dwg_getall_BLOCKFLIPACTION", _wrap_dwg_getall_BLOCKFLIPACTION},
{"LibreDWGc::dwg_getall_BLOCKFLIPGRIP", _wrap_dwg_getall_BLOCKFLIPGRIP},
{"LibreDWGc::dwg_getall_BLOCKFLIPPARAMETER", _wrap_dwg_getall_BLOCKFLIPPARAMETER},
{"LibreDWGc::dwg_getall_BLOCKGRIPLOCATIONCOMPONENT", _wrap_dwg_getall_BLOCKGRIPLOCATIONCOMPONENT},
{"LibreDWGc::dwg_getall_BLOCKLINEARGRIP", _wrap_dwg_getall_BLOCKLINEARGRIP},
{"LibreDWGc::dwg_getall_BLOCKLOOKUPGRIP", _wrap_dwg_getall_BLOCKLOOKUPGRIP},
{"LibreDWGc::dwg_getall_BLOCKMOVEACTION", _wrap_dwg_getall_BLOCKMOVEACTION},
{"LibreDWGc::dwg_getall_BLOCKROTATEACTION", _wrap_dwg_getall_BLOCKROTATEACTION},
{"LibreDWGc::dwg_getall_BLOCKROTATIONGRIP", _wrap_dwg_getall_BLOCKROTATIONGRIP},
{"LibreDWGc::dwg_getall_BLOCKSCALEACTION", _wrap_dwg_getall_BLOCKSCALEACTION},
{"LibreDWGc::dwg_getall_BLOCKVISIBILITYGRIP", _wrap_dwg_getall_BLOCKVISIBILITYGRIP},
{"LibreDWGc::dwg_getall_BLOCK_CONTROL", _wrap_dwg_getall_BLOCK_CONTROL},
{"LibreDWGc::dwg_getall_BLOCK_HEADER", _wrap_dwg_getall_BLOCK_HEADER},
{"LibreDWGc::dwg_getall_CELLSTYLEMAP", _wrap_dwg_getall_CELLSTYLEMAP},
{"LibreDWGc::dwg_getall_DETAILVIEWSTYLE", _wrap_dwg_getall_DETAILVIEWSTYLE},
{"LibreDWGc::dwg_getall_DICTIONARY", _wrap_dwg_getall_DICTIONARY},
{"LibreDWGc::dwg_getall_DICTIONARYVAR", _wrap_dwg_getall_DICTIONARYVAR},
{"LibreDWGc::dwg_getall_DICTIONARYWDFLT", _wrap_dwg_getall_DICTIONARYWDFLT},
{"LibreDWGc::dwg_getall_DIMSTYLE", _wrap_dwg_getall_DIMSTYLE},
{"LibreDWGc::dwg_getall_DIMSTYLE_CONTROL", _wrap_dwg_getall_DIMSTYLE_CONTROL},
{"LibreDWGc::dwg_getall_DUMMY", _wrap_dwg_getall_DUMMY},
{"LibreDWGc::dwg_getall_DYNAMICBLOCKPURGEPREVENTER", _wrap_dwg_getall_DYNAMICBLOCKPURGEPREVENTER},
{"LibreDWGc::dwg_getall_FIELD", _wrap_dwg_getall_FIELD},
{"LibreDWGc::dwg_getall_FIELDLIST", _wrap_dwg_getall_FIELDLIST},
{"LibreDWGc::dwg_getall_GEODATA", _wrap_dwg_getall_GEODATA},
{"LibreDWGc::dwg_getall_GROUP", _wrap_dwg_getall_GROUP},
{"LibreDWGc::dwg_getall_IDBUFFER", _wrap_dwg_getall_IDBUFFER},
{"LibreDWGc::dwg_getall_IMAGEDEF", _wrap_dwg_getall_IMAGEDEF},
{"LibreDWGc::dwg_getall_IMAGEDEF_REACTOR", _wrap_dwg_getall_IMAGEDEF_REACTOR},
{"LibreDWGc::dwg_getall_INDEX", _wrap_dwg_getall_INDEX},
{"LibreDWGc::dwg_getall_LAYER", _wrap_dwg_getall_LAYER},
{"LibreDWGc::dwg_getall_LAYERFILTER", _wrap_dwg_getall_LAYERFILTER},
{"LibreDWGc::dwg_getall_LAYER_CONTROL", _wrap_dwg_getall_LAYER_CONTROL},
{"LibreDWGc::dwg_getall_LAYER_INDEX", _wrap_dwg_getall_LAYER_INDEX},
{"LibreDWGc::dwg_getall_LAYOUT", _wrap_dwg_getall_LAYOUT},
{"LibreDWGc::dwg_getall_LONG_TRANSACTION", _wrap_dwg_getall_LONG_TRANSACTION},
{"LibreDWGc::dwg_getall_LTYPE", _wrap_dwg_getall_LTYPE},
{"LibreDWGc::dwg_getall_LTYPE_CONTROL", _wrap_dwg_getall_LTYPE_CONTROL},
{"LibreDWGc::dwg_getall_MLINESTYLE", _wrap_dwg_getall_MLINESTYLE},
{"LibreDWGc::dwg_getall_PLACEHOLDER", _wrap_dwg_getall_PLACEHOLDER},
{"LibreDWGc::dwg_getall_PLOTSETTINGS", _wrap_dwg_getall_PLOTSETTINGS},
{"LibreDWGc::dwg_getall_RASTERVARIABLES", _wrap_dwg_getall_RASTERVARIABLES},
{"LibreDWGc::dwg_getall_RENDERENVIRONMENT", _wrap_dwg_getall_RENDERENVIRONMENT},
{"LibreDWGc::dwg_getall_SCALE", _wrap_dwg_getall_SCALE},
{"LibreDWGc::dwg_getall_SECTIONVIEWSTYLE", _wrap_dwg_getall_SECTIONVIEWSTYLE},
{"LibreDWGc::dwg_getall_SECTION_MANAGER", _wrap_dwg_getall_SECTION_MANAGER},
{"LibreDWGc::dwg_getall_SORTENTSTABLE", _wrap_dwg_getall_SORTENTSTABLE},
{"LibreDWGc::dwg_getall_SPATIAL_FILTER", _wrap_dwg_getall_SPATIAL_FILTER},
{"LibreDWGc::dwg_getall_STYLE", _wrap_dwg_getall_STYLE},
{"LibreDWGc::dwg_getall_STYLE_CONTROL", _wrap_dwg_getall_STYLE_CONTROL},
{"LibreDWGc::dwg_getall_SUN", _wrap_dwg_getall_SUN},
{"LibreDWGc::dwg_getall_TABLEGEOMETRY", _wrap_dwg_getall_TABLEGEOMETRY},
{"LibreDWGc::dwg_getall_UCS", _wrap_dwg_getall_UCS},
{"LibreDWGc::dwg_getall_UCS_CONTROL", _wrap_dwg_getall_UCS_CONTROL},
{"LibreDWGc::dwg_getall_UNKNOWN_OBJ", _wrap_dwg_getall_UNKNOWN_OBJ},
{"LibreDWGc::dwg_getall_VBA_PROJECT", _wrap_dwg_getall_VBA_PROJECT},
{"LibreDWGc::dwg_getall_VIEW", _wrap_dwg_getall_VIEW},
{"LibreDWGc::dwg_getall_VIEW_CONTROL", _wrap_dwg_getall_VIEW_CONTROL},
{"LibreDWGc::dwg_getall_VISUALSTYLE", _wrap_dwg_getall_VISUALSTYLE},
{"LibreDWGc::dwg_getall_VPORT", _wrap_dwg_getall_VPORT},
{"LibreDWGc::dwg_getall_VPORT_CONTROL", _wrap_dwg_getall_VPORT_CONTROL},
{"LibreDWGc::dwg_getall_VX_CONTROL", _wrap_dwg_getall_VX_CONTROL},
{"LibreDWGc::dwg_getall_VX_TABLE_RECORD", _wrap_dwg_getall_VX_TABLE_RECORD},
{"LibreDWGc::dwg_getall_WIPEOUTVARIABLES", _wrap_dwg_getall_WIPEOUTVARIABLES},
{"LibreDWGc::dwg_getall_XRECORD", _wrap_dwg_getall_XRECORD},
{"LibreDWGc::dwg_getall_PDFDEFINITION", _wrap_dwg_getall_PDFDEFINITION},
{"LibreDWGc::dwg_getall_DGNDEFINITION", _wrap_dwg_getall_DGNDEFINITION},
{"LibreDWGc::dwg_getall_DWFDEFINITION", _wrap_dwg_getall_DWFDEFINITION},
{"LibreDWGc::dwg_getall_ACSH_BREP_CLASS", _wrap_dwg_getall_ACSH_BREP_CLASS},
{"LibreDWGc::dwg_getall_ACSH_CHAMFER_CLASS", _wrap_dwg_getall_ACSH_CHAMFER_CLASS},
{"LibreDWGc::dwg_getall_ACSH_PYRAMID_CLASS", _wrap_dwg_getall_ACSH_PYRAMID_CLASS},
{"LibreDWGc::dwg_getall_ALDIMOBJECTCONTEXTDATA", _wrap_dwg_getall_ALDIMOBJECTCONTEXTDATA},
{"LibreDWGc::dwg_getall_ASSOC2DCONSTRAINTGROUP", _wrap_dwg_getall_ASSOC2DCONSTRAINTGROUP},
{"LibreDWGc::dwg_getall_ASSOCACTION", _wrap_dwg_getall_ASSOCACTION},
{"LibreDWGc::dwg_getall_ASSOCACTIONPARAM", _wrap_dwg_getall_ASSOCACTIONPARAM},
{"LibreDWGc::dwg_getall_ASSOCARRAYACTIONBODY", _wrap_dwg_getall_ASSOCARRAYACTIONBODY},
{"LibreDWGc::dwg_getall_ASSOCASMBODYACTIONPARAM", _wrap_dwg_getall_ASSOCASMBODYACTIONPARAM},
{"LibreDWGc::dwg_getall_ASSOCBLENDSURFACEACTIONBODY", _wrap_dwg_getall_ASSOCBLENDSURFACEACTIONBODY},
{"LibreDWGc::dwg_getall_ASSOCCOMPOUNDACTIONPARAM", _wrap_dwg_getall_ASSOCCOMPOUNDACTIONPARAM},
{"LibreDWGc::dwg_getall_ASSOCDEPENDENCY", _wrap_dwg_getall_ASSOCDEPENDENCY},
{"LibreDWGc::dwg_getall_ASSOCDIMDEPENDENCYBODY", _wrap_dwg_getall_ASSOCDIMDEPENDENCYBODY},
{"LibreDWGc::dwg_getall_ASSOCEXTENDSURFACEACTIONBODY", _wrap_dwg_getall_ASSOCEXTENDSURFACEACTIONBODY},
{"LibreDWGc::dwg_getall_ASSOCEXTRUDEDSURFACEACTIONBODY", _wrap_dwg_getall_ASSOCEXTRUDEDSURFACEACTIONBODY},
{"LibreDWGc::dwg_getall_ASSOCFACEACTIONPARAM", _wrap_dwg_getall_ASSOCFACEACTIONPARAM},
{"LibreDWGc::dwg_getall_ASSOCFILLETSURFACEACTIONBODY", _wrap_dwg_getall_ASSOCFILLETSURFACEACTIONBODY},
{"LibreDWGc::dwg_getall_ASSOCLOFTEDSURFACEACTIONBODY", _wrap_dwg_getall_ASSOCLOFTEDSURFACEACTIONBODY},
{"LibreDWGc::dwg_getall_ASSOCNETWORKSURFACEACTIONBODY", _wrap_dwg_getall_ASSOCNETWORKSURFACEACTIONBODY},
{"LibreDWGc::dwg_getall_ASSOCOBJECTACTIONPARAM", _wrap_dwg_getall_ASSOCOBJECTACTIONPARAM},
{"LibreDWGc::dwg_getall_ASSOCOFFSETSURFACEACTIONBODY", _wrap_dwg_getall_ASSOCOFFSETSURFACEACTIONBODY},
{"LibreDWGc::dwg_getall_ASSOCOSNAPPOINTREFACTIONPARAM", _wrap_dwg_getall_ASSOCOSNAPPOINTREFACTIONPARAM},
{"LibreDWGc::dwg_getall_ASSOCPATCHSURFACEACTIONBODY", _wrap_dwg_getall_ASSOCPATCHSURFACEACTIONBODY},
{"LibreDWGc::dwg_getall_ASSOCPATHACTIONPARAM", _wrap_dwg_getall_ASSOCPATHACTIONPARAM},
{"LibreDWGc::dwg_getall_ASSOCPLANESURFACEACTIONBODY", _wrap_dwg_getall_ASSOCPLANESURFACEACTIONBODY},
{"LibreDWGc::dwg_getall_ASSOCPOINTREFACTIONPARAM", _wrap_dwg_getall_ASSOCPOINTREFACTIONPARAM},
{"LibreDWGc::dwg_getall_ASSOCREVOLVEDSURFACEACTIONBODY", _wrap_dwg_getall_ASSOCREVOLVEDSURFACEACTIONBODY},
{"LibreDWGc::dwg_getall_ASSOCTRIMSURFACEACTIONBODY", _wrap_dwg_getall_ASSOCTRIMSURFACEACTIONBODY},
{"LibreDWGc::dwg_getall_ASSOCVALUEDEPENDENCY", _wrap_dwg_getall_ASSOCVALUEDEPENDENCY},
{"LibreDWGc::dwg_getall_ASSOCVARIABLE", _wrap_dwg_getall_ASSOCVARIABLE},
{"LibreDWGc::dwg_getall_ASSOCVERTEXACTIONPARAM", _wrap_dwg_getall_ASSOCVERTEXACTIONPARAM},
{"LibreDWGc::dwg_getall_BLKREFOBJECTCONTEXTDATA", _wrap_dwg_getall_BLKREFOBJECTCONTEXTDATA},
{"LibreDWGc::dwg_getall_BLOCKALIGNEDCONSTRAINTPARAMETER", _wrap_dwg_getall_BLOCKALIGNEDCONSTRAINTPARAMETER},
{"LibreDWGc::dwg_getall_BLOCKANGULARCONSTRAINTPARAMETER", _wrap_dwg_getall_BLOCKANGULARCONSTRAINTPARAMETER},
{"LibreDWGc::dwg_getall_BLOCKARRAYACTION", _wrap_dwg_getall_BLOCKARRAYACTION},
{"LibreDWGc::dwg_getall_BLOCKDIAMETRICCONSTRAINTPARAMETER", _wrap_dwg_getall_BLOCKDIAMETRICCONSTRAINTPARAMETER},
{"LibreDWGc::dwg_getall_BLOCKHORIZONTALCONSTRAINTPARAMETER", _wrap_dwg_getall_BLOCKHORIZONTALCONSTRAINTPARAMETER},
{"LibreDWGc::dwg_getall_BLOCKLINEARCONSTRAINTPARAMETER", _wrap_dwg_getall_BLOCKLINEARCONSTRAINTPARAMETER},
{"LibreDWGc::dwg_getall_BLOCKLINEARPARAMETER", _wrap_dwg_getall_BLOCKLINEARPARAMETER},
{"LibreDWGc::dwg_getall_BLOCKLOOKUPACTION", _wrap_dwg_getall_BLOCKLOOKUPACTION},
{"LibreDWGc::dwg_getall_BLOCKLOOKUPPARAMETER", _wrap_dwg_getall_BLOCKLOOKUPPARAMETER},
{"LibreDWGc::dwg_getall_BLOCKPARAMDEPENDENCYBODY", _wrap_dwg_getall_BLOCKPARAMDEPENDENCYBODY},
{"LibreDWGc::dwg_getall_BLOCKPOINTPARAMETER", _wrap_dwg_getall_BLOCKPOINTPARAMETER},
{"LibreDWGc::dwg_getall_BLOCKPOLARGRIP", _wrap_dwg_getall_BLOCKPOLARGRIP},
{"LibreDWGc::dwg_getall_BLOCKPOLARPARAMETER", _wrap_dwg_getall_BLOCKPOLARPARAMETER},
{"LibreDWGc::dwg_getall_BLOCKPOLARSTRETCHACTION", _wrap_dwg_getall_BLOCKPOLARSTRETCHACTION},
{"LibreDWGc::dwg_getall_BLOCKRADIALCONSTRAINTPARAMETER", _wrap_dwg_getall_BLOCKRADIALCONSTRAINTPARAMETER},
{"LibreDWGc::dwg_getall_BLOCKREPRESENTATION", _wrap_dwg_getall_BLOCKREPRESENTATION},
{"LibreDWGc::dwg_getall_BLOCKROTATIONPARAMETER", _wrap_dwg_getall_BLOCKROTATIONPARAMETER},
{"LibreDWGc::dwg_getall_BLOCKSTRETCHACTION", _wrap_dwg_getall_BLOCKSTRETCHACTION},
{"LibreDWGc::dwg_getall_BLOCKUSERPARAMETER", _wrap_dwg_getall_BLOCKUSERPARAMETER},
{"LibreDWGc::dwg_getall_BLOCKVERTICALCONSTRAINTPARAMETER", _wrap_dwg_getall_BLOCKVERTICALCONSTRAINTPARAMETER},
{"LibreDWGc::dwg_getall_BLOCKVISIBILITYPARAMETER", _wrap_dwg_getall_BLOCKVISIBILITYPARAMETER},
{"LibreDWGc::dwg_getall_BLOCKXYGRIP", _wrap_dwg_getall_BLOCKXYGRIP},
{"LibreDWGc::dwg_getall_BLOCKXYPARAMETER", _wrap_dwg_getall_BLOCKXYPARAMETER},
{"LibreDWGc::dwg_getall_DATALINK", _wrap_dwg_getall_DATALINK},
{"LibreDWGc::dwg_getall_DBCOLOR", _wrap_dwg_getall_DBCOLOR},
{"LibreDWGc::dwg_getall_EVALUATION_GRAPH", _wrap_dwg_getall_EVALUATION_GRAPH},
{"LibreDWGc::dwg_getall_FCFOBJECTCONTEXTDATA", _wrap_dwg_getall_FCFOBJECTCONTEXTDATA},
{"LibreDWGc::dwg_getall_GRADIENT_BACKGROUND", _wrap_dwg_getall_GRADIENT_BACKGROUND},
{"LibreDWGc::dwg_getall_GROUND_PLANE_BACKGROUND", _wrap_dwg_getall_GROUND_PLANE_BACKGROUND},
{"LibreDWGc::dwg_getall_IBL_BACKGROUND", _wrap_dwg_getall_IBL_BACKGROUND},
{"LibreDWGc::dwg_getall_IMAGE_BACKGROUND", _wrap_dwg_getall_IMAGE_BACKGROUND},
{"LibreDWGc::dwg_getall_LEADEROBJECTCONTEXTDATA", _wrap_dwg_getall_LEADEROBJECTCONTEXTDATA},
{"LibreDWGc::dwg_getall_LIGHTLIST", _wrap_dwg_getall_LIGHTLIST},
{"LibreDWGc::dwg_getall_MATERIAL", _wrap_dwg_getall_MATERIAL},
{"LibreDWGc::dwg_getall_MENTALRAYRENDERSETTINGS", _wrap_dwg_getall_MENTALRAYRENDERSETTINGS},
{"LibreDWGc::dwg_getall_MLEADERSTYLE", _wrap_dwg_getall_MLEADERSTYLE},
{"LibreDWGc::dwg_getall_MTEXTOBJECTCONTEXTDATA", _wrap_dwg_getall_MTEXTOBJECTCONTEXTDATA},
{"LibreDWGc::dwg_getall_OBJECT_PTR", _wrap_dwg_getall_OBJECT_PTR},
{"LibreDWGc::dwg_getall_PARTIAL_VIEWING_INDEX", _wrap_dwg_getall_PARTIAL_VIEWING_INDEX},
{"LibreDWGc::dwg_getall_POINTCLOUDCOLORMAP", _wrap_dwg_getall_POINTCLOUDCOLORMAP},
{"LibreDWGc::dwg_getall_POINTCLOUDDEF", _wrap_dwg_getall_POINTCLOUDDEF},
{"LibreDWGc::dwg_getall_POINTCLOUDDEFEX", _wrap_dwg_getall_POINTCLOUDDEFEX},
{"LibreDWGc::dwg_getall_POINTCLOUDDEF_REACTOR", _wrap_dwg_getall_POINTCLOUDDEF_REACTOR},
{"LibreDWGc::dwg_getall_POINTCLOUDDEF_REACTOR_EX", _wrap_dwg_getall_POINTCLOUDDEF_REACTOR_EX},
{"LibreDWGc::dwg_getall_PROXY_OBJECT", _wrap_dwg_getall_PROXY_OBJECT},
{"LibreDWGc::dwg_getall_RAPIDRTRENDERSETTINGS", _wrap_dwg_getall_RAPIDRTRENDERSETTINGS},
{"LibreDWGc::dwg_getall_RENDERENTRY", _wrap_dwg_getall_RENDERENTRY},
{"LibreDWGc::dwg_getall_RENDERGLOBAL", _wrap_dwg_getall_RENDERGLOBAL},
{"LibreDWGc::dwg_getall_RENDERSETTINGS", _wrap_dwg_getall_RENDERSETTINGS},
{"LibreDWGc::dwg_getall_SECTION_SETTINGS", _wrap_dwg_getall_SECTION_SETTINGS},
{"LibreDWGc::dwg_getall_SKYLIGHT_BACKGROUND", _wrap_dwg_getall_SKYLIGHT_BACKGROUND},
{"LibreDWGc::dwg_getall_SOLID_BACKGROUND", _wrap_dwg_getall_SOLID_BACKGROUND},
{"LibreDWGc::dwg_getall_SPATIAL_INDEX", _wrap_dwg_getall_SPATIAL_INDEX},
{"LibreDWGc::dwg_getall_TABLESTYLE", _wrap_dwg_getall_TABLESTYLE},
{"LibreDWGc::dwg_getall_TEXTOBJECTCONTEXTDATA", _wrap_dwg_getall_TEXTOBJECTCONTEXTDATA},
{"LibreDWGc::dwg_getall_ASSOCARRAYMODIFYPARAMETERS", _wrap_dwg_getall_ASSOCARRAYMODIFYPARAMETERS},
{"LibreDWGc::dwg_getall_ASSOCARRAYPATHPARAMETERS", _wrap_dwg_getall_ASSOCARRAYPATHPARAMETERS},
{"LibreDWGc::dwg_getall_ASSOCARRAYPOLARPARAMETERS", _wrap_dwg_getall_ASSOCARRAYPOLARPARAMETERS},
{"LibreDWGc::dwg_getall_ASSOCARRAYRECTANGULARPARAMETERS", _wrap_dwg_getall_ASSOCARRAYRECTANGULARPARAMETERS},
{"LibreDWGc::dwg_object_to__3DFACE", _wrap_dwg_object_to__3DFACE},
{"LibreDWGc::dwg_object_to__3DLINE", _wrap_dwg_object_to__3DLINE},
{"LibreDWGc::dwg_object_to__3DSOLID", _wrap_dwg_object_to__3DSOLID},
{"LibreDWGc::dwg_object_to_ARC", _wrap_dwg_object_to_ARC},
{"LibreDWGc::dwg_object_to_ATTDEF", _wrap_dwg_object_to_ATTDEF},
{"LibreDWGc::dwg_object_to_ATTRIB", _wrap_dwg_object_to_ATTRIB},
{"LibreDWGc::dwg_object_to_BLOCK", _wrap_dwg_object_to_BLOCK},
{"LibreDWGc::dwg_object_to_BODY", _wrap_dwg_object_to_BODY},
{"LibreDWGc::dwg_object_to_CAMERA", _wrap_dwg_object_to_CAMERA},
{"LibreDWGc::dwg_object_to_CIRCLE", _wrap_dwg_object_to_CIRCLE},
{"LibreDWGc::dwg_object_to_DGNUNDERLAY", _wrap_dwg_object_to_DGNUNDERLAY},
{"LibreDWGc::dwg_object_to_DIMENSION_ALIGNED", _wrap_dwg_object_to_DIMENSION_ALIGNED},
{"LibreDWGc::dwg_object_to_DIMENSION_ANG2LN", _wrap_dwg_object_to_DIMENSION_ANG2LN},
{"LibreDWGc::dwg_object_to_DIMENSION_ANG3PT", _wrap_dwg_object_to_DIMENSION_ANG3PT},
{"LibreDWGc::dwg_object_to_DIMENSION_DIAMETER", _wrap_dwg_object_to_DIMENSION_DIAMETER},
{"LibreDWGc::dwg_object_to_DIMENSION_LINEAR", _wrap_dwg_object_to_DIMENSION_LINEAR},
{"LibreDWGc::dwg_object_to_DIMENSION_ORDINATE", _wrap_dwg_object_to_DIMENSION_ORDINATE},
{"LibreDWGc::dwg_object_to_DIMENSION_RADIUS", _wrap_dwg_object_to_DIMENSION_RADIUS},
{"LibreDWGc::dwg_object_to_DWFUNDERLAY", _wrap_dwg_object_to_DWFUNDERLAY},
{"LibreDWGc::dwg_object_to_ELLIPSE", _wrap_dwg_object_to_ELLIPSE},
{"LibreDWGc::dwg_object_to_ENDBLK", _wrap_dwg_object_to_ENDBLK},
{"LibreDWGc::dwg_object_to_ENDREP", _wrap_dwg_object_to_ENDREP},
{"LibreDWGc::dwg_object_to_HATCH", _wrap_dwg_object_to_HATCH},
{"LibreDWGc::dwg_object_to_IMAGE", _wrap_dwg_object_to_IMAGE},
{"LibreDWGc::dwg_object_to_INSERT", _wrap_dwg_object_to_INSERT},
{"LibreDWGc::dwg_object_to_JUMP", _wrap_dwg_object_to_JUMP},
{"LibreDWGc::dwg_object_to_LEADER", _wrap_dwg_object_to_LEADER},
{"LibreDWGc::dwg_object_to_LIGHT", _wrap_dwg_object_to_LIGHT},
{"LibreDWGc::dwg_object_to_LINE", _wrap_dwg_object_to_LINE},
{"LibreDWGc::dwg_object_to_LOAD", _wrap_dwg_object_to_LOAD},
{"LibreDWGc::dwg_object_to_LWPOLYLINE", _wrap_dwg_object_to_LWPOLYLINE},
{"LibreDWGc::dwg_object_to_MESH", _wrap_dwg_object_to_MESH},
{"LibreDWGc::dwg_object_to_MINSERT", _wrap_dwg_object_to_MINSERT},
{"LibreDWGc::dwg_object_to_MLINE", _wrap_dwg_object_to_MLINE},
{"LibreDWGc::dwg_object_to_MTEXT", _wrap_dwg_object_to_MTEXT},
{"LibreDWGc::dwg_object_to_MULTILEADER", _wrap_dwg_object_to_MULTILEADER},
{"LibreDWGc::dwg_object_to_OLE2FRAME", _wrap_dwg_object_to_OLE2FRAME},
{"LibreDWGc::dwg_object_to_OLEFRAME", _wrap_dwg_object_to_OLEFRAME},
{"LibreDWGc::dwg_object_to_PDFUNDERLAY", _wrap_dwg_object_to_PDFUNDERLAY},
{"LibreDWGc::dwg_object_to_POINT", _wrap_dwg_object_to_POINT},
{"LibreDWGc::dwg_object_to_POLYLINE_2D", _wrap_dwg_object_to_POLYLINE_2D},
{"LibreDWGc::dwg_object_to_POLYLINE_3D", _wrap_dwg_object_to_POLYLINE_3D},
{"LibreDWGc::dwg_object_to_POLYLINE_MESH", _wrap_dwg_object_to_POLYLINE_MESH},
{"LibreDWGc::dwg_object_to_POLYLINE_PFACE", _wrap_dwg_object_to_POLYLINE_PFACE},
{"LibreDWGc::dwg_object_to_PROXY_ENTITY", _wrap_dwg_object_to_PROXY_ENTITY},
{"LibreDWGc::dwg_object_to_RAY", _wrap_dwg_object_to_RAY},
{"LibreDWGc::dwg_object_to_REGION", _wrap_dwg_object_to_REGION},
{"LibreDWGc::dwg_object_to_REPEAT", _wrap_dwg_object_to_REPEAT},
{"LibreDWGc::dwg_object_to_SECTIONOBJECT", _wrap_dwg_object_to_SECTIONOBJECT},
{"LibreDWGc::dwg_object_to_SEQEND", _wrap_dwg_object_to_SEQEND},
{"LibreDWGc::dwg_object_to_SHAPE", _wrap_dwg_object_to_SHAPE},
{"LibreDWGc::dwg_object_to_SOLID", _wrap_dwg_object_to_SOLID},
{"LibreDWGc::dwg_object_to_SPLINE", _wrap_dwg_object_to_SPLINE},
{"LibreDWGc::dwg_object_to_TEXT", _wrap_dwg_object_to_TEXT},
{"LibreDWGc::dwg_object_to_TOLERANCE", _wrap_dwg_object_to_TOLERANCE},
{"LibreDWGc::dwg_object_to_TRACE", _wrap_dwg_object_to_TRACE},
{"LibreDWGc::dwg_object_to_UNKNOWN_ENT", _wrap_dwg_object_to_UNKNOWN_ENT},
{"LibreDWGc::dwg_object_to_VERTEX_2D", _wrap_dwg_object_to_VERTEX_2D},
{"LibreDWGc::dwg_object_to_VERTEX_3D", _wrap_dwg_object_to_VERTEX_3D},
{"LibreDWGc::dwg_object_to_VERTEX_MESH", _wrap_dwg_object_to_VERTEX_MESH},
{"LibreDWGc::dwg_object_to_VERTEX_PFACE", _wrap_dwg_object_to_VERTEX_PFACE},
{"LibreDWGc::dwg_object_to_VERTEX_PFACE_FACE", _wrap_dwg_object_to_VERTEX_PFACE_FACE},
{"LibreDWGc::dwg_object_to_VIEWPORT", _wrap_dwg_object_to_VIEWPORT},
{"LibreDWGc::dwg_object_to_WIPEOUT", _wrap_dwg_object_to_WIPEOUT},
{"LibreDWGc::dwg_object_to_XLINE", _wrap_dwg_object_to_XLINE},
{"LibreDWGc::dwg_object_to_ARC_DIMENSION", _wrap_dwg_object_to_ARC_DIMENSION},
{"LibreDWGc::dwg_object_to_HELIX", _wrap_dwg_object_to_HELIX},
{"LibreDWGc::dwg_object_to_LARGE_RADIAL_DIMENSION", _wrap_dwg_object_to_LARGE_RADIAL_DIMENSION},
{"LibreDWGc::dwg_object_to_LAYOUTPRINTCONFIG", _wrap_dwg_object_to_LAYOUTPRINTCONFIG},
{"LibreDWGc::dwg_object_to_PLANESURFACE", _wrap_dwg_object_to_PLANESURFACE},
{"LibreDWGc::dwg_object_to_POINTCLOUD", _wrap_dwg_object_to_POINTCLOUD},
{"LibreDWGc::dwg_object_to_POINTCLOUDEX", _wrap_dwg_object_to_POINTCLOUDEX},
{"LibreDWGc::dwg_object_to_ACSH_BOOLEAN_CLASS", _wrap_dwg_object_to_ACSH_BOOLEAN_CLASS},
{"LibreDWGc::dwg_object_to_ACSH_BOX_CLASS", _wrap_dwg_object_to_ACSH_BOX_CLASS},
{"LibreDWGc::dwg_object_to_ACSH_CONE_CLASS", _wrap_dwg_object_to_ACSH_CONE_CLASS},
{"LibreDWGc::dwg_object_to_ACSH_CYLINDER_CLASS", _wrap_dwg_object_to_ACSH_CYLINDER_CLASS},
{"LibreDWGc::dwg_object_to_ACSH_FILLET_CLASS", _wrap_dwg_object_to_ACSH_FILLET_CLASS},
{"LibreDWGc::dwg_object_to_ACSH_HISTORY_CLASS", _wrap_dwg_object_to_ACSH_HISTORY_CLASS},
{"LibreDWGc::dwg_object_to_ACSH_SPHERE_CLASS", _wrap_dwg_object_to_ACSH_SPHERE_CLASS},
{"LibreDWGc::dwg_object_to_ACSH_TORUS_CLASS", _wrap_dwg_object_to_ACSH_TORUS_CLASS},
{"LibreDWGc::dwg_object_to_ACSH_WEDGE_CLASS", _wrap_dwg_object_to_ACSH_WEDGE_CLASS},
{"LibreDWGc::dwg_object_to_APPID", _wrap_dwg_object_to_APPID},
{"LibreDWGc::dwg_object_to_APPID_CONTROL", _wrap_dwg_object_to_APPID_CONTROL},
{"LibreDWGc::dwg_object_to_ASSOCGEOMDEPENDENCY", _wrap_dwg_object_to_ASSOCGEOMDEPENDENCY},
{"LibreDWGc::dwg_object_to_ASSOCNETWORK", _wrap_dwg_object_to_ASSOCNETWORK},
{"LibreDWGc::dwg_object_to_BLOCKALIGNMENTGRIP", _wrap_dwg_object_to_BLOCKALIGNMENTGRIP},
{"LibreDWGc::dwg_object_to_BLOCKALIGNMENTPARAMETER", _wrap_dwg_object_to_BLOCKALIGNMENTPARAMETER},
{"LibreDWGc::dwg_object_to_BLOCKBASEPOINTPARAMETER", _wrap_dwg_object_to_BLOCKBASEPOINTPARAMETER},
{"LibreDWGc::dwg_object_to_BLOCKFLIPACTION", _wrap_dwg_object_to_BLOCKFLIPACTION},
{"LibreDWGc::dwg_object_to_BLOCKFLIPGRIP", _wrap_dwg_object_to_BLOCKFLIPGRIP},
{"LibreDWGc::dwg_object_to_BLOCKFLIPPARAMETER", _wrap_dwg_object_to_BLOCKFLIPPARAMETER},
{"LibreDWGc::dwg_object_to_BLOCKGRIPLOCATIONCOMPONENT", _wrap_dwg_object_to_BLOCKGRIPLOCATIONCOMPONENT},
{"LibreDWGc::dwg_object_to_BLOCKLINEARGRIP", _wrap_dwg_object_to_BLOCKLINEARGRIP},
{"LibreDWGc::dwg_object_to_BLOCKLOOKUPGRIP", _wrap_dwg_object_to_BLOCKLOOKUPGRIP},
{"LibreDWGc::dwg_object_to_BLOCKMOVEACTION", _wrap_dwg_object_to_BLOCKMOVEACTION},
{"LibreDWGc::dwg_object_to_BLOCKROTATEACTION", _wrap_dwg_object_to_BLOCKROTATEACTION},
{"LibreDWGc::dwg_object_to_BLOCKROTATIONGRIP", _wrap_dwg_object_to_BLOCKROTATIONGRIP},
{"LibreDWGc::dwg_object_to_BLOCKSCALEACTION", _wrap_dwg_object_to_BLOCKSCALEACTION},
{"LibreDWGc::dwg_object_to_BLOCKVISIBILITYGRIP", _wrap_dwg_object_to_BLOCKVISIBILITYGRIP},
{"LibreDWGc::dwg_object_to_BLOCK_CONTROL", _wrap_dwg_object_to_BLOCK_CONTROL},
{"LibreDWGc::dwg_object_to_BLOCK_HEADER", _wrap_dwg_object_to_BLOCK_HEADER},
{"LibreDWGc::dwg_object_to_CELLSTYLEMAP", _wrap_dwg_object_to_CELLSTYLEMAP},
{"LibreDWGc::dwg_object_to_DETAILVIEWSTYLE", _wrap_dwg_object_to_DETAILVIEWSTYLE},
{"LibreDWGc::dwg_object_to_DICTIONARY", _wrap_dwg_object_to_DICTIONARY},
{"LibreDWGc::dwg_object_to_DICTIONARYVAR", _wrap_dwg_object_to_DICTIONARYVAR},
{"LibreDWGc::dwg_object_to_DICTIONARYWDFLT", _wrap_dwg_object_to_DICTIONARYWDFLT},
{"LibreDWGc::dwg_object_to_DIMSTYLE", _wrap_dwg_object_to_DIMSTYLE},
{"LibreDWGc::dwg_object_to_DIMSTYLE_CONTROL", _wrap_dwg_object_to_DIMSTYLE_CONTROL},
{"LibreDWGc::dwg_object_to_DUMMY", _wrap_dwg_object_to_DUMMY},
{"LibreDWGc::dwg_object_to_DYNAMICBLOCKPURGEPREVENTER", _wrap_dwg_object_to_DYNAMICBLOCKPURGEPREVENTER},
{"LibreDWGc::dwg_object_to_FIELD", _wrap_dwg_object_to_FIELD},
{"LibreDWGc::dwg_object_to_FIELDLIST", _wrap_dwg_object_to_FIELDLIST},
{"LibreDWGc::dwg_object_to_GEODATA", _wrap_dwg_object_to_GEODATA},
{"LibreDWGc::dwg_object_to_GROUP", _wrap_dwg_object_to_GROUP},
{"LibreDWGc::dwg_object_to_IDBUFFER", _wrap_dwg_object_to_IDBUFFER},
{"LibreDWGc::dwg_object_to_IMAGEDEF", _wrap_dwg_object_to_IMAGEDEF},
{"LibreDWGc::dwg_object_to_IMAGEDEF_REACTOR", _wrap_dwg_object_to_IMAGEDEF_REACTOR},
{"LibreDWGc::dwg_object_to_INDEX", _wrap_dwg_object_to_INDEX},
{"LibreDWGc::dwg_object_to_LAYER", _wrap_dwg_object_to_LAYER},
{"LibreDWGc::dwg_object_to_LAYERFILTER", _wrap_dwg_object_to_LAYERFILTER},
{"LibreDWGc::dwg_object_to_LAYER_CONTROL", _wrap_dwg_object_to_LAYER_CONTROL},
{"LibreDWGc::dwg_object_to_LAYER_INDEX", _wrap_dwg_object_to_LAYER_INDEX},
{"LibreDWGc::dwg_object_to_LAYOUT", _wrap_dwg_object_to_LAYOUT},
{"LibreDWGc::dwg_object_to_LONG_TRANSACTION", _wrap_dwg_object_to_LONG_TRANSACTION},
{"LibreDWGc::dwg_object_to_LTYPE", _wrap_dwg_object_to_LTYPE},
{"LibreDWGc::dwg_object_to_LTYPE_CONTROL", _wrap_dwg_object_to_LTYPE_CONTROL},
{"LibreDWGc::dwg_object_to_MLINESTYLE", _wrap_dwg_object_to_MLINESTYLE},
{"LibreDWGc::dwg_object_to_PLACEHOLDER", _wrap_dwg_object_to_PLACEHOLDER},
{"LibreDWGc::dwg_object_to_PLOTSETTINGS", _wrap_dwg_object_to_PLOTSETTINGS},
{"LibreDWGc::dwg_object_to_RASTERVARIABLES", _wrap_dwg_object_to_RASTERVARIABLES},
{"LibreDWGc::dwg_object_to_RENDERENVIRONMENT", _wrap_dwg_object_to_RENDERENVIRONMENT},
{"LibreDWGc::dwg_object_to_SCALE", _wrap_dwg_object_to_SCALE},
{"LibreDWGc::dwg_object_to_SECTIONVIEWSTYLE", _wrap_dwg_object_to_SECTIONVIEWSTYLE},
{"LibreDWGc::dwg_object_to_SECTION_MANAGER", _wrap_dwg_object_to_SECTION_MANAGER},
{"LibreDWGc::dwg_object_to_SORTENTSTABLE", _wrap_dwg_object_to_SORTENTSTABLE},
{"LibreDWGc::dwg_object_to_SPATIAL_FILTER", _wrap_dwg_object_to_SPATIAL_FILTER},
{"LibreDWGc::dwg_object_to_STYLE", _wrap_dwg_object_to_STYLE},
{"LibreDWGc::dwg_object_to_STYLE_CONTROL", _wrap_dwg_object_to_STYLE_CONTROL},
{"LibreDWGc::dwg_object_to_SUN", _wrap_dwg_object_to_SUN},
{"LibreDWGc::dwg_object_to_TABLEGEOMETRY", _wrap_dwg_object_to_TABLEGEOMETRY},
{"LibreDWGc::dwg_object_to_UCS", _wrap_dwg_object_to_UCS},
{"LibreDWGc::dwg_object_to_UCS_CONTROL", _wrap_dwg_object_to_UCS_CONTROL},
{"LibreDWGc::dwg_object_to_UNKNOWN_OBJ", _wrap_dwg_object_to_UNKNOWN_OBJ},
{"LibreDWGc::dwg_object_to_VBA_PROJECT", _wrap_dwg_object_to_VBA_PROJECT},
{"LibreDWGc::dwg_object_to_VIEW", _wrap_dwg_object_to_VIEW},
{"LibreDWGc::dwg_object_to_VIEW_CONTROL", _wrap_dwg_object_to_VIEW_CONTROL},
{"LibreDWGc::dwg_object_to_VISUALSTYLE", _wrap_dwg_object_to_VISUALSTYLE},
{"LibreDWGc::dwg_object_to_VPORT", _wrap_dwg_object_to_VPORT},
{"LibreDWGc::dwg_object_to_VPORT_CONTROL", _wrap_dwg_object_to_VPORT_CONTROL},
{"LibreDWGc::dwg_object_to_VX_CONTROL", _wrap_dwg_object_to_VX_CONTROL},
{"LibreDWGc::dwg_object_to_VX_TABLE_RECORD", _wrap_dwg_object_to_VX_TABLE_RECORD},
{"LibreDWGc::dwg_object_to_WIPEOUTVARIABLES", _wrap_dwg_object_to_WIPEOUTVARIABLES},
{"LibreDWGc::dwg_object_to_XRECORD", _wrap_dwg_object_to_XRECORD},
{"LibreDWGc::dwg_object_to_PDFDEFINITION", _wrap_dwg_object_to_PDFDEFINITION},
{"LibreDWGc::dwg_object_to_DGNDEFINITION", _wrap_dwg_object_to_DGNDEFINITION},
{"LibreDWGc::dwg_object_to_DWFDEFINITION", _wrap_dwg_object_to_DWFDEFINITION},
{"LibreDWGc::dwg_object_to_ACSH_BREP_CLASS", _wrap_dwg_object_to_ACSH_BREP_CLASS},
{"LibreDWGc::dwg_object_to_ACSH_CHAMFER_CLASS", _wrap_dwg_object_to_ACSH_CHAMFER_CLASS},
{"LibreDWGc::dwg_object_to_ACSH_PYRAMID_CLASS", _wrap_dwg_object_to_ACSH_PYRAMID_CLASS},
{"LibreDWGc::dwg_object_to_ALDIMOBJECTCONTEXTDATA", _wrap_dwg_object_to_ALDIMOBJECTCONTEXTDATA},
{"LibreDWGc::dwg_object_to_ASSOC2DCONSTRAINTGROUP", _wrap_dwg_object_to_ASSOC2DCONSTRAINTGROUP},
{"LibreDWGc::dwg_object_to_ASSOCACTION", _wrap_dwg_object_to_ASSOCACTION},
{"LibreDWGc::dwg_object_to_ASSOCACTIONPARAM", _wrap_dwg_object_to_ASSOCACTIONPARAM},
{"LibreDWGc::dwg_object_to_ASSOCARRAYACTIONBODY", _wrap_dwg_object_to_ASSOCARRAYACTIONBODY},
{"LibreDWGc::dwg_object_to_ASSOCASMBODYACTIONPARAM", _wrap_dwg_object_to_ASSOCASMBODYACTIONPARAM},
{"LibreDWGc::dwg_object_to_ASSOCBLENDSURFACEACTIONBODY", _wrap_dwg_object_to_ASSOCBLENDSURFACEACTIONBODY},
{"LibreDWGc::dwg_object_to_ASSOCCOMPOUNDACTIONPARAM", _wrap_dwg_object_to_ASSOCCOMPOUNDACTIONPARAM},
{"LibreDWGc::dwg_object_to_ASSOCDEPENDENCY", _wrap_dwg_object_to_ASSOCDEPENDENCY},
{"LibreDWGc::dwg_object_to_ASSOCDIMDEPENDENCYBODY", _wrap_dwg_object_to_ASSOCDIMDEPENDENCYBODY},
{"LibreDWGc::dwg_object_to_ASSOCEXTENDSURFACEACTIONBODY", _wrap_dwg_object_to_ASSOCEXTENDSURFACEACTIONBODY},
{"LibreDWGc::dwg_object_to_ASSOCEXTRUDEDSURFACEACTIONBODY", _wrap_dwg_object_to_ASSOCEXTRUDEDSURFACEACTIONBODY},
{"LibreDWGc::dwg_object_to_ASSOCFACEACTIONPARAM", _wrap_dwg_object_to_ASSOCFACEACTIONPARAM},
{"LibreDWGc::dwg_object_to_ASSOCFILLETSURFACEACTIONBODY", _wrap_dwg_object_to_ASSOCFILLETSURFACEACTIONBODY},
{"LibreDWGc::dwg_object_to_ASSOCLOFTEDSURFACEACTIONBODY", _wrap_dwg_object_to_ASSOCLOFTEDSURFACEACTIONBODY},
{"LibreDWGc::dwg_object_to_ASSOCNETWORKSURFACEACTIONBODY", _wrap_dwg_object_to_ASSOCNETWORKSURFACEACTIONBODY},
{"LibreDWGc::dwg_object_to_ASSOCOBJECTACTIONPARAM", _wrap_dwg_object_to_ASSOCOBJECTACTIONPARAM},
{"LibreDWGc::dwg_object_to_ASSOCOFFSETSURFACEACTIONBODY", _wrap_dwg_object_to_ASSOCOFFSETSURFACEACTIONBODY},
{"LibreDWGc::dwg_object_to_ASSOCOSNAPPOINTREFACTIONPARAM", _wrap_dwg_object_to_ASSOCOSNAPPOINTREFACTIONPARAM},
{"LibreDWGc::dwg_object_to_ASSOCPATCHSURFACEACTIONBODY", _wrap_dwg_object_to_ASSOCPATCHSURFACEACTIONBODY},
{"LibreDWGc::dwg_object_to_ASSOCPATHACTIONPARAM", _wrap_dwg_object_to_ASSOCPATHACTIONPARAM},
{"LibreDWGc::dwg_object_to_ASSOCPLANESURFACEACTIONBODY", _wrap_dwg_object_to_ASSOCPLANESURFACEACTIONBODY},
{"LibreDWGc::dwg_object_to_ASSOCPOINTREFACTIONPARAM", _wrap_dwg_object_to_ASSOCPOINTREFACTIONPARAM},
{"LibreDWGc::dwg_object_to_ASSOCREVOLVEDSURFACEACTIONBODY", _wrap_dwg_object_to_ASSOCREVOLVEDSURFACEACTIONBODY},
{"LibreDWGc::dwg_object_to_ASSOCTRIMSURFACEACTIONBODY", _wrap_dwg_object_to_ASSOCTRIMSURFACEACTIONBODY},
{"LibreDWGc::dwg_object_to_ASSOCVALUEDEPENDENCY", _wrap_dwg_object_to_ASSOCVALUEDEPENDENCY},
{"LibreDWGc::dwg_object_to_ASSOCVARIABLE", _wrap_dwg_object_to_ASSOCVARIABLE},
{"LibreDWGc::dwg_object_to_ASSOCVERTEXACTIONPARAM", _wrap_dwg_object_to_ASSOCVERTEXACTIONPARAM},
{"LibreDWGc::dwg_object_to_BLKREFOBJECTCONTEXTDATA", _wrap_dwg_object_to_BLKREFOBJECTCONTEXTDATA},
{"LibreDWGc::dwg_object_to_BLOCKALIGNEDCONSTRAINTPARAMETER", _wrap_dwg_object_to_BLOCKALIGNEDCONSTRAINTPARAMETER},
{"LibreDWGc::dwg_object_to_BLOCKANGULARCONSTRAINTPARAMETER", _wrap_dwg_object_to_BLOCKANGULARCONSTRAINTPARAMETER},
{"LibreDWGc::dwg_object_to_BLOCKARRAYACTION", _wrap_dwg_object_to_BLOCKARRAYACTION},
{"LibreDWGc::dwg_object_to_BLOCKDIAMETRICCONSTRAINTPARAMETER", _wrap_dwg_object_to_BLOCKDIAMETRICCONSTRAINTPARAMETER},
{"LibreDWGc::dwg_object_to_BLOCKHORIZONTALCONSTRAINTPARAMETER", _wrap_dwg_object_to_BLOCKHORIZONTALCONSTRAINTPARAMETER},
{"LibreDWGc::dwg_object_to_BLOCKLINEARCONSTRAINTPARAMETER", _wrap_dwg_object_to_BLOCKLINEARCONSTRAINTPARAMETER},
{"LibreDWGc::dwg_object_to_BLOCKLINEARPARAMETER", _wrap_dwg_object_to_BLOCKLINEARPARAMETER},
{"LibreDWGc::dwg_object_to_BLOCKLOOKUPACTION", _wrap_dwg_object_to_BLOCKLOOKUPACTION},
{"LibreDWGc::dwg_object_to_BLOCKLOOKUPPARAMETER", _wrap_dwg_object_to_BLOCKLOOKUPPARAMETER},
{"LibreDWGc::dwg_object_to_BLOCKPARAMDEPENDENCYBODY", _wrap_dwg_object_to_BLOCKPARAMDEPENDENCYBODY},
{"LibreDWGc::dwg_object_to_BLOCKPOINTPARAMETER", _wrap_dwg_object_to_BLOCKPOINTPARAMETER},
{"LibreDWGc::dwg_object_to_BLOCKPOLARGRIP", _wrap_dwg_object_to_BLOCKPOLARGRIP},
{"LibreDWGc::dwg_object_to_BLOCKPOLARPARAMETER", _wrap_dwg_object_to_BLOCKPOLARPARAMETER},
{"LibreDWGc::dwg_object_to_BLOCKPOLARSTRETCHACTION", _wrap_dwg_object_to_BLOCKPOLARSTRETCHACTION},
{"LibreDWGc::dwg_object_to_BLOCKRADIALCONSTRAINTPARAMETER", _wrap_dwg_object_to_BLOCKRADIALCONSTRAINTPARAMETER},
{"LibreDWGc::dwg_object_to_BLOCKREPRESENTATION", _wrap_dwg_object_to_BLOCKREPRESENTATION},
{"LibreDWGc::dwg_object_to_BLOCKROTATIONPARAMETER", _wrap_dwg_object_to_BLOCKROTATIONPARAMETER},
{"LibreDWGc::dwg_object_to_BLOCKSTRETCHACTION", _wrap_dwg_object_to_BLOCKSTRETCHACTION},
{"LibreDWGc::dwg_object_to_BLOCKUSERPARAMETER", _wrap_dwg_object_to_BLOCKUSERPARAMETER},
{"LibreDWGc::dwg_object_to_BLOCKVERTICALCONSTRAINTPARAMETER", _wrap_dwg_object_to_BLOCKVERTICALCONSTRAINTPARAMETER},
{"LibreDWGc::dwg_object_to_BLOCKVISIBILITYPARAMETER", _wrap_dwg_object_to_BLOCKVISIBILITYPARAMETER},
{"LibreDWGc::dwg_object_to_BLOCKXYGRIP", _wrap_dwg_object_to_BLOCKXYGRIP},
{"LibreDWGc::dwg_object_to_BLOCKXYPARAMETER", _wrap_dwg_object_to_BLOCKXYPARAMETER},
{"LibreDWGc::dwg_object_to_DATALINK", _wrap_dwg_object_to_DATALINK},
{"LibreDWGc::dwg_object_to_DBCOLOR", _wrap_dwg_object_to_DBCOLOR},
{"LibreDWGc::dwg_object_to_EVALUATION_GRAPH", _wrap_dwg_object_to_EVALUATION_GRAPH},
{"LibreDWGc::dwg_object_to_FCFOBJECTCONTEXTDATA", _wrap_dwg_object_to_FCFOBJECTCONTEXTDATA},
{"LibreDWGc::dwg_object_to_GRADIENT_BACKGROUND", _wrap_dwg_object_to_GRADIENT_BACKGROUND},
{"LibreDWGc::dwg_object_to_GROUND_PLANE_BACKGROUND", _wrap_dwg_object_to_GROUND_PLANE_BACKGROUND},
{"LibreDWGc::dwg_object_to_IBL_BACKGROUND", _wrap_dwg_object_to_IBL_BACKGROUND},
{"LibreDWGc::dwg_object_to_IMAGE_BACKGROUND", _wrap_dwg_object_to_IMAGE_BACKGROUND},
{"LibreDWGc::dwg_object_to_LEADEROBJECTCONTEXTDATA", _wrap_dwg_object_to_LEADEROBJECTCONTEXTDATA},
{"LibreDWGc::dwg_object_to_LIGHTLIST", _wrap_dwg_object_to_LIGHTLIST},
{"LibreDWGc::dwg_object_to_MATERIAL", _wrap_dwg_object_to_MATERIAL},
{"LibreDWGc::dwg_object_to_MENTALRAYRENDERSETTINGS", _wrap_dwg_object_to_MENTALRAYRENDERSETTINGS},
{"LibreDWGc::dwg_object_to_MLEADERSTYLE", _wrap_dwg_object_to_MLEADERSTYLE},
{"LibreDWGc::dwg_object_to_MTEXTOBJECTCONTEXTDATA", _wrap_dwg_object_to_MTEXTOBJECTCONTEXTDATA},
{"LibreDWGc::dwg_object_to_OBJECT_PTR", _wrap_dwg_object_to_OBJECT_PTR},
{"LibreDWGc::dwg_object_to_PARTIAL_VIEWING_INDEX", _wrap_dwg_object_to_PARTIAL_VIEWING_INDEX},
{"LibreDWGc::dwg_object_to_POINTCLOUDCOLORMAP", _wrap_dwg_object_to_POINTCLOUDCOLORMAP},
{"LibreDWGc::dwg_object_to_POINTCLOUDDEF", _wrap_dwg_object_to_POINTCLOUDDEF},
{"LibreDWGc::dwg_object_to_POINTCLOUDDEFEX", _wrap_dwg_object_to_POINTCLOUDDEFEX},
{"LibreDWGc::dwg_object_to_POINTCLOUDDEF_REACTOR", _wrap_dwg_object_to_POINTCLOUDDEF_REACTOR},
{"LibreDWGc::dwg_object_to_POINTCLOUDDEF_REACTOR_EX", _wrap_dwg_object_to_POINTCLOUDDEF_REACTOR_EX},
{"LibreDWGc::dwg_object_to_PROXY_OBJECT", _wrap_dwg_object_to_PROXY_OBJECT},
{"LibreDWGc::dwg_object_to_RAPIDRTRENDERSETTINGS", _wrap_dwg_object_to_RAPIDRTRENDERSETTINGS},
{"LibreDWGc::dwg_object_to_RENDERENTRY", _wrap_dwg_object_to_RENDERENTRY},
{"LibreDWGc::dwg_object_to_RENDERGLOBAL", _wrap_dwg_object_to_RENDERGLOBAL},
{"LibreDWGc::dwg_object_to_RENDERSETTINGS", _wrap_dwg_object_to_RENDERSETTINGS},
{"LibreDWGc::dwg_object_to_SECTION_SETTINGS", _wrap_dwg_object_to_SECTION_SETTINGS},
{"LibreDWGc::dwg_object_to_SKYLIGHT_BACKGROUND", _wrap_dwg_object_to_SKYLIGHT_BACKGROUND},
{"LibreDWGc::dwg_object_to_SOLID_BACKGROUND", _wrap_dwg_object_to_SOLID_BACKGROUND},
{"LibreDWGc::dwg_object_to_SPATIAL_INDEX", _wrap_dwg_object_to_SPATIAL_INDEX},
{"LibreDWGc::dwg_object_to_TABLESTYLE", _wrap_dwg_object_to_TABLESTYLE},
{"LibreDWGc::dwg_object_to_TEXTOBJECTCONTEXTDATA", _wrap_dwg_object_to_TEXTOBJECTCONTEXTDATA},
{"LibreDWGc::dwg_object_to_ASSOCARRAYMODIFYPARAMETERS", _wrap_dwg_object_to_ASSOCARRAYMODIFYPARAMETERS},
{"LibreDWGc::dwg_object_to_ASSOCARRAYPATHPARAMETERS", _wrap_dwg_object_to_ASSOCARRAYPATHPARAMETERS},
{"LibreDWGc::dwg_object_to_ASSOCARRAYPOLARPARAMETERS", _wrap_dwg_object_to_ASSOCARRAYPOLARPARAMETERS},
{"LibreDWGc::dwg_object_to_ASSOCARRAYRECTANGULARPARAMETERS", _wrap_dwg_object_to_ASSOCARRAYRECTANGULARPARAMETERS},
{0,0}
};
/* -----------------------------------------------------------------------------
 * Type initialization:
 * This problem is tough by the requirement that no dynamic
 * memory is used. Also, since swig_type_info structures store pointers to
 * swig_cast_info structures and swig_cast_info structures store pointers back
 * to swig_type_info structures, we need some lookup code at initialization.
 * The idea is that swig generates all the structures that are needed.
 * The runtime then collects these partially filled structures.
 * The SWIG_InitializeModule function takes these initial arrays out of
 * swig_module, and does all the lookup, filling in the swig_module.types
 * array with the correct data and linking the correct swig_cast_info
 * structures together.
 *
 * The generated swig_type_info structures are assigned statically to an initial
 * array. We just loop through that array, and handle each type individually.
 * First we lookup if this type has been already loaded, and if so, use the
 * loaded structure instead of the generated one. Then we have to fill in the
 * cast linked list. The cast data is initially stored in something like a
 * two-dimensional array. Each row corresponds to a type (there are the same
 * number of rows as there are in the swig_type_initial array). Each entry in
 * a column is one of the swig_cast_info structures for that type.
 * The cast_initial array is actually an array of arrays, because each row has
 * a variable number of columns. So to actually build the cast linked list,
 * we find the array of casts associated with the type, and loop through it
 * adding the casts to the list. The one last trick we need to do is making
 * sure the type pointer in the swig_cast_info struct is correct.
 *
 * First off, we lookup the cast->type name to see if it is already loaded.
 * There are three cases to handle:
 *  1) If the cast->type has already been loaded AND the type we are adding
 *     casting info to has not been loaded (it is in this module), THEN we
 *     replace the cast->type pointer with the type pointer that has already
 *     been loaded.
 *  2) If BOTH types (the one we are adding casting info to, and the
 *     cast->type) are loaded, THEN the cast info has already been loaded by
 *     the previous module so we just ignore it.
 *  3) Finally, if cast->type has not already been loaded, then we add that
 *     swig_cast_info to the linked list (because the cast->type) pointer will
 *     be correct.
 * ----------------------------------------------------------------------------- */

#ifdef __cplusplus
extern "C" {
#if 0
} /* c-mode */
#endif
#endif

#if 0
#define SWIGRUNTIME_DEBUG
#endif

#ifndef SWIG_INIT_CLIENT_DATA_TYPE
#define SWIG_INIT_CLIENT_DATA_TYPE void *
#endif

SWIGRUNTIME void
SWIG_InitializeModule(SWIG_INIT_CLIENT_DATA_TYPE clientdata) {
  size_t i;
  swig_module_info *module_head, *iter;
  int init;
  
  /* check to see if the circular list has been setup, if not, set it up */
  if (swig_module.next==0) {
    /* Initialize the swig_module */
    swig_module.type_initial = swig_type_initial;
    swig_module.cast_initial = swig_cast_initial;
    swig_module.next = &swig_module;
    init = 1;
  } else {
    init = 0;
  }
  
  /* Try and load any already created modules */
  module_head = SWIG_GetModule(clientdata);
  if (!module_head) {
    /* This is the first module loaded for this interpreter */
    /* so set the swig module into the interpreter */
    SWIG_SetModule(clientdata, &swig_module);
  } else {
    /* the interpreter has loaded a SWIG module, but has it loaded this one? */
    iter=module_head;
    do {
      if (iter==&swig_module) {
        /* Our module is already in the list, so there's nothing more to do. */
        return;
      }
      iter=iter->next;
    } while (iter!= module_head);
    
    /* otherwise we must add our module into the list */
    swig_module.next = module_head->next;
    module_head->next = &swig_module;
  }
  
  /* When multiple interpreters are used, a module could have already been initialized in
       a different interpreter, but not yet have a pointer in this interpreter.
       In this case, we do not want to continue adding types... everything should be
       set up already */
  if (init == 0) return;
  
  /* Now work on filling in swig_module.types */
#ifdef SWIGRUNTIME_DEBUG
  printf("SWIG_InitializeModule: size %lu\n", (unsigned long)swig_module.size);
#endif
  for (i = 0; i < swig_module.size; ++i) {
    swig_type_info *type = 0;
    swig_type_info *ret;
    swig_cast_info *cast;
    
#ifdef SWIGRUNTIME_DEBUG
    printf("SWIG_InitializeModule: type %lu %s\n", (unsigned long)i, swig_module.type_initial[i]->name);
#endif
    
    /* if there is another module already loaded */
    if (swig_module.next != &swig_module) {
      type = SWIG_MangledTypeQueryModule(swig_module.next, &swig_module, swig_module.type_initial[i]->name);
    }
    if (type) {
      /* Overwrite clientdata field */
#ifdef SWIGRUNTIME_DEBUG
      printf("SWIG_InitializeModule: found type %s\n", type->name);
#endif
      if (swig_module.type_initial[i]->clientdata) {
        type->clientdata = swig_module.type_initial[i]->clientdata;
#ifdef SWIGRUNTIME_DEBUG
        printf("SWIG_InitializeModule: found and overwrite type %s \n", type->name);
#endif
      }
    } else {
      type = swig_module.type_initial[i];
    }
    
    /* Insert casting types */
    cast = swig_module.cast_initial[i];
    while (cast->type) {
      /* Don't need to add information already in the list */
      ret = 0;
#ifdef SWIGRUNTIME_DEBUG
      printf("SWIG_InitializeModule: look cast %s\n", cast->type->name);
#endif
      if (swig_module.next != &swig_module) {
        ret = SWIG_MangledTypeQueryModule(swig_module.next, &swig_module, cast->type->name);
#ifdef SWIGRUNTIME_DEBUG
        if (ret) printf("SWIG_InitializeModule: found cast %s\n", ret->name);
#endif
      }
      if (ret) {
        if (type == swig_module.type_initial[i]) {
#ifdef SWIGRUNTIME_DEBUG
          printf("SWIG_InitializeModule: skip old type %s\n", ret->name);
#endif
          cast->type = ret;
          ret = 0;
        } else {
          /* Check for casting already in the list */
          swig_cast_info *ocast = SWIG_TypeCheck(ret->name, type);
#ifdef SWIGRUNTIME_DEBUG
          if (ocast) printf("SWIG_InitializeModule: skip old cast %s\n", ret->name);
#endif
          if (!ocast) ret = 0;
        }
      }
      
      if (!ret) {
#ifdef SWIGRUNTIME_DEBUG
        printf("SWIG_InitializeModule: adding cast %s\n", cast->type->name);
#endif
        if (type->cast) {
          type->cast->prev = cast;
          cast->next = type->cast;
        }
        type->cast = cast;
      }
      cast++;
    }
    /* Set entry in modules->types array equal to the type */
    swig_module.types[i] = type;
  }
  swig_module.types[i] = 0;
  
#ifdef SWIGRUNTIME_DEBUG
  printf("**** SWIG_InitializeModule: Cast List ******\n");
  for (i = 0; i < swig_module.size; ++i) {
    int j = 0;
    swig_cast_info *cast = swig_module.cast_initial[i];
    printf("SWIG_InitializeModule: type %lu %s\n", (unsigned long)i, swig_module.type_initial[i]->name);
    while (cast->type) {
      printf("SWIG_InitializeModule: cast type %s\n", cast->type->name);
      cast++;
      ++j;
    }
    printf("---- Total casts: %d\n",j);
  }
  printf("**** SWIG_InitializeModule: Cast List ******\n");
#endif
}

/* This function will propagate the clientdata field of type to
* any new swig_type_info structures that have been added into the list
* of equivalent types.  It is like calling
* SWIG_TypeClientData(type, clientdata) a second time.
*/
SWIGRUNTIME void
SWIG_PropagateClientData(void) {
  size_t i;
  swig_cast_info *equiv;
  static int init_run = 0;
  
  if (init_run) return;
  init_run = 1;
  
  for (i = 0; i < swig_module.size; i++) {
    if (swig_module.types[i]->clientdata) {
      equiv = swig_module.types[i]->cast;
      while (equiv) {
        if (!equiv->converter) {
          if (equiv->type && !equiv->type->clientdata)
          SWIG_TypeClientData(equiv->type, swig_module.types[i]->clientdata);
        }
        equiv = equiv->next;
      }
    }
  }
}

#ifdef __cplusplus
#if 0
{
  /* c-mode */
#endif
}
#endif



#if defined(__cplusplus) && ! defined(XSPROTO)
extern "C"
#endif

XS(SWIG_init) {
  dXSARGS;
  int i;
  (void)items;
  
  SWIG_InitializeModule(0);
  
  /* Install commands */
  for (i = 0; swig_commands[i].name; i++) {
    /* Casts only needed for Perl < 5.10. */
#ifdef __cplusplus
    newXS(const_cast<char*>(swig_commands[i].name), swig_commands[i].wrapper, const_cast<char*>(__FILE__));
#else
    newXS((char*)swig_commands[i].name, swig_commands[i].wrapper, (char*)__FILE__);
#endif
  }
  
  /* Install variables */
  for (i = 0; swig_variables[i].name; i++) {
    SV *sv;
    sv = get_sv(swig_variables[i].name, TRUE | 0x2 | GV_ADDMULTI);
    if (swig_variables[i].type) {
      SWIG_MakePtr(sv,(void *)1, *swig_variables[i].type,0);
    } else {
      sv_setiv(sv,(IV) 0);
    }
    swig_create_magic(sv, swig_variables[i].name, swig_variables[i].set, swig_variables[i].get); 
  }
  
  /* Install constant */
  for (i = 0; swig_constants[i].type; i++) {
    SV *sv;
    sv = get_sv(swig_constants[i].name, TRUE | 0x2 | GV_ADDMULTI);
    switch(swig_constants[i].type) {
    case SWIG_INT:
      sv_setiv(sv, (IV) swig_constants[i].lvalue);
      break;
    case SWIG_FLOAT:
      sv_setnv(sv, (double) swig_constants[i].dvalue);
      break;
    case SWIG_STRING:
      sv_setpv(sv, (const char *) swig_constants[i].pvalue);
      break;
    case SWIG_POINTER:
      SWIG_MakePtr(sv, swig_constants[i].pvalue, *(swig_constants[i].ptype),0);
      break;
    case SWIG_BINARY:
      SWIG_MakePackedObj(sv, swig_constants[i].pvalue, swig_constants[i].lvalue, *(swig_constants[i].ptype));
      break;
    default:
      break;
    }
    SvREADONLY_on(sv);
  }
  
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "LIBREDWG_VERSION_MAJOR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(0)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "LIBREDWG_VERSION_MINOR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(13)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "LIBREDWG_VERSION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(((0*100)+13))));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FORMAT_RC", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_FromCharPtr("0x%hhx"));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SCANF_2X", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_FromCharPtr("%2hhX"));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FORMAT_RCd", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_FromCharPtr("%d"));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FORMAT_RCu", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_FromCharPtr("%u"));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FORMAT_RCx", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_FromCharPtr("0x%x"));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FORMAT_B", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_FromCharPtr("%d"));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FORMAT_BB", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_FromCharPtr("%u"));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FORMAT_3B", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_FromCharPtr("%u"));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FORMAT_RD", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_FromCharPtr("%g"));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FORMAT_TF", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_FromCharPtr("\"%s\""));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FORMAT_TV", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_FromCharPtr("\"%s\""));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FORMAT_T16", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_FromCharPtr("\"%s\""));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FORMAT_T32", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_FromCharPtr("\"%s\""));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FORMAT_TU32", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_FromCharPtr("\"%s\""));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FORMAT_BT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_FromCharPtr("%g"));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FORMAT_DD", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_FromCharPtr("%g"));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FORMAT_BD", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_FromCharPtr("%g"));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FORMAT_4BITS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_FromCharPtr("%1x"));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FORMAT_D2T", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_FromCharPtr("%s"));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FORMAT_TU", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_FromCharPtr("\"%hn\""));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  SWIG_TypeClientData(SWIGTYPE_p__dwg_time_bll, (void*) "LibreDWG::Dwg_Bitcode_TimeBLL");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_bitcode_2rd, (void*) "LibreDWG::Dwg_Bitcode_2RD");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_bitcode_2bd, (void*) "LibreDWG::Dwg_Bitcode_2BD");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_bitcode_3rd, (void*) "LibreDWG::Dwg_Bitcode_3RD");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_bitcode_3bd, (void*) "LibreDWG::Dwg_Bitcode_3BD");
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_INVALID", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_INVALID)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_1_1", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_1_1)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_1_2", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_1_2)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_1_3", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_1_3)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_1_4", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_1_4)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_2_0b", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_2_0b)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_2_0", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_2_0)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_2_10", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_2_10)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_2_21", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_2_21)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_2_22", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_2_22)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_2_4", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_2_4)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_2_5", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_2_5)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_2_6", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_2_6)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_9", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_9)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_9c1", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_9c1)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_10", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_10)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_11b1", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_11b1)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_11b2", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_11b2)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_11", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_11)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_12", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_12)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_13b1", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_13b1)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_13b2", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_13b2)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_13", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_13)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_13c3", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_13c3)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_14", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_14)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_2000b", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_2000b)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_2000", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_2000)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_2000i", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_2000i)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_2002", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_2002)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_2004a", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_2004a)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_2004b", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_2004b)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_2004c", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_2004c)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_2004", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_2004)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_2007a", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_2007a)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_2007b", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_2007b)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_2007", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_2007)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_2010b", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_2010b)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_2010", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_2010)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_2013b", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_2013b)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_2013", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_2013)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_2018b", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_2018b)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_2018", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_2018)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_2022b", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_2022b)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "R_AFTER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(R_AFTER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  SWIG_TypeClientData(SWIGTYPE_p_dwg_versions, (void*) "LibreDWG::Dwg_Versions");
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_CLASS_STABLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_CLASS_STABLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_CLASS_UNSTABLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_CLASS_UNSTABLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_CLASS_DEBUGGING", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_CLASS_DEBUGGING)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_CLASS_UNHANDLED", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_CLASS_UNHANDLED)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_ENTITY_SECTION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_ENTITY_SECTION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_BLOCKS_SECTION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_BLOCKS_SECTION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_EXTRA_SECTION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_EXTRA_SECTION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_SUPERTYPE_ENTITY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_SUPERTYPE_ENTITY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_SUPERTYPE_OBJECT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_SUPERTYPE_OBJECT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_UNUSED", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_UNUSED)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_TEXT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_TEXT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ATTRIB", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ATTRIB)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ATTDEF", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ATTDEF)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCK", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCK)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ENDBLK", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ENDBLK)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_SEQEND", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_SEQEND)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_INSERT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_INSERT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_MINSERT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_MINSERT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_VERTEX_2D", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_VERTEX_2D)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_VERTEX_3D", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_VERTEX_3D)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_VERTEX_MESH", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_VERTEX_MESH)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_VERTEX_PFACE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_VERTEX_PFACE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_VERTEX_PFACE_FACE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_VERTEX_PFACE_FACE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_POLYLINE_2D", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_POLYLINE_2D)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_POLYLINE_3D", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_POLYLINE_3D)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ARC", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ARC)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_CIRCLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_CIRCLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_LINE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_LINE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DIMENSION_ORDINATE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DIMENSION_ORDINATE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DIMENSION_LINEAR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DIMENSION_LINEAR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DIMENSION_ALIGNED", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DIMENSION_ALIGNED)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DIMENSION_ANG3PT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DIMENSION_ANG3PT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DIMENSION_ANG2LN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DIMENSION_ANG2LN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DIMENSION_RADIUS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DIMENSION_RADIUS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DIMENSION_DIAMETER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DIMENSION_DIAMETER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_POINT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_POINT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE__3DFACE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE__3DFACE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_POLYLINE_PFACE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_POLYLINE_PFACE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_POLYLINE_MESH", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_POLYLINE_MESH)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_SOLID", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_SOLID)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_TRACE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_TRACE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_SHAPE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_SHAPE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_VIEWPORT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_VIEWPORT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ELLIPSE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ELLIPSE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_SPLINE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_SPLINE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_REGION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_REGION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE__3DSOLID", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE__3DSOLID)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BODY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BODY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_RAY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_RAY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_XLINE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_XLINE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DICTIONARY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DICTIONARY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_OLEFRAME", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_OLEFRAME)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_MTEXT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_MTEXT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_LEADER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_LEADER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_TOLERANCE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_TOLERANCE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_MLINE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_MLINE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCK_CONTROL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCK_CONTROL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCK_HEADER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCK_HEADER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_LAYER_CONTROL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_LAYER_CONTROL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_LAYER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_LAYER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_STYLE_CONTROL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_STYLE_CONTROL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_LTYPE_CONTROL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_LTYPE_CONTROL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_LTYPE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_LTYPE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_VIEW_CONTROL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_VIEW_CONTROL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_VIEW", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_VIEW)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_UCS_CONTROL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_UCS_CONTROL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_UCS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_UCS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_VPORT_CONTROL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_VPORT_CONTROL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_VPORT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_VPORT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_APPID_CONTROL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_APPID_CONTROL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_APPID", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_APPID)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DIMSTYLE_CONTROL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DIMSTYLE_CONTROL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DIMSTYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DIMSTYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_VX_CONTROL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_VX_CONTROL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_VX_TABLE_RECORD", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_VX_TABLE_RECORD)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_GROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_GROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_MLINESTYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_MLINESTYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_OLE2FRAME", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_OLE2FRAME)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DUMMY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DUMMY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_LONG_TRANSACTION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_LONG_TRANSACTION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_LWPOLYLINE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_LWPOLYLINE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_HATCH", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_HATCH)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_XRECORD", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_XRECORD)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_PLACEHOLDER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_PLACEHOLDER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_VBA_PROJECT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_VBA_PROJECT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_LAYOUT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_LAYOUT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_PROXY_ENTITY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_PROXY_ENTITY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_PROXY_OBJECT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_PROXY_OBJECT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACDSRECORD", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACDSRECORD)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACDSSCHEMA", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACDSSCHEMA)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMECOMMANDHISTORY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMECOMMANDHISTORY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMESCOPE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMESCOPE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMESTATEMGR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMESTATEMGR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACSH_BOOLEAN_CLASS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACSH_BOOLEAN_CLASS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACSH_BOX_CLASS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACSH_BOX_CLASS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACSH_BREP_CLASS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACSH_BREP_CLASS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACSH_CHAMFER_CLASS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACSH_CHAMFER_CLASS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACSH_CONE_CLASS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACSH_CONE_CLASS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACSH_CYLINDER_CLASS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACSH_CYLINDER_CLASS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACSH_EXTRUSION_CLASS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACSH_EXTRUSION_CLASS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACSH_FILLET_CLASS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACSH_FILLET_CLASS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACSH_HISTORY_CLASS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACSH_HISTORY_CLASS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACSH_LOFT_CLASS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACSH_LOFT_CLASS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACSH_PYRAMID_CLASS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACSH_PYRAMID_CLASS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACSH_REVOLVE_CLASS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACSH_REVOLVE_CLASS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACSH_SPHERE_CLASS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACSH_SPHERE_CLASS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACSH_SWEEP_CLASS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACSH_SWEEP_CLASS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACSH_TORUS_CLASS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACSH_TORUS_CLASS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACSH_WEDGE_CLASS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACSH_WEDGE_CLASS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ALDIMOBJECTCONTEXTDATA", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ALDIMOBJECTCONTEXTDATA)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ALIGNMENTPARAMETERENTITY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ALIGNMENTPARAMETERENTITY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ANGDIMOBJECTCONTEXTDATA", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ANGDIMOBJECTCONTEXTDATA)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ANNOTSCALEOBJECTCONTEXTDATA", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ANNOTSCALEOBJECTCONTEXTDATA)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ARCALIGNEDTEXT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ARCALIGNEDTEXT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ARC_DIMENSION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ARC_DIMENSION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOC2DCONSTRAINTGROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOC2DCONSTRAINTGROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOC3POINTANGULARDIMACTIONBODY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOC3POINTANGULARDIMACTIONBODY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCACTION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCACTION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCACTIONPARAM", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCACTIONPARAM)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCALIGNEDDIMACTIONBODY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCALIGNEDDIMACTIONBODY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCARRAYACTIONBODY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCARRAYACTIONBODY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCARRAYMODIFYACTIONBODY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCARRAYMODIFYACTIONBODY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCARRAYMODIFYPARAMETERS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCARRAYMODIFYPARAMETERS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCARRAYPATHPARAMETERS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCARRAYPATHPARAMETERS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCARRAYPOLARPARAMETERS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCARRAYPOLARPARAMETERS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCARRAYRECTANGULARPARAMETERS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCARRAYRECTANGULARPARAMETERS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCASMBODYACTIONPARAM", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCASMBODYACTIONPARAM)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCBLENDSURFACEACTIONBODY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCBLENDSURFACEACTIONBODY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCCOMPOUNDACTIONPARAM", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCCOMPOUNDACTIONPARAM)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCDEPENDENCY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCDEPENDENCY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCDIMDEPENDENCYBODY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCDIMDEPENDENCYBODY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCEDGEACTIONPARAM", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCEDGEACTIONPARAM)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCEDGECHAMFERACTIONBODY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCEDGECHAMFERACTIONBODY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCEDGEFILLETACTIONBODY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCEDGEFILLETACTIONBODY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCEXTENDSURFACEACTIONBODY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCEXTENDSURFACEACTIONBODY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCEXTRUDEDSURFACEACTIONBODY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCEXTRUDEDSURFACEACTIONBODY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCFACEACTIONPARAM", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCFACEACTIONPARAM)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCFILLETSURFACEACTIONBODY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCFILLETSURFACEACTIONBODY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCGEOMDEPENDENCY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCGEOMDEPENDENCY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCLOFTEDSURFACEACTIONBODY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCLOFTEDSURFACEACTIONBODY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCMLEADERACTIONBODY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCMLEADERACTIONBODY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCNETWORK", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCNETWORK)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCNETWORKSURFACEACTIONBODY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCNETWORKSURFACEACTIONBODY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCOBJECTACTIONPARAM", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCOBJECTACTIONPARAM)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCOFFSETSURFACEACTIONBODY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCOFFSETSURFACEACTIONBODY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCORDINATEDIMACTIONBODY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCORDINATEDIMACTIONBODY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCOSNAPPOINTREFACTIONPARAM", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCOSNAPPOINTREFACTIONPARAM)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCPATCHSURFACEACTIONBODY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCPATCHSURFACEACTIONBODY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCPATHACTIONPARAM", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCPATHACTIONPARAM)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCPERSSUBENTMANAGER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCPERSSUBENTMANAGER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCPLANESURFACEACTIONBODY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCPLANESURFACEACTIONBODY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCPOINTREFACTIONPARAM", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCPOINTREFACTIONPARAM)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCRESTOREENTITYSTATEACTIONBODY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCRESTOREENTITYSTATEACTIONBODY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCREVOLVEDSURFACEACTIONBODY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCREVOLVEDSURFACEACTIONBODY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCROTATEDDIMACTIONBODY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCROTATEDDIMACTIONBODY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCSWEPTSURFACEACTIONBODY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCSWEPTSURFACEACTIONBODY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCTRIMSURFACEACTIONBODY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCTRIMSURFACEACTIONBODY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCVALUEDEPENDENCY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCVALUEDEPENDENCY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCVARIABLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCVARIABLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ASSOCVERTEXACTIONPARAM", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ASSOCVERTEXACTIONPARAM)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BASEPOINTPARAMETERENTITY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BASEPOINTPARAMETERENTITY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLKREFOBJECTCONTEXTDATA", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLKREFOBJECTCONTEXTDATA)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKALIGNEDCONSTRAINTPARAMETER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKALIGNEDCONSTRAINTPARAMETER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKALIGNMENTGRIP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKALIGNMENTGRIP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKALIGNMENTPARAMETER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKALIGNMENTPARAMETER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKANGULARCONSTRAINTPARAMETER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKANGULARCONSTRAINTPARAMETER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKARRAYACTION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKARRAYACTION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKBASEPOINTPARAMETER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKBASEPOINTPARAMETER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKDIAMETRICCONSTRAINTPARAMETER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKDIAMETRICCONSTRAINTPARAMETER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKFLIPACTION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKFLIPACTION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKFLIPGRIP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKFLIPGRIP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKFLIPPARAMETER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKFLIPPARAMETER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKGRIPLOCATIONCOMPONENT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKGRIPLOCATIONCOMPONENT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKHORIZONTALCONSTRAINTPARAMETER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKHORIZONTALCONSTRAINTPARAMETER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKLINEARCONSTRAINTPARAMETER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKLINEARCONSTRAINTPARAMETER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKLINEARGRIP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKLINEARGRIP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKLINEARPARAMETER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKLINEARPARAMETER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKLOOKUPACTION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKLOOKUPACTION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKLOOKUPGRIP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKLOOKUPGRIP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKLOOKUPPARAMETER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKLOOKUPPARAMETER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKMOVEACTION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKMOVEACTION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKPARAMDEPENDENCYBODY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKPARAMDEPENDENCYBODY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKPOINTPARAMETER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKPOINTPARAMETER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKPOLARGRIP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKPOLARGRIP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKPOLARPARAMETER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKPOLARPARAMETER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKPOLARSTRETCHACTION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKPOLARSTRETCHACTION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKPROPERTIESTABLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKPROPERTIESTABLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKPROPERTIESTABLEGRIP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKPROPERTIESTABLEGRIP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKRADIALCONSTRAINTPARAMETER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKRADIALCONSTRAINTPARAMETER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKREPRESENTATION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKREPRESENTATION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKROTATEACTION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKROTATEACTION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKROTATIONGRIP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKROTATIONGRIP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKROTATIONPARAMETER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKROTATIONPARAMETER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKSCALEACTION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKSCALEACTION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKSTRETCHACTION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKSTRETCHACTION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKUSERPARAMETER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKUSERPARAMETER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKVERTICALCONSTRAINTPARAMETER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKVERTICALCONSTRAINTPARAMETER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKVISIBILITYGRIP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKVISIBILITYGRIP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKVISIBILITYPARAMETER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKVISIBILITYPARAMETER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKXYGRIP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKXYGRIP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCKXYPARAMETER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCKXYPARAMETER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_CAMERA", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_CAMERA)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_CELLSTYLEMAP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_CELLSTYLEMAP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_CONTEXTDATAMANAGER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_CONTEXTDATAMANAGER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_CSACDOCUMENTOPTIONS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_CSACDOCUMENTOPTIONS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_CURVEPATH", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_CURVEPATH)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DATALINK", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DATALINK)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DATATABLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DATATABLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DBCOLOR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DBCOLOR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DETAILVIEWSTYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DETAILVIEWSTYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DGNDEFINITION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DGNDEFINITION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DGNUNDERLAY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DGNUNDERLAY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DICTIONARYVAR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DICTIONARYVAR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DICTIONARYWDFLT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DICTIONARYWDFLT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DIMASSOC", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DIMASSOC)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DMDIMOBJECTCONTEXTDATA", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DMDIMOBJECTCONTEXTDATA)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DWFDEFINITION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DWFDEFINITION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DWFUNDERLAY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DWFUNDERLAY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DYNAMICBLOCKPROXYNODE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DYNAMICBLOCKPROXYNODE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DYNAMICBLOCKPURGEPREVENTER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DYNAMICBLOCKPURGEPREVENTER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_EVALUATION_GRAPH", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_EVALUATION_GRAPH)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_EXTRUDEDSURFACE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_EXTRUDEDSURFACE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_FCFOBJECTCONTEXTDATA", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_FCFOBJECTCONTEXTDATA)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_FIELD", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_FIELD)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_FIELDLIST", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_FIELDLIST)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_FLIPPARAMETERENTITY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_FLIPPARAMETERENTITY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_FLIPACTIONENTITY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_FLIPACTIONENTITY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_GEODATA", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_GEODATA)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_GEOMAPIMAGE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_GEOMAPIMAGE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_GEOPOSITIONMARKER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_GEOPOSITIONMARKER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_GRADIENT_BACKGROUND", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_GRADIENT_BACKGROUND)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_GROUND_PLANE_BACKGROUND", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_GROUND_PLANE_BACKGROUND)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_HELIX", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_HELIX)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_IBL_BACKGROUND", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_IBL_BACKGROUND)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_IDBUFFER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_IDBUFFER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_IMAGE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_IMAGE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_IMAGEDEF", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_IMAGEDEF)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_IMAGEDEF_REACTOR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_IMAGEDEF_REACTOR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_IMAGE_BACKGROUND", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_IMAGE_BACKGROUND)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_INDEX", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_INDEX)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_LARGE_RADIAL_DIMENSION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_LARGE_RADIAL_DIMENSION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_LAYERFILTER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_LAYERFILTER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_LAYER_INDEX", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_LAYER_INDEX)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_LAYOUTPRINTCONFIG", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_LAYOUTPRINTCONFIG)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_LEADEROBJECTCONTEXTDATA", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_LEADEROBJECTCONTEXTDATA)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_LIGHT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_LIGHT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_LIGHTLIST", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_LIGHTLIST)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_LINEARPARAMETERENTITY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_LINEARPARAMETERENTITY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_LOFTEDSURFACE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_LOFTEDSURFACE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_MATERIAL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_MATERIAL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_MENTALRAYRENDERSETTINGS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_MENTALRAYRENDERSETTINGS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_MESH", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_MESH)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_MLEADEROBJECTCONTEXTDATA", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_MLEADEROBJECTCONTEXTDATA)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_MLEADERSTYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_MLEADERSTYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_MOVEACTIONENTITY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_MOVEACTIONENTITY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_MOTIONPATH", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_MOTIONPATH)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_MPOLYGON", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_MPOLYGON)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_MTEXTATTRIBUTEOBJECTCONTEXTDATA", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_MTEXTATTRIBUTEOBJECTCONTEXTDATA)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_MTEXTOBJECTCONTEXTDATA", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_MTEXTOBJECTCONTEXTDATA)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_MULTILEADER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_MULTILEADER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_NAVISWORKSMODEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_NAVISWORKSMODEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_NAVISWORKSMODELDEF", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_NAVISWORKSMODELDEF)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_NPOCOLLECTION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_NPOCOLLECTION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_NURBSURFACE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_NURBSURFACE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_OBJECT_PTR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_OBJECT_PTR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ORDDIMOBJECTCONTEXTDATA", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ORDDIMOBJECTCONTEXTDATA)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_PARTIAL_VIEWING_INDEX", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_PARTIAL_VIEWING_INDEX)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_PDFDEFINITION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_PDFDEFINITION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_PDFUNDERLAY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_PDFUNDERLAY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_PERSUBENTMGR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_PERSUBENTMGR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_PLANESURFACE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_PLANESURFACE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_PLOTSETTINGS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_PLOTSETTINGS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_POINTCLOUD", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_POINTCLOUD)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_POINTCLOUDCOLORMAP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_POINTCLOUDCOLORMAP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_POINTCLOUDDEF", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_POINTCLOUDDEF)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_POINTCLOUDDEFEX", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_POINTCLOUDDEFEX)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_POINTCLOUDDEF_REACTOR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_POINTCLOUDDEF_REACTOR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_POINTCLOUDDEF_REACTOR_EX", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_POINTCLOUDDEF_REACTOR_EX)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_POINTCLOUDEX", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_POINTCLOUDEX)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_POINTPARAMETERENTITY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_POINTPARAMETERENTITY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_POINTPATH", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_POINTPATH)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_POLARGRIPENTITY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_POLARGRIPENTITY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_RADIMLGOBJECTCONTEXTDATA", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_RADIMLGOBJECTCONTEXTDATA)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_RADIMOBJECTCONTEXTDATA", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_RADIMOBJECTCONTEXTDATA)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_RAPIDRTRENDERSETTINGS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_RAPIDRTRENDERSETTINGS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_RASTERVARIABLES", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_RASTERVARIABLES)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_RENDERENTRY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_RENDERENTRY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_RENDERENVIRONMENT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_RENDERENVIRONMENT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_RENDERGLOBAL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_RENDERGLOBAL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_RENDERSETTINGS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_RENDERSETTINGS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_REVOLVEDSURFACE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_REVOLVEDSURFACE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ROTATIONPARAMETERENTITY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ROTATIONPARAMETERENTITY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ROTATEACTIONENTITY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ROTATEACTIONENTITY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_RTEXT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_RTEXT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_SCALE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_SCALE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_SCALEACTIONENTITY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_SCALEACTIONENTITY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_SECTIONOBJECT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_SECTIONOBJECT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_SECTIONVIEWSTYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_SECTIONVIEWSTYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_SECTION_MANAGER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_SECTION_MANAGER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_SECTION_SETTINGS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_SECTION_SETTINGS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_SKYLIGHT_BACKGROUND", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_SKYLIGHT_BACKGROUND)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_SOLID_BACKGROUND", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_SOLID_BACKGROUND)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_SORTENTSTABLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_SORTENTSTABLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_SPATIAL_FILTER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_SPATIAL_FILTER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_SPATIAL_INDEX", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_SPATIAL_INDEX)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_STRETCHACTIONENTITY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_STRETCHACTIONENTITY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_SUN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_SUN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_SUNSTUDY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_SUNSTUDY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_SWEPTSURFACE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_SWEPTSURFACE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_TABLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_TABLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_TABLECONTENT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_TABLECONTENT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_TABLEGEOMETRY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_TABLEGEOMETRY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_TABLESTYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_TABLESTYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_TEXTOBJECTCONTEXTDATA", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_TEXTOBJECTCONTEXTDATA)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_TVDEVICEPROPERTIES", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_TVDEVICEPROPERTIES)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_VISIBILITYGRIPENTITY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_VISIBILITYGRIPENTITY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_VISIBILITYPARAMETERENTITY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_VISIBILITYPARAMETERENTITY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_VISUALSTYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_VISUALSTYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_WIPEOUT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_WIPEOUT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_WIPEOUTVARIABLES", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_WIPEOUTVARIABLES)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_XREFPANELOBJECT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_XREFPANELOBJECT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_XYPARAMETERENTITY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_XYPARAMETERENTITY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BREAKDATA", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BREAKDATA)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BREAKPOINTREF", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BREAKPOINTREF)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_FLIPGRIPENTITY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_FLIPGRIPENTITY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_LINEARGRIPENTITY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_LINEARGRIPENTITY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ROTATIONGRIPENTITY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ROTATIONGRIPENTITY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_XYGRIPENTITY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_XYGRIPENTITY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE__3DLINE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE__3DLINE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_REPEAT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_REPEAT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ENDREP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ENDREP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_JUMP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_JUMP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_LOAD", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_LOAD)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ABSHDRAWINGSETTINGS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ABSHDRAWINGSETTINGS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACAECUSTOBJ", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACAECUSTOBJ)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACAEEEMGROBJ", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACAEEEMGROBJ)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACAMCOMP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACAMCOMP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACAMCOMPDEF", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACAMCOMPDEF)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACAMCOMPDEFMGR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACAMCOMPDEFMGR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACAMCONTEXTMODELER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACAMCONTEXTMODELER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACAMGDIMSTD", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACAMGDIMSTD)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACAMGFILTERDAT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACAMGFILTERDAT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACAMGHOLECHARTSTDCSN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACAMGHOLECHARTSTDCSN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACAMGHOLECHARTSTDDIN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACAMGHOLECHARTSTDDIN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACAMGHOLECHARTSTDISO", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACAMGHOLECHARTSTDISO)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACAMGLAYSTD", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACAMGLAYSTD)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACAMGRCOMPDEF", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACAMGRCOMPDEF)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACAMGRCOMPDEFSET", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACAMGRCOMPDEFSET)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACAMGTITLESTD", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACAMGTITLESTD)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACAMMVDBACKUPOBJECT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACAMMVDBACKUPOBJECT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACAMPROJECT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACAMPROJECT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACAMSHAFTCOMPDEF", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACAMSHAFTCOMPDEF)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACAMSTDPCOMPDEF", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACAMSTDPCOMPDEF)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACAMWBLOCKTEMPENTS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACAMWBLOCKTEMPENTS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACARRAYJIGENTITY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACARRAYJIGENTITY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACCMCONTEXT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACCMCONTEXT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACDBCIRCARCRES", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACDBCIRCARCRES)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACDBDIMENSIONRES", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACDBDIMENSIONRES)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACDBENTITYCACHE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACDBENTITYCACHE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACDBLINERES", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACDBLINERES)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACDBSTDPARTRES_ARC", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACDBSTDPARTRES_ARC)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACDBSTDPARTRES_LINE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACDBSTDPARTRES_LINE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACDB_HATCHSCALECONTEXTDATA_CLASS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACDB_HATCHSCALECONTEXTDATA_CLASS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACDB_HATCHVIEWCONTEXTDATA_CLASS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACDB_HATCHVIEWCONTEXTDATA_CLASS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACDB_PROXY_ENTITY_DATA", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACDB_PROXY_ENTITY_DATA)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACGREFACADMASTER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACGREFACADMASTER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACGREFMASTER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACGREFMASTER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACIMINTSYSVAR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACIMINTSYSVAR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACIMREALSYSVAR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACIMREALSYSVAR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACIMSTRSYSVAR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACIMSTRSYSVAR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACIMSYSVARMAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACIMSYSVARMAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMANOOTATIONVIEWSTANDARDANSI", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMANOOTATIONVIEWSTANDARDANSI)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMANOOTATIONVIEWSTANDARDCSN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMANOOTATIONVIEWSTANDARDCSN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMANOOTATIONVIEWSTANDARDDIN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMANOOTATIONVIEWSTANDARDDIN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMANOOTATIONVIEWSTANDARDISO", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMANOOTATIONVIEWSTANDARDISO)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMAPLEGENDDBOBJECT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMAPLEGENDDBOBJECT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMAPLEGENDITEMDBOBJECT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMAPLEGENDITEMDBOBJECT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMAPMAPVIEWPORTDBOBJECT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMAPMAPVIEWPORTDBOBJECT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMAPPRINTLAYOUTELEMENTDBOBJECTCONTAINER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMAPPRINTLAYOUTELEMENTDBOBJECTCONTAINER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMBALLOON", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMBALLOON)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMBOM", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMBOM)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMBOMROW", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMBOMROW)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMBOMROWSTRUCT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMBOMROWSTRUCT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMBOMSTANDARDANSI", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMBOMSTANDARDANSI)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMBOMSTANDARDCSN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMBOMSTANDARDCSN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMBOMSTANDARDDIN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMBOMSTANDARDDIN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMBOMSTANDARDISO", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMBOMSTANDARDISO)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMCENTERLINESTANDARDANSI", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMCENTERLINESTANDARDANSI)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMCENTERLINESTANDARDCSN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMCENTERLINESTANDARDCSN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMCENTERLINESTANDARDDIN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMCENTERLINESTANDARDDIN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMCENTERLINESTANDARDISO", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMCENTERLINESTANDARDISO)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMDATADICTIONARY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMDATADICTIONARY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMDATAENTRY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMDATAENTRY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMDATAENTRYBLOCK", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMDATAENTRYBLOCK)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMDATUMID", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMDATUMID)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMDATUMSTANDARDANSI", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMDATUMSTANDARDANSI)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMDATUMSTANDARDCSN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMDATUMSTANDARDCSN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMDATUMSTANDARDDIN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMDATUMSTANDARDDIN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMDATUMSTANDARDISO", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMDATUMSTANDARDISO)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMDATUMSTANDARDISO2012", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMDATUMSTANDARDISO2012)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMDETAILSTANDARDANSI", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMDETAILSTANDARDANSI)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMDETAILSTANDARDCSN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMDETAILSTANDARDCSN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMDETAILSTANDARDDIN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMDETAILSTANDARDDIN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMDETAILSTANDARDISO", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMDETAILSTANDARDISO)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMDETAILTANDARDCUSTOM", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMDETAILTANDARDCUSTOM)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMDIMBREAKPERSREACTOR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMDIMBREAKPERSREACTOR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMEDRAWINGMAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMEDRAWINGMAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMEVIEW", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMEVIEW)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACME_DATABASE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACME_DATABASE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACME_DOCUMENT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACME_DOCUMENT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMFCFRAME", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMFCFRAME)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMFCFSTANDARDANSI", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMFCFSTANDARDANSI)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMFCFSTANDARDCSN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMFCFSTANDARDCSN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMFCFSTANDARDDIN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMFCFSTANDARDDIN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMFCFSTANDARDISO", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMFCFSTANDARDISO)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMFCFSTANDARDISO2004", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMFCFSTANDARDISO2004)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMFCFSTANDARDISO2012", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMFCFSTANDARDISO2012)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMIDSTANDARDANSI", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMIDSTANDARDANSI)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMIDSTANDARDCSN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMIDSTANDARDCSN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMIDSTANDARDDIN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMIDSTANDARDDIN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMIDSTANDARDISO", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMIDSTANDARDISO)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMIDSTANDARDISO2004", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMIDSTANDARDISO2004)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMIDSTANDARDISO2012", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMIDSTANDARDISO2012)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMNOTESTANDARDANSI", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMNOTESTANDARDANSI)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMNOTESTANDARDCSN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMNOTESTANDARDCSN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMNOTESTANDARDDIN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMNOTESTANDARDDIN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMNOTESTANDARDISO", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMNOTESTANDARDISO)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMPARTLIST", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMPARTLIST)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMPICKOBJ", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMPICKOBJ)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMSECTIONSTANDARDANSI", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMSECTIONSTANDARDANSI)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMSECTIONSTANDARDCSN2002", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMSECTIONSTANDARDCSN2002)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMSECTIONSTANDARDCUSTOM", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMSECTIONSTANDARDCUSTOM)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMSECTIONSTANDARDDIN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMSECTIONSTANDARDDIN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMSECTIONSTANDARDISO", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMSECTIONSTANDARDISO)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMSECTIONSTANDARDISO2001", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMSECTIONSTANDARDISO2001)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMSTANDARDANSI", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMSTANDARDANSI)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMSTANDARDCSN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMSTANDARDCSN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMSTANDARDDIN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMSTANDARDDIN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMSTANDARDISO", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMSTANDARDISO)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMSURFSTANDARDANSI", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMSURFSTANDARDANSI)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMSURFSTANDARDCSN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMSURFSTANDARDCSN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMSURFSTANDARDDIN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMSURFSTANDARDDIN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMSURFSTANDARDISO", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMSURFSTANDARDISO)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMSURFSTANDARDISO2002", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMSURFSTANDARDISO2002)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMSURFSYM", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMSURFSYM)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMTAPERSTANDARDANSI", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMTAPERSTANDARDANSI)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMTAPERSTANDARDCSN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMTAPERSTANDARDCSN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMTAPERSTANDARDDIN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMTAPERSTANDARDDIN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMTAPERSTANDARDISO", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMTAPERSTANDARDISO)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMTHREADLINESTANDARDANSI", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMTHREADLINESTANDARDANSI)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMTHREADLINESTANDARDCSN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMTHREADLINESTANDARDCSN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMTHREADLINESTANDARDDIN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMTHREADLINESTANDARDDIN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMTHREADLINESTANDARDISO", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMTHREADLINESTANDARDISO)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMWELDSTANDARDANSI", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMWELDSTANDARDANSI)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMWELDSTANDARDCSN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMWELDSTANDARDCSN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMWELDSTANDARDDIN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMWELDSTANDARDDIN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMWELDSTANDARDISO", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMWELDSTANDARDISO)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACMWELDSYM", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACMWELDSYM)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACRFATTGENMGR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACRFATTGENMGR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACRFINSADJ", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACRFINSADJ)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACRFINSADJUSTERMGR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACRFINSADJUSTERMGR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACRFMCADAPIATTHOLDER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACRFMCADAPIATTHOLDER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACRFOBJATTMGR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACRFOBJATTMGR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ACSH_SUBENT_MATERIAL_CLASS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ACSH_SUBENT_MATERIAL_CLASS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AC_AM_2D_XREF_MGR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AC_AM_2D_XREF_MGR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AC_AM_BASIC_VIEW", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AC_AM_BASIC_VIEW)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AC_AM_BASIC_VIEW_DEF", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AC_AM_BASIC_VIEW_DEF)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AC_AM_COMPLEX_HIDE_SITUATION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AC_AM_COMPLEX_HIDE_SITUATION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AC_AM_COMP_VIEW_DEF", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AC_AM_COMP_VIEW_DEF)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AC_AM_COMP_VIEW_INST", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AC_AM_COMP_VIEW_INST)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AC_AM_DIRTY_NODES", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AC_AM_DIRTY_NODES)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AC_AM_HIDE_SITUATION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AC_AM_HIDE_SITUATION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AC_AM_MAPPER_CACHE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AC_AM_MAPPER_CACHE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AC_AM_MASTER_VIEW_DEF", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AC_AM_MASTER_VIEW_DEF)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AC_AM_MVD_DEP_MGR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AC_AM_MVD_DEP_MGR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AC_AM_OVERRIDE_FILTER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AC_AM_OVERRIDE_FILTER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AC_AM_PROPS_OVERRIDE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AC_AM_PROPS_OVERRIDE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AC_AM_SHAFT_HIDE_SITUATION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AC_AM_SHAFT_HIDE_SITUATION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AC_AM_STDP_VIEW_DEF", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AC_AM_STDP_VIEW_DEF)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AC_AM_TRANSFORM_GHOST", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AC_AM_TRANSFORM_GHOST)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ADAPPL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ADAPPL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_ALIGNMENT_DESIGN_CHECK_SET", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_ALIGNMENT_DESIGN_CHECK_SET)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_ALIGNMENT_LABEL_SET", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_ALIGNMENT_LABEL_SET)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_ALIGNMENT_LABEL_SET_EXT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_ALIGNMENT_LABEL_SET_EXT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_ALIGNMENT_PARCEL_NODE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_ALIGNMENT_PARCEL_NODE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_ALIGNMENT_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_ALIGNMENT_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_APPURTENANCE_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_APPURTENANCE_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_ASSEMBLY_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_ASSEMBLY_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_BUILDING_SITE_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_BUILDING_SITE_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_CANT_DIAGRAM_VIEW_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_CANT_DIAGRAM_VIEW_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_CATCHMENT_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_CATCHMENT_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_CLASS_NODE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_CLASS_NODE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_CONTOURVIEW", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_CONTOURVIEW)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_CORRIDOR_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_CORRIDOR_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_ALIGNMENT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_ALIGNMENT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_ALIGNMENT_CANT_LABEL_GROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_ALIGNMENT_CANT_LABEL_GROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_ALIGNMENT_CSV", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_ALIGNMENT_CSV)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_ALIGNMENT_CURVE_LABEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_ALIGNMENT_CURVE_LABEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_ALIGNMENT_DESIGNSPEED_LABEL_GROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_ALIGNMENT_DESIGNSPEED_LABEL_GROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_ALIGNMENT_GEOMPT_LABEL_GROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_ALIGNMENT_GEOMPT_LABEL_GROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_ALIGNMENT_INDEXED_PI_LABEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_ALIGNMENT_INDEXED_PI_LABEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_ALIGNMENT_MINOR_STATION_LABEL_GROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_ALIGNMENT_MINOR_STATION_LABEL_GROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_ALIGNMENT_PI_LABEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_ALIGNMENT_PI_LABEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_ALIGNMENT_SPIRAL_LABEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_ALIGNMENT_SPIRAL_LABEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_ALIGNMENT_STAEQU_LABEL_GROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_ALIGNMENT_STAEQU_LABEL_GROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_ALIGNMENT_STATION_LABEL_GROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_ALIGNMENT_STATION_LABEL_GROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_ALIGNMENT_STATION_OFFSET_LABEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_ALIGNMENT_STATION_OFFSET_LABEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_ALIGNMENT_SUPERELEVATION_LABEL_GROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_ALIGNMENT_SUPERELEVATION_LABEL_GROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_ALIGNMENT_TABLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_ALIGNMENT_TABLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_ALIGNMENT_TANGENT_LABEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_ALIGNMENT_TANGENT_LABEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_ALIGNMENT_VERTICAL_GEOMPT_LABELING", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_ALIGNMENT_VERTICAL_GEOMPT_LABELING)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_APPURTENANCE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_APPURTENANCE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_APPURTENANCE_CSV", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_APPURTENANCE_CSV)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_APPURTENANCE_LABELING", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_APPURTENANCE_LABELING)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_APPURTENANCE_PROFILE_LABELING", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_APPURTENANCE_PROFILE_LABELING)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_ASSEMBLY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_ASSEMBLY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_AUTO_CORRIDOR_FEATURE_LINE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_AUTO_CORRIDOR_FEATURE_LINE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_AUTO_CORRIDOR_FEATURE_LINE_PROFILE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_AUTO_CORRIDOR_FEATURE_LINE_PROFILE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_AUTO_CORRIDOR_FEATURE_LINE_SECTION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_AUTO_CORRIDOR_FEATURE_LINE_SECTION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_AUTO_FEATURE_LINE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_AUTO_FEATURE_LINE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_AUTO_FEATURE_LINE_CSV", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_AUTO_FEATURE_LINE_CSV)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_AUTO_FEATURE_LINE_PROFILE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_AUTO_FEATURE_LINE_PROFILE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_AUTO_FEATURE_LINE_SECTION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_AUTO_FEATURE_LINE_SECTION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_BUILDINGSITE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_BUILDINGSITE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_BUILDINGUTIL_CONNECTOR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_BUILDINGUTIL_CONNECTOR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_CANT_DIAGRAM_VIEW", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_CANT_DIAGRAM_VIEW)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_CATCHMENT_AREA", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_CATCHMENT_AREA)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_CATCHMENT_AREA_LABEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_CATCHMENT_AREA_LABEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_CORRIDOR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_CORRIDOR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_CROSSING_PIPE_PROFILE_LABELING", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_CROSSING_PIPE_PROFILE_LABELING)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_CROSSING_PRESSURE_PIPE_PROFILE_LABELING", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_CROSSING_PRESSURE_PIPE_PROFILE_LABELING)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_CSVSTATIONSLIDER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_CSVSTATIONSLIDER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_FACE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_FACE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_FEATURE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_FEATURE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_FEATURE_LABEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_FEATURE_LABEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_FEATURE_LINE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_FEATURE_LINE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_FEATURE_LINE_CSV", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_FEATURE_LINE_CSV)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_FEATURE_LINE_PROFILE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_FEATURE_LINE_PROFILE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_FEATURE_LINE_SECTION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_FEATURE_LINE_SECTION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_FITTING", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_FITTING)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_FITTING_CSV", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_FITTING_CSV)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_FITTING_LABELING", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_FITTING_LABELING)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_FITTING_PROFILE_LABELING", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_FITTING_PROFILE_LABELING)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_FLOW_SEGMENT_LABEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_FLOW_SEGMENT_LABEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_GENERAL_SEGMENT_LABEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_GENERAL_SEGMENT_LABEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_GRADING", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_GRADING)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_GRAPH", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_GRAPH)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_GRAPHPROFILE_NETWORKPART", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_GRAPHPROFILE_NETWORKPART)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_GRAPHPROFILE_PRESSUREPART", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_GRAPHPROFILE_PRESSUREPART)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_GRID_SURFACE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_GRID_SURFACE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_GRID_SURFACE_CSV", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_GRID_SURFACE_CSV)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_HORGEOMETRY_BAND_LABEL_GROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_HORGEOMETRY_BAND_LABEL_GROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_HYDRO_REGION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_HYDRO_REGION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_INTERFERENCE_CHECK", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_INTERFERENCE_CHECK)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_INTERFERENCE_PART", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_INTERFERENCE_PART)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_INTERFERENCE_PART_SECTION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_INTERFERENCE_PART_SECTION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_INTERSECTION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_INTERSECTION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_INTERSECTION_LOCATION_LABELING", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_INTERSECTION_LOCATION_LABELING)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_LABELING", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_LABELING)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_LEGEND_TABLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_LEGEND_TABLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_LINE_BETWEEN_POINTS_LABEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_LINE_BETWEEN_POINTS_LABEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_LOTLINE_CSV", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_LOTLINE_CSV)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_MASSHAULLINE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_MASSHAULLINE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_MASS_HAUL_VIEW", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_MASS_HAUL_VIEW)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_MATCHLINE_LABELING", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_MATCHLINE_LABELING)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_MATCH_LINE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_MATCH_LINE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_MATERIAL_SECTION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_MATERIAL_SECTION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_NETWORK", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_NETWORK)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_NOTE_LABEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_NOTE_LABEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_OFFSET_ELEV_LABEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_OFFSET_ELEV_LABEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_PARCEL_BOUNDARY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_PARCEL_BOUNDARY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_PARCEL_FACE_LABEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_PARCEL_FACE_LABEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_PARCEL_SEGMENT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_PARCEL_SEGMENT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_PARCEL_SEGMENT_LABEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_PARCEL_SEGMENT_LABEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_PARCEL_SEGMENT_TABLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_PARCEL_SEGMENT_TABLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_PARCEL_TABLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_PARCEL_TABLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_PIPE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_PIPE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_PIPENETWORK_BAND_LABEL_GROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_PIPENETWORK_BAND_LABEL_GROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_PIPE_CSV", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_PIPE_CSV)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_PIPE_LABELING", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_PIPE_LABELING)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_PIPE_PROFILE_LABELING", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_PIPE_PROFILE_LABELING)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_PIPE_SECTION_LABELING", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_PIPE_SECTION_LABELING)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_PIPE_TABLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_PIPE_TABLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_POINT_ENT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_POINT_ENT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_POINT_GROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_POINT_GROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_POINT_TABLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_POINT_TABLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_PRESSUREPIPENETWORK", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_PRESSUREPIPENETWORK)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_PRESSURE_PART_TABLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_PRESSURE_PART_TABLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_PRESSURE_PIPE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_PRESSURE_PIPE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_PRESSURE_PIPE_CSV", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_PRESSURE_PIPE_CSV)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_PRESSURE_PIPE_LABELING", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_PRESSURE_PIPE_LABELING)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_PRESSURE_PIPE_PROFILE_LABELING", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_PRESSURE_PIPE_PROFILE_LABELING)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_PRESSURE_PIPE_SECTION_LABELING", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_PRESSURE_PIPE_SECTION_LABELING)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_PROFILE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_PROFILE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_PROFILEDATA_BAND_LABEL_GROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_PROFILEDATA_BAND_LABEL_GROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_PROFILE_PROJECTION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_PROFILE_PROJECTION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_PROFILE_PROJECTION_LABEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_PROFILE_PROJECTION_LABEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_PROFILE_VIEW", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_PROFILE_VIEW)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_PROFILE_VIEW_DEPTH_LABEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_PROFILE_VIEW_DEPTH_LABEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_QUANTITY_TAKEOFF_AGGREGATE_EARTHWORK_TABLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_QUANTITY_TAKEOFF_AGGREGATE_EARTHWORK_TABLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_RIGHT_OF_WAY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_RIGHT_OF_WAY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SAMPLELINE_LABELING", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SAMPLELINE_LABELING)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SAMPLE_LINE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SAMPLE_LINE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SAMPLE_LINE_GROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SAMPLE_LINE_GROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SECTION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SECTION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SECTIONALDATA_BAND_LABEL_GROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SECTIONALDATA_BAND_LABEL_GROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SECTIONDATA_BAND_LABEL_GROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SECTIONDATA_BAND_LABEL_GROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SECTIONSEGMENT_BAND_LABEL_GROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SECTIONSEGMENT_BAND_LABEL_GROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SECTION_CORRIDOR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SECTION_CORRIDOR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SECTION_CORRIDOR_POINT_LABEL_GROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SECTION_CORRIDOR_POINT_LABEL_GROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SECTION_GRADEBREAK_LABEL_GROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SECTION_GRADEBREAK_LABEL_GROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SECTION_MINOR_OFFSET_LABEL_GROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SECTION_MINOR_OFFSET_LABEL_GROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SECTION_OFFSET_LABEL_GROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SECTION_OFFSET_LABEL_GROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SECTION_PIPENETWORK", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SECTION_PIPENETWORK)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SECTION_PRESSUREPIPENETWORK", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SECTION_PRESSUREPIPENETWORK)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SECTION_PROJECTION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SECTION_PROJECTION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SECTION_PROJECTION_LABEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SECTION_PROJECTION_LABEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SECTION_SEGMENT_LABEL_GROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SECTION_SEGMENT_LABEL_GROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SECTION_VIEW", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SECTION_VIEW)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SECTION_VIEW_DEPTH_LABEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SECTION_VIEW_DEPTH_LABEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SECTION_VIEW_QUANTITY_TAKEOFF_TABLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SECTION_VIEW_QUANTITY_TAKEOFF_TABLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SHEET", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SHEET)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SPANNING_PIPE_LABELING", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SPANNING_PIPE_LABELING)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SPANNING_PIPE_PROFILE_LABELING", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SPANNING_PIPE_PROFILE_LABELING)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_STATION_ELEV_LABEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_STATION_ELEV_LABEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_STRUCTURE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_STRUCTURE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_STRUCTURE_CSV", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_STRUCTURE_CSV)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_STRUCTURE_LABELING", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_STRUCTURE_LABELING)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_STRUCTURE_PROFILE_LABELING", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_STRUCTURE_PROFILE_LABELING)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_STRUCTURE_SECTION_LABELING", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_STRUCTURE_SECTION_LABELING)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SUBASSEMBLY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SUBASSEMBLY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SUPERELEVATION_BAND_LABEL_GROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SUPERELEVATION_BAND_LABEL_GROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SUPERELEVATION_DIAGRAM_VIEW", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SUPERELEVATION_DIAGRAM_VIEW)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SURFACE_CONTOUR_LABEL_GROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SURFACE_CONTOUR_LABEL_GROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SURFACE_ELEVATION_LABEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SURFACE_ELEVATION_LABEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SURFACE_SLOPE_LABEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SURFACE_SLOPE_LABEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SURVEY_FIGURE_LABEL_GROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SURVEY_FIGURE_LABEL_GROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SVFIGURE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SVFIGURE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SVFIGURE_CSV", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SVFIGURE_CSV)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SVFIGURE_PROFILE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SVFIGURE_PROFILE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SVFIGURE_SECTION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SVFIGURE_SECTION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SVFIGURE_SEGMENT_LABEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SVFIGURE_SEGMENT_LABEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_SVNETWORK", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_SVNETWORK)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_TANGENT_INTERSECTION_TABLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_TANGENT_INTERSECTION_TABLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_TIN_SURFACE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_TIN_SURFACE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_TIN_SURFACE_CSV", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_TIN_SURFACE_CSV)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_VALIGNMENT_CRESTCURVE_LABEL_GROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_VALIGNMENT_CRESTCURVE_LABEL_GROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_VALIGNMENT_CSV", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_VALIGNMENT_CSV)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_VALIGNMENT_HAGEOMPT_LABEL_GROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_VALIGNMENT_HAGEOMPT_LABEL_GROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_VALIGNMENT_LINE_LABEL_GROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_VALIGNMENT_LINE_LABEL_GROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_VALIGNMENT_MINOR_STATION_LABEL_GROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_VALIGNMENT_MINOR_STATION_LABEL_GROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_VALIGNMENT_PVI_LABEL_GROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_VALIGNMENT_PVI_LABEL_GROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_VALIGNMENT_SAGCURVE_LABEL_GROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_VALIGNMENT_SAGCURVE_LABEL_GROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_VALIGNMENT_STATION_LABEL_GROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_VALIGNMENT_STATION_LABEL_GROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_VERTICALGEOMETRY_BAND_LABEL_GROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_VERTICALGEOMETRY_BAND_LABEL_GROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_VIEWFRAME_LABELING", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_VIEWFRAME_LABELING)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_DISP_REP_VIEW_FRAME", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_DISP_REP_VIEW_FRAME)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_FEATURELINE_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_FEATURELINE_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_FEATURE_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_FEATURE_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_FITTING_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_FITTING_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_FORMAT_MANAGER_OBJECT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_FORMAT_MANAGER_OBJECT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_GRADEVIEW", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_GRADEVIEW)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_GRADING_CRITERIA", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_GRADING_CRITERIA)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_GRADING_CRITERIA_SET", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_GRADING_CRITERIA_SET)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_GRADING_GROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_GRADING_GROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_GRADING_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_GRADING_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_IMPORT_STORM_SEWER_DEFAULTS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_IMPORT_STORM_SEWER_DEFAULTS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_INTERFERENCE_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_INTERFERENCE_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_INTERSECTION_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_INTERSECTION_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_LABEL_COLLECTOR_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_LABEL_COLLECTOR_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_LABEL_NODE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_LABEL_NODE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_LABEL_RADIAL_LINE_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_LABEL_RADIAL_LINE_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_LABEL_TEXT_ITERATOR_CURVE_OR_SPIRAL_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_LABEL_TEXT_ITERATOR_CURVE_OR_SPIRAL_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_LABEL_TEXT_ITERATOR_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_LABEL_TEXT_ITERATOR_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_LABEL_TEXT_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_LABEL_TEXT_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_LABEL_VECTOR_ARROW_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_LABEL_VECTOR_ARROW_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_LEGEND_TABLE_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_LEGEND_TABLE_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_MASS_HAUL_LINE_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_MASS_HAUL_LINE_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_MASS_HAUL_VIEW_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_MASS_HAUL_VIEW_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_MATCHLINE_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_MATCHLINE_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_MATERIAL_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_MATERIAL_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_NETWORK_PART_CATALOG_DEF_NODE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_NETWORK_PART_CATALOG_DEF_NODE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_NETWORK_PART_FAMILY_ITEM", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_NETWORK_PART_FAMILY_ITEM)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_NETWORK_PART_LIST", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_NETWORK_PART_LIST)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_NETWORK_RULE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_NETWORK_RULE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_PARCEL_NODE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_PARCEL_NODE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_PARCEL_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_PARCEL_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_PART_SIZE_FILTER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_PART_SIZE_FILTER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_PIPE_RULES", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_PIPE_RULES)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_PIPE_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_PIPE_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_PIPE_STYLE_EXTENSION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_PIPE_STYLE_EXTENSION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_POINTCLOUD_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_POINTCLOUD_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_POINTVIEW", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_POINTVIEW)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_POINT_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_POINT_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_PRESSURE_PART_LIST", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_PRESSURE_PART_LIST)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_PRESSURE_PIPE_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_PRESSURE_PIPE_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_PROFILESECTIONENTITY_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_PROFILESECTIONENTITY_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_PROFILE_DESIGN_CHECK_SET", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_PROFILE_DESIGN_CHECK_SET)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_PROFILE_LABEL_SET", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_PROFILE_LABEL_SET)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_PROFILE_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_PROFILE_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_PROFILE_VIEW_BAND_STYLE_SET", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_PROFILE_VIEW_BAND_STYLE_SET)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_PROFILE_VIEW_DATA_BAND_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_PROFILE_VIEW_DATA_BAND_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_PROFILE_VIEW_HORIZONTAL_GEOMETRY_BAND_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_PROFILE_VIEW_HORIZONTAL_GEOMETRY_BAND_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_PROFILE_VIEW_PIPE_NETWORK_BAND_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_PROFILE_VIEW_PIPE_NETWORK_BAND_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_PROFILE_VIEW_SECTIONAL_DATA_BAND_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_PROFILE_VIEW_SECTIONAL_DATA_BAND_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_PROFILE_VIEW_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_PROFILE_VIEW_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_PROFILE_VIEW_SUPERELEVATION_DIAGRAM_BAND_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_PROFILE_VIEW_SUPERELEVATION_DIAGRAM_BAND_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_PROFILE_VIEW_VERTICAL_GEOMETRY_BAND_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_PROFILE_VIEW_VERTICAL_GEOMETRY_BAND_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_QUANTITY_TAKEOFF_CRITERIA", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_QUANTITY_TAKEOFF_CRITERIA)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_ROADWAYLINK_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_ROADWAYLINK_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_ROADWAYMARKER_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_ROADWAYMARKER_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_ROADWAYSHAPE_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_ROADWAYSHAPE_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_ROADWAY_STYLE_SET", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_ROADWAY_STYLE_SET)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_ROOT_SETTINGS_NODE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_ROOT_SETTINGS_NODE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_SAMPLE_LINE_GROUP_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_SAMPLE_LINE_GROUP_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_SAMPLE_LINE_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_SAMPLE_LINE_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_SECTION_LABEL_SET", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_SECTION_LABEL_SET)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_SECTION_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_SECTION_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_SECTION_VIEW_BAND_STYLE_SET", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_SECTION_VIEW_BAND_STYLE_SET)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_SECTION_VIEW_DATA_BAND_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_SECTION_VIEW_DATA_BAND_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_SECTION_VIEW_ROAD_SURFACE_BAND_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_SECTION_VIEW_ROAD_SURFACE_BAND_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_SECTION_VIEW_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_SECTION_VIEW_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_SETTINGS_NODE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_SETTINGS_NODE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_SHEET_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_SHEET_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_SLOPE_PATTERN_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_SLOPE_PATTERN_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_STATION_FORMAT_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_STATION_FORMAT_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_STRUCTURE_RULES", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_STRUCTURE_RULES)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_STUCTURE_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_STUCTURE_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_SUPERELEVATION_DIAGRAM_VIEW_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_SUPERELEVATION_DIAGRAM_VIEW_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_SURFACE_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_SURFACE_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_SVFIGURE_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_SVFIGURE_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_SVNETWORK_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_SVNETWORK_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_TABLE_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_TABLE_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_TAG_MANAGER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_TAG_MANAGER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_TREE_NODE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_TREE_NODE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_USER_DEFINED_ATTRIBUTE_CLASSIFICATION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_USER_DEFINED_ATTRIBUTE_CLASSIFICATION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_VALIGNMENT_STYLE_EXTENSION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_VALIGNMENT_STYLE_EXTENSION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECC_VIEW_FRAME_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECC_VIEW_FRAME_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECS_DISP_PROPS_MEMBER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECS_DISP_PROPS_MEMBER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECS_DISP_PROPS_MEMBER_LOGICAL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECS_DISP_PROPS_MEMBER_LOGICAL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECS_DISP_PROPS_MEMBER_PLAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECS_DISP_PROPS_MEMBER_PLAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECS_DISP_PROPS_MEMBER_PLAN_SKETCH", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECS_DISP_PROPS_MEMBER_PLAN_SKETCH)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECS_DISP_PROPS_MEMBER_PROJECTED", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECS_DISP_PROPS_MEMBER_PROJECTED)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECS_DISP_REP_MEMBER_ELEVATION_DESIGN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECS_DISP_REP_MEMBER_ELEVATION_DESIGN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECS_DISP_REP_MEMBER_ELEVATION_DETAIL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECS_DISP_REP_MEMBER_ELEVATION_DETAIL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECS_DISP_REP_MEMBER_LOGICAL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECS_DISP_REP_MEMBER_LOGICAL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECS_DISP_REP_MEMBER_MODEL_DESIGN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECS_DISP_REP_MEMBER_MODEL_DESIGN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECS_DISP_REP_MEMBER_MODEL_DETAIL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECS_DISP_REP_MEMBER_MODEL_DETAIL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECS_DISP_REP_MEMBER_PLAN_DESIGN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECS_DISP_REP_MEMBER_PLAN_DESIGN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECS_DISP_REP_MEMBER_PLAN_DETAIL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECS_DISP_REP_MEMBER_PLAN_DETAIL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECS_DISP_REP_MEMBER_PLAN_SKETCH", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECS_DISP_REP_MEMBER_PLAN_SKETCH)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECS_MEMBER_NODE_SHAPE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECS_MEMBER_NODE_SHAPE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AECS_MEMBER_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AECS_MEMBER_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_2DSECTION_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_2DSECTION_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_AECDBDISPREPBDGELEVLINEPLAN100", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_AECDBDISPREPBDGELEVLINEPLAN100)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_AECDBDISPREPBDGELEVLINEPLAN50", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_AECDBDISPREPBDGELEVLINEPLAN50)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_AECDBDISPREPBDGSECTIONLINEPLAN100", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_AECDBDISPREPBDGSECTIONLINEPLAN100)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_AECDBDISPREPBDGSECTIONLINEPLAN50", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_AECDBDISPREPBDGSECTIONLINEPLAN50)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_AECDBDISPREPCEILINGGRIDPLAN100", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_AECDBDISPREPCEILINGGRIDPLAN100)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_AECDBDISPREPCEILINGGRIDPLAN50", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_AECDBDISPREPCEILINGGRIDPLAN50)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_AECDBDISPREPCOLUMNGRIDPLAN100", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_AECDBDISPREPCOLUMNGRIDPLAN100)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_AECDBDISPREPCOLUMNGRIDPLAN50", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_AECDBDISPREPCOLUMNGRIDPLAN50)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_AECDBDISPREPCURTAINWALLLAYOUTPLAN100", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_AECDBDISPREPCURTAINWALLLAYOUTPLAN100)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_AECDBDISPREPCURTAINWALLLAYOUTPLAN50", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_AECDBDISPREPCURTAINWALLLAYOUTPLAN50)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_AECDBDISPREPCURTAINWALLUNITPLAN100", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_AECDBDISPREPCURTAINWALLUNITPLAN100)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_AECDBDISPREPCURTAINWALLUNITPLAN50", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_AECDBDISPREPCURTAINWALLUNITPLAN50)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_AECDBDISPREPMVBLOCKREFPLAN100", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_AECDBDISPREPMVBLOCKREFPLAN100)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_AECDBDISPREPMVBLOCKREFPLAN50", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_AECDBDISPREPMVBLOCKREFPLAN50)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_AECDBDISPREPROOFPLAN100", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_AECDBDISPREPROOFPLAN100)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_AECDBDISPREPROOFPLAN50", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_AECDBDISPREPROOFPLAN50)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_AECDBDISPREPROOFSLABPLAN100", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_AECDBDISPREPROOFSLABPLAN100)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_AECDBDISPREPROOFSLABPLAN50", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_AECDBDISPREPROOFSLABPLAN50)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_AECDBDISPREPSLABPLAN100", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_AECDBDISPREPSLABPLAN100)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_AECDBDISPREPSLABPLAN50", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_AECDBDISPREPSLABPLAN50)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_AECDBDISPREPSPACEPLAN100", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_AECDBDISPREPSPACEPLAN100)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_AECDBDISPREPSPACEPLAN50", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_AECDBDISPREPSPACEPLAN50)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_AECDBDISPREPWALLPLAN100", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_AECDBDISPREPWALLPLAN100)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_AECDBDISPREPWALLPLAN50", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_AECDBDISPREPWALLPLAN50)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_AECDBDISPREPWINDOWASSEMBLYPLAN100", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_AECDBDISPREPWINDOWASSEMBLYPLAN100)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_AECDBDISPREPWINDOWASSEMBLYPLAN50", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_AECDBDISPREPWINDOWASSEMBLYPLAN50)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_AECDBDISPREPZONE100", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_AECDBDISPREPZONE100)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_AECDBDISPREPZONE50", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_AECDBDISPREPZONE50)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_AECDBZONEDEF", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_AECDBZONEDEF)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_AECDBZONESTYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_AECDBZONESTYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_ANCHOR_OPENINGBASE_TO_WALL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_ANCHOR_OPENINGBASE_TO_WALL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_CLASSIFICATION_DEF", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_CLASSIFICATION_DEF)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_CLASSIFICATION_SYSTEM_DEF", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_CLASSIFICATION_SYSTEM_DEF)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_CLEANUP_GROUP_DEF", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_CLEANUP_GROUP_DEF)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_CURTAIN_WALL_LAYOUT_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_CURTAIN_WALL_LAYOUT_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_CURTAIN_WALL_UNIT_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_CURTAIN_WALL_UNIT_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_CVSECTIONVIEW", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_CVSECTIONVIEW)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DB_DISP_REP_DIM_GROUP_PLAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DB_DISP_REP_DIM_GROUP_PLAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DB_DISP_REP_DIM_GROUP_PLAN100", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DB_DISP_REP_DIM_GROUP_PLAN100)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DB_DISP_REP_DIM_GROUP_PLAN50", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DB_DISP_REP_DIM_GROUP_PLAN50)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DIM_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DIM_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISPLAYTHEME_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISPLAYTHEME_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_2D_SECTION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_2D_SECTION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_CLIP_VOLUME", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_CLIP_VOLUME)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_CLIP_VOLUME_RESULT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_CLIP_VOLUME_RESULT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_DIM", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_DIM)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_DISPLAYTHEME", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_DISPLAYTHEME)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_DOOR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_DOOR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_DOOR_NOMINAL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_DOOR_NOMINAL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_DOOR_PLAN_100", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_DOOR_PLAN_100)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_DOOR_PLAN_50", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_DOOR_PLAN_50)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_DOOR_THRESHOLD_PLAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_DOOR_THRESHOLD_PLAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_DOOR_THRESHOLD_SYMBOL_PLAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_DOOR_THRESHOLD_SYMBOL_PLAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_EDITINPLACEPROFILE_MODEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_EDITINPLACEPROFILE_MODEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_ENT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_ENT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_ENT_REF", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_ENT_REF)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_GRID_ASSEMBLY_MODEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_GRID_ASSEMBLY_MODEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_GRID_ASSEMBLY_PLAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_GRID_ASSEMBLY_PLAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_LAYOUT_CURVE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_LAYOUT_CURVE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_LAYOUT_GRID2D", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_LAYOUT_GRID2D)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_LAYOUT_GRID3D", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_LAYOUT_GRID3D)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_MASKBLOCK", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_MASKBLOCK)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_MASS_ELEM_MODEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_MASS_ELEM_MODEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_MASS_GROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_MASS_GROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_MATERIAL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_MATERIAL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_OPENING", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_OPENING)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_POLYGON_MODEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_POLYGON_MODEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_POLYGON_TRUECOLOUR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_POLYGON_TRUECOLOUR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_RAILING_MODEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_RAILING_MODEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_RAILING_PLAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_RAILING_PLAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_ROOF", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_ROOF)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_ROOFSLAB", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_ROOFSLAB)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_ROOFSLAB_PLAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_ROOFSLAB_PLAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_SCHEDULE_TABLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_SCHEDULE_TABLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_SLAB", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_SLAB)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_SLAB_PLAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_SLAB_PLAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_SLICE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_SLICE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_SPACE_DECOMPOSED", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_SPACE_DECOMPOSED)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_SPACE_MODEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_SPACE_MODEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_SPACE_PLAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_SPACE_PLAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_STAIR_MODEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_STAIR_MODEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_STAIR_PLAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_STAIR_PLAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_STAIR_PLAN_OVERLAPPING", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_STAIR_PLAN_OVERLAPPING)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_WALL_GRAPH", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_WALL_GRAPH)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_WALL_MODEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_WALL_MODEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_WALL_PLAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_WALL_PLAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_WALL_SCHEM", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_WALL_SCHEM)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_WINDOW", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_WINDOW)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_WINDOW_ASSEMBLY_SILL_PLAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_WINDOW_ASSEMBLY_SILL_PLAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_WINDOW_NOMINAL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_WINDOW_NOMINAL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_WINDOW_PLAN_100", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_WINDOW_PLAN_100)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_WINDOW_PLAN_50", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_WINDOW_PLAN_50)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_WINDOW_SILL_PLAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_WINDOW_SILL_PLAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_PROPS_ZONE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_PROPS_ZONE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_2D_SECTION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_2D_SECTION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISPREPAECDBDISPREPMASSELEMPLAN100", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISPREPAECDBDISPREPMASSELEMPLAN100)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISPREPAECDBDISPREPMASSELEMPLAN50", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISPREPAECDBDISPREPMASSELEMPLAN50)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISPREPAECDBDISPREPMASSGROUPPLAN100", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISPREPAECDBDISPREPMASSGROUPPLAN100)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISPREPAECDBDISPREPMASSGROUPPLAN50", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISPREPAECDBDISPREPMASSGROUPPLAN50)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISPREPAECDBDISPREPOPENINGPLAN100", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISPREPAECDBDISPREPOPENINGPLAN100)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISPREPAECDBDISPREPOPENINGPLAN50", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISPREPAECDBDISPREPOPENINGPLAN50)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISPREPAECDBDISPREPOPENINGPLANREFLECTED", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISPREPAECDBDISPREPOPENINGPLANREFLECTED)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISPREPAECDBDISPREPOPENINGSILLPLAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISPREPAECDBDISPREPOPENINGSILLPLAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_ANCHOR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_ANCHOR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_ANCHOR_BUB_TO_GRID", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_ANCHOR_BUB_TO_GRID)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_ANCHOR_BUB_TO_GRID_MODEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_ANCHOR_BUB_TO_GRID_MODEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_ANCHOR_BUB_TO_GRID_RCP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_ANCHOR_BUB_TO_GRID_RCP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_ANCHOR_ENT_TO_NODE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_ANCHOR_ENT_TO_NODE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_ANCHOR_EXT_TAG_TO_ENT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_ANCHOR_EXT_TAG_TO_ENT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_ANCHOR_TAG_TO_ENT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_ANCHOR_TAG_TO_ENT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_BDG_ELEVLINE_MODEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_BDG_ELEVLINE_MODEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_BDG_ELEVLINE_PLAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_BDG_ELEVLINE_PLAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_BDG_ELEVLINE_RCP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_BDG_ELEVLINE_RCP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_BDG_SECTIONLINE_MODEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_BDG_SECTIONLINE_MODEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_BDG_SECTIONLINE_PLAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_BDG_SECTIONLINE_PLAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_BDG_SECTIONLINE_RCP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_BDG_SECTIONLINE_RCP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_BDG_SECTION_MODEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_BDG_SECTION_MODEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_BDG_SECTION_SUBDIV", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_BDG_SECTION_SUBDIV)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_CEILING_GRID", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_CEILING_GRID)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_CEILING_GRID_MODEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_CEILING_GRID_MODEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_CEILING_GRID_RCP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_CEILING_GRID_RCP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_CLIP_VOLUME_MODEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_CLIP_VOLUME_MODEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_CLIP_VOLUME_PLAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_CLIP_VOLUME_PLAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_CLIP_VOLUME_RESULT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_CLIP_VOLUME_RESULT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_CLIP_VOLUME_RESULT_SUBDIV", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_CLIP_VOLUME_RESULT_SUBDIV)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COL_BLOCK", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COL_BLOCK)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COL_CIRCARC2D", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COL_CIRCARC2D)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COL_CONCOINCIDENT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COL_CONCOINCIDENT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COL_CONCONCENTRIC", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COL_CONCONCENTRIC)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COL_CONEQUALDISTANCE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COL_CONEQUALDISTANCE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COL_CONMIDPOINT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COL_CONMIDPOINT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COL_CONNECTOR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COL_CONNECTOR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COL_CONNORMAL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COL_CONNORMAL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COL_CONPARALLEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COL_CONPARALLEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COL_CONPERPENDICULAR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COL_CONPERPENDICULAR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COL_CONSYMMETRIC", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COL_CONSYMMETRIC)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COL_CONTANGENT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COL_CONTANGENT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COL_DIMANGLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COL_DIMANGLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COL_DIMDIAMETER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COL_DIMDIAMETER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COL_DIMDISTANCE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COL_DIMDISTANCE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COL_DIMLENGTH", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COL_DIMLENGTH)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COL_DIMMAJORRADIUS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COL_DIMMAJORRADIUS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COL_DIMMINORRADIUS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COL_DIMMINORRADIUS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COL_ELLIPARC2D", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COL_ELLIPARC2D)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COL_LAYOUTDATA", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COL_LAYOUTDATA)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COL_LINE2D", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COL_LINE2D)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_ADD", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_ADD)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_CUTPLANE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_CUTPLANE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_EXTRUSION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_EXTRUSION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_GROUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_GROUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_LOFT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_LOFT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_PATH", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_PATH)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_REVOLVE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_REVOLVE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_SUBTRACT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_SUBTRACT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_TRANSITION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_TRANSITION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COL_POINT2D", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COL_POINT2D)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COL_PROFILE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COL_PROFILE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COLUMN_GRID", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COLUMN_GRID)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COLUMN_GRID_MODEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COLUMN_GRID_MODEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COLUMN_GRID_RCP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COLUMN_GRID_RCP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COL_WORKPLANE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COL_WORKPLANE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_COL_WORKPLANE_REF", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_COL_WORKPLANE_REF)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_CONFIG", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_CONFIG)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_CURTAIN_WALL_LAYOUT_MODEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_CURTAIN_WALL_LAYOUT_MODEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_CURTAIN_WALL_LAYOUT_PLAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_CURTAIN_WALL_LAYOUT_PLAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_CURTAIN_WALL_UNIT_MODEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_CURTAIN_WALL_UNIT_MODEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_CURTAIN_WALL_UNIT_PLAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_CURTAIN_WALL_UNIT_PLAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_DCM_DIMRADIUS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_DCM_DIMRADIUS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_DISPLAYTHEME", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_DISPLAYTHEME)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_DOOR_ELEV", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_DOOR_ELEV)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_DOOR_MODEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_DOOR_MODEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_DOOR_NOMINAL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_DOOR_NOMINAL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_DOOR_PLAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_DOOR_PLAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_DOOR_PLAN_50", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_DOOR_PLAN_50)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_DOOR_PLAN_HEKTO", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_DOOR_PLAN_HEKTO)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_DOOR_RCP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_DOOR_RCP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_DOOR_THRESHOLD_PLAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_DOOR_THRESHOLD_PLAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_DOOR_THRESHOLD_SYMBOL_PLAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_DOOR_THRESHOLD_SYMBOL_PLAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_EDITINPLACEPROFILE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_EDITINPLACEPROFILE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_ENT_REF", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_ENT_REF)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_LAYOUT_CURVE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_LAYOUT_CURVE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_LAYOUT_GRID2D", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_LAYOUT_GRID2D)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_LAYOUT_GRID3D", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_LAYOUT_GRID3D)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_MASKBLOCK_REF", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_MASKBLOCK_REF)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_MASKBLOCK_REF_RCP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_MASKBLOCK_REF_RCP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_MASS_ELEM_MODEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_MASS_ELEM_MODEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_MASS_ELEM_RCP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_MASS_ELEM_RCP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_MASS_ELEM_SCHEM", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_MASS_ELEM_SCHEM)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_MASS_GROUP_MODEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_MASS_GROUP_MODEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_MASS_GROUP_PLAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_MASS_GROUP_PLAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_MASS_GROUP_RCP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_MASS_GROUP_RCP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_MATERIAL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_MATERIAL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_MVBLOCK_REF", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_MVBLOCK_REF)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_MVBLOCK_REF_MODEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_MVBLOCK_REF_MODEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_MVBLOCK_REF_RCP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_MVBLOCK_REF_RCP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_OPENING", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_OPENING)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_OPENING_MODEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_OPENING_MODEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_POLYGON_MODEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_POLYGON_MODEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_POLYGON_TRUECOLOUR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_POLYGON_TRUECOLOUR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_RAILING_MODEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_RAILING_MODEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_RAILING_PLAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_RAILING_PLAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_RAILING_PLAN_100", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_RAILING_PLAN_100)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_RAILING_PLAN_50", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_RAILING_PLAN_50)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_ROOF_MODEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_ROOF_MODEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_ROOF_PLAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_ROOF_PLAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_ROOF_RCP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_ROOF_RCP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_ROOFSLAB_MODEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_ROOFSLAB_MODEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_ROOFSLAB_PLAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_ROOFSLAB_PLAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_SCHEDULE_TABLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_SCHEDULE_TABLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_SET", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_SET)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_SLAB_MODEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_SLAB_MODEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_SLAB_PLAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_SLAB_PLAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_SLICE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_SLICE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_SPACE_DECOMPOSED", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_SPACE_DECOMPOSED)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_SPACE_MODEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_SPACE_MODEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_SPACE_PLAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_SPACE_PLAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_SPACE_RCP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_SPACE_RCP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_SPACE_VOLUME", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_SPACE_VOLUME)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_STAIR_MODEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_STAIR_MODEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_STAIR_PLAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_STAIR_PLAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_STAIR_PLAN_100", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_STAIR_PLAN_100)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_STAIR_PLAN_50", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_STAIR_PLAN_50)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_STAIR_PLAN_OVERLAPPING", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_STAIR_PLAN_OVERLAPPING)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_STAIR_RCP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_STAIR_RCP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_WALL_GRAPH", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_WALL_GRAPH)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_WALL_MODEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_WALL_MODEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_WALL_PLAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_WALL_PLAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_WALL_RCP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_WALL_RCP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_WALL_SCHEM", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_WALL_SCHEM)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_WINDOW_ASSEMBLY_MODEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_WINDOW_ASSEMBLY_MODEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_WINDOW_ASSEMBLY_PLAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_WINDOW_ASSEMBLY_PLAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_WINDOWASSEMBLY_SILL_PLAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_WINDOWASSEMBLY_SILL_PLAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_WINDOW_ELEV", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_WINDOW_ELEV)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_WINDOW_MODEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_WINDOW_MODEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_WINDOW_NOMINAL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_WINDOW_NOMINAL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_WINDOW_PLAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_WINDOW_PLAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_WINDOW_PLAN_100", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_WINDOW_PLAN_100)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_WINDOW_PLAN_50", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_WINDOW_PLAN_50)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_WINDOW_RCP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_WINDOW_RCP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_WINDOW_SILL_PLAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_WINDOW_SILL_PLAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_REP_ZONE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_REP_ZONE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISPROPSMASSELEMPLANCOMMON", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISPROPSMASSELEMPLANCOMMON)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISPROPSMASSGROUPPLANCOMMON", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISPROPSMASSGROUPPLANCOMMON)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISPROPSOPENINGPLANCOMMON", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISPROPSOPENINGPLANCOMMON)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISPROPSOPENINGPLANCOMMONHATCHED", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISPROPSOPENINGPLANCOMMONHATCHED)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISPROPSOPENINGSILLPLAN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISPROPSOPENINGSILLPLAN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_ROPS_RAILING_PLAN_100", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_ROPS_RAILING_PLAN_100)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_ROPS_RAILING_PLAN_50", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_ROPS_RAILING_PLAN_50)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_ROPS_STAIR_PLAN_100", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_ROPS_STAIR_PLAN_100)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DISP_ROPS_STAIR_PLAN_50", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DISP_ROPS_STAIR_PLAN_50)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_DOOR_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_DOOR_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_ENDCAP_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_ENDCAP_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_FRAME_DEF", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_FRAME_DEF)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_LAYERKEY_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_LAYERKEY_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_LIST_DEF", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_LIST_DEF)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_MASKBLOCK_DEF", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_MASKBLOCK_DEF)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_MASS_ELEM_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_MASS_ELEM_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_MATERIAL_DEF", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_MATERIAL_DEF)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_MVBLOCK_DEF", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_MVBLOCK_DEF)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_MVBLOCK_REF", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_MVBLOCK_REF)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_NOTIFICATION_TRACKER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_NOTIFICATION_TRACKER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_POLYGON", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_POLYGON)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_POLYGON_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_POLYGON_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_PROPERTY_SET_DEF", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_PROPERTY_SET_DEF)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_RAILING_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_RAILING_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_REFEDIT_STATUS_TRACKER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_REFEDIT_STATUS_TRACKER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_ROOFSLABEDGE_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_ROOFSLABEDGE_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_ROOFSLAB_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_ROOFSLAB_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_SCHEDULE_DATA_FORMAT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_SCHEDULE_DATA_FORMAT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_SLABEDGE_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_SLABEDGE_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_SLAB_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_SLAB_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_SPACE_STYLES", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_SPACE_STYLES)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_STAIR_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_STAIR_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_STAIR_WINDER_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_STAIR_WINDER_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_STAIR_WINDER_TYPE_BALANCED", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_STAIR_WINDER_TYPE_BALANCED)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_STAIR_WINDER_TYPE_MANUAL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_STAIR_WINDER_TYPE_MANUAL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_STAIR_WINDER_TYPE_SINGLE_POINT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_STAIR_WINDER_TYPE_SINGLE_POINT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_VARS_AECBBLDSRV", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_VARS_AECBBLDSRV)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_VARS_ARCHBASE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_VARS_ARCHBASE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_VARS_DWG_SETUP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_VARS_DWG_SETUP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_VARS_MUNICH", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_VARS_MUNICH)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_VARS_STRUCTUREBASE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_VARS_STRUCTUREBASE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_WALLMOD_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_WALLMOD_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_WALL_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_WALL_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_WINDOW_ASSEMBLY_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_WINDOW_ASSEMBLY_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AEC_WINDOW_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AEC_WINDOW_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ALIGNMENTGRIPENTITY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ALIGNMENTGRIPENTITY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AMCONTEXTMGR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AMCONTEXTMGR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AMDTADMENUSTATE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AMDTADMENUSTATE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AMDTAMMENUSTATE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AMDTAMMENUSTATE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AMDTBROWSERDBTAB", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AMDTBROWSERDBTAB)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AMDTDMMENUSTATE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AMDTDMMENUSTATE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AMDTEDGESTANDARDDIN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AMDTEDGESTANDARDDIN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AMDTEDGESTANDARDDIN13715", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AMDTEDGESTANDARDDIN13715)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AMDTEDGESTANDARDISO", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AMDTEDGESTANDARDISO)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AMDTEDGESTANDARDISO13715", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AMDTEDGESTANDARDISO13715)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AMDTFORMULAUPDATEDISPATCHER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AMDTFORMULAUPDATEDISPATCHER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AMDTINTERNALREACTOR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AMDTINTERNALREACTOR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AMDTMCOMMENUSTATE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AMDTMCOMMENUSTATE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AMDTMENUSTATEMGR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AMDTMENUSTATEMGR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AMDTNOTE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AMDTNOTE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AMDTNOTETEMPLATEDB", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AMDTNOTETEMPLATEDB)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AMDTSECTIONSYM", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AMDTSECTIONSYM)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AMDTSECTIONSYMLABEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AMDTSECTIONSYMLABEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AMDTSYSATTR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AMDTSYSATTR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AMGOBJPROPCFG", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AMGOBJPROPCFG)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AMGSETTINGSOBJ", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AMGSETTINGSOBJ)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AMIMASTER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AMIMASTER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AM_DRAWING_MGR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AM_DRAWING_MGR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AM_DWGMGR_NAME", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AM_DWGMGR_NAME)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AM_DWG_DOCUMENT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AM_DWG_DOCUMENT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AM_DWG_SHEET", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AM_DWG_SHEET)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_AM_VIEWDIMPARMAP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_AM_VIEWDIMPARMAP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BINRECORD", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BINRECORD)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_CAMSCATALOGAPPOBJECT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_CAMSCATALOGAPPOBJECT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_CAMSSTRUCTBTNSTATE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_CAMSSTRUCTBTNSTATE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_CATALOGSTATE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_CATALOGSTATE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_CBROWSERAPPOBJECT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_CBROWSERAPPOBJECT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DEPMGR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DEPMGR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DMBASEELEMENT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DMBASEELEMENT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DMDEFAULTSTYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DMDEFAULTSTYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DMLEGEND", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DMLEGEND)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DMMAP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DMMAP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DMMAPMANAGER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DMMAPMANAGER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DMSTYLECATEGORY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DMSTYLECATEGORY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DMSTYLELIBRARY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DMSTYLELIBRARY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DMSTYLEREFERENCE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DMSTYLEREFERENCE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DMSTYLIZEDENTITIESTABLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DMSTYLIZEDENTITIESTABLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DMSURROGATESTYLESETS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DMSURROGATESTYLESETS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DM_PLACEHOLDER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DM_PLACEHOLDER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_EXACTERMXREFMAP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_EXACTERMXREFMAP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_EXACXREFPANELOBJECT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_EXACXREFPANELOBJECT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_EXPO_NOTIFYBLOCK", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_EXPO_NOTIFYBLOCK)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_EXPO_NOTIFYHALL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_EXPO_NOTIFYHALL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_EXPO_NOTIFYPILLAR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_EXPO_NOTIFYPILLAR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_EXPO_NOTIFYSTAND", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_EXPO_NOTIFYSTAND)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_EXPO_NOTIFYSTANDNOPOLY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_EXPO_NOTIFYSTANDNOPOLY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_GSMANAGER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_GSMANAGER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_IRD_DSC_DICT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_IRD_DSC_DICT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_IRD_DSC_RECORD", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_IRD_DSC_RECORD)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_IRD_OBJ_RECORD", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_IRD_OBJ_RECORD)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_MAPFSMRVOBJECT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_MAPFSMRVOBJECT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_MAPGWSUNDOOBJECT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_MAPGWSUNDOOBJECT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_MAPIAMMOUDLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_MAPIAMMOUDLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_MAPMETADATAOBJECT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_MAPMETADATAOBJECT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_MAPRESOURCEMANAGEROBJECT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_MAPRESOURCEMANAGEROBJECT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_McDbContainer2", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_McDbContainer2)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_McDbMarker", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_McDbMarker)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_NAMEDAPPL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_NAMEDAPPL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_NEWSTDPARTPARLIST", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_NEWSTDPARTPARLIST)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_NOTEPOSITION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_NOTEPOSITION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_OBJCLONER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_OBJCLONER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_PARAMMGR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_PARAMMGR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_PARAMSCOPE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_PARAMSCOPE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_PILLAR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_PILLAR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_STDPART2D", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_STDPART2D)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_TCH_ARROW", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_TCH_ARROW)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_TCH_AXIS_LABEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_TCH_AXIS_LABEL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_TCH_BLOCK_INSERT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_TCH_BLOCK_INSERT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_TCH_COLUMN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_TCH_COLUMN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_TCH_DBCONFIG", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_TCH_DBCONFIG)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_TCH_DIMENSION2", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_TCH_DIMENSION2)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_TCH_DRAWINGINDEX", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_TCH_DRAWINGINDEX)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_TCH_HANDRAIL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_TCH_HANDRAIL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_TCH_LINESTAIR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_TCH_LINESTAIR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_TCH_OPENING", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_TCH_OPENING)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_TCH_RECTSTAIR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_TCH_RECTSTAIR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_TCH_SLAB", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_TCH_SLAB)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_TCH_SPACE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_TCH_SPACE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_TCH_TEXT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_TCH_TEXT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_TCH_WALL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_TCH_WALL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_TGrupoPuntos", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_TGrupoPuntos)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_VAACIMAGEINVENTORY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_VAACIMAGEINVENTORY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_VAACXREFPANELOBJECT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_VAACXREFPANELOBJECT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_FREED", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_FREED)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_UNKNOWN_ENT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_UNKNOWN_ENT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_UNKNOWN_OBJ", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_UNKNOWN_OBJ)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_UNUSED_r11", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_UNUSED_r11)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_LINE_r11", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_LINE_r11)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_POINT_r11", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_POINT_r11)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_CIRCLE_r11", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_CIRCLE_r11)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_SHAPE_r11", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_SHAPE_r11)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_REPEAT_r11", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_REPEAT_r11)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ENDREP_r11", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ENDREP_r11)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_TEXT_r11", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_TEXT_r11)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ARC_r11", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ARC_r11)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_TRACE_r11", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_TRACE_r11)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_LOAD_r11", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_LOAD_r11)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_SOLID_r11", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_SOLID_r11)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_BLOCK_r11", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_BLOCK_r11)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ENDBLK_r11", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ENDBLK_r11)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_INSERT_r11", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_INSERT_r11)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ATTDEF_r11", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ATTDEF_r11)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_ATTRIB_r11", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_ATTRIB_r11)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_SEQEND_r11", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_SEQEND_r11)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_JUMP_r11", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_JUMP_r11)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_POLYLINE_r11", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_POLYLINE_r11)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_VERTEX_r11", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_VERTEX_r11)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_3DLINE_r11", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_3DLINE_r11)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_3DFACE_r11", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_3DFACE_r11)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_DIMENSION_r11", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_DIMENSION_r11)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_VIEWPORT_r11", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_VIEWPORT_r11)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_TYPE_UNKNOWN_r11", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_TYPE_UNKNOWN_r11)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_NOERR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_NOERR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_ERR_WRONGCRC", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_ERR_WRONGCRC)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_ERR_NOTYETSUPPORTED", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_ERR_NOTYETSUPPORTED)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_ERR_UNHANDLEDCLASS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_ERR_UNHANDLEDCLASS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_ERR_INVALIDTYPE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_ERR_INVALIDTYPE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_ERR_INVALIDHANDLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_ERR_INVALIDHANDLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_ERR_INVALIDEED", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_ERR_INVALIDEED)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_ERR_VALUEOUTOFBOUNDS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_ERR_VALUEOUTOFBOUNDS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_ERR_CLASSESNOTFOUND", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_ERR_CLASSESNOTFOUND)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_ERR_SECTIONNOTFOUND", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_ERR_SECTIONNOTFOUND)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_ERR_PAGENOTFOUND", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_ERR_PAGENOTFOUND)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_ERR_INTERNALERROR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_ERR_INTERNALERROR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_ERR_INVALIDDWG", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_ERR_INVALIDDWG)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_ERR_IOERROR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_ERR_IOERROR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_ERR_OUTOFMEM", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_ERR_OUTOFMEM)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  SWIG_TypeClientData(SWIGTYPE_p__dwg_handle, (void*) "LibreDWG::Dwg_Handle");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ref, (void*) "LibreDWG::Dwg_Object_Ref");
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_HDL_OWNER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_HDL_OWNER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_HDL_SOFTOWN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_HDL_SOFTOWN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_HDL_HARDOWN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_HDL_HARDOWN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_HDL_SOFTPTR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_HDL_SOFTPTR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_HDL_HARDPTR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_HDL_HARDPTR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_COLOR_METHOD_VOID", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_COLOR_METHOD_VOID)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_COLOR_METHOD_BYLAYER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_COLOR_METHOD_BYLAYER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_COLOR_METHOD_BYBLOCK", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_COLOR_METHOD_BYBLOCK)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_COLOR_METHOD_ACI", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_COLOR_METHOD_ACI)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_COLOR_METHOD_TRUECOLOR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_COLOR_METHOD_TRUECOLOR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_COLOR_METHOD_FGCOLOR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_COLOR_METHOD_FGCOLOR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_COLOR_METHOD_NONE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_COLOR_METHOD_NONE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  SWIG_TypeClientData(SWIGTYPE_p__dwg_color, (void*) "LibreDWG::Dwg_Color");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_binary_chunk, (void*) "LibreDWG::_dwg_binary_chunk");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_binary_chunk_u, (void*) "LibreDWG::_dwg_binary_chunk_u");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_resbuf, (void*) "LibreDWG::Dwg_Resbuf");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_resbuf_value, (void*) "LibreDWG::_dwg_resbuf_value");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_header_variables, (void*) "LibreDWG::Dwg_Header_Variables");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_TEXT, (void*) "LibreDWG::Dwg_Entity_TEXT");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_AcDbMTextObjectEmbedded, (void*) "LibreDWG::Dwg_AcDbMTextObjectEmbedded");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_ATTRIB, (void*) "LibreDWG::Dwg_Entity_ATTRIB");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_ATTDEF, (void*) "LibreDWG::Dwg_Entity_ATTDEF");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_BLOCK, (void*) "LibreDWG::Dwg_Entity_BLOCK");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_ENDBLK, (void*) "LibreDWG::Dwg_Entity_ENDBLK");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_SEQEND, (void*) "LibreDWG::Dwg_Entity_SEQEND");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_INSERT, (void*) "LibreDWG::Dwg_Entity_INSERT");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_MINSERT, (void*) "LibreDWG::Dwg_Entity_MINSERT");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_VERTEX_2D, (void*) "LibreDWG::Dwg_Entity_VERTEX_2D");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_VERTEX_3D, (void*) "LibreDWG::Dwg_Entity_VERTEX_3D");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_VERTEX_PFACE_FACE, (void*) "LibreDWG::Dwg_Entity_VERTEX_PFACE_FACE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_POLYLINE_2D, (void*) "LibreDWG::Dwg_Entity_POLYLINE_2D");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_POLYLINE_3D, (void*) "LibreDWG::Dwg_Entity_POLYLINE_3D");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_ARC, (void*) "LibreDWG::Dwg_Entity_ARC");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_CIRCLE, (void*) "LibreDWG::Dwg_Entity_CIRCLE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_LINE, (void*) "LibreDWG::Dwg_Entity_LINE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_DIMENSION_common, (void*) "LibreDWG::Dwg_DIMENSION_common");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_DIMENSION_ORDINATE, (void*) "LibreDWG::Dwg_Entity_DIMENSION_ORDINATE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_DIMENSION_LINEAR, (void*) "LibreDWG::Dwg_Entity_DIMENSION_LINEAR");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_DIMENSION_ALIGNED, (void*) "LibreDWG::Dwg_Entity_DIMENSION_ALIGNED");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_DIMENSION_ANG3PT, (void*) "LibreDWG::Dwg_Entity_DIMENSION_ANG3PT");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_DIMENSION_ANG2LN, (void*) "LibreDWG::Dwg_Entity_DIMENSION_ANG2LN");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_DIMENSION_RADIUS, (void*) "LibreDWG::Dwg_Entity_DIMENSION_RADIUS");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_DIMENSION_DIAMETER, (void*) "LibreDWG::Dwg_Entity_DIMENSION_DIAMETER");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_ARC_DIMENSION, (void*) "LibreDWG::Dwg_Entity_ARC_DIMENSION");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_LARGE_RADIAL_DIMENSION, (void*) "LibreDWG::Dwg_Entity_LARGE_RADIAL_DIMENSION");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_POINT, (void*) "LibreDWG::Dwg_Entity_POINT");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_3DFACE, (void*) "LibreDWG::Dwg_Entity__3DFACE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_POLYLINE_PFACE, (void*) "LibreDWG::Dwg_Entity_POLYLINE_PFACE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_POLYLINE_MESH, (void*) "LibreDWG::Dwg_Entity_POLYLINE_MESH");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_SOLID, (void*) "LibreDWG::Dwg_Entity_SOLID");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_TRACE, (void*) "LibreDWG::Dwg_Entity_TRACE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_SHAPE, (void*) "LibreDWG::Dwg_Entity_SHAPE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_VIEWPORT, (void*) "LibreDWG::Dwg_Entity_VIEWPORT");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_ELLIPSE, (void*) "LibreDWG::Dwg_Entity_ELLIPSE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_SPLINE_control_point, (void*) "LibreDWG::Dwg_SPLINE_control_point");
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SPLINETYPE_QUADRATIC", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SPLINETYPE_QUADRATIC)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SPLINETYPE_CUBIC", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SPLINETYPE_CUBIC)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SPLINE_SCENARIO_SPLINE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SPLINE_SCENARIO_SPLINE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SPLINE_SCENARIO_BEZIER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SPLINE_SCENARIO_BEZIER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SPLINE_SPLINEFLAGS_METHOD_FIT_POINTS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SPLINE_SPLINEFLAGS_METHOD_FIT_POINTS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SPLINE_SPLINEFLAGS_CV_FRAME_SHOW", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SPLINE_SPLINEFLAGS_CV_FRAME_SHOW)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SPLINE_SPLINEFLAGS_CLOSED", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SPLINE_SPLINEFLAGS_CLOSED)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SPLINE_KNOTPARAM_CHORD", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SPLINE_KNOTPARAM_CHORD)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SPLINE_KNOTPARAM_SQUARE_ROOT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SPLINE_KNOTPARAM_SQUARE_ROOT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SPLINE_KNOTPARAM_UNIFORM", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SPLINE_KNOTPARAM_UNIFORM)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SPLINE_KNOTPARAM_CUSTOM", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SPLINE_KNOTPARAM_CUSTOM)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_SPLINE, (void*) "LibreDWG::Dwg_Entity_SPLINE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_3DSOLID_wire, (void*) "LibreDWG::Dwg_3DSOLID_wire");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_3DSOLID_silhouette, (void*) "LibreDWG::Dwg_3DSOLID_silhouette");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_3DSOLID_material, (void*) "LibreDWG::Dwg_3DSOLID_material");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_3DSOLID, (void*) "LibreDWG::Dwg_Entity__3DSOLID");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_RAY, (void*) "LibreDWG::Dwg_Entity_RAY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_DICTIONARY, (void*) "LibreDWG::Dwg_Object_DICTIONARY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_DICTIONARYWDFLT, (void*) "LibreDWG::Dwg_Object_DICTIONARYWDFLT");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_OLEFRAME, (void*) "LibreDWG::Dwg_Entity_OLEFRAME");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_MTEXT, (void*) "LibreDWG::Dwg_Entity_MTEXT");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_LEADER, (void*) "LibreDWG::Dwg_Entity_LEADER");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_TOLERANCE, (void*) "LibreDWG::Dwg_Entity_TOLERANCE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_MLINE_line, (void*) "LibreDWG::Dwg_MLINE_line");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_MLINE_vertex, (void*) "LibreDWG::Dwg_MLINE_vertex");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_MLINE, (void*) "LibreDWG::Dwg_Entity_MLINE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCK_CONTROL, (void*) "LibreDWG::Dwg_Object_BLOCK_CONTROL");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCK_HEADER, (void*) "LibreDWG::Dwg_Object_BLOCK_HEADER");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_LAYER_CONTROL, (void*) "LibreDWG::Dwg_Object_LAYER_CONTROL");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_LAYER, (void*) "LibreDWG::Dwg_Object_LAYER");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_STYLE_CONTROL, (void*) "LibreDWG::Dwg_Object_STYLE_CONTROL");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_STYLE, (void*) "LibreDWG::Dwg_Object_STYLE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_LTYPE_CONTROL, (void*) "LibreDWG::Dwg_Object_LTYPE_CONTROL");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_LTYPE_dash, (void*) "LibreDWG::Dwg_LTYPE_dash");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_LTYPE, (void*) "LibreDWG::Dwg_Object_LTYPE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_VIEW_CONTROL, (void*) "LibreDWG::Dwg_Object_VIEW_CONTROL");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_VIEW, (void*) "LibreDWG::Dwg_Object_VIEW");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_UCS_CONTROL, (void*) "LibreDWG::Dwg_Object_UCS_CONTROL");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_UCS_orthopts, (void*) "LibreDWG::Dwg_UCS_orthopts");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_UCS, (void*) "LibreDWG::Dwg_Object_UCS");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_VPORT_CONTROL, (void*) "LibreDWG::Dwg_Object_VPORT_CONTROL");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_VPORT, (void*) "LibreDWG::Dwg_Object_VPORT");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_APPID_CONTROL, (void*) "LibreDWG::Dwg_Object_APPID_CONTROL");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_APPID, (void*) "LibreDWG::Dwg_Object_APPID");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_DIMSTYLE_CONTROL, (void*) "LibreDWG::Dwg_Object_DIMSTYLE_CONTROL");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_DIMSTYLE, (void*) "LibreDWG::Dwg_Object_DIMSTYLE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_VX_CONTROL, (void*) "LibreDWG::Dwg_Object_VX_CONTROL");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_VX_TABLE_RECORD, (void*) "LibreDWG::Dwg_Object_VX_TABLE_RECORD");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_GROUP, (void*) "LibreDWG::Dwg_Object_GROUP");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_MLINESTYLE_line, (void*) "LibreDWG::Dwg_MLINESTYLE_line");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_MLINESTYLE_line_lt, (void*) "LibreDWG::_dwg_MLINESTYLE_line_lt");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_MLINESTYLE, (void*) "LibreDWG::Dwg_Object_MLINESTYLE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_OLE2FRAME, (void*) "LibreDWG::Dwg_Entity_OLE2FRAME");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_DUMMY, (void*) "LibreDWG::Dwg_Object_DUMMY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_LONG_TRANSACTION, (void*) "LibreDWG::Dwg_Object_LONG_TRANSACTION");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_LWPOLYLINE_width, (void*) "LibreDWG::Dwg_LWPOLYLINE_width");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_PROXY_LWPOLYLINE, (void*) "LibreDWG::Dwg_PROXY_LWPOLYLINE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_PROXY_ENTITY, (void*) "LibreDWG::Dwg_Entity_PROXY_ENTITY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_PROXY_OBJECT, (void*) "LibreDWG::Dwg_Object_PROXY_OBJECT");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_HATCH_Color, (void*) "LibreDWG::Dwg_HATCH_Color");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_HATCH_ControlPoint, (void*) "LibreDWG::Dwg_HATCH_ControlPoint");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_HATCH_PathSeg, (void*) "LibreDWG::Dwg_HATCH_PathSeg");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_HATCH_PolylinePath, (void*) "LibreDWG::Dwg_HATCH_PolylinePath");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_HATCH_Path, (void*) "LibreDWG::Dwg_HATCH_Path");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_HATCH_DefLine, (void*) "LibreDWG::Dwg_HATCH_DefLine");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_HATCH, (void*) "LibreDWG::Dwg_Entity_HATCH");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_MPOLYGON, (void*) "LibreDWG::Dwg_Entity_MPOLYGON");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_XRECORD, (void*) "LibreDWG::Dwg_Object_XRECORD");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_PLACEHOLDER, (void*) "LibreDWG::Dwg_Object_PLACEHOLDER");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_LEADER_Break, (void*) "LibreDWG::Dwg_LEADER_Break");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_LEADER_Line, (void*) "LibreDWG::Dwg_LEADER_Line");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_LEADER_ArrowHead, (void*) "LibreDWG::Dwg_LEADER_ArrowHead");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_LEADER_BlockLabel, (void*) "LibreDWG::Dwg_LEADER_BlockLabel");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_LEADER_Node, (void*) "LibreDWG::Dwg_LEADER_Node");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_MLEADER_Content_MText, (void*) "LibreDWG::Dwg_MLEADER_Content_MText");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_MLEADER_Content_Block, (void*) "LibreDWG::Dwg_MLEADER_Content_Block");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_MLEADER_Content, (void*) "LibreDWG::Dwg_MLEADER_Content");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_MLEADER_AnnotContext, (void*) "LibreDWG::Dwg_MLEADER_AnnotContext");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_MULTILEADER, (void*) "LibreDWG::Dwg_Entity_MULTILEADER");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_MLEADERSTYLE, (void*) "LibreDWG::Dwg_Object_MLEADERSTYLE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_VBA_PROJECT, (void*) "LibreDWG::Dwg_Object_VBA_PROJECT");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_PLOTSETTINGS, (void*) "LibreDWG::Dwg_Object_PLOTSETTINGS");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_LAYOUT, (void*) "LibreDWG::Dwg_Object_LAYOUT");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_DICTIONARYVAR, (void*) "LibreDWG::Dwg_Object_DICTIONARYVAR");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_TABLE_value, (void*) "LibreDWG::Dwg_TABLE_value");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_TABLE_CustomDataItem, (void*) "LibreDWG::Dwg_TABLE_CustomDataItem");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_TABLE_AttrDef, (void*) "LibreDWG::Dwg_TABLE_AttrDef");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_TABLE_Cell, (void*) "LibreDWG::Dwg_TABLE_Cell");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_TABLE_BreakHeight, (void*) "LibreDWG::Dwg_TABLE_BreakHeight");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_TABLE_BreakRow, (void*) "LibreDWG::Dwg_TABLE_BreakRow");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_LinkedData, (void*) "LibreDWG::Dwg_LinkedData");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_TableCellContent_Attr, (void*) "LibreDWG::Dwg_TableCellContent_Attr");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_ContentFormat, (void*) "LibreDWG::Dwg_ContentFormat");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_TableCellContent, (void*) "LibreDWG::Dwg_TableCellContent");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_CellContentGeometry, (void*) "LibreDWG::Dwg_CellContentGeometry");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_TableCell, (void*) "LibreDWG::Dwg_TableCell");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_GridFormat, (void*) "LibreDWG::Dwg_GridFormat");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_CellStyle, (void*) "LibreDWG::Dwg_CellStyle");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_TableRow, (void*) "LibreDWG::Dwg_TableRow");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_TableDataColumn, (void*) "LibreDWG::Dwg_TableDataColumn");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_LinkedTableData, (void*) "LibreDWG::Dwg_LinkedTableData");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_FormattedTableMerged, (void*) "LibreDWG::Dwg_FormattedTableMerged");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_FormattedTableData, (void*) "LibreDWG::Dwg_FormattedTableData");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_TABLECONTENT, (void*) "LibreDWG::Dwg_Object_TABLECONTENT");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_TABLE, (void*) "LibreDWG::Dwg_Entity_TABLE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_TABLESTYLE_CellStyle, (void*) "LibreDWG::Dwg_TABLESTYLE_CellStyle");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_TABLESTYLE_border, (void*) "LibreDWG::Dwg_TABLESTYLE_border");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_TABLESTYLE_rowstyles, (void*) "LibreDWG::Dwg_TABLESTYLE_rowstyles");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_TABLESTYLE, (void*) "LibreDWG::Dwg_Object_TABLESTYLE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_CELLSTYLEMAP, (void*) "LibreDWG::Dwg_Object_CELLSTYLEMAP");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_TABLEGEOMETRY_Cell, (void*) "LibreDWG::Dwg_TABLEGEOMETRY_Cell");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_TABLEGEOMETRY, (void*) "LibreDWG::Dwg_Object_TABLEGEOMETRY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_abstractobject_UNDERLAYDEFINITION, (void*) "LibreDWG::Dwg_Object_UNDERLAYDEFINITION");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_abstractentity_UNDERLAY, (void*) "LibreDWG::Dwg_Entity_UNDERLAY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_DBCOLOR, (void*) "LibreDWG::Dwg_Object_DBCOLOR");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_FIELD_ChildValue, (void*) "LibreDWG::Dwg_FIELD_ChildValue");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_FIELD, (void*) "LibreDWG::Dwg_Object_FIELD");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_FIELDLIST, (void*) "LibreDWG::Dwg_Object_FIELDLIST");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_GEODATA_meshpt, (void*) "LibreDWG::Dwg_GEODATA_meshpt");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_GEODATA_meshface, (void*) "LibreDWG::Dwg_GEODATA_meshface");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_GEODATA, (void*) "LibreDWG::Dwg_Object_GEODATA");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_IDBUFFER, (void*) "LibreDWG::Dwg_Object_IDBUFFER");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_IMAGE, (void*) "LibreDWG::Dwg_Entity_IMAGE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_IMAGEDEF, (void*) "LibreDWG::Dwg_Object_IMAGEDEF");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_IMAGEDEF_REACTOR, (void*) "LibreDWG::Dwg_Object_IMAGEDEF_REACTOR");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_INDEX, (void*) "LibreDWG::Dwg_Object_INDEX");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_LAYER_entry, (void*) "LibreDWG::Dwg_LAYER_entry");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_LAYER_INDEX, (void*) "LibreDWG::Dwg_Object_LAYER_INDEX");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_LWPOLYLINE, (void*) "LibreDWG::Dwg_Entity_LWPOLYLINE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_RASTERVARIABLES, (void*) "LibreDWG::Dwg_Object_RASTERVARIABLES");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_SCALE, (void*) "LibreDWG::Dwg_Object_SCALE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_SORTENTSTABLE, (void*) "LibreDWG::Dwg_Object_SORTENTSTABLE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_SPATIAL_FILTER, (void*) "LibreDWG::Dwg_Object_SPATIAL_FILTER");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_SPATIAL_INDEX, (void*) "LibreDWG::Dwg_Object_SPATIAL_INDEX");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_WIPEOUT, (void*) "LibreDWG::Dwg_Entity_WIPEOUT");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_WIPEOUTVARIABLES, (void*) "LibreDWG::Dwg_Object_WIPEOUTVARIABLES");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_SECTIONOBJECT, (void*) "LibreDWG::Dwg_Entity_SECTIONOBJECT");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_VISUALSTYLE, (void*) "LibreDWG::Dwg_Object_VISUALSTYLE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_LIGHTLIST_light, (void*) "LibreDWG::Dwg_LIGHTLIST_light");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_LIGHTLIST, (void*) "LibreDWG::Dwg_Object_LIGHTLIST");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_MATERIAL_color, (void*) "LibreDWG::Dwg_MATERIAL_color");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_MATERIAL_mapper, (void*) "LibreDWG::Dwg_MATERIAL_mapper");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_MATERIAL_gentexture, (void*) "LibreDWG::Dwg_MATERIAL_gentexture");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_MATERIAL, (void*) "LibreDWG::Dwg_Object_MATERIAL");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_OBJECT_PTR, (void*) "LibreDWG::Dwg_Object_OBJECT_PTR");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_LIGHT, (void*) "LibreDWG::Dwg_Entity_LIGHT");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_CAMERA, (void*) "LibreDWG::Dwg_Entity_CAMERA");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_GEOPOSITIONMARKER, (void*) "LibreDWG::Dwg_Entity_GEOPOSITIONMARKER");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_GEOMAPIMAGE, (void*) "LibreDWG::Dwg_Object_GEOMAPIMAGE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_HELIX, (void*) "LibreDWG::Dwg_Entity_HELIX");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_EXTRUDEDSURFACE, (void*) "LibreDWG::Dwg_Entity_EXTRUDEDSURFACE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_SWEPTSURFACE, (void*) "LibreDWG::Dwg_Entity_SWEPTSURFACE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_LOFTEDSURFACE, (void*) "LibreDWG::Dwg_Entity_LOFTEDSURFACE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_NURBSURFACE, (void*) "LibreDWG::Dwg_Entity_NURBSURFACE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_PLANESURFACE, (void*) "LibreDWG::Dwg_Entity_PLANESURFACE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_REVOLVEDSURFACE, (void*) "LibreDWG::Dwg_Entity_REVOLVEDSURFACE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_MESH_edge, (void*) "LibreDWG::Dwg_MESH_edge");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_MESH, (void*) "LibreDWG::Dwg_Entity_MESH");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_SUN, (void*) "LibreDWG::Dwg_Object_SUN");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_SUNSTUDY_Dates, (void*) "LibreDWG::Dwg_SUNSTUDY_Dates");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_SUNSTUDY, (void*) "LibreDWG::Dwg_Object_SUNSTUDY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_DATATABLE_row, (void*) "LibreDWG::Dwg_DATATABLE_row");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_DATATABLE_column, (void*) "LibreDWG::Dwg_DATATABLE_column");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_DATATABLE, (void*) "LibreDWG::Dwg_Object_DATATABLE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_DATALINK_customdata, (void*) "LibreDWG::Dwg_DATALINK_customdata");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_DATALINK, (void*) "LibreDWG::Dwg_Object_DATALINK");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_DIMASSOC_Ref, (void*) "LibreDWG::Dwg_DIMASSOC_Ref");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_DIMASSOC, (void*) "LibreDWG::Dwg_Object_DIMASSOC");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_ACTIONBODY, (void*) "LibreDWG::Dwg_ACTIONBODY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_EvalVariant, (void*) "LibreDWG::Dwg_EvalVariant");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_EvalVariant_u, (void*) "LibreDWG::_dwg_EvalVariant_u");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_VALUEPARAM_vars, (void*) "LibreDWG::Dwg_VALUEPARAM_vars");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_VALUEPARAM, (void*) "LibreDWG::Dwg_VALUEPARAM");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_ASSOCPARAMBASEDACTIONBODY, (void*) "LibreDWG::Dwg_ASSOCPARAMBASEDACTIONBODY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_ASSOCACTION_Deps, (void*) "LibreDWG::Dwg_ASSOCACTION_Deps");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCDEPENDENCY, (void*) "LibreDWG::Dwg_Object_ASSOCDEPENDENCY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCVALUEDEPENDENCY, (void*) "LibreDWG::Dwg_Object_ASSOCVALUEDEPENDENCY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCGEOMDEPENDENCY, (void*) "LibreDWG::Dwg_Object_ASSOCGEOMDEPENDENCY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCACTION, (void*) "LibreDWG::Dwg_Object_ASSOCACTION");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCNETWORK, (void*) "LibreDWG::Dwg_Object_ASSOCNETWORK");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_CONSTRAINTGROUPNODE, (void*) "LibreDWG::Dwg_CONSTRAINTGROUPNODE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOC2DCONSTRAINTGROUP, (void*) "LibreDWG::Dwg_Object_ASSOC2DCONSTRAINTGROUP");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCVARIABLE, (void*) "LibreDWG::Dwg_Object_ASSOCVARIABLE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_EVAL_Node, (void*) "LibreDWG::Dwg_EVAL_Node");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_EVAL_Edge, (void*) "LibreDWG::Dwg_EVAL_Edge");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_EVALUATION_GRAPH, (void*) "LibreDWG::Dwg_Object_EVALUATION_GRAPH");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_DYNAMICBLOCKPURGEPREVENTER, (void*) "LibreDWG::Dwg_Object_DYNAMICBLOCKPURGEPREVENTER");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_PERSUBENTMGR, (void*) "LibreDWG::Dwg_Object_PERSUBENTMGR");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCPERSSUBENTMANAGER, (void*) "LibreDWG::Dwg_Object_ASSOCPERSSUBENTMANAGER");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCACTIONPARAM, (void*) "LibreDWG::Dwg_Object_ASSOCACTIONPARAM");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCOSNAPPOINTREFACTIONPARAM, (void*) "LibreDWG::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCPOINTREFACTIONPARAM, (void*) "LibreDWG::Dwg_Object_ASSOCPOINTREFACTIONPARAM");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCASMBODYACTIONPARAM, (void*) "LibreDWG::Dwg_Object_ASSOCASMBODYACTIONPARAM");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCCOMPOUNDACTIONPARAM, (void*) "LibreDWG::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCOBJECTACTIONPARAM, (void*) "LibreDWG::Dwg_Object_ASSOCOBJECTACTIONPARAM");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCEDGEACTIONPARAM, (void*) "LibreDWG::Dwg_Object_ASSOCEDGEACTIONPARAM");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCFACEACTIONPARAM, (void*) "LibreDWG::Dwg_Object_ASSOCFACEACTIONPARAM");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCPATHACTIONPARAM, (void*) "LibreDWG::Dwg_Object_ASSOCPATHACTIONPARAM");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCVERTEXACTIONPARAM, (void*) "LibreDWG::Dwg_Object_ASSOCVERTEXACTIONPARAM");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_ASSOCARRAYITEM, (void*) "LibreDWG::Dwg_ASSOCARRAYITEM");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_abstractobject_ASSOCARRAYPARAMETERS, (void*) "LibreDWG::Dwg_Object_ASSOCARRAYPARAMETERS");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCRESTOREENTITYSTATEACTIONBODY, (void*) "LibreDWG::Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_ASSOCSURFACEACTIONBODY, (void*) "LibreDWG::Dwg_ASSOCSURFACEACTIONBODY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCEXTENDSURFACEACTIONBODY, (void*) "LibreDWG::Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCEXTRUDEDSURFACEACTIONBODY, (void*) "LibreDWG::Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCPLANESURFACEACTIONBODY, (void*) "LibreDWG::Dwg_Object_ASSOCPLANESURFACEACTIONBODY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCLOFTEDSURFACEACTIONBODY, (void*) "LibreDWG::Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCNETWORKSURFACEACTIONBODY, (void*) "LibreDWG::Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCOFFSETSURFACEACTIONBODY, (void*) "LibreDWG::Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCREVOLVEDSURFACEACTIONBODY, (void*) "LibreDWG::Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCSWEPTSURFACEACTIONBODY, (void*) "LibreDWG::Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCEDGECHAMFERACTIONBODY, (void*) "LibreDWG::Dwg_Object_ASSOCEDGECHAMFERACTIONBODY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCEDGEFILLETACTIONBODY, (void*) "LibreDWG::Dwg_Object_ASSOCEDGEFILLETACTIONBODY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCTRIMSURFACEACTIONBODY, (void*) "LibreDWG::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCBLENDSURFACEACTIONBODY, (void*) "LibreDWG::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCFILLETSURFACEACTIONBODY, (void*) "LibreDWG::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCPATCHSURFACEACTIONBODY, (void*) "LibreDWG::Dwg_Object_ASSOCPATCHSURFACEACTIONBODY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_ASSOCACTIONBODY_action, (void*) "LibreDWG::Dwg_ASSOCACTIONBODY_action");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCMLEADERACTIONBODY, (void*) "LibreDWG::Dwg_Object_ASSOCMLEADERACTIONBODY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCALIGNEDDIMACTIONBODY, (void*) "LibreDWG::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOC3POINTANGULARDIMACTIONBODY, (void*) "LibreDWG::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCORDINATEDIMACTIONBODY, (void*) "LibreDWG::Dwg_Object_ASSOCORDINATEDIMACTIONBODY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCROTATEDDIMACTIONBODY, (void*) "LibreDWG::Dwg_Object_ASSOCROTATEDDIMACTIONBODY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCDIMDEPENDENCYBODY, (void*) "LibreDWG::Dwg_Object_ASSOCDIMDEPENDENCYBODY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKPARAMDEPENDENCYBODY, (void*) "LibreDWG::Dwg_Object_BLOCKPARAMDEPENDENCYBODY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_ARRAYITEMLOCATOR, (void*) "LibreDWG::Dwg_ARRAYITEMLOCATOR");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCARRAYACTIONBODY, (void*) "LibreDWG::Dwg_Object_ASSOCARRAYACTIONBODY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ASSOCARRAYMODIFYACTIONBODY, (void*) "LibreDWG::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_EvalExpr, (void*) "LibreDWG::Dwg_EvalExpr");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_EvalExpr_value, (void*) "LibreDWG::_dwg_EvalExpr_value");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_ACSH_SubentMaterial, (void*) "LibreDWG::Dwg_ACSH_SubentMaterial");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_ACSH_SubentColor, (void*) "LibreDWG::Dwg_ACSH_SubentColor");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_ACSH_HistoryNode, (void*) "LibreDWG::Dwg_ACSH_HistoryNode");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ACSH_HISTORY_CLASS, (void*) "LibreDWG::Dwg_Object_ACSH_HISTORY_CLASS");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ACSH_BOX_CLASS, (void*) "LibreDWG::Dwg_Object_ACSH_BOX_CLASS");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ACSH_WEDGE_CLASS, (void*) "LibreDWG::Dwg_Object_ACSH_WEDGE_CLASS");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ACSH_BOOLEAN_CLASS, (void*) "LibreDWG::Dwg_Object_ACSH_BOOLEAN_CLASS");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ACSH_BREP_CLASS, (void*) "LibreDWG::Dwg_Object_ACSH_BREP_CLASS");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ACSH_SWEEP_CLASS, (void*) "LibreDWG::Dwg_Object_ACSH_SWEEP_CLASS");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ACSH_EXTRUSION_CLASS, (void*) "LibreDWG::Dwg_Object_ACSH_EXTRUSION_CLASS");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ACSH_LOFT_CLASS, (void*) "LibreDWG::Dwg_Object_ACSH_LOFT_CLASS");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ACSH_FILLET_CLASS, (void*) "LibreDWG::Dwg_Object_ACSH_FILLET_CLASS");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ACSH_CHAMFER_CLASS, (void*) "LibreDWG::Dwg_Object_ACSH_CHAMFER_CLASS");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ACSH_CYLINDER_CLASS, (void*) "LibreDWG::Dwg_Object_ACSH_CYLINDER_CLASS");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ACSH_CONE_CLASS, (void*) "LibreDWG::Dwg_Object_ACSH_CONE_CLASS");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ACSH_PYRAMID_CLASS, (void*) "LibreDWG::Dwg_Object_ACSH_PYRAMID_CLASS");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ACSH_SPHERE_CLASS, (void*) "LibreDWG::Dwg_Object_ACSH_SPHERE_CLASS");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ACSH_TORUS_CLASS, (void*) "LibreDWG::Dwg_Object_ACSH_TORUS_CLASS");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ACSH_REVOLVE_CLASS, (void*) "LibreDWG::Dwg_Object_ACSH_REVOLVE_CLASS");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_NAVISWORKSMODEL, (void*) "LibreDWG::Dwg_Entity_NAVISWORKSMODEL");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_NAVISWORKSMODELDEF, (void*) "LibreDWG::Dwg_Object_NAVISWORKSMODELDEF");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_RENDERSETTINGS, (void*) "LibreDWG::Dwg_Object_RENDERSETTINGS");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_MENTALRAYRENDERSETTINGS, (void*) "LibreDWG::Dwg_Object_MENTALRAYRENDERSETTINGS");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_RAPIDRTRENDERSETTINGS, (void*) "LibreDWG::Dwg_Object_RAPIDRTRENDERSETTINGS");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_RENDERENVIRONMENT, (void*) "LibreDWG::Dwg_Object_RENDERENVIRONMENT");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_RENDERGLOBAL, (void*) "LibreDWG::Dwg_Object_RENDERGLOBAL");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_RENDERENTRY, (void*) "LibreDWG::Dwg_Object_RENDERENTRY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_MOTIONPATH, (void*) "LibreDWG::Dwg_Object_MOTIONPATH");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_CURVEPATH, (void*) "LibreDWG::Dwg_Object_CURVEPATH");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_POINTPATH, (void*) "LibreDWG::Dwg_Object_POINTPATH");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_TVDEVICEPROPERTIES, (void*) "LibreDWG::Dwg_Object_TVDEVICEPROPERTIES");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_SKYLIGHT_BACKGROUND, (void*) "LibreDWG::Dwg_Object_SKYLIGHT_BACKGROUND");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_SOLID_BACKGROUND, (void*) "LibreDWG::Dwg_Object_SOLID_BACKGROUND");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_IMAGE_BACKGROUND, (void*) "LibreDWG::Dwg_Object_IMAGE_BACKGROUND");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_IBL_BACKGROUND, (void*) "LibreDWG::Dwg_Object_IBL_BACKGROUND");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_GRADIENT_BACKGROUND, (void*) "LibreDWG::Dwg_Object_GRADIENT_BACKGROUND");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_GROUND_PLANE_BACKGROUND, (void*) "LibreDWG::Dwg_Object_GROUND_PLANE_BACKGROUND");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ANNOTSCALEOBJECTCONTEXTDATA, (void*) "LibreDWG::Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_CONTEXTDATA_dict, (void*) "LibreDWG::Dwg_CONTEXTDATA_dict");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_CONTEXTDATA_submgr, (void*) "LibreDWG::Dwg_CONTEXTDATA_submgr");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_CONTEXTDATAMANAGER, (void*) "LibreDWG::Dwg_Object_CONTEXTDATAMANAGER");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_TEXTOBJECTCONTEXTDATA, (void*) "LibreDWG::Dwg_Object_TEXTOBJECTCONTEXTDATA");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_MTEXTOBJECTCONTEXTDATA, (void*) "LibreDWG::Dwg_Object_MTEXTOBJECTCONTEXTDATA");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_OCD_Dimension, (void*) "LibreDWG::Dwg_OCD_Dimension");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ALDIMOBJECTCONTEXTDATA, (void*) "LibreDWG::Dwg_Object_ALDIMOBJECTCONTEXTDATA");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ANGDIMOBJECTCONTEXTDATA, (void*) "LibreDWG::Dwg_Object_ANGDIMOBJECTCONTEXTDATA");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_DMDIMOBJECTCONTEXTDATA, (void*) "LibreDWG::Dwg_Object_DMDIMOBJECTCONTEXTDATA");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ORDDIMOBJECTCONTEXTDATA, (void*) "LibreDWG::Dwg_Object_ORDDIMOBJECTCONTEXTDATA");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_RADIMOBJECTCONTEXTDATA, (void*) "LibreDWG::Dwg_Object_RADIMOBJECTCONTEXTDATA");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_RADIMLGOBJECTCONTEXTDATA, (void*) "LibreDWG::Dwg_Object_RADIMLGOBJECTCONTEXTDATA");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_MTEXTATTRIBUTEOBJECTCONTEXTDATA, (void*) "LibreDWG::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_MLEADEROBJECTCONTEXTDATA, (void*) "LibreDWG::Dwg_Object_MLEADEROBJECTCONTEXTDATA");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_LEADEROBJECTCONTEXTDATA, (void*) "LibreDWG::Dwg_Object_LEADEROBJECTCONTEXTDATA");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLKREFOBJECTCONTEXTDATA, (void*) "LibreDWG::Dwg_Object_BLKREFOBJECTCONTEXTDATA");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_FCFOBJECTCONTEXTDATA, (void*) "LibreDWG::Dwg_Object_FCFOBJECTCONTEXTDATA");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_DETAILVIEWSTYLE, (void*) "LibreDWG::Dwg_Object_DETAILVIEWSTYLE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_SECTIONVIEWSTYLE, (void*) "LibreDWG::Dwg_Object_SECTIONVIEWSTYLE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_SECTION_MANAGER, (void*) "LibreDWG::Dwg_Object_SECTION_MANAGER");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_SECTION_geometrysettings, (void*) "LibreDWG::Dwg_SECTION_geometrysettings");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_SECTION_typesettings, (void*) "LibreDWG::Dwg_SECTION_typesettings");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_SECTION_SETTINGS, (void*) "LibreDWG::Dwg_Object_SECTION_SETTINGS");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_LAYERFILTER, (void*) "LibreDWG::Dwg_Object_LAYERFILTER");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_ARCALIGNEDTEXT, (void*) "LibreDWG::Dwg_Entity_ARCALIGNEDTEXT");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_RTEXT, (void*) "LibreDWG::Dwg_Entity_RTEXT");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_LAYOUTPRINTCONFIG, (void*) "LibreDWG::Dwg_Entity_LAYOUTPRINTCONFIG");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ACMECOMMANDHISTORY, (void*) "LibreDWG::Dwg_Object_ACMECOMMANDHISTORY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ACMESCOPE, (void*) "LibreDWG::Dwg_Object_ACMESCOPE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_ACMESTATEMGR, (void*) "LibreDWG::Dwg_Object_ACMESTATEMGR");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_CSACDOCUMENTOPTIONS, (void*) "LibreDWG::Dwg_Object_CSACDOCUMENTOPTIONS");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_BLOCKPARAMETER_connection, (void*) "LibreDWG::Dwg_BLOCKPARAMETER_connection");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_BLOCKPARAMETER_PropInfo, (void*) "LibreDWG::Dwg_BLOCKPARAMETER_PropInfo");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_BLOCKPARAMVALUESET, (void*) "LibreDWG::Dwg_BLOCKPARAMVALUESET");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_BLOCKACTION_connectionpts, (void*) "LibreDWG::Dwg_BLOCKACTION_connectionpts");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_BLOCKVISIBILITYPARAMETER_state, (void*) "LibreDWG::Dwg_BLOCKVISIBILITYPARAMETER_state");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKVISIBILITYPARAMETER, (void*) "LibreDWG::Dwg_Object_BLOCKVISIBILITYPARAMETER");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKVISIBILITYGRIP, (void*) "LibreDWG::Dwg_Object_BLOCKVISIBILITYGRIP");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKGRIPLOCATIONCOMPONENT, (void*) "LibreDWG::Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BREAKDATA, (void*) "LibreDWG::Dwg_Object_BREAKDATA");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BREAKPOINTREF, (void*) "LibreDWG::Dwg_Object_BREAKPOINTREF");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_FLIPGRIPENTITY, (void*) "LibreDWG::Dwg_Entity_FLIPGRIPENTITY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_LINEARGRIPENTITY, (void*) "LibreDWG::Dwg_Entity_LINEARGRIPENTITY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_POLARGRIPENTITY, (void*) "LibreDWG::Dwg_Entity_POLARGRIPENTITY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_ROTATIONGRIPENTITY, (void*) "LibreDWG::Dwg_Entity_ROTATIONGRIPENTITY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_VISIBILITYGRIPENTITY, (void*) "LibreDWG::Dwg_Entity_VISIBILITYGRIPENTITY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_XYGRIPENTITY, (void*) "LibreDWG::Dwg_Entity_XYGRIPENTITY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_ALIGNMENTPARAMETERENTITY, (void*) "LibreDWG::Dwg_Entity_ALIGNMENTPARAMETERENTITY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_BASEPOINTPARAMETERENTITY, (void*) "LibreDWG::Dwg_Entity_BASEPOINTPARAMETERENTITY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_FLIPPARAMETERENTITY, (void*) "LibreDWG::Dwg_Entity_FLIPPARAMETERENTITY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_LINEARPARAMETERENTITY, (void*) "LibreDWG::Dwg_Entity_LINEARPARAMETERENTITY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_POINTPARAMETERENTITY, (void*) "LibreDWG::Dwg_Entity_POINTPARAMETERENTITY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_ROTATIONPARAMETERENTITY, (void*) "LibreDWG::Dwg_Entity_ROTATIONPARAMETERENTITY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_VISIBILITYPARAMETERENTITY, (void*) "LibreDWG::Dwg_Entity_VISIBILITYPARAMETERENTITY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_XYPARAMETERENTITY, (void*) "LibreDWG::Dwg_Entity_XYPARAMETERENTITY");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKALIGNMENTGRIP, (void*) "LibreDWG::Dwg_Object_BLOCKALIGNMENTGRIP");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKALIGNMENTPARAMETER, (void*) "LibreDWG::Dwg_Object_BLOCKALIGNMENTPARAMETER");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKANGULARCONSTRAINTPARAMETER, (void*) "LibreDWG::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKDIAMETRICCONSTRAINTPARAMETER, (void*) "LibreDWG::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKRADIALCONSTRAINTPARAMETER, (void*) "LibreDWG::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKARRAYACTION, (void*) "LibreDWG::Dwg_Object_BLOCKARRAYACTION");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKBASEPOINTPARAMETER, (void*) "LibreDWG::Dwg_Object_BLOCKBASEPOINTPARAMETER");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKFLIPACTION, (void*) "LibreDWG::Dwg_Object_BLOCKFLIPACTION");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKFLIPGRIP, (void*) "LibreDWG::Dwg_Object_BLOCKFLIPGRIP");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKFLIPPARAMETER, (void*) "LibreDWG::Dwg_Object_BLOCKFLIPPARAMETER");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKALIGNEDCONSTRAINTPARAMETER, (void*) "LibreDWG::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKLINEARCONSTRAINTPARAMETER, (void*) "LibreDWG::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKHORIZONTALCONSTRAINTPARAMETER, (void*) "LibreDWG::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKVERTICALCONSTRAINTPARAMETER, (void*) "LibreDWG::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKLINEARGRIP, (void*) "LibreDWG::Dwg_Object_BLOCKLINEARGRIP");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKLINEARPARAMETER, (void*) "LibreDWG::Dwg_Object_BLOCKLINEARPARAMETER");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_BLOCKLOOKUPACTION_lut, (void*) "LibreDWG::Dwg_BLOCKLOOKUPACTION_lut");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKLOOKUPACTION, (void*) "LibreDWG::Dwg_Object_BLOCKLOOKUPACTION");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKLOOKUPGRIP, (void*) "LibreDWG::Dwg_Object_BLOCKLOOKUPGRIP");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKLOOKUPPARAMETER, (void*) "LibreDWG::Dwg_Object_BLOCKLOOKUPPARAMETER");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKMOVEACTION, (void*) "LibreDWG::Dwg_Object_BLOCKMOVEACTION");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKPOINTPARAMETER, (void*) "LibreDWG::Dwg_Object_BLOCKPOINTPARAMETER");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKPOLARGRIP, (void*) "LibreDWG::Dwg_Object_BLOCKPOLARGRIP");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKPOLARPARAMETER, (void*) "LibreDWG::Dwg_Object_BLOCKPOLARPARAMETER");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKPOLARSTRETCHACTION, (void*) "LibreDWG::Dwg_Object_BLOCKPOLARSTRETCHACTION");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLE, (void*) "LibreDWG::Dwg_Object_BLOCKPROPERTIESTABLE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKPROPERTIESTABLEGRIP, (void*) "LibreDWG::Dwg_Object_BLOCKPROPERTIESTABLEGRIP");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKREPRESENTATION, (void*) "LibreDWG::Dwg_Object_BLOCKREPRESENTATION");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKROTATEACTION, (void*) "LibreDWG::Dwg_Object_BLOCKROTATEACTION");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKROTATIONGRIP, (void*) "LibreDWG::Dwg_Object_BLOCKROTATIONGRIP");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKROTATIONPARAMETER, (void*) "LibreDWG::Dwg_Object_BLOCKROTATIONPARAMETER");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKSCALEACTION, (void*) "LibreDWG::Dwg_Object_BLOCKSCALEACTION");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_BLOCKSTRETCHACTION_handles, (void*) "LibreDWG::Dwg_BLOCKSTRETCHACTION_handles");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_BLOCKSTRETCHACTION_codes, (void*) "LibreDWG::Dwg_BLOCKSTRETCHACTION_codes");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKSTRETCHACTION, (void*) "LibreDWG::Dwg_Object_BLOCKSTRETCHACTION");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKUSERPARAMETER, (void*) "LibreDWG::Dwg_Object_BLOCKUSERPARAMETER");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKXYGRIP, (void*) "LibreDWG::Dwg_Object_BLOCKXYGRIP");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_BLOCKXYPARAMETER, (void*) "LibreDWG::Dwg_Object_BLOCKXYPARAMETER");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_DYNAMICBLOCKPROXYNODE, (void*) "LibreDWG::Dwg_Object_DYNAMICBLOCKPROXYNODE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_POINTCLOUD_IntensityStyle, (void*) "LibreDWG::Dwg_POINTCLOUD_IntensityStyle");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_POINTCLOUD_Clippings, (void*) "LibreDWG::Dwg_POINTCLOUD_Clippings");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_POINTCLOUDEX_Croppings, (void*) "LibreDWG::Dwg_POINTCLOUDEX_Croppings");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_POINTCLOUD, (void*) "LibreDWG::Dwg_Entity_POINTCLOUD");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_POINTCLOUDEX, (void*) "LibreDWG::Dwg_Entity_POINTCLOUDEX");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_POINTCLOUDDEF, (void*) "LibreDWG::Dwg_Object_POINTCLOUDDEF");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_POINTCLOUDDEFEX, (void*) "LibreDWG::Dwg_Object_POINTCLOUDDEFEX");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_POINTCLOUDDEF_REACTOR, (void*) "LibreDWG::Dwg_Object_POINTCLOUDDEF_REACTOR");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_POINTCLOUDDEF_REACTOR_EX, (void*) "LibreDWG::Dwg_Object_POINTCLOUDDEF_REACTOR_EX");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_ColorRamp, (void*) "LibreDWG::Dwg_ColorRamp");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_POINTCLOUDCOLORMAP_Ramp, (void*) "LibreDWG::Dwg_POINTCLOUDCOLORMAP_Ramp");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_POINTCLOUDCOLORMAP, (void*) "LibreDWG::Dwg_Object_POINTCLOUDCOLORMAP");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_COMPOUNDOBJECTID, (void*) "LibreDWG::Dwg_COMPOUNDOBJECTID");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_PARTIAL_VIEWING_INDEX_Entry, (void*) "LibreDWG::Dwg_PARTIAL_VIEWING_INDEX_Entry");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_PARTIAL_VIEWING_INDEX, (void*) "LibreDWG::Dwg_Object_PARTIAL_VIEWING_INDEX");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_UNKNOWN_ENT, (void*) "LibreDWG::Dwg_Entity_UNKNOWN_ENT");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_UNKNOWN_OBJ, (void*) "LibreDWG::Dwg_Object_UNKNOWN_OBJ");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_REPEAT, (void*) "LibreDWG::Dwg_Entity_REPEAT");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_ENDREP, (void*) "LibreDWG::Dwg_Entity_ENDREP");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_LOAD, (void*) "LibreDWG::Dwg_Entity_LOAD");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_JUMP, (void*) "LibreDWG::Dwg_Entity_JUMP");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_3DLINE, (void*) "LibreDWG::Dwg_Entity__3DLINE");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_eed_data, (void*) "LibreDWG::Dwg_Eed_Data");
  SWIG_TypeClientData(SWIGTYPE_p_eed_data_t, (void*) "LibreDWG::eed_data_t");
  SWIG_TypeClientData(SWIGTYPE_p_eed_data_t_eed_71, (void*) "LibreDWG::eed_data_t_eed_71");
  SWIG_TypeClientData(SWIGTYPE_p_eed_data_t_eed_70, (void*) "LibreDWG::eed_data_t_eed_70");
  SWIG_TypeClientData(SWIGTYPE_p_eed_data_t_eed_40, (void*) "LibreDWG::eed_data_t_eed_40");
  SWIG_TypeClientData(SWIGTYPE_p_eed_data_t_eed_10, (void*) "LibreDWG::eed_data_t_eed_10");
  SWIG_TypeClientData(SWIGTYPE_p_eed_data_t_eed_5, (void*) "LibreDWG::eed_data_t_eed_5");
  SWIG_TypeClientData(SWIGTYPE_p_eed_data_t_eed_4, (void*) "LibreDWG::eed_data_t_eed_4");
  SWIG_TypeClientData(SWIGTYPE_p_eed_data_t_eed_3, (void*) "LibreDWG::eed_data_t_eed_3");
  SWIG_TypeClientData(SWIGTYPE_p_eed_data_t_eed_2, (void*) "LibreDWG::eed_data_t_eed_2");
  SWIG_TypeClientData(SWIGTYPE_p_eed_data_t_eed_1, (void*) "LibreDWG::eed_data_t_eed_1");
  SWIG_TypeClientData(SWIGTYPE_p_eed_data_t_eed_0_r2007, (void*) "LibreDWG::eed_data_t_eed_0_r2007");
  SWIG_TypeClientData(SWIGTYPE_p_eed_data_t_eed_0, (void*) "LibreDWG::eed_data_t_eed_0");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_entity_eed, (void*) "LibreDWG::Dwg_Eed");
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "EXTRA_R11_UNKNOWN_1", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(EXTRA_R11_UNKNOWN_1)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "EXTRA_R11_HAS_EED", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(EXTRA_R11_HAS_EED)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "EXTRA_R11_HAS_VIEWPORT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(EXTRA_R11_HAS_VIEWPORT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "EXTRA_R11_UNKNOWN_8", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(EXTRA_R11_UNKNOWN_8)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "EXTRA_R11_UNKNOWN_16", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(EXTRA_R11_UNKNOWN_16)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "EXTRA_R11_UNKNOWN_32", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(EXTRA_R11_UNKNOWN_32)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "EXTRA_R11_UNKNOWN_64", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(EXTRA_R11_UNKNOWN_64)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "EXTRA_R11_UNKNOWN_128", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(EXTRA_R11_UNKNOWN_128)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_R11_HAS_COLOR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_R11_HAS_COLOR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_R11_HAS_LTYPE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_R11_HAS_LTYPE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_R11_HAS_ELEVATION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_R11_HAS_ELEVATION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_R11_HAS_THICKNESS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_R11_HAS_THICKNESS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_R11_HAS_UNKNOWN_16", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_R11_HAS_UNKNOWN_16)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_R11_HAS_HANDLING", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_R11_HAS_HANDLING)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_R11_HAS_PSPACE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_R11_HAS_PSPACE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_R11_HAS_ATTRIBS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_R11_HAS_ATTRIBS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_R11_DIMENSION_LINEAR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_R11_DIMENSION_LINEAR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_R11_DIMENSION_ALIGNED", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_R11_DIMENSION_ALIGNED)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_R11_DIMENSION_ANG2LN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_R11_DIMENSION_ANG2LN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_R11_DIMENSION_DIAMETER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_R11_DIMENSION_DIAMETER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_R11_DIMENSION_RADIUS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_R11_DIMENSION_RADIUS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_R11_DIMENSION_ANG3PT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_R11_DIMENSION_ANG3PT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_R11_DIMENSION_ORDINATE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_R11_DIMENSION_ORDINATE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_R11_DIMENSION_ORDINATE_TYPE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_R11_DIMENSION_ORDINATE_TYPE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_R11_DIMENSION_TEXT_POSITIONED", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_R11_DIMENSION_TEXT_POSITIONED)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_XREF_DEP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_XREF_DEP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_XREF_REF", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_XREF_REF)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_XREF_RESOLVED", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_XREF_RESOLVED)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_XDATA", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_XDATA)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_3DFACE_HAS_Z_FIRST", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_3DFACE_HAS_Z_FIRST)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_3DFACE_HAS_Z_SECOND", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_3DFACE_HAS_Z_SECOND)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_3DFACE_HAS_Z_THIRD", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_3DFACE_HAS_Z_THIRD)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_3DFACE_HAS_Z_FOURTH", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_3DFACE_HAS_Z_FOURTH)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_3DLINE_HAS_Z_FIRST", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_3DLINE_HAS_Z_FIRST)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_3DLINE_HAS_Z_SECOND", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_3DLINE_HAS_Z_SECOND)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_ARC_HAS_EXTRUSION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_ARC_HAS_EXTRUSION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_ATTDEF_UNKNOWN_1", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_ATTDEF_UNKNOWN_1)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_ATTDEF_HAS_ROTATION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_ATTDEF_HAS_ROTATION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_ATTDEF_HAS_WIDTH_FACTOR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_ATTDEF_HAS_WIDTH_FACTOR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_ATTDEF_HAS_OBLIQUE_ANGLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_ATTDEF_HAS_OBLIQUE_ANGLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_ATTDEF_HAS_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_ATTDEF_HAS_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_ATTDEF_HAS_GENERATION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_ATTDEF_HAS_GENERATION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_ATTDEF_HAS_HORIZ_ALIGNMENT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_ATTDEF_HAS_HORIZ_ALIGNMENT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_ATTDEF_HAS_ALIGNMENT_POINT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_ATTDEF_HAS_ALIGNMENT_POINT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_ATTDEF_HAS_EXTRUSION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_ATTDEF_HAS_EXTRUSION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_ATTDEF_HAS_VERT_ALIGNMENT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_ATTDEF_HAS_VERT_ALIGNMENT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_ATTRIB_UNKNOWN_1", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_ATTRIB_UNKNOWN_1)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_ATTRIB_HAS_ROTATION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_ATTRIB_HAS_ROTATION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_ATTRIB_HAS_WIDTH_FACTOR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_ATTRIB_HAS_WIDTH_FACTOR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_ATTRIB_HAS_OBLIQUE_ANGLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_ATTRIB_HAS_OBLIQUE_ANGLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_ATTRIB_HAS_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_ATTRIB_HAS_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_ATTRIB_HAS_GENERATION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_ATTRIB_HAS_GENERATION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_ATTRIB_HAS_HORIZ_ALIGNMENT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_ATTRIB_HAS_HORIZ_ALIGNMENT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_ATTRIB_HAS_ALIGNMENT_POINT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_ATTRIB_HAS_ALIGNMENT_POINT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_ATTRIB_HAS_EXTRUSION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_ATTRIB_HAS_EXTRUSION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_ATTRIB_HAS_VERT_ALIGNMENT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_ATTRIB_HAS_VERT_ALIGNMENT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_BLOCK_UNKNOWN_1", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_BLOCK_UNKNOWN_1)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_BLOCK_HAS_XREF_PNAME", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_BLOCK_HAS_XREF_PNAME)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_BLOCK_HAS_BLOCK_NAME", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_BLOCK_HAS_BLOCK_NAME)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_CIRCLE_HAS_EXTRUSION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_CIRCLE_HAS_EXTRUSION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_DIMENSION_HAS_DXF12", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_DIMENSION_HAS_DXF12)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_DIMENSION_HAS_FLAG", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_DIMENSION_HAS_FLAG)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_DIMENSION_HAS_TEXT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_DIMENSION_HAS_TEXT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_DIMENSION_HAS_DXF13", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_DIMENSION_HAS_DXF13)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_DIMENSION_HAS_DXF14", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_DIMENSION_HAS_DXF14)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_DIMENSION_HAS_DXF15", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_DIMENSION_HAS_DXF15)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_DIMENSION_HAS_ANGLES", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_DIMENSION_HAS_ANGLES)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_DIMENSION_HAS_DXF40", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_DIMENSION_HAS_DXF40)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_DIMENSION_HAS_ROTATION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_DIMENSION_HAS_ROTATION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_DIMENSION_UNKNOWN_512", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_DIMENSION_UNKNOWN_512)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_DIMENSION_HAS_DXF53", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_DIMENSION_HAS_DXF53)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_DIMENSION_UNKNOWN_2048", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_DIMENSION_UNKNOWN_2048)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_DIMENSION_UNKNOWN_4096", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_DIMENSION_UNKNOWN_4096)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_DIMENSION_UNKNOWN_8192", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_DIMENSION_UNKNOWN_8192)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_DIMENSION_HAS_EXTRUSION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_DIMENSION_HAS_EXTRUSION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_DIMENSION_HAS_DIMSTYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_DIMENSION_HAS_DIMSTYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_INSERT_HAS_SCALE_X", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_INSERT_HAS_SCALE_X)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_INSERT_HAS_SCALE_Y", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_INSERT_HAS_SCALE_Y)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_INSERT_HAS_ROTATION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_INSERT_HAS_ROTATION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_INSERT_HAS_SCALE_Z", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_INSERT_HAS_SCALE_Z)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_INSERT_HAS_NUM_COLS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_INSERT_HAS_NUM_COLS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_INSERT_HAS_NUM_ROWS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_INSERT_HAS_NUM_ROWS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_INSERT_HAS_COL_SPACING", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_INSERT_HAS_COL_SPACING)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_INSERT_HAS_ROW_SPACING", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_INSERT_HAS_ROW_SPACING)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_INSERT_HAS_EXTRUSION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_INSERT_HAS_EXTRUSION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_LINE_HAS_EXTRUSION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_LINE_HAS_EXTRUSION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_POINT_HAS_EXTRUSION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_POINT_HAS_EXTRUSION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_POINT_HAS_X_ANG", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_POINT_HAS_X_ANG)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_POLYLINE_HAS_FLAG", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_POLYLINE_HAS_FLAG)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_POLYLINE_HAS_START_WIDTH", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_POLYLINE_HAS_START_WIDTH)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_POLYLINE_HAS_END_WIDTH", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_POLYLINE_HAS_END_WIDTH)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_POLYLINE_HAS_EXTRUSION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_POLYLINE_HAS_EXTRUSION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_POLYLINE_HAS_M_VERTS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_POLYLINE_HAS_M_VERTS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_POLYLINE_HAS_N_VERTS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_POLYLINE_HAS_N_VERTS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_POLYLINE_HAS_M_DENSITY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_POLYLINE_HAS_M_DENSITY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_POLYLINE_HAS_N_DENSITY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_POLYLINE_HAS_N_DENSITY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_POLYLINE_HAS_CURVETYPE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_POLYLINE_HAS_CURVETYPE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_POLYLINE_IN_EXTRA", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_POLYLINE_IN_EXTRA)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_SHAPE_HAS_ROTATION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_SHAPE_HAS_ROTATION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_SHAPE_HAS_LOAD_NUM", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_SHAPE_HAS_LOAD_NUM)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_SHAPE_HAS_WIDTH_FACTOR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_SHAPE_HAS_WIDTH_FACTOR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_SHAPE_HAS_OBLIQUE_ANGLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_SHAPE_HAS_OBLIQUE_ANGLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_SOLID_HAS_EXTRUSION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_SOLID_HAS_EXTRUSION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_TEXT_HAS_ROTATION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_TEXT_HAS_ROTATION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_TEXT_HAS_WIDTH_FACTOR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_TEXT_HAS_WIDTH_FACTOR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_TEXT_HAS_OBLIQUE_ANGLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_TEXT_HAS_OBLIQUE_ANGLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_TEXT_HAS_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_TEXT_HAS_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_TEXT_HAS_GENERATION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_TEXT_HAS_GENERATION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_TEXT_HAS_HORIZ_ALIGNMENT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_TEXT_HAS_HORIZ_ALIGNMENT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_TEXT_HAS_ALIGNMENT_POINT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_TEXT_HAS_ALIGNMENT_POINT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_TEXT_HAS_EXTRUSION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_TEXT_HAS_EXTRUSION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_TEXT_ALIGNED_VERT_TO", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_TEXT_ALIGNED_VERT_TO)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_VERTEX_HAS_START_WIDTH", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_VERTEX_HAS_START_WIDTH)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_VERTEX_HAS_END_WIDTH", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_VERTEX_HAS_END_WIDTH)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_VERTEX_HAS_BULGE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_VERTEX_HAS_BULGE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_VERTEX_HAS_FLAG", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_VERTEX_HAS_FLAG)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_VERTEX_HAS_TANGENT_DIR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_VERTEX_HAS_TANGENT_DIR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_VERTEX_HAS_INDEX1", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_VERTEX_HAS_INDEX1)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_VERTEX_HAS_INDEX2", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_VERTEX_HAS_INDEX2)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_VERTEX_HAS_INDEX3", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_VERTEX_HAS_INDEX3)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_VERTEX_HAS_INDEX4", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_VERTEX_HAS_INDEX4)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_VERTEX_UNKNOWN_512", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_VERTEX_UNKNOWN_512)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_VERTEX_UNKNOWN_1024", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_VERTEX_UNKNOWN_1024)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_VERTEX_UNKNOWN_2048", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_VERTEX_UNKNOWN_2048)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_VERTEX_UNKNOWN_4096", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_VERTEX_UNKNOWN_4096)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_VERTEX_UNKNOWN_8192", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_VERTEX_UNKNOWN_8192)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "OPTS_R11_VERTEX_HAS_NOT_X_Y", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(OPTS_R11_VERTEX_HAS_NOT_X_Y)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_TABLE_IS_XREF_REF", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_TABLE_IS_XREF_REF)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_TABLE_IS_XREF_RESOLVED", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_TABLE_IS_XREF_RESOLVED)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_TABLE_IS_XREF_DEP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_TABLE_IS_XREF_DEP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_TABLE_IS_REMOVED", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_TABLE_IS_REMOVED)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_BLOCK_ANONYMOUS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_BLOCK_ANONYMOUS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_BLOCK_HAS_ATTRIBS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_BLOCK_HAS_ATTRIBS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_BLOCK_IS_EXT_REF", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_BLOCK_IS_EXT_REF)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_BLOCK_IS_XREF_OVERLAY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_BLOCK_IS_XREF_OVERLAY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_ATTDEF_INVISIBLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_ATTDEF_INVISIBLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_ATTDEF_CONSTANT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_ATTDEF_CONSTANT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_ATTDEF_VERIFY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_ATTDEF_VERIFY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_ATTDEF_PRESET", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_ATTDEF_PRESET)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_ATTRIB_INVISIBLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_ATTRIB_INVISIBLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_ATTRIB_CONSTANT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_ATTRIB_CONSTANT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_ATTRIB_VERIFY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_ATTRIB_VERIFY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_ATTRIB_PRESET", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_ATTRIB_PRESET)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_POLYLINE_CLOSED", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_POLYLINE_CLOSED)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_POLYLINE_CURVE_FIT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_POLYLINE_CURVE_FIT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_POLYLINE_SPLINE_FIT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_POLYLINE_SPLINE_FIT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_POLYLINE_3D", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_POLYLINE_3D)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_POLYLINE_MESH", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_POLYLINE_MESH)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_POLYLINE_MESH_CLOSED", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_POLYLINE_MESH_CLOSED)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_POLYLINE_PFACE_MESH", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_POLYLINE_PFACE_MESH)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_POLYLINE_LT_PATTERN_CONTINUES", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_POLYLINE_LT_PATTERN_CONTINUES)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "LTYPE_SHAPE_FLAG_ABS_ROTATION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(LTYPE_SHAPE_FLAG_ABS_ROTATION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "LTYPE_SHAPE_FLAG_IS_TEXT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(LTYPE_SHAPE_FLAG_IS_TEXT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "LTYPE_SHAPE_FLAG_IS_SHAPE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(LTYPE_SHAPE_FLAG_IS_SHAPE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_LAYER_FROZEN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_LAYER_FROZEN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_LAYER_FROZEN_IN_NEW", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_LAYER_FROZEN_IN_NEW)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_LAYER_LOCKED", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_LAYER_LOCKED)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_LAYER_PLOTFLAG", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_LAYER_PLOTFLAG)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_STYLE_SHAPE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_STYLE_SHAPE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_STYLE_VERTICAL_TEXT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_STYLE_VERTICAL_TEXT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_VIEW_PSPACE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_VIEW_PSPACE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_VX_IS_ON", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_VX_IS_ON)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_LWPOLYLINE_HAS_EXTRUSION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_LWPOLYLINE_HAS_EXTRUSION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_LWPOLYLINE_HAS_THICKNESS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_LWPOLYLINE_HAS_THICKNESS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_LWPOLYLINE_HAS_CONSTWIDTH", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_LWPOLYLINE_HAS_CONSTWIDTH)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_LWPOLYLINE_HAS_ELEVATION", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_LWPOLYLINE_HAS_ELEVATION)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_LWPOLYLINE_HAS_NUM_BULGES", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_LWPOLYLINE_HAS_NUM_BULGES)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_LWPOLYLINE_HAS_NUM_WIDTHS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_LWPOLYLINE_HAS_NUM_WIDTHS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_LWPOLYLINE_UNKNOWN_64", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_LWPOLYLINE_UNKNOWN_64)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_LWPOLYLINE_UNKNOWN_128", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_LWPOLYLINE_UNKNOWN_128)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_LWPOLYLINE_PLINEGEN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_LWPOLYLINE_PLINEGEN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_LWPOLYLINE_CLOSED", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_LWPOLYLINE_CLOSED)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_LWPOLYLINE_VERTEXIDCOUNT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_LWPOLYLINE_VERTEXIDCOUNT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_VERTEX_EXTRA_VERTEX", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_VERTEX_EXTRA_VERTEX)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_VERTEX_CURVE_FIT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_VERTEX_CURVE_FIT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_VERTEX_UNUSED_4", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_VERTEX_UNUSED_4)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_VERTEX_SPLINE_FIT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_VERTEX_SPLINE_FIT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_VERTEX_SPLINE_FRAME_CONTROL_POINT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_VERTEX_SPLINE_FRAME_CONTROL_POINT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_VERTEX_3D", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_VERTEX_3D)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_VERTEX_MESH", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_VERTEX_MESH)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "FLAG_VERTEX_PFACE_MESH", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(FLAG_VERTEX_PFACE_MESH)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "TEXT_GENERATION_BACKWARDS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(TEXT_GENERATION_BACKWARDS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "TEXT_GENERATION_UPSIDE_DOWN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(TEXT_GENERATION_UPSIDE_DOWN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "HORIZ_ALIGNMENT_LEFT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(HORIZ_ALIGNMENT_LEFT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "HORIZ_ALIGNMENT_CENTER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(HORIZ_ALIGNMENT_CENTER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "HORIZ_ALIGNMENT_RIGHT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(HORIZ_ALIGNMENT_RIGHT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "HORIZ_ALIGNMENT_ALIGNED", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(HORIZ_ALIGNMENT_ALIGNED)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "HORIZ_ALIGNMENT_MIDDLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(HORIZ_ALIGNMENT_MIDDLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "HORIZ_ALIGNMENT_FIT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(HORIZ_ALIGNMENT_FIT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "VERT_ALIGNMENT_BASELINE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(VERT_ALIGNMENT_BASELINE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "VERT_ALIGNMENT_BOTTOM", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(VERT_ALIGNMENT_BOTTOM)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "VERT_ALIGNMENT_MIDDLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(VERT_ALIGNMENT_MIDDLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "VERT_ALIGNMENT_TOP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(VERT_ALIGNMENT_TOP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "MLINE_JUSTIFICATION_TOP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(MLINE_JUSTIFICATION_TOP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "MLINE_JUSTIFICATION_MIDDLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(MLINE_JUSTIFICATION_MIDDLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "MLINE_JUSTIFICATION_BOTTOM", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(MLINE_JUSTIFICATION_BOTTOM)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "POLYLINE_CURVETYPE_DEFAULT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(POLYLINE_CURVETYPE_DEFAULT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "POLYLINE_CURVETYPE_QUADR_BSPLINE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(POLYLINE_CURVETYPE_QUADR_BSPLINE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "POLYLINE_CURVETYPE_CUBIC_BSPLINE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(POLYLINE_CURVETYPE_CUBIC_BSPLINE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "POLYLINE_CURVETYPE_BEZIER_SURFACE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(POLYLINE_CURVETYPE_BEZIER_SURFACE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "LEADER_PATHTYPE_STRAIGHT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(LEADER_PATHTYPE_STRAIGHT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "LEADER_PATHTYPE_SPLINE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(LEADER_PATHTYPE_SPLINE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "LEADER_ANNOTTYPE_MTEXT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(LEADER_ANNOTTYPE_MTEXT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "LEADER_ANNOTTYPE_TOLERANCE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(LEADER_ANNOTTYPE_TOLERANCE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "LEADER_ANNOTTYPE_INSERT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(LEADER_ANNOTTYPE_INSERT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "LEADER_ANNOTTYPE_NO_ANNOT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(LEADER_ANNOTTYPE_NO_ANNOT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "MLINE_FLAGS_HAS_VERTEX", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(MLINE_FLAGS_HAS_VERTEX)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "MLINE_FLAGS_CLOSED", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(MLINE_FLAGS_CLOSED)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "MLINE_FLAGS_SUPPRESS_START_CAPS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(MLINE_FLAGS_SUPPRESS_START_CAPS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "MLINE_FLAGS_SUPPRESS_END_CAPS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(MLINE_FLAGS_SUPPRESS_END_CAPS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "MLINESTYLE_FLAG_FILL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(MLINESTYLE_FLAG_FILL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "MLINESTYLE_FLAG_MITERS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(MLINESTYLE_FLAG_MITERS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "MLINESTYLE_FLAG_UNKNOWN_4", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(MLINESTYLE_FLAG_UNKNOWN_4)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "MLINESTYLE_FLAG_UNKNOWN_8", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(MLINESTYLE_FLAG_UNKNOWN_8)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "MLINESTYLE_FLAG_START_SQUARE_END", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(MLINESTYLE_FLAG_START_SQUARE_END)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "MLINESTYLE_FLAG_START_INNER_ARC", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(MLINESTYLE_FLAG_START_INNER_ARC)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "MLINESTYLE_FLAG_START_ROUND", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(MLINESTYLE_FLAG_START_ROUND)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "MLINESTYLE_FLAG_UNKNOWN_128", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(MLINESTYLE_FLAG_UNKNOWN_128)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "MLINESTYLE_FLAG_END_SQUARE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(MLINESTYLE_FLAG_END_SQUARE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "MLINESTYLE_FLAG_END_INNER_ARC", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(MLINESTYLE_FLAG_END_INNER_ARC)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "MLINESTYLE_FLAG_END_ROUND", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(MLINESTYLE_FLAG_END_ROUND)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "LIGHT_TYPE_DISTANT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(LIGHT_TYPE_DISTANT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "LIGHT_TYPE_POINT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(LIGHT_TYPE_POINT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "LIGHT_TYPE_SPOT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(LIGHT_TYPE_SPOT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "LIGHT_ATTENUATION_TYPE_NONE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(LIGHT_ATTENUATION_TYPE_NONE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "LIGHT_ATTENUATION_TYPE_INV_LINEAR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(LIGHT_ATTENUATION_TYPE_INV_LINEAR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "LIGHT_ATTENUATION_TYPE_INV_SQUARE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(LIGHT_ATTENUATION_TYPE_INV_SQUARE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "LIGHT_EXTLIGHT_SHAPE_LINEAR", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(LIGHT_EXTLIGHT_SHAPE_LINEAR)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "LIGHT_EXTLIGHT_SHAPE_RECT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(LIGHT_EXTLIGHT_SHAPE_RECT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "LIGHT_EXTLIGHT_SHAPE_DISK", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(LIGHT_EXTLIGHT_SHAPE_DISK)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "LIGHT_EXTLIGHT_SHAPE_CYLINDER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(LIGHT_EXTLIGHT_SHAPE_CYLINDER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "LIGHT_EXTLIGHT_SHAPE_SPHERE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(LIGHT_EXTLIGHT_SHAPE_SPHERE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "_3DFACE_INVISIBLE_EDGE1", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(_3DFACE_INVISIBLE_EDGE1)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "_3DFACE_INVISIBLE_EDGE2", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(_3DFACE_INVISIBLE_EDGE2)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "_3DFACE_INVISIBLE_EDGE3", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(_3DFACE_INVISIBLE_EDGE3)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "_3DFACE_INVISIBLE_EDGE4", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(_3DFACE_INVISIBLE_EDGE4)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_entity, (void*) "LibreDWG::Dwg_Object_Entity");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_entity_tio, (void*) "LibreDWG::_dwg_object_entity_tio");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_object, (void*) "LibreDWG::Dwg_Object_Object");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_object_tio, (void*) "LibreDWG::_dwg_object_object_tio");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_class, (void*) "LibreDWG::Dwg_Class");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object, (void*) "LibreDWG::Dwg_Object");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_object_tio, (void*) "LibreDWG::_dwg_object_tio");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_chain, (void*) "LibreDWG::Dwg_Chain");
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_UNKNOWN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_UNKNOWN)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_HEADER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_HEADER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_AUXHEADER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_AUXHEADER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_CLASSES", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_CLASSES)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_HANDLES", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_HANDLES)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_TEMPLATE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_TEMPLATE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_OBJFREESPACE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_OBJFREESPACE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_OBJECTS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_OBJECTS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_REVHISTORY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_REVHISTORY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_SUMMARYINFO", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_SUMMARYINFO)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_PREVIEW", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_PREVIEW)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_APPINFO", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_APPINFO)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_APPINFOHISTORY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_APPINFOHISTORY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_FILEDEPLIST", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_FILEDEPLIST)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_SECURITY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_SECURITY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_VBAPROJECT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_VBAPROJECT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_SIGNATURE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_SIGNATURE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_ACDS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_ACDS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_INFO", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_INFO)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_SYSTEM_MAP", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_SYSTEM_MAP)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_HEADER_R13", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_HEADER_R13)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_CLASSES_R13", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_CLASSES_R13)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_HANDLES_R13", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_HANDLES_R13)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_OBJFREESPACE_R13", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_OBJFREESPACE_R13)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_TEMPLATE_R13", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_TEMPLATE_R13)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_AUXHEADER_R2000", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_AUXHEADER_R2000)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_THUMBNAIL_R13", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_THUMBNAIL_R13)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_HEADER_R11", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_HEADER_R11)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_BLOCK", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_BLOCK)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_LAYER", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_LAYER)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_STYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_STYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_LTYPE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_LTYPE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_VIEW", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_VIEW)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_UCS", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_UCS)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_VPORT", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_VPORT)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_APPID", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_APPID)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_DIMSTYLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_DIMSTYLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "SECTION_VX", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(SECTION_VX)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  SWIG_TypeClientData(SWIGTYPE_p__dwg_section, (void*) "LibreDWG::Dwg_Section");
  SWIG_TypeClientData(SWIGTYPE_p_Dwg_Section_InfoHdr, (void*) "LibreDWG::Dwg_Section_InfoHdr");
  SWIG_TypeClientData(SWIGTYPE_p_Dwg_Section_Info, (void*) "LibreDWG::Dwg_Section_Info");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_SummaryInfo_Property, (void*) "LibreDWG::Dwg_SummaryInfo_Property");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_FileDepList_Files, (void*) "LibreDWG::Dwg_FileDepList_Files");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_AcDs_SegmentIndex, (void*) "LibreDWG::Dwg_AcDs_SegmentIndex");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_AcDs_DataIndex_Entry, (void*) "LibreDWG::Dwg_AcDs_DataIndex_Entry");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_AcDs_DataIndex, (void*) "LibreDWG::Dwg_AcDs_DataIndex");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_AcDs_Data_RecordHdr, (void*) "LibreDWG::Dwg_AcDs_Data_RecordHdr");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_AcDs_Data_Record, (void*) "LibreDWG::Dwg_AcDs_Data_Record");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_AcDs_Data, (void*) "LibreDWG::Dwg_AcDs_Data");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_AcDs_DataBlobRef_Page, (void*) "LibreDWG::Dwg_AcDs_DataBlobRef_Page");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_AcDs_DataBlobRef, (void*) "LibreDWG::Dwg_AcDs_DataBlobRef");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_AcDs_DataBlob, (void*) "LibreDWG::Dwg_AcDs_DataBlob");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_AcDs_DataBlob01, (void*) "LibreDWG::Dwg_AcDs_DataBlob01");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_AcDs_SchemaIndex_Prop, (void*) "LibreDWG::Dwg_AcDs_SchemaIndex_Prop");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_AcDs_SchemaIndex, (void*) "LibreDWG::Dwg_AcDs_SchemaIndex");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_AcDs_Schema_Prop, (void*) "LibreDWG::Dwg_AcDs_Schema_Prop");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_AcDs_Schema, (void*) "LibreDWG::Dwg_AcDs_Schema");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_AcDs_SchemaData_UProp, (void*) "LibreDWG::Dwg_AcDs_SchemaData_UProp");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_AcDs_SchemaData, (void*) "LibreDWG::Dwg_AcDs_SchemaData");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_AcDs_Search_IdIdx, (void*) "LibreDWG::Dwg_AcDs_Search_IdIdx");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_AcDs_Search_IdIdxs, (void*) "LibreDWG::Dwg_AcDs_Search_IdIdxs");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_AcDs_Search_Data, (void*) "LibreDWG::Dwg_AcDs_Search_Data");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_AcDs_Search, (void*) "LibreDWG::Dwg_AcDs_Search");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_AcDs_Segment, (void*) "LibreDWG::Dwg_AcDs_Segment");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_AcDs, (void*) "LibreDWG::Dwg_AcDs");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_header, (void*) "LibreDWG::Dwg_Header");
  SWIG_TypeClientData(SWIGTYPE_p_r2007_file_header, (void*) "LibreDWG::Dwg_R2007_Header");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_R2004_Header, (void*) "LibreDWG::Dwg_R2004_Header");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_auxheader, (void*) "LibreDWG::Dwg_AuxHeader");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_summaryinfo, (void*) "LibreDWG::Dwg_SummaryInfo");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_appinfo, (void*) "LibreDWG::Dwg_AppInfo");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_filedeplist, (void*) "LibreDWG::Dwg_FileDepList");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_security, (void*) "LibreDWG::Dwg_Security");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_vbaproject, (void*) "LibreDWG::Dwg_VBAProject");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_appinfohistory, (void*) "LibreDWG::Dwg_AppInfoHistory");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_revhistory, (void*) "LibreDWG::Dwg_RevHistory");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_objfreespace, (void*) "LibreDWG::Dwg_ObjFreeSpace");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_template, (void*) "LibreDWG::Dwg_Template");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_secondheader_sections, (void*) "LibreDWG::Dwg_SecondHeader_Sections");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_secondheader_handles, (void*) "LibreDWG::Dwg_SecondHeader_Handles");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_secondheader, (void*) "LibreDWG::Dwg_SecondHeader");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_struct, (void*) "LibreDWG::Dwg_Data");
  SWIG_TypeClientData(SWIGTYPE_p__dwg_struct_fhdr, (void*) "LibreDWG::_dwg_struct_fhdr");
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_OPTS_LOGLEVEL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(0xf)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_OPTS_MINIMAL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(0x10)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_OPTS_DXFB", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(0x20)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_OPTS_JSONFIRST", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(0x20)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_OPTS_INDXF", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(0x40)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_OPTS_INJSON", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(0x80)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_OPTS_IN", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)((0x40|0x80))));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_VT_INVALID", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_VT_INVALID)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_VT_STRING", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_VT_STRING)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_VT_POINT3D", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_VT_POINT3D)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_VT_REAL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_VT_REAL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_VT_INT16", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_VT_INT16)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_VT_INT32", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_VT_INT32)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_VT_INT8", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_VT_INT8)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_VT_BINARY", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_VT_BINARY)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_VT_HANDLE", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_VT_HANDLE)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_VT_OBJECTID", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_VT_OBJECTID)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_VT_BOOL", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_VT_BOOL)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  /*@SWIG:/usr/share/swig/4.3.1/perl5/perltypemaps.swg,67,%set_constant@*/ do {
    SV *sv = get_sv((char*) SWIG_prefix "DWG_VT_INT64", TRUE | 0x2 | GV_ADDMULTI);
    sv_setsv(sv, SWIG_From_int  SWIG_PERL_CALL_ARGS_1((int)(DWG_VT_INT64)));
    SvREADONLY_on(sv);
  } while(0) /*@SWIG@*/;
  SWIG_TypeClientData(SWIGTYPE_p_rgbpalette, (void*) "LibreDWG::Dwg_RGB_Palette");
  SWIG_TypeClientData(SWIGTYPE_p_dwg_field_name_type_offset, (void*) "LibreDWG::Dwg_DYNAPI_field");
  ST(0) = &PL_sv_yes;
  XSRETURN(1);
}

